## File containing manual fixes for code generated by swig
## that is wrong.
## This file will get shorter and disappear as we improve swig.

PermuteAxesImageFilter_DefaultOrder =
function(value, .copy = FALSE)
{
  PermuteAxesImageFilter_DefaultOrder_get(.copy)
}

if (existsMethod("$", "ExternalReference")) {
  packageStartupMessage("SimpleITK - swig has been fixed so the $ method insize zFixes can be removed")
} else {
  setMethod("$", "ExternalReference",function(x, name){
    msg <- paste0("Unknown method: ", sQuote(name), ". Check spelling, documentation etc.")
    stop(msg)
  })
}
