# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.4.0
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

##   Generated via the command line invocation:
##	 /usr/bin/swig -r -namespace -I/build/simpleitk/src/simpleitk/Wrapping/Common -DSWIGWORDSIZE64 -outdir /build/simpleitk/src/build/Wrapping/R -c++ -I/build/simpleitk/src/simpleitk/Code/Common/include -I/build/simpleitk/src/simpleitk/Code/Registration/include -I/build/simpleitk/src/simpleitk/Code/BasicFilters/include -I/build/simpleitk/src/simpleitk/Code/IO/include -I/build/simpleitk/src/build/Code/BasicFilters/include -I/build/simpleitk/src/build/Code/Common/include -o /build/simpleitk/src/build/Wrapping/R/SimpleITKR_wrap.cxx /build/simpleitk/src/simpleitk/Wrapping/R/SimpleITK.i


#                         srun.swg                            #
#
# This is the basic code that is needed at run time within R to
# provide and define the relevant classes.  It is included
# automatically in the generated code by copying the contents of
# srun.swg into the newly created binding code.


# This could be provided as a separate run-time library but this
# approach allows the code to be included directly into the
# generated bindings and so removes the need to have and install an
# additional library.  We may however end up with multiple copies of
# this and some confusion at run-time as to which class to use. This
# is an issue when we use NAMESPACES as we may need to export certain
# classes.

######################################################################

if(length(getClassDef("RSWIGStruct")) == 0) 
  setClass("RSWIGStruct", representation("VIRTUAL"))



if(length(getClassDef("ExternalReference")) == 0) 
# Should be virtual but this means it loses its slots currently
#representation("VIRTUAL")
  setClass("ExternalReference", representation( ref = "externalptr"))



if(length(getClassDef("NativeRoutinePointer")) == 0) 
  setClass("NativeRoutinePointer", 
              representation(parameterTypes = "character",
                             returnType = "character",
                             "VIRTUAL"), 
              contains = "ExternalReference")

if(length(getClassDef("CRoutinePointer")) == 0) 
  setClass("CRoutinePointer", contains = "NativeRoutinePointer")


if(length(getClassDef("EnumerationValue")) == 0) 
  setClass("EnumerationValue", contains = "integer")


if(!isGeneric("copyToR")) 
 setGeneric("copyToR",
            function(value, obj = new(gsub("Ref$", "", class(value)))) 
               standardGeneric("copyToR"
           ))

setGeneric("delete", function(obj) standardGeneric("delete"))


SWIG_createNewRef = 
function(className, ..., append = TRUE)
{
  f = get(paste("new", className, sep = "_"), mode = "function")

  f(...)
}

if(!isGeneric("copyToC")) 
 setGeneric("copyToC", 
             function(value, obj = SWIG_createNewRef(class(value)))
              standardGeneric("copyToC"
            ))


# 
defineEnumeration =
function(name, .values, where = topenv(parent.frame()), suffix = "Value")
{
   # Mirror the class definitions via the E analogous to .__C__
  defName = paste(".__E__", name, sep = "")
  delayedAssign(defName,  .values, assign.env = where)

  if(nchar(suffix))
    name = paste(name, suffix, sep = "")

  setClass(name, contains = "EnumerationValue", where = where)
}

enumToInteger <- function(name,type)
{
   if (is.character(name)) {
   ans <- as.integer(get(paste(".__E__", type, sep = ""))[name])
   if (is.na(ans)) {warning("enum not found ", name, " ", type)}
   ans
   } 
}

enumFromInteger =
function(i,type)
{
  itemlist <- get(paste(".__E__", type, sep=""))
  names(itemlist)[match(i, itemlist)]
}

coerceIfNotSubclass =
function(obj, type) 
{
    if(!is(obj, type)) {as(obj, type)} else obj
}


setClass("SWIGArray", representation(dims = "integer"), contains = "ExternalReference")

setMethod("length", "SWIGArray", function(x) x@dims[1])


defineEnumeration("SCopyReferences",
                   .values = c( "FALSE" = 0, "TRUE" = 1, "DEEP" = 2))

assert = 
function(condition, message = "")
{
  if(!condition)
    stop(message)

  TRUE
}


if(FALSE) {
print.SWIGFunction =
function(x, ...)
 {
 }
}


#######################################################################

R_SWIG_getCallbackFunctionStack =
function()
{
    # No PACKAGE argument as we don't know what the DLL is.
  .Call("R_SWIG_debug_getCallbackFunctionData")
}

R_SWIG_addCallbackFunctionStack =
function(fun, userData = NULL)
{
    # No PACKAGE argument as we don't know what the DLL is.
  .Call("R_SWIG_R_pushCallbackFunctionData", fun, userData)
}


#######################################################################



setClass('C++Reference', contains = 'ExternalReference')
setClass('_p_std__vectorT_bool_t', contains = 'C++Reference')
setClass('_p_std__vectorT_uint8_t_t', contains = 'C++Reference')
setClass('_p_std__vectorT_int8_t_t', contains = 'C++Reference')
setClass('_p_std__vectorT_uint16_t_t', contains = 'C++Reference')
setClass('_p_std__vectorT_int16_t_t', contains = 'C++Reference')
setClass('_p_std__vectorT_uint32_t_t', contains = 'C++Reference')
setClass('_p_std__vectorT_int32_t_t', contains = 'C++Reference')
setClass('_p_std__vectorT_uint64_t_t', contains = 'C++Reference')
setClass('_p_std__vectorT_int64_t_t', contains = 'C++Reference')
setClass('_p_std__vectorT_float_t', contains = 'C++Reference')
setClass('_p_std__vectorT_double_t', contains = 'C++Reference')
setClass('_p_std__vectorT_itk__simple__Image_t', contains = 'C++Reference')
setClass('_p_std__vectorT_itk__simple__Transform_t', contains = 'C++Reference')
setClass('_p_std__vectorT_std__vectorT_uint32_t_t_std__allocatorT_std__vectorT_uint32_t_t_t_t', contains = 'C++Reference')
setClass('_p_std__vectorT_std__string_t', contains = 'C++Reference')
setClass('_p_std__mapT_double_double_t', contains = 'C++Reference')
setClass('_p_itk__simple__Version', contains = 'C++Reference')
setClass('_p_itk__simple__Image', contains = 'C++Reference')
setClass('_p_itk__simple__ObjectOwnedBase', contains = 'C++Reference')
setClass('_p_itk__simple__Command', contains = c('_p_itk__simple__ObjectOwnedBase'))
setClass('_p_itk__simple__LoggerBase', contains = c('_p_itk__simple__ObjectOwnedBase'))
setClass('_p_itk__simple__ITKLogger', contains = c('_p_itk__simple__LoggerBase'))
setClass('_p_itk__simple__Transform', contains = 'C++Reference')
setClass('_p_itk__simple__BSplineTransform', contains = c('_p_itk__simple__Transform'))
setClass('_p_itk__simple__DisplacementFieldTransform', contains = c('_p_itk__simple__Transform'))
setClass('_p_itk__simple__AffineTransform', contains = c('_p_itk__simple__Transform'))
setClass('_p_itk__simple__Euler3DTransform', contains = c('_p_itk__simple__Transform'))
setClass('_p_itk__simple__Euler2DTransform', contains = c('_p_itk__simple__Transform'))
setClass('_p_itk__simple__ScaleTransform', contains = c('_p_itk__simple__Transform'))
setClass('_p_itk__simple__ScaleSkewVersor3DTransform', contains = c('_p_itk__simple__Transform'))
setClass('_p_itk__simple__ComposeScaleSkewVersor3DTransform', contains = c('_p_itk__simple__Transform'))
setClass('_p_itk__simple__ScaleVersor3DTransform', contains = c('_p_itk__simple__Transform'))
setClass('_p_itk__simple__Similarity2DTransform', contains = c('_p_itk__simple__Transform'))
setClass('_p_itk__simple__Similarity3DTransform', contains = c('_p_itk__simple__Transform'))
setClass('_p_itk__simple__TranslationTransform', contains = c('_p_itk__simple__Transform'))
setClass('_p_itk__simple__VersorTransform', contains = c('_p_itk__simple__Transform'))
setClass('_p_itk__simple__VersorRigid3DTransform', contains = c('_p_itk__simple__Transform'))
setClass('_p_itk__simple__CompositeTransform', contains = c('_p_itk__simple__Transform'))
setClass('_p_itk__simple__ProcessObject', contains = 'C++Reference')
setClass('_p_itk__simple__ImageFilter', contains = c('_p_itk__simple__ProcessObject'))
setClass('_p_itk__simple__ImageFileWriter', contains = c('_p_itk__simple__ProcessObject'))
setClass('_p_itk__simple__ImageSeriesWriter', contains = c('_p_itk__simple__ProcessObject'))
setClass('_p_itk__simple__ImageReaderBase', contains = c('_p_itk__simple__ProcessObject'))
setClass('_p_itk__simple__ImageSeriesReader', contains = c('_p_itk__simple__ImageReaderBase'))
setClass('_p_itk__simple__ImageFileReader', contains = c('_p_itk__simple__ImageReaderBase'))
setClass('_p_itk__simple__ImageViewer', contains = 'C++Reference')
setClass('_p_itk__simple__HashImageFilter', contains = c('_p_itk__simple__ProcessObject'))
setClass('_p_itk__simple__BSplineTransformInitializerFilter', contains = c('_p_itk__simple__ProcessObject'))
setClass('_p_itk__simple__CenteredTransformInitializerFilter', contains = c('_p_itk__simple__ProcessObject'))
setClass('_p_itk__simple__CenteredVersorTransformInitializerFilter', contains = c('_p_itk__simple__ProcessObject'))
setClass('_p_itk__simple__LandmarkBasedTransformInitializerFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__CastImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ExtractImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__PasteImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ImageRegistrationMethod', contains = c('_p_itk__simple__ProcessObject'))
setClass('_p_itk__simple__AbsImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__AbsoluteValueDifferenceImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__AcosImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__AdaptiveHistogramEqualizationImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__AddImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__AdditiveGaussianNoiseImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__AggregateLabelMapFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__AndImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__AntiAliasBinaryImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ApproximateSignedDistanceMapImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__AreaClosingImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__AreaOpeningImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__AsinImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__Atan2ImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__AtanImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BSplineDecompositionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BilateralImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinShrinkImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryClosingByReconstructionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryContourImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryDilateImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryErodeImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryFillholeImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryGrindPeakImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryImageToLabelMapFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryMagnitudeImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryMedianImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryMorphologicalClosingImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryMorphologicalOpeningImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryNotImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryOpeningByReconstructionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryProjectionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryPruningImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryReconstructionByDilationImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryReconstructionByErosionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryThinningImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinaryThresholdProjectionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BinomialBlurImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BitwiseNotImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BlackTopHatImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BoundedReciprocalImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BoxMeanImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__BoxSigmaImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__CannyEdgeDetectionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__CannySegmentationLevelSetImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ChangeLabelImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ChangeLabelLabelMapFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__CheckerBoardImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ClampImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ClosingByReconstructionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__CollidingFrontsImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ComplexToImaginaryImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ComplexToModulusImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ComplexToPhaseImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ComplexToRealImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ComposeImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ConfidenceConnectedImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ConnectedComponentImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ConnectedThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ConstantPadImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ConvolutionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__CosImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__CropImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__CurvatureFlowImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__CyclicShiftImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__DICOMOrientImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__DanielssonDistanceMapImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__DemonsRegistrationFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__DerivativeImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__DilateObjectMorphologyImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__DiscreteGaussianDerivativeImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__DiscreteGaussianImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__DisplacementFieldJacobianDeterminantFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__DivideFloorImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__DivideImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__DivideRealImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__DoubleThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__EdgePotentialImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__EqualImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ErodeObjectMorphologyImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ExpImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ExpNegativeImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ExpandImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__FFTConvolutionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__FFTNormalizedCorrelationImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__FFTPadImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__FFTShiftImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__FastApproximateRankImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__FastMarchingBaseImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__FastMarchingImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__FastMarchingUpwindGradientImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__FlipImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ForwardFFTImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GaborImageSource', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GaussianImageSource', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GradientAnisotropicDiffusionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GradientImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GradientMagnitudeImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GradientMagnitudeRecursiveGaussianImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GradientRecursiveGaussianImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GrayscaleConnectedClosingImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GrayscaleConnectedOpeningImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GrayscaleDilateImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GrayscaleErodeImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GrayscaleFillholeImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GrayscaleGeodesicDilateImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GrayscaleGeodesicErodeImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GrayscaleGrindPeakImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GrayscaleMorphologicalClosingImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GreaterEqualImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GreaterImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__GridImageSource', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__HConcaveImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__HConvexImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__HMaximaImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__HMinimaImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__HalfHermitianToRealInverseFFTImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__HausdorffDistanceImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__HistogramMatchingImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__HuangThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__IntensityWindowingImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__IntermodesThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__InverseDeconvolutionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__InverseDisplacementFieldImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__InverseFFTImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__InvertDisplacementFieldImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__InvertIntensityImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__IsoContourDistanceImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__IsoDataThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__IsolatedConnectedImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__IsolatedWatershedImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__IterativeInverseDisplacementFieldImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__JoinSeriesImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__KittlerIllingworthThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelContourImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelImageToLabelMapFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelIntensityStatisticsImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelMapContourOverlayImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelMapMaskImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelMapOverlayImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelMapToBinaryImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelMapToLabelImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelMapToRGBImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelOverlapMeasuresImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelOverlayImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelShapeStatisticsImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelStatisticsImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelToRGBImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelUniqueLabelMapFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LabelVotingImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LandweberDeconvolutionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LaplacianImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LaplacianRecursiveGaussianImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LaplacianSharpeningImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LessEqualImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LessImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LevelSetMotionRegistrationFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LiThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__Log10ImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__LogImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MagnitudeAndPhaseToComplexImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MaskImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MaskNegatedImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MaskedAssignImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MaskedFFTNormalizedCorrelationImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MaximumEntropyThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MaximumImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MaximumProjectionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MeanImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MeanProjectionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MedianImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MedianProjectionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MergeLabelMapFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MinMaxCurvatureFlowImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MinimumImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MinimumMaximumImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MinimumProjectionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MirrorPadImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ModulusImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MomentsThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MorphologicalGradientImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MorphologicalWatershedImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MultiLabelSTAPLEImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__MultiplyImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__N4BiasFieldCorrectionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__NaryAddImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__NaryMaximumImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__NeighborhoodConnectedImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__NoiseImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__NormalizeImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__NormalizeToConstantImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__NormalizedCorrelationImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__NotEqualImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__NotImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ObjectnessMeasureImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__OpeningByReconstructionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__OrImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__OtsuMultipleThresholdsImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__OtsuThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__PatchBasedDenoisingImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__PermuteAxesImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__PhysicalPointImageSource', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__PowImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__RankImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__RealAndImaginaryToComplexImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__RealToHalfHermitianForwardFFTImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ReconstructionByDilationImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ReconstructionByErosionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__RecursiveGaussianImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__RegionOfInterestImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__RegionalMaximaImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__RegionalMinimaImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ReinitializeLevelSetImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__RelabelComponentImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__RelabelLabelMapFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__RenyiEntropyThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ResampleImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__RescaleIntensityImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__RichardsonLucyDeconvolutionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__RoundImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SLICImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__STAPLEImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SaltAndPepperNoiseImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ScalarConnectedComponentImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ScalarImageKmeansImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ScalarToRGBColormapImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ShanbhagThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ShapeDetectionLevelSetImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ShiftScaleImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ShotNoiseImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ShrinkImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SigmoidImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SignedDanielssonDistanceMapImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SignedMaurerDistanceMapImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SimilarityIndexImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SimpleContourExtractorImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SinImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SliceImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SmoothingRecursiveGaussianImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SobelEdgeDetectionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SpeckleNoiseImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SqrtImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SquareImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SquaredDifferenceImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__StandardDeviationProjectionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__StatisticsImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__StochasticFractalDimensionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SubtractImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SumProjectionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__TanImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__TernaryAddImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__TernaryMagnitudeImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__TernaryMagnitudeSquaredImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__TikhonovDeconvolutionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__TileImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__TobogganImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__TransformGeometryImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__TransformToDisplacementFieldFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__TriangleThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__UnaryMinusImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__UnsharpMaskImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ValuedRegionalMaximaImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ValuedRegionalMinimaImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__VectorConfidenceConnectedImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__VectorConnectedComponentImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__VectorIndexSelectionCastImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__VectorMagnitudeImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__VotingBinaryHoleFillingImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__VotingBinaryImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__WarpImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__WhiteTopHatImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__WienerDeconvolutionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__WrapPadImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__XorImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__YenThresholdImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ZeroCrossingImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__ZeroFluxNeumannPadImageFilter', contains = c('_p_itk__simple__ImageFilter'))
setClass('_p_itk__simple__RCommand', contains = c('_p_itk__simple__Command'))



setMethod('[', "ExternalReference",
function(x,i,j, ..., drop=TRUE) 
if (!is.null(x$"__getitem__")) 
sapply(i, function(n) x$"__getitem__"(i=as.integer(n-1))))

setMethod('[<-' , "ExternalReference",
function(x,i,j, ..., value) 
if (!is.null(x$"__setitem__")) {
sapply(1:length(i), function(n) 
x$"__setitem__"(i=as.integer(i[n]-1), x=value[n]))
x
})

setAs('ExternalReference', 'character',
function(from) {if (!is.null(from$"__str__")) from$"__str__"()})

suppressMessages(suppressWarnings(setMethod('print', 'ExternalReference',
function(x) {print(as(x, "character"))})))

# Start of VectorBool___nonzero__

`VectorBool___nonzero__` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorBool___nonzero__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorBool___nonzero__`, 'returnType') = 'logical'
attr(`VectorBool___nonzero__`, "inputTypes") = c('_p_std__vectorT_bool_t')
class(`VectorBool___nonzero__`) = c("SWIGFunction", class('VectorBool___nonzero__'))

# Start of VectorBool___len__

`VectorBool___len__` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorBool___len__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorBool___len__`, 'returnType') = 'integer'
attr(`VectorBool___len__`, "inputTypes") = c('_p_std__vectorT_bool_t')
class(`VectorBool___len__`) = c("SWIGFunction", class('VectorBool___len__'))

# Start of VectorBool_pop

`VectorBool_pop` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorBool_pop', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorBool_pop`, 'returnType') = 'logical'
attr(`VectorBool_pop`, "inputTypes") = c('_p_std__vectorT_bool_t')
class(`VectorBool_pop`) = c("SWIGFunction", class('VectorBool_pop'))

# Start of VectorBool___getslice__

`VectorBool___getslice__` = function(self, i, j)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorBool___getslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorBool___getslice__`, 'returnType') = '_p_std__vectorT_bool_t'
attr(`VectorBool___getslice__`, "inputTypes") = c('_p_std__vectorT_bool_t', 'integer', 'integer')
class(`VectorBool___getslice__`) = c("SWIGFunction", class('VectorBool___getslice__'))

# Start of VectorBool___setslice__

`VectorBool___setslice__` = function(self, i, j, is)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  is = as.logical(is);
  ;.Call('R_swig_VectorBool___setslice__', self, i, j, is, PACKAGE='SimpleITK');
  
}

attr(`VectorBool___setslice__`, 'returnType') = 'void'
attr(`VectorBool___setslice__`, "inputTypes") = c('_p_std__vectorT_bool_t', 'integer', 'integer', '_p_std__vectorT_bool_t')
class(`VectorBool___setslice__`) = c("SWIGFunction", class('VectorBool___setslice__'))

# Start of VectorBool___delslice__

`VectorBool___delslice__` = function(self, i, j)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorBool___delslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorBool___delslice__`, 'returnType') = 'void'
attr(`VectorBool___delslice__`, "inputTypes") = c('_p_std__vectorT_bool_t', 'integer', 'integer')
class(`VectorBool___delslice__`) = c("SWIGFunction", class('VectorBool___delslice__'))

# Start of VectorBool___delitem__

`VectorBool___delitem__` = function(self, i)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorBool___delitem__', self, i, PACKAGE='SimpleITK');
  
}

attr(`VectorBool___delitem__`, 'returnType') = 'void'
attr(`VectorBool___delitem__`, "inputTypes") = c('_p_std__vectorT_bool_t', 'integer')
class(`VectorBool___delitem__`) = c("SWIGFunction", class('VectorBool___delitem__'))

# Start of VectorBool___getitem__

`VectorBool___getitem__` = function(self, i, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorBool___getitem__', self, i, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorBool___getitem__`, 'returnType') = 'logical'
attr(`VectorBool___getitem__`, "inputTypes") = c('_p_std__vectorT_bool_t', 'integer')
class(`VectorBool___getitem__`) = c("SWIGFunction", class('VectorBool___getitem__'))

# Start of VectorBool___setitem__

`VectorBool___setitem__` = function(self, i, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  x = as.logical(x);
  ;.Call('R_swig_VectorBool___setitem__', self, i, x, PACKAGE='SimpleITK');
  
}

attr(`VectorBool___setitem__`, 'returnType') = 'void'
attr(`VectorBool___setitem__`, "inputTypes") = c('_p_std__vectorT_bool_t', 'integer', 'logical')
class(`VectorBool___setitem__`) = c("SWIGFunction", class('VectorBool___setitem__'))

# Start of VectorBool_append

`VectorBool_append` = function(self, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  x = as.logical(x);
  ;.Call('R_swig_VectorBool_append', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorBool_append`, 'returnType') = 'void'
attr(`VectorBool_append`, "inputTypes") = c('_p_std__vectorT_bool_t', 'logical')
class(`VectorBool_append`) = c("SWIGFunction", class('VectorBool_append'))

# Start of new_VectorBool

`VectorBool__SWIG_0` = function()
{
  ;.Call('R_swig_new_VectorBool__SWIG_0', PACKAGE='SimpleITK');
  
}

attr(`VectorBool__SWIG_0`, 'returnType') = '_p_std__vectorT_bool_t'
class(`VectorBool__SWIG_0`) = c("SWIGFunction", class('VectorBool__SWIG_0'))

# Start of new_VectorBool

`VectorBool__SWIG_1` = function(other)
{
  other = as.logical(other);
  ;.Call('R_swig_new_VectorBool__SWIG_1', other, PACKAGE='SimpleITK');
  
}

attr(`VectorBool__SWIG_1`, 'returnType') = '_p_std__vectorT_bool_t'
attr(`VectorBool__SWIG_1`, "inputTypes") = c('_p_std__vectorT_bool_t')
class(`VectorBool__SWIG_1`) = c("SWIGFunction", class('VectorBool__SWIG_1'))

# Start of VectorBool_empty

`VectorBool_empty` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorBool_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorBool_empty`, 'returnType') = 'logical'
attr(`VectorBool_empty`, "inputTypes") = c('_p_std__vectorT_bool_t')
class(`VectorBool_empty`) = c("SWIGFunction", class('VectorBool_empty'))

# Start of VectorBool_size

`VectorBool_size` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorBool_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorBool_size`, 'returnType') = 'integer'
attr(`VectorBool_size`, "inputTypes") = c('_p_std__vectorT_bool_t')
class(`VectorBool_size`) = c("SWIGFunction", class('VectorBool_size'))

# Start of VectorBool_swap

`VectorBool_swap` = function(self, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  v = as.logical(v);
  ;.Call('R_swig_VectorBool_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`VectorBool_swap`, 'returnType') = 'void'
attr(`VectorBool_swap`, "inputTypes") = c('_p_std__vectorT_bool_t', '_p_std__vectorT_bool_t')
class(`VectorBool_swap`) = c("SWIGFunction", class('VectorBool_swap'))

# Start of VectorBool_clear

`VectorBool_clear` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorBool_clear', self, PACKAGE='SimpleITK');
  
}

attr(`VectorBool_clear`, 'returnType') = 'void'
attr(`VectorBool_clear`, "inputTypes") = c('_p_std__vectorT_bool_t')
class(`VectorBool_clear`) = c("SWIGFunction", class('VectorBool_clear'))

# Start of VectorBool_get_allocator

`VectorBool_get_allocator` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorBool_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_bool_t", ref=ans);
  
  ans
  
}

attr(`VectorBool_get_allocator`, 'returnType') = '_p_std__allocatorT_bool_t'
attr(`VectorBool_get_allocator`, "inputTypes") = c('_p_std__vectorT_bool_t')
class(`VectorBool_get_allocator`) = c("SWIGFunction", class('VectorBool_get_allocator'))

# Start of new_VectorBool

`VectorBool__SWIG_2` = function(size)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  ;.Call('R_swig_new_VectorBool__SWIG_2', size, PACKAGE='SimpleITK');
  
}

attr(`VectorBool__SWIG_2`, 'returnType') = '_p_std__vectorT_bool_t'
attr(`VectorBool__SWIG_2`, "inputTypes") = c('integer')
class(`VectorBool__SWIG_2`) = c("SWIGFunction", class('VectorBool__SWIG_2'))

# Start of VectorBool_pop_back

`VectorBool_pop_back` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorBool_pop_back', self, PACKAGE='SimpleITK');
  
}

attr(`VectorBool_pop_back`, 'returnType') = 'void'
attr(`VectorBool_pop_back`, "inputTypes") = c('_p_std__vectorT_bool_t')
class(`VectorBool_pop_back`) = c("SWIGFunction", class('VectorBool_pop_back'))

# Start of VectorBool_resize

`VectorBool_resize__SWIG_0` = function(self, new_size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  ;.Call('R_swig_VectorBool_resize__SWIG_0', self, new_size, PACKAGE='SimpleITK');
  
}

attr(`VectorBool_resize__SWIG_0`, 'returnType') = 'void'
attr(`VectorBool_resize__SWIG_0`, "inputTypes") = c('_p_std__vectorT_bool_t', 'integer')
class(`VectorBool_resize__SWIG_0`) = c("SWIGFunction", class('VectorBool_resize__SWIG_0'))

# Start of new_VectorBool

`VectorBool__SWIG_3` = function(size, value)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  value = as.logical(value);
  ;.Call('R_swig_new_VectorBool__SWIG_3', size, value, PACKAGE='SimpleITK');
  
}

attr(`VectorBool__SWIG_3`, 'returnType') = '_p_std__vectorT_bool_t'
attr(`VectorBool__SWIG_3`, "inputTypes") = c('integer', 'logical')
class(`VectorBool__SWIG_3`) = c("SWIGFunction", class('VectorBool__SWIG_3'))

`VectorBool` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 0) {
    f <- VectorBool__SWIG_0; 
  } else if (argc == 1) {
    if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- VectorBool__SWIG_2; 
    }
    else if ( is.logical(argv[[1]]) ) {
      f <- VectorBool__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- VectorBool__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorBool with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorBool_push_back

`VectorBool_push_back` = function(self, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  x = as.logical(x);
  ;.Call('R_swig_VectorBool_push_back', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorBool_push_back`, 'returnType') = 'void'
attr(`VectorBool_push_back`, "inputTypes") = c('_p_std__vectorT_bool_t', 'logical')
class(`VectorBool_push_back`) = c("SWIGFunction", class('VectorBool_push_back'))

# Start of VectorBool_front

`VectorBool_front` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorBool_front', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorBool_front`, 'returnType') = 'logical'
attr(`VectorBool_front`, "inputTypes") = c('_p_std__vectorT_bool_t')
class(`VectorBool_front`) = c("SWIGFunction", class('VectorBool_front'))

# Start of VectorBool_back

`VectorBool_back` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorBool_back', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorBool_back`, 'returnType') = 'logical'
attr(`VectorBool_back`, "inputTypes") = c('_p_std__vectorT_bool_t')
class(`VectorBool_back`) = c("SWIGFunction", class('VectorBool_back'))

# Start of VectorBool_assign

`VectorBool_assign` = function(self, n, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  x = as.logical(x);
  ;.Call('R_swig_VectorBool_assign', self, n, x, PACKAGE='SimpleITK');
  
}

attr(`VectorBool_assign`, 'returnType') = 'void'
attr(`VectorBool_assign`, "inputTypes") = c('_p_std__vectorT_bool_t', 'integer', 'logical')
class(`VectorBool_assign`) = c("SWIGFunction", class('VectorBool_assign'))

# Start of VectorBool_resize

`VectorBool_resize__SWIG_1` = function(self, new_size, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  x = as.logical(x);
  ;.Call('R_swig_VectorBool_resize__SWIG_1', self, new_size, x, PACKAGE='SimpleITK');
  
}

attr(`VectorBool_resize__SWIG_1`, 'returnType') = 'void'
attr(`VectorBool_resize__SWIG_1`, "inputTypes") = c('_p_std__vectorT_bool_t', 'integer', 'logical')
class(`VectorBool_resize__SWIG_1`) = c("SWIGFunction", class('VectorBool_resize__SWIG_1'))

`VectorBool_resize` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( is.logical(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorBool_resize__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( is.logical(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- VectorBool_resize__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorBool_resize with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorBool_reserve

`VectorBool_reserve` = function(self, n)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;.Call('R_swig_VectorBool_reserve', self, n, PACKAGE='SimpleITK');
  
}

attr(`VectorBool_reserve`, 'returnType') = 'void'
attr(`VectorBool_reserve`, "inputTypes") = c('_p_std__vectorT_bool_t', 'integer')
class(`VectorBool_reserve`) = c("SWIGFunction", class('VectorBool_reserve'))

# Start of VectorBool_capacity

`VectorBool_capacity` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorBool_capacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorBool_capacity`, 'returnType') = 'integer'
attr(`VectorBool_capacity`, "inputTypes") = c('_p_std__vectorT_bool_t')
class(`VectorBool_capacity`) = c("SWIGFunction", class('VectorBool_capacity'))

# Start of delete_VectorBool

`delete_VectorBool` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_delete_VectorBool', self, PACKAGE='SimpleITK');
  
}

attr(`delete_VectorBool`, 'returnType') = 'void'
attr(`delete_VectorBool`, "inputTypes") = c('_p_std__vectorT_bool_t')
class(`delete_VectorBool`) = c("SWIGFunction", class('delete_VectorBool'))

# Start of accessor method for std::vector< bool >
setMethod('$', '_p_std__vectorT_bool_t', function(x, name)

{
  accessorFuns = list('__nonzero__' = VectorBool___nonzero__, '__len__' = VectorBool___len__, 'pop' = VectorBool_pop, '__getslice__' = VectorBool___getslice__, '__setslice__' = VectorBool___setslice__, '__delslice__' = VectorBool___delslice__, '__delitem__' = VectorBool___delitem__, '__getitem__' = VectorBool___getitem__, '__setitem__' = VectorBool___setitem__, 'append' = VectorBool_append, 'empty' = VectorBool_empty, 'size' = VectorBool_size, 'swap' = VectorBool_swap, 'clear' = VectorBool_clear, 'get_allocator' = VectorBool_get_allocator, 'pop_back' = VectorBool_pop_back, 'resize' = VectorBool_resize, 'push_back' = VectorBool_push_back, 'front' = VectorBool_front, 'back' = VectorBool_back, 'assign' = VectorBool_assign, 'reserve' = VectorBool_reserve, 'capacity' = VectorBool_capacity);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::vector< bool >
# Start of VectorUInt8___nonzero__

`VectorUInt8___nonzero__` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt8___nonzero__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8___nonzero__`, 'returnType') = 'logical'
attr(`VectorUInt8___nonzero__`, "inputTypes") = c('integer')
class(`VectorUInt8___nonzero__`) = c("SWIGFunction", class('VectorUInt8___nonzero__'))

# Start of VectorUInt8___len__

`VectorUInt8___len__` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt8___len__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8___len__`, 'returnType') = 'integer'
attr(`VectorUInt8___len__`, "inputTypes") = c('integer')
class(`VectorUInt8___len__`) = c("SWIGFunction", class('VectorUInt8___len__'))

# Start of VectorUInt8_pop

`VectorUInt8_pop` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt8_pop', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8_pop`, 'returnType') = 'integer'
attr(`VectorUInt8_pop`, "inputTypes") = c('integer')
class(`VectorUInt8_pop`) = c("SWIGFunction", class('VectorUInt8_pop'))

# Start of VectorUInt8___getslice__

`VectorUInt8___getslice__` = function(self, i, j)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorUInt8___getslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8___getslice__`, 'returnType') = 'integer'
attr(`VectorUInt8___getslice__`, "inputTypes") = c('integer', 'integer', 'integer')
class(`VectorUInt8___getslice__`) = c("SWIGFunction", class('VectorUInt8___getslice__'))

# Start of VectorUInt8___setslice__

`VectorUInt8___setslice__` = function(self, i, j, is)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  is = as.integer(is);
  ;.Call('R_swig_VectorUInt8___setslice__', self, i, j, is, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8___setslice__`, 'returnType') = 'void'
attr(`VectorUInt8___setslice__`, "inputTypes") = c('integer', 'integer', 'integer', 'integer')
class(`VectorUInt8___setslice__`) = c("SWIGFunction", class('VectorUInt8___setslice__'))

# Start of VectorUInt8___delslice__

`VectorUInt8___delslice__` = function(self, i, j)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorUInt8___delslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8___delslice__`, 'returnType') = 'void'
attr(`VectorUInt8___delslice__`, "inputTypes") = c('integer', 'integer', 'integer')
class(`VectorUInt8___delslice__`) = c("SWIGFunction", class('VectorUInt8___delslice__'))

# Start of VectorUInt8___delitem__

`VectorUInt8___delitem__` = function(self, i)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorUInt8___delitem__', self, i, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8___delitem__`, 'returnType') = 'void'
attr(`VectorUInt8___delitem__`, "inputTypes") = c('integer', 'integer')
class(`VectorUInt8___delitem__`) = c("SWIGFunction", class('VectorUInt8___delitem__'))

# Start of VectorUInt8___getitem__

`VectorUInt8___getitem__` = function(self, i, .copy = FALSE)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorUInt8___getitem__', self, i, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8___getitem__`, 'returnType') = '_p_unsigned_char'
attr(`VectorUInt8___getitem__`, "inputTypes") = c('integer', 'integer')
class(`VectorUInt8___getitem__`) = c("SWIGFunction", class('VectorUInt8___getitem__'))

# Start of VectorUInt8___setitem__

`VectorUInt8___setitem__` = function(self, i, x)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt8___setitem__', self, i, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8___setitem__`, 'returnType') = 'void'
attr(`VectorUInt8___setitem__`, "inputTypes") = c('integer', 'integer', '_p_unsigned_char')
class(`VectorUInt8___setitem__`) = c("SWIGFunction", class('VectorUInt8___setitem__'))

# Start of VectorUInt8_append

`VectorUInt8_append` = function(self, x)
{
  self = as.integer(self);
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt8_append', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8_append`, 'returnType') = 'void'
attr(`VectorUInt8_append`, "inputTypes") = c('integer', '_p_unsigned_char')
class(`VectorUInt8_append`) = c("SWIGFunction", class('VectorUInt8_append'))

# Start of new_VectorUInt8

`VectorUInt8__SWIG_0` = function()
{
  ;.Call('R_swig_new_VectorUInt8__SWIG_0', PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8__SWIG_0`, 'returnType') = 'integer'
class(`VectorUInt8__SWIG_0`) = c("SWIGFunction", class('VectorUInt8__SWIG_0'))

# Start of new_VectorUInt8

`VectorUInt8__SWIG_1` = function(other)
{
  other = as.integer(other);
  ;.Call('R_swig_new_VectorUInt8__SWIG_1', other, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8__SWIG_1`, 'returnType') = 'integer'
attr(`VectorUInt8__SWIG_1`, "inputTypes") = c('integer')
class(`VectorUInt8__SWIG_1`) = c("SWIGFunction", class('VectorUInt8__SWIG_1'))

# Start of VectorUInt8_empty

`VectorUInt8_empty` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt8_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8_empty`, 'returnType') = 'logical'
attr(`VectorUInt8_empty`, "inputTypes") = c('integer')
class(`VectorUInt8_empty`) = c("SWIGFunction", class('VectorUInt8_empty'))

# Start of VectorUInt8_size

`VectorUInt8_size` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt8_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8_size`, 'returnType') = 'integer'
attr(`VectorUInt8_size`, "inputTypes") = c('integer')
class(`VectorUInt8_size`) = c("SWIGFunction", class('VectorUInt8_size'))

# Start of VectorUInt8_swap

`VectorUInt8_swap` = function(self, v)
{
  self = as.integer(self);
  v = as.integer(v);
  ;.Call('R_swig_VectorUInt8_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8_swap`, 'returnType') = 'void'
attr(`VectorUInt8_swap`, "inputTypes") = c('integer', 'integer')
class(`VectorUInt8_swap`) = c("SWIGFunction", class('VectorUInt8_swap'))

# Start of VectorUInt8_clear

`VectorUInt8_clear` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt8_clear', self, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8_clear`, 'returnType') = 'void'
attr(`VectorUInt8_clear`, "inputTypes") = c('integer')
class(`VectorUInt8_clear`) = c("SWIGFunction", class('VectorUInt8_clear'))

# Start of VectorUInt8_get_allocator

`VectorUInt8_get_allocator` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;ans = .Call('R_swig_VectorUInt8_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_unsigned_char_t", ref=ans);
  
  ans
  
}

attr(`VectorUInt8_get_allocator`, 'returnType') = '_p_std__allocatorT_unsigned_char_t'
attr(`VectorUInt8_get_allocator`, "inputTypes") = c('integer')
class(`VectorUInt8_get_allocator`) = c("SWIGFunction", class('VectorUInt8_get_allocator'))

# Start of new_VectorUInt8

`VectorUInt8__SWIG_2` = function(size)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  ;.Call('R_swig_new_VectorUInt8__SWIG_2', size, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8__SWIG_2`, 'returnType') = 'integer'
attr(`VectorUInt8__SWIG_2`, "inputTypes") = c('integer')
class(`VectorUInt8__SWIG_2`) = c("SWIGFunction", class('VectorUInt8__SWIG_2'))

# Start of VectorUInt8_pop_back

`VectorUInt8_pop_back` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt8_pop_back', self, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8_pop_back`, 'returnType') = 'void'
attr(`VectorUInt8_pop_back`, "inputTypes") = c('integer')
class(`VectorUInt8_pop_back`) = c("SWIGFunction", class('VectorUInt8_pop_back'))

# Start of VectorUInt8_resize

`VectorUInt8_resize__SWIG_0` = function(self, new_size)
{
  self = as.integer(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  ;.Call('R_swig_VectorUInt8_resize__SWIG_0', self, new_size, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8_resize__SWIG_0`, 'returnType') = 'void'
attr(`VectorUInt8_resize__SWIG_0`, "inputTypes") = c('integer', 'integer')
class(`VectorUInt8_resize__SWIG_0`) = c("SWIGFunction", class('VectorUInt8_resize__SWIG_0'))

# Start of new_VectorUInt8

`VectorUInt8__SWIG_3` = function(size, value)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  if (inherits(value, "ExternalReference")) value = slot(value,"ref"); 
  ;.Call('R_swig_new_VectorUInt8__SWIG_3', size, value, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8__SWIG_3`, 'returnType') = 'integer'
attr(`VectorUInt8__SWIG_3`, "inputTypes") = c('integer', '_p_unsigned_char')
class(`VectorUInt8__SWIG_3`) = c("SWIGFunction", class('VectorUInt8__SWIG_3'))

`VectorUInt8` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 0) {
    f <- VectorUInt8__SWIG_0; 
  } else if (argc == 1) {
    if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- VectorUInt8__SWIG_2; 
    }
    else if ( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) {
      f <- VectorUInt8__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_unsigned_char') && length(argv[[2]]) == 1 )) {
      f <- VectorUInt8__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorUInt8 with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorUInt8_push_back

`VectorUInt8_push_back` = function(self, x)
{
  self = as.integer(self);
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt8_push_back', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8_push_back`, 'returnType') = 'void'
attr(`VectorUInt8_push_back`, "inputTypes") = c('integer', '_p_unsigned_char')
class(`VectorUInt8_push_back`) = c("SWIGFunction", class('VectorUInt8_push_back'))

# Start of VectorUInt8_front

`VectorUInt8_front` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt8_front', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8_front`, 'returnType') = '_p_unsigned_char'
attr(`VectorUInt8_front`, "inputTypes") = c('integer')
class(`VectorUInt8_front`) = c("SWIGFunction", class('VectorUInt8_front'))

# Start of VectorUInt8_back

`VectorUInt8_back` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt8_back', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8_back`, 'returnType') = '_p_unsigned_char'
attr(`VectorUInt8_back`, "inputTypes") = c('integer')
class(`VectorUInt8_back`) = c("SWIGFunction", class('VectorUInt8_back'))

# Start of VectorUInt8_assign

`VectorUInt8_assign` = function(self, n, x)
{
  self = as.integer(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt8_assign', self, n, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8_assign`, 'returnType') = 'void'
attr(`VectorUInt8_assign`, "inputTypes") = c('integer', 'integer', '_p_unsigned_char')
class(`VectorUInt8_assign`) = c("SWIGFunction", class('VectorUInt8_assign'))

# Start of VectorUInt8_resize

`VectorUInt8_resize__SWIG_1` = function(self, new_size, x)
{
  self = as.integer(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt8_resize__SWIG_1', self, new_size, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8_resize__SWIG_1`, 'returnType') = 'void'
attr(`VectorUInt8_resize__SWIG_1`, "inputTypes") = c('integer', 'integer', '_p_unsigned_char')
class(`VectorUInt8_resize__SWIG_1`) = c("SWIGFunction", class('VectorUInt8_resize__SWIG_1'))

`VectorUInt8_resize` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorUInt8_resize__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_unsigned_char') && length(argv[[3]]) == 1 )) {
      f <- VectorUInt8_resize__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorUInt8_resize with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorUInt8_reserve

`VectorUInt8_reserve` = function(self, n)
{
  self = as.integer(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;.Call('R_swig_VectorUInt8_reserve', self, n, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8_reserve`, 'returnType') = 'void'
attr(`VectorUInt8_reserve`, "inputTypes") = c('integer', 'integer')
class(`VectorUInt8_reserve`) = c("SWIGFunction", class('VectorUInt8_reserve'))

# Start of VectorUInt8_capacity

`VectorUInt8_capacity` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt8_capacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt8_capacity`, 'returnType') = 'integer'
attr(`VectorUInt8_capacity`, "inputTypes") = c('integer')
class(`VectorUInt8_capacity`) = c("SWIGFunction", class('VectorUInt8_capacity'))

# Start of delete_VectorUInt8

`delete_VectorUInt8` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_delete_VectorUInt8', self, PACKAGE='SimpleITK');
  
}

attr(`delete_VectorUInt8`, 'returnType') = 'void'
attr(`delete_VectorUInt8`, "inputTypes") = c('integer')
class(`delete_VectorUInt8`) = c("SWIGFunction", class('delete_VectorUInt8'))

# Start of accessor method for std::vector< uint8_t >
setMethod('$', '_p_std__vectorT_uint8_t_t', function(x, name)

{
  accessorFuns = list('__nonzero__' = VectorUInt8___nonzero__, '__len__' = VectorUInt8___len__, 'pop' = VectorUInt8_pop, '__getslice__' = VectorUInt8___getslice__, '__setslice__' = VectorUInt8___setslice__, '__delslice__' = VectorUInt8___delslice__, '__delitem__' = VectorUInt8___delitem__, '__getitem__' = VectorUInt8___getitem__, '__setitem__' = VectorUInt8___setitem__, 'append' = VectorUInt8_append, 'empty' = VectorUInt8_empty, 'size' = VectorUInt8_size, 'swap' = VectorUInt8_swap, 'clear' = VectorUInt8_clear, 'get_allocator' = VectorUInt8_get_allocator, 'pop_back' = VectorUInt8_pop_back, 'resize' = VectorUInt8_resize, 'push_back' = VectorUInt8_push_back, 'front' = VectorUInt8_front, 'back' = VectorUInt8_back, 'assign' = VectorUInt8_assign, 'reserve' = VectorUInt8_reserve, 'capacity' = VectorUInt8_capacity);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::vector< uint8_t >
# Start of VectorInt8___nonzero__

`VectorInt8___nonzero__` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt8___nonzero__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt8___nonzero__`, 'returnType') = 'logical'
attr(`VectorInt8___nonzero__`, "inputTypes") = c('integer')
class(`VectorInt8___nonzero__`) = c("SWIGFunction", class('VectorInt8___nonzero__'))

# Start of VectorInt8___len__

`VectorInt8___len__` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt8___len__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt8___len__`, 'returnType') = 'integer'
attr(`VectorInt8___len__`, "inputTypes") = c('integer')
class(`VectorInt8___len__`) = c("SWIGFunction", class('VectorInt8___len__'))

# Start of VectorInt8_pop

`VectorInt8_pop` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt8_pop', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt8_pop`, 'returnType') = 'integer'
attr(`VectorInt8_pop`, "inputTypes") = c('integer')
class(`VectorInt8_pop`) = c("SWIGFunction", class('VectorInt8_pop'))

# Start of VectorInt8___getslice__

`VectorInt8___getslice__` = function(self, i, j)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorInt8___getslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8___getslice__`, 'returnType') = 'integer'
attr(`VectorInt8___getslice__`, "inputTypes") = c('integer', 'integer', 'integer')
class(`VectorInt8___getslice__`) = c("SWIGFunction", class('VectorInt8___getslice__'))

# Start of VectorInt8___setslice__

`VectorInt8___setslice__` = function(self, i, j, is)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  is = as.integer(is);
  ;.Call('R_swig_VectorInt8___setslice__', self, i, j, is, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8___setslice__`, 'returnType') = 'void'
attr(`VectorInt8___setslice__`, "inputTypes") = c('integer', 'integer', 'integer', 'integer')
class(`VectorInt8___setslice__`) = c("SWIGFunction", class('VectorInt8___setslice__'))

# Start of VectorInt8___delslice__

`VectorInt8___delslice__` = function(self, i, j)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorInt8___delslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8___delslice__`, 'returnType') = 'void'
attr(`VectorInt8___delslice__`, "inputTypes") = c('integer', 'integer', 'integer')
class(`VectorInt8___delslice__`) = c("SWIGFunction", class('VectorInt8___delslice__'))

# Start of VectorInt8___delitem__

`VectorInt8___delitem__` = function(self, i)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorInt8___delitem__', self, i, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8___delitem__`, 'returnType') = 'void'
attr(`VectorInt8___delitem__`, "inputTypes") = c('integer', 'integer')
class(`VectorInt8___delitem__`) = c("SWIGFunction", class('VectorInt8___delitem__'))

# Start of VectorInt8___getitem__

`VectorInt8___getitem__` = function(self, i, .copy = FALSE)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorInt8___getitem__', self, i, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt8___getitem__`, 'returnType') = '_p_signed_char'
attr(`VectorInt8___getitem__`, "inputTypes") = c('integer', 'integer')
class(`VectorInt8___getitem__`) = c("SWIGFunction", class('VectorInt8___getitem__'))

# Start of VectorInt8___setitem__

`VectorInt8___setitem__` = function(self, i, x)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorInt8___setitem__', self, i, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8___setitem__`, 'returnType') = 'void'
attr(`VectorInt8___setitem__`, "inputTypes") = c('integer', 'integer', '_p_signed_char')
class(`VectorInt8___setitem__`) = c("SWIGFunction", class('VectorInt8___setitem__'))

# Start of VectorInt8_append

`VectorInt8_append` = function(self, x)
{
  self = as.integer(self);
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorInt8_append', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8_append`, 'returnType') = 'void'
attr(`VectorInt8_append`, "inputTypes") = c('integer', '_p_signed_char')
class(`VectorInt8_append`) = c("SWIGFunction", class('VectorInt8_append'))

# Start of new_VectorInt8

`VectorInt8__SWIG_0` = function()
{
  ;.Call('R_swig_new_VectorInt8__SWIG_0', PACKAGE='SimpleITK');
  
}

attr(`VectorInt8__SWIG_0`, 'returnType') = 'integer'
class(`VectorInt8__SWIG_0`) = c("SWIGFunction", class('VectorInt8__SWIG_0'))

# Start of new_VectorInt8

`VectorInt8__SWIG_1` = function(other)
{
  other = as.integer(other);
  ;.Call('R_swig_new_VectorInt8__SWIG_1', other, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8__SWIG_1`, 'returnType') = 'integer'
attr(`VectorInt8__SWIG_1`, "inputTypes") = c('integer')
class(`VectorInt8__SWIG_1`) = c("SWIGFunction", class('VectorInt8__SWIG_1'))

# Start of VectorInt8_empty

`VectorInt8_empty` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt8_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt8_empty`, 'returnType') = 'logical'
attr(`VectorInt8_empty`, "inputTypes") = c('integer')
class(`VectorInt8_empty`) = c("SWIGFunction", class('VectorInt8_empty'))

# Start of VectorInt8_size

`VectorInt8_size` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt8_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt8_size`, 'returnType') = 'integer'
attr(`VectorInt8_size`, "inputTypes") = c('integer')
class(`VectorInt8_size`) = c("SWIGFunction", class('VectorInt8_size'))

# Start of VectorInt8_swap

`VectorInt8_swap` = function(self, v)
{
  self = as.integer(self);
  v = as.integer(v);
  ;.Call('R_swig_VectorInt8_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8_swap`, 'returnType') = 'void'
attr(`VectorInt8_swap`, "inputTypes") = c('integer', 'integer')
class(`VectorInt8_swap`) = c("SWIGFunction", class('VectorInt8_swap'))

# Start of VectorInt8_clear

`VectorInt8_clear` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt8_clear', self, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8_clear`, 'returnType') = 'void'
attr(`VectorInt8_clear`, "inputTypes") = c('integer')
class(`VectorInt8_clear`) = c("SWIGFunction", class('VectorInt8_clear'))

# Start of VectorInt8_get_allocator

`VectorInt8_get_allocator` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;ans = .Call('R_swig_VectorInt8_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_signed_char_t", ref=ans);
  
  ans
  
}

attr(`VectorInt8_get_allocator`, 'returnType') = '_p_std__allocatorT_signed_char_t'
attr(`VectorInt8_get_allocator`, "inputTypes") = c('integer')
class(`VectorInt8_get_allocator`) = c("SWIGFunction", class('VectorInt8_get_allocator'))

# Start of new_VectorInt8

`VectorInt8__SWIG_2` = function(size)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  ;.Call('R_swig_new_VectorInt8__SWIG_2', size, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8__SWIG_2`, 'returnType') = 'integer'
attr(`VectorInt8__SWIG_2`, "inputTypes") = c('integer')
class(`VectorInt8__SWIG_2`) = c("SWIGFunction", class('VectorInt8__SWIG_2'))

# Start of VectorInt8_pop_back

`VectorInt8_pop_back` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt8_pop_back', self, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8_pop_back`, 'returnType') = 'void'
attr(`VectorInt8_pop_back`, "inputTypes") = c('integer')
class(`VectorInt8_pop_back`) = c("SWIGFunction", class('VectorInt8_pop_back'))

# Start of VectorInt8_resize

`VectorInt8_resize__SWIG_0` = function(self, new_size)
{
  self = as.integer(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  ;.Call('R_swig_VectorInt8_resize__SWIG_0', self, new_size, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8_resize__SWIG_0`, 'returnType') = 'void'
attr(`VectorInt8_resize__SWIG_0`, "inputTypes") = c('integer', 'integer')
class(`VectorInt8_resize__SWIG_0`) = c("SWIGFunction", class('VectorInt8_resize__SWIG_0'))

# Start of new_VectorInt8

`VectorInt8__SWIG_3` = function(size, value)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  if (inherits(value, "ExternalReference")) value = slot(value,"ref"); 
  ;.Call('R_swig_new_VectorInt8__SWIG_3', size, value, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8__SWIG_3`, 'returnType') = 'integer'
attr(`VectorInt8__SWIG_3`, "inputTypes") = c('integer', '_p_signed_char')
class(`VectorInt8__SWIG_3`) = c("SWIGFunction", class('VectorInt8__SWIG_3'))

`VectorInt8` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 0) {
    f <- VectorInt8__SWIG_0; 
  } else if (argc == 1) {
    if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- VectorInt8__SWIG_2; 
    }
    else if ( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) {
      f <- VectorInt8__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_signed_char') && length(argv[[2]]) == 1 )) {
      f <- VectorInt8__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorInt8 with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorInt8_push_back

`VectorInt8_push_back` = function(self, x)
{
  self = as.integer(self);
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorInt8_push_back', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8_push_back`, 'returnType') = 'void'
attr(`VectorInt8_push_back`, "inputTypes") = c('integer', '_p_signed_char')
class(`VectorInt8_push_back`) = c("SWIGFunction", class('VectorInt8_push_back'))

# Start of VectorInt8_front

`VectorInt8_front` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt8_front', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt8_front`, 'returnType') = '_p_signed_char'
attr(`VectorInt8_front`, "inputTypes") = c('integer')
class(`VectorInt8_front`) = c("SWIGFunction", class('VectorInt8_front'))

# Start of VectorInt8_back

`VectorInt8_back` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt8_back', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt8_back`, 'returnType') = '_p_signed_char'
attr(`VectorInt8_back`, "inputTypes") = c('integer')
class(`VectorInt8_back`) = c("SWIGFunction", class('VectorInt8_back'))

# Start of VectorInt8_assign

`VectorInt8_assign` = function(self, n, x)
{
  self = as.integer(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorInt8_assign', self, n, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8_assign`, 'returnType') = 'void'
attr(`VectorInt8_assign`, "inputTypes") = c('integer', 'integer', '_p_signed_char')
class(`VectorInt8_assign`) = c("SWIGFunction", class('VectorInt8_assign'))

# Start of VectorInt8_resize

`VectorInt8_resize__SWIG_1` = function(self, new_size, x)
{
  self = as.integer(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorInt8_resize__SWIG_1', self, new_size, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8_resize__SWIG_1`, 'returnType') = 'void'
attr(`VectorInt8_resize__SWIG_1`, "inputTypes") = c('integer', 'integer', '_p_signed_char')
class(`VectorInt8_resize__SWIG_1`) = c("SWIGFunction", class('VectorInt8_resize__SWIG_1'))

`VectorInt8_resize` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorInt8_resize__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_signed_char') && length(argv[[3]]) == 1 )) {
      f <- VectorInt8_resize__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorInt8_resize with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorInt8_reserve

`VectorInt8_reserve` = function(self, n)
{
  self = as.integer(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;.Call('R_swig_VectorInt8_reserve', self, n, PACKAGE='SimpleITK');
  
}

attr(`VectorInt8_reserve`, 'returnType') = 'void'
attr(`VectorInt8_reserve`, "inputTypes") = c('integer', 'integer')
class(`VectorInt8_reserve`) = c("SWIGFunction", class('VectorInt8_reserve'))

# Start of VectorInt8_capacity

`VectorInt8_capacity` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt8_capacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt8_capacity`, 'returnType') = 'integer'
attr(`VectorInt8_capacity`, "inputTypes") = c('integer')
class(`VectorInt8_capacity`) = c("SWIGFunction", class('VectorInt8_capacity'))

# Start of delete_VectorInt8

`delete_VectorInt8` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_delete_VectorInt8', self, PACKAGE='SimpleITK');
  
}

attr(`delete_VectorInt8`, 'returnType') = 'void'
attr(`delete_VectorInt8`, "inputTypes") = c('integer')
class(`delete_VectorInt8`) = c("SWIGFunction", class('delete_VectorInt8'))

# Start of accessor method for std::vector< int8_t >
setMethod('$', '_p_std__vectorT_int8_t_t', function(x, name)

{
  accessorFuns = list('__nonzero__' = VectorInt8___nonzero__, '__len__' = VectorInt8___len__, 'pop' = VectorInt8_pop, '__getslice__' = VectorInt8___getslice__, '__setslice__' = VectorInt8___setslice__, '__delslice__' = VectorInt8___delslice__, '__delitem__' = VectorInt8___delitem__, '__getitem__' = VectorInt8___getitem__, '__setitem__' = VectorInt8___setitem__, 'append' = VectorInt8_append, 'empty' = VectorInt8_empty, 'size' = VectorInt8_size, 'swap' = VectorInt8_swap, 'clear' = VectorInt8_clear, 'get_allocator' = VectorInt8_get_allocator, 'pop_back' = VectorInt8_pop_back, 'resize' = VectorInt8_resize, 'push_back' = VectorInt8_push_back, 'front' = VectorInt8_front, 'back' = VectorInt8_back, 'assign' = VectorInt8_assign, 'reserve' = VectorInt8_reserve, 'capacity' = VectorInt8_capacity);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::vector< int8_t >
# Start of VectorUInt16___nonzero__

`VectorUInt16___nonzero__` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUInt16___nonzero__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16___nonzero__`, 'returnType') = 'logical'
attr(`VectorUInt16___nonzero__`, "inputTypes") = c('_p_std__vectorT_uint16_t_t')
class(`VectorUInt16___nonzero__`) = c("SWIGFunction", class('VectorUInt16___nonzero__'))

# Start of VectorUInt16___len__

`VectorUInt16___len__` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUInt16___len__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16___len__`, 'returnType') = 'integer'
attr(`VectorUInt16___len__`, "inputTypes") = c('_p_std__vectorT_uint16_t_t')
class(`VectorUInt16___len__`) = c("SWIGFunction", class('VectorUInt16___len__'))

# Start of VectorUInt16_pop

`VectorUInt16_pop` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUInt16_pop', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16_pop`, 'returnType') = 'integer'
attr(`VectorUInt16_pop`, "inputTypes") = c('_p_std__vectorT_uint16_t_t')
class(`VectorUInt16_pop`) = c("SWIGFunction", class('VectorUInt16_pop'))

# Start of VectorUInt16___getslice__

`VectorUInt16___getslice__` = function(self, i, j)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorUInt16___getslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16___getslice__`, 'returnType') = '_p_std__vectorT_uint16_t_t'
attr(`VectorUInt16___getslice__`, "inputTypes") = c('_p_std__vectorT_uint16_t_t', 'integer', 'integer')
class(`VectorUInt16___getslice__`) = c("SWIGFunction", class('VectorUInt16___getslice__'))

# Start of VectorUInt16___setslice__

`VectorUInt16___setslice__` = function(self, i, j, is)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  if (inherits(is, "ExternalReference")) is = slot(is,"ref"); 
  ;.Call('R_swig_VectorUInt16___setslice__', self, i, j, is, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16___setslice__`, 'returnType') = 'void'
attr(`VectorUInt16___setslice__`, "inputTypes") = c('_p_std__vectorT_uint16_t_t', 'integer', 'integer', '_p_std__vectorT_uint16_t_t')
class(`VectorUInt16___setslice__`) = c("SWIGFunction", class('VectorUInt16___setslice__'))

# Start of VectorUInt16___delslice__

`VectorUInt16___delslice__` = function(self, i, j)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorUInt16___delslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16___delslice__`, 'returnType') = 'void'
attr(`VectorUInt16___delslice__`, "inputTypes") = c('_p_std__vectorT_uint16_t_t', 'integer', 'integer')
class(`VectorUInt16___delslice__`) = c("SWIGFunction", class('VectorUInt16___delslice__'))

# Start of VectorUInt16___delitem__

`VectorUInt16___delitem__` = function(self, i)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorUInt16___delitem__', self, i, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16___delitem__`, 'returnType') = 'void'
attr(`VectorUInt16___delitem__`, "inputTypes") = c('_p_std__vectorT_uint16_t_t', 'integer')
class(`VectorUInt16___delitem__`) = c("SWIGFunction", class('VectorUInt16___delitem__'))

# Start of VectorUInt16___getitem__

`VectorUInt16___getitem__` = function(self, i, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorUInt16___getitem__', self, i, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16___getitem__`, 'returnType') = '_p_unsigned_short'
attr(`VectorUInt16___getitem__`, "inputTypes") = c('_p_std__vectorT_uint16_t_t', 'integer')
class(`VectorUInt16___getitem__`) = c("SWIGFunction", class('VectorUInt16___getitem__'))

# Start of VectorUInt16___setitem__

`VectorUInt16___setitem__` = function(self, i, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt16___setitem__', self, i, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16___setitem__`, 'returnType') = 'void'
attr(`VectorUInt16___setitem__`, "inputTypes") = c('_p_std__vectorT_uint16_t_t', 'integer', '_p_unsigned_short')
class(`VectorUInt16___setitem__`) = c("SWIGFunction", class('VectorUInt16___setitem__'))

# Start of VectorUInt16_append

`VectorUInt16_append` = function(self, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt16_append', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16_append`, 'returnType') = 'void'
attr(`VectorUInt16_append`, "inputTypes") = c('_p_std__vectorT_uint16_t_t', '_p_unsigned_short')
class(`VectorUInt16_append`) = c("SWIGFunction", class('VectorUInt16_append'))

# Start of new_VectorUInt16

`VectorUInt16__SWIG_0` = function()
{
  ;.Call('R_swig_new_VectorUInt16__SWIG_0', PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16__SWIG_0`, 'returnType') = '_p_std__vectorT_uint16_t_t'
class(`VectorUInt16__SWIG_0`) = c("SWIGFunction", class('VectorUInt16__SWIG_0'))

# Start of new_VectorUInt16

`VectorUInt16__SWIG_1` = function(other)
{
  if (inherits(other, "ExternalReference")) other = slot(other,"ref"); 
  ;.Call('R_swig_new_VectorUInt16__SWIG_1', other, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16__SWIG_1`, 'returnType') = '_p_std__vectorT_uint16_t_t'
attr(`VectorUInt16__SWIG_1`, "inputTypes") = c('_p_std__vectorT_uint16_t_t')
class(`VectorUInt16__SWIG_1`) = c("SWIGFunction", class('VectorUInt16__SWIG_1'))

# Start of VectorUInt16_empty

`VectorUInt16_empty` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUInt16_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16_empty`, 'returnType') = 'logical'
attr(`VectorUInt16_empty`, "inputTypes") = c('_p_std__vectorT_uint16_t_t')
class(`VectorUInt16_empty`) = c("SWIGFunction", class('VectorUInt16_empty'))

# Start of VectorUInt16_size

`VectorUInt16_size` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUInt16_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16_size`, 'returnType') = 'integer'
attr(`VectorUInt16_size`, "inputTypes") = c('_p_std__vectorT_uint16_t_t')
class(`VectorUInt16_size`) = c("SWIGFunction", class('VectorUInt16_size'))

# Start of VectorUInt16_swap

`VectorUInt16_swap` = function(self, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(v, "ExternalReference")) v = slot(v,"ref"); 
  ;.Call('R_swig_VectorUInt16_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16_swap`, 'returnType') = 'void'
attr(`VectorUInt16_swap`, "inputTypes") = c('_p_std__vectorT_uint16_t_t', '_p_std__vectorT_uint16_t_t')
class(`VectorUInt16_swap`) = c("SWIGFunction", class('VectorUInt16_swap'))

# Start of VectorUInt16_clear

`VectorUInt16_clear` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUInt16_clear', self, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16_clear`, 'returnType') = 'void'
attr(`VectorUInt16_clear`, "inputTypes") = c('_p_std__vectorT_uint16_t_t')
class(`VectorUInt16_clear`) = c("SWIGFunction", class('VectorUInt16_clear'))

# Start of VectorUInt16_get_allocator

`VectorUInt16_get_allocator` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorUInt16_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_unsigned_short_t", ref=ans);
  
  ans
  
}

attr(`VectorUInt16_get_allocator`, 'returnType') = '_p_std__allocatorT_unsigned_short_t'
attr(`VectorUInt16_get_allocator`, "inputTypes") = c('_p_std__vectorT_uint16_t_t')
class(`VectorUInt16_get_allocator`) = c("SWIGFunction", class('VectorUInt16_get_allocator'))

# Start of new_VectorUInt16

`VectorUInt16__SWIG_2` = function(size)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  ;.Call('R_swig_new_VectorUInt16__SWIG_2', size, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16__SWIG_2`, 'returnType') = '_p_std__vectorT_uint16_t_t'
attr(`VectorUInt16__SWIG_2`, "inputTypes") = c('integer')
class(`VectorUInt16__SWIG_2`) = c("SWIGFunction", class('VectorUInt16__SWIG_2'))

# Start of VectorUInt16_pop_back

`VectorUInt16_pop_back` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUInt16_pop_back', self, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16_pop_back`, 'returnType') = 'void'
attr(`VectorUInt16_pop_back`, "inputTypes") = c('_p_std__vectorT_uint16_t_t')
class(`VectorUInt16_pop_back`) = c("SWIGFunction", class('VectorUInt16_pop_back'))

# Start of VectorUInt16_resize

`VectorUInt16_resize__SWIG_0` = function(self, new_size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  ;.Call('R_swig_VectorUInt16_resize__SWIG_0', self, new_size, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16_resize__SWIG_0`, 'returnType') = 'void'
attr(`VectorUInt16_resize__SWIG_0`, "inputTypes") = c('_p_std__vectorT_uint16_t_t', 'integer')
class(`VectorUInt16_resize__SWIG_0`) = c("SWIGFunction", class('VectorUInt16_resize__SWIG_0'))

# Start of new_VectorUInt16

`VectorUInt16__SWIG_3` = function(size, value)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  if (inherits(value, "ExternalReference")) value = slot(value,"ref"); 
  ;.Call('R_swig_new_VectorUInt16__SWIG_3', size, value, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16__SWIG_3`, 'returnType') = '_p_std__vectorT_uint16_t_t'
attr(`VectorUInt16__SWIG_3`, "inputTypes") = c('integer', '_p_unsigned_short')
class(`VectorUInt16__SWIG_3`) = c("SWIGFunction", class('VectorUInt16__SWIG_3'))

`VectorUInt16` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 0) {
    f <- VectorUInt16__SWIG_0; 
  } else if (argc == 1) {
    if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- VectorUInt16__SWIG_2; 
    }
    else if ( extends(argtypes[1], '_p_std__vectorT_uint16_t_t') && length(argv[[1]]) == 1 ) {
      f <- VectorUInt16__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_unsigned_short') && length(argv[[2]]) == 1 )) {
      f <- VectorUInt16__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorUInt16 with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorUInt16_push_back

`VectorUInt16_push_back` = function(self, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt16_push_back', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16_push_back`, 'returnType') = 'void'
attr(`VectorUInt16_push_back`, "inputTypes") = c('_p_std__vectorT_uint16_t_t', '_p_unsigned_short')
class(`VectorUInt16_push_back`) = c("SWIGFunction", class('VectorUInt16_push_back'))

# Start of VectorUInt16_front

`VectorUInt16_front` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUInt16_front', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16_front`, 'returnType') = '_p_unsigned_short'
attr(`VectorUInt16_front`, "inputTypes") = c('_p_std__vectorT_uint16_t_t')
class(`VectorUInt16_front`) = c("SWIGFunction", class('VectorUInt16_front'))

# Start of VectorUInt16_back

`VectorUInt16_back` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUInt16_back', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16_back`, 'returnType') = '_p_unsigned_short'
attr(`VectorUInt16_back`, "inputTypes") = c('_p_std__vectorT_uint16_t_t')
class(`VectorUInt16_back`) = c("SWIGFunction", class('VectorUInt16_back'))

# Start of VectorUInt16_assign

`VectorUInt16_assign` = function(self, n, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt16_assign', self, n, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16_assign`, 'returnType') = 'void'
attr(`VectorUInt16_assign`, "inputTypes") = c('_p_std__vectorT_uint16_t_t', 'integer', '_p_unsigned_short')
class(`VectorUInt16_assign`) = c("SWIGFunction", class('VectorUInt16_assign'))

# Start of VectorUInt16_resize

`VectorUInt16_resize__SWIG_1` = function(self, new_size, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt16_resize__SWIG_1', self, new_size, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16_resize__SWIG_1`, 'returnType') = 'void'
attr(`VectorUInt16_resize__SWIG_1`, "inputTypes") = c('_p_std__vectorT_uint16_t_t', 'integer', '_p_unsigned_short')
class(`VectorUInt16_resize__SWIG_1`) = c("SWIGFunction", class('VectorUInt16_resize__SWIG_1'))

`VectorUInt16_resize` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_std__vectorT_uint16_t_t') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorUInt16_resize__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_std__vectorT_uint16_t_t') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_unsigned_short') && length(argv[[3]]) == 1 )) {
      f <- VectorUInt16_resize__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorUInt16_resize with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorUInt16_reserve

`VectorUInt16_reserve` = function(self, n)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;.Call('R_swig_VectorUInt16_reserve', self, n, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16_reserve`, 'returnType') = 'void'
attr(`VectorUInt16_reserve`, "inputTypes") = c('_p_std__vectorT_uint16_t_t', 'integer')
class(`VectorUInt16_reserve`) = c("SWIGFunction", class('VectorUInt16_reserve'))

# Start of VectorUInt16_capacity

`VectorUInt16_capacity` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUInt16_capacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt16_capacity`, 'returnType') = 'integer'
attr(`VectorUInt16_capacity`, "inputTypes") = c('_p_std__vectorT_uint16_t_t')
class(`VectorUInt16_capacity`) = c("SWIGFunction", class('VectorUInt16_capacity'))

# Start of delete_VectorUInt16

`delete_VectorUInt16` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_delete_VectorUInt16', self, PACKAGE='SimpleITK');
  
}

attr(`delete_VectorUInt16`, 'returnType') = 'void'
attr(`delete_VectorUInt16`, "inputTypes") = c('_p_std__vectorT_uint16_t_t')
class(`delete_VectorUInt16`) = c("SWIGFunction", class('delete_VectorUInt16'))

# Start of accessor method for std::vector< uint16_t >
setMethod('$', '_p_std__vectorT_uint16_t_t', function(x, name)

{
  accessorFuns = list('__nonzero__' = VectorUInt16___nonzero__, '__len__' = VectorUInt16___len__, 'pop' = VectorUInt16_pop, '__getslice__' = VectorUInt16___getslice__, '__setslice__' = VectorUInt16___setslice__, '__delslice__' = VectorUInt16___delslice__, '__delitem__' = VectorUInt16___delitem__, '__getitem__' = VectorUInt16___getitem__, '__setitem__' = VectorUInt16___setitem__, 'append' = VectorUInt16_append, 'empty' = VectorUInt16_empty, 'size' = VectorUInt16_size, 'swap' = VectorUInt16_swap, 'clear' = VectorUInt16_clear, 'get_allocator' = VectorUInt16_get_allocator, 'pop_back' = VectorUInt16_pop_back, 'resize' = VectorUInt16_resize, 'push_back' = VectorUInt16_push_back, 'front' = VectorUInt16_front, 'back' = VectorUInt16_back, 'assign' = VectorUInt16_assign, 'reserve' = VectorUInt16_reserve, 'capacity' = VectorUInt16_capacity);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::vector< uint16_t >
# Start of VectorInt16___nonzero__

`VectorInt16___nonzero__` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt16___nonzero__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt16___nonzero__`, 'returnType') = 'logical'
attr(`VectorInt16___nonzero__`, "inputTypes") = c('integer')
class(`VectorInt16___nonzero__`) = c("SWIGFunction", class('VectorInt16___nonzero__'))

# Start of VectorInt16___len__

`VectorInt16___len__` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt16___len__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt16___len__`, 'returnType') = 'integer'
attr(`VectorInt16___len__`, "inputTypes") = c('integer')
class(`VectorInt16___len__`) = c("SWIGFunction", class('VectorInt16___len__'))

# Start of VectorInt16_pop

`VectorInt16_pop` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt16_pop', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt16_pop`, 'returnType') = 'integer'
attr(`VectorInt16_pop`, "inputTypes") = c('integer')
class(`VectorInt16_pop`) = c("SWIGFunction", class('VectorInt16_pop'))

# Start of VectorInt16___getslice__

`VectorInt16___getslice__` = function(self, i, j)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorInt16___getslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16___getslice__`, 'returnType') = 'integer'
attr(`VectorInt16___getslice__`, "inputTypes") = c('integer', 'integer', 'integer')
class(`VectorInt16___getslice__`) = c("SWIGFunction", class('VectorInt16___getslice__'))

# Start of VectorInt16___setslice__

`VectorInt16___setslice__` = function(self, i, j, is)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  is = as.integer(is);
  ;.Call('R_swig_VectorInt16___setslice__', self, i, j, is, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16___setslice__`, 'returnType') = 'void'
attr(`VectorInt16___setslice__`, "inputTypes") = c('integer', 'integer', 'integer', 'integer')
class(`VectorInt16___setslice__`) = c("SWIGFunction", class('VectorInt16___setslice__'))

# Start of VectorInt16___delslice__

`VectorInt16___delslice__` = function(self, i, j)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorInt16___delslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16___delslice__`, 'returnType') = 'void'
attr(`VectorInt16___delslice__`, "inputTypes") = c('integer', 'integer', 'integer')
class(`VectorInt16___delslice__`) = c("SWIGFunction", class('VectorInt16___delslice__'))

# Start of VectorInt16___delitem__

`VectorInt16___delitem__` = function(self, i)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorInt16___delitem__', self, i, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16___delitem__`, 'returnType') = 'void'
attr(`VectorInt16___delitem__`, "inputTypes") = c('integer', 'integer')
class(`VectorInt16___delitem__`) = c("SWIGFunction", class('VectorInt16___delitem__'))

# Start of VectorInt16___getitem__

`VectorInt16___getitem__` = function(self, i, .copy = FALSE)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorInt16___getitem__', self, i, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt16___getitem__`, 'returnType') = '_p_short'
attr(`VectorInt16___getitem__`, "inputTypes") = c('integer', 'integer')
class(`VectorInt16___getitem__`) = c("SWIGFunction", class('VectorInt16___getitem__'))

# Start of VectorInt16___setitem__

`VectorInt16___setitem__` = function(self, i, x)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorInt16___setitem__', self, i, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16___setitem__`, 'returnType') = 'void'
attr(`VectorInt16___setitem__`, "inputTypes") = c('integer', 'integer', '_p_short')
class(`VectorInt16___setitem__`) = c("SWIGFunction", class('VectorInt16___setitem__'))

# Start of VectorInt16_append

`VectorInt16_append` = function(self, x)
{
  self = as.integer(self);
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorInt16_append', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16_append`, 'returnType') = 'void'
attr(`VectorInt16_append`, "inputTypes") = c('integer', '_p_short')
class(`VectorInt16_append`) = c("SWIGFunction", class('VectorInt16_append'))

# Start of new_VectorInt16

`VectorInt16__SWIG_0` = function()
{
  ;.Call('R_swig_new_VectorInt16__SWIG_0', PACKAGE='SimpleITK');
  
}

attr(`VectorInt16__SWIG_0`, 'returnType') = 'integer'
class(`VectorInt16__SWIG_0`) = c("SWIGFunction", class('VectorInt16__SWIG_0'))

# Start of new_VectorInt16

`VectorInt16__SWIG_1` = function(other)
{
  other = as.integer(other);
  ;.Call('R_swig_new_VectorInt16__SWIG_1', other, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16__SWIG_1`, 'returnType') = 'integer'
attr(`VectorInt16__SWIG_1`, "inputTypes") = c('integer')
class(`VectorInt16__SWIG_1`) = c("SWIGFunction", class('VectorInt16__SWIG_1'))

# Start of VectorInt16_empty

`VectorInt16_empty` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt16_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt16_empty`, 'returnType') = 'logical'
attr(`VectorInt16_empty`, "inputTypes") = c('integer')
class(`VectorInt16_empty`) = c("SWIGFunction", class('VectorInt16_empty'))

# Start of VectorInt16_size

`VectorInt16_size` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt16_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt16_size`, 'returnType') = 'integer'
attr(`VectorInt16_size`, "inputTypes") = c('integer')
class(`VectorInt16_size`) = c("SWIGFunction", class('VectorInt16_size'))

# Start of VectorInt16_swap

`VectorInt16_swap` = function(self, v)
{
  self = as.integer(self);
  v = as.integer(v);
  ;.Call('R_swig_VectorInt16_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16_swap`, 'returnType') = 'void'
attr(`VectorInt16_swap`, "inputTypes") = c('integer', 'integer')
class(`VectorInt16_swap`) = c("SWIGFunction", class('VectorInt16_swap'))

# Start of VectorInt16_clear

`VectorInt16_clear` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt16_clear', self, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16_clear`, 'returnType') = 'void'
attr(`VectorInt16_clear`, "inputTypes") = c('integer')
class(`VectorInt16_clear`) = c("SWIGFunction", class('VectorInt16_clear'))

# Start of VectorInt16_get_allocator

`VectorInt16_get_allocator` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;ans = .Call('R_swig_VectorInt16_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_short_t", ref=ans);
  
  ans
  
}

attr(`VectorInt16_get_allocator`, 'returnType') = '_p_std__allocatorT_short_t'
attr(`VectorInt16_get_allocator`, "inputTypes") = c('integer')
class(`VectorInt16_get_allocator`) = c("SWIGFunction", class('VectorInt16_get_allocator'))

# Start of new_VectorInt16

`VectorInt16__SWIG_2` = function(size)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  ;.Call('R_swig_new_VectorInt16__SWIG_2', size, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16__SWIG_2`, 'returnType') = 'integer'
attr(`VectorInt16__SWIG_2`, "inputTypes") = c('integer')
class(`VectorInt16__SWIG_2`) = c("SWIGFunction", class('VectorInt16__SWIG_2'))

# Start of VectorInt16_pop_back

`VectorInt16_pop_back` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt16_pop_back', self, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16_pop_back`, 'returnType') = 'void'
attr(`VectorInt16_pop_back`, "inputTypes") = c('integer')
class(`VectorInt16_pop_back`) = c("SWIGFunction", class('VectorInt16_pop_back'))

# Start of VectorInt16_resize

`VectorInt16_resize__SWIG_0` = function(self, new_size)
{
  self = as.integer(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  ;.Call('R_swig_VectorInt16_resize__SWIG_0', self, new_size, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16_resize__SWIG_0`, 'returnType') = 'void'
attr(`VectorInt16_resize__SWIG_0`, "inputTypes") = c('integer', 'integer')
class(`VectorInt16_resize__SWIG_0`) = c("SWIGFunction", class('VectorInt16_resize__SWIG_0'))

# Start of new_VectorInt16

`VectorInt16__SWIG_3` = function(size, value)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  if (inherits(value, "ExternalReference")) value = slot(value,"ref"); 
  ;.Call('R_swig_new_VectorInt16__SWIG_3', size, value, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16__SWIG_3`, 'returnType') = 'integer'
attr(`VectorInt16__SWIG_3`, "inputTypes") = c('integer', '_p_short')
class(`VectorInt16__SWIG_3`) = c("SWIGFunction", class('VectorInt16__SWIG_3'))

`VectorInt16` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 0) {
    f <- VectorInt16__SWIG_0; 
  } else if (argc == 1) {
    if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- VectorInt16__SWIG_2; 
    }
    else if ( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) {
      f <- VectorInt16__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_short') && length(argv[[2]]) == 1 )) {
      f <- VectorInt16__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorInt16 with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorInt16_push_back

`VectorInt16_push_back` = function(self, x)
{
  self = as.integer(self);
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorInt16_push_back', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16_push_back`, 'returnType') = 'void'
attr(`VectorInt16_push_back`, "inputTypes") = c('integer', '_p_short')
class(`VectorInt16_push_back`) = c("SWIGFunction", class('VectorInt16_push_back'))

# Start of VectorInt16_front

`VectorInt16_front` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt16_front', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt16_front`, 'returnType') = '_p_short'
attr(`VectorInt16_front`, "inputTypes") = c('integer')
class(`VectorInt16_front`) = c("SWIGFunction", class('VectorInt16_front'))

# Start of VectorInt16_back

`VectorInt16_back` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt16_back', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt16_back`, 'returnType') = '_p_short'
attr(`VectorInt16_back`, "inputTypes") = c('integer')
class(`VectorInt16_back`) = c("SWIGFunction", class('VectorInt16_back'))

# Start of VectorInt16_assign

`VectorInt16_assign` = function(self, n, x)
{
  self = as.integer(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorInt16_assign', self, n, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16_assign`, 'returnType') = 'void'
attr(`VectorInt16_assign`, "inputTypes") = c('integer', 'integer', '_p_short')
class(`VectorInt16_assign`) = c("SWIGFunction", class('VectorInt16_assign'))

# Start of VectorInt16_resize

`VectorInt16_resize__SWIG_1` = function(self, new_size, x)
{
  self = as.integer(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorInt16_resize__SWIG_1', self, new_size, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16_resize__SWIG_1`, 'returnType') = 'void'
attr(`VectorInt16_resize__SWIG_1`, "inputTypes") = c('integer', 'integer', '_p_short')
class(`VectorInt16_resize__SWIG_1`) = c("SWIGFunction", class('VectorInt16_resize__SWIG_1'))

`VectorInt16_resize` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorInt16_resize__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_short') && length(argv[[3]]) == 1 )) {
      f <- VectorInt16_resize__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorInt16_resize with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorInt16_reserve

`VectorInt16_reserve` = function(self, n)
{
  self = as.integer(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;.Call('R_swig_VectorInt16_reserve', self, n, PACKAGE='SimpleITK');
  
}

attr(`VectorInt16_reserve`, 'returnType') = 'void'
attr(`VectorInt16_reserve`, "inputTypes") = c('integer', 'integer')
class(`VectorInt16_reserve`) = c("SWIGFunction", class('VectorInt16_reserve'))

# Start of VectorInt16_capacity

`VectorInt16_capacity` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt16_capacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt16_capacity`, 'returnType') = 'integer'
attr(`VectorInt16_capacity`, "inputTypes") = c('integer')
class(`VectorInt16_capacity`) = c("SWIGFunction", class('VectorInt16_capacity'))

# Start of delete_VectorInt16

`delete_VectorInt16` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_delete_VectorInt16', self, PACKAGE='SimpleITK');
  
}

attr(`delete_VectorInt16`, 'returnType') = 'void'
attr(`delete_VectorInt16`, "inputTypes") = c('integer')
class(`delete_VectorInt16`) = c("SWIGFunction", class('delete_VectorInt16'))

# Start of accessor method for std::vector< int16_t >
setMethod('$', '_p_std__vectorT_int16_t_t', function(x, name)

{
  accessorFuns = list('__nonzero__' = VectorInt16___nonzero__, '__len__' = VectorInt16___len__, 'pop' = VectorInt16_pop, '__getslice__' = VectorInt16___getslice__, '__setslice__' = VectorInt16___setslice__, '__delslice__' = VectorInt16___delslice__, '__delitem__' = VectorInt16___delitem__, '__getitem__' = VectorInt16___getitem__, '__setitem__' = VectorInt16___setitem__, 'append' = VectorInt16_append, 'empty' = VectorInt16_empty, 'size' = VectorInt16_size, 'swap' = VectorInt16_swap, 'clear' = VectorInt16_clear, 'get_allocator' = VectorInt16_get_allocator, 'pop_back' = VectorInt16_pop_back, 'resize' = VectorInt16_resize, 'push_back' = VectorInt16_push_back, 'front' = VectorInt16_front, 'back' = VectorInt16_back, 'assign' = VectorInt16_assign, 'reserve' = VectorInt16_reserve, 'capacity' = VectorInt16_capacity);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::vector< int16_t >
# Start of VectorUInt32___nonzero__

`VectorUInt32___nonzero__` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt32___nonzero__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32___nonzero__`, 'returnType') = 'logical'
attr(`VectorUInt32___nonzero__`, "inputTypes") = c('integer')
class(`VectorUInt32___nonzero__`) = c("SWIGFunction", class('VectorUInt32___nonzero__'))

# Start of VectorUInt32___len__

`VectorUInt32___len__` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt32___len__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32___len__`, 'returnType') = 'integer'
attr(`VectorUInt32___len__`, "inputTypes") = c('integer')
class(`VectorUInt32___len__`) = c("SWIGFunction", class('VectorUInt32___len__'))

# Start of VectorUInt32_pop

`VectorUInt32_pop` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt32_pop', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32_pop`, 'returnType') = 'integer'
attr(`VectorUInt32_pop`, "inputTypes") = c('integer')
class(`VectorUInt32_pop`) = c("SWIGFunction", class('VectorUInt32_pop'))

# Start of VectorUInt32___getslice__

`VectorUInt32___getslice__` = function(self, i, j)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorUInt32___getslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32___getslice__`, 'returnType') = 'integer'
attr(`VectorUInt32___getslice__`, "inputTypes") = c('integer', 'integer', 'integer')
class(`VectorUInt32___getslice__`) = c("SWIGFunction", class('VectorUInt32___getslice__'))

# Start of VectorUInt32___setslice__

`VectorUInt32___setslice__` = function(self, i, j, is)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  is = as.integer(is);
  ;.Call('R_swig_VectorUInt32___setslice__', self, i, j, is, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32___setslice__`, 'returnType') = 'void'
attr(`VectorUInt32___setslice__`, "inputTypes") = c('integer', 'integer', 'integer', 'integer')
class(`VectorUInt32___setslice__`) = c("SWIGFunction", class('VectorUInt32___setslice__'))

# Start of VectorUInt32___delslice__

`VectorUInt32___delslice__` = function(self, i, j)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorUInt32___delslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32___delslice__`, 'returnType') = 'void'
attr(`VectorUInt32___delslice__`, "inputTypes") = c('integer', 'integer', 'integer')
class(`VectorUInt32___delslice__`) = c("SWIGFunction", class('VectorUInt32___delslice__'))

# Start of VectorUInt32___delitem__

`VectorUInt32___delitem__` = function(self, i)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorUInt32___delitem__', self, i, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32___delitem__`, 'returnType') = 'void'
attr(`VectorUInt32___delitem__`, "inputTypes") = c('integer', 'integer')
class(`VectorUInt32___delitem__`) = c("SWIGFunction", class('VectorUInt32___delitem__'))

# Start of VectorUInt32___getitem__

`VectorUInt32___getitem__` = function(self, i, .copy = FALSE)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorUInt32___getitem__', self, i, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32___getitem__`, 'returnType') = '_p_unsigned_int'
attr(`VectorUInt32___getitem__`, "inputTypes") = c('integer', 'integer')
class(`VectorUInt32___getitem__`) = c("SWIGFunction", class('VectorUInt32___getitem__'))

# Start of VectorUInt32___setitem__

`VectorUInt32___setitem__` = function(self, i, x)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt32___setitem__', self, i, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32___setitem__`, 'returnType') = 'void'
attr(`VectorUInt32___setitem__`, "inputTypes") = c('integer', 'integer', '_p_unsigned_int')
class(`VectorUInt32___setitem__`) = c("SWIGFunction", class('VectorUInt32___setitem__'))

# Start of VectorUInt32_append

`VectorUInt32_append` = function(self, x)
{
  self = as.integer(self);
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt32_append', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32_append`, 'returnType') = 'void'
attr(`VectorUInt32_append`, "inputTypes") = c('integer', '_p_unsigned_int')
class(`VectorUInt32_append`) = c("SWIGFunction", class('VectorUInt32_append'))

# Start of new_VectorUInt32

`VectorUInt32__SWIG_0` = function()
{
  ;.Call('R_swig_new_VectorUInt32__SWIG_0', PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32__SWIG_0`, 'returnType') = 'integer'
class(`VectorUInt32__SWIG_0`) = c("SWIGFunction", class('VectorUInt32__SWIG_0'))

# Start of new_VectorUInt32

`VectorUInt32__SWIG_1` = function(other)
{
  other = as.integer(other);
  ;.Call('R_swig_new_VectorUInt32__SWIG_1', other, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32__SWIG_1`, 'returnType') = 'integer'
attr(`VectorUInt32__SWIG_1`, "inputTypes") = c('integer')
class(`VectorUInt32__SWIG_1`) = c("SWIGFunction", class('VectorUInt32__SWIG_1'))

# Start of VectorUInt32_empty

`VectorUInt32_empty` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt32_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32_empty`, 'returnType') = 'logical'
attr(`VectorUInt32_empty`, "inputTypes") = c('integer')
class(`VectorUInt32_empty`) = c("SWIGFunction", class('VectorUInt32_empty'))

# Start of VectorUInt32_size

`VectorUInt32_size` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt32_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32_size`, 'returnType') = 'integer'
attr(`VectorUInt32_size`, "inputTypes") = c('integer')
class(`VectorUInt32_size`) = c("SWIGFunction", class('VectorUInt32_size'))

# Start of VectorUInt32_swap

`VectorUInt32_swap` = function(self, v)
{
  self = as.integer(self);
  v = as.integer(v);
  ;.Call('R_swig_VectorUInt32_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32_swap`, 'returnType') = 'void'
attr(`VectorUInt32_swap`, "inputTypes") = c('integer', 'integer')
class(`VectorUInt32_swap`) = c("SWIGFunction", class('VectorUInt32_swap'))

# Start of VectorUInt32_clear

`VectorUInt32_clear` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt32_clear', self, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32_clear`, 'returnType') = 'void'
attr(`VectorUInt32_clear`, "inputTypes") = c('integer')
class(`VectorUInt32_clear`) = c("SWIGFunction", class('VectorUInt32_clear'))

# Start of VectorUInt32_get_allocator

`VectorUInt32_get_allocator` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;ans = .Call('R_swig_VectorUInt32_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_unsigned_int_t", ref=ans);
  
  ans
  
}

attr(`VectorUInt32_get_allocator`, 'returnType') = '_p_std__allocatorT_unsigned_int_t'
attr(`VectorUInt32_get_allocator`, "inputTypes") = c('integer')
class(`VectorUInt32_get_allocator`) = c("SWIGFunction", class('VectorUInt32_get_allocator'))

# Start of new_VectorUInt32

`VectorUInt32__SWIG_2` = function(size)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  ;.Call('R_swig_new_VectorUInt32__SWIG_2', size, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32__SWIG_2`, 'returnType') = 'integer'
attr(`VectorUInt32__SWIG_2`, "inputTypes") = c('integer')
class(`VectorUInt32__SWIG_2`) = c("SWIGFunction", class('VectorUInt32__SWIG_2'))

# Start of VectorUInt32_pop_back

`VectorUInt32_pop_back` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt32_pop_back', self, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32_pop_back`, 'returnType') = 'void'
attr(`VectorUInt32_pop_back`, "inputTypes") = c('integer')
class(`VectorUInt32_pop_back`) = c("SWIGFunction", class('VectorUInt32_pop_back'))

# Start of VectorUInt32_resize

`VectorUInt32_resize__SWIG_0` = function(self, new_size)
{
  self = as.integer(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  ;.Call('R_swig_VectorUInt32_resize__SWIG_0', self, new_size, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32_resize__SWIG_0`, 'returnType') = 'void'
attr(`VectorUInt32_resize__SWIG_0`, "inputTypes") = c('integer', 'integer')
class(`VectorUInt32_resize__SWIG_0`) = c("SWIGFunction", class('VectorUInt32_resize__SWIG_0'))

# Start of new_VectorUInt32

`VectorUInt32__SWIG_3` = function(size, value)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  if (inherits(value, "ExternalReference")) value = slot(value,"ref"); 
  ;.Call('R_swig_new_VectorUInt32__SWIG_3', size, value, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32__SWIG_3`, 'returnType') = 'integer'
attr(`VectorUInt32__SWIG_3`, "inputTypes") = c('integer', '_p_unsigned_int')
class(`VectorUInt32__SWIG_3`) = c("SWIGFunction", class('VectorUInt32__SWIG_3'))

`VectorUInt32` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 0) {
    f <- VectorUInt32__SWIG_0; 
  } else if (argc == 1) {
    if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- VectorUInt32__SWIG_2; 
    }
    else if ( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) {
      f <- VectorUInt32__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_unsigned_int') && length(argv[[2]]) == 1 )) {
      f <- VectorUInt32__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorUInt32 with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorUInt32_push_back

`VectorUInt32_push_back` = function(self, x)
{
  self = as.integer(self);
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt32_push_back', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32_push_back`, 'returnType') = 'void'
attr(`VectorUInt32_push_back`, "inputTypes") = c('integer', '_p_unsigned_int')
class(`VectorUInt32_push_back`) = c("SWIGFunction", class('VectorUInt32_push_back'))

# Start of VectorUInt32_front

`VectorUInt32_front` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt32_front', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32_front`, 'returnType') = '_p_unsigned_int'
attr(`VectorUInt32_front`, "inputTypes") = c('integer')
class(`VectorUInt32_front`) = c("SWIGFunction", class('VectorUInt32_front'))

# Start of VectorUInt32_back

`VectorUInt32_back` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt32_back', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32_back`, 'returnType') = '_p_unsigned_int'
attr(`VectorUInt32_back`, "inputTypes") = c('integer')
class(`VectorUInt32_back`) = c("SWIGFunction", class('VectorUInt32_back'))

# Start of VectorUInt32_assign

`VectorUInt32_assign` = function(self, n, x)
{
  self = as.integer(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt32_assign', self, n, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32_assign`, 'returnType') = 'void'
attr(`VectorUInt32_assign`, "inputTypes") = c('integer', 'integer', '_p_unsigned_int')
class(`VectorUInt32_assign`) = c("SWIGFunction", class('VectorUInt32_assign'))

# Start of VectorUInt32_resize

`VectorUInt32_resize__SWIG_1` = function(self, new_size, x)
{
  self = as.integer(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt32_resize__SWIG_1', self, new_size, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32_resize__SWIG_1`, 'returnType') = 'void'
attr(`VectorUInt32_resize__SWIG_1`, "inputTypes") = c('integer', 'integer', '_p_unsigned_int')
class(`VectorUInt32_resize__SWIG_1`) = c("SWIGFunction", class('VectorUInt32_resize__SWIG_1'))

`VectorUInt32_resize` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorUInt32_resize__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_unsigned_int') && length(argv[[3]]) == 1 )) {
      f <- VectorUInt32_resize__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorUInt32_resize with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorUInt32_reserve

`VectorUInt32_reserve` = function(self, n)
{
  self = as.integer(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;.Call('R_swig_VectorUInt32_reserve', self, n, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32_reserve`, 'returnType') = 'void'
attr(`VectorUInt32_reserve`, "inputTypes") = c('integer', 'integer')
class(`VectorUInt32_reserve`) = c("SWIGFunction", class('VectorUInt32_reserve'))

# Start of VectorUInt32_capacity

`VectorUInt32_capacity` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt32_capacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt32_capacity`, 'returnType') = 'integer'
attr(`VectorUInt32_capacity`, "inputTypes") = c('integer')
class(`VectorUInt32_capacity`) = c("SWIGFunction", class('VectorUInt32_capacity'))

# Start of delete_VectorUInt32

`delete_VectorUInt32` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_delete_VectorUInt32', self, PACKAGE='SimpleITK');
  
}

attr(`delete_VectorUInt32`, 'returnType') = 'void'
attr(`delete_VectorUInt32`, "inputTypes") = c('integer')
class(`delete_VectorUInt32`) = c("SWIGFunction", class('delete_VectorUInt32'))

# Start of accessor method for std::vector< uint32_t >
setMethod('$', '_p_std__vectorT_uint32_t_t', function(x, name)

{
  accessorFuns = list('__nonzero__' = VectorUInt32___nonzero__, '__len__' = VectorUInt32___len__, 'pop' = VectorUInt32_pop, '__getslice__' = VectorUInt32___getslice__, '__setslice__' = VectorUInt32___setslice__, '__delslice__' = VectorUInt32___delslice__, '__delitem__' = VectorUInt32___delitem__, '__getitem__' = VectorUInt32___getitem__, '__setitem__' = VectorUInt32___setitem__, 'append' = VectorUInt32_append, 'empty' = VectorUInt32_empty, 'size' = VectorUInt32_size, 'swap' = VectorUInt32_swap, 'clear' = VectorUInt32_clear, 'get_allocator' = VectorUInt32_get_allocator, 'pop_back' = VectorUInt32_pop_back, 'resize' = VectorUInt32_resize, 'push_back' = VectorUInt32_push_back, 'front' = VectorUInt32_front, 'back' = VectorUInt32_back, 'assign' = VectorUInt32_assign, 'reserve' = VectorUInt32_reserve, 'capacity' = VectorUInt32_capacity);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::vector< uint32_t >
# Start of VectorInt32___nonzero__

`VectorInt32___nonzero__` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt32___nonzero__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt32___nonzero__`, 'returnType') = 'logical'
attr(`VectorInt32___nonzero__`, "inputTypes") = c('_p_std__vectorT_int32_t_t')
class(`VectorInt32___nonzero__`) = c("SWIGFunction", class('VectorInt32___nonzero__'))

# Start of VectorInt32___len__

`VectorInt32___len__` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt32___len__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt32___len__`, 'returnType') = 'integer'
attr(`VectorInt32___len__`, "inputTypes") = c('_p_std__vectorT_int32_t_t')
class(`VectorInt32___len__`) = c("SWIGFunction", class('VectorInt32___len__'))

# Start of VectorInt32_pop

`VectorInt32_pop` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt32_pop', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt32_pop`, 'returnType') = 'integer'
attr(`VectorInt32_pop`, "inputTypes") = c('_p_std__vectorT_int32_t_t')
class(`VectorInt32_pop`) = c("SWIGFunction", class('VectorInt32_pop'))

# Start of VectorInt32___getslice__

`VectorInt32___getslice__` = function(self, i, j)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorInt32___getslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32___getslice__`, 'returnType') = '_p_std__vectorT_int32_t_t'
attr(`VectorInt32___getslice__`, "inputTypes") = c('_p_std__vectorT_int32_t_t', 'integer', 'integer')
class(`VectorInt32___getslice__`) = c("SWIGFunction", class('VectorInt32___getslice__'))

# Start of VectorInt32___setslice__

`VectorInt32___setslice__` = function(self, i, j, is)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  is = as.integer(is);
  ;.Call('R_swig_VectorInt32___setslice__', self, i, j, is, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32___setslice__`, 'returnType') = 'void'
attr(`VectorInt32___setslice__`, "inputTypes") = c('_p_std__vectorT_int32_t_t', 'integer', 'integer', '_p_std__vectorT_int32_t_t')
class(`VectorInt32___setslice__`) = c("SWIGFunction", class('VectorInt32___setslice__'))

# Start of VectorInt32___delslice__

`VectorInt32___delslice__` = function(self, i, j)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorInt32___delslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32___delslice__`, 'returnType') = 'void'
attr(`VectorInt32___delslice__`, "inputTypes") = c('_p_std__vectorT_int32_t_t', 'integer', 'integer')
class(`VectorInt32___delslice__`) = c("SWIGFunction", class('VectorInt32___delslice__'))

# Start of VectorInt32___delitem__

`VectorInt32___delitem__` = function(self, i)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorInt32___delitem__', self, i, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32___delitem__`, 'returnType') = 'void'
attr(`VectorInt32___delitem__`, "inputTypes") = c('_p_std__vectorT_int32_t_t', 'integer')
class(`VectorInt32___delitem__`) = c("SWIGFunction", class('VectorInt32___delitem__'))

# Start of VectorInt32___getitem__

`VectorInt32___getitem__` = function(self, i, .copy = FALSE)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorInt32___getitem__', self, i, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt32___getitem__`, 'returnType') = 'integer'
attr(`VectorInt32___getitem__`, "inputTypes") = c('_p_std__vectorT_int32_t_t', 'integer')
class(`VectorInt32___getitem__`) = c("SWIGFunction", class('VectorInt32___getitem__'))

# Start of VectorInt32___setitem__

`VectorInt32___setitem__` = function(self, i, x)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  x = as.integer(x);
  ;.Call('R_swig_VectorInt32___setitem__', self, i, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32___setitem__`, 'returnType') = 'void'
attr(`VectorInt32___setitem__`, "inputTypes") = c('_p_std__vectorT_int32_t_t', 'integer', 'integer')
class(`VectorInt32___setitem__`) = c("SWIGFunction", class('VectorInt32___setitem__'))

# Start of VectorInt32_append

`VectorInt32_append` = function(self, x)
{
  self = as.integer(self);
  x = as.integer(x);
  ;.Call('R_swig_VectorInt32_append', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32_append`, 'returnType') = 'void'
attr(`VectorInt32_append`, "inputTypes") = c('_p_std__vectorT_int32_t_t', 'integer')
class(`VectorInt32_append`) = c("SWIGFunction", class('VectorInt32_append'))

# Start of new_VectorInt32

`VectorInt32__SWIG_0` = function()
{
  ;.Call('R_swig_new_VectorInt32__SWIG_0', PACKAGE='SimpleITK');
  
}

attr(`VectorInt32__SWIG_0`, 'returnType') = '_p_std__vectorT_int32_t_t'
class(`VectorInt32__SWIG_0`) = c("SWIGFunction", class('VectorInt32__SWIG_0'))

# Start of new_VectorInt32

`VectorInt32__SWIG_1` = function(other)
{
  other = as.integer(other);
  ;.Call('R_swig_new_VectorInt32__SWIG_1', other, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32__SWIG_1`, 'returnType') = '_p_std__vectorT_int32_t_t'
attr(`VectorInt32__SWIG_1`, "inputTypes") = c('_p_std__vectorT_int32_t_t')
class(`VectorInt32__SWIG_1`) = c("SWIGFunction", class('VectorInt32__SWIG_1'))

# Start of VectorInt32_empty

`VectorInt32_empty` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt32_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt32_empty`, 'returnType') = 'logical'
attr(`VectorInt32_empty`, "inputTypes") = c('_p_std__vectorT_int32_t_t')
class(`VectorInt32_empty`) = c("SWIGFunction", class('VectorInt32_empty'))

# Start of VectorInt32_size

`VectorInt32_size` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt32_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt32_size`, 'returnType') = 'integer'
attr(`VectorInt32_size`, "inputTypes") = c('_p_std__vectorT_int32_t_t')
class(`VectorInt32_size`) = c("SWIGFunction", class('VectorInt32_size'))

# Start of VectorInt32_swap

`VectorInt32_swap` = function(self, v)
{
  self = as.integer(self);
  v = as.integer(v);
  ;.Call('R_swig_VectorInt32_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32_swap`, 'returnType') = 'void'
attr(`VectorInt32_swap`, "inputTypes") = c('_p_std__vectorT_int32_t_t', '_p_std__vectorT_int32_t_t')
class(`VectorInt32_swap`) = c("SWIGFunction", class('VectorInt32_swap'))

# Start of VectorInt32_clear

`VectorInt32_clear` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt32_clear', self, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32_clear`, 'returnType') = 'void'
attr(`VectorInt32_clear`, "inputTypes") = c('_p_std__vectorT_int32_t_t')
class(`VectorInt32_clear`) = c("SWIGFunction", class('VectorInt32_clear'))

# Start of VectorInt32_get_allocator

`VectorInt32_get_allocator` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;ans = .Call('R_swig_VectorInt32_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_int_t", ref=ans);
  
  ans
  
}

attr(`VectorInt32_get_allocator`, 'returnType') = '_p_std__allocatorT_int_t'
attr(`VectorInt32_get_allocator`, "inputTypes") = c('_p_std__vectorT_int32_t_t')
class(`VectorInt32_get_allocator`) = c("SWIGFunction", class('VectorInt32_get_allocator'))

# Start of new_VectorInt32

`VectorInt32__SWIG_2` = function(size)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  ;.Call('R_swig_new_VectorInt32__SWIG_2', size, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32__SWIG_2`, 'returnType') = '_p_std__vectorT_int32_t_t'
attr(`VectorInt32__SWIG_2`, "inputTypes") = c('integer')
class(`VectorInt32__SWIG_2`) = c("SWIGFunction", class('VectorInt32__SWIG_2'))

# Start of VectorInt32_pop_back

`VectorInt32_pop_back` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt32_pop_back', self, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32_pop_back`, 'returnType') = 'void'
attr(`VectorInt32_pop_back`, "inputTypes") = c('_p_std__vectorT_int32_t_t')
class(`VectorInt32_pop_back`) = c("SWIGFunction", class('VectorInt32_pop_back'))

# Start of VectorInt32_resize

`VectorInt32_resize__SWIG_0` = function(self, new_size)
{
  self = as.integer(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  ;.Call('R_swig_VectorInt32_resize__SWIG_0', self, new_size, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32_resize__SWIG_0`, 'returnType') = 'void'
attr(`VectorInt32_resize__SWIG_0`, "inputTypes") = c('_p_std__vectorT_int32_t_t', 'integer')
class(`VectorInt32_resize__SWIG_0`) = c("SWIGFunction", class('VectorInt32_resize__SWIG_0'))

# Start of new_VectorInt32

`VectorInt32__SWIG_3` = function(size, value)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  value = as.integer(value);
  ;.Call('R_swig_new_VectorInt32__SWIG_3', size, value, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32__SWIG_3`, 'returnType') = '_p_std__vectorT_int32_t_t'
attr(`VectorInt32__SWIG_3`, "inputTypes") = c('integer', 'integer')
class(`VectorInt32__SWIG_3`) = c("SWIGFunction", class('VectorInt32__SWIG_3'))

`VectorInt32` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 0) {
    f <- VectorInt32__SWIG_0; 
  } else if (argc == 1) {
    if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- VectorInt32__SWIG_2; 
    }
    else if ( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) {
      f <- VectorInt32__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorInt32__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorInt32 with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorInt32_push_back

`VectorInt32_push_back` = function(self, x)
{
  self = as.integer(self);
  x = as.integer(x);
  ;.Call('R_swig_VectorInt32_push_back', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32_push_back`, 'returnType') = 'void'
attr(`VectorInt32_push_back`, "inputTypes") = c('_p_std__vectorT_int32_t_t', 'integer')
class(`VectorInt32_push_back`) = c("SWIGFunction", class('VectorInt32_push_back'))

# Start of VectorInt32_front

`VectorInt32_front` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt32_front', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt32_front`, 'returnType') = 'integer'
attr(`VectorInt32_front`, "inputTypes") = c('_p_std__vectorT_int32_t_t')
class(`VectorInt32_front`) = c("SWIGFunction", class('VectorInt32_front'))

# Start of VectorInt32_back

`VectorInt32_back` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt32_back', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt32_back`, 'returnType') = 'integer'
attr(`VectorInt32_back`, "inputTypes") = c('_p_std__vectorT_int32_t_t')
class(`VectorInt32_back`) = c("SWIGFunction", class('VectorInt32_back'))

# Start of VectorInt32_assign

`VectorInt32_assign` = function(self, n, x)
{
  self = as.integer(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  x = as.integer(x);
  ;.Call('R_swig_VectorInt32_assign', self, n, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32_assign`, 'returnType') = 'void'
attr(`VectorInt32_assign`, "inputTypes") = c('_p_std__vectorT_int32_t_t', 'integer', 'integer')
class(`VectorInt32_assign`) = c("SWIGFunction", class('VectorInt32_assign'))

# Start of VectorInt32_resize

`VectorInt32_resize__SWIG_1` = function(self, new_size, x)
{
  self = as.integer(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  x = as.integer(x);
  ;.Call('R_swig_VectorInt32_resize__SWIG_1', self, new_size, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32_resize__SWIG_1`, 'returnType') = 'void'
attr(`VectorInt32_resize__SWIG_1`, "inputTypes") = c('_p_std__vectorT_int32_t_t', 'integer', 'integer')
class(`VectorInt32_resize__SWIG_1`) = c("SWIGFunction", class('VectorInt32_resize__SWIG_1'))

`VectorInt32_resize` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorInt32_resize__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- VectorInt32_resize__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorInt32_resize with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorInt32_reserve

`VectorInt32_reserve` = function(self, n)
{
  self = as.integer(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;.Call('R_swig_VectorInt32_reserve', self, n, PACKAGE='SimpleITK');
  
}

attr(`VectorInt32_reserve`, 'returnType') = 'void'
attr(`VectorInt32_reserve`, "inputTypes") = c('_p_std__vectorT_int32_t_t', 'integer')
class(`VectorInt32_reserve`) = c("SWIGFunction", class('VectorInt32_reserve'))

# Start of VectorInt32_capacity

`VectorInt32_capacity` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt32_capacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt32_capacity`, 'returnType') = 'integer'
attr(`VectorInt32_capacity`, "inputTypes") = c('_p_std__vectorT_int32_t_t')
class(`VectorInt32_capacity`) = c("SWIGFunction", class('VectorInt32_capacity'))

# Start of delete_VectorInt32

`delete_VectorInt32` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_delete_VectorInt32', self, PACKAGE='SimpleITK');
  
}

attr(`delete_VectorInt32`, 'returnType') = 'void'
attr(`delete_VectorInt32`, "inputTypes") = c('_p_std__vectorT_int32_t_t')
class(`delete_VectorInt32`) = c("SWIGFunction", class('delete_VectorInt32'))

# Start of accessor method for std::vector< int32_t >
setMethod('$', '_p_std__vectorT_int32_t_t', function(x, name)

{
  accessorFuns = list('__nonzero__' = VectorInt32___nonzero__, '__len__' = VectorInt32___len__, 'pop' = VectorInt32_pop, '__getslice__' = VectorInt32___getslice__, '__setslice__' = VectorInt32___setslice__, '__delslice__' = VectorInt32___delslice__, '__delitem__' = VectorInt32___delitem__, '__getitem__' = VectorInt32___getitem__, '__setitem__' = VectorInt32___setitem__, 'append' = VectorInt32_append, 'empty' = VectorInt32_empty, 'size' = VectorInt32_size, 'swap' = VectorInt32_swap, 'clear' = VectorInt32_clear, 'get_allocator' = VectorInt32_get_allocator, 'pop_back' = VectorInt32_pop_back, 'resize' = VectorInt32_resize, 'push_back' = VectorInt32_push_back, 'front' = VectorInt32_front, 'back' = VectorInt32_back, 'assign' = VectorInt32_assign, 'reserve' = VectorInt32_reserve, 'capacity' = VectorInt32_capacity);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::vector< int32_t >
# Start of VectorUInt64___nonzero__

`VectorUInt64___nonzero__` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt64___nonzero__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64___nonzero__`, 'returnType') = 'logical'
attr(`VectorUInt64___nonzero__`, "inputTypes") = c('integer')
class(`VectorUInt64___nonzero__`) = c("SWIGFunction", class('VectorUInt64___nonzero__'))

# Start of VectorUInt64___len__

`VectorUInt64___len__` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt64___len__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64___len__`, 'returnType') = 'integer'
attr(`VectorUInt64___len__`, "inputTypes") = c('integer')
class(`VectorUInt64___len__`) = c("SWIGFunction", class('VectorUInt64___len__'))

# Start of VectorUInt64_pop

`VectorUInt64_pop` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt64_pop', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64_pop`, 'returnType') = 'integer'
attr(`VectorUInt64_pop`, "inputTypes") = c('integer')
class(`VectorUInt64_pop`) = c("SWIGFunction", class('VectorUInt64_pop'))

# Start of VectorUInt64___getslice__

`VectorUInt64___getslice__` = function(self, i, j)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorUInt64___getslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64___getslice__`, 'returnType') = 'integer'
attr(`VectorUInt64___getslice__`, "inputTypes") = c('integer', 'integer', 'integer')
class(`VectorUInt64___getslice__`) = c("SWIGFunction", class('VectorUInt64___getslice__'))

# Start of VectorUInt64___setslice__

`VectorUInt64___setslice__` = function(self, i, j, is)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  is = as.integer(is);
  ;.Call('R_swig_VectorUInt64___setslice__', self, i, j, is, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64___setslice__`, 'returnType') = 'void'
attr(`VectorUInt64___setslice__`, "inputTypes") = c('integer', 'integer', 'integer', 'integer')
class(`VectorUInt64___setslice__`) = c("SWIGFunction", class('VectorUInt64___setslice__'))

# Start of VectorUInt64___delslice__

`VectorUInt64___delslice__` = function(self, i, j)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorUInt64___delslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64___delslice__`, 'returnType') = 'void'
attr(`VectorUInt64___delslice__`, "inputTypes") = c('integer', 'integer', 'integer')
class(`VectorUInt64___delslice__`) = c("SWIGFunction", class('VectorUInt64___delslice__'))

# Start of VectorUInt64___delitem__

`VectorUInt64___delitem__` = function(self, i)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorUInt64___delitem__', self, i, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64___delitem__`, 'returnType') = 'void'
attr(`VectorUInt64___delitem__`, "inputTypes") = c('integer', 'integer')
class(`VectorUInt64___delitem__`) = c("SWIGFunction", class('VectorUInt64___delitem__'))

# Start of VectorUInt64___getitem__

`VectorUInt64___getitem__` = function(self, i, .copy = FALSE)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorUInt64___getitem__', self, i, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64___getitem__`, 'returnType') = '_p_unsigned_long'
attr(`VectorUInt64___getitem__`, "inputTypes") = c('integer', 'integer')
class(`VectorUInt64___getitem__`) = c("SWIGFunction", class('VectorUInt64___getitem__'))

# Start of VectorUInt64___setitem__

`VectorUInt64___setitem__` = function(self, i, x)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt64___setitem__', self, i, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64___setitem__`, 'returnType') = 'void'
attr(`VectorUInt64___setitem__`, "inputTypes") = c('integer', 'integer', '_p_unsigned_long')
class(`VectorUInt64___setitem__`) = c("SWIGFunction", class('VectorUInt64___setitem__'))

# Start of VectorUInt64_append

`VectorUInt64_append` = function(self, x)
{
  self = as.integer(self);
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt64_append', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64_append`, 'returnType') = 'void'
attr(`VectorUInt64_append`, "inputTypes") = c('integer', '_p_unsigned_long')
class(`VectorUInt64_append`) = c("SWIGFunction", class('VectorUInt64_append'))

# Start of new_VectorUInt64

`VectorUInt64__SWIG_0` = function()
{
  ;.Call('R_swig_new_VectorUInt64__SWIG_0', PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64__SWIG_0`, 'returnType') = 'integer'
class(`VectorUInt64__SWIG_0`) = c("SWIGFunction", class('VectorUInt64__SWIG_0'))

# Start of new_VectorUInt64

`VectorUInt64__SWIG_1` = function(other)
{
  other = as.integer(other);
  ;.Call('R_swig_new_VectorUInt64__SWIG_1', other, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64__SWIG_1`, 'returnType') = 'integer'
attr(`VectorUInt64__SWIG_1`, "inputTypes") = c('integer')
class(`VectorUInt64__SWIG_1`) = c("SWIGFunction", class('VectorUInt64__SWIG_1'))

# Start of VectorUInt64_empty

`VectorUInt64_empty` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt64_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64_empty`, 'returnType') = 'logical'
attr(`VectorUInt64_empty`, "inputTypes") = c('integer')
class(`VectorUInt64_empty`) = c("SWIGFunction", class('VectorUInt64_empty'))

# Start of VectorUInt64_size

`VectorUInt64_size` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt64_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64_size`, 'returnType') = 'integer'
attr(`VectorUInt64_size`, "inputTypes") = c('integer')
class(`VectorUInt64_size`) = c("SWIGFunction", class('VectorUInt64_size'))

# Start of VectorUInt64_swap

`VectorUInt64_swap` = function(self, v)
{
  self = as.integer(self);
  v = as.integer(v);
  ;.Call('R_swig_VectorUInt64_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64_swap`, 'returnType') = 'void'
attr(`VectorUInt64_swap`, "inputTypes") = c('integer', 'integer')
class(`VectorUInt64_swap`) = c("SWIGFunction", class('VectorUInt64_swap'))

# Start of VectorUInt64_clear

`VectorUInt64_clear` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt64_clear', self, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64_clear`, 'returnType') = 'void'
attr(`VectorUInt64_clear`, "inputTypes") = c('integer')
class(`VectorUInt64_clear`) = c("SWIGFunction", class('VectorUInt64_clear'))

# Start of VectorUInt64_get_allocator

`VectorUInt64_get_allocator` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;ans = .Call('R_swig_VectorUInt64_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_unsigned_long_t", ref=ans);
  
  ans
  
}

attr(`VectorUInt64_get_allocator`, 'returnType') = '_p_std__allocatorT_unsigned_long_t'
attr(`VectorUInt64_get_allocator`, "inputTypes") = c('integer')
class(`VectorUInt64_get_allocator`) = c("SWIGFunction", class('VectorUInt64_get_allocator'))

# Start of new_VectorUInt64

`VectorUInt64__SWIG_2` = function(size)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  ;.Call('R_swig_new_VectorUInt64__SWIG_2', size, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64__SWIG_2`, 'returnType') = 'integer'
attr(`VectorUInt64__SWIG_2`, "inputTypes") = c('integer')
class(`VectorUInt64__SWIG_2`) = c("SWIGFunction", class('VectorUInt64__SWIG_2'))

# Start of VectorUInt64_pop_back

`VectorUInt64_pop_back` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt64_pop_back', self, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64_pop_back`, 'returnType') = 'void'
attr(`VectorUInt64_pop_back`, "inputTypes") = c('integer')
class(`VectorUInt64_pop_back`) = c("SWIGFunction", class('VectorUInt64_pop_back'))

# Start of VectorUInt64_resize

`VectorUInt64_resize__SWIG_0` = function(self, new_size)
{
  self = as.integer(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  ;.Call('R_swig_VectorUInt64_resize__SWIG_0', self, new_size, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64_resize__SWIG_0`, 'returnType') = 'void'
attr(`VectorUInt64_resize__SWIG_0`, "inputTypes") = c('integer', 'integer')
class(`VectorUInt64_resize__SWIG_0`) = c("SWIGFunction", class('VectorUInt64_resize__SWIG_0'))

# Start of new_VectorUInt64

`VectorUInt64__SWIG_3` = function(size, value)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  if (inherits(value, "ExternalReference")) value = slot(value,"ref"); 
  ;.Call('R_swig_new_VectorUInt64__SWIG_3', size, value, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64__SWIG_3`, 'returnType') = 'integer'
attr(`VectorUInt64__SWIG_3`, "inputTypes") = c('integer', '_p_unsigned_long')
class(`VectorUInt64__SWIG_3`) = c("SWIGFunction", class('VectorUInt64__SWIG_3'))

`VectorUInt64` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 0) {
    f <- VectorUInt64__SWIG_0; 
  } else if (argc == 1) {
    if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- VectorUInt64__SWIG_2; 
    }
    else if ( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) {
      f <- VectorUInt64__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_unsigned_long') && length(argv[[2]]) == 1 )) {
      f <- VectorUInt64__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorUInt64 with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorUInt64_push_back

`VectorUInt64_push_back` = function(self, x)
{
  self = as.integer(self);
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt64_push_back', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64_push_back`, 'returnType') = 'void'
attr(`VectorUInt64_push_back`, "inputTypes") = c('integer', '_p_unsigned_long')
class(`VectorUInt64_push_back`) = c("SWIGFunction", class('VectorUInt64_push_back'))

# Start of VectorUInt64_front

`VectorUInt64_front` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt64_front', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64_front`, 'returnType') = '_p_unsigned_long'
attr(`VectorUInt64_front`, "inputTypes") = c('integer')
class(`VectorUInt64_front`) = c("SWIGFunction", class('VectorUInt64_front'))

# Start of VectorUInt64_back

`VectorUInt64_back` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt64_back', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64_back`, 'returnType') = '_p_unsigned_long'
attr(`VectorUInt64_back`, "inputTypes") = c('integer')
class(`VectorUInt64_back`) = c("SWIGFunction", class('VectorUInt64_back'))

# Start of VectorUInt64_assign

`VectorUInt64_assign` = function(self, n, x)
{
  self = as.integer(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt64_assign', self, n, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64_assign`, 'returnType') = 'void'
attr(`VectorUInt64_assign`, "inputTypes") = c('integer', 'integer', '_p_unsigned_long')
class(`VectorUInt64_assign`) = c("SWIGFunction", class('VectorUInt64_assign'))

# Start of VectorUInt64_resize

`VectorUInt64_resize__SWIG_1` = function(self, new_size, x)
{
  self = as.integer(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorUInt64_resize__SWIG_1', self, new_size, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64_resize__SWIG_1`, 'returnType') = 'void'
attr(`VectorUInt64_resize__SWIG_1`, "inputTypes") = c('integer', 'integer', '_p_unsigned_long')
class(`VectorUInt64_resize__SWIG_1`) = c("SWIGFunction", class('VectorUInt64_resize__SWIG_1'))

`VectorUInt64_resize` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorUInt64_resize__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_unsigned_long') && length(argv[[3]]) == 1 )) {
      f <- VectorUInt64_resize__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorUInt64_resize with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorUInt64_reserve

`VectorUInt64_reserve` = function(self, n)
{
  self = as.integer(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;.Call('R_swig_VectorUInt64_reserve', self, n, PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64_reserve`, 'returnType') = 'void'
attr(`VectorUInt64_reserve`, "inputTypes") = c('integer', 'integer')
class(`VectorUInt64_reserve`) = c("SWIGFunction", class('VectorUInt64_reserve'))

# Start of VectorUInt64_capacity

`VectorUInt64_capacity` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorUInt64_capacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUInt64_capacity`, 'returnType') = 'integer'
attr(`VectorUInt64_capacity`, "inputTypes") = c('integer')
class(`VectorUInt64_capacity`) = c("SWIGFunction", class('VectorUInt64_capacity'))

# Start of delete_VectorUInt64

`delete_VectorUInt64` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_delete_VectorUInt64', self, PACKAGE='SimpleITK');
  
}

attr(`delete_VectorUInt64`, 'returnType') = 'void'
attr(`delete_VectorUInt64`, "inputTypes") = c('integer')
class(`delete_VectorUInt64`) = c("SWIGFunction", class('delete_VectorUInt64'))

# Start of accessor method for std::vector< uint64_t >
setMethod('$', '_p_std__vectorT_uint64_t_t', function(x, name)

{
  accessorFuns = list('__nonzero__' = VectorUInt64___nonzero__, '__len__' = VectorUInt64___len__, 'pop' = VectorUInt64_pop, '__getslice__' = VectorUInt64___getslice__, '__setslice__' = VectorUInt64___setslice__, '__delslice__' = VectorUInt64___delslice__, '__delitem__' = VectorUInt64___delitem__, '__getitem__' = VectorUInt64___getitem__, '__setitem__' = VectorUInt64___setitem__, 'append' = VectorUInt64_append, 'empty' = VectorUInt64_empty, 'size' = VectorUInt64_size, 'swap' = VectorUInt64_swap, 'clear' = VectorUInt64_clear, 'get_allocator' = VectorUInt64_get_allocator, 'pop_back' = VectorUInt64_pop_back, 'resize' = VectorUInt64_resize, 'push_back' = VectorUInt64_push_back, 'front' = VectorUInt64_front, 'back' = VectorUInt64_back, 'assign' = VectorUInt64_assign, 'reserve' = VectorUInt64_reserve, 'capacity' = VectorUInt64_capacity);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::vector< uint64_t >
# Start of VectorInt64___nonzero__

`VectorInt64___nonzero__` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt64___nonzero__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt64___nonzero__`, 'returnType') = 'logical'
attr(`VectorInt64___nonzero__`, "inputTypes") = c('integer')
class(`VectorInt64___nonzero__`) = c("SWIGFunction", class('VectorInt64___nonzero__'))

# Start of VectorInt64___len__

`VectorInt64___len__` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt64___len__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt64___len__`, 'returnType') = 'integer'
attr(`VectorInt64___len__`, "inputTypes") = c('integer')
class(`VectorInt64___len__`) = c("SWIGFunction", class('VectorInt64___len__'))

# Start of VectorInt64_pop

`VectorInt64_pop` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt64_pop', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt64_pop`, 'returnType') = 'integer'
attr(`VectorInt64_pop`, "inputTypes") = c('integer')
class(`VectorInt64_pop`) = c("SWIGFunction", class('VectorInt64_pop'))

# Start of VectorInt64___getslice__

`VectorInt64___getslice__` = function(self, i, j)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorInt64___getslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64___getslice__`, 'returnType') = 'integer'
attr(`VectorInt64___getslice__`, "inputTypes") = c('integer', 'integer', 'integer')
class(`VectorInt64___getslice__`) = c("SWIGFunction", class('VectorInt64___getslice__'))

# Start of VectorInt64___setslice__

`VectorInt64___setslice__` = function(self, i, j, is)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  is = as.integer(is);
  ;.Call('R_swig_VectorInt64___setslice__', self, i, j, is, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64___setslice__`, 'returnType') = 'void'
attr(`VectorInt64___setslice__`, "inputTypes") = c('integer', 'integer', 'integer', 'integer')
class(`VectorInt64___setslice__`) = c("SWIGFunction", class('VectorInt64___setslice__'))

# Start of VectorInt64___delslice__

`VectorInt64___delslice__` = function(self, i, j)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorInt64___delslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64___delslice__`, 'returnType') = 'void'
attr(`VectorInt64___delslice__`, "inputTypes") = c('integer', 'integer', 'integer')
class(`VectorInt64___delslice__`) = c("SWIGFunction", class('VectorInt64___delslice__'))

# Start of VectorInt64___delitem__

`VectorInt64___delitem__` = function(self, i)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorInt64___delitem__', self, i, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64___delitem__`, 'returnType') = 'void'
attr(`VectorInt64___delitem__`, "inputTypes") = c('integer', 'integer')
class(`VectorInt64___delitem__`) = c("SWIGFunction", class('VectorInt64___delitem__'))

# Start of VectorInt64___getitem__

`VectorInt64___getitem__` = function(self, i, .copy = FALSE)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorInt64___getitem__', self, i, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt64___getitem__`, 'returnType') = 'integer'
attr(`VectorInt64___getitem__`, "inputTypes") = c('integer', 'integer')
class(`VectorInt64___getitem__`) = c("SWIGFunction", class('VectorInt64___getitem__'))

# Start of VectorInt64___setitem__

`VectorInt64___setitem__` = function(self, i, x)
{
  self = as.integer(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  x = as.integer(x);
  ;.Call('R_swig_VectorInt64___setitem__', self, i, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64___setitem__`, 'returnType') = 'void'
attr(`VectorInt64___setitem__`, "inputTypes") = c('integer', 'integer', 'integer')
class(`VectorInt64___setitem__`) = c("SWIGFunction", class('VectorInt64___setitem__'))

# Start of VectorInt64_append

`VectorInt64_append` = function(self, x)
{
  self = as.integer(self);
  x = as.integer(x);
  ;.Call('R_swig_VectorInt64_append', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64_append`, 'returnType') = 'void'
attr(`VectorInt64_append`, "inputTypes") = c('integer', 'integer')
class(`VectorInt64_append`) = c("SWIGFunction", class('VectorInt64_append'))

# Start of new_VectorInt64

`VectorInt64__SWIG_0` = function()
{
  ;.Call('R_swig_new_VectorInt64__SWIG_0', PACKAGE='SimpleITK');
  
}

attr(`VectorInt64__SWIG_0`, 'returnType') = 'integer'
class(`VectorInt64__SWIG_0`) = c("SWIGFunction", class('VectorInt64__SWIG_0'))

# Start of new_VectorInt64

`VectorInt64__SWIG_1` = function(other)
{
  other = as.integer(other);
  ;.Call('R_swig_new_VectorInt64__SWIG_1', other, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64__SWIG_1`, 'returnType') = 'integer'
attr(`VectorInt64__SWIG_1`, "inputTypes") = c('integer')
class(`VectorInt64__SWIG_1`) = c("SWIGFunction", class('VectorInt64__SWIG_1'))

# Start of VectorInt64_empty

`VectorInt64_empty` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt64_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt64_empty`, 'returnType') = 'logical'
attr(`VectorInt64_empty`, "inputTypes") = c('integer')
class(`VectorInt64_empty`) = c("SWIGFunction", class('VectorInt64_empty'))

# Start of VectorInt64_size

`VectorInt64_size` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt64_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt64_size`, 'returnType') = 'integer'
attr(`VectorInt64_size`, "inputTypes") = c('integer')
class(`VectorInt64_size`) = c("SWIGFunction", class('VectorInt64_size'))

# Start of VectorInt64_swap

`VectorInt64_swap` = function(self, v)
{
  self = as.integer(self);
  v = as.integer(v);
  ;.Call('R_swig_VectorInt64_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64_swap`, 'returnType') = 'void'
attr(`VectorInt64_swap`, "inputTypes") = c('integer', 'integer')
class(`VectorInt64_swap`) = c("SWIGFunction", class('VectorInt64_swap'))

# Start of VectorInt64_clear

`VectorInt64_clear` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt64_clear', self, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64_clear`, 'returnType') = 'void'
attr(`VectorInt64_clear`, "inputTypes") = c('integer')
class(`VectorInt64_clear`) = c("SWIGFunction", class('VectorInt64_clear'))

# Start of VectorInt64_get_allocator

`VectorInt64_get_allocator` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;ans = .Call('R_swig_VectorInt64_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_long_t", ref=ans);
  
  ans
  
}

attr(`VectorInt64_get_allocator`, 'returnType') = '_p_std__allocatorT_long_t'
attr(`VectorInt64_get_allocator`, "inputTypes") = c('integer')
class(`VectorInt64_get_allocator`) = c("SWIGFunction", class('VectorInt64_get_allocator'))

# Start of new_VectorInt64

`VectorInt64__SWIG_2` = function(size)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  ;.Call('R_swig_new_VectorInt64__SWIG_2', size, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64__SWIG_2`, 'returnType') = 'integer'
attr(`VectorInt64__SWIG_2`, "inputTypes") = c('integer')
class(`VectorInt64__SWIG_2`) = c("SWIGFunction", class('VectorInt64__SWIG_2'))

# Start of VectorInt64_pop_back

`VectorInt64_pop_back` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt64_pop_back', self, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64_pop_back`, 'returnType') = 'void'
attr(`VectorInt64_pop_back`, "inputTypes") = c('integer')
class(`VectorInt64_pop_back`) = c("SWIGFunction", class('VectorInt64_pop_back'))

# Start of VectorInt64_resize

`VectorInt64_resize__SWIG_0` = function(self, new_size)
{
  self = as.integer(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  ;.Call('R_swig_VectorInt64_resize__SWIG_0', self, new_size, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64_resize__SWIG_0`, 'returnType') = 'void'
attr(`VectorInt64_resize__SWIG_0`, "inputTypes") = c('integer', 'integer')
class(`VectorInt64_resize__SWIG_0`) = c("SWIGFunction", class('VectorInt64_resize__SWIG_0'))

# Start of new_VectorInt64

`VectorInt64__SWIG_3` = function(size, value)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  value = as.integer(value);
  ;.Call('R_swig_new_VectorInt64__SWIG_3', size, value, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64__SWIG_3`, 'returnType') = 'integer'
attr(`VectorInt64__SWIG_3`, "inputTypes") = c('integer', 'integer')
class(`VectorInt64__SWIG_3`) = c("SWIGFunction", class('VectorInt64__SWIG_3'))

`VectorInt64` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 0) {
    f <- VectorInt64__SWIG_0; 
  } else if (argc == 1) {
    if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- VectorInt64__SWIG_2; 
    }
    else if ( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) {
      f <- VectorInt64__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorInt64__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorInt64 with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorInt64_push_back

`VectorInt64_push_back` = function(self, x)
{
  self = as.integer(self);
  x = as.integer(x);
  ;.Call('R_swig_VectorInt64_push_back', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64_push_back`, 'returnType') = 'void'
attr(`VectorInt64_push_back`, "inputTypes") = c('integer', 'integer')
class(`VectorInt64_push_back`) = c("SWIGFunction", class('VectorInt64_push_back'))

# Start of VectorInt64_front

`VectorInt64_front` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt64_front', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt64_front`, 'returnType') = 'integer'
attr(`VectorInt64_front`, "inputTypes") = c('integer')
class(`VectorInt64_front`) = c("SWIGFunction", class('VectorInt64_front'))

# Start of VectorInt64_back

`VectorInt64_back` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt64_back', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt64_back`, 'returnType') = 'integer'
attr(`VectorInt64_back`, "inputTypes") = c('integer')
class(`VectorInt64_back`) = c("SWIGFunction", class('VectorInt64_back'))

# Start of VectorInt64_assign

`VectorInt64_assign` = function(self, n, x)
{
  self = as.integer(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  x = as.integer(x);
  ;.Call('R_swig_VectorInt64_assign', self, n, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64_assign`, 'returnType') = 'void'
attr(`VectorInt64_assign`, "inputTypes") = c('integer', 'integer', 'integer')
class(`VectorInt64_assign`) = c("SWIGFunction", class('VectorInt64_assign'))

# Start of VectorInt64_resize

`VectorInt64_resize__SWIG_1` = function(self, new_size, x)
{
  self = as.integer(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  x = as.integer(x);
  ;.Call('R_swig_VectorInt64_resize__SWIG_1', self, new_size, x, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64_resize__SWIG_1`, 'returnType') = 'void'
attr(`VectorInt64_resize__SWIG_1`, "inputTypes") = c('integer', 'integer', 'integer')
class(`VectorInt64_resize__SWIG_1`) = c("SWIGFunction", class('VectorInt64_resize__SWIG_1'))

`VectorInt64_resize` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorInt64_resize__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- VectorInt64_resize__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorInt64_resize with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorInt64_reserve

`VectorInt64_reserve` = function(self, n)
{
  self = as.integer(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;.Call('R_swig_VectorInt64_reserve', self, n, PACKAGE='SimpleITK');
  
}

attr(`VectorInt64_reserve`, 'returnType') = 'void'
attr(`VectorInt64_reserve`, "inputTypes") = c('integer', 'integer')
class(`VectorInt64_reserve`) = c("SWIGFunction", class('VectorInt64_reserve'))

# Start of VectorInt64_capacity

`VectorInt64_capacity` = function(self, .copy = FALSE)
{
  self = as.integer(self);
  ;.Call('R_swig_VectorInt64_capacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorInt64_capacity`, 'returnType') = 'integer'
attr(`VectorInt64_capacity`, "inputTypes") = c('integer')
class(`VectorInt64_capacity`) = c("SWIGFunction", class('VectorInt64_capacity'))

# Start of delete_VectorInt64

`delete_VectorInt64` = function(self)
{
  self = as.integer(self);
  ;.Call('R_swig_delete_VectorInt64', self, PACKAGE='SimpleITK');
  
}

attr(`delete_VectorInt64`, 'returnType') = 'void'
attr(`delete_VectorInt64`, "inputTypes") = c('integer')
class(`delete_VectorInt64`) = c("SWIGFunction", class('delete_VectorInt64'))

# Start of accessor method for std::vector< int64_t >
setMethod('$', '_p_std__vectorT_int64_t_t', function(x, name)

{
  accessorFuns = list('__nonzero__' = VectorInt64___nonzero__, '__len__' = VectorInt64___len__, 'pop' = VectorInt64_pop, '__getslice__' = VectorInt64___getslice__, '__setslice__' = VectorInt64___setslice__, '__delslice__' = VectorInt64___delslice__, '__delitem__' = VectorInt64___delitem__, '__getitem__' = VectorInt64___getitem__, '__setitem__' = VectorInt64___setitem__, 'append' = VectorInt64_append, 'empty' = VectorInt64_empty, 'size' = VectorInt64_size, 'swap' = VectorInt64_swap, 'clear' = VectorInt64_clear, 'get_allocator' = VectorInt64_get_allocator, 'pop_back' = VectorInt64_pop_back, 'resize' = VectorInt64_resize, 'push_back' = VectorInt64_push_back, 'front' = VectorInt64_front, 'back' = VectorInt64_back, 'assign' = VectorInt64_assign, 'reserve' = VectorInt64_reserve, 'capacity' = VectorInt64_capacity);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::vector< int64_t >
# Start of VectorFloat___nonzero__

`VectorFloat___nonzero__` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorFloat___nonzero__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorFloat___nonzero__`, 'returnType') = 'logical'
attr(`VectorFloat___nonzero__`, "inputTypes") = c('numeric')
class(`VectorFloat___nonzero__`) = c("SWIGFunction", class('VectorFloat___nonzero__'))

# Start of VectorFloat___len__

`VectorFloat___len__` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorFloat___len__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorFloat___len__`, 'returnType') = 'integer'
attr(`VectorFloat___len__`, "inputTypes") = c('numeric')
class(`VectorFloat___len__`) = c("SWIGFunction", class('VectorFloat___len__'))

# Start of VectorFloat_pop

`VectorFloat_pop` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorFloat_pop', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorFloat_pop`, 'returnType') = 'numeric'
attr(`VectorFloat_pop`, "inputTypes") = c('numeric')
class(`VectorFloat_pop`) = c("SWIGFunction", class('VectorFloat_pop'))

# Start of VectorFloat___getslice__

`VectorFloat___getslice__` = function(self, i, j)
{
  self = as.numeric(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorFloat___getslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat___getslice__`, 'returnType') = 'numeric'
attr(`VectorFloat___getslice__`, "inputTypes") = c('numeric', 'integer', 'integer')
class(`VectorFloat___getslice__`) = c("SWIGFunction", class('VectorFloat___getslice__'))

# Start of VectorFloat___setslice__

`VectorFloat___setslice__` = function(self, i, j, is)
{
  self = as.numeric(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  is = as.numeric(is);
  ;.Call('R_swig_VectorFloat___setslice__', self, i, j, is, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat___setslice__`, 'returnType') = 'void'
attr(`VectorFloat___setslice__`, "inputTypes") = c('numeric', 'integer', 'integer', 'numeric')
class(`VectorFloat___setslice__`) = c("SWIGFunction", class('VectorFloat___setslice__'))

# Start of VectorFloat___delslice__

`VectorFloat___delslice__` = function(self, i, j)
{
  self = as.numeric(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorFloat___delslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat___delslice__`, 'returnType') = 'void'
attr(`VectorFloat___delslice__`, "inputTypes") = c('numeric', 'integer', 'integer')
class(`VectorFloat___delslice__`) = c("SWIGFunction", class('VectorFloat___delslice__'))

# Start of VectorFloat___delitem__

`VectorFloat___delitem__` = function(self, i)
{
  self = as.numeric(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorFloat___delitem__', self, i, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat___delitem__`, 'returnType') = 'void'
attr(`VectorFloat___delitem__`, "inputTypes") = c('numeric', 'integer')
class(`VectorFloat___delitem__`) = c("SWIGFunction", class('VectorFloat___delitem__'))

# Start of VectorFloat___getitem__

`VectorFloat___getitem__` = function(self, i, .copy = FALSE)
{
  self = as.numeric(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorFloat___getitem__', self, i, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorFloat___getitem__`, 'returnType') = 'numeric'
attr(`VectorFloat___getitem__`, "inputTypes") = c('numeric', 'integer')
class(`VectorFloat___getitem__`) = c("SWIGFunction", class('VectorFloat___getitem__'))

# Start of VectorFloat___setitem__

`VectorFloat___setitem__` = function(self, i, x)
{
  self = as.numeric(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  
  ;.Call('R_swig_VectorFloat___setitem__', self, i, x, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat___setitem__`, 'returnType') = 'void'
attr(`VectorFloat___setitem__`, "inputTypes") = c('numeric', 'integer', 'numeric')
class(`VectorFloat___setitem__`) = c("SWIGFunction", class('VectorFloat___setitem__'))

# Start of VectorFloat_append

`VectorFloat_append` = function(self, x)
{
  self = as.numeric(self);
  
  ;.Call('R_swig_VectorFloat_append', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat_append`, 'returnType') = 'void'
attr(`VectorFloat_append`, "inputTypes") = c('numeric', 'numeric')
class(`VectorFloat_append`) = c("SWIGFunction", class('VectorFloat_append'))

# Start of new_VectorFloat

`VectorFloat__SWIG_0` = function()
{
  ;.Call('R_swig_new_VectorFloat__SWIG_0', PACKAGE='SimpleITK');
  
}

attr(`VectorFloat__SWIG_0`, 'returnType') = 'numeric'
class(`VectorFloat__SWIG_0`) = c("SWIGFunction", class('VectorFloat__SWIG_0'))

# Start of new_VectorFloat

`VectorFloat__SWIG_1` = function(other)
{
  other = as.numeric(other);
  ;.Call('R_swig_new_VectorFloat__SWIG_1', other, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat__SWIG_1`, 'returnType') = 'numeric'
attr(`VectorFloat__SWIG_1`, "inputTypes") = c('numeric')
class(`VectorFloat__SWIG_1`) = c("SWIGFunction", class('VectorFloat__SWIG_1'))

# Start of VectorFloat_empty

`VectorFloat_empty` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorFloat_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorFloat_empty`, 'returnType') = 'logical'
attr(`VectorFloat_empty`, "inputTypes") = c('numeric')
class(`VectorFloat_empty`) = c("SWIGFunction", class('VectorFloat_empty'))

# Start of VectorFloat_size

`VectorFloat_size` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorFloat_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorFloat_size`, 'returnType') = 'integer'
attr(`VectorFloat_size`, "inputTypes") = c('numeric')
class(`VectorFloat_size`) = c("SWIGFunction", class('VectorFloat_size'))

# Start of VectorFloat_swap

`VectorFloat_swap` = function(self, v)
{
  self = as.numeric(self);
  v = as.numeric(v);
  ;.Call('R_swig_VectorFloat_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat_swap`, 'returnType') = 'void'
attr(`VectorFloat_swap`, "inputTypes") = c('numeric', 'numeric')
class(`VectorFloat_swap`) = c("SWIGFunction", class('VectorFloat_swap'))

# Start of VectorFloat_clear

`VectorFloat_clear` = function(self)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorFloat_clear', self, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat_clear`, 'returnType') = 'void'
attr(`VectorFloat_clear`, "inputTypes") = c('numeric')
class(`VectorFloat_clear`) = c("SWIGFunction", class('VectorFloat_clear'))

# Start of VectorFloat_get_allocator

`VectorFloat_get_allocator` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;ans = .Call('R_swig_VectorFloat_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_float_t", ref=ans);
  
  ans
  
}

attr(`VectorFloat_get_allocator`, 'returnType') = '_p_std__allocatorT_float_t'
attr(`VectorFloat_get_allocator`, "inputTypes") = c('numeric')
class(`VectorFloat_get_allocator`) = c("SWIGFunction", class('VectorFloat_get_allocator'))

# Start of new_VectorFloat

`VectorFloat__SWIG_2` = function(size)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  ;.Call('R_swig_new_VectorFloat__SWIG_2', size, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat__SWIG_2`, 'returnType') = 'numeric'
attr(`VectorFloat__SWIG_2`, "inputTypes") = c('integer')
class(`VectorFloat__SWIG_2`) = c("SWIGFunction", class('VectorFloat__SWIG_2'))

# Start of VectorFloat_pop_back

`VectorFloat_pop_back` = function(self)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorFloat_pop_back', self, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat_pop_back`, 'returnType') = 'void'
attr(`VectorFloat_pop_back`, "inputTypes") = c('numeric')
class(`VectorFloat_pop_back`) = c("SWIGFunction", class('VectorFloat_pop_back'))

# Start of VectorFloat_resize

`VectorFloat_resize__SWIG_0` = function(self, new_size)
{
  self = as.numeric(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  ;.Call('R_swig_VectorFloat_resize__SWIG_0', self, new_size, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat_resize__SWIG_0`, 'returnType') = 'void'
attr(`VectorFloat_resize__SWIG_0`, "inputTypes") = c('numeric', 'integer')
class(`VectorFloat_resize__SWIG_0`) = c("SWIGFunction", class('VectorFloat_resize__SWIG_0'))

# Start of new_VectorFloat

`VectorFloat__SWIG_3` = function(size, value)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  
  ;.Call('R_swig_new_VectorFloat__SWIG_3', size, value, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat__SWIG_3`, 'returnType') = 'numeric'
attr(`VectorFloat__SWIG_3`, "inputTypes") = c('integer', 'numeric')
class(`VectorFloat__SWIG_3`) = c("SWIGFunction", class('VectorFloat__SWIG_3'))

`VectorFloat` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 0) {
    f <- VectorFloat__SWIG_0; 
  } else if (argc == 1) {
    if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- VectorFloat__SWIG_2; 
    }
    else if ( is.numeric(argv[[1]]) ) {
      f <- VectorFloat__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- VectorFloat__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorFloat with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorFloat_push_back

`VectorFloat_push_back` = function(self, x)
{
  self = as.numeric(self);
  
  ;.Call('R_swig_VectorFloat_push_back', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat_push_back`, 'returnType') = 'void'
attr(`VectorFloat_push_back`, "inputTypes") = c('numeric', 'numeric')
class(`VectorFloat_push_back`) = c("SWIGFunction", class('VectorFloat_push_back'))

# Start of VectorFloat_front

`VectorFloat_front` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorFloat_front', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorFloat_front`, 'returnType') = 'numeric'
attr(`VectorFloat_front`, "inputTypes") = c('numeric')
class(`VectorFloat_front`) = c("SWIGFunction", class('VectorFloat_front'))

# Start of VectorFloat_back

`VectorFloat_back` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorFloat_back', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorFloat_back`, 'returnType') = 'numeric'
attr(`VectorFloat_back`, "inputTypes") = c('numeric')
class(`VectorFloat_back`) = c("SWIGFunction", class('VectorFloat_back'))

# Start of VectorFloat_assign

`VectorFloat_assign` = function(self, n, x)
{
  self = as.numeric(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  
  ;.Call('R_swig_VectorFloat_assign', self, n, x, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat_assign`, 'returnType') = 'void'
attr(`VectorFloat_assign`, "inputTypes") = c('numeric', 'integer', 'numeric')
class(`VectorFloat_assign`) = c("SWIGFunction", class('VectorFloat_assign'))

# Start of VectorFloat_resize

`VectorFloat_resize__SWIG_1` = function(self, new_size, x)
{
  self = as.numeric(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  
  ;.Call('R_swig_VectorFloat_resize__SWIG_1', self, new_size, x, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat_resize__SWIG_1`, 'returnType') = 'void'
attr(`VectorFloat_resize__SWIG_1`, "inputTypes") = c('numeric', 'integer', 'numeric')
class(`VectorFloat_resize__SWIG_1`) = c("SWIGFunction", class('VectorFloat_resize__SWIG_1'))

`VectorFloat_resize` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorFloat_resize__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- VectorFloat_resize__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorFloat_resize with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorFloat_reserve

`VectorFloat_reserve` = function(self, n)
{
  self = as.numeric(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;.Call('R_swig_VectorFloat_reserve', self, n, PACKAGE='SimpleITK');
  
}

attr(`VectorFloat_reserve`, 'returnType') = 'void'
attr(`VectorFloat_reserve`, "inputTypes") = c('numeric', 'integer')
class(`VectorFloat_reserve`) = c("SWIGFunction", class('VectorFloat_reserve'))

# Start of VectorFloat_capacity

`VectorFloat_capacity` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorFloat_capacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorFloat_capacity`, 'returnType') = 'integer'
attr(`VectorFloat_capacity`, "inputTypes") = c('numeric')
class(`VectorFloat_capacity`) = c("SWIGFunction", class('VectorFloat_capacity'))

# Start of delete_VectorFloat

`delete_VectorFloat` = function(self)
{
  self = as.numeric(self);
  ;.Call('R_swig_delete_VectorFloat', self, PACKAGE='SimpleITK');
  
}

attr(`delete_VectorFloat`, 'returnType') = 'void'
attr(`delete_VectorFloat`, "inputTypes") = c('numeric')
class(`delete_VectorFloat`) = c("SWIGFunction", class('delete_VectorFloat'))

# Start of accessor method for std::vector< float >
setMethod('$', '_p_std__vectorT_float_t', function(x, name)

{
  accessorFuns = list('__nonzero__' = VectorFloat___nonzero__, '__len__' = VectorFloat___len__, 'pop' = VectorFloat_pop, '__getslice__' = VectorFloat___getslice__, '__setslice__' = VectorFloat___setslice__, '__delslice__' = VectorFloat___delslice__, '__delitem__' = VectorFloat___delitem__, '__getitem__' = VectorFloat___getitem__, '__setitem__' = VectorFloat___setitem__, 'append' = VectorFloat_append, 'empty' = VectorFloat_empty, 'size' = VectorFloat_size, 'swap' = VectorFloat_swap, 'clear' = VectorFloat_clear, 'get_allocator' = VectorFloat_get_allocator, 'pop_back' = VectorFloat_pop_back, 'resize' = VectorFloat_resize, 'push_back' = VectorFloat_push_back, 'front' = VectorFloat_front, 'back' = VectorFloat_back, 'assign' = VectorFloat_assign, 'reserve' = VectorFloat_reserve, 'capacity' = VectorFloat_capacity);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::vector< float >
# Start of VectorDouble___nonzero__

`VectorDouble___nonzero__` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorDouble___nonzero__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorDouble___nonzero__`, 'returnType') = 'logical'
attr(`VectorDouble___nonzero__`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`VectorDouble___nonzero__`) = c("SWIGFunction", class('VectorDouble___nonzero__'))

# Start of VectorDouble___len__

`VectorDouble___len__` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorDouble___len__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorDouble___len__`, 'returnType') = 'integer'
attr(`VectorDouble___len__`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`VectorDouble___len__`) = c("SWIGFunction", class('VectorDouble___len__'))

# Start of VectorDouble_pop

`VectorDouble_pop` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorDouble_pop', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorDouble_pop`, 'returnType') = 'numeric'
attr(`VectorDouble_pop`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`VectorDouble_pop`) = c("SWIGFunction", class('VectorDouble_pop'))

# Start of VectorDouble___getslice__

`VectorDouble___getslice__` = function(self, i, j)
{
  self = as.numeric(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorDouble___getslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble___getslice__`, 'returnType') = '_p_std__vectorT_double_t'
attr(`VectorDouble___getslice__`, "inputTypes") = c('_p_std__vectorT_double_t', 'integer', 'integer')
class(`VectorDouble___getslice__`) = c("SWIGFunction", class('VectorDouble___getslice__'))

# Start of VectorDouble___setslice__

`VectorDouble___setslice__` = function(self, i, j, is)
{
  self = as.numeric(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  is = as.numeric(is);
  ;.Call('R_swig_VectorDouble___setslice__', self, i, j, is, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble___setslice__`, 'returnType') = 'void'
attr(`VectorDouble___setslice__`, "inputTypes") = c('_p_std__vectorT_double_t', 'integer', 'integer', '_p_std__vectorT_double_t')
class(`VectorDouble___setslice__`) = c("SWIGFunction", class('VectorDouble___setslice__'))

# Start of VectorDouble___delslice__

`VectorDouble___delslice__` = function(self, i, j)
{
  self = as.numeric(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorDouble___delslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble___delslice__`, 'returnType') = 'void'
attr(`VectorDouble___delslice__`, "inputTypes") = c('_p_std__vectorT_double_t', 'integer', 'integer')
class(`VectorDouble___delslice__`) = c("SWIGFunction", class('VectorDouble___delslice__'))

# Start of VectorDouble___delitem__

`VectorDouble___delitem__` = function(self, i)
{
  self = as.numeric(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorDouble___delitem__', self, i, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble___delitem__`, 'returnType') = 'void'
attr(`VectorDouble___delitem__`, "inputTypes") = c('_p_std__vectorT_double_t', 'integer')
class(`VectorDouble___delitem__`) = c("SWIGFunction", class('VectorDouble___delitem__'))

# Start of VectorDouble___getitem__

`VectorDouble___getitem__` = function(self, i, .copy = FALSE)
{
  self = as.numeric(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorDouble___getitem__', self, i, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorDouble___getitem__`, 'returnType') = 'numeric'
attr(`VectorDouble___getitem__`, "inputTypes") = c('_p_std__vectorT_double_t', 'integer')
class(`VectorDouble___getitem__`) = c("SWIGFunction", class('VectorDouble___getitem__'))

# Start of VectorDouble___setitem__

`VectorDouble___setitem__` = function(self, i, x)
{
  self = as.numeric(self);
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  
  ;.Call('R_swig_VectorDouble___setitem__', self, i, x, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble___setitem__`, 'returnType') = 'void'
attr(`VectorDouble___setitem__`, "inputTypes") = c('_p_std__vectorT_double_t', 'integer', 'numeric')
class(`VectorDouble___setitem__`) = c("SWIGFunction", class('VectorDouble___setitem__'))

# Start of VectorDouble_append

`VectorDouble_append` = function(self, x)
{
  self = as.numeric(self);
  
  ;.Call('R_swig_VectorDouble_append', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble_append`, 'returnType') = 'void'
attr(`VectorDouble_append`, "inputTypes") = c('_p_std__vectorT_double_t', 'numeric')
class(`VectorDouble_append`) = c("SWIGFunction", class('VectorDouble_append'))

# Start of new_VectorDouble

`VectorDouble__SWIG_0` = function()
{
  ;.Call('R_swig_new_VectorDouble__SWIG_0', PACKAGE='SimpleITK');
  
}

attr(`VectorDouble__SWIG_0`, 'returnType') = '_p_std__vectorT_double_t'
class(`VectorDouble__SWIG_0`) = c("SWIGFunction", class('VectorDouble__SWIG_0'))

# Start of new_VectorDouble

`VectorDouble__SWIG_1` = function(other)
{
  other = as.numeric(other);
  ;.Call('R_swig_new_VectorDouble__SWIG_1', other, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble__SWIG_1`, 'returnType') = '_p_std__vectorT_double_t'
attr(`VectorDouble__SWIG_1`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`VectorDouble__SWIG_1`) = c("SWIGFunction", class('VectorDouble__SWIG_1'))

# Start of VectorDouble_empty

`VectorDouble_empty` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorDouble_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorDouble_empty`, 'returnType') = 'logical'
attr(`VectorDouble_empty`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`VectorDouble_empty`) = c("SWIGFunction", class('VectorDouble_empty'))

# Start of VectorDouble_size

`VectorDouble_size` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorDouble_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorDouble_size`, 'returnType') = 'integer'
attr(`VectorDouble_size`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`VectorDouble_size`) = c("SWIGFunction", class('VectorDouble_size'))

# Start of VectorDouble_swap

`VectorDouble_swap` = function(self, v)
{
  self = as.numeric(self);
  v = as.numeric(v);
  ;.Call('R_swig_VectorDouble_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble_swap`, 'returnType') = 'void'
attr(`VectorDouble_swap`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`VectorDouble_swap`) = c("SWIGFunction", class('VectorDouble_swap'))

# Start of VectorDouble_clear

`VectorDouble_clear` = function(self)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorDouble_clear', self, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble_clear`, 'returnType') = 'void'
attr(`VectorDouble_clear`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`VectorDouble_clear`) = c("SWIGFunction", class('VectorDouble_clear'))

# Start of VectorDouble_get_allocator

`VectorDouble_get_allocator` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;ans = .Call('R_swig_VectorDouble_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_double_t", ref=ans);
  
  ans
  
}

attr(`VectorDouble_get_allocator`, 'returnType') = '_p_std__allocatorT_double_t'
attr(`VectorDouble_get_allocator`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`VectorDouble_get_allocator`) = c("SWIGFunction", class('VectorDouble_get_allocator'))

# Start of new_VectorDouble

`VectorDouble__SWIG_2` = function(size)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  ;.Call('R_swig_new_VectorDouble__SWIG_2', size, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble__SWIG_2`, 'returnType') = '_p_std__vectorT_double_t'
attr(`VectorDouble__SWIG_2`, "inputTypes") = c('integer')
class(`VectorDouble__SWIG_2`) = c("SWIGFunction", class('VectorDouble__SWIG_2'))

# Start of VectorDouble_pop_back

`VectorDouble_pop_back` = function(self)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorDouble_pop_back', self, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble_pop_back`, 'returnType') = 'void'
attr(`VectorDouble_pop_back`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`VectorDouble_pop_back`) = c("SWIGFunction", class('VectorDouble_pop_back'))

# Start of VectorDouble_resize

`VectorDouble_resize__SWIG_0` = function(self, new_size)
{
  self = as.numeric(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  ;.Call('R_swig_VectorDouble_resize__SWIG_0', self, new_size, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble_resize__SWIG_0`, 'returnType') = 'void'
attr(`VectorDouble_resize__SWIG_0`, "inputTypes") = c('_p_std__vectorT_double_t', 'integer')
class(`VectorDouble_resize__SWIG_0`) = c("SWIGFunction", class('VectorDouble_resize__SWIG_0'))

# Start of new_VectorDouble

`VectorDouble__SWIG_3` = function(size, value)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  
  ;.Call('R_swig_new_VectorDouble__SWIG_3', size, value, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble__SWIG_3`, 'returnType') = '_p_std__vectorT_double_t'
attr(`VectorDouble__SWIG_3`, "inputTypes") = c('integer', 'numeric')
class(`VectorDouble__SWIG_3`) = c("SWIGFunction", class('VectorDouble__SWIG_3'))

`VectorDouble` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 0) {
    f <- VectorDouble__SWIG_0; 
  } else if (argc == 1) {
    if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- VectorDouble__SWIG_2; 
    }
    else if ( is.numeric(argv[[1]]) ) {
      f <- VectorDouble__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- VectorDouble__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorDouble with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorDouble_push_back

`VectorDouble_push_back` = function(self, x)
{
  self = as.numeric(self);
  
  ;.Call('R_swig_VectorDouble_push_back', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble_push_back`, 'returnType') = 'void'
attr(`VectorDouble_push_back`, "inputTypes") = c('_p_std__vectorT_double_t', 'numeric')
class(`VectorDouble_push_back`) = c("SWIGFunction", class('VectorDouble_push_back'))

# Start of VectorDouble_front

`VectorDouble_front` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorDouble_front', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorDouble_front`, 'returnType') = 'numeric'
attr(`VectorDouble_front`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`VectorDouble_front`) = c("SWIGFunction", class('VectorDouble_front'))

# Start of VectorDouble_back

`VectorDouble_back` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorDouble_back', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorDouble_back`, 'returnType') = 'numeric'
attr(`VectorDouble_back`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`VectorDouble_back`) = c("SWIGFunction", class('VectorDouble_back'))

# Start of VectorDouble_assign

`VectorDouble_assign` = function(self, n, x)
{
  self = as.numeric(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  
  ;.Call('R_swig_VectorDouble_assign', self, n, x, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble_assign`, 'returnType') = 'void'
attr(`VectorDouble_assign`, "inputTypes") = c('_p_std__vectorT_double_t', 'integer', 'numeric')
class(`VectorDouble_assign`) = c("SWIGFunction", class('VectorDouble_assign'))

# Start of VectorDouble_resize

`VectorDouble_resize__SWIG_1` = function(self, new_size, x)
{
  self = as.numeric(self);
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  
  ;.Call('R_swig_VectorDouble_resize__SWIG_1', self, new_size, x, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble_resize__SWIG_1`, 'returnType') = 'void'
attr(`VectorDouble_resize__SWIG_1`, "inputTypes") = c('_p_std__vectorT_double_t', 'integer', 'numeric')
class(`VectorDouble_resize__SWIG_1`) = c("SWIGFunction", class('VectorDouble_resize__SWIG_1'))

`VectorDouble_resize` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorDouble_resize__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( is.numeric(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- VectorDouble_resize__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorDouble_resize with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorDouble_reserve

`VectorDouble_reserve` = function(self, n)
{
  self = as.numeric(self);
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;.Call('R_swig_VectorDouble_reserve', self, n, PACKAGE='SimpleITK');
  
}

attr(`VectorDouble_reserve`, 'returnType') = 'void'
attr(`VectorDouble_reserve`, "inputTypes") = c('_p_std__vectorT_double_t', 'integer')
class(`VectorDouble_reserve`) = c("SWIGFunction", class('VectorDouble_reserve'))

# Start of VectorDouble_capacity

`VectorDouble_capacity` = function(self, .copy = FALSE)
{
  self = as.numeric(self);
  ;.Call('R_swig_VectorDouble_capacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorDouble_capacity`, 'returnType') = 'integer'
attr(`VectorDouble_capacity`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`VectorDouble_capacity`) = c("SWIGFunction", class('VectorDouble_capacity'))

# Start of delete_VectorDouble

`delete_VectorDouble` = function(self)
{
  self = as.numeric(self);
  ;.Call('R_swig_delete_VectorDouble', self, PACKAGE='SimpleITK');
  
}

attr(`delete_VectorDouble`, 'returnType') = 'void'
attr(`delete_VectorDouble`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`delete_VectorDouble`) = c("SWIGFunction", class('delete_VectorDouble'))

# Start of accessor method for std::vector< double >
setMethod('$', '_p_std__vectorT_double_t', function(x, name)

{
  accessorFuns = list('__nonzero__' = VectorDouble___nonzero__, '__len__' = VectorDouble___len__, 'pop' = VectorDouble_pop, '__getslice__' = VectorDouble___getslice__, '__setslice__' = VectorDouble___setslice__, '__delslice__' = VectorDouble___delslice__, '__delitem__' = VectorDouble___delitem__, '__getitem__' = VectorDouble___getitem__, '__setitem__' = VectorDouble___setitem__, 'append' = VectorDouble_append, 'empty' = VectorDouble_empty, 'size' = VectorDouble_size, 'swap' = VectorDouble_swap, 'clear' = VectorDouble_clear, 'get_allocator' = VectorDouble_get_allocator, 'pop_back' = VectorDouble_pop_back, 'resize' = VectorDouble_resize, 'push_back' = VectorDouble_push_back, 'front' = VectorDouble_front, 'back' = VectorDouble_back, 'assign' = VectorDouble_assign, 'reserve' = VectorDouble_reserve, 'capacity' = VectorDouble_capacity);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::vector< double >
# Start of VectorOfImage___nonzero__

`VectorOfImage___nonzero__` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorOfImage___nonzero__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage___nonzero__`, 'returnType') = 'logical'
attr(`VectorOfImage___nonzero__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t')
class(`VectorOfImage___nonzero__`) = c("SWIGFunction", class('VectorOfImage___nonzero__'))

# Start of VectorOfImage___len__

`VectorOfImage___len__` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorOfImage___len__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage___len__`, 'returnType') = 'integer'
attr(`VectorOfImage___len__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t')
class(`VectorOfImage___len__`) = c("SWIGFunction", class('VectorOfImage___len__'))

# Start of VectorOfImage_pop

`VectorOfImage_pop` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorOfImage_pop', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_itk__simple__Image", ref=ans);
  
  ans
  
}

attr(`VectorOfImage_pop`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorOfImage_pop`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t')
class(`VectorOfImage_pop`) = c("SWIGFunction", class('VectorOfImage_pop'))

# Start of VectorOfImage___getslice__

`VectorOfImage___getslice__` = function(self, i, j)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;ans = .Call('R_swig_VectorOfImage___getslice__', self, i, j, PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__vectorT_itk__simple__Image_t", ref=ans);
  
  ans
  
}

attr(`VectorOfImage___getslice__`, 'returnType') = '_p_std__vectorT_itk__simple__Image_t'
attr(`VectorOfImage___getslice__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t', 'integer', 'integer')
class(`VectorOfImage___getslice__`) = c("SWIGFunction", class('VectorOfImage___getslice__'))

# Start of VectorOfImage___setslice__

`VectorOfImage___setslice__` = function(self, i, j, is)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  if (inherits(is, "ExternalReference")) is = slot(is,"ref"); 
  ;.Call('R_swig_VectorOfImage___setslice__', self, i, j, is, PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage___setslice__`, 'returnType') = 'void'
attr(`VectorOfImage___setslice__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t', 'integer', 'integer', '_p_std__vectorT_itk__simple__Image_t')
class(`VectorOfImage___setslice__`) = c("SWIGFunction", class('VectorOfImage___setslice__'))

# Start of VectorOfImage___delslice__

`VectorOfImage___delslice__` = function(self, i, j)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorOfImage___delslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage___delslice__`, 'returnType') = 'void'
attr(`VectorOfImage___delslice__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t', 'integer', 'integer')
class(`VectorOfImage___delslice__`) = c("SWIGFunction", class('VectorOfImage___delslice__'))

# Start of VectorOfImage___delitem__

`VectorOfImage___delitem__` = function(self, i)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorOfImage___delitem__', self, i, PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage___delitem__`, 'returnType') = 'void'
attr(`VectorOfImage___delitem__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t', 'integer')
class(`VectorOfImage___delitem__`) = c("SWIGFunction", class('VectorOfImage___delitem__'))

# Start of VectorOfImage___getitem__

`VectorOfImage___getitem__` = function(self, i, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;ans = .Call('R_swig_VectorOfImage___getitem__', self, i, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_itk__simple__Image", ref=ans);
  
  ans
  
}

attr(`VectorOfImage___getitem__`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorOfImage___getitem__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t', 'integer')
class(`VectorOfImage___getitem__`) = c("SWIGFunction", class('VectorOfImage___getitem__'))

# Start of VectorOfImage___setitem__

`VectorOfImage___setitem__` = function(self, i, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorOfImage___setitem__', self, i, x, PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage___setitem__`, 'returnType') = 'void'
attr(`VectorOfImage___setitem__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t', 'integer', '_p_itk__simple__Image')
class(`VectorOfImage___setitem__`) = c("SWIGFunction", class('VectorOfImage___setitem__'))

# Start of VectorOfImage_append

`VectorOfImage_append` = function(self, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorOfImage_append', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage_append`, 'returnType') = 'void'
attr(`VectorOfImage_append`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t', '_p_itk__simple__Image')
class(`VectorOfImage_append`) = c("SWIGFunction", class('VectorOfImage_append'))

# Start of new_VectorOfImage

`VectorOfImage__SWIG_0` = function()
{
  ;ans = .Call('R_swig_new_VectorOfImage__SWIG_0', PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__vectorT_itk__simple__Image_t", ref=ans);
  
  reg.finalizer(ans@ref, delete_VectorOfImage);
  ans
  
}

attr(`VectorOfImage__SWIG_0`, 'returnType') = '_p_std__vectorT_itk__simple__Image_t'
class(`VectorOfImage__SWIG_0`) = c("SWIGFunction", class('VectorOfImage__SWIG_0'))

# Start of new_VectorOfImage

`VectorOfImage__SWIG_1` = function(other)
{
  if (inherits(other, "ExternalReference")) other = slot(other,"ref"); 
  ;ans = .Call('R_swig_new_VectorOfImage__SWIG_1', other, PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__vectorT_itk__simple__Image_t", ref=ans);
  
  reg.finalizer(ans@ref, delete_VectorOfImage);
  ans
  
}

attr(`VectorOfImage__SWIG_1`, 'returnType') = '_p_std__vectorT_itk__simple__Image_t'
attr(`VectorOfImage__SWIG_1`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t')
class(`VectorOfImage__SWIG_1`) = c("SWIGFunction", class('VectorOfImage__SWIG_1'))

# Start of VectorOfImage_empty

`VectorOfImage_empty` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorOfImage_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage_empty`, 'returnType') = 'logical'
attr(`VectorOfImage_empty`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t')
class(`VectorOfImage_empty`) = c("SWIGFunction", class('VectorOfImage_empty'))

# Start of VectorOfImage_size

`VectorOfImage_size` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorOfImage_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage_size`, 'returnType') = 'integer'
attr(`VectorOfImage_size`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t')
class(`VectorOfImage_size`) = c("SWIGFunction", class('VectorOfImage_size'))

# Start of VectorOfImage_swap

`VectorOfImage_swap` = function(self, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(v, "ExternalReference")) v = slot(v,"ref"); 
  ;.Call('R_swig_VectorOfImage_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage_swap`, 'returnType') = 'void'
attr(`VectorOfImage_swap`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t', '_p_std__vectorT_itk__simple__Image_t')
class(`VectorOfImage_swap`) = c("SWIGFunction", class('VectorOfImage_swap'))

# Start of VectorOfImage_clear

`VectorOfImage_clear` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorOfImage_clear', self, PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage_clear`, 'returnType') = 'void'
attr(`VectorOfImage_clear`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t')
class(`VectorOfImage_clear`) = c("SWIGFunction", class('VectorOfImage_clear'))

# Start of VectorOfImage_get_allocator

`VectorOfImage_get_allocator` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorOfImage_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_itk__simple__Image_t", ref=ans);
  
  ans
  
}

attr(`VectorOfImage_get_allocator`, 'returnType') = '_p_std__allocatorT_itk__simple__Image_t'
attr(`VectorOfImage_get_allocator`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t')
class(`VectorOfImage_get_allocator`) = c("SWIGFunction", class('VectorOfImage_get_allocator'))

# Start of new_VectorOfImage

`VectorOfImage__SWIG_2` = function(size)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  ;ans = .Call('R_swig_new_VectorOfImage__SWIG_2', size, PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__vectorT_itk__simple__Image_t", ref=ans);
  
  reg.finalizer(ans@ref, delete_VectorOfImage);
  ans
  
}

attr(`VectorOfImage__SWIG_2`, 'returnType') = '_p_std__vectorT_itk__simple__Image_t'
attr(`VectorOfImage__SWIG_2`, "inputTypes") = c('integer')
class(`VectorOfImage__SWIG_2`) = c("SWIGFunction", class('VectorOfImage__SWIG_2'))

# Start of VectorOfImage_pop_back

`VectorOfImage_pop_back` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorOfImage_pop_back', self, PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage_pop_back`, 'returnType') = 'void'
attr(`VectorOfImage_pop_back`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t')
class(`VectorOfImage_pop_back`) = c("SWIGFunction", class('VectorOfImage_pop_back'))

# Start of VectorOfImage_resize

`VectorOfImage_resize__SWIG_0` = function(self, new_size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  ;.Call('R_swig_VectorOfImage_resize__SWIG_0', self, new_size, PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage_resize__SWIG_0`, 'returnType') = 'void'
attr(`VectorOfImage_resize__SWIG_0`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t', 'integer')
class(`VectorOfImage_resize__SWIG_0`) = c("SWIGFunction", class('VectorOfImage_resize__SWIG_0'))

# Start of new_VectorOfImage

`VectorOfImage__SWIG_3` = function(size, value)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  if (inherits(value, "ExternalReference")) value = slot(value,"ref"); 
  ;ans = .Call('R_swig_new_VectorOfImage__SWIG_3', size, value, PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__vectorT_itk__simple__Image_t", ref=ans);
  
  reg.finalizer(ans@ref, delete_VectorOfImage);
  ans
  
}

attr(`VectorOfImage__SWIG_3`, 'returnType') = '_p_std__vectorT_itk__simple__Image_t'
attr(`VectorOfImage__SWIG_3`, "inputTypes") = c('integer', '_p_itk__simple__Image')
class(`VectorOfImage__SWIG_3`) = c("SWIGFunction", class('VectorOfImage__SWIG_3'))

`VectorOfImage` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 0) {
    f <- VectorOfImage__SWIG_0; 
  } else if (argc == 1) {
    if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- VectorOfImage__SWIG_2; 
    }
    else if ( extends(argtypes[1], '_p_std__vectorT_itk__simple__Image_t') && length(argv[[1]]) == 1 ) {
      f <- VectorOfImage__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- VectorOfImage__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorOfImage with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorOfImage_push_back

`VectorOfImage_push_back` = function(self, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorOfImage_push_back', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage_push_back`, 'returnType') = 'void'
attr(`VectorOfImage_push_back`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t', '_p_itk__simple__Image')
class(`VectorOfImage_push_back`) = c("SWIGFunction", class('VectorOfImage_push_back'))

# Start of VectorOfImage_front

`VectorOfImage_front` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorOfImage_front', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_itk__simple__Image", ref=ans);
  
  ans
  
}

attr(`VectorOfImage_front`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorOfImage_front`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t')
class(`VectorOfImage_front`) = c("SWIGFunction", class('VectorOfImage_front'))

# Start of VectorOfImage_back

`VectorOfImage_back` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorOfImage_back', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_itk__simple__Image", ref=ans);
  
  ans
  
}

attr(`VectorOfImage_back`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorOfImage_back`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t')
class(`VectorOfImage_back`) = c("SWIGFunction", class('VectorOfImage_back'))

# Start of VectorOfImage_assign

`VectorOfImage_assign` = function(self, n, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorOfImage_assign', self, n, x, PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage_assign`, 'returnType') = 'void'
attr(`VectorOfImage_assign`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t', 'integer', '_p_itk__simple__Image')
class(`VectorOfImage_assign`) = c("SWIGFunction", class('VectorOfImage_assign'))

# Start of VectorOfImage_resize

`VectorOfImage_resize__SWIG_1` = function(self, new_size, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorOfImage_resize__SWIG_1', self, new_size, x, PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage_resize__SWIG_1`, 'returnType') = 'void'
attr(`VectorOfImage_resize__SWIG_1`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t', 'integer', '_p_itk__simple__Image')
class(`VectorOfImage_resize__SWIG_1`) = c("SWIGFunction", class('VectorOfImage_resize__SWIG_1'))

`VectorOfImage_resize` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_std__vectorT_itk__simple__Image_t') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorOfImage_resize__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_std__vectorT_itk__simple__Image_t') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- VectorOfImage_resize__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorOfImage_resize with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorOfImage_reserve

`VectorOfImage_reserve` = function(self, n)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;.Call('R_swig_VectorOfImage_reserve', self, n, PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage_reserve`, 'returnType') = 'void'
attr(`VectorOfImage_reserve`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t', 'integer')
class(`VectorOfImage_reserve`) = c("SWIGFunction", class('VectorOfImage_reserve'))

# Start of VectorOfImage_capacity

`VectorOfImage_capacity` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorOfImage_capacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorOfImage_capacity`, 'returnType') = 'integer'
attr(`VectorOfImage_capacity`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t')
class(`VectorOfImage_capacity`) = c("SWIGFunction", class('VectorOfImage_capacity'))

# Start of delete_VectorOfImage

`delete_VectorOfImage` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_delete_VectorOfImage', self, PACKAGE='SimpleITK');
  
}

attr(`delete_VectorOfImage`, 'returnType') = 'void'
attr(`delete_VectorOfImage`, "inputTypes") = c('_p_std__vectorT_itk__simple__Image_t')
class(`delete_VectorOfImage`) = c("SWIGFunction", class('delete_VectorOfImage'))

# Start of accessor method for std::vector< itk::simple::Image >
setMethod('$', '_p_std__vectorT_itk__simple__Image_t', function(x, name)

{
  accessorFuns = list('__nonzero__' = VectorOfImage___nonzero__, '__len__' = VectorOfImage___len__, 'pop' = VectorOfImage_pop, '__getslice__' = VectorOfImage___getslice__, '__setslice__' = VectorOfImage___setslice__, '__delslice__' = VectorOfImage___delslice__, '__delitem__' = VectorOfImage___delitem__, '__getitem__' = VectorOfImage___getitem__, '__setitem__' = VectorOfImage___setitem__, 'append' = VectorOfImage_append, 'empty' = VectorOfImage_empty, 'size' = VectorOfImage_size, 'swap' = VectorOfImage_swap, 'clear' = VectorOfImage_clear, 'get_allocator' = VectorOfImage_get_allocator, 'pop_back' = VectorOfImage_pop_back, 'resize' = VectorOfImage_resize, 'push_back' = VectorOfImage_push_back, 'front' = VectorOfImage_front, 'back' = VectorOfImage_back, 'assign' = VectorOfImage_assign, 'reserve' = VectorOfImage_reserve, 'capacity' = VectorOfImage_capacity);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::vector< itk::simple::Image >
# Start of VectorOfTransform___nonzero__

`VectorOfTransform___nonzero__` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorOfTransform___nonzero__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform___nonzero__`, 'returnType') = 'logical'
attr(`VectorOfTransform___nonzero__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t')
class(`VectorOfTransform___nonzero__`) = c("SWIGFunction", class('VectorOfTransform___nonzero__'))

# Start of VectorOfTransform___len__

`VectorOfTransform___len__` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorOfTransform___len__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform___len__`, 'returnType') = 'integer'
attr(`VectorOfTransform___len__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t')
class(`VectorOfTransform___len__`) = c("SWIGFunction", class('VectorOfTransform___len__'))

# Start of VectorOfTransform_pop

`VectorOfTransform_pop` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorOfTransform_pop', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_itk__simple__Transform", ref=ans);
  
  ans
  
}

attr(`VectorOfTransform_pop`, 'returnType') = '_p_itk__simple__Transform'
attr(`VectorOfTransform_pop`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t')
class(`VectorOfTransform_pop`) = c("SWIGFunction", class('VectorOfTransform_pop'))

# Start of VectorOfTransform___getslice__

`VectorOfTransform___getslice__` = function(self, i, j)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;ans = .Call('R_swig_VectorOfTransform___getslice__', self, i, j, PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__vectorT_itk__simple__Transform_t", ref=ans);
  
  ans
  
}

attr(`VectorOfTransform___getslice__`, 'returnType') = '_p_std__vectorT_itk__simple__Transform_t'
attr(`VectorOfTransform___getslice__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t', 'integer', 'integer')
class(`VectorOfTransform___getslice__`) = c("SWIGFunction", class('VectorOfTransform___getslice__'))

# Start of VectorOfTransform___setslice__

`VectorOfTransform___setslice__` = function(self, i, j, is)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  if (inherits(is, "ExternalReference")) is = slot(is,"ref"); 
  ;.Call('R_swig_VectorOfTransform___setslice__', self, i, j, is, PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform___setslice__`, 'returnType') = 'void'
attr(`VectorOfTransform___setslice__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t', 'integer', 'integer', '_p_std__vectorT_itk__simple__Transform_t')
class(`VectorOfTransform___setslice__`) = c("SWIGFunction", class('VectorOfTransform___setslice__'))

# Start of VectorOfTransform___delslice__

`VectorOfTransform___delslice__` = function(self, i, j)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorOfTransform___delslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform___delslice__`, 'returnType') = 'void'
attr(`VectorOfTransform___delslice__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t', 'integer', 'integer')
class(`VectorOfTransform___delslice__`) = c("SWIGFunction", class('VectorOfTransform___delslice__'))

# Start of VectorOfTransform___delitem__

`VectorOfTransform___delitem__` = function(self, i)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorOfTransform___delitem__', self, i, PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform___delitem__`, 'returnType') = 'void'
attr(`VectorOfTransform___delitem__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t', 'integer')
class(`VectorOfTransform___delitem__`) = c("SWIGFunction", class('VectorOfTransform___delitem__'))

# Start of VectorOfTransform___getitem__

`VectorOfTransform___getitem__` = function(self, i, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;ans = .Call('R_swig_VectorOfTransform___getitem__', self, i, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_itk__simple__Transform", ref=ans);
  
  ans
  
}

attr(`VectorOfTransform___getitem__`, 'returnType') = '_p_itk__simple__Transform'
attr(`VectorOfTransform___getitem__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t', 'integer')
class(`VectorOfTransform___getitem__`) = c("SWIGFunction", class('VectorOfTransform___getitem__'))

# Start of VectorOfTransform___setitem__

`VectorOfTransform___setitem__` = function(self, i, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorOfTransform___setitem__', self, i, x, PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform___setitem__`, 'returnType') = 'void'
attr(`VectorOfTransform___setitem__`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t', 'integer', '_p_itk__simple__Transform')
class(`VectorOfTransform___setitem__`) = c("SWIGFunction", class('VectorOfTransform___setitem__'))

# Start of VectorOfTransform_append

`VectorOfTransform_append` = function(self, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorOfTransform_append', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform_append`, 'returnType') = 'void'
attr(`VectorOfTransform_append`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t', '_p_itk__simple__Transform')
class(`VectorOfTransform_append`) = c("SWIGFunction", class('VectorOfTransform_append'))

# Start of new_VectorOfTransform

`VectorOfTransform__SWIG_0` = function()
{
  ;ans = .Call('R_swig_new_VectorOfTransform__SWIG_0', PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__vectorT_itk__simple__Transform_t", ref=ans);
  
  reg.finalizer(ans@ref, delete_VectorOfTransform);
  ans
  
}

attr(`VectorOfTransform__SWIG_0`, 'returnType') = '_p_std__vectorT_itk__simple__Transform_t'
class(`VectorOfTransform__SWIG_0`) = c("SWIGFunction", class('VectorOfTransform__SWIG_0'))

# Start of new_VectorOfTransform

`VectorOfTransform__SWIG_1` = function(other)
{
  if (inherits(other, "ExternalReference")) other = slot(other,"ref"); 
  ;ans = .Call('R_swig_new_VectorOfTransform__SWIG_1', other, PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__vectorT_itk__simple__Transform_t", ref=ans);
  
  reg.finalizer(ans@ref, delete_VectorOfTransform);
  ans
  
}

attr(`VectorOfTransform__SWIG_1`, 'returnType') = '_p_std__vectorT_itk__simple__Transform_t'
attr(`VectorOfTransform__SWIG_1`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t')
class(`VectorOfTransform__SWIG_1`) = c("SWIGFunction", class('VectorOfTransform__SWIG_1'))

# Start of VectorOfTransform_empty

`VectorOfTransform_empty` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorOfTransform_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform_empty`, 'returnType') = 'logical'
attr(`VectorOfTransform_empty`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t')
class(`VectorOfTransform_empty`) = c("SWIGFunction", class('VectorOfTransform_empty'))

# Start of VectorOfTransform_size

`VectorOfTransform_size` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorOfTransform_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform_size`, 'returnType') = 'integer'
attr(`VectorOfTransform_size`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t')
class(`VectorOfTransform_size`) = c("SWIGFunction", class('VectorOfTransform_size'))

# Start of VectorOfTransform_swap

`VectorOfTransform_swap` = function(self, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(v, "ExternalReference")) v = slot(v,"ref"); 
  ;.Call('R_swig_VectorOfTransform_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform_swap`, 'returnType') = 'void'
attr(`VectorOfTransform_swap`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t', '_p_std__vectorT_itk__simple__Transform_t')
class(`VectorOfTransform_swap`) = c("SWIGFunction", class('VectorOfTransform_swap'))

# Start of VectorOfTransform_clear

`VectorOfTransform_clear` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorOfTransform_clear', self, PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform_clear`, 'returnType') = 'void'
attr(`VectorOfTransform_clear`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t')
class(`VectorOfTransform_clear`) = c("SWIGFunction", class('VectorOfTransform_clear'))

# Start of VectorOfTransform_get_allocator

`VectorOfTransform_get_allocator` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorOfTransform_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_itk__simple__Transform_t", ref=ans);
  
  ans
  
}

attr(`VectorOfTransform_get_allocator`, 'returnType') = '_p_std__allocatorT_itk__simple__Transform_t'
attr(`VectorOfTransform_get_allocator`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t')
class(`VectorOfTransform_get_allocator`) = c("SWIGFunction", class('VectorOfTransform_get_allocator'))

# Start of new_VectorOfTransform

`VectorOfTransform__SWIG_2` = function(size)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  ;ans = .Call('R_swig_new_VectorOfTransform__SWIG_2', size, PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__vectorT_itk__simple__Transform_t", ref=ans);
  
  reg.finalizer(ans@ref, delete_VectorOfTransform);
  ans
  
}

attr(`VectorOfTransform__SWIG_2`, 'returnType') = '_p_std__vectorT_itk__simple__Transform_t'
attr(`VectorOfTransform__SWIG_2`, "inputTypes") = c('integer')
class(`VectorOfTransform__SWIG_2`) = c("SWIGFunction", class('VectorOfTransform__SWIG_2'))

# Start of VectorOfTransform_pop_back

`VectorOfTransform_pop_back` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorOfTransform_pop_back', self, PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform_pop_back`, 'returnType') = 'void'
attr(`VectorOfTransform_pop_back`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t')
class(`VectorOfTransform_pop_back`) = c("SWIGFunction", class('VectorOfTransform_pop_back'))

# Start of VectorOfTransform_resize

`VectorOfTransform_resize__SWIG_0` = function(self, new_size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  ;.Call('R_swig_VectorOfTransform_resize__SWIG_0', self, new_size, PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform_resize__SWIG_0`, 'returnType') = 'void'
attr(`VectorOfTransform_resize__SWIG_0`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t', 'integer')
class(`VectorOfTransform_resize__SWIG_0`) = c("SWIGFunction", class('VectorOfTransform_resize__SWIG_0'))

# Start of new_VectorOfTransform

`VectorOfTransform__SWIG_3` = function(size, value)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  if (inherits(value, "ExternalReference")) value = slot(value,"ref"); 
  ;ans = .Call('R_swig_new_VectorOfTransform__SWIG_3', size, value, PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__vectorT_itk__simple__Transform_t", ref=ans);
  
  reg.finalizer(ans@ref, delete_VectorOfTransform);
  ans
  
}

attr(`VectorOfTransform__SWIG_3`, 'returnType') = '_p_std__vectorT_itk__simple__Transform_t'
attr(`VectorOfTransform__SWIG_3`, "inputTypes") = c('integer', '_p_itk__simple__Transform')
class(`VectorOfTransform__SWIG_3`) = c("SWIGFunction", class('VectorOfTransform__SWIG_3'))

`VectorOfTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 0) {
    f <- VectorOfTransform__SWIG_0; 
  } else if (argc == 1) {
    if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- VectorOfTransform__SWIG_2; 
    }
    else if ( extends(argtypes[1], '_p_std__vectorT_itk__simple__Transform_t') && length(argv[[1]]) == 1 ) {
      f <- VectorOfTransform__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Transform') && length(argv[[2]]) == 1 )) {
      f <- VectorOfTransform__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorOfTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorOfTransform_push_back

`VectorOfTransform_push_back` = function(self, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorOfTransform_push_back', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform_push_back`, 'returnType') = 'void'
attr(`VectorOfTransform_push_back`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t', '_p_itk__simple__Transform')
class(`VectorOfTransform_push_back`) = c("SWIGFunction", class('VectorOfTransform_push_back'))

# Start of VectorOfTransform_front

`VectorOfTransform_front` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorOfTransform_front', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_itk__simple__Transform", ref=ans);
  
  ans
  
}

attr(`VectorOfTransform_front`, 'returnType') = '_p_itk__simple__Transform'
attr(`VectorOfTransform_front`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t')
class(`VectorOfTransform_front`) = c("SWIGFunction", class('VectorOfTransform_front'))

# Start of VectorOfTransform_back

`VectorOfTransform_back` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorOfTransform_back', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_itk__simple__Transform", ref=ans);
  
  ans
  
}

attr(`VectorOfTransform_back`, 'returnType') = '_p_itk__simple__Transform'
attr(`VectorOfTransform_back`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t')
class(`VectorOfTransform_back`) = c("SWIGFunction", class('VectorOfTransform_back'))

# Start of VectorOfTransform_assign

`VectorOfTransform_assign` = function(self, n, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorOfTransform_assign', self, n, x, PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform_assign`, 'returnType') = 'void'
attr(`VectorOfTransform_assign`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t', 'integer', '_p_itk__simple__Transform')
class(`VectorOfTransform_assign`) = c("SWIGFunction", class('VectorOfTransform_assign'))

# Start of VectorOfTransform_resize

`VectorOfTransform_resize__SWIG_1` = function(self, new_size, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  if (inherits(x, "ExternalReference")) x = slot(x,"ref"); 
  ;.Call('R_swig_VectorOfTransform_resize__SWIG_1', self, new_size, x, PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform_resize__SWIG_1`, 'returnType') = 'void'
attr(`VectorOfTransform_resize__SWIG_1`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t', 'integer', '_p_itk__simple__Transform')
class(`VectorOfTransform_resize__SWIG_1`) = c("SWIGFunction", class('VectorOfTransform_resize__SWIG_1'))

`VectorOfTransform_resize` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_std__vectorT_itk__simple__Transform_t') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorOfTransform_resize__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_std__vectorT_itk__simple__Transform_t') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 )) {
      f <- VectorOfTransform_resize__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorOfTransform_resize with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorOfTransform_reserve

`VectorOfTransform_reserve` = function(self, n)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;.Call('R_swig_VectorOfTransform_reserve', self, n, PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform_reserve`, 'returnType') = 'void'
attr(`VectorOfTransform_reserve`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t', 'integer')
class(`VectorOfTransform_reserve`) = c("SWIGFunction", class('VectorOfTransform_reserve'))

# Start of VectorOfTransform_capacity

`VectorOfTransform_capacity` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorOfTransform_capacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorOfTransform_capacity`, 'returnType') = 'integer'
attr(`VectorOfTransform_capacity`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t')
class(`VectorOfTransform_capacity`) = c("SWIGFunction", class('VectorOfTransform_capacity'))

# Start of delete_VectorOfTransform

`delete_VectorOfTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_delete_VectorOfTransform', self, PACKAGE='SimpleITK');
  
}

attr(`delete_VectorOfTransform`, 'returnType') = 'void'
attr(`delete_VectorOfTransform`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t')
class(`delete_VectorOfTransform`) = c("SWIGFunction", class('delete_VectorOfTransform'))

# Start of accessor method for std::vector< itk::simple::Transform >
setMethod('$', '_p_std__vectorT_itk__simple__Transform_t', function(x, name)

{
  accessorFuns = list('__nonzero__' = VectorOfTransform___nonzero__, '__len__' = VectorOfTransform___len__, 'pop' = VectorOfTransform_pop, '__getslice__' = VectorOfTransform___getslice__, '__setslice__' = VectorOfTransform___setslice__, '__delslice__' = VectorOfTransform___delslice__, '__delitem__' = VectorOfTransform___delitem__, '__getitem__' = VectorOfTransform___getitem__, '__setitem__' = VectorOfTransform___setitem__, 'append' = VectorOfTransform_append, 'empty' = VectorOfTransform_empty, 'size' = VectorOfTransform_size, 'swap' = VectorOfTransform_swap, 'clear' = VectorOfTransform_clear, 'get_allocator' = VectorOfTransform_get_allocator, 'pop_back' = VectorOfTransform_pop_back, 'resize' = VectorOfTransform_resize, 'push_back' = VectorOfTransform_push_back, 'front' = VectorOfTransform_front, 'back' = VectorOfTransform_back, 'assign' = VectorOfTransform_assign, 'reserve' = VectorOfTransform_reserve, 'capacity' = VectorOfTransform_capacity);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::vector< itk::simple::Transform >
# Start of VectorUIntList___nonzero__

`VectorUIntList___nonzero__` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUIntList___nonzero__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList___nonzero__`, 'returnType') = 'logical'
attr(`VectorUIntList___nonzero__`, "inputTypes") = c('list')
class(`VectorUIntList___nonzero__`) = c("SWIGFunction", class('VectorUIntList___nonzero__'))

# Start of VectorUIntList___len__

`VectorUIntList___len__` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUIntList___len__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList___len__`, 'returnType') = 'integer'
attr(`VectorUIntList___len__`, "inputTypes") = c('list')
class(`VectorUIntList___len__`) = c("SWIGFunction", class('VectorUIntList___len__'))

# Start of VectorUIntList_pop

`VectorUIntList_pop` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUIntList_pop', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList_pop`, 'returnType') = 'integer'
attr(`VectorUIntList_pop`, "inputTypes") = c('list')
class(`VectorUIntList_pop`) = c("SWIGFunction", class('VectorUIntList_pop'))

# Start of VectorUIntList___getslice__

`VectorUIntList___getslice__` = function(self, i, j)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorUIntList___getslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList___getslice__`, 'returnType') = 'list'
attr(`VectorUIntList___getslice__`, "inputTypes") = c('list', 'integer', 'integer')
class(`VectorUIntList___getslice__`) = c("SWIGFunction", class('VectorUIntList___getslice__'))

# Start of VectorUIntList___setslice__

`VectorUIntList___setslice__` = function(self, i, j, is)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  is = lapply(is, as.integer);
  ;.Call('R_swig_VectorUIntList___setslice__', self, i, j, is, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList___setslice__`, 'returnType') = 'void'
attr(`VectorUIntList___setslice__`, "inputTypes") = c('list', 'integer', 'integer', 'list')
class(`VectorUIntList___setslice__`) = c("SWIGFunction", class('VectorUIntList___setslice__'))

# Start of VectorUIntList___delslice__

`VectorUIntList___delslice__` = function(self, i, j)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorUIntList___delslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList___delslice__`, 'returnType') = 'void'
attr(`VectorUIntList___delslice__`, "inputTypes") = c('list', 'integer', 'integer')
class(`VectorUIntList___delslice__`) = c("SWIGFunction", class('VectorUIntList___delslice__'))

# Start of VectorUIntList___delitem__

`VectorUIntList___delitem__` = function(self, i)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorUIntList___delitem__', self, i, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList___delitem__`, 'returnType') = 'void'
attr(`VectorUIntList___delitem__`, "inputTypes") = c('list', 'integer')
class(`VectorUIntList___delitem__`) = c("SWIGFunction", class('VectorUIntList___delitem__'))

# Start of VectorUIntList___getitem__

`VectorUIntList___getitem__` = function(self, i, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorUIntList___getitem__', self, i, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList___getitem__`, 'returnType') = 'integer'
attr(`VectorUIntList___getitem__`, "inputTypes") = c('list', 'integer')
class(`VectorUIntList___getitem__`) = c("SWIGFunction", class('VectorUIntList___getitem__'))

# Start of VectorUIntList___setitem__

`VectorUIntList___setitem__` = function(self, i, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  x = as.integer(x);
  ;.Call('R_swig_VectorUIntList___setitem__', self, i, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList___setitem__`, 'returnType') = 'void'
attr(`VectorUIntList___setitem__`, "inputTypes") = c('list', 'integer', 'integer')
class(`VectorUIntList___setitem__`) = c("SWIGFunction", class('VectorUIntList___setitem__'))

# Start of VectorUIntList_append

`VectorUIntList_append` = function(self, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  x = as.integer(x);
  ;.Call('R_swig_VectorUIntList_append', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList_append`, 'returnType') = 'void'
attr(`VectorUIntList_append`, "inputTypes") = c('list', 'integer')
class(`VectorUIntList_append`) = c("SWIGFunction", class('VectorUIntList_append'))

# Start of new_VectorUIntList

`VectorUIntList__SWIG_0` = function()
{
  ;.Call('R_swig_new_VectorUIntList__SWIG_0', PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList__SWIG_0`, 'returnType') = 'list'
class(`VectorUIntList__SWIG_0`) = c("SWIGFunction", class('VectorUIntList__SWIG_0'))

# Start of new_VectorUIntList

`VectorUIntList__SWIG_1` = function(other)
{
  other = lapply(other, as.integer);
  ;.Call('R_swig_new_VectorUIntList__SWIG_1', other, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList__SWIG_1`, 'returnType') = 'list'
attr(`VectorUIntList__SWIG_1`, "inputTypes") = c('list')
class(`VectorUIntList__SWIG_1`) = c("SWIGFunction", class('VectorUIntList__SWIG_1'))

# Start of VectorUIntList_empty

`VectorUIntList_empty` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUIntList_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList_empty`, 'returnType') = 'logical'
attr(`VectorUIntList_empty`, "inputTypes") = c('list')
class(`VectorUIntList_empty`) = c("SWIGFunction", class('VectorUIntList_empty'))

# Start of VectorUIntList_size

`VectorUIntList_size` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUIntList_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList_size`, 'returnType') = 'integer'
attr(`VectorUIntList_size`, "inputTypes") = c('list')
class(`VectorUIntList_size`) = c("SWIGFunction", class('VectorUIntList_size'))

# Start of VectorUIntList_swap

`VectorUIntList_swap` = function(self, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  v = lapply(v, as.integer);
  ;.Call('R_swig_VectorUIntList_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList_swap`, 'returnType') = 'void'
attr(`VectorUIntList_swap`, "inputTypes") = c('list', 'list')
class(`VectorUIntList_swap`) = c("SWIGFunction", class('VectorUIntList_swap'))

# Start of VectorUIntList_clear

`VectorUIntList_clear` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUIntList_clear', self, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList_clear`, 'returnType') = 'void'
attr(`VectorUIntList_clear`, "inputTypes") = c('list')
class(`VectorUIntList_clear`) = c("SWIGFunction", class('VectorUIntList_clear'))

# Start of VectorUIntList_get_allocator

`VectorUIntList_get_allocator` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorUIntList_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_std__vectorT_uint32_t_t_t", ref=ans);
  
  ans
  
}

attr(`VectorUIntList_get_allocator`, 'returnType') = '_p_std__allocatorT_std__vectorT_uint32_t_t_t'
attr(`VectorUIntList_get_allocator`, "inputTypes") = c('list')
class(`VectorUIntList_get_allocator`) = c("SWIGFunction", class('VectorUIntList_get_allocator'))

# Start of new_VectorUIntList

`VectorUIntList__SWIG_2` = function(size)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  ;.Call('R_swig_new_VectorUIntList__SWIG_2', size, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList__SWIG_2`, 'returnType') = 'list'
attr(`VectorUIntList__SWIG_2`, "inputTypes") = c('integer')
class(`VectorUIntList__SWIG_2`) = c("SWIGFunction", class('VectorUIntList__SWIG_2'))

# Start of VectorUIntList_pop_back

`VectorUIntList_pop_back` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUIntList_pop_back', self, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList_pop_back`, 'returnType') = 'void'
attr(`VectorUIntList_pop_back`, "inputTypes") = c('list')
class(`VectorUIntList_pop_back`) = c("SWIGFunction", class('VectorUIntList_pop_back'))

# Start of VectorUIntList_resize

`VectorUIntList_resize__SWIG_0` = function(self, new_size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  ;.Call('R_swig_VectorUIntList_resize__SWIG_0', self, new_size, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList_resize__SWIG_0`, 'returnType') = 'void'
attr(`VectorUIntList_resize__SWIG_0`, "inputTypes") = c('list', 'integer')
class(`VectorUIntList_resize__SWIG_0`) = c("SWIGFunction", class('VectorUIntList_resize__SWIG_0'))

# Start of new_VectorUIntList

`VectorUIntList__SWIG_3` = function(size, value)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  value = as.integer(value);
  ;.Call('R_swig_new_VectorUIntList__SWIG_3', size, value, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList__SWIG_3`, 'returnType') = 'list'
attr(`VectorUIntList__SWIG_3`, "inputTypes") = c('integer', 'integer')
class(`VectorUIntList__SWIG_3`) = c("SWIGFunction", class('VectorUIntList__SWIG_3'))

`VectorUIntList` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 0) {
    f <- VectorUIntList__SWIG_0; 
  } else if (argc == 1) {
    if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- VectorUIntList__SWIG_2; 
    }
    else if ( is.list(argv[[1]]) && all(sapply(argv[[1]] , is.integer) || sapply(argv[[1]], is.numeric)) ) {
      f <- VectorUIntList__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- VectorUIntList__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorUIntList with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorUIntList_push_back

`VectorUIntList_push_back` = function(self, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  x = as.integer(x);
  ;.Call('R_swig_VectorUIntList_push_back', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList_push_back`, 'returnType') = 'void'
attr(`VectorUIntList_push_back`, "inputTypes") = c('list', 'integer')
class(`VectorUIntList_push_back`) = c("SWIGFunction", class('VectorUIntList_push_back'))

# Start of VectorUIntList_front

`VectorUIntList_front` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUIntList_front', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList_front`, 'returnType') = 'integer'
attr(`VectorUIntList_front`, "inputTypes") = c('list')
class(`VectorUIntList_front`) = c("SWIGFunction", class('VectorUIntList_front'))

# Start of VectorUIntList_back

`VectorUIntList_back` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUIntList_back', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList_back`, 'returnType') = 'integer'
attr(`VectorUIntList_back`, "inputTypes") = c('list')
class(`VectorUIntList_back`) = c("SWIGFunction", class('VectorUIntList_back'))

# Start of VectorUIntList_assign

`VectorUIntList_assign` = function(self, n, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  x = as.integer(x);
  ;.Call('R_swig_VectorUIntList_assign', self, n, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList_assign`, 'returnType') = 'void'
attr(`VectorUIntList_assign`, "inputTypes") = c('list', 'integer', 'integer')
class(`VectorUIntList_assign`) = c("SWIGFunction", class('VectorUIntList_assign'))

# Start of VectorUIntList_resize

`VectorUIntList_resize__SWIG_1` = function(self, new_size, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  x = as.integer(x);
  ;.Call('R_swig_VectorUIntList_resize__SWIG_1', self, new_size, x, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList_resize__SWIG_1`, 'returnType') = 'void'
attr(`VectorUIntList_resize__SWIG_1`, "inputTypes") = c('list', 'integer', 'integer')
class(`VectorUIntList_resize__SWIG_1`) = c("SWIGFunction", class('VectorUIntList_resize__SWIG_1'))

`VectorUIntList_resize` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( is.list(argv[[1]]) && all(sapply(argv[[1]] , is.integer) || sapply(argv[[1]], is.numeric)) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorUIntList_resize__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( is.list(argv[[1]]) && all(sapply(argv[[1]] , is.integer) || sapply(argv[[1]], is.numeric)) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- VectorUIntList_resize__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorUIntList_resize with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorUIntList_reserve

`VectorUIntList_reserve` = function(self, n)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;.Call('R_swig_VectorUIntList_reserve', self, n, PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList_reserve`, 'returnType') = 'void'
attr(`VectorUIntList_reserve`, "inputTypes") = c('list', 'integer')
class(`VectorUIntList_reserve`) = c("SWIGFunction", class('VectorUIntList_reserve'))

# Start of VectorUIntList_capacity

`VectorUIntList_capacity` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorUIntList_capacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorUIntList_capacity`, 'returnType') = 'integer'
attr(`VectorUIntList_capacity`, "inputTypes") = c('list')
class(`VectorUIntList_capacity`) = c("SWIGFunction", class('VectorUIntList_capacity'))

# Start of delete_VectorUIntList

`delete_VectorUIntList` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_delete_VectorUIntList', self, PACKAGE='SimpleITK');
  
}

attr(`delete_VectorUIntList`, 'returnType') = 'void'
attr(`delete_VectorUIntList`, "inputTypes") = c('list')
class(`delete_VectorUIntList`) = c("SWIGFunction", class('delete_VectorUIntList'))

# Start of accessor method for std::vector< std::vector< unsigned int > >
setMethod('$', '_p_std__vectorT_std__vectorT_uint32_t_t_std__allocatorT_std__vectorT_uint32_t_t_t_t', function(x, name)

{
  accessorFuns = list('__nonzero__' = VectorUIntList___nonzero__, '__len__' = VectorUIntList___len__, 'pop' = VectorUIntList_pop, '__getslice__' = VectorUIntList___getslice__, '__setslice__' = VectorUIntList___setslice__, '__delslice__' = VectorUIntList___delslice__, '__delitem__' = VectorUIntList___delitem__, '__getitem__' = VectorUIntList___getitem__, '__setitem__' = VectorUIntList___setitem__, 'append' = VectorUIntList_append, 'empty' = VectorUIntList_empty, 'size' = VectorUIntList_size, 'swap' = VectorUIntList_swap, 'clear' = VectorUIntList_clear, 'get_allocator' = VectorUIntList_get_allocator, 'pop_back' = VectorUIntList_pop_back, 'resize' = VectorUIntList_resize, 'push_back' = VectorUIntList_push_back, 'front' = VectorUIntList_front, 'back' = VectorUIntList_back, 'assign' = VectorUIntList_assign, 'reserve' = VectorUIntList_reserve, 'capacity' = VectorUIntList_capacity);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::vector< std::vector< unsigned int > >
# Start of VectorString___nonzero__

`VectorString___nonzero__` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorString___nonzero__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorString___nonzero__`, 'returnType') = 'logical'
attr(`VectorString___nonzero__`, "inputTypes") = c('_p_std__vectorT_std__string_t')
class(`VectorString___nonzero__`) = c("SWIGFunction", class('VectorString___nonzero__'))

# Start of VectorString___len__

`VectorString___len__` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorString___len__', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorString___len__`, 'returnType') = 'integer'
attr(`VectorString___len__`, "inputTypes") = c('_p_std__vectorT_std__string_t')
class(`VectorString___len__`) = c("SWIGFunction", class('VectorString___len__'))

# Start of VectorString_pop

`VectorString_pop` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorString_pop', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorString_pop`, 'returnType') = 'character'
attr(`VectorString_pop`, "inputTypes") = c('_p_std__vectorT_std__string_t')
class(`VectorString_pop`) = c("SWIGFunction", class('VectorString_pop'))

# Start of VectorString___getslice__

`VectorString___getslice__` = function(self, i, j)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;ans = .Call('R_swig_VectorString___getslice__', self, i, j, PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__vectorT_std__string_t", ref=ans);
  
  ans
  
}

attr(`VectorString___getslice__`, 'returnType') = '_p_std__vectorT_std__string_t'
attr(`VectorString___getslice__`, "inputTypes") = c('_p_std__vectorT_std__string_t', 'integer', 'integer')
class(`VectorString___getslice__`) = c("SWIGFunction", class('VectorString___getslice__'))

# Start of VectorString___setslice__

`VectorString___setslice__` = function(self, i, j, is)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  if (inherits(is, "ExternalReference")) is = slot(is,"ref"); 
  ;.Call('R_swig_VectorString___setslice__', self, i, j, is, PACKAGE='SimpleITK');
  
}

attr(`VectorString___setslice__`, 'returnType') = 'void'
attr(`VectorString___setslice__`, "inputTypes") = c('_p_std__vectorT_std__string_t', 'integer', 'integer', '_p_std__vectorT_std__string_t')
class(`VectorString___setslice__`) = c("SWIGFunction", class('VectorString___setslice__'))

# Start of VectorString___delslice__

`VectorString___delslice__` = function(self, i, j)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  j = as.integer(j);
  
  if(length(j) > 1) {
    warning("using only the first element of j");
  };
  
  ;.Call('R_swig_VectorString___delslice__', self, i, j, PACKAGE='SimpleITK');
  
}

attr(`VectorString___delslice__`, 'returnType') = 'void'
attr(`VectorString___delslice__`, "inputTypes") = c('_p_std__vectorT_std__string_t', 'integer', 'integer')
class(`VectorString___delslice__`) = c("SWIGFunction", class('VectorString___delslice__'))

# Start of VectorString___delitem__

`VectorString___delitem__` = function(self, i)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorString___delitem__', self, i, PACKAGE='SimpleITK');
  
}

attr(`VectorString___delitem__`, 'returnType') = 'void'
attr(`VectorString___delitem__`, "inputTypes") = c('_p_std__vectorT_std__string_t', 'integer')
class(`VectorString___delitem__`) = c("SWIGFunction", class('VectorString___delitem__'))

# Start of VectorString___getitem__

`VectorString___getitem__` = function(self, i, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  ;.Call('R_swig_VectorString___getitem__', self, i, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorString___getitem__`, 'returnType') = 'character'
attr(`VectorString___getitem__`, "inputTypes") = c('_p_std__vectorT_std__string_t', 'integer')
class(`VectorString___getitem__`) = c("SWIGFunction", class('VectorString___getitem__'))

# Start of VectorString___setitem__

`VectorString___setitem__` = function(self, i, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  i = as.integer(i);
  
  if(length(i) > 1) {
    warning("using only the first element of i");
  };
  
  x = as(x, "character"); 
  ;.Call('R_swig_VectorString___setitem__', self, i, x, PACKAGE='SimpleITK');
  
}

attr(`VectorString___setitem__`, 'returnType') = 'void'
attr(`VectorString___setitem__`, "inputTypes") = c('_p_std__vectorT_std__string_t', 'integer', 'character')
class(`VectorString___setitem__`) = c("SWIGFunction", class('VectorString___setitem__'))

# Start of VectorString_append

`VectorString_append` = function(self, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  x = as(x, "character"); 
  ;.Call('R_swig_VectorString_append', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorString_append`, 'returnType') = 'void'
attr(`VectorString_append`, "inputTypes") = c('_p_std__vectorT_std__string_t', 'character')
class(`VectorString_append`) = c("SWIGFunction", class('VectorString_append'))

# Start of new_VectorString

`VectorString__SWIG_0` = function()
{
  ;ans = .Call('R_swig_new_VectorString__SWIG_0', PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__vectorT_std__string_t", ref=ans);
  
  reg.finalizer(ans@ref, delete_VectorString);
  ans
  
}

attr(`VectorString__SWIG_0`, 'returnType') = '_p_std__vectorT_std__string_t'
class(`VectorString__SWIG_0`) = c("SWIGFunction", class('VectorString__SWIG_0'))

# Start of new_VectorString

`VectorString__SWIG_1` = function(other)
{
  if (inherits(other, "ExternalReference")) other = slot(other,"ref"); 
  ;ans = .Call('R_swig_new_VectorString__SWIG_1', other, PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__vectorT_std__string_t", ref=ans);
  
  reg.finalizer(ans@ref, delete_VectorString);
  ans
  
}

attr(`VectorString__SWIG_1`, 'returnType') = '_p_std__vectorT_std__string_t'
attr(`VectorString__SWIG_1`, "inputTypes") = c('_p_std__vectorT_std__string_t')
class(`VectorString__SWIG_1`) = c("SWIGFunction", class('VectorString__SWIG_1'))

# Start of VectorString_empty

`VectorString_empty` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorString_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorString_empty`, 'returnType') = 'logical'
attr(`VectorString_empty`, "inputTypes") = c('_p_std__vectorT_std__string_t')
class(`VectorString_empty`) = c("SWIGFunction", class('VectorString_empty'))

# Start of VectorString_size

`VectorString_size` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorString_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorString_size`, 'returnType') = 'integer'
attr(`VectorString_size`, "inputTypes") = c('_p_std__vectorT_std__string_t')
class(`VectorString_size`) = c("SWIGFunction", class('VectorString_size'))

# Start of VectorString_swap

`VectorString_swap` = function(self, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(v, "ExternalReference")) v = slot(v,"ref"); 
  ;.Call('R_swig_VectorString_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`VectorString_swap`, 'returnType') = 'void'
attr(`VectorString_swap`, "inputTypes") = c('_p_std__vectorT_std__string_t', '_p_std__vectorT_std__string_t')
class(`VectorString_swap`) = c("SWIGFunction", class('VectorString_swap'))

# Start of VectorString_clear

`VectorString_clear` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorString_clear', self, PACKAGE='SimpleITK');
  
}

attr(`VectorString_clear`, 'returnType') = 'void'
attr(`VectorString_clear`, "inputTypes") = c('_p_std__vectorT_std__string_t')
class(`VectorString_clear`) = c("SWIGFunction", class('VectorString_clear'))

# Start of VectorString_get_allocator

`VectorString_get_allocator` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorString_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_std__string_t", ref=ans);
  
  ans
  
}

attr(`VectorString_get_allocator`, 'returnType') = '_p_std__allocatorT_std__string_t'
attr(`VectorString_get_allocator`, "inputTypes") = c('_p_std__vectorT_std__string_t')
class(`VectorString_get_allocator`) = c("SWIGFunction", class('VectorString_get_allocator'))

# Start of new_VectorString

`VectorString__SWIG_2` = function(size)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  ;ans = .Call('R_swig_new_VectorString__SWIG_2', size, PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__vectorT_std__string_t", ref=ans);
  
  reg.finalizer(ans@ref, delete_VectorString);
  ans
  
}

attr(`VectorString__SWIG_2`, 'returnType') = '_p_std__vectorT_std__string_t'
attr(`VectorString__SWIG_2`, "inputTypes") = c('integer')
class(`VectorString__SWIG_2`) = c("SWIGFunction", class('VectorString__SWIG_2'))

# Start of VectorString_pop_back

`VectorString_pop_back` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorString_pop_back', self, PACKAGE='SimpleITK');
  
}

attr(`VectorString_pop_back`, 'returnType') = 'void'
attr(`VectorString_pop_back`, "inputTypes") = c('_p_std__vectorT_std__string_t')
class(`VectorString_pop_back`) = c("SWIGFunction", class('VectorString_pop_back'))

# Start of VectorString_resize

`VectorString_resize__SWIG_0` = function(self, new_size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  ;.Call('R_swig_VectorString_resize__SWIG_0', self, new_size, PACKAGE='SimpleITK');
  
}

attr(`VectorString_resize__SWIG_0`, 'returnType') = 'void'
attr(`VectorString_resize__SWIG_0`, "inputTypes") = c('_p_std__vectorT_std__string_t', 'integer')
class(`VectorString_resize__SWIG_0`) = c("SWIGFunction", class('VectorString_resize__SWIG_0'))

# Start of new_VectorString

`VectorString__SWIG_3` = function(size, value)
{
  size = as.integer(size);
  
  if(length(size) > 1) {
    warning("using only the first element of size");
  };
  
  value = as(value, "character"); 
  ;ans = .Call('R_swig_new_VectorString__SWIG_3', size, value, PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__vectorT_std__string_t", ref=ans);
  
  reg.finalizer(ans@ref, delete_VectorString);
  ans
  
}

attr(`VectorString__SWIG_3`, 'returnType') = '_p_std__vectorT_std__string_t'
attr(`VectorString__SWIG_3`, "inputTypes") = c('integer', 'character')
class(`VectorString__SWIG_3`) = c("SWIGFunction", class('VectorString__SWIG_3'))

`VectorString` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 0) {
    f <- VectorString__SWIG_0; 
  } else if (argc == 1) {
    if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- VectorString__SWIG_2; 
    }
    else if ( extends(argtypes[1], '_p_std__vectorT_std__string_t') && length(argv[[1]]) == 1 ) {
      f <- VectorString__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- VectorString__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorString with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorString_push_back

`VectorString_push_back` = function(self, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  x = as(x, "character"); 
  ;.Call('R_swig_VectorString_push_back', self, x, PACKAGE='SimpleITK');
  
}

attr(`VectorString_push_back`, 'returnType') = 'void'
attr(`VectorString_push_back`, "inputTypes") = c('_p_std__vectorT_std__string_t', 'character')
class(`VectorString_push_back`) = c("SWIGFunction", class('VectorString_push_back'))

# Start of VectorString_front

`VectorString_front` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorString_front', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorString_front`, 'returnType') = 'character'
attr(`VectorString_front`, "inputTypes") = c('_p_std__vectorT_std__string_t')
class(`VectorString_front`) = c("SWIGFunction", class('VectorString_front'))

# Start of VectorString_back

`VectorString_back` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorString_back', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorString_back`, 'returnType') = 'character'
attr(`VectorString_back`, "inputTypes") = c('_p_std__vectorT_std__string_t')
class(`VectorString_back`) = c("SWIGFunction", class('VectorString_back'))

# Start of VectorString_assign

`VectorString_assign` = function(self, n, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  x = as(x, "character"); 
  ;.Call('R_swig_VectorString_assign', self, n, x, PACKAGE='SimpleITK');
  
}

attr(`VectorString_assign`, 'returnType') = 'void'
attr(`VectorString_assign`, "inputTypes") = c('_p_std__vectorT_std__string_t', 'integer', 'character')
class(`VectorString_assign`) = c("SWIGFunction", class('VectorString_assign'))

# Start of VectorString_resize

`VectorString_resize__SWIG_1` = function(self, new_size, x)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  new_size = as.integer(new_size);
  
  if(length(new_size) > 1) {
    warning("using only the first element of new_size");
  };
  
  x = as(x, "character"); 
  ;.Call('R_swig_VectorString_resize__SWIG_1', self, new_size, x, PACKAGE='SimpleITK');
  
}

attr(`VectorString_resize__SWIG_1`, 'returnType') = 'void'
attr(`VectorString_resize__SWIG_1`, "inputTypes") = c('_p_std__vectorT_std__string_t', 'integer', 'character')
class(`VectorString_resize__SWIG_1`) = c("SWIGFunction", class('VectorString_resize__SWIG_1'))

`VectorString_resize` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_std__vectorT_std__string_t') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorString_resize__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_std__vectorT_std__string_t') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- VectorString_resize__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorString_resize with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VectorString_reserve

`VectorString_reserve` = function(self, n)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;.Call('R_swig_VectorString_reserve', self, n, PACKAGE='SimpleITK');
  
}

attr(`VectorString_reserve`, 'returnType') = 'void'
attr(`VectorString_reserve`, "inputTypes") = c('_p_std__vectorT_std__string_t', 'integer')
class(`VectorString_reserve`) = c("SWIGFunction", class('VectorString_reserve'))

# Start of VectorString_capacity

`VectorString_capacity` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorString_capacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorString_capacity`, 'returnType') = 'integer'
attr(`VectorString_capacity`, "inputTypes") = c('_p_std__vectorT_std__string_t')
class(`VectorString_capacity`) = c("SWIGFunction", class('VectorString_capacity'))

# Start of delete_VectorString

`delete_VectorString` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_delete_VectorString', self, PACKAGE='SimpleITK');
  
}

attr(`delete_VectorString`, 'returnType') = 'void'
attr(`delete_VectorString`, "inputTypes") = c('_p_std__vectorT_std__string_t')
class(`delete_VectorString`) = c("SWIGFunction", class('delete_VectorString'))

# Start of accessor method for std::vector< std::string >
setMethod('$', '_p_std__vectorT_std__string_t', function(x, name)

{
  accessorFuns = list('__nonzero__' = VectorString___nonzero__, '__len__' = VectorString___len__, 'pop' = VectorString_pop, '__getslice__' = VectorString___getslice__, '__setslice__' = VectorString___setslice__, '__delslice__' = VectorString___delslice__, '__delitem__' = VectorString___delitem__, '__getitem__' = VectorString___getitem__, '__setitem__' = VectorString___setitem__, 'append' = VectorString_append, 'empty' = VectorString_empty, 'size' = VectorString_size, 'swap' = VectorString_swap, 'clear' = VectorString_clear, 'get_allocator' = VectorString_get_allocator, 'pop_back' = VectorString_pop_back, 'resize' = VectorString_resize, 'push_back' = VectorString_push_back, 'front' = VectorString_front, 'back' = VectorString_back, 'assign' = VectorString_assign, 'reserve' = VectorString_reserve, 'capacity' = VectorString_capacity);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::vector< std::string >
# Start of new_DoubleDoubleMap

`DoubleDoubleMap__SWIG_0` = function(other)
{
  if (inherits(other, "ExternalReference")) other = slot(other,"ref"); 
  ;ans = .Call('R_swig_new_DoubleDoubleMap__SWIG_0', other, PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__mapT_double_double_t", ref=ans);
  
  reg.finalizer(ans@ref, delete_DoubleDoubleMap);
  ans
  
}

attr(`DoubleDoubleMap__SWIG_0`, 'returnType') = '_p_std__mapT_double_double_t'
attr(`DoubleDoubleMap__SWIG_0`, "inputTypes") = c('_p_std__lessT_double_t')
class(`DoubleDoubleMap__SWIG_0`) = c("SWIGFunction", class('DoubleDoubleMap__SWIG_0'))

# Start of new_DoubleDoubleMap

`DoubleDoubleMap__SWIG_1` = function()
{
  ;ans = .Call('R_swig_new_DoubleDoubleMap__SWIG_1', PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__mapT_double_double_t", ref=ans);
  
  reg.finalizer(ans@ref, delete_DoubleDoubleMap);
  ans
  
}

attr(`DoubleDoubleMap__SWIG_1`, 'returnType') = '_p_std__mapT_double_double_t'
class(`DoubleDoubleMap__SWIG_1`) = c("SWIGFunction", class('DoubleDoubleMap__SWIG_1'))

# Start of new_DoubleDoubleMap

`DoubleDoubleMap__SWIG_2` = function(other)
{
  if (inherits(other, "ExternalReference")) other = slot(other,"ref"); 
  ;ans = .Call('R_swig_new_DoubleDoubleMap__SWIG_2', other, PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__mapT_double_double_t", ref=ans);
  
  reg.finalizer(ans@ref, delete_DoubleDoubleMap);
  ans
  
}

attr(`DoubleDoubleMap__SWIG_2`, 'returnType') = '_p_std__mapT_double_double_t'
attr(`DoubleDoubleMap__SWIG_2`, "inputTypes") = c('_p_std__mapT_double_double_t')
class(`DoubleDoubleMap__SWIG_2`) = c("SWIGFunction", class('DoubleDoubleMap__SWIG_2'))

`DoubleDoubleMap` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 0) {
    f <- DoubleDoubleMap__SWIG_1; 
  } else if (argc == 1) {
    if ( extends(argtypes[1], '_p_std__lessT_double_t') && length(argv[[1]]) == 1 ) {
      f <- DoubleDoubleMap__SWIG_0; 
    }
    else if ( extends(argtypes[1], '_p_std__mapT_double_double_t') && length(argv[[1]]) == 1 ) {
      f <- DoubleDoubleMap__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DoubleDoubleMap with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of DoubleDoubleMap_empty

`DoubleDoubleMap_empty` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DoubleDoubleMap_empty', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DoubleDoubleMap_empty`, 'returnType') = 'logical'
attr(`DoubleDoubleMap_empty`, "inputTypes") = c('_p_std__mapT_double_double_t')
class(`DoubleDoubleMap_empty`) = c("SWIGFunction", class('DoubleDoubleMap_empty'))

# Start of DoubleDoubleMap_size

`DoubleDoubleMap_size` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DoubleDoubleMap_size', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DoubleDoubleMap_size`, 'returnType') = 'integer'
attr(`DoubleDoubleMap_size`, "inputTypes") = c('_p_std__mapT_double_double_t')
class(`DoubleDoubleMap_size`) = c("SWIGFunction", class('DoubleDoubleMap_size'))

# Start of DoubleDoubleMap_swap

`DoubleDoubleMap_swap` = function(self, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(v, "ExternalReference")) v = slot(v,"ref"); 
  ;.Call('R_swig_DoubleDoubleMap_swap', self, v, PACKAGE='SimpleITK');
  
}

attr(`DoubleDoubleMap_swap`, 'returnType') = 'void'
attr(`DoubleDoubleMap_swap`, "inputTypes") = c('_p_std__mapT_double_double_t', '_p_std__mapT_double_double_t')
class(`DoubleDoubleMap_swap`) = c("SWIGFunction", class('DoubleDoubleMap_swap'))

# Start of DoubleDoubleMap_clear

`DoubleDoubleMap_clear` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DoubleDoubleMap_clear', self, PACKAGE='SimpleITK');
  
}

attr(`DoubleDoubleMap_clear`, 'returnType') = 'void'
attr(`DoubleDoubleMap_clear`, "inputTypes") = c('_p_std__mapT_double_double_t')
class(`DoubleDoubleMap_clear`) = c("SWIGFunction", class('DoubleDoubleMap_clear'))

# Start of DoubleDoubleMap_get_allocator

`DoubleDoubleMap_get_allocator` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DoubleDoubleMap_get_allocator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans <- if (is.null(ans)) ans
  else new("_p_std__allocatorT_std__pairT_double_const_double_t_t", ref=ans);
  
  ans
  
}

attr(`DoubleDoubleMap_get_allocator`, 'returnType') = '_p_std__allocatorT_std__pairT_double_const_double_t_t'
attr(`DoubleDoubleMap_get_allocator`, "inputTypes") = c('_p_std__mapT_double_double_t')
class(`DoubleDoubleMap_get_allocator`) = c("SWIGFunction", class('DoubleDoubleMap_get_allocator'))

# Start of DoubleDoubleMap_erase

`DoubleDoubleMap_erase` = function(self, x, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;.Call('R_swig_DoubleDoubleMap_erase', self, x, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DoubleDoubleMap_erase`, 'returnType') = 'integer'
attr(`DoubleDoubleMap_erase`, "inputTypes") = c('_p_std__mapT_double_double_t', 'numeric')
class(`DoubleDoubleMap_erase`) = c("SWIGFunction", class('DoubleDoubleMap_erase'))

# Start of DoubleDoubleMap_count

`DoubleDoubleMap_count` = function(self, x, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;.Call('R_swig_DoubleDoubleMap_count', self, x, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DoubleDoubleMap_count`, 'returnType') = 'integer'
attr(`DoubleDoubleMap_count`, "inputTypes") = c('_p_std__mapT_double_double_t', 'numeric')
class(`DoubleDoubleMap_count`) = c("SWIGFunction", class('DoubleDoubleMap_count'))

# Start of delete_DoubleDoubleMap

`delete_DoubleDoubleMap` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_delete_DoubleDoubleMap', self, PACKAGE='SimpleITK');
  
}

attr(`delete_DoubleDoubleMap`, 'returnType') = 'void'
attr(`delete_DoubleDoubleMap`, "inputTypes") = c('_p_std__mapT_double_double_t')
class(`delete_DoubleDoubleMap`) = c("SWIGFunction", class('delete_DoubleDoubleMap'))

# Start of accessor method for std::map< double,double >
setMethod('$', '_p_std__mapT_double_double_t', function(x, name)

{
  accessorFuns = list('empty' = DoubleDoubleMap_empty, 'size' = DoubleDoubleMap_size, 'swap' = DoubleDoubleMap_swap, 'clear' = DoubleDoubleMap_clear, 'get_allocator' = DoubleDoubleMap_get_allocator, 'erase' = DoubleDoubleMap_erase, 'count' = DoubleDoubleMap_count);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for std::map< double,double >
# Start of ImAsArray

`ImAsArray` = function(src, .copy = FALSE)
{
  if (inherits(src, "ExternalReference")) src = slot(src,"ref"); 
  ;.Call('R_swig_ImAsArray', src, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImAsArray`, 'returnType') = 'numeric'
attr(`ImAsArray`, "inputTypes") = c('_p_itk__simple__Image')
class(`ImAsArray`) = c("SWIGFunction", class('ImAsArray'))

# Start of ArrayAsIm

`ArrayAsIm` = function(arr, size, spacing, origin, numberOfComponents, .copy = FALSE)
{
  if (inherits(arr, "ExternalReference")) arr = slot(arr,"ref"); 
  size = as.integer(size);
  spacing = as.numeric(spacing);
  origin = as.numeric(origin);
  numberOfComponents = as.integer(numberOfComponents);
  
  if(length(numberOfComponents) > 1) {
    warning("using only the first element of numberOfComponents");
  };
  
  ;ans = .Call('R_swig_ArrayAsIm', arr, size, spacing, origin, numberOfComponents, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ArrayAsIm`, 'returnType') = '_p_itk__simple__Image'
attr(`ArrayAsIm`, "inputTypes") = c('numeric', 'integer', 'numeric', 'numeric', 'integer')
class(`ArrayAsIm`) = c("SWIGFunction", class('ArrayAsIm'))



  setMethod('show', '_p_itk__simple__Image', function(object) Show(object))
  setMethod('print', '_p_itk__simple__Image', function(x, ...)cat(x$ToString()))

  setMethod('print', 'C++Reference', function(x, ...)cat(x$ToString()))
  setMethod('show', 'C++Reference', function(object)cat(object$ToString()))

ImportSwigVec <- function(ImVec)
{
  # an import function for unsupported std::vector types
  # intended for vectors of sitk images
  l <- ImVec$'__len__'()
  return(lapply(1:l, function(idx, V)return(V$'__getitem__'(idx-1))))
}

ImListToVec <- function(ImList)
{
  l <- length(ImList)
  cppvec <- VectorOfImage()
  for (idx in 1:l)
    {
      cppvec$push_back(ImList[[idx]])
    }
   return(cppvec)
}


# Start of Version_MajorVersion

`Version_MajorVersion` = function(.copy = FALSE)
{
  ;.Call('R_swig_Version_MajorVersion', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Version_MajorVersion`, 'returnType') = 'integer'
class(`Version_MajorVersion`) = c("SWIGFunction", class('Version_MajorVersion'))

# Start of Version_MinorVersion

`Version_MinorVersion` = function(.copy = FALSE)
{
  ;.Call('R_swig_Version_MinorVersion', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Version_MinorVersion`, 'returnType') = 'integer'
class(`Version_MinorVersion`) = c("SWIGFunction", class('Version_MinorVersion'))

# Start of Version_PatchVersion

`Version_PatchVersion` = function(.copy = FALSE)
{
  ;.Call('R_swig_Version_PatchVersion', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Version_PatchVersion`, 'returnType') = 'integer'
class(`Version_PatchVersion`) = c("SWIGFunction", class('Version_PatchVersion'))

# Start of Version_TweakVersion

`Version_TweakVersion` = function(.copy = FALSE)
{
  ;.Call('R_swig_Version_TweakVersion', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Version_TweakVersion`, 'returnType') = 'integer'
class(`Version_TweakVersion`) = c("SWIGFunction", class('Version_TweakVersion'))

# Start of Version_VersionString

`Version_VersionString` = function(.copy = FALSE)
{
  ;.Call('R_swig_Version_VersionString', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Version_VersionString`, 'returnType') = 'character'
class(`Version_VersionString`) = c("SWIGFunction", class('Version_VersionString'))

# Start of Version_BuildDate

`Version_BuildDate` = function(.copy = FALSE)
{
  ;.Call('R_swig_Version_BuildDate', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Version_BuildDate`, 'returnType') = 'character'
class(`Version_BuildDate`) = c("SWIGFunction", class('Version_BuildDate'))

# Start of Version_ITKMajorVersion

`Version_ITKMajorVersion` = function(.copy = FALSE)
{
  ;.Call('R_swig_Version_ITKMajorVersion', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Version_ITKMajorVersion`, 'returnType') = 'integer'
class(`Version_ITKMajorVersion`) = c("SWIGFunction", class('Version_ITKMajorVersion'))

# Start of Version_ITKMinorVersion

`Version_ITKMinorVersion` = function(.copy = FALSE)
{
  ;.Call('R_swig_Version_ITKMinorVersion', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Version_ITKMinorVersion`, 'returnType') = 'integer'
class(`Version_ITKMinorVersion`) = c("SWIGFunction", class('Version_ITKMinorVersion'))

# Start of Version_ITKPatchVersion

`Version_ITKPatchVersion` = function(.copy = FALSE)
{
  ;.Call('R_swig_Version_ITKPatchVersion', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Version_ITKPatchVersion`, 'returnType') = 'integer'
class(`Version_ITKPatchVersion`) = c("SWIGFunction", class('Version_ITKPatchVersion'))

# Start of Version_ITKVersionString

`Version_ITKVersionString` = function(.copy = FALSE)
{
  ;.Call('R_swig_Version_ITKVersionString', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Version_ITKVersionString`, 'returnType') = 'character'
class(`Version_ITKVersionString`) = c("SWIGFunction", class('Version_ITKVersionString'))

# Start of Version_ITKModulesEnabled

`Version_ITKModulesEnabled` = function(.copy = FALSE)
{
  ;.Call('R_swig_Version_ITKModulesEnabled', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Version_ITKModulesEnabled`, 'returnType') = 'character'
class(`Version_ITKModulesEnabled`) = c("SWIGFunction", class('Version_ITKModulesEnabled'))

# Start of Version_ExtendedVersionString

`Version_ExtendedVersionString` = function(.copy = FALSE)
{
  ;.Call('R_swig_Version_ExtendedVersionString', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Version_ExtendedVersionString`, 'returnType') = 'character'
class(`Version_ExtendedVersionString`) = c("SWIGFunction", class('Version_ExtendedVersionString'))

# Start of Version_ToString

`Version_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Version_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Version_ToString`, 'returnType') = 'character'
attr(`Version_ToString`, "inputTypes") = c('_p_itk__simple__Version')
class(`Version_ToString`) = c("SWIGFunction", class('Version_ToString'))

# Start of new_Version

`Version` = function()
{
  ;ans = .Call('R_swig_new_Version', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Version", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Version);
  ans
  
}

attr(`Version`, 'returnType') = '_p_itk__simple__Version'
class(`Version`) = c("SWIGFunction", class('Version'))

# Start of delete_Version

`delete_Version` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_Version', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_Version`, 'returnType') = 'void'
attr(`delete_Version`, "inputTypes") = c('_p_itk__simple__Version')
class(`delete_Version`) = c("SWIGFunction", class('delete_Version'))

# Start of accessor method for itk::simple::Version
setMethod('$', '_p_itk__simple__Version', function(x, name)

{
  accessorFuns = list('ToString' = Version_ToString);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::Version
# Start of PixelIDValueEnum_sitkUnknown_get

`PixelIDValueEnum_sitkUnknown_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkUnknown_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkUnknown_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkUnknown_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkUnknown_get'))

# Start of PixelIDValueEnum_sitkUInt8_get

`PixelIDValueEnum_sitkUInt8_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkUInt8_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkUInt8_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkUInt8_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkUInt8_get'))

# Start of PixelIDValueEnum_sitkInt8_get

`PixelIDValueEnum_sitkInt8_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkInt8_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkInt8_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkInt8_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkInt8_get'))

# Start of PixelIDValueEnum_sitkUInt16_get

`PixelIDValueEnum_sitkUInt16_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkUInt16_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkUInt16_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkUInt16_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkUInt16_get'))

# Start of PixelIDValueEnum_sitkInt16_get

`PixelIDValueEnum_sitkInt16_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkInt16_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkInt16_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkInt16_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkInt16_get'))

# Start of PixelIDValueEnum_sitkUInt32_get

`PixelIDValueEnum_sitkUInt32_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkUInt32_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkUInt32_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkUInt32_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkUInt32_get'))

# Start of PixelIDValueEnum_sitkInt32_get

`PixelIDValueEnum_sitkInt32_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkInt32_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkInt32_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkInt32_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkInt32_get'))

# Start of PixelIDValueEnum_sitkUInt64_get

`PixelIDValueEnum_sitkUInt64_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkUInt64_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkUInt64_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkUInt64_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkUInt64_get'))

# Start of PixelIDValueEnum_sitkInt64_get

`PixelIDValueEnum_sitkInt64_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkInt64_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkInt64_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkInt64_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkInt64_get'))

# Start of PixelIDValueEnum_sitkFloat32_get

`PixelIDValueEnum_sitkFloat32_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkFloat32_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkFloat32_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkFloat32_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkFloat32_get'))

# Start of PixelIDValueEnum_sitkFloat64_get

`PixelIDValueEnum_sitkFloat64_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkFloat64_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkFloat64_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkFloat64_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkFloat64_get'))

# Start of PixelIDValueEnum_sitkComplexFloat32_get

`PixelIDValueEnum_sitkComplexFloat32_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkComplexFloat32_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkComplexFloat32_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkComplexFloat32_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkComplexFloat32_get'))

# Start of PixelIDValueEnum_sitkComplexFloat64_get

`PixelIDValueEnum_sitkComplexFloat64_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkComplexFloat64_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkComplexFloat64_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkComplexFloat64_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkComplexFloat64_get'))

# Start of PixelIDValueEnum_sitkVectorUInt8_get

`PixelIDValueEnum_sitkVectorUInt8_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkVectorUInt8_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkVectorUInt8_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkVectorUInt8_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkVectorUInt8_get'))

# Start of PixelIDValueEnum_sitkVectorInt8_get

`PixelIDValueEnum_sitkVectorInt8_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkVectorInt8_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkVectorInt8_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkVectorInt8_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkVectorInt8_get'))

# Start of PixelIDValueEnum_sitkVectorUInt16_get

`PixelIDValueEnum_sitkVectorUInt16_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkVectorUInt16_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkVectorUInt16_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkVectorUInt16_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkVectorUInt16_get'))

# Start of PixelIDValueEnum_sitkVectorInt16_get

`PixelIDValueEnum_sitkVectorInt16_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkVectorInt16_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkVectorInt16_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkVectorInt16_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkVectorInt16_get'))

# Start of PixelIDValueEnum_sitkVectorUInt32_get

`PixelIDValueEnum_sitkVectorUInt32_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkVectorUInt32_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkVectorUInt32_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkVectorUInt32_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkVectorUInt32_get'))

# Start of PixelIDValueEnum_sitkVectorInt32_get

`PixelIDValueEnum_sitkVectorInt32_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkVectorInt32_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkVectorInt32_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkVectorInt32_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkVectorInt32_get'))

# Start of PixelIDValueEnum_sitkVectorUInt64_get

`PixelIDValueEnum_sitkVectorUInt64_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkVectorUInt64_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkVectorUInt64_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkVectorUInt64_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkVectorUInt64_get'))

# Start of PixelIDValueEnum_sitkVectorInt64_get

`PixelIDValueEnum_sitkVectorInt64_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkVectorInt64_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkVectorInt64_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkVectorInt64_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkVectorInt64_get'))

# Start of PixelIDValueEnum_sitkVectorFloat32_get

`PixelIDValueEnum_sitkVectorFloat32_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkVectorFloat32_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkVectorFloat32_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkVectorFloat32_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkVectorFloat32_get'))

# Start of PixelIDValueEnum_sitkVectorFloat64_get

`PixelIDValueEnum_sitkVectorFloat64_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkVectorFloat64_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkVectorFloat64_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkVectorFloat64_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkVectorFloat64_get'))

# Start of PixelIDValueEnum_sitkLabelUInt8_get

`PixelIDValueEnum_sitkLabelUInt8_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkLabelUInt8_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkLabelUInt8_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkLabelUInt8_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkLabelUInt8_get'))

# Start of PixelIDValueEnum_sitkLabelUInt16_get

`PixelIDValueEnum_sitkLabelUInt16_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkLabelUInt16_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkLabelUInt16_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkLabelUInt16_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkLabelUInt16_get'))

# Start of PixelIDValueEnum_sitkLabelUInt32_get

`PixelIDValueEnum_sitkLabelUInt32_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkLabelUInt32_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkLabelUInt32_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkLabelUInt32_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkLabelUInt32_get'))

# Start of PixelIDValueEnum_sitkLabelUInt64_get

`PixelIDValueEnum_sitkLabelUInt64_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PixelIDValueEnum_sitkLabelUInt64_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PixelIDValueEnum_sitkLabelUInt64_get`, 'returnType') = 'character'
class(`PixelIDValueEnum_sitkLabelUInt64_get`) = c("SWIGFunction", class('PixelIDValueEnum_sitkLabelUInt64_get'))

# Start of GetPixelIDValueAsString

`GetPixelIDValueAsString` = function(type, .copy = FALSE)
{
  type = enumToInteger(type, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(type) > 1) {
    warning("using only the first element of type");
  };
  
  ;.Call('R_swig_GetPixelIDValueAsString', type, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GetPixelIDValueAsString`, 'returnType') = 'character'
attr(`GetPixelIDValueAsString`, "inputTypes") = c('character')
class(`GetPixelIDValueAsString`) = c("SWIGFunction", class('GetPixelIDValueAsString'))

# Start of GetPixelIDValueFromString

`GetPixelIDValueFromString` = function(enumString, .copy = FALSE)
{
  enumString = as(enumString, "character"); 
  ;.Call('R_swig_GetPixelIDValueFromString', enumString, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GetPixelIDValueFromString`, 'returnType') = 'integer'
attr(`GetPixelIDValueFromString`, "inputTypes") = c('character')
class(`GetPixelIDValueFromString`) = c("SWIGFunction", class('GetPixelIDValueFromString'))

# Start of InterpolatorEnum_sitkNearestNeighbor_get

`InterpolatorEnum_sitkNearestNeighbor_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkNearestNeighbor_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkNearestNeighbor_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkNearestNeighbor_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkNearestNeighbor_get'))

# Start of InterpolatorEnum_sitkLinear_get

`InterpolatorEnum_sitkLinear_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkLinear_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkLinear_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkLinear_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkLinear_get'))

# Start of InterpolatorEnum_sitkBSpline1_get

`InterpolatorEnum_sitkBSpline1_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkBSpline1_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkBSpline1_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkBSpline1_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkBSpline1_get'))

# Start of InterpolatorEnum_sitkBSpline2_get

`InterpolatorEnum_sitkBSpline2_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkBSpline2_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkBSpline2_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkBSpline2_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkBSpline2_get'))

# Start of InterpolatorEnum_sitkBSpline_get

`InterpolatorEnum_sitkBSpline_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkBSpline_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkBSpline_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkBSpline_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkBSpline_get'))

# Start of InterpolatorEnum_sitkBSpline3_get

`InterpolatorEnum_sitkBSpline3_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkBSpline3_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkBSpline3_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkBSpline3_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkBSpline3_get'))

# Start of InterpolatorEnum_sitkBSpline4_get

`InterpolatorEnum_sitkBSpline4_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkBSpline4_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkBSpline4_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkBSpline4_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkBSpline4_get'))

# Start of InterpolatorEnum_sitkBSpline5_get

`InterpolatorEnum_sitkBSpline5_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkBSpline5_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkBSpline5_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkBSpline5_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkBSpline5_get'))

# Start of InterpolatorEnum_sitkGaussian_get

`InterpolatorEnum_sitkGaussian_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkGaussian_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkGaussian_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkGaussian_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkGaussian_get'))

# Start of InterpolatorEnum_sitkLabelGaussian_get

`InterpolatorEnum_sitkLabelGaussian_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkLabelGaussian_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkLabelGaussian_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkLabelGaussian_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkLabelGaussian_get'))

# Start of InterpolatorEnum_sitkLabelLinear_get

`InterpolatorEnum_sitkLabelLinear_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkLabelLinear_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkLabelLinear_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkLabelLinear_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkLabelLinear_get'))

# Start of InterpolatorEnum_sitkHammingWindowedSinc_get

`InterpolatorEnum_sitkHammingWindowedSinc_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkHammingWindowedSinc_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkHammingWindowedSinc_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkHammingWindowedSinc_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkHammingWindowedSinc_get'))

# Start of InterpolatorEnum_sitkCosineWindowedSinc_get

`InterpolatorEnum_sitkCosineWindowedSinc_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkCosineWindowedSinc_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkCosineWindowedSinc_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkCosineWindowedSinc_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkCosineWindowedSinc_get'))

# Start of InterpolatorEnum_sitkWelchWindowedSinc_get

`InterpolatorEnum_sitkWelchWindowedSinc_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkWelchWindowedSinc_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkWelchWindowedSinc_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkWelchWindowedSinc_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkWelchWindowedSinc_get'))

# Start of InterpolatorEnum_sitkLanczosWindowedSinc_get

`InterpolatorEnum_sitkLanczosWindowedSinc_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkLanczosWindowedSinc_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkLanczosWindowedSinc_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkLanczosWindowedSinc_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkLanczosWindowedSinc_get'))

# Start of InterpolatorEnum_sitkBlackmanWindowedSinc_get

`InterpolatorEnum_sitkBlackmanWindowedSinc_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkBlackmanWindowedSinc_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkBlackmanWindowedSinc_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkBlackmanWindowedSinc_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkBlackmanWindowedSinc_get'))

# Start of InterpolatorEnum_sitkBSplineResampler_get

`InterpolatorEnum_sitkBSplineResampler_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkBSplineResampler_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkBSplineResampler_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkBSplineResampler_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkBSplineResampler_get'))

# Start of InterpolatorEnum_sitkBSplineResamplerOrder3_get

`InterpolatorEnum_sitkBSplineResamplerOrder3_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkBSplineResamplerOrder3_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkBSplineResamplerOrder3_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkBSplineResamplerOrder3_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkBSplineResamplerOrder3_get'))

# Start of InterpolatorEnum_sitkBSplineResamplerOrder1_get

`InterpolatorEnum_sitkBSplineResamplerOrder1_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkBSplineResamplerOrder1_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkBSplineResamplerOrder1_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkBSplineResamplerOrder1_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkBSplineResamplerOrder1_get'))

# Start of InterpolatorEnum_sitkBSplineResamplerOrder2_get

`InterpolatorEnum_sitkBSplineResamplerOrder2_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkBSplineResamplerOrder2_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkBSplineResamplerOrder2_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkBSplineResamplerOrder2_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkBSplineResamplerOrder2_get'))

# Start of InterpolatorEnum_sitkBSplineResamplerOrder4_get

`InterpolatorEnum_sitkBSplineResamplerOrder4_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkBSplineResamplerOrder4_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkBSplineResamplerOrder4_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkBSplineResamplerOrder4_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkBSplineResamplerOrder4_get'))

# Start of InterpolatorEnum_sitkBSplineResamplerOrder5_get

`InterpolatorEnum_sitkBSplineResamplerOrder5_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InterpolatorEnum_sitkBSplineResamplerOrder5_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`InterpolatorEnum_sitkBSplineResamplerOrder5_get`, 'returnType') = 'character'
class(`InterpolatorEnum_sitkBSplineResamplerOrder5_get`) = c("SWIGFunction", class('InterpolatorEnum_sitkBSplineResamplerOrder5_get'))

# Start of delete_Image

`delete_Image` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_Image', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_Image`, 'returnType') = 'void'
attr(`delete_Image`, "inputTypes") = c('_p_itk__simple__Image')
class(`delete_Image`) = c("SWIGFunction", class('delete_Image'))

# Start of new_Image

`Image__SWIG_0` = function()
{
  ;ans = .Call('R_swig_new_Image__SWIG_0', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Image);
  ans
  
}

attr(`Image__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
class(`Image__SWIG_0`) = c("SWIGFunction", class('Image__SWIG_0'))

# Start of new_Image

`Image__SWIG_1` = function(img)
{
  if (inherits(img, "ExternalReference")) img = slot(img,"ref"); 
  ;ans = .Call('R_swig_new_Image__SWIG_1', img, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Image);
  ans
  
}

attr(`Image__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Image__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image__SWIG_1`) = c("SWIGFunction", class('Image__SWIG_1'))

# Start of Image_Equal

`Image_Equal` = function(self, img, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(img, "ExternalReference")) img = slot(img,"ref"); 
  ;ans = .Call('R_swig_Image_Equal', self, img, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Image_Equal`, 'returnType') = '_p_itk__simple__Image'
attr(`Image_Equal`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Image_Equal`) = c("SWIGFunction", class('Image_Equal'))

# Start of new_Image

`Image__SWIG_2` = function(width, height, valueEnum)
{
  width = as.integer(width);
  
  if(length(width) > 1) {
    warning("using only the first element of width");
  };
  
  height = as.integer(height);
  
  if(length(height) > 1) {
    warning("using only the first element of height");
  };
  
  valueEnum = enumToInteger(valueEnum, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(valueEnum) > 1) {
    warning("using only the first element of valueEnum");
  };
  
  ;ans = .Call('R_swig_new_Image__SWIG_2', width, height, valueEnum, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Image);
  ans
  
}

attr(`Image__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Image__SWIG_2`, "inputTypes") = c('integer', 'integer', 'character')
class(`Image__SWIG_2`) = c("SWIGFunction", class('Image__SWIG_2'))

# Start of new_Image

`Image__SWIG_3` = function(width, height, depth, valueEnum)
{
  width = as.integer(width);
  
  if(length(width) > 1) {
    warning("using only the first element of width");
  };
  
  height = as.integer(height);
  
  if(length(height) > 1) {
    warning("using only the first element of height");
  };
  
  depth = as.integer(depth);
  
  if(length(depth) > 1) {
    warning("using only the first element of depth");
  };
  
  valueEnum = enumToInteger(valueEnum, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(valueEnum) > 1) {
    warning("using only the first element of valueEnum");
  };
  
  ;ans = .Call('R_swig_new_Image__SWIG_3', width, height, depth, valueEnum, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Image);
  ans
  
}

attr(`Image__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Image__SWIG_3`, "inputTypes") = c('integer', 'integer', 'integer', 'character')
class(`Image__SWIG_3`) = c("SWIGFunction", class('Image__SWIG_3'))

# Start of new_Image

`Image__SWIG_4` = function(size, valueEnum, numberOfComponents)
{
  size = as.integer(size);
  valueEnum = enumToInteger(valueEnum, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(valueEnum) > 1) {
    warning("using only the first element of valueEnum");
  };
  
  numberOfComponents = as.integer(numberOfComponents);
  
  if(length(numberOfComponents) > 1) {
    warning("using only the first element of numberOfComponents");
  };
  
  ;ans = .Call('R_swig_new_Image__SWIG_4', size, valueEnum, numberOfComponents, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Image);
  ans
  
}

attr(`Image__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`Image__SWIG_4`, "inputTypes") = c('integer', 'character', 'integer')
class(`Image__SWIG_4`) = c("SWIGFunction", class('Image__SWIG_4'))

# Start of new_Image

`Image__SWIG_5` = function(size, valueEnum)
{
  size = as.integer(size);
  valueEnum = enumToInteger(valueEnum, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(valueEnum) > 1) {
    warning("using only the first element of valueEnum");
  };
  
  ;ans = .Call('R_swig_new_Image__SWIG_5', size, valueEnum, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Image);
  ans
  
}

attr(`Image__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`Image__SWIG_5`, "inputTypes") = c('integer', 'character')
class(`Image__SWIG_5`) = c("SWIGFunction", class('Image__SWIG_5'))

`Image` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 0) {
    f <- Image__SWIG_0; 
  } else if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Image__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Image__SWIG_5; 
    }
  } else if (argc == 3) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Image__SWIG_2; 
    }
    else if (( is.integer(argv[[1]]) || is.numeric(argv[[1]]) ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- Image__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- Image__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Image with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of Image_GetITKBase

`Image_GetITKBase__SWIG_0` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_Image_GetITKBase__SWIG_0', self, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__DataObject", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Image_GetITKBase__SWIG_0`, 'returnType') = '_p_itk__DataObject'
attr(`Image_GetITKBase__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetITKBase__SWIG_0`) = c("SWIGFunction", class('Image_GetITKBase__SWIG_0'))

# Start of Image_GetITKBase

`Image_GetITKBase__SWIG_1` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_Image_GetITKBase__SWIG_1', self, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__DataObject", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Image_GetITKBase__SWIG_1`, 'returnType') = '_p_itk__DataObject'
attr(`Image_GetITKBase__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetITKBase__SWIG_1`) = c("SWIGFunction", class('Image_GetITKBase__SWIG_1'))

`Image_GetITKBase` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') || is.null(argv[[1]]) ) {
      f <- Image_GetITKBase__SWIG_0; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Image') || is.null(argv[[1]]) ) {
      f <- Image_GetITKBase__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Image_GetITKBase with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of Image_GetPixelID

`Image_GetPixelID` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_Image_GetPixelID', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`Image_GetPixelID`, 'returnType') = 'character'
attr(`Image_GetPixelID`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetPixelID`) = c("SWIGFunction", class('Image_GetPixelID'))

# Start of Image_GetPixelIDValue

`Image_GetPixelIDValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_GetPixelIDValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelIDValue`, 'returnType') = 'integer'
attr(`Image_GetPixelIDValue`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetPixelIDValue`) = c("SWIGFunction", class('Image_GetPixelIDValue'))

# Start of Image_GetPixelIDTypeAsString

`Image_GetPixelIDTypeAsString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_GetPixelIDTypeAsString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelIDTypeAsString`, 'returnType') = 'character'
attr(`Image_GetPixelIDTypeAsString`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetPixelIDTypeAsString`) = c("SWIGFunction", class('Image_GetPixelIDTypeAsString'))

# Start of Image_GetDimension

`Image_GetDimension` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_GetDimension', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetDimension`, 'returnType') = 'integer'
attr(`Image_GetDimension`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetDimension`) = c("SWIGFunction", class('Image_GetDimension'))

# Start of Image_GetNumberOfComponentsPerPixel

`Image_GetNumberOfComponentsPerPixel` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_GetNumberOfComponentsPerPixel', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetNumberOfComponentsPerPixel`, 'returnType') = 'integer'
attr(`Image_GetNumberOfComponentsPerPixel`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetNumberOfComponentsPerPixel`) = c("SWIGFunction", class('Image_GetNumberOfComponentsPerPixel'))

# Start of Image_GetNumberOfPixels

`Image_GetNumberOfPixels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_GetNumberOfPixels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetNumberOfPixels`, 'returnType') = 'integer'
attr(`Image_GetNumberOfPixels`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetNumberOfPixels`) = c("SWIGFunction", class('Image_GetNumberOfPixels'))

# Start of Image_GetSizeOfPixelComponent

`Image_GetSizeOfPixelComponent` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_GetSizeOfPixelComponent', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetSizeOfPixelComponent`, 'returnType') = 'integer'
attr(`Image_GetSizeOfPixelComponent`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetSizeOfPixelComponent`) = c("SWIGFunction", class('Image_GetSizeOfPixelComponent'))

# Start of Image_GetOrigin

`Image_GetOrigin` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_GetOrigin', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetOrigin`, 'returnType') = 'numeric'
attr(`Image_GetOrigin`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetOrigin`) = c("SWIGFunction", class('Image_GetOrigin'))

# Start of Image_SetOrigin

`Image_SetOrigin` = function(self, origin)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  origin = as.numeric(origin);
  ;ans = .Call('R_swig_Image_SetOrigin', self, origin, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetOrigin`, 'returnType') = 'void'
attr(`Image_SetOrigin`, "inputTypes") = c('_p_itk__simple__Image', '_p_std__vectorT_double_t')
class(`Image_SetOrigin`) = c("SWIGFunction", class('Image_SetOrigin'))

# Start of Image_GetSpacing

`Image_GetSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_GetSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetSpacing`, 'returnType') = 'numeric'
attr(`Image_GetSpacing`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetSpacing`) = c("SWIGFunction", class('Image_GetSpacing'))

# Start of Image_SetSpacing

`Image_SetSpacing` = function(self, spacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  spacing = as.numeric(spacing);
  ;ans = .Call('R_swig_Image_SetSpacing', self, spacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetSpacing`, 'returnType') = 'void'
attr(`Image_SetSpacing`, "inputTypes") = c('_p_itk__simple__Image', '_p_std__vectorT_double_t')
class(`Image_SetSpacing`) = c("SWIGFunction", class('Image_SetSpacing'))

# Start of Image_GetDirection

`Image_GetDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_GetDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetDirection`, 'returnType') = 'numeric'
attr(`Image_GetDirection`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetDirection`) = c("SWIGFunction", class('Image_GetDirection'))

# Start of Image_SetDirection

`Image_SetDirection` = function(self, direction)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  direction = as.numeric(direction);
  ;ans = .Call('R_swig_Image_SetDirection', self, direction, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetDirection`, 'returnType') = 'void'
attr(`Image_SetDirection`, "inputTypes") = c('_p_itk__simple__Image', '_p_std__vectorT_double_t')
class(`Image_SetDirection`) = c("SWIGFunction", class('Image_SetDirection'))

# Start of Image_TransformIndexToPhysicalPoint

`Image_TransformIndexToPhysicalPoint` = function(self, index, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  index = as.integer(index);
  ;.Call('R_swig_Image_TransformIndexToPhysicalPoint', self, index, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_TransformIndexToPhysicalPoint`, 'returnType') = 'numeric'
attr(`Image_TransformIndexToPhysicalPoint`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_TransformIndexToPhysicalPoint`) = c("SWIGFunction", class('Image_TransformIndexToPhysicalPoint'))

# Start of Image_TransformPhysicalPointToIndex

`Image_TransformPhysicalPointToIndex` = function(self, point, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  point = as.numeric(point);
  ;.Call('R_swig_Image_TransformPhysicalPointToIndex', self, point, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_TransformPhysicalPointToIndex`, 'returnType') = 'integer'
attr(`Image_TransformPhysicalPointToIndex`, "inputTypes") = c('_p_itk__simple__Image', '_p_std__vectorT_double_t')
class(`Image_TransformPhysicalPointToIndex`) = c("SWIGFunction", class('Image_TransformPhysicalPointToIndex'))

# Start of Image_TransformPhysicalPointToContinuousIndex

`Image_TransformPhysicalPointToContinuousIndex` = function(self, point, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  point = as.numeric(point);
  ;.Call('R_swig_Image_TransformPhysicalPointToContinuousIndex', self, point, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_TransformPhysicalPointToContinuousIndex`, 'returnType') = 'numeric'
attr(`Image_TransformPhysicalPointToContinuousIndex`, "inputTypes") = c('_p_itk__simple__Image', '_p_std__vectorT_double_t')
class(`Image_TransformPhysicalPointToContinuousIndex`) = c("SWIGFunction", class('Image_TransformPhysicalPointToContinuousIndex'))

# Start of Image_TransformContinuousIndexToPhysicalPoint

`Image_TransformContinuousIndexToPhysicalPoint` = function(self, index, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  index = as.numeric(index);
  ;.Call('R_swig_Image_TransformContinuousIndexToPhysicalPoint', self, index, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_TransformContinuousIndexToPhysicalPoint`, 'returnType') = 'numeric'
attr(`Image_TransformContinuousIndexToPhysicalPoint`, "inputTypes") = c('_p_itk__simple__Image', '_p_std__vectorT_double_t')
class(`Image_TransformContinuousIndexToPhysicalPoint`) = c("SWIGFunction", class('Image_TransformContinuousIndexToPhysicalPoint'))

# Start of Image_EvaluateAtContinuousIndex

`Image_EvaluateAtContinuousIndex__SWIG_0` = function(self, index, interp, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  index = as.numeric(index);
  interp = enumToInteger(interp, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interp) > 1) {
    warning("using only the first element of interp");
  };
  
  ;.Call('R_swig_Image_EvaluateAtContinuousIndex__SWIG_0', self, index, interp, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_EvaluateAtContinuousIndex__SWIG_0`, 'returnType') = 'numeric'
attr(`Image_EvaluateAtContinuousIndex__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_std__vectorT_double_t', 'character')
class(`Image_EvaluateAtContinuousIndex__SWIG_0`) = c("SWIGFunction", class('Image_EvaluateAtContinuousIndex__SWIG_0'))

# Start of Image_EvaluateAtContinuousIndex

`Image_EvaluateAtContinuousIndex__SWIG_1` = function(self, index, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  index = as.numeric(index);
  ;.Call('R_swig_Image_EvaluateAtContinuousIndex__SWIG_1', self, index, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_EvaluateAtContinuousIndex__SWIG_1`, 'returnType') = 'numeric'
attr(`Image_EvaluateAtContinuousIndex__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_std__vectorT_double_t')
class(`Image_EvaluateAtContinuousIndex__SWIG_1`) = c("SWIGFunction", class('Image_EvaluateAtContinuousIndex__SWIG_1'))

`Image_EvaluateAtContinuousIndex` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- Image_EvaluateAtContinuousIndex__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Image_EvaluateAtContinuousIndex__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Image_EvaluateAtContinuousIndex with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of Image_EvaluateAtPhysicalPoint

`Image_EvaluateAtPhysicalPoint__SWIG_0` = function(self, point, interp, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  point = as.numeric(point);
  interp = enumToInteger(interp, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interp) > 1) {
    warning("using only the first element of interp");
  };
  
  ;.Call('R_swig_Image_EvaluateAtPhysicalPoint__SWIG_0', self, point, interp, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_EvaluateAtPhysicalPoint__SWIG_0`, 'returnType') = 'numeric'
attr(`Image_EvaluateAtPhysicalPoint__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_std__vectorT_double_t', 'character')
class(`Image_EvaluateAtPhysicalPoint__SWIG_0`) = c("SWIGFunction", class('Image_EvaluateAtPhysicalPoint__SWIG_0'))

# Start of Image_EvaluateAtPhysicalPoint

`Image_EvaluateAtPhysicalPoint__SWIG_1` = function(self, point, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  point = as.numeric(point);
  ;.Call('R_swig_Image_EvaluateAtPhysicalPoint__SWIG_1', self, point, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_EvaluateAtPhysicalPoint__SWIG_1`, 'returnType') = 'numeric'
attr(`Image_EvaluateAtPhysicalPoint__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_std__vectorT_double_t')
class(`Image_EvaluateAtPhysicalPoint__SWIG_1`) = c("SWIGFunction", class('Image_EvaluateAtPhysicalPoint__SWIG_1'))

`Image_EvaluateAtPhysicalPoint` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- Image_EvaluateAtPhysicalPoint__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Image_EvaluateAtPhysicalPoint__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Image_EvaluateAtPhysicalPoint with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of Image_IsCongruentImageGeometry

`Image_IsCongruentImageGeometry` = function(self, otherImage, coordinateTolerance, directionTolerance, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(otherImage, "ExternalReference")) otherImage = slot(otherImage,"ref"); 
  
  
  ;.Call('R_swig_Image_IsCongruentImageGeometry', self, otherImage, coordinateTolerance, directionTolerance, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_IsCongruentImageGeometry`, 'returnType') = 'logical'
attr(`Image_IsCongruentImageGeometry`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`Image_IsCongruentImageGeometry`) = c("SWIGFunction", class('Image_IsCongruentImageGeometry'))

# Start of Image_IsSameImageGeometryAs

`Image_IsSameImageGeometryAs__SWIG_0` = function(self, otherImage, s_arg3, s_arg4, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(otherImage, "ExternalReference")) otherImage = slot(otherImage,"ref"); 
  
  
  ;.Call('R_swig_Image_IsSameImageGeometryAs__SWIG_0', self, otherImage, s_arg3, s_arg4, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_IsSameImageGeometryAs__SWIG_0`, 'returnType') = 'logical'
attr(`Image_IsSameImageGeometryAs__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`Image_IsSameImageGeometryAs__SWIG_0`) = c("SWIGFunction", class('Image_IsSameImageGeometryAs__SWIG_0'))

# Start of Image_IsSameImageGeometryAs

`Image_IsSameImageGeometryAs__SWIG_1` = function(self, otherImage, s_arg3, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(otherImage, "ExternalReference")) otherImage = slot(otherImage,"ref"); 
  
  ;.Call('R_swig_Image_IsSameImageGeometryAs__SWIG_1', self, otherImage, s_arg3, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_IsSameImageGeometryAs__SWIG_1`, 'returnType') = 'logical'
attr(`Image_IsSameImageGeometryAs__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`Image_IsSameImageGeometryAs__SWIG_1`) = c("SWIGFunction", class('Image_IsSameImageGeometryAs__SWIG_1'))

# Start of Image_IsSameImageGeometryAs

`Image_IsSameImageGeometryAs__SWIG_2` = function(self, otherImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(otherImage, "ExternalReference")) otherImage = slot(otherImage,"ref"); 
  ;.Call('R_swig_Image_IsSameImageGeometryAs__SWIG_2', self, otherImage, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_IsSameImageGeometryAs__SWIG_2`, 'returnType') = 'logical'
attr(`Image_IsSameImageGeometryAs__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Image_IsSameImageGeometryAs__SWIG_2`) = c("SWIGFunction", class('Image_IsSameImageGeometryAs__SWIG_2'))

`Image_IsSameImageGeometryAs` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Image_IsSameImageGeometryAs__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Image_IsSameImageGeometryAs__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- Image_IsSameImageGeometryAs__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Image_IsSameImageGeometryAs with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of Image_GetSize

`Image_GetSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_GetSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetSize`, 'returnType') = 'integer'
attr(`Image_GetSize`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetSize`) = c("SWIGFunction", class('Image_GetSize'))

# Start of Image_GetWidth

`Image_GetWidth` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_GetWidth', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetWidth`, 'returnType') = 'integer'
attr(`Image_GetWidth`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetWidth`) = c("SWIGFunction", class('Image_GetWidth'))

# Start of Image_GetHeight

`Image_GetHeight` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_GetHeight', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetHeight`, 'returnType') = 'integer'
attr(`Image_GetHeight`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetHeight`) = c("SWIGFunction", class('Image_GetHeight'))

# Start of Image_GetDepth

`Image_GetDepth` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_GetDepth', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetDepth`, 'returnType') = 'integer'
attr(`Image_GetDepth`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetDepth`) = c("SWIGFunction", class('Image_GetDepth'))

# Start of Image_CopyInformation

`Image_CopyInformation` = function(self, srcImage)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(srcImage, "ExternalReference")) srcImage = slot(srcImage,"ref"); 
  ;ans = .Call('R_swig_Image_CopyInformation', self, srcImage, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_CopyInformation`, 'returnType') = 'void'
attr(`Image_CopyInformation`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Image_CopyInformation`) = c("SWIGFunction", class('Image_CopyInformation'))

# Start of Image_GetMetaDataKeys

`Image_GetMetaDataKeys` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_GetMetaDataKeys', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetMetaDataKeys`, 'returnType') = 'character'
attr(`Image_GetMetaDataKeys`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_GetMetaDataKeys`) = c("SWIGFunction", class('Image_GetMetaDataKeys'))

# Start of Image_HasMetaDataKey

`Image_HasMetaDataKey` = function(self, key, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  key = as(key, "character"); 
  ;.Call('R_swig_Image_HasMetaDataKey', self, key, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_HasMetaDataKey`, 'returnType') = 'logical'
attr(`Image_HasMetaDataKey`, "inputTypes") = c('_p_itk__simple__Image', 'character')
class(`Image_HasMetaDataKey`) = c("SWIGFunction", class('Image_HasMetaDataKey'))

# Start of Image_GetMetaData

`Image_GetMetaData` = function(self, key, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  key = as(key, "character"); 
  ;.Call('R_swig_Image_GetMetaData', self, key, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetMetaData`, 'returnType') = 'character'
attr(`Image_GetMetaData`, "inputTypes") = c('_p_itk__simple__Image', 'character')
class(`Image_GetMetaData`) = c("SWIGFunction", class('Image_GetMetaData'))

# Start of Image_SetMetaData

`Image_SetMetaData` = function(self, key, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  key = as(key, "character"); 
  value = as(value, "character"); 
  ;ans = .Call('R_swig_Image_SetMetaData', self, key, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetMetaData`, 'returnType') = 'void'
attr(`Image_SetMetaData`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'character')
class(`Image_SetMetaData`) = c("SWIGFunction", class('Image_SetMetaData'))

# Start of Image_EraseMetaData

`Image_EraseMetaData` = function(self, key, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  key = as(key, "character"); 
  ;.Call('R_swig_Image_EraseMetaData', self, key, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_EraseMetaData`, 'returnType') = 'logical'
attr(`Image_EraseMetaData`, "inputTypes") = c('_p_itk__simple__Image', 'character')
class(`Image_EraseMetaData`) = c("SWIGFunction", class('Image_EraseMetaData'))

# Start of Image_ToString

`Image_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_ToString`, 'returnType') = 'character'
attr(`Image_ToString`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_ToString`) = c("SWIGFunction", class('Image_ToString'))

# Start of Image_ToVectorImage

`Image_ToVectorImage__SWIG_0` = function(self, inPlace, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  inPlace = as.logical(inPlace);
  ;ans = .Call('R_swig_Image_ToVectorImage__SWIG_0', self, inPlace, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Image_ToVectorImage__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Image_ToVectorImage__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`Image_ToVectorImage__SWIG_0`) = c("SWIGFunction", class('Image_ToVectorImage__SWIG_0'))

# Start of Image_ToVectorImage

`Image_ToVectorImage__SWIG_1` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_Image_ToVectorImage__SWIG_1', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Image_ToVectorImage__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Image_ToVectorImage__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_ToVectorImage__SWIG_1`) = c("SWIGFunction", class('Image_ToVectorImage__SWIG_1'))

`Image_ToVectorImage` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') || is.null(argv[[1]]) ) {
      f <- Image_ToVectorImage__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') || is.null(argv[[1]]) ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Image_ToVectorImage__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Image_ToVectorImage with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of Image_ToScalarImage

`Image_ToScalarImage__SWIG_0` = function(self, inPlace, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  inPlace = as.logical(inPlace);
  ;ans = .Call('R_swig_Image_ToScalarImage__SWIG_0', self, inPlace, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Image_ToScalarImage__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Image_ToScalarImage__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`Image_ToScalarImage__SWIG_0`) = c("SWIGFunction", class('Image_ToScalarImage__SWIG_0'))

# Start of Image_ToScalarImage

`Image_ToScalarImage__SWIG_1` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_Image_ToScalarImage__SWIG_1', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Image_ToScalarImage__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Image_ToScalarImage__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_ToScalarImage__SWIG_1`) = c("SWIGFunction", class('Image_ToScalarImage__SWIG_1'))

`Image_ToScalarImage` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') || is.null(argv[[1]]) ) {
      f <- Image_ToScalarImage__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') || is.null(argv[[1]]) ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Image_ToScalarImage__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Image_ToScalarImage with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of Image_GetPixelAsInt8

`Image_GetPixelAsInt8` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsInt8', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsInt8`, 'returnType') = 'integer'
attr(`Image_GetPixelAsInt8`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsInt8`) = c("SWIGFunction", class('Image_GetPixelAsInt8'))

# Start of Image_GetPixelAsUInt8

`Image_GetPixelAsUInt8` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsUInt8', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsUInt8`, 'returnType') = 'integer'
attr(`Image_GetPixelAsUInt8`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsUInt8`) = c("SWIGFunction", class('Image_GetPixelAsUInt8'))

# Start of Image_GetPixelAsInt16

`Image_GetPixelAsInt16` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsInt16', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsInt16`, 'returnType') = 'integer'
attr(`Image_GetPixelAsInt16`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsInt16`) = c("SWIGFunction", class('Image_GetPixelAsInt16'))

# Start of Image_GetPixelAsUInt16

`Image_GetPixelAsUInt16` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsUInt16', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsUInt16`, 'returnType') = 'integer'
attr(`Image_GetPixelAsUInt16`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsUInt16`) = c("SWIGFunction", class('Image_GetPixelAsUInt16'))

# Start of Image_GetPixelAsInt32

`Image_GetPixelAsInt32` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsInt32', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsInt32`, 'returnType') = 'integer'
attr(`Image_GetPixelAsInt32`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsInt32`) = c("SWIGFunction", class('Image_GetPixelAsInt32'))

# Start of Image_GetPixelAsUInt32

`Image_GetPixelAsUInt32` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsUInt32', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsUInt32`, 'returnType') = 'integer'
attr(`Image_GetPixelAsUInt32`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsUInt32`) = c("SWIGFunction", class('Image_GetPixelAsUInt32'))

# Start of Image_GetPixelAsInt64

`Image_GetPixelAsInt64` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsInt64', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsInt64`, 'returnType') = 'integer'
attr(`Image_GetPixelAsInt64`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsInt64`) = c("SWIGFunction", class('Image_GetPixelAsInt64'))

# Start of Image_GetPixelAsUInt64

`Image_GetPixelAsUInt64` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsUInt64', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsUInt64`, 'returnType') = 'integer'
attr(`Image_GetPixelAsUInt64`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsUInt64`) = c("SWIGFunction", class('Image_GetPixelAsUInt64'))

# Start of Image_GetPixelAsFloat

`Image_GetPixelAsFloat` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsFloat', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsFloat`, 'returnType') = 'numeric'
attr(`Image_GetPixelAsFloat`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsFloat`) = c("SWIGFunction", class('Image_GetPixelAsFloat'))

# Start of Image_GetPixelAsDouble

`Image_GetPixelAsDouble` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsDouble', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsDouble`, 'returnType') = 'numeric'
attr(`Image_GetPixelAsDouble`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsDouble`) = c("SWIGFunction", class('Image_GetPixelAsDouble'))

# Start of Image_GetPixelAsVectorInt8

`Image_GetPixelAsVectorInt8` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsVectorInt8', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsVectorInt8`, 'returnType') = 'integer'
attr(`Image_GetPixelAsVectorInt8`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsVectorInt8`) = c("SWIGFunction", class('Image_GetPixelAsVectorInt8'))

# Start of Image_GetPixelAsVectorUInt8

`Image_GetPixelAsVectorUInt8` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsVectorUInt8', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsVectorUInt8`, 'returnType') = 'integer'
attr(`Image_GetPixelAsVectorUInt8`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsVectorUInt8`) = c("SWIGFunction", class('Image_GetPixelAsVectorUInt8'))

# Start of Image_GetPixelAsVectorInt16

`Image_GetPixelAsVectorInt16` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsVectorInt16', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsVectorInt16`, 'returnType') = 'integer'
attr(`Image_GetPixelAsVectorInt16`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsVectorInt16`) = c("SWIGFunction", class('Image_GetPixelAsVectorInt16'))

# Start of Image_GetPixelAsVectorUInt16

`Image_GetPixelAsVectorUInt16` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsVectorUInt16', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsVectorUInt16`, 'returnType') = '_p_std__vectorT_uint16_t_t'
attr(`Image_GetPixelAsVectorUInt16`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsVectorUInt16`) = c("SWIGFunction", class('Image_GetPixelAsVectorUInt16'))

# Start of Image_GetPixelAsVectorInt32

`Image_GetPixelAsVectorInt32` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsVectorInt32', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsVectorInt32`, 'returnType') = 'integer'
attr(`Image_GetPixelAsVectorInt32`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsVectorInt32`) = c("SWIGFunction", class('Image_GetPixelAsVectorInt32'))

# Start of Image_GetPixelAsVectorUInt32

`Image_GetPixelAsVectorUInt32` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsVectorUInt32', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsVectorUInt32`, 'returnType') = 'integer'
attr(`Image_GetPixelAsVectorUInt32`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsVectorUInt32`) = c("SWIGFunction", class('Image_GetPixelAsVectorUInt32'))

# Start of Image_GetPixelAsVectorInt64

`Image_GetPixelAsVectorInt64` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsVectorInt64', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsVectorInt64`, 'returnType') = 'integer'
attr(`Image_GetPixelAsVectorInt64`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsVectorInt64`) = c("SWIGFunction", class('Image_GetPixelAsVectorInt64'))

# Start of Image_GetPixelAsVectorUInt64

`Image_GetPixelAsVectorUInt64` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsVectorUInt64', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsVectorUInt64`, 'returnType') = 'integer'
attr(`Image_GetPixelAsVectorUInt64`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsVectorUInt64`) = c("SWIGFunction", class('Image_GetPixelAsVectorUInt64'))

# Start of Image_GetPixelAsVectorFloat32

`Image_GetPixelAsVectorFloat32` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsVectorFloat32', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsVectorFloat32`, 'returnType') = 'numeric'
attr(`Image_GetPixelAsVectorFloat32`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsVectorFloat32`) = c("SWIGFunction", class('Image_GetPixelAsVectorFloat32'))

# Start of Image_GetPixelAsVectorFloat64

`Image_GetPixelAsVectorFloat64` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixelAsVectorFloat64', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixelAsVectorFloat64`, 'returnType') = 'numeric'
attr(`Image_GetPixelAsVectorFloat64`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsVectorFloat64`) = c("SWIGFunction", class('Image_GetPixelAsVectorFloat64'))

# Start of Image_GetPixelAsComplexFloat32

`Image_GetPixelAsComplexFloat32` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;ans = .Call('R_swig_Image_GetPixelAsComplexFloat32', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_std__complexT_float_t", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Image_GetPixelAsComplexFloat32`, 'returnType') = '_p_std__complexT_float_t'
attr(`Image_GetPixelAsComplexFloat32`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsComplexFloat32`) = c("SWIGFunction", class('Image_GetPixelAsComplexFloat32'))

# Start of Image_GetPixelAsComplexFloat64

`Image_GetPixelAsComplexFloat64` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;ans = .Call('R_swig_Image_GetPixelAsComplexFloat64', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_std__complexT_double_t", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Image_GetPixelAsComplexFloat64`, 'returnType') = '_p_std__complexT_double_t'
attr(`Image_GetPixelAsComplexFloat64`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixelAsComplexFloat64`) = c("SWIGFunction", class('Image_GetPixelAsComplexFloat64'))

# Start of Image_SetPixelAsInt8

`Image_SetPixelAsInt8` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.integer(v);
  
  if(length(v) > 1) {
    warning("using only the first element of v");
  };
  
  ;ans = .Call('R_swig_Image_SetPixelAsInt8', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsInt8`, 'returnType') = 'void'
attr(`Image_SetPixelAsInt8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Image_SetPixelAsInt8`) = c("SWIGFunction", class('Image_SetPixelAsInt8'))

# Start of Image_SetPixelAsUInt8

`Image_SetPixelAsUInt8` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.integer(v);
  
  if(length(v) > 1) {
    warning("using only the first element of v");
  };
  
  ;ans = .Call('R_swig_Image_SetPixelAsUInt8', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsUInt8`, 'returnType') = 'void'
attr(`Image_SetPixelAsUInt8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Image_SetPixelAsUInt8`) = c("SWIGFunction", class('Image_SetPixelAsUInt8'))

# Start of Image_SetPixelAsInt16

`Image_SetPixelAsInt16` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.integer(v);
  
  if(length(v) > 1) {
    warning("using only the first element of v");
  };
  
  ;ans = .Call('R_swig_Image_SetPixelAsInt16', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsInt16`, 'returnType') = 'void'
attr(`Image_SetPixelAsInt16`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Image_SetPixelAsInt16`) = c("SWIGFunction", class('Image_SetPixelAsInt16'))

# Start of Image_SetPixelAsUInt16

`Image_SetPixelAsUInt16` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.integer(v);
  
  if(length(v) > 1) {
    warning("using only the first element of v");
  };
  
  ;ans = .Call('R_swig_Image_SetPixelAsUInt16', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsUInt16`, 'returnType') = 'void'
attr(`Image_SetPixelAsUInt16`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Image_SetPixelAsUInt16`) = c("SWIGFunction", class('Image_SetPixelAsUInt16'))

# Start of Image_SetPixelAsInt32

`Image_SetPixelAsInt32` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.integer(v);
  
  if(length(v) > 1) {
    warning("using only the first element of v");
  };
  
  ;ans = .Call('R_swig_Image_SetPixelAsInt32', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsInt32`, 'returnType') = 'void'
attr(`Image_SetPixelAsInt32`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Image_SetPixelAsInt32`) = c("SWIGFunction", class('Image_SetPixelAsInt32'))

# Start of Image_SetPixelAsUInt32

`Image_SetPixelAsUInt32` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.integer(v);
  
  if(length(v) > 1) {
    warning("using only the first element of v");
  };
  
  ;ans = .Call('R_swig_Image_SetPixelAsUInt32', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsUInt32`, 'returnType') = 'void'
attr(`Image_SetPixelAsUInt32`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Image_SetPixelAsUInt32`) = c("SWIGFunction", class('Image_SetPixelAsUInt32'))

# Start of Image_SetPixelAsInt64

`Image_SetPixelAsInt64` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.integer(v);
  
  if(length(v) > 1) {
    warning("using only the first element of v");
  };
  
  ;ans = .Call('R_swig_Image_SetPixelAsInt64', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsInt64`, 'returnType') = 'void'
attr(`Image_SetPixelAsInt64`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Image_SetPixelAsInt64`) = c("SWIGFunction", class('Image_SetPixelAsInt64'))

# Start of Image_SetPixelAsUInt64

`Image_SetPixelAsUInt64` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.integer(v);
  
  if(length(v) > 1) {
    warning("using only the first element of v");
  };
  
  ;ans = .Call('R_swig_Image_SetPixelAsUInt64', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsUInt64`, 'returnType') = 'void'
attr(`Image_SetPixelAsUInt64`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Image_SetPixelAsUInt64`) = c("SWIGFunction", class('Image_SetPixelAsUInt64'))

# Start of Image_SetPixelAsFloat

`Image_SetPixelAsFloat` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  
  ;ans = .Call('R_swig_Image_SetPixelAsFloat', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsFloat`, 'returnType') = 'void'
attr(`Image_SetPixelAsFloat`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric')
class(`Image_SetPixelAsFloat`) = c("SWIGFunction", class('Image_SetPixelAsFloat'))

# Start of Image_SetPixelAsDouble

`Image_SetPixelAsDouble` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  
  ;ans = .Call('R_swig_Image_SetPixelAsDouble', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsDouble`, 'returnType') = 'void'
attr(`Image_SetPixelAsDouble`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric')
class(`Image_SetPixelAsDouble`) = c("SWIGFunction", class('Image_SetPixelAsDouble'))

# Start of Image_SetPixelAsVectorInt8

`Image_SetPixelAsVectorInt8` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.integer(v);
  ;ans = .Call('R_swig_Image_SetPixelAsVectorInt8', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsVectorInt8`, 'returnType') = 'void'
attr(`Image_SetPixelAsVectorInt8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Image_SetPixelAsVectorInt8`) = c("SWIGFunction", class('Image_SetPixelAsVectorInt8'))

# Start of Image_SetPixelAsVectorUInt8

`Image_SetPixelAsVectorUInt8` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.integer(v);
  ;ans = .Call('R_swig_Image_SetPixelAsVectorUInt8', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsVectorUInt8`, 'returnType') = 'void'
attr(`Image_SetPixelAsVectorUInt8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Image_SetPixelAsVectorUInt8`) = c("SWIGFunction", class('Image_SetPixelAsVectorUInt8'))

# Start of Image_SetPixelAsVectorInt16

`Image_SetPixelAsVectorInt16` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.integer(v);
  ;ans = .Call('R_swig_Image_SetPixelAsVectorInt16', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsVectorInt16`, 'returnType') = 'void'
attr(`Image_SetPixelAsVectorInt16`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Image_SetPixelAsVectorInt16`) = c("SWIGFunction", class('Image_SetPixelAsVectorInt16'))

# Start of Image_SetPixelAsVectorUInt16

`Image_SetPixelAsVectorUInt16` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  if (inherits(v, "ExternalReference")) v = slot(v,"ref"); 
  ;ans = .Call('R_swig_Image_SetPixelAsVectorUInt16', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsVectorUInt16`, 'returnType') = 'void'
attr(`Image_SetPixelAsVectorUInt16`, "inputTypes") = c('_p_itk__simple__Image', 'integer', '_p_std__vectorT_uint16_t_t')
class(`Image_SetPixelAsVectorUInt16`) = c("SWIGFunction", class('Image_SetPixelAsVectorUInt16'))

# Start of Image_SetPixelAsVectorInt32

`Image_SetPixelAsVectorInt32` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.integer(v);
  ;ans = .Call('R_swig_Image_SetPixelAsVectorInt32', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsVectorInt32`, 'returnType') = 'void'
attr(`Image_SetPixelAsVectorInt32`, "inputTypes") = c('_p_itk__simple__Image', 'integer', '_p_std__vectorT_int32_t_t')
class(`Image_SetPixelAsVectorInt32`) = c("SWIGFunction", class('Image_SetPixelAsVectorInt32'))

# Start of Image_SetPixelAsVectorUInt32

`Image_SetPixelAsVectorUInt32` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.integer(v);
  ;ans = .Call('R_swig_Image_SetPixelAsVectorUInt32', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsVectorUInt32`, 'returnType') = 'void'
attr(`Image_SetPixelAsVectorUInt32`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Image_SetPixelAsVectorUInt32`) = c("SWIGFunction", class('Image_SetPixelAsVectorUInt32'))

# Start of Image_SetPixelAsVectorInt64

`Image_SetPixelAsVectorInt64` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.integer(v);
  ;ans = .Call('R_swig_Image_SetPixelAsVectorInt64', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsVectorInt64`, 'returnType') = 'void'
attr(`Image_SetPixelAsVectorInt64`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Image_SetPixelAsVectorInt64`) = c("SWIGFunction", class('Image_SetPixelAsVectorInt64'))

# Start of Image_SetPixelAsVectorUInt64

`Image_SetPixelAsVectorUInt64` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.integer(v);
  ;ans = .Call('R_swig_Image_SetPixelAsVectorUInt64', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsVectorUInt64`, 'returnType') = 'void'
attr(`Image_SetPixelAsVectorUInt64`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Image_SetPixelAsVectorUInt64`) = c("SWIGFunction", class('Image_SetPixelAsVectorUInt64'))

# Start of Image_SetPixelAsVectorFloat32

`Image_SetPixelAsVectorFloat32` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.numeric(v);
  ;ans = .Call('R_swig_Image_SetPixelAsVectorFloat32', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsVectorFloat32`, 'returnType') = 'void'
attr(`Image_SetPixelAsVectorFloat32`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric')
class(`Image_SetPixelAsVectorFloat32`) = c("SWIGFunction", class('Image_SetPixelAsVectorFloat32'))

# Start of Image_SetPixelAsVectorFloat64

`Image_SetPixelAsVectorFloat64` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  v = as.numeric(v);
  ;ans = .Call('R_swig_Image_SetPixelAsVectorFloat64', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsVectorFloat64`, 'returnType') = 'void'
attr(`Image_SetPixelAsVectorFloat64`, "inputTypes") = c('_p_itk__simple__Image', 'integer', '_p_std__vectorT_double_t')
class(`Image_SetPixelAsVectorFloat64`) = c("SWIGFunction", class('Image_SetPixelAsVectorFloat64'))

# Start of Image_SetPixelAsComplexFloat32

`Image_SetPixelAsComplexFloat32` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  if (inherits(v, "ExternalReference")) v = slot(v,"ref"); 
  ;ans = .Call('R_swig_Image_SetPixelAsComplexFloat32', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsComplexFloat32`, 'returnType') = 'void'
attr(`Image_SetPixelAsComplexFloat32`, "inputTypes") = c('_p_itk__simple__Image', 'integer', '_p_std__complexT_float_t')
class(`Image_SetPixelAsComplexFloat32`) = c("SWIGFunction", class('Image_SetPixelAsComplexFloat32'))

# Start of Image_SetPixelAsComplexFloat64

`Image_SetPixelAsComplexFloat64` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  if (inherits(v, "ExternalReference")) v = slot(v,"ref"); 
  ;ans = .Call('R_swig_Image_SetPixelAsComplexFloat64', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixelAsComplexFloat64`, 'returnType') = 'void'
attr(`Image_SetPixelAsComplexFloat64`, "inputTypes") = c('_p_itk__simple__Image', 'integer', '_p_std__complexT_double_t')
class(`Image_SetPixelAsComplexFloat64`) = c("SWIGFunction", class('Image_SetPixelAsComplexFloat64'))

# Start of Image_MakeUnique

`Image_MakeUnique` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_Image_MakeUnique', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_MakeUnique`, 'returnType') = 'void'
attr(`Image_MakeUnique`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_MakeUnique`) = c("SWIGFunction", class('Image_MakeUnique'))

# Start of Image_IsUnique

`Image_IsUnique` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Image_IsUnique', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_IsUnique`, 'returnType') = 'logical'
attr(`Image_IsUnique`, "inputTypes") = c('_p_itk__simple__Image')
class(`Image_IsUnique`) = c("SWIGFunction", class('Image_IsUnique'))

# Start of Image_SetPixel

`Image_SetPixel` = function(self, idx, v)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  
  ;ans = .Call('R_swig_Image_SetPixel', self, idx, v, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Image_SetPixel`, 'returnType') = 'void'
attr(`Image_SetPixel`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric')
class(`Image_SetPixel`) = c("SWIGFunction", class('Image_SetPixel'))

# Start of Image_GetPixel

`Image_GetPixel` = function(self, idx, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  idx = as.integer(idx);
  ;.Call('R_swig_Image_GetPixel', self, idx, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Image_GetPixel`, 'returnType') = 'numeric'
attr(`Image_GetPixel`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Image_GetPixel`) = c("SWIGFunction", class('Image_GetPixel'))

# Start of accessor method for itk::simple::Image
setMethod('$', '_p_itk__simple__Image', function(x, name)

{
  accessorFuns = list('Equal' = Image_Equal, 'GetITKBase' = Image_GetITKBase, 'GetPixelID' = Image_GetPixelID, 'GetPixelIDValue' = Image_GetPixelIDValue, 'GetPixelIDTypeAsString' = Image_GetPixelIDTypeAsString, 'GetDimension' = Image_GetDimension, 'GetNumberOfComponentsPerPixel' = Image_GetNumberOfComponentsPerPixel, 'GetNumberOfPixels' = Image_GetNumberOfPixels, 'GetSizeOfPixelComponent' = Image_GetSizeOfPixelComponent, 'GetOrigin' = Image_GetOrigin, 'SetOrigin' = Image_SetOrigin, 'GetSpacing' = Image_GetSpacing, 'SetSpacing' = Image_SetSpacing, 'GetDirection' = Image_GetDirection, 'SetDirection' = Image_SetDirection, 'TransformIndexToPhysicalPoint' = Image_TransformIndexToPhysicalPoint, 'TransformPhysicalPointToIndex' = Image_TransformPhysicalPointToIndex, 'TransformPhysicalPointToContinuousIndex' = Image_TransformPhysicalPointToContinuousIndex, 'TransformContinuousIndexToPhysicalPoint' = Image_TransformContinuousIndexToPhysicalPoint, 'EvaluateAtContinuousIndex' = Image_EvaluateAtContinuousIndex, 'EvaluateAtPhysicalPoint' = Image_EvaluateAtPhysicalPoint, 'IsCongruentImageGeometry' = Image_IsCongruentImageGeometry, 'IsSameImageGeometryAs' = Image_IsSameImageGeometryAs, 'GetSize' = Image_GetSize, 'GetWidth' = Image_GetWidth, 'GetHeight' = Image_GetHeight, 'GetDepth' = Image_GetDepth, 'CopyInformation' = Image_CopyInformation, 'GetMetaDataKeys' = Image_GetMetaDataKeys, 'HasMetaDataKey' = Image_HasMetaDataKey, 'GetMetaData' = Image_GetMetaData, 'SetMetaData' = Image_SetMetaData, 'EraseMetaData' = Image_EraseMetaData, 'ToString' = Image_ToString, 'ToVectorImage' = Image_ToVectorImage, 'ToScalarImage' = Image_ToScalarImage, 'GetPixelAsInt8' = Image_GetPixelAsInt8, 'GetPixelAsUInt8' = Image_GetPixelAsUInt8, 'GetPixelAsInt16' = Image_GetPixelAsInt16, 'GetPixelAsUInt16' = Image_GetPixelAsUInt16, 'GetPixelAsInt32' = Image_GetPixelAsInt32, 'GetPixelAsUInt32' = Image_GetPixelAsUInt32, 'GetPixelAsInt64' = Image_GetPixelAsInt64, 'GetPixelAsUInt64' = Image_GetPixelAsUInt64, 'GetPixelAsFloat' = Image_GetPixelAsFloat, 'GetPixelAsDouble' = Image_GetPixelAsDouble, 'GetPixelAsVectorInt8' = Image_GetPixelAsVectorInt8, 'GetPixelAsVectorUInt8' = Image_GetPixelAsVectorUInt8, 'GetPixelAsVectorInt16' = Image_GetPixelAsVectorInt16, 'GetPixelAsVectorUInt16' = Image_GetPixelAsVectorUInt16, 'GetPixelAsVectorInt32' = Image_GetPixelAsVectorInt32, 'GetPixelAsVectorUInt32' = Image_GetPixelAsVectorUInt32, 'GetPixelAsVectorInt64' = Image_GetPixelAsVectorInt64, 'GetPixelAsVectorUInt64' = Image_GetPixelAsVectorUInt64, 'GetPixelAsVectorFloat32' = Image_GetPixelAsVectorFloat32, 'GetPixelAsVectorFloat64' = Image_GetPixelAsVectorFloat64, 'GetPixelAsComplexFloat32' = Image_GetPixelAsComplexFloat32, 'GetPixelAsComplexFloat64' = Image_GetPixelAsComplexFloat64, 'SetPixelAsInt8' = Image_SetPixelAsInt8, 'SetPixelAsUInt8' = Image_SetPixelAsUInt8, 'SetPixelAsInt16' = Image_SetPixelAsInt16, 'SetPixelAsUInt16' = Image_SetPixelAsUInt16, 'SetPixelAsInt32' = Image_SetPixelAsInt32, 'SetPixelAsUInt32' = Image_SetPixelAsUInt32, 'SetPixelAsInt64' = Image_SetPixelAsInt64, 'SetPixelAsUInt64' = Image_SetPixelAsUInt64, 'SetPixelAsFloat' = Image_SetPixelAsFloat, 'SetPixelAsDouble' = Image_SetPixelAsDouble, 'SetPixelAsVectorInt8' = Image_SetPixelAsVectorInt8, 'SetPixelAsVectorUInt8' = Image_SetPixelAsVectorUInt8, 'SetPixelAsVectorInt16' = Image_SetPixelAsVectorInt16, 'SetPixelAsVectorUInt16' = Image_SetPixelAsVectorUInt16, 'SetPixelAsVectorInt32' = Image_SetPixelAsVectorInt32, 'SetPixelAsVectorUInt32' = Image_SetPixelAsVectorUInt32, 'SetPixelAsVectorInt64' = Image_SetPixelAsVectorInt64, 'SetPixelAsVectorUInt64' = Image_SetPixelAsVectorUInt64, 'SetPixelAsVectorFloat32' = Image_SetPixelAsVectorFloat32, 'SetPixelAsVectorFloat64' = Image_SetPixelAsVectorFloat64, 'SetPixelAsComplexFloat32' = Image_SetPixelAsComplexFloat32, 'SetPixelAsComplexFloat64' = Image_SetPixelAsComplexFloat64, 'MakeUnique' = Image_MakeUnique, 'IsUnique' = Image_IsUnique, 'SetPixel' = Image_SetPixel, 'GetPixel' = Image_GetPixel);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::Image
# Start of delete_ObjectOwnedBase

`delete_ObjectOwnedBase` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ObjectOwnedBase', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ObjectOwnedBase`, 'returnType') = 'void'
attr(`delete_ObjectOwnedBase`, "inputTypes") = c('_p_itk__simple__ObjectOwnedBase')
class(`delete_ObjectOwnedBase`) = c("SWIGFunction", class('delete_ObjectOwnedBase'))

# Start of new_ObjectOwnedBase

`ObjectOwnedBase` = function()
{
  ;ans = .Call('R_swig_new_ObjectOwnedBase', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ObjectOwnedBase", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ObjectOwnedBase);
  ans
  
}

attr(`ObjectOwnedBase`, 'returnType') = '_p_itk__simple__ObjectOwnedBase'
class(`ObjectOwnedBase`) = c("SWIGFunction", class('ObjectOwnedBase'))

# Start of ObjectOwnedBase_GetName

`ObjectOwnedBase_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ObjectOwnedBase_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ObjectOwnedBase_GetName`, 'returnType') = 'character'
attr(`ObjectOwnedBase_GetName`, "inputTypes") = c('_p_itk__simple__ObjectOwnedBase')
class(`ObjectOwnedBase_GetName`) = c("SWIGFunction", class('ObjectOwnedBase_GetName'))

# Start of ObjectOwnedBase_SetName

`ObjectOwnedBase_SetName` = function(self, name)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  name = as(name, "character"); 
  ;ans = .Call('R_swig_ObjectOwnedBase_SetName', self, name, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ObjectOwnedBase_SetName`, 'returnType') = 'void'
attr(`ObjectOwnedBase_SetName`, "inputTypes") = c('_p_itk__simple__ObjectOwnedBase', 'character')
class(`ObjectOwnedBase_SetName`) = c("SWIGFunction", class('ObjectOwnedBase_SetName'))

# Start of accessor method for itk::simple::ObjectOwnedBase
setMethod('$', '_p_itk__simple__ObjectOwnedBase', function(x, name)

{
  accessorFuns = list('GetName' = ObjectOwnedBase_GetName, 'SetName' = ObjectOwnedBase_SetName);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ObjectOwnedBase
# Start of new_Command

`Command` = function()
{
  ;ans = .Call('R_swig_new_Command', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Command", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Command);
  ans
  
}

attr(`Command`, 'returnType') = '_p_itk__simple__Command'
class(`Command`) = c("SWIGFunction", class('Command'))

# Start of delete_Command

`delete_Command` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_Command', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_Command`, 'returnType') = 'void'
attr(`delete_Command`, "inputTypes") = c('_p_itk__simple__Command')
class(`delete_Command`) = c("SWIGFunction", class('delete_Command'))

# Start of Command_GetName

`Command_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Command_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Command_GetName`, 'returnType') = 'character'
attr(`Command_GetName`, "inputTypes") = c('_p_itk__simple__Command')
class(`Command_GetName`) = c("SWIGFunction", class('Command_GetName'))

# Start of Command_SetName

`Command_SetName` = function(self, n)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as(n, "character"); 
  ;ans = .Call('R_swig_Command_SetName', self, n, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Command_SetName`, 'returnType') = 'void'
attr(`Command_SetName`, "inputTypes") = c('_p_itk__simple__Command', 'character')
class(`Command_SetName`) = c("SWIGFunction", class('Command_SetName'))

# Start of Command_Execute

`Command_Execute` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_Command_Execute', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Command_Execute`, 'returnType') = 'void'
attr(`Command_Execute`, "inputTypes") = c('_p_itk__simple__Command')
class(`Command_Execute`) = c("SWIGFunction", class('Command_Execute'))

# Start of accessor method for itk::simple::Command
setMethod('$', '_p_itk__simple__Command', function(x, name)

{
  accessorFuns = list('GetName' = Command_GetName, 'SetName' = Command_SetName, 'Execute' = Command_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::Command
# Start of new_LoggerBase

`LoggerBase` = function()
{
  ;ans = .Call('R_swig_new_LoggerBase', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LoggerBase", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LoggerBase);
  ans
  
}

attr(`LoggerBase`, 'returnType') = '_p_itk__simple__LoggerBase'
class(`LoggerBase`) = c("SWIGFunction", class('LoggerBase'))

# Start of delete_LoggerBase

`delete_LoggerBase` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LoggerBase', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LoggerBase`, 'returnType') = 'void'
attr(`delete_LoggerBase`, "inputTypes") = c('_p_itk__simple__LoggerBase')
class(`delete_LoggerBase`) = c("SWIGFunction", class('delete_LoggerBase'))

# Start of LoggerBase_DisplayText

`LoggerBase_DisplayText` = function(self, t)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  t = as(t, "character"); 
  ;ans = .Call('R_swig_LoggerBase_DisplayText', self, t, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LoggerBase_DisplayText`, 'returnType') = 'void'
attr(`LoggerBase_DisplayText`, "inputTypes") = c('_p_itk__simple__LoggerBase', 'character')
class(`LoggerBase_DisplayText`) = c("SWIGFunction", class('LoggerBase_DisplayText'))

# Start of LoggerBase_DisplayErrorText

`LoggerBase_DisplayErrorText` = function(self, t)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  t = as(t, "character"); 
  ;ans = .Call('R_swig_LoggerBase_DisplayErrorText', self, t, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LoggerBase_DisplayErrorText`, 'returnType') = 'void'
attr(`LoggerBase_DisplayErrorText`, "inputTypes") = c('_p_itk__simple__LoggerBase', 'character')
class(`LoggerBase_DisplayErrorText`) = c("SWIGFunction", class('LoggerBase_DisplayErrorText'))

# Start of LoggerBase_DisplayWarningText

`LoggerBase_DisplayWarningText` = function(self, t)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  t = as(t, "character"); 
  ;ans = .Call('R_swig_LoggerBase_DisplayWarningText', self, t, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LoggerBase_DisplayWarningText`, 'returnType') = 'void'
attr(`LoggerBase_DisplayWarningText`, "inputTypes") = c('_p_itk__simple__LoggerBase', 'character')
class(`LoggerBase_DisplayWarningText`) = c("SWIGFunction", class('LoggerBase_DisplayWarningText'))

# Start of LoggerBase_DisplayGenericOutputText

`LoggerBase_DisplayGenericOutputText` = function(self, t)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  t = as(t, "character"); 
  ;ans = .Call('R_swig_LoggerBase_DisplayGenericOutputText', self, t, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LoggerBase_DisplayGenericOutputText`, 'returnType') = 'void'
attr(`LoggerBase_DisplayGenericOutputText`, "inputTypes") = c('_p_itk__simple__LoggerBase', 'character')
class(`LoggerBase_DisplayGenericOutputText`) = c("SWIGFunction", class('LoggerBase_DisplayGenericOutputText'))

# Start of LoggerBase_DisplayDebugText

`LoggerBase_DisplayDebugText` = function(self, t)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  t = as(t, "character"); 
  ;ans = .Call('R_swig_LoggerBase_DisplayDebugText', self, t, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LoggerBase_DisplayDebugText`, 'returnType') = 'void'
attr(`LoggerBase_DisplayDebugText`, "inputTypes") = c('_p_itk__simple__LoggerBase', 'character')
class(`LoggerBase_DisplayDebugText`) = c("SWIGFunction", class('LoggerBase_DisplayDebugText'))

# Start of LoggerBase_GetName

`LoggerBase_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LoggerBase_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LoggerBase_GetName`, 'returnType') = 'character'
attr(`LoggerBase_GetName`, "inputTypes") = c('_p_itk__simple__LoggerBase')
class(`LoggerBase_GetName`) = c("SWIGFunction", class('LoggerBase_GetName'))

# Start of LoggerBase_SetName

`LoggerBase_SetName` = function(self, n)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as(n, "character"); 
  ;ans = .Call('R_swig_LoggerBase_SetName', self, n, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LoggerBase_SetName`, 'returnType') = 'void'
attr(`LoggerBase_SetName`, "inputTypes") = c('_p_itk__simple__LoggerBase', 'character')
class(`LoggerBase_SetName`) = c("SWIGFunction", class('LoggerBase_SetName'))

# Start of LoggerBase_SetAsGlobalITKLogger

`LoggerBase_SetAsGlobalITKLogger` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LoggerBase_SetAsGlobalITKLogger', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ITKLogger", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LoggerBase_SetAsGlobalITKLogger`, 'returnType') = '_p_itk__simple__ITKLogger'
attr(`LoggerBase_SetAsGlobalITKLogger`, "inputTypes") = c('_p_itk__simple__LoggerBase')
class(`LoggerBase_SetAsGlobalITKLogger`) = c("SWIGFunction", class('LoggerBase_SetAsGlobalITKLogger'))

# Start of LoggerBase_GetGlobalITKLogger

`LoggerBase_GetGlobalITKLogger` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_LoggerBase_GetGlobalITKLogger', as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ITKLogger", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LoggerBase_GetGlobalITKLogger`, 'returnType') = '_p_itk__simple__ITKLogger'
class(`LoggerBase_GetGlobalITKLogger`) = c("SWIGFunction", class('LoggerBase_GetGlobalITKLogger'))

# Start of accessor method for itk::simple::LoggerBase
setMethod('$', '_p_itk__simple__LoggerBase', function(x, name)

{
  accessorFuns = list('DisplayText' = LoggerBase_DisplayText, 'DisplayErrorText' = LoggerBase_DisplayErrorText, 'DisplayWarningText' = LoggerBase_DisplayWarningText, 'DisplayGenericOutputText' = LoggerBase_DisplayGenericOutputText, 'DisplayDebugText' = LoggerBase_DisplayDebugText, 'GetName' = LoggerBase_GetName, 'SetName' = LoggerBase_SetName, 'SetAsGlobalITKLogger' = LoggerBase_SetAsGlobalITKLogger);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LoggerBase
# Start of new_ITKLogger

`ITKLogger__SWIG_0` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_ITKLogger__SWIG_0', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ITKLogger", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ITKLogger);
  ans
  
}

attr(`ITKLogger__SWIG_0`, 'returnType') = '_p_itk__simple__ITKLogger'
attr(`ITKLogger__SWIG_0`, "inputTypes") = c('_p_itk__OutputWindow')
class(`ITKLogger__SWIG_0`) = c("SWIGFunction", class('ITKLogger__SWIG_0'))

# Start of delete_ITKLogger

`delete_ITKLogger` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ITKLogger', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ITKLogger`, 'returnType') = 'void'
attr(`delete_ITKLogger`, "inputTypes") = c('_p_itk__simple__ITKLogger')
class(`delete_ITKLogger`) = c("SWIGFunction", class('delete_ITKLogger'))

# Start of new_ITKLogger

`ITKLogger__SWIG_1` = function()
{
  ;ans = .Call('R_swig_new_ITKLogger__SWIG_1', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ITKLogger", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ITKLogger);
  ans
  
}

attr(`ITKLogger__SWIG_1`, 'returnType') = '_p_itk__simple__ITKLogger'
class(`ITKLogger__SWIG_1`) = c("SWIGFunction", class('ITKLogger__SWIG_1'))

# Start of new_ITKLogger

`ITKLogger__SWIG_2` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_ITKLogger__SWIG_2', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ITKLogger", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ITKLogger);
  ans
  
}

attr(`ITKLogger__SWIG_2`, 'returnType') = '_p_itk__simple__ITKLogger'
attr(`ITKLogger__SWIG_2`, "inputTypes") = c('_p_itk__simple__ITKLogger')
class(`ITKLogger__SWIG_2`) = c("SWIGFunction", class('ITKLogger__SWIG_2'))

`ITKLogger` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 0) {
    f <- ITKLogger__SWIG_1; 
  } else if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__OutputWindow') || is.null(argv[[1]]) ) {
      f <- ITKLogger__SWIG_0; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__ITKLogger') && length(argv[[1]]) == 1 ) {
      f <- ITKLogger__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ITKLogger with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ITKLogger_Equal

`ITKLogger_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_ITKLogger_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ITKLogger", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ITKLogger_Equal`, 'returnType') = '_p_itk__simple__ITKLogger'
attr(`ITKLogger_Equal`, "inputTypes") = c('_p_itk__simple__ITKLogger', '_p_itk__simple__ITKLogger')
class(`ITKLogger_Equal`) = c("SWIGFunction", class('ITKLogger_Equal'))

# Start of ITKLogger_DisplayText

`ITKLogger_DisplayText` = function(self, t)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  t = as(t, "character"); 
  ;ans = .Call('R_swig_ITKLogger_DisplayText', self, t, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ITKLogger_DisplayText`, 'returnType') = 'void'
attr(`ITKLogger_DisplayText`, "inputTypes") = c('_p_itk__simple__ITKLogger', 'character')
class(`ITKLogger_DisplayText`) = c("SWIGFunction", class('ITKLogger_DisplayText'))

# Start of ITKLogger_DisplayErrorText

`ITKLogger_DisplayErrorText` = function(self, t)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  t = as(t, "character"); 
  ;ans = .Call('R_swig_ITKLogger_DisplayErrorText', self, t, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ITKLogger_DisplayErrorText`, 'returnType') = 'void'
attr(`ITKLogger_DisplayErrorText`, "inputTypes") = c('_p_itk__simple__ITKLogger', 'character')
class(`ITKLogger_DisplayErrorText`) = c("SWIGFunction", class('ITKLogger_DisplayErrorText'))

# Start of ITKLogger_DisplayWarningText

`ITKLogger_DisplayWarningText` = function(self, t)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  t = as(t, "character"); 
  ;ans = .Call('R_swig_ITKLogger_DisplayWarningText', self, t, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ITKLogger_DisplayWarningText`, 'returnType') = 'void'
attr(`ITKLogger_DisplayWarningText`, "inputTypes") = c('_p_itk__simple__ITKLogger', 'character')
class(`ITKLogger_DisplayWarningText`) = c("SWIGFunction", class('ITKLogger_DisplayWarningText'))

# Start of ITKLogger_DisplayGenericOutputText

`ITKLogger_DisplayGenericOutputText` = function(self, t)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  t = as(t, "character"); 
  ;ans = .Call('R_swig_ITKLogger_DisplayGenericOutputText', self, t, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ITKLogger_DisplayGenericOutputText`, 'returnType') = 'void'
attr(`ITKLogger_DisplayGenericOutputText`, "inputTypes") = c('_p_itk__simple__ITKLogger', 'character')
class(`ITKLogger_DisplayGenericOutputText`) = c("SWIGFunction", class('ITKLogger_DisplayGenericOutputText'))

# Start of ITKLogger_DisplayDebugText

`ITKLogger_DisplayDebugText` = function(self, t)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  t = as(t, "character"); 
  ;ans = .Call('R_swig_ITKLogger_DisplayDebugText', self, t, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ITKLogger_DisplayDebugText`, 'returnType') = 'void'
attr(`ITKLogger_DisplayDebugText`, "inputTypes") = c('_p_itk__simple__ITKLogger', 'character')
class(`ITKLogger_DisplayDebugText`) = c("SWIGFunction", class('ITKLogger_DisplayDebugText'))

# Start of ITKLogger_SetAsGlobalITKLogger

`ITKLogger_SetAsGlobalITKLogger` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ITKLogger_SetAsGlobalITKLogger', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ITKLogger", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ITKLogger_SetAsGlobalITKLogger`, 'returnType') = '_p_itk__simple__ITKLogger'
attr(`ITKLogger_SetAsGlobalITKLogger`, "inputTypes") = c('_p_itk__simple__ITKLogger')
class(`ITKLogger_SetAsGlobalITKLogger`) = c("SWIGFunction", class('ITKLogger_SetAsGlobalITKLogger'))

# Start of accessor method for itk::simple::ITKLogger
setMethod('$', '_p_itk__simple__ITKLogger', function(x, name)

{
  accessorFuns = list('Equal' = ITKLogger_Equal, 'DisplayText' = ITKLogger_DisplayText, 'DisplayErrorText' = ITKLogger_DisplayErrorText, 'DisplayWarningText' = ITKLogger_DisplayWarningText, 'DisplayGenericOutputText' = ITKLogger_DisplayGenericOutputText, 'DisplayDebugText' = ITKLogger_DisplayDebugText, 'SetAsGlobalITKLogger' = ITKLogger_SetAsGlobalITKLogger);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ITKLogger
# Start of KernelEnum_sitkAnnulus_get

`KernelEnum_sitkAnnulus_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_KernelEnum_sitkAnnulus_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`KernelEnum_sitkAnnulus_get`, 'returnType') = 'character'
class(`KernelEnum_sitkAnnulus_get`) = c("SWIGFunction", class('KernelEnum_sitkAnnulus_get'))

# Start of KernelEnum_sitkBall_get

`KernelEnum_sitkBall_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_KernelEnum_sitkBall_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`KernelEnum_sitkBall_get`, 'returnType') = 'character'
class(`KernelEnum_sitkBall_get`) = c("SWIGFunction", class('KernelEnum_sitkBall_get'))

# Start of KernelEnum_sitkBox_get

`KernelEnum_sitkBox_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_KernelEnum_sitkBox_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`KernelEnum_sitkBox_get`, 'returnType') = 'character'
class(`KernelEnum_sitkBox_get`) = c("SWIGFunction", class('KernelEnum_sitkBox_get'))

# Start of KernelEnum_sitkCross_get

`KernelEnum_sitkCross_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_KernelEnum_sitkCross_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`KernelEnum_sitkCross_get`, 'returnType') = 'character'
class(`KernelEnum_sitkCross_get`) = c("SWIGFunction", class('KernelEnum_sitkCross_get'))

# Start of KernelEnum_sitkPolygon3_get

`KernelEnum_sitkPolygon3_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_KernelEnum_sitkPolygon3_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`KernelEnum_sitkPolygon3_get`, 'returnType') = 'character'
class(`KernelEnum_sitkPolygon3_get`) = c("SWIGFunction", class('KernelEnum_sitkPolygon3_get'))

# Start of KernelEnum_sitkPolygon4_get

`KernelEnum_sitkPolygon4_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_KernelEnum_sitkPolygon4_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`KernelEnum_sitkPolygon4_get`, 'returnType') = 'character'
class(`KernelEnum_sitkPolygon4_get`) = c("SWIGFunction", class('KernelEnum_sitkPolygon4_get'))

# Start of KernelEnum_sitkPolygon5_get

`KernelEnum_sitkPolygon5_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_KernelEnum_sitkPolygon5_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`KernelEnum_sitkPolygon5_get`, 'returnType') = 'character'
class(`KernelEnum_sitkPolygon5_get`) = c("SWIGFunction", class('KernelEnum_sitkPolygon5_get'))

# Start of KernelEnum_sitkPolygon6_get

`KernelEnum_sitkPolygon6_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_KernelEnum_sitkPolygon6_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`KernelEnum_sitkPolygon6_get`, 'returnType') = 'character'
class(`KernelEnum_sitkPolygon6_get`) = c("SWIGFunction", class('KernelEnum_sitkPolygon6_get'))

# Start of KernelEnum_sitkPolygon7_get

`KernelEnum_sitkPolygon7_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_KernelEnum_sitkPolygon7_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`KernelEnum_sitkPolygon7_get`, 'returnType') = 'character'
class(`KernelEnum_sitkPolygon7_get`) = c("SWIGFunction", class('KernelEnum_sitkPolygon7_get'))

# Start of KernelEnum_sitkPolygon8_get

`KernelEnum_sitkPolygon8_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_KernelEnum_sitkPolygon8_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`KernelEnum_sitkPolygon8_get`, 'returnType') = 'character'
class(`KernelEnum_sitkPolygon8_get`) = c("SWIGFunction", class('KernelEnum_sitkPolygon8_get'))

# Start of KernelEnum_sitkPolygon9_get

`KernelEnum_sitkPolygon9_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_KernelEnum_sitkPolygon9_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`KernelEnum_sitkPolygon9_get`, 'returnType') = 'character'
class(`KernelEnum_sitkPolygon9_get`) = c("SWIGFunction", class('KernelEnum_sitkPolygon9_get'))

# Start of EventEnum_sitkAnyEvent_get

`EventEnum_sitkAnyEvent_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_EventEnum_sitkAnyEvent_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__EventEnum");
  
  ans
  
}

attr(`EventEnum_sitkAnyEvent_get`, 'returnType') = 'character'
class(`EventEnum_sitkAnyEvent_get`) = c("SWIGFunction", class('EventEnum_sitkAnyEvent_get'))

# Start of EventEnum_sitkAbortEvent_get

`EventEnum_sitkAbortEvent_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_EventEnum_sitkAbortEvent_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__EventEnum");
  
  ans
  
}

attr(`EventEnum_sitkAbortEvent_get`, 'returnType') = 'character'
class(`EventEnum_sitkAbortEvent_get`) = c("SWIGFunction", class('EventEnum_sitkAbortEvent_get'))

# Start of EventEnum_sitkDeleteEvent_get

`EventEnum_sitkDeleteEvent_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_EventEnum_sitkDeleteEvent_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__EventEnum");
  
  ans
  
}

attr(`EventEnum_sitkDeleteEvent_get`, 'returnType') = 'character'
class(`EventEnum_sitkDeleteEvent_get`) = c("SWIGFunction", class('EventEnum_sitkDeleteEvent_get'))

# Start of EventEnum_sitkEndEvent_get

`EventEnum_sitkEndEvent_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_EventEnum_sitkEndEvent_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__EventEnum");
  
  ans
  
}

attr(`EventEnum_sitkEndEvent_get`, 'returnType') = 'character'
class(`EventEnum_sitkEndEvent_get`) = c("SWIGFunction", class('EventEnum_sitkEndEvent_get'))

# Start of EventEnum_sitkIterationEvent_get

`EventEnum_sitkIterationEvent_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_EventEnum_sitkIterationEvent_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__EventEnum");
  
  ans
  
}

attr(`EventEnum_sitkIterationEvent_get`, 'returnType') = 'character'
class(`EventEnum_sitkIterationEvent_get`) = c("SWIGFunction", class('EventEnum_sitkIterationEvent_get'))

# Start of EventEnum_sitkProgressEvent_get

`EventEnum_sitkProgressEvent_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_EventEnum_sitkProgressEvent_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__EventEnum");
  
  ans
  
}

attr(`EventEnum_sitkProgressEvent_get`, 'returnType') = 'character'
class(`EventEnum_sitkProgressEvent_get`) = c("SWIGFunction", class('EventEnum_sitkProgressEvent_get'))

# Start of EventEnum_sitkStartEvent_get

`EventEnum_sitkStartEvent_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_EventEnum_sitkStartEvent_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__EventEnum");
  
  ans
  
}

attr(`EventEnum_sitkStartEvent_get`, 'returnType') = 'character'
class(`EventEnum_sitkStartEvent_get`) = c("SWIGFunction", class('EventEnum_sitkStartEvent_get'))

# Start of EventEnum_sitkMultiResolutionIterationEvent_get

`EventEnum_sitkMultiResolutionIterationEvent_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_EventEnum_sitkMultiResolutionIterationEvent_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__EventEnum");
  
  ans
  
}

attr(`EventEnum_sitkMultiResolutionIterationEvent_get`, 'returnType') = 'character'
class(`EventEnum_sitkMultiResolutionIterationEvent_get`) = c("SWIGFunction", class('EventEnum_sitkMultiResolutionIterationEvent_get'))

# Start of EventEnum_sitkUserEvent_get

`EventEnum_sitkUserEvent_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_EventEnum_sitkUserEvent_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__EventEnum");
  
  ans
  
}

attr(`EventEnum_sitkUserEvent_get`, 'returnType') = 'character'
class(`EventEnum_sitkUserEvent_get`) = c("SWIGFunction", class('EventEnum_sitkUserEvent_get'))

# Start of SeedEnum_sitkWallClock_get

`SeedEnum_sitkWallClock_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_SeedEnum_sitkWallClock_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__SeedEnum");
  
  ans
  
}

attr(`SeedEnum_sitkWallClock_get`, 'returnType') = 'character'
class(`SeedEnum_sitkWallClock_get`) = c("SWIGFunction", class('SeedEnum_sitkWallClock_get'))

# Start of TransformEnum_sitkUnknownTransform_get

`TransformEnum_sitkUnknownTransform_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkUnknownTransform_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkUnknownTransform_get`, 'returnType') = 'character'
class(`TransformEnum_sitkUnknownTransform_get`) = c("SWIGFunction", class('TransformEnum_sitkUnknownTransform_get'))

# Start of TransformEnum_sitkIdentity_get

`TransformEnum_sitkIdentity_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkIdentity_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkIdentity_get`, 'returnType') = 'character'
class(`TransformEnum_sitkIdentity_get`) = c("SWIGFunction", class('TransformEnum_sitkIdentity_get'))

# Start of TransformEnum_sitkTranslation_get

`TransformEnum_sitkTranslation_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkTranslation_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkTranslation_get`, 'returnType') = 'character'
class(`TransformEnum_sitkTranslation_get`) = c("SWIGFunction", class('TransformEnum_sitkTranslation_get'))

# Start of TransformEnum_sitkScale_get

`TransformEnum_sitkScale_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkScale_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkScale_get`, 'returnType') = 'character'
class(`TransformEnum_sitkScale_get`) = c("SWIGFunction", class('TransformEnum_sitkScale_get'))

# Start of TransformEnum_sitkScaleLogarithmic_get

`TransformEnum_sitkScaleLogarithmic_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkScaleLogarithmic_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkScaleLogarithmic_get`, 'returnType') = 'character'
class(`TransformEnum_sitkScaleLogarithmic_get`) = c("SWIGFunction", class('TransformEnum_sitkScaleLogarithmic_get'))

# Start of TransformEnum_sitkEuler_get

`TransformEnum_sitkEuler_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkEuler_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkEuler_get`, 'returnType') = 'character'
class(`TransformEnum_sitkEuler_get`) = c("SWIGFunction", class('TransformEnum_sitkEuler_get'))

# Start of TransformEnum_sitkSimilarity_get

`TransformEnum_sitkSimilarity_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkSimilarity_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkSimilarity_get`, 'returnType') = 'character'
class(`TransformEnum_sitkSimilarity_get`) = c("SWIGFunction", class('TransformEnum_sitkSimilarity_get'))

# Start of TransformEnum_sitkQuaternionRigid_get

`TransformEnum_sitkQuaternionRigid_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkQuaternionRigid_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkQuaternionRigid_get`, 'returnType') = 'character'
class(`TransformEnum_sitkQuaternionRigid_get`) = c("SWIGFunction", class('TransformEnum_sitkQuaternionRigid_get'))

# Start of TransformEnum_sitkVersor_get

`TransformEnum_sitkVersor_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkVersor_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkVersor_get`, 'returnType') = 'character'
class(`TransformEnum_sitkVersor_get`) = c("SWIGFunction", class('TransformEnum_sitkVersor_get'))

# Start of TransformEnum_sitkVersorRigid_get

`TransformEnum_sitkVersorRigid_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkVersorRigid_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkVersorRigid_get`, 'returnType') = 'character'
class(`TransformEnum_sitkVersorRigid_get`) = c("SWIGFunction", class('TransformEnum_sitkVersorRigid_get'))

# Start of TransformEnum_sitkScaleSkewVersor_get

`TransformEnum_sitkScaleSkewVersor_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkScaleSkewVersor_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkScaleSkewVersor_get`, 'returnType') = 'character'
class(`TransformEnum_sitkScaleSkewVersor_get`) = c("SWIGFunction", class('TransformEnum_sitkScaleSkewVersor_get'))

# Start of TransformEnum_sitkComposeScaleSkewVersor_get

`TransformEnum_sitkComposeScaleSkewVersor_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkComposeScaleSkewVersor_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkComposeScaleSkewVersor_get`, 'returnType') = 'character'
class(`TransformEnum_sitkComposeScaleSkewVersor_get`) = c("SWIGFunction", class('TransformEnum_sitkComposeScaleSkewVersor_get'))

# Start of TransformEnum_sitkScaleVersor_get

`TransformEnum_sitkScaleVersor_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkScaleVersor_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkScaleVersor_get`, 'returnType') = 'character'
class(`TransformEnum_sitkScaleVersor_get`) = c("SWIGFunction", class('TransformEnum_sitkScaleVersor_get'))

# Start of TransformEnum_sitkAffine_get

`TransformEnum_sitkAffine_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkAffine_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkAffine_get`, 'returnType') = 'character'
class(`TransformEnum_sitkAffine_get`) = c("SWIGFunction", class('TransformEnum_sitkAffine_get'))

# Start of TransformEnum_sitkComposite_get

`TransformEnum_sitkComposite_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkComposite_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkComposite_get`, 'returnType') = 'character'
class(`TransformEnum_sitkComposite_get`) = c("SWIGFunction", class('TransformEnum_sitkComposite_get'))

# Start of TransformEnum_sitkDisplacementField_get

`TransformEnum_sitkDisplacementField_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkDisplacementField_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkDisplacementField_get`, 'returnType') = 'character'
class(`TransformEnum_sitkDisplacementField_get`) = c("SWIGFunction", class('TransformEnum_sitkDisplacementField_get'))

# Start of TransformEnum_sitkBSplineTransform_get

`TransformEnum_sitkBSplineTransform_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TransformEnum_sitkBSplineTransform_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`TransformEnum_sitkBSplineTransform_get`, 'returnType') = 'character'
class(`TransformEnum_sitkBSplineTransform_get`) = c("SWIGFunction", class('TransformEnum_sitkBSplineTransform_get'))

# Start of new_Transform

`Transform__SWIG_0` = function()
{
  ;ans = .Call('R_swig_new_Transform__SWIG_0', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Transform);
  ans
  
}

attr(`Transform__SWIG_0`, 'returnType') = '_p_itk__simple__Transform'
class(`Transform__SWIG_0`) = c("SWIGFunction", class('Transform__SWIG_0'))

# Start of new_Transform

`Transform__SWIG_1` = function(transform)
{
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_new_Transform__SWIG_1', transform, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Transform);
  ans
  
}

attr(`Transform__SWIG_1`, 'returnType') = '_p_itk__simple__Transform'
attr(`Transform__SWIG_1`, "inputTypes") = c('_p_itk__TransformBaseTemplateT_double_t')
class(`Transform__SWIG_1`) = c("SWIGFunction", class('Transform__SWIG_1'))

# Start of new_Transform

`Transform__SWIG_2` = function(dimensions, type)
{
  dimensions = as.integer(dimensions);
  
  if(length(dimensions) > 1) {
    warning("using only the first element of dimensions");
  };
  
  type = enumToInteger(type, "_itk__simple__TransformEnum"); 
  
  if(length(type) > 1) {
    warning("using only the first element of type");
  };
  
  ;ans = .Call('R_swig_new_Transform__SWIG_2', dimensions, type, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Transform);
  ans
  
}

attr(`Transform__SWIG_2`, 'returnType') = '_p_itk__simple__Transform'
attr(`Transform__SWIG_2`, "inputTypes") = c('integer', 'character')
class(`Transform__SWIG_2`) = c("SWIGFunction", class('Transform__SWIG_2'))

# Start of new_Transform

`Transform__SWIG_3` = function(displacement, type)
{
  if (inherits(displacement, "ExternalReference")) displacement = slot(displacement,"ref"); 
  type = enumToInteger(type, "_itk__simple__TransformEnum"); 
  
  if(length(type) > 1) {
    warning("using only the first element of type");
  };
  
  ;ans = .Call('R_swig_new_Transform__SWIG_3', displacement, type, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Transform);
  ans
  
}

attr(`Transform__SWIG_3`, 'returnType') = '_p_itk__simple__Transform'
attr(`Transform__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'character')
class(`Transform__SWIG_3`) = c("SWIGFunction", class('Transform__SWIG_3'))

# Start of new_Transform

`Transform__SWIG_4` = function(displacement)
{
  if (inherits(displacement, "ExternalReference")) displacement = slot(displacement,"ref"); 
  ;ans = .Call('R_swig_new_Transform__SWIG_4', displacement, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Transform);
  ans
  
}

attr(`Transform__SWIG_4`, 'returnType') = '_p_itk__simple__Transform'
attr(`Transform__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`Transform__SWIG_4`) = c("SWIGFunction", class('Transform__SWIG_4'))

# Start of delete_Transform

`delete_Transform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_Transform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_Transform`, 'returnType') = 'void'
attr(`delete_Transform`, "inputTypes") = c('_p_itk__simple__Transform')
class(`delete_Transform`) = c("SWIGFunction", class('delete_Transform'))

# Start of Transform_Equal

`Transform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_Transform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Transform_Equal`, 'returnType') = '_p_itk__simple__Transform'
attr(`Transform_Equal`, "inputTypes") = c('_p_itk__simple__Transform', '_p_itk__simple__Transform')
class(`Transform_Equal`) = c("SWIGFunction", class('Transform_Equal'))

# Start of new_Transform

`Transform__SWIG_5` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_Transform__SWIG_5', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Transform);
  ans
  
}

attr(`Transform__SWIG_5`, 'returnType') = '_p_itk__simple__Transform'
attr(`Transform__SWIG_5`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform__SWIG_5`) = c("SWIGFunction", class('Transform__SWIG_5'))

`Transform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 0) {
    f <- Transform__SWIG_0; 
  } else if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__TransformBaseTemplateT_double_t') || is.null(argv[[1]]) ) {
      f <- Transform__SWIG_1; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Transform__SWIG_4; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- Transform__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Transform__SWIG_3; 
    }
    else if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Transform__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Transform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of Transform_GetITKBase

`Transform_GetITKBase__SWIG_0` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_Transform_GetITKBase__SWIG_0', self, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__TransformBaseTemplateT_double_t", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Transform_GetITKBase__SWIG_0`, 'returnType') = '_p_itk__TransformBaseTemplateT_double_t'
attr(`Transform_GetITKBase__SWIG_0`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform_GetITKBase__SWIG_0`) = c("SWIGFunction", class('Transform_GetITKBase__SWIG_0'))

# Start of Transform_GetITKBase

`Transform_GetITKBase__SWIG_1` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_Transform_GetITKBase__SWIG_1', self, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__TransformBaseTemplateT_double_t", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Transform_GetITKBase__SWIG_1`, 'returnType') = '_p_itk__TransformBaseTemplateT_double_t'
attr(`Transform_GetITKBase__SWIG_1`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform_GetITKBase__SWIG_1`) = c("SWIGFunction", class('Transform_GetITKBase__SWIG_1'))

`Transform_GetITKBase` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Transform') || is.null(argv[[1]]) ) {
      f <- Transform_GetITKBase__SWIG_0; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') || is.null(argv[[1]]) ) {
      f <- Transform_GetITKBase__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Transform_GetITKBase with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of Transform_GetDimension

`Transform_GetDimension` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Transform_GetDimension', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Transform_GetDimension`, 'returnType') = 'integer'
attr(`Transform_GetDimension`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform_GetDimension`) = c("SWIGFunction", class('Transform_GetDimension'))

# Start of Transform_SetParameters

`Transform_SetParameters` = function(self, parameters)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  parameters = as.numeric(parameters);
  ;ans = .Call('R_swig_Transform_SetParameters', self, parameters, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Transform_SetParameters`, 'returnType') = 'void'
attr(`Transform_SetParameters`, "inputTypes") = c('_p_itk__simple__Transform', '_p_std__vectorT_double_t')
class(`Transform_SetParameters`) = c("SWIGFunction", class('Transform_SetParameters'))

# Start of Transform_GetParameters

`Transform_GetParameters` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Transform_GetParameters', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Transform_GetParameters`, 'returnType') = 'numeric'
attr(`Transform_GetParameters`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform_GetParameters`) = c("SWIGFunction", class('Transform_GetParameters'))

# Start of Transform_GetNumberOfParameters

`Transform_GetNumberOfParameters` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Transform_GetNumberOfParameters', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Transform_GetNumberOfParameters`, 'returnType') = 'integer'
attr(`Transform_GetNumberOfParameters`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform_GetNumberOfParameters`) = c("SWIGFunction", class('Transform_GetNumberOfParameters'))

# Start of Transform_SetFixedParameters

`Transform_SetFixedParameters` = function(self, parameters)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  parameters = as.numeric(parameters);
  ;ans = .Call('R_swig_Transform_SetFixedParameters', self, parameters, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Transform_SetFixedParameters`, 'returnType') = 'void'
attr(`Transform_SetFixedParameters`, "inputTypes") = c('_p_itk__simple__Transform', '_p_std__vectorT_double_t')
class(`Transform_SetFixedParameters`) = c("SWIGFunction", class('Transform_SetFixedParameters'))

# Start of Transform_GetFixedParameters

`Transform_GetFixedParameters` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Transform_GetFixedParameters', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Transform_GetFixedParameters`, 'returnType') = 'numeric'
attr(`Transform_GetFixedParameters`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform_GetFixedParameters`) = c("SWIGFunction", class('Transform_GetFixedParameters'))

# Start of Transform_GetNumberOfFixedParameters

`Transform_GetNumberOfFixedParameters` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Transform_GetNumberOfFixedParameters', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Transform_GetNumberOfFixedParameters`, 'returnType') = 'integer'
attr(`Transform_GetNumberOfFixedParameters`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform_GetNumberOfFixedParameters`) = c("SWIGFunction", class('Transform_GetNumberOfFixedParameters'))

# Start of Transform_TransformPoint

`Transform_TransformPoint` = function(self, point, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  point = as.numeric(point);
  ;.Call('R_swig_Transform_TransformPoint', self, point, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Transform_TransformPoint`, 'returnType') = 'numeric'
attr(`Transform_TransformPoint`, "inputTypes") = c('_p_itk__simple__Transform', '_p_std__vectorT_double_t')
class(`Transform_TransformPoint`) = c("SWIGFunction", class('Transform_TransformPoint'))

# Start of Transform_TransformVector

`Transform_TransformVector` = function(self, vector, point, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  vector = as.numeric(vector);
  point = as.numeric(point);
  ;.Call('R_swig_Transform_TransformVector', self, vector, point, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Transform_TransformVector`, 'returnType') = 'numeric'
attr(`Transform_TransformVector`, "inputTypes") = c('_p_itk__simple__Transform', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`Transform_TransformVector`) = c("SWIGFunction", class('Transform_TransformVector'))

# Start of Transform_WriteTransform

`Transform_WriteTransform` = function(self, filename)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  filename = as(filename, "character"); 
  ;ans = .Call('R_swig_Transform_WriteTransform', self, filename, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Transform_WriteTransform`, 'returnType') = 'void'
attr(`Transform_WriteTransform`, "inputTypes") = c('_p_itk__simple__Transform', 'character')
class(`Transform_WriteTransform`) = c("SWIGFunction", class('Transform_WriteTransform'))

# Start of Transform_IsLinear

`Transform_IsLinear` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Transform_IsLinear', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Transform_IsLinear`, 'returnType') = 'logical'
attr(`Transform_IsLinear`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform_IsLinear`) = c("SWIGFunction", class('Transform_IsLinear'))

# Start of Transform_SetIdentity

`Transform_SetIdentity` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_Transform_SetIdentity', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Transform_SetIdentity`, 'returnType') = 'void'
attr(`Transform_SetIdentity`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform_SetIdentity`) = c("SWIGFunction", class('Transform_SetIdentity'))

# Start of Transform_SetInverse

`Transform_SetInverse` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Transform_SetInverse', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Transform_SetInverse`, 'returnType') = 'logical'
attr(`Transform_SetInverse`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform_SetInverse`) = c("SWIGFunction", class('Transform_SetInverse'))

# Start of Transform_GetInverse

`Transform_GetInverse` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_Transform_GetInverse', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Transform_GetInverse`, 'returnType') = '_p_itk__simple__Transform'
attr(`Transform_GetInverse`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform_GetInverse`) = c("SWIGFunction", class('Transform_GetInverse'))

# Start of Transform_ToString

`Transform_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Transform_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Transform_ToString`, 'returnType') = 'character'
attr(`Transform_ToString`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform_ToString`) = c("SWIGFunction", class('Transform_ToString'))

# Start of Transform_GetName

`Transform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Transform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Transform_GetName`, 'returnType') = 'character'
attr(`Transform_GetName`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform_GetName`) = c("SWIGFunction", class('Transform_GetName'))

# Start of Transform_MakeUnique

`Transform_MakeUnique` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_Transform_MakeUnique', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Transform_MakeUnique`, 'returnType') = 'void'
attr(`Transform_MakeUnique`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform_MakeUnique`) = c("SWIGFunction", class('Transform_MakeUnique'))

# Start of Transform_GetTransformEnum

`Transform_GetTransformEnum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_Transform_GetTransformEnum', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TransformEnum");
  
  ans
  
}

attr(`Transform_GetTransformEnum`, 'returnType') = 'character'
attr(`Transform_GetTransformEnum`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Transform_GetTransformEnum`) = c("SWIGFunction", class('Transform_GetTransformEnum'))

# Start of accessor method for itk::simple::Transform
setMethod('$', '_p_itk__simple__Transform', function(x, name)

{
  accessorFuns = list('Equal' = Transform_Equal, 'GetITKBase' = Transform_GetITKBase, 'GetDimension' = Transform_GetDimension, 'SetParameters' = Transform_SetParameters, 'GetParameters' = Transform_GetParameters, 'GetNumberOfParameters' = Transform_GetNumberOfParameters, 'SetFixedParameters' = Transform_SetFixedParameters, 'GetFixedParameters' = Transform_GetFixedParameters, 'GetNumberOfFixedParameters' = Transform_GetNumberOfFixedParameters, 'TransformPoint' = Transform_TransformPoint, 'TransformVector' = Transform_TransformVector, 'WriteTransform' = Transform_WriteTransform, 'IsLinear' = Transform_IsLinear, 'SetIdentity' = Transform_SetIdentity, 'SetInverse' = Transform_SetInverse, 'GetInverse' = Transform_GetInverse, 'ToString' = Transform_ToString, 'GetName' = Transform_GetName, 'MakeUnique' = Transform_MakeUnique, 'GetTransformEnum' = Transform_GetTransformEnum);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::Transform
# Start of ReadTransform

`ReadTransform` = function(filename, .copy = FALSE)
{
  filename = as(filename, "character"); 
  ;ans = .Call('R_swig_ReadTransform', filename, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReadTransform`, 'returnType') = '_p_itk__simple__Transform'
attr(`ReadTransform`, "inputTypes") = c('character')
class(`ReadTransform`) = c("SWIGFunction", class('ReadTransform'))

# Start of WriteTransform

`WriteTransform` = function(transform, filename)
{
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  filename = as(filename, "character"); 
  ;ans = .Call('R_swig_WriteTransform', transform, filename, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WriteTransform`, 'returnType') = 'void'
attr(`WriteTransform`, "inputTypes") = c('_p_itk__simple__Transform', 'character')
class(`WriteTransform`) = c("SWIGFunction", class('WriteTransform'))

# Start of delete_BSplineTransform

`delete_BSplineTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BSplineTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BSplineTransform`, 'returnType') = 'void'
attr(`delete_BSplineTransform`, "inputTypes") = c('_p_itk__simple__BSplineTransform')
class(`delete_BSplineTransform`) = c("SWIGFunction", class('delete_BSplineTransform'))

# Start of new_BSplineTransform

`BSplineTransform__SWIG_0` = function(dimensions, order)
{
  dimensions = as.integer(dimensions);
  
  if(length(dimensions) > 1) {
    warning("using only the first element of dimensions");
  };
  
  order = as.integer(order);
  
  if(length(order) > 1) {
    warning("using only the first element of order");
  };
  
  ;ans = .Call('R_swig_new_BSplineTransform__SWIG_0', dimensions, order, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BSplineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BSplineTransform);
  ans
  
}

attr(`BSplineTransform__SWIG_0`, 'returnType') = '_p_itk__simple__BSplineTransform'
attr(`BSplineTransform__SWIG_0`, "inputTypes") = c('integer', 'integer')
class(`BSplineTransform__SWIG_0`) = c("SWIGFunction", class('BSplineTransform__SWIG_0'))

# Start of new_BSplineTransform

`BSplineTransform__SWIG_1` = function(dimensions)
{
  dimensions = as.integer(dimensions);
  
  if(length(dimensions) > 1) {
    warning("using only the first element of dimensions");
  };
  
  ;ans = .Call('R_swig_new_BSplineTransform__SWIG_1', dimensions, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BSplineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BSplineTransform);
  ans
  
}

attr(`BSplineTransform__SWIG_1`, 'returnType') = '_p_itk__simple__BSplineTransform'
attr(`BSplineTransform__SWIG_1`, "inputTypes") = c('integer')
class(`BSplineTransform__SWIG_1`) = c("SWIGFunction", class('BSplineTransform__SWIG_1'))

# Start of new_BSplineTransform

`BSplineTransform__SWIG_2` = function(coefficientImages, order)
{
  coefficientImages=ImListToVec(coefficientImages);
  coefficientImages=slot(coefficientImages, "ref")
  
  order = as.integer(order);
  
  if(length(order) > 1) {
    warning("using only the first element of order");
  };
  
  ;ans = .Call('R_swig_new_BSplineTransform__SWIG_2', coefficientImages, order, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BSplineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BSplineTransform);
  ans
  
}

attr(`BSplineTransform__SWIG_2`, 'returnType') = '_p_itk__simple__BSplineTransform'
attr(`BSplineTransform__SWIG_2`, "inputTypes") = c('list', 'integer')
class(`BSplineTransform__SWIG_2`) = c("SWIGFunction", class('BSplineTransform__SWIG_2'))

# Start of new_BSplineTransform

`BSplineTransform__SWIG_3` = function(coefficientImages)
{
  coefficientImages=ImListToVec(coefficientImages);
  coefficientImages=slot(coefficientImages, "ref")
  
  ;ans = .Call('R_swig_new_BSplineTransform__SWIG_3', coefficientImages, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BSplineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BSplineTransform);
  ans
  
}

attr(`BSplineTransform__SWIG_3`, 'returnType') = '_p_itk__simple__BSplineTransform'
attr(`BSplineTransform__SWIG_3`, "inputTypes") = c('list')
class(`BSplineTransform__SWIG_3`) = c("SWIGFunction", class('BSplineTransform__SWIG_3'))

# Start of new_BSplineTransform

`BSplineTransform__SWIG_4` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_BSplineTransform__SWIG_4', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BSplineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BSplineTransform);
  ans
  
}

attr(`BSplineTransform__SWIG_4`, 'returnType') = '_p_itk__simple__BSplineTransform'
attr(`BSplineTransform__SWIG_4`, "inputTypes") = c('_p_itk__simple__BSplineTransform')
class(`BSplineTransform__SWIG_4`) = c("SWIGFunction", class('BSplineTransform__SWIG_4'))

# Start of new_BSplineTransform

`BSplineTransform__SWIG_5` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_BSplineTransform__SWIG_5', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BSplineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BSplineTransform);
  ans
  
}

attr(`BSplineTransform__SWIG_5`, 'returnType') = '_p_itk__simple__BSplineTransform'
attr(`BSplineTransform__SWIG_5`, "inputTypes") = c('_p_itk__simple__Transform')
class(`BSplineTransform__SWIG_5`) = c("SWIGFunction", class('BSplineTransform__SWIG_5'))

`BSplineTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__BSplineTransform') && length(argv[[1]]) == 1 ) {
      f <- BSplineTransform__SWIG_4; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- BSplineTransform__SWIG_5; 
    }
    else if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- BSplineTransform__SWIG_1; 
    }
    else if (
      (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
      ) {
      f <- BSplineTransform__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BSplineTransform__SWIG_0; 
    }
    else if ((
        (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
        ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BSplineTransform__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BSplineTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of BSplineTransform_Equal

`BSplineTransform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_BSplineTransform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BSplineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BSplineTransform_Equal`, 'returnType') = '_p_itk__simple__BSplineTransform'
attr(`BSplineTransform_Equal`, "inputTypes") = c('_p_itk__simple__BSplineTransform', '_p_itk__simple__BSplineTransform')
class(`BSplineTransform_Equal`) = c("SWIGFunction", class('BSplineTransform_Equal'))

# Start of BSplineTransform_GetName

`BSplineTransform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BSplineTransform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BSplineTransform_GetName`, 'returnType') = 'character'
attr(`BSplineTransform_GetName`, "inputTypes") = c('_p_itk__simple__BSplineTransform')
class(`BSplineTransform_GetName`) = c("SWIGFunction", class('BSplineTransform_GetName'))

# Start of BSplineTransform_SetTransformDomainDirection

`BSplineTransform_SetTransformDomainDirection` = function(self, s_arg2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  s_arg2 = as.numeric(s_arg2);
  ;ans = .Call('R_swig_BSplineTransform_SetTransformDomainDirection', self, s_arg2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BSplineTransform_SetTransformDomainDirection`, 'returnType') = 'void'
attr(`BSplineTransform_SetTransformDomainDirection`, "inputTypes") = c('_p_itk__simple__BSplineTransform', '_p_std__vectorT_double_t')
class(`BSplineTransform_SetTransformDomainDirection`) = c("SWIGFunction", class('BSplineTransform_SetTransformDomainDirection'))

# Start of BSplineTransform_GetTransformDomainDirection

`BSplineTransform_GetTransformDomainDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BSplineTransform_GetTransformDomainDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BSplineTransform_GetTransformDomainDirection`, 'returnType') = 'numeric'
attr(`BSplineTransform_GetTransformDomainDirection`, "inputTypes") = c('_p_itk__simple__BSplineTransform')
class(`BSplineTransform_GetTransformDomainDirection`) = c("SWIGFunction", class('BSplineTransform_GetTransformDomainDirection'))

# Start of BSplineTransform_SetTransformDomainMeshSize

`BSplineTransform_SetTransformDomainMeshSize` = function(self, s_arg2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  s_arg2 = as.integer(s_arg2);
  ;ans = .Call('R_swig_BSplineTransform_SetTransformDomainMeshSize', self, s_arg2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BSplineTransform_SetTransformDomainMeshSize`, 'returnType') = 'void'
attr(`BSplineTransform_SetTransformDomainMeshSize`, "inputTypes") = c('_p_itk__simple__BSplineTransform', 'integer')
class(`BSplineTransform_SetTransformDomainMeshSize`) = c("SWIGFunction", class('BSplineTransform_SetTransformDomainMeshSize'))

# Start of BSplineTransform_GetTransformDomainMeshSize

`BSplineTransform_GetTransformDomainMeshSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BSplineTransform_GetTransformDomainMeshSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BSplineTransform_GetTransformDomainMeshSize`, 'returnType') = 'integer'
attr(`BSplineTransform_GetTransformDomainMeshSize`, "inputTypes") = c('_p_itk__simple__BSplineTransform')
class(`BSplineTransform_GetTransformDomainMeshSize`) = c("SWIGFunction", class('BSplineTransform_GetTransformDomainMeshSize'))

# Start of BSplineTransform_SetTransformDomainOrigin

`BSplineTransform_SetTransformDomainOrigin` = function(self, s_arg2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  s_arg2 = as.numeric(s_arg2);
  ;ans = .Call('R_swig_BSplineTransform_SetTransformDomainOrigin', self, s_arg2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BSplineTransform_SetTransformDomainOrigin`, 'returnType') = 'void'
attr(`BSplineTransform_SetTransformDomainOrigin`, "inputTypes") = c('_p_itk__simple__BSplineTransform', '_p_std__vectorT_double_t')
class(`BSplineTransform_SetTransformDomainOrigin`) = c("SWIGFunction", class('BSplineTransform_SetTransformDomainOrigin'))

# Start of BSplineTransform_GetTransformDomainOrigin

`BSplineTransform_GetTransformDomainOrigin` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BSplineTransform_GetTransformDomainOrigin', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BSplineTransform_GetTransformDomainOrigin`, 'returnType') = 'numeric'
attr(`BSplineTransform_GetTransformDomainOrigin`, "inputTypes") = c('_p_itk__simple__BSplineTransform')
class(`BSplineTransform_GetTransformDomainOrigin`) = c("SWIGFunction", class('BSplineTransform_GetTransformDomainOrigin'))

# Start of BSplineTransform_SetTransformDomainPhysicalDimensions

`BSplineTransform_SetTransformDomainPhysicalDimensions` = function(self, s_arg2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  s_arg2 = as.numeric(s_arg2);
  ;ans = .Call('R_swig_BSplineTransform_SetTransformDomainPhysicalDimensions', self, s_arg2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BSplineTransform_SetTransformDomainPhysicalDimensions`, 'returnType') = 'void'
attr(`BSplineTransform_SetTransformDomainPhysicalDimensions`, "inputTypes") = c('_p_itk__simple__BSplineTransform', '_p_std__vectorT_double_t')
class(`BSplineTransform_SetTransformDomainPhysicalDimensions`) = c("SWIGFunction", class('BSplineTransform_SetTransformDomainPhysicalDimensions'))

# Start of BSplineTransform_GetTransformDomainPhysicalDimensions

`BSplineTransform_GetTransformDomainPhysicalDimensions` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BSplineTransform_GetTransformDomainPhysicalDimensions', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BSplineTransform_GetTransformDomainPhysicalDimensions`, 'returnType') = 'numeric'
attr(`BSplineTransform_GetTransformDomainPhysicalDimensions`, "inputTypes") = c('_p_itk__simple__BSplineTransform')
class(`BSplineTransform_GetTransformDomainPhysicalDimensions`) = c("SWIGFunction", class('BSplineTransform_GetTransformDomainPhysicalDimensions'))

# Start of BSplineTransform_GetCoefficientImages

`BSplineTransform_GetCoefficientImages` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BSplineTransform_GetCoefficientImages', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  ans=ImportSwigVec(ans);
  
  ans
  
}

attr(`BSplineTransform_GetCoefficientImages`, 'returnType') = 'list'
attr(`BSplineTransform_GetCoefficientImages`, "inputTypes") = c('_p_itk__simple__BSplineTransform')
class(`BSplineTransform_GetCoefficientImages`) = c("SWIGFunction", class('BSplineTransform_GetCoefficientImages'))

# Start of BSplineTransform_GetOrder

`BSplineTransform_GetOrder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BSplineTransform_GetOrder', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BSplineTransform_GetOrder`, 'returnType') = 'integer'
attr(`BSplineTransform_GetOrder`, "inputTypes") = c('_p_itk__simple__BSplineTransform')
class(`BSplineTransform_GetOrder`) = c("SWIGFunction", class('BSplineTransform_GetOrder'))

# Start of accessor method for itk::simple::BSplineTransform
setMethod('$', '_p_itk__simple__BSplineTransform', function(x, name)

{
  accessorFuns = list('Equal' = BSplineTransform_Equal, 'GetName' = BSplineTransform_GetName, 'SetTransformDomainDirection' = BSplineTransform_SetTransformDomainDirection, 'GetTransformDomainDirection' = BSplineTransform_GetTransformDomainDirection, 'SetTransformDomainMeshSize' = BSplineTransform_SetTransformDomainMeshSize, 'GetTransformDomainMeshSize' = BSplineTransform_GetTransformDomainMeshSize, 'SetTransformDomainOrigin' = BSplineTransform_SetTransformDomainOrigin, 'GetTransformDomainOrigin' = BSplineTransform_GetTransformDomainOrigin, 'SetTransformDomainPhysicalDimensions' = BSplineTransform_SetTransformDomainPhysicalDimensions, 'GetTransformDomainPhysicalDimensions' = BSplineTransform_GetTransformDomainPhysicalDimensions, 'GetCoefficientImages' = BSplineTransform_GetCoefficientImages, 'GetOrder' = BSplineTransform_GetOrder);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BSplineTransform
# Start of delete_DisplacementFieldTransform

`delete_DisplacementFieldTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_DisplacementFieldTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_DisplacementFieldTransform`, 'returnType') = 'void'
attr(`delete_DisplacementFieldTransform`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform')
class(`delete_DisplacementFieldTransform`) = c("SWIGFunction", class('delete_DisplacementFieldTransform'))

# Start of new_DisplacementFieldTransform

`DisplacementFieldTransform__SWIG_0` = function(dimensions)
{
  dimensions = as.integer(dimensions);
  
  if(length(dimensions) > 1) {
    warning("using only the first element of dimensions");
  };
  
  ;ans = .Call('R_swig_new_DisplacementFieldTransform__SWIG_0', dimensions, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DisplacementFieldTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DisplacementFieldTransform);
  ans
  
}

attr(`DisplacementFieldTransform__SWIG_0`, 'returnType') = '_p_itk__simple__DisplacementFieldTransform'
attr(`DisplacementFieldTransform__SWIG_0`, "inputTypes") = c('integer')
class(`DisplacementFieldTransform__SWIG_0`) = c("SWIGFunction", class('DisplacementFieldTransform__SWIG_0'))

# Start of new_DisplacementFieldTransform

`DisplacementFieldTransform__SWIG_1` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_DisplacementFieldTransform__SWIG_1', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DisplacementFieldTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DisplacementFieldTransform);
  ans
  
}

attr(`DisplacementFieldTransform__SWIG_1`, 'returnType') = '_p_itk__simple__DisplacementFieldTransform'
attr(`DisplacementFieldTransform__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`DisplacementFieldTransform__SWIG_1`) = c("SWIGFunction", class('DisplacementFieldTransform__SWIG_1'))

# Start of new_DisplacementFieldTransform

`DisplacementFieldTransform__SWIG_2` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_DisplacementFieldTransform__SWIG_2', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DisplacementFieldTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DisplacementFieldTransform);
  ans
  
}

attr(`DisplacementFieldTransform__SWIG_2`, 'returnType') = '_p_itk__simple__DisplacementFieldTransform'
attr(`DisplacementFieldTransform__SWIG_2`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform')
class(`DisplacementFieldTransform__SWIG_2`) = c("SWIGFunction", class('DisplacementFieldTransform__SWIG_2'))

# Start of new_DisplacementFieldTransform

`DisplacementFieldTransform__SWIG_3` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_DisplacementFieldTransform__SWIG_3', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DisplacementFieldTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DisplacementFieldTransform);
  ans
  
}

attr(`DisplacementFieldTransform__SWIG_3`, 'returnType') = '_p_itk__simple__DisplacementFieldTransform'
attr(`DisplacementFieldTransform__SWIG_3`, "inputTypes") = c('_p_itk__simple__Transform')
class(`DisplacementFieldTransform__SWIG_3`) = c("SWIGFunction", class('DisplacementFieldTransform__SWIG_3'))

`DisplacementFieldTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- DisplacementFieldTransform__SWIG_1; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__DisplacementFieldTransform') && length(argv[[1]]) == 1 ) {
      f <- DisplacementFieldTransform__SWIG_2; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- DisplacementFieldTransform__SWIG_3; 
    }
    else if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- DisplacementFieldTransform__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DisplacementFieldTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of DisplacementFieldTransform_Equal

`DisplacementFieldTransform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_DisplacementFieldTransform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DisplacementFieldTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DisplacementFieldTransform_Equal`, 'returnType') = '_p_itk__simple__DisplacementFieldTransform'
attr(`DisplacementFieldTransform_Equal`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform', '_p_itk__simple__DisplacementFieldTransform')
class(`DisplacementFieldTransform_Equal`) = c("SWIGFunction", class('DisplacementFieldTransform_Equal'))

# Start of DisplacementFieldTransform_GetName

`DisplacementFieldTransform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DisplacementFieldTransform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DisplacementFieldTransform_GetName`, 'returnType') = 'character'
attr(`DisplacementFieldTransform_GetName`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform')
class(`DisplacementFieldTransform_GetName`) = c("SWIGFunction", class('DisplacementFieldTransform_GetName'))

# Start of DisplacementFieldTransform_SetDisplacementField

`DisplacementFieldTransform_SetDisplacementField` = function(self, s_arg2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_DisplacementFieldTransform_SetDisplacementField', self, s_arg2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldTransform_SetDisplacementField`, 'returnType') = 'void'
attr(`DisplacementFieldTransform_SetDisplacementField`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform', '_p_itk__simple__Image')
class(`DisplacementFieldTransform_SetDisplacementField`) = c("SWIGFunction", class('DisplacementFieldTransform_SetDisplacementField'))

# Start of DisplacementFieldTransform_GetDisplacementField

`DisplacementFieldTransform_GetDisplacementField` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DisplacementFieldTransform_GetDisplacementField', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DisplacementFieldTransform_GetDisplacementField`, 'returnType') = '_p_itk__simple__Image'
attr(`DisplacementFieldTransform_GetDisplacementField`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform')
class(`DisplacementFieldTransform_GetDisplacementField`) = c("SWIGFunction", class('DisplacementFieldTransform_GetDisplacementField'))

# Start of DisplacementFieldTransform_SetInverseDisplacementField

`DisplacementFieldTransform_SetInverseDisplacementField` = function(self, s_arg2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_DisplacementFieldTransform_SetInverseDisplacementField', self, s_arg2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldTransform_SetInverseDisplacementField`, 'returnType') = 'void'
attr(`DisplacementFieldTransform_SetInverseDisplacementField`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform', '_p_itk__simple__Image')
class(`DisplacementFieldTransform_SetInverseDisplacementField`) = c("SWIGFunction", class('DisplacementFieldTransform_SetInverseDisplacementField'))

# Start of DisplacementFieldTransform_GetInverseDisplacementField

`DisplacementFieldTransform_GetInverseDisplacementField` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DisplacementFieldTransform_GetInverseDisplacementField', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DisplacementFieldTransform_GetInverseDisplacementField`, 'returnType') = '_p_itk__simple__Image'
attr(`DisplacementFieldTransform_GetInverseDisplacementField`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform')
class(`DisplacementFieldTransform_GetInverseDisplacementField`) = c("SWIGFunction", class('DisplacementFieldTransform_GetInverseDisplacementField'))

# Start of DisplacementFieldTransform_SetInterpolator

`DisplacementFieldTransform_SetInterpolator` = function(self, interp)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  interp = enumToInteger(interp, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interp) > 1) {
    warning("using only the first element of interp");
  };
  
  ;ans = .Call('R_swig_DisplacementFieldTransform_SetInterpolator', self, interp, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldTransform_SetInterpolator`, 'returnType') = 'void'
attr(`DisplacementFieldTransform_SetInterpolator`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform', 'character')
class(`DisplacementFieldTransform_SetInterpolator`) = c("SWIGFunction", class('DisplacementFieldTransform_SetInterpolator'))

# Start of DisplacementFieldTransform_SetSmoothingOff

`DisplacementFieldTransform_SetSmoothingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DisplacementFieldTransform_SetSmoothingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldTransform_SetSmoothingOff`, 'returnType') = 'void'
attr(`DisplacementFieldTransform_SetSmoothingOff`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform')
class(`DisplacementFieldTransform_SetSmoothingOff`) = c("SWIGFunction", class('DisplacementFieldTransform_SetSmoothingOff'))

# Start of DisplacementFieldTransform_SetSmoothingGaussianOnUpdate

`DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_0` = function(self, varianceForUpdateField, varianceForTotalField)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  
  ;ans = .Call('R_swig_DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_0', self, varianceForUpdateField, varianceForTotalField, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_0`, 'returnType') = 'void'
attr(`DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_0`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform', 'numeric', 'numeric')
class(`DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_0`) = c("SWIGFunction", class('DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_0'))

# Start of DisplacementFieldTransform_SetSmoothingGaussianOnUpdate

`DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_1` = function(self, varianceForUpdateField)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_1', self, varianceForUpdateField, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_1`, 'returnType') = 'void'
attr(`DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_1`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform', 'numeric')
class(`DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_1`) = c("SWIGFunction", class('DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_1'))

# Start of DisplacementFieldTransform_SetSmoothingGaussianOnUpdate

`DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_2` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_2', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_2`, 'returnType') = 'void'
attr(`DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_2`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform')
class(`DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_2`) = c("SWIGFunction", class('DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_2'))

`DisplacementFieldTransform_SetSmoothingGaussianOnUpdate` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__DisplacementFieldTransform') || is.null(argv[[1]]) ) {
      f <- DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__DisplacementFieldTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__DisplacementFieldTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- DisplacementFieldTransform_SetSmoothingGaussianOnUpdate__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DisplacementFieldTransform_SetSmoothingGaussianOnUpdate with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of DisplacementFieldTransform_SetSmoothingBSplineOnUpdate

`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_0` = function(self, numberOfControlPointsForUpdateField, numberOfControlPointsForTotalField, enforceStationaryBoundary, order)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfControlPointsForUpdateField = as.integer(numberOfControlPointsForUpdateField);
  numberOfControlPointsForTotalField = as.integer(numberOfControlPointsForTotalField);
  enforceStationaryBoundary = as.logical(enforceStationaryBoundary);
  order = as.integer(order);
  
  if(length(order) > 1) {
    warning("using only the first element of order");
  };
  
  ;ans = .Call('R_swig_DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_0', self, numberOfControlPointsForUpdateField, numberOfControlPointsForTotalField, enforceStationaryBoundary, order, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_0`, 'returnType') = 'void'
attr(`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_0`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform', 'integer', 'integer', 'logical', 'integer')
class(`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_0`) = c("SWIGFunction", class('DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_0'))

# Start of DisplacementFieldTransform_SetSmoothingBSplineOnUpdate

`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_1` = function(self, numberOfControlPointsForUpdateField, numberOfControlPointsForTotalField, enforceStationaryBoundary)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfControlPointsForUpdateField = as.integer(numberOfControlPointsForUpdateField);
  numberOfControlPointsForTotalField = as.integer(numberOfControlPointsForTotalField);
  enforceStationaryBoundary = as.logical(enforceStationaryBoundary);
  ;ans = .Call('R_swig_DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_1', self, numberOfControlPointsForUpdateField, numberOfControlPointsForTotalField, enforceStationaryBoundary, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_1`, 'returnType') = 'void'
attr(`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_1`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform', 'integer', 'integer', 'logical')
class(`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_1`) = c("SWIGFunction", class('DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_1'))

# Start of DisplacementFieldTransform_SetSmoothingBSplineOnUpdate

`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_2` = function(self, numberOfControlPointsForUpdateField, numberOfControlPointsForTotalField)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfControlPointsForUpdateField = as.integer(numberOfControlPointsForUpdateField);
  numberOfControlPointsForTotalField = as.integer(numberOfControlPointsForTotalField);
  ;ans = .Call('R_swig_DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_2', self, numberOfControlPointsForUpdateField, numberOfControlPointsForTotalField, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_2`, 'returnType') = 'void'
attr(`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_2`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform', 'integer', 'integer')
class(`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_2`) = c("SWIGFunction", class('DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_2'))

# Start of DisplacementFieldTransform_SetSmoothingBSplineOnUpdate

`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_3` = function(self, numberOfControlPointsForUpdateField)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfControlPointsForUpdateField = as.integer(numberOfControlPointsForUpdateField);
  ;ans = .Call('R_swig_DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_3', self, numberOfControlPointsForUpdateField, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_3`, 'returnType') = 'void'
attr(`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_3`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform', 'integer')
class(`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_3`) = c("SWIGFunction", class('DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_3'))

# Start of DisplacementFieldTransform_SetSmoothingBSplineOnUpdate

`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_4` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_4', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_4`, 'returnType') = 'void'
attr(`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_4`, "inputTypes") = c('_p_itk__simple__DisplacementFieldTransform')
class(`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_4`) = c("SWIGFunction", class('DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_4'))

`DisplacementFieldTransform_SetSmoothingBSplineOnUpdate` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__DisplacementFieldTransform') || is.null(argv[[1]]) ) {
      f <- DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__DisplacementFieldTransform') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__DisplacementFieldTransform') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__DisplacementFieldTransform') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__DisplacementFieldTransform') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- DisplacementFieldTransform_SetSmoothingBSplineOnUpdate__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DisplacementFieldTransform_SetSmoothingBSplineOnUpdate with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::DisplacementFieldTransform
setMethod('$', '_p_itk__simple__DisplacementFieldTransform', function(x, name)

{
  accessorFuns = list('Equal' = DisplacementFieldTransform_Equal, 'GetName' = DisplacementFieldTransform_GetName, 'SetDisplacementField' = DisplacementFieldTransform_SetDisplacementField, 'GetDisplacementField' = DisplacementFieldTransform_GetDisplacementField, 'SetInverseDisplacementField' = DisplacementFieldTransform_SetInverseDisplacementField, 'GetInverseDisplacementField' = DisplacementFieldTransform_GetInverseDisplacementField, 'SetInterpolator' = DisplacementFieldTransform_SetInterpolator, 'SetSmoothingOff' = DisplacementFieldTransform_SetSmoothingOff, 'SetSmoothingGaussianOnUpdate' = DisplacementFieldTransform_SetSmoothingGaussianOnUpdate, 'SetSmoothingBSplineOnUpdate' = DisplacementFieldTransform_SetSmoothingBSplineOnUpdate);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::DisplacementFieldTransform
# Start of delete_AffineTransform

`delete_AffineTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_AffineTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_AffineTransform`, 'returnType') = 'void'
attr(`delete_AffineTransform`, "inputTypes") = c('_p_itk__simple__AffineTransform')
class(`delete_AffineTransform`) = c("SWIGFunction", class('delete_AffineTransform'))

# Start of new_AffineTransform

`AffineTransform__SWIG_0` = function(dimensions)
{
  dimensions = as.integer(dimensions);
  
  if(length(dimensions) > 1) {
    warning("using only the first element of dimensions");
  };
  
  ;ans = .Call('R_swig_new_AffineTransform__SWIG_0', dimensions, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AffineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AffineTransform);
  ans
  
}

attr(`AffineTransform__SWIG_0`, 'returnType') = '_p_itk__simple__AffineTransform'
attr(`AffineTransform__SWIG_0`, "inputTypes") = c('integer')
class(`AffineTransform__SWIG_0`) = c("SWIGFunction", class('AffineTransform__SWIG_0'))

# Start of new_AffineTransform

`AffineTransform__SWIG_1` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_AffineTransform__SWIG_1', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AffineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AffineTransform);
  ans
  
}

attr(`AffineTransform__SWIG_1`, 'returnType') = '_p_itk__simple__AffineTransform'
attr(`AffineTransform__SWIG_1`, "inputTypes") = c('_p_itk__simple__AffineTransform')
class(`AffineTransform__SWIG_1`) = c("SWIGFunction", class('AffineTransform__SWIG_1'))

# Start of new_AffineTransform

`AffineTransform__SWIG_2` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_AffineTransform__SWIG_2', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AffineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AffineTransform);
  ans
  
}

attr(`AffineTransform__SWIG_2`, 'returnType') = '_p_itk__simple__AffineTransform'
attr(`AffineTransform__SWIG_2`, "inputTypes") = c('_p_itk__simple__Transform')
class(`AffineTransform__SWIG_2`) = c("SWIGFunction", class('AffineTransform__SWIG_2'))

# Start of new_AffineTransform

`AffineTransform__SWIG_3` = function(matrix, translation, fixedCenter)
{
  matrix = as.numeric(matrix);
  translation = as.numeric(translation);
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_AffineTransform__SWIG_3', matrix, translation, fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AffineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AffineTransform);
  ans
  
}

attr(`AffineTransform__SWIG_3`, 'returnType') = '_p_itk__simple__AffineTransform'
attr(`AffineTransform__SWIG_3`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`AffineTransform__SWIG_3`) = c("SWIGFunction", class('AffineTransform__SWIG_3'))

# Start of new_AffineTransform

`AffineTransform__SWIG_4` = function(matrix, translation)
{
  matrix = as.numeric(matrix);
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_new_AffineTransform__SWIG_4', matrix, translation, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AffineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AffineTransform);
  ans
  
}

attr(`AffineTransform__SWIG_4`, 'returnType') = '_p_itk__simple__AffineTransform'
attr(`AffineTransform__SWIG_4`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`AffineTransform__SWIG_4`) = c("SWIGFunction", class('AffineTransform__SWIG_4'))

`AffineTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__AffineTransform') && length(argv[[1]]) == 1 ) {
      f <- AffineTransform__SWIG_1; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- AffineTransform__SWIG_2; 
    }
    else if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- AffineTransform__SWIG_0; 
    }
  } else if (argc == 2) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- AffineTransform__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) )) {
      f <- AffineTransform__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for AffineTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of AffineTransform_Equal

`AffineTransform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_AffineTransform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AffineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AffineTransform_Equal`, 'returnType') = '_p_itk__simple__AffineTransform'
attr(`AffineTransform_Equal`, "inputTypes") = c('_p_itk__simple__AffineTransform', '_p_itk__simple__AffineTransform')
class(`AffineTransform_Equal`) = c("SWIGFunction", class('AffineTransform_Equal'))

# Start of AffineTransform_GetName

`AffineTransform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AffineTransform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AffineTransform_GetName`, 'returnType') = 'character'
attr(`AffineTransform_GetName`, "inputTypes") = c('_p_itk__simple__AffineTransform')
class(`AffineTransform_GetName`) = c("SWIGFunction", class('AffineTransform_GetName'))

# Start of AffineTransform_GetTranslation

`AffineTransform_GetTranslation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AffineTransform_GetTranslation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AffineTransform_GetTranslation`, 'returnType') = 'numeric'
attr(`AffineTransform_GetTranslation`, "inputTypes") = c('_p_itk__simple__AffineTransform')
class(`AffineTransform_GetTranslation`) = c("SWIGFunction", class('AffineTransform_GetTranslation'))

# Start of AffineTransform_SetTranslation

`AffineTransform_SetTranslation` = function(self, translation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_AffineTransform_SetTranslation', self, translation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AffineTransform_SetTranslation`, 'returnType') = 'void'
attr(`AffineTransform_SetTranslation`, "inputTypes") = c('_p_itk__simple__AffineTransform', '_p_std__vectorT_double_t')
class(`AffineTransform_SetTranslation`) = c("SWIGFunction", class('AffineTransform_SetTranslation'))

# Start of AffineTransform_SetMatrix

`AffineTransform_SetMatrix` = function(self, matrix)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  matrix = as.numeric(matrix);
  ;ans = .Call('R_swig_AffineTransform_SetMatrix', self, matrix, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AffineTransform_SetMatrix`, 'returnType') = 'void'
attr(`AffineTransform_SetMatrix`, "inputTypes") = c('_p_itk__simple__AffineTransform', '_p_std__vectorT_double_t')
class(`AffineTransform_SetMatrix`) = c("SWIGFunction", class('AffineTransform_SetMatrix'))

# Start of AffineTransform_GetMatrix

`AffineTransform_GetMatrix` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AffineTransform_GetMatrix', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AffineTransform_GetMatrix`, 'returnType') = 'numeric'
attr(`AffineTransform_GetMatrix`, "inputTypes") = c('_p_itk__simple__AffineTransform')
class(`AffineTransform_GetMatrix`) = c("SWIGFunction", class('AffineTransform_GetMatrix'))

# Start of AffineTransform_SetCenter

`AffineTransform_SetCenter` = function(self, params)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  params = as.numeric(params);
  ;ans = .Call('R_swig_AffineTransform_SetCenter', self, params, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AffineTransform_SetCenter`, 'returnType') = 'void'
attr(`AffineTransform_SetCenter`, "inputTypes") = c('_p_itk__simple__AffineTransform', '_p_std__vectorT_double_t')
class(`AffineTransform_SetCenter`) = c("SWIGFunction", class('AffineTransform_SetCenter'))

# Start of AffineTransform_GetCenter

`AffineTransform_GetCenter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AffineTransform_GetCenter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AffineTransform_GetCenter`, 'returnType') = 'numeric'
attr(`AffineTransform_GetCenter`, "inputTypes") = c('_p_itk__simple__AffineTransform')
class(`AffineTransform_GetCenter`) = c("SWIGFunction", class('AffineTransform_GetCenter'))

# Start of AffineTransform_Scale

`AffineTransform_Scale__SWIG_0` = function(self, factor, pre)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  factor = as.numeric(factor);
  pre = as.logical(pre);
  ;ans = .Call('R_swig_AffineTransform_Scale__SWIG_0', self, factor, pre, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AffineTransform_Scale__SWIG_0`, 'returnType') = 'void'
attr(`AffineTransform_Scale__SWIG_0`, "inputTypes") = c('_p_itk__simple__AffineTransform', '_p_std__vectorT_double_t', 'logical')
class(`AffineTransform_Scale__SWIG_0`) = c("SWIGFunction", class('AffineTransform_Scale__SWIG_0'))

# Start of AffineTransform_Scale

`AffineTransform_Scale__SWIG_1` = function(self, factor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  factor = as.numeric(factor);
  ;ans = .Call('R_swig_AffineTransform_Scale__SWIG_1', self, factor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AffineTransform_Scale__SWIG_1`, 'returnType') = 'void'
attr(`AffineTransform_Scale__SWIG_1`, "inputTypes") = c('_p_itk__simple__AffineTransform', '_p_std__vectorT_double_t')
class(`AffineTransform_Scale__SWIG_1`) = c("SWIGFunction", class('AffineTransform_Scale__SWIG_1'))

# Start of AffineTransform_Scale

`AffineTransform_Scale__SWIG_2` = function(self, factor, pre)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  pre = as.logical(pre);
  ;ans = .Call('R_swig_AffineTransform_Scale__SWIG_2', self, factor, pre, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AffineTransform_Scale__SWIG_2`, 'returnType') = 'void'
attr(`AffineTransform_Scale__SWIG_2`, "inputTypes") = c('_p_itk__simple__AffineTransform', 'numeric', 'logical')
class(`AffineTransform_Scale__SWIG_2`) = c("SWIGFunction", class('AffineTransform_Scale__SWIG_2'))

# Start of AffineTransform_Scale

`AffineTransform_Scale__SWIG_3` = function(self, factor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_AffineTransform_Scale__SWIG_3', self, factor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AffineTransform_Scale__SWIG_3`, 'returnType') = 'void'
attr(`AffineTransform_Scale__SWIG_3`, "inputTypes") = c('_p_itk__simple__AffineTransform', 'numeric')
class(`AffineTransform_Scale__SWIG_3`) = c("SWIGFunction", class('AffineTransform_Scale__SWIG_3'))

`AffineTransform_Scale` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__AffineTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- AffineTransform_Scale__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__AffineTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- AffineTransform_Scale__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__AffineTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- AffineTransform_Scale__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__AffineTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- AffineTransform_Scale__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for AffineTransform_Scale with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of AffineTransform_Shear

`AffineTransform_Shear__SWIG_0` = function(self, axis1, axis2, coef, pre)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  axis1 = as.integer(axis1);
  
  if(length(axis1) > 1) {
    warning("using only the first element of axis1");
  };
  
  axis2 = as.integer(axis2);
  
  if(length(axis2) > 1) {
    warning("using only the first element of axis2");
  };
  
  
  pre = as.logical(pre);
  ;ans = .Call('R_swig_AffineTransform_Shear__SWIG_0', self, axis1, axis2, coef, pre, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AffineTransform_Shear__SWIG_0`, 'returnType') = 'void'
attr(`AffineTransform_Shear__SWIG_0`, "inputTypes") = c('_p_itk__simple__AffineTransform', 'integer', 'integer', 'numeric', 'logical')
class(`AffineTransform_Shear__SWIG_0`) = c("SWIGFunction", class('AffineTransform_Shear__SWIG_0'))

# Start of AffineTransform_Shear

`AffineTransform_Shear__SWIG_1` = function(self, axis1, axis2, coef)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  axis1 = as.integer(axis1);
  
  if(length(axis1) > 1) {
    warning("using only the first element of axis1");
  };
  
  axis2 = as.integer(axis2);
  
  if(length(axis2) > 1) {
    warning("using only the first element of axis2");
  };
  
  
  ;ans = .Call('R_swig_AffineTransform_Shear__SWIG_1', self, axis1, axis2, coef, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AffineTransform_Shear__SWIG_1`, 'returnType') = 'void'
attr(`AffineTransform_Shear__SWIG_1`, "inputTypes") = c('_p_itk__simple__AffineTransform', 'integer', 'integer', 'numeric')
class(`AffineTransform_Shear__SWIG_1`) = c("SWIGFunction", class('AffineTransform_Shear__SWIG_1'))

`AffineTransform_Shear` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__AffineTransform') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- AffineTransform_Shear__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__AffineTransform') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- AffineTransform_Shear__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for AffineTransform_Shear with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of AffineTransform_Translate

`AffineTransform_Translate__SWIG_0` = function(self, offset, pre)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  offset = as.numeric(offset);
  pre = as.logical(pre);
  ;ans = .Call('R_swig_AffineTransform_Translate__SWIG_0', self, offset, pre, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AffineTransform_Translate__SWIG_0`, 'returnType') = 'void'
attr(`AffineTransform_Translate__SWIG_0`, "inputTypes") = c('_p_itk__simple__AffineTransform', '_p_std__vectorT_double_t', 'logical')
class(`AffineTransform_Translate__SWIG_0`) = c("SWIGFunction", class('AffineTransform_Translate__SWIG_0'))

# Start of AffineTransform_Translate

`AffineTransform_Translate__SWIG_1` = function(self, offset)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  offset = as.numeric(offset);
  ;ans = .Call('R_swig_AffineTransform_Translate__SWIG_1', self, offset, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AffineTransform_Translate__SWIG_1`, 'returnType') = 'void'
attr(`AffineTransform_Translate__SWIG_1`, "inputTypes") = c('_p_itk__simple__AffineTransform', '_p_std__vectorT_double_t')
class(`AffineTransform_Translate__SWIG_1`) = c("SWIGFunction", class('AffineTransform_Translate__SWIG_1'))

`AffineTransform_Translate` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__AffineTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- AffineTransform_Translate__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__AffineTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- AffineTransform_Translate__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for AffineTransform_Translate with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of AffineTransform_Rotate

`AffineTransform_Rotate__SWIG_0` = function(self, axis1, axis2, angle, pre)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  axis1 = as.integer(axis1);
  
  if(length(axis1) > 1) {
    warning("using only the first element of axis1");
  };
  
  axis2 = as.integer(axis2);
  
  if(length(axis2) > 1) {
    warning("using only the first element of axis2");
  };
  
  
  pre = as.logical(pre);
  ;ans = .Call('R_swig_AffineTransform_Rotate__SWIG_0', self, axis1, axis2, angle, pre, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AffineTransform_Rotate__SWIG_0`, 'returnType') = 'void'
attr(`AffineTransform_Rotate__SWIG_0`, "inputTypes") = c('_p_itk__simple__AffineTransform', 'integer', 'integer', 'numeric', 'logical')
class(`AffineTransform_Rotate__SWIG_0`) = c("SWIGFunction", class('AffineTransform_Rotate__SWIG_0'))

# Start of AffineTransform_Rotate

`AffineTransform_Rotate__SWIG_1` = function(self, axis1, axis2, angle)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  axis1 = as.integer(axis1);
  
  if(length(axis1) > 1) {
    warning("using only the first element of axis1");
  };
  
  axis2 = as.integer(axis2);
  
  if(length(axis2) > 1) {
    warning("using only the first element of axis2");
  };
  
  
  ;ans = .Call('R_swig_AffineTransform_Rotate__SWIG_1', self, axis1, axis2, angle, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AffineTransform_Rotate__SWIG_1`, 'returnType') = 'void'
attr(`AffineTransform_Rotate__SWIG_1`, "inputTypes") = c('_p_itk__simple__AffineTransform', 'integer', 'integer', 'numeric')
class(`AffineTransform_Rotate__SWIG_1`) = c("SWIGFunction", class('AffineTransform_Rotate__SWIG_1'))

`AffineTransform_Rotate` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__AffineTransform') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- AffineTransform_Rotate__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__AffineTransform') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- AffineTransform_Rotate__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for AffineTransform_Rotate with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::AffineTransform
setMethod('$', '_p_itk__simple__AffineTransform', function(x, name)

{
  accessorFuns = list('Equal' = AffineTransform_Equal, 'GetName' = AffineTransform_GetName, 'GetTranslation' = AffineTransform_GetTranslation, 'SetTranslation' = AffineTransform_SetTranslation, 'SetMatrix' = AffineTransform_SetMatrix, 'GetMatrix' = AffineTransform_GetMatrix, 'SetCenter' = AffineTransform_SetCenter, 'GetCenter' = AffineTransform_GetCenter, 'Scale' = AffineTransform_Scale, 'Shear' = AffineTransform_Shear, 'Translate' = AffineTransform_Translate, 'Rotate' = AffineTransform_Rotate);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::AffineTransform
# Start of delete_Euler3DTransform

`delete_Euler3DTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_Euler3DTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_Euler3DTransform`, 'returnType') = 'void'
attr(`delete_Euler3DTransform`, "inputTypes") = c('_p_itk__simple__Euler3DTransform')
class(`delete_Euler3DTransform`) = c("SWIGFunction", class('delete_Euler3DTransform'))

# Start of new_Euler3DTransform

`Euler3DTransform__SWIG_0` = function()
{
  ;ans = .Call('R_swig_new_Euler3DTransform__SWIG_0', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Euler3DTransform);
  ans
  
}

attr(`Euler3DTransform__SWIG_0`, 'returnType') = '_p_itk__simple__Euler3DTransform'
class(`Euler3DTransform__SWIG_0`) = c("SWIGFunction", class('Euler3DTransform__SWIG_0'))

# Start of new_Euler3DTransform

`Euler3DTransform__SWIG_1` = function(fixedCenter, angleX, angleY, angleZ, translation)
{
  fixedCenter = as.numeric(fixedCenter);
  
  
  
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_new_Euler3DTransform__SWIG_1', fixedCenter, angleX, angleY, angleZ, translation, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Euler3DTransform);
  ans
  
}

attr(`Euler3DTransform__SWIG_1`, 'returnType') = '_p_itk__simple__Euler3DTransform'
attr(`Euler3DTransform__SWIG_1`, "inputTypes") = c('_p_std__vectorT_double_t', 'numeric', 'numeric', 'numeric', '_p_std__vectorT_double_t')
class(`Euler3DTransform__SWIG_1`) = c("SWIGFunction", class('Euler3DTransform__SWIG_1'))

# Start of new_Euler3DTransform

`Euler3DTransform__SWIG_2` = function(fixedCenter, angleX, angleY, angleZ)
{
  fixedCenter = as.numeric(fixedCenter);
  
  
  
  ;ans = .Call('R_swig_new_Euler3DTransform__SWIG_2', fixedCenter, angleX, angleY, angleZ, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Euler3DTransform);
  ans
  
}

attr(`Euler3DTransform__SWIG_2`, 'returnType') = '_p_itk__simple__Euler3DTransform'
attr(`Euler3DTransform__SWIG_2`, "inputTypes") = c('_p_std__vectorT_double_t', 'numeric', 'numeric', 'numeric')
class(`Euler3DTransform__SWIG_2`) = c("SWIGFunction", class('Euler3DTransform__SWIG_2'))

# Start of new_Euler3DTransform

`Euler3DTransform__SWIG_3` = function(fixedCenter, angleX, angleY)
{
  fixedCenter = as.numeric(fixedCenter);
  
  
  ;ans = .Call('R_swig_new_Euler3DTransform__SWIG_3', fixedCenter, angleX, angleY, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Euler3DTransform);
  ans
  
}

attr(`Euler3DTransform__SWIG_3`, 'returnType') = '_p_itk__simple__Euler3DTransform'
attr(`Euler3DTransform__SWIG_3`, "inputTypes") = c('_p_std__vectorT_double_t', 'numeric', 'numeric')
class(`Euler3DTransform__SWIG_3`) = c("SWIGFunction", class('Euler3DTransform__SWIG_3'))

# Start of new_Euler3DTransform

`Euler3DTransform__SWIG_4` = function(fixedCenter, angleX)
{
  fixedCenter = as.numeric(fixedCenter);
  
  ;ans = .Call('R_swig_new_Euler3DTransform__SWIG_4', fixedCenter, angleX, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Euler3DTransform);
  ans
  
}

attr(`Euler3DTransform__SWIG_4`, 'returnType') = '_p_itk__simple__Euler3DTransform'
attr(`Euler3DTransform__SWIG_4`, "inputTypes") = c('_p_std__vectorT_double_t', 'numeric')
class(`Euler3DTransform__SWIG_4`) = c("SWIGFunction", class('Euler3DTransform__SWIG_4'))

# Start of new_Euler3DTransform

`Euler3DTransform__SWIG_5` = function(fixedCenter)
{
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_Euler3DTransform__SWIG_5', fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Euler3DTransform);
  ans
  
}

attr(`Euler3DTransform__SWIG_5`, 'returnType') = '_p_itk__simple__Euler3DTransform'
attr(`Euler3DTransform__SWIG_5`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`Euler3DTransform__SWIG_5`) = c("SWIGFunction", class('Euler3DTransform__SWIG_5'))

# Start of new_Euler3DTransform

`Euler3DTransform__SWIG_6` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_Euler3DTransform__SWIG_6', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Euler3DTransform);
  ans
  
}

attr(`Euler3DTransform__SWIG_6`, 'returnType') = '_p_itk__simple__Euler3DTransform'
attr(`Euler3DTransform__SWIG_6`, "inputTypes") = c('_p_itk__simple__Euler3DTransform')
class(`Euler3DTransform__SWIG_6`) = c("SWIGFunction", class('Euler3DTransform__SWIG_6'))

# Start of new_Euler3DTransform

`Euler3DTransform__SWIG_7` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_Euler3DTransform__SWIG_7', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Euler3DTransform);
  ans
  
}

attr(`Euler3DTransform__SWIG_7`, 'returnType') = '_p_itk__simple__Euler3DTransform'
attr(`Euler3DTransform__SWIG_7`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Euler3DTransform__SWIG_7`) = c("SWIGFunction", class('Euler3DTransform__SWIG_7'))

`Euler3DTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 8
  if (argc == 0) {
    f <- Euler3DTransform__SWIG_0; 
  } else if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Euler3DTransform') && length(argv[[1]]) == 1 ) {
      f <- Euler3DTransform__SWIG_6; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- Euler3DTransform__SWIG_7; 
    }
    else if ( is.numeric(argv[[1]]) ) {
      f <- Euler3DTransform__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Euler3DTransform__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Euler3DTransform__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- Euler3DTransform__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) )) {
      f <- Euler3DTransform__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Euler3DTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of Euler3DTransform_Equal

`Euler3DTransform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_Euler3DTransform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Euler3DTransform_Equal`, 'returnType') = '_p_itk__simple__Euler3DTransform'
attr(`Euler3DTransform_Equal`, "inputTypes") = c('_p_itk__simple__Euler3DTransform', '_p_itk__simple__Euler3DTransform')
class(`Euler3DTransform_Equal`) = c("SWIGFunction", class('Euler3DTransform_Equal'))

# Start of Euler3DTransform_GetName

`Euler3DTransform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Euler3DTransform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Euler3DTransform_GetName`, 'returnType') = 'character'
attr(`Euler3DTransform_GetName`, "inputTypes") = c('_p_itk__simple__Euler3DTransform')
class(`Euler3DTransform_GetName`) = c("SWIGFunction", class('Euler3DTransform_GetName'))

# Start of Euler3DTransform_SetCenter

`Euler3DTransform_SetCenter` = function(self, params)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  params = as.numeric(params);
  ;ans = .Call('R_swig_Euler3DTransform_SetCenter', self, params, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Euler3DTransform_SetCenter`, 'returnType') = 'void'
attr(`Euler3DTransform_SetCenter`, "inputTypes") = c('_p_itk__simple__Euler3DTransform', '_p_std__vectorT_double_t')
class(`Euler3DTransform_SetCenter`) = c("SWIGFunction", class('Euler3DTransform_SetCenter'))

# Start of Euler3DTransform_GetCenter

`Euler3DTransform_GetCenter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Euler3DTransform_GetCenter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Euler3DTransform_GetCenter`, 'returnType') = 'numeric'
attr(`Euler3DTransform_GetCenter`, "inputTypes") = c('_p_itk__simple__Euler3DTransform')
class(`Euler3DTransform_GetCenter`) = c("SWIGFunction", class('Euler3DTransform_GetCenter'))

# Start of Euler3DTransform_GetAngleX

`Euler3DTransform_GetAngleX` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Euler3DTransform_GetAngleX', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Euler3DTransform_GetAngleX`, 'returnType') = 'numeric'
attr(`Euler3DTransform_GetAngleX`, "inputTypes") = c('_p_itk__simple__Euler3DTransform')
class(`Euler3DTransform_GetAngleX`) = c("SWIGFunction", class('Euler3DTransform_GetAngleX'))

# Start of Euler3DTransform_GetAngleY

`Euler3DTransform_GetAngleY` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Euler3DTransform_GetAngleY', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Euler3DTransform_GetAngleY`, 'returnType') = 'numeric'
attr(`Euler3DTransform_GetAngleY`, "inputTypes") = c('_p_itk__simple__Euler3DTransform')
class(`Euler3DTransform_GetAngleY`) = c("SWIGFunction", class('Euler3DTransform_GetAngleY'))

# Start of Euler3DTransform_GetAngleZ

`Euler3DTransform_GetAngleZ` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Euler3DTransform_GetAngleZ', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Euler3DTransform_GetAngleZ`, 'returnType') = 'numeric'
attr(`Euler3DTransform_GetAngleZ`, "inputTypes") = c('_p_itk__simple__Euler3DTransform')
class(`Euler3DTransform_GetAngleZ`) = c("SWIGFunction", class('Euler3DTransform_GetAngleZ'))

# Start of Euler3DTransform_SetRotation

`Euler3DTransform_SetRotation` = function(self, angleX, angleY, angleZ)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  
  
  ;ans = .Call('R_swig_Euler3DTransform_SetRotation', self, angleX, angleY, angleZ, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Euler3DTransform_SetRotation`, 'returnType') = 'void'
attr(`Euler3DTransform_SetRotation`, "inputTypes") = c('_p_itk__simple__Euler3DTransform', 'numeric', 'numeric', 'numeric')
class(`Euler3DTransform_SetRotation`) = c("SWIGFunction", class('Euler3DTransform_SetRotation'))

# Start of Euler3DTransform_GetTranslation

`Euler3DTransform_GetTranslation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Euler3DTransform_GetTranslation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Euler3DTransform_GetTranslation`, 'returnType') = 'numeric'
attr(`Euler3DTransform_GetTranslation`, "inputTypes") = c('_p_itk__simple__Euler3DTransform')
class(`Euler3DTransform_GetTranslation`) = c("SWIGFunction", class('Euler3DTransform_GetTranslation'))

# Start of Euler3DTransform_SetTranslation

`Euler3DTransform_SetTranslation` = function(self, translation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_Euler3DTransform_SetTranslation', self, translation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Euler3DTransform_SetTranslation`, 'returnType') = 'void'
attr(`Euler3DTransform_SetTranslation`, "inputTypes") = c('_p_itk__simple__Euler3DTransform', '_p_std__vectorT_double_t')
class(`Euler3DTransform_SetTranslation`) = c("SWIGFunction", class('Euler3DTransform_SetTranslation'))

# Start of Euler3DTransform_SetComputeZYX

`Euler3DTransform_SetComputeZYX` = function(self, s_arg)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  s_arg = as.logical(s_arg);
  ;ans = .Call('R_swig_Euler3DTransform_SetComputeZYX', self, s_arg, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Euler3DTransform_SetComputeZYX`, 'returnType') = 'void'
attr(`Euler3DTransform_SetComputeZYX`, "inputTypes") = c('_p_itk__simple__Euler3DTransform', 'logical')
class(`Euler3DTransform_SetComputeZYX`) = c("SWIGFunction", class('Euler3DTransform_SetComputeZYX'))

# Start of Euler3DTransform_GetComputeZYX

`Euler3DTransform_GetComputeZYX` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Euler3DTransform_GetComputeZYX', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Euler3DTransform_GetComputeZYX`, 'returnType') = 'logical'
attr(`Euler3DTransform_GetComputeZYX`, "inputTypes") = c('_p_itk__simple__Euler3DTransform')
class(`Euler3DTransform_GetComputeZYX`) = c("SWIGFunction", class('Euler3DTransform_GetComputeZYX'))

# Start of Euler3DTransform_ComputeZYXOn

`Euler3DTransform_ComputeZYXOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_Euler3DTransform_ComputeZYXOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Euler3DTransform_ComputeZYXOn`, 'returnType') = 'void'
attr(`Euler3DTransform_ComputeZYXOn`, "inputTypes") = c('_p_itk__simple__Euler3DTransform')
class(`Euler3DTransform_ComputeZYXOn`) = c("SWIGFunction", class('Euler3DTransform_ComputeZYXOn'))

# Start of Euler3DTransform_ComputeZYXOff

`Euler3DTransform_ComputeZYXOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_Euler3DTransform_ComputeZYXOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Euler3DTransform_ComputeZYXOff`, 'returnType') = 'void'
attr(`Euler3DTransform_ComputeZYXOff`, "inputTypes") = c('_p_itk__simple__Euler3DTransform')
class(`Euler3DTransform_ComputeZYXOff`) = c("SWIGFunction", class('Euler3DTransform_ComputeZYXOff'))

# Start of Euler3DTransform_GetMatrix

`Euler3DTransform_GetMatrix` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Euler3DTransform_GetMatrix', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Euler3DTransform_GetMatrix`, 'returnType') = 'numeric'
attr(`Euler3DTransform_GetMatrix`, "inputTypes") = c('_p_itk__simple__Euler3DTransform')
class(`Euler3DTransform_GetMatrix`) = c("SWIGFunction", class('Euler3DTransform_GetMatrix'))

# Start of Euler3DTransform_SetMatrix

`Euler3DTransform_SetMatrix__SWIG_0` = function(self, matrix, tolerance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  matrix = as.numeric(matrix);
  
  ;ans = .Call('R_swig_Euler3DTransform_SetMatrix__SWIG_0', self, matrix, tolerance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Euler3DTransform_SetMatrix__SWIG_0`, 'returnType') = 'void'
attr(`Euler3DTransform_SetMatrix__SWIG_0`, "inputTypes") = c('_p_itk__simple__Euler3DTransform', '_p_std__vectorT_double_t', 'numeric')
class(`Euler3DTransform_SetMatrix__SWIG_0`) = c("SWIGFunction", class('Euler3DTransform_SetMatrix__SWIG_0'))

# Start of Euler3DTransform_SetMatrix

`Euler3DTransform_SetMatrix__SWIG_1` = function(self, matrix)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  matrix = as.numeric(matrix);
  ;ans = .Call('R_swig_Euler3DTransform_SetMatrix__SWIG_1', self, matrix, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Euler3DTransform_SetMatrix__SWIG_1`, 'returnType') = 'void'
attr(`Euler3DTransform_SetMatrix__SWIG_1`, "inputTypes") = c('_p_itk__simple__Euler3DTransform', '_p_std__vectorT_double_t')
class(`Euler3DTransform_SetMatrix__SWIG_1`) = c("SWIGFunction", class('Euler3DTransform_SetMatrix__SWIG_1'))

`Euler3DTransform_SetMatrix` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Euler3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- Euler3DTransform_SetMatrix__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Euler3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Euler3DTransform_SetMatrix__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Euler3DTransform_SetMatrix with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::Euler3DTransform
setMethod('$', '_p_itk__simple__Euler3DTransform', function(x, name)

{
  accessorFuns = list('Equal' = Euler3DTransform_Equal, 'GetName' = Euler3DTransform_GetName, 'SetCenter' = Euler3DTransform_SetCenter, 'GetCenter' = Euler3DTransform_GetCenter, 'GetAngleX' = Euler3DTransform_GetAngleX, 'GetAngleY' = Euler3DTransform_GetAngleY, 'GetAngleZ' = Euler3DTransform_GetAngleZ, 'SetRotation' = Euler3DTransform_SetRotation, 'GetTranslation' = Euler3DTransform_GetTranslation, 'SetTranslation' = Euler3DTransform_SetTranslation, 'SetComputeZYX' = Euler3DTransform_SetComputeZYX, 'GetComputeZYX' = Euler3DTransform_GetComputeZYX, 'ComputeZYXOn' = Euler3DTransform_ComputeZYXOn, 'ComputeZYXOff' = Euler3DTransform_ComputeZYXOff, 'GetMatrix' = Euler3DTransform_GetMatrix, 'SetMatrix' = Euler3DTransform_SetMatrix);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::Euler3DTransform
# Start of delete_Euler2DTransform

`delete_Euler2DTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_Euler2DTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_Euler2DTransform`, 'returnType') = 'void'
attr(`delete_Euler2DTransform`, "inputTypes") = c('_p_itk__simple__Euler2DTransform')
class(`delete_Euler2DTransform`) = c("SWIGFunction", class('delete_Euler2DTransform'))

# Start of new_Euler2DTransform

`Euler2DTransform__SWIG_0` = function()
{
  ;ans = .Call('R_swig_new_Euler2DTransform__SWIG_0', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler2DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Euler2DTransform);
  ans
  
}

attr(`Euler2DTransform__SWIG_0`, 'returnType') = '_p_itk__simple__Euler2DTransform'
class(`Euler2DTransform__SWIG_0`) = c("SWIGFunction", class('Euler2DTransform__SWIG_0'))

# Start of new_Euler2DTransform

`Euler2DTransform__SWIG_1` = function(fixedCenter, angle, translation)
{
  fixedCenter = as.numeric(fixedCenter);
  
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_new_Euler2DTransform__SWIG_1', fixedCenter, angle, translation, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler2DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Euler2DTransform);
  ans
  
}

attr(`Euler2DTransform__SWIG_1`, 'returnType') = '_p_itk__simple__Euler2DTransform'
attr(`Euler2DTransform__SWIG_1`, "inputTypes") = c('_p_std__vectorT_double_t', 'numeric', '_p_std__vectorT_double_t')
class(`Euler2DTransform__SWIG_1`) = c("SWIGFunction", class('Euler2DTransform__SWIG_1'))

# Start of new_Euler2DTransform

`Euler2DTransform__SWIG_2` = function(fixedCenter, angle)
{
  fixedCenter = as.numeric(fixedCenter);
  
  ;ans = .Call('R_swig_new_Euler2DTransform__SWIG_2', fixedCenter, angle, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler2DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Euler2DTransform);
  ans
  
}

attr(`Euler2DTransform__SWIG_2`, 'returnType') = '_p_itk__simple__Euler2DTransform'
attr(`Euler2DTransform__SWIG_2`, "inputTypes") = c('_p_std__vectorT_double_t', 'numeric')
class(`Euler2DTransform__SWIG_2`) = c("SWIGFunction", class('Euler2DTransform__SWIG_2'))

# Start of new_Euler2DTransform

`Euler2DTransform__SWIG_3` = function(fixedCenter)
{
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_Euler2DTransform__SWIG_3', fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler2DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Euler2DTransform);
  ans
  
}

attr(`Euler2DTransform__SWIG_3`, 'returnType') = '_p_itk__simple__Euler2DTransform'
attr(`Euler2DTransform__SWIG_3`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`Euler2DTransform__SWIG_3`) = c("SWIGFunction", class('Euler2DTransform__SWIG_3'))

# Start of new_Euler2DTransform

`Euler2DTransform__SWIG_4` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_Euler2DTransform__SWIG_4', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler2DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Euler2DTransform);
  ans
  
}

attr(`Euler2DTransform__SWIG_4`, 'returnType') = '_p_itk__simple__Euler2DTransform'
attr(`Euler2DTransform__SWIG_4`, "inputTypes") = c('_p_itk__simple__Euler2DTransform')
class(`Euler2DTransform__SWIG_4`) = c("SWIGFunction", class('Euler2DTransform__SWIG_4'))

# Start of new_Euler2DTransform

`Euler2DTransform__SWIG_5` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_Euler2DTransform__SWIG_5', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler2DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Euler2DTransform);
  ans
  
}

attr(`Euler2DTransform__SWIG_5`, 'returnType') = '_p_itk__simple__Euler2DTransform'
attr(`Euler2DTransform__SWIG_5`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Euler2DTransform__SWIG_5`) = c("SWIGFunction", class('Euler2DTransform__SWIG_5'))

`Euler2DTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 0) {
    f <- Euler2DTransform__SWIG_0; 
  } else if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Euler2DTransform') && length(argv[[1]]) == 1 ) {
      f <- Euler2DTransform__SWIG_4; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- Euler2DTransform__SWIG_5; 
    }
    else if ( is.numeric(argv[[1]]) ) {
      f <- Euler2DTransform__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Euler2DTransform__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) )) {
      f <- Euler2DTransform__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Euler2DTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of Euler2DTransform_Equal

`Euler2DTransform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_Euler2DTransform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Euler2DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Euler2DTransform_Equal`, 'returnType') = '_p_itk__simple__Euler2DTransform'
attr(`Euler2DTransform_Equal`, "inputTypes") = c('_p_itk__simple__Euler2DTransform', '_p_itk__simple__Euler2DTransform')
class(`Euler2DTransform_Equal`) = c("SWIGFunction", class('Euler2DTransform_Equal'))

# Start of Euler2DTransform_GetName

`Euler2DTransform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Euler2DTransform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Euler2DTransform_GetName`, 'returnType') = 'character'
attr(`Euler2DTransform_GetName`, "inputTypes") = c('_p_itk__simple__Euler2DTransform')
class(`Euler2DTransform_GetName`) = c("SWIGFunction", class('Euler2DTransform_GetName'))

# Start of Euler2DTransform_SetCenter

`Euler2DTransform_SetCenter` = function(self, params)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  params = as.numeric(params);
  ;ans = .Call('R_swig_Euler2DTransform_SetCenter', self, params, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Euler2DTransform_SetCenter`, 'returnType') = 'void'
attr(`Euler2DTransform_SetCenter`, "inputTypes") = c('_p_itk__simple__Euler2DTransform', '_p_std__vectorT_double_t')
class(`Euler2DTransform_SetCenter`) = c("SWIGFunction", class('Euler2DTransform_SetCenter'))

# Start of Euler2DTransform_GetCenter

`Euler2DTransform_GetCenter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Euler2DTransform_GetCenter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Euler2DTransform_GetCenter`, 'returnType') = 'numeric'
attr(`Euler2DTransform_GetCenter`, "inputTypes") = c('_p_itk__simple__Euler2DTransform')
class(`Euler2DTransform_GetCenter`) = c("SWIGFunction", class('Euler2DTransform_GetCenter'))

# Start of Euler2DTransform_SetAngle

`Euler2DTransform_SetAngle` = function(self, angle)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_Euler2DTransform_SetAngle', self, angle, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Euler2DTransform_SetAngle`, 'returnType') = 'void'
attr(`Euler2DTransform_SetAngle`, "inputTypes") = c('_p_itk__simple__Euler2DTransform', 'numeric')
class(`Euler2DTransform_SetAngle`) = c("SWIGFunction", class('Euler2DTransform_SetAngle'))

# Start of Euler2DTransform_GetAngle

`Euler2DTransform_GetAngle` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Euler2DTransform_GetAngle', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Euler2DTransform_GetAngle`, 'returnType') = 'numeric'
attr(`Euler2DTransform_GetAngle`, "inputTypes") = c('_p_itk__simple__Euler2DTransform')
class(`Euler2DTransform_GetAngle`) = c("SWIGFunction", class('Euler2DTransform_GetAngle'))

# Start of Euler2DTransform_GetTranslation

`Euler2DTransform_GetTranslation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Euler2DTransform_GetTranslation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Euler2DTransform_GetTranslation`, 'returnType') = 'numeric'
attr(`Euler2DTransform_GetTranslation`, "inputTypes") = c('_p_itk__simple__Euler2DTransform')
class(`Euler2DTransform_GetTranslation`) = c("SWIGFunction", class('Euler2DTransform_GetTranslation'))

# Start of Euler2DTransform_SetTranslation

`Euler2DTransform_SetTranslation` = function(self, translation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_Euler2DTransform_SetTranslation', self, translation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Euler2DTransform_SetTranslation`, 'returnType') = 'void'
attr(`Euler2DTransform_SetTranslation`, "inputTypes") = c('_p_itk__simple__Euler2DTransform', '_p_std__vectorT_double_t')
class(`Euler2DTransform_SetTranslation`) = c("SWIGFunction", class('Euler2DTransform_SetTranslation'))

# Start of Euler2DTransform_GetMatrix

`Euler2DTransform_GetMatrix` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Euler2DTransform_GetMatrix', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Euler2DTransform_GetMatrix`, 'returnType') = 'numeric'
attr(`Euler2DTransform_GetMatrix`, "inputTypes") = c('_p_itk__simple__Euler2DTransform')
class(`Euler2DTransform_GetMatrix`) = c("SWIGFunction", class('Euler2DTransform_GetMatrix'))

# Start of Euler2DTransform_SetMatrix

`Euler2DTransform_SetMatrix__SWIG_0` = function(self, matrix, tolerance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  matrix = as.numeric(matrix);
  
  ;ans = .Call('R_swig_Euler2DTransform_SetMatrix__SWIG_0', self, matrix, tolerance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Euler2DTransform_SetMatrix__SWIG_0`, 'returnType') = 'void'
attr(`Euler2DTransform_SetMatrix__SWIG_0`, "inputTypes") = c('_p_itk__simple__Euler2DTransform', '_p_std__vectorT_double_t', 'numeric')
class(`Euler2DTransform_SetMatrix__SWIG_0`) = c("SWIGFunction", class('Euler2DTransform_SetMatrix__SWIG_0'))

# Start of Euler2DTransform_SetMatrix

`Euler2DTransform_SetMatrix__SWIG_1` = function(self, matrix)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  matrix = as.numeric(matrix);
  ;ans = .Call('R_swig_Euler2DTransform_SetMatrix__SWIG_1', self, matrix, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Euler2DTransform_SetMatrix__SWIG_1`, 'returnType') = 'void'
attr(`Euler2DTransform_SetMatrix__SWIG_1`, "inputTypes") = c('_p_itk__simple__Euler2DTransform', '_p_std__vectorT_double_t')
class(`Euler2DTransform_SetMatrix__SWIG_1`) = c("SWIGFunction", class('Euler2DTransform_SetMatrix__SWIG_1'))

`Euler2DTransform_SetMatrix` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Euler2DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- Euler2DTransform_SetMatrix__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Euler2DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Euler2DTransform_SetMatrix__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Euler2DTransform_SetMatrix with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::Euler2DTransform
setMethod('$', '_p_itk__simple__Euler2DTransform', function(x, name)

{
  accessorFuns = list('Equal' = Euler2DTransform_Equal, 'GetName' = Euler2DTransform_GetName, 'SetCenter' = Euler2DTransform_SetCenter, 'GetCenter' = Euler2DTransform_GetCenter, 'SetAngle' = Euler2DTransform_SetAngle, 'GetAngle' = Euler2DTransform_GetAngle, 'GetTranslation' = Euler2DTransform_GetTranslation, 'SetTranslation' = Euler2DTransform_SetTranslation, 'GetMatrix' = Euler2DTransform_GetMatrix, 'SetMatrix' = Euler2DTransform_SetMatrix);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::Euler2DTransform
# Start of delete_ScaleTransform

`delete_ScaleTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ScaleTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ScaleTransform`, 'returnType') = 'void'
attr(`delete_ScaleTransform`, "inputTypes") = c('_p_itk__simple__ScaleTransform')
class(`delete_ScaleTransform`) = c("SWIGFunction", class('delete_ScaleTransform'))

# Start of new_ScaleTransform

`ScaleTransform__SWIG_0` = function(dimensions, scale)
{
  dimensions = as.integer(dimensions);
  
  if(length(dimensions) > 1) {
    warning("using only the first element of dimensions");
  };
  
  scale = as.numeric(scale);
  ;ans = .Call('R_swig_new_ScaleTransform__SWIG_0', dimensions, scale, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleTransform);
  ans
  
}

attr(`ScaleTransform__SWIG_0`, 'returnType') = '_p_itk__simple__ScaleTransform'
attr(`ScaleTransform__SWIG_0`, "inputTypes") = c('integer', '_p_std__vectorT_double_t')
class(`ScaleTransform__SWIG_0`) = c("SWIGFunction", class('ScaleTransform__SWIG_0'))

# Start of new_ScaleTransform

`ScaleTransform__SWIG_1` = function(dimensions)
{
  dimensions = as.integer(dimensions);
  
  if(length(dimensions) > 1) {
    warning("using only the first element of dimensions");
  };
  
  ;ans = .Call('R_swig_new_ScaleTransform__SWIG_1', dimensions, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleTransform);
  ans
  
}

attr(`ScaleTransform__SWIG_1`, 'returnType') = '_p_itk__simple__ScaleTransform'
attr(`ScaleTransform__SWIG_1`, "inputTypes") = c('integer')
class(`ScaleTransform__SWIG_1`) = c("SWIGFunction", class('ScaleTransform__SWIG_1'))

# Start of new_ScaleTransform

`ScaleTransform__SWIG_2` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_ScaleTransform__SWIG_2', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleTransform);
  ans
  
}

attr(`ScaleTransform__SWIG_2`, 'returnType') = '_p_itk__simple__ScaleTransform'
attr(`ScaleTransform__SWIG_2`, "inputTypes") = c('_p_itk__simple__ScaleTransform')
class(`ScaleTransform__SWIG_2`) = c("SWIGFunction", class('ScaleTransform__SWIG_2'))

# Start of new_ScaleTransform

`ScaleTransform__SWIG_3` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_ScaleTransform__SWIG_3', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleTransform);
  ans
  
}

attr(`ScaleTransform__SWIG_3`, 'returnType') = '_p_itk__simple__ScaleTransform'
attr(`ScaleTransform__SWIG_3`, "inputTypes") = c('_p_itk__simple__Transform')
class(`ScaleTransform__SWIG_3`) = c("SWIGFunction", class('ScaleTransform__SWIG_3'))

`ScaleTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__ScaleTransform') && length(argv[[1]]) == 1 ) {
      f <- ScaleTransform__SWIG_2; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- ScaleTransform__SWIG_3; 
    }
    else if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- ScaleTransform__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) )) {
      f <- ScaleTransform__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ScaleTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ScaleTransform_GetName

`ScaleTransform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleTransform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleTransform_GetName`, 'returnType') = 'character'
attr(`ScaleTransform_GetName`, "inputTypes") = c('_p_itk__simple__ScaleTransform')
class(`ScaleTransform_GetName`) = c("SWIGFunction", class('ScaleTransform_GetName'))

# Start of ScaleTransform_Equal

`ScaleTransform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_ScaleTransform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScaleTransform_Equal`, 'returnType') = '_p_itk__simple__ScaleTransform'
attr(`ScaleTransform_Equal`, "inputTypes") = c('_p_itk__simple__ScaleTransform', '_p_itk__simple__ScaleTransform')
class(`ScaleTransform_Equal`) = c("SWIGFunction", class('ScaleTransform_Equal'))

# Start of ScaleTransform_SetScale

`ScaleTransform_SetScale` = function(self, params)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  params = as.numeric(params);
  ;ans = .Call('R_swig_ScaleTransform_SetScale', self, params, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScaleTransform_SetScale`, 'returnType') = 'void'
attr(`ScaleTransform_SetScale`, "inputTypes") = c('_p_itk__simple__ScaleTransform', '_p_std__vectorT_double_t')
class(`ScaleTransform_SetScale`) = c("SWIGFunction", class('ScaleTransform_SetScale'))

# Start of ScaleTransform_GetScale

`ScaleTransform_GetScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleTransform_GetScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleTransform_GetScale`, 'returnType') = 'numeric'
attr(`ScaleTransform_GetScale`, "inputTypes") = c('_p_itk__simple__ScaleTransform')
class(`ScaleTransform_GetScale`) = c("SWIGFunction", class('ScaleTransform_GetScale'))

# Start of ScaleTransform_SetCenter

`ScaleTransform_SetCenter` = function(self, params)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  params = as.numeric(params);
  ;ans = .Call('R_swig_ScaleTransform_SetCenter', self, params, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScaleTransform_SetCenter`, 'returnType') = 'void'
attr(`ScaleTransform_SetCenter`, "inputTypes") = c('_p_itk__simple__ScaleTransform', '_p_std__vectorT_double_t')
class(`ScaleTransform_SetCenter`) = c("SWIGFunction", class('ScaleTransform_SetCenter'))

# Start of ScaleTransform_GetCenter

`ScaleTransform_GetCenter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleTransform_GetCenter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleTransform_GetCenter`, 'returnType') = 'numeric'
attr(`ScaleTransform_GetCenter`, "inputTypes") = c('_p_itk__simple__ScaleTransform')
class(`ScaleTransform_GetCenter`) = c("SWIGFunction", class('ScaleTransform_GetCenter'))

# Start of ScaleTransform_GetMatrix

`ScaleTransform_GetMatrix` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleTransform_GetMatrix', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleTransform_GetMatrix`, 'returnType') = 'numeric'
attr(`ScaleTransform_GetMatrix`, "inputTypes") = c('_p_itk__simple__ScaleTransform')
class(`ScaleTransform_GetMatrix`) = c("SWIGFunction", class('ScaleTransform_GetMatrix'))

# Start of accessor method for itk::simple::ScaleTransform
setMethod('$', '_p_itk__simple__ScaleTransform', function(x, name)

{
  accessorFuns = list('GetName' = ScaleTransform_GetName, 'Equal' = ScaleTransform_Equal, 'SetScale' = ScaleTransform_SetScale, 'GetScale' = ScaleTransform_GetScale, 'SetCenter' = ScaleTransform_SetCenter, 'GetCenter' = ScaleTransform_GetCenter, 'GetMatrix' = ScaleTransform_GetMatrix);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ScaleTransform
# Start of delete_ScaleSkewVersor3DTransform

`delete_ScaleSkewVersor3DTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ScaleSkewVersor3DTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ScaleSkewVersor3DTransform`, 'returnType') = 'void'
attr(`delete_ScaleSkewVersor3DTransform`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform')
class(`delete_ScaleSkewVersor3DTransform`) = c("SWIGFunction", class('delete_ScaleSkewVersor3DTransform'))

# Start of new_ScaleSkewVersor3DTransform

`ScaleSkewVersor3DTransform__SWIG_0` = function()
{
  ;ans = .Call('R_swig_new_ScaleSkewVersor3DTransform__SWIG_0', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleSkewVersor3DTransform);
  ans
  
}

attr(`ScaleSkewVersor3DTransform__SWIG_0`, 'returnType') = '_p_itk__simple__ScaleSkewVersor3DTransform'
class(`ScaleSkewVersor3DTransform__SWIG_0`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform__SWIG_0'))

# Start of new_ScaleSkewVersor3DTransform

`ScaleSkewVersor3DTransform__SWIG_1` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_ScaleSkewVersor3DTransform__SWIG_1', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleSkewVersor3DTransform);
  ans
  
}

attr(`ScaleSkewVersor3DTransform__SWIG_1`, 'returnType') = '_p_itk__simple__ScaleSkewVersor3DTransform'
attr(`ScaleSkewVersor3DTransform__SWIG_1`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform')
class(`ScaleSkewVersor3DTransform__SWIG_1`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform__SWIG_1'))

# Start of new_ScaleSkewVersor3DTransform

`ScaleSkewVersor3DTransform__SWIG_2` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_ScaleSkewVersor3DTransform__SWIG_2', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleSkewVersor3DTransform);
  ans
  
}

attr(`ScaleSkewVersor3DTransform__SWIG_2`, 'returnType') = '_p_itk__simple__ScaleSkewVersor3DTransform'
attr(`ScaleSkewVersor3DTransform__SWIG_2`, "inputTypes") = c('_p_itk__simple__Transform')
class(`ScaleSkewVersor3DTransform__SWIG_2`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform__SWIG_2'))

# Start of new_ScaleSkewVersor3DTransform

`ScaleSkewVersor3DTransform__SWIG_3` = function(scale, skew, versor, translation, fixedCenter)
{
  scale = as.numeric(scale);
  skew = as.numeric(skew);
  versor = as.numeric(versor);
  translation = as.numeric(translation);
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_ScaleSkewVersor3DTransform__SWIG_3', scale, skew, versor, translation, fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleSkewVersor3DTransform);
  ans
  
}

attr(`ScaleSkewVersor3DTransform__SWIG_3`, 'returnType') = '_p_itk__simple__ScaleSkewVersor3DTransform'
attr(`ScaleSkewVersor3DTransform__SWIG_3`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`ScaleSkewVersor3DTransform__SWIG_3`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform__SWIG_3'))

# Start of new_ScaleSkewVersor3DTransform

`ScaleSkewVersor3DTransform__SWIG_4` = function(scale, skew, versor, translation)
{
  scale = as.numeric(scale);
  skew = as.numeric(skew);
  versor = as.numeric(versor);
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_new_ScaleSkewVersor3DTransform__SWIG_4', scale, skew, versor, translation, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleSkewVersor3DTransform);
  ans
  
}

attr(`ScaleSkewVersor3DTransform__SWIG_4`, 'returnType') = '_p_itk__simple__ScaleSkewVersor3DTransform'
attr(`ScaleSkewVersor3DTransform__SWIG_4`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`ScaleSkewVersor3DTransform__SWIG_4`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform__SWIG_4'))

# Start of new_ScaleSkewVersor3DTransform

`ScaleSkewVersor3DTransform__SWIG_5` = function(scale, skew, versor)
{
  scale = as.numeric(scale);
  skew = as.numeric(skew);
  versor = as.numeric(versor);
  ;ans = .Call('R_swig_new_ScaleSkewVersor3DTransform__SWIG_5', scale, skew, versor, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleSkewVersor3DTransform);
  ans
  
}

attr(`ScaleSkewVersor3DTransform__SWIG_5`, 'returnType') = '_p_itk__simple__ScaleSkewVersor3DTransform'
attr(`ScaleSkewVersor3DTransform__SWIG_5`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`ScaleSkewVersor3DTransform__SWIG_5`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform__SWIG_5'))

# Start of new_ScaleSkewVersor3DTransform

`ScaleSkewVersor3DTransform__SWIG_6` = function(scale, skew, axis, angle, translation, fixedCenter)
{
  scale = as.numeric(scale);
  skew = as.numeric(skew);
  axis = as.numeric(axis);
  
  translation = as.numeric(translation);
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_ScaleSkewVersor3DTransform__SWIG_6', scale, skew, axis, angle, translation, fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleSkewVersor3DTransform);
  ans
  
}

attr(`ScaleSkewVersor3DTransform__SWIG_6`, 'returnType') = '_p_itk__simple__ScaleSkewVersor3DTransform'
attr(`ScaleSkewVersor3DTransform__SWIG_6`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', 'numeric', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`ScaleSkewVersor3DTransform__SWIG_6`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform__SWIG_6'))

# Start of new_ScaleSkewVersor3DTransform

`ScaleSkewVersor3DTransform__SWIG_7` = function(scale, skew, axis, angle, translation)
{
  scale = as.numeric(scale);
  skew = as.numeric(skew);
  axis = as.numeric(axis);
  
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_new_ScaleSkewVersor3DTransform__SWIG_7', scale, skew, axis, angle, translation, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleSkewVersor3DTransform);
  ans
  
}

attr(`ScaleSkewVersor3DTransform__SWIG_7`, 'returnType') = '_p_itk__simple__ScaleSkewVersor3DTransform'
attr(`ScaleSkewVersor3DTransform__SWIG_7`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', 'numeric', '_p_std__vectorT_double_t')
class(`ScaleSkewVersor3DTransform__SWIG_7`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform__SWIG_7'))

# Start of new_ScaleSkewVersor3DTransform

`ScaleSkewVersor3DTransform__SWIG_8` = function(scale, skew, axis, angle)
{
  scale = as.numeric(scale);
  skew = as.numeric(skew);
  axis = as.numeric(axis);
  
  ;ans = .Call('R_swig_new_ScaleSkewVersor3DTransform__SWIG_8', scale, skew, axis, angle, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleSkewVersor3DTransform);
  ans
  
}

attr(`ScaleSkewVersor3DTransform__SWIG_8`, 'returnType') = '_p_itk__simple__ScaleSkewVersor3DTransform'
attr(`ScaleSkewVersor3DTransform__SWIG_8`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', 'numeric')
class(`ScaleSkewVersor3DTransform__SWIG_8`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform__SWIG_8'))

`ScaleSkewVersor3DTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 0) {
    f <- ScaleSkewVersor3DTransform__SWIG_0; 
  } else if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__ScaleSkewVersor3DTransform') && length(argv[[1]]) == 1 ) {
      f <- ScaleSkewVersor3DTransform__SWIG_1; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- ScaleSkewVersor3DTransform__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) )) {
      f <- ScaleSkewVersor3DTransform__SWIG_5; 
    }
  } else if (argc == 4) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ScaleSkewVersor3DTransform__SWIG_8; 
    }
    else if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) )) {
      f <- ScaleSkewVersor3DTransform__SWIG_4; 
    }
  } else if (argc == 5) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) )) {
      f <- ScaleSkewVersor3DTransform__SWIG_7; 
    }
    else if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) )) {
      f <- ScaleSkewVersor3DTransform__SWIG_3; 
    }
  } else if (argc == 6) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) )) {
      f <- ScaleSkewVersor3DTransform__SWIG_6; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ScaleSkewVersor3DTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ScaleSkewVersor3DTransform_Equal

`ScaleSkewVersor3DTransform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_ScaleSkewVersor3DTransform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScaleSkewVersor3DTransform_Equal`, 'returnType') = '_p_itk__simple__ScaleSkewVersor3DTransform'
attr(`ScaleSkewVersor3DTransform_Equal`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform', '_p_itk__simple__ScaleSkewVersor3DTransform')
class(`ScaleSkewVersor3DTransform_Equal`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform_Equal'))

# Start of ScaleSkewVersor3DTransform_GetName

`ScaleSkewVersor3DTransform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleSkewVersor3DTransform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleSkewVersor3DTransform_GetName`, 'returnType') = 'character'
attr(`ScaleSkewVersor3DTransform_GetName`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform')
class(`ScaleSkewVersor3DTransform_GetName`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform_GetName'))

# Start of ScaleSkewVersor3DTransform_SetCenter

`ScaleSkewVersor3DTransform_SetCenter` = function(self, params)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  params = as.numeric(params);
  ;ans = .Call('R_swig_ScaleSkewVersor3DTransform_SetCenter', self, params, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScaleSkewVersor3DTransform_SetCenter`, 'returnType') = 'void'
attr(`ScaleSkewVersor3DTransform_SetCenter`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform', '_p_std__vectorT_double_t')
class(`ScaleSkewVersor3DTransform_SetCenter`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform_SetCenter'))

# Start of ScaleSkewVersor3DTransform_GetCenter

`ScaleSkewVersor3DTransform_GetCenter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleSkewVersor3DTransform_GetCenter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleSkewVersor3DTransform_GetCenter`, 'returnType') = 'numeric'
attr(`ScaleSkewVersor3DTransform_GetCenter`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform')
class(`ScaleSkewVersor3DTransform_GetCenter`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform_GetCenter'))

# Start of ScaleSkewVersor3DTransform_SetRotation

`ScaleSkewVersor3DTransform_SetRotation__SWIG_0` = function(self, versor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  versor = as.numeric(versor);
  ;ans = .Call('R_swig_ScaleSkewVersor3DTransform_SetRotation__SWIG_0', self, versor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScaleSkewVersor3DTransform_SetRotation__SWIG_0`, 'returnType') = 'void'
attr(`ScaleSkewVersor3DTransform_SetRotation__SWIG_0`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform', '_p_std__vectorT_double_t')
class(`ScaleSkewVersor3DTransform_SetRotation__SWIG_0`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform_SetRotation__SWIG_0'))

# Start of ScaleSkewVersor3DTransform_SetRotation

`ScaleSkewVersor3DTransform_SetRotation__SWIG_1` = function(self, axis, angle)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  axis = as.numeric(axis);
  
  ;ans = .Call('R_swig_ScaleSkewVersor3DTransform_SetRotation__SWIG_1', self, axis, angle, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScaleSkewVersor3DTransform_SetRotation__SWIG_1`, 'returnType') = 'void'
attr(`ScaleSkewVersor3DTransform_SetRotation__SWIG_1`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform', '_p_std__vectorT_double_t', 'numeric')
class(`ScaleSkewVersor3DTransform_SetRotation__SWIG_1`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform_SetRotation__SWIG_1'))

`ScaleSkewVersor3DTransform_SetRotation` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ScaleSkewVersor3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- ScaleSkewVersor3DTransform_SetRotation__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ScaleSkewVersor3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ScaleSkewVersor3DTransform_SetRotation__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ScaleSkewVersor3DTransform_SetRotation with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ScaleSkewVersor3DTransform_GetVersor

`ScaleSkewVersor3DTransform_GetVersor` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleSkewVersor3DTransform_GetVersor', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleSkewVersor3DTransform_GetVersor`, 'returnType') = 'numeric'
attr(`ScaleSkewVersor3DTransform_GetVersor`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform')
class(`ScaleSkewVersor3DTransform_GetVersor`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform_GetVersor'))

# Start of ScaleSkewVersor3DTransform_GetTranslation

`ScaleSkewVersor3DTransform_GetTranslation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleSkewVersor3DTransform_GetTranslation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleSkewVersor3DTransform_GetTranslation`, 'returnType') = 'numeric'
attr(`ScaleSkewVersor3DTransform_GetTranslation`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform')
class(`ScaleSkewVersor3DTransform_GetTranslation`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform_GetTranslation'))

# Start of ScaleSkewVersor3DTransform_SetTranslation

`ScaleSkewVersor3DTransform_SetTranslation` = function(self, translation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_ScaleSkewVersor3DTransform_SetTranslation', self, translation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScaleSkewVersor3DTransform_SetTranslation`, 'returnType') = 'void'
attr(`ScaleSkewVersor3DTransform_SetTranslation`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform', '_p_std__vectorT_double_t')
class(`ScaleSkewVersor3DTransform_SetTranslation`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform_SetTranslation'))

# Start of ScaleSkewVersor3DTransform_GetScale

`ScaleSkewVersor3DTransform_GetScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleSkewVersor3DTransform_GetScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleSkewVersor3DTransform_GetScale`, 'returnType') = 'numeric'
attr(`ScaleSkewVersor3DTransform_GetScale`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform')
class(`ScaleSkewVersor3DTransform_GetScale`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform_GetScale'))

# Start of ScaleSkewVersor3DTransform_SetScale

`ScaleSkewVersor3DTransform_SetScale` = function(self, scale)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  scale = as.numeric(scale);
  ;ans = .Call('R_swig_ScaleSkewVersor3DTransform_SetScale', self, scale, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScaleSkewVersor3DTransform_SetScale`, 'returnType') = 'void'
attr(`ScaleSkewVersor3DTransform_SetScale`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform', '_p_std__vectorT_double_t')
class(`ScaleSkewVersor3DTransform_SetScale`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform_SetScale'))

# Start of ScaleSkewVersor3DTransform_GetSkew

`ScaleSkewVersor3DTransform_GetSkew` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleSkewVersor3DTransform_GetSkew', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleSkewVersor3DTransform_GetSkew`, 'returnType') = 'numeric'
attr(`ScaleSkewVersor3DTransform_GetSkew`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform')
class(`ScaleSkewVersor3DTransform_GetSkew`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform_GetSkew'))

# Start of ScaleSkewVersor3DTransform_SetSkew

`ScaleSkewVersor3DTransform_SetSkew` = function(self, skew)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  skew = as.numeric(skew);
  ;ans = .Call('R_swig_ScaleSkewVersor3DTransform_SetSkew', self, skew, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScaleSkewVersor3DTransform_SetSkew`, 'returnType') = 'void'
attr(`ScaleSkewVersor3DTransform_SetSkew`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform', '_p_std__vectorT_double_t')
class(`ScaleSkewVersor3DTransform_SetSkew`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform_SetSkew'))

# Start of ScaleSkewVersor3DTransform_Translate

`ScaleSkewVersor3DTransform_Translate` = function(self, offset)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  offset = as.numeric(offset);
  ;ans = .Call('R_swig_ScaleSkewVersor3DTransform_Translate', self, offset, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScaleSkewVersor3DTransform_Translate`, 'returnType') = 'void'
attr(`ScaleSkewVersor3DTransform_Translate`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform', '_p_std__vectorT_double_t')
class(`ScaleSkewVersor3DTransform_Translate`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform_Translate'))

# Start of ScaleSkewVersor3DTransform_GetMatrix

`ScaleSkewVersor3DTransform_GetMatrix` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleSkewVersor3DTransform_GetMatrix', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleSkewVersor3DTransform_GetMatrix`, 'returnType') = 'numeric'
attr(`ScaleSkewVersor3DTransform_GetMatrix`, "inputTypes") = c('_p_itk__simple__ScaleSkewVersor3DTransform')
class(`ScaleSkewVersor3DTransform_GetMatrix`) = c("SWIGFunction", class('ScaleSkewVersor3DTransform_GetMatrix'))

# Start of accessor method for itk::simple::ScaleSkewVersor3DTransform
setMethod('$', '_p_itk__simple__ScaleSkewVersor3DTransform', function(x, name)

{
  accessorFuns = list('Equal' = ScaleSkewVersor3DTransform_Equal, 'GetName' = ScaleSkewVersor3DTransform_GetName, 'SetCenter' = ScaleSkewVersor3DTransform_SetCenter, 'GetCenter' = ScaleSkewVersor3DTransform_GetCenter, 'SetRotation' = ScaleSkewVersor3DTransform_SetRotation, 'GetVersor' = ScaleSkewVersor3DTransform_GetVersor, 'GetTranslation' = ScaleSkewVersor3DTransform_GetTranslation, 'SetTranslation' = ScaleSkewVersor3DTransform_SetTranslation, 'GetScale' = ScaleSkewVersor3DTransform_GetScale, 'SetScale' = ScaleSkewVersor3DTransform_SetScale, 'GetSkew' = ScaleSkewVersor3DTransform_GetSkew, 'SetSkew' = ScaleSkewVersor3DTransform_SetSkew, 'Translate' = ScaleSkewVersor3DTransform_Translate, 'GetMatrix' = ScaleSkewVersor3DTransform_GetMatrix);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ScaleSkewVersor3DTransform
# Start of delete_ComposeScaleSkewVersor3DTransform

`delete_ComposeScaleSkewVersor3DTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ComposeScaleSkewVersor3DTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ComposeScaleSkewVersor3DTransform`, 'returnType') = 'void'
attr(`delete_ComposeScaleSkewVersor3DTransform`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform')
class(`delete_ComposeScaleSkewVersor3DTransform`) = c("SWIGFunction", class('delete_ComposeScaleSkewVersor3DTransform'))

# Start of new_ComposeScaleSkewVersor3DTransform

`ComposeScaleSkewVersor3DTransform__SWIG_0` = function()
{
  ;ans = .Call('R_swig_new_ComposeScaleSkewVersor3DTransform__SWIG_0', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ComposeScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ComposeScaleSkewVersor3DTransform);
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform__SWIG_0`, 'returnType') = '_p_itk__simple__ComposeScaleSkewVersor3DTransform'
class(`ComposeScaleSkewVersor3DTransform__SWIG_0`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform__SWIG_0'))

# Start of new_ComposeScaleSkewVersor3DTransform

`ComposeScaleSkewVersor3DTransform__SWIG_1` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_ComposeScaleSkewVersor3DTransform__SWIG_1', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ComposeScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ComposeScaleSkewVersor3DTransform);
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform__SWIG_1`, 'returnType') = '_p_itk__simple__ComposeScaleSkewVersor3DTransform'
attr(`ComposeScaleSkewVersor3DTransform__SWIG_1`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform')
class(`ComposeScaleSkewVersor3DTransform__SWIG_1`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform__SWIG_1'))

# Start of new_ComposeScaleSkewVersor3DTransform

`ComposeScaleSkewVersor3DTransform__SWIG_2` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_ComposeScaleSkewVersor3DTransform__SWIG_2', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ComposeScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ComposeScaleSkewVersor3DTransform);
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform__SWIG_2`, 'returnType') = '_p_itk__simple__ComposeScaleSkewVersor3DTransform'
attr(`ComposeScaleSkewVersor3DTransform__SWIG_2`, "inputTypes") = c('_p_itk__simple__Transform')
class(`ComposeScaleSkewVersor3DTransform__SWIG_2`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform__SWIG_2'))

# Start of new_ComposeScaleSkewVersor3DTransform

`ComposeScaleSkewVersor3DTransform__SWIG_3` = function(scale, skew, versor, translation, fixedCenter)
{
  scale = as.numeric(scale);
  skew = as.numeric(skew);
  versor = as.numeric(versor);
  translation = as.numeric(translation);
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_ComposeScaleSkewVersor3DTransform__SWIG_3', scale, skew, versor, translation, fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ComposeScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ComposeScaleSkewVersor3DTransform);
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform__SWIG_3`, 'returnType') = '_p_itk__simple__ComposeScaleSkewVersor3DTransform'
attr(`ComposeScaleSkewVersor3DTransform__SWIG_3`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`ComposeScaleSkewVersor3DTransform__SWIG_3`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform__SWIG_3'))

# Start of new_ComposeScaleSkewVersor3DTransform

`ComposeScaleSkewVersor3DTransform__SWIG_4` = function(scale, skew, versor, translation)
{
  scale = as.numeric(scale);
  skew = as.numeric(skew);
  versor = as.numeric(versor);
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_new_ComposeScaleSkewVersor3DTransform__SWIG_4', scale, skew, versor, translation, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ComposeScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ComposeScaleSkewVersor3DTransform);
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform__SWIG_4`, 'returnType') = '_p_itk__simple__ComposeScaleSkewVersor3DTransform'
attr(`ComposeScaleSkewVersor3DTransform__SWIG_4`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`ComposeScaleSkewVersor3DTransform__SWIG_4`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform__SWIG_4'))

# Start of new_ComposeScaleSkewVersor3DTransform

`ComposeScaleSkewVersor3DTransform__SWIG_5` = function(scale, skew, versor)
{
  scale = as.numeric(scale);
  skew = as.numeric(skew);
  versor = as.numeric(versor);
  ;ans = .Call('R_swig_new_ComposeScaleSkewVersor3DTransform__SWIG_5', scale, skew, versor, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ComposeScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ComposeScaleSkewVersor3DTransform);
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform__SWIG_5`, 'returnType') = '_p_itk__simple__ComposeScaleSkewVersor3DTransform'
attr(`ComposeScaleSkewVersor3DTransform__SWIG_5`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`ComposeScaleSkewVersor3DTransform__SWIG_5`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform__SWIG_5'))

# Start of new_ComposeScaleSkewVersor3DTransform

`ComposeScaleSkewVersor3DTransform__SWIG_6` = function(scale, skew, axis, angle, translation, fixedCenter)
{
  scale = as.numeric(scale);
  skew = as.numeric(skew);
  axis = as.numeric(axis);
  
  translation = as.numeric(translation);
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_ComposeScaleSkewVersor3DTransform__SWIG_6', scale, skew, axis, angle, translation, fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ComposeScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ComposeScaleSkewVersor3DTransform);
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform__SWIG_6`, 'returnType') = '_p_itk__simple__ComposeScaleSkewVersor3DTransform'
attr(`ComposeScaleSkewVersor3DTransform__SWIG_6`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', 'numeric', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`ComposeScaleSkewVersor3DTransform__SWIG_6`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform__SWIG_6'))

# Start of new_ComposeScaleSkewVersor3DTransform

`ComposeScaleSkewVersor3DTransform__SWIG_7` = function(scale, skew, axis, angle, translation)
{
  scale = as.numeric(scale);
  skew = as.numeric(skew);
  axis = as.numeric(axis);
  
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_new_ComposeScaleSkewVersor3DTransform__SWIG_7', scale, skew, axis, angle, translation, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ComposeScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ComposeScaleSkewVersor3DTransform);
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform__SWIG_7`, 'returnType') = '_p_itk__simple__ComposeScaleSkewVersor3DTransform'
attr(`ComposeScaleSkewVersor3DTransform__SWIG_7`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', 'numeric', '_p_std__vectorT_double_t')
class(`ComposeScaleSkewVersor3DTransform__SWIG_7`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform__SWIG_7'))

# Start of new_ComposeScaleSkewVersor3DTransform

`ComposeScaleSkewVersor3DTransform__SWIG_8` = function(scale, skew, axis, angle)
{
  scale = as.numeric(scale);
  skew = as.numeric(skew);
  axis = as.numeric(axis);
  
  ;ans = .Call('R_swig_new_ComposeScaleSkewVersor3DTransform__SWIG_8', scale, skew, axis, angle, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ComposeScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ComposeScaleSkewVersor3DTransform);
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform__SWIG_8`, 'returnType') = '_p_itk__simple__ComposeScaleSkewVersor3DTransform'
attr(`ComposeScaleSkewVersor3DTransform__SWIG_8`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', 'numeric')
class(`ComposeScaleSkewVersor3DTransform__SWIG_8`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform__SWIG_8'))

`ComposeScaleSkewVersor3DTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 0) {
    f <- ComposeScaleSkewVersor3DTransform__SWIG_0; 
  } else if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__ComposeScaleSkewVersor3DTransform') && length(argv[[1]]) == 1 ) {
      f <- ComposeScaleSkewVersor3DTransform__SWIG_1; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- ComposeScaleSkewVersor3DTransform__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) )) {
      f <- ComposeScaleSkewVersor3DTransform__SWIG_5; 
    }
  } else if (argc == 4) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ComposeScaleSkewVersor3DTransform__SWIG_8; 
    }
    else if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) )) {
      f <- ComposeScaleSkewVersor3DTransform__SWIG_4; 
    }
  } else if (argc == 5) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) )) {
      f <- ComposeScaleSkewVersor3DTransform__SWIG_7; 
    }
    else if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) )) {
      f <- ComposeScaleSkewVersor3DTransform__SWIG_3; 
    }
  } else if (argc == 6) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) )) {
      f <- ComposeScaleSkewVersor3DTransform__SWIG_6; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ComposeScaleSkewVersor3DTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ComposeScaleSkewVersor3DTransform_Equal

`ComposeScaleSkewVersor3DTransform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_ComposeScaleSkewVersor3DTransform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ComposeScaleSkewVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform_Equal`, 'returnType') = '_p_itk__simple__ComposeScaleSkewVersor3DTransform'
attr(`ComposeScaleSkewVersor3DTransform_Equal`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform', '_p_itk__simple__ComposeScaleSkewVersor3DTransform')
class(`ComposeScaleSkewVersor3DTransform_Equal`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform_Equal'))

# Start of ComposeScaleSkewVersor3DTransform_GetName

`ComposeScaleSkewVersor3DTransform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComposeScaleSkewVersor3DTransform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComposeScaleSkewVersor3DTransform_GetName`, 'returnType') = 'character'
attr(`ComposeScaleSkewVersor3DTransform_GetName`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform')
class(`ComposeScaleSkewVersor3DTransform_GetName`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform_GetName'))

# Start of ComposeScaleSkewVersor3DTransform_SetCenter

`ComposeScaleSkewVersor3DTransform_SetCenter` = function(self, params)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  params = as.numeric(params);
  ;ans = .Call('R_swig_ComposeScaleSkewVersor3DTransform_SetCenter', self, params, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform_SetCenter`, 'returnType') = 'void'
attr(`ComposeScaleSkewVersor3DTransform_SetCenter`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform', '_p_std__vectorT_double_t')
class(`ComposeScaleSkewVersor3DTransform_SetCenter`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform_SetCenter'))

# Start of ComposeScaleSkewVersor3DTransform_GetCenter

`ComposeScaleSkewVersor3DTransform_GetCenter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComposeScaleSkewVersor3DTransform_GetCenter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComposeScaleSkewVersor3DTransform_GetCenter`, 'returnType') = 'numeric'
attr(`ComposeScaleSkewVersor3DTransform_GetCenter`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform')
class(`ComposeScaleSkewVersor3DTransform_GetCenter`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform_GetCenter'))

# Start of ComposeScaleSkewVersor3DTransform_SetRotation

`ComposeScaleSkewVersor3DTransform_SetRotation__SWIG_0` = function(self, versor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  versor = as.numeric(versor);
  ;ans = .Call('R_swig_ComposeScaleSkewVersor3DTransform_SetRotation__SWIG_0', self, versor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform_SetRotation__SWIG_0`, 'returnType') = 'void'
attr(`ComposeScaleSkewVersor3DTransform_SetRotation__SWIG_0`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform', '_p_std__vectorT_double_t')
class(`ComposeScaleSkewVersor3DTransform_SetRotation__SWIG_0`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform_SetRotation__SWIG_0'))

# Start of ComposeScaleSkewVersor3DTransform_SetRotation

`ComposeScaleSkewVersor3DTransform_SetRotation__SWIG_1` = function(self, axis, angle)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  axis = as.numeric(axis);
  
  ;ans = .Call('R_swig_ComposeScaleSkewVersor3DTransform_SetRotation__SWIG_1', self, axis, angle, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform_SetRotation__SWIG_1`, 'returnType') = 'void'
attr(`ComposeScaleSkewVersor3DTransform_SetRotation__SWIG_1`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform', '_p_std__vectorT_double_t', 'numeric')
class(`ComposeScaleSkewVersor3DTransform_SetRotation__SWIG_1`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform_SetRotation__SWIG_1'))

`ComposeScaleSkewVersor3DTransform_SetRotation` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ComposeScaleSkewVersor3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- ComposeScaleSkewVersor3DTransform_SetRotation__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ComposeScaleSkewVersor3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ComposeScaleSkewVersor3DTransform_SetRotation__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ComposeScaleSkewVersor3DTransform_SetRotation with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ComposeScaleSkewVersor3DTransform_GetVersor

`ComposeScaleSkewVersor3DTransform_GetVersor` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComposeScaleSkewVersor3DTransform_GetVersor', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComposeScaleSkewVersor3DTransform_GetVersor`, 'returnType') = 'numeric'
attr(`ComposeScaleSkewVersor3DTransform_GetVersor`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform')
class(`ComposeScaleSkewVersor3DTransform_GetVersor`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform_GetVersor'))

# Start of ComposeScaleSkewVersor3DTransform_GetTranslation

`ComposeScaleSkewVersor3DTransform_GetTranslation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComposeScaleSkewVersor3DTransform_GetTranslation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComposeScaleSkewVersor3DTransform_GetTranslation`, 'returnType') = 'numeric'
attr(`ComposeScaleSkewVersor3DTransform_GetTranslation`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform')
class(`ComposeScaleSkewVersor3DTransform_GetTranslation`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform_GetTranslation'))

# Start of ComposeScaleSkewVersor3DTransform_SetTranslation

`ComposeScaleSkewVersor3DTransform_SetTranslation` = function(self, translation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_ComposeScaleSkewVersor3DTransform_SetTranslation', self, translation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform_SetTranslation`, 'returnType') = 'void'
attr(`ComposeScaleSkewVersor3DTransform_SetTranslation`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform', '_p_std__vectorT_double_t')
class(`ComposeScaleSkewVersor3DTransform_SetTranslation`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform_SetTranslation'))

# Start of ComposeScaleSkewVersor3DTransform_GetScale

`ComposeScaleSkewVersor3DTransform_GetScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComposeScaleSkewVersor3DTransform_GetScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComposeScaleSkewVersor3DTransform_GetScale`, 'returnType') = 'numeric'
attr(`ComposeScaleSkewVersor3DTransform_GetScale`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform')
class(`ComposeScaleSkewVersor3DTransform_GetScale`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform_GetScale'))

# Start of ComposeScaleSkewVersor3DTransform_SetScale

`ComposeScaleSkewVersor3DTransform_SetScale` = function(self, scale)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  scale = as.numeric(scale);
  ;ans = .Call('R_swig_ComposeScaleSkewVersor3DTransform_SetScale', self, scale, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform_SetScale`, 'returnType') = 'void'
attr(`ComposeScaleSkewVersor3DTransform_SetScale`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform', '_p_std__vectorT_double_t')
class(`ComposeScaleSkewVersor3DTransform_SetScale`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform_SetScale'))

# Start of ComposeScaleSkewVersor3DTransform_GetSkew

`ComposeScaleSkewVersor3DTransform_GetSkew` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComposeScaleSkewVersor3DTransform_GetSkew', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComposeScaleSkewVersor3DTransform_GetSkew`, 'returnType') = 'numeric'
attr(`ComposeScaleSkewVersor3DTransform_GetSkew`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform')
class(`ComposeScaleSkewVersor3DTransform_GetSkew`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform_GetSkew'))

# Start of ComposeScaleSkewVersor3DTransform_SetSkew

`ComposeScaleSkewVersor3DTransform_SetSkew` = function(self, skew)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  skew = as.numeric(skew);
  ;ans = .Call('R_swig_ComposeScaleSkewVersor3DTransform_SetSkew', self, skew, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform_SetSkew`, 'returnType') = 'void'
attr(`ComposeScaleSkewVersor3DTransform_SetSkew`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform', '_p_std__vectorT_double_t')
class(`ComposeScaleSkewVersor3DTransform_SetSkew`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform_SetSkew'))

# Start of ComposeScaleSkewVersor3DTransform_Translate

`ComposeScaleSkewVersor3DTransform_Translate` = function(self, offset)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  offset = as.numeric(offset);
  ;ans = .Call('R_swig_ComposeScaleSkewVersor3DTransform_Translate', self, offset, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ComposeScaleSkewVersor3DTransform_Translate`, 'returnType') = 'void'
attr(`ComposeScaleSkewVersor3DTransform_Translate`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform', '_p_std__vectorT_double_t')
class(`ComposeScaleSkewVersor3DTransform_Translate`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform_Translate'))

# Start of ComposeScaleSkewVersor3DTransform_GetMatrix

`ComposeScaleSkewVersor3DTransform_GetMatrix` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComposeScaleSkewVersor3DTransform_GetMatrix', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComposeScaleSkewVersor3DTransform_GetMatrix`, 'returnType') = 'numeric'
attr(`ComposeScaleSkewVersor3DTransform_GetMatrix`, "inputTypes") = c('_p_itk__simple__ComposeScaleSkewVersor3DTransform')
class(`ComposeScaleSkewVersor3DTransform_GetMatrix`) = c("SWIGFunction", class('ComposeScaleSkewVersor3DTransform_GetMatrix'))

# Start of accessor method for itk::simple::ComposeScaleSkewVersor3DTransform
setMethod('$', '_p_itk__simple__ComposeScaleSkewVersor3DTransform', function(x, name)

{
  accessorFuns = list('Equal' = ComposeScaleSkewVersor3DTransform_Equal, 'GetName' = ComposeScaleSkewVersor3DTransform_GetName, 'SetCenter' = ComposeScaleSkewVersor3DTransform_SetCenter, 'GetCenter' = ComposeScaleSkewVersor3DTransform_GetCenter, 'SetRotation' = ComposeScaleSkewVersor3DTransform_SetRotation, 'GetVersor' = ComposeScaleSkewVersor3DTransform_GetVersor, 'GetTranslation' = ComposeScaleSkewVersor3DTransform_GetTranslation, 'SetTranslation' = ComposeScaleSkewVersor3DTransform_SetTranslation, 'GetScale' = ComposeScaleSkewVersor3DTransform_GetScale, 'SetScale' = ComposeScaleSkewVersor3DTransform_SetScale, 'GetSkew' = ComposeScaleSkewVersor3DTransform_GetSkew, 'SetSkew' = ComposeScaleSkewVersor3DTransform_SetSkew, 'Translate' = ComposeScaleSkewVersor3DTransform_Translate, 'GetMatrix' = ComposeScaleSkewVersor3DTransform_GetMatrix);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ComposeScaleSkewVersor3DTransform
# Start of delete_ScaleVersor3DTransform

`delete_ScaleVersor3DTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ScaleVersor3DTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ScaleVersor3DTransform`, 'returnType') = 'void'
attr(`delete_ScaleVersor3DTransform`, "inputTypes") = c('_p_itk__simple__ScaleVersor3DTransform')
class(`delete_ScaleVersor3DTransform`) = c("SWIGFunction", class('delete_ScaleVersor3DTransform'))

# Start of new_ScaleVersor3DTransform

`ScaleVersor3DTransform__SWIG_0` = function()
{
  ;ans = .Call('R_swig_new_ScaleVersor3DTransform__SWIG_0', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleVersor3DTransform);
  ans
  
}

attr(`ScaleVersor3DTransform__SWIG_0`, 'returnType') = '_p_itk__simple__ScaleVersor3DTransform'
class(`ScaleVersor3DTransform__SWIG_0`) = c("SWIGFunction", class('ScaleVersor3DTransform__SWIG_0'))

# Start of new_ScaleVersor3DTransform

`ScaleVersor3DTransform__SWIG_1` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_ScaleVersor3DTransform__SWIG_1', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleVersor3DTransform);
  ans
  
}

attr(`ScaleVersor3DTransform__SWIG_1`, 'returnType') = '_p_itk__simple__ScaleVersor3DTransform'
attr(`ScaleVersor3DTransform__SWIG_1`, "inputTypes") = c('_p_itk__simple__ScaleVersor3DTransform')
class(`ScaleVersor3DTransform__SWIG_1`) = c("SWIGFunction", class('ScaleVersor3DTransform__SWIG_1'))

# Start of new_ScaleVersor3DTransform

`ScaleVersor3DTransform__SWIG_2` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_ScaleVersor3DTransform__SWIG_2', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleVersor3DTransform);
  ans
  
}

attr(`ScaleVersor3DTransform__SWIG_2`, 'returnType') = '_p_itk__simple__ScaleVersor3DTransform'
attr(`ScaleVersor3DTransform__SWIG_2`, "inputTypes") = c('_p_itk__simple__Transform')
class(`ScaleVersor3DTransform__SWIG_2`) = c("SWIGFunction", class('ScaleVersor3DTransform__SWIG_2'))

# Start of new_ScaleVersor3DTransform

`ScaleVersor3DTransform__SWIG_3` = function(scale, versor, translation, fixedCenter)
{
  scale = as.numeric(scale);
  versor = as.numeric(versor);
  translation = as.numeric(translation);
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_ScaleVersor3DTransform__SWIG_3', scale, versor, translation, fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleVersor3DTransform);
  ans
  
}

attr(`ScaleVersor3DTransform__SWIG_3`, 'returnType') = '_p_itk__simple__ScaleVersor3DTransform'
attr(`ScaleVersor3DTransform__SWIG_3`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`ScaleVersor3DTransform__SWIG_3`) = c("SWIGFunction", class('ScaleVersor3DTransform__SWIG_3'))

# Start of new_ScaleVersor3DTransform

`ScaleVersor3DTransform__SWIG_4` = function(scale, versor, translation)
{
  scale = as.numeric(scale);
  versor = as.numeric(versor);
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_new_ScaleVersor3DTransform__SWIG_4', scale, versor, translation, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleVersor3DTransform);
  ans
  
}

attr(`ScaleVersor3DTransform__SWIG_4`, 'returnType') = '_p_itk__simple__ScaleVersor3DTransform'
attr(`ScaleVersor3DTransform__SWIG_4`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`ScaleVersor3DTransform__SWIG_4`) = c("SWIGFunction", class('ScaleVersor3DTransform__SWIG_4'))

# Start of new_ScaleVersor3DTransform

`ScaleVersor3DTransform__SWIG_5` = function(scale, versor)
{
  scale = as.numeric(scale);
  versor = as.numeric(versor);
  ;ans = .Call('R_swig_new_ScaleVersor3DTransform__SWIG_5', scale, versor, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleVersor3DTransform);
  ans
  
}

attr(`ScaleVersor3DTransform__SWIG_5`, 'returnType') = '_p_itk__simple__ScaleVersor3DTransform'
attr(`ScaleVersor3DTransform__SWIG_5`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`ScaleVersor3DTransform__SWIG_5`) = c("SWIGFunction", class('ScaleVersor3DTransform__SWIG_5'))

# Start of new_ScaleVersor3DTransform

`ScaleVersor3DTransform__SWIG_6` = function(scale, axis, angle, translation, fixedCenter)
{
  scale = as.numeric(scale);
  axis = as.numeric(axis);
  
  translation = as.numeric(translation);
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_ScaleVersor3DTransform__SWIG_6', scale, axis, angle, translation, fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleVersor3DTransform);
  ans
  
}

attr(`ScaleVersor3DTransform__SWIG_6`, 'returnType') = '_p_itk__simple__ScaleVersor3DTransform'
attr(`ScaleVersor3DTransform__SWIG_6`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', 'numeric', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`ScaleVersor3DTransform__SWIG_6`) = c("SWIGFunction", class('ScaleVersor3DTransform__SWIG_6'))

# Start of new_ScaleVersor3DTransform

`ScaleVersor3DTransform__SWIG_7` = function(scale, axis, angle, translation)
{
  scale = as.numeric(scale);
  axis = as.numeric(axis);
  
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_new_ScaleVersor3DTransform__SWIG_7', scale, axis, angle, translation, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleVersor3DTransform);
  ans
  
}

attr(`ScaleVersor3DTransform__SWIG_7`, 'returnType') = '_p_itk__simple__ScaleVersor3DTransform'
attr(`ScaleVersor3DTransform__SWIG_7`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', 'numeric', '_p_std__vectorT_double_t')
class(`ScaleVersor3DTransform__SWIG_7`) = c("SWIGFunction", class('ScaleVersor3DTransform__SWIG_7'))

# Start of new_ScaleVersor3DTransform

`ScaleVersor3DTransform__SWIG_8` = function(scale, axis, angle)
{
  scale = as.numeric(scale);
  axis = as.numeric(axis);
  
  ;ans = .Call('R_swig_new_ScaleVersor3DTransform__SWIG_8', scale, axis, angle, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScaleVersor3DTransform);
  ans
  
}

attr(`ScaleVersor3DTransform__SWIG_8`, 'returnType') = '_p_itk__simple__ScaleVersor3DTransform'
attr(`ScaleVersor3DTransform__SWIG_8`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', 'numeric')
class(`ScaleVersor3DTransform__SWIG_8`) = c("SWIGFunction", class('ScaleVersor3DTransform__SWIG_8'))

`ScaleVersor3DTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 0) {
    f <- ScaleVersor3DTransform__SWIG_0; 
  } else if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__ScaleVersor3DTransform') && length(argv[[1]]) == 1 ) {
      f <- ScaleVersor3DTransform__SWIG_1; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- ScaleVersor3DTransform__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- ScaleVersor3DTransform__SWIG_5; 
    }
  } else if (argc == 3) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ScaleVersor3DTransform__SWIG_8; 
    }
    else if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) )) {
      f <- ScaleVersor3DTransform__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) )) {
      f <- ScaleVersor3DTransform__SWIG_7; 
    }
    else if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) )) {
      f <- ScaleVersor3DTransform__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) )) {
      f <- ScaleVersor3DTransform__SWIG_6; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ScaleVersor3DTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ScaleVersor3DTransform_Equal

`ScaleVersor3DTransform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_ScaleVersor3DTransform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScaleVersor3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScaleVersor3DTransform_Equal`, 'returnType') = '_p_itk__simple__ScaleVersor3DTransform'
attr(`ScaleVersor3DTransform_Equal`, "inputTypes") = c('_p_itk__simple__ScaleVersor3DTransform', '_p_itk__simple__ScaleVersor3DTransform')
class(`ScaleVersor3DTransform_Equal`) = c("SWIGFunction", class('ScaleVersor3DTransform_Equal'))

# Start of ScaleVersor3DTransform_GetName

`ScaleVersor3DTransform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleVersor3DTransform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleVersor3DTransform_GetName`, 'returnType') = 'character'
attr(`ScaleVersor3DTransform_GetName`, "inputTypes") = c('_p_itk__simple__ScaleVersor3DTransform')
class(`ScaleVersor3DTransform_GetName`) = c("SWIGFunction", class('ScaleVersor3DTransform_GetName'))

# Start of ScaleVersor3DTransform_SetCenter

`ScaleVersor3DTransform_SetCenter` = function(self, params)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  params = as.numeric(params);
  ;ans = .Call('R_swig_ScaleVersor3DTransform_SetCenter', self, params, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScaleVersor3DTransform_SetCenter`, 'returnType') = 'void'
attr(`ScaleVersor3DTransform_SetCenter`, "inputTypes") = c('_p_itk__simple__ScaleVersor3DTransform', '_p_std__vectorT_double_t')
class(`ScaleVersor3DTransform_SetCenter`) = c("SWIGFunction", class('ScaleVersor3DTransform_SetCenter'))

# Start of ScaleVersor3DTransform_GetCenter

`ScaleVersor3DTransform_GetCenter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleVersor3DTransform_GetCenter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleVersor3DTransform_GetCenter`, 'returnType') = 'numeric'
attr(`ScaleVersor3DTransform_GetCenter`, "inputTypes") = c('_p_itk__simple__ScaleVersor3DTransform')
class(`ScaleVersor3DTransform_GetCenter`) = c("SWIGFunction", class('ScaleVersor3DTransform_GetCenter'))

# Start of ScaleVersor3DTransform_SetRotation

`ScaleVersor3DTransform_SetRotation__SWIG_0` = function(self, versor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  versor = as.numeric(versor);
  ;ans = .Call('R_swig_ScaleVersor3DTransform_SetRotation__SWIG_0', self, versor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScaleVersor3DTransform_SetRotation__SWIG_0`, 'returnType') = 'void'
attr(`ScaleVersor3DTransform_SetRotation__SWIG_0`, "inputTypes") = c('_p_itk__simple__ScaleVersor3DTransform', '_p_std__vectorT_double_t')
class(`ScaleVersor3DTransform_SetRotation__SWIG_0`) = c("SWIGFunction", class('ScaleVersor3DTransform_SetRotation__SWIG_0'))

# Start of ScaleVersor3DTransform_SetRotation

`ScaleVersor3DTransform_SetRotation__SWIG_1` = function(self, axis, angle)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  axis = as.numeric(axis);
  
  ;ans = .Call('R_swig_ScaleVersor3DTransform_SetRotation__SWIG_1', self, axis, angle, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScaleVersor3DTransform_SetRotation__SWIG_1`, 'returnType') = 'void'
attr(`ScaleVersor3DTransform_SetRotation__SWIG_1`, "inputTypes") = c('_p_itk__simple__ScaleVersor3DTransform', '_p_std__vectorT_double_t', 'numeric')
class(`ScaleVersor3DTransform_SetRotation__SWIG_1`) = c("SWIGFunction", class('ScaleVersor3DTransform_SetRotation__SWIG_1'))

`ScaleVersor3DTransform_SetRotation` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ScaleVersor3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- ScaleVersor3DTransform_SetRotation__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ScaleVersor3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ScaleVersor3DTransform_SetRotation__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ScaleVersor3DTransform_SetRotation with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ScaleVersor3DTransform_GetVersor

`ScaleVersor3DTransform_GetVersor` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleVersor3DTransform_GetVersor', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleVersor3DTransform_GetVersor`, 'returnType') = 'numeric'
attr(`ScaleVersor3DTransform_GetVersor`, "inputTypes") = c('_p_itk__simple__ScaleVersor3DTransform')
class(`ScaleVersor3DTransform_GetVersor`) = c("SWIGFunction", class('ScaleVersor3DTransform_GetVersor'))

# Start of ScaleVersor3DTransform_GetTranslation

`ScaleVersor3DTransform_GetTranslation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleVersor3DTransform_GetTranslation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleVersor3DTransform_GetTranslation`, 'returnType') = 'numeric'
attr(`ScaleVersor3DTransform_GetTranslation`, "inputTypes") = c('_p_itk__simple__ScaleVersor3DTransform')
class(`ScaleVersor3DTransform_GetTranslation`) = c("SWIGFunction", class('ScaleVersor3DTransform_GetTranslation'))

# Start of ScaleVersor3DTransform_SetTranslation

`ScaleVersor3DTransform_SetTranslation` = function(self, translation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_ScaleVersor3DTransform_SetTranslation', self, translation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScaleVersor3DTransform_SetTranslation`, 'returnType') = 'void'
attr(`ScaleVersor3DTransform_SetTranslation`, "inputTypes") = c('_p_itk__simple__ScaleVersor3DTransform', '_p_std__vectorT_double_t')
class(`ScaleVersor3DTransform_SetTranslation`) = c("SWIGFunction", class('ScaleVersor3DTransform_SetTranslation'))

# Start of ScaleVersor3DTransform_GetScale

`ScaleVersor3DTransform_GetScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleVersor3DTransform_GetScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleVersor3DTransform_GetScale`, 'returnType') = 'numeric'
attr(`ScaleVersor3DTransform_GetScale`, "inputTypes") = c('_p_itk__simple__ScaleVersor3DTransform')
class(`ScaleVersor3DTransform_GetScale`) = c("SWIGFunction", class('ScaleVersor3DTransform_GetScale'))

# Start of ScaleVersor3DTransform_SetScale

`ScaleVersor3DTransform_SetScale` = function(self, scale)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  scale = as.numeric(scale);
  ;ans = .Call('R_swig_ScaleVersor3DTransform_SetScale', self, scale, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScaleVersor3DTransform_SetScale`, 'returnType') = 'void'
attr(`ScaleVersor3DTransform_SetScale`, "inputTypes") = c('_p_itk__simple__ScaleVersor3DTransform', '_p_std__vectorT_double_t')
class(`ScaleVersor3DTransform_SetScale`) = c("SWIGFunction", class('ScaleVersor3DTransform_SetScale'))

# Start of ScaleVersor3DTransform_Translate

`ScaleVersor3DTransform_Translate` = function(self, offset)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  offset = as.numeric(offset);
  ;ans = .Call('R_swig_ScaleVersor3DTransform_Translate', self, offset, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScaleVersor3DTransform_Translate`, 'returnType') = 'void'
attr(`ScaleVersor3DTransform_Translate`, "inputTypes") = c('_p_itk__simple__ScaleVersor3DTransform', '_p_std__vectorT_double_t')
class(`ScaleVersor3DTransform_Translate`) = c("SWIGFunction", class('ScaleVersor3DTransform_Translate'))

# Start of ScaleVersor3DTransform_GetMatrix

`ScaleVersor3DTransform_GetMatrix` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScaleVersor3DTransform_GetMatrix', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScaleVersor3DTransform_GetMatrix`, 'returnType') = 'numeric'
attr(`ScaleVersor3DTransform_GetMatrix`, "inputTypes") = c('_p_itk__simple__ScaleVersor3DTransform')
class(`ScaleVersor3DTransform_GetMatrix`) = c("SWIGFunction", class('ScaleVersor3DTransform_GetMatrix'))

# Start of accessor method for itk::simple::ScaleVersor3DTransform
setMethod('$', '_p_itk__simple__ScaleVersor3DTransform', function(x, name)

{
  accessorFuns = list('Equal' = ScaleVersor3DTransform_Equal, 'GetName' = ScaleVersor3DTransform_GetName, 'SetCenter' = ScaleVersor3DTransform_SetCenter, 'GetCenter' = ScaleVersor3DTransform_GetCenter, 'SetRotation' = ScaleVersor3DTransform_SetRotation, 'GetVersor' = ScaleVersor3DTransform_GetVersor, 'GetTranslation' = ScaleVersor3DTransform_GetTranslation, 'SetTranslation' = ScaleVersor3DTransform_SetTranslation, 'GetScale' = ScaleVersor3DTransform_GetScale, 'SetScale' = ScaleVersor3DTransform_SetScale, 'Translate' = ScaleVersor3DTransform_Translate, 'GetMatrix' = ScaleVersor3DTransform_GetMatrix);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ScaleVersor3DTransform
# Start of delete_Similarity2DTransform

`delete_Similarity2DTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_Similarity2DTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_Similarity2DTransform`, 'returnType') = 'void'
attr(`delete_Similarity2DTransform`, "inputTypes") = c('_p_itk__simple__Similarity2DTransform')
class(`delete_Similarity2DTransform`) = c("SWIGFunction", class('delete_Similarity2DTransform'))

# Start of new_Similarity2DTransform

`Similarity2DTransform__SWIG_0` = function()
{
  ;ans = .Call('R_swig_new_Similarity2DTransform__SWIG_0', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity2DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity2DTransform);
  ans
  
}

attr(`Similarity2DTransform__SWIG_0`, 'returnType') = '_p_itk__simple__Similarity2DTransform'
class(`Similarity2DTransform__SWIG_0`) = c("SWIGFunction", class('Similarity2DTransform__SWIG_0'))

# Start of new_Similarity2DTransform

`Similarity2DTransform__SWIG_1` = function(scaleFactor, angle, translation, fixedCenter)
{
  translation = as.numeric(translation);
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_Similarity2DTransform__SWIG_1', scaleFactor, angle, translation, fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity2DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity2DTransform);
  ans
  
}

attr(`Similarity2DTransform__SWIG_1`, 'returnType') = '_p_itk__simple__Similarity2DTransform'
attr(`Similarity2DTransform__SWIG_1`, "inputTypes") = c('numeric', 'numeric', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`Similarity2DTransform__SWIG_1`) = c("SWIGFunction", class('Similarity2DTransform__SWIG_1'))

# Start of new_Similarity2DTransform

`Similarity2DTransform__SWIG_2` = function(scaleFactor, angle, translation)
{
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_new_Similarity2DTransform__SWIG_2', scaleFactor, angle, translation, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity2DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity2DTransform);
  ans
  
}

attr(`Similarity2DTransform__SWIG_2`, 'returnType') = '_p_itk__simple__Similarity2DTransform'
attr(`Similarity2DTransform__SWIG_2`, "inputTypes") = c('numeric', 'numeric', '_p_std__vectorT_double_t')
class(`Similarity2DTransform__SWIG_2`) = c("SWIGFunction", class('Similarity2DTransform__SWIG_2'))

# Start of new_Similarity2DTransform

`Similarity2DTransform__SWIG_3` = function(scaleFactor, angle)
{
  ;ans = .Call('R_swig_new_Similarity2DTransform__SWIG_3', scaleFactor, angle, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity2DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity2DTransform);
  ans
  
}

attr(`Similarity2DTransform__SWIG_3`, 'returnType') = '_p_itk__simple__Similarity2DTransform'
attr(`Similarity2DTransform__SWIG_3`, "inputTypes") = c('numeric', 'numeric')
class(`Similarity2DTransform__SWIG_3`) = c("SWIGFunction", class('Similarity2DTransform__SWIG_3'))

# Start of new_Similarity2DTransform

`Similarity2DTransform__SWIG_4` = function(scaleFactor)
{
  ;ans = .Call('R_swig_new_Similarity2DTransform__SWIG_4', scaleFactor, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity2DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity2DTransform);
  ans
  
}

attr(`Similarity2DTransform__SWIG_4`, 'returnType') = '_p_itk__simple__Similarity2DTransform'
attr(`Similarity2DTransform__SWIG_4`, "inputTypes") = c('numeric')
class(`Similarity2DTransform__SWIG_4`) = c("SWIGFunction", class('Similarity2DTransform__SWIG_4'))

# Start of new_Similarity2DTransform

`Similarity2DTransform__SWIG_5` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_Similarity2DTransform__SWIG_5', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity2DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity2DTransform);
  ans
  
}

attr(`Similarity2DTransform__SWIG_5`, 'returnType') = '_p_itk__simple__Similarity2DTransform'
attr(`Similarity2DTransform__SWIG_5`, "inputTypes") = c('_p_itk__simple__Similarity2DTransform')
class(`Similarity2DTransform__SWIG_5`) = c("SWIGFunction", class('Similarity2DTransform__SWIG_5'))

# Start of new_Similarity2DTransform

`Similarity2DTransform__SWIG_6` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_Similarity2DTransform__SWIG_6', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity2DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity2DTransform);
  ans
  
}

attr(`Similarity2DTransform__SWIG_6`, 'returnType') = '_p_itk__simple__Similarity2DTransform'
attr(`Similarity2DTransform__SWIG_6`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Similarity2DTransform__SWIG_6`) = c("SWIGFunction", class('Similarity2DTransform__SWIG_6'))

`Similarity2DTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 7
  if (argc == 0) {
    f <- Similarity2DTransform__SWIG_0; 
  } else if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Similarity2DTransform') && length(argv[[1]]) == 1 ) {
      f <- Similarity2DTransform__SWIG_5; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- Similarity2DTransform__SWIG_6; 
    }
    else if ( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) {
      f <- Similarity2DTransform__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Similarity2DTransform__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) )) {
      f <- Similarity2DTransform__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) )) {
      f <- Similarity2DTransform__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Similarity2DTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of Similarity2DTransform_Equal

`Similarity2DTransform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_Similarity2DTransform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity2DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Similarity2DTransform_Equal`, 'returnType') = '_p_itk__simple__Similarity2DTransform'
attr(`Similarity2DTransform_Equal`, "inputTypes") = c('_p_itk__simple__Similarity2DTransform', '_p_itk__simple__Similarity2DTransform')
class(`Similarity2DTransform_Equal`) = c("SWIGFunction", class('Similarity2DTransform_Equal'))

# Start of Similarity2DTransform_GetName

`Similarity2DTransform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Similarity2DTransform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Similarity2DTransform_GetName`, 'returnType') = 'character'
attr(`Similarity2DTransform_GetName`, "inputTypes") = c('_p_itk__simple__Similarity2DTransform')
class(`Similarity2DTransform_GetName`) = c("SWIGFunction", class('Similarity2DTransform_GetName'))

# Start of Similarity2DTransform_SetCenter

`Similarity2DTransform_SetCenter` = function(self, params)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  params = as.numeric(params);
  ;ans = .Call('R_swig_Similarity2DTransform_SetCenter', self, params, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Similarity2DTransform_SetCenter`, 'returnType') = 'void'
attr(`Similarity2DTransform_SetCenter`, "inputTypes") = c('_p_itk__simple__Similarity2DTransform', '_p_std__vectorT_double_t')
class(`Similarity2DTransform_SetCenter`) = c("SWIGFunction", class('Similarity2DTransform_SetCenter'))

# Start of Similarity2DTransform_GetCenter

`Similarity2DTransform_GetCenter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Similarity2DTransform_GetCenter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Similarity2DTransform_GetCenter`, 'returnType') = 'numeric'
attr(`Similarity2DTransform_GetCenter`, "inputTypes") = c('_p_itk__simple__Similarity2DTransform')
class(`Similarity2DTransform_GetCenter`) = c("SWIGFunction", class('Similarity2DTransform_GetCenter'))

# Start of Similarity2DTransform_SetAngle

`Similarity2DTransform_SetAngle` = function(self, angle)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_Similarity2DTransform_SetAngle', self, angle, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Similarity2DTransform_SetAngle`, 'returnType') = 'void'
attr(`Similarity2DTransform_SetAngle`, "inputTypes") = c('_p_itk__simple__Similarity2DTransform', 'numeric')
class(`Similarity2DTransform_SetAngle`) = c("SWIGFunction", class('Similarity2DTransform_SetAngle'))

# Start of Similarity2DTransform_GetAngle

`Similarity2DTransform_GetAngle` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Similarity2DTransform_GetAngle', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Similarity2DTransform_GetAngle`, 'returnType') = 'numeric'
attr(`Similarity2DTransform_GetAngle`, "inputTypes") = c('_p_itk__simple__Similarity2DTransform')
class(`Similarity2DTransform_GetAngle`) = c("SWIGFunction", class('Similarity2DTransform_GetAngle'))

# Start of Similarity2DTransform_GetTranslation

`Similarity2DTransform_GetTranslation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Similarity2DTransform_GetTranslation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Similarity2DTransform_GetTranslation`, 'returnType') = 'numeric'
attr(`Similarity2DTransform_GetTranslation`, "inputTypes") = c('_p_itk__simple__Similarity2DTransform')
class(`Similarity2DTransform_GetTranslation`) = c("SWIGFunction", class('Similarity2DTransform_GetTranslation'))

# Start of Similarity2DTransform_SetTranslation

`Similarity2DTransform_SetTranslation` = function(self, translation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_Similarity2DTransform_SetTranslation', self, translation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Similarity2DTransform_SetTranslation`, 'returnType') = 'void'
attr(`Similarity2DTransform_SetTranslation`, "inputTypes") = c('_p_itk__simple__Similarity2DTransform', '_p_std__vectorT_double_t')
class(`Similarity2DTransform_SetTranslation`) = c("SWIGFunction", class('Similarity2DTransform_SetTranslation'))

# Start of Similarity2DTransform_SetScale

`Similarity2DTransform_SetScale` = function(self, scale)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_Similarity2DTransform_SetScale', self, scale, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Similarity2DTransform_SetScale`, 'returnType') = 'void'
attr(`Similarity2DTransform_SetScale`, "inputTypes") = c('_p_itk__simple__Similarity2DTransform', 'numeric')
class(`Similarity2DTransform_SetScale`) = c("SWIGFunction", class('Similarity2DTransform_SetScale'))

# Start of Similarity2DTransform_GetScale

`Similarity2DTransform_GetScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Similarity2DTransform_GetScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Similarity2DTransform_GetScale`, 'returnType') = 'numeric'
attr(`Similarity2DTransform_GetScale`, "inputTypes") = c('_p_itk__simple__Similarity2DTransform')
class(`Similarity2DTransform_GetScale`) = c("SWIGFunction", class('Similarity2DTransform_GetScale'))

# Start of Similarity2DTransform_GetMatrix

`Similarity2DTransform_GetMatrix` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Similarity2DTransform_GetMatrix', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Similarity2DTransform_GetMatrix`, 'returnType') = 'numeric'
attr(`Similarity2DTransform_GetMatrix`, "inputTypes") = c('_p_itk__simple__Similarity2DTransform')
class(`Similarity2DTransform_GetMatrix`) = c("SWIGFunction", class('Similarity2DTransform_GetMatrix'))

# Start of Similarity2DTransform_SetMatrix

`Similarity2DTransform_SetMatrix__SWIG_0` = function(self, matrix, tolerance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  matrix = as.numeric(matrix);
  
  ;ans = .Call('R_swig_Similarity2DTransform_SetMatrix__SWIG_0', self, matrix, tolerance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Similarity2DTransform_SetMatrix__SWIG_0`, 'returnType') = 'void'
attr(`Similarity2DTransform_SetMatrix__SWIG_0`, "inputTypes") = c('_p_itk__simple__Similarity2DTransform', '_p_std__vectorT_double_t', 'numeric')
class(`Similarity2DTransform_SetMatrix__SWIG_0`) = c("SWIGFunction", class('Similarity2DTransform_SetMatrix__SWIG_0'))

# Start of Similarity2DTransform_SetMatrix

`Similarity2DTransform_SetMatrix__SWIG_1` = function(self, matrix)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  matrix = as.numeric(matrix);
  ;ans = .Call('R_swig_Similarity2DTransform_SetMatrix__SWIG_1', self, matrix, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Similarity2DTransform_SetMatrix__SWIG_1`, 'returnType') = 'void'
attr(`Similarity2DTransform_SetMatrix__SWIG_1`, "inputTypes") = c('_p_itk__simple__Similarity2DTransform', '_p_std__vectorT_double_t')
class(`Similarity2DTransform_SetMatrix__SWIG_1`) = c("SWIGFunction", class('Similarity2DTransform_SetMatrix__SWIG_1'))

`Similarity2DTransform_SetMatrix` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Similarity2DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- Similarity2DTransform_SetMatrix__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Similarity2DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Similarity2DTransform_SetMatrix__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Similarity2DTransform_SetMatrix with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::Similarity2DTransform
setMethod('$', '_p_itk__simple__Similarity2DTransform', function(x, name)

{
  accessorFuns = list('Equal' = Similarity2DTransform_Equal, 'GetName' = Similarity2DTransform_GetName, 'SetCenter' = Similarity2DTransform_SetCenter, 'GetCenter' = Similarity2DTransform_GetCenter, 'SetAngle' = Similarity2DTransform_SetAngle, 'GetAngle' = Similarity2DTransform_GetAngle, 'GetTranslation' = Similarity2DTransform_GetTranslation, 'SetTranslation' = Similarity2DTransform_SetTranslation, 'SetScale' = Similarity2DTransform_SetScale, 'GetScale' = Similarity2DTransform_GetScale, 'GetMatrix' = Similarity2DTransform_GetMatrix, 'SetMatrix' = Similarity2DTransform_SetMatrix);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::Similarity2DTransform
# Start of delete_Similarity3DTransform

`delete_Similarity3DTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_Similarity3DTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_Similarity3DTransform`, 'returnType') = 'void'
attr(`delete_Similarity3DTransform`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform')
class(`delete_Similarity3DTransform`) = c("SWIGFunction", class('delete_Similarity3DTransform'))

# Start of new_Similarity3DTransform

`Similarity3DTransform__SWIG_0` = function()
{
  ;ans = .Call('R_swig_new_Similarity3DTransform__SWIG_0', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity3DTransform);
  ans
  
}

attr(`Similarity3DTransform__SWIG_0`, 'returnType') = '_p_itk__simple__Similarity3DTransform'
class(`Similarity3DTransform__SWIG_0`) = c("SWIGFunction", class('Similarity3DTransform__SWIG_0'))

# Start of new_Similarity3DTransform

`Similarity3DTransform__SWIG_1` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_Similarity3DTransform__SWIG_1', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity3DTransform);
  ans
  
}

attr(`Similarity3DTransform__SWIG_1`, 'returnType') = '_p_itk__simple__Similarity3DTransform'
attr(`Similarity3DTransform__SWIG_1`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform')
class(`Similarity3DTransform__SWIG_1`) = c("SWIGFunction", class('Similarity3DTransform__SWIG_1'))

# Start of new_Similarity3DTransform

`Similarity3DTransform__SWIG_2` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_Similarity3DTransform__SWIG_2', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity3DTransform);
  ans
  
}

attr(`Similarity3DTransform__SWIG_2`, 'returnType') = '_p_itk__simple__Similarity3DTransform'
attr(`Similarity3DTransform__SWIG_2`, "inputTypes") = c('_p_itk__simple__Transform')
class(`Similarity3DTransform__SWIG_2`) = c("SWIGFunction", class('Similarity3DTransform__SWIG_2'))

# Start of new_Similarity3DTransform

`Similarity3DTransform__SWIG_3` = function(scaleFactor, versor, translation, fixedCenter)
{
  versor = as.numeric(versor);
  translation = as.numeric(translation);
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_Similarity3DTransform__SWIG_3', scaleFactor, versor, translation, fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity3DTransform);
  ans
  
}

attr(`Similarity3DTransform__SWIG_3`, 'returnType') = '_p_itk__simple__Similarity3DTransform'
attr(`Similarity3DTransform__SWIG_3`, "inputTypes") = c('numeric', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`Similarity3DTransform__SWIG_3`) = c("SWIGFunction", class('Similarity3DTransform__SWIG_3'))

# Start of new_Similarity3DTransform

`Similarity3DTransform__SWIG_4` = function(scaleFactor, versor, translation)
{
  versor = as.numeric(versor);
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_new_Similarity3DTransform__SWIG_4', scaleFactor, versor, translation, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity3DTransform);
  ans
  
}

attr(`Similarity3DTransform__SWIG_4`, 'returnType') = '_p_itk__simple__Similarity3DTransform'
attr(`Similarity3DTransform__SWIG_4`, "inputTypes") = c('numeric', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`Similarity3DTransform__SWIG_4`) = c("SWIGFunction", class('Similarity3DTransform__SWIG_4'))

# Start of new_Similarity3DTransform

`Similarity3DTransform__SWIG_5` = function(scaleFactor, versor)
{
  versor = as.numeric(versor);
  ;ans = .Call('R_swig_new_Similarity3DTransform__SWIG_5', scaleFactor, versor, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity3DTransform);
  ans
  
}

attr(`Similarity3DTransform__SWIG_5`, 'returnType') = '_p_itk__simple__Similarity3DTransform'
attr(`Similarity3DTransform__SWIG_5`, "inputTypes") = c('numeric', '_p_std__vectorT_double_t')
class(`Similarity3DTransform__SWIG_5`) = c("SWIGFunction", class('Similarity3DTransform__SWIG_5'))

# Start of new_Similarity3DTransform

`Similarity3DTransform__SWIG_6` = function(scaleFactor, axis, angle, translation, fixedCenter)
{
  axis = as.numeric(axis);
  
  translation = as.numeric(translation);
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_Similarity3DTransform__SWIG_6', scaleFactor, axis, angle, translation, fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity3DTransform);
  ans
  
}

attr(`Similarity3DTransform__SWIG_6`, 'returnType') = '_p_itk__simple__Similarity3DTransform'
attr(`Similarity3DTransform__SWIG_6`, "inputTypes") = c('numeric', '_p_std__vectorT_double_t', 'numeric', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`Similarity3DTransform__SWIG_6`) = c("SWIGFunction", class('Similarity3DTransform__SWIG_6'))

# Start of new_Similarity3DTransform

`Similarity3DTransform__SWIG_7` = function(scaleFactor, axis, angle, translation)
{
  axis = as.numeric(axis);
  
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_new_Similarity3DTransform__SWIG_7', scaleFactor, axis, angle, translation, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity3DTransform);
  ans
  
}

attr(`Similarity3DTransform__SWIG_7`, 'returnType') = '_p_itk__simple__Similarity3DTransform'
attr(`Similarity3DTransform__SWIG_7`, "inputTypes") = c('numeric', '_p_std__vectorT_double_t', 'numeric', '_p_std__vectorT_double_t')
class(`Similarity3DTransform__SWIG_7`) = c("SWIGFunction", class('Similarity3DTransform__SWIG_7'))

# Start of new_Similarity3DTransform

`Similarity3DTransform__SWIG_8` = function(scaleFactor, axis, angle)
{
  axis = as.numeric(axis);
  
  ;ans = .Call('R_swig_new_Similarity3DTransform__SWIG_8', scaleFactor, axis, angle, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Similarity3DTransform);
  ans
  
}

attr(`Similarity3DTransform__SWIG_8`, 'returnType') = '_p_itk__simple__Similarity3DTransform'
attr(`Similarity3DTransform__SWIG_8`, "inputTypes") = c('numeric', '_p_std__vectorT_double_t', 'numeric')
class(`Similarity3DTransform__SWIG_8`) = c("SWIGFunction", class('Similarity3DTransform__SWIG_8'))

`Similarity3DTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 0) {
    f <- Similarity3DTransform__SWIG_0; 
  } else if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Similarity3DTransform') && length(argv[[1]]) == 1 ) {
      f <- Similarity3DTransform__SWIG_1; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- Similarity3DTransform__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) )) {
      f <- Similarity3DTransform__SWIG_5; 
    }
  } else if (argc == 3) {
    if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Similarity3DTransform__SWIG_8; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) )) {
      f <- Similarity3DTransform__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) )) {
      f <- Similarity3DTransform__SWIG_7; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) )) {
      f <- Similarity3DTransform__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) )) {
      f <- Similarity3DTransform__SWIG_6; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Similarity3DTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of Similarity3DTransform_Equal

`Similarity3DTransform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_Similarity3DTransform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Similarity3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Similarity3DTransform_Equal`, 'returnType') = '_p_itk__simple__Similarity3DTransform'
attr(`Similarity3DTransform_Equal`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform', '_p_itk__simple__Similarity3DTransform')
class(`Similarity3DTransform_Equal`) = c("SWIGFunction", class('Similarity3DTransform_Equal'))

# Start of Similarity3DTransform_GetName

`Similarity3DTransform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Similarity3DTransform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Similarity3DTransform_GetName`, 'returnType') = 'character'
attr(`Similarity3DTransform_GetName`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform')
class(`Similarity3DTransform_GetName`) = c("SWIGFunction", class('Similarity3DTransform_GetName'))

# Start of Similarity3DTransform_SetCenter

`Similarity3DTransform_SetCenter` = function(self, params)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  params = as.numeric(params);
  ;ans = .Call('R_swig_Similarity3DTransform_SetCenter', self, params, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Similarity3DTransform_SetCenter`, 'returnType') = 'void'
attr(`Similarity3DTransform_SetCenter`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform', '_p_std__vectorT_double_t')
class(`Similarity3DTransform_SetCenter`) = c("SWIGFunction", class('Similarity3DTransform_SetCenter'))

# Start of Similarity3DTransform_GetCenter

`Similarity3DTransform_GetCenter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Similarity3DTransform_GetCenter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Similarity3DTransform_GetCenter`, 'returnType') = 'numeric'
attr(`Similarity3DTransform_GetCenter`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform')
class(`Similarity3DTransform_GetCenter`) = c("SWIGFunction", class('Similarity3DTransform_GetCenter'))

# Start of Similarity3DTransform_SetRotation

`Similarity3DTransform_SetRotation__SWIG_0` = function(self, versor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  versor = as.numeric(versor);
  ;ans = .Call('R_swig_Similarity3DTransform_SetRotation__SWIG_0', self, versor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Similarity3DTransform_SetRotation__SWIG_0`, 'returnType') = 'void'
attr(`Similarity3DTransform_SetRotation__SWIG_0`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform', '_p_std__vectorT_double_t')
class(`Similarity3DTransform_SetRotation__SWIG_0`) = c("SWIGFunction", class('Similarity3DTransform_SetRotation__SWIG_0'))

# Start of Similarity3DTransform_SetRotation

`Similarity3DTransform_SetRotation__SWIG_1` = function(self, axis, angle)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  axis = as.numeric(axis);
  
  ;ans = .Call('R_swig_Similarity3DTransform_SetRotation__SWIG_1', self, axis, angle, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Similarity3DTransform_SetRotation__SWIG_1`, 'returnType') = 'void'
attr(`Similarity3DTransform_SetRotation__SWIG_1`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform', '_p_std__vectorT_double_t', 'numeric')
class(`Similarity3DTransform_SetRotation__SWIG_1`) = c("SWIGFunction", class('Similarity3DTransform_SetRotation__SWIG_1'))

`Similarity3DTransform_SetRotation` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Similarity3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- Similarity3DTransform_SetRotation__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Similarity3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Similarity3DTransform_SetRotation__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Similarity3DTransform_SetRotation with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of Similarity3DTransform_GetVersor

`Similarity3DTransform_GetVersor` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Similarity3DTransform_GetVersor', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Similarity3DTransform_GetVersor`, 'returnType') = 'numeric'
attr(`Similarity3DTransform_GetVersor`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform')
class(`Similarity3DTransform_GetVersor`) = c("SWIGFunction", class('Similarity3DTransform_GetVersor'))

# Start of Similarity3DTransform_SetScale

`Similarity3DTransform_SetScale` = function(self, scale)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_Similarity3DTransform_SetScale', self, scale, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Similarity3DTransform_SetScale`, 'returnType') = 'void'
attr(`Similarity3DTransform_SetScale`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform', 'numeric')
class(`Similarity3DTransform_SetScale`) = c("SWIGFunction", class('Similarity3DTransform_SetScale'))

# Start of Similarity3DTransform_GetScale

`Similarity3DTransform_GetScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Similarity3DTransform_GetScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Similarity3DTransform_GetScale`, 'returnType') = 'numeric'
attr(`Similarity3DTransform_GetScale`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform')
class(`Similarity3DTransform_GetScale`) = c("SWIGFunction", class('Similarity3DTransform_GetScale'))

# Start of Similarity3DTransform_GetTranslation

`Similarity3DTransform_GetTranslation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Similarity3DTransform_GetTranslation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Similarity3DTransform_GetTranslation`, 'returnType') = 'numeric'
attr(`Similarity3DTransform_GetTranslation`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform')
class(`Similarity3DTransform_GetTranslation`) = c("SWIGFunction", class('Similarity3DTransform_GetTranslation'))

# Start of Similarity3DTransform_SetTranslation

`Similarity3DTransform_SetTranslation` = function(self, translation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_Similarity3DTransform_SetTranslation', self, translation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Similarity3DTransform_SetTranslation`, 'returnType') = 'void'
attr(`Similarity3DTransform_SetTranslation`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform', '_p_std__vectorT_double_t')
class(`Similarity3DTransform_SetTranslation`) = c("SWIGFunction", class('Similarity3DTransform_SetTranslation'))

# Start of Similarity3DTransform_Translate

`Similarity3DTransform_Translate` = function(self, offset)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  offset = as.numeric(offset);
  ;ans = .Call('R_swig_Similarity3DTransform_Translate', self, offset, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Similarity3DTransform_Translate`, 'returnType') = 'void'
attr(`Similarity3DTransform_Translate`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform', '_p_std__vectorT_double_t')
class(`Similarity3DTransform_Translate`) = c("SWIGFunction", class('Similarity3DTransform_Translate'))

# Start of Similarity3DTransform_GetMatrix

`Similarity3DTransform_GetMatrix` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Similarity3DTransform_GetMatrix', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Similarity3DTransform_GetMatrix`, 'returnType') = 'numeric'
attr(`Similarity3DTransform_GetMatrix`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform')
class(`Similarity3DTransform_GetMatrix`) = c("SWIGFunction", class('Similarity3DTransform_GetMatrix'))

# Start of Similarity3DTransform_SetMatrix

`Similarity3DTransform_SetMatrix__SWIG_0` = function(self, matrix, tolerance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  matrix = as.numeric(matrix);
  
  ;ans = .Call('R_swig_Similarity3DTransform_SetMatrix__SWIG_0', self, matrix, tolerance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Similarity3DTransform_SetMatrix__SWIG_0`, 'returnType') = 'void'
attr(`Similarity3DTransform_SetMatrix__SWIG_0`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform', '_p_std__vectorT_double_t', 'numeric')
class(`Similarity3DTransform_SetMatrix__SWIG_0`) = c("SWIGFunction", class('Similarity3DTransform_SetMatrix__SWIG_0'))

# Start of Similarity3DTransform_SetMatrix

`Similarity3DTransform_SetMatrix__SWIG_1` = function(self, matrix)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  matrix = as.numeric(matrix);
  ;ans = .Call('R_swig_Similarity3DTransform_SetMatrix__SWIG_1', self, matrix, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Similarity3DTransform_SetMatrix__SWIG_1`, 'returnType') = 'void'
attr(`Similarity3DTransform_SetMatrix__SWIG_1`, "inputTypes") = c('_p_itk__simple__Similarity3DTransform', '_p_std__vectorT_double_t')
class(`Similarity3DTransform_SetMatrix__SWIG_1`) = c("SWIGFunction", class('Similarity3DTransform_SetMatrix__SWIG_1'))

`Similarity3DTransform_SetMatrix` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Similarity3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- Similarity3DTransform_SetMatrix__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Similarity3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Similarity3DTransform_SetMatrix__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Similarity3DTransform_SetMatrix with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::Similarity3DTransform
setMethod('$', '_p_itk__simple__Similarity3DTransform', function(x, name)

{
  accessorFuns = list('Equal' = Similarity3DTransform_Equal, 'GetName' = Similarity3DTransform_GetName, 'SetCenter' = Similarity3DTransform_SetCenter, 'GetCenter' = Similarity3DTransform_GetCenter, 'SetRotation' = Similarity3DTransform_SetRotation, 'GetVersor' = Similarity3DTransform_GetVersor, 'SetScale' = Similarity3DTransform_SetScale, 'GetScale' = Similarity3DTransform_GetScale, 'GetTranslation' = Similarity3DTransform_GetTranslation, 'SetTranslation' = Similarity3DTransform_SetTranslation, 'Translate' = Similarity3DTransform_Translate, 'GetMatrix' = Similarity3DTransform_GetMatrix, 'SetMatrix' = Similarity3DTransform_SetMatrix);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::Similarity3DTransform
# Start of delete_TranslationTransform

`delete_TranslationTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_TranslationTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_TranslationTransform`, 'returnType') = 'void'
attr(`delete_TranslationTransform`, "inputTypes") = c('_p_itk__simple__TranslationTransform')
class(`delete_TranslationTransform`) = c("SWIGFunction", class('delete_TranslationTransform'))

# Start of new_TranslationTransform

`TranslationTransform__SWIG_0` = function(dimensions, offset)
{
  dimensions = as.integer(dimensions);
  
  if(length(dimensions) > 1) {
    warning("using only the first element of dimensions");
  };
  
  offset = as.numeric(offset);
  ;ans = .Call('R_swig_new_TranslationTransform__SWIG_0', dimensions, offset, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__TranslationTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_TranslationTransform);
  ans
  
}

attr(`TranslationTransform__SWIG_0`, 'returnType') = '_p_itk__simple__TranslationTransform'
attr(`TranslationTransform__SWIG_0`, "inputTypes") = c('integer', '_p_std__vectorT_double_t')
class(`TranslationTransform__SWIG_0`) = c("SWIGFunction", class('TranslationTransform__SWIG_0'))

# Start of new_TranslationTransform

`TranslationTransform__SWIG_1` = function(dimensions)
{
  dimensions = as.integer(dimensions);
  
  if(length(dimensions) > 1) {
    warning("using only the first element of dimensions");
  };
  
  ;ans = .Call('R_swig_new_TranslationTransform__SWIG_1', dimensions, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__TranslationTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_TranslationTransform);
  ans
  
}

attr(`TranslationTransform__SWIG_1`, 'returnType') = '_p_itk__simple__TranslationTransform'
attr(`TranslationTransform__SWIG_1`, "inputTypes") = c('integer')
class(`TranslationTransform__SWIG_1`) = c("SWIGFunction", class('TranslationTransform__SWIG_1'))

# Start of new_TranslationTransform

`TranslationTransform__SWIG_2` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_TranslationTransform__SWIG_2', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__TranslationTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_TranslationTransform);
  ans
  
}

attr(`TranslationTransform__SWIG_2`, 'returnType') = '_p_itk__simple__TranslationTransform'
attr(`TranslationTransform__SWIG_2`, "inputTypes") = c('_p_itk__simple__TranslationTransform')
class(`TranslationTransform__SWIG_2`) = c("SWIGFunction", class('TranslationTransform__SWIG_2'))

# Start of new_TranslationTransform

`TranslationTransform__SWIG_3` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_TranslationTransform__SWIG_3', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__TranslationTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_TranslationTransform);
  ans
  
}

attr(`TranslationTransform__SWIG_3`, 'returnType') = '_p_itk__simple__TranslationTransform'
attr(`TranslationTransform__SWIG_3`, "inputTypes") = c('_p_itk__simple__Transform')
class(`TranslationTransform__SWIG_3`) = c("SWIGFunction", class('TranslationTransform__SWIG_3'))

`TranslationTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__TranslationTransform') && length(argv[[1]]) == 1 ) {
      f <- TranslationTransform__SWIG_2; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- TranslationTransform__SWIG_3; 
    }
    else if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- TranslationTransform__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) )) {
      f <- TranslationTransform__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for TranslationTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of TranslationTransform_Equal

`TranslationTransform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_TranslationTransform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__TranslationTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TranslationTransform_Equal`, 'returnType') = '_p_itk__simple__TranslationTransform'
attr(`TranslationTransform_Equal`, "inputTypes") = c('_p_itk__simple__TranslationTransform', '_p_itk__simple__TranslationTransform')
class(`TranslationTransform_Equal`) = c("SWIGFunction", class('TranslationTransform_Equal'))

# Start of TranslationTransform_GetName

`TranslationTransform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TranslationTransform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TranslationTransform_GetName`, 'returnType') = 'character'
attr(`TranslationTransform_GetName`, "inputTypes") = c('_p_itk__simple__TranslationTransform')
class(`TranslationTransform_GetName`) = c("SWIGFunction", class('TranslationTransform_GetName'))

# Start of TranslationTransform_SetOffset

`TranslationTransform_SetOffset` = function(self, params)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  params = as.numeric(params);
  ;ans = .Call('R_swig_TranslationTransform_SetOffset', self, params, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TranslationTransform_SetOffset`, 'returnType') = 'void'
attr(`TranslationTransform_SetOffset`, "inputTypes") = c('_p_itk__simple__TranslationTransform', '_p_std__vectorT_double_t')
class(`TranslationTransform_SetOffset`) = c("SWIGFunction", class('TranslationTransform_SetOffset'))

# Start of TranslationTransform_GetOffset

`TranslationTransform_GetOffset` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TranslationTransform_GetOffset', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TranslationTransform_GetOffset`, 'returnType') = 'numeric'
attr(`TranslationTransform_GetOffset`, "inputTypes") = c('_p_itk__simple__TranslationTransform')
class(`TranslationTransform_GetOffset`) = c("SWIGFunction", class('TranslationTransform_GetOffset'))

# Start of accessor method for itk::simple::TranslationTransform
setMethod('$', '_p_itk__simple__TranslationTransform', function(x, name)

{
  accessorFuns = list('Equal' = TranslationTransform_Equal, 'GetName' = TranslationTransform_GetName, 'SetOffset' = TranslationTransform_SetOffset, 'GetOffset' = TranslationTransform_GetOffset);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::TranslationTransform
# Start of delete_VersorTransform

`delete_VersorTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_VersorTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_VersorTransform`, 'returnType') = 'void'
attr(`delete_VersorTransform`, "inputTypes") = c('_p_itk__simple__VersorTransform')
class(`delete_VersorTransform`) = c("SWIGFunction", class('delete_VersorTransform'))

# Start of new_VersorTransform

`VersorTransform__SWIG_0` = function()
{
  ;ans = .Call('R_swig_new_VersorTransform__SWIG_0', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorTransform);
  ans
  
}

attr(`VersorTransform__SWIG_0`, 'returnType') = '_p_itk__simple__VersorTransform'
class(`VersorTransform__SWIG_0`) = c("SWIGFunction", class('VersorTransform__SWIG_0'))

# Start of new_VersorTransform

`VersorTransform__SWIG_1` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_VersorTransform__SWIG_1', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorTransform);
  ans
  
}

attr(`VersorTransform__SWIG_1`, 'returnType') = '_p_itk__simple__VersorTransform'
attr(`VersorTransform__SWIG_1`, "inputTypes") = c('_p_itk__simple__VersorTransform')
class(`VersorTransform__SWIG_1`) = c("SWIGFunction", class('VersorTransform__SWIG_1'))

# Start of new_VersorTransform

`VersorTransform__SWIG_2` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_VersorTransform__SWIG_2', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorTransform);
  ans
  
}

attr(`VersorTransform__SWIG_2`, 'returnType') = '_p_itk__simple__VersorTransform'
attr(`VersorTransform__SWIG_2`, "inputTypes") = c('_p_itk__simple__Transform')
class(`VersorTransform__SWIG_2`) = c("SWIGFunction", class('VersorTransform__SWIG_2'))

# Start of new_VersorTransform

`VersorTransform__SWIG_3` = function(versor, fixedCenter)
{
  versor = as.numeric(versor);
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_VersorTransform__SWIG_3', versor, fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorTransform);
  ans
  
}

attr(`VersorTransform__SWIG_3`, 'returnType') = '_p_itk__simple__VersorTransform'
attr(`VersorTransform__SWIG_3`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`VersorTransform__SWIG_3`) = c("SWIGFunction", class('VersorTransform__SWIG_3'))

# Start of new_VersorTransform

`VersorTransform__SWIG_4` = function(versor)
{
  versor = as.numeric(versor);
  ;ans = .Call('R_swig_new_VersorTransform__SWIG_4', versor, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorTransform);
  ans
  
}

attr(`VersorTransform__SWIG_4`, 'returnType') = '_p_itk__simple__VersorTransform'
attr(`VersorTransform__SWIG_4`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`VersorTransform__SWIG_4`) = c("SWIGFunction", class('VersorTransform__SWIG_4'))

# Start of new_VersorTransform

`VersorTransform__SWIG_5` = function(axis, angle, fixedCenter)
{
  axis = as.numeric(axis);
  
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_VersorTransform__SWIG_5', axis, angle, fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorTransform);
  ans
  
}

attr(`VersorTransform__SWIG_5`, 'returnType') = '_p_itk__simple__VersorTransform'
attr(`VersorTransform__SWIG_5`, "inputTypes") = c('_p_std__vectorT_double_t', 'numeric', '_p_std__vectorT_double_t')
class(`VersorTransform__SWIG_5`) = c("SWIGFunction", class('VersorTransform__SWIG_5'))

# Start of new_VersorTransform

`VersorTransform__SWIG_6` = function(axis, angle)
{
  axis = as.numeric(axis);
  
  ;ans = .Call('R_swig_new_VersorTransform__SWIG_6', axis, angle, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorTransform);
  ans
  
}

attr(`VersorTransform__SWIG_6`, 'returnType') = '_p_itk__simple__VersorTransform'
attr(`VersorTransform__SWIG_6`, "inputTypes") = c('_p_std__vectorT_double_t', 'numeric')
class(`VersorTransform__SWIG_6`) = c("SWIGFunction", class('VersorTransform__SWIG_6'))

`VersorTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 7
  if (argc == 0) {
    f <- VersorTransform__SWIG_0; 
  } else if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__VersorTransform') && length(argv[[1]]) == 1 ) {
      f <- VersorTransform__SWIG_1; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- VersorTransform__SWIG_2; 
    }
    else if ( is.numeric(argv[[1]]) ) {
      f <- VersorTransform__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- VersorTransform__SWIG_6; 
    }
    else if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- VersorTransform__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) )) {
      f <- VersorTransform__SWIG_5; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VersorTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VersorTransform_Equal

`VersorTransform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_VersorTransform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VersorTransform_Equal`, 'returnType') = '_p_itk__simple__VersorTransform'
attr(`VersorTransform_Equal`, "inputTypes") = c('_p_itk__simple__VersorTransform', '_p_itk__simple__VersorTransform')
class(`VersorTransform_Equal`) = c("SWIGFunction", class('VersorTransform_Equal'))

# Start of VersorTransform_GetName

`VersorTransform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VersorTransform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VersorTransform_GetName`, 'returnType') = 'character'
attr(`VersorTransform_GetName`, "inputTypes") = c('_p_itk__simple__VersorTransform')
class(`VersorTransform_GetName`) = c("SWIGFunction", class('VersorTransform_GetName'))

# Start of VersorTransform_SetCenter

`VersorTransform_SetCenter` = function(self, params)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  params = as.numeric(params);
  ;ans = .Call('R_swig_VersorTransform_SetCenter', self, params, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VersorTransform_SetCenter`, 'returnType') = 'void'
attr(`VersorTransform_SetCenter`, "inputTypes") = c('_p_itk__simple__VersorTransform', '_p_std__vectorT_double_t')
class(`VersorTransform_SetCenter`) = c("SWIGFunction", class('VersorTransform_SetCenter'))

# Start of VersorTransform_GetCenter

`VersorTransform_GetCenter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VersorTransform_GetCenter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VersorTransform_GetCenter`, 'returnType') = 'numeric'
attr(`VersorTransform_GetCenter`, "inputTypes") = c('_p_itk__simple__VersorTransform')
class(`VersorTransform_GetCenter`) = c("SWIGFunction", class('VersorTransform_GetCenter'))

# Start of VersorTransform_SetRotation

`VersorTransform_SetRotation__SWIG_0` = function(self, versor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  versor = as.numeric(versor);
  ;ans = .Call('R_swig_VersorTransform_SetRotation__SWIG_0', self, versor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VersorTransform_SetRotation__SWIG_0`, 'returnType') = 'void'
attr(`VersorTransform_SetRotation__SWIG_0`, "inputTypes") = c('_p_itk__simple__VersorTransform', '_p_std__vectorT_double_t')
class(`VersorTransform_SetRotation__SWIG_0`) = c("SWIGFunction", class('VersorTransform_SetRotation__SWIG_0'))

# Start of VersorTransform_SetRotation

`VersorTransform_SetRotation__SWIG_1` = function(self, axis, angle)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  axis = as.numeric(axis);
  
  ;ans = .Call('R_swig_VersorTransform_SetRotation__SWIG_1', self, axis, angle, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VersorTransform_SetRotation__SWIG_1`, 'returnType') = 'void'
attr(`VersorTransform_SetRotation__SWIG_1`, "inputTypes") = c('_p_itk__simple__VersorTransform', '_p_std__vectorT_double_t', 'numeric')
class(`VersorTransform_SetRotation__SWIG_1`) = c("SWIGFunction", class('VersorTransform_SetRotation__SWIG_1'))

`VersorTransform_SetRotation` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__VersorTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- VersorTransform_SetRotation__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__VersorTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- VersorTransform_SetRotation__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VersorTransform_SetRotation with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VersorTransform_GetVersor

`VersorTransform_GetVersor` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VersorTransform_GetVersor', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VersorTransform_GetVersor`, 'returnType') = 'numeric'
attr(`VersorTransform_GetVersor`, "inputTypes") = c('_p_itk__simple__VersorTransform')
class(`VersorTransform_GetVersor`) = c("SWIGFunction", class('VersorTransform_GetVersor'))

# Start of VersorTransform_GetMatrix

`VersorTransform_GetMatrix` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VersorTransform_GetMatrix', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VersorTransform_GetMatrix`, 'returnType') = 'numeric'
attr(`VersorTransform_GetMatrix`, "inputTypes") = c('_p_itk__simple__VersorTransform')
class(`VersorTransform_GetMatrix`) = c("SWIGFunction", class('VersorTransform_GetMatrix'))

# Start of VersorTransform_SetMatrix

`VersorTransform_SetMatrix__SWIG_0` = function(self, matrix, tolerance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  matrix = as.numeric(matrix);
  
  ;ans = .Call('R_swig_VersorTransform_SetMatrix__SWIG_0', self, matrix, tolerance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VersorTransform_SetMatrix__SWIG_0`, 'returnType') = 'void'
attr(`VersorTransform_SetMatrix__SWIG_0`, "inputTypes") = c('_p_itk__simple__VersorTransform', '_p_std__vectorT_double_t', 'numeric')
class(`VersorTransform_SetMatrix__SWIG_0`) = c("SWIGFunction", class('VersorTransform_SetMatrix__SWIG_0'))

# Start of VersorTransform_SetMatrix

`VersorTransform_SetMatrix__SWIG_1` = function(self, matrix)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  matrix = as.numeric(matrix);
  ;ans = .Call('R_swig_VersorTransform_SetMatrix__SWIG_1', self, matrix, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VersorTransform_SetMatrix__SWIG_1`, 'returnType') = 'void'
attr(`VersorTransform_SetMatrix__SWIG_1`, "inputTypes") = c('_p_itk__simple__VersorTransform', '_p_std__vectorT_double_t')
class(`VersorTransform_SetMatrix__SWIG_1`) = c("SWIGFunction", class('VersorTransform_SetMatrix__SWIG_1'))

`VersorTransform_SetMatrix` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__VersorTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- VersorTransform_SetMatrix__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__VersorTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- VersorTransform_SetMatrix__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VersorTransform_SetMatrix with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::VersorTransform
setMethod('$', '_p_itk__simple__VersorTransform', function(x, name)

{
  accessorFuns = list('Equal' = VersorTransform_Equal, 'GetName' = VersorTransform_GetName, 'SetCenter' = VersorTransform_SetCenter, 'GetCenter' = VersorTransform_GetCenter, 'SetRotation' = VersorTransform_SetRotation, 'GetVersor' = VersorTransform_GetVersor, 'GetMatrix' = VersorTransform_GetMatrix, 'SetMatrix' = VersorTransform_SetMatrix);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::VersorTransform
# Start of delete_VersorRigid3DTransform

`delete_VersorRigid3DTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_VersorRigid3DTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_VersorRigid3DTransform`, 'returnType') = 'void'
attr(`delete_VersorRigid3DTransform`, "inputTypes") = c('_p_itk__simple__VersorRigid3DTransform')
class(`delete_VersorRigid3DTransform`) = c("SWIGFunction", class('delete_VersorRigid3DTransform'))

# Start of new_VersorRigid3DTransform

`VersorRigid3DTransform__SWIG_0` = function()
{
  ;ans = .Call('R_swig_new_VersorRigid3DTransform__SWIG_0', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorRigid3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorRigid3DTransform);
  ans
  
}

attr(`VersorRigid3DTransform__SWIG_0`, 'returnType') = '_p_itk__simple__VersorRigid3DTransform'
class(`VersorRigid3DTransform__SWIG_0`) = c("SWIGFunction", class('VersorRigid3DTransform__SWIG_0'))

# Start of new_VersorRigid3DTransform

`VersorRigid3DTransform__SWIG_1` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_VersorRigid3DTransform__SWIG_1', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorRigid3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorRigid3DTransform);
  ans
  
}

attr(`VersorRigid3DTransform__SWIG_1`, 'returnType') = '_p_itk__simple__VersorRigid3DTransform'
attr(`VersorRigid3DTransform__SWIG_1`, "inputTypes") = c('_p_itk__simple__VersorRigid3DTransform')
class(`VersorRigid3DTransform__SWIG_1`) = c("SWIGFunction", class('VersorRigid3DTransform__SWIG_1'))

# Start of new_VersorRigid3DTransform

`VersorRigid3DTransform__SWIG_2` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_VersorRigid3DTransform__SWIG_2', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorRigid3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorRigid3DTransform);
  ans
  
}

attr(`VersorRigid3DTransform__SWIG_2`, 'returnType') = '_p_itk__simple__VersorRigid3DTransform'
attr(`VersorRigid3DTransform__SWIG_2`, "inputTypes") = c('_p_itk__simple__Transform')
class(`VersorRigid3DTransform__SWIG_2`) = c("SWIGFunction", class('VersorRigid3DTransform__SWIG_2'))

# Start of new_VersorRigid3DTransform

`VersorRigid3DTransform__SWIG_3` = function(versor, translation, fixedCenter)
{
  versor = as.numeric(versor);
  translation = as.numeric(translation);
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_VersorRigid3DTransform__SWIG_3', versor, translation, fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorRigid3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorRigid3DTransform);
  ans
  
}

attr(`VersorRigid3DTransform__SWIG_3`, 'returnType') = '_p_itk__simple__VersorRigid3DTransform'
attr(`VersorRigid3DTransform__SWIG_3`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`VersorRigid3DTransform__SWIG_3`) = c("SWIGFunction", class('VersorRigid3DTransform__SWIG_3'))

# Start of new_VersorRigid3DTransform

`VersorRigid3DTransform__SWIG_4` = function(versor, translation)
{
  versor = as.numeric(versor);
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_new_VersorRigid3DTransform__SWIG_4', versor, translation, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorRigid3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorRigid3DTransform);
  ans
  
}

attr(`VersorRigid3DTransform__SWIG_4`, 'returnType') = '_p_itk__simple__VersorRigid3DTransform'
attr(`VersorRigid3DTransform__SWIG_4`, "inputTypes") = c('_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`VersorRigid3DTransform__SWIG_4`) = c("SWIGFunction", class('VersorRigid3DTransform__SWIG_4'))

# Start of new_VersorRigid3DTransform

`VersorRigid3DTransform__SWIG_5` = function(versor)
{
  versor = as.numeric(versor);
  ;ans = .Call('R_swig_new_VersorRigid3DTransform__SWIG_5', versor, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorRigid3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorRigid3DTransform);
  ans
  
}

attr(`VersorRigid3DTransform__SWIG_5`, 'returnType') = '_p_itk__simple__VersorRigid3DTransform'
attr(`VersorRigid3DTransform__SWIG_5`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`VersorRigid3DTransform__SWIG_5`) = c("SWIGFunction", class('VersorRigid3DTransform__SWIG_5'))

# Start of new_VersorRigid3DTransform

`VersorRigid3DTransform__SWIG_6` = function(axis, angle, translation, fixedCenter)
{
  axis = as.numeric(axis);
  
  translation = as.numeric(translation);
  fixedCenter = as.numeric(fixedCenter);
  ;ans = .Call('R_swig_new_VersorRigid3DTransform__SWIG_6', axis, angle, translation, fixedCenter, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorRigid3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorRigid3DTransform);
  ans
  
}

attr(`VersorRigid3DTransform__SWIG_6`, 'returnType') = '_p_itk__simple__VersorRigid3DTransform'
attr(`VersorRigid3DTransform__SWIG_6`, "inputTypes") = c('_p_std__vectorT_double_t', 'numeric', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`VersorRigid3DTransform__SWIG_6`) = c("SWIGFunction", class('VersorRigid3DTransform__SWIG_6'))

# Start of new_VersorRigid3DTransform

`VersorRigid3DTransform__SWIG_7` = function(axis, angle, translation)
{
  axis = as.numeric(axis);
  
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_new_VersorRigid3DTransform__SWIG_7', axis, angle, translation, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorRigid3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorRigid3DTransform);
  ans
  
}

attr(`VersorRigid3DTransform__SWIG_7`, 'returnType') = '_p_itk__simple__VersorRigid3DTransform'
attr(`VersorRigid3DTransform__SWIG_7`, "inputTypes") = c('_p_std__vectorT_double_t', 'numeric', '_p_std__vectorT_double_t')
class(`VersorRigid3DTransform__SWIG_7`) = c("SWIGFunction", class('VersorRigid3DTransform__SWIG_7'))

# Start of new_VersorRigid3DTransform

`VersorRigid3DTransform__SWIG_8` = function(axis, angle)
{
  axis = as.numeric(axis);
  
  ;ans = .Call('R_swig_new_VersorRigid3DTransform__SWIG_8', axis, angle, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorRigid3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VersorRigid3DTransform);
  ans
  
}

attr(`VersorRigid3DTransform__SWIG_8`, 'returnType') = '_p_itk__simple__VersorRigid3DTransform'
attr(`VersorRigid3DTransform__SWIG_8`, "inputTypes") = c('_p_std__vectorT_double_t', 'numeric')
class(`VersorRigid3DTransform__SWIG_8`) = c("SWIGFunction", class('VersorRigid3DTransform__SWIG_8'))

`VersorRigid3DTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 0) {
    f <- VersorRigid3DTransform__SWIG_0; 
  } else if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__VersorRigid3DTransform') && length(argv[[1]]) == 1 ) {
      f <- VersorRigid3DTransform__SWIG_1; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- VersorRigid3DTransform__SWIG_2; 
    }
    else if ( is.numeric(argv[[1]]) ) {
      f <- VersorRigid3DTransform__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- VersorRigid3DTransform__SWIG_8; 
    }
    else if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- VersorRigid3DTransform__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) )) {
      f <- VersorRigid3DTransform__SWIG_7; 
    }
    else if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) )) {
      f <- VersorRigid3DTransform__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( is.numeric(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) )) {
      f <- VersorRigid3DTransform__SWIG_6; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VersorRigid3DTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VersorRigid3DTransform_Equal

`VersorRigid3DTransform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_VersorRigid3DTransform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VersorRigid3DTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VersorRigid3DTransform_Equal`, 'returnType') = '_p_itk__simple__VersorRigid3DTransform'
attr(`VersorRigid3DTransform_Equal`, "inputTypes") = c('_p_itk__simple__VersorRigid3DTransform', '_p_itk__simple__VersorRigid3DTransform')
class(`VersorRigid3DTransform_Equal`) = c("SWIGFunction", class('VersorRigid3DTransform_Equal'))

# Start of VersorRigid3DTransform_GetName

`VersorRigid3DTransform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VersorRigid3DTransform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VersorRigid3DTransform_GetName`, 'returnType') = 'character'
attr(`VersorRigid3DTransform_GetName`, "inputTypes") = c('_p_itk__simple__VersorRigid3DTransform')
class(`VersorRigid3DTransform_GetName`) = c("SWIGFunction", class('VersorRigid3DTransform_GetName'))

# Start of VersorRigid3DTransform_SetCenter

`VersorRigid3DTransform_SetCenter` = function(self, params)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  params = as.numeric(params);
  ;ans = .Call('R_swig_VersorRigid3DTransform_SetCenter', self, params, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VersorRigid3DTransform_SetCenter`, 'returnType') = 'void'
attr(`VersorRigid3DTransform_SetCenter`, "inputTypes") = c('_p_itk__simple__VersorRigid3DTransform', '_p_std__vectorT_double_t')
class(`VersorRigid3DTransform_SetCenter`) = c("SWIGFunction", class('VersorRigid3DTransform_SetCenter'))

# Start of VersorRigid3DTransform_GetCenter

`VersorRigid3DTransform_GetCenter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VersorRigid3DTransform_GetCenter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VersorRigid3DTransform_GetCenter`, 'returnType') = 'numeric'
attr(`VersorRigid3DTransform_GetCenter`, "inputTypes") = c('_p_itk__simple__VersorRigid3DTransform')
class(`VersorRigid3DTransform_GetCenter`) = c("SWIGFunction", class('VersorRigid3DTransform_GetCenter'))

# Start of VersorRigid3DTransform_SetRotation

`VersorRigid3DTransform_SetRotation__SWIG_0` = function(self, versor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  versor = as.numeric(versor);
  ;ans = .Call('R_swig_VersorRigid3DTransform_SetRotation__SWIG_0', self, versor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VersorRigid3DTransform_SetRotation__SWIG_0`, 'returnType') = 'void'
attr(`VersorRigid3DTransform_SetRotation__SWIG_0`, "inputTypes") = c('_p_itk__simple__VersorRigid3DTransform', '_p_std__vectorT_double_t')
class(`VersorRigid3DTransform_SetRotation__SWIG_0`) = c("SWIGFunction", class('VersorRigid3DTransform_SetRotation__SWIG_0'))

# Start of VersorRigid3DTransform_SetRotation

`VersorRigid3DTransform_SetRotation__SWIG_1` = function(self, axis, angle)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  axis = as.numeric(axis);
  
  ;ans = .Call('R_swig_VersorRigid3DTransform_SetRotation__SWIG_1', self, axis, angle, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VersorRigid3DTransform_SetRotation__SWIG_1`, 'returnType') = 'void'
attr(`VersorRigid3DTransform_SetRotation__SWIG_1`, "inputTypes") = c('_p_itk__simple__VersorRigid3DTransform', '_p_std__vectorT_double_t', 'numeric')
class(`VersorRigid3DTransform_SetRotation__SWIG_1`) = c("SWIGFunction", class('VersorRigid3DTransform_SetRotation__SWIG_1'))

`VersorRigid3DTransform_SetRotation` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__VersorRigid3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- VersorRigid3DTransform_SetRotation__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__VersorRigid3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- VersorRigid3DTransform_SetRotation__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VersorRigid3DTransform_SetRotation with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VersorRigid3DTransform_GetVersor

`VersorRigid3DTransform_GetVersor` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VersorRigid3DTransform_GetVersor', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VersorRigid3DTransform_GetVersor`, 'returnType') = 'numeric'
attr(`VersorRigid3DTransform_GetVersor`, "inputTypes") = c('_p_itk__simple__VersorRigid3DTransform')
class(`VersorRigid3DTransform_GetVersor`) = c("SWIGFunction", class('VersorRigid3DTransform_GetVersor'))

# Start of VersorRigid3DTransform_GetTranslation

`VersorRigid3DTransform_GetTranslation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VersorRigid3DTransform_GetTranslation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VersorRigid3DTransform_GetTranslation`, 'returnType') = 'numeric'
attr(`VersorRigid3DTransform_GetTranslation`, "inputTypes") = c('_p_itk__simple__VersorRigid3DTransform')
class(`VersorRigid3DTransform_GetTranslation`) = c("SWIGFunction", class('VersorRigid3DTransform_GetTranslation'))

# Start of VersorRigid3DTransform_SetTranslation

`VersorRigid3DTransform_SetTranslation` = function(self, translation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  translation = as.numeric(translation);
  ;ans = .Call('R_swig_VersorRigid3DTransform_SetTranslation', self, translation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VersorRigid3DTransform_SetTranslation`, 'returnType') = 'void'
attr(`VersorRigid3DTransform_SetTranslation`, "inputTypes") = c('_p_itk__simple__VersorRigid3DTransform', '_p_std__vectorT_double_t')
class(`VersorRigid3DTransform_SetTranslation`) = c("SWIGFunction", class('VersorRigid3DTransform_SetTranslation'))

# Start of VersorRigid3DTransform_Translate

`VersorRigid3DTransform_Translate` = function(self, offset)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  offset = as.numeric(offset);
  ;ans = .Call('R_swig_VersorRigid3DTransform_Translate', self, offset, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VersorRigid3DTransform_Translate`, 'returnType') = 'void'
attr(`VersorRigid3DTransform_Translate`, "inputTypes") = c('_p_itk__simple__VersorRigid3DTransform', '_p_std__vectorT_double_t')
class(`VersorRigid3DTransform_Translate`) = c("SWIGFunction", class('VersorRigid3DTransform_Translate'))

# Start of VersorRigid3DTransform_GetMatrix

`VersorRigid3DTransform_GetMatrix` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VersorRigid3DTransform_GetMatrix', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VersorRigid3DTransform_GetMatrix`, 'returnType') = 'numeric'
attr(`VersorRigid3DTransform_GetMatrix`, "inputTypes") = c('_p_itk__simple__VersorRigid3DTransform')
class(`VersorRigid3DTransform_GetMatrix`) = c("SWIGFunction", class('VersorRigid3DTransform_GetMatrix'))

# Start of VersorRigid3DTransform_SetMatrix

`VersorRigid3DTransform_SetMatrix__SWIG_0` = function(self, matrix, tolerance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  matrix = as.numeric(matrix);
  
  ;ans = .Call('R_swig_VersorRigid3DTransform_SetMatrix__SWIG_0', self, matrix, tolerance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VersorRigid3DTransform_SetMatrix__SWIG_0`, 'returnType') = 'void'
attr(`VersorRigid3DTransform_SetMatrix__SWIG_0`, "inputTypes") = c('_p_itk__simple__VersorRigid3DTransform', '_p_std__vectorT_double_t', 'numeric')
class(`VersorRigid3DTransform_SetMatrix__SWIG_0`) = c("SWIGFunction", class('VersorRigid3DTransform_SetMatrix__SWIG_0'))

# Start of VersorRigid3DTransform_SetMatrix

`VersorRigid3DTransform_SetMatrix__SWIG_1` = function(self, matrix)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  matrix = as.numeric(matrix);
  ;ans = .Call('R_swig_VersorRigid3DTransform_SetMatrix__SWIG_1', self, matrix, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VersorRigid3DTransform_SetMatrix__SWIG_1`, 'returnType') = 'void'
attr(`VersorRigid3DTransform_SetMatrix__SWIG_1`, "inputTypes") = c('_p_itk__simple__VersorRigid3DTransform', '_p_std__vectorT_double_t')
class(`VersorRigid3DTransform_SetMatrix__SWIG_1`) = c("SWIGFunction", class('VersorRigid3DTransform_SetMatrix__SWIG_1'))

`VersorRigid3DTransform_SetMatrix` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__VersorRigid3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- VersorRigid3DTransform_SetMatrix__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__VersorRigid3DTransform') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- VersorRigid3DTransform_SetMatrix__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VersorRigid3DTransform_SetMatrix with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::VersorRigid3DTransform
setMethod('$', '_p_itk__simple__VersorRigid3DTransform', function(x, name)

{
  accessorFuns = list('Equal' = VersorRigid3DTransform_Equal, 'GetName' = VersorRigid3DTransform_GetName, 'SetCenter' = VersorRigid3DTransform_SetCenter, 'GetCenter' = VersorRigid3DTransform_GetCenter, 'SetRotation' = VersorRigid3DTransform_SetRotation, 'GetVersor' = VersorRigid3DTransform_GetVersor, 'GetTranslation' = VersorRigid3DTransform_GetTranslation, 'SetTranslation' = VersorRigid3DTransform_SetTranslation, 'Translate' = VersorRigid3DTransform_Translate, 'GetMatrix' = VersorRigid3DTransform_GetMatrix, 'SetMatrix' = VersorRigid3DTransform_SetMatrix);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::VersorRigid3DTransform
# Start of new_CompositeTransform

`CompositeTransform__SWIG_0` = function(dimensions)
{
  dimensions = as.integer(dimensions);
  
  if(length(dimensions) > 1) {
    warning("using only the first element of dimensions");
  };
  
  ;ans = .Call('R_swig_new_CompositeTransform__SWIG_0', dimensions, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CompositeTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CompositeTransform);
  ans
  
}

attr(`CompositeTransform__SWIG_0`, 'returnType') = '_p_itk__simple__CompositeTransform'
attr(`CompositeTransform__SWIG_0`, "inputTypes") = c('integer')
class(`CompositeTransform__SWIG_0`) = c("SWIGFunction", class('CompositeTransform__SWIG_0'))

# Start of new_CompositeTransform

`CompositeTransform__SWIG_1` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_CompositeTransform__SWIG_1', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CompositeTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CompositeTransform);
  ans
  
}

attr(`CompositeTransform__SWIG_1`, 'returnType') = '_p_itk__simple__CompositeTransform'
attr(`CompositeTransform__SWIG_1`, "inputTypes") = c('_p_itk__simple__Transform')
class(`CompositeTransform__SWIG_1`) = c("SWIGFunction", class('CompositeTransform__SWIG_1'))

# Start of new_CompositeTransform

`CompositeTransform__SWIG_2` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_CompositeTransform__SWIG_2', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CompositeTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CompositeTransform);
  ans
  
}

attr(`CompositeTransform__SWIG_2`, 'returnType') = '_p_itk__simple__CompositeTransform'
attr(`CompositeTransform__SWIG_2`, "inputTypes") = c('_p_itk__simple__CompositeTransform')
class(`CompositeTransform__SWIG_2`) = c("SWIGFunction", class('CompositeTransform__SWIG_2'))

# Start of new_CompositeTransform

`CompositeTransform__SWIG_3` = function(s_arg1)
{
  if (inherits(s_arg1, "ExternalReference")) s_arg1 = slot(s_arg1,"ref"); 
  ;ans = .Call('R_swig_new_CompositeTransform__SWIG_3', s_arg1, PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CompositeTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CompositeTransform);
  ans
  
}

attr(`CompositeTransform__SWIG_3`, 'returnType') = '_p_itk__simple__CompositeTransform'
attr(`CompositeTransform__SWIG_3`, "inputTypes") = c('_p_std__vectorT_itk__simple__Transform_t')
class(`CompositeTransform__SWIG_3`) = c("SWIGFunction", class('CompositeTransform__SWIG_3'))

`CompositeTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__CompositeTransform') && length(argv[[1]]) == 1 ) {
      f <- CompositeTransform__SWIG_2; 
    }
    else if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- CompositeTransform__SWIG_1; 
    }
    else if ( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) {
      f <- CompositeTransform__SWIG_0; 
    }
    else if ( extends(argtypes[1], '_p_std__vectorT_itk__simple__Transform_t') && length(argv[[1]]) == 1 ) {
      f <- CompositeTransform__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for CompositeTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_CompositeTransform

`delete_CompositeTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_CompositeTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_CompositeTransform`, 'returnType') = 'void'
attr(`delete_CompositeTransform`, "inputTypes") = c('_p_itk__simple__CompositeTransform')
class(`delete_CompositeTransform`) = c("SWIGFunction", class('delete_CompositeTransform'))

# Start of CompositeTransform_Equal

`CompositeTransform_Equal` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_CompositeTransform_Equal', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CompositeTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CompositeTransform_Equal`, 'returnType') = '_p_itk__simple__CompositeTransform'
attr(`CompositeTransform_Equal`, "inputTypes") = c('_p_itk__simple__CompositeTransform', '_p_itk__simple__CompositeTransform')
class(`CompositeTransform_Equal`) = c("SWIGFunction", class('CompositeTransform_Equal'))

# Start of CompositeTransform_GetName

`CompositeTransform_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CompositeTransform_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CompositeTransform_GetName`, 'returnType') = 'character'
attr(`CompositeTransform_GetName`, "inputTypes") = c('_p_itk__simple__CompositeTransform')
class(`CompositeTransform_GetName`) = c("SWIGFunction", class('CompositeTransform_GetName'))

# Start of CompositeTransform_FlattenTransform

`CompositeTransform_FlattenTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CompositeTransform_FlattenTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CompositeTransform_FlattenTransform`, 'returnType') = 'void'
attr(`CompositeTransform_FlattenTransform`, "inputTypes") = c('_p_itk__simple__CompositeTransform')
class(`CompositeTransform_FlattenTransform`) = c("SWIGFunction", class('CompositeTransform_FlattenTransform'))

# Start of CompositeTransform_AddTransform

`CompositeTransform_AddTransform` = function(self, t)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(t, "ExternalReference")) t = slot(t,"ref"); 
  ;ans = .Call('R_swig_CompositeTransform_AddTransform', self, t, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CompositeTransform_AddTransform`, 'returnType') = 'void'
attr(`CompositeTransform_AddTransform`, "inputTypes") = c('_p_itk__simple__CompositeTransform', '_p_itk__simple__Transform')
class(`CompositeTransform_AddTransform`) = c("SWIGFunction", class('CompositeTransform_AddTransform'))

# Start of CompositeTransform_GetNumberOfTransforms

`CompositeTransform_GetNumberOfTransforms` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CompositeTransform_GetNumberOfTransforms', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CompositeTransform_GetNumberOfTransforms`, 'returnType') = 'integer'
attr(`CompositeTransform_GetNumberOfTransforms`, "inputTypes") = c('_p_itk__simple__CompositeTransform')
class(`CompositeTransform_GetNumberOfTransforms`) = c("SWIGFunction", class('CompositeTransform_GetNumberOfTransforms'))

# Start of CompositeTransform_ClearTransforms

`CompositeTransform_ClearTransforms` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CompositeTransform_ClearTransforms', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CompositeTransform_ClearTransforms`, 'returnType') = 'void'
attr(`CompositeTransform_ClearTransforms`, "inputTypes") = c('_p_itk__simple__CompositeTransform')
class(`CompositeTransform_ClearTransforms`) = c("SWIGFunction", class('CompositeTransform_ClearTransforms'))

# Start of CompositeTransform_RemoveTransform

`CompositeTransform_RemoveTransform` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CompositeTransform_RemoveTransform', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CompositeTransform_RemoveTransform`, 'returnType') = 'void'
attr(`CompositeTransform_RemoveTransform`, "inputTypes") = c('_p_itk__simple__CompositeTransform')
class(`CompositeTransform_RemoveTransform`) = c("SWIGFunction", class('CompositeTransform_RemoveTransform'))

# Start of CompositeTransform_GetBackTransform

`CompositeTransform_GetBackTransform` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CompositeTransform_GetBackTransform', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CompositeTransform_GetBackTransform`, 'returnType') = '_p_itk__simple__Transform'
attr(`CompositeTransform_GetBackTransform`, "inputTypes") = c('_p_itk__simple__CompositeTransform')
class(`CompositeTransform_GetBackTransform`) = c("SWIGFunction", class('CompositeTransform_GetBackTransform'))

# Start of CompositeTransform_GetNthTransform

`CompositeTransform_GetNthTransform` = function(self, n, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;ans = .Call('R_swig_CompositeTransform_GetNthTransform', self, n, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CompositeTransform_GetNthTransform`, 'returnType') = '_p_itk__simple__Transform'
attr(`CompositeTransform_GetNthTransform`, "inputTypes") = c('_p_itk__simple__CompositeTransform', 'integer')
class(`CompositeTransform_GetNthTransform`) = c("SWIGFunction", class('CompositeTransform_GetNthTransform'))

# Start of accessor method for itk::simple::CompositeTransform
setMethod('$', '_p_itk__simple__CompositeTransform', function(x, name)

{
  accessorFuns = list('Equal' = CompositeTransform_Equal, 'GetName' = CompositeTransform_GetName, 'FlattenTransform' = CompositeTransform_FlattenTransform, 'AddTransform' = CompositeTransform_AddTransform, 'GetNumberOfTransforms' = CompositeTransform_GetNumberOfTransforms, 'ClearTransforms' = CompositeTransform_ClearTransforms, 'RemoveTransform' = CompositeTransform_RemoveTransform, 'GetBackTransform' = CompositeTransform_GetBackTransform, 'GetNthTransform' = CompositeTransform_GetNthTransform);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::CompositeTransform
# Start of delete_ProcessObject

`delete_ProcessObject` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ProcessObject', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ProcessObject`, 'returnType') = 'void'
attr(`delete_ProcessObject`, "inputTypes") = c('_p_itk__simple__ProcessObject')
class(`delete_ProcessObject`) = c("SWIGFunction", class('delete_ProcessObject'))

# Start of ProcessObject_ToString

`ProcessObject_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ProcessObject_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProcessObject_ToString`, 'returnType') = 'character'
attr(`ProcessObject_ToString`, "inputTypes") = c('_p_itk__simple__ProcessObject')
class(`ProcessObject_ToString`) = c("SWIGFunction", class('ProcessObject_ToString'))

# Start of ProcessObject_GetName

`ProcessObject_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ProcessObject_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProcessObject_GetName`, 'returnType') = 'character'
attr(`ProcessObject_GetName`, "inputTypes") = c('_p_itk__simple__ProcessObject')
class(`ProcessObject_GetName`) = c("SWIGFunction", class('ProcessObject_GetName'))

# Start of ProcessObject_DebugOn

`ProcessObject_DebugOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ProcessObject_DebugOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_DebugOn`, 'returnType') = 'void'
attr(`ProcessObject_DebugOn`, "inputTypes") = c('_p_itk__simple__ProcessObject')
class(`ProcessObject_DebugOn`) = c("SWIGFunction", class('ProcessObject_DebugOn'))

# Start of ProcessObject_DebugOff

`ProcessObject_DebugOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ProcessObject_DebugOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_DebugOff`, 'returnType') = 'void'
attr(`ProcessObject_DebugOff`, "inputTypes") = c('_p_itk__simple__ProcessObject')
class(`ProcessObject_DebugOff`) = c("SWIGFunction", class('ProcessObject_DebugOff'))

# Start of ProcessObject_GetDebug

`ProcessObject_GetDebug` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ProcessObject_GetDebug', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProcessObject_GetDebug`, 'returnType') = 'logical'
attr(`ProcessObject_GetDebug`, "inputTypes") = c('_p_itk__simple__ProcessObject')
class(`ProcessObject_GetDebug`) = c("SWIGFunction", class('ProcessObject_GetDebug'))

# Start of ProcessObject_SetDebug

`ProcessObject_SetDebug` = function(self, debugFlag)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  debugFlag = as.logical(debugFlag);
  ;ans = .Call('R_swig_ProcessObject_SetDebug', self, debugFlag, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_SetDebug`, 'returnType') = 'void'
attr(`ProcessObject_SetDebug`, "inputTypes") = c('_p_itk__simple__ProcessObject', 'logical')
class(`ProcessObject_SetDebug`) = c("SWIGFunction", class('ProcessObject_SetDebug'))

# Start of ProcessObject_GlobalDefaultDebugOn

`ProcessObject_GlobalDefaultDebugOn` = function()
{
  ;ans = .Call('R_swig_ProcessObject_GlobalDefaultDebugOn', PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_GlobalDefaultDebugOn`, 'returnType') = 'void'
class(`ProcessObject_GlobalDefaultDebugOn`) = c("SWIGFunction", class('ProcessObject_GlobalDefaultDebugOn'))

# Start of ProcessObject_GlobalDefaultDebugOff

`ProcessObject_GlobalDefaultDebugOff` = function()
{
  ;ans = .Call('R_swig_ProcessObject_GlobalDefaultDebugOff', PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_GlobalDefaultDebugOff`, 'returnType') = 'void'
class(`ProcessObject_GlobalDefaultDebugOff`) = c("SWIGFunction", class('ProcessObject_GlobalDefaultDebugOff'))

# Start of ProcessObject_GetGlobalDefaultDebug

`ProcessObject_GetGlobalDefaultDebug` = function(.copy = FALSE)
{
  ;.Call('R_swig_ProcessObject_GetGlobalDefaultDebug', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProcessObject_GetGlobalDefaultDebug`, 'returnType') = 'logical'
class(`ProcessObject_GetGlobalDefaultDebug`) = c("SWIGFunction", class('ProcessObject_GetGlobalDefaultDebug'))

# Start of ProcessObject_SetGlobalDefaultDebug

`ProcessObject_SetGlobalDefaultDebug` = function(debugFlag)
{
  debugFlag = as.logical(debugFlag);
  ;ans = .Call('R_swig_ProcessObject_SetGlobalDefaultDebug', debugFlag, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_SetGlobalDefaultDebug`, 'returnType') = 'void'
attr(`ProcessObject_SetGlobalDefaultDebug`, "inputTypes") = c('logical')
class(`ProcessObject_SetGlobalDefaultDebug`) = c("SWIGFunction", class('ProcessObject_SetGlobalDefaultDebug'))

# Start of ProcessObject_GlobalWarningDisplayOn

`ProcessObject_GlobalWarningDisplayOn` = function()
{
  ;ans = .Call('R_swig_ProcessObject_GlobalWarningDisplayOn', PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_GlobalWarningDisplayOn`, 'returnType') = 'void'
class(`ProcessObject_GlobalWarningDisplayOn`) = c("SWIGFunction", class('ProcessObject_GlobalWarningDisplayOn'))

# Start of ProcessObject_GlobalWarningDisplayOff

`ProcessObject_GlobalWarningDisplayOff` = function()
{
  ;ans = .Call('R_swig_ProcessObject_GlobalWarningDisplayOff', PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_GlobalWarningDisplayOff`, 'returnType') = 'void'
class(`ProcessObject_GlobalWarningDisplayOff`) = c("SWIGFunction", class('ProcessObject_GlobalWarningDisplayOff'))

# Start of ProcessObject_SetGlobalWarningDisplay

`ProcessObject_SetGlobalWarningDisplay` = function(flag)
{
  flag = as.logical(flag);
  ;ans = .Call('R_swig_ProcessObject_SetGlobalWarningDisplay', flag, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_SetGlobalWarningDisplay`, 'returnType') = 'void'
attr(`ProcessObject_SetGlobalWarningDisplay`, "inputTypes") = c('logical')
class(`ProcessObject_SetGlobalWarningDisplay`) = c("SWIGFunction", class('ProcessObject_SetGlobalWarningDisplay'))

# Start of ProcessObject_GetGlobalWarningDisplay

`ProcessObject_GetGlobalWarningDisplay` = function(.copy = FALSE)
{
  ;.Call('R_swig_ProcessObject_GetGlobalWarningDisplay', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProcessObject_GetGlobalWarningDisplay`, 'returnType') = 'logical'
class(`ProcessObject_GetGlobalWarningDisplay`) = c("SWIGFunction", class('ProcessObject_GetGlobalWarningDisplay'))

# Start of ProcessObject_GetGlobalDefaultCoordinateTolerance

`ProcessObject_GetGlobalDefaultCoordinateTolerance` = function(.copy = FALSE)
{
  ;.Call('R_swig_ProcessObject_GetGlobalDefaultCoordinateTolerance', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProcessObject_GetGlobalDefaultCoordinateTolerance`, 'returnType') = 'numeric'
class(`ProcessObject_GetGlobalDefaultCoordinateTolerance`) = c("SWIGFunction", class('ProcessObject_GetGlobalDefaultCoordinateTolerance'))

# Start of ProcessObject_SetGlobalDefaultCoordinateTolerance

`ProcessObject_SetGlobalDefaultCoordinateTolerance` = function(s_arg1)
{
  ;ans = .Call('R_swig_ProcessObject_SetGlobalDefaultCoordinateTolerance', s_arg1, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_SetGlobalDefaultCoordinateTolerance`, 'returnType') = 'void'
attr(`ProcessObject_SetGlobalDefaultCoordinateTolerance`, "inputTypes") = c('numeric')
class(`ProcessObject_SetGlobalDefaultCoordinateTolerance`) = c("SWIGFunction", class('ProcessObject_SetGlobalDefaultCoordinateTolerance'))

# Start of ProcessObject_GetGlobalDefaultDirectionTolerance

`ProcessObject_GetGlobalDefaultDirectionTolerance` = function(.copy = FALSE)
{
  ;.Call('R_swig_ProcessObject_GetGlobalDefaultDirectionTolerance', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProcessObject_GetGlobalDefaultDirectionTolerance`, 'returnType') = 'numeric'
class(`ProcessObject_GetGlobalDefaultDirectionTolerance`) = c("SWIGFunction", class('ProcessObject_GetGlobalDefaultDirectionTolerance'))

# Start of ProcessObject_SetGlobalDefaultDirectionTolerance

`ProcessObject_SetGlobalDefaultDirectionTolerance` = function(s_arg1)
{
  ;ans = .Call('R_swig_ProcessObject_SetGlobalDefaultDirectionTolerance', s_arg1, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_SetGlobalDefaultDirectionTolerance`, 'returnType') = 'void'
attr(`ProcessObject_SetGlobalDefaultDirectionTolerance`, "inputTypes") = c('numeric')
class(`ProcessObject_SetGlobalDefaultDirectionTolerance`) = c("SWIGFunction", class('ProcessObject_SetGlobalDefaultDirectionTolerance'))

# Start of ProcessObject_SetGlobalDefaultThreader

`ProcessObject_SetGlobalDefaultThreader` = function(threader, .copy = FALSE)
{
  threader = as(threader, "character"); 
  ;.Call('R_swig_ProcessObject_SetGlobalDefaultThreader', threader, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProcessObject_SetGlobalDefaultThreader`, 'returnType') = 'logical'
attr(`ProcessObject_SetGlobalDefaultThreader`, "inputTypes") = c('character')
class(`ProcessObject_SetGlobalDefaultThreader`) = c("SWIGFunction", class('ProcessObject_SetGlobalDefaultThreader'))

# Start of ProcessObject_GetGlobalDefaultThreader

`ProcessObject_GetGlobalDefaultThreader` = function(.copy = FALSE)
{
  ;.Call('R_swig_ProcessObject_GetGlobalDefaultThreader', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProcessObject_GetGlobalDefaultThreader`, 'returnType') = 'character'
class(`ProcessObject_GetGlobalDefaultThreader`) = c("SWIGFunction", class('ProcessObject_GetGlobalDefaultThreader'))

# Start of ProcessObject_SetGlobalDefaultNumberOfThreads

`ProcessObject_SetGlobalDefaultNumberOfThreads` = function(n)
{
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;ans = .Call('R_swig_ProcessObject_SetGlobalDefaultNumberOfThreads', n, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_SetGlobalDefaultNumberOfThreads`, 'returnType') = 'void'
attr(`ProcessObject_SetGlobalDefaultNumberOfThreads`, "inputTypes") = c('integer')
class(`ProcessObject_SetGlobalDefaultNumberOfThreads`) = c("SWIGFunction", class('ProcessObject_SetGlobalDefaultNumberOfThreads'))

# Start of ProcessObject_GetGlobalDefaultNumberOfThreads

`ProcessObject_GetGlobalDefaultNumberOfThreads` = function(.copy = FALSE)
{
  ;.Call('R_swig_ProcessObject_GetGlobalDefaultNumberOfThreads', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProcessObject_GetGlobalDefaultNumberOfThreads`, 'returnType') = 'integer'
class(`ProcessObject_GetGlobalDefaultNumberOfThreads`) = c("SWIGFunction", class('ProcessObject_GetGlobalDefaultNumberOfThreads'))

# Start of ProcessObject_SetNumberOfThreads

`ProcessObject_SetNumberOfThreads` = function(self, n)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;ans = .Call('R_swig_ProcessObject_SetNumberOfThreads', self, n, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_SetNumberOfThreads`, 'returnType') = 'void'
attr(`ProcessObject_SetNumberOfThreads`, "inputTypes") = c('_p_itk__simple__ProcessObject', 'integer')
class(`ProcessObject_SetNumberOfThreads`) = c("SWIGFunction", class('ProcessObject_SetNumberOfThreads'))

# Start of ProcessObject_GetNumberOfThreads

`ProcessObject_GetNumberOfThreads` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ProcessObject_GetNumberOfThreads', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProcessObject_GetNumberOfThreads`, 'returnType') = 'integer'
attr(`ProcessObject_GetNumberOfThreads`, "inputTypes") = c('_p_itk__simple__ProcessObject')
class(`ProcessObject_GetNumberOfThreads`) = c("SWIGFunction", class('ProcessObject_GetNumberOfThreads'))

# Start of ProcessObject_SetNumberOfWorkUnits

`ProcessObject_SetNumberOfWorkUnits` = function(self, n)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  n = as.integer(n);
  
  if(length(n) > 1) {
    warning("using only the first element of n");
  };
  
  ;ans = .Call('R_swig_ProcessObject_SetNumberOfWorkUnits', self, n, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_SetNumberOfWorkUnits`, 'returnType') = 'void'
attr(`ProcessObject_SetNumberOfWorkUnits`, "inputTypes") = c('_p_itk__simple__ProcessObject', 'integer')
class(`ProcessObject_SetNumberOfWorkUnits`) = c("SWIGFunction", class('ProcessObject_SetNumberOfWorkUnits'))

# Start of ProcessObject_GetNumberOfWorkUnits

`ProcessObject_GetNumberOfWorkUnits` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ProcessObject_GetNumberOfWorkUnits', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProcessObject_GetNumberOfWorkUnits`, 'returnType') = 'integer'
attr(`ProcessObject_GetNumberOfWorkUnits`, "inputTypes") = c('_p_itk__simple__ProcessObject')
class(`ProcessObject_GetNumberOfWorkUnits`) = c("SWIGFunction", class('ProcessObject_GetNumberOfWorkUnits'))

# Start of ProcessObject_RemoveAllCommands

`ProcessObject_RemoveAllCommands` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ProcessObject_RemoveAllCommands', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_RemoveAllCommands`, 'returnType') = 'void'
attr(`ProcessObject_RemoveAllCommands`, "inputTypes") = c('_p_itk__simple__ProcessObject')
class(`ProcessObject_RemoveAllCommands`) = c("SWIGFunction", class('ProcessObject_RemoveAllCommands'))

# Start of ProcessObject_HasCommand

`ProcessObject_HasCommand` = function(self, event, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  event = enumToInteger(event, "_itk__simple__EventEnum"); 
  
  if(length(event) > 1) {
    warning("using only the first element of event");
  };
  
  ;.Call('R_swig_ProcessObject_HasCommand', self, event, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProcessObject_HasCommand`, 'returnType') = 'logical'
attr(`ProcessObject_HasCommand`, "inputTypes") = c('_p_itk__simple__ProcessObject', 'character')
class(`ProcessObject_HasCommand`) = c("SWIGFunction", class('ProcessObject_HasCommand'))

# Start of ProcessObject_GetProgress

`ProcessObject_GetProgress` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ProcessObject_GetProgress', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProcessObject_GetProgress`, 'returnType') = 'numeric'
attr(`ProcessObject_GetProgress`, "inputTypes") = c('_p_itk__simple__ProcessObject')
class(`ProcessObject_GetProgress`) = c("SWIGFunction", class('ProcessObject_GetProgress'))

# Start of ProcessObject_Abort

`ProcessObject_Abort` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ProcessObject_Abort', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProcessObject_Abort`, 'returnType') = 'void'
attr(`ProcessObject_Abort`, "inputTypes") = c('_p_itk__simple__ProcessObject')
class(`ProcessObject_Abort`) = c("SWIGFunction", class('ProcessObject_Abort'))

# Start of ProcessObject_AddCommand

`ProcessObject_AddCommand` = function(self, e, obj, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  e = enumToInteger(e, "_itk__simple__EventEnum"); 
  
  if(length(e) > 1) {
    warning("using only the first element of e");
  };
  
  if (inherits(obj, "ExternalReference")) obj = slot(obj,"ref"); 
  ;.Call('R_swig_ProcessObject_AddCommand', self, e, obj, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProcessObject_AddCommand`, 'returnType') = 'integer'
attr(`ProcessObject_AddCommand`, "inputTypes") = c('_p_itk__simple__ProcessObject', 'character', 'numeric')
class(`ProcessObject_AddCommand`) = c("SWIGFunction", class('ProcessObject_AddCommand'))

# Start of accessor method for itk::simple::ProcessObject
setMethod('$', '_p_itk__simple__ProcessObject', function(x, name)

{
  accessorFuns = list('ToString' = ProcessObject_ToString, 'GetName' = ProcessObject_GetName, 'DebugOn' = ProcessObject_DebugOn, 'DebugOff' = ProcessObject_DebugOff, 'GetDebug' = ProcessObject_GetDebug, 'SetDebug' = ProcessObject_SetDebug, 'SetNumberOfThreads' = ProcessObject_SetNumberOfThreads, 'GetNumberOfThreads' = ProcessObject_GetNumberOfThreads, 'SetNumberOfWorkUnits' = ProcessObject_SetNumberOfWorkUnits, 'GetNumberOfWorkUnits' = ProcessObject_GetNumberOfWorkUnits, 'RemoveAllCommands' = ProcessObject_RemoveAllCommands, 'HasCommand' = ProcessObject_HasCommand, 'GetProgress' = ProcessObject_GetProgress, 'Abort' = ProcessObject_Abort, 'AddCommand' = ProcessObject_AddCommand);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ProcessObject
# Start of delete_ImageFilter

`delete_ImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ImageFilter`, 'returnType') = 'void'
attr(`delete_ImageFilter`, "inputTypes") = c('_p_itk__simple__ImageFilter')
class(`delete_ImageFilter`) = c("SWIGFunction", class('delete_ImageFilter'))

# Start of Show

`Show__SWIG_0` = function(image, title, debugOn)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  title = as(title, "character"); 
  debugOn = as.logical(debugOn);
  ;ans = .Call('R_swig_Show__SWIG_0', image, title, debugOn, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Show__SWIG_0`, 'returnType') = 'void'
attr(`Show__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'logical')
class(`Show__SWIG_0`) = c("SWIGFunction", class('Show__SWIG_0'))

# Start of Show

`Show__SWIG_1` = function(image, title)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  title = as(title, "character"); 
  ;ans = .Call('R_swig_Show__SWIG_1', image, title, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Show__SWIG_1`, 'returnType') = 'void'
attr(`Show__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'character')
class(`Show__SWIG_1`) = c("SWIGFunction", class('Show__SWIG_1'))

# Start of Show

`Show__SWIG_2` = function(image)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_Show__SWIG_2', image, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`Show__SWIG_2`, 'returnType') = 'void'
attr(`Show__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`Show__SWIG_2`) = c("SWIGFunction", class('Show__SWIG_2'))

`Show` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Show__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Show__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Show__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Show with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ImageFileWriter

`delete_ImageFileWriter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ImageFileWriter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ImageFileWriter`, 'returnType') = 'void'
attr(`delete_ImageFileWriter`, "inputTypes") = c('_p_itk__simple__ImageFileWriter')
class(`delete_ImageFileWriter`) = c("SWIGFunction", class('delete_ImageFileWriter'))

# Start of new_ImageFileWriter

`ImageFileWriter` = function()
{
  ;ans = .Call('R_swig_new_ImageFileWriter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ImageFileWriter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ImageFileWriter);
  ans
  
}

attr(`ImageFileWriter`, 'returnType') = '_p_itk__simple__ImageFileWriter'
class(`ImageFileWriter`) = c("SWIGFunction", class('ImageFileWriter'))

# Start of ImageFileWriter_ToString

`ImageFileWriter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileWriter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileWriter_ToString`, 'returnType') = 'character'
attr(`ImageFileWriter_ToString`, "inputTypes") = c('_p_itk__simple__ImageFileWriter')
class(`ImageFileWriter_ToString`) = c("SWIGFunction", class('ImageFileWriter_ToString'))

# Start of ImageFileWriter_GetName

`ImageFileWriter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileWriter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileWriter_GetName`, 'returnType') = 'character'
attr(`ImageFileWriter_GetName`, "inputTypes") = c('_p_itk__simple__ImageFileWriter')
class(`ImageFileWriter_GetName`) = c("SWIGFunction", class('ImageFileWriter_GetName'))

# Start of ImageFileWriter_GetRegisteredImageIOs

`ImageFileWriter_GetRegisteredImageIOs` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileWriter_GetRegisteredImageIOs', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileWriter_GetRegisteredImageIOs`, 'returnType') = 'character'
attr(`ImageFileWriter_GetRegisteredImageIOs`, "inputTypes") = c('_p_itk__simple__ImageFileWriter')
class(`ImageFileWriter_GetRegisteredImageIOs`) = c("SWIGFunction", class('ImageFileWriter_GetRegisteredImageIOs'))

# Start of ImageFileWriter_SetUseCompression

`ImageFileWriter_SetUseCompression` = function(self, UseCompression)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseCompression = as.logical(UseCompression);
  ;ans = .Call('R_swig_ImageFileWriter_SetUseCompression', self, UseCompression, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileWriter_SetUseCompression`, 'returnType') = 'void'
attr(`ImageFileWriter_SetUseCompression`, "inputTypes") = c('_p_itk__simple__ImageFileWriter', 'logical')
class(`ImageFileWriter_SetUseCompression`) = c("SWIGFunction", class('ImageFileWriter_SetUseCompression'))

# Start of ImageFileWriter_GetUseCompression

`ImageFileWriter_GetUseCompression` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileWriter_GetUseCompression', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileWriter_GetUseCompression`, 'returnType') = 'logical'
attr(`ImageFileWriter_GetUseCompression`, "inputTypes") = c('_p_itk__simple__ImageFileWriter')
class(`ImageFileWriter_GetUseCompression`) = c("SWIGFunction", class('ImageFileWriter_GetUseCompression'))

# Start of ImageFileWriter_UseCompressionOn

`ImageFileWriter_UseCompressionOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageFileWriter_UseCompressionOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileWriter_UseCompressionOn`, 'returnType') = 'void'
attr(`ImageFileWriter_UseCompressionOn`, "inputTypes") = c('_p_itk__simple__ImageFileWriter')
class(`ImageFileWriter_UseCompressionOn`) = c("SWIGFunction", class('ImageFileWriter_UseCompressionOn'))

# Start of ImageFileWriter_UseCompressionOff

`ImageFileWriter_UseCompressionOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageFileWriter_UseCompressionOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileWriter_UseCompressionOff`, 'returnType') = 'void'
attr(`ImageFileWriter_UseCompressionOff`, "inputTypes") = c('_p_itk__simple__ImageFileWriter')
class(`ImageFileWriter_UseCompressionOff`) = c("SWIGFunction", class('ImageFileWriter_UseCompressionOff'))

# Start of ImageFileWriter_SetCompressionLevel

`ImageFileWriter_SetCompressionLevel` = function(self, s_arg2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  s_arg2 = as.integer(s_arg2);
  
  if(length(s_arg2) > 1) {
    warning("using only the first element of s_arg2");
  };
  
  ;ans = .Call('R_swig_ImageFileWriter_SetCompressionLevel', self, s_arg2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileWriter_SetCompressionLevel`, 'returnType') = 'void'
attr(`ImageFileWriter_SetCompressionLevel`, "inputTypes") = c('_p_itk__simple__ImageFileWriter', 'integer')
class(`ImageFileWriter_SetCompressionLevel`) = c("SWIGFunction", class('ImageFileWriter_SetCompressionLevel'))

# Start of ImageFileWriter_GetCompressionLevel

`ImageFileWriter_GetCompressionLevel` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileWriter_GetCompressionLevel', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileWriter_GetCompressionLevel`, 'returnType') = 'integer'
attr(`ImageFileWriter_GetCompressionLevel`, "inputTypes") = c('_p_itk__simple__ImageFileWriter')
class(`ImageFileWriter_GetCompressionLevel`) = c("SWIGFunction", class('ImageFileWriter_GetCompressionLevel'))

# Start of ImageFileWriter_SetCompressor

`ImageFileWriter_SetCompressor` = function(self, s_arg2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  s_arg2 = as(s_arg2, "character"); 
  ;ans = .Call('R_swig_ImageFileWriter_SetCompressor', self, s_arg2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileWriter_SetCompressor`, 'returnType') = 'void'
attr(`ImageFileWriter_SetCompressor`, "inputTypes") = c('_p_itk__simple__ImageFileWriter', 'character')
class(`ImageFileWriter_SetCompressor`) = c("SWIGFunction", class('ImageFileWriter_SetCompressor'))

# Start of ImageFileWriter_GetCompressor

`ImageFileWriter_GetCompressor` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileWriter_GetCompressor', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileWriter_GetCompressor`, 'returnType') = 'character'
attr(`ImageFileWriter_GetCompressor`, "inputTypes") = c('_p_itk__simple__ImageFileWriter')
class(`ImageFileWriter_GetCompressor`) = c("SWIGFunction", class('ImageFileWriter_GetCompressor'))

# Start of ImageFileWriter_SetImageIO

`ImageFileWriter_SetImageIO` = function(self, imageio)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  imageio = as(imageio, "character"); 
  ;ans = .Call('R_swig_ImageFileWriter_SetImageIO', self, imageio, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileWriter_SetImageIO`, 'returnType') = 'void'
attr(`ImageFileWriter_SetImageIO`, "inputTypes") = c('_p_itk__simple__ImageFileWriter', 'character')
class(`ImageFileWriter_SetImageIO`) = c("SWIGFunction", class('ImageFileWriter_SetImageIO'))

# Start of ImageFileWriter_GetImageIO

`ImageFileWriter_GetImageIO` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileWriter_GetImageIO', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileWriter_GetImageIO`, 'returnType') = 'character'
attr(`ImageFileWriter_GetImageIO`, "inputTypes") = c('_p_itk__simple__ImageFileWriter')
class(`ImageFileWriter_GetImageIO`) = c("SWIGFunction", class('ImageFileWriter_GetImageIO'))

# Start of ImageFileWriter_SetKeepOriginalImageUID

`ImageFileWriter_SetKeepOriginalImageUID` = function(self, KeepOriginalImageUID)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KeepOriginalImageUID = as.logical(KeepOriginalImageUID);
  ;ans = .Call('R_swig_ImageFileWriter_SetKeepOriginalImageUID', self, KeepOriginalImageUID, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileWriter_SetKeepOriginalImageUID`, 'returnType') = 'void'
attr(`ImageFileWriter_SetKeepOriginalImageUID`, "inputTypes") = c('_p_itk__simple__ImageFileWriter', 'logical')
class(`ImageFileWriter_SetKeepOriginalImageUID`) = c("SWIGFunction", class('ImageFileWriter_SetKeepOriginalImageUID'))

# Start of ImageFileWriter_GetKeepOriginalImageUID

`ImageFileWriter_GetKeepOriginalImageUID` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileWriter_GetKeepOriginalImageUID', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileWriter_GetKeepOriginalImageUID`, 'returnType') = 'logical'
attr(`ImageFileWriter_GetKeepOriginalImageUID`, "inputTypes") = c('_p_itk__simple__ImageFileWriter')
class(`ImageFileWriter_GetKeepOriginalImageUID`) = c("SWIGFunction", class('ImageFileWriter_GetKeepOriginalImageUID'))

# Start of ImageFileWriter_KeepOriginalImageUIDOn

`ImageFileWriter_KeepOriginalImageUIDOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageFileWriter_KeepOriginalImageUIDOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileWriter_KeepOriginalImageUIDOn`, 'returnType') = 'void'
attr(`ImageFileWriter_KeepOriginalImageUIDOn`, "inputTypes") = c('_p_itk__simple__ImageFileWriter')
class(`ImageFileWriter_KeepOriginalImageUIDOn`) = c("SWIGFunction", class('ImageFileWriter_KeepOriginalImageUIDOn'))

# Start of ImageFileWriter_KeepOriginalImageUIDOff

`ImageFileWriter_KeepOriginalImageUIDOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageFileWriter_KeepOriginalImageUIDOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileWriter_KeepOriginalImageUIDOff`, 'returnType') = 'void'
attr(`ImageFileWriter_KeepOriginalImageUIDOff`, "inputTypes") = c('_p_itk__simple__ImageFileWriter')
class(`ImageFileWriter_KeepOriginalImageUIDOff`) = c("SWIGFunction", class('ImageFileWriter_KeepOriginalImageUIDOff'))

# Start of ImageFileWriter_SetFileName

`ImageFileWriter_SetFileName` = function(self, fileName)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  fileName = as(fileName, "character"); 
  ;ans = .Call('R_swig_ImageFileWriter_SetFileName', self, fileName, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileWriter_SetFileName`, 'returnType') = 'void'
attr(`ImageFileWriter_SetFileName`, "inputTypes") = c('_p_itk__simple__ImageFileWriter', 'character')
class(`ImageFileWriter_SetFileName`) = c("SWIGFunction", class('ImageFileWriter_SetFileName'))

# Start of ImageFileWriter_GetFileName

`ImageFileWriter_GetFileName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileWriter_GetFileName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileWriter_GetFileName`, 'returnType') = 'character'
attr(`ImageFileWriter_GetFileName`, "inputTypes") = c('_p_itk__simple__ImageFileWriter')
class(`ImageFileWriter_GetFileName`) = c("SWIGFunction", class('ImageFileWriter_GetFileName'))

# Start of ImageFileWriter_Execute

`ImageFileWriter_Execute__SWIG_0` = function(self, s_arg2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_ImageFileWriter_Execute__SWIG_0', self, s_arg2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileWriter_Execute__SWIG_0`, 'returnType') = 'void'
attr(`ImageFileWriter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageFileWriter', '_p_itk__simple__Image')
class(`ImageFileWriter_Execute__SWIG_0`) = c("SWIGFunction", class('ImageFileWriter_Execute__SWIG_0'))

# Start of ImageFileWriter_Execute

`ImageFileWriter_Execute__SWIG_1` = function(self, s_arg2, inFileName, useCompression, compressionLevel)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  inFileName = as(inFileName, "character"); 
  useCompression = as.logical(useCompression);
  compressionLevel = as.integer(compressionLevel);
  
  if(length(compressionLevel) > 1) {
    warning("using only the first element of compressionLevel");
  };
  
  ;ans = .Call('R_swig_ImageFileWriter_Execute__SWIG_1', self, s_arg2, inFileName, useCompression, compressionLevel, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileWriter_Execute__SWIG_1`, 'returnType') = 'void'
attr(`ImageFileWriter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageFileWriter', '_p_itk__simple__Image', 'character', 'logical', 'integer')
class(`ImageFileWriter_Execute__SWIG_1`) = c("SWIGFunction", class('ImageFileWriter_Execute__SWIG_1'))

`ImageFileWriter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageFileWriter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- ImageFileWriter_Execute__SWIG_0; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageFileWriter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- ImageFileWriter_Execute__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageFileWriter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::ImageFileWriter
setMethod('$', '_p_itk__simple__ImageFileWriter', function(x, name)

{
  accessorFuns = list('ToString' = ImageFileWriter_ToString, 'GetName' = ImageFileWriter_GetName, 'GetRegisteredImageIOs' = ImageFileWriter_GetRegisteredImageIOs, 'SetUseCompression' = ImageFileWriter_SetUseCompression, 'GetUseCompression' = ImageFileWriter_GetUseCompression, 'UseCompressionOn' = ImageFileWriter_UseCompressionOn, 'UseCompressionOff' = ImageFileWriter_UseCompressionOff, 'SetCompressionLevel' = ImageFileWriter_SetCompressionLevel, 'GetCompressionLevel' = ImageFileWriter_GetCompressionLevel, 'SetCompressor' = ImageFileWriter_SetCompressor, 'GetCompressor' = ImageFileWriter_GetCompressor, 'SetImageIO' = ImageFileWriter_SetImageIO, 'GetImageIO' = ImageFileWriter_GetImageIO, 'SetKeepOriginalImageUID' = ImageFileWriter_SetKeepOriginalImageUID, 'GetKeepOriginalImageUID' = ImageFileWriter_GetKeepOriginalImageUID, 'KeepOriginalImageUIDOn' = ImageFileWriter_KeepOriginalImageUIDOn, 'KeepOriginalImageUIDOff' = ImageFileWriter_KeepOriginalImageUIDOff, 'SetFileName' = ImageFileWriter_SetFileName, 'GetFileName' = ImageFileWriter_GetFileName, 'Execute' = ImageFileWriter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ImageFileWriter
# Start of WriteImage

`WriteImage__SWIG_0` = function(image, fileName, useCompression, compressionLevel)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  fileName = as(fileName, "character"); 
  useCompression = as.logical(useCompression);
  compressionLevel = as.integer(compressionLevel);
  
  if(length(compressionLevel) > 1) {
    warning("using only the first element of compressionLevel");
  };
  
  ;ans = .Call('R_swig_WriteImage__SWIG_0', image, fileName, useCompression, compressionLevel, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WriteImage__SWIG_0`, 'returnType') = 'void'
attr(`WriteImage__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'logical', 'integer')
class(`WriteImage__SWIG_0`) = c("SWIGFunction", class('WriteImage__SWIG_0'))

# Start of WriteImage

`WriteImage__SWIG_1` = function(image, fileName, useCompression)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  fileName = as(fileName, "character"); 
  useCompression = as.logical(useCompression);
  ;ans = .Call('R_swig_WriteImage__SWIG_1', image, fileName, useCompression, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WriteImage__SWIG_1`, 'returnType') = 'void'
attr(`WriteImage__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'logical')
class(`WriteImage__SWIG_1`) = c("SWIGFunction", class('WriteImage__SWIG_1'))

# Start of WriteImage

`WriteImage__SWIG_2` = function(image, fileName)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  fileName = as(fileName, "character"); 
  ;ans = .Call('R_swig_WriteImage__SWIG_2', image, fileName, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WriteImage__SWIG_2`, 'returnType') = 'void'
attr(`WriteImage__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'character')
class(`WriteImage__SWIG_2`) = c("SWIGFunction", class('WriteImage__SWIG_2'))

# Start of delete_ImageSeriesWriter

`delete_ImageSeriesWriter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ImageSeriesWriter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ImageSeriesWriter`, 'returnType') = 'void'
attr(`delete_ImageSeriesWriter`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter')
class(`delete_ImageSeriesWriter`) = c("SWIGFunction", class('delete_ImageSeriesWriter'))

# Start of new_ImageSeriesWriter

`ImageSeriesWriter` = function()
{
  ;ans = .Call('R_swig_new_ImageSeriesWriter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ImageSeriesWriter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ImageSeriesWriter);
  ans
  
}

attr(`ImageSeriesWriter`, 'returnType') = '_p_itk__simple__ImageSeriesWriter'
class(`ImageSeriesWriter`) = c("SWIGFunction", class('ImageSeriesWriter'))

# Start of ImageSeriesWriter_ToString

`ImageSeriesWriter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageSeriesWriter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesWriter_ToString`, 'returnType') = 'character'
attr(`ImageSeriesWriter_ToString`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter')
class(`ImageSeriesWriter_ToString`) = c("SWIGFunction", class('ImageSeriesWriter_ToString'))

# Start of ImageSeriesWriter_GetRegisteredImageIOs

`ImageSeriesWriter_GetRegisteredImageIOs` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageSeriesWriter_GetRegisteredImageIOs', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesWriter_GetRegisteredImageIOs`, 'returnType') = 'character'
attr(`ImageSeriesWriter_GetRegisteredImageIOs`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter')
class(`ImageSeriesWriter_GetRegisteredImageIOs`) = c("SWIGFunction", class('ImageSeriesWriter_GetRegisteredImageIOs'))

# Start of ImageSeriesWriter_SetImageIO

`ImageSeriesWriter_SetImageIO` = function(self, imageio)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  imageio = as(imageio, "character"); 
  ;ans = .Call('R_swig_ImageSeriesWriter_SetImageIO', self, imageio, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesWriter_SetImageIO`, 'returnType') = 'void'
attr(`ImageSeriesWriter_SetImageIO`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter', 'character')
class(`ImageSeriesWriter_SetImageIO`) = c("SWIGFunction", class('ImageSeriesWriter_SetImageIO'))

# Start of ImageSeriesWriter_GetImageIO

`ImageSeriesWriter_GetImageIO` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageSeriesWriter_GetImageIO', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesWriter_GetImageIO`, 'returnType') = 'character'
attr(`ImageSeriesWriter_GetImageIO`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter')
class(`ImageSeriesWriter_GetImageIO`) = c("SWIGFunction", class('ImageSeriesWriter_GetImageIO'))

# Start of ImageSeriesWriter_GetName

`ImageSeriesWriter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageSeriesWriter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesWriter_GetName`, 'returnType') = 'character'
attr(`ImageSeriesWriter_GetName`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter')
class(`ImageSeriesWriter_GetName`) = c("SWIGFunction", class('ImageSeriesWriter_GetName'))

# Start of ImageSeriesWriter_SetUseCompression

`ImageSeriesWriter_SetUseCompression` = function(self, UseCompression)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseCompression = as.logical(UseCompression);
  ;ans = .Call('R_swig_ImageSeriesWriter_SetUseCompression', self, UseCompression, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesWriter_SetUseCompression`, 'returnType') = 'void'
attr(`ImageSeriesWriter_SetUseCompression`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter', 'logical')
class(`ImageSeriesWriter_SetUseCompression`) = c("SWIGFunction", class('ImageSeriesWriter_SetUseCompression'))

# Start of ImageSeriesWriter_GetUseCompression

`ImageSeriesWriter_GetUseCompression` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageSeriesWriter_GetUseCompression', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesWriter_GetUseCompression`, 'returnType') = 'logical'
attr(`ImageSeriesWriter_GetUseCompression`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter')
class(`ImageSeriesWriter_GetUseCompression`) = c("SWIGFunction", class('ImageSeriesWriter_GetUseCompression'))

# Start of ImageSeriesWriter_UseCompressionOn

`ImageSeriesWriter_UseCompressionOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageSeriesWriter_UseCompressionOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesWriter_UseCompressionOn`, 'returnType') = 'void'
attr(`ImageSeriesWriter_UseCompressionOn`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter')
class(`ImageSeriesWriter_UseCompressionOn`) = c("SWIGFunction", class('ImageSeriesWriter_UseCompressionOn'))

# Start of ImageSeriesWriter_UseCompressionOff

`ImageSeriesWriter_UseCompressionOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageSeriesWriter_UseCompressionOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesWriter_UseCompressionOff`, 'returnType') = 'void'
attr(`ImageSeriesWriter_UseCompressionOff`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter')
class(`ImageSeriesWriter_UseCompressionOff`) = c("SWIGFunction", class('ImageSeriesWriter_UseCompressionOff'))

# Start of ImageSeriesWriter_SetCompressionLevel

`ImageSeriesWriter_SetCompressionLevel` = function(self, s_arg2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  s_arg2 = as.integer(s_arg2);
  
  if(length(s_arg2) > 1) {
    warning("using only the first element of s_arg2");
  };
  
  ;ans = .Call('R_swig_ImageSeriesWriter_SetCompressionLevel', self, s_arg2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesWriter_SetCompressionLevel`, 'returnType') = 'void'
attr(`ImageSeriesWriter_SetCompressionLevel`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter', 'integer')
class(`ImageSeriesWriter_SetCompressionLevel`) = c("SWIGFunction", class('ImageSeriesWriter_SetCompressionLevel'))

# Start of ImageSeriesWriter_GetCompressionLevel

`ImageSeriesWriter_GetCompressionLevel` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageSeriesWriter_GetCompressionLevel', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesWriter_GetCompressionLevel`, 'returnType') = 'integer'
attr(`ImageSeriesWriter_GetCompressionLevel`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter')
class(`ImageSeriesWriter_GetCompressionLevel`) = c("SWIGFunction", class('ImageSeriesWriter_GetCompressionLevel'))

# Start of ImageSeriesWriter_SetCompressor

`ImageSeriesWriter_SetCompressor` = function(self, s_arg2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  s_arg2 = as(s_arg2, "character"); 
  ;ans = .Call('R_swig_ImageSeriesWriter_SetCompressor', self, s_arg2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesWriter_SetCompressor`, 'returnType') = 'void'
attr(`ImageSeriesWriter_SetCompressor`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter', 'character')
class(`ImageSeriesWriter_SetCompressor`) = c("SWIGFunction", class('ImageSeriesWriter_SetCompressor'))

# Start of ImageSeriesWriter_GetCompressor

`ImageSeriesWriter_GetCompressor` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageSeriesWriter_GetCompressor', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesWriter_GetCompressor`, 'returnType') = 'character'
attr(`ImageSeriesWriter_GetCompressor`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter')
class(`ImageSeriesWriter_GetCompressor`) = c("SWIGFunction", class('ImageSeriesWriter_GetCompressor'))

# Start of ImageSeriesWriter_SetFileNames

`ImageSeriesWriter_SetFileNames` = function(self, fileNames)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  fileNames = as.character(fileNames);
  ;ans = .Call('R_swig_ImageSeriesWriter_SetFileNames', self, fileNames, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesWriter_SetFileNames`, 'returnType') = 'void'
attr(`ImageSeriesWriter_SetFileNames`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter', 'character')
class(`ImageSeriesWriter_SetFileNames`) = c("SWIGFunction", class('ImageSeriesWriter_SetFileNames'))

# Start of ImageSeriesWriter_GetFileNames

`ImageSeriesWriter_GetFileNames` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageSeriesWriter_GetFileNames', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesWriter_GetFileNames`, 'returnType') = 'character'
attr(`ImageSeriesWriter_GetFileNames`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter')
class(`ImageSeriesWriter_GetFileNames`) = c("SWIGFunction", class('ImageSeriesWriter_GetFileNames'))

# Start of ImageSeriesWriter_Execute

`ImageSeriesWriter_Execute__SWIG_0` = function(self, s_arg2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_ImageSeriesWriter_Execute__SWIG_0', self, s_arg2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesWriter_Execute__SWIG_0`, 'returnType') = 'void'
attr(`ImageSeriesWriter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter', '_p_itk__simple__Image')
class(`ImageSeriesWriter_Execute__SWIG_0`) = c("SWIGFunction", class('ImageSeriesWriter_Execute__SWIG_0'))

# Start of ImageSeriesWriter_Execute

`ImageSeriesWriter_Execute__SWIG_1` = function(self, image, inFileNames, useCompression, compressionLevel)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  inFileNames = as.character(inFileNames);
  useCompression = as.logical(useCompression);
  compressionLevel = as.integer(compressionLevel);
  
  if(length(compressionLevel) > 1) {
    warning("using only the first element of compressionLevel");
  };
  
  ;ans = .Call('R_swig_ImageSeriesWriter_Execute__SWIG_1', self, image, inFileNames, useCompression, compressionLevel, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesWriter_Execute__SWIG_1`, 'returnType') = 'void'
attr(`ImageSeriesWriter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageSeriesWriter', '_p_itk__simple__Image', 'character', 'logical', 'integer')
class(`ImageSeriesWriter_Execute__SWIG_1`) = c("SWIGFunction", class('ImageSeriesWriter_Execute__SWIG_1'))

`ImageSeriesWriter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageSeriesWriter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- ImageSeriesWriter_Execute__SWIG_0; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageSeriesWriter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- ImageSeriesWriter_Execute__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageSeriesWriter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::ImageSeriesWriter
setMethod('$', '_p_itk__simple__ImageSeriesWriter', function(x, name)

{
  accessorFuns = list('ToString' = ImageSeriesWriter_ToString, 'GetRegisteredImageIOs' = ImageSeriesWriter_GetRegisteredImageIOs, 'SetImageIO' = ImageSeriesWriter_SetImageIO, 'GetImageIO' = ImageSeriesWriter_GetImageIO, 'GetName' = ImageSeriesWriter_GetName, 'SetUseCompression' = ImageSeriesWriter_SetUseCompression, 'GetUseCompression' = ImageSeriesWriter_GetUseCompression, 'UseCompressionOn' = ImageSeriesWriter_UseCompressionOn, 'UseCompressionOff' = ImageSeriesWriter_UseCompressionOff, 'SetCompressionLevel' = ImageSeriesWriter_SetCompressionLevel, 'GetCompressionLevel' = ImageSeriesWriter_GetCompressionLevel, 'SetCompressor' = ImageSeriesWriter_SetCompressor, 'GetCompressor' = ImageSeriesWriter_GetCompressor, 'SetFileNames' = ImageSeriesWriter_SetFileNames, 'GetFileNames' = ImageSeriesWriter_GetFileNames, 'Execute' = ImageSeriesWriter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ImageSeriesWriter
# Start of WriteImage

`WriteImage__SWIG_3` = function(image, fileNames, useCompression, compressionLevel)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  fileNames = as.character(fileNames);
  useCompression = as.logical(useCompression);
  compressionLevel = as.integer(compressionLevel);
  
  if(length(compressionLevel) > 1) {
    warning("using only the first element of compressionLevel");
  };
  
  ;ans = .Call('R_swig_WriteImage__SWIG_3', image, fileNames, useCompression, compressionLevel, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WriteImage__SWIG_3`, 'returnType') = 'void'
attr(`WriteImage__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'logical', 'integer')
class(`WriteImage__SWIG_3`) = c("SWIGFunction", class('WriteImage__SWIG_3'))

# Start of WriteImage

`WriteImage__SWIG_4` = function(image, fileNames, useCompression)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  fileNames = as.character(fileNames);
  useCompression = as.logical(useCompression);
  ;ans = .Call('R_swig_WriteImage__SWIG_4', image, fileNames, useCompression, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WriteImage__SWIG_4`, 'returnType') = 'void'
attr(`WriteImage__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'logical')
class(`WriteImage__SWIG_4`) = c("SWIGFunction", class('WriteImage__SWIG_4'))

# Start of WriteImage

`WriteImage__SWIG_5` = function(image, fileNames)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  fileNames = as.character(fileNames);
  ;ans = .Call('R_swig_WriteImage__SWIG_5', image, fileNames, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WriteImage__SWIG_5`, 'returnType') = 'void'
attr(`WriteImage__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'character')
class(`WriteImage__SWIG_5`) = c("SWIGFunction", class('WriteImage__SWIG_5'))

`WriteImage` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- WriteImage__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) )) {
      f <- WriteImage__SWIG_5; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- WriteImage__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- WriteImage__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- WriteImage__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- WriteImage__SWIG_3; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for WriteImage with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ImageReaderBase

`delete_ImageReaderBase` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ImageReaderBase', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ImageReaderBase`, 'returnType') = 'void'
attr(`delete_ImageReaderBase`, "inputTypes") = c('_p_itk__simple__ImageReaderBase')
class(`delete_ImageReaderBase`) = c("SWIGFunction", class('delete_ImageReaderBase'))

# Start of ImageReaderBase_SetOutputPixelType

`ImageReaderBase_SetOutputPixelType` = function(self, pixelID)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  pixelID = enumToInteger(pixelID, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(pixelID) > 1) {
    warning("using only the first element of pixelID");
  };
  
  ;ans = .Call('R_swig_ImageReaderBase_SetOutputPixelType', self, pixelID, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageReaderBase_SetOutputPixelType`, 'returnType') = 'void'
attr(`ImageReaderBase_SetOutputPixelType`, "inputTypes") = c('_p_itk__simple__ImageReaderBase', 'character')
class(`ImageReaderBase_SetOutputPixelType`) = c("SWIGFunction", class('ImageReaderBase_SetOutputPixelType'))

# Start of ImageReaderBase_GetOutputPixelType

`ImageReaderBase_GetOutputPixelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageReaderBase_GetOutputPixelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`ImageReaderBase_GetOutputPixelType`, 'returnType') = 'character'
attr(`ImageReaderBase_GetOutputPixelType`, "inputTypes") = c('_p_itk__simple__ImageReaderBase')
class(`ImageReaderBase_GetOutputPixelType`) = c("SWIGFunction", class('ImageReaderBase_GetOutputPixelType'))

# Start of ImageReaderBase_Execute

`ImageReaderBase_Execute` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageReaderBase_Execute', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ImageReaderBase_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ImageReaderBase_Execute`, "inputTypes") = c('_p_itk__simple__ImageReaderBase')
class(`ImageReaderBase_Execute`) = c("SWIGFunction", class('ImageReaderBase_Execute'))

# Start of ImageReaderBase_ToString

`ImageReaderBase_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageReaderBase_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageReaderBase_ToString`, 'returnType') = 'character'
attr(`ImageReaderBase_ToString`, "inputTypes") = c('_p_itk__simple__ImageReaderBase')
class(`ImageReaderBase_ToString`) = c("SWIGFunction", class('ImageReaderBase_ToString'))

# Start of ImageReaderBase_GetRegisteredImageIOs

`ImageReaderBase_GetRegisteredImageIOs` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageReaderBase_GetRegisteredImageIOs', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageReaderBase_GetRegisteredImageIOs`, 'returnType') = 'character'
attr(`ImageReaderBase_GetRegisteredImageIOs`, "inputTypes") = c('_p_itk__simple__ImageReaderBase')
class(`ImageReaderBase_GetRegisteredImageIOs`) = c("SWIGFunction", class('ImageReaderBase_GetRegisteredImageIOs'))

# Start of ImageReaderBase_SetLoadPrivateTags

`ImageReaderBase_SetLoadPrivateTags` = function(self, loadPrivateTags)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  loadPrivateTags = as.logical(loadPrivateTags);
  ;ans = .Call('R_swig_ImageReaderBase_SetLoadPrivateTags', self, loadPrivateTags, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageReaderBase_SetLoadPrivateTags`, 'returnType') = 'void'
attr(`ImageReaderBase_SetLoadPrivateTags`, "inputTypes") = c('_p_itk__simple__ImageReaderBase', 'logical')
class(`ImageReaderBase_SetLoadPrivateTags`) = c("SWIGFunction", class('ImageReaderBase_SetLoadPrivateTags'))

# Start of ImageReaderBase_GetLoadPrivateTags

`ImageReaderBase_GetLoadPrivateTags` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageReaderBase_GetLoadPrivateTags', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageReaderBase_GetLoadPrivateTags`, 'returnType') = 'logical'
attr(`ImageReaderBase_GetLoadPrivateTags`, "inputTypes") = c('_p_itk__simple__ImageReaderBase')
class(`ImageReaderBase_GetLoadPrivateTags`) = c("SWIGFunction", class('ImageReaderBase_GetLoadPrivateTags'))

# Start of ImageReaderBase_LoadPrivateTagsOn

`ImageReaderBase_LoadPrivateTagsOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageReaderBase_LoadPrivateTagsOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageReaderBase_LoadPrivateTagsOn`, 'returnType') = 'void'
attr(`ImageReaderBase_LoadPrivateTagsOn`, "inputTypes") = c('_p_itk__simple__ImageReaderBase')
class(`ImageReaderBase_LoadPrivateTagsOn`) = c("SWIGFunction", class('ImageReaderBase_LoadPrivateTagsOn'))

# Start of ImageReaderBase_LoadPrivateTagsOff

`ImageReaderBase_LoadPrivateTagsOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageReaderBase_LoadPrivateTagsOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageReaderBase_LoadPrivateTagsOff`, 'returnType') = 'void'
attr(`ImageReaderBase_LoadPrivateTagsOff`, "inputTypes") = c('_p_itk__simple__ImageReaderBase')
class(`ImageReaderBase_LoadPrivateTagsOff`) = c("SWIGFunction", class('ImageReaderBase_LoadPrivateTagsOff'))

# Start of ImageReaderBase_SetImageIO

`ImageReaderBase_SetImageIO` = function(self, imageio)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  imageio = as(imageio, "character"); 
  ;ans = .Call('R_swig_ImageReaderBase_SetImageIO', self, imageio, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageReaderBase_SetImageIO`, 'returnType') = 'void'
attr(`ImageReaderBase_SetImageIO`, "inputTypes") = c('_p_itk__simple__ImageReaderBase', 'character')
class(`ImageReaderBase_SetImageIO`) = c("SWIGFunction", class('ImageReaderBase_SetImageIO'))

# Start of ImageReaderBase_GetImageIO

`ImageReaderBase_GetImageIO` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageReaderBase_GetImageIO', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageReaderBase_GetImageIO`, 'returnType') = 'character'
attr(`ImageReaderBase_GetImageIO`, "inputTypes") = c('_p_itk__simple__ImageReaderBase')
class(`ImageReaderBase_GetImageIO`) = c("SWIGFunction", class('ImageReaderBase_GetImageIO'))

# Start of ImageReaderBase_GetImageIOFromFileName

`ImageReaderBase_GetImageIOFromFileName` = function(fileName, .copy = FALSE)
{
  fileName = as(fileName, "character"); 
  ;.Call('R_swig_ImageReaderBase_GetImageIOFromFileName', fileName, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageReaderBase_GetImageIOFromFileName`, 'returnType') = 'character'
attr(`ImageReaderBase_GetImageIOFromFileName`, "inputTypes") = c('character')
class(`ImageReaderBase_GetImageIOFromFileName`) = c("SWIGFunction", class('ImageReaderBase_GetImageIOFromFileName'))

# Start of accessor method for itk::simple::ImageReaderBase
setMethod('$', '_p_itk__simple__ImageReaderBase', function(x, name)

{
  accessorFuns = list('SetOutputPixelType' = ImageReaderBase_SetOutputPixelType, 'GetOutputPixelType' = ImageReaderBase_GetOutputPixelType, 'Execute' = ImageReaderBase_Execute, 'ToString' = ImageReaderBase_ToString, 'GetRegisteredImageIOs' = ImageReaderBase_GetRegisteredImageIOs, 'SetLoadPrivateTags' = ImageReaderBase_SetLoadPrivateTags, 'GetLoadPrivateTags' = ImageReaderBase_GetLoadPrivateTags, 'LoadPrivateTagsOn' = ImageReaderBase_LoadPrivateTagsOn, 'LoadPrivateTagsOff' = ImageReaderBase_LoadPrivateTagsOff, 'SetImageIO' = ImageReaderBase_SetImageIO, 'GetImageIO' = ImageReaderBase_GetImageIO);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ImageReaderBase
# Start of delete_ImageSeriesReader

`delete_ImageSeriesReader` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ImageSeriesReader', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ImageSeriesReader`, 'returnType') = 'void'
attr(`delete_ImageSeriesReader`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader')
class(`delete_ImageSeriesReader`) = c("SWIGFunction", class('delete_ImageSeriesReader'))

# Start of new_ImageSeriesReader

`ImageSeriesReader` = function()
{
  ;ans = .Call('R_swig_new_ImageSeriesReader', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ImageSeriesReader", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ImageSeriesReader);
  ans
  
}

attr(`ImageSeriesReader`, 'returnType') = '_p_itk__simple__ImageSeriesReader'
class(`ImageSeriesReader`) = c("SWIGFunction", class('ImageSeriesReader'))

# Start of ImageSeriesReader_ToString

`ImageSeriesReader_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageSeriesReader_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_ToString`, 'returnType') = 'character'
attr(`ImageSeriesReader_ToString`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader')
class(`ImageSeriesReader_ToString`) = c("SWIGFunction", class('ImageSeriesReader_ToString'))

# Start of ImageSeriesReader_GetName

`ImageSeriesReader_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageSeriesReader_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_GetName`, 'returnType') = 'character'
attr(`ImageSeriesReader_GetName`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader')
class(`ImageSeriesReader_GetName`) = c("SWIGFunction", class('ImageSeriesReader_GetName'))

# Start of ImageSeriesReader_SetMetaDataDictionaryArrayUpdate

`ImageSeriesReader_SetMetaDataDictionaryArrayUpdate` = function(self, metaDataDictionaryArrayUpdate)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  metaDataDictionaryArrayUpdate = as.logical(metaDataDictionaryArrayUpdate);
  ;ans = .Call('R_swig_ImageSeriesReader_SetMetaDataDictionaryArrayUpdate', self, metaDataDictionaryArrayUpdate, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesReader_SetMetaDataDictionaryArrayUpdate`, 'returnType') = 'void'
attr(`ImageSeriesReader_SetMetaDataDictionaryArrayUpdate`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader', 'logical')
class(`ImageSeriesReader_SetMetaDataDictionaryArrayUpdate`) = c("SWIGFunction", class('ImageSeriesReader_SetMetaDataDictionaryArrayUpdate'))

# Start of ImageSeriesReader_GetMetaDataDictionaryArrayUpdate

`ImageSeriesReader_GetMetaDataDictionaryArrayUpdate` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageSeriesReader_GetMetaDataDictionaryArrayUpdate', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_GetMetaDataDictionaryArrayUpdate`, 'returnType') = 'logical'
attr(`ImageSeriesReader_GetMetaDataDictionaryArrayUpdate`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader')
class(`ImageSeriesReader_GetMetaDataDictionaryArrayUpdate`) = c("SWIGFunction", class('ImageSeriesReader_GetMetaDataDictionaryArrayUpdate'))

# Start of ImageSeriesReader_MetaDataDictionaryArrayUpdateOn

`ImageSeriesReader_MetaDataDictionaryArrayUpdateOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageSeriesReader_MetaDataDictionaryArrayUpdateOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesReader_MetaDataDictionaryArrayUpdateOn`, 'returnType') = 'void'
attr(`ImageSeriesReader_MetaDataDictionaryArrayUpdateOn`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader')
class(`ImageSeriesReader_MetaDataDictionaryArrayUpdateOn`) = c("SWIGFunction", class('ImageSeriesReader_MetaDataDictionaryArrayUpdateOn'))

# Start of ImageSeriesReader_MetaDataDictionaryArrayUpdateOff

`ImageSeriesReader_MetaDataDictionaryArrayUpdateOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageSeriesReader_MetaDataDictionaryArrayUpdateOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesReader_MetaDataDictionaryArrayUpdateOff`, 'returnType') = 'void'
attr(`ImageSeriesReader_MetaDataDictionaryArrayUpdateOff`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader')
class(`ImageSeriesReader_MetaDataDictionaryArrayUpdateOff`) = c("SWIGFunction", class('ImageSeriesReader_MetaDataDictionaryArrayUpdateOff'))

# Start of ImageSeriesReader_GetGDCMSeriesFileNames

`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_0` = function(directory, seriesID, useSeriesDetails, recursive, loadSequences, .copy = FALSE)
{
  directory = as(directory, "character"); 
  seriesID = as(seriesID, "character"); 
  useSeriesDetails = as.logical(useSeriesDetails);
  recursive = as.logical(recursive);
  loadSequences = as.logical(loadSequences);
  ;.Call('R_swig_ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_0', directory, seriesID, useSeriesDetails, recursive, loadSequences, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_0`, 'returnType') = 'character'
attr(`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_0`, "inputTypes") = c('character', 'character', 'logical', 'logical', 'logical')
class(`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_0`) = c("SWIGFunction", class('ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_0'))

# Start of ImageSeriesReader_GetGDCMSeriesFileNames

`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_1` = function(directory, seriesID, useSeriesDetails, recursive, .copy = FALSE)
{
  directory = as(directory, "character"); 
  seriesID = as(seriesID, "character"); 
  useSeriesDetails = as.logical(useSeriesDetails);
  recursive = as.logical(recursive);
  ;.Call('R_swig_ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_1', directory, seriesID, useSeriesDetails, recursive, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_1`, 'returnType') = 'character'
attr(`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_1`, "inputTypes") = c('character', 'character', 'logical', 'logical')
class(`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_1`) = c("SWIGFunction", class('ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_1'))

# Start of ImageSeriesReader_GetGDCMSeriesFileNames

`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_2` = function(directory, seriesID, useSeriesDetails, .copy = FALSE)
{
  directory = as(directory, "character"); 
  seriesID = as(seriesID, "character"); 
  useSeriesDetails = as.logical(useSeriesDetails);
  ;.Call('R_swig_ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_2', directory, seriesID, useSeriesDetails, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_2`, 'returnType') = 'character'
attr(`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_2`, "inputTypes") = c('character', 'character', 'logical')
class(`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_2`) = c("SWIGFunction", class('ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_2'))

# Start of ImageSeriesReader_GetGDCMSeriesFileNames

`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_3` = function(directory, seriesID, .copy = FALSE)
{
  directory = as(directory, "character"); 
  seriesID = as(seriesID, "character"); 
  ;.Call('R_swig_ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_3', directory, seriesID, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_3`, 'returnType') = 'character'
attr(`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_3`, "inputTypes") = c('character', 'character')
class(`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_3`) = c("SWIGFunction", class('ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_3'))

# Start of ImageSeriesReader_GetGDCMSeriesFileNames

`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_4` = function(directory, .copy = FALSE)
{
  directory = as(directory, "character"); 
  ;.Call('R_swig_ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_4', directory, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_4`, 'returnType') = 'character'
attr(`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_4`, "inputTypes") = c('character')
class(`ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_4`) = c("SWIGFunction", class('ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_4'))

`ImageSeriesReader_GetGDCMSeriesFileNames` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) {
      f <- ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- ImageSeriesReader_GetGDCMSeriesFileNames__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageSeriesReader_GetGDCMSeriesFileNames with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageSeriesReader_GetGDCMSeriesIDs

`ImageSeriesReader_GetGDCMSeriesIDs__SWIG_0` = function(directory, useSeriesDetails, .copy = FALSE)
{
  directory = as(directory, "character"); 
  useSeriesDetails = as.logical(useSeriesDetails);
  ;.Call('R_swig_ImageSeriesReader_GetGDCMSeriesIDs__SWIG_0', directory, useSeriesDetails, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_GetGDCMSeriesIDs__SWIG_0`, 'returnType') = 'character'
attr(`ImageSeriesReader_GetGDCMSeriesIDs__SWIG_0`, "inputTypes") = c('character', 'logical')
class(`ImageSeriesReader_GetGDCMSeriesIDs__SWIG_0`) = c("SWIGFunction", class('ImageSeriesReader_GetGDCMSeriesIDs__SWIG_0'))

# Start of ImageSeriesReader_GetGDCMSeriesIDs

`ImageSeriesReader_GetGDCMSeriesIDs__SWIG_1` = function(directory, .copy = FALSE)
{
  directory = as(directory, "character"); 
  ;.Call('R_swig_ImageSeriesReader_GetGDCMSeriesIDs__SWIG_1', directory, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_GetGDCMSeriesIDs__SWIG_1`, 'returnType') = 'character'
attr(`ImageSeriesReader_GetGDCMSeriesIDs__SWIG_1`, "inputTypes") = c('character')
class(`ImageSeriesReader_GetGDCMSeriesIDs__SWIG_1`) = c("SWIGFunction", class('ImageSeriesReader_GetGDCMSeriesIDs__SWIG_1'))

`ImageSeriesReader_GetGDCMSeriesIDs` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) {
      f <- ImageSeriesReader_GetGDCMSeriesIDs__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ImageSeriesReader_GetGDCMSeriesIDs__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageSeriesReader_GetGDCMSeriesIDs with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageSeriesReader_SetFileNames

`ImageSeriesReader_SetFileNames` = function(self, fileNames)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  fileNames = as.character(fileNames);
  ;ans = .Call('R_swig_ImageSeriesReader_SetFileNames', self, fileNames, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesReader_SetFileNames`, 'returnType') = 'void'
attr(`ImageSeriesReader_SetFileNames`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader', 'character')
class(`ImageSeriesReader_SetFileNames`) = c("SWIGFunction", class('ImageSeriesReader_SetFileNames'))

# Start of ImageSeriesReader_GetFileNames

`ImageSeriesReader_GetFileNames` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageSeriesReader_GetFileNames', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_GetFileNames`, 'returnType') = 'character'
attr(`ImageSeriesReader_GetFileNames`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader')
class(`ImageSeriesReader_GetFileNames`) = c("SWIGFunction", class('ImageSeriesReader_GetFileNames'))

# Start of ImageSeriesReader_SetSpacingWarningRelThreshold

`ImageSeriesReader_SetSpacingWarningRelThreshold` = function(self, spacingWarningRelThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ImageSeriesReader_SetSpacingWarningRelThreshold', self, spacingWarningRelThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesReader_SetSpacingWarningRelThreshold`, 'returnType') = 'void'
attr(`ImageSeriesReader_SetSpacingWarningRelThreshold`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader', 'numeric')
class(`ImageSeriesReader_SetSpacingWarningRelThreshold`) = c("SWIGFunction", class('ImageSeriesReader_SetSpacingWarningRelThreshold'))

# Start of ImageSeriesReader_GetSpacingWarningRelThreshold

`ImageSeriesReader_GetSpacingWarningRelThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageSeriesReader_GetSpacingWarningRelThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_GetSpacingWarningRelThreshold`, 'returnType') = 'numeric'
attr(`ImageSeriesReader_GetSpacingWarningRelThreshold`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader')
class(`ImageSeriesReader_GetSpacingWarningRelThreshold`) = c("SWIGFunction", class('ImageSeriesReader_GetSpacingWarningRelThreshold'))

# Start of ImageSeriesReader_SetForceOrthogonalDirection

`ImageSeriesReader_SetForceOrthogonalDirection` = function(self, forceOrthogonalDirection)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  forceOrthogonalDirection = as.logical(forceOrthogonalDirection);
  ;ans = .Call('R_swig_ImageSeriesReader_SetForceOrthogonalDirection', self, forceOrthogonalDirection, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesReader_SetForceOrthogonalDirection`, 'returnType') = 'void'
attr(`ImageSeriesReader_SetForceOrthogonalDirection`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader', 'logical')
class(`ImageSeriesReader_SetForceOrthogonalDirection`) = c("SWIGFunction", class('ImageSeriesReader_SetForceOrthogonalDirection'))

# Start of ImageSeriesReader_GetForceOrthogonalDirection

`ImageSeriesReader_GetForceOrthogonalDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageSeriesReader_GetForceOrthogonalDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_GetForceOrthogonalDirection`, 'returnType') = 'logical'
attr(`ImageSeriesReader_GetForceOrthogonalDirection`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader')
class(`ImageSeriesReader_GetForceOrthogonalDirection`) = c("SWIGFunction", class('ImageSeriesReader_GetForceOrthogonalDirection'))

# Start of ImageSeriesReader_ForceOrthogonalDirectionOn

`ImageSeriesReader_ForceOrthogonalDirectionOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageSeriesReader_ForceOrthogonalDirectionOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesReader_ForceOrthogonalDirectionOn`, 'returnType') = 'void'
attr(`ImageSeriesReader_ForceOrthogonalDirectionOn`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader')
class(`ImageSeriesReader_ForceOrthogonalDirectionOn`) = c("SWIGFunction", class('ImageSeriesReader_ForceOrthogonalDirectionOn'))

# Start of ImageSeriesReader_ForceOrthogonalDirectionOff

`ImageSeriesReader_ForceOrthogonalDirectionOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageSeriesReader_ForceOrthogonalDirectionOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesReader_ForceOrthogonalDirectionOff`, 'returnType') = 'void'
attr(`ImageSeriesReader_ForceOrthogonalDirectionOff`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader')
class(`ImageSeriesReader_ForceOrthogonalDirectionOff`) = c("SWIGFunction", class('ImageSeriesReader_ForceOrthogonalDirectionOff'))

# Start of ImageSeriesReader_SetReverseOrder

`ImageSeriesReader_SetReverseOrder` = function(self, reverseOrder)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  reverseOrder = as.logical(reverseOrder);
  ;ans = .Call('R_swig_ImageSeriesReader_SetReverseOrder', self, reverseOrder, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesReader_SetReverseOrder`, 'returnType') = 'void'
attr(`ImageSeriesReader_SetReverseOrder`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader', 'logical')
class(`ImageSeriesReader_SetReverseOrder`) = c("SWIGFunction", class('ImageSeriesReader_SetReverseOrder'))

# Start of ImageSeriesReader_GetReverseOrder

`ImageSeriesReader_GetReverseOrder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageSeriesReader_GetReverseOrder', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_GetReverseOrder`, 'returnType') = 'logical'
attr(`ImageSeriesReader_GetReverseOrder`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader')
class(`ImageSeriesReader_GetReverseOrder`) = c("SWIGFunction", class('ImageSeriesReader_GetReverseOrder'))

# Start of ImageSeriesReader_ReverseOrderOn

`ImageSeriesReader_ReverseOrderOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageSeriesReader_ReverseOrderOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesReader_ReverseOrderOn`, 'returnType') = 'void'
attr(`ImageSeriesReader_ReverseOrderOn`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader')
class(`ImageSeriesReader_ReverseOrderOn`) = c("SWIGFunction", class('ImageSeriesReader_ReverseOrderOn'))

# Start of ImageSeriesReader_ReverseOrderOff

`ImageSeriesReader_ReverseOrderOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageSeriesReader_ReverseOrderOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageSeriesReader_ReverseOrderOff`, 'returnType') = 'void'
attr(`ImageSeriesReader_ReverseOrderOff`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader')
class(`ImageSeriesReader_ReverseOrderOff`) = c("SWIGFunction", class('ImageSeriesReader_ReverseOrderOff'))

# Start of ImageSeriesReader_Execute

`ImageSeriesReader_Execute` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageSeriesReader_Execute', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ImageSeriesReader_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ImageSeriesReader_Execute`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader')
class(`ImageSeriesReader_Execute`) = c("SWIGFunction", class('ImageSeriesReader_Execute'))

# Start of ImageSeriesReader_GetMetaDataKeys

`ImageSeriesReader_GetMetaDataKeys` = function(self, slice, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  slice = as.integer(slice);
  
  if(length(slice) > 1) {
    warning("using only the first element of slice");
  };
  
  ;.Call('R_swig_ImageSeriesReader_GetMetaDataKeys', self, slice, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_GetMetaDataKeys`, 'returnType') = 'character'
attr(`ImageSeriesReader_GetMetaDataKeys`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader', 'integer')
class(`ImageSeriesReader_GetMetaDataKeys`) = c("SWIGFunction", class('ImageSeriesReader_GetMetaDataKeys'))

# Start of ImageSeriesReader_HasMetaDataKey

`ImageSeriesReader_HasMetaDataKey` = function(self, slice, key, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  slice = as.integer(slice);
  
  if(length(slice) > 1) {
    warning("using only the first element of slice");
  };
  
  key = as(key, "character"); 
  ;.Call('R_swig_ImageSeriesReader_HasMetaDataKey', self, slice, key, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_HasMetaDataKey`, 'returnType') = 'logical'
attr(`ImageSeriesReader_HasMetaDataKey`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader', 'integer', 'character')
class(`ImageSeriesReader_HasMetaDataKey`) = c("SWIGFunction", class('ImageSeriesReader_HasMetaDataKey'))

# Start of ImageSeriesReader_GetMetaData

`ImageSeriesReader_GetMetaData` = function(self, slice, key, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  slice = as.integer(slice);
  
  if(length(slice) > 1) {
    warning("using only the first element of slice");
  };
  
  key = as(key, "character"); 
  ;.Call('R_swig_ImageSeriesReader_GetMetaData', self, slice, key, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageSeriesReader_GetMetaData`, 'returnType') = 'character'
attr(`ImageSeriesReader_GetMetaData`, "inputTypes") = c('_p_itk__simple__ImageSeriesReader', 'integer', 'character')
class(`ImageSeriesReader_GetMetaData`) = c("SWIGFunction", class('ImageSeriesReader_GetMetaData'))

# Start of accessor method for itk::simple::ImageSeriesReader
setMethod('$', '_p_itk__simple__ImageSeriesReader', function(x, name)

{
  accessorFuns = list('ToString' = ImageSeriesReader_ToString, 'GetName' = ImageSeriesReader_GetName, 'SetMetaDataDictionaryArrayUpdate' = ImageSeriesReader_SetMetaDataDictionaryArrayUpdate, 'GetMetaDataDictionaryArrayUpdate' = ImageSeriesReader_GetMetaDataDictionaryArrayUpdate, 'MetaDataDictionaryArrayUpdateOn' = ImageSeriesReader_MetaDataDictionaryArrayUpdateOn, 'MetaDataDictionaryArrayUpdateOff' = ImageSeriesReader_MetaDataDictionaryArrayUpdateOff, 'SetFileNames' = ImageSeriesReader_SetFileNames, 'GetFileNames' = ImageSeriesReader_GetFileNames, 'SetSpacingWarningRelThreshold' = ImageSeriesReader_SetSpacingWarningRelThreshold, 'GetSpacingWarningRelThreshold' = ImageSeriesReader_GetSpacingWarningRelThreshold, 'SetForceOrthogonalDirection' = ImageSeriesReader_SetForceOrthogonalDirection, 'GetForceOrthogonalDirection' = ImageSeriesReader_GetForceOrthogonalDirection, 'ForceOrthogonalDirectionOn' = ImageSeriesReader_ForceOrthogonalDirectionOn, 'ForceOrthogonalDirectionOff' = ImageSeriesReader_ForceOrthogonalDirectionOff, 'SetReverseOrder' = ImageSeriesReader_SetReverseOrder, 'GetReverseOrder' = ImageSeriesReader_GetReverseOrder, 'ReverseOrderOn' = ImageSeriesReader_ReverseOrderOn, 'ReverseOrderOff' = ImageSeriesReader_ReverseOrderOff, 'Execute' = ImageSeriesReader_Execute, 'GetMetaDataKeys' = ImageSeriesReader_GetMetaDataKeys, 'HasMetaDataKey' = ImageSeriesReader_HasMetaDataKey, 'GetMetaData' = ImageSeriesReader_GetMetaData);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ImageSeriesReader
# Start of ReadImage

`ReadImage__SWIG_0` = function(fileNames, outputPixelType, imageIO, .copy = FALSE)
{
  fileNames = as.character(fileNames);
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  imageIO = as(imageIO, "character"); 
  ;ans = .Call('R_swig_ReadImage__SWIG_0', fileNames, outputPixelType, imageIO, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReadImage__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ReadImage__SWIG_0`, "inputTypes") = c('character', 'character', 'character')
class(`ReadImage__SWIG_0`) = c("SWIGFunction", class('ReadImage__SWIG_0'))

# Start of ReadImage

`ReadImage__SWIG_1` = function(fileNames, outputPixelType, .copy = FALSE)
{
  fileNames = as.character(fileNames);
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  ;ans = .Call('R_swig_ReadImage__SWIG_1', fileNames, outputPixelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReadImage__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ReadImage__SWIG_1`, "inputTypes") = c('character', 'character')
class(`ReadImage__SWIG_1`) = c("SWIGFunction", class('ReadImage__SWIG_1'))

# Start of ReadImage

`ReadImage__SWIG_2` = function(fileNames, .copy = FALSE)
{
  fileNames = as.character(fileNames);
  ;ans = .Call('R_swig_ReadImage__SWIG_2', fileNames, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReadImage__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ReadImage__SWIG_2`, "inputTypes") = c('character')
class(`ReadImage__SWIG_2`) = c("SWIGFunction", class('ReadImage__SWIG_2'))

# Start of delete_ImageFileReader

`delete_ImageFileReader` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ImageFileReader', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ImageFileReader`, 'returnType') = 'void'
attr(`delete_ImageFileReader`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`delete_ImageFileReader`) = c("SWIGFunction", class('delete_ImageFileReader'))

# Start of new_ImageFileReader

`ImageFileReader` = function()
{
  ;ans = .Call('R_swig_new_ImageFileReader', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ImageFileReader", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ImageFileReader);
  ans
  
}

attr(`ImageFileReader`, 'returnType') = '_p_itk__simple__ImageFileReader'
class(`ImageFileReader`) = c("SWIGFunction", class('ImageFileReader'))

# Start of ImageFileReader_ToString

`ImageFileReader_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileReader_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileReader_ToString`, 'returnType') = 'character'
attr(`ImageFileReader_ToString`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_ToString`) = c("SWIGFunction", class('ImageFileReader_ToString'))

# Start of ImageFileReader_GetName

`ImageFileReader_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileReader_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileReader_GetName`, 'returnType') = 'character'
attr(`ImageFileReader_GetName`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_GetName`) = c("SWIGFunction", class('ImageFileReader_GetName'))

# Start of ImageFileReader_SetFileName

`ImageFileReader_SetFileName` = function(self, fn)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  fn = as(fn, "character"); 
  ;ans = .Call('R_swig_ImageFileReader_SetFileName', self, fn, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileReader_SetFileName`, 'returnType') = 'void'
attr(`ImageFileReader_SetFileName`, "inputTypes") = c('_p_itk__simple__ImageFileReader', 'character')
class(`ImageFileReader_SetFileName`) = c("SWIGFunction", class('ImageFileReader_SetFileName'))

# Start of ImageFileReader_GetFileName

`ImageFileReader_GetFileName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileReader_GetFileName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileReader_GetFileName`, 'returnType') = 'character'
attr(`ImageFileReader_GetFileName`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_GetFileName`) = c("SWIGFunction", class('ImageFileReader_GetFileName'))

# Start of ImageFileReader_Execute

`ImageFileReader_Execute` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageFileReader_Execute', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ImageFileReader_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ImageFileReader_Execute`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_Execute`) = c("SWIGFunction", class('ImageFileReader_Execute'))

# Start of ImageFileReader_ReadImageInformation

`ImageFileReader_ReadImageInformation` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageFileReader_ReadImageInformation', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileReader_ReadImageInformation`, 'returnType') = 'void'
attr(`ImageFileReader_ReadImageInformation`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_ReadImageInformation`) = c("SWIGFunction", class('ImageFileReader_ReadImageInformation'))

# Start of ImageFileReader_GetPixelID

`ImageFileReader_GetPixelID` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageFileReader_GetPixelID', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`ImageFileReader_GetPixelID`, 'returnType') = 'character'
attr(`ImageFileReader_GetPixelID`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_GetPixelID`) = c("SWIGFunction", class('ImageFileReader_GetPixelID'))

# Start of ImageFileReader_GetPixelIDValue

`ImageFileReader_GetPixelIDValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileReader_GetPixelIDValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileReader_GetPixelIDValue`, 'returnType') = 'integer'
attr(`ImageFileReader_GetPixelIDValue`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_GetPixelIDValue`) = c("SWIGFunction", class('ImageFileReader_GetPixelIDValue'))

# Start of ImageFileReader_GetDimension

`ImageFileReader_GetDimension` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileReader_GetDimension', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileReader_GetDimension`, 'returnType') = 'integer'
attr(`ImageFileReader_GetDimension`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_GetDimension`) = c("SWIGFunction", class('ImageFileReader_GetDimension'))

# Start of ImageFileReader_GetNumberOfComponents

`ImageFileReader_GetNumberOfComponents` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileReader_GetNumberOfComponents', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileReader_GetNumberOfComponents`, 'returnType') = 'integer'
attr(`ImageFileReader_GetNumberOfComponents`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_GetNumberOfComponents`) = c("SWIGFunction", class('ImageFileReader_GetNumberOfComponents'))

# Start of ImageFileReader_GetOrigin

`ImageFileReader_GetOrigin` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileReader_GetOrigin', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileReader_GetOrigin`, 'returnType') = '_p_std__vectorT_double_t'
attr(`ImageFileReader_GetOrigin`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_GetOrigin`) = c("SWIGFunction", class('ImageFileReader_GetOrigin'))

# Start of ImageFileReader_GetSpacing

`ImageFileReader_GetSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileReader_GetSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileReader_GetSpacing`, 'returnType') = '_p_std__vectorT_double_t'
attr(`ImageFileReader_GetSpacing`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_GetSpacing`) = c("SWIGFunction", class('ImageFileReader_GetSpacing'))

# Start of ImageFileReader_GetDirection

`ImageFileReader_GetDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileReader_GetDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileReader_GetDirection`, 'returnType') = '_p_std__vectorT_double_t'
attr(`ImageFileReader_GetDirection`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_GetDirection`) = c("SWIGFunction", class('ImageFileReader_GetDirection'))

# Start of ImageFileReader_GetSize

`ImageFileReader_GetSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileReader_GetSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileReader_GetSize`, 'returnType') = 'integer'
attr(`ImageFileReader_GetSize`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_GetSize`) = c("SWIGFunction", class('ImageFileReader_GetSize'))

# Start of ImageFileReader_GetMetaDataKeys

`ImageFileReader_GetMetaDataKeys` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileReader_GetMetaDataKeys', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileReader_GetMetaDataKeys`, 'returnType') = 'character'
attr(`ImageFileReader_GetMetaDataKeys`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_GetMetaDataKeys`) = c("SWIGFunction", class('ImageFileReader_GetMetaDataKeys'))

# Start of ImageFileReader_HasMetaDataKey

`ImageFileReader_HasMetaDataKey` = function(self, key, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  key = as(key, "character"); 
  ;.Call('R_swig_ImageFileReader_HasMetaDataKey', self, key, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileReader_HasMetaDataKey`, 'returnType') = 'logical'
attr(`ImageFileReader_HasMetaDataKey`, "inputTypes") = c('_p_itk__simple__ImageFileReader', 'character')
class(`ImageFileReader_HasMetaDataKey`) = c("SWIGFunction", class('ImageFileReader_HasMetaDataKey'))

# Start of ImageFileReader_GetMetaData

`ImageFileReader_GetMetaData` = function(self, key, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  key = as(key, "character"); 
  ;.Call('R_swig_ImageFileReader_GetMetaData', self, key, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileReader_GetMetaData`, 'returnType') = 'character'
attr(`ImageFileReader_GetMetaData`, "inputTypes") = c('_p_itk__simple__ImageFileReader', 'character')
class(`ImageFileReader_GetMetaData`) = c("SWIGFunction", class('ImageFileReader_GetMetaData'))

# Start of ImageFileReader_SetExtractSize

`ImageFileReader_SetExtractSize` = function(self, size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  size = as.integer(size);
  ;ans = .Call('R_swig_ImageFileReader_SetExtractSize', self, size, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileReader_SetExtractSize`, 'returnType') = 'void'
attr(`ImageFileReader_SetExtractSize`, "inputTypes") = c('_p_itk__simple__ImageFileReader', 'integer')
class(`ImageFileReader_SetExtractSize`) = c("SWIGFunction", class('ImageFileReader_SetExtractSize'))

# Start of ImageFileReader_GetExtractSize

`ImageFileReader_GetExtractSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileReader_GetExtractSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileReader_GetExtractSize`, 'returnType') = 'integer'
attr(`ImageFileReader_GetExtractSize`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_GetExtractSize`) = c("SWIGFunction", class('ImageFileReader_GetExtractSize'))

# Start of ImageFileReader_SetExtractIndex

`ImageFileReader_SetExtractIndex` = function(self, index)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  index = as.integer(index);
  ;ans = .Call('R_swig_ImageFileReader_SetExtractIndex', self, index, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageFileReader_SetExtractIndex`, 'returnType') = 'void'
attr(`ImageFileReader_SetExtractIndex`, "inputTypes") = c('_p_itk__simple__ImageFileReader', '_p_std__vectorT_int32_t_t')
class(`ImageFileReader_SetExtractIndex`) = c("SWIGFunction", class('ImageFileReader_SetExtractIndex'))

# Start of ImageFileReader_GetExtractIndex

`ImageFileReader_GetExtractIndex` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageFileReader_GetExtractIndex', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageFileReader_GetExtractIndex`, 'returnType') = '_p_std__vectorT_int32_t_t'
attr(`ImageFileReader_GetExtractIndex`, "inputTypes") = c('_p_itk__simple__ImageFileReader')
class(`ImageFileReader_GetExtractIndex`) = c("SWIGFunction", class('ImageFileReader_GetExtractIndex'))

# Start of accessor method for itk::simple::ImageFileReader
setMethod('$', '_p_itk__simple__ImageFileReader', function(x, name)

{
  accessorFuns = list('ToString' = ImageFileReader_ToString, 'GetName' = ImageFileReader_GetName, 'SetFileName' = ImageFileReader_SetFileName, 'GetFileName' = ImageFileReader_GetFileName, 'Execute' = ImageFileReader_Execute, 'ReadImageInformation' = ImageFileReader_ReadImageInformation, 'GetPixelID' = ImageFileReader_GetPixelID, 'GetPixelIDValue' = ImageFileReader_GetPixelIDValue, 'GetDimension' = ImageFileReader_GetDimension, 'GetNumberOfComponents' = ImageFileReader_GetNumberOfComponents, 'GetOrigin' = ImageFileReader_GetOrigin, 'GetSpacing' = ImageFileReader_GetSpacing, 'GetDirection' = ImageFileReader_GetDirection, 'GetSize' = ImageFileReader_GetSize, 'GetMetaDataKeys' = ImageFileReader_GetMetaDataKeys, 'HasMetaDataKey' = ImageFileReader_HasMetaDataKey, 'GetMetaData' = ImageFileReader_GetMetaData, 'SetExtractSize' = ImageFileReader_SetExtractSize, 'GetExtractSize' = ImageFileReader_GetExtractSize, 'SetExtractIndex' = ImageFileReader_SetExtractIndex, 'GetExtractIndex' = ImageFileReader_GetExtractIndex);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ImageFileReader
# Start of ReadImage

`ReadImage__SWIG_3` = function(filename, outputPixelType, imageIO, .copy = FALSE)
{
  filename = as(filename, "character"); 
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  imageIO = as(imageIO, "character"); 
  ;ans = .Call('R_swig_ReadImage__SWIG_3', filename, outputPixelType, imageIO, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReadImage__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ReadImage__SWIG_3`, "inputTypes") = c('character', 'character', 'character')
class(`ReadImage__SWIG_3`) = c("SWIGFunction", class('ReadImage__SWIG_3'))

# Start of ReadImage

`ReadImage__SWIG_4` = function(filename, outputPixelType, .copy = FALSE)
{
  filename = as(filename, "character"); 
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  ;ans = .Call('R_swig_ReadImage__SWIG_4', filename, outputPixelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReadImage__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ReadImage__SWIG_4`, "inputTypes") = c('character', 'character')
class(`ReadImage__SWIG_4`) = c("SWIGFunction", class('ReadImage__SWIG_4'))

# Start of ReadImage

`ReadImage__SWIG_5` = function(filename, .copy = FALSE)
{
  filename = as(filename, "character"); 
  ;ans = .Call('R_swig_ReadImage__SWIG_5', filename, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReadImage__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`ReadImage__SWIG_5`, "inputTypes") = c('character')
class(`ReadImage__SWIG_5`) = c("SWIGFunction", class('ReadImage__SWIG_5'))

`ReadImage` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) {
      f <- ReadImage__SWIG_5; 
    }
    else if ( is.character(argv[[1]]) ) {
      f <- ReadImage__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ReadImage__SWIG_4; 
    }
    else if (( is.character(argv[[1]]) ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ReadImage__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ReadImage__SWIG_3; 
    }
    else if (( is.character(argv[[1]]) ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ReadImage__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ReadImage with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of new_ImageViewer

`ImageViewer` = function()
{
  ;ans = .Call('R_swig_new_ImageViewer', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ImageViewer", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ImageViewer);
  ans
  
}

attr(`ImageViewer`, 'returnType') = '_p_itk__simple__ImageViewer'
class(`ImageViewer`) = c("SWIGFunction", class('ImageViewer'))

# Start of ImageViewer_GetName

`ImageViewer_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageViewer_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageViewer_GetName`, 'returnType') = 'character'
attr(`ImageViewer_GetName`, "inputTypes") = c('_p_itk__simple__ImageViewer')
class(`ImageViewer_GetName`) = c("SWIGFunction", class('ImageViewer_GetName'))

# Start of ImageViewer_SetGlobalDefaultSearchPath

`ImageViewer_SetGlobalDefaultSearchPath` = function(path)
{
  path = as.character(path);
  ;ans = .Call('R_swig_ImageViewer_SetGlobalDefaultSearchPath', path, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageViewer_SetGlobalDefaultSearchPath`, 'returnType') = 'void'
attr(`ImageViewer_SetGlobalDefaultSearchPath`, "inputTypes") = c('character')
class(`ImageViewer_SetGlobalDefaultSearchPath`) = c("SWIGFunction", class('ImageViewer_SetGlobalDefaultSearchPath'))

# Start of ImageViewer_GetGlobalDefaultSearchPath

`ImageViewer_GetGlobalDefaultSearchPath` = function(.copy = FALSE)
{
  ;.Call('R_swig_ImageViewer_GetGlobalDefaultSearchPath', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageViewer_GetGlobalDefaultSearchPath`, 'returnType') = 'character'
class(`ImageViewer_GetGlobalDefaultSearchPath`) = c("SWIGFunction", class('ImageViewer_GetGlobalDefaultSearchPath'))

# Start of ImageViewer_SetGlobalDefaultExecutableNames

`ImageViewer_SetGlobalDefaultExecutableNames` = function(names)
{
  names = as.character(names);
  ;ans = .Call('R_swig_ImageViewer_SetGlobalDefaultExecutableNames', names, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageViewer_SetGlobalDefaultExecutableNames`, 'returnType') = 'void'
attr(`ImageViewer_SetGlobalDefaultExecutableNames`, "inputTypes") = c('character')
class(`ImageViewer_SetGlobalDefaultExecutableNames`) = c("SWIGFunction", class('ImageViewer_SetGlobalDefaultExecutableNames'))

# Start of ImageViewer_GetGlobalDefaultExecutableNames

`ImageViewer_GetGlobalDefaultExecutableNames` = function(.copy = FALSE)
{
  ;.Call('R_swig_ImageViewer_GetGlobalDefaultExecutableNames', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageViewer_GetGlobalDefaultExecutableNames`, 'returnType') = 'character'
class(`ImageViewer_GetGlobalDefaultExecutableNames`) = c("SWIGFunction", class('ImageViewer_GetGlobalDefaultExecutableNames'))

# Start of ImageViewer_SetGlobalDefaultFileExtension

`ImageViewer_SetGlobalDefaultFileExtension` = function(ext)
{
  ext = as(ext, "character"); 
  ;ans = .Call('R_swig_ImageViewer_SetGlobalDefaultFileExtension', ext, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageViewer_SetGlobalDefaultFileExtension`, 'returnType') = 'void'
attr(`ImageViewer_SetGlobalDefaultFileExtension`, "inputTypes") = c('character')
class(`ImageViewer_SetGlobalDefaultFileExtension`) = c("SWIGFunction", class('ImageViewer_SetGlobalDefaultFileExtension'))

# Start of ImageViewer_GetGlobalDefaultFileExtension

`ImageViewer_GetGlobalDefaultFileExtension` = function(.copy = FALSE)
{
  ;.Call('R_swig_ImageViewer_GetGlobalDefaultFileExtension', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageViewer_GetGlobalDefaultFileExtension`, 'returnType') = 'character'
class(`ImageViewer_GetGlobalDefaultFileExtension`) = c("SWIGFunction", class('ImageViewer_GetGlobalDefaultFileExtension'))

# Start of ImageViewer_SetGlobalDefaultApplication

`ImageViewer_SetGlobalDefaultApplication` = function(app)
{
  app = as(app, "character"); 
  ;ans = .Call('R_swig_ImageViewer_SetGlobalDefaultApplication', app, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageViewer_SetGlobalDefaultApplication`, 'returnType') = 'void'
attr(`ImageViewer_SetGlobalDefaultApplication`, "inputTypes") = c('character')
class(`ImageViewer_SetGlobalDefaultApplication`) = c("SWIGFunction", class('ImageViewer_SetGlobalDefaultApplication'))

# Start of ImageViewer_GetGlobalDefaultApplication

`ImageViewer_GetGlobalDefaultApplication` = function(.copy = FALSE)
{
  ;.Call('R_swig_ImageViewer_GetGlobalDefaultApplication', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageViewer_GetGlobalDefaultApplication`, 'returnType') = 'character'
class(`ImageViewer_GetGlobalDefaultApplication`) = c("SWIGFunction", class('ImageViewer_GetGlobalDefaultApplication'))

# Start of ImageViewer_SetApplication

`ImageViewer_SetApplication__SWIG_0` = function(self, app, command)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  app = as(app, "character"); 
  command = as(command, "character"); 
  ;ans = .Call('R_swig_ImageViewer_SetApplication__SWIG_0', self, app, command, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageViewer_SetApplication__SWIG_0`, 'returnType') = 'void'
attr(`ImageViewer_SetApplication__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageViewer', 'character', 'character')
class(`ImageViewer_SetApplication__SWIG_0`) = c("SWIGFunction", class('ImageViewer_SetApplication__SWIG_0'))

# Start of ImageViewer_SetApplication

`ImageViewer_SetApplication__SWIG_1` = function(self, app)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  app = as(app, "character"); 
  ;ans = .Call('R_swig_ImageViewer_SetApplication__SWIG_1', self, app, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageViewer_SetApplication__SWIG_1`, 'returnType') = 'void'
attr(`ImageViewer_SetApplication__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageViewer', 'character')
class(`ImageViewer_SetApplication__SWIG_1`) = c("SWIGFunction", class('ImageViewer_SetApplication__SWIG_1'))

`ImageViewer_SetApplication` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageViewer') || is.null(argv[[1]]) ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ImageViewer_SetApplication__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageViewer') || is.null(argv[[1]]) ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ImageViewer_SetApplication__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageViewer_SetApplication with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageViewer_GetApplication

`ImageViewer_GetApplication` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageViewer_GetApplication', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageViewer_GetApplication`, 'returnType') = 'character'
attr(`ImageViewer_GetApplication`, "inputTypes") = c('_p_itk__simple__ImageViewer')
class(`ImageViewer_GetApplication`) = c("SWIGFunction", class('ImageViewer_GetApplication'))

# Start of ImageViewer_SetCommand

`ImageViewer_SetCommand` = function(self, command)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  command = as(command, "character"); 
  ;ans = .Call('R_swig_ImageViewer_SetCommand', self, command, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageViewer_SetCommand`, 'returnType') = 'void'
attr(`ImageViewer_SetCommand`, "inputTypes") = c('_p_itk__simple__ImageViewer', 'character')
class(`ImageViewer_SetCommand`) = c("SWIGFunction", class('ImageViewer_SetCommand'))

# Start of ImageViewer_GetCommand

`ImageViewer_GetCommand` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageViewer_GetCommand', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageViewer_GetCommand`, 'returnType') = 'character'
attr(`ImageViewer_GetCommand`, "inputTypes") = c('_p_itk__simple__ImageViewer')
class(`ImageViewer_GetCommand`) = c("SWIGFunction", class('ImageViewer_GetCommand'))

# Start of ImageViewer_SetFileExtension

`ImageViewer_SetFileExtension` = function(self, ext)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ext = as(ext, "character"); 
  ;ans = .Call('R_swig_ImageViewer_SetFileExtension', self, ext, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageViewer_SetFileExtension`, 'returnType') = 'void'
attr(`ImageViewer_SetFileExtension`, "inputTypes") = c('_p_itk__simple__ImageViewer', 'character')
class(`ImageViewer_SetFileExtension`) = c("SWIGFunction", class('ImageViewer_SetFileExtension'))

# Start of ImageViewer_GetFileExtension

`ImageViewer_GetFileExtension` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageViewer_GetFileExtension', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageViewer_GetFileExtension`, 'returnType') = 'character'
attr(`ImageViewer_GetFileExtension`, "inputTypes") = c('_p_itk__simple__ImageViewer')
class(`ImageViewer_GetFileExtension`) = c("SWIGFunction", class('ImageViewer_GetFileExtension'))

# Start of ImageViewer_SetGlobalDefaultDebug

`ImageViewer_SetGlobalDefaultDebug` = function(dbg)
{
  dbg = as.logical(dbg);
  ;ans = .Call('R_swig_ImageViewer_SetGlobalDefaultDebug', dbg, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageViewer_SetGlobalDefaultDebug`, 'returnType') = 'void'
attr(`ImageViewer_SetGlobalDefaultDebug`, "inputTypes") = c('logical')
class(`ImageViewer_SetGlobalDefaultDebug`) = c("SWIGFunction", class('ImageViewer_SetGlobalDefaultDebug'))

# Start of ImageViewer_GetGlobalDefaultDebug

`ImageViewer_GetGlobalDefaultDebug` = function(.copy = FALSE)
{
  ;.Call('R_swig_ImageViewer_GetGlobalDefaultDebug', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageViewer_GetGlobalDefaultDebug`, 'returnType') = 'logical'
class(`ImageViewer_GetGlobalDefaultDebug`) = c("SWIGFunction", class('ImageViewer_GetGlobalDefaultDebug'))

# Start of ImageViewer_SetGlobalDefaultDebugOn

`ImageViewer_SetGlobalDefaultDebugOn` = function()
{
  ;ans = .Call('R_swig_ImageViewer_SetGlobalDefaultDebugOn', PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageViewer_SetGlobalDefaultDebugOn`, 'returnType') = 'void'
class(`ImageViewer_SetGlobalDefaultDebugOn`) = c("SWIGFunction", class('ImageViewer_SetGlobalDefaultDebugOn'))

# Start of ImageViewer_SetGlobalDefaultDebugOff

`ImageViewer_SetGlobalDefaultDebugOff` = function()
{
  ;ans = .Call('R_swig_ImageViewer_SetGlobalDefaultDebugOff', PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageViewer_SetGlobalDefaultDebugOff`, 'returnType') = 'void'
class(`ImageViewer_SetGlobalDefaultDebugOff`) = c("SWIGFunction", class('ImageViewer_SetGlobalDefaultDebugOff'))

# Start of ImageViewer_SetProcessDelay

`ImageViewer_SetProcessDelay` = function(delay)
{
  delay = as.integer(delay);
  
  if(length(delay) > 1) {
    warning("using only the first element of delay");
  };
  
  ;ans = .Call('R_swig_ImageViewer_SetProcessDelay', delay, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageViewer_SetProcessDelay`, 'returnType') = 'void'
attr(`ImageViewer_SetProcessDelay`, "inputTypes") = c('integer')
class(`ImageViewer_SetProcessDelay`) = c("SWIGFunction", class('ImageViewer_SetProcessDelay'))

# Start of ImageViewer_GetProcessDelay

`ImageViewer_GetProcessDelay` = function(.copy = FALSE)
{
  ;.Call('R_swig_ImageViewer_GetProcessDelay', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageViewer_GetProcessDelay`, 'returnType') = 'integer'
class(`ImageViewer_GetProcessDelay`) = c("SWIGFunction", class('ImageViewer_GetProcessDelay'))

# Start of ImageViewer_SetTitle

`ImageViewer_SetTitle` = function(self, t)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  t = as(t, "character"); 
  ;ans = .Call('R_swig_ImageViewer_SetTitle', self, t, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageViewer_SetTitle`, 'returnType') = 'void'
attr(`ImageViewer_SetTitle`, "inputTypes") = c('_p_itk__simple__ImageViewer', 'character')
class(`ImageViewer_SetTitle`) = c("SWIGFunction", class('ImageViewer_SetTitle'))

# Start of ImageViewer_GetTitle

`ImageViewer_GetTitle` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageViewer_GetTitle', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageViewer_GetTitle`, 'returnType') = 'character'
attr(`ImageViewer_GetTitle`, "inputTypes") = c('_p_itk__simple__ImageViewer')
class(`ImageViewer_GetTitle`) = c("SWIGFunction", class('ImageViewer_GetTitle'))

# Start of ImageViewer_Execute

`ImageViewer_Execute` = function(self, image)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_ImageViewer_Execute', self, image, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageViewer_Execute`, 'returnType') = 'void'
attr(`ImageViewer_Execute`, "inputTypes") = c('_p_itk__simple__ImageViewer', '_p_itk__simple__Image')
class(`ImageViewer_Execute`) = c("SWIGFunction", class('ImageViewer_Execute'))

# Start of ImageViewer_ToString

`ImageViewer_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageViewer_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageViewer_ToString`, 'returnType') = 'character'
attr(`ImageViewer_ToString`, "inputTypes") = c('_p_itk__simple__ImageViewer')
class(`ImageViewer_ToString`) = c("SWIGFunction", class('ImageViewer_ToString'))

# Start of delete_ImageViewer

`delete_ImageViewer` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ImageViewer', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ImageViewer`, 'returnType') = 'void'
attr(`delete_ImageViewer`, "inputTypes") = c('_p_itk__simple__ImageViewer')
class(`delete_ImageViewer`) = c("SWIGFunction", class('delete_ImageViewer'))

# Start of accessor method for itk::simple::ImageViewer
setMethod('$', '_p_itk__simple__ImageViewer', function(x, name)

{
  accessorFuns = list('GetName' = ImageViewer_GetName, 'SetApplication' = ImageViewer_SetApplication, 'GetApplication' = ImageViewer_GetApplication, 'SetCommand' = ImageViewer_SetCommand, 'GetCommand' = ImageViewer_GetCommand, 'SetFileExtension' = ImageViewer_SetFileExtension, 'GetFileExtension' = ImageViewer_GetFileExtension, 'SetTitle' = ImageViewer_SetTitle, 'GetTitle' = ImageViewer_GetTitle, 'Execute' = ImageViewer_Execute, 'ToString' = ImageViewer_ToString);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ImageViewer
# Start of delete_HashImageFilter

`delete_HashImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_HashImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_HashImageFilter`, 'returnType') = 'void'
attr(`delete_HashImageFilter`, "inputTypes") = c('_p_itk__simple__HashImageFilter')
class(`delete_HashImageFilter`) = c("SWIGFunction", class('delete_HashImageFilter'))

# Start of new_HashImageFilter

`HashImageFilter` = function()
{
  ;ans = .Call('R_swig_new_HashImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__HashImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_HashImageFilter);
  ans
  
}

attr(`HashImageFilter`, 'returnType') = '_p_itk__simple__HashImageFilter'
class(`HashImageFilter`) = c("SWIGFunction", class('HashImageFilter'))

# Start of HashImageFilter_SHA1_get

`HashImageFilter_SHA1_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_HashImageFilter_SHA1_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__HashImageFilter__HashFunction");
  
  ans
  
}

attr(`HashImageFilter_SHA1_get`, 'returnType') = 'character'
class(`HashImageFilter_SHA1_get`) = c("SWIGFunction", class('HashImageFilter_SHA1_get'))

# Start of HashImageFilter_MD5_get

`HashImageFilter_MD5_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_HashImageFilter_MD5_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__HashImageFilter__HashFunction");
  
  ans
  
}

attr(`HashImageFilter_MD5_get`, 'returnType') = 'character'
class(`HashImageFilter_MD5_get`) = c("SWIGFunction", class('HashImageFilter_MD5_get'))

# Start of HashImageFilter_SetHashFunction

`HashImageFilter_SetHashFunction` = function(self, hashFunction)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  hashFunction = enumToInteger(hashFunction, "_itk__simple__HashImageFilter__HashFunction"); 
  
  if(length(hashFunction) > 1) {
    warning("using only the first element of hashFunction");
  };
  
  ;ans = .Call('R_swig_HashImageFilter_SetHashFunction', self, hashFunction, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HashImageFilter_SetHashFunction`, 'returnType') = 'void'
attr(`HashImageFilter_SetHashFunction`, "inputTypes") = c('_p_itk__simple__HashImageFilter', 'character')
class(`HashImageFilter_SetHashFunction`) = c("SWIGFunction", class('HashImageFilter_SetHashFunction'))

# Start of HashImageFilter_GetHashFunction

`HashImageFilter_GetHashFunction` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_HashImageFilter_GetHashFunction', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__HashImageFilter__HashFunction");
  
  ans
  
}

attr(`HashImageFilter_GetHashFunction`, 'returnType') = 'character'
attr(`HashImageFilter_GetHashFunction`, "inputTypes") = c('_p_itk__simple__HashImageFilter')
class(`HashImageFilter_GetHashFunction`) = c("SWIGFunction", class('HashImageFilter_GetHashFunction'))

# Start of HashImageFilter_GetName

`HashImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HashImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HashImageFilter_GetName`, 'returnType') = 'character'
attr(`HashImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__HashImageFilter')
class(`HashImageFilter_GetName`) = c("SWIGFunction", class('HashImageFilter_GetName'))

# Start of HashImageFilter_ToString

`HashImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HashImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HashImageFilter_ToString`, 'returnType') = 'character'
attr(`HashImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__HashImageFilter')
class(`HashImageFilter_ToString`) = c("SWIGFunction", class('HashImageFilter_ToString'))

# Start of HashImageFilter_Execute

`HashImageFilter_Execute` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;.Call('R_swig_HashImageFilter_Execute', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HashImageFilter_Execute`, 'returnType') = 'character'
attr(`HashImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__HashImageFilter', '_p_itk__simple__Image')
class(`HashImageFilter_Execute`) = c("SWIGFunction", class('HashImageFilter_Execute'))

# Start of accessor method for itk::simple::HashImageFilter
setMethod('$', '_p_itk__simple__HashImageFilter', function(x, name)

{
  accessorFuns = list('SetHashFunction' = HashImageFilter_SetHashFunction, 'GetHashFunction' = HashImageFilter_GetHashFunction, 'GetName' = HashImageFilter_GetName, 'ToString' = HashImageFilter_ToString, 'Execute' = HashImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::HashImageFilter
# Start of Hash

`Hash__SWIG_0` = function(image, s_function, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  s_function = enumToInteger(s_function, "_itk__simple__HashImageFilter__HashFunction"); 
  
  if(length(s_function) > 1) {
    warning("using only the first element of s_function");
  };
  
  ;.Call('R_swig_Hash__SWIG_0', image, s_function, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Hash__SWIG_0`, 'returnType') = 'character'
attr(`Hash__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'character')
class(`Hash__SWIG_0`) = c("SWIGFunction", class('Hash__SWIG_0'))

# Start of Hash

`Hash__SWIG_1` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;.Call('R_swig_Hash__SWIG_1', image, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Hash__SWIG_1`, 'returnType') = 'character'
attr(`Hash__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`Hash__SWIG_1`) = c("SWIGFunction", class('Hash__SWIG_1'))

`Hash` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Hash__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Hash__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Hash with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of new_BSplineTransformInitializerFilter

`BSplineTransformInitializerFilter` = function()
{
  ;ans = .Call('R_swig_new_BSplineTransformInitializerFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BSplineTransformInitializerFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BSplineTransformInitializerFilter);
  ans
  
}

attr(`BSplineTransformInitializerFilter`, 'returnType') = '_p_itk__simple__BSplineTransformInitializerFilter'
class(`BSplineTransformInitializerFilter`) = c("SWIGFunction", class('BSplineTransformInitializerFilter'))

# Start of delete_BSplineTransformInitializerFilter

`delete_BSplineTransformInitializerFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BSplineTransformInitializerFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BSplineTransformInitializerFilter`, 'returnType') = 'void'
attr(`delete_BSplineTransformInitializerFilter`, "inputTypes") = c('_p_itk__simple__BSplineTransformInitializerFilter')
class(`delete_BSplineTransformInitializerFilter`) = c("SWIGFunction", class('delete_BSplineTransformInitializerFilter'))

# Start of BSplineTransformInitializerFilter_SetTransformDomainMeshSize

`BSplineTransformInitializerFilter_SetTransformDomainMeshSize` = function(self, TransformDomainMeshSize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  TransformDomainMeshSize = as.integer(TransformDomainMeshSize);
  ;ans = .Call('R_swig_BSplineTransformInitializerFilter_SetTransformDomainMeshSize', self, TransformDomainMeshSize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BSplineTransformInitializerFilter_SetTransformDomainMeshSize`, 'returnType') = 'void'
attr(`BSplineTransformInitializerFilter_SetTransformDomainMeshSize`, "inputTypes") = c('_p_itk__simple__BSplineTransformInitializerFilter', 'integer')
class(`BSplineTransformInitializerFilter_SetTransformDomainMeshSize`) = c("SWIGFunction", class('BSplineTransformInitializerFilter_SetTransformDomainMeshSize'))

# Start of BSplineTransformInitializerFilter_GetTransformDomainMeshSize

`BSplineTransformInitializerFilter_GetTransformDomainMeshSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BSplineTransformInitializerFilter_GetTransformDomainMeshSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BSplineTransformInitializerFilter_GetTransformDomainMeshSize`, 'returnType') = 'integer'
attr(`BSplineTransformInitializerFilter_GetTransformDomainMeshSize`, "inputTypes") = c('_p_itk__simple__BSplineTransformInitializerFilter')
class(`BSplineTransformInitializerFilter_GetTransformDomainMeshSize`) = c("SWIGFunction", class('BSplineTransformInitializerFilter_GetTransformDomainMeshSize'))

# Start of BSplineTransformInitializerFilter_SetOrder

`BSplineTransformInitializerFilter_SetOrder` = function(self, order)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  order = as.integer(order);
  
  if(length(order) > 1) {
    warning("using only the first element of order");
  };
  
  ;ans = .Call('R_swig_BSplineTransformInitializerFilter_SetOrder', self, order, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BSplineTransformInitializerFilter_SetOrder`, 'returnType') = 'void'
attr(`BSplineTransformInitializerFilter_SetOrder`, "inputTypes") = c('_p_itk__simple__BSplineTransformInitializerFilter', 'integer')
class(`BSplineTransformInitializerFilter_SetOrder`) = c("SWIGFunction", class('BSplineTransformInitializerFilter_SetOrder'))

# Start of BSplineTransformInitializerFilter_GetOrder

`BSplineTransformInitializerFilter_GetOrder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BSplineTransformInitializerFilter_GetOrder', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BSplineTransformInitializerFilter_GetOrder`, 'returnType') = 'integer'
attr(`BSplineTransformInitializerFilter_GetOrder`, "inputTypes") = c('_p_itk__simple__BSplineTransformInitializerFilter')
class(`BSplineTransformInitializerFilter_GetOrder`) = c("SWIGFunction", class('BSplineTransformInitializerFilter_GetOrder'))

# Start of BSplineTransformInitializerFilter_GetName

`BSplineTransformInitializerFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BSplineTransformInitializerFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BSplineTransformInitializerFilter_GetName`, 'returnType') = 'character'
attr(`BSplineTransformInitializerFilter_GetName`, "inputTypes") = c('_p_itk__simple__BSplineTransformInitializerFilter')
class(`BSplineTransformInitializerFilter_GetName`) = c("SWIGFunction", class('BSplineTransformInitializerFilter_GetName'))

# Start of BSplineTransformInitializerFilter_ToString

`BSplineTransformInitializerFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BSplineTransformInitializerFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BSplineTransformInitializerFilter_ToString`, 'returnType') = 'character'
attr(`BSplineTransformInitializerFilter_ToString`, "inputTypes") = c('_p_itk__simple__BSplineTransformInitializerFilter')
class(`BSplineTransformInitializerFilter_ToString`) = c("SWIGFunction", class('BSplineTransformInitializerFilter_ToString'))

# Start of BSplineTransformInitializerFilter_Execute

`BSplineTransformInitializerFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BSplineTransformInitializerFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BSplineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BSplineTransformInitializerFilter_Execute`, 'returnType') = '_p_itk__simple__BSplineTransform'
attr(`BSplineTransformInitializerFilter_Execute`, "inputTypes") = c('_p_itk__simple__BSplineTransformInitializerFilter', '_p_itk__simple__Image')
class(`BSplineTransformInitializerFilter_Execute`) = c("SWIGFunction", class('BSplineTransformInitializerFilter_Execute'))

# Start of accessor method for itk::simple::BSplineTransformInitializerFilter
setMethod('$', '_p_itk__simple__BSplineTransformInitializerFilter', function(x, name)

{
  accessorFuns = list('SetTransformDomainMeshSize' = BSplineTransformInitializerFilter_SetTransformDomainMeshSize, 'GetTransformDomainMeshSize' = BSplineTransformInitializerFilter_GetTransformDomainMeshSize, 'SetOrder' = BSplineTransformInitializerFilter_SetOrder, 'GetOrder' = BSplineTransformInitializerFilter_GetOrder, 'GetName' = BSplineTransformInitializerFilter_GetName, 'ToString' = BSplineTransformInitializerFilter_ToString, 'Execute' = BSplineTransformInitializerFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BSplineTransformInitializerFilter
# Start of BSplineTransformInitializer

`BSplineTransformInitializer__SWIG_0` = function(image1, transformDomainMeshSize, order, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  transformDomainMeshSize = as.integer(transformDomainMeshSize);
  order = as.integer(order);
  
  if(length(order) > 1) {
    warning("using only the first element of order");
  };
  
  ;ans = .Call('R_swig_BSplineTransformInitializer__SWIG_0', image1, transformDomainMeshSize, order, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BSplineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BSplineTransformInitializer__SWIG_0`, 'returnType') = '_p_itk__simple__BSplineTransform'
attr(`BSplineTransformInitializer__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`BSplineTransformInitializer__SWIG_0`) = c("SWIGFunction", class('BSplineTransformInitializer__SWIG_0'))

# Start of BSplineTransformInitializer

`BSplineTransformInitializer__SWIG_1` = function(image1, transformDomainMeshSize, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  transformDomainMeshSize = as.integer(transformDomainMeshSize);
  ;ans = .Call('R_swig_BSplineTransformInitializer__SWIG_1', image1, transformDomainMeshSize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BSplineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BSplineTransformInitializer__SWIG_1`, 'returnType') = '_p_itk__simple__BSplineTransform'
attr(`BSplineTransformInitializer__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BSplineTransformInitializer__SWIG_1`) = c("SWIGFunction", class('BSplineTransformInitializer__SWIG_1'))

# Start of BSplineTransformInitializer

`BSplineTransformInitializer__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BSplineTransformInitializer__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BSplineTransform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BSplineTransformInitializer__SWIG_2`, 'returnType') = '_p_itk__simple__BSplineTransform'
attr(`BSplineTransformInitializer__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`BSplineTransformInitializer__SWIG_2`) = c("SWIGFunction", class('BSplineTransformInitializer__SWIG_2'))

`BSplineTransformInitializer` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BSplineTransformInitializer__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BSplineTransformInitializer__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- BSplineTransformInitializer__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BSplineTransformInitializer with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_CenteredTransformInitializerFilter

`delete_CenteredTransformInitializerFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_CenteredTransformInitializerFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_CenteredTransformInitializerFilter`, 'returnType') = 'void'
attr(`delete_CenteredTransformInitializerFilter`, "inputTypes") = c('_p_itk__simple__CenteredTransformInitializerFilter')
class(`delete_CenteredTransformInitializerFilter`) = c("SWIGFunction", class('delete_CenteredTransformInitializerFilter'))

# Start of new_CenteredTransformInitializerFilter

`CenteredTransformInitializerFilter` = function()
{
  ;ans = .Call('R_swig_new_CenteredTransformInitializerFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CenteredTransformInitializerFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CenteredTransformInitializerFilter);
  ans
  
}

attr(`CenteredTransformInitializerFilter`, 'returnType') = '_p_itk__simple__CenteredTransformInitializerFilter'
class(`CenteredTransformInitializerFilter`) = c("SWIGFunction", class('CenteredTransformInitializerFilter'))

# Start of CenteredTransformInitializerFilter_GEOMETRY_get

`CenteredTransformInitializerFilter_GEOMETRY_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_CenteredTransformInitializerFilter_GEOMETRY_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__CenteredTransformInitializerFilter__OperationModeType");
  
  ans
  
}

attr(`CenteredTransformInitializerFilter_GEOMETRY_get`, 'returnType') = 'character'
class(`CenteredTransformInitializerFilter_GEOMETRY_get`) = c("SWIGFunction", class('CenteredTransformInitializerFilter_GEOMETRY_get'))

# Start of CenteredTransformInitializerFilter_MOMENTS_get

`CenteredTransformInitializerFilter_MOMENTS_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_CenteredTransformInitializerFilter_MOMENTS_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__CenteredTransformInitializerFilter__OperationModeType");
  
  ans
  
}

attr(`CenteredTransformInitializerFilter_MOMENTS_get`, 'returnType') = 'character'
class(`CenteredTransformInitializerFilter_MOMENTS_get`) = c("SWIGFunction", class('CenteredTransformInitializerFilter_MOMENTS_get'))

# Start of CenteredTransformInitializerFilter_SetOperationMode

`CenteredTransformInitializerFilter_SetOperationMode` = function(self, OperationMode)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OperationMode = enumToInteger(OperationMode, "_itk__simple__CenteredTransformInitializerFilter__OperationModeType"); 
  
  if(length(OperationMode) > 1) {
    warning("using only the first element of OperationMode");
  };
  
  ;ans = .Call('R_swig_CenteredTransformInitializerFilter_SetOperationMode', self, OperationMode, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CenteredTransformInitializerFilter_SetOperationMode`, 'returnType') = 'void'
attr(`CenteredTransformInitializerFilter_SetOperationMode`, "inputTypes") = c('_p_itk__simple__CenteredTransformInitializerFilter', 'character')
class(`CenteredTransformInitializerFilter_SetOperationMode`) = c("SWIGFunction", class('CenteredTransformInitializerFilter_SetOperationMode'))

# Start of CenteredTransformInitializerFilter_GetOperationMode

`CenteredTransformInitializerFilter_GetOperationMode` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CenteredTransformInitializerFilter_GetOperationMode', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__CenteredTransformInitializerFilter__OperationModeType");
  
  ans
  
}

attr(`CenteredTransformInitializerFilter_GetOperationMode`, 'returnType') = 'character'
attr(`CenteredTransformInitializerFilter_GetOperationMode`, "inputTypes") = c('_p_itk__simple__CenteredTransformInitializerFilter')
class(`CenteredTransformInitializerFilter_GetOperationMode`) = c("SWIGFunction", class('CenteredTransformInitializerFilter_GetOperationMode'))

# Start of CenteredTransformInitializerFilter_GetName

`CenteredTransformInitializerFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CenteredTransformInitializerFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CenteredTransformInitializerFilter_GetName`, 'returnType') = 'character'
attr(`CenteredTransformInitializerFilter_GetName`, "inputTypes") = c('_p_itk__simple__CenteredTransformInitializerFilter')
class(`CenteredTransformInitializerFilter_GetName`) = c("SWIGFunction", class('CenteredTransformInitializerFilter_GetName'))

# Start of CenteredTransformInitializerFilter_ToString

`CenteredTransformInitializerFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CenteredTransformInitializerFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CenteredTransformInitializerFilter_ToString`, 'returnType') = 'character'
attr(`CenteredTransformInitializerFilter_ToString`, "inputTypes") = c('_p_itk__simple__CenteredTransformInitializerFilter')
class(`CenteredTransformInitializerFilter_ToString`) = c("SWIGFunction", class('CenteredTransformInitializerFilter_ToString'))

# Start of CenteredTransformInitializerFilter_Execute

`CenteredTransformInitializerFilter_Execute` = function(self, fixedImage, movingImage, transform, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_CenteredTransformInitializerFilter_Execute', self, fixedImage, movingImage, transform, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CenteredTransformInitializerFilter_Execute`, 'returnType') = '_p_itk__simple__Transform'
attr(`CenteredTransformInitializerFilter_Execute`, "inputTypes") = c('_p_itk__simple__CenteredTransformInitializerFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Transform')
class(`CenteredTransformInitializerFilter_Execute`) = c("SWIGFunction", class('CenteredTransformInitializerFilter_Execute'))

# Start of CenteredTransformInitializerFilter_MomentsOn

`CenteredTransformInitializerFilter_MomentsOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CenteredTransformInitializerFilter_MomentsOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CenteredTransformInitializerFilter_MomentsOn`, 'returnType') = 'void'
attr(`CenteredTransformInitializerFilter_MomentsOn`, "inputTypes") = c('_p_itk__simple__CenteredTransformInitializerFilter')
class(`CenteredTransformInitializerFilter_MomentsOn`) = c("SWIGFunction", class('CenteredTransformInitializerFilter_MomentsOn'))

# Start of CenteredTransformInitializerFilter_GeometryOn

`CenteredTransformInitializerFilter_GeometryOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CenteredTransformInitializerFilter_GeometryOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CenteredTransformInitializerFilter_GeometryOn`, 'returnType') = 'void'
attr(`CenteredTransformInitializerFilter_GeometryOn`, "inputTypes") = c('_p_itk__simple__CenteredTransformInitializerFilter')
class(`CenteredTransformInitializerFilter_GeometryOn`) = c("SWIGFunction", class('CenteredTransformInitializerFilter_GeometryOn'))

# Start of accessor method for itk::simple::CenteredTransformInitializerFilter
setMethod('$', '_p_itk__simple__CenteredTransformInitializerFilter', function(x, name)

{
  accessorFuns = list('SetOperationMode' = CenteredTransformInitializerFilter_SetOperationMode, 'GetOperationMode' = CenteredTransformInitializerFilter_GetOperationMode, 'GetName' = CenteredTransformInitializerFilter_GetName, 'ToString' = CenteredTransformInitializerFilter_ToString, 'Execute' = CenteredTransformInitializerFilter_Execute, 'MomentsOn' = CenteredTransformInitializerFilter_MomentsOn, 'GeometryOn' = CenteredTransformInitializerFilter_GeometryOn);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::CenteredTransformInitializerFilter
# Start of CenteredTransformInitializer

`CenteredTransformInitializer__SWIG_0` = function(fixedImage, movingImage, transform, operationMode, .copy = FALSE)
{
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  operationMode = enumToInteger(operationMode, "_itk__simple__CenteredTransformInitializerFilter__OperationModeType"); 
  
  if(length(operationMode) > 1) {
    warning("using only the first element of operationMode");
  };
  
  ;ans = .Call('R_swig_CenteredTransformInitializer__SWIG_0', fixedImage, movingImage, transform, operationMode, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CenteredTransformInitializer__SWIG_0`, 'returnType') = '_p_itk__simple__Transform'
attr(`CenteredTransformInitializer__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Transform', 'character')
class(`CenteredTransformInitializer__SWIG_0`) = c("SWIGFunction", class('CenteredTransformInitializer__SWIG_0'))

# Start of CenteredTransformInitializer

`CenteredTransformInitializer__SWIG_1` = function(fixedImage, movingImage, transform, .copy = FALSE)
{
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_CenteredTransformInitializer__SWIG_1', fixedImage, movingImage, transform, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CenteredTransformInitializer__SWIG_1`, 'returnType') = '_p_itk__simple__Transform'
attr(`CenteredTransformInitializer__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Transform')
class(`CenteredTransformInitializer__SWIG_1`) = c("SWIGFunction", class('CenteredTransformInitializer__SWIG_1'))

`CenteredTransformInitializer` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 )) {
      f <- CenteredTransformInitializer__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- CenteredTransformInitializer__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for CenteredTransformInitializer with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_CenteredVersorTransformInitializerFilter

`delete_CenteredVersorTransformInitializerFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_CenteredVersorTransformInitializerFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_CenteredVersorTransformInitializerFilter`, 'returnType') = 'void'
attr(`delete_CenteredVersorTransformInitializerFilter`, "inputTypes") = c('_p_itk__simple__CenteredVersorTransformInitializerFilter')
class(`delete_CenteredVersorTransformInitializerFilter`) = c("SWIGFunction", class('delete_CenteredVersorTransformInitializerFilter'))

# Start of new_CenteredVersorTransformInitializerFilter

`CenteredVersorTransformInitializerFilter` = function()
{
  ;ans = .Call('R_swig_new_CenteredVersorTransformInitializerFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CenteredVersorTransformInitializerFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CenteredVersorTransformInitializerFilter);
  ans
  
}

attr(`CenteredVersorTransformInitializerFilter`, 'returnType') = '_p_itk__simple__CenteredVersorTransformInitializerFilter'
class(`CenteredVersorTransformInitializerFilter`) = c("SWIGFunction", class('CenteredVersorTransformInitializerFilter'))

# Start of CenteredVersorTransformInitializerFilter_SetComputeRotation

`CenteredVersorTransformInitializerFilter_SetComputeRotation` = function(self, ComputeRotation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ComputeRotation = as.logical(ComputeRotation);
  ;ans = .Call('R_swig_CenteredVersorTransformInitializerFilter_SetComputeRotation', self, ComputeRotation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CenteredVersorTransformInitializerFilter_SetComputeRotation`, 'returnType') = 'void'
attr(`CenteredVersorTransformInitializerFilter_SetComputeRotation`, "inputTypes") = c('_p_itk__simple__CenteredVersorTransformInitializerFilter', 'logical')
class(`CenteredVersorTransformInitializerFilter_SetComputeRotation`) = c("SWIGFunction", class('CenteredVersorTransformInitializerFilter_SetComputeRotation'))

# Start of CenteredVersorTransformInitializerFilter_ComputeRotationOn

`CenteredVersorTransformInitializerFilter_ComputeRotationOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CenteredVersorTransformInitializerFilter_ComputeRotationOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CenteredVersorTransformInitializerFilter_ComputeRotationOn`, 'returnType') = 'void'
attr(`CenteredVersorTransformInitializerFilter_ComputeRotationOn`, "inputTypes") = c('_p_itk__simple__CenteredVersorTransformInitializerFilter')
class(`CenteredVersorTransformInitializerFilter_ComputeRotationOn`) = c("SWIGFunction", class('CenteredVersorTransformInitializerFilter_ComputeRotationOn'))

# Start of CenteredVersorTransformInitializerFilter_ComputeRotationOff

`CenteredVersorTransformInitializerFilter_ComputeRotationOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CenteredVersorTransformInitializerFilter_ComputeRotationOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CenteredVersorTransformInitializerFilter_ComputeRotationOff`, 'returnType') = 'void'
attr(`CenteredVersorTransformInitializerFilter_ComputeRotationOff`, "inputTypes") = c('_p_itk__simple__CenteredVersorTransformInitializerFilter')
class(`CenteredVersorTransformInitializerFilter_ComputeRotationOff`) = c("SWIGFunction", class('CenteredVersorTransformInitializerFilter_ComputeRotationOff'))

# Start of CenteredVersorTransformInitializerFilter_GetComputeRotation

`CenteredVersorTransformInitializerFilter_GetComputeRotation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CenteredVersorTransformInitializerFilter_GetComputeRotation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CenteredVersorTransformInitializerFilter_GetComputeRotation`, 'returnType') = 'logical'
attr(`CenteredVersorTransformInitializerFilter_GetComputeRotation`, "inputTypes") = c('_p_itk__simple__CenteredVersorTransformInitializerFilter')
class(`CenteredVersorTransformInitializerFilter_GetComputeRotation`) = c("SWIGFunction", class('CenteredVersorTransformInitializerFilter_GetComputeRotation'))

# Start of CenteredVersorTransformInitializerFilter_GetName

`CenteredVersorTransformInitializerFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CenteredVersorTransformInitializerFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CenteredVersorTransformInitializerFilter_GetName`, 'returnType') = 'character'
attr(`CenteredVersorTransformInitializerFilter_GetName`, "inputTypes") = c('_p_itk__simple__CenteredVersorTransformInitializerFilter')
class(`CenteredVersorTransformInitializerFilter_GetName`) = c("SWIGFunction", class('CenteredVersorTransformInitializerFilter_GetName'))

# Start of CenteredVersorTransformInitializerFilter_ToString

`CenteredVersorTransformInitializerFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CenteredVersorTransformInitializerFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CenteredVersorTransformInitializerFilter_ToString`, 'returnType') = 'character'
attr(`CenteredVersorTransformInitializerFilter_ToString`, "inputTypes") = c('_p_itk__simple__CenteredVersorTransformInitializerFilter')
class(`CenteredVersorTransformInitializerFilter_ToString`) = c("SWIGFunction", class('CenteredVersorTransformInitializerFilter_ToString'))

# Start of CenteredVersorTransformInitializerFilter_Execute

`CenteredVersorTransformInitializerFilter_Execute` = function(self, fixedImage, movingImage, transform, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_CenteredVersorTransformInitializerFilter_Execute', self, fixedImage, movingImage, transform, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CenteredVersorTransformInitializerFilter_Execute`, 'returnType') = '_p_itk__simple__Transform'
attr(`CenteredVersorTransformInitializerFilter_Execute`, "inputTypes") = c('_p_itk__simple__CenteredVersorTransformInitializerFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Transform')
class(`CenteredVersorTransformInitializerFilter_Execute`) = c("SWIGFunction", class('CenteredVersorTransformInitializerFilter_Execute'))

# Start of accessor method for itk::simple::CenteredVersorTransformInitializerFilter
setMethod('$', '_p_itk__simple__CenteredVersorTransformInitializerFilter', function(x, name)

{
  accessorFuns = list('SetComputeRotation' = CenteredVersorTransformInitializerFilter_SetComputeRotation, 'ComputeRotationOn' = CenteredVersorTransformInitializerFilter_ComputeRotationOn, 'ComputeRotationOff' = CenteredVersorTransformInitializerFilter_ComputeRotationOff, 'GetComputeRotation' = CenteredVersorTransformInitializerFilter_GetComputeRotation, 'GetName' = CenteredVersorTransformInitializerFilter_GetName, 'ToString' = CenteredVersorTransformInitializerFilter_ToString, 'Execute' = CenteredVersorTransformInitializerFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::CenteredVersorTransformInitializerFilter
# Start of CenteredVersorTransformInitializer

`CenteredVersorTransformInitializer__SWIG_0` = function(fixedImage, movingImage, transform, computeRotation, .copy = FALSE)
{
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  computeRotation = as.logical(computeRotation);
  ;ans = .Call('R_swig_CenteredVersorTransformInitializer__SWIG_0', fixedImage, movingImage, transform, computeRotation, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CenteredVersorTransformInitializer__SWIG_0`, 'returnType') = '_p_itk__simple__Transform'
attr(`CenteredVersorTransformInitializer__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Transform', 'logical')
class(`CenteredVersorTransformInitializer__SWIG_0`) = c("SWIGFunction", class('CenteredVersorTransformInitializer__SWIG_0'))

# Start of CenteredVersorTransformInitializer

`CenteredVersorTransformInitializer__SWIG_1` = function(fixedImage, movingImage, transform, .copy = FALSE)
{
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_CenteredVersorTransformInitializer__SWIG_1', fixedImage, movingImage, transform, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CenteredVersorTransformInitializer__SWIG_1`, 'returnType') = '_p_itk__simple__Transform'
attr(`CenteredVersorTransformInitializer__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Transform')
class(`CenteredVersorTransformInitializer__SWIG_1`) = c("SWIGFunction", class('CenteredVersorTransformInitializer__SWIG_1'))

`CenteredVersorTransformInitializer` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 )) {
      f <- CenteredVersorTransformInitializer__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- CenteredVersorTransformInitializer__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for CenteredVersorTransformInitializer with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LandmarkBasedTransformInitializerFilter

`delete_LandmarkBasedTransformInitializerFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LandmarkBasedTransformInitializerFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LandmarkBasedTransformInitializerFilter`, 'returnType') = 'void'
attr(`delete_LandmarkBasedTransformInitializerFilter`, "inputTypes") = c('_p_itk__simple__LandmarkBasedTransformInitializerFilter')
class(`delete_LandmarkBasedTransformInitializerFilter`) = c("SWIGFunction", class('delete_LandmarkBasedTransformInitializerFilter'))

# Start of new_LandmarkBasedTransformInitializerFilter

`LandmarkBasedTransformInitializerFilter` = function()
{
  ;ans = .Call('R_swig_new_LandmarkBasedTransformInitializerFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LandmarkBasedTransformInitializerFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LandmarkBasedTransformInitializerFilter);
  ans
  
}

attr(`LandmarkBasedTransformInitializerFilter`, 'returnType') = '_p_itk__simple__LandmarkBasedTransformInitializerFilter'
class(`LandmarkBasedTransformInitializerFilter`) = c("SWIGFunction", class('LandmarkBasedTransformInitializerFilter'))

# Start of LandmarkBasedTransformInitializerFilter_SetFixedLandmarks

`LandmarkBasedTransformInitializerFilter_SetFixedLandmarks` = function(self, FixedLandmarks)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FixedLandmarks = as.numeric(FixedLandmarks);
  ;ans = .Call('R_swig_LandmarkBasedTransformInitializerFilter_SetFixedLandmarks', self, FixedLandmarks, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LandmarkBasedTransformInitializerFilter_SetFixedLandmarks`, 'returnType') = 'void'
attr(`LandmarkBasedTransformInitializerFilter_SetFixedLandmarks`, "inputTypes") = c('_p_itk__simple__LandmarkBasedTransformInitializerFilter', '_p_std__vectorT_double_t')
class(`LandmarkBasedTransformInitializerFilter_SetFixedLandmarks`) = c("SWIGFunction", class('LandmarkBasedTransformInitializerFilter_SetFixedLandmarks'))

# Start of LandmarkBasedTransformInitializerFilter_GetFixedLandmarks

`LandmarkBasedTransformInitializerFilter_GetFixedLandmarks` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LandmarkBasedTransformInitializerFilter_GetFixedLandmarks', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LandmarkBasedTransformInitializerFilter_GetFixedLandmarks`, 'returnType') = 'numeric'
attr(`LandmarkBasedTransformInitializerFilter_GetFixedLandmarks`, "inputTypes") = c('_p_itk__simple__LandmarkBasedTransformInitializerFilter')
class(`LandmarkBasedTransformInitializerFilter_GetFixedLandmarks`) = c("SWIGFunction", class('LandmarkBasedTransformInitializerFilter_GetFixedLandmarks'))

# Start of LandmarkBasedTransformInitializerFilter_SetMovingLandmarks

`LandmarkBasedTransformInitializerFilter_SetMovingLandmarks` = function(self, MovingLandmarks)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MovingLandmarks = as.numeric(MovingLandmarks);
  ;ans = .Call('R_swig_LandmarkBasedTransformInitializerFilter_SetMovingLandmarks', self, MovingLandmarks, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LandmarkBasedTransformInitializerFilter_SetMovingLandmarks`, 'returnType') = 'void'
attr(`LandmarkBasedTransformInitializerFilter_SetMovingLandmarks`, "inputTypes") = c('_p_itk__simple__LandmarkBasedTransformInitializerFilter', '_p_std__vectorT_double_t')
class(`LandmarkBasedTransformInitializerFilter_SetMovingLandmarks`) = c("SWIGFunction", class('LandmarkBasedTransformInitializerFilter_SetMovingLandmarks'))

# Start of LandmarkBasedTransformInitializerFilter_GetMovingLandmarks

`LandmarkBasedTransformInitializerFilter_GetMovingLandmarks` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LandmarkBasedTransformInitializerFilter_GetMovingLandmarks', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LandmarkBasedTransformInitializerFilter_GetMovingLandmarks`, 'returnType') = 'numeric'
attr(`LandmarkBasedTransformInitializerFilter_GetMovingLandmarks`, "inputTypes") = c('_p_itk__simple__LandmarkBasedTransformInitializerFilter')
class(`LandmarkBasedTransformInitializerFilter_GetMovingLandmarks`) = c("SWIGFunction", class('LandmarkBasedTransformInitializerFilter_GetMovingLandmarks'))

# Start of LandmarkBasedTransformInitializerFilter_SetLandmarkWeight

`LandmarkBasedTransformInitializerFilter_SetLandmarkWeight` = function(self, LandmarkWeight)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  LandmarkWeight = as.numeric(LandmarkWeight);
  ;ans = .Call('R_swig_LandmarkBasedTransformInitializerFilter_SetLandmarkWeight', self, LandmarkWeight, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LandmarkBasedTransformInitializerFilter_SetLandmarkWeight`, 'returnType') = 'void'
attr(`LandmarkBasedTransformInitializerFilter_SetLandmarkWeight`, "inputTypes") = c('_p_itk__simple__LandmarkBasedTransformInitializerFilter', '_p_std__vectorT_double_t')
class(`LandmarkBasedTransformInitializerFilter_SetLandmarkWeight`) = c("SWIGFunction", class('LandmarkBasedTransformInitializerFilter_SetLandmarkWeight'))

# Start of LandmarkBasedTransformInitializerFilter_GetLandmarkWeight

`LandmarkBasedTransformInitializerFilter_GetLandmarkWeight` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LandmarkBasedTransformInitializerFilter_GetLandmarkWeight', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LandmarkBasedTransformInitializerFilter_GetLandmarkWeight`, 'returnType') = 'numeric'
attr(`LandmarkBasedTransformInitializerFilter_GetLandmarkWeight`, "inputTypes") = c('_p_itk__simple__LandmarkBasedTransformInitializerFilter')
class(`LandmarkBasedTransformInitializerFilter_GetLandmarkWeight`) = c("SWIGFunction", class('LandmarkBasedTransformInitializerFilter_GetLandmarkWeight'))

# Start of LandmarkBasedTransformInitializerFilter_SetReferenceImage

`LandmarkBasedTransformInitializerFilter_SetReferenceImage` = function(self, ReferenceImage)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(ReferenceImage, "ExternalReference")) ReferenceImage = slot(ReferenceImage,"ref"); 
  ;ans = .Call('R_swig_LandmarkBasedTransformInitializerFilter_SetReferenceImage', self, ReferenceImage, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LandmarkBasedTransformInitializerFilter_SetReferenceImage`, 'returnType') = 'void'
attr(`LandmarkBasedTransformInitializerFilter_SetReferenceImage`, "inputTypes") = c('_p_itk__simple__LandmarkBasedTransformInitializerFilter', '_p_itk__simple__Image')
class(`LandmarkBasedTransformInitializerFilter_SetReferenceImage`) = c("SWIGFunction", class('LandmarkBasedTransformInitializerFilter_SetReferenceImage'))

# Start of LandmarkBasedTransformInitializerFilter_GetReferenceImage

`LandmarkBasedTransformInitializerFilter_GetReferenceImage` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LandmarkBasedTransformInitializerFilter_GetReferenceImage', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LandmarkBasedTransformInitializerFilter_GetReferenceImage`, 'returnType') = '_p_itk__simple__Image'
attr(`LandmarkBasedTransformInitializerFilter_GetReferenceImage`, "inputTypes") = c('_p_itk__simple__LandmarkBasedTransformInitializerFilter')
class(`LandmarkBasedTransformInitializerFilter_GetReferenceImage`) = c("SWIGFunction", class('LandmarkBasedTransformInitializerFilter_GetReferenceImage'))

# Start of LandmarkBasedTransformInitializerFilter_SetBSplineNumberOfControlPoints

`LandmarkBasedTransformInitializerFilter_SetBSplineNumberOfControlPoints` = function(self, BSplineNumberOfControlPoints)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BSplineNumberOfControlPoints = as.integer(BSplineNumberOfControlPoints);
  
  if(length(BSplineNumberOfControlPoints) > 1) {
    warning("using only the first element of BSplineNumberOfControlPoints");
  };
  
  ;ans = .Call('R_swig_LandmarkBasedTransformInitializerFilter_SetBSplineNumberOfControlPoints', self, BSplineNumberOfControlPoints, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LandmarkBasedTransformInitializerFilter_SetBSplineNumberOfControlPoints`, 'returnType') = 'void'
attr(`LandmarkBasedTransformInitializerFilter_SetBSplineNumberOfControlPoints`, "inputTypes") = c('_p_itk__simple__LandmarkBasedTransformInitializerFilter', 'integer')
class(`LandmarkBasedTransformInitializerFilter_SetBSplineNumberOfControlPoints`) = c("SWIGFunction", class('LandmarkBasedTransformInitializerFilter_SetBSplineNumberOfControlPoints'))

# Start of LandmarkBasedTransformInitializerFilter_GetBSplineNumberOfControlPoints

`LandmarkBasedTransformInitializerFilter_GetBSplineNumberOfControlPoints` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LandmarkBasedTransformInitializerFilter_GetBSplineNumberOfControlPoints', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LandmarkBasedTransformInitializerFilter_GetBSplineNumberOfControlPoints`, 'returnType') = 'integer'
attr(`LandmarkBasedTransformInitializerFilter_GetBSplineNumberOfControlPoints`, "inputTypes") = c('_p_itk__simple__LandmarkBasedTransformInitializerFilter')
class(`LandmarkBasedTransformInitializerFilter_GetBSplineNumberOfControlPoints`) = c("SWIGFunction", class('LandmarkBasedTransformInitializerFilter_GetBSplineNumberOfControlPoints'))

# Start of LandmarkBasedTransformInitializerFilter_GetName

`LandmarkBasedTransformInitializerFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LandmarkBasedTransformInitializerFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LandmarkBasedTransformInitializerFilter_GetName`, 'returnType') = 'character'
attr(`LandmarkBasedTransformInitializerFilter_GetName`, "inputTypes") = c('_p_itk__simple__LandmarkBasedTransformInitializerFilter')
class(`LandmarkBasedTransformInitializerFilter_GetName`) = c("SWIGFunction", class('LandmarkBasedTransformInitializerFilter_GetName'))

# Start of LandmarkBasedTransformInitializerFilter_ToString

`LandmarkBasedTransformInitializerFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LandmarkBasedTransformInitializerFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LandmarkBasedTransformInitializerFilter_ToString`, 'returnType') = 'character'
attr(`LandmarkBasedTransformInitializerFilter_ToString`, "inputTypes") = c('_p_itk__simple__LandmarkBasedTransformInitializerFilter')
class(`LandmarkBasedTransformInitializerFilter_ToString`) = c("SWIGFunction", class('LandmarkBasedTransformInitializerFilter_ToString'))

# Start of LandmarkBasedTransformInitializerFilter_Execute

`LandmarkBasedTransformInitializerFilter_Execute` = function(self, transform, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_LandmarkBasedTransformInitializerFilter_Execute', self, transform, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LandmarkBasedTransformInitializerFilter_Execute`, 'returnType') = '_p_itk__simple__Transform'
attr(`LandmarkBasedTransformInitializerFilter_Execute`, "inputTypes") = c('_p_itk__simple__LandmarkBasedTransformInitializerFilter', '_p_itk__simple__Transform')
class(`LandmarkBasedTransformInitializerFilter_Execute`) = c("SWIGFunction", class('LandmarkBasedTransformInitializerFilter_Execute'))

# Start of accessor method for itk::simple::LandmarkBasedTransformInitializerFilter
setMethod('$', '_p_itk__simple__LandmarkBasedTransformInitializerFilter', function(x, name)

{
  accessorFuns = list('SetFixedLandmarks' = LandmarkBasedTransformInitializerFilter_SetFixedLandmarks, 'GetFixedLandmarks' = LandmarkBasedTransformInitializerFilter_GetFixedLandmarks, 'SetMovingLandmarks' = LandmarkBasedTransformInitializerFilter_SetMovingLandmarks, 'GetMovingLandmarks' = LandmarkBasedTransformInitializerFilter_GetMovingLandmarks, 'SetLandmarkWeight' = LandmarkBasedTransformInitializerFilter_SetLandmarkWeight, 'GetLandmarkWeight' = LandmarkBasedTransformInitializerFilter_GetLandmarkWeight, 'SetReferenceImage' = LandmarkBasedTransformInitializerFilter_SetReferenceImage, 'GetReferenceImage' = LandmarkBasedTransformInitializerFilter_GetReferenceImage, 'SetBSplineNumberOfControlPoints' = LandmarkBasedTransformInitializerFilter_SetBSplineNumberOfControlPoints, 'GetBSplineNumberOfControlPoints' = LandmarkBasedTransformInitializerFilter_GetBSplineNumberOfControlPoints, 'GetName' = LandmarkBasedTransformInitializerFilter_GetName, 'ToString' = LandmarkBasedTransformInitializerFilter_ToString, 'Execute' = LandmarkBasedTransformInitializerFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LandmarkBasedTransformInitializerFilter
# Start of LandmarkBasedTransformInitializer

`LandmarkBasedTransformInitializer__SWIG_0` = function(transform, fixedLandmarks, movingLandmarks, landmarkWeight, referenceImage, numberOfControlPoints, .copy = FALSE)
{
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  fixedLandmarks = as.numeric(fixedLandmarks);
  movingLandmarks = as.numeric(movingLandmarks);
  landmarkWeight = as.numeric(landmarkWeight);
  if (inherits(referenceImage, "ExternalReference")) referenceImage = slot(referenceImage,"ref"); 
  numberOfControlPoints = as.integer(numberOfControlPoints);
  
  if(length(numberOfControlPoints) > 1) {
    warning("using only the first element of numberOfControlPoints");
  };
  
  ;ans = .Call('R_swig_LandmarkBasedTransformInitializer__SWIG_0', transform, fixedLandmarks, movingLandmarks, landmarkWeight, referenceImage, numberOfControlPoints, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LandmarkBasedTransformInitializer__SWIG_0`, 'returnType') = '_p_itk__simple__Transform'
attr(`LandmarkBasedTransformInitializer__SWIG_0`, "inputTypes") = c('_p_itk__simple__Transform', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_itk__simple__Image', 'integer')
class(`LandmarkBasedTransformInitializer__SWIG_0`) = c("SWIGFunction", class('LandmarkBasedTransformInitializer__SWIG_0'))

# Start of LandmarkBasedTransformInitializer

`LandmarkBasedTransformInitializer__SWIG_1` = function(transform, fixedLandmarks, movingLandmarks, landmarkWeight, referenceImage, .copy = FALSE)
{
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  fixedLandmarks = as.numeric(fixedLandmarks);
  movingLandmarks = as.numeric(movingLandmarks);
  landmarkWeight = as.numeric(landmarkWeight);
  if (inherits(referenceImage, "ExternalReference")) referenceImage = slot(referenceImage,"ref"); 
  ;ans = .Call('R_swig_LandmarkBasedTransformInitializer__SWIG_1', transform, fixedLandmarks, movingLandmarks, landmarkWeight, referenceImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LandmarkBasedTransformInitializer__SWIG_1`, 'returnType') = '_p_itk__simple__Transform'
attr(`LandmarkBasedTransformInitializer__SWIG_1`, "inputTypes") = c('_p_itk__simple__Transform', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_itk__simple__Image')
class(`LandmarkBasedTransformInitializer__SWIG_1`) = c("SWIGFunction", class('LandmarkBasedTransformInitializer__SWIG_1'))

# Start of LandmarkBasedTransformInitializer

`LandmarkBasedTransformInitializer__SWIG_2` = function(transform, fixedLandmarks, movingLandmarks, landmarkWeight, .copy = FALSE)
{
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  fixedLandmarks = as.numeric(fixedLandmarks);
  movingLandmarks = as.numeric(movingLandmarks);
  landmarkWeight = as.numeric(landmarkWeight);
  ;ans = .Call('R_swig_LandmarkBasedTransformInitializer__SWIG_2', transform, fixedLandmarks, movingLandmarks, landmarkWeight, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LandmarkBasedTransformInitializer__SWIG_2`, 'returnType') = '_p_itk__simple__Transform'
attr(`LandmarkBasedTransformInitializer__SWIG_2`, "inputTypes") = c('_p_itk__simple__Transform', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`LandmarkBasedTransformInitializer__SWIG_2`) = c("SWIGFunction", class('LandmarkBasedTransformInitializer__SWIG_2'))

# Start of LandmarkBasedTransformInitializer

`LandmarkBasedTransformInitializer__SWIG_3` = function(transform, fixedLandmarks, movingLandmarks, .copy = FALSE)
{
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  fixedLandmarks = as.numeric(fixedLandmarks);
  movingLandmarks = as.numeric(movingLandmarks);
  ;ans = .Call('R_swig_LandmarkBasedTransformInitializer__SWIG_3', transform, fixedLandmarks, movingLandmarks, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LandmarkBasedTransformInitializer__SWIG_3`, 'returnType') = '_p_itk__simple__Transform'
attr(`LandmarkBasedTransformInitializer__SWIG_3`, "inputTypes") = c('_p_itk__simple__Transform', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`LandmarkBasedTransformInitializer__SWIG_3`) = c("SWIGFunction", class('LandmarkBasedTransformInitializer__SWIG_3'))

# Start of LandmarkBasedTransformInitializer

`LandmarkBasedTransformInitializer__SWIG_4` = function(transform, fixedLandmarks, .copy = FALSE)
{
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  fixedLandmarks = as.numeric(fixedLandmarks);
  ;ans = .Call('R_swig_LandmarkBasedTransformInitializer__SWIG_4', transform, fixedLandmarks, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LandmarkBasedTransformInitializer__SWIG_4`, 'returnType') = '_p_itk__simple__Transform'
attr(`LandmarkBasedTransformInitializer__SWIG_4`, "inputTypes") = c('_p_itk__simple__Transform', '_p_std__vectorT_double_t')
class(`LandmarkBasedTransformInitializer__SWIG_4`) = c("SWIGFunction", class('LandmarkBasedTransformInitializer__SWIG_4'))

# Start of LandmarkBasedTransformInitializer

`LandmarkBasedTransformInitializer__SWIG_5` = function(transform, .copy = FALSE)
{
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_LandmarkBasedTransformInitializer__SWIG_5', transform, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LandmarkBasedTransformInitializer__SWIG_5`, 'returnType') = '_p_itk__simple__Transform'
attr(`LandmarkBasedTransformInitializer__SWIG_5`, "inputTypes") = c('_p_itk__simple__Transform')
class(`LandmarkBasedTransformInitializer__SWIG_5`) = c("SWIGFunction", class('LandmarkBasedTransformInitializer__SWIG_5'))

`LandmarkBasedTransformInitializer` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- LandmarkBasedTransformInitializer__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) )) {
      f <- LandmarkBasedTransformInitializer__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) )) {
      f <- LandmarkBasedTransformInitializer__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) )) {
      f <- LandmarkBasedTransformInitializer__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- LandmarkBasedTransformInitializer__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- LandmarkBasedTransformInitializer__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LandmarkBasedTransformInitializer with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of CastImageFilter_SetOutputPixelType

`CastImageFilter_SetOutputPixelType` = function(self, pixelID)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  pixelID = enumToInteger(pixelID, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(pixelID) > 1) {
    warning("using only the first element of pixelID");
  };
  
  ;ans = .Call('R_swig_CastImageFilter_SetOutputPixelType', self, pixelID, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CastImageFilter_SetOutputPixelType`, 'returnType') = 'void'
attr(`CastImageFilter_SetOutputPixelType`, "inputTypes") = c('_p_itk__simple__CastImageFilter', 'character')
class(`CastImageFilter_SetOutputPixelType`) = c("SWIGFunction", class('CastImageFilter_SetOutputPixelType'))

# Start of CastImageFilter_GetOutputPixelType

`CastImageFilter_GetOutputPixelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CastImageFilter_GetOutputPixelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`CastImageFilter_GetOutputPixelType`, 'returnType') = 'character'
attr(`CastImageFilter_GetOutputPixelType`, "inputTypes") = c('_p_itk__simple__CastImageFilter')
class(`CastImageFilter_GetOutputPixelType`) = c("SWIGFunction", class('CastImageFilter_GetOutputPixelType'))

# Start of delete_CastImageFilter

`delete_CastImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_CastImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_CastImageFilter`, 'returnType') = 'void'
attr(`delete_CastImageFilter`, "inputTypes") = c('_p_itk__simple__CastImageFilter')
class(`delete_CastImageFilter`) = c("SWIGFunction", class('delete_CastImageFilter'))

# Start of new_CastImageFilter

`CastImageFilter` = function()
{
  ;ans = .Call('R_swig_new_CastImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CastImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CastImageFilter);
  ans
  
}

attr(`CastImageFilter`, 'returnType') = '_p_itk__simple__CastImageFilter'
class(`CastImageFilter`) = c("SWIGFunction", class('CastImageFilter'))

# Start of CastImageFilter_GetName

`CastImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CastImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CastImageFilter_GetName`, 'returnType') = 'character'
attr(`CastImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__CastImageFilter')
class(`CastImageFilter_GetName`) = c("SWIGFunction", class('CastImageFilter_GetName'))

# Start of CastImageFilter_ToString

`CastImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CastImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CastImageFilter_ToString`, 'returnType') = 'character'
attr(`CastImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__CastImageFilter')
class(`CastImageFilter_ToString`) = c("SWIGFunction", class('CastImageFilter_ToString'))

# Start of CastImageFilter_Execute

`CastImageFilter_Execute` = function(self, s_arg2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(s_arg2, "ExternalReference")) s_arg2 = slot(s_arg2,"ref"); 
  ;ans = .Call('R_swig_CastImageFilter_Execute', self, s_arg2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CastImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`CastImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__CastImageFilter', '_p_itk__simple__Image')
class(`CastImageFilter_Execute`) = c("SWIGFunction", class('CastImageFilter_Execute'))

# Start of accessor method for itk::simple::CastImageFilter
setMethod('$', '_p_itk__simple__CastImageFilter', function(x, name)

{
  accessorFuns = list('SetOutputPixelType' = CastImageFilter_SetOutputPixelType, 'GetOutputPixelType' = CastImageFilter_GetOutputPixelType, 'GetName' = CastImageFilter_GetName, 'ToString' = CastImageFilter_ToString, 'Execute' = CastImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::CastImageFilter
# Start of Cast

`Cast` = function(image, pixelID, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  pixelID = enumToInteger(pixelID, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(pixelID) > 1) {
    warning("using only the first element of pixelID");
  };
  
  ;ans = .Call('R_swig_Cast', image, pixelID, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Cast`, 'returnType') = '_p_itk__simple__Image'
attr(`Cast`, "inputTypes") = c('_p_itk__simple__Image', 'character')
class(`Cast`) = c("SWIGFunction", class('Cast'))

# Start of delete_ExtractImageFilter

`delete_ExtractImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ExtractImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ExtractImageFilter`, 'returnType') = 'void'
attr(`delete_ExtractImageFilter`, "inputTypes") = c('_p_itk__simple__ExtractImageFilter')
class(`delete_ExtractImageFilter`) = c("SWIGFunction", class('delete_ExtractImageFilter'))

# Start of new_ExtractImageFilter

`ExtractImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ExtractImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ExtractImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ExtractImageFilter);
  ans
  
}

attr(`ExtractImageFilter`, 'returnType') = '_p_itk__simple__ExtractImageFilter'
class(`ExtractImageFilter`) = c("SWIGFunction", class('ExtractImageFilter'))

# Start of ExtractImageFilter_SetSize

`ExtractImageFilter_SetSize` = function(self, Size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Size = as.integer(Size);
  ;ans = .Call('R_swig_ExtractImageFilter_SetSize', self, Size, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ExtractImageFilter_SetSize`, 'returnType') = 'void'
attr(`ExtractImageFilter_SetSize`, "inputTypes") = c('_p_itk__simple__ExtractImageFilter', 'integer')
class(`ExtractImageFilter_SetSize`) = c("SWIGFunction", class('ExtractImageFilter_SetSize'))

# Start of ExtractImageFilter_GetSize

`ExtractImageFilter_GetSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ExtractImageFilter_GetSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ExtractImageFilter_GetSize`, 'returnType') = 'integer'
attr(`ExtractImageFilter_GetSize`, "inputTypes") = c('_p_itk__simple__ExtractImageFilter')
class(`ExtractImageFilter_GetSize`) = c("SWIGFunction", class('ExtractImageFilter_GetSize'))

# Start of ExtractImageFilter_SetIndex

`ExtractImageFilter_SetIndex` = function(self, Index)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Index = as.integer(Index);
  ;ans = .Call('R_swig_ExtractImageFilter_SetIndex', self, Index, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ExtractImageFilter_SetIndex`, 'returnType') = 'void'
attr(`ExtractImageFilter_SetIndex`, "inputTypes") = c('_p_itk__simple__ExtractImageFilter', 'integer')
class(`ExtractImageFilter_SetIndex`) = c("SWIGFunction", class('ExtractImageFilter_SetIndex'))

# Start of ExtractImageFilter_GetIndex

`ExtractImageFilter_GetIndex` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ExtractImageFilter_GetIndex', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ExtractImageFilter_GetIndex`, 'returnType') = 'integer'
attr(`ExtractImageFilter_GetIndex`, "inputTypes") = c('_p_itk__simple__ExtractImageFilter')
class(`ExtractImageFilter_GetIndex`) = c("SWIGFunction", class('ExtractImageFilter_GetIndex'))

# Start of ExtractImageFilter_DIRECTIONCOLLAPSETOUNKOWN_get

`ExtractImageFilter_DIRECTIONCOLLAPSETOUNKOWN_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ExtractImageFilter_DIRECTIONCOLLAPSETOUNKOWN_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ExtractImageFilter__DirectionCollapseToStrategyType");
  
  ans
  
}

attr(`ExtractImageFilter_DIRECTIONCOLLAPSETOUNKOWN_get`, 'returnType') = 'character'
class(`ExtractImageFilter_DIRECTIONCOLLAPSETOUNKOWN_get`) = c("SWIGFunction", class('ExtractImageFilter_DIRECTIONCOLLAPSETOUNKOWN_get'))

# Start of ExtractImageFilter_DIRECTIONCOLLAPSETOIDENTITY_get

`ExtractImageFilter_DIRECTIONCOLLAPSETOIDENTITY_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ExtractImageFilter_DIRECTIONCOLLAPSETOIDENTITY_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ExtractImageFilter__DirectionCollapseToStrategyType");
  
  ans
  
}

attr(`ExtractImageFilter_DIRECTIONCOLLAPSETOIDENTITY_get`, 'returnType') = 'character'
class(`ExtractImageFilter_DIRECTIONCOLLAPSETOIDENTITY_get`) = c("SWIGFunction", class('ExtractImageFilter_DIRECTIONCOLLAPSETOIDENTITY_get'))

# Start of ExtractImageFilter_DIRECTIONCOLLAPSETOSUBMATRIX_get

`ExtractImageFilter_DIRECTIONCOLLAPSETOSUBMATRIX_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ExtractImageFilter_DIRECTIONCOLLAPSETOSUBMATRIX_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ExtractImageFilter__DirectionCollapseToStrategyType");
  
  ans
  
}

attr(`ExtractImageFilter_DIRECTIONCOLLAPSETOSUBMATRIX_get`, 'returnType') = 'character'
class(`ExtractImageFilter_DIRECTIONCOLLAPSETOSUBMATRIX_get`) = c("SWIGFunction", class('ExtractImageFilter_DIRECTIONCOLLAPSETOSUBMATRIX_get'))

# Start of ExtractImageFilter_DIRECTIONCOLLAPSETOGUESS_get

`ExtractImageFilter_DIRECTIONCOLLAPSETOGUESS_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ExtractImageFilter_DIRECTIONCOLLAPSETOGUESS_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ExtractImageFilter__DirectionCollapseToStrategyType");
  
  ans
  
}

attr(`ExtractImageFilter_DIRECTIONCOLLAPSETOGUESS_get`, 'returnType') = 'character'
class(`ExtractImageFilter_DIRECTIONCOLLAPSETOGUESS_get`) = c("SWIGFunction", class('ExtractImageFilter_DIRECTIONCOLLAPSETOGUESS_get'))

# Start of ExtractImageFilter_SetDirectionCollapseToStrategy

`ExtractImageFilter_SetDirectionCollapseToStrategy` = function(self, DirectionCollapseToStrategy)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  DirectionCollapseToStrategy = enumToInteger(DirectionCollapseToStrategy, "_itk__simple__ExtractImageFilter__DirectionCollapseToStrategyType"); 
  
  if(length(DirectionCollapseToStrategy) > 1) {
    warning("using only the first element of DirectionCollapseToStrategy");
  };
  
  ;ans = .Call('R_swig_ExtractImageFilter_SetDirectionCollapseToStrategy', self, DirectionCollapseToStrategy, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ExtractImageFilter_SetDirectionCollapseToStrategy`, 'returnType') = 'void'
attr(`ExtractImageFilter_SetDirectionCollapseToStrategy`, "inputTypes") = c('_p_itk__simple__ExtractImageFilter', 'character')
class(`ExtractImageFilter_SetDirectionCollapseToStrategy`) = c("SWIGFunction", class('ExtractImageFilter_SetDirectionCollapseToStrategy'))

# Start of ExtractImageFilter_GetDirectionCollapseToStrategy

`ExtractImageFilter_GetDirectionCollapseToStrategy` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ExtractImageFilter_GetDirectionCollapseToStrategy', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ExtractImageFilter__DirectionCollapseToStrategyType");
  
  ans
  
}

attr(`ExtractImageFilter_GetDirectionCollapseToStrategy`, 'returnType') = 'character'
attr(`ExtractImageFilter_GetDirectionCollapseToStrategy`, "inputTypes") = c('_p_itk__simple__ExtractImageFilter')
class(`ExtractImageFilter_GetDirectionCollapseToStrategy`) = c("SWIGFunction", class('ExtractImageFilter_GetDirectionCollapseToStrategy'))

# Start of ExtractImageFilter_GetName

`ExtractImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ExtractImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ExtractImageFilter_GetName`, 'returnType') = 'character'
attr(`ExtractImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ExtractImageFilter')
class(`ExtractImageFilter_GetName`) = c("SWIGFunction", class('ExtractImageFilter_GetName'))

# Start of ExtractImageFilter_ToString

`ExtractImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ExtractImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ExtractImageFilter_ToString`, 'returnType') = 'character'
attr(`ExtractImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ExtractImageFilter')
class(`ExtractImageFilter_ToString`) = c("SWIGFunction", class('ExtractImageFilter_ToString'))

# Start of ExtractImageFilter_Execute

`ExtractImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ExtractImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ExtractImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ExtractImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ExtractImageFilter', '_p_itk__simple__Image')
class(`ExtractImageFilter_Execute`) = c("SWIGFunction", class('ExtractImageFilter_Execute'))

# Start of accessor method for itk::simple::ExtractImageFilter
setMethod('$', '_p_itk__simple__ExtractImageFilter', function(x, name)

{
  accessorFuns = list('SetSize' = ExtractImageFilter_SetSize, 'GetSize' = ExtractImageFilter_GetSize, 'SetIndex' = ExtractImageFilter_SetIndex, 'GetIndex' = ExtractImageFilter_GetIndex, 'SetDirectionCollapseToStrategy' = ExtractImageFilter_SetDirectionCollapseToStrategy, 'GetDirectionCollapseToStrategy' = ExtractImageFilter_GetDirectionCollapseToStrategy, 'GetName' = ExtractImageFilter_GetName, 'ToString' = ExtractImageFilter_ToString, 'Execute' = ExtractImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ExtractImageFilter
# Start of Extract

`Extract__SWIG_0` = function(image1, size, index, directionCollapseToStrategy, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  index = as.integer(index);
  directionCollapseToStrategy = enumToInteger(directionCollapseToStrategy, "_itk__simple__ExtractImageFilter__DirectionCollapseToStrategyType"); 
  
  if(length(directionCollapseToStrategy) > 1) {
    warning("using only the first element of directionCollapseToStrategy");
  };
  
  ;ans = .Call('R_swig_Extract__SWIG_0', image1, size, index, directionCollapseToStrategy, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Extract__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Extract__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'character')
class(`Extract__SWIG_0`) = c("SWIGFunction", class('Extract__SWIG_0'))

# Start of Extract

`Extract__SWIG_1` = function(image1, size, index, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  index = as.integer(index);
  ;ans = .Call('R_swig_Extract__SWIG_1', image1, size, index, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Extract__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Extract__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Extract__SWIG_1`) = c("SWIGFunction", class('Extract__SWIG_1'))

# Start of Extract

`Extract__SWIG_2` = function(image1, size, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  ;ans = .Call('R_swig_Extract__SWIG_2', image1, size, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Extract__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Extract__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Extract__SWIG_2`) = c("SWIGFunction", class('Extract__SWIG_2'))

# Start of Extract

`Extract__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Extract__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Extract__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Extract__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`Extract__SWIG_3`) = c("SWIGFunction", class('Extract__SWIG_3'))

`Extract` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Extract__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- Extract__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- Extract__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- Extract__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Extract with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_PasteImageFilter

`delete_PasteImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_PasteImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_PasteImageFilter`, 'returnType') = 'void'
attr(`delete_PasteImageFilter`, "inputTypes") = c('_p_itk__simple__PasteImageFilter')
class(`delete_PasteImageFilter`) = c("SWIGFunction", class('delete_PasteImageFilter'))

# Start of new_PasteImageFilter

`PasteImageFilter` = function()
{
  ;ans = .Call('R_swig_new_PasteImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__PasteImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_PasteImageFilter);
  ans
  
}

attr(`PasteImageFilter`, 'returnType') = '_p_itk__simple__PasteImageFilter'
class(`PasteImageFilter`) = c("SWIGFunction", class('PasteImageFilter'))

# Start of PasteImageFilter_SetSourceSize

`PasteImageFilter_SetSourceSize` = function(self, SourceSize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SourceSize = as.integer(SourceSize);
  ;ans = .Call('R_swig_PasteImageFilter_SetSourceSize', self, SourceSize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PasteImageFilter_SetSourceSize`, 'returnType') = 'void'
attr(`PasteImageFilter_SetSourceSize`, "inputTypes") = c('_p_itk__simple__PasteImageFilter', 'integer')
class(`PasteImageFilter_SetSourceSize`) = c("SWIGFunction", class('PasteImageFilter_SetSourceSize'))

# Start of PasteImageFilter_GetSourceSize

`PasteImageFilter_GetSourceSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PasteImageFilter_GetSourceSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PasteImageFilter_GetSourceSize`, 'returnType') = 'integer'
attr(`PasteImageFilter_GetSourceSize`, "inputTypes") = c('_p_itk__simple__PasteImageFilter')
class(`PasteImageFilter_GetSourceSize`) = c("SWIGFunction", class('PasteImageFilter_GetSourceSize'))

# Start of PasteImageFilter_SetSourceIndex

`PasteImageFilter_SetSourceIndex` = function(self, SourceIndex)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SourceIndex = as.integer(SourceIndex);
  ;ans = .Call('R_swig_PasteImageFilter_SetSourceIndex', self, SourceIndex, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PasteImageFilter_SetSourceIndex`, 'returnType') = 'void'
attr(`PasteImageFilter_SetSourceIndex`, "inputTypes") = c('_p_itk__simple__PasteImageFilter', 'integer')
class(`PasteImageFilter_SetSourceIndex`) = c("SWIGFunction", class('PasteImageFilter_SetSourceIndex'))

# Start of PasteImageFilter_GetSourceIndex

`PasteImageFilter_GetSourceIndex` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PasteImageFilter_GetSourceIndex', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PasteImageFilter_GetSourceIndex`, 'returnType') = 'integer'
attr(`PasteImageFilter_GetSourceIndex`, "inputTypes") = c('_p_itk__simple__PasteImageFilter')
class(`PasteImageFilter_GetSourceIndex`) = c("SWIGFunction", class('PasteImageFilter_GetSourceIndex'))

# Start of PasteImageFilter_SetDestinationIndex

`PasteImageFilter_SetDestinationIndex` = function(self, DestinationIndex)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  DestinationIndex = as.integer(DestinationIndex);
  ;ans = .Call('R_swig_PasteImageFilter_SetDestinationIndex', self, DestinationIndex, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PasteImageFilter_SetDestinationIndex`, 'returnType') = 'void'
attr(`PasteImageFilter_SetDestinationIndex`, "inputTypes") = c('_p_itk__simple__PasteImageFilter', 'integer')
class(`PasteImageFilter_SetDestinationIndex`) = c("SWIGFunction", class('PasteImageFilter_SetDestinationIndex'))

# Start of PasteImageFilter_GetDestinationIndex

`PasteImageFilter_GetDestinationIndex` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PasteImageFilter_GetDestinationIndex', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PasteImageFilter_GetDestinationIndex`, 'returnType') = 'integer'
attr(`PasteImageFilter_GetDestinationIndex`, "inputTypes") = c('_p_itk__simple__PasteImageFilter')
class(`PasteImageFilter_GetDestinationIndex`) = c("SWIGFunction", class('PasteImageFilter_GetDestinationIndex'))

# Start of PasteImageFilter_SetDestinationSkipAxes

`PasteImageFilter_SetDestinationSkipAxes` = function(self, DestinationSkipAxes)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  DestinationSkipAxes = as.logical(DestinationSkipAxes);
  ;ans = .Call('R_swig_PasteImageFilter_SetDestinationSkipAxes', self, DestinationSkipAxes, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PasteImageFilter_SetDestinationSkipAxes`, 'returnType') = 'void'
attr(`PasteImageFilter_SetDestinationSkipAxes`, "inputTypes") = c('_p_itk__simple__PasteImageFilter', 'logical')
class(`PasteImageFilter_SetDestinationSkipAxes`) = c("SWIGFunction", class('PasteImageFilter_SetDestinationSkipAxes'))

# Start of PasteImageFilter_GetDestinationSkipAxes

`PasteImageFilter_GetDestinationSkipAxes` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PasteImageFilter_GetDestinationSkipAxes', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PasteImageFilter_GetDestinationSkipAxes`, 'returnType') = 'logical'
attr(`PasteImageFilter_GetDestinationSkipAxes`, "inputTypes") = c('_p_itk__simple__PasteImageFilter')
class(`PasteImageFilter_GetDestinationSkipAxes`) = c("SWIGFunction", class('PasteImageFilter_GetDestinationSkipAxes'))

# Start of PasteImageFilter_GetName

`PasteImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PasteImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PasteImageFilter_GetName`, 'returnType') = 'character'
attr(`PasteImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__PasteImageFilter')
class(`PasteImageFilter_GetName`) = c("SWIGFunction", class('PasteImageFilter_GetName'))

# Start of PasteImageFilter_ToString

`PasteImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PasteImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PasteImageFilter_ToString`, 'returnType') = 'character'
attr(`PasteImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__PasteImageFilter')
class(`PasteImageFilter_ToString`) = c("SWIGFunction", class('PasteImageFilter_ToString'))

# Start of PasteImageFilter_Execute

`PasteImageFilter_Execute__SWIG_0` = function(self, destinationImage, sourceImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(destinationImage, "ExternalReference")) destinationImage = slot(destinationImage,"ref"); 
  if (inherits(sourceImage, "ExternalReference")) sourceImage = slot(sourceImage,"ref"); 
  ;ans = .Call('R_swig_PasteImageFilter_Execute__SWIG_0', self, destinationImage, sourceImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PasteImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`PasteImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__PasteImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`PasteImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('PasteImageFilter_Execute__SWIG_0'))

# Start of PasteImageFilter_Execute

`PasteImageFilter_Execute__SWIG_1` = function(self, destinationImage, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(destinationImage, "ExternalReference")) destinationImage = slot(destinationImage,"ref"); 
  
  ;ans = .Call('R_swig_PasteImageFilter_Execute__SWIG_1', self, destinationImage, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PasteImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`PasteImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__PasteImageFilter', '_p_itk__simple__Image', 'numeric')
class(`PasteImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('PasteImageFilter_Execute__SWIG_1'))

`PasteImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__PasteImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- PasteImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__PasteImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- PasteImageFilter_Execute__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for PasteImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::PasteImageFilter
setMethod('$', '_p_itk__simple__PasteImageFilter', function(x, name)

{
  accessorFuns = list('SetSourceSize' = PasteImageFilter_SetSourceSize, 'GetSourceSize' = PasteImageFilter_GetSourceSize, 'SetSourceIndex' = PasteImageFilter_SetSourceIndex, 'GetSourceIndex' = PasteImageFilter_GetSourceIndex, 'SetDestinationIndex' = PasteImageFilter_SetDestinationIndex, 'GetDestinationIndex' = PasteImageFilter_GetDestinationIndex, 'SetDestinationSkipAxes' = PasteImageFilter_SetDestinationSkipAxes, 'GetDestinationSkipAxes' = PasteImageFilter_GetDestinationSkipAxes, 'GetName' = PasteImageFilter_GetName, 'ToString' = PasteImageFilter_ToString, 'Execute' = PasteImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::PasteImageFilter
# Start of Paste

`Paste__SWIG_0` = function(destinationImage, sourceImage, sourceSize, sourceIndex, destinationIndex, DestinationSkipAxes, .copy = FALSE)
{
  if (inherits(destinationImage, "ExternalReference")) destinationImage = slot(destinationImage,"ref"); 
  if (inherits(sourceImage, "ExternalReference")) sourceImage = slot(sourceImage,"ref"); 
  sourceSize = as.integer(sourceSize);
  sourceIndex = as.integer(sourceIndex);
  destinationIndex = as.integer(destinationIndex);
  DestinationSkipAxes = as.logical(DestinationSkipAxes);
  ;ans = .Call('R_swig_Paste__SWIG_0', destinationImage, sourceImage, sourceSize, sourceIndex, destinationIndex, DestinationSkipAxes, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Paste__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Paste__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`Paste__SWIG_0`) = c("SWIGFunction", class('Paste__SWIG_0'))

# Start of Paste

`Paste__SWIG_1` = function(destinationImage, sourceImage, sourceSize, sourceIndex, destinationIndex, .copy = FALSE)
{
  if (inherits(destinationImage, "ExternalReference")) destinationImage = slot(destinationImage,"ref"); 
  if (inherits(sourceImage, "ExternalReference")) sourceImage = slot(sourceImage,"ref"); 
  sourceSize = as.integer(sourceSize);
  sourceIndex = as.integer(sourceIndex);
  destinationIndex = as.integer(destinationIndex);
  ;ans = .Call('R_swig_Paste__SWIG_1', destinationImage, sourceImage, sourceSize, sourceIndex, destinationIndex, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Paste__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Paste__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`Paste__SWIG_1`) = c("SWIGFunction", class('Paste__SWIG_1'))

# Start of Paste

`Paste__SWIG_2` = function(destinationImage, sourceImage, sourceSize, sourceIndex, .copy = FALSE)
{
  if (inherits(destinationImage, "ExternalReference")) destinationImage = slot(destinationImage,"ref"); 
  if (inherits(sourceImage, "ExternalReference")) sourceImage = slot(sourceImage,"ref"); 
  sourceSize = as.integer(sourceSize);
  sourceIndex = as.integer(sourceIndex);
  ;ans = .Call('R_swig_Paste__SWIG_2', destinationImage, sourceImage, sourceSize, sourceIndex, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Paste__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Paste__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`Paste__SWIG_2`) = c("SWIGFunction", class('Paste__SWIG_2'))

# Start of Paste

`Paste__SWIG_3` = function(destinationImage, sourceImage, sourceSize, .copy = FALSE)
{
  if (inherits(destinationImage, "ExternalReference")) destinationImage = slot(destinationImage,"ref"); 
  if (inherits(sourceImage, "ExternalReference")) sourceImage = slot(sourceImage,"ref"); 
  sourceSize = as.integer(sourceSize);
  ;ans = .Call('R_swig_Paste__SWIG_3', destinationImage, sourceImage, sourceSize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Paste__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Paste__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`Paste__SWIG_3`) = c("SWIGFunction", class('Paste__SWIG_3'))

# Start of Paste

`Paste__SWIG_4` = function(destinationImage, sourceImage, .copy = FALSE)
{
  if (inherits(destinationImage, "ExternalReference")) destinationImage = slot(destinationImage,"ref"); 
  if (inherits(sourceImage, "ExternalReference")) sourceImage = slot(sourceImage,"ref"); 
  ;ans = .Call('R_swig_Paste__SWIG_4', destinationImage, sourceImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Paste__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`Paste__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Paste__SWIG_4`) = c("SWIGFunction", class('Paste__SWIG_4'))

`Paste` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Paste__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- Paste__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) )) {
      f <- Paste__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.integer(argv[[5]]) || is.numeric(argv[[5]]) )) {
      f <- Paste__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.integer(argv[[5]]) || is.numeric(argv[[5]]) ) && ( is.logical(argv[[6]]) )) {
      f <- Paste__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Paste with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of Resample

`Resample__SWIG_0` = function(image1, transform, interpolator, defaultPixelValue, outputPixelType, useNearestNeighborExtrapolator, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  useNearestNeighborExtrapolator = as.logical(useNearestNeighborExtrapolator);
  ;ans = .Call('R_swig_Resample__SWIG_0', image1, transform, interpolator, defaultPixelValue, outputPixelType, useNearestNeighborExtrapolator, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Transform', 'character', 'numeric', 'character', 'logical')
class(`Resample__SWIG_0`) = c("SWIGFunction", class('Resample__SWIG_0'))

# Start of Resample

`Resample__SWIG_1` = function(image1, transform, interpolator, defaultPixelValue, outputPixelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  ;ans = .Call('R_swig_Resample__SWIG_1', image1, transform, interpolator, defaultPixelValue, outputPixelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Transform', 'character', 'numeric', 'character')
class(`Resample__SWIG_1`) = c("SWIGFunction", class('Resample__SWIG_1'))

# Start of Resample

`Resample__SWIG_2` = function(image1, transform, interpolator, defaultPixelValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  
  ;ans = .Call('R_swig_Resample__SWIG_2', image1, transform, interpolator, defaultPixelValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Transform', 'character', 'numeric')
class(`Resample__SWIG_2`) = c("SWIGFunction", class('Resample__SWIG_2'))

# Start of Resample

`Resample__SWIG_3` = function(image1, transform, interpolator, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  ;ans = .Call('R_swig_Resample__SWIG_3', image1, transform, interpolator, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Transform', 'character')
class(`Resample__SWIG_3`) = c("SWIGFunction", class('Resample__SWIG_3'))

# Start of Resample

`Resample__SWIG_4` = function(image1, transform, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_Resample__SWIG_4', image1, transform, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Transform')
class(`Resample__SWIG_4`) = c("SWIGFunction", class('Resample__SWIG_4'))

# Start of Resample

`Resample__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Resample__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`Resample__SWIG_5`) = c("SWIGFunction", class('Resample__SWIG_5'))

# Start of Resample

`Resample__SWIG_6` = function(image1, referenceImage, transform, interpolator, defaultPixelValue, outputPixelType, useNearestNeighborExtrapolator, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(referenceImage, "ExternalReference")) referenceImage = slot(referenceImage,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  useNearestNeighborExtrapolator = as.logical(useNearestNeighborExtrapolator);
  ;ans = .Call('R_swig_Resample__SWIG_6', image1, referenceImage, transform, interpolator, defaultPixelValue, outputPixelType, useNearestNeighborExtrapolator, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Transform', 'character', 'numeric', 'character', 'logical')
class(`Resample__SWIG_6`) = c("SWIGFunction", class('Resample__SWIG_6'))

# Start of Resample

`Resample__SWIG_7` = function(image1, referenceImage, transform, interpolator, defaultPixelValue, outputPixelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(referenceImage, "ExternalReference")) referenceImage = slot(referenceImage,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  ;ans = .Call('R_swig_Resample__SWIG_7', image1, referenceImage, transform, interpolator, defaultPixelValue, outputPixelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Transform', 'character', 'numeric', 'character')
class(`Resample__SWIG_7`) = c("SWIGFunction", class('Resample__SWIG_7'))

# Start of Resample

`Resample__SWIG_8` = function(image1, referenceImage, transform, interpolator, defaultPixelValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(referenceImage, "ExternalReference")) referenceImage = slot(referenceImage,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  
  ;ans = .Call('R_swig_Resample__SWIG_8', image1, referenceImage, transform, interpolator, defaultPixelValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Transform', 'character', 'numeric')
class(`Resample__SWIG_8`) = c("SWIGFunction", class('Resample__SWIG_8'))

# Start of Resample

`Resample__SWIG_9` = function(image1, referenceImage, transform, interpolator, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(referenceImage, "ExternalReference")) referenceImage = slot(referenceImage,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  ;ans = .Call('R_swig_Resample__SWIG_9', image1, referenceImage, transform, interpolator, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Transform', 'character')
class(`Resample__SWIG_9`) = c("SWIGFunction", class('Resample__SWIG_9'))

# Start of Resample

`Resample__SWIG_10` = function(image1, referenceImage, transform, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(referenceImage, "ExternalReference")) referenceImage = slot(referenceImage,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_Resample__SWIG_10', image1, referenceImage, transform, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Transform')
class(`Resample__SWIG_10`) = c("SWIGFunction", class('Resample__SWIG_10'))

# Start of Resample

`Resample__SWIG_11` = function(image1, referenceImage, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(referenceImage, "ExternalReference")) referenceImage = slot(referenceImage,"ref"); 
  ;ans = .Call('R_swig_Resample__SWIG_11', image1, referenceImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Resample__SWIG_11`) = c("SWIGFunction", class('Resample__SWIG_11'))

# Start of Resample

`Resample__SWIG_12` = function(image1, size, transform, interpolator, outputOrigin, outputSpacing, outputDirection, defaultPixelValue, outputPixelType, useNearestNeighborExtrapolator, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  outputOrigin = as.numeric(outputOrigin);
  outputSpacing = as.numeric(outputSpacing);
  outputDirection = as.numeric(outputDirection);
  
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  useNearestNeighborExtrapolator = as.logical(useNearestNeighborExtrapolator);
  ;ans = .Call('R_swig_Resample__SWIG_12', image1, size, transform, interpolator, outputOrigin, outputSpacing, outputDirection, defaultPixelValue, outputPixelType, useNearestNeighborExtrapolator, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_12`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_12`, "inputTypes") = c('_p_itk__simple__Image', 'integer', '_p_itk__simple__Transform', 'character', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', 'numeric', 'character', 'logical')
class(`Resample__SWIG_12`) = c("SWIGFunction", class('Resample__SWIG_12'))

# Start of Resample

`Resample__SWIG_13` = function(image1, size, transform, interpolator, outputOrigin, outputSpacing, outputDirection, defaultPixelValue, outputPixelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  outputOrigin = as.numeric(outputOrigin);
  outputSpacing = as.numeric(outputSpacing);
  outputDirection = as.numeric(outputDirection);
  
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  ;ans = .Call('R_swig_Resample__SWIG_13', image1, size, transform, interpolator, outputOrigin, outputSpacing, outputDirection, defaultPixelValue, outputPixelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_13`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_13`, "inputTypes") = c('_p_itk__simple__Image', 'integer', '_p_itk__simple__Transform', 'character', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', 'numeric', 'character')
class(`Resample__SWIG_13`) = c("SWIGFunction", class('Resample__SWIG_13'))

# Start of Resample

`Resample__SWIG_14` = function(image1, size, transform, interpolator, outputOrigin, outputSpacing, outputDirection, defaultPixelValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  outputOrigin = as.numeric(outputOrigin);
  outputSpacing = as.numeric(outputSpacing);
  outputDirection = as.numeric(outputDirection);
  
  ;ans = .Call('R_swig_Resample__SWIG_14', image1, size, transform, interpolator, outputOrigin, outputSpacing, outputDirection, defaultPixelValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_14`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_14`, "inputTypes") = c('_p_itk__simple__Image', 'integer', '_p_itk__simple__Transform', 'character', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', 'numeric')
class(`Resample__SWIG_14`) = c("SWIGFunction", class('Resample__SWIG_14'))

# Start of Resample

`Resample__SWIG_15` = function(image1, size, transform, interpolator, outputOrigin, outputSpacing, outputDirection, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  outputOrigin = as.numeric(outputOrigin);
  outputSpacing = as.numeric(outputSpacing);
  outputDirection = as.numeric(outputDirection);
  ;ans = .Call('R_swig_Resample__SWIG_15', image1, size, transform, interpolator, outputOrigin, outputSpacing, outputDirection, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_15`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_15`, "inputTypes") = c('_p_itk__simple__Image', 'integer', '_p_itk__simple__Transform', 'character', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`Resample__SWIG_15`) = c("SWIGFunction", class('Resample__SWIG_15'))

# Start of Resample

`Resample__SWIG_16` = function(image1, size, transform, interpolator, outputOrigin, outputSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  outputOrigin = as.numeric(outputOrigin);
  outputSpacing = as.numeric(outputSpacing);
  ;ans = .Call('R_swig_Resample__SWIG_16', image1, size, transform, interpolator, outputOrigin, outputSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_16`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_16`, "inputTypes") = c('_p_itk__simple__Image', 'integer', '_p_itk__simple__Transform', 'character', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`Resample__SWIG_16`) = c("SWIGFunction", class('Resample__SWIG_16'))

# Start of Resample

`Resample__SWIG_17` = function(image1, size, transform, interpolator, outputOrigin, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  outputOrigin = as.numeric(outputOrigin);
  ;ans = .Call('R_swig_Resample__SWIG_17', image1, size, transform, interpolator, outputOrigin, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_17`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_17`, "inputTypes") = c('_p_itk__simple__Image', 'integer', '_p_itk__simple__Transform', 'character', '_p_std__vectorT_double_t')
class(`Resample__SWIG_17`) = c("SWIGFunction", class('Resample__SWIG_17'))

# Start of Resample

`Resample__SWIG_18` = function(image1, size, transform, interpolator, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  ;ans = .Call('R_swig_Resample__SWIG_18', image1, size, transform, interpolator, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_18`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_18`, "inputTypes") = c('_p_itk__simple__Image', 'integer', '_p_itk__simple__Transform', 'character')
class(`Resample__SWIG_18`) = c("SWIGFunction", class('Resample__SWIG_18'))

# Start of Resample

`Resample__SWIG_19` = function(image1, size, transform, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_Resample__SWIG_19', image1, size, transform, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_19`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_19`, "inputTypes") = c('_p_itk__simple__Image', 'integer', '_p_itk__simple__Transform')
class(`Resample__SWIG_19`) = c("SWIGFunction", class('Resample__SWIG_19'))

# Start of Resample

`Resample__SWIG_20` = function(image1, size, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  ;ans = .Call('R_swig_Resample__SWIG_20', image1, size, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Resample__SWIG_20`, 'returnType') = '_p_itk__simple__Image'
attr(`Resample__SWIG_20`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Resample__SWIG_20`) = c("SWIGFunction", class('Resample__SWIG_20'))

`Resample` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 21
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Resample__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Transform') && length(argv[[2]]) == 1 )) {
      f <- Resample__SWIG_4; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Resample__SWIG_11; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- Resample__SWIG_20; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Transform') && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Resample__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 )) {
      f <- Resample__SWIG_10; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 )) {
      f <- Resample__SWIG_19; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Transform') && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- Resample__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- Resample__SWIG_9; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- Resample__SWIG_18; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- Resample__SWIG_8; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Transform') && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.character(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- Resample__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) )) {
      f <- Resample__SWIG_17; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.character(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- Resample__SWIG_7; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Transform') && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.character(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- Resample__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) )) {
      f <- Resample__SWIG_16; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.character(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.logical(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- Resample__SWIG_6; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) ) && ( is.numeric(argv[[7]]) )) {
      f <- Resample__SWIG_15; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) ) && ( is.numeric(argv[[7]]) ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 )) {
      f <- Resample__SWIG_14; 
    }
  } else if (argc == 9) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) ) && ( is.numeric(argv[[7]]) ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( is.character(argv[[9]]) && length(argv[[9]]) == 1 )) {
      f <- Resample__SWIG_13; 
    }
  } else if (argc == 10) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( extends(argtypes[3], '_p_itk__simple__Transform') && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) ) && ( is.numeric(argv[[7]]) ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( is.character(argv[[9]]) && length(argv[[9]]) == 1 ) && ( is.logical(argv[[10]]) && length(argv[[10]]) == 1 )) {
      f <- Resample__SWIG_12; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Resample with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of PatchBasedDenoising

`PatchBasedDenoising__SWIG_0` = function(image1, noiseModel, kernelBandwidthSigma, patchRadius, numberOfIterations, numberOfSamplePatches, sampleVariance, noiseSigma, noiseModelFidelityWeight, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  noiseModel = enumToInteger(noiseModel, "_itk__simple__PatchBasedDenoisingImageFilter__NoiseModelType"); 
  
  if(length(noiseModel) > 1) {
    warning("using only the first element of noiseModel");
  };
  
  
  patchRadius = as.integer(patchRadius);
  
  if(length(patchRadius) > 1) {
    warning("using only the first element of patchRadius");
  };
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  numberOfSamplePatches = as.integer(numberOfSamplePatches);
  
  if(length(numberOfSamplePatches) > 1) {
    warning("using only the first element of numberOfSamplePatches");
  };
  
  
  
  
  ;ans = .Call('R_swig_PatchBasedDenoising__SWIG_0', image1, noiseModel, kernelBandwidthSigma, patchRadius, numberOfIterations, numberOfSamplePatches, sampleVariance, noiseSigma, noiseModelFidelityWeight, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PatchBasedDenoising__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`PatchBasedDenoising__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'numeric', 'integer', 'integer', 'integer', 'numeric', 'numeric', 'numeric')
class(`PatchBasedDenoising__SWIG_0`) = c("SWIGFunction", class('PatchBasedDenoising__SWIG_0'))

# Start of PatchBasedDenoising

`PatchBasedDenoising__SWIG_1` = function(image1, noiseModel, kernelBandwidthSigma, patchRadius, numberOfIterations, numberOfSamplePatches, sampleVariance, noiseSigma, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  noiseModel = enumToInteger(noiseModel, "_itk__simple__PatchBasedDenoisingImageFilter__NoiseModelType"); 
  
  if(length(noiseModel) > 1) {
    warning("using only the first element of noiseModel");
  };
  
  
  patchRadius = as.integer(patchRadius);
  
  if(length(patchRadius) > 1) {
    warning("using only the first element of patchRadius");
  };
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  numberOfSamplePatches = as.integer(numberOfSamplePatches);
  
  if(length(numberOfSamplePatches) > 1) {
    warning("using only the first element of numberOfSamplePatches");
  };
  
  
  
  ;ans = .Call('R_swig_PatchBasedDenoising__SWIG_1', image1, noiseModel, kernelBandwidthSigma, patchRadius, numberOfIterations, numberOfSamplePatches, sampleVariance, noiseSigma, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PatchBasedDenoising__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`PatchBasedDenoising__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'numeric', 'integer', 'integer', 'integer', 'numeric', 'numeric')
class(`PatchBasedDenoising__SWIG_1`) = c("SWIGFunction", class('PatchBasedDenoising__SWIG_1'))

# Start of PatchBasedDenoising

`PatchBasedDenoising__SWIG_2` = function(image1, noiseModel, kernelBandwidthSigma, patchRadius, numberOfIterations, numberOfSamplePatches, sampleVariance, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  noiseModel = enumToInteger(noiseModel, "_itk__simple__PatchBasedDenoisingImageFilter__NoiseModelType"); 
  
  if(length(noiseModel) > 1) {
    warning("using only the first element of noiseModel");
  };
  
  
  patchRadius = as.integer(patchRadius);
  
  if(length(patchRadius) > 1) {
    warning("using only the first element of patchRadius");
  };
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  numberOfSamplePatches = as.integer(numberOfSamplePatches);
  
  if(length(numberOfSamplePatches) > 1) {
    warning("using only the first element of numberOfSamplePatches");
  };
  
  
  ;ans = .Call('R_swig_PatchBasedDenoising__SWIG_2', image1, noiseModel, kernelBandwidthSigma, patchRadius, numberOfIterations, numberOfSamplePatches, sampleVariance, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PatchBasedDenoising__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`PatchBasedDenoising__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'numeric', 'integer', 'integer', 'integer', 'numeric')
class(`PatchBasedDenoising__SWIG_2`) = c("SWIGFunction", class('PatchBasedDenoising__SWIG_2'))

# Start of PatchBasedDenoising

`PatchBasedDenoising__SWIG_3` = function(image1, noiseModel, kernelBandwidthSigma, patchRadius, numberOfIterations, numberOfSamplePatches, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  noiseModel = enumToInteger(noiseModel, "_itk__simple__PatchBasedDenoisingImageFilter__NoiseModelType"); 
  
  if(length(noiseModel) > 1) {
    warning("using only the first element of noiseModel");
  };
  
  
  patchRadius = as.integer(patchRadius);
  
  if(length(patchRadius) > 1) {
    warning("using only the first element of patchRadius");
  };
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  numberOfSamplePatches = as.integer(numberOfSamplePatches);
  
  if(length(numberOfSamplePatches) > 1) {
    warning("using only the first element of numberOfSamplePatches");
  };
  
  ;ans = .Call('R_swig_PatchBasedDenoising__SWIG_3', image1, noiseModel, kernelBandwidthSigma, patchRadius, numberOfIterations, numberOfSamplePatches, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PatchBasedDenoising__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`PatchBasedDenoising__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'numeric', 'integer', 'integer', 'integer')
class(`PatchBasedDenoising__SWIG_3`) = c("SWIGFunction", class('PatchBasedDenoising__SWIG_3'))

# Start of PatchBasedDenoising

`PatchBasedDenoising__SWIG_4` = function(image1, noiseModel, kernelBandwidthSigma, patchRadius, numberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  noiseModel = enumToInteger(noiseModel, "_itk__simple__PatchBasedDenoisingImageFilter__NoiseModelType"); 
  
  if(length(noiseModel) > 1) {
    warning("using only the first element of noiseModel");
  };
  
  
  patchRadius = as.integer(patchRadius);
  
  if(length(patchRadius) > 1) {
    warning("using only the first element of patchRadius");
  };
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_PatchBasedDenoising__SWIG_4', image1, noiseModel, kernelBandwidthSigma, patchRadius, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PatchBasedDenoising__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`PatchBasedDenoising__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'numeric', 'integer', 'integer')
class(`PatchBasedDenoising__SWIG_4`) = c("SWIGFunction", class('PatchBasedDenoising__SWIG_4'))

# Start of PatchBasedDenoising

`PatchBasedDenoising__SWIG_5` = function(image1, noiseModel, kernelBandwidthSigma, patchRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  noiseModel = enumToInteger(noiseModel, "_itk__simple__PatchBasedDenoisingImageFilter__NoiseModelType"); 
  
  if(length(noiseModel) > 1) {
    warning("using only the first element of noiseModel");
  };
  
  
  patchRadius = as.integer(patchRadius);
  
  if(length(patchRadius) > 1) {
    warning("using only the first element of patchRadius");
  };
  
  ;ans = .Call('R_swig_PatchBasedDenoising__SWIG_5', image1, noiseModel, kernelBandwidthSigma, patchRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PatchBasedDenoising__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`PatchBasedDenoising__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'numeric', 'integer')
class(`PatchBasedDenoising__SWIG_5`) = c("SWIGFunction", class('PatchBasedDenoising__SWIG_5'))

# Start of PatchBasedDenoising

`PatchBasedDenoising__SWIG_6` = function(image1, noiseModel, kernelBandwidthSigma, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  noiseModel = enumToInteger(noiseModel, "_itk__simple__PatchBasedDenoisingImageFilter__NoiseModelType"); 
  
  if(length(noiseModel) > 1) {
    warning("using only the first element of noiseModel");
  };
  
  
  ;ans = .Call('R_swig_PatchBasedDenoising__SWIG_6', image1, noiseModel, kernelBandwidthSigma, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PatchBasedDenoising__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`PatchBasedDenoising__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'numeric')
class(`PatchBasedDenoising__SWIG_6`) = c("SWIGFunction", class('PatchBasedDenoising__SWIG_6'))

# Start of PatchBasedDenoising

`PatchBasedDenoising__SWIG_7` = function(image1, noiseModel, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  noiseModel = enumToInteger(noiseModel, "_itk__simple__PatchBasedDenoisingImageFilter__NoiseModelType"); 
  
  if(length(noiseModel) > 1) {
    warning("using only the first element of noiseModel");
  };
  
  ;ans = .Call('R_swig_PatchBasedDenoising__SWIG_7', image1, noiseModel, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PatchBasedDenoising__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`PatchBasedDenoising__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', 'character')
class(`PatchBasedDenoising__SWIG_7`) = c("SWIGFunction", class('PatchBasedDenoising__SWIG_7'))

# Start of PatchBasedDenoising

`PatchBasedDenoising__SWIG_8` = function(image1, kernelBandwidthSigma, patchRadius, numberOfIterations, numberOfSamplePatches, sampleVariance, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  patchRadius = as.integer(patchRadius);
  
  if(length(patchRadius) > 1) {
    warning("using only the first element of patchRadius");
  };
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  numberOfSamplePatches = as.integer(numberOfSamplePatches);
  
  if(length(numberOfSamplePatches) > 1) {
    warning("using only the first element of numberOfSamplePatches");
  };
  
  
  ;ans = .Call('R_swig_PatchBasedDenoising__SWIG_8', image1, kernelBandwidthSigma, patchRadius, numberOfIterations, numberOfSamplePatches, sampleVariance, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PatchBasedDenoising__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`PatchBasedDenoising__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer', 'integer', 'numeric')
class(`PatchBasedDenoising__SWIG_8`) = c("SWIGFunction", class('PatchBasedDenoising__SWIG_8'))

# Start of PatchBasedDenoising

`PatchBasedDenoising__SWIG_9` = function(image1, kernelBandwidthSigma, patchRadius, numberOfIterations, numberOfSamplePatches, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  patchRadius = as.integer(patchRadius);
  
  if(length(patchRadius) > 1) {
    warning("using only the first element of patchRadius");
  };
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  numberOfSamplePatches = as.integer(numberOfSamplePatches);
  
  if(length(numberOfSamplePatches) > 1) {
    warning("using only the first element of numberOfSamplePatches");
  };
  
  ;ans = .Call('R_swig_PatchBasedDenoising__SWIG_9', image1, kernelBandwidthSigma, patchRadius, numberOfIterations, numberOfSamplePatches, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PatchBasedDenoising__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`PatchBasedDenoising__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer', 'integer')
class(`PatchBasedDenoising__SWIG_9`) = c("SWIGFunction", class('PatchBasedDenoising__SWIG_9'))

# Start of PatchBasedDenoising

`PatchBasedDenoising__SWIG_10` = function(image1, kernelBandwidthSigma, patchRadius, numberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  patchRadius = as.integer(patchRadius);
  
  if(length(patchRadius) > 1) {
    warning("using only the first element of patchRadius");
  };
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_PatchBasedDenoising__SWIG_10', image1, kernelBandwidthSigma, patchRadius, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PatchBasedDenoising__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`PatchBasedDenoising__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`PatchBasedDenoising__SWIG_10`) = c("SWIGFunction", class('PatchBasedDenoising__SWIG_10'))

# Start of PatchBasedDenoising

`PatchBasedDenoising__SWIG_11` = function(image1, kernelBandwidthSigma, patchRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  patchRadius = as.integer(patchRadius);
  
  if(length(patchRadius) > 1) {
    warning("using only the first element of patchRadius");
  };
  
  ;ans = .Call('R_swig_PatchBasedDenoising__SWIG_11', image1, kernelBandwidthSigma, patchRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PatchBasedDenoising__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`PatchBasedDenoising__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`PatchBasedDenoising__SWIG_11`) = c("SWIGFunction", class('PatchBasedDenoising__SWIG_11'))

# Start of PatchBasedDenoising

`PatchBasedDenoising__SWIG_12` = function(image1, kernelBandwidthSigma, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_PatchBasedDenoising__SWIG_12', image1, kernelBandwidthSigma, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PatchBasedDenoising__SWIG_12`, 'returnType') = '_p_itk__simple__Image'
attr(`PatchBasedDenoising__SWIG_12`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`PatchBasedDenoising__SWIG_12`) = c("SWIGFunction", class('PatchBasedDenoising__SWIG_12'))

# Start of PatchBasedDenoising

`PatchBasedDenoising__SWIG_13` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_PatchBasedDenoising__SWIG_13', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PatchBasedDenoising__SWIG_13`, 'returnType') = '_p_itk__simple__Image'
attr(`PatchBasedDenoising__SWIG_13`, "inputTypes") = c('_p_itk__simple__Image')
class(`PatchBasedDenoising__SWIG_13`) = c("SWIGFunction", class('PatchBasedDenoising__SWIG_13'))

`PatchBasedDenoising` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 14
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- PatchBasedDenoising__SWIG_13; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- PatchBasedDenoising__SWIG_7; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- PatchBasedDenoising__SWIG_12; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- PatchBasedDenoising__SWIG_6; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- PatchBasedDenoising__SWIG_11; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- PatchBasedDenoising__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- PatchBasedDenoising__SWIG_10; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- PatchBasedDenoising__SWIG_4; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- PatchBasedDenoising__SWIG_9; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- PatchBasedDenoising__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- PatchBasedDenoising__SWIG_8; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- PatchBasedDenoising__SWIG_2; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 )) {
      f <- PatchBasedDenoising__SWIG_1; 
    }
  } else if (argc == 9) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( is.numeric(argv[[9]]) && length(argv[[9]]) == 1 )) {
      f <- PatchBasedDenoising__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for PatchBasedDenoising with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of DiscreteGaussian

`DiscreteGaussian__SWIG_0` = function(image1, variance, maximumKernelWidth, maximumError, useImageSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  maximumKernelWidth = as.integer(maximumKernelWidth);
  
  if(length(maximumKernelWidth) > 1) {
    warning("using only the first element of maximumKernelWidth");
  };
  
  
  useImageSpacing = as.logical(useImageSpacing);
  ;ans = .Call('R_swig_DiscreteGaussian__SWIG_0', image1, variance, maximumKernelWidth, maximumError, useImageSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussian__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussian__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'logical')
class(`DiscreteGaussian__SWIG_0`) = c("SWIGFunction", class('DiscreteGaussian__SWIG_0'))

# Start of DiscreteGaussian

`DiscreteGaussian__SWIG_1` = function(image1, variance, maximumKernelWidth, maximumError, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  maximumKernelWidth = as.integer(maximumKernelWidth);
  
  if(length(maximumKernelWidth) > 1) {
    warning("using only the first element of maximumKernelWidth");
  };
  
  
  ;ans = .Call('R_swig_DiscreteGaussian__SWIG_1', image1, variance, maximumKernelWidth, maximumError, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussian__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussian__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'numeric')
class(`DiscreteGaussian__SWIG_1`) = c("SWIGFunction", class('DiscreteGaussian__SWIG_1'))

# Start of DiscreteGaussian

`DiscreteGaussian__SWIG_2` = function(image1, variance, maximumKernelWidth, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  maximumKernelWidth = as.integer(maximumKernelWidth);
  
  if(length(maximumKernelWidth) > 1) {
    warning("using only the first element of maximumKernelWidth");
  };
  
  ;ans = .Call('R_swig_DiscreteGaussian__SWIG_2', image1, variance, maximumKernelWidth, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussian__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussian__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`DiscreteGaussian__SWIG_2`) = c("SWIGFunction", class('DiscreteGaussian__SWIG_2'))

# Start of DiscreteGaussian

`DiscreteGaussian__SWIG_3` = function(image1, variance, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_DiscreteGaussian__SWIG_3', image1, variance, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussian__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussian__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`DiscreteGaussian__SWIG_3`) = c("SWIGFunction", class('DiscreteGaussian__SWIG_3'))

# Start of SmoothingRecursiveGaussian

`SmoothingRecursiveGaussian__SWIG_0` = function(image1, sigma, normalizeAcrossScale, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  normalizeAcrossScale = as.logical(normalizeAcrossScale);
  ;ans = .Call('R_swig_SmoothingRecursiveGaussian__SWIG_0', image1, sigma, normalizeAcrossScale, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SmoothingRecursiveGaussian__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`SmoothingRecursiveGaussian__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`SmoothingRecursiveGaussian__SWIG_0`) = c("SWIGFunction", class('SmoothingRecursiveGaussian__SWIG_0'))

# Start of SmoothingRecursiveGaussian

`SmoothingRecursiveGaussian__SWIG_1` = function(image1, sigma, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_SmoothingRecursiveGaussian__SWIG_1', image1, sigma, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SmoothingRecursiveGaussian__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`SmoothingRecursiveGaussian__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`SmoothingRecursiveGaussian__SWIG_1`) = c("SWIGFunction", class('SmoothingRecursiveGaussian__SWIG_1'))

# Start of delete_ImageRegistrationMethod

`delete_ImageRegistrationMethod` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ImageRegistrationMethod', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ImageRegistrationMethod`, 'returnType') = 'void'
attr(`delete_ImageRegistrationMethod`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`delete_ImageRegistrationMethod`) = c("SWIGFunction", class('delete_ImageRegistrationMethod'))

# Start of new_ImageRegistrationMethod

`ImageRegistrationMethod` = function()
{
  ;ans = .Call('R_swig_new_ImageRegistrationMethod', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ImageRegistrationMethod", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ImageRegistrationMethod);
  ans
  
}

attr(`ImageRegistrationMethod`, 'returnType') = '_p_itk__simple__ImageRegistrationMethod'
class(`ImageRegistrationMethod`) = c("SWIGFunction", class('ImageRegistrationMethod'))

# Start of ImageRegistrationMethod_GetName

`ImageRegistrationMethod_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_GetName`, 'returnType') = 'character'
attr(`ImageRegistrationMethod_GetName`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetName`) = c("SWIGFunction", class('ImageRegistrationMethod_GetName'))

# Start of ImageRegistrationMethod_ToString

`ImageRegistrationMethod_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_ToString`, 'returnType') = 'character'
attr(`ImageRegistrationMethod_ToString`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_ToString`) = c("SWIGFunction", class('ImageRegistrationMethod_ToString'))

# Start of ImageRegistrationMethod_GetInterpolator

`ImageRegistrationMethod_GetInterpolator` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_GetInterpolator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`ImageRegistrationMethod_GetInterpolator`, 'returnType') = 'character'
attr(`ImageRegistrationMethod_GetInterpolator`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetInterpolator`) = c("SWIGFunction", class('ImageRegistrationMethod_GetInterpolator'))

# Start of ImageRegistrationMethod_SetInterpolator

`ImageRegistrationMethod_SetInterpolator` = function(self, Interpolator)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Interpolator = enumToInteger(Interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(Interpolator) > 1) {
    warning("using only the first element of Interpolator");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetInterpolator', self, Interpolator, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetInterpolator`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetInterpolator`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'character')
class(`ImageRegistrationMethod_SetInterpolator`) = c("SWIGFunction", class('ImageRegistrationMethod_SetInterpolator'))

# Start of ImageRegistrationMethod_SetInitialTransform

`ImageRegistrationMethod_SetInitialTransform__SWIG_0` = function(self, transform, inPlace)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  inPlace = as.logical(inPlace);
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetInitialTransform__SWIG_0', self, transform, inPlace, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetInitialTransform__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetInitialTransform__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_itk__simple__Transform', 'logical')
class(`ImageRegistrationMethod_SetInitialTransform__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetInitialTransform__SWIG_0'))

# Start of ImageRegistrationMethod_SetInitialTransform

`ImageRegistrationMethod_SetInitialTransform__SWIG_1` = function(self, transform)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetInitialTransform__SWIG_1', self, transform, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetInitialTransform__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetInitialTransform__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_itk__simple__Transform')
class(`ImageRegistrationMethod_SetInitialTransform__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetInitialTransform__SWIG_1'))

`ImageRegistrationMethod_SetInitialTransform` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Transform') && length(argv[[2]]) == 1 )) {
      f <- ImageRegistrationMethod_SetInitialTransform__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Transform') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetInitialTransform__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetInitialTransform with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_GetInitialTransform

`ImageRegistrationMethod_GetInitialTransform` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_GetInitialTransform', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_GetInitialTransform`, 'returnType') = '_p_itk__simple__Transform'
attr(`ImageRegistrationMethod_GetInitialTransform`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetInitialTransform`) = c("SWIGFunction", class('ImageRegistrationMethod_GetInitialTransform'))

# Start of ImageRegistrationMethod_GetInitialTransformInPlace

`ImageRegistrationMethod_GetInitialTransformInPlace` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_GetInitialTransformInPlace', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_GetInitialTransformInPlace`, 'returnType') = 'logical'
attr(`ImageRegistrationMethod_GetInitialTransformInPlace`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetInitialTransformInPlace`) = c("SWIGFunction", class('ImageRegistrationMethod_GetInitialTransformInPlace'))

# Start of ImageRegistrationMethod_SetInitialTransformAsBSpline

`ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_0` = function(self, transform, inPlace, scaleFactors)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  inPlace = as.logical(inPlace);
  scaleFactors = as.integer(scaleFactors);
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_0', self, transform, inPlace, scaleFactors, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_itk__simple__BSplineTransform', 'logical', 'integer')
class(`ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_0'))

# Start of ImageRegistrationMethod_SetInitialTransformAsBSpline

`ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_1` = function(self, transform, inPlace)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  inPlace = as.logical(inPlace);
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_1', self, transform, inPlace, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_itk__simple__BSplineTransform', 'logical')
class(`ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_1'))

# Start of ImageRegistrationMethod_SetInitialTransformAsBSpline

`ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_2` = function(self, transform)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_2', self, transform, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_2`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_2`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_itk__simple__BSplineTransform')
class(`ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_2`) = c("SWIGFunction", class('ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_2'))

`ImageRegistrationMethod_SetInitialTransformAsBSpline` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__BSplineTransform') && length(argv[[2]]) == 1 )) {
      f <- ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__BSplineTransform') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__BSplineTransform') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) )) {
      f <- ImageRegistrationMethod_SetInitialTransformAsBSpline__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetInitialTransformAsBSpline with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetMovingInitialTransform

`ImageRegistrationMethod_SetMovingInitialTransform` = function(self, transform)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMovingInitialTransform', self, transform, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMovingInitialTransform`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMovingInitialTransform`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_itk__simple__Transform')
class(`ImageRegistrationMethod_SetMovingInitialTransform`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMovingInitialTransform'))

# Start of ImageRegistrationMethod_GetMovingInitialTransform

`ImageRegistrationMethod_GetMovingInitialTransform` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_GetMovingInitialTransform', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_GetMovingInitialTransform`, 'returnType') = '_p_itk__simple__Transform'
attr(`ImageRegistrationMethod_GetMovingInitialTransform`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetMovingInitialTransform`) = c("SWIGFunction", class('ImageRegistrationMethod_GetMovingInitialTransform'))

# Start of ImageRegistrationMethod_SetFixedInitialTransform

`ImageRegistrationMethod_SetFixedInitialTransform` = function(self, transform)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetFixedInitialTransform', self, transform, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetFixedInitialTransform`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetFixedInitialTransform`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_itk__simple__Transform')
class(`ImageRegistrationMethod_SetFixedInitialTransform`) = c("SWIGFunction", class('ImageRegistrationMethod_SetFixedInitialTransform'))

# Start of ImageRegistrationMethod_GetFixedInitialTransform

`ImageRegistrationMethod_GetFixedInitialTransform` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_GetFixedInitialTransform', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_GetFixedInitialTransform`, 'returnType') = '_p_itk__simple__Transform'
attr(`ImageRegistrationMethod_GetFixedInitialTransform`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetFixedInitialTransform`) = c("SWIGFunction", class('ImageRegistrationMethod_GetFixedInitialTransform'))

# Start of ImageRegistrationMethod_SetVirtualDomain

`ImageRegistrationMethod_SetVirtualDomain` = function(self, virtualSize, virtualOrigin, virtualSpacing, virtualDirection)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  virtualSize = as.integer(virtualSize);
  virtualOrigin = as.numeric(virtualOrigin);
  virtualSpacing = as.numeric(virtualSpacing);
  virtualDirection = as.numeric(virtualDirection);
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetVirtualDomain', self, virtualSize, virtualOrigin, virtualSpacing, virtualDirection, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetVirtualDomain`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetVirtualDomain`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t', '_p_std__vectorT_double_t')
class(`ImageRegistrationMethod_SetVirtualDomain`) = c("SWIGFunction", class('ImageRegistrationMethod_SetVirtualDomain'))

# Start of ImageRegistrationMethod_SetVirtualDomainFromImage

`ImageRegistrationMethod_SetVirtualDomainFromImage` = function(self, virtualImage)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(virtualImage, "ExternalReference")) virtualImage = slot(virtualImage,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetVirtualDomainFromImage', self, virtualImage, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetVirtualDomainFromImage`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetVirtualDomainFromImage`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_itk__simple__Image')
class(`ImageRegistrationMethod_SetVirtualDomainFromImage`) = c("SWIGFunction", class('ImageRegistrationMethod_SetVirtualDomainFromImage'))

# Start of ImageRegistrationMethod_SetMetricAsANTSNeighborhoodCorrelation

`ImageRegistrationMethod_SetMetricAsANTSNeighborhoodCorrelation` = function(self, radius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  radius = as.integer(radius);
  
  if(length(radius) > 1) {
    warning("using only the first element of radius");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricAsANTSNeighborhoodCorrelation', self, radius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricAsANTSNeighborhoodCorrelation`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricAsANTSNeighborhoodCorrelation`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer')
class(`ImageRegistrationMethod_SetMetricAsANTSNeighborhoodCorrelation`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricAsANTSNeighborhoodCorrelation'))

# Start of ImageRegistrationMethod_SetMetricAsCorrelation

`ImageRegistrationMethod_SetMetricAsCorrelation` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricAsCorrelation', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricAsCorrelation`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricAsCorrelation`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_SetMetricAsCorrelation`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricAsCorrelation'))

# Start of ImageRegistrationMethod_SetMetricAsDemons

`ImageRegistrationMethod_SetMetricAsDemons__SWIG_0` = function(self, intensityDifferenceThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricAsDemons__SWIG_0', self, intensityDifferenceThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricAsDemons__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricAsDemons__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric')
class(`ImageRegistrationMethod_SetMetricAsDemons__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricAsDemons__SWIG_0'))

# Start of ImageRegistrationMethod_SetMetricAsDemons

`ImageRegistrationMethod_SetMetricAsDemons__SWIG_1` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricAsDemons__SWIG_1', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricAsDemons__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricAsDemons__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_SetMetricAsDemons__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricAsDemons__SWIG_1'))

`ImageRegistrationMethod_SetMetricAsDemons` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) {
      f <- ImageRegistrationMethod_SetMetricAsDemons__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ImageRegistrationMethod_SetMetricAsDemons__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetMetricAsDemons with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation

`ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_0` = function(self, numberOfHistogramBins, varianceForJointPDFSmoothing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_0', self, numberOfHistogramBins, varianceForJointPDFSmoothing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_0'))

# Start of ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation

`ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_1` = function(self, numberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_1', self, numberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer')
class(`ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_1'))

# Start of ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation

`ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_2` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_2', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_2`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_2`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_2`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_2'))

`ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) {
      f <- ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetMetricAsMeanSquares

`ImageRegistrationMethod_SetMetricAsMeanSquares` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricAsMeanSquares', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricAsMeanSquares`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricAsMeanSquares`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_SetMetricAsMeanSquares`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricAsMeanSquares'))

# Start of ImageRegistrationMethod_SetMetricAsMattesMutualInformation

`ImageRegistrationMethod_SetMetricAsMattesMutualInformation__SWIG_0` = function(self, numberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricAsMattesMutualInformation__SWIG_0', self, numberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricAsMattesMutualInformation__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricAsMattesMutualInformation__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer')
class(`ImageRegistrationMethod_SetMetricAsMattesMutualInformation__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricAsMattesMutualInformation__SWIG_0'))

# Start of ImageRegistrationMethod_SetMetricAsMattesMutualInformation

`ImageRegistrationMethod_SetMetricAsMattesMutualInformation__SWIG_1` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricAsMattesMutualInformation__SWIG_1', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricAsMattesMutualInformation__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricAsMattesMutualInformation__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_SetMetricAsMattesMutualInformation__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricAsMattesMutualInformation__SWIG_1'))

`ImageRegistrationMethod_SetMetricAsMattesMutualInformation` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) {
      f <- ImageRegistrationMethod_SetMetricAsMattesMutualInformation__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- ImageRegistrationMethod_SetMetricAsMattesMutualInformation__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetMetricAsMattesMutualInformation with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_Never_get

`ImageRegistrationMethod_Never_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ImageRegistrationMethod_Never_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ImageRegistrationMethod__EstimateLearningRateType");
  
  ans
  
}

attr(`ImageRegistrationMethod_Never_get`, 'returnType') = 'character'
class(`ImageRegistrationMethod_Never_get`) = c("SWIGFunction", class('ImageRegistrationMethod_Never_get'))

# Start of ImageRegistrationMethod_Once_get

`ImageRegistrationMethod_Once_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ImageRegistrationMethod_Once_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ImageRegistrationMethod__EstimateLearningRateType");
  
  ans
  
}

attr(`ImageRegistrationMethod_Once_get`, 'returnType') = 'character'
class(`ImageRegistrationMethod_Once_get`) = c("SWIGFunction", class('ImageRegistrationMethod_Once_get'))

# Start of ImageRegistrationMethod_EachIteration_get

`ImageRegistrationMethod_EachIteration_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ImageRegistrationMethod_EachIteration_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ImageRegistrationMethod__EstimateLearningRateType");
  
  ans
  
}

attr(`ImageRegistrationMethod_EachIteration_get`, 'returnType') = 'character'
class(`ImageRegistrationMethod_EachIteration_get`) = c("SWIGFunction", class('ImageRegistrationMethod_EachIteration_get'))

# Start of ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch

`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_0` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon, lineSearchMaximumIterations, estimateLearningRate, maximumStepSizeInPhysicalUnits)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  
  
  
  lineSearchMaximumIterations = as.integer(lineSearchMaximumIterations);
  
  if(length(lineSearchMaximumIterations) > 1) {
    warning("using only the first element of lineSearchMaximumIterations");
  };
  
  estimateLearningRate = enumToInteger(estimateLearningRate, "_itk__simple__ImageRegistrationMethod__EstimateLearningRateType"); 
  
  if(length(estimateLearningRate) > 1) {
    warning("using only the first element of estimateLearningRate");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_0', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon, lineSearchMaximumIterations, estimateLearningRate, maximumStepSizeInPhysicalUnits, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer', 'numeric', 'numeric', 'numeric', 'integer', 'character', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_0'))

# Start of ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch

`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_1` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon, lineSearchMaximumIterations, estimateLearningRate)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  
  
  
  lineSearchMaximumIterations = as.integer(lineSearchMaximumIterations);
  
  if(length(lineSearchMaximumIterations) > 1) {
    warning("using only the first element of lineSearchMaximumIterations");
  };
  
  estimateLearningRate = enumToInteger(estimateLearningRate, "_itk__simple__ImageRegistrationMethod__EstimateLearningRateType"); 
  
  if(length(estimateLearningRate) > 1) {
    warning("using only the first element of estimateLearningRate");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_1', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon, lineSearchMaximumIterations, estimateLearningRate, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer', 'numeric', 'numeric', 'numeric', 'integer', 'character')
class(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_1'))

# Start of ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch

`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_2` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon, lineSearchMaximumIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  
  
  
  lineSearchMaximumIterations = as.integer(lineSearchMaximumIterations);
  
  if(length(lineSearchMaximumIterations) > 1) {
    warning("using only the first element of lineSearchMaximumIterations");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_2', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon, lineSearchMaximumIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_2`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_2`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer', 'numeric', 'numeric', 'numeric', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_2`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_2'))

# Start of ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch

`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_3` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_3', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_3`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_3`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer', 'numeric', 'numeric', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_3`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_3'))

# Start of ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch

`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_4` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_4', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_4`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_4`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer', 'numeric', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_4`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_4'))

# Start of ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch

`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_5` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_5', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_5`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_5`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_5`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_5'))

# Start of ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch

`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_6` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_6', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_6`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_6`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_6`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_6'))

# Start of ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch

`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_7` = function(self, learningRate, numberOfIterations, convergenceMinimumValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_7', self, learningRate, numberOfIterations, convergenceMinimumValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_7`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_7`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_7`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_7'))

# Start of ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch

`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_8` = function(self, learningRate, numberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_8', self, learningRate, numberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_8`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_8`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_8`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_8'))

`ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_8; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_7; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_6; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_5; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_4; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_3; 
    }
  } else if (argc == 9) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( (is.integer(argv[[9]]) || is.numeric(argv[[9]])) && length(argv[[9]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_2; 
    }
  } else if (argc == 10) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( (is.integer(argv[[9]]) || is.numeric(argv[[9]])) && length(argv[[9]]) == 1 ) && ( is.character(argv[[10]]) && length(argv[[10]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_1; 
    }
  } else if (argc == 11) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( (is.integer(argv[[9]]) || is.numeric(argv[[9]])) && length(argv[[9]]) == 1 ) && ( is.character(argv[[10]]) && length(argv[[10]]) == 1 ) && ( is.numeric(argv[[11]]) && length(argv[[11]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent

`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_0` = function(self, learningRate, minStep, numberOfIterations, relaxationFactor, gradientMagnitudeTolerance, estimateLearningRate, maximumStepSizeInPhysicalUnits)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  estimateLearningRate = enumToInteger(estimateLearningRate, "_itk__simple__ImageRegistrationMethod__EstimateLearningRateType"); 
  
  if(length(estimateLearningRate) > 1) {
    warning("using only the first element of estimateLearningRate");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_0', self, learningRate, minStep, numberOfIterations, relaxationFactor, gradientMagnitudeTolerance, estimateLearningRate, maximumStepSizeInPhysicalUnits, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'numeric', 'integer', 'numeric', 'numeric', 'character', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_0'))

# Start of ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent

`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_1` = function(self, learningRate, minStep, numberOfIterations, relaxationFactor, gradientMagnitudeTolerance, estimateLearningRate)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  estimateLearningRate = enumToInteger(estimateLearningRate, "_itk__simple__ImageRegistrationMethod__EstimateLearningRateType"); 
  
  if(length(estimateLearningRate) > 1) {
    warning("using only the first element of estimateLearningRate");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_1', self, learningRate, minStep, numberOfIterations, relaxationFactor, gradientMagnitudeTolerance, estimateLearningRate, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'numeric', 'integer', 'numeric', 'numeric', 'character')
class(`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_1'))

# Start of ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent

`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_2` = function(self, learningRate, minStep, numberOfIterations, relaxationFactor, gradientMagnitudeTolerance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_2', self, learningRate, minStep, numberOfIterations, relaxationFactor, gradientMagnitudeTolerance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_2`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_2`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'numeric', 'integer', 'numeric', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_2`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_2'))

# Start of ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent

`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_3` = function(self, learningRate, minStep, numberOfIterations, relaxationFactor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_3', self, learningRate, minStep, numberOfIterations, relaxationFactor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_3`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_3`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'numeric', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_3`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_3'))

# Start of ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent

`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_4` = function(self, learningRate, minStep, numberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_4', self, learningRate, minStep, numberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_4`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_4`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'numeric', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_4`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_4'))

`ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_4; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_3; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_2; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.character(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_1; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.character(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetOptimizerAsGradientDescent

`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_0` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, estimateLearningRate, maximumStepSizeInPhysicalUnits)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  estimateLearningRate = enumToInteger(estimateLearningRate, "_itk__simple__ImageRegistrationMethod__EstimateLearningRateType"); 
  
  if(length(estimateLearningRate) > 1) {
    warning("using only the first element of estimateLearningRate");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_0', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, estimateLearningRate, maximumStepSizeInPhysicalUnits, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer', 'character', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_0'))

# Start of ImageRegistrationMethod_SetOptimizerAsGradientDescent

`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_1` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, estimateLearningRate)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  estimateLearningRate = enumToInteger(estimateLearningRate, "_itk__simple__ImageRegistrationMethod__EstimateLearningRateType"); 
  
  if(length(estimateLearningRate) > 1) {
    warning("using only the first element of estimateLearningRate");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_1', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, estimateLearningRate, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer', 'character')
class(`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_1'))

# Start of ImageRegistrationMethod_SetOptimizerAsGradientDescent

`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_2` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_2', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_2`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_2`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_2`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_2'))

# Start of ImageRegistrationMethod_SetOptimizerAsGradientDescent

`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_3` = function(self, learningRate, numberOfIterations, convergenceMinimumValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_3', self, learningRate, numberOfIterations, convergenceMinimumValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_3`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_3`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_3`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_3'))

# Start of ImageRegistrationMethod_SetOptimizerAsGradientDescent

`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_4` = function(self, learningRate, numberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_4', self, learningRate, numberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_4`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_4`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_4`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_4'))

`ImageRegistrationMethod_SetOptimizerAsGradientDescent` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_2; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.character(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_1; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.character(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsGradientDescent__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetOptimizerAsGradientDescent with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch

`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_0` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon, lineSearchMaximumIterations, estimateLearningRate, maximumStepSizeInPhysicalUnits)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  
  
  
  lineSearchMaximumIterations = as.integer(lineSearchMaximumIterations);
  
  if(length(lineSearchMaximumIterations) > 1) {
    warning("using only the first element of lineSearchMaximumIterations");
  };
  
  estimateLearningRate = enumToInteger(estimateLearningRate, "_itk__simple__ImageRegistrationMethod__EstimateLearningRateType"); 
  
  if(length(estimateLearningRate) > 1) {
    warning("using only the first element of estimateLearningRate");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_0', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon, lineSearchMaximumIterations, estimateLearningRate, maximumStepSizeInPhysicalUnits, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer', 'numeric', 'numeric', 'numeric', 'integer', 'character', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_0'))

# Start of ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch

`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_1` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon, lineSearchMaximumIterations, estimateLearningRate)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  
  
  
  lineSearchMaximumIterations = as.integer(lineSearchMaximumIterations);
  
  if(length(lineSearchMaximumIterations) > 1) {
    warning("using only the first element of lineSearchMaximumIterations");
  };
  
  estimateLearningRate = enumToInteger(estimateLearningRate, "_itk__simple__ImageRegistrationMethod__EstimateLearningRateType"); 
  
  if(length(estimateLearningRate) > 1) {
    warning("using only the first element of estimateLearningRate");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_1', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon, lineSearchMaximumIterations, estimateLearningRate, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer', 'numeric', 'numeric', 'numeric', 'integer', 'character')
class(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_1'))

# Start of ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch

`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_2` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon, lineSearchMaximumIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  
  
  
  lineSearchMaximumIterations = as.integer(lineSearchMaximumIterations);
  
  if(length(lineSearchMaximumIterations) > 1) {
    warning("using only the first element of lineSearchMaximumIterations");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_2', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon, lineSearchMaximumIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_2`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_2`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer', 'numeric', 'numeric', 'numeric', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_2`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_2'))

# Start of ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch

`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_3` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_3', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, lineSearchEpsilon, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_3`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_3`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer', 'numeric', 'numeric', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_3`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_3'))

# Start of ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch

`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_4` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_4', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, lineSearchUpperLimit, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_4`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_4`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer', 'numeric', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_4`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_4'))

# Start of ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch

`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_5` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_5', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, lineSearchLowerLimit, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_5`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_5`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_5`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_5'))

# Start of ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch

`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_6` = function(self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  convergenceWindowSize = as.integer(convergenceWindowSize);
  
  if(length(convergenceWindowSize) > 1) {
    warning("using only the first element of convergenceWindowSize");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_6', self, learningRate, numberOfIterations, convergenceMinimumValue, convergenceWindowSize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_6`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_6`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_6`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_6'))

# Start of ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch

`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_7` = function(self, learningRate, numberOfIterations, convergenceMinimumValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_7', self, learningRate, numberOfIterations, convergenceMinimumValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_7`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_7`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_7`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_7'))

# Start of ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch

`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_8` = function(self, learningRate, numberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_8', self, learningRate, numberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_8`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_8`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_8`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_8'))

`ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_8; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_7; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_6; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_5; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_4; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_3; 
    }
  } else if (argc == 9) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( (is.integer(argv[[9]]) || is.numeric(argv[[9]])) && length(argv[[9]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_2; 
    }
  } else if (argc == 10) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( (is.integer(argv[[9]]) || is.numeric(argv[[9]])) && length(argv[[9]]) == 1 ) && ( is.character(argv[[10]]) && length(argv[[10]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_1; 
    }
  } else if (argc == 11) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( (is.integer(argv[[9]]) || is.numeric(argv[[9]])) && length(argv[[9]]) == 1 ) && ( is.character(argv[[10]]) && length(argv[[10]]) == 1 ) && ( is.numeric(argv[[11]]) && length(argv[[11]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetOptimizerAsLBFGSB

`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_0` = function(self, gradientConvergenceTolerance, numberOfIterations, maximumNumberOfCorrections, maximumNumberOfFunctionEvaluations, costFunctionConvergenceFactor, lowerBound, upperBound, trace)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  maximumNumberOfCorrections = as.integer(maximumNumberOfCorrections);
  
  if(length(maximumNumberOfCorrections) > 1) {
    warning("using only the first element of maximumNumberOfCorrections");
  };
  
  maximumNumberOfFunctionEvaluations = as.integer(maximumNumberOfFunctionEvaluations);
  
  if(length(maximumNumberOfFunctionEvaluations) > 1) {
    warning("using only the first element of maximumNumberOfFunctionEvaluations");
  };
  
  
  
  
  trace = as.logical(trace);
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_0', self, gradientConvergenceTolerance, numberOfIterations, maximumNumberOfCorrections, maximumNumberOfFunctionEvaluations, costFunctionConvergenceFactor, lowerBound, upperBound, trace, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'integer', 'integer', 'numeric', 'numeric', 'numeric', 'logical')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_0'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGSB

`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_1` = function(self, gradientConvergenceTolerance, numberOfIterations, maximumNumberOfCorrections, maximumNumberOfFunctionEvaluations, costFunctionConvergenceFactor, lowerBound, upperBound)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  maximumNumberOfCorrections = as.integer(maximumNumberOfCorrections);
  
  if(length(maximumNumberOfCorrections) > 1) {
    warning("using only the first element of maximumNumberOfCorrections");
  };
  
  maximumNumberOfFunctionEvaluations = as.integer(maximumNumberOfFunctionEvaluations);
  
  if(length(maximumNumberOfFunctionEvaluations) > 1) {
    warning("using only the first element of maximumNumberOfFunctionEvaluations");
  };
  
  
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_1', self, gradientConvergenceTolerance, numberOfIterations, maximumNumberOfCorrections, maximumNumberOfFunctionEvaluations, costFunctionConvergenceFactor, lowerBound, upperBound, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'integer', 'integer', 'numeric', 'numeric', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_1'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGSB

`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_2` = function(self, gradientConvergenceTolerance, numberOfIterations, maximumNumberOfCorrections, maximumNumberOfFunctionEvaluations, costFunctionConvergenceFactor, lowerBound)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  maximumNumberOfCorrections = as.integer(maximumNumberOfCorrections);
  
  if(length(maximumNumberOfCorrections) > 1) {
    warning("using only the first element of maximumNumberOfCorrections");
  };
  
  maximumNumberOfFunctionEvaluations = as.integer(maximumNumberOfFunctionEvaluations);
  
  if(length(maximumNumberOfFunctionEvaluations) > 1) {
    warning("using only the first element of maximumNumberOfFunctionEvaluations");
  };
  
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_2', self, gradientConvergenceTolerance, numberOfIterations, maximumNumberOfCorrections, maximumNumberOfFunctionEvaluations, costFunctionConvergenceFactor, lowerBound, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_2`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_2`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'integer', 'integer', 'numeric', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_2`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_2'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGSB

`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_3` = function(self, gradientConvergenceTolerance, numberOfIterations, maximumNumberOfCorrections, maximumNumberOfFunctionEvaluations, costFunctionConvergenceFactor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  maximumNumberOfCorrections = as.integer(maximumNumberOfCorrections);
  
  if(length(maximumNumberOfCorrections) > 1) {
    warning("using only the first element of maximumNumberOfCorrections");
  };
  
  maximumNumberOfFunctionEvaluations = as.integer(maximumNumberOfFunctionEvaluations);
  
  if(length(maximumNumberOfFunctionEvaluations) > 1) {
    warning("using only the first element of maximumNumberOfFunctionEvaluations");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_3', self, gradientConvergenceTolerance, numberOfIterations, maximumNumberOfCorrections, maximumNumberOfFunctionEvaluations, costFunctionConvergenceFactor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_3`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_3`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'integer', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_3`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_3'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGSB

`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_4` = function(self, gradientConvergenceTolerance, numberOfIterations, maximumNumberOfCorrections, maximumNumberOfFunctionEvaluations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  maximumNumberOfCorrections = as.integer(maximumNumberOfCorrections);
  
  if(length(maximumNumberOfCorrections) > 1) {
    warning("using only the first element of maximumNumberOfCorrections");
  };
  
  maximumNumberOfFunctionEvaluations = as.integer(maximumNumberOfFunctionEvaluations);
  
  if(length(maximumNumberOfFunctionEvaluations) > 1) {
    warning("using only the first element of maximumNumberOfFunctionEvaluations");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_4', self, gradientConvergenceTolerance, numberOfIterations, maximumNumberOfCorrections, maximumNumberOfFunctionEvaluations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_4`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_4`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'integer', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_4`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_4'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGSB

`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_5` = function(self, gradientConvergenceTolerance, numberOfIterations, maximumNumberOfCorrections)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  maximumNumberOfCorrections = as.integer(maximumNumberOfCorrections);
  
  if(length(maximumNumberOfCorrections) > 1) {
    warning("using only the first element of maximumNumberOfCorrections");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_5', self, gradientConvergenceTolerance, numberOfIterations, maximumNumberOfCorrections, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_5`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_5`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_5`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_5'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGSB

`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_6` = function(self, gradientConvergenceTolerance, numberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_6', self, gradientConvergenceTolerance, numberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_6`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_6`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_6`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_6'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGSB

`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_7` = function(self, gradientConvergenceTolerance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_7', self, gradientConvergenceTolerance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_7`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_7`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_7`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_7'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGSB

`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_8` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_8', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_8`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_8`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_8`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_8'))

`ImageRegistrationMethod_SetOptimizerAsLBFGSB` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_8; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_7; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_6; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_5; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_4; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_3; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_2; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_1; 
    }
  } else if (argc == 9) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( is.logical(argv[[9]]) && length(argv[[9]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGSB__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetOptimizerAsLBFGSB with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetOptimizerAsLBFGS2

`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_0` = function(self, solutionAccuracy, numberOfIterations, hessianApproximateAccuracy, deltaConvergenceDistance, deltaConvergenceTolerance, lineSearchMaximumEvaluations, lineSearchMinimumStep, lineSearchMaximumStep, lineSearchAccuracy)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  hessianApproximateAccuracy = as.integer(hessianApproximateAccuracy);
  
  if(length(hessianApproximateAccuracy) > 1) {
    warning("using only the first element of hessianApproximateAccuracy");
  };
  
  deltaConvergenceDistance = as.integer(deltaConvergenceDistance);
  
  if(length(deltaConvergenceDistance) > 1) {
    warning("using only the first element of deltaConvergenceDistance");
  };
  
  
  lineSearchMaximumEvaluations = as.integer(lineSearchMaximumEvaluations);
  
  if(length(lineSearchMaximumEvaluations) > 1) {
    warning("using only the first element of lineSearchMaximumEvaluations");
  };
  
  
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_0', self, solutionAccuracy, numberOfIterations, hessianApproximateAccuracy, deltaConvergenceDistance, deltaConvergenceTolerance, lineSearchMaximumEvaluations, lineSearchMinimumStep, lineSearchMaximumStep, lineSearchAccuracy, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'integer', 'integer', 'numeric', 'integer', 'numeric', 'numeric', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_0'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGS2

`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_1` = function(self, solutionAccuracy, numberOfIterations, hessianApproximateAccuracy, deltaConvergenceDistance, deltaConvergenceTolerance, lineSearchMaximumEvaluations, lineSearchMinimumStep, lineSearchMaximumStep)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  hessianApproximateAccuracy = as.integer(hessianApproximateAccuracy);
  
  if(length(hessianApproximateAccuracy) > 1) {
    warning("using only the first element of hessianApproximateAccuracy");
  };
  
  deltaConvergenceDistance = as.integer(deltaConvergenceDistance);
  
  if(length(deltaConvergenceDistance) > 1) {
    warning("using only the first element of deltaConvergenceDistance");
  };
  
  
  lineSearchMaximumEvaluations = as.integer(lineSearchMaximumEvaluations);
  
  if(length(lineSearchMaximumEvaluations) > 1) {
    warning("using only the first element of lineSearchMaximumEvaluations");
  };
  
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_1', self, solutionAccuracy, numberOfIterations, hessianApproximateAccuracy, deltaConvergenceDistance, deltaConvergenceTolerance, lineSearchMaximumEvaluations, lineSearchMinimumStep, lineSearchMaximumStep, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'integer', 'integer', 'numeric', 'integer', 'numeric', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_1'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGS2

`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_2` = function(self, solutionAccuracy, numberOfIterations, hessianApproximateAccuracy, deltaConvergenceDistance, deltaConvergenceTolerance, lineSearchMaximumEvaluations, lineSearchMinimumStep)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  hessianApproximateAccuracy = as.integer(hessianApproximateAccuracy);
  
  if(length(hessianApproximateAccuracy) > 1) {
    warning("using only the first element of hessianApproximateAccuracy");
  };
  
  deltaConvergenceDistance = as.integer(deltaConvergenceDistance);
  
  if(length(deltaConvergenceDistance) > 1) {
    warning("using only the first element of deltaConvergenceDistance");
  };
  
  
  lineSearchMaximumEvaluations = as.integer(lineSearchMaximumEvaluations);
  
  if(length(lineSearchMaximumEvaluations) > 1) {
    warning("using only the first element of lineSearchMaximumEvaluations");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_2', self, solutionAccuracy, numberOfIterations, hessianApproximateAccuracy, deltaConvergenceDistance, deltaConvergenceTolerance, lineSearchMaximumEvaluations, lineSearchMinimumStep, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_2`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_2`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'integer', 'integer', 'numeric', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_2`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_2'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGS2

`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_3` = function(self, solutionAccuracy, numberOfIterations, hessianApproximateAccuracy, deltaConvergenceDistance, deltaConvergenceTolerance, lineSearchMaximumEvaluations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  hessianApproximateAccuracy = as.integer(hessianApproximateAccuracy);
  
  if(length(hessianApproximateAccuracy) > 1) {
    warning("using only the first element of hessianApproximateAccuracy");
  };
  
  deltaConvergenceDistance = as.integer(deltaConvergenceDistance);
  
  if(length(deltaConvergenceDistance) > 1) {
    warning("using only the first element of deltaConvergenceDistance");
  };
  
  
  lineSearchMaximumEvaluations = as.integer(lineSearchMaximumEvaluations);
  
  if(length(lineSearchMaximumEvaluations) > 1) {
    warning("using only the first element of lineSearchMaximumEvaluations");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_3', self, solutionAccuracy, numberOfIterations, hessianApproximateAccuracy, deltaConvergenceDistance, deltaConvergenceTolerance, lineSearchMaximumEvaluations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_3`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_3`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'integer', 'integer', 'numeric', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_3`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_3'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGS2

`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_4` = function(self, solutionAccuracy, numberOfIterations, hessianApproximateAccuracy, deltaConvergenceDistance, deltaConvergenceTolerance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  hessianApproximateAccuracy = as.integer(hessianApproximateAccuracy);
  
  if(length(hessianApproximateAccuracy) > 1) {
    warning("using only the first element of hessianApproximateAccuracy");
  };
  
  deltaConvergenceDistance = as.integer(deltaConvergenceDistance);
  
  if(length(deltaConvergenceDistance) > 1) {
    warning("using only the first element of deltaConvergenceDistance");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_4', self, solutionAccuracy, numberOfIterations, hessianApproximateAccuracy, deltaConvergenceDistance, deltaConvergenceTolerance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_4`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_4`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'integer', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_4`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_4'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGS2

`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_5` = function(self, solutionAccuracy, numberOfIterations, hessianApproximateAccuracy, deltaConvergenceDistance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  hessianApproximateAccuracy = as.integer(hessianApproximateAccuracy);
  
  if(length(hessianApproximateAccuracy) > 1) {
    warning("using only the first element of hessianApproximateAccuracy");
  };
  
  deltaConvergenceDistance = as.integer(deltaConvergenceDistance);
  
  if(length(deltaConvergenceDistance) > 1) {
    warning("using only the first element of deltaConvergenceDistance");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_5', self, solutionAccuracy, numberOfIterations, hessianApproximateAccuracy, deltaConvergenceDistance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_5`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_5`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'integer', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_5`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_5'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGS2

`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_6` = function(self, solutionAccuracy, numberOfIterations, hessianApproximateAccuracy)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  hessianApproximateAccuracy = as.integer(hessianApproximateAccuracy);
  
  if(length(hessianApproximateAccuracy) > 1) {
    warning("using only the first element of hessianApproximateAccuracy");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_6', self, solutionAccuracy, numberOfIterations, hessianApproximateAccuracy, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_6`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_6`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_6`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_6'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGS2

`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_7` = function(self, solutionAccuracy, numberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_7', self, solutionAccuracy, numberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_7`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_7`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_7`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_7'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGS2

`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_8` = function(self, solutionAccuracy)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_8', self, solutionAccuracy, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_8`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_8`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_8`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_8'))

# Start of ImageRegistrationMethod_SetOptimizerAsLBFGS2

`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_9` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_9', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_9`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_9`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_9`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_9'))

`ImageRegistrationMethod_SetOptimizerAsLBFGS2` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 10
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_9; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_8; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_7; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_6; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_5; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_4; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_3; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_2; 
    }
  } else if (argc == 9) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( is.numeric(argv[[9]]) && length(argv[[9]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_1; 
    }
  } else if (argc == 10) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( is.numeric(argv[[9]]) && length(argv[[9]]) == 1 ) && ( is.numeric(argv[[10]]) && length(argv[[10]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsLBFGS2__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetOptimizerAsLBFGS2 with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetOptimizerAsExhaustive

`ImageRegistrationMethod_SetOptimizerAsExhaustive__SWIG_0` = function(self, numberOfSteps, stepLength)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfSteps = as.integer(numberOfSteps);
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsExhaustive__SWIG_0', self, numberOfSteps, stepLength, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsExhaustive__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsExhaustive__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsExhaustive__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsExhaustive__SWIG_0'))

# Start of ImageRegistrationMethod_SetOptimizerAsExhaustive

`ImageRegistrationMethod_SetOptimizerAsExhaustive__SWIG_1` = function(self, numberOfSteps)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfSteps = as.integer(numberOfSteps);
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsExhaustive__SWIG_1', self, numberOfSteps, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsExhaustive__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsExhaustive__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsExhaustive__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsExhaustive__SWIG_1'))

`ImageRegistrationMethod_SetOptimizerAsExhaustive` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- ImageRegistrationMethod_SetOptimizerAsExhaustive__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsExhaustive__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetOptimizerAsExhaustive with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetOptimizerAsAmoeba

`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_0` = function(self, simplexDelta, numberOfIterations, parametersConvergenceTolerance, functionConvergenceTolerance, withRestarts)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  withRestarts = as.logical(withRestarts);
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_0', self, simplexDelta, numberOfIterations, parametersConvergenceTolerance, functionConvergenceTolerance, withRestarts, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'numeric', 'logical')
class(`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_0'))

# Start of ImageRegistrationMethod_SetOptimizerAsAmoeba

`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_1` = function(self, simplexDelta, numberOfIterations, parametersConvergenceTolerance, functionConvergenceTolerance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_1', self, simplexDelta, numberOfIterations, parametersConvergenceTolerance, functionConvergenceTolerance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_1'))

# Start of ImageRegistrationMethod_SetOptimizerAsAmoeba

`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_2` = function(self, simplexDelta, numberOfIterations, parametersConvergenceTolerance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_2', self, simplexDelta, numberOfIterations, parametersConvergenceTolerance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_2`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_2`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_2`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_2'))

# Start of ImageRegistrationMethod_SetOptimizerAsAmoeba

`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_3` = function(self, simplexDelta, numberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_3', self, simplexDelta, numberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_3`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_3`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_3`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_3'))

`ImageRegistrationMethod_SetOptimizerAsAmoeba` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsAmoeba__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetOptimizerAsAmoeba with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetOptimizerWeights

`ImageRegistrationMethod_SetOptimizerWeights` = function(self, weights)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  weights = as.numeric(weights);
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerWeights', self, weights, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerWeights`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerWeights`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_std__vectorT_double_t')
class(`ImageRegistrationMethod_SetOptimizerWeights`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerWeights'))

# Start of ImageRegistrationMethod_GetOptimizerWeights

`ImageRegistrationMethod_GetOptimizerWeights` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_GetOptimizerWeights', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_GetOptimizerWeights`, 'returnType') = 'numeric'
attr(`ImageRegistrationMethod_GetOptimizerWeights`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetOptimizerWeights`) = c("SWIGFunction", class('ImageRegistrationMethod_GetOptimizerWeights'))

# Start of ImageRegistrationMethod_SetOptimizerAsPowell

`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_0` = function(self, numberOfIterations, maximumLineIterations, stepLength, stepTolerance, valueTolerance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  maximumLineIterations = as.integer(maximumLineIterations);
  
  if(length(maximumLineIterations) > 1) {
    warning("using only the first element of maximumLineIterations");
  };
  
  
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_0', self, numberOfIterations, maximumLineIterations, stepLength, stepTolerance, valueTolerance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer', 'integer', 'numeric', 'numeric', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_0'))

# Start of ImageRegistrationMethod_SetOptimizerAsPowell

`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_1` = function(self, numberOfIterations, maximumLineIterations, stepLength, stepTolerance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  maximumLineIterations = as.integer(maximumLineIterations);
  
  if(length(maximumLineIterations) > 1) {
    warning("using only the first element of maximumLineIterations");
  };
  
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_1', self, numberOfIterations, maximumLineIterations, stepLength, stepTolerance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer', 'integer', 'numeric', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_1'))

# Start of ImageRegistrationMethod_SetOptimizerAsPowell

`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_2` = function(self, numberOfIterations, maximumLineIterations, stepLength)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  maximumLineIterations = as.integer(maximumLineIterations);
  
  if(length(maximumLineIterations) > 1) {
    warning("using only the first element of maximumLineIterations");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_2', self, numberOfIterations, maximumLineIterations, stepLength, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_2`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_2`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_2`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_2'))

# Start of ImageRegistrationMethod_SetOptimizerAsPowell

`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_3` = function(self, numberOfIterations, maximumLineIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  maximumLineIterations = as.integer(maximumLineIterations);
  
  if(length(maximumLineIterations) > 1) {
    warning("using only the first element of maximumLineIterations");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_3', self, numberOfIterations, maximumLineIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_3`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_3`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_3`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_3'))

# Start of ImageRegistrationMethod_SetOptimizerAsPowell

`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_4` = function(self, numberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_4', self, numberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_4`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_4`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_4`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_4'))

# Start of ImageRegistrationMethod_SetOptimizerAsPowell

`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_5` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_5', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_5`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_5`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_5`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_5'))

`ImageRegistrationMethod_SetOptimizerAsPowell` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) {
      f <- ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsPowell__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetOptimizerAsPowell with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary

`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_0` = function(self, numberOfIterations, epsilon, initialRadius, growthFactor, shrinkFactor, seed)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  
  
  seed = as.integer(seed);
  
  if(length(seed) > 1) {
    warning("using only the first element of seed");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_0', self, numberOfIterations, epsilon, initialRadius, growthFactor, shrinkFactor, seed, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_0'))

# Start of ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary

`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_1` = function(self, numberOfIterations, epsilon, initialRadius, growthFactor, shrinkFactor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_1', self, numberOfIterations, epsilon, initialRadius, growthFactor, shrinkFactor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer', 'numeric', 'numeric', 'numeric', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_1'))

# Start of ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary

`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_2` = function(self, numberOfIterations, epsilon, initialRadius, growthFactor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_2', self, numberOfIterations, epsilon, initialRadius, growthFactor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_2`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_2`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer', 'numeric', 'numeric', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_2`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_2'))

# Start of ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary

`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_3` = function(self, numberOfIterations, epsilon, initialRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_3', self, numberOfIterations, epsilon, initialRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_3`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_3`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer', 'numeric', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_3`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_3'))

# Start of ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary

`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_4` = function(self, numberOfIterations, epsilon)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_4', self, numberOfIterations, epsilon, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_4`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_4`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_4`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_4'))

# Start of ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary

`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_5` = function(self, numberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_5', self, numberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_5`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_5`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer')
class(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_5`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_5'))

# Start of ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary

`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_6` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_6', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_6`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_6`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_6`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_6'))

`ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 7
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) {
      f <- ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_6; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_5; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_2; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_1; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetOptimizerScales

`ImageRegistrationMethod_SetOptimizerScales` = function(self, scales)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  scales = as.numeric(scales);
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerScales', self, scales, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerScales`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerScales`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_std__vectorT_double_t')
class(`ImageRegistrationMethod_SetOptimizerScales`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerScales'))

# Start of ImageRegistrationMethod_SetOptimizerScalesFromJacobian

`ImageRegistrationMethod_SetOptimizerScalesFromJacobian__SWIG_0` = function(self, centralRegionRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  centralRegionRadius = as.integer(centralRegionRadius);
  
  if(length(centralRegionRadius) > 1) {
    warning("using only the first element of centralRegionRadius");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerScalesFromJacobian__SWIG_0', self, centralRegionRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerScalesFromJacobian__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerScalesFromJacobian__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer')
class(`ImageRegistrationMethod_SetOptimizerScalesFromJacobian__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerScalesFromJacobian__SWIG_0'))

# Start of ImageRegistrationMethod_SetOptimizerScalesFromJacobian

`ImageRegistrationMethod_SetOptimizerScalesFromJacobian__SWIG_1` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerScalesFromJacobian__SWIG_1', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerScalesFromJacobian__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerScalesFromJacobian__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_SetOptimizerScalesFromJacobian__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerScalesFromJacobian__SWIG_1'))

`ImageRegistrationMethod_SetOptimizerScalesFromJacobian` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) {
      f <- ImageRegistrationMethod_SetOptimizerScalesFromJacobian__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerScalesFromJacobian__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetOptimizerScalesFromJacobian with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetOptimizerScalesFromIndexShift

`ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_0` = function(self, centralRegionRadius, smallParameterVariation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  centralRegionRadius = as.integer(centralRegionRadius);
  
  if(length(centralRegionRadius) > 1) {
    warning("using only the first element of centralRegionRadius");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_0', self, centralRegionRadius, smallParameterVariation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_0'))

# Start of ImageRegistrationMethod_SetOptimizerScalesFromIndexShift

`ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_1` = function(self, centralRegionRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  centralRegionRadius = as.integer(centralRegionRadius);
  
  if(length(centralRegionRadius) > 1) {
    warning("using only the first element of centralRegionRadius");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_1', self, centralRegionRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer')
class(`ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_1'))

# Start of ImageRegistrationMethod_SetOptimizerScalesFromIndexShift

`ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_2` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_2', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_2`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_2`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_2`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_2'))

`ImageRegistrationMethod_SetOptimizerScalesFromIndexShift` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) {
      f <- ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerScalesFromIndexShift__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetOptimizerScalesFromIndexShift with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift

`ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_0` = function(self, centralRegionRadius, smallParameterVariation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  centralRegionRadius = as.integer(centralRegionRadius);
  
  if(length(centralRegionRadius) > 1) {
    warning("using only the first element of centralRegionRadius");
  };
  
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_0', self, centralRegionRadius, smallParameterVariation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer', 'numeric')
class(`ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_0'))

# Start of ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift

`ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_1` = function(self, centralRegionRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  centralRegionRadius = as.integer(centralRegionRadius);
  
  if(length(centralRegionRadius) > 1) {
    warning("using only the first element of centralRegionRadius");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_1', self, centralRegionRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer')
class(`ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_1'))

# Start of ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift

`ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_2` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_2', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_2`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_2`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_2`) = c("SWIGFunction", class('ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_2'))

`ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) {
      f <- ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetMetricFixedMask

`ImageRegistrationMethod_SetMetricFixedMask` = function(self, binaryMask)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(binaryMask, "ExternalReference")) binaryMask = slot(binaryMask,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricFixedMask', self, binaryMask, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricFixedMask`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricFixedMask`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_itk__simple__Image')
class(`ImageRegistrationMethod_SetMetricFixedMask`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricFixedMask'))

# Start of ImageRegistrationMethod_SetMetricMovingMask

`ImageRegistrationMethod_SetMetricMovingMask` = function(self, binaryMask)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(binaryMask, "ExternalReference")) binaryMask = slot(binaryMask,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricMovingMask', self, binaryMask, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricMovingMask`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricMovingMask`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_itk__simple__Image')
class(`ImageRegistrationMethod_SetMetricMovingMask`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricMovingMask'))

# Start of ImageRegistrationMethod_SetMetricSamplingPercentage

`ImageRegistrationMethod_SetMetricSamplingPercentage__SWIG_0` = function(self, percentage, seed)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  seed = as.integer(seed);
  
  if(length(seed) > 1) {
    warning("using only the first element of seed");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricSamplingPercentage__SWIG_0', self, percentage, seed, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricSamplingPercentage__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricSamplingPercentage__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric', 'integer')
class(`ImageRegistrationMethod_SetMetricSamplingPercentage__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricSamplingPercentage__SWIG_0'))

# Start of ImageRegistrationMethod_SetMetricSamplingPercentage

`ImageRegistrationMethod_SetMetricSamplingPercentage__SWIG_1` = function(self, percentage)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricSamplingPercentage__SWIG_1', self, percentage, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricSamplingPercentage__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricSamplingPercentage__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'numeric')
class(`ImageRegistrationMethod_SetMetricSamplingPercentage__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricSamplingPercentage__SWIG_1'))

`ImageRegistrationMethod_SetMetricSamplingPercentage` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ImageRegistrationMethod_SetMetricSamplingPercentage__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetMetricSamplingPercentage__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetMetricSamplingPercentage with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel

`ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel__SWIG_0` = function(self, percentage, seed)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  percentage = as.numeric(percentage);
  seed = as.integer(seed);
  
  if(length(seed) > 1) {
    warning("using only the first element of seed");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel__SWIG_0', self, percentage, seed, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel__SWIG_0`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel__SWIG_0`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_std__vectorT_double_t', 'integer')
class(`ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel__SWIG_0`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel__SWIG_0'))

# Start of ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel

`ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel__SWIG_1` = function(self, percentage)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  percentage = as.numeric(percentage);
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel__SWIG_1', self, percentage, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel__SWIG_1`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel__SWIG_1`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_std__vectorT_double_t')
class(`ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel__SWIG_1`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel__SWIG_1'))

`ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ImageRegistrationMethod') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ImageRegistrationMethod_GetMetricSamplingPercentagePerLevel

`ImageRegistrationMethod_GetMetricSamplingPercentagePerLevel` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_GetMetricSamplingPercentagePerLevel', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_GetMetricSamplingPercentagePerLevel`, 'returnType') = '_p_std__vectorT_double_t'
attr(`ImageRegistrationMethod_GetMetricSamplingPercentagePerLevel`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetMetricSamplingPercentagePerLevel`) = c("SWIGFunction", class('ImageRegistrationMethod_GetMetricSamplingPercentagePerLevel'))

# Start of ImageRegistrationMethod_NONE_get

`ImageRegistrationMethod_NONE_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ImageRegistrationMethod_NONE_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ImageRegistrationMethod__MetricSamplingStrategyType");
  
  ans
  
}

attr(`ImageRegistrationMethod_NONE_get`, 'returnType') = 'character'
class(`ImageRegistrationMethod_NONE_get`) = c("SWIGFunction", class('ImageRegistrationMethod_NONE_get'))

# Start of ImageRegistrationMethod_REGULAR_get

`ImageRegistrationMethod_REGULAR_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ImageRegistrationMethod_REGULAR_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ImageRegistrationMethod__MetricSamplingStrategyType");
  
  ans
  
}

attr(`ImageRegistrationMethod_REGULAR_get`, 'returnType') = 'character'
class(`ImageRegistrationMethod_REGULAR_get`) = c("SWIGFunction", class('ImageRegistrationMethod_REGULAR_get'))

# Start of ImageRegistrationMethod_RANDOM_get

`ImageRegistrationMethod_RANDOM_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ImageRegistrationMethod_RANDOM_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ImageRegistrationMethod__MetricSamplingStrategyType");
  
  ans
  
}

attr(`ImageRegistrationMethod_RANDOM_get`, 'returnType') = 'character'
class(`ImageRegistrationMethod_RANDOM_get`) = c("SWIGFunction", class('ImageRegistrationMethod_RANDOM_get'))

# Start of ImageRegistrationMethod_SetMetricSamplingStrategy

`ImageRegistrationMethod_SetMetricSamplingStrategy` = function(self, strategy)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  strategy = enumToInteger(strategy, "_itk__simple__ImageRegistrationMethod__MetricSamplingStrategyType"); 
  
  if(length(strategy) > 1) {
    warning("using only the first element of strategy");
  };
  
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricSamplingStrategy', self, strategy, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricSamplingStrategy`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricSamplingStrategy`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'character')
class(`ImageRegistrationMethod_SetMetricSamplingStrategy`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricSamplingStrategy'))

# Start of ImageRegistrationMethod_SetMetricUseFixedImageGradientFilter

`ImageRegistrationMethod_SetMetricUseFixedImageGradientFilter` = function(self, s_arg2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  s_arg2 = as.logical(s_arg2);
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricUseFixedImageGradientFilter', self, s_arg2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricUseFixedImageGradientFilter`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricUseFixedImageGradientFilter`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'logical')
class(`ImageRegistrationMethod_SetMetricUseFixedImageGradientFilter`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricUseFixedImageGradientFilter'))

# Start of ImageRegistrationMethod_MetricUseFixedImageGradientFilterOn

`ImageRegistrationMethod_MetricUseFixedImageGradientFilterOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_MetricUseFixedImageGradientFilterOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_MetricUseFixedImageGradientFilterOn`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_MetricUseFixedImageGradientFilterOn`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_MetricUseFixedImageGradientFilterOn`) = c("SWIGFunction", class('ImageRegistrationMethod_MetricUseFixedImageGradientFilterOn'))

# Start of ImageRegistrationMethod_MetricUseFixedImageGradientFilterOff

`ImageRegistrationMethod_MetricUseFixedImageGradientFilterOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_MetricUseFixedImageGradientFilterOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_MetricUseFixedImageGradientFilterOff`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_MetricUseFixedImageGradientFilterOff`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_MetricUseFixedImageGradientFilterOff`) = c("SWIGFunction", class('ImageRegistrationMethod_MetricUseFixedImageGradientFilterOff'))

# Start of ImageRegistrationMethod_SetMetricUseMovingImageGradientFilter

`ImageRegistrationMethod_SetMetricUseMovingImageGradientFilter` = function(self, s_arg2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  s_arg2 = as.logical(s_arg2);
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetMetricUseMovingImageGradientFilter', self, s_arg2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetMetricUseMovingImageGradientFilter`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetMetricUseMovingImageGradientFilter`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'logical')
class(`ImageRegistrationMethod_SetMetricUseMovingImageGradientFilter`) = c("SWIGFunction", class('ImageRegistrationMethod_SetMetricUseMovingImageGradientFilter'))

# Start of ImageRegistrationMethod_MetricUseMovingImageGradientFilterOn

`ImageRegistrationMethod_MetricUseMovingImageGradientFilterOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_MetricUseMovingImageGradientFilterOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_MetricUseMovingImageGradientFilterOn`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_MetricUseMovingImageGradientFilterOn`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_MetricUseMovingImageGradientFilterOn`) = c("SWIGFunction", class('ImageRegistrationMethod_MetricUseMovingImageGradientFilterOn'))

# Start of ImageRegistrationMethod_MetricUseMovingImageGradientFilterOff

`ImageRegistrationMethod_MetricUseMovingImageGradientFilterOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_MetricUseMovingImageGradientFilterOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_MetricUseMovingImageGradientFilterOff`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_MetricUseMovingImageGradientFilterOff`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_MetricUseMovingImageGradientFilterOff`) = c("SWIGFunction", class('ImageRegistrationMethod_MetricUseMovingImageGradientFilterOff'))

# Start of ImageRegistrationMethod_SetShrinkFactorsPerLevel

`ImageRegistrationMethod_SetShrinkFactorsPerLevel` = function(self, shrinkFactors)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  shrinkFactors = as.integer(shrinkFactors);
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetShrinkFactorsPerLevel', self, shrinkFactors, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetShrinkFactorsPerLevel`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetShrinkFactorsPerLevel`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'integer')
class(`ImageRegistrationMethod_SetShrinkFactorsPerLevel`) = c("SWIGFunction", class('ImageRegistrationMethod_SetShrinkFactorsPerLevel'))

# Start of ImageRegistrationMethod_SetSmoothingSigmasPerLevel

`ImageRegistrationMethod_SetSmoothingSigmasPerLevel` = function(self, smoothingSigmas)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  smoothingSigmas = as.numeric(smoothingSigmas);
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetSmoothingSigmasPerLevel', self, smoothingSigmas, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetSmoothingSigmasPerLevel`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetSmoothingSigmasPerLevel`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_std__vectorT_double_t')
class(`ImageRegistrationMethod_SetSmoothingSigmasPerLevel`) = c("SWIGFunction", class('ImageRegistrationMethod_SetSmoothingSigmasPerLevel'))

# Start of ImageRegistrationMethod_SetSmoothingSigmasAreSpecifiedInPhysicalUnits

`ImageRegistrationMethod_SetSmoothingSigmasAreSpecifiedInPhysicalUnits` = function(self, s_arg)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  s_arg = as.logical(s_arg);
  ;ans = .Call('R_swig_ImageRegistrationMethod_SetSmoothingSigmasAreSpecifiedInPhysicalUnits', self, s_arg, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SetSmoothingSigmasAreSpecifiedInPhysicalUnits`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SetSmoothingSigmasAreSpecifiedInPhysicalUnits`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', 'logical')
class(`ImageRegistrationMethod_SetSmoothingSigmasAreSpecifiedInPhysicalUnits`) = c("SWIGFunction", class('ImageRegistrationMethod_SetSmoothingSigmasAreSpecifiedInPhysicalUnits'))

# Start of ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOn

`ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOn`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOn`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOn`) = c("SWIGFunction", class('ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOn'))

# Start of ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOff

`ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOff`, 'returnType') = 'void'
attr(`ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOff`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOff`) = c("SWIGFunction", class('ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOff'))

# Start of ImageRegistrationMethod_Execute

`ImageRegistrationMethod_Execute` = function(self, fixed, moving, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixed, "ExternalReference")) fixed = slot(fixed,"ref"); 
  if (inherits(moving, "ExternalReference")) moving = slot(moving,"ref"); 
  ;ans = .Call('R_swig_ImageRegistrationMethod_Execute', self, fixed, moving, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ImageRegistrationMethod_Execute`, 'returnType') = '_p_itk__simple__Transform'
attr(`ImageRegistrationMethod_Execute`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ImageRegistrationMethod_Execute`) = c("SWIGFunction", class('ImageRegistrationMethod_Execute'))

# Start of ImageRegistrationMethod_MetricEvaluate

`ImageRegistrationMethod_MetricEvaluate` = function(self, fixed, moving, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixed, "ExternalReference")) fixed = slot(fixed,"ref"); 
  if (inherits(moving, "ExternalReference")) moving = slot(moving,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_MetricEvaluate', self, fixed, moving, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_MetricEvaluate`, 'returnType') = 'numeric'
attr(`ImageRegistrationMethod_MetricEvaluate`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ImageRegistrationMethod_MetricEvaluate`) = c("SWIGFunction", class('ImageRegistrationMethod_MetricEvaluate'))

# Start of ImageRegistrationMethod_GetOptimizerIteration

`ImageRegistrationMethod_GetOptimizerIteration` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_GetOptimizerIteration', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_GetOptimizerIteration`, 'returnType') = 'integer'
attr(`ImageRegistrationMethod_GetOptimizerIteration`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetOptimizerIteration`) = c("SWIGFunction", class('ImageRegistrationMethod_GetOptimizerIteration'))

# Start of ImageRegistrationMethod_GetOptimizerPosition

`ImageRegistrationMethod_GetOptimizerPosition` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_GetOptimizerPosition', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_GetOptimizerPosition`, 'returnType') = 'numeric'
attr(`ImageRegistrationMethod_GetOptimizerPosition`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetOptimizerPosition`) = c("SWIGFunction", class('ImageRegistrationMethod_GetOptimizerPosition'))

# Start of ImageRegistrationMethod_GetOptimizerLearningRate

`ImageRegistrationMethod_GetOptimizerLearningRate` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_GetOptimizerLearningRate', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_GetOptimizerLearningRate`, 'returnType') = 'numeric'
attr(`ImageRegistrationMethod_GetOptimizerLearningRate`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetOptimizerLearningRate`) = c("SWIGFunction", class('ImageRegistrationMethod_GetOptimizerLearningRate'))

# Start of ImageRegistrationMethod_GetOptimizerConvergenceValue

`ImageRegistrationMethod_GetOptimizerConvergenceValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_GetOptimizerConvergenceValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_GetOptimizerConvergenceValue`, 'returnType') = 'numeric'
attr(`ImageRegistrationMethod_GetOptimizerConvergenceValue`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetOptimizerConvergenceValue`) = c("SWIGFunction", class('ImageRegistrationMethod_GetOptimizerConvergenceValue'))

# Start of ImageRegistrationMethod_GetMetricValue

`ImageRegistrationMethod_GetMetricValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_GetMetricValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_GetMetricValue`, 'returnType') = 'numeric'
attr(`ImageRegistrationMethod_GetMetricValue`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetMetricValue`) = c("SWIGFunction", class('ImageRegistrationMethod_GetMetricValue'))

# Start of ImageRegistrationMethod_GetMetricNumberOfValidPoints

`ImageRegistrationMethod_GetMetricNumberOfValidPoints` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_GetMetricNumberOfValidPoints', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_GetMetricNumberOfValidPoints`, 'returnType') = 'integer'
attr(`ImageRegistrationMethod_GetMetricNumberOfValidPoints`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetMetricNumberOfValidPoints`) = c("SWIGFunction", class('ImageRegistrationMethod_GetMetricNumberOfValidPoints'))

# Start of ImageRegistrationMethod_GetCurrentLevel

`ImageRegistrationMethod_GetCurrentLevel` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_GetCurrentLevel', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_GetCurrentLevel`, 'returnType') = 'integer'
attr(`ImageRegistrationMethod_GetCurrentLevel`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetCurrentLevel`) = c("SWIGFunction", class('ImageRegistrationMethod_GetCurrentLevel'))

# Start of ImageRegistrationMethod_GetOptimizerScales

`ImageRegistrationMethod_GetOptimizerScales` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_GetOptimizerScales', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_GetOptimizerScales`, 'returnType') = 'numeric'
attr(`ImageRegistrationMethod_GetOptimizerScales`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetOptimizerScales`) = c("SWIGFunction", class('ImageRegistrationMethod_GetOptimizerScales'))

# Start of ImageRegistrationMethod_GetOptimizerStopConditionDescription

`ImageRegistrationMethod_GetOptimizerStopConditionDescription` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_GetOptimizerStopConditionDescription', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_GetOptimizerStopConditionDescription`, 'returnType') = 'character'
attr(`ImageRegistrationMethod_GetOptimizerStopConditionDescription`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_GetOptimizerStopConditionDescription`) = c("SWIGFunction", class('ImageRegistrationMethod_GetOptimizerStopConditionDescription'))

# Start of ImageRegistrationMethod_StopRegistration

`ImageRegistrationMethod_StopRegistration` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ImageRegistrationMethod_StopRegistration', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ImageRegistrationMethod_StopRegistration`, 'returnType') = 'logical'
attr(`ImageRegistrationMethod_StopRegistration`, "inputTypes") = c('_p_itk__simple__ImageRegistrationMethod')
class(`ImageRegistrationMethod_StopRegistration`) = c("SWIGFunction", class('ImageRegistrationMethod_StopRegistration'))

# Start of accessor method for itk::simple::ImageRegistrationMethod
setMethod('$', '_p_itk__simple__ImageRegistrationMethod', function(x, name)

{
  accessorFuns = list('GetName' = ImageRegistrationMethod_GetName, 'ToString' = ImageRegistrationMethod_ToString, 'GetInterpolator' = ImageRegistrationMethod_GetInterpolator, 'SetInterpolator' = ImageRegistrationMethod_SetInterpolator, 'SetInitialTransform' = ImageRegistrationMethod_SetInitialTransform, 'GetInitialTransform' = ImageRegistrationMethod_GetInitialTransform, 'GetInitialTransformInPlace' = ImageRegistrationMethod_GetInitialTransformInPlace, 'SetInitialTransformAsBSpline' = ImageRegistrationMethod_SetInitialTransformAsBSpline, 'SetMovingInitialTransform' = ImageRegistrationMethod_SetMovingInitialTransform, 'GetMovingInitialTransform' = ImageRegistrationMethod_GetMovingInitialTransform, 'SetFixedInitialTransform' = ImageRegistrationMethod_SetFixedInitialTransform, 'GetFixedInitialTransform' = ImageRegistrationMethod_GetFixedInitialTransform, 'SetVirtualDomain' = ImageRegistrationMethod_SetVirtualDomain, 'SetVirtualDomainFromImage' = ImageRegistrationMethod_SetVirtualDomainFromImage, 'SetMetricAsANTSNeighborhoodCorrelation' = ImageRegistrationMethod_SetMetricAsANTSNeighborhoodCorrelation, 'SetMetricAsCorrelation' = ImageRegistrationMethod_SetMetricAsCorrelation, 'SetMetricAsDemons' = ImageRegistrationMethod_SetMetricAsDemons, 'SetMetricAsJointHistogramMutualInformation' = ImageRegistrationMethod_SetMetricAsJointHistogramMutualInformation, 'SetMetricAsMeanSquares' = ImageRegistrationMethod_SetMetricAsMeanSquares, 'SetMetricAsMattesMutualInformation' = ImageRegistrationMethod_SetMetricAsMattesMutualInformation, 'SetOptimizerAsConjugateGradientLineSearch' = ImageRegistrationMethod_SetOptimizerAsConjugateGradientLineSearch, 'SetOptimizerAsRegularStepGradientDescent' = ImageRegistrationMethod_SetOptimizerAsRegularStepGradientDescent, 'SetOptimizerAsGradientDescent' = ImageRegistrationMethod_SetOptimizerAsGradientDescent, 'SetOptimizerAsGradientDescentLineSearch' = ImageRegistrationMethod_SetOptimizerAsGradientDescentLineSearch, 'SetOptimizerAsLBFGSB' = ImageRegistrationMethod_SetOptimizerAsLBFGSB, 'SetOptimizerAsLBFGS2' = ImageRegistrationMethod_SetOptimizerAsLBFGS2, 'SetOptimizerAsExhaustive' = ImageRegistrationMethod_SetOptimizerAsExhaustive, 'SetOptimizerAsAmoeba' = ImageRegistrationMethod_SetOptimizerAsAmoeba, 'SetOptimizerWeights' = ImageRegistrationMethod_SetOptimizerWeights, 'GetOptimizerWeights' = ImageRegistrationMethod_GetOptimizerWeights, 'SetOptimizerAsPowell' = ImageRegistrationMethod_SetOptimizerAsPowell, 'SetOptimizerAsOnePlusOneEvolutionary' = ImageRegistrationMethod_SetOptimizerAsOnePlusOneEvolutionary, 'SetOptimizerScales' = ImageRegistrationMethod_SetOptimizerScales, 'SetOptimizerScalesFromJacobian' = ImageRegistrationMethod_SetOptimizerScalesFromJacobian, 'SetOptimizerScalesFromIndexShift' = ImageRegistrationMethod_SetOptimizerScalesFromIndexShift, 'SetOptimizerScalesFromPhysicalShift' = ImageRegistrationMethod_SetOptimizerScalesFromPhysicalShift, 'SetMetricFixedMask' = ImageRegistrationMethod_SetMetricFixedMask, 'SetMetricMovingMask' = ImageRegistrationMethod_SetMetricMovingMask, 'SetMetricSamplingPercentage' = ImageRegistrationMethod_SetMetricSamplingPercentage, 'SetMetricSamplingPercentagePerLevel' = ImageRegistrationMethod_SetMetricSamplingPercentagePerLevel, 'GetMetricSamplingPercentagePerLevel' = ImageRegistrationMethod_GetMetricSamplingPercentagePerLevel, 'SetMetricSamplingStrategy' = ImageRegistrationMethod_SetMetricSamplingStrategy, 'SetMetricUseFixedImageGradientFilter' = ImageRegistrationMethod_SetMetricUseFixedImageGradientFilter, 'MetricUseFixedImageGradientFilterOn' = ImageRegistrationMethod_MetricUseFixedImageGradientFilterOn, 'MetricUseFixedImageGradientFilterOff' = ImageRegistrationMethod_MetricUseFixedImageGradientFilterOff, 'SetMetricUseMovingImageGradientFilter' = ImageRegistrationMethod_SetMetricUseMovingImageGradientFilter, 'MetricUseMovingImageGradientFilterOn' = ImageRegistrationMethod_MetricUseMovingImageGradientFilterOn, 'MetricUseMovingImageGradientFilterOff' = ImageRegistrationMethod_MetricUseMovingImageGradientFilterOff, 'SetShrinkFactorsPerLevel' = ImageRegistrationMethod_SetShrinkFactorsPerLevel, 'SetSmoothingSigmasPerLevel' = ImageRegistrationMethod_SetSmoothingSigmasPerLevel, 'SetSmoothingSigmasAreSpecifiedInPhysicalUnits' = ImageRegistrationMethod_SetSmoothingSigmasAreSpecifiedInPhysicalUnits, 'SmoothingSigmasAreSpecifiedInPhysicalUnitsOn' = ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOn, 'SmoothingSigmasAreSpecifiedInPhysicalUnitsOff' = ImageRegistrationMethod_SmoothingSigmasAreSpecifiedInPhysicalUnitsOff, 'Execute' = ImageRegistrationMethod_Execute, 'MetricEvaluate' = ImageRegistrationMethod_MetricEvaluate, 'GetOptimizerIteration' = ImageRegistrationMethod_GetOptimizerIteration, 'GetOptimizerPosition' = ImageRegistrationMethod_GetOptimizerPosition, 'GetOptimizerLearningRate' = ImageRegistrationMethod_GetOptimizerLearningRate, 'GetOptimizerConvergenceValue' = ImageRegistrationMethod_GetOptimizerConvergenceValue, 'GetMetricValue' = ImageRegistrationMethod_GetMetricValue, 'GetMetricNumberOfValidPoints' = ImageRegistrationMethod_GetMetricNumberOfValidPoints, 'GetCurrentLevel' = ImageRegistrationMethod_GetCurrentLevel, 'GetOptimizerScales' = ImageRegistrationMethod_GetOptimizerScales, 'GetOptimizerStopConditionDescription' = ImageRegistrationMethod_GetOptimizerStopConditionDescription, 'StopRegistration' = ImageRegistrationMethod_StopRegistration);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ImageRegistrationMethod
# Start of delete_AbsImageFilter

`delete_AbsImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_AbsImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_AbsImageFilter`, 'returnType') = 'void'
attr(`delete_AbsImageFilter`, "inputTypes") = c('_p_itk__simple__AbsImageFilter')
class(`delete_AbsImageFilter`) = c("SWIGFunction", class('delete_AbsImageFilter'))

# Start of new_AbsImageFilter

`AbsImageFilter` = function()
{
  ;ans = .Call('R_swig_new_AbsImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AbsImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AbsImageFilter);
  ans
  
}

attr(`AbsImageFilter`, 'returnType') = '_p_itk__simple__AbsImageFilter'
class(`AbsImageFilter`) = c("SWIGFunction", class('AbsImageFilter'))

# Start of AbsImageFilter_GetName

`AbsImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AbsImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AbsImageFilter_GetName`, 'returnType') = 'character'
attr(`AbsImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__AbsImageFilter')
class(`AbsImageFilter_GetName`) = c("SWIGFunction", class('AbsImageFilter_GetName'))

# Start of AbsImageFilter_ToString

`AbsImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AbsImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AbsImageFilter_ToString`, 'returnType') = 'character'
attr(`AbsImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__AbsImageFilter')
class(`AbsImageFilter_ToString`) = c("SWIGFunction", class('AbsImageFilter_ToString'))

# Start of AbsImageFilter_Execute

`AbsImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AbsImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AbsImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`AbsImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__AbsImageFilter', '_p_itk__simple__Image')
class(`AbsImageFilter_Execute`) = c("SWIGFunction", class('AbsImageFilter_Execute'))

# Start of accessor method for itk::simple::AbsImageFilter
setMethod('$', '_p_itk__simple__AbsImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = AbsImageFilter_GetName, 'ToString' = AbsImageFilter_ToString, 'Execute' = AbsImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::AbsImageFilter
# Start of Abs

`Abs` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Abs', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Abs`, 'returnType') = '_p_itk__simple__Image'
attr(`Abs`, "inputTypes") = c('_p_itk__simple__Image')
class(`Abs`) = c("SWIGFunction", class('Abs'))

# Start of delete_AbsoluteValueDifferenceImageFilter

`delete_AbsoluteValueDifferenceImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_AbsoluteValueDifferenceImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_AbsoluteValueDifferenceImageFilter`, 'returnType') = 'void'
attr(`delete_AbsoluteValueDifferenceImageFilter`, "inputTypes") = c('_p_itk__simple__AbsoluteValueDifferenceImageFilter')
class(`delete_AbsoluteValueDifferenceImageFilter`) = c("SWIGFunction", class('delete_AbsoluteValueDifferenceImageFilter'))

# Start of new_AbsoluteValueDifferenceImageFilter

`AbsoluteValueDifferenceImageFilter` = function()
{
  ;ans = .Call('R_swig_new_AbsoluteValueDifferenceImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AbsoluteValueDifferenceImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AbsoluteValueDifferenceImageFilter);
  ans
  
}

attr(`AbsoluteValueDifferenceImageFilter`, 'returnType') = '_p_itk__simple__AbsoluteValueDifferenceImageFilter'
class(`AbsoluteValueDifferenceImageFilter`) = c("SWIGFunction", class('AbsoluteValueDifferenceImageFilter'))

# Start of AbsoluteValueDifferenceImageFilter_GetName

`AbsoluteValueDifferenceImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AbsoluteValueDifferenceImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AbsoluteValueDifferenceImageFilter_GetName`, 'returnType') = 'character'
attr(`AbsoluteValueDifferenceImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__AbsoluteValueDifferenceImageFilter')
class(`AbsoluteValueDifferenceImageFilter_GetName`) = c("SWIGFunction", class('AbsoluteValueDifferenceImageFilter_GetName'))

# Start of AbsoluteValueDifferenceImageFilter_ToString

`AbsoluteValueDifferenceImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AbsoluteValueDifferenceImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AbsoluteValueDifferenceImageFilter_ToString`, 'returnType') = 'character'
attr(`AbsoluteValueDifferenceImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__AbsoluteValueDifferenceImageFilter')
class(`AbsoluteValueDifferenceImageFilter_ToString`) = c("SWIGFunction", class('AbsoluteValueDifferenceImageFilter_ToString'))

# Start of AbsoluteValueDifferenceImageFilter_Execute

`AbsoluteValueDifferenceImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_AbsoluteValueDifferenceImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AbsoluteValueDifferenceImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`AbsoluteValueDifferenceImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__AbsoluteValueDifferenceImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`AbsoluteValueDifferenceImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('AbsoluteValueDifferenceImageFilter_Execute__SWIG_0'))

# Start of AbsoluteValueDifferenceImageFilter_Execute

`AbsoluteValueDifferenceImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_AbsoluteValueDifferenceImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AbsoluteValueDifferenceImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`AbsoluteValueDifferenceImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__AbsoluteValueDifferenceImageFilter', '_p_itk__simple__Image', 'numeric')
class(`AbsoluteValueDifferenceImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('AbsoluteValueDifferenceImageFilter_Execute__SWIG_1'))

# Start of AbsoluteValueDifferenceImageFilter_Execute

`AbsoluteValueDifferenceImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_AbsoluteValueDifferenceImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AbsoluteValueDifferenceImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`AbsoluteValueDifferenceImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__AbsoluteValueDifferenceImageFilter', 'numeric', '_p_itk__simple__Image')
class(`AbsoluteValueDifferenceImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('AbsoluteValueDifferenceImageFilter_Execute__SWIG_2'))

`AbsoluteValueDifferenceImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__AbsoluteValueDifferenceImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- AbsoluteValueDifferenceImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__AbsoluteValueDifferenceImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- AbsoluteValueDifferenceImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__AbsoluteValueDifferenceImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- AbsoluteValueDifferenceImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for AbsoluteValueDifferenceImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::AbsoluteValueDifferenceImageFilter
setMethod('$', '_p_itk__simple__AbsoluteValueDifferenceImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = AbsoluteValueDifferenceImageFilter_GetName, 'ToString' = AbsoluteValueDifferenceImageFilter_ToString, 'Execute' = AbsoluteValueDifferenceImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::AbsoluteValueDifferenceImageFilter
# Start of AbsoluteValueDifference

`AbsoluteValueDifference__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_AbsoluteValueDifference__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AbsoluteValueDifference__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`AbsoluteValueDifference__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`AbsoluteValueDifference__SWIG_0`) = c("SWIGFunction", class('AbsoluteValueDifference__SWIG_0'))

# Start of AbsoluteValueDifference

`AbsoluteValueDifference__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_AbsoluteValueDifference__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AbsoluteValueDifference__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`AbsoluteValueDifference__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`AbsoluteValueDifference__SWIG_1`) = c("SWIGFunction", class('AbsoluteValueDifference__SWIG_1'))

# Start of AbsoluteValueDifference

`AbsoluteValueDifference__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_AbsoluteValueDifference__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AbsoluteValueDifference__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`AbsoluteValueDifference__SWIG_2`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`AbsoluteValueDifference__SWIG_2`) = c("SWIGFunction", class('AbsoluteValueDifference__SWIG_2'))

`AbsoluteValueDifference` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- AbsoluteValueDifference__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- AbsoluteValueDifference__SWIG_1; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- AbsoluteValueDifference__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for AbsoluteValueDifference with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_AcosImageFilter

`delete_AcosImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_AcosImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_AcosImageFilter`, 'returnType') = 'void'
attr(`delete_AcosImageFilter`, "inputTypes") = c('_p_itk__simple__AcosImageFilter')
class(`delete_AcosImageFilter`) = c("SWIGFunction", class('delete_AcosImageFilter'))

# Start of new_AcosImageFilter

`AcosImageFilter` = function()
{
  ;ans = .Call('R_swig_new_AcosImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AcosImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AcosImageFilter);
  ans
  
}

attr(`AcosImageFilter`, 'returnType') = '_p_itk__simple__AcosImageFilter'
class(`AcosImageFilter`) = c("SWIGFunction", class('AcosImageFilter'))

# Start of AcosImageFilter_GetName

`AcosImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AcosImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AcosImageFilter_GetName`, 'returnType') = 'character'
attr(`AcosImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__AcosImageFilter')
class(`AcosImageFilter_GetName`) = c("SWIGFunction", class('AcosImageFilter_GetName'))

# Start of AcosImageFilter_ToString

`AcosImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AcosImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AcosImageFilter_ToString`, 'returnType') = 'character'
attr(`AcosImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__AcosImageFilter')
class(`AcosImageFilter_ToString`) = c("SWIGFunction", class('AcosImageFilter_ToString'))

# Start of AcosImageFilter_Execute

`AcosImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AcosImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AcosImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`AcosImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__AcosImageFilter', '_p_itk__simple__Image')
class(`AcosImageFilter_Execute`) = c("SWIGFunction", class('AcosImageFilter_Execute'))

# Start of accessor method for itk::simple::AcosImageFilter
setMethod('$', '_p_itk__simple__AcosImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = AcosImageFilter_GetName, 'ToString' = AcosImageFilter_ToString, 'Execute' = AcosImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::AcosImageFilter
# Start of Acos

`Acos` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Acos', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Acos`, 'returnType') = '_p_itk__simple__Image'
attr(`Acos`, "inputTypes") = c('_p_itk__simple__Image')
class(`Acos`) = c("SWIGFunction", class('Acos'))

# Start of delete_AdaptiveHistogramEqualizationImageFilter

`delete_AdaptiveHistogramEqualizationImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_AdaptiveHistogramEqualizationImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_AdaptiveHistogramEqualizationImageFilter`, 'returnType') = 'void'
attr(`delete_AdaptiveHistogramEqualizationImageFilter`, "inputTypes") = c('_p_itk__simple__AdaptiveHistogramEqualizationImageFilter')
class(`delete_AdaptiveHistogramEqualizationImageFilter`) = c("SWIGFunction", class('delete_AdaptiveHistogramEqualizationImageFilter'))

# Start of new_AdaptiveHistogramEqualizationImageFilter

`AdaptiveHistogramEqualizationImageFilter` = function()
{
  ;ans = .Call('R_swig_new_AdaptiveHistogramEqualizationImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AdaptiveHistogramEqualizationImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AdaptiveHistogramEqualizationImageFilter);
  ans
  
}

attr(`AdaptiveHistogramEqualizationImageFilter`, 'returnType') = '_p_itk__simple__AdaptiveHistogramEqualizationImageFilter'
class(`AdaptiveHistogramEqualizationImageFilter`) = c("SWIGFunction", class('AdaptiveHistogramEqualizationImageFilter'))

# Start of AdaptiveHistogramEqualizationImageFilter_SetRadius

`AdaptiveHistogramEqualizationImageFilter_SetRadius__SWIG_0` = function(self, Radius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Radius = as.integer(Radius);
  ;ans = .Call('R_swig_AdaptiveHistogramEqualizationImageFilter_SetRadius__SWIG_0', self, Radius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AdaptiveHistogramEqualizationImageFilter_SetRadius__SWIG_0`, 'returnType') = 'void'
attr(`AdaptiveHistogramEqualizationImageFilter_SetRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__AdaptiveHistogramEqualizationImageFilter', 'integer')
class(`AdaptiveHistogramEqualizationImageFilter_SetRadius__SWIG_0`) = c("SWIGFunction", class('AdaptiveHistogramEqualizationImageFilter_SetRadius__SWIG_0'))

# Start of AdaptiveHistogramEqualizationImageFilter_SetRadius

`AdaptiveHistogramEqualizationImageFilter_SetRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_AdaptiveHistogramEqualizationImageFilter_SetRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AdaptiveHistogramEqualizationImageFilter_SetRadius__SWIG_1`, 'returnType') = 'void'
attr(`AdaptiveHistogramEqualizationImageFilter_SetRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__AdaptiveHistogramEqualizationImageFilter', 'integer')
class(`AdaptiveHistogramEqualizationImageFilter_SetRadius__SWIG_1`) = c("SWIGFunction", class('AdaptiveHistogramEqualizationImageFilter_SetRadius__SWIG_1'))

`AdaptiveHistogramEqualizationImageFilter_SetRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__AdaptiveHistogramEqualizationImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- AdaptiveHistogramEqualizationImageFilter_SetRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__AdaptiveHistogramEqualizationImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- AdaptiveHistogramEqualizationImageFilter_SetRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for AdaptiveHistogramEqualizationImageFilter_SetRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of AdaptiveHistogramEqualizationImageFilter_GetRadius

`AdaptiveHistogramEqualizationImageFilter_GetRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AdaptiveHistogramEqualizationImageFilter_GetRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AdaptiveHistogramEqualizationImageFilter_GetRadius`, 'returnType') = 'integer'
attr(`AdaptiveHistogramEqualizationImageFilter_GetRadius`, "inputTypes") = c('_p_itk__simple__AdaptiveHistogramEqualizationImageFilter')
class(`AdaptiveHistogramEqualizationImageFilter_GetRadius`) = c("SWIGFunction", class('AdaptiveHistogramEqualizationImageFilter_GetRadius'))

# Start of AdaptiveHistogramEqualizationImageFilter_SetAlpha

`AdaptiveHistogramEqualizationImageFilter_SetAlpha` = function(self, Alpha)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_AdaptiveHistogramEqualizationImageFilter_SetAlpha', self, Alpha, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AdaptiveHistogramEqualizationImageFilter_SetAlpha`, 'returnType') = 'void'
attr(`AdaptiveHistogramEqualizationImageFilter_SetAlpha`, "inputTypes") = c('_p_itk__simple__AdaptiveHistogramEqualizationImageFilter', 'numeric')
class(`AdaptiveHistogramEqualizationImageFilter_SetAlpha`) = c("SWIGFunction", class('AdaptiveHistogramEqualizationImageFilter_SetAlpha'))

# Start of AdaptiveHistogramEqualizationImageFilter_GetAlpha

`AdaptiveHistogramEqualizationImageFilter_GetAlpha` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AdaptiveHistogramEqualizationImageFilter_GetAlpha', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AdaptiveHistogramEqualizationImageFilter_GetAlpha`, 'returnType') = 'numeric'
attr(`AdaptiveHistogramEqualizationImageFilter_GetAlpha`, "inputTypes") = c('_p_itk__simple__AdaptiveHistogramEqualizationImageFilter')
class(`AdaptiveHistogramEqualizationImageFilter_GetAlpha`) = c("SWIGFunction", class('AdaptiveHistogramEqualizationImageFilter_GetAlpha'))

# Start of AdaptiveHistogramEqualizationImageFilter_SetBeta

`AdaptiveHistogramEqualizationImageFilter_SetBeta` = function(self, Beta)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_AdaptiveHistogramEqualizationImageFilter_SetBeta', self, Beta, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AdaptiveHistogramEqualizationImageFilter_SetBeta`, 'returnType') = 'void'
attr(`AdaptiveHistogramEqualizationImageFilter_SetBeta`, "inputTypes") = c('_p_itk__simple__AdaptiveHistogramEqualizationImageFilter', 'numeric')
class(`AdaptiveHistogramEqualizationImageFilter_SetBeta`) = c("SWIGFunction", class('AdaptiveHistogramEqualizationImageFilter_SetBeta'))

# Start of AdaptiveHistogramEqualizationImageFilter_GetBeta

`AdaptiveHistogramEqualizationImageFilter_GetBeta` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AdaptiveHistogramEqualizationImageFilter_GetBeta', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AdaptiveHistogramEqualizationImageFilter_GetBeta`, 'returnType') = 'numeric'
attr(`AdaptiveHistogramEqualizationImageFilter_GetBeta`, "inputTypes") = c('_p_itk__simple__AdaptiveHistogramEqualizationImageFilter')
class(`AdaptiveHistogramEqualizationImageFilter_GetBeta`) = c("SWIGFunction", class('AdaptiveHistogramEqualizationImageFilter_GetBeta'))

# Start of AdaptiveHistogramEqualizationImageFilter_GetName

`AdaptiveHistogramEqualizationImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AdaptiveHistogramEqualizationImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AdaptiveHistogramEqualizationImageFilter_GetName`, 'returnType') = 'character'
attr(`AdaptiveHistogramEqualizationImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__AdaptiveHistogramEqualizationImageFilter')
class(`AdaptiveHistogramEqualizationImageFilter_GetName`) = c("SWIGFunction", class('AdaptiveHistogramEqualizationImageFilter_GetName'))

# Start of AdaptiveHistogramEqualizationImageFilter_ToString

`AdaptiveHistogramEqualizationImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AdaptiveHistogramEqualizationImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AdaptiveHistogramEqualizationImageFilter_ToString`, 'returnType') = 'character'
attr(`AdaptiveHistogramEqualizationImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__AdaptiveHistogramEqualizationImageFilter')
class(`AdaptiveHistogramEqualizationImageFilter_ToString`) = c("SWIGFunction", class('AdaptiveHistogramEqualizationImageFilter_ToString'))

# Start of AdaptiveHistogramEqualizationImageFilter_Execute

`AdaptiveHistogramEqualizationImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AdaptiveHistogramEqualizationImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AdaptiveHistogramEqualizationImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`AdaptiveHistogramEqualizationImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__AdaptiveHistogramEqualizationImageFilter', '_p_itk__simple__Image')
class(`AdaptiveHistogramEqualizationImageFilter_Execute`) = c("SWIGFunction", class('AdaptiveHistogramEqualizationImageFilter_Execute'))

# Start of accessor method for itk::simple::AdaptiveHistogramEqualizationImageFilter
setMethod('$', '_p_itk__simple__AdaptiveHistogramEqualizationImageFilter', function(x, name)

{
  accessorFuns = list('SetRadius' = AdaptiveHistogramEqualizationImageFilter_SetRadius, 'GetRadius' = AdaptiveHistogramEqualizationImageFilter_GetRadius, 'SetAlpha' = AdaptiveHistogramEqualizationImageFilter_SetAlpha, 'GetAlpha' = AdaptiveHistogramEqualizationImageFilter_GetAlpha, 'SetBeta' = AdaptiveHistogramEqualizationImageFilter_SetBeta, 'GetBeta' = AdaptiveHistogramEqualizationImageFilter_GetBeta, 'GetName' = AdaptiveHistogramEqualizationImageFilter_GetName, 'ToString' = AdaptiveHistogramEqualizationImageFilter_ToString, 'Execute' = AdaptiveHistogramEqualizationImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::AdaptiveHistogramEqualizationImageFilter
# Start of AdaptiveHistogramEqualization

`AdaptiveHistogramEqualization__SWIG_0` = function(image1, radius, alpha, beta, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  
  
  ;ans = .Call('R_swig_AdaptiveHistogramEqualization__SWIG_0', image1, radius, alpha, beta, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AdaptiveHistogramEqualization__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`AdaptiveHistogramEqualization__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'numeric')
class(`AdaptiveHistogramEqualization__SWIG_0`) = c("SWIGFunction", class('AdaptiveHistogramEqualization__SWIG_0'))

# Start of AdaptiveHistogramEqualization

`AdaptiveHistogramEqualization__SWIG_1` = function(image1, radius, alpha, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  
  ;ans = .Call('R_swig_AdaptiveHistogramEqualization__SWIG_1', image1, radius, alpha, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AdaptiveHistogramEqualization__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`AdaptiveHistogramEqualization__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric')
class(`AdaptiveHistogramEqualization__SWIG_1`) = c("SWIGFunction", class('AdaptiveHistogramEqualization__SWIG_1'))

# Start of AdaptiveHistogramEqualization

`AdaptiveHistogramEqualization__SWIG_2` = function(image1, radius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  ;ans = .Call('R_swig_AdaptiveHistogramEqualization__SWIG_2', image1, radius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AdaptiveHistogramEqualization__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`AdaptiveHistogramEqualization__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`AdaptiveHistogramEqualization__SWIG_2`) = c("SWIGFunction", class('AdaptiveHistogramEqualization__SWIG_2'))

# Start of AdaptiveHistogramEqualization

`AdaptiveHistogramEqualization__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AdaptiveHistogramEqualization__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AdaptiveHistogramEqualization__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`AdaptiveHistogramEqualization__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`AdaptiveHistogramEqualization__SWIG_3`) = c("SWIGFunction", class('AdaptiveHistogramEqualization__SWIG_3'))

`AdaptiveHistogramEqualization` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- AdaptiveHistogramEqualization__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- AdaptiveHistogramEqualization__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- AdaptiveHistogramEqualization__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- AdaptiveHistogramEqualization__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for AdaptiveHistogramEqualization with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_AddImageFilter

`delete_AddImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_AddImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_AddImageFilter`, 'returnType') = 'void'
attr(`delete_AddImageFilter`, "inputTypes") = c('_p_itk__simple__AddImageFilter')
class(`delete_AddImageFilter`) = c("SWIGFunction", class('delete_AddImageFilter'))

# Start of new_AddImageFilter

`AddImageFilter` = function()
{
  ;ans = .Call('R_swig_new_AddImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AddImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AddImageFilter);
  ans
  
}

attr(`AddImageFilter`, 'returnType') = '_p_itk__simple__AddImageFilter'
class(`AddImageFilter`) = c("SWIGFunction", class('AddImageFilter'))

# Start of AddImageFilter_GetName

`AddImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AddImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AddImageFilter_GetName`, 'returnType') = 'character'
attr(`AddImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__AddImageFilter')
class(`AddImageFilter_GetName`) = c("SWIGFunction", class('AddImageFilter_GetName'))

# Start of AddImageFilter_ToString

`AddImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AddImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AddImageFilter_ToString`, 'returnType') = 'character'
attr(`AddImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__AddImageFilter')
class(`AddImageFilter_ToString`) = c("SWIGFunction", class('AddImageFilter_ToString'))

# Start of AddImageFilter_Execute

`AddImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_AddImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AddImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`AddImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__AddImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`AddImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('AddImageFilter_Execute__SWIG_0'))

# Start of AddImageFilter_Execute

`AddImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_AddImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AddImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`AddImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__AddImageFilter', '_p_itk__simple__Image', 'numeric')
class(`AddImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('AddImageFilter_Execute__SWIG_1'))

# Start of AddImageFilter_Execute

`AddImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_AddImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AddImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`AddImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__AddImageFilter', 'numeric', '_p_itk__simple__Image')
class(`AddImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('AddImageFilter_Execute__SWIG_2'))

`AddImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__AddImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- AddImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__AddImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- AddImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__AddImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- AddImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for AddImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::AddImageFilter
setMethod('$', '_p_itk__simple__AddImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = AddImageFilter_GetName, 'ToString' = AddImageFilter_ToString, 'Execute' = AddImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::AddImageFilter
# Start of Add

`Add__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Add__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Add__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Add__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Add__SWIG_0`) = c("SWIGFunction", class('Add__SWIG_0'))

# Start of Add

`Add__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Add__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Add__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Add__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`Add__SWIG_1`) = c("SWIGFunction", class('Add__SWIG_1'))

# Start of Add

`Add__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Add__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Add__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Add__SWIG_2`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`Add__SWIG_2`) = c("SWIGFunction", class('Add__SWIG_2'))

`Add` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Add__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Add__SWIG_1; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Add__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Add with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_AdditiveGaussianNoiseImageFilter

`delete_AdditiveGaussianNoiseImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_AdditiveGaussianNoiseImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_AdditiveGaussianNoiseImageFilter`, 'returnType') = 'void'
attr(`delete_AdditiveGaussianNoiseImageFilter`, "inputTypes") = c('_p_itk__simple__AdditiveGaussianNoiseImageFilter')
class(`delete_AdditiveGaussianNoiseImageFilter`) = c("SWIGFunction", class('delete_AdditiveGaussianNoiseImageFilter'))

# Start of new_AdditiveGaussianNoiseImageFilter

`AdditiveGaussianNoiseImageFilter` = function()
{
  ;ans = .Call('R_swig_new_AdditiveGaussianNoiseImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AdditiveGaussianNoiseImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AdditiveGaussianNoiseImageFilter);
  ans
  
}

attr(`AdditiveGaussianNoiseImageFilter`, 'returnType') = '_p_itk__simple__AdditiveGaussianNoiseImageFilter'
class(`AdditiveGaussianNoiseImageFilter`) = c("SWIGFunction", class('AdditiveGaussianNoiseImageFilter'))

# Start of AdditiveGaussianNoiseImageFilter_SetStandardDeviation

`AdditiveGaussianNoiseImageFilter_SetStandardDeviation` = function(self, StandardDeviation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_AdditiveGaussianNoiseImageFilter_SetStandardDeviation', self, StandardDeviation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AdditiveGaussianNoiseImageFilter_SetStandardDeviation`, 'returnType') = 'void'
attr(`AdditiveGaussianNoiseImageFilter_SetStandardDeviation`, "inputTypes") = c('_p_itk__simple__AdditiveGaussianNoiseImageFilter', 'numeric')
class(`AdditiveGaussianNoiseImageFilter_SetStandardDeviation`) = c("SWIGFunction", class('AdditiveGaussianNoiseImageFilter_SetStandardDeviation'))

# Start of AdditiveGaussianNoiseImageFilter_GetStandardDeviation

`AdditiveGaussianNoiseImageFilter_GetStandardDeviation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AdditiveGaussianNoiseImageFilter_GetStandardDeviation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AdditiveGaussianNoiseImageFilter_GetStandardDeviation`, 'returnType') = 'numeric'
attr(`AdditiveGaussianNoiseImageFilter_GetStandardDeviation`, "inputTypes") = c('_p_itk__simple__AdditiveGaussianNoiseImageFilter')
class(`AdditiveGaussianNoiseImageFilter_GetStandardDeviation`) = c("SWIGFunction", class('AdditiveGaussianNoiseImageFilter_GetStandardDeviation'))

# Start of AdditiveGaussianNoiseImageFilter_SetMean

`AdditiveGaussianNoiseImageFilter_SetMean` = function(self, Mean)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_AdditiveGaussianNoiseImageFilter_SetMean', self, Mean, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AdditiveGaussianNoiseImageFilter_SetMean`, 'returnType') = 'void'
attr(`AdditiveGaussianNoiseImageFilter_SetMean`, "inputTypes") = c('_p_itk__simple__AdditiveGaussianNoiseImageFilter', 'numeric')
class(`AdditiveGaussianNoiseImageFilter_SetMean`) = c("SWIGFunction", class('AdditiveGaussianNoiseImageFilter_SetMean'))

# Start of AdditiveGaussianNoiseImageFilter_GetMean

`AdditiveGaussianNoiseImageFilter_GetMean` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AdditiveGaussianNoiseImageFilter_GetMean', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AdditiveGaussianNoiseImageFilter_GetMean`, 'returnType') = 'numeric'
attr(`AdditiveGaussianNoiseImageFilter_GetMean`, "inputTypes") = c('_p_itk__simple__AdditiveGaussianNoiseImageFilter')
class(`AdditiveGaussianNoiseImageFilter_GetMean`) = c("SWIGFunction", class('AdditiveGaussianNoiseImageFilter_GetMean'))

# Start of AdditiveGaussianNoiseImageFilter_SetSeed

`AdditiveGaussianNoiseImageFilter_SetSeed` = function(self, Seed)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Seed = as.integer(Seed);
  
  if(length(Seed) > 1) {
    warning("using only the first element of Seed");
  };
  
  ;ans = .Call('R_swig_AdditiveGaussianNoiseImageFilter_SetSeed', self, Seed, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AdditiveGaussianNoiseImageFilter_SetSeed`, 'returnType') = 'void'
attr(`AdditiveGaussianNoiseImageFilter_SetSeed`, "inputTypes") = c('_p_itk__simple__AdditiveGaussianNoiseImageFilter', 'integer')
class(`AdditiveGaussianNoiseImageFilter_SetSeed`) = c("SWIGFunction", class('AdditiveGaussianNoiseImageFilter_SetSeed'))

# Start of AdditiveGaussianNoiseImageFilter_GetSeed

`AdditiveGaussianNoiseImageFilter_GetSeed` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AdditiveGaussianNoiseImageFilter_GetSeed', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AdditiveGaussianNoiseImageFilter_GetSeed`, 'returnType') = 'integer'
attr(`AdditiveGaussianNoiseImageFilter_GetSeed`, "inputTypes") = c('_p_itk__simple__AdditiveGaussianNoiseImageFilter')
class(`AdditiveGaussianNoiseImageFilter_GetSeed`) = c("SWIGFunction", class('AdditiveGaussianNoiseImageFilter_GetSeed'))

# Start of AdditiveGaussianNoiseImageFilter_GetName

`AdditiveGaussianNoiseImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AdditiveGaussianNoiseImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AdditiveGaussianNoiseImageFilter_GetName`, 'returnType') = 'character'
attr(`AdditiveGaussianNoiseImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__AdditiveGaussianNoiseImageFilter')
class(`AdditiveGaussianNoiseImageFilter_GetName`) = c("SWIGFunction", class('AdditiveGaussianNoiseImageFilter_GetName'))

# Start of AdditiveGaussianNoiseImageFilter_ToString

`AdditiveGaussianNoiseImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AdditiveGaussianNoiseImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AdditiveGaussianNoiseImageFilter_ToString`, 'returnType') = 'character'
attr(`AdditiveGaussianNoiseImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__AdditiveGaussianNoiseImageFilter')
class(`AdditiveGaussianNoiseImageFilter_ToString`) = c("SWIGFunction", class('AdditiveGaussianNoiseImageFilter_ToString'))

# Start of AdditiveGaussianNoiseImageFilter_Execute

`AdditiveGaussianNoiseImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AdditiveGaussianNoiseImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AdditiveGaussianNoiseImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`AdditiveGaussianNoiseImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__AdditiveGaussianNoiseImageFilter', '_p_itk__simple__Image')
class(`AdditiveGaussianNoiseImageFilter_Execute`) = c("SWIGFunction", class('AdditiveGaussianNoiseImageFilter_Execute'))

# Start of accessor method for itk::simple::AdditiveGaussianNoiseImageFilter
setMethod('$', '_p_itk__simple__AdditiveGaussianNoiseImageFilter', function(x, name)

{
  accessorFuns = list('SetStandardDeviation' = AdditiveGaussianNoiseImageFilter_SetStandardDeviation, 'GetStandardDeviation' = AdditiveGaussianNoiseImageFilter_GetStandardDeviation, 'SetMean' = AdditiveGaussianNoiseImageFilter_SetMean, 'GetMean' = AdditiveGaussianNoiseImageFilter_GetMean, 'SetSeed' = AdditiveGaussianNoiseImageFilter_SetSeed, 'GetSeed' = AdditiveGaussianNoiseImageFilter_GetSeed, 'GetName' = AdditiveGaussianNoiseImageFilter_GetName, 'ToString' = AdditiveGaussianNoiseImageFilter_ToString, 'Execute' = AdditiveGaussianNoiseImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::AdditiveGaussianNoiseImageFilter
# Start of AdditiveGaussianNoise

`AdditiveGaussianNoise__SWIG_0` = function(image1, standardDeviation, mean, seed, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  seed = as.integer(seed);
  
  if(length(seed) > 1) {
    warning("using only the first element of seed");
  };
  
  ;ans = .Call('R_swig_AdditiveGaussianNoise__SWIG_0', image1, standardDeviation, mean, seed, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AdditiveGaussianNoise__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`AdditiveGaussianNoise__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'integer')
class(`AdditiveGaussianNoise__SWIG_0`) = c("SWIGFunction", class('AdditiveGaussianNoise__SWIG_0'))

# Start of AdditiveGaussianNoise

`AdditiveGaussianNoise__SWIG_1` = function(image1, standardDeviation, mean, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_AdditiveGaussianNoise__SWIG_1', image1, standardDeviation, mean, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AdditiveGaussianNoise__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`AdditiveGaussianNoise__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`AdditiveGaussianNoise__SWIG_1`) = c("SWIGFunction", class('AdditiveGaussianNoise__SWIG_1'))

# Start of AdditiveGaussianNoise

`AdditiveGaussianNoise__SWIG_2` = function(image1, standardDeviation, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_AdditiveGaussianNoise__SWIG_2', image1, standardDeviation, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AdditiveGaussianNoise__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`AdditiveGaussianNoise__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`AdditiveGaussianNoise__SWIG_2`) = c("SWIGFunction", class('AdditiveGaussianNoise__SWIG_2'))

# Start of AdditiveGaussianNoise

`AdditiveGaussianNoise__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AdditiveGaussianNoise__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AdditiveGaussianNoise__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`AdditiveGaussianNoise__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`AdditiveGaussianNoise__SWIG_3`) = c("SWIGFunction", class('AdditiveGaussianNoise__SWIG_3'))

`AdditiveGaussianNoise` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- AdditiveGaussianNoise__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- AdditiveGaussianNoise__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- AdditiveGaussianNoise__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- AdditiveGaussianNoise__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for AdditiveGaussianNoise with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_AggregateLabelMapFilter

`delete_AggregateLabelMapFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_AggregateLabelMapFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_AggregateLabelMapFilter`, 'returnType') = 'void'
attr(`delete_AggregateLabelMapFilter`, "inputTypes") = c('_p_itk__simple__AggregateLabelMapFilter')
class(`delete_AggregateLabelMapFilter`) = c("SWIGFunction", class('delete_AggregateLabelMapFilter'))

# Start of new_AggregateLabelMapFilter

`AggregateLabelMapFilter` = function()
{
  ;ans = .Call('R_swig_new_AggregateLabelMapFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AggregateLabelMapFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AggregateLabelMapFilter);
  ans
  
}

attr(`AggregateLabelMapFilter`, 'returnType') = '_p_itk__simple__AggregateLabelMapFilter'
class(`AggregateLabelMapFilter`) = c("SWIGFunction", class('AggregateLabelMapFilter'))

# Start of AggregateLabelMapFilter_GetName

`AggregateLabelMapFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AggregateLabelMapFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AggregateLabelMapFilter_GetName`, 'returnType') = 'character'
attr(`AggregateLabelMapFilter_GetName`, "inputTypes") = c('_p_itk__simple__AggregateLabelMapFilter')
class(`AggregateLabelMapFilter_GetName`) = c("SWIGFunction", class('AggregateLabelMapFilter_GetName'))

# Start of AggregateLabelMapFilter_ToString

`AggregateLabelMapFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AggregateLabelMapFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AggregateLabelMapFilter_ToString`, 'returnType') = 'character'
attr(`AggregateLabelMapFilter_ToString`, "inputTypes") = c('_p_itk__simple__AggregateLabelMapFilter')
class(`AggregateLabelMapFilter_ToString`) = c("SWIGFunction", class('AggregateLabelMapFilter_ToString'))

# Start of AggregateLabelMapFilter_Execute

`AggregateLabelMapFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AggregateLabelMapFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AggregateLabelMapFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`AggregateLabelMapFilter_Execute`, "inputTypes") = c('_p_itk__simple__AggregateLabelMapFilter', '_p_itk__simple__Image')
class(`AggregateLabelMapFilter_Execute`) = c("SWIGFunction", class('AggregateLabelMapFilter_Execute'))

# Start of accessor method for itk::simple::AggregateLabelMapFilter
setMethod('$', '_p_itk__simple__AggregateLabelMapFilter', function(x, name)

{
  accessorFuns = list('GetName' = AggregateLabelMapFilter_GetName, 'ToString' = AggregateLabelMapFilter_ToString, 'Execute' = AggregateLabelMapFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::AggregateLabelMapFilter
# Start of AggregateLabelMap

`AggregateLabelMap` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AggregateLabelMap', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AggregateLabelMap`, 'returnType') = '_p_itk__simple__Image'
attr(`AggregateLabelMap`, "inputTypes") = c('_p_itk__simple__Image')
class(`AggregateLabelMap`) = c("SWIGFunction", class('AggregateLabelMap'))

# Start of delete_AndImageFilter

`delete_AndImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_AndImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_AndImageFilter`, 'returnType') = 'void'
attr(`delete_AndImageFilter`, "inputTypes") = c('_p_itk__simple__AndImageFilter')
class(`delete_AndImageFilter`) = c("SWIGFunction", class('delete_AndImageFilter'))

# Start of new_AndImageFilter

`AndImageFilter` = function()
{
  ;ans = .Call('R_swig_new_AndImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AndImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AndImageFilter);
  ans
  
}

attr(`AndImageFilter`, 'returnType') = '_p_itk__simple__AndImageFilter'
class(`AndImageFilter`) = c("SWIGFunction", class('AndImageFilter'))

# Start of AndImageFilter_GetName

`AndImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AndImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AndImageFilter_GetName`, 'returnType') = 'character'
attr(`AndImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__AndImageFilter')
class(`AndImageFilter_GetName`) = c("SWIGFunction", class('AndImageFilter_GetName'))

# Start of AndImageFilter_ToString

`AndImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AndImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AndImageFilter_ToString`, 'returnType') = 'character'
attr(`AndImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__AndImageFilter')
class(`AndImageFilter_ToString`) = c("SWIGFunction", class('AndImageFilter_ToString'))

# Start of AndImageFilter_Execute

`AndImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_AndImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AndImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`AndImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__AndImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`AndImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('AndImageFilter_Execute__SWIG_0'))

# Start of AndImageFilter_Execute

`AndImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  ;ans = .Call('R_swig_AndImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AndImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`AndImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__AndImageFilter', '_p_itk__simple__Image', 'integer')
class(`AndImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('AndImageFilter_Execute__SWIG_1'))

# Start of AndImageFilter_Execute

`AndImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_AndImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AndImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`AndImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__AndImageFilter', 'integer', '_p_itk__simple__Image')
class(`AndImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('AndImageFilter_Execute__SWIG_2'))

`AndImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__AndImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- AndImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__AndImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- AndImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__AndImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- AndImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for AndImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::AndImageFilter
setMethod('$', '_p_itk__simple__AndImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = AndImageFilter_GetName, 'ToString' = AndImageFilter_ToString, 'Execute' = AndImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::AndImageFilter
# Start of And

`And__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_And__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`And__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`And__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`And__SWIG_0`) = c("SWIGFunction", class('And__SWIG_0'))

# Start of And

`And__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  ;ans = .Call('R_swig_And__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`And__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`And__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`And__SWIG_1`) = c("SWIGFunction", class('And__SWIG_1'))

# Start of And

`And__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_And__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`And__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`And__SWIG_2`, "inputTypes") = c('integer', '_p_itk__simple__Image')
class(`And__SWIG_2`) = c("SWIGFunction", class('And__SWIG_2'))

`And` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- And__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- And__SWIG_1; 
    }
    else if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- And__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for And with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_AntiAliasBinaryImageFilter

`delete_AntiAliasBinaryImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_AntiAliasBinaryImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_AntiAliasBinaryImageFilter`, 'returnType') = 'void'
attr(`delete_AntiAliasBinaryImageFilter`, "inputTypes") = c('_p_itk__simple__AntiAliasBinaryImageFilter')
class(`delete_AntiAliasBinaryImageFilter`) = c("SWIGFunction", class('delete_AntiAliasBinaryImageFilter'))

# Start of new_AntiAliasBinaryImageFilter

`AntiAliasBinaryImageFilter` = function()
{
  ;ans = .Call('R_swig_new_AntiAliasBinaryImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AntiAliasBinaryImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AntiAliasBinaryImageFilter);
  ans
  
}

attr(`AntiAliasBinaryImageFilter`, 'returnType') = '_p_itk__simple__AntiAliasBinaryImageFilter'
class(`AntiAliasBinaryImageFilter`) = c("SWIGFunction", class('AntiAliasBinaryImageFilter'))

# Start of AntiAliasBinaryImageFilter_SetMaximumRMSError

`AntiAliasBinaryImageFilter_SetMaximumRMSError` = function(self, MaximumRMSError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_AntiAliasBinaryImageFilter_SetMaximumRMSError', self, MaximumRMSError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AntiAliasBinaryImageFilter_SetMaximumRMSError`, 'returnType') = 'void'
attr(`AntiAliasBinaryImageFilter_SetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__AntiAliasBinaryImageFilter', 'numeric')
class(`AntiAliasBinaryImageFilter_SetMaximumRMSError`) = c("SWIGFunction", class('AntiAliasBinaryImageFilter_SetMaximumRMSError'))

# Start of AntiAliasBinaryImageFilter_GetMaximumRMSError

`AntiAliasBinaryImageFilter_GetMaximumRMSError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AntiAliasBinaryImageFilter_GetMaximumRMSError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AntiAliasBinaryImageFilter_GetMaximumRMSError`, 'returnType') = 'numeric'
attr(`AntiAliasBinaryImageFilter_GetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__AntiAliasBinaryImageFilter')
class(`AntiAliasBinaryImageFilter_GetMaximumRMSError`) = c("SWIGFunction", class('AntiAliasBinaryImageFilter_GetMaximumRMSError'))

# Start of AntiAliasBinaryImageFilter_SetNumberOfIterations

`AntiAliasBinaryImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_AntiAliasBinaryImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AntiAliasBinaryImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`AntiAliasBinaryImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__AntiAliasBinaryImageFilter', 'integer')
class(`AntiAliasBinaryImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('AntiAliasBinaryImageFilter_SetNumberOfIterations'))

# Start of AntiAliasBinaryImageFilter_GetNumberOfIterations

`AntiAliasBinaryImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AntiAliasBinaryImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AntiAliasBinaryImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`AntiAliasBinaryImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__AntiAliasBinaryImageFilter')
class(`AntiAliasBinaryImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('AntiAliasBinaryImageFilter_GetNumberOfIterations'))

# Start of AntiAliasBinaryImageFilter_GetElapsedIterations

`AntiAliasBinaryImageFilter_GetElapsedIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AntiAliasBinaryImageFilter_GetElapsedIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AntiAliasBinaryImageFilter_GetElapsedIterations`, 'returnType') = 'integer'
attr(`AntiAliasBinaryImageFilter_GetElapsedIterations`, "inputTypes") = c('_p_itk__simple__AntiAliasBinaryImageFilter')
class(`AntiAliasBinaryImageFilter_GetElapsedIterations`) = c("SWIGFunction", class('AntiAliasBinaryImageFilter_GetElapsedIterations'))

# Start of AntiAliasBinaryImageFilter_GetRMSChange

`AntiAliasBinaryImageFilter_GetRMSChange` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AntiAliasBinaryImageFilter_GetRMSChange', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AntiAliasBinaryImageFilter_GetRMSChange`, 'returnType') = 'numeric'
attr(`AntiAliasBinaryImageFilter_GetRMSChange`, "inputTypes") = c('_p_itk__simple__AntiAliasBinaryImageFilter')
class(`AntiAliasBinaryImageFilter_GetRMSChange`) = c("SWIGFunction", class('AntiAliasBinaryImageFilter_GetRMSChange'))

# Start of AntiAliasBinaryImageFilter_GetName

`AntiAliasBinaryImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AntiAliasBinaryImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AntiAliasBinaryImageFilter_GetName`, 'returnType') = 'character'
attr(`AntiAliasBinaryImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__AntiAliasBinaryImageFilter')
class(`AntiAliasBinaryImageFilter_GetName`) = c("SWIGFunction", class('AntiAliasBinaryImageFilter_GetName'))

# Start of AntiAliasBinaryImageFilter_ToString

`AntiAliasBinaryImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AntiAliasBinaryImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AntiAliasBinaryImageFilter_ToString`, 'returnType') = 'character'
attr(`AntiAliasBinaryImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__AntiAliasBinaryImageFilter')
class(`AntiAliasBinaryImageFilter_ToString`) = c("SWIGFunction", class('AntiAliasBinaryImageFilter_ToString'))

# Start of AntiAliasBinaryImageFilter_Execute

`AntiAliasBinaryImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AntiAliasBinaryImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AntiAliasBinaryImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`AntiAliasBinaryImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__AntiAliasBinaryImageFilter', '_p_itk__simple__Image')
class(`AntiAliasBinaryImageFilter_Execute`) = c("SWIGFunction", class('AntiAliasBinaryImageFilter_Execute'))

# Start of accessor method for itk::simple::AntiAliasBinaryImageFilter
setMethod('$', '_p_itk__simple__AntiAliasBinaryImageFilter', function(x, name)

{
  accessorFuns = list('SetMaximumRMSError' = AntiAliasBinaryImageFilter_SetMaximumRMSError, 'GetMaximumRMSError' = AntiAliasBinaryImageFilter_GetMaximumRMSError, 'SetNumberOfIterations' = AntiAliasBinaryImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = AntiAliasBinaryImageFilter_GetNumberOfIterations, 'GetElapsedIterations' = AntiAliasBinaryImageFilter_GetElapsedIterations, 'GetRMSChange' = AntiAliasBinaryImageFilter_GetRMSChange, 'GetName' = AntiAliasBinaryImageFilter_GetName, 'ToString' = AntiAliasBinaryImageFilter_ToString, 'Execute' = AntiAliasBinaryImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::AntiAliasBinaryImageFilter
# Start of AntiAliasBinary

`AntiAliasBinary__SWIG_0` = function(image1, maximumRMSError, numberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_AntiAliasBinary__SWIG_0', image1, maximumRMSError, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AntiAliasBinary__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`AntiAliasBinary__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`AntiAliasBinary__SWIG_0`) = c("SWIGFunction", class('AntiAliasBinary__SWIG_0'))

# Start of AntiAliasBinary

`AntiAliasBinary__SWIG_1` = function(image1, maximumRMSError, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_AntiAliasBinary__SWIG_1', image1, maximumRMSError, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AntiAliasBinary__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`AntiAliasBinary__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`AntiAliasBinary__SWIG_1`) = c("SWIGFunction", class('AntiAliasBinary__SWIG_1'))

# Start of AntiAliasBinary

`AntiAliasBinary__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AntiAliasBinary__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AntiAliasBinary__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`AntiAliasBinary__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`AntiAliasBinary__SWIG_2`) = c("SWIGFunction", class('AntiAliasBinary__SWIG_2'))

`AntiAliasBinary` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- AntiAliasBinary__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- AntiAliasBinary__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- AntiAliasBinary__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for AntiAliasBinary with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ApproximateSignedDistanceMapImageFilter

`delete_ApproximateSignedDistanceMapImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ApproximateSignedDistanceMapImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ApproximateSignedDistanceMapImageFilter`, 'returnType') = 'void'
attr(`delete_ApproximateSignedDistanceMapImageFilter`, "inputTypes") = c('_p_itk__simple__ApproximateSignedDistanceMapImageFilter')
class(`delete_ApproximateSignedDistanceMapImageFilter`) = c("SWIGFunction", class('delete_ApproximateSignedDistanceMapImageFilter'))

# Start of new_ApproximateSignedDistanceMapImageFilter

`ApproximateSignedDistanceMapImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ApproximateSignedDistanceMapImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ApproximateSignedDistanceMapImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ApproximateSignedDistanceMapImageFilter);
  ans
  
}

attr(`ApproximateSignedDistanceMapImageFilter`, 'returnType') = '_p_itk__simple__ApproximateSignedDistanceMapImageFilter'
class(`ApproximateSignedDistanceMapImageFilter`) = c("SWIGFunction", class('ApproximateSignedDistanceMapImageFilter'))

# Start of ApproximateSignedDistanceMapImageFilter_SetInsideValue

`ApproximateSignedDistanceMapImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ApproximateSignedDistanceMapImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ApproximateSignedDistanceMapImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`ApproximateSignedDistanceMapImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__ApproximateSignedDistanceMapImageFilter', 'numeric')
class(`ApproximateSignedDistanceMapImageFilter_SetInsideValue`) = c("SWIGFunction", class('ApproximateSignedDistanceMapImageFilter_SetInsideValue'))

# Start of ApproximateSignedDistanceMapImageFilter_GetInsideValue

`ApproximateSignedDistanceMapImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ApproximateSignedDistanceMapImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ApproximateSignedDistanceMapImageFilter_GetInsideValue`, 'returnType') = 'numeric'
attr(`ApproximateSignedDistanceMapImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__ApproximateSignedDistanceMapImageFilter')
class(`ApproximateSignedDistanceMapImageFilter_GetInsideValue`) = c("SWIGFunction", class('ApproximateSignedDistanceMapImageFilter_GetInsideValue'))

# Start of ApproximateSignedDistanceMapImageFilter_SetOutsideValue

`ApproximateSignedDistanceMapImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ApproximateSignedDistanceMapImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ApproximateSignedDistanceMapImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`ApproximateSignedDistanceMapImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__ApproximateSignedDistanceMapImageFilter', 'numeric')
class(`ApproximateSignedDistanceMapImageFilter_SetOutsideValue`) = c("SWIGFunction", class('ApproximateSignedDistanceMapImageFilter_SetOutsideValue'))

# Start of ApproximateSignedDistanceMapImageFilter_GetOutsideValue

`ApproximateSignedDistanceMapImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ApproximateSignedDistanceMapImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ApproximateSignedDistanceMapImageFilter_GetOutsideValue`, 'returnType') = 'numeric'
attr(`ApproximateSignedDistanceMapImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__ApproximateSignedDistanceMapImageFilter')
class(`ApproximateSignedDistanceMapImageFilter_GetOutsideValue`) = c("SWIGFunction", class('ApproximateSignedDistanceMapImageFilter_GetOutsideValue'))

# Start of ApproximateSignedDistanceMapImageFilter_GetName

`ApproximateSignedDistanceMapImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ApproximateSignedDistanceMapImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ApproximateSignedDistanceMapImageFilter_GetName`, 'returnType') = 'character'
attr(`ApproximateSignedDistanceMapImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ApproximateSignedDistanceMapImageFilter')
class(`ApproximateSignedDistanceMapImageFilter_GetName`) = c("SWIGFunction", class('ApproximateSignedDistanceMapImageFilter_GetName'))

# Start of ApproximateSignedDistanceMapImageFilter_ToString

`ApproximateSignedDistanceMapImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ApproximateSignedDistanceMapImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ApproximateSignedDistanceMapImageFilter_ToString`, 'returnType') = 'character'
attr(`ApproximateSignedDistanceMapImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ApproximateSignedDistanceMapImageFilter')
class(`ApproximateSignedDistanceMapImageFilter_ToString`) = c("SWIGFunction", class('ApproximateSignedDistanceMapImageFilter_ToString'))

# Start of ApproximateSignedDistanceMapImageFilter_Execute

`ApproximateSignedDistanceMapImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ApproximateSignedDistanceMapImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ApproximateSignedDistanceMapImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ApproximateSignedDistanceMapImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ApproximateSignedDistanceMapImageFilter', '_p_itk__simple__Image')
class(`ApproximateSignedDistanceMapImageFilter_Execute`) = c("SWIGFunction", class('ApproximateSignedDistanceMapImageFilter_Execute'))

# Start of accessor method for itk::simple::ApproximateSignedDistanceMapImageFilter
setMethod('$', '_p_itk__simple__ApproximateSignedDistanceMapImageFilter', function(x, name)

{
  accessorFuns = list('SetInsideValue' = ApproximateSignedDistanceMapImageFilter_SetInsideValue, 'GetInsideValue' = ApproximateSignedDistanceMapImageFilter_GetInsideValue, 'SetOutsideValue' = ApproximateSignedDistanceMapImageFilter_SetOutsideValue, 'GetOutsideValue' = ApproximateSignedDistanceMapImageFilter_GetOutsideValue, 'GetName' = ApproximateSignedDistanceMapImageFilter_GetName, 'ToString' = ApproximateSignedDistanceMapImageFilter_ToString, 'Execute' = ApproximateSignedDistanceMapImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ApproximateSignedDistanceMapImageFilter
# Start of ApproximateSignedDistanceMap

`ApproximateSignedDistanceMap__SWIG_0` = function(image1, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_ApproximateSignedDistanceMap__SWIG_0', image1, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ApproximateSignedDistanceMap__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ApproximateSignedDistanceMap__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`ApproximateSignedDistanceMap__SWIG_0`) = c("SWIGFunction", class('ApproximateSignedDistanceMap__SWIG_0'))

# Start of ApproximateSignedDistanceMap

`ApproximateSignedDistanceMap__SWIG_1` = function(image1, insideValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_ApproximateSignedDistanceMap__SWIG_1', image1, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ApproximateSignedDistanceMap__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ApproximateSignedDistanceMap__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`ApproximateSignedDistanceMap__SWIG_1`) = c("SWIGFunction", class('ApproximateSignedDistanceMap__SWIG_1'))

# Start of ApproximateSignedDistanceMap

`ApproximateSignedDistanceMap__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ApproximateSignedDistanceMap__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ApproximateSignedDistanceMap__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ApproximateSignedDistanceMap__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`ApproximateSignedDistanceMap__SWIG_2`) = c("SWIGFunction", class('ApproximateSignedDistanceMap__SWIG_2'))

`ApproximateSignedDistanceMap` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ApproximateSignedDistanceMap__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ApproximateSignedDistanceMap__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ApproximateSignedDistanceMap__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ApproximateSignedDistanceMap with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_AreaClosingImageFilter

`delete_AreaClosingImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_AreaClosingImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_AreaClosingImageFilter`, 'returnType') = 'void'
attr(`delete_AreaClosingImageFilter`, "inputTypes") = c('_p_itk__simple__AreaClosingImageFilter')
class(`delete_AreaClosingImageFilter`) = c("SWIGFunction", class('delete_AreaClosingImageFilter'))

# Start of new_AreaClosingImageFilter

`AreaClosingImageFilter` = function()
{
  ;ans = .Call('R_swig_new_AreaClosingImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AreaClosingImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AreaClosingImageFilter);
  ans
  
}

attr(`AreaClosingImageFilter`, 'returnType') = '_p_itk__simple__AreaClosingImageFilter'
class(`AreaClosingImageFilter`) = c("SWIGFunction", class('AreaClosingImageFilter'))

# Start of AreaClosingImageFilter_SetLambda

`AreaClosingImageFilter_SetLambda` = function(self, Lambda)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_AreaClosingImageFilter_SetLambda', self, Lambda, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AreaClosingImageFilter_SetLambda`, 'returnType') = 'void'
attr(`AreaClosingImageFilter_SetLambda`, "inputTypes") = c('_p_itk__simple__AreaClosingImageFilter', 'numeric')
class(`AreaClosingImageFilter_SetLambda`) = c("SWIGFunction", class('AreaClosingImageFilter_SetLambda'))

# Start of AreaClosingImageFilter_GetLambda

`AreaClosingImageFilter_GetLambda` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AreaClosingImageFilter_GetLambda', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AreaClosingImageFilter_GetLambda`, 'returnType') = 'numeric'
attr(`AreaClosingImageFilter_GetLambda`, "inputTypes") = c('_p_itk__simple__AreaClosingImageFilter')
class(`AreaClosingImageFilter_GetLambda`) = c("SWIGFunction", class('AreaClosingImageFilter_GetLambda'))

# Start of AreaClosingImageFilter_SetUseImageSpacing

`AreaClosingImageFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_AreaClosingImageFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AreaClosingImageFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`AreaClosingImageFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__AreaClosingImageFilter', 'logical')
class(`AreaClosingImageFilter_SetUseImageSpacing`) = c("SWIGFunction", class('AreaClosingImageFilter_SetUseImageSpacing'))

# Start of AreaClosingImageFilter_UseImageSpacingOn

`AreaClosingImageFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_AreaClosingImageFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AreaClosingImageFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`AreaClosingImageFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__AreaClosingImageFilter')
class(`AreaClosingImageFilter_UseImageSpacingOn`) = c("SWIGFunction", class('AreaClosingImageFilter_UseImageSpacingOn'))

# Start of AreaClosingImageFilter_UseImageSpacingOff

`AreaClosingImageFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_AreaClosingImageFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AreaClosingImageFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`AreaClosingImageFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__AreaClosingImageFilter')
class(`AreaClosingImageFilter_UseImageSpacingOff`) = c("SWIGFunction", class('AreaClosingImageFilter_UseImageSpacingOff'))

# Start of AreaClosingImageFilter_GetUseImageSpacing

`AreaClosingImageFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AreaClosingImageFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AreaClosingImageFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`AreaClosingImageFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__AreaClosingImageFilter')
class(`AreaClosingImageFilter_GetUseImageSpacing`) = c("SWIGFunction", class('AreaClosingImageFilter_GetUseImageSpacing'))

# Start of AreaClosingImageFilter_SetFullyConnected

`AreaClosingImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_AreaClosingImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AreaClosingImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`AreaClosingImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__AreaClosingImageFilter', 'logical')
class(`AreaClosingImageFilter_SetFullyConnected`) = c("SWIGFunction", class('AreaClosingImageFilter_SetFullyConnected'))

# Start of AreaClosingImageFilter_FullyConnectedOn

`AreaClosingImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_AreaClosingImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AreaClosingImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`AreaClosingImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__AreaClosingImageFilter')
class(`AreaClosingImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('AreaClosingImageFilter_FullyConnectedOn'))

# Start of AreaClosingImageFilter_FullyConnectedOff

`AreaClosingImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_AreaClosingImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AreaClosingImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`AreaClosingImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__AreaClosingImageFilter')
class(`AreaClosingImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('AreaClosingImageFilter_FullyConnectedOff'))

# Start of AreaClosingImageFilter_GetFullyConnected

`AreaClosingImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AreaClosingImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AreaClosingImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`AreaClosingImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__AreaClosingImageFilter')
class(`AreaClosingImageFilter_GetFullyConnected`) = c("SWIGFunction", class('AreaClosingImageFilter_GetFullyConnected'))

# Start of AreaClosingImageFilter_GetName

`AreaClosingImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AreaClosingImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AreaClosingImageFilter_GetName`, 'returnType') = 'character'
attr(`AreaClosingImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__AreaClosingImageFilter')
class(`AreaClosingImageFilter_GetName`) = c("SWIGFunction", class('AreaClosingImageFilter_GetName'))

# Start of AreaClosingImageFilter_ToString

`AreaClosingImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AreaClosingImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AreaClosingImageFilter_ToString`, 'returnType') = 'character'
attr(`AreaClosingImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__AreaClosingImageFilter')
class(`AreaClosingImageFilter_ToString`) = c("SWIGFunction", class('AreaClosingImageFilter_ToString'))

# Start of AreaClosingImageFilter_Execute

`AreaClosingImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AreaClosingImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AreaClosingImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`AreaClosingImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__AreaClosingImageFilter', '_p_itk__simple__Image')
class(`AreaClosingImageFilter_Execute`) = c("SWIGFunction", class('AreaClosingImageFilter_Execute'))

# Start of accessor method for itk::simple::AreaClosingImageFilter
setMethod('$', '_p_itk__simple__AreaClosingImageFilter', function(x, name)

{
  accessorFuns = list('SetLambda' = AreaClosingImageFilter_SetLambda, 'GetLambda' = AreaClosingImageFilter_GetLambda, 'SetUseImageSpacing' = AreaClosingImageFilter_SetUseImageSpacing, 'UseImageSpacingOn' = AreaClosingImageFilter_UseImageSpacingOn, 'UseImageSpacingOff' = AreaClosingImageFilter_UseImageSpacingOff, 'GetUseImageSpacing' = AreaClosingImageFilter_GetUseImageSpacing, 'SetFullyConnected' = AreaClosingImageFilter_SetFullyConnected, 'FullyConnectedOn' = AreaClosingImageFilter_FullyConnectedOn, 'FullyConnectedOff' = AreaClosingImageFilter_FullyConnectedOff, 'GetFullyConnected' = AreaClosingImageFilter_GetFullyConnected, 'GetName' = AreaClosingImageFilter_GetName, 'ToString' = AreaClosingImageFilter_ToString, 'Execute' = AreaClosingImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::AreaClosingImageFilter
# Start of AreaClosing

`AreaClosing__SWIG_0` = function(image1, lambda, useImageSpacing, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  useImageSpacing = as.logical(useImageSpacing);
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_AreaClosing__SWIG_0', image1, lambda, useImageSpacing, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AreaClosing__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`AreaClosing__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical', 'logical')
class(`AreaClosing__SWIG_0`) = c("SWIGFunction", class('AreaClosing__SWIG_0'))

# Start of AreaClosing

`AreaClosing__SWIG_1` = function(image1, lambda, useImageSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  useImageSpacing = as.logical(useImageSpacing);
  ;ans = .Call('R_swig_AreaClosing__SWIG_1', image1, lambda, useImageSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AreaClosing__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`AreaClosing__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`AreaClosing__SWIG_1`) = c("SWIGFunction", class('AreaClosing__SWIG_1'))

# Start of AreaClosing

`AreaClosing__SWIG_2` = function(image1, lambda, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_AreaClosing__SWIG_2', image1, lambda, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AreaClosing__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`AreaClosing__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`AreaClosing__SWIG_2`) = c("SWIGFunction", class('AreaClosing__SWIG_2'))

# Start of AreaClosing

`AreaClosing__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AreaClosing__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AreaClosing__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`AreaClosing__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`AreaClosing__SWIG_3`) = c("SWIGFunction", class('AreaClosing__SWIG_3'))

`AreaClosing` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- AreaClosing__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- AreaClosing__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- AreaClosing__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- AreaClosing__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for AreaClosing with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_AreaOpeningImageFilter

`delete_AreaOpeningImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_AreaOpeningImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_AreaOpeningImageFilter`, 'returnType') = 'void'
attr(`delete_AreaOpeningImageFilter`, "inputTypes") = c('_p_itk__simple__AreaOpeningImageFilter')
class(`delete_AreaOpeningImageFilter`) = c("SWIGFunction", class('delete_AreaOpeningImageFilter'))

# Start of new_AreaOpeningImageFilter

`AreaOpeningImageFilter` = function()
{
  ;ans = .Call('R_swig_new_AreaOpeningImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AreaOpeningImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AreaOpeningImageFilter);
  ans
  
}

attr(`AreaOpeningImageFilter`, 'returnType') = '_p_itk__simple__AreaOpeningImageFilter'
class(`AreaOpeningImageFilter`) = c("SWIGFunction", class('AreaOpeningImageFilter'))

# Start of AreaOpeningImageFilter_SetLambda

`AreaOpeningImageFilter_SetLambda` = function(self, Lambda)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_AreaOpeningImageFilter_SetLambda', self, Lambda, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AreaOpeningImageFilter_SetLambda`, 'returnType') = 'void'
attr(`AreaOpeningImageFilter_SetLambda`, "inputTypes") = c('_p_itk__simple__AreaOpeningImageFilter', 'numeric')
class(`AreaOpeningImageFilter_SetLambda`) = c("SWIGFunction", class('AreaOpeningImageFilter_SetLambda'))

# Start of AreaOpeningImageFilter_GetLambda

`AreaOpeningImageFilter_GetLambda` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AreaOpeningImageFilter_GetLambda', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AreaOpeningImageFilter_GetLambda`, 'returnType') = 'numeric'
attr(`AreaOpeningImageFilter_GetLambda`, "inputTypes") = c('_p_itk__simple__AreaOpeningImageFilter')
class(`AreaOpeningImageFilter_GetLambda`) = c("SWIGFunction", class('AreaOpeningImageFilter_GetLambda'))

# Start of AreaOpeningImageFilter_SetUseImageSpacing

`AreaOpeningImageFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_AreaOpeningImageFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AreaOpeningImageFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`AreaOpeningImageFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__AreaOpeningImageFilter', 'logical')
class(`AreaOpeningImageFilter_SetUseImageSpacing`) = c("SWIGFunction", class('AreaOpeningImageFilter_SetUseImageSpacing'))

# Start of AreaOpeningImageFilter_UseImageSpacingOn

`AreaOpeningImageFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_AreaOpeningImageFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AreaOpeningImageFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`AreaOpeningImageFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__AreaOpeningImageFilter')
class(`AreaOpeningImageFilter_UseImageSpacingOn`) = c("SWIGFunction", class('AreaOpeningImageFilter_UseImageSpacingOn'))

# Start of AreaOpeningImageFilter_UseImageSpacingOff

`AreaOpeningImageFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_AreaOpeningImageFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AreaOpeningImageFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`AreaOpeningImageFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__AreaOpeningImageFilter')
class(`AreaOpeningImageFilter_UseImageSpacingOff`) = c("SWIGFunction", class('AreaOpeningImageFilter_UseImageSpacingOff'))

# Start of AreaOpeningImageFilter_GetUseImageSpacing

`AreaOpeningImageFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AreaOpeningImageFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AreaOpeningImageFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`AreaOpeningImageFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__AreaOpeningImageFilter')
class(`AreaOpeningImageFilter_GetUseImageSpacing`) = c("SWIGFunction", class('AreaOpeningImageFilter_GetUseImageSpacing'))

# Start of AreaOpeningImageFilter_SetFullyConnected

`AreaOpeningImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_AreaOpeningImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AreaOpeningImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`AreaOpeningImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__AreaOpeningImageFilter', 'logical')
class(`AreaOpeningImageFilter_SetFullyConnected`) = c("SWIGFunction", class('AreaOpeningImageFilter_SetFullyConnected'))

# Start of AreaOpeningImageFilter_FullyConnectedOn

`AreaOpeningImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_AreaOpeningImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AreaOpeningImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`AreaOpeningImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__AreaOpeningImageFilter')
class(`AreaOpeningImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('AreaOpeningImageFilter_FullyConnectedOn'))

# Start of AreaOpeningImageFilter_FullyConnectedOff

`AreaOpeningImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_AreaOpeningImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`AreaOpeningImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`AreaOpeningImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__AreaOpeningImageFilter')
class(`AreaOpeningImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('AreaOpeningImageFilter_FullyConnectedOff'))

# Start of AreaOpeningImageFilter_GetFullyConnected

`AreaOpeningImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AreaOpeningImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AreaOpeningImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`AreaOpeningImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__AreaOpeningImageFilter')
class(`AreaOpeningImageFilter_GetFullyConnected`) = c("SWIGFunction", class('AreaOpeningImageFilter_GetFullyConnected'))

# Start of AreaOpeningImageFilter_GetName

`AreaOpeningImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AreaOpeningImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AreaOpeningImageFilter_GetName`, 'returnType') = 'character'
attr(`AreaOpeningImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__AreaOpeningImageFilter')
class(`AreaOpeningImageFilter_GetName`) = c("SWIGFunction", class('AreaOpeningImageFilter_GetName'))

# Start of AreaOpeningImageFilter_ToString

`AreaOpeningImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AreaOpeningImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AreaOpeningImageFilter_ToString`, 'returnType') = 'character'
attr(`AreaOpeningImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__AreaOpeningImageFilter')
class(`AreaOpeningImageFilter_ToString`) = c("SWIGFunction", class('AreaOpeningImageFilter_ToString'))

# Start of AreaOpeningImageFilter_Execute

`AreaOpeningImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AreaOpeningImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AreaOpeningImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`AreaOpeningImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__AreaOpeningImageFilter', '_p_itk__simple__Image')
class(`AreaOpeningImageFilter_Execute`) = c("SWIGFunction", class('AreaOpeningImageFilter_Execute'))

# Start of accessor method for itk::simple::AreaOpeningImageFilter
setMethod('$', '_p_itk__simple__AreaOpeningImageFilter', function(x, name)

{
  accessorFuns = list('SetLambda' = AreaOpeningImageFilter_SetLambda, 'GetLambda' = AreaOpeningImageFilter_GetLambda, 'SetUseImageSpacing' = AreaOpeningImageFilter_SetUseImageSpacing, 'UseImageSpacingOn' = AreaOpeningImageFilter_UseImageSpacingOn, 'UseImageSpacingOff' = AreaOpeningImageFilter_UseImageSpacingOff, 'GetUseImageSpacing' = AreaOpeningImageFilter_GetUseImageSpacing, 'SetFullyConnected' = AreaOpeningImageFilter_SetFullyConnected, 'FullyConnectedOn' = AreaOpeningImageFilter_FullyConnectedOn, 'FullyConnectedOff' = AreaOpeningImageFilter_FullyConnectedOff, 'GetFullyConnected' = AreaOpeningImageFilter_GetFullyConnected, 'GetName' = AreaOpeningImageFilter_GetName, 'ToString' = AreaOpeningImageFilter_ToString, 'Execute' = AreaOpeningImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::AreaOpeningImageFilter
# Start of AreaOpening

`AreaOpening__SWIG_0` = function(image1, lambda, useImageSpacing, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  useImageSpacing = as.logical(useImageSpacing);
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_AreaOpening__SWIG_0', image1, lambda, useImageSpacing, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AreaOpening__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`AreaOpening__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical', 'logical')
class(`AreaOpening__SWIG_0`) = c("SWIGFunction", class('AreaOpening__SWIG_0'))

# Start of AreaOpening

`AreaOpening__SWIG_1` = function(image1, lambda, useImageSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  useImageSpacing = as.logical(useImageSpacing);
  ;ans = .Call('R_swig_AreaOpening__SWIG_1', image1, lambda, useImageSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AreaOpening__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`AreaOpening__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`AreaOpening__SWIG_1`) = c("SWIGFunction", class('AreaOpening__SWIG_1'))

# Start of AreaOpening

`AreaOpening__SWIG_2` = function(image1, lambda, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_AreaOpening__SWIG_2', image1, lambda, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AreaOpening__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`AreaOpening__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`AreaOpening__SWIG_2`) = c("SWIGFunction", class('AreaOpening__SWIG_2'))

# Start of AreaOpening

`AreaOpening__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AreaOpening__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AreaOpening__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`AreaOpening__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`AreaOpening__SWIG_3`) = c("SWIGFunction", class('AreaOpening__SWIG_3'))

`AreaOpening` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- AreaOpening__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- AreaOpening__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- AreaOpening__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- AreaOpening__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for AreaOpening with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_AsinImageFilter

`delete_AsinImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_AsinImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_AsinImageFilter`, 'returnType') = 'void'
attr(`delete_AsinImageFilter`, "inputTypes") = c('_p_itk__simple__AsinImageFilter')
class(`delete_AsinImageFilter`) = c("SWIGFunction", class('delete_AsinImageFilter'))

# Start of new_AsinImageFilter

`AsinImageFilter` = function()
{
  ;ans = .Call('R_swig_new_AsinImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AsinImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AsinImageFilter);
  ans
  
}

attr(`AsinImageFilter`, 'returnType') = '_p_itk__simple__AsinImageFilter'
class(`AsinImageFilter`) = c("SWIGFunction", class('AsinImageFilter'))

# Start of AsinImageFilter_GetName

`AsinImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AsinImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AsinImageFilter_GetName`, 'returnType') = 'character'
attr(`AsinImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__AsinImageFilter')
class(`AsinImageFilter_GetName`) = c("SWIGFunction", class('AsinImageFilter_GetName'))

# Start of AsinImageFilter_ToString

`AsinImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AsinImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AsinImageFilter_ToString`, 'returnType') = 'character'
attr(`AsinImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__AsinImageFilter')
class(`AsinImageFilter_ToString`) = c("SWIGFunction", class('AsinImageFilter_ToString'))

# Start of AsinImageFilter_Execute

`AsinImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AsinImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AsinImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`AsinImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__AsinImageFilter', '_p_itk__simple__Image')
class(`AsinImageFilter_Execute`) = c("SWIGFunction", class('AsinImageFilter_Execute'))

# Start of accessor method for itk::simple::AsinImageFilter
setMethod('$', '_p_itk__simple__AsinImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = AsinImageFilter_GetName, 'ToString' = AsinImageFilter_ToString, 'Execute' = AsinImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::AsinImageFilter
# Start of Asin

`Asin` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Asin', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Asin`, 'returnType') = '_p_itk__simple__Image'
attr(`Asin`, "inputTypes") = c('_p_itk__simple__Image')
class(`Asin`) = c("SWIGFunction", class('Asin'))

# Start of delete_Atan2ImageFilter

`delete_Atan2ImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_Atan2ImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_Atan2ImageFilter`, 'returnType') = 'void'
attr(`delete_Atan2ImageFilter`, "inputTypes") = c('_p_itk__simple__Atan2ImageFilter')
class(`delete_Atan2ImageFilter`) = c("SWIGFunction", class('delete_Atan2ImageFilter'))

# Start of new_Atan2ImageFilter

`Atan2ImageFilter` = function()
{
  ;ans = .Call('R_swig_new_Atan2ImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Atan2ImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Atan2ImageFilter);
  ans
  
}

attr(`Atan2ImageFilter`, 'returnType') = '_p_itk__simple__Atan2ImageFilter'
class(`Atan2ImageFilter`) = c("SWIGFunction", class('Atan2ImageFilter'))

# Start of Atan2ImageFilter_GetName

`Atan2ImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Atan2ImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Atan2ImageFilter_GetName`, 'returnType') = 'character'
attr(`Atan2ImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__Atan2ImageFilter')
class(`Atan2ImageFilter_GetName`) = c("SWIGFunction", class('Atan2ImageFilter_GetName'))

# Start of Atan2ImageFilter_ToString

`Atan2ImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Atan2ImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Atan2ImageFilter_ToString`, 'returnType') = 'character'
attr(`Atan2ImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__Atan2ImageFilter')
class(`Atan2ImageFilter_ToString`) = c("SWIGFunction", class('Atan2ImageFilter_ToString'))

# Start of Atan2ImageFilter_Execute

`Atan2ImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Atan2ImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Atan2ImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Atan2ImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__Atan2ImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Atan2ImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('Atan2ImageFilter_Execute__SWIG_0'))

# Start of Atan2ImageFilter_Execute

`Atan2ImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Atan2ImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Atan2ImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Atan2ImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__Atan2ImageFilter', '_p_itk__simple__Image', 'numeric')
class(`Atan2ImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('Atan2ImageFilter_Execute__SWIG_1'))

# Start of Atan2ImageFilter_Execute

`Atan2ImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Atan2ImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Atan2ImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Atan2ImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__Atan2ImageFilter', 'numeric', '_p_itk__simple__Image')
class(`Atan2ImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('Atan2ImageFilter_Execute__SWIG_2'))

`Atan2ImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Atan2ImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- Atan2ImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Atan2ImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Atan2ImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Atan2ImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- Atan2ImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Atan2ImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::Atan2ImageFilter
setMethod('$', '_p_itk__simple__Atan2ImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = Atan2ImageFilter_GetName, 'ToString' = Atan2ImageFilter_ToString, 'Execute' = Atan2ImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::Atan2ImageFilter
# Start of Atan2

`Atan2__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Atan2__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Atan2__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Atan2__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Atan2__SWIG_0`) = c("SWIGFunction", class('Atan2__SWIG_0'))

# Start of Atan2

`Atan2__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Atan2__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Atan2__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Atan2__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`Atan2__SWIG_1`) = c("SWIGFunction", class('Atan2__SWIG_1'))

# Start of Atan2

`Atan2__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Atan2__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Atan2__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Atan2__SWIG_2`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`Atan2__SWIG_2`) = c("SWIGFunction", class('Atan2__SWIG_2'))

`Atan2` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Atan2__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Atan2__SWIG_1; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Atan2__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Atan2 with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_AtanImageFilter

`delete_AtanImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_AtanImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_AtanImageFilter`, 'returnType') = 'void'
attr(`delete_AtanImageFilter`, "inputTypes") = c('_p_itk__simple__AtanImageFilter')
class(`delete_AtanImageFilter`) = c("SWIGFunction", class('delete_AtanImageFilter'))

# Start of new_AtanImageFilter

`AtanImageFilter` = function()
{
  ;ans = .Call('R_swig_new_AtanImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__AtanImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_AtanImageFilter);
  ans
  
}

attr(`AtanImageFilter`, 'returnType') = '_p_itk__simple__AtanImageFilter'
class(`AtanImageFilter`) = c("SWIGFunction", class('AtanImageFilter'))

# Start of AtanImageFilter_GetName

`AtanImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AtanImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AtanImageFilter_GetName`, 'returnType') = 'character'
attr(`AtanImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__AtanImageFilter')
class(`AtanImageFilter_GetName`) = c("SWIGFunction", class('AtanImageFilter_GetName'))

# Start of AtanImageFilter_ToString

`AtanImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_AtanImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`AtanImageFilter_ToString`, 'returnType') = 'character'
attr(`AtanImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__AtanImageFilter')
class(`AtanImageFilter_ToString`) = c("SWIGFunction", class('AtanImageFilter_ToString'))

# Start of AtanImageFilter_Execute

`AtanImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_AtanImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`AtanImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`AtanImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__AtanImageFilter', '_p_itk__simple__Image')
class(`AtanImageFilter_Execute`) = c("SWIGFunction", class('AtanImageFilter_Execute'))

# Start of accessor method for itk::simple::AtanImageFilter
setMethod('$', '_p_itk__simple__AtanImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = AtanImageFilter_GetName, 'ToString' = AtanImageFilter_ToString, 'Execute' = AtanImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::AtanImageFilter
# Start of Atan

`Atan` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Atan', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Atan`, 'returnType') = '_p_itk__simple__Image'
attr(`Atan`, "inputTypes") = c('_p_itk__simple__Image')
class(`Atan`) = c("SWIGFunction", class('Atan'))

# Start of delete_BSplineDecompositionImageFilter

`delete_BSplineDecompositionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BSplineDecompositionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BSplineDecompositionImageFilter`, 'returnType') = 'void'
attr(`delete_BSplineDecompositionImageFilter`, "inputTypes") = c('_p_itk__simple__BSplineDecompositionImageFilter')
class(`delete_BSplineDecompositionImageFilter`) = c("SWIGFunction", class('delete_BSplineDecompositionImageFilter'))

# Start of new_BSplineDecompositionImageFilter

`BSplineDecompositionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BSplineDecompositionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BSplineDecompositionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BSplineDecompositionImageFilter);
  ans
  
}

attr(`BSplineDecompositionImageFilter`, 'returnType') = '_p_itk__simple__BSplineDecompositionImageFilter'
class(`BSplineDecompositionImageFilter`) = c("SWIGFunction", class('BSplineDecompositionImageFilter'))

# Start of BSplineDecompositionImageFilter_SetSplineOrder

`BSplineDecompositionImageFilter_SetSplineOrder` = function(self, SplineOrder)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SplineOrder = as.integer(SplineOrder);
  
  if(length(SplineOrder) > 1) {
    warning("using only the first element of SplineOrder");
  };
  
  ;ans = .Call('R_swig_BSplineDecompositionImageFilter_SetSplineOrder', self, SplineOrder, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BSplineDecompositionImageFilter_SetSplineOrder`, 'returnType') = 'void'
attr(`BSplineDecompositionImageFilter_SetSplineOrder`, "inputTypes") = c('_p_itk__simple__BSplineDecompositionImageFilter', 'integer')
class(`BSplineDecompositionImageFilter_SetSplineOrder`) = c("SWIGFunction", class('BSplineDecompositionImageFilter_SetSplineOrder'))

# Start of BSplineDecompositionImageFilter_GetSplineOrder

`BSplineDecompositionImageFilter_GetSplineOrder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BSplineDecompositionImageFilter_GetSplineOrder', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BSplineDecompositionImageFilter_GetSplineOrder`, 'returnType') = 'integer'
attr(`BSplineDecompositionImageFilter_GetSplineOrder`, "inputTypes") = c('_p_itk__simple__BSplineDecompositionImageFilter')
class(`BSplineDecompositionImageFilter_GetSplineOrder`) = c("SWIGFunction", class('BSplineDecompositionImageFilter_GetSplineOrder'))

# Start of BSplineDecompositionImageFilter_GetSplinePoles

`BSplineDecompositionImageFilter_GetSplinePoles` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BSplineDecompositionImageFilter_GetSplinePoles', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BSplineDecompositionImageFilter_GetSplinePoles`, 'returnType') = 'numeric'
attr(`BSplineDecompositionImageFilter_GetSplinePoles`, "inputTypes") = c('_p_itk__simple__BSplineDecompositionImageFilter')
class(`BSplineDecompositionImageFilter_GetSplinePoles`) = c("SWIGFunction", class('BSplineDecompositionImageFilter_GetSplinePoles'))

# Start of BSplineDecompositionImageFilter_GetName

`BSplineDecompositionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BSplineDecompositionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BSplineDecompositionImageFilter_GetName`, 'returnType') = 'character'
attr(`BSplineDecompositionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BSplineDecompositionImageFilter')
class(`BSplineDecompositionImageFilter_GetName`) = c("SWIGFunction", class('BSplineDecompositionImageFilter_GetName'))

# Start of BSplineDecompositionImageFilter_ToString

`BSplineDecompositionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BSplineDecompositionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BSplineDecompositionImageFilter_ToString`, 'returnType') = 'character'
attr(`BSplineDecompositionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BSplineDecompositionImageFilter')
class(`BSplineDecompositionImageFilter_ToString`) = c("SWIGFunction", class('BSplineDecompositionImageFilter_ToString'))

# Start of BSplineDecompositionImageFilter_Execute

`BSplineDecompositionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BSplineDecompositionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BSplineDecompositionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BSplineDecompositionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BSplineDecompositionImageFilter', '_p_itk__simple__Image')
class(`BSplineDecompositionImageFilter_Execute`) = c("SWIGFunction", class('BSplineDecompositionImageFilter_Execute'))

# Start of accessor method for itk::simple::BSplineDecompositionImageFilter
setMethod('$', '_p_itk__simple__BSplineDecompositionImageFilter', function(x, name)

{
  accessorFuns = list('SetSplineOrder' = BSplineDecompositionImageFilter_SetSplineOrder, 'GetSplineOrder' = BSplineDecompositionImageFilter_GetSplineOrder, 'GetSplinePoles' = BSplineDecompositionImageFilter_GetSplinePoles, 'GetName' = BSplineDecompositionImageFilter_GetName, 'ToString' = BSplineDecompositionImageFilter_ToString, 'Execute' = BSplineDecompositionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BSplineDecompositionImageFilter
# Start of BSplineDecomposition

`BSplineDecomposition__SWIG_0` = function(image1, splineOrder, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  splineOrder = as.integer(splineOrder);
  
  if(length(splineOrder) > 1) {
    warning("using only the first element of splineOrder");
  };
  
  ;ans = .Call('R_swig_BSplineDecomposition__SWIG_0', image1, splineOrder, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BSplineDecomposition__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BSplineDecomposition__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BSplineDecomposition__SWIG_0`) = c("SWIGFunction", class('BSplineDecomposition__SWIG_0'))

# Start of BSplineDecomposition

`BSplineDecomposition__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BSplineDecomposition__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BSplineDecomposition__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BSplineDecomposition__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`BSplineDecomposition__SWIG_1`) = c("SWIGFunction", class('BSplineDecomposition__SWIG_1'))

`BSplineDecomposition` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BSplineDecomposition__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BSplineDecomposition__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BSplineDecomposition with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BilateralImageFilter

`delete_BilateralImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BilateralImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BilateralImageFilter`, 'returnType') = 'void'
attr(`delete_BilateralImageFilter`, "inputTypes") = c('_p_itk__simple__BilateralImageFilter')
class(`delete_BilateralImageFilter`) = c("SWIGFunction", class('delete_BilateralImageFilter'))

# Start of new_BilateralImageFilter

`BilateralImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BilateralImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BilateralImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BilateralImageFilter);
  ans
  
}

attr(`BilateralImageFilter`, 'returnType') = '_p_itk__simple__BilateralImageFilter'
class(`BilateralImageFilter`) = c("SWIGFunction", class('BilateralImageFilter'))

# Start of BilateralImageFilter_SetDomainSigma

`BilateralImageFilter_SetDomainSigma` = function(self, DomainSigma)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BilateralImageFilter_SetDomainSigma', self, DomainSigma, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BilateralImageFilter_SetDomainSigma`, 'returnType') = 'void'
attr(`BilateralImageFilter_SetDomainSigma`, "inputTypes") = c('_p_itk__simple__BilateralImageFilter', 'numeric')
class(`BilateralImageFilter_SetDomainSigma`) = c("SWIGFunction", class('BilateralImageFilter_SetDomainSigma'))

# Start of BilateralImageFilter_GetDomainSigma

`BilateralImageFilter_GetDomainSigma` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BilateralImageFilter_GetDomainSigma', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BilateralImageFilter_GetDomainSigma`, 'returnType') = 'numeric'
attr(`BilateralImageFilter_GetDomainSigma`, "inputTypes") = c('_p_itk__simple__BilateralImageFilter')
class(`BilateralImageFilter_GetDomainSigma`) = c("SWIGFunction", class('BilateralImageFilter_GetDomainSigma'))

# Start of BilateralImageFilter_SetRangeSigma

`BilateralImageFilter_SetRangeSigma` = function(self, RangeSigma)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BilateralImageFilter_SetRangeSigma', self, RangeSigma, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BilateralImageFilter_SetRangeSigma`, 'returnType') = 'void'
attr(`BilateralImageFilter_SetRangeSigma`, "inputTypes") = c('_p_itk__simple__BilateralImageFilter', 'numeric')
class(`BilateralImageFilter_SetRangeSigma`) = c("SWIGFunction", class('BilateralImageFilter_SetRangeSigma'))

# Start of BilateralImageFilter_GetRangeSigma

`BilateralImageFilter_GetRangeSigma` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BilateralImageFilter_GetRangeSigma', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BilateralImageFilter_GetRangeSigma`, 'returnType') = 'numeric'
attr(`BilateralImageFilter_GetRangeSigma`, "inputTypes") = c('_p_itk__simple__BilateralImageFilter')
class(`BilateralImageFilter_GetRangeSigma`) = c("SWIGFunction", class('BilateralImageFilter_GetRangeSigma'))

# Start of BilateralImageFilter_SetNumberOfRangeGaussianSamples

`BilateralImageFilter_SetNumberOfRangeGaussianSamples` = function(self, NumberOfRangeGaussianSamples)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfRangeGaussianSamples = as.integer(NumberOfRangeGaussianSamples);
  
  if(length(NumberOfRangeGaussianSamples) > 1) {
    warning("using only the first element of NumberOfRangeGaussianSamples");
  };
  
  ;ans = .Call('R_swig_BilateralImageFilter_SetNumberOfRangeGaussianSamples', self, NumberOfRangeGaussianSamples, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BilateralImageFilter_SetNumberOfRangeGaussianSamples`, 'returnType') = 'void'
attr(`BilateralImageFilter_SetNumberOfRangeGaussianSamples`, "inputTypes") = c('_p_itk__simple__BilateralImageFilter', 'integer')
class(`BilateralImageFilter_SetNumberOfRangeGaussianSamples`) = c("SWIGFunction", class('BilateralImageFilter_SetNumberOfRangeGaussianSamples'))

# Start of BilateralImageFilter_GetNumberOfRangeGaussianSamples

`BilateralImageFilter_GetNumberOfRangeGaussianSamples` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BilateralImageFilter_GetNumberOfRangeGaussianSamples', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BilateralImageFilter_GetNumberOfRangeGaussianSamples`, 'returnType') = 'integer'
attr(`BilateralImageFilter_GetNumberOfRangeGaussianSamples`, "inputTypes") = c('_p_itk__simple__BilateralImageFilter')
class(`BilateralImageFilter_GetNumberOfRangeGaussianSamples`) = c("SWIGFunction", class('BilateralImageFilter_GetNumberOfRangeGaussianSamples'))

# Start of BilateralImageFilter_GetName

`BilateralImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BilateralImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BilateralImageFilter_GetName`, 'returnType') = 'character'
attr(`BilateralImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BilateralImageFilter')
class(`BilateralImageFilter_GetName`) = c("SWIGFunction", class('BilateralImageFilter_GetName'))

# Start of BilateralImageFilter_ToString

`BilateralImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BilateralImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BilateralImageFilter_ToString`, 'returnType') = 'character'
attr(`BilateralImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BilateralImageFilter')
class(`BilateralImageFilter_ToString`) = c("SWIGFunction", class('BilateralImageFilter_ToString'))

# Start of BilateralImageFilter_Execute

`BilateralImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BilateralImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BilateralImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BilateralImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BilateralImageFilter', '_p_itk__simple__Image')
class(`BilateralImageFilter_Execute`) = c("SWIGFunction", class('BilateralImageFilter_Execute'))

# Start of accessor method for itk::simple::BilateralImageFilter
setMethod('$', '_p_itk__simple__BilateralImageFilter', function(x, name)

{
  accessorFuns = list('SetDomainSigma' = BilateralImageFilter_SetDomainSigma, 'GetDomainSigma' = BilateralImageFilter_GetDomainSigma, 'SetRangeSigma' = BilateralImageFilter_SetRangeSigma, 'GetRangeSigma' = BilateralImageFilter_GetRangeSigma, 'SetNumberOfRangeGaussianSamples' = BilateralImageFilter_SetNumberOfRangeGaussianSamples, 'GetNumberOfRangeGaussianSamples' = BilateralImageFilter_GetNumberOfRangeGaussianSamples, 'GetName' = BilateralImageFilter_GetName, 'ToString' = BilateralImageFilter_ToString, 'Execute' = BilateralImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BilateralImageFilter
# Start of Bilateral

`Bilateral__SWIG_0` = function(image1, domainSigma, rangeSigma, numberOfRangeGaussianSamples, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  numberOfRangeGaussianSamples = as.integer(numberOfRangeGaussianSamples);
  
  if(length(numberOfRangeGaussianSamples) > 1) {
    warning("using only the first element of numberOfRangeGaussianSamples");
  };
  
  ;ans = .Call('R_swig_Bilateral__SWIG_0', image1, domainSigma, rangeSigma, numberOfRangeGaussianSamples, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Bilateral__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Bilateral__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'integer')
class(`Bilateral__SWIG_0`) = c("SWIGFunction", class('Bilateral__SWIG_0'))

# Start of Bilateral

`Bilateral__SWIG_1` = function(image1, domainSigma, rangeSigma, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_Bilateral__SWIG_1', image1, domainSigma, rangeSigma, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Bilateral__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Bilateral__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`Bilateral__SWIG_1`) = c("SWIGFunction", class('Bilateral__SWIG_1'))

# Start of Bilateral

`Bilateral__SWIG_2` = function(image1, domainSigma, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Bilateral__SWIG_2', image1, domainSigma, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Bilateral__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Bilateral__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`Bilateral__SWIG_2`) = c("SWIGFunction", class('Bilateral__SWIG_2'))

# Start of Bilateral

`Bilateral__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Bilateral__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Bilateral__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Bilateral__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`Bilateral__SWIG_3`) = c("SWIGFunction", class('Bilateral__SWIG_3'))

`Bilateral` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Bilateral__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Bilateral__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Bilateral__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- Bilateral__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Bilateral with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinShrinkImageFilter

`delete_BinShrinkImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinShrinkImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinShrinkImageFilter`, 'returnType') = 'void'
attr(`delete_BinShrinkImageFilter`, "inputTypes") = c('_p_itk__simple__BinShrinkImageFilter')
class(`delete_BinShrinkImageFilter`) = c("SWIGFunction", class('delete_BinShrinkImageFilter'))

# Start of new_BinShrinkImageFilter

`BinShrinkImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinShrinkImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinShrinkImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinShrinkImageFilter);
  ans
  
}

attr(`BinShrinkImageFilter`, 'returnType') = '_p_itk__simple__BinShrinkImageFilter'
class(`BinShrinkImageFilter`) = c("SWIGFunction", class('BinShrinkImageFilter'))

# Start of BinShrinkImageFilter_SetShrinkFactor

`BinShrinkImageFilter_SetShrinkFactor` = function(self, s)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  s = as.integer(s);
  
  if(length(s) > 1) {
    warning("using only the first element of s");
  };
  
  ;ans = .Call('R_swig_BinShrinkImageFilter_SetShrinkFactor', self, s, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinShrinkImageFilter_SetShrinkFactor`, 'returnType') = 'void'
attr(`BinShrinkImageFilter_SetShrinkFactor`, "inputTypes") = c('_p_itk__simple__BinShrinkImageFilter', 'integer')
class(`BinShrinkImageFilter_SetShrinkFactor`) = c("SWIGFunction", class('BinShrinkImageFilter_SetShrinkFactor'))

# Start of BinShrinkImageFilter_SetShrinkFactors

`BinShrinkImageFilter_SetShrinkFactors` = function(self, ShrinkFactors)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ShrinkFactors = as.integer(ShrinkFactors);
  ;ans = .Call('R_swig_BinShrinkImageFilter_SetShrinkFactors', self, ShrinkFactors, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinShrinkImageFilter_SetShrinkFactors`, 'returnType') = 'void'
attr(`BinShrinkImageFilter_SetShrinkFactors`, "inputTypes") = c('_p_itk__simple__BinShrinkImageFilter', 'integer')
class(`BinShrinkImageFilter_SetShrinkFactors`) = c("SWIGFunction", class('BinShrinkImageFilter_SetShrinkFactors'))

# Start of BinShrinkImageFilter_GetShrinkFactors

`BinShrinkImageFilter_GetShrinkFactors` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinShrinkImageFilter_GetShrinkFactors', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinShrinkImageFilter_GetShrinkFactors`, 'returnType') = 'integer'
attr(`BinShrinkImageFilter_GetShrinkFactors`, "inputTypes") = c('_p_itk__simple__BinShrinkImageFilter')
class(`BinShrinkImageFilter_GetShrinkFactors`) = c("SWIGFunction", class('BinShrinkImageFilter_GetShrinkFactors'))

# Start of BinShrinkImageFilter_GetName

`BinShrinkImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinShrinkImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinShrinkImageFilter_GetName`, 'returnType') = 'character'
attr(`BinShrinkImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinShrinkImageFilter')
class(`BinShrinkImageFilter_GetName`) = c("SWIGFunction", class('BinShrinkImageFilter_GetName'))

# Start of BinShrinkImageFilter_ToString

`BinShrinkImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinShrinkImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinShrinkImageFilter_ToString`, 'returnType') = 'character'
attr(`BinShrinkImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinShrinkImageFilter')
class(`BinShrinkImageFilter_ToString`) = c("SWIGFunction", class('BinShrinkImageFilter_ToString'))

# Start of BinShrinkImageFilter_Execute

`BinShrinkImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinShrinkImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinShrinkImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinShrinkImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinShrinkImageFilter', '_p_itk__simple__Image')
class(`BinShrinkImageFilter_Execute`) = c("SWIGFunction", class('BinShrinkImageFilter_Execute'))

# Start of accessor method for itk::simple::BinShrinkImageFilter
setMethod('$', '_p_itk__simple__BinShrinkImageFilter', function(x, name)

{
  accessorFuns = list('SetShrinkFactor' = BinShrinkImageFilter_SetShrinkFactor, 'SetShrinkFactors' = BinShrinkImageFilter_SetShrinkFactors, 'GetShrinkFactors' = BinShrinkImageFilter_GetShrinkFactors, 'GetName' = BinShrinkImageFilter_GetName, 'ToString' = BinShrinkImageFilter_ToString, 'Execute' = BinShrinkImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinShrinkImageFilter
# Start of BinShrink

`BinShrink__SWIG_0` = function(image1, shrinkFactors, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  shrinkFactors = as.integer(shrinkFactors);
  ;ans = .Call('R_swig_BinShrink__SWIG_0', image1, shrinkFactors, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinShrink__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinShrink__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BinShrink__SWIG_0`) = c("SWIGFunction", class('BinShrink__SWIG_0'))

# Start of BinShrink

`BinShrink__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinShrink__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinShrink__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinShrink__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinShrink__SWIG_1`) = c("SWIGFunction", class('BinShrink__SWIG_1'))

`BinShrink` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinShrink__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BinShrink__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinShrink with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryClosingByReconstructionImageFilter

`delete_BinaryClosingByReconstructionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryClosingByReconstructionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryClosingByReconstructionImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryClosingByReconstructionImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryClosingByReconstructionImageFilter')
class(`delete_BinaryClosingByReconstructionImageFilter`) = c("SWIGFunction", class('delete_BinaryClosingByReconstructionImageFilter'))

# Start of new_BinaryClosingByReconstructionImageFilter

`BinaryClosingByReconstructionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryClosingByReconstructionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryClosingByReconstructionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryClosingByReconstructionImageFilter);
  ans
  
}

attr(`BinaryClosingByReconstructionImageFilter`, 'returnType') = '_p_itk__simple__BinaryClosingByReconstructionImageFilter'
class(`BinaryClosingByReconstructionImageFilter`) = c("SWIGFunction", class('BinaryClosingByReconstructionImageFilter'))

# Start of BinaryClosingByReconstructionImageFilter_SetKernelRadius

`BinaryClosingByReconstructionImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_BinaryClosingByReconstructionImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryClosingByReconstructionImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`BinaryClosingByReconstructionImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__BinaryClosingByReconstructionImageFilter', 'integer')
class(`BinaryClosingByReconstructionImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('BinaryClosingByReconstructionImageFilter_SetKernelRadius__SWIG_0'))

# Start of BinaryClosingByReconstructionImageFilter_SetKernelRadius

`BinaryClosingByReconstructionImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_BinaryClosingByReconstructionImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryClosingByReconstructionImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`BinaryClosingByReconstructionImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__BinaryClosingByReconstructionImageFilter', 'integer')
class(`BinaryClosingByReconstructionImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('BinaryClosingByReconstructionImageFilter_SetKernelRadius__SWIG_1'))

`BinaryClosingByReconstructionImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__BinaryClosingByReconstructionImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BinaryClosingByReconstructionImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__BinaryClosingByReconstructionImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BinaryClosingByReconstructionImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryClosingByReconstructionImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of BinaryClosingByReconstructionImageFilter_GetKernelRadius

`BinaryClosingByReconstructionImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryClosingByReconstructionImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryClosingByReconstructionImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`BinaryClosingByReconstructionImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__BinaryClosingByReconstructionImageFilter')
class(`BinaryClosingByReconstructionImageFilter_GetKernelRadius`) = c("SWIGFunction", class('BinaryClosingByReconstructionImageFilter_GetKernelRadius'))

# Start of BinaryClosingByReconstructionImageFilter_SetKernelType

`BinaryClosingByReconstructionImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_BinaryClosingByReconstructionImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryClosingByReconstructionImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`BinaryClosingByReconstructionImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__BinaryClosingByReconstructionImageFilter', 'character')
class(`BinaryClosingByReconstructionImageFilter_SetKernelType`) = c("SWIGFunction", class('BinaryClosingByReconstructionImageFilter_SetKernelType'))

# Start of BinaryClosingByReconstructionImageFilter_GetKernelType

`BinaryClosingByReconstructionImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryClosingByReconstructionImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`BinaryClosingByReconstructionImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`BinaryClosingByReconstructionImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__BinaryClosingByReconstructionImageFilter')
class(`BinaryClosingByReconstructionImageFilter_GetKernelType`) = c("SWIGFunction", class('BinaryClosingByReconstructionImageFilter_GetKernelType'))

# Start of BinaryClosingByReconstructionImageFilter_SetForegroundValue

`BinaryClosingByReconstructionImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryClosingByReconstructionImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryClosingByReconstructionImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`BinaryClosingByReconstructionImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryClosingByReconstructionImageFilter', 'numeric')
class(`BinaryClosingByReconstructionImageFilter_SetForegroundValue`) = c("SWIGFunction", class('BinaryClosingByReconstructionImageFilter_SetForegroundValue'))

# Start of BinaryClosingByReconstructionImageFilter_GetForegroundValue

`BinaryClosingByReconstructionImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryClosingByReconstructionImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryClosingByReconstructionImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`BinaryClosingByReconstructionImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryClosingByReconstructionImageFilter')
class(`BinaryClosingByReconstructionImageFilter_GetForegroundValue`) = c("SWIGFunction", class('BinaryClosingByReconstructionImageFilter_GetForegroundValue'))

# Start of BinaryClosingByReconstructionImageFilter_SetFullyConnected

`BinaryClosingByReconstructionImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_BinaryClosingByReconstructionImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryClosingByReconstructionImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`BinaryClosingByReconstructionImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryClosingByReconstructionImageFilter', 'logical')
class(`BinaryClosingByReconstructionImageFilter_SetFullyConnected`) = c("SWIGFunction", class('BinaryClosingByReconstructionImageFilter_SetFullyConnected'))

# Start of BinaryClosingByReconstructionImageFilter_FullyConnectedOn

`BinaryClosingByReconstructionImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryClosingByReconstructionImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryClosingByReconstructionImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`BinaryClosingByReconstructionImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__BinaryClosingByReconstructionImageFilter')
class(`BinaryClosingByReconstructionImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('BinaryClosingByReconstructionImageFilter_FullyConnectedOn'))

# Start of BinaryClosingByReconstructionImageFilter_FullyConnectedOff

`BinaryClosingByReconstructionImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryClosingByReconstructionImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryClosingByReconstructionImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`BinaryClosingByReconstructionImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__BinaryClosingByReconstructionImageFilter')
class(`BinaryClosingByReconstructionImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('BinaryClosingByReconstructionImageFilter_FullyConnectedOff'))

# Start of BinaryClosingByReconstructionImageFilter_GetFullyConnected

`BinaryClosingByReconstructionImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryClosingByReconstructionImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryClosingByReconstructionImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`BinaryClosingByReconstructionImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryClosingByReconstructionImageFilter')
class(`BinaryClosingByReconstructionImageFilter_GetFullyConnected`) = c("SWIGFunction", class('BinaryClosingByReconstructionImageFilter_GetFullyConnected'))

# Start of BinaryClosingByReconstructionImageFilter_GetName

`BinaryClosingByReconstructionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryClosingByReconstructionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryClosingByReconstructionImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryClosingByReconstructionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryClosingByReconstructionImageFilter')
class(`BinaryClosingByReconstructionImageFilter_GetName`) = c("SWIGFunction", class('BinaryClosingByReconstructionImageFilter_GetName'))

# Start of BinaryClosingByReconstructionImageFilter_ToString

`BinaryClosingByReconstructionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryClosingByReconstructionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryClosingByReconstructionImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryClosingByReconstructionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryClosingByReconstructionImageFilter')
class(`BinaryClosingByReconstructionImageFilter_ToString`) = c("SWIGFunction", class('BinaryClosingByReconstructionImageFilter_ToString'))

# Start of BinaryClosingByReconstructionImageFilter_Execute

`BinaryClosingByReconstructionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryClosingByReconstructionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryClosingByReconstructionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryClosingByReconstructionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryClosingByReconstructionImageFilter', '_p_itk__simple__Image')
class(`BinaryClosingByReconstructionImageFilter_Execute`) = c("SWIGFunction", class('BinaryClosingByReconstructionImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryClosingByReconstructionImageFilter
setMethod('$', '_p_itk__simple__BinaryClosingByReconstructionImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = BinaryClosingByReconstructionImageFilter_SetKernelRadius, 'GetKernelRadius' = BinaryClosingByReconstructionImageFilter_GetKernelRadius, 'SetKernelType' = BinaryClosingByReconstructionImageFilter_SetKernelType, 'GetKernelType' = BinaryClosingByReconstructionImageFilter_GetKernelType, 'SetForegroundValue' = BinaryClosingByReconstructionImageFilter_SetForegroundValue, 'GetForegroundValue' = BinaryClosingByReconstructionImageFilter_GetForegroundValue, 'SetFullyConnected' = BinaryClosingByReconstructionImageFilter_SetFullyConnected, 'FullyConnectedOn' = BinaryClosingByReconstructionImageFilter_FullyConnectedOn, 'FullyConnectedOff' = BinaryClosingByReconstructionImageFilter_FullyConnectedOff, 'GetFullyConnected' = BinaryClosingByReconstructionImageFilter_GetFullyConnected, 'GetName' = BinaryClosingByReconstructionImageFilter_GetName, 'ToString' = BinaryClosingByReconstructionImageFilter_ToString, 'Execute' = BinaryClosingByReconstructionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryClosingByReconstructionImageFilter
# Start of BinaryClosingByReconstruction

`BinaryClosingByReconstruction__SWIG_0` = function(image1, kernelRadius, kernelType, foregroundValue, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_BinaryClosingByReconstruction__SWIG_0', image1, kernelRadius, kernelType, foregroundValue, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryClosingByReconstruction__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryClosingByReconstruction__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric', 'logical')
class(`BinaryClosingByReconstruction__SWIG_0`) = c("SWIGFunction", class('BinaryClosingByReconstruction__SWIG_0'))

# Start of BinaryClosingByReconstruction

`BinaryClosingByReconstruction__SWIG_1` = function(image1, kernelRadius, kernelType, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  ;ans = .Call('R_swig_BinaryClosingByReconstruction__SWIG_1', image1, kernelRadius, kernelType, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryClosingByReconstruction__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryClosingByReconstruction__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric')
class(`BinaryClosingByReconstruction__SWIG_1`) = c("SWIGFunction", class('BinaryClosingByReconstruction__SWIG_1'))

# Start of BinaryClosingByReconstruction

`BinaryClosingByReconstruction__SWIG_2` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_BinaryClosingByReconstruction__SWIG_2', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryClosingByReconstruction__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryClosingByReconstruction__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`BinaryClosingByReconstruction__SWIG_2`) = c("SWIGFunction", class('BinaryClosingByReconstruction__SWIG_2'))

# Start of BinaryClosingByReconstruction

`BinaryClosingByReconstruction__SWIG_3` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_BinaryClosingByReconstruction__SWIG_3', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryClosingByReconstruction__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryClosingByReconstruction__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BinaryClosingByReconstruction__SWIG_3`) = c("SWIGFunction", class('BinaryClosingByReconstruction__SWIG_3'))

# Start of BinaryClosingByReconstruction

`BinaryClosingByReconstruction__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryClosingByReconstruction__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryClosingByReconstruction__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryClosingByReconstruction__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryClosingByReconstruction__SWIG_4`) = c("SWIGFunction", class('BinaryClosingByReconstruction__SWIG_4'))

`BinaryClosingByReconstruction` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryClosingByReconstruction__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BinaryClosingByReconstruction__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryClosingByReconstruction__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- BinaryClosingByReconstruction__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- BinaryClosingByReconstruction__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryClosingByReconstruction with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryContourImageFilter

`delete_BinaryContourImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryContourImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryContourImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryContourImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryContourImageFilter')
class(`delete_BinaryContourImageFilter`) = c("SWIGFunction", class('delete_BinaryContourImageFilter'))

# Start of new_BinaryContourImageFilter

`BinaryContourImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryContourImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryContourImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryContourImageFilter);
  ans
  
}

attr(`BinaryContourImageFilter`, 'returnType') = '_p_itk__simple__BinaryContourImageFilter'
class(`BinaryContourImageFilter`) = c("SWIGFunction", class('BinaryContourImageFilter'))

# Start of BinaryContourImageFilter_SetFullyConnected

`BinaryContourImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_BinaryContourImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryContourImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`BinaryContourImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryContourImageFilter', 'logical')
class(`BinaryContourImageFilter_SetFullyConnected`) = c("SWIGFunction", class('BinaryContourImageFilter_SetFullyConnected'))

# Start of BinaryContourImageFilter_FullyConnectedOn

`BinaryContourImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryContourImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryContourImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`BinaryContourImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__BinaryContourImageFilter')
class(`BinaryContourImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('BinaryContourImageFilter_FullyConnectedOn'))

# Start of BinaryContourImageFilter_FullyConnectedOff

`BinaryContourImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryContourImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryContourImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`BinaryContourImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__BinaryContourImageFilter')
class(`BinaryContourImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('BinaryContourImageFilter_FullyConnectedOff'))

# Start of BinaryContourImageFilter_GetFullyConnected

`BinaryContourImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryContourImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryContourImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`BinaryContourImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryContourImageFilter')
class(`BinaryContourImageFilter_GetFullyConnected`) = c("SWIGFunction", class('BinaryContourImageFilter_GetFullyConnected'))

# Start of BinaryContourImageFilter_SetBackgroundValue

`BinaryContourImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryContourImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryContourImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`BinaryContourImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryContourImageFilter', 'numeric')
class(`BinaryContourImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('BinaryContourImageFilter_SetBackgroundValue'))

# Start of BinaryContourImageFilter_GetBackgroundValue

`BinaryContourImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryContourImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryContourImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`BinaryContourImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryContourImageFilter')
class(`BinaryContourImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('BinaryContourImageFilter_GetBackgroundValue'))

# Start of BinaryContourImageFilter_SetForegroundValue

`BinaryContourImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryContourImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryContourImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`BinaryContourImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryContourImageFilter', 'numeric')
class(`BinaryContourImageFilter_SetForegroundValue`) = c("SWIGFunction", class('BinaryContourImageFilter_SetForegroundValue'))

# Start of BinaryContourImageFilter_GetForegroundValue

`BinaryContourImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryContourImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryContourImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`BinaryContourImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryContourImageFilter')
class(`BinaryContourImageFilter_GetForegroundValue`) = c("SWIGFunction", class('BinaryContourImageFilter_GetForegroundValue'))

# Start of BinaryContourImageFilter_GetName

`BinaryContourImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryContourImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryContourImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryContourImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryContourImageFilter')
class(`BinaryContourImageFilter_GetName`) = c("SWIGFunction", class('BinaryContourImageFilter_GetName'))

# Start of BinaryContourImageFilter_ToString

`BinaryContourImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryContourImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryContourImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryContourImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryContourImageFilter')
class(`BinaryContourImageFilter_ToString`) = c("SWIGFunction", class('BinaryContourImageFilter_ToString'))

# Start of BinaryContourImageFilter_Execute

`BinaryContourImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryContourImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryContourImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryContourImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryContourImageFilter', '_p_itk__simple__Image')
class(`BinaryContourImageFilter_Execute`) = c("SWIGFunction", class('BinaryContourImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryContourImageFilter
setMethod('$', '_p_itk__simple__BinaryContourImageFilter', function(x, name)

{
  accessorFuns = list('SetFullyConnected' = BinaryContourImageFilter_SetFullyConnected, 'FullyConnectedOn' = BinaryContourImageFilter_FullyConnectedOn, 'FullyConnectedOff' = BinaryContourImageFilter_FullyConnectedOff, 'GetFullyConnected' = BinaryContourImageFilter_GetFullyConnected, 'SetBackgroundValue' = BinaryContourImageFilter_SetBackgroundValue, 'GetBackgroundValue' = BinaryContourImageFilter_GetBackgroundValue, 'SetForegroundValue' = BinaryContourImageFilter_SetForegroundValue, 'GetForegroundValue' = BinaryContourImageFilter_GetForegroundValue, 'GetName' = BinaryContourImageFilter_GetName, 'ToString' = BinaryContourImageFilter_ToString, 'Execute' = BinaryContourImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryContourImageFilter
# Start of BinaryContour

`BinaryContour__SWIG_0` = function(image1, fullyConnected, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  
  
  ;ans = .Call('R_swig_BinaryContour__SWIG_0', image1, fullyConnected, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryContour__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryContour__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'numeric', 'numeric')
class(`BinaryContour__SWIG_0`) = c("SWIGFunction", class('BinaryContour__SWIG_0'))

# Start of BinaryContour

`BinaryContour__SWIG_1` = function(image1, fullyConnected, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  
  ;ans = .Call('R_swig_BinaryContour__SWIG_1', image1, fullyConnected, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryContour__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryContour__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'numeric')
class(`BinaryContour__SWIG_1`) = c("SWIGFunction", class('BinaryContour__SWIG_1'))

# Start of BinaryContour

`BinaryContour__SWIG_2` = function(image1, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_BinaryContour__SWIG_2', image1, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryContour__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryContour__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`BinaryContour__SWIG_2`) = c("SWIGFunction", class('BinaryContour__SWIG_2'))

# Start of BinaryContour

`BinaryContour__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryContour__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryContour__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryContour__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryContour__SWIG_3`) = c("SWIGFunction", class('BinaryContour__SWIG_3'))

`BinaryContour` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryContour__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- BinaryContour__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryContour__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- BinaryContour__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryContour with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryDilateImageFilter

`delete_BinaryDilateImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryDilateImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryDilateImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryDilateImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter')
class(`delete_BinaryDilateImageFilter`) = c("SWIGFunction", class('delete_BinaryDilateImageFilter'))

# Start of new_BinaryDilateImageFilter

`BinaryDilateImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryDilateImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryDilateImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryDilateImageFilter);
  ans
  
}

attr(`BinaryDilateImageFilter`, 'returnType') = '_p_itk__simple__BinaryDilateImageFilter'
class(`BinaryDilateImageFilter`) = c("SWIGFunction", class('BinaryDilateImageFilter'))

# Start of BinaryDilateImageFilter_SetKernelRadius

`BinaryDilateImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_BinaryDilateImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryDilateImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`BinaryDilateImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter', 'integer')
class(`BinaryDilateImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('BinaryDilateImageFilter_SetKernelRadius__SWIG_0'))

# Start of BinaryDilateImageFilter_SetKernelRadius

`BinaryDilateImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_BinaryDilateImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryDilateImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`BinaryDilateImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter', 'integer')
class(`BinaryDilateImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('BinaryDilateImageFilter_SetKernelRadius__SWIG_1'))

`BinaryDilateImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__BinaryDilateImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BinaryDilateImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__BinaryDilateImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BinaryDilateImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryDilateImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of BinaryDilateImageFilter_GetKernelRadius

`BinaryDilateImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryDilateImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryDilateImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`BinaryDilateImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter')
class(`BinaryDilateImageFilter_GetKernelRadius`) = c("SWIGFunction", class('BinaryDilateImageFilter_GetKernelRadius'))

# Start of BinaryDilateImageFilter_SetKernelType

`BinaryDilateImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_BinaryDilateImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryDilateImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`BinaryDilateImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter', 'character')
class(`BinaryDilateImageFilter_SetKernelType`) = c("SWIGFunction", class('BinaryDilateImageFilter_SetKernelType'))

# Start of BinaryDilateImageFilter_GetKernelType

`BinaryDilateImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryDilateImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`BinaryDilateImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`BinaryDilateImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter')
class(`BinaryDilateImageFilter_GetKernelType`) = c("SWIGFunction", class('BinaryDilateImageFilter_GetKernelType'))

# Start of BinaryDilateImageFilter_SetBackgroundValue

`BinaryDilateImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryDilateImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryDilateImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`BinaryDilateImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter', 'numeric')
class(`BinaryDilateImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('BinaryDilateImageFilter_SetBackgroundValue'))

# Start of BinaryDilateImageFilter_GetBackgroundValue

`BinaryDilateImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryDilateImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryDilateImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`BinaryDilateImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter')
class(`BinaryDilateImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('BinaryDilateImageFilter_GetBackgroundValue'))

# Start of BinaryDilateImageFilter_SetForegroundValue

`BinaryDilateImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryDilateImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryDilateImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`BinaryDilateImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter', 'numeric')
class(`BinaryDilateImageFilter_SetForegroundValue`) = c("SWIGFunction", class('BinaryDilateImageFilter_SetForegroundValue'))

# Start of BinaryDilateImageFilter_GetForegroundValue

`BinaryDilateImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryDilateImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryDilateImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`BinaryDilateImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter')
class(`BinaryDilateImageFilter_GetForegroundValue`) = c("SWIGFunction", class('BinaryDilateImageFilter_GetForegroundValue'))

# Start of BinaryDilateImageFilter_SetBoundaryToForeground

`BinaryDilateImageFilter_SetBoundaryToForeground` = function(self, BoundaryToForeground)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BoundaryToForeground = as.logical(BoundaryToForeground);
  ;ans = .Call('R_swig_BinaryDilateImageFilter_SetBoundaryToForeground', self, BoundaryToForeground, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryDilateImageFilter_SetBoundaryToForeground`, 'returnType') = 'void'
attr(`BinaryDilateImageFilter_SetBoundaryToForeground`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter', 'logical')
class(`BinaryDilateImageFilter_SetBoundaryToForeground`) = c("SWIGFunction", class('BinaryDilateImageFilter_SetBoundaryToForeground'))

# Start of BinaryDilateImageFilter_BoundaryToForegroundOn

`BinaryDilateImageFilter_BoundaryToForegroundOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryDilateImageFilter_BoundaryToForegroundOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryDilateImageFilter_BoundaryToForegroundOn`, 'returnType') = 'void'
attr(`BinaryDilateImageFilter_BoundaryToForegroundOn`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter')
class(`BinaryDilateImageFilter_BoundaryToForegroundOn`) = c("SWIGFunction", class('BinaryDilateImageFilter_BoundaryToForegroundOn'))

# Start of BinaryDilateImageFilter_BoundaryToForegroundOff

`BinaryDilateImageFilter_BoundaryToForegroundOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryDilateImageFilter_BoundaryToForegroundOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryDilateImageFilter_BoundaryToForegroundOff`, 'returnType') = 'void'
attr(`BinaryDilateImageFilter_BoundaryToForegroundOff`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter')
class(`BinaryDilateImageFilter_BoundaryToForegroundOff`) = c("SWIGFunction", class('BinaryDilateImageFilter_BoundaryToForegroundOff'))

# Start of BinaryDilateImageFilter_GetBoundaryToForeground

`BinaryDilateImageFilter_GetBoundaryToForeground` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryDilateImageFilter_GetBoundaryToForeground', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryDilateImageFilter_GetBoundaryToForeground`, 'returnType') = 'logical'
attr(`BinaryDilateImageFilter_GetBoundaryToForeground`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter')
class(`BinaryDilateImageFilter_GetBoundaryToForeground`) = c("SWIGFunction", class('BinaryDilateImageFilter_GetBoundaryToForeground'))

# Start of BinaryDilateImageFilter_GetName

`BinaryDilateImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryDilateImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryDilateImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryDilateImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter')
class(`BinaryDilateImageFilter_GetName`) = c("SWIGFunction", class('BinaryDilateImageFilter_GetName'))

# Start of BinaryDilateImageFilter_ToString

`BinaryDilateImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryDilateImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryDilateImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryDilateImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter')
class(`BinaryDilateImageFilter_ToString`) = c("SWIGFunction", class('BinaryDilateImageFilter_ToString'))

# Start of BinaryDilateImageFilter_Execute

`BinaryDilateImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryDilateImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryDilateImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryDilateImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryDilateImageFilter', '_p_itk__simple__Image')
class(`BinaryDilateImageFilter_Execute`) = c("SWIGFunction", class('BinaryDilateImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryDilateImageFilter
setMethod('$', '_p_itk__simple__BinaryDilateImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = BinaryDilateImageFilter_SetKernelRadius, 'GetKernelRadius' = BinaryDilateImageFilter_GetKernelRadius, 'SetKernelType' = BinaryDilateImageFilter_SetKernelType, 'GetKernelType' = BinaryDilateImageFilter_GetKernelType, 'SetBackgroundValue' = BinaryDilateImageFilter_SetBackgroundValue, 'GetBackgroundValue' = BinaryDilateImageFilter_GetBackgroundValue, 'SetForegroundValue' = BinaryDilateImageFilter_SetForegroundValue, 'GetForegroundValue' = BinaryDilateImageFilter_GetForegroundValue, 'SetBoundaryToForeground' = BinaryDilateImageFilter_SetBoundaryToForeground, 'BoundaryToForegroundOn' = BinaryDilateImageFilter_BoundaryToForegroundOn, 'BoundaryToForegroundOff' = BinaryDilateImageFilter_BoundaryToForegroundOff, 'GetBoundaryToForeground' = BinaryDilateImageFilter_GetBoundaryToForeground, 'GetName' = BinaryDilateImageFilter_GetName, 'ToString' = BinaryDilateImageFilter_ToString, 'Execute' = BinaryDilateImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryDilateImageFilter
# Start of BinaryDilate

`BinaryDilate__SWIG_0` = function(image1, kernelRadius, kernelType, backgroundValue, foregroundValue, boundaryToForeground, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  
  boundaryToForeground = as.logical(boundaryToForeground);
  ;ans = .Call('R_swig_BinaryDilate__SWIG_0', image1, kernelRadius, kernelType, backgroundValue, foregroundValue, boundaryToForeground, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryDilate__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryDilate__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric', 'numeric', 'logical')
class(`BinaryDilate__SWIG_0`) = c("SWIGFunction", class('BinaryDilate__SWIG_0'))

# Start of BinaryDilate

`BinaryDilate__SWIG_1` = function(image1, kernelRadius, kernelType, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  
  ;ans = .Call('R_swig_BinaryDilate__SWIG_1', image1, kernelRadius, kernelType, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryDilate__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryDilate__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric', 'numeric')
class(`BinaryDilate__SWIG_1`) = c("SWIGFunction", class('BinaryDilate__SWIG_1'))

# Start of BinaryDilate

`BinaryDilate__SWIG_2` = function(image1, kernelRadius, kernelType, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  ;ans = .Call('R_swig_BinaryDilate__SWIG_2', image1, kernelRadius, kernelType, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryDilate__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryDilate__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric')
class(`BinaryDilate__SWIG_2`) = c("SWIGFunction", class('BinaryDilate__SWIG_2'))

# Start of BinaryDilate

`BinaryDilate__SWIG_3` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_BinaryDilate__SWIG_3', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryDilate__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryDilate__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`BinaryDilate__SWIG_3`) = c("SWIGFunction", class('BinaryDilate__SWIG_3'))

# Start of BinaryDilate

`BinaryDilate__SWIG_4` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_BinaryDilate__SWIG_4', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryDilate__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryDilate__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BinaryDilate__SWIG_4`) = c("SWIGFunction", class('BinaryDilate__SWIG_4'))

# Start of BinaryDilate

`BinaryDilate__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryDilate__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryDilate__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryDilate__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryDilate__SWIG_5`) = c("SWIGFunction", class('BinaryDilate__SWIG_5'))

`BinaryDilate` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryDilate__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BinaryDilate__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryDilate__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- BinaryDilate__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- BinaryDilate__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- BinaryDilate__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryDilate with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryErodeImageFilter

`delete_BinaryErodeImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryErodeImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryErodeImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryErodeImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter')
class(`delete_BinaryErodeImageFilter`) = c("SWIGFunction", class('delete_BinaryErodeImageFilter'))

# Start of new_BinaryErodeImageFilter

`BinaryErodeImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryErodeImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryErodeImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryErodeImageFilter);
  ans
  
}

attr(`BinaryErodeImageFilter`, 'returnType') = '_p_itk__simple__BinaryErodeImageFilter'
class(`BinaryErodeImageFilter`) = c("SWIGFunction", class('BinaryErodeImageFilter'))

# Start of BinaryErodeImageFilter_SetKernelRadius

`BinaryErodeImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_BinaryErodeImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryErodeImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`BinaryErodeImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter', 'integer')
class(`BinaryErodeImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('BinaryErodeImageFilter_SetKernelRadius__SWIG_0'))

# Start of BinaryErodeImageFilter_SetKernelRadius

`BinaryErodeImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_BinaryErodeImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryErodeImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`BinaryErodeImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter', 'integer')
class(`BinaryErodeImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('BinaryErodeImageFilter_SetKernelRadius__SWIG_1'))

`BinaryErodeImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__BinaryErodeImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BinaryErodeImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__BinaryErodeImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BinaryErodeImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryErodeImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of BinaryErodeImageFilter_GetKernelRadius

`BinaryErodeImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryErodeImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryErodeImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`BinaryErodeImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter')
class(`BinaryErodeImageFilter_GetKernelRadius`) = c("SWIGFunction", class('BinaryErodeImageFilter_GetKernelRadius'))

# Start of BinaryErodeImageFilter_SetKernelType

`BinaryErodeImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_BinaryErodeImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryErodeImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`BinaryErodeImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter', 'character')
class(`BinaryErodeImageFilter_SetKernelType`) = c("SWIGFunction", class('BinaryErodeImageFilter_SetKernelType'))

# Start of BinaryErodeImageFilter_GetKernelType

`BinaryErodeImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryErodeImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`BinaryErodeImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`BinaryErodeImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter')
class(`BinaryErodeImageFilter_GetKernelType`) = c("SWIGFunction", class('BinaryErodeImageFilter_GetKernelType'))

# Start of BinaryErodeImageFilter_SetBackgroundValue

`BinaryErodeImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryErodeImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryErodeImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`BinaryErodeImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter', 'numeric')
class(`BinaryErodeImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('BinaryErodeImageFilter_SetBackgroundValue'))

# Start of BinaryErodeImageFilter_GetBackgroundValue

`BinaryErodeImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryErodeImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryErodeImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`BinaryErodeImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter')
class(`BinaryErodeImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('BinaryErodeImageFilter_GetBackgroundValue'))

# Start of BinaryErodeImageFilter_SetForegroundValue

`BinaryErodeImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryErodeImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryErodeImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`BinaryErodeImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter', 'numeric')
class(`BinaryErodeImageFilter_SetForegroundValue`) = c("SWIGFunction", class('BinaryErodeImageFilter_SetForegroundValue'))

# Start of BinaryErodeImageFilter_GetForegroundValue

`BinaryErodeImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryErodeImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryErodeImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`BinaryErodeImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter')
class(`BinaryErodeImageFilter_GetForegroundValue`) = c("SWIGFunction", class('BinaryErodeImageFilter_GetForegroundValue'))

# Start of BinaryErodeImageFilter_SetBoundaryToForeground

`BinaryErodeImageFilter_SetBoundaryToForeground` = function(self, BoundaryToForeground)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BoundaryToForeground = as.logical(BoundaryToForeground);
  ;ans = .Call('R_swig_BinaryErodeImageFilter_SetBoundaryToForeground', self, BoundaryToForeground, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryErodeImageFilter_SetBoundaryToForeground`, 'returnType') = 'void'
attr(`BinaryErodeImageFilter_SetBoundaryToForeground`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter', 'logical')
class(`BinaryErodeImageFilter_SetBoundaryToForeground`) = c("SWIGFunction", class('BinaryErodeImageFilter_SetBoundaryToForeground'))

# Start of BinaryErodeImageFilter_BoundaryToForegroundOn

`BinaryErodeImageFilter_BoundaryToForegroundOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryErodeImageFilter_BoundaryToForegroundOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryErodeImageFilter_BoundaryToForegroundOn`, 'returnType') = 'void'
attr(`BinaryErodeImageFilter_BoundaryToForegroundOn`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter')
class(`BinaryErodeImageFilter_BoundaryToForegroundOn`) = c("SWIGFunction", class('BinaryErodeImageFilter_BoundaryToForegroundOn'))

# Start of BinaryErodeImageFilter_BoundaryToForegroundOff

`BinaryErodeImageFilter_BoundaryToForegroundOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryErodeImageFilter_BoundaryToForegroundOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryErodeImageFilter_BoundaryToForegroundOff`, 'returnType') = 'void'
attr(`BinaryErodeImageFilter_BoundaryToForegroundOff`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter')
class(`BinaryErodeImageFilter_BoundaryToForegroundOff`) = c("SWIGFunction", class('BinaryErodeImageFilter_BoundaryToForegroundOff'))

# Start of BinaryErodeImageFilter_GetBoundaryToForeground

`BinaryErodeImageFilter_GetBoundaryToForeground` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryErodeImageFilter_GetBoundaryToForeground', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryErodeImageFilter_GetBoundaryToForeground`, 'returnType') = 'logical'
attr(`BinaryErodeImageFilter_GetBoundaryToForeground`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter')
class(`BinaryErodeImageFilter_GetBoundaryToForeground`) = c("SWIGFunction", class('BinaryErodeImageFilter_GetBoundaryToForeground'))

# Start of BinaryErodeImageFilter_GetName

`BinaryErodeImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryErodeImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryErodeImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryErodeImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter')
class(`BinaryErodeImageFilter_GetName`) = c("SWIGFunction", class('BinaryErodeImageFilter_GetName'))

# Start of BinaryErodeImageFilter_ToString

`BinaryErodeImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryErodeImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryErodeImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryErodeImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter')
class(`BinaryErodeImageFilter_ToString`) = c("SWIGFunction", class('BinaryErodeImageFilter_ToString'))

# Start of BinaryErodeImageFilter_Execute

`BinaryErodeImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryErodeImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryErodeImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryErodeImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryErodeImageFilter', '_p_itk__simple__Image')
class(`BinaryErodeImageFilter_Execute`) = c("SWIGFunction", class('BinaryErodeImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryErodeImageFilter
setMethod('$', '_p_itk__simple__BinaryErodeImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = BinaryErodeImageFilter_SetKernelRadius, 'GetKernelRadius' = BinaryErodeImageFilter_GetKernelRadius, 'SetKernelType' = BinaryErodeImageFilter_SetKernelType, 'GetKernelType' = BinaryErodeImageFilter_GetKernelType, 'SetBackgroundValue' = BinaryErodeImageFilter_SetBackgroundValue, 'GetBackgroundValue' = BinaryErodeImageFilter_GetBackgroundValue, 'SetForegroundValue' = BinaryErodeImageFilter_SetForegroundValue, 'GetForegroundValue' = BinaryErodeImageFilter_GetForegroundValue, 'SetBoundaryToForeground' = BinaryErodeImageFilter_SetBoundaryToForeground, 'BoundaryToForegroundOn' = BinaryErodeImageFilter_BoundaryToForegroundOn, 'BoundaryToForegroundOff' = BinaryErodeImageFilter_BoundaryToForegroundOff, 'GetBoundaryToForeground' = BinaryErodeImageFilter_GetBoundaryToForeground, 'GetName' = BinaryErodeImageFilter_GetName, 'ToString' = BinaryErodeImageFilter_ToString, 'Execute' = BinaryErodeImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryErodeImageFilter
# Start of BinaryErode

`BinaryErode__SWIG_0` = function(image1, kernelRadius, kernelType, backgroundValue, foregroundValue, boundaryToForeground, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  
  boundaryToForeground = as.logical(boundaryToForeground);
  ;ans = .Call('R_swig_BinaryErode__SWIG_0', image1, kernelRadius, kernelType, backgroundValue, foregroundValue, boundaryToForeground, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryErode__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryErode__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric', 'numeric', 'logical')
class(`BinaryErode__SWIG_0`) = c("SWIGFunction", class('BinaryErode__SWIG_0'))

# Start of BinaryErode

`BinaryErode__SWIG_1` = function(image1, kernelRadius, kernelType, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  
  ;ans = .Call('R_swig_BinaryErode__SWIG_1', image1, kernelRadius, kernelType, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryErode__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryErode__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric', 'numeric')
class(`BinaryErode__SWIG_1`) = c("SWIGFunction", class('BinaryErode__SWIG_1'))

# Start of BinaryErode

`BinaryErode__SWIG_2` = function(image1, kernelRadius, kernelType, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  ;ans = .Call('R_swig_BinaryErode__SWIG_2', image1, kernelRadius, kernelType, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryErode__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryErode__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric')
class(`BinaryErode__SWIG_2`) = c("SWIGFunction", class('BinaryErode__SWIG_2'))

# Start of BinaryErode

`BinaryErode__SWIG_3` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_BinaryErode__SWIG_3', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryErode__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryErode__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`BinaryErode__SWIG_3`) = c("SWIGFunction", class('BinaryErode__SWIG_3'))

# Start of BinaryErode

`BinaryErode__SWIG_4` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_BinaryErode__SWIG_4', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryErode__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryErode__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BinaryErode__SWIG_4`) = c("SWIGFunction", class('BinaryErode__SWIG_4'))

# Start of BinaryErode

`BinaryErode__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryErode__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryErode__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryErode__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryErode__SWIG_5`) = c("SWIGFunction", class('BinaryErode__SWIG_5'))

`BinaryErode` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryErode__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BinaryErode__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryErode__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- BinaryErode__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- BinaryErode__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- BinaryErode__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryErode with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryFillholeImageFilter

`delete_BinaryFillholeImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryFillholeImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryFillholeImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryFillholeImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryFillholeImageFilter')
class(`delete_BinaryFillholeImageFilter`) = c("SWIGFunction", class('delete_BinaryFillholeImageFilter'))

# Start of new_BinaryFillholeImageFilter

`BinaryFillholeImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryFillholeImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryFillholeImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryFillholeImageFilter);
  ans
  
}

attr(`BinaryFillholeImageFilter`, 'returnType') = '_p_itk__simple__BinaryFillholeImageFilter'
class(`BinaryFillholeImageFilter`) = c("SWIGFunction", class('BinaryFillholeImageFilter'))

# Start of BinaryFillholeImageFilter_SetFullyConnected

`BinaryFillholeImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_BinaryFillholeImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryFillholeImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`BinaryFillholeImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryFillholeImageFilter', 'logical')
class(`BinaryFillholeImageFilter_SetFullyConnected`) = c("SWIGFunction", class('BinaryFillholeImageFilter_SetFullyConnected'))

# Start of BinaryFillholeImageFilter_FullyConnectedOn

`BinaryFillholeImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryFillholeImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryFillholeImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`BinaryFillholeImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__BinaryFillholeImageFilter')
class(`BinaryFillholeImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('BinaryFillholeImageFilter_FullyConnectedOn'))

# Start of BinaryFillholeImageFilter_FullyConnectedOff

`BinaryFillholeImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryFillholeImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryFillholeImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`BinaryFillholeImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__BinaryFillholeImageFilter')
class(`BinaryFillholeImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('BinaryFillholeImageFilter_FullyConnectedOff'))

# Start of BinaryFillholeImageFilter_GetFullyConnected

`BinaryFillholeImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryFillholeImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryFillholeImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`BinaryFillholeImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryFillholeImageFilter')
class(`BinaryFillholeImageFilter_GetFullyConnected`) = c("SWIGFunction", class('BinaryFillholeImageFilter_GetFullyConnected'))

# Start of BinaryFillholeImageFilter_SetForegroundValue

`BinaryFillholeImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryFillholeImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryFillholeImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`BinaryFillholeImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryFillholeImageFilter', 'numeric')
class(`BinaryFillholeImageFilter_SetForegroundValue`) = c("SWIGFunction", class('BinaryFillholeImageFilter_SetForegroundValue'))

# Start of BinaryFillholeImageFilter_GetForegroundValue

`BinaryFillholeImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryFillholeImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryFillholeImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`BinaryFillholeImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryFillholeImageFilter')
class(`BinaryFillholeImageFilter_GetForegroundValue`) = c("SWIGFunction", class('BinaryFillholeImageFilter_GetForegroundValue'))

# Start of BinaryFillholeImageFilter_GetName

`BinaryFillholeImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryFillholeImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryFillholeImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryFillholeImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryFillholeImageFilter')
class(`BinaryFillholeImageFilter_GetName`) = c("SWIGFunction", class('BinaryFillholeImageFilter_GetName'))

# Start of BinaryFillholeImageFilter_ToString

`BinaryFillholeImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryFillholeImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryFillholeImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryFillholeImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryFillholeImageFilter')
class(`BinaryFillholeImageFilter_ToString`) = c("SWIGFunction", class('BinaryFillholeImageFilter_ToString'))

# Start of BinaryFillholeImageFilter_Execute

`BinaryFillholeImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryFillholeImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryFillholeImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryFillholeImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryFillholeImageFilter', '_p_itk__simple__Image')
class(`BinaryFillholeImageFilter_Execute`) = c("SWIGFunction", class('BinaryFillholeImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryFillholeImageFilter
setMethod('$', '_p_itk__simple__BinaryFillholeImageFilter', function(x, name)

{
  accessorFuns = list('SetFullyConnected' = BinaryFillholeImageFilter_SetFullyConnected, 'FullyConnectedOn' = BinaryFillholeImageFilter_FullyConnectedOn, 'FullyConnectedOff' = BinaryFillholeImageFilter_FullyConnectedOff, 'GetFullyConnected' = BinaryFillholeImageFilter_GetFullyConnected, 'SetForegroundValue' = BinaryFillholeImageFilter_SetForegroundValue, 'GetForegroundValue' = BinaryFillholeImageFilter_GetForegroundValue, 'GetName' = BinaryFillholeImageFilter_GetName, 'ToString' = BinaryFillholeImageFilter_ToString, 'Execute' = BinaryFillholeImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryFillholeImageFilter
# Start of BinaryFillhole

`BinaryFillhole__SWIG_0` = function(image1, fullyConnected, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  
  ;ans = .Call('R_swig_BinaryFillhole__SWIG_0', image1, fullyConnected, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryFillhole__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryFillhole__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'numeric')
class(`BinaryFillhole__SWIG_0`) = c("SWIGFunction", class('BinaryFillhole__SWIG_0'))

# Start of BinaryFillhole

`BinaryFillhole__SWIG_1` = function(image1, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_BinaryFillhole__SWIG_1', image1, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryFillhole__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryFillhole__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`BinaryFillhole__SWIG_1`) = c("SWIGFunction", class('BinaryFillhole__SWIG_1'))

# Start of BinaryFillhole

`BinaryFillhole__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryFillhole__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryFillhole__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryFillhole__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryFillhole__SWIG_2`) = c("SWIGFunction", class('BinaryFillhole__SWIG_2'))

`BinaryFillhole` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryFillhole__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- BinaryFillhole__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryFillhole__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryFillhole with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryGrindPeakImageFilter

`delete_BinaryGrindPeakImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryGrindPeakImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryGrindPeakImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryGrindPeakImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryGrindPeakImageFilter')
class(`delete_BinaryGrindPeakImageFilter`) = c("SWIGFunction", class('delete_BinaryGrindPeakImageFilter'))

# Start of new_BinaryGrindPeakImageFilter

`BinaryGrindPeakImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryGrindPeakImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryGrindPeakImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryGrindPeakImageFilter);
  ans
  
}

attr(`BinaryGrindPeakImageFilter`, 'returnType') = '_p_itk__simple__BinaryGrindPeakImageFilter'
class(`BinaryGrindPeakImageFilter`) = c("SWIGFunction", class('BinaryGrindPeakImageFilter'))

# Start of BinaryGrindPeakImageFilter_SetFullyConnected

`BinaryGrindPeakImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_BinaryGrindPeakImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryGrindPeakImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`BinaryGrindPeakImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryGrindPeakImageFilter', 'logical')
class(`BinaryGrindPeakImageFilter_SetFullyConnected`) = c("SWIGFunction", class('BinaryGrindPeakImageFilter_SetFullyConnected'))

# Start of BinaryGrindPeakImageFilter_FullyConnectedOn

`BinaryGrindPeakImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryGrindPeakImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryGrindPeakImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`BinaryGrindPeakImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__BinaryGrindPeakImageFilter')
class(`BinaryGrindPeakImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('BinaryGrindPeakImageFilter_FullyConnectedOn'))

# Start of BinaryGrindPeakImageFilter_FullyConnectedOff

`BinaryGrindPeakImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryGrindPeakImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryGrindPeakImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`BinaryGrindPeakImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__BinaryGrindPeakImageFilter')
class(`BinaryGrindPeakImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('BinaryGrindPeakImageFilter_FullyConnectedOff'))

# Start of BinaryGrindPeakImageFilter_GetFullyConnected

`BinaryGrindPeakImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryGrindPeakImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryGrindPeakImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`BinaryGrindPeakImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryGrindPeakImageFilter')
class(`BinaryGrindPeakImageFilter_GetFullyConnected`) = c("SWIGFunction", class('BinaryGrindPeakImageFilter_GetFullyConnected'))

# Start of BinaryGrindPeakImageFilter_SetForegroundValue

`BinaryGrindPeakImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryGrindPeakImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryGrindPeakImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`BinaryGrindPeakImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryGrindPeakImageFilter', 'numeric')
class(`BinaryGrindPeakImageFilter_SetForegroundValue`) = c("SWIGFunction", class('BinaryGrindPeakImageFilter_SetForegroundValue'))

# Start of BinaryGrindPeakImageFilter_GetForegroundValue

`BinaryGrindPeakImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryGrindPeakImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryGrindPeakImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`BinaryGrindPeakImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryGrindPeakImageFilter')
class(`BinaryGrindPeakImageFilter_GetForegroundValue`) = c("SWIGFunction", class('BinaryGrindPeakImageFilter_GetForegroundValue'))

# Start of BinaryGrindPeakImageFilter_SetBackgroundValue

`BinaryGrindPeakImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryGrindPeakImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryGrindPeakImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`BinaryGrindPeakImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryGrindPeakImageFilter', 'numeric')
class(`BinaryGrindPeakImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('BinaryGrindPeakImageFilter_SetBackgroundValue'))

# Start of BinaryGrindPeakImageFilter_GetBackgroundValue

`BinaryGrindPeakImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryGrindPeakImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryGrindPeakImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`BinaryGrindPeakImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryGrindPeakImageFilter')
class(`BinaryGrindPeakImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('BinaryGrindPeakImageFilter_GetBackgroundValue'))

# Start of BinaryGrindPeakImageFilter_GetName

`BinaryGrindPeakImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryGrindPeakImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryGrindPeakImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryGrindPeakImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryGrindPeakImageFilter')
class(`BinaryGrindPeakImageFilter_GetName`) = c("SWIGFunction", class('BinaryGrindPeakImageFilter_GetName'))

# Start of BinaryGrindPeakImageFilter_ToString

`BinaryGrindPeakImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryGrindPeakImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryGrindPeakImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryGrindPeakImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryGrindPeakImageFilter')
class(`BinaryGrindPeakImageFilter_ToString`) = c("SWIGFunction", class('BinaryGrindPeakImageFilter_ToString'))

# Start of BinaryGrindPeakImageFilter_Execute

`BinaryGrindPeakImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryGrindPeakImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryGrindPeakImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryGrindPeakImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryGrindPeakImageFilter', '_p_itk__simple__Image')
class(`BinaryGrindPeakImageFilter_Execute`) = c("SWIGFunction", class('BinaryGrindPeakImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryGrindPeakImageFilter
setMethod('$', '_p_itk__simple__BinaryGrindPeakImageFilter', function(x, name)

{
  accessorFuns = list('SetFullyConnected' = BinaryGrindPeakImageFilter_SetFullyConnected, 'FullyConnectedOn' = BinaryGrindPeakImageFilter_FullyConnectedOn, 'FullyConnectedOff' = BinaryGrindPeakImageFilter_FullyConnectedOff, 'GetFullyConnected' = BinaryGrindPeakImageFilter_GetFullyConnected, 'SetForegroundValue' = BinaryGrindPeakImageFilter_SetForegroundValue, 'GetForegroundValue' = BinaryGrindPeakImageFilter_GetForegroundValue, 'SetBackgroundValue' = BinaryGrindPeakImageFilter_SetBackgroundValue, 'GetBackgroundValue' = BinaryGrindPeakImageFilter_GetBackgroundValue, 'GetName' = BinaryGrindPeakImageFilter_GetName, 'ToString' = BinaryGrindPeakImageFilter_ToString, 'Execute' = BinaryGrindPeakImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryGrindPeakImageFilter
# Start of BinaryGrindPeak

`BinaryGrindPeak__SWIG_0` = function(image1, fullyConnected, foregroundValue, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  
  
  ;ans = .Call('R_swig_BinaryGrindPeak__SWIG_0', image1, fullyConnected, foregroundValue, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryGrindPeak__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryGrindPeak__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'numeric', 'numeric')
class(`BinaryGrindPeak__SWIG_0`) = c("SWIGFunction", class('BinaryGrindPeak__SWIG_0'))

# Start of BinaryGrindPeak

`BinaryGrindPeak__SWIG_1` = function(image1, fullyConnected, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  
  ;ans = .Call('R_swig_BinaryGrindPeak__SWIG_1', image1, fullyConnected, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryGrindPeak__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryGrindPeak__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'numeric')
class(`BinaryGrindPeak__SWIG_1`) = c("SWIGFunction", class('BinaryGrindPeak__SWIG_1'))

# Start of BinaryGrindPeak

`BinaryGrindPeak__SWIG_2` = function(image1, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_BinaryGrindPeak__SWIG_2', image1, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryGrindPeak__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryGrindPeak__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`BinaryGrindPeak__SWIG_2`) = c("SWIGFunction", class('BinaryGrindPeak__SWIG_2'))

# Start of BinaryGrindPeak

`BinaryGrindPeak__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryGrindPeak__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryGrindPeak__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryGrindPeak__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryGrindPeak__SWIG_3`) = c("SWIGFunction", class('BinaryGrindPeak__SWIG_3'))

`BinaryGrindPeak` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryGrindPeak__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- BinaryGrindPeak__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryGrindPeak__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- BinaryGrindPeak__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryGrindPeak with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryImageToLabelMapFilter

`delete_BinaryImageToLabelMapFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryImageToLabelMapFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryImageToLabelMapFilter`, 'returnType') = 'void'
attr(`delete_BinaryImageToLabelMapFilter`, "inputTypes") = c('_p_itk__simple__BinaryImageToLabelMapFilter')
class(`delete_BinaryImageToLabelMapFilter`) = c("SWIGFunction", class('delete_BinaryImageToLabelMapFilter'))

# Start of new_BinaryImageToLabelMapFilter

`BinaryImageToLabelMapFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryImageToLabelMapFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryImageToLabelMapFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryImageToLabelMapFilter);
  ans
  
}

attr(`BinaryImageToLabelMapFilter`, 'returnType') = '_p_itk__simple__BinaryImageToLabelMapFilter'
class(`BinaryImageToLabelMapFilter`) = c("SWIGFunction", class('BinaryImageToLabelMapFilter'))

# Start of BinaryImageToLabelMapFilter_SetFullyConnected

`BinaryImageToLabelMapFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_BinaryImageToLabelMapFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryImageToLabelMapFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`BinaryImageToLabelMapFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryImageToLabelMapFilter', 'logical')
class(`BinaryImageToLabelMapFilter_SetFullyConnected`) = c("SWIGFunction", class('BinaryImageToLabelMapFilter_SetFullyConnected'))

# Start of BinaryImageToLabelMapFilter_FullyConnectedOn

`BinaryImageToLabelMapFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryImageToLabelMapFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryImageToLabelMapFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`BinaryImageToLabelMapFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__BinaryImageToLabelMapFilter')
class(`BinaryImageToLabelMapFilter_FullyConnectedOn`) = c("SWIGFunction", class('BinaryImageToLabelMapFilter_FullyConnectedOn'))

# Start of BinaryImageToLabelMapFilter_FullyConnectedOff

`BinaryImageToLabelMapFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryImageToLabelMapFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryImageToLabelMapFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`BinaryImageToLabelMapFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__BinaryImageToLabelMapFilter')
class(`BinaryImageToLabelMapFilter_FullyConnectedOff`) = c("SWIGFunction", class('BinaryImageToLabelMapFilter_FullyConnectedOff'))

# Start of BinaryImageToLabelMapFilter_GetFullyConnected

`BinaryImageToLabelMapFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryImageToLabelMapFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryImageToLabelMapFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`BinaryImageToLabelMapFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryImageToLabelMapFilter')
class(`BinaryImageToLabelMapFilter_GetFullyConnected`) = c("SWIGFunction", class('BinaryImageToLabelMapFilter_GetFullyConnected'))

# Start of BinaryImageToLabelMapFilter_SetInputForegroundValue

`BinaryImageToLabelMapFilter_SetInputForegroundValue` = function(self, InputForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryImageToLabelMapFilter_SetInputForegroundValue', self, InputForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryImageToLabelMapFilter_SetInputForegroundValue`, 'returnType') = 'void'
attr(`BinaryImageToLabelMapFilter_SetInputForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryImageToLabelMapFilter', 'numeric')
class(`BinaryImageToLabelMapFilter_SetInputForegroundValue`) = c("SWIGFunction", class('BinaryImageToLabelMapFilter_SetInputForegroundValue'))

# Start of BinaryImageToLabelMapFilter_GetInputForegroundValue

`BinaryImageToLabelMapFilter_GetInputForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryImageToLabelMapFilter_GetInputForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryImageToLabelMapFilter_GetInputForegroundValue`, 'returnType') = 'numeric'
attr(`BinaryImageToLabelMapFilter_GetInputForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryImageToLabelMapFilter')
class(`BinaryImageToLabelMapFilter_GetInputForegroundValue`) = c("SWIGFunction", class('BinaryImageToLabelMapFilter_GetInputForegroundValue'))

# Start of BinaryImageToLabelMapFilter_SetOutputBackgroundValue

`BinaryImageToLabelMapFilter_SetOutputBackgroundValue` = function(self, OutputBackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryImageToLabelMapFilter_SetOutputBackgroundValue', self, OutputBackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryImageToLabelMapFilter_SetOutputBackgroundValue`, 'returnType') = 'void'
attr(`BinaryImageToLabelMapFilter_SetOutputBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryImageToLabelMapFilter', 'numeric')
class(`BinaryImageToLabelMapFilter_SetOutputBackgroundValue`) = c("SWIGFunction", class('BinaryImageToLabelMapFilter_SetOutputBackgroundValue'))

# Start of BinaryImageToLabelMapFilter_GetOutputBackgroundValue

`BinaryImageToLabelMapFilter_GetOutputBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryImageToLabelMapFilter_GetOutputBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryImageToLabelMapFilter_GetOutputBackgroundValue`, 'returnType') = 'numeric'
attr(`BinaryImageToLabelMapFilter_GetOutputBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryImageToLabelMapFilter')
class(`BinaryImageToLabelMapFilter_GetOutputBackgroundValue`) = c("SWIGFunction", class('BinaryImageToLabelMapFilter_GetOutputBackgroundValue'))

# Start of BinaryImageToLabelMapFilter_GetNumberOfObjects

`BinaryImageToLabelMapFilter_GetNumberOfObjects` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryImageToLabelMapFilter_GetNumberOfObjects', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryImageToLabelMapFilter_GetNumberOfObjects`, 'returnType') = 'integer'
attr(`BinaryImageToLabelMapFilter_GetNumberOfObjects`, "inputTypes") = c('_p_itk__simple__BinaryImageToLabelMapFilter')
class(`BinaryImageToLabelMapFilter_GetNumberOfObjects`) = c("SWIGFunction", class('BinaryImageToLabelMapFilter_GetNumberOfObjects'))

# Start of BinaryImageToLabelMapFilter_GetName

`BinaryImageToLabelMapFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryImageToLabelMapFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryImageToLabelMapFilter_GetName`, 'returnType') = 'character'
attr(`BinaryImageToLabelMapFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryImageToLabelMapFilter')
class(`BinaryImageToLabelMapFilter_GetName`) = c("SWIGFunction", class('BinaryImageToLabelMapFilter_GetName'))

# Start of BinaryImageToLabelMapFilter_ToString

`BinaryImageToLabelMapFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryImageToLabelMapFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryImageToLabelMapFilter_ToString`, 'returnType') = 'character'
attr(`BinaryImageToLabelMapFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryImageToLabelMapFilter')
class(`BinaryImageToLabelMapFilter_ToString`) = c("SWIGFunction", class('BinaryImageToLabelMapFilter_ToString'))

# Start of BinaryImageToLabelMapFilter_Execute

`BinaryImageToLabelMapFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryImageToLabelMapFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryImageToLabelMapFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryImageToLabelMapFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryImageToLabelMapFilter', '_p_itk__simple__Image')
class(`BinaryImageToLabelMapFilter_Execute`) = c("SWIGFunction", class('BinaryImageToLabelMapFilter_Execute'))

# Start of accessor method for itk::simple::BinaryImageToLabelMapFilter
setMethod('$', '_p_itk__simple__BinaryImageToLabelMapFilter', function(x, name)

{
  accessorFuns = list('SetFullyConnected' = BinaryImageToLabelMapFilter_SetFullyConnected, 'FullyConnectedOn' = BinaryImageToLabelMapFilter_FullyConnectedOn, 'FullyConnectedOff' = BinaryImageToLabelMapFilter_FullyConnectedOff, 'GetFullyConnected' = BinaryImageToLabelMapFilter_GetFullyConnected, 'SetInputForegroundValue' = BinaryImageToLabelMapFilter_SetInputForegroundValue, 'GetInputForegroundValue' = BinaryImageToLabelMapFilter_GetInputForegroundValue, 'SetOutputBackgroundValue' = BinaryImageToLabelMapFilter_SetOutputBackgroundValue, 'GetOutputBackgroundValue' = BinaryImageToLabelMapFilter_GetOutputBackgroundValue, 'GetNumberOfObjects' = BinaryImageToLabelMapFilter_GetNumberOfObjects, 'GetName' = BinaryImageToLabelMapFilter_GetName, 'ToString' = BinaryImageToLabelMapFilter_ToString, 'Execute' = BinaryImageToLabelMapFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryImageToLabelMapFilter
# Start of BinaryImageToLabelMap

`BinaryImageToLabelMap__SWIG_0` = function(image1, fullyConnected, inputForegroundValue, outputBackgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  
  
  ;ans = .Call('R_swig_BinaryImageToLabelMap__SWIG_0', image1, fullyConnected, inputForegroundValue, outputBackgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryImageToLabelMap__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryImageToLabelMap__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'numeric', 'numeric')
class(`BinaryImageToLabelMap__SWIG_0`) = c("SWIGFunction", class('BinaryImageToLabelMap__SWIG_0'))

# Start of BinaryImageToLabelMap

`BinaryImageToLabelMap__SWIG_1` = function(image1, fullyConnected, inputForegroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  
  ;ans = .Call('R_swig_BinaryImageToLabelMap__SWIG_1', image1, fullyConnected, inputForegroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryImageToLabelMap__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryImageToLabelMap__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'numeric')
class(`BinaryImageToLabelMap__SWIG_1`) = c("SWIGFunction", class('BinaryImageToLabelMap__SWIG_1'))

# Start of BinaryImageToLabelMap

`BinaryImageToLabelMap__SWIG_2` = function(image1, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_BinaryImageToLabelMap__SWIG_2', image1, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryImageToLabelMap__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryImageToLabelMap__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`BinaryImageToLabelMap__SWIG_2`) = c("SWIGFunction", class('BinaryImageToLabelMap__SWIG_2'))

# Start of BinaryImageToLabelMap

`BinaryImageToLabelMap__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryImageToLabelMap__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryImageToLabelMap__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryImageToLabelMap__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryImageToLabelMap__SWIG_3`) = c("SWIGFunction", class('BinaryImageToLabelMap__SWIG_3'))

`BinaryImageToLabelMap` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryImageToLabelMap__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- BinaryImageToLabelMap__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryImageToLabelMap__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- BinaryImageToLabelMap__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryImageToLabelMap with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryMagnitudeImageFilter

`delete_BinaryMagnitudeImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryMagnitudeImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryMagnitudeImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryMagnitudeImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryMagnitudeImageFilter')
class(`delete_BinaryMagnitudeImageFilter`) = c("SWIGFunction", class('delete_BinaryMagnitudeImageFilter'))

# Start of new_BinaryMagnitudeImageFilter

`BinaryMagnitudeImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryMagnitudeImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryMagnitudeImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryMagnitudeImageFilter);
  ans
  
}

attr(`BinaryMagnitudeImageFilter`, 'returnType') = '_p_itk__simple__BinaryMagnitudeImageFilter'
class(`BinaryMagnitudeImageFilter`) = c("SWIGFunction", class('BinaryMagnitudeImageFilter'))

# Start of BinaryMagnitudeImageFilter_GetName

`BinaryMagnitudeImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMagnitudeImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMagnitudeImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryMagnitudeImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryMagnitudeImageFilter')
class(`BinaryMagnitudeImageFilter_GetName`) = c("SWIGFunction", class('BinaryMagnitudeImageFilter_GetName'))

# Start of BinaryMagnitudeImageFilter_ToString

`BinaryMagnitudeImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMagnitudeImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMagnitudeImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryMagnitudeImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryMagnitudeImageFilter')
class(`BinaryMagnitudeImageFilter_ToString`) = c("SWIGFunction", class('BinaryMagnitudeImageFilter_ToString'))

# Start of BinaryMagnitudeImageFilter_Execute

`BinaryMagnitudeImageFilter_Execute` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_BinaryMagnitudeImageFilter_Execute', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMagnitudeImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMagnitudeImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryMagnitudeImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`BinaryMagnitudeImageFilter_Execute`) = c("SWIGFunction", class('BinaryMagnitudeImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryMagnitudeImageFilter
setMethod('$', '_p_itk__simple__BinaryMagnitudeImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = BinaryMagnitudeImageFilter_GetName, 'ToString' = BinaryMagnitudeImageFilter_ToString, 'Execute' = BinaryMagnitudeImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryMagnitudeImageFilter
# Start of BinaryMagnitude

`BinaryMagnitude` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_BinaryMagnitude', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMagnitude`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMagnitude`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`BinaryMagnitude`) = c("SWIGFunction", class('BinaryMagnitude'))

# Start of delete_BinaryMedianImageFilter

`delete_BinaryMedianImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryMedianImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryMedianImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryMedianImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryMedianImageFilter')
class(`delete_BinaryMedianImageFilter`) = c("SWIGFunction", class('delete_BinaryMedianImageFilter'))

# Start of new_BinaryMedianImageFilter

`BinaryMedianImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryMedianImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryMedianImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryMedianImageFilter);
  ans
  
}

attr(`BinaryMedianImageFilter`, 'returnType') = '_p_itk__simple__BinaryMedianImageFilter'
class(`BinaryMedianImageFilter`) = c("SWIGFunction", class('BinaryMedianImageFilter'))

# Start of BinaryMedianImageFilter_SetRadius

`BinaryMedianImageFilter_SetRadius__SWIG_0` = function(self, Radius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Radius = as.integer(Radius);
  ;ans = .Call('R_swig_BinaryMedianImageFilter_SetRadius__SWIG_0', self, Radius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMedianImageFilter_SetRadius__SWIG_0`, 'returnType') = 'void'
attr(`BinaryMedianImageFilter_SetRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__BinaryMedianImageFilter', 'integer')
class(`BinaryMedianImageFilter_SetRadius__SWIG_0`) = c("SWIGFunction", class('BinaryMedianImageFilter_SetRadius__SWIG_0'))

# Start of BinaryMedianImageFilter_SetRadius

`BinaryMedianImageFilter_SetRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_BinaryMedianImageFilter_SetRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMedianImageFilter_SetRadius__SWIG_1`, 'returnType') = 'void'
attr(`BinaryMedianImageFilter_SetRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__BinaryMedianImageFilter', 'integer')
class(`BinaryMedianImageFilter_SetRadius__SWIG_1`) = c("SWIGFunction", class('BinaryMedianImageFilter_SetRadius__SWIG_1'))

`BinaryMedianImageFilter_SetRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__BinaryMedianImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BinaryMedianImageFilter_SetRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__BinaryMedianImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BinaryMedianImageFilter_SetRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryMedianImageFilter_SetRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of BinaryMedianImageFilter_GetRadius

`BinaryMedianImageFilter_GetRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMedianImageFilter_GetRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMedianImageFilter_GetRadius`, 'returnType') = 'integer'
attr(`BinaryMedianImageFilter_GetRadius`, "inputTypes") = c('_p_itk__simple__BinaryMedianImageFilter')
class(`BinaryMedianImageFilter_GetRadius`) = c("SWIGFunction", class('BinaryMedianImageFilter_GetRadius'))

# Start of BinaryMedianImageFilter_SetForegroundValue

`BinaryMedianImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryMedianImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMedianImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`BinaryMedianImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryMedianImageFilter', 'numeric')
class(`BinaryMedianImageFilter_SetForegroundValue`) = c("SWIGFunction", class('BinaryMedianImageFilter_SetForegroundValue'))

# Start of BinaryMedianImageFilter_GetForegroundValue

`BinaryMedianImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMedianImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMedianImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`BinaryMedianImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryMedianImageFilter')
class(`BinaryMedianImageFilter_GetForegroundValue`) = c("SWIGFunction", class('BinaryMedianImageFilter_GetForegroundValue'))

# Start of BinaryMedianImageFilter_SetBackgroundValue

`BinaryMedianImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryMedianImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMedianImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`BinaryMedianImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryMedianImageFilter', 'numeric')
class(`BinaryMedianImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('BinaryMedianImageFilter_SetBackgroundValue'))

# Start of BinaryMedianImageFilter_GetBackgroundValue

`BinaryMedianImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMedianImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMedianImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`BinaryMedianImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryMedianImageFilter')
class(`BinaryMedianImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('BinaryMedianImageFilter_GetBackgroundValue'))

# Start of BinaryMedianImageFilter_GetName

`BinaryMedianImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMedianImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMedianImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryMedianImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryMedianImageFilter')
class(`BinaryMedianImageFilter_GetName`) = c("SWIGFunction", class('BinaryMedianImageFilter_GetName'))

# Start of BinaryMedianImageFilter_ToString

`BinaryMedianImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMedianImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMedianImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryMedianImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryMedianImageFilter')
class(`BinaryMedianImageFilter_ToString`) = c("SWIGFunction", class('BinaryMedianImageFilter_ToString'))

# Start of BinaryMedianImageFilter_Execute

`BinaryMedianImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryMedianImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMedianImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMedianImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryMedianImageFilter', '_p_itk__simple__Image')
class(`BinaryMedianImageFilter_Execute`) = c("SWIGFunction", class('BinaryMedianImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryMedianImageFilter
setMethod('$', '_p_itk__simple__BinaryMedianImageFilter', function(x, name)

{
  accessorFuns = list('SetRadius' = BinaryMedianImageFilter_SetRadius, 'GetRadius' = BinaryMedianImageFilter_GetRadius, 'SetForegroundValue' = BinaryMedianImageFilter_SetForegroundValue, 'GetForegroundValue' = BinaryMedianImageFilter_GetForegroundValue, 'SetBackgroundValue' = BinaryMedianImageFilter_SetBackgroundValue, 'GetBackgroundValue' = BinaryMedianImageFilter_GetBackgroundValue, 'GetName' = BinaryMedianImageFilter_GetName, 'ToString' = BinaryMedianImageFilter_ToString, 'Execute' = BinaryMedianImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryMedianImageFilter
# Start of BinaryMedian

`BinaryMedian__SWIG_0` = function(image1, radius, foregroundValue, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  
  
  ;ans = .Call('R_swig_BinaryMedian__SWIG_0', image1, radius, foregroundValue, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMedian__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMedian__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'numeric')
class(`BinaryMedian__SWIG_0`) = c("SWIGFunction", class('BinaryMedian__SWIG_0'))

# Start of BinaryMedian

`BinaryMedian__SWIG_1` = function(image1, radius, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  
  ;ans = .Call('R_swig_BinaryMedian__SWIG_1', image1, radius, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMedian__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMedian__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric')
class(`BinaryMedian__SWIG_1`) = c("SWIGFunction", class('BinaryMedian__SWIG_1'))

# Start of BinaryMedian

`BinaryMedian__SWIG_2` = function(image1, radius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  ;ans = .Call('R_swig_BinaryMedian__SWIG_2', image1, radius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMedian__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMedian__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BinaryMedian__SWIG_2`) = c("SWIGFunction", class('BinaryMedian__SWIG_2'))

# Start of BinaryMedian

`BinaryMedian__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryMedian__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMedian__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMedian__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryMedian__SWIG_3`) = c("SWIGFunction", class('BinaryMedian__SWIG_3'))

`BinaryMedian` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryMedian__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BinaryMedian__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryMedian__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- BinaryMedian__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryMedian with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryMinMaxCurvatureFlowImageFilter

`delete_BinaryMinMaxCurvatureFlowImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryMinMaxCurvatureFlowImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryMinMaxCurvatureFlowImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryMinMaxCurvatureFlowImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter')
class(`delete_BinaryMinMaxCurvatureFlowImageFilter`) = c("SWIGFunction", class('delete_BinaryMinMaxCurvatureFlowImageFilter'))

# Start of new_BinaryMinMaxCurvatureFlowImageFilter

`BinaryMinMaxCurvatureFlowImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryMinMaxCurvatureFlowImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryMinMaxCurvatureFlowImageFilter);
  ans
  
}

attr(`BinaryMinMaxCurvatureFlowImageFilter`, 'returnType') = '_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter'
class(`BinaryMinMaxCurvatureFlowImageFilter`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlowImageFilter'))

# Start of BinaryMinMaxCurvatureFlowImageFilter_SetTimeStep

`BinaryMinMaxCurvatureFlowImageFilter_SetTimeStep` = function(self, TimeStep)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryMinMaxCurvatureFlowImageFilter_SetTimeStep', self, TimeStep, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMinMaxCurvatureFlowImageFilter_SetTimeStep`, 'returnType') = 'void'
attr(`BinaryMinMaxCurvatureFlowImageFilter_SetTimeStep`, "inputTypes") = c('_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter', 'numeric')
class(`BinaryMinMaxCurvatureFlowImageFilter_SetTimeStep`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlowImageFilter_SetTimeStep'))

# Start of BinaryMinMaxCurvatureFlowImageFilter_GetTimeStep

`BinaryMinMaxCurvatureFlowImageFilter_GetTimeStep` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMinMaxCurvatureFlowImageFilter_GetTimeStep', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMinMaxCurvatureFlowImageFilter_GetTimeStep`, 'returnType') = 'numeric'
attr(`BinaryMinMaxCurvatureFlowImageFilter_GetTimeStep`, "inputTypes") = c('_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter')
class(`BinaryMinMaxCurvatureFlowImageFilter_GetTimeStep`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlowImageFilter_GetTimeStep'))

# Start of BinaryMinMaxCurvatureFlowImageFilter_SetNumberOfIterations

`BinaryMinMaxCurvatureFlowImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_BinaryMinMaxCurvatureFlowImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMinMaxCurvatureFlowImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`BinaryMinMaxCurvatureFlowImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter', 'integer')
class(`BinaryMinMaxCurvatureFlowImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlowImageFilter_SetNumberOfIterations'))

# Start of BinaryMinMaxCurvatureFlowImageFilter_GetNumberOfIterations

`BinaryMinMaxCurvatureFlowImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMinMaxCurvatureFlowImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMinMaxCurvatureFlowImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`BinaryMinMaxCurvatureFlowImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter')
class(`BinaryMinMaxCurvatureFlowImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlowImageFilter_GetNumberOfIterations'))

# Start of BinaryMinMaxCurvatureFlowImageFilter_SetStencilRadius

`BinaryMinMaxCurvatureFlowImageFilter_SetStencilRadius` = function(self, StencilRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  StencilRadius = as.integer(StencilRadius);
  
  if(length(StencilRadius) > 1) {
    warning("using only the first element of StencilRadius");
  };
  
  ;ans = .Call('R_swig_BinaryMinMaxCurvatureFlowImageFilter_SetStencilRadius', self, StencilRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMinMaxCurvatureFlowImageFilter_SetStencilRadius`, 'returnType') = 'void'
attr(`BinaryMinMaxCurvatureFlowImageFilter_SetStencilRadius`, "inputTypes") = c('_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter', 'integer')
class(`BinaryMinMaxCurvatureFlowImageFilter_SetStencilRadius`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlowImageFilter_SetStencilRadius'))

# Start of BinaryMinMaxCurvatureFlowImageFilter_GetStencilRadius

`BinaryMinMaxCurvatureFlowImageFilter_GetStencilRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMinMaxCurvatureFlowImageFilter_GetStencilRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMinMaxCurvatureFlowImageFilter_GetStencilRadius`, 'returnType') = 'integer'
attr(`BinaryMinMaxCurvatureFlowImageFilter_GetStencilRadius`, "inputTypes") = c('_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter')
class(`BinaryMinMaxCurvatureFlowImageFilter_GetStencilRadius`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlowImageFilter_GetStencilRadius'))

# Start of BinaryMinMaxCurvatureFlowImageFilter_SetThreshold

`BinaryMinMaxCurvatureFlowImageFilter_SetThreshold` = function(self, Threshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryMinMaxCurvatureFlowImageFilter_SetThreshold', self, Threshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMinMaxCurvatureFlowImageFilter_SetThreshold`, 'returnType') = 'void'
attr(`BinaryMinMaxCurvatureFlowImageFilter_SetThreshold`, "inputTypes") = c('_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter', 'numeric')
class(`BinaryMinMaxCurvatureFlowImageFilter_SetThreshold`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlowImageFilter_SetThreshold'))

# Start of BinaryMinMaxCurvatureFlowImageFilter_GetThreshold

`BinaryMinMaxCurvatureFlowImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMinMaxCurvatureFlowImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMinMaxCurvatureFlowImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`BinaryMinMaxCurvatureFlowImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter')
class(`BinaryMinMaxCurvatureFlowImageFilter_GetThreshold`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlowImageFilter_GetThreshold'))

# Start of BinaryMinMaxCurvatureFlowImageFilter_GetName

`BinaryMinMaxCurvatureFlowImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMinMaxCurvatureFlowImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMinMaxCurvatureFlowImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryMinMaxCurvatureFlowImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter')
class(`BinaryMinMaxCurvatureFlowImageFilter_GetName`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlowImageFilter_GetName'))

# Start of BinaryMinMaxCurvatureFlowImageFilter_ToString

`BinaryMinMaxCurvatureFlowImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMinMaxCurvatureFlowImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMinMaxCurvatureFlowImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryMinMaxCurvatureFlowImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter')
class(`BinaryMinMaxCurvatureFlowImageFilter_ToString`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlowImageFilter_ToString'))

# Start of BinaryMinMaxCurvatureFlowImageFilter_Execute

`BinaryMinMaxCurvatureFlowImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryMinMaxCurvatureFlowImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMinMaxCurvatureFlowImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMinMaxCurvatureFlowImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter', '_p_itk__simple__Image')
class(`BinaryMinMaxCurvatureFlowImageFilter_Execute`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlowImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryMinMaxCurvatureFlowImageFilter
setMethod('$', '_p_itk__simple__BinaryMinMaxCurvatureFlowImageFilter', function(x, name)

{
  accessorFuns = list('SetTimeStep' = BinaryMinMaxCurvatureFlowImageFilter_SetTimeStep, 'GetTimeStep' = BinaryMinMaxCurvatureFlowImageFilter_GetTimeStep, 'SetNumberOfIterations' = BinaryMinMaxCurvatureFlowImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = BinaryMinMaxCurvatureFlowImageFilter_GetNumberOfIterations, 'SetStencilRadius' = BinaryMinMaxCurvatureFlowImageFilter_SetStencilRadius, 'GetStencilRadius' = BinaryMinMaxCurvatureFlowImageFilter_GetStencilRadius, 'SetThreshold' = BinaryMinMaxCurvatureFlowImageFilter_SetThreshold, 'GetThreshold' = BinaryMinMaxCurvatureFlowImageFilter_GetThreshold, 'GetName' = BinaryMinMaxCurvatureFlowImageFilter_GetName, 'ToString' = BinaryMinMaxCurvatureFlowImageFilter_ToString, 'Execute' = BinaryMinMaxCurvatureFlowImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryMinMaxCurvatureFlowImageFilter
# Start of BinaryMinMaxCurvatureFlow

`BinaryMinMaxCurvatureFlow__SWIG_0` = function(image1, timeStep, numberOfIterations, stencilRadius, threshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  stencilRadius = as.integer(stencilRadius);
  
  if(length(stencilRadius) > 1) {
    warning("using only the first element of stencilRadius");
  };
  
  
  ;ans = .Call('R_swig_BinaryMinMaxCurvatureFlow__SWIG_0', image1, timeStep, numberOfIterations, stencilRadius, threshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMinMaxCurvatureFlow__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMinMaxCurvatureFlow__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer', 'numeric')
class(`BinaryMinMaxCurvatureFlow__SWIG_0`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlow__SWIG_0'))

# Start of BinaryMinMaxCurvatureFlow

`BinaryMinMaxCurvatureFlow__SWIG_1` = function(image1, timeStep, numberOfIterations, stencilRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  stencilRadius = as.integer(stencilRadius);
  
  if(length(stencilRadius) > 1) {
    warning("using only the first element of stencilRadius");
  };
  
  ;ans = .Call('R_swig_BinaryMinMaxCurvatureFlow__SWIG_1', image1, timeStep, numberOfIterations, stencilRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMinMaxCurvatureFlow__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMinMaxCurvatureFlow__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`BinaryMinMaxCurvatureFlow__SWIG_1`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlow__SWIG_1'))

# Start of BinaryMinMaxCurvatureFlow

`BinaryMinMaxCurvatureFlow__SWIG_2` = function(image1, timeStep, numberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_BinaryMinMaxCurvatureFlow__SWIG_2', image1, timeStep, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMinMaxCurvatureFlow__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMinMaxCurvatureFlow__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`BinaryMinMaxCurvatureFlow__SWIG_2`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlow__SWIG_2'))

# Start of BinaryMinMaxCurvatureFlow

`BinaryMinMaxCurvatureFlow__SWIG_3` = function(image1, timeStep, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_BinaryMinMaxCurvatureFlow__SWIG_3', image1, timeStep, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMinMaxCurvatureFlow__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMinMaxCurvatureFlow__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`BinaryMinMaxCurvatureFlow__SWIG_3`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlow__SWIG_3'))

# Start of BinaryMinMaxCurvatureFlow

`BinaryMinMaxCurvatureFlow__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryMinMaxCurvatureFlow__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMinMaxCurvatureFlow__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMinMaxCurvatureFlow__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryMinMaxCurvatureFlow__SWIG_4`) = c("SWIGFunction", class('BinaryMinMaxCurvatureFlow__SWIG_4'))

`BinaryMinMaxCurvatureFlow` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryMinMaxCurvatureFlow__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- BinaryMinMaxCurvatureFlow__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- BinaryMinMaxCurvatureFlow__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- BinaryMinMaxCurvatureFlow__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- BinaryMinMaxCurvatureFlow__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryMinMaxCurvatureFlow with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryMorphologicalClosingImageFilter

`delete_BinaryMorphologicalClosingImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryMorphologicalClosingImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryMorphologicalClosingImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryMorphologicalClosingImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalClosingImageFilter')
class(`delete_BinaryMorphologicalClosingImageFilter`) = c("SWIGFunction", class('delete_BinaryMorphologicalClosingImageFilter'))

# Start of new_BinaryMorphologicalClosingImageFilter

`BinaryMorphologicalClosingImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryMorphologicalClosingImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryMorphologicalClosingImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryMorphologicalClosingImageFilter);
  ans
  
}

attr(`BinaryMorphologicalClosingImageFilter`, 'returnType') = '_p_itk__simple__BinaryMorphologicalClosingImageFilter'
class(`BinaryMorphologicalClosingImageFilter`) = c("SWIGFunction", class('BinaryMorphologicalClosingImageFilter'))

# Start of BinaryMorphologicalClosingImageFilter_SetKernelRadius

`BinaryMorphologicalClosingImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_BinaryMorphologicalClosingImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMorphologicalClosingImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`BinaryMorphologicalClosingImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalClosingImageFilter', 'integer')
class(`BinaryMorphologicalClosingImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('BinaryMorphologicalClosingImageFilter_SetKernelRadius__SWIG_0'))

# Start of BinaryMorphologicalClosingImageFilter_SetKernelRadius

`BinaryMorphologicalClosingImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_BinaryMorphologicalClosingImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMorphologicalClosingImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`BinaryMorphologicalClosingImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalClosingImageFilter', 'integer')
class(`BinaryMorphologicalClosingImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('BinaryMorphologicalClosingImageFilter_SetKernelRadius__SWIG_1'))

`BinaryMorphologicalClosingImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__BinaryMorphologicalClosingImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BinaryMorphologicalClosingImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__BinaryMorphologicalClosingImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BinaryMorphologicalClosingImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryMorphologicalClosingImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of BinaryMorphologicalClosingImageFilter_GetKernelRadius

`BinaryMorphologicalClosingImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMorphologicalClosingImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMorphologicalClosingImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`BinaryMorphologicalClosingImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalClosingImageFilter')
class(`BinaryMorphologicalClosingImageFilter_GetKernelRadius`) = c("SWIGFunction", class('BinaryMorphologicalClosingImageFilter_GetKernelRadius'))

# Start of BinaryMorphologicalClosingImageFilter_SetKernelType

`BinaryMorphologicalClosingImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_BinaryMorphologicalClosingImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMorphologicalClosingImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`BinaryMorphologicalClosingImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalClosingImageFilter', 'character')
class(`BinaryMorphologicalClosingImageFilter_SetKernelType`) = c("SWIGFunction", class('BinaryMorphologicalClosingImageFilter_SetKernelType'))

# Start of BinaryMorphologicalClosingImageFilter_GetKernelType

`BinaryMorphologicalClosingImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryMorphologicalClosingImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`BinaryMorphologicalClosingImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`BinaryMorphologicalClosingImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalClosingImageFilter')
class(`BinaryMorphologicalClosingImageFilter_GetKernelType`) = c("SWIGFunction", class('BinaryMorphologicalClosingImageFilter_GetKernelType'))

# Start of BinaryMorphologicalClosingImageFilter_SetForegroundValue

`BinaryMorphologicalClosingImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryMorphologicalClosingImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMorphologicalClosingImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`BinaryMorphologicalClosingImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalClosingImageFilter', 'numeric')
class(`BinaryMorphologicalClosingImageFilter_SetForegroundValue`) = c("SWIGFunction", class('BinaryMorphologicalClosingImageFilter_SetForegroundValue'))

# Start of BinaryMorphologicalClosingImageFilter_GetForegroundValue

`BinaryMorphologicalClosingImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMorphologicalClosingImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMorphologicalClosingImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`BinaryMorphologicalClosingImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalClosingImageFilter')
class(`BinaryMorphologicalClosingImageFilter_GetForegroundValue`) = c("SWIGFunction", class('BinaryMorphologicalClosingImageFilter_GetForegroundValue'))

# Start of BinaryMorphologicalClosingImageFilter_SetSafeBorder

`BinaryMorphologicalClosingImageFilter_SetSafeBorder` = function(self, SafeBorder)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SafeBorder = as.logical(SafeBorder);
  ;ans = .Call('R_swig_BinaryMorphologicalClosingImageFilter_SetSafeBorder', self, SafeBorder, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMorphologicalClosingImageFilter_SetSafeBorder`, 'returnType') = 'void'
attr(`BinaryMorphologicalClosingImageFilter_SetSafeBorder`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalClosingImageFilter', 'logical')
class(`BinaryMorphologicalClosingImageFilter_SetSafeBorder`) = c("SWIGFunction", class('BinaryMorphologicalClosingImageFilter_SetSafeBorder'))

# Start of BinaryMorphologicalClosingImageFilter_SafeBorderOn

`BinaryMorphologicalClosingImageFilter_SafeBorderOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryMorphologicalClosingImageFilter_SafeBorderOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMorphologicalClosingImageFilter_SafeBorderOn`, 'returnType') = 'void'
attr(`BinaryMorphologicalClosingImageFilter_SafeBorderOn`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalClosingImageFilter')
class(`BinaryMorphologicalClosingImageFilter_SafeBorderOn`) = c("SWIGFunction", class('BinaryMorphologicalClosingImageFilter_SafeBorderOn'))

# Start of BinaryMorphologicalClosingImageFilter_SafeBorderOff

`BinaryMorphologicalClosingImageFilter_SafeBorderOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryMorphologicalClosingImageFilter_SafeBorderOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMorphologicalClosingImageFilter_SafeBorderOff`, 'returnType') = 'void'
attr(`BinaryMorphologicalClosingImageFilter_SafeBorderOff`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalClosingImageFilter')
class(`BinaryMorphologicalClosingImageFilter_SafeBorderOff`) = c("SWIGFunction", class('BinaryMorphologicalClosingImageFilter_SafeBorderOff'))

# Start of BinaryMorphologicalClosingImageFilter_GetSafeBorder

`BinaryMorphologicalClosingImageFilter_GetSafeBorder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMorphologicalClosingImageFilter_GetSafeBorder', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMorphologicalClosingImageFilter_GetSafeBorder`, 'returnType') = 'logical'
attr(`BinaryMorphologicalClosingImageFilter_GetSafeBorder`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalClosingImageFilter')
class(`BinaryMorphologicalClosingImageFilter_GetSafeBorder`) = c("SWIGFunction", class('BinaryMorphologicalClosingImageFilter_GetSafeBorder'))

# Start of BinaryMorphologicalClosingImageFilter_GetName

`BinaryMorphologicalClosingImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMorphologicalClosingImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMorphologicalClosingImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryMorphologicalClosingImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalClosingImageFilter')
class(`BinaryMorphologicalClosingImageFilter_GetName`) = c("SWIGFunction", class('BinaryMorphologicalClosingImageFilter_GetName'))

# Start of BinaryMorphologicalClosingImageFilter_ToString

`BinaryMorphologicalClosingImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMorphologicalClosingImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMorphologicalClosingImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryMorphologicalClosingImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalClosingImageFilter')
class(`BinaryMorphologicalClosingImageFilter_ToString`) = c("SWIGFunction", class('BinaryMorphologicalClosingImageFilter_ToString'))

# Start of BinaryMorphologicalClosingImageFilter_Execute

`BinaryMorphologicalClosingImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryMorphologicalClosingImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMorphologicalClosingImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMorphologicalClosingImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalClosingImageFilter', '_p_itk__simple__Image')
class(`BinaryMorphologicalClosingImageFilter_Execute`) = c("SWIGFunction", class('BinaryMorphologicalClosingImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryMorphologicalClosingImageFilter
setMethod('$', '_p_itk__simple__BinaryMorphologicalClosingImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = BinaryMorphologicalClosingImageFilter_SetKernelRadius, 'GetKernelRadius' = BinaryMorphologicalClosingImageFilter_GetKernelRadius, 'SetKernelType' = BinaryMorphologicalClosingImageFilter_SetKernelType, 'GetKernelType' = BinaryMorphologicalClosingImageFilter_GetKernelType, 'SetForegroundValue' = BinaryMorphologicalClosingImageFilter_SetForegroundValue, 'GetForegroundValue' = BinaryMorphologicalClosingImageFilter_GetForegroundValue, 'SetSafeBorder' = BinaryMorphologicalClosingImageFilter_SetSafeBorder, 'SafeBorderOn' = BinaryMorphologicalClosingImageFilter_SafeBorderOn, 'SafeBorderOff' = BinaryMorphologicalClosingImageFilter_SafeBorderOff, 'GetSafeBorder' = BinaryMorphologicalClosingImageFilter_GetSafeBorder, 'GetName' = BinaryMorphologicalClosingImageFilter_GetName, 'ToString' = BinaryMorphologicalClosingImageFilter_ToString, 'Execute' = BinaryMorphologicalClosingImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryMorphologicalClosingImageFilter
# Start of BinaryMorphologicalClosing

`BinaryMorphologicalClosing__SWIG_0` = function(image1, kernelRadius, kernelType, foregroundValue, safeBorder, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  safeBorder = as.logical(safeBorder);
  ;ans = .Call('R_swig_BinaryMorphologicalClosing__SWIG_0', image1, kernelRadius, kernelType, foregroundValue, safeBorder, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMorphologicalClosing__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMorphologicalClosing__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric', 'logical')
class(`BinaryMorphologicalClosing__SWIG_0`) = c("SWIGFunction", class('BinaryMorphologicalClosing__SWIG_0'))

# Start of BinaryMorphologicalClosing

`BinaryMorphologicalClosing__SWIG_1` = function(image1, kernelRadius, kernelType, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  ;ans = .Call('R_swig_BinaryMorphologicalClosing__SWIG_1', image1, kernelRadius, kernelType, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMorphologicalClosing__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMorphologicalClosing__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric')
class(`BinaryMorphologicalClosing__SWIG_1`) = c("SWIGFunction", class('BinaryMorphologicalClosing__SWIG_1'))

# Start of BinaryMorphologicalClosing

`BinaryMorphologicalClosing__SWIG_2` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_BinaryMorphologicalClosing__SWIG_2', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMorphologicalClosing__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMorphologicalClosing__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`BinaryMorphologicalClosing__SWIG_2`) = c("SWIGFunction", class('BinaryMorphologicalClosing__SWIG_2'))

# Start of BinaryMorphologicalClosing

`BinaryMorphologicalClosing__SWIG_3` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_BinaryMorphologicalClosing__SWIG_3', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMorphologicalClosing__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMorphologicalClosing__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BinaryMorphologicalClosing__SWIG_3`) = c("SWIGFunction", class('BinaryMorphologicalClosing__SWIG_3'))

# Start of BinaryMorphologicalClosing

`BinaryMorphologicalClosing__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryMorphologicalClosing__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMorphologicalClosing__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMorphologicalClosing__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryMorphologicalClosing__SWIG_4`) = c("SWIGFunction", class('BinaryMorphologicalClosing__SWIG_4'))

`BinaryMorphologicalClosing` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryMorphologicalClosing__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BinaryMorphologicalClosing__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryMorphologicalClosing__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- BinaryMorphologicalClosing__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- BinaryMorphologicalClosing__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryMorphologicalClosing with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryMorphologicalOpeningImageFilter

`delete_BinaryMorphologicalOpeningImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryMorphologicalOpeningImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryMorphologicalOpeningImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryMorphologicalOpeningImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalOpeningImageFilter')
class(`delete_BinaryMorphologicalOpeningImageFilter`) = c("SWIGFunction", class('delete_BinaryMorphologicalOpeningImageFilter'))

# Start of new_BinaryMorphologicalOpeningImageFilter

`BinaryMorphologicalOpeningImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryMorphologicalOpeningImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryMorphologicalOpeningImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryMorphologicalOpeningImageFilter);
  ans
  
}

attr(`BinaryMorphologicalOpeningImageFilter`, 'returnType') = '_p_itk__simple__BinaryMorphologicalOpeningImageFilter'
class(`BinaryMorphologicalOpeningImageFilter`) = c("SWIGFunction", class('BinaryMorphologicalOpeningImageFilter'))

# Start of BinaryMorphologicalOpeningImageFilter_SetKernelRadius

`BinaryMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_BinaryMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`BinaryMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalOpeningImageFilter', 'integer')
class(`BinaryMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('BinaryMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_0'))

# Start of BinaryMorphologicalOpeningImageFilter_SetKernelRadius

`BinaryMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_BinaryMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`BinaryMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalOpeningImageFilter', 'integer')
class(`BinaryMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('BinaryMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_1'))

`BinaryMorphologicalOpeningImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__BinaryMorphologicalOpeningImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BinaryMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__BinaryMorphologicalOpeningImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BinaryMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryMorphologicalOpeningImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of BinaryMorphologicalOpeningImageFilter_GetKernelRadius

`BinaryMorphologicalOpeningImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMorphologicalOpeningImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMorphologicalOpeningImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`BinaryMorphologicalOpeningImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalOpeningImageFilter')
class(`BinaryMorphologicalOpeningImageFilter_GetKernelRadius`) = c("SWIGFunction", class('BinaryMorphologicalOpeningImageFilter_GetKernelRadius'))

# Start of BinaryMorphologicalOpeningImageFilter_SetKernelType

`BinaryMorphologicalOpeningImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_BinaryMorphologicalOpeningImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMorphologicalOpeningImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`BinaryMorphologicalOpeningImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalOpeningImageFilter', 'character')
class(`BinaryMorphologicalOpeningImageFilter_SetKernelType`) = c("SWIGFunction", class('BinaryMorphologicalOpeningImageFilter_SetKernelType'))

# Start of BinaryMorphologicalOpeningImageFilter_GetKernelType

`BinaryMorphologicalOpeningImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryMorphologicalOpeningImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`BinaryMorphologicalOpeningImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`BinaryMorphologicalOpeningImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalOpeningImageFilter')
class(`BinaryMorphologicalOpeningImageFilter_GetKernelType`) = c("SWIGFunction", class('BinaryMorphologicalOpeningImageFilter_GetKernelType'))

# Start of BinaryMorphologicalOpeningImageFilter_SetBackgroundValue

`BinaryMorphologicalOpeningImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryMorphologicalOpeningImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMorphologicalOpeningImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`BinaryMorphologicalOpeningImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalOpeningImageFilter', 'numeric')
class(`BinaryMorphologicalOpeningImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('BinaryMorphologicalOpeningImageFilter_SetBackgroundValue'))

# Start of BinaryMorphologicalOpeningImageFilter_GetBackgroundValue

`BinaryMorphologicalOpeningImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMorphologicalOpeningImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMorphologicalOpeningImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`BinaryMorphologicalOpeningImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalOpeningImageFilter')
class(`BinaryMorphologicalOpeningImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('BinaryMorphologicalOpeningImageFilter_GetBackgroundValue'))

# Start of BinaryMorphologicalOpeningImageFilter_SetForegroundValue

`BinaryMorphologicalOpeningImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryMorphologicalOpeningImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryMorphologicalOpeningImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`BinaryMorphologicalOpeningImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalOpeningImageFilter', 'numeric')
class(`BinaryMorphologicalOpeningImageFilter_SetForegroundValue`) = c("SWIGFunction", class('BinaryMorphologicalOpeningImageFilter_SetForegroundValue'))

# Start of BinaryMorphologicalOpeningImageFilter_GetForegroundValue

`BinaryMorphologicalOpeningImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMorphologicalOpeningImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMorphologicalOpeningImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`BinaryMorphologicalOpeningImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalOpeningImageFilter')
class(`BinaryMorphologicalOpeningImageFilter_GetForegroundValue`) = c("SWIGFunction", class('BinaryMorphologicalOpeningImageFilter_GetForegroundValue'))

# Start of BinaryMorphologicalOpeningImageFilter_GetName

`BinaryMorphologicalOpeningImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMorphologicalOpeningImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMorphologicalOpeningImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryMorphologicalOpeningImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalOpeningImageFilter')
class(`BinaryMorphologicalOpeningImageFilter_GetName`) = c("SWIGFunction", class('BinaryMorphologicalOpeningImageFilter_GetName'))

# Start of BinaryMorphologicalOpeningImageFilter_ToString

`BinaryMorphologicalOpeningImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryMorphologicalOpeningImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryMorphologicalOpeningImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryMorphologicalOpeningImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalOpeningImageFilter')
class(`BinaryMorphologicalOpeningImageFilter_ToString`) = c("SWIGFunction", class('BinaryMorphologicalOpeningImageFilter_ToString'))

# Start of BinaryMorphologicalOpeningImageFilter_Execute

`BinaryMorphologicalOpeningImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryMorphologicalOpeningImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMorphologicalOpeningImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMorphologicalOpeningImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryMorphologicalOpeningImageFilter', '_p_itk__simple__Image')
class(`BinaryMorphologicalOpeningImageFilter_Execute`) = c("SWIGFunction", class('BinaryMorphologicalOpeningImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryMorphologicalOpeningImageFilter
setMethod('$', '_p_itk__simple__BinaryMorphologicalOpeningImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = BinaryMorphologicalOpeningImageFilter_SetKernelRadius, 'GetKernelRadius' = BinaryMorphologicalOpeningImageFilter_GetKernelRadius, 'SetKernelType' = BinaryMorphologicalOpeningImageFilter_SetKernelType, 'GetKernelType' = BinaryMorphologicalOpeningImageFilter_GetKernelType, 'SetBackgroundValue' = BinaryMorphologicalOpeningImageFilter_SetBackgroundValue, 'GetBackgroundValue' = BinaryMorphologicalOpeningImageFilter_GetBackgroundValue, 'SetForegroundValue' = BinaryMorphologicalOpeningImageFilter_SetForegroundValue, 'GetForegroundValue' = BinaryMorphologicalOpeningImageFilter_GetForegroundValue, 'GetName' = BinaryMorphologicalOpeningImageFilter_GetName, 'ToString' = BinaryMorphologicalOpeningImageFilter_ToString, 'Execute' = BinaryMorphologicalOpeningImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryMorphologicalOpeningImageFilter
# Start of BinaryMorphologicalOpening

`BinaryMorphologicalOpening__SWIG_0` = function(image1, kernelRadius, kernelType, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  
  ;ans = .Call('R_swig_BinaryMorphologicalOpening__SWIG_0', image1, kernelRadius, kernelType, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMorphologicalOpening__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMorphologicalOpening__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric', 'numeric')
class(`BinaryMorphologicalOpening__SWIG_0`) = c("SWIGFunction", class('BinaryMorphologicalOpening__SWIG_0'))

# Start of BinaryMorphologicalOpening

`BinaryMorphologicalOpening__SWIG_1` = function(image1, kernelRadius, kernelType, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  ;ans = .Call('R_swig_BinaryMorphologicalOpening__SWIG_1', image1, kernelRadius, kernelType, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMorphologicalOpening__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMorphologicalOpening__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric')
class(`BinaryMorphologicalOpening__SWIG_1`) = c("SWIGFunction", class('BinaryMorphologicalOpening__SWIG_1'))

# Start of BinaryMorphologicalOpening

`BinaryMorphologicalOpening__SWIG_2` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_BinaryMorphologicalOpening__SWIG_2', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMorphologicalOpening__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMorphologicalOpening__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`BinaryMorphologicalOpening__SWIG_2`) = c("SWIGFunction", class('BinaryMorphologicalOpening__SWIG_2'))

# Start of BinaryMorphologicalOpening

`BinaryMorphologicalOpening__SWIG_3` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_BinaryMorphologicalOpening__SWIG_3', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMorphologicalOpening__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMorphologicalOpening__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BinaryMorphologicalOpening__SWIG_3`) = c("SWIGFunction", class('BinaryMorphologicalOpening__SWIG_3'))

# Start of BinaryMorphologicalOpening

`BinaryMorphologicalOpening__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryMorphologicalOpening__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryMorphologicalOpening__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryMorphologicalOpening__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryMorphologicalOpening__SWIG_4`) = c("SWIGFunction", class('BinaryMorphologicalOpening__SWIG_4'))

`BinaryMorphologicalOpening` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryMorphologicalOpening__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BinaryMorphologicalOpening__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryMorphologicalOpening__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- BinaryMorphologicalOpening__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- BinaryMorphologicalOpening__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryMorphologicalOpening with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryNotImageFilter

`delete_BinaryNotImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryNotImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryNotImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryNotImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryNotImageFilter')
class(`delete_BinaryNotImageFilter`) = c("SWIGFunction", class('delete_BinaryNotImageFilter'))

# Start of new_BinaryNotImageFilter

`BinaryNotImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryNotImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryNotImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryNotImageFilter);
  ans
  
}

attr(`BinaryNotImageFilter`, 'returnType') = '_p_itk__simple__BinaryNotImageFilter'
class(`BinaryNotImageFilter`) = c("SWIGFunction", class('BinaryNotImageFilter'))

# Start of BinaryNotImageFilter_SetForegroundValue

`BinaryNotImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryNotImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryNotImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`BinaryNotImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryNotImageFilter', 'numeric')
class(`BinaryNotImageFilter_SetForegroundValue`) = c("SWIGFunction", class('BinaryNotImageFilter_SetForegroundValue'))

# Start of BinaryNotImageFilter_GetForegroundValue

`BinaryNotImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryNotImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryNotImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`BinaryNotImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryNotImageFilter')
class(`BinaryNotImageFilter_GetForegroundValue`) = c("SWIGFunction", class('BinaryNotImageFilter_GetForegroundValue'))

# Start of BinaryNotImageFilter_SetBackgroundValue

`BinaryNotImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryNotImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryNotImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`BinaryNotImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryNotImageFilter', 'numeric')
class(`BinaryNotImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('BinaryNotImageFilter_SetBackgroundValue'))

# Start of BinaryNotImageFilter_GetBackgroundValue

`BinaryNotImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryNotImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryNotImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`BinaryNotImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryNotImageFilter')
class(`BinaryNotImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('BinaryNotImageFilter_GetBackgroundValue'))

# Start of BinaryNotImageFilter_GetName

`BinaryNotImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryNotImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryNotImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryNotImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryNotImageFilter')
class(`BinaryNotImageFilter_GetName`) = c("SWIGFunction", class('BinaryNotImageFilter_GetName'))

# Start of BinaryNotImageFilter_ToString

`BinaryNotImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryNotImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryNotImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryNotImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryNotImageFilter')
class(`BinaryNotImageFilter_ToString`) = c("SWIGFunction", class('BinaryNotImageFilter_ToString'))

# Start of BinaryNotImageFilter_Execute

`BinaryNotImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryNotImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryNotImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryNotImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryNotImageFilter', '_p_itk__simple__Image')
class(`BinaryNotImageFilter_Execute`) = c("SWIGFunction", class('BinaryNotImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryNotImageFilter
setMethod('$', '_p_itk__simple__BinaryNotImageFilter', function(x, name)

{
  accessorFuns = list('SetForegroundValue' = BinaryNotImageFilter_SetForegroundValue, 'GetForegroundValue' = BinaryNotImageFilter_GetForegroundValue, 'SetBackgroundValue' = BinaryNotImageFilter_SetBackgroundValue, 'GetBackgroundValue' = BinaryNotImageFilter_GetBackgroundValue, 'GetName' = BinaryNotImageFilter_GetName, 'ToString' = BinaryNotImageFilter_ToString, 'Execute' = BinaryNotImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryNotImageFilter
# Start of BinaryNot

`BinaryNot__SWIG_0` = function(image1, foregroundValue, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_BinaryNot__SWIG_0', image1, foregroundValue, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryNot__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryNot__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`BinaryNot__SWIG_0`) = c("SWIGFunction", class('BinaryNot__SWIG_0'))

# Start of BinaryNot

`BinaryNot__SWIG_1` = function(image1, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_BinaryNot__SWIG_1', image1, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryNot__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryNot__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`BinaryNot__SWIG_1`) = c("SWIGFunction", class('BinaryNot__SWIG_1'))

# Start of BinaryNot

`BinaryNot__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryNot__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryNot__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryNot__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryNot__SWIG_2`) = c("SWIGFunction", class('BinaryNot__SWIG_2'))

`BinaryNot` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryNot__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- BinaryNot__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryNot__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryNot with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryOpeningByReconstructionImageFilter

`delete_BinaryOpeningByReconstructionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryOpeningByReconstructionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryOpeningByReconstructionImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryOpeningByReconstructionImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter')
class(`delete_BinaryOpeningByReconstructionImageFilter`) = c("SWIGFunction", class('delete_BinaryOpeningByReconstructionImageFilter'))

# Start of new_BinaryOpeningByReconstructionImageFilter

`BinaryOpeningByReconstructionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryOpeningByReconstructionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryOpeningByReconstructionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryOpeningByReconstructionImageFilter);
  ans
  
}

attr(`BinaryOpeningByReconstructionImageFilter`, 'returnType') = '_p_itk__simple__BinaryOpeningByReconstructionImageFilter'
class(`BinaryOpeningByReconstructionImageFilter`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter'))

# Start of BinaryOpeningByReconstructionImageFilter_SetKernelRadius

`BinaryOpeningByReconstructionImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_BinaryOpeningByReconstructionImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryOpeningByReconstructionImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`BinaryOpeningByReconstructionImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter', 'integer')
class(`BinaryOpeningByReconstructionImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_SetKernelRadius__SWIG_0'))

# Start of BinaryOpeningByReconstructionImageFilter_SetKernelRadius

`BinaryOpeningByReconstructionImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_BinaryOpeningByReconstructionImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryOpeningByReconstructionImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`BinaryOpeningByReconstructionImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter', 'integer')
class(`BinaryOpeningByReconstructionImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_SetKernelRadius__SWIG_1'))

`BinaryOpeningByReconstructionImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__BinaryOpeningByReconstructionImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BinaryOpeningByReconstructionImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__BinaryOpeningByReconstructionImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BinaryOpeningByReconstructionImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryOpeningByReconstructionImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of BinaryOpeningByReconstructionImageFilter_GetKernelRadius

`BinaryOpeningByReconstructionImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryOpeningByReconstructionImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryOpeningByReconstructionImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`BinaryOpeningByReconstructionImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter')
class(`BinaryOpeningByReconstructionImageFilter_GetKernelRadius`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_GetKernelRadius'))

# Start of BinaryOpeningByReconstructionImageFilter_SetKernelType

`BinaryOpeningByReconstructionImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_BinaryOpeningByReconstructionImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryOpeningByReconstructionImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`BinaryOpeningByReconstructionImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter', 'character')
class(`BinaryOpeningByReconstructionImageFilter_SetKernelType`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_SetKernelType'))

# Start of BinaryOpeningByReconstructionImageFilter_GetKernelType

`BinaryOpeningByReconstructionImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryOpeningByReconstructionImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`BinaryOpeningByReconstructionImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`BinaryOpeningByReconstructionImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter')
class(`BinaryOpeningByReconstructionImageFilter_GetKernelType`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_GetKernelType'))

# Start of BinaryOpeningByReconstructionImageFilter_SetForegroundValue

`BinaryOpeningByReconstructionImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryOpeningByReconstructionImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryOpeningByReconstructionImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`BinaryOpeningByReconstructionImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter', 'numeric')
class(`BinaryOpeningByReconstructionImageFilter_SetForegroundValue`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_SetForegroundValue'))

# Start of BinaryOpeningByReconstructionImageFilter_GetForegroundValue

`BinaryOpeningByReconstructionImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryOpeningByReconstructionImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryOpeningByReconstructionImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`BinaryOpeningByReconstructionImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter')
class(`BinaryOpeningByReconstructionImageFilter_GetForegroundValue`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_GetForegroundValue'))

# Start of BinaryOpeningByReconstructionImageFilter_SetBackgroundValue

`BinaryOpeningByReconstructionImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryOpeningByReconstructionImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryOpeningByReconstructionImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`BinaryOpeningByReconstructionImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter', 'numeric')
class(`BinaryOpeningByReconstructionImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_SetBackgroundValue'))

# Start of BinaryOpeningByReconstructionImageFilter_GetBackgroundValue

`BinaryOpeningByReconstructionImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryOpeningByReconstructionImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryOpeningByReconstructionImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`BinaryOpeningByReconstructionImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter')
class(`BinaryOpeningByReconstructionImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_GetBackgroundValue'))

# Start of BinaryOpeningByReconstructionImageFilter_SetFullyConnected

`BinaryOpeningByReconstructionImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_BinaryOpeningByReconstructionImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryOpeningByReconstructionImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`BinaryOpeningByReconstructionImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter', 'logical')
class(`BinaryOpeningByReconstructionImageFilter_SetFullyConnected`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_SetFullyConnected'))

# Start of BinaryOpeningByReconstructionImageFilter_FullyConnectedOn

`BinaryOpeningByReconstructionImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryOpeningByReconstructionImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryOpeningByReconstructionImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`BinaryOpeningByReconstructionImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter')
class(`BinaryOpeningByReconstructionImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_FullyConnectedOn'))

# Start of BinaryOpeningByReconstructionImageFilter_FullyConnectedOff

`BinaryOpeningByReconstructionImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryOpeningByReconstructionImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryOpeningByReconstructionImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`BinaryOpeningByReconstructionImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter')
class(`BinaryOpeningByReconstructionImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_FullyConnectedOff'))

# Start of BinaryOpeningByReconstructionImageFilter_GetFullyConnected

`BinaryOpeningByReconstructionImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryOpeningByReconstructionImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryOpeningByReconstructionImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`BinaryOpeningByReconstructionImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter')
class(`BinaryOpeningByReconstructionImageFilter_GetFullyConnected`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_GetFullyConnected'))

# Start of BinaryOpeningByReconstructionImageFilter_GetName

`BinaryOpeningByReconstructionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryOpeningByReconstructionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryOpeningByReconstructionImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryOpeningByReconstructionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter')
class(`BinaryOpeningByReconstructionImageFilter_GetName`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_GetName'))

# Start of BinaryOpeningByReconstructionImageFilter_ToString

`BinaryOpeningByReconstructionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryOpeningByReconstructionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryOpeningByReconstructionImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryOpeningByReconstructionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter')
class(`BinaryOpeningByReconstructionImageFilter_ToString`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_ToString'))

# Start of BinaryOpeningByReconstructionImageFilter_Execute

`BinaryOpeningByReconstructionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryOpeningByReconstructionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryOpeningByReconstructionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryOpeningByReconstructionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryOpeningByReconstructionImageFilter', '_p_itk__simple__Image')
class(`BinaryOpeningByReconstructionImageFilter_Execute`) = c("SWIGFunction", class('BinaryOpeningByReconstructionImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryOpeningByReconstructionImageFilter
setMethod('$', '_p_itk__simple__BinaryOpeningByReconstructionImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = BinaryOpeningByReconstructionImageFilter_SetKernelRadius, 'GetKernelRadius' = BinaryOpeningByReconstructionImageFilter_GetKernelRadius, 'SetKernelType' = BinaryOpeningByReconstructionImageFilter_SetKernelType, 'GetKernelType' = BinaryOpeningByReconstructionImageFilter_GetKernelType, 'SetForegroundValue' = BinaryOpeningByReconstructionImageFilter_SetForegroundValue, 'GetForegroundValue' = BinaryOpeningByReconstructionImageFilter_GetForegroundValue, 'SetBackgroundValue' = BinaryOpeningByReconstructionImageFilter_SetBackgroundValue, 'GetBackgroundValue' = BinaryOpeningByReconstructionImageFilter_GetBackgroundValue, 'SetFullyConnected' = BinaryOpeningByReconstructionImageFilter_SetFullyConnected, 'FullyConnectedOn' = BinaryOpeningByReconstructionImageFilter_FullyConnectedOn, 'FullyConnectedOff' = BinaryOpeningByReconstructionImageFilter_FullyConnectedOff, 'GetFullyConnected' = BinaryOpeningByReconstructionImageFilter_GetFullyConnected, 'GetName' = BinaryOpeningByReconstructionImageFilter_GetName, 'ToString' = BinaryOpeningByReconstructionImageFilter_ToString, 'Execute' = BinaryOpeningByReconstructionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryOpeningByReconstructionImageFilter
# Start of BinaryOpeningByReconstruction

`BinaryOpeningByReconstruction__SWIG_0` = function(image1, kernelRadius, kernelType, foregroundValue, backgroundValue, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_BinaryOpeningByReconstruction__SWIG_0', image1, kernelRadius, kernelType, foregroundValue, backgroundValue, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryOpeningByReconstruction__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryOpeningByReconstruction__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric', 'numeric', 'logical')
class(`BinaryOpeningByReconstruction__SWIG_0`) = c("SWIGFunction", class('BinaryOpeningByReconstruction__SWIG_0'))

# Start of BinaryOpeningByReconstruction

`BinaryOpeningByReconstruction__SWIG_1` = function(image1, kernelRadius, kernelType, foregroundValue, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  
  ;ans = .Call('R_swig_BinaryOpeningByReconstruction__SWIG_1', image1, kernelRadius, kernelType, foregroundValue, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryOpeningByReconstruction__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryOpeningByReconstruction__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric', 'numeric')
class(`BinaryOpeningByReconstruction__SWIG_1`) = c("SWIGFunction", class('BinaryOpeningByReconstruction__SWIG_1'))

# Start of BinaryOpeningByReconstruction

`BinaryOpeningByReconstruction__SWIG_2` = function(image1, kernelRadius, kernelType, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  ;ans = .Call('R_swig_BinaryOpeningByReconstruction__SWIG_2', image1, kernelRadius, kernelType, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryOpeningByReconstruction__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryOpeningByReconstruction__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric')
class(`BinaryOpeningByReconstruction__SWIG_2`) = c("SWIGFunction", class('BinaryOpeningByReconstruction__SWIG_2'))

# Start of BinaryOpeningByReconstruction

`BinaryOpeningByReconstruction__SWIG_3` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_BinaryOpeningByReconstruction__SWIG_3', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryOpeningByReconstruction__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryOpeningByReconstruction__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`BinaryOpeningByReconstruction__SWIG_3`) = c("SWIGFunction", class('BinaryOpeningByReconstruction__SWIG_3'))

# Start of BinaryOpeningByReconstruction

`BinaryOpeningByReconstruction__SWIG_4` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_BinaryOpeningByReconstruction__SWIG_4', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryOpeningByReconstruction__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryOpeningByReconstruction__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BinaryOpeningByReconstruction__SWIG_4`) = c("SWIGFunction", class('BinaryOpeningByReconstruction__SWIG_4'))

# Start of BinaryOpeningByReconstruction

`BinaryOpeningByReconstruction__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryOpeningByReconstruction__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryOpeningByReconstruction__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryOpeningByReconstruction__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryOpeningByReconstruction__SWIG_5`) = c("SWIGFunction", class('BinaryOpeningByReconstruction__SWIG_5'))

`BinaryOpeningByReconstruction` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryOpeningByReconstruction__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BinaryOpeningByReconstruction__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryOpeningByReconstruction__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- BinaryOpeningByReconstruction__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- BinaryOpeningByReconstruction__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- BinaryOpeningByReconstruction__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryOpeningByReconstruction with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryProjectionImageFilter

`delete_BinaryProjectionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryProjectionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryProjectionImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryProjectionImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryProjectionImageFilter')
class(`delete_BinaryProjectionImageFilter`) = c("SWIGFunction", class('delete_BinaryProjectionImageFilter'))

# Start of new_BinaryProjectionImageFilter

`BinaryProjectionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryProjectionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryProjectionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryProjectionImageFilter);
  ans
  
}

attr(`BinaryProjectionImageFilter`, 'returnType') = '_p_itk__simple__BinaryProjectionImageFilter'
class(`BinaryProjectionImageFilter`) = c("SWIGFunction", class('BinaryProjectionImageFilter'))

# Start of BinaryProjectionImageFilter_SetProjectionDimension

`BinaryProjectionImageFilter_SetProjectionDimension` = function(self, ProjectionDimension)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ProjectionDimension = as.integer(ProjectionDimension);
  
  if(length(ProjectionDimension) > 1) {
    warning("using only the first element of ProjectionDimension");
  };
  
  ;ans = .Call('R_swig_BinaryProjectionImageFilter_SetProjectionDimension', self, ProjectionDimension, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryProjectionImageFilter_SetProjectionDimension`, 'returnType') = 'void'
attr(`BinaryProjectionImageFilter_SetProjectionDimension`, "inputTypes") = c('_p_itk__simple__BinaryProjectionImageFilter', 'integer')
class(`BinaryProjectionImageFilter_SetProjectionDimension`) = c("SWIGFunction", class('BinaryProjectionImageFilter_SetProjectionDimension'))

# Start of BinaryProjectionImageFilter_GetProjectionDimension

`BinaryProjectionImageFilter_GetProjectionDimension` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryProjectionImageFilter_GetProjectionDimension', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryProjectionImageFilter_GetProjectionDimension`, 'returnType') = 'integer'
attr(`BinaryProjectionImageFilter_GetProjectionDimension`, "inputTypes") = c('_p_itk__simple__BinaryProjectionImageFilter')
class(`BinaryProjectionImageFilter_GetProjectionDimension`) = c("SWIGFunction", class('BinaryProjectionImageFilter_GetProjectionDimension'))

# Start of BinaryProjectionImageFilter_SetForegroundValue

`BinaryProjectionImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryProjectionImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryProjectionImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`BinaryProjectionImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryProjectionImageFilter', 'numeric')
class(`BinaryProjectionImageFilter_SetForegroundValue`) = c("SWIGFunction", class('BinaryProjectionImageFilter_SetForegroundValue'))

# Start of BinaryProjectionImageFilter_GetForegroundValue

`BinaryProjectionImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryProjectionImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryProjectionImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`BinaryProjectionImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryProjectionImageFilter')
class(`BinaryProjectionImageFilter_GetForegroundValue`) = c("SWIGFunction", class('BinaryProjectionImageFilter_GetForegroundValue'))

# Start of BinaryProjectionImageFilter_SetBackgroundValue

`BinaryProjectionImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryProjectionImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryProjectionImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`BinaryProjectionImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryProjectionImageFilter', 'numeric')
class(`BinaryProjectionImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('BinaryProjectionImageFilter_SetBackgroundValue'))

# Start of BinaryProjectionImageFilter_GetBackgroundValue

`BinaryProjectionImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryProjectionImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryProjectionImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`BinaryProjectionImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryProjectionImageFilter')
class(`BinaryProjectionImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('BinaryProjectionImageFilter_GetBackgroundValue'))

# Start of BinaryProjectionImageFilter_GetName

`BinaryProjectionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryProjectionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryProjectionImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryProjectionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryProjectionImageFilter')
class(`BinaryProjectionImageFilter_GetName`) = c("SWIGFunction", class('BinaryProjectionImageFilter_GetName'))

# Start of BinaryProjectionImageFilter_ToString

`BinaryProjectionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryProjectionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryProjectionImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryProjectionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryProjectionImageFilter')
class(`BinaryProjectionImageFilter_ToString`) = c("SWIGFunction", class('BinaryProjectionImageFilter_ToString'))

# Start of BinaryProjectionImageFilter_Execute

`BinaryProjectionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryProjectionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryProjectionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryProjectionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryProjectionImageFilter', '_p_itk__simple__Image')
class(`BinaryProjectionImageFilter_Execute`) = c("SWIGFunction", class('BinaryProjectionImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryProjectionImageFilter
setMethod('$', '_p_itk__simple__BinaryProjectionImageFilter', function(x, name)

{
  accessorFuns = list('SetProjectionDimension' = BinaryProjectionImageFilter_SetProjectionDimension, 'GetProjectionDimension' = BinaryProjectionImageFilter_GetProjectionDimension, 'SetForegroundValue' = BinaryProjectionImageFilter_SetForegroundValue, 'GetForegroundValue' = BinaryProjectionImageFilter_GetForegroundValue, 'SetBackgroundValue' = BinaryProjectionImageFilter_SetBackgroundValue, 'GetBackgroundValue' = BinaryProjectionImageFilter_GetBackgroundValue, 'GetName' = BinaryProjectionImageFilter_GetName, 'ToString' = BinaryProjectionImageFilter_ToString, 'Execute' = BinaryProjectionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryProjectionImageFilter
# Start of BinaryProjection

`BinaryProjection__SWIG_0` = function(image1, projectionDimension, foregroundValue, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  projectionDimension = as.integer(projectionDimension);
  
  if(length(projectionDimension) > 1) {
    warning("using only the first element of projectionDimension");
  };
  
  
  
  ;ans = .Call('R_swig_BinaryProjection__SWIG_0', image1, projectionDimension, foregroundValue, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryProjection__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryProjection__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'numeric')
class(`BinaryProjection__SWIG_0`) = c("SWIGFunction", class('BinaryProjection__SWIG_0'))

# Start of BinaryProjection

`BinaryProjection__SWIG_1` = function(image1, projectionDimension, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  projectionDimension = as.integer(projectionDimension);
  
  if(length(projectionDimension) > 1) {
    warning("using only the first element of projectionDimension");
  };
  
  
  ;ans = .Call('R_swig_BinaryProjection__SWIG_1', image1, projectionDimension, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryProjection__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryProjection__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric')
class(`BinaryProjection__SWIG_1`) = c("SWIGFunction", class('BinaryProjection__SWIG_1'))

# Start of BinaryProjection

`BinaryProjection__SWIG_2` = function(image1, projectionDimension, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  projectionDimension = as.integer(projectionDimension);
  
  if(length(projectionDimension) > 1) {
    warning("using only the first element of projectionDimension");
  };
  
  ;ans = .Call('R_swig_BinaryProjection__SWIG_2', image1, projectionDimension, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryProjection__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryProjection__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BinaryProjection__SWIG_2`) = c("SWIGFunction", class('BinaryProjection__SWIG_2'))

# Start of BinaryProjection

`BinaryProjection__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryProjection__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryProjection__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryProjection__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryProjection__SWIG_3`) = c("SWIGFunction", class('BinaryProjection__SWIG_3'))

`BinaryProjection` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryProjection__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BinaryProjection__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryProjection__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- BinaryProjection__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryProjection with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryPruningImageFilter

`delete_BinaryPruningImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryPruningImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryPruningImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryPruningImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryPruningImageFilter')
class(`delete_BinaryPruningImageFilter`) = c("SWIGFunction", class('delete_BinaryPruningImageFilter'))

# Start of new_BinaryPruningImageFilter

`BinaryPruningImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryPruningImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryPruningImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryPruningImageFilter);
  ans
  
}

attr(`BinaryPruningImageFilter`, 'returnType') = '_p_itk__simple__BinaryPruningImageFilter'
class(`BinaryPruningImageFilter`) = c("SWIGFunction", class('BinaryPruningImageFilter'))

# Start of BinaryPruningImageFilter_SetIteration

`BinaryPruningImageFilter_SetIteration` = function(self, Iteration)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Iteration = as.integer(Iteration);
  
  if(length(Iteration) > 1) {
    warning("using only the first element of Iteration");
  };
  
  ;ans = .Call('R_swig_BinaryPruningImageFilter_SetIteration', self, Iteration, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryPruningImageFilter_SetIteration`, 'returnType') = 'void'
attr(`BinaryPruningImageFilter_SetIteration`, "inputTypes") = c('_p_itk__simple__BinaryPruningImageFilter', 'integer')
class(`BinaryPruningImageFilter_SetIteration`) = c("SWIGFunction", class('BinaryPruningImageFilter_SetIteration'))

# Start of BinaryPruningImageFilter_GetIteration

`BinaryPruningImageFilter_GetIteration` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryPruningImageFilter_GetIteration', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryPruningImageFilter_GetIteration`, 'returnType') = 'integer'
attr(`BinaryPruningImageFilter_GetIteration`, "inputTypes") = c('_p_itk__simple__BinaryPruningImageFilter')
class(`BinaryPruningImageFilter_GetIteration`) = c("SWIGFunction", class('BinaryPruningImageFilter_GetIteration'))

# Start of BinaryPruningImageFilter_GetName

`BinaryPruningImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryPruningImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryPruningImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryPruningImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryPruningImageFilter')
class(`BinaryPruningImageFilter_GetName`) = c("SWIGFunction", class('BinaryPruningImageFilter_GetName'))

# Start of BinaryPruningImageFilter_ToString

`BinaryPruningImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryPruningImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryPruningImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryPruningImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryPruningImageFilter')
class(`BinaryPruningImageFilter_ToString`) = c("SWIGFunction", class('BinaryPruningImageFilter_ToString'))

# Start of BinaryPruningImageFilter_Execute

`BinaryPruningImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryPruningImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryPruningImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryPruningImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryPruningImageFilter', '_p_itk__simple__Image')
class(`BinaryPruningImageFilter_Execute`) = c("SWIGFunction", class('BinaryPruningImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryPruningImageFilter
setMethod('$', '_p_itk__simple__BinaryPruningImageFilter', function(x, name)

{
  accessorFuns = list('SetIteration' = BinaryPruningImageFilter_SetIteration, 'GetIteration' = BinaryPruningImageFilter_GetIteration, 'GetName' = BinaryPruningImageFilter_GetName, 'ToString' = BinaryPruningImageFilter_ToString, 'Execute' = BinaryPruningImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryPruningImageFilter
# Start of BinaryPruning

`BinaryPruning__SWIG_0` = function(image1, iteration, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  iteration = as.integer(iteration);
  
  if(length(iteration) > 1) {
    warning("using only the first element of iteration");
  };
  
  ;ans = .Call('R_swig_BinaryPruning__SWIG_0', image1, iteration, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryPruning__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryPruning__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BinaryPruning__SWIG_0`) = c("SWIGFunction", class('BinaryPruning__SWIG_0'))

# Start of BinaryPruning

`BinaryPruning__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryPruning__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryPruning__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryPruning__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryPruning__SWIG_1`) = c("SWIGFunction", class('BinaryPruning__SWIG_1'))

`BinaryPruning` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryPruning__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BinaryPruning__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryPruning with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryReconstructionByDilationImageFilter

`delete_BinaryReconstructionByDilationImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryReconstructionByDilationImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryReconstructionByDilationImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryReconstructionByDilationImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByDilationImageFilter')
class(`delete_BinaryReconstructionByDilationImageFilter`) = c("SWIGFunction", class('delete_BinaryReconstructionByDilationImageFilter'))

# Start of new_BinaryReconstructionByDilationImageFilter

`BinaryReconstructionByDilationImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryReconstructionByDilationImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryReconstructionByDilationImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryReconstructionByDilationImageFilter);
  ans
  
}

attr(`BinaryReconstructionByDilationImageFilter`, 'returnType') = '_p_itk__simple__BinaryReconstructionByDilationImageFilter'
class(`BinaryReconstructionByDilationImageFilter`) = c("SWIGFunction", class('BinaryReconstructionByDilationImageFilter'))

# Start of BinaryReconstructionByDilationImageFilter_SetBackgroundValue

`BinaryReconstructionByDilationImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryReconstructionByDilationImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryReconstructionByDilationImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`BinaryReconstructionByDilationImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByDilationImageFilter', 'numeric')
class(`BinaryReconstructionByDilationImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('BinaryReconstructionByDilationImageFilter_SetBackgroundValue'))

# Start of BinaryReconstructionByDilationImageFilter_GetBackgroundValue

`BinaryReconstructionByDilationImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryReconstructionByDilationImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryReconstructionByDilationImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`BinaryReconstructionByDilationImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByDilationImageFilter')
class(`BinaryReconstructionByDilationImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('BinaryReconstructionByDilationImageFilter_GetBackgroundValue'))

# Start of BinaryReconstructionByDilationImageFilter_SetForegroundValue

`BinaryReconstructionByDilationImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryReconstructionByDilationImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryReconstructionByDilationImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`BinaryReconstructionByDilationImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByDilationImageFilter', 'numeric')
class(`BinaryReconstructionByDilationImageFilter_SetForegroundValue`) = c("SWIGFunction", class('BinaryReconstructionByDilationImageFilter_SetForegroundValue'))

# Start of BinaryReconstructionByDilationImageFilter_GetForegroundValue

`BinaryReconstructionByDilationImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryReconstructionByDilationImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryReconstructionByDilationImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`BinaryReconstructionByDilationImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByDilationImageFilter')
class(`BinaryReconstructionByDilationImageFilter_GetForegroundValue`) = c("SWIGFunction", class('BinaryReconstructionByDilationImageFilter_GetForegroundValue'))

# Start of BinaryReconstructionByDilationImageFilter_SetFullyConnected

`BinaryReconstructionByDilationImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_BinaryReconstructionByDilationImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryReconstructionByDilationImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`BinaryReconstructionByDilationImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByDilationImageFilter', 'logical')
class(`BinaryReconstructionByDilationImageFilter_SetFullyConnected`) = c("SWIGFunction", class('BinaryReconstructionByDilationImageFilter_SetFullyConnected'))

# Start of BinaryReconstructionByDilationImageFilter_FullyConnectedOn

`BinaryReconstructionByDilationImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryReconstructionByDilationImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryReconstructionByDilationImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`BinaryReconstructionByDilationImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByDilationImageFilter')
class(`BinaryReconstructionByDilationImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('BinaryReconstructionByDilationImageFilter_FullyConnectedOn'))

# Start of BinaryReconstructionByDilationImageFilter_FullyConnectedOff

`BinaryReconstructionByDilationImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryReconstructionByDilationImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryReconstructionByDilationImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`BinaryReconstructionByDilationImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByDilationImageFilter')
class(`BinaryReconstructionByDilationImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('BinaryReconstructionByDilationImageFilter_FullyConnectedOff'))

# Start of BinaryReconstructionByDilationImageFilter_GetFullyConnected

`BinaryReconstructionByDilationImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryReconstructionByDilationImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryReconstructionByDilationImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`BinaryReconstructionByDilationImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByDilationImageFilter')
class(`BinaryReconstructionByDilationImageFilter_GetFullyConnected`) = c("SWIGFunction", class('BinaryReconstructionByDilationImageFilter_GetFullyConnected'))

# Start of BinaryReconstructionByDilationImageFilter_GetName

`BinaryReconstructionByDilationImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryReconstructionByDilationImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryReconstructionByDilationImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryReconstructionByDilationImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByDilationImageFilter')
class(`BinaryReconstructionByDilationImageFilter_GetName`) = c("SWIGFunction", class('BinaryReconstructionByDilationImageFilter_GetName'))

# Start of BinaryReconstructionByDilationImageFilter_ToString

`BinaryReconstructionByDilationImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryReconstructionByDilationImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryReconstructionByDilationImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryReconstructionByDilationImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByDilationImageFilter')
class(`BinaryReconstructionByDilationImageFilter_ToString`) = c("SWIGFunction", class('BinaryReconstructionByDilationImageFilter_ToString'))

# Start of BinaryReconstructionByDilationImageFilter_Execute

`BinaryReconstructionByDilationImageFilter_Execute` = function(self, markerImage, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_BinaryReconstructionByDilationImageFilter_Execute', self, markerImage, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryReconstructionByDilationImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryReconstructionByDilationImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByDilationImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`BinaryReconstructionByDilationImageFilter_Execute`) = c("SWIGFunction", class('BinaryReconstructionByDilationImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryReconstructionByDilationImageFilter
setMethod('$', '_p_itk__simple__BinaryReconstructionByDilationImageFilter', function(x, name)

{
  accessorFuns = list('SetBackgroundValue' = BinaryReconstructionByDilationImageFilter_SetBackgroundValue, 'GetBackgroundValue' = BinaryReconstructionByDilationImageFilter_GetBackgroundValue, 'SetForegroundValue' = BinaryReconstructionByDilationImageFilter_SetForegroundValue, 'GetForegroundValue' = BinaryReconstructionByDilationImageFilter_GetForegroundValue, 'SetFullyConnected' = BinaryReconstructionByDilationImageFilter_SetFullyConnected, 'FullyConnectedOn' = BinaryReconstructionByDilationImageFilter_FullyConnectedOn, 'FullyConnectedOff' = BinaryReconstructionByDilationImageFilter_FullyConnectedOff, 'GetFullyConnected' = BinaryReconstructionByDilationImageFilter_GetFullyConnected, 'GetName' = BinaryReconstructionByDilationImageFilter_GetName, 'ToString' = BinaryReconstructionByDilationImageFilter_ToString, 'Execute' = BinaryReconstructionByDilationImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryReconstructionByDilationImageFilter
# Start of BinaryReconstructionByDilation

`BinaryReconstructionByDilation__SWIG_0` = function(markerImage, maskImage, backgroundValue, foregroundValue, fullyConnected, .copy = FALSE)
{
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_BinaryReconstructionByDilation__SWIG_0', markerImage, maskImage, backgroundValue, foregroundValue, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryReconstructionByDilation__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryReconstructionByDilation__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'logical')
class(`BinaryReconstructionByDilation__SWIG_0`) = c("SWIGFunction", class('BinaryReconstructionByDilation__SWIG_0'))

# Start of BinaryReconstructionByDilation

`BinaryReconstructionByDilation__SWIG_1` = function(markerImage, maskImage, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  
  ;ans = .Call('R_swig_BinaryReconstructionByDilation__SWIG_1', markerImage, maskImage, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryReconstructionByDilation__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryReconstructionByDilation__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`BinaryReconstructionByDilation__SWIG_1`) = c("SWIGFunction", class('BinaryReconstructionByDilation__SWIG_1'))

# Start of BinaryReconstructionByDilation

`BinaryReconstructionByDilation__SWIG_2` = function(markerImage, maskImage, backgroundValue, .copy = FALSE)
{
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  ;ans = .Call('R_swig_BinaryReconstructionByDilation__SWIG_2', markerImage, maskImage, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryReconstructionByDilation__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryReconstructionByDilation__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`BinaryReconstructionByDilation__SWIG_2`) = c("SWIGFunction", class('BinaryReconstructionByDilation__SWIG_2'))

# Start of BinaryReconstructionByDilation

`BinaryReconstructionByDilation__SWIG_3` = function(markerImage, maskImage, .copy = FALSE)
{
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_BinaryReconstructionByDilation__SWIG_3', markerImage, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryReconstructionByDilation__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryReconstructionByDilation__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`BinaryReconstructionByDilation__SWIG_3`) = c("SWIGFunction", class('BinaryReconstructionByDilation__SWIG_3'))

`BinaryReconstructionByDilation` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- BinaryReconstructionByDilation__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryReconstructionByDilation__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- BinaryReconstructionByDilation__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- BinaryReconstructionByDilation__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryReconstructionByDilation with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryReconstructionByErosionImageFilter

`delete_BinaryReconstructionByErosionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryReconstructionByErosionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryReconstructionByErosionImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryReconstructionByErosionImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByErosionImageFilter')
class(`delete_BinaryReconstructionByErosionImageFilter`) = c("SWIGFunction", class('delete_BinaryReconstructionByErosionImageFilter'))

# Start of new_BinaryReconstructionByErosionImageFilter

`BinaryReconstructionByErosionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryReconstructionByErosionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryReconstructionByErosionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryReconstructionByErosionImageFilter);
  ans
  
}

attr(`BinaryReconstructionByErosionImageFilter`, 'returnType') = '_p_itk__simple__BinaryReconstructionByErosionImageFilter'
class(`BinaryReconstructionByErosionImageFilter`) = c("SWIGFunction", class('BinaryReconstructionByErosionImageFilter'))

# Start of BinaryReconstructionByErosionImageFilter_SetBackgroundValue

`BinaryReconstructionByErosionImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryReconstructionByErosionImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryReconstructionByErosionImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`BinaryReconstructionByErosionImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByErosionImageFilter', 'numeric')
class(`BinaryReconstructionByErosionImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('BinaryReconstructionByErosionImageFilter_SetBackgroundValue'))

# Start of BinaryReconstructionByErosionImageFilter_GetBackgroundValue

`BinaryReconstructionByErosionImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryReconstructionByErosionImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryReconstructionByErosionImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`BinaryReconstructionByErosionImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByErosionImageFilter')
class(`BinaryReconstructionByErosionImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('BinaryReconstructionByErosionImageFilter_GetBackgroundValue'))

# Start of BinaryReconstructionByErosionImageFilter_SetForegroundValue

`BinaryReconstructionByErosionImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryReconstructionByErosionImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryReconstructionByErosionImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`BinaryReconstructionByErosionImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByErosionImageFilter', 'numeric')
class(`BinaryReconstructionByErosionImageFilter_SetForegroundValue`) = c("SWIGFunction", class('BinaryReconstructionByErosionImageFilter_SetForegroundValue'))

# Start of BinaryReconstructionByErosionImageFilter_GetForegroundValue

`BinaryReconstructionByErosionImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryReconstructionByErosionImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryReconstructionByErosionImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`BinaryReconstructionByErosionImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByErosionImageFilter')
class(`BinaryReconstructionByErosionImageFilter_GetForegroundValue`) = c("SWIGFunction", class('BinaryReconstructionByErosionImageFilter_GetForegroundValue'))

# Start of BinaryReconstructionByErosionImageFilter_SetFullyConnected

`BinaryReconstructionByErosionImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_BinaryReconstructionByErosionImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryReconstructionByErosionImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`BinaryReconstructionByErosionImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByErosionImageFilter', 'logical')
class(`BinaryReconstructionByErosionImageFilter_SetFullyConnected`) = c("SWIGFunction", class('BinaryReconstructionByErosionImageFilter_SetFullyConnected'))

# Start of BinaryReconstructionByErosionImageFilter_FullyConnectedOn

`BinaryReconstructionByErosionImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryReconstructionByErosionImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryReconstructionByErosionImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`BinaryReconstructionByErosionImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByErosionImageFilter')
class(`BinaryReconstructionByErosionImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('BinaryReconstructionByErosionImageFilter_FullyConnectedOn'))

# Start of BinaryReconstructionByErosionImageFilter_FullyConnectedOff

`BinaryReconstructionByErosionImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BinaryReconstructionByErosionImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryReconstructionByErosionImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`BinaryReconstructionByErosionImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByErosionImageFilter')
class(`BinaryReconstructionByErosionImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('BinaryReconstructionByErosionImageFilter_FullyConnectedOff'))

# Start of BinaryReconstructionByErosionImageFilter_GetFullyConnected

`BinaryReconstructionByErosionImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryReconstructionByErosionImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryReconstructionByErosionImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`BinaryReconstructionByErosionImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByErosionImageFilter')
class(`BinaryReconstructionByErosionImageFilter_GetFullyConnected`) = c("SWIGFunction", class('BinaryReconstructionByErosionImageFilter_GetFullyConnected'))

# Start of BinaryReconstructionByErosionImageFilter_GetName

`BinaryReconstructionByErosionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryReconstructionByErosionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryReconstructionByErosionImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryReconstructionByErosionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByErosionImageFilter')
class(`BinaryReconstructionByErosionImageFilter_GetName`) = c("SWIGFunction", class('BinaryReconstructionByErosionImageFilter_GetName'))

# Start of BinaryReconstructionByErosionImageFilter_ToString

`BinaryReconstructionByErosionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryReconstructionByErosionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryReconstructionByErosionImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryReconstructionByErosionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByErosionImageFilter')
class(`BinaryReconstructionByErosionImageFilter_ToString`) = c("SWIGFunction", class('BinaryReconstructionByErosionImageFilter_ToString'))

# Start of BinaryReconstructionByErosionImageFilter_Execute

`BinaryReconstructionByErosionImageFilter_Execute` = function(self, markerImage, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_BinaryReconstructionByErosionImageFilter_Execute', self, markerImage, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryReconstructionByErosionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryReconstructionByErosionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryReconstructionByErosionImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`BinaryReconstructionByErosionImageFilter_Execute`) = c("SWIGFunction", class('BinaryReconstructionByErosionImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryReconstructionByErosionImageFilter
setMethod('$', '_p_itk__simple__BinaryReconstructionByErosionImageFilter', function(x, name)

{
  accessorFuns = list('SetBackgroundValue' = BinaryReconstructionByErosionImageFilter_SetBackgroundValue, 'GetBackgroundValue' = BinaryReconstructionByErosionImageFilter_GetBackgroundValue, 'SetForegroundValue' = BinaryReconstructionByErosionImageFilter_SetForegroundValue, 'GetForegroundValue' = BinaryReconstructionByErosionImageFilter_GetForegroundValue, 'SetFullyConnected' = BinaryReconstructionByErosionImageFilter_SetFullyConnected, 'FullyConnectedOn' = BinaryReconstructionByErosionImageFilter_FullyConnectedOn, 'FullyConnectedOff' = BinaryReconstructionByErosionImageFilter_FullyConnectedOff, 'GetFullyConnected' = BinaryReconstructionByErosionImageFilter_GetFullyConnected, 'GetName' = BinaryReconstructionByErosionImageFilter_GetName, 'ToString' = BinaryReconstructionByErosionImageFilter_ToString, 'Execute' = BinaryReconstructionByErosionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryReconstructionByErosionImageFilter
# Start of BinaryReconstructionByErosion

`BinaryReconstructionByErosion__SWIG_0` = function(markerImage, maskImage, backgroundValue, foregroundValue, fullyConnected, .copy = FALSE)
{
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_BinaryReconstructionByErosion__SWIG_0', markerImage, maskImage, backgroundValue, foregroundValue, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryReconstructionByErosion__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryReconstructionByErosion__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'logical')
class(`BinaryReconstructionByErosion__SWIG_0`) = c("SWIGFunction", class('BinaryReconstructionByErosion__SWIG_0'))

# Start of BinaryReconstructionByErosion

`BinaryReconstructionByErosion__SWIG_1` = function(markerImage, maskImage, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  
  ;ans = .Call('R_swig_BinaryReconstructionByErosion__SWIG_1', markerImage, maskImage, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryReconstructionByErosion__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryReconstructionByErosion__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`BinaryReconstructionByErosion__SWIG_1`) = c("SWIGFunction", class('BinaryReconstructionByErosion__SWIG_1'))

# Start of BinaryReconstructionByErosion

`BinaryReconstructionByErosion__SWIG_2` = function(markerImage, maskImage, backgroundValue, .copy = FALSE)
{
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  ;ans = .Call('R_swig_BinaryReconstructionByErosion__SWIG_2', markerImage, maskImage, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryReconstructionByErosion__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryReconstructionByErosion__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`BinaryReconstructionByErosion__SWIG_2`) = c("SWIGFunction", class('BinaryReconstructionByErosion__SWIG_2'))

# Start of BinaryReconstructionByErosion

`BinaryReconstructionByErosion__SWIG_3` = function(markerImage, maskImage, .copy = FALSE)
{
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_BinaryReconstructionByErosion__SWIG_3', markerImage, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryReconstructionByErosion__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryReconstructionByErosion__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`BinaryReconstructionByErosion__SWIG_3`) = c("SWIGFunction", class('BinaryReconstructionByErosion__SWIG_3'))

`BinaryReconstructionByErosion` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- BinaryReconstructionByErosion__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryReconstructionByErosion__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- BinaryReconstructionByErosion__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- BinaryReconstructionByErosion__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryReconstructionByErosion with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryThinningImageFilter

`delete_BinaryThinningImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryThinningImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryThinningImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryThinningImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryThinningImageFilter')
class(`delete_BinaryThinningImageFilter`) = c("SWIGFunction", class('delete_BinaryThinningImageFilter'))

# Start of new_BinaryThinningImageFilter

`BinaryThinningImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryThinningImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryThinningImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryThinningImageFilter);
  ans
  
}

attr(`BinaryThinningImageFilter`, 'returnType') = '_p_itk__simple__BinaryThinningImageFilter'
class(`BinaryThinningImageFilter`) = c("SWIGFunction", class('BinaryThinningImageFilter'))

# Start of BinaryThinningImageFilter_GetName

`BinaryThinningImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryThinningImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryThinningImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryThinningImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryThinningImageFilter')
class(`BinaryThinningImageFilter_GetName`) = c("SWIGFunction", class('BinaryThinningImageFilter_GetName'))

# Start of BinaryThinningImageFilter_ToString

`BinaryThinningImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryThinningImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryThinningImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryThinningImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryThinningImageFilter')
class(`BinaryThinningImageFilter_ToString`) = c("SWIGFunction", class('BinaryThinningImageFilter_ToString'))

# Start of BinaryThinningImageFilter_Execute

`BinaryThinningImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryThinningImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryThinningImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryThinningImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryThinningImageFilter', '_p_itk__simple__Image')
class(`BinaryThinningImageFilter_Execute`) = c("SWIGFunction", class('BinaryThinningImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryThinningImageFilter
setMethod('$', '_p_itk__simple__BinaryThinningImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = BinaryThinningImageFilter_GetName, 'ToString' = BinaryThinningImageFilter_ToString, 'Execute' = BinaryThinningImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryThinningImageFilter
# Start of BinaryThinning

`BinaryThinning` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryThinning', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryThinning`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryThinning`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryThinning`) = c("SWIGFunction", class('BinaryThinning'))

# Start of delete_BinaryThresholdImageFilter

`delete_BinaryThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryThresholdImageFilter')
class(`delete_BinaryThresholdImageFilter`) = c("SWIGFunction", class('delete_BinaryThresholdImageFilter'))

# Start of new_BinaryThresholdImageFilter

`BinaryThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryThresholdImageFilter);
  ans
  
}

attr(`BinaryThresholdImageFilter`, 'returnType') = '_p_itk__simple__BinaryThresholdImageFilter'
class(`BinaryThresholdImageFilter`) = c("SWIGFunction", class('BinaryThresholdImageFilter'))

# Start of BinaryThresholdImageFilter_SetLowerThreshold

`BinaryThresholdImageFilter_SetLowerThreshold` = function(self, LowerThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryThresholdImageFilter_SetLowerThreshold', self, LowerThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryThresholdImageFilter_SetLowerThreshold`, 'returnType') = 'void'
attr(`BinaryThresholdImageFilter_SetLowerThreshold`, "inputTypes") = c('_p_itk__simple__BinaryThresholdImageFilter', 'numeric')
class(`BinaryThresholdImageFilter_SetLowerThreshold`) = c("SWIGFunction", class('BinaryThresholdImageFilter_SetLowerThreshold'))

# Start of BinaryThresholdImageFilter_GetLowerThreshold

`BinaryThresholdImageFilter_GetLowerThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryThresholdImageFilter_GetLowerThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryThresholdImageFilter_GetLowerThreshold`, 'returnType') = 'numeric'
attr(`BinaryThresholdImageFilter_GetLowerThreshold`, "inputTypes") = c('_p_itk__simple__BinaryThresholdImageFilter')
class(`BinaryThresholdImageFilter_GetLowerThreshold`) = c("SWIGFunction", class('BinaryThresholdImageFilter_GetLowerThreshold'))

# Start of BinaryThresholdImageFilter_SetUpperThreshold

`BinaryThresholdImageFilter_SetUpperThreshold` = function(self, UpperThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryThresholdImageFilter_SetUpperThreshold', self, UpperThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryThresholdImageFilter_SetUpperThreshold`, 'returnType') = 'void'
attr(`BinaryThresholdImageFilter_SetUpperThreshold`, "inputTypes") = c('_p_itk__simple__BinaryThresholdImageFilter', 'numeric')
class(`BinaryThresholdImageFilter_SetUpperThreshold`) = c("SWIGFunction", class('BinaryThresholdImageFilter_SetUpperThreshold'))

# Start of BinaryThresholdImageFilter_GetUpperThreshold

`BinaryThresholdImageFilter_GetUpperThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryThresholdImageFilter_GetUpperThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryThresholdImageFilter_GetUpperThreshold`, 'returnType') = 'numeric'
attr(`BinaryThresholdImageFilter_GetUpperThreshold`, "inputTypes") = c('_p_itk__simple__BinaryThresholdImageFilter')
class(`BinaryThresholdImageFilter_GetUpperThreshold`) = c("SWIGFunction", class('BinaryThresholdImageFilter_GetUpperThreshold'))

# Start of BinaryThresholdImageFilter_SetInsideValue

`BinaryThresholdImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideValue = as.integer(InsideValue);
  
  if(length(InsideValue) > 1) {
    warning("using only the first element of InsideValue");
  };
  
  ;ans = .Call('R_swig_BinaryThresholdImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryThresholdImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`BinaryThresholdImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__BinaryThresholdImageFilter', 'integer')
class(`BinaryThresholdImageFilter_SetInsideValue`) = c("SWIGFunction", class('BinaryThresholdImageFilter_SetInsideValue'))

# Start of BinaryThresholdImageFilter_GetInsideValue

`BinaryThresholdImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryThresholdImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryThresholdImageFilter_GetInsideValue`, 'returnType') = 'integer'
attr(`BinaryThresholdImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__BinaryThresholdImageFilter')
class(`BinaryThresholdImageFilter_GetInsideValue`) = c("SWIGFunction", class('BinaryThresholdImageFilter_GetInsideValue'))

# Start of BinaryThresholdImageFilter_SetOutsideValue

`BinaryThresholdImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutsideValue = as.integer(OutsideValue);
  
  if(length(OutsideValue) > 1) {
    warning("using only the first element of OutsideValue");
  };
  
  ;ans = .Call('R_swig_BinaryThresholdImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryThresholdImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`BinaryThresholdImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__BinaryThresholdImageFilter', 'integer')
class(`BinaryThresholdImageFilter_SetOutsideValue`) = c("SWIGFunction", class('BinaryThresholdImageFilter_SetOutsideValue'))

# Start of BinaryThresholdImageFilter_GetOutsideValue

`BinaryThresholdImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryThresholdImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryThresholdImageFilter_GetOutsideValue`, 'returnType') = 'integer'
attr(`BinaryThresholdImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__BinaryThresholdImageFilter')
class(`BinaryThresholdImageFilter_GetOutsideValue`) = c("SWIGFunction", class('BinaryThresholdImageFilter_GetOutsideValue'))

# Start of BinaryThresholdImageFilter_GetName

`BinaryThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryThresholdImageFilter')
class(`BinaryThresholdImageFilter_GetName`) = c("SWIGFunction", class('BinaryThresholdImageFilter_GetName'))

# Start of BinaryThresholdImageFilter_ToString

`BinaryThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryThresholdImageFilter')
class(`BinaryThresholdImageFilter_ToString`) = c("SWIGFunction", class('BinaryThresholdImageFilter_ToString'))

# Start of BinaryThresholdImageFilter_Execute

`BinaryThresholdImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryThresholdImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryThresholdImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryThresholdImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryThresholdImageFilter', '_p_itk__simple__Image')
class(`BinaryThresholdImageFilter_Execute`) = c("SWIGFunction", class('BinaryThresholdImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryThresholdImageFilter
setMethod('$', '_p_itk__simple__BinaryThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetLowerThreshold' = BinaryThresholdImageFilter_SetLowerThreshold, 'GetLowerThreshold' = BinaryThresholdImageFilter_GetLowerThreshold, 'SetUpperThreshold' = BinaryThresholdImageFilter_SetUpperThreshold, 'GetUpperThreshold' = BinaryThresholdImageFilter_GetUpperThreshold, 'SetInsideValue' = BinaryThresholdImageFilter_SetInsideValue, 'GetInsideValue' = BinaryThresholdImageFilter_GetInsideValue, 'SetOutsideValue' = BinaryThresholdImageFilter_SetOutsideValue, 'GetOutsideValue' = BinaryThresholdImageFilter_GetOutsideValue, 'GetName' = BinaryThresholdImageFilter_GetName, 'ToString' = BinaryThresholdImageFilter_ToString, 'Execute' = BinaryThresholdImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryThresholdImageFilter
# Start of BinaryThreshold

`BinaryThreshold__SWIG_0` = function(image1, lowerThreshold, upperThreshold, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_BinaryThreshold__SWIG_0', image1, lowerThreshold, upperThreshold, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryThreshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryThreshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'integer', 'integer')
class(`BinaryThreshold__SWIG_0`) = c("SWIGFunction", class('BinaryThreshold__SWIG_0'))

# Start of BinaryThreshold

`BinaryThreshold__SWIG_1` = function(image1, lowerThreshold, upperThreshold, insideValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_BinaryThreshold__SWIG_1', image1, lowerThreshold, upperThreshold, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryThreshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryThreshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'integer')
class(`BinaryThreshold__SWIG_1`) = c("SWIGFunction", class('BinaryThreshold__SWIG_1'))

# Start of BinaryThreshold

`BinaryThreshold__SWIG_2` = function(image1, lowerThreshold, upperThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_BinaryThreshold__SWIG_2', image1, lowerThreshold, upperThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryThreshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryThreshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`BinaryThreshold__SWIG_2`) = c("SWIGFunction", class('BinaryThreshold__SWIG_2'))

# Start of BinaryThreshold

`BinaryThreshold__SWIG_3` = function(image1, lowerThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_BinaryThreshold__SWIG_3', image1, lowerThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryThreshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryThreshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`BinaryThreshold__SWIG_3`) = c("SWIGFunction", class('BinaryThreshold__SWIG_3'))

# Start of BinaryThreshold

`BinaryThreshold__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryThreshold__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryThreshold__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryThreshold__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryThreshold__SWIG_4`) = c("SWIGFunction", class('BinaryThreshold__SWIG_4'))

`BinaryThreshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryThreshold__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- BinaryThreshold__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryThreshold__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- BinaryThreshold__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- BinaryThreshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryThreshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinaryThresholdProjectionImageFilter

`delete_BinaryThresholdProjectionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinaryThresholdProjectionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinaryThresholdProjectionImageFilter`, 'returnType') = 'void'
attr(`delete_BinaryThresholdProjectionImageFilter`, "inputTypes") = c('_p_itk__simple__BinaryThresholdProjectionImageFilter')
class(`delete_BinaryThresholdProjectionImageFilter`) = c("SWIGFunction", class('delete_BinaryThresholdProjectionImageFilter'))

# Start of new_BinaryThresholdProjectionImageFilter

`BinaryThresholdProjectionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinaryThresholdProjectionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinaryThresholdProjectionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinaryThresholdProjectionImageFilter);
  ans
  
}

attr(`BinaryThresholdProjectionImageFilter`, 'returnType') = '_p_itk__simple__BinaryThresholdProjectionImageFilter'
class(`BinaryThresholdProjectionImageFilter`) = c("SWIGFunction", class('BinaryThresholdProjectionImageFilter'))

# Start of BinaryThresholdProjectionImageFilter_SetProjectionDimension

`BinaryThresholdProjectionImageFilter_SetProjectionDimension` = function(self, ProjectionDimension)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ProjectionDimension = as.integer(ProjectionDimension);
  
  if(length(ProjectionDimension) > 1) {
    warning("using only the first element of ProjectionDimension");
  };
  
  ;ans = .Call('R_swig_BinaryThresholdProjectionImageFilter_SetProjectionDimension', self, ProjectionDimension, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryThresholdProjectionImageFilter_SetProjectionDimension`, 'returnType') = 'void'
attr(`BinaryThresholdProjectionImageFilter_SetProjectionDimension`, "inputTypes") = c('_p_itk__simple__BinaryThresholdProjectionImageFilter', 'integer')
class(`BinaryThresholdProjectionImageFilter_SetProjectionDimension`) = c("SWIGFunction", class('BinaryThresholdProjectionImageFilter_SetProjectionDimension'))

# Start of BinaryThresholdProjectionImageFilter_GetProjectionDimension

`BinaryThresholdProjectionImageFilter_GetProjectionDimension` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryThresholdProjectionImageFilter_GetProjectionDimension', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryThresholdProjectionImageFilter_GetProjectionDimension`, 'returnType') = 'integer'
attr(`BinaryThresholdProjectionImageFilter_GetProjectionDimension`, "inputTypes") = c('_p_itk__simple__BinaryThresholdProjectionImageFilter')
class(`BinaryThresholdProjectionImageFilter_GetProjectionDimension`) = c("SWIGFunction", class('BinaryThresholdProjectionImageFilter_GetProjectionDimension'))

# Start of BinaryThresholdProjectionImageFilter_SetThresholdValue

`BinaryThresholdProjectionImageFilter_SetThresholdValue` = function(self, ThresholdValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_BinaryThresholdProjectionImageFilter_SetThresholdValue', self, ThresholdValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryThresholdProjectionImageFilter_SetThresholdValue`, 'returnType') = 'void'
attr(`BinaryThresholdProjectionImageFilter_SetThresholdValue`, "inputTypes") = c('_p_itk__simple__BinaryThresholdProjectionImageFilter', 'numeric')
class(`BinaryThresholdProjectionImageFilter_SetThresholdValue`) = c("SWIGFunction", class('BinaryThresholdProjectionImageFilter_SetThresholdValue'))

# Start of BinaryThresholdProjectionImageFilter_GetThresholdValue

`BinaryThresholdProjectionImageFilter_GetThresholdValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryThresholdProjectionImageFilter_GetThresholdValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryThresholdProjectionImageFilter_GetThresholdValue`, 'returnType') = 'numeric'
attr(`BinaryThresholdProjectionImageFilter_GetThresholdValue`, "inputTypes") = c('_p_itk__simple__BinaryThresholdProjectionImageFilter')
class(`BinaryThresholdProjectionImageFilter_GetThresholdValue`) = c("SWIGFunction", class('BinaryThresholdProjectionImageFilter_GetThresholdValue'))

# Start of BinaryThresholdProjectionImageFilter_SetForegroundValue

`BinaryThresholdProjectionImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ForegroundValue = as.integer(ForegroundValue);
  
  if(length(ForegroundValue) > 1) {
    warning("using only the first element of ForegroundValue");
  };
  
  ;ans = .Call('R_swig_BinaryThresholdProjectionImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryThresholdProjectionImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`BinaryThresholdProjectionImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryThresholdProjectionImageFilter', 'integer')
class(`BinaryThresholdProjectionImageFilter_SetForegroundValue`) = c("SWIGFunction", class('BinaryThresholdProjectionImageFilter_SetForegroundValue'))

# Start of BinaryThresholdProjectionImageFilter_GetForegroundValue

`BinaryThresholdProjectionImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryThresholdProjectionImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryThresholdProjectionImageFilter_GetForegroundValue`, 'returnType') = 'integer'
attr(`BinaryThresholdProjectionImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__BinaryThresholdProjectionImageFilter')
class(`BinaryThresholdProjectionImageFilter_GetForegroundValue`) = c("SWIGFunction", class('BinaryThresholdProjectionImageFilter_GetForegroundValue'))

# Start of BinaryThresholdProjectionImageFilter_SetBackgroundValue

`BinaryThresholdProjectionImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BackgroundValue = as.integer(BackgroundValue);
  
  if(length(BackgroundValue) > 1) {
    warning("using only the first element of BackgroundValue");
  };
  
  ;ans = .Call('R_swig_BinaryThresholdProjectionImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinaryThresholdProjectionImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`BinaryThresholdProjectionImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryThresholdProjectionImageFilter', 'integer')
class(`BinaryThresholdProjectionImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('BinaryThresholdProjectionImageFilter_SetBackgroundValue'))

# Start of BinaryThresholdProjectionImageFilter_GetBackgroundValue

`BinaryThresholdProjectionImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryThresholdProjectionImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryThresholdProjectionImageFilter_GetBackgroundValue`, 'returnType') = 'integer'
attr(`BinaryThresholdProjectionImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__BinaryThresholdProjectionImageFilter')
class(`BinaryThresholdProjectionImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('BinaryThresholdProjectionImageFilter_GetBackgroundValue'))

# Start of BinaryThresholdProjectionImageFilter_GetName

`BinaryThresholdProjectionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryThresholdProjectionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryThresholdProjectionImageFilter_GetName`, 'returnType') = 'character'
attr(`BinaryThresholdProjectionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinaryThresholdProjectionImageFilter')
class(`BinaryThresholdProjectionImageFilter_GetName`) = c("SWIGFunction", class('BinaryThresholdProjectionImageFilter_GetName'))

# Start of BinaryThresholdProjectionImageFilter_ToString

`BinaryThresholdProjectionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinaryThresholdProjectionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinaryThresholdProjectionImageFilter_ToString`, 'returnType') = 'character'
attr(`BinaryThresholdProjectionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinaryThresholdProjectionImageFilter')
class(`BinaryThresholdProjectionImageFilter_ToString`) = c("SWIGFunction", class('BinaryThresholdProjectionImageFilter_ToString'))

# Start of BinaryThresholdProjectionImageFilter_Execute

`BinaryThresholdProjectionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryThresholdProjectionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryThresholdProjectionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryThresholdProjectionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinaryThresholdProjectionImageFilter', '_p_itk__simple__Image')
class(`BinaryThresholdProjectionImageFilter_Execute`) = c("SWIGFunction", class('BinaryThresholdProjectionImageFilter_Execute'))

# Start of accessor method for itk::simple::BinaryThresholdProjectionImageFilter
setMethod('$', '_p_itk__simple__BinaryThresholdProjectionImageFilter', function(x, name)

{
  accessorFuns = list('SetProjectionDimension' = BinaryThresholdProjectionImageFilter_SetProjectionDimension, 'GetProjectionDimension' = BinaryThresholdProjectionImageFilter_GetProjectionDimension, 'SetThresholdValue' = BinaryThresholdProjectionImageFilter_SetThresholdValue, 'GetThresholdValue' = BinaryThresholdProjectionImageFilter_GetThresholdValue, 'SetForegroundValue' = BinaryThresholdProjectionImageFilter_SetForegroundValue, 'GetForegroundValue' = BinaryThresholdProjectionImageFilter_GetForegroundValue, 'SetBackgroundValue' = BinaryThresholdProjectionImageFilter_SetBackgroundValue, 'GetBackgroundValue' = BinaryThresholdProjectionImageFilter_GetBackgroundValue, 'GetName' = BinaryThresholdProjectionImageFilter_GetName, 'ToString' = BinaryThresholdProjectionImageFilter_ToString, 'Execute' = BinaryThresholdProjectionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinaryThresholdProjectionImageFilter
# Start of BinaryThresholdProjection

`BinaryThresholdProjection__SWIG_0` = function(image1, projectionDimension, thresholdValue, foregroundValue, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  projectionDimension = as.integer(projectionDimension);
  
  if(length(projectionDimension) > 1) {
    warning("using only the first element of projectionDimension");
  };
  
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_BinaryThresholdProjection__SWIG_0', image1, projectionDimension, thresholdValue, foregroundValue, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryThresholdProjection__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryThresholdProjection__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'integer', 'integer')
class(`BinaryThresholdProjection__SWIG_0`) = c("SWIGFunction", class('BinaryThresholdProjection__SWIG_0'))

# Start of BinaryThresholdProjection

`BinaryThresholdProjection__SWIG_1` = function(image1, projectionDimension, thresholdValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  projectionDimension = as.integer(projectionDimension);
  
  if(length(projectionDimension) > 1) {
    warning("using only the first element of projectionDimension");
  };
  
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_BinaryThresholdProjection__SWIG_1', image1, projectionDimension, thresholdValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryThresholdProjection__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryThresholdProjection__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'integer')
class(`BinaryThresholdProjection__SWIG_1`) = c("SWIGFunction", class('BinaryThresholdProjection__SWIG_1'))

# Start of BinaryThresholdProjection

`BinaryThresholdProjection__SWIG_2` = function(image1, projectionDimension, thresholdValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  projectionDimension = as.integer(projectionDimension);
  
  if(length(projectionDimension) > 1) {
    warning("using only the first element of projectionDimension");
  };
  
  
  ;ans = .Call('R_swig_BinaryThresholdProjection__SWIG_2', image1, projectionDimension, thresholdValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryThresholdProjection__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryThresholdProjection__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric')
class(`BinaryThresholdProjection__SWIG_2`) = c("SWIGFunction", class('BinaryThresholdProjection__SWIG_2'))

# Start of BinaryThresholdProjection

`BinaryThresholdProjection__SWIG_3` = function(image1, projectionDimension, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  projectionDimension = as.integer(projectionDimension);
  
  if(length(projectionDimension) > 1) {
    warning("using only the first element of projectionDimension");
  };
  
  ;ans = .Call('R_swig_BinaryThresholdProjection__SWIG_3', image1, projectionDimension, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryThresholdProjection__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryThresholdProjection__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BinaryThresholdProjection__SWIG_3`) = c("SWIGFunction", class('BinaryThresholdProjection__SWIG_3'))

# Start of BinaryThresholdProjection

`BinaryThresholdProjection__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinaryThresholdProjection__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinaryThresholdProjection__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`BinaryThresholdProjection__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinaryThresholdProjection__SWIG_4`) = c("SWIGFunction", class('BinaryThresholdProjection__SWIG_4'))

`BinaryThresholdProjection` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinaryThresholdProjection__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BinaryThresholdProjection__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BinaryThresholdProjection__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- BinaryThresholdProjection__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- BinaryThresholdProjection__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinaryThresholdProjection with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BinomialBlurImageFilter

`delete_BinomialBlurImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BinomialBlurImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BinomialBlurImageFilter`, 'returnType') = 'void'
attr(`delete_BinomialBlurImageFilter`, "inputTypes") = c('_p_itk__simple__BinomialBlurImageFilter')
class(`delete_BinomialBlurImageFilter`) = c("SWIGFunction", class('delete_BinomialBlurImageFilter'))

# Start of new_BinomialBlurImageFilter

`BinomialBlurImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BinomialBlurImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BinomialBlurImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BinomialBlurImageFilter);
  ans
  
}

attr(`BinomialBlurImageFilter`, 'returnType') = '_p_itk__simple__BinomialBlurImageFilter'
class(`BinomialBlurImageFilter`) = c("SWIGFunction", class('BinomialBlurImageFilter'))

# Start of BinomialBlurImageFilter_SetRepetitions

`BinomialBlurImageFilter_SetRepetitions` = function(self, Repetitions)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Repetitions = as.integer(Repetitions);
  
  if(length(Repetitions) > 1) {
    warning("using only the first element of Repetitions");
  };
  
  ;ans = .Call('R_swig_BinomialBlurImageFilter_SetRepetitions', self, Repetitions, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BinomialBlurImageFilter_SetRepetitions`, 'returnType') = 'void'
attr(`BinomialBlurImageFilter_SetRepetitions`, "inputTypes") = c('_p_itk__simple__BinomialBlurImageFilter', 'integer')
class(`BinomialBlurImageFilter_SetRepetitions`) = c("SWIGFunction", class('BinomialBlurImageFilter_SetRepetitions'))

# Start of BinomialBlurImageFilter_GetRepetitions

`BinomialBlurImageFilter_GetRepetitions` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinomialBlurImageFilter_GetRepetitions', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinomialBlurImageFilter_GetRepetitions`, 'returnType') = 'integer'
attr(`BinomialBlurImageFilter_GetRepetitions`, "inputTypes") = c('_p_itk__simple__BinomialBlurImageFilter')
class(`BinomialBlurImageFilter_GetRepetitions`) = c("SWIGFunction", class('BinomialBlurImageFilter_GetRepetitions'))

# Start of BinomialBlurImageFilter_GetName

`BinomialBlurImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinomialBlurImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinomialBlurImageFilter_GetName`, 'returnType') = 'character'
attr(`BinomialBlurImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BinomialBlurImageFilter')
class(`BinomialBlurImageFilter_GetName`) = c("SWIGFunction", class('BinomialBlurImageFilter_GetName'))

# Start of BinomialBlurImageFilter_ToString

`BinomialBlurImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BinomialBlurImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BinomialBlurImageFilter_ToString`, 'returnType') = 'character'
attr(`BinomialBlurImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BinomialBlurImageFilter')
class(`BinomialBlurImageFilter_ToString`) = c("SWIGFunction", class('BinomialBlurImageFilter_ToString'))

# Start of BinomialBlurImageFilter_Execute

`BinomialBlurImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinomialBlurImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinomialBlurImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BinomialBlurImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BinomialBlurImageFilter', '_p_itk__simple__Image')
class(`BinomialBlurImageFilter_Execute`) = c("SWIGFunction", class('BinomialBlurImageFilter_Execute'))

# Start of accessor method for itk::simple::BinomialBlurImageFilter
setMethod('$', '_p_itk__simple__BinomialBlurImageFilter', function(x, name)

{
  accessorFuns = list('SetRepetitions' = BinomialBlurImageFilter_SetRepetitions, 'GetRepetitions' = BinomialBlurImageFilter_GetRepetitions, 'GetName' = BinomialBlurImageFilter_GetName, 'ToString' = BinomialBlurImageFilter_ToString, 'Execute' = BinomialBlurImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BinomialBlurImageFilter
# Start of BinomialBlur

`BinomialBlur__SWIG_0` = function(image1, repetitions, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  repetitions = as.integer(repetitions);
  
  if(length(repetitions) > 1) {
    warning("using only the first element of repetitions");
  };
  
  ;ans = .Call('R_swig_BinomialBlur__SWIG_0', image1, repetitions, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinomialBlur__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BinomialBlur__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BinomialBlur__SWIG_0`) = c("SWIGFunction", class('BinomialBlur__SWIG_0'))

# Start of BinomialBlur

`BinomialBlur__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BinomialBlur__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BinomialBlur__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BinomialBlur__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`BinomialBlur__SWIG_1`) = c("SWIGFunction", class('BinomialBlur__SWIG_1'))

`BinomialBlur` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BinomialBlur__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BinomialBlur__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BinomialBlur with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BitwiseNotImageFilter

`delete_BitwiseNotImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BitwiseNotImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BitwiseNotImageFilter`, 'returnType') = 'void'
attr(`delete_BitwiseNotImageFilter`, "inputTypes") = c('_p_itk__simple__BitwiseNotImageFilter')
class(`delete_BitwiseNotImageFilter`) = c("SWIGFunction", class('delete_BitwiseNotImageFilter'))

# Start of new_BitwiseNotImageFilter

`BitwiseNotImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BitwiseNotImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BitwiseNotImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BitwiseNotImageFilter);
  ans
  
}

attr(`BitwiseNotImageFilter`, 'returnType') = '_p_itk__simple__BitwiseNotImageFilter'
class(`BitwiseNotImageFilter`) = c("SWIGFunction", class('BitwiseNotImageFilter'))

# Start of BitwiseNotImageFilter_GetName

`BitwiseNotImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BitwiseNotImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BitwiseNotImageFilter_GetName`, 'returnType') = 'character'
attr(`BitwiseNotImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BitwiseNotImageFilter')
class(`BitwiseNotImageFilter_GetName`) = c("SWIGFunction", class('BitwiseNotImageFilter_GetName'))

# Start of BitwiseNotImageFilter_ToString

`BitwiseNotImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BitwiseNotImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BitwiseNotImageFilter_ToString`, 'returnType') = 'character'
attr(`BitwiseNotImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BitwiseNotImageFilter')
class(`BitwiseNotImageFilter_ToString`) = c("SWIGFunction", class('BitwiseNotImageFilter_ToString'))

# Start of BitwiseNotImageFilter_Execute

`BitwiseNotImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BitwiseNotImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BitwiseNotImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BitwiseNotImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BitwiseNotImageFilter', '_p_itk__simple__Image')
class(`BitwiseNotImageFilter_Execute`) = c("SWIGFunction", class('BitwiseNotImageFilter_Execute'))

# Start of accessor method for itk::simple::BitwiseNotImageFilter
setMethod('$', '_p_itk__simple__BitwiseNotImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = BitwiseNotImageFilter_GetName, 'ToString' = BitwiseNotImageFilter_ToString, 'Execute' = BitwiseNotImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BitwiseNotImageFilter
# Start of BitwiseNot

`BitwiseNot` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BitwiseNot', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BitwiseNot`, 'returnType') = '_p_itk__simple__Image'
attr(`BitwiseNot`, "inputTypes") = c('_p_itk__simple__Image')
class(`BitwiseNot`) = c("SWIGFunction", class('BitwiseNot'))

# Start of delete_BlackTopHatImageFilter

`delete_BlackTopHatImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BlackTopHatImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BlackTopHatImageFilter`, 'returnType') = 'void'
attr(`delete_BlackTopHatImageFilter`, "inputTypes") = c('_p_itk__simple__BlackTopHatImageFilter')
class(`delete_BlackTopHatImageFilter`) = c("SWIGFunction", class('delete_BlackTopHatImageFilter'))

# Start of new_BlackTopHatImageFilter

`BlackTopHatImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BlackTopHatImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BlackTopHatImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BlackTopHatImageFilter);
  ans
  
}

attr(`BlackTopHatImageFilter`, 'returnType') = '_p_itk__simple__BlackTopHatImageFilter'
class(`BlackTopHatImageFilter`) = c("SWIGFunction", class('BlackTopHatImageFilter'))

# Start of BlackTopHatImageFilter_SetKernelRadius

`BlackTopHatImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_BlackTopHatImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BlackTopHatImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`BlackTopHatImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__BlackTopHatImageFilter', 'integer')
class(`BlackTopHatImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('BlackTopHatImageFilter_SetKernelRadius__SWIG_0'))

# Start of BlackTopHatImageFilter_SetKernelRadius

`BlackTopHatImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_BlackTopHatImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BlackTopHatImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`BlackTopHatImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__BlackTopHatImageFilter', 'integer')
class(`BlackTopHatImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('BlackTopHatImageFilter_SetKernelRadius__SWIG_1'))

`BlackTopHatImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__BlackTopHatImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BlackTopHatImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__BlackTopHatImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BlackTopHatImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BlackTopHatImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of BlackTopHatImageFilter_GetKernelRadius

`BlackTopHatImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BlackTopHatImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BlackTopHatImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`BlackTopHatImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__BlackTopHatImageFilter')
class(`BlackTopHatImageFilter_GetKernelRadius`) = c("SWIGFunction", class('BlackTopHatImageFilter_GetKernelRadius'))

# Start of BlackTopHatImageFilter_SetKernelType

`BlackTopHatImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_BlackTopHatImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BlackTopHatImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`BlackTopHatImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__BlackTopHatImageFilter', 'character')
class(`BlackTopHatImageFilter_SetKernelType`) = c("SWIGFunction", class('BlackTopHatImageFilter_SetKernelType'))

# Start of BlackTopHatImageFilter_GetKernelType

`BlackTopHatImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BlackTopHatImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`BlackTopHatImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`BlackTopHatImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__BlackTopHatImageFilter')
class(`BlackTopHatImageFilter_GetKernelType`) = c("SWIGFunction", class('BlackTopHatImageFilter_GetKernelType'))

# Start of BlackTopHatImageFilter_SetSafeBorder

`BlackTopHatImageFilter_SetSafeBorder` = function(self, SafeBorder)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SafeBorder = as.logical(SafeBorder);
  ;ans = .Call('R_swig_BlackTopHatImageFilter_SetSafeBorder', self, SafeBorder, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BlackTopHatImageFilter_SetSafeBorder`, 'returnType') = 'void'
attr(`BlackTopHatImageFilter_SetSafeBorder`, "inputTypes") = c('_p_itk__simple__BlackTopHatImageFilter', 'logical')
class(`BlackTopHatImageFilter_SetSafeBorder`) = c("SWIGFunction", class('BlackTopHatImageFilter_SetSafeBorder'))

# Start of BlackTopHatImageFilter_SafeBorderOn

`BlackTopHatImageFilter_SafeBorderOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BlackTopHatImageFilter_SafeBorderOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BlackTopHatImageFilter_SafeBorderOn`, 'returnType') = 'void'
attr(`BlackTopHatImageFilter_SafeBorderOn`, "inputTypes") = c('_p_itk__simple__BlackTopHatImageFilter')
class(`BlackTopHatImageFilter_SafeBorderOn`) = c("SWIGFunction", class('BlackTopHatImageFilter_SafeBorderOn'))

# Start of BlackTopHatImageFilter_SafeBorderOff

`BlackTopHatImageFilter_SafeBorderOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_BlackTopHatImageFilter_SafeBorderOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BlackTopHatImageFilter_SafeBorderOff`, 'returnType') = 'void'
attr(`BlackTopHatImageFilter_SafeBorderOff`, "inputTypes") = c('_p_itk__simple__BlackTopHatImageFilter')
class(`BlackTopHatImageFilter_SafeBorderOff`) = c("SWIGFunction", class('BlackTopHatImageFilter_SafeBorderOff'))

# Start of BlackTopHatImageFilter_GetSafeBorder

`BlackTopHatImageFilter_GetSafeBorder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BlackTopHatImageFilter_GetSafeBorder', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BlackTopHatImageFilter_GetSafeBorder`, 'returnType') = 'logical'
attr(`BlackTopHatImageFilter_GetSafeBorder`, "inputTypes") = c('_p_itk__simple__BlackTopHatImageFilter')
class(`BlackTopHatImageFilter_GetSafeBorder`) = c("SWIGFunction", class('BlackTopHatImageFilter_GetSafeBorder'))

# Start of BlackTopHatImageFilter_GetName

`BlackTopHatImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BlackTopHatImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BlackTopHatImageFilter_GetName`, 'returnType') = 'character'
attr(`BlackTopHatImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BlackTopHatImageFilter')
class(`BlackTopHatImageFilter_GetName`) = c("SWIGFunction", class('BlackTopHatImageFilter_GetName'))

# Start of BlackTopHatImageFilter_ToString

`BlackTopHatImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BlackTopHatImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BlackTopHatImageFilter_ToString`, 'returnType') = 'character'
attr(`BlackTopHatImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BlackTopHatImageFilter')
class(`BlackTopHatImageFilter_ToString`) = c("SWIGFunction", class('BlackTopHatImageFilter_ToString'))

# Start of BlackTopHatImageFilter_Execute

`BlackTopHatImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BlackTopHatImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BlackTopHatImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BlackTopHatImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BlackTopHatImageFilter', '_p_itk__simple__Image')
class(`BlackTopHatImageFilter_Execute`) = c("SWIGFunction", class('BlackTopHatImageFilter_Execute'))

# Start of accessor method for itk::simple::BlackTopHatImageFilter
setMethod('$', '_p_itk__simple__BlackTopHatImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = BlackTopHatImageFilter_SetKernelRadius, 'GetKernelRadius' = BlackTopHatImageFilter_GetKernelRadius, 'SetKernelType' = BlackTopHatImageFilter_SetKernelType, 'GetKernelType' = BlackTopHatImageFilter_GetKernelType, 'SetSafeBorder' = BlackTopHatImageFilter_SetSafeBorder, 'SafeBorderOn' = BlackTopHatImageFilter_SafeBorderOn, 'SafeBorderOff' = BlackTopHatImageFilter_SafeBorderOff, 'GetSafeBorder' = BlackTopHatImageFilter_GetSafeBorder, 'GetName' = BlackTopHatImageFilter_GetName, 'ToString' = BlackTopHatImageFilter_ToString, 'Execute' = BlackTopHatImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BlackTopHatImageFilter
# Start of BlackTopHat

`BlackTopHat__SWIG_0` = function(image1, kernelRadius, kernelType, safeBorder, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  safeBorder = as.logical(safeBorder);
  ;ans = .Call('R_swig_BlackTopHat__SWIG_0', image1, kernelRadius, kernelType, safeBorder, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BlackTopHat__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BlackTopHat__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'logical')
class(`BlackTopHat__SWIG_0`) = c("SWIGFunction", class('BlackTopHat__SWIG_0'))

# Start of BlackTopHat

`BlackTopHat__SWIG_1` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_BlackTopHat__SWIG_1', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BlackTopHat__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BlackTopHat__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`BlackTopHat__SWIG_1`) = c("SWIGFunction", class('BlackTopHat__SWIG_1'))

# Start of BlackTopHat

`BlackTopHat__SWIG_2` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_BlackTopHat__SWIG_2', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BlackTopHat__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`BlackTopHat__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BlackTopHat__SWIG_2`) = c("SWIGFunction", class('BlackTopHat__SWIG_2'))

# Start of BlackTopHat

`BlackTopHat__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BlackTopHat__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BlackTopHat__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`BlackTopHat__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`BlackTopHat__SWIG_3`) = c("SWIGFunction", class('BlackTopHat__SWIG_3'))

`BlackTopHat` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BlackTopHat__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BlackTopHat__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- BlackTopHat__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- BlackTopHat__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BlackTopHat with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BoundedReciprocalImageFilter

`delete_BoundedReciprocalImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BoundedReciprocalImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BoundedReciprocalImageFilter`, 'returnType') = 'void'
attr(`delete_BoundedReciprocalImageFilter`, "inputTypes") = c('_p_itk__simple__BoundedReciprocalImageFilter')
class(`delete_BoundedReciprocalImageFilter`) = c("SWIGFunction", class('delete_BoundedReciprocalImageFilter'))

# Start of new_BoundedReciprocalImageFilter

`BoundedReciprocalImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BoundedReciprocalImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BoundedReciprocalImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BoundedReciprocalImageFilter);
  ans
  
}

attr(`BoundedReciprocalImageFilter`, 'returnType') = '_p_itk__simple__BoundedReciprocalImageFilter'
class(`BoundedReciprocalImageFilter`) = c("SWIGFunction", class('BoundedReciprocalImageFilter'))

# Start of BoundedReciprocalImageFilter_GetName

`BoundedReciprocalImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BoundedReciprocalImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BoundedReciprocalImageFilter_GetName`, 'returnType') = 'character'
attr(`BoundedReciprocalImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BoundedReciprocalImageFilter')
class(`BoundedReciprocalImageFilter_GetName`) = c("SWIGFunction", class('BoundedReciprocalImageFilter_GetName'))

# Start of BoundedReciprocalImageFilter_ToString

`BoundedReciprocalImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BoundedReciprocalImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BoundedReciprocalImageFilter_ToString`, 'returnType') = 'character'
attr(`BoundedReciprocalImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BoundedReciprocalImageFilter')
class(`BoundedReciprocalImageFilter_ToString`) = c("SWIGFunction", class('BoundedReciprocalImageFilter_ToString'))

# Start of BoundedReciprocalImageFilter_Execute

`BoundedReciprocalImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BoundedReciprocalImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BoundedReciprocalImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BoundedReciprocalImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BoundedReciprocalImageFilter', '_p_itk__simple__Image')
class(`BoundedReciprocalImageFilter_Execute`) = c("SWIGFunction", class('BoundedReciprocalImageFilter_Execute'))

# Start of accessor method for itk::simple::BoundedReciprocalImageFilter
setMethod('$', '_p_itk__simple__BoundedReciprocalImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = BoundedReciprocalImageFilter_GetName, 'ToString' = BoundedReciprocalImageFilter_ToString, 'Execute' = BoundedReciprocalImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BoundedReciprocalImageFilter
# Start of BoundedReciprocal

`BoundedReciprocal` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BoundedReciprocal', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BoundedReciprocal`, 'returnType') = '_p_itk__simple__Image'
attr(`BoundedReciprocal`, "inputTypes") = c('_p_itk__simple__Image')
class(`BoundedReciprocal`) = c("SWIGFunction", class('BoundedReciprocal'))

# Start of delete_BoxMeanImageFilter

`delete_BoxMeanImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BoxMeanImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BoxMeanImageFilter`, 'returnType') = 'void'
attr(`delete_BoxMeanImageFilter`, "inputTypes") = c('_p_itk__simple__BoxMeanImageFilter')
class(`delete_BoxMeanImageFilter`) = c("SWIGFunction", class('delete_BoxMeanImageFilter'))

# Start of new_BoxMeanImageFilter

`BoxMeanImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BoxMeanImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BoxMeanImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BoxMeanImageFilter);
  ans
  
}

attr(`BoxMeanImageFilter`, 'returnType') = '_p_itk__simple__BoxMeanImageFilter'
class(`BoxMeanImageFilter`) = c("SWIGFunction", class('BoxMeanImageFilter'))

# Start of BoxMeanImageFilter_SetRadius

`BoxMeanImageFilter_SetRadius__SWIG_0` = function(self, Radius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Radius = as.integer(Radius);
  ;ans = .Call('R_swig_BoxMeanImageFilter_SetRadius__SWIG_0', self, Radius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BoxMeanImageFilter_SetRadius__SWIG_0`, 'returnType') = 'void'
attr(`BoxMeanImageFilter_SetRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__BoxMeanImageFilter', 'integer')
class(`BoxMeanImageFilter_SetRadius__SWIG_0`) = c("SWIGFunction", class('BoxMeanImageFilter_SetRadius__SWIG_0'))

# Start of BoxMeanImageFilter_SetRadius

`BoxMeanImageFilter_SetRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_BoxMeanImageFilter_SetRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BoxMeanImageFilter_SetRadius__SWIG_1`, 'returnType') = 'void'
attr(`BoxMeanImageFilter_SetRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__BoxMeanImageFilter', 'integer')
class(`BoxMeanImageFilter_SetRadius__SWIG_1`) = c("SWIGFunction", class('BoxMeanImageFilter_SetRadius__SWIG_1'))

`BoxMeanImageFilter_SetRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__BoxMeanImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BoxMeanImageFilter_SetRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__BoxMeanImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BoxMeanImageFilter_SetRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BoxMeanImageFilter_SetRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of BoxMeanImageFilter_GetRadius

`BoxMeanImageFilter_GetRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BoxMeanImageFilter_GetRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BoxMeanImageFilter_GetRadius`, 'returnType') = 'integer'
attr(`BoxMeanImageFilter_GetRadius`, "inputTypes") = c('_p_itk__simple__BoxMeanImageFilter')
class(`BoxMeanImageFilter_GetRadius`) = c("SWIGFunction", class('BoxMeanImageFilter_GetRadius'))

# Start of BoxMeanImageFilter_GetName

`BoxMeanImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BoxMeanImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BoxMeanImageFilter_GetName`, 'returnType') = 'character'
attr(`BoxMeanImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BoxMeanImageFilter')
class(`BoxMeanImageFilter_GetName`) = c("SWIGFunction", class('BoxMeanImageFilter_GetName'))

# Start of BoxMeanImageFilter_ToString

`BoxMeanImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BoxMeanImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BoxMeanImageFilter_ToString`, 'returnType') = 'character'
attr(`BoxMeanImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BoxMeanImageFilter')
class(`BoxMeanImageFilter_ToString`) = c("SWIGFunction", class('BoxMeanImageFilter_ToString'))

# Start of BoxMeanImageFilter_Execute

`BoxMeanImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BoxMeanImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BoxMeanImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BoxMeanImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BoxMeanImageFilter', '_p_itk__simple__Image')
class(`BoxMeanImageFilter_Execute`) = c("SWIGFunction", class('BoxMeanImageFilter_Execute'))

# Start of accessor method for itk::simple::BoxMeanImageFilter
setMethod('$', '_p_itk__simple__BoxMeanImageFilter', function(x, name)

{
  accessorFuns = list('SetRadius' = BoxMeanImageFilter_SetRadius, 'GetRadius' = BoxMeanImageFilter_GetRadius, 'GetName' = BoxMeanImageFilter_GetName, 'ToString' = BoxMeanImageFilter_ToString, 'Execute' = BoxMeanImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BoxMeanImageFilter
# Start of BoxMean

`BoxMean__SWIG_0` = function(image1, radius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  ;ans = .Call('R_swig_BoxMean__SWIG_0', image1, radius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BoxMean__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BoxMean__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BoxMean__SWIG_0`) = c("SWIGFunction", class('BoxMean__SWIG_0'))

# Start of BoxMean

`BoxMean__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BoxMean__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BoxMean__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BoxMean__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`BoxMean__SWIG_1`) = c("SWIGFunction", class('BoxMean__SWIG_1'))

`BoxMean` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BoxMean__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BoxMean__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BoxMean with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_BoxSigmaImageFilter

`delete_BoxSigmaImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_BoxSigmaImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_BoxSigmaImageFilter`, 'returnType') = 'void'
attr(`delete_BoxSigmaImageFilter`, "inputTypes") = c('_p_itk__simple__BoxSigmaImageFilter')
class(`delete_BoxSigmaImageFilter`) = c("SWIGFunction", class('delete_BoxSigmaImageFilter'))

# Start of new_BoxSigmaImageFilter

`BoxSigmaImageFilter` = function()
{
  ;ans = .Call('R_swig_new_BoxSigmaImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__BoxSigmaImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_BoxSigmaImageFilter);
  ans
  
}

attr(`BoxSigmaImageFilter`, 'returnType') = '_p_itk__simple__BoxSigmaImageFilter'
class(`BoxSigmaImageFilter`) = c("SWIGFunction", class('BoxSigmaImageFilter'))

# Start of BoxSigmaImageFilter_SetRadius

`BoxSigmaImageFilter_SetRadius__SWIG_0` = function(self, Radius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Radius = as.integer(Radius);
  ;ans = .Call('R_swig_BoxSigmaImageFilter_SetRadius__SWIG_0', self, Radius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BoxSigmaImageFilter_SetRadius__SWIG_0`, 'returnType') = 'void'
attr(`BoxSigmaImageFilter_SetRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__BoxSigmaImageFilter', 'integer')
class(`BoxSigmaImageFilter_SetRadius__SWIG_0`) = c("SWIGFunction", class('BoxSigmaImageFilter_SetRadius__SWIG_0'))

# Start of BoxSigmaImageFilter_SetRadius

`BoxSigmaImageFilter_SetRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_BoxSigmaImageFilter_SetRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`BoxSigmaImageFilter_SetRadius__SWIG_1`, 'returnType') = 'void'
attr(`BoxSigmaImageFilter_SetRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__BoxSigmaImageFilter', 'integer')
class(`BoxSigmaImageFilter_SetRadius__SWIG_1`) = c("SWIGFunction", class('BoxSigmaImageFilter_SetRadius__SWIG_1'))

`BoxSigmaImageFilter_SetRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__BoxSigmaImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- BoxSigmaImageFilter_SetRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__BoxSigmaImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BoxSigmaImageFilter_SetRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BoxSigmaImageFilter_SetRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of BoxSigmaImageFilter_GetRadius

`BoxSigmaImageFilter_GetRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BoxSigmaImageFilter_GetRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BoxSigmaImageFilter_GetRadius`, 'returnType') = 'integer'
attr(`BoxSigmaImageFilter_GetRadius`, "inputTypes") = c('_p_itk__simple__BoxSigmaImageFilter')
class(`BoxSigmaImageFilter_GetRadius`) = c("SWIGFunction", class('BoxSigmaImageFilter_GetRadius'))

# Start of BoxSigmaImageFilter_GetName

`BoxSigmaImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BoxSigmaImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BoxSigmaImageFilter_GetName`, 'returnType') = 'character'
attr(`BoxSigmaImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__BoxSigmaImageFilter')
class(`BoxSigmaImageFilter_GetName`) = c("SWIGFunction", class('BoxSigmaImageFilter_GetName'))

# Start of BoxSigmaImageFilter_ToString

`BoxSigmaImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_BoxSigmaImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`BoxSigmaImageFilter_ToString`, 'returnType') = 'character'
attr(`BoxSigmaImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__BoxSigmaImageFilter')
class(`BoxSigmaImageFilter_ToString`) = c("SWIGFunction", class('BoxSigmaImageFilter_ToString'))

# Start of BoxSigmaImageFilter_Execute

`BoxSigmaImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BoxSigmaImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BoxSigmaImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`BoxSigmaImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__BoxSigmaImageFilter', '_p_itk__simple__Image')
class(`BoxSigmaImageFilter_Execute`) = c("SWIGFunction", class('BoxSigmaImageFilter_Execute'))

# Start of accessor method for itk::simple::BoxSigmaImageFilter
setMethod('$', '_p_itk__simple__BoxSigmaImageFilter', function(x, name)

{
  accessorFuns = list('SetRadius' = BoxSigmaImageFilter_SetRadius, 'GetRadius' = BoxSigmaImageFilter_GetRadius, 'GetName' = BoxSigmaImageFilter_GetName, 'ToString' = BoxSigmaImageFilter_ToString, 'Execute' = BoxSigmaImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::BoxSigmaImageFilter
# Start of BoxSigma

`BoxSigma__SWIG_0` = function(image1, radius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  ;ans = .Call('R_swig_BoxSigma__SWIG_0', image1, radius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BoxSigma__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`BoxSigma__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`BoxSigma__SWIG_0`) = c("SWIGFunction", class('BoxSigma__SWIG_0'))

# Start of BoxSigma

`BoxSigma__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_BoxSigma__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`BoxSigma__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`BoxSigma__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`BoxSigma__SWIG_1`) = c("SWIGFunction", class('BoxSigma__SWIG_1'))

`BoxSigma` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- BoxSigma__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- BoxSigma__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for BoxSigma with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_CannyEdgeDetectionImageFilter

`delete_CannyEdgeDetectionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_CannyEdgeDetectionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_CannyEdgeDetectionImageFilter`, 'returnType') = 'void'
attr(`delete_CannyEdgeDetectionImageFilter`, "inputTypes") = c('_p_itk__simple__CannyEdgeDetectionImageFilter')
class(`delete_CannyEdgeDetectionImageFilter`) = c("SWIGFunction", class('delete_CannyEdgeDetectionImageFilter'))

# Start of new_CannyEdgeDetectionImageFilter

`CannyEdgeDetectionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_CannyEdgeDetectionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CannyEdgeDetectionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CannyEdgeDetectionImageFilter);
  ans
  
}

attr(`CannyEdgeDetectionImageFilter`, 'returnType') = '_p_itk__simple__CannyEdgeDetectionImageFilter'
class(`CannyEdgeDetectionImageFilter`) = c("SWIGFunction", class('CannyEdgeDetectionImageFilter'))

# Start of CannyEdgeDetectionImageFilter_SetLowerThreshold

`CannyEdgeDetectionImageFilter_SetLowerThreshold` = function(self, LowerThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_CannyEdgeDetectionImageFilter_SetLowerThreshold', self, LowerThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannyEdgeDetectionImageFilter_SetLowerThreshold`, 'returnType') = 'void'
attr(`CannyEdgeDetectionImageFilter_SetLowerThreshold`, "inputTypes") = c('_p_itk__simple__CannyEdgeDetectionImageFilter', 'numeric')
class(`CannyEdgeDetectionImageFilter_SetLowerThreshold`) = c("SWIGFunction", class('CannyEdgeDetectionImageFilter_SetLowerThreshold'))

# Start of CannyEdgeDetectionImageFilter_GetLowerThreshold

`CannyEdgeDetectionImageFilter_GetLowerThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannyEdgeDetectionImageFilter_GetLowerThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannyEdgeDetectionImageFilter_GetLowerThreshold`, 'returnType') = 'numeric'
attr(`CannyEdgeDetectionImageFilter_GetLowerThreshold`, "inputTypes") = c('_p_itk__simple__CannyEdgeDetectionImageFilter')
class(`CannyEdgeDetectionImageFilter_GetLowerThreshold`) = c("SWIGFunction", class('CannyEdgeDetectionImageFilter_GetLowerThreshold'))

# Start of CannyEdgeDetectionImageFilter_SetUpperThreshold

`CannyEdgeDetectionImageFilter_SetUpperThreshold` = function(self, UpperThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_CannyEdgeDetectionImageFilter_SetUpperThreshold', self, UpperThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannyEdgeDetectionImageFilter_SetUpperThreshold`, 'returnType') = 'void'
attr(`CannyEdgeDetectionImageFilter_SetUpperThreshold`, "inputTypes") = c('_p_itk__simple__CannyEdgeDetectionImageFilter', 'numeric')
class(`CannyEdgeDetectionImageFilter_SetUpperThreshold`) = c("SWIGFunction", class('CannyEdgeDetectionImageFilter_SetUpperThreshold'))

# Start of CannyEdgeDetectionImageFilter_GetUpperThreshold

`CannyEdgeDetectionImageFilter_GetUpperThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannyEdgeDetectionImageFilter_GetUpperThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannyEdgeDetectionImageFilter_GetUpperThreshold`, 'returnType') = 'numeric'
attr(`CannyEdgeDetectionImageFilter_GetUpperThreshold`, "inputTypes") = c('_p_itk__simple__CannyEdgeDetectionImageFilter')
class(`CannyEdgeDetectionImageFilter_GetUpperThreshold`) = c("SWIGFunction", class('CannyEdgeDetectionImageFilter_GetUpperThreshold'))

# Start of CannyEdgeDetectionImageFilter_SetVariance

`CannyEdgeDetectionImageFilter_SetVariance__SWIG_0` = function(self, Variance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Variance = as.numeric(Variance);
  ;ans = .Call('R_swig_CannyEdgeDetectionImageFilter_SetVariance__SWIG_0', self, Variance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannyEdgeDetectionImageFilter_SetVariance__SWIG_0`, 'returnType') = 'void'
attr(`CannyEdgeDetectionImageFilter_SetVariance__SWIG_0`, "inputTypes") = c('_p_itk__simple__CannyEdgeDetectionImageFilter', 'numeric')
class(`CannyEdgeDetectionImageFilter_SetVariance__SWIG_0`) = c("SWIGFunction", class('CannyEdgeDetectionImageFilter_SetVariance__SWIG_0'))

# Start of CannyEdgeDetectionImageFilter_SetVariance

`CannyEdgeDetectionImageFilter_SetVariance__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_CannyEdgeDetectionImageFilter_SetVariance__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannyEdgeDetectionImageFilter_SetVariance__SWIG_1`, 'returnType') = 'void'
attr(`CannyEdgeDetectionImageFilter_SetVariance__SWIG_1`, "inputTypes") = c('_p_itk__simple__CannyEdgeDetectionImageFilter', 'numeric')
class(`CannyEdgeDetectionImageFilter_SetVariance__SWIG_1`) = c("SWIGFunction", class('CannyEdgeDetectionImageFilter_SetVariance__SWIG_1'))

`CannyEdgeDetectionImageFilter_SetVariance` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__CannyEdgeDetectionImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- CannyEdgeDetectionImageFilter_SetVariance__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__CannyEdgeDetectionImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- CannyEdgeDetectionImageFilter_SetVariance__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for CannyEdgeDetectionImageFilter_SetVariance with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of CannyEdgeDetectionImageFilter_GetVariance

`CannyEdgeDetectionImageFilter_GetVariance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannyEdgeDetectionImageFilter_GetVariance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannyEdgeDetectionImageFilter_GetVariance`, 'returnType') = 'numeric'
attr(`CannyEdgeDetectionImageFilter_GetVariance`, "inputTypes") = c('_p_itk__simple__CannyEdgeDetectionImageFilter')
class(`CannyEdgeDetectionImageFilter_GetVariance`) = c("SWIGFunction", class('CannyEdgeDetectionImageFilter_GetVariance'))

# Start of CannyEdgeDetectionImageFilter_SetMaximumError

`CannyEdgeDetectionImageFilter_SetMaximumError__SWIG_0` = function(self, MaximumError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaximumError = as.numeric(MaximumError);
  ;ans = .Call('R_swig_CannyEdgeDetectionImageFilter_SetMaximumError__SWIG_0', self, MaximumError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannyEdgeDetectionImageFilter_SetMaximumError__SWIG_0`, 'returnType') = 'void'
attr(`CannyEdgeDetectionImageFilter_SetMaximumError__SWIG_0`, "inputTypes") = c('_p_itk__simple__CannyEdgeDetectionImageFilter', 'numeric')
class(`CannyEdgeDetectionImageFilter_SetMaximumError__SWIG_0`) = c("SWIGFunction", class('CannyEdgeDetectionImageFilter_SetMaximumError__SWIG_0'))

# Start of CannyEdgeDetectionImageFilter_SetMaximumError

`CannyEdgeDetectionImageFilter_SetMaximumError__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_CannyEdgeDetectionImageFilter_SetMaximumError__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannyEdgeDetectionImageFilter_SetMaximumError__SWIG_1`, 'returnType') = 'void'
attr(`CannyEdgeDetectionImageFilter_SetMaximumError__SWIG_1`, "inputTypes") = c('_p_itk__simple__CannyEdgeDetectionImageFilter', 'numeric')
class(`CannyEdgeDetectionImageFilter_SetMaximumError__SWIG_1`) = c("SWIGFunction", class('CannyEdgeDetectionImageFilter_SetMaximumError__SWIG_1'))

`CannyEdgeDetectionImageFilter_SetMaximumError` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__CannyEdgeDetectionImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- CannyEdgeDetectionImageFilter_SetMaximumError__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__CannyEdgeDetectionImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- CannyEdgeDetectionImageFilter_SetMaximumError__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for CannyEdgeDetectionImageFilter_SetMaximumError with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of CannyEdgeDetectionImageFilter_GetMaximumError

`CannyEdgeDetectionImageFilter_GetMaximumError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannyEdgeDetectionImageFilter_GetMaximumError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannyEdgeDetectionImageFilter_GetMaximumError`, 'returnType') = 'numeric'
attr(`CannyEdgeDetectionImageFilter_GetMaximumError`, "inputTypes") = c('_p_itk__simple__CannyEdgeDetectionImageFilter')
class(`CannyEdgeDetectionImageFilter_GetMaximumError`) = c("SWIGFunction", class('CannyEdgeDetectionImageFilter_GetMaximumError'))

# Start of CannyEdgeDetectionImageFilter_GetName

`CannyEdgeDetectionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannyEdgeDetectionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannyEdgeDetectionImageFilter_GetName`, 'returnType') = 'character'
attr(`CannyEdgeDetectionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__CannyEdgeDetectionImageFilter')
class(`CannyEdgeDetectionImageFilter_GetName`) = c("SWIGFunction", class('CannyEdgeDetectionImageFilter_GetName'))

# Start of CannyEdgeDetectionImageFilter_ToString

`CannyEdgeDetectionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannyEdgeDetectionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannyEdgeDetectionImageFilter_ToString`, 'returnType') = 'character'
attr(`CannyEdgeDetectionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__CannyEdgeDetectionImageFilter')
class(`CannyEdgeDetectionImageFilter_ToString`) = c("SWIGFunction", class('CannyEdgeDetectionImageFilter_ToString'))

# Start of CannyEdgeDetectionImageFilter_Execute

`CannyEdgeDetectionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_CannyEdgeDetectionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannyEdgeDetectionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`CannyEdgeDetectionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__CannyEdgeDetectionImageFilter', '_p_itk__simple__Image')
class(`CannyEdgeDetectionImageFilter_Execute`) = c("SWIGFunction", class('CannyEdgeDetectionImageFilter_Execute'))

# Start of accessor method for itk::simple::CannyEdgeDetectionImageFilter
setMethod('$', '_p_itk__simple__CannyEdgeDetectionImageFilter', function(x, name)

{
  accessorFuns = list('SetLowerThreshold' = CannyEdgeDetectionImageFilter_SetLowerThreshold, 'GetLowerThreshold' = CannyEdgeDetectionImageFilter_GetLowerThreshold, 'SetUpperThreshold' = CannyEdgeDetectionImageFilter_SetUpperThreshold, 'GetUpperThreshold' = CannyEdgeDetectionImageFilter_GetUpperThreshold, 'SetVariance' = CannyEdgeDetectionImageFilter_SetVariance, 'GetVariance' = CannyEdgeDetectionImageFilter_GetVariance, 'SetMaximumError' = CannyEdgeDetectionImageFilter_SetMaximumError, 'GetMaximumError' = CannyEdgeDetectionImageFilter_GetMaximumError, 'GetName' = CannyEdgeDetectionImageFilter_GetName, 'ToString' = CannyEdgeDetectionImageFilter_ToString, 'Execute' = CannyEdgeDetectionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::CannyEdgeDetectionImageFilter
# Start of CannyEdgeDetection

`CannyEdgeDetection__SWIG_0` = function(image1, lowerThreshold, upperThreshold, variance, maximumError, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  variance = as.numeric(variance);
  maximumError = as.numeric(maximumError);
  ;ans = .Call('R_swig_CannyEdgeDetection__SWIG_0', image1, lowerThreshold, upperThreshold, variance, maximumError, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannyEdgeDetection__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`CannyEdgeDetection__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric')
class(`CannyEdgeDetection__SWIG_0`) = c("SWIGFunction", class('CannyEdgeDetection__SWIG_0'))

# Start of CannyEdgeDetection

`CannyEdgeDetection__SWIG_1` = function(image1, lowerThreshold, upperThreshold, variance, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  variance = as.numeric(variance);
  ;ans = .Call('R_swig_CannyEdgeDetection__SWIG_1', image1, lowerThreshold, upperThreshold, variance, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannyEdgeDetection__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`CannyEdgeDetection__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric')
class(`CannyEdgeDetection__SWIG_1`) = c("SWIGFunction", class('CannyEdgeDetection__SWIG_1'))

# Start of CannyEdgeDetection

`CannyEdgeDetection__SWIG_2` = function(image1, lowerThreshold, upperThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_CannyEdgeDetection__SWIG_2', image1, lowerThreshold, upperThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannyEdgeDetection__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`CannyEdgeDetection__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`CannyEdgeDetection__SWIG_2`) = c("SWIGFunction", class('CannyEdgeDetection__SWIG_2'))

# Start of CannyEdgeDetection

`CannyEdgeDetection__SWIG_3` = function(image1, lowerThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_CannyEdgeDetection__SWIG_3', image1, lowerThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannyEdgeDetection__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`CannyEdgeDetection__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`CannyEdgeDetection__SWIG_3`) = c("SWIGFunction", class('CannyEdgeDetection__SWIG_3'))

# Start of CannyEdgeDetection

`CannyEdgeDetection__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_CannyEdgeDetection__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannyEdgeDetection__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`CannyEdgeDetection__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`CannyEdgeDetection__SWIG_4`) = c("SWIGFunction", class('CannyEdgeDetection__SWIG_4'))

`CannyEdgeDetection` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- CannyEdgeDetection__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- CannyEdgeDetection__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- CannyEdgeDetection__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) )) {
      f <- CannyEdgeDetection__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) )) {
      f <- CannyEdgeDetection__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for CannyEdgeDetection with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_CannySegmentationLevelSetImageFilter

`delete_CannySegmentationLevelSetImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_CannySegmentationLevelSetImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_CannySegmentationLevelSetImageFilter`, 'returnType') = 'void'
attr(`delete_CannySegmentationLevelSetImageFilter`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`delete_CannySegmentationLevelSetImageFilter`) = c("SWIGFunction", class('delete_CannySegmentationLevelSetImageFilter'))

# Start of new_CannySegmentationLevelSetImageFilter

`CannySegmentationLevelSetImageFilter` = function()
{
  ;ans = .Call('R_swig_new_CannySegmentationLevelSetImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CannySegmentationLevelSetImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CannySegmentationLevelSetImageFilter);
  ans
  
}

attr(`CannySegmentationLevelSetImageFilter`, 'returnType') = '_p_itk__simple__CannySegmentationLevelSetImageFilter'
class(`CannySegmentationLevelSetImageFilter`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter'))

# Start of CannySegmentationLevelSetImageFilter_SetThreshold

`CannySegmentationLevelSetImageFilter_SetThreshold` = function(self, Threshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_CannySegmentationLevelSetImageFilter_SetThreshold', self, Threshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSetImageFilter_SetThreshold`, 'returnType') = 'void'
attr(`CannySegmentationLevelSetImageFilter_SetThreshold`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter', 'numeric')
class(`CannySegmentationLevelSetImageFilter_SetThreshold`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_SetThreshold'))

# Start of CannySegmentationLevelSetImageFilter_GetThreshold

`CannySegmentationLevelSetImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannySegmentationLevelSetImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannySegmentationLevelSetImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`CannySegmentationLevelSetImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_GetThreshold`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_GetThreshold'))

# Start of CannySegmentationLevelSetImageFilter_SetVariance

`CannySegmentationLevelSetImageFilter_SetVariance` = function(self, Variance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_CannySegmentationLevelSetImageFilter_SetVariance', self, Variance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSetImageFilter_SetVariance`, 'returnType') = 'void'
attr(`CannySegmentationLevelSetImageFilter_SetVariance`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter', 'numeric')
class(`CannySegmentationLevelSetImageFilter_SetVariance`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_SetVariance'))

# Start of CannySegmentationLevelSetImageFilter_GetVariance

`CannySegmentationLevelSetImageFilter_GetVariance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannySegmentationLevelSetImageFilter_GetVariance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannySegmentationLevelSetImageFilter_GetVariance`, 'returnType') = 'numeric'
attr(`CannySegmentationLevelSetImageFilter_GetVariance`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_GetVariance`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_GetVariance'))

# Start of CannySegmentationLevelSetImageFilter_SetMaximumRMSError

`CannySegmentationLevelSetImageFilter_SetMaximumRMSError` = function(self, MaximumRMSError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_CannySegmentationLevelSetImageFilter_SetMaximumRMSError', self, MaximumRMSError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSetImageFilter_SetMaximumRMSError`, 'returnType') = 'void'
attr(`CannySegmentationLevelSetImageFilter_SetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter', 'numeric')
class(`CannySegmentationLevelSetImageFilter_SetMaximumRMSError`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_SetMaximumRMSError'))

# Start of CannySegmentationLevelSetImageFilter_GetMaximumRMSError

`CannySegmentationLevelSetImageFilter_GetMaximumRMSError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannySegmentationLevelSetImageFilter_GetMaximumRMSError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannySegmentationLevelSetImageFilter_GetMaximumRMSError`, 'returnType') = 'numeric'
attr(`CannySegmentationLevelSetImageFilter_GetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_GetMaximumRMSError`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_GetMaximumRMSError'))

# Start of CannySegmentationLevelSetImageFilter_SetPropagationScaling

`CannySegmentationLevelSetImageFilter_SetPropagationScaling` = function(self, PropagationScaling)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_CannySegmentationLevelSetImageFilter_SetPropagationScaling', self, PropagationScaling, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSetImageFilter_SetPropagationScaling`, 'returnType') = 'void'
attr(`CannySegmentationLevelSetImageFilter_SetPropagationScaling`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter', 'numeric')
class(`CannySegmentationLevelSetImageFilter_SetPropagationScaling`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_SetPropagationScaling'))

# Start of CannySegmentationLevelSetImageFilter_GetPropagationScaling

`CannySegmentationLevelSetImageFilter_GetPropagationScaling` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannySegmentationLevelSetImageFilter_GetPropagationScaling', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannySegmentationLevelSetImageFilter_GetPropagationScaling`, 'returnType') = 'numeric'
attr(`CannySegmentationLevelSetImageFilter_GetPropagationScaling`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_GetPropagationScaling`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_GetPropagationScaling'))

# Start of CannySegmentationLevelSetImageFilter_SetCurvatureScaling

`CannySegmentationLevelSetImageFilter_SetCurvatureScaling` = function(self, CurvatureScaling)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_CannySegmentationLevelSetImageFilter_SetCurvatureScaling', self, CurvatureScaling, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSetImageFilter_SetCurvatureScaling`, 'returnType') = 'void'
attr(`CannySegmentationLevelSetImageFilter_SetCurvatureScaling`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter', 'numeric')
class(`CannySegmentationLevelSetImageFilter_SetCurvatureScaling`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_SetCurvatureScaling'))

# Start of CannySegmentationLevelSetImageFilter_GetCurvatureScaling

`CannySegmentationLevelSetImageFilter_GetCurvatureScaling` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannySegmentationLevelSetImageFilter_GetCurvatureScaling', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannySegmentationLevelSetImageFilter_GetCurvatureScaling`, 'returnType') = 'numeric'
attr(`CannySegmentationLevelSetImageFilter_GetCurvatureScaling`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_GetCurvatureScaling`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_GetCurvatureScaling'))

# Start of CannySegmentationLevelSetImageFilter_SetAdvectionScaling

`CannySegmentationLevelSetImageFilter_SetAdvectionScaling` = function(self, AdvectionScaling)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_CannySegmentationLevelSetImageFilter_SetAdvectionScaling', self, AdvectionScaling, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSetImageFilter_SetAdvectionScaling`, 'returnType') = 'void'
attr(`CannySegmentationLevelSetImageFilter_SetAdvectionScaling`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter', 'numeric')
class(`CannySegmentationLevelSetImageFilter_SetAdvectionScaling`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_SetAdvectionScaling'))

# Start of CannySegmentationLevelSetImageFilter_GetAdvectionScaling

`CannySegmentationLevelSetImageFilter_GetAdvectionScaling` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannySegmentationLevelSetImageFilter_GetAdvectionScaling', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannySegmentationLevelSetImageFilter_GetAdvectionScaling`, 'returnType') = 'numeric'
attr(`CannySegmentationLevelSetImageFilter_GetAdvectionScaling`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_GetAdvectionScaling`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_GetAdvectionScaling'))

# Start of CannySegmentationLevelSetImageFilter_SetNumberOfIterations

`CannySegmentationLevelSetImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_CannySegmentationLevelSetImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSetImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`CannySegmentationLevelSetImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter', 'integer')
class(`CannySegmentationLevelSetImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_SetNumberOfIterations'))

# Start of CannySegmentationLevelSetImageFilter_GetNumberOfIterations

`CannySegmentationLevelSetImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannySegmentationLevelSetImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannySegmentationLevelSetImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`CannySegmentationLevelSetImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_GetNumberOfIterations'))

# Start of CannySegmentationLevelSetImageFilter_SetReverseExpansionDirection

`CannySegmentationLevelSetImageFilter_SetReverseExpansionDirection` = function(self, ReverseExpansionDirection)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ReverseExpansionDirection = as.logical(ReverseExpansionDirection);
  ;ans = .Call('R_swig_CannySegmentationLevelSetImageFilter_SetReverseExpansionDirection', self, ReverseExpansionDirection, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSetImageFilter_SetReverseExpansionDirection`, 'returnType') = 'void'
attr(`CannySegmentationLevelSetImageFilter_SetReverseExpansionDirection`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter', 'logical')
class(`CannySegmentationLevelSetImageFilter_SetReverseExpansionDirection`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_SetReverseExpansionDirection'))

# Start of CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOn

`CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOn`, 'returnType') = 'void'
attr(`CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOn`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOn`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOn'))

# Start of CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOff

`CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOff`, 'returnType') = 'void'
attr(`CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOff`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOff`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOff'))

# Start of CannySegmentationLevelSetImageFilter_GetReverseExpansionDirection

`CannySegmentationLevelSetImageFilter_GetReverseExpansionDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannySegmentationLevelSetImageFilter_GetReverseExpansionDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannySegmentationLevelSetImageFilter_GetReverseExpansionDirection`, 'returnType') = 'logical'
attr(`CannySegmentationLevelSetImageFilter_GetReverseExpansionDirection`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_GetReverseExpansionDirection`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_GetReverseExpansionDirection'))

# Start of CannySegmentationLevelSetImageFilter_SetIsoSurfaceValue

`CannySegmentationLevelSetImageFilter_SetIsoSurfaceValue` = function(self, IsoSurfaceValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_CannySegmentationLevelSetImageFilter_SetIsoSurfaceValue', self, IsoSurfaceValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSetImageFilter_SetIsoSurfaceValue`, 'returnType') = 'void'
attr(`CannySegmentationLevelSetImageFilter_SetIsoSurfaceValue`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter', 'numeric')
class(`CannySegmentationLevelSetImageFilter_SetIsoSurfaceValue`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_SetIsoSurfaceValue'))

# Start of CannySegmentationLevelSetImageFilter_GetIsoSurfaceValue

`CannySegmentationLevelSetImageFilter_GetIsoSurfaceValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannySegmentationLevelSetImageFilter_GetIsoSurfaceValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannySegmentationLevelSetImageFilter_GetIsoSurfaceValue`, 'returnType') = 'numeric'
attr(`CannySegmentationLevelSetImageFilter_GetIsoSurfaceValue`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_GetIsoSurfaceValue`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_GetIsoSurfaceValue'))

# Start of CannySegmentationLevelSetImageFilter_GetElapsedIterations

`CannySegmentationLevelSetImageFilter_GetElapsedIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannySegmentationLevelSetImageFilter_GetElapsedIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannySegmentationLevelSetImageFilter_GetElapsedIterations`, 'returnType') = 'integer'
attr(`CannySegmentationLevelSetImageFilter_GetElapsedIterations`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_GetElapsedIterations`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_GetElapsedIterations'))

# Start of CannySegmentationLevelSetImageFilter_GetRMSChange

`CannySegmentationLevelSetImageFilter_GetRMSChange` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannySegmentationLevelSetImageFilter_GetRMSChange', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannySegmentationLevelSetImageFilter_GetRMSChange`, 'returnType') = 'numeric'
attr(`CannySegmentationLevelSetImageFilter_GetRMSChange`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_GetRMSChange`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_GetRMSChange'))

# Start of CannySegmentationLevelSetImageFilter_GetCannyImage

`CannySegmentationLevelSetImageFilter_GetCannyImage` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CannySegmentationLevelSetImageFilter_GetCannyImage', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSetImageFilter_GetCannyImage`, 'returnType') = '_p_itk__simple__Image'
attr(`CannySegmentationLevelSetImageFilter_GetCannyImage`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_GetCannyImage`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_GetCannyImage'))

# Start of CannySegmentationLevelSetImageFilter_GetName

`CannySegmentationLevelSetImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannySegmentationLevelSetImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannySegmentationLevelSetImageFilter_GetName`, 'returnType') = 'character'
attr(`CannySegmentationLevelSetImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_GetName`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_GetName'))

# Start of CannySegmentationLevelSetImageFilter_ToString

`CannySegmentationLevelSetImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CannySegmentationLevelSetImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CannySegmentationLevelSetImageFilter_ToString`, 'returnType') = 'character'
attr(`CannySegmentationLevelSetImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter')
class(`CannySegmentationLevelSetImageFilter_ToString`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_ToString'))

# Start of CannySegmentationLevelSetImageFilter_Execute

`CannySegmentationLevelSetImageFilter_Execute` = function(self, initialImage, featureImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_CannySegmentationLevelSetImageFilter_Execute', self, initialImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSetImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`CannySegmentationLevelSetImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__CannySegmentationLevelSetImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`CannySegmentationLevelSetImageFilter_Execute`) = c("SWIGFunction", class('CannySegmentationLevelSetImageFilter_Execute'))

# Start of accessor method for itk::simple::CannySegmentationLevelSetImageFilter
setMethod('$', '_p_itk__simple__CannySegmentationLevelSetImageFilter', function(x, name)

{
  accessorFuns = list('SetThreshold' = CannySegmentationLevelSetImageFilter_SetThreshold, 'GetThreshold' = CannySegmentationLevelSetImageFilter_GetThreshold, 'SetVariance' = CannySegmentationLevelSetImageFilter_SetVariance, 'GetVariance' = CannySegmentationLevelSetImageFilter_GetVariance, 'SetMaximumRMSError' = CannySegmentationLevelSetImageFilter_SetMaximumRMSError, 'GetMaximumRMSError' = CannySegmentationLevelSetImageFilter_GetMaximumRMSError, 'SetPropagationScaling' = CannySegmentationLevelSetImageFilter_SetPropagationScaling, 'GetPropagationScaling' = CannySegmentationLevelSetImageFilter_GetPropagationScaling, 'SetCurvatureScaling' = CannySegmentationLevelSetImageFilter_SetCurvatureScaling, 'GetCurvatureScaling' = CannySegmentationLevelSetImageFilter_GetCurvatureScaling, 'SetAdvectionScaling' = CannySegmentationLevelSetImageFilter_SetAdvectionScaling, 'GetAdvectionScaling' = CannySegmentationLevelSetImageFilter_GetAdvectionScaling, 'SetNumberOfIterations' = CannySegmentationLevelSetImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = CannySegmentationLevelSetImageFilter_GetNumberOfIterations, 'SetReverseExpansionDirection' = CannySegmentationLevelSetImageFilter_SetReverseExpansionDirection, 'ReverseExpansionDirectionOn' = CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOn, 'ReverseExpansionDirectionOff' = CannySegmentationLevelSetImageFilter_ReverseExpansionDirectionOff, 'GetReverseExpansionDirection' = CannySegmentationLevelSetImageFilter_GetReverseExpansionDirection, 'SetIsoSurfaceValue' = CannySegmentationLevelSetImageFilter_SetIsoSurfaceValue, 'GetIsoSurfaceValue' = CannySegmentationLevelSetImageFilter_GetIsoSurfaceValue, 'GetElapsedIterations' = CannySegmentationLevelSetImageFilter_GetElapsedIterations, 'GetRMSChange' = CannySegmentationLevelSetImageFilter_GetRMSChange, 'GetCannyImage' = CannySegmentationLevelSetImageFilter_GetCannyImage, 'GetName' = CannySegmentationLevelSetImageFilter_GetName, 'ToString' = CannySegmentationLevelSetImageFilter_ToString, 'Execute' = CannySegmentationLevelSetImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::CannySegmentationLevelSetImageFilter
# Start of CannySegmentationLevelSet

`CannySegmentationLevelSet__SWIG_0` = function(initialImage, featureImage, threshold, variance, maximumRMSError, propagationScaling, curvatureScaling, advectionScaling, numberOfIterations, reverseExpansionDirection, isoSurfaceValue, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  reverseExpansionDirection = as.logical(reverseExpansionDirection);
  
  ;ans = .Call('R_swig_CannySegmentationLevelSet__SWIG_0', initialImage, featureImage, threshold, variance, maximumRMSError, propagationScaling, curvatureScaling, advectionScaling, numberOfIterations, reverseExpansionDirection, isoSurfaceValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSet__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`CannySegmentationLevelSet__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'integer', 'logical', 'numeric')
class(`CannySegmentationLevelSet__SWIG_0`) = c("SWIGFunction", class('CannySegmentationLevelSet__SWIG_0'))

# Start of CannySegmentationLevelSet

`CannySegmentationLevelSet__SWIG_1` = function(initialImage, featureImage, threshold, variance, maximumRMSError, propagationScaling, curvatureScaling, advectionScaling, numberOfIterations, reverseExpansionDirection, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  reverseExpansionDirection = as.logical(reverseExpansionDirection);
  ;ans = .Call('R_swig_CannySegmentationLevelSet__SWIG_1', initialImage, featureImage, threshold, variance, maximumRMSError, propagationScaling, curvatureScaling, advectionScaling, numberOfIterations, reverseExpansionDirection, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSet__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`CannySegmentationLevelSet__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'integer', 'logical')
class(`CannySegmentationLevelSet__SWIG_1`) = c("SWIGFunction", class('CannySegmentationLevelSet__SWIG_1'))

# Start of CannySegmentationLevelSet

`CannySegmentationLevelSet__SWIG_2` = function(initialImage, featureImage, threshold, variance, maximumRMSError, propagationScaling, curvatureScaling, advectionScaling, numberOfIterations, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_CannySegmentationLevelSet__SWIG_2', initialImage, featureImage, threshold, variance, maximumRMSError, propagationScaling, curvatureScaling, advectionScaling, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSet__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`CannySegmentationLevelSet__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'integer')
class(`CannySegmentationLevelSet__SWIG_2`) = c("SWIGFunction", class('CannySegmentationLevelSet__SWIG_2'))

# Start of CannySegmentationLevelSet

`CannySegmentationLevelSet__SWIG_3` = function(initialImage, featureImage, threshold, variance, maximumRMSError, propagationScaling, curvatureScaling, advectionScaling, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  
  
  
  ;ans = .Call('R_swig_CannySegmentationLevelSet__SWIG_3', initialImage, featureImage, threshold, variance, maximumRMSError, propagationScaling, curvatureScaling, advectionScaling, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSet__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`CannySegmentationLevelSet__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
class(`CannySegmentationLevelSet__SWIG_3`) = c("SWIGFunction", class('CannySegmentationLevelSet__SWIG_3'))

# Start of CannySegmentationLevelSet

`CannySegmentationLevelSet__SWIG_4` = function(initialImage, featureImage, threshold, variance, maximumRMSError, propagationScaling, curvatureScaling, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  
  
  ;ans = .Call('R_swig_CannySegmentationLevelSet__SWIG_4', initialImage, featureImage, threshold, variance, maximumRMSError, propagationScaling, curvatureScaling, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSet__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`CannySegmentationLevelSet__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
class(`CannySegmentationLevelSet__SWIG_4`) = c("SWIGFunction", class('CannySegmentationLevelSet__SWIG_4'))

# Start of CannySegmentationLevelSet

`CannySegmentationLevelSet__SWIG_5` = function(initialImage, featureImage, threshold, variance, maximumRMSError, propagationScaling, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  
  ;ans = .Call('R_swig_CannySegmentationLevelSet__SWIG_5', initialImage, featureImage, threshold, variance, maximumRMSError, propagationScaling, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSet__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`CannySegmentationLevelSet__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric')
class(`CannySegmentationLevelSet__SWIG_5`) = c("SWIGFunction", class('CannySegmentationLevelSet__SWIG_5'))

# Start of CannySegmentationLevelSet

`CannySegmentationLevelSet__SWIG_6` = function(initialImage, featureImage, threshold, variance, maximumRMSError, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  ;ans = .Call('R_swig_CannySegmentationLevelSet__SWIG_6', initialImage, featureImage, threshold, variance, maximumRMSError, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSet__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`CannySegmentationLevelSet__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric')
class(`CannySegmentationLevelSet__SWIG_6`) = c("SWIGFunction", class('CannySegmentationLevelSet__SWIG_6'))

# Start of CannySegmentationLevelSet

`CannySegmentationLevelSet__SWIG_7` = function(initialImage, featureImage, threshold, variance, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  ;ans = .Call('R_swig_CannySegmentationLevelSet__SWIG_7', initialImage, featureImage, threshold, variance, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSet__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`CannySegmentationLevelSet__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`CannySegmentationLevelSet__SWIG_7`) = c("SWIGFunction", class('CannySegmentationLevelSet__SWIG_7'))

# Start of CannySegmentationLevelSet

`CannySegmentationLevelSet__SWIG_8` = function(initialImage, featureImage, threshold, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  ;ans = .Call('R_swig_CannySegmentationLevelSet__SWIG_8', initialImage, featureImage, threshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSet__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`CannySegmentationLevelSet__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`CannySegmentationLevelSet__SWIG_8`) = c("SWIGFunction", class('CannySegmentationLevelSet__SWIG_8'))

# Start of CannySegmentationLevelSet

`CannySegmentationLevelSet__SWIG_9` = function(initialImage, featureImage, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_CannySegmentationLevelSet__SWIG_9', initialImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CannySegmentationLevelSet__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`CannySegmentationLevelSet__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`CannySegmentationLevelSet__SWIG_9`) = c("SWIGFunction", class('CannySegmentationLevelSet__SWIG_9'))

`CannySegmentationLevelSet` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 10
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- CannySegmentationLevelSet__SWIG_9; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- CannySegmentationLevelSet__SWIG_8; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- CannySegmentationLevelSet__SWIG_7; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- CannySegmentationLevelSet__SWIG_6; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- CannySegmentationLevelSet__SWIG_5; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- CannySegmentationLevelSet__SWIG_4; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 )) {
      f <- CannySegmentationLevelSet__SWIG_3; 
    }
  } else if (argc == 9) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( (is.integer(argv[[9]]) || is.numeric(argv[[9]])) && length(argv[[9]]) == 1 )) {
      f <- CannySegmentationLevelSet__SWIG_2; 
    }
  } else if (argc == 10) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( (is.integer(argv[[9]]) || is.numeric(argv[[9]])) && length(argv[[9]]) == 1 ) && ( is.logical(argv[[10]]) && length(argv[[10]]) == 1 )) {
      f <- CannySegmentationLevelSet__SWIG_1; 
    }
  } else if (argc == 11) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( (is.integer(argv[[9]]) || is.numeric(argv[[9]])) && length(argv[[9]]) == 1 ) && ( is.logical(argv[[10]]) && length(argv[[10]]) == 1 ) && ( is.numeric(argv[[11]]) && length(argv[[11]]) == 1 )) {
      f <- CannySegmentationLevelSet__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for CannySegmentationLevelSet with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ChangeLabelImageFilter

`delete_ChangeLabelImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ChangeLabelImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ChangeLabelImageFilter`, 'returnType') = 'void'
attr(`delete_ChangeLabelImageFilter`, "inputTypes") = c('_p_itk__simple__ChangeLabelImageFilter')
class(`delete_ChangeLabelImageFilter`) = c("SWIGFunction", class('delete_ChangeLabelImageFilter'))

# Start of new_ChangeLabelImageFilter

`ChangeLabelImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ChangeLabelImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ChangeLabelImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ChangeLabelImageFilter);
  ans
  
}

attr(`ChangeLabelImageFilter`, 'returnType') = '_p_itk__simple__ChangeLabelImageFilter'
class(`ChangeLabelImageFilter`) = c("SWIGFunction", class('ChangeLabelImageFilter'))

# Start of ChangeLabelImageFilter_SetChangeMap

`ChangeLabelImageFilter_SetChangeMap` = function(self, ChangeMap)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(ChangeMap, "ExternalReference")) ChangeMap = slot(ChangeMap,"ref"); 
  ;ans = .Call('R_swig_ChangeLabelImageFilter_SetChangeMap', self, ChangeMap, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ChangeLabelImageFilter_SetChangeMap`, 'returnType') = 'void'
attr(`ChangeLabelImageFilter_SetChangeMap`, "inputTypes") = c('_p_itk__simple__ChangeLabelImageFilter', '_p_std__mapT_double_double_t')
class(`ChangeLabelImageFilter_SetChangeMap`) = c("SWIGFunction", class('ChangeLabelImageFilter_SetChangeMap'))

# Start of ChangeLabelImageFilter_GetChangeMap

`ChangeLabelImageFilter_GetChangeMap` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ChangeLabelImageFilter_GetChangeMap', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_std__mapT_double_double_t", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ChangeLabelImageFilter_GetChangeMap`, 'returnType') = '_p_std__mapT_double_double_t'
attr(`ChangeLabelImageFilter_GetChangeMap`, "inputTypes") = c('_p_itk__simple__ChangeLabelImageFilter')
class(`ChangeLabelImageFilter_GetChangeMap`) = c("SWIGFunction", class('ChangeLabelImageFilter_GetChangeMap'))

# Start of ChangeLabelImageFilter_GetName

`ChangeLabelImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ChangeLabelImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ChangeLabelImageFilter_GetName`, 'returnType') = 'character'
attr(`ChangeLabelImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ChangeLabelImageFilter')
class(`ChangeLabelImageFilter_GetName`) = c("SWIGFunction", class('ChangeLabelImageFilter_GetName'))

# Start of ChangeLabelImageFilter_ToString

`ChangeLabelImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ChangeLabelImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ChangeLabelImageFilter_ToString`, 'returnType') = 'character'
attr(`ChangeLabelImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ChangeLabelImageFilter')
class(`ChangeLabelImageFilter_ToString`) = c("SWIGFunction", class('ChangeLabelImageFilter_ToString'))

# Start of ChangeLabelImageFilter_Execute

`ChangeLabelImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ChangeLabelImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ChangeLabelImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ChangeLabelImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ChangeLabelImageFilter', '_p_itk__simple__Image')
class(`ChangeLabelImageFilter_Execute`) = c("SWIGFunction", class('ChangeLabelImageFilter_Execute'))

# Start of accessor method for itk::simple::ChangeLabelImageFilter
setMethod('$', '_p_itk__simple__ChangeLabelImageFilter', function(x, name)

{
  accessorFuns = list('SetChangeMap' = ChangeLabelImageFilter_SetChangeMap, 'GetChangeMap' = ChangeLabelImageFilter_GetChangeMap, 'GetName' = ChangeLabelImageFilter_GetName, 'ToString' = ChangeLabelImageFilter_ToString, 'Execute' = ChangeLabelImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ChangeLabelImageFilter
# Start of ChangeLabel

`ChangeLabel__SWIG_0` = function(image1, changeMap, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(changeMap, "ExternalReference")) changeMap = slot(changeMap,"ref"); 
  ;ans = .Call('R_swig_ChangeLabel__SWIG_0', image1, changeMap, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ChangeLabel__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ChangeLabel__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_std__mapT_double_double_t')
class(`ChangeLabel__SWIG_0`) = c("SWIGFunction", class('ChangeLabel__SWIG_0'))

# Start of ChangeLabel

`ChangeLabel__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ChangeLabel__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ChangeLabel__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ChangeLabel__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`ChangeLabel__SWIG_1`) = c("SWIGFunction", class('ChangeLabel__SWIG_1'))

`ChangeLabel` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ChangeLabel__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_std__mapT_double_double_t') && length(argv[[2]]) == 1 )) {
      f <- ChangeLabel__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ChangeLabel with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ChangeLabelLabelMapFilter

`delete_ChangeLabelLabelMapFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ChangeLabelLabelMapFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ChangeLabelLabelMapFilter`, 'returnType') = 'void'
attr(`delete_ChangeLabelLabelMapFilter`, "inputTypes") = c('_p_itk__simple__ChangeLabelLabelMapFilter')
class(`delete_ChangeLabelLabelMapFilter`) = c("SWIGFunction", class('delete_ChangeLabelLabelMapFilter'))

# Start of new_ChangeLabelLabelMapFilter

`ChangeLabelLabelMapFilter` = function()
{
  ;ans = .Call('R_swig_new_ChangeLabelLabelMapFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ChangeLabelLabelMapFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ChangeLabelLabelMapFilter);
  ans
  
}

attr(`ChangeLabelLabelMapFilter`, 'returnType') = '_p_itk__simple__ChangeLabelLabelMapFilter'
class(`ChangeLabelLabelMapFilter`) = c("SWIGFunction", class('ChangeLabelLabelMapFilter'))

# Start of ChangeLabelLabelMapFilter_SetChangeMap

`ChangeLabelLabelMapFilter_SetChangeMap` = function(self, ChangeMap)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(ChangeMap, "ExternalReference")) ChangeMap = slot(ChangeMap,"ref"); 
  ;ans = .Call('R_swig_ChangeLabelLabelMapFilter_SetChangeMap', self, ChangeMap, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ChangeLabelLabelMapFilter_SetChangeMap`, 'returnType') = 'void'
attr(`ChangeLabelLabelMapFilter_SetChangeMap`, "inputTypes") = c('_p_itk__simple__ChangeLabelLabelMapFilter', '_p_std__mapT_double_double_t')
class(`ChangeLabelLabelMapFilter_SetChangeMap`) = c("SWIGFunction", class('ChangeLabelLabelMapFilter_SetChangeMap'))

# Start of ChangeLabelLabelMapFilter_GetChangeMap

`ChangeLabelLabelMapFilter_GetChangeMap` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ChangeLabelLabelMapFilter_GetChangeMap', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_std__mapT_double_double_t", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ChangeLabelLabelMapFilter_GetChangeMap`, 'returnType') = '_p_std__mapT_double_double_t'
attr(`ChangeLabelLabelMapFilter_GetChangeMap`, "inputTypes") = c('_p_itk__simple__ChangeLabelLabelMapFilter')
class(`ChangeLabelLabelMapFilter_GetChangeMap`) = c("SWIGFunction", class('ChangeLabelLabelMapFilter_GetChangeMap'))

# Start of ChangeLabelLabelMapFilter_GetName

`ChangeLabelLabelMapFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ChangeLabelLabelMapFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ChangeLabelLabelMapFilter_GetName`, 'returnType') = 'character'
attr(`ChangeLabelLabelMapFilter_GetName`, "inputTypes") = c('_p_itk__simple__ChangeLabelLabelMapFilter')
class(`ChangeLabelLabelMapFilter_GetName`) = c("SWIGFunction", class('ChangeLabelLabelMapFilter_GetName'))

# Start of ChangeLabelLabelMapFilter_ToString

`ChangeLabelLabelMapFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ChangeLabelLabelMapFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ChangeLabelLabelMapFilter_ToString`, 'returnType') = 'character'
attr(`ChangeLabelLabelMapFilter_ToString`, "inputTypes") = c('_p_itk__simple__ChangeLabelLabelMapFilter')
class(`ChangeLabelLabelMapFilter_ToString`) = c("SWIGFunction", class('ChangeLabelLabelMapFilter_ToString'))

# Start of ChangeLabelLabelMapFilter_Execute

`ChangeLabelLabelMapFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ChangeLabelLabelMapFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ChangeLabelLabelMapFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ChangeLabelLabelMapFilter_Execute`, "inputTypes") = c('_p_itk__simple__ChangeLabelLabelMapFilter', '_p_itk__simple__Image')
class(`ChangeLabelLabelMapFilter_Execute`) = c("SWIGFunction", class('ChangeLabelLabelMapFilter_Execute'))

# Start of accessor method for itk::simple::ChangeLabelLabelMapFilter
setMethod('$', '_p_itk__simple__ChangeLabelLabelMapFilter', function(x, name)

{
  accessorFuns = list('SetChangeMap' = ChangeLabelLabelMapFilter_SetChangeMap, 'GetChangeMap' = ChangeLabelLabelMapFilter_GetChangeMap, 'GetName' = ChangeLabelLabelMapFilter_GetName, 'ToString' = ChangeLabelLabelMapFilter_ToString, 'Execute' = ChangeLabelLabelMapFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ChangeLabelLabelMapFilter
# Start of ChangeLabelLabelMap

`ChangeLabelLabelMap__SWIG_0` = function(image1, changeMap, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(changeMap, "ExternalReference")) changeMap = slot(changeMap,"ref"); 
  ;ans = .Call('R_swig_ChangeLabelLabelMap__SWIG_0', image1, changeMap, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ChangeLabelLabelMap__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ChangeLabelLabelMap__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_std__mapT_double_double_t')
class(`ChangeLabelLabelMap__SWIG_0`) = c("SWIGFunction", class('ChangeLabelLabelMap__SWIG_0'))

# Start of ChangeLabelLabelMap

`ChangeLabelLabelMap__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ChangeLabelLabelMap__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ChangeLabelLabelMap__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ChangeLabelLabelMap__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`ChangeLabelLabelMap__SWIG_1`) = c("SWIGFunction", class('ChangeLabelLabelMap__SWIG_1'))

`ChangeLabelLabelMap` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ChangeLabelLabelMap__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_std__mapT_double_double_t') && length(argv[[2]]) == 1 )) {
      f <- ChangeLabelLabelMap__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ChangeLabelLabelMap with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_CheckerBoardImageFilter

`delete_CheckerBoardImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_CheckerBoardImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_CheckerBoardImageFilter`, 'returnType') = 'void'
attr(`delete_CheckerBoardImageFilter`, "inputTypes") = c('_p_itk__simple__CheckerBoardImageFilter')
class(`delete_CheckerBoardImageFilter`) = c("SWIGFunction", class('delete_CheckerBoardImageFilter'))

# Start of new_CheckerBoardImageFilter

`CheckerBoardImageFilter` = function()
{
  ;ans = .Call('R_swig_new_CheckerBoardImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CheckerBoardImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CheckerBoardImageFilter);
  ans
  
}

attr(`CheckerBoardImageFilter`, 'returnType') = '_p_itk__simple__CheckerBoardImageFilter'
class(`CheckerBoardImageFilter`) = c("SWIGFunction", class('CheckerBoardImageFilter'))

# Start of CheckerBoardImageFilter_SetCheckerPattern

`CheckerBoardImageFilter_SetCheckerPattern__SWIG_0` = function(self, CheckerPattern)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  CheckerPattern = as.integer(CheckerPattern);
  ;ans = .Call('R_swig_CheckerBoardImageFilter_SetCheckerPattern__SWIG_0', self, CheckerPattern, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CheckerBoardImageFilter_SetCheckerPattern__SWIG_0`, 'returnType') = 'void'
attr(`CheckerBoardImageFilter_SetCheckerPattern__SWIG_0`, "inputTypes") = c('_p_itk__simple__CheckerBoardImageFilter', 'integer')
class(`CheckerBoardImageFilter_SetCheckerPattern__SWIG_0`) = c("SWIGFunction", class('CheckerBoardImageFilter_SetCheckerPattern__SWIG_0'))

# Start of CheckerBoardImageFilter_SetCheckerPattern

`CheckerBoardImageFilter_SetCheckerPattern__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_CheckerBoardImageFilter_SetCheckerPattern__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CheckerBoardImageFilter_SetCheckerPattern__SWIG_1`, 'returnType') = 'void'
attr(`CheckerBoardImageFilter_SetCheckerPattern__SWIG_1`, "inputTypes") = c('_p_itk__simple__CheckerBoardImageFilter', 'integer')
class(`CheckerBoardImageFilter_SetCheckerPattern__SWIG_1`) = c("SWIGFunction", class('CheckerBoardImageFilter_SetCheckerPattern__SWIG_1'))

`CheckerBoardImageFilter_SetCheckerPattern` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__CheckerBoardImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- CheckerBoardImageFilter_SetCheckerPattern__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__CheckerBoardImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- CheckerBoardImageFilter_SetCheckerPattern__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for CheckerBoardImageFilter_SetCheckerPattern with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of CheckerBoardImageFilter_GetCheckerPattern

`CheckerBoardImageFilter_GetCheckerPattern` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CheckerBoardImageFilter_GetCheckerPattern', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CheckerBoardImageFilter_GetCheckerPattern`, 'returnType') = 'integer'
attr(`CheckerBoardImageFilter_GetCheckerPattern`, "inputTypes") = c('_p_itk__simple__CheckerBoardImageFilter')
class(`CheckerBoardImageFilter_GetCheckerPattern`) = c("SWIGFunction", class('CheckerBoardImageFilter_GetCheckerPattern'))

# Start of CheckerBoardImageFilter_GetName

`CheckerBoardImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CheckerBoardImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CheckerBoardImageFilter_GetName`, 'returnType') = 'character'
attr(`CheckerBoardImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__CheckerBoardImageFilter')
class(`CheckerBoardImageFilter_GetName`) = c("SWIGFunction", class('CheckerBoardImageFilter_GetName'))

# Start of CheckerBoardImageFilter_ToString

`CheckerBoardImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CheckerBoardImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CheckerBoardImageFilter_ToString`, 'returnType') = 'character'
attr(`CheckerBoardImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__CheckerBoardImageFilter')
class(`CheckerBoardImageFilter_ToString`) = c("SWIGFunction", class('CheckerBoardImageFilter_ToString'))

# Start of CheckerBoardImageFilter_Execute

`CheckerBoardImageFilter_Execute` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_CheckerBoardImageFilter_Execute', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CheckerBoardImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`CheckerBoardImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__CheckerBoardImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`CheckerBoardImageFilter_Execute`) = c("SWIGFunction", class('CheckerBoardImageFilter_Execute'))

# Start of accessor method for itk::simple::CheckerBoardImageFilter
setMethod('$', '_p_itk__simple__CheckerBoardImageFilter', function(x, name)

{
  accessorFuns = list('SetCheckerPattern' = CheckerBoardImageFilter_SetCheckerPattern, 'GetCheckerPattern' = CheckerBoardImageFilter_GetCheckerPattern, 'GetName' = CheckerBoardImageFilter_GetName, 'ToString' = CheckerBoardImageFilter_ToString, 'Execute' = CheckerBoardImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::CheckerBoardImageFilter
# Start of CheckerBoard

`CheckerBoard__SWIG_0` = function(image1, image2, checkerPattern, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  checkerPattern = as.integer(checkerPattern);
  ;ans = .Call('R_swig_CheckerBoard__SWIG_0', image1, image2, checkerPattern, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CheckerBoard__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`CheckerBoard__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`CheckerBoard__SWIG_0`) = c("SWIGFunction", class('CheckerBoard__SWIG_0'))

# Start of CheckerBoard

`CheckerBoard__SWIG_1` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_CheckerBoard__SWIG_1', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CheckerBoard__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`CheckerBoard__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`CheckerBoard__SWIG_1`) = c("SWIGFunction", class('CheckerBoard__SWIG_1'))

`CheckerBoard` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- CheckerBoard__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- CheckerBoard__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for CheckerBoard with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ClampImageFilter

`delete_ClampImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ClampImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ClampImageFilter`, 'returnType') = 'void'
attr(`delete_ClampImageFilter`, "inputTypes") = c('_p_itk__simple__ClampImageFilter')
class(`delete_ClampImageFilter`) = c("SWIGFunction", class('delete_ClampImageFilter'))

# Start of new_ClampImageFilter

`ClampImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ClampImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ClampImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ClampImageFilter);
  ans
  
}

attr(`ClampImageFilter`, 'returnType') = '_p_itk__simple__ClampImageFilter'
class(`ClampImageFilter`) = c("SWIGFunction", class('ClampImageFilter'))

# Start of ClampImageFilter_SetOutputPixelType

`ClampImageFilter_SetOutputPixelType` = function(self, OutputPixelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputPixelType = enumToInteger(OutputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(OutputPixelType) > 1) {
    warning("using only the first element of OutputPixelType");
  };
  
  ;ans = .Call('R_swig_ClampImageFilter_SetOutputPixelType', self, OutputPixelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ClampImageFilter_SetOutputPixelType`, 'returnType') = 'void'
attr(`ClampImageFilter_SetOutputPixelType`, "inputTypes") = c('_p_itk__simple__ClampImageFilter', 'character')
class(`ClampImageFilter_SetOutputPixelType`) = c("SWIGFunction", class('ClampImageFilter_SetOutputPixelType'))

# Start of ClampImageFilter_GetOutputPixelType

`ClampImageFilter_GetOutputPixelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ClampImageFilter_GetOutputPixelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`ClampImageFilter_GetOutputPixelType`, 'returnType') = 'character'
attr(`ClampImageFilter_GetOutputPixelType`, "inputTypes") = c('_p_itk__simple__ClampImageFilter')
class(`ClampImageFilter_GetOutputPixelType`) = c("SWIGFunction", class('ClampImageFilter_GetOutputPixelType'))

# Start of ClampImageFilter_SetLowerBound

`ClampImageFilter_SetLowerBound` = function(self, LowerBound)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ClampImageFilter_SetLowerBound', self, LowerBound, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ClampImageFilter_SetLowerBound`, 'returnType') = 'void'
attr(`ClampImageFilter_SetLowerBound`, "inputTypes") = c('_p_itk__simple__ClampImageFilter', 'numeric')
class(`ClampImageFilter_SetLowerBound`) = c("SWIGFunction", class('ClampImageFilter_SetLowerBound'))

# Start of ClampImageFilter_GetLowerBound

`ClampImageFilter_GetLowerBound` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ClampImageFilter_GetLowerBound', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ClampImageFilter_GetLowerBound`, 'returnType') = 'numeric'
attr(`ClampImageFilter_GetLowerBound`, "inputTypes") = c('_p_itk__simple__ClampImageFilter')
class(`ClampImageFilter_GetLowerBound`) = c("SWIGFunction", class('ClampImageFilter_GetLowerBound'))

# Start of ClampImageFilter_SetUpperBound

`ClampImageFilter_SetUpperBound` = function(self, UpperBound)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ClampImageFilter_SetUpperBound', self, UpperBound, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ClampImageFilter_SetUpperBound`, 'returnType') = 'void'
attr(`ClampImageFilter_SetUpperBound`, "inputTypes") = c('_p_itk__simple__ClampImageFilter', 'numeric')
class(`ClampImageFilter_SetUpperBound`) = c("SWIGFunction", class('ClampImageFilter_SetUpperBound'))

# Start of ClampImageFilter_GetUpperBound

`ClampImageFilter_GetUpperBound` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ClampImageFilter_GetUpperBound', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ClampImageFilter_GetUpperBound`, 'returnType') = 'numeric'
attr(`ClampImageFilter_GetUpperBound`, "inputTypes") = c('_p_itk__simple__ClampImageFilter')
class(`ClampImageFilter_GetUpperBound`) = c("SWIGFunction", class('ClampImageFilter_GetUpperBound'))

# Start of ClampImageFilter_GetName

`ClampImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ClampImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ClampImageFilter_GetName`, 'returnType') = 'character'
attr(`ClampImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ClampImageFilter')
class(`ClampImageFilter_GetName`) = c("SWIGFunction", class('ClampImageFilter_GetName'))

# Start of ClampImageFilter_ToString

`ClampImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ClampImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ClampImageFilter_ToString`, 'returnType') = 'character'
attr(`ClampImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ClampImageFilter')
class(`ClampImageFilter_ToString`) = c("SWIGFunction", class('ClampImageFilter_ToString'))

# Start of ClampImageFilter_Execute

`ClampImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ClampImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ClampImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ClampImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ClampImageFilter', '_p_itk__simple__Image')
class(`ClampImageFilter_Execute`) = c("SWIGFunction", class('ClampImageFilter_Execute'))

# Start of accessor method for itk::simple::ClampImageFilter
setMethod('$', '_p_itk__simple__ClampImageFilter', function(x, name)

{
  accessorFuns = list('SetOutputPixelType' = ClampImageFilter_SetOutputPixelType, 'GetOutputPixelType' = ClampImageFilter_GetOutputPixelType, 'SetLowerBound' = ClampImageFilter_SetLowerBound, 'GetLowerBound' = ClampImageFilter_GetLowerBound, 'SetUpperBound' = ClampImageFilter_SetUpperBound, 'GetUpperBound' = ClampImageFilter_GetUpperBound, 'GetName' = ClampImageFilter_GetName, 'ToString' = ClampImageFilter_ToString, 'Execute' = ClampImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ClampImageFilter
# Start of Clamp

`Clamp__SWIG_0` = function(image1, outputPixelType, lowerBound, upperBound, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  
  
  ;ans = .Call('R_swig_Clamp__SWIG_0', image1, outputPixelType, lowerBound, upperBound, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Clamp__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Clamp__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'numeric', 'numeric')
class(`Clamp__SWIG_0`) = c("SWIGFunction", class('Clamp__SWIG_0'))

# Start of Clamp

`Clamp__SWIG_1` = function(image1, outputPixelType, lowerBound, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  
  ;ans = .Call('R_swig_Clamp__SWIG_1', image1, outputPixelType, lowerBound, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Clamp__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Clamp__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'numeric')
class(`Clamp__SWIG_1`) = c("SWIGFunction", class('Clamp__SWIG_1'))

# Start of Clamp

`Clamp__SWIG_2` = function(image1, outputPixelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  ;ans = .Call('R_swig_Clamp__SWIG_2', image1, outputPixelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Clamp__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Clamp__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'character')
class(`Clamp__SWIG_2`) = c("SWIGFunction", class('Clamp__SWIG_2'))

# Start of Clamp

`Clamp__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Clamp__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Clamp__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Clamp__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`Clamp__SWIG_3`) = c("SWIGFunction", class('Clamp__SWIG_3'))

`Clamp` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Clamp__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Clamp__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Clamp__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- Clamp__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Clamp with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ClosingByReconstructionImageFilter

`delete_ClosingByReconstructionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ClosingByReconstructionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ClosingByReconstructionImageFilter`, 'returnType') = 'void'
attr(`delete_ClosingByReconstructionImageFilter`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter')
class(`delete_ClosingByReconstructionImageFilter`) = c("SWIGFunction", class('delete_ClosingByReconstructionImageFilter'))

# Start of new_ClosingByReconstructionImageFilter

`ClosingByReconstructionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ClosingByReconstructionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ClosingByReconstructionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ClosingByReconstructionImageFilter);
  ans
  
}

attr(`ClosingByReconstructionImageFilter`, 'returnType') = '_p_itk__simple__ClosingByReconstructionImageFilter'
class(`ClosingByReconstructionImageFilter`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter'))

# Start of ClosingByReconstructionImageFilter_SetKernelRadius

`ClosingByReconstructionImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_ClosingByReconstructionImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ClosingByReconstructionImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`ClosingByReconstructionImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter', 'integer')
class(`ClosingByReconstructionImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_SetKernelRadius__SWIG_0'))

# Start of ClosingByReconstructionImageFilter_SetKernelRadius

`ClosingByReconstructionImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_ClosingByReconstructionImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ClosingByReconstructionImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`ClosingByReconstructionImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter', 'integer')
class(`ClosingByReconstructionImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_SetKernelRadius__SWIG_1'))

`ClosingByReconstructionImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ClosingByReconstructionImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- ClosingByReconstructionImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__ClosingByReconstructionImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- ClosingByReconstructionImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ClosingByReconstructionImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ClosingByReconstructionImageFilter_GetKernelRadius

`ClosingByReconstructionImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ClosingByReconstructionImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ClosingByReconstructionImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`ClosingByReconstructionImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter')
class(`ClosingByReconstructionImageFilter_GetKernelRadius`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_GetKernelRadius'))

# Start of ClosingByReconstructionImageFilter_SetKernelType

`ClosingByReconstructionImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_ClosingByReconstructionImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ClosingByReconstructionImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`ClosingByReconstructionImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter', 'character')
class(`ClosingByReconstructionImageFilter_SetKernelType`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_SetKernelType'))

# Start of ClosingByReconstructionImageFilter_GetKernelType

`ClosingByReconstructionImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ClosingByReconstructionImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`ClosingByReconstructionImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`ClosingByReconstructionImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter')
class(`ClosingByReconstructionImageFilter_GetKernelType`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_GetKernelType'))

# Start of ClosingByReconstructionImageFilter_SetFullyConnected

`ClosingByReconstructionImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_ClosingByReconstructionImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ClosingByReconstructionImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`ClosingByReconstructionImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter', 'logical')
class(`ClosingByReconstructionImageFilter_SetFullyConnected`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_SetFullyConnected'))

# Start of ClosingByReconstructionImageFilter_FullyConnectedOn

`ClosingByReconstructionImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ClosingByReconstructionImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ClosingByReconstructionImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`ClosingByReconstructionImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter')
class(`ClosingByReconstructionImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_FullyConnectedOn'))

# Start of ClosingByReconstructionImageFilter_FullyConnectedOff

`ClosingByReconstructionImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ClosingByReconstructionImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ClosingByReconstructionImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`ClosingByReconstructionImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter')
class(`ClosingByReconstructionImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_FullyConnectedOff'))

# Start of ClosingByReconstructionImageFilter_GetFullyConnected

`ClosingByReconstructionImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ClosingByReconstructionImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ClosingByReconstructionImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`ClosingByReconstructionImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter')
class(`ClosingByReconstructionImageFilter_GetFullyConnected`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_GetFullyConnected'))

# Start of ClosingByReconstructionImageFilter_SetPreserveIntensities

`ClosingByReconstructionImageFilter_SetPreserveIntensities` = function(self, PreserveIntensities)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  PreserveIntensities = as.logical(PreserveIntensities);
  ;ans = .Call('R_swig_ClosingByReconstructionImageFilter_SetPreserveIntensities', self, PreserveIntensities, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ClosingByReconstructionImageFilter_SetPreserveIntensities`, 'returnType') = 'void'
attr(`ClosingByReconstructionImageFilter_SetPreserveIntensities`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter', 'logical')
class(`ClosingByReconstructionImageFilter_SetPreserveIntensities`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_SetPreserveIntensities'))

# Start of ClosingByReconstructionImageFilter_PreserveIntensitiesOn

`ClosingByReconstructionImageFilter_PreserveIntensitiesOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ClosingByReconstructionImageFilter_PreserveIntensitiesOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ClosingByReconstructionImageFilter_PreserveIntensitiesOn`, 'returnType') = 'void'
attr(`ClosingByReconstructionImageFilter_PreserveIntensitiesOn`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter')
class(`ClosingByReconstructionImageFilter_PreserveIntensitiesOn`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_PreserveIntensitiesOn'))

# Start of ClosingByReconstructionImageFilter_PreserveIntensitiesOff

`ClosingByReconstructionImageFilter_PreserveIntensitiesOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ClosingByReconstructionImageFilter_PreserveIntensitiesOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ClosingByReconstructionImageFilter_PreserveIntensitiesOff`, 'returnType') = 'void'
attr(`ClosingByReconstructionImageFilter_PreserveIntensitiesOff`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter')
class(`ClosingByReconstructionImageFilter_PreserveIntensitiesOff`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_PreserveIntensitiesOff'))

# Start of ClosingByReconstructionImageFilter_GetPreserveIntensities

`ClosingByReconstructionImageFilter_GetPreserveIntensities` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ClosingByReconstructionImageFilter_GetPreserveIntensities', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ClosingByReconstructionImageFilter_GetPreserveIntensities`, 'returnType') = 'logical'
attr(`ClosingByReconstructionImageFilter_GetPreserveIntensities`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter')
class(`ClosingByReconstructionImageFilter_GetPreserveIntensities`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_GetPreserveIntensities'))

# Start of ClosingByReconstructionImageFilter_GetName

`ClosingByReconstructionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ClosingByReconstructionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ClosingByReconstructionImageFilter_GetName`, 'returnType') = 'character'
attr(`ClosingByReconstructionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter')
class(`ClosingByReconstructionImageFilter_GetName`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_GetName'))

# Start of ClosingByReconstructionImageFilter_ToString

`ClosingByReconstructionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ClosingByReconstructionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ClosingByReconstructionImageFilter_ToString`, 'returnType') = 'character'
attr(`ClosingByReconstructionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter')
class(`ClosingByReconstructionImageFilter_ToString`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_ToString'))

# Start of ClosingByReconstructionImageFilter_Execute

`ClosingByReconstructionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ClosingByReconstructionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ClosingByReconstructionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ClosingByReconstructionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ClosingByReconstructionImageFilter', '_p_itk__simple__Image')
class(`ClosingByReconstructionImageFilter_Execute`) = c("SWIGFunction", class('ClosingByReconstructionImageFilter_Execute'))

# Start of accessor method for itk::simple::ClosingByReconstructionImageFilter
setMethod('$', '_p_itk__simple__ClosingByReconstructionImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = ClosingByReconstructionImageFilter_SetKernelRadius, 'GetKernelRadius' = ClosingByReconstructionImageFilter_GetKernelRadius, 'SetKernelType' = ClosingByReconstructionImageFilter_SetKernelType, 'GetKernelType' = ClosingByReconstructionImageFilter_GetKernelType, 'SetFullyConnected' = ClosingByReconstructionImageFilter_SetFullyConnected, 'FullyConnectedOn' = ClosingByReconstructionImageFilter_FullyConnectedOn, 'FullyConnectedOff' = ClosingByReconstructionImageFilter_FullyConnectedOff, 'GetFullyConnected' = ClosingByReconstructionImageFilter_GetFullyConnected, 'SetPreserveIntensities' = ClosingByReconstructionImageFilter_SetPreserveIntensities, 'PreserveIntensitiesOn' = ClosingByReconstructionImageFilter_PreserveIntensitiesOn, 'PreserveIntensitiesOff' = ClosingByReconstructionImageFilter_PreserveIntensitiesOff, 'GetPreserveIntensities' = ClosingByReconstructionImageFilter_GetPreserveIntensities, 'GetName' = ClosingByReconstructionImageFilter_GetName, 'ToString' = ClosingByReconstructionImageFilter_ToString, 'Execute' = ClosingByReconstructionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ClosingByReconstructionImageFilter
# Start of ClosingByReconstruction

`ClosingByReconstruction__SWIG_0` = function(image1, kernelRadius, kernelType, fullyConnected, preserveIntensities, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  fullyConnected = as.logical(fullyConnected);
  preserveIntensities = as.logical(preserveIntensities);
  ;ans = .Call('R_swig_ClosingByReconstruction__SWIG_0', image1, kernelRadius, kernelType, fullyConnected, preserveIntensities, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ClosingByReconstruction__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ClosingByReconstruction__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'logical', 'logical')
class(`ClosingByReconstruction__SWIG_0`) = c("SWIGFunction", class('ClosingByReconstruction__SWIG_0'))

# Start of ClosingByReconstruction

`ClosingByReconstruction__SWIG_1` = function(image1, kernelRadius, kernelType, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_ClosingByReconstruction__SWIG_1', image1, kernelRadius, kernelType, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ClosingByReconstruction__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ClosingByReconstruction__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'logical')
class(`ClosingByReconstruction__SWIG_1`) = c("SWIGFunction", class('ClosingByReconstruction__SWIG_1'))

# Start of ClosingByReconstruction

`ClosingByReconstruction__SWIG_2` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_ClosingByReconstruction__SWIG_2', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ClosingByReconstruction__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ClosingByReconstruction__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`ClosingByReconstruction__SWIG_2`) = c("SWIGFunction", class('ClosingByReconstruction__SWIG_2'))

# Start of ClosingByReconstruction

`ClosingByReconstruction__SWIG_3` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_ClosingByReconstruction__SWIG_3', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ClosingByReconstruction__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ClosingByReconstruction__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`ClosingByReconstruction__SWIG_3`) = c("SWIGFunction", class('ClosingByReconstruction__SWIG_3'))

# Start of ClosingByReconstruction

`ClosingByReconstruction__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ClosingByReconstruction__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ClosingByReconstruction__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ClosingByReconstruction__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`ClosingByReconstruction__SWIG_4`) = c("SWIGFunction", class('ClosingByReconstruction__SWIG_4'))

`ClosingByReconstruction` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ClosingByReconstruction__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- ClosingByReconstruction__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ClosingByReconstruction__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ClosingByReconstruction__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- ClosingByReconstruction__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ClosingByReconstruction with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_CollidingFrontsImageFilter

`delete_CollidingFrontsImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_CollidingFrontsImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_CollidingFrontsImageFilter`, 'returnType') = 'void'
attr(`delete_CollidingFrontsImageFilter`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter')
class(`delete_CollidingFrontsImageFilter`) = c("SWIGFunction", class('delete_CollidingFrontsImageFilter'))

# Start of new_CollidingFrontsImageFilter

`CollidingFrontsImageFilter` = function()
{
  ;ans = .Call('R_swig_new_CollidingFrontsImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CollidingFrontsImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CollidingFrontsImageFilter);
  ans
  
}

attr(`CollidingFrontsImageFilter`, 'returnType') = '_p_itk__simple__CollidingFrontsImageFilter'
class(`CollidingFrontsImageFilter`) = c("SWIGFunction", class('CollidingFrontsImageFilter'))

# Start of CollidingFrontsImageFilter_SetSeedPoints1

`CollidingFrontsImageFilter_SetSeedPoints1` = function(self, SeedPoints1)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SeedPoints1 = lapply(SeedPoints1, as.integer);
  ;ans = .Call('R_swig_CollidingFrontsImageFilter_SetSeedPoints1', self, SeedPoints1, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CollidingFrontsImageFilter_SetSeedPoints1`, 'returnType') = 'void'
attr(`CollidingFrontsImageFilter_SetSeedPoints1`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter', 'list')
class(`CollidingFrontsImageFilter_SetSeedPoints1`) = c("SWIGFunction", class('CollidingFrontsImageFilter_SetSeedPoints1'))

# Start of CollidingFrontsImageFilter_GetSeedPoints1

`CollidingFrontsImageFilter_GetSeedPoints1` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CollidingFrontsImageFilter_GetSeedPoints1', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CollidingFrontsImageFilter_GetSeedPoints1`, 'returnType') = 'list'
attr(`CollidingFrontsImageFilter_GetSeedPoints1`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter')
class(`CollidingFrontsImageFilter_GetSeedPoints1`) = c("SWIGFunction", class('CollidingFrontsImageFilter_GetSeedPoints1'))

# Start of CollidingFrontsImageFilter_AddSeedPoint1

`CollidingFrontsImageFilter_AddSeedPoint1` = function(self, point)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  point = as.integer(point);
  ;ans = .Call('R_swig_CollidingFrontsImageFilter_AddSeedPoint1', self, point, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CollidingFrontsImageFilter_AddSeedPoint1`, 'returnType') = 'void'
attr(`CollidingFrontsImageFilter_AddSeedPoint1`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter', 'integer')
class(`CollidingFrontsImageFilter_AddSeedPoint1`) = c("SWIGFunction", class('CollidingFrontsImageFilter_AddSeedPoint1'))

# Start of CollidingFrontsImageFilter_ClearSeedPoints1

`CollidingFrontsImageFilter_ClearSeedPoints1` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CollidingFrontsImageFilter_ClearSeedPoints1', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CollidingFrontsImageFilter_ClearSeedPoints1`, 'returnType') = 'void'
attr(`CollidingFrontsImageFilter_ClearSeedPoints1`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter')
class(`CollidingFrontsImageFilter_ClearSeedPoints1`) = c("SWIGFunction", class('CollidingFrontsImageFilter_ClearSeedPoints1'))

# Start of CollidingFrontsImageFilter_SetSeedPoints2

`CollidingFrontsImageFilter_SetSeedPoints2` = function(self, SeedPoints2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SeedPoints2 = lapply(SeedPoints2, as.integer);
  ;ans = .Call('R_swig_CollidingFrontsImageFilter_SetSeedPoints2', self, SeedPoints2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CollidingFrontsImageFilter_SetSeedPoints2`, 'returnType') = 'void'
attr(`CollidingFrontsImageFilter_SetSeedPoints2`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter', 'list')
class(`CollidingFrontsImageFilter_SetSeedPoints2`) = c("SWIGFunction", class('CollidingFrontsImageFilter_SetSeedPoints2'))

# Start of CollidingFrontsImageFilter_GetSeedPoints2

`CollidingFrontsImageFilter_GetSeedPoints2` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CollidingFrontsImageFilter_GetSeedPoints2', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CollidingFrontsImageFilter_GetSeedPoints2`, 'returnType') = 'list'
attr(`CollidingFrontsImageFilter_GetSeedPoints2`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter')
class(`CollidingFrontsImageFilter_GetSeedPoints2`) = c("SWIGFunction", class('CollidingFrontsImageFilter_GetSeedPoints2'))

# Start of CollidingFrontsImageFilter_AddSeedPoint2

`CollidingFrontsImageFilter_AddSeedPoint2` = function(self, point)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  point = as.integer(point);
  ;ans = .Call('R_swig_CollidingFrontsImageFilter_AddSeedPoint2', self, point, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CollidingFrontsImageFilter_AddSeedPoint2`, 'returnType') = 'void'
attr(`CollidingFrontsImageFilter_AddSeedPoint2`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter', 'integer')
class(`CollidingFrontsImageFilter_AddSeedPoint2`) = c("SWIGFunction", class('CollidingFrontsImageFilter_AddSeedPoint2'))

# Start of CollidingFrontsImageFilter_ClearSeedPoints2

`CollidingFrontsImageFilter_ClearSeedPoints2` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CollidingFrontsImageFilter_ClearSeedPoints2', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CollidingFrontsImageFilter_ClearSeedPoints2`, 'returnType') = 'void'
attr(`CollidingFrontsImageFilter_ClearSeedPoints2`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter')
class(`CollidingFrontsImageFilter_ClearSeedPoints2`) = c("SWIGFunction", class('CollidingFrontsImageFilter_ClearSeedPoints2'))

# Start of CollidingFrontsImageFilter_SetApplyConnectivity

`CollidingFrontsImageFilter_SetApplyConnectivity` = function(self, ApplyConnectivity)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ApplyConnectivity = as.logical(ApplyConnectivity);
  ;ans = .Call('R_swig_CollidingFrontsImageFilter_SetApplyConnectivity', self, ApplyConnectivity, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CollidingFrontsImageFilter_SetApplyConnectivity`, 'returnType') = 'void'
attr(`CollidingFrontsImageFilter_SetApplyConnectivity`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter', 'logical')
class(`CollidingFrontsImageFilter_SetApplyConnectivity`) = c("SWIGFunction", class('CollidingFrontsImageFilter_SetApplyConnectivity'))

# Start of CollidingFrontsImageFilter_ApplyConnectivityOn

`CollidingFrontsImageFilter_ApplyConnectivityOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CollidingFrontsImageFilter_ApplyConnectivityOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CollidingFrontsImageFilter_ApplyConnectivityOn`, 'returnType') = 'void'
attr(`CollidingFrontsImageFilter_ApplyConnectivityOn`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter')
class(`CollidingFrontsImageFilter_ApplyConnectivityOn`) = c("SWIGFunction", class('CollidingFrontsImageFilter_ApplyConnectivityOn'))

# Start of CollidingFrontsImageFilter_ApplyConnectivityOff

`CollidingFrontsImageFilter_ApplyConnectivityOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CollidingFrontsImageFilter_ApplyConnectivityOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CollidingFrontsImageFilter_ApplyConnectivityOff`, 'returnType') = 'void'
attr(`CollidingFrontsImageFilter_ApplyConnectivityOff`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter')
class(`CollidingFrontsImageFilter_ApplyConnectivityOff`) = c("SWIGFunction", class('CollidingFrontsImageFilter_ApplyConnectivityOff'))

# Start of CollidingFrontsImageFilter_GetApplyConnectivity

`CollidingFrontsImageFilter_GetApplyConnectivity` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CollidingFrontsImageFilter_GetApplyConnectivity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CollidingFrontsImageFilter_GetApplyConnectivity`, 'returnType') = 'logical'
attr(`CollidingFrontsImageFilter_GetApplyConnectivity`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter')
class(`CollidingFrontsImageFilter_GetApplyConnectivity`) = c("SWIGFunction", class('CollidingFrontsImageFilter_GetApplyConnectivity'))

# Start of CollidingFrontsImageFilter_SetNegativeEpsilon

`CollidingFrontsImageFilter_SetNegativeEpsilon` = function(self, NegativeEpsilon)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_CollidingFrontsImageFilter_SetNegativeEpsilon', self, NegativeEpsilon, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CollidingFrontsImageFilter_SetNegativeEpsilon`, 'returnType') = 'void'
attr(`CollidingFrontsImageFilter_SetNegativeEpsilon`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter', 'numeric')
class(`CollidingFrontsImageFilter_SetNegativeEpsilon`) = c("SWIGFunction", class('CollidingFrontsImageFilter_SetNegativeEpsilon'))

# Start of CollidingFrontsImageFilter_GetNegativeEpsilon

`CollidingFrontsImageFilter_GetNegativeEpsilon` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CollidingFrontsImageFilter_GetNegativeEpsilon', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CollidingFrontsImageFilter_GetNegativeEpsilon`, 'returnType') = 'numeric'
attr(`CollidingFrontsImageFilter_GetNegativeEpsilon`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter')
class(`CollidingFrontsImageFilter_GetNegativeEpsilon`) = c("SWIGFunction", class('CollidingFrontsImageFilter_GetNegativeEpsilon'))

# Start of CollidingFrontsImageFilter_SetStopOnTargets

`CollidingFrontsImageFilter_SetStopOnTargets` = function(self, StopOnTargets)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  StopOnTargets = as.logical(StopOnTargets);
  ;ans = .Call('R_swig_CollidingFrontsImageFilter_SetStopOnTargets', self, StopOnTargets, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CollidingFrontsImageFilter_SetStopOnTargets`, 'returnType') = 'void'
attr(`CollidingFrontsImageFilter_SetStopOnTargets`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter', 'logical')
class(`CollidingFrontsImageFilter_SetStopOnTargets`) = c("SWIGFunction", class('CollidingFrontsImageFilter_SetStopOnTargets'))

# Start of CollidingFrontsImageFilter_StopOnTargetsOn

`CollidingFrontsImageFilter_StopOnTargetsOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CollidingFrontsImageFilter_StopOnTargetsOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CollidingFrontsImageFilter_StopOnTargetsOn`, 'returnType') = 'void'
attr(`CollidingFrontsImageFilter_StopOnTargetsOn`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter')
class(`CollidingFrontsImageFilter_StopOnTargetsOn`) = c("SWIGFunction", class('CollidingFrontsImageFilter_StopOnTargetsOn'))

# Start of CollidingFrontsImageFilter_StopOnTargetsOff

`CollidingFrontsImageFilter_StopOnTargetsOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_CollidingFrontsImageFilter_StopOnTargetsOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CollidingFrontsImageFilter_StopOnTargetsOff`, 'returnType') = 'void'
attr(`CollidingFrontsImageFilter_StopOnTargetsOff`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter')
class(`CollidingFrontsImageFilter_StopOnTargetsOff`) = c("SWIGFunction", class('CollidingFrontsImageFilter_StopOnTargetsOff'))

# Start of CollidingFrontsImageFilter_GetStopOnTargets

`CollidingFrontsImageFilter_GetStopOnTargets` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CollidingFrontsImageFilter_GetStopOnTargets', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CollidingFrontsImageFilter_GetStopOnTargets`, 'returnType') = 'logical'
attr(`CollidingFrontsImageFilter_GetStopOnTargets`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter')
class(`CollidingFrontsImageFilter_GetStopOnTargets`) = c("SWIGFunction", class('CollidingFrontsImageFilter_GetStopOnTargets'))

# Start of CollidingFrontsImageFilter_GetName

`CollidingFrontsImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CollidingFrontsImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CollidingFrontsImageFilter_GetName`, 'returnType') = 'character'
attr(`CollidingFrontsImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter')
class(`CollidingFrontsImageFilter_GetName`) = c("SWIGFunction", class('CollidingFrontsImageFilter_GetName'))

# Start of CollidingFrontsImageFilter_ToString

`CollidingFrontsImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CollidingFrontsImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CollidingFrontsImageFilter_ToString`, 'returnType') = 'character'
attr(`CollidingFrontsImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter')
class(`CollidingFrontsImageFilter_ToString`) = c("SWIGFunction", class('CollidingFrontsImageFilter_ToString'))

# Start of CollidingFrontsImageFilter_Execute

`CollidingFrontsImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_CollidingFrontsImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CollidingFrontsImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`CollidingFrontsImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__CollidingFrontsImageFilter', '_p_itk__simple__Image')
class(`CollidingFrontsImageFilter_Execute`) = c("SWIGFunction", class('CollidingFrontsImageFilter_Execute'))

# Start of accessor method for itk::simple::CollidingFrontsImageFilter
setMethod('$', '_p_itk__simple__CollidingFrontsImageFilter', function(x, name)

{
  accessorFuns = list('SetSeedPoints1' = CollidingFrontsImageFilter_SetSeedPoints1, 'GetSeedPoints1' = CollidingFrontsImageFilter_GetSeedPoints1, 'AddSeedPoint1' = CollidingFrontsImageFilter_AddSeedPoint1, 'ClearSeedPoints1' = CollidingFrontsImageFilter_ClearSeedPoints1, 'SetSeedPoints2' = CollidingFrontsImageFilter_SetSeedPoints2, 'GetSeedPoints2' = CollidingFrontsImageFilter_GetSeedPoints2, 'AddSeedPoint2' = CollidingFrontsImageFilter_AddSeedPoint2, 'ClearSeedPoints2' = CollidingFrontsImageFilter_ClearSeedPoints2, 'SetApplyConnectivity' = CollidingFrontsImageFilter_SetApplyConnectivity, 'ApplyConnectivityOn' = CollidingFrontsImageFilter_ApplyConnectivityOn, 'ApplyConnectivityOff' = CollidingFrontsImageFilter_ApplyConnectivityOff, 'GetApplyConnectivity' = CollidingFrontsImageFilter_GetApplyConnectivity, 'SetNegativeEpsilon' = CollidingFrontsImageFilter_SetNegativeEpsilon, 'GetNegativeEpsilon' = CollidingFrontsImageFilter_GetNegativeEpsilon, 'SetStopOnTargets' = CollidingFrontsImageFilter_SetStopOnTargets, 'StopOnTargetsOn' = CollidingFrontsImageFilter_StopOnTargetsOn, 'StopOnTargetsOff' = CollidingFrontsImageFilter_StopOnTargetsOff, 'GetStopOnTargets' = CollidingFrontsImageFilter_GetStopOnTargets, 'GetName' = CollidingFrontsImageFilter_GetName, 'ToString' = CollidingFrontsImageFilter_ToString, 'Execute' = CollidingFrontsImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::CollidingFrontsImageFilter
# Start of CollidingFronts

`CollidingFronts__SWIG_0` = function(image1, seedPoints1, seedPoints2, applyConnectivity, negativeEpsilon, stopOnTargets, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedPoints1 = lapply(seedPoints1, as.integer);
  seedPoints2 = lapply(seedPoints2, as.integer);
  applyConnectivity = as.logical(applyConnectivity);
  
  stopOnTargets = as.logical(stopOnTargets);
  ;ans = .Call('R_swig_CollidingFronts__SWIG_0', image1, seedPoints1, seedPoints2, applyConnectivity, negativeEpsilon, stopOnTargets, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CollidingFronts__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`CollidingFronts__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'list', 'logical', 'numeric', 'logical')
class(`CollidingFronts__SWIG_0`) = c("SWIGFunction", class('CollidingFronts__SWIG_0'))

# Start of CollidingFronts

`CollidingFronts__SWIG_1` = function(image1, seedPoints1, seedPoints2, applyConnectivity, negativeEpsilon, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedPoints1 = lapply(seedPoints1, as.integer);
  seedPoints2 = lapply(seedPoints2, as.integer);
  applyConnectivity = as.logical(applyConnectivity);
  
  ;ans = .Call('R_swig_CollidingFronts__SWIG_1', image1, seedPoints1, seedPoints2, applyConnectivity, negativeEpsilon, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CollidingFronts__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`CollidingFronts__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'list', 'logical', 'numeric')
class(`CollidingFronts__SWIG_1`) = c("SWIGFunction", class('CollidingFronts__SWIG_1'))

# Start of CollidingFronts

`CollidingFronts__SWIG_2` = function(image1, seedPoints1, seedPoints2, applyConnectivity, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedPoints1 = lapply(seedPoints1, as.integer);
  seedPoints2 = lapply(seedPoints2, as.integer);
  applyConnectivity = as.logical(applyConnectivity);
  ;ans = .Call('R_swig_CollidingFronts__SWIG_2', image1, seedPoints1, seedPoints2, applyConnectivity, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CollidingFronts__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`CollidingFronts__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'list', 'logical')
class(`CollidingFronts__SWIG_2`) = c("SWIGFunction", class('CollidingFronts__SWIG_2'))

# Start of CollidingFronts

`CollidingFronts__SWIG_3` = function(image1, seedPoints1, seedPoints2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedPoints1 = lapply(seedPoints1, as.integer);
  seedPoints2 = lapply(seedPoints2, as.integer);
  ;ans = .Call('R_swig_CollidingFronts__SWIG_3', image1, seedPoints1, seedPoints2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CollidingFronts__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`CollidingFronts__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'list')
class(`CollidingFronts__SWIG_3`) = c("SWIGFunction", class('CollidingFronts__SWIG_3'))

# Start of CollidingFronts

`CollidingFronts__SWIG_4` = function(image1, seedPoints1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedPoints1 = lapply(seedPoints1, as.integer);
  ;ans = .Call('R_swig_CollidingFronts__SWIG_4', image1, seedPoints1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CollidingFronts__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`CollidingFronts__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'list')
class(`CollidingFronts__SWIG_4`) = c("SWIGFunction", class('CollidingFronts__SWIG_4'))

# Start of CollidingFronts

`CollidingFronts__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_CollidingFronts__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CollidingFronts__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`CollidingFronts__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`CollidingFronts__SWIG_5`) = c("SWIGFunction", class('CollidingFronts__SWIG_5'))

`CollidingFronts` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- CollidingFronts__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) )) {
      f <- CollidingFronts__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.list(argv[[3]]) && all(sapply(argv[[3]] , is.integer) || sapply(argv[[3]], is.numeric)) )) {
      f <- CollidingFronts__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.list(argv[[3]]) && all(sapply(argv[[3]] , is.integer) || sapply(argv[[3]], is.numeric)) ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- CollidingFronts__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.list(argv[[3]]) && all(sapply(argv[[3]] , is.integer) || sapply(argv[[3]], is.numeric)) ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- CollidingFronts__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.list(argv[[3]]) && all(sapply(argv[[3]] , is.integer) || sapply(argv[[3]], is.numeric)) ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- CollidingFronts__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for CollidingFronts with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ComplexToImaginaryImageFilter

`delete_ComplexToImaginaryImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ComplexToImaginaryImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ComplexToImaginaryImageFilter`, 'returnType') = 'void'
attr(`delete_ComplexToImaginaryImageFilter`, "inputTypes") = c('_p_itk__simple__ComplexToImaginaryImageFilter')
class(`delete_ComplexToImaginaryImageFilter`) = c("SWIGFunction", class('delete_ComplexToImaginaryImageFilter'))

# Start of new_ComplexToImaginaryImageFilter

`ComplexToImaginaryImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ComplexToImaginaryImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ComplexToImaginaryImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ComplexToImaginaryImageFilter);
  ans
  
}

attr(`ComplexToImaginaryImageFilter`, 'returnType') = '_p_itk__simple__ComplexToImaginaryImageFilter'
class(`ComplexToImaginaryImageFilter`) = c("SWIGFunction", class('ComplexToImaginaryImageFilter'))

# Start of ComplexToImaginaryImageFilter_GetName

`ComplexToImaginaryImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComplexToImaginaryImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComplexToImaginaryImageFilter_GetName`, 'returnType') = 'character'
attr(`ComplexToImaginaryImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ComplexToImaginaryImageFilter')
class(`ComplexToImaginaryImageFilter_GetName`) = c("SWIGFunction", class('ComplexToImaginaryImageFilter_GetName'))

# Start of ComplexToImaginaryImageFilter_ToString

`ComplexToImaginaryImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComplexToImaginaryImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComplexToImaginaryImageFilter_ToString`, 'returnType') = 'character'
attr(`ComplexToImaginaryImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ComplexToImaginaryImageFilter')
class(`ComplexToImaginaryImageFilter_ToString`) = c("SWIGFunction", class('ComplexToImaginaryImageFilter_ToString'))

# Start of ComplexToImaginaryImageFilter_Execute

`ComplexToImaginaryImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ComplexToImaginaryImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ComplexToImaginaryImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ComplexToImaginaryImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ComplexToImaginaryImageFilter', '_p_itk__simple__Image')
class(`ComplexToImaginaryImageFilter_Execute`) = c("SWIGFunction", class('ComplexToImaginaryImageFilter_Execute'))

# Start of accessor method for itk::simple::ComplexToImaginaryImageFilter
setMethod('$', '_p_itk__simple__ComplexToImaginaryImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = ComplexToImaginaryImageFilter_GetName, 'ToString' = ComplexToImaginaryImageFilter_ToString, 'Execute' = ComplexToImaginaryImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ComplexToImaginaryImageFilter
# Start of ComplexToImaginary

`ComplexToImaginary` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ComplexToImaginary', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ComplexToImaginary`, 'returnType') = '_p_itk__simple__Image'
attr(`ComplexToImaginary`, "inputTypes") = c('_p_itk__simple__Image')
class(`ComplexToImaginary`) = c("SWIGFunction", class('ComplexToImaginary'))

# Start of delete_ComplexToModulusImageFilter

`delete_ComplexToModulusImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ComplexToModulusImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ComplexToModulusImageFilter`, 'returnType') = 'void'
attr(`delete_ComplexToModulusImageFilter`, "inputTypes") = c('_p_itk__simple__ComplexToModulusImageFilter')
class(`delete_ComplexToModulusImageFilter`) = c("SWIGFunction", class('delete_ComplexToModulusImageFilter'))

# Start of new_ComplexToModulusImageFilter

`ComplexToModulusImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ComplexToModulusImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ComplexToModulusImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ComplexToModulusImageFilter);
  ans
  
}

attr(`ComplexToModulusImageFilter`, 'returnType') = '_p_itk__simple__ComplexToModulusImageFilter'
class(`ComplexToModulusImageFilter`) = c("SWIGFunction", class('ComplexToModulusImageFilter'))

# Start of ComplexToModulusImageFilter_GetName

`ComplexToModulusImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComplexToModulusImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComplexToModulusImageFilter_GetName`, 'returnType') = 'character'
attr(`ComplexToModulusImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ComplexToModulusImageFilter')
class(`ComplexToModulusImageFilter_GetName`) = c("SWIGFunction", class('ComplexToModulusImageFilter_GetName'))

# Start of ComplexToModulusImageFilter_ToString

`ComplexToModulusImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComplexToModulusImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComplexToModulusImageFilter_ToString`, 'returnType') = 'character'
attr(`ComplexToModulusImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ComplexToModulusImageFilter')
class(`ComplexToModulusImageFilter_ToString`) = c("SWIGFunction", class('ComplexToModulusImageFilter_ToString'))

# Start of ComplexToModulusImageFilter_Execute

`ComplexToModulusImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ComplexToModulusImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ComplexToModulusImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ComplexToModulusImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ComplexToModulusImageFilter', '_p_itk__simple__Image')
class(`ComplexToModulusImageFilter_Execute`) = c("SWIGFunction", class('ComplexToModulusImageFilter_Execute'))

# Start of accessor method for itk::simple::ComplexToModulusImageFilter
setMethod('$', '_p_itk__simple__ComplexToModulusImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = ComplexToModulusImageFilter_GetName, 'ToString' = ComplexToModulusImageFilter_ToString, 'Execute' = ComplexToModulusImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ComplexToModulusImageFilter
# Start of ComplexToModulus

`ComplexToModulus` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ComplexToModulus', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ComplexToModulus`, 'returnType') = '_p_itk__simple__Image'
attr(`ComplexToModulus`, "inputTypes") = c('_p_itk__simple__Image')
class(`ComplexToModulus`) = c("SWIGFunction", class('ComplexToModulus'))

# Start of delete_ComplexToPhaseImageFilter

`delete_ComplexToPhaseImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ComplexToPhaseImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ComplexToPhaseImageFilter`, 'returnType') = 'void'
attr(`delete_ComplexToPhaseImageFilter`, "inputTypes") = c('_p_itk__simple__ComplexToPhaseImageFilter')
class(`delete_ComplexToPhaseImageFilter`) = c("SWIGFunction", class('delete_ComplexToPhaseImageFilter'))

# Start of new_ComplexToPhaseImageFilter

`ComplexToPhaseImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ComplexToPhaseImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ComplexToPhaseImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ComplexToPhaseImageFilter);
  ans
  
}

attr(`ComplexToPhaseImageFilter`, 'returnType') = '_p_itk__simple__ComplexToPhaseImageFilter'
class(`ComplexToPhaseImageFilter`) = c("SWIGFunction", class('ComplexToPhaseImageFilter'))

# Start of ComplexToPhaseImageFilter_GetName

`ComplexToPhaseImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComplexToPhaseImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComplexToPhaseImageFilter_GetName`, 'returnType') = 'character'
attr(`ComplexToPhaseImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ComplexToPhaseImageFilter')
class(`ComplexToPhaseImageFilter_GetName`) = c("SWIGFunction", class('ComplexToPhaseImageFilter_GetName'))

# Start of ComplexToPhaseImageFilter_ToString

`ComplexToPhaseImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComplexToPhaseImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComplexToPhaseImageFilter_ToString`, 'returnType') = 'character'
attr(`ComplexToPhaseImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ComplexToPhaseImageFilter')
class(`ComplexToPhaseImageFilter_ToString`) = c("SWIGFunction", class('ComplexToPhaseImageFilter_ToString'))

# Start of ComplexToPhaseImageFilter_Execute

`ComplexToPhaseImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ComplexToPhaseImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ComplexToPhaseImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ComplexToPhaseImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ComplexToPhaseImageFilter', '_p_itk__simple__Image')
class(`ComplexToPhaseImageFilter_Execute`) = c("SWIGFunction", class('ComplexToPhaseImageFilter_Execute'))

# Start of accessor method for itk::simple::ComplexToPhaseImageFilter
setMethod('$', '_p_itk__simple__ComplexToPhaseImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = ComplexToPhaseImageFilter_GetName, 'ToString' = ComplexToPhaseImageFilter_ToString, 'Execute' = ComplexToPhaseImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ComplexToPhaseImageFilter
# Start of ComplexToPhase

`ComplexToPhase` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ComplexToPhase', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ComplexToPhase`, 'returnType') = '_p_itk__simple__Image'
attr(`ComplexToPhase`, "inputTypes") = c('_p_itk__simple__Image')
class(`ComplexToPhase`) = c("SWIGFunction", class('ComplexToPhase'))

# Start of delete_ComplexToRealImageFilter

`delete_ComplexToRealImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ComplexToRealImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ComplexToRealImageFilter`, 'returnType') = 'void'
attr(`delete_ComplexToRealImageFilter`, "inputTypes") = c('_p_itk__simple__ComplexToRealImageFilter')
class(`delete_ComplexToRealImageFilter`) = c("SWIGFunction", class('delete_ComplexToRealImageFilter'))

# Start of new_ComplexToRealImageFilter

`ComplexToRealImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ComplexToRealImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ComplexToRealImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ComplexToRealImageFilter);
  ans
  
}

attr(`ComplexToRealImageFilter`, 'returnType') = '_p_itk__simple__ComplexToRealImageFilter'
class(`ComplexToRealImageFilter`) = c("SWIGFunction", class('ComplexToRealImageFilter'))

# Start of ComplexToRealImageFilter_GetName

`ComplexToRealImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComplexToRealImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComplexToRealImageFilter_GetName`, 'returnType') = 'character'
attr(`ComplexToRealImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ComplexToRealImageFilter')
class(`ComplexToRealImageFilter_GetName`) = c("SWIGFunction", class('ComplexToRealImageFilter_GetName'))

# Start of ComplexToRealImageFilter_ToString

`ComplexToRealImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComplexToRealImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComplexToRealImageFilter_ToString`, 'returnType') = 'character'
attr(`ComplexToRealImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ComplexToRealImageFilter')
class(`ComplexToRealImageFilter_ToString`) = c("SWIGFunction", class('ComplexToRealImageFilter_ToString'))

# Start of ComplexToRealImageFilter_Execute

`ComplexToRealImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ComplexToRealImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ComplexToRealImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ComplexToRealImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ComplexToRealImageFilter', '_p_itk__simple__Image')
class(`ComplexToRealImageFilter_Execute`) = c("SWIGFunction", class('ComplexToRealImageFilter_Execute'))

# Start of accessor method for itk::simple::ComplexToRealImageFilter
setMethod('$', '_p_itk__simple__ComplexToRealImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = ComplexToRealImageFilter_GetName, 'ToString' = ComplexToRealImageFilter_ToString, 'Execute' = ComplexToRealImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ComplexToRealImageFilter
# Start of ComplexToReal

`ComplexToReal` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ComplexToReal', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ComplexToReal`, 'returnType') = '_p_itk__simple__Image'
attr(`ComplexToReal`, "inputTypes") = c('_p_itk__simple__Image')
class(`ComplexToReal`) = c("SWIGFunction", class('ComplexToReal'))

# Start of delete_ComposeImageFilter

`delete_ComposeImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ComposeImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ComposeImageFilter`, 'returnType') = 'void'
attr(`delete_ComposeImageFilter`, "inputTypes") = c('_p_itk__simple__ComposeImageFilter')
class(`delete_ComposeImageFilter`) = c("SWIGFunction", class('delete_ComposeImageFilter'))

# Start of new_ComposeImageFilter

`ComposeImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ComposeImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ComposeImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ComposeImageFilter);
  ans
  
}

attr(`ComposeImageFilter`, 'returnType') = '_p_itk__simple__ComposeImageFilter'
class(`ComposeImageFilter`) = c("SWIGFunction", class('ComposeImageFilter'))

# Start of ComposeImageFilter_GetName

`ComposeImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComposeImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComposeImageFilter_GetName`, 'returnType') = 'character'
attr(`ComposeImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ComposeImageFilter')
class(`ComposeImageFilter_GetName`) = c("SWIGFunction", class('ComposeImageFilter_GetName'))

# Start of ComposeImageFilter_ToString

`ComposeImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ComposeImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ComposeImageFilter_ToString`, 'returnType') = 'character'
attr(`ComposeImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ComposeImageFilter')
class(`ComposeImageFilter_ToString`) = c("SWIGFunction", class('ComposeImageFilter_ToString'))

# Start of ComposeImageFilter_Execute

`ComposeImageFilter_Execute__SWIG_0` = function(self, images, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_ComposeImageFilter_Execute__SWIG_0', self, images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ComposeImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ComposeImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__ComposeImageFilter', 'list')
class(`ComposeImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('ComposeImageFilter_Execute__SWIG_0'))

# Start of ComposeImageFilter_Execute

`ComposeImageFilter_Execute__SWIG_1` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ComposeImageFilter_Execute__SWIG_1', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ComposeImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ComposeImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__ComposeImageFilter', '_p_itk__simple__Image')
class(`ComposeImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('ComposeImageFilter_Execute__SWIG_1'))

# Start of ComposeImageFilter_Execute

`ComposeImageFilter_Execute__SWIG_2` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_ComposeImageFilter_Execute__SWIG_2', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ComposeImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ComposeImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__ComposeImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ComposeImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('ComposeImageFilter_Execute__SWIG_2'))

# Start of ComposeImageFilter_Execute

`ComposeImageFilter_Execute__SWIG_3` = function(self, image1, image2, image3, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_ComposeImageFilter_Execute__SWIG_3', self, image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ComposeImageFilter_Execute__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ComposeImageFilter_Execute__SWIG_3`, "inputTypes") = c('_p_itk__simple__ComposeImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ComposeImageFilter_Execute__SWIG_3`) = c("SWIGFunction", class('ComposeImageFilter_Execute__SWIG_3'))

# Start of ComposeImageFilter_Execute

`ComposeImageFilter_Execute__SWIG_4` = function(self, image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_ComposeImageFilter_Execute__SWIG_4', self, image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ComposeImageFilter_Execute__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ComposeImageFilter_Execute__SWIG_4`, "inputTypes") = c('_p_itk__simple__ComposeImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ComposeImageFilter_Execute__SWIG_4`) = c("SWIGFunction", class('ComposeImageFilter_Execute__SWIG_4'))

# Start of ComposeImageFilter_Execute

`ComposeImageFilter_Execute__SWIG_5` = function(self, image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_ComposeImageFilter_Execute__SWIG_5', self, image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ComposeImageFilter_Execute__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`ComposeImageFilter_Execute__SWIG_5`, "inputTypes") = c('_p_itk__simple__ComposeImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ComposeImageFilter_Execute__SWIG_5`) = c("SWIGFunction", class('ComposeImageFilter_Execute__SWIG_5'))

`ComposeImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ComposeImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- ComposeImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__ComposeImageFilter') || is.null(argv[[1]]) ) && (
        (is.list(argv[[2]]) & all(sapply(argv[[2]], inherits, what='_p_itk__simple__Image')))
        )) {
      f <- ComposeImageFilter_Execute__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ComposeImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- ComposeImageFilter_Execute__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__ComposeImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- ComposeImageFilter_Execute__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__ComposeImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- ComposeImageFilter_Execute__SWIG_4; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__ComposeImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( extends(argtypes[6], '_p_itk__simple__Image') && length(argv[[6]]) == 1 )) {
      f <- ComposeImageFilter_Execute__SWIG_5; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ComposeImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::ComposeImageFilter
setMethod('$', '_p_itk__simple__ComposeImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = ComposeImageFilter_GetName, 'ToString' = ComposeImageFilter_ToString, 'Execute' = ComposeImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ComposeImageFilter
# Start of Compose

`Compose__SWIG_0` = function(images, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_Compose__SWIG_0', images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Compose__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Compose__SWIG_0`, "inputTypes") = c('list')
class(`Compose__SWIG_0`) = c("SWIGFunction", class('Compose__SWIG_0'))

# Start of Compose

`Compose__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Compose__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Compose__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Compose__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`Compose__SWIG_1`) = c("SWIGFunction", class('Compose__SWIG_1'))

# Start of Compose

`Compose__SWIG_2` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Compose__SWIG_2', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Compose__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Compose__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Compose__SWIG_2`) = c("SWIGFunction", class('Compose__SWIG_2'))

# Start of Compose

`Compose__SWIG_3` = function(image1, image2, image3, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_Compose__SWIG_3', image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Compose__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Compose__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Compose__SWIG_3`) = c("SWIGFunction", class('Compose__SWIG_3'))

# Start of Compose

`Compose__SWIG_4` = function(image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_Compose__SWIG_4', image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Compose__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`Compose__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Compose__SWIG_4`) = c("SWIGFunction", class('Compose__SWIG_4'))

# Start of Compose

`Compose__SWIG_5` = function(image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_Compose__SWIG_5', image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Compose__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`Compose__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Compose__SWIG_5`) = c("SWIGFunction", class('Compose__SWIG_5'))

`Compose` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Compose__SWIG_1; 
    }
    else if (
      (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
      ) {
      f <- Compose__SWIG_0; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Compose__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- Compose__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- Compose__SWIG_4; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- Compose__SWIG_5; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Compose with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ConfidenceConnectedImageFilter

`delete_ConfidenceConnectedImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ConfidenceConnectedImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ConfidenceConnectedImageFilter`, 'returnType') = 'void'
attr(`delete_ConfidenceConnectedImageFilter`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter')
class(`delete_ConfidenceConnectedImageFilter`) = c("SWIGFunction", class('delete_ConfidenceConnectedImageFilter'))

# Start of new_ConfidenceConnectedImageFilter

`ConfidenceConnectedImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ConfidenceConnectedImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ConfidenceConnectedImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ConfidenceConnectedImageFilter);
  ans
  
}

attr(`ConfidenceConnectedImageFilter`, 'returnType') = '_p_itk__simple__ConfidenceConnectedImageFilter'
class(`ConfidenceConnectedImageFilter`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter'))

# Start of ConfidenceConnectedImageFilter_SetSeedList

`ConfidenceConnectedImageFilter_SetSeedList` = function(self, SeedList)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SeedList = lapply(SeedList, as.integer);
  ;ans = .Call('R_swig_ConfidenceConnectedImageFilter_SetSeedList', self, SeedList, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConfidenceConnectedImageFilter_SetSeedList`, 'returnType') = 'void'
attr(`ConfidenceConnectedImageFilter_SetSeedList`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter', 'list')
class(`ConfidenceConnectedImageFilter_SetSeedList`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_SetSeedList'))

# Start of ConfidenceConnectedImageFilter_GetSeedList

`ConfidenceConnectedImageFilter_GetSeedList` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConfidenceConnectedImageFilter_GetSeedList', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConfidenceConnectedImageFilter_GetSeedList`, 'returnType') = 'list'
attr(`ConfidenceConnectedImageFilter_GetSeedList`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter')
class(`ConfidenceConnectedImageFilter_GetSeedList`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_GetSeedList'))

# Start of ConfidenceConnectedImageFilter_AddSeed

`ConfidenceConnectedImageFilter_AddSeed` = function(self, point)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  point = as.integer(point);
  ;ans = .Call('R_swig_ConfidenceConnectedImageFilter_AddSeed', self, point, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConfidenceConnectedImageFilter_AddSeed`, 'returnType') = 'void'
attr(`ConfidenceConnectedImageFilter_AddSeed`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter', 'integer')
class(`ConfidenceConnectedImageFilter_AddSeed`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_AddSeed'))

# Start of ConfidenceConnectedImageFilter_ClearSeeds

`ConfidenceConnectedImageFilter_ClearSeeds` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ConfidenceConnectedImageFilter_ClearSeeds', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConfidenceConnectedImageFilter_ClearSeeds`, 'returnType') = 'void'
attr(`ConfidenceConnectedImageFilter_ClearSeeds`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter')
class(`ConfidenceConnectedImageFilter_ClearSeeds`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_ClearSeeds'))

# Start of ConfidenceConnectedImageFilter_SetNumberOfIterations

`ConfidenceConnectedImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_ConfidenceConnectedImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConfidenceConnectedImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`ConfidenceConnectedImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter', 'integer')
class(`ConfidenceConnectedImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_SetNumberOfIterations'))

# Start of ConfidenceConnectedImageFilter_GetNumberOfIterations

`ConfidenceConnectedImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConfidenceConnectedImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConfidenceConnectedImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`ConfidenceConnectedImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter')
class(`ConfidenceConnectedImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_GetNumberOfIterations'))

# Start of ConfidenceConnectedImageFilter_SetMultiplier

`ConfidenceConnectedImageFilter_SetMultiplier` = function(self, Multiplier)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ConfidenceConnectedImageFilter_SetMultiplier', self, Multiplier, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConfidenceConnectedImageFilter_SetMultiplier`, 'returnType') = 'void'
attr(`ConfidenceConnectedImageFilter_SetMultiplier`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter', 'numeric')
class(`ConfidenceConnectedImageFilter_SetMultiplier`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_SetMultiplier'))

# Start of ConfidenceConnectedImageFilter_GetMultiplier

`ConfidenceConnectedImageFilter_GetMultiplier` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConfidenceConnectedImageFilter_GetMultiplier', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConfidenceConnectedImageFilter_GetMultiplier`, 'returnType') = 'numeric'
attr(`ConfidenceConnectedImageFilter_GetMultiplier`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter')
class(`ConfidenceConnectedImageFilter_GetMultiplier`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_GetMultiplier'))

# Start of ConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius

`ConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius` = function(self, InitialNeighborhoodRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InitialNeighborhoodRadius = as.integer(InitialNeighborhoodRadius);
  
  if(length(InitialNeighborhoodRadius) > 1) {
    warning("using only the first element of InitialNeighborhoodRadius");
  };
  
  ;ans = .Call('R_swig_ConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius', self, InitialNeighborhoodRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius`, 'returnType') = 'void'
attr(`ConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter', 'integer')
class(`ConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius'))

# Start of ConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius

`ConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius`, 'returnType') = 'integer'
attr(`ConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter')
class(`ConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius'))

# Start of ConfidenceConnectedImageFilter_SetReplaceValue

`ConfidenceConnectedImageFilter_SetReplaceValue` = function(self, ReplaceValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ReplaceValue = as.integer(ReplaceValue);
  
  if(length(ReplaceValue) > 1) {
    warning("using only the first element of ReplaceValue");
  };
  
  ;ans = .Call('R_swig_ConfidenceConnectedImageFilter_SetReplaceValue', self, ReplaceValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConfidenceConnectedImageFilter_SetReplaceValue`, 'returnType') = 'void'
attr(`ConfidenceConnectedImageFilter_SetReplaceValue`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter', 'integer')
class(`ConfidenceConnectedImageFilter_SetReplaceValue`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_SetReplaceValue'))

# Start of ConfidenceConnectedImageFilter_GetReplaceValue

`ConfidenceConnectedImageFilter_GetReplaceValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConfidenceConnectedImageFilter_GetReplaceValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConfidenceConnectedImageFilter_GetReplaceValue`, 'returnType') = 'integer'
attr(`ConfidenceConnectedImageFilter_GetReplaceValue`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter')
class(`ConfidenceConnectedImageFilter_GetReplaceValue`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_GetReplaceValue'))

# Start of ConfidenceConnectedImageFilter_GetMean

`ConfidenceConnectedImageFilter_GetMean` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConfidenceConnectedImageFilter_GetMean', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConfidenceConnectedImageFilter_GetMean`, 'returnType') = 'numeric'
attr(`ConfidenceConnectedImageFilter_GetMean`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter')
class(`ConfidenceConnectedImageFilter_GetMean`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_GetMean'))

# Start of ConfidenceConnectedImageFilter_GetVariance

`ConfidenceConnectedImageFilter_GetVariance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConfidenceConnectedImageFilter_GetVariance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConfidenceConnectedImageFilter_GetVariance`, 'returnType') = 'numeric'
attr(`ConfidenceConnectedImageFilter_GetVariance`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter')
class(`ConfidenceConnectedImageFilter_GetVariance`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_GetVariance'))

# Start of ConfidenceConnectedImageFilter_GetName

`ConfidenceConnectedImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConfidenceConnectedImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConfidenceConnectedImageFilter_GetName`, 'returnType') = 'character'
attr(`ConfidenceConnectedImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter')
class(`ConfidenceConnectedImageFilter_GetName`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_GetName'))

# Start of ConfidenceConnectedImageFilter_ToString

`ConfidenceConnectedImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConfidenceConnectedImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConfidenceConnectedImageFilter_ToString`, 'returnType') = 'character'
attr(`ConfidenceConnectedImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter')
class(`ConfidenceConnectedImageFilter_ToString`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_ToString'))

# Start of ConfidenceConnectedImageFilter_Execute

`ConfidenceConnectedImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ConfidenceConnectedImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConfidenceConnectedImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ConfidenceConnectedImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ConfidenceConnectedImageFilter', '_p_itk__simple__Image')
class(`ConfidenceConnectedImageFilter_Execute`) = c("SWIGFunction", class('ConfidenceConnectedImageFilter_Execute'))

# Start of accessor method for itk::simple::ConfidenceConnectedImageFilter
setMethod('$', '_p_itk__simple__ConfidenceConnectedImageFilter', function(x, name)

{
  accessorFuns = list('SetSeedList' = ConfidenceConnectedImageFilter_SetSeedList, 'GetSeedList' = ConfidenceConnectedImageFilter_GetSeedList, 'AddSeed' = ConfidenceConnectedImageFilter_AddSeed, 'ClearSeeds' = ConfidenceConnectedImageFilter_ClearSeeds, 'SetNumberOfIterations' = ConfidenceConnectedImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = ConfidenceConnectedImageFilter_GetNumberOfIterations, 'SetMultiplier' = ConfidenceConnectedImageFilter_SetMultiplier, 'GetMultiplier' = ConfidenceConnectedImageFilter_GetMultiplier, 'SetInitialNeighborhoodRadius' = ConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius, 'GetInitialNeighborhoodRadius' = ConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius, 'SetReplaceValue' = ConfidenceConnectedImageFilter_SetReplaceValue, 'GetReplaceValue' = ConfidenceConnectedImageFilter_GetReplaceValue, 'GetMean' = ConfidenceConnectedImageFilter_GetMean, 'GetVariance' = ConfidenceConnectedImageFilter_GetVariance, 'GetName' = ConfidenceConnectedImageFilter_GetName, 'ToString' = ConfidenceConnectedImageFilter_ToString, 'Execute' = ConfidenceConnectedImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ConfidenceConnectedImageFilter
# Start of ConfidenceConnected

`ConfidenceConnected__SWIG_0` = function(image1, seedList, numberOfIterations, multiplier, initialNeighborhoodRadius, replaceValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  initialNeighborhoodRadius = as.integer(initialNeighborhoodRadius);
  
  if(length(initialNeighborhoodRadius) > 1) {
    warning("using only the first element of initialNeighborhoodRadius");
  };
  
  replaceValue = as.integer(replaceValue);
  
  if(length(replaceValue) > 1) {
    warning("using only the first element of replaceValue");
  };
  
  ;ans = .Call('R_swig_ConfidenceConnected__SWIG_0', image1, seedList, numberOfIterations, multiplier, initialNeighborhoodRadius, replaceValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConfidenceConnected__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ConfidenceConnected__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'integer', 'numeric', 'integer', 'integer')
class(`ConfidenceConnected__SWIG_0`) = c("SWIGFunction", class('ConfidenceConnected__SWIG_0'))

# Start of ConfidenceConnected

`ConfidenceConnected__SWIG_1` = function(image1, seedList, numberOfIterations, multiplier, initialNeighborhoodRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  initialNeighborhoodRadius = as.integer(initialNeighborhoodRadius);
  
  if(length(initialNeighborhoodRadius) > 1) {
    warning("using only the first element of initialNeighborhoodRadius");
  };
  
  ;ans = .Call('R_swig_ConfidenceConnected__SWIG_1', image1, seedList, numberOfIterations, multiplier, initialNeighborhoodRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConfidenceConnected__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ConfidenceConnected__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'integer', 'numeric', 'integer')
class(`ConfidenceConnected__SWIG_1`) = c("SWIGFunction", class('ConfidenceConnected__SWIG_1'))

# Start of ConfidenceConnected

`ConfidenceConnected__SWIG_2` = function(image1, seedList, numberOfIterations, multiplier, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  ;ans = .Call('R_swig_ConfidenceConnected__SWIG_2', image1, seedList, numberOfIterations, multiplier, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConfidenceConnected__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ConfidenceConnected__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'integer', 'numeric')
class(`ConfidenceConnected__SWIG_2`) = c("SWIGFunction", class('ConfidenceConnected__SWIG_2'))

# Start of ConfidenceConnected

`ConfidenceConnected__SWIG_3` = function(image1, seedList, numberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_ConfidenceConnected__SWIG_3', image1, seedList, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConfidenceConnected__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ConfidenceConnected__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'integer')
class(`ConfidenceConnected__SWIG_3`) = c("SWIGFunction", class('ConfidenceConnected__SWIG_3'))

# Start of ConfidenceConnected

`ConfidenceConnected__SWIG_4` = function(image1, seedList, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  ;ans = .Call('R_swig_ConfidenceConnected__SWIG_4', image1, seedList, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConfidenceConnected__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ConfidenceConnected__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'list')
class(`ConfidenceConnected__SWIG_4`) = c("SWIGFunction", class('ConfidenceConnected__SWIG_4'))

# Start of ConfidenceConnected

`ConfidenceConnected__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ConfidenceConnected__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConfidenceConnected__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`ConfidenceConnected__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`ConfidenceConnected__SWIG_5`) = c("SWIGFunction", class('ConfidenceConnected__SWIG_5'))

`ConfidenceConnected` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ConfidenceConnected__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) )) {
      f <- ConfidenceConnected__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ConfidenceConnected__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ConfidenceConnected__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- ConfidenceConnected__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- ConfidenceConnected__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ConfidenceConnected with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ConnectedComponentImageFilter

`delete_ConnectedComponentImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ConnectedComponentImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ConnectedComponentImageFilter`, 'returnType') = 'void'
attr(`delete_ConnectedComponentImageFilter`, "inputTypes") = c('_p_itk__simple__ConnectedComponentImageFilter')
class(`delete_ConnectedComponentImageFilter`) = c("SWIGFunction", class('delete_ConnectedComponentImageFilter'))

# Start of new_ConnectedComponentImageFilter

`ConnectedComponentImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ConnectedComponentImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ConnectedComponentImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ConnectedComponentImageFilter);
  ans
  
}

attr(`ConnectedComponentImageFilter`, 'returnType') = '_p_itk__simple__ConnectedComponentImageFilter'
class(`ConnectedComponentImageFilter`) = c("SWIGFunction", class('ConnectedComponentImageFilter'))

# Start of ConnectedComponentImageFilter_SetFullyConnected

`ConnectedComponentImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_ConnectedComponentImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConnectedComponentImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`ConnectedComponentImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__ConnectedComponentImageFilter', 'logical')
class(`ConnectedComponentImageFilter_SetFullyConnected`) = c("SWIGFunction", class('ConnectedComponentImageFilter_SetFullyConnected'))

# Start of ConnectedComponentImageFilter_FullyConnectedOn

`ConnectedComponentImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ConnectedComponentImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConnectedComponentImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`ConnectedComponentImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__ConnectedComponentImageFilter')
class(`ConnectedComponentImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('ConnectedComponentImageFilter_FullyConnectedOn'))

# Start of ConnectedComponentImageFilter_FullyConnectedOff

`ConnectedComponentImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ConnectedComponentImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConnectedComponentImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`ConnectedComponentImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__ConnectedComponentImageFilter')
class(`ConnectedComponentImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('ConnectedComponentImageFilter_FullyConnectedOff'))

# Start of ConnectedComponentImageFilter_GetFullyConnected

`ConnectedComponentImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConnectedComponentImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConnectedComponentImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`ConnectedComponentImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__ConnectedComponentImageFilter')
class(`ConnectedComponentImageFilter_GetFullyConnected`) = c("SWIGFunction", class('ConnectedComponentImageFilter_GetFullyConnected'))

# Start of ConnectedComponentImageFilter_GetObjectCount

`ConnectedComponentImageFilter_GetObjectCount` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConnectedComponentImageFilter_GetObjectCount', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConnectedComponentImageFilter_GetObjectCount`, 'returnType') = 'integer'
attr(`ConnectedComponentImageFilter_GetObjectCount`, "inputTypes") = c('_p_itk__simple__ConnectedComponentImageFilter')
class(`ConnectedComponentImageFilter_GetObjectCount`) = c("SWIGFunction", class('ConnectedComponentImageFilter_GetObjectCount'))

# Start of ConnectedComponentImageFilter_GetName

`ConnectedComponentImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConnectedComponentImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConnectedComponentImageFilter_GetName`, 'returnType') = 'character'
attr(`ConnectedComponentImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ConnectedComponentImageFilter')
class(`ConnectedComponentImageFilter_GetName`) = c("SWIGFunction", class('ConnectedComponentImageFilter_GetName'))

# Start of ConnectedComponentImageFilter_ToString

`ConnectedComponentImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConnectedComponentImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConnectedComponentImageFilter_ToString`, 'returnType') = 'character'
attr(`ConnectedComponentImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ConnectedComponentImageFilter')
class(`ConnectedComponentImageFilter_ToString`) = c("SWIGFunction", class('ConnectedComponentImageFilter_ToString'))

# Start of ConnectedComponentImageFilter_Execute

`ConnectedComponentImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_ConnectedComponentImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConnectedComponentImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ConnectedComponentImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__ConnectedComponentImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ConnectedComponentImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('ConnectedComponentImageFilter_Execute__SWIG_0'))

# Start of ConnectedComponentImageFilter_Execute

`ConnectedComponentImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_ConnectedComponentImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConnectedComponentImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ConnectedComponentImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__ConnectedComponentImageFilter', '_p_itk__simple__Image')
class(`ConnectedComponentImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('ConnectedComponentImageFilter_Execute__SWIG_1'))

`ConnectedComponentImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ConnectedComponentImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- ConnectedComponentImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ConnectedComponentImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- ConnectedComponentImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ConnectedComponentImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::ConnectedComponentImageFilter
setMethod('$', '_p_itk__simple__ConnectedComponentImageFilter', function(x, name)

{
  accessorFuns = list('SetFullyConnected' = ConnectedComponentImageFilter_SetFullyConnected, 'FullyConnectedOn' = ConnectedComponentImageFilter_FullyConnectedOn, 'FullyConnectedOff' = ConnectedComponentImageFilter_FullyConnectedOff, 'GetFullyConnected' = ConnectedComponentImageFilter_GetFullyConnected, 'GetObjectCount' = ConnectedComponentImageFilter_GetObjectCount, 'GetName' = ConnectedComponentImageFilter_GetName, 'ToString' = ConnectedComponentImageFilter_ToString, 'Execute' = ConnectedComponentImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ConnectedComponentImageFilter
# Start of ConnectedComponent

`ConnectedComponent__SWIG_0` = function(image, maskImage, fullyConnected, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_ConnectedComponent__SWIG_0', image, maskImage, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConnectedComponent__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ConnectedComponent__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical')
class(`ConnectedComponent__SWIG_0`) = c("SWIGFunction", class('ConnectedComponent__SWIG_0'))

# Start of ConnectedComponent

`ConnectedComponent__SWIG_1` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_ConnectedComponent__SWIG_1', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConnectedComponent__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ConnectedComponent__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ConnectedComponent__SWIG_1`) = c("SWIGFunction", class('ConnectedComponent__SWIG_1'))

# Start of ConnectedComponent

`ConnectedComponent__SWIG_2` = function(image, fullyConnected, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_ConnectedComponent__SWIG_2', image, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConnectedComponent__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ConnectedComponent__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`ConnectedComponent__SWIG_2`) = c("SWIGFunction", class('ConnectedComponent__SWIG_2'))

# Start of ConnectedComponent

`ConnectedComponent__SWIG_3` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_ConnectedComponent__SWIG_3', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConnectedComponent__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ConnectedComponent__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`ConnectedComponent__SWIG_3`) = c("SWIGFunction", class('ConnectedComponent__SWIG_3'))

`ConnectedComponent` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ConnectedComponent__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- ConnectedComponent__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ConnectedComponent__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ConnectedComponent__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ConnectedComponent with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ConnectedThresholdImageFilter

`delete_ConnectedThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ConnectedThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ConnectedThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_ConnectedThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter')
class(`delete_ConnectedThresholdImageFilter`) = c("SWIGFunction", class('delete_ConnectedThresholdImageFilter'))

# Start of new_ConnectedThresholdImageFilter

`ConnectedThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ConnectedThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ConnectedThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ConnectedThresholdImageFilter);
  ans
  
}

attr(`ConnectedThresholdImageFilter`, 'returnType') = '_p_itk__simple__ConnectedThresholdImageFilter'
class(`ConnectedThresholdImageFilter`) = c("SWIGFunction", class('ConnectedThresholdImageFilter'))

# Start of ConnectedThresholdImageFilter_SetSeedList

`ConnectedThresholdImageFilter_SetSeedList` = function(self, SeedList)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SeedList = lapply(SeedList, as.integer);
  ;ans = .Call('R_swig_ConnectedThresholdImageFilter_SetSeedList', self, SeedList, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConnectedThresholdImageFilter_SetSeedList`, 'returnType') = 'void'
attr(`ConnectedThresholdImageFilter_SetSeedList`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter', 'list')
class(`ConnectedThresholdImageFilter_SetSeedList`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_SetSeedList'))

# Start of ConnectedThresholdImageFilter_GetSeedList

`ConnectedThresholdImageFilter_GetSeedList` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConnectedThresholdImageFilter_GetSeedList', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConnectedThresholdImageFilter_GetSeedList`, 'returnType') = 'list'
attr(`ConnectedThresholdImageFilter_GetSeedList`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter')
class(`ConnectedThresholdImageFilter_GetSeedList`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_GetSeedList'))

# Start of ConnectedThresholdImageFilter_AddSeed

`ConnectedThresholdImageFilter_AddSeed` = function(self, point)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  point = as.integer(point);
  ;ans = .Call('R_swig_ConnectedThresholdImageFilter_AddSeed', self, point, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConnectedThresholdImageFilter_AddSeed`, 'returnType') = 'void'
attr(`ConnectedThresholdImageFilter_AddSeed`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter', 'integer')
class(`ConnectedThresholdImageFilter_AddSeed`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_AddSeed'))

# Start of ConnectedThresholdImageFilter_ClearSeeds

`ConnectedThresholdImageFilter_ClearSeeds` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ConnectedThresholdImageFilter_ClearSeeds', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConnectedThresholdImageFilter_ClearSeeds`, 'returnType') = 'void'
attr(`ConnectedThresholdImageFilter_ClearSeeds`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter')
class(`ConnectedThresholdImageFilter_ClearSeeds`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_ClearSeeds'))

# Start of ConnectedThresholdImageFilter_SetLower

`ConnectedThresholdImageFilter_SetLower` = function(self, Lower)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ConnectedThresholdImageFilter_SetLower', self, Lower, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConnectedThresholdImageFilter_SetLower`, 'returnType') = 'void'
attr(`ConnectedThresholdImageFilter_SetLower`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter', 'numeric')
class(`ConnectedThresholdImageFilter_SetLower`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_SetLower'))

# Start of ConnectedThresholdImageFilter_GetLower

`ConnectedThresholdImageFilter_GetLower` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConnectedThresholdImageFilter_GetLower', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConnectedThresholdImageFilter_GetLower`, 'returnType') = 'numeric'
attr(`ConnectedThresholdImageFilter_GetLower`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter')
class(`ConnectedThresholdImageFilter_GetLower`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_GetLower'))

# Start of ConnectedThresholdImageFilter_SetUpper

`ConnectedThresholdImageFilter_SetUpper` = function(self, Upper)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ConnectedThresholdImageFilter_SetUpper', self, Upper, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConnectedThresholdImageFilter_SetUpper`, 'returnType') = 'void'
attr(`ConnectedThresholdImageFilter_SetUpper`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter', 'numeric')
class(`ConnectedThresholdImageFilter_SetUpper`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_SetUpper'))

# Start of ConnectedThresholdImageFilter_GetUpper

`ConnectedThresholdImageFilter_GetUpper` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConnectedThresholdImageFilter_GetUpper', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConnectedThresholdImageFilter_GetUpper`, 'returnType') = 'numeric'
attr(`ConnectedThresholdImageFilter_GetUpper`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter')
class(`ConnectedThresholdImageFilter_GetUpper`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_GetUpper'))

# Start of ConnectedThresholdImageFilter_SetReplaceValue

`ConnectedThresholdImageFilter_SetReplaceValue` = function(self, ReplaceValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ReplaceValue = as.integer(ReplaceValue);
  
  if(length(ReplaceValue) > 1) {
    warning("using only the first element of ReplaceValue");
  };
  
  ;ans = .Call('R_swig_ConnectedThresholdImageFilter_SetReplaceValue', self, ReplaceValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConnectedThresholdImageFilter_SetReplaceValue`, 'returnType') = 'void'
attr(`ConnectedThresholdImageFilter_SetReplaceValue`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter', 'integer')
class(`ConnectedThresholdImageFilter_SetReplaceValue`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_SetReplaceValue'))

# Start of ConnectedThresholdImageFilter_GetReplaceValue

`ConnectedThresholdImageFilter_GetReplaceValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConnectedThresholdImageFilter_GetReplaceValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConnectedThresholdImageFilter_GetReplaceValue`, 'returnType') = 'integer'
attr(`ConnectedThresholdImageFilter_GetReplaceValue`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter')
class(`ConnectedThresholdImageFilter_GetReplaceValue`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_GetReplaceValue'))

# Start of ConnectedThresholdImageFilter_FaceConnectivity_get

`ConnectedThresholdImageFilter_FaceConnectivity_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ConnectedThresholdImageFilter_FaceConnectivity_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ConnectedThresholdImageFilter__ConnectivityType");
  
  ans
  
}

attr(`ConnectedThresholdImageFilter_FaceConnectivity_get`, 'returnType') = 'character'
class(`ConnectedThresholdImageFilter_FaceConnectivity_get`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_FaceConnectivity_get'))

# Start of ConnectedThresholdImageFilter_FullConnectivity_get

`ConnectedThresholdImageFilter_FullConnectivity_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ConnectedThresholdImageFilter_FullConnectivity_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ConnectedThresholdImageFilter__ConnectivityType");
  
  ans
  
}

attr(`ConnectedThresholdImageFilter_FullConnectivity_get`, 'returnType') = 'character'
class(`ConnectedThresholdImageFilter_FullConnectivity_get`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_FullConnectivity_get'))

# Start of ConnectedThresholdImageFilter_SetConnectivity

`ConnectedThresholdImageFilter_SetConnectivity` = function(self, Connectivity)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Connectivity = enumToInteger(Connectivity, "_itk__simple__ConnectedThresholdImageFilter__ConnectivityType"); 
  
  if(length(Connectivity) > 1) {
    warning("using only the first element of Connectivity");
  };
  
  ;ans = .Call('R_swig_ConnectedThresholdImageFilter_SetConnectivity', self, Connectivity, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConnectedThresholdImageFilter_SetConnectivity`, 'returnType') = 'void'
attr(`ConnectedThresholdImageFilter_SetConnectivity`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter', 'character')
class(`ConnectedThresholdImageFilter_SetConnectivity`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_SetConnectivity'))

# Start of ConnectedThresholdImageFilter_GetConnectivity

`ConnectedThresholdImageFilter_GetConnectivity` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ConnectedThresholdImageFilter_GetConnectivity', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ConnectedThresholdImageFilter__ConnectivityType");
  
  ans
  
}

attr(`ConnectedThresholdImageFilter_GetConnectivity`, 'returnType') = 'character'
attr(`ConnectedThresholdImageFilter_GetConnectivity`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter')
class(`ConnectedThresholdImageFilter_GetConnectivity`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_GetConnectivity'))

# Start of ConnectedThresholdImageFilter_GetName

`ConnectedThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConnectedThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConnectedThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`ConnectedThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter')
class(`ConnectedThresholdImageFilter_GetName`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_GetName'))

# Start of ConnectedThresholdImageFilter_ToString

`ConnectedThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConnectedThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConnectedThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`ConnectedThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter')
class(`ConnectedThresholdImageFilter_ToString`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_ToString'))

# Start of ConnectedThresholdImageFilter_Execute

`ConnectedThresholdImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ConnectedThresholdImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConnectedThresholdImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ConnectedThresholdImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ConnectedThresholdImageFilter', '_p_itk__simple__Image')
class(`ConnectedThresholdImageFilter_Execute`) = c("SWIGFunction", class('ConnectedThresholdImageFilter_Execute'))

# Start of accessor method for itk::simple::ConnectedThresholdImageFilter
setMethod('$', '_p_itk__simple__ConnectedThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetSeedList' = ConnectedThresholdImageFilter_SetSeedList, 'GetSeedList' = ConnectedThresholdImageFilter_GetSeedList, 'AddSeed' = ConnectedThresholdImageFilter_AddSeed, 'ClearSeeds' = ConnectedThresholdImageFilter_ClearSeeds, 'SetLower' = ConnectedThresholdImageFilter_SetLower, 'GetLower' = ConnectedThresholdImageFilter_GetLower, 'SetUpper' = ConnectedThresholdImageFilter_SetUpper, 'GetUpper' = ConnectedThresholdImageFilter_GetUpper, 'SetReplaceValue' = ConnectedThresholdImageFilter_SetReplaceValue, 'GetReplaceValue' = ConnectedThresholdImageFilter_GetReplaceValue, 'SetConnectivity' = ConnectedThresholdImageFilter_SetConnectivity, 'GetConnectivity' = ConnectedThresholdImageFilter_GetConnectivity, 'GetName' = ConnectedThresholdImageFilter_GetName, 'ToString' = ConnectedThresholdImageFilter_ToString, 'Execute' = ConnectedThresholdImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ConnectedThresholdImageFilter
# Start of ConnectedThreshold

`ConnectedThreshold__SWIG_0` = function(image1, seedList, lower, upper, replaceValue, connectivity, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  
  
  replaceValue = as.integer(replaceValue);
  
  if(length(replaceValue) > 1) {
    warning("using only the first element of replaceValue");
  };
  
  connectivity = enumToInteger(connectivity, "_itk__simple__ConnectedThresholdImageFilter__ConnectivityType"); 
  
  if(length(connectivity) > 1) {
    warning("using only the first element of connectivity");
  };
  
  ;ans = .Call('R_swig_ConnectedThreshold__SWIG_0', image1, seedList, lower, upper, replaceValue, connectivity, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConnectedThreshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ConnectedThreshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'numeric', 'numeric', 'integer', 'character')
class(`ConnectedThreshold__SWIG_0`) = c("SWIGFunction", class('ConnectedThreshold__SWIG_0'))

# Start of ConnectedThreshold

`ConnectedThreshold__SWIG_1` = function(image1, seedList, lower, upper, replaceValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  
  
  replaceValue = as.integer(replaceValue);
  
  if(length(replaceValue) > 1) {
    warning("using only the first element of replaceValue");
  };
  
  ;ans = .Call('R_swig_ConnectedThreshold__SWIG_1', image1, seedList, lower, upper, replaceValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConnectedThreshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ConnectedThreshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'numeric', 'numeric', 'integer')
class(`ConnectedThreshold__SWIG_1`) = c("SWIGFunction", class('ConnectedThreshold__SWIG_1'))

# Start of ConnectedThreshold

`ConnectedThreshold__SWIG_2` = function(image1, seedList, lower, upper, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  
  
  ;ans = .Call('R_swig_ConnectedThreshold__SWIG_2', image1, seedList, lower, upper, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConnectedThreshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ConnectedThreshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'numeric', 'numeric')
class(`ConnectedThreshold__SWIG_2`) = c("SWIGFunction", class('ConnectedThreshold__SWIG_2'))

# Start of ConnectedThreshold

`ConnectedThreshold__SWIG_3` = function(image1, seedList, lower, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  
  ;ans = .Call('R_swig_ConnectedThreshold__SWIG_3', image1, seedList, lower, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConnectedThreshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ConnectedThreshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'numeric')
class(`ConnectedThreshold__SWIG_3`) = c("SWIGFunction", class('ConnectedThreshold__SWIG_3'))

# Start of ConnectedThreshold

`ConnectedThreshold__SWIG_4` = function(image1, seedList, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  ;ans = .Call('R_swig_ConnectedThreshold__SWIG_4', image1, seedList, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConnectedThreshold__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ConnectedThreshold__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'list')
class(`ConnectedThreshold__SWIG_4`) = c("SWIGFunction", class('ConnectedThreshold__SWIG_4'))

# Start of ConnectedThreshold

`ConnectedThreshold__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ConnectedThreshold__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConnectedThreshold__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`ConnectedThreshold__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`ConnectedThreshold__SWIG_5`) = c("SWIGFunction", class('ConnectedThreshold__SWIG_5'))

`ConnectedThreshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ConnectedThreshold__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) )) {
      f <- ConnectedThreshold__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ConnectedThreshold__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ConnectedThreshold__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- ConnectedThreshold__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.character(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- ConnectedThreshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ConnectedThreshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ConstantPadImageFilter

`delete_ConstantPadImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ConstantPadImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ConstantPadImageFilter`, 'returnType') = 'void'
attr(`delete_ConstantPadImageFilter`, "inputTypes") = c('_p_itk__simple__ConstantPadImageFilter')
class(`delete_ConstantPadImageFilter`) = c("SWIGFunction", class('delete_ConstantPadImageFilter'))

# Start of new_ConstantPadImageFilter

`ConstantPadImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ConstantPadImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ConstantPadImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ConstantPadImageFilter);
  ans
  
}

attr(`ConstantPadImageFilter`, 'returnType') = '_p_itk__simple__ConstantPadImageFilter'
class(`ConstantPadImageFilter`) = c("SWIGFunction", class('ConstantPadImageFilter'))

# Start of ConstantPadImageFilter_SetPadLowerBound

`ConstantPadImageFilter_SetPadLowerBound` = function(self, PadLowerBound)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  PadLowerBound = as.integer(PadLowerBound);
  ;ans = .Call('R_swig_ConstantPadImageFilter_SetPadLowerBound', self, PadLowerBound, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConstantPadImageFilter_SetPadLowerBound`, 'returnType') = 'void'
attr(`ConstantPadImageFilter_SetPadLowerBound`, "inputTypes") = c('_p_itk__simple__ConstantPadImageFilter', 'integer')
class(`ConstantPadImageFilter_SetPadLowerBound`) = c("SWIGFunction", class('ConstantPadImageFilter_SetPadLowerBound'))

# Start of ConstantPadImageFilter_GetPadLowerBound

`ConstantPadImageFilter_GetPadLowerBound` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConstantPadImageFilter_GetPadLowerBound', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConstantPadImageFilter_GetPadLowerBound`, 'returnType') = 'integer'
attr(`ConstantPadImageFilter_GetPadLowerBound`, "inputTypes") = c('_p_itk__simple__ConstantPadImageFilter')
class(`ConstantPadImageFilter_GetPadLowerBound`) = c("SWIGFunction", class('ConstantPadImageFilter_GetPadLowerBound'))

# Start of ConstantPadImageFilter_SetPadUpperBound

`ConstantPadImageFilter_SetPadUpperBound` = function(self, PadUpperBound)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  PadUpperBound = as.integer(PadUpperBound);
  ;ans = .Call('R_swig_ConstantPadImageFilter_SetPadUpperBound', self, PadUpperBound, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConstantPadImageFilter_SetPadUpperBound`, 'returnType') = 'void'
attr(`ConstantPadImageFilter_SetPadUpperBound`, "inputTypes") = c('_p_itk__simple__ConstantPadImageFilter', 'integer')
class(`ConstantPadImageFilter_SetPadUpperBound`) = c("SWIGFunction", class('ConstantPadImageFilter_SetPadUpperBound'))

# Start of ConstantPadImageFilter_GetPadUpperBound

`ConstantPadImageFilter_GetPadUpperBound` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConstantPadImageFilter_GetPadUpperBound', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConstantPadImageFilter_GetPadUpperBound`, 'returnType') = 'integer'
attr(`ConstantPadImageFilter_GetPadUpperBound`, "inputTypes") = c('_p_itk__simple__ConstantPadImageFilter')
class(`ConstantPadImageFilter_GetPadUpperBound`) = c("SWIGFunction", class('ConstantPadImageFilter_GetPadUpperBound'))

# Start of ConstantPadImageFilter_SetConstant

`ConstantPadImageFilter_SetConstant` = function(self, Constant)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ConstantPadImageFilter_SetConstant', self, Constant, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConstantPadImageFilter_SetConstant`, 'returnType') = 'void'
attr(`ConstantPadImageFilter_SetConstant`, "inputTypes") = c('_p_itk__simple__ConstantPadImageFilter', 'numeric')
class(`ConstantPadImageFilter_SetConstant`) = c("SWIGFunction", class('ConstantPadImageFilter_SetConstant'))

# Start of ConstantPadImageFilter_GetConstant

`ConstantPadImageFilter_GetConstant` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConstantPadImageFilter_GetConstant', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConstantPadImageFilter_GetConstant`, 'returnType') = 'numeric'
attr(`ConstantPadImageFilter_GetConstant`, "inputTypes") = c('_p_itk__simple__ConstantPadImageFilter')
class(`ConstantPadImageFilter_GetConstant`) = c("SWIGFunction", class('ConstantPadImageFilter_GetConstant'))

# Start of ConstantPadImageFilter_GetName

`ConstantPadImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConstantPadImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConstantPadImageFilter_GetName`, 'returnType') = 'character'
attr(`ConstantPadImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ConstantPadImageFilter')
class(`ConstantPadImageFilter_GetName`) = c("SWIGFunction", class('ConstantPadImageFilter_GetName'))

# Start of ConstantPadImageFilter_ToString

`ConstantPadImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConstantPadImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConstantPadImageFilter_ToString`, 'returnType') = 'character'
attr(`ConstantPadImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ConstantPadImageFilter')
class(`ConstantPadImageFilter_ToString`) = c("SWIGFunction", class('ConstantPadImageFilter_ToString'))

# Start of ConstantPadImageFilter_Execute

`ConstantPadImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ConstantPadImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConstantPadImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ConstantPadImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ConstantPadImageFilter', '_p_itk__simple__Image')
class(`ConstantPadImageFilter_Execute`) = c("SWIGFunction", class('ConstantPadImageFilter_Execute'))

# Start of accessor method for itk::simple::ConstantPadImageFilter
setMethod('$', '_p_itk__simple__ConstantPadImageFilter', function(x, name)

{
  accessorFuns = list('SetPadLowerBound' = ConstantPadImageFilter_SetPadLowerBound, 'GetPadLowerBound' = ConstantPadImageFilter_GetPadLowerBound, 'SetPadUpperBound' = ConstantPadImageFilter_SetPadUpperBound, 'GetPadUpperBound' = ConstantPadImageFilter_GetPadUpperBound, 'SetConstant' = ConstantPadImageFilter_SetConstant, 'GetConstant' = ConstantPadImageFilter_GetConstant, 'GetName' = ConstantPadImageFilter_GetName, 'ToString' = ConstantPadImageFilter_ToString, 'Execute' = ConstantPadImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ConstantPadImageFilter
# Start of ConstantPad

`ConstantPad__SWIG_0` = function(image1, padLowerBound, padUpperBound, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  padLowerBound = as.integer(padLowerBound);
  padUpperBound = as.integer(padUpperBound);
  
  ;ans = .Call('R_swig_ConstantPad__SWIG_0', image1, padLowerBound, padUpperBound, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConstantPad__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ConstantPad__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'numeric')
class(`ConstantPad__SWIG_0`) = c("SWIGFunction", class('ConstantPad__SWIG_0'))

# Start of ConstantPad

`ConstantPad__SWIG_1` = function(image1, padLowerBound, padUpperBound, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  padLowerBound = as.integer(padLowerBound);
  padUpperBound = as.integer(padUpperBound);
  ;ans = .Call('R_swig_ConstantPad__SWIG_1', image1, padLowerBound, padUpperBound, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConstantPad__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ConstantPad__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`ConstantPad__SWIG_1`) = c("SWIGFunction", class('ConstantPad__SWIG_1'))

# Start of ConstantPad

`ConstantPad__SWIG_2` = function(image1, padLowerBound, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  padLowerBound = as.integer(padLowerBound);
  ;ans = .Call('R_swig_ConstantPad__SWIG_2', image1, padLowerBound, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConstantPad__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ConstantPad__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`ConstantPad__SWIG_2`) = c("SWIGFunction", class('ConstantPad__SWIG_2'))

# Start of ConstantPad

`ConstantPad__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ConstantPad__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConstantPad__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ConstantPad__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`ConstantPad__SWIG_3`) = c("SWIGFunction", class('ConstantPad__SWIG_3'))

`ConstantPad` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ConstantPad__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- ConstantPad__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- ConstantPad__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ConstantPad__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ConstantPad with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ConvolutionImageFilter

`delete_ConvolutionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ConvolutionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ConvolutionImageFilter`, 'returnType') = 'void'
attr(`delete_ConvolutionImageFilter`, "inputTypes") = c('_p_itk__simple__ConvolutionImageFilter')
class(`delete_ConvolutionImageFilter`) = c("SWIGFunction", class('delete_ConvolutionImageFilter'))

# Start of new_ConvolutionImageFilter

`ConvolutionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ConvolutionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ConvolutionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ConvolutionImageFilter);
  ans
  
}

attr(`ConvolutionImageFilter`, 'returnType') = '_p_itk__simple__ConvolutionImageFilter'
class(`ConvolutionImageFilter`) = c("SWIGFunction", class('ConvolutionImageFilter'))

# Start of ConvolutionImageFilter_SetNormalize

`ConvolutionImageFilter_SetNormalize` = function(self, Normalize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Normalize = as.logical(Normalize);
  ;ans = .Call('R_swig_ConvolutionImageFilter_SetNormalize', self, Normalize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConvolutionImageFilter_SetNormalize`, 'returnType') = 'void'
attr(`ConvolutionImageFilter_SetNormalize`, "inputTypes") = c('_p_itk__simple__ConvolutionImageFilter', 'logical')
class(`ConvolutionImageFilter_SetNormalize`) = c("SWIGFunction", class('ConvolutionImageFilter_SetNormalize'))

# Start of ConvolutionImageFilter_NormalizeOn

`ConvolutionImageFilter_NormalizeOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ConvolutionImageFilter_NormalizeOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConvolutionImageFilter_NormalizeOn`, 'returnType') = 'void'
attr(`ConvolutionImageFilter_NormalizeOn`, "inputTypes") = c('_p_itk__simple__ConvolutionImageFilter')
class(`ConvolutionImageFilter_NormalizeOn`) = c("SWIGFunction", class('ConvolutionImageFilter_NormalizeOn'))

# Start of ConvolutionImageFilter_NormalizeOff

`ConvolutionImageFilter_NormalizeOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ConvolutionImageFilter_NormalizeOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConvolutionImageFilter_NormalizeOff`, 'returnType') = 'void'
attr(`ConvolutionImageFilter_NormalizeOff`, "inputTypes") = c('_p_itk__simple__ConvolutionImageFilter')
class(`ConvolutionImageFilter_NormalizeOff`) = c("SWIGFunction", class('ConvolutionImageFilter_NormalizeOff'))

# Start of ConvolutionImageFilter_GetNormalize

`ConvolutionImageFilter_GetNormalize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConvolutionImageFilter_GetNormalize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConvolutionImageFilter_GetNormalize`, 'returnType') = 'logical'
attr(`ConvolutionImageFilter_GetNormalize`, "inputTypes") = c('_p_itk__simple__ConvolutionImageFilter')
class(`ConvolutionImageFilter_GetNormalize`) = c("SWIGFunction", class('ConvolutionImageFilter_GetNormalize'))

# Start of ConvolutionImageFilter_ZERO_PAD_get

`ConvolutionImageFilter_ZERO_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ConvolutionImageFilter_ZERO_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ConvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`ConvolutionImageFilter_ZERO_PAD_get`, 'returnType') = 'character'
class(`ConvolutionImageFilter_ZERO_PAD_get`) = c("SWIGFunction", class('ConvolutionImageFilter_ZERO_PAD_get'))

# Start of ConvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get

`ConvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ConvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ConvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`ConvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`, 'returnType') = 'character'
class(`ConvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`) = c("SWIGFunction", class('ConvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get'))

# Start of ConvolutionImageFilter_PERIODIC_PAD_get

`ConvolutionImageFilter_PERIODIC_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ConvolutionImageFilter_PERIODIC_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ConvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`ConvolutionImageFilter_PERIODIC_PAD_get`, 'returnType') = 'character'
class(`ConvolutionImageFilter_PERIODIC_PAD_get`) = c("SWIGFunction", class('ConvolutionImageFilter_PERIODIC_PAD_get'))

# Start of ConvolutionImageFilter_SetBoundaryCondition

`ConvolutionImageFilter_SetBoundaryCondition` = function(self, BoundaryCondition)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BoundaryCondition = enumToInteger(BoundaryCondition, "_itk__simple__ConvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(BoundaryCondition) > 1) {
    warning("using only the first element of BoundaryCondition");
  };
  
  ;ans = .Call('R_swig_ConvolutionImageFilter_SetBoundaryCondition', self, BoundaryCondition, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConvolutionImageFilter_SetBoundaryCondition`, 'returnType') = 'void'
attr(`ConvolutionImageFilter_SetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__ConvolutionImageFilter', 'character')
class(`ConvolutionImageFilter_SetBoundaryCondition`) = c("SWIGFunction", class('ConvolutionImageFilter_SetBoundaryCondition'))

# Start of ConvolutionImageFilter_GetBoundaryCondition

`ConvolutionImageFilter_GetBoundaryCondition` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ConvolutionImageFilter_GetBoundaryCondition', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ConvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`ConvolutionImageFilter_GetBoundaryCondition`, 'returnType') = 'character'
attr(`ConvolutionImageFilter_GetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__ConvolutionImageFilter')
class(`ConvolutionImageFilter_GetBoundaryCondition`) = c("SWIGFunction", class('ConvolutionImageFilter_GetBoundaryCondition'))

# Start of ConvolutionImageFilter_SAME_get

`ConvolutionImageFilter_SAME_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ConvolutionImageFilter_SAME_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ConvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`ConvolutionImageFilter_SAME_get`, 'returnType') = 'character'
class(`ConvolutionImageFilter_SAME_get`) = c("SWIGFunction", class('ConvolutionImageFilter_SAME_get'))

# Start of ConvolutionImageFilter_VALID_get

`ConvolutionImageFilter_VALID_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ConvolutionImageFilter_VALID_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ConvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`ConvolutionImageFilter_VALID_get`, 'returnType') = 'character'
class(`ConvolutionImageFilter_VALID_get`) = c("SWIGFunction", class('ConvolutionImageFilter_VALID_get'))

# Start of ConvolutionImageFilter_SetOutputRegionMode

`ConvolutionImageFilter_SetOutputRegionMode` = function(self, OutputRegionMode)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputRegionMode = enumToInteger(OutputRegionMode, "_itk__simple__ConvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(OutputRegionMode) > 1) {
    warning("using only the first element of OutputRegionMode");
  };
  
  ;ans = .Call('R_swig_ConvolutionImageFilter_SetOutputRegionMode', self, OutputRegionMode, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ConvolutionImageFilter_SetOutputRegionMode`, 'returnType') = 'void'
attr(`ConvolutionImageFilter_SetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__ConvolutionImageFilter', 'character')
class(`ConvolutionImageFilter_SetOutputRegionMode`) = c("SWIGFunction", class('ConvolutionImageFilter_SetOutputRegionMode'))

# Start of ConvolutionImageFilter_GetOutputRegionMode

`ConvolutionImageFilter_GetOutputRegionMode` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ConvolutionImageFilter_GetOutputRegionMode', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ConvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`ConvolutionImageFilter_GetOutputRegionMode`, 'returnType') = 'character'
attr(`ConvolutionImageFilter_GetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__ConvolutionImageFilter')
class(`ConvolutionImageFilter_GetOutputRegionMode`) = c("SWIGFunction", class('ConvolutionImageFilter_GetOutputRegionMode'))

# Start of ConvolutionImageFilter_GetName

`ConvolutionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConvolutionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConvolutionImageFilter_GetName`, 'returnType') = 'character'
attr(`ConvolutionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ConvolutionImageFilter')
class(`ConvolutionImageFilter_GetName`) = c("SWIGFunction", class('ConvolutionImageFilter_GetName'))

# Start of ConvolutionImageFilter_ToString

`ConvolutionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ConvolutionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ConvolutionImageFilter_ToString`, 'returnType') = 'character'
attr(`ConvolutionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ConvolutionImageFilter')
class(`ConvolutionImageFilter_ToString`) = c("SWIGFunction", class('ConvolutionImageFilter_ToString'))

# Start of ConvolutionImageFilter_Execute

`ConvolutionImageFilter_Execute` = function(self, image, kernelImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(kernelImage, "ExternalReference")) kernelImage = slot(kernelImage,"ref"); 
  ;ans = .Call('R_swig_ConvolutionImageFilter_Execute', self, image, kernelImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ConvolutionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ConvolutionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ConvolutionImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ConvolutionImageFilter_Execute`) = c("SWIGFunction", class('ConvolutionImageFilter_Execute'))

# Start of accessor method for itk::simple::ConvolutionImageFilter
setMethod('$', '_p_itk__simple__ConvolutionImageFilter', function(x, name)

{
  accessorFuns = list('SetNormalize' = ConvolutionImageFilter_SetNormalize, 'NormalizeOn' = ConvolutionImageFilter_NormalizeOn, 'NormalizeOff' = ConvolutionImageFilter_NormalizeOff, 'GetNormalize' = ConvolutionImageFilter_GetNormalize, 'SetBoundaryCondition' = ConvolutionImageFilter_SetBoundaryCondition, 'GetBoundaryCondition' = ConvolutionImageFilter_GetBoundaryCondition, 'SetOutputRegionMode' = ConvolutionImageFilter_SetOutputRegionMode, 'GetOutputRegionMode' = ConvolutionImageFilter_GetOutputRegionMode, 'GetName' = ConvolutionImageFilter_GetName, 'ToString' = ConvolutionImageFilter_ToString, 'Execute' = ConvolutionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ConvolutionImageFilter
# Start of Convolution

`Convolution__SWIG_0` = function(image, kernelImage, normalize, boundaryCondition, outputRegionMode, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(kernelImage, "ExternalReference")) kernelImage = slot(kernelImage,"ref"); 
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__ConvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  outputRegionMode = enumToInteger(outputRegionMode, "_itk__simple__ConvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(outputRegionMode) > 1) {
    warning("using only the first element of outputRegionMode");
  };
  
  ;ans = .Call('R_swig_Convolution__SWIG_0', image, kernelImage, normalize, boundaryCondition, outputRegionMode, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Convolution__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Convolution__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical', 'character', 'character')
class(`Convolution__SWIG_0`) = c("SWIGFunction", class('Convolution__SWIG_0'))

# Start of Convolution

`Convolution__SWIG_1` = function(image, kernelImage, normalize, boundaryCondition, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(kernelImage, "ExternalReference")) kernelImage = slot(kernelImage,"ref"); 
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__ConvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  ;ans = .Call('R_swig_Convolution__SWIG_1', image, kernelImage, normalize, boundaryCondition, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Convolution__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Convolution__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical', 'character')
class(`Convolution__SWIG_1`) = c("SWIGFunction", class('Convolution__SWIG_1'))

# Start of Convolution

`Convolution__SWIG_2` = function(image, kernelImage, normalize, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(kernelImage, "ExternalReference")) kernelImage = slot(kernelImage,"ref"); 
  normalize = as.logical(normalize);
  ;ans = .Call('R_swig_Convolution__SWIG_2', image, kernelImage, normalize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Convolution__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Convolution__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical')
class(`Convolution__SWIG_2`) = c("SWIGFunction", class('Convolution__SWIG_2'))

# Start of Convolution

`Convolution__SWIG_3` = function(image, kernelImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(kernelImage, "ExternalReference")) kernelImage = slot(kernelImage,"ref"); 
  ;ans = .Call('R_swig_Convolution__SWIG_3', image, kernelImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Convolution__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Convolution__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Convolution__SWIG_3`) = c("SWIGFunction", class('Convolution__SWIG_3'))

`Convolution` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Convolution__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Convolution__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- Convolution__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.character(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- Convolution__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Convolution with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_CosImageFilter

`delete_CosImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_CosImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_CosImageFilter`, 'returnType') = 'void'
attr(`delete_CosImageFilter`, "inputTypes") = c('_p_itk__simple__CosImageFilter')
class(`delete_CosImageFilter`) = c("SWIGFunction", class('delete_CosImageFilter'))

# Start of new_CosImageFilter

`CosImageFilter` = function()
{
  ;ans = .Call('R_swig_new_CosImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CosImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CosImageFilter);
  ans
  
}

attr(`CosImageFilter`, 'returnType') = '_p_itk__simple__CosImageFilter'
class(`CosImageFilter`) = c("SWIGFunction", class('CosImageFilter'))

# Start of CosImageFilter_GetName

`CosImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CosImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CosImageFilter_GetName`, 'returnType') = 'character'
attr(`CosImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__CosImageFilter')
class(`CosImageFilter_GetName`) = c("SWIGFunction", class('CosImageFilter_GetName'))

# Start of CosImageFilter_ToString

`CosImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CosImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CosImageFilter_ToString`, 'returnType') = 'character'
attr(`CosImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__CosImageFilter')
class(`CosImageFilter_ToString`) = c("SWIGFunction", class('CosImageFilter_ToString'))

# Start of CosImageFilter_Execute

`CosImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_CosImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CosImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`CosImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__CosImageFilter', '_p_itk__simple__Image')
class(`CosImageFilter_Execute`) = c("SWIGFunction", class('CosImageFilter_Execute'))

# Start of accessor method for itk::simple::CosImageFilter
setMethod('$', '_p_itk__simple__CosImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = CosImageFilter_GetName, 'ToString' = CosImageFilter_ToString, 'Execute' = CosImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::CosImageFilter
# Start of Cos

`Cos` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Cos', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Cos`, 'returnType') = '_p_itk__simple__Image'
attr(`Cos`, "inputTypes") = c('_p_itk__simple__Image')
class(`Cos`) = c("SWIGFunction", class('Cos'))

# Start of delete_CropImageFilter

`delete_CropImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_CropImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_CropImageFilter`, 'returnType') = 'void'
attr(`delete_CropImageFilter`, "inputTypes") = c('_p_itk__simple__CropImageFilter')
class(`delete_CropImageFilter`) = c("SWIGFunction", class('delete_CropImageFilter'))

# Start of new_CropImageFilter

`CropImageFilter` = function()
{
  ;ans = .Call('R_swig_new_CropImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CropImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CropImageFilter);
  ans
  
}

attr(`CropImageFilter`, 'returnType') = '_p_itk__simple__CropImageFilter'
class(`CropImageFilter`) = c("SWIGFunction", class('CropImageFilter'))

# Start of CropImageFilter_SetLowerBoundaryCropSize

`CropImageFilter_SetLowerBoundaryCropSize` = function(self, LowerBoundaryCropSize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  LowerBoundaryCropSize = as.integer(LowerBoundaryCropSize);
  ;ans = .Call('R_swig_CropImageFilter_SetLowerBoundaryCropSize', self, LowerBoundaryCropSize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CropImageFilter_SetLowerBoundaryCropSize`, 'returnType') = 'void'
attr(`CropImageFilter_SetLowerBoundaryCropSize`, "inputTypes") = c('_p_itk__simple__CropImageFilter', 'integer')
class(`CropImageFilter_SetLowerBoundaryCropSize`) = c("SWIGFunction", class('CropImageFilter_SetLowerBoundaryCropSize'))

# Start of CropImageFilter_GetLowerBoundaryCropSize

`CropImageFilter_GetLowerBoundaryCropSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CropImageFilter_GetLowerBoundaryCropSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CropImageFilter_GetLowerBoundaryCropSize`, 'returnType') = 'integer'
attr(`CropImageFilter_GetLowerBoundaryCropSize`, "inputTypes") = c('_p_itk__simple__CropImageFilter')
class(`CropImageFilter_GetLowerBoundaryCropSize`) = c("SWIGFunction", class('CropImageFilter_GetLowerBoundaryCropSize'))

# Start of CropImageFilter_SetUpperBoundaryCropSize

`CropImageFilter_SetUpperBoundaryCropSize` = function(self, UpperBoundaryCropSize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UpperBoundaryCropSize = as.integer(UpperBoundaryCropSize);
  ;ans = .Call('R_swig_CropImageFilter_SetUpperBoundaryCropSize', self, UpperBoundaryCropSize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CropImageFilter_SetUpperBoundaryCropSize`, 'returnType') = 'void'
attr(`CropImageFilter_SetUpperBoundaryCropSize`, "inputTypes") = c('_p_itk__simple__CropImageFilter', 'integer')
class(`CropImageFilter_SetUpperBoundaryCropSize`) = c("SWIGFunction", class('CropImageFilter_SetUpperBoundaryCropSize'))

# Start of CropImageFilter_GetUpperBoundaryCropSize

`CropImageFilter_GetUpperBoundaryCropSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CropImageFilter_GetUpperBoundaryCropSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CropImageFilter_GetUpperBoundaryCropSize`, 'returnType') = 'integer'
attr(`CropImageFilter_GetUpperBoundaryCropSize`, "inputTypes") = c('_p_itk__simple__CropImageFilter')
class(`CropImageFilter_GetUpperBoundaryCropSize`) = c("SWIGFunction", class('CropImageFilter_GetUpperBoundaryCropSize'))

# Start of CropImageFilter_GetName

`CropImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CropImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CropImageFilter_GetName`, 'returnType') = 'character'
attr(`CropImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__CropImageFilter')
class(`CropImageFilter_GetName`) = c("SWIGFunction", class('CropImageFilter_GetName'))

# Start of CropImageFilter_ToString

`CropImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CropImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CropImageFilter_ToString`, 'returnType') = 'character'
attr(`CropImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__CropImageFilter')
class(`CropImageFilter_ToString`) = c("SWIGFunction", class('CropImageFilter_ToString'))

# Start of CropImageFilter_Execute

`CropImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_CropImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CropImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`CropImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__CropImageFilter', '_p_itk__simple__Image')
class(`CropImageFilter_Execute`) = c("SWIGFunction", class('CropImageFilter_Execute'))

# Start of accessor method for itk::simple::CropImageFilter
setMethod('$', '_p_itk__simple__CropImageFilter', function(x, name)

{
  accessorFuns = list('SetLowerBoundaryCropSize' = CropImageFilter_SetLowerBoundaryCropSize, 'GetLowerBoundaryCropSize' = CropImageFilter_GetLowerBoundaryCropSize, 'SetUpperBoundaryCropSize' = CropImageFilter_SetUpperBoundaryCropSize, 'GetUpperBoundaryCropSize' = CropImageFilter_GetUpperBoundaryCropSize, 'GetName' = CropImageFilter_GetName, 'ToString' = CropImageFilter_ToString, 'Execute' = CropImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::CropImageFilter
# Start of Crop

`Crop__SWIG_0` = function(image1, lowerBoundaryCropSize, upperBoundaryCropSize, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  lowerBoundaryCropSize = as.integer(lowerBoundaryCropSize);
  upperBoundaryCropSize = as.integer(upperBoundaryCropSize);
  ;ans = .Call('R_swig_Crop__SWIG_0', image1, lowerBoundaryCropSize, upperBoundaryCropSize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Crop__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Crop__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Crop__SWIG_0`) = c("SWIGFunction", class('Crop__SWIG_0'))

# Start of Crop

`Crop__SWIG_1` = function(image1, lowerBoundaryCropSize, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  lowerBoundaryCropSize = as.integer(lowerBoundaryCropSize);
  ;ans = .Call('R_swig_Crop__SWIG_1', image1, lowerBoundaryCropSize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Crop__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Crop__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Crop__SWIG_1`) = c("SWIGFunction", class('Crop__SWIG_1'))

# Start of Crop

`Crop__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Crop__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Crop__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Crop__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`Crop__SWIG_2`) = c("SWIGFunction", class('Crop__SWIG_2'))

`Crop` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Crop__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- Crop__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- Crop__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Crop with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_CurvatureAnisotropicDiffusionImageFilter

`delete_CurvatureAnisotropicDiffusionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_CurvatureAnisotropicDiffusionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_CurvatureAnisotropicDiffusionImageFilter`, 'returnType') = 'void'
attr(`delete_CurvatureAnisotropicDiffusionImageFilter`, "inputTypes") = c('_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter')
class(`delete_CurvatureAnisotropicDiffusionImageFilter`) = c("SWIGFunction", class('delete_CurvatureAnisotropicDiffusionImageFilter'))

# Start of new_CurvatureAnisotropicDiffusionImageFilter

`CurvatureAnisotropicDiffusionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_CurvatureAnisotropicDiffusionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CurvatureAnisotropicDiffusionImageFilter);
  ans
  
}

attr(`CurvatureAnisotropicDiffusionImageFilter`, 'returnType') = '_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter'
class(`CurvatureAnisotropicDiffusionImageFilter`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusionImageFilter'))

# Start of CurvatureAnisotropicDiffusionImageFilter_SetTimeStep

`CurvatureAnisotropicDiffusionImageFilter_SetTimeStep` = function(self, TimeStep)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_CurvatureAnisotropicDiffusionImageFilter_SetTimeStep', self, TimeStep, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CurvatureAnisotropicDiffusionImageFilter_SetTimeStep`, 'returnType') = 'void'
attr(`CurvatureAnisotropicDiffusionImageFilter_SetTimeStep`, "inputTypes") = c('_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter', 'numeric')
class(`CurvatureAnisotropicDiffusionImageFilter_SetTimeStep`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusionImageFilter_SetTimeStep'))

# Start of CurvatureAnisotropicDiffusionImageFilter_GetTimeStep

`CurvatureAnisotropicDiffusionImageFilter_GetTimeStep` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CurvatureAnisotropicDiffusionImageFilter_GetTimeStep', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CurvatureAnisotropicDiffusionImageFilter_GetTimeStep`, 'returnType') = 'numeric'
attr(`CurvatureAnisotropicDiffusionImageFilter_GetTimeStep`, "inputTypes") = c('_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter')
class(`CurvatureAnisotropicDiffusionImageFilter_GetTimeStep`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusionImageFilter_GetTimeStep'))

# Start of CurvatureAnisotropicDiffusionImageFilter_SetConductanceParameter

`CurvatureAnisotropicDiffusionImageFilter_SetConductanceParameter` = function(self, ConductanceParameter)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_CurvatureAnisotropicDiffusionImageFilter_SetConductanceParameter', self, ConductanceParameter, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CurvatureAnisotropicDiffusionImageFilter_SetConductanceParameter`, 'returnType') = 'void'
attr(`CurvatureAnisotropicDiffusionImageFilter_SetConductanceParameter`, "inputTypes") = c('_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter', 'numeric')
class(`CurvatureAnisotropicDiffusionImageFilter_SetConductanceParameter`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusionImageFilter_SetConductanceParameter'))

# Start of CurvatureAnisotropicDiffusionImageFilter_GetConductanceParameter

`CurvatureAnisotropicDiffusionImageFilter_GetConductanceParameter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CurvatureAnisotropicDiffusionImageFilter_GetConductanceParameter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CurvatureAnisotropicDiffusionImageFilter_GetConductanceParameter`, 'returnType') = 'numeric'
attr(`CurvatureAnisotropicDiffusionImageFilter_GetConductanceParameter`, "inputTypes") = c('_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter')
class(`CurvatureAnisotropicDiffusionImageFilter_GetConductanceParameter`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusionImageFilter_GetConductanceParameter'))

# Start of CurvatureAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval

`CurvatureAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval` = function(self, ConductanceScalingUpdateInterval)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ConductanceScalingUpdateInterval = as.integer(ConductanceScalingUpdateInterval);
  
  if(length(ConductanceScalingUpdateInterval) > 1) {
    warning("using only the first element of ConductanceScalingUpdateInterval");
  };
  
  ;ans = .Call('R_swig_CurvatureAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval', self, ConductanceScalingUpdateInterval, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CurvatureAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval`, 'returnType') = 'void'
attr(`CurvatureAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval`, "inputTypes") = c('_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter', 'integer')
class(`CurvatureAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval'))

# Start of CurvatureAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval

`CurvatureAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CurvatureAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CurvatureAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval`, 'returnType') = 'integer'
attr(`CurvatureAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval`, "inputTypes") = c('_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter')
class(`CurvatureAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval'))

# Start of CurvatureAnisotropicDiffusionImageFilter_SetNumberOfIterations

`CurvatureAnisotropicDiffusionImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_CurvatureAnisotropicDiffusionImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CurvatureAnisotropicDiffusionImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`CurvatureAnisotropicDiffusionImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter', 'integer')
class(`CurvatureAnisotropicDiffusionImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusionImageFilter_SetNumberOfIterations'))

# Start of CurvatureAnisotropicDiffusionImageFilter_GetNumberOfIterations

`CurvatureAnisotropicDiffusionImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CurvatureAnisotropicDiffusionImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CurvatureAnisotropicDiffusionImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`CurvatureAnisotropicDiffusionImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter')
class(`CurvatureAnisotropicDiffusionImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusionImageFilter_GetNumberOfIterations'))

# Start of CurvatureAnisotropicDiffusionImageFilter_GetName

`CurvatureAnisotropicDiffusionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CurvatureAnisotropicDiffusionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CurvatureAnisotropicDiffusionImageFilter_GetName`, 'returnType') = 'character'
attr(`CurvatureAnisotropicDiffusionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter')
class(`CurvatureAnisotropicDiffusionImageFilter_GetName`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusionImageFilter_GetName'))

# Start of CurvatureAnisotropicDiffusionImageFilter_ToString

`CurvatureAnisotropicDiffusionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CurvatureAnisotropicDiffusionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CurvatureAnisotropicDiffusionImageFilter_ToString`, 'returnType') = 'character'
attr(`CurvatureAnisotropicDiffusionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter')
class(`CurvatureAnisotropicDiffusionImageFilter_ToString`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusionImageFilter_ToString'))

# Start of CurvatureAnisotropicDiffusionImageFilter_Execute

`CurvatureAnisotropicDiffusionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_CurvatureAnisotropicDiffusionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CurvatureAnisotropicDiffusionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`CurvatureAnisotropicDiffusionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter', '_p_itk__simple__Image')
class(`CurvatureAnisotropicDiffusionImageFilter_Execute`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusionImageFilter_Execute'))

# Start of CurvatureAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep

`CurvatureAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep` = function(self, inImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(inImage, "ExternalReference")) inImage = slot(inImage,"ref"); 
  ;.Call('R_swig_CurvatureAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep', self, inImage, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CurvatureAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep`, 'returnType') = 'numeric'
attr(`CurvatureAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep`, "inputTypes") = c('_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter', '_p_itk__simple__Image')
class(`CurvatureAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep'))

# Start of accessor method for itk::simple::CurvatureAnisotropicDiffusionImageFilter
setMethod('$', '_p_itk__simple__CurvatureAnisotropicDiffusionImageFilter', function(x, name)

{
  accessorFuns = list('SetTimeStep' = CurvatureAnisotropicDiffusionImageFilter_SetTimeStep, 'GetTimeStep' = CurvatureAnisotropicDiffusionImageFilter_GetTimeStep, 'SetConductanceParameter' = CurvatureAnisotropicDiffusionImageFilter_SetConductanceParameter, 'GetConductanceParameter' = CurvatureAnisotropicDiffusionImageFilter_GetConductanceParameter, 'SetConductanceScalingUpdateInterval' = CurvatureAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval, 'GetConductanceScalingUpdateInterval' = CurvatureAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval, 'SetNumberOfIterations' = CurvatureAnisotropicDiffusionImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = CurvatureAnisotropicDiffusionImageFilter_GetNumberOfIterations, 'GetName' = CurvatureAnisotropicDiffusionImageFilter_GetName, 'ToString' = CurvatureAnisotropicDiffusionImageFilter_ToString, 'Execute' = CurvatureAnisotropicDiffusionImageFilter_Execute, 'EstimateOptimalTimeStep' = CurvatureAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::CurvatureAnisotropicDiffusionImageFilter
# Start of CurvatureAnisotropicDiffusion

`CurvatureAnisotropicDiffusion__SWIG_0` = function(image1, timeStep, conductanceParameter, conductanceScalingUpdateInterval, numberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  conductanceScalingUpdateInterval = as.integer(conductanceScalingUpdateInterval);
  
  if(length(conductanceScalingUpdateInterval) > 1) {
    warning("using only the first element of conductanceScalingUpdateInterval");
  };
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_CurvatureAnisotropicDiffusion__SWIG_0', image1, timeStep, conductanceParameter, conductanceScalingUpdateInterval, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CurvatureAnisotropicDiffusion__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`CurvatureAnisotropicDiffusion__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'integer', 'integer')
class(`CurvatureAnisotropicDiffusion__SWIG_0`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusion__SWIG_0'))

# Start of CurvatureAnisotropicDiffusion

`CurvatureAnisotropicDiffusion__SWIG_1` = function(image1, timeStep, conductanceParameter, conductanceScalingUpdateInterval, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  conductanceScalingUpdateInterval = as.integer(conductanceScalingUpdateInterval);
  
  if(length(conductanceScalingUpdateInterval) > 1) {
    warning("using only the first element of conductanceScalingUpdateInterval");
  };
  
  ;ans = .Call('R_swig_CurvatureAnisotropicDiffusion__SWIG_1', image1, timeStep, conductanceParameter, conductanceScalingUpdateInterval, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CurvatureAnisotropicDiffusion__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`CurvatureAnisotropicDiffusion__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'integer')
class(`CurvatureAnisotropicDiffusion__SWIG_1`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusion__SWIG_1'))

# Start of CurvatureAnisotropicDiffusion

`CurvatureAnisotropicDiffusion__SWIG_2` = function(image1, timeStep, conductanceParameter, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_CurvatureAnisotropicDiffusion__SWIG_2', image1, timeStep, conductanceParameter, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CurvatureAnisotropicDiffusion__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`CurvatureAnisotropicDiffusion__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`CurvatureAnisotropicDiffusion__SWIG_2`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusion__SWIG_2'))

# Start of CurvatureAnisotropicDiffusion

`CurvatureAnisotropicDiffusion__SWIG_3` = function(image1, timeStep, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_CurvatureAnisotropicDiffusion__SWIG_3', image1, timeStep, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CurvatureAnisotropicDiffusion__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`CurvatureAnisotropicDiffusion__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`CurvatureAnisotropicDiffusion__SWIG_3`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusion__SWIG_3'))

# Start of CurvatureAnisotropicDiffusion

`CurvatureAnisotropicDiffusion__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_CurvatureAnisotropicDiffusion__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CurvatureAnisotropicDiffusion__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`CurvatureAnisotropicDiffusion__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`CurvatureAnisotropicDiffusion__SWIG_4`) = c("SWIGFunction", class('CurvatureAnisotropicDiffusion__SWIG_4'))

`CurvatureAnisotropicDiffusion` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- CurvatureAnisotropicDiffusion__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- CurvatureAnisotropicDiffusion__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- CurvatureAnisotropicDiffusion__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- CurvatureAnisotropicDiffusion__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- CurvatureAnisotropicDiffusion__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for CurvatureAnisotropicDiffusion with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_CurvatureFlowImageFilter

`delete_CurvatureFlowImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_CurvatureFlowImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_CurvatureFlowImageFilter`, 'returnType') = 'void'
attr(`delete_CurvatureFlowImageFilter`, "inputTypes") = c('_p_itk__simple__CurvatureFlowImageFilter')
class(`delete_CurvatureFlowImageFilter`) = c("SWIGFunction", class('delete_CurvatureFlowImageFilter'))

# Start of new_CurvatureFlowImageFilter

`CurvatureFlowImageFilter` = function()
{
  ;ans = .Call('R_swig_new_CurvatureFlowImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CurvatureFlowImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CurvatureFlowImageFilter);
  ans
  
}

attr(`CurvatureFlowImageFilter`, 'returnType') = '_p_itk__simple__CurvatureFlowImageFilter'
class(`CurvatureFlowImageFilter`) = c("SWIGFunction", class('CurvatureFlowImageFilter'))

# Start of CurvatureFlowImageFilter_SetTimeStep

`CurvatureFlowImageFilter_SetTimeStep` = function(self, TimeStep)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_CurvatureFlowImageFilter_SetTimeStep', self, TimeStep, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CurvatureFlowImageFilter_SetTimeStep`, 'returnType') = 'void'
attr(`CurvatureFlowImageFilter_SetTimeStep`, "inputTypes") = c('_p_itk__simple__CurvatureFlowImageFilter', 'numeric')
class(`CurvatureFlowImageFilter_SetTimeStep`) = c("SWIGFunction", class('CurvatureFlowImageFilter_SetTimeStep'))

# Start of CurvatureFlowImageFilter_GetTimeStep

`CurvatureFlowImageFilter_GetTimeStep` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CurvatureFlowImageFilter_GetTimeStep', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CurvatureFlowImageFilter_GetTimeStep`, 'returnType') = 'numeric'
attr(`CurvatureFlowImageFilter_GetTimeStep`, "inputTypes") = c('_p_itk__simple__CurvatureFlowImageFilter')
class(`CurvatureFlowImageFilter_GetTimeStep`) = c("SWIGFunction", class('CurvatureFlowImageFilter_GetTimeStep'))

# Start of CurvatureFlowImageFilter_SetNumberOfIterations

`CurvatureFlowImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_CurvatureFlowImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CurvatureFlowImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`CurvatureFlowImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__CurvatureFlowImageFilter', 'integer')
class(`CurvatureFlowImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('CurvatureFlowImageFilter_SetNumberOfIterations'))

# Start of CurvatureFlowImageFilter_GetNumberOfIterations

`CurvatureFlowImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CurvatureFlowImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CurvatureFlowImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`CurvatureFlowImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__CurvatureFlowImageFilter')
class(`CurvatureFlowImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('CurvatureFlowImageFilter_GetNumberOfIterations'))

# Start of CurvatureFlowImageFilter_GetName

`CurvatureFlowImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CurvatureFlowImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CurvatureFlowImageFilter_GetName`, 'returnType') = 'character'
attr(`CurvatureFlowImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__CurvatureFlowImageFilter')
class(`CurvatureFlowImageFilter_GetName`) = c("SWIGFunction", class('CurvatureFlowImageFilter_GetName'))

# Start of CurvatureFlowImageFilter_ToString

`CurvatureFlowImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CurvatureFlowImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CurvatureFlowImageFilter_ToString`, 'returnType') = 'character'
attr(`CurvatureFlowImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__CurvatureFlowImageFilter')
class(`CurvatureFlowImageFilter_ToString`) = c("SWIGFunction", class('CurvatureFlowImageFilter_ToString'))

# Start of CurvatureFlowImageFilter_Execute

`CurvatureFlowImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_CurvatureFlowImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CurvatureFlowImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`CurvatureFlowImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__CurvatureFlowImageFilter', '_p_itk__simple__Image')
class(`CurvatureFlowImageFilter_Execute`) = c("SWIGFunction", class('CurvatureFlowImageFilter_Execute'))

# Start of accessor method for itk::simple::CurvatureFlowImageFilter
setMethod('$', '_p_itk__simple__CurvatureFlowImageFilter', function(x, name)

{
  accessorFuns = list('SetTimeStep' = CurvatureFlowImageFilter_SetTimeStep, 'GetTimeStep' = CurvatureFlowImageFilter_GetTimeStep, 'SetNumberOfIterations' = CurvatureFlowImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = CurvatureFlowImageFilter_GetNumberOfIterations, 'GetName' = CurvatureFlowImageFilter_GetName, 'ToString' = CurvatureFlowImageFilter_ToString, 'Execute' = CurvatureFlowImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::CurvatureFlowImageFilter
# Start of CurvatureFlow

`CurvatureFlow__SWIG_0` = function(image1, timeStep, numberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_CurvatureFlow__SWIG_0', image1, timeStep, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CurvatureFlow__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`CurvatureFlow__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`CurvatureFlow__SWIG_0`) = c("SWIGFunction", class('CurvatureFlow__SWIG_0'))

# Start of CurvatureFlow

`CurvatureFlow__SWIG_1` = function(image1, timeStep, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_CurvatureFlow__SWIG_1', image1, timeStep, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CurvatureFlow__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`CurvatureFlow__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`CurvatureFlow__SWIG_1`) = c("SWIGFunction", class('CurvatureFlow__SWIG_1'))

# Start of CurvatureFlow

`CurvatureFlow__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_CurvatureFlow__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CurvatureFlow__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`CurvatureFlow__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`CurvatureFlow__SWIG_2`) = c("SWIGFunction", class('CurvatureFlow__SWIG_2'))

`CurvatureFlow` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- CurvatureFlow__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- CurvatureFlow__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- CurvatureFlow__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for CurvatureFlow with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_CyclicShiftImageFilter

`delete_CyclicShiftImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_CyclicShiftImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_CyclicShiftImageFilter`, 'returnType') = 'void'
attr(`delete_CyclicShiftImageFilter`, "inputTypes") = c('_p_itk__simple__CyclicShiftImageFilter')
class(`delete_CyclicShiftImageFilter`) = c("SWIGFunction", class('delete_CyclicShiftImageFilter'))

# Start of new_CyclicShiftImageFilter

`CyclicShiftImageFilter` = function()
{
  ;ans = .Call('R_swig_new_CyclicShiftImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__CyclicShiftImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_CyclicShiftImageFilter);
  ans
  
}

attr(`CyclicShiftImageFilter`, 'returnType') = '_p_itk__simple__CyclicShiftImageFilter'
class(`CyclicShiftImageFilter`) = c("SWIGFunction", class('CyclicShiftImageFilter'))

# Start of CyclicShiftImageFilter_SetShift

`CyclicShiftImageFilter_SetShift__SWIG_0` = function(self, Shift)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Shift = as.integer(Shift);
  ;ans = .Call('R_swig_CyclicShiftImageFilter_SetShift__SWIG_0', self, Shift, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CyclicShiftImageFilter_SetShift__SWIG_0`, 'returnType') = 'void'
attr(`CyclicShiftImageFilter_SetShift__SWIG_0`, "inputTypes") = c('_p_itk__simple__CyclicShiftImageFilter', 'integer')
class(`CyclicShiftImageFilter_SetShift__SWIG_0`) = c("SWIGFunction", class('CyclicShiftImageFilter_SetShift__SWIG_0'))

# Start of CyclicShiftImageFilter_SetShift

`CyclicShiftImageFilter_SetShift__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_CyclicShiftImageFilter_SetShift__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`CyclicShiftImageFilter_SetShift__SWIG_1`, 'returnType') = 'void'
attr(`CyclicShiftImageFilter_SetShift__SWIG_1`, "inputTypes") = c('_p_itk__simple__CyclicShiftImageFilter', 'integer')
class(`CyclicShiftImageFilter_SetShift__SWIG_1`) = c("SWIGFunction", class('CyclicShiftImageFilter_SetShift__SWIG_1'))

`CyclicShiftImageFilter_SetShift` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__CyclicShiftImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- CyclicShiftImageFilter_SetShift__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__CyclicShiftImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- CyclicShiftImageFilter_SetShift__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for CyclicShiftImageFilter_SetShift with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of CyclicShiftImageFilter_GetShift

`CyclicShiftImageFilter_GetShift` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CyclicShiftImageFilter_GetShift', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CyclicShiftImageFilter_GetShift`, 'returnType') = 'integer'
attr(`CyclicShiftImageFilter_GetShift`, "inputTypes") = c('_p_itk__simple__CyclicShiftImageFilter')
class(`CyclicShiftImageFilter_GetShift`) = c("SWIGFunction", class('CyclicShiftImageFilter_GetShift'))

# Start of CyclicShiftImageFilter_GetName

`CyclicShiftImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CyclicShiftImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CyclicShiftImageFilter_GetName`, 'returnType') = 'character'
attr(`CyclicShiftImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__CyclicShiftImageFilter')
class(`CyclicShiftImageFilter_GetName`) = c("SWIGFunction", class('CyclicShiftImageFilter_GetName'))

# Start of CyclicShiftImageFilter_ToString

`CyclicShiftImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_CyclicShiftImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`CyclicShiftImageFilter_ToString`, 'returnType') = 'character'
attr(`CyclicShiftImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__CyclicShiftImageFilter')
class(`CyclicShiftImageFilter_ToString`) = c("SWIGFunction", class('CyclicShiftImageFilter_ToString'))

# Start of CyclicShiftImageFilter_Execute

`CyclicShiftImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_CyclicShiftImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CyclicShiftImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`CyclicShiftImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__CyclicShiftImageFilter', '_p_itk__simple__Image')
class(`CyclicShiftImageFilter_Execute`) = c("SWIGFunction", class('CyclicShiftImageFilter_Execute'))

# Start of accessor method for itk::simple::CyclicShiftImageFilter
setMethod('$', '_p_itk__simple__CyclicShiftImageFilter', function(x, name)

{
  accessorFuns = list('SetShift' = CyclicShiftImageFilter_SetShift, 'GetShift' = CyclicShiftImageFilter_GetShift, 'GetName' = CyclicShiftImageFilter_GetName, 'ToString' = CyclicShiftImageFilter_ToString, 'Execute' = CyclicShiftImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::CyclicShiftImageFilter
# Start of CyclicShift

`CyclicShift__SWIG_0` = function(image1, shift, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  shift = as.integer(shift);
  ;ans = .Call('R_swig_CyclicShift__SWIG_0', image1, shift, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CyclicShift__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`CyclicShift__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`CyclicShift__SWIG_0`) = c("SWIGFunction", class('CyclicShift__SWIG_0'))

# Start of CyclicShift

`CyclicShift__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_CyclicShift__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`CyclicShift__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`CyclicShift__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`CyclicShift__SWIG_1`) = c("SWIGFunction", class('CyclicShift__SWIG_1'))

`CyclicShift` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- CyclicShift__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- CyclicShift__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for CyclicShift with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_DICOMOrientImageFilter

`delete_DICOMOrientImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_DICOMOrientImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_DICOMOrientImageFilter`, 'returnType') = 'void'
attr(`delete_DICOMOrientImageFilter`, "inputTypes") = c('_p_itk__simple__DICOMOrientImageFilter')
class(`delete_DICOMOrientImageFilter`) = c("SWIGFunction", class('delete_DICOMOrientImageFilter'))

# Start of new_DICOMOrientImageFilter

`DICOMOrientImageFilter` = function()
{
  ;ans = .Call('R_swig_new_DICOMOrientImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DICOMOrientImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DICOMOrientImageFilter);
  ans
  
}

attr(`DICOMOrientImageFilter`, 'returnType') = '_p_itk__simple__DICOMOrientImageFilter'
class(`DICOMOrientImageFilter`) = c("SWIGFunction", class('DICOMOrientImageFilter'))

# Start of DICOMOrientImageFilter_GetOrientationFromDirectionCosines

`DICOMOrientImageFilter_GetOrientationFromDirectionCosines` = function(direction, .copy = FALSE)
{
  direction = as.numeric(direction);
  ;.Call('R_swig_DICOMOrientImageFilter_GetOrientationFromDirectionCosines', direction, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DICOMOrientImageFilter_GetOrientationFromDirectionCosines`, 'returnType') = 'character'
attr(`DICOMOrientImageFilter_GetOrientationFromDirectionCosines`, "inputTypes") = c('_p_std__vectorT_double_t')
class(`DICOMOrientImageFilter_GetOrientationFromDirectionCosines`) = c("SWIGFunction", class('DICOMOrientImageFilter_GetOrientationFromDirectionCosines'))

# Start of DICOMOrientImageFilter_GetDirectionCosinesFromOrientation

`DICOMOrientImageFilter_GetDirectionCosinesFromOrientation` = function(str, .copy = FALSE)
{
  str = as(str, "character"); 
  ;.Call('R_swig_DICOMOrientImageFilter_GetDirectionCosinesFromOrientation', str, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DICOMOrientImageFilter_GetDirectionCosinesFromOrientation`, 'returnType') = 'numeric'
attr(`DICOMOrientImageFilter_GetDirectionCosinesFromOrientation`, "inputTypes") = c('character')
class(`DICOMOrientImageFilter_GetDirectionCosinesFromOrientation`) = c("SWIGFunction", class('DICOMOrientImageFilter_GetDirectionCosinesFromOrientation'))

# Start of DICOMOrientImageFilter_SetDesiredCoordinateOrientation

`DICOMOrientImageFilter_SetDesiredCoordinateOrientation` = function(self, DesiredCoordinateOrientation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  DesiredCoordinateOrientation = as(DesiredCoordinateOrientation, "character"); 
  ;ans = .Call('R_swig_DICOMOrientImageFilter_SetDesiredCoordinateOrientation', self, DesiredCoordinateOrientation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DICOMOrientImageFilter_SetDesiredCoordinateOrientation`, 'returnType') = 'void'
attr(`DICOMOrientImageFilter_SetDesiredCoordinateOrientation`, "inputTypes") = c('_p_itk__simple__DICOMOrientImageFilter', 'character')
class(`DICOMOrientImageFilter_SetDesiredCoordinateOrientation`) = c("SWIGFunction", class('DICOMOrientImageFilter_SetDesiredCoordinateOrientation'))

# Start of DICOMOrientImageFilter_GetDesiredCoordinateOrientation

`DICOMOrientImageFilter_GetDesiredCoordinateOrientation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DICOMOrientImageFilter_GetDesiredCoordinateOrientation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DICOMOrientImageFilter_GetDesiredCoordinateOrientation`, 'returnType') = 'character'
attr(`DICOMOrientImageFilter_GetDesiredCoordinateOrientation`, "inputTypes") = c('_p_itk__simple__DICOMOrientImageFilter')
class(`DICOMOrientImageFilter_GetDesiredCoordinateOrientation`) = c("SWIGFunction", class('DICOMOrientImageFilter_GetDesiredCoordinateOrientation'))

# Start of DICOMOrientImageFilter_GetFlipAxes

`DICOMOrientImageFilter_GetFlipAxes` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DICOMOrientImageFilter_GetFlipAxes', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DICOMOrientImageFilter_GetFlipAxes`, 'returnType') = 'logical'
attr(`DICOMOrientImageFilter_GetFlipAxes`, "inputTypes") = c('_p_itk__simple__DICOMOrientImageFilter')
class(`DICOMOrientImageFilter_GetFlipAxes`) = c("SWIGFunction", class('DICOMOrientImageFilter_GetFlipAxes'))

# Start of DICOMOrientImageFilter_GetPermuteOrder

`DICOMOrientImageFilter_GetPermuteOrder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DICOMOrientImageFilter_GetPermuteOrder', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DICOMOrientImageFilter_GetPermuteOrder`, 'returnType') = 'integer'
attr(`DICOMOrientImageFilter_GetPermuteOrder`, "inputTypes") = c('_p_itk__simple__DICOMOrientImageFilter')
class(`DICOMOrientImageFilter_GetPermuteOrder`) = c("SWIGFunction", class('DICOMOrientImageFilter_GetPermuteOrder'))

# Start of DICOMOrientImageFilter_GetName

`DICOMOrientImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DICOMOrientImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DICOMOrientImageFilter_GetName`, 'returnType') = 'character'
attr(`DICOMOrientImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__DICOMOrientImageFilter')
class(`DICOMOrientImageFilter_GetName`) = c("SWIGFunction", class('DICOMOrientImageFilter_GetName'))

# Start of DICOMOrientImageFilter_ToString

`DICOMOrientImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DICOMOrientImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DICOMOrientImageFilter_ToString`, 'returnType') = 'character'
attr(`DICOMOrientImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__DICOMOrientImageFilter')
class(`DICOMOrientImageFilter_ToString`) = c("SWIGFunction", class('DICOMOrientImageFilter_ToString'))

# Start of DICOMOrientImageFilter_Execute

`DICOMOrientImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_DICOMOrientImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DICOMOrientImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`DICOMOrientImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__DICOMOrientImageFilter', '_p_itk__simple__Image')
class(`DICOMOrientImageFilter_Execute`) = c("SWIGFunction", class('DICOMOrientImageFilter_Execute'))

# Start of accessor method for itk::simple::DICOMOrientImageFilter
setMethod('$', '_p_itk__simple__DICOMOrientImageFilter', function(x, name)

{
  accessorFuns = list('SetDesiredCoordinateOrientation' = DICOMOrientImageFilter_SetDesiredCoordinateOrientation, 'GetDesiredCoordinateOrientation' = DICOMOrientImageFilter_GetDesiredCoordinateOrientation, 'GetFlipAxes' = DICOMOrientImageFilter_GetFlipAxes, 'GetPermuteOrder' = DICOMOrientImageFilter_GetPermuteOrder, 'GetName' = DICOMOrientImageFilter_GetName, 'ToString' = DICOMOrientImageFilter_ToString, 'Execute' = DICOMOrientImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::DICOMOrientImageFilter
# Start of DICOMOrient

`DICOMOrient__SWIG_0` = function(image1, desiredCoordinateOrientation, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  desiredCoordinateOrientation = as(desiredCoordinateOrientation, "character"); 
  ;ans = .Call('R_swig_DICOMOrient__SWIG_0', image1, desiredCoordinateOrientation, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DICOMOrient__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`DICOMOrient__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'character')
class(`DICOMOrient__SWIG_0`) = c("SWIGFunction", class('DICOMOrient__SWIG_0'))

# Start of DICOMOrient

`DICOMOrient__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_DICOMOrient__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DICOMOrient__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`DICOMOrient__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`DICOMOrient__SWIG_1`) = c("SWIGFunction", class('DICOMOrient__SWIG_1'))

`DICOMOrient` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- DICOMOrient__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- DICOMOrient__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DICOMOrient with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_DanielssonDistanceMapImageFilter

`delete_DanielssonDistanceMapImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_DanielssonDistanceMapImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_DanielssonDistanceMapImageFilter`, 'returnType') = 'void'
attr(`delete_DanielssonDistanceMapImageFilter`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter')
class(`delete_DanielssonDistanceMapImageFilter`) = c("SWIGFunction", class('delete_DanielssonDistanceMapImageFilter'))

# Start of new_DanielssonDistanceMapImageFilter

`DanielssonDistanceMapImageFilter` = function()
{
  ;ans = .Call('R_swig_new_DanielssonDistanceMapImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DanielssonDistanceMapImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DanielssonDistanceMapImageFilter);
  ans
  
}

attr(`DanielssonDistanceMapImageFilter`, 'returnType') = '_p_itk__simple__DanielssonDistanceMapImageFilter'
class(`DanielssonDistanceMapImageFilter`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter'))

# Start of DanielssonDistanceMapImageFilter_SetInputIsBinary

`DanielssonDistanceMapImageFilter_SetInputIsBinary` = function(self, InputIsBinary)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InputIsBinary = as.logical(InputIsBinary);
  ;ans = .Call('R_swig_DanielssonDistanceMapImageFilter_SetInputIsBinary', self, InputIsBinary, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DanielssonDistanceMapImageFilter_SetInputIsBinary`, 'returnType') = 'void'
attr(`DanielssonDistanceMapImageFilter_SetInputIsBinary`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter', 'logical')
class(`DanielssonDistanceMapImageFilter_SetInputIsBinary`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_SetInputIsBinary'))

# Start of DanielssonDistanceMapImageFilter_InputIsBinaryOn

`DanielssonDistanceMapImageFilter_InputIsBinaryOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DanielssonDistanceMapImageFilter_InputIsBinaryOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DanielssonDistanceMapImageFilter_InputIsBinaryOn`, 'returnType') = 'void'
attr(`DanielssonDistanceMapImageFilter_InputIsBinaryOn`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter')
class(`DanielssonDistanceMapImageFilter_InputIsBinaryOn`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_InputIsBinaryOn'))

# Start of DanielssonDistanceMapImageFilter_InputIsBinaryOff

`DanielssonDistanceMapImageFilter_InputIsBinaryOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DanielssonDistanceMapImageFilter_InputIsBinaryOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DanielssonDistanceMapImageFilter_InputIsBinaryOff`, 'returnType') = 'void'
attr(`DanielssonDistanceMapImageFilter_InputIsBinaryOff`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter')
class(`DanielssonDistanceMapImageFilter_InputIsBinaryOff`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_InputIsBinaryOff'))

# Start of DanielssonDistanceMapImageFilter_GetInputIsBinary

`DanielssonDistanceMapImageFilter_GetInputIsBinary` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DanielssonDistanceMapImageFilter_GetInputIsBinary', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DanielssonDistanceMapImageFilter_GetInputIsBinary`, 'returnType') = 'logical'
attr(`DanielssonDistanceMapImageFilter_GetInputIsBinary`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter')
class(`DanielssonDistanceMapImageFilter_GetInputIsBinary`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_GetInputIsBinary'))

# Start of DanielssonDistanceMapImageFilter_SetSquaredDistance

`DanielssonDistanceMapImageFilter_SetSquaredDistance` = function(self, SquaredDistance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SquaredDistance = as.logical(SquaredDistance);
  ;ans = .Call('R_swig_DanielssonDistanceMapImageFilter_SetSquaredDistance', self, SquaredDistance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DanielssonDistanceMapImageFilter_SetSquaredDistance`, 'returnType') = 'void'
attr(`DanielssonDistanceMapImageFilter_SetSquaredDistance`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter', 'logical')
class(`DanielssonDistanceMapImageFilter_SetSquaredDistance`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_SetSquaredDistance'))

# Start of DanielssonDistanceMapImageFilter_SquaredDistanceOn

`DanielssonDistanceMapImageFilter_SquaredDistanceOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DanielssonDistanceMapImageFilter_SquaredDistanceOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DanielssonDistanceMapImageFilter_SquaredDistanceOn`, 'returnType') = 'void'
attr(`DanielssonDistanceMapImageFilter_SquaredDistanceOn`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter')
class(`DanielssonDistanceMapImageFilter_SquaredDistanceOn`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_SquaredDistanceOn'))

# Start of DanielssonDistanceMapImageFilter_SquaredDistanceOff

`DanielssonDistanceMapImageFilter_SquaredDistanceOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DanielssonDistanceMapImageFilter_SquaredDistanceOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DanielssonDistanceMapImageFilter_SquaredDistanceOff`, 'returnType') = 'void'
attr(`DanielssonDistanceMapImageFilter_SquaredDistanceOff`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter')
class(`DanielssonDistanceMapImageFilter_SquaredDistanceOff`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_SquaredDistanceOff'))

# Start of DanielssonDistanceMapImageFilter_GetSquaredDistance

`DanielssonDistanceMapImageFilter_GetSquaredDistance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DanielssonDistanceMapImageFilter_GetSquaredDistance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DanielssonDistanceMapImageFilter_GetSquaredDistance`, 'returnType') = 'logical'
attr(`DanielssonDistanceMapImageFilter_GetSquaredDistance`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter')
class(`DanielssonDistanceMapImageFilter_GetSquaredDistance`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_GetSquaredDistance'))

# Start of DanielssonDistanceMapImageFilter_SetUseImageSpacing

`DanielssonDistanceMapImageFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_DanielssonDistanceMapImageFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DanielssonDistanceMapImageFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`DanielssonDistanceMapImageFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter', 'logical')
class(`DanielssonDistanceMapImageFilter_SetUseImageSpacing`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_SetUseImageSpacing'))

# Start of DanielssonDistanceMapImageFilter_UseImageSpacingOn

`DanielssonDistanceMapImageFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DanielssonDistanceMapImageFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DanielssonDistanceMapImageFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`DanielssonDistanceMapImageFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter')
class(`DanielssonDistanceMapImageFilter_UseImageSpacingOn`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_UseImageSpacingOn'))

# Start of DanielssonDistanceMapImageFilter_UseImageSpacingOff

`DanielssonDistanceMapImageFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DanielssonDistanceMapImageFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DanielssonDistanceMapImageFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`DanielssonDistanceMapImageFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter')
class(`DanielssonDistanceMapImageFilter_UseImageSpacingOff`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_UseImageSpacingOff'))

# Start of DanielssonDistanceMapImageFilter_GetUseImageSpacing

`DanielssonDistanceMapImageFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DanielssonDistanceMapImageFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DanielssonDistanceMapImageFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`DanielssonDistanceMapImageFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter')
class(`DanielssonDistanceMapImageFilter_GetUseImageSpacing`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_GetUseImageSpacing'))

# Start of DanielssonDistanceMapImageFilter_GetVoronoiMap

`DanielssonDistanceMapImageFilter_GetVoronoiMap` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DanielssonDistanceMapImageFilter_GetVoronoiMap', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DanielssonDistanceMapImageFilter_GetVoronoiMap`, 'returnType') = '_p_itk__simple__Image'
attr(`DanielssonDistanceMapImageFilter_GetVoronoiMap`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter')
class(`DanielssonDistanceMapImageFilter_GetVoronoiMap`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_GetVoronoiMap'))

# Start of DanielssonDistanceMapImageFilter_GetVectorDistanceMap

`DanielssonDistanceMapImageFilter_GetVectorDistanceMap` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DanielssonDistanceMapImageFilter_GetVectorDistanceMap', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DanielssonDistanceMapImageFilter_GetVectorDistanceMap`, 'returnType') = '_p_itk__simple__Image'
attr(`DanielssonDistanceMapImageFilter_GetVectorDistanceMap`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter')
class(`DanielssonDistanceMapImageFilter_GetVectorDistanceMap`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_GetVectorDistanceMap'))

# Start of DanielssonDistanceMapImageFilter_GetName

`DanielssonDistanceMapImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DanielssonDistanceMapImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DanielssonDistanceMapImageFilter_GetName`, 'returnType') = 'character'
attr(`DanielssonDistanceMapImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter')
class(`DanielssonDistanceMapImageFilter_GetName`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_GetName'))

# Start of DanielssonDistanceMapImageFilter_ToString

`DanielssonDistanceMapImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DanielssonDistanceMapImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DanielssonDistanceMapImageFilter_ToString`, 'returnType') = 'character'
attr(`DanielssonDistanceMapImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter')
class(`DanielssonDistanceMapImageFilter_ToString`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_ToString'))

# Start of DanielssonDistanceMapImageFilter_Execute

`DanielssonDistanceMapImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_DanielssonDistanceMapImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DanielssonDistanceMapImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`DanielssonDistanceMapImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__DanielssonDistanceMapImageFilter', '_p_itk__simple__Image')
class(`DanielssonDistanceMapImageFilter_Execute`) = c("SWIGFunction", class('DanielssonDistanceMapImageFilter_Execute'))

# Start of accessor method for itk::simple::DanielssonDistanceMapImageFilter
setMethod('$', '_p_itk__simple__DanielssonDistanceMapImageFilter', function(x, name)

{
  accessorFuns = list('SetInputIsBinary' = DanielssonDistanceMapImageFilter_SetInputIsBinary, 'InputIsBinaryOn' = DanielssonDistanceMapImageFilter_InputIsBinaryOn, 'InputIsBinaryOff' = DanielssonDistanceMapImageFilter_InputIsBinaryOff, 'GetInputIsBinary' = DanielssonDistanceMapImageFilter_GetInputIsBinary, 'SetSquaredDistance' = DanielssonDistanceMapImageFilter_SetSquaredDistance, 'SquaredDistanceOn' = DanielssonDistanceMapImageFilter_SquaredDistanceOn, 'SquaredDistanceOff' = DanielssonDistanceMapImageFilter_SquaredDistanceOff, 'GetSquaredDistance' = DanielssonDistanceMapImageFilter_GetSquaredDistance, 'SetUseImageSpacing' = DanielssonDistanceMapImageFilter_SetUseImageSpacing, 'UseImageSpacingOn' = DanielssonDistanceMapImageFilter_UseImageSpacingOn, 'UseImageSpacingOff' = DanielssonDistanceMapImageFilter_UseImageSpacingOff, 'GetUseImageSpacing' = DanielssonDistanceMapImageFilter_GetUseImageSpacing, 'GetVoronoiMap' = DanielssonDistanceMapImageFilter_GetVoronoiMap, 'GetVectorDistanceMap' = DanielssonDistanceMapImageFilter_GetVectorDistanceMap, 'GetName' = DanielssonDistanceMapImageFilter_GetName, 'ToString' = DanielssonDistanceMapImageFilter_ToString, 'Execute' = DanielssonDistanceMapImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::DanielssonDistanceMapImageFilter
# Start of DanielssonDistanceMap

`DanielssonDistanceMap__SWIG_0` = function(image1, inputIsBinary, squaredDistance, useImageSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  inputIsBinary = as.logical(inputIsBinary);
  squaredDistance = as.logical(squaredDistance);
  useImageSpacing = as.logical(useImageSpacing);
  ;ans = .Call('R_swig_DanielssonDistanceMap__SWIG_0', image1, inputIsBinary, squaredDistance, useImageSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DanielssonDistanceMap__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`DanielssonDistanceMap__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'logical', 'logical')
class(`DanielssonDistanceMap__SWIG_0`) = c("SWIGFunction", class('DanielssonDistanceMap__SWIG_0'))

# Start of DanielssonDistanceMap

`DanielssonDistanceMap__SWIG_1` = function(image1, inputIsBinary, squaredDistance, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  inputIsBinary = as.logical(inputIsBinary);
  squaredDistance = as.logical(squaredDistance);
  ;ans = .Call('R_swig_DanielssonDistanceMap__SWIG_1', image1, inputIsBinary, squaredDistance, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DanielssonDistanceMap__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`DanielssonDistanceMap__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'logical')
class(`DanielssonDistanceMap__SWIG_1`) = c("SWIGFunction", class('DanielssonDistanceMap__SWIG_1'))

# Start of DanielssonDistanceMap

`DanielssonDistanceMap__SWIG_2` = function(image1, inputIsBinary, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  inputIsBinary = as.logical(inputIsBinary);
  ;ans = .Call('R_swig_DanielssonDistanceMap__SWIG_2', image1, inputIsBinary, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DanielssonDistanceMap__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`DanielssonDistanceMap__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`DanielssonDistanceMap__SWIG_2`) = c("SWIGFunction", class('DanielssonDistanceMap__SWIG_2'))

# Start of DanielssonDistanceMap

`DanielssonDistanceMap__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_DanielssonDistanceMap__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DanielssonDistanceMap__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`DanielssonDistanceMap__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`DanielssonDistanceMap__SWIG_3`) = c("SWIGFunction", class('DanielssonDistanceMap__SWIG_3'))

`DanielssonDistanceMap` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- DanielssonDistanceMap__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- DanielssonDistanceMap__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- DanielssonDistanceMap__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- DanielssonDistanceMap__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DanielssonDistanceMap with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_DemonsRegistrationFilter

`delete_DemonsRegistrationFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_DemonsRegistrationFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_DemonsRegistrationFilter`, 'returnType') = 'void'
attr(`delete_DemonsRegistrationFilter`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`delete_DemonsRegistrationFilter`) = c("SWIGFunction", class('delete_DemonsRegistrationFilter'))

# Start of new_DemonsRegistrationFilter

`DemonsRegistrationFilter` = function()
{
  ;ans = .Call('R_swig_new_DemonsRegistrationFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DemonsRegistrationFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DemonsRegistrationFilter);
  ans
  
}

attr(`DemonsRegistrationFilter`, 'returnType') = '_p_itk__simple__DemonsRegistrationFilter'
class(`DemonsRegistrationFilter`) = c("SWIGFunction", class('DemonsRegistrationFilter'))

# Start of DemonsRegistrationFilter_SetStandardDeviations

`DemonsRegistrationFilter_SetStandardDeviations__SWIG_0` = function(self, StandardDeviations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  StandardDeviations = as.numeric(StandardDeviations);
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SetStandardDeviations__SWIG_0', self, StandardDeviations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SetStandardDeviations__SWIG_0`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SetStandardDeviations__SWIG_0`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter', 'numeric')
class(`DemonsRegistrationFilter_SetStandardDeviations__SWIG_0`) = c("SWIGFunction", class('DemonsRegistrationFilter_SetStandardDeviations__SWIG_0'))

# Start of DemonsRegistrationFilter_SetStandardDeviations

`DemonsRegistrationFilter_SetStandardDeviations__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SetStandardDeviations__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SetStandardDeviations__SWIG_1`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SetStandardDeviations__SWIG_1`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter', 'numeric')
class(`DemonsRegistrationFilter_SetStandardDeviations__SWIG_1`) = c("SWIGFunction", class('DemonsRegistrationFilter_SetStandardDeviations__SWIG_1'))

`DemonsRegistrationFilter_SetStandardDeviations` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__DemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- DemonsRegistrationFilter_SetStandardDeviations__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__DemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- DemonsRegistrationFilter_SetStandardDeviations__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DemonsRegistrationFilter_SetStandardDeviations with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of DemonsRegistrationFilter_GetStandardDeviations

`DemonsRegistrationFilter_GetStandardDeviations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_GetStandardDeviations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_GetStandardDeviations`, 'returnType') = 'numeric'
attr(`DemonsRegistrationFilter_GetStandardDeviations`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_GetStandardDeviations`) = c("SWIGFunction", class('DemonsRegistrationFilter_GetStandardDeviations'))

# Start of DemonsRegistrationFilter_SetNumberOfIterations

`DemonsRegistrationFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter', 'integer')
class(`DemonsRegistrationFilter_SetNumberOfIterations`) = c("SWIGFunction", class('DemonsRegistrationFilter_SetNumberOfIterations'))

# Start of DemonsRegistrationFilter_GetNumberOfIterations

`DemonsRegistrationFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`DemonsRegistrationFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_GetNumberOfIterations`) = c("SWIGFunction", class('DemonsRegistrationFilter_GetNumberOfIterations'))

# Start of DemonsRegistrationFilter_SetMaximumRMSError

`DemonsRegistrationFilter_SetMaximumRMSError` = function(self, MaximumRMSError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SetMaximumRMSError', self, MaximumRMSError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SetMaximumRMSError`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter', 'numeric')
class(`DemonsRegistrationFilter_SetMaximumRMSError`) = c("SWIGFunction", class('DemonsRegistrationFilter_SetMaximumRMSError'))

# Start of DemonsRegistrationFilter_GetMaximumRMSError

`DemonsRegistrationFilter_GetMaximumRMSError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_GetMaximumRMSError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_GetMaximumRMSError`, 'returnType') = 'numeric'
attr(`DemonsRegistrationFilter_GetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_GetMaximumRMSError`) = c("SWIGFunction", class('DemonsRegistrationFilter_GetMaximumRMSError'))

# Start of DemonsRegistrationFilter_SetUseMovingImageGradient

`DemonsRegistrationFilter_SetUseMovingImageGradient` = function(self, UseMovingImageGradient)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseMovingImageGradient = as.logical(UseMovingImageGradient);
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SetUseMovingImageGradient', self, UseMovingImageGradient, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SetUseMovingImageGradient`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SetUseMovingImageGradient`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter', 'logical')
class(`DemonsRegistrationFilter_SetUseMovingImageGradient`) = c("SWIGFunction", class('DemonsRegistrationFilter_SetUseMovingImageGradient'))

# Start of DemonsRegistrationFilter_UseMovingImageGradientOn

`DemonsRegistrationFilter_UseMovingImageGradientOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DemonsRegistrationFilter_UseMovingImageGradientOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_UseMovingImageGradientOn`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_UseMovingImageGradientOn`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_UseMovingImageGradientOn`) = c("SWIGFunction", class('DemonsRegistrationFilter_UseMovingImageGradientOn'))

# Start of DemonsRegistrationFilter_UseMovingImageGradientOff

`DemonsRegistrationFilter_UseMovingImageGradientOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DemonsRegistrationFilter_UseMovingImageGradientOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_UseMovingImageGradientOff`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_UseMovingImageGradientOff`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_UseMovingImageGradientOff`) = c("SWIGFunction", class('DemonsRegistrationFilter_UseMovingImageGradientOff'))

# Start of DemonsRegistrationFilter_GetUseMovingImageGradient

`DemonsRegistrationFilter_GetUseMovingImageGradient` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_GetUseMovingImageGradient', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_GetUseMovingImageGradient`, 'returnType') = 'logical'
attr(`DemonsRegistrationFilter_GetUseMovingImageGradient`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_GetUseMovingImageGradient`) = c("SWIGFunction", class('DemonsRegistrationFilter_GetUseMovingImageGradient'))

# Start of DemonsRegistrationFilter_SetSmoothDisplacementField

`DemonsRegistrationFilter_SetSmoothDisplacementField` = function(self, SmoothDisplacementField)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SmoothDisplacementField = as.logical(SmoothDisplacementField);
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SetSmoothDisplacementField', self, SmoothDisplacementField, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SetSmoothDisplacementField`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SetSmoothDisplacementField`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter', 'logical')
class(`DemonsRegistrationFilter_SetSmoothDisplacementField`) = c("SWIGFunction", class('DemonsRegistrationFilter_SetSmoothDisplacementField'))

# Start of DemonsRegistrationFilter_SmoothDisplacementFieldOn

`DemonsRegistrationFilter_SmoothDisplacementFieldOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SmoothDisplacementFieldOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SmoothDisplacementFieldOn`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SmoothDisplacementFieldOn`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_SmoothDisplacementFieldOn`) = c("SWIGFunction", class('DemonsRegistrationFilter_SmoothDisplacementFieldOn'))

# Start of DemonsRegistrationFilter_SmoothDisplacementFieldOff

`DemonsRegistrationFilter_SmoothDisplacementFieldOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SmoothDisplacementFieldOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SmoothDisplacementFieldOff`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SmoothDisplacementFieldOff`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_SmoothDisplacementFieldOff`) = c("SWIGFunction", class('DemonsRegistrationFilter_SmoothDisplacementFieldOff'))

# Start of DemonsRegistrationFilter_GetSmoothDisplacementField

`DemonsRegistrationFilter_GetSmoothDisplacementField` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_GetSmoothDisplacementField', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_GetSmoothDisplacementField`, 'returnType') = 'logical'
attr(`DemonsRegistrationFilter_GetSmoothDisplacementField`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_GetSmoothDisplacementField`) = c("SWIGFunction", class('DemonsRegistrationFilter_GetSmoothDisplacementField'))

# Start of DemonsRegistrationFilter_SetSmoothUpdateField

`DemonsRegistrationFilter_SetSmoothUpdateField` = function(self, SmoothUpdateField)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SmoothUpdateField = as.logical(SmoothUpdateField);
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SetSmoothUpdateField', self, SmoothUpdateField, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SetSmoothUpdateField`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SetSmoothUpdateField`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter', 'logical')
class(`DemonsRegistrationFilter_SetSmoothUpdateField`) = c("SWIGFunction", class('DemonsRegistrationFilter_SetSmoothUpdateField'))

# Start of DemonsRegistrationFilter_SmoothUpdateFieldOn

`DemonsRegistrationFilter_SmoothUpdateFieldOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SmoothUpdateFieldOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SmoothUpdateFieldOn`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SmoothUpdateFieldOn`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_SmoothUpdateFieldOn`) = c("SWIGFunction", class('DemonsRegistrationFilter_SmoothUpdateFieldOn'))

# Start of DemonsRegistrationFilter_SmoothUpdateFieldOff

`DemonsRegistrationFilter_SmoothUpdateFieldOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SmoothUpdateFieldOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SmoothUpdateFieldOff`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SmoothUpdateFieldOff`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_SmoothUpdateFieldOff`) = c("SWIGFunction", class('DemonsRegistrationFilter_SmoothUpdateFieldOff'))

# Start of DemonsRegistrationFilter_GetSmoothUpdateField

`DemonsRegistrationFilter_GetSmoothUpdateField` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_GetSmoothUpdateField', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_GetSmoothUpdateField`, 'returnType') = 'logical'
attr(`DemonsRegistrationFilter_GetSmoothUpdateField`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_GetSmoothUpdateField`) = c("SWIGFunction", class('DemonsRegistrationFilter_GetSmoothUpdateField'))

# Start of DemonsRegistrationFilter_SetUpdateFieldStandardDeviations

`DemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0` = function(self, UpdateFieldStandardDeviations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UpdateFieldStandardDeviations = as.numeric(UpdateFieldStandardDeviations);
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0', self, UpdateFieldStandardDeviations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter', 'numeric')
class(`DemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0`) = c("SWIGFunction", class('DemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0'))

# Start of DemonsRegistrationFilter_SetUpdateFieldStandardDeviations

`DemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter', 'numeric')
class(`DemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1`) = c("SWIGFunction", class('DemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1'))

`DemonsRegistrationFilter_SetUpdateFieldStandardDeviations` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__DemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- DemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__DemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- DemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DemonsRegistrationFilter_SetUpdateFieldStandardDeviations with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of DemonsRegistrationFilter_GetUpdateFieldStandardDeviations

`DemonsRegistrationFilter_GetUpdateFieldStandardDeviations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_GetUpdateFieldStandardDeviations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_GetUpdateFieldStandardDeviations`, 'returnType') = 'numeric'
attr(`DemonsRegistrationFilter_GetUpdateFieldStandardDeviations`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_GetUpdateFieldStandardDeviations`) = c("SWIGFunction", class('DemonsRegistrationFilter_GetUpdateFieldStandardDeviations'))

# Start of DemonsRegistrationFilter_SetMaximumKernelWidth

`DemonsRegistrationFilter_SetMaximumKernelWidth` = function(self, MaximumKernelWidth)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaximumKernelWidth = as.integer(MaximumKernelWidth);
  
  if(length(MaximumKernelWidth) > 1) {
    warning("using only the first element of MaximumKernelWidth");
  };
  
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SetMaximumKernelWidth', self, MaximumKernelWidth, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SetMaximumKernelWidth`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SetMaximumKernelWidth`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter', 'integer')
class(`DemonsRegistrationFilter_SetMaximumKernelWidth`) = c("SWIGFunction", class('DemonsRegistrationFilter_SetMaximumKernelWidth'))

# Start of DemonsRegistrationFilter_GetMaximumKernelWidth

`DemonsRegistrationFilter_GetMaximumKernelWidth` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_GetMaximumKernelWidth', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_GetMaximumKernelWidth`, 'returnType') = 'integer'
attr(`DemonsRegistrationFilter_GetMaximumKernelWidth`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_GetMaximumKernelWidth`) = c("SWIGFunction", class('DemonsRegistrationFilter_GetMaximumKernelWidth'))

# Start of DemonsRegistrationFilter_SetMaximumError

`DemonsRegistrationFilter_SetMaximumError` = function(self, MaximumError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SetMaximumError', self, MaximumError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SetMaximumError`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SetMaximumError`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter', 'numeric')
class(`DemonsRegistrationFilter_SetMaximumError`) = c("SWIGFunction", class('DemonsRegistrationFilter_SetMaximumError'))

# Start of DemonsRegistrationFilter_GetMaximumError

`DemonsRegistrationFilter_GetMaximumError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_GetMaximumError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_GetMaximumError`, 'returnType') = 'numeric'
attr(`DemonsRegistrationFilter_GetMaximumError`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_GetMaximumError`) = c("SWIGFunction", class('DemonsRegistrationFilter_GetMaximumError'))

# Start of DemonsRegistrationFilter_SetIntensityDifferenceThreshold

`DemonsRegistrationFilter_SetIntensityDifferenceThreshold` = function(self, IntensityDifferenceThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SetIntensityDifferenceThreshold', self, IntensityDifferenceThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SetIntensityDifferenceThreshold`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SetIntensityDifferenceThreshold`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter', 'numeric')
class(`DemonsRegistrationFilter_SetIntensityDifferenceThreshold`) = c("SWIGFunction", class('DemonsRegistrationFilter_SetIntensityDifferenceThreshold'))

# Start of DemonsRegistrationFilter_GetIntensityDifferenceThreshold

`DemonsRegistrationFilter_GetIntensityDifferenceThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_GetIntensityDifferenceThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_GetIntensityDifferenceThreshold`, 'returnType') = 'numeric'
attr(`DemonsRegistrationFilter_GetIntensityDifferenceThreshold`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_GetIntensityDifferenceThreshold`) = c("SWIGFunction", class('DemonsRegistrationFilter_GetIntensityDifferenceThreshold'))

# Start of DemonsRegistrationFilter_SetUseImageSpacing

`DemonsRegistrationFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_DemonsRegistrationFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter', 'logical')
class(`DemonsRegistrationFilter_SetUseImageSpacing`) = c("SWIGFunction", class('DemonsRegistrationFilter_SetUseImageSpacing'))

# Start of DemonsRegistrationFilter_UseImageSpacingOn

`DemonsRegistrationFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DemonsRegistrationFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_UseImageSpacingOn`) = c("SWIGFunction", class('DemonsRegistrationFilter_UseImageSpacingOn'))

# Start of DemonsRegistrationFilter_UseImageSpacingOff

`DemonsRegistrationFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DemonsRegistrationFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_UseImageSpacingOff`) = c("SWIGFunction", class('DemonsRegistrationFilter_UseImageSpacingOff'))

# Start of DemonsRegistrationFilter_GetUseImageSpacing

`DemonsRegistrationFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`DemonsRegistrationFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_GetUseImageSpacing`) = c("SWIGFunction", class('DemonsRegistrationFilter_GetUseImageSpacing'))

# Start of DemonsRegistrationFilter_GetElapsedIterations

`DemonsRegistrationFilter_GetElapsedIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_GetElapsedIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_GetElapsedIterations`, 'returnType') = 'integer'
attr(`DemonsRegistrationFilter_GetElapsedIterations`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_GetElapsedIterations`) = c("SWIGFunction", class('DemonsRegistrationFilter_GetElapsedIterations'))

# Start of DemonsRegistrationFilter_GetRMSChange

`DemonsRegistrationFilter_GetRMSChange` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_GetRMSChange', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_GetRMSChange`, 'returnType') = 'numeric'
attr(`DemonsRegistrationFilter_GetRMSChange`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_GetRMSChange`) = c("SWIGFunction", class('DemonsRegistrationFilter_GetRMSChange'))

# Start of DemonsRegistrationFilter_GetMetric

`DemonsRegistrationFilter_GetMetric` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_GetMetric', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_GetMetric`, 'returnType') = 'numeric'
attr(`DemonsRegistrationFilter_GetMetric`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_GetMetric`) = c("SWIGFunction", class('DemonsRegistrationFilter_GetMetric'))

# Start of DemonsRegistrationFilter_StopRegistration

`DemonsRegistrationFilter_StopRegistration` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DemonsRegistrationFilter_StopRegistration', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_StopRegistration`, 'returnType') = 'void'
attr(`DemonsRegistrationFilter_StopRegistration`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_StopRegistration`) = c("SWIGFunction", class('DemonsRegistrationFilter_StopRegistration'))

# Start of DemonsRegistrationFilter_GetName

`DemonsRegistrationFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_GetName`, 'returnType') = 'character'
attr(`DemonsRegistrationFilter_GetName`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_GetName`) = c("SWIGFunction", class('DemonsRegistrationFilter_GetName'))

# Start of DemonsRegistrationFilter_ToString

`DemonsRegistrationFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DemonsRegistrationFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DemonsRegistrationFilter_ToString`, 'returnType') = 'character'
attr(`DemonsRegistrationFilter_ToString`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter')
class(`DemonsRegistrationFilter_ToString`) = c("SWIGFunction", class('DemonsRegistrationFilter_ToString'))

# Start of DemonsRegistrationFilter_Execute

`DemonsRegistrationFilter_Execute__SWIG_0` = function(self, fixedImage, movingImage, initialDisplacementField, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  if (inherits(initialDisplacementField, "ExternalReference")) initialDisplacementField = slot(initialDisplacementField,"ref"); 
  ;ans = .Call('R_swig_DemonsRegistrationFilter_Execute__SWIG_0', self, fixedImage, movingImage, initialDisplacementField, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`DemonsRegistrationFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`DemonsRegistrationFilter_Execute__SWIG_0`) = c("SWIGFunction", class('DemonsRegistrationFilter_Execute__SWIG_0'))

# Start of DemonsRegistrationFilter_Execute

`DemonsRegistrationFilter_Execute__SWIG_1` = function(self, fixedImage, movingImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  ;ans = .Call('R_swig_DemonsRegistrationFilter_Execute__SWIG_1', self, fixedImage, movingImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DemonsRegistrationFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`DemonsRegistrationFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__DemonsRegistrationFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`DemonsRegistrationFilter_Execute__SWIG_1`) = c("SWIGFunction", class('DemonsRegistrationFilter_Execute__SWIG_1'))

`DemonsRegistrationFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__DemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- DemonsRegistrationFilter_Execute__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__DemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- DemonsRegistrationFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DemonsRegistrationFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::DemonsRegistrationFilter
setMethod('$', '_p_itk__simple__DemonsRegistrationFilter', function(x, name)

{
  accessorFuns = list('SetStandardDeviations' = DemonsRegistrationFilter_SetStandardDeviations, 'GetStandardDeviations' = DemonsRegistrationFilter_GetStandardDeviations, 'SetNumberOfIterations' = DemonsRegistrationFilter_SetNumberOfIterations, 'GetNumberOfIterations' = DemonsRegistrationFilter_GetNumberOfIterations, 'SetMaximumRMSError' = DemonsRegistrationFilter_SetMaximumRMSError, 'GetMaximumRMSError' = DemonsRegistrationFilter_GetMaximumRMSError, 'SetUseMovingImageGradient' = DemonsRegistrationFilter_SetUseMovingImageGradient, 'UseMovingImageGradientOn' = DemonsRegistrationFilter_UseMovingImageGradientOn, 'UseMovingImageGradientOff' = DemonsRegistrationFilter_UseMovingImageGradientOff, 'GetUseMovingImageGradient' = DemonsRegistrationFilter_GetUseMovingImageGradient, 'SetSmoothDisplacementField' = DemonsRegistrationFilter_SetSmoothDisplacementField, 'SmoothDisplacementFieldOn' = DemonsRegistrationFilter_SmoothDisplacementFieldOn, 'SmoothDisplacementFieldOff' = DemonsRegistrationFilter_SmoothDisplacementFieldOff, 'GetSmoothDisplacementField' = DemonsRegistrationFilter_GetSmoothDisplacementField, 'SetSmoothUpdateField' = DemonsRegistrationFilter_SetSmoothUpdateField, 'SmoothUpdateFieldOn' = DemonsRegistrationFilter_SmoothUpdateFieldOn, 'SmoothUpdateFieldOff' = DemonsRegistrationFilter_SmoothUpdateFieldOff, 'GetSmoothUpdateField' = DemonsRegistrationFilter_GetSmoothUpdateField, 'SetUpdateFieldStandardDeviations' = DemonsRegistrationFilter_SetUpdateFieldStandardDeviations, 'GetUpdateFieldStandardDeviations' = DemonsRegistrationFilter_GetUpdateFieldStandardDeviations, 'SetMaximumKernelWidth' = DemonsRegistrationFilter_SetMaximumKernelWidth, 'GetMaximumKernelWidth' = DemonsRegistrationFilter_GetMaximumKernelWidth, 'SetMaximumError' = DemonsRegistrationFilter_SetMaximumError, 'GetMaximumError' = DemonsRegistrationFilter_GetMaximumError, 'SetIntensityDifferenceThreshold' = DemonsRegistrationFilter_SetIntensityDifferenceThreshold, 'GetIntensityDifferenceThreshold' = DemonsRegistrationFilter_GetIntensityDifferenceThreshold, 'SetUseImageSpacing' = DemonsRegistrationFilter_SetUseImageSpacing, 'UseImageSpacingOn' = DemonsRegistrationFilter_UseImageSpacingOn, 'UseImageSpacingOff' = DemonsRegistrationFilter_UseImageSpacingOff, 'GetUseImageSpacing' = DemonsRegistrationFilter_GetUseImageSpacing, 'GetElapsedIterations' = DemonsRegistrationFilter_GetElapsedIterations, 'GetRMSChange' = DemonsRegistrationFilter_GetRMSChange, 'GetMetric' = DemonsRegistrationFilter_GetMetric, 'StopRegistration' = DemonsRegistrationFilter_StopRegistration, 'GetName' = DemonsRegistrationFilter_GetName, 'ToString' = DemonsRegistrationFilter_ToString, 'Execute' = DemonsRegistrationFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::DemonsRegistrationFilter
# Start of delete_DerivativeImageFilter

`delete_DerivativeImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_DerivativeImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_DerivativeImageFilter`, 'returnType') = 'void'
attr(`delete_DerivativeImageFilter`, "inputTypes") = c('_p_itk__simple__DerivativeImageFilter')
class(`delete_DerivativeImageFilter`) = c("SWIGFunction", class('delete_DerivativeImageFilter'))

# Start of new_DerivativeImageFilter

`DerivativeImageFilter` = function()
{
  ;ans = .Call('R_swig_new_DerivativeImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DerivativeImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DerivativeImageFilter);
  ans
  
}

attr(`DerivativeImageFilter`, 'returnType') = '_p_itk__simple__DerivativeImageFilter'
class(`DerivativeImageFilter`) = c("SWIGFunction", class('DerivativeImageFilter'))

# Start of DerivativeImageFilter_SetDirection

`DerivativeImageFilter_SetDirection` = function(self, Direction)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Direction = as.integer(Direction);
  
  if(length(Direction) > 1) {
    warning("using only the first element of Direction");
  };
  
  ;ans = .Call('R_swig_DerivativeImageFilter_SetDirection', self, Direction, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DerivativeImageFilter_SetDirection`, 'returnType') = 'void'
attr(`DerivativeImageFilter_SetDirection`, "inputTypes") = c('_p_itk__simple__DerivativeImageFilter', 'integer')
class(`DerivativeImageFilter_SetDirection`) = c("SWIGFunction", class('DerivativeImageFilter_SetDirection'))

# Start of DerivativeImageFilter_GetDirection

`DerivativeImageFilter_GetDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DerivativeImageFilter_GetDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DerivativeImageFilter_GetDirection`, 'returnType') = 'integer'
attr(`DerivativeImageFilter_GetDirection`, "inputTypes") = c('_p_itk__simple__DerivativeImageFilter')
class(`DerivativeImageFilter_GetDirection`) = c("SWIGFunction", class('DerivativeImageFilter_GetDirection'))

# Start of DerivativeImageFilter_SetOrder

`DerivativeImageFilter_SetOrder` = function(self, Order)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Order = as.integer(Order);
  
  if(length(Order) > 1) {
    warning("using only the first element of Order");
  };
  
  ;ans = .Call('R_swig_DerivativeImageFilter_SetOrder', self, Order, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DerivativeImageFilter_SetOrder`, 'returnType') = 'void'
attr(`DerivativeImageFilter_SetOrder`, "inputTypes") = c('_p_itk__simple__DerivativeImageFilter', 'integer')
class(`DerivativeImageFilter_SetOrder`) = c("SWIGFunction", class('DerivativeImageFilter_SetOrder'))

# Start of DerivativeImageFilter_GetOrder

`DerivativeImageFilter_GetOrder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DerivativeImageFilter_GetOrder', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DerivativeImageFilter_GetOrder`, 'returnType') = 'integer'
attr(`DerivativeImageFilter_GetOrder`, "inputTypes") = c('_p_itk__simple__DerivativeImageFilter')
class(`DerivativeImageFilter_GetOrder`) = c("SWIGFunction", class('DerivativeImageFilter_GetOrder'))

# Start of DerivativeImageFilter_SetUseImageSpacing

`DerivativeImageFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_DerivativeImageFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DerivativeImageFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`DerivativeImageFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__DerivativeImageFilter', 'logical')
class(`DerivativeImageFilter_SetUseImageSpacing`) = c("SWIGFunction", class('DerivativeImageFilter_SetUseImageSpacing'))

# Start of DerivativeImageFilter_UseImageSpacingOn

`DerivativeImageFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DerivativeImageFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DerivativeImageFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`DerivativeImageFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__DerivativeImageFilter')
class(`DerivativeImageFilter_UseImageSpacingOn`) = c("SWIGFunction", class('DerivativeImageFilter_UseImageSpacingOn'))

# Start of DerivativeImageFilter_UseImageSpacingOff

`DerivativeImageFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DerivativeImageFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DerivativeImageFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`DerivativeImageFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__DerivativeImageFilter')
class(`DerivativeImageFilter_UseImageSpacingOff`) = c("SWIGFunction", class('DerivativeImageFilter_UseImageSpacingOff'))

# Start of DerivativeImageFilter_GetUseImageSpacing

`DerivativeImageFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DerivativeImageFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DerivativeImageFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`DerivativeImageFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__DerivativeImageFilter')
class(`DerivativeImageFilter_GetUseImageSpacing`) = c("SWIGFunction", class('DerivativeImageFilter_GetUseImageSpacing'))

# Start of DerivativeImageFilter_GetName

`DerivativeImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DerivativeImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DerivativeImageFilter_GetName`, 'returnType') = 'character'
attr(`DerivativeImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__DerivativeImageFilter')
class(`DerivativeImageFilter_GetName`) = c("SWIGFunction", class('DerivativeImageFilter_GetName'))

# Start of DerivativeImageFilter_ToString

`DerivativeImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DerivativeImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DerivativeImageFilter_ToString`, 'returnType') = 'character'
attr(`DerivativeImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__DerivativeImageFilter')
class(`DerivativeImageFilter_ToString`) = c("SWIGFunction", class('DerivativeImageFilter_ToString'))

# Start of DerivativeImageFilter_Execute

`DerivativeImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_DerivativeImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DerivativeImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`DerivativeImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__DerivativeImageFilter', '_p_itk__simple__Image')
class(`DerivativeImageFilter_Execute`) = c("SWIGFunction", class('DerivativeImageFilter_Execute'))

# Start of accessor method for itk::simple::DerivativeImageFilter
setMethod('$', '_p_itk__simple__DerivativeImageFilter', function(x, name)

{
  accessorFuns = list('SetDirection' = DerivativeImageFilter_SetDirection, 'GetDirection' = DerivativeImageFilter_GetDirection, 'SetOrder' = DerivativeImageFilter_SetOrder, 'GetOrder' = DerivativeImageFilter_GetOrder, 'SetUseImageSpacing' = DerivativeImageFilter_SetUseImageSpacing, 'UseImageSpacingOn' = DerivativeImageFilter_UseImageSpacingOn, 'UseImageSpacingOff' = DerivativeImageFilter_UseImageSpacingOff, 'GetUseImageSpacing' = DerivativeImageFilter_GetUseImageSpacing, 'GetName' = DerivativeImageFilter_GetName, 'ToString' = DerivativeImageFilter_ToString, 'Execute' = DerivativeImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::DerivativeImageFilter
# Start of Derivative

`Derivative__SWIG_0` = function(image1, direction, order, useImageSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  direction = as.integer(direction);
  
  if(length(direction) > 1) {
    warning("using only the first element of direction");
  };
  
  order = as.integer(order);
  
  if(length(order) > 1) {
    warning("using only the first element of order");
  };
  
  useImageSpacing = as.logical(useImageSpacing);
  ;ans = .Call('R_swig_Derivative__SWIG_0', image1, direction, order, useImageSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Derivative__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Derivative__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'logical')
class(`Derivative__SWIG_0`) = c("SWIGFunction", class('Derivative__SWIG_0'))

# Start of Derivative

`Derivative__SWIG_1` = function(image1, direction, order, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  direction = as.integer(direction);
  
  if(length(direction) > 1) {
    warning("using only the first element of direction");
  };
  
  order = as.integer(order);
  
  if(length(order) > 1) {
    warning("using only the first element of order");
  };
  
  ;ans = .Call('R_swig_Derivative__SWIG_1', image1, direction, order, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Derivative__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Derivative__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Derivative__SWIG_1`) = c("SWIGFunction", class('Derivative__SWIG_1'))

# Start of Derivative

`Derivative__SWIG_2` = function(image1, direction, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  direction = as.integer(direction);
  
  if(length(direction) > 1) {
    warning("using only the first element of direction");
  };
  
  ;ans = .Call('R_swig_Derivative__SWIG_2', image1, direction, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Derivative__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Derivative__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Derivative__SWIG_2`) = c("SWIGFunction", class('Derivative__SWIG_2'))

# Start of Derivative

`Derivative__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Derivative__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Derivative__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Derivative__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`Derivative__SWIG_3`) = c("SWIGFunction", class('Derivative__SWIG_3'))

`Derivative` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Derivative__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- Derivative__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- Derivative__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- Derivative__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Derivative with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_DiffeomorphicDemonsRegistrationFilter

`delete_DiffeomorphicDemonsRegistrationFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_DiffeomorphicDemonsRegistrationFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_DiffeomorphicDemonsRegistrationFilter`, 'returnType') = 'void'
attr(`delete_DiffeomorphicDemonsRegistrationFilter`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`delete_DiffeomorphicDemonsRegistrationFilter`) = c("SWIGFunction", class('delete_DiffeomorphicDemonsRegistrationFilter'))

# Start of new_DiffeomorphicDemonsRegistrationFilter

`DiffeomorphicDemonsRegistrationFilter` = function()
{
  ;ans = .Call('R_swig_new_DiffeomorphicDemonsRegistrationFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DiffeomorphicDemonsRegistrationFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DiffeomorphicDemonsRegistrationFilter);
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter`, 'returnType') = '_p_itk__simple__DiffeomorphicDemonsRegistrationFilter'
class(`DiffeomorphicDemonsRegistrationFilter`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter'))

# Start of DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations

`DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations__SWIG_0` = function(self, StandardDeviations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  StandardDeviations = as.numeric(StandardDeviations);
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations__SWIG_0', self, StandardDeviations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations__SWIG_0`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations__SWIG_0`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', 'numeric')
class(`DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations__SWIG_0`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations__SWIG_0'))

# Start of DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations

`DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations__SWIG_1`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations__SWIG_1`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', 'numeric')
class(`DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations__SWIG_1`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations__SWIG_1'))

`DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__DiffeomorphicDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__DiffeomorphicDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of DiffeomorphicDemonsRegistrationFilter_GetStandardDeviations

`DiffeomorphicDemonsRegistrationFilter_GetStandardDeviations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetStandardDeviations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetStandardDeviations`, 'returnType') = 'numeric'
attr(`DiffeomorphicDemonsRegistrationFilter_GetStandardDeviations`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetStandardDeviations`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetStandardDeviations'))

# Start of DiffeomorphicDemonsRegistrationFilter_SetNumberOfIterations

`DiffeomorphicDemonsRegistrationFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', 'integer')
class(`DiffeomorphicDemonsRegistrationFilter_SetNumberOfIterations`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SetNumberOfIterations'))

# Start of DiffeomorphicDemonsRegistrationFilter_GetNumberOfIterations

`DiffeomorphicDemonsRegistrationFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`DiffeomorphicDemonsRegistrationFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetNumberOfIterations`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetNumberOfIterations'))

# Start of DiffeomorphicDemonsRegistrationFilter_SetMaximumRMSError

`DiffeomorphicDemonsRegistrationFilter_SetMaximumRMSError` = function(self, MaximumRMSError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SetMaximumRMSError', self, MaximumRMSError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SetMaximumRMSError`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', 'numeric')
class(`DiffeomorphicDemonsRegistrationFilter_SetMaximumRMSError`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SetMaximumRMSError'))

# Start of DiffeomorphicDemonsRegistrationFilter_GetMaximumRMSError

`DiffeomorphicDemonsRegistrationFilter_GetMaximumRMSError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetMaximumRMSError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetMaximumRMSError`, 'returnType') = 'numeric'
attr(`DiffeomorphicDemonsRegistrationFilter_GetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetMaximumRMSError`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetMaximumRMSError'))

# Start of DiffeomorphicDemonsRegistrationFilter_Symmetric_get

`DiffeomorphicDemonsRegistrationFilter_Symmetric_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_Symmetric_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__DiffeomorphicDemonsRegistrationFilter__UseGradientTypeType");
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_Symmetric_get`, 'returnType') = 'character'
class(`DiffeomorphicDemonsRegistrationFilter_Symmetric_get`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_Symmetric_get'))

# Start of DiffeomorphicDemonsRegistrationFilter_Fixed_get

`DiffeomorphicDemonsRegistrationFilter_Fixed_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_Fixed_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__DiffeomorphicDemonsRegistrationFilter__UseGradientTypeType");
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_Fixed_get`, 'returnType') = 'character'
class(`DiffeomorphicDemonsRegistrationFilter_Fixed_get`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_Fixed_get'))

# Start of DiffeomorphicDemonsRegistrationFilter_WarpedMoving_get

`DiffeomorphicDemonsRegistrationFilter_WarpedMoving_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_WarpedMoving_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__DiffeomorphicDemonsRegistrationFilter__UseGradientTypeType");
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_WarpedMoving_get`, 'returnType') = 'character'
class(`DiffeomorphicDemonsRegistrationFilter_WarpedMoving_get`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_WarpedMoving_get'))

# Start of DiffeomorphicDemonsRegistrationFilter_MappedMoving_get

`DiffeomorphicDemonsRegistrationFilter_MappedMoving_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_MappedMoving_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__DiffeomorphicDemonsRegistrationFilter__UseGradientTypeType");
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_MappedMoving_get`, 'returnType') = 'character'
class(`DiffeomorphicDemonsRegistrationFilter_MappedMoving_get`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_MappedMoving_get'))

# Start of DiffeomorphicDemonsRegistrationFilter_SetUseGradientType

`DiffeomorphicDemonsRegistrationFilter_SetUseGradientType` = function(self, UseGradientType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseGradientType = enumToInteger(UseGradientType, "_itk__simple__DiffeomorphicDemonsRegistrationFilter__UseGradientTypeType"); 
  
  if(length(UseGradientType) > 1) {
    warning("using only the first element of UseGradientType");
  };
  
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SetUseGradientType', self, UseGradientType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SetUseGradientType`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SetUseGradientType`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', 'character')
class(`DiffeomorphicDemonsRegistrationFilter_SetUseGradientType`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SetUseGradientType'))

# Start of DiffeomorphicDemonsRegistrationFilter_GetUseGradientType

`DiffeomorphicDemonsRegistrationFilter_GetUseGradientType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetUseGradientType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__DiffeomorphicDemonsRegistrationFilter__UseGradientTypeType");
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetUseGradientType`, 'returnType') = 'character'
attr(`DiffeomorphicDemonsRegistrationFilter_GetUseGradientType`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetUseGradientType`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetUseGradientType'))

# Start of DiffeomorphicDemonsRegistrationFilter_SetUseFirstOrderExp

`DiffeomorphicDemonsRegistrationFilter_SetUseFirstOrderExp` = function(self, UseFirstOrderExp)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseFirstOrderExp = as.logical(UseFirstOrderExp);
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SetUseFirstOrderExp', self, UseFirstOrderExp, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SetUseFirstOrderExp`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SetUseFirstOrderExp`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', 'logical')
class(`DiffeomorphicDemonsRegistrationFilter_SetUseFirstOrderExp`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SetUseFirstOrderExp'))

# Start of DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOn

`DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOn`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOn`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOn`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOn'))

# Start of DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOff

`DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOff`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOff`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOff`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOff'))

# Start of DiffeomorphicDemonsRegistrationFilter_GetUseFirstOrderExp

`DiffeomorphicDemonsRegistrationFilter_GetUseFirstOrderExp` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetUseFirstOrderExp', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetUseFirstOrderExp`, 'returnType') = 'logical'
attr(`DiffeomorphicDemonsRegistrationFilter_GetUseFirstOrderExp`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetUseFirstOrderExp`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetUseFirstOrderExp'))

# Start of DiffeomorphicDemonsRegistrationFilter_SetMaximumUpdateStepLength

`DiffeomorphicDemonsRegistrationFilter_SetMaximumUpdateStepLength` = function(self, MaximumUpdateStepLength)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SetMaximumUpdateStepLength', self, MaximumUpdateStepLength, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SetMaximumUpdateStepLength`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SetMaximumUpdateStepLength`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', 'numeric')
class(`DiffeomorphicDemonsRegistrationFilter_SetMaximumUpdateStepLength`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SetMaximumUpdateStepLength'))

# Start of DiffeomorphicDemonsRegistrationFilter_GetMaximumUpdateStepLength

`DiffeomorphicDemonsRegistrationFilter_GetMaximumUpdateStepLength` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetMaximumUpdateStepLength', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetMaximumUpdateStepLength`, 'returnType') = 'numeric'
attr(`DiffeomorphicDemonsRegistrationFilter_GetMaximumUpdateStepLength`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetMaximumUpdateStepLength`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetMaximumUpdateStepLength'))

# Start of DiffeomorphicDemonsRegistrationFilter_SetSmoothDisplacementField

`DiffeomorphicDemonsRegistrationFilter_SetSmoothDisplacementField` = function(self, SmoothDisplacementField)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SmoothDisplacementField = as.logical(SmoothDisplacementField);
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SetSmoothDisplacementField', self, SmoothDisplacementField, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SetSmoothDisplacementField`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SetSmoothDisplacementField`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', 'logical')
class(`DiffeomorphicDemonsRegistrationFilter_SetSmoothDisplacementField`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SetSmoothDisplacementField'))

# Start of DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOn

`DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOn`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOn`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOn`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOn'))

# Start of DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOff

`DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOff`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOff`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOff`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOff'))

# Start of DiffeomorphicDemonsRegistrationFilter_GetSmoothDisplacementField

`DiffeomorphicDemonsRegistrationFilter_GetSmoothDisplacementField` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetSmoothDisplacementField', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetSmoothDisplacementField`, 'returnType') = 'logical'
attr(`DiffeomorphicDemonsRegistrationFilter_GetSmoothDisplacementField`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetSmoothDisplacementField`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetSmoothDisplacementField'))

# Start of DiffeomorphicDemonsRegistrationFilter_SetSmoothUpdateField

`DiffeomorphicDemonsRegistrationFilter_SetSmoothUpdateField` = function(self, SmoothUpdateField)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SmoothUpdateField = as.logical(SmoothUpdateField);
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SetSmoothUpdateField', self, SmoothUpdateField, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SetSmoothUpdateField`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SetSmoothUpdateField`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', 'logical')
class(`DiffeomorphicDemonsRegistrationFilter_SetSmoothUpdateField`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SetSmoothUpdateField'))

# Start of DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOn

`DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOn`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOn`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOn`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOn'))

# Start of DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOff

`DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOff`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOff`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOff`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOff'))

# Start of DiffeomorphicDemonsRegistrationFilter_GetSmoothUpdateField

`DiffeomorphicDemonsRegistrationFilter_GetSmoothUpdateField` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetSmoothUpdateField', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetSmoothUpdateField`, 'returnType') = 'logical'
attr(`DiffeomorphicDemonsRegistrationFilter_GetSmoothUpdateField`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetSmoothUpdateField`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetSmoothUpdateField'))

# Start of DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations

`DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0` = function(self, UpdateFieldStandardDeviations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UpdateFieldStandardDeviations = as.numeric(UpdateFieldStandardDeviations);
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0', self, UpdateFieldStandardDeviations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', 'numeric')
class(`DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0'))

# Start of DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations

`DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', 'numeric')
class(`DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1'))

`DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__DiffeomorphicDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__DiffeomorphicDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of DiffeomorphicDemonsRegistrationFilter_GetUpdateFieldStandardDeviations

`DiffeomorphicDemonsRegistrationFilter_GetUpdateFieldStandardDeviations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetUpdateFieldStandardDeviations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetUpdateFieldStandardDeviations`, 'returnType') = 'numeric'
attr(`DiffeomorphicDemonsRegistrationFilter_GetUpdateFieldStandardDeviations`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetUpdateFieldStandardDeviations`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetUpdateFieldStandardDeviations'))

# Start of DiffeomorphicDemonsRegistrationFilter_SetMaximumKernelWidth

`DiffeomorphicDemonsRegistrationFilter_SetMaximumKernelWidth` = function(self, MaximumKernelWidth)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaximumKernelWidth = as.integer(MaximumKernelWidth);
  
  if(length(MaximumKernelWidth) > 1) {
    warning("using only the first element of MaximumKernelWidth");
  };
  
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SetMaximumKernelWidth', self, MaximumKernelWidth, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SetMaximumKernelWidth`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SetMaximumKernelWidth`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', 'integer')
class(`DiffeomorphicDemonsRegistrationFilter_SetMaximumKernelWidth`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SetMaximumKernelWidth'))

# Start of DiffeomorphicDemonsRegistrationFilter_GetMaximumKernelWidth

`DiffeomorphicDemonsRegistrationFilter_GetMaximumKernelWidth` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetMaximumKernelWidth', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetMaximumKernelWidth`, 'returnType') = 'integer'
attr(`DiffeomorphicDemonsRegistrationFilter_GetMaximumKernelWidth`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetMaximumKernelWidth`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetMaximumKernelWidth'))

# Start of DiffeomorphicDemonsRegistrationFilter_SetMaximumError

`DiffeomorphicDemonsRegistrationFilter_SetMaximumError` = function(self, MaximumError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SetMaximumError', self, MaximumError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SetMaximumError`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SetMaximumError`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', 'numeric')
class(`DiffeomorphicDemonsRegistrationFilter_SetMaximumError`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SetMaximumError'))

# Start of DiffeomorphicDemonsRegistrationFilter_GetMaximumError

`DiffeomorphicDemonsRegistrationFilter_GetMaximumError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetMaximumError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetMaximumError`, 'returnType') = 'numeric'
attr(`DiffeomorphicDemonsRegistrationFilter_GetMaximumError`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetMaximumError`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetMaximumError'))

# Start of DiffeomorphicDemonsRegistrationFilter_SetIntensityDifferenceThreshold

`DiffeomorphicDemonsRegistrationFilter_SetIntensityDifferenceThreshold` = function(self, IntensityDifferenceThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SetIntensityDifferenceThreshold', self, IntensityDifferenceThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SetIntensityDifferenceThreshold`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SetIntensityDifferenceThreshold`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', 'numeric')
class(`DiffeomorphicDemonsRegistrationFilter_SetIntensityDifferenceThreshold`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SetIntensityDifferenceThreshold'))

# Start of DiffeomorphicDemonsRegistrationFilter_GetIntensityDifferenceThreshold

`DiffeomorphicDemonsRegistrationFilter_GetIntensityDifferenceThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetIntensityDifferenceThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetIntensityDifferenceThreshold`, 'returnType') = 'numeric'
attr(`DiffeomorphicDemonsRegistrationFilter_GetIntensityDifferenceThreshold`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetIntensityDifferenceThreshold`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetIntensityDifferenceThreshold'))

# Start of DiffeomorphicDemonsRegistrationFilter_SetUseImageSpacing

`DiffeomorphicDemonsRegistrationFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', 'logical')
class(`DiffeomorphicDemonsRegistrationFilter_SetUseImageSpacing`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_SetUseImageSpacing'))

# Start of DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOn

`DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOn`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOn'))

# Start of DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOff

`DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOff`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOff'))

# Start of DiffeomorphicDemonsRegistrationFilter_GetUseImageSpacing

`DiffeomorphicDemonsRegistrationFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`DiffeomorphicDemonsRegistrationFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetUseImageSpacing`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetUseImageSpacing'))

# Start of DiffeomorphicDemonsRegistrationFilter_GetElapsedIterations

`DiffeomorphicDemonsRegistrationFilter_GetElapsedIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetElapsedIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetElapsedIterations`, 'returnType') = 'integer'
attr(`DiffeomorphicDemonsRegistrationFilter_GetElapsedIterations`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetElapsedIterations`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetElapsedIterations'))

# Start of DiffeomorphicDemonsRegistrationFilter_GetRMSChange

`DiffeomorphicDemonsRegistrationFilter_GetRMSChange` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetRMSChange', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetRMSChange`, 'returnType') = 'numeric'
attr(`DiffeomorphicDemonsRegistrationFilter_GetRMSChange`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetRMSChange`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetRMSChange'))

# Start of DiffeomorphicDemonsRegistrationFilter_GetMetric

`DiffeomorphicDemonsRegistrationFilter_GetMetric` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetMetric', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetMetric`, 'returnType') = 'numeric'
attr(`DiffeomorphicDemonsRegistrationFilter_GetMetric`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetMetric`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetMetric'))

# Start of DiffeomorphicDemonsRegistrationFilter_StopRegistration

`DiffeomorphicDemonsRegistrationFilter_StopRegistration` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_StopRegistration', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_StopRegistration`, 'returnType') = 'void'
attr(`DiffeomorphicDemonsRegistrationFilter_StopRegistration`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_StopRegistration`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_StopRegistration'))

# Start of DiffeomorphicDemonsRegistrationFilter_GetName

`DiffeomorphicDemonsRegistrationFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_GetName`, 'returnType') = 'character'
attr(`DiffeomorphicDemonsRegistrationFilter_GetName`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_GetName`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_GetName'))

# Start of DiffeomorphicDemonsRegistrationFilter_ToString

`DiffeomorphicDemonsRegistrationFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiffeomorphicDemonsRegistrationFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_ToString`, 'returnType') = 'character'
attr(`DiffeomorphicDemonsRegistrationFilter_ToString`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter')
class(`DiffeomorphicDemonsRegistrationFilter_ToString`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_ToString'))

# Start of DiffeomorphicDemonsRegistrationFilter_Execute

`DiffeomorphicDemonsRegistrationFilter_Execute__SWIG_0` = function(self, fixedImage, movingImage, initialDisplacementField, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  if (inherits(initialDisplacementField, "ExternalReference")) initialDisplacementField = slot(initialDisplacementField,"ref"); 
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_Execute__SWIG_0', self, fixedImage, movingImage, initialDisplacementField, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`DiffeomorphicDemonsRegistrationFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`DiffeomorphicDemonsRegistrationFilter_Execute__SWIG_0`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_Execute__SWIG_0'))

# Start of DiffeomorphicDemonsRegistrationFilter_Execute

`DiffeomorphicDemonsRegistrationFilter_Execute__SWIG_1` = function(self, fixedImage, movingImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  ;ans = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_Execute__SWIG_1', self, fixedImage, movingImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiffeomorphicDemonsRegistrationFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`DiffeomorphicDemonsRegistrationFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`DiffeomorphicDemonsRegistrationFilter_Execute__SWIG_1`) = c("SWIGFunction", class('DiffeomorphicDemonsRegistrationFilter_Execute__SWIG_1'))

`DiffeomorphicDemonsRegistrationFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__DiffeomorphicDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- DiffeomorphicDemonsRegistrationFilter_Execute__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__DiffeomorphicDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- DiffeomorphicDemonsRegistrationFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DiffeomorphicDemonsRegistrationFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::DiffeomorphicDemonsRegistrationFilter
setMethod('$', '_p_itk__simple__DiffeomorphicDemonsRegistrationFilter', function(x, name)

{
  accessorFuns = list('SetStandardDeviations' = DiffeomorphicDemonsRegistrationFilter_SetStandardDeviations, 'GetStandardDeviations' = DiffeomorphicDemonsRegistrationFilter_GetStandardDeviations, 'SetNumberOfIterations' = DiffeomorphicDemonsRegistrationFilter_SetNumberOfIterations, 'GetNumberOfIterations' = DiffeomorphicDemonsRegistrationFilter_GetNumberOfIterations, 'SetMaximumRMSError' = DiffeomorphicDemonsRegistrationFilter_SetMaximumRMSError, 'GetMaximumRMSError' = DiffeomorphicDemonsRegistrationFilter_GetMaximumRMSError, 'SetUseGradientType' = DiffeomorphicDemonsRegistrationFilter_SetUseGradientType, 'GetUseGradientType' = DiffeomorphicDemonsRegistrationFilter_GetUseGradientType, 'SetUseFirstOrderExp' = DiffeomorphicDemonsRegistrationFilter_SetUseFirstOrderExp, 'UseFirstOrderExpOn' = DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOn, 'UseFirstOrderExpOff' = DiffeomorphicDemonsRegistrationFilter_UseFirstOrderExpOff, 'GetUseFirstOrderExp' = DiffeomorphicDemonsRegistrationFilter_GetUseFirstOrderExp, 'SetMaximumUpdateStepLength' = DiffeomorphicDemonsRegistrationFilter_SetMaximumUpdateStepLength, 'GetMaximumUpdateStepLength' = DiffeomorphicDemonsRegistrationFilter_GetMaximumUpdateStepLength, 'SetSmoothDisplacementField' = DiffeomorphicDemonsRegistrationFilter_SetSmoothDisplacementField, 'SmoothDisplacementFieldOn' = DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOn, 'SmoothDisplacementFieldOff' = DiffeomorphicDemonsRegistrationFilter_SmoothDisplacementFieldOff, 'GetSmoothDisplacementField' = DiffeomorphicDemonsRegistrationFilter_GetSmoothDisplacementField, 'SetSmoothUpdateField' = DiffeomorphicDemonsRegistrationFilter_SetSmoothUpdateField, 'SmoothUpdateFieldOn' = DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOn, 'SmoothUpdateFieldOff' = DiffeomorphicDemonsRegistrationFilter_SmoothUpdateFieldOff, 'GetSmoothUpdateField' = DiffeomorphicDemonsRegistrationFilter_GetSmoothUpdateField, 'SetUpdateFieldStandardDeviations' = DiffeomorphicDemonsRegistrationFilter_SetUpdateFieldStandardDeviations, 'GetUpdateFieldStandardDeviations' = DiffeomorphicDemonsRegistrationFilter_GetUpdateFieldStandardDeviations, 'SetMaximumKernelWidth' = DiffeomorphicDemonsRegistrationFilter_SetMaximumKernelWidth, 'GetMaximumKernelWidth' = DiffeomorphicDemonsRegistrationFilter_GetMaximumKernelWidth, 'SetMaximumError' = DiffeomorphicDemonsRegistrationFilter_SetMaximumError, 'GetMaximumError' = DiffeomorphicDemonsRegistrationFilter_GetMaximumError, 'SetIntensityDifferenceThreshold' = DiffeomorphicDemonsRegistrationFilter_SetIntensityDifferenceThreshold, 'GetIntensityDifferenceThreshold' = DiffeomorphicDemonsRegistrationFilter_GetIntensityDifferenceThreshold, 'SetUseImageSpacing' = DiffeomorphicDemonsRegistrationFilter_SetUseImageSpacing, 'UseImageSpacingOn' = DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOn, 'UseImageSpacingOff' = DiffeomorphicDemonsRegistrationFilter_UseImageSpacingOff, 'GetUseImageSpacing' = DiffeomorphicDemonsRegistrationFilter_GetUseImageSpacing, 'GetElapsedIterations' = DiffeomorphicDemonsRegistrationFilter_GetElapsedIterations, 'GetRMSChange' = DiffeomorphicDemonsRegistrationFilter_GetRMSChange, 'GetMetric' = DiffeomorphicDemonsRegistrationFilter_GetMetric, 'StopRegistration' = DiffeomorphicDemonsRegistrationFilter_StopRegistration, 'GetName' = DiffeomorphicDemonsRegistrationFilter_GetName, 'ToString' = DiffeomorphicDemonsRegistrationFilter_ToString, 'Execute' = DiffeomorphicDemonsRegistrationFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::DiffeomorphicDemonsRegistrationFilter
# Start of delete_DilateObjectMorphologyImageFilter

`delete_DilateObjectMorphologyImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_DilateObjectMorphologyImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_DilateObjectMorphologyImageFilter`, 'returnType') = 'void'
attr(`delete_DilateObjectMorphologyImageFilter`, "inputTypes") = c('_p_itk__simple__DilateObjectMorphologyImageFilter')
class(`delete_DilateObjectMorphologyImageFilter`) = c("SWIGFunction", class('delete_DilateObjectMorphologyImageFilter'))

# Start of new_DilateObjectMorphologyImageFilter

`DilateObjectMorphologyImageFilter` = function()
{
  ;ans = .Call('R_swig_new_DilateObjectMorphologyImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DilateObjectMorphologyImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DilateObjectMorphologyImageFilter);
  ans
  
}

attr(`DilateObjectMorphologyImageFilter`, 'returnType') = '_p_itk__simple__DilateObjectMorphologyImageFilter'
class(`DilateObjectMorphologyImageFilter`) = c("SWIGFunction", class('DilateObjectMorphologyImageFilter'))

# Start of DilateObjectMorphologyImageFilter_SetKernelRadius

`DilateObjectMorphologyImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_DilateObjectMorphologyImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DilateObjectMorphologyImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`DilateObjectMorphologyImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__DilateObjectMorphologyImageFilter', 'integer')
class(`DilateObjectMorphologyImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('DilateObjectMorphologyImageFilter_SetKernelRadius__SWIG_0'))

# Start of DilateObjectMorphologyImageFilter_SetKernelRadius

`DilateObjectMorphologyImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_DilateObjectMorphologyImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DilateObjectMorphologyImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`DilateObjectMorphologyImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__DilateObjectMorphologyImageFilter', 'integer')
class(`DilateObjectMorphologyImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('DilateObjectMorphologyImageFilter_SetKernelRadius__SWIG_1'))

`DilateObjectMorphologyImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__DilateObjectMorphologyImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- DilateObjectMorphologyImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__DilateObjectMorphologyImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- DilateObjectMorphologyImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DilateObjectMorphologyImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of DilateObjectMorphologyImageFilter_GetKernelRadius

`DilateObjectMorphologyImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DilateObjectMorphologyImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DilateObjectMorphologyImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`DilateObjectMorphologyImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__DilateObjectMorphologyImageFilter')
class(`DilateObjectMorphologyImageFilter_GetKernelRadius`) = c("SWIGFunction", class('DilateObjectMorphologyImageFilter_GetKernelRadius'))

# Start of DilateObjectMorphologyImageFilter_SetKernelType

`DilateObjectMorphologyImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_DilateObjectMorphologyImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DilateObjectMorphologyImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`DilateObjectMorphologyImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__DilateObjectMorphologyImageFilter', 'character')
class(`DilateObjectMorphologyImageFilter_SetKernelType`) = c("SWIGFunction", class('DilateObjectMorphologyImageFilter_SetKernelType'))

# Start of DilateObjectMorphologyImageFilter_GetKernelType

`DilateObjectMorphologyImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DilateObjectMorphologyImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`DilateObjectMorphologyImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`DilateObjectMorphologyImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__DilateObjectMorphologyImageFilter')
class(`DilateObjectMorphologyImageFilter_GetKernelType`) = c("SWIGFunction", class('DilateObjectMorphologyImageFilter_GetKernelType'))

# Start of DilateObjectMorphologyImageFilter_SetObjectValue

`DilateObjectMorphologyImageFilter_SetObjectValue` = function(self, ObjectValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DilateObjectMorphologyImageFilter_SetObjectValue', self, ObjectValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DilateObjectMorphologyImageFilter_SetObjectValue`, 'returnType') = 'void'
attr(`DilateObjectMorphologyImageFilter_SetObjectValue`, "inputTypes") = c('_p_itk__simple__DilateObjectMorphologyImageFilter', 'numeric')
class(`DilateObjectMorphologyImageFilter_SetObjectValue`) = c("SWIGFunction", class('DilateObjectMorphologyImageFilter_SetObjectValue'))

# Start of DilateObjectMorphologyImageFilter_GetObjectValue

`DilateObjectMorphologyImageFilter_GetObjectValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DilateObjectMorphologyImageFilter_GetObjectValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DilateObjectMorphologyImageFilter_GetObjectValue`, 'returnType') = 'numeric'
attr(`DilateObjectMorphologyImageFilter_GetObjectValue`, "inputTypes") = c('_p_itk__simple__DilateObjectMorphologyImageFilter')
class(`DilateObjectMorphologyImageFilter_GetObjectValue`) = c("SWIGFunction", class('DilateObjectMorphologyImageFilter_GetObjectValue'))

# Start of DilateObjectMorphologyImageFilter_GetName

`DilateObjectMorphologyImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DilateObjectMorphologyImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DilateObjectMorphologyImageFilter_GetName`, 'returnType') = 'character'
attr(`DilateObjectMorphologyImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__DilateObjectMorphologyImageFilter')
class(`DilateObjectMorphologyImageFilter_GetName`) = c("SWIGFunction", class('DilateObjectMorphologyImageFilter_GetName'))

# Start of DilateObjectMorphologyImageFilter_ToString

`DilateObjectMorphologyImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DilateObjectMorphologyImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DilateObjectMorphologyImageFilter_ToString`, 'returnType') = 'character'
attr(`DilateObjectMorphologyImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__DilateObjectMorphologyImageFilter')
class(`DilateObjectMorphologyImageFilter_ToString`) = c("SWIGFunction", class('DilateObjectMorphologyImageFilter_ToString'))

# Start of DilateObjectMorphologyImageFilter_Execute

`DilateObjectMorphologyImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_DilateObjectMorphologyImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DilateObjectMorphologyImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`DilateObjectMorphologyImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__DilateObjectMorphologyImageFilter', '_p_itk__simple__Image')
class(`DilateObjectMorphologyImageFilter_Execute`) = c("SWIGFunction", class('DilateObjectMorphologyImageFilter_Execute'))

# Start of accessor method for itk::simple::DilateObjectMorphologyImageFilter
setMethod('$', '_p_itk__simple__DilateObjectMorphologyImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = DilateObjectMorphologyImageFilter_SetKernelRadius, 'GetKernelRadius' = DilateObjectMorphologyImageFilter_GetKernelRadius, 'SetKernelType' = DilateObjectMorphologyImageFilter_SetKernelType, 'GetKernelType' = DilateObjectMorphologyImageFilter_GetKernelType, 'SetObjectValue' = DilateObjectMorphologyImageFilter_SetObjectValue, 'GetObjectValue' = DilateObjectMorphologyImageFilter_GetObjectValue, 'GetName' = DilateObjectMorphologyImageFilter_GetName, 'ToString' = DilateObjectMorphologyImageFilter_ToString, 'Execute' = DilateObjectMorphologyImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::DilateObjectMorphologyImageFilter
# Start of DilateObjectMorphology

`DilateObjectMorphology__SWIG_0` = function(image1, kernelRadius, kernelType, objectValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  ;ans = .Call('R_swig_DilateObjectMorphology__SWIG_0', image1, kernelRadius, kernelType, objectValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DilateObjectMorphology__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`DilateObjectMorphology__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric')
class(`DilateObjectMorphology__SWIG_0`) = c("SWIGFunction", class('DilateObjectMorphology__SWIG_0'))

# Start of DilateObjectMorphology

`DilateObjectMorphology__SWIG_1` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_DilateObjectMorphology__SWIG_1', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DilateObjectMorphology__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`DilateObjectMorphology__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`DilateObjectMorphology__SWIG_1`) = c("SWIGFunction", class('DilateObjectMorphology__SWIG_1'))

# Start of DilateObjectMorphology

`DilateObjectMorphology__SWIG_2` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_DilateObjectMorphology__SWIG_2', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DilateObjectMorphology__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`DilateObjectMorphology__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`DilateObjectMorphology__SWIG_2`) = c("SWIGFunction", class('DilateObjectMorphology__SWIG_2'))

# Start of DilateObjectMorphology

`DilateObjectMorphology__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_DilateObjectMorphology__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DilateObjectMorphology__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`DilateObjectMorphology__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`DilateObjectMorphology__SWIG_3`) = c("SWIGFunction", class('DilateObjectMorphology__SWIG_3'))

`DilateObjectMorphology` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- DilateObjectMorphology__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- DilateObjectMorphology__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- DilateObjectMorphology__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- DilateObjectMorphology__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DilateObjectMorphology with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_DiscreteGaussianDerivativeImageFilter

`delete_DiscreteGaussianDerivativeImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_DiscreteGaussianDerivativeImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_DiscreteGaussianDerivativeImageFilter`, 'returnType') = 'void'
attr(`delete_DiscreteGaussianDerivativeImageFilter`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter')
class(`delete_DiscreteGaussianDerivativeImageFilter`) = c("SWIGFunction", class('delete_DiscreteGaussianDerivativeImageFilter'))

# Start of new_DiscreteGaussianDerivativeImageFilter

`DiscreteGaussianDerivativeImageFilter` = function()
{
  ;ans = .Call('R_swig_new_DiscreteGaussianDerivativeImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DiscreteGaussianDerivativeImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DiscreteGaussianDerivativeImageFilter);
  ans
  
}

attr(`DiscreteGaussianDerivativeImageFilter`, 'returnType') = '_p_itk__simple__DiscreteGaussianDerivativeImageFilter'
class(`DiscreteGaussianDerivativeImageFilter`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter'))

# Start of DiscreteGaussianDerivativeImageFilter_SetVariance

`DiscreteGaussianDerivativeImageFilter_SetVariance__SWIG_0` = function(self, Variance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Variance = as.numeric(Variance);
  ;ans = .Call('R_swig_DiscreteGaussianDerivativeImageFilter_SetVariance__SWIG_0', self, Variance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivativeImageFilter_SetVariance__SWIG_0`, 'returnType') = 'void'
attr(`DiscreteGaussianDerivativeImageFilter_SetVariance__SWIG_0`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter', 'numeric')
class(`DiscreteGaussianDerivativeImageFilter_SetVariance__SWIG_0`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_SetVariance__SWIG_0'))

# Start of DiscreteGaussianDerivativeImageFilter_SetVariance

`DiscreteGaussianDerivativeImageFilter_SetVariance__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DiscreteGaussianDerivativeImageFilter_SetVariance__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivativeImageFilter_SetVariance__SWIG_1`, 'returnType') = 'void'
attr(`DiscreteGaussianDerivativeImageFilter_SetVariance__SWIG_1`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter', 'numeric')
class(`DiscreteGaussianDerivativeImageFilter_SetVariance__SWIG_1`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_SetVariance__SWIG_1'))

`DiscreteGaussianDerivativeImageFilter_SetVariance` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__DiscreteGaussianDerivativeImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- DiscreteGaussianDerivativeImageFilter_SetVariance__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__DiscreteGaussianDerivativeImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- DiscreteGaussianDerivativeImageFilter_SetVariance__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DiscreteGaussianDerivativeImageFilter_SetVariance with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of DiscreteGaussianDerivativeImageFilter_GetVariance

`DiscreteGaussianDerivativeImageFilter_GetVariance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiscreteGaussianDerivativeImageFilter_GetVariance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiscreteGaussianDerivativeImageFilter_GetVariance`, 'returnType') = 'numeric'
attr(`DiscreteGaussianDerivativeImageFilter_GetVariance`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter')
class(`DiscreteGaussianDerivativeImageFilter_GetVariance`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_GetVariance'))

# Start of DiscreteGaussianDerivativeImageFilter_SetOrder

`DiscreteGaussianDerivativeImageFilter_SetOrder__SWIG_0` = function(self, Order)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Order = as.integer(Order);
  ;ans = .Call('R_swig_DiscreteGaussianDerivativeImageFilter_SetOrder__SWIG_0', self, Order, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivativeImageFilter_SetOrder__SWIG_0`, 'returnType') = 'void'
attr(`DiscreteGaussianDerivativeImageFilter_SetOrder__SWIG_0`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter', 'integer')
class(`DiscreteGaussianDerivativeImageFilter_SetOrder__SWIG_0`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_SetOrder__SWIG_0'))

# Start of DiscreteGaussianDerivativeImageFilter_SetOrder

`DiscreteGaussianDerivativeImageFilter_SetOrder__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_DiscreteGaussianDerivativeImageFilter_SetOrder__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivativeImageFilter_SetOrder__SWIG_1`, 'returnType') = 'void'
attr(`DiscreteGaussianDerivativeImageFilter_SetOrder__SWIG_1`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter', 'integer')
class(`DiscreteGaussianDerivativeImageFilter_SetOrder__SWIG_1`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_SetOrder__SWIG_1'))

`DiscreteGaussianDerivativeImageFilter_SetOrder` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__DiscreteGaussianDerivativeImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- DiscreteGaussianDerivativeImageFilter_SetOrder__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__DiscreteGaussianDerivativeImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- DiscreteGaussianDerivativeImageFilter_SetOrder__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DiscreteGaussianDerivativeImageFilter_SetOrder with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of DiscreteGaussianDerivativeImageFilter_GetOrder

`DiscreteGaussianDerivativeImageFilter_GetOrder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiscreteGaussianDerivativeImageFilter_GetOrder', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiscreteGaussianDerivativeImageFilter_GetOrder`, 'returnType') = 'integer'
attr(`DiscreteGaussianDerivativeImageFilter_GetOrder`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter')
class(`DiscreteGaussianDerivativeImageFilter_GetOrder`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_GetOrder'))

# Start of DiscreteGaussianDerivativeImageFilter_SetMaximumKernelWidth

`DiscreteGaussianDerivativeImageFilter_SetMaximumKernelWidth` = function(self, MaximumKernelWidth)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaximumKernelWidth = as.integer(MaximumKernelWidth);
  
  if(length(MaximumKernelWidth) > 1) {
    warning("using only the first element of MaximumKernelWidth");
  };
  
  ;ans = .Call('R_swig_DiscreteGaussianDerivativeImageFilter_SetMaximumKernelWidth', self, MaximumKernelWidth, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivativeImageFilter_SetMaximumKernelWidth`, 'returnType') = 'void'
attr(`DiscreteGaussianDerivativeImageFilter_SetMaximumKernelWidth`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter', 'integer')
class(`DiscreteGaussianDerivativeImageFilter_SetMaximumKernelWidth`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_SetMaximumKernelWidth'))

# Start of DiscreteGaussianDerivativeImageFilter_GetMaximumKernelWidth

`DiscreteGaussianDerivativeImageFilter_GetMaximumKernelWidth` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiscreteGaussianDerivativeImageFilter_GetMaximumKernelWidth', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiscreteGaussianDerivativeImageFilter_GetMaximumKernelWidth`, 'returnType') = 'integer'
attr(`DiscreteGaussianDerivativeImageFilter_GetMaximumKernelWidth`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter')
class(`DiscreteGaussianDerivativeImageFilter_GetMaximumKernelWidth`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_GetMaximumKernelWidth'))

# Start of DiscreteGaussianDerivativeImageFilter_SetMaximumError

`DiscreteGaussianDerivativeImageFilter_SetMaximumError` = function(self, MaximumError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DiscreteGaussianDerivativeImageFilter_SetMaximumError', self, MaximumError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivativeImageFilter_SetMaximumError`, 'returnType') = 'void'
attr(`DiscreteGaussianDerivativeImageFilter_SetMaximumError`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter', 'numeric')
class(`DiscreteGaussianDerivativeImageFilter_SetMaximumError`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_SetMaximumError'))

# Start of DiscreteGaussianDerivativeImageFilter_GetMaximumError

`DiscreteGaussianDerivativeImageFilter_GetMaximumError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiscreteGaussianDerivativeImageFilter_GetMaximumError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiscreteGaussianDerivativeImageFilter_GetMaximumError`, 'returnType') = 'numeric'
attr(`DiscreteGaussianDerivativeImageFilter_GetMaximumError`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter')
class(`DiscreteGaussianDerivativeImageFilter_GetMaximumError`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_GetMaximumError'))

# Start of DiscreteGaussianDerivativeImageFilter_SetUseImageSpacing

`DiscreteGaussianDerivativeImageFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_DiscreteGaussianDerivativeImageFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivativeImageFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`DiscreteGaussianDerivativeImageFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter', 'logical')
class(`DiscreteGaussianDerivativeImageFilter_SetUseImageSpacing`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_SetUseImageSpacing'))

# Start of DiscreteGaussianDerivativeImageFilter_UseImageSpacingOn

`DiscreteGaussianDerivativeImageFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiscreteGaussianDerivativeImageFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivativeImageFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`DiscreteGaussianDerivativeImageFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter')
class(`DiscreteGaussianDerivativeImageFilter_UseImageSpacingOn`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_UseImageSpacingOn'))

# Start of DiscreteGaussianDerivativeImageFilter_UseImageSpacingOff

`DiscreteGaussianDerivativeImageFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiscreteGaussianDerivativeImageFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivativeImageFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`DiscreteGaussianDerivativeImageFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter')
class(`DiscreteGaussianDerivativeImageFilter_UseImageSpacingOff`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_UseImageSpacingOff'))

# Start of DiscreteGaussianDerivativeImageFilter_GetUseImageSpacing

`DiscreteGaussianDerivativeImageFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiscreteGaussianDerivativeImageFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiscreteGaussianDerivativeImageFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`DiscreteGaussianDerivativeImageFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter')
class(`DiscreteGaussianDerivativeImageFilter_GetUseImageSpacing`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_GetUseImageSpacing'))

# Start of DiscreteGaussianDerivativeImageFilter_SetNormalizeAcrossScale

`DiscreteGaussianDerivativeImageFilter_SetNormalizeAcrossScale` = function(self, NormalizeAcrossScale)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NormalizeAcrossScale = as.logical(NormalizeAcrossScale);
  ;ans = .Call('R_swig_DiscreteGaussianDerivativeImageFilter_SetNormalizeAcrossScale', self, NormalizeAcrossScale, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivativeImageFilter_SetNormalizeAcrossScale`, 'returnType') = 'void'
attr(`DiscreteGaussianDerivativeImageFilter_SetNormalizeAcrossScale`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter', 'logical')
class(`DiscreteGaussianDerivativeImageFilter_SetNormalizeAcrossScale`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_SetNormalizeAcrossScale'))

# Start of DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOn

`DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOn`, 'returnType') = 'void'
attr(`DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOn`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter')
class(`DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOn`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOn'))

# Start of DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOff

`DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOff`, 'returnType') = 'void'
attr(`DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOff`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter')
class(`DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOff`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOff'))

# Start of DiscreteGaussianDerivativeImageFilter_GetNormalizeAcrossScale

`DiscreteGaussianDerivativeImageFilter_GetNormalizeAcrossScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiscreteGaussianDerivativeImageFilter_GetNormalizeAcrossScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiscreteGaussianDerivativeImageFilter_GetNormalizeAcrossScale`, 'returnType') = 'logical'
attr(`DiscreteGaussianDerivativeImageFilter_GetNormalizeAcrossScale`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter')
class(`DiscreteGaussianDerivativeImageFilter_GetNormalizeAcrossScale`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_GetNormalizeAcrossScale'))

# Start of DiscreteGaussianDerivativeImageFilter_GetName

`DiscreteGaussianDerivativeImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiscreteGaussianDerivativeImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiscreteGaussianDerivativeImageFilter_GetName`, 'returnType') = 'character'
attr(`DiscreteGaussianDerivativeImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter')
class(`DiscreteGaussianDerivativeImageFilter_GetName`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_GetName'))

# Start of DiscreteGaussianDerivativeImageFilter_ToString

`DiscreteGaussianDerivativeImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiscreteGaussianDerivativeImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiscreteGaussianDerivativeImageFilter_ToString`, 'returnType') = 'character'
attr(`DiscreteGaussianDerivativeImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter')
class(`DiscreteGaussianDerivativeImageFilter_ToString`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_ToString'))

# Start of DiscreteGaussianDerivativeImageFilter_Execute

`DiscreteGaussianDerivativeImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_DiscreteGaussianDerivativeImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivativeImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussianDerivativeImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianDerivativeImageFilter', '_p_itk__simple__Image')
class(`DiscreteGaussianDerivativeImageFilter_Execute`) = c("SWIGFunction", class('DiscreteGaussianDerivativeImageFilter_Execute'))

# Start of accessor method for itk::simple::DiscreteGaussianDerivativeImageFilter
setMethod('$', '_p_itk__simple__DiscreteGaussianDerivativeImageFilter', function(x, name)

{
  accessorFuns = list('SetVariance' = DiscreteGaussianDerivativeImageFilter_SetVariance, 'GetVariance' = DiscreteGaussianDerivativeImageFilter_GetVariance, 'SetOrder' = DiscreteGaussianDerivativeImageFilter_SetOrder, 'GetOrder' = DiscreteGaussianDerivativeImageFilter_GetOrder, 'SetMaximumKernelWidth' = DiscreteGaussianDerivativeImageFilter_SetMaximumKernelWidth, 'GetMaximumKernelWidth' = DiscreteGaussianDerivativeImageFilter_GetMaximumKernelWidth, 'SetMaximumError' = DiscreteGaussianDerivativeImageFilter_SetMaximumError, 'GetMaximumError' = DiscreteGaussianDerivativeImageFilter_GetMaximumError, 'SetUseImageSpacing' = DiscreteGaussianDerivativeImageFilter_SetUseImageSpacing, 'UseImageSpacingOn' = DiscreteGaussianDerivativeImageFilter_UseImageSpacingOn, 'UseImageSpacingOff' = DiscreteGaussianDerivativeImageFilter_UseImageSpacingOff, 'GetUseImageSpacing' = DiscreteGaussianDerivativeImageFilter_GetUseImageSpacing, 'SetNormalizeAcrossScale' = DiscreteGaussianDerivativeImageFilter_SetNormalizeAcrossScale, 'NormalizeAcrossScaleOn' = DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOn, 'NormalizeAcrossScaleOff' = DiscreteGaussianDerivativeImageFilter_NormalizeAcrossScaleOff, 'GetNormalizeAcrossScale' = DiscreteGaussianDerivativeImageFilter_GetNormalizeAcrossScale, 'GetName' = DiscreteGaussianDerivativeImageFilter_GetName, 'ToString' = DiscreteGaussianDerivativeImageFilter_ToString, 'Execute' = DiscreteGaussianDerivativeImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::DiscreteGaussianDerivativeImageFilter
# Start of DiscreteGaussianDerivative

`DiscreteGaussianDerivative__SWIG_0` = function(image1, variance, order, maximumKernelWidth, maximumError, useImageSpacing, normalizeAcrossScale, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  variance = as.numeric(variance);
  order = as.integer(order);
  maximumKernelWidth = as.integer(maximumKernelWidth);
  
  if(length(maximumKernelWidth) > 1) {
    warning("using only the first element of maximumKernelWidth");
  };
  
  
  useImageSpacing = as.logical(useImageSpacing);
  normalizeAcrossScale = as.logical(normalizeAcrossScale);
  ;ans = .Call('R_swig_DiscreteGaussianDerivative__SWIG_0', image1, variance, order, maximumKernelWidth, maximumError, useImageSpacing, normalizeAcrossScale, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivative__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussianDerivative__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer', 'numeric', 'logical', 'logical')
class(`DiscreteGaussianDerivative__SWIG_0`) = c("SWIGFunction", class('DiscreteGaussianDerivative__SWIG_0'))

# Start of DiscreteGaussianDerivative

`DiscreteGaussianDerivative__SWIG_1` = function(image1, variance, order, maximumKernelWidth, maximumError, useImageSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  variance = as.numeric(variance);
  order = as.integer(order);
  maximumKernelWidth = as.integer(maximumKernelWidth);
  
  if(length(maximumKernelWidth) > 1) {
    warning("using only the first element of maximumKernelWidth");
  };
  
  
  useImageSpacing = as.logical(useImageSpacing);
  ;ans = .Call('R_swig_DiscreteGaussianDerivative__SWIG_1', image1, variance, order, maximumKernelWidth, maximumError, useImageSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivative__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussianDerivative__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer', 'numeric', 'logical')
class(`DiscreteGaussianDerivative__SWIG_1`) = c("SWIGFunction", class('DiscreteGaussianDerivative__SWIG_1'))

# Start of DiscreteGaussianDerivative

`DiscreteGaussianDerivative__SWIG_2` = function(image1, variance, order, maximumKernelWidth, maximumError, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  variance = as.numeric(variance);
  order = as.integer(order);
  maximumKernelWidth = as.integer(maximumKernelWidth);
  
  if(length(maximumKernelWidth) > 1) {
    warning("using only the first element of maximumKernelWidth");
  };
  
  
  ;ans = .Call('R_swig_DiscreteGaussianDerivative__SWIG_2', image1, variance, order, maximumKernelWidth, maximumError, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivative__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussianDerivative__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer', 'numeric')
class(`DiscreteGaussianDerivative__SWIG_2`) = c("SWIGFunction", class('DiscreteGaussianDerivative__SWIG_2'))

# Start of DiscreteGaussianDerivative

`DiscreteGaussianDerivative__SWIG_3` = function(image1, variance, order, maximumKernelWidth, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  variance = as.numeric(variance);
  order = as.integer(order);
  maximumKernelWidth = as.integer(maximumKernelWidth);
  
  if(length(maximumKernelWidth) > 1) {
    warning("using only the first element of maximumKernelWidth");
  };
  
  ;ans = .Call('R_swig_DiscreteGaussianDerivative__SWIG_3', image1, variance, order, maximumKernelWidth, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivative__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussianDerivative__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`DiscreteGaussianDerivative__SWIG_3`) = c("SWIGFunction", class('DiscreteGaussianDerivative__SWIG_3'))

# Start of DiscreteGaussianDerivative

`DiscreteGaussianDerivative__SWIG_4` = function(image1, variance, order, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  variance = as.numeric(variance);
  order = as.integer(order);
  ;ans = .Call('R_swig_DiscreteGaussianDerivative__SWIG_4', image1, variance, order, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivative__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussianDerivative__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`DiscreteGaussianDerivative__SWIG_4`) = c("SWIGFunction", class('DiscreteGaussianDerivative__SWIG_4'))

# Start of DiscreteGaussianDerivative

`DiscreteGaussianDerivative__SWIG_5` = function(image1, variance, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  variance = as.numeric(variance);
  ;ans = .Call('R_swig_DiscreteGaussianDerivative__SWIG_5', image1, variance, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivative__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussianDerivative__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`DiscreteGaussianDerivative__SWIG_5`) = c("SWIGFunction", class('DiscreteGaussianDerivative__SWIG_5'))

# Start of DiscreteGaussianDerivative

`DiscreteGaussianDerivative__SWIG_6` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_DiscreteGaussianDerivative__SWIG_6', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussianDerivative__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussianDerivative__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image')
class(`DiscreteGaussianDerivative__SWIG_6`) = c("SWIGFunction", class('DiscreteGaussianDerivative__SWIG_6'))

`DiscreteGaussianDerivative` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 7
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- DiscreteGaussianDerivative__SWIG_6; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) )) {
      f <- DiscreteGaussianDerivative__SWIG_5; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- DiscreteGaussianDerivative__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- DiscreteGaussianDerivative__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- DiscreteGaussianDerivative__SWIG_2; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- DiscreteGaussianDerivative__SWIG_1; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.logical(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- DiscreteGaussianDerivative__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DiscreteGaussianDerivative with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_DiscreteGaussianImageFilter

`delete_DiscreteGaussianImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_DiscreteGaussianImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_DiscreteGaussianImageFilter`, 'returnType') = 'void'
attr(`delete_DiscreteGaussianImageFilter`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter')
class(`delete_DiscreteGaussianImageFilter`) = c("SWIGFunction", class('delete_DiscreteGaussianImageFilter'))

# Start of new_DiscreteGaussianImageFilter

`DiscreteGaussianImageFilter` = function()
{
  ;ans = .Call('R_swig_new_DiscreteGaussianImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DiscreteGaussianImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DiscreteGaussianImageFilter);
  ans
  
}

attr(`DiscreteGaussianImageFilter`, 'returnType') = '_p_itk__simple__DiscreteGaussianImageFilter'
class(`DiscreteGaussianImageFilter`) = c("SWIGFunction", class('DiscreteGaussianImageFilter'))

# Start of DiscreteGaussianImageFilter_SetVariance

`DiscreteGaussianImageFilter_SetVariance__SWIG_0` = function(self, Variance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Variance = as.numeric(Variance);
  ;ans = .Call('R_swig_DiscreteGaussianImageFilter_SetVariance__SWIG_0', self, Variance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianImageFilter_SetVariance__SWIG_0`, 'returnType') = 'void'
attr(`DiscreteGaussianImageFilter_SetVariance__SWIG_0`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter', 'numeric')
class(`DiscreteGaussianImageFilter_SetVariance__SWIG_0`) = c("SWIGFunction", class('DiscreteGaussianImageFilter_SetVariance__SWIG_0'))

# Start of DiscreteGaussianImageFilter_SetVariance

`DiscreteGaussianImageFilter_SetVariance__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DiscreteGaussianImageFilter_SetVariance__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianImageFilter_SetVariance__SWIG_1`, 'returnType') = 'void'
attr(`DiscreteGaussianImageFilter_SetVariance__SWIG_1`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter', 'numeric')
class(`DiscreteGaussianImageFilter_SetVariance__SWIG_1`) = c("SWIGFunction", class('DiscreteGaussianImageFilter_SetVariance__SWIG_1'))

`DiscreteGaussianImageFilter_SetVariance` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__DiscreteGaussianImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- DiscreteGaussianImageFilter_SetVariance__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__DiscreteGaussianImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- DiscreteGaussianImageFilter_SetVariance__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DiscreteGaussianImageFilter_SetVariance with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of DiscreteGaussianImageFilter_GetVariance

`DiscreteGaussianImageFilter_GetVariance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiscreteGaussianImageFilter_GetVariance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiscreteGaussianImageFilter_GetVariance`, 'returnType') = 'numeric'
attr(`DiscreteGaussianImageFilter_GetVariance`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter')
class(`DiscreteGaussianImageFilter_GetVariance`) = c("SWIGFunction", class('DiscreteGaussianImageFilter_GetVariance'))

# Start of DiscreteGaussianImageFilter_SetMaximumKernelWidth

`DiscreteGaussianImageFilter_SetMaximumKernelWidth` = function(self, MaximumKernelWidth)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaximumKernelWidth = as.integer(MaximumKernelWidth);
  
  if(length(MaximumKernelWidth) > 1) {
    warning("using only the first element of MaximumKernelWidth");
  };
  
  ;ans = .Call('R_swig_DiscreteGaussianImageFilter_SetMaximumKernelWidth', self, MaximumKernelWidth, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianImageFilter_SetMaximumKernelWidth`, 'returnType') = 'void'
attr(`DiscreteGaussianImageFilter_SetMaximumKernelWidth`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter', 'integer')
class(`DiscreteGaussianImageFilter_SetMaximumKernelWidth`) = c("SWIGFunction", class('DiscreteGaussianImageFilter_SetMaximumKernelWidth'))

# Start of DiscreteGaussianImageFilter_GetMaximumKernelWidth

`DiscreteGaussianImageFilter_GetMaximumKernelWidth` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiscreteGaussianImageFilter_GetMaximumKernelWidth', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiscreteGaussianImageFilter_GetMaximumKernelWidth`, 'returnType') = 'integer'
attr(`DiscreteGaussianImageFilter_GetMaximumKernelWidth`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter')
class(`DiscreteGaussianImageFilter_GetMaximumKernelWidth`) = c("SWIGFunction", class('DiscreteGaussianImageFilter_GetMaximumKernelWidth'))

# Start of DiscreteGaussianImageFilter_SetMaximumError

`DiscreteGaussianImageFilter_SetMaximumError__SWIG_0` = function(self, MaximumError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaximumError = as.numeric(MaximumError);
  ;ans = .Call('R_swig_DiscreteGaussianImageFilter_SetMaximumError__SWIG_0', self, MaximumError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianImageFilter_SetMaximumError__SWIG_0`, 'returnType') = 'void'
attr(`DiscreteGaussianImageFilter_SetMaximumError__SWIG_0`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter', 'numeric')
class(`DiscreteGaussianImageFilter_SetMaximumError__SWIG_0`) = c("SWIGFunction", class('DiscreteGaussianImageFilter_SetMaximumError__SWIG_0'))

# Start of DiscreteGaussianImageFilter_SetMaximumError

`DiscreteGaussianImageFilter_SetMaximumError__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DiscreteGaussianImageFilter_SetMaximumError__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianImageFilter_SetMaximumError__SWIG_1`, 'returnType') = 'void'
attr(`DiscreteGaussianImageFilter_SetMaximumError__SWIG_1`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter', 'numeric')
class(`DiscreteGaussianImageFilter_SetMaximumError__SWIG_1`) = c("SWIGFunction", class('DiscreteGaussianImageFilter_SetMaximumError__SWIG_1'))

`DiscreteGaussianImageFilter_SetMaximumError` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__DiscreteGaussianImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- DiscreteGaussianImageFilter_SetMaximumError__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__DiscreteGaussianImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- DiscreteGaussianImageFilter_SetMaximumError__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DiscreteGaussianImageFilter_SetMaximumError with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of DiscreteGaussianImageFilter_GetMaximumError

`DiscreteGaussianImageFilter_GetMaximumError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiscreteGaussianImageFilter_GetMaximumError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiscreteGaussianImageFilter_GetMaximumError`, 'returnType') = 'numeric'
attr(`DiscreteGaussianImageFilter_GetMaximumError`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter')
class(`DiscreteGaussianImageFilter_GetMaximumError`) = c("SWIGFunction", class('DiscreteGaussianImageFilter_GetMaximumError'))

# Start of DiscreteGaussianImageFilter_SetUseImageSpacing

`DiscreteGaussianImageFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_DiscreteGaussianImageFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianImageFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`DiscreteGaussianImageFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter', 'logical')
class(`DiscreteGaussianImageFilter_SetUseImageSpacing`) = c("SWIGFunction", class('DiscreteGaussianImageFilter_SetUseImageSpacing'))

# Start of DiscreteGaussianImageFilter_UseImageSpacingOn

`DiscreteGaussianImageFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiscreteGaussianImageFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianImageFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`DiscreteGaussianImageFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter')
class(`DiscreteGaussianImageFilter_UseImageSpacingOn`) = c("SWIGFunction", class('DiscreteGaussianImageFilter_UseImageSpacingOn'))

# Start of DiscreteGaussianImageFilter_UseImageSpacingOff

`DiscreteGaussianImageFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DiscreteGaussianImageFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DiscreteGaussianImageFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`DiscreteGaussianImageFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter')
class(`DiscreteGaussianImageFilter_UseImageSpacingOff`) = c("SWIGFunction", class('DiscreteGaussianImageFilter_UseImageSpacingOff'))

# Start of DiscreteGaussianImageFilter_GetUseImageSpacing

`DiscreteGaussianImageFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiscreteGaussianImageFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiscreteGaussianImageFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`DiscreteGaussianImageFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter')
class(`DiscreteGaussianImageFilter_GetUseImageSpacing`) = c("SWIGFunction", class('DiscreteGaussianImageFilter_GetUseImageSpacing'))

# Start of DiscreteGaussianImageFilter_GetName

`DiscreteGaussianImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiscreteGaussianImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiscreteGaussianImageFilter_GetName`, 'returnType') = 'character'
attr(`DiscreteGaussianImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter')
class(`DiscreteGaussianImageFilter_GetName`) = c("SWIGFunction", class('DiscreteGaussianImageFilter_GetName'))

# Start of DiscreteGaussianImageFilter_ToString

`DiscreteGaussianImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DiscreteGaussianImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DiscreteGaussianImageFilter_ToString`, 'returnType') = 'character'
attr(`DiscreteGaussianImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter')
class(`DiscreteGaussianImageFilter_ToString`) = c("SWIGFunction", class('DiscreteGaussianImageFilter_ToString'))

# Start of DiscreteGaussianImageFilter_Execute

`DiscreteGaussianImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_DiscreteGaussianImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussianImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussianImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__DiscreteGaussianImageFilter', '_p_itk__simple__Image')
class(`DiscreteGaussianImageFilter_Execute`) = c("SWIGFunction", class('DiscreteGaussianImageFilter_Execute'))

# Start of accessor method for itk::simple::DiscreteGaussianImageFilter
setMethod('$', '_p_itk__simple__DiscreteGaussianImageFilter', function(x, name)

{
  accessorFuns = list('SetVariance' = DiscreteGaussianImageFilter_SetVariance, 'GetVariance' = DiscreteGaussianImageFilter_GetVariance, 'SetMaximumKernelWidth' = DiscreteGaussianImageFilter_SetMaximumKernelWidth, 'GetMaximumKernelWidth' = DiscreteGaussianImageFilter_GetMaximumKernelWidth, 'SetMaximumError' = DiscreteGaussianImageFilter_SetMaximumError, 'GetMaximumError' = DiscreteGaussianImageFilter_GetMaximumError, 'SetUseImageSpacing' = DiscreteGaussianImageFilter_SetUseImageSpacing, 'UseImageSpacingOn' = DiscreteGaussianImageFilter_UseImageSpacingOn, 'UseImageSpacingOff' = DiscreteGaussianImageFilter_UseImageSpacingOff, 'GetUseImageSpacing' = DiscreteGaussianImageFilter_GetUseImageSpacing, 'GetName' = DiscreteGaussianImageFilter_GetName, 'ToString' = DiscreteGaussianImageFilter_ToString, 'Execute' = DiscreteGaussianImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::DiscreteGaussianImageFilter
# Start of DiscreteGaussian

`DiscreteGaussian__SWIG_4` = function(image1, variance, maximumKernelWidth, maximumError, useImageSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  variance = as.numeric(variance);
  maximumKernelWidth = as.integer(maximumKernelWidth);
  
  if(length(maximumKernelWidth) > 1) {
    warning("using only the first element of maximumKernelWidth");
  };
  
  maximumError = as.numeric(maximumError);
  useImageSpacing = as.logical(useImageSpacing);
  ;ans = .Call('R_swig_DiscreteGaussian__SWIG_4', image1, variance, maximumKernelWidth, maximumError, useImageSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussian__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussian__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'logical')
class(`DiscreteGaussian__SWIG_4`) = c("SWIGFunction", class('DiscreteGaussian__SWIG_4'))

# Start of DiscreteGaussian

`DiscreteGaussian__SWIG_5` = function(image1, variance, maximumKernelWidth, maximumError, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  variance = as.numeric(variance);
  maximumKernelWidth = as.integer(maximumKernelWidth);
  
  if(length(maximumKernelWidth) > 1) {
    warning("using only the first element of maximumKernelWidth");
  };
  
  maximumError = as.numeric(maximumError);
  ;ans = .Call('R_swig_DiscreteGaussian__SWIG_5', image1, variance, maximumKernelWidth, maximumError, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussian__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussian__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'numeric')
class(`DiscreteGaussian__SWIG_5`) = c("SWIGFunction", class('DiscreteGaussian__SWIG_5'))

# Start of DiscreteGaussian

`DiscreteGaussian__SWIG_6` = function(image1, variance, maximumKernelWidth, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  variance = as.numeric(variance);
  maximumKernelWidth = as.integer(maximumKernelWidth);
  
  if(length(maximumKernelWidth) > 1) {
    warning("using only the first element of maximumKernelWidth");
  };
  
  ;ans = .Call('R_swig_DiscreteGaussian__SWIG_6', image1, variance, maximumKernelWidth, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussian__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussian__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`DiscreteGaussian__SWIG_6`) = c("SWIGFunction", class('DiscreteGaussian__SWIG_6'))

# Start of DiscreteGaussian

`DiscreteGaussian__SWIG_7` = function(image1, variance, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  variance = as.numeric(variance);
  ;ans = .Call('R_swig_DiscreteGaussian__SWIG_7', image1, variance, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussian__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussian__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`DiscreteGaussian__SWIG_7`) = c("SWIGFunction", class('DiscreteGaussian__SWIG_7'))

# Start of DiscreteGaussian

`DiscreteGaussian__SWIG_8` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_DiscreteGaussian__SWIG_8', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DiscreteGaussian__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`DiscreteGaussian__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image')
class(`DiscreteGaussian__SWIG_8`) = c("SWIGFunction", class('DiscreteGaussian__SWIG_8'))

`DiscreteGaussian` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- DiscreteGaussian__SWIG_8; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- DiscreteGaussian__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) )) {
      f <- DiscreteGaussian__SWIG_7; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- DiscreteGaussian__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- DiscreteGaussian__SWIG_6; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- DiscreteGaussian__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) )) {
      f <- DiscreteGaussian__SWIG_5; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- DiscreteGaussian__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- DiscreteGaussian__SWIG_4; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DiscreteGaussian with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_DisplacementFieldJacobianDeterminantFilter

`delete_DisplacementFieldJacobianDeterminantFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_DisplacementFieldJacobianDeterminantFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_DisplacementFieldJacobianDeterminantFilter`, 'returnType') = 'void'
attr(`delete_DisplacementFieldJacobianDeterminantFilter`, "inputTypes") = c('_p_itk__simple__DisplacementFieldJacobianDeterminantFilter')
class(`delete_DisplacementFieldJacobianDeterminantFilter`) = c("SWIGFunction", class('delete_DisplacementFieldJacobianDeterminantFilter'))

# Start of new_DisplacementFieldJacobianDeterminantFilter

`DisplacementFieldJacobianDeterminantFilter` = function()
{
  ;ans = .Call('R_swig_new_DisplacementFieldJacobianDeterminantFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DisplacementFieldJacobianDeterminantFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DisplacementFieldJacobianDeterminantFilter);
  ans
  
}

attr(`DisplacementFieldJacobianDeterminantFilter`, 'returnType') = '_p_itk__simple__DisplacementFieldJacobianDeterminantFilter'
class(`DisplacementFieldJacobianDeterminantFilter`) = c("SWIGFunction", class('DisplacementFieldJacobianDeterminantFilter'))

# Start of DisplacementFieldJacobianDeterminantFilter_SetUseImageSpacing

`DisplacementFieldJacobianDeterminantFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_DisplacementFieldJacobianDeterminantFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldJacobianDeterminantFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`DisplacementFieldJacobianDeterminantFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__DisplacementFieldJacobianDeterminantFilter', 'logical')
class(`DisplacementFieldJacobianDeterminantFilter_SetUseImageSpacing`) = c("SWIGFunction", class('DisplacementFieldJacobianDeterminantFilter_SetUseImageSpacing'))

# Start of DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOn

`DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__DisplacementFieldJacobianDeterminantFilter')
class(`DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOn`) = c("SWIGFunction", class('DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOn'))

# Start of DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOff

`DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__DisplacementFieldJacobianDeterminantFilter')
class(`DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOff`) = c("SWIGFunction", class('DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOff'))

# Start of DisplacementFieldJacobianDeterminantFilter_GetUseImageSpacing

`DisplacementFieldJacobianDeterminantFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DisplacementFieldJacobianDeterminantFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DisplacementFieldJacobianDeterminantFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`DisplacementFieldJacobianDeterminantFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__DisplacementFieldJacobianDeterminantFilter')
class(`DisplacementFieldJacobianDeterminantFilter_GetUseImageSpacing`) = c("SWIGFunction", class('DisplacementFieldJacobianDeterminantFilter_GetUseImageSpacing'))

# Start of DisplacementFieldJacobianDeterminantFilter_SetDerivativeWeights

`DisplacementFieldJacobianDeterminantFilter_SetDerivativeWeights` = function(self, DerivativeWeights)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  DerivativeWeights = as.numeric(DerivativeWeights);
  ;ans = .Call('R_swig_DisplacementFieldJacobianDeterminantFilter_SetDerivativeWeights', self, DerivativeWeights, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DisplacementFieldJacobianDeterminantFilter_SetDerivativeWeights`, 'returnType') = 'void'
attr(`DisplacementFieldJacobianDeterminantFilter_SetDerivativeWeights`, "inputTypes") = c('_p_itk__simple__DisplacementFieldJacobianDeterminantFilter', 'numeric')
class(`DisplacementFieldJacobianDeterminantFilter_SetDerivativeWeights`) = c("SWIGFunction", class('DisplacementFieldJacobianDeterminantFilter_SetDerivativeWeights'))

# Start of DisplacementFieldJacobianDeterminantFilter_GetDerivativeWeights

`DisplacementFieldJacobianDeterminantFilter_GetDerivativeWeights` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DisplacementFieldJacobianDeterminantFilter_GetDerivativeWeights', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DisplacementFieldJacobianDeterminantFilter_GetDerivativeWeights`, 'returnType') = 'numeric'
attr(`DisplacementFieldJacobianDeterminantFilter_GetDerivativeWeights`, "inputTypes") = c('_p_itk__simple__DisplacementFieldJacobianDeterminantFilter')
class(`DisplacementFieldJacobianDeterminantFilter_GetDerivativeWeights`) = c("SWIGFunction", class('DisplacementFieldJacobianDeterminantFilter_GetDerivativeWeights'))

# Start of DisplacementFieldJacobianDeterminantFilter_GetName

`DisplacementFieldJacobianDeterminantFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DisplacementFieldJacobianDeterminantFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DisplacementFieldJacobianDeterminantFilter_GetName`, 'returnType') = 'character'
attr(`DisplacementFieldJacobianDeterminantFilter_GetName`, "inputTypes") = c('_p_itk__simple__DisplacementFieldJacobianDeterminantFilter')
class(`DisplacementFieldJacobianDeterminantFilter_GetName`) = c("SWIGFunction", class('DisplacementFieldJacobianDeterminantFilter_GetName'))

# Start of DisplacementFieldJacobianDeterminantFilter_ToString

`DisplacementFieldJacobianDeterminantFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DisplacementFieldJacobianDeterminantFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DisplacementFieldJacobianDeterminantFilter_ToString`, 'returnType') = 'character'
attr(`DisplacementFieldJacobianDeterminantFilter_ToString`, "inputTypes") = c('_p_itk__simple__DisplacementFieldJacobianDeterminantFilter')
class(`DisplacementFieldJacobianDeterminantFilter_ToString`) = c("SWIGFunction", class('DisplacementFieldJacobianDeterminantFilter_ToString'))

# Start of DisplacementFieldJacobianDeterminantFilter_Execute

`DisplacementFieldJacobianDeterminantFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_DisplacementFieldJacobianDeterminantFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DisplacementFieldJacobianDeterminantFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`DisplacementFieldJacobianDeterminantFilter_Execute`, "inputTypes") = c('_p_itk__simple__DisplacementFieldJacobianDeterminantFilter', '_p_itk__simple__Image')
class(`DisplacementFieldJacobianDeterminantFilter_Execute`) = c("SWIGFunction", class('DisplacementFieldJacobianDeterminantFilter_Execute'))

# Start of accessor method for itk::simple::DisplacementFieldJacobianDeterminantFilter
setMethod('$', '_p_itk__simple__DisplacementFieldJacobianDeterminantFilter', function(x, name)

{
  accessorFuns = list('SetUseImageSpacing' = DisplacementFieldJacobianDeterminantFilter_SetUseImageSpacing, 'UseImageSpacingOn' = DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOn, 'UseImageSpacingOff' = DisplacementFieldJacobianDeterminantFilter_UseImageSpacingOff, 'GetUseImageSpacing' = DisplacementFieldJacobianDeterminantFilter_GetUseImageSpacing, 'SetDerivativeWeights' = DisplacementFieldJacobianDeterminantFilter_SetDerivativeWeights, 'GetDerivativeWeights' = DisplacementFieldJacobianDeterminantFilter_GetDerivativeWeights, 'GetName' = DisplacementFieldJacobianDeterminantFilter_GetName, 'ToString' = DisplacementFieldJacobianDeterminantFilter_ToString, 'Execute' = DisplacementFieldJacobianDeterminantFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::DisplacementFieldJacobianDeterminantFilter
# Start of DisplacementFieldJacobianDeterminant

`DisplacementFieldJacobianDeterminant__SWIG_0` = function(image1, useImageSpacing, derivativeWeights, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  useImageSpacing = as.logical(useImageSpacing);
  derivativeWeights = as.numeric(derivativeWeights);
  ;ans = .Call('R_swig_DisplacementFieldJacobianDeterminant__SWIG_0', image1, useImageSpacing, derivativeWeights, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DisplacementFieldJacobianDeterminant__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`DisplacementFieldJacobianDeterminant__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'numeric')
class(`DisplacementFieldJacobianDeterminant__SWIG_0`) = c("SWIGFunction", class('DisplacementFieldJacobianDeterminant__SWIG_0'))

# Start of DisplacementFieldJacobianDeterminant

`DisplacementFieldJacobianDeterminant__SWIG_1` = function(image1, useImageSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  useImageSpacing = as.logical(useImageSpacing);
  ;ans = .Call('R_swig_DisplacementFieldJacobianDeterminant__SWIG_1', image1, useImageSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DisplacementFieldJacobianDeterminant__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`DisplacementFieldJacobianDeterminant__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`DisplacementFieldJacobianDeterminant__SWIG_1`) = c("SWIGFunction", class('DisplacementFieldJacobianDeterminant__SWIG_1'))

# Start of DisplacementFieldJacobianDeterminant

`DisplacementFieldJacobianDeterminant__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_DisplacementFieldJacobianDeterminant__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DisplacementFieldJacobianDeterminant__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`DisplacementFieldJacobianDeterminant__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`DisplacementFieldJacobianDeterminant__SWIG_2`) = c("SWIGFunction", class('DisplacementFieldJacobianDeterminant__SWIG_2'))

`DisplacementFieldJacobianDeterminant` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- DisplacementFieldJacobianDeterminant__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- DisplacementFieldJacobianDeterminant__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) )) {
      f <- DisplacementFieldJacobianDeterminant__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DisplacementFieldJacobianDeterminant with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_DivideFloorImageFilter

`delete_DivideFloorImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_DivideFloorImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_DivideFloorImageFilter`, 'returnType') = 'void'
attr(`delete_DivideFloorImageFilter`, "inputTypes") = c('_p_itk__simple__DivideFloorImageFilter')
class(`delete_DivideFloorImageFilter`) = c("SWIGFunction", class('delete_DivideFloorImageFilter'))

# Start of new_DivideFloorImageFilter

`DivideFloorImageFilter` = function()
{
  ;ans = .Call('R_swig_new_DivideFloorImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DivideFloorImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DivideFloorImageFilter);
  ans
  
}

attr(`DivideFloorImageFilter`, 'returnType') = '_p_itk__simple__DivideFloorImageFilter'
class(`DivideFloorImageFilter`) = c("SWIGFunction", class('DivideFloorImageFilter'))

# Start of DivideFloorImageFilter_GetName

`DivideFloorImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DivideFloorImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DivideFloorImageFilter_GetName`, 'returnType') = 'character'
attr(`DivideFloorImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__DivideFloorImageFilter')
class(`DivideFloorImageFilter_GetName`) = c("SWIGFunction", class('DivideFloorImageFilter_GetName'))

# Start of DivideFloorImageFilter_ToString

`DivideFloorImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DivideFloorImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DivideFloorImageFilter_ToString`, 'returnType') = 'character'
attr(`DivideFloorImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__DivideFloorImageFilter')
class(`DivideFloorImageFilter_ToString`) = c("SWIGFunction", class('DivideFloorImageFilter_ToString'))

# Start of DivideFloorImageFilter_Execute

`DivideFloorImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_DivideFloorImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DivideFloorImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`DivideFloorImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__DivideFloorImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`DivideFloorImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('DivideFloorImageFilter_Execute__SWIG_0'))

# Start of DivideFloorImageFilter_Execute

`DivideFloorImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_DivideFloorImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DivideFloorImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`DivideFloorImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__DivideFloorImageFilter', '_p_itk__simple__Image', 'numeric')
class(`DivideFloorImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('DivideFloorImageFilter_Execute__SWIG_1'))

# Start of DivideFloorImageFilter_Execute

`DivideFloorImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_DivideFloorImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DivideFloorImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`DivideFloorImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__DivideFloorImageFilter', 'numeric', '_p_itk__simple__Image')
class(`DivideFloorImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('DivideFloorImageFilter_Execute__SWIG_2'))

`DivideFloorImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__DivideFloorImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- DivideFloorImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__DivideFloorImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- DivideFloorImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__DivideFloorImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- DivideFloorImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DivideFloorImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::DivideFloorImageFilter
setMethod('$', '_p_itk__simple__DivideFloorImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = DivideFloorImageFilter_GetName, 'ToString' = DivideFloorImageFilter_ToString, 'Execute' = DivideFloorImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::DivideFloorImageFilter
# Start of DivideFloor

`DivideFloor__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_DivideFloor__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DivideFloor__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`DivideFloor__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`DivideFloor__SWIG_0`) = c("SWIGFunction", class('DivideFloor__SWIG_0'))

# Start of DivideFloor

`DivideFloor__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_DivideFloor__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DivideFloor__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`DivideFloor__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`DivideFloor__SWIG_1`) = c("SWIGFunction", class('DivideFloor__SWIG_1'))

# Start of DivideFloor

`DivideFloor__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_DivideFloor__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DivideFloor__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`DivideFloor__SWIG_2`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`DivideFloor__SWIG_2`) = c("SWIGFunction", class('DivideFloor__SWIG_2'))

`DivideFloor` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- DivideFloor__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- DivideFloor__SWIG_1; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- DivideFloor__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DivideFloor with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_DivideImageFilter

`delete_DivideImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_DivideImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_DivideImageFilter`, 'returnType') = 'void'
attr(`delete_DivideImageFilter`, "inputTypes") = c('_p_itk__simple__DivideImageFilter')
class(`delete_DivideImageFilter`) = c("SWIGFunction", class('delete_DivideImageFilter'))

# Start of new_DivideImageFilter

`DivideImageFilter` = function()
{
  ;ans = .Call('R_swig_new_DivideImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DivideImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DivideImageFilter);
  ans
  
}

attr(`DivideImageFilter`, 'returnType') = '_p_itk__simple__DivideImageFilter'
class(`DivideImageFilter`) = c("SWIGFunction", class('DivideImageFilter'))

# Start of DivideImageFilter_GetName

`DivideImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DivideImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DivideImageFilter_GetName`, 'returnType') = 'character'
attr(`DivideImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__DivideImageFilter')
class(`DivideImageFilter_GetName`) = c("SWIGFunction", class('DivideImageFilter_GetName'))

# Start of DivideImageFilter_ToString

`DivideImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DivideImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DivideImageFilter_ToString`, 'returnType') = 'character'
attr(`DivideImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__DivideImageFilter')
class(`DivideImageFilter_ToString`) = c("SWIGFunction", class('DivideImageFilter_ToString'))

# Start of DivideImageFilter_Execute

`DivideImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_DivideImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DivideImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`DivideImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__DivideImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`DivideImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('DivideImageFilter_Execute__SWIG_0'))

# Start of DivideImageFilter_Execute

`DivideImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_DivideImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DivideImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`DivideImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__DivideImageFilter', '_p_itk__simple__Image', 'numeric')
class(`DivideImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('DivideImageFilter_Execute__SWIG_1'))

# Start of DivideImageFilter_Execute

`DivideImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_DivideImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DivideImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`DivideImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__DivideImageFilter', 'numeric', '_p_itk__simple__Image')
class(`DivideImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('DivideImageFilter_Execute__SWIG_2'))

`DivideImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__DivideImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- DivideImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__DivideImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- DivideImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__DivideImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- DivideImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DivideImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::DivideImageFilter
setMethod('$', '_p_itk__simple__DivideImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = DivideImageFilter_GetName, 'ToString' = DivideImageFilter_ToString, 'Execute' = DivideImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::DivideImageFilter
# Start of Divide

`Divide__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Divide__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Divide__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Divide__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Divide__SWIG_0`) = c("SWIGFunction", class('Divide__SWIG_0'))

# Start of Divide

`Divide__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Divide__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Divide__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Divide__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`Divide__SWIG_1`) = c("SWIGFunction", class('Divide__SWIG_1'))

# Start of Divide

`Divide__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Divide__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Divide__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Divide__SWIG_2`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`Divide__SWIG_2`) = c("SWIGFunction", class('Divide__SWIG_2'))

`Divide` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Divide__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Divide__SWIG_1; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Divide__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Divide with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_DivideRealImageFilter

`delete_DivideRealImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_DivideRealImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_DivideRealImageFilter`, 'returnType') = 'void'
attr(`delete_DivideRealImageFilter`, "inputTypes") = c('_p_itk__simple__DivideRealImageFilter')
class(`delete_DivideRealImageFilter`) = c("SWIGFunction", class('delete_DivideRealImageFilter'))

# Start of new_DivideRealImageFilter

`DivideRealImageFilter` = function()
{
  ;ans = .Call('R_swig_new_DivideRealImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DivideRealImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DivideRealImageFilter);
  ans
  
}

attr(`DivideRealImageFilter`, 'returnType') = '_p_itk__simple__DivideRealImageFilter'
class(`DivideRealImageFilter`) = c("SWIGFunction", class('DivideRealImageFilter'))

# Start of DivideRealImageFilter_GetName

`DivideRealImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DivideRealImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DivideRealImageFilter_GetName`, 'returnType') = 'character'
attr(`DivideRealImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__DivideRealImageFilter')
class(`DivideRealImageFilter_GetName`) = c("SWIGFunction", class('DivideRealImageFilter_GetName'))

# Start of DivideRealImageFilter_ToString

`DivideRealImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DivideRealImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DivideRealImageFilter_ToString`, 'returnType') = 'character'
attr(`DivideRealImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__DivideRealImageFilter')
class(`DivideRealImageFilter_ToString`) = c("SWIGFunction", class('DivideRealImageFilter_ToString'))

# Start of DivideRealImageFilter_Execute

`DivideRealImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_DivideRealImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DivideRealImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`DivideRealImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__DivideRealImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`DivideRealImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('DivideRealImageFilter_Execute__SWIG_0'))

# Start of DivideRealImageFilter_Execute

`DivideRealImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_DivideRealImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DivideRealImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`DivideRealImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__DivideRealImageFilter', '_p_itk__simple__Image', 'numeric')
class(`DivideRealImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('DivideRealImageFilter_Execute__SWIG_1'))

# Start of DivideRealImageFilter_Execute

`DivideRealImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_DivideRealImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DivideRealImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`DivideRealImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__DivideRealImageFilter', 'numeric', '_p_itk__simple__Image')
class(`DivideRealImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('DivideRealImageFilter_Execute__SWIG_2'))

`DivideRealImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__DivideRealImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- DivideRealImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__DivideRealImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- DivideRealImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__DivideRealImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- DivideRealImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DivideRealImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::DivideRealImageFilter
setMethod('$', '_p_itk__simple__DivideRealImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = DivideRealImageFilter_GetName, 'ToString' = DivideRealImageFilter_ToString, 'Execute' = DivideRealImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::DivideRealImageFilter
# Start of DivideReal

`DivideReal__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_DivideReal__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DivideReal__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`DivideReal__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`DivideReal__SWIG_0`) = c("SWIGFunction", class('DivideReal__SWIG_0'))

# Start of DivideReal

`DivideReal__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_DivideReal__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DivideReal__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`DivideReal__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`DivideReal__SWIG_1`) = c("SWIGFunction", class('DivideReal__SWIG_1'))

# Start of DivideReal

`DivideReal__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_DivideReal__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DivideReal__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`DivideReal__SWIG_2`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`DivideReal__SWIG_2`) = c("SWIGFunction", class('DivideReal__SWIG_2'))

`DivideReal` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- DivideReal__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- DivideReal__SWIG_1; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- DivideReal__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DivideReal with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_DoubleThresholdImageFilter

`delete_DoubleThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_DoubleThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_DoubleThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_DoubleThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter')
class(`delete_DoubleThresholdImageFilter`) = c("SWIGFunction", class('delete_DoubleThresholdImageFilter'))

# Start of new_DoubleThresholdImageFilter

`DoubleThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_DoubleThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__DoubleThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_DoubleThresholdImageFilter);
  ans
  
}

attr(`DoubleThresholdImageFilter`, 'returnType') = '_p_itk__simple__DoubleThresholdImageFilter'
class(`DoubleThresholdImageFilter`) = c("SWIGFunction", class('DoubleThresholdImageFilter'))

# Start of DoubleThresholdImageFilter_SetThreshold1

`DoubleThresholdImageFilter_SetThreshold1` = function(self, Threshold1)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DoubleThresholdImageFilter_SetThreshold1', self, Threshold1, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DoubleThresholdImageFilter_SetThreshold1`, 'returnType') = 'void'
attr(`DoubleThresholdImageFilter_SetThreshold1`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter', 'numeric')
class(`DoubleThresholdImageFilter_SetThreshold1`) = c("SWIGFunction", class('DoubleThresholdImageFilter_SetThreshold1'))

# Start of DoubleThresholdImageFilter_GetThreshold1

`DoubleThresholdImageFilter_GetThreshold1` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DoubleThresholdImageFilter_GetThreshold1', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DoubleThresholdImageFilter_GetThreshold1`, 'returnType') = 'numeric'
attr(`DoubleThresholdImageFilter_GetThreshold1`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter')
class(`DoubleThresholdImageFilter_GetThreshold1`) = c("SWIGFunction", class('DoubleThresholdImageFilter_GetThreshold1'))

# Start of DoubleThresholdImageFilter_SetThreshold2

`DoubleThresholdImageFilter_SetThreshold2` = function(self, Threshold2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DoubleThresholdImageFilter_SetThreshold2', self, Threshold2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DoubleThresholdImageFilter_SetThreshold2`, 'returnType') = 'void'
attr(`DoubleThresholdImageFilter_SetThreshold2`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter', 'numeric')
class(`DoubleThresholdImageFilter_SetThreshold2`) = c("SWIGFunction", class('DoubleThresholdImageFilter_SetThreshold2'))

# Start of DoubleThresholdImageFilter_GetThreshold2

`DoubleThresholdImageFilter_GetThreshold2` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DoubleThresholdImageFilter_GetThreshold2', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DoubleThresholdImageFilter_GetThreshold2`, 'returnType') = 'numeric'
attr(`DoubleThresholdImageFilter_GetThreshold2`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter')
class(`DoubleThresholdImageFilter_GetThreshold2`) = c("SWIGFunction", class('DoubleThresholdImageFilter_GetThreshold2'))

# Start of DoubleThresholdImageFilter_SetThreshold3

`DoubleThresholdImageFilter_SetThreshold3` = function(self, Threshold3)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DoubleThresholdImageFilter_SetThreshold3', self, Threshold3, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DoubleThresholdImageFilter_SetThreshold3`, 'returnType') = 'void'
attr(`DoubleThresholdImageFilter_SetThreshold3`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter', 'numeric')
class(`DoubleThresholdImageFilter_SetThreshold3`) = c("SWIGFunction", class('DoubleThresholdImageFilter_SetThreshold3'))

# Start of DoubleThresholdImageFilter_GetThreshold3

`DoubleThresholdImageFilter_GetThreshold3` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DoubleThresholdImageFilter_GetThreshold3', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DoubleThresholdImageFilter_GetThreshold3`, 'returnType') = 'numeric'
attr(`DoubleThresholdImageFilter_GetThreshold3`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter')
class(`DoubleThresholdImageFilter_GetThreshold3`) = c("SWIGFunction", class('DoubleThresholdImageFilter_GetThreshold3'))

# Start of DoubleThresholdImageFilter_SetThreshold4

`DoubleThresholdImageFilter_SetThreshold4` = function(self, Threshold4)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_DoubleThresholdImageFilter_SetThreshold4', self, Threshold4, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DoubleThresholdImageFilter_SetThreshold4`, 'returnType') = 'void'
attr(`DoubleThresholdImageFilter_SetThreshold4`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter', 'numeric')
class(`DoubleThresholdImageFilter_SetThreshold4`) = c("SWIGFunction", class('DoubleThresholdImageFilter_SetThreshold4'))

# Start of DoubleThresholdImageFilter_GetThreshold4

`DoubleThresholdImageFilter_GetThreshold4` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DoubleThresholdImageFilter_GetThreshold4', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DoubleThresholdImageFilter_GetThreshold4`, 'returnType') = 'numeric'
attr(`DoubleThresholdImageFilter_GetThreshold4`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter')
class(`DoubleThresholdImageFilter_GetThreshold4`) = c("SWIGFunction", class('DoubleThresholdImageFilter_GetThreshold4'))

# Start of DoubleThresholdImageFilter_SetInsideValue

`DoubleThresholdImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideValue = as.integer(InsideValue);
  
  if(length(InsideValue) > 1) {
    warning("using only the first element of InsideValue");
  };
  
  ;ans = .Call('R_swig_DoubleThresholdImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DoubleThresholdImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`DoubleThresholdImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter', 'integer')
class(`DoubleThresholdImageFilter_SetInsideValue`) = c("SWIGFunction", class('DoubleThresholdImageFilter_SetInsideValue'))

# Start of DoubleThresholdImageFilter_GetInsideValue

`DoubleThresholdImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DoubleThresholdImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DoubleThresholdImageFilter_GetInsideValue`, 'returnType') = 'integer'
attr(`DoubleThresholdImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter')
class(`DoubleThresholdImageFilter_GetInsideValue`) = c("SWIGFunction", class('DoubleThresholdImageFilter_GetInsideValue'))

# Start of DoubleThresholdImageFilter_SetOutsideValue

`DoubleThresholdImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutsideValue = as.integer(OutsideValue);
  
  if(length(OutsideValue) > 1) {
    warning("using only the first element of OutsideValue");
  };
  
  ;ans = .Call('R_swig_DoubleThresholdImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DoubleThresholdImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`DoubleThresholdImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter', 'integer')
class(`DoubleThresholdImageFilter_SetOutsideValue`) = c("SWIGFunction", class('DoubleThresholdImageFilter_SetOutsideValue'))

# Start of DoubleThresholdImageFilter_GetOutsideValue

`DoubleThresholdImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DoubleThresholdImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DoubleThresholdImageFilter_GetOutsideValue`, 'returnType') = 'integer'
attr(`DoubleThresholdImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter')
class(`DoubleThresholdImageFilter_GetOutsideValue`) = c("SWIGFunction", class('DoubleThresholdImageFilter_GetOutsideValue'))

# Start of DoubleThresholdImageFilter_SetFullyConnected

`DoubleThresholdImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_DoubleThresholdImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DoubleThresholdImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`DoubleThresholdImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter', 'logical')
class(`DoubleThresholdImageFilter_SetFullyConnected`) = c("SWIGFunction", class('DoubleThresholdImageFilter_SetFullyConnected'))

# Start of DoubleThresholdImageFilter_FullyConnectedOn

`DoubleThresholdImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DoubleThresholdImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DoubleThresholdImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`DoubleThresholdImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter')
class(`DoubleThresholdImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('DoubleThresholdImageFilter_FullyConnectedOn'))

# Start of DoubleThresholdImageFilter_FullyConnectedOff

`DoubleThresholdImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_DoubleThresholdImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`DoubleThresholdImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`DoubleThresholdImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter')
class(`DoubleThresholdImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('DoubleThresholdImageFilter_FullyConnectedOff'))

# Start of DoubleThresholdImageFilter_GetFullyConnected

`DoubleThresholdImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DoubleThresholdImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DoubleThresholdImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`DoubleThresholdImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter')
class(`DoubleThresholdImageFilter_GetFullyConnected`) = c("SWIGFunction", class('DoubleThresholdImageFilter_GetFullyConnected'))

# Start of DoubleThresholdImageFilter_GetName

`DoubleThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DoubleThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DoubleThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`DoubleThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter')
class(`DoubleThresholdImageFilter_GetName`) = c("SWIGFunction", class('DoubleThresholdImageFilter_GetName'))

# Start of DoubleThresholdImageFilter_ToString

`DoubleThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_DoubleThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`DoubleThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`DoubleThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter')
class(`DoubleThresholdImageFilter_ToString`) = c("SWIGFunction", class('DoubleThresholdImageFilter_ToString'))

# Start of DoubleThresholdImageFilter_Execute

`DoubleThresholdImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_DoubleThresholdImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DoubleThresholdImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`DoubleThresholdImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__DoubleThresholdImageFilter', '_p_itk__simple__Image')
class(`DoubleThresholdImageFilter_Execute`) = c("SWIGFunction", class('DoubleThresholdImageFilter_Execute'))

# Start of accessor method for itk::simple::DoubleThresholdImageFilter
setMethod('$', '_p_itk__simple__DoubleThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetThreshold1' = DoubleThresholdImageFilter_SetThreshold1, 'GetThreshold1' = DoubleThresholdImageFilter_GetThreshold1, 'SetThreshold2' = DoubleThresholdImageFilter_SetThreshold2, 'GetThreshold2' = DoubleThresholdImageFilter_GetThreshold2, 'SetThreshold3' = DoubleThresholdImageFilter_SetThreshold3, 'GetThreshold3' = DoubleThresholdImageFilter_GetThreshold3, 'SetThreshold4' = DoubleThresholdImageFilter_SetThreshold4, 'GetThreshold4' = DoubleThresholdImageFilter_GetThreshold4, 'SetInsideValue' = DoubleThresholdImageFilter_SetInsideValue, 'GetInsideValue' = DoubleThresholdImageFilter_GetInsideValue, 'SetOutsideValue' = DoubleThresholdImageFilter_SetOutsideValue, 'GetOutsideValue' = DoubleThresholdImageFilter_GetOutsideValue, 'SetFullyConnected' = DoubleThresholdImageFilter_SetFullyConnected, 'FullyConnectedOn' = DoubleThresholdImageFilter_FullyConnectedOn, 'FullyConnectedOff' = DoubleThresholdImageFilter_FullyConnectedOff, 'GetFullyConnected' = DoubleThresholdImageFilter_GetFullyConnected, 'GetName' = DoubleThresholdImageFilter_GetName, 'ToString' = DoubleThresholdImageFilter_ToString, 'Execute' = DoubleThresholdImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::DoubleThresholdImageFilter
# Start of DoubleThreshold

`DoubleThreshold__SWIG_0` = function(image1, threshold1, threshold2, threshold3, threshold4, insideValue, outsideValue, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  
  
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_DoubleThreshold__SWIG_0', image1, threshold1, threshold2, threshold3, threshold4, insideValue, outsideValue, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DoubleThreshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`DoubleThreshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric', 'integer', 'integer', 'logical')
class(`DoubleThreshold__SWIG_0`) = c("SWIGFunction", class('DoubleThreshold__SWIG_0'))

# Start of DoubleThreshold

`DoubleThreshold__SWIG_1` = function(image1, threshold1, threshold2, threshold3, threshold4, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  
  
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_DoubleThreshold__SWIG_1', image1, threshold1, threshold2, threshold3, threshold4, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DoubleThreshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`DoubleThreshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric', 'integer', 'integer')
class(`DoubleThreshold__SWIG_1`) = c("SWIGFunction", class('DoubleThreshold__SWIG_1'))

# Start of DoubleThreshold

`DoubleThreshold__SWIG_2` = function(image1, threshold1, threshold2, threshold3, threshold4, insideValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  
  
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_DoubleThreshold__SWIG_2', image1, threshold1, threshold2, threshold3, threshold4, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DoubleThreshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`DoubleThreshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric', 'integer')
class(`DoubleThreshold__SWIG_2`) = c("SWIGFunction", class('DoubleThreshold__SWIG_2'))

# Start of DoubleThreshold

`DoubleThreshold__SWIG_3` = function(image1, threshold1, threshold2, threshold3, threshold4, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  
  
  ;ans = .Call('R_swig_DoubleThreshold__SWIG_3', image1, threshold1, threshold2, threshold3, threshold4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DoubleThreshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`DoubleThreshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric')
class(`DoubleThreshold__SWIG_3`) = c("SWIGFunction", class('DoubleThreshold__SWIG_3'))

# Start of DoubleThreshold

`DoubleThreshold__SWIG_4` = function(image1, threshold1, threshold2, threshold3, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  
  ;ans = .Call('R_swig_DoubleThreshold__SWIG_4', image1, threshold1, threshold2, threshold3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DoubleThreshold__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`DoubleThreshold__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric')
class(`DoubleThreshold__SWIG_4`) = c("SWIGFunction", class('DoubleThreshold__SWIG_4'))

# Start of DoubleThreshold

`DoubleThreshold__SWIG_5` = function(image1, threshold1, threshold2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_DoubleThreshold__SWIG_5', image1, threshold1, threshold2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DoubleThreshold__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`DoubleThreshold__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`DoubleThreshold__SWIG_5`) = c("SWIGFunction", class('DoubleThreshold__SWIG_5'))

# Start of DoubleThreshold

`DoubleThreshold__SWIG_6` = function(image1, threshold1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_DoubleThreshold__SWIG_6', image1, threshold1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DoubleThreshold__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`DoubleThreshold__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`DoubleThreshold__SWIG_6`) = c("SWIGFunction", class('DoubleThreshold__SWIG_6'))

# Start of DoubleThreshold

`DoubleThreshold__SWIG_7` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_DoubleThreshold__SWIG_7', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`DoubleThreshold__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`DoubleThreshold__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image')
class(`DoubleThreshold__SWIG_7`) = c("SWIGFunction", class('DoubleThreshold__SWIG_7'))

`DoubleThreshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 8
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- DoubleThreshold__SWIG_7; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- DoubleThreshold__SWIG_6; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- DoubleThreshold__SWIG_5; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- DoubleThreshold__SWIG_4; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- DoubleThreshold__SWIG_3; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- DoubleThreshold__SWIG_2; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- DoubleThreshold__SWIG_1; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 ) && ( is.logical(argv[[8]]) && length(argv[[8]]) == 1 )) {
      f <- DoubleThreshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for DoubleThreshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_EdgePotentialImageFilter

`delete_EdgePotentialImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_EdgePotentialImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_EdgePotentialImageFilter`, 'returnType') = 'void'
attr(`delete_EdgePotentialImageFilter`, "inputTypes") = c('_p_itk__simple__EdgePotentialImageFilter')
class(`delete_EdgePotentialImageFilter`) = c("SWIGFunction", class('delete_EdgePotentialImageFilter'))

# Start of new_EdgePotentialImageFilter

`EdgePotentialImageFilter` = function()
{
  ;ans = .Call('R_swig_new_EdgePotentialImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__EdgePotentialImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_EdgePotentialImageFilter);
  ans
  
}

attr(`EdgePotentialImageFilter`, 'returnType') = '_p_itk__simple__EdgePotentialImageFilter'
class(`EdgePotentialImageFilter`) = c("SWIGFunction", class('EdgePotentialImageFilter'))

# Start of EdgePotentialImageFilter_GetName

`EdgePotentialImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_EdgePotentialImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`EdgePotentialImageFilter_GetName`, 'returnType') = 'character'
attr(`EdgePotentialImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__EdgePotentialImageFilter')
class(`EdgePotentialImageFilter_GetName`) = c("SWIGFunction", class('EdgePotentialImageFilter_GetName'))

# Start of EdgePotentialImageFilter_ToString

`EdgePotentialImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_EdgePotentialImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`EdgePotentialImageFilter_ToString`, 'returnType') = 'character'
attr(`EdgePotentialImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__EdgePotentialImageFilter')
class(`EdgePotentialImageFilter_ToString`) = c("SWIGFunction", class('EdgePotentialImageFilter_ToString'))

# Start of EdgePotentialImageFilter_Execute

`EdgePotentialImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_EdgePotentialImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`EdgePotentialImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`EdgePotentialImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__EdgePotentialImageFilter', '_p_itk__simple__Image')
class(`EdgePotentialImageFilter_Execute`) = c("SWIGFunction", class('EdgePotentialImageFilter_Execute'))

# Start of accessor method for itk::simple::EdgePotentialImageFilter
setMethod('$', '_p_itk__simple__EdgePotentialImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = EdgePotentialImageFilter_GetName, 'ToString' = EdgePotentialImageFilter_ToString, 'Execute' = EdgePotentialImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::EdgePotentialImageFilter
# Start of EdgePotential

`EdgePotential` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_EdgePotential', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`EdgePotential`, 'returnType') = '_p_itk__simple__Image'
attr(`EdgePotential`, "inputTypes") = c('_p_itk__simple__Image')
class(`EdgePotential`) = c("SWIGFunction", class('EdgePotential'))

# Start of delete_EqualImageFilter

`delete_EqualImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_EqualImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_EqualImageFilter`, 'returnType') = 'void'
attr(`delete_EqualImageFilter`, "inputTypes") = c('_p_itk__simple__EqualImageFilter')
class(`delete_EqualImageFilter`) = c("SWIGFunction", class('delete_EqualImageFilter'))

# Start of new_EqualImageFilter

`EqualImageFilter` = function()
{
  ;ans = .Call('R_swig_new_EqualImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__EqualImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_EqualImageFilter);
  ans
  
}

attr(`EqualImageFilter`, 'returnType') = '_p_itk__simple__EqualImageFilter'
class(`EqualImageFilter`) = c("SWIGFunction", class('EqualImageFilter'))

# Start of EqualImageFilter_SetBackgroundValue

`EqualImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BackgroundValue = as.integer(BackgroundValue);
  
  if(length(BackgroundValue) > 1) {
    warning("using only the first element of BackgroundValue");
  };
  
  ;ans = .Call('R_swig_EqualImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`EqualImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`EqualImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__EqualImageFilter', 'integer')
class(`EqualImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('EqualImageFilter_SetBackgroundValue'))

# Start of EqualImageFilter_GetBackgroundValue

`EqualImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_EqualImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`EqualImageFilter_GetBackgroundValue`, 'returnType') = 'integer'
attr(`EqualImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__EqualImageFilter')
class(`EqualImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('EqualImageFilter_GetBackgroundValue'))

# Start of EqualImageFilter_SetForegroundValue

`EqualImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ForegroundValue = as.integer(ForegroundValue);
  
  if(length(ForegroundValue) > 1) {
    warning("using only the first element of ForegroundValue");
  };
  
  ;ans = .Call('R_swig_EqualImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`EqualImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`EqualImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__EqualImageFilter', 'integer')
class(`EqualImageFilter_SetForegroundValue`) = c("SWIGFunction", class('EqualImageFilter_SetForegroundValue'))

# Start of EqualImageFilter_GetForegroundValue

`EqualImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_EqualImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`EqualImageFilter_GetForegroundValue`, 'returnType') = 'integer'
attr(`EqualImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__EqualImageFilter')
class(`EqualImageFilter_GetForegroundValue`) = c("SWIGFunction", class('EqualImageFilter_GetForegroundValue'))

# Start of EqualImageFilter_GetName

`EqualImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_EqualImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`EqualImageFilter_GetName`, 'returnType') = 'character'
attr(`EqualImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__EqualImageFilter')
class(`EqualImageFilter_GetName`) = c("SWIGFunction", class('EqualImageFilter_GetName'))

# Start of EqualImageFilter_ToString

`EqualImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_EqualImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`EqualImageFilter_ToString`, 'returnType') = 'character'
attr(`EqualImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__EqualImageFilter')
class(`EqualImageFilter_ToString`) = c("SWIGFunction", class('EqualImageFilter_ToString'))

# Start of EqualImageFilter_Execute

`EqualImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_EqualImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`EqualImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`EqualImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__EqualImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`EqualImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('EqualImageFilter_Execute__SWIG_0'))

# Start of EqualImageFilter_Execute

`EqualImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_EqualImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`EqualImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`EqualImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__EqualImageFilter', '_p_itk__simple__Image', 'numeric')
class(`EqualImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('EqualImageFilter_Execute__SWIG_1'))

# Start of EqualImageFilter_Execute

`EqualImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_EqualImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`EqualImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`EqualImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__EqualImageFilter', 'numeric', '_p_itk__simple__Image')
class(`EqualImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('EqualImageFilter_Execute__SWIG_2'))

# Start of EqualImageFilter_Execute

`EqualImageFilter_Execute__SWIG_3` = function(self, image1, constant, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_EqualImageFilter_Execute__SWIG_3', self, image1, constant, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`EqualImageFilter_Execute__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`EqualImageFilter_Execute__SWIG_3`, "inputTypes") = c('_p_itk__simple__EqualImageFilter', '_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`EqualImageFilter_Execute__SWIG_3`) = c("SWIGFunction", class('EqualImageFilter_Execute__SWIG_3'))

# Start of EqualImageFilter_Execute

`EqualImageFilter_Execute__SWIG_4` = function(self, constant, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_EqualImageFilter_Execute__SWIG_4', self, constant, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`EqualImageFilter_Execute__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`EqualImageFilter_Execute__SWIG_4`, "inputTypes") = c('_p_itk__simple__EqualImageFilter', 'numeric', '_p_itk__simple__Image', 'integer', 'integer')
class(`EqualImageFilter_Execute__SWIG_4`) = c("SWIGFunction", class('EqualImageFilter_Execute__SWIG_4'))

`EqualImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__EqualImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- EqualImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__EqualImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- EqualImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__EqualImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- EqualImageFilter_Execute__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__EqualImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- EqualImageFilter_Execute__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__EqualImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- EqualImageFilter_Execute__SWIG_4; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for EqualImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::EqualImageFilter
setMethod('$', '_p_itk__simple__EqualImageFilter', function(x, name)

{
  accessorFuns = list('SetBackgroundValue' = EqualImageFilter_SetBackgroundValue, 'GetBackgroundValue' = EqualImageFilter_GetBackgroundValue, 'SetForegroundValue' = EqualImageFilter_SetForegroundValue, 'GetForegroundValue' = EqualImageFilter_GetForegroundValue, 'GetName' = EqualImageFilter_GetName, 'ToString' = EqualImageFilter_ToString, 'Execute' = EqualImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::EqualImageFilter
# Start of Equal

`Equal__SWIG_0` = function(image1, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_Equal__SWIG_0', image1, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Equal__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Equal__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`Equal__SWIG_0`) = c("SWIGFunction", class('Equal__SWIG_0'))

# Start of Equal

`Equal__SWIG_1` = function(image1, image2, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_Equal__SWIG_1', image1, image2, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Equal__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Equal__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`Equal__SWIG_1`) = c("SWIGFunction", class('Equal__SWIG_1'))

# Start of Equal

`Equal__SWIG_2` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Equal__SWIG_2', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Equal__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Equal__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Equal__SWIG_2`) = c("SWIGFunction", class('Equal__SWIG_2'))

# Start of Equal

`Equal__SWIG_3` = function(image1, constant, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_Equal__SWIG_3', image1, constant, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Equal__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Equal__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`Equal__SWIG_3`) = c("SWIGFunction", class('Equal__SWIG_3'))

# Start of Equal

`Equal__SWIG_4` = function(image1, constant, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_Equal__SWIG_4', image1, constant, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Equal__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`Equal__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`Equal__SWIG_4`) = c("SWIGFunction", class('Equal__SWIG_4'))

# Start of Equal

`Equal__SWIG_5` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Equal__SWIG_5', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Equal__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`Equal__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`Equal__SWIG_5`) = c("SWIGFunction", class('Equal__SWIG_5'))

# Start of Equal

`Equal__SWIG_6` = function(constant, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_Equal__SWIG_6', constant, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Equal__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`Equal__SWIG_6`, "inputTypes") = c('numeric', '_p_itk__simple__Image', 'integer', 'integer')
class(`Equal__SWIG_6`) = c("SWIGFunction", class('Equal__SWIG_6'))

# Start of Equal

`Equal__SWIG_7` = function(constant, image2, backgroundValue, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_Equal__SWIG_7', constant, image2, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Equal__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`Equal__SWIG_7`, "inputTypes") = c('numeric', '_p_itk__simple__Image', 'integer')
class(`Equal__SWIG_7`) = c("SWIGFunction", class('Equal__SWIG_7'))

# Start of Equal

`Equal__SWIG_8` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Equal__SWIG_8', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Equal__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`Equal__SWIG_8`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`Equal__SWIG_8`) = c("SWIGFunction", class('Equal__SWIG_8'))

`Equal` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Equal__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Equal__SWIG_5; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Equal__SWIG_8; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- Equal__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- Equal__SWIG_4; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- Equal__SWIG_7; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- Equal__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- Equal__SWIG_3; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- Equal__SWIG_6; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Equal with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ErodeObjectMorphologyImageFilter

`delete_ErodeObjectMorphologyImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ErodeObjectMorphologyImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ErodeObjectMorphologyImageFilter`, 'returnType') = 'void'
attr(`delete_ErodeObjectMorphologyImageFilter`, "inputTypes") = c('_p_itk__simple__ErodeObjectMorphologyImageFilter')
class(`delete_ErodeObjectMorphologyImageFilter`) = c("SWIGFunction", class('delete_ErodeObjectMorphologyImageFilter'))

# Start of new_ErodeObjectMorphologyImageFilter

`ErodeObjectMorphologyImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ErodeObjectMorphologyImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ErodeObjectMorphologyImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ErodeObjectMorphologyImageFilter);
  ans
  
}

attr(`ErodeObjectMorphologyImageFilter`, 'returnType') = '_p_itk__simple__ErodeObjectMorphologyImageFilter'
class(`ErodeObjectMorphologyImageFilter`) = c("SWIGFunction", class('ErodeObjectMorphologyImageFilter'))

# Start of ErodeObjectMorphologyImageFilter_SetKernelRadius

`ErodeObjectMorphologyImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_ErodeObjectMorphologyImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ErodeObjectMorphologyImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`ErodeObjectMorphologyImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__ErodeObjectMorphologyImageFilter', 'integer')
class(`ErodeObjectMorphologyImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('ErodeObjectMorphologyImageFilter_SetKernelRadius__SWIG_0'))

# Start of ErodeObjectMorphologyImageFilter_SetKernelRadius

`ErodeObjectMorphologyImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_ErodeObjectMorphologyImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ErodeObjectMorphologyImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`ErodeObjectMorphologyImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__ErodeObjectMorphologyImageFilter', 'integer')
class(`ErodeObjectMorphologyImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('ErodeObjectMorphologyImageFilter_SetKernelRadius__SWIG_1'))

`ErodeObjectMorphologyImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ErodeObjectMorphologyImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- ErodeObjectMorphologyImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__ErodeObjectMorphologyImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- ErodeObjectMorphologyImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ErodeObjectMorphologyImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ErodeObjectMorphologyImageFilter_GetKernelRadius

`ErodeObjectMorphologyImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ErodeObjectMorphologyImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ErodeObjectMorphologyImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`ErodeObjectMorphologyImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__ErodeObjectMorphologyImageFilter')
class(`ErodeObjectMorphologyImageFilter_GetKernelRadius`) = c("SWIGFunction", class('ErodeObjectMorphologyImageFilter_GetKernelRadius'))

# Start of ErodeObjectMorphologyImageFilter_SetKernelType

`ErodeObjectMorphologyImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_ErodeObjectMorphologyImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ErodeObjectMorphologyImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`ErodeObjectMorphologyImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__ErodeObjectMorphologyImageFilter', 'character')
class(`ErodeObjectMorphologyImageFilter_SetKernelType`) = c("SWIGFunction", class('ErodeObjectMorphologyImageFilter_SetKernelType'))

# Start of ErodeObjectMorphologyImageFilter_GetKernelType

`ErodeObjectMorphologyImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ErodeObjectMorphologyImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`ErodeObjectMorphologyImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`ErodeObjectMorphologyImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__ErodeObjectMorphologyImageFilter')
class(`ErodeObjectMorphologyImageFilter_GetKernelType`) = c("SWIGFunction", class('ErodeObjectMorphologyImageFilter_GetKernelType'))

# Start of ErodeObjectMorphologyImageFilter_SetObjectValue

`ErodeObjectMorphologyImageFilter_SetObjectValue` = function(self, ObjectValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ErodeObjectMorphologyImageFilter_SetObjectValue', self, ObjectValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ErodeObjectMorphologyImageFilter_SetObjectValue`, 'returnType') = 'void'
attr(`ErodeObjectMorphologyImageFilter_SetObjectValue`, "inputTypes") = c('_p_itk__simple__ErodeObjectMorphologyImageFilter', 'numeric')
class(`ErodeObjectMorphologyImageFilter_SetObjectValue`) = c("SWIGFunction", class('ErodeObjectMorphologyImageFilter_SetObjectValue'))

# Start of ErodeObjectMorphologyImageFilter_GetObjectValue

`ErodeObjectMorphologyImageFilter_GetObjectValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ErodeObjectMorphologyImageFilter_GetObjectValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ErodeObjectMorphologyImageFilter_GetObjectValue`, 'returnType') = 'numeric'
attr(`ErodeObjectMorphologyImageFilter_GetObjectValue`, "inputTypes") = c('_p_itk__simple__ErodeObjectMorphologyImageFilter')
class(`ErodeObjectMorphologyImageFilter_GetObjectValue`) = c("SWIGFunction", class('ErodeObjectMorphologyImageFilter_GetObjectValue'))

# Start of ErodeObjectMorphologyImageFilter_SetBackgroundValue

`ErodeObjectMorphologyImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ErodeObjectMorphologyImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ErodeObjectMorphologyImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`ErodeObjectMorphologyImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__ErodeObjectMorphologyImageFilter', 'numeric')
class(`ErodeObjectMorphologyImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('ErodeObjectMorphologyImageFilter_SetBackgroundValue'))

# Start of ErodeObjectMorphologyImageFilter_GetBackgroundValue

`ErodeObjectMorphologyImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ErodeObjectMorphologyImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ErodeObjectMorphologyImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`ErodeObjectMorphologyImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__ErodeObjectMorphologyImageFilter')
class(`ErodeObjectMorphologyImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('ErodeObjectMorphologyImageFilter_GetBackgroundValue'))

# Start of ErodeObjectMorphologyImageFilter_GetName

`ErodeObjectMorphologyImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ErodeObjectMorphologyImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ErodeObjectMorphologyImageFilter_GetName`, 'returnType') = 'character'
attr(`ErodeObjectMorphologyImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ErodeObjectMorphologyImageFilter')
class(`ErodeObjectMorphologyImageFilter_GetName`) = c("SWIGFunction", class('ErodeObjectMorphologyImageFilter_GetName'))

# Start of ErodeObjectMorphologyImageFilter_ToString

`ErodeObjectMorphologyImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ErodeObjectMorphologyImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ErodeObjectMorphologyImageFilter_ToString`, 'returnType') = 'character'
attr(`ErodeObjectMorphologyImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ErodeObjectMorphologyImageFilter')
class(`ErodeObjectMorphologyImageFilter_ToString`) = c("SWIGFunction", class('ErodeObjectMorphologyImageFilter_ToString'))

# Start of ErodeObjectMorphologyImageFilter_Execute

`ErodeObjectMorphologyImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ErodeObjectMorphologyImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ErodeObjectMorphologyImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ErodeObjectMorphologyImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ErodeObjectMorphologyImageFilter', '_p_itk__simple__Image')
class(`ErodeObjectMorphologyImageFilter_Execute`) = c("SWIGFunction", class('ErodeObjectMorphologyImageFilter_Execute'))

# Start of accessor method for itk::simple::ErodeObjectMorphologyImageFilter
setMethod('$', '_p_itk__simple__ErodeObjectMorphologyImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = ErodeObjectMorphologyImageFilter_SetKernelRadius, 'GetKernelRadius' = ErodeObjectMorphologyImageFilter_GetKernelRadius, 'SetKernelType' = ErodeObjectMorphologyImageFilter_SetKernelType, 'GetKernelType' = ErodeObjectMorphologyImageFilter_GetKernelType, 'SetObjectValue' = ErodeObjectMorphologyImageFilter_SetObjectValue, 'GetObjectValue' = ErodeObjectMorphologyImageFilter_GetObjectValue, 'SetBackgroundValue' = ErodeObjectMorphologyImageFilter_SetBackgroundValue, 'GetBackgroundValue' = ErodeObjectMorphologyImageFilter_GetBackgroundValue, 'GetName' = ErodeObjectMorphologyImageFilter_GetName, 'ToString' = ErodeObjectMorphologyImageFilter_ToString, 'Execute' = ErodeObjectMorphologyImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ErodeObjectMorphologyImageFilter
# Start of ErodeObjectMorphology

`ErodeObjectMorphology__SWIG_0` = function(image1, kernelRadius, kernelType, objectValue, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  
  ;ans = .Call('R_swig_ErodeObjectMorphology__SWIG_0', image1, kernelRadius, kernelType, objectValue, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ErodeObjectMorphology__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ErodeObjectMorphology__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric', 'numeric')
class(`ErodeObjectMorphology__SWIG_0`) = c("SWIGFunction", class('ErodeObjectMorphology__SWIG_0'))

# Start of ErodeObjectMorphology

`ErodeObjectMorphology__SWIG_1` = function(image1, kernelRadius, kernelType, objectValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  
  ;ans = .Call('R_swig_ErodeObjectMorphology__SWIG_1', image1, kernelRadius, kernelType, objectValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ErodeObjectMorphology__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ErodeObjectMorphology__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'numeric')
class(`ErodeObjectMorphology__SWIG_1`) = c("SWIGFunction", class('ErodeObjectMorphology__SWIG_1'))

# Start of ErodeObjectMorphology

`ErodeObjectMorphology__SWIG_2` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_ErodeObjectMorphology__SWIG_2', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ErodeObjectMorphology__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ErodeObjectMorphology__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`ErodeObjectMorphology__SWIG_2`) = c("SWIGFunction", class('ErodeObjectMorphology__SWIG_2'))

# Start of ErodeObjectMorphology

`ErodeObjectMorphology__SWIG_3` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_ErodeObjectMorphology__SWIG_3', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ErodeObjectMorphology__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ErodeObjectMorphology__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`ErodeObjectMorphology__SWIG_3`) = c("SWIGFunction", class('ErodeObjectMorphology__SWIG_3'))

# Start of ErodeObjectMorphology

`ErodeObjectMorphology__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ErodeObjectMorphology__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ErodeObjectMorphology__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ErodeObjectMorphology__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`ErodeObjectMorphology__SWIG_4`) = c("SWIGFunction", class('ErodeObjectMorphology__SWIG_4'))

`ErodeObjectMorphology` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ErodeObjectMorphology__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- ErodeObjectMorphology__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ErodeObjectMorphology__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ErodeObjectMorphology__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- ErodeObjectMorphology__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ErodeObjectMorphology with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ExpImageFilter

`delete_ExpImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ExpImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ExpImageFilter`, 'returnType') = 'void'
attr(`delete_ExpImageFilter`, "inputTypes") = c('_p_itk__simple__ExpImageFilter')
class(`delete_ExpImageFilter`) = c("SWIGFunction", class('delete_ExpImageFilter'))

# Start of new_ExpImageFilter

`ExpImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ExpImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ExpImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ExpImageFilter);
  ans
  
}

attr(`ExpImageFilter`, 'returnType') = '_p_itk__simple__ExpImageFilter'
class(`ExpImageFilter`) = c("SWIGFunction", class('ExpImageFilter'))

# Start of ExpImageFilter_GetName

`ExpImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ExpImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ExpImageFilter_GetName`, 'returnType') = 'character'
attr(`ExpImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ExpImageFilter')
class(`ExpImageFilter_GetName`) = c("SWIGFunction", class('ExpImageFilter_GetName'))

# Start of ExpImageFilter_ToString

`ExpImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ExpImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ExpImageFilter_ToString`, 'returnType') = 'character'
attr(`ExpImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ExpImageFilter')
class(`ExpImageFilter_ToString`) = c("SWIGFunction", class('ExpImageFilter_ToString'))

# Start of ExpImageFilter_Execute

`ExpImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ExpImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ExpImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ExpImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ExpImageFilter', '_p_itk__simple__Image')
class(`ExpImageFilter_Execute`) = c("SWIGFunction", class('ExpImageFilter_Execute'))

# Start of accessor method for itk::simple::ExpImageFilter
setMethod('$', '_p_itk__simple__ExpImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = ExpImageFilter_GetName, 'ToString' = ExpImageFilter_ToString, 'Execute' = ExpImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ExpImageFilter
# Start of Exp

`Exp` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Exp', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Exp`, 'returnType') = '_p_itk__simple__Image'
attr(`Exp`, "inputTypes") = c('_p_itk__simple__Image')
class(`Exp`) = c("SWIGFunction", class('Exp'))

# Start of delete_ExpNegativeImageFilter

`delete_ExpNegativeImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ExpNegativeImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ExpNegativeImageFilter`, 'returnType') = 'void'
attr(`delete_ExpNegativeImageFilter`, "inputTypes") = c('_p_itk__simple__ExpNegativeImageFilter')
class(`delete_ExpNegativeImageFilter`) = c("SWIGFunction", class('delete_ExpNegativeImageFilter'))

# Start of new_ExpNegativeImageFilter

`ExpNegativeImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ExpNegativeImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ExpNegativeImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ExpNegativeImageFilter);
  ans
  
}

attr(`ExpNegativeImageFilter`, 'returnType') = '_p_itk__simple__ExpNegativeImageFilter'
class(`ExpNegativeImageFilter`) = c("SWIGFunction", class('ExpNegativeImageFilter'))

# Start of ExpNegativeImageFilter_GetName

`ExpNegativeImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ExpNegativeImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ExpNegativeImageFilter_GetName`, 'returnType') = 'character'
attr(`ExpNegativeImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ExpNegativeImageFilter')
class(`ExpNegativeImageFilter_GetName`) = c("SWIGFunction", class('ExpNegativeImageFilter_GetName'))

# Start of ExpNegativeImageFilter_ToString

`ExpNegativeImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ExpNegativeImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ExpNegativeImageFilter_ToString`, 'returnType') = 'character'
attr(`ExpNegativeImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ExpNegativeImageFilter')
class(`ExpNegativeImageFilter_ToString`) = c("SWIGFunction", class('ExpNegativeImageFilter_ToString'))

# Start of ExpNegativeImageFilter_Execute

`ExpNegativeImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ExpNegativeImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ExpNegativeImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ExpNegativeImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ExpNegativeImageFilter', '_p_itk__simple__Image')
class(`ExpNegativeImageFilter_Execute`) = c("SWIGFunction", class('ExpNegativeImageFilter_Execute'))

# Start of accessor method for itk::simple::ExpNegativeImageFilter
setMethod('$', '_p_itk__simple__ExpNegativeImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = ExpNegativeImageFilter_GetName, 'ToString' = ExpNegativeImageFilter_ToString, 'Execute' = ExpNegativeImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ExpNegativeImageFilter
# Start of ExpNegative

`ExpNegative` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ExpNegative', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ExpNegative`, 'returnType') = '_p_itk__simple__Image'
attr(`ExpNegative`, "inputTypes") = c('_p_itk__simple__Image')
class(`ExpNegative`) = c("SWIGFunction", class('ExpNegative'))

# Start of delete_ExpandImageFilter

`delete_ExpandImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ExpandImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ExpandImageFilter`, 'returnType') = 'void'
attr(`delete_ExpandImageFilter`, "inputTypes") = c('_p_itk__simple__ExpandImageFilter')
class(`delete_ExpandImageFilter`) = c("SWIGFunction", class('delete_ExpandImageFilter'))

# Start of new_ExpandImageFilter

`ExpandImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ExpandImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ExpandImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ExpandImageFilter);
  ans
  
}

attr(`ExpandImageFilter`, 'returnType') = '_p_itk__simple__ExpandImageFilter'
class(`ExpandImageFilter`) = c("SWIGFunction", class('ExpandImageFilter'))

# Start of ExpandImageFilter_SetExpandFactor

`ExpandImageFilter_SetExpandFactor` = function(self, e)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  e = as.integer(e);
  
  if(length(e) > 1) {
    warning("using only the first element of e");
  };
  
  ;ans = .Call('R_swig_ExpandImageFilter_SetExpandFactor', self, e, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ExpandImageFilter_SetExpandFactor`, 'returnType') = 'void'
attr(`ExpandImageFilter_SetExpandFactor`, "inputTypes") = c('_p_itk__simple__ExpandImageFilter', 'integer')
class(`ExpandImageFilter_SetExpandFactor`) = c("SWIGFunction", class('ExpandImageFilter_SetExpandFactor'))

# Start of ExpandImageFilter_SetExpandFactors

`ExpandImageFilter_SetExpandFactors__SWIG_0` = function(self, ExpandFactors)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ExpandFactors = as.integer(ExpandFactors);
  ;ans = .Call('R_swig_ExpandImageFilter_SetExpandFactors__SWIG_0', self, ExpandFactors, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ExpandImageFilter_SetExpandFactors__SWIG_0`, 'returnType') = 'void'
attr(`ExpandImageFilter_SetExpandFactors__SWIG_0`, "inputTypes") = c('_p_itk__simple__ExpandImageFilter', 'integer')
class(`ExpandImageFilter_SetExpandFactors__SWIG_0`) = c("SWIGFunction", class('ExpandImageFilter_SetExpandFactors__SWIG_0'))

# Start of ExpandImageFilter_SetExpandFactors

`ExpandImageFilter_SetExpandFactors__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_ExpandImageFilter_SetExpandFactors__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ExpandImageFilter_SetExpandFactors__SWIG_1`, 'returnType') = 'void'
attr(`ExpandImageFilter_SetExpandFactors__SWIG_1`, "inputTypes") = c('_p_itk__simple__ExpandImageFilter', 'integer')
class(`ExpandImageFilter_SetExpandFactors__SWIG_1`) = c("SWIGFunction", class('ExpandImageFilter_SetExpandFactors__SWIG_1'))

`ExpandImageFilter_SetExpandFactors` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ExpandImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- ExpandImageFilter_SetExpandFactors__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__ExpandImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- ExpandImageFilter_SetExpandFactors__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ExpandImageFilter_SetExpandFactors with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of ExpandImageFilter_GetExpandFactors

`ExpandImageFilter_GetExpandFactors` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ExpandImageFilter_GetExpandFactors', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ExpandImageFilter_GetExpandFactors`, 'returnType') = 'integer'
attr(`ExpandImageFilter_GetExpandFactors`, "inputTypes") = c('_p_itk__simple__ExpandImageFilter')
class(`ExpandImageFilter_GetExpandFactors`) = c("SWIGFunction", class('ExpandImageFilter_GetExpandFactors'))

# Start of ExpandImageFilter_SetInterpolator

`ExpandImageFilter_SetInterpolator` = function(self, Interpolator)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Interpolator = enumToInteger(Interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(Interpolator) > 1) {
    warning("using only the first element of Interpolator");
  };
  
  ;ans = .Call('R_swig_ExpandImageFilter_SetInterpolator', self, Interpolator, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ExpandImageFilter_SetInterpolator`, 'returnType') = 'void'
attr(`ExpandImageFilter_SetInterpolator`, "inputTypes") = c('_p_itk__simple__ExpandImageFilter', 'character')
class(`ExpandImageFilter_SetInterpolator`) = c("SWIGFunction", class('ExpandImageFilter_SetInterpolator'))

# Start of ExpandImageFilter_GetInterpolator

`ExpandImageFilter_GetInterpolator` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ExpandImageFilter_GetInterpolator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`ExpandImageFilter_GetInterpolator`, 'returnType') = 'character'
attr(`ExpandImageFilter_GetInterpolator`, "inputTypes") = c('_p_itk__simple__ExpandImageFilter')
class(`ExpandImageFilter_GetInterpolator`) = c("SWIGFunction", class('ExpandImageFilter_GetInterpolator'))

# Start of ExpandImageFilter_GetName

`ExpandImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ExpandImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ExpandImageFilter_GetName`, 'returnType') = 'character'
attr(`ExpandImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ExpandImageFilter')
class(`ExpandImageFilter_GetName`) = c("SWIGFunction", class('ExpandImageFilter_GetName'))

# Start of ExpandImageFilter_ToString

`ExpandImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ExpandImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ExpandImageFilter_ToString`, 'returnType') = 'character'
attr(`ExpandImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ExpandImageFilter')
class(`ExpandImageFilter_ToString`) = c("SWIGFunction", class('ExpandImageFilter_ToString'))

# Start of ExpandImageFilter_Execute

`ExpandImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ExpandImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ExpandImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ExpandImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ExpandImageFilter', '_p_itk__simple__Image')
class(`ExpandImageFilter_Execute`) = c("SWIGFunction", class('ExpandImageFilter_Execute'))

# Start of accessor method for itk::simple::ExpandImageFilter
setMethod('$', '_p_itk__simple__ExpandImageFilter', function(x, name)

{
  accessorFuns = list('SetExpandFactor' = ExpandImageFilter_SetExpandFactor, 'SetExpandFactors' = ExpandImageFilter_SetExpandFactors, 'GetExpandFactors' = ExpandImageFilter_GetExpandFactors, 'SetInterpolator' = ExpandImageFilter_SetInterpolator, 'GetInterpolator' = ExpandImageFilter_GetInterpolator, 'GetName' = ExpandImageFilter_GetName, 'ToString' = ExpandImageFilter_ToString, 'Execute' = ExpandImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ExpandImageFilter
# Start of Expand

`Expand__SWIG_0` = function(image1, expandFactors, interpolator, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  expandFactors = as.integer(expandFactors);
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  ;ans = .Call('R_swig_Expand__SWIG_0', image1, expandFactors, interpolator, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Expand__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Expand__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`Expand__SWIG_0`) = c("SWIGFunction", class('Expand__SWIG_0'))

# Start of Expand

`Expand__SWIG_1` = function(image1, expandFactors, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  expandFactors = as.integer(expandFactors);
  ;ans = .Call('R_swig_Expand__SWIG_1', image1, expandFactors, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Expand__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Expand__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Expand__SWIG_1`) = c("SWIGFunction", class('Expand__SWIG_1'))

# Start of Expand

`Expand__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Expand__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Expand__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Expand__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`Expand__SWIG_2`) = c("SWIGFunction", class('Expand__SWIG_2'))

`Expand` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Expand__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- Expand__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Expand__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Expand with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_FFTConvolutionImageFilter

`delete_FFTConvolutionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_FFTConvolutionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_FFTConvolutionImageFilter`, 'returnType') = 'void'
attr(`delete_FFTConvolutionImageFilter`, "inputTypes") = c('_p_itk__simple__FFTConvolutionImageFilter')
class(`delete_FFTConvolutionImageFilter`) = c("SWIGFunction", class('delete_FFTConvolutionImageFilter'))

# Start of new_FFTConvolutionImageFilter

`FFTConvolutionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_FFTConvolutionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__FFTConvolutionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_FFTConvolutionImageFilter);
  ans
  
}

attr(`FFTConvolutionImageFilter`, 'returnType') = '_p_itk__simple__FFTConvolutionImageFilter'
class(`FFTConvolutionImageFilter`) = c("SWIGFunction", class('FFTConvolutionImageFilter'))

# Start of FFTConvolutionImageFilter_SetNormalize

`FFTConvolutionImageFilter_SetNormalize` = function(self, Normalize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Normalize = as.logical(Normalize);
  ;ans = .Call('R_swig_FFTConvolutionImageFilter_SetNormalize', self, Normalize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FFTConvolutionImageFilter_SetNormalize`, 'returnType') = 'void'
attr(`FFTConvolutionImageFilter_SetNormalize`, "inputTypes") = c('_p_itk__simple__FFTConvolutionImageFilter', 'logical')
class(`FFTConvolutionImageFilter_SetNormalize`) = c("SWIGFunction", class('FFTConvolutionImageFilter_SetNormalize'))

# Start of FFTConvolutionImageFilter_NormalizeOn

`FFTConvolutionImageFilter_NormalizeOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FFTConvolutionImageFilter_NormalizeOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FFTConvolutionImageFilter_NormalizeOn`, 'returnType') = 'void'
attr(`FFTConvolutionImageFilter_NormalizeOn`, "inputTypes") = c('_p_itk__simple__FFTConvolutionImageFilter')
class(`FFTConvolutionImageFilter_NormalizeOn`) = c("SWIGFunction", class('FFTConvolutionImageFilter_NormalizeOn'))

# Start of FFTConvolutionImageFilter_NormalizeOff

`FFTConvolutionImageFilter_NormalizeOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FFTConvolutionImageFilter_NormalizeOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FFTConvolutionImageFilter_NormalizeOff`, 'returnType') = 'void'
attr(`FFTConvolutionImageFilter_NormalizeOff`, "inputTypes") = c('_p_itk__simple__FFTConvolutionImageFilter')
class(`FFTConvolutionImageFilter_NormalizeOff`) = c("SWIGFunction", class('FFTConvolutionImageFilter_NormalizeOff'))

# Start of FFTConvolutionImageFilter_GetNormalize

`FFTConvolutionImageFilter_GetNormalize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FFTConvolutionImageFilter_GetNormalize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FFTConvolutionImageFilter_GetNormalize`, 'returnType') = 'logical'
attr(`FFTConvolutionImageFilter_GetNormalize`, "inputTypes") = c('_p_itk__simple__FFTConvolutionImageFilter')
class(`FFTConvolutionImageFilter_GetNormalize`) = c("SWIGFunction", class('FFTConvolutionImageFilter_GetNormalize'))

# Start of FFTConvolutionImageFilter_ZERO_PAD_get

`FFTConvolutionImageFilter_ZERO_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_FFTConvolutionImageFilter_ZERO_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FFTConvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`FFTConvolutionImageFilter_ZERO_PAD_get`, 'returnType') = 'character'
class(`FFTConvolutionImageFilter_ZERO_PAD_get`) = c("SWIGFunction", class('FFTConvolutionImageFilter_ZERO_PAD_get'))

# Start of FFTConvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get

`FFTConvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_FFTConvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FFTConvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`FFTConvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`, 'returnType') = 'character'
class(`FFTConvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`) = c("SWIGFunction", class('FFTConvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get'))

# Start of FFTConvolutionImageFilter_PERIODIC_PAD_get

`FFTConvolutionImageFilter_PERIODIC_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_FFTConvolutionImageFilter_PERIODIC_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FFTConvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`FFTConvolutionImageFilter_PERIODIC_PAD_get`, 'returnType') = 'character'
class(`FFTConvolutionImageFilter_PERIODIC_PAD_get`) = c("SWIGFunction", class('FFTConvolutionImageFilter_PERIODIC_PAD_get'))

# Start of FFTConvolutionImageFilter_SetBoundaryCondition

`FFTConvolutionImageFilter_SetBoundaryCondition` = function(self, BoundaryCondition)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BoundaryCondition = enumToInteger(BoundaryCondition, "_itk__simple__FFTConvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(BoundaryCondition) > 1) {
    warning("using only the first element of BoundaryCondition");
  };
  
  ;ans = .Call('R_swig_FFTConvolutionImageFilter_SetBoundaryCondition', self, BoundaryCondition, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FFTConvolutionImageFilter_SetBoundaryCondition`, 'returnType') = 'void'
attr(`FFTConvolutionImageFilter_SetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__FFTConvolutionImageFilter', 'character')
class(`FFTConvolutionImageFilter_SetBoundaryCondition`) = c("SWIGFunction", class('FFTConvolutionImageFilter_SetBoundaryCondition'))

# Start of FFTConvolutionImageFilter_GetBoundaryCondition

`FFTConvolutionImageFilter_GetBoundaryCondition` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FFTConvolutionImageFilter_GetBoundaryCondition', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FFTConvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`FFTConvolutionImageFilter_GetBoundaryCondition`, 'returnType') = 'character'
attr(`FFTConvolutionImageFilter_GetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__FFTConvolutionImageFilter')
class(`FFTConvolutionImageFilter_GetBoundaryCondition`) = c("SWIGFunction", class('FFTConvolutionImageFilter_GetBoundaryCondition'))

# Start of FFTConvolutionImageFilter_SAME_get

`FFTConvolutionImageFilter_SAME_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_FFTConvolutionImageFilter_SAME_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FFTConvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`FFTConvolutionImageFilter_SAME_get`, 'returnType') = 'character'
class(`FFTConvolutionImageFilter_SAME_get`) = c("SWIGFunction", class('FFTConvolutionImageFilter_SAME_get'))

# Start of FFTConvolutionImageFilter_VALID_get

`FFTConvolutionImageFilter_VALID_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_FFTConvolutionImageFilter_VALID_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FFTConvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`FFTConvolutionImageFilter_VALID_get`, 'returnType') = 'character'
class(`FFTConvolutionImageFilter_VALID_get`) = c("SWIGFunction", class('FFTConvolutionImageFilter_VALID_get'))

# Start of FFTConvolutionImageFilter_SetOutputRegionMode

`FFTConvolutionImageFilter_SetOutputRegionMode` = function(self, OutputRegionMode)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputRegionMode = enumToInteger(OutputRegionMode, "_itk__simple__FFTConvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(OutputRegionMode) > 1) {
    warning("using only the first element of OutputRegionMode");
  };
  
  ;ans = .Call('R_swig_FFTConvolutionImageFilter_SetOutputRegionMode', self, OutputRegionMode, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FFTConvolutionImageFilter_SetOutputRegionMode`, 'returnType') = 'void'
attr(`FFTConvolutionImageFilter_SetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__FFTConvolutionImageFilter', 'character')
class(`FFTConvolutionImageFilter_SetOutputRegionMode`) = c("SWIGFunction", class('FFTConvolutionImageFilter_SetOutputRegionMode'))

# Start of FFTConvolutionImageFilter_GetOutputRegionMode

`FFTConvolutionImageFilter_GetOutputRegionMode` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FFTConvolutionImageFilter_GetOutputRegionMode', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FFTConvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`FFTConvolutionImageFilter_GetOutputRegionMode`, 'returnType') = 'character'
attr(`FFTConvolutionImageFilter_GetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__FFTConvolutionImageFilter')
class(`FFTConvolutionImageFilter_GetOutputRegionMode`) = c("SWIGFunction", class('FFTConvolutionImageFilter_GetOutputRegionMode'))

# Start of FFTConvolutionImageFilter_GetName

`FFTConvolutionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FFTConvolutionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FFTConvolutionImageFilter_GetName`, 'returnType') = 'character'
attr(`FFTConvolutionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__FFTConvolutionImageFilter')
class(`FFTConvolutionImageFilter_GetName`) = c("SWIGFunction", class('FFTConvolutionImageFilter_GetName'))

# Start of FFTConvolutionImageFilter_ToString

`FFTConvolutionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FFTConvolutionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FFTConvolutionImageFilter_ToString`, 'returnType') = 'character'
attr(`FFTConvolutionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__FFTConvolutionImageFilter')
class(`FFTConvolutionImageFilter_ToString`) = c("SWIGFunction", class('FFTConvolutionImageFilter_ToString'))

# Start of FFTConvolutionImageFilter_Execute

`FFTConvolutionImageFilter_Execute` = function(self, image, kernelImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(kernelImage, "ExternalReference")) kernelImage = slot(kernelImage,"ref"); 
  ;ans = .Call('R_swig_FFTConvolutionImageFilter_Execute', self, image, kernelImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTConvolutionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTConvolutionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__FFTConvolutionImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`FFTConvolutionImageFilter_Execute`) = c("SWIGFunction", class('FFTConvolutionImageFilter_Execute'))

# Start of accessor method for itk::simple::FFTConvolutionImageFilter
setMethod('$', '_p_itk__simple__FFTConvolutionImageFilter', function(x, name)

{
  accessorFuns = list('SetNormalize' = FFTConvolutionImageFilter_SetNormalize, 'NormalizeOn' = FFTConvolutionImageFilter_NormalizeOn, 'NormalizeOff' = FFTConvolutionImageFilter_NormalizeOff, 'GetNormalize' = FFTConvolutionImageFilter_GetNormalize, 'SetBoundaryCondition' = FFTConvolutionImageFilter_SetBoundaryCondition, 'GetBoundaryCondition' = FFTConvolutionImageFilter_GetBoundaryCondition, 'SetOutputRegionMode' = FFTConvolutionImageFilter_SetOutputRegionMode, 'GetOutputRegionMode' = FFTConvolutionImageFilter_GetOutputRegionMode, 'GetName' = FFTConvolutionImageFilter_GetName, 'ToString' = FFTConvolutionImageFilter_ToString, 'Execute' = FFTConvolutionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::FFTConvolutionImageFilter
# Start of FFTConvolution

`FFTConvolution__SWIG_0` = function(image, kernelImage, normalize, boundaryCondition, outputRegionMode, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(kernelImage, "ExternalReference")) kernelImage = slot(kernelImage,"ref"); 
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__FFTConvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  outputRegionMode = enumToInteger(outputRegionMode, "_itk__simple__FFTConvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(outputRegionMode) > 1) {
    warning("using only the first element of outputRegionMode");
  };
  
  ;ans = .Call('R_swig_FFTConvolution__SWIG_0', image, kernelImage, normalize, boundaryCondition, outputRegionMode, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTConvolution__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTConvolution__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical', 'character', 'character')
class(`FFTConvolution__SWIG_0`) = c("SWIGFunction", class('FFTConvolution__SWIG_0'))

# Start of FFTConvolution

`FFTConvolution__SWIG_1` = function(image, kernelImage, normalize, boundaryCondition, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(kernelImage, "ExternalReference")) kernelImage = slot(kernelImage,"ref"); 
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__FFTConvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  ;ans = .Call('R_swig_FFTConvolution__SWIG_1', image, kernelImage, normalize, boundaryCondition, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTConvolution__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTConvolution__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical', 'character')
class(`FFTConvolution__SWIG_1`) = c("SWIGFunction", class('FFTConvolution__SWIG_1'))

# Start of FFTConvolution

`FFTConvolution__SWIG_2` = function(image, kernelImage, normalize, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(kernelImage, "ExternalReference")) kernelImage = slot(kernelImage,"ref"); 
  normalize = as.logical(normalize);
  ;ans = .Call('R_swig_FFTConvolution__SWIG_2', image, kernelImage, normalize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTConvolution__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTConvolution__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical')
class(`FFTConvolution__SWIG_2`) = c("SWIGFunction", class('FFTConvolution__SWIG_2'))

# Start of FFTConvolution

`FFTConvolution__SWIG_3` = function(image, kernelImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(kernelImage, "ExternalReference")) kernelImage = slot(kernelImage,"ref"); 
  ;ans = .Call('R_swig_FFTConvolution__SWIG_3', image, kernelImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTConvolution__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTConvolution__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`FFTConvolution__SWIG_3`) = c("SWIGFunction", class('FFTConvolution__SWIG_3'))

`FFTConvolution` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- FFTConvolution__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- FFTConvolution__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- FFTConvolution__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.character(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- FFTConvolution__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for FFTConvolution with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_FFTNormalizedCorrelationImageFilter

`delete_FFTNormalizedCorrelationImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_FFTNormalizedCorrelationImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_FFTNormalizedCorrelationImageFilter`, 'returnType') = 'void'
attr(`delete_FFTNormalizedCorrelationImageFilter`, "inputTypes") = c('_p_itk__simple__FFTNormalizedCorrelationImageFilter')
class(`delete_FFTNormalizedCorrelationImageFilter`) = c("SWIGFunction", class('delete_FFTNormalizedCorrelationImageFilter'))

# Start of new_FFTNormalizedCorrelationImageFilter

`FFTNormalizedCorrelationImageFilter` = function()
{
  ;ans = .Call('R_swig_new_FFTNormalizedCorrelationImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__FFTNormalizedCorrelationImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_FFTNormalizedCorrelationImageFilter);
  ans
  
}

attr(`FFTNormalizedCorrelationImageFilter`, 'returnType') = '_p_itk__simple__FFTNormalizedCorrelationImageFilter'
class(`FFTNormalizedCorrelationImageFilter`) = c("SWIGFunction", class('FFTNormalizedCorrelationImageFilter'))

# Start of FFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels

`FFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels` = function(self, RequiredNumberOfOverlappingPixels)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  RequiredNumberOfOverlappingPixels = as.integer(RequiredNumberOfOverlappingPixels);
  
  if(length(RequiredNumberOfOverlappingPixels) > 1) {
    warning("using only the first element of RequiredNumberOfOverlappingPixels");
  };
  
  ;ans = .Call('R_swig_FFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels', self, RequiredNumberOfOverlappingPixels, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels`, 'returnType') = 'void'
attr(`FFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels`, "inputTypes") = c('_p_itk__simple__FFTNormalizedCorrelationImageFilter', 'integer')
class(`FFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels`) = c("SWIGFunction", class('FFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels'))

# Start of FFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels

`FFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels`, 'returnType') = 'integer'
attr(`FFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels`, "inputTypes") = c('_p_itk__simple__FFTNormalizedCorrelationImageFilter')
class(`FFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels`) = c("SWIGFunction", class('FFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels'))

# Start of FFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels

`FFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels` = function(self, RequiredFractionOfOverlappingPixels)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_FFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels', self, RequiredFractionOfOverlappingPixels, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels`, 'returnType') = 'void'
attr(`FFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels`, "inputTypes") = c('_p_itk__simple__FFTNormalizedCorrelationImageFilter', 'numeric')
class(`FFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels`) = c("SWIGFunction", class('FFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels'))

# Start of FFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels

`FFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels`, 'returnType') = 'numeric'
attr(`FFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels`, "inputTypes") = c('_p_itk__simple__FFTNormalizedCorrelationImageFilter')
class(`FFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels`) = c("SWIGFunction", class('FFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels'))

# Start of FFTNormalizedCorrelationImageFilter_GetName

`FFTNormalizedCorrelationImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FFTNormalizedCorrelationImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FFTNormalizedCorrelationImageFilter_GetName`, 'returnType') = 'character'
attr(`FFTNormalizedCorrelationImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__FFTNormalizedCorrelationImageFilter')
class(`FFTNormalizedCorrelationImageFilter_GetName`) = c("SWIGFunction", class('FFTNormalizedCorrelationImageFilter_GetName'))

# Start of FFTNormalizedCorrelationImageFilter_ToString

`FFTNormalizedCorrelationImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FFTNormalizedCorrelationImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FFTNormalizedCorrelationImageFilter_ToString`, 'returnType') = 'character'
attr(`FFTNormalizedCorrelationImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__FFTNormalizedCorrelationImageFilter')
class(`FFTNormalizedCorrelationImageFilter_ToString`) = c("SWIGFunction", class('FFTNormalizedCorrelationImageFilter_ToString'))

# Start of FFTNormalizedCorrelationImageFilter_Execute

`FFTNormalizedCorrelationImageFilter_Execute` = function(self, fixedImage, movingImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  ;ans = .Call('R_swig_FFTNormalizedCorrelationImageFilter_Execute', self, fixedImage, movingImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTNormalizedCorrelationImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTNormalizedCorrelationImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__FFTNormalizedCorrelationImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`FFTNormalizedCorrelationImageFilter_Execute`) = c("SWIGFunction", class('FFTNormalizedCorrelationImageFilter_Execute'))

# Start of accessor method for itk::simple::FFTNormalizedCorrelationImageFilter
setMethod('$', '_p_itk__simple__FFTNormalizedCorrelationImageFilter', function(x, name)

{
  accessorFuns = list('SetRequiredNumberOfOverlappingPixels' = FFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels, 'GetRequiredNumberOfOverlappingPixels' = FFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels, 'SetRequiredFractionOfOverlappingPixels' = FFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels, 'GetRequiredFractionOfOverlappingPixels' = FFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels, 'GetName' = FFTNormalizedCorrelationImageFilter_GetName, 'ToString' = FFTNormalizedCorrelationImageFilter_ToString, 'Execute' = FFTNormalizedCorrelationImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::FFTNormalizedCorrelationImageFilter
# Start of FFTNormalizedCorrelation

`FFTNormalizedCorrelation__SWIG_0` = function(fixedImage, movingImage, requiredNumberOfOverlappingPixels, requiredFractionOfOverlappingPixels, .copy = FALSE)
{
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  requiredNumberOfOverlappingPixels = as.integer(requiredNumberOfOverlappingPixels);
  
  if(length(requiredNumberOfOverlappingPixels) > 1) {
    warning("using only the first element of requiredNumberOfOverlappingPixels");
  };
  
  
  ;ans = .Call('R_swig_FFTNormalizedCorrelation__SWIG_0', fixedImage, movingImage, requiredNumberOfOverlappingPixels, requiredFractionOfOverlappingPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTNormalizedCorrelation__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTNormalizedCorrelation__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric')
class(`FFTNormalizedCorrelation__SWIG_0`) = c("SWIGFunction", class('FFTNormalizedCorrelation__SWIG_0'))

# Start of FFTNormalizedCorrelation

`FFTNormalizedCorrelation__SWIG_1` = function(fixedImage, movingImage, requiredNumberOfOverlappingPixels, .copy = FALSE)
{
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  requiredNumberOfOverlappingPixels = as.integer(requiredNumberOfOverlappingPixels);
  
  if(length(requiredNumberOfOverlappingPixels) > 1) {
    warning("using only the first element of requiredNumberOfOverlappingPixels");
  };
  
  ;ans = .Call('R_swig_FFTNormalizedCorrelation__SWIG_1', fixedImage, movingImage, requiredNumberOfOverlappingPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTNormalizedCorrelation__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTNormalizedCorrelation__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`FFTNormalizedCorrelation__SWIG_1`) = c("SWIGFunction", class('FFTNormalizedCorrelation__SWIG_1'))

# Start of FFTNormalizedCorrelation

`FFTNormalizedCorrelation__SWIG_2` = function(fixedImage, movingImage, .copy = FALSE)
{
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  ;ans = .Call('R_swig_FFTNormalizedCorrelation__SWIG_2', fixedImage, movingImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTNormalizedCorrelation__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTNormalizedCorrelation__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`FFTNormalizedCorrelation__SWIG_2`) = c("SWIGFunction", class('FFTNormalizedCorrelation__SWIG_2'))

`FFTNormalizedCorrelation` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- FFTNormalizedCorrelation__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- FFTNormalizedCorrelation__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- FFTNormalizedCorrelation__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for FFTNormalizedCorrelation with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_FFTPadImageFilter

`delete_FFTPadImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_FFTPadImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_FFTPadImageFilter`, 'returnType') = 'void'
attr(`delete_FFTPadImageFilter`, "inputTypes") = c('_p_itk__simple__FFTPadImageFilter')
class(`delete_FFTPadImageFilter`) = c("SWIGFunction", class('delete_FFTPadImageFilter'))

# Start of new_FFTPadImageFilter

`FFTPadImageFilter` = function()
{
  ;ans = .Call('R_swig_new_FFTPadImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__FFTPadImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_FFTPadImageFilter);
  ans
  
}

attr(`FFTPadImageFilter`, 'returnType') = '_p_itk__simple__FFTPadImageFilter'
class(`FFTPadImageFilter`) = c("SWIGFunction", class('FFTPadImageFilter'))

# Start of FFTPadImageFilter_ZERO_PAD_get

`FFTPadImageFilter_ZERO_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_FFTPadImageFilter_ZERO_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FFTPadImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`FFTPadImageFilter_ZERO_PAD_get`, 'returnType') = 'character'
class(`FFTPadImageFilter_ZERO_PAD_get`) = c("SWIGFunction", class('FFTPadImageFilter_ZERO_PAD_get'))

# Start of FFTPadImageFilter_ZERO_FLUX_NEUMANN_PAD_get

`FFTPadImageFilter_ZERO_FLUX_NEUMANN_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_FFTPadImageFilter_ZERO_FLUX_NEUMANN_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FFTPadImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`FFTPadImageFilter_ZERO_FLUX_NEUMANN_PAD_get`, 'returnType') = 'character'
class(`FFTPadImageFilter_ZERO_FLUX_NEUMANN_PAD_get`) = c("SWIGFunction", class('FFTPadImageFilter_ZERO_FLUX_NEUMANN_PAD_get'))

# Start of FFTPadImageFilter_PERIODIC_PAD_get

`FFTPadImageFilter_PERIODIC_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_FFTPadImageFilter_PERIODIC_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FFTPadImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`FFTPadImageFilter_PERIODIC_PAD_get`, 'returnType') = 'character'
class(`FFTPadImageFilter_PERIODIC_PAD_get`) = c("SWIGFunction", class('FFTPadImageFilter_PERIODIC_PAD_get'))

# Start of FFTPadImageFilter_SetBoundaryCondition

`FFTPadImageFilter_SetBoundaryCondition` = function(self, BoundaryCondition)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BoundaryCondition = enumToInteger(BoundaryCondition, "_itk__simple__FFTPadImageFilter__BoundaryConditionType"); 
  
  if(length(BoundaryCondition) > 1) {
    warning("using only the first element of BoundaryCondition");
  };
  
  ;ans = .Call('R_swig_FFTPadImageFilter_SetBoundaryCondition', self, BoundaryCondition, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FFTPadImageFilter_SetBoundaryCondition`, 'returnType') = 'void'
attr(`FFTPadImageFilter_SetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__FFTPadImageFilter', 'character')
class(`FFTPadImageFilter_SetBoundaryCondition`) = c("SWIGFunction", class('FFTPadImageFilter_SetBoundaryCondition'))

# Start of FFTPadImageFilter_GetBoundaryCondition

`FFTPadImageFilter_GetBoundaryCondition` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FFTPadImageFilter_GetBoundaryCondition', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FFTPadImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`FFTPadImageFilter_GetBoundaryCondition`, 'returnType') = 'character'
attr(`FFTPadImageFilter_GetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__FFTPadImageFilter')
class(`FFTPadImageFilter_GetBoundaryCondition`) = c("SWIGFunction", class('FFTPadImageFilter_GetBoundaryCondition'))

# Start of FFTPadImageFilter_SetSizeGreatestPrimeFactor

`FFTPadImageFilter_SetSizeGreatestPrimeFactor` = function(self, SizeGreatestPrimeFactor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SizeGreatestPrimeFactor = as.integer(SizeGreatestPrimeFactor);
  
  if(length(SizeGreatestPrimeFactor) > 1) {
    warning("using only the first element of SizeGreatestPrimeFactor");
  };
  
  ;ans = .Call('R_swig_FFTPadImageFilter_SetSizeGreatestPrimeFactor', self, SizeGreatestPrimeFactor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FFTPadImageFilter_SetSizeGreatestPrimeFactor`, 'returnType') = 'void'
attr(`FFTPadImageFilter_SetSizeGreatestPrimeFactor`, "inputTypes") = c('_p_itk__simple__FFTPadImageFilter', 'integer')
class(`FFTPadImageFilter_SetSizeGreatestPrimeFactor`) = c("SWIGFunction", class('FFTPadImageFilter_SetSizeGreatestPrimeFactor'))

# Start of FFTPadImageFilter_GetSizeGreatestPrimeFactor

`FFTPadImageFilter_GetSizeGreatestPrimeFactor` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FFTPadImageFilter_GetSizeGreatestPrimeFactor', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FFTPadImageFilter_GetSizeGreatestPrimeFactor`, 'returnType') = 'integer'
attr(`FFTPadImageFilter_GetSizeGreatestPrimeFactor`, "inputTypes") = c('_p_itk__simple__FFTPadImageFilter')
class(`FFTPadImageFilter_GetSizeGreatestPrimeFactor`) = c("SWIGFunction", class('FFTPadImageFilter_GetSizeGreatestPrimeFactor'))

# Start of FFTPadImageFilter_GetName

`FFTPadImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FFTPadImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FFTPadImageFilter_GetName`, 'returnType') = 'character'
attr(`FFTPadImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__FFTPadImageFilter')
class(`FFTPadImageFilter_GetName`) = c("SWIGFunction", class('FFTPadImageFilter_GetName'))

# Start of FFTPadImageFilter_ToString

`FFTPadImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FFTPadImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FFTPadImageFilter_ToString`, 'returnType') = 'character'
attr(`FFTPadImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__FFTPadImageFilter')
class(`FFTPadImageFilter_ToString`) = c("SWIGFunction", class('FFTPadImageFilter_ToString'))

# Start of FFTPadImageFilter_Execute

`FFTPadImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_FFTPadImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTPadImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTPadImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__FFTPadImageFilter', '_p_itk__simple__Image')
class(`FFTPadImageFilter_Execute`) = c("SWIGFunction", class('FFTPadImageFilter_Execute'))

# Start of FFTPadImageFilter_DefaultSizeGreatestPrimeFactor

`FFTPadImageFilter_DefaultSizeGreatestPrimeFactor` = function(.copy = FALSE)
{
  ;.Call('R_swig_FFTPadImageFilter_DefaultSizeGreatestPrimeFactor', as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FFTPadImageFilter_DefaultSizeGreatestPrimeFactor`, 'returnType') = 'integer'
class(`FFTPadImageFilter_DefaultSizeGreatestPrimeFactor`) = c("SWIGFunction", class('FFTPadImageFilter_DefaultSizeGreatestPrimeFactor'))

# Start of accessor method for itk::simple::FFTPadImageFilter
setMethod('$', '_p_itk__simple__FFTPadImageFilter', function(x, name)

{
  accessorFuns = list('SetBoundaryCondition' = FFTPadImageFilter_SetBoundaryCondition, 'GetBoundaryCondition' = FFTPadImageFilter_GetBoundaryCondition, 'SetSizeGreatestPrimeFactor' = FFTPadImageFilter_SetSizeGreatestPrimeFactor, 'GetSizeGreatestPrimeFactor' = FFTPadImageFilter_GetSizeGreatestPrimeFactor, 'GetName' = FFTPadImageFilter_GetName, 'ToString' = FFTPadImageFilter_ToString, 'Execute' = FFTPadImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::FFTPadImageFilter
# Start of FFTPad

`FFTPad__SWIG_0` = function(image1, boundaryCondition, sizeGreatestPrimeFactor, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__FFTPadImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  sizeGreatestPrimeFactor = as.integer(sizeGreatestPrimeFactor);
  
  if(length(sizeGreatestPrimeFactor) > 1) {
    warning("using only the first element of sizeGreatestPrimeFactor");
  };
  
  ;ans = .Call('R_swig_FFTPad__SWIG_0', image1, boundaryCondition, sizeGreatestPrimeFactor, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTPad__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTPad__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'integer')
class(`FFTPad__SWIG_0`) = c("SWIGFunction", class('FFTPad__SWIG_0'))

# Start of FFTPad

`FFTPad__SWIG_1` = function(image1, boundaryCondition, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__FFTPadImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  ;ans = .Call('R_swig_FFTPad__SWIG_1', image1, boundaryCondition, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTPad__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTPad__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'character')
class(`FFTPad__SWIG_1`) = c("SWIGFunction", class('FFTPad__SWIG_1'))

# Start of FFTPad

`FFTPad__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_FFTPad__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTPad__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTPad__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`FFTPad__SWIG_2`) = c("SWIGFunction", class('FFTPad__SWIG_2'))

`FFTPad` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- FFTPad__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- FFTPad__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- FFTPad__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for FFTPad with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_FFTShiftImageFilter

`delete_FFTShiftImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_FFTShiftImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_FFTShiftImageFilter`, 'returnType') = 'void'
attr(`delete_FFTShiftImageFilter`, "inputTypes") = c('_p_itk__simple__FFTShiftImageFilter')
class(`delete_FFTShiftImageFilter`) = c("SWIGFunction", class('delete_FFTShiftImageFilter'))

# Start of new_FFTShiftImageFilter

`FFTShiftImageFilter` = function()
{
  ;ans = .Call('R_swig_new_FFTShiftImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__FFTShiftImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_FFTShiftImageFilter);
  ans
  
}

attr(`FFTShiftImageFilter`, 'returnType') = '_p_itk__simple__FFTShiftImageFilter'
class(`FFTShiftImageFilter`) = c("SWIGFunction", class('FFTShiftImageFilter'))

# Start of FFTShiftImageFilter_SetInverse

`FFTShiftImageFilter_SetInverse` = function(self, Inverse)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Inverse = as.logical(Inverse);
  ;ans = .Call('R_swig_FFTShiftImageFilter_SetInverse', self, Inverse, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FFTShiftImageFilter_SetInverse`, 'returnType') = 'void'
attr(`FFTShiftImageFilter_SetInverse`, "inputTypes") = c('_p_itk__simple__FFTShiftImageFilter', 'logical')
class(`FFTShiftImageFilter_SetInverse`) = c("SWIGFunction", class('FFTShiftImageFilter_SetInverse'))

# Start of FFTShiftImageFilter_InverseOn

`FFTShiftImageFilter_InverseOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FFTShiftImageFilter_InverseOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FFTShiftImageFilter_InverseOn`, 'returnType') = 'void'
attr(`FFTShiftImageFilter_InverseOn`, "inputTypes") = c('_p_itk__simple__FFTShiftImageFilter')
class(`FFTShiftImageFilter_InverseOn`) = c("SWIGFunction", class('FFTShiftImageFilter_InverseOn'))

# Start of FFTShiftImageFilter_InverseOff

`FFTShiftImageFilter_InverseOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FFTShiftImageFilter_InverseOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FFTShiftImageFilter_InverseOff`, 'returnType') = 'void'
attr(`FFTShiftImageFilter_InverseOff`, "inputTypes") = c('_p_itk__simple__FFTShiftImageFilter')
class(`FFTShiftImageFilter_InverseOff`) = c("SWIGFunction", class('FFTShiftImageFilter_InverseOff'))

# Start of FFTShiftImageFilter_GetInverse

`FFTShiftImageFilter_GetInverse` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FFTShiftImageFilter_GetInverse', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FFTShiftImageFilter_GetInverse`, 'returnType') = 'logical'
attr(`FFTShiftImageFilter_GetInverse`, "inputTypes") = c('_p_itk__simple__FFTShiftImageFilter')
class(`FFTShiftImageFilter_GetInverse`) = c("SWIGFunction", class('FFTShiftImageFilter_GetInverse'))

# Start of FFTShiftImageFilter_GetName

`FFTShiftImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FFTShiftImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FFTShiftImageFilter_GetName`, 'returnType') = 'character'
attr(`FFTShiftImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__FFTShiftImageFilter')
class(`FFTShiftImageFilter_GetName`) = c("SWIGFunction", class('FFTShiftImageFilter_GetName'))

# Start of FFTShiftImageFilter_ToString

`FFTShiftImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FFTShiftImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FFTShiftImageFilter_ToString`, 'returnType') = 'character'
attr(`FFTShiftImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__FFTShiftImageFilter')
class(`FFTShiftImageFilter_ToString`) = c("SWIGFunction", class('FFTShiftImageFilter_ToString'))

# Start of FFTShiftImageFilter_Execute

`FFTShiftImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_FFTShiftImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTShiftImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTShiftImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__FFTShiftImageFilter', '_p_itk__simple__Image')
class(`FFTShiftImageFilter_Execute`) = c("SWIGFunction", class('FFTShiftImageFilter_Execute'))

# Start of accessor method for itk::simple::FFTShiftImageFilter
setMethod('$', '_p_itk__simple__FFTShiftImageFilter', function(x, name)

{
  accessorFuns = list('SetInverse' = FFTShiftImageFilter_SetInverse, 'InverseOn' = FFTShiftImageFilter_InverseOn, 'InverseOff' = FFTShiftImageFilter_InverseOff, 'GetInverse' = FFTShiftImageFilter_GetInverse, 'GetName' = FFTShiftImageFilter_GetName, 'ToString' = FFTShiftImageFilter_ToString, 'Execute' = FFTShiftImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::FFTShiftImageFilter
# Start of FFTShift

`FFTShift__SWIG_0` = function(image1, inverse, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  inverse = as.logical(inverse);
  ;ans = .Call('R_swig_FFTShift__SWIG_0', image1, inverse, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTShift__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTShift__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`FFTShift__SWIG_0`) = c("SWIGFunction", class('FFTShift__SWIG_0'))

# Start of FFTShift

`FFTShift__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_FFTShift__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FFTShift__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`FFTShift__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`FFTShift__SWIG_1`) = c("SWIGFunction", class('FFTShift__SWIG_1'))

`FFTShift` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- FFTShift__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- FFTShift__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for FFTShift with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_FastApproximateRankImageFilter

`delete_FastApproximateRankImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_FastApproximateRankImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_FastApproximateRankImageFilter`, 'returnType') = 'void'
attr(`delete_FastApproximateRankImageFilter`, "inputTypes") = c('_p_itk__simple__FastApproximateRankImageFilter')
class(`delete_FastApproximateRankImageFilter`) = c("SWIGFunction", class('delete_FastApproximateRankImageFilter'))

# Start of new_FastApproximateRankImageFilter

`FastApproximateRankImageFilter` = function()
{
  ;ans = .Call('R_swig_new_FastApproximateRankImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__FastApproximateRankImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_FastApproximateRankImageFilter);
  ans
  
}

attr(`FastApproximateRankImageFilter`, 'returnType') = '_p_itk__simple__FastApproximateRankImageFilter'
class(`FastApproximateRankImageFilter`) = c("SWIGFunction", class('FastApproximateRankImageFilter'))

# Start of FastApproximateRankImageFilter_SetRank

`FastApproximateRankImageFilter_SetRank` = function(self, Rank)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_FastApproximateRankImageFilter_SetRank', self, Rank, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastApproximateRankImageFilter_SetRank`, 'returnType') = 'void'
attr(`FastApproximateRankImageFilter_SetRank`, "inputTypes") = c('_p_itk__simple__FastApproximateRankImageFilter', 'numeric')
class(`FastApproximateRankImageFilter_SetRank`) = c("SWIGFunction", class('FastApproximateRankImageFilter_SetRank'))

# Start of FastApproximateRankImageFilter_GetRank

`FastApproximateRankImageFilter_GetRank` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastApproximateRankImageFilter_GetRank', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastApproximateRankImageFilter_GetRank`, 'returnType') = 'numeric'
attr(`FastApproximateRankImageFilter_GetRank`, "inputTypes") = c('_p_itk__simple__FastApproximateRankImageFilter')
class(`FastApproximateRankImageFilter_GetRank`) = c("SWIGFunction", class('FastApproximateRankImageFilter_GetRank'))

# Start of FastApproximateRankImageFilter_SetRadius

`FastApproximateRankImageFilter_SetRadius__SWIG_0` = function(self, Radius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Radius = as.integer(Radius);
  ;ans = .Call('R_swig_FastApproximateRankImageFilter_SetRadius__SWIG_0', self, Radius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastApproximateRankImageFilter_SetRadius__SWIG_0`, 'returnType') = 'void'
attr(`FastApproximateRankImageFilter_SetRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__FastApproximateRankImageFilter', 'integer')
class(`FastApproximateRankImageFilter_SetRadius__SWIG_0`) = c("SWIGFunction", class('FastApproximateRankImageFilter_SetRadius__SWIG_0'))

# Start of FastApproximateRankImageFilter_SetRadius

`FastApproximateRankImageFilter_SetRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_FastApproximateRankImageFilter_SetRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastApproximateRankImageFilter_SetRadius__SWIG_1`, 'returnType') = 'void'
attr(`FastApproximateRankImageFilter_SetRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__FastApproximateRankImageFilter', 'integer')
class(`FastApproximateRankImageFilter_SetRadius__SWIG_1`) = c("SWIGFunction", class('FastApproximateRankImageFilter_SetRadius__SWIG_1'))

`FastApproximateRankImageFilter_SetRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__FastApproximateRankImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- FastApproximateRankImageFilter_SetRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__FastApproximateRankImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- FastApproximateRankImageFilter_SetRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for FastApproximateRankImageFilter_SetRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of FastApproximateRankImageFilter_GetRadius

`FastApproximateRankImageFilter_GetRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastApproximateRankImageFilter_GetRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastApproximateRankImageFilter_GetRadius`, 'returnType') = 'integer'
attr(`FastApproximateRankImageFilter_GetRadius`, "inputTypes") = c('_p_itk__simple__FastApproximateRankImageFilter')
class(`FastApproximateRankImageFilter_GetRadius`) = c("SWIGFunction", class('FastApproximateRankImageFilter_GetRadius'))

# Start of FastApproximateRankImageFilter_GetName

`FastApproximateRankImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastApproximateRankImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastApproximateRankImageFilter_GetName`, 'returnType') = 'character'
attr(`FastApproximateRankImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__FastApproximateRankImageFilter')
class(`FastApproximateRankImageFilter_GetName`) = c("SWIGFunction", class('FastApproximateRankImageFilter_GetName'))

# Start of FastApproximateRankImageFilter_ToString

`FastApproximateRankImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastApproximateRankImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastApproximateRankImageFilter_ToString`, 'returnType') = 'character'
attr(`FastApproximateRankImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__FastApproximateRankImageFilter')
class(`FastApproximateRankImageFilter_ToString`) = c("SWIGFunction", class('FastApproximateRankImageFilter_ToString'))

# Start of FastApproximateRankImageFilter_Execute

`FastApproximateRankImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_FastApproximateRankImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastApproximateRankImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`FastApproximateRankImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__FastApproximateRankImageFilter', '_p_itk__simple__Image')
class(`FastApproximateRankImageFilter_Execute`) = c("SWIGFunction", class('FastApproximateRankImageFilter_Execute'))

# Start of accessor method for itk::simple::FastApproximateRankImageFilter
setMethod('$', '_p_itk__simple__FastApproximateRankImageFilter', function(x, name)

{
  accessorFuns = list('SetRank' = FastApproximateRankImageFilter_SetRank, 'GetRank' = FastApproximateRankImageFilter_GetRank, 'SetRadius' = FastApproximateRankImageFilter_SetRadius, 'GetRadius' = FastApproximateRankImageFilter_GetRadius, 'GetName' = FastApproximateRankImageFilter_GetName, 'ToString' = FastApproximateRankImageFilter_ToString, 'Execute' = FastApproximateRankImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::FastApproximateRankImageFilter
# Start of FastApproximateRank

`FastApproximateRank__SWIG_0` = function(image1, rank, radius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  radius = as.integer(radius);
  ;ans = .Call('R_swig_FastApproximateRank__SWIG_0', image1, rank, radius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastApproximateRank__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`FastApproximateRank__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`FastApproximateRank__SWIG_0`) = c("SWIGFunction", class('FastApproximateRank__SWIG_0'))

# Start of FastApproximateRank

`FastApproximateRank__SWIG_1` = function(image1, rank, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_FastApproximateRank__SWIG_1', image1, rank, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastApproximateRank__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`FastApproximateRank__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`FastApproximateRank__SWIG_1`) = c("SWIGFunction", class('FastApproximateRank__SWIG_1'))

# Start of FastApproximateRank

`FastApproximateRank__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_FastApproximateRank__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastApproximateRank__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`FastApproximateRank__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`FastApproximateRank__SWIG_2`) = c("SWIGFunction", class('FastApproximateRank__SWIG_2'))

`FastApproximateRank` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- FastApproximateRank__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- FastApproximateRank__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- FastApproximateRank__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for FastApproximateRank with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_FastMarchingBaseImageFilter

`delete_FastMarchingBaseImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_FastMarchingBaseImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_FastMarchingBaseImageFilter`, 'returnType') = 'void'
attr(`delete_FastMarchingBaseImageFilter`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter')
class(`delete_FastMarchingBaseImageFilter`) = c("SWIGFunction", class('delete_FastMarchingBaseImageFilter'))

# Start of new_FastMarchingBaseImageFilter

`FastMarchingBaseImageFilter` = function()
{
  ;ans = .Call('R_swig_new_FastMarchingBaseImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__FastMarchingBaseImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_FastMarchingBaseImageFilter);
  ans
  
}

attr(`FastMarchingBaseImageFilter`, 'returnType') = '_p_itk__simple__FastMarchingBaseImageFilter'
class(`FastMarchingBaseImageFilter`) = c("SWIGFunction", class('FastMarchingBaseImageFilter'))

# Start of FastMarchingBaseImageFilter_SetTrialPoints

`FastMarchingBaseImageFilter_SetTrialPoints` = function(self, TrialPoints)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  TrialPoints = lapply(TrialPoints, as.integer);
  ;ans = .Call('R_swig_FastMarchingBaseImageFilter_SetTrialPoints', self, TrialPoints, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingBaseImageFilter_SetTrialPoints`, 'returnType') = 'void'
attr(`FastMarchingBaseImageFilter_SetTrialPoints`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter', 'list')
class(`FastMarchingBaseImageFilter_SetTrialPoints`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_SetTrialPoints'))

# Start of FastMarchingBaseImageFilter_GetTrialPoints

`FastMarchingBaseImageFilter_GetTrialPoints` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingBaseImageFilter_GetTrialPoints', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingBaseImageFilter_GetTrialPoints`, 'returnType') = 'list'
attr(`FastMarchingBaseImageFilter_GetTrialPoints`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter')
class(`FastMarchingBaseImageFilter_GetTrialPoints`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_GetTrialPoints'))

# Start of FastMarchingBaseImageFilter_AddTrialPoint

`FastMarchingBaseImageFilter_AddTrialPoint` = function(self, point)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  point = as.integer(point);
  ;ans = .Call('R_swig_FastMarchingBaseImageFilter_AddTrialPoint', self, point, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingBaseImageFilter_AddTrialPoint`, 'returnType') = 'void'
attr(`FastMarchingBaseImageFilter_AddTrialPoint`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter', 'integer')
class(`FastMarchingBaseImageFilter_AddTrialPoint`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_AddTrialPoint'))

# Start of FastMarchingBaseImageFilter_ClearTrialPoints

`FastMarchingBaseImageFilter_ClearTrialPoints` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FastMarchingBaseImageFilter_ClearTrialPoints', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingBaseImageFilter_ClearTrialPoints`, 'returnType') = 'void'
attr(`FastMarchingBaseImageFilter_ClearTrialPoints`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter')
class(`FastMarchingBaseImageFilter_ClearTrialPoints`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_ClearTrialPoints'))

# Start of FastMarchingBaseImageFilter_SetNormalizationFactor

`FastMarchingBaseImageFilter_SetNormalizationFactor` = function(self, NormalizationFactor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_FastMarchingBaseImageFilter_SetNormalizationFactor', self, NormalizationFactor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingBaseImageFilter_SetNormalizationFactor`, 'returnType') = 'void'
attr(`FastMarchingBaseImageFilter_SetNormalizationFactor`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter', 'numeric')
class(`FastMarchingBaseImageFilter_SetNormalizationFactor`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_SetNormalizationFactor'))

# Start of FastMarchingBaseImageFilter_GetNormalizationFactor

`FastMarchingBaseImageFilter_GetNormalizationFactor` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingBaseImageFilter_GetNormalizationFactor', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingBaseImageFilter_GetNormalizationFactor`, 'returnType') = 'numeric'
attr(`FastMarchingBaseImageFilter_GetNormalizationFactor`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter')
class(`FastMarchingBaseImageFilter_GetNormalizationFactor`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_GetNormalizationFactor'))

# Start of FastMarchingBaseImageFilter_SetStoppingValue

`FastMarchingBaseImageFilter_SetStoppingValue` = function(self, StoppingValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_FastMarchingBaseImageFilter_SetStoppingValue', self, StoppingValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingBaseImageFilter_SetStoppingValue`, 'returnType') = 'void'
attr(`FastMarchingBaseImageFilter_SetStoppingValue`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter', 'numeric')
class(`FastMarchingBaseImageFilter_SetStoppingValue`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_SetStoppingValue'))

# Start of FastMarchingBaseImageFilter_GetStoppingValue

`FastMarchingBaseImageFilter_GetStoppingValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingBaseImageFilter_GetStoppingValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingBaseImageFilter_GetStoppingValue`, 'returnType') = 'numeric'
attr(`FastMarchingBaseImageFilter_GetStoppingValue`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter')
class(`FastMarchingBaseImageFilter_GetStoppingValue`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_GetStoppingValue'))

# Start of FastMarchingBaseImageFilter_Nothing_get

`FastMarchingBaseImageFilter_Nothing_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_FastMarchingBaseImageFilter_Nothing_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FastMarchingBaseImageFilter__TopologyCheckType");
  
  ans
  
}

attr(`FastMarchingBaseImageFilter_Nothing_get`, 'returnType') = 'character'
class(`FastMarchingBaseImageFilter_Nothing_get`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_Nothing_get'))

# Start of FastMarchingBaseImageFilter_NoHandles_get

`FastMarchingBaseImageFilter_NoHandles_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_FastMarchingBaseImageFilter_NoHandles_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FastMarchingBaseImageFilter__TopologyCheckType");
  
  ans
  
}

attr(`FastMarchingBaseImageFilter_NoHandles_get`, 'returnType') = 'character'
class(`FastMarchingBaseImageFilter_NoHandles_get`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_NoHandles_get'))

# Start of FastMarchingBaseImageFilter_Strict_get

`FastMarchingBaseImageFilter_Strict_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_FastMarchingBaseImageFilter_Strict_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FastMarchingBaseImageFilter__TopologyCheckType");
  
  ans
  
}

attr(`FastMarchingBaseImageFilter_Strict_get`, 'returnType') = 'character'
class(`FastMarchingBaseImageFilter_Strict_get`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_Strict_get'))

# Start of FastMarchingBaseImageFilter_SetTopologyCheck

`FastMarchingBaseImageFilter_SetTopologyCheck` = function(self, TopologyCheck)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  TopologyCheck = enumToInteger(TopologyCheck, "_itk__simple__FastMarchingBaseImageFilter__TopologyCheckType"); 
  
  if(length(TopologyCheck) > 1) {
    warning("using only the first element of TopologyCheck");
  };
  
  ;ans = .Call('R_swig_FastMarchingBaseImageFilter_SetTopologyCheck', self, TopologyCheck, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingBaseImageFilter_SetTopologyCheck`, 'returnType') = 'void'
attr(`FastMarchingBaseImageFilter_SetTopologyCheck`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter', 'character')
class(`FastMarchingBaseImageFilter_SetTopologyCheck`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_SetTopologyCheck'))

# Start of FastMarchingBaseImageFilter_GetTopologyCheck

`FastMarchingBaseImageFilter_GetTopologyCheck` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FastMarchingBaseImageFilter_GetTopologyCheck', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FastMarchingBaseImageFilter__TopologyCheckType");
  
  ans
  
}

attr(`FastMarchingBaseImageFilter_GetTopologyCheck`, 'returnType') = 'character'
attr(`FastMarchingBaseImageFilter_GetTopologyCheck`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter')
class(`FastMarchingBaseImageFilter_GetTopologyCheck`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_GetTopologyCheck'))

# Start of FastMarchingBaseImageFilter_SetInitialTrialValues

`FastMarchingBaseImageFilter_SetInitialTrialValues` = function(self, InitialTrialValues)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InitialTrialValues = as.numeric(InitialTrialValues);
  ;ans = .Call('R_swig_FastMarchingBaseImageFilter_SetInitialTrialValues', self, InitialTrialValues, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingBaseImageFilter_SetInitialTrialValues`, 'returnType') = 'void'
attr(`FastMarchingBaseImageFilter_SetInitialTrialValues`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter', 'numeric')
class(`FastMarchingBaseImageFilter_SetInitialTrialValues`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_SetInitialTrialValues'))

# Start of FastMarchingBaseImageFilter_GetInitialTrialValues

`FastMarchingBaseImageFilter_GetInitialTrialValues` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingBaseImageFilter_GetInitialTrialValues', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingBaseImageFilter_GetInitialTrialValues`, 'returnType') = 'numeric'
attr(`FastMarchingBaseImageFilter_GetInitialTrialValues`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter')
class(`FastMarchingBaseImageFilter_GetInitialTrialValues`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_GetInitialTrialValues'))

# Start of FastMarchingBaseImageFilter_GetName

`FastMarchingBaseImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingBaseImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingBaseImageFilter_GetName`, 'returnType') = 'character'
attr(`FastMarchingBaseImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter')
class(`FastMarchingBaseImageFilter_GetName`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_GetName'))

# Start of FastMarchingBaseImageFilter_ToString

`FastMarchingBaseImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingBaseImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingBaseImageFilter_ToString`, 'returnType') = 'character'
attr(`FastMarchingBaseImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter')
class(`FastMarchingBaseImageFilter_ToString`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_ToString'))

# Start of FastMarchingBaseImageFilter_Execute

`FastMarchingBaseImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_FastMarchingBaseImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingBaseImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingBaseImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__FastMarchingBaseImageFilter', '_p_itk__simple__Image')
class(`FastMarchingBaseImageFilter_Execute`) = c("SWIGFunction", class('FastMarchingBaseImageFilter_Execute'))

# Start of accessor method for itk::simple::FastMarchingBaseImageFilter
setMethod('$', '_p_itk__simple__FastMarchingBaseImageFilter', function(x, name)

{
  accessorFuns = list('SetTrialPoints' = FastMarchingBaseImageFilter_SetTrialPoints, 'GetTrialPoints' = FastMarchingBaseImageFilter_GetTrialPoints, 'AddTrialPoint' = FastMarchingBaseImageFilter_AddTrialPoint, 'ClearTrialPoints' = FastMarchingBaseImageFilter_ClearTrialPoints, 'SetNormalizationFactor' = FastMarchingBaseImageFilter_SetNormalizationFactor, 'GetNormalizationFactor' = FastMarchingBaseImageFilter_GetNormalizationFactor, 'SetStoppingValue' = FastMarchingBaseImageFilter_SetStoppingValue, 'GetStoppingValue' = FastMarchingBaseImageFilter_GetStoppingValue, 'SetTopologyCheck' = FastMarchingBaseImageFilter_SetTopologyCheck, 'GetTopologyCheck' = FastMarchingBaseImageFilter_GetTopologyCheck, 'SetInitialTrialValues' = FastMarchingBaseImageFilter_SetInitialTrialValues, 'GetInitialTrialValues' = FastMarchingBaseImageFilter_GetInitialTrialValues, 'GetName' = FastMarchingBaseImageFilter_GetName, 'ToString' = FastMarchingBaseImageFilter_ToString, 'Execute' = FastMarchingBaseImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::FastMarchingBaseImageFilter
# Start of FastMarchingBase

`FastMarchingBase__SWIG_0` = function(image1, trialPoints, normalizationFactor, stoppingValue, topologyCheck, initialTrialValues, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  trialPoints = lapply(trialPoints, as.integer);
  
  
  topologyCheck = enumToInteger(topologyCheck, "_itk__simple__FastMarchingBaseImageFilter__TopologyCheckType"); 
  
  if(length(topologyCheck) > 1) {
    warning("using only the first element of topologyCheck");
  };
  
  initialTrialValues = as.numeric(initialTrialValues);
  ;ans = .Call('R_swig_FastMarchingBase__SWIG_0', image1, trialPoints, normalizationFactor, stoppingValue, topologyCheck, initialTrialValues, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingBase__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingBase__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'numeric', 'numeric', 'character', 'numeric')
class(`FastMarchingBase__SWIG_0`) = c("SWIGFunction", class('FastMarchingBase__SWIG_0'))

# Start of FastMarchingBase

`FastMarchingBase__SWIG_1` = function(image1, trialPoints, normalizationFactor, stoppingValue, topologyCheck, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  trialPoints = lapply(trialPoints, as.integer);
  
  
  topologyCheck = enumToInteger(topologyCheck, "_itk__simple__FastMarchingBaseImageFilter__TopologyCheckType"); 
  
  if(length(topologyCheck) > 1) {
    warning("using only the first element of topologyCheck");
  };
  
  ;ans = .Call('R_swig_FastMarchingBase__SWIG_1', image1, trialPoints, normalizationFactor, stoppingValue, topologyCheck, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingBase__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingBase__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'numeric', 'numeric', 'character')
class(`FastMarchingBase__SWIG_1`) = c("SWIGFunction", class('FastMarchingBase__SWIG_1'))

# Start of FastMarchingBase

`FastMarchingBase__SWIG_2` = function(image1, trialPoints, normalizationFactor, stoppingValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  trialPoints = lapply(trialPoints, as.integer);
  
  
  ;ans = .Call('R_swig_FastMarchingBase__SWIG_2', image1, trialPoints, normalizationFactor, stoppingValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingBase__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingBase__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'numeric', 'numeric')
class(`FastMarchingBase__SWIG_2`) = c("SWIGFunction", class('FastMarchingBase__SWIG_2'))

# Start of FastMarchingBase

`FastMarchingBase__SWIG_3` = function(image1, trialPoints, normalizationFactor, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  trialPoints = lapply(trialPoints, as.integer);
  
  ;ans = .Call('R_swig_FastMarchingBase__SWIG_3', image1, trialPoints, normalizationFactor, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingBase__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingBase__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'numeric')
class(`FastMarchingBase__SWIG_3`) = c("SWIGFunction", class('FastMarchingBase__SWIG_3'))

# Start of FastMarchingBase

`FastMarchingBase__SWIG_4` = function(image1, trialPoints, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  trialPoints = lapply(trialPoints, as.integer);
  ;ans = .Call('R_swig_FastMarchingBase__SWIG_4', image1, trialPoints, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingBase__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingBase__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'list')
class(`FastMarchingBase__SWIG_4`) = c("SWIGFunction", class('FastMarchingBase__SWIG_4'))

# Start of FastMarchingBase

`FastMarchingBase__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_FastMarchingBase__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingBase__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingBase__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`FastMarchingBase__SWIG_5`) = c("SWIGFunction", class('FastMarchingBase__SWIG_5'))

`FastMarchingBase` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- FastMarchingBase__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) )) {
      f <- FastMarchingBase__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- FastMarchingBase__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- FastMarchingBase__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.character(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- FastMarchingBase__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.character(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) )) {
      f <- FastMarchingBase__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for FastMarchingBase with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_FastMarchingImageFilter

`delete_FastMarchingImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_FastMarchingImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_FastMarchingImageFilter`, 'returnType') = 'void'
attr(`delete_FastMarchingImageFilter`, "inputTypes") = c('_p_itk__simple__FastMarchingImageFilter')
class(`delete_FastMarchingImageFilter`) = c("SWIGFunction", class('delete_FastMarchingImageFilter'))

# Start of new_FastMarchingImageFilter

`FastMarchingImageFilter` = function()
{
  ;ans = .Call('R_swig_new_FastMarchingImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__FastMarchingImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_FastMarchingImageFilter);
  ans
  
}

attr(`FastMarchingImageFilter`, 'returnType') = '_p_itk__simple__FastMarchingImageFilter'
class(`FastMarchingImageFilter`) = c("SWIGFunction", class('FastMarchingImageFilter'))

# Start of FastMarchingImageFilter_SetTrialPoints

`FastMarchingImageFilter_SetTrialPoints` = function(self, TrialPoints)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  TrialPoints = lapply(TrialPoints, as.integer);
  ;ans = .Call('R_swig_FastMarchingImageFilter_SetTrialPoints', self, TrialPoints, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingImageFilter_SetTrialPoints`, 'returnType') = 'void'
attr(`FastMarchingImageFilter_SetTrialPoints`, "inputTypes") = c('_p_itk__simple__FastMarchingImageFilter', 'list')
class(`FastMarchingImageFilter_SetTrialPoints`) = c("SWIGFunction", class('FastMarchingImageFilter_SetTrialPoints'))

# Start of FastMarchingImageFilter_GetTrialPoints

`FastMarchingImageFilter_GetTrialPoints` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingImageFilter_GetTrialPoints', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingImageFilter_GetTrialPoints`, 'returnType') = 'list'
attr(`FastMarchingImageFilter_GetTrialPoints`, "inputTypes") = c('_p_itk__simple__FastMarchingImageFilter')
class(`FastMarchingImageFilter_GetTrialPoints`) = c("SWIGFunction", class('FastMarchingImageFilter_GetTrialPoints'))

# Start of FastMarchingImageFilter_AddTrialPoint

`FastMarchingImageFilter_AddTrialPoint` = function(self, point)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  point = as.integer(point);
  ;ans = .Call('R_swig_FastMarchingImageFilter_AddTrialPoint', self, point, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingImageFilter_AddTrialPoint`, 'returnType') = 'void'
attr(`FastMarchingImageFilter_AddTrialPoint`, "inputTypes") = c('_p_itk__simple__FastMarchingImageFilter', 'integer')
class(`FastMarchingImageFilter_AddTrialPoint`) = c("SWIGFunction", class('FastMarchingImageFilter_AddTrialPoint'))

# Start of FastMarchingImageFilter_ClearTrialPoints

`FastMarchingImageFilter_ClearTrialPoints` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FastMarchingImageFilter_ClearTrialPoints', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingImageFilter_ClearTrialPoints`, 'returnType') = 'void'
attr(`FastMarchingImageFilter_ClearTrialPoints`, "inputTypes") = c('_p_itk__simple__FastMarchingImageFilter')
class(`FastMarchingImageFilter_ClearTrialPoints`) = c("SWIGFunction", class('FastMarchingImageFilter_ClearTrialPoints'))

# Start of FastMarchingImageFilter_SetNormalizationFactor

`FastMarchingImageFilter_SetNormalizationFactor` = function(self, NormalizationFactor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_FastMarchingImageFilter_SetNormalizationFactor', self, NormalizationFactor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingImageFilter_SetNormalizationFactor`, 'returnType') = 'void'
attr(`FastMarchingImageFilter_SetNormalizationFactor`, "inputTypes") = c('_p_itk__simple__FastMarchingImageFilter', 'numeric')
class(`FastMarchingImageFilter_SetNormalizationFactor`) = c("SWIGFunction", class('FastMarchingImageFilter_SetNormalizationFactor'))

# Start of FastMarchingImageFilter_GetNormalizationFactor

`FastMarchingImageFilter_GetNormalizationFactor` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingImageFilter_GetNormalizationFactor', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingImageFilter_GetNormalizationFactor`, 'returnType') = 'numeric'
attr(`FastMarchingImageFilter_GetNormalizationFactor`, "inputTypes") = c('_p_itk__simple__FastMarchingImageFilter')
class(`FastMarchingImageFilter_GetNormalizationFactor`) = c("SWIGFunction", class('FastMarchingImageFilter_GetNormalizationFactor'))

# Start of FastMarchingImageFilter_SetStoppingValue

`FastMarchingImageFilter_SetStoppingValue` = function(self, StoppingValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_FastMarchingImageFilter_SetStoppingValue', self, StoppingValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingImageFilter_SetStoppingValue`, 'returnType') = 'void'
attr(`FastMarchingImageFilter_SetStoppingValue`, "inputTypes") = c('_p_itk__simple__FastMarchingImageFilter', 'numeric')
class(`FastMarchingImageFilter_SetStoppingValue`) = c("SWIGFunction", class('FastMarchingImageFilter_SetStoppingValue'))

# Start of FastMarchingImageFilter_GetStoppingValue

`FastMarchingImageFilter_GetStoppingValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingImageFilter_GetStoppingValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingImageFilter_GetStoppingValue`, 'returnType') = 'numeric'
attr(`FastMarchingImageFilter_GetStoppingValue`, "inputTypes") = c('_p_itk__simple__FastMarchingImageFilter')
class(`FastMarchingImageFilter_GetStoppingValue`) = c("SWIGFunction", class('FastMarchingImageFilter_GetStoppingValue'))

# Start of FastMarchingImageFilter_SetInitialTrialValues

`FastMarchingImageFilter_SetInitialTrialValues` = function(self, InitialTrialValues)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InitialTrialValues = as.numeric(InitialTrialValues);
  ;ans = .Call('R_swig_FastMarchingImageFilter_SetInitialTrialValues', self, InitialTrialValues, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingImageFilter_SetInitialTrialValues`, 'returnType') = 'void'
attr(`FastMarchingImageFilter_SetInitialTrialValues`, "inputTypes") = c('_p_itk__simple__FastMarchingImageFilter', 'numeric')
class(`FastMarchingImageFilter_SetInitialTrialValues`) = c("SWIGFunction", class('FastMarchingImageFilter_SetInitialTrialValues'))

# Start of FastMarchingImageFilter_GetInitialTrialValues

`FastMarchingImageFilter_GetInitialTrialValues` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingImageFilter_GetInitialTrialValues', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingImageFilter_GetInitialTrialValues`, 'returnType') = 'numeric'
attr(`FastMarchingImageFilter_GetInitialTrialValues`, "inputTypes") = c('_p_itk__simple__FastMarchingImageFilter')
class(`FastMarchingImageFilter_GetInitialTrialValues`) = c("SWIGFunction", class('FastMarchingImageFilter_GetInitialTrialValues'))

# Start of FastMarchingImageFilter_GetName

`FastMarchingImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingImageFilter_GetName`, 'returnType') = 'character'
attr(`FastMarchingImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__FastMarchingImageFilter')
class(`FastMarchingImageFilter_GetName`) = c("SWIGFunction", class('FastMarchingImageFilter_GetName'))

# Start of FastMarchingImageFilter_ToString

`FastMarchingImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingImageFilter_ToString`, 'returnType') = 'character'
attr(`FastMarchingImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__FastMarchingImageFilter')
class(`FastMarchingImageFilter_ToString`) = c("SWIGFunction", class('FastMarchingImageFilter_ToString'))

# Start of FastMarchingImageFilter_Execute

`FastMarchingImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_FastMarchingImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__FastMarchingImageFilter', '_p_itk__simple__Image')
class(`FastMarchingImageFilter_Execute`) = c("SWIGFunction", class('FastMarchingImageFilter_Execute'))

# Start of accessor method for itk::simple::FastMarchingImageFilter
setMethod('$', '_p_itk__simple__FastMarchingImageFilter', function(x, name)

{
  accessorFuns = list('SetTrialPoints' = FastMarchingImageFilter_SetTrialPoints, 'GetTrialPoints' = FastMarchingImageFilter_GetTrialPoints, 'AddTrialPoint' = FastMarchingImageFilter_AddTrialPoint, 'ClearTrialPoints' = FastMarchingImageFilter_ClearTrialPoints, 'SetNormalizationFactor' = FastMarchingImageFilter_SetNormalizationFactor, 'GetNormalizationFactor' = FastMarchingImageFilter_GetNormalizationFactor, 'SetStoppingValue' = FastMarchingImageFilter_SetStoppingValue, 'GetStoppingValue' = FastMarchingImageFilter_GetStoppingValue, 'SetInitialTrialValues' = FastMarchingImageFilter_SetInitialTrialValues, 'GetInitialTrialValues' = FastMarchingImageFilter_GetInitialTrialValues, 'GetName' = FastMarchingImageFilter_GetName, 'ToString' = FastMarchingImageFilter_ToString, 'Execute' = FastMarchingImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::FastMarchingImageFilter
# Start of FastMarching

`FastMarching__SWIG_0` = function(image1, trialPoints, normalizationFactor, stoppingValue, initialTrialValues, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  trialPoints = lapply(trialPoints, as.integer);
  
  
  initialTrialValues = as.numeric(initialTrialValues);
  ;ans = .Call('R_swig_FastMarching__SWIG_0', image1, trialPoints, normalizationFactor, stoppingValue, initialTrialValues, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarching__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarching__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'numeric', 'numeric', 'numeric')
class(`FastMarching__SWIG_0`) = c("SWIGFunction", class('FastMarching__SWIG_0'))

# Start of FastMarching

`FastMarching__SWIG_1` = function(image1, trialPoints, normalizationFactor, stoppingValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  trialPoints = lapply(trialPoints, as.integer);
  
  
  ;ans = .Call('R_swig_FastMarching__SWIG_1', image1, trialPoints, normalizationFactor, stoppingValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarching__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarching__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'numeric', 'numeric')
class(`FastMarching__SWIG_1`) = c("SWIGFunction", class('FastMarching__SWIG_1'))

# Start of FastMarching

`FastMarching__SWIG_2` = function(image1, trialPoints, normalizationFactor, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  trialPoints = lapply(trialPoints, as.integer);
  
  ;ans = .Call('R_swig_FastMarching__SWIG_2', image1, trialPoints, normalizationFactor, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarching__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarching__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'numeric')
class(`FastMarching__SWIG_2`) = c("SWIGFunction", class('FastMarching__SWIG_2'))

# Start of FastMarching

`FastMarching__SWIG_3` = function(image1, trialPoints, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  trialPoints = lapply(trialPoints, as.integer);
  ;ans = .Call('R_swig_FastMarching__SWIG_3', image1, trialPoints, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarching__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarching__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'list')
class(`FastMarching__SWIG_3`) = c("SWIGFunction", class('FastMarching__SWIG_3'))

# Start of FastMarching

`FastMarching__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_FastMarching__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarching__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarching__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`FastMarching__SWIG_4`) = c("SWIGFunction", class('FastMarching__SWIG_4'))

`FastMarching` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- FastMarching__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) )) {
      f <- FastMarching__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- FastMarching__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- FastMarching__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) )) {
      f <- FastMarching__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for FastMarching with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_FastMarchingUpwindGradientImageFilter

`delete_FastMarchingUpwindGradientImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_FastMarchingUpwindGradientImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_FastMarchingUpwindGradientImageFilter`, 'returnType') = 'void'
attr(`delete_FastMarchingUpwindGradientImageFilter`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter')
class(`delete_FastMarchingUpwindGradientImageFilter`) = c("SWIGFunction", class('delete_FastMarchingUpwindGradientImageFilter'))

# Start of new_FastMarchingUpwindGradientImageFilter

`FastMarchingUpwindGradientImageFilter` = function()
{
  ;ans = .Call('R_swig_new_FastMarchingUpwindGradientImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__FastMarchingUpwindGradientImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_FastMarchingUpwindGradientImageFilter);
  ans
  
}

attr(`FastMarchingUpwindGradientImageFilter`, 'returnType') = '_p_itk__simple__FastMarchingUpwindGradientImageFilter'
class(`FastMarchingUpwindGradientImageFilter`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter'))

# Start of FastMarchingUpwindGradientImageFilter_SetTrialPoints

`FastMarchingUpwindGradientImageFilter_SetTrialPoints` = function(self, TrialPoints)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  TrialPoints = lapply(TrialPoints, as.integer);
  ;ans = .Call('R_swig_FastMarchingUpwindGradientImageFilter_SetTrialPoints', self, TrialPoints, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradientImageFilter_SetTrialPoints`, 'returnType') = 'void'
attr(`FastMarchingUpwindGradientImageFilter_SetTrialPoints`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter', 'list')
class(`FastMarchingUpwindGradientImageFilter_SetTrialPoints`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_SetTrialPoints'))

# Start of FastMarchingUpwindGradientImageFilter_GetTrialPoints

`FastMarchingUpwindGradientImageFilter_GetTrialPoints` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingUpwindGradientImageFilter_GetTrialPoints', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingUpwindGradientImageFilter_GetTrialPoints`, 'returnType') = 'list'
attr(`FastMarchingUpwindGradientImageFilter_GetTrialPoints`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter')
class(`FastMarchingUpwindGradientImageFilter_GetTrialPoints`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_GetTrialPoints'))

# Start of FastMarchingUpwindGradientImageFilter_AddTrialPoint

`FastMarchingUpwindGradientImageFilter_AddTrialPoint` = function(self, point)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  point = as.integer(point);
  ;ans = .Call('R_swig_FastMarchingUpwindGradientImageFilter_AddTrialPoint', self, point, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradientImageFilter_AddTrialPoint`, 'returnType') = 'void'
attr(`FastMarchingUpwindGradientImageFilter_AddTrialPoint`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter', 'integer')
class(`FastMarchingUpwindGradientImageFilter_AddTrialPoint`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_AddTrialPoint'))

# Start of FastMarchingUpwindGradientImageFilter_ClearTrialPoints

`FastMarchingUpwindGradientImageFilter_ClearTrialPoints` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FastMarchingUpwindGradientImageFilter_ClearTrialPoints', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradientImageFilter_ClearTrialPoints`, 'returnType') = 'void'
attr(`FastMarchingUpwindGradientImageFilter_ClearTrialPoints`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter')
class(`FastMarchingUpwindGradientImageFilter_ClearTrialPoints`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_ClearTrialPoints'))

# Start of FastMarchingUpwindGradientImageFilter_SetNumberOfTargets

`FastMarchingUpwindGradientImageFilter_SetNumberOfTargets` = function(self, NumberOfTargets)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfTargets = as.integer(NumberOfTargets);
  
  if(length(NumberOfTargets) > 1) {
    warning("using only the first element of NumberOfTargets");
  };
  
  ;ans = .Call('R_swig_FastMarchingUpwindGradientImageFilter_SetNumberOfTargets', self, NumberOfTargets, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradientImageFilter_SetNumberOfTargets`, 'returnType') = 'void'
attr(`FastMarchingUpwindGradientImageFilter_SetNumberOfTargets`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter', 'integer')
class(`FastMarchingUpwindGradientImageFilter_SetNumberOfTargets`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_SetNumberOfTargets'))

# Start of FastMarchingUpwindGradientImageFilter_GetNumberOfTargets

`FastMarchingUpwindGradientImageFilter_GetNumberOfTargets` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingUpwindGradientImageFilter_GetNumberOfTargets', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingUpwindGradientImageFilter_GetNumberOfTargets`, 'returnType') = 'integer'
attr(`FastMarchingUpwindGradientImageFilter_GetNumberOfTargets`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter')
class(`FastMarchingUpwindGradientImageFilter_GetNumberOfTargets`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_GetNumberOfTargets'))

# Start of FastMarchingUpwindGradientImageFilter_SetTargetPoints

`FastMarchingUpwindGradientImageFilter_SetTargetPoints` = function(self, TargetPoints)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  TargetPoints = lapply(TargetPoints, as.integer);
  ;ans = .Call('R_swig_FastMarchingUpwindGradientImageFilter_SetTargetPoints', self, TargetPoints, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradientImageFilter_SetTargetPoints`, 'returnType') = 'void'
attr(`FastMarchingUpwindGradientImageFilter_SetTargetPoints`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter', 'list')
class(`FastMarchingUpwindGradientImageFilter_SetTargetPoints`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_SetTargetPoints'))

# Start of FastMarchingUpwindGradientImageFilter_GetTargetPoints

`FastMarchingUpwindGradientImageFilter_GetTargetPoints` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingUpwindGradientImageFilter_GetTargetPoints', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingUpwindGradientImageFilter_GetTargetPoints`, 'returnType') = 'list'
attr(`FastMarchingUpwindGradientImageFilter_GetTargetPoints`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter')
class(`FastMarchingUpwindGradientImageFilter_GetTargetPoints`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_GetTargetPoints'))

# Start of FastMarchingUpwindGradientImageFilter_AddTargetPoint

`FastMarchingUpwindGradientImageFilter_AddTargetPoint` = function(self, point)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  point = as.integer(point);
  ;ans = .Call('R_swig_FastMarchingUpwindGradientImageFilter_AddTargetPoint', self, point, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradientImageFilter_AddTargetPoint`, 'returnType') = 'void'
attr(`FastMarchingUpwindGradientImageFilter_AddTargetPoint`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter', 'integer')
class(`FastMarchingUpwindGradientImageFilter_AddTargetPoint`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_AddTargetPoint'))

# Start of FastMarchingUpwindGradientImageFilter_ClearTargetPoints

`FastMarchingUpwindGradientImageFilter_ClearTargetPoints` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FastMarchingUpwindGradientImageFilter_ClearTargetPoints', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradientImageFilter_ClearTargetPoints`, 'returnType') = 'void'
attr(`FastMarchingUpwindGradientImageFilter_ClearTargetPoints`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter')
class(`FastMarchingUpwindGradientImageFilter_ClearTargetPoints`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_ClearTargetPoints'))

# Start of FastMarchingUpwindGradientImageFilter_SetTargetOffset

`FastMarchingUpwindGradientImageFilter_SetTargetOffset` = function(self, TargetOffset)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_FastMarchingUpwindGradientImageFilter_SetTargetOffset', self, TargetOffset, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradientImageFilter_SetTargetOffset`, 'returnType') = 'void'
attr(`FastMarchingUpwindGradientImageFilter_SetTargetOffset`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter', 'numeric')
class(`FastMarchingUpwindGradientImageFilter_SetTargetOffset`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_SetTargetOffset'))

# Start of FastMarchingUpwindGradientImageFilter_GetTargetOffset

`FastMarchingUpwindGradientImageFilter_GetTargetOffset` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingUpwindGradientImageFilter_GetTargetOffset', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingUpwindGradientImageFilter_GetTargetOffset`, 'returnType') = 'numeric'
attr(`FastMarchingUpwindGradientImageFilter_GetTargetOffset`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter')
class(`FastMarchingUpwindGradientImageFilter_GetTargetOffset`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_GetTargetOffset'))

# Start of FastMarchingUpwindGradientImageFilter_SetNormalizationFactor

`FastMarchingUpwindGradientImageFilter_SetNormalizationFactor` = function(self, NormalizationFactor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_FastMarchingUpwindGradientImageFilter_SetNormalizationFactor', self, NormalizationFactor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradientImageFilter_SetNormalizationFactor`, 'returnType') = 'void'
attr(`FastMarchingUpwindGradientImageFilter_SetNormalizationFactor`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter', 'numeric')
class(`FastMarchingUpwindGradientImageFilter_SetNormalizationFactor`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_SetNormalizationFactor'))

# Start of FastMarchingUpwindGradientImageFilter_GetNormalizationFactor

`FastMarchingUpwindGradientImageFilter_GetNormalizationFactor` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingUpwindGradientImageFilter_GetNormalizationFactor', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingUpwindGradientImageFilter_GetNormalizationFactor`, 'returnType') = 'numeric'
attr(`FastMarchingUpwindGradientImageFilter_GetNormalizationFactor`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter')
class(`FastMarchingUpwindGradientImageFilter_GetNormalizationFactor`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_GetNormalizationFactor'))

# Start of FastMarchingUpwindGradientImageFilter_SetInitialTrialValues

`FastMarchingUpwindGradientImageFilter_SetInitialTrialValues` = function(self, InitialTrialValues)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InitialTrialValues = as.numeric(InitialTrialValues);
  ;ans = .Call('R_swig_FastMarchingUpwindGradientImageFilter_SetInitialTrialValues', self, InitialTrialValues, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradientImageFilter_SetInitialTrialValues`, 'returnType') = 'void'
attr(`FastMarchingUpwindGradientImageFilter_SetInitialTrialValues`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter', 'numeric')
class(`FastMarchingUpwindGradientImageFilter_SetInitialTrialValues`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_SetInitialTrialValues'))

# Start of FastMarchingUpwindGradientImageFilter_GetInitialTrialValues

`FastMarchingUpwindGradientImageFilter_GetInitialTrialValues` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingUpwindGradientImageFilter_GetInitialTrialValues', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingUpwindGradientImageFilter_GetInitialTrialValues`, 'returnType') = 'numeric'
attr(`FastMarchingUpwindGradientImageFilter_GetInitialTrialValues`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter')
class(`FastMarchingUpwindGradientImageFilter_GetInitialTrialValues`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_GetInitialTrialValues'))

# Start of FastMarchingUpwindGradientImageFilter_GetGradientImage

`FastMarchingUpwindGradientImageFilter_GetGradientImage` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FastMarchingUpwindGradientImageFilter_GetGradientImage', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradientImageFilter_GetGradientImage`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingUpwindGradientImageFilter_GetGradientImage`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter')
class(`FastMarchingUpwindGradientImageFilter_GetGradientImage`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_GetGradientImage'))

# Start of FastMarchingUpwindGradientImageFilter_GetTargetValue

`FastMarchingUpwindGradientImageFilter_GetTargetValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingUpwindGradientImageFilter_GetTargetValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingUpwindGradientImageFilter_GetTargetValue`, 'returnType') = 'numeric'
attr(`FastMarchingUpwindGradientImageFilter_GetTargetValue`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter')
class(`FastMarchingUpwindGradientImageFilter_GetTargetValue`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_GetTargetValue'))

# Start of FastMarchingUpwindGradientImageFilter_GetName

`FastMarchingUpwindGradientImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingUpwindGradientImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingUpwindGradientImageFilter_GetName`, 'returnType') = 'character'
attr(`FastMarchingUpwindGradientImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter')
class(`FastMarchingUpwindGradientImageFilter_GetName`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_GetName'))

# Start of FastMarchingUpwindGradientImageFilter_ToString

`FastMarchingUpwindGradientImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastMarchingUpwindGradientImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastMarchingUpwindGradientImageFilter_ToString`, 'returnType') = 'character'
attr(`FastMarchingUpwindGradientImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter')
class(`FastMarchingUpwindGradientImageFilter_ToString`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_ToString'))

# Start of FastMarchingUpwindGradientImageFilter_Execute

`FastMarchingUpwindGradientImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_FastMarchingUpwindGradientImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradientImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingUpwindGradientImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__FastMarchingUpwindGradientImageFilter', '_p_itk__simple__Image')
class(`FastMarchingUpwindGradientImageFilter_Execute`) = c("SWIGFunction", class('FastMarchingUpwindGradientImageFilter_Execute'))

# Start of accessor method for itk::simple::FastMarchingUpwindGradientImageFilter
setMethod('$', '_p_itk__simple__FastMarchingUpwindGradientImageFilter', function(x, name)

{
  accessorFuns = list('SetTrialPoints' = FastMarchingUpwindGradientImageFilter_SetTrialPoints, 'GetTrialPoints' = FastMarchingUpwindGradientImageFilter_GetTrialPoints, 'AddTrialPoint' = FastMarchingUpwindGradientImageFilter_AddTrialPoint, 'ClearTrialPoints' = FastMarchingUpwindGradientImageFilter_ClearTrialPoints, 'SetNumberOfTargets' = FastMarchingUpwindGradientImageFilter_SetNumberOfTargets, 'GetNumberOfTargets' = FastMarchingUpwindGradientImageFilter_GetNumberOfTargets, 'SetTargetPoints' = FastMarchingUpwindGradientImageFilter_SetTargetPoints, 'GetTargetPoints' = FastMarchingUpwindGradientImageFilter_GetTargetPoints, 'AddTargetPoint' = FastMarchingUpwindGradientImageFilter_AddTargetPoint, 'ClearTargetPoints' = FastMarchingUpwindGradientImageFilter_ClearTargetPoints, 'SetTargetOffset' = FastMarchingUpwindGradientImageFilter_SetTargetOffset, 'GetTargetOffset' = FastMarchingUpwindGradientImageFilter_GetTargetOffset, 'SetNormalizationFactor' = FastMarchingUpwindGradientImageFilter_SetNormalizationFactor, 'GetNormalizationFactor' = FastMarchingUpwindGradientImageFilter_GetNormalizationFactor, 'SetInitialTrialValues' = FastMarchingUpwindGradientImageFilter_SetInitialTrialValues, 'GetInitialTrialValues' = FastMarchingUpwindGradientImageFilter_GetInitialTrialValues, 'GetGradientImage' = FastMarchingUpwindGradientImageFilter_GetGradientImage, 'GetTargetValue' = FastMarchingUpwindGradientImageFilter_GetTargetValue, 'GetName' = FastMarchingUpwindGradientImageFilter_GetName, 'ToString' = FastMarchingUpwindGradientImageFilter_ToString, 'Execute' = FastMarchingUpwindGradientImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::FastMarchingUpwindGradientImageFilter
# Start of FastMarchingUpwindGradient

`FastMarchingUpwindGradient__SWIG_0` = function(image1, trialPoints, numberOfTargets, targetPoints, targetOffset, normalizationFactor, initialTrialValues, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  trialPoints = lapply(trialPoints, as.integer);
  numberOfTargets = as.integer(numberOfTargets);
  
  if(length(numberOfTargets) > 1) {
    warning("using only the first element of numberOfTargets");
  };
  
  targetPoints = lapply(targetPoints, as.integer);
  
  
  initialTrialValues = as.numeric(initialTrialValues);
  ;ans = .Call('R_swig_FastMarchingUpwindGradient__SWIG_0', image1, trialPoints, numberOfTargets, targetPoints, targetOffset, normalizationFactor, initialTrialValues, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradient__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingUpwindGradient__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'integer', 'list', 'numeric', 'numeric', 'numeric')
class(`FastMarchingUpwindGradient__SWIG_0`) = c("SWIGFunction", class('FastMarchingUpwindGradient__SWIG_0'))

# Start of FastMarchingUpwindGradient

`FastMarchingUpwindGradient__SWIG_1` = function(image1, trialPoints, numberOfTargets, targetPoints, targetOffset, normalizationFactor, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  trialPoints = lapply(trialPoints, as.integer);
  numberOfTargets = as.integer(numberOfTargets);
  
  if(length(numberOfTargets) > 1) {
    warning("using only the first element of numberOfTargets");
  };
  
  targetPoints = lapply(targetPoints, as.integer);
  
  
  ;ans = .Call('R_swig_FastMarchingUpwindGradient__SWIG_1', image1, trialPoints, numberOfTargets, targetPoints, targetOffset, normalizationFactor, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradient__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingUpwindGradient__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'integer', 'list', 'numeric', 'numeric')
class(`FastMarchingUpwindGradient__SWIG_1`) = c("SWIGFunction", class('FastMarchingUpwindGradient__SWIG_1'))

# Start of FastMarchingUpwindGradient

`FastMarchingUpwindGradient__SWIG_2` = function(image1, trialPoints, numberOfTargets, targetPoints, targetOffset, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  trialPoints = lapply(trialPoints, as.integer);
  numberOfTargets = as.integer(numberOfTargets);
  
  if(length(numberOfTargets) > 1) {
    warning("using only the first element of numberOfTargets");
  };
  
  targetPoints = lapply(targetPoints, as.integer);
  
  ;ans = .Call('R_swig_FastMarchingUpwindGradient__SWIG_2', image1, trialPoints, numberOfTargets, targetPoints, targetOffset, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradient__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingUpwindGradient__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'integer', 'list', 'numeric')
class(`FastMarchingUpwindGradient__SWIG_2`) = c("SWIGFunction", class('FastMarchingUpwindGradient__SWIG_2'))

# Start of FastMarchingUpwindGradient

`FastMarchingUpwindGradient__SWIG_3` = function(image1, trialPoints, numberOfTargets, targetPoints, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  trialPoints = lapply(trialPoints, as.integer);
  numberOfTargets = as.integer(numberOfTargets);
  
  if(length(numberOfTargets) > 1) {
    warning("using only the first element of numberOfTargets");
  };
  
  targetPoints = lapply(targetPoints, as.integer);
  ;ans = .Call('R_swig_FastMarchingUpwindGradient__SWIG_3', image1, trialPoints, numberOfTargets, targetPoints, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradient__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingUpwindGradient__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'integer', 'list')
class(`FastMarchingUpwindGradient__SWIG_3`) = c("SWIGFunction", class('FastMarchingUpwindGradient__SWIG_3'))

# Start of FastMarchingUpwindGradient

`FastMarchingUpwindGradient__SWIG_4` = function(image1, trialPoints, numberOfTargets, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  trialPoints = lapply(trialPoints, as.integer);
  numberOfTargets = as.integer(numberOfTargets);
  
  if(length(numberOfTargets) > 1) {
    warning("using only the first element of numberOfTargets");
  };
  
  ;ans = .Call('R_swig_FastMarchingUpwindGradient__SWIG_4', image1, trialPoints, numberOfTargets, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradient__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingUpwindGradient__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'integer')
class(`FastMarchingUpwindGradient__SWIG_4`) = c("SWIGFunction", class('FastMarchingUpwindGradient__SWIG_4'))

# Start of FastMarchingUpwindGradient

`FastMarchingUpwindGradient__SWIG_5` = function(image1, trialPoints, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  trialPoints = lapply(trialPoints, as.integer);
  ;ans = .Call('R_swig_FastMarchingUpwindGradient__SWIG_5', image1, trialPoints, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradient__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingUpwindGradient__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'list')
class(`FastMarchingUpwindGradient__SWIG_5`) = c("SWIGFunction", class('FastMarchingUpwindGradient__SWIG_5'))

# Start of FastMarchingUpwindGradient

`FastMarchingUpwindGradient__SWIG_6` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_FastMarchingUpwindGradient__SWIG_6', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastMarchingUpwindGradient__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`FastMarchingUpwindGradient__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image')
class(`FastMarchingUpwindGradient__SWIG_6`) = c("SWIGFunction", class('FastMarchingUpwindGradient__SWIG_6'))

`FastMarchingUpwindGradient` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 7
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- FastMarchingUpwindGradient__SWIG_6; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) )) {
      f <- FastMarchingUpwindGradient__SWIG_5; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- FastMarchingUpwindGradient__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.list(argv[[4]]) && all(sapply(argv[[4]] , is.integer) || sapply(argv[[4]], is.numeric)) )) {
      f <- FastMarchingUpwindGradient__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.list(argv[[4]]) && all(sapply(argv[[4]] , is.integer) || sapply(argv[[4]], is.numeric)) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- FastMarchingUpwindGradient__SWIG_2; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.list(argv[[4]]) && all(sapply(argv[[4]] , is.integer) || sapply(argv[[4]], is.numeric)) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- FastMarchingUpwindGradient__SWIG_1; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.list(argv[[4]]) && all(sapply(argv[[4]] , is.integer) || sapply(argv[[4]], is.numeric)) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) )) {
      f <- FastMarchingUpwindGradient__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for FastMarchingUpwindGradient with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_FastSymmetricForcesDemonsRegistrationFilter

`delete_FastSymmetricForcesDemonsRegistrationFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_FastSymmetricForcesDemonsRegistrationFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_FastSymmetricForcesDemonsRegistrationFilter`, 'returnType') = 'void'
attr(`delete_FastSymmetricForcesDemonsRegistrationFilter`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`delete_FastSymmetricForcesDemonsRegistrationFilter`) = c("SWIGFunction", class('delete_FastSymmetricForcesDemonsRegistrationFilter'))

# Start of new_FastSymmetricForcesDemonsRegistrationFilter

`FastSymmetricForcesDemonsRegistrationFilter` = function()
{
  ;ans = .Call('R_swig_new_FastSymmetricForcesDemonsRegistrationFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_FastSymmetricForcesDemonsRegistrationFilter);
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter`, 'returnType') = '_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter'
class(`FastSymmetricForcesDemonsRegistrationFilter`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations

`FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_0` = function(self, StandardDeviations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  StandardDeviations = as.numeric(StandardDeviations);
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_0', self, StandardDeviations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_0`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_0`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', 'numeric')
class(`FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_0`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_0'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations

`FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_1`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_1`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', 'numeric')
class(`FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_1`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_1'))

`FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of FastSymmetricForcesDemonsRegistrationFilter_GetStandardDeviations

`FastSymmetricForcesDemonsRegistrationFilter_GetStandardDeviations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetStandardDeviations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetStandardDeviations`, 'returnType') = 'numeric'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetStandardDeviations`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetStandardDeviations`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetStandardDeviations'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations

`FastSymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', 'integer')
class(`FastSymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations

`FastSymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError

`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError` = function(self, MaximumRMSError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError', self, MaximumRMSError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', 'numeric')
class(`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError

`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError`, 'returnType') = 'numeric'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_Symmetric_get

`FastSymmetricForcesDemonsRegistrationFilter_Symmetric_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_Symmetric_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FastSymmetricForcesDemonsRegistrationFilter__UseGradientTypeType");
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_Symmetric_get`, 'returnType') = 'character'
class(`FastSymmetricForcesDemonsRegistrationFilter_Symmetric_get`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_Symmetric_get'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_Fixed_get

`FastSymmetricForcesDemonsRegistrationFilter_Fixed_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_Fixed_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FastSymmetricForcesDemonsRegistrationFilter__UseGradientTypeType");
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_Fixed_get`, 'returnType') = 'character'
class(`FastSymmetricForcesDemonsRegistrationFilter_Fixed_get`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_Fixed_get'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_WarpedMoving_get

`FastSymmetricForcesDemonsRegistrationFilter_WarpedMoving_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_WarpedMoving_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FastSymmetricForcesDemonsRegistrationFilter__UseGradientTypeType");
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_WarpedMoving_get`, 'returnType') = 'character'
class(`FastSymmetricForcesDemonsRegistrationFilter_WarpedMoving_get`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_WarpedMoving_get'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_MappedMoving_get

`FastSymmetricForcesDemonsRegistrationFilter_MappedMoving_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_MappedMoving_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FastSymmetricForcesDemonsRegistrationFilter__UseGradientTypeType");
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_MappedMoving_get`, 'returnType') = 'character'
class(`FastSymmetricForcesDemonsRegistrationFilter_MappedMoving_get`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_MappedMoving_get'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SetUseGradientType

`FastSymmetricForcesDemonsRegistrationFilter_SetUseGradientType` = function(self, UseGradientType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseGradientType = enumToInteger(UseGradientType, "_itk__simple__FastSymmetricForcesDemonsRegistrationFilter__UseGradientTypeType"); 
  
  if(length(UseGradientType) > 1) {
    warning("using only the first element of UseGradientType");
  };
  
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SetUseGradientType', self, UseGradientType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SetUseGradientType`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SetUseGradientType`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', 'character')
class(`FastSymmetricForcesDemonsRegistrationFilter_SetUseGradientType`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SetUseGradientType'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_GetUseGradientType

`FastSymmetricForcesDemonsRegistrationFilter_GetUseGradientType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetUseGradientType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__FastSymmetricForcesDemonsRegistrationFilter__UseGradientTypeType");
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetUseGradientType`, 'returnType') = 'character'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetUseGradientType`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetUseGradientType`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetUseGradientType'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SetMaximumUpdateStepLength

`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumUpdateStepLength` = function(self, MaximumUpdateStepLength)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SetMaximumUpdateStepLength', self, MaximumUpdateStepLength, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumUpdateStepLength`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumUpdateStepLength`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', 'numeric')
class(`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumUpdateStepLength`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SetMaximumUpdateStepLength'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_GetMaximumUpdateStepLength

`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumUpdateStepLength` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetMaximumUpdateStepLength', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumUpdateStepLength`, 'returnType') = 'numeric'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumUpdateStepLength`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumUpdateStepLength`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetMaximumUpdateStepLength'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField

`FastSymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField` = function(self, SmoothDisplacementField)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SmoothDisplacementField = as.logical(SmoothDisplacementField);
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField', self, SmoothDisplacementField, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', 'logical')
class(`FastSymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn

`FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff

`FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField

`FastSymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField`, 'returnType') = 'logical'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField

`FastSymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField` = function(self, SmoothUpdateField)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SmoothUpdateField = as.logical(SmoothUpdateField);
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField', self, SmoothUpdateField, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', 'logical')
class(`FastSymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn

`FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff

`FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField

`FastSymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField`, 'returnType') = 'logical'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations

`FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0` = function(self, UpdateFieldStandardDeviations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UpdateFieldStandardDeviations = as.numeric(UpdateFieldStandardDeviations);
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0', self, UpdateFieldStandardDeviations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', 'numeric')
class(`FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations

`FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', 'numeric')
class(`FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1'))

`FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of FastSymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations

`FastSymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations`, 'returnType') = 'numeric'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth

`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth` = function(self, MaximumKernelWidth)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaximumKernelWidth = as.integer(MaximumKernelWidth);
  
  if(length(MaximumKernelWidth) > 1) {
    warning("using only the first element of MaximumKernelWidth");
  };
  
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth', self, MaximumKernelWidth, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', 'integer')
class(`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth

`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth`, 'returnType') = 'integer'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SetMaximumError

`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumError` = function(self, MaximumError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SetMaximumError', self, MaximumError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumError`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumError`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', 'numeric')
class(`FastSymmetricForcesDemonsRegistrationFilter_SetMaximumError`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SetMaximumError'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_GetMaximumError

`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetMaximumError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumError`, 'returnType') = 'numeric'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumError`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetMaximumError`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetMaximumError'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold

`FastSymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold` = function(self, IntensityDifferenceThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold', self, IntensityDifferenceThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', 'numeric')
class(`FastSymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold

`FastSymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold`, 'returnType') = 'numeric'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing

`FastSymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', 'logical')
class(`FastSymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn

`FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff

`FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing

`FastSymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_GetElapsedIterations

`FastSymmetricForcesDemonsRegistrationFilter_GetElapsedIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetElapsedIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetElapsedIterations`, 'returnType') = 'integer'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetElapsedIterations`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetElapsedIterations`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetElapsedIterations'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_GetRMSChange

`FastSymmetricForcesDemonsRegistrationFilter_GetRMSChange` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetRMSChange', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetRMSChange`, 'returnType') = 'numeric'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetRMSChange`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetRMSChange`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetRMSChange'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_GetMetric

`FastSymmetricForcesDemonsRegistrationFilter_GetMetric` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetMetric', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetMetric`, 'returnType') = 'numeric'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetMetric`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetMetric`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetMetric'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_StopRegistration

`FastSymmetricForcesDemonsRegistrationFilter_StopRegistration` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_StopRegistration', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_StopRegistration`, 'returnType') = 'void'
attr(`FastSymmetricForcesDemonsRegistrationFilter_StopRegistration`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_StopRegistration`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_StopRegistration'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_GetName

`FastSymmetricForcesDemonsRegistrationFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_GetName`, 'returnType') = 'character'
attr(`FastSymmetricForcesDemonsRegistrationFilter_GetName`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_GetName`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_GetName'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_ToString

`FastSymmetricForcesDemonsRegistrationFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_ToString`, 'returnType') = 'character'
attr(`FastSymmetricForcesDemonsRegistrationFilter_ToString`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter')
class(`FastSymmetricForcesDemonsRegistrationFilter_ToString`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_ToString'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_Execute

`FastSymmetricForcesDemonsRegistrationFilter_Execute__SWIG_0` = function(self, fixedImage, movingImage, initialDisplacementField, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  if (inherits(initialDisplacementField, "ExternalReference")) initialDisplacementField = slot(initialDisplacementField,"ref"); 
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_Execute__SWIG_0', self, fixedImage, movingImage, initialDisplacementField, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`FastSymmetricForcesDemonsRegistrationFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`FastSymmetricForcesDemonsRegistrationFilter_Execute__SWIG_0`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_Execute__SWIG_0'))

# Start of FastSymmetricForcesDemonsRegistrationFilter_Execute

`FastSymmetricForcesDemonsRegistrationFilter_Execute__SWIG_1` = function(self, fixedImage, movingImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  ;ans = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_Execute__SWIG_1', self, fixedImage, movingImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FastSymmetricForcesDemonsRegistrationFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`FastSymmetricForcesDemonsRegistrationFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`FastSymmetricForcesDemonsRegistrationFilter_Execute__SWIG_1`) = c("SWIGFunction", class('FastSymmetricForcesDemonsRegistrationFilter_Execute__SWIG_1'))

`FastSymmetricForcesDemonsRegistrationFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- FastSymmetricForcesDemonsRegistrationFilter_Execute__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- FastSymmetricForcesDemonsRegistrationFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for FastSymmetricForcesDemonsRegistrationFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::FastSymmetricForcesDemonsRegistrationFilter
setMethod('$', '_p_itk__simple__FastSymmetricForcesDemonsRegistrationFilter', function(x, name)

{
  accessorFuns = list('SetStandardDeviations' = FastSymmetricForcesDemonsRegistrationFilter_SetStandardDeviations, 'GetStandardDeviations' = FastSymmetricForcesDemonsRegistrationFilter_GetStandardDeviations, 'SetNumberOfIterations' = FastSymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations, 'GetNumberOfIterations' = FastSymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations, 'SetMaximumRMSError' = FastSymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError, 'GetMaximumRMSError' = FastSymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError, 'SetUseGradientType' = FastSymmetricForcesDemonsRegistrationFilter_SetUseGradientType, 'GetUseGradientType' = FastSymmetricForcesDemonsRegistrationFilter_GetUseGradientType, 'SetMaximumUpdateStepLength' = FastSymmetricForcesDemonsRegistrationFilter_SetMaximumUpdateStepLength, 'GetMaximumUpdateStepLength' = FastSymmetricForcesDemonsRegistrationFilter_GetMaximumUpdateStepLength, 'SetSmoothDisplacementField' = FastSymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField, 'SmoothDisplacementFieldOn' = FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn, 'SmoothDisplacementFieldOff' = FastSymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff, 'GetSmoothDisplacementField' = FastSymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField, 'SetSmoothUpdateField' = FastSymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField, 'SmoothUpdateFieldOn' = FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn, 'SmoothUpdateFieldOff' = FastSymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff, 'GetSmoothUpdateField' = FastSymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField, 'SetUpdateFieldStandardDeviations' = FastSymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations, 'GetUpdateFieldStandardDeviations' = FastSymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations, 'SetMaximumKernelWidth' = FastSymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth, 'GetMaximumKernelWidth' = FastSymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth, 'SetMaximumError' = FastSymmetricForcesDemonsRegistrationFilter_SetMaximumError, 'GetMaximumError' = FastSymmetricForcesDemonsRegistrationFilter_GetMaximumError, 'SetIntensityDifferenceThreshold' = FastSymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold, 'GetIntensityDifferenceThreshold' = FastSymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold, 'SetUseImageSpacing' = FastSymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing, 'UseImageSpacingOn' = FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn, 'UseImageSpacingOff' = FastSymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff, 'GetUseImageSpacing' = FastSymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing, 'GetElapsedIterations' = FastSymmetricForcesDemonsRegistrationFilter_GetElapsedIterations, 'GetRMSChange' = FastSymmetricForcesDemonsRegistrationFilter_GetRMSChange, 'GetMetric' = FastSymmetricForcesDemonsRegistrationFilter_GetMetric, 'StopRegistration' = FastSymmetricForcesDemonsRegistrationFilter_StopRegistration, 'GetName' = FastSymmetricForcesDemonsRegistrationFilter_GetName, 'ToString' = FastSymmetricForcesDemonsRegistrationFilter_ToString, 'Execute' = FastSymmetricForcesDemonsRegistrationFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::FastSymmetricForcesDemonsRegistrationFilter
# Start of delete_FlipImageFilter

`delete_FlipImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_FlipImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_FlipImageFilter`, 'returnType') = 'void'
attr(`delete_FlipImageFilter`, "inputTypes") = c('_p_itk__simple__FlipImageFilter')
class(`delete_FlipImageFilter`) = c("SWIGFunction", class('delete_FlipImageFilter'))

# Start of new_FlipImageFilter

`FlipImageFilter` = function()
{
  ;ans = .Call('R_swig_new_FlipImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__FlipImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_FlipImageFilter);
  ans
  
}

attr(`FlipImageFilter`, 'returnType') = '_p_itk__simple__FlipImageFilter'
class(`FlipImageFilter`) = c("SWIGFunction", class('FlipImageFilter'))

# Start of FlipImageFilter_SetFlipAxes

`FlipImageFilter_SetFlipAxes` = function(self, FlipAxes)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FlipAxes = as.logical(FlipAxes);
  ;ans = .Call('R_swig_FlipImageFilter_SetFlipAxes', self, FlipAxes, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FlipImageFilter_SetFlipAxes`, 'returnType') = 'void'
attr(`FlipImageFilter_SetFlipAxes`, "inputTypes") = c('_p_itk__simple__FlipImageFilter', 'logical')
class(`FlipImageFilter_SetFlipAxes`) = c("SWIGFunction", class('FlipImageFilter_SetFlipAxes'))

# Start of FlipImageFilter_GetFlipAxes

`FlipImageFilter_GetFlipAxes` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FlipImageFilter_GetFlipAxes', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FlipImageFilter_GetFlipAxes`, 'returnType') = 'logical'
attr(`FlipImageFilter_GetFlipAxes`, "inputTypes") = c('_p_itk__simple__FlipImageFilter')
class(`FlipImageFilter_GetFlipAxes`) = c("SWIGFunction", class('FlipImageFilter_GetFlipAxes'))

# Start of FlipImageFilter_SetFlipAboutOrigin

`FlipImageFilter_SetFlipAboutOrigin` = function(self, FlipAboutOrigin)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FlipAboutOrigin = as.logical(FlipAboutOrigin);
  ;ans = .Call('R_swig_FlipImageFilter_SetFlipAboutOrigin', self, FlipAboutOrigin, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FlipImageFilter_SetFlipAboutOrigin`, 'returnType') = 'void'
attr(`FlipImageFilter_SetFlipAboutOrigin`, "inputTypes") = c('_p_itk__simple__FlipImageFilter', 'logical')
class(`FlipImageFilter_SetFlipAboutOrigin`) = c("SWIGFunction", class('FlipImageFilter_SetFlipAboutOrigin'))

# Start of FlipImageFilter_FlipAboutOriginOn

`FlipImageFilter_FlipAboutOriginOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FlipImageFilter_FlipAboutOriginOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FlipImageFilter_FlipAboutOriginOn`, 'returnType') = 'void'
attr(`FlipImageFilter_FlipAboutOriginOn`, "inputTypes") = c('_p_itk__simple__FlipImageFilter')
class(`FlipImageFilter_FlipAboutOriginOn`) = c("SWIGFunction", class('FlipImageFilter_FlipAboutOriginOn'))

# Start of FlipImageFilter_FlipAboutOriginOff

`FlipImageFilter_FlipAboutOriginOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_FlipImageFilter_FlipAboutOriginOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`FlipImageFilter_FlipAboutOriginOff`, 'returnType') = 'void'
attr(`FlipImageFilter_FlipAboutOriginOff`, "inputTypes") = c('_p_itk__simple__FlipImageFilter')
class(`FlipImageFilter_FlipAboutOriginOff`) = c("SWIGFunction", class('FlipImageFilter_FlipAboutOriginOff'))

# Start of FlipImageFilter_GetFlipAboutOrigin

`FlipImageFilter_GetFlipAboutOrigin` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FlipImageFilter_GetFlipAboutOrigin', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FlipImageFilter_GetFlipAboutOrigin`, 'returnType') = 'logical'
attr(`FlipImageFilter_GetFlipAboutOrigin`, "inputTypes") = c('_p_itk__simple__FlipImageFilter')
class(`FlipImageFilter_GetFlipAboutOrigin`) = c("SWIGFunction", class('FlipImageFilter_GetFlipAboutOrigin'))

# Start of FlipImageFilter_GetName

`FlipImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FlipImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FlipImageFilter_GetName`, 'returnType') = 'character'
attr(`FlipImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__FlipImageFilter')
class(`FlipImageFilter_GetName`) = c("SWIGFunction", class('FlipImageFilter_GetName'))

# Start of FlipImageFilter_ToString

`FlipImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_FlipImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`FlipImageFilter_ToString`, 'returnType') = 'character'
attr(`FlipImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__FlipImageFilter')
class(`FlipImageFilter_ToString`) = c("SWIGFunction", class('FlipImageFilter_ToString'))

# Start of FlipImageFilter_Execute

`FlipImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_FlipImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`FlipImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`FlipImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__FlipImageFilter', '_p_itk__simple__Image')
class(`FlipImageFilter_Execute`) = c("SWIGFunction", class('FlipImageFilter_Execute'))

# Start of accessor method for itk::simple::FlipImageFilter
setMethod('$', '_p_itk__simple__FlipImageFilter', function(x, name)

{
  accessorFuns = list('SetFlipAxes' = FlipImageFilter_SetFlipAxes, 'GetFlipAxes' = FlipImageFilter_GetFlipAxes, 'SetFlipAboutOrigin' = FlipImageFilter_SetFlipAboutOrigin, 'FlipAboutOriginOn' = FlipImageFilter_FlipAboutOriginOn, 'FlipAboutOriginOff' = FlipImageFilter_FlipAboutOriginOff, 'GetFlipAboutOrigin' = FlipImageFilter_GetFlipAboutOrigin, 'GetName' = FlipImageFilter_GetName, 'ToString' = FlipImageFilter_ToString, 'Execute' = FlipImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::FlipImageFilter
# Start of Flip

`Flip__SWIG_0` = function(image1, flipAxes, flipAboutOrigin, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  flipAxes = as.logical(flipAxes);
  flipAboutOrigin = as.logical(flipAboutOrigin);
  ;ans = .Call('R_swig_Flip__SWIG_0', image1, flipAxes, flipAboutOrigin, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Flip__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Flip__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'logical')
class(`Flip__SWIG_0`) = c("SWIGFunction", class('Flip__SWIG_0'))

# Start of Flip

`Flip__SWIG_1` = function(image1, flipAxes, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  flipAxes = as.logical(flipAxes);
  ;ans = .Call('R_swig_Flip__SWIG_1', image1, flipAxes, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Flip__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Flip__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`Flip__SWIG_1`) = c("SWIGFunction", class('Flip__SWIG_1'))

# Start of Flip

`Flip__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Flip__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Flip__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Flip__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`Flip__SWIG_2`) = c("SWIGFunction", class('Flip__SWIG_2'))

`Flip` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Flip__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) )) {
      f <- Flip__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Flip__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Flip with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ForwardFFTImageFilter

`delete_ForwardFFTImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ForwardFFTImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ForwardFFTImageFilter`, 'returnType') = 'void'
attr(`delete_ForwardFFTImageFilter`, "inputTypes") = c('_p_itk__simple__ForwardFFTImageFilter')
class(`delete_ForwardFFTImageFilter`) = c("SWIGFunction", class('delete_ForwardFFTImageFilter'))

# Start of new_ForwardFFTImageFilter

`ForwardFFTImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ForwardFFTImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ForwardFFTImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ForwardFFTImageFilter);
  ans
  
}

attr(`ForwardFFTImageFilter`, 'returnType') = '_p_itk__simple__ForwardFFTImageFilter'
class(`ForwardFFTImageFilter`) = c("SWIGFunction", class('ForwardFFTImageFilter'))

# Start of ForwardFFTImageFilter_GetName

`ForwardFFTImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ForwardFFTImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ForwardFFTImageFilter_GetName`, 'returnType') = 'character'
attr(`ForwardFFTImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ForwardFFTImageFilter')
class(`ForwardFFTImageFilter_GetName`) = c("SWIGFunction", class('ForwardFFTImageFilter_GetName'))

# Start of ForwardFFTImageFilter_ToString

`ForwardFFTImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ForwardFFTImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ForwardFFTImageFilter_ToString`, 'returnType') = 'character'
attr(`ForwardFFTImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ForwardFFTImageFilter')
class(`ForwardFFTImageFilter_ToString`) = c("SWIGFunction", class('ForwardFFTImageFilter_ToString'))

# Start of ForwardFFTImageFilter_Execute

`ForwardFFTImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ForwardFFTImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ForwardFFTImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ForwardFFTImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ForwardFFTImageFilter', '_p_itk__simple__Image')
class(`ForwardFFTImageFilter_Execute`) = c("SWIGFunction", class('ForwardFFTImageFilter_Execute'))

# Start of accessor method for itk::simple::ForwardFFTImageFilter
setMethod('$', '_p_itk__simple__ForwardFFTImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = ForwardFFTImageFilter_GetName, 'ToString' = ForwardFFTImageFilter_ToString, 'Execute' = ForwardFFTImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ForwardFFTImageFilter
# Start of ForwardFFT

`ForwardFFT` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ForwardFFT', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ForwardFFT`, 'returnType') = '_p_itk__simple__Image'
attr(`ForwardFFT`, "inputTypes") = c('_p_itk__simple__Image')
class(`ForwardFFT`) = c("SWIGFunction", class('ForwardFFT'))

# Start of delete_GaborImageSource

`delete_GaborImageSource` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GaborImageSource', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GaborImageSource`, 'returnType') = 'void'
attr(`delete_GaborImageSource`, "inputTypes") = c('_p_itk__simple__GaborImageSource')
class(`delete_GaborImageSource`) = c("SWIGFunction", class('delete_GaborImageSource'))

# Start of new_GaborImageSource

`GaborImageSource` = function()
{
  ;ans = .Call('R_swig_new_GaborImageSource', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GaborImageSource", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GaborImageSource);
  ans
  
}

attr(`GaborImageSource`, 'returnType') = '_p_itk__simple__GaborImageSource'
class(`GaborImageSource`) = c("SWIGFunction", class('GaborImageSource'))

# Start of GaborImageSource_SetOutputPixelType

`GaborImageSource_SetOutputPixelType` = function(self, OutputPixelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputPixelType = enumToInteger(OutputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(OutputPixelType) > 1) {
    warning("using only the first element of OutputPixelType");
  };
  
  ;ans = .Call('R_swig_GaborImageSource_SetOutputPixelType', self, OutputPixelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaborImageSource_SetOutputPixelType`, 'returnType') = 'void'
attr(`GaborImageSource_SetOutputPixelType`, "inputTypes") = c('_p_itk__simple__GaborImageSource', 'character')
class(`GaborImageSource_SetOutputPixelType`) = c("SWIGFunction", class('GaborImageSource_SetOutputPixelType'))

# Start of GaborImageSource_GetOutputPixelType

`GaborImageSource_GetOutputPixelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GaborImageSource_GetOutputPixelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`GaborImageSource_GetOutputPixelType`, 'returnType') = 'character'
attr(`GaborImageSource_GetOutputPixelType`, "inputTypes") = c('_p_itk__simple__GaborImageSource')
class(`GaborImageSource_GetOutputPixelType`) = c("SWIGFunction", class('GaborImageSource_GetOutputPixelType'))

# Start of GaborImageSource_SetSize

`GaborImageSource_SetSize` = function(self, Size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Size = as.integer(Size);
  ;ans = .Call('R_swig_GaborImageSource_SetSize', self, Size, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaborImageSource_SetSize`, 'returnType') = 'void'
attr(`GaborImageSource_SetSize`, "inputTypes") = c('_p_itk__simple__GaborImageSource', 'integer')
class(`GaborImageSource_SetSize`) = c("SWIGFunction", class('GaborImageSource_SetSize'))

# Start of GaborImageSource_GetSize

`GaborImageSource_GetSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaborImageSource_GetSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaborImageSource_GetSize`, 'returnType') = 'integer'
attr(`GaborImageSource_GetSize`, "inputTypes") = c('_p_itk__simple__GaborImageSource')
class(`GaborImageSource_GetSize`) = c("SWIGFunction", class('GaborImageSource_GetSize'))

# Start of GaborImageSource_SetSigma

`GaborImageSource_SetSigma__SWIG_0` = function(self, Sigma)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Sigma = as.numeric(Sigma);
  ;ans = .Call('R_swig_GaborImageSource_SetSigma__SWIG_0', self, Sigma, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaborImageSource_SetSigma__SWIG_0`, 'returnType') = 'void'
attr(`GaborImageSource_SetSigma__SWIG_0`, "inputTypes") = c('_p_itk__simple__GaborImageSource', 'numeric')
class(`GaborImageSource_SetSigma__SWIG_0`) = c("SWIGFunction", class('GaborImageSource_SetSigma__SWIG_0'))

# Start of GaborImageSource_SetSigma

`GaborImageSource_SetSigma__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GaborImageSource_SetSigma__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaborImageSource_SetSigma__SWIG_1`, 'returnType') = 'void'
attr(`GaborImageSource_SetSigma__SWIG_1`, "inputTypes") = c('_p_itk__simple__GaborImageSource', 'numeric')
class(`GaborImageSource_SetSigma__SWIG_1`) = c("SWIGFunction", class('GaborImageSource_SetSigma__SWIG_1'))

`GaborImageSource_SetSigma` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__GaborImageSource') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- GaborImageSource_SetSigma__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__GaborImageSource') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- GaborImageSource_SetSigma__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GaborImageSource_SetSigma with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of GaborImageSource_GetSigma

`GaborImageSource_GetSigma` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaborImageSource_GetSigma', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaborImageSource_GetSigma`, 'returnType') = 'numeric'
attr(`GaborImageSource_GetSigma`, "inputTypes") = c('_p_itk__simple__GaborImageSource')
class(`GaborImageSource_GetSigma`) = c("SWIGFunction", class('GaborImageSource_GetSigma'))

# Start of GaborImageSource_SetMean

`GaborImageSource_SetMean__SWIG_0` = function(self, Mean)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Mean = as.numeric(Mean);
  ;ans = .Call('R_swig_GaborImageSource_SetMean__SWIG_0', self, Mean, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaborImageSource_SetMean__SWIG_0`, 'returnType') = 'void'
attr(`GaborImageSource_SetMean__SWIG_0`, "inputTypes") = c('_p_itk__simple__GaborImageSource', 'numeric')
class(`GaborImageSource_SetMean__SWIG_0`) = c("SWIGFunction", class('GaborImageSource_SetMean__SWIG_0'))

# Start of GaborImageSource_SetMean

`GaborImageSource_SetMean__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GaborImageSource_SetMean__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaborImageSource_SetMean__SWIG_1`, 'returnType') = 'void'
attr(`GaborImageSource_SetMean__SWIG_1`, "inputTypes") = c('_p_itk__simple__GaborImageSource', 'numeric')
class(`GaborImageSource_SetMean__SWIG_1`) = c("SWIGFunction", class('GaborImageSource_SetMean__SWIG_1'))

`GaborImageSource_SetMean` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__GaborImageSource') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- GaborImageSource_SetMean__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__GaborImageSource') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- GaborImageSource_SetMean__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GaborImageSource_SetMean with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of GaborImageSource_GetMean

`GaborImageSource_GetMean` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaborImageSource_GetMean', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaborImageSource_GetMean`, 'returnType') = 'numeric'
attr(`GaborImageSource_GetMean`, "inputTypes") = c('_p_itk__simple__GaborImageSource')
class(`GaborImageSource_GetMean`) = c("SWIGFunction", class('GaborImageSource_GetMean'))

# Start of GaborImageSource_SetFrequency

`GaborImageSource_SetFrequency` = function(self, Frequency)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GaborImageSource_SetFrequency', self, Frequency, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaborImageSource_SetFrequency`, 'returnType') = 'void'
attr(`GaborImageSource_SetFrequency`, "inputTypes") = c('_p_itk__simple__GaborImageSource', 'numeric')
class(`GaborImageSource_SetFrequency`) = c("SWIGFunction", class('GaborImageSource_SetFrequency'))

# Start of GaborImageSource_GetFrequency

`GaborImageSource_GetFrequency` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaborImageSource_GetFrequency', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaborImageSource_GetFrequency`, 'returnType') = 'numeric'
attr(`GaborImageSource_GetFrequency`, "inputTypes") = c('_p_itk__simple__GaborImageSource')
class(`GaborImageSource_GetFrequency`) = c("SWIGFunction", class('GaborImageSource_GetFrequency'))

# Start of GaborImageSource_SetOrigin

`GaborImageSource_SetOrigin` = function(self, Origin)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Origin = as.numeric(Origin);
  ;ans = .Call('R_swig_GaborImageSource_SetOrigin', self, Origin, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaborImageSource_SetOrigin`, 'returnType') = 'void'
attr(`GaborImageSource_SetOrigin`, "inputTypes") = c('_p_itk__simple__GaborImageSource', 'numeric')
class(`GaborImageSource_SetOrigin`) = c("SWIGFunction", class('GaborImageSource_SetOrigin'))

# Start of GaborImageSource_GetOrigin

`GaborImageSource_GetOrigin` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaborImageSource_GetOrigin', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaborImageSource_GetOrigin`, 'returnType') = 'numeric'
attr(`GaborImageSource_GetOrigin`, "inputTypes") = c('_p_itk__simple__GaborImageSource')
class(`GaborImageSource_GetOrigin`) = c("SWIGFunction", class('GaborImageSource_GetOrigin'))

# Start of GaborImageSource_SetSpacing

`GaborImageSource_SetSpacing` = function(self, Spacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Spacing = as.numeric(Spacing);
  ;ans = .Call('R_swig_GaborImageSource_SetSpacing', self, Spacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaborImageSource_SetSpacing`, 'returnType') = 'void'
attr(`GaborImageSource_SetSpacing`, "inputTypes") = c('_p_itk__simple__GaborImageSource', 'numeric')
class(`GaborImageSource_SetSpacing`) = c("SWIGFunction", class('GaborImageSource_SetSpacing'))

# Start of GaborImageSource_GetSpacing

`GaborImageSource_GetSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaborImageSource_GetSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaborImageSource_GetSpacing`, 'returnType') = 'numeric'
attr(`GaborImageSource_GetSpacing`, "inputTypes") = c('_p_itk__simple__GaborImageSource')
class(`GaborImageSource_GetSpacing`) = c("SWIGFunction", class('GaborImageSource_GetSpacing'))

# Start of GaborImageSource_SetDirection

`GaborImageSource_SetDirection` = function(self, Direction)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Direction = as.numeric(Direction);
  ;ans = .Call('R_swig_GaborImageSource_SetDirection', self, Direction, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaborImageSource_SetDirection`, 'returnType') = 'void'
attr(`GaborImageSource_SetDirection`, "inputTypes") = c('_p_itk__simple__GaborImageSource', 'numeric')
class(`GaborImageSource_SetDirection`) = c("SWIGFunction", class('GaborImageSource_SetDirection'))

# Start of GaborImageSource_GetDirection

`GaborImageSource_GetDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaborImageSource_GetDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaborImageSource_GetDirection`, 'returnType') = 'numeric'
attr(`GaborImageSource_GetDirection`, "inputTypes") = c('_p_itk__simple__GaborImageSource')
class(`GaborImageSource_GetDirection`) = c("SWIGFunction", class('GaborImageSource_GetDirection'))

# Start of GaborImageSource_GetName

`GaborImageSource_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaborImageSource_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaborImageSource_GetName`, 'returnType') = 'character'
attr(`GaborImageSource_GetName`, "inputTypes") = c('_p_itk__simple__GaborImageSource')
class(`GaborImageSource_GetName`) = c("SWIGFunction", class('GaborImageSource_GetName'))

# Start of GaborImageSource_ToString

`GaborImageSource_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaborImageSource_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaborImageSource_ToString`, 'returnType') = 'character'
attr(`GaborImageSource_ToString`, "inputTypes") = c('_p_itk__simple__GaborImageSource')
class(`GaborImageSource_ToString`) = c("SWIGFunction", class('GaborImageSource_ToString'))

# Start of GaborImageSource_Execute

`GaborImageSource_Execute` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GaborImageSource_Execute', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaborImageSource_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GaborImageSource_Execute`, "inputTypes") = c('_p_itk__simple__GaborImageSource')
class(`GaborImageSource_Execute`) = c("SWIGFunction", class('GaborImageSource_Execute'))

# Start of accessor method for itk::simple::GaborImageSource
setMethod('$', '_p_itk__simple__GaborImageSource', function(x, name)

{
  accessorFuns = list('SetOutputPixelType' = GaborImageSource_SetOutputPixelType, 'GetOutputPixelType' = GaborImageSource_GetOutputPixelType, 'SetSize' = GaborImageSource_SetSize, 'GetSize' = GaborImageSource_GetSize, 'SetSigma' = GaborImageSource_SetSigma, 'GetSigma' = GaborImageSource_GetSigma, 'SetMean' = GaborImageSource_SetMean, 'GetMean' = GaborImageSource_GetMean, 'SetFrequency' = GaborImageSource_SetFrequency, 'GetFrequency' = GaborImageSource_GetFrequency, 'SetOrigin' = GaborImageSource_SetOrigin, 'GetOrigin' = GaborImageSource_GetOrigin, 'SetSpacing' = GaborImageSource_SetSpacing, 'GetSpacing' = GaborImageSource_GetSpacing, 'SetDirection' = GaborImageSource_SetDirection, 'GetDirection' = GaborImageSource_GetDirection, 'GetName' = GaborImageSource_GetName, 'ToString' = GaborImageSource_ToString, 'Execute' = GaborImageSource_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GaborImageSource
# Start of GaborSource

`GaborSource__SWIG_0` = function(outputPixelType, size, sigma, mean, frequency, origin, spacing, direction, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  mean = as.numeric(mean);
  
  origin = as.numeric(origin);
  spacing = as.numeric(spacing);
  direction = as.numeric(direction);
  ;ans = .Call('R_swig_GaborSource__SWIG_0', outputPixelType, size, sigma, mean, frequency, origin, spacing, direction, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaborSource__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GaborSource__SWIG_0`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
class(`GaborSource__SWIG_0`) = c("SWIGFunction", class('GaborSource__SWIG_0'))

# Start of GaborSource

`GaborSource__SWIG_1` = function(outputPixelType, size, sigma, mean, frequency, origin, spacing, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  mean = as.numeric(mean);
  
  origin = as.numeric(origin);
  spacing = as.numeric(spacing);
  ;ans = .Call('R_swig_GaborSource__SWIG_1', outputPixelType, size, sigma, mean, frequency, origin, spacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaborSource__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GaborSource__SWIG_1`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
class(`GaborSource__SWIG_1`) = c("SWIGFunction", class('GaborSource__SWIG_1'))

# Start of GaborSource

`GaborSource__SWIG_2` = function(outputPixelType, size, sigma, mean, frequency, origin, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  mean = as.numeric(mean);
  
  origin = as.numeric(origin);
  ;ans = .Call('R_swig_GaborSource__SWIG_2', outputPixelType, size, sigma, mean, frequency, origin, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaborSource__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GaborSource__SWIG_2`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric', 'numeric')
class(`GaborSource__SWIG_2`) = c("SWIGFunction", class('GaborSource__SWIG_2'))

# Start of GaborSource

`GaborSource__SWIG_3` = function(outputPixelType, size, sigma, mean, frequency, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  mean = as.numeric(mean);
  
  ;ans = .Call('R_swig_GaborSource__SWIG_3', outputPixelType, size, sigma, mean, frequency, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaborSource__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`GaborSource__SWIG_3`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric')
class(`GaborSource__SWIG_3`) = c("SWIGFunction", class('GaborSource__SWIG_3'))

# Start of GaborSource

`GaborSource__SWIG_4` = function(outputPixelType, size, sigma, mean, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  mean = as.numeric(mean);
  ;ans = .Call('R_swig_GaborSource__SWIG_4', outputPixelType, size, sigma, mean, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaborSource__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`GaborSource__SWIG_4`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric')
class(`GaborSource__SWIG_4`) = c("SWIGFunction", class('GaborSource__SWIG_4'))

# Start of GaborSource

`GaborSource__SWIG_5` = function(outputPixelType, size, sigma, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  ;ans = .Call('R_swig_GaborSource__SWIG_5', outputPixelType, size, sigma, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaborSource__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`GaborSource__SWIG_5`, "inputTypes") = c('character', 'integer', 'numeric')
class(`GaborSource__SWIG_5`) = c("SWIGFunction", class('GaborSource__SWIG_5'))

# Start of GaborSource

`GaborSource__SWIG_6` = function(outputPixelType, size, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  ;ans = .Call('R_swig_GaborSource__SWIG_6', outputPixelType, size, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaborSource__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`GaborSource__SWIG_6`, "inputTypes") = c('character', 'integer')
class(`GaborSource__SWIG_6`) = c("SWIGFunction", class('GaborSource__SWIG_6'))

# Start of GaborSource

`GaborSource__SWIG_7` = function(outputPixelType, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  ;ans = .Call('R_swig_GaborSource__SWIG_7', outputPixelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaborSource__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`GaborSource__SWIG_7`, "inputTypes") = c('character')
class(`GaborSource__SWIG_7`) = c("SWIGFunction", class('GaborSource__SWIG_7'))

# Start of GaborSource

`GaborSource__SWIG_8` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_GaborSource__SWIG_8', as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaborSource__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
class(`GaborSource__SWIG_8`) = c("SWIGFunction", class('GaborSource__SWIG_8'))

`GaborSource` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 0) {
    f <- GaborSource__SWIG_8; 
  } else if (argc == 1) {
    if ( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) {
      f <- GaborSource__SWIG_7; 
    }
  } else if (argc == 2) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- GaborSource__SWIG_6; 
    }
  } else if (argc == 3) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) )) {
      f <- GaborSource__SWIG_5; 
    }
  } else if (argc == 4) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) )) {
      f <- GaborSource__SWIG_4; 
    }
  } else if (argc == 5) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- GaborSource__SWIG_3; 
    }
  } else if (argc == 6) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) )) {
      f <- GaborSource__SWIG_2; 
    }
  } else if (argc == 7) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) ) && ( is.numeric(argv[[7]]) )) {
      f <- GaborSource__SWIG_1; 
    }
  } else if (argc == 8) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) ) && ( is.numeric(argv[[7]]) ) && ( is.numeric(argv[[8]]) )) {
      f <- GaborSource__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GaborSource with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GaussianImageSource

`delete_GaussianImageSource` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GaussianImageSource', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GaussianImageSource`, 'returnType') = 'void'
attr(`delete_GaussianImageSource`, "inputTypes") = c('_p_itk__simple__GaussianImageSource')
class(`delete_GaussianImageSource`) = c("SWIGFunction", class('delete_GaussianImageSource'))

# Start of new_GaussianImageSource

`GaussianImageSource` = function()
{
  ;ans = .Call('R_swig_new_GaussianImageSource', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GaussianImageSource", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GaussianImageSource);
  ans
  
}

attr(`GaussianImageSource`, 'returnType') = '_p_itk__simple__GaussianImageSource'
class(`GaussianImageSource`) = c("SWIGFunction", class('GaussianImageSource'))

# Start of GaussianImageSource_SetOutputPixelType

`GaussianImageSource_SetOutputPixelType` = function(self, OutputPixelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputPixelType = enumToInteger(OutputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(OutputPixelType) > 1) {
    warning("using only the first element of OutputPixelType");
  };
  
  ;ans = .Call('R_swig_GaussianImageSource_SetOutputPixelType', self, OutputPixelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaussianImageSource_SetOutputPixelType`, 'returnType') = 'void'
attr(`GaussianImageSource_SetOutputPixelType`, "inputTypes") = c('_p_itk__simple__GaussianImageSource', 'character')
class(`GaussianImageSource_SetOutputPixelType`) = c("SWIGFunction", class('GaussianImageSource_SetOutputPixelType'))

# Start of GaussianImageSource_GetOutputPixelType

`GaussianImageSource_GetOutputPixelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GaussianImageSource_GetOutputPixelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`GaussianImageSource_GetOutputPixelType`, 'returnType') = 'character'
attr(`GaussianImageSource_GetOutputPixelType`, "inputTypes") = c('_p_itk__simple__GaussianImageSource')
class(`GaussianImageSource_GetOutputPixelType`) = c("SWIGFunction", class('GaussianImageSource_GetOutputPixelType'))

# Start of GaussianImageSource_SetSize

`GaussianImageSource_SetSize` = function(self, Size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Size = as.integer(Size);
  ;ans = .Call('R_swig_GaussianImageSource_SetSize', self, Size, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaussianImageSource_SetSize`, 'returnType') = 'void'
attr(`GaussianImageSource_SetSize`, "inputTypes") = c('_p_itk__simple__GaussianImageSource', 'integer')
class(`GaussianImageSource_SetSize`) = c("SWIGFunction", class('GaussianImageSource_SetSize'))

# Start of GaussianImageSource_GetSize

`GaussianImageSource_GetSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaussianImageSource_GetSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaussianImageSource_GetSize`, 'returnType') = 'integer'
attr(`GaussianImageSource_GetSize`, "inputTypes") = c('_p_itk__simple__GaussianImageSource')
class(`GaussianImageSource_GetSize`) = c("SWIGFunction", class('GaussianImageSource_GetSize'))

# Start of GaussianImageSource_SetSigma

`GaussianImageSource_SetSigma__SWIG_0` = function(self, Sigma)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Sigma = as.numeric(Sigma);
  ;ans = .Call('R_swig_GaussianImageSource_SetSigma__SWIG_0', self, Sigma, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaussianImageSource_SetSigma__SWIG_0`, 'returnType') = 'void'
attr(`GaussianImageSource_SetSigma__SWIG_0`, "inputTypes") = c('_p_itk__simple__GaussianImageSource', 'numeric')
class(`GaussianImageSource_SetSigma__SWIG_0`) = c("SWIGFunction", class('GaussianImageSource_SetSigma__SWIG_0'))

# Start of GaussianImageSource_SetSigma

`GaussianImageSource_SetSigma__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GaussianImageSource_SetSigma__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaussianImageSource_SetSigma__SWIG_1`, 'returnType') = 'void'
attr(`GaussianImageSource_SetSigma__SWIG_1`, "inputTypes") = c('_p_itk__simple__GaussianImageSource', 'numeric')
class(`GaussianImageSource_SetSigma__SWIG_1`) = c("SWIGFunction", class('GaussianImageSource_SetSigma__SWIG_1'))

`GaussianImageSource_SetSigma` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__GaussianImageSource') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- GaussianImageSource_SetSigma__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__GaussianImageSource') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- GaussianImageSource_SetSigma__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GaussianImageSource_SetSigma with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of GaussianImageSource_GetSigma

`GaussianImageSource_GetSigma` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaussianImageSource_GetSigma', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaussianImageSource_GetSigma`, 'returnType') = 'numeric'
attr(`GaussianImageSource_GetSigma`, "inputTypes") = c('_p_itk__simple__GaussianImageSource')
class(`GaussianImageSource_GetSigma`) = c("SWIGFunction", class('GaussianImageSource_GetSigma'))

# Start of GaussianImageSource_SetMean

`GaussianImageSource_SetMean__SWIG_0` = function(self, Mean)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Mean = as.numeric(Mean);
  ;ans = .Call('R_swig_GaussianImageSource_SetMean__SWIG_0', self, Mean, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaussianImageSource_SetMean__SWIG_0`, 'returnType') = 'void'
attr(`GaussianImageSource_SetMean__SWIG_0`, "inputTypes") = c('_p_itk__simple__GaussianImageSource', 'numeric')
class(`GaussianImageSource_SetMean__SWIG_0`) = c("SWIGFunction", class('GaussianImageSource_SetMean__SWIG_0'))

# Start of GaussianImageSource_SetMean

`GaussianImageSource_SetMean__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GaussianImageSource_SetMean__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaussianImageSource_SetMean__SWIG_1`, 'returnType') = 'void'
attr(`GaussianImageSource_SetMean__SWIG_1`, "inputTypes") = c('_p_itk__simple__GaussianImageSource', 'numeric')
class(`GaussianImageSource_SetMean__SWIG_1`) = c("SWIGFunction", class('GaussianImageSource_SetMean__SWIG_1'))

`GaussianImageSource_SetMean` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__GaussianImageSource') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- GaussianImageSource_SetMean__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__GaussianImageSource') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- GaussianImageSource_SetMean__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GaussianImageSource_SetMean with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of GaussianImageSource_GetMean

`GaussianImageSource_GetMean` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaussianImageSource_GetMean', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaussianImageSource_GetMean`, 'returnType') = 'numeric'
attr(`GaussianImageSource_GetMean`, "inputTypes") = c('_p_itk__simple__GaussianImageSource')
class(`GaussianImageSource_GetMean`) = c("SWIGFunction", class('GaussianImageSource_GetMean'))

# Start of GaussianImageSource_SetScale

`GaussianImageSource_SetScale` = function(self, Scale)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GaussianImageSource_SetScale', self, Scale, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaussianImageSource_SetScale`, 'returnType') = 'void'
attr(`GaussianImageSource_SetScale`, "inputTypes") = c('_p_itk__simple__GaussianImageSource', 'numeric')
class(`GaussianImageSource_SetScale`) = c("SWIGFunction", class('GaussianImageSource_SetScale'))

# Start of GaussianImageSource_GetScale

`GaussianImageSource_GetScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaussianImageSource_GetScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaussianImageSource_GetScale`, 'returnType') = 'numeric'
attr(`GaussianImageSource_GetScale`, "inputTypes") = c('_p_itk__simple__GaussianImageSource')
class(`GaussianImageSource_GetScale`) = c("SWIGFunction", class('GaussianImageSource_GetScale'))

# Start of GaussianImageSource_SetOrigin

`GaussianImageSource_SetOrigin` = function(self, Origin)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Origin = as.numeric(Origin);
  ;ans = .Call('R_swig_GaussianImageSource_SetOrigin', self, Origin, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaussianImageSource_SetOrigin`, 'returnType') = 'void'
attr(`GaussianImageSource_SetOrigin`, "inputTypes") = c('_p_itk__simple__GaussianImageSource', 'numeric')
class(`GaussianImageSource_SetOrigin`) = c("SWIGFunction", class('GaussianImageSource_SetOrigin'))

# Start of GaussianImageSource_GetOrigin

`GaussianImageSource_GetOrigin` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaussianImageSource_GetOrigin', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaussianImageSource_GetOrigin`, 'returnType') = 'numeric'
attr(`GaussianImageSource_GetOrigin`, "inputTypes") = c('_p_itk__simple__GaussianImageSource')
class(`GaussianImageSource_GetOrigin`) = c("SWIGFunction", class('GaussianImageSource_GetOrigin'))

# Start of GaussianImageSource_SetSpacing

`GaussianImageSource_SetSpacing` = function(self, Spacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Spacing = as.numeric(Spacing);
  ;ans = .Call('R_swig_GaussianImageSource_SetSpacing', self, Spacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaussianImageSource_SetSpacing`, 'returnType') = 'void'
attr(`GaussianImageSource_SetSpacing`, "inputTypes") = c('_p_itk__simple__GaussianImageSource', 'numeric')
class(`GaussianImageSource_SetSpacing`) = c("SWIGFunction", class('GaussianImageSource_SetSpacing'))

# Start of GaussianImageSource_GetSpacing

`GaussianImageSource_GetSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaussianImageSource_GetSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaussianImageSource_GetSpacing`, 'returnType') = 'numeric'
attr(`GaussianImageSource_GetSpacing`, "inputTypes") = c('_p_itk__simple__GaussianImageSource')
class(`GaussianImageSource_GetSpacing`) = c("SWIGFunction", class('GaussianImageSource_GetSpacing'))

# Start of GaussianImageSource_SetDirection

`GaussianImageSource_SetDirection` = function(self, Direction)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Direction = as.numeric(Direction);
  ;ans = .Call('R_swig_GaussianImageSource_SetDirection', self, Direction, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaussianImageSource_SetDirection`, 'returnType') = 'void'
attr(`GaussianImageSource_SetDirection`, "inputTypes") = c('_p_itk__simple__GaussianImageSource', 'numeric')
class(`GaussianImageSource_SetDirection`) = c("SWIGFunction", class('GaussianImageSource_SetDirection'))

# Start of GaussianImageSource_GetDirection

`GaussianImageSource_GetDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaussianImageSource_GetDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaussianImageSource_GetDirection`, 'returnType') = 'numeric'
attr(`GaussianImageSource_GetDirection`, "inputTypes") = c('_p_itk__simple__GaussianImageSource')
class(`GaussianImageSource_GetDirection`) = c("SWIGFunction", class('GaussianImageSource_GetDirection'))

# Start of GaussianImageSource_SetNormalized

`GaussianImageSource_SetNormalized` = function(self, Normalized)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Normalized = as.logical(Normalized);
  ;ans = .Call('R_swig_GaussianImageSource_SetNormalized', self, Normalized, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaussianImageSource_SetNormalized`, 'returnType') = 'void'
attr(`GaussianImageSource_SetNormalized`, "inputTypes") = c('_p_itk__simple__GaussianImageSource', 'logical')
class(`GaussianImageSource_SetNormalized`) = c("SWIGFunction", class('GaussianImageSource_SetNormalized'))

# Start of GaussianImageSource_NormalizedOn

`GaussianImageSource_NormalizedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GaussianImageSource_NormalizedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaussianImageSource_NormalizedOn`, 'returnType') = 'void'
attr(`GaussianImageSource_NormalizedOn`, "inputTypes") = c('_p_itk__simple__GaussianImageSource')
class(`GaussianImageSource_NormalizedOn`) = c("SWIGFunction", class('GaussianImageSource_NormalizedOn'))

# Start of GaussianImageSource_NormalizedOff

`GaussianImageSource_NormalizedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GaussianImageSource_NormalizedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GaussianImageSource_NormalizedOff`, 'returnType') = 'void'
attr(`GaussianImageSource_NormalizedOff`, "inputTypes") = c('_p_itk__simple__GaussianImageSource')
class(`GaussianImageSource_NormalizedOff`) = c("SWIGFunction", class('GaussianImageSource_NormalizedOff'))

# Start of GaussianImageSource_GetNormalized

`GaussianImageSource_GetNormalized` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaussianImageSource_GetNormalized', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaussianImageSource_GetNormalized`, 'returnType') = 'logical'
attr(`GaussianImageSource_GetNormalized`, "inputTypes") = c('_p_itk__simple__GaussianImageSource')
class(`GaussianImageSource_GetNormalized`) = c("SWIGFunction", class('GaussianImageSource_GetNormalized'))

# Start of GaussianImageSource_GetName

`GaussianImageSource_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaussianImageSource_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaussianImageSource_GetName`, 'returnType') = 'character'
attr(`GaussianImageSource_GetName`, "inputTypes") = c('_p_itk__simple__GaussianImageSource')
class(`GaussianImageSource_GetName`) = c("SWIGFunction", class('GaussianImageSource_GetName'))

# Start of GaussianImageSource_ToString

`GaussianImageSource_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GaussianImageSource_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GaussianImageSource_ToString`, 'returnType') = 'character'
attr(`GaussianImageSource_ToString`, "inputTypes") = c('_p_itk__simple__GaussianImageSource')
class(`GaussianImageSource_ToString`) = c("SWIGFunction", class('GaussianImageSource_ToString'))

# Start of GaussianImageSource_Execute

`GaussianImageSource_Execute` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GaussianImageSource_Execute', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaussianImageSource_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GaussianImageSource_Execute`, "inputTypes") = c('_p_itk__simple__GaussianImageSource')
class(`GaussianImageSource_Execute`) = c("SWIGFunction", class('GaussianImageSource_Execute'))

# Start of accessor method for itk::simple::GaussianImageSource
setMethod('$', '_p_itk__simple__GaussianImageSource', function(x, name)

{
  accessorFuns = list('SetOutputPixelType' = GaussianImageSource_SetOutputPixelType, 'GetOutputPixelType' = GaussianImageSource_GetOutputPixelType, 'SetSize' = GaussianImageSource_SetSize, 'GetSize' = GaussianImageSource_GetSize, 'SetSigma' = GaussianImageSource_SetSigma, 'GetSigma' = GaussianImageSource_GetSigma, 'SetMean' = GaussianImageSource_SetMean, 'GetMean' = GaussianImageSource_GetMean, 'SetScale' = GaussianImageSource_SetScale, 'GetScale' = GaussianImageSource_GetScale, 'SetOrigin' = GaussianImageSource_SetOrigin, 'GetOrigin' = GaussianImageSource_GetOrigin, 'SetSpacing' = GaussianImageSource_SetSpacing, 'GetSpacing' = GaussianImageSource_GetSpacing, 'SetDirection' = GaussianImageSource_SetDirection, 'GetDirection' = GaussianImageSource_GetDirection, 'SetNormalized' = GaussianImageSource_SetNormalized, 'NormalizedOn' = GaussianImageSource_NormalizedOn, 'NormalizedOff' = GaussianImageSource_NormalizedOff, 'GetNormalized' = GaussianImageSource_GetNormalized, 'GetName' = GaussianImageSource_GetName, 'ToString' = GaussianImageSource_ToString, 'Execute' = GaussianImageSource_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GaussianImageSource
# Start of GaussianSource

`GaussianSource__SWIG_0` = function(outputPixelType, size, sigma, mean, scale, origin, spacing, direction, normalized, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  mean = as.numeric(mean);
  
  origin = as.numeric(origin);
  spacing = as.numeric(spacing);
  direction = as.numeric(direction);
  normalized = as.logical(normalized);
  ;ans = .Call('R_swig_GaussianSource__SWIG_0', outputPixelType, size, sigma, mean, scale, origin, spacing, direction, normalized, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaussianSource__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GaussianSource__SWIG_0`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'logical')
class(`GaussianSource__SWIG_0`) = c("SWIGFunction", class('GaussianSource__SWIG_0'))

# Start of GaussianSource

`GaussianSource__SWIG_1` = function(outputPixelType, size, sigma, mean, scale, origin, spacing, direction, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  mean = as.numeric(mean);
  
  origin = as.numeric(origin);
  spacing = as.numeric(spacing);
  direction = as.numeric(direction);
  ;ans = .Call('R_swig_GaussianSource__SWIG_1', outputPixelType, size, sigma, mean, scale, origin, spacing, direction, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaussianSource__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GaussianSource__SWIG_1`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
class(`GaussianSource__SWIG_1`) = c("SWIGFunction", class('GaussianSource__SWIG_1'))

# Start of GaussianSource

`GaussianSource__SWIG_2` = function(outputPixelType, size, sigma, mean, scale, origin, spacing, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  mean = as.numeric(mean);
  
  origin = as.numeric(origin);
  spacing = as.numeric(spacing);
  ;ans = .Call('R_swig_GaussianSource__SWIG_2', outputPixelType, size, sigma, mean, scale, origin, spacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaussianSource__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GaussianSource__SWIG_2`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
class(`GaussianSource__SWIG_2`) = c("SWIGFunction", class('GaussianSource__SWIG_2'))

# Start of GaussianSource

`GaussianSource__SWIG_3` = function(outputPixelType, size, sigma, mean, scale, origin, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  mean = as.numeric(mean);
  
  origin = as.numeric(origin);
  ;ans = .Call('R_swig_GaussianSource__SWIG_3', outputPixelType, size, sigma, mean, scale, origin, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaussianSource__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`GaussianSource__SWIG_3`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric', 'numeric')
class(`GaussianSource__SWIG_3`) = c("SWIGFunction", class('GaussianSource__SWIG_3'))

# Start of GaussianSource

`GaussianSource__SWIG_4` = function(outputPixelType, size, sigma, mean, scale, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  mean = as.numeric(mean);
  
  ;ans = .Call('R_swig_GaussianSource__SWIG_4', outputPixelType, size, sigma, mean, scale, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaussianSource__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`GaussianSource__SWIG_4`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric')
class(`GaussianSource__SWIG_4`) = c("SWIGFunction", class('GaussianSource__SWIG_4'))

# Start of GaussianSource

`GaussianSource__SWIG_5` = function(outputPixelType, size, sigma, mean, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  mean = as.numeric(mean);
  ;ans = .Call('R_swig_GaussianSource__SWIG_5', outputPixelType, size, sigma, mean, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaussianSource__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`GaussianSource__SWIG_5`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric')
class(`GaussianSource__SWIG_5`) = c("SWIGFunction", class('GaussianSource__SWIG_5'))

# Start of GaussianSource

`GaussianSource__SWIG_6` = function(outputPixelType, size, sigma, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  ;ans = .Call('R_swig_GaussianSource__SWIG_6', outputPixelType, size, sigma, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaussianSource__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`GaussianSource__SWIG_6`, "inputTypes") = c('character', 'integer', 'numeric')
class(`GaussianSource__SWIG_6`) = c("SWIGFunction", class('GaussianSource__SWIG_6'))

# Start of GaussianSource

`GaussianSource__SWIG_7` = function(outputPixelType, size, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  ;ans = .Call('R_swig_GaussianSource__SWIG_7', outputPixelType, size, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaussianSource__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`GaussianSource__SWIG_7`, "inputTypes") = c('character', 'integer')
class(`GaussianSource__SWIG_7`) = c("SWIGFunction", class('GaussianSource__SWIG_7'))

# Start of GaussianSource

`GaussianSource__SWIG_8` = function(outputPixelType, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  ;ans = .Call('R_swig_GaussianSource__SWIG_8', outputPixelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaussianSource__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`GaussianSource__SWIG_8`, "inputTypes") = c('character')
class(`GaussianSource__SWIG_8`) = c("SWIGFunction", class('GaussianSource__SWIG_8'))

# Start of GaussianSource

`GaussianSource__SWIG_9` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_GaussianSource__SWIG_9', as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GaussianSource__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
class(`GaussianSource__SWIG_9`) = c("SWIGFunction", class('GaussianSource__SWIG_9'))

`GaussianSource` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 10
  if (argc == 0) {
    f <- GaussianSource__SWIG_9; 
  } else if (argc == 1) {
    if ( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) {
      f <- GaussianSource__SWIG_8; 
    }
  } else if (argc == 2) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- GaussianSource__SWIG_7; 
    }
  } else if (argc == 3) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) )) {
      f <- GaussianSource__SWIG_6; 
    }
  } else if (argc == 4) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) )) {
      f <- GaussianSource__SWIG_5; 
    }
  } else if (argc == 5) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- GaussianSource__SWIG_4; 
    }
  } else if (argc == 6) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) )) {
      f <- GaussianSource__SWIG_3; 
    }
  } else if (argc == 7) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) ) && ( is.numeric(argv[[7]]) )) {
      f <- GaussianSource__SWIG_2; 
    }
  } else if (argc == 8) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) ) && ( is.numeric(argv[[7]]) ) && ( is.numeric(argv[[8]]) )) {
      f <- GaussianSource__SWIG_1; 
    }
  } else if (argc == 9) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) ) && ( is.numeric(argv[[7]]) ) && ( is.numeric(argv[[8]]) ) && ( is.logical(argv[[9]]) && length(argv[[9]]) == 1 )) {
      f <- GaussianSource__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GaussianSource with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GeodesicActiveContourLevelSetImageFilter

`delete_GeodesicActiveContourLevelSetImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GeodesicActiveContourLevelSetImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GeodesicActiveContourLevelSetImageFilter`, 'returnType') = 'void'
attr(`delete_GeodesicActiveContourLevelSetImageFilter`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter')
class(`delete_GeodesicActiveContourLevelSetImageFilter`) = c("SWIGFunction", class('delete_GeodesicActiveContourLevelSetImageFilter'))

# Start of new_GeodesicActiveContourLevelSetImageFilter

`GeodesicActiveContourLevelSetImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GeodesicActiveContourLevelSetImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GeodesicActiveContourLevelSetImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GeodesicActiveContourLevelSetImageFilter);
  ans
  
}

attr(`GeodesicActiveContourLevelSetImageFilter`, 'returnType') = '_p_itk__simple__GeodesicActiveContourLevelSetImageFilter'
class(`GeodesicActiveContourLevelSetImageFilter`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter'))

# Start of GeodesicActiveContourLevelSetImageFilter_SetMaximumRMSError

`GeodesicActiveContourLevelSetImageFilter_SetMaximumRMSError` = function(self, MaximumRMSError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSetImageFilter_SetMaximumRMSError', self, MaximumRMSError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_SetMaximumRMSError`, 'returnType') = 'void'
attr(`GeodesicActiveContourLevelSetImageFilter_SetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter', 'numeric')
class(`GeodesicActiveContourLevelSetImageFilter_SetMaximumRMSError`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_SetMaximumRMSError'))

# Start of GeodesicActiveContourLevelSetImageFilter_GetMaximumRMSError

`GeodesicActiveContourLevelSetImageFilter_GetMaximumRMSError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GeodesicActiveContourLevelSetImageFilter_GetMaximumRMSError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_GetMaximumRMSError`, 'returnType') = 'numeric'
attr(`GeodesicActiveContourLevelSetImageFilter_GetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter')
class(`GeodesicActiveContourLevelSetImageFilter_GetMaximumRMSError`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_GetMaximumRMSError'))

# Start of GeodesicActiveContourLevelSetImageFilter_SetPropagationScaling

`GeodesicActiveContourLevelSetImageFilter_SetPropagationScaling` = function(self, PropagationScaling)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSetImageFilter_SetPropagationScaling', self, PropagationScaling, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_SetPropagationScaling`, 'returnType') = 'void'
attr(`GeodesicActiveContourLevelSetImageFilter_SetPropagationScaling`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter', 'numeric')
class(`GeodesicActiveContourLevelSetImageFilter_SetPropagationScaling`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_SetPropagationScaling'))

# Start of GeodesicActiveContourLevelSetImageFilter_GetPropagationScaling

`GeodesicActiveContourLevelSetImageFilter_GetPropagationScaling` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GeodesicActiveContourLevelSetImageFilter_GetPropagationScaling', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_GetPropagationScaling`, 'returnType') = 'numeric'
attr(`GeodesicActiveContourLevelSetImageFilter_GetPropagationScaling`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter')
class(`GeodesicActiveContourLevelSetImageFilter_GetPropagationScaling`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_GetPropagationScaling'))

# Start of GeodesicActiveContourLevelSetImageFilter_SetCurvatureScaling

`GeodesicActiveContourLevelSetImageFilter_SetCurvatureScaling` = function(self, CurvatureScaling)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSetImageFilter_SetCurvatureScaling', self, CurvatureScaling, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_SetCurvatureScaling`, 'returnType') = 'void'
attr(`GeodesicActiveContourLevelSetImageFilter_SetCurvatureScaling`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter', 'numeric')
class(`GeodesicActiveContourLevelSetImageFilter_SetCurvatureScaling`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_SetCurvatureScaling'))

# Start of GeodesicActiveContourLevelSetImageFilter_GetCurvatureScaling

`GeodesicActiveContourLevelSetImageFilter_GetCurvatureScaling` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GeodesicActiveContourLevelSetImageFilter_GetCurvatureScaling', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_GetCurvatureScaling`, 'returnType') = 'numeric'
attr(`GeodesicActiveContourLevelSetImageFilter_GetCurvatureScaling`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter')
class(`GeodesicActiveContourLevelSetImageFilter_GetCurvatureScaling`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_GetCurvatureScaling'))

# Start of GeodesicActiveContourLevelSetImageFilter_SetAdvectionScaling

`GeodesicActiveContourLevelSetImageFilter_SetAdvectionScaling` = function(self, AdvectionScaling)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSetImageFilter_SetAdvectionScaling', self, AdvectionScaling, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_SetAdvectionScaling`, 'returnType') = 'void'
attr(`GeodesicActiveContourLevelSetImageFilter_SetAdvectionScaling`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter', 'numeric')
class(`GeodesicActiveContourLevelSetImageFilter_SetAdvectionScaling`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_SetAdvectionScaling'))

# Start of GeodesicActiveContourLevelSetImageFilter_GetAdvectionScaling

`GeodesicActiveContourLevelSetImageFilter_GetAdvectionScaling` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GeodesicActiveContourLevelSetImageFilter_GetAdvectionScaling', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_GetAdvectionScaling`, 'returnType') = 'numeric'
attr(`GeodesicActiveContourLevelSetImageFilter_GetAdvectionScaling`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter')
class(`GeodesicActiveContourLevelSetImageFilter_GetAdvectionScaling`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_GetAdvectionScaling'))

# Start of GeodesicActiveContourLevelSetImageFilter_SetNumberOfIterations

`GeodesicActiveContourLevelSetImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSetImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`GeodesicActiveContourLevelSetImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter', 'integer')
class(`GeodesicActiveContourLevelSetImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_SetNumberOfIterations'))

# Start of GeodesicActiveContourLevelSetImageFilter_GetNumberOfIterations

`GeodesicActiveContourLevelSetImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GeodesicActiveContourLevelSetImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`GeodesicActiveContourLevelSetImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter')
class(`GeodesicActiveContourLevelSetImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_GetNumberOfIterations'))

# Start of GeodesicActiveContourLevelSetImageFilter_SetReverseExpansionDirection

`GeodesicActiveContourLevelSetImageFilter_SetReverseExpansionDirection` = function(self, ReverseExpansionDirection)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ReverseExpansionDirection = as.logical(ReverseExpansionDirection);
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSetImageFilter_SetReverseExpansionDirection', self, ReverseExpansionDirection, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_SetReverseExpansionDirection`, 'returnType') = 'void'
attr(`GeodesicActiveContourLevelSetImageFilter_SetReverseExpansionDirection`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter', 'logical')
class(`GeodesicActiveContourLevelSetImageFilter_SetReverseExpansionDirection`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_SetReverseExpansionDirection'))

# Start of GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOn

`GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOn`, 'returnType') = 'void'
attr(`GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOn`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter')
class(`GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOn`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOn'))

# Start of GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOff

`GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOff`, 'returnType') = 'void'
attr(`GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOff`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter')
class(`GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOff`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOff'))

# Start of GeodesicActiveContourLevelSetImageFilter_GetReverseExpansionDirection

`GeodesicActiveContourLevelSetImageFilter_GetReverseExpansionDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GeodesicActiveContourLevelSetImageFilter_GetReverseExpansionDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_GetReverseExpansionDirection`, 'returnType') = 'logical'
attr(`GeodesicActiveContourLevelSetImageFilter_GetReverseExpansionDirection`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter')
class(`GeodesicActiveContourLevelSetImageFilter_GetReverseExpansionDirection`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_GetReverseExpansionDirection'))

# Start of GeodesicActiveContourLevelSetImageFilter_GetElapsedIterations

`GeodesicActiveContourLevelSetImageFilter_GetElapsedIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GeodesicActiveContourLevelSetImageFilter_GetElapsedIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_GetElapsedIterations`, 'returnType') = 'integer'
attr(`GeodesicActiveContourLevelSetImageFilter_GetElapsedIterations`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter')
class(`GeodesicActiveContourLevelSetImageFilter_GetElapsedIterations`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_GetElapsedIterations'))

# Start of GeodesicActiveContourLevelSetImageFilter_GetRMSChange

`GeodesicActiveContourLevelSetImageFilter_GetRMSChange` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GeodesicActiveContourLevelSetImageFilter_GetRMSChange', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_GetRMSChange`, 'returnType') = 'numeric'
attr(`GeodesicActiveContourLevelSetImageFilter_GetRMSChange`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter')
class(`GeodesicActiveContourLevelSetImageFilter_GetRMSChange`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_GetRMSChange'))

# Start of GeodesicActiveContourLevelSetImageFilter_GetName

`GeodesicActiveContourLevelSetImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GeodesicActiveContourLevelSetImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_GetName`, 'returnType') = 'character'
attr(`GeodesicActiveContourLevelSetImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter')
class(`GeodesicActiveContourLevelSetImageFilter_GetName`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_GetName'))

# Start of GeodesicActiveContourLevelSetImageFilter_ToString

`GeodesicActiveContourLevelSetImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GeodesicActiveContourLevelSetImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_ToString`, 'returnType') = 'character'
attr(`GeodesicActiveContourLevelSetImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter')
class(`GeodesicActiveContourLevelSetImageFilter_ToString`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_ToString'))

# Start of GeodesicActiveContourLevelSetImageFilter_Execute

`GeodesicActiveContourLevelSetImageFilter_Execute` = function(self, initialImage, featureImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSetImageFilter_Execute', self, initialImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSetImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GeodesicActiveContourLevelSetImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GeodesicActiveContourLevelSetImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`GeodesicActiveContourLevelSetImageFilter_Execute`) = c("SWIGFunction", class('GeodesicActiveContourLevelSetImageFilter_Execute'))

# Start of accessor method for itk::simple::GeodesicActiveContourLevelSetImageFilter
setMethod('$', '_p_itk__simple__GeodesicActiveContourLevelSetImageFilter', function(x, name)

{
  accessorFuns = list('SetMaximumRMSError' = GeodesicActiveContourLevelSetImageFilter_SetMaximumRMSError, 'GetMaximumRMSError' = GeodesicActiveContourLevelSetImageFilter_GetMaximumRMSError, 'SetPropagationScaling' = GeodesicActiveContourLevelSetImageFilter_SetPropagationScaling, 'GetPropagationScaling' = GeodesicActiveContourLevelSetImageFilter_GetPropagationScaling, 'SetCurvatureScaling' = GeodesicActiveContourLevelSetImageFilter_SetCurvatureScaling, 'GetCurvatureScaling' = GeodesicActiveContourLevelSetImageFilter_GetCurvatureScaling, 'SetAdvectionScaling' = GeodesicActiveContourLevelSetImageFilter_SetAdvectionScaling, 'GetAdvectionScaling' = GeodesicActiveContourLevelSetImageFilter_GetAdvectionScaling, 'SetNumberOfIterations' = GeodesicActiveContourLevelSetImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = GeodesicActiveContourLevelSetImageFilter_GetNumberOfIterations, 'SetReverseExpansionDirection' = GeodesicActiveContourLevelSetImageFilter_SetReverseExpansionDirection, 'ReverseExpansionDirectionOn' = GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOn, 'ReverseExpansionDirectionOff' = GeodesicActiveContourLevelSetImageFilter_ReverseExpansionDirectionOff, 'GetReverseExpansionDirection' = GeodesicActiveContourLevelSetImageFilter_GetReverseExpansionDirection, 'GetElapsedIterations' = GeodesicActiveContourLevelSetImageFilter_GetElapsedIterations, 'GetRMSChange' = GeodesicActiveContourLevelSetImageFilter_GetRMSChange, 'GetName' = GeodesicActiveContourLevelSetImageFilter_GetName, 'ToString' = GeodesicActiveContourLevelSetImageFilter_ToString, 'Execute' = GeodesicActiveContourLevelSetImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GeodesicActiveContourLevelSetImageFilter
# Start of GeodesicActiveContourLevelSet

`GeodesicActiveContourLevelSet__SWIG_0` = function(initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, advectionScaling, numberOfIterations, reverseExpansionDirection, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  reverseExpansionDirection = as.logical(reverseExpansionDirection);
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSet__SWIG_0', initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, advectionScaling, numberOfIterations, reverseExpansionDirection, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSet__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GeodesicActiveContourLevelSet__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric', 'integer', 'logical')
class(`GeodesicActiveContourLevelSet__SWIG_0`) = c("SWIGFunction", class('GeodesicActiveContourLevelSet__SWIG_0'))

# Start of GeodesicActiveContourLevelSet

`GeodesicActiveContourLevelSet__SWIG_1` = function(initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, advectionScaling, numberOfIterations, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSet__SWIG_1', initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, advectionScaling, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSet__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GeodesicActiveContourLevelSet__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric', 'integer')
class(`GeodesicActiveContourLevelSet__SWIG_1`) = c("SWIGFunction", class('GeodesicActiveContourLevelSet__SWIG_1'))

# Start of GeodesicActiveContourLevelSet

`GeodesicActiveContourLevelSet__SWIG_2` = function(initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, advectionScaling, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSet__SWIG_2', initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, advectionScaling, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSet__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GeodesicActiveContourLevelSet__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric')
class(`GeodesicActiveContourLevelSet__SWIG_2`) = c("SWIGFunction", class('GeodesicActiveContourLevelSet__SWIG_2'))

# Start of GeodesicActiveContourLevelSet

`GeodesicActiveContourLevelSet__SWIG_3` = function(initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSet__SWIG_3', initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSet__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`GeodesicActiveContourLevelSet__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric')
class(`GeodesicActiveContourLevelSet__SWIG_3`) = c("SWIGFunction", class('GeodesicActiveContourLevelSet__SWIG_3'))

# Start of GeodesicActiveContourLevelSet

`GeodesicActiveContourLevelSet__SWIG_4` = function(initialImage, featureImage, maximumRMSError, propagationScaling, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSet__SWIG_4', initialImage, featureImage, maximumRMSError, propagationScaling, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSet__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`GeodesicActiveContourLevelSet__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`GeodesicActiveContourLevelSet__SWIG_4`) = c("SWIGFunction", class('GeodesicActiveContourLevelSet__SWIG_4'))

# Start of GeodesicActiveContourLevelSet

`GeodesicActiveContourLevelSet__SWIG_5` = function(initialImage, featureImage, maximumRMSError, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSet__SWIG_5', initialImage, featureImage, maximumRMSError, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSet__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`GeodesicActiveContourLevelSet__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`GeodesicActiveContourLevelSet__SWIG_5`) = c("SWIGFunction", class('GeodesicActiveContourLevelSet__SWIG_5'))

# Start of GeodesicActiveContourLevelSet

`GeodesicActiveContourLevelSet__SWIG_6` = function(initialImage, featureImage, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_GeodesicActiveContourLevelSet__SWIG_6', initialImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GeodesicActiveContourLevelSet__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`GeodesicActiveContourLevelSet__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`GeodesicActiveContourLevelSet__SWIG_6`) = c("SWIGFunction", class('GeodesicActiveContourLevelSet__SWIG_6'))

`GeodesicActiveContourLevelSet` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 7
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- GeodesicActiveContourLevelSet__SWIG_6; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- GeodesicActiveContourLevelSet__SWIG_5; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- GeodesicActiveContourLevelSet__SWIG_4; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- GeodesicActiveContourLevelSet__SWIG_3; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- GeodesicActiveContourLevelSet__SWIG_2; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- GeodesicActiveContourLevelSet__SWIG_1; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 ) && ( is.logical(argv[[8]]) && length(argv[[8]]) == 1 )) {
      f <- GeodesicActiveContourLevelSet__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GeodesicActiveContourLevelSet with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GradientAnisotropicDiffusionImageFilter

`delete_GradientAnisotropicDiffusionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GradientAnisotropicDiffusionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GradientAnisotropicDiffusionImageFilter`, 'returnType') = 'void'
attr(`delete_GradientAnisotropicDiffusionImageFilter`, "inputTypes") = c('_p_itk__simple__GradientAnisotropicDiffusionImageFilter')
class(`delete_GradientAnisotropicDiffusionImageFilter`) = c("SWIGFunction", class('delete_GradientAnisotropicDiffusionImageFilter'))

# Start of new_GradientAnisotropicDiffusionImageFilter

`GradientAnisotropicDiffusionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GradientAnisotropicDiffusionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GradientAnisotropicDiffusionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GradientAnisotropicDiffusionImageFilter);
  ans
  
}

attr(`GradientAnisotropicDiffusionImageFilter`, 'returnType') = '_p_itk__simple__GradientAnisotropicDiffusionImageFilter'
class(`GradientAnisotropicDiffusionImageFilter`) = c("SWIGFunction", class('GradientAnisotropicDiffusionImageFilter'))

# Start of GradientAnisotropicDiffusionImageFilter_SetTimeStep

`GradientAnisotropicDiffusionImageFilter_SetTimeStep` = function(self, TimeStep)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GradientAnisotropicDiffusionImageFilter_SetTimeStep', self, TimeStep, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientAnisotropicDiffusionImageFilter_SetTimeStep`, 'returnType') = 'void'
attr(`GradientAnisotropicDiffusionImageFilter_SetTimeStep`, "inputTypes") = c('_p_itk__simple__GradientAnisotropicDiffusionImageFilter', 'numeric')
class(`GradientAnisotropicDiffusionImageFilter_SetTimeStep`) = c("SWIGFunction", class('GradientAnisotropicDiffusionImageFilter_SetTimeStep'))

# Start of GradientAnisotropicDiffusionImageFilter_GetTimeStep

`GradientAnisotropicDiffusionImageFilter_GetTimeStep` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientAnisotropicDiffusionImageFilter_GetTimeStep', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientAnisotropicDiffusionImageFilter_GetTimeStep`, 'returnType') = 'numeric'
attr(`GradientAnisotropicDiffusionImageFilter_GetTimeStep`, "inputTypes") = c('_p_itk__simple__GradientAnisotropicDiffusionImageFilter')
class(`GradientAnisotropicDiffusionImageFilter_GetTimeStep`) = c("SWIGFunction", class('GradientAnisotropicDiffusionImageFilter_GetTimeStep'))

# Start of GradientAnisotropicDiffusionImageFilter_SetConductanceParameter

`GradientAnisotropicDiffusionImageFilter_SetConductanceParameter` = function(self, ConductanceParameter)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GradientAnisotropicDiffusionImageFilter_SetConductanceParameter', self, ConductanceParameter, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientAnisotropicDiffusionImageFilter_SetConductanceParameter`, 'returnType') = 'void'
attr(`GradientAnisotropicDiffusionImageFilter_SetConductanceParameter`, "inputTypes") = c('_p_itk__simple__GradientAnisotropicDiffusionImageFilter', 'numeric')
class(`GradientAnisotropicDiffusionImageFilter_SetConductanceParameter`) = c("SWIGFunction", class('GradientAnisotropicDiffusionImageFilter_SetConductanceParameter'))

# Start of GradientAnisotropicDiffusionImageFilter_GetConductanceParameter

`GradientAnisotropicDiffusionImageFilter_GetConductanceParameter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientAnisotropicDiffusionImageFilter_GetConductanceParameter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientAnisotropicDiffusionImageFilter_GetConductanceParameter`, 'returnType') = 'numeric'
attr(`GradientAnisotropicDiffusionImageFilter_GetConductanceParameter`, "inputTypes") = c('_p_itk__simple__GradientAnisotropicDiffusionImageFilter')
class(`GradientAnisotropicDiffusionImageFilter_GetConductanceParameter`) = c("SWIGFunction", class('GradientAnisotropicDiffusionImageFilter_GetConductanceParameter'))

# Start of GradientAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval

`GradientAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval` = function(self, ConductanceScalingUpdateInterval)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ConductanceScalingUpdateInterval = as.integer(ConductanceScalingUpdateInterval);
  
  if(length(ConductanceScalingUpdateInterval) > 1) {
    warning("using only the first element of ConductanceScalingUpdateInterval");
  };
  
  ;ans = .Call('R_swig_GradientAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval', self, ConductanceScalingUpdateInterval, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval`, 'returnType') = 'void'
attr(`GradientAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval`, "inputTypes") = c('_p_itk__simple__GradientAnisotropicDiffusionImageFilter', 'integer')
class(`GradientAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval`) = c("SWIGFunction", class('GradientAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval'))

# Start of GradientAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval

`GradientAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval`, 'returnType') = 'integer'
attr(`GradientAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval`, "inputTypes") = c('_p_itk__simple__GradientAnisotropicDiffusionImageFilter')
class(`GradientAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval`) = c("SWIGFunction", class('GradientAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval'))

# Start of GradientAnisotropicDiffusionImageFilter_SetNumberOfIterations

`GradientAnisotropicDiffusionImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_GradientAnisotropicDiffusionImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientAnisotropicDiffusionImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`GradientAnisotropicDiffusionImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__GradientAnisotropicDiffusionImageFilter', 'integer')
class(`GradientAnisotropicDiffusionImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('GradientAnisotropicDiffusionImageFilter_SetNumberOfIterations'))

# Start of GradientAnisotropicDiffusionImageFilter_GetNumberOfIterations

`GradientAnisotropicDiffusionImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientAnisotropicDiffusionImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientAnisotropicDiffusionImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`GradientAnisotropicDiffusionImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__GradientAnisotropicDiffusionImageFilter')
class(`GradientAnisotropicDiffusionImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('GradientAnisotropicDiffusionImageFilter_GetNumberOfIterations'))

# Start of GradientAnisotropicDiffusionImageFilter_GetName

`GradientAnisotropicDiffusionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientAnisotropicDiffusionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientAnisotropicDiffusionImageFilter_GetName`, 'returnType') = 'character'
attr(`GradientAnisotropicDiffusionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GradientAnisotropicDiffusionImageFilter')
class(`GradientAnisotropicDiffusionImageFilter_GetName`) = c("SWIGFunction", class('GradientAnisotropicDiffusionImageFilter_GetName'))

# Start of GradientAnisotropicDiffusionImageFilter_ToString

`GradientAnisotropicDiffusionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientAnisotropicDiffusionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientAnisotropicDiffusionImageFilter_ToString`, 'returnType') = 'character'
attr(`GradientAnisotropicDiffusionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GradientAnisotropicDiffusionImageFilter')
class(`GradientAnisotropicDiffusionImageFilter_ToString`) = c("SWIGFunction", class('GradientAnisotropicDiffusionImageFilter_ToString'))

# Start of GradientAnisotropicDiffusionImageFilter_Execute

`GradientAnisotropicDiffusionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GradientAnisotropicDiffusionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientAnisotropicDiffusionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientAnisotropicDiffusionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GradientAnisotropicDiffusionImageFilter', '_p_itk__simple__Image')
class(`GradientAnisotropicDiffusionImageFilter_Execute`) = c("SWIGFunction", class('GradientAnisotropicDiffusionImageFilter_Execute'))

# Start of GradientAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep

`GradientAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep` = function(self, inImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(inImage, "ExternalReference")) inImage = slot(inImage,"ref"); 
  ;.Call('R_swig_GradientAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep', self, inImage, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep`, 'returnType') = 'numeric'
attr(`GradientAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep`, "inputTypes") = c('_p_itk__simple__GradientAnisotropicDiffusionImageFilter', '_p_itk__simple__Image')
class(`GradientAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep`) = c("SWIGFunction", class('GradientAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep'))

# Start of accessor method for itk::simple::GradientAnisotropicDiffusionImageFilter
setMethod('$', '_p_itk__simple__GradientAnisotropicDiffusionImageFilter', function(x, name)

{
  accessorFuns = list('SetTimeStep' = GradientAnisotropicDiffusionImageFilter_SetTimeStep, 'GetTimeStep' = GradientAnisotropicDiffusionImageFilter_GetTimeStep, 'SetConductanceParameter' = GradientAnisotropicDiffusionImageFilter_SetConductanceParameter, 'GetConductanceParameter' = GradientAnisotropicDiffusionImageFilter_GetConductanceParameter, 'SetConductanceScalingUpdateInterval' = GradientAnisotropicDiffusionImageFilter_SetConductanceScalingUpdateInterval, 'GetConductanceScalingUpdateInterval' = GradientAnisotropicDiffusionImageFilter_GetConductanceScalingUpdateInterval, 'SetNumberOfIterations' = GradientAnisotropicDiffusionImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = GradientAnisotropicDiffusionImageFilter_GetNumberOfIterations, 'GetName' = GradientAnisotropicDiffusionImageFilter_GetName, 'ToString' = GradientAnisotropicDiffusionImageFilter_ToString, 'Execute' = GradientAnisotropicDiffusionImageFilter_Execute, 'EstimateOptimalTimeStep' = GradientAnisotropicDiffusionImageFilter_EstimateOptimalTimeStep);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GradientAnisotropicDiffusionImageFilter
# Start of GradientAnisotropicDiffusion

`GradientAnisotropicDiffusion__SWIG_0` = function(image1, timeStep, conductanceParameter, conductanceScalingUpdateInterval, numberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  conductanceScalingUpdateInterval = as.integer(conductanceScalingUpdateInterval);
  
  if(length(conductanceScalingUpdateInterval) > 1) {
    warning("using only the first element of conductanceScalingUpdateInterval");
  };
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_GradientAnisotropicDiffusion__SWIG_0', image1, timeStep, conductanceParameter, conductanceScalingUpdateInterval, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientAnisotropicDiffusion__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientAnisotropicDiffusion__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'integer', 'integer')
class(`GradientAnisotropicDiffusion__SWIG_0`) = c("SWIGFunction", class('GradientAnisotropicDiffusion__SWIG_0'))

# Start of GradientAnisotropicDiffusion

`GradientAnisotropicDiffusion__SWIG_1` = function(image1, timeStep, conductanceParameter, conductanceScalingUpdateInterval, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  conductanceScalingUpdateInterval = as.integer(conductanceScalingUpdateInterval);
  
  if(length(conductanceScalingUpdateInterval) > 1) {
    warning("using only the first element of conductanceScalingUpdateInterval");
  };
  
  ;ans = .Call('R_swig_GradientAnisotropicDiffusion__SWIG_1', image1, timeStep, conductanceParameter, conductanceScalingUpdateInterval, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientAnisotropicDiffusion__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientAnisotropicDiffusion__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'integer')
class(`GradientAnisotropicDiffusion__SWIG_1`) = c("SWIGFunction", class('GradientAnisotropicDiffusion__SWIG_1'))

# Start of GradientAnisotropicDiffusion

`GradientAnisotropicDiffusion__SWIG_2` = function(image1, timeStep, conductanceParameter, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_GradientAnisotropicDiffusion__SWIG_2', image1, timeStep, conductanceParameter, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientAnisotropicDiffusion__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientAnisotropicDiffusion__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`GradientAnisotropicDiffusion__SWIG_2`) = c("SWIGFunction", class('GradientAnisotropicDiffusion__SWIG_2'))

# Start of GradientAnisotropicDiffusion

`GradientAnisotropicDiffusion__SWIG_3` = function(image1, timeStep, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_GradientAnisotropicDiffusion__SWIG_3', image1, timeStep, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientAnisotropicDiffusion__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientAnisotropicDiffusion__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`GradientAnisotropicDiffusion__SWIG_3`) = c("SWIGFunction", class('GradientAnisotropicDiffusion__SWIG_3'))

# Start of GradientAnisotropicDiffusion

`GradientAnisotropicDiffusion__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GradientAnisotropicDiffusion__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientAnisotropicDiffusion__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientAnisotropicDiffusion__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`GradientAnisotropicDiffusion__SWIG_4`) = c("SWIGFunction", class('GradientAnisotropicDiffusion__SWIG_4'))

`GradientAnisotropicDiffusion` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- GradientAnisotropicDiffusion__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- GradientAnisotropicDiffusion__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- GradientAnisotropicDiffusion__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- GradientAnisotropicDiffusion__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- GradientAnisotropicDiffusion__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GradientAnisotropicDiffusion with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GradientImageFilter

`delete_GradientImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GradientImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GradientImageFilter`, 'returnType') = 'void'
attr(`delete_GradientImageFilter`, "inputTypes") = c('_p_itk__simple__GradientImageFilter')
class(`delete_GradientImageFilter`) = c("SWIGFunction", class('delete_GradientImageFilter'))

# Start of new_GradientImageFilter

`GradientImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GradientImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GradientImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GradientImageFilter);
  ans
  
}

attr(`GradientImageFilter`, 'returnType') = '_p_itk__simple__GradientImageFilter'
class(`GradientImageFilter`) = c("SWIGFunction", class('GradientImageFilter'))

# Start of GradientImageFilter_SetUseImageSpacing

`GradientImageFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_GradientImageFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientImageFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`GradientImageFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__GradientImageFilter', 'logical')
class(`GradientImageFilter_SetUseImageSpacing`) = c("SWIGFunction", class('GradientImageFilter_SetUseImageSpacing'))

# Start of GradientImageFilter_UseImageSpacingOn

`GradientImageFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GradientImageFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientImageFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`GradientImageFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__GradientImageFilter')
class(`GradientImageFilter_UseImageSpacingOn`) = c("SWIGFunction", class('GradientImageFilter_UseImageSpacingOn'))

# Start of GradientImageFilter_UseImageSpacingOff

`GradientImageFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GradientImageFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientImageFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`GradientImageFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__GradientImageFilter')
class(`GradientImageFilter_UseImageSpacingOff`) = c("SWIGFunction", class('GradientImageFilter_UseImageSpacingOff'))

# Start of GradientImageFilter_GetUseImageSpacing

`GradientImageFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientImageFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientImageFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`GradientImageFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__GradientImageFilter')
class(`GradientImageFilter_GetUseImageSpacing`) = c("SWIGFunction", class('GradientImageFilter_GetUseImageSpacing'))

# Start of GradientImageFilter_SetUseImageDirection

`GradientImageFilter_SetUseImageDirection` = function(self, UseImageDirection)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageDirection = as.logical(UseImageDirection);
  ;ans = .Call('R_swig_GradientImageFilter_SetUseImageDirection', self, UseImageDirection, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientImageFilter_SetUseImageDirection`, 'returnType') = 'void'
attr(`GradientImageFilter_SetUseImageDirection`, "inputTypes") = c('_p_itk__simple__GradientImageFilter', 'logical')
class(`GradientImageFilter_SetUseImageDirection`) = c("SWIGFunction", class('GradientImageFilter_SetUseImageDirection'))

# Start of GradientImageFilter_UseImageDirectionOn

`GradientImageFilter_UseImageDirectionOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GradientImageFilter_UseImageDirectionOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientImageFilter_UseImageDirectionOn`, 'returnType') = 'void'
attr(`GradientImageFilter_UseImageDirectionOn`, "inputTypes") = c('_p_itk__simple__GradientImageFilter')
class(`GradientImageFilter_UseImageDirectionOn`) = c("SWIGFunction", class('GradientImageFilter_UseImageDirectionOn'))

# Start of GradientImageFilter_UseImageDirectionOff

`GradientImageFilter_UseImageDirectionOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GradientImageFilter_UseImageDirectionOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientImageFilter_UseImageDirectionOff`, 'returnType') = 'void'
attr(`GradientImageFilter_UseImageDirectionOff`, "inputTypes") = c('_p_itk__simple__GradientImageFilter')
class(`GradientImageFilter_UseImageDirectionOff`) = c("SWIGFunction", class('GradientImageFilter_UseImageDirectionOff'))

# Start of GradientImageFilter_GetUseImageDirection

`GradientImageFilter_GetUseImageDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientImageFilter_GetUseImageDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientImageFilter_GetUseImageDirection`, 'returnType') = 'logical'
attr(`GradientImageFilter_GetUseImageDirection`, "inputTypes") = c('_p_itk__simple__GradientImageFilter')
class(`GradientImageFilter_GetUseImageDirection`) = c("SWIGFunction", class('GradientImageFilter_GetUseImageDirection'))

# Start of GradientImageFilter_GetName

`GradientImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientImageFilter_GetName`, 'returnType') = 'character'
attr(`GradientImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GradientImageFilter')
class(`GradientImageFilter_GetName`) = c("SWIGFunction", class('GradientImageFilter_GetName'))

# Start of GradientImageFilter_ToString

`GradientImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientImageFilter_ToString`, 'returnType') = 'character'
attr(`GradientImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GradientImageFilter')
class(`GradientImageFilter_ToString`) = c("SWIGFunction", class('GradientImageFilter_ToString'))

# Start of GradientImageFilter_Execute

`GradientImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GradientImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GradientImageFilter', '_p_itk__simple__Image')
class(`GradientImageFilter_Execute`) = c("SWIGFunction", class('GradientImageFilter_Execute'))

# Start of accessor method for itk::simple::GradientImageFilter
setMethod('$', '_p_itk__simple__GradientImageFilter', function(x, name)

{
  accessorFuns = list('SetUseImageSpacing' = GradientImageFilter_SetUseImageSpacing, 'UseImageSpacingOn' = GradientImageFilter_UseImageSpacingOn, 'UseImageSpacingOff' = GradientImageFilter_UseImageSpacingOff, 'GetUseImageSpacing' = GradientImageFilter_GetUseImageSpacing, 'SetUseImageDirection' = GradientImageFilter_SetUseImageDirection, 'UseImageDirectionOn' = GradientImageFilter_UseImageDirectionOn, 'UseImageDirectionOff' = GradientImageFilter_UseImageDirectionOff, 'GetUseImageDirection' = GradientImageFilter_GetUseImageDirection, 'GetName' = GradientImageFilter_GetName, 'ToString' = GradientImageFilter_ToString, 'Execute' = GradientImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GradientImageFilter
# Start of Gradient

`Gradient__SWIG_0` = function(image1, useImageSpacing, useImageDirection, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  useImageSpacing = as.logical(useImageSpacing);
  useImageDirection = as.logical(useImageDirection);
  ;ans = .Call('R_swig_Gradient__SWIG_0', image1, useImageSpacing, useImageDirection, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Gradient__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Gradient__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'logical')
class(`Gradient__SWIG_0`) = c("SWIGFunction", class('Gradient__SWIG_0'))

# Start of Gradient

`Gradient__SWIG_1` = function(image1, useImageSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  useImageSpacing = as.logical(useImageSpacing);
  ;ans = .Call('R_swig_Gradient__SWIG_1', image1, useImageSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Gradient__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Gradient__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`Gradient__SWIG_1`) = c("SWIGFunction", class('Gradient__SWIG_1'))

# Start of Gradient

`Gradient__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Gradient__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Gradient__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Gradient__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`Gradient__SWIG_2`) = c("SWIGFunction", class('Gradient__SWIG_2'))

`Gradient` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Gradient__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Gradient__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Gradient__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Gradient with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GradientMagnitudeImageFilter

`delete_GradientMagnitudeImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GradientMagnitudeImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GradientMagnitudeImageFilter`, 'returnType') = 'void'
attr(`delete_GradientMagnitudeImageFilter`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeImageFilter')
class(`delete_GradientMagnitudeImageFilter`) = c("SWIGFunction", class('delete_GradientMagnitudeImageFilter'))

# Start of new_GradientMagnitudeImageFilter

`GradientMagnitudeImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GradientMagnitudeImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GradientMagnitudeImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GradientMagnitudeImageFilter);
  ans
  
}

attr(`GradientMagnitudeImageFilter`, 'returnType') = '_p_itk__simple__GradientMagnitudeImageFilter'
class(`GradientMagnitudeImageFilter`) = c("SWIGFunction", class('GradientMagnitudeImageFilter'))

# Start of GradientMagnitudeImageFilter_SetUseImageSpacing

`GradientMagnitudeImageFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_GradientMagnitudeImageFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientMagnitudeImageFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`GradientMagnitudeImageFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeImageFilter', 'logical')
class(`GradientMagnitudeImageFilter_SetUseImageSpacing`) = c("SWIGFunction", class('GradientMagnitudeImageFilter_SetUseImageSpacing'))

# Start of GradientMagnitudeImageFilter_UseImageSpacingOn

`GradientMagnitudeImageFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GradientMagnitudeImageFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientMagnitudeImageFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`GradientMagnitudeImageFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeImageFilter')
class(`GradientMagnitudeImageFilter_UseImageSpacingOn`) = c("SWIGFunction", class('GradientMagnitudeImageFilter_UseImageSpacingOn'))

# Start of GradientMagnitudeImageFilter_UseImageSpacingOff

`GradientMagnitudeImageFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GradientMagnitudeImageFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientMagnitudeImageFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`GradientMagnitudeImageFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeImageFilter')
class(`GradientMagnitudeImageFilter_UseImageSpacingOff`) = c("SWIGFunction", class('GradientMagnitudeImageFilter_UseImageSpacingOff'))

# Start of GradientMagnitudeImageFilter_GetUseImageSpacing

`GradientMagnitudeImageFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientMagnitudeImageFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientMagnitudeImageFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`GradientMagnitudeImageFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeImageFilter')
class(`GradientMagnitudeImageFilter_GetUseImageSpacing`) = c("SWIGFunction", class('GradientMagnitudeImageFilter_GetUseImageSpacing'))

# Start of GradientMagnitudeImageFilter_GetName

`GradientMagnitudeImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientMagnitudeImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientMagnitudeImageFilter_GetName`, 'returnType') = 'character'
attr(`GradientMagnitudeImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeImageFilter')
class(`GradientMagnitudeImageFilter_GetName`) = c("SWIGFunction", class('GradientMagnitudeImageFilter_GetName'))

# Start of GradientMagnitudeImageFilter_ToString

`GradientMagnitudeImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientMagnitudeImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientMagnitudeImageFilter_ToString`, 'returnType') = 'character'
attr(`GradientMagnitudeImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeImageFilter')
class(`GradientMagnitudeImageFilter_ToString`) = c("SWIGFunction", class('GradientMagnitudeImageFilter_ToString'))

# Start of GradientMagnitudeImageFilter_Execute

`GradientMagnitudeImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GradientMagnitudeImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientMagnitudeImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientMagnitudeImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeImageFilter', '_p_itk__simple__Image')
class(`GradientMagnitudeImageFilter_Execute`) = c("SWIGFunction", class('GradientMagnitudeImageFilter_Execute'))

# Start of accessor method for itk::simple::GradientMagnitudeImageFilter
setMethod('$', '_p_itk__simple__GradientMagnitudeImageFilter', function(x, name)

{
  accessorFuns = list('SetUseImageSpacing' = GradientMagnitudeImageFilter_SetUseImageSpacing, 'UseImageSpacingOn' = GradientMagnitudeImageFilter_UseImageSpacingOn, 'UseImageSpacingOff' = GradientMagnitudeImageFilter_UseImageSpacingOff, 'GetUseImageSpacing' = GradientMagnitudeImageFilter_GetUseImageSpacing, 'GetName' = GradientMagnitudeImageFilter_GetName, 'ToString' = GradientMagnitudeImageFilter_ToString, 'Execute' = GradientMagnitudeImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GradientMagnitudeImageFilter
# Start of GradientMagnitude

`GradientMagnitude__SWIG_0` = function(image1, useImageSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  useImageSpacing = as.logical(useImageSpacing);
  ;ans = .Call('R_swig_GradientMagnitude__SWIG_0', image1, useImageSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientMagnitude__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientMagnitude__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`GradientMagnitude__SWIG_0`) = c("SWIGFunction", class('GradientMagnitude__SWIG_0'))

# Start of GradientMagnitude

`GradientMagnitude__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GradientMagnitude__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientMagnitude__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientMagnitude__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`GradientMagnitude__SWIG_1`) = c("SWIGFunction", class('GradientMagnitude__SWIG_1'))

`GradientMagnitude` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- GradientMagnitude__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- GradientMagnitude__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GradientMagnitude with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GradientMagnitudeRecursiveGaussianImageFilter

`delete_GradientMagnitudeRecursiveGaussianImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GradientMagnitudeRecursiveGaussianImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GradientMagnitudeRecursiveGaussianImageFilter`, 'returnType') = 'void'
attr(`delete_GradientMagnitudeRecursiveGaussianImageFilter`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeRecursiveGaussianImageFilter')
class(`delete_GradientMagnitudeRecursiveGaussianImageFilter`) = c("SWIGFunction", class('delete_GradientMagnitudeRecursiveGaussianImageFilter'))

# Start of new_GradientMagnitudeRecursiveGaussianImageFilter

`GradientMagnitudeRecursiveGaussianImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GradientMagnitudeRecursiveGaussianImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GradientMagnitudeRecursiveGaussianImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GradientMagnitudeRecursiveGaussianImageFilter);
  ans
  
}

attr(`GradientMagnitudeRecursiveGaussianImageFilter`, 'returnType') = '_p_itk__simple__GradientMagnitudeRecursiveGaussianImageFilter'
class(`GradientMagnitudeRecursiveGaussianImageFilter`) = c("SWIGFunction", class('GradientMagnitudeRecursiveGaussianImageFilter'))

# Start of GradientMagnitudeRecursiveGaussianImageFilter_SetSigma

`GradientMagnitudeRecursiveGaussianImageFilter_SetSigma` = function(self, Sigma)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GradientMagnitudeRecursiveGaussianImageFilter_SetSigma', self, Sigma, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientMagnitudeRecursiveGaussianImageFilter_SetSigma`, 'returnType') = 'void'
attr(`GradientMagnitudeRecursiveGaussianImageFilter_SetSigma`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeRecursiveGaussianImageFilter', 'numeric')
class(`GradientMagnitudeRecursiveGaussianImageFilter_SetSigma`) = c("SWIGFunction", class('GradientMagnitudeRecursiveGaussianImageFilter_SetSigma'))

# Start of GradientMagnitudeRecursiveGaussianImageFilter_GetSigma

`GradientMagnitudeRecursiveGaussianImageFilter_GetSigma` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientMagnitudeRecursiveGaussianImageFilter_GetSigma', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientMagnitudeRecursiveGaussianImageFilter_GetSigma`, 'returnType') = 'numeric'
attr(`GradientMagnitudeRecursiveGaussianImageFilter_GetSigma`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeRecursiveGaussianImageFilter')
class(`GradientMagnitudeRecursiveGaussianImageFilter_GetSigma`) = c("SWIGFunction", class('GradientMagnitudeRecursiveGaussianImageFilter_GetSigma'))

# Start of GradientMagnitudeRecursiveGaussianImageFilter_SetNormalizeAcrossScale

`GradientMagnitudeRecursiveGaussianImageFilter_SetNormalizeAcrossScale` = function(self, NormalizeAcrossScale)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NormalizeAcrossScale = as.logical(NormalizeAcrossScale);
  ;ans = .Call('R_swig_GradientMagnitudeRecursiveGaussianImageFilter_SetNormalizeAcrossScale', self, NormalizeAcrossScale, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientMagnitudeRecursiveGaussianImageFilter_SetNormalizeAcrossScale`, 'returnType') = 'void'
attr(`GradientMagnitudeRecursiveGaussianImageFilter_SetNormalizeAcrossScale`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeRecursiveGaussianImageFilter', 'logical')
class(`GradientMagnitudeRecursiveGaussianImageFilter_SetNormalizeAcrossScale`) = c("SWIGFunction", class('GradientMagnitudeRecursiveGaussianImageFilter_SetNormalizeAcrossScale'))

# Start of GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOn

`GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOn`, 'returnType') = 'void'
attr(`GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOn`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeRecursiveGaussianImageFilter')
class(`GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOn`) = c("SWIGFunction", class('GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOn'))

# Start of GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOff

`GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOff`, 'returnType') = 'void'
attr(`GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOff`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeRecursiveGaussianImageFilter')
class(`GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOff`) = c("SWIGFunction", class('GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOff'))

# Start of GradientMagnitudeRecursiveGaussianImageFilter_GetNormalizeAcrossScale

`GradientMagnitudeRecursiveGaussianImageFilter_GetNormalizeAcrossScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientMagnitudeRecursiveGaussianImageFilter_GetNormalizeAcrossScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientMagnitudeRecursiveGaussianImageFilter_GetNormalizeAcrossScale`, 'returnType') = 'logical'
attr(`GradientMagnitudeRecursiveGaussianImageFilter_GetNormalizeAcrossScale`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeRecursiveGaussianImageFilter')
class(`GradientMagnitudeRecursiveGaussianImageFilter_GetNormalizeAcrossScale`) = c("SWIGFunction", class('GradientMagnitudeRecursiveGaussianImageFilter_GetNormalizeAcrossScale'))

# Start of GradientMagnitudeRecursiveGaussianImageFilter_GetName

`GradientMagnitudeRecursiveGaussianImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientMagnitudeRecursiveGaussianImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientMagnitudeRecursiveGaussianImageFilter_GetName`, 'returnType') = 'character'
attr(`GradientMagnitudeRecursiveGaussianImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeRecursiveGaussianImageFilter')
class(`GradientMagnitudeRecursiveGaussianImageFilter_GetName`) = c("SWIGFunction", class('GradientMagnitudeRecursiveGaussianImageFilter_GetName'))

# Start of GradientMagnitudeRecursiveGaussianImageFilter_ToString

`GradientMagnitudeRecursiveGaussianImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientMagnitudeRecursiveGaussianImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientMagnitudeRecursiveGaussianImageFilter_ToString`, 'returnType') = 'character'
attr(`GradientMagnitudeRecursiveGaussianImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeRecursiveGaussianImageFilter')
class(`GradientMagnitudeRecursiveGaussianImageFilter_ToString`) = c("SWIGFunction", class('GradientMagnitudeRecursiveGaussianImageFilter_ToString'))

# Start of GradientMagnitudeRecursiveGaussianImageFilter_Execute

`GradientMagnitudeRecursiveGaussianImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GradientMagnitudeRecursiveGaussianImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientMagnitudeRecursiveGaussianImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientMagnitudeRecursiveGaussianImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GradientMagnitudeRecursiveGaussianImageFilter', '_p_itk__simple__Image')
class(`GradientMagnitudeRecursiveGaussianImageFilter_Execute`) = c("SWIGFunction", class('GradientMagnitudeRecursiveGaussianImageFilter_Execute'))

# Start of accessor method for itk::simple::GradientMagnitudeRecursiveGaussianImageFilter
setMethod('$', '_p_itk__simple__GradientMagnitudeRecursiveGaussianImageFilter', function(x, name)

{
  accessorFuns = list('SetSigma' = GradientMagnitudeRecursiveGaussianImageFilter_SetSigma, 'GetSigma' = GradientMagnitudeRecursiveGaussianImageFilter_GetSigma, 'SetNormalizeAcrossScale' = GradientMagnitudeRecursiveGaussianImageFilter_SetNormalizeAcrossScale, 'NormalizeAcrossScaleOn' = GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOn, 'NormalizeAcrossScaleOff' = GradientMagnitudeRecursiveGaussianImageFilter_NormalizeAcrossScaleOff, 'GetNormalizeAcrossScale' = GradientMagnitudeRecursiveGaussianImageFilter_GetNormalizeAcrossScale, 'GetName' = GradientMagnitudeRecursiveGaussianImageFilter_GetName, 'ToString' = GradientMagnitudeRecursiveGaussianImageFilter_ToString, 'Execute' = GradientMagnitudeRecursiveGaussianImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GradientMagnitudeRecursiveGaussianImageFilter
# Start of GradientMagnitudeRecursiveGaussian

`GradientMagnitudeRecursiveGaussian__SWIG_0` = function(image1, sigma, normalizeAcrossScale, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  normalizeAcrossScale = as.logical(normalizeAcrossScale);
  ;ans = .Call('R_swig_GradientMagnitudeRecursiveGaussian__SWIG_0', image1, sigma, normalizeAcrossScale, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientMagnitudeRecursiveGaussian__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientMagnitudeRecursiveGaussian__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`GradientMagnitudeRecursiveGaussian__SWIG_0`) = c("SWIGFunction", class('GradientMagnitudeRecursiveGaussian__SWIG_0'))

# Start of GradientMagnitudeRecursiveGaussian

`GradientMagnitudeRecursiveGaussian__SWIG_1` = function(image1, sigma, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_GradientMagnitudeRecursiveGaussian__SWIG_1', image1, sigma, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientMagnitudeRecursiveGaussian__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientMagnitudeRecursiveGaussian__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`GradientMagnitudeRecursiveGaussian__SWIG_1`) = c("SWIGFunction", class('GradientMagnitudeRecursiveGaussian__SWIG_1'))

# Start of GradientMagnitudeRecursiveGaussian

`GradientMagnitudeRecursiveGaussian__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GradientMagnitudeRecursiveGaussian__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientMagnitudeRecursiveGaussian__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientMagnitudeRecursiveGaussian__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`GradientMagnitudeRecursiveGaussian__SWIG_2`) = c("SWIGFunction", class('GradientMagnitudeRecursiveGaussian__SWIG_2'))

`GradientMagnitudeRecursiveGaussian` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- GradientMagnitudeRecursiveGaussian__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- GradientMagnitudeRecursiveGaussian__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- GradientMagnitudeRecursiveGaussian__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GradientMagnitudeRecursiveGaussian with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GradientRecursiveGaussianImageFilter

`delete_GradientRecursiveGaussianImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GradientRecursiveGaussianImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GradientRecursiveGaussianImageFilter`, 'returnType') = 'void'
attr(`delete_GradientRecursiveGaussianImageFilter`, "inputTypes") = c('_p_itk__simple__GradientRecursiveGaussianImageFilter')
class(`delete_GradientRecursiveGaussianImageFilter`) = c("SWIGFunction", class('delete_GradientRecursiveGaussianImageFilter'))

# Start of new_GradientRecursiveGaussianImageFilter

`GradientRecursiveGaussianImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GradientRecursiveGaussianImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GradientRecursiveGaussianImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GradientRecursiveGaussianImageFilter);
  ans
  
}

attr(`GradientRecursiveGaussianImageFilter`, 'returnType') = '_p_itk__simple__GradientRecursiveGaussianImageFilter'
class(`GradientRecursiveGaussianImageFilter`) = c("SWIGFunction", class('GradientRecursiveGaussianImageFilter'))

# Start of GradientRecursiveGaussianImageFilter_SetSigma

`GradientRecursiveGaussianImageFilter_SetSigma` = function(self, Sigma)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GradientRecursiveGaussianImageFilter_SetSigma', self, Sigma, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientRecursiveGaussianImageFilter_SetSigma`, 'returnType') = 'void'
attr(`GradientRecursiveGaussianImageFilter_SetSigma`, "inputTypes") = c('_p_itk__simple__GradientRecursiveGaussianImageFilter', 'numeric')
class(`GradientRecursiveGaussianImageFilter_SetSigma`) = c("SWIGFunction", class('GradientRecursiveGaussianImageFilter_SetSigma'))

# Start of GradientRecursiveGaussianImageFilter_GetSigma

`GradientRecursiveGaussianImageFilter_GetSigma` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientRecursiveGaussianImageFilter_GetSigma', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientRecursiveGaussianImageFilter_GetSigma`, 'returnType') = 'numeric'
attr(`GradientRecursiveGaussianImageFilter_GetSigma`, "inputTypes") = c('_p_itk__simple__GradientRecursiveGaussianImageFilter')
class(`GradientRecursiveGaussianImageFilter_GetSigma`) = c("SWIGFunction", class('GradientRecursiveGaussianImageFilter_GetSigma'))

# Start of GradientRecursiveGaussianImageFilter_SetNormalizeAcrossScale

`GradientRecursiveGaussianImageFilter_SetNormalizeAcrossScale` = function(self, NormalizeAcrossScale)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NormalizeAcrossScale = as.logical(NormalizeAcrossScale);
  ;ans = .Call('R_swig_GradientRecursiveGaussianImageFilter_SetNormalizeAcrossScale', self, NormalizeAcrossScale, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientRecursiveGaussianImageFilter_SetNormalizeAcrossScale`, 'returnType') = 'void'
attr(`GradientRecursiveGaussianImageFilter_SetNormalizeAcrossScale`, "inputTypes") = c('_p_itk__simple__GradientRecursiveGaussianImageFilter', 'logical')
class(`GradientRecursiveGaussianImageFilter_SetNormalizeAcrossScale`) = c("SWIGFunction", class('GradientRecursiveGaussianImageFilter_SetNormalizeAcrossScale'))

# Start of GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOn

`GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOn`, 'returnType') = 'void'
attr(`GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOn`, "inputTypes") = c('_p_itk__simple__GradientRecursiveGaussianImageFilter')
class(`GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOn`) = c("SWIGFunction", class('GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOn'))

# Start of GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOff

`GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOff`, 'returnType') = 'void'
attr(`GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOff`, "inputTypes") = c('_p_itk__simple__GradientRecursiveGaussianImageFilter')
class(`GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOff`) = c("SWIGFunction", class('GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOff'))

# Start of GradientRecursiveGaussianImageFilter_GetNormalizeAcrossScale

`GradientRecursiveGaussianImageFilter_GetNormalizeAcrossScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientRecursiveGaussianImageFilter_GetNormalizeAcrossScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientRecursiveGaussianImageFilter_GetNormalizeAcrossScale`, 'returnType') = 'logical'
attr(`GradientRecursiveGaussianImageFilter_GetNormalizeAcrossScale`, "inputTypes") = c('_p_itk__simple__GradientRecursiveGaussianImageFilter')
class(`GradientRecursiveGaussianImageFilter_GetNormalizeAcrossScale`) = c("SWIGFunction", class('GradientRecursiveGaussianImageFilter_GetNormalizeAcrossScale'))

# Start of GradientRecursiveGaussianImageFilter_SetUseImageDirection

`GradientRecursiveGaussianImageFilter_SetUseImageDirection` = function(self, UseImageDirection)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageDirection = as.logical(UseImageDirection);
  ;ans = .Call('R_swig_GradientRecursiveGaussianImageFilter_SetUseImageDirection', self, UseImageDirection, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientRecursiveGaussianImageFilter_SetUseImageDirection`, 'returnType') = 'void'
attr(`GradientRecursiveGaussianImageFilter_SetUseImageDirection`, "inputTypes") = c('_p_itk__simple__GradientRecursiveGaussianImageFilter', 'logical')
class(`GradientRecursiveGaussianImageFilter_SetUseImageDirection`) = c("SWIGFunction", class('GradientRecursiveGaussianImageFilter_SetUseImageDirection'))

# Start of GradientRecursiveGaussianImageFilter_UseImageDirectionOn

`GradientRecursiveGaussianImageFilter_UseImageDirectionOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GradientRecursiveGaussianImageFilter_UseImageDirectionOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientRecursiveGaussianImageFilter_UseImageDirectionOn`, 'returnType') = 'void'
attr(`GradientRecursiveGaussianImageFilter_UseImageDirectionOn`, "inputTypes") = c('_p_itk__simple__GradientRecursiveGaussianImageFilter')
class(`GradientRecursiveGaussianImageFilter_UseImageDirectionOn`) = c("SWIGFunction", class('GradientRecursiveGaussianImageFilter_UseImageDirectionOn'))

# Start of GradientRecursiveGaussianImageFilter_UseImageDirectionOff

`GradientRecursiveGaussianImageFilter_UseImageDirectionOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GradientRecursiveGaussianImageFilter_UseImageDirectionOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GradientRecursiveGaussianImageFilter_UseImageDirectionOff`, 'returnType') = 'void'
attr(`GradientRecursiveGaussianImageFilter_UseImageDirectionOff`, "inputTypes") = c('_p_itk__simple__GradientRecursiveGaussianImageFilter')
class(`GradientRecursiveGaussianImageFilter_UseImageDirectionOff`) = c("SWIGFunction", class('GradientRecursiveGaussianImageFilter_UseImageDirectionOff'))

# Start of GradientRecursiveGaussianImageFilter_GetUseImageDirection

`GradientRecursiveGaussianImageFilter_GetUseImageDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientRecursiveGaussianImageFilter_GetUseImageDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientRecursiveGaussianImageFilter_GetUseImageDirection`, 'returnType') = 'logical'
attr(`GradientRecursiveGaussianImageFilter_GetUseImageDirection`, "inputTypes") = c('_p_itk__simple__GradientRecursiveGaussianImageFilter')
class(`GradientRecursiveGaussianImageFilter_GetUseImageDirection`) = c("SWIGFunction", class('GradientRecursiveGaussianImageFilter_GetUseImageDirection'))

# Start of GradientRecursiveGaussianImageFilter_GetName

`GradientRecursiveGaussianImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientRecursiveGaussianImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientRecursiveGaussianImageFilter_GetName`, 'returnType') = 'character'
attr(`GradientRecursiveGaussianImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GradientRecursiveGaussianImageFilter')
class(`GradientRecursiveGaussianImageFilter_GetName`) = c("SWIGFunction", class('GradientRecursiveGaussianImageFilter_GetName'))

# Start of GradientRecursiveGaussianImageFilter_ToString

`GradientRecursiveGaussianImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GradientRecursiveGaussianImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GradientRecursiveGaussianImageFilter_ToString`, 'returnType') = 'character'
attr(`GradientRecursiveGaussianImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GradientRecursiveGaussianImageFilter')
class(`GradientRecursiveGaussianImageFilter_ToString`) = c("SWIGFunction", class('GradientRecursiveGaussianImageFilter_ToString'))

# Start of GradientRecursiveGaussianImageFilter_Execute

`GradientRecursiveGaussianImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GradientRecursiveGaussianImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientRecursiveGaussianImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientRecursiveGaussianImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GradientRecursiveGaussianImageFilter', '_p_itk__simple__Image')
class(`GradientRecursiveGaussianImageFilter_Execute`) = c("SWIGFunction", class('GradientRecursiveGaussianImageFilter_Execute'))

# Start of accessor method for itk::simple::GradientRecursiveGaussianImageFilter
setMethod('$', '_p_itk__simple__GradientRecursiveGaussianImageFilter', function(x, name)

{
  accessorFuns = list('SetSigma' = GradientRecursiveGaussianImageFilter_SetSigma, 'GetSigma' = GradientRecursiveGaussianImageFilter_GetSigma, 'SetNormalizeAcrossScale' = GradientRecursiveGaussianImageFilter_SetNormalizeAcrossScale, 'NormalizeAcrossScaleOn' = GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOn, 'NormalizeAcrossScaleOff' = GradientRecursiveGaussianImageFilter_NormalizeAcrossScaleOff, 'GetNormalizeAcrossScale' = GradientRecursiveGaussianImageFilter_GetNormalizeAcrossScale, 'SetUseImageDirection' = GradientRecursiveGaussianImageFilter_SetUseImageDirection, 'UseImageDirectionOn' = GradientRecursiveGaussianImageFilter_UseImageDirectionOn, 'UseImageDirectionOff' = GradientRecursiveGaussianImageFilter_UseImageDirectionOff, 'GetUseImageDirection' = GradientRecursiveGaussianImageFilter_GetUseImageDirection, 'GetName' = GradientRecursiveGaussianImageFilter_GetName, 'ToString' = GradientRecursiveGaussianImageFilter_ToString, 'Execute' = GradientRecursiveGaussianImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GradientRecursiveGaussianImageFilter
# Start of GradientRecursiveGaussian

`GradientRecursiveGaussian__SWIG_0` = function(image1, sigma, normalizeAcrossScale, useImageDirection, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  normalizeAcrossScale = as.logical(normalizeAcrossScale);
  useImageDirection = as.logical(useImageDirection);
  ;ans = .Call('R_swig_GradientRecursiveGaussian__SWIG_0', image1, sigma, normalizeAcrossScale, useImageDirection, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientRecursiveGaussian__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientRecursiveGaussian__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical', 'logical')
class(`GradientRecursiveGaussian__SWIG_0`) = c("SWIGFunction", class('GradientRecursiveGaussian__SWIG_0'))

# Start of GradientRecursiveGaussian

`GradientRecursiveGaussian__SWIG_1` = function(image1, sigma, normalizeAcrossScale, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  normalizeAcrossScale = as.logical(normalizeAcrossScale);
  ;ans = .Call('R_swig_GradientRecursiveGaussian__SWIG_1', image1, sigma, normalizeAcrossScale, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientRecursiveGaussian__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientRecursiveGaussian__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`GradientRecursiveGaussian__SWIG_1`) = c("SWIGFunction", class('GradientRecursiveGaussian__SWIG_1'))

# Start of GradientRecursiveGaussian

`GradientRecursiveGaussian__SWIG_2` = function(image1, sigma, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_GradientRecursiveGaussian__SWIG_2', image1, sigma, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientRecursiveGaussian__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientRecursiveGaussian__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`GradientRecursiveGaussian__SWIG_2`) = c("SWIGFunction", class('GradientRecursiveGaussian__SWIG_2'))

# Start of GradientRecursiveGaussian

`GradientRecursiveGaussian__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GradientRecursiveGaussian__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GradientRecursiveGaussian__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`GradientRecursiveGaussian__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`GradientRecursiveGaussian__SWIG_3`) = c("SWIGFunction", class('GradientRecursiveGaussian__SWIG_3'))

`GradientRecursiveGaussian` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- GradientRecursiveGaussian__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- GradientRecursiveGaussian__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- GradientRecursiveGaussian__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- GradientRecursiveGaussian__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GradientRecursiveGaussian with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GrayscaleConnectedClosingImageFilter

`delete_GrayscaleConnectedClosingImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GrayscaleConnectedClosingImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GrayscaleConnectedClosingImageFilter`, 'returnType') = 'void'
attr(`delete_GrayscaleConnectedClosingImageFilter`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedClosingImageFilter')
class(`delete_GrayscaleConnectedClosingImageFilter`) = c("SWIGFunction", class('delete_GrayscaleConnectedClosingImageFilter'))

# Start of new_GrayscaleConnectedClosingImageFilter

`GrayscaleConnectedClosingImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GrayscaleConnectedClosingImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GrayscaleConnectedClosingImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GrayscaleConnectedClosingImageFilter);
  ans
  
}

attr(`GrayscaleConnectedClosingImageFilter`, 'returnType') = '_p_itk__simple__GrayscaleConnectedClosingImageFilter'
class(`GrayscaleConnectedClosingImageFilter`) = c("SWIGFunction", class('GrayscaleConnectedClosingImageFilter'))

# Start of GrayscaleConnectedClosingImageFilter_SetSeed

`GrayscaleConnectedClosingImageFilter_SetSeed` = function(self, Seed)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Seed = as.integer(Seed);
  ;ans = .Call('R_swig_GrayscaleConnectedClosingImageFilter_SetSeed', self, Seed, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleConnectedClosingImageFilter_SetSeed`, 'returnType') = 'void'
attr(`GrayscaleConnectedClosingImageFilter_SetSeed`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedClosingImageFilter', 'integer')
class(`GrayscaleConnectedClosingImageFilter_SetSeed`) = c("SWIGFunction", class('GrayscaleConnectedClosingImageFilter_SetSeed'))

# Start of GrayscaleConnectedClosingImageFilter_GetSeed

`GrayscaleConnectedClosingImageFilter_GetSeed` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleConnectedClosingImageFilter_GetSeed', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleConnectedClosingImageFilter_GetSeed`, 'returnType') = 'integer'
attr(`GrayscaleConnectedClosingImageFilter_GetSeed`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedClosingImageFilter')
class(`GrayscaleConnectedClosingImageFilter_GetSeed`) = c("SWIGFunction", class('GrayscaleConnectedClosingImageFilter_GetSeed'))

# Start of GrayscaleConnectedClosingImageFilter_SetFullyConnected

`GrayscaleConnectedClosingImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_GrayscaleConnectedClosingImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleConnectedClosingImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`GrayscaleConnectedClosingImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedClosingImageFilter', 'logical')
class(`GrayscaleConnectedClosingImageFilter_SetFullyConnected`) = c("SWIGFunction", class('GrayscaleConnectedClosingImageFilter_SetFullyConnected'))

# Start of GrayscaleConnectedClosingImageFilter_FullyConnectedOn

`GrayscaleConnectedClosingImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleConnectedClosingImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleConnectedClosingImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`GrayscaleConnectedClosingImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedClosingImageFilter')
class(`GrayscaleConnectedClosingImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('GrayscaleConnectedClosingImageFilter_FullyConnectedOn'))

# Start of GrayscaleConnectedClosingImageFilter_FullyConnectedOff

`GrayscaleConnectedClosingImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleConnectedClosingImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleConnectedClosingImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`GrayscaleConnectedClosingImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedClosingImageFilter')
class(`GrayscaleConnectedClosingImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('GrayscaleConnectedClosingImageFilter_FullyConnectedOff'))

# Start of GrayscaleConnectedClosingImageFilter_GetFullyConnected

`GrayscaleConnectedClosingImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleConnectedClosingImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleConnectedClosingImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`GrayscaleConnectedClosingImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedClosingImageFilter')
class(`GrayscaleConnectedClosingImageFilter_GetFullyConnected`) = c("SWIGFunction", class('GrayscaleConnectedClosingImageFilter_GetFullyConnected'))

# Start of GrayscaleConnectedClosingImageFilter_GetName

`GrayscaleConnectedClosingImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleConnectedClosingImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleConnectedClosingImageFilter_GetName`, 'returnType') = 'character'
attr(`GrayscaleConnectedClosingImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedClosingImageFilter')
class(`GrayscaleConnectedClosingImageFilter_GetName`) = c("SWIGFunction", class('GrayscaleConnectedClosingImageFilter_GetName'))

# Start of GrayscaleConnectedClosingImageFilter_ToString

`GrayscaleConnectedClosingImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleConnectedClosingImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleConnectedClosingImageFilter_ToString`, 'returnType') = 'character'
attr(`GrayscaleConnectedClosingImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedClosingImageFilter')
class(`GrayscaleConnectedClosingImageFilter_ToString`) = c("SWIGFunction", class('GrayscaleConnectedClosingImageFilter_ToString'))

# Start of GrayscaleConnectedClosingImageFilter_Execute

`GrayscaleConnectedClosingImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleConnectedClosingImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleConnectedClosingImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleConnectedClosingImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedClosingImageFilter', '_p_itk__simple__Image')
class(`GrayscaleConnectedClosingImageFilter_Execute`) = c("SWIGFunction", class('GrayscaleConnectedClosingImageFilter_Execute'))

# Start of accessor method for itk::simple::GrayscaleConnectedClosingImageFilter
setMethod('$', '_p_itk__simple__GrayscaleConnectedClosingImageFilter', function(x, name)

{
  accessorFuns = list('SetSeed' = GrayscaleConnectedClosingImageFilter_SetSeed, 'GetSeed' = GrayscaleConnectedClosingImageFilter_GetSeed, 'SetFullyConnected' = GrayscaleConnectedClosingImageFilter_SetFullyConnected, 'FullyConnectedOn' = GrayscaleConnectedClosingImageFilter_FullyConnectedOn, 'FullyConnectedOff' = GrayscaleConnectedClosingImageFilter_FullyConnectedOff, 'GetFullyConnected' = GrayscaleConnectedClosingImageFilter_GetFullyConnected, 'GetName' = GrayscaleConnectedClosingImageFilter_GetName, 'ToString' = GrayscaleConnectedClosingImageFilter_ToString, 'Execute' = GrayscaleConnectedClosingImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GrayscaleConnectedClosingImageFilter
# Start of GrayscaleConnectedClosing

`GrayscaleConnectedClosing__SWIG_0` = function(image1, seed, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed = as.integer(seed);
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_GrayscaleConnectedClosing__SWIG_0', image1, seed, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleConnectedClosing__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleConnectedClosing__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'logical')
class(`GrayscaleConnectedClosing__SWIG_0`) = c("SWIGFunction", class('GrayscaleConnectedClosing__SWIG_0'))

# Start of GrayscaleConnectedClosing

`GrayscaleConnectedClosing__SWIG_1` = function(image1, seed, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed = as.integer(seed);
  ;ans = .Call('R_swig_GrayscaleConnectedClosing__SWIG_1', image1, seed, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleConnectedClosing__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleConnectedClosing__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`GrayscaleConnectedClosing__SWIG_1`) = c("SWIGFunction", class('GrayscaleConnectedClosing__SWIG_1'))

# Start of GrayscaleConnectedClosing

`GrayscaleConnectedClosing__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleConnectedClosing__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleConnectedClosing__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleConnectedClosing__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`GrayscaleConnectedClosing__SWIG_2`) = c("SWIGFunction", class('GrayscaleConnectedClosing__SWIG_2'))

`GrayscaleConnectedClosing` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- GrayscaleConnectedClosing__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- GrayscaleConnectedClosing__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- GrayscaleConnectedClosing__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GrayscaleConnectedClosing with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GrayscaleConnectedOpeningImageFilter

`delete_GrayscaleConnectedOpeningImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GrayscaleConnectedOpeningImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GrayscaleConnectedOpeningImageFilter`, 'returnType') = 'void'
attr(`delete_GrayscaleConnectedOpeningImageFilter`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedOpeningImageFilter')
class(`delete_GrayscaleConnectedOpeningImageFilter`) = c("SWIGFunction", class('delete_GrayscaleConnectedOpeningImageFilter'))

# Start of new_GrayscaleConnectedOpeningImageFilter

`GrayscaleConnectedOpeningImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GrayscaleConnectedOpeningImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GrayscaleConnectedOpeningImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GrayscaleConnectedOpeningImageFilter);
  ans
  
}

attr(`GrayscaleConnectedOpeningImageFilter`, 'returnType') = '_p_itk__simple__GrayscaleConnectedOpeningImageFilter'
class(`GrayscaleConnectedOpeningImageFilter`) = c("SWIGFunction", class('GrayscaleConnectedOpeningImageFilter'))

# Start of GrayscaleConnectedOpeningImageFilter_SetSeed

`GrayscaleConnectedOpeningImageFilter_SetSeed` = function(self, Seed)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Seed = as.integer(Seed);
  ;ans = .Call('R_swig_GrayscaleConnectedOpeningImageFilter_SetSeed', self, Seed, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleConnectedOpeningImageFilter_SetSeed`, 'returnType') = 'void'
attr(`GrayscaleConnectedOpeningImageFilter_SetSeed`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedOpeningImageFilter', 'integer')
class(`GrayscaleConnectedOpeningImageFilter_SetSeed`) = c("SWIGFunction", class('GrayscaleConnectedOpeningImageFilter_SetSeed'))

# Start of GrayscaleConnectedOpeningImageFilter_GetSeed

`GrayscaleConnectedOpeningImageFilter_GetSeed` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleConnectedOpeningImageFilter_GetSeed', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleConnectedOpeningImageFilter_GetSeed`, 'returnType') = 'integer'
attr(`GrayscaleConnectedOpeningImageFilter_GetSeed`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedOpeningImageFilter')
class(`GrayscaleConnectedOpeningImageFilter_GetSeed`) = c("SWIGFunction", class('GrayscaleConnectedOpeningImageFilter_GetSeed'))

# Start of GrayscaleConnectedOpeningImageFilter_SetFullyConnected

`GrayscaleConnectedOpeningImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_GrayscaleConnectedOpeningImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleConnectedOpeningImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`GrayscaleConnectedOpeningImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedOpeningImageFilter', 'logical')
class(`GrayscaleConnectedOpeningImageFilter_SetFullyConnected`) = c("SWIGFunction", class('GrayscaleConnectedOpeningImageFilter_SetFullyConnected'))

# Start of GrayscaleConnectedOpeningImageFilter_FullyConnectedOn

`GrayscaleConnectedOpeningImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleConnectedOpeningImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleConnectedOpeningImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`GrayscaleConnectedOpeningImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedOpeningImageFilter')
class(`GrayscaleConnectedOpeningImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('GrayscaleConnectedOpeningImageFilter_FullyConnectedOn'))

# Start of GrayscaleConnectedOpeningImageFilter_FullyConnectedOff

`GrayscaleConnectedOpeningImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleConnectedOpeningImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleConnectedOpeningImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`GrayscaleConnectedOpeningImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedOpeningImageFilter')
class(`GrayscaleConnectedOpeningImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('GrayscaleConnectedOpeningImageFilter_FullyConnectedOff'))

# Start of GrayscaleConnectedOpeningImageFilter_GetFullyConnected

`GrayscaleConnectedOpeningImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleConnectedOpeningImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleConnectedOpeningImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`GrayscaleConnectedOpeningImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedOpeningImageFilter')
class(`GrayscaleConnectedOpeningImageFilter_GetFullyConnected`) = c("SWIGFunction", class('GrayscaleConnectedOpeningImageFilter_GetFullyConnected'))

# Start of GrayscaleConnectedOpeningImageFilter_GetName

`GrayscaleConnectedOpeningImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleConnectedOpeningImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleConnectedOpeningImageFilter_GetName`, 'returnType') = 'character'
attr(`GrayscaleConnectedOpeningImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedOpeningImageFilter')
class(`GrayscaleConnectedOpeningImageFilter_GetName`) = c("SWIGFunction", class('GrayscaleConnectedOpeningImageFilter_GetName'))

# Start of GrayscaleConnectedOpeningImageFilter_ToString

`GrayscaleConnectedOpeningImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleConnectedOpeningImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleConnectedOpeningImageFilter_ToString`, 'returnType') = 'character'
attr(`GrayscaleConnectedOpeningImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedOpeningImageFilter')
class(`GrayscaleConnectedOpeningImageFilter_ToString`) = c("SWIGFunction", class('GrayscaleConnectedOpeningImageFilter_ToString'))

# Start of GrayscaleConnectedOpeningImageFilter_Execute

`GrayscaleConnectedOpeningImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleConnectedOpeningImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleConnectedOpeningImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleConnectedOpeningImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GrayscaleConnectedOpeningImageFilter', '_p_itk__simple__Image')
class(`GrayscaleConnectedOpeningImageFilter_Execute`) = c("SWIGFunction", class('GrayscaleConnectedOpeningImageFilter_Execute'))

# Start of accessor method for itk::simple::GrayscaleConnectedOpeningImageFilter
setMethod('$', '_p_itk__simple__GrayscaleConnectedOpeningImageFilter', function(x, name)

{
  accessorFuns = list('SetSeed' = GrayscaleConnectedOpeningImageFilter_SetSeed, 'GetSeed' = GrayscaleConnectedOpeningImageFilter_GetSeed, 'SetFullyConnected' = GrayscaleConnectedOpeningImageFilter_SetFullyConnected, 'FullyConnectedOn' = GrayscaleConnectedOpeningImageFilter_FullyConnectedOn, 'FullyConnectedOff' = GrayscaleConnectedOpeningImageFilter_FullyConnectedOff, 'GetFullyConnected' = GrayscaleConnectedOpeningImageFilter_GetFullyConnected, 'GetName' = GrayscaleConnectedOpeningImageFilter_GetName, 'ToString' = GrayscaleConnectedOpeningImageFilter_ToString, 'Execute' = GrayscaleConnectedOpeningImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GrayscaleConnectedOpeningImageFilter
# Start of GrayscaleConnectedOpening

`GrayscaleConnectedOpening__SWIG_0` = function(image1, seed, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed = as.integer(seed);
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_GrayscaleConnectedOpening__SWIG_0', image1, seed, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleConnectedOpening__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleConnectedOpening__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'logical')
class(`GrayscaleConnectedOpening__SWIG_0`) = c("SWIGFunction", class('GrayscaleConnectedOpening__SWIG_0'))

# Start of GrayscaleConnectedOpening

`GrayscaleConnectedOpening__SWIG_1` = function(image1, seed, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed = as.integer(seed);
  ;ans = .Call('R_swig_GrayscaleConnectedOpening__SWIG_1', image1, seed, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleConnectedOpening__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleConnectedOpening__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`GrayscaleConnectedOpening__SWIG_1`) = c("SWIGFunction", class('GrayscaleConnectedOpening__SWIG_1'))

# Start of GrayscaleConnectedOpening

`GrayscaleConnectedOpening__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleConnectedOpening__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleConnectedOpening__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleConnectedOpening__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`GrayscaleConnectedOpening__SWIG_2`) = c("SWIGFunction", class('GrayscaleConnectedOpening__SWIG_2'))

`GrayscaleConnectedOpening` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- GrayscaleConnectedOpening__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- GrayscaleConnectedOpening__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- GrayscaleConnectedOpening__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GrayscaleConnectedOpening with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GrayscaleDilateImageFilter

`delete_GrayscaleDilateImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GrayscaleDilateImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GrayscaleDilateImageFilter`, 'returnType') = 'void'
attr(`delete_GrayscaleDilateImageFilter`, "inputTypes") = c('_p_itk__simple__GrayscaleDilateImageFilter')
class(`delete_GrayscaleDilateImageFilter`) = c("SWIGFunction", class('delete_GrayscaleDilateImageFilter'))

# Start of new_GrayscaleDilateImageFilter

`GrayscaleDilateImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GrayscaleDilateImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GrayscaleDilateImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GrayscaleDilateImageFilter);
  ans
  
}

attr(`GrayscaleDilateImageFilter`, 'returnType') = '_p_itk__simple__GrayscaleDilateImageFilter'
class(`GrayscaleDilateImageFilter`) = c("SWIGFunction", class('GrayscaleDilateImageFilter'))

# Start of GrayscaleDilateImageFilter_SetKernelRadius

`GrayscaleDilateImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_GrayscaleDilateImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleDilateImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`GrayscaleDilateImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__GrayscaleDilateImageFilter', 'integer')
class(`GrayscaleDilateImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('GrayscaleDilateImageFilter_SetKernelRadius__SWIG_0'))

# Start of GrayscaleDilateImageFilter_SetKernelRadius

`GrayscaleDilateImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_GrayscaleDilateImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleDilateImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`GrayscaleDilateImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__GrayscaleDilateImageFilter', 'integer')
class(`GrayscaleDilateImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('GrayscaleDilateImageFilter_SetKernelRadius__SWIG_1'))

`GrayscaleDilateImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__GrayscaleDilateImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- GrayscaleDilateImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__GrayscaleDilateImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- GrayscaleDilateImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GrayscaleDilateImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of GrayscaleDilateImageFilter_GetKernelRadius

`GrayscaleDilateImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleDilateImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleDilateImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`GrayscaleDilateImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__GrayscaleDilateImageFilter')
class(`GrayscaleDilateImageFilter_GetKernelRadius`) = c("SWIGFunction", class('GrayscaleDilateImageFilter_GetKernelRadius'))

# Start of GrayscaleDilateImageFilter_SetKernelType

`GrayscaleDilateImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_GrayscaleDilateImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleDilateImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`GrayscaleDilateImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__GrayscaleDilateImageFilter', 'character')
class(`GrayscaleDilateImageFilter_SetKernelType`) = c("SWIGFunction", class('GrayscaleDilateImageFilter_SetKernelType'))

# Start of GrayscaleDilateImageFilter_GetKernelType

`GrayscaleDilateImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleDilateImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`GrayscaleDilateImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`GrayscaleDilateImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__GrayscaleDilateImageFilter')
class(`GrayscaleDilateImageFilter_GetKernelType`) = c("SWIGFunction", class('GrayscaleDilateImageFilter_GetKernelType'))

# Start of GrayscaleDilateImageFilter_GetName

`GrayscaleDilateImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleDilateImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleDilateImageFilter_GetName`, 'returnType') = 'character'
attr(`GrayscaleDilateImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GrayscaleDilateImageFilter')
class(`GrayscaleDilateImageFilter_GetName`) = c("SWIGFunction", class('GrayscaleDilateImageFilter_GetName'))

# Start of GrayscaleDilateImageFilter_ToString

`GrayscaleDilateImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleDilateImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleDilateImageFilter_ToString`, 'returnType') = 'character'
attr(`GrayscaleDilateImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GrayscaleDilateImageFilter')
class(`GrayscaleDilateImageFilter_ToString`) = c("SWIGFunction", class('GrayscaleDilateImageFilter_ToString'))

# Start of GrayscaleDilateImageFilter_Execute

`GrayscaleDilateImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleDilateImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleDilateImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleDilateImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GrayscaleDilateImageFilter', '_p_itk__simple__Image')
class(`GrayscaleDilateImageFilter_Execute`) = c("SWIGFunction", class('GrayscaleDilateImageFilter_Execute'))

# Start of accessor method for itk::simple::GrayscaleDilateImageFilter
setMethod('$', '_p_itk__simple__GrayscaleDilateImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = GrayscaleDilateImageFilter_SetKernelRadius, 'GetKernelRadius' = GrayscaleDilateImageFilter_GetKernelRadius, 'SetKernelType' = GrayscaleDilateImageFilter_SetKernelType, 'GetKernelType' = GrayscaleDilateImageFilter_GetKernelType, 'GetName' = GrayscaleDilateImageFilter_GetName, 'ToString' = GrayscaleDilateImageFilter_ToString, 'Execute' = GrayscaleDilateImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GrayscaleDilateImageFilter
# Start of GrayscaleDilate

`GrayscaleDilate__SWIG_0` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_GrayscaleDilate__SWIG_0', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleDilate__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleDilate__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`GrayscaleDilate__SWIG_0`) = c("SWIGFunction", class('GrayscaleDilate__SWIG_0'))

# Start of GrayscaleDilate

`GrayscaleDilate__SWIG_1` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_GrayscaleDilate__SWIG_1', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleDilate__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleDilate__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`GrayscaleDilate__SWIG_1`) = c("SWIGFunction", class('GrayscaleDilate__SWIG_1'))

# Start of GrayscaleDilate

`GrayscaleDilate__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleDilate__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleDilate__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleDilate__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`GrayscaleDilate__SWIG_2`) = c("SWIGFunction", class('GrayscaleDilate__SWIG_2'))

`GrayscaleDilate` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- GrayscaleDilate__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- GrayscaleDilate__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- GrayscaleDilate__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GrayscaleDilate with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GrayscaleErodeImageFilter

`delete_GrayscaleErodeImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GrayscaleErodeImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GrayscaleErodeImageFilter`, 'returnType') = 'void'
attr(`delete_GrayscaleErodeImageFilter`, "inputTypes") = c('_p_itk__simple__GrayscaleErodeImageFilter')
class(`delete_GrayscaleErodeImageFilter`) = c("SWIGFunction", class('delete_GrayscaleErodeImageFilter'))

# Start of new_GrayscaleErodeImageFilter

`GrayscaleErodeImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GrayscaleErodeImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GrayscaleErodeImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GrayscaleErodeImageFilter);
  ans
  
}

attr(`GrayscaleErodeImageFilter`, 'returnType') = '_p_itk__simple__GrayscaleErodeImageFilter'
class(`GrayscaleErodeImageFilter`) = c("SWIGFunction", class('GrayscaleErodeImageFilter'))

# Start of GrayscaleErodeImageFilter_SetKernelRadius

`GrayscaleErodeImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_GrayscaleErodeImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleErodeImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`GrayscaleErodeImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__GrayscaleErodeImageFilter', 'integer')
class(`GrayscaleErodeImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('GrayscaleErodeImageFilter_SetKernelRadius__SWIG_0'))

# Start of GrayscaleErodeImageFilter_SetKernelRadius

`GrayscaleErodeImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_GrayscaleErodeImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleErodeImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`GrayscaleErodeImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__GrayscaleErodeImageFilter', 'integer')
class(`GrayscaleErodeImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('GrayscaleErodeImageFilter_SetKernelRadius__SWIG_1'))

`GrayscaleErodeImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__GrayscaleErodeImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- GrayscaleErodeImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__GrayscaleErodeImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- GrayscaleErodeImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GrayscaleErodeImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of GrayscaleErodeImageFilter_GetKernelRadius

`GrayscaleErodeImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleErodeImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleErodeImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`GrayscaleErodeImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__GrayscaleErodeImageFilter')
class(`GrayscaleErodeImageFilter_GetKernelRadius`) = c("SWIGFunction", class('GrayscaleErodeImageFilter_GetKernelRadius'))

# Start of GrayscaleErodeImageFilter_SetKernelType

`GrayscaleErodeImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_GrayscaleErodeImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleErodeImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`GrayscaleErodeImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__GrayscaleErodeImageFilter', 'character')
class(`GrayscaleErodeImageFilter_SetKernelType`) = c("SWIGFunction", class('GrayscaleErodeImageFilter_SetKernelType'))

# Start of GrayscaleErodeImageFilter_GetKernelType

`GrayscaleErodeImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleErodeImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`GrayscaleErodeImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`GrayscaleErodeImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__GrayscaleErodeImageFilter')
class(`GrayscaleErodeImageFilter_GetKernelType`) = c("SWIGFunction", class('GrayscaleErodeImageFilter_GetKernelType'))

# Start of GrayscaleErodeImageFilter_GetName

`GrayscaleErodeImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleErodeImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleErodeImageFilter_GetName`, 'returnType') = 'character'
attr(`GrayscaleErodeImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GrayscaleErodeImageFilter')
class(`GrayscaleErodeImageFilter_GetName`) = c("SWIGFunction", class('GrayscaleErodeImageFilter_GetName'))

# Start of GrayscaleErodeImageFilter_ToString

`GrayscaleErodeImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleErodeImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleErodeImageFilter_ToString`, 'returnType') = 'character'
attr(`GrayscaleErodeImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GrayscaleErodeImageFilter')
class(`GrayscaleErodeImageFilter_ToString`) = c("SWIGFunction", class('GrayscaleErodeImageFilter_ToString'))

# Start of GrayscaleErodeImageFilter_Execute

`GrayscaleErodeImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleErodeImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleErodeImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleErodeImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GrayscaleErodeImageFilter', '_p_itk__simple__Image')
class(`GrayscaleErodeImageFilter_Execute`) = c("SWIGFunction", class('GrayscaleErodeImageFilter_Execute'))

# Start of accessor method for itk::simple::GrayscaleErodeImageFilter
setMethod('$', '_p_itk__simple__GrayscaleErodeImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = GrayscaleErodeImageFilter_SetKernelRadius, 'GetKernelRadius' = GrayscaleErodeImageFilter_GetKernelRadius, 'SetKernelType' = GrayscaleErodeImageFilter_SetKernelType, 'GetKernelType' = GrayscaleErodeImageFilter_GetKernelType, 'GetName' = GrayscaleErodeImageFilter_GetName, 'ToString' = GrayscaleErodeImageFilter_ToString, 'Execute' = GrayscaleErodeImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GrayscaleErodeImageFilter
# Start of GrayscaleErode

`GrayscaleErode__SWIG_0` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_GrayscaleErode__SWIG_0', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleErode__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleErode__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`GrayscaleErode__SWIG_0`) = c("SWIGFunction", class('GrayscaleErode__SWIG_0'))

# Start of GrayscaleErode

`GrayscaleErode__SWIG_1` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_GrayscaleErode__SWIG_1', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleErode__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleErode__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`GrayscaleErode__SWIG_1`) = c("SWIGFunction", class('GrayscaleErode__SWIG_1'))

# Start of GrayscaleErode

`GrayscaleErode__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleErode__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleErode__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleErode__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`GrayscaleErode__SWIG_2`) = c("SWIGFunction", class('GrayscaleErode__SWIG_2'))

`GrayscaleErode` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- GrayscaleErode__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- GrayscaleErode__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- GrayscaleErode__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GrayscaleErode with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GrayscaleFillholeImageFilter

`delete_GrayscaleFillholeImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GrayscaleFillholeImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GrayscaleFillholeImageFilter`, 'returnType') = 'void'
attr(`delete_GrayscaleFillholeImageFilter`, "inputTypes") = c('_p_itk__simple__GrayscaleFillholeImageFilter')
class(`delete_GrayscaleFillholeImageFilter`) = c("SWIGFunction", class('delete_GrayscaleFillholeImageFilter'))

# Start of new_GrayscaleFillholeImageFilter

`GrayscaleFillholeImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GrayscaleFillholeImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GrayscaleFillholeImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GrayscaleFillholeImageFilter);
  ans
  
}

attr(`GrayscaleFillholeImageFilter`, 'returnType') = '_p_itk__simple__GrayscaleFillholeImageFilter'
class(`GrayscaleFillholeImageFilter`) = c("SWIGFunction", class('GrayscaleFillholeImageFilter'))

# Start of GrayscaleFillholeImageFilter_SetFullyConnected

`GrayscaleFillholeImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_GrayscaleFillholeImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleFillholeImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`GrayscaleFillholeImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__GrayscaleFillholeImageFilter', 'logical')
class(`GrayscaleFillholeImageFilter_SetFullyConnected`) = c("SWIGFunction", class('GrayscaleFillholeImageFilter_SetFullyConnected'))

# Start of GrayscaleFillholeImageFilter_FullyConnectedOn

`GrayscaleFillholeImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleFillholeImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleFillholeImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`GrayscaleFillholeImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__GrayscaleFillholeImageFilter')
class(`GrayscaleFillholeImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('GrayscaleFillholeImageFilter_FullyConnectedOn'))

# Start of GrayscaleFillholeImageFilter_FullyConnectedOff

`GrayscaleFillholeImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleFillholeImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleFillholeImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`GrayscaleFillholeImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__GrayscaleFillholeImageFilter')
class(`GrayscaleFillholeImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('GrayscaleFillholeImageFilter_FullyConnectedOff'))

# Start of GrayscaleFillholeImageFilter_GetFullyConnected

`GrayscaleFillholeImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleFillholeImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleFillholeImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`GrayscaleFillholeImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__GrayscaleFillholeImageFilter')
class(`GrayscaleFillholeImageFilter_GetFullyConnected`) = c("SWIGFunction", class('GrayscaleFillholeImageFilter_GetFullyConnected'))

# Start of GrayscaleFillholeImageFilter_GetName

`GrayscaleFillholeImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleFillholeImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleFillholeImageFilter_GetName`, 'returnType') = 'character'
attr(`GrayscaleFillholeImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GrayscaleFillholeImageFilter')
class(`GrayscaleFillholeImageFilter_GetName`) = c("SWIGFunction", class('GrayscaleFillholeImageFilter_GetName'))

# Start of GrayscaleFillholeImageFilter_ToString

`GrayscaleFillholeImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleFillholeImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleFillholeImageFilter_ToString`, 'returnType') = 'character'
attr(`GrayscaleFillholeImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GrayscaleFillholeImageFilter')
class(`GrayscaleFillholeImageFilter_ToString`) = c("SWIGFunction", class('GrayscaleFillholeImageFilter_ToString'))

# Start of GrayscaleFillholeImageFilter_Execute

`GrayscaleFillholeImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleFillholeImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleFillholeImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleFillholeImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GrayscaleFillholeImageFilter', '_p_itk__simple__Image')
class(`GrayscaleFillholeImageFilter_Execute`) = c("SWIGFunction", class('GrayscaleFillholeImageFilter_Execute'))

# Start of accessor method for itk::simple::GrayscaleFillholeImageFilter
setMethod('$', '_p_itk__simple__GrayscaleFillholeImageFilter', function(x, name)

{
  accessorFuns = list('SetFullyConnected' = GrayscaleFillholeImageFilter_SetFullyConnected, 'FullyConnectedOn' = GrayscaleFillholeImageFilter_FullyConnectedOn, 'FullyConnectedOff' = GrayscaleFillholeImageFilter_FullyConnectedOff, 'GetFullyConnected' = GrayscaleFillholeImageFilter_GetFullyConnected, 'GetName' = GrayscaleFillholeImageFilter_GetName, 'ToString' = GrayscaleFillholeImageFilter_ToString, 'Execute' = GrayscaleFillholeImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GrayscaleFillholeImageFilter
# Start of GrayscaleFillhole

`GrayscaleFillhole__SWIG_0` = function(image1, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_GrayscaleFillhole__SWIG_0', image1, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleFillhole__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleFillhole__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`GrayscaleFillhole__SWIG_0`) = c("SWIGFunction", class('GrayscaleFillhole__SWIG_0'))

# Start of GrayscaleFillhole

`GrayscaleFillhole__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleFillhole__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleFillhole__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleFillhole__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`GrayscaleFillhole__SWIG_1`) = c("SWIGFunction", class('GrayscaleFillhole__SWIG_1'))

`GrayscaleFillhole` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- GrayscaleFillhole__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- GrayscaleFillhole__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GrayscaleFillhole with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GrayscaleGeodesicDilateImageFilter

`delete_GrayscaleGeodesicDilateImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GrayscaleGeodesicDilateImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GrayscaleGeodesicDilateImageFilter`, 'returnType') = 'void'
attr(`delete_GrayscaleGeodesicDilateImageFilter`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicDilateImageFilter')
class(`delete_GrayscaleGeodesicDilateImageFilter`) = c("SWIGFunction", class('delete_GrayscaleGeodesicDilateImageFilter'))

# Start of new_GrayscaleGeodesicDilateImageFilter

`GrayscaleGeodesicDilateImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GrayscaleGeodesicDilateImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GrayscaleGeodesicDilateImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GrayscaleGeodesicDilateImageFilter);
  ans
  
}

attr(`GrayscaleGeodesicDilateImageFilter`, 'returnType') = '_p_itk__simple__GrayscaleGeodesicDilateImageFilter'
class(`GrayscaleGeodesicDilateImageFilter`) = c("SWIGFunction", class('GrayscaleGeodesicDilateImageFilter'))

# Start of GrayscaleGeodesicDilateImageFilter_SetRunOneIteration

`GrayscaleGeodesicDilateImageFilter_SetRunOneIteration` = function(self, RunOneIteration)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  RunOneIteration = as.logical(RunOneIteration);
  ;ans = .Call('R_swig_GrayscaleGeodesicDilateImageFilter_SetRunOneIteration', self, RunOneIteration, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicDilateImageFilter_SetRunOneIteration`, 'returnType') = 'void'
attr(`GrayscaleGeodesicDilateImageFilter_SetRunOneIteration`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicDilateImageFilter', 'logical')
class(`GrayscaleGeodesicDilateImageFilter_SetRunOneIteration`) = c("SWIGFunction", class('GrayscaleGeodesicDilateImageFilter_SetRunOneIteration'))

# Start of GrayscaleGeodesicDilateImageFilter_RunOneIterationOn

`GrayscaleGeodesicDilateImageFilter_RunOneIterationOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGeodesicDilateImageFilter_RunOneIterationOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicDilateImageFilter_RunOneIterationOn`, 'returnType') = 'void'
attr(`GrayscaleGeodesicDilateImageFilter_RunOneIterationOn`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicDilateImageFilter')
class(`GrayscaleGeodesicDilateImageFilter_RunOneIterationOn`) = c("SWIGFunction", class('GrayscaleGeodesicDilateImageFilter_RunOneIterationOn'))

# Start of GrayscaleGeodesicDilateImageFilter_RunOneIterationOff

`GrayscaleGeodesicDilateImageFilter_RunOneIterationOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGeodesicDilateImageFilter_RunOneIterationOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicDilateImageFilter_RunOneIterationOff`, 'returnType') = 'void'
attr(`GrayscaleGeodesicDilateImageFilter_RunOneIterationOff`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicDilateImageFilter')
class(`GrayscaleGeodesicDilateImageFilter_RunOneIterationOff`) = c("SWIGFunction", class('GrayscaleGeodesicDilateImageFilter_RunOneIterationOff'))

# Start of GrayscaleGeodesicDilateImageFilter_GetRunOneIteration

`GrayscaleGeodesicDilateImageFilter_GetRunOneIteration` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleGeodesicDilateImageFilter_GetRunOneIteration', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleGeodesicDilateImageFilter_GetRunOneIteration`, 'returnType') = 'logical'
attr(`GrayscaleGeodesicDilateImageFilter_GetRunOneIteration`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicDilateImageFilter')
class(`GrayscaleGeodesicDilateImageFilter_GetRunOneIteration`) = c("SWIGFunction", class('GrayscaleGeodesicDilateImageFilter_GetRunOneIteration'))

# Start of GrayscaleGeodesicDilateImageFilter_SetFullyConnected

`GrayscaleGeodesicDilateImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_GrayscaleGeodesicDilateImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicDilateImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`GrayscaleGeodesicDilateImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicDilateImageFilter', 'logical')
class(`GrayscaleGeodesicDilateImageFilter_SetFullyConnected`) = c("SWIGFunction", class('GrayscaleGeodesicDilateImageFilter_SetFullyConnected'))

# Start of GrayscaleGeodesicDilateImageFilter_FullyConnectedOn

`GrayscaleGeodesicDilateImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGeodesicDilateImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicDilateImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`GrayscaleGeodesicDilateImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicDilateImageFilter')
class(`GrayscaleGeodesicDilateImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('GrayscaleGeodesicDilateImageFilter_FullyConnectedOn'))

# Start of GrayscaleGeodesicDilateImageFilter_FullyConnectedOff

`GrayscaleGeodesicDilateImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGeodesicDilateImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicDilateImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`GrayscaleGeodesicDilateImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicDilateImageFilter')
class(`GrayscaleGeodesicDilateImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('GrayscaleGeodesicDilateImageFilter_FullyConnectedOff'))

# Start of GrayscaleGeodesicDilateImageFilter_GetFullyConnected

`GrayscaleGeodesicDilateImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleGeodesicDilateImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleGeodesicDilateImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`GrayscaleGeodesicDilateImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicDilateImageFilter')
class(`GrayscaleGeodesicDilateImageFilter_GetFullyConnected`) = c("SWIGFunction", class('GrayscaleGeodesicDilateImageFilter_GetFullyConnected'))

# Start of GrayscaleGeodesicDilateImageFilter_GetName

`GrayscaleGeodesicDilateImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleGeodesicDilateImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleGeodesicDilateImageFilter_GetName`, 'returnType') = 'character'
attr(`GrayscaleGeodesicDilateImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicDilateImageFilter')
class(`GrayscaleGeodesicDilateImageFilter_GetName`) = c("SWIGFunction", class('GrayscaleGeodesicDilateImageFilter_GetName'))

# Start of GrayscaleGeodesicDilateImageFilter_ToString

`GrayscaleGeodesicDilateImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleGeodesicDilateImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleGeodesicDilateImageFilter_ToString`, 'returnType') = 'character'
attr(`GrayscaleGeodesicDilateImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicDilateImageFilter')
class(`GrayscaleGeodesicDilateImageFilter_ToString`) = c("SWIGFunction", class('GrayscaleGeodesicDilateImageFilter_ToString'))

# Start of GrayscaleGeodesicDilateImageFilter_Execute

`GrayscaleGeodesicDilateImageFilter_Execute` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGeodesicDilateImageFilter_Execute', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicDilateImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleGeodesicDilateImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicDilateImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`GrayscaleGeodesicDilateImageFilter_Execute`) = c("SWIGFunction", class('GrayscaleGeodesicDilateImageFilter_Execute'))

# Start of accessor method for itk::simple::GrayscaleGeodesicDilateImageFilter
setMethod('$', '_p_itk__simple__GrayscaleGeodesicDilateImageFilter', function(x, name)

{
  accessorFuns = list('SetRunOneIteration' = GrayscaleGeodesicDilateImageFilter_SetRunOneIteration, 'RunOneIterationOn' = GrayscaleGeodesicDilateImageFilter_RunOneIterationOn, 'RunOneIterationOff' = GrayscaleGeodesicDilateImageFilter_RunOneIterationOff, 'GetRunOneIteration' = GrayscaleGeodesicDilateImageFilter_GetRunOneIteration, 'SetFullyConnected' = GrayscaleGeodesicDilateImageFilter_SetFullyConnected, 'FullyConnectedOn' = GrayscaleGeodesicDilateImageFilter_FullyConnectedOn, 'FullyConnectedOff' = GrayscaleGeodesicDilateImageFilter_FullyConnectedOff, 'GetFullyConnected' = GrayscaleGeodesicDilateImageFilter_GetFullyConnected, 'GetName' = GrayscaleGeodesicDilateImageFilter_GetName, 'ToString' = GrayscaleGeodesicDilateImageFilter_ToString, 'Execute' = GrayscaleGeodesicDilateImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GrayscaleGeodesicDilateImageFilter
# Start of GrayscaleGeodesicDilate

`GrayscaleGeodesicDilate__SWIG_0` = function(image1, image2, runOneIteration, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  runOneIteration = as.logical(runOneIteration);
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_GrayscaleGeodesicDilate__SWIG_0', image1, image2, runOneIteration, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicDilate__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleGeodesicDilate__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical', 'logical')
class(`GrayscaleGeodesicDilate__SWIG_0`) = c("SWIGFunction", class('GrayscaleGeodesicDilate__SWIG_0'))

# Start of GrayscaleGeodesicDilate

`GrayscaleGeodesicDilate__SWIG_1` = function(image1, image2, runOneIteration, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  runOneIteration = as.logical(runOneIteration);
  ;ans = .Call('R_swig_GrayscaleGeodesicDilate__SWIG_1', image1, image2, runOneIteration, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicDilate__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleGeodesicDilate__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical')
class(`GrayscaleGeodesicDilate__SWIG_1`) = c("SWIGFunction", class('GrayscaleGeodesicDilate__SWIG_1'))

# Start of GrayscaleGeodesicDilate

`GrayscaleGeodesicDilate__SWIG_2` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGeodesicDilate__SWIG_2', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicDilate__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleGeodesicDilate__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`GrayscaleGeodesicDilate__SWIG_2`) = c("SWIGFunction", class('GrayscaleGeodesicDilate__SWIG_2'))

`GrayscaleGeodesicDilate` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- GrayscaleGeodesicDilate__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- GrayscaleGeodesicDilate__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- GrayscaleGeodesicDilate__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GrayscaleGeodesicDilate with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GrayscaleGeodesicErodeImageFilter

`delete_GrayscaleGeodesicErodeImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GrayscaleGeodesicErodeImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GrayscaleGeodesicErodeImageFilter`, 'returnType') = 'void'
attr(`delete_GrayscaleGeodesicErodeImageFilter`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicErodeImageFilter')
class(`delete_GrayscaleGeodesicErodeImageFilter`) = c("SWIGFunction", class('delete_GrayscaleGeodesicErodeImageFilter'))

# Start of new_GrayscaleGeodesicErodeImageFilter

`GrayscaleGeodesicErodeImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GrayscaleGeodesicErodeImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GrayscaleGeodesicErodeImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GrayscaleGeodesicErodeImageFilter);
  ans
  
}

attr(`GrayscaleGeodesicErodeImageFilter`, 'returnType') = '_p_itk__simple__GrayscaleGeodesicErodeImageFilter'
class(`GrayscaleGeodesicErodeImageFilter`) = c("SWIGFunction", class('GrayscaleGeodesicErodeImageFilter'))

# Start of GrayscaleGeodesicErodeImageFilter_SetRunOneIteration

`GrayscaleGeodesicErodeImageFilter_SetRunOneIteration` = function(self, RunOneIteration)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  RunOneIteration = as.logical(RunOneIteration);
  ;ans = .Call('R_swig_GrayscaleGeodesicErodeImageFilter_SetRunOneIteration', self, RunOneIteration, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicErodeImageFilter_SetRunOneIteration`, 'returnType') = 'void'
attr(`GrayscaleGeodesicErodeImageFilter_SetRunOneIteration`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicErodeImageFilter', 'logical')
class(`GrayscaleGeodesicErodeImageFilter_SetRunOneIteration`) = c("SWIGFunction", class('GrayscaleGeodesicErodeImageFilter_SetRunOneIteration'))

# Start of GrayscaleGeodesicErodeImageFilter_RunOneIterationOn

`GrayscaleGeodesicErodeImageFilter_RunOneIterationOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGeodesicErodeImageFilter_RunOneIterationOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicErodeImageFilter_RunOneIterationOn`, 'returnType') = 'void'
attr(`GrayscaleGeodesicErodeImageFilter_RunOneIterationOn`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicErodeImageFilter')
class(`GrayscaleGeodesicErodeImageFilter_RunOneIterationOn`) = c("SWIGFunction", class('GrayscaleGeodesicErodeImageFilter_RunOneIterationOn'))

# Start of GrayscaleGeodesicErodeImageFilter_RunOneIterationOff

`GrayscaleGeodesicErodeImageFilter_RunOneIterationOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGeodesicErodeImageFilter_RunOneIterationOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicErodeImageFilter_RunOneIterationOff`, 'returnType') = 'void'
attr(`GrayscaleGeodesicErodeImageFilter_RunOneIterationOff`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicErodeImageFilter')
class(`GrayscaleGeodesicErodeImageFilter_RunOneIterationOff`) = c("SWIGFunction", class('GrayscaleGeodesicErodeImageFilter_RunOneIterationOff'))

# Start of GrayscaleGeodesicErodeImageFilter_GetRunOneIteration

`GrayscaleGeodesicErodeImageFilter_GetRunOneIteration` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleGeodesicErodeImageFilter_GetRunOneIteration', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleGeodesicErodeImageFilter_GetRunOneIteration`, 'returnType') = 'logical'
attr(`GrayscaleGeodesicErodeImageFilter_GetRunOneIteration`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicErodeImageFilter')
class(`GrayscaleGeodesicErodeImageFilter_GetRunOneIteration`) = c("SWIGFunction", class('GrayscaleGeodesicErodeImageFilter_GetRunOneIteration'))

# Start of GrayscaleGeodesicErodeImageFilter_SetFullyConnected

`GrayscaleGeodesicErodeImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_GrayscaleGeodesicErodeImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicErodeImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`GrayscaleGeodesicErodeImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicErodeImageFilter', 'logical')
class(`GrayscaleGeodesicErodeImageFilter_SetFullyConnected`) = c("SWIGFunction", class('GrayscaleGeodesicErodeImageFilter_SetFullyConnected'))

# Start of GrayscaleGeodesicErodeImageFilter_FullyConnectedOn

`GrayscaleGeodesicErodeImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGeodesicErodeImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicErodeImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`GrayscaleGeodesicErodeImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicErodeImageFilter')
class(`GrayscaleGeodesicErodeImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('GrayscaleGeodesicErodeImageFilter_FullyConnectedOn'))

# Start of GrayscaleGeodesicErodeImageFilter_FullyConnectedOff

`GrayscaleGeodesicErodeImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGeodesicErodeImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicErodeImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`GrayscaleGeodesicErodeImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicErodeImageFilter')
class(`GrayscaleGeodesicErodeImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('GrayscaleGeodesicErodeImageFilter_FullyConnectedOff'))

# Start of GrayscaleGeodesicErodeImageFilter_GetFullyConnected

`GrayscaleGeodesicErodeImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleGeodesicErodeImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleGeodesicErodeImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`GrayscaleGeodesicErodeImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicErodeImageFilter')
class(`GrayscaleGeodesicErodeImageFilter_GetFullyConnected`) = c("SWIGFunction", class('GrayscaleGeodesicErodeImageFilter_GetFullyConnected'))

# Start of GrayscaleGeodesicErodeImageFilter_GetName

`GrayscaleGeodesicErodeImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleGeodesicErodeImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleGeodesicErodeImageFilter_GetName`, 'returnType') = 'character'
attr(`GrayscaleGeodesicErodeImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicErodeImageFilter')
class(`GrayscaleGeodesicErodeImageFilter_GetName`) = c("SWIGFunction", class('GrayscaleGeodesicErodeImageFilter_GetName'))

# Start of GrayscaleGeodesicErodeImageFilter_ToString

`GrayscaleGeodesicErodeImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleGeodesicErodeImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleGeodesicErodeImageFilter_ToString`, 'returnType') = 'character'
attr(`GrayscaleGeodesicErodeImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicErodeImageFilter')
class(`GrayscaleGeodesicErodeImageFilter_ToString`) = c("SWIGFunction", class('GrayscaleGeodesicErodeImageFilter_ToString'))

# Start of GrayscaleGeodesicErodeImageFilter_Execute

`GrayscaleGeodesicErodeImageFilter_Execute` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGeodesicErodeImageFilter_Execute', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicErodeImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleGeodesicErodeImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GrayscaleGeodesicErodeImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`GrayscaleGeodesicErodeImageFilter_Execute`) = c("SWIGFunction", class('GrayscaleGeodesicErodeImageFilter_Execute'))

# Start of accessor method for itk::simple::GrayscaleGeodesicErodeImageFilter
setMethod('$', '_p_itk__simple__GrayscaleGeodesicErodeImageFilter', function(x, name)

{
  accessorFuns = list('SetRunOneIteration' = GrayscaleGeodesicErodeImageFilter_SetRunOneIteration, 'RunOneIterationOn' = GrayscaleGeodesicErodeImageFilter_RunOneIterationOn, 'RunOneIterationOff' = GrayscaleGeodesicErodeImageFilter_RunOneIterationOff, 'GetRunOneIteration' = GrayscaleGeodesicErodeImageFilter_GetRunOneIteration, 'SetFullyConnected' = GrayscaleGeodesicErodeImageFilter_SetFullyConnected, 'FullyConnectedOn' = GrayscaleGeodesicErodeImageFilter_FullyConnectedOn, 'FullyConnectedOff' = GrayscaleGeodesicErodeImageFilter_FullyConnectedOff, 'GetFullyConnected' = GrayscaleGeodesicErodeImageFilter_GetFullyConnected, 'GetName' = GrayscaleGeodesicErodeImageFilter_GetName, 'ToString' = GrayscaleGeodesicErodeImageFilter_ToString, 'Execute' = GrayscaleGeodesicErodeImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GrayscaleGeodesicErodeImageFilter
# Start of GrayscaleGeodesicErode

`GrayscaleGeodesicErode__SWIG_0` = function(image1, image2, runOneIteration, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  runOneIteration = as.logical(runOneIteration);
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_GrayscaleGeodesicErode__SWIG_0', image1, image2, runOneIteration, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicErode__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleGeodesicErode__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical', 'logical')
class(`GrayscaleGeodesicErode__SWIG_0`) = c("SWIGFunction", class('GrayscaleGeodesicErode__SWIG_0'))

# Start of GrayscaleGeodesicErode

`GrayscaleGeodesicErode__SWIG_1` = function(image1, image2, runOneIteration, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  runOneIteration = as.logical(runOneIteration);
  ;ans = .Call('R_swig_GrayscaleGeodesicErode__SWIG_1', image1, image2, runOneIteration, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicErode__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleGeodesicErode__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical')
class(`GrayscaleGeodesicErode__SWIG_1`) = c("SWIGFunction", class('GrayscaleGeodesicErode__SWIG_1'))

# Start of GrayscaleGeodesicErode

`GrayscaleGeodesicErode__SWIG_2` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGeodesicErode__SWIG_2', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleGeodesicErode__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleGeodesicErode__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`GrayscaleGeodesicErode__SWIG_2`) = c("SWIGFunction", class('GrayscaleGeodesicErode__SWIG_2'))

`GrayscaleGeodesicErode` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- GrayscaleGeodesicErode__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- GrayscaleGeodesicErode__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- GrayscaleGeodesicErode__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GrayscaleGeodesicErode with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GrayscaleGrindPeakImageFilter

`delete_GrayscaleGrindPeakImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GrayscaleGrindPeakImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GrayscaleGrindPeakImageFilter`, 'returnType') = 'void'
attr(`delete_GrayscaleGrindPeakImageFilter`, "inputTypes") = c('_p_itk__simple__GrayscaleGrindPeakImageFilter')
class(`delete_GrayscaleGrindPeakImageFilter`) = c("SWIGFunction", class('delete_GrayscaleGrindPeakImageFilter'))

# Start of new_GrayscaleGrindPeakImageFilter

`GrayscaleGrindPeakImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GrayscaleGrindPeakImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GrayscaleGrindPeakImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GrayscaleGrindPeakImageFilter);
  ans
  
}

attr(`GrayscaleGrindPeakImageFilter`, 'returnType') = '_p_itk__simple__GrayscaleGrindPeakImageFilter'
class(`GrayscaleGrindPeakImageFilter`) = c("SWIGFunction", class('GrayscaleGrindPeakImageFilter'))

# Start of GrayscaleGrindPeakImageFilter_SetFullyConnected

`GrayscaleGrindPeakImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_GrayscaleGrindPeakImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleGrindPeakImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`GrayscaleGrindPeakImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__GrayscaleGrindPeakImageFilter', 'logical')
class(`GrayscaleGrindPeakImageFilter_SetFullyConnected`) = c("SWIGFunction", class('GrayscaleGrindPeakImageFilter_SetFullyConnected'))

# Start of GrayscaleGrindPeakImageFilter_FullyConnectedOn

`GrayscaleGrindPeakImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGrindPeakImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleGrindPeakImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`GrayscaleGrindPeakImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__GrayscaleGrindPeakImageFilter')
class(`GrayscaleGrindPeakImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('GrayscaleGrindPeakImageFilter_FullyConnectedOn'))

# Start of GrayscaleGrindPeakImageFilter_FullyConnectedOff

`GrayscaleGrindPeakImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGrindPeakImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleGrindPeakImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`GrayscaleGrindPeakImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__GrayscaleGrindPeakImageFilter')
class(`GrayscaleGrindPeakImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('GrayscaleGrindPeakImageFilter_FullyConnectedOff'))

# Start of GrayscaleGrindPeakImageFilter_GetFullyConnected

`GrayscaleGrindPeakImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleGrindPeakImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleGrindPeakImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`GrayscaleGrindPeakImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__GrayscaleGrindPeakImageFilter')
class(`GrayscaleGrindPeakImageFilter_GetFullyConnected`) = c("SWIGFunction", class('GrayscaleGrindPeakImageFilter_GetFullyConnected'))

# Start of GrayscaleGrindPeakImageFilter_GetName

`GrayscaleGrindPeakImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleGrindPeakImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleGrindPeakImageFilter_GetName`, 'returnType') = 'character'
attr(`GrayscaleGrindPeakImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GrayscaleGrindPeakImageFilter')
class(`GrayscaleGrindPeakImageFilter_GetName`) = c("SWIGFunction", class('GrayscaleGrindPeakImageFilter_GetName'))

# Start of GrayscaleGrindPeakImageFilter_ToString

`GrayscaleGrindPeakImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleGrindPeakImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleGrindPeakImageFilter_ToString`, 'returnType') = 'character'
attr(`GrayscaleGrindPeakImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GrayscaleGrindPeakImageFilter')
class(`GrayscaleGrindPeakImageFilter_ToString`) = c("SWIGFunction", class('GrayscaleGrindPeakImageFilter_ToString'))

# Start of GrayscaleGrindPeakImageFilter_Execute

`GrayscaleGrindPeakImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGrindPeakImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleGrindPeakImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleGrindPeakImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GrayscaleGrindPeakImageFilter', '_p_itk__simple__Image')
class(`GrayscaleGrindPeakImageFilter_Execute`) = c("SWIGFunction", class('GrayscaleGrindPeakImageFilter_Execute'))

# Start of accessor method for itk::simple::GrayscaleGrindPeakImageFilter
setMethod('$', '_p_itk__simple__GrayscaleGrindPeakImageFilter', function(x, name)

{
  accessorFuns = list('SetFullyConnected' = GrayscaleGrindPeakImageFilter_SetFullyConnected, 'FullyConnectedOn' = GrayscaleGrindPeakImageFilter_FullyConnectedOn, 'FullyConnectedOff' = GrayscaleGrindPeakImageFilter_FullyConnectedOff, 'GetFullyConnected' = GrayscaleGrindPeakImageFilter_GetFullyConnected, 'GetName' = GrayscaleGrindPeakImageFilter_GetName, 'ToString' = GrayscaleGrindPeakImageFilter_ToString, 'Execute' = GrayscaleGrindPeakImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GrayscaleGrindPeakImageFilter
# Start of GrayscaleGrindPeak

`GrayscaleGrindPeak__SWIG_0` = function(image1, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_GrayscaleGrindPeak__SWIG_0', image1, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleGrindPeak__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleGrindPeak__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`GrayscaleGrindPeak__SWIG_0`) = c("SWIGFunction", class('GrayscaleGrindPeak__SWIG_0'))

# Start of GrayscaleGrindPeak

`GrayscaleGrindPeak__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleGrindPeak__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleGrindPeak__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleGrindPeak__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`GrayscaleGrindPeak__SWIG_1`) = c("SWIGFunction", class('GrayscaleGrindPeak__SWIG_1'))

`GrayscaleGrindPeak` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- GrayscaleGrindPeak__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- GrayscaleGrindPeak__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GrayscaleGrindPeak with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GrayscaleMorphologicalClosingImageFilter

`delete_GrayscaleMorphologicalClosingImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GrayscaleMorphologicalClosingImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GrayscaleMorphologicalClosingImageFilter`, 'returnType') = 'void'
attr(`delete_GrayscaleMorphologicalClosingImageFilter`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalClosingImageFilter')
class(`delete_GrayscaleMorphologicalClosingImageFilter`) = c("SWIGFunction", class('delete_GrayscaleMorphologicalClosingImageFilter'))

# Start of new_GrayscaleMorphologicalClosingImageFilter

`GrayscaleMorphologicalClosingImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GrayscaleMorphologicalClosingImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GrayscaleMorphologicalClosingImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GrayscaleMorphologicalClosingImageFilter);
  ans
  
}

attr(`GrayscaleMorphologicalClosingImageFilter`, 'returnType') = '_p_itk__simple__GrayscaleMorphologicalClosingImageFilter'
class(`GrayscaleMorphologicalClosingImageFilter`) = c("SWIGFunction", class('GrayscaleMorphologicalClosingImageFilter'))

# Start of GrayscaleMorphologicalClosingImageFilter_SetKernelRadius

`GrayscaleMorphologicalClosingImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_GrayscaleMorphologicalClosingImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalClosingImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`GrayscaleMorphologicalClosingImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalClosingImageFilter', 'integer')
class(`GrayscaleMorphologicalClosingImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('GrayscaleMorphologicalClosingImageFilter_SetKernelRadius__SWIG_0'))

# Start of GrayscaleMorphologicalClosingImageFilter_SetKernelRadius

`GrayscaleMorphologicalClosingImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_GrayscaleMorphologicalClosingImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalClosingImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`GrayscaleMorphologicalClosingImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalClosingImageFilter', 'integer')
class(`GrayscaleMorphologicalClosingImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('GrayscaleMorphologicalClosingImageFilter_SetKernelRadius__SWIG_1'))

`GrayscaleMorphologicalClosingImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__GrayscaleMorphologicalClosingImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- GrayscaleMorphologicalClosingImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__GrayscaleMorphologicalClosingImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- GrayscaleMorphologicalClosingImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GrayscaleMorphologicalClosingImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of GrayscaleMorphologicalClosingImageFilter_GetKernelRadius

`GrayscaleMorphologicalClosingImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleMorphologicalClosingImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleMorphologicalClosingImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`GrayscaleMorphologicalClosingImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalClosingImageFilter')
class(`GrayscaleMorphologicalClosingImageFilter_GetKernelRadius`) = c("SWIGFunction", class('GrayscaleMorphologicalClosingImageFilter_GetKernelRadius'))

# Start of GrayscaleMorphologicalClosingImageFilter_SetKernelType

`GrayscaleMorphologicalClosingImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_GrayscaleMorphologicalClosingImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalClosingImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`GrayscaleMorphologicalClosingImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalClosingImageFilter', 'character')
class(`GrayscaleMorphologicalClosingImageFilter_SetKernelType`) = c("SWIGFunction", class('GrayscaleMorphologicalClosingImageFilter_SetKernelType'))

# Start of GrayscaleMorphologicalClosingImageFilter_GetKernelType

`GrayscaleMorphologicalClosingImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleMorphologicalClosingImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`GrayscaleMorphologicalClosingImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`GrayscaleMorphologicalClosingImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalClosingImageFilter')
class(`GrayscaleMorphologicalClosingImageFilter_GetKernelType`) = c("SWIGFunction", class('GrayscaleMorphologicalClosingImageFilter_GetKernelType'))

# Start of GrayscaleMorphologicalClosingImageFilter_SetSafeBorder

`GrayscaleMorphologicalClosingImageFilter_SetSafeBorder` = function(self, SafeBorder)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SafeBorder = as.logical(SafeBorder);
  ;ans = .Call('R_swig_GrayscaleMorphologicalClosingImageFilter_SetSafeBorder', self, SafeBorder, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalClosingImageFilter_SetSafeBorder`, 'returnType') = 'void'
attr(`GrayscaleMorphologicalClosingImageFilter_SetSafeBorder`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalClosingImageFilter', 'logical')
class(`GrayscaleMorphologicalClosingImageFilter_SetSafeBorder`) = c("SWIGFunction", class('GrayscaleMorphologicalClosingImageFilter_SetSafeBorder'))

# Start of GrayscaleMorphologicalClosingImageFilter_SafeBorderOn

`GrayscaleMorphologicalClosingImageFilter_SafeBorderOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleMorphologicalClosingImageFilter_SafeBorderOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalClosingImageFilter_SafeBorderOn`, 'returnType') = 'void'
attr(`GrayscaleMorphologicalClosingImageFilter_SafeBorderOn`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalClosingImageFilter')
class(`GrayscaleMorphologicalClosingImageFilter_SafeBorderOn`) = c("SWIGFunction", class('GrayscaleMorphologicalClosingImageFilter_SafeBorderOn'))

# Start of GrayscaleMorphologicalClosingImageFilter_SafeBorderOff

`GrayscaleMorphologicalClosingImageFilter_SafeBorderOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleMorphologicalClosingImageFilter_SafeBorderOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalClosingImageFilter_SafeBorderOff`, 'returnType') = 'void'
attr(`GrayscaleMorphologicalClosingImageFilter_SafeBorderOff`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalClosingImageFilter')
class(`GrayscaleMorphologicalClosingImageFilter_SafeBorderOff`) = c("SWIGFunction", class('GrayscaleMorphologicalClosingImageFilter_SafeBorderOff'))

# Start of GrayscaleMorphologicalClosingImageFilter_GetSafeBorder

`GrayscaleMorphologicalClosingImageFilter_GetSafeBorder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleMorphologicalClosingImageFilter_GetSafeBorder', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleMorphologicalClosingImageFilter_GetSafeBorder`, 'returnType') = 'logical'
attr(`GrayscaleMorphologicalClosingImageFilter_GetSafeBorder`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalClosingImageFilter')
class(`GrayscaleMorphologicalClosingImageFilter_GetSafeBorder`) = c("SWIGFunction", class('GrayscaleMorphologicalClosingImageFilter_GetSafeBorder'))

# Start of GrayscaleMorphologicalClosingImageFilter_GetName

`GrayscaleMorphologicalClosingImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleMorphologicalClosingImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleMorphologicalClosingImageFilter_GetName`, 'returnType') = 'character'
attr(`GrayscaleMorphologicalClosingImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalClosingImageFilter')
class(`GrayscaleMorphologicalClosingImageFilter_GetName`) = c("SWIGFunction", class('GrayscaleMorphologicalClosingImageFilter_GetName'))

# Start of GrayscaleMorphologicalClosingImageFilter_ToString

`GrayscaleMorphologicalClosingImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleMorphologicalClosingImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleMorphologicalClosingImageFilter_ToString`, 'returnType') = 'character'
attr(`GrayscaleMorphologicalClosingImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalClosingImageFilter')
class(`GrayscaleMorphologicalClosingImageFilter_ToString`) = c("SWIGFunction", class('GrayscaleMorphologicalClosingImageFilter_ToString'))

# Start of GrayscaleMorphologicalClosingImageFilter_Execute

`GrayscaleMorphologicalClosingImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleMorphologicalClosingImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalClosingImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleMorphologicalClosingImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalClosingImageFilter', '_p_itk__simple__Image')
class(`GrayscaleMorphologicalClosingImageFilter_Execute`) = c("SWIGFunction", class('GrayscaleMorphologicalClosingImageFilter_Execute'))

# Start of accessor method for itk::simple::GrayscaleMorphologicalClosingImageFilter
setMethod('$', '_p_itk__simple__GrayscaleMorphologicalClosingImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = GrayscaleMorphologicalClosingImageFilter_SetKernelRadius, 'GetKernelRadius' = GrayscaleMorphologicalClosingImageFilter_GetKernelRadius, 'SetKernelType' = GrayscaleMorphologicalClosingImageFilter_SetKernelType, 'GetKernelType' = GrayscaleMorphologicalClosingImageFilter_GetKernelType, 'SetSafeBorder' = GrayscaleMorphologicalClosingImageFilter_SetSafeBorder, 'SafeBorderOn' = GrayscaleMorphologicalClosingImageFilter_SafeBorderOn, 'SafeBorderOff' = GrayscaleMorphologicalClosingImageFilter_SafeBorderOff, 'GetSafeBorder' = GrayscaleMorphologicalClosingImageFilter_GetSafeBorder, 'GetName' = GrayscaleMorphologicalClosingImageFilter_GetName, 'ToString' = GrayscaleMorphologicalClosingImageFilter_ToString, 'Execute' = GrayscaleMorphologicalClosingImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GrayscaleMorphologicalClosingImageFilter
# Start of GrayscaleMorphologicalClosing

`GrayscaleMorphologicalClosing__SWIG_0` = function(image1, kernelRadius, kernelType, safeBorder, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  safeBorder = as.logical(safeBorder);
  ;ans = .Call('R_swig_GrayscaleMorphologicalClosing__SWIG_0', image1, kernelRadius, kernelType, safeBorder, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalClosing__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleMorphologicalClosing__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'logical')
class(`GrayscaleMorphologicalClosing__SWIG_0`) = c("SWIGFunction", class('GrayscaleMorphologicalClosing__SWIG_0'))

# Start of GrayscaleMorphologicalClosing

`GrayscaleMorphologicalClosing__SWIG_1` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_GrayscaleMorphologicalClosing__SWIG_1', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalClosing__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleMorphologicalClosing__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`GrayscaleMorphologicalClosing__SWIG_1`) = c("SWIGFunction", class('GrayscaleMorphologicalClosing__SWIG_1'))

# Start of GrayscaleMorphologicalClosing

`GrayscaleMorphologicalClosing__SWIG_2` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_GrayscaleMorphologicalClosing__SWIG_2', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalClosing__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleMorphologicalClosing__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`GrayscaleMorphologicalClosing__SWIG_2`) = c("SWIGFunction", class('GrayscaleMorphologicalClosing__SWIG_2'))

# Start of GrayscaleMorphologicalClosing

`GrayscaleMorphologicalClosing__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleMorphologicalClosing__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalClosing__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleMorphologicalClosing__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`GrayscaleMorphologicalClosing__SWIG_3`) = c("SWIGFunction", class('GrayscaleMorphologicalClosing__SWIG_3'))

`GrayscaleMorphologicalClosing` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- GrayscaleMorphologicalClosing__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- GrayscaleMorphologicalClosing__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- GrayscaleMorphologicalClosing__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- GrayscaleMorphologicalClosing__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GrayscaleMorphologicalClosing with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GrayscaleMorphologicalOpeningImageFilter

`delete_GrayscaleMorphologicalOpeningImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GrayscaleMorphologicalOpeningImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GrayscaleMorphologicalOpeningImageFilter`, 'returnType') = 'void'
attr(`delete_GrayscaleMorphologicalOpeningImageFilter`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter')
class(`delete_GrayscaleMorphologicalOpeningImageFilter`) = c("SWIGFunction", class('delete_GrayscaleMorphologicalOpeningImageFilter'))

# Start of new_GrayscaleMorphologicalOpeningImageFilter

`GrayscaleMorphologicalOpeningImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GrayscaleMorphologicalOpeningImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GrayscaleMorphologicalOpeningImageFilter);
  ans
  
}

attr(`GrayscaleMorphologicalOpeningImageFilter`, 'returnType') = '_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter'
class(`GrayscaleMorphologicalOpeningImageFilter`) = c("SWIGFunction", class('GrayscaleMorphologicalOpeningImageFilter'))

# Start of GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius

`GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter', 'integer')
class(`GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_0'))

# Start of GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius

`GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter', 'integer')
class(`GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_1'))

`GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of GrayscaleMorphologicalOpeningImageFilter_GetKernelRadius

`GrayscaleMorphologicalOpeningImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleMorphologicalOpeningImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleMorphologicalOpeningImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`GrayscaleMorphologicalOpeningImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter')
class(`GrayscaleMorphologicalOpeningImageFilter_GetKernelRadius`) = c("SWIGFunction", class('GrayscaleMorphologicalOpeningImageFilter_GetKernelRadius'))

# Start of GrayscaleMorphologicalOpeningImageFilter_SetKernelType

`GrayscaleMorphologicalOpeningImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_GrayscaleMorphologicalOpeningImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalOpeningImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`GrayscaleMorphologicalOpeningImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter', 'character')
class(`GrayscaleMorphologicalOpeningImageFilter_SetKernelType`) = c("SWIGFunction", class('GrayscaleMorphologicalOpeningImageFilter_SetKernelType'))

# Start of GrayscaleMorphologicalOpeningImageFilter_GetKernelType

`GrayscaleMorphologicalOpeningImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleMorphologicalOpeningImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`GrayscaleMorphologicalOpeningImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`GrayscaleMorphologicalOpeningImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter')
class(`GrayscaleMorphologicalOpeningImageFilter_GetKernelType`) = c("SWIGFunction", class('GrayscaleMorphologicalOpeningImageFilter_GetKernelType'))

# Start of GrayscaleMorphologicalOpeningImageFilter_SetSafeBorder

`GrayscaleMorphologicalOpeningImageFilter_SetSafeBorder` = function(self, SafeBorder)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SafeBorder = as.logical(SafeBorder);
  ;ans = .Call('R_swig_GrayscaleMorphologicalOpeningImageFilter_SetSafeBorder', self, SafeBorder, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalOpeningImageFilter_SetSafeBorder`, 'returnType') = 'void'
attr(`GrayscaleMorphologicalOpeningImageFilter_SetSafeBorder`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter', 'logical')
class(`GrayscaleMorphologicalOpeningImageFilter_SetSafeBorder`) = c("SWIGFunction", class('GrayscaleMorphologicalOpeningImageFilter_SetSafeBorder'))

# Start of GrayscaleMorphologicalOpeningImageFilter_SafeBorderOn

`GrayscaleMorphologicalOpeningImageFilter_SafeBorderOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleMorphologicalOpeningImageFilter_SafeBorderOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalOpeningImageFilter_SafeBorderOn`, 'returnType') = 'void'
attr(`GrayscaleMorphologicalOpeningImageFilter_SafeBorderOn`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter')
class(`GrayscaleMorphologicalOpeningImageFilter_SafeBorderOn`) = c("SWIGFunction", class('GrayscaleMorphologicalOpeningImageFilter_SafeBorderOn'))

# Start of GrayscaleMorphologicalOpeningImageFilter_SafeBorderOff

`GrayscaleMorphologicalOpeningImageFilter_SafeBorderOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GrayscaleMorphologicalOpeningImageFilter_SafeBorderOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalOpeningImageFilter_SafeBorderOff`, 'returnType') = 'void'
attr(`GrayscaleMorphologicalOpeningImageFilter_SafeBorderOff`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter')
class(`GrayscaleMorphologicalOpeningImageFilter_SafeBorderOff`) = c("SWIGFunction", class('GrayscaleMorphologicalOpeningImageFilter_SafeBorderOff'))

# Start of GrayscaleMorphologicalOpeningImageFilter_GetSafeBorder

`GrayscaleMorphologicalOpeningImageFilter_GetSafeBorder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleMorphologicalOpeningImageFilter_GetSafeBorder', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleMorphologicalOpeningImageFilter_GetSafeBorder`, 'returnType') = 'logical'
attr(`GrayscaleMorphologicalOpeningImageFilter_GetSafeBorder`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter')
class(`GrayscaleMorphologicalOpeningImageFilter_GetSafeBorder`) = c("SWIGFunction", class('GrayscaleMorphologicalOpeningImageFilter_GetSafeBorder'))

# Start of GrayscaleMorphologicalOpeningImageFilter_GetName

`GrayscaleMorphologicalOpeningImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleMorphologicalOpeningImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleMorphologicalOpeningImageFilter_GetName`, 'returnType') = 'character'
attr(`GrayscaleMorphologicalOpeningImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter')
class(`GrayscaleMorphologicalOpeningImageFilter_GetName`) = c("SWIGFunction", class('GrayscaleMorphologicalOpeningImageFilter_GetName'))

# Start of GrayscaleMorphologicalOpeningImageFilter_ToString

`GrayscaleMorphologicalOpeningImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GrayscaleMorphologicalOpeningImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GrayscaleMorphologicalOpeningImageFilter_ToString`, 'returnType') = 'character'
attr(`GrayscaleMorphologicalOpeningImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter')
class(`GrayscaleMorphologicalOpeningImageFilter_ToString`) = c("SWIGFunction", class('GrayscaleMorphologicalOpeningImageFilter_ToString'))

# Start of GrayscaleMorphologicalOpeningImageFilter_Execute

`GrayscaleMorphologicalOpeningImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleMorphologicalOpeningImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalOpeningImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleMorphologicalOpeningImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter', '_p_itk__simple__Image')
class(`GrayscaleMorphologicalOpeningImageFilter_Execute`) = c("SWIGFunction", class('GrayscaleMorphologicalOpeningImageFilter_Execute'))

# Start of accessor method for itk::simple::GrayscaleMorphologicalOpeningImageFilter
setMethod('$', '_p_itk__simple__GrayscaleMorphologicalOpeningImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = GrayscaleMorphologicalOpeningImageFilter_SetKernelRadius, 'GetKernelRadius' = GrayscaleMorphologicalOpeningImageFilter_GetKernelRadius, 'SetKernelType' = GrayscaleMorphologicalOpeningImageFilter_SetKernelType, 'GetKernelType' = GrayscaleMorphologicalOpeningImageFilter_GetKernelType, 'SetSafeBorder' = GrayscaleMorphologicalOpeningImageFilter_SetSafeBorder, 'SafeBorderOn' = GrayscaleMorphologicalOpeningImageFilter_SafeBorderOn, 'SafeBorderOff' = GrayscaleMorphologicalOpeningImageFilter_SafeBorderOff, 'GetSafeBorder' = GrayscaleMorphologicalOpeningImageFilter_GetSafeBorder, 'GetName' = GrayscaleMorphologicalOpeningImageFilter_GetName, 'ToString' = GrayscaleMorphologicalOpeningImageFilter_ToString, 'Execute' = GrayscaleMorphologicalOpeningImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GrayscaleMorphologicalOpeningImageFilter
# Start of GrayscaleMorphologicalOpening

`GrayscaleMorphologicalOpening__SWIG_0` = function(image1, kernelRadius, kernelType, safeBorder, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  safeBorder = as.logical(safeBorder);
  ;ans = .Call('R_swig_GrayscaleMorphologicalOpening__SWIG_0', image1, kernelRadius, kernelType, safeBorder, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalOpening__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleMorphologicalOpening__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'logical')
class(`GrayscaleMorphologicalOpening__SWIG_0`) = c("SWIGFunction", class('GrayscaleMorphologicalOpening__SWIG_0'))

# Start of GrayscaleMorphologicalOpening

`GrayscaleMorphologicalOpening__SWIG_1` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_GrayscaleMorphologicalOpening__SWIG_1', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalOpening__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleMorphologicalOpening__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`GrayscaleMorphologicalOpening__SWIG_1`) = c("SWIGFunction", class('GrayscaleMorphologicalOpening__SWIG_1'))

# Start of GrayscaleMorphologicalOpening

`GrayscaleMorphologicalOpening__SWIG_2` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_GrayscaleMorphologicalOpening__SWIG_2', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalOpening__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleMorphologicalOpening__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`GrayscaleMorphologicalOpening__SWIG_2`) = c("SWIGFunction", class('GrayscaleMorphologicalOpening__SWIG_2'))

# Start of GrayscaleMorphologicalOpening

`GrayscaleMorphologicalOpening__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_GrayscaleMorphologicalOpening__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GrayscaleMorphologicalOpening__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`GrayscaleMorphologicalOpening__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`GrayscaleMorphologicalOpening__SWIG_3`) = c("SWIGFunction", class('GrayscaleMorphologicalOpening__SWIG_3'))

`GrayscaleMorphologicalOpening` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- GrayscaleMorphologicalOpening__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- GrayscaleMorphologicalOpening__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- GrayscaleMorphologicalOpening__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- GrayscaleMorphologicalOpening__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GrayscaleMorphologicalOpening with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GreaterEqualImageFilter

`delete_GreaterEqualImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GreaterEqualImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GreaterEqualImageFilter`, 'returnType') = 'void'
attr(`delete_GreaterEqualImageFilter`, "inputTypes") = c('_p_itk__simple__GreaterEqualImageFilter')
class(`delete_GreaterEqualImageFilter`) = c("SWIGFunction", class('delete_GreaterEqualImageFilter'))

# Start of new_GreaterEqualImageFilter

`GreaterEqualImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GreaterEqualImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GreaterEqualImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GreaterEqualImageFilter);
  ans
  
}

attr(`GreaterEqualImageFilter`, 'returnType') = '_p_itk__simple__GreaterEqualImageFilter'
class(`GreaterEqualImageFilter`) = c("SWIGFunction", class('GreaterEqualImageFilter'))

# Start of GreaterEqualImageFilter_SetBackgroundValue

`GreaterEqualImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BackgroundValue = as.integer(BackgroundValue);
  
  if(length(BackgroundValue) > 1) {
    warning("using only the first element of BackgroundValue");
  };
  
  ;ans = .Call('R_swig_GreaterEqualImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GreaterEqualImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`GreaterEqualImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__GreaterEqualImageFilter', 'integer')
class(`GreaterEqualImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('GreaterEqualImageFilter_SetBackgroundValue'))

# Start of GreaterEqualImageFilter_GetBackgroundValue

`GreaterEqualImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GreaterEqualImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GreaterEqualImageFilter_GetBackgroundValue`, 'returnType') = 'integer'
attr(`GreaterEqualImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__GreaterEqualImageFilter')
class(`GreaterEqualImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('GreaterEqualImageFilter_GetBackgroundValue'))

# Start of GreaterEqualImageFilter_SetForegroundValue

`GreaterEqualImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ForegroundValue = as.integer(ForegroundValue);
  
  if(length(ForegroundValue) > 1) {
    warning("using only the first element of ForegroundValue");
  };
  
  ;ans = .Call('R_swig_GreaterEqualImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GreaterEqualImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`GreaterEqualImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__GreaterEqualImageFilter', 'integer')
class(`GreaterEqualImageFilter_SetForegroundValue`) = c("SWIGFunction", class('GreaterEqualImageFilter_SetForegroundValue'))

# Start of GreaterEqualImageFilter_GetForegroundValue

`GreaterEqualImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GreaterEqualImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GreaterEqualImageFilter_GetForegroundValue`, 'returnType') = 'integer'
attr(`GreaterEqualImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__GreaterEqualImageFilter')
class(`GreaterEqualImageFilter_GetForegroundValue`) = c("SWIGFunction", class('GreaterEqualImageFilter_GetForegroundValue'))

# Start of GreaterEqualImageFilter_GetName

`GreaterEqualImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GreaterEqualImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GreaterEqualImageFilter_GetName`, 'returnType') = 'character'
attr(`GreaterEqualImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GreaterEqualImageFilter')
class(`GreaterEqualImageFilter_GetName`) = c("SWIGFunction", class('GreaterEqualImageFilter_GetName'))

# Start of GreaterEqualImageFilter_ToString

`GreaterEqualImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GreaterEqualImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GreaterEqualImageFilter_ToString`, 'returnType') = 'character'
attr(`GreaterEqualImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GreaterEqualImageFilter')
class(`GreaterEqualImageFilter_ToString`) = c("SWIGFunction", class('GreaterEqualImageFilter_ToString'))

# Start of GreaterEqualImageFilter_Execute

`GreaterEqualImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_GreaterEqualImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterEqualImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterEqualImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__GreaterEqualImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`GreaterEqualImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('GreaterEqualImageFilter_Execute__SWIG_0'))

# Start of GreaterEqualImageFilter_Execute

`GreaterEqualImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_GreaterEqualImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterEqualImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterEqualImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__GreaterEqualImageFilter', '_p_itk__simple__Image', 'numeric')
class(`GreaterEqualImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('GreaterEqualImageFilter_Execute__SWIG_1'))

# Start of GreaterEqualImageFilter_Execute

`GreaterEqualImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_GreaterEqualImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterEqualImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterEqualImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__GreaterEqualImageFilter', 'numeric', '_p_itk__simple__Image')
class(`GreaterEqualImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('GreaterEqualImageFilter_Execute__SWIG_2'))

# Start of GreaterEqualImageFilter_Execute

`GreaterEqualImageFilter_Execute__SWIG_3` = function(self, image1, constant, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_GreaterEqualImageFilter_Execute__SWIG_3', self, image1, constant, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterEqualImageFilter_Execute__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterEqualImageFilter_Execute__SWIG_3`, "inputTypes") = c('_p_itk__simple__GreaterEqualImageFilter', '_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`GreaterEqualImageFilter_Execute__SWIG_3`) = c("SWIGFunction", class('GreaterEqualImageFilter_Execute__SWIG_3'))

# Start of GreaterEqualImageFilter_Execute

`GreaterEqualImageFilter_Execute__SWIG_4` = function(self, constant, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_GreaterEqualImageFilter_Execute__SWIG_4', self, constant, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterEqualImageFilter_Execute__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterEqualImageFilter_Execute__SWIG_4`, "inputTypes") = c('_p_itk__simple__GreaterEqualImageFilter', 'numeric', '_p_itk__simple__Image', 'integer', 'integer')
class(`GreaterEqualImageFilter_Execute__SWIG_4`) = c("SWIGFunction", class('GreaterEqualImageFilter_Execute__SWIG_4'))

`GreaterEqualImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__GreaterEqualImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- GreaterEqualImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__GreaterEqualImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- GreaterEqualImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__GreaterEqualImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- GreaterEqualImageFilter_Execute__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__GreaterEqualImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- GreaterEqualImageFilter_Execute__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__GreaterEqualImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- GreaterEqualImageFilter_Execute__SWIG_4; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GreaterEqualImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::GreaterEqualImageFilter
setMethod('$', '_p_itk__simple__GreaterEqualImageFilter', function(x, name)

{
  accessorFuns = list('SetBackgroundValue' = GreaterEqualImageFilter_SetBackgroundValue, 'GetBackgroundValue' = GreaterEqualImageFilter_GetBackgroundValue, 'SetForegroundValue' = GreaterEqualImageFilter_SetForegroundValue, 'GetForegroundValue' = GreaterEqualImageFilter_GetForegroundValue, 'GetName' = GreaterEqualImageFilter_GetName, 'ToString' = GreaterEqualImageFilter_ToString, 'Execute' = GreaterEqualImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GreaterEqualImageFilter
# Start of GreaterEqual

`GreaterEqual__SWIG_0` = function(image1, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_GreaterEqual__SWIG_0', image1, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterEqual__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterEqual__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`GreaterEqual__SWIG_0`) = c("SWIGFunction", class('GreaterEqual__SWIG_0'))

# Start of GreaterEqual

`GreaterEqual__SWIG_1` = function(image1, image2, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_GreaterEqual__SWIG_1', image1, image2, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterEqual__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterEqual__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`GreaterEqual__SWIG_1`) = c("SWIGFunction", class('GreaterEqual__SWIG_1'))

# Start of GreaterEqual

`GreaterEqual__SWIG_2` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_GreaterEqual__SWIG_2', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterEqual__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterEqual__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`GreaterEqual__SWIG_2`) = c("SWIGFunction", class('GreaterEqual__SWIG_2'))

# Start of GreaterEqual

`GreaterEqual__SWIG_3` = function(image1, constant, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_GreaterEqual__SWIG_3', image1, constant, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterEqual__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterEqual__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`GreaterEqual__SWIG_3`) = c("SWIGFunction", class('GreaterEqual__SWIG_3'))

# Start of GreaterEqual

`GreaterEqual__SWIG_4` = function(image1, constant, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_GreaterEqual__SWIG_4', image1, constant, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterEqual__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterEqual__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`GreaterEqual__SWIG_4`) = c("SWIGFunction", class('GreaterEqual__SWIG_4'))

# Start of GreaterEqual

`GreaterEqual__SWIG_5` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_GreaterEqual__SWIG_5', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterEqual__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterEqual__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`GreaterEqual__SWIG_5`) = c("SWIGFunction", class('GreaterEqual__SWIG_5'))

# Start of GreaterEqual

`GreaterEqual__SWIG_6` = function(constant, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_GreaterEqual__SWIG_6', constant, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterEqual__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterEqual__SWIG_6`, "inputTypes") = c('numeric', '_p_itk__simple__Image', 'integer', 'integer')
class(`GreaterEqual__SWIG_6`) = c("SWIGFunction", class('GreaterEqual__SWIG_6'))

# Start of GreaterEqual

`GreaterEqual__SWIG_7` = function(constant, image2, backgroundValue, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_GreaterEqual__SWIG_7', constant, image2, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterEqual__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterEqual__SWIG_7`, "inputTypes") = c('numeric', '_p_itk__simple__Image', 'integer')
class(`GreaterEqual__SWIG_7`) = c("SWIGFunction", class('GreaterEqual__SWIG_7'))

# Start of GreaterEqual

`GreaterEqual__SWIG_8` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_GreaterEqual__SWIG_8', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterEqual__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterEqual__SWIG_8`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`GreaterEqual__SWIG_8`) = c("SWIGFunction", class('GreaterEqual__SWIG_8'))

`GreaterEqual` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- GreaterEqual__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- GreaterEqual__SWIG_5; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- GreaterEqual__SWIG_8; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- GreaterEqual__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- GreaterEqual__SWIG_4; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- GreaterEqual__SWIG_7; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- GreaterEqual__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- GreaterEqual__SWIG_3; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- GreaterEqual__SWIG_6; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GreaterEqual with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GreaterImageFilter

`delete_GreaterImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GreaterImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GreaterImageFilter`, 'returnType') = 'void'
attr(`delete_GreaterImageFilter`, "inputTypes") = c('_p_itk__simple__GreaterImageFilter')
class(`delete_GreaterImageFilter`) = c("SWIGFunction", class('delete_GreaterImageFilter'))

# Start of new_GreaterImageFilter

`GreaterImageFilter` = function()
{
  ;ans = .Call('R_swig_new_GreaterImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GreaterImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GreaterImageFilter);
  ans
  
}

attr(`GreaterImageFilter`, 'returnType') = '_p_itk__simple__GreaterImageFilter'
class(`GreaterImageFilter`) = c("SWIGFunction", class('GreaterImageFilter'))

# Start of GreaterImageFilter_SetBackgroundValue

`GreaterImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BackgroundValue = as.integer(BackgroundValue);
  
  if(length(BackgroundValue) > 1) {
    warning("using only the first element of BackgroundValue");
  };
  
  ;ans = .Call('R_swig_GreaterImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GreaterImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`GreaterImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__GreaterImageFilter', 'integer')
class(`GreaterImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('GreaterImageFilter_SetBackgroundValue'))

# Start of GreaterImageFilter_GetBackgroundValue

`GreaterImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GreaterImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GreaterImageFilter_GetBackgroundValue`, 'returnType') = 'integer'
attr(`GreaterImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__GreaterImageFilter')
class(`GreaterImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('GreaterImageFilter_GetBackgroundValue'))

# Start of GreaterImageFilter_SetForegroundValue

`GreaterImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ForegroundValue = as.integer(ForegroundValue);
  
  if(length(ForegroundValue) > 1) {
    warning("using only the first element of ForegroundValue");
  };
  
  ;ans = .Call('R_swig_GreaterImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GreaterImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`GreaterImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__GreaterImageFilter', 'integer')
class(`GreaterImageFilter_SetForegroundValue`) = c("SWIGFunction", class('GreaterImageFilter_SetForegroundValue'))

# Start of GreaterImageFilter_GetForegroundValue

`GreaterImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GreaterImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GreaterImageFilter_GetForegroundValue`, 'returnType') = 'integer'
attr(`GreaterImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__GreaterImageFilter')
class(`GreaterImageFilter_GetForegroundValue`) = c("SWIGFunction", class('GreaterImageFilter_GetForegroundValue'))

# Start of GreaterImageFilter_GetName

`GreaterImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GreaterImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GreaterImageFilter_GetName`, 'returnType') = 'character'
attr(`GreaterImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__GreaterImageFilter')
class(`GreaterImageFilter_GetName`) = c("SWIGFunction", class('GreaterImageFilter_GetName'))

# Start of GreaterImageFilter_ToString

`GreaterImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GreaterImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GreaterImageFilter_ToString`, 'returnType') = 'character'
attr(`GreaterImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__GreaterImageFilter')
class(`GreaterImageFilter_ToString`) = c("SWIGFunction", class('GreaterImageFilter_ToString'))

# Start of GreaterImageFilter_Execute

`GreaterImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_GreaterImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__GreaterImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`GreaterImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('GreaterImageFilter_Execute__SWIG_0'))

# Start of GreaterImageFilter_Execute

`GreaterImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_GreaterImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__GreaterImageFilter', '_p_itk__simple__Image', 'numeric')
class(`GreaterImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('GreaterImageFilter_Execute__SWIG_1'))

# Start of GreaterImageFilter_Execute

`GreaterImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_GreaterImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__GreaterImageFilter', 'numeric', '_p_itk__simple__Image')
class(`GreaterImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('GreaterImageFilter_Execute__SWIG_2'))

# Start of GreaterImageFilter_Execute

`GreaterImageFilter_Execute__SWIG_3` = function(self, image1, constant, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_GreaterImageFilter_Execute__SWIG_3', self, image1, constant, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterImageFilter_Execute__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterImageFilter_Execute__SWIG_3`, "inputTypes") = c('_p_itk__simple__GreaterImageFilter', '_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`GreaterImageFilter_Execute__SWIG_3`) = c("SWIGFunction", class('GreaterImageFilter_Execute__SWIG_3'))

# Start of GreaterImageFilter_Execute

`GreaterImageFilter_Execute__SWIG_4` = function(self, constant, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_GreaterImageFilter_Execute__SWIG_4', self, constant, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GreaterImageFilter_Execute__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`GreaterImageFilter_Execute__SWIG_4`, "inputTypes") = c('_p_itk__simple__GreaterImageFilter', 'numeric', '_p_itk__simple__Image', 'integer', 'integer')
class(`GreaterImageFilter_Execute__SWIG_4`) = c("SWIGFunction", class('GreaterImageFilter_Execute__SWIG_4'))

`GreaterImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__GreaterImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- GreaterImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__GreaterImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- GreaterImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__GreaterImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- GreaterImageFilter_Execute__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__GreaterImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- GreaterImageFilter_Execute__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__GreaterImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- GreaterImageFilter_Execute__SWIG_4; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GreaterImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::GreaterImageFilter
setMethod('$', '_p_itk__simple__GreaterImageFilter', function(x, name)

{
  accessorFuns = list('SetBackgroundValue' = GreaterImageFilter_SetBackgroundValue, 'GetBackgroundValue' = GreaterImageFilter_GetBackgroundValue, 'SetForegroundValue' = GreaterImageFilter_SetForegroundValue, 'GetForegroundValue' = GreaterImageFilter_GetForegroundValue, 'GetName' = GreaterImageFilter_GetName, 'ToString' = GreaterImageFilter_ToString, 'Execute' = GreaterImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GreaterImageFilter
# Start of Greater

`Greater__SWIG_0` = function(image1, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_Greater__SWIG_0', image1, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Greater__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Greater__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`Greater__SWIG_0`) = c("SWIGFunction", class('Greater__SWIG_0'))

# Start of Greater

`Greater__SWIG_1` = function(image1, image2, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_Greater__SWIG_1', image1, image2, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Greater__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Greater__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`Greater__SWIG_1`) = c("SWIGFunction", class('Greater__SWIG_1'))

# Start of Greater

`Greater__SWIG_2` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Greater__SWIG_2', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Greater__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Greater__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Greater__SWIG_2`) = c("SWIGFunction", class('Greater__SWIG_2'))

# Start of Greater

`Greater__SWIG_3` = function(image1, constant, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_Greater__SWIG_3', image1, constant, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Greater__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Greater__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`Greater__SWIG_3`) = c("SWIGFunction", class('Greater__SWIG_3'))

# Start of Greater

`Greater__SWIG_4` = function(image1, constant, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_Greater__SWIG_4', image1, constant, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Greater__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`Greater__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`Greater__SWIG_4`) = c("SWIGFunction", class('Greater__SWIG_4'))

# Start of Greater

`Greater__SWIG_5` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Greater__SWIG_5', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Greater__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`Greater__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`Greater__SWIG_5`) = c("SWIGFunction", class('Greater__SWIG_5'))

# Start of Greater

`Greater__SWIG_6` = function(constant, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_Greater__SWIG_6', constant, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Greater__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`Greater__SWIG_6`, "inputTypes") = c('numeric', '_p_itk__simple__Image', 'integer', 'integer')
class(`Greater__SWIG_6`) = c("SWIGFunction", class('Greater__SWIG_6'))

# Start of Greater

`Greater__SWIG_7` = function(constant, image2, backgroundValue, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_Greater__SWIG_7', constant, image2, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Greater__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`Greater__SWIG_7`, "inputTypes") = c('numeric', '_p_itk__simple__Image', 'integer')
class(`Greater__SWIG_7`) = c("SWIGFunction", class('Greater__SWIG_7'))

# Start of Greater

`Greater__SWIG_8` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Greater__SWIG_8', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Greater__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`Greater__SWIG_8`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`Greater__SWIG_8`) = c("SWIGFunction", class('Greater__SWIG_8'))

`Greater` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Greater__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Greater__SWIG_5; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Greater__SWIG_8; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- Greater__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- Greater__SWIG_4; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- Greater__SWIG_7; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- Greater__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- Greater__SWIG_3; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- Greater__SWIG_6; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Greater with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_GridImageSource

`delete_GridImageSource` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_GridImageSource', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_GridImageSource`, 'returnType') = 'void'
attr(`delete_GridImageSource`, "inputTypes") = c('_p_itk__simple__GridImageSource')
class(`delete_GridImageSource`) = c("SWIGFunction", class('delete_GridImageSource'))

# Start of new_GridImageSource

`GridImageSource` = function()
{
  ;ans = .Call('R_swig_new_GridImageSource', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__GridImageSource", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_GridImageSource);
  ans
  
}

attr(`GridImageSource`, 'returnType') = '_p_itk__simple__GridImageSource'
class(`GridImageSource`) = c("SWIGFunction", class('GridImageSource'))

# Start of GridImageSource_SetOutputPixelType

`GridImageSource_SetOutputPixelType` = function(self, OutputPixelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputPixelType = enumToInteger(OutputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(OutputPixelType) > 1) {
    warning("using only the first element of OutputPixelType");
  };
  
  ;ans = .Call('R_swig_GridImageSource_SetOutputPixelType', self, OutputPixelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GridImageSource_SetOutputPixelType`, 'returnType') = 'void'
attr(`GridImageSource_SetOutputPixelType`, "inputTypes") = c('_p_itk__simple__GridImageSource', 'character')
class(`GridImageSource_SetOutputPixelType`) = c("SWIGFunction", class('GridImageSource_SetOutputPixelType'))

# Start of GridImageSource_GetOutputPixelType

`GridImageSource_GetOutputPixelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GridImageSource_GetOutputPixelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`GridImageSource_GetOutputPixelType`, 'returnType') = 'character'
attr(`GridImageSource_GetOutputPixelType`, "inputTypes") = c('_p_itk__simple__GridImageSource')
class(`GridImageSource_GetOutputPixelType`) = c("SWIGFunction", class('GridImageSource_GetOutputPixelType'))

# Start of GridImageSource_SetSize

`GridImageSource_SetSize` = function(self, Size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Size = as.integer(Size);
  ;ans = .Call('R_swig_GridImageSource_SetSize', self, Size, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GridImageSource_SetSize`, 'returnType') = 'void'
attr(`GridImageSource_SetSize`, "inputTypes") = c('_p_itk__simple__GridImageSource', 'integer')
class(`GridImageSource_SetSize`) = c("SWIGFunction", class('GridImageSource_SetSize'))

# Start of GridImageSource_GetSize

`GridImageSource_GetSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GridImageSource_GetSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GridImageSource_GetSize`, 'returnType') = 'integer'
attr(`GridImageSource_GetSize`, "inputTypes") = c('_p_itk__simple__GridImageSource')
class(`GridImageSource_GetSize`) = c("SWIGFunction", class('GridImageSource_GetSize'))

# Start of GridImageSource_SetSigma

`GridImageSource_SetSigma__SWIG_0` = function(self, Sigma)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Sigma = as.numeric(Sigma);
  ;ans = .Call('R_swig_GridImageSource_SetSigma__SWIG_0', self, Sigma, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GridImageSource_SetSigma__SWIG_0`, 'returnType') = 'void'
attr(`GridImageSource_SetSigma__SWIG_0`, "inputTypes") = c('_p_itk__simple__GridImageSource', 'numeric')
class(`GridImageSource_SetSigma__SWIG_0`) = c("SWIGFunction", class('GridImageSource_SetSigma__SWIG_0'))

# Start of GridImageSource_SetSigma

`GridImageSource_SetSigma__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GridImageSource_SetSigma__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GridImageSource_SetSigma__SWIG_1`, 'returnType') = 'void'
attr(`GridImageSource_SetSigma__SWIG_1`, "inputTypes") = c('_p_itk__simple__GridImageSource', 'numeric')
class(`GridImageSource_SetSigma__SWIG_1`) = c("SWIGFunction", class('GridImageSource_SetSigma__SWIG_1'))

`GridImageSource_SetSigma` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__GridImageSource') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- GridImageSource_SetSigma__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__GridImageSource') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- GridImageSource_SetSigma__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GridImageSource_SetSigma with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of GridImageSource_GetSigma

`GridImageSource_GetSigma` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GridImageSource_GetSigma', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GridImageSource_GetSigma`, 'returnType') = 'numeric'
attr(`GridImageSource_GetSigma`, "inputTypes") = c('_p_itk__simple__GridImageSource')
class(`GridImageSource_GetSigma`) = c("SWIGFunction", class('GridImageSource_GetSigma'))

# Start of GridImageSource_SetGridSpacing

`GridImageSource_SetGridSpacing` = function(self, GridSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  GridSpacing = as.numeric(GridSpacing);
  ;ans = .Call('R_swig_GridImageSource_SetGridSpacing', self, GridSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GridImageSource_SetGridSpacing`, 'returnType') = 'void'
attr(`GridImageSource_SetGridSpacing`, "inputTypes") = c('_p_itk__simple__GridImageSource', 'numeric')
class(`GridImageSource_SetGridSpacing`) = c("SWIGFunction", class('GridImageSource_SetGridSpacing'))

# Start of GridImageSource_GetGridSpacing

`GridImageSource_GetGridSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GridImageSource_GetGridSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GridImageSource_GetGridSpacing`, 'returnType') = 'numeric'
attr(`GridImageSource_GetGridSpacing`, "inputTypes") = c('_p_itk__simple__GridImageSource')
class(`GridImageSource_GetGridSpacing`) = c("SWIGFunction", class('GridImageSource_GetGridSpacing'))

# Start of GridImageSource_SetGridOffset

`GridImageSource_SetGridOffset` = function(self, GridOffset)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  GridOffset = as.numeric(GridOffset);
  ;ans = .Call('R_swig_GridImageSource_SetGridOffset', self, GridOffset, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GridImageSource_SetGridOffset`, 'returnType') = 'void'
attr(`GridImageSource_SetGridOffset`, "inputTypes") = c('_p_itk__simple__GridImageSource', 'numeric')
class(`GridImageSource_SetGridOffset`) = c("SWIGFunction", class('GridImageSource_SetGridOffset'))

# Start of GridImageSource_GetGridOffset

`GridImageSource_GetGridOffset` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GridImageSource_GetGridOffset', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GridImageSource_GetGridOffset`, 'returnType') = 'numeric'
attr(`GridImageSource_GetGridOffset`, "inputTypes") = c('_p_itk__simple__GridImageSource')
class(`GridImageSource_GetGridOffset`) = c("SWIGFunction", class('GridImageSource_GetGridOffset'))

# Start of GridImageSource_SetScale

`GridImageSource_SetScale` = function(self, Scale)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_GridImageSource_SetScale', self, Scale, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GridImageSource_SetScale`, 'returnType') = 'void'
attr(`GridImageSource_SetScale`, "inputTypes") = c('_p_itk__simple__GridImageSource', 'numeric')
class(`GridImageSource_SetScale`) = c("SWIGFunction", class('GridImageSource_SetScale'))

# Start of GridImageSource_GetScale

`GridImageSource_GetScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GridImageSource_GetScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GridImageSource_GetScale`, 'returnType') = 'numeric'
attr(`GridImageSource_GetScale`, "inputTypes") = c('_p_itk__simple__GridImageSource')
class(`GridImageSource_GetScale`) = c("SWIGFunction", class('GridImageSource_GetScale'))

# Start of GridImageSource_SetOrigin

`GridImageSource_SetOrigin` = function(self, Origin)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Origin = as.numeric(Origin);
  ;ans = .Call('R_swig_GridImageSource_SetOrigin', self, Origin, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GridImageSource_SetOrigin`, 'returnType') = 'void'
attr(`GridImageSource_SetOrigin`, "inputTypes") = c('_p_itk__simple__GridImageSource', 'numeric')
class(`GridImageSource_SetOrigin`) = c("SWIGFunction", class('GridImageSource_SetOrigin'))

# Start of GridImageSource_GetOrigin

`GridImageSource_GetOrigin` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GridImageSource_GetOrigin', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GridImageSource_GetOrigin`, 'returnType') = 'numeric'
attr(`GridImageSource_GetOrigin`, "inputTypes") = c('_p_itk__simple__GridImageSource')
class(`GridImageSource_GetOrigin`) = c("SWIGFunction", class('GridImageSource_GetOrigin'))

# Start of GridImageSource_SetSpacing

`GridImageSource_SetSpacing` = function(self, Spacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Spacing = as.numeric(Spacing);
  ;ans = .Call('R_swig_GridImageSource_SetSpacing', self, Spacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GridImageSource_SetSpacing`, 'returnType') = 'void'
attr(`GridImageSource_SetSpacing`, "inputTypes") = c('_p_itk__simple__GridImageSource', 'numeric')
class(`GridImageSource_SetSpacing`) = c("SWIGFunction", class('GridImageSource_SetSpacing'))

# Start of GridImageSource_GetSpacing

`GridImageSource_GetSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GridImageSource_GetSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GridImageSource_GetSpacing`, 'returnType') = 'numeric'
attr(`GridImageSource_GetSpacing`, "inputTypes") = c('_p_itk__simple__GridImageSource')
class(`GridImageSource_GetSpacing`) = c("SWIGFunction", class('GridImageSource_GetSpacing'))

# Start of GridImageSource_SetDirection

`GridImageSource_SetDirection` = function(self, Direction)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Direction = as.numeric(Direction);
  ;ans = .Call('R_swig_GridImageSource_SetDirection', self, Direction, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GridImageSource_SetDirection`, 'returnType') = 'void'
attr(`GridImageSource_SetDirection`, "inputTypes") = c('_p_itk__simple__GridImageSource', 'numeric')
class(`GridImageSource_SetDirection`) = c("SWIGFunction", class('GridImageSource_SetDirection'))

# Start of GridImageSource_GetDirection

`GridImageSource_GetDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GridImageSource_GetDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GridImageSource_GetDirection`, 'returnType') = 'numeric'
attr(`GridImageSource_GetDirection`, "inputTypes") = c('_p_itk__simple__GridImageSource')
class(`GridImageSource_GetDirection`) = c("SWIGFunction", class('GridImageSource_GetDirection'))

# Start of GridImageSource_SetWhichDimensions

`GridImageSource_SetWhichDimensions` = function(self, WhichDimensions)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  WhichDimensions = as.logical(WhichDimensions);
  ;ans = .Call('R_swig_GridImageSource_SetWhichDimensions', self, WhichDimensions, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`GridImageSource_SetWhichDimensions`, 'returnType') = 'void'
attr(`GridImageSource_SetWhichDimensions`, "inputTypes") = c('_p_itk__simple__GridImageSource', 'logical')
class(`GridImageSource_SetWhichDimensions`) = c("SWIGFunction", class('GridImageSource_SetWhichDimensions'))

# Start of GridImageSource_GetWhichDimensions

`GridImageSource_GetWhichDimensions` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GridImageSource_GetWhichDimensions', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GridImageSource_GetWhichDimensions`, 'returnType') = 'logical'
attr(`GridImageSource_GetWhichDimensions`, "inputTypes") = c('_p_itk__simple__GridImageSource')
class(`GridImageSource_GetWhichDimensions`) = c("SWIGFunction", class('GridImageSource_GetWhichDimensions'))

# Start of GridImageSource_GetName

`GridImageSource_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GridImageSource_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GridImageSource_GetName`, 'returnType') = 'character'
attr(`GridImageSource_GetName`, "inputTypes") = c('_p_itk__simple__GridImageSource')
class(`GridImageSource_GetName`) = c("SWIGFunction", class('GridImageSource_GetName'))

# Start of GridImageSource_ToString

`GridImageSource_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_GridImageSource_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`GridImageSource_ToString`, 'returnType') = 'character'
attr(`GridImageSource_ToString`, "inputTypes") = c('_p_itk__simple__GridImageSource')
class(`GridImageSource_ToString`) = c("SWIGFunction", class('GridImageSource_ToString'))

# Start of GridImageSource_Execute

`GridImageSource_Execute` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_GridImageSource_Execute', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GridImageSource_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`GridImageSource_Execute`, "inputTypes") = c('_p_itk__simple__GridImageSource')
class(`GridImageSource_Execute`) = c("SWIGFunction", class('GridImageSource_Execute'))

# Start of accessor method for itk::simple::GridImageSource
setMethod('$', '_p_itk__simple__GridImageSource', function(x, name)

{
  accessorFuns = list('SetOutputPixelType' = GridImageSource_SetOutputPixelType, 'GetOutputPixelType' = GridImageSource_GetOutputPixelType, 'SetSize' = GridImageSource_SetSize, 'GetSize' = GridImageSource_GetSize, 'SetSigma' = GridImageSource_SetSigma, 'GetSigma' = GridImageSource_GetSigma, 'SetGridSpacing' = GridImageSource_SetGridSpacing, 'GetGridSpacing' = GridImageSource_GetGridSpacing, 'SetGridOffset' = GridImageSource_SetGridOffset, 'GetGridOffset' = GridImageSource_GetGridOffset, 'SetScale' = GridImageSource_SetScale, 'GetScale' = GridImageSource_GetScale, 'SetOrigin' = GridImageSource_SetOrigin, 'GetOrigin' = GridImageSource_GetOrigin, 'SetSpacing' = GridImageSource_SetSpacing, 'GetSpacing' = GridImageSource_GetSpacing, 'SetDirection' = GridImageSource_SetDirection, 'GetDirection' = GridImageSource_GetDirection, 'SetWhichDimensions' = GridImageSource_SetWhichDimensions, 'GetWhichDimensions' = GridImageSource_GetWhichDimensions, 'GetName' = GridImageSource_GetName, 'ToString' = GridImageSource_ToString, 'Execute' = GridImageSource_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::GridImageSource
# Start of GridSource

`GridSource__SWIG_0` = function(outputPixelType, size, sigma, gridSpacing, gridOffset, scale, origin, spacing, direction, whichDimensions, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  gridSpacing = as.numeric(gridSpacing);
  gridOffset = as.numeric(gridOffset);
  
  origin = as.numeric(origin);
  spacing = as.numeric(spacing);
  direction = as.numeric(direction);
  whichDimensions = as.logical(whichDimensions);
  ;ans = .Call('R_swig_GridSource__SWIG_0', outputPixelType, size, sigma, gridSpacing, gridOffset, scale, origin, spacing, direction, whichDimensions, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GridSource__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`GridSource__SWIG_0`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'logical')
class(`GridSource__SWIG_0`) = c("SWIGFunction", class('GridSource__SWIG_0'))

# Start of GridSource

`GridSource__SWIG_1` = function(outputPixelType, size, sigma, gridSpacing, gridOffset, scale, origin, spacing, direction, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  gridSpacing = as.numeric(gridSpacing);
  gridOffset = as.numeric(gridOffset);
  
  origin = as.numeric(origin);
  spacing = as.numeric(spacing);
  direction = as.numeric(direction);
  ;ans = .Call('R_swig_GridSource__SWIG_1', outputPixelType, size, sigma, gridSpacing, gridOffset, scale, origin, spacing, direction, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GridSource__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`GridSource__SWIG_1`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
class(`GridSource__SWIG_1`) = c("SWIGFunction", class('GridSource__SWIG_1'))

# Start of GridSource

`GridSource__SWIG_2` = function(outputPixelType, size, sigma, gridSpacing, gridOffset, scale, origin, spacing, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  gridSpacing = as.numeric(gridSpacing);
  gridOffset = as.numeric(gridOffset);
  
  origin = as.numeric(origin);
  spacing = as.numeric(spacing);
  ;ans = .Call('R_swig_GridSource__SWIG_2', outputPixelType, size, sigma, gridSpacing, gridOffset, scale, origin, spacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GridSource__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`GridSource__SWIG_2`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
class(`GridSource__SWIG_2`) = c("SWIGFunction", class('GridSource__SWIG_2'))

# Start of GridSource

`GridSource__SWIG_3` = function(outputPixelType, size, sigma, gridSpacing, gridOffset, scale, origin, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  gridSpacing = as.numeric(gridSpacing);
  gridOffset = as.numeric(gridOffset);
  
  origin = as.numeric(origin);
  ;ans = .Call('R_swig_GridSource__SWIG_3', outputPixelType, size, sigma, gridSpacing, gridOffset, scale, origin, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GridSource__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`GridSource__SWIG_3`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
class(`GridSource__SWIG_3`) = c("SWIGFunction", class('GridSource__SWIG_3'))

# Start of GridSource

`GridSource__SWIG_4` = function(outputPixelType, size, sigma, gridSpacing, gridOffset, scale, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  gridSpacing = as.numeric(gridSpacing);
  gridOffset = as.numeric(gridOffset);
  
  ;ans = .Call('R_swig_GridSource__SWIG_4', outputPixelType, size, sigma, gridSpacing, gridOffset, scale, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GridSource__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`GridSource__SWIG_4`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric', 'numeric')
class(`GridSource__SWIG_4`) = c("SWIGFunction", class('GridSource__SWIG_4'))

# Start of GridSource

`GridSource__SWIG_5` = function(outputPixelType, size, sigma, gridSpacing, gridOffset, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  gridSpacing = as.numeric(gridSpacing);
  gridOffset = as.numeric(gridOffset);
  ;ans = .Call('R_swig_GridSource__SWIG_5', outputPixelType, size, sigma, gridSpacing, gridOffset, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GridSource__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`GridSource__SWIG_5`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric')
class(`GridSource__SWIG_5`) = c("SWIGFunction", class('GridSource__SWIG_5'))

# Start of GridSource

`GridSource__SWIG_6` = function(outputPixelType, size, sigma, gridSpacing, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  gridSpacing = as.numeric(gridSpacing);
  ;ans = .Call('R_swig_GridSource__SWIG_6', outputPixelType, size, sigma, gridSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GridSource__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`GridSource__SWIG_6`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric')
class(`GridSource__SWIG_6`) = c("SWIGFunction", class('GridSource__SWIG_6'))

# Start of GridSource

`GridSource__SWIG_7` = function(outputPixelType, size, sigma, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  sigma = as.numeric(sigma);
  ;ans = .Call('R_swig_GridSource__SWIG_7', outputPixelType, size, sigma, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GridSource__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`GridSource__SWIG_7`, "inputTypes") = c('character', 'integer', 'numeric')
class(`GridSource__SWIG_7`) = c("SWIGFunction", class('GridSource__SWIG_7'))

# Start of GridSource

`GridSource__SWIG_8` = function(outputPixelType, size, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  ;ans = .Call('R_swig_GridSource__SWIG_8', outputPixelType, size, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GridSource__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`GridSource__SWIG_8`, "inputTypes") = c('character', 'integer')
class(`GridSource__SWIG_8`) = c("SWIGFunction", class('GridSource__SWIG_8'))

# Start of GridSource

`GridSource__SWIG_9` = function(outputPixelType, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  ;ans = .Call('R_swig_GridSource__SWIG_9', outputPixelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GridSource__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`GridSource__SWIG_9`, "inputTypes") = c('character')
class(`GridSource__SWIG_9`) = c("SWIGFunction", class('GridSource__SWIG_9'))

# Start of GridSource

`GridSource__SWIG_10` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_GridSource__SWIG_10', as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`GridSource__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
class(`GridSource__SWIG_10`) = c("SWIGFunction", class('GridSource__SWIG_10'))

`GridSource` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 11
  if (argc == 0) {
    f <- GridSource__SWIG_10; 
  } else if (argc == 1) {
    if ( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) {
      f <- GridSource__SWIG_9; 
    }
  } else if (argc == 2) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- GridSource__SWIG_8; 
    }
  } else if (argc == 3) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) )) {
      f <- GridSource__SWIG_7; 
    }
  } else if (argc == 4) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) )) {
      f <- GridSource__SWIG_6; 
    }
  } else if (argc == 5) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) )) {
      f <- GridSource__SWIG_5; 
    }
  } else if (argc == 6) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- GridSource__SWIG_4; 
    }
  } else if (argc == 7) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) )) {
      f <- GridSource__SWIG_3; 
    }
  } else if (argc == 8) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) ) && ( is.numeric(argv[[8]]) )) {
      f <- GridSource__SWIG_2; 
    }
  } else if (argc == 9) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) ) && ( is.numeric(argv[[8]]) ) && ( is.numeric(argv[[9]]) )) {
      f <- GridSource__SWIG_1; 
    }
  } else if (argc == 10) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) ) && ( is.numeric(argv[[8]]) ) && ( is.numeric(argv[[9]]) ) && ( is.logical(argv[[10]]) )) {
      f <- GridSource__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for GridSource with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_HConcaveImageFilter

`delete_HConcaveImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_HConcaveImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_HConcaveImageFilter`, 'returnType') = 'void'
attr(`delete_HConcaveImageFilter`, "inputTypes") = c('_p_itk__simple__HConcaveImageFilter')
class(`delete_HConcaveImageFilter`) = c("SWIGFunction", class('delete_HConcaveImageFilter'))

# Start of new_HConcaveImageFilter

`HConcaveImageFilter` = function()
{
  ;ans = .Call('R_swig_new_HConcaveImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__HConcaveImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_HConcaveImageFilter);
  ans
  
}

attr(`HConcaveImageFilter`, 'returnType') = '_p_itk__simple__HConcaveImageFilter'
class(`HConcaveImageFilter`) = c("SWIGFunction", class('HConcaveImageFilter'))

# Start of HConcaveImageFilter_SetHeight

`HConcaveImageFilter_SetHeight` = function(self, Height)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_HConcaveImageFilter_SetHeight', self, Height, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HConcaveImageFilter_SetHeight`, 'returnType') = 'void'
attr(`HConcaveImageFilter_SetHeight`, "inputTypes") = c('_p_itk__simple__HConcaveImageFilter', 'numeric')
class(`HConcaveImageFilter_SetHeight`) = c("SWIGFunction", class('HConcaveImageFilter_SetHeight'))

# Start of HConcaveImageFilter_GetHeight

`HConcaveImageFilter_GetHeight` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HConcaveImageFilter_GetHeight', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HConcaveImageFilter_GetHeight`, 'returnType') = 'numeric'
attr(`HConcaveImageFilter_GetHeight`, "inputTypes") = c('_p_itk__simple__HConcaveImageFilter')
class(`HConcaveImageFilter_GetHeight`) = c("SWIGFunction", class('HConcaveImageFilter_GetHeight'))

# Start of HConcaveImageFilter_SetFullyConnected

`HConcaveImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_HConcaveImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HConcaveImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`HConcaveImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__HConcaveImageFilter', 'logical')
class(`HConcaveImageFilter_SetFullyConnected`) = c("SWIGFunction", class('HConcaveImageFilter_SetFullyConnected'))

# Start of HConcaveImageFilter_FullyConnectedOn

`HConcaveImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_HConcaveImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HConcaveImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`HConcaveImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__HConcaveImageFilter')
class(`HConcaveImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('HConcaveImageFilter_FullyConnectedOn'))

# Start of HConcaveImageFilter_FullyConnectedOff

`HConcaveImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_HConcaveImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HConcaveImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`HConcaveImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__HConcaveImageFilter')
class(`HConcaveImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('HConcaveImageFilter_FullyConnectedOff'))

# Start of HConcaveImageFilter_GetFullyConnected

`HConcaveImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HConcaveImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HConcaveImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`HConcaveImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__HConcaveImageFilter')
class(`HConcaveImageFilter_GetFullyConnected`) = c("SWIGFunction", class('HConcaveImageFilter_GetFullyConnected'))

# Start of HConcaveImageFilter_GetName

`HConcaveImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HConcaveImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HConcaveImageFilter_GetName`, 'returnType') = 'character'
attr(`HConcaveImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__HConcaveImageFilter')
class(`HConcaveImageFilter_GetName`) = c("SWIGFunction", class('HConcaveImageFilter_GetName'))

# Start of HConcaveImageFilter_ToString

`HConcaveImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HConcaveImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HConcaveImageFilter_ToString`, 'returnType') = 'character'
attr(`HConcaveImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__HConcaveImageFilter')
class(`HConcaveImageFilter_ToString`) = c("SWIGFunction", class('HConcaveImageFilter_ToString'))

# Start of HConcaveImageFilter_Execute

`HConcaveImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_HConcaveImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HConcaveImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`HConcaveImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__HConcaveImageFilter', '_p_itk__simple__Image')
class(`HConcaveImageFilter_Execute`) = c("SWIGFunction", class('HConcaveImageFilter_Execute'))

# Start of accessor method for itk::simple::HConcaveImageFilter
setMethod('$', '_p_itk__simple__HConcaveImageFilter', function(x, name)

{
  accessorFuns = list('SetHeight' = HConcaveImageFilter_SetHeight, 'GetHeight' = HConcaveImageFilter_GetHeight, 'SetFullyConnected' = HConcaveImageFilter_SetFullyConnected, 'FullyConnectedOn' = HConcaveImageFilter_FullyConnectedOn, 'FullyConnectedOff' = HConcaveImageFilter_FullyConnectedOff, 'GetFullyConnected' = HConcaveImageFilter_GetFullyConnected, 'GetName' = HConcaveImageFilter_GetName, 'ToString' = HConcaveImageFilter_ToString, 'Execute' = HConcaveImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::HConcaveImageFilter
# Start of HConcave

`HConcave__SWIG_0` = function(image1, height, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_HConcave__SWIG_0', image1, height, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HConcave__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`HConcave__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`HConcave__SWIG_0`) = c("SWIGFunction", class('HConcave__SWIG_0'))

# Start of HConcave

`HConcave__SWIG_1` = function(image1, height, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_HConcave__SWIG_1', image1, height, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HConcave__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`HConcave__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`HConcave__SWIG_1`) = c("SWIGFunction", class('HConcave__SWIG_1'))

# Start of HConcave

`HConcave__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_HConcave__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HConcave__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`HConcave__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`HConcave__SWIG_2`) = c("SWIGFunction", class('HConcave__SWIG_2'))

`HConcave` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- HConcave__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- HConcave__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- HConcave__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for HConcave with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_HConvexImageFilter

`delete_HConvexImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_HConvexImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_HConvexImageFilter`, 'returnType') = 'void'
attr(`delete_HConvexImageFilter`, "inputTypes") = c('_p_itk__simple__HConvexImageFilter')
class(`delete_HConvexImageFilter`) = c("SWIGFunction", class('delete_HConvexImageFilter'))

# Start of new_HConvexImageFilter

`HConvexImageFilter` = function()
{
  ;ans = .Call('R_swig_new_HConvexImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__HConvexImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_HConvexImageFilter);
  ans
  
}

attr(`HConvexImageFilter`, 'returnType') = '_p_itk__simple__HConvexImageFilter'
class(`HConvexImageFilter`) = c("SWIGFunction", class('HConvexImageFilter'))

# Start of HConvexImageFilter_SetHeight

`HConvexImageFilter_SetHeight` = function(self, Height)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_HConvexImageFilter_SetHeight', self, Height, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HConvexImageFilter_SetHeight`, 'returnType') = 'void'
attr(`HConvexImageFilter_SetHeight`, "inputTypes") = c('_p_itk__simple__HConvexImageFilter', 'numeric')
class(`HConvexImageFilter_SetHeight`) = c("SWIGFunction", class('HConvexImageFilter_SetHeight'))

# Start of HConvexImageFilter_GetHeight

`HConvexImageFilter_GetHeight` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HConvexImageFilter_GetHeight', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HConvexImageFilter_GetHeight`, 'returnType') = 'numeric'
attr(`HConvexImageFilter_GetHeight`, "inputTypes") = c('_p_itk__simple__HConvexImageFilter')
class(`HConvexImageFilter_GetHeight`) = c("SWIGFunction", class('HConvexImageFilter_GetHeight'))

# Start of HConvexImageFilter_SetFullyConnected

`HConvexImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_HConvexImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HConvexImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`HConvexImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__HConvexImageFilter', 'logical')
class(`HConvexImageFilter_SetFullyConnected`) = c("SWIGFunction", class('HConvexImageFilter_SetFullyConnected'))

# Start of HConvexImageFilter_FullyConnectedOn

`HConvexImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_HConvexImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HConvexImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`HConvexImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__HConvexImageFilter')
class(`HConvexImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('HConvexImageFilter_FullyConnectedOn'))

# Start of HConvexImageFilter_FullyConnectedOff

`HConvexImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_HConvexImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HConvexImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`HConvexImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__HConvexImageFilter')
class(`HConvexImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('HConvexImageFilter_FullyConnectedOff'))

# Start of HConvexImageFilter_GetFullyConnected

`HConvexImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HConvexImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HConvexImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`HConvexImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__HConvexImageFilter')
class(`HConvexImageFilter_GetFullyConnected`) = c("SWIGFunction", class('HConvexImageFilter_GetFullyConnected'))

# Start of HConvexImageFilter_GetName

`HConvexImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HConvexImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HConvexImageFilter_GetName`, 'returnType') = 'character'
attr(`HConvexImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__HConvexImageFilter')
class(`HConvexImageFilter_GetName`) = c("SWIGFunction", class('HConvexImageFilter_GetName'))

# Start of HConvexImageFilter_ToString

`HConvexImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HConvexImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HConvexImageFilter_ToString`, 'returnType') = 'character'
attr(`HConvexImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__HConvexImageFilter')
class(`HConvexImageFilter_ToString`) = c("SWIGFunction", class('HConvexImageFilter_ToString'))

# Start of HConvexImageFilter_Execute

`HConvexImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_HConvexImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HConvexImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`HConvexImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__HConvexImageFilter', '_p_itk__simple__Image')
class(`HConvexImageFilter_Execute`) = c("SWIGFunction", class('HConvexImageFilter_Execute'))

# Start of accessor method for itk::simple::HConvexImageFilter
setMethod('$', '_p_itk__simple__HConvexImageFilter', function(x, name)

{
  accessorFuns = list('SetHeight' = HConvexImageFilter_SetHeight, 'GetHeight' = HConvexImageFilter_GetHeight, 'SetFullyConnected' = HConvexImageFilter_SetFullyConnected, 'FullyConnectedOn' = HConvexImageFilter_FullyConnectedOn, 'FullyConnectedOff' = HConvexImageFilter_FullyConnectedOff, 'GetFullyConnected' = HConvexImageFilter_GetFullyConnected, 'GetName' = HConvexImageFilter_GetName, 'ToString' = HConvexImageFilter_ToString, 'Execute' = HConvexImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::HConvexImageFilter
# Start of HConvex

`HConvex__SWIG_0` = function(image1, height, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_HConvex__SWIG_0', image1, height, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HConvex__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`HConvex__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`HConvex__SWIG_0`) = c("SWIGFunction", class('HConvex__SWIG_0'))

# Start of HConvex

`HConvex__SWIG_1` = function(image1, height, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_HConvex__SWIG_1', image1, height, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HConvex__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`HConvex__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`HConvex__SWIG_1`) = c("SWIGFunction", class('HConvex__SWIG_1'))

# Start of HConvex

`HConvex__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_HConvex__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HConvex__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`HConvex__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`HConvex__SWIG_2`) = c("SWIGFunction", class('HConvex__SWIG_2'))

`HConvex` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- HConvex__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- HConvex__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- HConvex__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for HConvex with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_HMaximaImageFilter

`delete_HMaximaImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_HMaximaImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_HMaximaImageFilter`, 'returnType') = 'void'
attr(`delete_HMaximaImageFilter`, "inputTypes") = c('_p_itk__simple__HMaximaImageFilter')
class(`delete_HMaximaImageFilter`) = c("SWIGFunction", class('delete_HMaximaImageFilter'))

# Start of new_HMaximaImageFilter

`HMaximaImageFilter` = function()
{
  ;ans = .Call('R_swig_new_HMaximaImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__HMaximaImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_HMaximaImageFilter);
  ans
  
}

attr(`HMaximaImageFilter`, 'returnType') = '_p_itk__simple__HMaximaImageFilter'
class(`HMaximaImageFilter`) = c("SWIGFunction", class('HMaximaImageFilter'))

# Start of HMaximaImageFilter_SetHeight

`HMaximaImageFilter_SetHeight` = function(self, Height)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_HMaximaImageFilter_SetHeight', self, Height, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HMaximaImageFilter_SetHeight`, 'returnType') = 'void'
attr(`HMaximaImageFilter_SetHeight`, "inputTypes") = c('_p_itk__simple__HMaximaImageFilter', 'numeric')
class(`HMaximaImageFilter_SetHeight`) = c("SWIGFunction", class('HMaximaImageFilter_SetHeight'))

# Start of HMaximaImageFilter_GetHeight

`HMaximaImageFilter_GetHeight` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HMaximaImageFilter_GetHeight', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HMaximaImageFilter_GetHeight`, 'returnType') = 'numeric'
attr(`HMaximaImageFilter_GetHeight`, "inputTypes") = c('_p_itk__simple__HMaximaImageFilter')
class(`HMaximaImageFilter_GetHeight`) = c("SWIGFunction", class('HMaximaImageFilter_GetHeight'))

# Start of HMaximaImageFilter_GetName

`HMaximaImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HMaximaImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HMaximaImageFilter_GetName`, 'returnType') = 'character'
attr(`HMaximaImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__HMaximaImageFilter')
class(`HMaximaImageFilter_GetName`) = c("SWIGFunction", class('HMaximaImageFilter_GetName'))

# Start of HMaximaImageFilter_ToString

`HMaximaImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HMaximaImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HMaximaImageFilter_ToString`, 'returnType') = 'character'
attr(`HMaximaImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__HMaximaImageFilter')
class(`HMaximaImageFilter_ToString`) = c("SWIGFunction", class('HMaximaImageFilter_ToString'))

# Start of HMaximaImageFilter_Execute

`HMaximaImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_HMaximaImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HMaximaImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`HMaximaImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__HMaximaImageFilter', '_p_itk__simple__Image')
class(`HMaximaImageFilter_Execute`) = c("SWIGFunction", class('HMaximaImageFilter_Execute'))

# Start of accessor method for itk::simple::HMaximaImageFilter
setMethod('$', '_p_itk__simple__HMaximaImageFilter', function(x, name)

{
  accessorFuns = list('SetHeight' = HMaximaImageFilter_SetHeight, 'GetHeight' = HMaximaImageFilter_GetHeight, 'GetName' = HMaximaImageFilter_GetName, 'ToString' = HMaximaImageFilter_ToString, 'Execute' = HMaximaImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::HMaximaImageFilter
# Start of HMaxima

`HMaxima__SWIG_0` = function(image1, height, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_HMaxima__SWIG_0', image1, height, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HMaxima__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`HMaxima__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`HMaxima__SWIG_0`) = c("SWIGFunction", class('HMaxima__SWIG_0'))

# Start of HMaxima

`HMaxima__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_HMaxima__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HMaxima__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`HMaxima__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`HMaxima__SWIG_1`) = c("SWIGFunction", class('HMaxima__SWIG_1'))

`HMaxima` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- HMaxima__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- HMaxima__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for HMaxima with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_HMinimaImageFilter

`delete_HMinimaImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_HMinimaImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_HMinimaImageFilter`, 'returnType') = 'void'
attr(`delete_HMinimaImageFilter`, "inputTypes") = c('_p_itk__simple__HMinimaImageFilter')
class(`delete_HMinimaImageFilter`) = c("SWIGFunction", class('delete_HMinimaImageFilter'))

# Start of new_HMinimaImageFilter

`HMinimaImageFilter` = function()
{
  ;ans = .Call('R_swig_new_HMinimaImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__HMinimaImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_HMinimaImageFilter);
  ans
  
}

attr(`HMinimaImageFilter`, 'returnType') = '_p_itk__simple__HMinimaImageFilter'
class(`HMinimaImageFilter`) = c("SWIGFunction", class('HMinimaImageFilter'))

# Start of HMinimaImageFilter_SetHeight

`HMinimaImageFilter_SetHeight` = function(self, Height)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_HMinimaImageFilter_SetHeight', self, Height, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HMinimaImageFilter_SetHeight`, 'returnType') = 'void'
attr(`HMinimaImageFilter_SetHeight`, "inputTypes") = c('_p_itk__simple__HMinimaImageFilter', 'numeric')
class(`HMinimaImageFilter_SetHeight`) = c("SWIGFunction", class('HMinimaImageFilter_SetHeight'))

# Start of HMinimaImageFilter_GetHeight

`HMinimaImageFilter_GetHeight` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HMinimaImageFilter_GetHeight', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HMinimaImageFilter_GetHeight`, 'returnType') = 'numeric'
attr(`HMinimaImageFilter_GetHeight`, "inputTypes") = c('_p_itk__simple__HMinimaImageFilter')
class(`HMinimaImageFilter_GetHeight`) = c("SWIGFunction", class('HMinimaImageFilter_GetHeight'))

# Start of HMinimaImageFilter_SetFullyConnected

`HMinimaImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_HMinimaImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HMinimaImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`HMinimaImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__HMinimaImageFilter', 'logical')
class(`HMinimaImageFilter_SetFullyConnected`) = c("SWIGFunction", class('HMinimaImageFilter_SetFullyConnected'))

# Start of HMinimaImageFilter_FullyConnectedOn

`HMinimaImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_HMinimaImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HMinimaImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`HMinimaImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__HMinimaImageFilter')
class(`HMinimaImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('HMinimaImageFilter_FullyConnectedOn'))

# Start of HMinimaImageFilter_FullyConnectedOff

`HMinimaImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_HMinimaImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HMinimaImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`HMinimaImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__HMinimaImageFilter')
class(`HMinimaImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('HMinimaImageFilter_FullyConnectedOff'))

# Start of HMinimaImageFilter_GetFullyConnected

`HMinimaImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HMinimaImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HMinimaImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`HMinimaImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__HMinimaImageFilter')
class(`HMinimaImageFilter_GetFullyConnected`) = c("SWIGFunction", class('HMinimaImageFilter_GetFullyConnected'))

# Start of HMinimaImageFilter_GetName

`HMinimaImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HMinimaImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HMinimaImageFilter_GetName`, 'returnType') = 'character'
attr(`HMinimaImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__HMinimaImageFilter')
class(`HMinimaImageFilter_GetName`) = c("SWIGFunction", class('HMinimaImageFilter_GetName'))

# Start of HMinimaImageFilter_ToString

`HMinimaImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HMinimaImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HMinimaImageFilter_ToString`, 'returnType') = 'character'
attr(`HMinimaImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__HMinimaImageFilter')
class(`HMinimaImageFilter_ToString`) = c("SWIGFunction", class('HMinimaImageFilter_ToString'))

# Start of HMinimaImageFilter_Execute

`HMinimaImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_HMinimaImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HMinimaImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`HMinimaImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__HMinimaImageFilter', '_p_itk__simple__Image')
class(`HMinimaImageFilter_Execute`) = c("SWIGFunction", class('HMinimaImageFilter_Execute'))

# Start of accessor method for itk::simple::HMinimaImageFilter
setMethod('$', '_p_itk__simple__HMinimaImageFilter', function(x, name)

{
  accessorFuns = list('SetHeight' = HMinimaImageFilter_SetHeight, 'GetHeight' = HMinimaImageFilter_GetHeight, 'SetFullyConnected' = HMinimaImageFilter_SetFullyConnected, 'FullyConnectedOn' = HMinimaImageFilter_FullyConnectedOn, 'FullyConnectedOff' = HMinimaImageFilter_FullyConnectedOff, 'GetFullyConnected' = HMinimaImageFilter_GetFullyConnected, 'GetName' = HMinimaImageFilter_GetName, 'ToString' = HMinimaImageFilter_ToString, 'Execute' = HMinimaImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::HMinimaImageFilter
# Start of HMinima

`HMinima__SWIG_0` = function(image1, height, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_HMinima__SWIG_0', image1, height, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HMinima__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`HMinima__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`HMinima__SWIG_0`) = c("SWIGFunction", class('HMinima__SWIG_0'))

# Start of HMinima

`HMinima__SWIG_1` = function(image1, height, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_HMinima__SWIG_1', image1, height, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HMinima__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`HMinima__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`HMinima__SWIG_1`) = c("SWIGFunction", class('HMinima__SWIG_1'))

# Start of HMinima

`HMinima__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_HMinima__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HMinima__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`HMinima__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`HMinima__SWIG_2`) = c("SWIGFunction", class('HMinima__SWIG_2'))

`HMinima` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- HMinima__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- HMinima__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- HMinima__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for HMinima with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_HalfHermitianToRealInverseFFTImageFilter

`delete_HalfHermitianToRealInverseFFTImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_HalfHermitianToRealInverseFFTImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_HalfHermitianToRealInverseFFTImageFilter`, 'returnType') = 'void'
attr(`delete_HalfHermitianToRealInverseFFTImageFilter`, "inputTypes") = c('_p_itk__simple__HalfHermitianToRealInverseFFTImageFilter')
class(`delete_HalfHermitianToRealInverseFFTImageFilter`) = c("SWIGFunction", class('delete_HalfHermitianToRealInverseFFTImageFilter'))

# Start of new_HalfHermitianToRealInverseFFTImageFilter

`HalfHermitianToRealInverseFFTImageFilter` = function()
{
  ;ans = .Call('R_swig_new_HalfHermitianToRealInverseFFTImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__HalfHermitianToRealInverseFFTImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_HalfHermitianToRealInverseFFTImageFilter);
  ans
  
}

attr(`HalfHermitianToRealInverseFFTImageFilter`, 'returnType') = '_p_itk__simple__HalfHermitianToRealInverseFFTImageFilter'
class(`HalfHermitianToRealInverseFFTImageFilter`) = c("SWIGFunction", class('HalfHermitianToRealInverseFFTImageFilter'))

# Start of HalfHermitianToRealInverseFFTImageFilter_SetActualXDimensionIsOdd

`HalfHermitianToRealInverseFFTImageFilter_SetActualXDimensionIsOdd` = function(self, ActualXDimensionIsOdd)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ActualXDimensionIsOdd = as.logical(ActualXDimensionIsOdd);
  ;ans = .Call('R_swig_HalfHermitianToRealInverseFFTImageFilter_SetActualXDimensionIsOdd', self, ActualXDimensionIsOdd, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HalfHermitianToRealInverseFFTImageFilter_SetActualXDimensionIsOdd`, 'returnType') = 'void'
attr(`HalfHermitianToRealInverseFFTImageFilter_SetActualXDimensionIsOdd`, "inputTypes") = c('_p_itk__simple__HalfHermitianToRealInverseFFTImageFilter', 'logical')
class(`HalfHermitianToRealInverseFFTImageFilter_SetActualXDimensionIsOdd`) = c("SWIGFunction", class('HalfHermitianToRealInverseFFTImageFilter_SetActualXDimensionIsOdd'))

# Start of HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOn

`HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOn`, 'returnType') = 'void'
attr(`HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOn`, "inputTypes") = c('_p_itk__simple__HalfHermitianToRealInverseFFTImageFilter')
class(`HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOn`) = c("SWIGFunction", class('HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOn'))

# Start of HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOff

`HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOff`, 'returnType') = 'void'
attr(`HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOff`, "inputTypes") = c('_p_itk__simple__HalfHermitianToRealInverseFFTImageFilter')
class(`HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOff`) = c("SWIGFunction", class('HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOff'))

# Start of HalfHermitianToRealInverseFFTImageFilter_GetActualXDimensionIsOdd

`HalfHermitianToRealInverseFFTImageFilter_GetActualXDimensionIsOdd` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HalfHermitianToRealInverseFFTImageFilter_GetActualXDimensionIsOdd', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HalfHermitianToRealInverseFFTImageFilter_GetActualXDimensionIsOdd`, 'returnType') = 'logical'
attr(`HalfHermitianToRealInverseFFTImageFilter_GetActualXDimensionIsOdd`, "inputTypes") = c('_p_itk__simple__HalfHermitianToRealInverseFFTImageFilter')
class(`HalfHermitianToRealInverseFFTImageFilter_GetActualXDimensionIsOdd`) = c("SWIGFunction", class('HalfHermitianToRealInverseFFTImageFilter_GetActualXDimensionIsOdd'))

# Start of HalfHermitianToRealInverseFFTImageFilter_GetName

`HalfHermitianToRealInverseFFTImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HalfHermitianToRealInverseFFTImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HalfHermitianToRealInverseFFTImageFilter_GetName`, 'returnType') = 'character'
attr(`HalfHermitianToRealInverseFFTImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__HalfHermitianToRealInverseFFTImageFilter')
class(`HalfHermitianToRealInverseFFTImageFilter_GetName`) = c("SWIGFunction", class('HalfHermitianToRealInverseFFTImageFilter_GetName'))

# Start of HalfHermitianToRealInverseFFTImageFilter_ToString

`HalfHermitianToRealInverseFFTImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HalfHermitianToRealInverseFFTImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HalfHermitianToRealInverseFFTImageFilter_ToString`, 'returnType') = 'character'
attr(`HalfHermitianToRealInverseFFTImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__HalfHermitianToRealInverseFFTImageFilter')
class(`HalfHermitianToRealInverseFFTImageFilter_ToString`) = c("SWIGFunction", class('HalfHermitianToRealInverseFFTImageFilter_ToString'))

# Start of HalfHermitianToRealInverseFFTImageFilter_Execute

`HalfHermitianToRealInverseFFTImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_HalfHermitianToRealInverseFFTImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HalfHermitianToRealInverseFFTImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`HalfHermitianToRealInverseFFTImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__HalfHermitianToRealInverseFFTImageFilter', '_p_itk__simple__Image')
class(`HalfHermitianToRealInverseFFTImageFilter_Execute`) = c("SWIGFunction", class('HalfHermitianToRealInverseFFTImageFilter_Execute'))

# Start of accessor method for itk::simple::HalfHermitianToRealInverseFFTImageFilter
setMethod('$', '_p_itk__simple__HalfHermitianToRealInverseFFTImageFilter', function(x, name)

{
  accessorFuns = list('SetActualXDimensionIsOdd' = HalfHermitianToRealInverseFFTImageFilter_SetActualXDimensionIsOdd, 'ActualXDimensionIsOddOn' = HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOn, 'ActualXDimensionIsOddOff' = HalfHermitianToRealInverseFFTImageFilter_ActualXDimensionIsOddOff, 'GetActualXDimensionIsOdd' = HalfHermitianToRealInverseFFTImageFilter_GetActualXDimensionIsOdd, 'GetName' = HalfHermitianToRealInverseFFTImageFilter_GetName, 'ToString' = HalfHermitianToRealInverseFFTImageFilter_ToString, 'Execute' = HalfHermitianToRealInverseFFTImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::HalfHermitianToRealInverseFFTImageFilter
# Start of HalfHermitianToRealInverseFFT

`HalfHermitianToRealInverseFFT__SWIG_0` = function(image1, actualXDimensionIsOdd, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  actualXDimensionIsOdd = as.logical(actualXDimensionIsOdd);
  ;ans = .Call('R_swig_HalfHermitianToRealInverseFFT__SWIG_0', image1, actualXDimensionIsOdd, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HalfHermitianToRealInverseFFT__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`HalfHermitianToRealInverseFFT__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`HalfHermitianToRealInverseFFT__SWIG_0`) = c("SWIGFunction", class('HalfHermitianToRealInverseFFT__SWIG_0'))

# Start of HalfHermitianToRealInverseFFT

`HalfHermitianToRealInverseFFT__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_HalfHermitianToRealInverseFFT__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HalfHermitianToRealInverseFFT__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`HalfHermitianToRealInverseFFT__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`HalfHermitianToRealInverseFFT__SWIG_1`) = c("SWIGFunction", class('HalfHermitianToRealInverseFFT__SWIG_1'))

`HalfHermitianToRealInverseFFT` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- HalfHermitianToRealInverseFFT__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- HalfHermitianToRealInverseFFT__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for HalfHermitianToRealInverseFFT with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_HausdorffDistanceImageFilter

`delete_HausdorffDistanceImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_HausdorffDistanceImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_HausdorffDistanceImageFilter`, 'returnType') = 'void'
attr(`delete_HausdorffDistanceImageFilter`, "inputTypes") = c('_p_itk__simple__HausdorffDistanceImageFilter')
class(`delete_HausdorffDistanceImageFilter`) = c("SWIGFunction", class('delete_HausdorffDistanceImageFilter'))

# Start of new_HausdorffDistanceImageFilter

`HausdorffDistanceImageFilter` = function()
{
  ;ans = .Call('R_swig_new_HausdorffDistanceImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__HausdorffDistanceImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_HausdorffDistanceImageFilter);
  ans
  
}

attr(`HausdorffDistanceImageFilter`, 'returnType') = '_p_itk__simple__HausdorffDistanceImageFilter'
class(`HausdorffDistanceImageFilter`) = c("SWIGFunction", class('HausdorffDistanceImageFilter'))

# Start of HausdorffDistanceImageFilter_GetHausdorffDistance

`HausdorffDistanceImageFilter_GetHausdorffDistance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HausdorffDistanceImageFilter_GetHausdorffDistance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HausdorffDistanceImageFilter_GetHausdorffDistance`, 'returnType') = 'numeric'
attr(`HausdorffDistanceImageFilter_GetHausdorffDistance`, "inputTypes") = c('_p_itk__simple__HausdorffDistanceImageFilter')
class(`HausdorffDistanceImageFilter_GetHausdorffDistance`) = c("SWIGFunction", class('HausdorffDistanceImageFilter_GetHausdorffDistance'))

# Start of HausdorffDistanceImageFilter_GetAverageHausdorffDistance

`HausdorffDistanceImageFilter_GetAverageHausdorffDistance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HausdorffDistanceImageFilter_GetAverageHausdorffDistance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HausdorffDistanceImageFilter_GetAverageHausdorffDistance`, 'returnType') = 'numeric'
attr(`HausdorffDistanceImageFilter_GetAverageHausdorffDistance`, "inputTypes") = c('_p_itk__simple__HausdorffDistanceImageFilter')
class(`HausdorffDistanceImageFilter_GetAverageHausdorffDistance`) = c("SWIGFunction", class('HausdorffDistanceImageFilter_GetAverageHausdorffDistance'))

# Start of HausdorffDistanceImageFilter_GetName

`HausdorffDistanceImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HausdorffDistanceImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HausdorffDistanceImageFilter_GetName`, 'returnType') = 'character'
attr(`HausdorffDistanceImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__HausdorffDistanceImageFilter')
class(`HausdorffDistanceImageFilter_GetName`) = c("SWIGFunction", class('HausdorffDistanceImageFilter_GetName'))

# Start of HausdorffDistanceImageFilter_ToString

`HausdorffDistanceImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HausdorffDistanceImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HausdorffDistanceImageFilter_ToString`, 'returnType') = 'character'
attr(`HausdorffDistanceImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__HausdorffDistanceImageFilter')
class(`HausdorffDistanceImageFilter_ToString`) = c("SWIGFunction", class('HausdorffDistanceImageFilter_ToString'))

# Start of HausdorffDistanceImageFilter_Execute

`HausdorffDistanceImageFilter_Execute` = function(self, image1, image2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_HausdorffDistanceImageFilter_Execute', self, image1, image2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HausdorffDistanceImageFilter_Execute`, 'returnType') = 'void'
attr(`HausdorffDistanceImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__HausdorffDistanceImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`HausdorffDistanceImageFilter_Execute`) = c("SWIGFunction", class('HausdorffDistanceImageFilter_Execute'))

# Start of accessor method for itk::simple::HausdorffDistanceImageFilter
setMethod('$', '_p_itk__simple__HausdorffDistanceImageFilter', function(x, name)

{
  accessorFuns = list('GetHausdorffDistance' = HausdorffDistanceImageFilter_GetHausdorffDistance, 'GetAverageHausdorffDistance' = HausdorffDistanceImageFilter_GetAverageHausdorffDistance, 'GetName' = HausdorffDistanceImageFilter_GetName, 'ToString' = HausdorffDistanceImageFilter_ToString, 'Execute' = HausdorffDistanceImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::HausdorffDistanceImageFilter
# Start of delete_HistogramMatchingImageFilter

`delete_HistogramMatchingImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_HistogramMatchingImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_HistogramMatchingImageFilter`, 'returnType') = 'void'
attr(`delete_HistogramMatchingImageFilter`, "inputTypes") = c('_p_itk__simple__HistogramMatchingImageFilter')
class(`delete_HistogramMatchingImageFilter`) = c("SWIGFunction", class('delete_HistogramMatchingImageFilter'))

# Start of new_HistogramMatchingImageFilter

`HistogramMatchingImageFilter` = function()
{
  ;ans = .Call('R_swig_new_HistogramMatchingImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__HistogramMatchingImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_HistogramMatchingImageFilter);
  ans
  
}

attr(`HistogramMatchingImageFilter`, 'returnType') = '_p_itk__simple__HistogramMatchingImageFilter'
class(`HistogramMatchingImageFilter`) = c("SWIGFunction", class('HistogramMatchingImageFilter'))

# Start of HistogramMatchingImageFilter_SetNumberOfHistogramLevels

`HistogramMatchingImageFilter_SetNumberOfHistogramLevels` = function(self, NumberOfHistogramLevels)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfHistogramLevels = as.integer(NumberOfHistogramLevels);
  
  if(length(NumberOfHistogramLevels) > 1) {
    warning("using only the first element of NumberOfHistogramLevels");
  };
  
  ;ans = .Call('R_swig_HistogramMatchingImageFilter_SetNumberOfHistogramLevels', self, NumberOfHistogramLevels, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HistogramMatchingImageFilter_SetNumberOfHistogramLevels`, 'returnType') = 'void'
attr(`HistogramMatchingImageFilter_SetNumberOfHistogramLevels`, "inputTypes") = c('_p_itk__simple__HistogramMatchingImageFilter', 'integer')
class(`HistogramMatchingImageFilter_SetNumberOfHistogramLevels`) = c("SWIGFunction", class('HistogramMatchingImageFilter_SetNumberOfHistogramLevels'))

# Start of HistogramMatchingImageFilter_GetNumberOfHistogramLevels

`HistogramMatchingImageFilter_GetNumberOfHistogramLevels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HistogramMatchingImageFilter_GetNumberOfHistogramLevels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HistogramMatchingImageFilter_GetNumberOfHistogramLevels`, 'returnType') = 'integer'
attr(`HistogramMatchingImageFilter_GetNumberOfHistogramLevels`, "inputTypes") = c('_p_itk__simple__HistogramMatchingImageFilter')
class(`HistogramMatchingImageFilter_GetNumberOfHistogramLevels`) = c("SWIGFunction", class('HistogramMatchingImageFilter_GetNumberOfHistogramLevels'))

# Start of HistogramMatchingImageFilter_SetNumberOfMatchPoints

`HistogramMatchingImageFilter_SetNumberOfMatchPoints` = function(self, NumberOfMatchPoints)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfMatchPoints = as.integer(NumberOfMatchPoints);
  
  if(length(NumberOfMatchPoints) > 1) {
    warning("using only the first element of NumberOfMatchPoints");
  };
  
  ;ans = .Call('R_swig_HistogramMatchingImageFilter_SetNumberOfMatchPoints', self, NumberOfMatchPoints, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HistogramMatchingImageFilter_SetNumberOfMatchPoints`, 'returnType') = 'void'
attr(`HistogramMatchingImageFilter_SetNumberOfMatchPoints`, "inputTypes") = c('_p_itk__simple__HistogramMatchingImageFilter', 'integer')
class(`HistogramMatchingImageFilter_SetNumberOfMatchPoints`) = c("SWIGFunction", class('HistogramMatchingImageFilter_SetNumberOfMatchPoints'))

# Start of HistogramMatchingImageFilter_GetNumberOfMatchPoints

`HistogramMatchingImageFilter_GetNumberOfMatchPoints` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HistogramMatchingImageFilter_GetNumberOfMatchPoints', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HistogramMatchingImageFilter_GetNumberOfMatchPoints`, 'returnType') = 'integer'
attr(`HistogramMatchingImageFilter_GetNumberOfMatchPoints`, "inputTypes") = c('_p_itk__simple__HistogramMatchingImageFilter')
class(`HistogramMatchingImageFilter_GetNumberOfMatchPoints`) = c("SWIGFunction", class('HistogramMatchingImageFilter_GetNumberOfMatchPoints'))

# Start of HistogramMatchingImageFilter_SetThresholdAtMeanIntensity

`HistogramMatchingImageFilter_SetThresholdAtMeanIntensity` = function(self, ThresholdAtMeanIntensity)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ThresholdAtMeanIntensity = as.logical(ThresholdAtMeanIntensity);
  ;ans = .Call('R_swig_HistogramMatchingImageFilter_SetThresholdAtMeanIntensity', self, ThresholdAtMeanIntensity, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HistogramMatchingImageFilter_SetThresholdAtMeanIntensity`, 'returnType') = 'void'
attr(`HistogramMatchingImageFilter_SetThresholdAtMeanIntensity`, "inputTypes") = c('_p_itk__simple__HistogramMatchingImageFilter', 'logical')
class(`HistogramMatchingImageFilter_SetThresholdAtMeanIntensity`) = c("SWIGFunction", class('HistogramMatchingImageFilter_SetThresholdAtMeanIntensity'))

# Start of HistogramMatchingImageFilter_ThresholdAtMeanIntensityOn

`HistogramMatchingImageFilter_ThresholdAtMeanIntensityOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_HistogramMatchingImageFilter_ThresholdAtMeanIntensityOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HistogramMatchingImageFilter_ThresholdAtMeanIntensityOn`, 'returnType') = 'void'
attr(`HistogramMatchingImageFilter_ThresholdAtMeanIntensityOn`, "inputTypes") = c('_p_itk__simple__HistogramMatchingImageFilter')
class(`HistogramMatchingImageFilter_ThresholdAtMeanIntensityOn`) = c("SWIGFunction", class('HistogramMatchingImageFilter_ThresholdAtMeanIntensityOn'))

# Start of HistogramMatchingImageFilter_ThresholdAtMeanIntensityOff

`HistogramMatchingImageFilter_ThresholdAtMeanIntensityOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_HistogramMatchingImageFilter_ThresholdAtMeanIntensityOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HistogramMatchingImageFilter_ThresholdAtMeanIntensityOff`, 'returnType') = 'void'
attr(`HistogramMatchingImageFilter_ThresholdAtMeanIntensityOff`, "inputTypes") = c('_p_itk__simple__HistogramMatchingImageFilter')
class(`HistogramMatchingImageFilter_ThresholdAtMeanIntensityOff`) = c("SWIGFunction", class('HistogramMatchingImageFilter_ThresholdAtMeanIntensityOff'))

# Start of HistogramMatchingImageFilter_GetThresholdAtMeanIntensity

`HistogramMatchingImageFilter_GetThresholdAtMeanIntensity` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HistogramMatchingImageFilter_GetThresholdAtMeanIntensity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HistogramMatchingImageFilter_GetThresholdAtMeanIntensity`, 'returnType') = 'logical'
attr(`HistogramMatchingImageFilter_GetThresholdAtMeanIntensity`, "inputTypes") = c('_p_itk__simple__HistogramMatchingImageFilter')
class(`HistogramMatchingImageFilter_GetThresholdAtMeanIntensity`) = c("SWIGFunction", class('HistogramMatchingImageFilter_GetThresholdAtMeanIntensity'))

# Start of HistogramMatchingImageFilter_GetName

`HistogramMatchingImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HistogramMatchingImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HistogramMatchingImageFilter_GetName`, 'returnType') = 'character'
attr(`HistogramMatchingImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__HistogramMatchingImageFilter')
class(`HistogramMatchingImageFilter_GetName`) = c("SWIGFunction", class('HistogramMatchingImageFilter_GetName'))

# Start of HistogramMatchingImageFilter_ToString

`HistogramMatchingImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HistogramMatchingImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HistogramMatchingImageFilter_ToString`, 'returnType') = 'character'
attr(`HistogramMatchingImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__HistogramMatchingImageFilter')
class(`HistogramMatchingImageFilter_ToString`) = c("SWIGFunction", class('HistogramMatchingImageFilter_ToString'))

# Start of HistogramMatchingImageFilter_Execute

`HistogramMatchingImageFilter_Execute` = function(self, image, referenceImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(referenceImage, "ExternalReference")) referenceImage = slot(referenceImage,"ref"); 
  ;ans = .Call('R_swig_HistogramMatchingImageFilter_Execute', self, image, referenceImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HistogramMatchingImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`HistogramMatchingImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__HistogramMatchingImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`HistogramMatchingImageFilter_Execute`) = c("SWIGFunction", class('HistogramMatchingImageFilter_Execute'))

# Start of accessor method for itk::simple::HistogramMatchingImageFilter
setMethod('$', '_p_itk__simple__HistogramMatchingImageFilter', function(x, name)

{
  accessorFuns = list('SetNumberOfHistogramLevels' = HistogramMatchingImageFilter_SetNumberOfHistogramLevels, 'GetNumberOfHistogramLevels' = HistogramMatchingImageFilter_GetNumberOfHistogramLevels, 'SetNumberOfMatchPoints' = HistogramMatchingImageFilter_SetNumberOfMatchPoints, 'GetNumberOfMatchPoints' = HistogramMatchingImageFilter_GetNumberOfMatchPoints, 'SetThresholdAtMeanIntensity' = HistogramMatchingImageFilter_SetThresholdAtMeanIntensity, 'ThresholdAtMeanIntensityOn' = HistogramMatchingImageFilter_ThresholdAtMeanIntensityOn, 'ThresholdAtMeanIntensityOff' = HistogramMatchingImageFilter_ThresholdAtMeanIntensityOff, 'GetThresholdAtMeanIntensity' = HistogramMatchingImageFilter_GetThresholdAtMeanIntensity, 'GetName' = HistogramMatchingImageFilter_GetName, 'ToString' = HistogramMatchingImageFilter_ToString, 'Execute' = HistogramMatchingImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::HistogramMatchingImageFilter
# Start of HistogramMatching

`HistogramMatching__SWIG_0` = function(image, referenceImage, numberOfHistogramLevels, numberOfMatchPoints, thresholdAtMeanIntensity, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(referenceImage, "ExternalReference")) referenceImage = slot(referenceImage,"ref"); 
  numberOfHistogramLevels = as.integer(numberOfHistogramLevels);
  
  if(length(numberOfHistogramLevels) > 1) {
    warning("using only the first element of numberOfHistogramLevels");
  };
  
  numberOfMatchPoints = as.integer(numberOfMatchPoints);
  
  if(length(numberOfMatchPoints) > 1) {
    warning("using only the first element of numberOfMatchPoints");
  };
  
  thresholdAtMeanIntensity = as.logical(thresholdAtMeanIntensity);
  ;ans = .Call('R_swig_HistogramMatching__SWIG_0', image, referenceImage, numberOfHistogramLevels, numberOfMatchPoints, thresholdAtMeanIntensity, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HistogramMatching__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`HistogramMatching__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'logical')
class(`HistogramMatching__SWIG_0`) = c("SWIGFunction", class('HistogramMatching__SWIG_0'))

# Start of HistogramMatching

`HistogramMatching__SWIG_1` = function(image, referenceImage, numberOfHistogramLevels, numberOfMatchPoints, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(referenceImage, "ExternalReference")) referenceImage = slot(referenceImage,"ref"); 
  numberOfHistogramLevels = as.integer(numberOfHistogramLevels);
  
  if(length(numberOfHistogramLevels) > 1) {
    warning("using only the first element of numberOfHistogramLevels");
  };
  
  numberOfMatchPoints = as.integer(numberOfMatchPoints);
  
  if(length(numberOfMatchPoints) > 1) {
    warning("using only the first element of numberOfMatchPoints");
  };
  
  ;ans = .Call('R_swig_HistogramMatching__SWIG_1', image, referenceImage, numberOfHistogramLevels, numberOfMatchPoints, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HistogramMatching__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`HistogramMatching__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`HistogramMatching__SWIG_1`) = c("SWIGFunction", class('HistogramMatching__SWIG_1'))

# Start of HistogramMatching

`HistogramMatching__SWIG_2` = function(image, referenceImage, numberOfHistogramLevels, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(referenceImage, "ExternalReference")) referenceImage = slot(referenceImage,"ref"); 
  numberOfHistogramLevels = as.integer(numberOfHistogramLevels);
  
  if(length(numberOfHistogramLevels) > 1) {
    warning("using only the first element of numberOfHistogramLevels");
  };
  
  ;ans = .Call('R_swig_HistogramMatching__SWIG_2', image, referenceImage, numberOfHistogramLevels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HistogramMatching__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`HistogramMatching__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`HistogramMatching__SWIG_2`) = c("SWIGFunction", class('HistogramMatching__SWIG_2'))

# Start of HistogramMatching

`HistogramMatching__SWIG_3` = function(image, referenceImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(referenceImage, "ExternalReference")) referenceImage = slot(referenceImage,"ref"); 
  ;ans = .Call('R_swig_HistogramMatching__SWIG_3', image, referenceImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HistogramMatching__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`HistogramMatching__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`HistogramMatching__SWIG_3`) = c("SWIGFunction", class('HistogramMatching__SWIG_3'))

`HistogramMatching` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- HistogramMatching__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- HistogramMatching__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- HistogramMatching__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- HistogramMatching__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for HistogramMatching with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_HuangThresholdImageFilter

`delete_HuangThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_HuangThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_HuangThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_HuangThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter')
class(`delete_HuangThresholdImageFilter`) = c("SWIGFunction", class('delete_HuangThresholdImageFilter'))

# Start of new_HuangThresholdImageFilter

`HuangThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_HuangThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__HuangThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_HuangThresholdImageFilter);
  ans
  
}

attr(`HuangThresholdImageFilter`, 'returnType') = '_p_itk__simple__HuangThresholdImageFilter'
class(`HuangThresholdImageFilter`) = c("SWIGFunction", class('HuangThresholdImageFilter'))

# Start of HuangThresholdImageFilter_SetInsideValue

`HuangThresholdImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideValue = as.integer(InsideValue);
  
  if(length(InsideValue) > 1) {
    warning("using only the first element of InsideValue");
  };
  
  ;ans = .Call('R_swig_HuangThresholdImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HuangThresholdImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`HuangThresholdImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter', 'integer')
class(`HuangThresholdImageFilter_SetInsideValue`) = c("SWIGFunction", class('HuangThresholdImageFilter_SetInsideValue'))

# Start of HuangThresholdImageFilter_GetInsideValue

`HuangThresholdImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HuangThresholdImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HuangThresholdImageFilter_GetInsideValue`, 'returnType') = 'integer'
attr(`HuangThresholdImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter')
class(`HuangThresholdImageFilter_GetInsideValue`) = c("SWIGFunction", class('HuangThresholdImageFilter_GetInsideValue'))

# Start of HuangThresholdImageFilter_SetOutsideValue

`HuangThresholdImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutsideValue = as.integer(OutsideValue);
  
  if(length(OutsideValue) > 1) {
    warning("using only the first element of OutsideValue");
  };
  
  ;ans = .Call('R_swig_HuangThresholdImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HuangThresholdImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`HuangThresholdImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter', 'integer')
class(`HuangThresholdImageFilter_SetOutsideValue`) = c("SWIGFunction", class('HuangThresholdImageFilter_SetOutsideValue'))

# Start of HuangThresholdImageFilter_GetOutsideValue

`HuangThresholdImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HuangThresholdImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HuangThresholdImageFilter_GetOutsideValue`, 'returnType') = 'integer'
attr(`HuangThresholdImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter')
class(`HuangThresholdImageFilter_GetOutsideValue`) = c("SWIGFunction", class('HuangThresholdImageFilter_GetOutsideValue'))

# Start of HuangThresholdImageFilter_SetNumberOfHistogramBins

`HuangThresholdImageFilter_SetNumberOfHistogramBins` = function(self, NumberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfHistogramBins = as.integer(NumberOfHistogramBins);
  
  if(length(NumberOfHistogramBins) > 1) {
    warning("using only the first element of NumberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_HuangThresholdImageFilter_SetNumberOfHistogramBins', self, NumberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HuangThresholdImageFilter_SetNumberOfHistogramBins`, 'returnType') = 'void'
attr(`HuangThresholdImageFilter_SetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter', 'integer')
class(`HuangThresholdImageFilter_SetNumberOfHistogramBins`) = c("SWIGFunction", class('HuangThresholdImageFilter_SetNumberOfHistogramBins'))

# Start of HuangThresholdImageFilter_GetNumberOfHistogramBins

`HuangThresholdImageFilter_GetNumberOfHistogramBins` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HuangThresholdImageFilter_GetNumberOfHistogramBins', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HuangThresholdImageFilter_GetNumberOfHistogramBins`, 'returnType') = 'integer'
attr(`HuangThresholdImageFilter_GetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter')
class(`HuangThresholdImageFilter_GetNumberOfHistogramBins`) = c("SWIGFunction", class('HuangThresholdImageFilter_GetNumberOfHistogramBins'))

# Start of HuangThresholdImageFilter_SetMaskOutput

`HuangThresholdImageFilter_SetMaskOutput` = function(self, MaskOutput)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskOutput = as.logical(MaskOutput);
  ;ans = .Call('R_swig_HuangThresholdImageFilter_SetMaskOutput', self, MaskOutput, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HuangThresholdImageFilter_SetMaskOutput`, 'returnType') = 'void'
attr(`HuangThresholdImageFilter_SetMaskOutput`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter', 'logical')
class(`HuangThresholdImageFilter_SetMaskOutput`) = c("SWIGFunction", class('HuangThresholdImageFilter_SetMaskOutput'))

# Start of HuangThresholdImageFilter_MaskOutputOn

`HuangThresholdImageFilter_MaskOutputOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_HuangThresholdImageFilter_MaskOutputOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HuangThresholdImageFilter_MaskOutputOn`, 'returnType') = 'void'
attr(`HuangThresholdImageFilter_MaskOutputOn`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter')
class(`HuangThresholdImageFilter_MaskOutputOn`) = c("SWIGFunction", class('HuangThresholdImageFilter_MaskOutputOn'))

# Start of HuangThresholdImageFilter_MaskOutputOff

`HuangThresholdImageFilter_MaskOutputOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_HuangThresholdImageFilter_MaskOutputOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HuangThresholdImageFilter_MaskOutputOff`, 'returnType') = 'void'
attr(`HuangThresholdImageFilter_MaskOutputOff`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter')
class(`HuangThresholdImageFilter_MaskOutputOff`) = c("SWIGFunction", class('HuangThresholdImageFilter_MaskOutputOff'))

# Start of HuangThresholdImageFilter_GetMaskOutput

`HuangThresholdImageFilter_GetMaskOutput` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HuangThresholdImageFilter_GetMaskOutput', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HuangThresholdImageFilter_GetMaskOutput`, 'returnType') = 'logical'
attr(`HuangThresholdImageFilter_GetMaskOutput`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter')
class(`HuangThresholdImageFilter_GetMaskOutput`) = c("SWIGFunction", class('HuangThresholdImageFilter_GetMaskOutput'))

# Start of HuangThresholdImageFilter_SetMaskValue

`HuangThresholdImageFilter_SetMaskValue` = function(self, MaskValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskValue = as.integer(MaskValue);
  
  if(length(MaskValue) > 1) {
    warning("using only the first element of MaskValue");
  };
  
  ;ans = .Call('R_swig_HuangThresholdImageFilter_SetMaskValue', self, MaskValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`HuangThresholdImageFilter_SetMaskValue`, 'returnType') = 'void'
attr(`HuangThresholdImageFilter_SetMaskValue`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter', 'integer')
class(`HuangThresholdImageFilter_SetMaskValue`) = c("SWIGFunction", class('HuangThresholdImageFilter_SetMaskValue'))

# Start of HuangThresholdImageFilter_GetMaskValue

`HuangThresholdImageFilter_GetMaskValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HuangThresholdImageFilter_GetMaskValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HuangThresholdImageFilter_GetMaskValue`, 'returnType') = 'integer'
attr(`HuangThresholdImageFilter_GetMaskValue`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter')
class(`HuangThresholdImageFilter_GetMaskValue`) = c("SWIGFunction", class('HuangThresholdImageFilter_GetMaskValue'))

# Start of HuangThresholdImageFilter_GetThreshold

`HuangThresholdImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HuangThresholdImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HuangThresholdImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`HuangThresholdImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter')
class(`HuangThresholdImageFilter_GetThreshold`) = c("SWIGFunction", class('HuangThresholdImageFilter_GetThreshold'))

# Start of HuangThresholdImageFilter_GetName

`HuangThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HuangThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HuangThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`HuangThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter')
class(`HuangThresholdImageFilter_GetName`) = c("SWIGFunction", class('HuangThresholdImageFilter_GetName'))

# Start of HuangThresholdImageFilter_ToString

`HuangThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_HuangThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`HuangThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`HuangThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter')
class(`HuangThresholdImageFilter_ToString`) = c("SWIGFunction", class('HuangThresholdImageFilter_ToString'))

# Start of HuangThresholdImageFilter_Execute

`HuangThresholdImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_HuangThresholdImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HuangThresholdImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`HuangThresholdImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`HuangThresholdImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('HuangThresholdImageFilter_Execute__SWIG_0'))

# Start of HuangThresholdImageFilter_Execute

`HuangThresholdImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_HuangThresholdImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HuangThresholdImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`HuangThresholdImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__HuangThresholdImageFilter', '_p_itk__simple__Image')
class(`HuangThresholdImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('HuangThresholdImageFilter_Execute__SWIG_1'))

`HuangThresholdImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__HuangThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- HuangThresholdImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__HuangThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- HuangThresholdImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for HuangThresholdImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::HuangThresholdImageFilter
setMethod('$', '_p_itk__simple__HuangThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetInsideValue' = HuangThresholdImageFilter_SetInsideValue, 'GetInsideValue' = HuangThresholdImageFilter_GetInsideValue, 'SetOutsideValue' = HuangThresholdImageFilter_SetOutsideValue, 'GetOutsideValue' = HuangThresholdImageFilter_GetOutsideValue, 'SetNumberOfHistogramBins' = HuangThresholdImageFilter_SetNumberOfHistogramBins, 'GetNumberOfHistogramBins' = HuangThresholdImageFilter_GetNumberOfHistogramBins, 'SetMaskOutput' = HuangThresholdImageFilter_SetMaskOutput, 'MaskOutputOn' = HuangThresholdImageFilter_MaskOutputOn, 'MaskOutputOff' = HuangThresholdImageFilter_MaskOutputOff, 'GetMaskOutput' = HuangThresholdImageFilter_GetMaskOutput, 'SetMaskValue' = HuangThresholdImageFilter_SetMaskValue, 'GetMaskValue' = HuangThresholdImageFilter_GetMaskValue, 'GetThreshold' = HuangThresholdImageFilter_GetThreshold, 'GetName' = HuangThresholdImageFilter_GetName, 'ToString' = HuangThresholdImageFilter_ToString, 'Execute' = HuangThresholdImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::HuangThresholdImageFilter
# Start of HuangThreshold

`HuangThreshold__SWIG_0` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_HuangThreshold__SWIG_0', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HuangThreshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`HuangThreshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`HuangThreshold__SWIG_0`) = c("SWIGFunction", class('HuangThreshold__SWIG_0'))

# Start of HuangThreshold

`HuangThreshold__SWIG_1` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_HuangThreshold__SWIG_1', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HuangThreshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`HuangThreshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`HuangThreshold__SWIG_1`) = c("SWIGFunction", class('HuangThreshold__SWIG_1'))

# Start of HuangThreshold

`HuangThreshold__SWIG_2` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_HuangThreshold__SWIG_2', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HuangThreshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`HuangThreshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`HuangThreshold__SWIG_2`) = c("SWIGFunction", class('HuangThreshold__SWIG_2'))

# Start of HuangThreshold

`HuangThreshold__SWIG_3` = function(image, maskImage, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_HuangThreshold__SWIG_3', image, maskImage, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HuangThreshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`HuangThreshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`HuangThreshold__SWIG_3`) = c("SWIGFunction", class('HuangThreshold__SWIG_3'))

# Start of HuangThreshold

`HuangThreshold__SWIG_4` = function(image, maskImage, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_HuangThreshold__SWIG_4', image, maskImage, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HuangThreshold__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`HuangThreshold__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`HuangThreshold__SWIG_4`) = c("SWIGFunction", class('HuangThreshold__SWIG_4'))

# Start of HuangThreshold

`HuangThreshold__SWIG_5` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_HuangThreshold__SWIG_5', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HuangThreshold__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`HuangThreshold__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`HuangThreshold__SWIG_5`) = c("SWIGFunction", class('HuangThreshold__SWIG_5'))

# Start of HuangThreshold

`HuangThreshold__SWIG_6` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_HuangThreshold__SWIG_6', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HuangThreshold__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`HuangThreshold__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`HuangThreshold__SWIG_6`) = c("SWIGFunction", class('HuangThreshold__SWIG_6'))

# Start of HuangThreshold

`HuangThreshold__SWIG_7` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_HuangThreshold__SWIG_7', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HuangThreshold__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`HuangThreshold__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`HuangThreshold__SWIG_7`) = c("SWIGFunction", class('HuangThreshold__SWIG_7'))

# Start of HuangThreshold

`HuangThreshold__SWIG_8` = function(image, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_HuangThreshold__SWIG_8', image, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HuangThreshold__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`HuangThreshold__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`HuangThreshold__SWIG_8`) = c("SWIGFunction", class('HuangThreshold__SWIG_8'))

# Start of HuangThreshold

`HuangThreshold__SWIG_9` = function(image, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_HuangThreshold__SWIG_9', image, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HuangThreshold__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`HuangThreshold__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`HuangThreshold__SWIG_9`) = c("SWIGFunction", class('HuangThreshold__SWIG_9'))

# Start of HuangThreshold

`HuangThreshold__SWIG_10` = function(image, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_HuangThreshold__SWIG_10', image, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HuangThreshold__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`HuangThreshold__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`HuangThreshold__SWIG_10`) = c("SWIGFunction", class('HuangThreshold__SWIG_10'))

# Start of HuangThreshold

`HuangThreshold__SWIG_11` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_HuangThreshold__SWIG_11', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`HuangThreshold__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`HuangThreshold__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image')
class(`HuangThreshold__SWIG_11`) = c("SWIGFunction", class('HuangThreshold__SWIG_11'))

`HuangThreshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 12
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- HuangThreshold__SWIG_11; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- HuangThreshold__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- HuangThreshold__SWIG_10; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- HuangThreshold__SWIG_4; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- HuangThreshold__SWIG_9; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- HuangThreshold__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- HuangThreshold__SWIG_8; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- HuangThreshold__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- HuangThreshold__SWIG_7; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- HuangThreshold__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- HuangThreshold__SWIG_6; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- HuangThreshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for HuangThreshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_IntensityWindowingImageFilter

`delete_IntensityWindowingImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_IntensityWindowingImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_IntensityWindowingImageFilter`, 'returnType') = 'void'
attr(`delete_IntensityWindowingImageFilter`, "inputTypes") = c('_p_itk__simple__IntensityWindowingImageFilter')
class(`delete_IntensityWindowingImageFilter`) = c("SWIGFunction", class('delete_IntensityWindowingImageFilter'))

# Start of new_IntensityWindowingImageFilter

`IntensityWindowingImageFilter` = function()
{
  ;ans = .Call('R_swig_new_IntensityWindowingImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__IntensityWindowingImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_IntensityWindowingImageFilter);
  ans
  
}

attr(`IntensityWindowingImageFilter`, 'returnType') = '_p_itk__simple__IntensityWindowingImageFilter'
class(`IntensityWindowingImageFilter`) = c("SWIGFunction", class('IntensityWindowingImageFilter'))

# Start of IntensityWindowingImageFilter_SetWindowMinimum

`IntensityWindowingImageFilter_SetWindowMinimum` = function(self, WindowMinimum)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_IntensityWindowingImageFilter_SetWindowMinimum', self, WindowMinimum, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IntensityWindowingImageFilter_SetWindowMinimum`, 'returnType') = 'void'
attr(`IntensityWindowingImageFilter_SetWindowMinimum`, "inputTypes") = c('_p_itk__simple__IntensityWindowingImageFilter', 'numeric')
class(`IntensityWindowingImageFilter_SetWindowMinimum`) = c("SWIGFunction", class('IntensityWindowingImageFilter_SetWindowMinimum'))

# Start of IntensityWindowingImageFilter_GetWindowMinimum

`IntensityWindowingImageFilter_GetWindowMinimum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IntensityWindowingImageFilter_GetWindowMinimum', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IntensityWindowingImageFilter_GetWindowMinimum`, 'returnType') = 'numeric'
attr(`IntensityWindowingImageFilter_GetWindowMinimum`, "inputTypes") = c('_p_itk__simple__IntensityWindowingImageFilter')
class(`IntensityWindowingImageFilter_GetWindowMinimum`) = c("SWIGFunction", class('IntensityWindowingImageFilter_GetWindowMinimum'))

# Start of IntensityWindowingImageFilter_SetWindowMaximum

`IntensityWindowingImageFilter_SetWindowMaximum` = function(self, WindowMaximum)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_IntensityWindowingImageFilter_SetWindowMaximum', self, WindowMaximum, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IntensityWindowingImageFilter_SetWindowMaximum`, 'returnType') = 'void'
attr(`IntensityWindowingImageFilter_SetWindowMaximum`, "inputTypes") = c('_p_itk__simple__IntensityWindowingImageFilter', 'numeric')
class(`IntensityWindowingImageFilter_SetWindowMaximum`) = c("SWIGFunction", class('IntensityWindowingImageFilter_SetWindowMaximum'))

# Start of IntensityWindowingImageFilter_GetWindowMaximum

`IntensityWindowingImageFilter_GetWindowMaximum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IntensityWindowingImageFilter_GetWindowMaximum', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IntensityWindowingImageFilter_GetWindowMaximum`, 'returnType') = 'numeric'
attr(`IntensityWindowingImageFilter_GetWindowMaximum`, "inputTypes") = c('_p_itk__simple__IntensityWindowingImageFilter')
class(`IntensityWindowingImageFilter_GetWindowMaximum`) = c("SWIGFunction", class('IntensityWindowingImageFilter_GetWindowMaximum'))

# Start of IntensityWindowingImageFilter_SetOutputMinimum

`IntensityWindowingImageFilter_SetOutputMinimum` = function(self, OutputMinimum)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_IntensityWindowingImageFilter_SetOutputMinimum', self, OutputMinimum, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IntensityWindowingImageFilter_SetOutputMinimum`, 'returnType') = 'void'
attr(`IntensityWindowingImageFilter_SetOutputMinimum`, "inputTypes") = c('_p_itk__simple__IntensityWindowingImageFilter', 'numeric')
class(`IntensityWindowingImageFilter_SetOutputMinimum`) = c("SWIGFunction", class('IntensityWindowingImageFilter_SetOutputMinimum'))

# Start of IntensityWindowingImageFilter_GetOutputMinimum

`IntensityWindowingImageFilter_GetOutputMinimum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IntensityWindowingImageFilter_GetOutputMinimum', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IntensityWindowingImageFilter_GetOutputMinimum`, 'returnType') = 'numeric'
attr(`IntensityWindowingImageFilter_GetOutputMinimum`, "inputTypes") = c('_p_itk__simple__IntensityWindowingImageFilter')
class(`IntensityWindowingImageFilter_GetOutputMinimum`) = c("SWIGFunction", class('IntensityWindowingImageFilter_GetOutputMinimum'))

# Start of IntensityWindowingImageFilter_SetOutputMaximum

`IntensityWindowingImageFilter_SetOutputMaximum` = function(self, OutputMaximum)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_IntensityWindowingImageFilter_SetOutputMaximum', self, OutputMaximum, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IntensityWindowingImageFilter_SetOutputMaximum`, 'returnType') = 'void'
attr(`IntensityWindowingImageFilter_SetOutputMaximum`, "inputTypes") = c('_p_itk__simple__IntensityWindowingImageFilter', 'numeric')
class(`IntensityWindowingImageFilter_SetOutputMaximum`) = c("SWIGFunction", class('IntensityWindowingImageFilter_SetOutputMaximum'))

# Start of IntensityWindowingImageFilter_GetOutputMaximum

`IntensityWindowingImageFilter_GetOutputMaximum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IntensityWindowingImageFilter_GetOutputMaximum', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IntensityWindowingImageFilter_GetOutputMaximum`, 'returnType') = 'numeric'
attr(`IntensityWindowingImageFilter_GetOutputMaximum`, "inputTypes") = c('_p_itk__simple__IntensityWindowingImageFilter')
class(`IntensityWindowingImageFilter_GetOutputMaximum`) = c("SWIGFunction", class('IntensityWindowingImageFilter_GetOutputMaximum'))

# Start of IntensityWindowingImageFilter_GetName

`IntensityWindowingImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IntensityWindowingImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IntensityWindowingImageFilter_GetName`, 'returnType') = 'character'
attr(`IntensityWindowingImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__IntensityWindowingImageFilter')
class(`IntensityWindowingImageFilter_GetName`) = c("SWIGFunction", class('IntensityWindowingImageFilter_GetName'))

# Start of IntensityWindowingImageFilter_ToString

`IntensityWindowingImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IntensityWindowingImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IntensityWindowingImageFilter_ToString`, 'returnType') = 'character'
attr(`IntensityWindowingImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__IntensityWindowingImageFilter')
class(`IntensityWindowingImageFilter_ToString`) = c("SWIGFunction", class('IntensityWindowingImageFilter_ToString'))

# Start of IntensityWindowingImageFilter_Execute

`IntensityWindowingImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_IntensityWindowingImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntensityWindowingImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`IntensityWindowingImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__IntensityWindowingImageFilter', '_p_itk__simple__Image')
class(`IntensityWindowingImageFilter_Execute`) = c("SWIGFunction", class('IntensityWindowingImageFilter_Execute'))

# Start of accessor method for itk::simple::IntensityWindowingImageFilter
setMethod('$', '_p_itk__simple__IntensityWindowingImageFilter', function(x, name)

{
  accessorFuns = list('SetWindowMinimum' = IntensityWindowingImageFilter_SetWindowMinimum, 'GetWindowMinimum' = IntensityWindowingImageFilter_GetWindowMinimum, 'SetWindowMaximum' = IntensityWindowingImageFilter_SetWindowMaximum, 'GetWindowMaximum' = IntensityWindowingImageFilter_GetWindowMaximum, 'SetOutputMinimum' = IntensityWindowingImageFilter_SetOutputMinimum, 'GetOutputMinimum' = IntensityWindowingImageFilter_GetOutputMinimum, 'SetOutputMaximum' = IntensityWindowingImageFilter_SetOutputMaximum, 'GetOutputMaximum' = IntensityWindowingImageFilter_GetOutputMaximum, 'GetName' = IntensityWindowingImageFilter_GetName, 'ToString' = IntensityWindowingImageFilter_ToString, 'Execute' = IntensityWindowingImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::IntensityWindowingImageFilter
# Start of IntensityWindowing

`IntensityWindowing__SWIG_0` = function(image1, windowMinimum, windowMaximum, outputMinimum, outputMaximum, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  
  
  ;ans = .Call('R_swig_IntensityWindowing__SWIG_0', image1, windowMinimum, windowMaximum, outputMinimum, outputMaximum, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntensityWindowing__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`IntensityWindowing__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric')
class(`IntensityWindowing__SWIG_0`) = c("SWIGFunction", class('IntensityWindowing__SWIG_0'))

# Start of IntensityWindowing

`IntensityWindowing__SWIG_1` = function(image1, windowMinimum, windowMaximum, outputMinimum, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  
  ;ans = .Call('R_swig_IntensityWindowing__SWIG_1', image1, windowMinimum, windowMaximum, outputMinimum, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntensityWindowing__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`IntensityWindowing__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric')
class(`IntensityWindowing__SWIG_1`) = c("SWIGFunction", class('IntensityWindowing__SWIG_1'))

# Start of IntensityWindowing

`IntensityWindowing__SWIG_2` = function(image1, windowMinimum, windowMaximum, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_IntensityWindowing__SWIG_2', image1, windowMinimum, windowMaximum, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntensityWindowing__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`IntensityWindowing__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`IntensityWindowing__SWIG_2`) = c("SWIGFunction", class('IntensityWindowing__SWIG_2'))

# Start of IntensityWindowing

`IntensityWindowing__SWIG_3` = function(image1, windowMinimum, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_IntensityWindowing__SWIG_3', image1, windowMinimum, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntensityWindowing__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`IntensityWindowing__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`IntensityWindowing__SWIG_3`) = c("SWIGFunction", class('IntensityWindowing__SWIG_3'))

# Start of IntensityWindowing

`IntensityWindowing__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_IntensityWindowing__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntensityWindowing__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`IntensityWindowing__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`IntensityWindowing__SWIG_4`) = c("SWIGFunction", class('IntensityWindowing__SWIG_4'))

`IntensityWindowing` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- IntensityWindowing__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- IntensityWindowing__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- IntensityWindowing__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- IntensityWindowing__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- IntensityWindowing__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for IntensityWindowing with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_IntermodesThresholdImageFilter

`delete_IntermodesThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_IntermodesThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_IntermodesThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_IntermodesThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter')
class(`delete_IntermodesThresholdImageFilter`) = c("SWIGFunction", class('delete_IntermodesThresholdImageFilter'))

# Start of new_IntermodesThresholdImageFilter

`IntermodesThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_IntermodesThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__IntermodesThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_IntermodesThresholdImageFilter);
  ans
  
}

attr(`IntermodesThresholdImageFilter`, 'returnType') = '_p_itk__simple__IntermodesThresholdImageFilter'
class(`IntermodesThresholdImageFilter`) = c("SWIGFunction", class('IntermodesThresholdImageFilter'))

# Start of IntermodesThresholdImageFilter_SetInsideValue

`IntermodesThresholdImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideValue = as.integer(InsideValue);
  
  if(length(InsideValue) > 1) {
    warning("using only the first element of InsideValue");
  };
  
  ;ans = .Call('R_swig_IntermodesThresholdImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IntermodesThresholdImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`IntermodesThresholdImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter', 'integer')
class(`IntermodesThresholdImageFilter_SetInsideValue`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_SetInsideValue'))

# Start of IntermodesThresholdImageFilter_GetInsideValue

`IntermodesThresholdImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IntermodesThresholdImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IntermodesThresholdImageFilter_GetInsideValue`, 'returnType') = 'integer'
attr(`IntermodesThresholdImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter')
class(`IntermodesThresholdImageFilter_GetInsideValue`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_GetInsideValue'))

# Start of IntermodesThresholdImageFilter_SetOutsideValue

`IntermodesThresholdImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutsideValue = as.integer(OutsideValue);
  
  if(length(OutsideValue) > 1) {
    warning("using only the first element of OutsideValue");
  };
  
  ;ans = .Call('R_swig_IntermodesThresholdImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IntermodesThresholdImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`IntermodesThresholdImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter', 'integer')
class(`IntermodesThresholdImageFilter_SetOutsideValue`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_SetOutsideValue'))

# Start of IntermodesThresholdImageFilter_GetOutsideValue

`IntermodesThresholdImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IntermodesThresholdImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IntermodesThresholdImageFilter_GetOutsideValue`, 'returnType') = 'integer'
attr(`IntermodesThresholdImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter')
class(`IntermodesThresholdImageFilter_GetOutsideValue`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_GetOutsideValue'))

# Start of IntermodesThresholdImageFilter_SetNumberOfHistogramBins

`IntermodesThresholdImageFilter_SetNumberOfHistogramBins` = function(self, NumberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfHistogramBins = as.integer(NumberOfHistogramBins);
  
  if(length(NumberOfHistogramBins) > 1) {
    warning("using only the first element of NumberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_IntermodesThresholdImageFilter_SetNumberOfHistogramBins', self, NumberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IntermodesThresholdImageFilter_SetNumberOfHistogramBins`, 'returnType') = 'void'
attr(`IntermodesThresholdImageFilter_SetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter', 'integer')
class(`IntermodesThresholdImageFilter_SetNumberOfHistogramBins`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_SetNumberOfHistogramBins'))

# Start of IntermodesThresholdImageFilter_GetNumberOfHistogramBins

`IntermodesThresholdImageFilter_GetNumberOfHistogramBins` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IntermodesThresholdImageFilter_GetNumberOfHistogramBins', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IntermodesThresholdImageFilter_GetNumberOfHistogramBins`, 'returnType') = 'integer'
attr(`IntermodesThresholdImageFilter_GetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter')
class(`IntermodesThresholdImageFilter_GetNumberOfHistogramBins`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_GetNumberOfHistogramBins'))

# Start of IntermodesThresholdImageFilter_SetMaskOutput

`IntermodesThresholdImageFilter_SetMaskOutput` = function(self, MaskOutput)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskOutput = as.logical(MaskOutput);
  ;ans = .Call('R_swig_IntermodesThresholdImageFilter_SetMaskOutput', self, MaskOutput, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IntermodesThresholdImageFilter_SetMaskOutput`, 'returnType') = 'void'
attr(`IntermodesThresholdImageFilter_SetMaskOutput`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter', 'logical')
class(`IntermodesThresholdImageFilter_SetMaskOutput`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_SetMaskOutput'))

# Start of IntermodesThresholdImageFilter_MaskOutputOn

`IntermodesThresholdImageFilter_MaskOutputOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_IntermodesThresholdImageFilter_MaskOutputOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IntermodesThresholdImageFilter_MaskOutputOn`, 'returnType') = 'void'
attr(`IntermodesThresholdImageFilter_MaskOutputOn`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter')
class(`IntermodesThresholdImageFilter_MaskOutputOn`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_MaskOutputOn'))

# Start of IntermodesThresholdImageFilter_MaskOutputOff

`IntermodesThresholdImageFilter_MaskOutputOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_IntermodesThresholdImageFilter_MaskOutputOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IntermodesThresholdImageFilter_MaskOutputOff`, 'returnType') = 'void'
attr(`IntermodesThresholdImageFilter_MaskOutputOff`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter')
class(`IntermodesThresholdImageFilter_MaskOutputOff`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_MaskOutputOff'))

# Start of IntermodesThresholdImageFilter_GetMaskOutput

`IntermodesThresholdImageFilter_GetMaskOutput` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IntermodesThresholdImageFilter_GetMaskOutput', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IntermodesThresholdImageFilter_GetMaskOutput`, 'returnType') = 'logical'
attr(`IntermodesThresholdImageFilter_GetMaskOutput`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter')
class(`IntermodesThresholdImageFilter_GetMaskOutput`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_GetMaskOutput'))

# Start of IntermodesThresholdImageFilter_SetMaskValue

`IntermodesThresholdImageFilter_SetMaskValue` = function(self, MaskValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskValue = as.integer(MaskValue);
  
  if(length(MaskValue) > 1) {
    warning("using only the first element of MaskValue");
  };
  
  ;ans = .Call('R_swig_IntermodesThresholdImageFilter_SetMaskValue', self, MaskValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IntermodesThresholdImageFilter_SetMaskValue`, 'returnType') = 'void'
attr(`IntermodesThresholdImageFilter_SetMaskValue`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter', 'integer')
class(`IntermodesThresholdImageFilter_SetMaskValue`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_SetMaskValue'))

# Start of IntermodesThresholdImageFilter_GetMaskValue

`IntermodesThresholdImageFilter_GetMaskValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IntermodesThresholdImageFilter_GetMaskValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IntermodesThresholdImageFilter_GetMaskValue`, 'returnType') = 'integer'
attr(`IntermodesThresholdImageFilter_GetMaskValue`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter')
class(`IntermodesThresholdImageFilter_GetMaskValue`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_GetMaskValue'))

# Start of IntermodesThresholdImageFilter_GetThreshold

`IntermodesThresholdImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IntermodesThresholdImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IntermodesThresholdImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`IntermodesThresholdImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter')
class(`IntermodesThresholdImageFilter_GetThreshold`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_GetThreshold'))

# Start of IntermodesThresholdImageFilter_GetName

`IntermodesThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IntermodesThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IntermodesThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`IntermodesThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter')
class(`IntermodesThresholdImageFilter_GetName`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_GetName'))

# Start of IntermodesThresholdImageFilter_ToString

`IntermodesThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IntermodesThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IntermodesThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`IntermodesThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter')
class(`IntermodesThresholdImageFilter_ToString`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_ToString'))

# Start of IntermodesThresholdImageFilter_Execute

`IntermodesThresholdImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_IntermodesThresholdImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntermodesThresholdImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`IntermodesThresholdImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`IntermodesThresholdImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_Execute__SWIG_0'))

# Start of IntermodesThresholdImageFilter_Execute

`IntermodesThresholdImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_IntermodesThresholdImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntermodesThresholdImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`IntermodesThresholdImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__IntermodesThresholdImageFilter', '_p_itk__simple__Image')
class(`IntermodesThresholdImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('IntermodesThresholdImageFilter_Execute__SWIG_1'))

`IntermodesThresholdImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__IntermodesThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- IntermodesThresholdImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__IntermodesThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- IntermodesThresholdImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for IntermodesThresholdImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::IntermodesThresholdImageFilter
setMethod('$', '_p_itk__simple__IntermodesThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetInsideValue' = IntermodesThresholdImageFilter_SetInsideValue, 'GetInsideValue' = IntermodesThresholdImageFilter_GetInsideValue, 'SetOutsideValue' = IntermodesThresholdImageFilter_SetOutsideValue, 'GetOutsideValue' = IntermodesThresholdImageFilter_GetOutsideValue, 'SetNumberOfHistogramBins' = IntermodesThresholdImageFilter_SetNumberOfHistogramBins, 'GetNumberOfHistogramBins' = IntermodesThresholdImageFilter_GetNumberOfHistogramBins, 'SetMaskOutput' = IntermodesThresholdImageFilter_SetMaskOutput, 'MaskOutputOn' = IntermodesThresholdImageFilter_MaskOutputOn, 'MaskOutputOff' = IntermodesThresholdImageFilter_MaskOutputOff, 'GetMaskOutput' = IntermodesThresholdImageFilter_GetMaskOutput, 'SetMaskValue' = IntermodesThresholdImageFilter_SetMaskValue, 'GetMaskValue' = IntermodesThresholdImageFilter_GetMaskValue, 'GetThreshold' = IntermodesThresholdImageFilter_GetThreshold, 'GetName' = IntermodesThresholdImageFilter_GetName, 'ToString' = IntermodesThresholdImageFilter_ToString, 'Execute' = IntermodesThresholdImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::IntermodesThresholdImageFilter
# Start of IntermodesThreshold

`IntermodesThreshold__SWIG_0` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_IntermodesThreshold__SWIG_0', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntermodesThreshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`IntermodesThreshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`IntermodesThreshold__SWIG_0`) = c("SWIGFunction", class('IntermodesThreshold__SWIG_0'))

# Start of IntermodesThreshold

`IntermodesThreshold__SWIG_1` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_IntermodesThreshold__SWIG_1', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntermodesThreshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`IntermodesThreshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`IntermodesThreshold__SWIG_1`) = c("SWIGFunction", class('IntermodesThreshold__SWIG_1'))

# Start of IntermodesThreshold

`IntermodesThreshold__SWIG_2` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_IntermodesThreshold__SWIG_2', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntermodesThreshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`IntermodesThreshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`IntermodesThreshold__SWIG_2`) = c("SWIGFunction", class('IntermodesThreshold__SWIG_2'))

# Start of IntermodesThreshold

`IntermodesThreshold__SWIG_3` = function(image, maskImage, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_IntermodesThreshold__SWIG_3', image, maskImage, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntermodesThreshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`IntermodesThreshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`IntermodesThreshold__SWIG_3`) = c("SWIGFunction", class('IntermodesThreshold__SWIG_3'))

# Start of IntermodesThreshold

`IntermodesThreshold__SWIG_4` = function(image, maskImage, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_IntermodesThreshold__SWIG_4', image, maskImage, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntermodesThreshold__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`IntermodesThreshold__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`IntermodesThreshold__SWIG_4`) = c("SWIGFunction", class('IntermodesThreshold__SWIG_4'))

# Start of IntermodesThreshold

`IntermodesThreshold__SWIG_5` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_IntermodesThreshold__SWIG_5', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntermodesThreshold__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`IntermodesThreshold__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`IntermodesThreshold__SWIG_5`) = c("SWIGFunction", class('IntermodesThreshold__SWIG_5'))

# Start of IntermodesThreshold

`IntermodesThreshold__SWIG_6` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_IntermodesThreshold__SWIG_6', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntermodesThreshold__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`IntermodesThreshold__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`IntermodesThreshold__SWIG_6`) = c("SWIGFunction", class('IntermodesThreshold__SWIG_6'))

# Start of IntermodesThreshold

`IntermodesThreshold__SWIG_7` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_IntermodesThreshold__SWIG_7', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntermodesThreshold__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`IntermodesThreshold__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`IntermodesThreshold__SWIG_7`) = c("SWIGFunction", class('IntermodesThreshold__SWIG_7'))

# Start of IntermodesThreshold

`IntermodesThreshold__SWIG_8` = function(image, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_IntermodesThreshold__SWIG_8', image, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntermodesThreshold__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`IntermodesThreshold__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`IntermodesThreshold__SWIG_8`) = c("SWIGFunction", class('IntermodesThreshold__SWIG_8'))

# Start of IntermodesThreshold

`IntermodesThreshold__SWIG_9` = function(image, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_IntermodesThreshold__SWIG_9', image, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntermodesThreshold__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`IntermodesThreshold__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`IntermodesThreshold__SWIG_9`) = c("SWIGFunction", class('IntermodesThreshold__SWIG_9'))

# Start of IntermodesThreshold

`IntermodesThreshold__SWIG_10` = function(image, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_IntermodesThreshold__SWIG_10', image, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntermodesThreshold__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`IntermodesThreshold__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`IntermodesThreshold__SWIG_10`) = c("SWIGFunction", class('IntermodesThreshold__SWIG_10'))

# Start of IntermodesThreshold

`IntermodesThreshold__SWIG_11` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_IntermodesThreshold__SWIG_11', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IntermodesThreshold__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`IntermodesThreshold__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image')
class(`IntermodesThreshold__SWIG_11`) = c("SWIGFunction", class('IntermodesThreshold__SWIG_11'))

`IntermodesThreshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 12
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- IntermodesThreshold__SWIG_11; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- IntermodesThreshold__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- IntermodesThreshold__SWIG_10; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- IntermodesThreshold__SWIG_4; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- IntermodesThreshold__SWIG_9; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- IntermodesThreshold__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- IntermodesThreshold__SWIG_8; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- IntermodesThreshold__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- IntermodesThreshold__SWIG_7; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- IntermodesThreshold__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- IntermodesThreshold__SWIG_6; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- IntermodesThreshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for IntermodesThreshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_InverseDeconvolutionImageFilter

`delete_InverseDeconvolutionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_InverseDeconvolutionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_InverseDeconvolutionImageFilter`, 'returnType') = 'void'
attr(`delete_InverseDeconvolutionImageFilter`, "inputTypes") = c('_p_itk__simple__InverseDeconvolutionImageFilter')
class(`delete_InverseDeconvolutionImageFilter`) = c("SWIGFunction", class('delete_InverseDeconvolutionImageFilter'))

# Start of new_InverseDeconvolutionImageFilter

`InverseDeconvolutionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_InverseDeconvolutionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__InverseDeconvolutionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_InverseDeconvolutionImageFilter);
  ans
  
}

attr(`InverseDeconvolutionImageFilter`, 'returnType') = '_p_itk__simple__InverseDeconvolutionImageFilter'
class(`InverseDeconvolutionImageFilter`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter'))

# Start of InverseDeconvolutionImageFilter_SetKernelZeroMagnitudeThreshold

`InverseDeconvolutionImageFilter_SetKernelZeroMagnitudeThreshold` = function(self, KernelZeroMagnitudeThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_InverseDeconvolutionImageFilter_SetKernelZeroMagnitudeThreshold', self, KernelZeroMagnitudeThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InverseDeconvolutionImageFilter_SetKernelZeroMagnitudeThreshold`, 'returnType') = 'void'
attr(`InverseDeconvolutionImageFilter_SetKernelZeroMagnitudeThreshold`, "inputTypes") = c('_p_itk__simple__InverseDeconvolutionImageFilter', 'numeric')
class(`InverseDeconvolutionImageFilter_SetKernelZeroMagnitudeThreshold`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_SetKernelZeroMagnitudeThreshold'))

# Start of InverseDeconvolutionImageFilter_GetKernelZeroMagnitudeThreshold

`InverseDeconvolutionImageFilter_GetKernelZeroMagnitudeThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InverseDeconvolutionImageFilter_GetKernelZeroMagnitudeThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InverseDeconvolutionImageFilter_GetKernelZeroMagnitudeThreshold`, 'returnType') = 'numeric'
attr(`InverseDeconvolutionImageFilter_GetKernelZeroMagnitudeThreshold`, "inputTypes") = c('_p_itk__simple__InverseDeconvolutionImageFilter')
class(`InverseDeconvolutionImageFilter_GetKernelZeroMagnitudeThreshold`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_GetKernelZeroMagnitudeThreshold'))

# Start of InverseDeconvolutionImageFilter_SetNormalize

`InverseDeconvolutionImageFilter_SetNormalize` = function(self, Normalize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Normalize = as.logical(Normalize);
  ;ans = .Call('R_swig_InverseDeconvolutionImageFilter_SetNormalize', self, Normalize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InverseDeconvolutionImageFilter_SetNormalize`, 'returnType') = 'void'
attr(`InverseDeconvolutionImageFilter_SetNormalize`, "inputTypes") = c('_p_itk__simple__InverseDeconvolutionImageFilter', 'logical')
class(`InverseDeconvolutionImageFilter_SetNormalize`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_SetNormalize'))

# Start of InverseDeconvolutionImageFilter_NormalizeOn

`InverseDeconvolutionImageFilter_NormalizeOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_InverseDeconvolutionImageFilter_NormalizeOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InverseDeconvolutionImageFilter_NormalizeOn`, 'returnType') = 'void'
attr(`InverseDeconvolutionImageFilter_NormalizeOn`, "inputTypes") = c('_p_itk__simple__InverseDeconvolutionImageFilter')
class(`InverseDeconvolutionImageFilter_NormalizeOn`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_NormalizeOn'))

# Start of InverseDeconvolutionImageFilter_NormalizeOff

`InverseDeconvolutionImageFilter_NormalizeOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_InverseDeconvolutionImageFilter_NormalizeOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InverseDeconvolutionImageFilter_NormalizeOff`, 'returnType') = 'void'
attr(`InverseDeconvolutionImageFilter_NormalizeOff`, "inputTypes") = c('_p_itk__simple__InverseDeconvolutionImageFilter')
class(`InverseDeconvolutionImageFilter_NormalizeOff`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_NormalizeOff'))

# Start of InverseDeconvolutionImageFilter_GetNormalize

`InverseDeconvolutionImageFilter_GetNormalize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InverseDeconvolutionImageFilter_GetNormalize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InverseDeconvolutionImageFilter_GetNormalize`, 'returnType') = 'logical'
attr(`InverseDeconvolutionImageFilter_GetNormalize`, "inputTypes") = c('_p_itk__simple__InverseDeconvolutionImageFilter')
class(`InverseDeconvolutionImageFilter_GetNormalize`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_GetNormalize'))

# Start of InverseDeconvolutionImageFilter_ZERO_PAD_get

`InverseDeconvolutionImageFilter_ZERO_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InverseDeconvolutionImageFilter_ZERO_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InverseDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`InverseDeconvolutionImageFilter_ZERO_PAD_get`, 'returnType') = 'character'
class(`InverseDeconvolutionImageFilter_ZERO_PAD_get`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_ZERO_PAD_get'))

# Start of InverseDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get

`InverseDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InverseDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InverseDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`InverseDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`, 'returnType') = 'character'
class(`InverseDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get'))

# Start of InverseDeconvolutionImageFilter_PERIODIC_PAD_get

`InverseDeconvolutionImageFilter_PERIODIC_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InverseDeconvolutionImageFilter_PERIODIC_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InverseDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`InverseDeconvolutionImageFilter_PERIODIC_PAD_get`, 'returnType') = 'character'
class(`InverseDeconvolutionImageFilter_PERIODIC_PAD_get`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_PERIODIC_PAD_get'))

# Start of InverseDeconvolutionImageFilter_SetBoundaryCondition

`InverseDeconvolutionImageFilter_SetBoundaryCondition` = function(self, BoundaryCondition)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BoundaryCondition = enumToInteger(BoundaryCondition, "_itk__simple__InverseDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(BoundaryCondition) > 1) {
    warning("using only the first element of BoundaryCondition");
  };
  
  ;ans = .Call('R_swig_InverseDeconvolutionImageFilter_SetBoundaryCondition', self, BoundaryCondition, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InverseDeconvolutionImageFilter_SetBoundaryCondition`, 'returnType') = 'void'
attr(`InverseDeconvolutionImageFilter_SetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__InverseDeconvolutionImageFilter', 'character')
class(`InverseDeconvolutionImageFilter_SetBoundaryCondition`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_SetBoundaryCondition'))

# Start of InverseDeconvolutionImageFilter_GetBoundaryCondition

`InverseDeconvolutionImageFilter_GetBoundaryCondition` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_InverseDeconvolutionImageFilter_GetBoundaryCondition', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InverseDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`InverseDeconvolutionImageFilter_GetBoundaryCondition`, 'returnType') = 'character'
attr(`InverseDeconvolutionImageFilter_GetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__InverseDeconvolutionImageFilter')
class(`InverseDeconvolutionImageFilter_GetBoundaryCondition`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_GetBoundaryCondition'))

# Start of InverseDeconvolutionImageFilter_SAME_get

`InverseDeconvolutionImageFilter_SAME_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InverseDeconvolutionImageFilter_SAME_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InverseDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`InverseDeconvolutionImageFilter_SAME_get`, 'returnType') = 'character'
class(`InverseDeconvolutionImageFilter_SAME_get`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_SAME_get'))

# Start of InverseDeconvolutionImageFilter_VALID_get

`InverseDeconvolutionImageFilter_VALID_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_InverseDeconvolutionImageFilter_VALID_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InverseDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`InverseDeconvolutionImageFilter_VALID_get`, 'returnType') = 'character'
class(`InverseDeconvolutionImageFilter_VALID_get`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_VALID_get'))

# Start of InverseDeconvolutionImageFilter_SetOutputRegionMode

`InverseDeconvolutionImageFilter_SetOutputRegionMode` = function(self, OutputRegionMode)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputRegionMode = enumToInteger(OutputRegionMode, "_itk__simple__InverseDeconvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(OutputRegionMode) > 1) {
    warning("using only the first element of OutputRegionMode");
  };
  
  ;ans = .Call('R_swig_InverseDeconvolutionImageFilter_SetOutputRegionMode', self, OutputRegionMode, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InverseDeconvolutionImageFilter_SetOutputRegionMode`, 'returnType') = 'void'
attr(`InverseDeconvolutionImageFilter_SetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__InverseDeconvolutionImageFilter', 'character')
class(`InverseDeconvolutionImageFilter_SetOutputRegionMode`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_SetOutputRegionMode'))

# Start of InverseDeconvolutionImageFilter_GetOutputRegionMode

`InverseDeconvolutionImageFilter_GetOutputRegionMode` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_InverseDeconvolutionImageFilter_GetOutputRegionMode', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InverseDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`InverseDeconvolutionImageFilter_GetOutputRegionMode`, 'returnType') = 'character'
attr(`InverseDeconvolutionImageFilter_GetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__InverseDeconvolutionImageFilter')
class(`InverseDeconvolutionImageFilter_GetOutputRegionMode`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_GetOutputRegionMode'))

# Start of InverseDeconvolutionImageFilter_GetName

`InverseDeconvolutionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InverseDeconvolutionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InverseDeconvolutionImageFilter_GetName`, 'returnType') = 'character'
attr(`InverseDeconvolutionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__InverseDeconvolutionImageFilter')
class(`InverseDeconvolutionImageFilter_GetName`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_GetName'))

# Start of InverseDeconvolutionImageFilter_ToString

`InverseDeconvolutionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InverseDeconvolutionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InverseDeconvolutionImageFilter_ToString`, 'returnType') = 'character'
attr(`InverseDeconvolutionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__InverseDeconvolutionImageFilter')
class(`InverseDeconvolutionImageFilter_ToString`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_ToString'))

# Start of InverseDeconvolutionImageFilter_Execute

`InverseDeconvolutionImageFilter_Execute` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_InverseDeconvolutionImageFilter_Execute', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InverseDeconvolutionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`InverseDeconvolutionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__InverseDeconvolutionImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`InverseDeconvolutionImageFilter_Execute`) = c("SWIGFunction", class('InverseDeconvolutionImageFilter_Execute'))

# Start of accessor method for itk::simple::InverseDeconvolutionImageFilter
setMethod('$', '_p_itk__simple__InverseDeconvolutionImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelZeroMagnitudeThreshold' = InverseDeconvolutionImageFilter_SetKernelZeroMagnitudeThreshold, 'GetKernelZeroMagnitudeThreshold' = InverseDeconvolutionImageFilter_GetKernelZeroMagnitudeThreshold, 'SetNormalize' = InverseDeconvolutionImageFilter_SetNormalize, 'NormalizeOn' = InverseDeconvolutionImageFilter_NormalizeOn, 'NormalizeOff' = InverseDeconvolutionImageFilter_NormalizeOff, 'GetNormalize' = InverseDeconvolutionImageFilter_GetNormalize, 'SetBoundaryCondition' = InverseDeconvolutionImageFilter_SetBoundaryCondition, 'GetBoundaryCondition' = InverseDeconvolutionImageFilter_GetBoundaryCondition, 'SetOutputRegionMode' = InverseDeconvolutionImageFilter_SetOutputRegionMode, 'GetOutputRegionMode' = InverseDeconvolutionImageFilter_GetOutputRegionMode, 'GetName' = InverseDeconvolutionImageFilter_GetName, 'ToString' = InverseDeconvolutionImageFilter_ToString, 'Execute' = InverseDeconvolutionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::InverseDeconvolutionImageFilter
# Start of InverseDeconvolution

`InverseDeconvolution__SWIG_0` = function(image1, image2, kernelZeroMagnitudeThreshold, normalize, boundaryCondition, outputRegionMode, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__InverseDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  outputRegionMode = enumToInteger(outputRegionMode, "_itk__simple__InverseDeconvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(outputRegionMode) > 1) {
    warning("using only the first element of outputRegionMode");
  };
  
  ;ans = .Call('R_swig_InverseDeconvolution__SWIG_0', image1, image2, kernelZeroMagnitudeThreshold, normalize, boundaryCondition, outputRegionMode, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InverseDeconvolution__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`InverseDeconvolution__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'logical', 'character', 'character')
class(`InverseDeconvolution__SWIG_0`) = c("SWIGFunction", class('InverseDeconvolution__SWIG_0'))

# Start of InverseDeconvolution

`InverseDeconvolution__SWIG_1` = function(image1, image2, kernelZeroMagnitudeThreshold, normalize, boundaryCondition, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__InverseDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  ;ans = .Call('R_swig_InverseDeconvolution__SWIG_1', image1, image2, kernelZeroMagnitudeThreshold, normalize, boundaryCondition, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InverseDeconvolution__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`InverseDeconvolution__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'logical', 'character')
class(`InverseDeconvolution__SWIG_1`) = c("SWIGFunction", class('InverseDeconvolution__SWIG_1'))

# Start of InverseDeconvolution

`InverseDeconvolution__SWIG_2` = function(image1, image2, kernelZeroMagnitudeThreshold, normalize, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  normalize = as.logical(normalize);
  ;ans = .Call('R_swig_InverseDeconvolution__SWIG_2', image1, image2, kernelZeroMagnitudeThreshold, normalize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InverseDeconvolution__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`InverseDeconvolution__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'logical')
class(`InverseDeconvolution__SWIG_2`) = c("SWIGFunction", class('InverseDeconvolution__SWIG_2'))

# Start of InverseDeconvolution

`InverseDeconvolution__SWIG_3` = function(image1, image2, kernelZeroMagnitudeThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  ;ans = .Call('R_swig_InverseDeconvolution__SWIG_3', image1, image2, kernelZeroMagnitudeThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InverseDeconvolution__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`InverseDeconvolution__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`InverseDeconvolution__SWIG_3`) = c("SWIGFunction", class('InverseDeconvolution__SWIG_3'))

# Start of InverseDeconvolution

`InverseDeconvolution__SWIG_4` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_InverseDeconvolution__SWIG_4', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InverseDeconvolution__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`InverseDeconvolution__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`InverseDeconvolution__SWIG_4`) = c("SWIGFunction", class('InverseDeconvolution__SWIG_4'))

`InverseDeconvolution` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- InverseDeconvolution__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- InverseDeconvolution__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- InverseDeconvolution__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.character(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- InverseDeconvolution__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.character(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.character(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- InverseDeconvolution__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for InverseDeconvolution with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_InverseDisplacementFieldImageFilter

`delete_InverseDisplacementFieldImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_InverseDisplacementFieldImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_InverseDisplacementFieldImageFilter`, 'returnType') = 'void'
attr(`delete_InverseDisplacementFieldImageFilter`, "inputTypes") = c('_p_itk__simple__InverseDisplacementFieldImageFilter')
class(`delete_InverseDisplacementFieldImageFilter`) = c("SWIGFunction", class('delete_InverseDisplacementFieldImageFilter'))

# Start of new_InverseDisplacementFieldImageFilter

`InverseDisplacementFieldImageFilter` = function()
{
  ;ans = .Call('R_swig_new_InverseDisplacementFieldImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__InverseDisplacementFieldImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_InverseDisplacementFieldImageFilter);
  ans
  
}

attr(`InverseDisplacementFieldImageFilter`, 'returnType') = '_p_itk__simple__InverseDisplacementFieldImageFilter'
class(`InverseDisplacementFieldImageFilter`) = c("SWIGFunction", class('InverseDisplacementFieldImageFilter'))

# Start of InverseDisplacementFieldImageFilter_SetSize

`InverseDisplacementFieldImageFilter_SetSize` = function(self, Size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Size = as.integer(Size);
  ;ans = .Call('R_swig_InverseDisplacementFieldImageFilter_SetSize', self, Size, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InverseDisplacementFieldImageFilter_SetSize`, 'returnType') = 'void'
attr(`InverseDisplacementFieldImageFilter_SetSize`, "inputTypes") = c('_p_itk__simple__InverseDisplacementFieldImageFilter', 'integer')
class(`InverseDisplacementFieldImageFilter_SetSize`) = c("SWIGFunction", class('InverseDisplacementFieldImageFilter_SetSize'))

# Start of InverseDisplacementFieldImageFilter_GetSize

`InverseDisplacementFieldImageFilter_GetSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InverseDisplacementFieldImageFilter_GetSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InverseDisplacementFieldImageFilter_GetSize`, 'returnType') = 'integer'
attr(`InverseDisplacementFieldImageFilter_GetSize`, "inputTypes") = c('_p_itk__simple__InverseDisplacementFieldImageFilter')
class(`InverseDisplacementFieldImageFilter_GetSize`) = c("SWIGFunction", class('InverseDisplacementFieldImageFilter_GetSize'))

# Start of InverseDisplacementFieldImageFilter_SetOutputOrigin

`InverseDisplacementFieldImageFilter_SetOutputOrigin` = function(self, OutputOrigin)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputOrigin = as.numeric(OutputOrigin);
  ;ans = .Call('R_swig_InverseDisplacementFieldImageFilter_SetOutputOrigin', self, OutputOrigin, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InverseDisplacementFieldImageFilter_SetOutputOrigin`, 'returnType') = 'void'
attr(`InverseDisplacementFieldImageFilter_SetOutputOrigin`, "inputTypes") = c('_p_itk__simple__InverseDisplacementFieldImageFilter', 'numeric')
class(`InverseDisplacementFieldImageFilter_SetOutputOrigin`) = c("SWIGFunction", class('InverseDisplacementFieldImageFilter_SetOutputOrigin'))

# Start of InverseDisplacementFieldImageFilter_GetOutputOrigin

`InverseDisplacementFieldImageFilter_GetOutputOrigin` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InverseDisplacementFieldImageFilter_GetOutputOrigin', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InverseDisplacementFieldImageFilter_GetOutputOrigin`, 'returnType') = 'numeric'
attr(`InverseDisplacementFieldImageFilter_GetOutputOrigin`, "inputTypes") = c('_p_itk__simple__InverseDisplacementFieldImageFilter')
class(`InverseDisplacementFieldImageFilter_GetOutputOrigin`) = c("SWIGFunction", class('InverseDisplacementFieldImageFilter_GetOutputOrigin'))

# Start of InverseDisplacementFieldImageFilter_SetOutputSpacing

`InverseDisplacementFieldImageFilter_SetOutputSpacing` = function(self, OutputSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputSpacing = as.numeric(OutputSpacing);
  ;ans = .Call('R_swig_InverseDisplacementFieldImageFilter_SetOutputSpacing', self, OutputSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InverseDisplacementFieldImageFilter_SetOutputSpacing`, 'returnType') = 'void'
attr(`InverseDisplacementFieldImageFilter_SetOutputSpacing`, "inputTypes") = c('_p_itk__simple__InverseDisplacementFieldImageFilter', 'numeric')
class(`InverseDisplacementFieldImageFilter_SetOutputSpacing`) = c("SWIGFunction", class('InverseDisplacementFieldImageFilter_SetOutputSpacing'))

# Start of InverseDisplacementFieldImageFilter_GetOutputSpacing

`InverseDisplacementFieldImageFilter_GetOutputSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InverseDisplacementFieldImageFilter_GetOutputSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InverseDisplacementFieldImageFilter_GetOutputSpacing`, 'returnType') = 'numeric'
attr(`InverseDisplacementFieldImageFilter_GetOutputSpacing`, "inputTypes") = c('_p_itk__simple__InverseDisplacementFieldImageFilter')
class(`InverseDisplacementFieldImageFilter_GetOutputSpacing`) = c("SWIGFunction", class('InverseDisplacementFieldImageFilter_GetOutputSpacing'))

# Start of InverseDisplacementFieldImageFilter_SetSubsamplingFactor

`InverseDisplacementFieldImageFilter_SetSubsamplingFactor` = function(self, SubsamplingFactor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SubsamplingFactor = as.integer(SubsamplingFactor);
  
  if(length(SubsamplingFactor) > 1) {
    warning("using only the first element of SubsamplingFactor");
  };
  
  ;ans = .Call('R_swig_InverseDisplacementFieldImageFilter_SetSubsamplingFactor', self, SubsamplingFactor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InverseDisplacementFieldImageFilter_SetSubsamplingFactor`, 'returnType') = 'void'
attr(`InverseDisplacementFieldImageFilter_SetSubsamplingFactor`, "inputTypes") = c('_p_itk__simple__InverseDisplacementFieldImageFilter', 'integer')
class(`InverseDisplacementFieldImageFilter_SetSubsamplingFactor`) = c("SWIGFunction", class('InverseDisplacementFieldImageFilter_SetSubsamplingFactor'))

# Start of InverseDisplacementFieldImageFilter_GetSubsamplingFactor

`InverseDisplacementFieldImageFilter_GetSubsamplingFactor` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InverseDisplacementFieldImageFilter_GetSubsamplingFactor', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InverseDisplacementFieldImageFilter_GetSubsamplingFactor`, 'returnType') = 'integer'
attr(`InverseDisplacementFieldImageFilter_GetSubsamplingFactor`, "inputTypes") = c('_p_itk__simple__InverseDisplacementFieldImageFilter')
class(`InverseDisplacementFieldImageFilter_GetSubsamplingFactor`) = c("SWIGFunction", class('InverseDisplacementFieldImageFilter_GetSubsamplingFactor'))

# Start of InverseDisplacementFieldImageFilter_GetName

`InverseDisplacementFieldImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InverseDisplacementFieldImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InverseDisplacementFieldImageFilter_GetName`, 'returnType') = 'character'
attr(`InverseDisplacementFieldImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__InverseDisplacementFieldImageFilter')
class(`InverseDisplacementFieldImageFilter_GetName`) = c("SWIGFunction", class('InverseDisplacementFieldImageFilter_GetName'))

# Start of InverseDisplacementFieldImageFilter_ToString

`InverseDisplacementFieldImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InverseDisplacementFieldImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InverseDisplacementFieldImageFilter_ToString`, 'returnType') = 'character'
attr(`InverseDisplacementFieldImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__InverseDisplacementFieldImageFilter')
class(`InverseDisplacementFieldImageFilter_ToString`) = c("SWIGFunction", class('InverseDisplacementFieldImageFilter_ToString'))

# Start of InverseDisplacementFieldImageFilter_Execute

`InverseDisplacementFieldImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_InverseDisplacementFieldImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InverseDisplacementFieldImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`InverseDisplacementFieldImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__InverseDisplacementFieldImageFilter', '_p_itk__simple__Image')
class(`InverseDisplacementFieldImageFilter_Execute`) = c("SWIGFunction", class('InverseDisplacementFieldImageFilter_Execute'))

# Start of InverseDisplacementFieldImageFilter_SetReferenceImage

`InverseDisplacementFieldImageFilter_SetReferenceImage` = function(self, refImage)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(refImage, "ExternalReference")) refImage = slot(refImage,"ref"); 
  ;ans = .Call('R_swig_InverseDisplacementFieldImageFilter_SetReferenceImage', self, refImage, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InverseDisplacementFieldImageFilter_SetReferenceImage`, 'returnType') = 'void'
attr(`InverseDisplacementFieldImageFilter_SetReferenceImage`, "inputTypes") = c('_p_itk__simple__InverseDisplacementFieldImageFilter', '_p_itk__simple__Image')
class(`InverseDisplacementFieldImageFilter_SetReferenceImage`) = c("SWIGFunction", class('InverseDisplacementFieldImageFilter_SetReferenceImage'))

# Start of accessor method for itk::simple::InverseDisplacementFieldImageFilter
setMethod('$', '_p_itk__simple__InverseDisplacementFieldImageFilter', function(x, name)

{
  accessorFuns = list('SetSize' = InverseDisplacementFieldImageFilter_SetSize, 'GetSize' = InverseDisplacementFieldImageFilter_GetSize, 'SetOutputOrigin' = InverseDisplacementFieldImageFilter_SetOutputOrigin, 'GetOutputOrigin' = InverseDisplacementFieldImageFilter_GetOutputOrigin, 'SetOutputSpacing' = InverseDisplacementFieldImageFilter_SetOutputSpacing, 'GetOutputSpacing' = InverseDisplacementFieldImageFilter_GetOutputSpacing, 'SetSubsamplingFactor' = InverseDisplacementFieldImageFilter_SetSubsamplingFactor, 'GetSubsamplingFactor' = InverseDisplacementFieldImageFilter_GetSubsamplingFactor, 'GetName' = InverseDisplacementFieldImageFilter_GetName, 'ToString' = InverseDisplacementFieldImageFilter_ToString, 'Execute' = InverseDisplacementFieldImageFilter_Execute, 'SetReferenceImage' = InverseDisplacementFieldImageFilter_SetReferenceImage);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::InverseDisplacementFieldImageFilter
# Start of InverseDisplacementField

`InverseDisplacementField__SWIG_0` = function(image1, size, outputOrigin, outputSpacing, subsamplingFactor, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  outputOrigin = as.numeric(outputOrigin);
  outputSpacing = as.numeric(outputSpacing);
  subsamplingFactor = as.integer(subsamplingFactor);
  
  if(length(subsamplingFactor) > 1) {
    warning("using only the first element of subsamplingFactor");
  };
  
  ;ans = .Call('R_swig_InverseDisplacementField__SWIG_0', image1, size, outputOrigin, outputSpacing, subsamplingFactor, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InverseDisplacementField__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`InverseDisplacementField__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'numeric', 'integer')
class(`InverseDisplacementField__SWIG_0`) = c("SWIGFunction", class('InverseDisplacementField__SWIG_0'))

# Start of InverseDisplacementField

`InverseDisplacementField__SWIG_1` = function(image1, size, outputOrigin, outputSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  outputOrigin = as.numeric(outputOrigin);
  outputSpacing = as.numeric(outputSpacing);
  ;ans = .Call('R_swig_InverseDisplacementField__SWIG_1', image1, size, outputOrigin, outputSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InverseDisplacementField__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`InverseDisplacementField__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'numeric')
class(`InverseDisplacementField__SWIG_1`) = c("SWIGFunction", class('InverseDisplacementField__SWIG_1'))

# Start of InverseDisplacementField

`InverseDisplacementField__SWIG_2` = function(image1, size, outputOrigin, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  outputOrigin = as.numeric(outputOrigin);
  ;ans = .Call('R_swig_InverseDisplacementField__SWIG_2', image1, size, outputOrigin, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InverseDisplacementField__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`InverseDisplacementField__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric')
class(`InverseDisplacementField__SWIG_2`) = c("SWIGFunction", class('InverseDisplacementField__SWIG_2'))

# Start of InverseDisplacementField

`InverseDisplacementField__SWIG_3` = function(image1, size, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  ;ans = .Call('R_swig_InverseDisplacementField__SWIG_3', image1, size, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InverseDisplacementField__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`InverseDisplacementField__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`InverseDisplacementField__SWIG_3`) = c("SWIGFunction", class('InverseDisplacementField__SWIG_3'))

# Start of InverseDisplacementField

`InverseDisplacementField__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_InverseDisplacementField__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InverseDisplacementField__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`InverseDisplacementField__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`InverseDisplacementField__SWIG_4`) = c("SWIGFunction", class('InverseDisplacementField__SWIG_4'))

`InverseDisplacementField` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- InverseDisplacementField__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- InverseDisplacementField__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) )) {
      f <- InverseDisplacementField__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) )) {
      f <- InverseDisplacementField__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- InverseDisplacementField__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for InverseDisplacementField with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_InverseFFTImageFilter

`delete_InverseFFTImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_InverseFFTImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_InverseFFTImageFilter`, 'returnType') = 'void'
attr(`delete_InverseFFTImageFilter`, "inputTypes") = c('_p_itk__simple__InverseFFTImageFilter')
class(`delete_InverseFFTImageFilter`) = c("SWIGFunction", class('delete_InverseFFTImageFilter'))

# Start of new_InverseFFTImageFilter

`InverseFFTImageFilter` = function()
{
  ;ans = .Call('R_swig_new_InverseFFTImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__InverseFFTImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_InverseFFTImageFilter);
  ans
  
}

attr(`InverseFFTImageFilter`, 'returnType') = '_p_itk__simple__InverseFFTImageFilter'
class(`InverseFFTImageFilter`) = c("SWIGFunction", class('InverseFFTImageFilter'))

# Start of InverseFFTImageFilter_GetName

`InverseFFTImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InverseFFTImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InverseFFTImageFilter_GetName`, 'returnType') = 'character'
attr(`InverseFFTImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__InverseFFTImageFilter')
class(`InverseFFTImageFilter_GetName`) = c("SWIGFunction", class('InverseFFTImageFilter_GetName'))

# Start of InverseFFTImageFilter_ToString

`InverseFFTImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InverseFFTImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InverseFFTImageFilter_ToString`, 'returnType') = 'character'
attr(`InverseFFTImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__InverseFFTImageFilter')
class(`InverseFFTImageFilter_ToString`) = c("SWIGFunction", class('InverseFFTImageFilter_ToString'))

# Start of InverseFFTImageFilter_Execute

`InverseFFTImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_InverseFFTImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InverseFFTImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`InverseFFTImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__InverseFFTImageFilter', '_p_itk__simple__Image')
class(`InverseFFTImageFilter_Execute`) = c("SWIGFunction", class('InverseFFTImageFilter_Execute'))

# Start of accessor method for itk::simple::InverseFFTImageFilter
setMethod('$', '_p_itk__simple__InverseFFTImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = InverseFFTImageFilter_GetName, 'ToString' = InverseFFTImageFilter_ToString, 'Execute' = InverseFFTImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::InverseFFTImageFilter
# Start of InverseFFT

`InverseFFT` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_InverseFFT', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InverseFFT`, 'returnType') = '_p_itk__simple__Image'
attr(`InverseFFT`, "inputTypes") = c('_p_itk__simple__Image')
class(`InverseFFT`) = c("SWIGFunction", class('InverseFFT'))

# Start of delete_InvertDisplacementFieldImageFilter

`delete_InvertDisplacementFieldImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_InvertDisplacementFieldImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_InvertDisplacementFieldImageFilter`, 'returnType') = 'void'
attr(`delete_InvertDisplacementFieldImageFilter`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter')
class(`delete_InvertDisplacementFieldImageFilter`) = c("SWIGFunction", class('delete_InvertDisplacementFieldImageFilter'))

# Start of new_InvertDisplacementFieldImageFilter

`InvertDisplacementFieldImageFilter` = function()
{
  ;ans = .Call('R_swig_new_InvertDisplacementFieldImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__InvertDisplacementFieldImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_InvertDisplacementFieldImageFilter);
  ans
  
}

attr(`InvertDisplacementFieldImageFilter`, 'returnType') = '_p_itk__simple__InvertDisplacementFieldImageFilter'
class(`InvertDisplacementFieldImageFilter`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter'))

# Start of InvertDisplacementFieldImageFilter_SetMaximumNumberOfIterations

`InvertDisplacementFieldImageFilter_SetMaximumNumberOfIterations` = function(self, MaximumNumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaximumNumberOfIterations = as.integer(MaximumNumberOfIterations);
  
  if(length(MaximumNumberOfIterations) > 1) {
    warning("using only the first element of MaximumNumberOfIterations");
  };
  
  ;ans = .Call('R_swig_InvertDisplacementFieldImageFilter_SetMaximumNumberOfIterations', self, MaximumNumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InvertDisplacementFieldImageFilter_SetMaximumNumberOfIterations`, 'returnType') = 'void'
attr(`InvertDisplacementFieldImageFilter_SetMaximumNumberOfIterations`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter', 'integer')
class(`InvertDisplacementFieldImageFilter_SetMaximumNumberOfIterations`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter_SetMaximumNumberOfIterations'))

# Start of InvertDisplacementFieldImageFilter_GetMaximumNumberOfIterations

`InvertDisplacementFieldImageFilter_GetMaximumNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InvertDisplacementFieldImageFilter_GetMaximumNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InvertDisplacementFieldImageFilter_GetMaximumNumberOfIterations`, 'returnType') = 'integer'
attr(`InvertDisplacementFieldImageFilter_GetMaximumNumberOfIterations`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter')
class(`InvertDisplacementFieldImageFilter_GetMaximumNumberOfIterations`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter_GetMaximumNumberOfIterations'))

# Start of InvertDisplacementFieldImageFilter_SetMaxErrorToleranceThreshold

`InvertDisplacementFieldImageFilter_SetMaxErrorToleranceThreshold` = function(self, MaxErrorToleranceThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_InvertDisplacementFieldImageFilter_SetMaxErrorToleranceThreshold', self, MaxErrorToleranceThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InvertDisplacementFieldImageFilter_SetMaxErrorToleranceThreshold`, 'returnType') = 'void'
attr(`InvertDisplacementFieldImageFilter_SetMaxErrorToleranceThreshold`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter', 'numeric')
class(`InvertDisplacementFieldImageFilter_SetMaxErrorToleranceThreshold`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter_SetMaxErrorToleranceThreshold'))

# Start of InvertDisplacementFieldImageFilter_GetMaxErrorToleranceThreshold

`InvertDisplacementFieldImageFilter_GetMaxErrorToleranceThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InvertDisplacementFieldImageFilter_GetMaxErrorToleranceThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InvertDisplacementFieldImageFilter_GetMaxErrorToleranceThreshold`, 'returnType') = 'numeric'
attr(`InvertDisplacementFieldImageFilter_GetMaxErrorToleranceThreshold`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter')
class(`InvertDisplacementFieldImageFilter_GetMaxErrorToleranceThreshold`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter_GetMaxErrorToleranceThreshold'))

# Start of InvertDisplacementFieldImageFilter_SetMeanErrorToleranceThreshold

`InvertDisplacementFieldImageFilter_SetMeanErrorToleranceThreshold` = function(self, MeanErrorToleranceThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_InvertDisplacementFieldImageFilter_SetMeanErrorToleranceThreshold', self, MeanErrorToleranceThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InvertDisplacementFieldImageFilter_SetMeanErrorToleranceThreshold`, 'returnType') = 'void'
attr(`InvertDisplacementFieldImageFilter_SetMeanErrorToleranceThreshold`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter', 'numeric')
class(`InvertDisplacementFieldImageFilter_SetMeanErrorToleranceThreshold`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter_SetMeanErrorToleranceThreshold'))

# Start of InvertDisplacementFieldImageFilter_GetMeanErrorToleranceThreshold

`InvertDisplacementFieldImageFilter_GetMeanErrorToleranceThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InvertDisplacementFieldImageFilter_GetMeanErrorToleranceThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InvertDisplacementFieldImageFilter_GetMeanErrorToleranceThreshold`, 'returnType') = 'numeric'
attr(`InvertDisplacementFieldImageFilter_GetMeanErrorToleranceThreshold`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter')
class(`InvertDisplacementFieldImageFilter_GetMeanErrorToleranceThreshold`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter_GetMeanErrorToleranceThreshold'))

# Start of InvertDisplacementFieldImageFilter_SetEnforceBoundaryCondition

`InvertDisplacementFieldImageFilter_SetEnforceBoundaryCondition` = function(self, EnforceBoundaryCondition)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  EnforceBoundaryCondition = as.logical(EnforceBoundaryCondition);
  ;ans = .Call('R_swig_InvertDisplacementFieldImageFilter_SetEnforceBoundaryCondition', self, EnforceBoundaryCondition, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InvertDisplacementFieldImageFilter_SetEnforceBoundaryCondition`, 'returnType') = 'void'
attr(`InvertDisplacementFieldImageFilter_SetEnforceBoundaryCondition`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter', 'logical')
class(`InvertDisplacementFieldImageFilter_SetEnforceBoundaryCondition`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter_SetEnforceBoundaryCondition'))

# Start of InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOn

`InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOn`, 'returnType') = 'void'
attr(`InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOn`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter')
class(`InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOn`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOn'))

# Start of InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOff

`InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOff`, 'returnType') = 'void'
attr(`InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOff`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter')
class(`InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOff`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOff'))

# Start of InvertDisplacementFieldImageFilter_GetEnforceBoundaryCondition

`InvertDisplacementFieldImageFilter_GetEnforceBoundaryCondition` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InvertDisplacementFieldImageFilter_GetEnforceBoundaryCondition', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InvertDisplacementFieldImageFilter_GetEnforceBoundaryCondition`, 'returnType') = 'logical'
attr(`InvertDisplacementFieldImageFilter_GetEnforceBoundaryCondition`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter')
class(`InvertDisplacementFieldImageFilter_GetEnforceBoundaryCondition`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter_GetEnforceBoundaryCondition'))

# Start of InvertDisplacementFieldImageFilter_GetMaxErrorNorm

`InvertDisplacementFieldImageFilter_GetMaxErrorNorm` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InvertDisplacementFieldImageFilter_GetMaxErrorNorm', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InvertDisplacementFieldImageFilter_GetMaxErrorNorm`, 'returnType') = 'numeric'
attr(`InvertDisplacementFieldImageFilter_GetMaxErrorNorm`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter')
class(`InvertDisplacementFieldImageFilter_GetMaxErrorNorm`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter_GetMaxErrorNorm'))

# Start of InvertDisplacementFieldImageFilter_GetMeanErrorNorm

`InvertDisplacementFieldImageFilter_GetMeanErrorNorm` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InvertDisplacementFieldImageFilter_GetMeanErrorNorm', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InvertDisplacementFieldImageFilter_GetMeanErrorNorm`, 'returnType') = 'numeric'
attr(`InvertDisplacementFieldImageFilter_GetMeanErrorNorm`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter')
class(`InvertDisplacementFieldImageFilter_GetMeanErrorNorm`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter_GetMeanErrorNorm'))

# Start of InvertDisplacementFieldImageFilter_GetName

`InvertDisplacementFieldImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InvertDisplacementFieldImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InvertDisplacementFieldImageFilter_GetName`, 'returnType') = 'character'
attr(`InvertDisplacementFieldImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter')
class(`InvertDisplacementFieldImageFilter_GetName`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter_GetName'))

# Start of InvertDisplacementFieldImageFilter_ToString

`InvertDisplacementFieldImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InvertDisplacementFieldImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InvertDisplacementFieldImageFilter_ToString`, 'returnType') = 'character'
attr(`InvertDisplacementFieldImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter')
class(`InvertDisplacementFieldImageFilter_ToString`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter_ToString'))

# Start of InvertDisplacementFieldImageFilter_Execute

`InvertDisplacementFieldImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_InvertDisplacementFieldImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InvertDisplacementFieldImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`InvertDisplacementFieldImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__InvertDisplacementFieldImageFilter', '_p_itk__simple__Image')
class(`InvertDisplacementFieldImageFilter_Execute`) = c("SWIGFunction", class('InvertDisplacementFieldImageFilter_Execute'))

# Start of accessor method for itk::simple::InvertDisplacementFieldImageFilter
setMethod('$', '_p_itk__simple__InvertDisplacementFieldImageFilter', function(x, name)

{
  accessorFuns = list('SetMaximumNumberOfIterations' = InvertDisplacementFieldImageFilter_SetMaximumNumberOfIterations, 'GetMaximumNumberOfIterations' = InvertDisplacementFieldImageFilter_GetMaximumNumberOfIterations, 'SetMaxErrorToleranceThreshold' = InvertDisplacementFieldImageFilter_SetMaxErrorToleranceThreshold, 'GetMaxErrorToleranceThreshold' = InvertDisplacementFieldImageFilter_GetMaxErrorToleranceThreshold, 'SetMeanErrorToleranceThreshold' = InvertDisplacementFieldImageFilter_SetMeanErrorToleranceThreshold, 'GetMeanErrorToleranceThreshold' = InvertDisplacementFieldImageFilter_GetMeanErrorToleranceThreshold, 'SetEnforceBoundaryCondition' = InvertDisplacementFieldImageFilter_SetEnforceBoundaryCondition, 'EnforceBoundaryConditionOn' = InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOn, 'EnforceBoundaryConditionOff' = InvertDisplacementFieldImageFilter_EnforceBoundaryConditionOff, 'GetEnforceBoundaryCondition' = InvertDisplacementFieldImageFilter_GetEnforceBoundaryCondition, 'GetMaxErrorNorm' = InvertDisplacementFieldImageFilter_GetMaxErrorNorm, 'GetMeanErrorNorm' = InvertDisplacementFieldImageFilter_GetMeanErrorNorm, 'GetName' = InvertDisplacementFieldImageFilter_GetName, 'ToString' = InvertDisplacementFieldImageFilter_ToString, 'Execute' = InvertDisplacementFieldImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::InvertDisplacementFieldImageFilter
# Start of InvertDisplacementField

`InvertDisplacementField__SWIG_0` = function(image1, maximumNumberOfIterations, maxErrorToleranceThreshold, meanErrorToleranceThreshold, enforceBoundaryCondition, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  
  
  enforceBoundaryCondition = as.logical(enforceBoundaryCondition);
  ;ans = .Call('R_swig_InvertDisplacementField__SWIG_0', image1, maximumNumberOfIterations, maxErrorToleranceThreshold, meanErrorToleranceThreshold, enforceBoundaryCondition, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InvertDisplacementField__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`InvertDisplacementField__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'numeric', 'logical')
class(`InvertDisplacementField__SWIG_0`) = c("SWIGFunction", class('InvertDisplacementField__SWIG_0'))

# Start of InvertDisplacementField

`InvertDisplacementField__SWIG_1` = function(image1, maximumNumberOfIterations, maxErrorToleranceThreshold, meanErrorToleranceThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  
  
  ;ans = .Call('R_swig_InvertDisplacementField__SWIG_1', image1, maximumNumberOfIterations, maxErrorToleranceThreshold, meanErrorToleranceThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InvertDisplacementField__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`InvertDisplacementField__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'numeric')
class(`InvertDisplacementField__SWIG_1`) = c("SWIGFunction", class('InvertDisplacementField__SWIG_1'))

# Start of InvertDisplacementField

`InvertDisplacementField__SWIG_2` = function(image1, maximumNumberOfIterations, maxErrorToleranceThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  
  ;ans = .Call('R_swig_InvertDisplacementField__SWIG_2', image1, maximumNumberOfIterations, maxErrorToleranceThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InvertDisplacementField__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`InvertDisplacementField__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric')
class(`InvertDisplacementField__SWIG_2`) = c("SWIGFunction", class('InvertDisplacementField__SWIG_2'))

# Start of InvertDisplacementField

`InvertDisplacementField__SWIG_3` = function(image1, maximumNumberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  ;ans = .Call('R_swig_InvertDisplacementField__SWIG_3', image1, maximumNumberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InvertDisplacementField__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`InvertDisplacementField__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`InvertDisplacementField__SWIG_3`) = c("SWIGFunction", class('InvertDisplacementField__SWIG_3'))

# Start of InvertDisplacementField

`InvertDisplacementField__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_InvertDisplacementField__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InvertDisplacementField__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`InvertDisplacementField__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`InvertDisplacementField__SWIG_4`) = c("SWIGFunction", class('InvertDisplacementField__SWIG_4'))

`InvertDisplacementField` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- InvertDisplacementField__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- InvertDisplacementField__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- InvertDisplacementField__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- InvertDisplacementField__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- InvertDisplacementField__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for InvertDisplacementField with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_InvertIntensityImageFilter

`delete_InvertIntensityImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_InvertIntensityImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_InvertIntensityImageFilter`, 'returnType') = 'void'
attr(`delete_InvertIntensityImageFilter`, "inputTypes") = c('_p_itk__simple__InvertIntensityImageFilter')
class(`delete_InvertIntensityImageFilter`) = c("SWIGFunction", class('delete_InvertIntensityImageFilter'))

# Start of new_InvertIntensityImageFilter

`InvertIntensityImageFilter` = function()
{
  ;ans = .Call('R_swig_new_InvertIntensityImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__InvertIntensityImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_InvertIntensityImageFilter);
  ans
  
}

attr(`InvertIntensityImageFilter`, 'returnType') = '_p_itk__simple__InvertIntensityImageFilter'
class(`InvertIntensityImageFilter`) = c("SWIGFunction", class('InvertIntensityImageFilter'))

# Start of InvertIntensityImageFilter_SetMaximum

`InvertIntensityImageFilter_SetMaximum` = function(self, Maximum)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_InvertIntensityImageFilter_SetMaximum', self, Maximum, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`InvertIntensityImageFilter_SetMaximum`, 'returnType') = 'void'
attr(`InvertIntensityImageFilter_SetMaximum`, "inputTypes") = c('_p_itk__simple__InvertIntensityImageFilter', 'numeric')
class(`InvertIntensityImageFilter_SetMaximum`) = c("SWIGFunction", class('InvertIntensityImageFilter_SetMaximum'))

# Start of InvertIntensityImageFilter_GetMaximum

`InvertIntensityImageFilter_GetMaximum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InvertIntensityImageFilter_GetMaximum', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InvertIntensityImageFilter_GetMaximum`, 'returnType') = 'numeric'
attr(`InvertIntensityImageFilter_GetMaximum`, "inputTypes") = c('_p_itk__simple__InvertIntensityImageFilter')
class(`InvertIntensityImageFilter_GetMaximum`) = c("SWIGFunction", class('InvertIntensityImageFilter_GetMaximum'))

# Start of InvertIntensityImageFilter_GetName

`InvertIntensityImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InvertIntensityImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InvertIntensityImageFilter_GetName`, 'returnType') = 'character'
attr(`InvertIntensityImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__InvertIntensityImageFilter')
class(`InvertIntensityImageFilter_GetName`) = c("SWIGFunction", class('InvertIntensityImageFilter_GetName'))

# Start of InvertIntensityImageFilter_ToString

`InvertIntensityImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_InvertIntensityImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`InvertIntensityImageFilter_ToString`, 'returnType') = 'character'
attr(`InvertIntensityImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__InvertIntensityImageFilter')
class(`InvertIntensityImageFilter_ToString`) = c("SWIGFunction", class('InvertIntensityImageFilter_ToString'))

# Start of InvertIntensityImageFilter_Execute

`InvertIntensityImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_InvertIntensityImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InvertIntensityImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`InvertIntensityImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__InvertIntensityImageFilter', '_p_itk__simple__Image')
class(`InvertIntensityImageFilter_Execute`) = c("SWIGFunction", class('InvertIntensityImageFilter_Execute'))

# Start of accessor method for itk::simple::InvertIntensityImageFilter
setMethod('$', '_p_itk__simple__InvertIntensityImageFilter', function(x, name)

{
  accessorFuns = list('SetMaximum' = InvertIntensityImageFilter_SetMaximum, 'GetMaximum' = InvertIntensityImageFilter_GetMaximum, 'GetName' = InvertIntensityImageFilter_GetName, 'ToString' = InvertIntensityImageFilter_ToString, 'Execute' = InvertIntensityImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::InvertIntensityImageFilter
# Start of InvertIntensity

`InvertIntensity__SWIG_0` = function(image1, maximum, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_InvertIntensity__SWIG_0', image1, maximum, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InvertIntensity__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`InvertIntensity__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`InvertIntensity__SWIG_0`) = c("SWIGFunction", class('InvertIntensity__SWIG_0'))

# Start of InvertIntensity

`InvertIntensity__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_InvertIntensity__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`InvertIntensity__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`InvertIntensity__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`InvertIntensity__SWIG_1`) = c("SWIGFunction", class('InvertIntensity__SWIG_1'))

`InvertIntensity` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- InvertIntensity__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- InvertIntensity__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for InvertIntensity with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_IsoContourDistanceImageFilter

`delete_IsoContourDistanceImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_IsoContourDistanceImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_IsoContourDistanceImageFilter`, 'returnType') = 'void'
attr(`delete_IsoContourDistanceImageFilter`, "inputTypes") = c('_p_itk__simple__IsoContourDistanceImageFilter')
class(`delete_IsoContourDistanceImageFilter`) = c("SWIGFunction", class('delete_IsoContourDistanceImageFilter'))

# Start of new_IsoContourDistanceImageFilter

`IsoContourDistanceImageFilter` = function()
{
  ;ans = .Call('R_swig_new_IsoContourDistanceImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__IsoContourDistanceImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_IsoContourDistanceImageFilter);
  ans
  
}

attr(`IsoContourDistanceImageFilter`, 'returnType') = '_p_itk__simple__IsoContourDistanceImageFilter'
class(`IsoContourDistanceImageFilter`) = c("SWIGFunction", class('IsoContourDistanceImageFilter'))

# Start of IsoContourDistanceImageFilter_SetLevelSetValue

`IsoContourDistanceImageFilter_SetLevelSetValue` = function(self, LevelSetValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_IsoContourDistanceImageFilter_SetLevelSetValue', self, LevelSetValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsoContourDistanceImageFilter_SetLevelSetValue`, 'returnType') = 'void'
attr(`IsoContourDistanceImageFilter_SetLevelSetValue`, "inputTypes") = c('_p_itk__simple__IsoContourDistanceImageFilter', 'numeric')
class(`IsoContourDistanceImageFilter_SetLevelSetValue`) = c("SWIGFunction", class('IsoContourDistanceImageFilter_SetLevelSetValue'))

# Start of IsoContourDistanceImageFilter_GetLevelSetValue

`IsoContourDistanceImageFilter_GetLevelSetValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsoContourDistanceImageFilter_GetLevelSetValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsoContourDistanceImageFilter_GetLevelSetValue`, 'returnType') = 'numeric'
attr(`IsoContourDistanceImageFilter_GetLevelSetValue`, "inputTypes") = c('_p_itk__simple__IsoContourDistanceImageFilter')
class(`IsoContourDistanceImageFilter_GetLevelSetValue`) = c("SWIGFunction", class('IsoContourDistanceImageFilter_GetLevelSetValue'))

# Start of IsoContourDistanceImageFilter_SetFarValue

`IsoContourDistanceImageFilter_SetFarValue` = function(self, FarValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_IsoContourDistanceImageFilter_SetFarValue', self, FarValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsoContourDistanceImageFilter_SetFarValue`, 'returnType') = 'void'
attr(`IsoContourDistanceImageFilter_SetFarValue`, "inputTypes") = c('_p_itk__simple__IsoContourDistanceImageFilter', 'numeric')
class(`IsoContourDistanceImageFilter_SetFarValue`) = c("SWIGFunction", class('IsoContourDistanceImageFilter_SetFarValue'))

# Start of IsoContourDistanceImageFilter_GetFarValue

`IsoContourDistanceImageFilter_GetFarValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsoContourDistanceImageFilter_GetFarValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsoContourDistanceImageFilter_GetFarValue`, 'returnType') = 'numeric'
attr(`IsoContourDistanceImageFilter_GetFarValue`, "inputTypes") = c('_p_itk__simple__IsoContourDistanceImageFilter')
class(`IsoContourDistanceImageFilter_GetFarValue`) = c("SWIGFunction", class('IsoContourDistanceImageFilter_GetFarValue'))

# Start of IsoContourDistanceImageFilter_GetName

`IsoContourDistanceImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsoContourDistanceImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsoContourDistanceImageFilter_GetName`, 'returnType') = 'character'
attr(`IsoContourDistanceImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__IsoContourDistanceImageFilter')
class(`IsoContourDistanceImageFilter_GetName`) = c("SWIGFunction", class('IsoContourDistanceImageFilter_GetName'))

# Start of IsoContourDistanceImageFilter_ToString

`IsoContourDistanceImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsoContourDistanceImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsoContourDistanceImageFilter_ToString`, 'returnType') = 'character'
attr(`IsoContourDistanceImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__IsoContourDistanceImageFilter')
class(`IsoContourDistanceImageFilter_ToString`) = c("SWIGFunction", class('IsoContourDistanceImageFilter_ToString'))

# Start of IsoContourDistanceImageFilter_Execute

`IsoContourDistanceImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_IsoContourDistanceImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoContourDistanceImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoContourDistanceImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__IsoContourDistanceImageFilter', '_p_itk__simple__Image')
class(`IsoContourDistanceImageFilter_Execute`) = c("SWIGFunction", class('IsoContourDistanceImageFilter_Execute'))

# Start of accessor method for itk::simple::IsoContourDistanceImageFilter
setMethod('$', '_p_itk__simple__IsoContourDistanceImageFilter', function(x, name)

{
  accessorFuns = list('SetLevelSetValue' = IsoContourDistanceImageFilter_SetLevelSetValue, 'GetLevelSetValue' = IsoContourDistanceImageFilter_GetLevelSetValue, 'SetFarValue' = IsoContourDistanceImageFilter_SetFarValue, 'GetFarValue' = IsoContourDistanceImageFilter_GetFarValue, 'GetName' = IsoContourDistanceImageFilter_GetName, 'ToString' = IsoContourDistanceImageFilter_ToString, 'Execute' = IsoContourDistanceImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::IsoContourDistanceImageFilter
# Start of IsoContourDistance

`IsoContourDistance__SWIG_0` = function(image1, levelSetValue, farValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_IsoContourDistance__SWIG_0', image1, levelSetValue, farValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoContourDistance__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoContourDistance__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`IsoContourDistance__SWIG_0`) = c("SWIGFunction", class('IsoContourDistance__SWIG_0'))

# Start of IsoContourDistance

`IsoContourDistance__SWIG_1` = function(image1, levelSetValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_IsoContourDistance__SWIG_1', image1, levelSetValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoContourDistance__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoContourDistance__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`IsoContourDistance__SWIG_1`) = c("SWIGFunction", class('IsoContourDistance__SWIG_1'))

# Start of IsoContourDistance

`IsoContourDistance__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_IsoContourDistance__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoContourDistance__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoContourDistance__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`IsoContourDistance__SWIG_2`) = c("SWIGFunction", class('IsoContourDistance__SWIG_2'))

`IsoContourDistance` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- IsoContourDistance__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- IsoContourDistance__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- IsoContourDistance__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for IsoContourDistance with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_IsoDataThresholdImageFilter

`delete_IsoDataThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_IsoDataThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_IsoDataThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_IsoDataThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter')
class(`delete_IsoDataThresholdImageFilter`) = c("SWIGFunction", class('delete_IsoDataThresholdImageFilter'))

# Start of new_IsoDataThresholdImageFilter

`IsoDataThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_IsoDataThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__IsoDataThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_IsoDataThresholdImageFilter);
  ans
  
}

attr(`IsoDataThresholdImageFilter`, 'returnType') = '_p_itk__simple__IsoDataThresholdImageFilter'
class(`IsoDataThresholdImageFilter`) = c("SWIGFunction", class('IsoDataThresholdImageFilter'))

# Start of IsoDataThresholdImageFilter_SetInsideValue

`IsoDataThresholdImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideValue = as.integer(InsideValue);
  
  if(length(InsideValue) > 1) {
    warning("using only the first element of InsideValue");
  };
  
  ;ans = .Call('R_swig_IsoDataThresholdImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsoDataThresholdImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`IsoDataThresholdImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter', 'integer')
class(`IsoDataThresholdImageFilter_SetInsideValue`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_SetInsideValue'))

# Start of IsoDataThresholdImageFilter_GetInsideValue

`IsoDataThresholdImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsoDataThresholdImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsoDataThresholdImageFilter_GetInsideValue`, 'returnType') = 'integer'
attr(`IsoDataThresholdImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter')
class(`IsoDataThresholdImageFilter_GetInsideValue`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_GetInsideValue'))

# Start of IsoDataThresholdImageFilter_SetOutsideValue

`IsoDataThresholdImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutsideValue = as.integer(OutsideValue);
  
  if(length(OutsideValue) > 1) {
    warning("using only the first element of OutsideValue");
  };
  
  ;ans = .Call('R_swig_IsoDataThresholdImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsoDataThresholdImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`IsoDataThresholdImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter', 'integer')
class(`IsoDataThresholdImageFilter_SetOutsideValue`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_SetOutsideValue'))

# Start of IsoDataThresholdImageFilter_GetOutsideValue

`IsoDataThresholdImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsoDataThresholdImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsoDataThresholdImageFilter_GetOutsideValue`, 'returnType') = 'integer'
attr(`IsoDataThresholdImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter')
class(`IsoDataThresholdImageFilter_GetOutsideValue`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_GetOutsideValue'))

# Start of IsoDataThresholdImageFilter_SetNumberOfHistogramBins

`IsoDataThresholdImageFilter_SetNumberOfHistogramBins` = function(self, NumberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfHistogramBins = as.integer(NumberOfHistogramBins);
  
  if(length(NumberOfHistogramBins) > 1) {
    warning("using only the first element of NumberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_IsoDataThresholdImageFilter_SetNumberOfHistogramBins', self, NumberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsoDataThresholdImageFilter_SetNumberOfHistogramBins`, 'returnType') = 'void'
attr(`IsoDataThresholdImageFilter_SetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter', 'integer')
class(`IsoDataThresholdImageFilter_SetNumberOfHistogramBins`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_SetNumberOfHistogramBins'))

# Start of IsoDataThresholdImageFilter_GetNumberOfHistogramBins

`IsoDataThresholdImageFilter_GetNumberOfHistogramBins` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsoDataThresholdImageFilter_GetNumberOfHistogramBins', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsoDataThresholdImageFilter_GetNumberOfHistogramBins`, 'returnType') = 'integer'
attr(`IsoDataThresholdImageFilter_GetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter')
class(`IsoDataThresholdImageFilter_GetNumberOfHistogramBins`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_GetNumberOfHistogramBins'))

# Start of IsoDataThresholdImageFilter_SetMaskOutput

`IsoDataThresholdImageFilter_SetMaskOutput` = function(self, MaskOutput)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskOutput = as.logical(MaskOutput);
  ;ans = .Call('R_swig_IsoDataThresholdImageFilter_SetMaskOutput', self, MaskOutput, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsoDataThresholdImageFilter_SetMaskOutput`, 'returnType') = 'void'
attr(`IsoDataThresholdImageFilter_SetMaskOutput`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter', 'logical')
class(`IsoDataThresholdImageFilter_SetMaskOutput`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_SetMaskOutput'))

# Start of IsoDataThresholdImageFilter_MaskOutputOn

`IsoDataThresholdImageFilter_MaskOutputOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_IsoDataThresholdImageFilter_MaskOutputOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsoDataThresholdImageFilter_MaskOutputOn`, 'returnType') = 'void'
attr(`IsoDataThresholdImageFilter_MaskOutputOn`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter')
class(`IsoDataThresholdImageFilter_MaskOutputOn`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_MaskOutputOn'))

# Start of IsoDataThresholdImageFilter_MaskOutputOff

`IsoDataThresholdImageFilter_MaskOutputOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_IsoDataThresholdImageFilter_MaskOutputOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsoDataThresholdImageFilter_MaskOutputOff`, 'returnType') = 'void'
attr(`IsoDataThresholdImageFilter_MaskOutputOff`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter')
class(`IsoDataThresholdImageFilter_MaskOutputOff`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_MaskOutputOff'))

# Start of IsoDataThresholdImageFilter_GetMaskOutput

`IsoDataThresholdImageFilter_GetMaskOutput` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsoDataThresholdImageFilter_GetMaskOutput', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsoDataThresholdImageFilter_GetMaskOutput`, 'returnType') = 'logical'
attr(`IsoDataThresholdImageFilter_GetMaskOutput`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter')
class(`IsoDataThresholdImageFilter_GetMaskOutput`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_GetMaskOutput'))

# Start of IsoDataThresholdImageFilter_SetMaskValue

`IsoDataThresholdImageFilter_SetMaskValue` = function(self, MaskValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskValue = as.integer(MaskValue);
  
  if(length(MaskValue) > 1) {
    warning("using only the first element of MaskValue");
  };
  
  ;ans = .Call('R_swig_IsoDataThresholdImageFilter_SetMaskValue', self, MaskValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsoDataThresholdImageFilter_SetMaskValue`, 'returnType') = 'void'
attr(`IsoDataThresholdImageFilter_SetMaskValue`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter', 'integer')
class(`IsoDataThresholdImageFilter_SetMaskValue`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_SetMaskValue'))

# Start of IsoDataThresholdImageFilter_GetMaskValue

`IsoDataThresholdImageFilter_GetMaskValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsoDataThresholdImageFilter_GetMaskValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsoDataThresholdImageFilter_GetMaskValue`, 'returnType') = 'integer'
attr(`IsoDataThresholdImageFilter_GetMaskValue`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter')
class(`IsoDataThresholdImageFilter_GetMaskValue`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_GetMaskValue'))

# Start of IsoDataThresholdImageFilter_GetThreshold

`IsoDataThresholdImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsoDataThresholdImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsoDataThresholdImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`IsoDataThresholdImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter')
class(`IsoDataThresholdImageFilter_GetThreshold`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_GetThreshold'))

# Start of IsoDataThresholdImageFilter_GetName

`IsoDataThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsoDataThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsoDataThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`IsoDataThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter')
class(`IsoDataThresholdImageFilter_GetName`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_GetName'))

# Start of IsoDataThresholdImageFilter_ToString

`IsoDataThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsoDataThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsoDataThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`IsoDataThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter')
class(`IsoDataThresholdImageFilter_ToString`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_ToString'))

# Start of IsoDataThresholdImageFilter_Execute

`IsoDataThresholdImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_IsoDataThresholdImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoDataThresholdImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoDataThresholdImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`IsoDataThresholdImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_Execute__SWIG_0'))

# Start of IsoDataThresholdImageFilter_Execute

`IsoDataThresholdImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_IsoDataThresholdImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoDataThresholdImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoDataThresholdImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__IsoDataThresholdImageFilter', '_p_itk__simple__Image')
class(`IsoDataThresholdImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('IsoDataThresholdImageFilter_Execute__SWIG_1'))

`IsoDataThresholdImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__IsoDataThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- IsoDataThresholdImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__IsoDataThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- IsoDataThresholdImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for IsoDataThresholdImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::IsoDataThresholdImageFilter
setMethod('$', '_p_itk__simple__IsoDataThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetInsideValue' = IsoDataThresholdImageFilter_SetInsideValue, 'GetInsideValue' = IsoDataThresholdImageFilter_GetInsideValue, 'SetOutsideValue' = IsoDataThresholdImageFilter_SetOutsideValue, 'GetOutsideValue' = IsoDataThresholdImageFilter_GetOutsideValue, 'SetNumberOfHistogramBins' = IsoDataThresholdImageFilter_SetNumberOfHistogramBins, 'GetNumberOfHistogramBins' = IsoDataThresholdImageFilter_GetNumberOfHistogramBins, 'SetMaskOutput' = IsoDataThresholdImageFilter_SetMaskOutput, 'MaskOutputOn' = IsoDataThresholdImageFilter_MaskOutputOn, 'MaskOutputOff' = IsoDataThresholdImageFilter_MaskOutputOff, 'GetMaskOutput' = IsoDataThresholdImageFilter_GetMaskOutput, 'SetMaskValue' = IsoDataThresholdImageFilter_SetMaskValue, 'GetMaskValue' = IsoDataThresholdImageFilter_GetMaskValue, 'GetThreshold' = IsoDataThresholdImageFilter_GetThreshold, 'GetName' = IsoDataThresholdImageFilter_GetName, 'ToString' = IsoDataThresholdImageFilter_ToString, 'Execute' = IsoDataThresholdImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::IsoDataThresholdImageFilter
# Start of IsoDataThreshold

`IsoDataThreshold__SWIG_0` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_IsoDataThreshold__SWIG_0', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoDataThreshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoDataThreshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`IsoDataThreshold__SWIG_0`) = c("SWIGFunction", class('IsoDataThreshold__SWIG_0'))

# Start of IsoDataThreshold

`IsoDataThreshold__SWIG_1` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_IsoDataThreshold__SWIG_1', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoDataThreshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoDataThreshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`IsoDataThreshold__SWIG_1`) = c("SWIGFunction", class('IsoDataThreshold__SWIG_1'))

# Start of IsoDataThreshold

`IsoDataThreshold__SWIG_2` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_IsoDataThreshold__SWIG_2', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoDataThreshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoDataThreshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`IsoDataThreshold__SWIG_2`) = c("SWIGFunction", class('IsoDataThreshold__SWIG_2'))

# Start of IsoDataThreshold

`IsoDataThreshold__SWIG_3` = function(image, maskImage, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_IsoDataThreshold__SWIG_3', image, maskImage, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoDataThreshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoDataThreshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`IsoDataThreshold__SWIG_3`) = c("SWIGFunction", class('IsoDataThreshold__SWIG_3'))

# Start of IsoDataThreshold

`IsoDataThreshold__SWIG_4` = function(image, maskImage, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_IsoDataThreshold__SWIG_4', image, maskImage, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoDataThreshold__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoDataThreshold__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`IsoDataThreshold__SWIG_4`) = c("SWIGFunction", class('IsoDataThreshold__SWIG_4'))

# Start of IsoDataThreshold

`IsoDataThreshold__SWIG_5` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_IsoDataThreshold__SWIG_5', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoDataThreshold__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoDataThreshold__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`IsoDataThreshold__SWIG_5`) = c("SWIGFunction", class('IsoDataThreshold__SWIG_5'))

# Start of IsoDataThreshold

`IsoDataThreshold__SWIG_6` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_IsoDataThreshold__SWIG_6', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoDataThreshold__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoDataThreshold__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`IsoDataThreshold__SWIG_6`) = c("SWIGFunction", class('IsoDataThreshold__SWIG_6'))

# Start of IsoDataThreshold

`IsoDataThreshold__SWIG_7` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_IsoDataThreshold__SWIG_7', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoDataThreshold__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoDataThreshold__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`IsoDataThreshold__SWIG_7`) = c("SWIGFunction", class('IsoDataThreshold__SWIG_7'))

# Start of IsoDataThreshold

`IsoDataThreshold__SWIG_8` = function(image, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_IsoDataThreshold__SWIG_8', image, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoDataThreshold__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoDataThreshold__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`IsoDataThreshold__SWIG_8`) = c("SWIGFunction", class('IsoDataThreshold__SWIG_8'))

# Start of IsoDataThreshold

`IsoDataThreshold__SWIG_9` = function(image, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_IsoDataThreshold__SWIG_9', image, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoDataThreshold__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoDataThreshold__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`IsoDataThreshold__SWIG_9`) = c("SWIGFunction", class('IsoDataThreshold__SWIG_9'))

# Start of IsoDataThreshold

`IsoDataThreshold__SWIG_10` = function(image, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_IsoDataThreshold__SWIG_10', image, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoDataThreshold__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoDataThreshold__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`IsoDataThreshold__SWIG_10`) = c("SWIGFunction", class('IsoDataThreshold__SWIG_10'))

# Start of IsoDataThreshold

`IsoDataThreshold__SWIG_11` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_IsoDataThreshold__SWIG_11', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsoDataThreshold__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`IsoDataThreshold__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image')
class(`IsoDataThreshold__SWIG_11`) = c("SWIGFunction", class('IsoDataThreshold__SWIG_11'))

`IsoDataThreshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 12
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- IsoDataThreshold__SWIG_11; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- IsoDataThreshold__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- IsoDataThreshold__SWIG_10; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- IsoDataThreshold__SWIG_4; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- IsoDataThreshold__SWIG_9; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- IsoDataThreshold__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- IsoDataThreshold__SWIG_8; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- IsoDataThreshold__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- IsoDataThreshold__SWIG_7; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- IsoDataThreshold__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- IsoDataThreshold__SWIG_6; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- IsoDataThreshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for IsoDataThreshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_IsolatedConnectedImageFilter

`delete_IsolatedConnectedImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_IsolatedConnectedImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_IsolatedConnectedImageFilter`, 'returnType') = 'void'
attr(`delete_IsolatedConnectedImageFilter`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter')
class(`delete_IsolatedConnectedImageFilter`) = c("SWIGFunction", class('delete_IsolatedConnectedImageFilter'))

# Start of new_IsolatedConnectedImageFilter

`IsolatedConnectedImageFilter` = function()
{
  ;ans = .Call('R_swig_new_IsolatedConnectedImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__IsolatedConnectedImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_IsolatedConnectedImageFilter);
  ans
  
}

attr(`IsolatedConnectedImageFilter`, 'returnType') = '_p_itk__simple__IsolatedConnectedImageFilter'
class(`IsolatedConnectedImageFilter`) = c("SWIGFunction", class('IsolatedConnectedImageFilter'))

# Start of IsolatedConnectedImageFilter_SetSeed1

`IsolatedConnectedImageFilter_SetSeed1` = function(self, Seed1)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Seed1 = as.integer(Seed1);
  ;ans = .Call('R_swig_IsolatedConnectedImageFilter_SetSeed1', self, Seed1, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedConnectedImageFilter_SetSeed1`, 'returnType') = 'void'
attr(`IsolatedConnectedImageFilter_SetSeed1`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter', 'integer')
class(`IsolatedConnectedImageFilter_SetSeed1`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_SetSeed1'))

# Start of IsolatedConnectedImageFilter_GetSeed1

`IsolatedConnectedImageFilter_GetSeed1` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedConnectedImageFilter_GetSeed1', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedConnectedImageFilter_GetSeed1`, 'returnType') = 'integer'
attr(`IsolatedConnectedImageFilter_GetSeed1`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter')
class(`IsolatedConnectedImageFilter_GetSeed1`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_GetSeed1'))

# Start of IsolatedConnectedImageFilter_SetSeed2

`IsolatedConnectedImageFilter_SetSeed2` = function(self, Seed2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Seed2 = as.integer(Seed2);
  ;ans = .Call('R_swig_IsolatedConnectedImageFilter_SetSeed2', self, Seed2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedConnectedImageFilter_SetSeed2`, 'returnType') = 'void'
attr(`IsolatedConnectedImageFilter_SetSeed2`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter', 'integer')
class(`IsolatedConnectedImageFilter_SetSeed2`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_SetSeed2'))

# Start of IsolatedConnectedImageFilter_GetSeed2

`IsolatedConnectedImageFilter_GetSeed2` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedConnectedImageFilter_GetSeed2', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedConnectedImageFilter_GetSeed2`, 'returnType') = 'integer'
attr(`IsolatedConnectedImageFilter_GetSeed2`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter')
class(`IsolatedConnectedImageFilter_GetSeed2`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_GetSeed2'))

# Start of IsolatedConnectedImageFilter_SetLower

`IsolatedConnectedImageFilter_SetLower` = function(self, Lower)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_IsolatedConnectedImageFilter_SetLower', self, Lower, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedConnectedImageFilter_SetLower`, 'returnType') = 'void'
attr(`IsolatedConnectedImageFilter_SetLower`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter', 'numeric')
class(`IsolatedConnectedImageFilter_SetLower`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_SetLower'))

# Start of IsolatedConnectedImageFilter_GetLower

`IsolatedConnectedImageFilter_GetLower` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedConnectedImageFilter_GetLower', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedConnectedImageFilter_GetLower`, 'returnType') = 'numeric'
attr(`IsolatedConnectedImageFilter_GetLower`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter')
class(`IsolatedConnectedImageFilter_GetLower`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_GetLower'))

# Start of IsolatedConnectedImageFilter_SetUpper

`IsolatedConnectedImageFilter_SetUpper` = function(self, Upper)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_IsolatedConnectedImageFilter_SetUpper', self, Upper, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedConnectedImageFilter_SetUpper`, 'returnType') = 'void'
attr(`IsolatedConnectedImageFilter_SetUpper`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter', 'numeric')
class(`IsolatedConnectedImageFilter_SetUpper`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_SetUpper'))

# Start of IsolatedConnectedImageFilter_GetUpper

`IsolatedConnectedImageFilter_GetUpper` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedConnectedImageFilter_GetUpper', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedConnectedImageFilter_GetUpper`, 'returnType') = 'numeric'
attr(`IsolatedConnectedImageFilter_GetUpper`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter')
class(`IsolatedConnectedImageFilter_GetUpper`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_GetUpper'))

# Start of IsolatedConnectedImageFilter_SetReplaceValue

`IsolatedConnectedImageFilter_SetReplaceValue` = function(self, ReplaceValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ReplaceValue = as.integer(ReplaceValue);
  
  if(length(ReplaceValue) > 1) {
    warning("using only the first element of ReplaceValue");
  };
  
  ;ans = .Call('R_swig_IsolatedConnectedImageFilter_SetReplaceValue', self, ReplaceValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedConnectedImageFilter_SetReplaceValue`, 'returnType') = 'void'
attr(`IsolatedConnectedImageFilter_SetReplaceValue`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter', 'integer')
class(`IsolatedConnectedImageFilter_SetReplaceValue`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_SetReplaceValue'))

# Start of IsolatedConnectedImageFilter_GetReplaceValue

`IsolatedConnectedImageFilter_GetReplaceValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedConnectedImageFilter_GetReplaceValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedConnectedImageFilter_GetReplaceValue`, 'returnType') = 'integer'
attr(`IsolatedConnectedImageFilter_GetReplaceValue`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter')
class(`IsolatedConnectedImageFilter_GetReplaceValue`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_GetReplaceValue'))

# Start of IsolatedConnectedImageFilter_SetIsolatedValueTolerance

`IsolatedConnectedImageFilter_SetIsolatedValueTolerance` = function(self, IsolatedValueTolerance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_IsolatedConnectedImageFilter_SetIsolatedValueTolerance', self, IsolatedValueTolerance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedConnectedImageFilter_SetIsolatedValueTolerance`, 'returnType') = 'void'
attr(`IsolatedConnectedImageFilter_SetIsolatedValueTolerance`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter', 'numeric')
class(`IsolatedConnectedImageFilter_SetIsolatedValueTolerance`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_SetIsolatedValueTolerance'))

# Start of IsolatedConnectedImageFilter_GetIsolatedValueTolerance

`IsolatedConnectedImageFilter_GetIsolatedValueTolerance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedConnectedImageFilter_GetIsolatedValueTolerance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedConnectedImageFilter_GetIsolatedValueTolerance`, 'returnType') = 'numeric'
attr(`IsolatedConnectedImageFilter_GetIsolatedValueTolerance`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter')
class(`IsolatedConnectedImageFilter_GetIsolatedValueTolerance`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_GetIsolatedValueTolerance'))

# Start of IsolatedConnectedImageFilter_SetFindUpperThreshold

`IsolatedConnectedImageFilter_SetFindUpperThreshold` = function(self, FindUpperThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FindUpperThreshold = as.logical(FindUpperThreshold);
  ;ans = .Call('R_swig_IsolatedConnectedImageFilter_SetFindUpperThreshold', self, FindUpperThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedConnectedImageFilter_SetFindUpperThreshold`, 'returnType') = 'void'
attr(`IsolatedConnectedImageFilter_SetFindUpperThreshold`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter', 'logical')
class(`IsolatedConnectedImageFilter_SetFindUpperThreshold`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_SetFindUpperThreshold'))

# Start of IsolatedConnectedImageFilter_FindUpperThresholdOn

`IsolatedConnectedImageFilter_FindUpperThresholdOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_IsolatedConnectedImageFilter_FindUpperThresholdOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedConnectedImageFilter_FindUpperThresholdOn`, 'returnType') = 'void'
attr(`IsolatedConnectedImageFilter_FindUpperThresholdOn`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter')
class(`IsolatedConnectedImageFilter_FindUpperThresholdOn`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_FindUpperThresholdOn'))

# Start of IsolatedConnectedImageFilter_FindUpperThresholdOff

`IsolatedConnectedImageFilter_FindUpperThresholdOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_IsolatedConnectedImageFilter_FindUpperThresholdOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedConnectedImageFilter_FindUpperThresholdOff`, 'returnType') = 'void'
attr(`IsolatedConnectedImageFilter_FindUpperThresholdOff`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter')
class(`IsolatedConnectedImageFilter_FindUpperThresholdOff`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_FindUpperThresholdOff'))

# Start of IsolatedConnectedImageFilter_GetFindUpperThreshold

`IsolatedConnectedImageFilter_GetFindUpperThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedConnectedImageFilter_GetFindUpperThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedConnectedImageFilter_GetFindUpperThreshold`, 'returnType') = 'logical'
attr(`IsolatedConnectedImageFilter_GetFindUpperThreshold`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter')
class(`IsolatedConnectedImageFilter_GetFindUpperThreshold`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_GetFindUpperThreshold'))

# Start of IsolatedConnectedImageFilter_GetThresholdingFailed

`IsolatedConnectedImageFilter_GetThresholdingFailed` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedConnectedImageFilter_GetThresholdingFailed', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedConnectedImageFilter_GetThresholdingFailed`, 'returnType') = 'logical'
attr(`IsolatedConnectedImageFilter_GetThresholdingFailed`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter')
class(`IsolatedConnectedImageFilter_GetThresholdingFailed`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_GetThresholdingFailed'))

# Start of IsolatedConnectedImageFilter_GetIsolatedValue

`IsolatedConnectedImageFilter_GetIsolatedValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedConnectedImageFilter_GetIsolatedValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedConnectedImageFilter_GetIsolatedValue`, 'returnType') = 'numeric'
attr(`IsolatedConnectedImageFilter_GetIsolatedValue`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter')
class(`IsolatedConnectedImageFilter_GetIsolatedValue`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_GetIsolatedValue'))

# Start of IsolatedConnectedImageFilter_GetName

`IsolatedConnectedImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedConnectedImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedConnectedImageFilter_GetName`, 'returnType') = 'character'
attr(`IsolatedConnectedImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter')
class(`IsolatedConnectedImageFilter_GetName`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_GetName'))

# Start of IsolatedConnectedImageFilter_ToString

`IsolatedConnectedImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedConnectedImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedConnectedImageFilter_ToString`, 'returnType') = 'character'
attr(`IsolatedConnectedImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter')
class(`IsolatedConnectedImageFilter_ToString`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_ToString'))

# Start of IsolatedConnectedImageFilter_Execute

`IsolatedConnectedImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_IsolatedConnectedImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedConnectedImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedConnectedImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__IsolatedConnectedImageFilter', '_p_itk__simple__Image')
class(`IsolatedConnectedImageFilter_Execute`) = c("SWIGFunction", class('IsolatedConnectedImageFilter_Execute'))

# Start of accessor method for itk::simple::IsolatedConnectedImageFilter
setMethod('$', '_p_itk__simple__IsolatedConnectedImageFilter', function(x, name)

{
  accessorFuns = list('SetSeed1' = IsolatedConnectedImageFilter_SetSeed1, 'GetSeed1' = IsolatedConnectedImageFilter_GetSeed1, 'SetSeed2' = IsolatedConnectedImageFilter_SetSeed2, 'GetSeed2' = IsolatedConnectedImageFilter_GetSeed2, 'SetLower' = IsolatedConnectedImageFilter_SetLower, 'GetLower' = IsolatedConnectedImageFilter_GetLower, 'SetUpper' = IsolatedConnectedImageFilter_SetUpper, 'GetUpper' = IsolatedConnectedImageFilter_GetUpper, 'SetReplaceValue' = IsolatedConnectedImageFilter_SetReplaceValue, 'GetReplaceValue' = IsolatedConnectedImageFilter_GetReplaceValue, 'SetIsolatedValueTolerance' = IsolatedConnectedImageFilter_SetIsolatedValueTolerance, 'GetIsolatedValueTolerance' = IsolatedConnectedImageFilter_GetIsolatedValueTolerance, 'SetFindUpperThreshold' = IsolatedConnectedImageFilter_SetFindUpperThreshold, 'FindUpperThresholdOn' = IsolatedConnectedImageFilter_FindUpperThresholdOn, 'FindUpperThresholdOff' = IsolatedConnectedImageFilter_FindUpperThresholdOff, 'GetFindUpperThreshold' = IsolatedConnectedImageFilter_GetFindUpperThreshold, 'GetThresholdingFailed' = IsolatedConnectedImageFilter_GetThresholdingFailed, 'GetIsolatedValue' = IsolatedConnectedImageFilter_GetIsolatedValue, 'GetName' = IsolatedConnectedImageFilter_GetName, 'ToString' = IsolatedConnectedImageFilter_ToString, 'Execute' = IsolatedConnectedImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::IsolatedConnectedImageFilter
# Start of IsolatedConnected

`IsolatedConnected__SWIG_0` = function(image1, seed1, seed2, lower, upper, replaceValue, isolatedValueTolerance, findUpperThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed1 = as.integer(seed1);
  seed2 = as.integer(seed2);
  
  
  replaceValue = as.integer(replaceValue);
  
  if(length(replaceValue) > 1) {
    warning("using only the first element of replaceValue");
  };
  
  
  findUpperThreshold = as.logical(findUpperThreshold);
  ;ans = .Call('R_swig_IsolatedConnected__SWIG_0', image1, seed1, seed2, lower, upper, replaceValue, isolatedValueTolerance, findUpperThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedConnected__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedConnected__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'numeric', 'numeric', 'integer', 'numeric', 'logical')
class(`IsolatedConnected__SWIG_0`) = c("SWIGFunction", class('IsolatedConnected__SWIG_0'))

# Start of IsolatedConnected

`IsolatedConnected__SWIG_1` = function(image1, seed1, seed2, lower, upper, replaceValue, isolatedValueTolerance, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed1 = as.integer(seed1);
  seed2 = as.integer(seed2);
  
  
  replaceValue = as.integer(replaceValue);
  
  if(length(replaceValue) > 1) {
    warning("using only the first element of replaceValue");
  };
  
  
  ;ans = .Call('R_swig_IsolatedConnected__SWIG_1', image1, seed1, seed2, lower, upper, replaceValue, isolatedValueTolerance, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedConnected__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedConnected__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'numeric', 'numeric', 'integer', 'numeric')
class(`IsolatedConnected__SWIG_1`) = c("SWIGFunction", class('IsolatedConnected__SWIG_1'))

# Start of IsolatedConnected

`IsolatedConnected__SWIG_2` = function(image1, seed1, seed2, lower, upper, replaceValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed1 = as.integer(seed1);
  seed2 = as.integer(seed2);
  
  
  replaceValue = as.integer(replaceValue);
  
  if(length(replaceValue) > 1) {
    warning("using only the first element of replaceValue");
  };
  
  ;ans = .Call('R_swig_IsolatedConnected__SWIG_2', image1, seed1, seed2, lower, upper, replaceValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedConnected__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedConnected__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'numeric', 'numeric', 'integer')
class(`IsolatedConnected__SWIG_2`) = c("SWIGFunction", class('IsolatedConnected__SWIG_2'))

# Start of IsolatedConnected

`IsolatedConnected__SWIG_3` = function(image1, seed1, seed2, lower, upper, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed1 = as.integer(seed1);
  seed2 = as.integer(seed2);
  
  
  ;ans = .Call('R_swig_IsolatedConnected__SWIG_3', image1, seed1, seed2, lower, upper, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedConnected__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedConnected__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'numeric', 'numeric')
class(`IsolatedConnected__SWIG_3`) = c("SWIGFunction", class('IsolatedConnected__SWIG_3'))

# Start of IsolatedConnected

`IsolatedConnected__SWIG_4` = function(image1, seed1, seed2, lower, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed1 = as.integer(seed1);
  seed2 = as.integer(seed2);
  
  ;ans = .Call('R_swig_IsolatedConnected__SWIG_4', image1, seed1, seed2, lower, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedConnected__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedConnected__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'numeric')
class(`IsolatedConnected__SWIG_4`) = c("SWIGFunction", class('IsolatedConnected__SWIG_4'))

# Start of IsolatedConnected

`IsolatedConnected__SWIG_5` = function(image1, seed1, seed2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed1 = as.integer(seed1);
  seed2 = as.integer(seed2);
  ;ans = .Call('R_swig_IsolatedConnected__SWIG_5', image1, seed1, seed2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedConnected__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedConnected__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`IsolatedConnected__SWIG_5`) = c("SWIGFunction", class('IsolatedConnected__SWIG_5'))

# Start of IsolatedConnected

`IsolatedConnected__SWIG_6` = function(image1, seed1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed1 = as.integer(seed1);
  ;ans = .Call('R_swig_IsolatedConnected__SWIG_6', image1, seed1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedConnected__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedConnected__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`IsolatedConnected__SWIG_6`) = c("SWIGFunction", class('IsolatedConnected__SWIG_6'))

# Start of IsolatedConnected

`IsolatedConnected__SWIG_7` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_IsolatedConnected__SWIG_7', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedConnected__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedConnected__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image')
class(`IsolatedConnected__SWIG_7`) = c("SWIGFunction", class('IsolatedConnected__SWIG_7'))

`IsolatedConnected` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 8
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- IsolatedConnected__SWIG_7; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- IsolatedConnected__SWIG_6; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- IsolatedConnected__SWIG_5; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- IsolatedConnected__SWIG_4; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- IsolatedConnected__SWIG_3; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- IsolatedConnected__SWIG_2; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- IsolatedConnected__SWIG_1; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.logical(argv[[8]]) && length(argv[[8]]) == 1 )) {
      f <- IsolatedConnected__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for IsolatedConnected with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_IsolatedWatershedImageFilter

`delete_IsolatedWatershedImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_IsolatedWatershedImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_IsolatedWatershedImageFilter`, 'returnType') = 'void'
attr(`delete_IsolatedWatershedImageFilter`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter')
class(`delete_IsolatedWatershedImageFilter`) = c("SWIGFunction", class('delete_IsolatedWatershedImageFilter'))

# Start of new_IsolatedWatershedImageFilter

`IsolatedWatershedImageFilter` = function()
{
  ;ans = .Call('R_swig_new_IsolatedWatershedImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__IsolatedWatershedImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_IsolatedWatershedImageFilter);
  ans
  
}

attr(`IsolatedWatershedImageFilter`, 'returnType') = '_p_itk__simple__IsolatedWatershedImageFilter'
class(`IsolatedWatershedImageFilter`) = c("SWIGFunction", class('IsolatedWatershedImageFilter'))

# Start of IsolatedWatershedImageFilter_SetSeed1

`IsolatedWatershedImageFilter_SetSeed1` = function(self, Seed1)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Seed1 = as.integer(Seed1);
  ;ans = .Call('R_swig_IsolatedWatershedImageFilter_SetSeed1', self, Seed1, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedWatershedImageFilter_SetSeed1`, 'returnType') = 'void'
attr(`IsolatedWatershedImageFilter_SetSeed1`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter', 'integer')
class(`IsolatedWatershedImageFilter_SetSeed1`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_SetSeed1'))

# Start of IsolatedWatershedImageFilter_GetSeed1

`IsolatedWatershedImageFilter_GetSeed1` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedWatershedImageFilter_GetSeed1', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedWatershedImageFilter_GetSeed1`, 'returnType') = 'integer'
attr(`IsolatedWatershedImageFilter_GetSeed1`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter')
class(`IsolatedWatershedImageFilter_GetSeed1`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_GetSeed1'))

# Start of IsolatedWatershedImageFilter_SetSeed2

`IsolatedWatershedImageFilter_SetSeed2` = function(self, Seed2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Seed2 = as.integer(Seed2);
  ;ans = .Call('R_swig_IsolatedWatershedImageFilter_SetSeed2', self, Seed2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedWatershedImageFilter_SetSeed2`, 'returnType') = 'void'
attr(`IsolatedWatershedImageFilter_SetSeed2`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter', 'integer')
class(`IsolatedWatershedImageFilter_SetSeed2`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_SetSeed2'))

# Start of IsolatedWatershedImageFilter_GetSeed2

`IsolatedWatershedImageFilter_GetSeed2` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedWatershedImageFilter_GetSeed2', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedWatershedImageFilter_GetSeed2`, 'returnType') = 'integer'
attr(`IsolatedWatershedImageFilter_GetSeed2`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter')
class(`IsolatedWatershedImageFilter_GetSeed2`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_GetSeed2'))

# Start of IsolatedWatershedImageFilter_SetThreshold

`IsolatedWatershedImageFilter_SetThreshold` = function(self, Threshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_IsolatedWatershedImageFilter_SetThreshold', self, Threshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedWatershedImageFilter_SetThreshold`, 'returnType') = 'void'
attr(`IsolatedWatershedImageFilter_SetThreshold`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter', 'numeric')
class(`IsolatedWatershedImageFilter_SetThreshold`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_SetThreshold'))

# Start of IsolatedWatershedImageFilter_GetThreshold

`IsolatedWatershedImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedWatershedImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedWatershedImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`IsolatedWatershedImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter')
class(`IsolatedWatershedImageFilter_GetThreshold`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_GetThreshold'))

# Start of IsolatedWatershedImageFilter_SetUpperValueLimit

`IsolatedWatershedImageFilter_SetUpperValueLimit` = function(self, UpperValueLimit)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_IsolatedWatershedImageFilter_SetUpperValueLimit', self, UpperValueLimit, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedWatershedImageFilter_SetUpperValueLimit`, 'returnType') = 'void'
attr(`IsolatedWatershedImageFilter_SetUpperValueLimit`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter', 'numeric')
class(`IsolatedWatershedImageFilter_SetUpperValueLimit`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_SetUpperValueLimit'))

# Start of IsolatedWatershedImageFilter_GetUpperValueLimit

`IsolatedWatershedImageFilter_GetUpperValueLimit` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedWatershedImageFilter_GetUpperValueLimit', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedWatershedImageFilter_GetUpperValueLimit`, 'returnType') = 'numeric'
attr(`IsolatedWatershedImageFilter_GetUpperValueLimit`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter')
class(`IsolatedWatershedImageFilter_GetUpperValueLimit`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_GetUpperValueLimit'))

# Start of IsolatedWatershedImageFilter_SetIsolatedValueTolerance

`IsolatedWatershedImageFilter_SetIsolatedValueTolerance` = function(self, IsolatedValueTolerance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_IsolatedWatershedImageFilter_SetIsolatedValueTolerance', self, IsolatedValueTolerance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedWatershedImageFilter_SetIsolatedValueTolerance`, 'returnType') = 'void'
attr(`IsolatedWatershedImageFilter_SetIsolatedValueTolerance`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter', 'numeric')
class(`IsolatedWatershedImageFilter_SetIsolatedValueTolerance`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_SetIsolatedValueTolerance'))

# Start of IsolatedWatershedImageFilter_GetIsolatedValueTolerance

`IsolatedWatershedImageFilter_GetIsolatedValueTolerance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedWatershedImageFilter_GetIsolatedValueTolerance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedWatershedImageFilter_GetIsolatedValueTolerance`, 'returnType') = 'numeric'
attr(`IsolatedWatershedImageFilter_GetIsolatedValueTolerance`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter')
class(`IsolatedWatershedImageFilter_GetIsolatedValueTolerance`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_GetIsolatedValueTolerance'))

# Start of IsolatedWatershedImageFilter_SetReplaceValue1

`IsolatedWatershedImageFilter_SetReplaceValue1` = function(self, ReplaceValue1)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ReplaceValue1 = as.integer(ReplaceValue1);
  
  if(length(ReplaceValue1) > 1) {
    warning("using only the first element of ReplaceValue1");
  };
  
  ;ans = .Call('R_swig_IsolatedWatershedImageFilter_SetReplaceValue1', self, ReplaceValue1, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedWatershedImageFilter_SetReplaceValue1`, 'returnType') = 'void'
attr(`IsolatedWatershedImageFilter_SetReplaceValue1`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter', 'integer')
class(`IsolatedWatershedImageFilter_SetReplaceValue1`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_SetReplaceValue1'))

# Start of IsolatedWatershedImageFilter_GetReplaceValue1

`IsolatedWatershedImageFilter_GetReplaceValue1` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedWatershedImageFilter_GetReplaceValue1', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedWatershedImageFilter_GetReplaceValue1`, 'returnType') = 'integer'
attr(`IsolatedWatershedImageFilter_GetReplaceValue1`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter')
class(`IsolatedWatershedImageFilter_GetReplaceValue1`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_GetReplaceValue1'))

# Start of IsolatedWatershedImageFilter_SetReplaceValue2

`IsolatedWatershedImageFilter_SetReplaceValue2` = function(self, ReplaceValue2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ReplaceValue2 = as.integer(ReplaceValue2);
  
  if(length(ReplaceValue2) > 1) {
    warning("using only the first element of ReplaceValue2");
  };
  
  ;ans = .Call('R_swig_IsolatedWatershedImageFilter_SetReplaceValue2', self, ReplaceValue2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IsolatedWatershedImageFilter_SetReplaceValue2`, 'returnType') = 'void'
attr(`IsolatedWatershedImageFilter_SetReplaceValue2`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter', 'integer')
class(`IsolatedWatershedImageFilter_SetReplaceValue2`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_SetReplaceValue2'))

# Start of IsolatedWatershedImageFilter_GetReplaceValue2

`IsolatedWatershedImageFilter_GetReplaceValue2` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedWatershedImageFilter_GetReplaceValue2', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedWatershedImageFilter_GetReplaceValue2`, 'returnType') = 'integer'
attr(`IsolatedWatershedImageFilter_GetReplaceValue2`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter')
class(`IsolatedWatershedImageFilter_GetReplaceValue2`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_GetReplaceValue2'))

# Start of IsolatedWatershedImageFilter_GetName

`IsolatedWatershedImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedWatershedImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedWatershedImageFilter_GetName`, 'returnType') = 'character'
attr(`IsolatedWatershedImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter')
class(`IsolatedWatershedImageFilter_GetName`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_GetName'))

# Start of IsolatedWatershedImageFilter_ToString

`IsolatedWatershedImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IsolatedWatershedImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IsolatedWatershedImageFilter_ToString`, 'returnType') = 'character'
attr(`IsolatedWatershedImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter')
class(`IsolatedWatershedImageFilter_ToString`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_ToString'))

# Start of IsolatedWatershedImageFilter_Execute

`IsolatedWatershedImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_IsolatedWatershedImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedWatershedImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedWatershedImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__IsolatedWatershedImageFilter', '_p_itk__simple__Image')
class(`IsolatedWatershedImageFilter_Execute`) = c("SWIGFunction", class('IsolatedWatershedImageFilter_Execute'))

# Start of accessor method for itk::simple::IsolatedWatershedImageFilter
setMethod('$', '_p_itk__simple__IsolatedWatershedImageFilter', function(x, name)

{
  accessorFuns = list('SetSeed1' = IsolatedWatershedImageFilter_SetSeed1, 'GetSeed1' = IsolatedWatershedImageFilter_GetSeed1, 'SetSeed2' = IsolatedWatershedImageFilter_SetSeed2, 'GetSeed2' = IsolatedWatershedImageFilter_GetSeed2, 'SetThreshold' = IsolatedWatershedImageFilter_SetThreshold, 'GetThreshold' = IsolatedWatershedImageFilter_GetThreshold, 'SetUpperValueLimit' = IsolatedWatershedImageFilter_SetUpperValueLimit, 'GetUpperValueLimit' = IsolatedWatershedImageFilter_GetUpperValueLimit, 'SetIsolatedValueTolerance' = IsolatedWatershedImageFilter_SetIsolatedValueTolerance, 'GetIsolatedValueTolerance' = IsolatedWatershedImageFilter_GetIsolatedValueTolerance, 'SetReplaceValue1' = IsolatedWatershedImageFilter_SetReplaceValue1, 'GetReplaceValue1' = IsolatedWatershedImageFilter_GetReplaceValue1, 'SetReplaceValue2' = IsolatedWatershedImageFilter_SetReplaceValue2, 'GetReplaceValue2' = IsolatedWatershedImageFilter_GetReplaceValue2, 'GetName' = IsolatedWatershedImageFilter_GetName, 'ToString' = IsolatedWatershedImageFilter_ToString, 'Execute' = IsolatedWatershedImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::IsolatedWatershedImageFilter
# Start of IsolatedWatershed

`IsolatedWatershed__SWIG_0` = function(image1, seed1, seed2, threshold, upperValueLimit, isolatedValueTolerance, replaceValue1, replaceValue2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed1 = as.integer(seed1);
  seed2 = as.integer(seed2);
  
  
  
  replaceValue1 = as.integer(replaceValue1);
  
  if(length(replaceValue1) > 1) {
    warning("using only the first element of replaceValue1");
  };
  
  replaceValue2 = as.integer(replaceValue2);
  
  if(length(replaceValue2) > 1) {
    warning("using only the first element of replaceValue2");
  };
  
  ;ans = .Call('R_swig_IsolatedWatershed__SWIG_0', image1, seed1, seed2, threshold, upperValueLimit, isolatedValueTolerance, replaceValue1, replaceValue2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedWatershed__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedWatershed__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'numeric', 'numeric', 'numeric', 'integer', 'integer')
class(`IsolatedWatershed__SWIG_0`) = c("SWIGFunction", class('IsolatedWatershed__SWIG_0'))

# Start of IsolatedWatershed

`IsolatedWatershed__SWIG_1` = function(image1, seed1, seed2, threshold, upperValueLimit, isolatedValueTolerance, replaceValue1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed1 = as.integer(seed1);
  seed2 = as.integer(seed2);
  
  
  
  replaceValue1 = as.integer(replaceValue1);
  
  if(length(replaceValue1) > 1) {
    warning("using only the first element of replaceValue1");
  };
  
  ;ans = .Call('R_swig_IsolatedWatershed__SWIG_1', image1, seed1, seed2, threshold, upperValueLimit, isolatedValueTolerance, replaceValue1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedWatershed__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedWatershed__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'numeric', 'numeric', 'numeric', 'integer')
class(`IsolatedWatershed__SWIG_1`) = c("SWIGFunction", class('IsolatedWatershed__SWIG_1'))

# Start of IsolatedWatershed

`IsolatedWatershed__SWIG_2` = function(image1, seed1, seed2, threshold, upperValueLimit, isolatedValueTolerance, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed1 = as.integer(seed1);
  seed2 = as.integer(seed2);
  
  
  
  ;ans = .Call('R_swig_IsolatedWatershed__SWIG_2', image1, seed1, seed2, threshold, upperValueLimit, isolatedValueTolerance, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedWatershed__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedWatershed__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'numeric', 'numeric', 'numeric')
class(`IsolatedWatershed__SWIG_2`) = c("SWIGFunction", class('IsolatedWatershed__SWIG_2'))

# Start of IsolatedWatershed

`IsolatedWatershed__SWIG_3` = function(image1, seed1, seed2, threshold, upperValueLimit, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed1 = as.integer(seed1);
  seed2 = as.integer(seed2);
  
  
  ;ans = .Call('R_swig_IsolatedWatershed__SWIG_3', image1, seed1, seed2, threshold, upperValueLimit, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedWatershed__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedWatershed__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'numeric', 'numeric')
class(`IsolatedWatershed__SWIG_3`) = c("SWIGFunction", class('IsolatedWatershed__SWIG_3'))

# Start of IsolatedWatershed

`IsolatedWatershed__SWIG_4` = function(image1, seed1, seed2, threshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed1 = as.integer(seed1);
  seed2 = as.integer(seed2);
  
  ;ans = .Call('R_swig_IsolatedWatershed__SWIG_4', image1, seed1, seed2, threshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedWatershed__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedWatershed__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'numeric')
class(`IsolatedWatershed__SWIG_4`) = c("SWIGFunction", class('IsolatedWatershed__SWIG_4'))

# Start of IsolatedWatershed

`IsolatedWatershed__SWIG_5` = function(image1, seed1, seed2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed1 = as.integer(seed1);
  seed2 = as.integer(seed2);
  ;ans = .Call('R_swig_IsolatedWatershed__SWIG_5', image1, seed1, seed2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedWatershed__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedWatershed__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`IsolatedWatershed__SWIG_5`) = c("SWIGFunction", class('IsolatedWatershed__SWIG_5'))

# Start of IsolatedWatershed

`IsolatedWatershed__SWIG_6` = function(image1, seed1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seed1 = as.integer(seed1);
  ;ans = .Call('R_swig_IsolatedWatershed__SWIG_6', image1, seed1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedWatershed__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedWatershed__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`IsolatedWatershed__SWIG_6`) = c("SWIGFunction", class('IsolatedWatershed__SWIG_6'))

# Start of IsolatedWatershed

`IsolatedWatershed__SWIG_7` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_IsolatedWatershed__SWIG_7', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IsolatedWatershed__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`IsolatedWatershed__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image')
class(`IsolatedWatershed__SWIG_7`) = c("SWIGFunction", class('IsolatedWatershed__SWIG_7'))

`IsolatedWatershed` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 8
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- IsolatedWatershed__SWIG_7; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- IsolatedWatershed__SWIG_6; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- IsolatedWatershed__SWIG_5; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- IsolatedWatershed__SWIG_4; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- IsolatedWatershed__SWIG_3; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- IsolatedWatershed__SWIG_2; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- IsolatedWatershed__SWIG_1; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 ) && ( (is.integer(argv[[8]]) || is.numeric(argv[[8]])) && length(argv[[8]]) == 1 )) {
      f <- IsolatedWatershed__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for IsolatedWatershed with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_IterativeInverseDisplacementFieldImageFilter

`delete_IterativeInverseDisplacementFieldImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_IterativeInverseDisplacementFieldImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_IterativeInverseDisplacementFieldImageFilter`, 'returnType') = 'void'
attr(`delete_IterativeInverseDisplacementFieldImageFilter`, "inputTypes") = c('_p_itk__simple__IterativeInverseDisplacementFieldImageFilter')
class(`delete_IterativeInverseDisplacementFieldImageFilter`) = c("SWIGFunction", class('delete_IterativeInverseDisplacementFieldImageFilter'))

# Start of new_IterativeInverseDisplacementFieldImageFilter

`IterativeInverseDisplacementFieldImageFilter` = function()
{
  ;ans = .Call('R_swig_new_IterativeInverseDisplacementFieldImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__IterativeInverseDisplacementFieldImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_IterativeInverseDisplacementFieldImageFilter);
  ans
  
}

attr(`IterativeInverseDisplacementFieldImageFilter`, 'returnType') = '_p_itk__simple__IterativeInverseDisplacementFieldImageFilter'
class(`IterativeInverseDisplacementFieldImageFilter`) = c("SWIGFunction", class('IterativeInverseDisplacementFieldImageFilter'))

# Start of IterativeInverseDisplacementFieldImageFilter_SetNumberOfIterations

`IterativeInverseDisplacementFieldImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_IterativeInverseDisplacementFieldImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IterativeInverseDisplacementFieldImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`IterativeInverseDisplacementFieldImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__IterativeInverseDisplacementFieldImageFilter', 'integer')
class(`IterativeInverseDisplacementFieldImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('IterativeInverseDisplacementFieldImageFilter_SetNumberOfIterations'))

# Start of IterativeInverseDisplacementFieldImageFilter_GetNumberOfIterations

`IterativeInverseDisplacementFieldImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IterativeInverseDisplacementFieldImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IterativeInverseDisplacementFieldImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`IterativeInverseDisplacementFieldImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__IterativeInverseDisplacementFieldImageFilter')
class(`IterativeInverseDisplacementFieldImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('IterativeInverseDisplacementFieldImageFilter_GetNumberOfIterations'))

# Start of IterativeInverseDisplacementFieldImageFilter_SetStopValue

`IterativeInverseDisplacementFieldImageFilter_SetStopValue` = function(self, StopValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_IterativeInverseDisplacementFieldImageFilter_SetStopValue', self, StopValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`IterativeInverseDisplacementFieldImageFilter_SetStopValue`, 'returnType') = 'void'
attr(`IterativeInverseDisplacementFieldImageFilter_SetStopValue`, "inputTypes") = c('_p_itk__simple__IterativeInverseDisplacementFieldImageFilter', 'numeric')
class(`IterativeInverseDisplacementFieldImageFilter_SetStopValue`) = c("SWIGFunction", class('IterativeInverseDisplacementFieldImageFilter_SetStopValue'))

# Start of IterativeInverseDisplacementFieldImageFilter_GetStopValue

`IterativeInverseDisplacementFieldImageFilter_GetStopValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IterativeInverseDisplacementFieldImageFilter_GetStopValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IterativeInverseDisplacementFieldImageFilter_GetStopValue`, 'returnType') = 'numeric'
attr(`IterativeInverseDisplacementFieldImageFilter_GetStopValue`, "inputTypes") = c('_p_itk__simple__IterativeInverseDisplacementFieldImageFilter')
class(`IterativeInverseDisplacementFieldImageFilter_GetStopValue`) = c("SWIGFunction", class('IterativeInverseDisplacementFieldImageFilter_GetStopValue'))

# Start of IterativeInverseDisplacementFieldImageFilter_GetName

`IterativeInverseDisplacementFieldImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IterativeInverseDisplacementFieldImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IterativeInverseDisplacementFieldImageFilter_GetName`, 'returnType') = 'character'
attr(`IterativeInverseDisplacementFieldImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__IterativeInverseDisplacementFieldImageFilter')
class(`IterativeInverseDisplacementFieldImageFilter_GetName`) = c("SWIGFunction", class('IterativeInverseDisplacementFieldImageFilter_GetName'))

# Start of IterativeInverseDisplacementFieldImageFilter_ToString

`IterativeInverseDisplacementFieldImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_IterativeInverseDisplacementFieldImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`IterativeInverseDisplacementFieldImageFilter_ToString`, 'returnType') = 'character'
attr(`IterativeInverseDisplacementFieldImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__IterativeInverseDisplacementFieldImageFilter')
class(`IterativeInverseDisplacementFieldImageFilter_ToString`) = c("SWIGFunction", class('IterativeInverseDisplacementFieldImageFilter_ToString'))

# Start of IterativeInverseDisplacementFieldImageFilter_Execute

`IterativeInverseDisplacementFieldImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_IterativeInverseDisplacementFieldImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IterativeInverseDisplacementFieldImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`IterativeInverseDisplacementFieldImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__IterativeInverseDisplacementFieldImageFilter', '_p_itk__simple__Image')
class(`IterativeInverseDisplacementFieldImageFilter_Execute`) = c("SWIGFunction", class('IterativeInverseDisplacementFieldImageFilter_Execute'))

# Start of accessor method for itk::simple::IterativeInverseDisplacementFieldImageFilter
setMethod('$', '_p_itk__simple__IterativeInverseDisplacementFieldImageFilter', function(x, name)

{
  accessorFuns = list('SetNumberOfIterations' = IterativeInverseDisplacementFieldImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = IterativeInverseDisplacementFieldImageFilter_GetNumberOfIterations, 'SetStopValue' = IterativeInverseDisplacementFieldImageFilter_SetStopValue, 'GetStopValue' = IterativeInverseDisplacementFieldImageFilter_GetStopValue, 'GetName' = IterativeInverseDisplacementFieldImageFilter_GetName, 'ToString' = IterativeInverseDisplacementFieldImageFilter_ToString, 'Execute' = IterativeInverseDisplacementFieldImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::IterativeInverseDisplacementFieldImageFilter
# Start of IterativeInverseDisplacementField

`IterativeInverseDisplacementField__SWIG_0` = function(image1, numberOfIterations, stopValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  ;ans = .Call('R_swig_IterativeInverseDisplacementField__SWIG_0', image1, numberOfIterations, stopValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IterativeInverseDisplacementField__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`IterativeInverseDisplacementField__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric')
class(`IterativeInverseDisplacementField__SWIG_0`) = c("SWIGFunction", class('IterativeInverseDisplacementField__SWIG_0'))

# Start of IterativeInverseDisplacementField

`IterativeInverseDisplacementField__SWIG_1` = function(image1, numberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_IterativeInverseDisplacementField__SWIG_1', image1, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IterativeInverseDisplacementField__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`IterativeInverseDisplacementField__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`IterativeInverseDisplacementField__SWIG_1`) = c("SWIGFunction", class('IterativeInverseDisplacementField__SWIG_1'))

# Start of IterativeInverseDisplacementField

`IterativeInverseDisplacementField__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_IterativeInverseDisplacementField__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`IterativeInverseDisplacementField__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`IterativeInverseDisplacementField__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`IterativeInverseDisplacementField__SWIG_2`) = c("SWIGFunction", class('IterativeInverseDisplacementField__SWIG_2'))

`IterativeInverseDisplacementField` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- IterativeInverseDisplacementField__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- IterativeInverseDisplacementField__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- IterativeInverseDisplacementField__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for IterativeInverseDisplacementField with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_JoinSeriesImageFilter

`delete_JoinSeriesImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_JoinSeriesImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_JoinSeriesImageFilter`, 'returnType') = 'void'
attr(`delete_JoinSeriesImageFilter`, "inputTypes") = c('_p_itk__simple__JoinSeriesImageFilter')
class(`delete_JoinSeriesImageFilter`) = c("SWIGFunction", class('delete_JoinSeriesImageFilter'))

# Start of new_JoinSeriesImageFilter

`JoinSeriesImageFilter` = function()
{
  ;ans = .Call('R_swig_new_JoinSeriesImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__JoinSeriesImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_JoinSeriesImageFilter);
  ans
  
}

attr(`JoinSeriesImageFilter`, 'returnType') = '_p_itk__simple__JoinSeriesImageFilter'
class(`JoinSeriesImageFilter`) = c("SWIGFunction", class('JoinSeriesImageFilter'))

# Start of JoinSeriesImageFilter_SetOrigin

`JoinSeriesImageFilter_SetOrigin` = function(self, Origin)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_JoinSeriesImageFilter_SetOrigin', self, Origin, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`JoinSeriesImageFilter_SetOrigin`, 'returnType') = 'void'
attr(`JoinSeriesImageFilter_SetOrigin`, "inputTypes") = c('_p_itk__simple__JoinSeriesImageFilter', 'numeric')
class(`JoinSeriesImageFilter_SetOrigin`) = c("SWIGFunction", class('JoinSeriesImageFilter_SetOrigin'))

# Start of JoinSeriesImageFilter_GetOrigin

`JoinSeriesImageFilter_GetOrigin` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_JoinSeriesImageFilter_GetOrigin', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`JoinSeriesImageFilter_GetOrigin`, 'returnType') = 'numeric'
attr(`JoinSeriesImageFilter_GetOrigin`, "inputTypes") = c('_p_itk__simple__JoinSeriesImageFilter')
class(`JoinSeriesImageFilter_GetOrigin`) = c("SWIGFunction", class('JoinSeriesImageFilter_GetOrigin'))

# Start of JoinSeriesImageFilter_SetSpacing

`JoinSeriesImageFilter_SetSpacing` = function(self, Spacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_JoinSeriesImageFilter_SetSpacing', self, Spacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`JoinSeriesImageFilter_SetSpacing`, 'returnType') = 'void'
attr(`JoinSeriesImageFilter_SetSpacing`, "inputTypes") = c('_p_itk__simple__JoinSeriesImageFilter', 'numeric')
class(`JoinSeriesImageFilter_SetSpacing`) = c("SWIGFunction", class('JoinSeriesImageFilter_SetSpacing'))

# Start of JoinSeriesImageFilter_GetSpacing

`JoinSeriesImageFilter_GetSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_JoinSeriesImageFilter_GetSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`JoinSeriesImageFilter_GetSpacing`, 'returnType') = 'numeric'
attr(`JoinSeriesImageFilter_GetSpacing`, "inputTypes") = c('_p_itk__simple__JoinSeriesImageFilter')
class(`JoinSeriesImageFilter_GetSpacing`) = c("SWIGFunction", class('JoinSeriesImageFilter_GetSpacing'))

# Start of JoinSeriesImageFilter_GetName

`JoinSeriesImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_JoinSeriesImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`JoinSeriesImageFilter_GetName`, 'returnType') = 'character'
attr(`JoinSeriesImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__JoinSeriesImageFilter')
class(`JoinSeriesImageFilter_GetName`) = c("SWIGFunction", class('JoinSeriesImageFilter_GetName'))

# Start of JoinSeriesImageFilter_ToString

`JoinSeriesImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_JoinSeriesImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`JoinSeriesImageFilter_ToString`, 'returnType') = 'character'
attr(`JoinSeriesImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__JoinSeriesImageFilter')
class(`JoinSeriesImageFilter_ToString`) = c("SWIGFunction", class('JoinSeriesImageFilter_ToString'))

# Start of JoinSeriesImageFilter_Execute

`JoinSeriesImageFilter_Execute__SWIG_0` = function(self, images, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_JoinSeriesImageFilter_Execute__SWIG_0', self, images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeriesImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeriesImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__JoinSeriesImageFilter', 'list')
class(`JoinSeriesImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('JoinSeriesImageFilter_Execute__SWIG_0'))

# Start of JoinSeriesImageFilter_Execute

`JoinSeriesImageFilter_Execute__SWIG_1` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_JoinSeriesImageFilter_Execute__SWIG_1', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeriesImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeriesImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__JoinSeriesImageFilter', '_p_itk__simple__Image')
class(`JoinSeriesImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('JoinSeriesImageFilter_Execute__SWIG_1'))

# Start of JoinSeriesImageFilter_Execute

`JoinSeriesImageFilter_Execute__SWIG_2` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_JoinSeriesImageFilter_Execute__SWIG_2', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeriesImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeriesImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__JoinSeriesImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`JoinSeriesImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('JoinSeriesImageFilter_Execute__SWIG_2'))

# Start of JoinSeriesImageFilter_Execute

`JoinSeriesImageFilter_Execute__SWIG_3` = function(self, image1, image2, image3, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_JoinSeriesImageFilter_Execute__SWIG_3', self, image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeriesImageFilter_Execute__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeriesImageFilter_Execute__SWIG_3`, "inputTypes") = c('_p_itk__simple__JoinSeriesImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`JoinSeriesImageFilter_Execute__SWIG_3`) = c("SWIGFunction", class('JoinSeriesImageFilter_Execute__SWIG_3'))

# Start of JoinSeriesImageFilter_Execute

`JoinSeriesImageFilter_Execute__SWIG_4` = function(self, image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_JoinSeriesImageFilter_Execute__SWIG_4', self, image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeriesImageFilter_Execute__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeriesImageFilter_Execute__SWIG_4`, "inputTypes") = c('_p_itk__simple__JoinSeriesImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`JoinSeriesImageFilter_Execute__SWIG_4`) = c("SWIGFunction", class('JoinSeriesImageFilter_Execute__SWIG_4'))

# Start of JoinSeriesImageFilter_Execute

`JoinSeriesImageFilter_Execute__SWIG_5` = function(self, image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_JoinSeriesImageFilter_Execute__SWIG_5', self, image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeriesImageFilter_Execute__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeriesImageFilter_Execute__SWIG_5`, "inputTypes") = c('_p_itk__simple__JoinSeriesImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`JoinSeriesImageFilter_Execute__SWIG_5`) = c("SWIGFunction", class('JoinSeriesImageFilter_Execute__SWIG_5'))

`JoinSeriesImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__JoinSeriesImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- JoinSeriesImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__JoinSeriesImageFilter') || is.null(argv[[1]]) ) && (
        (is.list(argv[[2]]) & all(sapply(argv[[2]], inherits, what='_p_itk__simple__Image')))
        )) {
      f <- JoinSeriesImageFilter_Execute__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__JoinSeriesImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- JoinSeriesImageFilter_Execute__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__JoinSeriesImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- JoinSeriesImageFilter_Execute__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__JoinSeriesImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- JoinSeriesImageFilter_Execute__SWIG_4; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__JoinSeriesImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( extends(argtypes[6], '_p_itk__simple__Image') && length(argv[[6]]) == 1 )) {
      f <- JoinSeriesImageFilter_Execute__SWIG_5; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for JoinSeriesImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::JoinSeriesImageFilter
setMethod('$', '_p_itk__simple__JoinSeriesImageFilter', function(x, name)

{
  accessorFuns = list('SetOrigin' = JoinSeriesImageFilter_SetOrigin, 'GetOrigin' = JoinSeriesImageFilter_GetOrigin, 'SetSpacing' = JoinSeriesImageFilter_SetSpacing, 'GetSpacing' = JoinSeriesImageFilter_GetSpacing, 'GetName' = JoinSeriesImageFilter_GetName, 'ToString' = JoinSeriesImageFilter_ToString, 'Execute' = JoinSeriesImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::JoinSeriesImageFilter
# Start of JoinSeries

`JoinSeries__SWIG_0` = function(images, origin, spacing, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  
  
  ;ans = .Call('R_swig_JoinSeries__SWIG_0', images, origin, spacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_0`, "inputTypes") = c('list', 'numeric', 'numeric')
class(`JoinSeries__SWIG_0`) = c("SWIGFunction", class('JoinSeries__SWIG_0'))

# Start of JoinSeries

`JoinSeries__SWIG_1` = function(images, origin, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  
  ;ans = .Call('R_swig_JoinSeries__SWIG_1', images, origin, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_1`, "inputTypes") = c('list', 'numeric')
class(`JoinSeries__SWIG_1`) = c("SWIGFunction", class('JoinSeries__SWIG_1'))

# Start of JoinSeries

`JoinSeries__SWIG_2` = function(images, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_JoinSeries__SWIG_2', images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_2`, "inputTypes") = c('list')
class(`JoinSeries__SWIG_2`) = c("SWIGFunction", class('JoinSeries__SWIG_2'))

# Start of JoinSeries

`JoinSeries__SWIG_3` = function(image1, origin, spacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_JoinSeries__SWIG_3', image1, origin, spacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`JoinSeries__SWIG_3`) = c("SWIGFunction", class('JoinSeries__SWIG_3'))

# Start of JoinSeries

`JoinSeries__SWIG_4` = function(image1, origin, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_JoinSeries__SWIG_4', image1, origin, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`JoinSeries__SWIG_4`) = c("SWIGFunction", class('JoinSeries__SWIG_4'))

# Start of JoinSeries

`JoinSeries__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_JoinSeries__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`JoinSeries__SWIG_5`) = c("SWIGFunction", class('JoinSeries__SWIG_5'))

# Start of JoinSeries

`JoinSeries__SWIG_6` = function(image1, image2, origin, spacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  
  ;ans = .Call('R_swig_JoinSeries__SWIG_6', image1, image2, origin, spacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`JoinSeries__SWIG_6`) = c("SWIGFunction", class('JoinSeries__SWIG_6'))

# Start of JoinSeries

`JoinSeries__SWIG_7` = function(image1, image2, origin, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  ;ans = .Call('R_swig_JoinSeries__SWIG_7', image1, image2, origin, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`JoinSeries__SWIG_7`) = c("SWIGFunction", class('JoinSeries__SWIG_7'))

# Start of JoinSeries

`JoinSeries__SWIG_8` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_JoinSeries__SWIG_8', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`JoinSeries__SWIG_8`) = c("SWIGFunction", class('JoinSeries__SWIG_8'))

# Start of JoinSeries

`JoinSeries__SWIG_9` = function(image1, image2, image3, origin, spacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  
  
  ;ans = .Call('R_swig_JoinSeries__SWIG_9', image1, image2, image3, origin, spacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`JoinSeries__SWIG_9`) = c("SWIGFunction", class('JoinSeries__SWIG_9'))

# Start of JoinSeries

`JoinSeries__SWIG_10` = function(image1, image2, image3, origin, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  
  ;ans = .Call('R_swig_JoinSeries__SWIG_10', image1, image2, image3, origin, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`JoinSeries__SWIG_10`) = c("SWIGFunction", class('JoinSeries__SWIG_10'))

# Start of JoinSeries

`JoinSeries__SWIG_11` = function(image1, image2, image3, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_JoinSeries__SWIG_11', image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`JoinSeries__SWIG_11`) = c("SWIGFunction", class('JoinSeries__SWIG_11'))

# Start of JoinSeries

`JoinSeries__SWIG_12` = function(image1, image2, image3, image4, origin, spacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  
  
  ;ans = .Call('R_swig_JoinSeries__SWIG_12', image1, image2, image3, image4, origin, spacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_12`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_12`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`JoinSeries__SWIG_12`) = c("SWIGFunction", class('JoinSeries__SWIG_12'))

# Start of JoinSeries

`JoinSeries__SWIG_13` = function(image1, image2, image3, image4, origin, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  
  ;ans = .Call('R_swig_JoinSeries__SWIG_13', image1, image2, image3, image4, origin, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_13`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_13`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`JoinSeries__SWIG_13`) = c("SWIGFunction", class('JoinSeries__SWIG_13'))

# Start of JoinSeries

`JoinSeries__SWIG_14` = function(image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_JoinSeries__SWIG_14', image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_14`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_14`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`JoinSeries__SWIG_14`) = c("SWIGFunction", class('JoinSeries__SWIG_14'))

# Start of JoinSeries

`JoinSeries__SWIG_15` = function(image1, image2, image3, image4, image5, origin, spacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  
  
  ;ans = .Call('R_swig_JoinSeries__SWIG_15', image1, image2, image3, image4, image5, origin, spacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_15`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_15`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`JoinSeries__SWIG_15`) = c("SWIGFunction", class('JoinSeries__SWIG_15'))

# Start of JoinSeries

`JoinSeries__SWIG_16` = function(image1, image2, image3, image4, image5, origin, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  
  ;ans = .Call('R_swig_JoinSeries__SWIG_16', image1, image2, image3, image4, image5, origin, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_16`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_16`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`JoinSeries__SWIG_16`) = c("SWIGFunction", class('JoinSeries__SWIG_16'))

# Start of JoinSeries

`JoinSeries__SWIG_17` = function(image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_JoinSeries__SWIG_17', image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`JoinSeries__SWIG_17`, 'returnType') = '_p_itk__simple__Image'
attr(`JoinSeries__SWIG_17`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`JoinSeries__SWIG_17`) = c("SWIGFunction", class('JoinSeries__SWIG_17'))

`JoinSeries` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 18
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- JoinSeries__SWIG_5; 
    }
    else if (
      (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
      ) {
      f <- JoinSeries__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- JoinSeries__SWIG_8; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- JoinSeries__SWIG_4; 
    }
    else if ((
        (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
        ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- JoinSeries__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- JoinSeries__SWIG_11; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- JoinSeries__SWIG_7; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- JoinSeries__SWIG_3; 
    }
    else if ((
        (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
        ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- JoinSeries__SWIG_0; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- JoinSeries__SWIG_14; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- JoinSeries__SWIG_10; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- JoinSeries__SWIG_6; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- JoinSeries__SWIG_17; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- JoinSeries__SWIG_13; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- JoinSeries__SWIG_9; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- JoinSeries__SWIG_16; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- JoinSeries__SWIG_12; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- JoinSeries__SWIG_15; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for JoinSeries with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_KittlerIllingworthThresholdImageFilter

`delete_KittlerIllingworthThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_KittlerIllingworthThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_KittlerIllingworthThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_KittlerIllingworthThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter')
class(`delete_KittlerIllingworthThresholdImageFilter`) = c("SWIGFunction", class('delete_KittlerIllingworthThresholdImageFilter'))

# Start of new_KittlerIllingworthThresholdImageFilter

`KittlerIllingworthThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_KittlerIllingworthThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__KittlerIllingworthThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_KittlerIllingworthThresholdImageFilter);
  ans
  
}

attr(`KittlerIllingworthThresholdImageFilter`, 'returnType') = '_p_itk__simple__KittlerIllingworthThresholdImageFilter'
class(`KittlerIllingworthThresholdImageFilter`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter'))

# Start of KittlerIllingworthThresholdImageFilter_SetInsideValue

`KittlerIllingworthThresholdImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideValue = as.integer(InsideValue);
  
  if(length(InsideValue) > 1) {
    warning("using only the first element of InsideValue");
  };
  
  ;ans = .Call('R_swig_KittlerIllingworthThresholdImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`KittlerIllingworthThresholdImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`KittlerIllingworthThresholdImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter', 'integer')
class(`KittlerIllingworthThresholdImageFilter_SetInsideValue`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_SetInsideValue'))

# Start of KittlerIllingworthThresholdImageFilter_GetInsideValue

`KittlerIllingworthThresholdImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_KittlerIllingworthThresholdImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`KittlerIllingworthThresholdImageFilter_GetInsideValue`, 'returnType') = 'integer'
attr(`KittlerIllingworthThresholdImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter')
class(`KittlerIllingworthThresholdImageFilter_GetInsideValue`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_GetInsideValue'))

# Start of KittlerIllingworthThresholdImageFilter_SetOutsideValue

`KittlerIllingworthThresholdImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutsideValue = as.integer(OutsideValue);
  
  if(length(OutsideValue) > 1) {
    warning("using only the first element of OutsideValue");
  };
  
  ;ans = .Call('R_swig_KittlerIllingworthThresholdImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`KittlerIllingworthThresholdImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`KittlerIllingworthThresholdImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter', 'integer')
class(`KittlerIllingworthThresholdImageFilter_SetOutsideValue`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_SetOutsideValue'))

# Start of KittlerIllingworthThresholdImageFilter_GetOutsideValue

`KittlerIllingworthThresholdImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_KittlerIllingworthThresholdImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`KittlerIllingworthThresholdImageFilter_GetOutsideValue`, 'returnType') = 'integer'
attr(`KittlerIllingworthThresholdImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter')
class(`KittlerIllingworthThresholdImageFilter_GetOutsideValue`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_GetOutsideValue'))

# Start of KittlerIllingworthThresholdImageFilter_SetNumberOfHistogramBins

`KittlerIllingworthThresholdImageFilter_SetNumberOfHistogramBins` = function(self, NumberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfHistogramBins = as.integer(NumberOfHistogramBins);
  
  if(length(NumberOfHistogramBins) > 1) {
    warning("using only the first element of NumberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_KittlerIllingworthThresholdImageFilter_SetNumberOfHistogramBins', self, NumberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`KittlerIllingworthThresholdImageFilter_SetNumberOfHistogramBins`, 'returnType') = 'void'
attr(`KittlerIllingworthThresholdImageFilter_SetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter', 'integer')
class(`KittlerIllingworthThresholdImageFilter_SetNumberOfHistogramBins`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_SetNumberOfHistogramBins'))

# Start of KittlerIllingworthThresholdImageFilter_GetNumberOfHistogramBins

`KittlerIllingworthThresholdImageFilter_GetNumberOfHistogramBins` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_KittlerIllingworthThresholdImageFilter_GetNumberOfHistogramBins', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`KittlerIllingworthThresholdImageFilter_GetNumberOfHistogramBins`, 'returnType') = 'integer'
attr(`KittlerIllingworthThresholdImageFilter_GetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter')
class(`KittlerIllingworthThresholdImageFilter_GetNumberOfHistogramBins`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_GetNumberOfHistogramBins'))

# Start of KittlerIllingworthThresholdImageFilter_SetMaskOutput

`KittlerIllingworthThresholdImageFilter_SetMaskOutput` = function(self, MaskOutput)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskOutput = as.logical(MaskOutput);
  ;ans = .Call('R_swig_KittlerIllingworthThresholdImageFilter_SetMaskOutput', self, MaskOutput, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`KittlerIllingworthThresholdImageFilter_SetMaskOutput`, 'returnType') = 'void'
attr(`KittlerIllingworthThresholdImageFilter_SetMaskOutput`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter', 'logical')
class(`KittlerIllingworthThresholdImageFilter_SetMaskOutput`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_SetMaskOutput'))

# Start of KittlerIllingworthThresholdImageFilter_MaskOutputOn

`KittlerIllingworthThresholdImageFilter_MaskOutputOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_KittlerIllingworthThresholdImageFilter_MaskOutputOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`KittlerIllingworthThresholdImageFilter_MaskOutputOn`, 'returnType') = 'void'
attr(`KittlerIllingworthThresholdImageFilter_MaskOutputOn`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter')
class(`KittlerIllingworthThresholdImageFilter_MaskOutputOn`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_MaskOutputOn'))

# Start of KittlerIllingworthThresholdImageFilter_MaskOutputOff

`KittlerIllingworthThresholdImageFilter_MaskOutputOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_KittlerIllingworthThresholdImageFilter_MaskOutputOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`KittlerIllingworthThresholdImageFilter_MaskOutputOff`, 'returnType') = 'void'
attr(`KittlerIllingworthThresholdImageFilter_MaskOutputOff`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter')
class(`KittlerIllingworthThresholdImageFilter_MaskOutputOff`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_MaskOutputOff'))

# Start of KittlerIllingworthThresholdImageFilter_GetMaskOutput

`KittlerIllingworthThresholdImageFilter_GetMaskOutput` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_KittlerIllingworthThresholdImageFilter_GetMaskOutput', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`KittlerIllingworthThresholdImageFilter_GetMaskOutput`, 'returnType') = 'logical'
attr(`KittlerIllingworthThresholdImageFilter_GetMaskOutput`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter')
class(`KittlerIllingworthThresholdImageFilter_GetMaskOutput`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_GetMaskOutput'))

# Start of KittlerIllingworthThresholdImageFilter_SetMaskValue

`KittlerIllingworthThresholdImageFilter_SetMaskValue` = function(self, MaskValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskValue = as.integer(MaskValue);
  
  if(length(MaskValue) > 1) {
    warning("using only the first element of MaskValue");
  };
  
  ;ans = .Call('R_swig_KittlerIllingworthThresholdImageFilter_SetMaskValue', self, MaskValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`KittlerIllingworthThresholdImageFilter_SetMaskValue`, 'returnType') = 'void'
attr(`KittlerIllingworthThresholdImageFilter_SetMaskValue`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter', 'integer')
class(`KittlerIllingworthThresholdImageFilter_SetMaskValue`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_SetMaskValue'))

# Start of KittlerIllingworthThresholdImageFilter_GetMaskValue

`KittlerIllingworthThresholdImageFilter_GetMaskValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_KittlerIllingworthThresholdImageFilter_GetMaskValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`KittlerIllingworthThresholdImageFilter_GetMaskValue`, 'returnType') = 'integer'
attr(`KittlerIllingworthThresholdImageFilter_GetMaskValue`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter')
class(`KittlerIllingworthThresholdImageFilter_GetMaskValue`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_GetMaskValue'))

# Start of KittlerIllingworthThresholdImageFilter_GetThreshold

`KittlerIllingworthThresholdImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_KittlerIllingworthThresholdImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`KittlerIllingworthThresholdImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`KittlerIllingworthThresholdImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter')
class(`KittlerIllingworthThresholdImageFilter_GetThreshold`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_GetThreshold'))

# Start of KittlerIllingworthThresholdImageFilter_GetName

`KittlerIllingworthThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_KittlerIllingworthThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`KittlerIllingworthThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`KittlerIllingworthThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter')
class(`KittlerIllingworthThresholdImageFilter_GetName`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_GetName'))

# Start of KittlerIllingworthThresholdImageFilter_ToString

`KittlerIllingworthThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_KittlerIllingworthThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`KittlerIllingworthThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`KittlerIllingworthThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter')
class(`KittlerIllingworthThresholdImageFilter_ToString`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_ToString'))

# Start of KittlerIllingworthThresholdImageFilter_Execute

`KittlerIllingworthThresholdImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_KittlerIllingworthThresholdImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`KittlerIllingworthThresholdImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`KittlerIllingworthThresholdImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`KittlerIllingworthThresholdImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_Execute__SWIG_0'))

# Start of KittlerIllingworthThresholdImageFilter_Execute

`KittlerIllingworthThresholdImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_KittlerIllingworthThresholdImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`KittlerIllingworthThresholdImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`KittlerIllingworthThresholdImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__KittlerIllingworthThresholdImageFilter', '_p_itk__simple__Image')
class(`KittlerIllingworthThresholdImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('KittlerIllingworthThresholdImageFilter_Execute__SWIG_1'))

`KittlerIllingworthThresholdImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__KittlerIllingworthThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- KittlerIllingworthThresholdImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__KittlerIllingworthThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- KittlerIllingworthThresholdImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for KittlerIllingworthThresholdImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::KittlerIllingworthThresholdImageFilter
setMethod('$', '_p_itk__simple__KittlerIllingworthThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetInsideValue' = KittlerIllingworthThresholdImageFilter_SetInsideValue, 'GetInsideValue' = KittlerIllingworthThresholdImageFilter_GetInsideValue, 'SetOutsideValue' = KittlerIllingworthThresholdImageFilter_SetOutsideValue, 'GetOutsideValue' = KittlerIllingworthThresholdImageFilter_GetOutsideValue, 'SetNumberOfHistogramBins' = KittlerIllingworthThresholdImageFilter_SetNumberOfHistogramBins, 'GetNumberOfHistogramBins' = KittlerIllingworthThresholdImageFilter_GetNumberOfHistogramBins, 'SetMaskOutput' = KittlerIllingworthThresholdImageFilter_SetMaskOutput, 'MaskOutputOn' = KittlerIllingworthThresholdImageFilter_MaskOutputOn, 'MaskOutputOff' = KittlerIllingworthThresholdImageFilter_MaskOutputOff, 'GetMaskOutput' = KittlerIllingworthThresholdImageFilter_GetMaskOutput, 'SetMaskValue' = KittlerIllingworthThresholdImageFilter_SetMaskValue, 'GetMaskValue' = KittlerIllingworthThresholdImageFilter_GetMaskValue, 'GetThreshold' = KittlerIllingworthThresholdImageFilter_GetThreshold, 'GetName' = KittlerIllingworthThresholdImageFilter_GetName, 'ToString' = KittlerIllingworthThresholdImageFilter_ToString, 'Execute' = KittlerIllingworthThresholdImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::KittlerIllingworthThresholdImageFilter
# Start of KittlerIllingworthThreshold

`KittlerIllingworthThreshold__SWIG_0` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_KittlerIllingworthThreshold__SWIG_0', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`KittlerIllingworthThreshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`KittlerIllingworthThreshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`KittlerIllingworthThreshold__SWIG_0`) = c("SWIGFunction", class('KittlerIllingworthThreshold__SWIG_0'))

# Start of KittlerIllingworthThreshold

`KittlerIllingworthThreshold__SWIG_1` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_KittlerIllingworthThreshold__SWIG_1', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`KittlerIllingworthThreshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`KittlerIllingworthThreshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`KittlerIllingworthThreshold__SWIG_1`) = c("SWIGFunction", class('KittlerIllingworthThreshold__SWIG_1'))

# Start of KittlerIllingworthThreshold

`KittlerIllingworthThreshold__SWIG_2` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_KittlerIllingworthThreshold__SWIG_2', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`KittlerIllingworthThreshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`KittlerIllingworthThreshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`KittlerIllingworthThreshold__SWIG_2`) = c("SWIGFunction", class('KittlerIllingworthThreshold__SWIG_2'))

# Start of KittlerIllingworthThreshold

`KittlerIllingworthThreshold__SWIG_3` = function(image, maskImage, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_KittlerIllingworthThreshold__SWIG_3', image, maskImage, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`KittlerIllingworthThreshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`KittlerIllingworthThreshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`KittlerIllingworthThreshold__SWIG_3`) = c("SWIGFunction", class('KittlerIllingworthThreshold__SWIG_3'))

# Start of KittlerIllingworthThreshold

`KittlerIllingworthThreshold__SWIG_4` = function(image, maskImage, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_KittlerIllingworthThreshold__SWIG_4', image, maskImage, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`KittlerIllingworthThreshold__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`KittlerIllingworthThreshold__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`KittlerIllingworthThreshold__SWIG_4`) = c("SWIGFunction", class('KittlerIllingworthThreshold__SWIG_4'))

# Start of KittlerIllingworthThreshold

`KittlerIllingworthThreshold__SWIG_5` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_KittlerIllingworthThreshold__SWIG_5', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`KittlerIllingworthThreshold__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`KittlerIllingworthThreshold__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`KittlerIllingworthThreshold__SWIG_5`) = c("SWIGFunction", class('KittlerIllingworthThreshold__SWIG_5'))

# Start of KittlerIllingworthThreshold

`KittlerIllingworthThreshold__SWIG_6` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_KittlerIllingworthThreshold__SWIG_6', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`KittlerIllingworthThreshold__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`KittlerIllingworthThreshold__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`KittlerIllingworthThreshold__SWIG_6`) = c("SWIGFunction", class('KittlerIllingworthThreshold__SWIG_6'))

# Start of KittlerIllingworthThreshold

`KittlerIllingworthThreshold__SWIG_7` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_KittlerIllingworthThreshold__SWIG_7', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`KittlerIllingworthThreshold__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`KittlerIllingworthThreshold__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`KittlerIllingworthThreshold__SWIG_7`) = c("SWIGFunction", class('KittlerIllingworthThreshold__SWIG_7'))

# Start of KittlerIllingworthThreshold

`KittlerIllingworthThreshold__SWIG_8` = function(image, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_KittlerIllingworthThreshold__SWIG_8', image, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`KittlerIllingworthThreshold__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`KittlerIllingworthThreshold__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`KittlerIllingworthThreshold__SWIG_8`) = c("SWIGFunction", class('KittlerIllingworthThreshold__SWIG_8'))

# Start of KittlerIllingworthThreshold

`KittlerIllingworthThreshold__SWIG_9` = function(image, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_KittlerIllingworthThreshold__SWIG_9', image, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`KittlerIllingworthThreshold__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`KittlerIllingworthThreshold__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`KittlerIllingworthThreshold__SWIG_9`) = c("SWIGFunction", class('KittlerIllingworthThreshold__SWIG_9'))

# Start of KittlerIllingworthThreshold

`KittlerIllingworthThreshold__SWIG_10` = function(image, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_KittlerIllingworthThreshold__SWIG_10', image, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`KittlerIllingworthThreshold__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`KittlerIllingworthThreshold__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`KittlerIllingworthThreshold__SWIG_10`) = c("SWIGFunction", class('KittlerIllingworthThreshold__SWIG_10'))

# Start of KittlerIllingworthThreshold

`KittlerIllingworthThreshold__SWIG_11` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_KittlerIllingworthThreshold__SWIG_11', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`KittlerIllingworthThreshold__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`KittlerIllingworthThreshold__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image')
class(`KittlerIllingworthThreshold__SWIG_11`) = c("SWIGFunction", class('KittlerIllingworthThreshold__SWIG_11'))

`KittlerIllingworthThreshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 12
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- KittlerIllingworthThreshold__SWIG_11; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- KittlerIllingworthThreshold__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- KittlerIllingworthThreshold__SWIG_10; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- KittlerIllingworthThreshold__SWIG_4; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- KittlerIllingworthThreshold__SWIG_9; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- KittlerIllingworthThreshold__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- KittlerIllingworthThreshold__SWIG_8; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- KittlerIllingworthThreshold__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- KittlerIllingworthThreshold__SWIG_7; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- KittlerIllingworthThreshold__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- KittlerIllingworthThreshold__SWIG_6; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- KittlerIllingworthThreshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for KittlerIllingworthThreshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LabelContourImageFilter

`delete_LabelContourImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelContourImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelContourImageFilter`, 'returnType') = 'void'
attr(`delete_LabelContourImageFilter`, "inputTypes") = c('_p_itk__simple__LabelContourImageFilter')
class(`delete_LabelContourImageFilter`) = c("SWIGFunction", class('delete_LabelContourImageFilter'))

# Start of new_LabelContourImageFilter

`LabelContourImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelContourImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelContourImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelContourImageFilter);
  ans
  
}

attr(`LabelContourImageFilter`, 'returnType') = '_p_itk__simple__LabelContourImageFilter'
class(`LabelContourImageFilter`) = c("SWIGFunction", class('LabelContourImageFilter'))

# Start of LabelContourImageFilter_SetFullyConnected

`LabelContourImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_LabelContourImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelContourImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`LabelContourImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__LabelContourImageFilter', 'logical')
class(`LabelContourImageFilter_SetFullyConnected`) = c("SWIGFunction", class('LabelContourImageFilter_SetFullyConnected'))

# Start of LabelContourImageFilter_FullyConnectedOn

`LabelContourImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelContourImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelContourImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`LabelContourImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__LabelContourImageFilter')
class(`LabelContourImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('LabelContourImageFilter_FullyConnectedOn'))

# Start of LabelContourImageFilter_FullyConnectedOff

`LabelContourImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelContourImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelContourImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`LabelContourImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__LabelContourImageFilter')
class(`LabelContourImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('LabelContourImageFilter_FullyConnectedOff'))

# Start of LabelContourImageFilter_GetFullyConnected

`LabelContourImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelContourImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelContourImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`LabelContourImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__LabelContourImageFilter')
class(`LabelContourImageFilter_GetFullyConnected`) = c("SWIGFunction", class('LabelContourImageFilter_GetFullyConnected'))

# Start of LabelContourImageFilter_SetBackgroundValue

`LabelContourImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LabelContourImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelContourImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`LabelContourImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelContourImageFilter', 'numeric')
class(`LabelContourImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('LabelContourImageFilter_SetBackgroundValue'))

# Start of LabelContourImageFilter_GetBackgroundValue

`LabelContourImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelContourImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelContourImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`LabelContourImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelContourImageFilter')
class(`LabelContourImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('LabelContourImageFilter_GetBackgroundValue'))

# Start of LabelContourImageFilter_GetName

`LabelContourImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelContourImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelContourImageFilter_GetName`, 'returnType') = 'character'
attr(`LabelContourImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelContourImageFilter')
class(`LabelContourImageFilter_GetName`) = c("SWIGFunction", class('LabelContourImageFilter_GetName'))

# Start of LabelContourImageFilter_ToString

`LabelContourImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelContourImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelContourImageFilter_ToString`, 'returnType') = 'character'
attr(`LabelContourImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelContourImageFilter')
class(`LabelContourImageFilter_ToString`) = c("SWIGFunction", class('LabelContourImageFilter_ToString'))

# Start of LabelContourImageFilter_Execute

`LabelContourImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelContourImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelContourImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelContourImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LabelContourImageFilter', '_p_itk__simple__Image')
class(`LabelContourImageFilter_Execute`) = c("SWIGFunction", class('LabelContourImageFilter_Execute'))

# Start of accessor method for itk::simple::LabelContourImageFilter
setMethod('$', '_p_itk__simple__LabelContourImageFilter', function(x, name)

{
  accessorFuns = list('SetFullyConnected' = LabelContourImageFilter_SetFullyConnected, 'FullyConnectedOn' = LabelContourImageFilter_FullyConnectedOn, 'FullyConnectedOff' = LabelContourImageFilter_FullyConnectedOff, 'GetFullyConnected' = LabelContourImageFilter_GetFullyConnected, 'SetBackgroundValue' = LabelContourImageFilter_SetBackgroundValue, 'GetBackgroundValue' = LabelContourImageFilter_GetBackgroundValue, 'GetName' = LabelContourImageFilter_GetName, 'ToString' = LabelContourImageFilter_ToString, 'Execute' = LabelContourImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelContourImageFilter
# Start of LabelContour

`LabelContour__SWIG_0` = function(image1, fullyConnected, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  
  ;ans = .Call('R_swig_LabelContour__SWIG_0', image1, fullyConnected, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelContour__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelContour__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'numeric')
class(`LabelContour__SWIG_0`) = c("SWIGFunction", class('LabelContour__SWIG_0'))

# Start of LabelContour

`LabelContour__SWIG_1` = function(image1, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_LabelContour__SWIG_1', image1, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelContour__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelContour__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`LabelContour__SWIG_1`) = c("SWIGFunction", class('LabelContour__SWIG_1'))

# Start of LabelContour

`LabelContour__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelContour__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelContour__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelContour__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`LabelContour__SWIG_2`) = c("SWIGFunction", class('LabelContour__SWIG_2'))

`LabelContour` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- LabelContour__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- LabelContour__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- LabelContour__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelContour with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LabelImageToLabelMapFilter

`delete_LabelImageToLabelMapFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelImageToLabelMapFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelImageToLabelMapFilter`, 'returnType') = 'void'
attr(`delete_LabelImageToLabelMapFilter`, "inputTypes") = c('_p_itk__simple__LabelImageToLabelMapFilter')
class(`delete_LabelImageToLabelMapFilter`) = c("SWIGFunction", class('delete_LabelImageToLabelMapFilter'))

# Start of new_LabelImageToLabelMapFilter

`LabelImageToLabelMapFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelImageToLabelMapFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelImageToLabelMapFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelImageToLabelMapFilter);
  ans
  
}

attr(`LabelImageToLabelMapFilter`, 'returnType') = '_p_itk__simple__LabelImageToLabelMapFilter'
class(`LabelImageToLabelMapFilter`) = c("SWIGFunction", class('LabelImageToLabelMapFilter'))

# Start of LabelImageToLabelMapFilter_SetBackgroundValue

`LabelImageToLabelMapFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LabelImageToLabelMapFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelImageToLabelMapFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`LabelImageToLabelMapFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelImageToLabelMapFilter', 'numeric')
class(`LabelImageToLabelMapFilter_SetBackgroundValue`) = c("SWIGFunction", class('LabelImageToLabelMapFilter_SetBackgroundValue'))

# Start of LabelImageToLabelMapFilter_GetBackgroundValue

`LabelImageToLabelMapFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelImageToLabelMapFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelImageToLabelMapFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`LabelImageToLabelMapFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelImageToLabelMapFilter')
class(`LabelImageToLabelMapFilter_GetBackgroundValue`) = c("SWIGFunction", class('LabelImageToLabelMapFilter_GetBackgroundValue'))

# Start of LabelImageToLabelMapFilter_GetName

`LabelImageToLabelMapFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelImageToLabelMapFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelImageToLabelMapFilter_GetName`, 'returnType') = 'character'
attr(`LabelImageToLabelMapFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelImageToLabelMapFilter')
class(`LabelImageToLabelMapFilter_GetName`) = c("SWIGFunction", class('LabelImageToLabelMapFilter_GetName'))

# Start of LabelImageToLabelMapFilter_ToString

`LabelImageToLabelMapFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelImageToLabelMapFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelImageToLabelMapFilter_ToString`, 'returnType') = 'character'
attr(`LabelImageToLabelMapFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelImageToLabelMapFilter')
class(`LabelImageToLabelMapFilter_ToString`) = c("SWIGFunction", class('LabelImageToLabelMapFilter_ToString'))

# Start of LabelImageToLabelMapFilter_Execute

`LabelImageToLabelMapFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelImageToLabelMapFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelImageToLabelMapFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelImageToLabelMapFilter_Execute`, "inputTypes") = c('_p_itk__simple__LabelImageToLabelMapFilter', '_p_itk__simple__Image')
class(`LabelImageToLabelMapFilter_Execute`) = c("SWIGFunction", class('LabelImageToLabelMapFilter_Execute'))

# Start of accessor method for itk::simple::LabelImageToLabelMapFilter
setMethod('$', '_p_itk__simple__LabelImageToLabelMapFilter', function(x, name)

{
  accessorFuns = list('SetBackgroundValue' = LabelImageToLabelMapFilter_SetBackgroundValue, 'GetBackgroundValue' = LabelImageToLabelMapFilter_GetBackgroundValue, 'GetName' = LabelImageToLabelMapFilter_GetName, 'ToString' = LabelImageToLabelMapFilter_ToString, 'Execute' = LabelImageToLabelMapFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelImageToLabelMapFilter
# Start of LabelImageToLabelMap

`LabelImageToLabelMap__SWIG_0` = function(image1, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_LabelImageToLabelMap__SWIG_0', image1, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelImageToLabelMap__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelImageToLabelMap__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`LabelImageToLabelMap__SWIG_0`) = c("SWIGFunction", class('LabelImageToLabelMap__SWIG_0'))

# Start of LabelImageToLabelMap

`LabelImageToLabelMap__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelImageToLabelMap__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelImageToLabelMap__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelImageToLabelMap__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`LabelImageToLabelMap__SWIG_1`) = c("SWIGFunction", class('LabelImageToLabelMap__SWIG_1'))

`LabelImageToLabelMap` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- LabelImageToLabelMap__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- LabelImageToLabelMap__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelImageToLabelMap with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LabelIntensityStatisticsImageFilter

`delete_LabelIntensityStatisticsImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelIntensityStatisticsImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelIntensityStatisticsImageFilter`, 'returnType') = 'void'
attr(`delete_LabelIntensityStatisticsImageFilter`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter')
class(`delete_LabelIntensityStatisticsImageFilter`) = c("SWIGFunction", class('delete_LabelIntensityStatisticsImageFilter'))

# Start of new_LabelIntensityStatisticsImageFilter

`LabelIntensityStatisticsImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelIntensityStatisticsImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelIntensityStatisticsImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelIntensityStatisticsImageFilter);
  ans
  
}

attr(`LabelIntensityStatisticsImageFilter`, 'returnType') = '_p_itk__simple__LabelIntensityStatisticsImageFilter'
class(`LabelIntensityStatisticsImageFilter`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter'))

# Start of LabelIntensityStatisticsImageFilter_SetBackgroundValue

`LabelIntensityStatisticsImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LabelIntensityStatisticsImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelIntensityStatisticsImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`LabelIntensityStatisticsImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'numeric')
class(`LabelIntensityStatisticsImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_SetBackgroundValue'))

# Start of LabelIntensityStatisticsImageFilter_GetBackgroundValue

`LabelIntensityStatisticsImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter')
class(`LabelIntensityStatisticsImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetBackgroundValue'))

# Start of LabelIntensityStatisticsImageFilter_SetComputeFeretDiameter

`LabelIntensityStatisticsImageFilter_SetComputeFeretDiameter` = function(self, ComputeFeretDiameter)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ComputeFeretDiameter = as.logical(ComputeFeretDiameter);
  ;ans = .Call('R_swig_LabelIntensityStatisticsImageFilter_SetComputeFeretDiameter', self, ComputeFeretDiameter, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelIntensityStatisticsImageFilter_SetComputeFeretDiameter`, 'returnType') = 'void'
attr(`LabelIntensityStatisticsImageFilter_SetComputeFeretDiameter`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'logical')
class(`LabelIntensityStatisticsImageFilter_SetComputeFeretDiameter`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_SetComputeFeretDiameter'))

# Start of LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOn

`LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOn`, 'returnType') = 'void'
attr(`LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOn`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter')
class(`LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOn`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOn'))

# Start of LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOff

`LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOff`, 'returnType') = 'void'
attr(`LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOff`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter')
class(`LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOff`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOff'))

# Start of LabelIntensityStatisticsImageFilter_GetComputeFeretDiameter

`LabelIntensityStatisticsImageFilter_GetComputeFeretDiameter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetComputeFeretDiameter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetComputeFeretDiameter`, 'returnType') = 'logical'
attr(`LabelIntensityStatisticsImageFilter_GetComputeFeretDiameter`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter')
class(`LabelIntensityStatisticsImageFilter_GetComputeFeretDiameter`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetComputeFeretDiameter'))

# Start of LabelIntensityStatisticsImageFilter_SetComputePerimeter

`LabelIntensityStatisticsImageFilter_SetComputePerimeter` = function(self, ComputePerimeter)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ComputePerimeter = as.logical(ComputePerimeter);
  ;ans = .Call('R_swig_LabelIntensityStatisticsImageFilter_SetComputePerimeter', self, ComputePerimeter, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelIntensityStatisticsImageFilter_SetComputePerimeter`, 'returnType') = 'void'
attr(`LabelIntensityStatisticsImageFilter_SetComputePerimeter`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'logical')
class(`LabelIntensityStatisticsImageFilter_SetComputePerimeter`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_SetComputePerimeter'))

# Start of LabelIntensityStatisticsImageFilter_ComputePerimeterOn

`LabelIntensityStatisticsImageFilter_ComputePerimeterOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelIntensityStatisticsImageFilter_ComputePerimeterOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelIntensityStatisticsImageFilter_ComputePerimeterOn`, 'returnType') = 'void'
attr(`LabelIntensityStatisticsImageFilter_ComputePerimeterOn`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter')
class(`LabelIntensityStatisticsImageFilter_ComputePerimeterOn`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_ComputePerimeterOn'))

# Start of LabelIntensityStatisticsImageFilter_ComputePerimeterOff

`LabelIntensityStatisticsImageFilter_ComputePerimeterOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelIntensityStatisticsImageFilter_ComputePerimeterOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelIntensityStatisticsImageFilter_ComputePerimeterOff`, 'returnType') = 'void'
attr(`LabelIntensityStatisticsImageFilter_ComputePerimeterOff`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter')
class(`LabelIntensityStatisticsImageFilter_ComputePerimeterOff`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_ComputePerimeterOff'))

# Start of LabelIntensityStatisticsImageFilter_GetComputePerimeter

`LabelIntensityStatisticsImageFilter_GetComputePerimeter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetComputePerimeter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetComputePerimeter`, 'returnType') = 'logical'
attr(`LabelIntensityStatisticsImageFilter_GetComputePerimeter`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter')
class(`LabelIntensityStatisticsImageFilter_GetComputePerimeter`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetComputePerimeter'))

# Start of LabelIntensityStatisticsImageFilter_SetNumberOfBins

`LabelIntensityStatisticsImageFilter_SetNumberOfBins` = function(self, NumberOfBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfBins = as.integer(NumberOfBins);
  
  if(length(NumberOfBins) > 1) {
    warning("using only the first element of NumberOfBins");
  };
  
  ;ans = .Call('R_swig_LabelIntensityStatisticsImageFilter_SetNumberOfBins', self, NumberOfBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelIntensityStatisticsImageFilter_SetNumberOfBins`, 'returnType') = 'void'
attr(`LabelIntensityStatisticsImageFilter_SetNumberOfBins`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_SetNumberOfBins`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_SetNumberOfBins'))

# Start of LabelIntensityStatisticsImageFilter_GetNumberOfBins

`LabelIntensityStatisticsImageFilter_GetNumberOfBins` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetNumberOfBins', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetNumberOfBins`, 'returnType') = 'integer'
attr(`LabelIntensityStatisticsImageFilter_GetNumberOfBins`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter')
class(`LabelIntensityStatisticsImageFilter_GetNumberOfBins`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetNumberOfBins'))

# Start of LabelIntensityStatisticsImageFilter_GetBoundingBox

`LabelIntensityStatisticsImageFilter_GetBoundingBox` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetBoundingBox', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetBoundingBox`, 'returnType') = 'integer'
attr(`LabelIntensityStatisticsImageFilter_GetBoundingBox`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetBoundingBox`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetBoundingBox'))

# Start of LabelIntensityStatisticsImageFilter_GetRegion

`LabelIntensityStatisticsImageFilter_GetRegion` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetRegion', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetRegion`, 'returnType') = 'integer'
attr(`LabelIntensityStatisticsImageFilter_GetRegion`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetRegion`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetRegion'))

# Start of LabelIntensityStatisticsImageFilter_GetCentroid

`LabelIntensityStatisticsImageFilter_GetCentroid` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetCentroid', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetCentroid`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetCentroid`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetCentroid`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetCentroid'))

# Start of LabelIntensityStatisticsImageFilter_GetElongation

`LabelIntensityStatisticsImageFilter_GetElongation` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetElongation', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetElongation`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetElongation`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetElongation`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetElongation'))

# Start of LabelIntensityStatisticsImageFilter_GetEquivalentEllipsoidDiameter

`LabelIntensityStatisticsImageFilter_GetEquivalentEllipsoidDiameter` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetEquivalentEllipsoidDiameter', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetEquivalentEllipsoidDiameter`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetEquivalentEllipsoidDiameter`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetEquivalentEllipsoidDiameter`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetEquivalentEllipsoidDiameter'))

# Start of LabelIntensityStatisticsImageFilter_GetEquivalentSphericalPerimeter

`LabelIntensityStatisticsImageFilter_GetEquivalentSphericalPerimeter` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetEquivalentSphericalPerimeter', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetEquivalentSphericalPerimeter`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetEquivalentSphericalPerimeter`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetEquivalentSphericalPerimeter`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetEquivalentSphericalPerimeter'))

# Start of LabelIntensityStatisticsImageFilter_GetEquivalentSphericalRadius

`LabelIntensityStatisticsImageFilter_GetEquivalentSphericalRadius` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetEquivalentSphericalRadius', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetEquivalentSphericalRadius`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetEquivalentSphericalRadius`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetEquivalentSphericalRadius`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetEquivalentSphericalRadius'))

# Start of LabelIntensityStatisticsImageFilter_GetFeretDiameter

`LabelIntensityStatisticsImageFilter_GetFeretDiameter` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetFeretDiameter', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetFeretDiameter`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetFeretDiameter`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetFeretDiameter`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetFeretDiameter'))

# Start of LabelIntensityStatisticsImageFilter_GetFlatness

`LabelIntensityStatisticsImageFilter_GetFlatness` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetFlatness', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetFlatness`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetFlatness`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetFlatness`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetFlatness'))

# Start of LabelIntensityStatisticsImageFilter_GetLabels

`LabelIntensityStatisticsImageFilter_GetLabels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetLabels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetLabels`, 'returnType') = 'integer'
attr(`LabelIntensityStatisticsImageFilter_GetLabels`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter')
class(`LabelIntensityStatisticsImageFilter_GetLabels`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetLabels'))

# Start of LabelIntensityStatisticsImageFilter_GetNumberOfPixels

`LabelIntensityStatisticsImageFilter_GetNumberOfPixels` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetNumberOfPixels', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetNumberOfPixels`, 'returnType') = 'integer'
attr(`LabelIntensityStatisticsImageFilter_GetNumberOfPixels`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetNumberOfPixels`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetNumberOfPixels'))

# Start of LabelIntensityStatisticsImageFilter_GetNumberOfPixelsOnBorder

`LabelIntensityStatisticsImageFilter_GetNumberOfPixelsOnBorder` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetNumberOfPixelsOnBorder', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetNumberOfPixelsOnBorder`, 'returnType') = 'integer'
attr(`LabelIntensityStatisticsImageFilter_GetNumberOfPixelsOnBorder`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetNumberOfPixelsOnBorder`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetNumberOfPixelsOnBorder'))

# Start of LabelIntensityStatisticsImageFilter_GetPerimeter

`LabelIntensityStatisticsImageFilter_GetPerimeter` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetPerimeter', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetPerimeter`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetPerimeter`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetPerimeter`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetPerimeter'))

# Start of LabelIntensityStatisticsImageFilter_GetPerimeterOnBorder

`LabelIntensityStatisticsImageFilter_GetPerimeterOnBorder` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetPerimeterOnBorder', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetPerimeterOnBorder`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetPerimeterOnBorder`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetPerimeterOnBorder`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetPerimeterOnBorder'))

# Start of LabelIntensityStatisticsImageFilter_GetPerimeterOnBorderRatio

`LabelIntensityStatisticsImageFilter_GetPerimeterOnBorderRatio` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetPerimeterOnBorderRatio', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetPerimeterOnBorderRatio`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetPerimeterOnBorderRatio`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetPerimeterOnBorderRatio`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetPerimeterOnBorderRatio'))

# Start of LabelIntensityStatisticsImageFilter_GetPhysicalSize

`LabelIntensityStatisticsImageFilter_GetPhysicalSize` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetPhysicalSize', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetPhysicalSize`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetPhysicalSize`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetPhysicalSize`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetPhysicalSize'))

# Start of LabelIntensityStatisticsImageFilter_GetPrincipalAxes

`LabelIntensityStatisticsImageFilter_GetPrincipalAxes` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetPrincipalAxes', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetPrincipalAxes`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetPrincipalAxes`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetPrincipalAxes`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetPrincipalAxes'))

# Start of LabelIntensityStatisticsImageFilter_GetPrincipalMoments

`LabelIntensityStatisticsImageFilter_GetPrincipalMoments` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetPrincipalMoments', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetPrincipalMoments`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetPrincipalMoments`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetPrincipalMoments`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetPrincipalMoments'))

# Start of LabelIntensityStatisticsImageFilter_GetRoundness

`LabelIntensityStatisticsImageFilter_GetRoundness` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetRoundness', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetRoundness`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetRoundness`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetRoundness`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetRoundness'))

# Start of LabelIntensityStatisticsImageFilter_GetCenterOfGravity

`LabelIntensityStatisticsImageFilter_GetCenterOfGravity` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetCenterOfGravity', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetCenterOfGravity`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetCenterOfGravity`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetCenterOfGravity`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetCenterOfGravity'))

# Start of LabelIntensityStatisticsImageFilter_GetKurtosis

`LabelIntensityStatisticsImageFilter_GetKurtosis` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetKurtosis', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetKurtosis`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetKurtosis`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetKurtosis`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetKurtosis'))

# Start of LabelIntensityStatisticsImageFilter_GetMaximum

`LabelIntensityStatisticsImageFilter_GetMaximum` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetMaximum', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetMaximum`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetMaximum`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetMaximum`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetMaximum'))

# Start of LabelIntensityStatisticsImageFilter_GetMaximumIndex

`LabelIntensityStatisticsImageFilter_GetMaximumIndex` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetMaximumIndex', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetMaximumIndex`, 'returnType') = 'integer'
attr(`LabelIntensityStatisticsImageFilter_GetMaximumIndex`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetMaximumIndex`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetMaximumIndex'))

# Start of LabelIntensityStatisticsImageFilter_GetMean

`LabelIntensityStatisticsImageFilter_GetMean` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetMean', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetMean`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetMean`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetMean`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetMean'))

# Start of LabelIntensityStatisticsImageFilter_GetMedian

`LabelIntensityStatisticsImageFilter_GetMedian` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetMedian', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetMedian`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetMedian`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetMedian`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetMedian'))

# Start of LabelIntensityStatisticsImageFilter_GetMinimum

`LabelIntensityStatisticsImageFilter_GetMinimum` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetMinimum', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetMinimum`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetMinimum`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetMinimum`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetMinimum'))

# Start of LabelIntensityStatisticsImageFilter_GetMinimumIndex

`LabelIntensityStatisticsImageFilter_GetMinimumIndex` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetMinimumIndex', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetMinimumIndex`, 'returnType') = 'integer'
attr(`LabelIntensityStatisticsImageFilter_GetMinimumIndex`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetMinimumIndex`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetMinimumIndex'))

# Start of LabelIntensityStatisticsImageFilter_GetSkewness

`LabelIntensityStatisticsImageFilter_GetSkewness` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetSkewness', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetSkewness`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetSkewness`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetSkewness`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetSkewness'))

# Start of LabelIntensityStatisticsImageFilter_GetStandardDeviation

`LabelIntensityStatisticsImageFilter_GetStandardDeviation` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetStandardDeviation', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetStandardDeviation`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetStandardDeviation`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetStandardDeviation`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetStandardDeviation'))

# Start of LabelIntensityStatisticsImageFilter_GetSum

`LabelIntensityStatisticsImageFilter_GetSum` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetSum', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetSum`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetSum`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetSum`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetSum'))

# Start of LabelIntensityStatisticsImageFilter_GetVariance

`LabelIntensityStatisticsImageFilter_GetVariance` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetVariance', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetVariance`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetVariance`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetVariance`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetVariance'))

# Start of LabelIntensityStatisticsImageFilter_GetWeightedElongation

`LabelIntensityStatisticsImageFilter_GetWeightedElongation` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetWeightedElongation', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetWeightedElongation`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetWeightedElongation`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetWeightedElongation`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetWeightedElongation'))

# Start of LabelIntensityStatisticsImageFilter_GetWeightedFlatness

`LabelIntensityStatisticsImageFilter_GetWeightedFlatness` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetWeightedFlatness', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetWeightedFlatness`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetWeightedFlatness`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetWeightedFlatness`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetWeightedFlatness'))

# Start of LabelIntensityStatisticsImageFilter_GetWeightedPrincipalAxes

`LabelIntensityStatisticsImageFilter_GetWeightedPrincipalAxes` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetWeightedPrincipalAxes', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetWeightedPrincipalAxes`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetWeightedPrincipalAxes`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetWeightedPrincipalAxes`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetWeightedPrincipalAxes'))

# Start of LabelIntensityStatisticsImageFilter_GetWeightedPrincipalMoments

`LabelIntensityStatisticsImageFilter_GetWeightedPrincipalMoments` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetWeightedPrincipalMoments', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetWeightedPrincipalMoments`, 'returnType') = 'numeric'
attr(`LabelIntensityStatisticsImageFilter_GetWeightedPrincipalMoments`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_GetWeightedPrincipalMoments`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetWeightedPrincipalMoments'))

# Start of LabelIntensityStatisticsImageFilter_GetName

`LabelIntensityStatisticsImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetName`, 'returnType') = 'character'
attr(`LabelIntensityStatisticsImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter')
class(`LabelIntensityStatisticsImageFilter_GetName`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetName'))

# Start of LabelIntensityStatisticsImageFilter_ToString

`LabelIntensityStatisticsImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_ToString`, 'returnType') = 'character'
attr(`LabelIntensityStatisticsImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter')
class(`LabelIntensityStatisticsImageFilter_ToString`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_ToString'))

# Start of LabelIntensityStatisticsImageFilter_Execute

`LabelIntensityStatisticsImageFilter_Execute` = function(self, image, featureImage)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_LabelIntensityStatisticsImageFilter_Execute', self, image, featureImage, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelIntensityStatisticsImageFilter_Execute`, 'returnType') = 'void'
attr(`LabelIntensityStatisticsImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelIntensityStatisticsImageFilter_Execute`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_Execute'))

# Start of LabelIntensityStatisticsImageFilter_HasLabel

`LabelIntensityStatisticsImageFilter_HasLabel` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_HasLabel', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_HasLabel`, 'returnType') = 'logical'
attr(`LabelIntensityStatisticsImageFilter_HasLabel`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter', 'integer')
class(`LabelIntensityStatisticsImageFilter_HasLabel`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_HasLabel'))

# Start of LabelIntensityStatisticsImageFilter_GetNumberOfLabels

`LabelIntensityStatisticsImageFilter_GetNumberOfLabels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelIntensityStatisticsImageFilter_GetNumberOfLabels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelIntensityStatisticsImageFilter_GetNumberOfLabels`, 'returnType') = 'integer'
attr(`LabelIntensityStatisticsImageFilter_GetNumberOfLabels`, "inputTypes") = c('_p_itk__simple__LabelIntensityStatisticsImageFilter')
class(`LabelIntensityStatisticsImageFilter_GetNumberOfLabels`) = c("SWIGFunction", class('LabelIntensityStatisticsImageFilter_GetNumberOfLabels'))

# Start of accessor method for itk::simple::LabelIntensityStatisticsImageFilter
setMethod('$', '_p_itk__simple__LabelIntensityStatisticsImageFilter', function(x, name)

{
  accessorFuns = list('SetBackgroundValue' = LabelIntensityStatisticsImageFilter_SetBackgroundValue, 'GetBackgroundValue' = LabelIntensityStatisticsImageFilter_GetBackgroundValue, 'SetComputeFeretDiameter' = LabelIntensityStatisticsImageFilter_SetComputeFeretDiameter, 'ComputeFeretDiameterOn' = LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOn, 'ComputeFeretDiameterOff' = LabelIntensityStatisticsImageFilter_ComputeFeretDiameterOff, 'GetComputeFeretDiameter' = LabelIntensityStatisticsImageFilter_GetComputeFeretDiameter, 'SetComputePerimeter' = LabelIntensityStatisticsImageFilter_SetComputePerimeter, 'ComputePerimeterOn' = LabelIntensityStatisticsImageFilter_ComputePerimeterOn, 'ComputePerimeterOff' = LabelIntensityStatisticsImageFilter_ComputePerimeterOff, 'GetComputePerimeter' = LabelIntensityStatisticsImageFilter_GetComputePerimeter, 'SetNumberOfBins' = LabelIntensityStatisticsImageFilter_SetNumberOfBins, 'GetNumberOfBins' = LabelIntensityStatisticsImageFilter_GetNumberOfBins, 'GetBoundingBox' = LabelIntensityStatisticsImageFilter_GetBoundingBox, 'GetRegion' = LabelIntensityStatisticsImageFilter_GetRegion, 'GetCentroid' = LabelIntensityStatisticsImageFilter_GetCentroid, 'GetElongation' = LabelIntensityStatisticsImageFilter_GetElongation, 'GetEquivalentEllipsoidDiameter' = LabelIntensityStatisticsImageFilter_GetEquivalentEllipsoidDiameter, 'GetEquivalentSphericalPerimeter' = LabelIntensityStatisticsImageFilter_GetEquivalentSphericalPerimeter, 'GetEquivalentSphericalRadius' = LabelIntensityStatisticsImageFilter_GetEquivalentSphericalRadius, 'GetFeretDiameter' = LabelIntensityStatisticsImageFilter_GetFeretDiameter, 'GetFlatness' = LabelIntensityStatisticsImageFilter_GetFlatness, 'GetLabels' = LabelIntensityStatisticsImageFilter_GetLabels, 'GetNumberOfPixels' = LabelIntensityStatisticsImageFilter_GetNumberOfPixels, 'GetNumberOfPixelsOnBorder' = LabelIntensityStatisticsImageFilter_GetNumberOfPixelsOnBorder, 'GetPerimeter' = LabelIntensityStatisticsImageFilter_GetPerimeter, 'GetPerimeterOnBorder' = LabelIntensityStatisticsImageFilter_GetPerimeterOnBorder, 'GetPerimeterOnBorderRatio' = LabelIntensityStatisticsImageFilter_GetPerimeterOnBorderRatio, 'GetPhysicalSize' = LabelIntensityStatisticsImageFilter_GetPhysicalSize, 'GetPrincipalAxes' = LabelIntensityStatisticsImageFilter_GetPrincipalAxes, 'GetPrincipalMoments' = LabelIntensityStatisticsImageFilter_GetPrincipalMoments, 'GetRoundness' = LabelIntensityStatisticsImageFilter_GetRoundness, 'GetCenterOfGravity' = LabelIntensityStatisticsImageFilter_GetCenterOfGravity, 'GetKurtosis' = LabelIntensityStatisticsImageFilter_GetKurtosis, 'GetMaximum' = LabelIntensityStatisticsImageFilter_GetMaximum, 'GetMaximumIndex' = LabelIntensityStatisticsImageFilter_GetMaximumIndex, 'GetMean' = LabelIntensityStatisticsImageFilter_GetMean, 'GetMedian' = LabelIntensityStatisticsImageFilter_GetMedian, 'GetMinimum' = LabelIntensityStatisticsImageFilter_GetMinimum, 'GetMinimumIndex' = LabelIntensityStatisticsImageFilter_GetMinimumIndex, 'GetSkewness' = LabelIntensityStatisticsImageFilter_GetSkewness, 'GetStandardDeviation' = LabelIntensityStatisticsImageFilter_GetStandardDeviation, 'GetSum' = LabelIntensityStatisticsImageFilter_GetSum, 'GetVariance' = LabelIntensityStatisticsImageFilter_GetVariance, 'GetWeightedElongation' = LabelIntensityStatisticsImageFilter_GetWeightedElongation, 'GetWeightedFlatness' = LabelIntensityStatisticsImageFilter_GetWeightedFlatness, 'GetWeightedPrincipalAxes' = LabelIntensityStatisticsImageFilter_GetWeightedPrincipalAxes, 'GetWeightedPrincipalMoments' = LabelIntensityStatisticsImageFilter_GetWeightedPrincipalMoments, 'GetName' = LabelIntensityStatisticsImageFilter_GetName, 'ToString' = LabelIntensityStatisticsImageFilter_ToString, 'Execute' = LabelIntensityStatisticsImageFilter_Execute, 'HasLabel' = LabelIntensityStatisticsImageFilter_HasLabel, 'GetNumberOfLabels' = LabelIntensityStatisticsImageFilter_GetNumberOfLabels);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelIntensityStatisticsImageFilter
# Start of delete_LabelMapContourOverlayImageFilter

`delete_LabelMapContourOverlayImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelMapContourOverlayImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelMapContourOverlayImageFilter`, 'returnType') = 'void'
attr(`delete_LabelMapContourOverlayImageFilter`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter')
class(`delete_LabelMapContourOverlayImageFilter`) = c("SWIGFunction", class('delete_LabelMapContourOverlayImageFilter'))

# Start of new_LabelMapContourOverlayImageFilter

`LabelMapContourOverlayImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelMapContourOverlayImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelMapContourOverlayImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelMapContourOverlayImageFilter);
  ans
  
}

attr(`LabelMapContourOverlayImageFilter`, 'returnType') = '_p_itk__simple__LabelMapContourOverlayImageFilter'
class(`LabelMapContourOverlayImageFilter`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter'))

# Start of LabelMapContourOverlayImageFilter_SetOpacity

`LabelMapContourOverlayImageFilter_SetOpacity` = function(self, Opacity)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_SetOpacity', self, Opacity, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_SetOpacity`, 'returnType') = 'void'
attr(`LabelMapContourOverlayImageFilter_SetOpacity`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter', 'numeric')
class(`LabelMapContourOverlayImageFilter_SetOpacity`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_SetOpacity'))

# Start of LabelMapContourOverlayImageFilter_GetOpacity

`LabelMapContourOverlayImageFilter_GetOpacity` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapContourOverlayImageFilter_GetOpacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapContourOverlayImageFilter_GetOpacity`, 'returnType') = 'numeric'
attr(`LabelMapContourOverlayImageFilter_GetOpacity`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter')
class(`LabelMapContourOverlayImageFilter_GetOpacity`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_GetOpacity'))

# Start of LabelMapContourOverlayImageFilter_SetDilationRadius

`LabelMapContourOverlayImageFilter_SetDilationRadius__SWIG_0` = function(self, DilationRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  DilationRadius = as.integer(DilationRadius);
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_SetDilationRadius__SWIG_0', self, DilationRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_SetDilationRadius__SWIG_0`, 'returnType') = 'void'
attr(`LabelMapContourOverlayImageFilter_SetDilationRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter', 'integer')
class(`LabelMapContourOverlayImageFilter_SetDilationRadius__SWIG_0`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_SetDilationRadius__SWIG_0'))

# Start of LabelMapContourOverlayImageFilter_SetDilationRadius

`LabelMapContourOverlayImageFilter_SetDilationRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_SetDilationRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_SetDilationRadius__SWIG_1`, 'returnType') = 'void'
attr(`LabelMapContourOverlayImageFilter_SetDilationRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter', 'integer')
class(`LabelMapContourOverlayImageFilter_SetDilationRadius__SWIG_1`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_SetDilationRadius__SWIG_1'))

`LabelMapContourOverlayImageFilter_SetDilationRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__LabelMapContourOverlayImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- LabelMapContourOverlayImageFilter_SetDilationRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__LabelMapContourOverlayImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- LabelMapContourOverlayImageFilter_SetDilationRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelMapContourOverlayImageFilter_SetDilationRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of LabelMapContourOverlayImageFilter_GetDilationRadius

`LabelMapContourOverlayImageFilter_GetDilationRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapContourOverlayImageFilter_GetDilationRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapContourOverlayImageFilter_GetDilationRadius`, 'returnType') = 'integer'
attr(`LabelMapContourOverlayImageFilter_GetDilationRadius`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter')
class(`LabelMapContourOverlayImageFilter_GetDilationRadius`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_GetDilationRadius'))

# Start of LabelMapContourOverlayImageFilter_SetContourThickness

`LabelMapContourOverlayImageFilter_SetContourThickness` = function(self, ContourThickness)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ContourThickness = as.integer(ContourThickness);
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_SetContourThickness', self, ContourThickness, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_SetContourThickness`, 'returnType') = 'void'
attr(`LabelMapContourOverlayImageFilter_SetContourThickness`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter', 'integer')
class(`LabelMapContourOverlayImageFilter_SetContourThickness`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_SetContourThickness'))

# Start of LabelMapContourOverlayImageFilter_GetContourThickness

`LabelMapContourOverlayImageFilter_GetContourThickness` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapContourOverlayImageFilter_GetContourThickness', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapContourOverlayImageFilter_GetContourThickness`, 'returnType') = 'integer'
attr(`LabelMapContourOverlayImageFilter_GetContourThickness`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter')
class(`LabelMapContourOverlayImageFilter_GetContourThickness`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_GetContourThickness'))

# Start of LabelMapContourOverlayImageFilter_SetSliceDimension

`LabelMapContourOverlayImageFilter_SetSliceDimension` = function(self, SliceDimension)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SliceDimension = as.integer(SliceDimension);
  
  if(length(SliceDimension) > 1) {
    warning("using only the first element of SliceDimension");
  };
  
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_SetSliceDimension', self, SliceDimension, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_SetSliceDimension`, 'returnType') = 'void'
attr(`LabelMapContourOverlayImageFilter_SetSliceDimension`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter', 'integer')
class(`LabelMapContourOverlayImageFilter_SetSliceDimension`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_SetSliceDimension'))

# Start of LabelMapContourOverlayImageFilter_GetSliceDimension

`LabelMapContourOverlayImageFilter_GetSliceDimension` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapContourOverlayImageFilter_GetSliceDimension', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapContourOverlayImageFilter_GetSliceDimension`, 'returnType') = 'integer'
attr(`LabelMapContourOverlayImageFilter_GetSliceDimension`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter')
class(`LabelMapContourOverlayImageFilter_GetSliceDimension`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_GetSliceDimension'))

# Start of LabelMapContourOverlayImageFilter_PLAIN_get

`LabelMapContourOverlayImageFilter_PLAIN_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_PLAIN_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__LabelMapContourOverlayImageFilter__ContourTypeType");
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_PLAIN_get`, 'returnType') = 'character'
class(`LabelMapContourOverlayImageFilter_PLAIN_get`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_PLAIN_get'))

# Start of LabelMapContourOverlayImageFilter_CONTOUR_get

`LabelMapContourOverlayImageFilter_CONTOUR_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_CONTOUR_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__LabelMapContourOverlayImageFilter__ContourTypeType");
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_CONTOUR_get`, 'returnType') = 'character'
class(`LabelMapContourOverlayImageFilter_CONTOUR_get`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_CONTOUR_get'))

# Start of LabelMapContourOverlayImageFilter_SLICE_CONTOUR_get

`LabelMapContourOverlayImageFilter_SLICE_CONTOUR_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_SLICE_CONTOUR_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__LabelMapContourOverlayImageFilter__ContourTypeType");
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_SLICE_CONTOUR_get`, 'returnType') = 'character'
class(`LabelMapContourOverlayImageFilter_SLICE_CONTOUR_get`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_SLICE_CONTOUR_get'))

# Start of LabelMapContourOverlayImageFilter_SetContourType

`LabelMapContourOverlayImageFilter_SetContourType` = function(self, ContourType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ContourType = enumToInteger(ContourType, "_itk__simple__LabelMapContourOverlayImageFilter__ContourTypeType"); 
  
  if(length(ContourType) > 1) {
    warning("using only the first element of ContourType");
  };
  
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_SetContourType', self, ContourType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_SetContourType`, 'returnType') = 'void'
attr(`LabelMapContourOverlayImageFilter_SetContourType`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter', 'character')
class(`LabelMapContourOverlayImageFilter_SetContourType`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_SetContourType'))

# Start of LabelMapContourOverlayImageFilter_GetContourType

`LabelMapContourOverlayImageFilter_GetContourType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_GetContourType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__LabelMapContourOverlayImageFilter__ContourTypeType");
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_GetContourType`, 'returnType') = 'character'
attr(`LabelMapContourOverlayImageFilter_GetContourType`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter')
class(`LabelMapContourOverlayImageFilter_GetContourType`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_GetContourType'))

# Start of LabelMapContourOverlayImageFilter_HIGH_LABEL_ON_TOP_get

`LabelMapContourOverlayImageFilter_HIGH_LABEL_ON_TOP_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_HIGH_LABEL_ON_TOP_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__LabelMapContourOverlayImageFilter__PriorityType");
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_HIGH_LABEL_ON_TOP_get`, 'returnType') = 'character'
class(`LabelMapContourOverlayImageFilter_HIGH_LABEL_ON_TOP_get`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_HIGH_LABEL_ON_TOP_get'))

# Start of LabelMapContourOverlayImageFilter_LOW_LABEL_ON_TOP_get

`LabelMapContourOverlayImageFilter_LOW_LABEL_ON_TOP_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_LOW_LABEL_ON_TOP_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__LabelMapContourOverlayImageFilter__PriorityType");
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_LOW_LABEL_ON_TOP_get`, 'returnType') = 'character'
class(`LabelMapContourOverlayImageFilter_LOW_LABEL_ON_TOP_get`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_LOW_LABEL_ON_TOP_get'))

# Start of LabelMapContourOverlayImageFilter_SetPriority

`LabelMapContourOverlayImageFilter_SetPriority` = function(self, Priority)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Priority = enumToInteger(Priority, "_itk__simple__LabelMapContourOverlayImageFilter__PriorityType"); 
  
  if(length(Priority) > 1) {
    warning("using only the first element of Priority");
  };
  
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_SetPriority', self, Priority, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_SetPriority`, 'returnType') = 'void'
attr(`LabelMapContourOverlayImageFilter_SetPriority`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter', 'character')
class(`LabelMapContourOverlayImageFilter_SetPriority`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_SetPriority'))

# Start of LabelMapContourOverlayImageFilter_GetPriority

`LabelMapContourOverlayImageFilter_GetPriority` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_GetPriority', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__LabelMapContourOverlayImageFilter__PriorityType");
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_GetPriority`, 'returnType') = 'character'
attr(`LabelMapContourOverlayImageFilter_GetPriority`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter')
class(`LabelMapContourOverlayImageFilter_GetPriority`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_GetPriority'))

# Start of LabelMapContourOverlayImageFilter_SetColormap

`LabelMapContourOverlayImageFilter_SetColormap` = function(self, Colormap)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Colormap = as.integer(Colormap);
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_SetColormap', self, Colormap, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_SetColormap`, 'returnType') = 'void'
attr(`LabelMapContourOverlayImageFilter_SetColormap`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter', 'integer')
class(`LabelMapContourOverlayImageFilter_SetColormap`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_SetColormap'))

# Start of LabelMapContourOverlayImageFilter_GetColormap

`LabelMapContourOverlayImageFilter_GetColormap` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapContourOverlayImageFilter_GetColormap', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapContourOverlayImageFilter_GetColormap`, 'returnType') = 'integer'
attr(`LabelMapContourOverlayImageFilter_GetColormap`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter')
class(`LabelMapContourOverlayImageFilter_GetColormap`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_GetColormap'))

# Start of LabelMapContourOverlayImageFilter_GetName

`LabelMapContourOverlayImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapContourOverlayImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapContourOverlayImageFilter_GetName`, 'returnType') = 'character'
attr(`LabelMapContourOverlayImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter')
class(`LabelMapContourOverlayImageFilter_GetName`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_GetName'))

# Start of LabelMapContourOverlayImageFilter_ToString

`LabelMapContourOverlayImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapContourOverlayImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapContourOverlayImageFilter_ToString`, 'returnType') = 'character'
attr(`LabelMapContourOverlayImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter')
class(`LabelMapContourOverlayImageFilter_ToString`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_ToString'))

# Start of LabelMapContourOverlayImageFilter_Execute

`LabelMapContourOverlayImageFilter_Execute` = function(self, labelMapImage, featureImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_LabelMapContourOverlayImageFilter_Execute', self, labelMapImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapContourOverlayImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapContourOverlayImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LabelMapContourOverlayImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelMapContourOverlayImageFilter_Execute`) = c("SWIGFunction", class('LabelMapContourOverlayImageFilter_Execute'))

# Start of accessor method for itk::simple::LabelMapContourOverlayImageFilter
setMethod('$', '_p_itk__simple__LabelMapContourOverlayImageFilter', function(x, name)

{
  accessorFuns = list('SetOpacity' = LabelMapContourOverlayImageFilter_SetOpacity, 'GetOpacity' = LabelMapContourOverlayImageFilter_GetOpacity, 'SetDilationRadius' = LabelMapContourOverlayImageFilter_SetDilationRadius, 'GetDilationRadius' = LabelMapContourOverlayImageFilter_GetDilationRadius, 'SetContourThickness' = LabelMapContourOverlayImageFilter_SetContourThickness, 'GetContourThickness' = LabelMapContourOverlayImageFilter_GetContourThickness, 'SetSliceDimension' = LabelMapContourOverlayImageFilter_SetSliceDimension, 'GetSliceDimension' = LabelMapContourOverlayImageFilter_GetSliceDimension, 'SetContourType' = LabelMapContourOverlayImageFilter_SetContourType, 'GetContourType' = LabelMapContourOverlayImageFilter_GetContourType, 'SetPriority' = LabelMapContourOverlayImageFilter_SetPriority, 'GetPriority' = LabelMapContourOverlayImageFilter_GetPriority, 'SetColormap' = LabelMapContourOverlayImageFilter_SetColormap, 'GetColormap' = LabelMapContourOverlayImageFilter_GetColormap, 'GetName' = LabelMapContourOverlayImageFilter_GetName, 'ToString' = LabelMapContourOverlayImageFilter_ToString, 'Execute' = LabelMapContourOverlayImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelMapContourOverlayImageFilter
# Start of LabelMapContourOverlay

`LabelMapContourOverlay__SWIG_0` = function(labelMapImage, featureImage, opacity, dilationRadius, contourThickness, sliceDimension, contourType, priority, colormap, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  dilationRadius = as.integer(dilationRadius);
  contourThickness = as.integer(contourThickness);
  sliceDimension = as.integer(sliceDimension);
  
  if(length(sliceDimension) > 1) {
    warning("using only the first element of sliceDimension");
  };
  
  contourType = enumToInteger(contourType, "_itk__simple__LabelMapContourOverlayImageFilter__ContourTypeType"); 
  
  if(length(contourType) > 1) {
    warning("using only the first element of contourType");
  };
  
  priority = enumToInteger(priority, "_itk__simple__LabelMapContourOverlayImageFilter__PriorityType"); 
  
  if(length(priority) > 1) {
    warning("using only the first element of priority");
  };
  
  colormap = as.integer(colormap);
  ;ans = .Call('R_swig_LabelMapContourOverlay__SWIG_0', labelMapImage, featureImage, opacity, dilationRadius, contourThickness, sliceDimension, contourType, priority, colormap, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapContourOverlay__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapContourOverlay__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'integer', 'integer', 'character', 'character', 'integer')
class(`LabelMapContourOverlay__SWIG_0`) = c("SWIGFunction", class('LabelMapContourOverlay__SWIG_0'))

# Start of LabelMapContourOverlay

`LabelMapContourOverlay__SWIG_1` = function(labelMapImage, featureImage, opacity, dilationRadius, contourThickness, sliceDimension, contourType, priority, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  dilationRadius = as.integer(dilationRadius);
  contourThickness = as.integer(contourThickness);
  sliceDimension = as.integer(sliceDimension);
  
  if(length(sliceDimension) > 1) {
    warning("using only the first element of sliceDimension");
  };
  
  contourType = enumToInteger(contourType, "_itk__simple__LabelMapContourOverlayImageFilter__ContourTypeType"); 
  
  if(length(contourType) > 1) {
    warning("using only the first element of contourType");
  };
  
  priority = enumToInteger(priority, "_itk__simple__LabelMapContourOverlayImageFilter__PriorityType"); 
  
  if(length(priority) > 1) {
    warning("using only the first element of priority");
  };
  
  ;ans = .Call('R_swig_LabelMapContourOverlay__SWIG_1', labelMapImage, featureImage, opacity, dilationRadius, contourThickness, sliceDimension, contourType, priority, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapContourOverlay__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapContourOverlay__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'integer', 'integer', 'character', 'character')
class(`LabelMapContourOverlay__SWIG_1`) = c("SWIGFunction", class('LabelMapContourOverlay__SWIG_1'))

# Start of LabelMapContourOverlay

`LabelMapContourOverlay__SWIG_2` = function(labelMapImage, featureImage, opacity, dilationRadius, contourThickness, sliceDimension, contourType, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  dilationRadius = as.integer(dilationRadius);
  contourThickness = as.integer(contourThickness);
  sliceDimension = as.integer(sliceDimension);
  
  if(length(sliceDimension) > 1) {
    warning("using only the first element of sliceDimension");
  };
  
  contourType = enumToInteger(contourType, "_itk__simple__LabelMapContourOverlayImageFilter__ContourTypeType"); 
  
  if(length(contourType) > 1) {
    warning("using only the first element of contourType");
  };
  
  ;ans = .Call('R_swig_LabelMapContourOverlay__SWIG_2', labelMapImage, featureImage, opacity, dilationRadius, contourThickness, sliceDimension, contourType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapContourOverlay__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapContourOverlay__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'integer', 'integer', 'character')
class(`LabelMapContourOverlay__SWIG_2`) = c("SWIGFunction", class('LabelMapContourOverlay__SWIG_2'))

# Start of LabelMapContourOverlay

`LabelMapContourOverlay__SWIG_3` = function(labelMapImage, featureImage, opacity, dilationRadius, contourThickness, sliceDimension, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  dilationRadius = as.integer(dilationRadius);
  contourThickness = as.integer(contourThickness);
  sliceDimension = as.integer(sliceDimension);
  
  if(length(sliceDimension) > 1) {
    warning("using only the first element of sliceDimension");
  };
  
  ;ans = .Call('R_swig_LabelMapContourOverlay__SWIG_3', labelMapImage, featureImage, opacity, dilationRadius, contourThickness, sliceDimension, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapContourOverlay__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapContourOverlay__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'integer', 'integer')
class(`LabelMapContourOverlay__SWIG_3`) = c("SWIGFunction", class('LabelMapContourOverlay__SWIG_3'))

# Start of LabelMapContourOverlay

`LabelMapContourOverlay__SWIG_4` = function(labelMapImage, featureImage, opacity, dilationRadius, contourThickness, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  dilationRadius = as.integer(dilationRadius);
  contourThickness = as.integer(contourThickness);
  ;ans = .Call('R_swig_LabelMapContourOverlay__SWIG_4', labelMapImage, featureImage, opacity, dilationRadius, contourThickness, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapContourOverlay__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapContourOverlay__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`LabelMapContourOverlay__SWIG_4`) = c("SWIGFunction", class('LabelMapContourOverlay__SWIG_4'))

# Start of LabelMapContourOverlay

`LabelMapContourOverlay__SWIG_5` = function(labelMapImage, featureImage, opacity, dilationRadius, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  dilationRadius = as.integer(dilationRadius);
  ;ans = .Call('R_swig_LabelMapContourOverlay__SWIG_5', labelMapImage, featureImage, opacity, dilationRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapContourOverlay__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapContourOverlay__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer')
class(`LabelMapContourOverlay__SWIG_5`) = c("SWIGFunction", class('LabelMapContourOverlay__SWIG_5'))

# Start of LabelMapContourOverlay

`LabelMapContourOverlay__SWIG_6` = function(labelMapImage, featureImage, opacity, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  ;ans = .Call('R_swig_LabelMapContourOverlay__SWIG_6', labelMapImage, featureImage, opacity, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapContourOverlay__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapContourOverlay__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`LabelMapContourOverlay__SWIG_6`) = c("SWIGFunction", class('LabelMapContourOverlay__SWIG_6'))

# Start of LabelMapContourOverlay

`LabelMapContourOverlay__SWIG_7` = function(labelMapImage, featureImage, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_LabelMapContourOverlay__SWIG_7', labelMapImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapContourOverlay__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapContourOverlay__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelMapContourOverlay__SWIG_7`) = c("SWIGFunction", class('LabelMapContourOverlay__SWIG_7'))

`LabelMapContourOverlay` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 8
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- LabelMapContourOverlay__SWIG_7; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- LabelMapContourOverlay__SWIG_6; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) )) {
      f <- LabelMapContourOverlay__SWIG_5; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.integer(argv[[5]]) || is.numeric(argv[[5]]) )) {
      f <- LabelMapContourOverlay__SWIG_4; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.integer(argv[[5]]) || is.numeric(argv[[5]]) ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- LabelMapContourOverlay__SWIG_3; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.integer(argv[[5]]) || is.numeric(argv[[5]]) ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.character(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- LabelMapContourOverlay__SWIG_2; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.integer(argv[[5]]) || is.numeric(argv[[5]]) ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.character(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.character(argv[[8]]) && length(argv[[8]]) == 1 )) {
      f <- LabelMapContourOverlay__SWIG_1; 
    }
  } else if (argc == 9) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.integer(argv[[5]]) || is.numeric(argv[[5]]) ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.character(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.character(argv[[8]]) && length(argv[[8]]) == 1 ) && ( is.integer(argv[[9]]) || is.numeric(argv[[9]]) )) {
      f <- LabelMapContourOverlay__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelMapContourOverlay with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LabelMapMaskImageFilter

`delete_LabelMapMaskImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelMapMaskImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelMapMaskImageFilter`, 'returnType') = 'void'
attr(`delete_LabelMapMaskImageFilter`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter')
class(`delete_LabelMapMaskImageFilter`) = c("SWIGFunction", class('delete_LabelMapMaskImageFilter'))

# Start of new_LabelMapMaskImageFilter

`LabelMapMaskImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelMapMaskImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelMapMaskImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelMapMaskImageFilter);
  ans
  
}

attr(`LabelMapMaskImageFilter`, 'returnType') = '_p_itk__simple__LabelMapMaskImageFilter'
class(`LabelMapMaskImageFilter`) = c("SWIGFunction", class('LabelMapMaskImageFilter'))

# Start of LabelMapMaskImageFilter_SetLabel

`LabelMapMaskImageFilter_SetLabel` = function(self, Label)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Label = as.integer(Label);
  
  if(length(Label) > 1) {
    warning("using only the first element of Label");
  };
  
  ;ans = .Call('R_swig_LabelMapMaskImageFilter_SetLabel', self, Label, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapMaskImageFilter_SetLabel`, 'returnType') = 'void'
attr(`LabelMapMaskImageFilter_SetLabel`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter', 'integer')
class(`LabelMapMaskImageFilter_SetLabel`) = c("SWIGFunction", class('LabelMapMaskImageFilter_SetLabel'))

# Start of LabelMapMaskImageFilter_GetLabel

`LabelMapMaskImageFilter_GetLabel` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapMaskImageFilter_GetLabel', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapMaskImageFilter_GetLabel`, 'returnType') = 'integer'
attr(`LabelMapMaskImageFilter_GetLabel`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter')
class(`LabelMapMaskImageFilter_GetLabel`) = c("SWIGFunction", class('LabelMapMaskImageFilter_GetLabel'))

# Start of LabelMapMaskImageFilter_SetBackgroundValue

`LabelMapMaskImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LabelMapMaskImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapMaskImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`LabelMapMaskImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter', 'numeric')
class(`LabelMapMaskImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('LabelMapMaskImageFilter_SetBackgroundValue'))

# Start of LabelMapMaskImageFilter_GetBackgroundValue

`LabelMapMaskImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapMaskImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapMaskImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`LabelMapMaskImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter')
class(`LabelMapMaskImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('LabelMapMaskImageFilter_GetBackgroundValue'))

# Start of LabelMapMaskImageFilter_SetNegated

`LabelMapMaskImageFilter_SetNegated` = function(self, Negated)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Negated = as.logical(Negated);
  ;ans = .Call('R_swig_LabelMapMaskImageFilter_SetNegated', self, Negated, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapMaskImageFilter_SetNegated`, 'returnType') = 'void'
attr(`LabelMapMaskImageFilter_SetNegated`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter', 'logical')
class(`LabelMapMaskImageFilter_SetNegated`) = c("SWIGFunction", class('LabelMapMaskImageFilter_SetNegated'))

# Start of LabelMapMaskImageFilter_NegatedOn

`LabelMapMaskImageFilter_NegatedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelMapMaskImageFilter_NegatedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapMaskImageFilter_NegatedOn`, 'returnType') = 'void'
attr(`LabelMapMaskImageFilter_NegatedOn`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter')
class(`LabelMapMaskImageFilter_NegatedOn`) = c("SWIGFunction", class('LabelMapMaskImageFilter_NegatedOn'))

# Start of LabelMapMaskImageFilter_NegatedOff

`LabelMapMaskImageFilter_NegatedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelMapMaskImageFilter_NegatedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapMaskImageFilter_NegatedOff`, 'returnType') = 'void'
attr(`LabelMapMaskImageFilter_NegatedOff`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter')
class(`LabelMapMaskImageFilter_NegatedOff`) = c("SWIGFunction", class('LabelMapMaskImageFilter_NegatedOff'))

# Start of LabelMapMaskImageFilter_GetNegated

`LabelMapMaskImageFilter_GetNegated` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapMaskImageFilter_GetNegated', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapMaskImageFilter_GetNegated`, 'returnType') = 'logical'
attr(`LabelMapMaskImageFilter_GetNegated`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter')
class(`LabelMapMaskImageFilter_GetNegated`) = c("SWIGFunction", class('LabelMapMaskImageFilter_GetNegated'))

# Start of LabelMapMaskImageFilter_SetCrop

`LabelMapMaskImageFilter_SetCrop` = function(self, Crop)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Crop = as.logical(Crop);
  ;ans = .Call('R_swig_LabelMapMaskImageFilter_SetCrop', self, Crop, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapMaskImageFilter_SetCrop`, 'returnType') = 'void'
attr(`LabelMapMaskImageFilter_SetCrop`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter', 'logical')
class(`LabelMapMaskImageFilter_SetCrop`) = c("SWIGFunction", class('LabelMapMaskImageFilter_SetCrop'))

# Start of LabelMapMaskImageFilter_CropOn

`LabelMapMaskImageFilter_CropOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelMapMaskImageFilter_CropOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapMaskImageFilter_CropOn`, 'returnType') = 'void'
attr(`LabelMapMaskImageFilter_CropOn`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter')
class(`LabelMapMaskImageFilter_CropOn`) = c("SWIGFunction", class('LabelMapMaskImageFilter_CropOn'))

# Start of LabelMapMaskImageFilter_CropOff

`LabelMapMaskImageFilter_CropOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelMapMaskImageFilter_CropOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapMaskImageFilter_CropOff`, 'returnType') = 'void'
attr(`LabelMapMaskImageFilter_CropOff`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter')
class(`LabelMapMaskImageFilter_CropOff`) = c("SWIGFunction", class('LabelMapMaskImageFilter_CropOff'))

# Start of LabelMapMaskImageFilter_GetCrop

`LabelMapMaskImageFilter_GetCrop` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapMaskImageFilter_GetCrop', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapMaskImageFilter_GetCrop`, 'returnType') = 'logical'
attr(`LabelMapMaskImageFilter_GetCrop`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter')
class(`LabelMapMaskImageFilter_GetCrop`) = c("SWIGFunction", class('LabelMapMaskImageFilter_GetCrop'))

# Start of LabelMapMaskImageFilter_SetCropBorder

`LabelMapMaskImageFilter_SetCropBorder__SWIG_0` = function(self, CropBorder)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  CropBorder = as.integer(CropBorder);
  ;ans = .Call('R_swig_LabelMapMaskImageFilter_SetCropBorder__SWIG_0', self, CropBorder, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapMaskImageFilter_SetCropBorder__SWIG_0`, 'returnType') = 'void'
attr(`LabelMapMaskImageFilter_SetCropBorder__SWIG_0`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter', 'integer')
class(`LabelMapMaskImageFilter_SetCropBorder__SWIG_0`) = c("SWIGFunction", class('LabelMapMaskImageFilter_SetCropBorder__SWIG_0'))

# Start of LabelMapMaskImageFilter_SetCropBorder

`LabelMapMaskImageFilter_SetCropBorder__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_LabelMapMaskImageFilter_SetCropBorder__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapMaskImageFilter_SetCropBorder__SWIG_1`, 'returnType') = 'void'
attr(`LabelMapMaskImageFilter_SetCropBorder__SWIG_1`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter', 'integer')
class(`LabelMapMaskImageFilter_SetCropBorder__SWIG_1`) = c("SWIGFunction", class('LabelMapMaskImageFilter_SetCropBorder__SWIG_1'))

`LabelMapMaskImageFilter_SetCropBorder` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__LabelMapMaskImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- LabelMapMaskImageFilter_SetCropBorder__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__LabelMapMaskImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- LabelMapMaskImageFilter_SetCropBorder__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelMapMaskImageFilter_SetCropBorder with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of LabelMapMaskImageFilter_GetCropBorder

`LabelMapMaskImageFilter_GetCropBorder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapMaskImageFilter_GetCropBorder', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapMaskImageFilter_GetCropBorder`, 'returnType') = 'integer'
attr(`LabelMapMaskImageFilter_GetCropBorder`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter')
class(`LabelMapMaskImageFilter_GetCropBorder`) = c("SWIGFunction", class('LabelMapMaskImageFilter_GetCropBorder'))

# Start of LabelMapMaskImageFilter_GetName

`LabelMapMaskImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapMaskImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapMaskImageFilter_GetName`, 'returnType') = 'character'
attr(`LabelMapMaskImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter')
class(`LabelMapMaskImageFilter_GetName`) = c("SWIGFunction", class('LabelMapMaskImageFilter_GetName'))

# Start of LabelMapMaskImageFilter_ToString

`LabelMapMaskImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapMaskImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapMaskImageFilter_ToString`, 'returnType') = 'character'
attr(`LabelMapMaskImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter')
class(`LabelMapMaskImageFilter_ToString`) = c("SWIGFunction", class('LabelMapMaskImageFilter_ToString'))

# Start of LabelMapMaskImageFilter_Execute

`LabelMapMaskImageFilter_Execute` = function(self, labelMapImage, featureImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_LabelMapMaskImageFilter_Execute', self, labelMapImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapMaskImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapMaskImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LabelMapMaskImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelMapMaskImageFilter_Execute`) = c("SWIGFunction", class('LabelMapMaskImageFilter_Execute'))

# Start of accessor method for itk::simple::LabelMapMaskImageFilter
setMethod('$', '_p_itk__simple__LabelMapMaskImageFilter', function(x, name)

{
  accessorFuns = list('SetLabel' = LabelMapMaskImageFilter_SetLabel, 'GetLabel' = LabelMapMaskImageFilter_GetLabel, 'SetBackgroundValue' = LabelMapMaskImageFilter_SetBackgroundValue, 'GetBackgroundValue' = LabelMapMaskImageFilter_GetBackgroundValue, 'SetNegated' = LabelMapMaskImageFilter_SetNegated, 'NegatedOn' = LabelMapMaskImageFilter_NegatedOn, 'NegatedOff' = LabelMapMaskImageFilter_NegatedOff, 'GetNegated' = LabelMapMaskImageFilter_GetNegated, 'SetCrop' = LabelMapMaskImageFilter_SetCrop, 'CropOn' = LabelMapMaskImageFilter_CropOn, 'CropOff' = LabelMapMaskImageFilter_CropOff, 'GetCrop' = LabelMapMaskImageFilter_GetCrop, 'SetCropBorder' = LabelMapMaskImageFilter_SetCropBorder, 'GetCropBorder' = LabelMapMaskImageFilter_GetCropBorder, 'GetName' = LabelMapMaskImageFilter_GetName, 'ToString' = LabelMapMaskImageFilter_ToString, 'Execute' = LabelMapMaskImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelMapMaskImageFilter
# Start of LabelMapMask

`LabelMapMask__SWIG_0` = function(labelMapImage, featureImage, label, backgroundValue, negated, crop, cropBorder, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  
  negated = as.logical(negated);
  crop = as.logical(crop);
  cropBorder = as.integer(cropBorder);
  ;ans = .Call('R_swig_LabelMapMask__SWIG_0', labelMapImage, featureImage, label, backgroundValue, negated, crop, cropBorder, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapMask__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapMask__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric', 'logical', 'logical', 'integer')
class(`LabelMapMask__SWIG_0`) = c("SWIGFunction", class('LabelMapMask__SWIG_0'))

# Start of LabelMapMask

`LabelMapMask__SWIG_1` = function(labelMapImage, featureImage, label, backgroundValue, negated, crop, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  
  negated = as.logical(negated);
  crop = as.logical(crop);
  ;ans = .Call('R_swig_LabelMapMask__SWIG_1', labelMapImage, featureImage, label, backgroundValue, negated, crop, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapMask__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapMask__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric', 'logical', 'logical')
class(`LabelMapMask__SWIG_1`) = c("SWIGFunction", class('LabelMapMask__SWIG_1'))

# Start of LabelMapMask

`LabelMapMask__SWIG_2` = function(labelMapImage, featureImage, label, backgroundValue, negated, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  
  negated = as.logical(negated);
  ;ans = .Call('R_swig_LabelMapMask__SWIG_2', labelMapImage, featureImage, label, backgroundValue, negated, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapMask__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapMask__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric', 'logical')
class(`LabelMapMask__SWIG_2`) = c("SWIGFunction", class('LabelMapMask__SWIG_2'))

# Start of LabelMapMask

`LabelMapMask__SWIG_3` = function(labelMapImage, featureImage, label, backgroundValue, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  
  ;ans = .Call('R_swig_LabelMapMask__SWIG_3', labelMapImage, featureImage, label, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapMask__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapMask__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric')
class(`LabelMapMask__SWIG_3`) = c("SWIGFunction", class('LabelMapMask__SWIG_3'))

# Start of LabelMapMask

`LabelMapMask__SWIG_4` = function(labelMapImage, featureImage, label, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;ans = .Call('R_swig_LabelMapMask__SWIG_4', labelMapImage, featureImage, label, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapMask__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapMask__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`LabelMapMask__SWIG_4`) = c("SWIGFunction", class('LabelMapMask__SWIG_4'))

# Start of LabelMapMask

`LabelMapMask__SWIG_5` = function(labelMapImage, featureImage, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_LabelMapMask__SWIG_5', labelMapImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapMask__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapMask__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelMapMask__SWIG_5`) = c("SWIGFunction", class('LabelMapMask__SWIG_5'))

`LabelMapMask` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- LabelMapMask__SWIG_5; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- LabelMapMask__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- LabelMapMask__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- LabelMapMask__SWIG_2; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- LabelMapMask__SWIG_1; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.integer(argv[[7]]) || is.numeric(argv[[7]]) )) {
      f <- LabelMapMask__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelMapMask with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LabelMapOverlayImageFilter

`delete_LabelMapOverlayImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelMapOverlayImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelMapOverlayImageFilter`, 'returnType') = 'void'
attr(`delete_LabelMapOverlayImageFilter`, "inputTypes") = c('_p_itk__simple__LabelMapOverlayImageFilter')
class(`delete_LabelMapOverlayImageFilter`) = c("SWIGFunction", class('delete_LabelMapOverlayImageFilter'))

# Start of new_LabelMapOverlayImageFilter

`LabelMapOverlayImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelMapOverlayImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelMapOverlayImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelMapOverlayImageFilter);
  ans
  
}

attr(`LabelMapOverlayImageFilter`, 'returnType') = '_p_itk__simple__LabelMapOverlayImageFilter'
class(`LabelMapOverlayImageFilter`) = c("SWIGFunction", class('LabelMapOverlayImageFilter'))

# Start of LabelMapOverlayImageFilter_SetOpacity

`LabelMapOverlayImageFilter_SetOpacity` = function(self, Opacity)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LabelMapOverlayImageFilter_SetOpacity', self, Opacity, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapOverlayImageFilter_SetOpacity`, 'returnType') = 'void'
attr(`LabelMapOverlayImageFilter_SetOpacity`, "inputTypes") = c('_p_itk__simple__LabelMapOverlayImageFilter', 'numeric')
class(`LabelMapOverlayImageFilter_SetOpacity`) = c("SWIGFunction", class('LabelMapOverlayImageFilter_SetOpacity'))

# Start of LabelMapOverlayImageFilter_GetOpacity

`LabelMapOverlayImageFilter_GetOpacity` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapOverlayImageFilter_GetOpacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapOverlayImageFilter_GetOpacity`, 'returnType') = 'numeric'
attr(`LabelMapOverlayImageFilter_GetOpacity`, "inputTypes") = c('_p_itk__simple__LabelMapOverlayImageFilter')
class(`LabelMapOverlayImageFilter_GetOpacity`) = c("SWIGFunction", class('LabelMapOverlayImageFilter_GetOpacity'))

# Start of LabelMapOverlayImageFilter_SetColormap

`LabelMapOverlayImageFilter_SetColormap` = function(self, Colormap)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Colormap = as.integer(Colormap);
  ;ans = .Call('R_swig_LabelMapOverlayImageFilter_SetColormap', self, Colormap, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapOverlayImageFilter_SetColormap`, 'returnType') = 'void'
attr(`LabelMapOverlayImageFilter_SetColormap`, "inputTypes") = c('_p_itk__simple__LabelMapOverlayImageFilter', 'integer')
class(`LabelMapOverlayImageFilter_SetColormap`) = c("SWIGFunction", class('LabelMapOverlayImageFilter_SetColormap'))

# Start of LabelMapOverlayImageFilter_GetColormap

`LabelMapOverlayImageFilter_GetColormap` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapOverlayImageFilter_GetColormap', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapOverlayImageFilter_GetColormap`, 'returnType') = 'integer'
attr(`LabelMapOverlayImageFilter_GetColormap`, "inputTypes") = c('_p_itk__simple__LabelMapOverlayImageFilter')
class(`LabelMapOverlayImageFilter_GetColormap`) = c("SWIGFunction", class('LabelMapOverlayImageFilter_GetColormap'))

# Start of LabelMapOverlayImageFilter_GetName

`LabelMapOverlayImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapOverlayImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapOverlayImageFilter_GetName`, 'returnType') = 'character'
attr(`LabelMapOverlayImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelMapOverlayImageFilter')
class(`LabelMapOverlayImageFilter_GetName`) = c("SWIGFunction", class('LabelMapOverlayImageFilter_GetName'))

# Start of LabelMapOverlayImageFilter_ToString

`LabelMapOverlayImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapOverlayImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapOverlayImageFilter_ToString`, 'returnType') = 'character'
attr(`LabelMapOverlayImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelMapOverlayImageFilter')
class(`LabelMapOverlayImageFilter_ToString`) = c("SWIGFunction", class('LabelMapOverlayImageFilter_ToString'))

# Start of LabelMapOverlayImageFilter_Execute

`LabelMapOverlayImageFilter_Execute` = function(self, labelMapImage, featureImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_LabelMapOverlayImageFilter_Execute', self, labelMapImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapOverlayImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapOverlayImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LabelMapOverlayImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelMapOverlayImageFilter_Execute`) = c("SWIGFunction", class('LabelMapOverlayImageFilter_Execute'))

# Start of accessor method for itk::simple::LabelMapOverlayImageFilter
setMethod('$', '_p_itk__simple__LabelMapOverlayImageFilter', function(x, name)

{
  accessorFuns = list('SetOpacity' = LabelMapOverlayImageFilter_SetOpacity, 'GetOpacity' = LabelMapOverlayImageFilter_GetOpacity, 'SetColormap' = LabelMapOverlayImageFilter_SetColormap, 'GetColormap' = LabelMapOverlayImageFilter_GetColormap, 'GetName' = LabelMapOverlayImageFilter_GetName, 'ToString' = LabelMapOverlayImageFilter_ToString, 'Execute' = LabelMapOverlayImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelMapOverlayImageFilter
# Start of LabelMapOverlay

`LabelMapOverlay__SWIG_0` = function(labelMapImage, featureImage, opacity, colormap, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  colormap = as.integer(colormap);
  ;ans = .Call('R_swig_LabelMapOverlay__SWIG_0', labelMapImage, featureImage, opacity, colormap, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapOverlay__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapOverlay__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer')
class(`LabelMapOverlay__SWIG_0`) = c("SWIGFunction", class('LabelMapOverlay__SWIG_0'))

# Start of LabelMapOverlay

`LabelMapOverlay__SWIG_1` = function(labelMapImage, featureImage, opacity, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  ;ans = .Call('R_swig_LabelMapOverlay__SWIG_1', labelMapImage, featureImage, opacity, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapOverlay__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapOverlay__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`LabelMapOverlay__SWIG_1`) = c("SWIGFunction", class('LabelMapOverlay__SWIG_1'))

# Start of LabelMapOverlay

`LabelMapOverlay__SWIG_2` = function(labelMapImage, featureImage, .copy = FALSE)
{
  if (inherits(labelMapImage, "ExternalReference")) labelMapImage = slot(labelMapImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_LabelMapOverlay__SWIG_2', labelMapImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapOverlay__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapOverlay__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelMapOverlay__SWIG_2`) = c("SWIGFunction", class('LabelMapOverlay__SWIG_2'))

`LabelMapOverlay` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- LabelMapOverlay__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- LabelMapOverlay__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) )) {
      f <- LabelMapOverlay__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelMapOverlay with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LabelMapToBinaryImageFilter

`delete_LabelMapToBinaryImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelMapToBinaryImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelMapToBinaryImageFilter`, 'returnType') = 'void'
attr(`delete_LabelMapToBinaryImageFilter`, "inputTypes") = c('_p_itk__simple__LabelMapToBinaryImageFilter')
class(`delete_LabelMapToBinaryImageFilter`) = c("SWIGFunction", class('delete_LabelMapToBinaryImageFilter'))

# Start of new_LabelMapToBinaryImageFilter

`LabelMapToBinaryImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelMapToBinaryImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelMapToBinaryImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelMapToBinaryImageFilter);
  ans
  
}

attr(`LabelMapToBinaryImageFilter`, 'returnType') = '_p_itk__simple__LabelMapToBinaryImageFilter'
class(`LabelMapToBinaryImageFilter`) = c("SWIGFunction", class('LabelMapToBinaryImageFilter'))

# Start of LabelMapToBinaryImageFilter_SetBackgroundValue

`LabelMapToBinaryImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LabelMapToBinaryImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapToBinaryImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`LabelMapToBinaryImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelMapToBinaryImageFilter', 'numeric')
class(`LabelMapToBinaryImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('LabelMapToBinaryImageFilter_SetBackgroundValue'))

# Start of LabelMapToBinaryImageFilter_GetBackgroundValue

`LabelMapToBinaryImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapToBinaryImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapToBinaryImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`LabelMapToBinaryImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelMapToBinaryImageFilter')
class(`LabelMapToBinaryImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('LabelMapToBinaryImageFilter_GetBackgroundValue'))

# Start of LabelMapToBinaryImageFilter_SetForegroundValue

`LabelMapToBinaryImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LabelMapToBinaryImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapToBinaryImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`LabelMapToBinaryImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__LabelMapToBinaryImageFilter', 'numeric')
class(`LabelMapToBinaryImageFilter_SetForegroundValue`) = c("SWIGFunction", class('LabelMapToBinaryImageFilter_SetForegroundValue'))

# Start of LabelMapToBinaryImageFilter_GetForegroundValue

`LabelMapToBinaryImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapToBinaryImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapToBinaryImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`LabelMapToBinaryImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__LabelMapToBinaryImageFilter')
class(`LabelMapToBinaryImageFilter_GetForegroundValue`) = c("SWIGFunction", class('LabelMapToBinaryImageFilter_GetForegroundValue'))

# Start of LabelMapToBinaryImageFilter_GetName

`LabelMapToBinaryImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapToBinaryImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapToBinaryImageFilter_GetName`, 'returnType') = 'character'
attr(`LabelMapToBinaryImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelMapToBinaryImageFilter')
class(`LabelMapToBinaryImageFilter_GetName`) = c("SWIGFunction", class('LabelMapToBinaryImageFilter_GetName'))

# Start of LabelMapToBinaryImageFilter_ToString

`LabelMapToBinaryImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapToBinaryImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapToBinaryImageFilter_ToString`, 'returnType') = 'character'
attr(`LabelMapToBinaryImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelMapToBinaryImageFilter')
class(`LabelMapToBinaryImageFilter_ToString`) = c("SWIGFunction", class('LabelMapToBinaryImageFilter_ToString'))

# Start of LabelMapToBinaryImageFilter_Execute

`LabelMapToBinaryImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelMapToBinaryImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapToBinaryImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapToBinaryImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LabelMapToBinaryImageFilter', '_p_itk__simple__Image')
class(`LabelMapToBinaryImageFilter_Execute`) = c("SWIGFunction", class('LabelMapToBinaryImageFilter_Execute'))

# Start of accessor method for itk::simple::LabelMapToBinaryImageFilter
setMethod('$', '_p_itk__simple__LabelMapToBinaryImageFilter', function(x, name)

{
  accessorFuns = list('SetBackgroundValue' = LabelMapToBinaryImageFilter_SetBackgroundValue, 'GetBackgroundValue' = LabelMapToBinaryImageFilter_GetBackgroundValue, 'SetForegroundValue' = LabelMapToBinaryImageFilter_SetForegroundValue, 'GetForegroundValue' = LabelMapToBinaryImageFilter_GetForegroundValue, 'GetName' = LabelMapToBinaryImageFilter_GetName, 'ToString' = LabelMapToBinaryImageFilter_ToString, 'Execute' = LabelMapToBinaryImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelMapToBinaryImageFilter
# Start of LabelMapToBinary

`LabelMapToBinary__SWIG_0` = function(image1, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_LabelMapToBinary__SWIG_0', image1, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapToBinary__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapToBinary__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`LabelMapToBinary__SWIG_0`) = c("SWIGFunction", class('LabelMapToBinary__SWIG_0'))

# Start of LabelMapToBinary

`LabelMapToBinary__SWIG_1` = function(image1, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_LabelMapToBinary__SWIG_1', image1, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapToBinary__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapToBinary__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`LabelMapToBinary__SWIG_1`) = c("SWIGFunction", class('LabelMapToBinary__SWIG_1'))

# Start of LabelMapToBinary

`LabelMapToBinary__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelMapToBinary__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapToBinary__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapToBinary__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`LabelMapToBinary__SWIG_2`) = c("SWIGFunction", class('LabelMapToBinary__SWIG_2'))

`LabelMapToBinary` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- LabelMapToBinary__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- LabelMapToBinary__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- LabelMapToBinary__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelMapToBinary with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LabelMapToLabelImageFilter

`delete_LabelMapToLabelImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelMapToLabelImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelMapToLabelImageFilter`, 'returnType') = 'void'
attr(`delete_LabelMapToLabelImageFilter`, "inputTypes") = c('_p_itk__simple__LabelMapToLabelImageFilter')
class(`delete_LabelMapToLabelImageFilter`) = c("SWIGFunction", class('delete_LabelMapToLabelImageFilter'))

# Start of new_LabelMapToLabelImageFilter

`LabelMapToLabelImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelMapToLabelImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelMapToLabelImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelMapToLabelImageFilter);
  ans
  
}

attr(`LabelMapToLabelImageFilter`, 'returnType') = '_p_itk__simple__LabelMapToLabelImageFilter'
class(`LabelMapToLabelImageFilter`) = c("SWIGFunction", class('LabelMapToLabelImageFilter'))

# Start of LabelMapToLabelImageFilter_GetName

`LabelMapToLabelImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapToLabelImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapToLabelImageFilter_GetName`, 'returnType') = 'character'
attr(`LabelMapToLabelImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelMapToLabelImageFilter')
class(`LabelMapToLabelImageFilter_GetName`) = c("SWIGFunction", class('LabelMapToLabelImageFilter_GetName'))

# Start of LabelMapToLabelImageFilter_ToString

`LabelMapToLabelImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapToLabelImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapToLabelImageFilter_ToString`, 'returnType') = 'character'
attr(`LabelMapToLabelImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelMapToLabelImageFilter')
class(`LabelMapToLabelImageFilter_ToString`) = c("SWIGFunction", class('LabelMapToLabelImageFilter_ToString'))

# Start of LabelMapToLabelImageFilter_Execute

`LabelMapToLabelImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelMapToLabelImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapToLabelImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapToLabelImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LabelMapToLabelImageFilter', '_p_itk__simple__Image')
class(`LabelMapToLabelImageFilter_Execute`) = c("SWIGFunction", class('LabelMapToLabelImageFilter_Execute'))

# Start of accessor method for itk::simple::LabelMapToLabelImageFilter
setMethod('$', '_p_itk__simple__LabelMapToLabelImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = LabelMapToLabelImageFilter_GetName, 'ToString' = LabelMapToLabelImageFilter_ToString, 'Execute' = LabelMapToLabelImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelMapToLabelImageFilter
# Start of LabelMapToLabel

`LabelMapToLabel` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelMapToLabel', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapToLabel`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapToLabel`, "inputTypes") = c('_p_itk__simple__Image')
class(`LabelMapToLabel`) = c("SWIGFunction", class('LabelMapToLabel'))

# Start of delete_LabelMapToRGBImageFilter

`delete_LabelMapToRGBImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelMapToRGBImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelMapToRGBImageFilter`, 'returnType') = 'void'
attr(`delete_LabelMapToRGBImageFilter`, "inputTypes") = c('_p_itk__simple__LabelMapToRGBImageFilter')
class(`delete_LabelMapToRGBImageFilter`) = c("SWIGFunction", class('delete_LabelMapToRGBImageFilter'))

# Start of new_LabelMapToRGBImageFilter

`LabelMapToRGBImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelMapToRGBImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelMapToRGBImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelMapToRGBImageFilter);
  ans
  
}

attr(`LabelMapToRGBImageFilter`, 'returnType') = '_p_itk__simple__LabelMapToRGBImageFilter'
class(`LabelMapToRGBImageFilter`) = c("SWIGFunction", class('LabelMapToRGBImageFilter'))

# Start of LabelMapToRGBImageFilter_SetColormap

`LabelMapToRGBImageFilter_SetColormap` = function(self, Colormap)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Colormap = as.integer(Colormap);
  ;ans = .Call('R_swig_LabelMapToRGBImageFilter_SetColormap', self, Colormap, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelMapToRGBImageFilter_SetColormap`, 'returnType') = 'void'
attr(`LabelMapToRGBImageFilter_SetColormap`, "inputTypes") = c('_p_itk__simple__LabelMapToRGBImageFilter', 'integer')
class(`LabelMapToRGBImageFilter_SetColormap`) = c("SWIGFunction", class('LabelMapToRGBImageFilter_SetColormap'))

# Start of LabelMapToRGBImageFilter_GetColormap

`LabelMapToRGBImageFilter_GetColormap` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapToRGBImageFilter_GetColormap', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapToRGBImageFilter_GetColormap`, 'returnType') = 'integer'
attr(`LabelMapToRGBImageFilter_GetColormap`, "inputTypes") = c('_p_itk__simple__LabelMapToRGBImageFilter')
class(`LabelMapToRGBImageFilter_GetColormap`) = c("SWIGFunction", class('LabelMapToRGBImageFilter_GetColormap'))

# Start of LabelMapToRGBImageFilter_GetName

`LabelMapToRGBImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapToRGBImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapToRGBImageFilter_GetName`, 'returnType') = 'character'
attr(`LabelMapToRGBImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelMapToRGBImageFilter')
class(`LabelMapToRGBImageFilter_GetName`) = c("SWIGFunction", class('LabelMapToRGBImageFilter_GetName'))

# Start of LabelMapToRGBImageFilter_ToString

`LabelMapToRGBImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelMapToRGBImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelMapToRGBImageFilter_ToString`, 'returnType') = 'character'
attr(`LabelMapToRGBImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelMapToRGBImageFilter')
class(`LabelMapToRGBImageFilter_ToString`) = c("SWIGFunction", class('LabelMapToRGBImageFilter_ToString'))

# Start of LabelMapToRGBImageFilter_Execute

`LabelMapToRGBImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelMapToRGBImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapToRGBImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapToRGBImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LabelMapToRGBImageFilter', '_p_itk__simple__Image')
class(`LabelMapToRGBImageFilter_Execute`) = c("SWIGFunction", class('LabelMapToRGBImageFilter_Execute'))

# Start of accessor method for itk::simple::LabelMapToRGBImageFilter
setMethod('$', '_p_itk__simple__LabelMapToRGBImageFilter', function(x, name)

{
  accessorFuns = list('SetColormap' = LabelMapToRGBImageFilter_SetColormap, 'GetColormap' = LabelMapToRGBImageFilter_GetColormap, 'GetName' = LabelMapToRGBImageFilter_GetName, 'ToString' = LabelMapToRGBImageFilter_ToString, 'Execute' = LabelMapToRGBImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelMapToRGBImageFilter
# Start of LabelMapToRGB

`LabelMapToRGB__SWIG_0` = function(image1, colormap, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  colormap = as.integer(colormap);
  ;ans = .Call('R_swig_LabelMapToRGB__SWIG_0', image1, colormap, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapToRGB__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapToRGB__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`LabelMapToRGB__SWIG_0`) = c("SWIGFunction", class('LabelMapToRGB__SWIG_0'))

# Start of LabelMapToRGB

`LabelMapToRGB__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelMapToRGB__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelMapToRGB__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelMapToRGB__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`LabelMapToRGB__SWIG_1`) = c("SWIGFunction", class('LabelMapToRGB__SWIG_1'))

`LabelMapToRGB` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- LabelMapToRGB__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- LabelMapToRGB__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelMapToRGB with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LabelOverlapMeasuresImageFilter

`delete_LabelOverlapMeasuresImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelOverlapMeasuresImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelOverlapMeasuresImageFilter`, 'returnType') = 'void'
attr(`delete_LabelOverlapMeasuresImageFilter`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter')
class(`delete_LabelOverlapMeasuresImageFilter`) = c("SWIGFunction", class('delete_LabelOverlapMeasuresImageFilter'))

# Start of new_LabelOverlapMeasuresImageFilter

`LabelOverlapMeasuresImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelOverlapMeasuresImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelOverlapMeasuresImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelOverlapMeasuresImageFilter);
  ans
  
}

attr(`LabelOverlapMeasuresImageFilter`, 'returnType') = '_p_itk__simple__LabelOverlapMeasuresImageFilter'
class(`LabelOverlapMeasuresImageFilter`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter'))

# Start of LabelOverlapMeasuresImageFilter_GetFalseNegativeError

`LabelOverlapMeasuresImageFilter_GetFalseNegativeError__SWIG_0` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetFalseNegativeError__SWIG_0', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetFalseNegativeError__SWIG_0`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetFalseNegativeError__SWIG_0`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter')
class(`LabelOverlapMeasuresImageFilter_GetFalseNegativeError__SWIG_0`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetFalseNegativeError__SWIG_0'))

# Start of LabelOverlapMeasuresImageFilter_GetFalseNegativeError

`LabelOverlapMeasuresImageFilter_GetFalseNegativeError__SWIG_1` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetFalseNegativeError__SWIG_1', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetFalseNegativeError__SWIG_1`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetFalseNegativeError__SWIG_1`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter', 'integer')
class(`LabelOverlapMeasuresImageFilter_GetFalseNegativeError__SWIG_1`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetFalseNegativeError__SWIG_1'))

`LabelOverlapMeasuresImageFilter_GetFalseNegativeError` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) {
      f <- LabelOverlapMeasuresImageFilter_GetFalseNegativeError__SWIG_0; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- LabelOverlapMeasuresImageFilter_GetFalseNegativeError__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelOverlapMeasuresImageFilter_GetFalseNegativeError with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of LabelOverlapMeasuresImageFilter_GetFalsePositiveError

`LabelOverlapMeasuresImageFilter_GetFalsePositiveError__SWIG_0` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetFalsePositiveError__SWIG_0', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetFalsePositiveError__SWIG_0`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetFalsePositiveError__SWIG_0`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter')
class(`LabelOverlapMeasuresImageFilter_GetFalsePositiveError__SWIG_0`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetFalsePositiveError__SWIG_0'))

# Start of LabelOverlapMeasuresImageFilter_GetFalsePositiveError

`LabelOverlapMeasuresImageFilter_GetFalsePositiveError__SWIG_1` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetFalsePositiveError__SWIG_1', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetFalsePositiveError__SWIG_1`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetFalsePositiveError__SWIG_1`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter', 'integer')
class(`LabelOverlapMeasuresImageFilter_GetFalsePositiveError__SWIG_1`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetFalsePositiveError__SWIG_1'))

`LabelOverlapMeasuresImageFilter_GetFalsePositiveError` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) {
      f <- LabelOverlapMeasuresImageFilter_GetFalsePositiveError__SWIG_0; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- LabelOverlapMeasuresImageFilter_GetFalsePositiveError__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelOverlapMeasuresImageFilter_GetFalsePositiveError with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of LabelOverlapMeasuresImageFilter_GetMeanOverlap

`LabelOverlapMeasuresImageFilter_GetMeanOverlap__SWIG_0` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetMeanOverlap__SWIG_0', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetMeanOverlap__SWIG_0`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetMeanOverlap__SWIG_0`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter')
class(`LabelOverlapMeasuresImageFilter_GetMeanOverlap__SWIG_0`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetMeanOverlap__SWIG_0'))

# Start of LabelOverlapMeasuresImageFilter_GetMeanOverlap

`LabelOverlapMeasuresImageFilter_GetMeanOverlap__SWIG_1` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetMeanOverlap__SWIG_1', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetMeanOverlap__SWIG_1`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetMeanOverlap__SWIG_1`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter', 'integer')
class(`LabelOverlapMeasuresImageFilter_GetMeanOverlap__SWIG_1`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetMeanOverlap__SWIG_1'))

`LabelOverlapMeasuresImageFilter_GetMeanOverlap` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) {
      f <- LabelOverlapMeasuresImageFilter_GetMeanOverlap__SWIG_0; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- LabelOverlapMeasuresImageFilter_GetMeanOverlap__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelOverlapMeasuresImageFilter_GetMeanOverlap with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of LabelOverlapMeasuresImageFilter_GetUnionOverlap

`LabelOverlapMeasuresImageFilter_GetUnionOverlap__SWIG_0` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetUnionOverlap__SWIG_0', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetUnionOverlap__SWIG_0`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetUnionOverlap__SWIG_0`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter')
class(`LabelOverlapMeasuresImageFilter_GetUnionOverlap__SWIG_0`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetUnionOverlap__SWIG_0'))

# Start of LabelOverlapMeasuresImageFilter_GetUnionOverlap

`LabelOverlapMeasuresImageFilter_GetUnionOverlap__SWIG_1` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetUnionOverlap__SWIG_1', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetUnionOverlap__SWIG_1`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetUnionOverlap__SWIG_1`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter', 'integer')
class(`LabelOverlapMeasuresImageFilter_GetUnionOverlap__SWIG_1`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetUnionOverlap__SWIG_1'))

`LabelOverlapMeasuresImageFilter_GetUnionOverlap` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) {
      f <- LabelOverlapMeasuresImageFilter_GetUnionOverlap__SWIG_0; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- LabelOverlapMeasuresImageFilter_GetUnionOverlap__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelOverlapMeasuresImageFilter_GetUnionOverlap with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of LabelOverlapMeasuresImageFilter_GetVolumeSimilarity

`LabelOverlapMeasuresImageFilter_GetVolumeSimilarity__SWIG_0` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetVolumeSimilarity__SWIG_0', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetVolumeSimilarity__SWIG_0`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetVolumeSimilarity__SWIG_0`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter')
class(`LabelOverlapMeasuresImageFilter_GetVolumeSimilarity__SWIG_0`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetVolumeSimilarity__SWIG_0'))

# Start of LabelOverlapMeasuresImageFilter_GetVolumeSimilarity

`LabelOverlapMeasuresImageFilter_GetVolumeSimilarity__SWIG_1` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetVolumeSimilarity__SWIG_1', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetVolumeSimilarity__SWIG_1`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetVolumeSimilarity__SWIG_1`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter', 'integer')
class(`LabelOverlapMeasuresImageFilter_GetVolumeSimilarity__SWIG_1`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetVolumeSimilarity__SWIG_1'))

`LabelOverlapMeasuresImageFilter_GetVolumeSimilarity` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) {
      f <- LabelOverlapMeasuresImageFilter_GetVolumeSimilarity__SWIG_0; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- LabelOverlapMeasuresImageFilter_GetVolumeSimilarity__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelOverlapMeasuresImageFilter_GetVolumeSimilarity with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of LabelOverlapMeasuresImageFilter_GetJaccardCoefficient

`LabelOverlapMeasuresImageFilter_GetJaccardCoefficient__SWIG_0` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetJaccardCoefficient__SWIG_0', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetJaccardCoefficient__SWIG_0`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetJaccardCoefficient__SWIG_0`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter')
class(`LabelOverlapMeasuresImageFilter_GetJaccardCoefficient__SWIG_0`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetJaccardCoefficient__SWIG_0'))

# Start of LabelOverlapMeasuresImageFilter_GetJaccardCoefficient

`LabelOverlapMeasuresImageFilter_GetJaccardCoefficient__SWIG_1` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetJaccardCoefficient__SWIG_1', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetJaccardCoefficient__SWIG_1`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetJaccardCoefficient__SWIG_1`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter', 'integer')
class(`LabelOverlapMeasuresImageFilter_GetJaccardCoefficient__SWIG_1`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetJaccardCoefficient__SWIG_1'))

`LabelOverlapMeasuresImageFilter_GetJaccardCoefficient` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) {
      f <- LabelOverlapMeasuresImageFilter_GetJaccardCoefficient__SWIG_0; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- LabelOverlapMeasuresImageFilter_GetJaccardCoefficient__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelOverlapMeasuresImageFilter_GetJaccardCoefficient with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of LabelOverlapMeasuresImageFilter_GetDiceCoefficient

`LabelOverlapMeasuresImageFilter_GetDiceCoefficient__SWIG_0` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetDiceCoefficient__SWIG_0', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetDiceCoefficient__SWIG_0`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetDiceCoefficient__SWIG_0`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter')
class(`LabelOverlapMeasuresImageFilter_GetDiceCoefficient__SWIG_0`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetDiceCoefficient__SWIG_0'))

# Start of LabelOverlapMeasuresImageFilter_GetDiceCoefficient

`LabelOverlapMeasuresImageFilter_GetDiceCoefficient__SWIG_1` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetDiceCoefficient__SWIG_1', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetDiceCoefficient__SWIG_1`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetDiceCoefficient__SWIG_1`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter', 'integer')
class(`LabelOverlapMeasuresImageFilter_GetDiceCoefficient__SWIG_1`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetDiceCoefficient__SWIG_1'))

`LabelOverlapMeasuresImageFilter_GetDiceCoefficient` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) {
      f <- LabelOverlapMeasuresImageFilter_GetDiceCoefficient__SWIG_0; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- LabelOverlapMeasuresImageFilter_GetDiceCoefficient__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelOverlapMeasuresImageFilter_GetDiceCoefficient with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate

`LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate__SWIG_0` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate__SWIG_0', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate__SWIG_0`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate__SWIG_0`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter')
class(`LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate__SWIG_0`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate__SWIG_0'))

# Start of LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate

`LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate__SWIG_1` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate__SWIG_1', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate__SWIG_1`, 'returnType') = 'numeric'
attr(`LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate__SWIG_1`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter', 'integer')
class(`LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate__SWIG_1`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate__SWIG_1'))

`LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) {
      f <- LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate__SWIG_0; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__LabelOverlapMeasuresImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of LabelOverlapMeasuresImageFilter_GetName

`LabelOverlapMeasuresImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_GetName`, 'returnType') = 'character'
attr(`LabelOverlapMeasuresImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter')
class(`LabelOverlapMeasuresImageFilter_GetName`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_GetName'))

# Start of LabelOverlapMeasuresImageFilter_ToString

`LabelOverlapMeasuresImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelOverlapMeasuresImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlapMeasuresImageFilter_ToString`, 'returnType') = 'character'
attr(`LabelOverlapMeasuresImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter')
class(`LabelOverlapMeasuresImageFilter_ToString`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_ToString'))

# Start of LabelOverlapMeasuresImageFilter_Execute

`LabelOverlapMeasuresImageFilter_Execute` = function(self, sourceImage, targetImage)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(sourceImage, "ExternalReference")) sourceImage = slot(sourceImage,"ref"); 
  if (inherits(targetImage, "ExternalReference")) targetImage = slot(targetImage,"ref"); 
  ;ans = .Call('R_swig_LabelOverlapMeasuresImageFilter_Execute', self, sourceImage, targetImage, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelOverlapMeasuresImageFilter_Execute`, 'returnType') = 'void'
attr(`LabelOverlapMeasuresImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LabelOverlapMeasuresImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelOverlapMeasuresImageFilter_Execute`) = c("SWIGFunction", class('LabelOverlapMeasuresImageFilter_Execute'))

# Start of accessor method for itk::simple::LabelOverlapMeasuresImageFilter
setMethod('$', '_p_itk__simple__LabelOverlapMeasuresImageFilter', function(x, name)

{
  accessorFuns = list('GetFalseNegativeError' = LabelOverlapMeasuresImageFilter_GetFalseNegativeError, 'GetFalsePositiveError' = LabelOverlapMeasuresImageFilter_GetFalsePositiveError, 'GetMeanOverlap' = LabelOverlapMeasuresImageFilter_GetMeanOverlap, 'GetUnionOverlap' = LabelOverlapMeasuresImageFilter_GetUnionOverlap, 'GetVolumeSimilarity' = LabelOverlapMeasuresImageFilter_GetVolumeSimilarity, 'GetJaccardCoefficient' = LabelOverlapMeasuresImageFilter_GetJaccardCoefficient, 'GetDiceCoefficient' = LabelOverlapMeasuresImageFilter_GetDiceCoefficient, 'GetFalseDiscoveryRate' = LabelOverlapMeasuresImageFilter_GetFalseDiscoveryRate, 'GetName' = LabelOverlapMeasuresImageFilter_GetName, 'ToString' = LabelOverlapMeasuresImageFilter_ToString, 'Execute' = LabelOverlapMeasuresImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelOverlapMeasuresImageFilter
# Start of delete_LabelOverlayImageFilter

`delete_LabelOverlayImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelOverlayImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelOverlayImageFilter`, 'returnType') = 'void'
attr(`delete_LabelOverlayImageFilter`, "inputTypes") = c('_p_itk__simple__LabelOverlayImageFilter')
class(`delete_LabelOverlayImageFilter`) = c("SWIGFunction", class('delete_LabelOverlayImageFilter'))

# Start of new_LabelOverlayImageFilter

`LabelOverlayImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelOverlayImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelOverlayImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelOverlayImageFilter);
  ans
  
}

attr(`LabelOverlayImageFilter`, 'returnType') = '_p_itk__simple__LabelOverlayImageFilter'
class(`LabelOverlayImageFilter`) = c("SWIGFunction", class('LabelOverlayImageFilter'))

# Start of LabelOverlayImageFilter_SetOpacity

`LabelOverlayImageFilter_SetOpacity` = function(self, Opacity)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LabelOverlayImageFilter_SetOpacity', self, Opacity, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelOverlayImageFilter_SetOpacity`, 'returnType') = 'void'
attr(`LabelOverlayImageFilter_SetOpacity`, "inputTypes") = c('_p_itk__simple__LabelOverlayImageFilter', 'numeric')
class(`LabelOverlayImageFilter_SetOpacity`) = c("SWIGFunction", class('LabelOverlayImageFilter_SetOpacity'))

# Start of LabelOverlayImageFilter_GetOpacity

`LabelOverlayImageFilter_GetOpacity` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelOverlayImageFilter_GetOpacity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlayImageFilter_GetOpacity`, 'returnType') = 'numeric'
attr(`LabelOverlayImageFilter_GetOpacity`, "inputTypes") = c('_p_itk__simple__LabelOverlayImageFilter')
class(`LabelOverlayImageFilter_GetOpacity`) = c("SWIGFunction", class('LabelOverlayImageFilter_GetOpacity'))

# Start of LabelOverlayImageFilter_SetBackgroundValue

`LabelOverlayImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LabelOverlayImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelOverlayImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`LabelOverlayImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelOverlayImageFilter', 'numeric')
class(`LabelOverlayImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('LabelOverlayImageFilter_SetBackgroundValue'))

# Start of LabelOverlayImageFilter_GetBackgroundValue

`LabelOverlayImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelOverlayImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlayImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`LabelOverlayImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelOverlayImageFilter')
class(`LabelOverlayImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('LabelOverlayImageFilter_GetBackgroundValue'))

# Start of LabelOverlayImageFilter_SetColormap

`LabelOverlayImageFilter_SetColormap` = function(self, Colormap)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Colormap = as.integer(Colormap);
  ;ans = .Call('R_swig_LabelOverlayImageFilter_SetColormap', self, Colormap, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelOverlayImageFilter_SetColormap`, 'returnType') = 'void'
attr(`LabelOverlayImageFilter_SetColormap`, "inputTypes") = c('_p_itk__simple__LabelOverlayImageFilter', 'integer')
class(`LabelOverlayImageFilter_SetColormap`) = c("SWIGFunction", class('LabelOverlayImageFilter_SetColormap'))

# Start of LabelOverlayImageFilter_GetColormap

`LabelOverlayImageFilter_GetColormap` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelOverlayImageFilter_GetColormap', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlayImageFilter_GetColormap`, 'returnType') = 'integer'
attr(`LabelOverlayImageFilter_GetColormap`, "inputTypes") = c('_p_itk__simple__LabelOverlayImageFilter')
class(`LabelOverlayImageFilter_GetColormap`) = c("SWIGFunction", class('LabelOverlayImageFilter_GetColormap'))

# Start of LabelOverlayImageFilter_GetName

`LabelOverlayImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelOverlayImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlayImageFilter_GetName`, 'returnType') = 'character'
attr(`LabelOverlayImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelOverlayImageFilter')
class(`LabelOverlayImageFilter_GetName`) = c("SWIGFunction", class('LabelOverlayImageFilter_GetName'))

# Start of LabelOverlayImageFilter_ToString

`LabelOverlayImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelOverlayImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelOverlayImageFilter_ToString`, 'returnType') = 'character'
attr(`LabelOverlayImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelOverlayImageFilter')
class(`LabelOverlayImageFilter_ToString`) = c("SWIGFunction", class('LabelOverlayImageFilter_ToString'))

# Start of LabelOverlayImageFilter_Execute

`LabelOverlayImageFilter_Execute` = function(self, image, labelImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(labelImage, "ExternalReference")) labelImage = slot(labelImage,"ref"); 
  ;ans = .Call('R_swig_LabelOverlayImageFilter_Execute', self, image, labelImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelOverlayImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelOverlayImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LabelOverlayImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelOverlayImageFilter_Execute`) = c("SWIGFunction", class('LabelOverlayImageFilter_Execute'))

# Start of accessor method for itk::simple::LabelOverlayImageFilter
setMethod('$', '_p_itk__simple__LabelOverlayImageFilter', function(x, name)

{
  accessorFuns = list('SetOpacity' = LabelOverlayImageFilter_SetOpacity, 'GetOpacity' = LabelOverlayImageFilter_GetOpacity, 'SetBackgroundValue' = LabelOverlayImageFilter_SetBackgroundValue, 'GetBackgroundValue' = LabelOverlayImageFilter_GetBackgroundValue, 'SetColormap' = LabelOverlayImageFilter_SetColormap, 'GetColormap' = LabelOverlayImageFilter_GetColormap, 'GetName' = LabelOverlayImageFilter_GetName, 'ToString' = LabelOverlayImageFilter_ToString, 'Execute' = LabelOverlayImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelOverlayImageFilter
# Start of LabelOverlay

`LabelOverlay__SWIG_0` = function(image, labelImage, opacity, backgroundValue, colormap, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(labelImage, "ExternalReference")) labelImage = slot(labelImage,"ref"); 
  
  
  colormap = as.integer(colormap);
  ;ans = .Call('R_swig_LabelOverlay__SWIG_0', image, labelImage, opacity, backgroundValue, colormap, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelOverlay__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelOverlay__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'integer')
class(`LabelOverlay__SWIG_0`) = c("SWIGFunction", class('LabelOverlay__SWIG_0'))

# Start of LabelOverlay

`LabelOverlay__SWIG_1` = function(image, labelImage, opacity, backgroundValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(labelImage, "ExternalReference")) labelImage = slot(labelImage,"ref"); 
  
  
  ;ans = .Call('R_swig_LabelOverlay__SWIG_1', image, labelImage, opacity, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelOverlay__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelOverlay__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`LabelOverlay__SWIG_1`) = c("SWIGFunction", class('LabelOverlay__SWIG_1'))

# Start of LabelOverlay

`LabelOverlay__SWIG_2` = function(image, labelImage, opacity, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(labelImage, "ExternalReference")) labelImage = slot(labelImage,"ref"); 
  
  ;ans = .Call('R_swig_LabelOverlay__SWIG_2', image, labelImage, opacity, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelOverlay__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelOverlay__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`LabelOverlay__SWIG_2`) = c("SWIGFunction", class('LabelOverlay__SWIG_2'))

# Start of LabelOverlay

`LabelOverlay__SWIG_3` = function(image, labelImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(labelImage, "ExternalReference")) labelImage = slot(labelImage,"ref"); 
  ;ans = .Call('R_swig_LabelOverlay__SWIG_3', image, labelImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelOverlay__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelOverlay__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelOverlay__SWIG_3`) = c("SWIGFunction", class('LabelOverlay__SWIG_3'))

`LabelOverlay` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- LabelOverlay__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- LabelOverlay__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- LabelOverlay__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.integer(argv[[5]]) || is.numeric(argv[[5]]) )) {
      f <- LabelOverlay__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelOverlay with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LabelShapeStatisticsImageFilter

`delete_LabelShapeStatisticsImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelShapeStatisticsImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelShapeStatisticsImageFilter`, 'returnType') = 'void'
attr(`delete_LabelShapeStatisticsImageFilter`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter')
class(`delete_LabelShapeStatisticsImageFilter`) = c("SWIGFunction", class('delete_LabelShapeStatisticsImageFilter'))

# Start of new_LabelShapeStatisticsImageFilter

`LabelShapeStatisticsImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelShapeStatisticsImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelShapeStatisticsImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelShapeStatisticsImageFilter);
  ans
  
}

attr(`LabelShapeStatisticsImageFilter`, 'returnType') = '_p_itk__simple__LabelShapeStatisticsImageFilter'
class(`LabelShapeStatisticsImageFilter`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter'))

# Start of LabelShapeStatisticsImageFilter_SetBackgroundValue

`LabelShapeStatisticsImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LabelShapeStatisticsImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelShapeStatisticsImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`LabelShapeStatisticsImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'numeric')
class(`LabelShapeStatisticsImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_SetBackgroundValue'))

# Start of LabelShapeStatisticsImageFilter_GetBackgroundValue

`LabelShapeStatisticsImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter')
class(`LabelShapeStatisticsImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetBackgroundValue'))

# Start of LabelShapeStatisticsImageFilter_SetComputeFeretDiameter

`LabelShapeStatisticsImageFilter_SetComputeFeretDiameter` = function(self, ComputeFeretDiameter)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ComputeFeretDiameter = as.logical(ComputeFeretDiameter);
  ;ans = .Call('R_swig_LabelShapeStatisticsImageFilter_SetComputeFeretDiameter', self, ComputeFeretDiameter, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelShapeStatisticsImageFilter_SetComputeFeretDiameter`, 'returnType') = 'void'
attr(`LabelShapeStatisticsImageFilter_SetComputeFeretDiameter`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'logical')
class(`LabelShapeStatisticsImageFilter_SetComputeFeretDiameter`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_SetComputeFeretDiameter'))

# Start of LabelShapeStatisticsImageFilter_ComputeFeretDiameterOn

`LabelShapeStatisticsImageFilter_ComputeFeretDiameterOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelShapeStatisticsImageFilter_ComputeFeretDiameterOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelShapeStatisticsImageFilter_ComputeFeretDiameterOn`, 'returnType') = 'void'
attr(`LabelShapeStatisticsImageFilter_ComputeFeretDiameterOn`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter')
class(`LabelShapeStatisticsImageFilter_ComputeFeretDiameterOn`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_ComputeFeretDiameterOn'))

# Start of LabelShapeStatisticsImageFilter_ComputeFeretDiameterOff

`LabelShapeStatisticsImageFilter_ComputeFeretDiameterOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelShapeStatisticsImageFilter_ComputeFeretDiameterOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelShapeStatisticsImageFilter_ComputeFeretDiameterOff`, 'returnType') = 'void'
attr(`LabelShapeStatisticsImageFilter_ComputeFeretDiameterOff`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter')
class(`LabelShapeStatisticsImageFilter_ComputeFeretDiameterOff`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_ComputeFeretDiameterOff'))

# Start of LabelShapeStatisticsImageFilter_GetComputeFeretDiameter

`LabelShapeStatisticsImageFilter_GetComputeFeretDiameter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetComputeFeretDiameter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetComputeFeretDiameter`, 'returnType') = 'logical'
attr(`LabelShapeStatisticsImageFilter_GetComputeFeretDiameter`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter')
class(`LabelShapeStatisticsImageFilter_GetComputeFeretDiameter`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetComputeFeretDiameter'))

# Start of LabelShapeStatisticsImageFilter_SetComputePerimeter

`LabelShapeStatisticsImageFilter_SetComputePerimeter` = function(self, ComputePerimeter)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ComputePerimeter = as.logical(ComputePerimeter);
  ;ans = .Call('R_swig_LabelShapeStatisticsImageFilter_SetComputePerimeter', self, ComputePerimeter, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelShapeStatisticsImageFilter_SetComputePerimeter`, 'returnType') = 'void'
attr(`LabelShapeStatisticsImageFilter_SetComputePerimeter`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'logical')
class(`LabelShapeStatisticsImageFilter_SetComputePerimeter`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_SetComputePerimeter'))

# Start of LabelShapeStatisticsImageFilter_ComputePerimeterOn

`LabelShapeStatisticsImageFilter_ComputePerimeterOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelShapeStatisticsImageFilter_ComputePerimeterOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelShapeStatisticsImageFilter_ComputePerimeterOn`, 'returnType') = 'void'
attr(`LabelShapeStatisticsImageFilter_ComputePerimeterOn`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter')
class(`LabelShapeStatisticsImageFilter_ComputePerimeterOn`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_ComputePerimeterOn'))

# Start of LabelShapeStatisticsImageFilter_ComputePerimeterOff

`LabelShapeStatisticsImageFilter_ComputePerimeterOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelShapeStatisticsImageFilter_ComputePerimeterOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelShapeStatisticsImageFilter_ComputePerimeterOff`, 'returnType') = 'void'
attr(`LabelShapeStatisticsImageFilter_ComputePerimeterOff`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter')
class(`LabelShapeStatisticsImageFilter_ComputePerimeterOff`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_ComputePerimeterOff'))

# Start of LabelShapeStatisticsImageFilter_GetComputePerimeter

`LabelShapeStatisticsImageFilter_GetComputePerimeter` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetComputePerimeter', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetComputePerimeter`, 'returnType') = 'logical'
attr(`LabelShapeStatisticsImageFilter_GetComputePerimeter`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter')
class(`LabelShapeStatisticsImageFilter_GetComputePerimeter`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetComputePerimeter'))

# Start of LabelShapeStatisticsImageFilter_SetComputeOrientedBoundingBox

`LabelShapeStatisticsImageFilter_SetComputeOrientedBoundingBox` = function(self, ComputeOrientedBoundingBox)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ComputeOrientedBoundingBox = as.logical(ComputeOrientedBoundingBox);
  ;ans = .Call('R_swig_LabelShapeStatisticsImageFilter_SetComputeOrientedBoundingBox', self, ComputeOrientedBoundingBox, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelShapeStatisticsImageFilter_SetComputeOrientedBoundingBox`, 'returnType') = 'void'
attr(`LabelShapeStatisticsImageFilter_SetComputeOrientedBoundingBox`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'logical')
class(`LabelShapeStatisticsImageFilter_SetComputeOrientedBoundingBox`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_SetComputeOrientedBoundingBox'))

# Start of LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOn

`LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOn`, 'returnType') = 'void'
attr(`LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOn`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter')
class(`LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOn`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOn'))

# Start of LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOff

`LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOff`, 'returnType') = 'void'
attr(`LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOff`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter')
class(`LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOff`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOff'))

# Start of LabelShapeStatisticsImageFilter_GetComputeOrientedBoundingBox

`LabelShapeStatisticsImageFilter_GetComputeOrientedBoundingBox` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetComputeOrientedBoundingBox', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetComputeOrientedBoundingBox`, 'returnType') = 'logical'
attr(`LabelShapeStatisticsImageFilter_GetComputeOrientedBoundingBox`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter')
class(`LabelShapeStatisticsImageFilter_GetComputeOrientedBoundingBox`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetComputeOrientedBoundingBox'))

# Start of LabelShapeStatisticsImageFilter_GetBoundingBox

`LabelShapeStatisticsImageFilter_GetBoundingBox` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetBoundingBox', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetBoundingBox`, 'returnType') = 'integer'
attr(`LabelShapeStatisticsImageFilter_GetBoundingBox`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetBoundingBox`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetBoundingBox'))

# Start of LabelShapeStatisticsImageFilter_GetRegion

`LabelShapeStatisticsImageFilter_GetRegion` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetRegion', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetRegion`, 'returnType') = 'integer'
attr(`LabelShapeStatisticsImageFilter_GetRegion`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetRegion`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetRegion'))

# Start of LabelShapeStatisticsImageFilter_GetCentroid

`LabelShapeStatisticsImageFilter_GetCentroid` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetCentroid', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetCentroid`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetCentroid`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetCentroid`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetCentroid'))

# Start of LabelShapeStatisticsImageFilter_GetElongation

`LabelShapeStatisticsImageFilter_GetElongation` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetElongation', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetElongation`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetElongation`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetElongation`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetElongation'))

# Start of LabelShapeStatisticsImageFilter_GetEquivalentEllipsoidDiameter

`LabelShapeStatisticsImageFilter_GetEquivalentEllipsoidDiameter` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetEquivalentEllipsoidDiameter', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetEquivalentEllipsoidDiameter`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetEquivalentEllipsoidDiameter`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetEquivalentEllipsoidDiameter`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetEquivalentEllipsoidDiameter'))

# Start of LabelShapeStatisticsImageFilter_GetEquivalentSphericalPerimeter

`LabelShapeStatisticsImageFilter_GetEquivalentSphericalPerimeter` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetEquivalentSphericalPerimeter', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetEquivalentSphericalPerimeter`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetEquivalentSphericalPerimeter`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetEquivalentSphericalPerimeter`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetEquivalentSphericalPerimeter'))

# Start of LabelShapeStatisticsImageFilter_GetEquivalentSphericalRadius

`LabelShapeStatisticsImageFilter_GetEquivalentSphericalRadius` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetEquivalentSphericalRadius', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetEquivalentSphericalRadius`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetEquivalentSphericalRadius`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetEquivalentSphericalRadius`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetEquivalentSphericalRadius'))

# Start of LabelShapeStatisticsImageFilter_GetFeretDiameter

`LabelShapeStatisticsImageFilter_GetFeretDiameter` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetFeretDiameter', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetFeretDiameter`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetFeretDiameter`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetFeretDiameter`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetFeretDiameter'))

# Start of LabelShapeStatisticsImageFilter_GetFlatness

`LabelShapeStatisticsImageFilter_GetFlatness` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetFlatness', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetFlatness`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetFlatness`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetFlatness`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetFlatness'))

# Start of LabelShapeStatisticsImageFilter_GetLabels

`LabelShapeStatisticsImageFilter_GetLabels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetLabels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetLabels`, 'returnType') = 'integer'
attr(`LabelShapeStatisticsImageFilter_GetLabels`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter')
class(`LabelShapeStatisticsImageFilter_GetLabels`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetLabels'))

# Start of LabelShapeStatisticsImageFilter_GetNumberOfPixels

`LabelShapeStatisticsImageFilter_GetNumberOfPixels` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetNumberOfPixels', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetNumberOfPixels`, 'returnType') = 'integer'
attr(`LabelShapeStatisticsImageFilter_GetNumberOfPixels`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetNumberOfPixels`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetNumberOfPixels'))

# Start of LabelShapeStatisticsImageFilter_GetNumberOfPixelsOnBorder

`LabelShapeStatisticsImageFilter_GetNumberOfPixelsOnBorder` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetNumberOfPixelsOnBorder', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetNumberOfPixelsOnBorder`, 'returnType') = 'integer'
attr(`LabelShapeStatisticsImageFilter_GetNumberOfPixelsOnBorder`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetNumberOfPixelsOnBorder`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetNumberOfPixelsOnBorder'))

# Start of LabelShapeStatisticsImageFilter_GetPerimeter

`LabelShapeStatisticsImageFilter_GetPerimeter` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetPerimeter', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetPerimeter`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetPerimeter`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetPerimeter`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetPerimeter'))

# Start of LabelShapeStatisticsImageFilter_GetPerimeterOnBorder

`LabelShapeStatisticsImageFilter_GetPerimeterOnBorder` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetPerimeterOnBorder', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetPerimeterOnBorder`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetPerimeterOnBorder`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetPerimeterOnBorder`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetPerimeterOnBorder'))

# Start of LabelShapeStatisticsImageFilter_GetPerimeterOnBorderRatio

`LabelShapeStatisticsImageFilter_GetPerimeterOnBorderRatio` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetPerimeterOnBorderRatio', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetPerimeterOnBorderRatio`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetPerimeterOnBorderRatio`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetPerimeterOnBorderRatio`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetPerimeterOnBorderRatio'))

# Start of LabelShapeStatisticsImageFilter_GetPhysicalSize

`LabelShapeStatisticsImageFilter_GetPhysicalSize` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetPhysicalSize', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetPhysicalSize`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetPhysicalSize`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetPhysicalSize`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetPhysicalSize'))

# Start of LabelShapeStatisticsImageFilter_GetPrincipalAxes

`LabelShapeStatisticsImageFilter_GetPrincipalAxes` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetPrincipalAxes', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetPrincipalAxes`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetPrincipalAxes`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetPrincipalAxes`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetPrincipalAxes'))

# Start of LabelShapeStatisticsImageFilter_GetPrincipalMoments

`LabelShapeStatisticsImageFilter_GetPrincipalMoments` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetPrincipalMoments', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetPrincipalMoments`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetPrincipalMoments`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetPrincipalMoments`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetPrincipalMoments'))

# Start of LabelShapeStatisticsImageFilter_GetRoundness

`LabelShapeStatisticsImageFilter_GetRoundness` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetRoundness', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetRoundness`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetRoundness`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetRoundness`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetRoundness'))

# Start of LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxSize

`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxSize` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxSize', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxSize`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxSize`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxSize`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxSize'))

# Start of LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxOrigin

`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxOrigin` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxOrigin', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxOrigin`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxOrigin`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxOrigin`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxOrigin'))

# Start of LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxDirection

`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxDirection` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxDirection', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxDirection`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxDirection`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxDirection`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxDirection'))

# Start of LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxVertices

`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxVertices` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxVertices', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxVertices`, 'returnType') = 'numeric'
attr(`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxVertices`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxVertices`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxVertices'))

# Start of LabelShapeStatisticsImageFilter_GetIndexes

`LabelShapeStatisticsImageFilter_GetIndexes` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetIndexes', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetIndexes`, 'returnType') = 'integer'
attr(`LabelShapeStatisticsImageFilter_GetIndexes`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetIndexes`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetIndexes'))

# Start of LabelShapeStatisticsImageFilter_GetRLEIndexes

`LabelShapeStatisticsImageFilter_GetRLEIndexes` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetRLEIndexes', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetRLEIndexes`, 'returnType') = 'integer'
attr(`LabelShapeStatisticsImageFilter_GetRLEIndexes`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_GetRLEIndexes`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetRLEIndexes'))

# Start of LabelShapeStatisticsImageFilter_GetName

`LabelShapeStatisticsImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetName`, 'returnType') = 'character'
attr(`LabelShapeStatisticsImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter')
class(`LabelShapeStatisticsImageFilter_GetName`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetName'))

# Start of LabelShapeStatisticsImageFilter_ToString

`LabelShapeStatisticsImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_ToString`, 'returnType') = 'character'
attr(`LabelShapeStatisticsImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter')
class(`LabelShapeStatisticsImageFilter_ToString`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_ToString'))

# Start of LabelShapeStatisticsImageFilter_Execute

`LabelShapeStatisticsImageFilter_Execute` = function(self, image1)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelShapeStatisticsImageFilter_Execute', self, image1, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelShapeStatisticsImageFilter_Execute`, 'returnType') = 'void'
attr(`LabelShapeStatisticsImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', '_p_itk__simple__Image')
class(`LabelShapeStatisticsImageFilter_Execute`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_Execute'))

# Start of LabelShapeStatisticsImageFilter_HasLabel

`LabelShapeStatisticsImageFilter_HasLabel` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_HasLabel', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_HasLabel`, 'returnType') = 'logical'
attr(`LabelShapeStatisticsImageFilter_HasLabel`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter', 'integer')
class(`LabelShapeStatisticsImageFilter_HasLabel`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_HasLabel'))

# Start of LabelShapeStatisticsImageFilter_GetNumberOfLabels

`LabelShapeStatisticsImageFilter_GetNumberOfLabels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelShapeStatisticsImageFilter_GetNumberOfLabels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelShapeStatisticsImageFilter_GetNumberOfLabels`, 'returnType') = 'integer'
attr(`LabelShapeStatisticsImageFilter_GetNumberOfLabels`, "inputTypes") = c('_p_itk__simple__LabelShapeStatisticsImageFilter')
class(`LabelShapeStatisticsImageFilter_GetNumberOfLabels`) = c("SWIGFunction", class('LabelShapeStatisticsImageFilter_GetNumberOfLabels'))

# Start of accessor method for itk::simple::LabelShapeStatisticsImageFilter
setMethod('$', '_p_itk__simple__LabelShapeStatisticsImageFilter', function(x, name)

{
  accessorFuns = list('SetBackgroundValue' = LabelShapeStatisticsImageFilter_SetBackgroundValue, 'GetBackgroundValue' = LabelShapeStatisticsImageFilter_GetBackgroundValue, 'SetComputeFeretDiameter' = LabelShapeStatisticsImageFilter_SetComputeFeretDiameter, 'ComputeFeretDiameterOn' = LabelShapeStatisticsImageFilter_ComputeFeretDiameterOn, 'ComputeFeretDiameterOff' = LabelShapeStatisticsImageFilter_ComputeFeretDiameterOff, 'GetComputeFeretDiameter' = LabelShapeStatisticsImageFilter_GetComputeFeretDiameter, 'SetComputePerimeter' = LabelShapeStatisticsImageFilter_SetComputePerimeter, 'ComputePerimeterOn' = LabelShapeStatisticsImageFilter_ComputePerimeterOn, 'ComputePerimeterOff' = LabelShapeStatisticsImageFilter_ComputePerimeterOff, 'GetComputePerimeter' = LabelShapeStatisticsImageFilter_GetComputePerimeter, 'SetComputeOrientedBoundingBox' = LabelShapeStatisticsImageFilter_SetComputeOrientedBoundingBox, 'ComputeOrientedBoundingBoxOn' = LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOn, 'ComputeOrientedBoundingBoxOff' = LabelShapeStatisticsImageFilter_ComputeOrientedBoundingBoxOff, 'GetComputeOrientedBoundingBox' = LabelShapeStatisticsImageFilter_GetComputeOrientedBoundingBox, 'GetBoundingBox' = LabelShapeStatisticsImageFilter_GetBoundingBox, 'GetRegion' = LabelShapeStatisticsImageFilter_GetRegion, 'GetCentroid' = LabelShapeStatisticsImageFilter_GetCentroid, 'GetElongation' = LabelShapeStatisticsImageFilter_GetElongation, 'GetEquivalentEllipsoidDiameter' = LabelShapeStatisticsImageFilter_GetEquivalentEllipsoidDiameter, 'GetEquivalentSphericalPerimeter' = LabelShapeStatisticsImageFilter_GetEquivalentSphericalPerimeter, 'GetEquivalentSphericalRadius' = LabelShapeStatisticsImageFilter_GetEquivalentSphericalRadius, 'GetFeretDiameter' = LabelShapeStatisticsImageFilter_GetFeretDiameter, 'GetFlatness' = LabelShapeStatisticsImageFilter_GetFlatness, 'GetLabels' = LabelShapeStatisticsImageFilter_GetLabels, 'GetNumberOfPixels' = LabelShapeStatisticsImageFilter_GetNumberOfPixels, 'GetNumberOfPixelsOnBorder' = LabelShapeStatisticsImageFilter_GetNumberOfPixelsOnBorder, 'GetPerimeter' = LabelShapeStatisticsImageFilter_GetPerimeter, 'GetPerimeterOnBorder' = LabelShapeStatisticsImageFilter_GetPerimeterOnBorder, 'GetPerimeterOnBorderRatio' = LabelShapeStatisticsImageFilter_GetPerimeterOnBorderRatio, 'GetPhysicalSize' = LabelShapeStatisticsImageFilter_GetPhysicalSize, 'GetPrincipalAxes' = LabelShapeStatisticsImageFilter_GetPrincipalAxes, 'GetPrincipalMoments' = LabelShapeStatisticsImageFilter_GetPrincipalMoments, 'GetRoundness' = LabelShapeStatisticsImageFilter_GetRoundness, 'GetOrientedBoundingBoxSize' = LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxSize, 'GetOrientedBoundingBoxOrigin' = LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxOrigin, 'GetOrientedBoundingBoxDirection' = LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxDirection, 'GetOrientedBoundingBoxVertices' = LabelShapeStatisticsImageFilter_GetOrientedBoundingBoxVertices, 'GetIndexes' = LabelShapeStatisticsImageFilter_GetIndexes, 'GetRLEIndexes' = LabelShapeStatisticsImageFilter_GetRLEIndexes, 'GetName' = LabelShapeStatisticsImageFilter_GetName, 'ToString' = LabelShapeStatisticsImageFilter_ToString, 'Execute' = LabelShapeStatisticsImageFilter_Execute, 'HasLabel' = LabelShapeStatisticsImageFilter_HasLabel, 'GetNumberOfLabels' = LabelShapeStatisticsImageFilter_GetNumberOfLabels);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelShapeStatisticsImageFilter
# Start of delete_LabelStatisticsImageFilter

`delete_LabelStatisticsImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelStatisticsImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelStatisticsImageFilter`, 'returnType') = 'void'
attr(`delete_LabelStatisticsImageFilter`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter')
class(`delete_LabelStatisticsImageFilter`) = c("SWIGFunction", class('delete_LabelStatisticsImageFilter'))

# Start of new_LabelStatisticsImageFilter

`LabelStatisticsImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelStatisticsImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelStatisticsImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelStatisticsImageFilter);
  ans
  
}

attr(`LabelStatisticsImageFilter`, 'returnType') = '_p_itk__simple__LabelStatisticsImageFilter'
class(`LabelStatisticsImageFilter`) = c("SWIGFunction", class('LabelStatisticsImageFilter'))

# Start of LabelStatisticsImageFilter_SetUseHistograms

`LabelStatisticsImageFilter_SetUseHistograms` = function(self, UseHistograms)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseHistograms = as.logical(UseHistograms);
  ;ans = .Call('R_swig_LabelStatisticsImageFilter_SetUseHistograms', self, UseHistograms, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelStatisticsImageFilter_SetUseHistograms`, 'returnType') = 'void'
attr(`LabelStatisticsImageFilter_SetUseHistograms`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter', 'logical')
class(`LabelStatisticsImageFilter_SetUseHistograms`) = c("SWIGFunction", class('LabelStatisticsImageFilter_SetUseHistograms'))

# Start of LabelStatisticsImageFilter_UseHistogramsOn

`LabelStatisticsImageFilter_UseHistogramsOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelStatisticsImageFilter_UseHistogramsOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelStatisticsImageFilter_UseHistogramsOn`, 'returnType') = 'void'
attr(`LabelStatisticsImageFilter_UseHistogramsOn`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter')
class(`LabelStatisticsImageFilter_UseHistogramsOn`) = c("SWIGFunction", class('LabelStatisticsImageFilter_UseHistogramsOn'))

# Start of LabelStatisticsImageFilter_UseHistogramsOff

`LabelStatisticsImageFilter_UseHistogramsOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelStatisticsImageFilter_UseHistogramsOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelStatisticsImageFilter_UseHistogramsOff`, 'returnType') = 'void'
attr(`LabelStatisticsImageFilter_UseHistogramsOff`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter')
class(`LabelStatisticsImageFilter_UseHistogramsOff`) = c("SWIGFunction", class('LabelStatisticsImageFilter_UseHistogramsOff'))

# Start of LabelStatisticsImageFilter_GetUseHistograms

`LabelStatisticsImageFilter_GetUseHistograms` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelStatisticsImageFilter_GetUseHistograms', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_GetUseHistograms`, 'returnType') = 'logical'
attr(`LabelStatisticsImageFilter_GetUseHistograms`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter')
class(`LabelStatisticsImageFilter_GetUseHistograms`) = c("SWIGFunction", class('LabelStatisticsImageFilter_GetUseHistograms'))

# Start of LabelStatisticsImageFilter_GetMinimum

`LabelStatisticsImageFilter_GetMinimum` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelStatisticsImageFilter_GetMinimum', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_GetMinimum`, 'returnType') = 'numeric'
attr(`LabelStatisticsImageFilter_GetMinimum`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter', 'integer')
class(`LabelStatisticsImageFilter_GetMinimum`) = c("SWIGFunction", class('LabelStatisticsImageFilter_GetMinimum'))

# Start of LabelStatisticsImageFilter_GetMaximum

`LabelStatisticsImageFilter_GetMaximum` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelStatisticsImageFilter_GetMaximum', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_GetMaximum`, 'returnType') = 'numeric'
attr(`LabelStatisticsImageFilter_GetMaximum`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter', 'integer')
class(`LabelStatisticsImageFilter_GetMaximum`) = c("SWIGFunction", class('LabelStatisticsImageFilter_GetMaximum'))

# Start of LabelStatisticsImageFilter_GetMean

`LabelStatisticsImageFilter_GetMean` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelStatisticsImageFilter_GetMean', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_GetMean`, 'returnType') = 'numeric'
attr(`LabelStatisticsImageFilter_GetMean`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter', 'integer')
class(`LabelStatisticsImageFilter_GetMean`) = c("SWIGFunction", class('LabelStatisticsImageFilter_GetMean'))

# Start of LabelStatisticsImageFilter_GetMedian

`LabelStatisticsImageFilter_GetMedian` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelStatisticsImageFilter_GetMedian', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_GetMedian`, 'returnType') = 'numeric'
attr(`LabelStatisticsImageFilter_GetMedian`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter', 'integer')
class(`LabelStatisticsImageFilter_GetMedian`) = c("SWIGFunction", class('LabelStatisticsImageFilter_GetMedian'))

# Start of LabelStatisticsImageFilter_GetSigma

`LabelStatisticsImageFilter_GetSigma` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelStatisticsImageFilter_GetSigma', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_GetSigma`, 'returnType') = 'numeric'
attr(`LabelStatisticsImageFilter_GetSigma`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter', 'integer')
class(`LabelStatisticsImageFilter_GetSigma`) = c("SWIGFunction", class('LabelStatisticsImageFilter_GetSigma'))

# Start of LabelStatisticsImageFilter_GetVariance

`LabelStatisticsImageFilter_GetVariance` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelStatisticsImageFilter_GetVariance', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_GetVariance`, 'returnType') = 'numeric'
attr(`LabelStatisticsImageFilter_GetVariance`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter', 'integer')
class(`LabelStatisticsImageFilter_GetVariance`) = c("SWIGFunction", class('LabelStatisticsImageFilter_GetVariance'))

# Start of LabelStatisticsImageFilter_GetSum

`LabelStatisticsImageFilter_GetSum` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelStatisticsImageFilter_GetSum', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_GetSum`, 'returnType') = 'numeric'
attr(`LabelStatisticsImageFilter_GetSum`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter', 'integer')
class(`LabelStatisticsImageFilter_GetSum`) = c("SWIGFunction", class('LabelStatisticsImageFilter_GetSum'))

# Start of LabelStatisticsImageFilter_GetCount

`LabelStatisticsImageFilter_GetCount` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelStatisticsImageFilter_GetCount', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_GetCount`, 'returnType') = 'integer'
attr(`LabelStatisticsImageFilter_GetCount`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter', 'integer')
class(`LabelStatisticsImageFilter_GetCount`) = c("SWIGFunction", class('LabelStatisticsImageFilter_GetCount'))

# Start of LabelStatisticsImageFilter_GetBoundingBox

`LabelStatisticsImageFilter_GetBoundingBox` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelStatisticsImageFilter_GetBoundingBox', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_GetBoundingBox`, 'returnType') = 'integer'
attr(`LabelStatisticsImageFilter_GetBoundingBox`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter', 'integer')
class(`LabelStatisticsImageFilter_GetBoundingBox`) = c("SWIGFunction", class('LabelStatisticsImageFilter_GetBoundingBox'))

# Start of LabelStatisticsImageFilter_GetRegion

`LabelStatisticsImageFilter_GetRegion` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelStatisticsImageFilter_GetRegion', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_GetRegion`, 'returnType') = 'integer'
attr(`LabelStatisticsImageFilter_GetRegion`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter', 'integer')
class(`LabelStatisticsImageFilter_GetRegion`) = c("SWIGFunction", class('LabelStatisticsImageFilter_GetRegion'))

# Start of LabelStatisticsImageFilter_GetLabels

`LabelStatisticsImageFilter_GetLabels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelStatisticsImageFilter_GetLabels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_GetLabels`, 'returnType') = 'integer'
attr(`LabelStatisticsImageFilter_GetLabels`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter')
class(`LabelStatisticsImageFilter_GetLabels`) = c("SWIGFunction", class('LabelStatisticsImageFilter_GetLabels'))

# Start of LabelStatisticsImageFilter_GetName

`LabelStatisticsImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelStatisticsImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_GetName`, 'returnType') = 'character'
attr(`LabelStatisticsImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter')
class(`LabelStatisticsImageFilter_GetName`) = c("SWIGFunction", class('LabelStatisticsImageFilter_GetName'))

# Start of LabelStatisticsImageFilter_ToString

`LabelStatisticsImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelStatisticsImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_ToString`, 'returnType') = 'character'
attr(`LabelStatisticsImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter')
class(`LabelStatisticsImageFilter_ToString`) = c("SWIGFunction", class('LabelStatisticsImageFilter_ToString'))

# Start of LabelStatisticsImageFilter_Execute

`LabelStatisticsImageFilter_Execute` = function(self, image, labelImage)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(labelImage, "ExternalReference")) labelImage = slot(labelImage,"ref"); 
  ;ans = .Call('R_swig_LabelStatisticsImageFilter_Execute', self, image, labelImage, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelStatisticsImageFilter_Execute`, 'returnType') = 'void'
attr(`LabelStatisticsImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelStatisticsImageFilter_Execute`) = c("SWIGFunction", class('LabelStatisticsImageFilter_Execute'))

# Start of LabelStatisticsImageFilter_HasLabel

`LabelStatisticsImageFilter_HasLabel` = function(self, label, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  label = as.integer(label);
  
  if(length(label) > 1) {
    warning("using only the first element of label");
  };
  
  ;.Call('R_swig_LabelStatisticsImageFilter_HasLabel', self, label, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_HasLabel`, 'returnType') = 'logical'
attr(`LabelStatisticsImageFilter_HasLabel`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter', 'integer')
class(`LabelStatisticsImageFilter_HasLabel`) = c("SWIGFunction", class('LabelStatisticsImageFilter_HasLabel'))

# Start of LabelStatisticsImageFilter_GetNumberOfLabels

`LabelStatisticsImageFilter_GetNumberOfLabels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelStatisticsImageFilter_GetNumberOfLabels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelStatisticsImageFilter_GetNumberOfLabels`, 'returnType') = 'integer'
attr(`LabelStatisticsImageFilter_GetNumberOfLabels`, "inputTypes") = c('_p_itk__simple__LabelStatisticsImageFilter')
class(`LabelStatisticsImageFilter_GetNumberOfLabels`) = c("SWIGFunction", class('LabelStatisticsImageFilter_GetNumberOfLabels'))

# Start of accessor method for itk::simple::LabelStatisticsImageFilter
setMethod('$', '_p_itk__simple__LabelStatisticsImageFilter', function(x, name)

{
  accessorFuns = list('SetUseHistograms' = LabelStatisticsImageFilter_SetUseHistograms, 'UseHistogramsOn' = LabelStatisticsImageFilter_UseHistogramsOn, 'UseHistogramsOff' = LabelStatisticsImageFilter_UseHistogramsOff, 'GetUseHistograms' = LabelStatisticsImageFilter_GetUseHistograms, 'GetMinimum' = LabelStatisticsImageFilter_GetMinimum, 'GetMaximum' = LabelStatisticsImageFilter_GetMaximum, 'GetMean' = LabelStatisticsImageFilter_GetMean, 'GetMedian' = LabelStatisticsImageFilter_GetMedian, 'GetSigma' = LabelStatisticsImageFilter_GetSigma, 'GetVariance' = LabelStatisticsImageFilter_GetVariance, 'GetSum' = LabelStatisticsImageFilter_GetSum, 'GetCount' = LabelStatisticsImageFilter_GetCount, 'GetBoundingBox' = LabelStatisticsImageFilter_GetBoundingBox, 'GetRegion' = LabelStatisticsImageFilter_GetRegion, 'GetLabels' = LabelStatisticsImageFilter_GetLabels, 'GetName' = LabelStatisticsImageFilter_GetName, 'ToString' = LabelStatisticsImageFilter_ToString, 'Execute' = LabelStatisticsImageFilter_Execute, 'HasLabel' = LabelStatisticsImageFilter_HasLabel, 'GetNumberOfLabels' = LabelStatisticsImageFilter_GetNumberOfLabels);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelStatisticsImageFilter
# Start of delete_LabelToRGBImageFilter

`delete_LabelToRGBImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelToRGBImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelToRGBImageFilter`, 'returnType') = 'void'
attr(`delete_LabelToRGBImageFilter`, "inputTypes") = c('_p_itk__simple__LabelToRGBImageFilter')
class(`delete_LabelToRGBImageFilter`) = c("SWIGFunction", class('delete_LabelToRGBImageFilter'))

# Start of new_LabelToRGBImageFilter

`LabelToRGBImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelToRGBImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelToRGBImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelToRGBImageFilter);
  ans
  
}

attr(`LabelToRGBImageFilter`, 'returnType') = '_p_itk__simple__LabelToRGBImageFilter'
class(`LabelToRGBImageFilter`) = c("SWIGFunction", class('LabelToRGBImageFilter'))

# Start of LabelToRGBImageFilter_SetBackgroundValue

`LabelToRGBImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LabelToRGBImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelToRGBImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`LabelToRGBImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelToRGBImageFilter', 'numeric')
class(`LabelToRGBImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('LabelToRGBImageFilter_SetBackgroundValue'))

# Start of LabelToRGBImageFilter_GetBackgroundValue

`LabelToRGBImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelToRGBImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelToRGBImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`LabelToRGBImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LabelToRGBImageFilter')
class(`LabelToRGBImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('LabelToRGBImageFilter_GetBackgroundValue'))

# Start of LabelToRGBImageFilter_SetColormap

`LabelToRGBImageFilter_SetColormap` = function(self, Colormap)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Colormap = as.integer(Colormap);
  ;ans = .Call('R_swig_LabelToRGBImageFilter_SetColormap', self, Colormap, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelToRGBImageFilter_SetColormap`, 'returnType') = 'void'
attr(`LabelToRGBImageFilter_SetColormap`, "inputTypes") = c('_p_itk__simple__LabelToRGBImageFilter', 'integer')
class(`LabelToRGBImageFilter_SetColormap`) = c("SWIGFunction", class('LabelToRGBImageFilter_SetColormap'))

# Start of LabelToRGBImageFilter_GetColormap

`LabelToRGBImageFilter_GetColormap` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelToRGBImageFilter_GetColormap', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelToRGBImageFilter_GetColormap`, 'returnType') = 'integer'
attr(`LabelToRGBImageFilter_GetColormap`, "inputTypes") = c('_p_itk__simple__LabelToRGBImageFilter')
class(`LabelToRGBImageFilter_GetColormap`) = c("SWIGFunction", class('LabelToRGBImageFilter_GetColormap'))

# Start of LabelToRGBImageFilter_GetName

`LabelToRGBImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelToRGBImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelToRGBImageFilter_GetName`, 'returnType') = 'character'
attr(`LabelToRGBImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelToRGBImageFilter')
class(`LabelToRGBImageFilter_GetName`) = c("SWIGFunction", class('LabelToRGBImageFilter_GetName'))

# Start of LabelToRGBImageFilter_ToString

`LabelToRGBImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelToRGBImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelToRGBImageFilter_ToString`, 'returnType') = 'character'
attr(`LabelToRGBImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelToRGBImageFilter')
class(`LabelToRGBImageFilter_ToString`) = c("SWIGFunction", class('LabelToRGBImageFilter_ToString'))

# Start of LabelToRGBImageFilter_Execute

`LabelToRGBImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelToRGBImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelToRGBImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelToRGBImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LabelToRGBImageFilter', '_p_itk__simple__Image')
class(`LabelToRGBImageFilter_Execute`) = c("SWIGFunction", class('LabelToRGBImageFilter_Execute'))

# Start of accessor method for itk::simple::LabelToRGBImageFilter
setMethod('$', '_p_itk__simple__LabelToRGBImageFilter', function(x, name)

{
  accessorFuns = list('SetBackgroundValue' = LabelToRGBImageFilter_SetBackgroundValue, 'GetBackgroundValue' = LabelToRGBImageFilter_GetBackgroundValue, 'SetColormap' = LabelToRGBImageFilter_SetColormap, 'GetColormap' = LabelToRGBImageFilter_GetColormap, 'GetName' = LabelToRGBImageFilter_GetName, 'ToString' = LabelToRGBImageFilter_ToString, 'Execute' = LabelToRGBImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelToRGBImageFilter
# Start of LabelToRGB

`LabelToRGB__SWIG_0` = function(image1, backgroundValue, colormap, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  colormap = as.integer(colormap);
  ;ans = .Call('R_swig_LabelToRGB__SWIG_0', image1, backgroundValue, colormap, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelToRGB__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelToRGB__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`LabelToRGB__SWIG_0`) = c("SWIGFunction", class('LabelToRGB__SWIG_0'))

# Start of LabelToRGB

`LabelToRGB__SWIG_1` = function(image1, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_LabelToRGB__SWIG_1', image1, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelToRGB__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelToRGB__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`LabelToRGB__SWIG_1`) = c("SWIGFunction", class('LabelToRGB__SWIG_1'))

# Start of LabelToRGB

`LabelToRGB__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelToRGB__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelToRGB__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelToRGB__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`LabelToRGB__SWIG_2`) = c("SWIGFunction", class('LabelToRGB__SWIG_2'))

`LabelToRGB` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- LabelToRGB__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- LabelToRGB__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- LabelToRGB__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelToRGB with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LabelUniqueLabelMapFilter

`delete_LabelUniqueLabelMapFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelUniqueLabelMapFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelUniqueLabelMapFilter`, 'returnType') = 'void'
attr(`delete_LabelUniqueLabelMapFilter`, "inputTypes") = c('_p_itk__simple__LabelUniqueLabelMapFilter')
class(`delete_LabelUniqueLabelMapFilter`) = c("SWIGFunction", class('delete_LabelUniqueLabelMapFilter'))

# Start of new_LabelUniqueLabelMapFilter

`LabelUniqueLabelMapFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelUniqueLabelMapFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelUniqueLabelMapFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelUniqueLabelMapFilter);
  ans
  
}

attr(`LabelUniqueLabelMapFilter`, 'returnType') = '_p_itk__simple__LabelUniqueLabelMapFilter'
class(`LabelUniqueLabelMapFilter`) = c("SWIGFunction", class('LabelUniqueLabelMapFilter'))

# Start of LabelUniqueLabelMapFilter_SetReverseOrdering

`LabelUniqueLabelMapFilter_SetReverseOrdering` = function(self, ReverseOrdering)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ReverseOrdering = as.logical(ReverseOrdering);
  ;ans = .Call('R_swig_LabelUniqueLabelMapFilter_SetReverseOrdering', self, ReverseOrdering, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelUniqueLabelMapFilter_SetReverseOrdering`, 'returnType') = 'void'
attr(`LabelUniqueLabelMapFilter_SetReverseOrdering`, "inputTypes") = c('_p_itk__simple__LabelUniqueLabelMapFilter', 'logical')
class(`LabelUniqueLabelMapFilter_SetReverseOrdering`) = c("SWIGFunction", class('LabelUniqueLabelMapFilter_SetReverseOrdering'))

# Start of LabelUniqueLabelMapFilter_ReverseOrderingOn

`LabelUniqueLabelMapFilter_ReverseOrderingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelUniqueLabelMapFilter_ReverseOrderingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelUniqueLabelMapFilter_ReverseOrderingOn`, 'returnType') = 'void'
attr(`LabelUniqueLabelMapFilter_ReverseOrderingOn`, "inputTypes") = c('_p_itk__simple__LabelUniqueLabelMapFilter')
class(`LabelUniqueLabelMapFilter_ReverseOrderingOn`) = c("SWIGFunction", class('LabelUniqueLabelMapFilter_ReverseOrderingOn'))

# Start of LabelUniqueLabelMapFilter_ReverseOrderingOff

`LabelUniqueLabelMapFilter_ReverseOrderingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LabelUniqueLabelMapFilter_ReverseOrderingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelUniqueLabelMapFilter_ReverseOrderingOff`, 'returnType') = 'void'
attr(`LabelUniqueLabelMapFilter_ReverseOrderingOff`, "inputTypes") = c('_p_itk__simple__LabelUniqueLabelMapFilter')
class(`LabelUniqueLabelMapFilter_ReverseOrderingOff`) = c("SWIGFunction", class('LabelUniqueLabelMapFilter_ReverseOrderingOff'))

# Start of LabelUniqueLabelMapFilter_GetReverseOrdering

`LabelUniqueLabelMapFilter_GetReverseOrdering` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelUniqueLabelMapFilter_GetReverseOrdering', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelUniqueLabelMapFilter_GetReverseOrdering`, 'returnType') = 'logical'
attr(`LabelUniqueLabelMapFilter_GetReverseOrdering`, "inputTypes") = c('_p_itk__simple__LabelUniqueLabelMapFilter')
class(`LabelUniqueLabelMapFilter_GetReverseOrdering`) = c("SWIGFunction", class('LabelUniqueLabelMapFilter_GetReverseOrdering'))

# Start of LabelUniqueLabelMapFilter_GetName

`LabelUniqueLabelMapFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelUniqueLabelMapFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelUniqueLabelMapFilter_GetName`, 'returnType') = 'character'
attr(`LabelUniqueLabelMapFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelUniqueLabelMapFilter')
class(`LabelUniqueLabelMapFilter_GetName`) = c("SWIGFunction", class('LabelUniqueLabelMapFilter_GetName'))

# Start of LabelUniqueLabelMapFilter_ToString

`LabelUniqueLabelMapFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelUniqueLabelMapFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelUniqueLabelMapFilter_ToString`, 'returnType') = 'character'
attr(`LabelUniqueLabelMapFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelUniqueLabelMapFilter')
class(`LabelUniqueLabelMapFilter_ToString`) = c("SWIGFunction", class('LabelUniqueLabelMapFilter_ToString'))

# Start of LabelUniqueLabelMapFilter_Execute

`LabelUniqueLabelMapFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelUniqueLabelMapFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelUniqueLabelMapFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelUniqueLabelMapFilter_Execute`, "inputTypes") = c('_p_itk__simple__LabelUniqueLabelMapFilter', '_p_itk__simple__Image')
class(`LabelUniqueLabelMapFilter_Execute`) = c("SWIGFunction", class('LabelUniqueLabelMapFilter_Execute'))

# Start of accessor method for itk::simple::LabelUniqueLabelMapFilter
setMethod('$', '_p_itk__simple__LabelUniqueLabelMapFilter', function(x, name)

{
  accessorFuns = list('SetReverseOrdering' = LabelUniqueLabelMapFilter_SetReverseOrdering, 'ReverseOrderingOn' = LabelUniqueLabelMapFilter_ReverseOrderingOn, 'ReverseOrderingOff' = LabelUniqueLabelMapFilter_ReverseOrderingOff, 'GetReverseOrdering' = LabelUniqueLabelMapFilter_GetReverseOrdering, 'GetName' = LabelUniqueLabelMapFilter_GetName, 'ToString' = LabelUniqueLabelMapFilter_ToString, 'Execute' = LabelUniqueLabelMapFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelUniqueLabelMapFilter
# Start of LabelUniqueLabelMap

`LabelUniqueLabelMap__SWIG_0` = function(image1, reverseOrdering, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  reverseOrdering = as.logical(reverseOrdering);
  ;ans = .Call('R_swig_LabelUniqueLabelMap__SWIG_0', image1, reverseOrdering, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelUniqueLabelMap__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelUniqueLabelMap__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`LabelUniqueLabelMap__SWIG_0`) = c("SWIGFunction", class('LabelUniqueLabelMap__SWIG_0'))

# Start of LabelUniqueLabelMap

`LabelUniqueLabelMap__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelUniqueLabelMap__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelUniqueLabelMap__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelUniqueLabelMap__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`LabelUniqueLabelMap__SWIG_1`) = c("SWIGFunction", class('LabelUniqueLabelMap__SWIG_1'))

`LabelUniqueLabelMap` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- LabelUniqueLabelMap__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- LabelUniqueLabelMap__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelUniqueLabelMap with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LabelVotingImageFilter

`delete_LabelVotingImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LabelVotingImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LabelVotingImageFilter`, 'returnType') = 'void'
attr(`delete_LabelVotingImageFilter`, "inputTypes") = c('_p_itk__simple__LabelVotingImageFilter')
class(`delete_LabelVotingImageFilter`) = c("SWIGFunction", class('delete_LabelVotingImageFilter'))

# Start of new_LabelVotingImageFilter

`LabelVotingImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LabelVotingImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LabelVotingImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LabelVotingImageFilter);
  ans
  
}

attr(`LabelVotingImageFilter`, 'returnType') = '_p_itk__simple__LabelVotingImageFilter'
class(`LabelVotingImageFilter`) = c("SWIGFunction", class('LabelVotingImageFilter'))

# Start of LabelVotingImageFilter_SetLabelForUndecidedPixels

`LabelVotingImageFilter_SetLabelForUndecidedPixels` = function(self, LabelForUndecidedPixels)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  LabelForUndecidedPixels = as.integer(LabelForUndecidedPixels);
  
  if(length(LabelForUndecidedPixels) > 1) {
    warning("using only the first element of LabelForUndecidedPixels");
  };
  
  ;ans = .Call('R_swig_LabelVotingImageFilter_SetLabelForUndecidedPixels', self, LabelForUndecidedPixels, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LabelVotingImageFilter_SetLabelForUndecidedPixels`, 'returnType') = 'void'
attr(`LabelVotingImageFilter_SetLabelForUndecidedPixels`, "inputTypes") = c('_p_itk__simple__LabelVotingImageFilter', 'integer')
class(`LabelVotingImageFilter_SetLabelForUndecidedPixels`) = c("SWIGFunction", class('LabelVotingImageFilter_SetLabelForUndecidedPixels'))

# Start of LabelVotingImageFilter_GetLabelForUndecidedPixels

`LabelVotingImageFilter_GetLabelForUndecidedPixels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelVotingImageFilter_GetLabelForUndecidedPixels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelVotingImageFilter_GetLabelForUndecidedPixels`, 'returnType') = 'integer'
attr(`LabelVotingImageFilter_GetLabelForUndecidedPixels`, "inputTypes") = c('_p_itk__simple__LabelVotingImageFilter')
class(`LabelVotingImageFilter_GetLabelForUndecidedPixels`) = c("SWIGFunction", class('LabelVotingImageFilter_GetLabelForUndecidedPixels'))

# Start of LabelVotingImageFilter_GetName

`LabelVotingImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelVotingImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelVotingImageFilter_GetName`, 'returnType') = 'character'
attr(`LabelVotingImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LabelVotingImageFilter')
class(`LabelVotingImageFilter_GetName`) = c("SWIGFunction", class('LabelVotingImageFilter_GetName'))

# Start of LabelVotingImageFilter_ToString

`LabelVotingImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LabelVotingImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LabelVotingImageFilter_ToString`, 'returnType') = 'character'
attr(`LabelVotingImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LabelVotingImageFilter')
class(`LabelVotingImageFilter_ToString`) = c("SWIGFunction", class('LabelVotingImageFilter_ToString'))

# Start of LabelVotingImageFilter_Execute

`LabelVotingImageFilter_Execute__SWIG_0` = function(self, images, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_LabelVotingImageFilter_Execute__SWIG_0', self, images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVotingImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVotingImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__LabelVotingImageFilter', 'list')
class(`LabelVotingImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('LabelVotingImageFilter_Execute__SWIG_0'))

# Start of LabelVotingImageFilter_Execute

`LabelVotingImageFilter_Execute__SWIG_1` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelVotingImageFilter_Execute__SWIG_1', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVotingImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVotingImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__LabelVotingImageFilter', '_p_itk__simple__Image')
class(`LabelVotingImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('LabelVotingImageFilter_Execute__SWIG_1'))

# Start of LabelVotingImageFilter_Execute

`LabelVotingImageFilter_Execute__SWIG_2` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_LabelVotingImageFilter_Execute__SWIG_2', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVotingImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVotingImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__LabelVotingImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelVotingImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('LabelVotingImageFilter_Execute__SWIG_2'))

# Start of LabelVotingImageFilter_Execute

`LabelVotingImageFilter_Execute__SWIG_3` = function(self, image1, image2, image3, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_LabelVotingImageFilter_Execute__SWIG_3', self, image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVotingImageFilter_Execute__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVotingImageFilter_Execute__SWIG_3`, "inputTypes") = c('_p_itk__simple__LabelVotingImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelVotingImageFilter_Execute__SWIG_3`) = c("SWIGFunction", class('LabelVotingImageFilter_Execute__SWIG_3'))

# Start of LabelVotingImageFilter_Execute

`LabelVotingImageFilter_Execute__SWIG_4` = function(self, image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_LabelVotingImageFilter_Execute__SWIG_4', self, image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVotingImageFilter_Execute__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVotingImageFilter_Execute__SWIG_4`, "inputTypes") = c('_p_itk__simple__LabelVotingImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelVotingImageFilter_Execute__SWIG_4`) = c("SWIGFunction", class('LabelVotingImageFilter_Execute__SWIG_4'))

# Start of LabelVotingImageFilter_Execute

`LabelVotingImageFilter_Execute__SWIG_5` = function(self, image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_LabelVotingImageFilter_Execute__SWIG_5', self, image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVotingImageFilter_Execute__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVotingImageFilter_Execute__SWIG_5`, "inputTypes") = c('_p_itk__simple__LabelVotingImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelVotingImageFilter_Execute__SWIG_5`) = c("SWIGFunction", class('LabelVotingImageFilter_Execute__SWIG_5'))

`LabelVotingImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__LabelVotingImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- LabelVotingImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__LabelVotingImageFilter') || is.null(argv[[1]]) ) && (
        (is.list(argv[[2]]) & all(sapply(argv[[2]], inherits, what='_p_itk__simple__Image')))
        )) {
      f <- LabelVotingImageFilter_Execute__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__LabelVotingImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- LabelVotingImageFilter_Execute__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__LabelVotingImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- LabelVotingImageFilter_Execute__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__LabelVotingImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- LabelVotingImageFilter_Execute__SWIG_4; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__LabelVotingImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( extends(argtypes[6], '_p_itk__simple__Image') && length(argv[[6]]) == 1 )) {
      f <- LabelVotingImageFilter_Execute__SWIG_5; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelVotingImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::LabelVotingImageFilter
setMethod('$', '_p_itk__simple__LabelVotingImageFilter', function(x, name)

{
  accessorFuns = list('SetLabelForUndecidedPixels' = LabelVotingImageFilter_SetLabelForUndecidedPixels, 'GetLabelForUndecidedPixels' = LabelVotingImageFilter_GetLabelForUndecidedPixels, 'GetName' = LabelVotingImageFilter_GetName, 'ToString' = LabelVotingImageFilter_ToString, 'Execute' = LabelVotingImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LabelVotingImageFilter
# Start of LabelVoting

`LabelVoting__SWIG_0` = function(images, labelForUndecidedPixels, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  ;ans = .Call('R_swig_LabelVoting__SWIG_0', images, labelForUndecidedPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVoting__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVoting__SWIG_0`, "inputTypes") = c('list', 'integer')
class(`LabelVoting__SWIG_0`) = c("SWIGFunction", class('LabelVoting__SWIG_0'))

# Start of LabelVoting

`LabelVoting__SWIG_1` = function(images, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_LabelVoting__SWIG_1', images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVoting__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVoting__SWIG_1`, "inputTypes") = c('list')
class(`LabelVoting__SWIG_1`) = c("SWIGFunction", class('LabelVoting__SWIG_1'))

# Start of LabelVoting

`LabelVoting__SWIG_2` = function(image1, labelForUndecidedPixels, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  ;ans = .Call('R_swig_LabelVoting__SWIG_2', image1, labelForUndecidedPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVoting__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVoting__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`LabelVoting__SWIG_2`) = c("SWIGFunction", class('LabelVoting__SWIG_2'))

# Start of LabelVoting

`LabelVoting__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LabelVoting__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVoting__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVoting__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`LabelVoting__SWIG_3`) = c("SWIGFunction", class('LabelVoting__SWIG_3'))

# Start of LabelVoting

`LabelVoting__SWIG_4` = function(image1, image2, labelForUndecidedPixels, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  ;ans = .Call('R_swig_LabelVoting__SWIG_4', image1, image2, labelForUndecidedPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVoting__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVoting__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`LabelVoting__SWIG_4`) = c("SWIGFunction", class('LabelVoting__SWIG_4'))

# Start of LabelVoting

`LabelVoting__SWIG_5` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_LabelVoting__SWIG_5', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVoting__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVoting__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelVoting__SWIG_5`) = c("SWIGFunction", class('LabelVoting__SWIG_5'))

# Start of LabelVoting

`LabelVoting__SWIG_6` = function(image1, image2, image3, labelForUndecidedPixels, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  ;ans = .Call('R_swig_LabelVoting__SWIG_6', image1, image2, image3, labelForUndecidedPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVoting__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVoting__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`LabelVoting__SWIG_6`) = c("SWIGFunction", class('LabelVoting__SWIG_6'))

# Start of LabelVoting

`LabelVoting__SWIG_7` = function(image1, image2, image3, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_LabelVoting__SWIG_7', image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVoting__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVoting__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelVoting__SWIG_7`) = c("SWIGFunction", class('LabelVoting__SWIG_7'))

# Start of LabelVoting

`LabelVoting__SWIG_8` = function(image1, image2, image3, image4, labelForUndecidedPixels, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  ;ans = .Call('R_swig_LabelVoting__SWIG_8', image1, image2, image3, image4, labelForUndecidedPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVoting__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVoting__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`LabelVoting__SWIG_8`) = c("SWIGFunction", class('LabelVoting__SWIG_8'))

# Start of LabelVoting

`LabelVoting__SWIG_9` = function(image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_LabelVoting__SWIG_9', image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVoting__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVoting__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelVoting__SWIG_9`) = c("SWIGFunction", class('LabelVoting__SWIG_9'))

# Start of LabelVoting

`LabelVoting__SWIG_10` = function(image1, image2, image3, image4, image5, labelForUndecidedPixels, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  ;ans = .Call('R_swig_LabelVoting__SWIG_10', image1, image2, image3, image4, image5, labelForUndecidedPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVoting__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVoting__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`LabelVoting__SWIG_10`) = c("SWIGFunction", class('LabelVoting__SWIG_10'))

# Start of LabelVoting

`LabelVoting__SWIG_11` = function(image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_LabelVoting__SWIG_11', image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LabelVoting__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`LabelVoting__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LabelVoting__SWIG_11`) = c("SWIGFunction", class('LabelVoting__SWIG_11'))

`LabelVoting` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 12
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- LabelVoting__SWIG_3; 
    }
    else if (
      (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
      ) {
      f <- LabelVoting__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- LabelVoting__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- LabelVoting__SWIG_2; 
    }
    else if ((
        (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
        ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- LabelVoting__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- LabelVoting__SWIG_7; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- LabelVoting__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- LabelVoting__SWIG_9; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- LabelVoting__SWIG_6; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- LabelVoting__SWIG_11; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- LabelVoting__SWIG_8; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- LabelVoting__SWIG_10; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LabelVoting with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LandweberDeconvolutionImageFilter

`delete_LandweberDeconvolutionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LandweberDeconvolutionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LandweberDeconvolutionImageFilter`, 'returnType') = 'void'
attr(`delete_LandweberDeconvolutionImageFilter`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter')
class(`delete_LandweberDeconvolutionImageFilter`) = c("SWIGFunction", class('delete_LandweberDeconvolutionImageFilter'))

# Start of new_LandweberDeconvolutionImageFilter

`LandweberDeconvolutionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LandweberDeconvolutionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LandweberDeconvolutionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LandweberDeconvolutionImageFilter);
  ans
  
}

attr(`LandweberDeconvolutionImageFilter`, 'returnType') = '_p_itk__simple__LandweberDeconvolutionImageFilter'
class(`LandweberDeconvolutionImageFilter`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter'))

# Start of LandweberDeconvolutionImageFilter_SetAlpha

`LandweberDeconvolutionImageFilter_SetAlpha` = function(self, Alpha)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LandweberDeconvolutionImageFilter_SetAlpha', self, Alpha, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LandweberDeconvolutionImageFilter_SetAlpha`, 'returnType') = 'void'
attr(`LandweberDeconvolutionImageFilter_SetAlpha`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter', 'numeric')
class(`LandweberDeconvolutionImageFilter_SetAlpha`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_SetAlpha'))

# Start of LandweberDeconvolutionImageFilter_GetAlpha

`LandweberDeconvolutionImageFilter_GetAlpha` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LandweberDeconvolutionImageFilter_GetAlpha', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LandweberDeconvolutionImageFilter_GetAlpha`, 'returnType') = 'numeric'
attr(`LandweberDeconvolutionImageFilter_GetAlpha`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter')
class(`LandweberDeconvolutionImageFilter_GetAlpha`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_GetAlpha'))

# Start of LandweberDeconvolutionImageFilter_SetNumberOfIterations

`LandweberDeconvolutionImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_LandweberDeconvolutionImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LandweberDeconvolutionImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`LandweberDeconvolutionImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter', 'integer')
class(`LandweberDeconvolutionImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_SetNumberOfIterations'))

# Start of LandweberDeconvolutionImageFilter_GetNumberOfIterations

`LandweberDeconvolutionImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LandweberDeconvolutionImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LandweberDeconvolutionImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`LandweberDeconvolutionImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter')
class(`LandweberDeconvolutionImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_GetNumberOfIterations'))

# Start of LandweberDeconvolutionImageFilter_SetNormalize

`LandweberDeconvolutionImageFilter_SetNormalize` = function(self, Normalize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Normalize = as.logical(Normalize);
  ;ans = .Call('R_swig_LandweberDeconvolutionImageFilter_SetNormalize', self, Normalize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LandweberDeconvolutionImageFilter_SetNormalize`, 'returnType') = 'void'
attr(`LandweberDeconvolutionImageFilter_SetNormalize`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter', 'logical')
class(`LandweberDeconvolutionImageFilter_SetNormalize`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_SetNormalize'))

# Start of LandweberDeconvolutionImageFilter_NormalizeOn

`LandweberDeconvolutionImageFilter_NormalizeOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LandweberDeconvolutionImageFilter_NormalizeOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LandweberDeconvolutionImageFilter_NormalizeOn`, 'returnType') = 'void'
attr(`LandweberDeconvolutionImageFilter_NormalizeOn`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter')
class(`LandweberDeconvolutionImageFilter_NormalizeOn`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_NormalizeOn'))

# Start of LandweberDeconvolutionImageFilter_NormalizeOff

`LandweberDeconvolutionImageFilter_NormalizeOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LandweberDeconvolutionImageFilter_NormalizeOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LandweberDeconvolutionImageFilter_NormalizeOff`, 'returnType') = 'void'
attr(`LandweberDeconvolutionImageFilter_NormalizeOff`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter')
class(`LandweberDeconvolutionImageFilter_NormalizeOff`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_NormalizeOff'))

# Start of LandweberDeconvolutionImageFilter_GetNormalize

`LandweberDeconvolutionImageFilter_GetNormalize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LandweberDeconvolutionImageFilter_GetNormalize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LandweberDeconvolutionImageFilter_GetNormalize`, 'returnType') = 'logical'
attr(`LandweberDeconvolutionImageFilter_GetNormalize`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter')
class(`LandweberDeconvolutionImageFilter_GetNormalize`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_GetNormalize'))

# Start of LandweberDeconvolutionImageFilter_ZERO_PAD_get

`LandweberDeconvolutionImageFilter_ZERO_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_LandweberDeconvolutionImageFilter_ZERO_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__LandweberDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`LandweberDeconvolutionImageFilter_ZERO_PAD_get`, 'returnType') = 'character'
class(`LandweberDeconvolutionImageFilter_ZERO_PAD_get`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_ZERO_PAD_get'))

# Start of LandweberDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get

`LandweberDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_LandweberDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__LandweberDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`LandweberDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`, 'returnType') = 'character'
class(`LandweberDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get'))

# Start of LandweberDeconvolutionImageFilter_PERIODIC_PAD_get

`LandweberDeconvolutionImageFilter_PERIODIC_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_LandweberDeconvolutionImageFilter_PERIODIC_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__LandweberDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`LandweberDeconvolutionImageFilter_PERIODIC_PAD_get`, 'returnType') = 'character'
class(`LandweberDeconvolutionImageFilter_PERIODIC_PAD_get`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_PERIODIC_PAD_get'))

# Start of LandweberDeconvolutionImageFilter_SetBoundaryCondition

`LandweberDeconvolutionImageFilter_SetBoundaryCondition` = function(self, BoundaryCondition)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BoundaryCondition = enumToInteger(BoundaryCondition, "_itk__simple__LandweberDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(BoundaryCondition) > 1) {
    warning("using only the first element of BoundaryCondition");
  };
  
  ;ans = .Call('R_swig_LandweberDeconvolutionImageFilter_SetBoundaryCondition', self, BoundaryCondition, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LandweberDeconvolutionImageFilter_SetBoundaryCondition`, 'returnType') = 'void'
attr(`LandweberDeconvolutionImageFilter_SetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter', 'character')
class(`LandweberDeconvolutionImageFilter_SetBoundaryCondition`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_SetBoundaryCondition'))

# Start of LandweberDeconvolutionImageFilter_GetBoundaryCondition

`LandweberDeconvolutionImageFilter_GetBoundaryCondition` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LandweberDeconvolutionImageFilter_GetBoundaryCondition', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__LandweberDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`LandweberDeconvolutionImageFilter_GetBoundaryCondition`, 'returnType') = 'character'
attr(`LandweberDeconvolutionImageFilter_GetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter')
class(`LandweberDeconvolutionImageFilter_GetBoundaryCondition`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_GetBoundaryCondition'))

# Start of LandweberDeconvolutionImageFilter_SAME_get

`LandweberDeconvolutionImageFilter_SAME_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_LandweberDeconvolutionImageFilter_SAME_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__LandweberDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`LandweberDeconvolutionImageFilter_SAME_get`, 'returnType') = 'character'
class(`LandweberDeconvolutionImageFilter_SAME_get`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_SAME_get'))

# Start of LandweberDeconvolutionImageFilter_VALID_get

`LandweberDeconvolutionImageFilter_VALID_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_LandweberDeconvolutionImageFilter_VALID_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__LandweberDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`LandweberDeconvolutionImageFilter_VALID_get`, 'returnType') = 'character'
class(`LandweberDeconvolutionImageFilter_VALID_get`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_VALID_get'))

# Start of LandweberDeconvolutionImageFilter_SetOutputRegionMode

`LandweberDeconvolutionImageFilter_SetOutputRegionMode` = function(self, OutputRegionMode)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputRegionMode = enumToInteger(OutputRegionMode, "_itk__simple__LandweberDeconvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(OutputRegionMode) > 1) {
    warning("using only the first element of OutputRegionMode");
  };
  
  ;ans = .Call('R_swig_LandweberDeconvolutionImageFilter_SetOutputRegionMode', self, OutputRegionMode, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LandweberDeconvolutionImageFilter_SetOutputRegionMode`, 'returnType') = 'void'
attr(`LandweberDeconvolutionImageFilter_SetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter', 'character')
class(`LandweberDeconvolutionImageFilter_SetOutputRegionMode`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_SetOutputRegionMode'))

# Start of LandweberDeconvolutionImageFilter_GetOutputRegionMode

`LandweberDeconvolutionImageFilter_GetOutputRegionMode` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LandweberDeconvolutionImageFilter_GetOutputRegionMode', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__LandweberDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`LandweberDeconvolutionImageFilter_GetOutputRegionMode`, 'returnType') = 'character'
attr(`LandweberDeconvolutionImageFilter_GetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter')
class(`LandweberDeconvolutionImageFilter_GetOutputRegionMode`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_GetOutputRegionMode'))

# Start of LandweberDeconvolutionImageFilter_GetName

`LandweberDeconvolutionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LandweberDeconvolutionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LandweberDeconvolutionImageFilter_GetName`, 'returnType') = 'character'
attr(`LandweberDeconvolutionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter')
class(`LandweberDeconvolutionImageFilter_GetName`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_GetName'))

# Start of LandweberDeconvolutionImageFilter_ToString

`LandweberDeconvolutionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LandweberDeconvolutionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LandweberDeconvolutionImageFilter_ToString`, 'returnType') = 'character'
attr(`LandweberDeconvolutionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter')
class(`LandweberDeconvolutionImageFilter_ToString`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_ToString'))

# Start of LandweberDeconvolutionImageFilter_Execute

`LandweberDeconvolutionImageFilter_Execute` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_LandweberDeconvolutionImageFilter_Execute', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LandweberDeconvolutionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LandweberDeconvolutionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LandweberDeconvolutionImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LandweberDeconvolutionImageFilter_Execute`) = c("SWIGFunction", class('LandweberDeconvolutionImageFilter_Execute'))

# Start of accessor method for itk::simple::LandweberDeconvolutionImageFilter
setMethod('$', '_p_itk__simple__LandweberDeconvolutionImageFilter', function(x, name)

{
  accessorFuns = list('SetAlpha' = LandweberDeconvolutionImageFilter_SetAlpha, 'GetAlpha' = LandweberDeconvolutionImageFilter_GetAlpha, 'SetNumberOfIterations' = LandweberDeconvolutionImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = LandweberDeconvolutionImageFilter_GetNumberOfIterations, 'SetNormalize' = LandweberDeconvolutionImageFilter_SetNormalize, 'NormalizeOn' = LandweberDeconvolutionImageFilter_NormalizeOn, 'NormalizeOff' = LandweberDeconvolutionImageFilter_NormalizeOff, 'GetNormalize' = LandweberDeconvolutionImageFilter_GetNormalize, 'SetBoundaryCondition' = LandweberDeconvolutionImageFilter_SetBoundaryCondition, 'GetBoundaryCondition' = LandweberDeconvolutionImageFilter_GetBoundaryCondition, 'SetOutputRegionMode' = LandweberDeconvolutionImageFilter_SetOutputRegionMode, 'GetOutputRegionMode' = LandweberDeconvolutionImageFilter_GetOutputRegionMode, 'GetName' = LandweberDeconvolutionImageFilter_GetName, 'ToString' = LandweberDeconvolutionImageFilter_ToString, 'Execute' = LandweberDeconvolutionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LandweberDeconvolutionImageFilter
# Start of LandweberDeconvolution

`LandweberDeconvolution__SWIG_0` = function(image1, image2, alpha, numberOfIterations, normalize, boundaryCondition, outputRegionMode, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__LandweberDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  outputRegionMode = enumToInteger(outputRegionMode, "_itk__simple__LandweberDeconvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(outputRegionMode) > 1) {
    warning("using only the first element of outputRegionMode");
  };
  
  ;ans = .Call('R_swig_LandweberDeconvolution__SWIG_0', image1, image2, alpha, numberOfIterations, normalize, boundaryCondition, outputRegionMode, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LandweberDeconvolution__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LandweberDeconvolution__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'logical', 'character', 'character')
class(`LandweberDeconvolution__SWIG_0`) = c("SWIGFunction", class('LandweberDeconvolution__SWIG_0'))

# Start of LandweberDeconvolution

`LandweberDeconvolution__SWIG_1` = function(image1, image2, alpha, numberOfIterations, normalize, boundaryCondition, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__LandweberDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  ;ans = .Call('R_swig_LandweberDeconvolution__SWIG_1', image1, image2, alpha, numberOfIterations, normalize, boundaryCondition, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LandweberDeconvolution__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LandweberDeconvolution__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'logical', 'character')
class(`LandweberDeconvolution__SWIG_1`) = c("SWIGFunction", class('LandweberDeconvolution__SWIG_1'))

# Start of LandweberDeconvolution

`LandweberDeconvolution__SWIG_2` = function(image1, image2, alpha, numberOfIterations, normalize, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  normalize = as.logical(normalize);
  ;ans = .Call('R_swig_LandweberDeconvolution__SWIG_2', image1, image2, alpha, numberOfIterations, normalize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LandweberDeconvolution__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LandweberDeconvolution__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'logical')
class(`LandweberDeconvolution__SWIG_2`) = c("SWIGFunction", class('LandweberDeconvolution__SWIG_2'))

# Start of LandweberDeconvolution

`LandweberDeconvolution__SWIG_3` = function(image1, image2, alpha, numberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_LandweberDeconvolution__SWIG_3', image1, image2, alpha, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LandweberDeconvolution__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`LandweberDeconvolution__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer')
class(`LandweberDeconvolution__SWIG_3`) = c("SWIGFunction", class('LandweberDeconvolution__SWIG_3'))

# Start of LandweberDeconvolution

`LandweberDeconvolution__SWIG_4` = function(image1, image2, alpha, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  ;ans = .Call('R_swig_LandweberDeconvolution__SWIG_4', image1, image2, alpha, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LandweberDeconvolution__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`LandweberDeconvolution__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`LandweberDeconvolution__SWIG_4`) = c("SWIGFunction", class('LandweberDeconvolution__SWIG_4'))

# Start of LandweberDeconvolution

`LandweberDeconvolution__SWIG_5` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_LandweberDeconvolution__SWIG_5', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LandweberDeconvolution__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`LandweberDeconvolution__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LandweberDeconvolution__SWIG_5`) = c("SWIGFunction", class('LandweberDeconvolution__SWIG_5'))

`LandweberDeconvolution` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- LandweberDeconvolution__SWIG_5; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- LandweberDeconvolution__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- LandweberDeconvolution__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- LandweberDeconvolution__SWIG_2; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.character(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- LandweberDeconvolution__SWIG_1; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.character(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.character(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- LandweberDeconvolution__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LandweberDeconvolution with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LaplacianImageFilter

`delete_LaplacianImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LaplacianImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LaplacianImageFilter`, 'returnType') = 'void'
attr(`delete_LaplacianImageFilter`, "inputTypes") = c('_p_itk__simple__LaplacianImageFilter')
class(`delete_LaplacianImageFilter`) = c("SWIGFunction", class('delete_LaplacianImageFilter'))

# Start of new_LaplacianImageFilter

`LaplacianImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LaplacianImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LaplacianImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LaplacianImageFilter);
  ans
  
}

attr(`LaplacianImageFilter`, 'returnType') = '_p_itk__simple__LaplacianImageFilter'
class(`LaplacianImageFilter`) = c("SWIGFunction", class('LaplacianImageFilter'))

# Start of LaplacianImageFilter_SetUseImageSpacing

`LaplacianImageFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_LaplacianImageFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianImageFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`LaplacianImageFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__LaplacianImageFilter', 'logical')
class(`LaplacianImageFilter_SetUseImageSpacing`) = c("SWIGFunction", class('LaplacianImageFilter_SetUseImageSpacing'))

# Start of LaplacianImageFilter_UseImageSpacingOn

`LaplacianImageFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LaplacianImageFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianImageFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`LaplacianImageFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__LaplacianImageFilter')
class(`LaplacianImageFilter_UseImageSpacingOn`) = c("SWIGFunction", class('LaplacianImageFilter_UseImageSpacingOn'))

# Start of LaplacianImageFilter_UseImageSpacingOff

`LaplacianImageFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LaplacianImageFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianImageFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`LaplacianImageFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__LaplacianImageFilter')
class(`LaplacianImageFilter_UseImageSpacingOff`) = c("SWIGFunction", class('LaplacianImageFilter_UseImageSpacingOff'))

# Start of LaplacianImageFilter_GetUseImageSpacing

`LaplacianImageFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianImageFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianImageFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`LaplacianImageFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__LaplacianImageFilter')
class(`LaplacianImageFilter_GetUseImageSpacing`) = c("SWIGFunction", class('LaplacianImageFilter_GetUseImageSpacing'))

# Start of LaplacianImageFilter_GetName

`LaplacianImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianImageFilter_GetName`, 'returnType') = 'character'
attr(`LaplacianImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LaplacianImageFilter')
class(`LaplacianImageFilter_GetName`) = c("SWIGFunction", class('LaplacianImageFilter_GetName'))

# Start of LaplacianImageFilter_ToString

`LaplacianImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianImageFilter_ToString`, 'returnType') = 'character'
attr(`LaplacianImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LaplacianImageFilter')
class(`LaplacianImageFilter_ToString`) = c("SWIGFunction", class('LaplacianImageFilter_ToString'))

# Start of LaplacianImageFilter_Execute

`LaplacianImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LaplacianImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LaplacianImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LaplacianImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LaplacianImageFilter', '_p_itk__simple__Image')
class(`LaplacianImageFilter_Execute`) = c("SWIGFunction", class('LaplacianImageFilter_Execute'))

# Start of accessor method for itk::simple::LaplacianImageFilter
setMethod('$', '_p_itk__simple__LaplacianImageFilter', function(x, name)

{
  accessorFuns = list('SetUseImageSpacing' = LaplacianImageFilter_SetUseImageSpacing, 'UseImageSpacingOn' = LaplacianImageFilter_UseImageSpacingOn, 'UseImageSpacingOff' = LaplacianImageFilter_UseImageSpacingOff, 'GetUseImageSpacing' = LaplacianImageFilter_GetUseImageSpacing, 'GetName' = LaplacianImageFilter_GetName, 'ToString' = LaplacianImageFilter_ToString, 'Execute' = LaplacianImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LaplacianImageFilter
# Start of Laplacian

`Laplacian__SWIG_0` = function(image1, useImageSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  useImageSpacing = as.logical(useImageSpacing);
  ;ans = .Call('R_swig_Laplacian__SWIG_0', image1, useImageSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Laplacian__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Laplacian__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`Laplacian__SWIG_0`) = c("SWIGFunction", class('Laplacian__SWIG_0'))

# Start of Laplacian

`Laplacian__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Laplacian__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Laplacian__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Laplacian__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`Laplacian__SWIG_1`) = c("SWIGFunction", class('Laplacian__SWIG_1'))

`Laplacian` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Laplacian__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Laplacian__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Laplacian with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LaplacianRecursiveGaussianImageFilter

`delete_LaplacianRecursiveGaussianImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LaplacianRecursiveGaussianImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LaplacianRecursiveGaussianImageFilter`, 'returnType') = 'void'
attr(`delete_LaplacianRecursiveGaussianImageFilter`, "inputTypes") = c('_p_itk__simple__LaplacianRecursiveGaussianImageFilter')
class(`delete_LaplacianRecursiveGaussianImageFilter`) = c("SWIGFunction", class('delete_LaplacianRecursiveGaussianImageFilter'))

# Start of new_LaplacianRecursiveGaussianImageFilter

`LaplacianRecursiveGaussianImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LaplacianRecursiveGaussianImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LaplacianRecursiveGaussianImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LaplacianRecursiveGaussianImageFilter);
  ans
  
}

attr(`LaplacianRecursiveGaussianImageFilter`, 'returnType') = '_p_itk__simple__LaplacianRecursiveGaussianImageFilter'
class(`LaplacianRecursiveGaussianImageFilter`) = c("SWIGFunction", class('LaplacianRecursiveGaussianImageFilter'))

# Start of LaplacianRecursiveGaussianImageFilter_SetSigma

`LaplacianRecursiveGaussianImageFilter_SetSigma` = function(self, Sigma)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LaplacianRecursiveGaussianImageFilter_SetSigma', self, Sigma, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianRecursiveGaussianImageFilter_SetSigma`, 'returnType') = 'void'
attr(`LaplacianRecursiveGaussianImageFilter_SetSigma`, "inputTypes") = c('_p_itk__simple__LaplacianRecursiveGaussianImageFilter', 'numeric')
class(`LaplacianRecursiveGaussianImageFilter_SetSigma`) = c("SWIGFunction", class('LaplacianRecursiveGaussianImageFilter_SetSigma'))

# Start of LaplacianRecursiveGaussianImageFilter_GetSigma

`LaplacianRecursiveGaussianImageFilter_GetSigma` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianRecursiveGaussianImageFilter_GetSigma', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianRecursiveGaussianImageFilter_GetSigma`, 'returnType') = 'numeric'
attr(`LaplacianRecursiveGaussianImageFilter_GetSigma`, "inputTypes") = c('_p_itk__simple__LaplacianRecursiveGaussianImageFilter')
class(`LaplacianRecursiveGaussianImageFilter_GetSigma`) = c("SWIGFunction", class('LaplacianRecursiveGaussianImageFilter_GetSigma'))

# Start of LaplacianRecursiveGaussianImageFilter_SetNormalizeAcrossScale

`LaplacianRecursiveGaussianImageFilter_SetNormalizeAcrossScale` = function(self, NormalizeAcrossScale)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NormalizeAcrossScale = as.logical(NormalizeAcrossScale);
  ;ans = .Call('R_swig_LaplacianRecursiveGaussianImageFilter_SetNormalizeAcrossScale', self, NormalizeAcrossScale, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianRecursiveGaussianImageFilter_SetNormalizeAcrossScale`, 'returnType') = 'void'
attr(`LaplacianRecursiveGaussianImageFilter_SetNormalizeAcrossScale`, "inputTypes") = c('_p_itk__simple__LaplacianRecursiveGaussianImageFilter', 'logical')
class(`LaplacianRecursiveGaussianImageFilter_SetNormalizeAcrossScale`) = c("SWIGFunction", class('LaplacianRecursiveGaussianImageFilter_SetNormalizeAcrossScale'))

# Start of LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOn

`LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOn`, 'returnType') = 'void'
attr(`LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOn`, "inputTypes") = c('_p_itk__simple__LaplacianRecursiveGaussianImageFilter')
class(`LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOn`) = c("SWIGFunction", class('LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOn'))

# Start of LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOff

`LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOff`, 'returnType') = 'void'
attr(`LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOff`, "inputTypes") = c('_p_itk__simple__LaplacianRecursiveGaussianImageFilter')
class(`LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOff`) = c("SWIGFunction", class('LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOff'))

# Start of LaplacianRecursiveGaussianImageFilter_GetNormalizeAcrossScale

`LaplacianRecursiveGaussianImageFilter_GetNormalizeAcrossScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianRecursiveGaussianImageFilter_GetNormalizeAcrossScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianRecursiveGaussianImageFilter_GetNormalizeAcrossScale`, 'returnType') = 'logical'
attr(`LaplacianRecursiveGaussianImageFilter_GetNormalizeAcrossScale`, "inputTypes") = c('_p_itk__simple__LaplacianRecursiveGaussianImageFilter')
class(`LaplacianRecursiveGaussianImageFilter_GetNormalizeAcrossScale`) = c("SWIGFunction", class('LaplacianRecursiveGaussianImageFilter_GetNormalizeAcrossScale'))

# Start of LaplacianRecursiveGaussianImageFilter_GetName

`LaplacianRecursiveGaussianImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianRecursiveGaussianImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianRecursiveGaussianImageFilter_GetName`, 'returnType') = 'character'
attr(`LaplacianRecursiveGaussianImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LaplacianRecursiveGaussianImageFilter')
class(`LaplacianRecursiveGaussianImageFilter_GetName`) = c("SWIGFunction", class('LaplacianRecursiveGaussianImageFilter_GetName'))

# Start of LaplacianRecursiveGaussianImageFilter_ToString

`LaplacianRecursiveGaussianImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianRecursiveGaussianImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianRecursiveGaussianImageFilter_ToString`, 'returnType') = 'character'
attr(`LaplacianRecursiveGaussianImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LaplacianRecursiveGaussianImageFilter')
class(`LaplacianRecursiveGaussianImageFilter_ToString`) = c("SWIGFunction", class('LaplacianRecursiveGaussianImageFilter_ToString'))

# Start of LaplacianRecursiveGaussianImageFilter_Execute

`LaplacianRecursiveGaussianImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LaplacianRecursiveGaussianImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LaplacianRecursiveGaussianImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LaplacianRecursiveGaussianImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LaplacianRecursiveGaussianImageFilter', '_p_itk__simple__Image')
class(`LaplacianRecursiveGaussianImageFilter_Execute`) = c("SWIGFunction", class('LaplacianRecursiveGaussianImageFilter_Execute'))

# Start of accessor method for itk::simple::LaplacianRecursiveGaussianImageFilter
setMethod('$', '_p_itk__simple__LaplacianRecursiveGaussianImageFilter', function(x, name)

{
  accessorFuns = list('SetSigma' = LaplacianRecursiveGaussianImageFilter_SetSigma, 'GetSigma' = LaplacianRecursiveGaussianImageFilter_GetSigma, 'SetNormalizeAcrossScale' = LaplacianRecursiveGaussianImageFilter_SetNormalizeAcrossScale, 'NormalizeAcrossScaleOn' = LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOn, 'NormalizeAcrossScaleOff' = LaplacianRecursiveGaussianImageFilter_NormalizeAcrossScaleOff, 'GetNormalizeAcrossScale' = LaplacianRecursiveGaussianImageFilter_GetNormalizeAcrossScale, 'GetName' = LaplacianRecursiveGaussianImageFilter_GetName, 'ToString' = LaplacianRecursiveGaussianImageFilter_ToString, 'Execute' = LaplacianRecursiveGaussianImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LaplacianRecursiveGaussianImageFilter
# Start of LaplacianRecursiveGaussian

`LaplacianRecursiveGaussian__SWIG_0` = function(image1, sigma, normalizeAcrossScale, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  normalizeAcrossScale = as.logical(normalizeAcrossScale);
  ;ans = .Call('R_swig_LaplacianRecursiveGaussian__SWIG_0', image1, sigma, normalizeAcrossScale, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LaplacianRecursiveGaussian__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LaplacianRecursiveGaussian__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`LaplacianRecursiveGaussian__SWIG_0`) = c("SWIGFunction", class('LaplacianRecursiveGaussian__SWIG_0'))

# Start of LaplacianRecursiveGaussian

`LaplacianRecursiveGaussian__SWIG_1` = function(image1, sigma, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_LaplacianRecursiveGaussian__SWIG_1', image1, sigma, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LaplacianRecursiveGaussian__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LaplacianRecursiveGaussian__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`LaplacianRecursiveGaussian__SWIG_1`) = c("SWIGFunction", class('LaplacianRecursiveGaussian__SWIG_1'))

# Start of LaplacianRecursiveGaussian

`LaplacianRecursiveGaussian__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LaplacianRecursiveGaussian__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LaplacianRecursiveGaussian__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LaplacianRecursiveGaussian__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`LaplacianRecursiveGaussian__SWIG_2`) = c("SWIGFunction", class('LaplacianRecursiveGaussian__SWIG_2'))

`LaplacianRecursiveGaussian` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- LaplacianRecursiveGaussian__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- LaplacianRecursiveGaussian__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- LaplacianRecursiveGaussian__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LaplacianRecursiveGaussian with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LaplacianSegmentationLevelSetImageFilter

`delete_LaplacianSegmentationLevelSetImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LaplacianSegmentationLevelSetImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LaplacianSegmentationLevelSetImageFilter`, 'returnType') = 'void'
attr(`delete_LaplacianSegmentationLevelSetImageFilter`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter')
class(`delete_LaplacianSegmentationLevelSetImageFilter`) = c("SWIGFunction", class('delete_LaplacianSegmentationLevelSetImageFilter'))

# Start of new_LaplacianSegmentationLevelSetImageFilter

`LaplacianSegmentationLevelSetImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LaplacianSegmentationLevelSetImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LaplacianSegmentationLevelSetImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LaplacianSegmentationLevelSetImageFilter);
  ans
  
}

attr(`LaplacianSegmentationLevelSetImageFilter`, 'returnType') = '_p_itk__simple__LaplacianSegmentationLevelSetImageFilter'
class(`LaplacianSegmentationLevelSetImageFilter`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter'))

# Start of LaplacianSegmentationLevelSetImageFilter_SetMaximumRMSError

`LaplacianSegmentationLevelSetImageFilter_SetMaximumRMSError` = function(self, MaximumRMSError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LaplacianSegmentationLevelSetImageFilter_SetMaximumRMSError', self, MaximumRMSError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_SetMaximumRMSError`, 'returnType') = 'void'
attr(`LaplacianSegmentationLevelSetImageFilter_SetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter', 'numeric')
class(`LaplacianSegmentationLevelSetImageFilter_SetMaximumRMSError`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_SetMaximumRMSError'))

# Start of LaplacianSegmentationLevelSetImageFilter_GetMaximumRMSError

`LaplacianSegmentationLevelSetImageFilter_GetMaximumRMSError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianSegmentationLevelSetImageFilter_GetMaximumRMSError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_GetMaximumRMSError`, 'returnType') = 'numeric'
attr(`LaplacianSegmentationLevelSetImageFilter_GetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter')
class(`LaplacianSegmentationLevelSetImageFilter_GetMaximumRMSError`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_GetMaximumRMSError'))

# Start of LaplacianSegmentationLevelSetImageFilter_SetPropagationScaling

`LaplacianSegmentationLevelSetImageFilter_SetPropagationScaling` = function(self, PropagationScaling)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LaplacianSegmentationLevelSetImageFilter_SetPropagationScaling', self, PropagationScaling, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_SetPropagationScaling`, 'returnType') = 'void'
attr(`LaplacianSegmentationLevelSetImageFilter_SetPropagationScaling`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter', 'numeric')
class(`LaplacianSegmentationLevelSetImageFilter_SetPropagationScaling`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_SetPropagationScaling'))

# Start of LaplacianSegmentationLevelSetImageFilter_GetPropagationScaling

`LaplacianSegmentationLevelSetImageFilter_GetPropagationScaling` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianSegmentationLevelSetImageFilter_GetPropagationScaling', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_GetPropagationScaling`, 'returnType') = 'numeric'
attr(`LaplacianSegmentationLevelSetImageFilter_GetPropagationScaling`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter')
class(`LaplacianSegmentationLevelSetImageFilter_GetPropagationScaling`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_GetPropagationScaling'))

# Start of LaplacianSegmentationLevelSetImageFilter_SetCurvatureScaling

`LaplacianSegmentationLevelSetImageFilter_SetCurvatureScaling` = function(self, CurvatureScaling)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LaplacianSegmentationLevelSetImageFilter_SetCurvatureScaling', self, CurvatureScaling, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_SetCurvatureScaling`, 'returnType') = 'void'
attr(`LaplacianSegmentationLevelSetImageFilter_SetCurvatureScaling`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter', 'numeric')
class(`LaplacianSegmentationLevelSetImageFilter_SetCurvatureScaling`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_SetCurvatureScaling'))

# Start of LaplacianSegmentationLevelSetImageFilter_GetCurvatureScaling

`LaplacianSegmentationLevelSetImageFilter_GetCurvatureScaling` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianSegmentationLevelSetImageFilter_GetCurvatureScaling', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_GetCurvatureScaling`, 'returnType') = 'numeric'
attr(`LaplacianSegmentationLevelSetImageFilter_GetCurvatureScaling`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter')
class(`LaplacianSegmentationLevelSetImageFilter_GetCurvatureScaling`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_GetCurvatureScaling'))

# Start of LaplacianSegmentationLevelSetImageFilter_SetNumberOfIterations

`LaplacianSegmentationLevelSetImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_LaplacianSegmentationLevelSetImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`LaplacianSegmentationLevelSetImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter', 'integer')
class(`LaplacianSegmentationLevelSetImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_SetNumberOfIterations'))

# Start of LaplacianSegmentationLevelSetImageFilter_GetNumberOfIterations

`LaplacianSegmentationLevelSetImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianSegmentationLevelSetImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`LaplacianSegmentationLevelSetImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter')
class(`LaplacianSegmentationLevelSetImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_GetNumberOfIterations'))

# Start of LaplacianSegmentationLevelSetImageFilter_SetReverseExpansionDirection

`LaplacianSegmentationLevelSetImageFilter_SetReverseExpansionDirection` = function(self, ReverseExpansionDirection)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ReverseExpansionDirection = as.logical(ReverseExpansionDirection);
  ;ans = .Call('R_swig_LaplacianSegmentationLevelSetImageFilter_SetReverseExpansionDirection', self, ReverseExpansionDirection, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_SetReverseExpansionDirection`, 'returnType') = 'void'
attr(`LaplacianSegmentationLevelSetImageFilter_SetReverseExpansionDirection`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter', 'logical')
class(`LaplacianSegmentationLevelSetImageFilter_SetReverseExpansionDirection`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_SetReverseExpansionDirection'))

# Start of LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn

`LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn`, 'returnType') = 'void'
attr(`LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter')
class(`LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn'))

# Start of LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff

`LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff`, 'returnType') = 'void'
attr(`LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter')
class(`LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff'))

# Start of LaplacianSegmentationLevelSetImageFilter_GetReverseExpansionDirection

`LaplacianSegmentationLevelSetImageFilter_GetReverseExpansionDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianSegmentationLevelSetImageFilter_GetReverseExpansionDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_GetReverseExpansionDirection`, 'returnType') = 'logical'
attr(`LaplacianSegmentationLevelSetImageFilter_GetReverseExpansionDirection`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter')
class(`LaplacianSegmentationLevelSetImageFilter_GetReverseExpansionDirection`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_GetReverseExpansionDirection'))

# Start of LaplacianSegmentationLevelSetImageFilter_GetElapsedIterations

`LaplacianSegmentationLevelSetImageFilter_GetElapsedIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianSegmentationLevelSetImageFilter_GetElapsedIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_GetElapsedIterations`, 'returnType') = 'integer'
attr(`LaplacianSegmentationLevelSetImageFilter_GetElapsedIterations`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter')
class(`LaplacianSegmentationLevelSetImageFilter_GetElapsedIterations`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_GetElapsedIterations'))

# Start of LaplacianSegmentationLevelSetImageFilter_GetRMSChange

`LaplacianSegmentationLevelSetImageFilter_GetRMSChange` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianSegmentationLevelSetImageFilter_GetRMSChange', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_GetRMSChange`, 'returnType') = 'numeric'
attr(`LaplacianSegmentationLevelSetImageFilter_GetRMSChange`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter')
class(`LaplacianSegmentationLevelSetImageFilter_GetRMSChange`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_GetRMSChange'))

# Start of LaplacianSegmentationLevelSetImageFilter_GetName

`LaplacianSegmentationLevelSetImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianSegmentationLevelSetImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_GetName`, 'returnType') = 'character'
attr(`LaplacianSegmentationLevelSetImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter')
class(`LaplacianSegmentationLevelSetImageFilter_GetName`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_GetName'))

# Start of LaplacianSegmentationLevelSetImageFilter_ToString

`LaplacianSegmentationLevelSetImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianSegmentationLevelSetImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_ToString`, 'returnType') = 'character'
attr(`LaplacianSegmentationLevelSetImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter')
class(`LaplacianSegmentationLevelSetImageFilter_ToString`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_ToString'))

# Start of LaplacianSegmentationLevelSetImageFilter_Execute

`LaplacianSegmentationLevelSetImageFilter_Execute` = function(self, initialImage, featureImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_LaplacianSegmentationLevelSetImageFilter_Execute', self, initialImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LaplacianSegmentationLevelSetImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LaplacianSegmentationLevelSetImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LaplacianSegmentationLevelSetImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LaplacianSegmentationLevelSetImageFilter_Execute`) = c("SWIGFunction", class('LaplacianSegmentationLevelSetImageFilter_Execute'))

# Start of accessor method for itk::simple::LaplacianSegmentationLevelSetImageFilter
setMethod('$', '_p_itk__simple__LaplacianSegmentationLevelSetImageFilter', function(x, name)

{
  accessorFuns = list('SetMaximumRMSError' = LaplacianSegmentationLevelSetImageFilter_SetMaximumRMSError, 'GetMaximumRMSError' = LaplacianSegmentationLevelSetImageFilter_GetMaximumRMSError, 'SetPropagationScaling' = LaplacianSegmentationLevelSetImageFilter_SetPropagationScaling, 'GetPropagationScaling' = LaplacianSegmentationLevelSetImageFilter_GetPropagationScaling, 'SetCurvatureScaling' = LaplacianSegmentationLevelSetImageFilter_SetCurvatureScaling, 'GetCurvatureScaling' = LaplacianSegmentationLevelSetImageFilter_GetCurvatureScaling, 'SetNumberOfIterations' = LaplacianSegmentationLevelSetImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = LaplacianSegmentationLevelSetImageFilter_GetNumberOfIterations, 'SetReverseExpansionDirection' = LaplacianSegmentationLevelSetImageFilter_SetReverseExpansionDirection, 'ReverseExpansionDirectionOn' = LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn, 'ReverseExpansionDirectionOff' = LaplacianSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff, 'GetReverseExpansionDirection' = LaplacianSegmentationLevelSetImageFilter_GetReverseExpansionDirection, 'GetElapsedIterations' = LaplacianSegmentationLevelSetImageFilter_GetElapsedIterations, 'GetRMSChange' = LaplacianSegmentationLevelSetImageFilter_GetRMSChange, 'GetName' = LaplacianSegmentationLevelSetImageFilter_GetName, 'ToString' = LaplacianSegmentationLevelSetImageFilter_ToString, 'Execute' = LaplacianSegmentationLevelSetImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LaplacianSegmentationLevelSetImageFilter
# Start of LaplacianSegmentationLevelSet

`LaplacianSegmentationLevelSet__SWIG_0` = function(initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, numberOfIterations, reverseExpansionDirection, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  reverseExpansionDirection = as.logical(reverseExpansionDirection);
  ;ans = .Call('R_swig_LaplacianSegmentationLevelSet__SWIG_0', initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, numberOfIterations, reverseExpansionDirection, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LaplacianSegmentationLevelSet__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LaplacianSegmentationLevelSet__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'integer', 'logical')
class(`LaplacianSegmentationLevelSet__SWIG_0`) = c("SWIGFunction", class('LaplacianSegmentationLevelSet__SWIG_0'))

# Start of LaplacianSegmentationLevelSet

`LaplacianSegmentationLevelSet__SWIG_1` = function(initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, numberOfIterations, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_LaplacianSegmentationLevelSet__SWIG_1', initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LaplacianSegmentationLevelSet__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LaplacianSegmentationLevelSet__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'integer')
class(`LaplacianSegmentationLevelSet__SWIG_1`) = c("SWIGFunction", class('LaplacianSegmentationLevelSet__SWIG_1'))

# Start of LaplacianSegmentationLevelSet

`LaplacianSegmentationLevelSet__SWIG_2` = function(initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  ;ans = .Call('R_swig_LaplacianSegmentationLevelSet__SWIG_2', initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LaplacianSegmentationLevelSet__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LaplacianSegmentationLevelSet__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric')
class(`LaplacianSegmentationLevelSet__SWIG_2`) = c("SWIGFunction", class('LaplacianSegmentationLevelSet__SWIG_2'))

# Start of LaplacianSegmentationLevelSet

`LaplacianSegmentationLevelSet__SWIG_3` = function(initialImage, featureImage, maximumRMSError, propagationScaling, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  ;ans = .Call('R_swig_LaplacianSegmentationLevelSet__SWIG_3', initialImage, featureImage, maximumRMSError, propagationScaling, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LaplacianSegmentationLevelSet__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`LaplacianSegmentationLevelSet__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`LaplacianSegmentationLevelSet__SWIG_3`) = c("SWIGFunction", class('LaplacianSegmentationLevelSet__SWIG_3'))

# Start of LaplacianSegmentationLevelSet

`LaplacianSegmentationLevelSet__SWIG_4` = function(initialImage, featureImage, maximumRMSError, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  ;ans = .Call('R_swig_LaplacianSegmentationLevelSet__SWIG_4', initialImage, featureImage, maximumRMSError, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LaplacianSegmentationLevelSet__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`LaplacianSegmentationLevelSet__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`LaplacianSegmentationLevelSet__SWIG_4`) = c("SWIGFunction", class('LaplacianSegmentationLevelSet__SWIG_4'))

# Start of LaplacianSegmentationLevelSet

`LaplacianSegmentationLevelSet__SWIG_5` = function(initialImage, featureImage, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_LaplacianSegmentationLevelSet__SWIG_5', initialImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LaplacianSegmentationLevelSet__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`LaplacianSegmentationLevelSet__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LaplacianSegmentationLevelSet__SWIG_5`) = c("SWIGFunction", class('LaplacianSegmentationLevelSet__SWIG_5'))

`LaplacianSegmentationLevelSet` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- LaplacianSegmentationLevelSet__SWIG_5; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- LaplacianSegmentationLevelSet__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- LaplacianSegmentationLevelSet__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- LaplacianSegmentationLevelSet__SWIG_2; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- LaplacianSegmentationLevelSet__SWIG_1; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.logical(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- LaplacianSegmentationLevelSet__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LaplacianSegmentationLevelSet with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LaplacianSharpeningImageFilter

`delete_LaplacianSharpeningImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LaplacianSharpeningImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LaplacianSharpeningImageFilter`, 'returnType') = 'void'
attr(`delete_LaplacianSharpeningImageFilter`, "inputTypes") = c('_p_itk__simple__LaplacianSharpeningImageFilter')
class(`delete_LaplacianSharpeningImageFilter`) = c("SWIGFunction", class('delete_LaplacianSharpeningImageFilter'))

# Start of new_LaplacianSharpeningImageFilter

`LaplacianSharpeningImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LaplacianSharpeningImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LaplacianSharpeningImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LaplacianSharpeningImageFilter);
  ans
  
}

attr(`LaplacianSharpeningImageFilter`, 'returnType') = '_p_itk__simple__LaplacianSharpeningImageFilter'
class(`LaplacianSharpeningImageFilter`) = c("SWIGFunction", class('LaplacianSharpeningImageFilter'))

# Start of LaplacianSharpeningImageFilter_SetUseImageSpacing

`LaplacianSharpeningImageFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_LaplacianSharpeningImageFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianSharpeningImageFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`LaplacianSharpeningImageFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__LaplacianSharpeningImageFilter', 'logical')
class(`LaplacianSharpeningImageFilter_SetUseImageSpacing`) = c("SWIGFunction", class('LaplacianSharpeningImageFilter_SetUseImageSpacing'))

# Start of LaplacianSharpeningImageFilter_UseImageSpacingOn

`LaplacianSharpeningImageFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LaplacianSharpeningImageFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianSharpeningImageFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`LaplacianSharpeningImageFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__LaplacianSharpeningImageFilter')
class(`LaplacianSharpeningImageFilter_UseImageSpacingOn`) = c("SWIGFunction", class('LaplacianSharpeningImageFilter_UseImageSpacingOn'))

# Start of LaplacianSharpeningImageFilter_UseImageSpacingOff

`LaplacianSharpeningImageFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LaplacianSharpeningImageFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LaplacianSharpeningImageFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`LaplacianSharpeningImageFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__LaplacianSharpeningImageFilter')
class(`LaplacianSharpeningImageFilter_UseImageSpacingOff`) = c("SWIGFunction", class('LaplacianSharpeningImageFilter_UseImageSpacingOff'))

# Start of LaplacianSharpeningImageFilter_GetUseImageSpacing

`LaplacianSharpeningImageFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianSharpeningImageFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianSharpeningImageFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`LaplacianSharpeningImageFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__LaplacianSharpeningImageFilter')
class(`LaplacianSharpeningImageFilter_GetUseImageSpacing`) = c("SWIGFunction", class('LaplacianSharpeningImageFilter_GetUseImageSpacing'))

# Start of LaplacianSharpeningImageFilter_GetName

`LaplacianSharpeningImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianSharpeningImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianSharpeningImageFilter_GetName`, 'returnType') = 'character'
attr(`LaplacianSharpeningImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LaplacianSharpeningImageFilter')
class(`LaplacianSharpeningImageFilter_GetName`) = c("SWIGFunction", class('LaplacianSharpeningImageFilter_GetName'))

# Start of LaplacianSharpeningImageFilter_ToString

`LaplacianSharpeningImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LaplacianSharpeningImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LaplacianSharpeningImageFilter_ToString`, 'returnType') = 'character'
attr(`LaplacianSharpeningImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LaplacianSharpeningImageFilter')
class(`LaplacianSharpeningImageFilter_ToString`) = c("SWIGFunction", class('LaplacianSharpeningImageFilter_ToString'))

# Start of LaplacianSharpeningImageFilter_Execute

`LaplacianSharpeningImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LaplacianSharpeningImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LaplacianSharpeningImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LaplacianSharpeningImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LaplacianSharpeningImageFilter', '_p_itk__simple__Image')
class(`LaplacianSharpeningImageFilter_Execute`) = c("SWIGFunction", class('LaplacianSharpeningImageFilter_Execute'))

# Start of accessor method for itk::simple::LaplacianSharpeningImageFilter
setMethod('$', '_p_itk__simple__LaplacianSharpeningImageFilter', function(x, name)

{
  accessorFuns = list('SetUseImageSpacing' = LaplacianSharpeningImageFilter_SetUseImageSpacing, 'UseImageSpacingOn' = LaplacianSharpeningImageFilter_UseImageSpacingOn, 'UseImageSpacingOff' = LaplacianSharpeningImageFilter_UseImageSpacingOff, 'GetUseImageSpacing' = LaplacianSharpeningImageFilter_GetUseImageSpacing, 'GetName' = LaplacianSharpeningImageFilter_GetName, 'ToString' = LaplacianSharpeningImageFilter_ToString, 'Execute' = LaplacianSharpeningImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LaplacianSharpeningImageFilter
# Start of LaplacianSharpening

`LaplacianSharpening__SWIG_0` = function(image1, useImageSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  useImageSpacing = as.logical(useImageSpacing);
  ;ans = .Call('R_swig_LaplacianSharpening__SWIG_0', image1, useImageSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LaplacianSharpening__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LaplacianSharpening__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`LaplacianSharpening__SWIG_0`) = c("SWIGFunction", class('LaplacianSharpening__SWIG_0'))

# Start of LaplacianSharpening

`LaplacianSharpening__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LaplacianSharpening__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LaplacianSharpening__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LaplacianSharpening__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`LaplacianSharpening__SWIG_1`) = c("SWIGFunction", class('LaplacianSharpening__SWIG_1'))

`LaplacianSharpening` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- LaplacianSharpening__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- LaplacianSharpening__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LaplacianSharpening with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LessEqualImageFilter

`delete_LessEqualImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LessEqualImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LessEqualImageFilter`, 'returnType') = 'void'
attr(`delete_LessEqualImageFilter`, "inputTypes") = c('_p_itk__simple__LessEqualImageFilter')
class(`delete_LessEqualImageFilter`) = c("SWIGFunction", class('delete_LessEqualImageFilter'))

# Start of new_LessEqualImageFilter

`LessEqualImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LessEqualImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LessEqualImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LessEqualImageFilter);
  ans
  
}

attr(`LessEqualImageFilter`, 'returnType') = '_p_itk__simple__LessEqualImageFilter'
class(`LessEqualImageFilter`) = c("SWIGFunction", class('LessEqualImageFilter'))

# Start of LessEqualImageFilter_SetBackgroundValue

`LessEqualImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BackgroundValue = as.integer(BackgroundValue);
  
  if(length(BackgroundValue) > 1) {
    warning("using only the first element of BackgroundValue");
  };
  
  ;ans = .Call('R_swig_LessEqualImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LessEqualImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`LessEqualImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LessEqualImageFilter', 'integer')
class(`LessEqualImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('LessEqualImageFilter_SetBackgroundValue'))

# Start of LessEqualImageFilter_GetBackgroundValue

`LessEqualImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LessEqualImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LessEqualImageFilter_GetBackgroundValue`, 'returnType') = 'integer'
attr(`LessEqualImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LessEqualImageFilter')
class(`LessEqualImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('LessEqualImageFilter_GetBackgroundValue'))

# Start of LessEqualImageFilter_SetForegroundValue

`LessEqualImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ForegroundValue = as.integer(ForegroundValue);
  
  if(length(ForegroundValue) > 1) {
    warning("using only the first element of ForegroundValue");
  };
  
  ;ans = .Call('R_swig_LessEqualImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LessEqualImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`LessEqualImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__LessEqualImageFilter', 'integer')
class(`LessEqualImageFilter_SetForegroundValue`) = c("SWIGFunction", class('LessEqualImageFilter_SetForegroundValue'))

# Start of LessEqualImageFilter_GetForegroundValue

`LessEqualImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LessEqualImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LessEqualImageFilter_GetForegroundValue`, 'returnType') = 'integer'
attr(`LessEqualImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__LessEqualImageFilter')
class(`LessEqualImageFilter_GetForegroundValue`) = c("SWIGFunction", class('LessEqualImageFilter_GetForegroundValue'))

# Start of LessEqualImageFilter_GetName

`LessEqualImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LessEqualImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LessEqualImageFilter_GetName`, 'returnType') = 'character'
attr(`LessEqualImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LessEqualImageFilter')
class(`LessEqualImageFilter_GetName`) = c("SWIGFunction", class('LessEqualImageFilter_GetName'))

# Start of LessEqualImageFilter_ToString

`LessEqualImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LessEqualImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LessEqualImageFilter_ToString`, 'returnType') = 'character'
attr(`LessEqualImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LessEqualImageFilter')
class(`LessEqualImageFilter_ToString`) = c("SWIGFunction", class('LessEqualImageFilter_ToString'))

# Start of LessEqualImageFilter_Execute

`LessEqualImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_LessEqualImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessEqualImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LessEqualImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__LessEqualImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LessEqualImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('LessEqualImageFilter_Execute__SWIG_0'))

# Start of LessEqualImageFilter_Execute

`LessEqualImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_LessEqualImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessEqualImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LessEqualImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__LessEqualImageFilter', '_p_itk__simple__Image', 'numeric')
class(`LessEqualImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('LessEqualImageFilter_Execute__SWIG_1'))

# Start of LessEqualImageFilter_Execute

`LessEqualImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_LessEqualImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessEqualImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LessEqualImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__LessEqualImageFilter', 'numeric', '_p_itk__simple__Image')
class(`LessEqualImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('LessEqualImageFilter_Execute__SWIG_2'))

# Start of LessEqualImageFilter_Execute

`LessEqualImageFilter_Execute__SWIG_3` = function(self, image1, constant, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_LessEqualImageFilter_Execute__SWIG_3', self, image1, constant, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessEqualImageFilter_Execute__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`LessEqualImageFilter_Execute__SWIG_3`, "inputTypes") = c('_p_itk__simple__LessEqualImageFilter', '_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`LessEqualImageFilter_Execute__SWIG_3`) = c("SWIGFunction", class('LessEqualImageFilter_Execute__SWIG_3'))

# Start of LessEqualImageFilter_Execute

`LessEqualImageFilter_Execute__SWIG_4` = function(self, constant, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_LessEqualImageFilter_Execute__SWIG_4', self, constant, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessEqualImageFilter_Execute__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`LessEqualImageFilter_Execute__SWIG_4`, "inputTypes") = c('_p_itk__simple__LessEqualImageFilter', 'numeric', '_p_itk__simple__Image', 'integer', 'integer')
class(`LessEqualImageFilter_Execute__SWIG_4`) = c("SWIGFunction", class('LessEqualImageFilter_Execute__SWIG_4'))

`LessEqualImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__LessEqualImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- LessEqualImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__LessEqualImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- LessEqualImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__LessEqualImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- LessEqualImageFilter_Execute__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__LessEqualImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- LessEqualImageFilter_Execute__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__LessEqualImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- LessEqualImageFilter_Execute__SWIG_4; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LessEqualImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::LessEqualImageFilter
setMethod('$', '_p_itk__simple__LessEqualImageFilter', function(x, name)

{
  accessorFuns = list('SetBackgroundValue' = LessEqualImageFilter_SetBackgroundValue, 'GetBackgroundValue' = LessEqualImageFilter_GetBackgroundValue, 'SetForegroundValue' = LessEqualImageFilter_SetForegroundValue, 'GetForegroundValue' = LessEqualImageFilter_GetForegroundValue, 'GetName' = LessEqualImageFilter_GetName, 'ToString' = LessEqualImageFilter_ToString, 'Execute' = LessEqualImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LessEqualImageFilter
# Start of LessEqual

`LessEqual__SWIG_0` = function(image1, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_LessEqual__SWIG_0', image1, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessEqual__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LessEqual__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`LessEqual__SWIG_0`) = c("SWIGFunction", class('LessEqual__SWIG_0'))

# Start of LessEqual

`LessEqual__SWIG_1` = function(image1, image2, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_LessEqual__SWIG_1', image1, image2, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessEqual__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LessEqual__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`LessEqual__SWIG_1`) = c("SWIGFunction", class('LessEqual__SWIG_1'))

# Start of LessEqual

`LessEqual__SWIG_2` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_LessEqual__SWIG_2', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessEqual__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LessEqual__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LessEqual__SWIG_2`) = c("SWIGFunction", class('LessEqual__SWIG_2'))

# Start of LessEqual

`LessEqual__SWIG_3` = function(image1, constant, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_LessEqual__SWIG_3', image1, constant, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessEqual__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`LessEqual__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`LessEqual__SWIG_3`) = c("SWIGFunction", class('LessEqual__SWIG_3'))

# Start of LessEqual

`LessEqual__SWIG_4` = function(image1, constant, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_LessEqual__SWIG_4', image1, constant, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessEqual__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`LessEqual__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`LessEqual__SWIG_4`) = c("SWIGFunction", class('LessEqual__SWIG_4'))

# Start of LessEqual

`LessEqual__SWIG_5` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_LessEqual__SWIG_5', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessEqual__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`LessEqual__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`LessEqual__SWIG_5`) = c("SWIGFunction", class('LessEqual__SWIG_5'))

# Start of LessEqual

`LessEqual__SWIG_6` = function(constant, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_LessEqual__SWIG_6', constant, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessEqual__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`LessEqual__SWIG_6`, "inputTypes") = c('numeric', '_p_itk__simple__Image', 'integer', 'integer')
class(`LessEqual__SWIG_6`) = c("SWIGFunction", class('LessEqual__SWIG_6'))

# Start of LessEqual

`LessEqual__SWIG_7` = function(constant, image2, backgroundValue, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_LessEqual__SWIG_7', constant, image2, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessEqual__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`LessEqual__SWIG_7`, "inputTypes") = c('numeric', '_p_itk__simple__Image', 'integer')
class(`LessEqual__SWIG_7`) = c("SWIGFunction", class('LessEqual__SWIG_7'))

# Start of LessEqual

`LessEqual__SWIG_8` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_LessEqual__SWIG_8', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessEqual__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`LessEqual__SWIG_8`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`LessEqual__SWIG_8`) = c("SWIGFunction", class('LessEqual__SWIG_8'))

`LessEqual` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- LessEqual__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- LessEqual__SWIG_5; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- LessEqual__SWIG_8; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- LessEqual__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- LessEqual__SWIG_4; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- LessEqual__SWIG_7; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- LessEqual__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- LessEqual__SWIG_3; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- LessEqual__SWIG_6; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LessEqual with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LessImageFilter

`delete_LessImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LessImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LessImageFilter`, 'returnType') = 'void'
attr(`delete_LessImageFilter`, "inputTypes") = c('_p_itk__simple__LessImageFilter')
class(`delete_LessImageFilter`) = c("SWIGFunction", class('delete_LessImageFilter'))

# Start of new_LessImageFilter

`LessImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LessImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LessImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LessImageFilter);
  ans
  
}

attr(`LessImageFilter`, 'returnType') = '_p_itk__simple__LessImageFilter'
class(`LessImageFilter`) = c("SWIGFunction", class('LessImageFilter'))

# Start of LessImageFilter_SetBackgroundValue

`LessImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BackgroundValue = as.integer(BackgroundValue);
  
  if(length(BackgroundValue) > 1) {
    warning("using only the first element of BackgroundValue");
  };
  
  ;ans = .Call('R_swig_LessImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LessImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`LessImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LessImageFilter', 'integer')
class(`LessImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('LessImageFilter_SetBackgroundValue'))

# Start of LessImageFilter_GetBackgroundValue

`LessImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LessImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LessImageFilter_GetBackgroundValue`, 'returnType') = 'integer'
attr(`LessImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__LessImageFilter')
class(`LessImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('LessImageFilter_GetBackgroundValue'))

# Start of LessImageFilter_SetForegroundValue

`LessImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ForegroundValue = as.integer(ForegroundValue);
  
  if(length(ForegroundValue) > 1) {
    warning("using only the first element of ForegroundValue");
  };
  
  ;ans = .Call('R_swig_LessImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LessImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`LessImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__LessImageFilter', 'integer')
class(`LessImageFilter_SetForegroundValue`) = c("SWIGFunction", class('LessImageFilter_SetForegroundValue'))

# Start of LessImageFilter_GetForegroundValue

`LessImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LessImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LessImageFilter_GetForegroundValue`, 'returnType') = 'integer'
attr(`LessImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__LessImageFilter')
class(`LessImageFilter_GetForegroundValue`) = c("SWIGFunction", class('LessImageFilter_GetForegroundValue'))

# Start of LessImageFilter_GetName

`LessImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LessImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LessImageFilter_GetName`, 'returnType') = 'character'
attr(`LessImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LessImageFilter')
class(`LessImageFilter_GetName`) = c("SWIGFunction", class('LessImageFilter_GetName'))

# Start of LessImageFilter_ToString

`LessImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LessImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LessImageFilter_ToString`, 'returnType') = 'character'
attr(`LessImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LessImageFilter')
class(`LessImageFilter_ToString`) = c("SWIGFunction", class('LessImageFilter_ToString'))

# Start of LessImageFilter_Execute

`LessImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_LessImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LessImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__LessImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LessImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('LessImageFilter_Execute__SWIG_0'))

# Start of LessImageFilter_Execute

`LessImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_LessImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LessImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__LessImageFilter', '_p_itk__simple__Image', 'numeric')
class(`LessImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('LessImageFilter_Execute__SWIG_1'))

# Start of LessImageFilter_Execute

`LessImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_LessImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LessImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__LessImageFilter', 'numeric', '_p_itk__simple__Image')
class(`LessImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('LessImageFilter_Execute__SWIG_2'))

# Start of LessImageFilter_Execute

`LessImageFilter_Execute__SWIG_3` = function(self, image1, constant, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_LessImageFilter_Execute__SWIG_3', self, image1, constant, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessImageFilter_Execute__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`LessImageFilter_Execute__SWIG_3`, "inputTypes") = c('_p_itk__simple__LessImageFilter', '_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`LessImageFilter_Execute__SWIG_3`) = c("SWIGFunction", class('LessImageFilter_Execute__SWIG_3'))

# Start of LessImageFilter_Execute

`LessImageFilter_Execute__SWIG_4` = function(self, constant, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_LessImageFilter_Execute__SWIG_4', self, constant, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LessImageFilter_Execute__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`LessImageFilter_Execute__SWIG_4`, "inputTypes") = c('_p_itk__simple__LessImageFilter', 'numeric', '_p_itk__simple__Image', 'integer', 'integer')
class(`LessImageFilter_Execute__SWIG_4`) = c("SWIGFunction", class('LessImageFilter_Execute__SWIG_4'))

`LessImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__LessImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- LessImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__LessImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- LessImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__LessImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- LessImageFilter_Execute__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__LessImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- LessImageFilter_Execute__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__LessImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- LessImageFilter_Execute__SWIG_4; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LessImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::LessImageFilter
setMethod('$', '_p_itk__simple__LessImageFilter', function(x, name)

{
  accessorFuns = list('SetBackgroundValue' = LessImageFilter_SetBackgroundValue, 'GetBackgroundValue' = LessImageFilter_GetBackgroundValue, 'SetForegroundValue' = LessImageFilter_SetForegroundValue, 'GetForegroundValue' = LessImageFilter_GetForegroundValue, 'GetName' = LessImageFilter_GetName, 'ToString' = LessImageFilter_ToString, 'Execute' = LessImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LessImageFilter
# Start of Less

`Less__SWIG_0` = function(image1, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_Less__SWIG_0', image1, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Less__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Less__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`Less__SWIG_0`) = c("SWIGFunction", class('Less__SWIG_0'))

# Start of Less

`Less__SWIG_1` = function(image1, image2, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_Less__SWIG_1', image1, image2, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Less__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Less__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`Less__SWIG_1`) = c("SWIGFunction", class('Less__SWIG_1'))

# Start of Less

`Less__SWIG_2` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Less__SWIG_2', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Less__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Less__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Less__SWIG_2`) = c("SWIGFunction", class('Less__SWIG_2'))

# Start of Less

`Less__SWIG_3` = function(image1, constant, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_Less__SWIG_3', image1, constant, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Less__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Less__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`Less__SWIG_3`) = c("SWIGFunction", class('Less__SWIG_3'))

# Start of Less

`Less__SWIG_4` = function(image1, constant, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_Less__SWIG_4', image1, constant, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Less__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`Less__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`Less__SWIG_4`) = c("SWIGFunction", class('Less__SWIG_4'))

# Start of Less

`Less__SWIG_5` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Less__SWIG_5', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Less__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`Less__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`Less__SWIG_5`) = c("SWIGFunction", class('Less__SWIG_5'))

# Start of Less

`Less__SWIG_6` = function(constant, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_Less__SWIG_6', constant, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Less__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`Less__SWIG_6`, "inputTypes") = c('numeric', '_p_itk__simple__Image', 'integer', 'integer')
class(`Less__SWIG_6`) = c("SWIGFunction", class('Less__SWIG_6'))

# Start of Less

`Less__SWIG_7` = function(constant, image2, backgroundValue, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_Less__SWIG_7', constant, image2, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Less__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`Less__SWIG_7`, "inputTypes") = c('numeric', '_p_itk__simple__Image', 'integer')
class(`Less__SWIG_7`) = c("SWIGFunction", class('Less__SWIG_7'))

# Start of Less

`Less__SWIG_8` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Less__SWIG_8', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Less__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`Less__SWIG_8`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`Less__SWIG_8`) = c("SWIGFunction", class('Less__SWIG_8'))

`Less` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Less__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Less__SWIG_5; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Less__SWIG_8; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- Less__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- Less__SWIG_4; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- Less__SWIG_7; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- Less__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- Less__SWIG_3; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- Less__SWIG_6; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Less with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_LevelSetMotionRegistrationFilter

`delete_LevelSetMotionRegistrationFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LevelSetMotionRegistrationFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LevelSetMotionRegistrationFilter`, 'returnType') = 'void'
attr(`delete_LevelSetMotionRegistrationFilter`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`delete_LevelSetMotionRegistrationFilter`) = c("SWIGFunction", class('delete_LevelSetMotionRegistrationFilter'))

# Start of new_LevelSetMotionRegistrationFilter

`LevelSetMotionRegistrationFilter` = function()
{
  ;ans = .Call('R_swig_new_LevelSetMotionRegistrationFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LevelSetMotionRegistrationFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LevelSetMotionRegistrationFilter);
  ans
  
}

attr(`LevelSetMotionRegistrationFilter`, 'returnType') = '_p_itk__simple__LevelSetMotionRegistrationFilter'
class(`LevelSetMotionRegistrationFilter`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter'))

# Start of LevelSetMotionRegistrationFilter_SetGradientSmoothingStandardDeviations

`LevelSetMotionRegistrationFilter_SetGradientSmoothingStandardDeviations` = function(self, GradientSmoothingStandardDeviations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SetGradientSmoothingStandardDeviations', self, GradientSmoothingStandardDeviations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SetGradientSmoothingStandardDeviations`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SetGradientSmoothingStandardDeviations`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', 'numeric')
class(`LevelSetMotionRegistrationFilter_SetGradientSmoothingStandardDeviations`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SetGradientSmoothingStandardDeviations'))

# Start of LevelSetMotionRegistrationFilter_GetGradientSmoothingStandardDeviations

`LevelSetMotionRegistrationFilter_GetGradientSmoothingStandardDeviations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetGradientSmoothingStandardDeviations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetGradientSmoothingStandardDeviations`, 'returnType') = 'numeric'
attr(`LevelSetMotionRegistrationFilter_GetGradientSmoothingStandardDeviations`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetGradientSmoothingStandardDeviations`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetGradientSmoothingStandardDeviations'))

# Start of LevelSetMotionRegistrationFilter_SetNumberOfIterations

`LevelSetMotionRegistrationFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', 'integer')
class(`LevelSetMotionRegistrationFilter_SetNumberOfIterations`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SetNumberOfIterations'))

# Start of LevelSetMotionRegistrationFilter_GetNumberOfIterations

`LevelSetMotionRegistrationFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`LevelSetMotionRegistrationFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetNumberOfIterations`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetNumberOfIterations'))

# Start of LevelSetMotionRegistrationFilter_SetMaximumRMSError

`LevelSetMotionRegistrationFilter_SetMaximumRMSError` = function(self, MaximumRMSError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SetMaximumRMSError', self, MaximumRMSError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SetMaximumRMSError`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', 'numeric')
class(`LevelSetMotionRegistrationFilter_SetMaximumRMSError`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SetMaximumRMSError'))

# Start of LevelSetMotionRegistrationFilter_GetMaximumRMSError

`LevelSetMotionRegistrationFilter_GetMaximumRMSError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetMaximumRMSError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetMaximumRMSError`, 'returnType') = 'numeric'
attr(`LevelSetMotionRegistrationFilter_GetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetMaximumRMSError`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetMaximumRMSError'))

# Start of LevelSetMotionRegistrationFilter_SetStandardDeviations

`LevelSetMotionRegistrationFilter_SetStandardDeviations__SWIG_0` = function(self, StandardDeviations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  StandardDeviations = as.numeric(StandardDeviations);
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SetStandardDeviations__SWIG_0', self, StandardDeviations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SetStandardDeviations__SWIG_0`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SetStandardDeviations__SWIG_0`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', 'numeric')
class(`LevelSetMotionRegistrationFilter_SetStandardDeviations__SWIG_0`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SetStandardDeviations__SWIG_0'))

# Start of LevelSetMotionRegistrationFilter_SetStandardDeviations

`LevelSetMotionRegistrationFilter_SetStandardDeviations__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SetStandardDeviations__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SetStandardDeviations__SWIG_1`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SetStandardDeviations__SWIG_1`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', 'numeric')
class(`LevelSetMotionRegistrationFilter_SetStandardDeviations__SWIG_1`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SetStandardDeviations__SWIG_1'))

`LevelSetMotionRegistrationFilter_SetStandardDeviations` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__LevelSetMotionRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- LevelSetMotionRegistrationFilter_SetStandardDeviations__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__LevelSetMotionRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- LevelSetMotionRegistrationFilter_SetStandardDeviations__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LevelSetMotionRegistrationFilter_SetStandardDeviations with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of LevelSetMotionRegistrationFilter_GetStandardDeviations

`LevelSetMotionRegistrationFilter_GetStandardDeviations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetStandardDeviations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetStandardDeviations`, 'returnType') = 'numeric'
attr(`LevelSetMotionRegistrationFilter_GetStandardDeviations`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetStandardDeviations`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetStandardDeviations'))

# Start of LevelSetMotionRegistrationFilter_SetSmoothDisplacementField

`LevelSetMotionRegistrationFilter_SetSmoothDisplacementField` = function(self, SmoothDisplacementField)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SmoothDisplacementField = as.logical(SmoothDisplacementField);
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SetSmoothDisplacementField', self, SmoothDisplacementField, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SetSmoothDisplacementField`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SetSmoothDisplacementField`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', 'logical')
class(`LevelSetMotionRegistrationFilter_SetSmoothDisplacementField`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SetSmoothDisplacementField'))

# Start of LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOn

`LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOn`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOn`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOn`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOn'))

# Start of LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOff

`LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOff`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOff`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOff`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOff'))

# Start of LevelSetMotionRegistrationFilter_GetSmoothDisplacementField

`LevelSetMotionRegistrationFilter_GetSmoothDisplacementField` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetSmoothDisplacementField', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetSmoothDisplacementField`, 'returnType') = 'logical'
attr(`LevelSetMotionRegistrationFilter_GetSmoothDisplacementField`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetSmoothDisplacementField`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetSmoothDisplacementField'))

# Start of LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations

`LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0` = function(self, UpdateFieldStandardDeviations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UpdateFieldStandardDeviations = as.numeric(UpdateFieldStandardDeviations);
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0', self, UpdateFieldStandardDeviations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', 'numeric')
class(`LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0'))

# Start of LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations

`LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', 'numeric')
class(`LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1'))

`LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__LevelSetMotionRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__LevelSetMotionRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of LevelSetMotionRegistrationFilter_GetUpdateFieldStandardDeviations

`LevelSetMotionRegistrationFilter_GetUpdateFieldStandardDeviations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetUpdateFieldStandardDeviations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetUpdateFieldStandardDeviations`, 'returnType') = 'numeric'
attr(`LevelSetMotionRegistrationFilter_GetUpdateFieldStandardDeviations`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetUpdateFieldStandardDeviations`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetUpdateFieldStandardDeviations'))

# Start of LevelSetMotionRegistrationFilter_SetSmoothUpdateField

`LevelSetMotionRegistrationFilter_SetSmoothUpdateField` = function(self, SmoothUpdateField)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SmoothUpdateField = as.logical(SmoothUpdateField);
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SetSmoothUpdateField', self, SmoothUpdateField, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SetSmoothUpdateField`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SetSmoothUpdateField`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', 'logical')
class(`LevelSetMotionRegistrationFilter_SetSmoothUpdateField`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SetSmoothUpdateField'))

# Start of LevelSetMotionRegistrationFilter_SmoothUpdateFieldOn

`LevelSetMotionRegistrationFilter_SmoothUpdateFieldOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SmoothUpdateFieldOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SmoothUpdateFieldOn`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SmoothUpdateFieldOn`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_SmoothUpdateFieldOn`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SmoothUpdateFieldOn'))

# Start of LevelSetMotionRegistrationFilter_SmoothUpdateFieldOff

`LevelSetMotionRegistrationFilter_SmoothUpdateFieldOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SmoothUpdateFieldOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SmoothUpdateFieldOff`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SmoothUpdateFieldOff`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_SmoothUpdateFieldOff`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SmoothUpdateFieldOff'))

# Start of LevelSetMotionRegistrationFilter_GetSmoothUpdateField

`LevelSetMotionRegistrationFilter_GetSmoothUpdateField` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetSmoothUpdateField', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetSmoothUpdateField`, 'returnType') = 'logical'
attr(`LevelSetMotionRegistrationFilter_GetSmoothUpdateField`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetSmoothUpdateField`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetSmoothUpdateField'))

# Start of LevelSetMotionRegistrationFilter_SetMaximumKernelWidth

`LevelSetMotionRegistrationFilter_SetMaximumKernelWidth` = function(self, MaximumKernelWidth)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaximumKernelWidth = as.integer(MaximumKernelWidth);
  
  if(length(MaximumKernelWidth) > 1) {
    warning("using only the first element of MaximumKernelWidth");
  };
  
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SetMaximumKernelWidth', self, MaximumKernelWidth, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SetMaximumKernelWidth`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SetMaximumKernelWidth`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', 'integer')
class(`LevelSetMotionRegistrationFilter_SetMaximumKernelWidth`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SetMaximumKernelWidth'))

# Start of LevelSetMotionRegistrationFilter_GetMaximumKernelWidth

`LevelSetMotionRegistrationFilter_GetMaximumKernelWidth` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetMaximumKernelWidth', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetMaximumKernelWidth`, 'returnType') = 'integer'
attr(`LevelSetMotionRegistrationFilter_GetMaximumKernelWidth`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetMaximumKernelWidth`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetMaximumKernelWidth'))

# Start of LevelSetMotionRegistrationFilter_SetMaximumError

`LevelSetMotionRegistrationFilter_SetMaximumError` = function(self, MaximumError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SetMaximumError', self, MaximumError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SetMaximumError`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SetMaximumError`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', 'numeric')
class(`LevelSetMotionRegistrationFilter_SetMaximumError`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SetMaximumError'))

# Start of LevelSetMotionRegistrationFilter_GetMaximumError

`LevelSetMotionRegistrationFilter_GetMaximumError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetMaximumError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetMaximumError`, 'returnType') = 'numeric'
attr(`LevelSetMotionRegistrationFilter_GetMaximumError`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetMaximumError`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetMaximumError'))

# Start of LevelSetMotionRegistrationFilter_SetAlpha

`LevelSetMotionRegistrationFilter_SetAlpha` = function(self, Alpha)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SetAlpha', self, Alpha, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SetAlpha`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SetAlpha`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', 'numeric')
class(`LevelSetMotionRegistrationFilter_SetAlpha`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SetAlpha'))

# Start of LevelSetMotionRegistrationFilter_GetAlpha

`LevelSetMotionRegistrationFilter_GetAlpha` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetAlpha', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetAlpha`, 'returnType') = 'numeric'
attr(`LevelSetMotionRegistrationFilter_GetAlpha`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetAlpha`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetAlpha'))

# Start of LevelSetMotionRegistrationFilter_SetIntensityDifferenceThreshold

`LevelSetMotionRegistrationFilter_SetIntensityDifferenceThreshold` = function(self, IntensityDifferenceThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SetIntensityDifferenceThreshold', self, IntensityDifferenceThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SetIntensityDifferenceThreshold`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SetIntensityDifferenceThreshold`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', 'numeric')
class(`LevelSetMotionRegistrationFilter_SetIntensityDifferenceThreshold`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SetIntensityDifferenceThreshold'))

# Start of LevelSetMotionRegistrationFilter_GetIntensityDifferenceThreshold

`LevelSetMotionRegistrationFilter_GetIntensityDifferenceThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetIntensityDifferenceThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetIntensityDifferenceThreshold`, 'returnType') = 'numeric'
attr(`LevelSetMotionRegistrationFilter_GetIntensityDifferenceThreshold`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetIntensityDifferenceThreshold`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetIntensityDifferenceThreshold'))

# Start of LevelSetMotionRegistrationFilter_SetGradientMagnitudeThreshold

`LevelSetMotionRegistrationFilter_SetGradientMagnitudeThreshold` = function(self, GradientMagnitudeThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SetGradientMagnitudeThreshold', self, GradientMagnitudeThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SetGradientMagnitudeThreshold`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SetGradientMagnitudeThreshold`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', 'numeric')
class(`LevelSetMotionRegistrationFilter_SetGradientMagnitudeThreshold`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SetGradientMagnitudeThreshold'))

# Start of LevelSetMotionRegistrationFilter_GetGradientMagnitudeThreshold

`LevelSetMotionRegistrationFilter_GetGradientMagnitudeThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetGradientMagnitudeThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetGradientMagnitudeThreshold`, 'returnType') = 'numeric'
attr(`LevelSetMotionRegistrationFilter_GetGradientMagnitudeThreshold`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetGradientMagnitudeThreshold`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetGradientMagnitudeThreshold'))

# Start of LevelSetMotionRegistrationFilter_SetUseImageSpacing

`LevelSetMotionRegistrationFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', 'logical')
class(`LevelSetMotionRegistrationFilter_SetUseImageSpacing`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_SetUseImageSpacing'))

# Start of LevelSetMotionRegistrationFilter_UseImageSpacingOn

`LevelSetMotionRegistrationFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_UseImageSpacingOn`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_UseImageSpacingOn'))

# Start of LevelSetMotionRegistrationFilter_UseImageSpacingOff

`LevelSetMotionRegistrationFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`LevelSetMotionRegistrationFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_UseImageSpacingOff`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_UseImageSpacingOff'))

# Start of LevelSetMotionRegistrationFilter_GetUseImageSpacing

`LevelSetMotionRegistrationFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`LevelSetMotionRegistrationFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetUseImageSpacing`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetUseImageSpacing'))

# Start of LevelSetMotionRegistrationFilter_GetElapsedIterations

`LevelSetMotionRegistrationFilter_GetElapsedIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetElapsedIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetElapsedIterations`, 'returnType') = 'integer'
attr(`LevelSetMotionRegistrationFilter_GetElapsedIterations`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetElapsedIterations`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetElapsedIterations'))

# Start of LevelSetMotionRegistrationFilter_GetRMSChange

`LevelSetMotionRegistrationFilter_GetRMSChange` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetRMSChange', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetRMSChange`, 'returnType') = 'numeric'
attr(`LevelSetMotionRegistrationFilter_GetRMSChange`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetRMSChange`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetRMSChange'))

# Start of LevelSetMotionRegistrationFilter_GetMetric

`LevelSetMotionRegistrationFilter_GetMetric` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetMetric', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetMetric`, 'returnType') = 'numeric'
attr(`LevelSetMotionRegistrationFilter_GetMetric`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetMetric`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetMetric'))

# Start of LevelSetMotionRegistrationFilter_GetName

`LevelSetMotionRegistrationFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_GetName`, 'returnType') = 'character'
attr(`LevelSetMotionRegistrationFilter_GetName`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_GetName`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_GetName'))

# Start of LevelSetMotionRegistrationFilter_ToString

`LevelSetMotionRegistrationFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LevelSetMotionRegistrationFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LevelSetMotionRegistrationFilter_ToString`, 'returnType') = 'character'
attr(`LevelSetMotionRegistrationFilter_ToString`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter')
class(`LevelSetMotionRegistrationFilter_ToString`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_ToString'))

# Start of LevelSetMotionRegistrationFilter_Execute

`LevelSetMotionRegistrationFilter_Execute__SWIG_0` = function(self, fixedImage, movingImage, initialDisplacementField, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  if (inherits(initialDisplacementField, "ExternalReference")) initialDisplacementField = slot(initialDisplacementField,"ref"); 
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_Execute__SWIG_0', self, fixedImage, movingImage, initialDisplacementField, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LevelSetMotionRegistrationFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LevelSetMotionRegistrationFilter_Execute__SWIG_0`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_Execute__SWIG_0'))

# Start of LevelSetMotionRegistrationFilter_Execute

`LevelSetMotionRegistrationFilter_Execute__SWIG_1` = function(self, fixedImage, movingImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  ;ans = .Call('R_swig_LevelSetMotionRegistrationFilter_Execute__SWIG_1', self, fixedImage, movingImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LevelSetMotionRegistrationFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LevelSetMotionRegistrationFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__LevelSetMotionRegistrationFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LevelSetMotionRegistrationFilter_Execute__SWIG_1`) = c("SWIGFunction", class('LevelSetMotionRegistrationFilter_Execute__SWIG_1'))

`LevelSetMotionRegistrationFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__LevelSetMotionRegistrationFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- LevelSetMotionRegistrationFilter_Execute__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__LevelSetMotionRegistrationFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- LevelSetMotionRegistrationFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LevelSetMotionRegistrationFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::LevelSetMotionRegistrationFilter
setMethod('$', '_p_itk__simple__LevelSetMotionRegistrationFilter', function(x, name)

{
  accessorFuns = list('SetGradientSmoothingStandardDeviations' = LevelSetMotionRegistrationFilter_SetGradientSmoothingStandardDeviations, 'GetGradientSmoothingStandardDeviations' = LevelSetMotionRegistrationFilter_GetGradientSmoothingStandardDeviations, 'SetNumberOfIterations' = LevelSetMotionRegistrationFilter_SetNumberOfIterations, 'GetNumberOfIterations' = LevelSetMotionRegistrationFilter_GetNumberOfIterations, 'SetMaximumRMSError' = LevelSetMotionRegistrationFilter_SetMaximumRMSError, 'GetMaximumRMSError' = LevelSetMotionRegistrationFilter_GetMaximumRMSError, 'SetStandardDeviations' = LevelSetMotionRegistrationFilter_SetStandardDeviations, 'GetStandardDeviations' = LevelSetMotionRegistrationFilter_GetStandardDeviations, 'SetSmoothDisplacementField' = LevelSetMotionRegistrationFilter_SetSmoothDisplacementField, 'SmoothDisplacementFieldOn' = LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOn, 'SmoothDisplacementFieldOff' = LevelSetMotionRegistrationFilter_SmoothDisplacementFieldOff, 'GetSmoothDisplacementField' = LevelSetMotionRegistrationFilter_GetSmoothDisplacementField, 'SetUpdateFieldStandardDeviations' = LevelSetMotionRegistrationFilter_SetUpdateFieldStandardDeviations, 'GetUpdateFieldStandardDeviations' = LevelSetMotionRegistrationFilter_GetUpdateFieldStandardDeviations, 'SetSmoothUpdateField' = LevelSetMotionRegistrationFilter_SetSmoothUpdateField, 'SmoothUpdateFieldOn' = LevelSetMotionRegistrationFilter_SmoothUpdateFieldOn, 'SmoothUpdateFieldOff' = LevelSetMotionRegistrationFilter_SmoothUpdateFieldOff, 'GetSmoothUpdateField' = LevelSetMotionRegistrationFilter_GetSmoothUpdateField, 'SetMaximumKernelWidth' = LevelSetMotionRegistrationFilter_SetMaximumKernelWidth, 'GetMaximumKernelWidth' = LevelSetMotionRegistrationFilter_GetMaximumKernelWidth, 'SetMaximumError' = LevelSetMotionRegistrationFilter_SetMaximumError, 'GetMaximumError' = LevelSetMotionRegistrationFilter_GetMaximumError, 'SetAlpha' = LevelSetMotionRegistrationFilter_SetAlpha, 'GetAlpha' = LevelSetMotionRegistrationFilter_GetAlpha, 'SetIntensityDifferenceThreshold' = LevelSetMotionRegistrationFilter_SetIntensityDifferenceThreshold, 'GetIntensityDifferenceThreshold' = LevelSetMotionRegistrationFilter_GetIntensityDifferenceThreshold, 'SetGradientMagnitudeThreshold' = LevelSetMotionRegistrationFilter_SetGradientMagnitudeThreshold, 'GetGradientMagnitudeThreshold' = LevelSetMotionRegistrationFilter_GetGradientMagnitudeThreshold, 'SetUseImageSpacing' = LevelSetMotionRegistrationFilter_SetUseImageSpacing, 'UseImageSpacingOn' = LevelSetMotionRegistrationFilter_UseImageSpacingOn, 'UseImageSpacingOff' = LevelSetMotionRegistrationFilter_UseImageSpacingOff, 'GetUseImageSpacing' = LevelSetMotionRegistrationFilter_GetUseImageSpacing, 'GetElapsedIterations' = LevelSetMotionRegistrationFilter_GetElapsedIterations, 'GetRMSChange' = LevelSetMotionRegistrationFilter_GetRMSChange, 'GetMetric' = LevelSetMotionRegistrationFilter_GetMetric, 'GetName' = LevelSetMotionRegistrationFilter_GetName, 'ToString' = LevelSetMotionRegistrationFilter_ToString, 'Execute' = LevelSetMotionRegistrationFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LevelSetMotionRegistrationFilter
# Start of delete_LiThresholdImageFilter

`delete_LiThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LiThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LiThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_LiThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter')
class(`delete_LiThresholdImageFilter`) = c("SWIGFunction", class('delete_LiThresholdImageFilter'))

# Start of new_LiThresholdImageFilter

`LiThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LiThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LiThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LiThresholdImageFilter);
  ans
  
}

attr(`LiThresholdImageFilter`, 'returnType') = '_p_itk__simple__LiThresholdImageFilter'
class(`LiThresholdImageFilter`) = c("SWIGFunction", class('LiThresholdImageFilter'))

# Start of LiThresholdImageFilter_SetInsideValue

`LiThresholdImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideValue = as.integer(InsideValue);
  
  if(length(InsideValue) > 1) {
    warning("using only the first element of InsideValue");
  };
  
  ;ans = .Call('R_swig_LiThresholdImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LiThresholdImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`LiThresholdImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter', 'integer')
class(`LiThresholdImageFilter_SetInsideValue`) = c("SWIGFunction", class('LiThresholdImageFilter_SetInsideValue'))

# Start of LiThresholdImageFilter_GetInsideValue

`LiThresholdImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LiThresholdImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LiThresholdImageFilter_GetInsideValue`, 'returnType') = 'integer'
attr(`LiThresholdImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter')
class(`LiThresholdImageFilter_GetInsideValue`) = c("SWIGFunction", class('LiThresholdImageFilter_GetInsideValue'))

# Start of LiThresholdImageFilter_SetOutsideValue

`LiThresholdImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutsideValue = as.integer(OutsideValue);
  
  if(length(OutsideValue) > 1) {
    warning("using only the first element of OutsideValue");
  };
  
  ;ans = .Call('R_swig_LiThresholdImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LiThresholdImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`LiThresholdImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter', 'integer')
class(`LiThresholdImageFilter_SetOutsideValue`) = c("SWIGFunction", class('LiThresholdImageFilter_SetOutsideValue'))

# Start of LiThresholdImageFilter_GetOutsideValue

`LiThresholdImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LiThresholdImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LiThresholdImageFilter_GetOutsideValue`, 'returnType') = 'integer'
attr(`LiThresholdImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter')
class(`LiThresholdImageFilter_GetOutsideValue`) = c("SWIGFunction", class('LiThresholdImageFilter_GetOutsideValue'))

# Start of LiThresholdImageFilter_SetNumberOfHistogramBins

`LiThresholdImageFilter_SetNumberOfHistogramBins` = function(self, NumberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfHistogramBins = as.integer(NumberOfHistogramBins);
  
  if(length(NumberOfHistogramBins) > 1) {
    warning("using only the first element of NumberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_LiThresholdImageFilter_SetNumberOfHistogramBins', self, NumberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LiThresholdImageFilter_SetNumberOfHistogramBins`, 'returnType') = 'void'
attr(`LiThresholdImageFilter_SetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter', 'integer')
class(`LiThresholdImageFilter_SetNumberOfHistogramBins`) = c("SWIGFunction", class('LiThresholdImageFilter_SetNumberOfHistogramBins'))

# Start of LiThresholdImageFilter_GetNumberOfHistogramBins

`LiThresholdImageFilter_GetNumberOfHistogramBins` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LiThresholdImageFilter_GetNumberOfHistogramBins', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LiThresholdImageFilter_GetNumberOfHistogramBins`, 'returnType') = 'integer'
attr(`LiThresholdImageFilter_GetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter')
class(`LiThresholdImageFilter_GetNumberOfHistogramBins`) = c("SWIGFunction", class('LiThresholdImageFilter_GetNumberOfHistogramBins'))

# Start of LiThresholdImageFilter_SetMaskOutput

`LiThresholdImageFilter_SetMaskOutput` = function(self, MaskOutput)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskOutput = as.logical(MaskOutput);
  ;ans = .Call('R_swig_LiThresholdImageFilter_SetMaskOutput', self, MaskOutput, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LiThresholdImageFilter_SetMaskOutput`, 'returnType') = 'void'
attr(`LiThresholdImageFilter_SetMaskOutput`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter', 'logical')
class(`LiThresholdImageFilter_SetMaskOutput`) = c("SWIGFunction", class('LiThresholdImageFilter_SetMaskOutput'))

# Start of LiThresholdImageFilter_MaskOutputOn

`LiThresholdImageFilter_MaskOutputOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LiThresholdImageFilter_MaskOutputOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LiThresholdImageFilter_MaskOutputOn`, 'returnType') = 'void'
attr(`LiThresholdImageFilter_MaskOutputOn`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter')
class(`LiThresholdImageFilter_MaskOutputOn`) = c("SWIGFunction", class('LiThresholdImageFilter_MaskOutputOn'))

# Start of LiThresholdImageFilter_MaskOutputOff

`LiThresholdImageFilter_MaskOutputOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_LiThresholdImageFilter_MaskOutputOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LiThresholdImageFilter_MaskOutputOff`, 'returnType') = 'void'
attr(`LiThresholdImageFilter_MaskOutputOff`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter')
class(`LiThresholdImageFilter_MaskOutputOff`) = c("SWIGFunction", class('LiThresholdImageFilter_MaskOutputOff'))

# Start of LiThresholdImageFilter_GetMaskOutput

`LiThresholdImageFilter_GetMaskOutput` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LiThresholdImageFilter_GetMaskOutput', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LiThresholdImageFilter_GetMaskOutput`, 'returnType') = 'logical'
attr(`LiThresholdImageFilter_GetMaskOutput`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter')
class(`LiThresholdImageFilter_GetMaskOutput`) = c("SWIGFunction", class('LiThresholdImageFilter_GetMaskOutput'))

# Start of LiThresholdImageFilter_SetMaskValue

`LiThresholdImageFilter_SetMaskValue` = function(self, MaskValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskValue = as.integer(MaskValue);
  
  if(length(MaskValue) > 1) {
    warning("using only the first element of MaskValue");
  };
  
  ;ans = .Call('R_swig_LiThresholdImageFilter_SetMaskValue', self, MaskValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`LiThresholdImageFilter_SetMaskValue`, 'returnType') = 'void'
attr(`LiThresholdImageFilter_SetMaskValue`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter', 'integer')
class(`LiThresholdImageFilter_SetMaskValue`) = c("SWIGFunction", class('LiThresholdImageFilter_SetMaskValue'))

# Start of LiThresholdImageFilter_GetMaskValue

`LiThresholdImageFilter_GetMaskValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LiThresholdImageFilter_GetMaskValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LiThresholdImageFilter_GetMaskValue`, 'returnType') = 'integer'
attr(`LiThresholdImageFilter_GetMaskValue`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter')
class(`LiThresholdImageFilter_GetMaskValue`) = c("SWIGFunction", class('LiThresholdImageFilter_GetMaskValue'))

# Start of LiThresholdImageFilter_GetThreshold

`LiThresholdImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LiThresholdImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LiThresholdImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`LiThresholdImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter')
class(`LiThresholdImageFilter_GetThreshold`) = c("SWIGFunction", class('LiThresholdImageFilter_GetThreshold'))

# Start of LiThresholdImageFilter_GetName

`LiThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LiThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LiThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`LiThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter')
class(`LiThresholdImageFilter_GetName`) = c("SWIGFunction", class('LiThresholdImageFilter_GetName'))

# Start of LiThresholdImageFilter_ToString

`LiThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LiThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LiThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`LiThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter')
class(`LiThresholdImageFilter_ToString`) = c("SWIGFunction", class('LiThresholdImageFilter_ToString'))

# Start of LiThresholdImageFilter_Execute

`LiThresholdImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_LiThresholdImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LiThresholdImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LiThresholdImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LiThresholdImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('LiThresholdImageFilter_Execute__SWIG_0'))

# Start of LiThresholdImageFilter_Execute

`LiThresholdImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_LiThresholdImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LiThresholdImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LiThresholdImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__LiThresholdImageFilter', '_p_itk__simple__Image')
class(`LiThresholdImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('LiThresholdImageFilter_Execute__SWIG_1'))

`LiThresholdImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__LiThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- LiThresholdImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__LiThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- LiThresholdImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LiThresholdImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::LiThresholdImageFilter
setMethod('$', '_p_itk__simple__LiThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetInsideValue' = LiThresholdImageFilter_SetInsideValue, 'GetInsideValue' = LiThresholdImageFilter_GetInsideValue, 'SetOutsideValue' = LiThresholdImageFilter_SetOutsideValue, 'GetOutsideValue' = LiThresholdImageFilter_GetOutsideValue, 'SetNumberOfHistogramBins' = LiThresholdImageFilter_SetNumberOfHistogramBins, 'GetNumberOfHistogramBins' = LiThresholdImageFilter_GetNumberOfHistogramBins, 'SetMaskOutput' = LiThresholdImageFilter_SetMaskOutput, 'MaskOutputOn' = LiThresholdImageFilter_MaskOutputOn, 'MaskOutputOff' = LiThresholdImageFilter_MaskOutputOff, 'GetMaskOutput' = LiThresholdImageFilter_GetMaskOutput, 'SetMaskValue' = LiThresholdImageFilter_SetMaskValue, 'GetMaskValue' = LiThresholdImageFilter_GetMaskValue, 'GetThreshold' = LiThresholdImageFilter_GetThreshold, 'GetName' = LiThresholdImageFilter_GetName, 'ToString' = LiThresholdImageFilter_ToString, 'Execute' = LiThresholdImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LiThresholdImageFilter
# Start of LiThreshold

`LiThreshold__SWIG_0` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_LiThreshold__SWIG_0', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LiThreshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`LiThreshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`LiThreshold__SWIG_0`) = c("SWIGFunction", class('LiThreshold__SWIG_0'))

# Start of LiThreshold

`LiThreshold__SWIG_1` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_LiThreshold__SWIG_1', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LiThreshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`LiThreshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`LiThreshold__SWIG_1`) = c("SWIGFunction", class('LiThreshold__SWIG_1'))

# Start of LiThreshold

`LiThreshold__SWIG_2` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_LiThreshold__SWIG_2', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LiThreshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`LiThreshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`LiThreshold__SWIG_2`) = c("SWIGFunction", class('LiThreshold__SWIG_2'))

# Start of LiThreshold

`LiThreshold__SWIG_3` = function(image, maskImage, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_LiThreshold__SWIG_3', image, maskImage, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LiThreshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`LiThreshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`LiThreshold__SWIG_3`) = c("SWIGFunction", class('LiThreshold__SWIG_3'))

# Start of LiThreshold

`LiThreshold__SWIG_4` = function(image, maskImage, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_LiThreshold__SWIG_4', image, maskImage, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LiThreshold__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`LiThreshold__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`LiThreshold__SWIG_4`) = c("SWIGFunction", class('LiThreshold__SWIG_4'))

# Start of LiThreshold

`LiThreshold__SWIG_5` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_LiThreshold__SWIG_5', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LiThreshold__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`LiThreshold__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`LiThreshold__SWIG_5`) = c("SWIGFunction", class('LiThreshold__SWIG_5'))

# Start of LiThreshold

`LiThreshold__SWIG_6` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_LiThreshold__SWIG_6', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LiThreshold__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`LiThreshold__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`LiThreshold__SWIG_6`) = c("SWIGFunction", class('LiThreshold__SWIG_6'))

# Start of LiThreshold

`LiThreshold__SWIG_7` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_LiThreshold__SWIG_7', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LiThreshold__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`LiThreshold__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`LiThreshold__SWIG_7`) = c("SWIGFunction", class('LiThreshold__SWIG_7'))

# Start of LiThreshold

`LiThreshold__SWIG_8` = function(image, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_LiThreshold__SWIG_8', image, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LiThreshold__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`LiThreshold__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`LiThreshold__SWIG_8`) = c("SWIGFunction", class('LiThreshold__SWIG_8'))

# Start of LiThreshold

`LiThreshold__SWIG_9` = function(image, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_LiThreshold__SWIG_9', image, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LiThreshold__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`LiThreshold__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`LiThreshold__SWIG_9`) = c("SWIGFunction", class('LiThreshold__SWIG_9'))

# Start of LiThreshold

`LiThreshold__SWIG_10` = function(image, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_LiThreshold__SWIG_10', image, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LiThreshold__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`LiThreshold__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`LiThreshold__SWIG_10`) = c("SWIGFunction", class('LiThreshold__SWIG_10'))

# Start of LiThreshold

`LiThreshold__SWIG_11` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_LiThreshold__SWIG_11', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LiThreshold__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`LiThreshold__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image')
class(`LiThreshold__SWIG_11`) = c("SWIGFunction", class('LiThreshold__SWIG_11'))

`LiThreshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 12
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- LiThreshold__SWIG_11; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- LiThreshold__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- LiThreshold__SWIG_10; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- LiThreshold__SWIG_4; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- LiThreshold__SWIG_9; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- LiThreshold__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- LiThreshold__SWIG_8; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- LiThreshold__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- LiThreshold__SWIG_7; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- LiThreshold__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- LiThreshold__SWIG_6; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- LiThreshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for LiThreshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_Log10ImageFilter

`delete_Log10ImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_Log10ImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_Log10ImageFilter`, 'returnType') = 'void'
attr(`delete_Log10ImageFilter`, "inputTypes") = c('_p_itk__simple__Log10ImageFilter')
class(`delete_Log10ImageFilter`) = c("SWIGFunction", class('delete_Log10ImageFilter'))

# Start of new_Log10ImageFilter

`Log10ImageFilter` = function()
{
  ;ans = .Call('R_swig_new_Log10ImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Log10ImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_Log10ImageFilter);
  ans
  
}

attr(`Log10ImageFilter`, 'returnType') = '_p_itk__simple__Log10ImageFilter'
class(`Log10ImageFilter`) = c("SWIGFunction", class('Log10ImageFilter'))

# Start of Log10ImageFilter_GetName

`Log10ImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Log10ImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Log10ImageFilter_GetName`, 'returnType') = 'character'
attr(`Log10ImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__Log10ImageFilter')
class(`Log10ImageFilter_GetName`) = c("SWIGFunction", class('Log10ImageFilter_GetName'))

# Start of Log10ImageFilter_ToString

`Log10ImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_Log10ImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`Log10ImageFilter_ToString`, 'returnType') = 'character'
attr(`Log10ImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__Log10ImageFilter')
class(`Log10ImageFilter_ToString`) = c("SWIGFunction", class('Log10ImageFilter_ToString'))

# Start of Log10ImageFilter_Execute

`Log10ImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Log10ImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Log10ImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`Log10ImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__Log10ImageFilter', '_p_itk__simple__Image')
class(`Log10ImageFilter_Execute`) = c("SWIGFunction", class('Log10ImageFilter_Execute'))

# Start of accessor method for itk::simple::Log10ImageFilter
setMethod('$', '_p_itk__simple__Log10ImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = Log10ImageFilter_GetName, 'ToString' = Log10ImageFilter_ToString, 'Execute' = Log10ImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::Log10ImageFilter
# Start of Log10

`Log10` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Log10', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Log10`, 'returnType') = '_p_itk__simple__Image'
attr(`Log10`, "inputTypes") = c('_p_itk__simple__Image')
class(`Log10`) = c("SWIGFunction", class('Log10'))

# Start of delete_LogImageFilter

`delete_LogImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_LogImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_LogImageFilter`, 'returnType') = 'void'
attr(`delete_LogImageFilter`, "inputTypes") = c('_p_itk__simple__LogImageFilter')
class(`delete_LogImageFilter`) = c("SWIGFunction", class('delete_LogImageFilter'))

# Start of new_LogImageFilter

`LogImageFilter` = function()
{
  ;ans = .Call('R_swig_new_LogImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__LogImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_LogImageFilter);
  ans
  
}

attr(`LogImageFilter`, 'returnType') = '_p_itk__simple__LogImageFilter'
class(`LogImageFilter`) = c("SWIGFunction", class('LogImageFilter'))

# Start of LogImageFilter_GetName

`LogImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LogImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LogImageFilter_GetName`, 'returnType') = 'character'
attr(`LogImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__LogImageFilter')
class(`LogImageFilter_GetName`) = c("SWIGFunction", class('LogImageFilter_GetName'))

# Start of LogImageFilter_ToString

`LogImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_LogImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`LogImageFilter_ToString`, 'returnType') = 'character'
attr(`LogImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__LogImageFilter')
class(`LogImageFilter_ToString`) = c("SWIGFunction", class('LogImageFilter_ToString'))

# Start of LogImageFilter_Execute

`LogImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_LogImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`LogImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`LogImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__LogImageFilter', '_p_itk__simple__Image')
class(`LogImageFilter_Execute`) = c("SWIGFunction", class('LogImageFilter_Execute'))

# Start of accessor method for itk::simple::LogImageFilter
setMethod('$', '_p_itk__simple__LogImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = LogImageFilter_GetName, 'ToString' = LogImageFilter_ToString, 'Execute' = LogImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::LogImageFilter
# Start of Log

`Log` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Log', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Log`, 'returnType') = '_p_itk__simple__Image'
attr(`Log`, "inputTypes") = c('_p_itk__simple__Image')
class(`Log`) = c("SWIGFunction", class('Log'))

# Start of delete_MagnitudeAndPhaseToComplexImageFilter

`delete_MagnitudeAndPhaseToComplexImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MagnitudeAndPhaseToComplexImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MagnitudeAndPhaseToComplexImageFilter`, 'returnType') = 'void'
attr(`delete_MagnitudeAndPhaseToComplexImageFilter`, "inputTypes") = c('_p_itk__simple__MagnitudeAndPhaseToComplexImageFilter')
class(`delete_MagnitudeAndPhaseToComplexImageFilter`) = c("SWIGFunction", class('delete_MagnitudeAndPhaseToComplexImageFilter'))

# Start of new_MagnitudeAndPhaseToComplexImageFilter

`MagnitudeAndPhaseToComplexImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MagnitudeAndPhaseToComplexImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MagnitudeAndPhaseToComplexImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MagnitudeAndPhaseToComplexImageFilter);
  ans
  
}

attr(`MagnitudeAndPhaseToComplexImageFilter`, 'returnType') = '_p_itk__simple__MagnitudeAndPhaseToComplexImageFilter'
class(`MagnitudeAndPhaseToComplexImageFilter`) = c("SWIGFunction", class('MagnitudeAndPhaseToComplexImageFilter'))

# Start of MagnitudeAndPhaseToComplexImageFilter_GetName

`MagnitudeAndPhaseToComplexImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MagnitudeAndPhaseToComplexImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MagnitudeAndPhaseToComplexImageFilter_GetName`, 'returnType') = 'character'
attr(`MagnitudeAndPhaseToComplexImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MagnitudeAndPhaseToComplexImageFilter')
class(`MagnitudeAndPhaseToComplexImageFilter_GetName`) = c("SWIGFunction", class('MagnitudeAndPhaseToComplexImageFilter_GetName'))

# Start of MagnitudeAndPhaseToComplexImageFilter_ToString

`MagnitudeAndPhaseToComplexImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MagnitudeAndPhaseToComplexImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MagnitudeAndPhaseToComplexImageFilter_ToString`, 'returnType') = 'character'
attr(`MagnitudeAndPhaseToComplexImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MagnitudeAndPhaseToComplexImageFilter')
class(`MagnitudeAndPhaseToComplexImageFilter_ToString`) = c("SWIGFunction", class('MagnitudeAndPhaseToComplexImageFilter_ToString'))

# Start of MagnitudeAndPhaseToComplexImageFilter_Execute

`MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__MagnitudeAndPhaseToComplexImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_0'))

# Start of MagnitudeAndPhaseToComplexImageFilter_Execute

`MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__MagnitudeAndPhaseToComplexImageFilter', '_p_itk__simple__Image', 'numeric')
class(`MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_1'))

# Start of MagnitudeAndPhaseToComplexImageFilter_Execute

`MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__MagnitudeAndPhaseToComplexImageFilter', 'numeric', '_p_itk__simple__Image')
class(`MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_2'))

`MagnitudeAndPhaseToComplexImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__MagnitudeAndPhaseToComplexImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__MagnitudeAndPhaseToComplexImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__MagnitudeAndPhaseToComplexImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MagnitudeAndPhaseToComplexImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MagnitudeAndPhaseToComplexImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::MagnitudeAndPhaseToComplexImageFilter
setMethod('$', '_p_itk__simple__MagnitudeAndPhaseToComplexImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = MagnitudeAndPhaseToComplexImageFilter_GetName, 'ToString' = MagnitudeAndPhaseToComplexImageFilter_ToString, 'Execute' = MagnitudeAndPhaseToComplexImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MagnitudeAndPhaseToComplexImageFilter
# Start of MagnitudeAndPhaseToComplex

`MagnitudeAndPhaseToComplex__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_MagnitudeAndPhaseToComplex__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MagnitudeAndPhaseToComplex__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MagnitudeAndPhaseToComplex__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MagnitudeAndPhaseToComplex__SWIG_0`) = c("SWIGFunction", class('MagnitudeAndPhaseToComplex__SWIG_0'))

# Start of MagnitudeAndPhaseToComplex

`MagnitudeAndPhaseToComplex__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_MagnitudeAndPhaseToComplex__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MagnitudeAndPhaseToComplex__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MagnitudeAndPhaseToComplex__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`MagnitudeAndPhaseToComplex__SWIG_1`) = c("SWIGFunction", class('MagnitudeAndPhaseToComplex__SWIG_1'))

# Start of MagnitudeAndPhaseToComplex

`MagnitudeAndPhaseToComplex__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_MagnitudeAndPhaseToComplex__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MagnitudeAndPhaseToComplex__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MagnitudeAndPhaseToComplex__SWIG_2`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`MagnitudeAndPhaseToComplex__SWIG_2`) = c("SWIGFunction", class('MagnitudeAndPhaseToComplex__SWIG_2'))

`MagnitudeAndPhaseToComplex` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- MagnitudeAndPhaseToComplex__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- MagnitudeAndPhaseToComplex__SWIG_1; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- MagnitudeAndPhaseToComplex__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MagnitudeAndPhaseToComplex with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MaskImageFilter

`delete_MaskImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MaskImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MaskImageFilter`, 'returnType') = 'void'
attr(`delete_MaskImageFilter`, "inputTypes") = c('_p_itk__simple__MaskImageFilter')
class(`delete_MaskImageFilter`) = c("SWIGFunction", class('delete_MaskImageFilter'))

# Start of new_MaskImageFilter

`MaskImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MaskImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MaskImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MaskImageFilter);
  ans
  
}

attr(`MaskImageFilter`, 'returnType') = '_p_itk__simple__MaskImageFilter'
class(`MaskImageFilter`) = c("SWIGFunction", class('MaskImageFilter'))

# Start of MaskImageFilter_SetOutsideValue

`MaskImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_MaskImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MaskImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`MaskImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__MaskImageFilter', 'numeric')
class(`MaskImageFilter_SetOutsideValue`) = c("SWIGFunction", class('MaskImageFilter_SetOutsideValue'))

# Start of MaskImageFilter_GetOutsideValue

`MaskImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaskImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaskImageFilter_GetOutsideValue`, 'returnType') = 'numeric'
attr(`MaskImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__MaskImageFilter')
class(`MaskImageFilter_GetOutsideValue`) = c("SWIGFunction", class('MaskImageFilter_GetOutsideValue'))

# Start of MaskImageFilter_SetMaskingValue

`MaskImageFilter_SetMaskingValue` = function(self, MaskingValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_MaskImageFilter_SetMaskingValue', self, MaskingValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MaskImageFilter_SetMaskingValue`, 'returnType') = 'void'
attr(`MaskImageFilter_SetMaskingValue`, "inputTypes") = c('_p_itk__simple__MaskImageFilter', 'numeric')
class(`MaskImageFilter_SetMaskingValue`) = c("SWIGFunction", class('MaskImageFilter_SetMaskingValue'))

# Start of MaskImageFilter_GetMaskingValue

`MaskImageFilter_GetMaskingValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaskImageFilter_GetMaskingValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaskImageFilter_GetMaskingValue`, 'returnType') = 'numeric'
attr(`MaskImageFilter_GetMaskingValue`, "inputTypes") = c('_p_itk__simple__MaskImageFilter')
class(`MaskImageFilter_GetMaskingValue`) = c("SWIGFunction", class('MaskImageFilter_GetMaskingValue'))

# Start of MaskImageFilter_GetName

`MaskImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaskImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaskImageFilter_GetName`, 'returnType') = 'character'
attr(`MaskImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MaskImageFilter')
class(`MaskImageFilter_GetName`) = c("SWIGFunction", class('MaskImageFilter_GetName'))

# Start of MaskImageFilter_ToString

`MaskImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaskImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaskImageFilter_ToString`, 'returnType') = 'character'
attr(`MaskImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MaskImageFilter')
class(`MaskImageFilter_ToString`) = c("SWIGFunction", class('MaskImageFilter_ToString'))

# Start of MaskImageFilter_Execute

`MaskImageFilter_Execute` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_MaskImageFilter_Execute', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaskImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`MaskImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__MaskImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MaskImageFilter_Execute`) = c("SWIGFunction", class('MaskImageFilter_Execute'))

# Start of accessor method for itk::simple::MaskImageFilter
setMethod('$', '_p_itk__simple__MaskImageFilter', function(x, name)

{
  accessorFuns = list('SetOutsideValue' = MaskImageFilter_SetOutsideValue, 'GetOutsideValue' = MaskImageFilter_GetOutsideValue, 'SetMaskingValue' = MaskImageFilter_SetMaskingValue, 'GetMaskingValue' = MaskImageFilter_GetMaskingValue, 'GetName' = MaskImageFilter_GetName, 'ToString' = MaskImageFilter_ToString, 'Execute' = MaskImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MaskImageFilter
# Start of Mask

`Mask__SWIG_0` = function(image, maskImage, outsideValue, maskingValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  
  ;ans = .Call('R_swig_Mask__SWIG_0', image, maskImage, outsideValue, maskingValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Mask__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Mask__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`Mask__SWIG_0`) = c("SWIGFunction", class('Mask__SWIG_0'))

# Start of Mask

`Mask__SWIG_1` = function(image, maskImage, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  ;ans = .Call('R_swig_Mask__SWIG_1', image, maskImage, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Mask__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Mask__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`Mask__SWIG_1`) = c("SWIGFunction", class('Mask__SWIG_1'))

# Start of Mask

`Mask__SWIG_2` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_Mask__SWIG_2', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Mask__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Mask__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Mask__SWIG_2`) = c("SWIGFunction", class('Mask__SWIG_2'))

`Mask` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Mask__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Mask__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- Mask__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Mask with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MaskNegatedImageFilter

`delete_MaskNegatedImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MaskNegatedImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MaskNegatedImageFilter`, 'returnType') = 'void'
attr(`delete_MaskNegatedImageFilter`, "inputTypes") = c('_p_itk__simple__MaskNegatedImageFilter')
class(`delete_MaskNegatedImageFilter`) = c("SWIGFunction", class('delete_MaskNegatedImageFilter'))

# Start of new_MaskNegatedImageFilter

`MaskNegatedImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MaskNegatedImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MaskNegatedImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MaskNegatedImageFilter);
  ans
  
}

attr(`MaskNegatedImageFilter`, 'returnType') = '_p_itk__simple__MaskNegatedImageFilter'
class(`MaskNegatedImageFilter`) = c("SWIGFunction", class('MaskNegatedImageFilter'))

# Start of MaskNegatedImageFilter_SetOutsideValue

`MaskNegatedImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_MaskNegatedImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MaskNegatedImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`MaskNegatedImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__MaskNegatedImageFilter', 'numeric')
class(`MaskNegatedImageFilter_SetOutsideValue`) = c("SWIGFunction", class('MaskNegatedImageFilter_SetOutsideValue'))

# Start of MaskNegatedImageFilter_GetOutsideValue

`MaskNegatedImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaskNegatedImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaskNegatedImageFilter_GetOutsideValue`, 'returnType') = 'numeric'
attr(`MaskNegatedImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__MaskNegatedImageFilter')
class(`MaskNegatedImageFilter_GetOutsideValue`) = c("SWIGFunction", class('MaskNegatedImageFilter_GetOutsideValue'))

# Start of MaskNegatedImageFilter_SetMaskingValue

`MaskNegatedImageFilter_SetMaskingValue` = function(self, MaskingValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_MaskNegatedImageFilter_SetMaskingValue', self, MaskingValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MaskNegatedImageFilter_SetMaskingValue`, 'returnType') = 'void'
attr(`MaskNegatedImageFilter_SetMaskingValue`, "inputTypes") = c('_p_itk__simple__MaskNegatedImageFilter', 'numeric')
class(`MaskNegatedImageFilter_SetMaskingValue`) = c("SWIGFunction", class('MaskNegatedImageFilter_SetMaskingValue'))

# Start of MaskNegatedImageFilter_GetMaskingValue

`MaskNegatedImageFilter_GetMaskingValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaskNegatedImageFilter_GetMaskingValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaskNegatedImageFilter_GetMaskingValue`, 'returnType') = 'numeric'
attr(`MaskNegatedImageFilter_GetMaskingValue`, "inputTypes") = c('_p_itk__simple__MaskNegatedImageFilter')
class(`MaskNegatedImageFilter_GetMaskingValue`) = c("SWIGFunction", class('MaskNegatedImageFilter_GetMaskingValue'))

# Start of MaskNegatedImageFilter_GetName

`MaskNegatedImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaskNegatedImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaskNegatedImageFilter_GetName`, 'returnType') = 'character'
attr(`MaskNegatedImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MaskNegatedImageFilter')
class(`MaskNegatedImageFilter_GetName`) = c("SWIGFunction", class('MaskNegatedImageFilter_GetName'))

# Start of MaskNegatedImageFilter_ToString

`MaskNegatedImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaskNegatedImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaskNegatedImageFilter_ToString`, 'returnType') = 'character'
attr(`MaskNegatedImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MaskNegatedImageFilter')
class(`MaskNegatedImageFilter_ToString`) = c("SWIGFunction", class('MaskNegatedImageFilter_ToString'))

# Start of MaskNegatedImageFilter_Execute

`MaskNegatedImageFilter_Execute` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_MaskNegatedImageFilter_Execute', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaskNegatedImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`MaskNegatedImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__MaskNegatedImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MaskNegatedImageFilter_Execute`) = c("SWIGFunction", class('MaskNegatedImageFilter_Execute'))

# Start of accessor method for itk::simple::MaskNegatedImageFilter
setMethod('$', '_p_itk__simple__MaskNegatedImageFilter', function(x, name)

{
  accessorFuns = list('SetOutsideValue' = MaskNegatedImageFilter_SetOutsideValue, 'GetOutsideValue' = MaskNegatedImageFilter_GetOutsideValue, 'SetMaskingValue' = MaskNegatedImageFilter_SetMaskingValue, 'GetMaskingValue' = MaskNegatedImageFilter_GetMaskingValue, 'GetName' = MaskNegatedImageFilter_GetName, 'ToString' = MaskNegatedImageFilter_ToString, 'Execute' = MaskNegatedImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MaskNegatedImageFilter
# Start of MaskNegated

`MaskNegated__SWIG_0` = function(image, maskImage, outsideValue, maskingValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  
  ;ans = .Call('R_swig_MaskNegated__SWIG_0', image, maskImage, outsideValue, maskingValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaskNegated__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MaskNegated__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`MaskNegated__SWIG_0`) = c("SWIGFunction", class('MaskNegated__SWIG_0'))

# Start of MaskNegated

`MaskNegated__SWIG_1` = function(image, maskImage, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  ;ans = .Call('R_swig_MaskNegated__SWIG_1', image, maskImage, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaskNegated__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MaskNegated__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`MaskNegated__SWIG_1`) = c("SWIGFunction", class('MaskNegated__SWIG_1'))

# Start of MaskNegated

`MaskNegated__SWIG_2` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_MaskNegated__SWIG_2', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaskNegated__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MaskNegated__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MaskNegated__SWIG_2`) = c("SWIGFunction", class('MaskNegated__SWIG_2'))

`MaskNegated` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- MaskNegated__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- MaskNegated__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- MaskNegated__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MaskNegated with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MaskedAssignImageFilter

`delete_MaskedAssignImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MaskedAssignImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MaskedAssignImageFilter`, 'returnType') = 'void'
attr(`delete_MaskedAssignImageFilter`, "inputTypes") = c('_p_itk__simple__MaskedAssignImageFilter')
class(`delete_MaskedAssignImageFilter`) = c("SWIGFunction", class('delete_MaskedAssignImageFilter'))

# Start of new_MaskedAssignImageFilter

`MaskedAssignImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MaskedAssignImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MaskedAssignImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MaskedAssignImageFilter);
  ans
  
}

attr(`MaskedAssignImageFilter`, 'returnType') = '_p_itk__simple__MaskedAssignImageFilter'
class(`MaskedAssignImageFilter`) = c("SWIGFunction", class('MaskedAssignImageFilter'))

# Start of MaskedAssignImageFilter_SetAssignConstant

`MaskedAssignImageFilter_SetAssignConstant` = function(self, AssignConstant)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_MaskedAssignImageFilter_SetAssignConstant', self, AssignConstant, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MaskedAssignImageFilter_SetAssignConstant`, 'returnType') = 'void'
attr(`MaskedAssignImageFilter_SetAssignConstant`, "inputTypes") = c('_p_itk__simple__MaskedAssignImageFilter', 'numeric')
class(`MaskedAssignImageFilter_SetAssignConstant`) = c("SWIGFunction", class('MaskedAssignImageFilter_SetAssignConstant'))

# Start of MaskedAssignImageFilter_GetAssignConstant

`MaskedAssignImageFilter_GetAssignConstant` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaskedAssignImageFilter_GetAssignConstant', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaskedAssignImageFilter_GetAssignConstant`, 'returnType') = 'numeric'
attr(`MaskedAssignImageFilter_GetAssignConstant`, "inputTypes") = c('_p_itk__simple__MaskedAssignImageFilter')
class(`MaskedAssignImageFilter_GetAssignConstant`) = c("SWIGFunction", class('MaskedAssignImageFilter_GetAssignConstant'))

# Start of MaskedAssignImageFilter_GetName

`MaskedAssignImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaskedAssignImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaskedAssignImageFilter_GetName`, 'returnType') = 'character'
attr(`MaskedAssignImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MaskedAssignImageFilter')
class(`MaskedAssignImageFilter_GetName`) = c("SWIGFunction", class('MaskedAssignImageFilter_GetName'))

# Start of MaskedAssignImageFilter_ToString

`MaskedAssignImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaskedAssignImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaskedAssignImageFilter_ToString`, 'returnType') = 'character'
attr(`MaskedAssignImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MaskedAssignImageFilter')
class(`MaskedAssignImageFilter_ToString`) = c("SWIGFunction", class('MaskedAssignImageFilter_ToString'))

# Start of MaskedAssignImageFilter_Execute

`MaskedAssignImageFilter_Execute__SWIG_0` = function(self, image, maskImage, assignImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  if (inherits(assignImage, "ExternalReference")) assignImage = slot(assignImage,"ref"); 
  ;ans = .Call('R_swig_MaskedAssignImageFilter_Execute__SWIG_0', self, image, maskImage, assignImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaskedAssignImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MaskedAssignImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__MaskedAssignImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MaskedAssignImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('MaskedAssignImageFilter_Execute__SWIG_0'))

# Start of MaskedAssignImageFilter_Execute

`MaskedAssignImageFilter_Execute__SWIG_1` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_MaskedAssignImageFilter_Execute__SWIG_1', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaskedAssignImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MaskedAssignImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__MaskedAssignImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MaskedAssignImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('MaskedAssignImageFilter_Execute__SWIG_1'))

`MaskedAssignImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__MaskedAssignImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MaskedAssignImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__MaskedAssignImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- MaskedAssignImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MaskedAssignImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::MaskedAssignImageFilter
setMethod('$', '_p_itk__simple__MaskedAssignImageFilter', function(x, name)

{
  accessorFuns = list('SetAssignConstant' = MaskedAssignImageFilter_SetAssignConstant, 'GetAssignConstant' = MaskedAssignImageFilter_GetAssignConstant, 'GetName' = MaskedAssignImageFilter_GetName, 'ToString' = MaskedAssignImageFilter_ToString, 'Execute' = MaskedAssignImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MaskedAssignImageFilter
# Start of MaskedAssign

`MaskedAssign__SWIG_0` = function(image, maskImage, assignImage, assignConstant, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  if (inherits(assignImage, "ExternalReference")) assignImage = slot(assignImage,"ref"); 
  
  ;ans = .Call('R_swig_MaskedAssign__SWIG_0', image, maskImage, assignImage, assignConstant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaskedAssign__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MaskedAssign__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`MaskedAssign__SWIG_0`) = c("SWIGFunction", class('MaskedAssign__SWIG_0'))

# Start of MaskedAssign

`MaskedAssign__SWIG_1` = function(image, maskImage, assignImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  if (inherits(assignImage, "ExternalReference")) assignImage = slot(assignImage,"ref"); 
  ;ans = .Call('R_swig_MaskedAssign__SWIG_1', image, maskImage, assignImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaskedAssign__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MaskedAssign__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MaskedAssign__SWIG_1`) = c("SWIGFunction", class('MaskedAssign__SWIG_1'))

# Start of MaskedAssign

`MaskedAssign__SWIG_2` = function(image, maskImage, assignConstant, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  ;ans = .Call('R_swig_MaskedAssign__SWIG_2', image, maskImage, assignConstant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaskedAssign__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MaskedAssign__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`MaskedAssign__SWIG_2`) = c("SWIGFunction", class('MaskedAssign__SWIG_2'))

# Start of MaskedAssign

`MaskedAssign__SWIG_3` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_MaskedAssign__SWIG_3', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaskedAssign__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`MaskedAssign__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MaskedAssign__SWIG_3`) = c("SWIGFunction", class('MaskedAssign__SWIG_3'))

`MaskedAssign` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- MaskedAssign__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MaskedAssign__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- MaskedAssign__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- MaskedAssign__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MaskedAssign with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MaskedFFTNormalizedCorrelationImageFilter

`delete_MaskedFFTNormalizedCorrelationImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MaskedFFTNormalizedCorrelationImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MaskedFFTNormalizedCorrelationImageFilter`, 'returnType') = 'void'
attr(`delete_MaskedFFTNormalizedCorrelationImageFilter`, "inputTypes") = c('_p_itk__simple__MaskedFFTNormalizedCorrelationImageFilter')
class(`delete_MaskedFFTNormalizedCorrelationImageFilter`) = c("SWIGFunction", class('delete_MaskedFFTNormalizedCorrelationImageFilter'))

# Start of new_MaskedFFTNormalizedCorrelationImageFilter

`MaskedFFTNormalizedCorrelationImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MaskedFFTNormalizedCorrelationImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MaskedFFTNormalizedCorrelationImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MaskedFFTNormalizedCorrelationImageFilter);
  ans
  
}

attr(`MaskedFFTNormalizedCorrelationImageFilter`, 'returnType') = '_p_itk__simple__MaskedFFTNormalizedCorrelationImageFilter'
class(`MaskedFFTNormalizedCorrelationImageFilter`) = c("SWIGFunction", class('MaskedFFTNormalizedCorrelationImageFilter'))

# Start of MaskedFFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels

`MaskedFFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels` = function(self, RequiredNumberOfOverlappingPixels)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  RequiredNumberOfOverlappingPixels = as.integer(RequiredNumberOfOverlappingPixels);
  
  if(length(RequiredNumberOfOverlappingPixels) > 1) {
    warning("using only the first element of RequiredNumberOfOverlappingPixels");
  };
  
  ;ans = .Call('R_swig_MaskedFFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels', self, RequiredNumberOfOverlappingPixels, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MaskedFFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels`, 'returnType') = 'void'
attr(`MaskedFFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels`, "inputTypes") = c('_p_itk__simple__MaskedFFTNormalizedCorrelationImageFilter', 'integer')
class(`MaskedFFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels`) = c("SWIGFunction", class('MaskedFFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels'))

# Start of MaskedFFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels

`MaskedFFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaskedFFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaskedFFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels`, 'returnType') = 'integer'
attr(`MaskedFFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels`, "inputTypes") = c('_p_itk__simple__MaskedFFTNormalizedCorrelationImageFilter')
class(`MaskedFFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels`) = c("SWIGFunction", class('MaskedFFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels'))

# Start of MaskedFFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels

`MaskedFFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels` = function(self, RequiredFractionOfOverlappingPixels)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_MaskedFFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels', self, RequiredFractionOfOverlappingPixels, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MaskedFFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels`, 'returnType') = 'void'
attr(`MaskedFFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels`, "inputTypes") = c('_p_itk__simple__MaskedFFTNormalizedCorrelationImageFilter', 'numeric')
class(`MaskedFFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels`) = c("SWIGFunction", class('MaskedFFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels'))

# Start of MaskedFFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels

`MaskedFFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaskedFFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaskedFFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels`, 'returnType') = 'numeric'
attr(`MaskedFFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels`, "inputTypes") = c('_p_itk__simple__MaskedFFTNormalizedCorrelationImageFilter')
class(`MaskedFFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels`) = c("SWIGFunction", class('MaskedFFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels'))

# Start of MaskedFFTNormalizedCorrelationImageFilter_GetName

`MaskedFFTNormalizedCorrelationImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaskedFFTNormalizedCorrelationImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaskedFFTNormalizedCorrelationImageFilter_GetName`, 'returnType') = 'character'
attr(`MaskedFFTNormalizedCorrelationImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MaskedFFTNormalizedCorrelationImageFilter')
class(`MaskedFFTNormalizedCorrelationImageFilter_GetName`) = c("SWIGFunction", class('MaskedFFTNormalizedCorrelationImageFilter_GetName'))

# Start of MaskedFFTNormalizedCorrelationImageFilter_ToString

`MaskedFFTNormalizedCorrelationImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaskedFFTNormalizedCorrelationImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaskedFFTNormalizedCorrelationImageFilter_ToString`, 'returnType') = 'character'
attr(`MaskedFFTNormalizedCorrelationImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MaskedFFTNormalizedCorrelationImageFilter')
class(`MaskedFFTNormalizedCorrelationImageFilter_ToString`) = c("SWIGFunction", class('MaskedFFTNormalizedCorrelationImageFilter_ToString'))

# Start of MaskedFFTNormalizedCorrelationImageFilter_Execute

`MaskedFFTNormalizedCorrelationImageFilter_Execute` = function(self, fixedImage, movingImage, fixedImageMask, movingImageMask, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  if (inherits(fixedImageMask, "ExternalReference")) fixedImageMask = slot(fixedImageMask,"ref"); 
  if (inherits(movingImageMask, "ExternalReference")) movingImageMask = slot(movingImageMask,"ref"); 
  ;ans = .Call('R_swig_MaskedFFTNormalizedCorrelationImageFilter_Execute', self, fixedImage, movingImage, fixedImageMask, movingImageMask, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaskedFFTNormalizedCorrelationImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`MaskedFFTNormalizedCorrelationImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__MaskedFFTNormalizedCorrelationImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MaskedFFTNormalizedCorrelationImageFilter_Execute`) = c("SWIGFunction", class('MaskedFFTNormalizedCorrelationImageFilter_Execute'))

# Start of accessor method for itk::simple::MaskedFFTNormalizedCorrelationImageFilter
setMethod('$', '_p_itk__simple__MaskedFFTNormalizedCorrelationImageFilter', function(x, name)

{
  accessorFuns = list('SetRequiredNumberOfOverlappingPixels' = MaskedFFTNormalizedCorrelationImageFilter_SetRequiredNumberOfOverlappingPixels, 'GetRequiredNumberOfOverlappingPixels' = MaskedFFTNormalizedCorrelationImageFilter_GetRequiredNumberOfOverlappingPixels, 'SetRequiredFractionOfOverlappingPixels' = MaskedFFTNormalizedCorrelationImageFilter_SetRequiredFractionOfOverlappingPixels, 'GetRequiredFractionOfOverlappingPixels' = MaskedFFTNormalizedCorrelationImageFilter_GetRequiredFractionOfOverlappingPixels, 'GetName' = MaskedFFTNormalizedCorrelationImageFilter_GetName, 'ToString' = MaskedFFTNormalizedCorrelationImageFilter_ToString, 'Execute' = MaskedFFTNormalizedCorrelationImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MaskedFFTNormalizedCorrelationImageFilter
# Start of MaskedFFTNormalizedCorrelation

`MaskedFFTNormalizedCorrelation__SWIG_0` = function(fixedImage, movingImage, fixedImageMask, movingImageMask, requiredNumberOfOverlappingPixels, requiredFractionOfOverlappingPixels, .copy = FALSE)
{
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  if (inherits(fixedImageMask, "ExternalReference")) fixedImageMask = slot(fixedImageMask,"ref"); 
  if (inherits(movingImageMask, "ExternalReference")) movingImageMask = slot(movingImageMask,"ref"); 
  requiredNumberOfOverlappingPixels = as.integer(requiredNumberOfOverlappingPixels);
  
  if(length(requiredNumberOfOverlappingPixels) > 1) {
    warning("using only the first element of requiredNumberOfOverlappingPixels");
  };
  
  
  ;ans = .Call('R_swig_MaskedFFTNormalizedCorrelation__SWIG_0', fixedImage, movingImage, fixedImageMask, movingImageMask, requiredNumberOfOverlappingPixels, requiredFractionOfOverlappingPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaskedFFTNormalizedCorrelation__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MaskedFFTNormalizedCorrelation__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric')
class(`MaskedFFTNormalizedCorrelation__SWIG_0`) = c("SWIGFunction", class('MaskedFFTNormalizedCorrelation__SWIG_0'))

# Start of MaskedFFTNormalizedCorrelation

`MaskedFFTNormalizedCorrelation__SWIG_1` = function(fixedImage, movingImage, fixedImageMask, movingImageMask, requiredNumberOfOverlappingPixels, .copy = FALSE)
{
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  if (inherits(fixedImageMask, "ExternalReference")) fixedImageMask = slot(fixedImageMask,"ref"); 
  if (inherits(movingImageMask, "ExternalReference")) movingImageMask = slot(movingImageMask,"ref"); 
  requiredNumberOfOverlappingPixels = as.integer(requiredNumberOfOverlappingPixels);
  
  if(length(requiredNumberOfOverlappingPixels) > 1) {
    warning("using only the first element of requiredNumberOfOverlappingPixels");
  };
  
  ;ans = .Call('R_swig_MaskedFFTNormalizedCorrelation__SWIG_1', fixedImage, movingImage, fixedImageMask, movingImageMask, requiredNumberOfOverlappingPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaskedFFTNormalizedCorrelation__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MaskedFFTNormalizedCorrelation__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`MaskedFFTNormalizedCorrelation__SWIG_1`) = c("SWIGFunction", class('MaskedFFTNormalizedCorrelation__SWIG_1'))

# Start of MaskedFFTNormalizedCorrelation

`MaskedFFTNormalizedCorrelation__SWIG_2` = function(fixedImage, movingImage, fixedImageMask, movingImageMask, .copy = FALSE)
{
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  if (inherits(fixedImageMask, "ExternalReference")) fixedImageMask = slot(fixedImageMask,"ref"); 
  if (inherits(movingImageMask, "ExternalReference")) movingImageMask = slot(movingImageMask,"ref"); 
  ;ans = .Call('R_swig_MaskedFFTNormalizedCorrelation__SWIG_2', fixedImage, movingImage, fixedImageMask, movingImageMask, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaskedFFTNormalizedCorrelation__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MaskedFFTNormalizedCorrelation__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MaskedFFTNormalizedCorrelation__SWIG_2`) = c("SWIGFunction", class('MaskedFFTNormalizedCorrelation__SWIG_2'))

`MaskedFFTNormalizedCorrelation` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- MaskedFFTNormalizedCorrelation__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- MaskedFFTNormalizedCorrelation__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- MaskedFFTNormalizedCorrelation__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MaskedFFTNormalizedCorrelation with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MaximumEntropyThresholdImageFilter

`delete_MaximumEntropyThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MaximumEntropyThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MaximumEntropyThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_MaximumEntropyThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter')
class(`delete_MaximumEntropyThresholdImageFilter`) = c("SWIGFunction", class('delete_MaximumEntropyThresholdImageFilter'))

# Start of new_MaximumEntropyThresholdImageFilter

`MaximumEntropyThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MaximumEntropyThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MaximumEntropyThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MaximumEntropyThresholdImageFilter);
  ans
  
}

attr(`MaximumEntropyThresholdImageFilter`, 'returnType') = '_p_itk__simple__MaximumEntropyThresholdImageFilter'
class(`MaximumEntropyThresholdImageFilter`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter'))

# Start of MaximumEntropyThresholdImageFilter_SetInsideValue

`MaximumEntropyThresholdImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideValue = as.integer(InsideValue);
  
  if(length(InsideValue) > 1) {
    warning("using only the first element of InsideValue");
  };
  
  ;ans = .Call('R_swig_MaximumEntropyThresholdImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MaximumEntropyThresholdImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`MaximumEntropyThresholdImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter', 'integer')
class(`MaximumEntropyThresholdImageFilter_SetInsideValue`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_SetInsideValue'))

# Start of MaximumEntropyThresholdImageFilter_GetInsideValue

`MaximumEntropyThresholdImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaximumEntropyThresholdImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaximumEntropyThresholdImageFilter_GetInsideValue`, 'returnType') = 'integer'
attr(`MaximumEntropyThresholdImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter')
class(`MaximumEntropyThresholdImageFilter_GetInsideValue`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_GetInsideValue'))

# Start of MaximumEntropyThresholdImageFilter_SetOutsideValue

`MaximumEntropyThresholdImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutsideValue = as.integer(OutsideValue);
  
  if(length(OutsideValue) > 1) {
    warning("using only the first element of OutsideValue");
  };
  
  ;ans = .Call('R_swig_MaximumEntropyThresholdImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MaximumEntropyThresholdImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`MaximumEntropyThresholdImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter', 'integer')
class(`MaximumEntropyThresholdImageFilter_SetOutsideValue`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_SetOutsideValue'))

# Start of MaximumEntropyThresholdImageFilter_GetOutsideValue

`MaximumEntropyThresholdImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaximumEntropyThresholdImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaximumEntropyThresholdImageFilter_GetOutsideValue`, 'returnType') = 'integer'
attr(`MaximumEntropyThresholdImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter')
class(`MaximumEntropyThresholdImageFilter_GetOutsideValue`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_GetOutsideValue'))

# Start of MaximumEntropyThresholdImageFilter_SetNumberOfHistogramBins

`MaximumEntropyThresholdImageFilter_SetNumberOfHistogramBins` = function(self, NumberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfHistogramBins = as.integer(NumberOfHistogramBins);
  
  if(length(NumberOfHistogramBins) > 1) {
    warning("using only the first element of NumberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_MaximumEntropyThresholdImageFilter_SetNumberOfHistogramBins', self, NumberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MaximumEntropyThresholdImageFilter_SetNumberOfHistogramBins`, 'returnType') = 'void'
attr(`MaximumEntropyThresholdImageFilter_SetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter', 'integer')
class(`MaximumEntropyThresholdImageFilter_SetNumberOfHistogramBins`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_SetNumberOfHistogramBins'))

# Start of MaximumEntropyThresholdImageFilter_GetNumberOfHistogramBins

`MaximumEntropyThresholdImageFilter_GetNumberOfHistogramBins` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaximumEntropyThresholdImageFilter_GetNumberOfHistogramBins', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaximumEntropyThresholdImageFilter_GetNumberOfHistogramBins`, 'returnType') = 'integer'
attr(`MaximumEntropyThresholdImageFilter_GetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter')
class(`MaximumEntropyThresholdImageFilter_GetNumberOfHistogramBins`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_GetNumberOfHistogramBins'))

# Start of MaximumEntropyThresholdImageFilter_SetMaskOutput

`MaximumEntropyThresholdImageFilter_SetMaskOutput` = function(self, MaskOutput)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskOutput = as.logical(MaskOutput);
  ;ans = .Call('R_swig_MaximumEntropyThresholdImageFilter_SetMaskOutput', self, MaskOutput, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MaximumEntropyThresholdImageFilter_SetMaskOutput`, 'returnType') = 'void'
attr(`MaximumEntropyThresholdImageFilter_SetMaskOutput`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter', 'logical')
class(`MaximumEntropyThresholdImageFilter_SetMaskOutput`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_SetMaskOutput'))

# Start of MaximumEntropyThresholdImageFilter_MaskOutputOn

`MaximumEntropyThresholdImageFilter_MaskOutputOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_MaximumEntropyThresholdImageFilter_MaskOutputOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MaximumEntropyThresholdImageFilter_MaskOutputOn`, 'returnType') = 'void'
attr(`MaximumEntropyThresholdImageFilter_MaskOutputOn`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter')
class(`MaximumEntropyThresholdImageFilter_MaskOutputOn`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_MaskOutputOn'))

# Start of MaximumEntropyThresholdImageFilter_MaskOutputOff

`MaximumEntropyThresholdImageFilter_MaskOutputOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_MaximumEntropyThresholdImageFilter_MaskOutputOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MaximumEntropyThresholdImageFilter_MaskOutputOff`, 'returnType') = 'void'
attr(`MaximumEntropyThresholdImageFilter_MaskOutputOff`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter')
class(`MaximumEntropyThresholdImageFilter_MaskOutputOff`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_MaskOutputOff'))

# Start of MaximumEntropyThresholdImageFilter_GetMaskOutput

`MaximumEntropyThresholdImageFilter_GetMaskOutput` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaximumEntropyThresholdImageFilter_GetMaskOutput', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaximumEntropyThresholdImageFilter_GetMaskOutput`, 'returnType') = 'logical'
attr(`MaximumEntropyThresholdImageFilter_GetMaskOutput`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter')
class(`MaximumEntropyThresholdImageFilter_GetMaskOutput`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_GetMaskOutput'))

# Start of MaximumEntropyThresholdImageFilter_SetMaskValue

`MaximumEntropyThresholdImageFilter_SetMaskValue` = function(self, MaskValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskValue = as.integer(MaskValue);
  
  if(length(MaskValue) > 1) {
    warning("using only the first element of MaskValue");
  };
  
  ;ans = .Call('R_swig_MaximumEntropyThresholdImageFilter_SetMaskValue', self, MaskValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MaximumEntropyThresholdImageFilter_SetMaskValue`, 'returnType') = 'void'
attr(`MaximumEntropyThresholdImageFilter_SetMaskValue`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter', 'integer')
class(`MaximumEntropyThresholdImageFilter_SetMaskValue`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_SetMaskValue'))

# Start of MaximumEntropyThresholdImageFilter_GetMaskValue

`MaximumEntropyThresholdImageFilter_GetMaskValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaximumEntropyThresholdImageFilter_GetMaskValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaximumEntropyThresholdImageFilter_GetMaskValue`, 'returnType') = 'integer'
attr(`MaximumEntropyThresholdImageFilter_GetMaskValue`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter')
class(`MaximumEntropyThresholdImageFilter_GetMaskValue`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_GetMaskValue'))

# Start of MaximumEntropyThresholdImageFilter_GetThreshold

`MaximumEntropyThresholdImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaximumEntropyThresholdImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaximumEntropyThresholdImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`MaximumEntropyThresholdImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter')
class(`MaximumEntropyThresholdImageFilter_GetThreshold`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_GetThreshold'))

# Start of MaximumEntropyThresholdImageFilter_GetName

`MaximumEntropyThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaximumEntropyThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaximumEntropyThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`MaximumEntropyThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter')
class(`MaximumEntropyThresholdImageFilter_GetName`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_GetName'))

# Start of MaximumEntropyThresholdImageFilter_ToString

`MaximumEntropyThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaximumEntropyThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaximumEntropyThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`MaximumEntropyThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter')
class(`MaximumEntropyThresholdImageFilter_ToString`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_ToString'))

# Start of MaximumEntropyThresholdImageFilter_Execute

`MaximumEntropyThresholdImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_MaximumEntropyThresholdImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumEntropyThresholdImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumEntropyThresholdImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MaximumEntropyThresholdImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_Execute__SWIG_0'))

# Start of MaximumEntropyThresholdImageFilter_Execute

`MaximumEntropyThresholdImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_MaximumEntropyThresholdImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumEntropyThresholdImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumEntropyThresholdImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__MaximumEntropyThresholdImageFilter', '_p_itk__simple__Image')
class(`MaximumEntropyThresholdImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('MaximumEntropyThresholdImageFilter_Execute__SWIG_1'))

`MaximumEntropyThresholdImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__MaximumEntropyThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- MaximumEntropyThresholdImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__MaximumEntropyThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MaximumEntropyThresholdImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MaximumEntropyThresholdImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::MaximumEntropyThresholdImageFilter
setMethod('$', '_p_itk__simple__MaximumEntropyThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetInsideValue' = MaximumEntropyThresholdImageFilter_SetInsideValue, 'GetInsideValue' = MaximumEntropyThresholdImageFilter_GetInsideValue, 'SetOutsideValue' = MaximumEntropyThresholdImageFilter_SetOutsideValue, 'GetOutsideValue' = MaximumEntropyThresholdImageFilter_GetOutsideValue, 'SetNumberOfHistogramBins' = MaximumEntropyThresholdImageFilter_SetNumberOfHistogramBins, 'GetNumberOfHistogramBins' = MaximumEntropyThresholdImageFilter_GetNumberOfHistogramBins, 'SetMaskOutput' = MaximumEntropyThresholdImageFilter_SetMaskOutput, 'MaskOutputOn' = MaximumEntropyThresholdImageFilter_MaskOutputOn, 'MaskOutputOff' = MaximumEntropyThresholdImageFilter_MaskOutputOff, 'GetMaskOutput' = MaximumEntropyThresholdImageFilter_GetMaskOutput, 'SetMaskValue' = MaximumEntropyThresholdImageFilter_SetMaskValue, 'GetMaskValue' = MaximumEntropyThresholdImageFilter_GetMaskValue, 'GetThreshold' = MaximumEntropyThresholdImageFilter_GetThreshold, 'GetName' = MaximumEntropyThresholdImageFilter_GetName, 'ToString' = MaximumEntropyThresholdImageFilter_ToString, 'Execute' = MaximumEntropyThresholdImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MaximumEntropyThresholdImageFilter
# Start of MaximumEntropyThreshold

`MaximumEntropyThreshold__SWIG_0` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_MaximumEntropyThreshold__SWIG_0', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumEntropyThreshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumEntropyThreshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`MaximumEntropyThreshold__SWIG_0`) = c("SWIGFunction", class('MaximumEntropyThreshold__SWIG_0'))

# Start of MaximumEntropyThreshold

`MaximumEntropyThreshold__SWIG_1` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_MaximumEntropyThreshold__SWIG_1', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumEntropyThreshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumEntropyThreshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`MaximumEntropyThreshold__SWIG_1`) = c("SWIGFunction", class('MaximumEntropyThreshold__SWIG_1'))

# Start of MaximumEntropyThreshold

`MaximumEntropyThreshold__SWIG_2` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_MaximumEntropyThreshold__SWIG_2', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumEntropyThreshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumEntropyThreshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`MaximumEntropyThreshold__SWIG_2`) = c("SWIGFunction", class('MaximumEntropyThreshold__SWIG_2'))

# Start of MaximumEntropyThreshold

`MaximumEntropyThreshold__SWIG_3` = function(image, maskImage, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_MaximumEntropyThreshold__SWIG_3', image, maskImage, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumEntropyThreshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumEntropyThreshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`MaximumEntropyThreshold__SWIG_3`) = c("SWIGFunction", class('MaximumEntropyThreshold__SWIG_3'))

# Start of MaximumEntropyThreshold

`MaximumEntropyThreshold__SWIG_4` = function(image, maskImage, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_MaximumEntropyThreshold__SWIG_4', image, maskImage, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumEntropyThreshold__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumEntropyThreshold__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`MaximumEntropyThreshold__SWIG_4`) = c("SWIGFunction", class('MaximumEntropyThreshold__SWIG_4'))

# Start of MaximumEntropyThreshold

`MaximumEntropyThreshold__SWIG_5` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_MaximumEntropyThreshold__SWIG_5', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumEntropyThreshold__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumEntropyThreshold__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MaximumEntropyThreshold__SWIG_5`) = c("SWIGFunction", class('MaximumEntropyThreshold__SWIG_5'))

# Start of MaximumEntropyThreshold

`MaximumEntropyThreshold__SWIG_6` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_MaximumEntropyThreshold__SWIG_6', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumEntropyThreshold__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumEntropyThreshold__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`MaximumEntropyThreshold__SWIG_6`) = c("SWIGFunction", class('MaximumEntropyThreshold__SWIG_6'))

# Start of MaximumEntropyThreshold

`MaximumEntropyThreshold__SWIG_7` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_MaximumEntropyThreshold__SWIG_7', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumEntropyThreshold__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumEntropyThreshold__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`MaximumEntropyThreshold__SWIG_7`) = c("SWIGFunction", class('MaximumEntropyThreshold__SWIG_7'))

# Start of MaximumEntropyThreshold

`MaximumEntropyThreshold__SWIG_8` = function(image, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_MaximumEntropyThreshold__SWIG_8', image, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumEntropyThreshold__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumEntropyThreshold__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`MaximumEntropyThreshold__SWIG_8`) = c("SWIGFunction", class('MaximumEntropyThreshold__SWIG_8'))

# Start of MaximumEntropyThreshold

`MaximumEntropyThreshold__SWIG_9` = function(image, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_MaximumEntropyThreshold__SWIG_9', image, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumEntropyThreshold__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumEntropyThreshold__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`MaximumEntropyThreshold__SWIG_9`) = c("SWIGFunction", class('MaximumEntropyThreshold__SWIG_9'))

# Start of MaximumEntropyThreshold

`MaximumEntropyThreshold__SWIG_10` = function(image, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_MaximumEntropyThreshold__SWIG_10', image, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumEntropyThreshold__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumEntropyThreshold__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`MaximumEntropyThreshold__SWIG_10`) = c("SWIGFunction", class('MaximumEntropyThreshold__SWIG_10'))

# Start of MaximumEntropyThreshold

`MaximumEntropyThreshold__SWIG_11` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_MaximumEntropyThreshold__SWIG_11', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumEntropyThreshold__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumEntropyThreshold__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image')
class(`MaximumEntropyThreshold__SWIG_11`) = c("SWIGFunction", class('MaximumEntropyThreshold__SWIG_11'))

`MaximumEntropyThreshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 12
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- MaximumEntropyThreshold__SWIG_11; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- MaximumEntropyThreshold__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- MaximumEntropyThreshold__SWIG_10; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- MaximumEntropyThreshold__SWIG_4; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- MaximumEntropyThreshold__SWIG_9; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- MaximumEntropyThreshold__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- MaximumEntropyThreshold__SWIG_8; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- MaximumEntropyThreshold__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- MaximumEntropyThreshold__SWIG_7; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- MaximumEntropyThreshold__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- MaximumEntropyThreshold__SWIG_6; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- MaximumEntropyThreshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MaximumEntropyThreshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MaximumImageFilter

`delete_MaximumImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MaximumImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MaximumImageFilter`, 'returnType') = 'void'
attr(`delete_MaximumImageFilter`, "inputTypes") = c('_p_itk__simple__MaximumImageFilter')
class(`delete_MaximumImageFilter`) = c("SWIGFunction", class('delete_MaximumImageFilter'))

# Start of new_MaximumImageFilter

`MaximumImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MaximumImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MaximumImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MaximumImageFilter);
  ans
  
}

attr(`MaximumImageFilter`, 'returnType') = '_p_itk__simple__MaximumImageFilter'
class(`MaximumImageFilter`) = c("SWIGFunction", class('MaximumImageFilter'))

# Start of MaximumImageFilter_GetName

`MaximumImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaximumImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaximumImageFilter_GetName`, 'returnType') = 'character'
attr(`MaximumImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MaximumImageFilter')
class(`MaximumImageFilter_GetName`) = c("SWIGFunction", class('MaximumImageFilter_GetName'))

# Start of MaximumImageFilter_ToString

`MaximumImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaximumImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaximumImageFilter_ToString`, 'returnType') = 'character'
attr(`MaximumImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MaximumImageFilter')
class(`MaximumImageFilter_ToString`) = c("SWIGFunction", class('MaximumImageFilter_ToString'))

# Start of MaximumImageFilter_Execute

`MaximumImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_MaximumImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__MaximumImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MaximumImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('MaximumImageFilter_Execute__SWIG_0'))

# Start of MaximumImageFilter_Execute

`MaximumImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_MaximumImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__MaximumImageFilter', '_p_itk__simple__Image', 'numeric')
class(`MaximumImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('MaximumImageFilter_Execute__SWIG_1'))

# Start of MaximumImageFilter_Execute

`MaximumImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_MaximumImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__MaximumImageFilter', 'numeric', '_p_itk__simple__Image')
class(`MaximumImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('MaximumImageFilter_Execute__SWIG_2'))

`MaximumImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__MaximumImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MaximumImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__MaximumImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- MaximumImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__MaximumImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MaximumImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MaximumImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::MaximumImageFilter
setMethod('$', '_p_itk__simple__MaximumImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = MaximumImageFilter_GetName, 'ToString' = MaximumImageFilter_ToString, 'Execute' = MaximumImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MaximumImageFilter
# Start of Maximum

`Maximum__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Maximum__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Maximum__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Maximum__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Maximum__SWIG_0`) = c("SWIGFunction", class('Maximum__SWIG_0'))

# Start of Maximum

`Maximum__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Maximum__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Maximum__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Maximum__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`Maximum__SWIG_1`) = c("SWIGFunction", class('Maximum__SWIG_1'))

# Start of Maximum

`Maximum__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Maximum__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Maximum__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Maximum__SWIG_2`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`Maximum__SWIG_2`) = c("SWIGFunction", class('Maximum__SWIG_2'))

`Maximum` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Maximum__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Maximum__SWIG_1; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Maximum__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Maximum with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MaximumProjectionImageFilter

`delete_MaximumProjectionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MaximumProjectionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MaximumProjectionImageFilter`, 'returnType') = 'void'
attr(`delete_MaximumProjectionImageFilter`, "inputTypes") = c('_p_itk__simple__MaximumProjectionImageFilter')
class(`delete_MaximumProjectionImageFilter`) = c("SWIGFunction", class('delete_MaximumProjectionImageFilter'))

# Start of new_MaximumProjectionImageFilter

`MaximumProjectionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MaximumProjectionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MaximumProjectionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MaximumProjectionImageFilter);
  ans
  
}

attr(`MaximumProjectionImageFilter`, 'returnType') = '_p_itk__simple__MaximumProjectionImageFilter'
class(`MaximumProjectionImageFilter`) = c("SWIGFunction", class('MaximumProjectionImageFilter'))

# Start of MaximumProjectionImageFilter_SetProjectionDimension

`MaximumProjectionImageFilter_SetProjectionDimension` = function(self, ProjectionDimension)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ProjectionDimension = as.integer(ProjectionDimension);
  
  if(length(ProjectionDimension) > 1) {
    warning("using only the first element of ProjectionDimension");
  };
  
  ;ans = .Call('R_swig_MaximumProjectionImageFilter_SetProjectionDimension', self, ProjectionDimension, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MaximumProjectionImageFilter_SetProjectionDimension`, 'returnType') = 'void'
attr(`MaximumProjectionImageFilter_SetProjectionDimension`, "inputTypes") = c('_p_itk__simple__MaximumProjectionImageFilter', 'integer')
class(`MaximumProjectionImageFilter_SetProjectionDimension`) = c("SWIGFunction", class('MaximumProjectionImageFilter_SetProjectionDimension'))

# Start of MaximumProjectionImageFilter_GetProjectionDimension

`MaximumProjectionImageFilter_GetProjectionDimension` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaximumProjectionImageFilter_GetProjectionDimension', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaximumProjectionImageFilter_GetProjectionDimension`, 'returnType') = 'integer'
attr(`MaximumProjectionImageFilter_GetProjectionDimension`, "inputTypes") = c('_p_itk__simple__MaximumProjectionImageFilter')
class(`MaximumProjectionImageFilter_GetProjectionDimension`) = c("SWIGFunction", class('MaximumProjectionImageFilter_GetProjectionDimension'))

# Start of MaximumProjectionImageFilter_GetName

`MaximumProjectionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaximumProjectionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaximumProjectionImageFilter_GetName`, 'returnType') = 'character'
attr(`MaximumProjectionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MaximumProjectionImageFilter')
class(`MaximumProjectionImageFilter_GetName`) = c("SWIGFunction", class('MaximumProjectionImageFilter_GetName'))

# Start of MaximumProjectionImageFilter_ToString

`MaximumProjectionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MaximumProjectionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MaximumProjectionImageFilter_ToString`, 'returnType') = 'character'
attr(`MaximumProjectionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MaximumProjectionImageFilter')
class(`MaximumProjectionImageFilter_ToString`) = c("SWIGFunction", class('MaximumProjectionImageFilter_ToString'))

# Start of MaximumProjectionImageFilter_Execute

`MaximumProjectionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MaximumProjectionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumProjectionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumProjectionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__MaximumProjectionImageFilter', '_p_itk__simple__Image')
class(`MaximumProjectionImageFilter_Execute`) = c("SWIGFunction", class('MaximumProjectionImageFilter_Execute'))

# Start of accessor method for itk::simple::MaximumProjectionImageFilter
setMethod('$', '_p_itk__simple__MaximumProjectionImageFilter', function(x, name)

{
  accessorFuns = list('SetProjectionDimension' = MaximumProjectionImageFilter_SetProjectionDimension, 'GetProjectionDimension' = MaximumProjectionImageFilter_GetProjectionDimension, 'GetName' = MaximumProjectionImageFilter_GetName, 'ToString' = MaximumProjectionImageFilter_ToString, 'Execute' = MaximumProjectionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MaximumProjectionImageFilter
# Start of MaximumProjection

`MaximumProjection__SWIG_0` = function(image1, projectionDimension, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  projectionDimension = as.integer(projectionDimension);
  
  if(length(projectionDimension) > 1) {
    warning("using only the first element of projectionDimension");
  };
  
  ;ans = .Call('R_swig_MaximumProjection__SWIG_0', image1, projectionDimension, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumProjection__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumProjection__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`MaximumProjection__SWIG_0`) = c("SWIGFunction", class('MaximumProjection__SWIG_0'))

# Start of MaximumProjection

`MaximumProjection__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MaximumProjection__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MaximumProjection__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MaximumProjection__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`MaximumProjection__SWIG_1`) = c("SWIGFunction", class('MaximumProjection__SWIG_1'))

`MaximumProjection` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- MaximumProjection__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- MaximumProjection__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MaximumProjection with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MeanImageFilter

`delete_MeanImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MeanImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MeanImageFilter`, 'returnType') = 'void'
attr(`delete_MeanImageFilter`, "inputTypes") = c('_p_itk__simple__MeanImageFilter')
class(`delete_MeanImageFilter`) = c("SWIGFunction", class('delete_MeanImageFilter'))

# Start of new_MeanImageFilter

`MeanImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MeanImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MeanImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MeanImageFilter);
  ans
  
}

attr(`MeanImageFilter`, 'returnType') = '_p_itk__simple__MeanImageFilter'
class(`MeanImageFilter`) = c("SWIGFunction", class('MeanImageFilter'))

# Start of MeanImageFilter_SetRadius

`MeanImageFilter_SetRadius__SWIG_0` = function(self, Radius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Radius = as.integer(Radius);
  ;ans = .Call('R_swig_MeanImageFilter_SetRadius__SWIG_0', self, Radius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MeanImageFilter_SetRadius__SWIG_0`, 'returnType') = 'void'
attr(`MeanImageFilter_SetRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__MeanImageFilter', 'integer')
class(`MeanImageFilter_SetRadius__SWIG_0`) = c("SWIGFunction", class('MeanImageFilter_SetRadius__SWIG_0'))

# Start of MeanImageFilter_SetRadius

`MeanImageFilter_SetRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_MeanImageFilter_SetRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MeanImageFilter_SetRadius__SWIG_1`, 'returnType') = 'void'
attr(`MeanImageFilter_SetRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__MeanImageFilter', 'integer')
class(`MeanImageFilter_SetRadius__SWIG_1`) = c("SWIGFunction", class('MeanImageFilter_SetRadius__SWIG_1'))

`MeanImageFilter_SetRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__MeanImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- MeanImageFilter_SetRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__MeanImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- MeanImageFilter_SetRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MeanImageFilter_SetRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of MeanImageFilter_GetRadius

`MeanImageFilter_GetRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MeanImageFilter_GetRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MeanImageFilter_GetRadius`, 'returnType') = 'integer'
attr(`MeanImageFilter_GetRadius`, "inputTypes") = c('_p_itk__simple__MeanImageFilter')
class(`MeanImageFilter_GetRadius`) = c("SWIGFunction", class('MeanImageFilter_GetRadius'))

# Start of MeanImageFilter_GetName

`MeanImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MeanImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MeanImageFilter_GetName`, 'returnType') = 'character'
attr(`MeanImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MeanImageFilter')
class(`MeanImageFilter_GetName`) = c("SWIGFunction", class('MeanImageFilter_GetName'))

# Start of MeanImageFilter_ToString

`MeanImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MeanImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MeanImageFilter_ToString`, 'returnType') = 'character'
attr(`MeanImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MeanImageFilter')
class(`MeanImageFilter_ToString`) = c("SWIGFunction", class('MeanImageFilter_ToString'))

# Start of MeanImageFilter_Execute

`MeanImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MeanImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MeanImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`MeanImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__MeanImageFilter', '_p_itk__simple__Image')
class(`MeanImageFilter_Execute`) = c("SWIGFunction", class('MeanImageFilter_Execute'))

# Start of accessor method for itk::simple::MeanImageFilter
setMethod('$', '_p_itk__simple__MeanImageFilter', function(x, name)

{
  accessorFuns = list('SetRadius' = MeanImageFilter_SetRadius, 'GetRadius' = MeanImageFilter_GetRadius, 'GetName' = MeanImageFilter_GetName, 'ToString' = MeanImageFilter_ToString, 'Execute' = MeanImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MeanImageFilter
# Start of Mean

`Mean__SWIG_0` = function(image1, radius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  ;ans = .Call('R_swig_Mean__SWIG_0', image1, radius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Mean__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Mean__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Mean__SWIG_0`) = c("SWIGFunction", class('Mean__SWIG_0'))

# Start of Mean

`Mean__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Mean__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Mean__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Mean__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`Mean__SWIG_1`) = c("SWIGFunction", class('Mean__SWIG_1'))

`Mean` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Mean__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- Mean__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Mean with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MeanProjectionImageFilter

`delete_MeanProjectionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MeanProjectionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MeanProjectionImageFilter`, 'returnType') = 'void'
attr(`delete_MeanProjectionImageFilter`, "inputTypes") = c('_p_itk__simple__MeanProjectionImageFilter')
class(`delete_MeanProjectionImageFilter`) = c("SWIGFunction", class('delete_MeanProjectionImageFilter'))

# Start of new_MeanProjectionImageFilter

`MeanProjectionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MeanProjectionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MeanProjectionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MeanProjectionImageFilter);
  ans
  
}

attr(`MeanProjectionImageFilter`, 'returnType') = '_p_itk__simple__MeanProjectionImageFilter'
class(`MeanProjectionImageFilter`) = c("SWIGFunction", class('MeanProjectionImageFilter'))

# Start of MeanProjectionImageFilter_SetProjectionDimension

`MeanProjectionImageFilter_SetProjectionDimension` = function(self, ProjectionDimension)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ProjectionDimension = as.integer(ProjectionDimension);
  
  if(length(ProjectionDimension) > 1) {
    warning("using only the first element of ProjectionDimension");
  };
  
  ;ans = .Call('R_swig_MeanProjectionImageFilter_SetProjectionDimension', self, ProjectionDimension, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MeanProjectionImageFilter_SetProjectionDimension`, 'returnType') = 'void'
attr(`MeanProjectionImageFilter_SetProjectionDimension`, "inputTypes") = c('_p_itk__simple__MeanProjectionImageFilter', 'integer')
class(`MeanProjectionImageFilter_SetProjectionDimension`) = c("SWIGFunction", class('MeanProjectionImageFilter_SetProjectionDimension'))

# Start of MeanProjectionImageFilter_GetProjectionDimension

`MeanProjectionImageFilter_GetProjectionDimension` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MeanProjectionImageFilter_GetProjectionDimension', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MeanProjectionImageFilter_GetProjectionDimension`, 'returnType') = 'integer'
attr(`MeanProjectionImageFilter_GetProjectionDimension`, "inputTypes") = c('_p_itk__simple__MeanProjectionImageFilter')
class(`MeanProjectionImageFilter_GetProjectionDimension`) = c("SWIGFunction", class('MeanProjectionImageFilter_GetProjectionDimension'))

# Start of MeanProjectionImageFilter_GetName

`MeanProjectionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MeanProjectionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MeanProjectionImageFilter_GetName`, 'returnType') = 'character'
attr(`MeanProjectionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MeanProjectionImageFilter')
class(`MeanProjectionImageFilter_GetName`) = c("SWIGFunction", class('MeanProjectionImageFilter_GetName'))

# Start of MeanProjectionImageFilter_ToString

`MeanProjectionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MeanProjectionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MeanProjectionImageFilter_ToString`, 'returnType') = 'character'
attr(`MeanProjectionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MeanProjectionImageFilter')
class(`MeanProjectionImageFilter_ToString`) = c("SWIGFunction", class('MeanProjectionImageFilter_ToString'))

# Start of MeanProjectionImageFilter_Execute

`MeanProjectionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MeanProjectionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MeanProjectionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`MeanProjectionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__MeanProjectionImageFilter', '_p_itk__simple__Image')
class(`MeanProjectionImageFilter_Execute`) = c("SWIGFunction", class('MeanProjectionImageFilter_Execute'))

# Start of accessor method for itk::simple::MeanProjectionImageFilter
setMethod('$', '_p_itk__simple__MeanProjectionImageFilter', function(x, name)

{
  accessorFuns = list('SetProjectionDimension' = MeanProjectionImageFilter_SetProjectionDimension, 'GetProjectionDimension' = MeanProjectionImageFilter_GetProjectionDimension, 'GetName' = MeanProjectionImageFilter_GetName, 'ToString' = MeanProjectionImageFilter_ToString, 'Execute' = MeanProjectionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MeanProjectionImageFilter
# Start of MeanProjection

`MeanProjection__SWIG_0` = function(image1, projectionDimension, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  projectionDimension = as.integer(projectionDimension);
  
  if(length(projectionDimension) > 1) {
    warning("using only the first element of projectionDimension");
  };
  
  ;ans = .Call('R_swig_MeanProjection__SWIG_0', image1, projectionDimension, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MeanProjection__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MeanProjection__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`MeanProjection__SWIG_0`) = c("SWIGFunction", class('MeanProjection__SWIG_0'))

# Start of MeanProjection

`MeanProjection__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MeanProjection__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MeanProjection__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MeanProjection__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`MeanProjection__SWIG_1`) = c("SWIGFunction", class('MeanProjection__SWIG_1'))

`MeanProjection` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- MeanProjection__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- MeanProjection__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MeanProjection with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MedianImageFilter

`delete_MedianImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MedianImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MedianImageFilter`, 'returnType') = 'void'
attr(`delete_MedianImageFilter`, "inputTypes") = c('_p_itk__simple__MedianImageFilter')
class(`delete_MedianImageFilter`) = c("SWIGFunction", class('delete_MedianImageFilter'))

# Start of new_MedianImageFilter

`MedianImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MedianImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MedianImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MedianImageFilter);
  ans
  
}

attr(`MedianImageFilter`, 'returnType') = '_p_itk__simple__MedianImageFilter'
class(`MedianImageFilter`) = c("SWIGFunction", class('MedianImageFilter'))

# Start of MedianImageFilter_SetRadius

`MedianImageFilter_SetRadius__SWIG_0` = function(self, Radius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Radius = as.integer(Radius);
  ;ans = .Call('R_swig_MedianImageFilter_SetRadius__SWIG_0', self, Radius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MedianImageFilter_SetRadius__SWIG_0`, 'returnType') = 'void'
attr(`MedianImageFilter_SetRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__MedianImageFilter', 'integer')
class(`MedianImageFilter_SetRadius__SWIG_0`) = c("SWIGFunction", class('MedianImageFilter_SetRadius__SWIG_0'))

# Start of MedianImageFilter_SetRadius

`MedianImageFilter_SetRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_MedianImageFilter_SetRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MedianImageFilter_SetRadius__SWIG_1`, 'returnType') = 'void'
attr(`MedianImageFilter_SetRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__MedianImageFilter', 'integer')
class(`MedianImageFilter_SetRadius__SWIG_1`) = c("SWIGFunction", class('MedianImageFilter_SetRadius__SWIG_1'))

`MedianImageFilter_SetRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__MedianImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- MedianImageFilter_SetRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__MedianImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- MedianImageFilter_SetRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MedianImageFilter_SetRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of MedianImageFilter_GetRadius

`MedianImageFilter_GetRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MedianImageFilter_GetRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MedianImageFilter_GetRadius`, 'returnType') = 'integer'
attr(`MedianImageFilter_GetRadius`, "inputTypes") = c('_p_itk__simple__MedianImageFilter')
class(`MedianImageFilter_GetRadius`) = c("SWIGFunction", class('MedianImageFilter_GetRadius'))

# Start of MedianImageFilter_GetName

`MedianImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MedianImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MedianImageFilter_GetName`, 'returnType') = 'character'
attr(`MedianImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MedianImageFilter')
class(`MedianImageFilter_GetName`) = c("SWIGFunction", class('MedianImageFilter_GetName'))

# Start of MedianImageFilter_ToString

`MedianImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MedianImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MedianImageFilter_ToString`, 'returnType') = 'character'
attr(`MedianImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MedianImageFilter')
class(`MedianImageFilter_ToString`) = c("SWIGFunction", class('MedianImageFilter_ToString'))

# Start of MedianImageFilter_Execute

`MedianImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MedianImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MedianImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`MedianImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__MedianImageFilter', '_p_itk__simple__Image')
class(`MedianImageFilter_Execute`) = c("SWIGFunction", class('MedianImageFilter_Execute'))

# Start of accessor method for itk::simple::MedianImageFilter
setMethod('$', '_p_itk__simple__MedianImageFilter', function(x, name)

{
  accessorFuns = list('SetRadius' = MedianImageFilter_SetRadius, 'GetRadius' = MedianImageFilter_GetRadius, 'GetName' = MedianImageFilter_GetName, 'ToString' = MedianImageFilter_ToString, 'Execute' = MedianImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MedianImageFilter
# Start of Median

`Median__SWIG_0` = function(image1, radius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  ;ans = .Call('R_swig_Median__SWIG_0', image1, radius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Median__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Median__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Median__SWIG_0`) = c("SWIGFunction", class('Median__SWIG_0'))

# Start of Median

`Median__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Median__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Median__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Median__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`Median__SWIG_1`) = c("SWIGFunction", class('Median__SWIG_1'))

`Median` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Median__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- Median__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Median with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MedianProjectionImageFilter

`delete_MedianProjectionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MedianProjectionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MedianProjectionImageFilter`, 'returnType') = 'void'
attr(`delete_MedianProjectionImageFilter`, "inputTypes") = c('_p_itk__simple__MedianProjectionImageFilter')
class(`delete_MedianProjectionImageFilter`) = c("SWIGFunction", class('delete_MedianProjectionImageFilter'))

# Start of new_MedianProjectionImageFilter

`MedianProjectionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MedianProjectionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MedianProjectionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MedianProjectionImageFilter);
  ans
  
}

attr(`MedianProjectionImageFilter`, 'returnType') = '_p_itk__simple__MedianProjectionImageFilter'
class(`MedianProjectionImageFilter`) = c("SWIGFunction", class('MedianProjectionImageFilter'))

# Start of MedianProjectionImageFilter_SetProjectionDimension

`MedianProjectionImageFilter_SetProjectionDimension` = function(self, ProjectionDimension)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ProjectionDimension = as.integer(ProjectionDimension);
  
  if(length(ProjectionDimension) > 1) {
    warning("using only the first element of ProjectionDimension");
  };
  
  ;ans = .Call('R_swig_MedianProjectionImageFilter_SetProjectionDimension', self, ProjectionDimension, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MedianProjectionImageFilter_SetProjectionDimension`, 'returnType') = 'void'
attr(`MedianProjectionImageFilter_SetProjectionDimension`, "inputTypes") = c('_p_itk__simple__MedianProjectionImageFilter', 'integer')
class(`MedianProjectionImageFilter_SetProjectionDimension`) = c("SWIGFunction", class('MedianProjectionImageFilter_SetProjectionDimension'))

# Start of MedianProjectionImageFilter_GetProjectionDimension

`MedianProjectionImageFilter_GetProjectionDimension` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MedianProjectionImageFilter_GetProjectionDimension', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MedianProjectionImageFilter_GetProjectionDimension`, 'returnType') = 'integer'
attr(`MedianProjectionImageFilter_GetProjectionDimension`, "inputTypes") = c('_p_itk__simple__MedianProjectionImageFilter')
class(`MedianProjectionImageFilter_GetProjectionDimension`) = c("SWIGFunction", class('MedianProjectionImageFilter_GetProjectionDimension'))

# Start of MedianProjectionImageFilter_GetName

`MedianProjectionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MedianProjectionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MedianProjectionImageFilter_GetName`, 'returnType') = 'character'
attr(`MedianProjectionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MedianProjectionImageFilter')
class(`MedianProjectionImageFilter_GetName`) = c("SWIGFunction", class('MedianProjectionImageFilter_GetName'))

# Start of MedianProjectionImageFilter_ToString

`MedianProjectionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MedianProjectionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MedianProjectionImageFilter_ToString`, 'returnType') = 'character'
attr(`MedianProjectionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MedianProjectionImageFilter')
class(`MedianProjectionImageFilter_ToString`) = c("SWIGFunction", class('MedianProjectionImageFilter_ToString'))

# Start of MedianProjectionImageFilter_Execute

`MedianProjectionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MedianProjectionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MedianProjectionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`MedianProjectionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__MedianProjectionImageFilter', '_p_itk__simple__Image')
class(`MedianProjectionImageFilter_Execute`) = c("SWIGFunction", class('MedianProjectionImageFilter_Execute'))

# Start of accessor method for itk::simple::MedianProjectionImageFilter
setMethod('$', '_p_itk__simple__MedianProjectionImageFilter', function(x, name)

{
  accessorFuns = list('SetProjectionDimension' = MedianProjectionImageFilter_SetProjectionDimension, 'GetProjectionDimension' = MedianProjectionImageFilter_GetProjectionDimension, 'GetName' = MedianProjectionImageFilter_GetName, 'ToString' = MedianProjectionImageFilter_ToString, 'Execute' = MedianProjectionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MedianProjectionImageFilter
# Start of MedianProjection

`MedianProjection__SWIG_0` = function(image1, projectionDimension, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  projectionDimension = as.integer(projectionDimension);
  
  if(length(projectionDimension) > 1) {
    warning("using only the first element of projectionDimension");
  };
  
  ;ans = .Call('R_swig_MedianProjection__SWIG_0', image1, projectionDimension, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MedianProjection__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MedianProjection__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`MedianProjection__SWIG_0`) = c("SWIGFunction", class('MedianProjection__SWIG_0'))

# Start of MedianProjection

`MedianProjection__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MedianProjection__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MedianProjection__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MedianProjection__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`MedianProjection__SWIG_1`) = c("SWIGFunction", class('MedianProjection__SWIG_1'))

`MedianProjection` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- MedianProjection__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- MedianProjection__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MedianProjection with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MergeLabelMapFilter

`delete_MergeLabelMapFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MergeLabelMapFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MergeLabelMapFilter`, 'returnType') = 'void'
attr(`delete_MergeLabelMapFilter`, "inputTypes") = c('_p_itk__simple__MergeLabelMapFilter')
class(`delete_MergeLabelMapFilter`) = c("SWIGFunction", class('delete_MergeLabelMapFilter'))

# Start of new_MergeLabelMapFilter

`MergeLabelMapFilter` = function()
{
  ;ans = .Call('R_swig_new_MergeLabelMapFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MergeLabelMapFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MergeLabelMapFilter);
  ans
  
}

attr(`MergeLabelMapFilter`, 'returnType') = '_p_itk__simple__MergeLabelMapFilter'
class(`MergeLabelMapFilter`) = c("SWIGFunction", class('MergeLabelMapFilter'))

# Start of MergeLabelMapFilter_Keep_get

`MergeLabelMapFilter_Keep_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_MergeLabelMapFilter_Keep_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__MergeLabelMapFilter__MethodType");
  
  ans
  
}

attr(`MergeLabelMapFilter_Keep_get`, 'returnType') = 'character'
class(`MergeLabelMapFilter_Keep_get`) = c("SWIGFunction", class('MergeLabelMapFilter_Keep_get'))

# Start of MergeLabelMapFilter_Aggregate_get

`MergeLabelMapFilter_Aggregate_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_MergeLabelMapFilter_Aggregate_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__MergeLabelMapFilter__MethodType");
  
  ans
  
}

attr(`MergeLabelMapFilter_Aggregate_get`, 'returnType') = 'character'
class(`MergeLabelMapFilter_Aggregate_get`) = c("SWIGFunction", class('MergeLabelMapFilter_Aggregate_get'))

# Start of MergeLabelMapFilter_Pack_get

`MergeLabelMapFilter_Pack_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_MergeLabelMapFilter_Pack_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__MergeLabelMapFilter__MethodType");
  
  ans
  
}

attr(`MergeLabelMapFilter_Pack_get`, 'returnType') = 'character'
class(`MergeLabelMapFilter_Pack_get`) = c("SWIGFunction", class('MergeLabelMapFilter_Pack_get'))

# Start of MergeLabelMapFilter_Strict_get

`MergeLabelMapFilter_Strict_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_MergeLabelMapFilter_Strict_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__MergeLabelMapFilter__MethodType");
  
  ans
  
}

attr(`MergeLabelMapFilter_Strict_get`, 'returnType') = 'character'
class(`MergeLabelMapFilter_Strict_get`) = c("SWIGFunction", class('MergeLabelMapFilter_Strict_get'))

# Start of MergeLabelMapFilter_SetMethod

`MergeLabelMapFilter_SetMethod` = function(self, Method)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Method = enumToInteger(Method, "_itk__simple__MergeLabelMapFilter__MethodType"); 
  
  if(length(Method) > 1) {
    warning("using only the first element of Method");
  };
  
  ;ans = .Call('R_swig_MergeLabelMapFilter_SetMethod', self, Method, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MergeLabelMapFilter_SetMethod`, 'returnType') = 'void'
attr(`MergeLabelMapFilter_SetMethod`, "inputTypes") = c('_p_itk__simple__MergeLabelMapFilter', 'character')
class(`MergeLabelMapFilter_SetMethod`) = c("SWIGFunction", class('MergeLabelMapFilter_SetMethod'))

# Start of MergeLabelMapFilter_GetMethod

`MergeLabelMapFilter_GetMethod` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_MergeLabelMapFilter_GetMethod', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__MergeLabelMapFilter__MethodType");
  
  ans
  
}

attr(`MergeLabelMapFilter_GetMethod`, 'returnType') = 'character'
attr(`MergeLabelMapFilter_GetMethod`, "inputTypes") = c('_p_itk__simple__MergeLabelMapFilter')
class(`MergeLabelMapFilter_GetMethod`) = c("SWIGFunction", class('MergeLabelMapFilter_GetMethod'))

# Start of MergeLabelMapFilter_GetName

`MergeLabelMapFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MergeLabelMapFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MergeLabelMapFilter_GetName`, 'returnType') = 'character'
attr(`MergeLabelMapFilter_GetName`, "inputTypes") = c('_p_itk__simple__MergeLabelMapFilter')
class(`MergeLabelMapFilter_GetName`) = c("SWIGFunction", class('MergeLabelMapFilter_GetName'))

# Start of MergeLabelMapFilter_ToString

`MergeLabelMapFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MergeLabelMapFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MergeLabelMapFilter_ToString`, 'returnType') = 'character'
attr(`MergeLabelMapFilter_ToString`, "inputTypes") = c('_p_itk__simple__MergeLabelMapFilter')
class(`MergeLabelMapFilter_ToString`) = c("SWIGFunction", class('MergeLabelMapFilter_ToString'))

# Start of MergeLabelMapFilter_Execute

`MergeLabelMapFilter_Execute__SWIG_0` = function(self, images, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_MergeLabelMapFilter_Execute__SWIG_0', self, images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMapFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMapFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__MergeLabelMapFilter', 'list')
class(`MergeLabelMapFilter_Execute__SWIG_0`) = c("SWIGFunction", class('MergeLabelMapFilter_Execute__SWIG_0'))

# Start of MergeLabelMapFilter_Execute

`MergeLabelMapFilter_Execute__SWIG_1` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MergeLabelMapFilter_Execute__SWIG_1', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMapFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMapFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__MergeLabelMapFilter', '_p_itk__simple__Image')
class(`MergeLabelMapFilter_Execute__SWIG_1`) = c("SWIGFunction", class('MergeLabelMapFilter_Execute__SWIG_1'))

# Start of MergeLabelMapFilter_Execute

`MergeLabelMapFilter_Execute__SWIG_2` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_MergeLabelMapFilter_Execute__SWIG_2', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMapFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMapFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__MergeLabelMapFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MergeLabelMapFilter_Execute__SWIG_2`) = c("SWIGFunction", class('MergeLabelMapFilter_Execute__SWIG_2'))

# Start of MergeLabelMapFilter_Execute

`MergeLabelMapFilter_Execute__SWIG_3` = function(self, image1, image2, image3, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_MergeLabelMapFilter_Execute__SWIG_3', self, image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMapFilter_Execute__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMapFilter_Execute__SWIG_3`, "inputTypes") = c('_p_itk__simple__MergeLabelMapFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MergeLabelMapFilter_Execute__SWIG_3`) = c("SWIGFunction", class('MergeLabelMapFilter_Execute__SWIG_3'))

# Start of MergeLabelMapFilter_Execute

`MergeLabelMapFilter_Execute__SWIG_4` = function(self, image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_MergeLabelMapFilter_Execute__SWIG_4', self, image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMapFilter_Execute__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMapFilter_Execute__SWIG_4`, "inputTypes") = c('_p_itk__simple__MergeLabelMapFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MergeLabelMapFilter_Execute__SWIG_4`) = c("SWIGFunction", class('MergeLabelMapFilter_Execute__SWIG_4'))

# Start of MergeLabelMapFilter_Execute

`MergeLabelMapFilter_Execute__SWIG_5` = function(self, image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_MergeLabelMapFilter_Execute__SWIG_5', self, image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMapFilter_Execute__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMapFilter_Execute__SWIG_5`, "inputTypes") = c('_p_itk__simple__MergeLabelMapFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MergeLabelMapFilter_Execute__SWIG_5`) = c("SWIGFunction", class('MergeLabelMapFilter_Execute__SWIG_5'))

`MergeLabelMapFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__MergeLabelMapFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- MergeLabelMapFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__MergeLabelMapFilter') || is.null(argv[[1]]) ) && (
        (is.list(argv[[2]]) & all(sapply(argv[[2]], inherits, what='_p_itk__simple__Image')))
        )) {
      f <- MergeLabelMapFilter_Execute__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__MergeLabelMapFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MergeLabelMapFilter_Execute__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__MergeLabelMapFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- MergeLabelMapFilter_Execute__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__MergeLabelMapFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- MergeLabelMapFilter_Execute__SWIG_4; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__MergeLabelMapFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( extends(argtypes[6], '_p_itk__simple__Image') && length(argv[[6]]) == 1 )) {
      f <- MergeLabelMapFilter_Execute__SWIG_5; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MergeLabelMapFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::MergeLabelMapFilter
setMethod('$', '_p_itk__simple__MergeLabelMapFilter', function(x, name)

{
  accessorFuns = list('SetMethod' = MergeLabelMapFilter_SetMethod, 'GetMethod' = MergeLabelMapFilter_GetMethod, 'GetName' = MergeLabelMapFilter_GetName, 'ToString' = MergeLabelMapFilter_ToString, 'Execute' = MergeLabelMapFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MergeLabelMapFilter
# Start of MergeLabelMap

`MergeLabelMap__SWIG_0` = function(images, method, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  method = enumToInteger(method, "_itk__simple__MergeLabelMapFilter__MethodType"); 
  
  if(length(method) > 1) {
    warning("using only the first element of method");
  };
  
  ;ans = .Call('R_swig_MergeLabelMap__SWIG_0', images, method, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMap__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMap__SWIG_0`, "inputTypes") = c('list', 'character')
class(`MergeLabelMap__SWIG_0`) = c("SWIGFunction", class('MergeLabelMap__SWIG_0'))

# Start of MergeLabelMap

`MergeLabelMap__SWIG_1` = function(images, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_MergeLabelMap__SWIG_1', images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMap__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMap__SWIG_1`, "inputTypes") = c('list')
class(`MergeLabelMap__SWIG_1`) = c("SWIGFunction", class('MergeLabelMap__SWIG_1'))

# Start of MergeLabelMap

`MergeLabelMap__SWIG_2` = function(image1, method, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  method = enumToInteger(method, "_itk__simple__MergeLabelMapFilter__MethodType"); 
  
  if(length(method) > 1) {
    warning("using only the first element of method");
  };
  
  ;ans = .Call('R_swig_MergeLabelMap__SWIG_2', image1, method, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMap__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMap__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'character')
class(`MergeLabelMap__SWIG_2`) = c("SWIGFunction", class('MergeLabelMap__SWIG_2'))

# Start of MergeLabelMap

`MergeLabelMap__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MergeLabelMap__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMap__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMap__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`MergeLabelMap__SWIG_3`) = c("SWIGFunction", class('MergeLabelMap__SWIG_3'))

# Start of MergeLabelMap

`MergeLabelMap__SWIG_4` = function(image1, image2, method, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  method = enumToInteger(method, "_itk__simple__MergeLabelMapFilter__MethodType"); 
  
  if(length(method) > 1) {
    warning("using only the first element of method");
  };
  
  ;ans = .Call('R_swig_MergeLabelMap__SWIG_4', image1, image2, method, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMap__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMap__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'character')
class(`MergeLabelMap__SWIG_4`) = c("SWIGFunction", class('MergeLabelMap__SWIG_4'))

# Start of MergeLabelMap

`MergeLabelMap__SWIG_5` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_MergeLabelMap__SWIG_5', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMap__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMap__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MergeLabelMap__SWIG_5`) = c("SWIGFunction", class('MergeLabelMap__SWIG_5'))

# Start of MergeLabelMap

`MergeLabelMap__SWIG_6` = function(image1, image2, image3, method, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  method = enumToInteger(method, "_itk__simple__MergeLabelMapFilter__MethodType"); 
  
  if(length(method) > 1) {
    warning("using only the first element of method");
  };
  
  ;ans = .Call('R_swig_MergeLabelMap__SWIG_6', image1, image2, image3, method, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMap__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMap__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'character')
class(`MergeLabelMap__SWIG_6`) = c("SWIGFunction", class('MergeLabelMap__SWIG_6'))

# Start of MergeLabelMap

`MergeLabelMap__SWIG_7` = function(image1, image2, image3, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_MergeLabelMap__SWIG_7', image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMap__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMap__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MergeLabelMap__SWIG_7`) = c("SWIGFunction", class('MergeLabelMap__SWIG_7'))

# Start of MergeLabelMap

`MergeLabelMap__SWIG_8` = function(image1, image2, image3, image4, method, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  method = enumToInteger(method, "_itk__simple__MergeLabelMapFilter__MethodType"); 
  
  if(length(method) > 1) {
    warning("using only the first element of method");
  };
  
  ;ans = .Call('R_swig_MergeLabelMap__SWIG_8', image1, image2, image3, image4, method, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMap__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMap__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'character')
class(`MergeLabelMap__SWIG_8`) = c("SWIGFunction", class('MergeLabelMap__SWIG_8'))

# Start of MergeLabelMap

`MergeLabelMap__SWIG_9` = function(image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_MergeLabelMap__SWIG_9', image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMap__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMap__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MergeLabelMap__SWIG_9`) = c("SWIGFunction", class('MergeLabelMap__SWIG_9'))

# Start of MergeLabelMap

`MergeLabelMap__SWIG_10` = function(image1, image2, image3, image4, image5, method, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  method = enumToInteger(method, "_itk__simple__MergeLabelMapFilter__MethodType"); 
  
  if(length(method) > 1) {
    warning("using only the first element of method");
  };
  
  ;ans = .Call('R_swig_MergeLabelMap__SWIG_10', image1, image2, image3, image4, image5, method, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMap__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMap__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'character')
class(`MergeLabelMap__SWIG_10`) = c("SWIGFunction", class('MergeLabelMap__SWIG_10'))

# Start of MergeLabelMap

`MergeLabelMap__SWIG_11` = function(image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_MergeLabelMap__SWIG_11', image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MergeLabelMap__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`MergeLabelMap__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MergeLabelMap__SWIG_11`) = c("SWIGFunction", class('MergeLabelMap__SWIG_11'))

`MergeLabelMap` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 12
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- MergeLabelMap__SWIG_3; 
    }
    else if (
      (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
      ) {
      f <- MergeLabelMap__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- MergeLabelMap__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- MergeLabelMap__SWIG_2; 
    }
    else if ((
        (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
        ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- MergeLabelMap__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MergeLabelMap__SWIG_7; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- MergeLabelMap__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- MergeLabelMap__SWIG_9; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- MergeLabelMap__SWIG_6; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- MergeLabelMap__SWIG_11; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( is.character(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- MergeLabelMap__SWIG_8; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( is.character(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- MergeLabelMap__SWIG_10; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MergeLabelMap with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MinMaxCurvatureFlowImageFilter

`delete_MinMaxCurvatureFlowImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MinMaxCurvatureFlowImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MinMaxCurvatureFlowImageFilter`, 'returnType') = 'void'
attr(`delete_MinMaxCurvatureFlowImageFilter`, "inputTypes") = c('_p_itk__simple__MinMaxCurvatureFlowImageFilter')
class(`delete_MinMaxCurvatureFlowImageFilter`) = c("SWIGFunction", class('delete_MinMaxCurvatureFlowImageFilter'))

# Start of new_MinMaxCurvatureFlowImageFilter

`MinMaxCurvatureFlowImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MinMaxCurvatureFlowImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MinMaxCurvatureFlowImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MinMaxCurvatureFlowImageFilter);
  ans
  
}

attr(`MinMaxCurvatureFlowImageFilter`, 'returnType') = '_p_itk__simple__MinMaxCurvatureFlowImageFilter'
class(`MinMaxCurvatureFlowImageFilter`) = c("SWIGFunction", class('MinMaxCurvatureFlowImageFilter'))

# Start of MinMaxCurvatureFlowImageFilter_SetTimeStep

`MinMaxCurvatureFlowImageFilter_SetTimeStep` = function(self, TimeStep)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_MinMaxCurvatureFlowImageFilter_SetTimeStep', self, TimeStep, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MinMaxCurvatureFlowImageFilter_SetTimeStep`, 'returnType') = 'void'
attr(`MinMaxCurvatureFlowImageFilter_SetTimeStep`, "inputTypes") = c('_p_itk__simple__MinMaxCurvatureFlowImageFilter', 'numeric')
class(`MinMaxCurvatureFlowImageFilter_SetTimeStep`) = c("SWIGFunction", class('MinMaxCurvatureFlowImageFilter_SetTimeStep'))

# Start of MinMaxCurvatureFlowImageFilter_GetTimeStep

`MinMaxCurvatureFlowImageFilter_GetTimeStep` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MinMaxCurvatureFlowImageFilter_GetTimeStep', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MinMaxCurvatureFlowImageFilter_GetTimeStep`, 'returnType') = 'numeric'
attr(`MinMaxCurvatureFlowImageFilter_GetTimeStep`, "inputTypes") = c('_p_itk__simple__MinMaxCurvatureFlowImageFilter')
class(`MinMaxCurvatureFlowImageFilter_GetTimeStep`) = c("SWIGFunction", class('MinMaxCurvatureFlowImageFilter_GetTimeStep'))

# Start of MinMaxCurvatureFlowImageFilter_SetNumberOfIterations

`MinMaxCurvatureFlowImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_MinMaxCurvatureFlowImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MinMaxCurvatureFlowImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`MinMaxCurvatureFlowImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__MinMaxCurvatureFlowImageFilter', 'integer')
class(`MinMaxCurvatureFlowImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('MinMaxCurvatureFlowImageFilter_SetNumberOfIterations'))

# Start of MinMaxCurvatureFlowImageFilter_GetNumberOfIterations

`MinMaxCurvatureFlowImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MinMaxCurvatureFlowImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MinMaxCurvatureFlowImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`MinMaxCurvatureFlowImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__MinMaxCurvatureFlowImageFilter')
class(`MinMaxCurvatureFlowImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('MinMaxCurvatureFlowImageFilter_GetNumberOfIterations'))

# Start of MinMaxCurvatureFlowImageFilter_SetStencilRadius

`MinMaxCurvatureFlowImageFilter_SetStencilRadius` = function(self, StencilRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  StencilRadius = as.integer(StencilRadius);
  
  if(length(StencilRadius) > 1) {
    warning("using only the first element of StencilRadius");
  };
  
  ;ans = .Call('R_swig_MinMaxCurvatureFlowImageFilter_SetStencilRadius', self, StencilRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MinMaxCurvatureFlowImageFilter_SetStencilRadius`, 'returnType') = 'void'
attr(`MinMaxCurvatureFlowImageFilter_SetStencilRadius`, "inputTypes") = c('_p_itk__simple__MinMaxCurvatureFlowImageFilter', 'integer')
class(`MinMaxCurvatureFlowImageFilter_SetStencilRadius`) = c("SWIGFunction", class('MinMaxCurvatureFlowImageFilter_SetStencilRadius'))

# Start of MinMaxCurvatureFlowImageFilter_GetStencilRadius

`MinMaxCurvatureFlowImageFilter_GetStencilRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MinMaxCurvatureFlowImageFilter_GetStencilRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MinMaxCurvatureFlowImageFilter_GetStencilRadius`, 'returnType') = 'integer'
attr(`MinMaxCurvatureFlowImageFilter_GetStencilRadius`, "inputTypes") = c('_p_itk__simple__MinMaxCurvatureFlowImageFilter')
class(`MinMaxCurvatureFlowImageFilter_GetStencilRadius`) = c("SWIGFunction", class('MinMaxCurvatureFlowImageFilter_GetStencilRadius'))

# Start of MinMaxCurvatureFlowImageFilter_GetName

`MinMaxCurvatureFlowImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MinMaxCurvatureFlowImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MinMaxCurvatureFlowImageFilter_GetName`, 'returnType') = 'character'
attr(`MinMaxCurvatureFlowImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MinMaxCurvatureFlowImageFilter')
class(`MinMaxCurvatureFlowImageFilter_GetName`) = c("SWIGFunction", class('MinMaxCurvatureFlowImageFilter_GetName'))

# Start of MinMaxCurvatureFlowImageFilter_ToString

`MinMaxCurvatureFlowImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MinMaxCurvatureFlowImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MinMaxCurvatureFlowImageFilter_ToString`, 'returnType') = 'character'
attr(`MinMaxCurvatureFlowImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MinMaxCurvatureFlowImageFilter')
class(`MinMaxCurvatureFlowImageFilter_ToString`) = c("SWIGFunction", class('MinMaxCurvatureFlowImageFilter_ToString'))

# Start of MinMaxCurvatureFlowImageFilter_Execute

`MinMaxCurvatureFlowImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MinMaxCurvatureFlowImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MinMaxCurvatureFlowImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`MinMaxCurvatureFlowImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__MinMaxCurvatureFlowImageFilter', '_p_itk__simple__Image')
class(`MinMaxCurvatureFlowImageFilter_Execute`) = c("SWIGFunction", class('MinMaxCurvatureFlowImageFilter_Execute'))

# Start of accessor method for itk::simple::MinMaxCurvatureFlowImageFilter
setMethod('$', '_p_itk__simple__MinMaxCurvatureFlowImageFilter', function(x, name)

{
  accessorFuns = list('SetTimeStep' = MinMaxCurvatureFlowImageFilter_SetTimeStep, 'GetTimeStep' = MinMaxCurvatureFlowImageFilter_GetTimeStep, 'SetNumberOfIterations' = MinMaxCurvatureFlowImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = MinMaxCurvatureFlowImageFilter_GetNumberOfIterations, 'SetStencilRadius' = MinMaxCurvatureFlowImageFilter_SetStencilRadius, 'GetStencilRadius' = MinMaxCurvatureFlowImageFilter_GetStencilRadius, 'GetName' = MinMaxCurvatureFlowImageFilter_GetName, 'ToString' = MinMaxCurvatureFlowImageFilter_ToString, 'Execute' = MinMaxCurvatureFlowImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MinMaxCurvatureFlowImageFilter
# Start of MinMaxCurvatureFlow

`MinMaxCurvatureFlow__SWIG_0` = function(image1, timeStep, numberOfIterations, stencilRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  stencilRadius = as.integer(stencilRadius);
  
  if(length(stencilRadius) > 1) {
    warning("using only the first element of stencilRadius");
  };
  
  ;ans = .Call('R_swig_MinMaxCurvatureFlow__SWIG_0', image1, timeStep, numberOfIterations, stencilRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MinMaxCurvatureFlow__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MinMaxCurvatureFlow__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`MinMaxCurvatureFlow__SWIG_0`) = c("SWIGFunction", class('MinMaxCurvatureFlow__SWIG_0'))

# Start of MinMaxCurvatureFlow

`MinMaxCurvatureFlow__SWIG_1` = function(image1, timeStep, numberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_MinMaxCurvatureFlow__SWIG_1', image1, timeStep, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MinMaxCurvatureFlow__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MinMaxCurvatureFlow__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`MinMaxCurvatureFlow__SWIG_1`) = c("SWIGFunction", class('MinMaxCurvatureFlow__SWIG_1'))

# Start of MinMaxCurvatureFlow

`MinMaxCurvatureFlow__SWIG_2` = function(image1, timeStep, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_MinMaxCurvatureFlow__SWIG_2', image1, timeStep, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MinMaxCurvatureFlow__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MinMaxCurvatureFlow__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`MinMaxCurvatureFlow__SWIG_2`) = c("SWIGFunction", class('MinMaxCurvatureFlow__SWIG_2'))

# Start of MinMaxCurvatureFlow

`MinMaxCurvatureFlow__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MinMaxCurvatureFlow__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MinMaxCurvatureFlow__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`MinMaxCurvatureFlow__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`MinMaxCurvatureFlow__SWIG_3`) = c("SWIGFunction", class('MinMaxCurvatureFlow__SWIG_3'))

`MinMaxCurvatureFlow` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- MinMaxCurvatureFlow__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- MinMaxCurvatureFlow__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- MinMaxCurvatureFlow__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- MinMaxCurvatureFlow__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MinMaxCurvatureFlow with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MinimumImageFilter

`delete_MinimumImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MinimumImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MinimumImageFilter`, 'returnType') = 'void'
attr(`delete_MinimumImageFilter`, "inputTypes") = c('_p_itk__simple__MinimumImageFilter')
class(`delete_MinimumImageFilter`) = c("SWIGFunction", class('delete_MinimumImageFilter'))

# Start of new_MinimumImageFilter

`MinimumImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MinimumImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MinimumImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MinimumImageFilter);
  ans
  
}

attr(`MinimumImageFilter`, 'returnType') = '_p_itk__simple__MinimumImageFilter'
class(`MinimumImageFilter`) = c("SWIGFunction", class('MinimumImageFilter'))

# Start of MinimumImageFilter_GetName

`MinimumImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MinimumImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MinimumImageFilter_GetName`, 'returnType') = 'character'
attr(`MinimumImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MinimumImageFilter')
class(`MinimumImageFilter_GetName`) = c("SWIGFunction", class('MinimumImageFilter_GetName'))

# Start of MinimumImageFilter_ToString

`MinimumImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MinimumImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MinimumImageFilter_ToString`, 'returnType') = 'character'
attr(`MinimumImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MinimumImageFilter')
class(`MinimumImageFilter_ToString`) = c("SWIGFunction", class('MinimumImageFilter_ToString'))

# Start of MinimumImageFilter_Execute

`MinimumImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_MinimumImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MinimumImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MinimumImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__MinimumImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MinimumImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('MinimumImageFilter_Execute__SWIG_0'))

# Start of MinimumImageFilter_Execute

`MinimumImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_MinimumImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MinimumImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MinimumImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__MinimumImageFilter', '_p_itk__simple__Image', 'numeric')
class(`MinimumImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('MinimumImageFilter_Execute__SWIG_1'))

# Start of MinimumImageFilter_Execute

`MinimumImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_MinimumImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MinimumImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MinimumImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__MinimumImageFilter', 'numeric', '_p_itk__simple__Image')
class(`MinimumImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('MinimumImageFilter_Execute__SWIG_2'))

`MinimumImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__MinimumImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MinimumImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__MinimumImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- MinimumImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__MinimumImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MinimumImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MinimumImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::MinimumImageFilter
setMethod('$', '_p_itk__simple__MinimumImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = MinimumImageFilter_GetName, 'ToString' = MinimumImageFilter_ToString, 'Execute' = MinimumImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MinimumImageFilter
# Start of Minimum

`Minimum__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Minimum__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Minimum__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Minimum__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Minimum__SWIG_0`) = c("SWIGFunction", class('Minimum__SWIG_0'))

# Start of Minimum

`Minimum__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Minimum__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Minimum__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Minimum__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`Minimum__SWIG_1`) = c("SWIGFunction", class('Minimum__SWIG_1'))

# Start of Minimum

`Minimum__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Minimum__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Minimum__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Minimum__SWIG_2`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`Minimum__SWIG_2`) = c("SWIGFunction", class('Minimum__SWIG_2'))

`Minimum` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Minimum__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Minimum__SWIG_1; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Minimum__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Minimum with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MinimumMaximumImageFilter

`delete_MinimumMaximumImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MinimumMaximumImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MinimumMaximumImageFilter`, 'returnType') = 'void'
attr(`delete_MinimumMaximumImageFilter`, "inputTypes") = c('_p_itk__simple__MinimumMaximumImageFilter')
class(`delete_MinimumMaximumImageFilter`) = c("SWIGFunction", class('delete_MinimumMaximumImageFilter'))

# Start of new_MinimumMaximumImageFilter

`MinimumMaximumImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MinimumMaximumImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MinimumMaximumImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MinimumMaximumImageFilter);
  ans
  
}

attr(`MinimumMaximumImageFilter`, 'returnType') = '_p_itk__simple__MinimumMaximumImageFilter'
class(`MinimumMaximumImageFilter`) = c("SWIGFunction", class('MinimumMaximumImageFilter'))

# Start of MinimumMaximumImageFilter_GetMinimum

`MinimumMaximumImageFilter_GetMinimum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MinimumMaximumImageFilter_GetMinimum', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MinimumMaximumImageFilter_GetMinimum`, 'returnType') = 'numeric'
attr(`MinimumMaximumImageFilter_GetMinimum`, "inputTypes") = c('_p_itk__simple__MinimumMaximumImageFilter')
class(`MinimumMaximumImageFilter_GetMinimum`) = c("SWIGFunction", class('MinimumMaximumImageFilter_GetMinimum'))

# Start of MinimumMaximumImageFilter_GetMaximum

`MinimumMaximumImageFilter_GetMaximum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MinimumMaximumImageFilter_GetMaximum', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MinimumMaximumImageFilter_GetMaximum`, 'returnType') = 'numeric'
attr(`MinimumMaximumImageFilter_GetMaximum`, "inputTypes") = c('_p_itk__simple__MinimumMaximumImageFilter')
class(`MinimumMaximumImageFilter_GetMaximum`) = c("SWIGFunction", class('MinimumMaximumImageFilter_GetMaximum'))

# Start of MinimumMaximumImageFilter_GetName

`MinimumMaximumImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MinimumMaximumImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MinimumMaximumImageFilter_GetName`, 'returnType') = 'character'
attr(`MinimumMaximumImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MinimumMaximumImageFilter')
class(`MinimumMaximumImageFilter_GetName`) = c("SWIGFunction", class('MinimumMaximumImageFilter_GetName'))

# Start of MinimumMaximumImageFilter_ToString

`MinimumMaximumImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MinimumMaximumImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MinimumMaximumImageFilter_ToString`, 'returnType') = 'character'
attr(`MinimumMaximumImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MinimumMaximumImageFilter')
class(`MinimumMaximumImageFilter_ToString`) = c("SWIGFunction", class('MinimumMaximumImageFilter_ToString'))

# Start of MinimumMaximumImageFilter_Execute

`MinimumMaximumImageFilter_Execute` = function(self, image)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_MinimumMaximumImageFilter_Execute', self, image, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MinimumMaximumImageFilter_Execute`, 'returnType') = 'void'
attr(`MinimumMaximumImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__MinimumMaximumImageFilter', '_p_itk__simple__Image')
class(`MinimumMaximumImageFilter_Execute`) = c("SWIGFunction", class('MinimumMaximumImageFilter_Execute'))

# Start of accessor method for itk::simple::MinimumMaximumImageFilter
setMethod('$', '_p_itk__simple__MinimumMaximumImageFilter', function(x, name)

{
  accessorFuns = list('GetMinimum' = MinimumMaximumImageFilter_GetMinimum, 'GetMaximum' = MinimumMaximumImageFilter_GetMaximum, 'GetName' = MinimumMaximumImageFilter_GetName, 'ToString' = MinimumMaximumImageFilter_ToString, 'Execute' = MinimumMaximumImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MinimumMaximumImageFilter
# Start of delete_MinimumProjectionImageFilter

`delete_MinimumProjectionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MinimumProjectionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MinimumProjectionImageFilter`, 'returnType') = 'void'
attr(`delete_MinimumProjectionImageFilter`, "inputTypes") = c('_p_itk__simple__MinimumProjectionImageFilter')
class(`delete_MinimumProjectionImageFilter`) = c("SWIGFunction", class('delete_MinimumProjectionImageFilter'))

# Start of new_MinimumProjectionImageFilter

`MinimumProjectionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MinimumProjectionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MinimumProjectionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MinimumProjectionImageFilter);
  ans
  
}

attr(`MinimumProjectionImageFilter`, 'returnType') = '_p_itk__simple__MinimumProjectionImageFilter'
class(`MinimumProjectionImageFilter`) = c("SWIGFunction", class('MinimumProjectionImageFilter'))

# Start of MinimumProjectionImageFilter_SetProjectionDimension

`MinimumProjectionImageFilter_SetProjectionDimension` = function(self, ProjectionDimension)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ProjectionDimension = as.integer(ProjectionDimension);
  
  if(length(ProjectionDimension) > 1) {
    warning("using only the first element of ProjectionDimension");
  };
  
  ;ans = .Call('R_swig_MinimumProjectionImageFilter_SetProjectionDimension', self, ProjectionDimension, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MinimumProjectionImageFilter_SetProjectionDimension`, 'returnType') = 'void'
attr(`MinimumProjectionImageFilter_SetProjectionDimension`, "inputTypes") = c('_p_itk__simple__MinimumProjectionImageFilter', 'integer')
class(`MinimumProjectionImageFilter_SetProjectionDimension`) = c("SWIGFunction", class('MinimumProjectionImageFilter_SetProjectionDimension'))

# Start of MinimumProjectionImageFilter_GetProjectionDimension

`MinimumProjectionImageFilter_GetProjectionDimension` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MinimumProjectionImageFilter_GetProjectionDimension', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MinimumProjectionImageFilter_GetProjectionDimension`, 'returnType') = 'integer'
attr(`MinimumProjectionImageFilter_GetProjectionDimension`, "inputTypes") = c('_p_itk__simple__MinimumProjectionImageFilter')
class(`MinimumProjectionImageFilter_GetProjectionDimension`) = c("SWIGFunction", class('MinimumProjectionImageFilter_GetProjectionDimension'))

# Start of MinimumProjectionImageFilter_GetName

`MinimumProjectionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MinimumProjectionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MinimumProjectionImageFilter_GetName`, 'returnType') = 'character'
attr(`MinimumProjectionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MinimumProjectionImageFilter')
class(`MinimumProjectionImageFilter_GetName`) = c("SWIGFunction", class('MinimumProjectionImageFilter_GetName'))

# Start of MinimumProjectionImageFilter_ToString

`MinimumProjectionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MinimumProjectionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MinimumProjectionImageFilter_ToString`, 'returnType') = 'character'
attr(`MinimumProjectionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MinimumProjectionImageFilter')
class(`MinimumProjectionImageFilter_ToString`) = c("SWIGFunction", class('MinimumProjectionImageFilter_ToString'))

# Start of MinimumProjectionImageFilter_Execute

`MinimumProjectionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MinimumProjectionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MinimumProjectionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`MinimumProjectionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__MinimumProjectionImageFilter', '_p_itk__simple__Image')
class(`MinimumProjectionImageFilter_Execute`) = c("SWIGFunction", class('MinimumProjectionImageFilter_Execute'))

# Start of accessor method for itk::simple::MinimumProjectionImageFilter
setMethod('$', '_p_itk__simple__MinimumProjectionImageFilter', function(x, name)

{
  accessorFuns = list('SetProjectionDimension' = MinimumProjectionImageFilter_SetProjectionDimension, 'GetProjectionDimension' = MinimumProjectionImageFilter_GetProjectionDimension, 'GetName' = MinimumProjectionImageFilter_GetName, 'ToString' = MinimumProjectionImageFilter_ToString, 'Execute' = MinimumProjectionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MinimumProjectionImageFilter
# Start of MinimumProjection

`MinimumProjection__SWIG_0` = function(image1, projectionDimension, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  projectionDimension = as.integer(projectionDimension);
  
  if(length(projectionDimension) > 1) {
    warning("using only the first element of projectionDimension");
  };
  
  ;ans = .Call('R_swig_MinimumProjection__SWIG_0', image1, projectionDimension, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MinimumProjection__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MinimumProjection__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`MinimumProjection__SWIG_0`) = c("SWIGFunction", class('MinimumProjection__SWIG_0'))

# Start of MinimumProjection

`MinimumProjection__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MinimumProjection__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MinimumProjection__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MinimumProjection__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`MinimumProjection__SWIG_1`) = c("SWIGFunction", class('MinimumProjection__SWIG_1'))

`MinimumProjection` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- MinimumProjection__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- MinimumProjection__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MinimumProjection with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MirrorPadImageFilter

`delete_MirrorPadImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MirrorPadImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MirrorPadImageFilter`, 'returnType') = 'void'
attr(`delete_MirrorPadImageFilter`, "inputTypes") = c('_p_itk__simple__MirrorPadImageFilter')
class(`delete_MirrorPadImageFilter`) = c("SWIGFunction", class('delete_MirrorPadImageFilter'))

# Start of new_MirrorPadImageFilter

`MirrorPadImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MirrorPadImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MirrorPadImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MirrorPadImageFilter);
  ans
  
}

attr(`MirrorPadImageFilter`, 'returnType') = '_p_itk__simple__MirrorPadImageFilter'
class(`MirrorPadImageFilter`) = c("SWIGFunction", class('MirrorPadImageFilter'))

# Start of MirrorPadImageFilter_SetPadLowerBound

`MirrorPadImageFilter_SetPadLowerBound` = function(self, PadLowerBound)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  PadLowerBound = as.integer(PadLowerBound);
  ;ans = .Call('R_swig_MirrorPadImageFilter_SetPadLowerBound', self, PadLowerBound, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MirrorPadImageFilter_SetPadLowerBound`, 'returnType') = 'void'
attr(`MirrorPadImageFilter_SetPadLowerBound`, "inputTypes") = c('_p_itk__simple__MirrorPadImageFilter', 'integer')
class(`MirrorPadImageFilter_SetPadLowerBound`) = c("SWIGFunction", class('MirrorPadImageFilter_SetPadLowerBound'))

# Start of MirrorPadImageFilter_GetPadLowerBound

`MirrorPadImageFilter_GetPadLowerBound` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MirrorPadImageFilter_GetPadLowerBound', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MirrorPadImageFilter_GetPadLowerBound`, 'returnType') = 'integer'
attr(`MirrorPadImageFilter_GetPadLowerBound`, "inputTypes") = c('_p_itk__simple__MirrorPadImageFilter')
class(`MirrorPadImageFilter_GetPadLowerBound`) = c("SWIGFunction", class('MirrorPadImageFilter_GetPadLowerBound'))

# Start of MirrorPadImageFilter_SetPadUpperBound

`MirrorPadImageFilter_SetPadUpperBound` = function(self, PadUpperBound)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  PadUpperBound = as.integer(PadUpperBound);
  ;ans = .Call('R_swig_MirrorPadImageFilter_SetPadUpperBound', self, PadUpperBound, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MirrorPadImageFilter_SetPadUpperBound`, 'returnType') = 'void'
attr(`MirrorPadImageFilter_SetPadUpperBound`, "inputTypes") = c('_p_itk__simple__MirrorPadImageFilter', 'integer')
class(`MirrorPadImageFilter_SetPadUpperBound`) = c("SWIGFunction", class('MirrorPadImageFilter_SetPadUpperBound'))

# Start of MirrorPadImageFilter_GetPadUpperBound

`MirrorPadImageFilter_GetPadUpperBound` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MirrorPadImageFilter_GetPadUpperBound', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MirrorPadImageFilter_GetPadUpperBound`, 'returnType') = 'integer'
attr(`MirrorPadImageFilter_GetPadUpperBound`, "inputTypes") = c('_p_itk__simple__MirrorPadImageFilter')
class(`MirrorPadImageFilter_GetPadUpperBound`) = c("SWIGFunction", class('MirrorPadImageFilter_GetPadUpperBound'))

# Start of MirrorPadImageFilter_SetDecayBase

`MirrorPadImageFilter_SetDecayBase` = function(self, DecayBase)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_MirrorPadImageFilter_SetDecayBase', self, DecayBase, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MirrorPadImageFilter_SetDecayBase`, 'returnType') = 'void'
attr(`MirrorPadImageFilter_SetDecayBase`, "inputTypes") = c('_p_itk__simple__MirrorPadImageFilter', 'numeric')
class(`MirrorPadImageFilter_SetDecayBase`) = c("SWIGFunction", class('MirrorPadImageFilter_SetDecayBase'))

# Start of MirrorPadImageFilter_GetDecayBase

`MirrorPadImageFilter_GetDecayBase` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MirrorPadImageFilter_GetDecayBase', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MirrorPadImageFilter_GetDecayBase`, 'returnType') = 'numeric'
attr(`MirrorPadImageFilter_GetDecayBase`, "inputTypes") = c('_p_itk__simple__MirrorPadImageFilter')
class(`MirrorPadImageFilter_GetDecayBase`) = c("SWIGFunction", class('MirrorPadImageFilter_GetDecayBase'))

# Start of MirrorPadImageFilter_GetName

`MirrorPadImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MirrorPadImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MirrorPadImageFilter_GetName`, 'returnType') = 'character'
attr(`MirrorPadImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MirrorPadImageFilter')
class(`MirrorPadImageFilter_GetName`) = c("SWIGFunction", class('MirrorPadImageFilter_GetName'))

# Start of MirrorPadImageFilter_ToString

`MirrorPadImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MirrorPadImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MirrorPadImageFilter_ToString`, 'returnType') = 'character'
attr(`MirrorPadImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MirrorPadImageFilter')
class(`MirrorPadImageFilter_ToString`) = c("SWIGFunction", class('MirrorPadImageFilter_ToString'))

# Start of MirrorPadImageFilter_Execute

`MirrorPadImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MirrorPadImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MirrorPadImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`MirrorPadImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__MirrorPadImageFilter', '_p_itk__simple__Image')
class(`MirrorPadImageFilter_Execute`) = c("SWIGFunction", class('MirrorPadImageFilter_Execute'))

# Start of accessor method for itk::simple::MirrorPadImageFilter
setMethod('$', '_p_itk__simple__MirrorPadImageFilter', function(x, name)

{
  accessorFuns = list('SetPadLowerBound' = MirrorPadImageFilter_SetPadLowerBound, 'GetPadLowerBound' = MirrorPadImageFilter_GetPadLowerBound, 'SetPadUpperBound' = MirrorPadImageFilter_SetPadUpperBound, 'GetPadUpperBound' = MirrorPadImageFilter_GetPadUpperBound, 'SetDecayBase' = MirrorPadImageFilter_SetDecayBase, 'GetDecayBase' = MirrorPadImageFilter_GetDecayBase, 'GetName' = MirrorPadImageFilter_GetName, 'ToString' = MirrorPadImageFilter_ToString, 'Execute' = MirrorPadImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MirrorPadImageFilter
# Start of MirrorPad

`MirrorPad__SWIG_0` = function(image1, padLowerBound, padUpperBound, decayBase, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  padLowerBound = as.integer(padLowerBound);
  padUpperBound = as.integer(padUpperBound);
  
  ;ans = .Call('R_swig_MirrorPad__SWIG_0', image1, padLowerBound, padUpperBound, decayBase, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MirrorPad__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MirrorPad__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'numeric')
class(`MirrorPad__SWIG_0`) = c("SWIGFunction", class('MirrorPad__SWIG_0'))

# Start of MirrorPad

`MirrorPad__SWIG_1` = function(image1, padLowerBound, padUpperBound, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  padLowerBound = as.integer(padLowerBound);
  padUpperBound = as.integer(padUpperBound);
  ;ans = .Call('R_swig_MirrorPad__SWIG_1', image1, padLowerBound, padUpperBound, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MirrorPad__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MirrorPad__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`MirrorPad__SWIG_1`) = c("SWIGFunction", class('MirrorPad__SWIG_1'))

# Start of MirrorPad

`MirrorPad__SWIG_2` = function(image1, padLowerBound, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  padLowerBound = as.integer(padLowerBound);
  ;ans = .Call('R_swig_MirrorPad__SWIG_2', image1, padLowerBound, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MirrorPad__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MirrorPad__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`MirrorPad__SWIG_2`) = c("SWIGFunction", class('MirrorPad__SWIG_2'))

# Start of MirrorPad

`MirrorPad__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MirrorPad__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MirrorPad__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`MirrorPad__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`MirrorPad__SWIG_3`) = c("SWIGFunction", class('MirrorPad__SWIG_3'))

`MirrorPad` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- MirrorPad__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- MirrorPad__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- MirrorPad__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- MirrorPad__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MirrorPad with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ModulusImageFilter

`delete_ModulusImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ModulusImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ModulusImageFilter`, 'returnType') = 'void'
attr(`delete_ModulusImageFilter`, "inputTypes") = c('_p_itk__simple__ModulusImageFilter')
class(`delete_ModulusImageFilter`) = c("SWIGFunction", class('delete_ModulusImageFilter'))

# Start of new_ModulusImageFilter

`ModulusImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ModulusImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ModulusImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ModulusImageFilter);
  ans
  
}

attr(`ModulusImageFilter`, 'returnType') = '_p_itk__simple__ModulusImageFilter'
class(`ModulusImageFilter`) = c("SWIGFunction", class('ModulusImageFilter'))

# Start of ModulusImageFilter_GetName

`ModulusImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ModulusImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ModulusImageFilter_GetName`, 'returnType') = 'character'
attr(`ModulusImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ModulusImageFilter')
class(`ModulusImageFilter_GetName`) = c("SWIGFunction", class('ModulusImageFilter_GetName'))

# Start of ModulusImageFilter_ToString

`ModulusImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ModulusImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ModulusImageFilter_ToString`, 'returnType') = 'character'
attr(`ModulusImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ModulusImageFilter')
class(`ModulusImageFilter_ToString`) = c("SWIGFunction", class('ModulusImageFilter_ToString'))

# Start of ModulusImageFilter_Execute

`ModulusImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_ModulusImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ModulusImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ModulusImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__ModulusImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ModulusImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('ModulusImageFilter_Execute__SWIG_0'))

# Start of ModulusImageFilter_Execute

`ModulusImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  ;ans = .Call('R_swig_ModulusImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ModulusImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ModulusImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__ModulusImageFilter', '_p_itk__simple__Image', 'integer')
class(`ModulusImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('ModulusImageFilter_Execute__SWIG_1'))

# Start of ModulusImageFilter_Execute

`ModulusImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_ModulusImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ModulusImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ModulusImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__ModulusImageFilter', 'integer', '_p_itk__simple__Image')
class(`ModulusImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('ModulusImageFilter_Execute__SWIG_2'))

`ModulusImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ModulusImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- ModulusImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__ModulusImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ModulusImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__ModulusImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- ModulusImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ModulusImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::ModulusImageFilter
setMethod('$', '_p_itk__simple__ModulusImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = ModulusImageFilter_GetName, 'ToString' = ModulusImageFilter_ToString, 'Execute' = ModulusImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ModulusImageFilter
# Start of Modulus

`Modulus__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Modulus__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Modulus__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Modulus__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Modulus__SWIG_0`) = c("SWIGFunction", class('Modulus__SWIG_0'))

# Start of Modulus

`Modulus__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  ;ans = .Call('R_swig_Modulus__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Modulus__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Modulus__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Modulus__SWIG_1`) = c("SWIGFunction", class('Modulus__SWIG_1'))

# Start of Modulus

`Modulus__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Modulus__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Modulus__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Modulus__SWIG_2`, "inputTypes") = c('integer', '_p_itk__simple__Image')
class(`Modulus__SWIG_2`) = c("SWIGFunction", class('Modulus__SWIG_2'))

`Modulus` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Modulus__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- Modulus__SWIG_1; 
    }
    else if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Modulus__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Modulus with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MomentsThresholdImageFilter

`delete_MomentsThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MomentsThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MomentsThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_MomentsThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter')
class(`delete_MomentsThresholdImageFilter`) = c("SWIGFunction", class('delete_MomentsThresholdImageFilter'))

# Start of new_MomentsThresholdImageFilter

`MomentsThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MomentsThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MomentsThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MomentsThresholdImageFilter);
  ans
  
}

attr(`MomentsThresholdImageFilter`, 'returnType') = '_p_itk__simple__MomentsThresholdImageFilter'
class(`MomentsThresholdImageFilter`) = c("SWIGFunction", class('MomentsThresholdImageFilter'))

# Start of MomentsThresholdImageFilter_SetInsideValue

`MomentsThresholdImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideValue = as.integer(InsideValue);
  
  if(length(InsideValue) > 1) {
    warning("using only the first element of InsideValue");
  };
  
  ;ans = .Call('R_swig_MomentsThresholdImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MomentsThresholdImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`MomentsThresholdImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter', 'integer')
class(`MomentsThresholdImageFilter_SetInsideValue`) = c("SWIGFunction", class('MomentsThresholdImageFilter_SetInsideValue'))

# Start of MomentsThresholdImageFilter_GetInsideValue

`MomentsThresholdImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MomentsThresholdImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MomentsThresholdImageFilter_GetInsideValue`, 'returnType') = 'integer'
attr(`MomentsThresholdImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter')
class(`MomentsThresholdImageFilter_GetInsideValue`) = c("SWIGFunction", class('MomentsThresholdImageFilter_GetInsideValue'))

# Start of MomentsThresholdImageFilter_SetOutsideValue

`MomentsThresholdImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutsideValue = as.integer(OutsideValue);
  
  if(length(OutsideValue) > 1) {
    warning("using only the first element of OutsideValue");
  };
  
  ;ans = .Call('R_swig_MomentsThresholdImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MomentsThresholdImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`MomentsThresholdImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter', 'integer')
class(`MomentsThresholdImageFilter_SetOutsideValue`) = c("SWIGFunction", class('MomentsThresholdImageFilter_SetOutsideValue'))

# Start of MomentsThresholdImageFilter_GetOutsideValue

`MomentsThresholdImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MomentsThresholdImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MomentsThresholdImageFilter_GetOutsideValue`, 'returnType') = 'integer'
attr(`MomentsThresholdImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter')
class(`MomentsThresholdImageFilter_GetOutsideValue`) = c("SWIGFunction", class('MomentsThresholdImageFilter_GetOutsideValue'))

# Start of MomentsThresholdImageFilter_SetNumberOfHistogramBins

`MomentsThresholdImageFilter_SetNumberOfHistogramBins` = function(self, NumberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfHistogramBins = as.integer(NumberOfHistogramBins);
  
  if(length(NumberOfHistogramBins) > 1) {
    warning("using only the first element of NumberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_MomentsThresholdImageFilter_SetNumberOfHistogramBins', self, NumberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MomentsThresholdImageFilter_SetNumberOfHistogramBins`, 'returnType') = 'void'
attr(`MomentsThresholdImageFilter_SetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter', 'integer')
class(`MomentsThresholdImageFilter_SetNumberOfHistogramBins`) = c("SWIGFunction", class('MomentsThresholdImageFilter_SetNumberOfHistogramBins'))

# Start of MomentsThresholdImageFilter_GetNumberOfHistogramBins

`MomentsThresholdImageFilter_GetNumberOfHistogramBins` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MomentsThresholdImageFilter_GetNumberOfHistogramBins', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MomentsThresholdImageFilter_GetNumberOfHistogramBins`, 'returnType') = 'integer'
attr(`MomentsThresholdImageFilter_GetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter')
class(`MomentsThresholdImageFilter_GetNumberOfHistogramBins`) = c("SWIGFunction", class('MomentsThresholdImageFilter_GetNumberOfHistogramBins'))

# Start of MomentsThresholdImageFilter_SetMaskOutput

`MomentsThresholdImageFilter_SetMaskOutput` = function(self, MaskOutput)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskOutput = as.logical(MaskOutput);
  ;ans = .Call('R_swig_MomentsThresholdImageFilter_SetMaskOutput', self, MaskOutput, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MomentsThresholdImageFilter_SetMaskOutput`, 'returnType') = 'void'
attr(`MomentsThresholdImageFilter_SetMaskOutput`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter', 'logical')
class(`MomentsThresholdImageFilter_SetMaskOutput`) = c("SWIGFunction", class('MomentsThresholdImageFilter_SetMaskOutput'))

# Start of MomentsThresholdImageFilter_MaskOutputOn

`MomentsThresholdImageFilter_MaskOutputOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_MomentsThresholdImageFilter_MaskOutputOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MomentsThresholdImageFilter_MaskOutputOn`, 'returnType') = 'void'
attr(`MomentsThresholdImageFilter_MaskOutputOn`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter')
class(`MomentsThresholdImageFilter_MaskOutputOn`) = c("SWIGFunction", class('MomentsThresholdImageFilter_MaskOutputOn'))

# Start of MomentsThresholdImageFilter_MaskOutputOff

`MomentsThresholdImageFilter_MaskOutputOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_MomentsThresholdImageFilter_MaskOutputOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MomentsThresholdImageFilter_MaskOutputOff`, 'returnType') = 'void'
attr(`MomentsThresholdImageFilter_MaskOutputOff`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter')
class(`MomentsThresholdImageFilter_MaskOutputOff`) = c("SWIGFunction", class('MomentsThresholdImageFilter_MaskOutputOff'))

# Start of MomentsThresholdImageFilter_GetMaskOutput

`MomentsThresholdImageFilter_GetMaskOutput` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MomentsThresholdImageFilter_GetMaskOutput', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MomentsThresholdImageFilter_GetMaskOutput`, 'returnType') = 'logical'
attr(`MomentsThresholdImageFilter_GetMaskOutput`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter')
class(`MomentsThresholdImageFilter_GetMaskOutput`) = c("SWIGFunction", class('MomentsThresholdImageFilter_GetMaskOutput'))

# Start of MomentsThresholdImageFilter_SetMaskValue

`MomentsThresholdImageFilter_SetMaskValue` = function(self, MaskValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskValue = as.integer(MaskValue);
  
  if(length(MaskValue) > 1) {
    warning("using only the first element of MaskValue");
  };
  
  ;ans = .Call('R_swig_MomentsThresholdImageFilter_SetMaskValue', self, MaskValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MomentsThresholdImageFilter_SetMaskValue`, 'returnType') = 'void'
attr(`MomentsThresholdImageFilter_SetMaskValue`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter', 'integer')
class(`MomentsThresholdImageFilter_SetMaskValue`) = c("SWIGFunction", class('MomentsThresholdImageFilter_SetMaskValue'))

# Start of MomentsThresholdImageFilter_GetMaskValue

`MomentsThresholdImageFilter_GetMaskValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MomentsThresholdImageFilter_GetMaskValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MomentsThresholdImageFilter_GetMaskValue`, 'returnType') = 'integer'
attr(`MomentsThresholdImageFilter_GetMaskValue`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter')
class(`MomentsThresholdImageFilter_GetMaskValue`) = c("SWIGFunction", class('MomentsThresholdImageFilter_GetMaskValue'))

# Start of MomentsThresholdImageFilter_GetThreshold

`MomentsThresholdImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MomentsThresholdImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MomentsThresholdImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`MomentsThresholdImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter')
class(`MomentsThresholdImageFilter_GetThreshold`) = c("SWIGFunction", class('MomentsThresholdImageFilter_GetThreshold'))

# Start of MomentsThresholdImageFilter_GetName

`MomentsThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MomentsThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MomentsThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`MomentsThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter')
class(`MomentsThresholdImageFilter_GetName`) = c("SWIGFunction", class('MomentsThresholdImageFilter_GetName'))

# Start of MomentsThresholdImageFilter_ToString

`MomentsThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MomentsThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MomentsThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`MomentsThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter')
class(`MomentsThresholdImageFilter_ToString`) = c("SWIGFunction", class('MomentsThresholdImageFilter_ToString'))

# Start of MomentsThresholdImageFilter_Execute

`MomentsThresholdImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_MomentsThresholdImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MomentsThresholdImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MomentsThresholdImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MomentsThresholdImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('MomentsThresholdImageFilter_Execute__SWIG_0'))

# Start of MomentsThresholdImageFilter_Execute

`MomentsThresholdImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_MomentsThresholdImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MomentsThresholdImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MomentsThresholdImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__MomentsThresholdImageFilter', '_p_itk__simple__Image')
class(`MomentsThresholdImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('MomentsThresholdImageFilter_Execute__SWIG_1'))

`MomentsThresholdImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__MomentsThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- MomentsThresholdImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__MomentsThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MomentsThresholdImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MomentsThresholdImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::MomentsThresholdImageFilter
setMethod('$', '_p_itk__simple__MomentsThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetInsideValue' = MomentsThresholdImageFilter_SetInsideValue, 'GetInsideValue' = MomentsThresholdImageFilter_GetInsideValue, 'SetOutsideValue' = MomentsThresholdImageFilter_SetOutsideValue, 'GetOutsideValue' = MomentsThresholdImageFilter_GetOutsideValue, 'SetNumberOfHistogramBins' = MomentsThresholdImageFilter_SetNumberOfHistogramBins, 'GetNumberOfHistogramBins' = MomentsThresholdImageFilter_GetNumberOfHistogramBins, 'SetMaskOutput' = MomentsThresholdImageFilter_SetMaskOutput, 'MaskOutputOn' = MomentsThresholdImageFilter_MaskOutputOn, 'MaskOutputOff' = MomentsThresholdImageFilter_MaskOutputOff, 'GetMaskOutput' = MomentsThresholdImageFilter_GetMaskOutput, 'SetMaskValue' = MomentsThresholdImageFilter_SetMaskValue, 'GetMaskValue' = MomentsThresholdImageFilter_GetMaskValue, 'GetThreshold' = MomentsThresholdImageFilter_GetThreshold, 'GetName' = MomentsThresholdImageFilter_GetName, 'ToString' = MomentsThresholdImageFilter_ToString, 'Execute' = MomentsThresholdImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MomentsThresholdImageFilter
# Start of MomentsThreshold

`MomentsThreshold__SWIG_0` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_MomentsThreshold__SWIG_0', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MomentsThreshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MomentsThreshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`MomentsThreshold__SWIG_0`) = c("SWIGFunction", class('MomentsThreshold__SWIG_0'))

# Start of MomentsThreshold

`MomentsThreshold__SWIG_1` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_MomentsThreshold__SWIG_1', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MomentsThreshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MomentsThreshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`MomentsThreshold__SWIG_1`) = c("SWIGFunction", class('MomentsThreshold__SWIG_1'))

# Start of MomentsThreshold

`MomentsThreshold__SWIG_2` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_MomentsThreshold__SWIG_2', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MomentsThreshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MomentsThreshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`MomentsThreshold__SWIG_2`) = c("SWIGFunction", class('MomentsThreshold__SWIG_2'))

# Start of MomentsThreshold

`MomentsThreshold__SWIG_3` = function(image, maskImage, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_MomentsThreshold__SWIG_3', image, maskImage, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MomentsThreshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`MomentsThreshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`MomentsThreshold__SWIG_3`) = c("SWIGFunction", class('MomentsThreshold__SWIG_3'))

# Start of MomentsThreshold

`MomentsThreshold__SWIG_4` = function(image, maskImage, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_MomentsThreshold__SWIG_4', image, maskImage, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MomentsThreshold__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`MomentsThreshold__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`MomentsThreshold__SWIG_4`) = c("SWIGFunction", class('MomentsThreshold__SWIG_4'))

# Start of MomentsThreshold

`MomentsThreshold__SWIG_5` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_MomentsThreshold__SWIG_5', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MomentsThreshold__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`MomentsThreshold__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MomentsThreshold__SWIG_5`) = c("SWIGFunction", class('MomentsThreshold__SWIG_5'))

# Start of MomentsThreshold

`MomentsThreshold__SWIG_6` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_MomentsThreshold__SWIG_6', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MomentsThreshold__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`MomentsThreshold__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`MomentsThreshold__SWIG_6`) = c("SWIGFunction", class('MomentsThreshold__SWIG_6'))

# Start of MomentsThreshold

`MomentsThreshold__SWIG_7` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_MomentsThreshold__SWIG_7', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MomentsThreshold__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`MomentsThreshold__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`MomentsThreshold__SWIG_7`) = c("SWIGFunction", class('MomentsThreshold__SWIG_7'))

# Start of MomentsThreshold

`MomentsThreshold__SWIG_8` = function(image, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_MomentsThreshold__SWIG_8', image, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MomentsThreshold__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`MomentsThreshold__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`MomentsThreshold__SWIG_8`) = c("SWIGFunction", class('MomentsThreshold__SWIG_8'))

# Start of MomentsThreshold

`MomentsThreshold__SWIG_9` = function(image, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_MomentsThreshold__SWIG_9', image, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MomentsThreshold__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`MomentsThreshold__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`MomentsThreshold__SWIG_9`) = c("SWIGFunction", class('MomentsThreshold__SWIG_9'))

# Start of MomentsThreshold

`MomentsThreshold__SWIG_10` = function(image, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_MomentsThreshold__SWIG_10', image, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MomentsThreshold__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`MomentsThreshold__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`MomentsThreshold__SWIG_10`) = c("SWIGFunction", class('MomentsThreshold__SWIG_10'))

# Start of MomentsThreshold

`MomentsThreshold__SWIG_11` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_MomentsThreshold__SWIG_11', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MomentsThreshold__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`MomentsThreshold__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image')
class(`MomentsThreshold__SWIG_11`) = c("SWIGFunction", class('MomentsThreshold__SWIG_11'))

`MomentsThreshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 12
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- MomentsThreshold__SWIG_11; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- MomentsThreshold__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- MomentsThreshold__SWIG_10; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- MomentsThreshold__SWIG_4; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- MomentsThreshold__SWIG_9; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- MomentsThreshold__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- MomentsThreshold__SWIG_8; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- MomentsThreshold__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- MomentsThreshold__SWIG_7; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- MomentsThreshold__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- MomentsThreshold__SWIG_6; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- MomentsThreshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MomentsThreshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MorphologicalGradientImageFilter

`delete_MorphologicalGradientImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MorphologicalGradientImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MorphologicalGradientImageFilter`, 'returnType') = 'void'
attr(`delete_MorphologicalGradientImageFilter`, "inputTypes") = c('_p_itk__simple__MorphologicalGradientImageFilter')
class(`delete_MorphologicalGradientImageFilter`) = c("SWIGFunction", class('delete_MorphologicalGradientImageFilter'))

# Start of new_MorphologicalGradientImageFilter

`MorphologicalGradientImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MorphologicalGradientImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MorphologicalGradientImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MorphologicalGradientImageFilter);
  ans
  
}

attr(`MorphologicalGradientImageFilter`, 'returnType') = '_p_itk__simple__MorphologicalGradientImageFilter'
class(`MorphologicalGradientImageFilter`) = c("SWIGFunction", class('MorphologicalGradientImageFilter'))

# Start of MorphologicalGradientImageFilter_SetKernelRadius

`MorphologicalGradientImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_MorphologicalGradientImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalGradientImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`MorphologicalGradientImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__MorphologicalGradientImageFilter', 'integer')
class(`MorphologicalGradientImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('MorphologicalGradientImageFilter_SetKernelRadius__SWIG_0'))

# Start of MorphologicalGradientImageFilter_SetKernelRadius

`MorphologicalGradientImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_MorphologicalGradientImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalGradientImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`MorphologicalGradientImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__MorphologicalGradientImageFilter', 'integer')
class(`MorphologicalGradientImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('MorphologicalGradientImageFilter_SetKernelRadius__SWIG_1'))

`MorphologicalGradientImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__MorphologicalGradientImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- MorphologicalGradientImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__MorphologicalGradientImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- MorphologicalGradientImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MorphologicalGradientImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of MorphologicalGradientImageFilter_GetKernelRadius

`MorphologicalGradientImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MorphologicalGradientImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MorphologicalGradientImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`MorphologicalGradientImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__MorphologicalGradientImageFilter')
class(`MorphologicalGradientImageFilter_GetKernelRadius`) = c("SWIGFunction", class('MorphologicalGradientImageFilter_GetKernelRadius'))

# Start of MorphologicalGradientImageFilter_SetKernelType

`MorphologicalGradientImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_MorphologicalGradientImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalGradientImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`MorphologicalGradientImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__MorphologicalGradientImageFilter', 'character')
class(`MorphologicalGradientImageFilter_SetKernelType`) = c("SWIGFunction", class('MorphologicalGradientImageFilter_SetKernelType'))

# Start of MorphologicalGradientImageFilter_GetKernelType

`MorphologicalGradientImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_MorphologicalGradientImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`MorphologicalGradientImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`MorphologicalGradientImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__MorphologicalGradientImageFilter')
class(`MorphologicalGradientImageFilter_GetKernelType`) = c("SWIGFunction", class('MorphologicalGradientImageFilter_GetKernelType'))

# Start of MorphologicalGradientImageFilter_GetName

`MorphologicalGradientImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MorphologicalGradientImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MorphologicalGradientImageFilter_GetName`, 'returnType') = 'character'
attr(`MorphologicalGradientImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MorphologicalGradientImageFilter')
class(`MorphologicalGradientImageFilter_GetName`) = c("SWIGFunction", class('MorphologicalGradientImageFilter_GetName'))

# Start of MorphologicalGradientImageFilter_ToString

`MorphologicalGradientImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MorphologicalGradientImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MorphologicalGradientImageFilter_ToString`, 'returnType') = 'character'
attr(`MorphologicalGradientImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MorphologicalGradientImageFilter')
class(`MorphologicalGradientImageFilter_ToString`) = c("SWIGFunction", class('MorphologicalGradientImageFilter_ToString'))

# Start of MorphologicalGradientImageFilter_Execute

`MorphologicalGradientImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MorphologicalGradientImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MorphologicalGradientImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`MorphologicalGradientImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__MorphologicalGradientImageFilter', '_p_itk__simple__Image')
class(`MorphologicalGradientImageFilter_Execute`) = c("SWIGFunction", class('MorphologicalGradientImageFilter_Execute'))

# Start of accessor method for itk::simple::MorphologicalGradientImageFilter
setMethod('$', '_p_itk__simple__MorphologicalGradientImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = MorphologicalGradientImageFilter_SetKernelRadius, 'GetKernelRadius' = MorphologicalGradientImageFilter_GetKernelRadius, 'SetKernelType' = MorphologicalGradientImageFilter_SetKernelType, 'GetKernelType' = MorphologicalGradientImageFilter_GetKernelType, 'GetName' = MorphologicalGradientImageFilter_GetName, 'ToString' = MorphologicalGradientImageFilter_ToString, 'Execute' = MorphologicalGradientImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MorphologicalGradientImageFilter
# Start of MorphologicalGradient

`MorphologicalGradient__SWIG_0` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_MorphologicalGradient__SWIG_0', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MorphologicalGradient__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MorphologicalGradient__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`MorphologicalGradient__SWIG_0`) = c("SWIGFunction", class('MorphologicalGradient__SWIG_0'))

# Start of MorphologicalGradient

`MorphologicalGradient__SWIG_1` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_MorphologicalGradient__SWIG_1', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MorphologicalGradient__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MorphologicalGradient__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`MorphologicalGradient__SWIG_1`) = c("SWIGFunction", class('MorphologicalGradient__SWIG_1'))

# Start of MorphologicalGradient

`MorphologicalGradient__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MorphologicalGradient__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MorphologicalGradient__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MorphologicalGradient__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`MorphologicalGradient__SWIG_2`) = c("SWIGFunction", class('MorphologicalGradient__SWIG_2'))

`MorphologicalGradient` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- MorphologicalGradient__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- MorphologicalGradient__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- MorphologicalGradient__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MorphologicalGradient with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MorphologicalWatershedFromMarkersImageFilter

`delete_MorphologicalWatershedFromMarkersImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MorphologicalWatershedFromMarkersImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MorphologicalWatershedFromMarkersImageFilter`, 'returnType') = 'void'
attr(`delete_MorphologicalWatershedFromMarkersImageFilter`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter')
class(`delete_MorphologicalWatershedFromMarkersImageFilter`) = c("SWIGFunction", class('delete_MorphologicalWatershedFromMarkersImageFilter'))

# Start of new_MorphologicalWatershedFromMarkersImageFilter

`MorphologicalWatershedFromMarkersImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MorphologicalWatershedFromMarkersImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MorphologicalWatershedFromMarkersImageFilter);
  ans
  
}

attr(`MorphologicalWatershedFromMarkersImageFilter`, 'returnType') = '_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter'
class(`MorphologicalWatershedFromMarkersImageFilter`) = c("SWIGFunction", class('MorphologicalWatershedFromMarkersImageFilter'))

# Start of MorphologicalWatershedFromMarkersImageFilter_SetMarkWatershedLine

`MorphologicalWatershedFromMarkersImageFilter_SetMarkWatershedLine` = function(self, MarkWatershedLine)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MarkWatershedLine = as.logical(MarkWatershedLine);
  ;ans = .Call('R_swig_MorphologicalWatershedFromMarkersImageFilter_SetMarkWatershedLine', self, MarkWatershedLine, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalWatershedFromMarkersImageFilter_SetMarkWatershedLine`, 'returnType') = 'void'
attr(`MorphologicalWatershedFromMarkersImageFilter_SetMarkWatershedLine`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter', 'logical')
class(`MorphologicalWatershedFromMarkersImageFilter_SetMarkWatershedLine`) = c("SWIGFunction", class('MorphologicalWatershedFromMarkersImageFilter_SetMarkWatershedLine'))

# Start of MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOn

`MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOn`, 'returnType') = 'void'
attr(`MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOn`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter')
class(`MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOn`) = c("SWIGFunction", class('MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOn'))

# Start of MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOff

`MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOff`, 'returnType') = 'void'
attr(`MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOff`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter')
class(`MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOff`) = c("SWIGFunction", class('MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOff'))

# Start of MorphologicalWatershedFromMarkersImageFilter_GetMarkWatershedLine

`MorphologicalWatershedFromMarkersImageFilter_GetMarkWatershedLine` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MorphologicalWatershedFromMarkersImageFilter_GetMarkWatershedLine', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MorphologicalWatershedFromMarkersImageFilter_GetMarkWatershedLine`, 'returnType') = 'logical'
attr(`MorphologicalWatershedFromMarkersImageFilter_GetMarkWatershedLine`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter')
class(`MorphologicalWatershedFromMarkersImageFilter_GetMarkWatershedLine`) = c("SWIGFunction", class('MorphologicalWatershedFromMarkersImageFilter_GetMarkWatershedLine'))

# Start of MorphologicalWatershedFromMarkersImageFilter_SetFullyConnected

`MorphologicalWatershedFromMarkersImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_MorphologicalWatershedFromMarkersImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalWatershedFromMarkersImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`MorphologicalWatershedFromMarkersImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter', 'logical')
class(`MorphologicalWatershedFromMarkersImageFilter_SetFullyConnected`) = c("SWIGFunction", class('MorphologicalWatershedFromMarkersImageFilter_SetFullyConnected'))

# Start of MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOn

`MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter')
class(`MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOn'))

# Start of MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOff

`MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter')
class(`MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOff'))

# Start of MorphologicalWatershedFromMarkersImageFilter_GetFullyConnected

`MorphologicalWatershedFromMarkersImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MorphologicalWatershedFromMarkersImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MorphologicalWatershedFromMarkersImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`MorphologicalWatershedFromMarkersImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter')
class(`MorphologicalWatershedFromMarkersImageFilter_GetFullyConnected`) = c("SWIGFunction", class('MorphologicalWatershedFromMarkersImageFilter_GetFullyConnected'))

# Start of MorphologicalWatershedFromMarkersImageFilter_GetName

`MorphologicalWatershedFromMarkersImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MorphologicalWatershedFromMarkersImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MorphologicalWatershedFromMarkersImageFilter_GetName`, 'returnType') = 'character'
attr(`MorphologicalWatershedFromMarkersImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter')
class(`MorphologicalWatershedFromMarkersImageFilter_GetName`) = c("SWIGFunction", class('MorphologicalWatershedFromMarkersImageFilter_GetName'))

# Start of MorphologicalWatershedFromMarkersImageFilter_ToString

`MorphologicalWatershedFromMarkersImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MorphologicalWatershedFromMarkersImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MorphologicalWatershedFromMarkersImageFilter_ToString`, 'returnType') = 'character'
attr(`MorphologicalWatershedFromMarkersImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter')
class(`MorphologicalWatershedFromMarkersImageFilter_ToString`) = c("SWIGFunction", class('MorphologicalWatershedFromMarkersImageFilter_ToString'))

# Start of MorphologicalWatershedFromMarkersImageFilter_Execute

`MorphologicalWatershedFromMarkersImageFilter_Execute` = function(self, image, markerImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  ;ans = .Call('R_swig_MorphologicalWatershedFromMarkersImageFilter_Execute', self, image, markerImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MorphologicalWatershedFromMarkersImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`MorphologicalWatershedFromMarkersImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MorphologicalWatershedFromMarkersImageFilter_Execute`) = c("SWIGFunction", class('MorphologicalWatershedFromMarkersImageFilter_Execute'))

# Start of accessor method for itk::simple::MorphologicalWatershedFromMarkersImageFilter
setMethod('$', '_p_itk__simple__MorphologicalWatershedFromMarkersImageFilter', function(x, name)

{
  accessorFuns = list('SetMarkWatershedLine' = MorphologicalWatershedFromMarkersImageFilter_SetMarkWatershedLine, 'MarkWatershedLineOn' = MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOn, 'MarkWatershedLineOff' = MorphologicalWatershedFromMarkersImageFilter_MarkWatershedLineOff, 'GetMarkWatershedLine' = MorphologicalWatershedFromMarkersImageFilter_GetMarkWatershedLine, 'SetFullyConnected' = MorphologicalWatershedFromMarkersImageFilter_SetFullyConnected, 'FullyConnectedOn' = MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOn, 'FullyConnectedOff' = MorphologicalWatershedFromMarkersImageFilter_FullyConnectedOff, 'GetFullyConnected' = MorphologicalWatershedFromMarkersImageFilter_GetFullyConnected, 'GetName' = MorphologicalWatershedFromMarkersImageFilter_GetName, 'ToString' = MorphologicalWatershedFromMarkersImageFilter_ToString, 'Execute' = MorphologicalWatershedFromMarkersImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MorphologicalWatershedFromMarkersImageFilter
# Start of MorphologicalWatershedFromMarkers

`MorphologicalWatershedFromMarkers__SWIG_0` = function(image, markerImage, markWatershedLine, fullyConnected, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  markWatershedLine = as.logical(markWatershedLine);
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_MorphologicalWatershedFromMarkers__SWIG_0', image, markerImage, markWatershedLine, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MorphologicalWatershedFromMarkers__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MorphologicalWatershedFromMarkers__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical', 'logical')
class(`MorphologicalWatershedFromMarkers__SWIG_0`) = c("SWIGFunction", class('MorphologicalWatershedFromMarkers__SWIG_0'))

# Start of MorphologicalWatershedFromMarkers

`MorphologicalWatershedFromMarkers__SWIG_1` = function(image, markerImage, markWatershedLine, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  markWatershedLine = as.logical(markWatershedLine);
  ;ans = .Call('R_swig_MorphologicalWatershedFromMarkers__SWIG_1', image, markerImage, markWatershedLine, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MorphologicalWatershedFromMarkers__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MorphologicalWatershedFromMarkers__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical')
class(`MorphologicalWatershedFromMarkers__SWIG_1`) = c("SWIGFunction", class('MorphologicalWatershedFromMarkers__SWIG_1'))

# Start of MorphologicalWatershedFromMarkers

`MorphologicalWatershedFromMarkers__SWIG_2` = function(image, markerImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  ;ans = .Call('R_swig_MorphologicalWatershedFromMarkers__SWIG_2', image, markerImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MorphologicalWatershedFromMarkers__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MorphologicalWatershedFromMarkers__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MorphologicalWatershedFromMarkers__SWIG_2`) = c("SWIGFunction", class('MorphologicalWatershedFromMarkers__SWIG_2'))

`MorphologicalWatershedFromMarkers` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- MorphologicalWatershedFromMarkers__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- MorphologicalWatershedFromMarkers__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- MorphologicalWatershedFromMarkers__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MorphologicalWatershedFromMarkers with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MorphologicalWatershedImageFilter

`delete_MorphologicalWatershedImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MorphologicalWatershedImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MorphologicalWatershedImageFilter`, 'returnType') = 'void'
attr(`delete_MorphologicalWatershedImageFilter`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedImageFilter')
class(`delete_MorphologicalWatershedImageFilter`) = c("SWIGFunction", class('delete_MorphologicalWatershedImageFilter'))

# Start of new_MorphologicalWatershedImageFilter

`MorphologicalWatershedImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MorphologicalWatershedImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MorphologicalWatershedImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MorphologicalWatershedImageFilter);
  ans
  
}

attr(`MorphologicalWatershedImageFilter`, 'returnType') = '_p_itk__simple__MorphologicalWatershedImageFilter'
class(`MorphologicalWatershedImageFilter`) = c("SWIGFunction", class('MorphologicalWatershedImageFilter'))

# Start of MorphologicalWatershedImageFilter_SetLevel

`MorphologicalWatershedImageFilter_SetLevel` = function(self, Level)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_MorphologicalWatershedImageFilter_SetLevel', self, Level, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalWatershedImageFilter_SetLevel`, 'returnType') = 'void'
attr(`MorphologicalWatershedImageFilter_SetLevel`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedImageFilter', 'numeric')
class(`MorphologicalWatershedImageFilter_SetLevel`) = c("SWIGFunction", class('MorphologicalWatershedImageFilter_SetLevel'))

# Start of MorphologicalWatershedImageFilter_GetLevel

`MorphologicalWatershedImageFilter_GetLevel` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MorphologicalWatershedImageFilter_GetLevel', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MorphologicalWatershedImageFilter_GetLevel`, 'returnType') = 'numeric'
attr(`MorphologicalWatershedImageFilter_GetLevel`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedImageFilter')
class(`MorphologicalWatershedImageFilter_GetLevel`) = c("SWIGFunction", class('MorphologicalWatershedImageFilter_GetLevel'))

# Start of MorphologicalWatershedImageFilter_SetMarkWatershedLine

`MorphologicalWatershedImageFilter_SetMarkWatershedLine` = function(self, MarkWatershedLine)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MarkWatershedLine = as.logical(MarkWatershedLine);
  ;ans = .Call('R_swig_MorphologicalWatershedImageFilter_SetMarkWatershedLine', self, MarkWatershedLine, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalWatershedImageFilter_SetMarkWatershedLine`, 'returnType') = 'void'
attr(`MorphologicalWatershedImageFilter_SetMarkWatershedLine`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedImageFilter', 'logical')
class(`MorphologicalWatershedImageFilter_SetMarkWatershedLine`) = c("SWIGFunction", class('MorphologicalWatershedImageFilter_SetMarkWatershedLine'))

# Start of MorphologicalWatershedImageFilter_MarkWatershedLineOn

`MorphologicalWatershedImageFilter_MarkWatershedLineOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_MorphologicalWatershedImageFilter_MarkWatershedLineOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalWatershedImageFilter_MarkWatershedLineOn`, 'returnType') = 'void'
attr(`MorphologicalWatershedImageFilter_MarkWatershedLineOn`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedImageFilter')
class(`MorphologicalWatershedImageFilter_MarkWatershedLineOn`) = c("SWIGFunction", class('MorphologicalWatershedImageFilter_MarkWatershedLineOn'))

# Start of MorphologicalWatershedImageFilter_MarkWatershedLineOff

`MorphologicalWatershedImageFilter_MarkWatershedLineOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_MorphologicalWatershedImageFilter_MarkWatershedLineOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalWatershedImageFilter_MarkWatershedLineOff`, 'returnType') = 'void'
attr(`MorphologicalWatershedImageFilter_MarkWatershedLineOff`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedImageFilter')
class(`MorphologicalWatershedImageFilter_MarkWatershedLineOff`) = c("SWIGFunction", class('MorphologicalWatershedImageFilter_MarkWatershedLineOff'))

# Start of MorphologicalWatershedImageFilter_GetMarkWatershedLine

`MorphologicalWatershedImageFilter_GetMarkWatershedLine` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MorphologicalWatershedImageFilter_GetMarkWatershedLine', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MorphologicalWatershedImageFilter_GetMarkWatershedLine`, 'returnType') = 'logical'
attr(`MorphologicalWatershedImageFilter_GetMarkWatershedLine`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedImageFilter')
class(`MorphologicalWatershedImageFilter_GetMarkWatershedLine`) = c("SWIGFunction", class('MorphologicalWatershedImageFilter_GetMarkWatershedLine'))

# Start of MorphologicalWatershedImageFilter_SetFullyConnected

`MorphologicalWatershedImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_MorphologicalWatershedImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalWatershedImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`MorphologicalWatershedImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedImageFilter', 'logical')
class(`MorphologicalWatershedImageFilter_SetFullyConnected`) = c("SWIGFunction", class('MorphologicalWatershedImageFilter_SetFullyConnected'))

# Start of MorphologicalWatershedImageFilter_FullyConnectedOn

`MorphologicalWatershedImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_MorphologicalWatershedImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalWatershedImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`MorphologicalWatershedImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedImageFilter')
class(`MorphologicalWatershedImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('MorphologicalWatershedImageFilter_FullyConnectedOn'))

# Start of MorphologicalWatershedImageFilter_FullyConnectedOff

`MorphologicalWatershedImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_MorphologicalWatershedImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MorphologicalWatershedImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`MorphologicalWatershedImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedImageFilter')
class(`MorphologicalWatershedImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('MorphologicalWatershedImageFilter_FullyConnectedOff'))

# Start of MorphologicalWatershedImageFilter_GetFullyConnected

`MorphologicalWatershedImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MorphologicalWatershedImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MorphologicalWatershedImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`MorphologicalWatershedImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedImageFilter')
class(`MorphologicalWatershedImageFilter_GetFullyConnected`) = c("SWIGFunction", class('MorphologicalWatershedImageFilter_GetFullyConnected'))

# Start of MorphologicalWatershedImageFilter_GetName

`MorphologicalWatershedImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MorphologicalWatershedImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MorphologicalWatershedImageFilter_GetName`, 'returnType') = 'character'
attr(`MorphologicalWatershedImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedImageFilter')
class(`MorphologicalWatershedImageFilter_GetName`) = c("SWIGFunction", class('MorphologicalWatershedImageFilter_GetName'))

# Start of MorphologicalWatershedImageFilter_ToString

`MorphologicalWatershedImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MorphologicalWatershedImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MorphologicalWatershedImageFilter_ToString`, 'returnType') = 'character'
attr(`MorphologicalWatershedImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedImageFilter')
class(`MorphologicalWatershedImageFilter_ToString`) = c("SWIGFunction", class('MorphologicalWatershedImageFilter_ToString'))

# Start of MorphologicalWatershedImageFilter_Execute

`MorphologicalWatershedImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MorphologicalWatershedImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MorphologicalWatershedImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`MorphologicalWatershedImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__MorphologicalWatershedImageFilter', '_p_itk__simple__Image')
class(`MorphologicalWatershedImageFilter_Execute`) = c("SWIGFunction", class('MorphologicalWatershedImageFilter_Execute'))

# Start of accessor method for itk::simple::MorphologicalWatershedImageFilter
setMethod('$', '_p_itk__simple__MorphologicalWatershedImageFilter', function(x, name)

{
  accessorFuns = list('SetLevel' = MorphologicalWatershedImageFilter_SetLevel, 'GetLevel' = MorphologicalWatershedImageFilter_GetLevel, 'SetMarkWatershedLine' = MorphologicalWatershedImageFilter_SetMarkWatershedLine, 'MarkWatershedLineOn' = MorphologicalWatershedImageFilter_MarkWatershedLineOn, 'MarkWatershedLineOff' = MorphologicalWatershedImageFilter_MarkWatershedLineOff, 'GetMarkWatershedLine' = MorphologicalWatershedImageFilter_GetMarkWatershedLine, 'SetFullyConnected' = MorphologicalWatershedImageFilter_SetFullyConnected, 'FullyConnectedOn' = MorphologicalWatershedImageFilter_FullyConnectedOn, 'FullyConnectedOff' = MorphologicalWatershedImageFilter_FullyConnectedOff, 'GetFullyConnected' = MorphologicalWatershedImageFilter_GetFullyConnected, 'GetName' = MorphologicalWatershedImageFilter_GetName, 'ToString' = MorphologicalWatershedImageFilter_ToString, 'Execute' = MorphologicalWatershedImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MorphologicalWatershedImageFilter
# Start of MorphologicalWatershed

`MorphologicalWatershed__SWIG_0` = function(image1, level, markWatershedLine, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  markWatershedLine = as.logical(markWatershedLine);
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_MorphologicalWatershed__SWIG_0', image1, level, markWatershedLine, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MorphologicalWatershed__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MorphologicalWatershed__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical', 'logical')
class(`MorphologicalWatershed__SWIG_0`) = c("SWIGFunction", class('MorphologicalWatershed__SWIG_0'))

# Start of MorphologicalWatershed

`MorphologicalWatershed__SWIG_1` = function(image1, level, markWatershedLine, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  markWatershedLine = as.logical(markWatershedLine);
  ;ans = .Call('R_swig_MorphologicalWatershed__SWIG_1', image1, level, markWatershedLine, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MorphologicalWatershed__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MorphologicalWatershed__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`MorphologicalWatershed__SWIG_1`) = c("SWIGFunction", class('MorphologicalWatershed__SWIG_1'))

# Start of MorphologicalWatershed

`MorphologicalWatershed__SWIG_2` = function(image1, level, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_MorphologicalWatershed__SWIG_2', image1, level, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MorphologicalWatershed__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MorphologicalWatershed__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`MorphologicalWatershed__SWIG_2`) = c("SWIGFunction", class('MorphologicalWatershed__SWIG_2'))

# Start of MorphologicalWatershed

`MorphologicalWatershed__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MorphologicalWatershed__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MorphologicalWatershed__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`MorphologicalWatershed__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`MorphologicalWatershed__SWIG_3`) = c("SWIGFunction", class('MorphologicalWatershed__SWIG_3'))

`MorphologicalWatershed` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- MorphologicalWatershed__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- MorphologicalWatershed__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- MorphologicalWatershed__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- MorphologicalWatershed__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MorphologicalWatershed with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MultiLabelSTAPLEImageFilter

`delete_MultiLabelSTAPLEImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MultiLabelSTAPLEImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MultiLabelSTAPLEImageFilter`, 'returnType') = 'void'
attr(`delete_MultiLabelSTAPLEImageFilter`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter')
class(`delete_MultiLabelSTAPLEImageFilter`) = c("SWIGFunction", class('delete_MultiLabelSTAPLEImageFilter'))

# Start of new_MultiLabelSTAPLEImageFilter

`MultiLabelSTAPLEImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MultiLabelSTAPLEImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MultiLabelSTAPLEImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MultiLabelSTAPLEImageFilter);
  ans
  
}

attr(`MultiLabelSTAPLEImageFilter`, 'returnType') = '_p_itk__simple__MultiLabelSTAPLEImageFilter'
class(`MultiLabelSTAPLEImageFilter`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter'))

# Start of MultiLabelSTAPLEImageFilter_SetLabelForUndecidedPixels

`MultiLabelSTAPLEImageFilter_SetLabelForUndecidedPixels` = function(self, LabelForUndecidedPixels)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  LabelForUndecidedPixels = as.integer(LabelForUndecidedPixels);
  
  if(length(LabelForUndecidedPixels) > 1) {
    warning("using only the first element of LabelForUndecidedPixels");
  };
  
  ;ans = .Call('R_swig_MultiLabelSTAPLEImageFilter_SetLabelForUndecidedPixels', self, LabelForUndecidedPixels, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLEImageFilter_SetLabelForUndecidedPixels`, 'returnType') = 'void'
attr(`MultiLabelSTAPLEImageFilter_SetLabelForUndecidedPixels`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter', 'integer')
class(`MultiLabelSTAPLEImageFilter_SetLabelForUndecidedPixels`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_SetLabelForUndecidedPixels'))

# Start of MultiLabelSTAPLEImageFilter_GetLabelForUndecidedPixels

`MultiLabelSTAPLEImageFilter_GetLabelForUndecidedPixels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MultiLabelSTAPLEImageFilter_GetLabelForUndecidedPixels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MultiLabelSTAPLEImageFilter_GetLabelForUndecidedPixels`, 'returnType') = 'integer'
attr(`MultiLabelSTAPLEImageFilter_GetLabelForUndecidedPixels`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter')
class(`MultiLabelSTAPLEImageFilter_GetLabelForUndecidedPixels`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_GetLabelForUndecidedPixels'))

# Start of MultiLabelSTAPLEImageFilter_SetTerminationUpdateThreshold

`MultiLabelSTAPLEImageFilter_SetTerminationUpdateThreshold` = function(self, TerminationUpdateThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_MultiLabelSTAPLEImageFilter_SetTerminationUpdateThreshold', self, TerminationUpdateThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLEImageFilter_SetTerminationUpdateThreshold`, 'returnType') = 'void'
attr(`MultiLabelSTAPLEImageFilter_SetTerminationUpdateThreshold`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter', 'numeric')
class(`MultiLabelSTAPLEImageFilter_SetTerminationUpdateThreshold`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_SetTerminationUpdateThreshold'))

# Start of MultiLabelSTAPLEImageFilter_GetTerminationUpdateThreshold

`MultiLabelSTAPLEImageFilter_GetTerminationUpdateThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MultiLabelSTAPLEImageFilter_GetTerminationUpdateThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MultiLabelSTAPLEImageFilter_GetTerminationUpdateThreshold`, 'returnType') = 'numeric'
attr(`MultiLabelSTAPLEImageFilter_GetTerminationUpdateThreshold`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter')
class(`MultiLabelSTAPLEImageFilter_GetTerminationUpdateThreshold`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_GetTerminationUpdateThreshold'))

# Start of MultiLabelSTAPLEImageFilter_SetMaximumNumberOfIterations

`MultiLabelSTAPLEImageFilter_SetMaximumNumberOfIterations` = function(self, MaximumNumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaximumNumberOfIterations = as.integer(MaximumNumberOfIterations);
  
  if(length(MaximumNumberOfIterations) > 1) {
    warning("using only the first element of MaximumNumberOfIterations");
  };
  
  ;ans = .Call('R_swig_MultiLabelSTAPLEImageFilter_SetMaximumNumberOfIterations', self, MaximumNumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLEImageFilter_SetMaximumNumberOfIterations`, 'returnType') = 'void'
attr(`MultiLabelSTAPLEImageFilter_SetMaximumNumberOfIterations`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter', 'integer')
class(`MultiLabelSTAPLEImageFilter_SetMaximumNumberOfIterations`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_SetMaximumNumberOfIterations'))

# Start of MultiLabelSTAPLEImageFilter_GetMaximumNumberOfIterations

`MultiLabelSTAPLEImageFilter_GetMaximumNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MultiLabelSTAPLEImageFilter_GetMaximumNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MultiLabelSTAPLEImageFilter_GetMaximumNumberOfIterations`, 'returnType') = 'integer'
attr(`MultiLabelSTAPLEImageFilter_GetMaximumNumberOfIterations`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter')
class(`MultiLabelSTAPLEImageFilter_GetMaximumNumberOfIterations`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_GetMaximumNumberOfIterations'))

# Start of MultiLabelSTAPLEImageFilter_SetPriorProbabilities

`MultiLabelSTAPLEImageFilter_SetPriorProbabilities` = function(self, PriorProbabilities)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  PriorProbabilities = as.numeric(PriorProbabilities);
  ;ans = .Call('R_swig_MultiLabelSTAPLEImageFilter_SetPriorProbabilities', self, PriorProbabilities, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLEImageFilter_SetPriorProbabilities`, 'returnType') = 'void'
attr(`MultiLabelSTAPLEImageFilter_SetPriorProbabilities`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter', 'numeric')
class(`MultiLabelSTAPLEImageFilter_SetPriorProbabilities`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_SetPriorProbabilities'))

# Start of MultiLabelSTAPLEImageFilter_GetPriorProbabilities

`MultiLabelSTAPLEImageFilter_GetPriorProbabilities` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MultiLabelSTAPLEImageFilter_GetPriorProbabilities', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MultiLabelSTAPLEImageFilter_GetPriorProbabilities`, 'returnType') = 'numeric'
attr(`MultiLabelSTAPLEImageFilter_GetPriorProbabilities`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter')
class(`MultiLabelSTAPLEImageFilter_GetPriorProbabilities`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_GetPriorProbabilities'))

# Start of MultiLabelSTAPLEImageFilter_GetConfusionMatrix

`MultiLabelSTAPLEImageFilter_GetConfusionMatrix` = function(self, input, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  input = as.integer(input);
  
  if(length(input) > 1) {
    warning("using only the first element of input");
  };
  
  ;.Call('R_swig_MultiLabelSTAPLEImageFilter_GetConfusionMatrix', self, input, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MultiLabelSTAPLEImageFilter_GetConfusionMatrix`, 'returnType') = 'numeric'
attr(`MultiLabelSTAPLEImageFilter_GetConfusionMatrix`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter', 'integer')
class(`MultiLabelSTAPLEImageFilter_GetConfusionMatrix`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_GetConfusionMatrix'))

# Start of MultiLabelSTAPLEImageFilter_GetName

`MultiLabelSTAPLEImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MultiLabelSTAPLEImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MultiLabelSTAPLEImageFilter_GetName`, 'returnType') = 'character'
attr(`MultiLabelSTAPLEImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter')
class(`MultiLabelSTAPLEImageFilter_GetName`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_GetName'))

# Start of MultiLabelSTAPLEImageFilter_ToString

`MultiLabelSTAPLEImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MultiLabelSTAPLEImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MultiLabelSTAPLEImageFilter_ToString`, 'returnType') = 'character'
attr(`MultiLabelSTAPLEImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter')
class(`MultiLabelSTAPLEImageFilter_ToString`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_ToString'))

# Start of MultiLabelSTAPLEImageFilter_Execute

`MultiLabelSTAPLEImageFilter_Execute__SWIG_0` = function(self, images, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_MultiLabelSTAPLEImageFilter_Execute__SWIG_0', self, images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLEImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLEImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter', 'list')
class(`MultiLabelSTAPLEImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_Execute__SWIG_0'))

# Start of MultiLabelSTAPLEImageFilter_Execute

`MultiLabelSTAPLEImageFilter_Execute__SWIG_1` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MultiLabelSTAPLEImageFilter_Execute__SWIG_1', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLEImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLEImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter', '_p_itk__simple__Image')
class(`MultiLabelSTAPLEImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_Execute__SWIG_1'))

# Start of MultiLabelSTAPLEImageFilter_Execute

`MultiLabelSTAPLEImageFilter_Execute__SWIG_2` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_MultiLabelSTAPLEImageFilter_Execute__SWIG_2', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLEImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLEImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MultiLabelSTAPLEImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_Execute__SWIG_2'))

# Start of MultiLabelSTAPLEImageFilter_Execute

`MultiLabelSTAPLEImageFilter_Execute__SWIG_3` = function(self, image1, image2, image3, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_MultiLabelSTAPLEImageFilter_Execute__SWIG_3', self, image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLEImageFilter_Execute__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLEImageFilter_Execute__SWIG_3`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MultiLabelSTAPLEImageFilter_Execute__SWIG_3`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_Execute__SWIG_3'))

# Start of MultiLabelSTAPLEImageFilter_Execute

`MultiLabelSTAPLEImageFilter_Execute__SWIG_4` = function(self, image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_MultiLabelSTAPLEImageFilter_Execute__SWIG_4', self, image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLEImageFilter_Execute__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLEImageFilter_Execute__SWIG_4`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MultiLabelSTAPLEImageFilter_Execute__SWIG_4`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_Execute__SWIG_4'))

# Start of MultiLabelSTAPLEImageFilter_Execute

`MultiLabelSTAPLEImageFilter_Execute__SWIG_5` = function(self, image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_MultiLabelSTAPLEImageFilter_Execute__SWIG_5', self, image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLEImageFilter_Execute__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLEImageFilter_Execute__SWIG_5`, "inputTypes") = c('_p_itk__simple__MultiLabelSTAPLEImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MultiLabelSTAPLEImageFilter_Execute__SWIG_5`) = c("SWIGFunction", class('MultiLabelSTAPLEImageFilter_Execute__SWIG_5'))

`MultiLabelSTAPLEImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__MultiLabelSTAPLEImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- MultiLabelSTAPLEImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__MultiLabelSTAPLEImageFilter') || is.null(argv[[1]]) ) && (
        (is.list(argv[[2]]) & all(sapply(argv[[2]], inherits, what='_p_itk__simple__Image')))
        )) {
      f <- MultiLabelSTAPLEImageFilter_Execute__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__MultiLabelSTAPLEImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MultiLabelSTAPLEImageFilter_Execute__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__MultiLabelSTAPLEImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- MultiLabelSTAPLEImageFilter_Execute__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__MultiLabelSTAPLEImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- MultiLabelSTAPLEImageFilter_Execute__SWIG_4; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__MultiLabelSTAPLEImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( extends(argtypes[6], '_p_itk__simple__Image') && length(argv[[6]]) == 1 )) {
      f <- MultiLabelSTAPLEImageFilter_Execute__SWIG_5; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MultiLabelSTAPLEImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::MultiLabelSTAPLEImageFilter
setMethod('$', '_p_itk__simple__MultiLabelSTAPLEImageFilter', function(x, name)

{
  accessorFuns = list('SetLabelForUndecidedPixels' = MultiLabelSTAPLEImageFilter_SetLabelForUndecidedPixels, 'GetLabelForUndecidedPixels' = MultiLabelSTAPLEImageFilter_GetLabelForUndecidedPixels, 'SetTerminationUpdateThreshold' = MultiLabelSTAPLEImageFilter_SetTerminationUpdateThreshold, 'GetTerminationUpdateThreshold' = MultiLabelSTAPLEImageFilter_GetTerminationUpdateThreshold, 'SetMaximumNumberOfIterations' = MultiLabelSTAPLEImageFilter_SetMaximumNumberOfIterations, 'GetMaximumNumberOfIterations' = MultiLabelSTAPLEImageFilter_GetMaximumNumberOfIterations, 'SetPriorProbabilities' = MultiLabelSTAPLEImageFilter_SetPriorProbabilities, 'GetPriorProbabilities' = MultiLabelSTAPLEImageFilter_GetPriorProbabilities, 'GetConfusionMatrix' = MultiLabelSTAPLEImageFilter_GetConfusionMatrix, 'GetName' = MultiLabelSTAPLEImageFilter_GetName, 'ToString' = MultiLabelSTAPLEImageFilter_ToString, 'Execute' = MultiLabelSTAPLEImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MultiLabelSTAPLEImageFilter
# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_0` = function(images, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, priorProbabilities, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  priorProbabilities = as.numeric(priorProbabilities);
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_0', images, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, priorProbabilities, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_0`, "inputTypes") = c('list', 'integer', 'numeric', 'integer', 'numeric')
class(`MultiLabelSTAPLE__SWIG_0`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_0'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_1` = function(images, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_1', images, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_1`, "inputTypes") = c('list', 'integer', 'numeric', 'integer')
class(`MultiLabelSTAPLE__SWIG_1`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_1'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_2` = function(images, labelForUndecidedPixels, terminationUpdateThreshold, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_2', images, labelForUndecidedPixels, terminationUpdateThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_2`, "inputTypes") = c('list', 'integer', 'numeric')
class(`MultiLabelSTAPLE__SWIG_2`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_2'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_3` = function(images, labelForUndecidedPixels, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_3', images, labelForUndecidedPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_3`, "inputTypes") = c('list', 'integer')
class(`MultiLabelSTAPLE__SWIG_3`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_3'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_4` = function(images, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_4', images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_4`, "inputTypes") = c('list')
class(`MultiLabelSTAPLE__SWIG_4`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_4'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_5` = function(image1, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, priorProbabilities, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  priorProbabilities = as.numeric(priorProbabilities);
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_5', image1, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, priorProbabilities, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'integer', 'numeric')
class(`MultiLabelSTAPLE__SWIG_5`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_5'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_6` = function(image1, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_6', image1, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'integer')
class(`MultiLabelSTAPLE__SWIG_6`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_6'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_7` = function(image1, labelForUndecidedPixels, terminationUpdateThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_7', image1, labelForUndecidedPixels, terminationUpdateThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric')
class(`MultiLabelSTAPLE__SWIG_7`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_7'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_8` = function(image1, labelForUndecidedPixels, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_8', image1, labelForUndecidedPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`MultiLabelSTAPLE__SWIG_8`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_8'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_9` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_9', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image')
class(`MultiLabelSTAPLE__SWIG_9`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_9'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_10` = function(image1, image2, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, priorProbabilities, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  priorProbabilities = as.numeric(priorProbabilities);
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_10', image1, image2, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, priorProbabilities, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric', 'integer', 'numeric')
class(`MultiLabelSTAPLE__SWIG_10`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_10'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_11` = function(image1, image2, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_11', image1, image2, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric', 'integer')
class(`MultiLabelSTAPLE__SWIG_11`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_11'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_12` = function(image1, image2, labelForUndecidedPixels, terminationUpdateThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_12', image1, image2, labelForUndecidedPixels, terminationUpdateThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_12`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_12`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric')
class(`MultiLabelSTAPLE__SWIG_12`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_12'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_13` = function(image1, image2, labelForUndecidedPixels, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_13', image1, image2, labelForUndecidedPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_13`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_13`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`MultiLabelSTAPLE__SWIG_13`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_13'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_14` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_14', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_14`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_14`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MultiLabelSTAPLE__SWIG_14`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_14'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_15` = function(image1, image2, image3, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, priorProbabilities, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  priorProbabilities = as.numeric(priorProbabilities);
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_15', image1, image2, image3, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, priorProbabilities, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_15`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_15`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric', 'integer', 'numeric')
class(`MultiLabelSTAPLE__SWIG_15`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_15'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_16` = function(image1, image2, image3, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_16', image1, image2, image3, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_16`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_16`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric', 'integer')
class(`MultiLabelSTAPLE__SWIG_16`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_16'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_17` = function(image1, image2, image3, labelForUndecidedPixels, terminationUpdateThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_17', image1, image2, image3, labelForUndecidedPixels, terminationUpdateThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_17`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_17`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric')
class(`MultiLabelSTAPLE__SWIG_17`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_17'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_18` = function(image1, image2, image3, labelForUndecidedPixels, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_18', image1, image2, image3, labelForUndecidedPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_18`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_18`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`MultiLabelSTAPLE__SWIG_18`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_18'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_19` = function(image1, image2, image3, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_19', image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_19`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_19`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MultiLabelSTAPLE__SWIG_19`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_19'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_20` = function(image1, image2, image3, image4, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, priorProbabilities, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  priorProbabilities = as.numeric(priorProbabilities);
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_20', image1, image2, image3, image4, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, priorProbabilities, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_20`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_20`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric', 'integer', 'numeric')
class(`MultiLabelSTAPLE__SWIG_20`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_20'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_21` = function(image1, image2, image3, image4, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_21', image1, image2, image3, image4, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_21`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_21`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric', 'integer')
class(`MultiLabelSTAPLE__SWIG_21`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_21'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_22` = function(image1, image2, image3, image4, labelForUndecidedPixels, terminationUpdateThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_22', image1, image2, image3, image4, labelForUndecidedPixels, terminationUpdateThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_22`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_22`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric')
class(`MultiLabelSTAPLE__SWIG_22`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_22'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_23` = function(image1, image2, image3, image4, labelForUndecidedPixels, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_23', image1, image2, image3, image4, labelForUndecidedPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_23`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_23`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`MultiLabelSTAPLE__SWIG_23`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_23'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_24` = function(image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_24', image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_24`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_24`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MultiLabelSTAPLE__SWIG_24`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_24'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_25` = function(image1, image2, image3, image4, image5, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, priorProbabilities, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  priorProbabilities = as.numeric(priorProbabilities);
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_25', image1, image2, image3, image4, image5, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, priorProbabilities, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_25`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_25`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric', 'integer', 'numeric')
class(`MultiLabelSTAPLE__SWIG_25`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_25'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_26` = function(image1, image2, image3, image4, image5, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_26', image1, image2, image3, image4, image5, labelForUndecidedPixels, terminationUpdateThreshold, maximumNumberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_26`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_26`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric', 'integer')
class(`MultiLabelSTAPLE__SWIG_26`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_26'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_27` = function(image1, image2, image3, image4, image5, labelForUndecidedPixels, terminationUpdateThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_27', image1, image2, image3, image4, image5, labelForUndecidedPixels, terminationUpdateThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_27`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_27`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric')
class(`MultiLabelSTAPLE__SWIG_27`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_27'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_28` = function(image1, image2, image3, image4, image5, labelForUndecidedPixels, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  labelForUndecidedPixels = as.integer(labelForUndecidedPixels);
  
  if(length(labelForUndecidedPixels) > 1) {
    warning("using only the first element of labelForUndecidedPixels");
  };
  
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_28', image1, image2, image3, image4, image5, labelForUndecidedPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_28`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_28`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`MultiLabelSTAPLE__SWIG_28`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_28'))

# Start of MultiLabelSTAPLE

`MultiLabelSTAPLE__SWIG_29` = function(image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_MultiLabelSTAPLE__SWIG_29', image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiLabelSTAPLE__SWIG_29`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiLabelSTAPLE__SWIG_29`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MultiLabelSTAPLE__SWIG_29`) = c("SWIGFunction", class('MultiLabelSTAPLE__SWIG_29'))

`MultiLabelSTAPLE` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 30
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- MultiLabelSTAPLE__SWIG_9; 
    }
    else if (
      (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
      ) {
      f <- MultiLabelSTAPLE__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_14; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_8; 
    }
    else if ((
        (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
        ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_19; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_13; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_7; 
    }
    else if ((
        (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
        ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_24; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_18; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_12; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_6; 
    }
    else if ((
        (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
        ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_29; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_23; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_17; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_11; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) )) {
      f <- MultiLabelSTAPLE__SWIG_5; 
    }
    else if ((
        (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
        ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) )) {
      f <- MultiLabelSTAPLE__SWIG_0; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_28; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_22; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_16; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) )) {
      f <- MultiLabelSTAPLE__SWIG_10; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_27; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_21; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) )) {
      f <- MultiLabelSTAPLE__SWIG_15; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( (is.integer(argv[[8]]) || is.numeric(argv[[8]])) && length(argv[[8]]) == 1 )) {
      f <- MultiLabelSTAPLE__SWIG_26; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) )) {
      f <- MultiLabelSTAPLE__SWIG_20; 
    }
  } else if (argc == 9) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( (is.integer(argv[[8]]) || is.numeric(argv[[8]])) && length(argv[[8]]) == 1 ) && ( is.numeric(argv[[9]]) )) {
      f <- MultiLabelSTAPLE__SWIG_25; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MultiLabelSTAPLE with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_MultiplyImageFilter

`delete_MultiplyImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_MultiplyImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_MultiplyImageFilter`, 'returnType') = 'void'
attr(`delete_MultiplyImageFilter`, "inputTypes") = c('_p_itk__simple__MultiplyImageFilter')
class(`delete_MultiplyImageFilter`) = c("SWIGFunction", class('delete_MultiplyImageFilter'))

# Start of new_MultiplyImageFilter

`MultiplyImageFilter` = function()
{
  ;ans = .Call('R_swig_new_MultiplyImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__MultiplyImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_MultiplyImageFilter);
  ans
  
}

attr(`MultiplyImageFilter`, 'returnType') = '_p_itk__simple__MultiplyImageFilter'
class(`MultiplyImageFilter`) = c("SWIGFunction", class('MultiplyImageFilter'))

# Start of MultiplyImageFilter_GetName

`MultiplyImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MultiplyImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MultiplyImageFilter_GetName`, 'returnType') = 'character'
attr(`MultiplyImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__MultiplyImageFilter')
class(`MultiplyImageFilter_GetName`) = c("SWIGFunction", class('MultiplyImageFilter_GetName'))

# Start of MultiplyImageFilter_ToString

`MultiplyImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_MultiplyImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`MultiplyImageFilter_ToString`, 'returnType') = 'character'
attr(`MultiplyImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__MultiplyImageFilter')
class(`MultiplyImageFilter_ToString`) = c("SWIGFunction", class('MultiplyImageFilter_ToString'))

# Start of MultiplyImageFilter_Execute

`MultiplyImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_MultiplyImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiplyImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiplyImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__MultiplyImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`MultiplyImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('MultiplyImageFilter_Execute__SWIG_0'))

# Start of MultiplyImageFilter_Execute

`MultiplyImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_MultiplyImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiplyImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiplyImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__MultiplyImageFilter', '_p_itk__simple__Image', 'numeric')
class(`MultiplyImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('MultiplyImageFilter_Execute__SWIG_1'))

# Start of MultiplyImageFilter_Execute

`MultiplyImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_MultiplyImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`MultiplyImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`MultiplyImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__MultiplyImageFilter', 'numeric', '_p_itk__simple__Image')
class(`MultiplyImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('MultiplyImageFilter_Execute__SWIG_2'))

`MultiplyImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__MultiplyImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MultiplyImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__MultiplyImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- MultiplyImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__MultiplyImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- MultiplyImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for MultiplyImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::MultiplyImageFilter
setMethod('$', '_p_itk__simple__MultiplyImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = MultiplyImageFilter_GetName, 'ToString' = MultiplyImageFilter_ToString, 'Execute' = MultiplyImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::MultiplyImageFilter
# Start of Multiply

`Multiply__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Multiply__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Multiply__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Multiply__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Multiply__SWIG_0`) = c("SWIGFunction", class('Multiply__SWIG_0'))

# Start of Multiply

`Multiply__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Multiply__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Multiply__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Multiply__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`Multiply__SWIG_1`) = c("SWIGFunction", class('Multiply__SWIG_1'))

# Start of Multiply

`Multiply__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Multiply__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Multiply__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Multiply__SWIG_2`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`Multiply__SWIG_2`) = c("SWIGFunction", class('Multiply__SWIG_2'))

`Multiply` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Multiply__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Multiply__SWIG_1; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Multiply__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Multiply with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_N4BiasFieldCorrectionImageFilter

`delete_N4BiasFieldCorrectionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_N4BiasFieldCorrectionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_N4BiasFieldCorrectionImageFilter`, 'returnType') = 'void'
attr(`delete_N4BiasFieldCorrectionImageFilter`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`delete_N4BiasFieldCorrectionImageFilter`) = c("SWIGFunction", class('delete_N4BiasFieldCorrectionImageFilter'))

# Start of new_N4BiasFieldCorrectionImageFilter

`N4BiasFieldCorrectionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_N4BiasFieldCorrectionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__N4BiasFieldCorrectionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_N4BiasFieldCorrectionImageFilter);
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter`, 'returnType') = '_p_itk__simple__N4BiasFieldCorrectionImageFilter'
class(`N4BiasFieldCorrectionImageFilter`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter'))

# Start of N4BiasFieldCorrectionImageFilter_SetConvergenceThreshold

`N4BiasFieldCorrectionImageFilter_SetConvergenceThreshold` = function(self, ConvergenceThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_N4BiasFieldCorrectionImageFilter_SetConvergenceThreshold', self, ConvergenceThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter_SetConvergenceThreshold`, 'returnType') = 'void'
attr(`N4BiasFieldCorrectionImageFilter_SetConvergenceThreshold`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter', 'numeric')
class(`N4BiasFieldCorrectionImageFilter_SetConvergenceThreshold`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_SetConvergenceThreshold'))

# Start of N4BiasFieldCorrectionImageFilter_GetConvergenceThreshold

`N4BiasFieldCorrectionImageFilter_GetConvergenceThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_N4BiasFieldCorrectionImageFilter_GetConvergenceThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`N4BiasFieldCorrectionImageFilter_GetConvergenceThreshold`, 'returnType') = 'numeric'
attr(`N4BiasFieldCorrectionImageFilter_GetConvergenceThreshold`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_GetConvergenceThreshold`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_GetConvergenceThreshold'))

# Start of N4BiasFieldCorrectionImageFilter_SetMaximumNumberOfIterations

`N4BiasFieldCorrectionImageFilter_SetMaximumNumberOfIterations` = function(self, MaximumNumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaximumNumberOfIterations = as.integer(MaximumNumberOfIterations);
  ;ans = .Call('R_swig_N4BiasFieldCorrectionImageFilter_SetMaximumNumberOfIterations', self, MaximumNumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter_SetMaximumNumberOfIterations`, 'returnType') = 'void'
attr(`N4BiasFieldCorrectionImageFilter_SetMaximumNumberOfIterations`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter', 'integer')
class(`N4BiasFieldCorrectionImageFilter_SetMaximumNumberOfIterations`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_SetMaximumNumberOfIterations'))

# Start of N4BiasFieldCorrectionImageFilter_GetMaximumNumberOfIterations

`N4BiasFieldCorrectionImageFilter_GetMaximumNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_N4BiasFieldCorrectionImageFilter_GetMaximumNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`N4BiasFieldCorrectionImageFilter_GetMaximumNumberOfIterations`, 'returnType') = 'integer'
attr(`N4BiasFieldCorrectionImageFilter_GetMaximumNumberOfIterations`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_GetMaximumNumberOfIterations`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_GetMaximumNumberOfIterations'))

# Start of N4BiasFieldCorrectionImageFilter_SetBiasFieldFullWidthAtHalfMaximum

`N4BiasFieldCorrectionImageFilter_SetBiasFieldFullWidthAtHalfMaximum` = function(self, BiasFieldFullWidthAtHalfMaximum)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_N4BiasFieldCorrectionImageFilter_SetBiasFieldFullWidthAtHalfMaximum', self, BiasFieldFullWidthAtHalfMaximum, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter_SetBiasFieldFullWidthAtHalfMaximum`, 'returnType') = 'void'
attr(`N4BiasFieldCorrectionImageFilter_SetBiasFieldFullWidthAtHalfMaximum`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter', 'numeric')
class(`N4BiasFieldCorrectionImageFilter_SetBiasFieldFullWidthAtHalfMaximum`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_SetBiasFieldFullWidthAtHalfMaximum'))

# Start of N4BiasFieldCorrectionImageFilter_GetBiasFieldFullWidthAtHalfMaximum

`N4BiasFieldCorrectionImageFilter_GetBiasFieldFullWidthAtHalfMaximum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_N4BiasFieldCorrectionImageFilter_GetBiasFieldFullWidthAtHalfMaximum', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`N4BiasFieldCorrectionImageFilter_GetBiasFieldFullWidthAtHalfMaximum`, 'returnType') = 'numeric'
attr(`N4BiasFieldCorrectionImageFilter_GetBiasFieldFullWidthAtHalfMaximum`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_GetBiasFieldFullWidthAtHalfMaximum`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_GetBiasFieldFullWidthAtHalfMaximum'))

# Start of N4BiasFieldCorrectionImageFilter_SetWienerFilterNoise

`N4BiasFieldCorrectionImageFilter_SetWienerFilterNoise` = function(self, WienerFilterNoise)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_N4BiasFieldCorrectionImageFilter_SetWienerFilterNoise', self, WienerFilterNoise, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter_SetWienerFilterNoise`, 'returnType') = 'void'
attr(`N4BiasFieldCorrectionImageFilter_SetWienerFilterNoise`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter', 'numeric')
class(`N4BiasFieldCorrectionImageFilter_SetWienerFilterNoise`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_SetWienerFilterNoise'))

# Start of N4BiasFieldCorrectionImageFilter_GetWienerFilterNoise

`N4BiasFieldCorrectionImageFilter_GetWienerFilterNoise` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_N4BiasFieldCorrectionImageFilter_GetWienerFilterNoise', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`N4BiasFieldCorrectionImageFilter_GetWienerFilterNoise`, 'returnType') = 'numeric'
attr(`N4BiasFieldCorrectionImageFilter_GetWienerFilterNoise`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_GetWienerFilterNoise`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_GetWienerFilterNoise'))

# Start of N4BiasFieldCorrectionImageFilter_SetNumberOfHistogramBins

`N4BiasFieldCorrectionImageFilter_SetNumberOfHistogramBins` = function(self, NumberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfHistogramBins = as.integer(NumberOfHistogramBins);
  
  if(length(NumberOfHistogramBins) > 1) {
    warning("using only the first element of NumberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_N4BiasFieldCorrectionImageFilter_SetNumberOfHistogramBins', self, NumberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter_SetNumberOfHistogramBins`, 'returnType') = 'void'
attr(`N4BiasFieldCorrectionImageFilter_SetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter', 'integer')
class(`N4BiasFieldCorrectionImageFilter_SetNumberOfHistogramBins`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_SetNumberOfHistogramBins'))

# Start of N4BiasFieldCorrectionImageFilter_GetNumberOfHistogramBins

`N4BiasFieldCorrectionImageFilter_GetNumberOfHistogramBins` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_N4BiasFieldCorrectionImageFilter_GetNumberOfHistogramBins', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`N4BiasFieldCorrectionImageFilter_GetNumberOfHistogramBins`, 'returnType') = 'integer'
attr(`N4BiasFieldCorrectionImageFilter_GetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_GetNumberOfHistogramBins`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_GetNumberOfHistogramBins'))

# Start of N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints

`N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints__SWIG_0` = function(self, NumberOfControlPoints)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfControlPoints = as.integer(NumberOfControlPoints);
  ;ans = .Call('R_swig_N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints__SWIG_0', self, NumberOfControlPoints, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints__SWIG_0`, 'returnType') = 'void'
attr(`N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints__SWIG_0`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter', 'integer')
class(`N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints__SWIG_0`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints__SWIG_0'))

# Start of N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints

`N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints__SWIG_1`, 'returnType') = 'void'
attr(`N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints__SWIG_1`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter', 'integer')
class(`N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints__SWIG_1`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints__SWIG_1'))

`N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__N4BiasFieldCorrectionImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__N4BiasFieldCorrectionImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of N4BiasFieldCorrectionImageFilter_GetNumberOfControlPoints

`N4BiasFieldCorrectionImageFilter_GetNumberOfControlPoints` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_N4BiasFieldCorrectionImageFilter_GetNumberOfControlPoints', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`N4BiasFieldCorrectionImageFilter_GetNumberOfControlPoints`, 'returnType') = 'integer'
attr(`N4BiasFieldCorrectionImageFilter_GetNumberOfControlPoints`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_GetNumberOfControlPoints`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_GetNumberOfControlPoints'))

# Start of N4BiasFieldCorrectionImageFilter_SetSplineOrder

`N4BiasFieldCorrectionImageFilter_SetSplineOrder` = function(self, SplineOrder)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SplineOrder = as.integer(SplineOrder);
  
  if(length(SplineOrder) > 1) {
    warning("using only the first element of SplineOrder");
  };
  
  ;ans = .Call('R_swig_N4BiasFieldCorrectionImageFilter_SetSplineOrder', self, SplineOrder, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter_SetSplineOrder`, 'returnType') = 'void'
attr(`N4BiasFieldCorrectionImageFilter_SetSplineOrder`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter', 'integer')
class(`N4BiasFieldCorrectionImageFilter_SetSplineOrder`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_SetSplineOrder'))

# Start of N4BiasFieldCorrectionImageFilter_GetSplineOrder

`N4BiasFieldCorrectionImageFilter_GetSplineOrder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_N4BiasFieldCorrectionImageFilter_GetSplineOrder', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`N4BiasFieldCorrectionImageFilter_GetSplineOrder`, 'returnType') = 'integer'
attr(`N4BiasFieldCorrectionImageFilter_GetSplineOrder`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_GetSplineOrder`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_GetSplineOrder'))

# Start of N4BiasFieldCorrectionImageFilter_SetUseMaskLabel

`N4BiasFieldCorrectionImageFilter_SetUseMaskLabel` = function(self, UseMaskLabel)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseMaskLabel = as.logical(UseMaskLabel);
  ;ans = .Call('R_swig_N4BiasFieldCorrectionImageFilter_SetUseMaskLabel', self, UseMaskLabel, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter_SetUseMaskLabel`, 'returnType') = 'void'
attr(`N4BiasFieldCorrectionImageFilter_SetUseMaskLabel`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter', 'logical')
class(`N4BiasFieldCorrectionImageFilter_SetUseMaskLabel`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_SetUseMaskLabel'))

# Start of N4BiasFieldCorrectionImageFilter_UseMaskLabelOn

`N4BiasFieldCorrectionImageFilter_UseMaskLabelOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_N4BiasFieldCorrectionImageFilter_UseMaskLabelOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter_UseMaskLabelOn`, 'returnType') = 'void'
attr(`N4BiasFieldCorrectionImageFilter_UseMaskLabelOn`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_UseMaskLabelOn`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_UseMaskLabelOn'))

# Start of N4BiasFieldCorrectionImageFilter_UseMaskLabelOff

`N4BiasFieldCorrectionImageFilter_UseMaskLabelOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_N4BiasFieldCorrectionImageFilter_UseMaskLabelOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter_UseMaskLabelOff`, 'returnType') = 'void'
attr(`N4BiasFieldCorrectionImageFilter_UseMaskLabelOff`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_UseMaskLabelOff`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_UseMaskLabelOff'))

# Start of N4BiasFieldCorrectionImageFilter_GetUseMaskLabel

`N4BiasFieldCorrectionImageFilter_GetUseMaskLabel` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_N4BiasFieldCorrectionImageFilter_GetUseMaskLabel', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`N4BiasFieldCorrectionImageFilter_GetUseMaskLabel`, 'returnType') = 'logical'
attr(`N4BiasFieldCorrectionImageFilter_GetUseMaskLabel`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_GetUseMaskLabel`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_GetUseMaskLabel'))

# Start of N4BiasFieldCorrectionImageFilter_SetMaskLabel

`N4BiasFieldCorrectionImageFilter_SetMaskLabel` = function(self, MaskLabel)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskLabel = as.integer(MaskLabel);
  
  if(length(MaskLabel) > 1) {
    warning("using only the first element of MaskLabel");
  };
  
  ;ans = .Call('R_swig_N4BiasFieldCorrectionImageFilter_SetMaskLabel', self, MaskLabel, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter_SetMaskLabel`, 'returnType') = 'void'
attr(`N4BiasFieldCorrectionImageFilter_SetMaskLabel`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter', 'integer')
class(`N4BiasFieldCorrectionImageFilter_SetMaskLabel`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_SetMaskLabel'))

# Start of N4BiasFieldCorrectionImageFilter_GetMaskLabel

`N4BiasFieldCorrectionImageFilter_GetMaskLabel` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_N4BiasFieldCorrectionImageFilter_GetMaskLabel', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`N4BiasFieldCorrectionImageFilter_GetMaskLabel`, 'returnType') = 'integer'
attr(`N4BiasFieldCorrectionImageFilter_GetMaskLabel`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_GetMaskLabel`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_GetMaskLabel'))

# Start of N4BiasFieldCorrectionImageFilter_GetCurrentLevel

`N4BiasFieldCorrectionImageFilter_GetCurrentLevel` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_N4BiasFieldCorrectionImageFilter_GetCurrentLevel', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`N4BiasFieldCorrectionImageFilter_GetCurrentLevel`, 'returnType') = 'integer'
attr(`N4BiasFieldCorrectionImageFilter_GetCurrentLevel`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_GetCurrentLevel`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_GetCurrentLevel'))

# Start of N4BiasFieldCorrectionImageFilter_GetElapsedIterations

`N4BiasFieldCorrectionImageFilter_GetElapsedIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_N4BiasFieldCorrectionImageFilter_GetElapsedIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`N4BiasFieldCorrectionImageFilter_GetElapsedIterations`, 'returnType') = 'integer'
attr(`N4BiasFieldCorrectionImageFilter_GetElapsedIterations`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_GetElapsedIterations`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_GetElapsedIterations'))

# Start of N4BiasFieldCorrectionImageFilter_GetCurrentConvergenceMeasurement

`N4BiasFieldCorrectionImageFilter_GetCurrentConvergenceMeasurement` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_N4BiasFieldCorrectionImageFilter_GetCurrentConvergenceMeasurement', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`N4BiasFieldCorrectionImageFilter_GetCurrentConvergenceMeasurement`, 'returnType') = 'numeric'
attr(`N4BiasFieldCorrectionImageFilter_GetCurrentConvergenceMeasurement`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_GetCurrentConvergenceMeasurement`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_GetCurrentConvergenceMeasurement'))

# Start of N4BiasFieldCorrectionImageFilter_GetLogBiasFieldAsImage

`N4BiasFieldCorrectionImageFilter_GetLogBiasFieldAsImage` = function(self, referenceImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(referenceImage, "ExternalReference")) referenceImage = slot(referenceImage,"ref"); 
  ;ans = .Call('R_swig_N4BiasFieldCorrectionImageFilter_GetLogBiasFieldAsImage', self, referenceImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter_GetLogBiasFieldAsImage`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrectionImageFilter_GetLogBiasFieldAsImage`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter', '_p_itk__simple__Image')
class(`N4BiasFieldCorrectionImageFilter_GetLogBiasFieldAsImage`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_GetLogBiasFieldAsImage'))

# Start of N4BiasFieldCorrectionImageFilter_GetName

`N4BiasFieldCorrectionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_N4BiasFieldCorrectionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`N4BiasFieldCorrectionImageFilter_GetName`, 'returnType') = 'character'
attr(`N4BiasFieldCorrectionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_GetName`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_GetName'))

# Start of N4BiasFieldCorrectionImageFilter_ToString

`N4BiasFieldCorrectionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_N4BiasFieldCorrectionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`N4BiasFieldCorrectionImageFilter_ToString`, 'returnType') = 'character'
attr(`N4BiasFieldCorrectionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter')
class(`N4BiasFieldCorrectionImageFilter_ToString`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_ToString'))

# Start of N4BiasFieldCorrectionImageFilter_Execute

`N4BiasFieldCorrectionImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_N4BiasFieldCorrectionImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrectionImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`N4BiasFieldCorrectionImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_Execute__SWIG_0'))

# Start of N4BiasFieldCorrectionImageFilter_Execute

`N4BiasFieldCorrectionImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_N4BiasFieldCorrectionImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrectionImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrectionImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__N4BiasFieldCorrectionImageFilter', '_p_itk__simple__Image')
class(`N4BiasFieldCorrectionImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('N4BiasFieldCorrectionImageFilter_Execute__SWIG_1'))

`N4BiasFieldCorrectionImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__N4BiasFieldCorrectionImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- N4BiasFieldCorrectionImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__N4BiasFieldCorrectionImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- N4BiasFieldCorrectionImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for N4BiasFieldCorrectionImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::N4BiasFieldCorrectionImageFilter
setMethod('$', '_p_itk__simple__N4BiasFieldCorrectionImageFilter', function(x, name)

{
  accessorFuns = list('SetConvergenceThreshold' = N4BiasFieldCorrectionImageFilter_SetConvergenceThreshold, 'GetConvergenceThreshold' = N4BiasFieldCorrectionImageFilter_GetConvergenceThreshold, 'SetMaximumNumberOfIterations' = N4BiasFieldCorrectionImageFilter_SetMaximumNumberOfIterations, 'GetMaximumNumberOfIterations' = N4BiasFieldCorrectionImageFilter_GetMaximumNumberOfIterations, 'SetBiasFieldFullWidthAtHalfMaximum' = N4BiasFieldCorrectionImageFilter_SetBiasFieldFullWidthAtHalfMaximum, 'GetBiasFieldFullWidthAtHalfMaximum' = N4BiasFieldCorrectionImageFilter_GetBiasFieldFullWidthAtHalfMaximum, 'SetWienerFilterNoise' = N4BiasFieldCorrectionImageFilter_SetWienerFilterNoise, 'GetWienerFilterNoise' = N4BiasFieldCorrectionImageFilter_GetWienerFilterNoise, 'SetNumberOfHistogramBins' = N4BiasFieldCorrectionImageFilter_SetNumberOfHistogramBins, 'GetNumberOfHistogramBins' = N4BiasFieldCorrectionImageFilter_GetNumberOfHistogramBins, 'SetNumberOfControlPoints' = N4BiasFieldCorrectionImageFilter_SetNumberOfControlPoints, 'GetNumberOfControlPoints' = N4BiasFieldCorrectionImageFilter_GetNumberOfControlPoints, 'SetSplineOrder' = N4BiasFieldCorrectionImageFilter_SetSplineOrder, 'GetSplineOrder' = N4BiasFieldCorrectionImageFilter_GetSplineOrder, 'SetUseMaskLabel' = N4BiasFieldCorrectionImageFilter_SetUseMaskLabel, 'UseMaskLabelOn' = N4BiasFieldCorrectionImageFilter_UseMaskLabelOn, 'UseMaskLabelOff' = N4BiasFieldCorrectionImageFilter_UseMaskLabelOff, 'GetUseMaskLabel' = N4BiasFieldCorrectionImageFilter_GetUseMaskLabel, 'SetMaskLabel' = N4BiasFieldCorrectionImageFilter_SetMaskLabel, 'GetMaskLabel' = N4BiasFieldCorrectionImageFilter_GetMaskLabel, 'GetCurrentLevel' = N4BiasFieldCorrectionImageFilter_GetCurrentLevel, 'GetElapsedIterations' = N4BiasFieldCorrectionImageFilter_GetElapsedIterations, 'GetCurrentConvergenceMeasurement' = N4BiasFieldCorrectionImageFilter_GetCurrentConvergenceMeasurement, 'GetLogBiasFieldAsImage' = N4BiasFieldCorrectionImageFilter_GetLogBiasFieldAsImage, 'GetName' = N4BiasFieldCorrectionImageFilter_GetName, 'ToString' = N4BiasFieldCorrectionImageFilter_ToString, 'Execute' = N4BiasFieldCorrectionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::N4BiasFieldCorrectionImageFilter
# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_0` = function(image, maskImage, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, splineOrder, useMaskLabel, maskLabel, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  numberOfControlPoints = as.integer(numberOfControlPoints);
  splineOrder = as.integer(splineOrder);
  
  if(length(splineOrder) > 1) {
    warning("using only the first element of splineOrder");
  };
  
  useMaskLabel = as.logical(useMaskLabel);
  maskLabel = as.integer(maskLabel);
  
  if(length(maskLabel) > 1) {
    warning("using only the first element of maskLabel");
  };
  
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_0', image, maskImage, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, splineOrder, useMaskLabel, maskLabel, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`N4BiasFieldCorrection__SWIG_0`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_0'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_1` = function(image, maskImage, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, splineOrder, useMaskLabel, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  numberOfControlPoints = as.integer(numberOfControlPoints);
  splineOrder = as.integer(splineOrder);
  
  if(length(splineOrder) > 1) {
    warning("using only the first element of splineOrder");
  };
  
  useMaskLabel = as.logical(useMaskLabel);
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_1', image, maskImage, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, splineOrder, useMaskLabel, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'integer', 'integer', 'integer', 'logical')
class(`N4BiasFieldCorrection__SWIG_1`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_1'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_2` = function(image, maskImage, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, splineOrder, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  numberOfControlPoints = as.integer(numberOfControlPoints);
  splineOrder = as.integer(splineOrder);
  
  if(length(splineOrder) > 1) {
    warning("using only the first element of splineOrder");
  };
  
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_2', image, maskImage, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, splineOrder, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'integer', 'integer', 'integer')
class(`N4BiasFieldCorrection__SWIG_2`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_2'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_3` = function(image, maskImage, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  numberOfControlPoints = as.integer(numberOfControlPoints);
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_3', image, maskImage, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'integer', 'integer')
class(`N4BiasFieldCorrection__SWIG_3`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_3'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_4` = function(image, maskImage, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_4', image, maskImage, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'integer')
class(`N4BiasFieldCorrection__SWIG_4`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_4'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_5` = function(image, maskImage, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_5', image, maskImage, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric')
class(`N4BiasFieldCorrection__SWIG_5`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_5'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_6` = function(image, maskImage, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_6', image, maskImage, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric')
class(`N4BiasFieldCorrection__SWIG_6`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_6'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_7` = function(image, maskImage, convergenceThreshold, maximumNumberOfIterations, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_7', image, maskImage, convergenceThreshold, maximumNumberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer')
class(`N4BiasFieldCorrection__SWIG_7`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_7'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_8` = function(image, maskImage, convergenceThreshold, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_8', image, maskImage, convergenceThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`N4BiasFieldCorrection__SWIG_8`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_8'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_9` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_9', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`N4BiasFieldCorrection__SWIG_9`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_9'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_10` = function(image, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, splineOrder, useMaskLabel, maskLabel, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  numberOfControlPoints = as.integer(numberOfControlPoints);
  splineOrder = as.integer(splineOrder);
  
  if(length(splineOrder) > 1) {
    warning("using only the first element of splineOrder");
  };
  
  useMaskLabel = as.logical(useMaskLabel);
  maskLabel = as.integer(maskLabel);
  
  if(length(maskLabel) > 1) {
    warning("using only the first element of maskLabel");
  };
  
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_10', image, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, splineOrder, useMaskLabel, maskLabel, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`N4BiasFieldCorrection__SWIG_10`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_10'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_11` = function(image, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, splineOrder, useMaskLabel, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  numberOfControlPoints = as.integer(numberOfControlPoints);
  splineOrder = as.integer(splineOrder);
  
  if(length(splineOrder) > 1) {
    warning("using only the first element of splineOrder");
  };
  
  useMaskLabel = as.logical(useMaskLabel);
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_11', image, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, splineOrder, useMaskLabel, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'integer', 'integer', 'integer', 'logical')
class(`N4BiasFieldCorrection__SWIG_11`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_11'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_12` = function(image, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, splineOrder, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  numberOfControlPoints = as.integer(numberOfControlPoints);
  splineOrder = as.integer(splineOrder);
  
  if(length(splineOrder) > 1) {
    warning("using only the first element of splineOrder");
  };
  
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_12', image, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, splineOrder, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_12`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_12`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'integer', 'integer', 'integer')
class(`N4BiasFieldCorrection__SWIG_12`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_12'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_13` = function(image, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  numberOfControlPoints = as.integer(numberOfControlPoints);
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_13', image, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, numberOfControlPoints, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_13`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_13`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'integer', 'integer')
class(`N4BiasFieldCorrection__SWIG_13`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_13'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_14` = function(image, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_14', image, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_14`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_14`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'integer')
class(`N4BiasFieldCorrection__SWIG_14`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_14'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_15` = function(image, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_15', image, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, wienerFilterNoise, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_15`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_15`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric')
class(`N4BiasFieldCorrection__SWIG_15`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_15'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_16` = function(image, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_16', image, convergenceThreshold, maximumNumberOfIterations, biasFieldFullWidthAtHalfMaximum, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_16`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_16`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'numeric')
class(`N4BiasFieldCorrection__SWIG_16`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_16'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_17` = function(image, convergenceThreshold, maximumNumberOfIterations, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_17', image, convergenceThreshold, maximumNumberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_17`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_17`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`N4BiasFieldCorrection__SWIG_17`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_17'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_18` = function(image, convergenceThreshold, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_18', image, convergenceThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_18`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_18`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`N4BiasFieldCorrection__SWIG_18`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_18'))

# Start of N4BiasFieldCorrection

`N4BiasFieldCorrection__SWIG_19` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_N4BiasFieldCorrection__SWIG_19', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`N4BiasFieldCorrection__SWIG_19`, 'returnType') = '_p_itk__simple__Image'
attr(`N4BiasFieldCorrection__SWIG_19`, "inputTypes") = c('_p_itk__simple__Image')
class(`N4BiasFieldCorrection__SWIG_19`) = c("SWIGFunction", class('N4BiasFieldCorrection__SWIG_19'))

`N4BiasFieldCorrection` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 20
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- N4BiasFieldCorrection__SWIG_19; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- N4BiasFieldCorrection__SWIG_9; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- N4BiasFieldCorrection__SWIG_18; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- N4BiasFieldCorrection__SWIG_8; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- N4BiasFieldCorrection__SWIG_17; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) )) {
      f <- N4BiasFieldCorrection__SWIG_7; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- N4BiasFieldCorrection__SWIG_16; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- N4BiasFieldCorrection__SWIG_6; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- N4BiasFieldCorrection__SWIG_15; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- N4BiasFieldCorrection__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- N4BiasFieldCorrection__SWIG_14; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- N4BiasFieldCorrection__SWIG_4; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.integer(argv[[7]]) || is.numeric(argv[[7]]) )) {
      f <- N4BiasFieldCorrection__SWIG_13; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 ) && ( is.integer(argv[[8]]) || is.numeric(argv[[8]]) )) {
      f <- N4BiasFieldCorrection__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.integer(argv[[7]]) || is.numeric(argv[[7]]) ) && ( (is.integer(argv[[8]]) || is.numeric(argv[[8]])) && length(argv[[8]]) == 1 )) {
      f <- N4BiasFieldCorrection__SWIG_12; 
    }
  } else if (argc == 9) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 ) && ( is.integer(argv[[8]]) || is.numeric(argv[[8]]) ) && ( (is.integer(argv[[9]]) || is.numeric(argv[[9]])) && length(argv[[9]]) == 1 )) {
      f <- N4BiasFieldCorrection__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.integer(argv[[7]]) || is.numeric(argv[[7]]) ) && ( (is.integer(argv[[8]]) || is.numeric(argv[[8]])) && length(argv[[8]]) == 1 ) && ( is.logical(argv[[9]]) && length(argv[[9]]) == 1 )) {
      f <- N4BiasFieldCorrection__SWIG_11; 
    }
  } else if (argc == 10) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 ) && ( is.integer(argv[[8]]) || is.numeric(argv[[8]]) ) && ( (is.integer(argv[[9]]) || is.numeric(argv[[9]])) && length(argv[[9]]) == 1 ) && ( is.logical(argv[[10]]) && length(argv[[10]]) == 1 )) {
      f <- N4BiasFieldCorrection__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.integer(argv[[7]]) || is.numeric(argv[[7]]) ) && ( (is.integer(argv[[8]]) || is.numeric(argv[[8]])) && length(argv[[8]]) == 1 ) && ( is.logical(argv[[9]]) && length(argv[[9]]) == 1 ) && ( (is.integer(argv[[10]]) || is.numeric(argv[[10]])) && length(argv[[10]]) == 1 )) {
      f <- N4BiasFieldCorrection__SWIG_10; 
    }
  } else if (argc == 11) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 ) && ( is.integer(argv[[8]]) || is.numeric(argv[[8]]) ) && ( (is.integer(argv[[9]]) || is.numeric(argv[[9]])) && length(argv[[9]]) == 1 ) && ( is.logical(argv[[10]]) && length(argv[[10]]) == 1 ) && ( (is.integer(argv[[11]]) || is.numeric(argv[[11]])) && length(argv[[11]]) == 1 )) {
      f <- N4BiasFieldCorrection__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for N4BiasFieldCorrection with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_NaryAddImageFilter

`delete_NaryAddImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_NaryAddImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_NaryAddImageFilter`, 'returnType') = 'void'
attr(`delete_NaryAddImageFilter`, "inputTypes") = c('_p_itk__simple__NaryAddImageFilter')
class(`delete_NaryAddImageFilter`) = c("SWIGFunction", class('delete_NaryAddImageFilter'))

# Start of new_NaryAddImageFilter

`NaryAddImageFilter` = function()
{
  ;ans = .Call('R_swig_new_NaryAddImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__NaryAddImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_NaryAddImageFilter);
  ans
  
}

attr(`NaryAddImageFilter`, 'returnType') = '_p_itk__simple__NaryAddImageFilter'
class(`NaryAddImageFilter`) = c("SWIGFunction", class('NaryAddImageFilter'))

# Start of NaryAddImageFilter_GetName

`NaryAddImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NaryAddImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NaryAddImageFilter_GetName`, 'returnType') = 'character'
attr(`NaryAddImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__NaryAddImageFilter')
class(`NaryAddImageFilter_GetName`) = c("SWIGFunction", class('NaryAddImageFilter_GetName'))

# Start of NaryAddImageFilter_ToString

`NaryAddImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NaryAddImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NaryAddImageFilter_ToString`, 'returnType') = 'character'
attr(`NaryAddImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__NaryAddImageFilter')
class(`NaryAddImageFilter_ToString`) = c("SWIGFunction", class('NaryAddImageFilter_ToString'))

# Start of NaryAddImageFilter_Execute

`NaryAddImageFilter_Execute__SWIG_0` = function(self, images, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_NaryAddImageFilter_Execute__SWIG_0', self, images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryAddImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryAddImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__NaryAddImageFilter', 'list')
class(`NaryAddImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('NaryAddImageFilter_Execute__SWIG_0'))

# Start of NaryAddImageFilter_Execute

`NaryAddImageFilter_Execute__SWIG_1` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_NaryAddImageFilter_Execute__SWIG_1', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryAddImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryAddImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__NaryAddImageFilter', '_p_itk__simple__Image')
class(`NaryAddImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('NaryAddImageFilter_Execute__SWIG_1'))

# Start of NaryAddImageFilter_Execute

`NaryAddImageFilter_Execute__SWIG_2` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_NaryAddImageFilter_Execute__SWIG_2', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryAddImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryAddImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__NaryAddImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryAddImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('NaryAddImageFilter_Execute__SWIG_2'))

# Start of NaryAddImageFilter_Execute

`NaryAddImageFilter_Execute__SWIG_3` = function(self, image1, image2, image3, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_NaryAddImageFilter_Execute__SWIG_3', self, image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryAddImageFilter_Execute__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryAddImageFilter_Execute__SWIG_3`, "inputTypes") = c('_p_itk__simple__NaryAddImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryAddImageFilter_Execute__SWIG_3`) = c("SWIGFunction", class('NaryAddImageFilter_Execute__SWIG_3'))

# Start of NaryAddImageFilter_Execute

`NaryAddImageFilter_Execute__SWIG_4` = function(self, image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_NaryAddImageFilter_Execute__SWIG_4', self, image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryAddImageFilter_Execute__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryAddImageFilter_Execute__SWIG_4`, "inputTypes") = c('_p_itk__simple__NaryAddImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryAddImageFilter_Execute__SWIG_4`) = c("SWIGFunction", class('NaryAddImageFilter_Execute__SWIG_4'))

# Start of NaryAddImageFilter_Execute

`NaryAddImageFilter_Execute__SWIG_5` = function(self, image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_NaryAddImageFilter_Execute__SWIG_5', self, image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryAddImageFilter_Execute__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryAddImageFilter_Execute__SWIG_5`, "inputTypes") = c('_p_itk__simple__NaryAddImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryAddImageFilter_Execute__SWIG_5`) = c("SWIGFunction", class('NaryAddImageFilter_Execute__SWIG_5'))

`NaryAddImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__NaryAddImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- NaryAddImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__NaryAddImageFilter') || is.null(argv[[1]]) ) && (
        (is.list(argv[[2]]) & all(sapply(argv[[2]], inherits, what='_p_itk__simple__Image')))
        )) {
      f <- NaryAddImageFilter_Execute__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__NaryAddImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- NaryAddImageFilter_Execute__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__NaryAddImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- NaryAddImageFilter_Execute__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__NaryAddImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- NaryAddImageFilter_Execute__SWIG_4; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__NaryAddImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( extends(argtypes[6], '_p_itk__simple__Image') && length(argv[[6]]) == 1 )) {
      f <- NaryAddImageFilter_Execute__SWIG_5; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for NaryAddImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::NaryAddImageFilter
setMethod('$', '_p_itk__simple__NaryAddImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = NaryAddImageFilter_GetName, 'ToString' = NaryAddImageFilter_ToString, 'Execute' = NaryAddImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::NaryAddImageFilter
# Start of NaryAdd

`NaryAdd__SWIG_0` = function(images, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_NaryAdd__SWIG_0', images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryAdd__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryAdd__SWIG_0`, "inputTypes") = c('list')
class(`NaryAdd__SWIG_0`) = c("SWIGFunction", class('NaryAdd__SWIG_0'))

# Start of NaryAdd

`NaryAdd__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_NaryAdd__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryAdd__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryAdd__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`NaryAdd__SWIG_1`) = c("SWIGFunction", class('NaryAdd__SWIG_1'))

# Start of NaryAdd

`NaryAdd__SWIG_2` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_NaryAdd__SWIG_2', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryAdd__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryAdd__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryAdd__SWIG_2`) = c("SWIGFunction", class('NaryAdd__SWIG_2'))

# Start of NaryAdd

`NaryAdd__SWIG_3` = function(image1, image2, image3, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_NaryAdd__SWIG_3', image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryAdd__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryAdd__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryAdd__SWIG_3`) = c("SWIGFunction", class('NaryAdd__SWIG_3'))

# Start of NaryAdd

`NaryAdd__SWIG_4` = function(image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_NaryAdd__SWIG_4', image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryAdd__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryAdd__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryAdd__SWIG_4`) = c("SWIGFunction", class('NaryAdd__SWIG_4'))

# Start of NaryAdd

`NaryAdd__SWIG_5` = function(image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_NaryAdd__SWIG_5', image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryAdd__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryAdd__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryAdd__SWIG_5`) = c("SWIGFunction", class('NaryAdd__SWIG_5'))

`NaryAdd` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- NaryAdd__SWIG_1; 
    }
    else if (
      (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
      ) {
      f <- NaryAdd__SWIG_0; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- NaryAdd__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- NaryAdd__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- NaryAdd__SWIG_4; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- NaryAdd__SWIG_5; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for NaryAdd with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_NaryMaximumImageFilter

`delete_NaryMaximumImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_NaryMaximumImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_NaryMaximumImageFilter`, 'returnType') = 'void'
attr(`delete_NaryMaximumImageFilter`, "inputTypes") = c('_p_itk__simple__NaryMaximumImageFilter')
class(`delete_NaryMaximumImageFilter`) = c("SWIGFunction", class('delete_NaryMaximumImageFilter'))

# Start of new_NaryMaximumImageFilter

`NaryMaximumImageFilter` = function()
{
  ;ans = .Call('R_swig_new_NaryMaximumImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__NaryMaximumImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_NaryMaximumImageFilter);
  ans
  
}

attr(`NaryMaximumImageFilter`, 'returnType') = '_p_itk__simple__NaryMaximumImageFilter'
class(`NaryMaximumImageFilter`) = c("SWIGFunction", class('NaryMaximumImageFilter'))

# Start of NaryMaximumImageFilter_GetName

`NaryMaximumImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NaryMaximumImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NaryMaximumImageFilter_GetName`, 'returnType') = 'character'
attr(`NaryMaximumImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__NaryMaximumImageFilter')
class(`NaryMaximumImageFilter_GetName`) = c("SWIGFunction", class('NaryMaximumImageFilter_GetName'))

# Start of NaryMaximumImageFilter_ToString

`NaryMaximumImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NaryMaximumImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NaryMaximumImageFilter_ToString`, 'returnType') = 'character'
attr(`NaryMaximumImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__NaryMaximumImageFilter')
class(`NaryMaximumImageFilter_ToString`) = c("SWIGFunction", class('NaryMaximumImageFilter_ToString'))

# Start of NaryMaximumImageFilter_Execute

`NaryMaximumImageFilter_Execute__SWIG_0` = function(self, images, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_NaryMaximumImageFilter_Execute__SWIG_0', self, images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryMaximumImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryMaximumImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__NaryMaximumImageFilter', 'list')
class(`NaryMaximumImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('NaryMaximumImageFilter_Execute__SWIG_0'))

# Start of NaryMaximumImageFilter_Execute

`NaryMaximumImageFilter_Execute__SWIG_1` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_NaryMaximumImageFilter_Execute__SWIG_1', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryMaximumImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryMaximumImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__NaryMaximumImageFilter', '_p_itk__simple__Image')
class(`NaryMaximumImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('NaryMaximumImageFilter_Execute__SWIG_1'))

# Start of NaryMaximumImageFilter_Execute

`NaryMaximumImageFilter_Execute__SWIG_2` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_NaryMaximumImageFilter_Execute__SWIG_2', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryMaximumImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryMaximumImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__NaryMaximumImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryMaximumImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('NaryMaximumImageFilter_Execute__SWIG_2'))

# Start of NaryMaximumImageFilter_Execute

`NaryMaximumImageFilter_Execute__SWIG_3` = function(self, image1, image2, image3, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_NaryMaximumImageFilter_Execute__SWIG_3', self, image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryMaximumImageFilter_Execute__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryMaximumImageFilter_Execute__SWIG_3`, "inputTypes") = c('_p_itk__simple__NaryMaximumImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryMaximumImageFilter_Execute__SWIG_3`) = c("SWIGFunction", class('NaryMaximumImageFilter_Execute__SWIG_3'))

# Start of NaryMaximumImageFilter_Execute

`NaryMaximumImageFilter_Execute__SWIG_4` = function(self, image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_NaryMaximumImageFilter_Execute__SWIG_4', self, image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryMaximumImageFilter_Execute__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryMaximumImageFilter_Execute__SWIG_4`, "inputTypes") = c('_p_itk__simple__NaryMaximumImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryMaximumImageFilter_Execute__SWIG_4`) = c("SWIGFunction", class('NaryMaximumImageFilter_Execute__SWIG_4'))

# Start of NaryMaximumImageFilter_Execute

`NaryMaximumImageFilter_Execute__SWIG_5` = function(self, image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_NaryMaximumImageFilter_Execute__SWIG_5', self, image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryMaximumImageFilter_Execute__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryMaximumImageFilter_Execute__SWIG_5`, "inputTypes") = c('_p_itk__simple__NaryMaximumImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryMaximumImageFilter_Execute__SWIG_5`) = c("SWIGFunction", class('NaryMaximumImageFilter_Execute__SWIG_5'))

`NaryMaximumImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__NaryMaximumImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- NaryMaximumImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__NaryMaximumImageFilter') || is.null(argv[[1]]) ) && (
        (is.list(argv[[2]]) & all(sapply(argv[[2]], inherits, what='_p_itk__simple__Image')))
        )) {
      f <- NaryMaximumImageFilter_Execute__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__NaryMaximumImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- NaryMaximumImageFilter_Execute__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__NaryMaximumImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- NaryMaximumImageFilter_Execute__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__NaryMaximumImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- NaryMaximumImageFilter_Execute__SWIG_4; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__NaryMaximumImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( extends(argtypes[6], '_p_itk__simple__Image') && length(argv[[6]]) == 1 )) {
      f <- NaryMaximumImageFilter_Execute__SWIG_5; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for NaryMaximumImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::NaryMaximumImageFilter
setMethod('$', '_p_itk__simple__NaryMaximumImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = NaryMaximumImageFilter_GetName, 'ToString' = NaryMaximumImageFilter_ToString, 'Execute' = NaryMaximumImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::NaryMaximumImageFilter
# Start of NaryMaximum

`NaryMaximum__SWIG_0` = function(images, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_NaryMaximum__SWIG_0', images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryMaximum__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryMaximum__SWIG_0`, "inputTypes") = c('list')
class(`NaryMaximum__SWIG_0`) = c("SWIGFunction", class('NaryMaximum__SWIG_0'))

# Start of NaryMaximum

`NaryMaximum__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_NaryMaximum__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryMaximum__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryMaximum__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`NaryMaximum__SWIG_1`) = c("SWIGFunction", class('NaryMaximum__SWIG_1'))

# Start of NaryMaximum

`NaryMaximum__SWIG_2` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_NaryMaximum__SWIG_2', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryMaximum__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryMaximum__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryMaximum__SWIG_2`) = c("SWIGFunction", class('NaryMaximum__SWIG_2'))

# Start of NaryMaximum

`NaryMaximum__SWIG_3` = function(image1, image2, image3, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_NaryMaximum__SWIG_3', image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryMaximum__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryMaximum__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryMaximum__SWIG_3`) = c("SWIGFunction", class('NaryMaximum__SWIG_3'))

# Start of NaryMaximum

`NaryMaximum__SWIG_4` = function(image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_NaryMaximum__SWIG_4', image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryMaximum__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryMaximum__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryMaximum__SWIG_4`) = c("SWIGFunction", class('NaryMaximum__SWIG_4'))

# Start of NaryMaximum

`NaryMaximum__SWIG_5` = function(image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_NaryMaximum__SWIG_5', image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NaryMaximum__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`NaryMaximum__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NaryMaximum__SWIG_5`) = c("SWIGFunction", class('NaryMaximum__SWIG_5'))

`NaryMaximum` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- NaryMaximum__SWIG_1; 
    }
    else if (
      (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
      ) {
      f <- NaryMaximum__SWIG_0; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- NaryMaximum__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- NaryMaximum__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- NaryMaximum__SWIG_4; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- NaryMaximum__SWIG_5; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for NaryMaximum with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_NeighborhoodConnectedImageFilter

`delete_NeighborhoodConnectedImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_NeighborhoodConnectedImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_NeighborhoodConnectedImageFilter`, 'returnType') = 'void'
attr(`delete_NeighborhoodConnectedImageFilter`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter')
class(`delete_NeighborhoodConnectedImageFilter`) = c("SWIGFunction", class('delete_NeighborhoodConnectedImageFilter'))

# Start of new_NeighborhoodConnectedImageFilter

`NeighborhoodConnectedImageFilter` = function()
{
  ;ans = .Call('R_swig_new_NeighborhoodConnectedImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__NeighborhoodConnectedImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_NeighborhoodConnectedImageFilter);
  ans
  
}

attr(`NeighborhoodConnectedImageFilter`, 'returnType') = '_p_itk__simple__NeighborhoodConnectedImageFilter'
class(`NeighborhoodConnectedImageFilter`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter'))

# Start of NeighborhoodConnectedImageFilter_SetSeedList

`NeighborhoodConnectedImageFilter_SetSeedList` = function(self, SeedList)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SeedList = lapply(SeedList, as.integer);
  ;ans = .Call('R_swig_NeighborhoodConnectedImageFilter_SetSeedList', self, SeedList, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`NeighborhoodConnectedImageFilter_SetSeedList`, 'returnType') = 'void'
attr(`NeighborhoodConnectedImageFilter_SetSeedList`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter', 'list')
class(`NeighborhoodConnectedImageFilter_SetSeedList`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_SetSeedList'))

# Start of NeighborhoodConnectedImageFilter_GetSeedList

`NeighborhoodConnectedImageFilter_GetSeedList` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NeighborhoodConnectedImageFilter_GetSeedList', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NeighborhoodConnectedImageFilter_GetSeedList`, 'returnType') = 'list'
attr(`NeighborhoodConnectedImageFilter_GetSeedList`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter')
class(`NeighborhoodConnectedImageFilter_GetSeedList`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_GetSeedList'))

# Start of NeighborhoodConnectedImageFilter_AddSeed

`NeighborhoodConnectedImageFilter_AddSeed` = function(self, point)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  point = as.integer(point);
  ;ans = .Call('R_swig_NeighborhoodConnectedImageFilter_AddSeed', self, point, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`NeighborhoodConnectedImageFilter_AddSeed`, 'returnType') = 'void'
attr(`NeighborhoodConnectedImageFilter_AddSeed`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter', 'integer')
class(`NeighborhoodConnectedImageFilter_AddSeed`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_AddSeed'))

# Start of NeighborhoodConnectedImageFilter_ClearSeeds

`NeighborhoodConnectedImageFilter_ClearSeeds` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_NeighborhoodConnectedImageFilter_ClearSeeds', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`NeighborhoodConnectedImageFilter_ClearSeeds`, 'returnType') = 'void'
attr(`NeighborhoodConnectedImageFilter_ClearSeeds`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter')
class(`NeighborhoodConnectedImageFilter_ClearSeeds`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_ClearSeeds'))

# Start of NeighborhoodConnectedImageFilter_SetLower

`NeighborhoodConnectedImageFilter_SetLower` = function(self, Lower)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_NeighborhoodConnectedImageFilter_SetLower', self, Lower, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`NeighborhoodConnectedImageFilter_SetLower`, 'returnType') = 'void'
attr(`NeighborhoodConnectedImageFilter_SetLower`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter', 'numeric')
class(`NeighborhoodConnectedImageFilter_SetLower`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_SetLower'))

# Start of NeighborhoodConnectedImageFilter_GetLower

`NeighborhoodConnectedImageFilter_GetLower` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NeighborhoodConnectedImageFilter_GetLower', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NeighborhoodConnectedImageFilter_GetLower`, 'returnType') = 'numeric'
attr(`NeighborhoodConnectedImageFilter_GetLower`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter')
class(`NeighborhoodConnectedImageFilter_GetLower`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_GetLower'))

# Start of NeighborhoodConnectedImageFilter_SetUpper

`NeighborhoodConnectedImageFilter_SetUpper` = function(self, Upper)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_NeighborhoodConnectedImageFilter_SetUpper', self, Upper, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`NeighborhoodConnectedImageFilter_SetUpper`, 'returnType') = 'void'
attr(`NeighborhoodConnectedImageFilter_SetUpper`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter', 'numeric')
class(`NeighborhoodConnectedImageFilter_SetUpper`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_SetUpper'))

# Start of NeighborhoodConnectedImageFilter_GetUpper

`NeighborhoodConnectedImageFilter_GetUpper` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NeighborhoodConnectedImageFilter_GetUpper', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NeighborhoodConnectedImageFilter_GetUpper`, 'returnType') = 'numeric'
attr(`NeighborhoodConnectedImageFilter_GetUpper`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter')
class(`NeighborhoodConnectedImageFilter_GetUpper`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_GetUpper'))

# Start of NeighborhoodConnectedImageFilter_SetRadius

`NeighborhoodConnectedImageFilter_SetRadius__SWIG_0` = function(self, Radius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Radius = as.integer(Radius);
  ;ans = .Call('R_swig_NeighborhoodConnectedImageFilter_SetRadius__SWIG_0', self, Radius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`NeighborhoodConnectedImageFilter_SetRadius__SWIG_0`, 'returnType') = 'void'
attr(`NeighborhoodConnectedImageFilter_SetRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter', 'integer')
class(`NeighborhoodConnectedImageFilter_SetRadius__SWIG_0`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_SetRadius__SWIG_0'))

# Start of NeighborhoodConnectedImageFilter_SetRadius

`NeighborhoodConnectedImageFilter_SetRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_NeighborhoodConnectedImageFilter_SetRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`NeighborhoodConnectedImageFilter_SetRadius__SWIG_1`, 'returnType') = 'void'
attr(`NeighborhoodConnectedImageFilter_SetRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter', 'integer')
class(`NeighborhoodConnectedImageFilter_SetRadius__SWIG_1`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_SetRadius__SWIG_1'))

`NeighborhoodConnectedImageFilter_SetRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__NeighborhoodConnectedImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- NeighborhoodConnectedImageFilter_SetRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__NeighborhoodConnectedImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- NeighborhoodConnectedImageFilter_SetRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for NeighborhoodConnectedImageFilter_SetRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of NeighborhoodConnectedImageFilter_GetRadius

`NeighborhoodConnectedImageFilter_GetRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NeighborhoodConnectedImageFilter_GetRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NeighborhoodConnectedImageFilter_GetRadius`, 'returnType') = 'integer'
attr(`NeighborhoodConnectedImageFilter_GetRadius`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter')
class(`NeighborhoodConnectedImageFilter_GetRadius`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_GetRadius'))

# Start of NeighborhoodConnectedImageFilter_SetReplaceValue

`NeighborhoodConnectedImageFilter_SetReplaceValue` = function(self, ReplaceValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_NeighborhoodConnectedImageFilter_SetReplaceValue', self, ReplaceValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`NeighborhoodConnectedImageFilter_SetReplaceValue`, 'returnType') = 'void'
attr(`NeighborhoodConnectedImageFilter_SetReplaceValue`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter', 'numeric')
class(`NeighborhoodConnectedImageFilter_SetReplaceValue`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_SetReplaceValue'))

# Start of NeighborhoodConnectedImageFilter_GetReplaceValue

`NeighborhoodConnectedImageFilter_GetReplaceValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NeighborhoodConnectedImageFilter_GetReplaceValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NeighborhoodConnectedImageFilter_GetReplaceValue`, 'returnType') = 'numeric'
attr(`NeighborhoodConnectedImageFilter_GetReplaceValue`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter')
class(`NeighborhoodConnectedImageFilter_GetReplaceValue`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_GetReplaceValue'))

# Start of NeighborhoodConnectedImageFilter_GetName

`NeighborhoodConnectedImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NeighborhoodConnectedImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NeighborhoodConnectedImageFilter_GetName`, 'returnType') = 'character'
attr(`NeighborhoodConnectedImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter')
class(`NeighborhoodConnectedImageFilter_GetName`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_GetName'))

# Start of NeighborhoodConnectedImageFilter_ToString

`NeighborhoodConnectedImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NeighborhoodConnectedImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NeighborhoodConnectedImageFilter_ToString`, 'returnType') = 'character'
attr(`NeighborhoodConnectedImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter')
class(`NeighborhoodConnectedImageFilter_ToString`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_ToString'))

# Start of NeighborhoodConnectedImageFilter_Execute

`NeighborhoodConnectedImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_NeighborhoodConnectedImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NeighborhoodConnectedImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`NeighborhoodConnectedImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__NeighborhoodConnectedImageFilter', '_p_itk__simple__Image')
class(`NeighborhoodConnectedImageFilter_Execute`) = c("SWIGFunction", class('NeighborhoodConnectedImageFilter_Execute'))

# Start of accessor method for itk::simple::NeighborhoodConnectedImageFilter
setMethod('$', '_p_itk__simple__NeighborhoodConnectedImageFilter', function(x, name)

{
  accessorFuns = list('SetSeedList' = NeighborhoodConnectedImageFilter_SetSeedList, 'GetSeedList' = NeighborhoodConnectedImageFilter_GetSeedList, 'AddSeed' = NeighborhoodConnectedImageFilter_AddSeed, 'ClearSeeds' = NeighborhoodConnectedImageFilter_ClearSeeds, 'SetLower' = NeighborhoodConnectedImageFilter_SetLower, 'GetLower' = NeighborhoodConnectedImageFilter_GetLower, 'SetUpper' = NeighborhoodConnectedImageFilter_SetUpper, 'GetUpper' = NeighborhoodConnectedImageFilter_GetUpper, 'SetRadius' = NeighborhoodConnectedImageFilter_SetRadius, 'GetRadius' = NeighborhoodConnectedImageFilter_GetRadius, 'SetReplaceValue' = NeighborhoodConnectedImageFilter_SetReplaceValue, 'GetReplaceValue' = NeighborhoodConnectedImageFilter_GetReplaceValue, 'GetName' = NeighborhoodConnectedImageFilter_GetName, 'ToString' = NeighborhoodConnectedImageFilter_ToString, 'Execute' = NeighborhoodConnectedImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::NeighborhoodConnectedImageFilter
# Start of NeighborhoodConnected

`NeighborhoodConnected__SWIG_0` = function(image1, seedList, lower, upper, radius, replaceValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  
  
  radius = as.integer(radius);
  
  ;ans = .Call('R_swig_NeighborhoodConnected__SWIG_0', image1, seedList, lower, upper, radius, replaceValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NeighborhoodConnected__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`NeighborhoodConnected__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'numeric', 'numeric', 'integer', 'numeric')
class(`NeighborhoodConnected__SWIG_0`) = c("SWIGFunction", class('NeighborhoodConnected__SWIG_0'))

# Start of NeighborhoodConnected

`NeighborhoodConnected__SWIG_1` = function(image1, seedList, lower, upper, radius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  
  
  radius = as.integer(radius);
  ;ans = .Call('R_swig_NeighborhoodConnected__SWIG_1', image1, seedList, lower, upper, radius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NeighborhoodConnected__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`NeighborhoodConnected__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'numeric', 'numeric', 'integer')
class(`NeighborhoodConnected__SWIG_1`) = c("SWIGFunction", class('NeighborhoodConnected__SWIG_1'))

# Start of NeighborhoodConnected

`NeighborhoodConnected__SWIG_2` = function(image1, seedList, lower, upper, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  
  
  ;ans = .Call('R_swig_NeighborhoodConnected__SWIG_2', image1, seedList, lower, upper, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NeighborhoodConnected__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`NeighborhoodConnected__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'numeric', 'numeric')
class(`NeighborhoodConnected__SWIG_2`) = c("SWIGFunction", class('NeighborhoodConnected__SWIG_2'))

# Start of NeighborhoodConnected

`NeighborhoodConnected__SWIG_3` = function(image1, seedList, lower, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  
  ;ans = .Call('R_swig_NeighborhoodConnected__SWIG_3', image1, seedList, lower, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NeighborhoodConnected__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`NeighborhoodConnected__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'numeric')
class(`NeighborhoodConnected__SWIG_3`) = c("SWIGFunction", class('NeighborhoodConnected__SWIG_3'))

# Start of NeighborhoodConnected

`NeighborhoodConnected__SWIG_4` = function(image1, seedList, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  ;ans = .Call('R_swig_NeighborhoodConnected__SWIG_4', image1, seedList, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NeighborhoodConnected__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`NeighborhoodConnected__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'list')
class(`NeighborhoodConnected__SWIG_4`) = c("SWIGFunction", class('NeighborhoodConnected__SWIG_4'))

# Start of NeighborhoodConnected

`NeighborhoodConnected__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_NeighborhoodConnected__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NeighborhoodConnected__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`NeighborhoodConnected__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`NeighborhoodConnected__SWIG_5`) = c("SWIGFunction", class('NeighborhoodConnected__SWIG_5'))

`NeighborhoodConnected` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- NeighborhoodConnected__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) )) {
      f <- NeighborhoodConnected__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- NeighborhoodConnected__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- NeighborhoodConnected__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.integer(argv[[5]]) || is.numeric(argv[[5]]) )) {
      f <- NeighborhoodConnected__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.integer(argv[[5]]) || is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- NeighborhoodConnected__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for NeighborhoodConnected with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_NoiseImageFilter

`delete_NoiseImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_NoiseImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_NoiseImageFilter`, 'returnType') = 'void'
attr(`delete_NoiseImageFilter`, "inputTypes") = c('_p_itk__simple__NoiseImageFilter')
class(`delete_NoiseImageFilter`) = c("SWIGFunction", class('delete_NoiseImageFilter'))

# Start of new_NoiseImageFilter

`NoiseImageFilter` = function()
{
  ;ans = .Call('R_swig_new_NoiseImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__NoiseImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_NoiseImageFilter);
  ans
  
}

attr(`NoiseImageFilter`, 'returnType') = '_p_itk__simple__NoiseImageFilter'
class(`NoiseImageFilter`) = c("SWIGFunction", class('NoiseImageFilter'))

# Start of NoiseImageFilter_SetRadius

`NoiseImageFilter_SetRadius__SWIG_0` = function(self, Radius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Radius = as.integer(Radius);
  ;ans = .Call('R_swig_NoiseImageFilter_SetRadius__SWIG_0', self, Radius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`NoiseImageFilter_SetRadius__SWIG_0`, 'returnType') = 'void'
attr(`NoiseImageFilter_SetRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__NoiseImageFilter', 'integer')
class(`NoiseImageFilter_SetRadius__SWIG_0`) = c("SWIGFunction", class('NoiseImageFilter_SetRadius__SWIG_0'))

# Start of NoiseImageFilter_SetRadius

`NoiseImageFilter_SetRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_NoiseImageFilter_SetRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`NoiseImageFilter_SetRadius__SWIG_1`, 'returnType') = 'void'
attr(`NoiseImageFilter_SetRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__NoiseImageFilter', 'integer')
class(`NoiseImageFilter_SetRadius__SWIG_1`) = c("SWIGFunction", class('NoiseImageFilter_SetRadius__SWIG_1'))

`NoiseImageFilter_SetRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__NoiseImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- NoiseImageFilter_SetRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__NoiseImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- NoiseImageFilter_SetRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for NoiseImageFilter_SetRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of NoiseImageFilter_GetRadius

`NoiseImageFilter_GetRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NoiseImageFilter_GetRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NoiseImageFilter_GetRadius`, 'returnType') = 'integer'
attr(`NoiseImageFilter_GetRadius`, "inputTypes") = c('_p_itk__simple__NoiseImageFilter')
class(`NoiseImageFilter_GetRadius`) = c("SWIGFunction", class('NoiseImageFilter_GetRadius'))

# Start of NoiseImageFilter_GetName

`NoiseImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NoiseImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NoiseImageFilter_GetName`, 'returnType') = 'character'
attr(`NoiseImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__NoiseImageFilter')
class(`NoiseImageFilter_GetName`) = c("SWIGFunction", class('NoiseImageFilter_GetName'))

# Start of NoiseImageFilter_ToString

`NoiseImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NoiseImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NoiseImageFilter_ToString`, 'returnType') = 'character'
attr(`NoiseImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__NoiseImageFilter')
class(`NoiseImageFilter_ToString`) = c("SWIGFunction", class('NoiseImageFilter_ToString'))

# Start of NoiseImageFilter_Execute

`NoiseImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_NoiseImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NoiseImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`NoiseImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__NoiseImageFilter', '_p_itk__simple__Image')
class(`NoiseImageFilter_Execute`) = c("SWIGFunction", class('NoiseImageFilter_Execute'))

# Start of accessor method for itk::simple::NoiseImageFilter
setMethod('$', '_p_itk__simple__NoiseImageFilter', function(x, name)

{
  accessorFuns = list('SetRadius' = NoiseImageFilter_SetRadius, 'GetRadius' = NoiseImageFilter_GetRadius, 'GetName' = NoiseImageFilter_GetName, 'ToString' = NoiseImageFilter_ToString, 'Execute' = NoiseImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::NoiseImageFilter
# Start of Noise

`Noise__SWIG_0` = function(image1, radius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  ;ans = .Call('R_swig_Noise__SWIG_0', image1, radius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Noise__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Noise__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Noise__SWIG_0`) = c("SWIGFunction", class('Noise__SWIG_0'))

# Start of Noise

`Noise__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Noise__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Noise__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Noise__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`Noise__SWIG_1`) = c("SWIGFunction", class('Noise__SWIG_1'))

`Noise` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Noise__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- Noise__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Noise with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_NormalizeImageFilter

`delete_NormalizeImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_NormalizeImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_NormalizeImageFilter`, 'returnType') = 'void'
attr(`delete_NormalizeImageFilter`, "inputTypes") = c('_p_itk__simple__NormalizeImageFilter')
class(`delete_NormalizeImageFilter`) = c("SWIGFunction", class('delete_NormalizeImageFilter'))

# Start of new_NormalizeImageFilter

`NormalizeImageFilter` = function()
{
  ;ans = .Call('R_swig_new_NormalizeImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__NormalizeImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_NormalizeImageFilter);
  ans
  
}

attr(`NormalizeImageFilter`, 'returnType') = '_p_itk__simple__NormalizeImageFilter'
class(`NormalizeImageFilter`) = c("SWIGFunction", class('NormalizeImageFilter'))

# Start of NormalizeImageFilter_GetName

`NormalizeImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NormalizeImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NormalizeImageFilter_GetName`, 'returnType') = 'character'
attr(`NormalizeImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__NormalizeImageFilter')
class(`NormalizeImageFilter_GetName`) = c("SWIGFunction", class('NormalizeImageFilter_GetName'))

# Start of NormalizeImageFilter_ToString

`NormalizeImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NormalizeImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NormalizeImageFilter_ToString`, 'returnType') = 'character'
attr(`NormalizeImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__NormalizeImageFilter')
class(`NormalizeImageFilter_ToString`) = c("SWIGFunction", class('NormalizeImageFilter_ToString'))

# Start of NormalizeImageFilter_Execute

`NormalizeImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_NormalizeImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NormalizeImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`NormalizeImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__NormalizeImageFilter', '_p_itk__simple__Image')
class(`NormalizeImageFilter_Execute`) = c("SWIGFunction", class('NormalizeImageFilter_Execute'))

# Start of accessor method for itk::simple::NormalizeImageFilter
setMethod('$', '_p_itk__simple__NormalizeImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = NormalizeImageFilter_GetName, 'ToString' = NormalizeImageFilter_ToString, 'Execute' = NormalizeImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::NormalizeImageFilter
# Start of Normalize

`Normalize` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Normalize', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Normalize`, 'returnType') = '_p_itk__simple__Image'
attr(`Normalize`, "inputTypes") = c('_p_itk__simple__Image')
class(`Normalize`) = c("SWIGFunction", class('Normalize'))

# Start of delete_NormalizeToConstantImageFilter

`delete_NormalizeToConstantImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_NormalizeToConstantImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_NormalizeToConstantImageFilter`, 'returnType') = 'void'
attr(`delete_NormalizeToConstantImageFilter`, "inputTypes") = c('_p_itk__simple__NormalizeToConstantImageFilter')
class(`delete_NormalizeToConstantImageFilter`) = c("SWIGFunction", class('delete_NormalizeToConstantImageFilter'))

# Start of new_NormalizeToConstantImageFilter

`NormalizeToConstantImageFilter` = function()
{
  ;ans = .Call('R_swig_new_NormalizeToConstantImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__NormalizeToConstantImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_NormalizeToConstantImageFilter);
  ans
  
}

attr(`NormalizeToConstantImageFilter`, 'returnType') = '_p_itk__simple__NormalizeToConstantImageFilter'
class(`NormalizeToConstantImageFilter`) = c("SWIGFunction", class('NormalizeToConstantImageFilter'))

# Start of NormalizeToConstantImageFilter_SetConstant

`NormalizeToConstantImageFilter_SetConstant` = function(self, Constant)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_NormalizeToConstantImageFilter_SetConstant', self, Constant, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`NormalizeToConstantImageFilter_SetConstant`, 'returnType') = 'void'
attr(`NormalizeToConstantImageFilter_SetConstant`, "inputTypes") = c('_p_itk__simple__NormalizeToConstantImageFilter', 'numeric')
class(`NormalizeToConstantImageFilter_SetConstant`) = c("SWIGFunction", class('NormalizeToConstantImageFilter_SetConstant'))

# Start of NormalizeToConstantImageFilter_GetConstant

`NormalizeToConstantImageFilter_GetConstant` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NormalizeToConstantImageFilter_GetConstant', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NormalizeToConstantImageFilter_GetConstant`, 'returnType') = 'numeric'
attr(`NormalizeToConstantImageFilter_GetConstant`, "inputTypes") = c('_p_itk__simple__NormalizeToConstantImageFilter')
class(`NormalizeToConstantImageFilter_GetConstant`) = c("SWIGFunction", class('NormalizeToConstantImageFilter_GetConstant'))

# Start of NormalizeToConstantImageFilter_GetName

`NormalizeToConstantImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NormalizeToConstantImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NormalizeToConstantImageFilter_GetName`, 'returnType') = 'character'
attr(`NormalizeToConstantImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__NormalizeToConstantImageFilter')
class(`NormalizeToConstantImageFilter_GetName`) = c("SWIGFunction", class('NormalizeToConstantImageFilter_GetName'))

# Start of NormalizeToConstantImageFilter_ToString

`NormalizeToConstantImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NormalizeToConstantImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NormalizeToConstantImageFilter_ToString`, 'returnType') = 'character'
attr(`NormalizeToConstantImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__NormalizeToConstantImageFilter')
class(`NormalizeToConstantImageFilter_ToString`) = c("SWIGFunction", class('NormalizeToConstantImageFilter_ToString'))

# Start of NormalizeToConstantImageFilter_Execute

`NormalizeToConstantImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_NormalizeToConstantImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NormalizeToConstantImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`NormalizeToConstantImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__NormalizeToConstantImageFilter', '_p_itk__simple__Image')
class(`NormalizeToConstantImageFilter_Execute`) = c("SWIGFunction", class('NormalizeToConstantImageFilter_Execute'))

# Start of accessor method for itk::simple::NormalizeToConstantImageFilter
setMethod('$', '_p_itk__simple__NormalizeToConstantImageFilter', function(x, name)

{
  accessorFuns = list('SetConstant' = NormalizeToConstantImageFilter_SetConstant, 'GetConstant' = NormalizeToConstantImageFilter_GetConstant, 'GetName' = NormalizeToConstantImageFilter_GetName, 'ToString' = NormalizeToConstantImageFilter_ToString, 'Execute' = NormalizeToConstantImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::NormalizeToConstantImageFilter
# Start of NormalizeToConstant

`NormalizeToConstant__SWIG_0` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_NormalizeToConstant__SWIG_0', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NormalizeToConstant__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`NormalizeToConstant__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`NormalizeToConstant__SWIG_0`) = c("SWIGFunction", class('NormalizeToConstant__SWIG_0'))

# Start of NormalizeToConstant

`NormalizeToConstant__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_NormalizeToConstant__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NormalizeToConstant__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`NormalizeToConstant__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`NormalizeToConstant__SWIG_1`) = c("SWIGFunction", class('NormalizeToConstant__SWIG_1'))

`NormalizeToConstant` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- NormalizeToConstant__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- NormalizeToConstant__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for NormalizeToConstant with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_NormalizedCorrelationImageFilter

`delete_NormalizedCorrelationImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_NormalizedCorrelationImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_NormalizedCorrelationImageFilter`, 'returnType') = 'void'
attr(`delete_NormalizedCorrelationImageFilter`, "inputTypes") = c('_p_itk__simple__NormalizedCorrelationImageFilter')
class(`delete_NormalizedCorrelationImageFilter`) = c("SWIGFunction", class('delete_NormalizedCorrelationImageFilter'))

# Start of new_NormalizedCorrelationImageFilter

`NormalizedCorrelationImageFilter` = function()
{
  ;ans = .Call('R_swig_new_NormalizedCorrelationImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__NormalizedCorrelationImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_NormalizedCorrelationImageFilter);
  ans
  
}

attr(`NormalizedCorrelationImageFilter`, 'returnType') = '_p_itk__simple__NormalizedCorrelationImageFilter'
class(`NormalizedCorrelationImageFilter`) = c("SWIGFunction", class('NormalizedCorrelationImageFilter'))

# Start of NormalizedCorrelationImageFilter_GetName

`NormalizedCorrelationImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NormalizedCorrelationImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NormalizedCorrelationImageFilter_GetName`, 'returnType') = 'character'
attr(`NormalizedCorrelationImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__NormalizedCorrelationImageFilter')
class(`NormalizedCorrelationImageFilter_GetName`) = c("SWIGFunction", class('NormalizedCorrelationImageFilter_GetName'))

# Start of NormalizedCorrelationImageFilter_ToString

`NormalizedCorrelationImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NormalizedCorrelationImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NormalizedCorrelationImageFilter_ToString`, 'returnType') = 'character'
attr(`NormalizedCorrelationImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__NormalizedCorrelationImageFilter')
class(`NormalizedCorrelationImageFilter_ToString`) = c("SWIGFunction", class('NormalizedCorrelationImageFilter_ToString'))

# Start of NormalizedCorrelationImageFilter_Execute

`NormalizedCorrelationImageFilter_Execute` = function(self, image, maskImage, templateImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  if (inherits(templateImage, "ExternalReference")) templateImage = slot(templateImage,"ref"); 
  ;ans = .Call('R_swig_NormalizedCorrelationImageFilter_Execute', self, image, maskImage, templateImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NormalizedCorrelationImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`NormalizedCorrelationImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__NormalizedCorrelationImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NormalizedCorrelationImageFilter_Execute`) = c("SWIGFunction", class('NormalizedCorrelationImageFilter_Execute'))

# Start of accessor method for itk::simple::NormalizedCorrelationImageFilter
setMethod('$', '_p_itk__simple__NormalizedCorrelationImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = NormalizedCorrelationImageFilter_GetName, 'ToString' = NormalizedCorrelationImageFilter_ToString, 'Execute' = NormalizedCorrelationImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::NormalizedCorrelationImageFilter
# Start of NormalizedCorrelation

`NormalizedCorrelation` = function(image, maskImage, templateImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  if (inherits(templateImage, "ExternalReference")) templateImage = slot(templateImage,"ref"); 
  ;ans = .Call('R_swig_NormalizedCorrelation', image, maskImage, templateImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NormalizedCorrelation`, 'returnType') = '_p_itk__simple__Image'
attr(`NormalizedCorrelation`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NormalizedCorrelation`) = c("SWIGFunction", class('NormalizedCorrelation'))

# Start of delete_NotEqualImageFilter

`delete_NotEqualImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_NotEqualImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_NotEqualImageFilter`, 'returnType') = 'void'
attr(`delete_NotEqualImageFilter`, "inputTypes") = c('_p_itk__simple__NotEqualImageFilter')
class(`delete_NotEqualImageFilter`) = c("SWIGFunction", class('delete_NotEqualImageFilter'))

# Start of new_NotEqualImageFilter

`NotEqualImageFilter` = function()
{
  ;ans = .Call('R_swig_new_NotEqualImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__NotEqualImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_NotEqualImageFilter);
  ans
  
}

attr(`NotEqualImageFilter`, 'returnType') = '_p_itk__simple__NotEqualImageFilter'
class(`NotEqualImageFilter`) = c("SWIGFunction", class('NotEqualImageFilter'))

# Start of NotEqualImageFilter_SetBackgroundValue

`NotEqualImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BackgroundValue = as.integer(BackgroundValue);
  
  if(length(BackgroundValue) > 1) {
    warning("using only the first element of BackgroundValue");
  };
  
  ;ans = .Call('R_swig_NotEqualImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`NotEqualImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`NotEqualImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__NotEqualImageFilter', 'integer')
class(`NotEqualImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('NotEqualImageFilter_SetBackgroundValue'))

# Start of NotEqualImageFilter_GetBackgroundValue

`NotEqualImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NotEqualImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NotEqualImageFilter_GetBackgroundValue`, 'returnType') = 'integer'
attr(`NotEqualImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__NotEqualImageFilter')
class(`NotEqualImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('NotEqualImageFilter_GetBackgroundValue'))

# Start of NotEqualImageFilter_SetForegroundValue

`NotEqualImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ForegroundValue = as.integer(ForegroundValue);
  
  if(length(ForegroundValue) > 1) {
    warning("using only the first element of ForegroundValue");
  };
  
  ;ans = .Call('R_swig_NotEqualImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`NotEqualImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`NotEqualImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__NotEqualImageFilter', 'integer')
class(`NotEqualImageFilter_SetForegroundValue`) = c("SWIGFunction", class('NotEqualImageFilter_SetForegroundValue'))

# Start of NotEqualImageFilter_GetForegroundValue

`NotEqualImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NotEqualImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NotEqualImageFilter_GetForegroundValue`, 'returnType') = 'integer'
attr(`NotEqualImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__NotEqualImageFilter')
class(`NotEqualImageFilter_GetForegroundValue`) = c("SWIGFunction", class('NotEqualImageFilter_GetForegroundValue'))

# Start of NotEqualImageFilter_GetName

`NotEqualImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NotEqualImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NotEqualImageFilter_GetName`, 'returnType') = 'character'
attr(`NotEqualImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__NotEqualImageFilter')
class(`NotEqualImageFilter_GetName`) = c("SWIGFunction", class('NotEqualImageFilter_GetName'))

# Start of NotEqualImageFilter_ToString

`NotEqualImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NotEqualImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NotEqualImageFilter_ToString`, 'returnType') = 'character'
attr(`NotEqualImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__NotEqualImageFilter')
class(`NotEqualImageFilter_ToString`) = c("SWIGFunction", class('NotEqualImageFilter_ToString'))

# Start of NotEqualImageFilter_Execute

`NotEqualImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_NotEqualImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NotEqualImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`NotEqualImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__NotEqualImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NotEqualImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('NotEqualImageFilter_Execute__SWIG_0'))

# Start of NotEqualImageFilter_Execute

`NotEqualImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_NotEqualImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NotEqualImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`NotEqualImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__NotEqualImageFilter', '_p_itk__simple__Image', 'numeric')
class(`NotEqualImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('NotEqualImageFilter_Execute__SWIG_1'))

# Start of NotEqualImageFilter_Execute

`NotEqualImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_NotEqualImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NotEqualImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`NotEqualImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__NotEqualImageFilter', 'numeric', '_p_itk__simple__Image')
class(`NotEqualImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('NotEqualImageFilter_Execute__SWIG_2'))

# Start of NotEqualImageFilter_Execute

`NotEqualImageFilter_Execute__SWIG_3` = function(self, image1, constant, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_NotEqualImageFilter_Execute__SWIG_3', self, image1, constant, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NotEqualImageFilter_Execute__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`NotEqualImageFilter_Execute__SWIG_3`, "inputTypes") = c('_p_itk__simple__NotEqualImageFilter', '_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`NotEqualImageFilter_Execute__SWIG_3`) = c("SWIGFunction", class('NotEqualImageFilter_Execute__SWIG_3'))

# Start of NotEqualImageFilter_Execute

`NotEqualImageFilter_Execute__SWIG_4` = function(self, constant, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_NotEqualImageFilter_Execute__SWIG_4', self, constant, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NotEqualImageFilter_Execute__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`NotEqualImageFilter_Execute__SWIG_4`, "inputTypes") = c('_p_itk__simple__NotEqualImageFilter', 'numeric', '_p_itk__simple__Image', 'integer', 'integer')
class(`NotEqualImageFilter_Execute__SWIG_4`) = c("SWIGFunction", class('NotEqualImageFilter_Execute__SWIG_4'))

`NotEqualImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__NotEqualImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- NotEqualImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__NotEqualImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- NotEqualImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__NotEqualImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- NotEqualImageFilter_Execute__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__NotEqualImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- NotEqualImageFilter_Execute__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__NotEqualImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- NotEqualImageFilter_Execute__SWIG_4; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for NotEqualImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::NotEqualImageFilter
setMethod('$', '_p_itk__simple__NotEqualImageFilter', function(x, name)

{
  accessorFuns = list('SetBackgroundValue' = NotEqualImageFilter_SetBackgroundValue, 'GetBackgroundValue' = NotEqualImageFilter_GetBackgroundValue, 'SetForegroundValue' = NotEqualImageFilter_SetForegroundValue, 'GetForegroundValue' = NotEqualImageFilter_GetForegroundValue, 'GetName' = NotEqualImageFilter_GetName, 'ToString' = NotEqualImageFilter_ToString, 'Execute' = NotEqualImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::NotEqualImageFilter
# Start of NotEqual

`NotEqual__SWIG_0` = function(image1, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_NotEqual__SWIG_0', image1, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NotEqual__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`NotEqual__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`NotEqual__SWIG_0`) = c("SWIGFunction", class('NotEqual__SWIG_0'))

# Start of NotEqual

`NotEqual__SWIG_1` = function(image1, image2, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_NotEqual__SWIG_1', image1, image2, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NotEqual__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`NotEqual__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`NotEqual__SWIG_1`) = c("SWIGFunction", class('NotEqual__SWIG_1'))

# Start of NotEqual

`NotEqual__SWIG_2` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_NotEqual__SWIG_2', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NotEqual__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`NotEqual__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`NotEqual__SWIG_2`) = c("SWIGFunction", class('NotEqual__SWIG_2'))

# Start of NotEqual

`NotEqual__SWIG_3` = function(image1, constant, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_NotEqual__SWIG_3', image1, constant, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NotEqual__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`NotEqual__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`NotEqual__SWIG_3`) = c("SWIGFunction", class('NotEqual__SWIG_3'))

# Start of NotEqual

`NotEqual__SWIG_4` = function(image1, constant, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_NotEqual__SWIG_4', image1, constant, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NotEqual__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`NotEqual__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`NotEqual__SWIG_4`) = c("SWIGFunction", class('NotEqual__SWIG_4'))

# Start of NotEqual

`NotEqual__SWIG_5` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_NotEqual__SWIG_5', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NotEqual__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`NotEqual__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`NotEqual__SWIG_5`) = c("SWIGFunction", class('NotEqual__SWIG_5'))

# Start of NotEqual

`NotEqual__SWIG_6` = function(constant, image2, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_NotEqual__SWIG_6', constant, image2, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NotEqual__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`NotEqual__SWIG_6`, "inputTypes") = c('numeric', '_p_itk__simple__Image', 'integer', 'integer')
class(`NotEqual__SWIG_6`) = c("SWIGFunction", class('NotEqual__SWIG_6'))

# Start of NotEqual

`NotEqual__SWIG_7` = function(constant, image2, backgroundValue, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_NotEqual__SWIG_7', constant, image2, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NotEqual__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`NotEqual__SWIG_7`, "inputTypes") = c('numeric', '_p_itk__simple__Image', 'integer')
class(`NotEqual__SWIG_7`) = c("SWIGFunction", class('NotEqual__SWIG_7'))

# Start of NotEqual

`NotEqual__SWIG_8` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_NotEqual__SWIG_8', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NotEqual__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`NotEqual__SWIG_8`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`NotEqual__SWIG_8`) = c("SWIGFunction", class('NotEqual__SWIG_8'))

`NotEqual` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 9
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- NotEqual__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- NotEqual__SWIG_5; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- NotEqual__SWIG_8; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- NotEqual__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- NotEqual__SWIG_4; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- NotEqual__SWIG_7; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- NotEqual__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- NotEqual__SWIG_3; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- NotEqual__SWIG_6; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for NotEqual with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_NotImageFilter

`delete_NotImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_NotImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_NotImageFilter`, 'returnType') = 'void'
attr(`delete_NotImageFilter`, "inputTypes") = c('_p_itk__simple__NotImageFilter')
class(`delete_NotImageFilter`) = c("SWIGFunction", class('delete_NotImageFilter'))

# Start of new_NotImageFilter

`NotImageFilter` = function()
{
  ;ans = .Call('R_swig_new_NotImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__NotImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_NotImageFilter);
  ans
  
}

attr(`NotImageFilter`, 'returnType') = '_p_itk__simple__NotImageFilter'
class(`NotImageFilter`) = c("SWIGFunction", class('NotImageFilter'))

# Start of NotImageFilter_GetName

`NotImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NotImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NotImageFilter_GetName`, 'returnType') = 'character'
attr(`NotImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__NotImageFilter')
class(`NotImageFilter_GetName`) = c("SWIGFunction", class('NotImageFilter_GetName'))

# Start of NotImageFilter_ToString

`NotImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_NotImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`NotImageFilter_ToString`, 'returnType') = 'character'
attr(`NotImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__NotImageFilter')
class(`NotImageFilter_ToString`) = c("SWIGFunction", class('NotImageFilter_ToString'))

# Start of NotImageFilter_Execute

`NotImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_NotImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`NotImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`NotImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__NotImageFilter', '_p_itk__simple__Image')
class(`NotImageFilter_Execute`) = c("SWIGFunction", class('NotImageFilter_Execute'))

# Start of accessor method for itk::simple::NotImageFilter
setMethod('$', '_p_itk__simple__NotImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = NotImageFilter_GetName, 'ToString' = NotImageFilter_ToString, 'Execute' = NotImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::NotImageFilter
# Start of Not

`Not` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Not', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Not`, 'returnType') = '_p_itk__simple__Image'
attr(`Not`, "inputTypes") = c('_p_itk__simple__Image')
class(`Not`) = c("SWIGFunction", class('Not'))

# Start of delete_ObjectnessMeasureImageFilter

`delete_ObjectnessMeasureImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ObjectnessMeasureImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ObjectnessMeasureImageFilter`, 'returnType') = 'void'
attr(`delete_ObjectnessMeasureImageFilter`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter')
class(`delete_ObjectnessMeasureImageFilter`) = c("SWIGFunction", class('delete_ObjectnessMeasureImageFilter'))

# Start of new_ObjectnessMeasureImageFilter

`ObjectnessMeasureImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ObjectnessMeasureImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ObjectnessMeasureImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ObjectnessMeasureImageFilter);
  ans
  
}

attr(`ObjectnessMeasureImageFilter`, 'returnType') = '_p_itk__simple__ObjectnessMeasureImageFilter'
class(`ObjectnessMeasureImageFilter`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter'))

# Start of ObjectnessMeasureImageFilter_SetAlpha

`ObjectnessMeasureImageFilter_SetAlpha` = function(self, Alpha)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ObjectnessMeasureImageFilter_SetAlpha', self, Alpha, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ObjectnessMeasureImageFilter_SetAlpha`, 'returnType') = 'void'
attr(`ObjectnessMeasureImageFilter_SetAlpha`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter', 'numeric')
class(`ObjectnessMeasureImageFilter_SetAlpha`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_SetAlpha'))

# Start of ObjectnessMeasureImageFilter_GetAlpha

`ObjectnessMeasureImageFilter_GetAlpha` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ObjectnessMeasureImageFilter_GetAlpha', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ObjectnessMeasureImageFilter_GetAlpha`, 'returnType') = 'numeric'
attr(`ObjectnessMeasureImageFilter_GetAlpha`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter')
class(`ObjectnessMeasureImageFilter_GetAlpha`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_GetAlpha'))

# Start of ObjectnessMeasureImageFilter_SetBeta

`ObjectnessMeasureImageFilter_SetBeta` = function(self, Beta)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ObjectnessMeasureImageFilter_SetBeta', self, Beta, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ObjectnessMeasureImageFilter_SetBeta`, 'returnType') = 'void'
attr(`ObjectnessMeasureImageFilter_SetBeta`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter', 'numeric')
class(`ObjectnessMeasureImageFilter_SetBeta`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_SetBeta'))

# Start of ObjectnessMeasureImageFilter_GetBeta

`ObjectnessMeasureImageFilter_GetBeta` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ObjectnessMeasureImageFilter_GetBeta', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ObjectnessMeasureImageFilter_GetBeta`, 'returnType') = 'numeric'
attr(`ObjectnessMeasureImageFilter_GetBeta`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter')
class(`ObjectnessMeasureImageFilter_GetBeta`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_GetBeta'))

# Start of ObjectnessMeasureImageFilter_SetGamma

`ObjectnessMeasureImageFilter_SetGamma` = function(self, Gamma)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ObjectnessMeasureImageFilter_SetGamma', self, Gamma, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ObjectnessMeasureImageFilter_SetGamma`, 'returnType') = 'void'
attr(`ObjectnessMeasureImageFilter_SetGamma`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter', 'numeric')
class(`ObjectnessMeasureImageFilter_SetGamma`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_SetGamma'))

# Start of ObjectnessMeasureImageFilter_GetGamma

`ObjectnessMeasureImageFilter_GetGamma` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ObjectnessMeasureImageFilter_GetGamma', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ObjectnessMeasureImageFilter_GetGamma`, 'returnType') = 'numeric'
attr(`ObjectnessMeasureImageFilter_GetGamma`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter')
class(`ObjectnessMeasureImageFilter_GetGamma`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_GetGamma'))

# Start of ObjectnessMeasureImageFilter_SetScaleObjectnessMeasure

`ObjectnessMeasureImageFilter_SetScaleObjectnessMeasure` = function(self, ScaleObjectnessMeasure)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ScaleObjectnessMeasure = as.logical(ScaleObjectnessMeasure);
  ;ans = .Call('R_swig_ObjectnessMeasureImageFilter_SetScaleObjectnessMeasure', self, ScaleObjectnessMeasure, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ObjectnessMeasureImageFilter_SetScaleObjectnessMeasure`, 'returnType') = 'void'
attr(`ObjectnessMeasureImageFilter_SetScaleObjectnessMeasure`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter', 'logical')
class(`ObjectnessMeasureImageFilter_SetScaleObjectnessMeasure`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_SetScaleObjectnessMeasure'))

# Start of ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOn

`ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOn`, 'returnType') = 'void'
attr(`ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOn`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter')
class(`ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOn`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOn'))

# Start of ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOff

`ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOff`, 'returnType') = 'void'
attr(`ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOff`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter')
class(`ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOff`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOff'))

# Start of ObjectnessMeasureImageFilter_GetScaleObjectnessMeasure

`ObjectnessMeasureImageFilter_GetScaleObjectnessMeasure` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ObjectnessMeasureImageFilter_GetScaleObjectnessMeasure', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ObjectnessMeasureImageFilter_GetScaleObjectnessMeasure`, 'returnType') = 'logical'
attr(`ObjectnessMeasureImageFilter_GetScaleObjectnessMeasure`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter')
class(`ObjectnessMeasureImageFilter_GetScaleObjectnessMeasure`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_GetScaleObjectnessMeasure'))

# Start of ObjectnessMeasureImageFilter_SetObjectDimension

`ObjectnessMeasureImageFilter_SetObjectDimension` = function(self, ObjectDimension)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ObjectDimension = as.integer(ObjectDimension);
  
  if(length(ObjectDimension) > 1) {
    warning("using only the first element of ObjectDimension");
  };
  
  ;ans = .Call('R_swig_ObjectnessMeasureImageFilter_SetObjectDimension', self, ObjectDimension, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ObjectnessMeasureImageFilter_SetObjectDimension`, 'returnType') = 'void'
attr(`ObjectnessMeasureImageFilter_SetObjectDimension`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter', 'integer')
class(`ObjectnessMeasureImageFilter_SetObjectDimension`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_SetObjectDimension'))

# Start of ObjectnessMeasureImageFilter_GetObjectDimension

`ObjectnessMeasureImageFilter_GetObjectDimension` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ObjectnessMeasureImageFilter_GetObjectDimension', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ObjectnessMeasureImageFilter_GetObjectDimension`, 'returnType') = 'integer'
attr(`ObjectnessMeasureImageFilter_GetObjectDimension`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter')
class(`ObjectnessMeasureImageFilter_GetObjectDimension`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_GetObjectDimension'))

# Start of ObjectnessMeasureImageFilter_SetBrightObject

`ObjectnessMeasureImageFilter_SetBrightObject` = function(self, BrightObject)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BrightObject = as.logical(BrightObject);
  ;ans = .Call('R_swig_ObjectnessMeasureImageFilter_SetBrightObject', self, BrightObject, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ObjectnessMeasureImageFilter_SetBrightObject`, 'returnType') = 'void'
attr(`ObjectnessMeasureImageFilter_SetBrightObject`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter', 'logical')
class(`ObjectnessMeasureImageFilter_SetBrightObject`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_SetBrightObject'))

# Start of ObjectnessMeasureImageFilter_BrightObjectOn

`ObjectnessMeasureImageFilter_BrightObjectOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ObjectnessMeasureImageFilter_BrightObjectOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ObjectnessMeasureImageFilter_BrightObjectOn`, 'returnType') = 'void'
attr(`ObjectnessMeasureImageFilter_BrightObjectOn`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter')
class(`ObjectnessMeasureImageFilter_BrightObjectOn`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_BrightObjectOn'))

# Start of ObjectnessMeasureImageFilter_BrightObjectOff

`ObjectnessMeasureImageFilter_BrightObjectOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ObjectnessMeasureImageFilter_BrightObjectOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ObjectnessMeasureImageFilter_BrightObjectOff`, 'returnType') = 'void'
attr(`ObjectnessMeasureImageFilter_BrightObjectOff`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter')
class(`ObjectnessMeasureImageFilter_BrightObjectOff`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_BrightObjectOff'))

# Start of ObjectnessMeasureImageFilter_GetBrightObject

`ObjectnessMeasureImageFilter_GetBrightObject` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ObjectnessMeasureImageFilter_GetBrightObject', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ObjectnessMeasureImageFilter_GetBrightObject`, 'returnType') = 'logical'
attr(`ObjectnessMeasureImageFilter_GetBrightObject`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter')
class(`ObjectnessMeasureImageFilter_GetBrightObject`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_GetBrightObject'))

# Start of ObjectnessMeasureImageFilter_GetName

`ObjectnessMeasureImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ObjectnessMeasureImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ObjectnessMeasureImageFilter_GetName`, 'returnType') = 'character'
attr(`ObjectnessMeasureImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter')
class(`ObjectnessMeasureImageFilter_GetName`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_GetName'))

# Start of ObjectnessMeasureImageFilter_ToString

`ObjectnessMeasureImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ObjectnessMeasureImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ObjectnessMeasureImageFilter_ToString`, 'returnType') = 'character'
attr(`ObjectnessMeasureImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter')
class(`ObjectnessMeasureImageFilter_ToString`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_ToString'))

# Start of ObjectnessMeasureImageFilter_Execute

`ObjectnessMeasureImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ObjectnessMeasureImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ObjectnessMeasureImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ObjectnessMeasureImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ObjectnessMeasureImageFilter', '_p_itk__simple__Image')
class(`ObjectnessMeasureImageFilter_Execute`) = c("SWIGFunction", class('ObjectnessMeasureImageFilter_Execute'))

# Start of accessor method for itk::simple::ObjectnessMeasureImageFilter
setMethod('$', '_p_itk__simple__ObjectnessMeasureImageFilter', function(x, name)

{
  accessorFuns = list('SetAlpha' = ObjectnessMeasureImageFilter_SetAlpha, 'GetAlpha' = ObjectnessMeasureImageFilter_GetAlpha, 'SetBeta' = ObjectnessMeasureImageFilter_SetBeta, 'GetBeta' = ObjectnessMeasureImageFilter_GetBeta, 'SetGamma' = ObjectnessMeasureImageFilter_SetGamma, 'GetGamma' = ObjectnessMeasureImageFilter_GetGamma, 'SetScaleObjectnessMeasure' = ObjectnessMeasureImageFilter_SetScaleObjectnessMeasure, 'ScaleObjectnessMeasureOn' = ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOn, 'ScaleObjectnessMeasureOff' = ObjectnessMeasureImageFilter_ScaleObjectnessMeasureOff, 'GetScaleObjectnessMeasure' = ObjectnessMeasureImageFilter_GetScaleObjectnessMeasure, 'SetObjectDimension' = ObjectnessMeasureImageFilter_SetObjectDimension, 'GetObjectDimension' = ObjectnessMeasureImageFilter_GetObjectDimension, 'SetBrightObject' = ObjectnessMeasureImageFilter_SetBrightObject, 'BrightObjectOn' = ObjectnessMeasureImageFilter_BrightObjectOn, 'BrightObjectOff' = ObjectnessMeasureImageFilter_BrightObjectOff, 'GetBrightObject' = ObjectnessMeasureImageFilter_GetBrightObject, 'GetName' = ObjectnessMeasureImageFilter_GetName, 'ToString' = ObjectnessMeasureImageFilter_ToString, 'Execute' = ObjectnessMeasureImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ObjectnessMeasureImageFilter
# Start of ObjectnessMeasure

`ObjectnessMeasure__SWIG_0` = function(image1, alpha, beta, gamma, scaleObjectnessMeasure, objectDimension, brightObject, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  
  scaleObjectnessMeasure = as.logical(scaleObjectnessMeasure);
  objectDimension = as.integer(objectDimension);
  
  if(length(objectDimension) > 1) {
    warning("using only the first element of objectDimension");
  };
  
  brightObject = as.logical(brightObject);
  ;ans = .Call('R_swig_ObjectnessMeasure__SWIG_0', image1, alpha, beta, gamma, scaleObjectnessMeasure, objectDimension, brightObject, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ObjectnessMeasure__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ObjectnessMeasure__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'logical', 'integer', 'logical')
class(`ObjectnessMeasure__SWIG_0`) = c("SWIGFunction", class('ObjectnessMeasure__SWIG_0'))

# Start of ObjectnessMeasure

`ObjectnessMeasure__SWIG_1` = function(image1, alpha, beta, gamma, scaleObjectnessMeasure, objectDimension, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  
  scaleObjectnessMeasure = as.logical(scaleObjectnessMeasure);
  objectDimension = as.integer(objectDimension);
  
  if(length(objectDimension) > 1) {
    warning("using only the first element of objectDimension");
  };
  
  ;ans = .Call('R_swig_ObjectnessMeasure__SWIG_1', image1, alpha, beta, gamma, scaleObjectnessMeasure, objectDimension, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ObjectnessMeasure__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ObjectnessMeasure__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'logical', 'integer')
class(`ObjectnessMeasure__SWIG_1`) = c("SWIGFunction", class('ObjectnessMeasure__SWIG_1'))

# Start of ObjectnessMeasure

`ObjectnessMeasure__SWIG_2` = function(image1, alpha, beta, gamma, scaleObjectnessMeasure, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  
  scaleObjectnessMeasure = as.logical(scaleObjectnessMeasure);
  ;ans = .Call('R_swig_ObjectnessMeasure__SWIG_2', image1, alpha, beta, gamma, scaleObjectnessMeasure, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ObjectnessMeasure__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ObjectnessMeasure__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'logical')
class(`ObjectnessMeasure__SWIG_2`) = c("SWIGFunction", class('ObjectnessMeasure__SWIG_2'))

# Start of ObjectnessMeasure

`ObjectnessMeasure__SWIG_3` = function(image1, alpha, beta, gamma, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  
  ;ans = .Call('R_swig_ObjectnessMeasure__SWIG_3', image1, alpha, beta, gamma, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ObjectnessMeasure__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ObjectnessMeasure__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric')
class(`ObjectnessMeasure__SWIG_3`) = c("SWIGFunction", class('ObjectnessMeasure__SWIG_3'))

# Start of ObjectnessMeasure

`ObjectnessMeasure__SWIG_4` = function(image1, alpha, beta, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_ObjectnessMeasure__SWIG_4', image1, alpha, beta, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ObjectnessMeasure__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ObjectnessMeasure__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`ObjectnessMeasure__SWIG_4`) = c("SWIGFunction", class('ObjectnessMeasure__SWIG_4'))

# Start of ObjectnessMeasure

`ObjectnessMeasure__SWIG_5` = function(image1, alpha, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_ObjectnessMeasure__SWIG_5', image1, alpha, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ObjectnessMeasure__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`ObjectnessMeasure__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`ObjectnessMeasure__SWIG_5`) = c("SWIGFunction", class('ObjectnessMeasure__SWIG_5'))

# Start of ObjectnessMeasure

`ObjectnessMeasure__SWIG_6` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ObjectnessMeasure__SWIG_6', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ObjectnessMeasure__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`ObjectnessMeasure__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image')
class(`ObjectnessMeasure__SWIG_6`) = c("SWIGFunction", class('ObjectnessMeasure__SWIG_6'))

`ObjectnessMeasure` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 7
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ObjectnessMeasure__SWIG_6; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ObjectnessMeasure__SWIG_5; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ObjectnessMeasure__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ObjectnessMeasure__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- ObjectnessMeasure__SWIG_2; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- ObjectnessMeasure__SWIG_1; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.logical(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- ObjectnessMeasure__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ObjectnessMeasure with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_OpeningByReconstructionImageFilter

`delete_OpeningByReconstructionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_OpeningByReconstructionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_OpeningByReconstructionImageFilter`, 'returnType') = 'void'
attr(`delete_OpeningByReconstructionImageFilter`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter')
class(`delete_OpeningByReconstructionImageFilter`) = c("SWIGFunction", class('delete_OpeningByReconstructionImageFilter'))

# Start of new_OpeningByReconstructionImageFilter

`OpeningByReconstructionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_OpeningByReconstructionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__OpeningByReconstructionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_OpeningByReconstructionImageFilter);
  ans
  
}

attr(`OpeningByReconstructionImageFilter`, 'returnType') = '_p_itk__simple__OpeningByReconstructionImageFilter'
class(`OpeningByReconstructionImageFilter`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter'))

# Start of OpeningByReconstructionImageFilter_SetKernelRadius

`OpeningByReconstructionImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_OpeningByReconstructionImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OpeningByReconstructionImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`OpeningByReconstructionImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter', 'integer')
class(`OpeningByReconstructionImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_SetKernelRadius__SWIG_0'))

# Start of OpeningByReconstructionImageFilter_SetKernelRadius

`OpeningByReconstructionImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_OpeningByReconstructionImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OpeningByReconstructionImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`OpeningByReconstructionImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter', 'integer')
class(`OpeningByReconstructionImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_SetKernelRadius__SWIG_1'))

`OpeningByReconstructionImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__OpeningByReconstructionImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- OpeningByReconstructionImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__OpeningByReconstructionImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- OpeningByReconstructionImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for OpeningByReconstructionImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of OpeningByReconstructionImageFilter_GetKernelRadius

`OpeningByReconstructionImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OpeningByReconstructionImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OpeningByReconstructionImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`OpeningByReconstructionImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter')
class(`OpeningByReconstructionImageFilter_GetKernelRadius`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_GetKernelRadius'))

# Start of OpeningByReconstructionImageFilter_SetKernelType

`OpeningByReconstructionImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_OpeningByReconstructionImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OpeningByReconstructionImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`OpeningByReconstructionImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter', 'character')
class(`OpeningByReconstructionImageFilter_SetKernelType`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_SetKernelType'))

# Start of OpeningByReconstructionImageFilter_GetKernelType

`OpeningByReconstructionImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_OpeningByReconstructionImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`OpeningByReconstructionImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`OpeningByReconstructionImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter')
class(`OpeningByReconstructionImageFilter_GetKernelType`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_GetKernelType'))

# Start of OpeningByReconstructionImageFilter_SetFullyConnected

`OpeningByReconstructionImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_OpeningByReconstructionImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OpeningByReconstructionImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`OpeningByReconstructionImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter', 'logical')
class(`OpeningByReconstructionImageFilter_SetFullyConnected`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_SetFullyConnected'))

# Start of OpeningByReconstructionImageFilter_FullyConnectedOn

`OpeningByReconstructionImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_OpeningByReconstructionImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OpeningByReconstructionImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`OpeningByReconstructionImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter')
class(`OpeningByReconstructionImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_FullyConnectedOn'))

# Start of OpeningByReconstructionImageFilter_FullyConnectedOff

`OpeningByReconstructionImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_OpeningByReconstructionImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OpeningByReconstructionImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`OpeningByReconstructionImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter')
class(`OpeningByReconstructionImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_FullyConnectedOff'))

# Start of OpeningByReconstructionImageFilter_GetFullyConnected

`OpeningByReconstructionImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OpeningByReconstructionImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OpeningByReconstructionImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`OpeningByReconstructionImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter')
class(`OpeningByReconstructionImageFilter_GetFullyConnected`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_GetFullyConnected'))

# Start of OpeningByReconstructionImageFilter_SetPreserveIntensities

`OpeningByReconstructionImageFilter_SetPreserveIntensities` = function(self, PreserveIntensities)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  PreserveIntensities = as.logical(PreserveIntensities);
  ;ans = .Call('R_swig_OpeningByReconstructionImageFilter_SetPreserveIntensities', self, PreserveIntensities, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OpeningByReconstructionImageFilter_SetPreserveIntensities`, 'returnType') = 'void'
attr(`OpeningByReconstructionImageFilter_SetPreserveIntensities`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter', 'logical')
class(`OpeningByReconstructionImageFilter_SetPreserveIntensities`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_SetPreserveIntensities'))

# Start of OpeningByReconstructionImageFilter_PreserveIntensitiesOn

`OpeningByReconstructionImageFilter_PreserveIntensitiesOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_OpeningByReconstructionImageFilter_PreserveIntensitiesOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OpeningByReconstructionImageFilter_PreserveIntensitiesOn`, 'returnType') = 'void'
attr(`OpeningByReconstructionImageFilter_PreserveIntensitiesOn`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter')
class(`OpeningByReconstructionImageFilter_PreserveIntensitiesOn`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_PreserveIntensitiesOn'))

# Start of OpeningByReconstructionImageFilter_PreserveIntensitiesOff

`OpeningByReconstructionImageFilter_PreserveIntensitiesOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_OpeningByReconstructionImageFilter_PreserveIntensitiesOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OpeningByReconstructionImageFilter_PreserveIntensitiesOff`, 'returnType') = 'void'
attr(`OpeningByReconstructionImageFilter_PreserveIntensitiesOff`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter')
class(`OpeningByReconstructionImageFilter_PreserveIntensitiesOff`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_PreserveIntensitiesOff'))

# Start of OpeningByReconstructionImageFilter_GetPreserveIntensities

`OpeningByReconstructionImageFilter_GetPreserveIntensities` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OpeningByReconstructionImageFilter_GetPreserveIntensities', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OpeningByReconstructionImageFilter_GetPreserveIntensities`, 'returnType') = 'logical'
attr(`OpeningByReconstructionImageFilter_GetPreserveIntensities`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter')
class(`OpeningByReconstructionImageFilter_GetPreserveIntensities`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_GetPreserveIntensities'))

# Start of OpeningByReconstructionImageFilter_GetName

`OpeningByReconstructionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OpeningByReconstructionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OpeningByReconstructionImageFilter_GetName`, 'returnType') = 'character'
attr(`OpeningByReconstructionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter')
class(`OpeningByReconstructionImageFilter_GetName`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_GetName'))

# Start of OpeningByReconstructionImageFilter_ToString

`OpeningByReconstructionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OpeningByReconstructionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OpeningByReconstructionImageFilter_ToString`, 'returnType') = 'character'
attr(`OpeningByReconstructionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter')
class(`OpeningByReconstructionImageFilter_ToString`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_ToString'))

# Start of OpeningByReconstructionImageFilter_Execute

`OpeningByReconstructionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_OpeningByReconstructionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OpeningByReconstructionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`OpeningByReconstructionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__OpeningByReconstructionImageFilter', '_p_itk__simple__Image')
class(`OpeningByReconstructionImageFilter_Execute`) = c("SWIGFunction", class('OpeningByReconstructionImageFilter_Execute'))

# Start of accessor method for itk::simple::OpeningByReconstructionImageFilter
setMethod('$', '_p_itk__simple__OpeningByReconstructionImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = OpeningByReconstructionImageFilter_SetKernelRadius, 'GetKernelRadius' = OpeningByReconstructionImageFilter_GetKernelRadius, 'SetKernelType' = OpeningByReconstructionImageFilter_SetKernelType, 'GetKernelType' = OpeningByReconstructionImageFilter_GetKernelType, 'SetFullyConnected' = OpeningByReconstructionImageFilter_SetFullyConnected, 'FullyConnectedOn' = OpeningByReconstructionImageFilter_FullyConnectedOn, 'FullyConnectedOff' = OpeningByReconstructionImageFilter_FullyConnectedOff, 'GetFullyConnected' = OpeningByReconstructionImageFilter_GetFullyConnected, 'SetPreserveIntensities' = OpeningByReconstructionImageFilter_SetPreserveIntensities, 'PreserveIntensitiesOn' = OpeningByReconstructionImageFilter_PreserveIntensitiesOn, 'PreserveIntensitiesOff' = OpeningByReconstructionImageFilter_PreserveIntensitiesOff, 'GetPreserveIntensities' = OpeningByReconstructionImageFilter_GetPreserveIntensities, 'GetName' = OpeningByReconstructionImageFilter_GetName, 'ToString' = OpeningByReconstructionImageFilter_ToString, 'Execute' = OpeningByReconstructionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::OpeningByReconstructionImageFilter
# Start of OpeningByReconstruction

`OpeningByReconstruction__SWIG_0` = function(image1, kernelRadius, kernelType, fullyConnected, preserveIntensities, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  fullyConnected = as.logical(fullyConnected);
  preserveIntensities = as.logical(preserveIntensities);
  ;ans = .Call('R_swig_OpeningByReconstruction__SWIG_0', image1, kernelRadius, kernelType, fullyConnected, preserveIntensities, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OpeningByReconstruction__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`OpeningByReconstruction__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'logical', 'logical')
class(`OpeningByReconstruction__SWIG_0`) = c("SWIGFunction", class('OpeningByReconstruction__SWIG_0'))

# Start of OpeningByReconstruction

`OpeningByReconstruction__SWIG_1` = function(image1, kernelRadius, kernelType, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_OpeningByReconstruction__SWIG_1', image1, kernelRadius, kernelType, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OpeningByReconstruction__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`OpeningByReconstruction__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'logical')
class(`OpeningByReconstruction__SWIG_1`) = c("SWIGFunction", class('OpeningByReconstruction__SWIG_1'))

# Start of OpeningByReconstruction

`OpeningByReconstruction__SWIG_2` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_OpeningByReconstruction__SWIG_2', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OpeningByReconstruction__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`OpeningByReconstruction__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`OpeningByReconstruction__SWIG_2`) = c("SWIGFunction", class('OpeningByReconstruction__SWIG_2'))

# Start of OpeningByReconstruction

`OpeningByReconstruction__SWIG_3` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_OpeningByReconstruction__SWIG_3', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OpeningByReconstruction__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`OpeningByReconstruction__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`OpeningByReconstruction__SWIG_3`) = c("SWIGFunction", class('OpeningByReconstruction__SWIG_3'))

# Start of OpeningByReconstruction

`OpeningByReconstruction__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_OpeningByReconstruction__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OpeningByReconstruction__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`OpeningByReconstruction__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`OpeningByReconstruction__SWIG_4`) = c("SWIGFunction", class('OpeningByReconstruction__SWIG_4'))

`OpeningByReconstruction` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- OpeningByReconstruction__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- OpeningByReconstruction__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- OpeningByReconstruction__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- OpeningByReconstruction__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- OpeningByReconstruction__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for OpeningByReconstruction with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_OrImageFilter

`delete_OrImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_OrImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_OrImageFilter`, 'returnType') = 'void'
attr(`delete_OrImageFilter`, "inputTypes") = c('_p_itk__simple__OrImageFilter')
class(`delete_OrImageFilter`) = c("SWIGFunction", class('delete_OrImageFilter'))

# Start of new_OrImageFilter

`OrImageFilter` = function()
{
  ;ans = .Call('R_swig_new_OrImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__OrImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_OrImageFilter);
  ans
  
}

attr(`OrImageFilter`, 'returnType') = '_p_itk__simple__OrImageFilter'
class(`OrImageFilter`) = c("SWIGFunction", class('OrImageFilter'))

# Start of OrImageFilter_GetName

`OrImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OrImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OrImageFilter_GetName`, 'returnType') = 'character'
attr(`OrImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__OrImageFilter')
class(`OrImageFilter_GetName`) = c("SWIGFunction", class('OrImageFilter_GetName'))

# Start of OrImageFilter_ToString

`OrImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OrImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OrImageFilter_ToString`, 'returnType') = 'character'
attr(`OrImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__OrImageFilter')
class(`OrImageFilter_ToString`) = c("SWIGFunction", class('OrImageFilter_ToString'))

# Start of OrImageFilter_Execute

`OrImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_OrImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OrImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`OrImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__OrImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`OrImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('OrImageFilter_Execute__SWIG_0'))

# Start of OrImageFilter_Execute

`OrImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  ;ans = .Call('R_swig_OrImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OrImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`OrImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__OrImageFilter', '_p_itk__simple__Image', 'integer')
class(`OrImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('OrImageFilter_Execute__SWIG_1'))

# Start of OrImageFilter_Execute

`OrImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_OrImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OrImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`OrImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__OrImageFilter', 'integer', '_p_itk__simple__Image')
class(`OrImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('OrImageFilter_Execute__SWIG_2'))

`OrImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__OrImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- OrImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__OrImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- OrImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__OrImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- OrImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for OrImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::OrImageFilter
setMethod('$', '_p_itk__simple__OrImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = OrImageFilter_GetName, 'ToString' = OrImageFilter_ToString, 'Execute' = OrImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::OrImageFilter
# Start of Or

`Or__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Or__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Or__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Or__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Or__SWIG_0`) = c("SWIGFunction", class('Or__SWIG_0'))

# Start of Or

`Or__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  ;ans = .Call('R_swig_Or__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Or__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Or__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Or__SWIG_1`) = c("SWIGFunction", class('Or__SWIG_1'))

# Start of Or

`Or__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Or__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Or__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Or__SWIG_2`, "inputTypes") = c('integer', '_p_itk__simple__Image')
class(`Or__SWIG_2`) = c("SWIGFunction", class('Or__SWIG_2'))

`Or` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Or__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- Or__SWIG_1; 
    }
    else if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Or__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Or with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_OtsuMultipleThresholdsImageFilter

`delete_OtsuMultipleThresholdsImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_OtsuMultipleThresholdsImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_OtsuMultipleThresholdsImageFilter`, 'returnType') = 'void'
attr(`delete_OtsuMultipleThresholdsImageFilter`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter')
class(`delete_OtsuMultipleThresholdsImageFilter`) = c("SWIGFunction", class('delete_OtsuMultipleThresholdsImageFilter'))

# Start of new_OtsuMultipleThresholdsImageFilter

`OtsuMultipleThresholdsImageFilter` = function()
{
  ;ans = .Call('R_swig_new_OtsuMultipleThresholdsImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__OtsuMultipleThresholdsImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_OtsuMultipleThresholdsImageFilter);
  ans
  
}

attr(`OtsuMultipleThresholdsImageFilter`, 'returnType') = '_p_itk__simple__OtsuMultipleThresholdsImageFilter'
class(`OtsuMultipleThresholdsImageFilter`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter'))

# Start of OtsuMultipleThresholdsImageFilter_SetNumberOfThresholds

`OtsuMultipleThresholdsImageFilter_SetNumberOfThresholds` = function(self, NumberOfThresholds)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfThresholds = as.integer(NumberOfThresholds);
  
  if(length(NumberOfThresholds) > 1) {
    warning("using only the first element of NumberOfThresholds");
  };
  
  ;ans = .Call('R_swig_OtsuMultipleThresholdsImageFilter_SetNumberOfThresholds', self, NumberOfThresholds, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholdsImageFilter_SetNumberOfThresholds`, 'returnType') = 'void'
attr(`OtsuMultipleThresholdsImageFilter_SetNumberOfThresholds`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter', 'integer')
class(`OtsuMultipleThresholdsImageFilter_SetNumberOfThresholds`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_SetNumberOfThresholds'))

# Start of OtsuMultipleThresholdsImageFilter_GetNumberOfThresholds

`OtsuMultipleThresholdsImageFilter_GetNumberOfThresholds` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuMultipleThresholdsImageFilter_GetNumberOfThresholds', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuMultipleThresholdsImageFilter_GetNumberOfThresholds`, 'returnType') = 'integer'
attr(`OtsuMultipleThresholdsImageFilter_GetNumberOfThresholds`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter')
class(`OtsuMultipleThresholdsImageFilter_GetNumberOfThresholds`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_GetNumberOfThresholds'))

# Start of OtsuMultipleThresholdsImageFilter_SetLabelOffset

`OtsuMultipleThresholdsImageFilter_SetLabelOffset` = function(self, LabelOffset)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  LabelOffset = as.integer(LabelOffset);
  
  if(length(LabelOffset) > 1) {
    warning("using only the first element of LabelOffset");
  };
  
  ;ans = .Call('R_swig_OtsuMultipleThresholdsImageFilter_SetLabelOffset', self, LabelOffset, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholdsImageFilter_SetLabelOffset`, 'returnType') = 'void'
attr(`OtsuMultipleThresholdsImageFilter_SetLabelOffset`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter', 'integer')
class(`OtsuMultipleThresholdsImageFilter_SetLabelOffset`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_SetLabelOffset'))

# Start of OtsuMultipleThresholdsImageFilter_GetLabelOffset

`OtsuMultipleThresholdsImageFilter_GetLabelOffset` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuMultipleThresholdsImageFilter_GetLabelOffset', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuMultipleThresholdsImageFilter_GetLabelOffset`, 'returnType') = 'integer'
attr(`OtsuMultipleThresholdsImageFilter_GetLabelOffset`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter')
class(`OtsuMultipleThresholdsImageFilter_GetLabelOffset`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_GetLabelOffset'))

# Start of OtsuMultipleThresholdsImageFilter_SetNumberOfHistogramBins

`OtsuMultipleThresholdsImageFilter_SetNumberOfHistogramBins` = function(self, NumberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfHistogramBins = as.integer(NumberOfHistogramBins);
  
  if(length(NumberOfHistogramBins) > 1) {
    warning("using only the first element of NumberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_OtsuMultipleThresholdsImageFilter_SetNumberOfHistogramBins', self, NumberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholdsImageFilter_SetNumberOfHistogramBins`, 'returnType') = 'void'
attr(`OtsuMultipleThresholdsImageFilter_SetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter', 'integer')
class(`OtsuMultipleThresholdsImageFilter_SetNumberOfHistogramBins`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_SetNumberOfHistogramBins'))

# Start of OtsuMultipleThresholdsImageFilter_GetNumberOfHistogramBins

`OtsuMultipleThresholdsImageFilter_GetNumberOfHistogramBins` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuMultipleThresholdsImageFilter_GetNumberOfHistogramBins', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuMultipleThresholdsImageFilter_GetNumberOfHistogramBins`, 'returnType') = 'integer'
attr(`OtsuMultipleThresholdsImageFilter_GetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter')
class(`OtsuMultipleThresholdsImageFilter_GetNumberOfHistogramBins`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_GetNumberOfHistogramBins'))

# Start of OtsuMultipleThresholdsImageFilter_SetValleyEmphasis

`OtsuMultipleThresholdsImageFilter_SetValleyEmphasis` = function(self, ValleyEmphasis)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ValleyEmphasis = as.logical(ValleyEmphasis);
  ;ans = .Call('R_swig_OtsuMultipleThresholdsImageFilter_SetValleyEmphasis', self, ValleyEmphasis, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholdsImageFilter_SetValleyEmphasis`, 'returnType') = 'void'
attr(`OtsuMultipleThresholdsImageFilter_SetValleyEmphasis`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter', 'logical')
class(`OtsuMultipleThresholdsImageFilter_SetValleyEmphasis`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_SetValleyEmphasis'))

# Start of OtsuMultipleThresholdsImageFilter_ValleyEmphasisOn

`OtsuMultipleThresholdsImageFilter_ValleyEmphasisOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_OtsuMultipleThresholdsImageFilter_ValleyEmphasisOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholdsImageFilter_ValleyEmphasisOn`, 'returnType') = 'void'
attr(`OtsuMultipleThresholdsImageFilter_ValleyEmphasisOn`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter')
class(`OtsuMultipleThresholdsImageFilter_ValleyEmphasisOn`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_ValleyEmphasisOn'))

# Start of OtsuMultipleThresholdsImageFilter_ValleyEmphasisOff

`OtsuMultipleThresholdsImageFilter_ValleyEmphasisOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_OtsuMultipleThresholdsImageFilter_ValleyEmphasisOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholdsImageFilter_ValleyEmphasisOff`, 'returnType') = 'void'
attr(`OtsuMultipleThresholdsImageFilter_ValleyEmphasisOff`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter')
class(`OtsuMultipleThresholdsImageFilter_ValleyEmphasisOff`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_ValleyEmphasisOff'))

# Start of OtsuMultipleThresholdsImageFilter_GetValleyEmphasis

`OtsuMultipleThresholdsImageFilter_GetValleyEmphasis` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuMultipleThresholdsImageFilter_GetValleyEmphasis', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuMultipleThresholdsImageFilter_GetValleyEmphasis`, 'returnType') = 'logical'
attr(`OtsuMultipleThresholdsImageFilter_GetValleyEmphasis`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter')
class(`OtsuMultipleThresholdsImageFilter_GetValleyEmphasis`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_GetValleyEmphasis'))

# Start of OtsuMultipleThresholdsImageFilter_SetReturnBinMidpoint

`OtsuMultipleThresholdsImageFilter_SetReturnBinMidpoint` = function(self, ReturnBinMidpoint)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ReturnBinMidpoint = as.logical(ReturnBinMidpoint);
  ;ans = .Call('R_swig_OtsuMultipleThresholdsImageFilter_SetReturnBinMidpoint', self, ReturnBinMidpoint, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholdsImageFilter_SetReturnBinMidpoint`, 'returnType') = 'void'
attr(`OtsuMultipleThresholdsImageFilter_SetReturnBinMidpoint`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter', 'logical')
class(`OtsuMultipleThresholdsImageFilter_SetReturnBinMidpoint`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_SetReturnBinMidpoint'))

# Start of OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOn

`OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOn`, 'returnType') = 'void'
attr(`OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOn`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter')
class(`OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOn`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOn'))

# Start of OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOff

`OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOff`, 'returnType') = 'void'
attr(`OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOff`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter')
class(`OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOff`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOff'))

# Start of OtsuMultipleThresholdsImageFilter_GetReturnBinMidpoint

`OtsuMultipleThresholdsImageFilter_GetReturnBinMidpoint` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuMultipleThresholdsImageFilter_GetReturnBinMidpoint', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuMultipleThresholdsImageFilter_GetReturnBinMidpoint`, 'returnType') = 'logical'
attr(`OtsuMultipleThresholdsImageFilter_GetReturnBinMidpoint`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter')
class(`OtsuMultipleThresholdsImageFilter_GetReturnBinMidpoint`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_GetReturnBinMidpoint'))

# Start of OtsuMultipleThresholdsImageFilter_GetThresholds

`OtsuMultipleThresholdsImageFilter_GetThresholds` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuMultipleThresholdsImageFilter_GetThresholds', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuMultipleThresholdsImageFilter_GetThresholds`, 'returnType') = 'numeric'
attr(`OtsuMultipleThresholdsImageFilter_GetThresholds`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter')
class(`OtsuMultipleThresholdsImageFilter_GetThresholds`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_GetThresholds'))

# Start of OtsuMultipleThresholdsImageFilter_GetName

`OtsuMultipleThresholdsImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuMultipleThresholdsImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuMultipleThresholdsImageFilter_GetName`, 'returnType') = 'character'
attr(`OtsuMultipleThresholdsImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter')
class(`OtsuMultipleThresholdsImageFilter_GetName`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_GetName'))

# Start of OtsuMultipleThresholdsImageFilter_ToString

`OtsuMultipleThresholdsImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuMultipleThresholdsImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuMultipleThresholdsImageFilter_ToString`, 'returnType') = 'character'
attr(`OtsuMultipleThresholdsImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter')
class(`OtsuMultipleThresholdsImageFilter_ToString`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_ToString'))

# Start of OtsuMultipleThresholdsImageFilter_Execute

`OtsuMultipleThresholdsImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_OtsuMultipleThresholdsImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholdsImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuMultipleThresholdsImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__OtsuMultipleThresholdsImageFilter', '_p_itk__simple__Image')
class(`OtsuMultipleThresholdsImageFilter_Execute`) = c("SWIGFunction", class('OtsuMultipleThresholdsImageFilter_Execute'))

# Start of accessor method for itk::simple::OtsuMultipleThresholdsImageFilter
setMethod('$', '_p_itk__simple__OtsuMultipleThresholdsImageFilter', function(x, name)

{
  accessorFuns = list('SetNumberOfThresholds' = OtsuMultipleThresholdsImageFilter_SetNumberOfThresholds, 'GetNumberOfThresholds' = OtsuMultipleThresholdsImageFilter_GetNumberOfThresholds, 'SetLabelOffset' = OtsuMultipleThresholdsImageFilter_SetLabelOffset, 'GetLabelOffset' = OtsuMultipleThresholdsImageFilter_GetLabelOffset, 'SetNumberOfHistogramBins' = OtsuMultipleThresholdsImageFilter_SetNumberOfHistogramBins, 'GetNumberOfHistogramBins' = OtsuMultipleThresholdsImageFilter_GetNumberOfHistogramBins, 'SetValleyEmphasis' = OtsuMultipleThresholdsImageFilter_SetValleyEmphasis, 'ValleyEmphasisOn' = OtsuMultipleThresholdsImageFilter_ValleyEmphasisOn, 'ValleyEmphasisOff' = OtsuMultipleThresholdsImageFilter_ValleyEmphasisOff, 'GetValleyEmphasis' = OtsuMultipleThresholdsImageFilter_GetValleyEmphasis, 'SetReturnBinMidpoint' = OtsuMultipleThresholdsImageFilter_SetReturnBinMidpoint, 'ReturnBinMidpointOn' = OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOn, 'ReturnBinMidpointOff' = OtsuMultipleThresholdsImageFilter_ReturnBinMidpointOff, 'GetReturnBinMidpoint' = OtsuMultipleThresholdsImageFilter_GetReturnBinMidpoint, 'GetThresholds' = OtsuMultipleThresholdsImageFilter_GetThresholds, 'GetName' = OtsuMultipleThresholdsImageFilter_GetName, 'ToString' = OtsuMultipleThresholdsImageFilter_ToString, 'Execute' = OtsuMultipleThresholdsImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::OtsuMultipleThresholdsImageFilter
# Start of OtsuMultipleThresholds

`OtsuMultipleThresholds__SWIG_0` = function(image1, numberOfThresholds, labelOffset, numberOfHistogramBins, valleyEmphasis, returnBinMidpoint, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  numberOfThresholds = as.integer(numberOfThresholds);
  
  if(length(numberOfThresholds) > 1) {
    warning("using only the first element of numberOfThresholds");
  };
  
  labelOffset = as.integer(labelOffset);
  
  if(length(labelOffset) > 1) {
    warning("using only the first element of labelOffset");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  valleyEmphasis = as.logical(valleyEmphasis);
  returnBinMidpoint = as.logical(returnBinMidpoint);
  ;ans = .Call('R_swig_OtsuMultipleThresholds__SWIG_0', image1, numberOfThresholds, labelOffset, numberOfHistogramBins, valleyEmphasis, returnBinMidpoint, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholds__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuMultipleThresholds__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'logical')
class(`OtsuMultipleThresholds__SWIG_0`) = c("SWIGFunction", class('OtsuMultipleThresholds__SWIG_0'))

# Start of OtsuMultipleThresholds

`OtsuMultipleThresholds__SWIG_1` = function(image1, numberOfThresholds, labelOffset, numberOfHistogramBins, valleyEmphasis, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  numberOfThresholds = as.integer(numberOfThresholds);
  
  if(length(numberOfThresholds) > 1) {
    warning("using only the first element of numberOfThresholds");
  };
  
  labelOffset = as.integer(labelOffset);
  
  if(length(labelOffset) > 1) {
    warning("using only the first element of labelOffset");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  valleyEmphasis = as.logical(valleyEmphasis);
  ;ans = .Call('R_swig_OtsuMultipleThresholds__SWIG_1', image1, numberOfThresholds, labelOffset, numberOfHistogramBins, valleyEmphasis, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholds__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuMultipleThresholds__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`OtsuMultipleThresholds__SWIG_1`) = c("SWIGFunction", class('OtsuMultipleThresholds__SWIG_1'))

# Start of OtsuMultipleThresholds

`OtsuMultipleThresholds__SWIG_2` = function(image1, numberOfThresholds, labelOffset, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  numberOfThresholds = as.integer(numberOfThresholds);
  
  if(length(numberOfThresholds) > 1) {
    warning("using only the first element of numberOfThresholds");
  };
  
  labelOffset = as.integer(labelOffset);
  
  if(length(labelOffset) > 1) {
    warning("using only the first element of labelOffset");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_OtsuMultipleThresholds__SWIG_2', image1, numberOfThresholds, labelOffset, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholds__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuMultipleThresholds__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`OtsuMultipleThresholds__SWIG_2`) = c("SWIGFunction", class('OtsuMultipleThresholds__SWIG_2'))

# Start of OtsuMultipleThresholds

`OtsuMultipleThresholds__SWIG_3` = function(image1, numberOfThresholds, labelOffset, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  numberOfThresholds = as.integer(numberOfThresholds);
  
  if(length(numberOfThresholds) > 1) {
    warning("using only the first element of numberOfThresholds");
  };
  
  labelOffset = as.integer(labelOffset);
  
  if(length(labelOffset) > 1) {
    warning("using only the first element of labelOffset");
  };
  
  ;ans = .Call('R_swig_OtsuMultipleThresholds__SWIG_3', image1, numberOfThresholds, labelOffset, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholds__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuMultipleThresholds__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`OtsuMultipleThresholds__SWIG_3`) = c("SWIGFunction", class('OtsuMultipleThresholds__SWIG_3'))

# Start of OtsuMultipleThresholds

`OtsuMultipleThresholds__SWIG_4` = function(image1, numberOfThresholds, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  numberOfThresholds = as.integer(numberOfThresholds);
  
  if(length(numberOfThresholds) > 1) {
    warning("using only the first element of numberOfThresholds");
  };
  
  ;ans = .Call('R_swig_OtsuMultipleThresholds__SWIG_4', image1, numberOfThresholds, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholds__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuMultipleThresholds__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`OtsuMultipleThresholds__SWIG_4`) = c("SWIGFunction", class('OtsuMultipleThresholds__SWIG_4'))

# Start of OtsuMultipleThresholds

`OtsuMultipleThresholds__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_OtsuMultipleThresholds__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuMultipleThresholds__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuMultipleThresholds__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`OtsuMultipleThresholds__SWIG_5`) = c("SWIGFunction", class('OtsuMultipleThresholds__SWIG_5'))

`OtsuMultipleThresholds` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- OtsuMultipleThresholds__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- OtsuMultipleThresholds__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- OtsuMultipleThresholds__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- OtsuMultipleThresholds__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- OtsuMultipleThresholds__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- OtsuMultipleThresholds__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for OtsuMultipleThresholds with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_OtsuThresholdImageFilter

`delete_OtsuThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_OtsuThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_OtsuThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_OtsuThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter')
class(`delete_OtsuThresholdImageFilter`) = c("SWIGFunction", class('delete_OtsuThresholdImageFilter'))

# Start of new_OtsuThresholdImageFilter

`OtsuThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_OtsuThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__OtsuThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_OtsuThresholdImageFilter);
  ans
  
}

attr(`OtsuThresholdImageFilter`, 'returnType') = '_p_itk__simple__OtsuThresholdImageFilter'
class(`OtsuThresholdImageFilter`) = c("SWIGFunction", class('OtsuThresholdImageFilter'))

# Start of OtsuThresholdImageFilter_SetInsideValue

`OtsuThresholdImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideValue = as.integer(InsideValue);
  
  if(length(InsideValue) > 1) {
    warning("using only the first element of InsideValue");
  };
  
  ;ans = .Call('R_swig_OtsuThresholdImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuThresholdImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`OtsuThresholdImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter', 'integer')
class(`OtsuThresholdImageFilter_SetInsideValue`) = c("SWIGFunction", class('OtsuThresholdImageFilter_SetInsideValue'))

# Start of OtsuThresholdImageFilter_GetInsideValue

`OtsuThresholdImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuThresholdImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuThresholdImageFilter_GetInsideValue`, 'returnType') = 'integer'
attr(`OtsuThresholdImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter')
class(`OtsuThresholdImageFilter_GetInsideValue`) = c("SWIGFunction", class('OtsuThresholdImageFilter_GetInsideValue'))

# Start of OtsuThresholdImageFilter_SetOutsideValue

`OtsuThresholdImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutsideValue = as.integer(OutsideValue);
  
  if(length(OutsideValue) > 1) {
    warning("using only the first element of OutsideValue");
  };
  
  ;ans = .Call('R_swig_OtsuThresholdImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuThresholdImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`OtsuThresholdImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter', 'integer')
class(`OtsuThresholdImageFilter_SetOutsideValue`) = c("SWIGFunction", class('OtsuThresholdImageFilter_SetOutsideValue'))

# Start of OtsuThresholdImageFilter_GetOutsideValue

`OtsuThresholdImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuThresholdImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuThresholdImageFilter_GetOutsideValue`, 'returnType') = 'integer'
attr(`OtsuThresholdImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter')
class(`OtsuThresholdImageFilter_GetOutsideValue`) = c("SWIGFunction", class('OtsuThresholdImageFilter_GetOutsideValue'))

# Start of OtsuThresholdImageFilter_SetNumberOfHistogramBins

`OtsuThresholdImageFilter_SetNumberOfHistogramBins` = function(self, NumberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfHistogramBins = as.integer(NumberOfHistogramBins);
  
  if(length(NumberOfHistogramBins) > 1) {
    warning("using only the first element of NumberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_OtsuThresholdImageFilter_SetNumberOfHistogramBins', self, NumberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuThresholdImageFilter_SetNumberOfHistogramBins`, 'returnType') = 'void'
attr(`OtsuThresholdImageFilter_SetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter', 'integer')
class(`OtsuThresholdImageFilter_SetNumberOfHistogramBins`) = c("SWIGFunction", class('OtsuThresholdImageFilter_SetNumberOfHistogramBins'))

# Start of OtsuThresholdImageFilter_GetNumberOfHistogramBins

`OtsuThresholdImageFilter_GetNumberOfHistogramBins` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuThresholdImageFilter_GetNumberOfHistogramBins', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuThresholdImageFilter_GetNumberOfHistogramBins`, 'returnType') = 'integer'
attr(`OtsuThresholdImageFilter_GetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter')
class(`OtsuThresholdImageFilter_GetNumberOfHistogramBins`) = c("SWIGFunction", class('OtsuThresholdImageFilter_GetNumberOfHistogramBins'))

# Start of OtsuThresholdImageFilter_SetMaskOutput

`OtsuThresholdImageFilter_SetMaskOutput` = function(self, MaskOutput)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskOutput = as.logical(MaskOutput);
  ;ans = .Call('R_swig_OtsuThresholdImageFilter_SetMaskOutput', self, MaskOutput, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuThresholdImageFilter_SetMaskOutput`, 'returnType') = 'void'
attr(`OtsuThresholdImageFilter_SetMaskOutput`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter', 'logical')
class(`OtsuThresholdImageFilter_SetMaskOutput`) = c("SWIGFunction", class('OtsuThresholdImageFilter_SetMaskOutput'))

# Start of OtsuThresholdImageFilter_MaskOutputOn

`OtsuThresholdImageFilter_MaskOutputOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_OtsuThresholdImageFilter_MaskOutputOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuThresholdImageFilter_MaskOutputOn`, 'returnType') = 'void'
attr(`OtsuThresholdImageFilter_MaskOutputOn`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter')
class(`OtsuThresholdImageFilter_MaskOutputOn`) = c("SWIGFunction", class('OtsuThresholdImageFilter_MaskOutputOn'))

# Start of OtsuThresholdImageFilter_MaskOutputOff

`OtsuThresholdImageFilter_MaskOutputOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_OtsuThresholdImageFilter_MaskOutputOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuThresholdImageFilter_MaskOutputOff`, 'returnType') = 'void'
attr(`OtsuThresholdImageFilter_MaskOutputOff`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter')
class(`OtsuThresholdImageFilter_MaskOutputOff`) = c("SWIGFunction", class('OtsuThresholdImageFilter_MaskOutputOff'))

# Start of OtsuThresholdImageFilter_GetMaskOutput

`OtsuThresholdImageFilter_GetMaskOutput` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuThresholdImageFilter_GetMaskOutput', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuThresholdImageFilter_GetMaskOutput`, 'returnType') = 'logical'
attr(`OtsuThresholdImageFilter_GetMaskOutput`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter')
class(`OtsuThresholdImageFilter_GetMaskOutput`) = c("SWIGFunction", class('OtsuThresholdImageFilter_GetMaskOutput'))

# Start of OtsuThresholdImageFilter_SetMaskValue

`OtsuThresholdImageFilter_SetMaskValue` = function(self, MaskValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskValue = as.integer(MaskValue);
  
  if(length(MaskValue) > 1) {
    warning("using only the first element of MaskValue");
  };
  
  ;ans = .Call('R_swig_OtsuThresholdImageFilter_SetMaskValue', self, MaskValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuThresholdImageFilter_SetMaskValue`, 'returnType') = 'void'
attr(`OtsuThresholdImageFilter_SetMaskValue`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter', 'integer')
class(`OtsuThresholdImageFilter_SetMaskValue`) = c("SWIGFunction", class('OtsuThresholdImageFilter_SetMaskValue'))

# Start of OtsuThresholdImageFilter_GetMaskValue

`OtsuThresholdImageFilter_GetMaskValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuThresholdImageFilter_GetMaskValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuThresholdImageFilter_GetMaskValue`, 'returnType') = 'integer'
attr(`OtsuThresholdImageFilter_GetMaskValue`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter')
class(`OtsuThresholdImageFilter_GetMaskValue`) = c("SWIGFunction", class('OtsuThresholdImageFilter_GetMaskValue'))

# Start of OtsuThresholdImageFilter_SetReturnBinMidpoint

`OtsuThresholdImageFilter_SetReturnBinMidpoint` = function(self, ReturnBinMidpoint)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ReturnBinMidpoint = as.logical(ReturnBinMidpoint);
  ;ans = .Call('R_swig_OtsuThresholdImageFilter_SetReturnBinMidpoint', self, ReturnBinMidpoint, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuThresholdImageFilter_SetReturnBinMidpoint`, 'returnType') = 'void'
attr(`OtsuThresholdImageFilter_SetReturnBinMidpoint`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter', 'logical')
class(`OtsuThresholdImageFilter_SetReturnBinMidpoint`) = c("SWIGFunction", class('OtsuThresholdImageFilter_SetReturnBinMidpoint'))

# Start of OtsuThresholdImageFilter_ReturnBinMidpointOn

`OtsuThresholdImageFilter_ReturnBinMidpointOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_OtsuThresholdImageFilter_ReturnBinMidpointOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuThresholdImageFilter_ReturnBinMidpointOn`, 'returnType') = 'void'
attr(`OtsuThresholdImageFilter_ReturnBinMidpointOn`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter')
class(`OtsuThresholdImageFilter_ReturnBinMidpointOn`) = c("SWIGFunction", class('OtsuThresholdImageFilter_ReturnBinMidpointOn'))

# Start of OtsuThresholdImageFilter_ReturnBinMidpointOff

`OtsuThresholdImageFilter_ReturnBinMidpointOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_OtsuThresholdImageFilter_ReturnBinMidpointOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`OtsuThresholdImageFilter_ReturnBinMidpointOff`, 'returnType') = 'void'
attr(`OtsuThresholdImageFilter_ReturnBinMidpointOff`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter')
class(`OtsuThresholdImageFilter_ReturnBinMidpointOff`) = c("SWIGFunction", class('OtsuThresholdImageFilter_ReturnBinMidpointOff'))

# Start of OtsuThresholdImageFilter_GetReturnBinMidpoint

`OtsuThresholdImageFilter_GetReturnBinMidpoint` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuThresholdImageFilter_GetReturnBinMidpoint', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuThresholdImageFilter_GetReturnBinMidpoint`, 'returnType') = 'logical'
attr(`OtsuThresholdImageFilter_GetReturnBinMidpoint`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter')
class(`OtsuThresholdImageFilter_GetReturnBinMidpoint`) = c("SWIGFunction", class('OtsuThresholdImageFilter_GetReturnBinMidpoint'))

# Start of OtsuThresholdImageFilter_GetThreshold

`OtsuThresholdImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuThresholdImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuThresholdImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`OtsuThresholdImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter')
class(`OtsuThresholdImageFilter_GetThreshold`) = c("SWIGFunction", class('OtsuThresholdImageFilter_GetThreshold'))

# Start of OtsuThresholdImageFilter_GetName

`OtsuThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`OtsuThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter')
class(`OtsuThresholdImageFilter_GetName`) = c("SWIGFunction", class('OtsuThresholdImageFilter_GetName'))

# Start of OtsuThresholdImageFilter_ToString

`OtsuThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_OtsuThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`OtsuThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`OtsuThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter')
class(`OtsuThresholdImageFilter_ToString`) = c("SWIGFunction", class('OtsuThresholdImageFilter_ToString'))

# Start of OtsuThresholdImageFilter_Execute

`OtsuThresholdImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_OtsuThresholdImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThresholdImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThresholdImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`OtsuThresholdImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('OtsuThresholdImageFilter_Execute__SWIG_0'))

# Start of OtsuThresholdImageFilter_Execute

`OtsuThresholdImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_OtsuThresholdImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThresholdImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThresholdImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__OtsuThresholdImageFilter', '_p_itk__simple__Image')
class(`OtsuThresholdImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('OtsuThresholdImageFilter_Execute__SWIG_1'))

`OtsuThresholdImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__OtsuThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- OtsuThresholdImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__OtsuThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- OtsuThresholdImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for OtsuThresholdImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::OtsuThresholdImageFilter
setMethod('$', '_p_itk__simple__OtsuThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetInsideValue' = OtsuThresholdImageFilter_SetInsideValue, 'GetInsideValue' = OtsuThresholdImageFilter_GetInsideValue, 'SetOutsideValue' = OtsuThresholdImageFilter_SetOutsideValue, 'GetOutsideValue' = OtsuThresholdImageFilter_GetOutsideValue, 'SetNumberOfHistogramBins' = OtsuThresholdImageFilter_SetNumberOfHistogramBins, 'GetNumberOfHistogramBins' = OtsuThresholdImageFilter_GetNumberOfHistogramBins, 'SetMaskOutput' = OtsuThresholdImageFilter_SetMaskOutput, 'MaskOutputOn' = OtsuThresholdImageFilter_MaskOutputOn, 'MaskOutputOff' = OtsuThresholdImageFilter_MaskOutputOff, 'GetMaskOutput' = OtsuThresholdImageFilter_GetMaskOutput, 'SetMaskValue' = OtsuThresholdImageFilter_SetMaskValue, 'GetMaskValue' = OtsuThresholdImageFilter_GetMaskValue, 'SetReturnBinMidpoint' = OtsuThresholdImageFilter_SetReturnBinMidpoint, 'ReturnBinMidpointOn' = OtsuThresholdImageFilter_ReturnBinMidpointOn, 'ReturnBinMidpointOff' = OtsuThresholdImageFilter_ReturnBinMidpointOff, 'GetReturnBinMidpoint' = OtsuThresholdImageFilter_GetReturnBinMidpoint, 'GetThreshold' = OtsuThresholdImageFilter_GetThreshold, 'GetName' = OtsuThresholdImageFilter_GetName, 'ToString' = OtsuThresholdImageFilter_ToString, 'Execute' = OtsuThresholdImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::OtsuThresholdImageFilter
# Start of OtsuThreshold

`OtsuThreshold__SWIG_0` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, returnBinMidpoint, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  returnBinMidpoint = as.logical(returnBinMidpoint);
  ;ans = .Call('R_swig_OtsuThreshold__SWIG_0', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, returnBinMidpoint, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThreshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThreshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer', 'logical')
class(`OtsuThreshold__SWIG_0`) = c("SWIGFunction", class('OtsuThreshold__SWIG_0'))

# Start of OtsuThreshold

`OtsuThreshold__SWIG_1` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_OtsuThreshold__SWIG_1', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThreshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThreshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`OtsuThreshold__SWIG_1`) = c("SWIGFunction", class('OtsuThreshold__SWIG_1'))

# Start of OtsuThreshold

`OtsuThreshold__SWIG_2` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_OtsuThreshold__SWIG_2', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThreshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThreshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`OtsuThreshold__SWIG_2`) = c("SWIGFunction", class('OtsuThreshold__SWIG_2'))

# Start of OtsuThreshold

`OtsuThreshold__SWIG_3` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_OtsuThreshold__SWIG_3', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThreshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThreshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`OtsuThreshold__SWIG_3`) = c("SWIGFunction", class('OtsuThreshold__SWIG_3'))

# Start of OtsuThreshold

`OtsuThreshold__SWIG_4` = function(image, maskImage, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_OtsuThreshold__SWIG_4', image, maskImage, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThreshold__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThreshold__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`OtsuThreshold__SWIG_4`) = c("SWIGFunction", class('OtsuThreshold__SWIG_4'))

# Start of OtsuThreshold

`OtsuThreshold__SWIG_5` = function(image, maskImage, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_OtsuThreshold__SWIG_5', image, maskImage, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThreshold__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThreshold__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`OtsuThreshold__SWIG_5`) = c("SWIGFunction", class('OtsuThreshold__SWIG_5'))

# Start of OtsuThreshold

`OtsuThreshold__SWIG_6` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_OtsuThreshold__SWIG_6', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThreshold__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThreshold__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`OtsuThreshold__SWIG_6`) = c("SWIGFunction", class('OtsuThreshold__SWIG_6'))

# Start of OtsuThreshold

`OtsuThreshold__SWIG_7` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, returnBinMidpoint, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  returnBinMidpoint = as.logical(returnBinMidpoint);
  ;ans = .Call('R_swig_OtsuThreshold__SWIG_7', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, returnBinMidpoint, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThreshold__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThreshold__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer', 'logical')
class(`OtsuThreshold__SWIG_7`) = c("SWIGFunction", class('OtsuThreshold__SWIG_7'))

# Start of OtsuThreshold

`OtsuThreshold__SWIG_8` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_OtsuThreshold__SWIG_8', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThreshold__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThreshold__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`OtsuThreshold__SWIG_8`) = c("SWIGFunction", class('OtsuThreshold__SWIG_8'))

# Start of OtsuThreshold

`OtsuThreshold__SWIG_9` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_OtsuThreshold__SWIG_9', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThreshold__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThreshold__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`OtsuThreshold__SWIG_9`) = c("SWIGFunction", class('OtsuThreshold__SWIG_9'))

# Start of OtsuThreshold

`OtsuThreshold__SWIG_10` = function(image, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_OtsuThreshold__SWIG_10', image, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThreshold__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThreshold__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`OtsuThreshold__SWIG_10`) = c("SWIGFunction", class('OtsuThreshold__SWIG_10'))

# Start of OtsuThreshold

`OtsuThreshold__SWIG_11` = function(image, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_OtsuThreshold__SWIG_11', image, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThreshold__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThreshold__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`OtsuThreshold__SWIG_11`) = c("SWIGFunction", class('OtsuThreshold__SWIG_11'))

# Start of OtsuThreshold

`OtsuThreshold__SWIG_12` = function(image, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_OtsuThreshold__SWIG_12', image, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThreshold__SWIG_12`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThreshold__SWIG_12`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`OtsuThreshold__SWIG_12`) = c("SWIGFunction", class('OtsuThreshold__SWIG_12'))

# Start of OtsuThreshold

`OtsuThreshold__SWIG_13` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_OtsuThreshold__SWIG_13', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`OtsuThreshold__SWIG_13`, 'returnType') = '_p_itk__simple__Image'
attr(`OtsuThreshold__SWIG_13`, "inputTypes") = c('_p_itk__simple__Image')
class(`OtsuThreshold__SWIG_13`) = c("SWIGFunction", class('OtsuThreshold__SWIG_13'))

`OtsuThreshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 14
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- OtsuThreshold__SWIG_13; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- OtsuThreshold__SWIG_6; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- OtsuThreshold__SWIG_12; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- OtsuThreshold__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- OtsuThreshold__SWIG_11; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- OtsuThreshold__SWIG_4; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- OtsuThreshold__SWIG_10; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- OtsuThreshold__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- OtsuThreshold__SWIG_9; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- OtsuThreshold__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- OtsuThreshold__SWIG_8; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- OtsuThreshold__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.logical(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- OtsuThreshold__SWIG_7; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 ) && ( is.logical(argv[[8]]) && length(argv[[8]]) == 1 )) {
      f <- OtsuThreshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for OtsuThreshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_PatchBasedDenoisingImageFilter

`delete_PatchBasedDenoisingImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_PatchBasedDenoisingImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_PatchBasedDenoisingImageFilter`, 'returnType') = 'void'
attr(`delete_PatchBasedDenoisingImageFilter`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`delete_PatchBasedDenoisingImageFilter`) = c("SWIGFunction", class('delete_PatchBasedDenoisingImageFilter'))

# Start of new_PatchBasedDenoisingImageFilter

`PatchBasedDenoisingImageFilter` = function()
{
  ;ans = .Call('R_swig_new_PatchBasedDenoisingImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__PatchBasedDenoisingImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_PatchBasedDenoisingImageFilter);
  ans
  
}

attr(`PatchBasedDenoisingImageFilter`, 'returnType') = '_p_itk__simple__PatchBasedDenoisingImageFilter'
class(`PatchBasedDenoisingImageFilter`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter'))

# Start of PatchBasedDenoisingImageFilter_SetKernelBandwidthSigma

`PatchBasedDenoisingImageFilter_SetKernelBandwidthSigma` = function(self, KernelBandwidthSigma)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_SetKernelBandwidthSigma', self, KernelBandwidthSigma, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_SetKernelBandwidthSigma`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_SetKernelBandwidthSigma`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter', 'numeric')
class(`PatchBasedDenoisingImageFilter_SetKernelBandwidthSigma`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_SetKernelBandwidthSigma'))

# Start of PatchBasedDenoisingImageFilter_GetKernelBandwidthSigma

`PatchBasedDenoisingImageFilter_GetKernelBandwidthSigma` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PatchBasedDenoisingImageFilter_GetKernelBandwidthSigma', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PatchBasedDenoisingImageFilter_GetKernelBandwidthSigma`, 'returnType') = 'numeric'
attr(`PatchBasedDenoisingImageFilter_GetKernelBandwidthSigma`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_GetKernelBandwidthSigma`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_GetKernelBandwidthSigma'))

# Start of PatchBasedDenoisingImageFilter_SetPatchRadius

`PatchBasedDenoisingImageFilter_SetPatchRadius` = function(self, PatchRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  PatchRadius = as.integer(PatchRadius);
  
  if(length(PatchRadius) > 1) {
    warning("using only the first element of PatchRadius");
  };
  
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_SetPatchRadius', self, PatchRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_SetPatchRadius`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_SetPatchRadius`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter', 'integer')
class(`PatchBasedDenoisingImageFilter_SetPatchRadius`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_SetPatchRadius'))

# Start of PatchBasedDenoisingImageFilter_GetPatchRadius

`PatchBasedDenoisingImageFilter_GetPatchRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PatchBasedDenoisingImageFilter_GetPatchRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PatchBasedDenoisingImageFilter_GetPatchRadius`, 'returnType') = 'integer'
attr(`PatchBasedDenoisingImageFilter_GetPatchRadius`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_GetPatchRadius`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_GetPatchRadius'))

# Start of PatchBasedDenoisingImageFilter_SetNumberOfIterations

`PatchBasedDenoisingImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter', 'integer')
class(`PatchBasedDenoisingImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_SetNumberOfIterations'))

# Start of PatchBasedDenoisingImageFilter_GetNumberOfIterations

`PatchBasedDenoisingImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PatchBasedDenoisingImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PatchBasedDenoisingImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`PatchBasedDenoisingImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_GetNumberOfIterations'))

# Start of PatchBasedDenoisingImageFilter_SetNumberOfSamplePatches

`PatchBasedDenoisingImageFilter_SetNumberOfSamplePatches` = function(self, NumberOfSamplePatches)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfSamplePatches = as.integer(NumberOfSamplePatches);
  
  if(length(NumberOfSamplePatches) > 1) {
    warning("using only the first element of NumberOfSamplePatches");
  };
  
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_SetNumberOfSamplePatches', self, NumberOfSamplePatches, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_SetNumberOfSamplePatches`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_SetNumberOfSamplePatches`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter', 'integer')
class(`PatchBasedDenoisingImageFilter_SetNumberOfSamplePatches`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_SetNumberOfSamplePatches'))

# Start of PatchBasedDenoisingImageFilter_GetNumberOfSamplePatches

`PatchBasedDenoisingImageFilter_GetNumberOfSamplePatches` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PatchBasedDenoisingImageFilter_GetNumberOfSamplePatches', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PatchBasedDenoisingImageFilter_GetNumberOfSamplePatches`, 'returnType') = 'integer'
attr(`PatchBasedDenoisingImageFilter_GetNumberOfSamplePatches`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_GetNumberOfSamplePatches`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_GetNumberOfSamplePatches'))

# Start of PatchBasedDenoisingImageFilter_SetSampleVariance

`PatchBasedDenoisingImageFilter_SetSampleVariance` = function(self, SampleVariance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_SetSampleVariance', self, SampleVariance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_SetSampleVariance`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_SetSampleVariance`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter', 'numeric')
class(`PatchBasedDenoisingImageFilter_SetSampleVariance`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_SetSampleVariance'))

# Start of PatchBasedDenoisingImageFilter_GetSampleVariance

`PatchBasedDenoisingImageFilter_GetSampleVariance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PatchBasedDenoisingImageFilter_GetSampleVariance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PatchBasedDenoisingImageFilter_GetSampleVariance`, 'returnType') = 'numeric'
attr(`PatchBasedDenoisingImageFilter_GetSampleVariance`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_GetSampleVariance`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_GetSampleVariance'))

# Start of PatchBasedDenoisingImageFilter_NOMODEL_get

`PatchBasedDenoisingImageFilter_NOMODEL_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_NOMODEL_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PatchBasedDenoisingImageFilter__NoiseModelType");
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_NOMODEL_get`, 'returnType') = 'character'
class(`PatchBasedDenoisingImageFilter_NOMODEL_get`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_NOMODEL_get'))

# Start of PatchBasedDenoisingImageFilter_GAUSSIAN_get

`PatchBasedDenoisingImageFilter_GAUSSIAN_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_GAUSSIAN_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PatchBasedDenoisingImageFilter__NoiseModelType");
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_GAUSSIAN_get`, 'returnType') = 'character'
class(`PatchBasedDenoisingImageFilter_GAUSSIAN_get`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_GAUSSIAN_get'))

# Start of PatchBasedDenoisingImageFilter_RICIAN_get

`PatchBasedDenoisingImageFilter_RICIAN_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_RICIAN_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PatchBasedDenoisingImageFilter__NoiseModelType");
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_RICIAN_get`, 'returnType') = 'character'
class(`PatchBasedDenoisingImageFilter_RICIAN_get`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_RICIAN_get'))

# Start of PatchBasedDenoisingImageFilter_POISSON_get

`PatchBasedDenoisingImageFilter_POISSON_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_POISSON_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PatchBasedDenoisingImageFilter__NoiseModelType");
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_POISSON_get`, 'returnType') = 'character'
class(`PatchBasedDenoisingImageFilter_POISSON_get`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_POISSON_get'))

# Start of PatchBasedDenoisingImageFilter_SetNoiseModel

`PatchBasedDenoisingImageFilter_SetNoiseModel` = function(self, NoiseModel)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NoiseModel = enumToInteger(NoiseModel, "_itk__simple__PatchBasedDenoisingImageFilter__NoiseModelType"); 
  
  if(length(NoiseModel) > 1) {
    warning("using only the first element of NoiseModel");
  };
  
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_SetNoiseModel', self, NoiseModel, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_SetNoiseModel`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_SetNoiseModel`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter', 'character')
class(`PatchBasedDenoisingImageFilter_SetNoiseModel`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_SetNoiseModel'))

# Start of PatchBasedDenoisingImageFilter_GetNoiseModel

`PatchBasedDenoisingImageFilter_GetNoiseModel` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_GetNoiseModel', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PatchBasedDenoisingImageFilter__NoiseModelType");
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_GetNoiseModel`, 'returnType') = 'character'
attr(`PatchBasedDenoisingImageFilter_GetNoiseModel`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_GetNoiseModel`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_GetNoiseModel'))

# Start of PatchBasedDenoisingImageFilter_SetNoiseSigma

`PatchBasedDenoisingImageFilter_SetNoiseSigma` = function(self, NoiseSigma)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_SetNoiseSigma', self, NoiseSigma, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_SetNoiseSigma`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_SetNoiseSigma`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter', 'numeric')
class(`PatchBasedDenoisingImageFilter_SetNoiseSigma`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_SetNoiseSigma'))

# Start of PatchBasedDenoisingImageFilter_GetNoiseSigma

`PatchBasedDenoisingImageFilter_GetNoiseSigma` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PatchBasedDenoisingImageFilter_GetNoiseSigma', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PatchBasedDenoisingImageFilter_GetNoiseSigma`, 'returnType') = 'numeric'
attr(`PatchBasedDenoisingImageFilter_GetNoiseSigma`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_GetNoiseSigma`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_GetNoiseSigma'))

# Start of PatchBasedDenoisingImageFilter_SetNoiseModelFidelityWeight

`PatchBasedDenoisingImageFilter_SetNoiseModelFidelityWeight` = function(self, NoiseModelFidelityWeight)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_SetNoiseModelFidelityWeight', self, NoiseModelFidelityWeight, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_SetNoiseModelFidelityWeight`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_SetNoiseModelFidelityWeight`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter', 'numeric')
class(`PatchBasedDenoisingImageFilter_SetNoiseModelFidelityWeight`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_SetNoiseModelFidelityWeight'))

# Start of PatchBasedDenoisingImageFilter_GetNoiseModelFidelityWeight

`PatchBasedDenoisingImageFilter_GetNoiseModelFidelityWeight` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PatchBasedDenoisingImageFilter_GetNoiseModelFidelityWeight', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PatchBasedDenoisingImageFilter_GetNoiseModelFidelityWeight`, 'returnType') = 'numeric'
attr(`PatchBasedDenoisingImageFilter_GetNoiseModelFidelityWeight`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_GetNoiseModelFidelityWeight`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_GetNoiseModelFidelityWeight'))

# Start of PatchBasedDenoisingImageFilter_SetAlwaysTreatComponentsAsEuclidean

`PatchBasedDenoisingImageFilter_SetAlwaysTreatComponentsAsEuclidean` = function(self, AlwaysTreatComponentsAsEuclidean)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  AlwaysTreatComponentsAsEuclidean = as.logical(AlwaysTreatComponentsAsEuclidean);
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_SetAlwaysTreatComponentsAsEuclidean', self, AlwaysTreatComponentsAsEuclidean, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_SetAlwaysTreatComponentsAsEuclidean`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_SetAlwaysTreatComponentsAsEuclidean`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter', 'logical')
class(`PatchBasedDenoisingImageFilter_SetAlwaysTreatComponentsAsEuclidean`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_SetAlwaysTreatComponentsAsEuclidean'))

# Start of PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOn

`PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOn`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOn`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOn`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOn'))

# Start of PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOff

`PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOff`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOff`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOff`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOff'))

# Start of PatchBasedDenoisingImageFilter_GetAlwaysTreatComponentsAsEuclidean

`PatchBasedDenoisingImageFilter_GetAlwaysTreatComponentsAsEuclidean` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PatchBasedDenoisingImageFilter_GetAlwaysTreatComponentsAsEuclidean', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PatchBasedDenoisingImageFilter_GetAlwaysTreatComponentsAsEuclidean`, 'returnType') = 'logical'
attr(`PatchBasedDenoisingImageFilter_GetAlwaysTreatComponentsAsEuclidean`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_GetAlwaysTreatComponentsAsEuclidean`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_GetAlwaysTreatComponentsAsEuclidean'))

# Start of PatchBasedDenoisingImageFilter_SetKernelBandwidthEstimation

`PatchBasedDenoisingImageFilter_SetKernelBandwidthEstimation` = function(self, KernelBandwidthEstimation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelBandwidthEstimation = as.logical(KernelBandwidthEstimation);
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_SetKernelBandwidthEstimation', self, KernelBandwidthEstimation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_SetKernelBandwidthEstimation`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_SetKernelBandwidthEstimation`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter', 'logical')
class(`PatchBasedDenoisingImageFilter_SetKernelBandwidthEstimation`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_SetKernelBandwidthEstimation'))

# Start of PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOn

`PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOn`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOn`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOn`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOn'))

# Start of PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOff

`PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOff`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOff`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOff`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOff'))

# Start of PatchBasedDenoisingImageFilter_GetKernelBandwidthEstimation

`PatchBasedDenoisingImageFilter_GetKernelBandwidthEstimation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PatchBasedDenoisingImageFilter_GetKernelBandwidthEstimation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PatchBasedDenoisingImageFilter_GetKernelBandwidthEstimation`, 'returnType') = 'logical'
attr(`PatchBasedDenoisingImageFilter_GetKernelBandwidthEstimation`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_GetKernelBandwidthEstimation`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_GetKernelBandwidthEstimation'))

# Start of PatchBasedDenoisingImageFilter_SetKernelBandwidthMultiplicationFactor

`PatchBasedDenoisingImageFilter_SetKernelBandwidthMultiplicationFactor` = function(self, KernelBandwidthMultiplicationFactor)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_SetKernelBandwidthMultiplicationFactor', self, KernelBandwidthMultiplicationFactor, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_SetKernelBandwidthMultiplicationFactor`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_SetKernelBandwidthMultiplicationFactor`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter', 'numeric')
class(`PatchBasedDenoisingImageFilter_SetKernelBandwidthMultiplicationFactor`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_SetKernelBandwidthMultiplicationFactor'))

# Start of PatchBasedDenoisingImageFilter_GetKernelBandwidthMultiplicationFactor

`PatchBasedDenoisingImageFilter_GetKernelBandwidthMultiplicationFactor` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PatchBasedDenoisingImageFilter_GetKernelBandwidthMultiplicationFactor', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PatchBasedDenoisingImageFilter_GetKernelBandwidthMultiplicationFactor`, 'returnType') = 'numeric'
attr(`PatchBasedDenoisingImageFilter_GetKernelBandwidthMultiplicationFactor`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_GetKernelBandwidthMultiplicationFactor`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_GetKernelBandwidthMultiplicationFactor'))

# Start of PatchBasedDenoisingImageFilter_SetKernelBandwidthUpdateFrequency

`PatchBasedDenoisingImageFilter_SetKernelBandwidthUpdateFrequency` = function(self, KernelBandwidthUpdateFrequency)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelBandwidthUpdateFrequency = as.integer(KernelBandwidthUpdateFrequency);
  
  if(length(KernelBandwidthUpdateFrequency) > 1) {
    warning("using only the first element of KernelBandwidthUpdateFrequency");
  };
  
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_SetKernelBandwidthUpdateFrequency', self, KernelBandwidthUpdateFrequency, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_SetKernelBandwidthUpdateFrequency`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_SetKernelBandwidthUpdateFrequency`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter', 'integer')
class(`PatchBasedDenoisingImageFilter_SetKernelBandwidthUpdateFrequency`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_SetKernelBandwidthUpdateFrequency'))

# Start of PatchBasedDenoisingImageFilter_GetKernelBandwidthUpdateFrequency

`PatchBasedDenoisingImageFilter_GetKernelBandwidthUpdateFrequency` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PatchBasedDenoisingImageFilter_GetKernelBandwidthUpdateFrequency', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PatchBasedDenoisingImageFilter_GetKernelBandwidthUpdateFrequency`, 'returnType') = 'integer'
attr(`PatchBasedDenoisingImageFilter_GetKernelBandwidthUpdateFrequency`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_GetKernelBandwidthUpdateFrequency`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_GetKernelBandwidthUpdateFrequency'))

# Start of PatchBasedDenoisingImageFilter_SetKernelBandwidthFractionPixelsForEstimation

`PatchBasedDenoisingImageFilter_SetKernelBandwidthFractionPixelsForEstimation` = function(self, KernelBandwidthFractionPixelsForEstimation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_SetKernelBandwidthFractionPixelsForEstimation', self, KernelBandwidthFractionPixelsForEstimation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_SetKernelBandwidthFractionPixelsForEstimation`, 'returnType') = 'void'
attr(`PatchBasedDenoisingImageFilter_SetKernelBandwidthFractionPixelsForEstimation`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter', 'numeric')
class(`PatchBasedDenoisingImageFilter_SetKernelBandwidthFractionPixelsForEstimation`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_SetKernelBandwidthFractionPixelsForEstimation'))

# Start of PatchBasedDenoisingImageFilter_GetKernelBandwidthFractionPixelsForEstimation

`PatchBasedDenoisingImageFilter_GetKernelBandwidthFractionPixelsForEstimation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PatchBasedDenoisingImageFilter_GetKernelBandwidthFractionPixelsForEstimation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PatchBasedDenoisingImageFilter_GetKernelBandwidthFractionPixelsForEstimation`, 'returnType') = 'numeric'
attr(`PatchBasedDenoisingImageFilter_GetKernelBandwidthFractionPixelsForEstimation`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_GetKernelBandwidthFractionPixelsForEstimation`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_GetKernelBandwidthFractionPixelsForEstimation'))

# Start of PatchBasedDenoisingImageFilter_GetName

`PatchBasedDenoisingImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PatchBasedDenoisingImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PatchBasedDenoisingImageFilter_GetName`, 'returnType') = 'character'
attr(`PatchBasedDenoisingImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_GetName`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_GetName'))

# Start of PatchBasedDenoisingImageFilter_ToString

`PatchBasedDenoisingImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PatchBasedDenoisingImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PatchBasedDenoisingImageFilter_ToString`, 'returnType') = 'character'
attr(`PatchBasedDenoisingImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter')
class(`PatchBasedDenoisingImageFilter_ToString`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_ToString'))

# Start of PatchBasedDenoisingImageFilter_Execute

`PatchBasedDenoisingImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_PatchBasedDenoisingImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PatchBasedDenoisingImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`PatchBasedDenoisingImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__PatchBasedDenoisingImageFilter', '_p_itk__simple__Image')
class(`PatchBasedDenoisingImageFilter_Execute`) = c("SWIGFunction", class('PatchBasedDenoisingImageFilter_Execute'))

# Start of accessor method for itk::simple::PatchBasedDenoisingImageFilter
setMethod('$', '_p_itk__simple__PatchBasedDenoisingImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelBandwidthSigma' = PatchBasedDenoisingImageFilter_SetKernelBandwidthSigma, 'GetKernelBandwidthSigma' = PatchBasedDenoisingImageFilter_GetKernelBandwidthSigma, 'SetPatchRadius' = PatchBasedDenoisingImageFilter_SetPatchRadius, 'GetPatchRadius' = PatchBasedDenoisingImageFilter_GetPatchRadius, 'SetNumberOfIterations' = PatchBasedDenoisingImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = PatchBasedDenoisingImageFilter_GetNumberOfIterations, 'SetNumberOfSamplePatches' = PatchBasedDenoisingImageFilter_SetNumberOfSamplePatches, 'GetNumberOfSamplePatches' = PatchBasedDenoisingImageFilter_GetNumberOfSamplePatches, 'SetSampleVariance' = PatchBasedDenoisingImageFilter_SetSampleVariance, 'GetSampleVariance' = PatchBasedDenoisingImageFilter_GetSampleVariance, 'SetNoiseModel' = PatchBasedDenoisingImageFilter_SetNoiseModel, 'GetNoiseModel' = PatchBasedDenoisingImageFilter_GetNoiseModel, 'SetNoiseSigma' = PatchBasedDenoisingImageFilter_SetNoiseSigma, 'GetNoiseSigma' = PatchBasedDenoisingImageFilter_GetNoiseSigma, 'SetNoiseModelFidelityWeight' = PatchBasedDenoisingImageFilter_SetNoiseModelFidelityWeight, 'GetNoiseModelFidelityWeight' = PatchBasedDenoisingImageFilter_GetNoiseModelFidelityWeight, 'SetAlwaysTreatComponentsAsEuclidean' = PatchBasedDenoisingImageFilter_SetAlwaysTreatComponentsAsEuclidean, 'AlwaysTreatComponentsAsEuclideanOn' = PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOn, 'AlwaysTreatComponentsAsEuclideanOff' = PatchBasedDenoisingImageFilter_AlwaysTreatComponentsAsEuclideanOff, 'GetAlwaysTreatComponentsAsEuclidean' = PatchBasedDenoisingImageFilter_GetAlwaysTreatComponentsAsEuclidean, 'SetKernelBandwidthEstimation' = PatchBasedDenoisingImageFilter_SetKernelBandwidthEstimation, 'KernelBandwidthEstimationOn' = PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOn, 'KernelBandwidthEstimationOff' = PatchBasedDenoisingImageFilter_KernelBandwidthEstimationOff, 'GetKernelBandwidthEstimation' = PatchBasedDenoisingImageFilter_GetKernelBandwidthEstimation, 'SetKernelBandwidthMultiplicationFactor' = PatchBasedDenoisingImageFilter_SetKernelBandwidthMultiplicationFactor, 'GetKernelBandwidthMultiplicationFactor' = PatchBasedDenoisingImageFilter_GetKernelBandwidthMultiplicationFactor, 'SetKernelBandwidthUpdateFrequency' = PatchBasedDenoisingImageFilter_SetKernelBandwidthUpdateFrequency, 'GetKernelBandwidthUpdateFrequency' = PatchBasedDenoisingImageFilter_GetKernelBandwidthUpdateFrequency, 'SetKernelBandwidthFractionPixelsForEstimation' = PatchBasedDenoisingImageFilter_SetKernelBandwidthFractionPixelsForEstimation, 'GetKernelBandwidthFractionPixelsForEstimation' = PatchBasedDenoisingImageFilter_GetKernelBandwidthFractionPixelsForEstimation, 'GetName' = PatchBasedDenoisingImageFilter_GetName, 'ToString' = PatchBasedDenoisingImageFilter_ToString, 'Execute' = PatchBasedDenoisingImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::PatchBasedDenoisingImageFilter
# Start of delete_PermuteAxesImageFilter

`delete_PermuteAxesImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_PermuteAxesImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_PermuteAxesImageFilter`, 'returnType') = 'void'
attr(`delete_PermuteAxesImageFilter`, "inputTypes") = c('_p_itk__simple__PermuteAxesImageFilter')
class(`delete_PermuteAxesImageFilter`) = c("SWIGFunction", class('delete_PermuteAxesImageFilter'))

# Start of new_PermuteAxesImageFilter

`PermuteAxesImageFilter` = function()
{
  ;ans = .Call('R_swig_new_PermuteAxesImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__PermuteAxesImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_PermuteAxesImageFilter);
  ans
  
}

attr(`PermuteAxesImageFilter`, 'returnType') = '_p_itk__simple__PermuteAxesImageFilter'
class(`PermuteAxesImageFilter`) = c("SWIGFunction", class('PermuteAxesImageFilter'))

# Start of PermuteAxesImageFilter_DefaultOrder_get

`PermuteAxesImageFilter_DefaultOrder_get` = function()
{
  ;.Call('R_swig_PermuteAxesImageFilter_DefaultOrder_get', PACKAGE='SimpleITK');
  
}

attr(`PermuteAxesImageFilter_DefaultOrder_get`, 'returnType') = 'integer'
class(`PermuteAxesImageFilter_DefaultOrder_get`) = c("SWIGFunction", class('PermuteAxesImageFilter_DefaultOrder_get'))

PermuteAxesImageFilter_DefaultOrder = PermuteAxesImageFilter_DefaultOrder_get
# Start of PermuteAxesImageFilter_SetOrder

`PermuteAxesImageFilter_SetOrder` = function(self, Order)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Order = as.integer(Order);
  ;ans = .Call('R_swig_PermuteAxesImageFilter_SetOrder', self, Order, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PermuteAxesImageFilter_SetOrder`, 'returnType') = 'void'
attr(`PermuteAxesImageFilter_SetOrder`, "inputTypes") = c('_p_itk__simple__PermuteAxesImageFilter', 'integer')
class(`PermuteAxesImageFilter_SetOrder`) = c("SWIGFunction", class('PermuteAxesImageFilter_SetOrder'))

# Start of PermuteAxesImageFilter_GetOrder

`PermuteAxesImageFilter_GetOrder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PermuteAxesImageFilter_GetOrder', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PermuteAxesImageFilter_GetOrder`, 'returnType') = 'integer'
attr(`PermuteAxesImageFilter_GetOrder`, "inputTypes") = c('_p_itk__simple__PermuteAxesImageFilter')
class(`PermuteAxesImageFilter_GetOrder`) = c("SWIGFunction", class('PermuteAxesImageFilter_GetOrder'))

# Start of PermuteAxesImageFilter_GetName

`PermuteAxesImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PermuteAxesImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PermuteAxesImageFilter_GetName`, 'returnType') = 'character'
attr(`PermuteAxesImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__PermuteAxesImageFilter')
class(`PermuteAxesImageFilter_GetName`) = c("SWIGFunction", class('PermuteAxesImageFilter_GetName'))

# Start of PermuteAxesImageFilter_ToString

`PermuteAxesImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PermuteAxesImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PermuteAxesImageFilter_ToString`, 'returnType') = 'character'
attr(`PermuteAxesImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__PermuteAxesImageFilter')
class(`PermuteAxesImageFilter_ToString`) = c("SWIGFunction", class('PermuteAxesImageFilter_ToString'))

# Start of PermuteAxesImageFilter_Execute

`PermuteAxesImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_PermuteAxesImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PermuteAxesImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`PermuteAxesImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__PermuteAxesImageFilter', '_p_itk__simple__Image')
class(`PermuteAxesImageFilter_Execute`) = c("SWIGFunction", class('PermuteAxesImageFilter_Execute'))

# Start of accessor method for itk::simple::PermuteAxesImageFilter
setMethod('$', '_p_itk__simple__PermuteAxesImageFilter', function(x, name)

{
  accessorFuns = list('SetOrder' = PermuteAxesImageFilter_SetOrder, 'GetOrder' = PermuteAxesImageFilter_GetOrder, 'GetName' = PermuteAxesImageFilter_GetName, 'ToString' = PermuteAxesImageFilter_ToString, 'Execute' = PermuteAxesImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::PermuteAxesImageFilter
# Start of PermuteAxes

`PermuteAxes__SWIG_0` = function(image1, order, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  order = as.integer(order);
  ;ans = .Call('R_swig_PermuteAxes__SWIG_0', image1, order, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PermuteAxes__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`PermuteAxes__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`PermuteAxes__SWIG_0`) = c("SWIGFunction", class('PermuteAxes__SWIG_0'))

# Start of PermuteAxes

`PermuteAxes__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_PermuteAxes__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PermuteAxes__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`PermuteAxes__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`PermuteAxes__SWIG_1`) = c("SWIGFunction", class('PermuteAxes__SWIG_1'))

`PermuteAxes` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- PermuteAxes__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- PermuteAxes__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for PermuteAxes with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_PhysicalPointImageSource

`delete_PhysicalPointImageSource` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_PhysicalPointImageSource', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_PhysicalPointImageSource`, 'returnType') = 'void'
attr(`delete_PhysicalPointImageSource`, "inputTypes") = c('_p_itk__simple__PhysicalPointImageSource')
class(`delete_PhysicalPointImageSource`) = c("SWIGFunction", class('delete_PhysicalPointImageSource'))

# Start of new_PhysicalPointImageSource

`PhysicalPointImageSource` = function()
{
  ;ans = .Call('R_swig_new_PhysicalPointImageSource', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__PhysicalPointImageSource", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_PhysicalPointImageSource);
  ans
  
}

attr(`PhysicalPointImageSource`, 'returnType') = '_p_itk__simple__PhysicalPointImageSource'
class(`PhysicalPointImageSource`) = c("SWIGFunction", class('PhysicalPointImageSource'))

# Start of PhysicalPointImageSource_SetOutputPixelType

`PhysicalPointImageSource_SetOutputPixelType` = function(self, OutputPixelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputPixelType = enumToInteger(OutputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(OutputPixelType) > 1) {
    warning("using only the first element of OutputPixelType");
  };
  
  ;ans = .Call('R_swig_PhysicalPointImageSource_SetOutputPixelType', self, OutputPixelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PhysicalPointImageSource_SetOutputPixelType`, 'returnType') = 'void'
attr(`PhysicalPointImageSource_SetOutputPixelType`, "inputTypes") = c('_p_itk__simple__PhysicalPointImageSource', 'character')
class(`PhysicalPointImageSource_SetOutputPixelType`) = c("SWIGFunction", class('PhysicalPointImageSource_SetOutputPixelType'))

# Start of PhysicalPointImageSource_GetOutputPixelType

`PhysicalPointImageSource_GetOutputPixelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_PhysicalPointImageSource_GetOutputPixelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`PhysicalPointImageSource_GetOutputPixelType`, 'returnType') = 'character'
attr(`PhysicalPointImageSource_GetOutputPixelType`, "inputTypes") = c('_p_itk__simple__PhysicalPointImageSource')
class(`PhysicalPointImageSource_GetOutputPixelType`) = c("SWIGFunction", class('PhysicalPointImageSource_GetOutputPixelType'))

# Start of PhysicalPointImageSource_SetSize

`PhysicalPointImageSource_SetSize` = function(self, Size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Size = as.integer(Size);
  ;ans = .Call('R_swig_PhysicalPointImageSource_SetSize', self, Size, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PhysicalPointImageSource_SetSize`, 'returnType') = 'void'
attr(`PhysicalPointImageSource_SetSize`, "inputTypes") = c('_p_itk__simple__PhysicalPointImageSource', 'integer')
class(`PhysicalPointImageSource_SetSize`) = c("SWIGFunction", class('PhysicalPointImageSource_SetSize'))

# Start of PhysicalPointImageSource_GetSize

`PhysicalPointImageSource_GetSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PhysicalPointImageSource_GetSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PhysicalPointImageSource_GetSize`, 'returnType') = 'integer'
attr(`PhysicalPointImageSource_GetSize`, "inputTypes") = c('_p_itk__simple__PhysicalPointImageSource')
class(`PhysicalPointImageSource_GetSize`) = c("SWIGFunction", class('PhysicalPointImageSource_GetSize'))

# Start of PhysicalPointImageSource_SetOrigin

`PhysicalPointImageSource_SetOrigin` = function(self, Origin)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Origin = as.numeric(Origin);
  ;ans = .Call('R_swig_PhysicalPointImageSource_SetOrigin', self, Origin, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PhysicalPointImageSource_SetOrigin`, 'returnType') = 'void'
attr(`PhysicalPointImageSource_SetOrigin`, "inputTypes") = c('_p_itk__simple__PhysicalPointImageSource', 'numeric')
class(`PhysicalPointImageSource_SetOrigin`) = c("SWIGFunction", class('PhysicalPointImageSource_SetOrigin'))

# Start of PhysicalPointImageSource_GetOrigin

`PhysicalPointImageSource_GetOrigin` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PhysicalPointImageSource_GetOrigin', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PhysicalPointImageSource_GetOrigin`, 'returnType') = 'numeric'
attr(`PhysicalPointImageSource_GetOrigin`, "inputTypes") = c('_p_itk__simple__PhysicalPointImageSource')
class(`PhysicalPointImageSource_GetOrigin`) = c("SWIGFunction", class('PhysicalPointImageSource_GetOrigin'))

# Start of PhysicalPointImageSource_SetSpacing

`PhysicalPointImageSource_SetSpacing` = function(self, Spacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Spacing = as.numeric(Spacing);
  ;ans = .Call('R_swig_PhysicalPointImageSource_SetSpacing', self, Spacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PhysicalPointImageSource_SetSpacing`, 'returnType') = 'void'
attr(`PhysicalPointImageSource_SetSpacing`, "inputTypes") = c('_p_itk__simple__PhysicalPointImageSource', 'numeric')
class(`PhysicalPointImageSource_SetSpacing`) = c("SWIGFunction", class('PhysicalPointImageSource_SetSpacing'))

# Start of PhysicalPointImageSource_GetSpacing

`PhysicalPointImageSource_GetSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PhysicalPointImageSource_GetSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PhysicalPointImageSource_GetSpacing`, 'returnType') = 'numeric'
attr(`PhysicalPointImageSource_GetSpacing`, "inputTypes") = c('_p_itk__simple__PhysicalPointImageSource')
class(`PhysicalPointImageSource_GetSpacing`) = c("SWIGFunction", class('PhysicalPointImageSource_GetSpacing'))

# Start of PhysicalPointImageSource_SetDirection

`PhysicalPointImageSource_SetDirection` = function(self, Direction)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Direction = as.numeric(Direction);
  ;ans = .Call('R_swig_PhysicalPointImageSource_SetDirection', self, Direction, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PhysicalPointImageSource_SetDirection`, 'returnType') = 'void'
attr(`PhysicalPointImageSource_SetDirection`, "inputTypes") = c('_p_itk__simple__PhysicalPointImageSource', 'numeric')
class(`PhysicalPointImageSource_SetDirection`) = c("SWIGFunction", class('PhysicalPointImageSource_SetDirection'))

# Start of PhysicalPointImageSource_GetDirection

`PhysicalPointImageSource_GetDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PhysicalPointImageSource_GetDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PhysicalPointImageSource_GetDirection`, 'returnType') = 'numeric'
attr(`PhysicalPointImageSource_GetDirection`, "inputTypes") = c('_p_itk__simple__PhysicalPointImageSource')
class(`PhysicalPointImageSource_GetDirection`) = c("SWIGFunction", class('PhysicalPointImageSource_GetDirection'))

# Start of PhysicalPointImageSource_GetName

`PhysicalPointImageSource_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PhysicalPointImageSource_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PhysicalPointImageSource_GetName`, 'returnType') = 'character'
attr(`PhysicalPointImageSource_GetName`, "inputTypes") = c('_p_itk__simple__PhysicalPointImageSource')
class(`PhysicalPointImageSource_GetName`) = c("SWIGFunction", class('PhysicalPointImageSource_GetName'))

# Start of PhysicalPointImageSource_ToString

`PhysicalPointImageSource_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PhysicalPointImageSource_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PhysicalPointImageSource_ToString`, 'returnType') = 'character'
attr(`PhysicalPointImageSource_ToString`, "inputTypes") = c('_p_itk__simple__PhysicalPointImageSource')
class(`PhysicalPointImageSource_ToString`) = c("SWIGFunction", class('PhysicalPointImageSource_ToString'))

# Start of PhysicalPointImageSource_Execute

`PhysicalPointImageSource_Execute` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_PhysicalPointImageSource_Execute', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PhysicalPointImageSource_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`PhysicalPointImageSource_Execute`, "inputTypes") = c('_p_itk__simple__PhysicalPointImageSource')
class(`PhysicalPointImageSource_Execute`) = c("SWIGFunction", class('PhysicalPointImageSource_Execute'))

# Start of PhysicalPointImageSource_SetReferenceImage

`PhysicalPointImageSource_SetReferenceImage` = function(self, refImage)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(refImage, "ExternalReference")) refImage = slot(refImage,"ref"); 
  ;ans = .Call('R_swig_PhysicalPointImageSource_SetReferenceImage', self, refImage, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`PhysicalPointImageSource_SetReferenceImage`, 'returnType') = 'void'
attr(`PhysicalPointImageSource_SetReferenceImage`, "inputTypes") = c('_p_itk__simple__PhysicalPointImageSource', '_p_itk__simple__Image')
class(`PhysicalPointImageSource_SetReferenceImage`) = c("SWIGFunction", class('PhysicalPointImageSource_SetReferenceImage'))

# Start of accessor method for itk::simple::PhysicalPointImageSource
setMethod('$', '_p_itk__simple__PhysicalPointImageSource', function(x, name)

{
  accessorFuns = list('SetOutputPixelType' = PhysicalPointImageSource_SetOutputPixelType, 'GetOutputPixelType' = PhysicalPointImageSource_GetOutputPixelType, 'SetSize' = PhysicalPointImageSource_SetSize, 'GetSize' = PhysicalPointImageSource_GetSize, 'SetOrigin' = PhysicalPointImageSource_SetOrigin, 'GetOrigin' = PhysicalPointImageSource_GetOrigin, 'SetSpacing' = PhysicalPointImageSource_SetSpacing, 'GetSpacing' = PhysicalPointImageSource_GetSpacing, 'SetDirection' = PhysicalPointImageSource_SetDirection, 'GetDirection' = PhysicalPointImageSource_GetDirection, 'GetName' = PhysicalPointImageSource_GetName, 'ToString' = PhysicalPointImageSource_ToString, 'Execute' = PhysicalPointImageSource_Execute, 'SetReferenceImage' = PhysicalPointImageSource_SetReferenceImage);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::PhysicalPointImageSource
# Start of PhysicalPointSource

`PhysicalPointSource__SWIG_0` = function(outputPixelType, size, origin, spacing, direction, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  origin = as.numeric(origin);
  spacing = as.numeric(spacing);
  direction = as.numeric(direction);
  ;ans = .Call('R_swig_PhysicalPointSource__SWIG_0', outputPixelType, size, origin, spacing, direction, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PhysicalPointSource__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`PhysicalPointSource__SWIG_0`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric', 'numeric')
class(`PhysicalPointSource__SWIG_0`) = c("SWIGFunction", class('PhysicalPointSource__SWIG_0'))

# Start of PhysicalPointSource

`PhysicalPointSource__SWIG_1` = function(outputPixelType, size, origin, spacing, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  origin = as.numeric(origin);
  spacing = as.numeric(spacing);
  ;ans = .Call('R_swig_PhysicalPointSource__SWIG_1', outputPixelType, size, origin, spacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PhysicalPointSource__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`PhysicalPointSource__SWIG_1`, "inputTypes") = c('character', 'integer', 'numeric', 'numeric')
class(`PhysicalPointSource__SWIG_1`) = c("SWIGFunction", class('PhysicalPointSource__SWIG_1'))

# Start of PhysicalPointSource

`PhysicalPointSource__SWIG_2` = function(outputPixelType, size, origin, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  origin = as.numeric(origin);
  ;ans = .Call('R_swig_PhysicalPointSource__SWIG_2', outputPixelType, size, origin, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PhysicalPointSource__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`PhysicalPointSource__SWIG_2`, "inputTypes") = c('character', 'integer', 'numeric')
class(`PhysicalPointSource__SWIG_2`) = c("SWIGFunction", class('PhysicalPointSource__SWIG_2'))

# Start of PhysicalPointSource

`PhysicalPointSource__SWIG_3` = function(outputPixelType, size, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  ;ans = .Call('R_swig_PhysicalPointSource__SWIG_3', outputPixelType, size, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PhysicalPointSource__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`PhysicalPointSource__SWIG_3`, "inputTypes") = c('character', 'integer')
class(`PhysicalPointSource__SWIG_3`) = c("SWIGFunction", class('PhysicalPointSource__SWIG_3'))

# Start of PhysicalPointSource

`PhysicalPointSource__SWIG_4` = function(outputPixelType, .copy = FALSE)
{
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  ;ans = .Call('R_swig_PhysicalPointSource__SWIG_4', outputPixelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PhysicalPointSource__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`PhysicalPointSource__SWIG_4`, "inputTypes") = c('character')
class(`PhysicalPointSource__SWIG_4`) = c("SWIGFunction", class('PhysicalPointSource__SWIG_4'))

# Start of PhysicalPointSource

`PhysicalPointSource__SWIG_5` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_PhysicalPointSource__SWIG_5', as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PhysicalPointSource__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
class(`PhysicalPointSource__SWIG_5`) = c("SWIGFunction", class('PhysicalPointSource__SWIG_5'))

`PhysicalPointSource` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 0) {
    f <- PhysicalPointSource__SWIG_5; 
  } else if (argc == 1) {
    if ( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) {
      f <- PhysicalPointSource__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- PhysicalPointSource__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) )) {
      f <- PhysicalPointSource__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) )) {
      f <- PhysicalPointSource__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( is.character(argv[[1]]) && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) )) {
      f <- PhysicalPointSource__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for PhysicalPointSource with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_PowImageFilter

`delete_PowImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_PowImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_PowImageFilter`, 'returnType') = 'void'
attr(`delete_PowImageFilter`, "inputTypes") = c('_p_itk__simple__PowImageFilter')
class(`delete_PowImageFilter`) = c("SWIGFunction", class('delete_PowImageFilter'))

# Start of new_PowImageFilter

`PowImageFilter` = function()
{
  ;ans = .Call('R_swig_new_PowImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__PowImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_PowImageFilter);
  ans
  
}

attr(`PowImageFilter`, 'returnType') = '_p_itk__simple__PowImageFilter'
class(`PowImageFilter`) = c("SWIGFunction", class('PowImageFilter'))

# Start of PowImageFilter_GetName

`PowImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PowImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PowImageFilter_GetName`, 'returnType') = 'character'
attr(`PowImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__PowImageFilter')
class(`PowImageFilter_GetName`) = c("SWIGFunction", class('PowImageFilter_GetName'))

# Start of PowImageFilter_ToString

`PowImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_PowImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`PowImageFilter_ToString`, 'returnType') = 'character'
attr(`PowImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__PowImageFilter')
class(`PowImageFilter_ToString`) = c("SWIGFunction", class('PowImageFilter_ToString'))

# Start of PowImageFilter_Execute

`PowImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_PowImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PowImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`PowImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__PowImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`PowImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('PowImageFilter_Execute__SWIG_0'))

# Start of PowImageFilter_Execute

`PowImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_PowImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PowImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`PowImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__PowImageFilter', '_p_itk__simple__Image', 'numeric')
class(`PowImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('PowImageFilter_Execute__SWIG_1'))

# Start of PowImageFilter_Execute

`PowImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_PowImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`PowImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`PowImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__PowImageFilter', 'numeric', '_p_itk__simple__Image')
class(`PowImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('PowImageFilter_Execute__SWIG_2'))

`PowImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__PowImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- PowImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__PowImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- PowImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__PowImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- PowImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for PowImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::PowImageFilter
setMethod('$', '_p_itk__simple__PowImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = PowImageFilter_GetName, 'ToString' = PowImageFilter_ToString, 'Execute' = PowImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::PowImageFilter
# Start of Pow

`Pow__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Pow__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Pow__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Pow__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Pow__SWIG_0`) = c("SWIGFunction", class('Pow__SWIG_0'))

# Start of Pow

`Pow__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Pow__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Pow__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Pow__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`Pow__SWIG_1`) = c("SWIGFunction", class('Pow__SWIG_1'))

# Start of Pow

`Pow__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Pow__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Pow__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Pow__SWIG_2`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`Pow__SWIG_2`) = c("SWIGFunction", class('Pow__SWIG_2'))

`Pow` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Pow__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Pow__SWIG_1; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Pow__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Pow with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ProjectedLandweberDeconvolutionImageFilter

`delete_ProjectedLandweberDeconvolutionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ProjectedLandweberDeconvolutionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ProjectedLandweberDeconvolutionImageFilter`, 'returnType') = 'void'
attr(`delete_ProjectedLandweberDeconvolutionImageFilter`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter')
class(`delete_ProjectedLandweberDeconvolutionImageFilter`) = c("SWIGFunction", class('delete_ProjectedLandweberDeconvolutionImageFilter'))

# Start of new_ProjectedLandweberDeconvolutionImageFilter

`ProjectedLandweberDeconvolutionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ProjectedLandweberDeconvolutionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ProjectedLandweberDeconvolutionImageFilter);
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter`, 'returnType') = '_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter'
class(`ProjectedLandweberDeconvolutionImageFilter`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter'))

# Start of ProjectedLandweberDeconvolutionImageFilter_SetAlpha

`ProjectedLandweberDeconvolutionImageFilter_SetAlpha` = function(self, Alpha)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_SetAlpha', self, Alpha, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_SetAlpha`, 'returnType') = 'void'
attr(`ProjectedLandweberDeconvolutionImageFilter_SetAlpha`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter', 'numeric')
class(`ProjectedLandweberDeconvolutionImageFilter_SetAlpha`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_SetAlpha'))

# Start of ProjectedLandweberDeconvolutionImageFilter_GetAlpha

`ProjectedLandweberDeconvolutionImageFilter_GetAlpha` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_GetAlpha', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_GetAlpha`, 'returnType') = 'numeric'
attr(`ProjectedLandweberDeconvolutionImageFilter_GetAlpha`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter')
class(`ProjectedLandweberDeconvolutionImageFilter_GetAlpha`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_GetAlpha'))

# Start of ProjectedLandweberDeconvolutionImageFilter_SetNumberOfIterations

`ProjectedLandweberDeconvolutionImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`ProjectedLandweberDeconvolutionImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter', 'integer')
class(`ProjectedLandweberDeconvolutionImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_SetNumberOfIterations'))

# Start of ProjectedLandweberDeconvolutionImageFilter_GetNumberOfIterations

`ProjectedLandweberDeconvolutionImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`ProjectedLandweberDeconvolutionImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter')
class(`ProjectedLandweberDeconvolutionImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_GetNumberOfIterations'))

# Start of ProjectedLandweberDeconvolutionImageFilter_SetNormalize

`ProjectedLandweberDeconvolutionImageFilter_SetNormalize` = function(self, Normalize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Normalize = as.logical(Normalize);
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_SetNormalize', self, Normalize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_SetNormalize`, 'returnType') = 'void'
attr(`ProjectedLandweberDeconvolutionImageFilter_SetNormalize`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter', 'logical')
class(`ProjectedLandweberDeconvolutionImageFilter_SetNormalize`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_SetNormalize'))

# Start of ProjectedLandweberDeconvolutionImageFilter_NormalizeOn

`ProjectedLandweberDeconvolutionImageFilter_NormalizeOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_NormalizeOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_NormalizeOn`, 'returnType') = 'void'
attr(`ProjectedLandweberDeconvolutionImageFilter_NormalizeOn`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter')
class(`ProjectedLandweberDeconvolutionImageFilter_NormalizeOn`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_NormalizeOn'))

# Start of ProjectedLandweberDeconvolutionImageFilter_NormalizeOff

`ProjectedLandweberDeconvolutionImageFilter_NormalizeOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_NormalizeOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_NormalizeOff`, 'returnType') = 'void'
attr(`ProjectedLandweberDeconvolutionImageFilter_NormalizeOff`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter')
class(`ProjectedLandweberDeconvolutionImageFilter_NormalizeOff`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_NormalizeOff'))

# Start of ProjectedLandweberDeconvolutionImageFilter_GetNormalize

`ProjectedLandweberDeconvolutionImageFilter_GetNormalize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_GetNormalize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_GetNormalize`, 'returnType') = 'logical'
attr(`ProjectedLandweberDeconvolutionImageFilter_GetNormalize`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter')
class(`ProjectedLandweberDeconvolutionImageFilter_GetNormalize`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_GetNormalize'))

# Start of ProjectedLandweberDeconvolutionImageFilter_ZERO_PAD_get

`ProjectedLandweberDeconvolutionImageFilter_ZERO_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_ZERO_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ProjectedLandweberDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_ZERO_PAD_get`, 'returnType') = 'character'
class(`ProjectedLandweberDeconvolutionImageFilter_ZERO_PAD_get`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_ZERO_PAD_get'))

# Start of ProjectedLandweberDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get

`ProjectedLandweberDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ProjectedLandweberDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`, 'returnType') = 'character'
class(`ProjectedLandweberDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get'))

# Start of ProjectedLandweberDeconvolutionImageFilter_PERIODIC_PAD_get

`ProjectedLandweberDeconvolutionImageFilter_PERIODIC_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_PERIODIC_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ProjectedLandweberDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_PERIODIC_PAD_get`, 'returnType') = 'character'
class(`ProjectedLandweberDeconvolutionImageFilter_PERIODIC_PAD_get`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_PERIODIC_PAD_get'))

# Start of ProjectedLandweberDeconvolutionImageFilter_SetBoundaryCondition

`ProjectedLandweberDeconvolutionImageFilter_SetBoundaryCondition` = function(self, BoundaryCondition)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BoundaryCondition = enumToInteger(BoundaryCondition, "_itk__simple__ProjectedLandweberDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(BoundaryCondition) > 1) {
    warning("using only the first element of BoundaryCondition");
  };
  
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_SetBoundaryCondition', self, BoundaryCondition, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_SetBoundaryCondition`, 'returnType') = 'void'
attr(`ProjectedLandweberDeconvolutionImageFilter_SetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter', 'character')
class(`ProjectedLandweberDeconvolutionImageFilter_SetBoundaryCondition`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_SetBoundaryCondition'))

# Start of ProjectedLandweberDeconvolutionImageFilter_GetBoundaryCondition

`ProjectedLandweberDeconvolutionImageFilter_GetBoundaryCondition` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_GetBoundaryCondition', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ProjectedLandweberDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_GetBoundaryCondition`, 'returnType') = 'character'
attr(`ProjectedLandweberDeconvolutionImageFilter_GetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter')
class(`ProjectedLandweberDeconvolutionImageFilter_GetBoundaryCondition`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_GetBoundaryCondition'))

# Start of ProjectedLandweberDeconvolutionImageFilter_SAME_get

`ProjectedLandweberDeconvolutionImageFilter_SAME_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_SAME_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ProjectedLandweberDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_SAME_get`, 'returnType') = 'character'
class(`ProjectedLandweberDeconvolutionImageFilter_SAME_get`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_SAME_get'))

# Start of ProjectedLandweberDeconvolutionImageFilter_VALID_get

`ProjectedLandweberDeconvolutionImageFilter_VALID_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_VALID_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ProjectedLandweberDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_VALID_get`, 'returnType') = 'character'
class(`ProjectedLandweberDeconvolutionImageFilter_VALID_get`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_VALID_get'))

# Start of ProjectedLandweberDeconvolutionImageFilter_SetOutputRegionMode

`ProjectedLandweberDeconvolutionImageFilter_SetOutputRegionMode` = function(self, OutputRegionMode)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputRegionMode = enumToInteger(OutputRegionMode, "_itk__simple__ProjectedLandweberDeconvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(OutputRegionMode) > 1) {
    warning("using only the first element of OutputRegionMode");
  };
  
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_SetOutputRegionMode', self, OutputRegionMode, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_SetOutputRegionMode`, 'returnType') = 'void'
attr(`ProjectedLandweberDeconvolutionImageFilter_SetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter', 'character')
class(`ProjectedLandweberDeconvolutionImageFilter_SetOutputRegionMode`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_SetOutputRegionMode'))

# Start of ProjectedLandweberDeconvolutionImageFilter_GetOutputRegionMode

`ProjectedLandweberDeconvolutionImageFilter_GetOutputRegionMode` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_GetOutputRegionMode', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ProjectedLandweberDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_GetOutputRegionMode`, 'returnType') = 'character'
attr(`ProjectedLandweberDeconvolutionImageFilter_GetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter')
class(`ProjectedLandweberDeconvolutionImageFilter_GetOutputRegionMode`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_GetOutputRegionMode'))

# Start of ProjectedLandweberDeconvolutionImageFilter_GetName

`ProjectedLandweberDeconvolutionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_GetName`, 'returnType') = 'character'
attr(`ProjectedLandweberDeconvolutionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter')
class(`ProjectedLandweberDeconvolutionImageFilter_GetName`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_GetName'))

# Start of ProjectedLandweberDeconvolutionImageFilter_ToString

`ProjectedLandweberDeconvolutionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_ToString`, 'returnType') = 'character'
attr(`ProjectedLandweberDeconvolutionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter')
class(`ProjectedLandweberDeconvolutionImageFilter_ToString`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_ToString'))

# Start of ProjectedLandweberDeconvolutionImageFilter_Execute

`ProjectedLandweberDeconvolutionImageFilter_Execute` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_Execute', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ProjectedLandweberDeconvolutionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ProjectedLandweberDeconvolutionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ProjectedLandweberDeconvolutionImageFilter_Execute`) = c("SWIGFunction", class('ProjectedLandweberDeconvolutionImageFilter_Execute'))

# Start of accessor method for itk::simple::ProjectedLandweberDeconvolutionImageFilter
setMethod('$', '_p_itk__simple__ProjectedLandweberDeconvolutionImageFilter', function(x, name)

{
  accessorFuns = list('SetAlpha' = ProjectedLandweberDeconvolutionImageFilter_SetAlpha, 'GetAlpha' = ProjectedLandweberDeconvolutionImageFilter_GetAlpha, 'SetNumberOfIterations' = ProjectedLandweberDeconvolutionImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = ProjectedLandweberDeconvolutionImageFilter_GetNumberOfIterations, 'SetNormalize' = ProjectedLandweberDeconvolutionImageFilter_SetNormalize, 'NormalizeOn' = ProjectedLandweberDeconvolutionImageFilter_NormalizeOn, 'NormalizeOff' = ProjectedLandweberDeconvolutionImageFilter_NormalizeOff, 'GetNormalize' = ProjectedLandweberDeconvolutionImageFilter_GetNormalize, 'SetBoundaryCondition' = ProjectedLandweberDeconvolutionImageFilter_SetBoundaryCondition, 'GetBoundaryCondition' = ProjectedLandweberDeconvolutionImageFilter_GetBoundaryCondition, 'SetOutputRegionMode' = ProjectedLandweberDeconvolutionImageFilter_SetOutputRegionMode, 'GetOutputRegionMode' = ProjectedLandweberDeconvolutionImageFilter_GetOutputRegionMode, 'GetName' = ProjectedLandweberDeconvolutionImageFilter_GetName, 'ToString' = ProjectedLandweberDeconvolutionImageFilter_ToString, 'Execute' = ProjectedLandweberDeconvolutionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ProjectedLandweberDeconvolutionImageFilter
# Start of ProjectedLandweberDeconvolution

`ProjectedLandweberDeconvolution__SWIG_0` = function(image1, image2, alpha, numberOfIterations, normalize, boundaryCondition, outputRegionMode, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__ProjectedLandweberDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  outputRegionMode = enumToInteger(outputRegionMode, "_itk__simple__ProjectedLandweberDeconvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(outputRegionMode) > 1) {
    warning("using only the first element of outputRegionMode");
  };
  
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolution__SWIG_0', image1, image2, alpha, numberOfIterations, normalize, boundaryCondition, outputRegionMode, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ProjectedLandweberDeconvolution__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ProjectedLandweberDeconvolution__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'logical', 'character', 'character')
class(`ProjectedLandweberDeconvolution__SWIG_0`) = c("SWIGFunction", class('ProjectedLandweberDeconvolution__SWIG_0'))

# Start of ProjectedLandweberDeconvolution

`ProjectedLandweberDeconvolution__SWIG_1` = function(image1, image2, alpha, numberOfIterations, normalize, boundaryCondition, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__ProjectedLandweberDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolution__SWIG_1', image1, image2, alpha, numberOfIterations, normalize, boundaryCondition, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ProjectedLandweberDeconvolution__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ProjectedLandweberDeconvolution__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'logical', 'character')
class(`ProjectedLandweberDeconvolution__SWIG_1`) = c("SWIGFunction", class('ProjectedLandweberDeconvolution__SWIG_1'))

# Start of ProjectedLandweberDeconvolution

`ProjectedLandweberDeconvolution__SWIG_2` = function(image1, image2, alpha, numberOfIterations, normalize, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  normalize = as.logical(normalize);
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolution__SWIG_2', image1, image2, alpha, numberOfIterations, normalize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ProjectedLandweberDeconvolution__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ProjectedLandweberDeconvolution__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'logical')
class(`ProjectedLandweberDeconvolution__SWIG_2`) = c("SWIGFunction", class('ProjectedLandweberDeconvolution__SWIG_2'))

# Start of ProjectedLandweberDeconvolution

`ProjectedLandweberDeconvolution__SWIG_3` = function(image1, image2, alpha, numberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolution__SWIG_3', image1, image2, alpha, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ProjectedLandweberDeconvolution__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ProjectedLandweberDeconvolution__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer')
class(`ProjectedLandweberDeconvolution__SWIG_3`) = c("SWIGFunction", class('ProjectedLandweberDeconvolution__SWIG_3'))

# Start of ProjectedLandweberDeconvolution

`ProjectedLandweberDeconvolution__SWIG_4` = function(image1, image2, alpha, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolution__SWIG_4', image1, image2, alpha, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ProjectedLandweberDeconvolution__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ProjectedLandweberDeconvolution__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`ProjectedLandweberDeconvolution__SWIG_4`) = c("SWIGFunction", class('ProjectedLandweberDeconvolution__SWIG_4'))

# Start of ProjectedLandweberDeconvolution

`ProjectedLandweberDeconvolution__SWIG_5` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_ProjectedLandweberDeconvolution__SWIG_5', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ProjectedLandweberDeconvolution__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`ProjectedLandweberDeconvolution__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ProjectedLandweberDeconvolution__SWIG_5`) = c("SWIGFunction", class('ProjectedLandweberDeconvolution__SWIG_5'))

`ProjectedLandweberDeconvolution` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- ProjectedLandweberDeconvolution__SWIG_5; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ProjectedLandweberDeconvolution__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- ProjectedLandweberDeconvolution__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- ProjectedLandweberDeconvolution__SWIG_2; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.character(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- ProjectedLandweberDeconvolution__SWIG_1; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.character(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.character(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- ProjectedLandweberDeconvolution__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ProjectedLandweberDeconvolution with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_RankImageFilter

`delete_RankImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_RankImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_RankImageFilter`, 'returnType') = 'void'
attr(`delete_RankImageFilter`, "inputTypes") = c('_p_itk__simple__RankImageFilter')
class(`delete_RankImageFilter`) = c("SWIGFunction", class('delete_RankImageFilter'))

# Start of new_RankImageFilter

`RankImageFilter` = function()
{
  ;ans = .Call('R_swig_new_RankImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__RankImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_RankImageFilter);
  ans
  
}

attr(`RankImageFilter`, 'returnType') = '_p_itk__simple__RankImageFilter'
class(`RankImageFilter`) = c("SWIGFunction", class('RankImageFilter'))

# Start of RankImageFilter_SetRank

`RankImageFilter_SetRank` = function(self, Rank)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_RankImageFilter_SetRank', self, Rank, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RankImageFilter_SetRank`, 'returnType') = 'void'
attr(`RankImageFilter_SetRank`, "inputTypes") = c('_p_itk__simple__RankImageFilter', 'numeric')
class(`RankImageFilter_SetRank`) = c("SWIGFunction", class('RankImageFilter_SetRank'))

# Start of RankImageFilter_GetRank

`RankImageFilter_GetRank` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RankImageFilter_GetRank', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RankImageFilter_GetRank`, 'returnType') = 'numeric'
attr(`RankImageFilter_GetRank`, "inputTypes") = c('_p_itk__simple__RankImageFilter')
class(`RankImageFilter_GetRank`) = c("SWIGFunction", class('RankImageFilter_GetRank'))

# Start of RankImageFilter_SetRadius

`RankImageFilter_SetRadius` = function(self, Radius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Radius = as.integer(Radius);
  ;ans = .Call('R_swig_RankImageFilter_SetRadius', self, Radius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RankImageFilter_SetRadius`, 'returnType') = 'void'
attr(`RankImageFilter_SetRadius`, "inputTypes") = c('_p_itk__simple__RankImageFilter', 'integer')
class(`RankImageFilter_SetRadius`) = c("SWIGFunction", class('RankImageFilter_SetRadius'))

# Start of RankImageFilter_GetRadius

`RankImageFilter_GetRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RankImageFilter_GetRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RankImageFilter_GetRadius`, 'returnType') = 'integer'
attr(`RankImageFilter_GetRadius`, "inputTypes") = c('_p_itk__simple__RankImageFilter')
class(`RankImageFilter_GetRadius`) = c("SWIGFunction", class('RankImageFilter_GetRadius'))

# Start of RankImageFilter_SetKernelType

`RankImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_RankImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RankImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`RankImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__RankImageFilter', 'character')
class(`RankImageFilter_SetKernelType`) = c("SWIGFunction", class('RankImageFilter_SetKernelType'))

# Start of RankImageFilter_GetKernelType

`RankImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RankImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`RankImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`RankImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__RankImageFilter')
class(`RankImageFilter_GetKernelType`) = c("SWIGFunction", class('RankImageFilter_GetKernelType'))

# Start of RankImageFilter_GetName

`RankImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RankImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RankImageFilter_GetName`, 'returnType') = 'character'
attr(`RankImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__RankImageFilter')
class(`RankImageFilter_GetName`) = c("SWIGFunction", class('RankImageFilter_GetName'))

# Start of RankImageFilter_ToString

`RankImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RankImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RankImageFilter_ToString`, 'returnType') = 'character'
attr(`RankImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__RankImageFilter')
class(`RankImageFilter_ToString`) = c("SWIGFunction", class('RankImageFilter_ToString'))

# Start of RankImageFilter_Execute

`RankImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RankImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RankImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`RankImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__RankImageFilter', '_p_itk__simple__Image')
class(`RankImageFilter_Execute`) = c("SWIGFunction", class('RankImageFilter_Execute'))

# Start of accessor method for itk::simple::RankImageFilter
setMethod('$', '_p_itk__simple__RankImageFilter', function(x, name)

{
  accessorFuns = list('SetRank' = RankImageFilter_SetRank, 'GetRank' = RankImageFilter_GetRank, 'SetRadius' = RankImageFilter_SetRadius, 'GetRadius' = RankImageFilter_GetRadius, 'SetKernelType' = RankImageFilter_SetKernelType, 'GetKernelType' = RankImageFilter_GetKernelType, 'GetName' = RankImageFilter_GetName, 'ToString' = RankImageFilter_ToString, 'Execute' = RankImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::RankImageFilter
# Start of Rank

`Rank__SWIG_0` = function(image1, rank, radius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  radius = as.integer(radius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_Rank__SWIG_0', image1, rank, radius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Rank__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Rank__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'character')
class(`Rank__SWIG_0`) = c("SWIGFunction", class('Rank__SWIG_0'))

# Start of Rank

`Rank__SWIG_1` = function(image1, rank, radius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  radius = as.integer(radius);
  ;ans = .Call('R_swig_Rank__SWIG_1', image1, rank, radius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Rank__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Rank__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`Rank__SWIG_1`) = c("SWIGFunction", class('Rank__SWIG_1'))

# Start of Rank

`Rank__SWIG_2` = function(image1, rank, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Rank__SWIG_2', image1, rank, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Rank__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Rank__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`Rank__SWIG_2`) = c("SWIGFunction", class('Rank__SWIG_2'))

# Start of Rank

`Rank__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Rank__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Rank__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Rank__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`Rank__SWIG_3`) = c("SWIGFunction", class('Rank__SWIG_3'))

`Rank` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Rank__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Rank__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- Rank__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- Rank__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Rank with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_RealAndImaginaryToComplexImageFilter

`delete_RealAndImaginaryToComplexImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_RealAndImaginaryToComplexImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_RealAndImaginaryToComplexImageFilter`, 'returnType') = 'void'
attr(`delete_RealAndImaginaryToComplexImageFilter`, "inputTypes") = c('_p_itk__simple__RealAndImaginaryToComplexImageFilter')
class(`delete_RealAndImaginaryToComplexImageFilter`) = c("SWIGFunction", class('delete_RealAndImaginaryToComplexImageFilter'))

# Start of new_RealAndImaginaryToComplexImageFilter

`RealAndImaginaryToComplexImageFilter` = function()
{
  ;ans = .Call('R_swig_new_RealAndImaginaryToComplexImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__RealAndImaginaryToComplexImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_RealAndImaginaryToComplexImageFilter);
  ans
  
}

attr(`RealAndImaginaryToComplexImageFilter`, 'returnType') = '_p_itk__simple__RealAndImaginaryToComplexImageFilter'
class(`RealAndImaginaryToComplexImageFilter`) = c("SWIGFunction", class('RealAndImaginaryToComplexImageFilter'))

# Start of RealAndImaginaryToComplexImageFilter_GetName

`RealAndImaginaryToComplexImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RealAndImaginaryToComplexImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RealAndImaginaryToComplexImageFilter_GetName`, 'returnType') = 'character'
attr(`RealAndImaginaryToComplexImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__RealAndImaginaryToComplexImageFilter')
class(`RealAndImaginaryToComplexImageFilter_GetName`) = c("SWIGFunction", class('RealAndImaginaryToComplexImageFilter_GetName'))

# Start of RealAndImaginaryToComplexImageFilter_ToString

`RealAndImaginaryToComplexImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RealAndImaginaryToComplexImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RealAndImaginaryToComplexImageFilter_ToString`, 'returnType') = 'character'
attr(`RealAndImaginaryToComplexImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__RealAndImaginaryToComplexImageFilter')
class(`RealAndImaginaryToComplexImageFilter_ToString`) = c("SWIGFunction", class('RealAndImaginaryToComplexImageFilter_ToString'))

# Start of RealAndImaginaryToComplexImageFilter_Execute

`RealAndImaginaryToComplexImageFilter_Execute` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_RealAndImaginaryToComplexImageFilter_Execute', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RealAndImaginaryToComplexImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`RealAndImaginaryToComplexImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__RealAndImaginaryToComplexImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`RealAndImaginaryToComplexImageFilter_Execute`) = c("SWIGFunction", class('RealAndImaginaryToComplexImageFilter_Execute'))

# Start of accessor method for itk::simple::RealAndImaginaryToComplexImageFilter
setMethod('$', '_p_itk__simple__RealAndImaginaryToComplexImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = RealAndImaginaryToComplexImageFilter_GetName, 'ToString' = RealAndImaginaryToComplexImageFilter_ToString, 'Execute' = RealAndImaginaryToComplexImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::RealAndImaginaryToComplexImageFilter
# Start of RealAndImaginaryToComplex

`RealAndImaginaryToComplex` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_RealAndImaginaryToComplex', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RealAndImaginaryToComplex`, 'returnType') = '_p_itk__simple__Image'
attr(`RealAndImaginaryToComplex`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`RealAndImaginaryToComplex`) = c("SWIGFunction", class('RealAndImaginaryToComplex'))

# Start of delete_RealToHalfHermitianForwardFFTImageFilter

`delete_RealToHalfHermitianForwardFFTImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_RealToHalfHermitianForwardFFTImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_RealToHalfHermitianForwardFFTImageFilter`, 'returnType') = 'void'
attr(`delete_RealToHalfHermitianForwardFFTImageFilter`, "inputTypes") = c('_p_itk__simple__RealToHalfHermitianForwardFFTImageFilter')
class(`delete_RealToHalfHermitianForwardFFTImageFilter`) = c("SWIGFunction", class('delete_RealToHalfHermitianForwardFFTImageFilter'))

# Start of new_RealToHalfHermitianForwardFFTImageFilter

`RealToHalfHermitianForwardFFTImageFilter` = function()
{
  ;ans = .Call('R_swig_new_RealToHalfHermitianForwardFFTImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__RealToHalfHermitianForwardFFTImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_RealToHalfHermitianForwardFFTImageFilter);
  ans
  
}

attr(`RealToHalfHermitianForwardFFTImageFilter`, 'returnType') = '_p_itk__simple__RealToHalfHermitianForwardFFTImageFilter'
class(`RealToHalfHermitianForwardFFTImageFilter`) = c("SWIGFunction", class('RealToHalfHermitianForwardFFTImageFilter'))

# Start of RealToHalfHermitianForwardFFTImageFilter_GetName

`RealToHalfHermitianForwardFFTImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RealToHalfHermitianForwardFFTImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RealToHalfHermitianForwardFFTImageFilter_GetName`, 'returnType') = 'character'
attr(`RealToHalfHermitianForwardFFTImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__RealToHalfHermitianForwardFFTImageFilter')
class(`RealToHalfHermitianForwardFFTImageFilter_GetName`) = c("SWIGFunction", class('RealToHalfHermitianForwardFFTImageFilter_GetName'))

# Start of RealToHalfHermitianForwardFFTImageFilter_ToString

`RealToHalfHermitianForwardFFTImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RealToHalfHermitianForwardFFTImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RealToHalfHermitianForwardFFTImageFilter_ToString`, 'returnType') = 'character'
attr(`RealToHalfHermitianForwardFFTImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__RealToHalfHermitianForwardFFTImageFilter')
class(`RealToHalfHermitianForwardFFTImageFilter_ToString`) = c("SWIGFunction", class('RealToHalfHermitianForwardFFTImageFilter_ToString'))

# Start of RealToHalfHermitianForwardFFTImageFilter_Execute

`RealToHalfHermitianForwardFFTImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RealToHalfHermitianForwardFFTImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RealToHalfHermitianForwardFFTImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`RealToHalfHermitianForwardFFTImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__RealToHalfHermitianForwardFFTImageFilter', '_p_itk__simple__Image')
class(`RealToHalfHermitianForwardFFTImageFilter_Execute`) = c("SWIGFunction", class('RealToHalfHermitianForwardFFTImageFilter_Execute'))

# Start of accessor method for itk::simple::RealToHalfHermitianForwardFFTImageFilter
setMethod('$', '_p_itk__simple__RealToHalfHermitianForwardFFTImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = RealToHalfHermitianForwardFFTImageFilter_GetName, 'ToString' = RealToHalfHermitianForwardFFTImageFilter_ToString, 'Execute' = RealToHalfHermitianForwardFFTImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::RealToHalfHermitianForwardFFTImageFilter
# Start of RealToHalfHermitianForwardFFT

`RealToHalfHermitianForwardFFT` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RealToHalfHermitianForwardFFT', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RealToHalfHermitianForwardFFT`, 'returnType') = '_p_itk__simple__Image'
attr(`RealToHalfHermitianForwardFFT`, "inputTypes") = c('_p_itk__simple__Image')
class(`RealToHalfHermitianForwardFFT`) = c("SWIGFunction", class('RealToHalfHermitianForwardFFT'))

# Start of delete_ReconstructionByDilationImageFilter

`delete_ReconstructionByDilationImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ReconstructionByDilationImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ReconstructionByDilationImageFilter`, 'returnType') = 'void'
attr(`delete_ReconstructionByDilationImageFilter`, "inputTypes") = c('_p_itk__simple__ReconstructionByDilationImageFilter')
class(`delete_ReconstructionByDilationImageFilter`) = c("SWIGFunction", class('delete_ReconstructionByDilationImageFilter'))

# Start of new_ReconstructionByDilationImageFilter

`ReconstructionByDilationImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ReconstructionByDilationImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ReconstructionByDilationImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ReconstructionByDilationImageFilter);
  ans
  
}

attr(`ReconstructionByDilationImageFilter`, 'returnType') = '_p_itk__simple__ReconstructionByDilationImageFilter'
class(`ReconstructionByDilationImageFilter`) = c("SWIGFunction", class('ReconstructionByDilationImageFilter'))

# Start of ReconstructionByDilationImageFilter_SetFullyConnected

`ReconstructionByDilationImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_ReconstructionByDilationImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReconstructionByDilationImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`ReconstructionByDilationImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__ReconstructionByDilationImageFilter', 'logical')
class(`ReconstructionByDilationImageFilter_SetFullyConnected`) = c("SWIGFunction", class('ReconstructionByDilationImageFilter_SetFullyConnected'))

# Start of ReconstructionByDilationImageFilter_FullyConnectedOn

`ReconstructionByDilationImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ReconstructionByDilationImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReconstructionByDilationImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`ReconstructionByDilationImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__ReconstructionByDilationImageFilter')
class(`ReconstructionByDilationImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('ReconstructionByDilationImageFilter_FullyConnectedOn'))

# Start of ReconstructionByDilationImageFilter_FullyConnectedOff

`ReconstructionByDilationImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ReconstructionByDilationImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReconstructionByDilationImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`ReconstructionByDilationImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__ReconstructionByDilationImageFilter')
class(`ReconstructionByDilationImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('ReconstructionByDilationImageFilter_FullyConnectedOff'))

# Start of ReconstructionByDilationImageFilter_GetFullyConnected

`ReconstructionByDilationImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ReconstructionByDilationImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ReconstructionByDilationImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`ReconstructionByDilationImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__ReconstructionByDilationImageFilter')
class(`ReconstructionByDilationImageFilter_GetFullyConnected`) = c("SWIGFunction", class('ReconstructionByDilationImageFilter_GetFullyConnected'))

# Start of ReconstructionByDilationImageFilter_SetUseInternalCopy

`ReconstructionByDilationImageFilter_SetUseInternalCopy` = function(self, UseInternalCopy)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseInternalCopy = as.logical(UseInternalCopy);
  ;ans = .Call('R_swig_ReconstructionByDilationImageFilter_SetUseInternalCopy', self, UseInternalCopy, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReconstructionByDilationImageFilter_SetUseInternalCopy`, 'returnType') = 'void'
attr(`ReconstructionByDilationImageFilter_SetUseInternalCopy`, "inputTypes") = c('_p_itk__simple__ReconstructionByDilationImageFilter', 'logical')
class(`ReconstructionByDilationImageFilter_SetUseInternalCopy`) = c("SWIGFunction", class('ReconstructionByDilationImageFilter_SetUseInternalCopy'))

# Start of ReconstructionByDilationImageFilter_UseInternalCopyOn

`ReconstructionByDilationImageFilter_UseInternalCopyOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ReconstructionByDilationImageFilter_UseInternalCopyOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReconstructionByDilationImageFilter_UseInternalCopyOn`, 'returnType') = 'void'
attr(`ReconstructionByDilationImageFilter_UseInternalCopyOn`, "inputTypes") = c('_p_itk__simple__ReconstructionByDilationImageFilter')
class(`ReconstructionByDilationImageFilter_UseInternalCopyOn`) = c("SWIGFunction", class('ReconstructionByDilationImageFilter_UseInternalCopyOn'))

# Start of ReconstructionByDilationImageFilter_UseInternalCopyOff

`ReconstructionByDilationImageFilter_UseInternalCopyOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ReconstructionByDilationImageFilter_UseInternalCopyOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReconstructionByDilationImageFilter_UseInternalCopyOff`, 'returnType') = 'void'
attr(`ReconstructionByDilationImageFilter_UseInternalCopyOff`, "inputTypes") = c('_p_itk__simple__ReconstructionByDilationImageFilter')
class(`ReconstructionByDilationImageFilter_UseInternalCopyOff`) = c("SWIGFunction", class('ReconstructionByDilationImageFilter_UseInternalCopyOff'))

# Start of ReconstructionByDilationImageFilter_GetUseInternalCopy

`ReconstructionByDilationImageFilter_GetUseInternalCopy` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ReconstructionByDilationImageFilter_GetUseInternalCopy', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ReconstructionByDilationImageFilter_GetUseInternalCopy`, 'returnType') = 'logical'
attr(`ReconstructionByDilationImageFilter_GetUseInternalCopy`, "inputTypes") = c('_p_itk__simple__ReconstructionByDilationImageFilter')
class(`ReconstructionByDilationImageFilter_GetUseInternalCopy`) = c("SWIGFunction", class('ReconstructionByDilationImageFilter_GetUseInternalCopy'))

# Start of ReconstructionByDilationImageFilter_GetName

`ReconstructionByDilationImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ReconstructionByDilationImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ReconstructionByDilationImageFilter_GetName`, 'returnType') = 'character'
attr(`ReconstructionByDilationImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ReconstructionByDilationImageFilter')
class(`ReconstructionByDilationImageFilter_GetName`) = c("SWIGFunction", class('ReconstructionByDilationImageFilter_GetName'))

# Start of ReconstructionByDilationImageFilter_ToString

`ReconstructionByDilationImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ReconstructionByDilationImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ReconstructionByDilationImageFilter_ToString`, 'returnType') = 'character'
attr(`ReconstructionByDilationImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ReconstructionByDilationImageFilter')
class(`ReconstructionByDilationImageFilter_ToString`) = c("SWIGFunction", class('ReconstructionByDilationImageFilter_ToString'))

# Start of ReconstructionByDilationImageFilter_Execute

`ReconstructionByDilationImageFilter_Execute` = function(self, markerImage, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_ReconstructionByDilationImageFilter_Execute', self, markerImage, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReconstructionByDilationImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ReconstructionByDilationImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ReconstructionByDilationImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ReconstructionByDilationImageFilter_Execute`) = c("SWIGFunction", class('ReconstructionByDilationImageFilter_Execute'))

# Start of accessor method for itk::simple::ReconstructionByDilationImageFilter
setMethod('$', '_p_itk__simple__ReconstructionByDilationImageFilter', function(x, name)

{
  accessorFuns = list('SetFullyConnected' = ReconstructionByDilationImageFilter_SetFullyConnected, 'FullyConnectedOn' = ReconstructionByDilationImageFilter_FullyConnectedOn, 'FullyConnectedOff' = ReconstructionByDilationImageFilter_FullyConnectedOff, 'GetFullyConnected' = ReconstructionByDilationImageFilter_GetFullyConnected, 'SetUseInternalCopy' = ReconstructionByDilationImageFilter_SetUseInternalCopy, 'UseInternalCopyOn' = ReconstructionByDilationImageFilter_UseInternalCopyOn, 'UseInternalCopyOff' = ReconstructionByDilationImageFilter_UseInternalCopyOff, 'GetUseInternalCopy' = ReconstructionByDilationImageFilter_GetUseInternalCopy, 'GetName' = ReconstructionByDilationImageFilter_GetName, 'ToString' = ReconstructionByDilationImageFilter_ToString, 'Execute' = ReconstructionByDilationImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ReconstructionByDilationImageFilter
# Start of ReconstructionByDilation

`ReconstructionByDilation__SWIG_0` = function(markerImage, maskImage, fullyConnected, useInternalCopy, .copy = FALSE)
{
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  useInternalCopy = as.logical(useInternalCopy);
  ;ans = .Call('R_swig_ReconstructionByDilation__SWIG_0', markerImage, maskImage, fullyConnected, useInternalCopy, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReconstructionByDilation__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ReconstructionByDilation__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical', 'logical')
class(`ReconstructionByDilation__SWIG_0`) = c("SWIGFunction", class('ReconstructionByDilation__SWIG_0'))

# Start of ReconstructionByDilation

`ReconstructionByDilation__SWIG_1` = function(markerImage, maskImage, fullyConnected, .copy = FALSE)
{
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_ReconstructionByDilation__SWIG_1', markerImage, maskImage, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReconstructionByDilation__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ReconstructionByDilation__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical')
class(`ReconstructionByDilation__SWIG_1`) = c("SWIGFunction", class('ReconstructionByDilation__SWIG_1'))

# Start of ReconstructionByDilation

`ReconstructionByDilation__SWIG_2` = function(markerImage, maskImage, .copy = FALSE)
{
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_ReconstructionByDilation__SWIG_2', markerImage, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReconstructionByDilation__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ReconstructionByDilation__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ReconstructionByDilation__SWIG_2`) = c("SWIGFunction", class('ReconstructionByDilation__SWIG_2'))

`ReconstructionByDilation` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- ReconstructionByDilation__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ReconstructionByDilation__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ReconstructionByDilation__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ReconstructionByDilation with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ReconstructionByErosionImageFilter

`delete_ReconstructionByErosionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ReconstructionByErosionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ReconstructionByErosionImageFilter`, 'returnType') = 'void'
attr(`delete_ReconstructionByErosionImageFilter`, "inputTypes") = c('_p_itk__simple__ReconstructionByErosionImageFilter')
class(`delete_ReconstructionByErosionImageFilter`) = c("SWIGFunction", class('delete_ReconstructionByErosionImageFilter'))

# Start of new_ReconstructionByErosionImageFilter

`ReconstructionByErosionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ReconstructionByErosionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ReconstructionByErosionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ReconstructionByErosionImageFilter);
  ans
  
}

attr(`ReconstructionByErosionImageFilter`, 'returnType') = '_p_itk__simple__ReconstructionByErosionImageFilter'
class(`ReconstructionByErosionImageFilter`) = c("SWIGFunction", class('ReconstructionByErosionImageFilter'))

# Start of ReconstructionByErosionImageFilter_SetFullyConnected

`ReconstructionByErosionImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_ReconstructionByErosionImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReconstructionByErosionImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`ReconstructionByErosionImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__ReconstructionByErosionImageFilter', 'logical')
class(`ReconstructionByErosionImageFilter_SetFullyConnected`) = c("SWIGFunction", class('ReconstructionByErosionImageFilter_SetFullyConnected'))

# Start of ReconstructionByErosionImageFilter_FullyConnectedOn

`ReconstructionByErosionImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ReconstructionByErosionImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReconstructionByErosionImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`ReconstructionByErosionImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__ReconstructionByErosionImageFilter')
class(`ReconstructionByErosionImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('ReconstructionByErosionImageFilter_FullyConnectedOn'))

# Start of ReconstructionByErosionImageFilter_FullyConnectedOff

`ReconstructionByErosionImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ReconstructionByErosionImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReconstructionByErosionImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`ReconstructionByErosionImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__ReconstructionByErosionImageFilter')
class(`ReconstructionByErosionImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('ReconstructionByErosionImageFilter_FullyConnectedOff'))

# Start of ReconstructionByErosionImageFilter_GetFullyConnected

`ReconstructionByErosionImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ReconstructionByErosionImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ReconstructionByErosionImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`ReconstructionByErosionImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__ReconstructionByErosionImageFilter')
class(`ReconstructionByErosionImageFilter_GetFullyConnected`) = c("SWIGFunction", class('ReconstructionByErosionImageFilter_GetFullyConnected'))

# Start of ReconstructionByErosionImageFilter_SetUseInternalCopy

`ReconstructionByErosionImageFilter_SetUseInternalCopy` = function(self, UseInternalCopy)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseInternalCopy = as.logical(UseInternalCopy);
  ;ans = .Call('R_swig_ReconstructionByErosionImageFilter_SetUseInternalCopy', self, UseInternalCopy, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReconstructionByErosionImageFilter_SetUseInternalCopy`, 'returnType') = 'void'
attr(`ReconstructionByErosionImageFilter_SetUseInternalCopy`, "inputTypes") = c('_p_itk__simple__ReconstructionByErosionImageFilter', 'logical')
class(`ReconstructionByErosionImageFilter_SetUseInternalCopy`) = c("SWIGFunction", class('ReconstructionByErosionImageFilter_SetUseInternalCopy'))

# Start of ReconstructionByErosionImageFilter_UseInternalCopyOn

`ReconstructionByErosionImageFilter_UseInternalCopyOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ReconstructionByErosionImageFilter_UseInternalCopyOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReconstructionByErosionImageFilter_UseInternalCopyOn`, 'returnType') = 'void'
attr(`ReconstructionByErosionImageFilter_UseInternalCopyOn`, "inputTypes") = c('_p_itk__simple__ReconstructionByErosionImageFilter')
class(`ReconstructionByErosionImageFilter_UseInternalCopyOn`) = c("SWIGFunction", class('ReconstructionByErosionImageFilter_UseInternalCopyOn'))

# Start of ReconstructionByErosionImageFilter_UseInternalCopyOff

`ReconstructionByErosionImageFilter_UseInternalCopyOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ReconstructionByErosionImageFilter_UseInternalCopyOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReconstructionByErosionImageFilter_UseInternalCopyOff`, 'returnType') = 'void'
attr(`ReconstructionByErosionImageFilter_UseInternalCopyOff`, "inputTypes") = c('_p_itk__simple__ReconstructionByErosionImageFilter')
class(`ReconstructionByErosionImageFilter_UseInternalCopyOff`) = c("SWIGFunction", class('ReconstructionByErosionImageFilter_UseInternalCopyOff'))

# Start of ReconstructionByErosionImageFilter_GetUseInternalCopy

`ReconstructionByErosionImageFilter_GetUseInternalCopy` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ReconstructionByErosionImageFilter_GetUseInternalCopy', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ReconstructionByErosionImageFilter_GetUseInternalCopy`, 'returnType') = 'logical'
attr(`ReconstructionByErosionImageFilter_GetUseInternalCopy`, "inputTypes") = c('_p_itk__simple__ReconstructionByErosionImageFilter')
class(`ReconstructionByErosionImageFilter_GetUseInternalCopy`) = c("SWIGFunction", class('ReconstructionByErosionImageFilter_GetUseInternalCopy'))

# Start of ReconstructionByErosionImageFilter_GetName

`ReconstructionByErosionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ReconstructionByErosionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ReconstructionByErosionImageFilter_GetName`, 'returnType') = 'character'
attr(`ReconstructionByErosionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ReconstructionByErosionImageFilter')
class(`ReconstructionByErosionImageFilter_GetName`) = c("SWIGFunction", class('ReconstructionByErosionImageFilter_GetName'))

# Start of ReconstructionByErosionImageFilter_ToString

`ReconstructionByErosionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ReconstructionByErosionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ReconstructionByErosionImageFilter_ToString`, 'returnType') = 'character'
attr(`ReconstructionByErosionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ReconstructionByErosionImageFilter')
class(`ReconstructionByErosionImageFilter_ToString`) = c("SWIGFunction", class('ReconstructionByErosionImageFilter_ToString'))

# Start of ReconstructionByErosionImageFilter_Execute

`ReconstructionByErosionImageFilter_Execute` = function(self, markerImage, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_ReconstructionByErosionImageFilter_Execute', self, markerImage, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReconstructionByErosionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ReconstructionByErosionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ReconstructionByErosionImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ReconstructionByErosionImageFilter_Execute`) = c("SWIGFunction", class('ReconstructionByErosionImageFilter_Execute'))

# Start of accessor method for itk::simple::ReconstructionByErosionImageFilter
setMethod('$', '_p_itk__simple__ReconstructionByErosionImageFilter', function(x, name)

{
  accessorFuns = list('SetFullyConnected' = ReconstructionByErosionImageFilter_SetFullyConnected, 'FullyConnectedOn' = ReconstructionByErosionImageFilter_FullyConnectedOn, 'FullyConnectedOff' = ReconstructionByErosionImageFilter_FullyConnectedOff, 'GetFullyConnected' = ReconstructionByErosionImageFilter_GetFullyConnected, 'SetUseInternalCopy' = ReconstructionByErosionImageFilter_SetUseInternalCopy, 'UseInternalCopyOn' = ReconstructionByErosionImageFilter_UseInternalCopyOn, 'UseInternalCopyOff' = ReconstructionByErosionImageFilter_UseInternalCopyOff, 'GetUseInternalCopy' = ReconstructionByErosionImageFilter_GetUseInternalCopy, 'GetName' = ReconstructionByErosionImageFilter_GetName, 'ToString' = ReconstructionByErosionImageFilter_ToString, 'Execute' = ReconstructionByErosionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ReconstructionByErosionImageFilter
# Start of ReconstructionByErosion

`ReconstructionByErosion__SWIG_0` = function(markerImage, maskImage, fullyConnected, useInternalCopy, .copy = FALSE)
{
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  useInternalCopy = as.logical(useInternalCopy);
  ;ans = .Call('R_swig_ReconstructionByErosion__SWIG_0', markerImage, maskImage, fullyConnected, useInternalCopy, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReconstructionByErosion__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ReconstructionByErosion__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical', 'logical')
class(`ReconstructionByErosion__SWIG_0`) = c("SWIGFunction", class('ReconstructionByErosion__SWIG_0'))

# Start of ReconstructionByErosion

`ReconstructionByErosion__SWIG_1` = function(markerImage, maskImage, fullyConnected, .copy = FALSE)
{
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_ReconstructionByErosion__SWIG_1', markerImage, maskImage, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReconstructionByErosion__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ReconstructionByErosion__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'logical')
class(`ReconstructionByErosion__SWIG_1`) = c("SWIGFunction", class('ReconstructionByErosion__SWIG_1'))

# Start of ReconstructionByErosion

`ReconstructionByErosion__SWIG_2` = function(markerImage, maskImage, .copy = FALSE)
{
  if (inherits(markerImage, "ExternalReference")) markerImage = slot(markerImage,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_ReconstructionByErosion__SWIG_2', markerImage, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReconstructionByErosion__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ReconstructionByErosion__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ReconstructionByErosion__SWIG_2`) = c("SWIGFunction", class('ReconstructionByErosion__SWIG_2'))

`ReconstructionByErosion` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- ReconstructionByErosion__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ReconstructionByErosion__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ReconstructionByErosion__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ReconstructionByErosion with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_RecursiveGaussianImageFilter

`delete_RecursiveGaussianImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_RecursiveGaussianImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_RecursiveGaussianImageFilter`, 'returnType') = 'void'
attr(`delete_RecursiveGaussianImageFilter`, "inputTypes") = c('_p_itk__simple__RecursiveGaussianImageFilter')
class(`delete_RecursiveGaussianImageFilter`) = c("SWIGFunction", class('delete_RecursiveGaussianImageFilter'))

# Start of new_RecursiveGaussianImageFilter

`RecursiveGaussianImageFilter` = function()
{
  ;ans = .Call('R_swig_new_RecursiveGaussianImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__RecursiveGaussianImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_RecursiveGaussianImageFilter);
  ans
  
}

attr(`RecursiveGaussianImageFilter`, 'returnType') = '_p_itk__simple__RecursiveGaussianImageFilter'
class(`RecursiveGaussianImageFilter`) = c("SWIGFunction", class('RecursiveGaussianImageFilter'))

# Start of RecursiveGaussianImageFilter_SetSigma

`RecursiveGaussianImageFilter_SetSigma` = function(self, Sigma)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_RecursiveGaussianImageFilter_SetSigma', self, Sigma, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RecursiveGaussianImageFilter_SetSigma`, 'returnType') = 'void'
attr(`RecursiveGaussianImageFilter_SetSigma`, "inputTypes") = c('_p_itk__simple__RecursiveGaussianImageFilter', 'numeric')
class(`RecursiveGaussianImageFilter_SetSigma`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_SetSigma'))

# Start of RecursiveGaussianImageFilter_GetSigma

`RecursiveGaussianImageFilter_GetSigma` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RecursiveGaussianImageFilter_GetSigma', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RecursiveGaussianImageFilter_GetSigma`, 'returnType') = 'numeric'
attr(`RecursiveGaussianImageFilter_GetSigma`, "inputTypes") = c('_p_itk__simple__RecursiveGaussianImageFilter')
class(`RecursiveGaussianImageFilter_GetSigma`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_GetSigma'))

# Start of RecursiveGaussianImageFilter_SetNormalizeAcrossScale

`RecursiveGaussianImageFilter_SetNormalizeAcrossScale` = function(self, NormalizeAcrossScale)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NormalizeAcrossScale = as.logical(NormalizeAcrossScale);
  ;ans = .Call('R_swig_RecursiveGaussianImageFilter_SetNormalizeAcrossScale', self, NormalizeAcrossScale, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RecursiveGaussianImageFilter_SetNormalizeAcrossScale`, 'returnType') = 'void'
attr(`RecursiveGaussianImageFilter_SetNormalizeAcrossScale`, "inputTypes") = c('_p_itk__simple__RecursiveGaussianImageFilter', 'logical')
class(`RecursiveGaussianImageFilter_SetNormalizeAcrossScale`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_SetNormalizeAcrossScale'))

# Start of RecursiveGaussianImageFilter_NormalizeAcrossScaleOn

`RecursiveGaussianImageFilter_NormalizeAcrossScaleOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RecursiveGaussianImageFilter_NormalizeAcrossScaleOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RecursiveGaussianImageFilter_NormalizeAcrossScaleOn`, 'returnType') = 'void'
attr(`RecursiveGaussianImageFilter_NormalizeAcrossScaleOn`, "inputTypes") = c('_p_itk__simple__RecursiveGaussianImageFilter')
class(`RecursiveGaussianImageFilter_NormalizeAcrossScaleOn`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_NormalizeAcrossScaleOn'))

# Start of RecursiveGaussianImageFilter_NormalizeAcrossScaleOff

`RecursiveGaussianImageFilter_NormalizeAcrossScaleOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RecursiveGaussianImageFilter_NormalizeAcrossScaleOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RecursiveGaussianImageFilter_NormalizeAcrossScaleOff`, 'returnType') = 'void'
attr(`RecursiveGaussianImageFilter_NormalizeAcrossScaleOff`, "inputTypes") = c('_p_itk__simple__RecursiveGaussianImageFilter')
class(`RecursiveGaussianImageFilter_NormalizeAcrossScaleOff`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_NormalizeAcrossScaleOff'))

# Start of RecursiveGaussianImageFilter_GetNormalizeAcrossScale

`RecursiveGaussianImageFilter_GetNormalizeAcrossScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RecursiveGaussianImageFilter_GetNormalizeAcrossScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RecursiveGaussianImageFilter_GetNormalizeAcrossScale`, 'returnType') = 'logical'
attr(`RecursiveGaussianImageFilter_GetNormalizeAcrossScale`, "inputTypes") = c('_p_itk__simple__RecursiveGaussianImageFilter')
class(`RecursiveGaussianImageFilter_GetNormalizeAcrossScale`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_GetNormalizeAcrossScale'))

# Start of RecursiveGaussianImageFilter_ZeroOrder_get

`RecursiveGaussianImageFilter_ZeroOrder_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_RecursiveGaussianImageFilter_ZeroOrder_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__RecursiveGaussianImageFilter__OrderType");
  
  ans
  
}

attr(`RecursiveGaussianImageFilter_ZeroOrder_get`, 'returnType') = 'character'
class(`RecursiveGaussianImageFilter_ZeroOrder_get`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_ZeroOrder_get'))

# Start of RecursiveGaussianImageFilter_FirstOrder_get

`RecursiveGaussianImageFilter_FirstOrder_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_RecursiveGaussianImageFilter_FirstOrder_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__RecursiveGaussianImageFilter__OrderType");
  
  ans
  
}

attr(`RecursiveGaussianImageFilter_FirstOrder_get`, 'returnType') = 'character'
class(`RecursiveGaussianImageFilter_FirstOrder_get`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_FirstOrder_get'))

# Start of RecursiveGaussianImageFilter_SecondOrder_get

`RecursiveGaussianImageFilter_SecondOrder_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_RecursiveGaussianImageFilter_SecondOrder_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__RecursiveGaussianImageFilter__OrderType");
  
  ans
  
}

attr(`RecursiveGaussianImageFilter_SecondOrder_get`, 'returnType') = 'character'
class(`RecursiveGaussianImageFilter_SecondOrder_get`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_SecondOrder_get'))

# Start of RecursiveGaussianImageFilter_SetOrder

`RecursiveGaussianImageFilter_SetOrder` = function(self, Order)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Order = enumToInteger(Order, "_itk__simple__RecursiveGaussianImageFilter__OrderType"); 
  
  if(length(Order) > 1) {
    warning("using only the first element of Order");
  };
  
  ;ans = .Call('R_swig_RecursiveGaussianImageFilter_SetOrder', self, Order, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RecursiveGaussianImageFilter_SetOrder`, 'returnType') = 'void'
attr(`RecursiveGaussianImageFilter_SetOrder`, "inputTypes") = c('_p_itk__simple__RecursiveGaussianImageFilter', 'character')
class(`RecursiveGaussianImageFilter_SetOrder`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_SetOrder'))

# Start of RecursiveGaussianImageFilter_GetOrder

`RecursiveGaussianImageFilter_GetOrder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RecursiveGaussianImageFilter_GetOrder', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__RecursiveGaussianImageFilter__OrderType");
  
  ans
  
}

attr(`RecursiveGaussianImageFilter_GetOrder`, 'returnType') = 'character'
attr(`RecursiveGaussianImageFilter_GetOrder`, "inputTypes") = c('_p_itk__simple__RecursiveGaussianImageFilter')
class(`RecursiveGaussianImageFilter_GetOrder`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_GetOrder'))

# Start of RecursiveGaussianImageFilter_SetDirection

`RecursiveGaussianImageFilter_SetDirection` = function(self, Direction)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Direction = as.integer(Direction);
  
  if(length(Direction) > 1) {
    warning("using only the first element of Direction");
  };
  
  ;ans = .Call('R_swig_RecursiveGaussianImageFilter_SetDirection', self, Direction, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RecursiveGaussianImageFilter_SetDirection`, 'returnType') = 'void'
attr(`RecursiveGaussianImageFilter_SetDirection`, "inputTypes") = c('_p_itk__simple__RecursiveGaussianImageFilter', 'integer')
class(`RecursiveGaussianImageFilter_SetDirection`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_SetDirection'))

# Start of RecursiveGaussianImageFilter_GetDirection

`RecursiveGaussianImageFilter_GetDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RecursiveGaussianImageFilter_GetDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RecursiveGaussianImageFilter_GetDirection`, 'returnType') = 'integer'
attr(`RecursiveGaussianImageFilter_GetDirection`, "inputTypes") = c('_p_itk__simple__RecursiveGaussianImageFilter')
class(`RecursiveGaussianImageFilter_GetDirection`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_GetDirection'))

# Start of RecursiveGaussianImageFilter_GetName

`RecursiveGaussianImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RecursiveGaussianImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RecursiveGaussianImageFilter_GetName`, 'returnType') = 'character'
attr(`RecursiveGaussianImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__RecursiveGaussianImageFilter')
class(`RecursiveGaussianImageFilter_GetName`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_GetName'))

# Start of RecursiveGaussianImageFilter_ToString

`RecursiveGaussianImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RecursiveGaussianImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RecursiveGaussianImageFilter_ToString`, 'returnType') = 'character'
attr(`RecursiveGaussianImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__RecursiveGaussianImageFilter')
class(`RecursiveGaussianImageFilter_ToString`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_ToString'))

# Start of RecursiveGaussianImageFilter_Execute

`RecursiveGaussianImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RecursiveGaussianImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RecursiveGaussianImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`RecursiveGaussianImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__RecursiveGaussianImageFilter', '_p_itk__simple__Image')
class(`RecursiveGaussianImageFilter_Execute`) = c("SWIGFunction", class('RecursiveGaussianImageFilter_Execute'))

# Start of accessor method for itk::simple::RecursiveGaussianImageFilter
setMethod('$', '_p_itk__simple__RecursiveGaussianImageFilter', function(x, name)

{
  accessorFuns = list('SetSigma' = RecursiveGaussianImageFilter_SetSigma, 'GetSigma' = RecursiveGaussianImageFilter_GetSigma, 'SetNormalizeAcrossScale' = RecursiveGaussianImageFilter_SetNormalizeAcrossScale, 'NormalizeAcrossScaleOn' = RecursiveGaussianImageFilter_NormalizeAcrossScaleOn, 'NormalizeAcrossScaleOff' = RecursiveGaussianImageFilter_NormalizeAcrossScaleOff, 'GetNormalizeAcrossScale' = RecursiveGaussianImageFilter_GetNormalizeAcrossScale, 'SetOrder' = RecursiveGaussianImageFilter_SetOrder, 'GetOrder' = RecursiveGaussianImageFilter_GetOrder, 'SetDirection' = RecursiveGaussianImageFilter_SetDirection, 'GetDirection' = RecursiveGaussianImageFilter_GetDirection, 'GetName' = RecursiveGaussianImageFilter_GetName, 'ToString' = RecursiveGaussianImageFilter_ToString, 'Execute' = RecursiveGaussianImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::RecursiveGaussianImageFilter
# Start of RecursiveGaussian

`RecursiveGaussian__SWIG_0` = function(image1, sigma, normalizeAcrossScale, order, direction, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  normalizeAcrossScale = as.logical(normalizeAcrossScale);
  order = enumToInteger(order, "_itk__simple__RecursiveGaussianImageFilter__OrderType"); 
  
  if(length(order) > 1) {
    warning("using only the first element of order");
  };
  
  direction = as.integer(direction);
  
  if(length(direction) > 1) {
    warning("using only the first element of direction");
  };
  
  ;ans = .Call('R_swig_RecursiveGaussian__SWIG_0', image1, sigma, normalizeAcrossScale, order, direction, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RecursiveGaussian__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`RecursiveGaussian__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical', 'character', 'integer')
class(`RecursiveGaussian__SWIG_0`) = c("SWIGFunction", class('RecursiveGaussian__SWIG_0'))

# Start of RecursiveGaussian

`RecursiveGaussian__SWIG_1` = function(image1, sigma, normalizeAcrossScale, order, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  normalizeAcrossScale = as.logical(normalizeAcrossScale);
  order = enumToInteger(order, "_itk__simple__RecursiveGaussianImageFilter__OrderType"); 
  
  if(length(order) > 1) {
    warning("using only the first element of order");
  };
  
  ;ans = .Call('R_swig_RecursiveGaussian__SWIG_1', image1, sigma, normalizeAcrossScale, order, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RecursiveGaussian__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`RecursiveGaussian__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical', 'character')
class(`RecursiveGaussian__SWIG_1`) = c("SWIGFunction", class('RecursiveGaussian__SWIG_1'))

# Start of RecursiveGaussian

`RecursiveGaussian__SWIG_2` = function(image1, sigma, normalizeAcrossScale, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  normalizeAcrossScale = as.logical(normalizeAcrossScale);
  ;ans = .Call('R_swig_RecursiveGaussian__SWIG_2', image1, sigma, normalizeAcrossScale, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RecursiveGaussian__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`RecursiveGaussian__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`RecursiveGaussian__SWIG_2`) = c("SWIGFunction", class('RecursiveGaussian__SWIG_2'))

# Start of RecursiveGaussian

`RecursiveGaussian__SWIG_3` = function(image1, sigma, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_RecursiveGaussian__SWIG_3', image1, sigma, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RecursiveGaussian__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`RecursiveGaussian__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`RecursiveGaussian__SWIG_3`) = c("SWIGFunction", class('RecursiveGaussian__SWIG_3'))

# Start of RecursiveGaussian

`RecursiveGaussian__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RecursiveGaussian__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RecursiveGaussian__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`RecursiveGaussian__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`RecursiveGaussian__SWIG_4`) = c("SWIGFunction", class('RecursiveGaussian__SWIG_4'))

`RecursiveGaussian` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- RecursiveGaussian__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- RecursiveGaussian__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- RecursiveGaussian__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- RecursiveGaussian__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- RecursiveGaussian__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for RecursiveGaussian with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_RegionOfInterestImageFilter

`delete_RegionOfInterestImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_RegionOfInterestImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_RegionOfInterestImageFilter`, 'returnType') = 'void'
attr(`delete_RegionOfInterestImageFilter`, "inputTypes") = c('_p_itk__simple__RegionOfInterestImageFilter')
class(`delete_RegionOfInterestImageFilter`) = c("SWIGFunction", class('delete_RegionOfInterestImageFilter'))

# Start of new_RegionOfInterestImageFilter

`RegionOfInterestImageFilter` = function()
{
  ;ans = .Call('R_swig_new_RegionOfInterestImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__RegionOfInterestImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_RegionOfInterestImageFilter);
  ans
  
}

attr(`RegionOfInterestImageFilter`, 'returnType') = '_p_itk__simple__RegionOfInterestImageFilter'
class(`RegionOfInterestImageFilter`) = c("SWIGFunction", class('RegionOfInterestImageFilter'))

# Start of RegionOfInterestImageFilter_SetSize

`RegionOfInterestImageFilter_SetSize` = function(self, Size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Size = as.integer(Size);
  ;ans = .Call('R_swig_RegionOfInterestImageFilter_SetSize', self, Size, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionOfInterestImageFilter_SetSize`, 'returnType') = 'void'
attr(`RegionOfInterestImageFilter_SetSize`, "inputTypes") = c('_p_itk__simple__RegionOfInterestImageFilter', 'integer')
class(`RegionOfInterestImageFilter_SetSize`) = c("SWIGFunction", class('RegionOfInterestImageFilter_SetSize'))

# Start of RegionOfInterestImageFilter_GetSize

`RegionOfInterestImageFilter_GetSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionOfInterestImageFilter_GetSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionOfInterestImageFilter_GetSize`, 'returnType') = 'integer'
attr(`RegionOfInterestImageFilter_GetSize`, "inputTypes") = c('_p_itk__simple__RegionOfInterestImageFilter')
class(`RegionOfInterestImageFilter_GetSize`) = c("SWIGFunction", class('RegionOfInterestImageFilter_GetSize'))

# Start of RegionOfInterestImageFilter_SetIndex

`RegionOfInterestImageFilter_SetIndex` = function(self, Index)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Index = as.integer(Index);
  ;ans = .Call('R_swig_RegionOfInterestImageFilter_SetIndex', self, Index, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionOfInterestImageFilter_SetIndex`, 'returnType') = 'void'
attr(`RegionOfInterestImageFilter_SetIndex`, "inputTypes") = c('_p_itk__simple__RegionOfInterestImageFilter', 'integer')
class(`RegionOfInterestImageFilter_SetIndex`) = c("SWIGFunction", class('RegionOfInterestImageFilter_SetIndex'))

# Start of RegionOfInterestImageFilter_GetIndex

`RegionOfInterestImageFilter_GetIndex` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionOfInterestImageFilter_GetIndex', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionOfInterestImageFilter_GetIndex`, 'returnType') = 'integer'
attr(`RegionOfInterestImageFilter_GetIndex`, "inputTypes") = c('_p_itk__simple__RegionOfInterestImageFilter')
class(`RegionOfInterestImageFilter_GetIndex`) = c("SWIGFunction", class('RegionOfInterestImageFilter_GetIndex'))

# Start of RegionOfInterestImageFilter_GetName

`RegionOfInterestImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionOfInterestImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionOfInterestImageFilter_GetName`, 'returnType') = 'character'
attr(`RegionOfInterestImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__RegionOfInterestImageFilter')
class(`RegionOfInterestImageFilter_GetName`) = c("SWIGFunction", class('RegionOfInterestImageFilter_GetName'))

# Start of RegionOfInterestImageFilter_ToString

`RegionOfInterestImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionOfInterestImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionOfInterestImageFilter_ToString`, 'returnType') = 'character'
attr(`RegionOfInterestImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__RegionOfInterestImageFilter')
class(`RegionOfInterestImageFilter_ToString`) = c("SWIGFunction", class('RegionOfInterestImageFilter_ToString'))

# Start of RegionOfInterestImageFilter_Execute

`RegionOfInterestImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RegionOfInterestImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionOfInterestImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionOfInterestImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__RegionOfInterestImageFilter', '_p_itk__simple__Image')
class(`RegionOfInterestImageFilter_Execute`) = c("SWIGFunction", class('RegionOfInterestImageFilter_Execute'))

# Start of RegionOfInterestImageFilter_SetRegionOfInterest

`RegionOfInterestImageFilter_SetRegionOfInterest` = function(self, Region)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Region = as.integer(Region);
  ;ans = .Call('R_swig_RegionOfInterestImageFilter_SetRegionOfInterest', self, Region, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionOfInterestImageFilter_SetRegionOfInterest`, 'returnType') = 'void'
attr(`RegionOfInterestImageFilter_SetRegionOfInterest`, "inputTypes") = c('_p_itk__simple__RegionOfInterestImageFilter', 'integer')
class(`RegionOfInterestImageFilter_SetRegionOfInterest`) = c("SWIGFunction", class('RegionOfInterestImageFilter_SetRegionOfInterest'))

# Start of accessor method for itk::simple::RegionOfInterestImageFilter
setMethod('$', '_p_itk__simple__RegionOfInterestImageFilter', function(x, name)

{
  accessorFuns = list('SetSize' = RegionOfInterestImageFilter_SetSize, 'GetSize' = RegionOfInterestImageFilter_GetSize, 'SetIndex' = RegionOfInterestImageFilter_SetIndex, 'GetIndex' = RegionOfInterestImageFilter_GetIndex, 'GetName' = RegionOfInterestImageFilter_GetName, 'ToString' = RegionOfInterestImageFilter_ToString, 'Execute' = RegionOfInterestImageFilter_Execute, 'SetRegionOfInterest' = RegionOfInterestImageFilter_SetRegionOfInterest);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::RegionOfInterestImageFilter
# Start of RegionOfInterest

`RegionOfInterest__SWIG_0` = function(image1, size, index, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  index = as.integer(index);
  ;ans = .Call('R_swig_RegionOfInterest__SWIG_0', image1, size, index, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionOfInterest__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionOfInterest__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`RegionOfInterest__SWIG_0`) = c("SWIGFunction", class('RegionOfInterest__SWIG_0'))

# Start of RegionOfInterest

`RegionOfInterest__SWIG_1` = function(image1, size, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  size = as.integer(size);
  ;ans = .Call('R_swig_RegionOfInterest__SWIG_1', image1, size, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionOfInterest__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionOfInterest__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`RegionOfInterest__SWIG_1`) = c("SWIGFunction", class('RegionOfInterest__SWIG_1'))

# Start of RegionOfInterest

`RegionOfInterest__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RegionOfInterest__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionOfInterest__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionOfInterest__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`RegionOfInterest__SWIG_2`) = c("SWIGFunction", class('RegionOfInterest__SWIG_2'))

`RegionOfInterest` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- RegionOfInterest__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- RegionOfInterest__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- RegionOfInterest__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for RegionOfInterest with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_RegionalMaximaImageFilter

`delete_RegionalMaximaImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_RegionalMaximaImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_RegionalMaximaImageFilter`, 'returnType') = 'void'
attr(`delete_RegionalMaximaImageFilter`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter')
class(`delete_RegionalMaximaImageFilter`) = c("SWIGFunction", class('delete_RegionalMaximaImageFilter'))

# Start of new_RegionalMaximaImageFilter

`RegionalMaximaImageFilter` = function()
{
  ;ans = .Call('R_swig_new_RegionalMaximaImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__RegionalMaximaImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_RegionalMaximaImageFilter);
  ans
  
}

attr(`RegionalMaximaImageFilter`, 'returnType') = '_p_itk__simple__RegionalMaximaImageFilter'
class(`RegionalMaximaImageFilter`) = c("SWIGFunction", class('RegionalMaximaImageFilter'))

# Start of RegionalMaximaImageFilter_SetBackgroundValue

`RegionalMaximaImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_RegionalMaximaImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMaximaImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`RegionalMaximaImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter', 'numeric')
class(`RegionalMaximaImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('RegionalMaximaImageFilter_SetBackgroundValue'))

# Start of RegionalMaximaImageFilter_GetBackgroundValue

`RegionalMaximaImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionalMaximaImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionalMaximaImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`RegionalMaximaImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter')
class(`RegionalMaximaImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('RegionalMaximaImageFilter_GetBackgroundValue'))

# Start of RegionalMaximaImageFilter_SetForegroundValue

`RegionalMaximaImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_RegionalMaximaImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMaximaImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`RegionalMaximaImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter', 'numeric')
class(`RegionalMaximaImageFilter_SetForegroundValue`) = c("SWIGFunction", class('RegionalMaximaImageFilter_SetForegroundValue'))

# Start of RegionalMaximaImageFilter_GetForegroundValue

`RegionalMaximaImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionalMaximaImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionalMaximaImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`RegionalMaximaImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter')
class(`RegionalMaximaImageFilter_GetForegroundValue`) = c("SWIGFunction", class('RegionalMaximaImageFilter_GetForegroundValue'))

# Start of RegionalMaximaImageFilter_SetFullyConnected

`RegionalMaximaImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_RegionalMaximaImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMaximaImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`RegionalMaximaImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter', 'logical')
class(`RegionalMaximaImageFilter_SetFullyConnected`) = c("SWIGFunction", class('RegionalMaximaImageFilter_SetFullyConnected'))

# Start of RegionalMaximaImageFilter_FullyConnectedOn

`RegionalMaximaImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RegionalMaximaImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMaximaImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`RegionalMaximaImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter')
class(`RegionalMaximaImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('RegionalMaximaImageFilter_FullyConnectedOn'))

# Start of RegionalMaximaImageFilter_FullyConnectedOff

`RegionalMaximaImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RegionalMaximaImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMaximaImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`RegionalMaximaImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter')
class(`RegionalMaximaImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('RegionalMaximaImageFilter_FullyConnectedOff'))

# Start of RegionalMaximaImageFilter_GetFullyConnected

`RegionalMaximaImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionalMaximaImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionalMaximaImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`RegionalMaximaImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter')
class(`RegionalMaximaImageFilter_GetFullyConnected`) = c("SWIGFunction", class('RegionalMaximaImageFilter_GetFullyConnected'))

# Start of RegionalMaximaImageFilter_SetFlatIsMaxima

`RegionalMaximaImageFilter_SetFlatIsMaxima` = function(self, FlatIsMaxima)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FlatIsMaxima = as.logical(FlatIsMaxima);
  ;ans = .Call('R_swig_RegionalMaximaImageFilter_SetFlatIsMaxima', self, FlatIsMaxima, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMaximaImageFilter_SetFlatIsMaxima`, 'returnType') = 'void'
attr(`RegionalMaximaImageFilter_SetFlatIsMaxima`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter', 'logical')
class(`RegionalMaximaImageFilter_SetFlatIsMaxima`) = c("SWIGFunction", class('RegionalMaximaImageFilter_SetFlatIsMaxima'))

# Start of RegionalMaximaImageFilter_FlatIsMaximaOn

`RegionalMaximaImageFilter_FlatIsMaximaOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RegionalMaximaImageFilter_FlatIsMaximaOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMaximaImageFilter_FlatIsMaximaOn`, 'returnType') = 'void'
attr(`RegionalMaximaImageFilter_FlatIsMaximaOn`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter')
class(`RegionalMaximaImageFilter_FlatIsMaximaOn`) = c("SWIGFunction", class('RegionalMaximaImageFilter_FlatIsMaximaOn'))

# Start of RegionalMaximaImageFilter_FlatIsMaximaOff

`RegionalMaximaImageFilter_FlatIsMaximaOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RegionalMaximaImageFilter_FlatIsMaximaOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMaximaImageFilter_FlatIsMaximaOff`, 'returnType') = 'void'
attr(`RegionalMaximaImageFilter_FlatIsMaximaOff`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter')
class(`RegionalMaximaImageFilter_FlatIsMaximaOff`) = c("SWIGFunction", class('RegionalMaximaImageFilter_FlatIsMaximaOff'))

# Start of RegionalMaximaImageFilter_GetFlatIsMaxima

`RegionalMaximaImageFilter_GetFlatIsMaxima` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionalMaximaImageFilter_GetFlatIsMaxima', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionalMaximaImageFilter_GetFlatIsMaxima`, 'returnType') = 'logical'
attr(`RegionalMaximaImageFilter_GetFlatIsMaxima`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter')
class(`RegionalMaximaImageFilter_GetFlatIsMaxima`) = c("SWIGFunction", class('RegionalMaximaImageFilter_GetFlatIsMaxima'))

# Start of RegionalMaximaImageFilter_GetName

`RegionalMaximaImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionalMaximaImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionalMaximaImageFilter_GetName`, 'returnType') = 'character'
attr(`RegionalMaximaImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter')
class(`RegionalMaximaImageFilter_GetName`) = c("SWIGFunction", class('RegionalMaximaImageFilter_GetName'))

# Start of RegionalMaximaImageFilter_ToString

`RegionalMaximaImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionalMaximaImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionalMaximaImageFilter_ToString`, 'returnType') = 'character'
attr(`RegionalMaximaImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter')
class(`RegionalMaximaImageFilter_ToString`) = c("SWIGFunction", class('RegionalMaximaImageFilter_ToString'))

# Start of RegionalMaximaImageFilter_Execute

`RegionalMaximaImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RegionalMaximaImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionalMaximaImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionalMaximaImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__RegionalMaximaImageFilter', '_p_itk__simple__Image')
class(`RegionalMaximaImageFilter_Execute`) = c("SWIGFunction", class('RegionalMaximaImageFilter_Execute'))

# Start of accessor method for itk::simple::RegionalMaximaImageFilter
setMethod('$', '_p_itk__simple__RegionalMaximaImageFilter', function(x, name)

{
  accessorFuns = list('SetBackgroundValue' = RegionalMaximaImageFilter_SetBackgroundValue, 'GetBackgroundValue' = RegionalMaximaImageFilter_GetBackgroundValue, 'SetForegroundValue' = RegionalMaximaImageFilter_SetForegroundValue, 'GetForegroundValue' = RegionalMaximaImageFilter_GetForegroundValue, 'SetFullyConnected' = RegionalMaximaImageFilter_SetFullyConnected, 'FullyConnectedOn' = RegionalMaximaImageFilter_FullyConnectedOn, 'FullyConnectedOff' = RegionalMaximaImageFilter_FullyConnectedOff, 'GetFullyConnected' = RegionalMaximaImageFilter_GetFullyConnected, 'SetFlatIsMaxima' = RegionalMaximaImageFilter_SetFlatIsMaxima, 'FlatIsMaximaOn' = RegionalMaximaImageFilter_FlatIsMaximaOn, 'FlatIsMaximaOff' = RegionalMaximaImageFilter_FlatIsMaximaOff, 'GetFlatIsMaxima' = RegionalMaximaImageFilter_GetFlatIsMaxima, 'GetName' = RegionalMaximaImageFilter_GetName, 'ToString' = RegionalMaximaImageFilter_ToString, 'Execute' = RegionalMaximaImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::RegionalMaximaImageFilter
# Start of RegionalMaxima

`RegionalMaxima__SWIG_0` = function(image1, backgroundValue, foregroundValue, fullyConnected, flatIsMaxima, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  fullyConnected = as.logical(fullyConnected);
  flatIsMaxima = as.logical(flatIsMaxima);
  ;ans = .Call('R_swig_RegionalMaxima__SWIG_0', image1, backgroundValue, foregroundValue, fullyConnected, flatIsMaxima, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionalMaxima__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionalMaxima__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'logical', 'logical')
class(`RegionalMaxima__SWIG_0`) = c("SWIGFunction", class('RegionalMaxima__SWIG_0'))

# Start of RegionalMaxima

`RegionalMaxima__SWIG_1` = function(image1, backgroundValue, foregroundValue, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_RegionalMaxima__SWIG_1', image1, backgroundValue, foregroundValue, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionalMaxima__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionalMaxima__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'logical')
class(`RegionalMaxima__SWIG_1`) = c("SWIGFunction", class('RegionalMaxima__SWIG_1'))

# Start of RegionalMaxima

`RegionalMaxima__SWIG_2` = function(image1, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_RegionalMaxima__SWIG_2', image1, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionalMaxima__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionalMaxima__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`RegionalMaxima__SWIG_2`) = c("SWIGFunction", class('RegionalMaxima__SWIG_2'))

# Start of RegionalMaxima

`RegionalMaxima__SWIG_3` = function(image1, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_RegionalMaxima__SWIG_3', image1, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionalMaxima__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionalMaxima__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`RegionalMaxima__SWIG_3`) = c("SWIGFunction", class('RegionalMaxima__SWIG_3'))

# Start of RegionalMaxima

`RegionalMaxima__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RegionalMaxima__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionalMaxima__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionalMaxima__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`RegionalMaxima__SWIG_4`) = c("SWIGFunction", class('RegionalMaxima__SWIG_4'))

`RegionalMaxima` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- RegionalMaxima__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- RegionalMaxima__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- RegionalMaxima__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- RegionalMaxima__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- RegionalMaxima__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for RegionalMaxima with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_RegionalMinimaImageFilter

`delete_RegionalMinimaImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_RegionalMinimaImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_RegionalMinimaImageFilter`, 'returnType') = 'void'
attr(`delete_RegionalMinimaImageFilter`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter')
class(`delete_RegionalMinimaImageFilter`) = c("SWIGFunction", class('delete_RegionalMinimaImageFilter'))

# Start of new_RegionalMinimaImageFilter

`RegionalMinimaImageFilter` = function()
{
  ;ans = .Call('R_swig_new_RegionalMinimaImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__RegionalMinimaImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_RegionalMinimaImageFilter);
  ans
  
}

attr(`RegionalMinimaImageFilter`, 'returnType') = '_p_itk__simple__RegionalMinimaImageFilter'
class(`RegionalMinimaImageFilter`) = c("SWIGFunction", class('RegionalMinimaImageFilter'))

# Start of RegionalMinimaImageFilter_SetBackgroundValue

`RegionalMinimaImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_RegionalMinimaImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMinimaImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`RegionalMinimaImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter', 'numeric')
class(`RegionalMinimaImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('RegionalMinimaImageFilter_SetBackgroundValue'))

# Start of RegionalMinimaImageFilter_GetBackgroundValue

`RegionalMinimaImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionalMinimaImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionalMinimaImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`RegionalMinimaImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter')
class(`RegionalMinimaImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('RegionalMinimaImageFilter_GetBackgroundValue'))

# Start of RegionalMinimaImageFilter_SetForegroundValue

`RegionalMinimaImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_RegionalMinimaImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMinimaImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`RegionalMinimaImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter', 'numeric')
class(`RegionalMinimaImageFilter_SetForegroundValue`) = c("SWIGFunction", class('RegionalMinimaImageFilter_SetForegroundValue'))

# Start of RegionalMinimaImageFilter_GetForegroundValue

`RegionalMinimaImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionalMinimaImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionalMinimaImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`RegionalMinimaImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter')
class(`RegionalMinimaImageFilter_GetForegroundValue`) = c("SWIGFunction", class('RegionalMinimaImageFilter_GetForegroundValue'))

# Start of RegionalMinimaImageFilter_SetFullyConnected

`RegionalMinimaImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_RegionalMinimaImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMinimaImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`RegionalMinimaImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter', 'logical')
class(`RegionalMinimaImageFilter_SetFullyConnected`) = c("SWIGFunction", class('RegionalMinimaImageFilter_SetFullyConnected'))

# Start of RegionalMinimaImageFilter_FullyConnectedOn

`RegionalMinimaImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RegionalMinimaImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMinimaImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`RegionalMinimaImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter')
class(`RegionalMinimaImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('RegionalMinimaImageFilter_FullyConnectedOn'))

# Start of RegionalMinimaImageFilter_FullyConnectedOff

`RegionalMinimaImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RegionalMinimaImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMinimaImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`RegionalMinimaImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter')
class(`RegionalMinimaImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('RegionalMinimaImageFilter_FullyConnectedOff'))

# Start of RegionalMinimaImageFilter_GetFullyConnected

`RegionalMinimaImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionalMinimaImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionalMinimaImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`RegionalMinimaImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter')
class(`RegionalMinimaImageFilter_GetFullyConnected`) = c("SWIGFunction", class('RegionalMinimaImageFilter_GetFullyConnected'))

# Start of RegionalMinimaImageFilter_SetFlatIsMinima

`RegionalMinimaImageFilter_SetFlatIsMinima` = function(self, FlatIsMinima)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FlatIsMinima = as.logical(FlatIsMinima);
  ;ans = .Call('R_swig_RegionalMinimaImageFilter_SetFlatIsMinima', self, FlatIsMinima, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMinimaImageFilter_SetFlatIsMinima`, 'returnType') = 'void'
attr(`RegionalMinimaImageFilter_SetFlatIsMinima`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter', 'logical')
class(`RegionalMinimaImageFilter_SetFlatIsMinima`) = c("SWIGFunction", class('RegionalMinimaImageFilter_SetFlatIsMinima'))

# Start of RegionalMinimaImageFilter_FlatIsMinimaOn

`RegionalMinimaImageFilter_FlatIsMinimaOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RegionalMinimaImageFilter_FlatIsMinimaOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMinimaImageFilter_FlatIsMinimaOn`, 'returnType') = 'void'
attr(`RegionalMinimaImageFilter_FlatIsMinimaOn`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter')
class(`RegionalMinimaImageFilter_FlatIsMinimaOn`) = c("SWIGFunction", class('RegionalMinimaImageFilter_FlatIsMinimaOn'))

# Start of RegionalMinimaImageFilter_FlatIsMinimaOff

`RegionalMinimaImageFilter_FlatIsMinimaOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RegionalMinimaImageFilter_FlatIsMinimaOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RegionalMinimaImageFilter_FlatIsMinimaOff`, 'returnType') = 'void'
attr(`RegionalMinimaImageFilter_FlatIsMinimaOff`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter')
class(`RegionalMinimaImageFilter_FlatIsMinimaOff`) = c("SWIGFunction", class('RegionalMinimaImageFilter_FlatIsMinimaOff'))

# Start of RegionalMinimaImageFilter_GetFlatIsMinima

`RegionalMinimaImageFilter_GetFlatIsMinima` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionalMinimaImageFilter_GetFlatIsMinima', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionalMinimaImageFilter_GetFlatIsMinima`, 'returnType') = 'logical'
attr(`RegionalMinimaImageFilter_GetFlatIsMinima`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter')
class(`RegionalMinimaImageFilter_GetFlatIsMinima`) = c("SWIGFunction", class('RegionalMinimaImageFilter_GetFlatIsMinima'))

# Start of RegionalMinimaImageFilter_GetName

`RegionalMinimaImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionalMinimaImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionalMinimaImageFilter_GetName`, 'returnType') = 'character'
attr(`RegionalMinimaImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter')
class(`RegionalMinimaImageFilter_GetName`) = c("SWIGFunction", class('RegionalMinimaImageFilter_GetName'))

# Start of RegionalMinimaImageFilter_ToString

`RegionalMinimaImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RegionalMinimaImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RegionalMinimaImageFilter_ToString`, 'returnType') = 'character'
attr(`RegionalMinimaImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter')
class(`RegionalMinimaImageFilter_ToString`) = c("SWIGFunction", class('RegionalMinimaImageFilter_ToString'))

# Start of RegionalMinimaImageFilter_Execute

`RegionalMinimaImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RegionalMinimaImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionalMinimaImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionalMinimaImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__RegionalMinimaImageFilter', '_p_itk__simple__Image')
class(`RegionalMinimaImageFilter_Execute`) = c("SWIGFunction", class('RegionalMinimaImageFilter_Execute'))

# Start of accessor method for itk::simple::RegionalMinimaImageFilter
setMethod('$', '_p_itk__simple__RegionalMinimaImageFilter', function(x, name)

{
  accessorFuns = list('SetBackgroundValue' = RegionalMinimaImageFilter_SetBackgroundValue, 'GetBackgroundValue' = RegionalMinimaImageFilter_GetBackgroundValue, 'SetForegroundValue' = RegionalMinimaImageFilter_SetForegroundValue, 'GetForegroundValue' = RegionalMinimaImageFilter_GetForegroundValue, 'SetFullyConnected' = RegionalMinimaImageFilter_SetFullyConnected, 'FullyConnectedOn' = RegionalMinimaImageFilter_FullyConnectedOn, 'FullyConnectedOff' = RegionalMinimaImageFilter_FullyConnectedOff, 'GetFullyConnected' = RegionalMinimaImageFilter_GetFullyConnected, 'SetFlatIsMinima' = RegionalMinimaImageFilter_SetFlatIsMinima, 'FlatIsMinimaOn' = RegionalMinimaImageFilter_FlatIsMinimaOn, 'FlatIsMinimaOff' = RegionalMinimaImageFilter_FlatIsMinimaOff, 'GetFlatIsMinima' = RegionalMinimaImageFilter_GetFlatIsMinima, 'GetName' = RegionalMinimaImageFilter_GetName, 'ToString' = RegionalMinimaImageFilter_ToString, 'Execute' = RegionalMinimaImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::RegionalMinimaImageFilter
# Start of RegionalMinima

`RegionalMinima__SWIG_0` = function(image1, backgroundValue, foregroundValue, fullyConnected, flatIsMinima, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  fullyConnected = as.logical(fullyConnected);
  flatIsMinima = as.logical(flatIsMinima);
  ;ans = .Call('R_swig_RegionalMinima__SWIG_0', image1, backgroundValue, foregroundValue, fullyConnected, flatIsMinima, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionalMinima__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionalMinima__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'logical', 'logical')
class(`RegionalMinima__SWIG_0`) = c("SWIGFunction", class('RegionalMinima__SWIG_0'))

# Start of RegionalMinima

`RegionalMinima__SWIG_1` = function(image1, backgroundValue, foregroundValue, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_RegionalMinima__SWIG_1', image1, backgroundValue, foregroundValue, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionalMinima__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionalMinima__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'logical')
class(`RegionalMinima__SWIG_1`) = c("SWIGFunction", class('RegionalMinima__SWIG_1'))

# Start of RegionalMinima

`RegionalMinima__SWIG_2` = function(image1, backgroundValue, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_RegionalMinima__SWIG_2', image1, backgroundValue, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionalMinima__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionalMinima__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`RegionalMinima__SWIG_2`) = c("SWIGFunction", class('RegionalMinima__SWIG_2'))

# Start of RegionalMinima

`RegionalMinima__SWIG_3` = function(image1, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_RegionalMinima__SWIG_3', image1, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionalMinima__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionalMinima__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`RegionalMinima__SWIG_3`) = c("SWIGFunction", class('RegionalMinima__SWIG_3'))

# Start of RegionalMinima

`RegionalMinima__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RegionalMinima__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RegionalMinima__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`RegionalMinima__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`RegionalMinima__SWIG_4`) = c("SWIGFunction", class('RegionalMinima__SWIG_4'))

`RegionalMinima` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- RegionalMinima__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- RegionalMinima__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- RegionalMinima__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- RegionalMinima__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- RegionalMinima__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for RegionalMinima with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ReinitializeLevelSetImageFilter

`delete_ReinitializeLevelSetImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ReinitializeLevelSetImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ReinitializeLevelSetImageFilter`, 'returnType') = 'void'
attr(`delete_ReinitializeLevelSetImageFilter`, "inputTypes") = c('_p_itk__simple__ReinitializeLevelSetImageFilter')
class(`delete_ReinitializeLevelSetImageFilter`) = c("SWIGFunction", class('delete_ReinitializeLevelSetImageFilter'))

# Start of new_ReinitializeLevelSetImageFilter

`ReinitializeLevelSetImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ReinitializeLevelSetImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ReinitializeLevelSetImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ReinitializeLevelSetImageFilter);
  ans
  
}

attr(`ReinitializeLevelSetImageFilter`, 'returnType') = '_p_itk__simple__ReinitializeLevelSetImageFilter'
class(`ReinitializeLevelSetImageFilter`) = c("SWIGFunction", class('ReinitializeLevelSetImageFilter'))

# Start of ReinitializeLevelSetImageFilter_SetLevelSetValue

`ReinitializeLevelSetImageFilter_SetLevelSetValue` = function(self, LevelSetValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ReinitializeLevelSetImageFilter_SetLevelSetValue', self, LevelSetValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReinitializeLevelSetImageFilter_SetLevelSetValue`, 'returnType') = 'void'
attr(`ReinitializeLevelSetImageFilter_SetLevelSetValue`, "inputTypes") = c('_p_itk__simple__ReinitializeLevelSetImageFilter', 'numeric')
class(`ReinitializeLevelSetImageFilter_SetLevelSetValue`) = c("SWIGFunction", class('ReinitializeLevelSetImageFilter_SetLevelSetValue'))

# Start of ReinitializeLevelSetImageFilter_GetLevelSetValue

`ReinitializeLevelSetImageFilter_GetLevelSetValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ReinitializeLevelSetImageFilter_GetLevelSetValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ReinitializeLevelSetImageFilter_GetLevelSetValue`, 'returnType') = 'numeric'
attr(`ReinitializeLevelSetImageFilter_GetLevelSetValue`, "inputTypes") = c('_p_itk__simple__ReinitializeLevelSetImageFilter')
class(`ReinitializeLevelSetImageFilter_GetLevelSetValue`) = c("SWIGFunction", class('ReinitializeLevelSetImageFilter_GetLevelSetValue'))

# Start of ReinitializeLevelSetImageFilter_SetNarrowBanding

`ReinitializeLevelSetImageFilter_SetNarrowBanding` = function(self, NarrowBanding)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NarrowBanding = as.logical(NarrowBanding);
  ;ans = .Call('R_swig_ReinitializeLevelSetImageFilter_SetNarrowBanding', self, NarrowBanding, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReinitializeLevelSetImageFilter_SetNarrowBanding`, 'returnType') = 'void'
attr(`ReinitializeLevelSetImageFilter_SetNarrowBanding`, "inputTypes") = c('_p_itk__simple__ReinitializeLevelSetImageFilter', 'logical')
class(`ReinitializeLevelSetImageFilter_SetNarrowBanding`) = c("SWIGFunction", class('ReinitializeLevelSetImageFilter_SetNarrowBanding'))

# Start of ReinitializeLevelSetImageFilter_NarrowBandingOn

`ReinitializeLevelSetImageFilter_NarrowBandingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ReinitializeLevelSetImageFilter_NarrowBandingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReinitializeLevelSetImageFilter_NarrowBandingOn`, 'returnType') = 'void'
attr(`ReinitializeLevelSetImageFilter_NarrowBandingOn`, "inputTypes") = c('_p_itk__simple__ReinitializeLevelSetImageFilter')
class(`ReinitializeLevelSetImageFilter_NarrowBandingOn`) = c("SWIGFunction", class('ReinitializeLevelSetImageFilter_NarrowBandingOn'))

# Start of ReinitializeLevelSetImageFilter_NarrowBandingOff

`ReinitializeLevelSetImageFilter_NarrowBandingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ReinitializeLevelSetImageFilter_NarrowBandingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReinitializeLevelSetImageFilter_NarrowBandingOff`, 'returnType') = 'void'
attr(`ReinitializeLevelSetImageFilter_NarrowBandingOff`, "inputTypes") = c('_p_itk__simple__ReinitializeLevelSetImageFilter')
class(`ReinitializeLevelSetImageFilter_NarrowBandingOff`) = c("SWIGFunction", class('ReinitializeLevelSetImageFilter_NarrowBandingOff'))

# Start of ReinitializeLevelSetImageFilter_GetNarrowBanding

`ReinitializeLevelSetImageFilter_GetNarrowBanding` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ReinitializeLevelSetImageFilter_GetNarrowBanding', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ReinitializeLevelSetImageFilter_GetNarrowBanding`, 'returnType') = 'logical'
attr(`ReinitializeLevelSetImageFilter_GetNarrowBanding`, "inputTypes") = c('_p_itk__simple__ReinitializeLevelSetImageFilter')
class(`ReinitializeLevelSetImageFilter_GetNarrowBanding`) = c("SWIGFunction", class('ReinitializeLevelSetImageFilter_GetNarrowBanding'))

# Start of ReinitializeLevelSetImageFilter_SetInputNarrowBandwidth

`ReinitializeLevelSetImageFilter_SetInputNarrowBandwidth` = function(self, InputNarrowBandwidth)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ReinitializeLevelSetImageFilter_SetInputNarrowBandwidth', self, InputNarrowBandwidth, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReinitializeLevelSetImageFilter_SetInputNarrowBandwidth`, 'returnType') = 'void'
attr(`ReinitializeLevelSetImageFilter_SetInputNarrowBandwidth`, "inputTypes") = c('_p_itk__simple__ReinitializeLevelSetImageFilter', 'numeric')
class(`ReinitializeLevelSetImageFilter_SetInputNarrowBandwidth`) = c("SWIGFunction", class('ReinitializeLevelSetImageFilter_SetInputNarrowBandwidth'))

# Start of ReinitializeLevelSetImageFilter_GetInputNarrowBandwidth

`ReinitializeLevelSetImageFilter_GetInputNarrowBandwidth` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ReinitializeLevelSetImageFilter_GetInputNarrowBandwidth', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ReinitializeLevelSetImageFilter_GetInputNarrowBandwidth`, 'returnType') = 'numeric'
attr(`ReinitializeLevelSetImageFilter_GetInputNarrowBandwidth`, "inputTypes") = c('_p_itk__simple__ReinitializeLevelSetImageFilter')
class(`ReinitializeLevelSetImageFilter_GetInputNarrowBandwidth`) = c("SWIGFunction", class('ReinitializeLevelSetImageFilter_GetInputNarrowBandwidth'))

# Start of ReinitializeLevelSetImageFilter_SetOutputNarrowBandwidth

`ReinitializeLevelSetImageFilter_SetOutputNarrowBandwidth` = function(self, OutputNarrowBandwidth)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ReinitializeLevelSetImageFilter_SetOutputNarrowBandwidth', self, OutputNarrowBandwidth, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReinitializeLevelSetImageFilter_SetOutputNarrowBandwidth`, 'returnType') = 'void'
attr(`ReinitializeLevelSetImageFilter_SetOutputNarrowBandwidth`, "inputTypes") = c('_p_itk__simple__ReinitializeLevelSetImageFilter', 'numeric')
class(`ReinitializeLevelSetImageFilter_SetOutputNarrowBandwidth`) = c("SWIGFunction", class('ReinitializeLevelSetImageFilter_SetOutputNarrowBandwidth'))

# Start of ReinitializeLevelSetImageFilter_GetOutputNarrowBandwidth

`ReinitializeLevelSetImageFilter_GetOutputNarrowBandwidth` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ReinitializeLevelSetImageFilter_GetOutputNarrowBandwidth', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ReinitializeLevelSetImageFilter_GetOutputNarrowBandwidth`, 'returnType') = 'numeric'
attr(`ReinitializeLevelSetImageFilter_GetOutputNarrowBandwidth`, "inputTypes") = c('_p_itk__simple__ReinitializeLevelSetImageFilter')
class(`ReinitializeLevelSetImageFilter_GetOutputNarrowBandwidth`) = c("SWIGFunction", class('ReinitializeLevelSetImageFilter_GetOutputNarrowBandwidth'))

# Start of ReinitializeLevelSetImageFilter_GetName

`ReinitializeLevelSetImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ReinitializeLevelSetImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ReinitializeLevelSetImageFilter_GetName`, 'returnType') = 'character'
attr(`ReinitializeLevelSetImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ReinitializeLevelSetImageFilter')
class(`ReinitializeLevelSetImageFilter_GetName`) = c("SWIGFunction", class('ReinitializeLevelSetImageFilter_GetName'))

# Start of ReinitializeLevelSetImageFilter_ToString

`ReinitializeLevelSetImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ReinitializeLevelSetImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ReinitializeLevelSetImageFilter_ToString`, 'returnType') = 'character'
attr(`ReinitializeLevelSetImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ReinitializeLevelSetImageFilter')
class(`ReinitializeLevelSetImageFilter_ToString`) = c("SWIGFunction", class('ReinitializeLevelSetImageFilter_ToString'))

# Start of ReinitializeLevelSetImageFilter_Execute

`ReinitializeLevelSetImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ReinitializeLevelSetImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReinitializeLevelSetImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ReinitializeLevelSetImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ReinitializeLevelSetImageFilter', '_p_itk__simple__Image')
class(`ReinitializeLevelSetImageFilter_Execute`) = c("SWIGFunction", class('ReinitializeLevelSetImageFilter_Execute'))

# Start of ReinitializeLevelSetImageFilter_NarrowBandwidth

`ReinitializeLevelSetImageFilter_NarrowBandwidth` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ReinitializeLevelSetImageFilter_NarrowBandwidth', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ReinitializeLevelSetImageFilter_NarrowBandwidth`, 'returnType') = 'void'
attr(`ReinitializeLevelSetImageFilter_NarrowBandwidth`, "inputTypes") = c('_p_itk__simple__ReinitializeLevelSetImageFilter', 'numeric')
class(`ReinitializeLevelSetImageFilter_NarrowBandwidth`) = c("SWIGFunction", class('ReinitializeLevelSetImageFilter_NarrowBandwidth'))

# Start of accessor method for itk::simple::ReinitializeLevelSetImageFilter
setMethod('$', '_p_itk__simple__ReinitializeLevelSetImageFilter', function(x, name)

{
  accessorFuns = list('SetLevelSetValue' = ReinitializeLevelSetImageFilter_SetLevelSetValue, 'GetLevelSetValue' = ReinitializeLevelSetImageFilter_GetLevelSetValue, 'SetNarrowBanding' = ReinitializeLevelSetImageFilter_SetNarrowBanding, 'NarrowBandingOn' = ReinitializeLevelSetImageFilter_NarrowBandingOn, 'NarrowBandingOff' = ReinitializeLevelSetImageFilter_NarrowBandingOff, 'GetNarrowBanding' = ReinitializeLevelSetImageFilter_GetNarrowBanding, 'SetInputNarrowBandwidth' = ReinitializeLevelSetImageFilter_SetInputNarrowBandwidth, 'GetInputNarrowBandwidth' = ReinitializeLevelSetImageFilter_GetInputNarrowBandwidth, 'SetOutputNarrowBandwidth' = ReinitializeLevelSetImageFilter_SetOutputNarrowBandwidth, 'GetOutputNarrowBandwidth' = ReinitializeLevelSetImageFilter_GetOutputNarrowBandwidth, 'GetName' = ReinitializeLevelSetImageFilter_GetName, 'ToString' = ReinitializeLevelSetImageFilter_ToString, 'Execute' = ReinitializeLevelSetImageFilter_Execute, 'NarrowBandwidth' = ReinitializeLevelSetImageFilter_NarrowBandwidth);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ReinitializeLevelSetImageFilter
# Start of ReinitializeLevelSet

`ReinitializeLevelSet__SWIG_0` = function(image1, levelSetValue, narrowBanding, inputNarrowBandwidth, outputNarrowBandwidth, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  narrowBanding = as.logical(narrowBanding);
  
  
  ;ans = .Call('R_swig_ReinitializeLevelSet__SWIG_0', image1, levelSetValue, narrowBanding, inputNarrowBandwidth, outputNarrowBandwidth, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReinitializeLevelSet__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ReinitializeLevelSet__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical', 'numeric', 'numeric')
class(`ReinitializeLevelSet__SWIG_0`) = c("SWIGFunction", class('ReinitializeLevelSet__SWIG_0'))

# Start of ReinitializeLevelSet

`ReinitializeLevelSet__SWIG_1` = function(image1, levelSetValue, narrowBanding, inputNarrowBandwidth, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  narrowBanding = as.logical(narrowBanding);
  
  ;ans = .Call('R_swig_ReinitializeLevelSet__SWIG_1', image1, levelSetValue, narrowBanding, inputNarrowBandwidth, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReinitializeLevelSet__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ReinitializeLevelSet__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical', 'numeric')
class(`ReinitializeLevelSet__SWIG_1`) = c("SWIGFunction", class('ReinitializeLevelSet__SWIG_1'))

# Start of ReinitializeLevelSet

`ReinitializeLevelSet__SWIG_2` = function(image1, levelSetValue, narrowBanding, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  narrowBanding = as.logical(narrowBanding);
  ;ans = .Call('R_swig_ReinitializeLevelSet__SWIG_2', image1, levelSetValue, narrowBanding, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReinitializeLevelSet__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ReinitializeLevelSet__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`ReinitializeLevelSet__SWIG_2`) = c("SWIGFunction", class('ReinitializeLevelSet__SWIG_2'))

# Start of ReinitializeLevelSet

`ReinitializeLevelSet__SWIG_3` = function(image1, levelSetValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_ReinitializeLevelSet__SWIG_3', image1, levelSetValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReinitializeLevelSet__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ReinitializeLevelSet__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`ReinitializeLevelSet__SWIG_3`) = c("SWIGFunction", class('ReinitializeLevelSet__SWIG_3'))

# Start of ReinitializeLevelSet

`ReinitializeLevelSet__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ReinitializeLevelSet__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ReinitializeLevelSet__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ReinitializeLevelSet__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`ReinitializeLevelSet__SWIG_4`) = c("SWIGFunction", class('ReinitializeLevelSet__SWIG_4'))

`ReinitializeLevelSet` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ReinitializeLevelSet__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ReinitializeLevelSet__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ReinitializeLevelSet__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ReinitializeLevelSet__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- ReinitializeLevelSet__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ReinitializeLevelSet with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_RelabelComponentImageFilter

`delete_RelabelComponentImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_RelabelComponentImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_RelabelComponentImageFilter`, 'returnType') = 'void'
attr(`delete_RelabelComponentImageFilter`, "inputTypes") = c('_p_itk__simple__RelabelComponentImageFilter')
class(`delete_RelabelComponentImageFilter`) = c("SWIGFunction", class('delete_RelabelComponentImageFilter'))

# Start of new_RelabelComponentImageFilter

`RelabelComponentImageFilter` = function()
{
  ;ans = .Call('R_swig_new_RelabelComponentImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__RelabelComponentImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_RelabelComponentImageFilter);
  ans
  
}

attr(`RelabelComponentImageFilter`, 'returnType') = '_p_itk__simple__RelabelComponentImageFilter'
class(`RelabelComponentImageFilter`) = c("SWIGFunction", class('RelabelComponentImageFilter'))

# Start of RelabelComponentImageFilter_SetMinimumObjectSize

`RelabelComponentImageFilter_SetMinimumObjectSize` = function(self, MinimumObjectSize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MinimumObjectSize = as.integer(MinimumObjectSize);
  
  if(length(MinimumObjectSize) > 1) {
    warning("using only the first element of MinimumObjectSize");
  };
  
  ;ans = .Call('R_swig_RelabelComponentImageFilter_SetMinimumObjectSize', self, MinimumObjectSize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RelabelComponentImageFilter_SetMinimumObjectSize`, 'returnType') = 'void'
attr(`RelabelComponentImageFilter_SetMinimumObjectSize`, "inputTypes") = c('_p_itk__simple__RelabelComponentImageFilter', 'integer')
class(`RelabelComponentImageFilter_SetMinimumObjectSize`) = c("SWIGFunction", class('RelabelComponentImageFilter_SetMinimumObjectSize'))

# Start of RelabelComponentImageFilter_GetMinimumObjectSize

`RelabelComponentImageFilter_GetMinimumObjectSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RelabelComponentImageFilter_GetMinimumObjectSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RelabelComponentImageFilter_GetMinimumObjectSize`, 'returnType') = 'integer'
attr(`RelabelComponentImageFilter_GetMinimumObjectSize`, "inputTypes") = c('_p_itk__simple__RelabelComponentImageFilter')
class(`RelabelComponentImageFilter_GetMinimumObjectSize`) = c("SWIGFunction", class('RelabelComponentImageFilter_GetMinimumObjectSize'))

# Start of RelabelComponentImageFilter_SetSortByObjectSize

`RelabelComponentImageFilter_SetSortByObjectSize` = function(self, SortByObjectSize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SortByObjectSize = as.logical(SortByObjectSize);
  ;ans = .Call('R_swig_RelabelComponentImageFilter_SetSortByObjectSize', self, SortByObjectSize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RelabelComponentImageFilter_SetSortByObjectSize`, 'returnType') = 'void'
attr(`RelabelComponentImageFilter_SetSortByObjectSize`, "inputTypes") = c('_p_itk__simple__RelabelComponentImageFilter', 'logical')
class(`RelabelComponentImageFilter_SetSortByObjectSize`) = c("SWIGFunction", class('RelabelComponentImageFilter_SetSortByObjectSize'))

# Start of RelabelComponentImageFilter_SortByObjectSizeOn

`RelabelComponentImageFilter_SortByObjectSizeOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RelabelComponentImageFilter_SortByObjectSizeOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RelabelComponentImageFilter_SortByObjectSizeOn`, 'returnType') = 'void'
attr(`RelabelComponentImageFilter_SortByObjectSizeOn`, "inputTypes") = c('_p_itk__simple__RelabelComponentImageFilter')
class(`RelabelComponentImageFilter_SortByObjectSizeOn`) = c("SWIGFunction", class('RelabelComponentImageFilter_SortByObjectSizeOn'))

# Start of RelabelComponentImageFilter_SortByObjectSizeOff

`RelabelComponentImageFilter_SortByObjectSizeOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RelabelComponentImageFilter_SortByObjectSizeOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RelabelComponentImageFilter_SortByObjectSizeOff`, 'returnType') = 'void'
attr(`RelabelComponentImageFilter_SortByObjectSizeOff`, "inputTypes") = c('_p_itk__simple__RelabelComponentImageFilter')
class(`RelabelComponentImageFilter_SortByObjectSizeOff`) = c("SWIGFunction", class('RelabelComponentImageFilter_SortByObjectSizeOff'))

# Start of RelabelComponentImageFilter_GetSortByObjectSize

`RelabelComponentImageFilter_GetSortByObjectSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RelabelComponentImageFilter_GetSortByObjectSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RelabelComponentImageFilter_GetSortByObjectSize`, 'returnType') = 'logical'
attr(`RelabelComponentImageFilter_GetSortByObjectSize`, "inputTypes") = c('_p_itk__simple__RelabelComponentImageFilter')
class(`RelabelComponentImageFilter_GetSortByObjectSize`) = c("SWIGFunction", class('RelabelComponentImageFilter_GetSortByObjectSize'))

# Start of RelabelComponentImageFilter_GetNumberOfObjects

`RelabelComponentImageFilter_GetNumberOfObjects` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RelabelComponentImageFilter_GetNumberOfObjects', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RelabelComponentImageFilter_GetNumberOfObjects`, 'returnType') = 'integer'
attr(`RelabelComponentImageFilter_GetNumberOfObjects`, "inputTypes") = c('_p_itk__simple__RelabelComponentImageFilter')
class(`RelabelComponentImageFilter_GetNumberOfObjects`) = c("SWIGFunction", class('RelabelComponentImageFilter_GetNumberOfObjects'))

# Start of RelabelComponentImageFilter_GetOriginalNumberOfObjects

`RelabelComponentImageFilter_GetOriginalNumberOfObjects` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RelabelComponentImageFilter_GetOriginalNumberOfObjects', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RelabelComponentImageFilter_GetOriginalNumberOfObjects`, 'returnType') = 'integer'
attr(`RelabelComponentImageFilter_GetOriginalNumberOfObjects`, "inputTypes") = c('_p_itk__simple__RelabelComponentImageFilter')
class(`RelabelComponentImageFilter_GetOriginalNumberOfObjects`) = c("SWIGFunction", class('RelabelComponentImageFilter_GetOriginalNumberOfObjects'))

# Start of RelabelComponentImageFilter_GetSizeOfObjectsInPhysicalUnits

`RelabelComponentImageFilter_GetSizeOfObjectsInPhysicalUnits` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RelabelComponentImageFilter_GetSizeOfObjectsInPhysicalUnits', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RelabelComponentImageFilter_GetSizeOfObjectsInPhysicalUnits`, 'returnType') = 'numeric'
attr(`RelabelComponentImageFilter_GetSizeOfObjectsInPhysicalUnits`, "inputTypes") = c('_p_itk__simple__RelabelComponentImageFilter')
class(`RelabelComponentImageFilter_GetSizeOfObjectsInPhysicalUnits`) = c("SWIGFunction", class('RelabelComponentImageFilter_GetSizeOfObjectsInPhysicalUnits'))

# Start of RelabelComponentImageFilter_GetSizeOfObjectsInPixels

`RelabelComponentImageFilter_GetSizeOfObjectsInPixels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RelabelComponentImageFilter_GetSizeOfObjectsInPixels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RelabelComponentImageFilter_GetSizeOfObjectsInPixels`, 'returnType') = 'integer'
attr(`RelabelComponentImageFilter_GetSizeOfObjectsInPixels`, "inputTypes") = c('_p_itk__simple__RelabelComponentImageFilter')
class(`RelabelComponentImageFilter_GetSizeOfObjectsInPixels`) = c("SWIGFunction", class('RelabelComponentImageFilter_GetSizeOfObjectsInPixels'))

# Start of RelabelComponentImageFilter_GetName

`RelabelComponentImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RelabelComponentImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RelabelComponentImageFilter_GetName`, 'returnType') = 'character'
attr(`RelabelComponentImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__RelabelComponentImageFilter')
class(`RelabelComponentImageFilter_GetName`) = c("SWIGFunction", class('RelabelComponentImageFilter_GetName'))

# Start of RelabelComponentImageFilter_ToString

`RelabelComponentImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RelabelComponentImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RelabelComponentImageFilter_ToString`, 'returnType') = 'character'
attr(`RelabelComponentImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__RelabelComponentImageFilter')
class(`RelabelComponentImageFilter_ToString`) = c("SWIGFunction", class('RelabelComponentImageFilter_ToString'))

# Start of RelabelComponentImageFilter_Execute

`RelabelComponentImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RelabelComponentImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RelabelComponentImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`RelabelComponentImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__RelabelComponentImageFilter', '_p_itk__simple__Image')
class(`RelabelComponentImageFilter_Execute`) = c("SWIGFunction", class('RelabelComponentImageFilter_Execute'))

# Start of accessor method for itk::simple::RelabelComponentImageFilter
setMethod('$', '_p_itk__simple__RelabelComponentImageFilter', function(x, name)

{
  accessorFuns = list('SetMinimumObjectSize' = RelabelComponentImageFilter_SetMinimumObjectSize, 'GetMinimumObjectSize' = RelabelComponentImageFilter_GetMinimumObjectSize, 'SetSortByObjectSize' = RelabelComponentImageFilter_SetSortByObjectSize, 'SortByObjectSizeOn' = RelabelComponentImageFilter_SortByObjectSizeOn, 'SortByObjectSizeOff' = RelabelComponentImageFilter_SortByObjectSizeOff, 'GetSortByObjectSize' = RelabelComponentImageFilter_GetSortByObjectSize, 'GetNumberOfObjects' = RelabelComponentImageFilter_GetNumberOfObjects, 'GetOriginalNumberOfObjects' = RelabelComponentImageFilter_GetOriginalNumberOfObjects, 'GetSizeOfObjectsInPhysicalUnits' = RelabelComponentImageFilter_GetSizeOfObjectsInPhysicalUnits, 'GetSizeOfObjectsInPixels' = RelabelComponentImageFilter_GetSizeOfObjectsInPixels, 'GetName' = RelabelComponentImageFilter_GetName, 'ToString' = RelabelComponentImageFilter_ToString, 'Execute' = RelabelComponentImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::RelabelComponentImageFilter
# Start of RelabelComponent

`RelabelComponent__SWIG_0` = function(image1, minimumObjectSize, sortByObjectSize, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  minimumObjectSize = as.integer(minimumObjectSize);
  
  if(length(minimumObjectSize) > 1) {
    warning("using only the first element of minimumObjectSize");
  };
  
  sortByObjectSize = as.logical(sortByObjectSize);
  ;ans = .Call('R_swig_RelabelComponent__SWIG_0', image1, minimumObjectSize, sortByObjectSize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RelabelComponent__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`RelabelComponent__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'logical')
class(`RelabelComponent__SWIG_0`) = c("SWIGFunction", class('RelabelComponent__SWIG_0'))

# Start of RelabelComponent

`RelabelComponent__SWIG_1` = function(image1, minimumObjectSize, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  minimumObjectSize = as.integer(minimumObjectSize);
  
  if(length(minimumObjectSize) > 1) {
    warning("using only the first element of minimumObjectSize");
  };
  
  ;ans = .Call('R_swig_RelabelComponent__SWIG_1', image1, minimumObjectSize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RelabelComponent__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`RelabelComponent__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`RelabelComponent__SWIG_1`) = c("SWIGFunction", class('RelabelComponent__SWIG_1'))

# Start of RelabelComponent

`RelabelComponent__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RelabelComponent__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RelabelComponent__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`RelabelComponent__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`RelabelComponent__SWIG_2`) = c("SWIGFunction", class('RelabelComponent__SWIG_2'))

`RelabelComponent` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- RelabelComponent__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- RelabelComponent__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- RelabelComponent__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for RelabelComponent with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_RelabelLabelMapFilter

`delete_RelabelLabelMapFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_RelabelLabelMapFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_RelabelLabelMapFilter`, 'returnType') = 'void'
attr(`delete_RelabelLabelMapFilter`, "inputTypes") = c('_p_itk__simple__RelabelLabelMapFilter')
class(`delete_RelabelLabelMapFilter`) = c("SWIGFunction", class('delete_RelabelLabelMapFilter'))

# Start of new_RelabelLabelMapFilter

`RelabelLabelMapFilter` = function()
{
  ;ans = .Call('R_swig_new_RelabelLabelMapFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__RelabelLabelMapFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_RelabelLabelMapFilter);
  ans
  
}

attr(`RelabelLabelMapFilter`, 'returnType') = '_p_itk__simple__RelabelLabelMapFilter'
class(`RelabelLabelMapFilter`) = c("SWIGFunction", class('RelabelLabelMapFilter'))

# Start of RelabelLabelMapFilter_SetReverseOrdering

`RelabelLabelMapFilter_SetReverseOrdering` = function(self, ReverseOrdering)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ReverseOrdering = as.logical(ReverseOrdering);
  ;ans = .Call('R_swig_RelabelLabelMapFilter_SetReverseOrdering', self, ReverseOrdering, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RelabelLabelMapFilter_SetReverseOrdering`, 'returnType') = 'void'
attr(`RelabelLabelMapFilter_SetReverseOrdering`, "inputTypes") = c('_p_itk__simple__RelabelLabelMapFilter', 'logical')
class(`RelabelLabelMapFilter_SetReverseOrdering`) = c("SWIGFunction", class('RelabelLabelMapFilter_SetReverseOrdering'))

# Start of RelabelLabelMapFilter_ReverseOrderingOn

`RelabelLabelMapFilter_ReverseOrderingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RelabelLabelMapFilter_ReverseOrderingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RelabelLabelMapFilter_ReverseOrderingOn`, 'returnType') = 'void'
attr(`RelabelLabelMapFilter_ReverseOrderingOn`, "inputTypes") = c('_p_itk__simple__RelabelLabelMapFilter')
class(`RelabelLabelMapFilter_ReverseOrderingOn`) = c("SWIGFunction", class('RelabelLabelMapFilter_ReverseOrderingOn'))

# Start of RelabelLabelMapFilter_ReverseOrderingOff

`RelabelLabelMapFilter_ReverseOrderingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RelabelLabelMapFilter_ReverseOrderingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RelabelLabelMapFilter_ReverseOrderingOff`, 'returnType') = 'void'
attr(`RelabelLabelMapFilter_ReverseOrderingOff`, "inputTypes") = c('_p_itk__simple__RelabelLabelMapFilter')
class(`RelabelLabelMapFilter_ReverseOrderingOff`) = c("SWIGFunction", class('RelabelLabelMapFilter_ReverseOrderingOff'))

# Start of RelabelLabelMapFilter_GetReverseOrdering

`RelabelLabelMapFilter_GetReverseOrdering` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RelabelLabelMapFilter_GetReverseOrdering', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RelabelLabelMapFilter_GetReverseOrdering`, 'returnType') = 'logical'
attr(`RelabelLabelMapFilter_GetReverseOrdering`, "inputTypes") = c('_p_itk__simple__RelabelLabelMapFilter')
class(`RelabelLabelMapFilter_GetReverseOrdering`) = c("SWIGFunction", class('RelabelLabelMapFilter_GetReverseOrdering'))

# Start of RelabelLabelMapFilter_GetName

`RelabelLabelMapFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RelabelLabelMapFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RelabelLabelMapFilter_GetName`, 'returnType') = 'character'
attr(`RelabelLabelMapFilter_GetName`, "inputTypes") = c('_p_itk__simple__RelabelLabelMapFilter')
class(`RelabelLabelMapFilter_GetName`) = c("SWIGFunction", class('RelabelLabelMapFilter_GetName'))

# Start of RelabelLabelMapFilter_ToString

`RelabelLabelMapFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RelabelLabelMapFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RelabelLabelMapFilter_ToString`, 'returnType') = 'character'
attr(`RelabelLabelMapFilter_ToString`, "inputTypes") = c('_p_itk__simple__RelabelLabelMapFilter')
class(`RelabelLabelMapFilter_ToString`) = c("SWIGFunction", class('RelabelLabelMapFilter_ToString'))

# Start of RelabelLabelMapFilter_Execute

`RelabelLabelMapFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RelabelLabelMapFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RelabelLabelMapFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`RelabelLabelMapFilter_Execute`, "inputTypes") = c('_p_itk__simple__RelabelLabelMapFilter', '_p_itk__simple__Image')
class(`RelabelLabelMapFilter_Execute`) = c("SWIGFunction", class('RelabelLabelMapFilter_Execute'))

# Start of accessor method for itk::simple::RelabelLabelMapFilter
setMethod('$', '_p_itk__simple__RelabelLabelMapFilter', function(x, name)

{
  accessorFuns = list('SetReverseOrdering' = RelabelLabelMapFilter_SetReverseOrdering, 'ReverseOrderingOn' = RelabelLabelMapFilter_ReverseOrderingOn, 'ReverseOrderingOff' = RelabelLabelMapFilter_ReverseOrderingOff, 'GetReverseOrdering' = RelabelLabelMapFilter_GetReverseOrdering, 'GetName' = RelabelLabelMapFilter_GetName, 'ToString' = RelabelLabelMapFilter_ToString, 'Execute' = RelabelLabelMapFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::RelabelLabelMapFilter
# Start of RelabelLabelMap

`RelabelLabelMap__SWIG_0` = function(image1, reverseOrdering, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  reverseOrdering = as.logical(reverseOrdering);
  ;ans = .Call('R_swig_RelabelLabelMap__SWIG_0', image1, reverseOrdering, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RelabelLabelMap__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`RelabelLabelMap__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`RelabelLabelMap__SWIG_0`) = c("SWIGFunction", class('RelabelLabelMap__SWIG_0'))

# Start of RelabelLabelMap

`RelabelLabelMap__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RelabelLabelMap__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RelabelLabelMap__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`RelabelLabelMap__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`RelabelLabelMap__SWIG_1`) = c("SWIGFunction", class('RelabelLabelMap__SWIG_1'))

`RelabelLabelMap` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- RelabelLabelMap__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- RelabelLabelMap__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for RelabelLabelMap with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_RenyiEntropyThresholdImageFilter

`delete_RenyiEntropyThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_RenyiEntropyThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_RenyiEntropyThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_RenyiEntropyThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter')
class(`delete_RenyiEntropyThresholdImageFilter`) = c("SWIGFunction", class('delete_RenyiEntropyThresholdImageFilter'))

# Start of new_RenyiEntropyThresholdImageFilter

`RenyiEntropyThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_RenyiEntropyThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__RenyiEntropyThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_RenyiEntropyThresholdImageFilter);
  ans
  
}

attr(`RenyiEntropyThresholdImageFilter`, 'returnType') = '_p_itk__simple__RenyiEntropyThresholdImageFilter'
class(`RenyiEntropyThresholdImageFilter`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter'))

# Start of RenyiEntropyThresholdImageFilter_SetInsideValue

`RenyiEntropyThresholdImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideValue = as.integer(InsideValue);
  
  if(length(InsideValue) > 1) {
    warning("using only the first element of InsideValue");
  };
  
  ;ans = .Call('R_swig_RenyiEntropyThresholdImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RenyiEntropyThresholdImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`RenyiEntropyThresholdImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter', 'integer')
class(`RenyiEntropyThresholdImageFilter_SetInsideValue`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_SetInsideValue'))

# Start of RenyiEntropyThresholdImageFilter_GetInsideValue

`RenyiEntropyThresholdImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RenyiEntropyThresholdImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RenyiEntropyThresholdImageFilter_GetInsideValue`, 'returnType') = 'integer'
attr(`RenyiEntropyThresholdImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter')
class(`RenyiEntropyThresholdImageFilter_GetInsideValue`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_GetInsideValue'))

# Start of RenyiEntropyThresholdImageFilter_SetOutsideValue

`RenyiEntropyThresholdImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutsideValue = as.integer(OutsideValue);
  
  if(length(OutsideValue) > 1) {
    warning("using only the first element of OutsideValue");
  };
  
  ;ans = .Call('R_swig_RenyiEntropyThresholdImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RenyiEntropyThresholdImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`RenyiEntropyThresholdImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter', 'integer')
class(`RenyiEntropyThresholdImageFilter_SetOutsideValue`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_SetOutsideValue'))

# Start of RenyiEntropyThresholdImageFilter_GetOutsideValue

`RenyiEntropyThresholdImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RenyiEntropyThresholdImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RenyiEntropyThresholdImageFilter_GetOutsideValue`, 'returnType') = 'integer'
attr(`RenyiEntropyThresholdImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter')
class(`RenyiEntropyThresholdImageFilter_GetOutsideValue`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_GetOutsideValue'))

# Start of RenyiEntropyThresholdImageFilter_SetNumberOfHistogramBins

`RenyiEntropyThresholdImageFilter_SetNumberOfHistogramBins` = function(self, NumberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfHistogramBins = as.integer(NumberOfHistogramBins);
  
  if(length(NumberOfHistogramBins) > 1) {
    warning("using only the first element of NumberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_RenyiEntropyThresholdImageFilter_SetNumberOfHistogramBins', self, NumberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RenyiEntropyThresholdImageFilter_SetNumberOfHistogramBins`, 'returnType') = 'void'
attr(`RenyiEntropyThresholdImageFilter_SetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter', 'integer')
class(`RenyiEntropyThresholdImageFilter_SetNumberOfHistogramBins`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_SetNumberOfHistogramBins'))

# Start of RenyiEntropyThresholdImageFilter_GetNumberOfHistogramBins

`RenyiEntropyThresholdImageFilter_GetNumberOfHistogramBins` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RenyiEntropyThresholdImageFilter_GetNumberOfHistogramBins', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RenyiEntropyThresholdImageFilter_GetNumberOfHistogramBins`, 'returnType') = 'integer'
attr(`RenyiEntropyThresholdImageFilter_GetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter')
class(`RenyiEntropyThresholdImageFilter_GetNumberOfHistogramBins`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_GetNumberOfHistogramBins'))

# Start of RenyiEntropyThresholdImageFilter_SetMaskOutput

`RenyiEntropyThresholdImageFilter_SetMaskOutput` = function(self, MaskOutput)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskOutput = as.logical(MaskOutput);
  ;ans = .Call('R_swig_RenyiEntropyThresholdImageFilter_SetMaskOutput', self, MaskOutput, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RenyiEntropyThresholdImageFilter_SetMaskOutput`, 'returnType') = 'void'
attr(`RenyiEntropyThresholdImageFilter_SetMaskOutput`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter', 'logical')
class(`RenyiEntropyThresholdImageFilter_SetMaskOutput`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_SetMaskOutput'))

# Start of RenyiEntropyThresholdImageFilter_MaskOutputOn

`RenyiEntropyThresholdImageFilter_MaskOutputOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RenyiEntropyThresholdImageFilter_MaskOutputOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RenyiEntropyThresholdImageFilter_MaskOutputOn`, 'returnType') = 'void'
attr(`RenyiEntropyThresholdImageFilter_MaskOutputOn`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter')
class(`RenyiEntropyThresholdImageFilter_MaskOutputOn`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_MaskOutputOn'))

# Start of RenyiEntropyThresholdImageFilter_MaskOutputOff

`RenyiEntropyThresholdImageFilter_MaskOutputOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RenyiEntropyThresholdImageFilter_MaskOutputOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RenyiEntropyThresholdImageFilter_MaskOutputOff`, 'returnType') = 'void'
attr(`RenyiEntropyThresholdImageFilter_MaskOutputOff`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter')
class(`RenyiEntropyThresholdImageFilter_MaskOutputOff`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_MaskOutputOff'))

# Start of RenyiEntropyThresholdImageFilter_GetMaskOutput

`RenyiEntropyThresholdImageFilter_GetMaskOutput` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RenyiEntropyThresholdImageFilter_GetMaskOutput', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RenyiEntropyThresholdImageFilter_GetMaskOutput`, 'returnType') = 'logical'
attr(`RenyiEntropyThresholdImageFilter_GetMaskOutput`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter')
class(`RenyiEntropyThresholdImageFilter_GetMaskOutput`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_GetMaskOutput'))

# Start of RenyiEntropyThresholdImageFilter_SetMaskValue

`RenyiEntropyThresholdImageFilter_SetMaskValue` = function(self, MaskValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskValue = as.integer(MaskValue);
  
  if(length(MaskValue) > 1) {
    warning("using only the first element of MaskValue");
  };
  
  ;ans = .Call('R_swig_RenyiEntropyThresholdImageFilter_SetMaskValue', self, MaskValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RenyiEntropyThresholdImageFilter_SetMaskValue`, 'returnType') = 'void'
attr(`RenyiEntropyThresholdImageFilter_SetMaskValue`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter', 'integer')
class(`RenyiEntropyThresholdImageFilter_SetMaskValue`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_SetMaskValue'))

# Start of RenyiEntropyThresholdImageFilter_GetMaskValue

`RenyiEntropyThresholdImageFilter_GetMaskValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RenyiEntropyThresholdImageFilter_GetMaskValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RenyiEntropyThresholdImageFilter_GetMaskValue`, 'returnType') = 'integer'
attr(`RenyiEntropyThresholdImageFilter_GetMaskValue`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter')
class(`RenyiEntropyThresholdImageFilter_GetMaskValue`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_GetMaskValue'))

# Start of RenyiEntropyThresholdImageFilter_GetThreshold

`RenyiEntropyThresholdImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RenyiEntropyThresholdImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RenyiEntropyThresholdImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`RenyiEntropyThresholdImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter')
class(`RenyiEntropyThresholdImageFilter_GetThreshold`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_GetThreshold'))

# Start of RenyiEntropyThresholdImageFilter_GetName

`RenyiEntropyThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RenyiEntropyThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RenyiEntropyThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`RenyiEntropyThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter')
class(`RenyiEntropyThresholdImageFilter_GetName`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_GetName'))

# Start of RenyiEntropyThresholdImageFilter_ToString

`RenyiEntropyThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RenyiEntropyThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RenyiEntropyThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`RenyiEntropyThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter')
class(`RenyiEntropyThresholdImageFilter_ToString`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_ToString'))

# Start of RenyiEntropyThresholdImageFilter_Execute

`RenyiEntropyThresholdImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_RenyiEntropyThresholdImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RenyiEntropyThresholdImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`RenyiEntropyThresholdImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`RenyiEntropyThresholdImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_Execute__SWIG_0'))

# Start of RenyiEntropyThresholdImageFilter_Execute

`RenyiEntropyThresholdImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_RenyiEntropyThresholdImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RenyiEntropyThresholdImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`RenyiEntropyThresholdImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__RenyiEntropyThresholdImageFilter', '_p_itk__simple__Image')
class(`RenyiEntropyThresholdImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('RenyiEntropyThresholdImageFilter_Execute__SWIG_1'))

`RenyiEntropyThresholdImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__RenyiEntropyThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- RenyiEntropyThresholdImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__RenyiEntropyThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- RenyiEntropyThresholdImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for RenyiEntropyThresholdImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::RenyiEntropyThresholdImageFilter
setMethod('$', '_p_itk__simple__RenyiEntropyThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetInsideValue' = RenyiEntropyThresholdImageFilter_SetInsideValue, 'GetInsideValue' = RenyiEntropyThresholdImageFilter_GetInsideValue, 'SetOutsideValue' = RenyiEntropyThresholdImageFilter_SetOutsideValue, 'GetOutsideValue' = RenyiEntropyThresholdImageFilter_GetOutsideValue, 'SetNumberOfHistogramBins' = RenyiEntropyThresholdImageFilter_SetNumberOfHistogramBins, 'GetNumberOfHistogramBins' = RenyiEntropyThresholdImageFilter_GetNumberOfHistogramBins, 'SetMaskOutput' = RenyiEntropyThresholdImageFilter_SetMaskOutput, 'MaskOutputOn' = RenyiEntropyThresholdImageFilter_MaskOutputOn, 'MaskOutputOff' = RenyiEntropyThresholdImageFilter_MaskOutputOff, 'GetMaskOutput' = RenyiEntropyThresholdImageFilter_GetMaskOutput, 'SetMaskValue' = RenyiEntropyThresholdImageFilter_SetMaskValue, 'GetMaskValue' = RenyiEntropyThresholdImageFilter_GetMaskValue, 'GetThreshold' = RenyiEntropyThresholdImageFilter_GetThreshold, 'GetName' = RenyiEntropyThresholdImageFilter_GetName, 'ToString' = RenyiEntropyThresholdImageFilter_ToString, 'Execute' = RenyiEntropyThresholdImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::RenyiEntropyThresholdImageFilter
# Start of RenyiEntropyThreshold

`RenyiEntropyThreshold__SWIG_0` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_RenyiEntropyThreshold__SWIG_0', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RenyiEntropyThreshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`RenyiEntropyThreshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`RenyiEntropyThreshold__SWIG_0`) = c("SWIGFunction", class('RenyiEntropyThreshold__SWIG_0'))

# Start of RenyiEntropyThreshold

`RenyiEntropyThreshold__SWIG_1` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_RenyiEntropyThreshold__SWIG_1', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RenyiEntropyThreshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`RenyiEntropyThreshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`RenyiEntropyThreshold__SWIG_1`) = c("SWIGFunction", class('RenyiEntropyThreshold__SWIG_1'))

# Start of RenyiEntropyThreshold

`RenyiEntropyThreshold__SWIG_2` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_RenyiEntropyThreshold__SWIG_2', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RenyiEntropyThreshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`RenyiEntropyThreshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`RenyiEntropyThreshold__SWIG_2`) = c("SWIGFunction", class('RenyiEntropyThreshold__SWIG_2'))

# Start of RenyiEntropyThreshold

`RenyiEntropyThreshold__SWIG_3` = function(image, maskImage, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_RenyiEntropyThreshold__SWIG_3', image, maskImage, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RenyiEntropyThreshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`RenyiEntropyThreshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`RenyiEntropyThreshold__SWIG_3`) = c("SWIGFunction", class('RenyiEntropyThreshold__SWIG_3'))

# Start of RenyiEntropyThreshold

`RenyiEntropyThreshold__SWIG_4` = function(image, maskImage, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_RenyiEntropyThreshold__SWIG_4', image, maskImage, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RenyiEntropyThreshold__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`RenyiEntropyThreshold__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`RenyiEntropyThreshold__SWIG_4`) = c("SWIGFunction", class('RenyiEntropyThreshold__SWIG_4'))

# Start of RenyiEntropyThreshold

`RenyiEntropyThreshold__SWIG_5` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_RenyiEntropyThreshold__SWIG_5', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RenyiEntropyThreshold__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`RenyiEntropyThreshold__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`RenyiEntropyThreshold__SWIG_5`) = c("SWIGFunction", class('RenyiEntropyThreshold__SWIG_5'))

# Start of RenyiEntropyThreshold

`RenyiEntropyThreshold__SWIG_6` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_RenyiEntropyThreshold__SWIG_6', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RenyiEntropyThreshold__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`RenyiEntropyThreshold__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`RenyiEntropyThreshold__SWIG_6`) = c("SWIGFunction", class('RenyiEntropyThreshold__SWIG_6'))

# Start of RenyiEntropyThreshold

`RenyiEntropyThreshold__SWIG_7` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_RenyiEntropyThreshold__SWIG_7', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RenyiEntropyThreshold__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`RenyiEntropyThreshold__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`RenyiEntropyThreshold__SWIG_7`) = c("SWIGFunction", class('RenyiEntropyThreshold__SWIG_7'))

# Start of RenyiEntropyThreshold

`RenyiEntropyThreshold__SWIG_8` = function(image, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_RenyiEntropyThreshold__SWIG_8', image, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RenyiEntropyThreshold__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`RenyiEntropyThreshold__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`RenyiEntropyThreshold__SWIG_8`) = c("SWIGFunction", class('RenyiEntropyThreshold__SWIG_8'))

# Start of RenyiEntropyThreshold

`RenyiEntropyThreshold__SWIG_9` = function(image, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_RenyiEntropyThreshold__SWIG_9', image, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RenyiEntropyThreshold__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`RenyiEntropyThreshold__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`RenyiEntropyThreshold__SWIG_9`) = c("SWIGFunction", class('RenyiEntropyThreshold__SWIG_9'))

# Start of RenyiEntropyThreshold

`RenyiEntropyThreshold__SWIG_10` = function(image, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_RenyiEntropyThreshold__SWIG_10', image, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RenyiEntropyThreshold__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`RenyiEntropyThreshold__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`RenyiEntropyThreshold__SWIG_10`) = c("SWIGFunction", class('RenyiEntropyThreshold__SWIG_10'))

# Start of RenyiEntropyThreshold

`RenyiEntropyThreshold__SWIG_11` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_RenyiEntropyThreshold__SWIG_11', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RenyiEntropyThreshold__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`RenyiEntropyThreshold__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image')
class(`RenyiEntropyThreshold__SWIG_11`) = c("SWIGFunction", class('RenyiEntropyThreshold__SWIG_11'))

`RenyiEntropyThreshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 12
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- RenyiEntropyThreshold__SWIG_11; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- RenyiEntropyThreshold__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- RenyiEntropyThreshold__SWIG_10; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- RenyiEntropyThreshold__SWIG_4; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- RenyiEntropyThreshold__SWIG_9; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- RenyiEntropyThreshold__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- RenyiEntropyThreshold__SWIG_8; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- RenyiEntropyThreshold__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- RenyiEntropyThreshold__SWIG_7; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- RenyiEntropyThreshold__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- RenyiEntropyThreshold__SWIG_6; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- RenyiEntropyThreshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for RenyiEntropyThreshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ResampleImageFilter

`delete_ResampleImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ResampleImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ResampleImageFilter`, 'returnType') = 'void'
attr(`delete_ResampleImageFilter`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter')
class(`delete_ResampleImageFilter`) = c("SWIGFunction", class('delete_ResampleImageFilter'))

# Start of new_ResampleImageFilter

`ResampleImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ResampleImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ResampleImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ResampleImageFilter);
  ans
  
}

attr(`ResampleImageFilter`, 'returnType') = '_p_itk__simple__ResampleImageFilter'
class(`ResampleImageFilter`) = c("SWIGFunction", class('ResampleImageFilter'))

# Start of ResampleImageFilter_SetSize

`ResampleImageFilter_SetSize` = function(self, Size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Size = as.integer(Size);
  ;ans = .Call('R_swig_ResampleImageFilter_SetSize', self, Size, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ResampleImageFilter_SetSize`, 'returnType') = 'void'
attr(`ResampleImageFilter_SetSize`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter', 'integer')
class(`ResampleImageFilter_SetSize`) = c("SWIGFunction", class('ResampleImageFilter_SetSize'))

# Start of ResampleImageFilter_GetSize

`ResampleImageFilter_GetSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ResampleImageFilter_GetSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ResampleImageFilter_GetSize`, 'returnType') = 'integer'
attr(`ResampleImageFilter_GetSize`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter')
class(`ResampleImageFilter_GetSize`) = c("SWIGFunction", class('ResampleImageFilter_GetSize'))

# Start of ResampleImageFilter_SetTransform

`ResampleImageFilter_SetTransform` = function(self, Transform)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(Transform, "ExternalReference")) Transform = slot(Transform,"ref"); 
  ;ans = .Call('R_swig_ResampleImageFilter_SetTransform', self, Transform, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ResampleImageFilter_SetTransform`, 'returnType') = 'void'
attr(`ResampleImageFilter_SetTransform`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter', '_p_itk__simple__Transform')
class(`ResampleImageFilter_SetTransform`) = c("SWIGFunction", class('ResampleImageFilter_SetTransform'))

# Start of ResampleImageFilter_GetTransform

`ResampleImageFilter_GetTransform` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ResampleImageFilter_GetTransform', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Transform", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ResampleImageFilter_GetTransform`, 'returnType') = '_p_itk__simple__Transform'
attr(`ResampleImageFilter_GetTransform`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter')
class(`ResampleImageFilter_GetTransform`) = c("SWIGFunction", class('ResampleImageFilter_GetTransform'))

# Start of ResampleImageFilter_SetInterpolator

`ResampleImageFilter_SetInterpolator` = function(self, Interpolator)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Interpolator = enumToInteger(Interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(Interpolator) > 1) {
    warning("using only the first element of Interpolator");
  };
  
  ;ans = .Call('R_swig_ResampleImageFilter_SetInterpolator', self, Interpolator, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ResampleImageFilter_SetInterpolator`, 'returnType') = 'void'
attr(`ResampleImageFilter_SetInterpolator`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter', 'character')
class(`ResampleImageFilter_SetInterpolator`) = c("SWIGFunction", class('ResampleImageFilter_SetInterpolator'))

# Start of ResampleImageFilter_GetInterpolator

`ResampleImageFilter_GetInterpolator` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ResampleImageFilter_GetInterpolator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`ResampleImageFilter_GetInterpolator`, 'returnType') = 'character'
attr(`ResampleImageFilter_GetInterpolator`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter')
class(`ResampleImageFilter_GetInterpolator`) = c("SWIGFunction", class('ResampleImageFilter_GetInterpolator'))

# Start of ResampleImageFilter_SetOutputOrigin

`ResampleImageFilter_SetOutputOrigin` = function(self, OutputOrigin)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputOrigin = as.numeric(OutputOrigin);
  ;ans = .Call('R_swig_ResampleImageFilter_SetOutputOrigin', self, OutputOrigin, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ResampleImageFilter_SetOutputOrigin`, 'returnType') = 'void'
attr(`ResampleImageFilter_SetOutputOrigin`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter', 'numeric')
class(`ResampleImageFilter_SetOutputOrigin`) = c("SWIGFunction", class('ResampleImageFilter_SetOutputOrigin'))

# Start of ResampleImageFilter_GetOutputOrigin

`ResampleImageFilter_GetOutputOrigin` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ResampleImageFilter_GetOutputOrigin', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ResampleImageFilter_GetOutputOrigin`, 'returnType') = 'numeric'
attr(`ResampleImageFilter_GetOutputOrigin`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter')
class(`ResampleImageFilter_GetOutputOrigin`) = c("SWIGFunction", class('ResampleImageFilter_GetOutputOrigin'))

# Start of ResampleImageFilter_SetOutputSpacing

`ResampleImageFilter_SetOutputSpacing` = function(self, OutputSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputSpacing = as.numeric(OutputSpacing);
  ;ans = .Call('R_swig_ResampleImageFilter_SetOutputSpacing', self, OutputSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ResampleImageFilter_SetOutputSpacing`, 'returnType') = 'void'
attr(`ResampleImageFilter_SetOutputSpacing`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter', 'numeric')
class(`ResampleImageFilter_SetOutputSpacing`) = c("SWIGFunction", class('ResampleImageFilter_SetOutputSpacing'))

# Start of ResampleImageFilter_GetOutputSpacing

`ResampleImageFilter_GetOutputSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ResampleImageFilter_GetOutputSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ResampleImageFilter_GetOutputSpacing`, 'returnType') = 'numeric'
attr(`ResampleImageFilter_GetOutputSpacing`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter')
class(`ResampleImageFilter_GetOutputSpacing`) = c("SWIGFunction", class('ResampleImageFilter_GetOutputSpacing'))

# Start of ResampleImageFilter_SetOutputDirection

`ResampleImageFilter_SetOutputDirection` = function(self, OutputDirection)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputDirection = as.numeric(OutputDirection);
  ;ans = .Call('R_swig_ResampleImageFilter_SetOutputDirection', self, OutputDirection, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ResampleImageFilter_SetOutputDirection`, 'returnType') = 'void'
attr(`ResampleImageFilter_SetOutputDirection`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter', 'numeric')
class(`ResampleImageFilter_SetOutputDirection`) = c("SWIGFunction", class('ResampleImageFilter_SetOutputDirection'))

# Start of ResampleImageFilter_GetOutputDirection

`ResampleImageFilter_GetOutputDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ResampleImageFilter_GetOutputDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ResampleImageFilter_GetOutputDirection`, 'returnType') = 'numeric'
attr(`ResampleImageFilter_GetOutputDirection`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter')
class(`ResampleImageFilter_GetOutputDirection`) = c("SWIGFunction", class('ResampleImageFilter_GetOutputDirection'))

# Start of ResampleImageFilter_SetDefaultPixelValue

`ResampleImageFilter_SetDefaultPixelValue` = function(self, DefaultPixelValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ResampleImageFilter_SetDefaultPixelValue', self, DefaultPixelValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ResampleImageFilter_SetDefaultPixelValue`, 'returnType') = 'void'
attr(`ResampleImageFilter_SetDefaultPixelValue`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter', 'numeric')
class(`ResampleImageFilter_SetDefaultPixelValue`) = c("SWIGFunction", class('ResampleImageFilter_SetDefaultPixelValue'))

# Start of ResampleImageFilter_GetDefaultPixelValue

`ResampleImageFilter_GetDefaultPixelValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ResampleImageFilter_GetDefaultPixelValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ResampleImageFilter_GetDefaultPixelValue`, 'returnType') = 'numeric'
attr(`ResampleImageFilter_GetDefaultPixelValue`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter')
class(`ResampleImageFilter_GetDefaultPixelValue`) = c("SWIGFunction", class('ResampleImageFilter_GetDefaultPixelValue'))

# Start of ResampleImageFilter_SetOutputPixelType

`ResampleImageFilter_SetOutputPixelType` = function(self, OutputPixelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputPixelType = enumToInteger(OutputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(OutputPixelType) > 1) {
    warning("using only the first element of OutputPixelType");
  };
  
  ;ans = .Call('R_swig_ResampleImageFilter_SetOutputPixelType', self, OutputPixelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ResampleImageFilter_SetOutputPixelType`, 'returnType') = 'void'
attr(`ResampleImageFilter_SetOutputPixelType`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter', 'character')
class(`ResampleImageFilter_SetOutputPixelType`) = c("SWIGFunction", class('ResampleImageFilter_SetOutputPixelType'))

# Start of ResampleImageFilter_GetOutputPixelType

`ResampleImageFilter_GetOutputPixelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ResampleImageFilter_GetOutputPixelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`ResampleImageFilter_GetOutputPixelType`, 'returnType') = 'character'
attr(`ResampleImageFilter_GetOutputPixelType`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter')
class(`ResampleImageFilter_GetOutputPixelType`) = c("SWIGFunction", class('ResampleImageFilter_GetOutputPixelType'))

# Start of ResampleImageFilter_SetUseNearestNeighborExtrapolator

`ResampleImageFilter_SetUseNearestNeighborExtrapolator` = function(self, UseNearestNeighborExtrapolator)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseNearestNeighborExtrapolator = as.logical(UseNearestNeighborExtrapolator);
  ;ans = .Call('R_swig_ResampleImageFilter_SetUseNearestNeighborExtrapolator', self, UseNearestNeighborExtrapolator, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ResampleImageFilter_SetUseNearestNeighborExtrapolator`, 'returnType') = 'void'
attr(`ResampleImageFilter_SetUseNearestNeighborExtrapolator`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter', 'logical')
class(`ResampleImageFilter_SetUseNearestNeighborExtrapolator`) = c("SWIGFunction", class('ResampleImageFilter_SetUseNearestNeighborExtrapolator'))

# Start of ResampleImageFilter_UseNearestNeighborExtrapolatorOn

`ResampleImageFilter_UseNearestNeighborExtrapolatorOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ResampleImageFilter_UseNearestNeighborExtrapolatorOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ResampleImageFilter_UseNearestNeighborExtrapolatorOn`, 'returnType') = 'void'
attr(`ResampleImageFilter_UseNearestNeighborExtrapolatorOn`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter')
class(`ResampleImageFilter_UseNearestNeighborExtrapolatorOn`) = c("SWIGFunction", class('ResampleImageFilter_UseNearestNeighborExtrapolatorOn'))

# Start of ResampleImageFilter_UseNearestNeighborExtrapolatorOff

`ResampleImageFilter_UseNearestNeighborExtrapolatorOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ResampleImageFilter_UseNearestNeighborExtrapolatorOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ResampleImageFilter_UseNearestNeighborExtrapolatorOff`, 'returnType') = 'void'
attr(`ResampleImageFilter_UseNearestNeighborExtrapolatorOff`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter')
class(`ResampleImageFilter_UseNearestNeighborExtrapolatorOff`) = c("SWIGFunction", class('ResampleImageFilter_UseNearestNeighborExtrapolatorOff'))

# Start of ResampleImageFilter_GetUseNearestNeighborExtrapolator

`ResampleImageFilter_GetUseNearestNeighborExtrapolator` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ResampleImageFilter_GetUseNearestNeighborExtrapolator', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ResampleImageFilter_GetUseNearestNeighborExtrapolator`, 'returnType') = 'logical'
attr(`ResampleImageFilter_GetUseNearestNeighborExtrapolator`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter')
class(`ResampleImageFilter_GetUseNearestNeighborExtrapolator`) = c("SWIGFunction", class('ResampleImageFilter_GetUseNearestNeighborExtrapolator'))

# Start of ResampleImageFilter_GetName

`ResampleImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ResampleImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ResampleImageFilter_GetName`, 'returnType') = 'character'
attr(`ResampleImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter')
class(`ResampleImageFilter_GetName`) = c("SWIGFunction", class('ResampleImageFilter_GetName'))

# Start of ResampleImageFilter_ToString

`ResampleImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ResampleImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ResampleImageFilter_ToString`, 'returnType') = 'character'
attr(`ResampleImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter')
class(`ResampleImageFilter_ToString`) = c("SWIGFunction", class('ResampleImageFilter_ToString'))

# Start of ResampleImageFilter_Execute

`ResampleImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ResampleImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ResampleImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ResampleImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter', '_p_itk__simple__Image')
class(`ResampleImageFilter_Execute`) = c("SWIGFunction", class('ResampleImageFilter_Execute'))

# Start of ResampleImageFilter_SetReferenceImage

`ResampleImageFilter_SetReferenceImage` = function(self, refImage)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(refImage, "ExternalReference")) refImage = slot(refImage,"ref"); 
  ;ans = .Call('R_swig_ResampleImageFilter_SetReferenceImage', self, refImage, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ResampleImageFilter_SetReferenceImage`, 'returnType') = 'void'
attr(`ResampleImageFilter_SetReferenceImage`, "inputTypes") = c('_p_itk__simple__ResampleImageFilter', '_p_itk__simple__Image')
class(`ResampleImageFilter_SetReferenceImage`) = c("SWIGFunction", class('ResampleImageFilter_SetReferenceImage'))

# Start of accessor method for itk::simple::ResampleImageFilter
setMethod('$', '_p_itk__simple__ResampleImageFilter', function(x, name)

{
  accessorFuns = list('SetSize' = ResampleImageFilter_SetSize, 'GetSize' = ResampleImageFilter_GetSize, 'SetTransform' = ResampleImageFilter_SetTransform, 'GetTransform' = ResampleImageFilter_GetTransform, 'SetInterpolator' = ResampleImageFilter_SetInterpolator, 'GetInterpolator' = ResampleImageFilter_GetInterpolator, 'SetOutputOrigin' = ResampleImageFilter_SetOutputOrigin, 'GetOutputOrigin' = ResampleImageFilter_GetOutputOrigin, 'SetOutputSpacing' = ResampleImageFilter_SetOutputSpacing, 'GetOutputSpacing' = ResampleImageFilter_GetOutputSpacing, 'SetOutputDirection' = ResampleImageFilter_SetOutputDirection, 'GetOutputDirection' = ResampleImageFilter_GetOutputDirection, 'SetDefaultPixelValue' = ResampleImageFilter_SetDefaultPixelValue, 'GetDefaultPixelValue' = ResampleImageFilter_GetDefaultPixelValue, 'SetOutputPixelType' = ResampleImageFilter_SetOutputPixelType, 'GetOutputPixelType' = ResampleImageFilter_GetOutputPixelType, 'SetUseNearestNeighborExtrapolator' = ResampleImageFilter_SetUseNearestNeighborExtrapolator, 'UseNearestNeighborExtrapolatorOn' = ResampleImageFilter_UseNearestNeighborExtrapolatorOn, 'UseNearestNeighborExtrapolatorOff' = ResampleImageFilter_UseNearestNeighborExtrapolatorOff, 'GetUseNearestNeighborExtrapolator' = ResampleImageFilter_GetUseNearestNeighborExtrapolator, 'GetName' = ResampleImageFilter_GetName, 'ToString' = ResampleImageFilter_ToString, 'Execute' = ResampleImageFilter_Execute, 'SetReferenceImage' = ResampleImageFilter_SetReferenceImage);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ResampleImageFilter
# Start of delete_RescaleIntensityImageFilter

`delete_RescaleIntensityImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_RescaleIntensityImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_RescaleIntensityImageFilter`, 'returnType') = 'void'
attr(`delete_RescaleIntensityImageFilter`, "inputTypes") = c('_p_itk__simple__RescaleIntensityImageFilter')
class(`delete_RescaleIntensityImageFilter`) = c("SWIGFunction", class('delete_RescaleIntensityImageFilter'))

# Start of new_RescaleIntensityImageFilter

`RescaleIntensityImageFilter` = function()
{
  ;ans = .Call('R_swig_new_RescaleIntensityImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__RescaleIntensityImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_RescaleIntensityImageFilter);
  ans
  
}

attr(`RescaleIntensityImageFilter`, 'returnType') = '_p_itk__simple__RescaleIntensityImageFilter'
class(`RescaleIntensityImageFilter`) = c("SWIGFunction", class('RescaleIntensityImageFilter'))

# Start of RescaleIntensityImageFilter_SetOutputMinimum

`RescaleIntensityImageFilter_SetOutputMinimum` = function(self, OutputMinimum)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_RescaleIntensityImageFilter_SetOutputMinimum', self, OutputMinimum, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RescaleIntensityImageFilter_SetOutputMinimum`, 'returnType') = 'void'
attr(`RescaleIntensityImageFilter_SetOutputMinimum`, "inputTypes") = c('_p_itk__simple__RescaleIntensityImageFilter', 'numeric')
class(`RescaleIntensityImageFilter_SetOutputMinimum`) = c("SWIGFunction", class('RescaleIntensityImageFilter_SetOutputMinimum'))

# Start of RescaleIntensityImageFilter_GetOutputMinimum

`RescaleIntensityImageFilter_GetOutputMinimum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RescaleIntensityImageFilter_GetOutputMinimum', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RescaleIntensityImageFilter_GetOutputMinimum`, 'returnType') = 'numeric'
attr(`RescaleIntensityImageFilter_GetOutputMinimum`, "inputTypes") = c('_p_itk__simple__RescaleIntensityImageFilter')
class(`RescaleIntensityImageFilter_GetOutputMinimum`) = c("SWIGFunction", class('RescaleIntensityImageFilter_GetOutputMinimum'))

# Start of RescaleIntensityImageFilter_SetOutputMaximum

`RescaleIntensityImageFilter_SetOutputMaximum` = function(self, OutputMaximum)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_RescaleIntensityImageFilter_SetOutputMaximum', self, OutputMaximum, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RescaleIntensityImageFilter_SetOutputMaximum`, 'returnType') = 'void'
attr(`RescaleIntensityImageFilter_SetOutputMaximum`, "inputTypes") = c('_p_itk__simple__RescaleIntensityImageFilter', 'numeric')
class(`RescaleIntensityImageFilter_SetOutputMaximum`) = c("SWIGFunction", class('RescaleIntensityImageFilter_SetOutputMaximum'))

# Start of RescaleIntensityImageFilter_GetOutputMaximum

`RescaleIntensityImageFilter_GetOutputMaximum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RescaleIntensityImageFilter_GetOutputMaximum', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RescaleIntensityImageFilter_GetOutputMaximum`, 'returnType') = 'numeric'
attr(`RescaleIntensityImageFilter_GetOutputMaximum`, "inputTypes") = c('_p_itk__simple__RescaleIntensityImageFilter')
class(`RescaleIntensityImageFilter_GetOutputMaximum`) = c("SWIGFunction", class('RescaleIntensityImageFilter_GetOutputMaximum'))

# Start of RescaleIntensityImageFilter_GetName

`RescaleIntensityImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RescaleIntensityImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RescaleIntensityImageFilter_GetName`, 'returnType') = 'character'
attr(`RescaleIntensityImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__RescaleIntensityImageFilter')
class(`RescaleIntensityImageFilter_GetName`) = c("SWIGFunction", class('RescaleIntensityImageFilter_GetName'))

# Start of RescaleIntensityImageFilter_ToString

`RescaleIntensityImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RescaleIntensityImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RescaleIntensityImageFilter_ToString`, 'returnType') = 'character'
attr(`RescaleIntensityImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__RescaleIntensityImageFilter')
class(`RescaleIntensityImageFilter_ToString`) = c("SWIGFunction", class('RescaleIntensityImageFilter_ToString'))

# Start of RescaleIntensityImageFilter_Execute

`RescaleIntensityImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RescaleIntensityImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RescaleIntensityImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`RescaleIntensityImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__RescaleIntensityImageFilter', '_p_itk__simple__Image')
class(`RescaleIntensityImageFilter_Execute`) = c("SWIGFunction", class('RescaleIntensityImageFilter_Execute'))

# Start of accessor method for itk::simple::RescaleIntensityImageFilter
setMethod('$', '_p_itk__simple__RescaleIntensityImageFilter', function(x, name)

{
  accessorFuns = list('SetOutputMinimum' = RescaleIntensityImageFilter_SetOutputMinimum, 'GetOutputMinimum' = RescaleIntensityImageFilter_GetOutputMinimum, 'SetOutputMaximum' = RescaleIntensityImageFilter_SetOutputMaximum, 'GetOutputMaximum' = RescaleIntensityImageFilter_GetOutputMaximum, 'GetName' = RescaleIntensityImageFilter_GetName, 'ToString' = RescaleIntensityImageFilter_ToString, 'Execute' = RescaleIntensityImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::RescaleIntensityImageFilter
# Start of RescaleIntensity

`RescaleIntensity__SWIG_0` = function(image1, outputMinimum, outputMaximum, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_RescaleIntensity__SWIG_0', image1, outputMinimum, outputMaximum, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RescaleIntensity__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`RescaleIntensity__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`RescaleIntensity__SWIG_0`) = c("SWIGFunction", class('RescaleIntensity__SWIG_0'))

# Start of RescaleIntensity

`RescaleIntensity__SWIG_1` = function(image1, outputMinimum, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_RescaleIntensity__SWIG_1', image1, outputMinimum, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RescaleIntensity__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`RescaleIntensity__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`RescaleIntensity__SWIG_1`) = c("SWIGFunction", class('RescaleIntensity__SWIG_1'))

# Start of RescaleIntensity

`RescaleIntensity__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RescaleIntensity__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RescaleIntensity__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`RescaleIntensity__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`RescaleIntensity__SWIG_2`) = c("SWIGFunction", class('RescaleIntensity__SWIG_2'))

`RescaleIntensity` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- RescaleIntensity__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- RescaleIntensity__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- RescaleIntensity__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for RescaleIntensity with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_RichardsonLucyDeconvolutionImageFilter

`delete_RichardsonLucyDeconvolutionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_RichardsonLucyDeconvolutionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_RichardsonLucyDeconvolutionImageFilter`, 'returnType') = 'void'
attr(`delete_RichardsonLucyDeconvolutionImageFilter`, "inputTypes") = c('_p_itk__simple__RichardsonLucyDeconvolutionImageFilter')
class(`delete_RichardsonLucyDeconvolutionImageFilter`) = c("SWIGFunction", class('delete_RichardsonLucyDeconvolutionImageFilter'))

# Start of new_RichardsonLucyDeconvolutionImageFilter

`RichardsonLucyDeconvolutionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_RichardsonLucyDeconvolutionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__RichardsonLucyDeconvolutionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_RichardsonLucyDeconvolutionImageFilter);
  ans
  
}

attr(`RichardsonLucyDeconvolutionImageFilter`, 'returnType') = '_p_itk__simple__RichardsonLucyDeconvolutionImageFilter'
class(`RichardsonLucyDeconvolutionImageFilter`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter'))

# Start of RichardsonLucyDeconvolutionImageFilter_SetNumberOfIterations

`RichardsonLucyDeconvolutionImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`RichardsonLucyDeconvolutionImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__RichardsonLucyDeconvolutionImageFilter', 'integer')
class(`RichardsonLucyDeconvolutionImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_SetNumberOfIterations'))

# Start of RichardsonLucyDeconvolutionImageFilter_GetNumberOfIterations

`RichardsonLucyDeconvolutionImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RichardsonLucyDeconvolutionImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`RichardsonLucyDeconvolutionImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__RichardsonLucyDeconvolutionImageFilter')
class(`RichardsonLucyDeconvolutionImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_GetNumberOfIterations'))

# Start of RichardsonLucyDeconvolutionImageFilter_SetNormalize

`RichardsonLucyDeconvolutionImageFilter_SetNormalize` = function(self, Normalize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Normalize = as.logical(Normalize);
  ;ans = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_SetNormalize', self, Normalize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_SetNormalize`, 'returnType') = 'void'
attr(`RichardsonLucyDeconvolutionImageFilter_SetNormalize`, "inputTypes") = c('_p_itk__simple__RichardsonLucyDeconvolutionImageFilter', 'logical')
class(`RichardsonLucyDeconvolutionImageFilter_SetNormalize`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_SetNormalize'))

# Start of RichardsonLucyDeconvolutionImageFilter_NormalizeOn

`RichardsonLucyDeconvolutionImageFilter_NormalizeOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_NormalizeOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_NormalizeOn`, 'returnType') = 'void'
attr(`RichardsonLucyDeconvolutionImageFilter_NormalizeOn`, "inputTypes") = c('_p_itk__simple__RichardsonLucyDeconvolutionImageFilter')
class(`RichardsonLucyDeconvolutionImageFilter_NormalizeOn`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_NormalizeOn'))

# Start of RichardsonLucyDeconvolutionImageFilter_NormalizeOff

`RichardsonLucyDeconvolutionImageFilter_NormalizeOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_NormalizeOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_NormalizeOff`, 'returnType') = 'void'
attr(`RichardsonLucyDeconvolutionImageFilter_NormalizeOff`, "inputTypes") = c('_p_itk__simple__RichardsonLucyDeconvolutionImageFilter')
class(`RichardsonLucyDeconvolutionImageFilter_NormalizeOff`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_NormalizeOff'))

# Start of RichardsonLucyDeconvolutionImageFilter_GetNormalize

`RichardsonLucyDeconvolutionImageFilter_GetNormalize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RichardsonLucyDeconvolutionImageFilter_GetNormalize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_GetNormalize`, 'returnType') = 'logical'
attr(`RichardsonLucyDeconvolutionImageFilter_GetNormalize`, "inputTypes") = c('_p_itk__simple__RichardsonLucyDeconvolutionImageFilter')
class(`RichardsonLucyDeconvolutionImageFilter_GetNormalize`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_GetNormalize'))

# Start of RichardsonLucyDeconvolutionImageFilter_ZERO_PAD_get

`RichardsonLucyDeconvolutionImageFilter_ZERO_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_ZERO_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__RichardsonLucyDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_ZERO_PAD_get`, 'returnType') = 'character'
class(`RichardsonLucyDeconvolutionImageFilter_ZERO_PAD_get`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_ZERO_PAD_get'))

# Start of RichardsonLucyDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get

`RichardsonLucyDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__RichardsonLucyDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`, 'returnType') = 'character'
class(`RichardsonLucyDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get'))

# Start of RichardsonLucyDeconvolutionImageFilter_PERIODIC_PAD_get

`RichardsonLucyDeconvolutionImageFilter_PERIODIC_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_PERIODIC_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__RichardsonLucyDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_PERIODIC_PAD_get`, 'returnType') = 'character'
class(`RichardsonLucyDeconvolutionImageFilter_PERIODIC_PAD_get`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_PERIODIC_PAD_get'))

# Start of RichardsonLucyDeconvolutionImageFilter_SetBoundaryCondition

`RichardsonLucyDeconvolutionImageFilter_SetBoundaryCondition` = function(self, BoundaryCondition)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BoundaryCondition = enumToInteger(BoundaryCondition, "_itk__simple__RichardsonLucyDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(BoundaryCondition) > 1) {
    warning("using only the first element of BoundaryCondition");
  };
  
  ;ans = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_SetBoundaryCondition', self, BoundaryCondition, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_SetBoundaryCondition`, 'returnType') = 'void'
attr(`RichardsonLucyDeconvolutionImageFilter_SetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__RichardsonLucyDeconvolutionImageFilter', 'character')
class(`RichardsonLucyDeconvolutionImageFilter_SetBoundaryCondition`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_SetBoundaryCondition'))

# Start of RichardsonLucyDeconvolutionImageFilter_GetBoundaryCondition

`RichardsonLucyDeconvolutionImageFilter_GetBoundaryCondition` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_GetBoundaryCondition', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__RichardsonLucyDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_GetBoundaryCondition`, 'returnType') = 'character'
attr(`RichardsonLucyDeconvolutionImageFilter_GetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__RichardsonLucyDeconvolutionImageFilter')
class(`RichardsonLucyDeconvolutionImageFilter_GetBoundaryCondition`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_GetBoundaryCondition'))

# Start of RichardsonLucyDeconvolutionImageFilter_SAME_get

`RichardsonLucyDeconvolutionImageFilter_SAME_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_SAME_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__RichardsonLucyDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_SAME_get`, 'returnType') = 'character'
class(`RichardsonLucyDeconvolutionImageFilter_SAME_get`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_SAME_get'))

# Start of RichardsonLucyDeconvolutionImageFilter_VALID_get

`RichardsonLucyDeconvolutionImageFilter_VALID_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_VALID_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__RichardsonLucyDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_VALID_get`, 'returnType') = 'character'
class(`RichardsonLucyDeconvolutionImageFilter_VALID_get`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_VALID_get'))

# Start of RichardsonLucyDeconvolutionImageFilter_SetOutputRegionMode

`RichardsonLucyDeconvolutionImageFilter_SetOutputRegionMode` = function(self, OutputRegionMode)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputRegionMode = enumToInteger(OutputRegionMode, "_itk__simple__RichardsonLucyDeconvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(OutputRegionMode) > 1) {
    warning("using only the first element of OutputRegionMode");
  };
  
  ;ans = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_SetOutputRegionMode', self, OutputRegionMode, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_SetOutputRegionMode`, 'returnType') = 'void'
attr(`RichardsonLucyDeconvolutionImageFilter_SetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__RichardsonLucyDeconvolutionImageFilter', 'character')
class(`RichardsonLucyDeconvolutionImageFilter_SetOutputRegionMode`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_SetOutputRegionMode'))

# Start of RichardsonLucyDeconvolutionImageFilter_GetOutputRegionMode

`RichardsonLucyDeconvolutionImageFilter_GetOutputRegionMode` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_GetOutputRegionMode', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__RichardsonLucyDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_GetOutputRegionMode`, 'returnType') = 'character'
attr(`RichardsonLucyDeconvolutionImageFilter_GetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__RichardsonLucyDeconvolutionImageFilter')
class(`RichardsonLucyDeconvolutionImageFilter_GetOutputRegionMode`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_GetOutputRegionMode'))

# Start of RichardsonLucyDeconvolutionImageFilter_GetName

`RichardsonLucyDeconvolutionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RichardsonLucyDeconvolutionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_GetName`, 'returnType') = 'character'
attr(`RichardsonLucyDeconvolutionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__RichardsonLucyDeconvolutionImageFilter')
class(`RichardsonLucyDeconvolutionImageFilter_GetName`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_GetName'))

# Start of RichardsonLucyDeconvolutionImageFilter_ToString

`RichardsonLucyDeconvolutionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RichardsonLucyDeconvolutionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_ToString`, 'returnType') = 'character'
attr(`RichardsonLucyDeconvolutionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__RichardsonLucyDeconvolutionImageFilter')
class(`RichardsonLucyDeconvolutionImageFilter_ToString`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_ToString'))

# Start of RichardsonLucyDeconvolutionImageFilter_Execute

`RichardsonLucyDeconvolutionImageFilter_Execute` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_Execute', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RichardsonLucyDeconvolutionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`RichardsonLucyDeconvolutionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__RichardsonLucyDeconvolutionImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`RichardsonLucyDeconvolutionImageFilter_Execute`) = c("SWIGFunction", class('RichardsonLucyDeconvolutionImageFilter_Execute'))

# Start of accessor method for itk::simple::RichardsonLucyDeconvolutionImageFilter
setMethod('$', '_p_itk__simple__RichardsonLucyDeconvolutionImageFilter', function(x, name)

{
  accessorFuns = list('SetNumberOfIterations' = RichardsonLucyDeconvolutionImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = RichardsonLucyDeconvolutionImageFilter_GetNumberOfIterations, 'SetNormalize' = RichardsonLucyDeconvolutionImageFilter_SetNormalize, 'NormalizeOn' = RichardsonLucyDeconvolutionImageFilter_NormalizeOn, 'NormalizeOff' = RichardsonLucyDeconvolutionImageFilter_NormalizeOff, 'GetNormalize' = RichardsonLucyDeconvolutionImageFilter_GetNormalize, 'SetBoundaryCondition' = RichardsonLucyDeconvolutionImageFilter_SetBoundaryCondition, 'GetBoundaryCondition' = RichardsonLucyDeconvolutionImageFilter_GetBoundaryCondition, 'SetOutputRegionMode' = RichardsonLucyDeconvolutionImageFilter_SetOutputRegionMode, 'GetOutputRegionMode' = RichardsonLucyDeconvolutionImageFilter_GetOutputRegionMode, 'GetName' = RichardsonLucyDeconvolutionImageFilter_GetName, 'ToString' = RichardsonLucyDeconvolutionImageFilter_ToString, 'Execute' = RichardsonLucyDeconvolutionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::RichardsonLucyDeconvolutionImageFilter
# Start of RichardsonLucyDeconvolution

`RichardsonLucyDeconvolution__SWIG_0` = function(image1, image2, numberOfIterations, normalize, boundaryCondition, outputRegionMode, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__RichardsonLucyDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  outputRegionMode = enumToInteger(outputRegionMode, "_itk__simple__RichardsonLucyDeconvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(outputRegionMode) > 1) {
    warning("using only the first element of outputRegionMode");
  };
  
  ;ans = .Call('R_swig_RichardsonLucyDeconvolution__SWIG_0', image1, image2, numberOfIterations, normalize, boundaryCondition, outputRegionMode, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RichardsonLucyDeconvolution__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`RichardsonLucyDeconvolution__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'logical', 'character', 'character')
class(`RichardsonLucyDeconvolution__SWIG_0`) = c("SWIGFunction", class('RichardsonLucyDeconvolution__SWIG_0'))

# Start of RichardsonLucyDeconvolution

`RichardsonLucyDeconvolution__SWIG_1` = function(image1, image2, numberOfIterations, normalize, boundaryCondition, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__RichardsonLucyDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  ;ans = .Call('R_swig_RichardsonLucyDeconvolution__SWIG_1', image1, image2, numberOfIterations, normalize, boundaryCondition, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RichardsonLucyDeconvolution__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`RichardsonLucyDeconvolution__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'logical', 'character')
class(`RichardsonLucyDeconvolution__SWIG_1`) = c("SWIGFunction", class('RichardsonLucyDeconvolution__SWIG_1'))

# Start of RichardsonLucyDeconvolution

`RichardsonLucyDeconvolution__SWIG_2` = function(image1, image2, numberOfIterations, normalize, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  normalize = as.logical(normalize);
  ;ans = .Call('R_swig_RichardsonLucyDeconvolution__SWIG_2', image1, image2, numberOfIterations, normalize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RichardsonLucyDeconvolution__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`RichardsonLucyDeconvolution__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'logical')
class(`RichardsonLucyDeconvolution__SWIG_2`) = c("SWIGFunction", class('RichardsonLucyDeconvolution__SWIG_2'))

# Start of RichardsonLucyDeconvolution

`RichardsonLucyDeconvolution__SWIG_3` = function(image1, image2, numberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_RichardsonLucyDeconvolution__SWIG_3', image1, image2, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RichardsonLucyDeconvolution__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`RichardsonLucyDeconvolution__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`RichardsonLucyDeconvolution__SWIG_3`) = c("SWIGFunction", class('RichardsonLucyDeconvolution__SWIG_3'))

# Start of RichardsonLucyDeconvolution

`RichardsonLucyDeconvolution__SWIG_4` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_RichardsonLucyDeconvolution__SWIG_4', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RichardsonLucyDeconvolution__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`RichardsonLucyDeconvolution__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`RichardsonLucyDeconvolution__SWIG_4`) = c("SWIGFunction", class('RichardsonLucyDeconvolution__SWIG_4'))

`RichardsonLucyDeconvolution` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- RichardsonLucyDeconvolution__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- RichardsonLucyDeconvolution__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- RichardsonLucyDeconvolution__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.character(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- RichardsonLucyDeconvolution__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.character(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.character(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- RichardsonLucyDeconvolution__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for RichardsonLucyDeconvolution with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_RoundImageFilter

`delete_RoundImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_RoundImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_RoundImageFilter`, 'returnType') = 'void'
attr(`delete_RoundImageFilter`, "inputTypes") = c('_p_itk__simple__RoundImageFilter')
class(`delete_RoundImageFilter`) = c("SWIGFunction", class('delete_RoundImageFilter'))

# Start of new_RoundImageFilter

`RoundImageFilter` = function()
{
  ;ans = .Call('R_swig_new_RoundImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__RoundImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_RoundImageFilter);
  ans
  
}

attr(`RoundImageFilter`, 'returnType') = '_p_itk__simple__RoundImageFilter'
class(`RoundImageFilter`) = c("SWIGFunction", class('RoundImageFilter'))

# Start of RoundImageFilter_GetName

`RoundImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RoundImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RoundImageFilter_GetName`, 'returnType') = 'character'
attr(`RoundImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__RoundImageFilter')
class(`RoundImageFilter_GetName`) = c("SWIGFunction", class('RoundImageFilter_GetName'))

# Start of RoundImageFilter_ToString

`RoundImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RoundImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RoundImageFilter_ToString`, 'returnType') = 'character'
attr(`RoundImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__RoundImageFilter')
class(`RoundImageFilter_ToString`) = c("SWIGFunction", class('RoundImageFilter_ToString'))

# Start of RoundImageFilter_Execute

`RoundImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_RoundImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`RoundImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`RoundImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__RoundImageFilter', '_p_itk__simple__Image')
class(`RoundImageFilter_Execute`) = c("SWIGFunction", class('RoundImageFilter_Execute'))

# Start of accessor method for itk::simple::RoundImageFilter
setMethod('$', '_p_itk__simple__RoundImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = RoundImageFilter_GetName, 'ToString' = RoundImageFilter_ToString, 'Execute' = RoundImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::RoundImageFilter
# Start of Round

`Round` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Round', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Round`, 'returnType') = '_p_itk__simple__Image'
attr(`Round`, "inputTypes") = c('_p_itk__simple__Image')
class(`Round`) = c("SWIGFunction", class('Round'))

# Start of delete_SLICImageFilter

`delete_SLICImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SLICImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SLICImageFilter`, 'returnType') = 'void'
attr(`delete_SLICImageFilter`, "inputTypes") = c('_p_itk__simple__SLICImageFilter')
class(`delete_SLICImageFilter`) = c("SWIGFunction", class('delete_SLICImageFilter'))

# Start of new_SLICImageFilter

`SLICImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SLICImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SLICImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SLICImageFilter);
  ans
  
}

attr(`SLICImageFilter`, 'returnType') = '_p_itk__simple__SLICImageFilter'
class(`SLICImageFilter`) = c("SWIGFunction", class('SLICImageFilter'))

# Start of SLICImageFilter_SetSuperGridSize

`SLICImageFilter_SetSuperGridSize` = function(self, SuperGridSize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SuperGridSize = as.integer(SuperGridSize);
  ;ans = .Call('R_swig_SLICImageFilter_SetSuperGridSize', self, SuperGridSize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SLICImageFilter_SetSuperGridSize`, 'returnType') = 'void'
attr(`SLICImageFilter_SetSuperGridSize`, "inputTypes") = c('_p_itk__simple__SLICImageFilter', 'integer')
class(`SLICImageFilter_SetSuperGridSize`) = c("SWIGFunction", class('SLICImageFilter_SetSuperGridSize'))

# Start of SLICImageFilter_GetSuperGridSize

`SLICImageFilter_GetSuperGridSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SLICImageFilter_GetSuperGridSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SLICImageFilter_GetSuperGridSize`, 'returnType') = 'integer'
attr(`SLICImageFilter_GetSuperGridSize`, "inputTypes") = c('_p_itk__simple__SLICImageFilter')
class(`SLICImageFilter_GetSuperGridSize`) = c("SWIGFunction", class('SLICImageFilter_GetSuperGridSize'))

# Start of SLICImageFilter_SetSpatialProximityWeight

`SLICImageFilter_SetSpatialProximityWeight` = function(self, SpatialProximityWeight)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SLICImageFilter_SetSpatialProximityWeight', self, SpatialProximityWeight, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SLICImageFilter_SetSpatialProximityWeight`, 'returnType') = 'void'
attr(`SLICImageFilter_SetSpatialProximityWeight`, "inputTypes") = c('_p_itk__simple__SLICImageFilter', 'numeric')
class(`SLICImageFilter_SetSpatialProximityWeight`) = c("SWIGFunction", class('SLICImageFilter_SetSpatialProximityWeight'))

# Start of SLICImageFilter_GetSpatialProximityWeight

`SLICImageFilter_GetSpatialProximityWeight` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SLICImageFilter_GetSpatialProximityWeight', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SLICImageFilter_GetSpatialProximityWeight`, 'returnType') = 'numeric'
attr(`SLICImageFilter_GetSpatialProximityWeight`, "inputTypes") = c('_p_itk__simple__SLICImageFilter')
class(`SLICImageFilter_GetSpatialProximityWeight`) = c("SWIGFunction", class('SLICImageFilter_GetSpatialProximityWeight'))

# Start of SLICImageFilter_SetMaximumNumberOfIterations

`SLICImageFilter_SetMaximumNumberOfIterations` = function(self, MaximumNumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaximumNumberOfIterations = as.integer(MaximumNumberOfIterations);
  
  if(length(MaximumNumberOfIterations) > 1) {
    warning("using only the first element of MaximumNumberOfIterations");
  };
  
  ;ans = .Call('R_swig_SLICImageFilter_SetMaximumNumberOfIterations', self, MaximumNumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SLICImageFilter_SetMaximumNumberOfIterations`, 'returnType') = 'void'
attr(`SLICImageFilter_SetMaximumNumberOfIterations`, "inputTypes") = c('_p_itk__simple__SLICImageFilter', 'integer')
class(`SLICImageFilter_SetMaximumNumberOfIterations`) = c("SWIGFunction", class('SLICImageFilter_SetMaximumNumberOfIterations'))

# Start of SLICImageFilter_GetMaximumNumberOfIterations

`SLICImageFilter_GetMaximumNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SLICImageFilter_GetMaximumNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SLICImageFilter_GetMaximumNumberOfIterations`, 'returnType') = 'integer'
attr(`SLICImageFilter_GetMaximumNumberOfIterations`, "inputTypes") = c('_p_itk__simple__SLICImageFilter')
class(`SLICImageFilter_GetMaximumNumberOfIterations`) = c("SWIGFunction", class('SLICImageFilter_GetMaximumNumberOfIterations'))

# Start of SLICImageFilter_SetEnforceConnectivity

`SLICImageFilter_SetEnforceConnectivity` = function(self, EnforceConnectivity)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  EnforceConnectivity = as.logical(EnforceConnectivity);
  ;ans = .Call('R_swig_SLICImageFilter_SetEnforceConnectivity', self, EnforceConnectivity, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SLICImageFilter_SetEnforceConnectivity`, 'returnType') = 'void'
attr(`SLICImageFilter_SetEnforceConnectivity`, "inputTypes") = c('_p_itk__simple__SLICImageFilter', 'logical')
class(`SLICImageFilter_SetEnforceConnectivity`) = c("SWIGFunction", class('SLICImageFilter_SetEnforceConnectivity'))

# Start of SLICImageFilter_EnforceConnectivityOn

`SLICImageFilter_EnforceConnectivityOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SLICImageFilter_EnforceConnectivityOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SLICImageFilter_EnforceConnectivityOn`, 'returnType') = 'void'
attr(`SLICImageFilter_EnforceConnectivityOn`, "inputTypes") = c('_p_itk__simple__SLICImageFilter')
class(`SLICImageFilter_EnforceConnectivityOn`) = c("SWIGFunction", class('SLICImageFilter_EnforceConnectivityOn'))

# Start of SLICImageFilter_EnforceConnectivityOff

`SLICImageFilter_EnforceConnectivityOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SLICImageFilter_EnforceConnectivityOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SLICImageFilter_EnforceConnectivityOff`, 'returnType') = 'void'
attr(`SLICImageFilter_EnforceConnectivityOff`, "inputTypes") = c('_p_itk__simple__SLICImageFilter')
class(`SLICImageFilter_EnforceConnectivityOff`) = c("SWIGFunction", class('SLICImageFilter_EnforceConnectivityOff'))

# Start of SLICImageFilter_GetEnforceConnectivity

`SLICImageFilter_GetEnforceConnectivity` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SLICImageFilter_GetEnforceConnectivity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SLICImageFilter_GetEnforceConnectivity`, 'returnType') = 'logical'
attr(`SLICImageFilter_GetEnforceConnectivity`, "inputTypes") = c('_p_itk__simple__SLICImageFilter')
class(`SLICImageFilter_GetEnforceConnectivity`) = c("SWIGFunction", class('SLICImageFilter_GetEnforceConnectivity'))

# Start of SLICImageFilter_SetInitializationPerturbation

`SLICImageFilter_SetInitializationPerturbation` = function(self, InitializationPerturbation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InitializationPerturbation = as.logical(InitializationPerturbation);
  ;ans = .Call('R_swig_SLICImageFilter_SetInitializationPerturbation', self, InitializationPerturbation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SLICImageFilter_SetInitializationPerturbation`, 'returnType') = 'void'
attr(`SLICImageFilter_SetInitializationPerturbation`, "inputTypes") = c('_p_itk__simple__SLICImageFilter', 'logical')
class(`SLICImageFilter_SetInitializationPerturbation`) = c("SWIGFunction", class('SLICImageFilter_SetInitializationPerturbation'))

# Start of SLICImageFilter_InitializationPerturbationOn

`SLICImageFilter_InitializationPerturbationOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SLICImageFilter_InitializationPerturbationOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SLICImageFilter_InitializationPerturbationOn`, 'returnType') = 'void'
attr(`SLICImageFilter_InitializationPerturbationOn`, "inputTypes") = c('_p_itk__simple__SLICImageFilter')
class(`SLICImageFilter_InitializationPerturbationOn`) = c("SWIGFunction", class('SLICImageFilter_InitializationPerturbationOn'))

# Start of SLICImageFilter_InitializationPerturbationOff

`SLICImageFilter_InitializationPerturbationOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SLICImageFilter_InitializationPerturbationOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SLICImageFilter_InitializationPerturbationOff`, 'returnType') = 'void'
attr(`SLICImageFilter_InitializationPerturbationOff`, "inputTypes") = c('_p_itk__simple__SLICImageFilter')
class(`SLICImageFilter_InitializationPerturbationOff`) = c("SWIGFunction", class('SLICImageFilter_InitializationPerturbationOff'))

# Start of SLICImageFilter_GetInitializationPerturbation

`SLICImageFilter_GetInitializationPerturbation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SLICImageFilter_GetInitializationPerturbation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SLICImageFilter_GetInitializationPerturbation`, 'returnType') = 'logical'
attr(`SLICImageFilter_GetInitializationPerturbation`, "inputTypes") = c('_p_itk__simple__SLICImageFilter')
class(`SLICImageFilter_GetInitializationPerturbation`) = c("SWIGFunction", class('SLICImageFilter_GetInitializationPerturbation'))

# Start of SLICImageFilter_GetAverageResidual

`SLICImageFilter_GetAverageResidual` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SLICImageFilter_GetAverageResidual', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SLICImageFilter_GetAverageResidual`, 'returnType') = 'numeric'
attr(`SLICImageFilter_GetAverageResidual`, "inputTypes") = c('_p_itk__simple__SLICImageFilter')
class(`SLICImageFilter_GetAverageResidual`) = c("SWIGFunction", class('SLICImageFilter_GetAverageResidual'))

# Start of SLICImageFilter_GetName

`SLICImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SLICImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SLICImageFilter_GetName`, 'returnType') = 'character'
attr(`SLICImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SLICImageFilter')
class(`SLICImageFilter_GetName`) = c("SWIGFunction", class('SLICImageFilter_GetName'))

# Start of SLICImageFilter_ToString

`SLICImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SLICImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SLICImageFilter_ToString`, 'returnType') = 'character'
attr(`SLICImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SLICImageFilter')
class(`SLICImageFilter_ToString`) = c("SWIGFunction", class('SLICImageFilter_ToString'))

# Start of SLICImageFilter_Execute

`SLICImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SLICImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SLICImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`SLICImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__SLICImageFilter', '_p_itk__simple__Image')
class(`SLICImageFilter_Execute`) = c("SWIGFunction", class('SLICImageFilter_Execute'))

# Start of accessor method for itk::simple::SLICImageFilter
setMethod('$', '_p_itk__simple__SLICImageFilter', function(x, name)

{
  accessorFuns = list('SetSuperGridSize' = SLICImageFilter_SetSuperGridSize, 'GetSuperGridSize' = SLICImageFilter_GetSuperGridSize, 'SetSpatialProximityWeight' = SLICImageFilter_SetSpatialProximityWeight, 'GetSpatialProximityWeight' = SLICImageFilter_GetSpatialProximityWeight, 'SetMaximumNumberOfIterations' = SLICImageFilter_SetMaximumNumberOfIterations, 'GetMaximumNumberOfIterations' = SLICImageFilter_GetMaximumNumberOfIterations, 'SetEnforceConnectivity' = SLICImageFilter_SetEnforceConnectivity, 'EnforceConnectivityOn' = SLICImageFilter_EnforceConnectivityOn, 'EnforceConnectivityOff' = SLICImageFilter_EnforceConnectivityOff, 'GetEnforceConnectivity' = SLICImageFilter_GetEnforceConnectivity, 'SetInitializationPerturbation' = SLICImageFilter_SetInitializationPerturbation, 'InitializationPerturbationOn' = SLICImageFilter_InitializationPerturbationOn, 'InitializationPerturbationOff' = SLICImageFilter_InitializationPerturbationOff, 'GetInitializationPerturbation' = SLICImageFilter_GetInitializationPerturbation, 'GetAverageResidual' = SLICImageFilter_GetAverageResidual, 'GetName' = SLICImageFilter_GetName, 'ToString' = SLICImageFilter_ToString, 'Execute' = SLICImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SLICImageFilter
# Start of SLIC

`SLIC__SWIG_0` = function(image1, superGridSize, spatialProximityWeight, maximumNumberOfIterations, enforceConnectivity, initializationPerturbation, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  superGridSize = as.integer(superGridSize);
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  enforceConnectivity = as.logical(enforceConnectivity);
  initializationPerturbation = as.logical(initializationPerturbation);
  ;ans = .Call('R_swig_SLIC__SWIG_0', image1, superGridSize, spatialProximityWeight, maximumNumberOfIterations, enforceConnectivity, initializationPerturbation, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SLIC__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`SLIC__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'integer', 'logical', 'logical')
class(`SLIC__SWIG_0`) = c("SWIGFunction", class('SLIC__SWIG_0'))

# Start of SLIC

`SLIC__SWIG_1` = function(image1, superGridSize, spatialProximityWeight, maximumNumberOfIterations, enforceConnectivity, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  superGridSize = as.integer(superGridSize);
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  enforceConnectivity = as.logical(enforceConnectivity);
  ;ans = .Call('R_swig_SLIC__SWIG_1', image1, superGridSize, spatialProximityWeight, maximumNumberOfIterations, enforceConnectivity, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SLIC__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`SLIC__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'integer', 'logical')
class(`SLIC__SWIG_1`) = c("SWIGFunction", class('SLIC__SWIG_1'))

# Start of SLIC

`SLIC__SWIG_2` = function(image1, superGridSize, spatialProximityWeight, maximumNumberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  superGridSize = as.integer(superGridSize);
  
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  ;ans = .Call('R_swig_SLIC__SWIG_2', image1, superGridSize, spatialProximityWeight, maximumNumberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SLIC__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`SLIC__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'integer')
class(`SLIC__SWIG_2`) = c("SWIGFunction", class('SLIC__SWIG_2'))

# Start of SLIC

`SLIC__SWIG_3` = function(image1, superGridSize, spatialProximityWeight, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  superGridSize = as.integer(superGridSize);
  
  ;ans = .Call('R_swig_SLIC__SWIG_3', image1, superGridSize, spatialProximityWeight, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SLIC__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`SLIC__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric')
class(`SLIC__SWIG_3`) = c("SWIGFunction", class('SLIC__SWIG_3'))

# Start of SLIC

`SLIC__SWIG_4` = function(image1, superGridSize, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  superGridSize = as.integer(superGridSize);
  ;ans = .Call('R_swig_SLIC__SWIG_4', image1, superGridSize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SLIC__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`SLIC__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`SLIC__SWIG_4`) = c("SWIGFunction", class('SLIC__SWIG_4'))

# Start of SLIC

`SLIC__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SLIC__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SLIC__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`SLIC__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`SLIC__SWIG_5`) = c("SWIGFunction", class('SLIC__SWIG_5'))

`SLIC` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- SLIC__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- SLIC__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- SLIC__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- SLIC__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- SLIC__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- SLIC__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SLIC with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_STAPLEImageFilter

`delete_STAPLEImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_STAPLEImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_STAPLEImageFilter`, 'returnType') = 'void'
attr(`delete_STAPLEImageFilter`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter')
class(`delete_STAPLEImageFilter`) = c("SWIGFunction", class('delete_STAPLEImageFilter'))

# Start of new_STAPLEImageFilter

`STAPLEImageFilter` = function()
{
  ;ans = .Call('R_swig_new_STAPLEImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__STAPLEImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_STAPLEImageFilter);
  ans
  
}

attr(`STAPLEImageFilter`, 'returnType') = '_p_itk__simple__STAPLEImageFilter'
class(`STAPLEImageFilter`) = c("SWIGFunction", class('STAPLEImageFilter'))

# Start of STAPLEImageFilter_SetConfidenceWeight

`STAPLEImageFilter_SetConfidenceWeight` = function(self, ConfidenceWeight)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_STAPLEImageFilter_SetConfidenceWeight', self, ConfidenceWeight, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`STAPLEImageFilter_SetConfidenceWeight`, 'returnType') = 'void'
attr(`STAPLEImageFilter_SetConfidenceWeight`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter', 'numeric')
class(`STAPLEImageFilter_SetConfidenceWeight`) = c("SWIGFunction", class('STAPLEImageFilter_SetConfidenceWeight'))

# Start of STAPLEImageFilter_GetConfidenceWeight

`STAPLEImageFilter_GetConfidenceWeight` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_STAPLEImageFilter_GetConfidenceWeight', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`STAPLEImageFilter_GetConfidenceWeight`, 'returnType') = 'numeric'
attr(`STAPLEImageFilter_GetConfidenceWeight`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter')
class(`STAPLEImageFilter_GetConfidenceWeight`) = c("SWIGFunction", class('STAPLEImageFilter_GetConfidenceWeight'))

# Start of STAPLEImageFilter_SetForegroundValue

`STAPLEImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_STAPLEImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`STAPLEImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`STAPLEImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter', 'numeric')
class(`STAPLEImageFilter_SetForegroundValue`) = c("SWIGFunction", class('STAPLEImageFilter_SetForegroundValue'))

# Start of STAPLEImageFilter_GetForegroundValue

`STAPLEImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_STAPLEImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`STAPLEImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`STAPLEImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter')
class(`STAPLEImageFilter_GetForegroundValue`) = c("SWIGFunction", class('STAPLEImageFilter_GetForegroundValue'))

# Start of STAPLEImageFilter_SetMaximumIterations

`STAPLEImageFilter_SetMaximumIterations` = function(self, MaximumIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaximumIterations = as.integer(MaximumIterations);
  
  if(length(MaximumIterations) > 1) {
    warning("using only the first element of MaximumIterations");
  };
  
  ;ans = .Call('R_swig_STAPLEImageFilter_SetMaximumIterations', self, MaximumIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`STAPLEImageFilter_SetMaximumIterations`, 'returnType') = 'void'
attr(`STAPLEImageFilter_SetMaximumIterations`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter', 'integer')
class(`STAPLEImageFilter_SetMaximumIterations`) = c("SWIGFunction", class('STAPLEImageFilter_SetMaximumIterations'))

# Start of STAPLEImageFilter_GetMaximumIterations

`STAPLEImageFilter_GetMaximumIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_STAPLEImageFilter_GetMaximumIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`STAPLEImageFilter_GetMaximumIterations`, 'returnType') = 'integer'
attr(`STAPLEImageFilter_GetMaximumIterations`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter')
class(`STAPLEImageFilter_GetMaximumIterations`) = c("SWIGFunction", class('STAPLEImageFilter_GetMaximumIterations'))

# Start of STAPLEImageFilter_GetElapsedIterations

`STAPLEImageFilter_GetElapsedIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_STAPLEImageFilter_GetElapsedIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`STAPLEImageFilter_GetElapsedIterations`, 'returnType') = 'integer'
attr(`STAPLEImageFilter_GetElapsedIterations`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter')
class(`STAPLEImageFilter_GetElapsedIterations`) = c("SWIGFunction", class('STAPLEImageFilter_GetElapsedIterations'))

# Start of STAPLEImageFilter_GetSensitivity

`STAPLEImageFilter_GetSensitivity` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_STAPLEImageFilter_GetSensitivity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`STAPLEImageFilter_GetSensitivity`, 'returnType') = 'numeric'
attr(`STAPLEImageFilter_GetSensitivity`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter')
class(`STAPLEImageFilter_GetSensitivity`) = c("SWIGFunction", class('STAPLEImageFilter_GetSensitivity'))

# Start of STAPLEImageFilter_GetSpecificity

`STAPLEImageFilter_GetSpecificity` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_STAPLEImageFilter_GetSpecificity', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`STAPLEImageFilter_GetSpecificity`, 'returnType') = 'numeric'
attr(`STAPLEImageFilter_GetSpecificity`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter')
class(`STAPLEImageFilter_GetSpecificity`) = c("SWIGFunction", class('STAPLEImageFilter_GetSpecificity'))

# Start of STAPLEImageFilter_GetName

`STAPLEImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_STAPLEImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`STAPLEImageFilter_GetName`, 'returnType') = 'character'
attr(`STAPLEImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter')
class(`STAPLEImageFilter_GetName`) = c("SWIGFunction", class('STAPLEImageFilter_GetName'))

# Start of STAPLEImageFilter_ToString

`STAPLEImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_STAPLEImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`STAPLEImageFilter_ToString`, 'returnType') = 'character'
attr(`STAPLEImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter')
class(`STAPLEImageFilter_ToString`) = c("SWIGFunction", class('STAPLEImageFilter_ToString'))

# Start of STAPLEImageFilter_Execute

`STAPLEImageFilter_Execute__SWIG_0` = function(self, images, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_STAPLEImageFilter_Execute__SWIG_0', self, images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLEImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLEImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter', 'list')
class(`STAPLEImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('STAPLEImageFilter_Execute__SWIG_0'))

# Start of STAPLEImageFilter_Execute

`STAPLEImageFilter_Execute__SWIG_1` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_STAPLEImageFilter_Execute__SWIG_1', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLEImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLEImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter', '_p_itk__simple__Image')
class(`STAPLEImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('STAPLEImageFilter_Execute__SWIG_1'))

# Start of STAPLEImageFilter_Execute

`STAPLEImageFilter_Execute__SWIG_2` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_STAPLEImageFilter_Execute__SWIG_2', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLEImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLEImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`STAPLEImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('STAPLEImageFilter_Execute__SWIG_2'))

# Start of STAPLEImageFilter_Execute

`STAPLEImageFilter_Execute__SWIG_3` = function(self, image1, image2, image3, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_STAPLEImageFilter_Execute__SWIG_3', self, image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLEImageFilter_Execute__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLEImageFilter_Execute__SWIG_3`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`STAPLEImageFilter_Execute__SWIG_3`) = c("SWIGFunction", class('STAPLEImageFilter_Execute__SWIG_3'))

# Start of STAPLEImageFilter_Execute

`STAPLEImageFilter_Execute__SWIG_4` = function(self, image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_STAPLEImageFilter_Execute__SWIG_4', self, image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLEImageFilter_Execute__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLEImageFilter_Execute__SWIG_4`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`STAPLEImageFilter_Execute__SWIG_4`) = c("SWIGFunction", class('STAPLEImageFilter_Execute__SWIG_4'))

# Start of STAPLEImageFilter_Execute

`STAPLEImageFilter_Execute__SWIG_5` = function(self, image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_STAPLEImageFilter_Execute__SWIG_5', self, image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLEImageFilter_Execute__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLEImageFilter_Execute__SWIG_5`, "inputTypes") = c('_p_itk__simple__STAPLEImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`STAPLEImageFilter_Execute__SWIG_5`) = c("SWIGFunction", class('STAPLEImageFilter_Execute__SWIG_5'))

`STAPLEImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__STAPLEImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- STAPLEImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__STAPLEImageFilter') || is.null(argv[[1]]) ) && (
        (is.list(argv[[2]]) & all(sapply(argv[[2]], inherits, what='_p_itk__simple__Image')))
        )) {
      f <- STAPLEImageFilter_Execute__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__STAPLEImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- STAPLEImageFilter_Execute__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__STAPLEImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- STAPLEImageFilter_Execute__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__STAPLEImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- STAPLEImageFilter_Execute__SWIG_4; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__STAPLEImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( extends(argtypes[6], '_p_itk__simple__Image') && length(argv[[6]]) == 1 )) {
      f <- STAPLEImageFilter_Execute__SWIG_5; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for STAPLEImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::STAPLEImageFilter
setMethod('$', '_p_itk__simple__STAPLEImageFilter', function(x, name)

{
  accessorFuns = list('SetConfidenceWeight' = STAPLEImageFilter_SetConfidenceWeight, 'GetConfidenceWeight' = STAPLEImageFilter_GetConfidenceWeight, 'SetForegroundValue' = STAPLEImageFilter_SetForegroundValue, 'GetForegroundValue' = STAPLEImageFilter_GetForegroundValue, 'SetMaximumIterations' = STAPLEImageFilter_SetMaximumIterations, 'GetMaximumIterations' = STAPLEImageFilter_GetMaximumIterations, 'GetElapsedIterations' = STAPLEImageFilter_GetElapsedIterations, 'GetSensitivity' = STAPLEImageFilter_GetSensitivity, 'GetSpecificity' = STAPLEImageFilter_GetSpecificity, 'GetName' = STAPLEImageFilter_GetName, 'ToString' = STAPLEImageFilter_ToString, 'Execute' = STAPLEImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::STAPLEImageFilter
# Start of STAPLE

`STAPLE__SWIG_0` = function(images, confidenceWeight, foregroundValue, maximumIterations, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  
  
  maximumIterations = as.integer(maximumIterations);
  
  if(length(maximumIterations) > 1) {
    warning("using only the first element of maximumIterations");
  };
  
  ;ans = .Call('R_swig_STAPLE__SWIG_0', images, confidenceWeight, foregroundValue, maximumIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_0`, "inputTypes") = c('list', 'numeric', 'numeric', 'integer')
class(`STAPLE__SWIG_0`) = c("SWIGFunction", class('STAPLE__SWIG_0'))

# Start of STAPLE

`STAPLE__SWIG_1` = function(images, confidenceWeight, foregroundValue, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  
  
  ;ans = .Call('R_swig_STAPLE__SWIG_1', images, confidenceWeight, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_1`, "inputTypes") = c('list', 'numeric', 'numeric')
class(`STAPLE__SWIG_1`) = c("SWIGFunction", class('STAPLE__SWIG_1'))

# Start of STAPLE

`STAPLE__SWIG_2` = function(images, confidenceWeight, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  
  ;ans = .Call('R_swig_STAPLE__SWIG_2', images, confidenceWeight, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_2`, "inputTypes") = c('list', 'numeric')
class(`STAPLE__SWIG_2`) = c("SWIGFunction", class('STAPLE__SWIG_2'))

# Start of STAPLE

`STAPLE__SWIG_3` = function(images, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_STAPLE__SWIG_3', images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_3`, "inputTypes") = c('list')
class(`STAPLE__SWIG_3`) = c("SWIGFunction", class('STAPLE__SWIG_3'))

# Start of STAPLE

`STAPLE__SWIG_4` = function(image1, confidenceWeight, foregroundValue, maximumIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  maximumIterations = as.integer(maximumIterations);
  
  if(length(maximumIterations) > 1) {
    warning("using only the first element of maximumIterations");
  };
  
  ;ans = .Call('R_swig_STAPLE__SWIG_4', image1, confidenceWeight, foregroundValue, maximumIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'integer')
class(`STAPLE__SWIG_4`) = c("SWIGFunction", class('STAPLE__SWIG_4'))

# Start of STAPLE

`STAPLE__SWIG_5` = function(image1, confidenceWeight, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_STAPLE__SWIG_5', image1, confidenceWeight, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`STAPLE__SWIG_5`) = c("SWIGFunction", class('STAPLE__SWIG_5'))

# Start of STAPLE

`STAPLE__SWIG_6` = function(image1, confidenceWeight, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_STAPLE__SWIG_6', image1, confidenceWeight, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`STAPLE__SWIG_6`) = c("SWIGFunction", class('STAPLE__SWIG_6'))

# Start of STAPLE

`STAPLE__SWIG_7` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_STAPLE__SWIG_7', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image')
class(`STAPLE__SWIG_7`) = c("SWIGFunction", class('STAPLE__SWIG_7'))

# Start of STAPLE

`STAPLE__SWIG_8` = function(image1, image2, confidenceWeight, foregroundValue, maximumIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  
  maximumIterations = as.integer(maximumIterations);
  
  if(length(maximumIterations) > 1) {
    warning("using only the first element of maximumIterations");
  };
  
  ;ans = .Call('R_swig_STAPLE__SWIG_8', image1, image2, confidenceWeight, foregroundValue, maximumIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'integer')
class(`STAPLE__SWIG_8`) = c("SWIGFunction", class('STAPLE__SWIG_8'))

# Start of STAPLE

`STAPLE__SWIG_9` = function(image1, image2, confidenceWeight, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  
  ;ans = .Call('R_swig_STAPLE__SWIG_9', image1, image2, confidenceWeight, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`STAPLE__SWIG_9`) = c("SWIGFunction", class('STAPLE__SWIG_9'))

# Start of STAPLE

`STAPLE__SWIG_10` = function(image1, image2, confidenceWeight, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  ;ans = .Call('R_swig_STAPLE__SWIG_10', image1, image2, confidenceWeight, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`STAPLE__SWIG_10`) = c("SWIGFunction", class('STAPLE__SWIG_10'))

# Start of STAPLE

`STAPLE__SWIG_11` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_STAPLE__SWIG_11', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`STAPLE__SWIG_11`) = c("SWIGFunction", class('STAPLE__SWIG_11'))

# Start of STAPLE

`STAPLE__SWIG_12` = function(image1, image2, image3, confidenceWeight, foregroundValue, maximumIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  
  
  maximumIterations = as.integer(maximumIterations);
  
  if(length(maximumIterations) > 1) {
    warning("using only the first element of maximumIterations");
  };
  
  ;ans = .Call('R_swig_STAPLE__SWIG_12', image1, image2, image3, confidenceWeight, foregroundValue, maximumIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_12`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_12`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'integer')
class(`STAPLE__SWIG_12`) = c("SWIGFunction", class('STAPLE__SWIG_12'))

# Start of STAPLE

`STAPLE__SWIG_13` = function(image1, image2, image3, confidenceWeight, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  
  
  ;ans = .Call('R_swig_STAPLE__SWIG_13', image1, image2, image3, confidenceWeight, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_13`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_13`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`STAPLE__SWIG_13`) = c("SWIGFunction", class('STAPLE__SWIG_13'))

# Start of STAPLE

`STAPLE__SWIG_14` = function(image1, image2, image3, confidenceWeight, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  
  ;ans = .Call('R_swig_STAPLE__SWIG_14', image1, image2, image3, confidenceWeight, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_14`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_14`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`STAPLE__SWIG_14`) = c("SWIGFunction", class('STAPLE__SWIG_14'))

# Start of STAPLE

`STAPLE__SWIG_15` = function(image1, image2, image3, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_STAPLE__SWIG_15', image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_15`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_15`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`STAPLE__SWIG_15`) = c("SWIGFunction", class('STAPLE__SWIG_15'))

# Start of STAPLE

`STAPLE__SWIG_16` = function(image1, image2, image3, image4, confidenceWeight, foregroundValue, maximumIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  
  
  maximumIterations = as.integer(maximumIterations);
  
  if(length(maximumIterations) > 1) {
    warning("using only the first element of maximumIterations");
  };
  
  ;ans = .Call('R_swig_STAPLE__SWIG_16', image1, image2, image3, image4, confidenceWeight, foregroundValue, maximumIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_16`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_16`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'integer')
class(`STAPLE__SWIG_16`) = c("SWIGFunction", class('STAPLE__SWIG_16'))

# Start of STAPLE

`STAPLE__SWIG_17` = function(image1, image2, image3, image4, confidenceWeight, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  
  
  ;ans = .Call('R_swig_STAPLE__SWIG_17', image1, image2, image3, image4, confidenceWeight, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_17`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_17`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`STAPLE__SWIG_17`) = c("SWIGFunction", class('STAPLE__SWIG_17'))

# Start of STAPLE

`STAPLE__SWIG_18` = function(image1, image2, image3, image4, confidenceWeight, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  
  ;ans = .Call('R_swig_STAPLE__SWIG_18', image1, image2, image3, image4, confidenceWeight, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_18`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_18`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`STAPLE__SWIG_18`) = c("SWIGFunction", class('STAPLE__SWIG_18'))

# Start of STAPLE

`STAPLE__SWIG_19` = function(image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_STAPLE__SWIG_19', image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_19`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_19`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`STAPLE__SWIG_19`) = c("SWIGFunction", class('STAPLE__SWIG_19'))

# Start of STAPLE

`STAPLE__SWIG_20` = function(image1, image2, image3, image4, image5, confidenceWeight, foregroundValue, maximumIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  
  
  maximumIterations = as.integer(maximumIterations);
  
  if(length(maximumIterations) > 1) {
    warning("using only the first element of maximumIterations");
  };
  
  ;ans = .Call('R_swig_STAPLE__SWIG_20', image1, image2, image3, image4, image5, confidenceWeight, foregroundValue, maximumIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_20`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_20`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'integer')
class(`STAPLE__SWIG_20`) = c("SWIGFunction", class('STAPLE__SWIG_20'))

# Start of STAPLE

`STAPLE__SWIG_21` = function(image1, image2, image3, image4, image5, confidenceWeight, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  
  
  ;ans = .Call('R_swig_STAPLE__SWIG_21', image1, image2, image3, image4, image5, confidenceWeight, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_21`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_21`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`STAPLE__SWIG_21`) = c("SWIGFunction", class('STAPLE__SWIG_21'))

# Start of STAPLE

`STAPLE__SWIG_22` = function(image1, image2, image3, image4, image5, confidenceWeight, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  
  ;ans = .Call('R_swig_STAPLE__SWIG_22', image1, image2, image3, image4, image5, confidenceWeight, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_22`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_22`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`STAPLE__SWIG_22`) = c("SWIGFunction", class('STAPLE__SWIG_22'))

# Start of STAPLE

`STAPLE__SWIG_23` = function(image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_STAPLE__SWIG_23', image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`STAPLE__SWIG_23`, 'returnType') = '_p_itk__simple__Image'
attr(`STAPLE__SWIG_23`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`STAPLE__SWIG_23`) = c("SWIGFunction", class('STAPLE__SWIG_23'))

`STAPLE` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 24
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- STAPLE__SWIG_7; 
    }
    else if (
      (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
      ) {
      f <- STAPLE__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- STAPLE__SWIG_11; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- STAPLE__SWIG_6; 
    }
    else if ((
        (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
        ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- STAPLE__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- STAPLE__SWIG_15; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- STAPLE__SWIG_10; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- STAPLE__SWIG_5; 
    }
    else if ((
        (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
        ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- STAPLE__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- STAPLE__SWIG_19; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- STAPLE__SWIG_14; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- STAPLE__SWIG_9; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- STAPLE__SWIG_4; 
    }
    else if ((
        (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
        ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- STAPLE__SWIG_0; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- STAPLE__SWIG_23; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- STAPLE__SWIG_18; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- STAPLE__SWIG_13; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- STAPLE__SWIG_8; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- STAPLE__SWIG_22; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- STAPLE__SWIG_17; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- STAPLE__SWIG_12; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- STAPLE__SWIG_21; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- STAPLE__SWIG_16; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( (is.integer(argv[[8]]) || is.numeric(argv[[8]])) && length(argv[[8]]) == 1 )) {
      f <- STAPLE__SWIG_20; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for STAPLE with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_SaltAndPepperNoiseImageFilter

`delete_SaltAndPepperNoiseImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SaltAndPepperNoiseImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SaltAndPepperNoiseImageFilter`, 'returnType') = 'void'
attr(`delete_SaltAndPepperNoiseImageFilter`, "inputTypes") = c('_p_itk__simple__SaltAndPepperNoiseImageFilter')
class(`delete_SaltAndPepperNoiseImageFilter`) = c("SWIGFunction", class('delete_SaltAndPepperNoiseImageFilter'))

# Start of new_SaltAndPepperNoiseImageFilter

`SaltAndPepperNoiseImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SaltAndPepperNoiseImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SaltAndPepperNoiseImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SaltAndPepperNoiseImageFilter);
  ans
  
}

attr(`SaltAndPepperNoiseImageFilter`, 'returnType') = '_p_itk__simple__SaltAndPepperNoiseImageFilter'
class(`SaltAndPepperNoiseImageFilter`) = c("SWIGFunction", class('SaltAndPepperNoiseImageFilter'))

# Start of SaltAndPepperNoiseImageFilter_SetProbability

`SaltAndPepperNoiseImageFilter_SetProbability` = function(self, Probability)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SaltAndPepperNoiseImageFilter_SetProbability', self, Probability, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SaltAndPepperNoiseImageFilter_SetProbability`, 'returnType') = 'void'
attr(`SaltAndPepperNoiseImageFilter_SetProbability`, "inputTypes") = c('_p_itk__simple__SaltAndPepperNoiseImageFilter', 'numeric')
class(`SaltAndPepperNoiseImageFilter_SetProbability`) = c("SWIGFunction", class('SaltAndPepperNoiseImageFilter_SetProbability'))

# Start of SaltAndPepperNoiseImageFilter_GetProbability

`SaltAndPepperNoiseImageFilter_GetProbability` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SaltAndPepperNoiseImageFilter_GetProbability', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SaltAndPepperNoiseImageFilter_GetProbability`, 'returnType') = 'numeric'
attr(`SaltAndPepperNoiseImageFilter_GetProbability`, "inputTypes") = c('_p_itk__simple__SaltAndPepperNoiseImageFilter')
class(`SaltAndPepperNoiseImageFilter_GetProbability`) = c("SWIGFunction", class('SaltAndPepperNoiseImageFilter_GetProbability'))

# Start of SaltAndPepperNoiseImageFilter_SetSeed

`SaltAndPepperNoiseImageFilter_SetSeed` = function(self, Seed)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Seed = as.integer(Seed);
  
  if(length(Seed) > 1) {
    warning("using only the first element of Seed");
  };
  
  ;ans = .Call('R_swig_SaltAndPepperNoiseImageFilter_SetSeed', self, Seed, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SaltAndPepperNoiseImageFilter_SetSeed`, 'returnType') = 'void'
attr(`SaltAndPepperNoiseImageFilter_SetSeed`, "inputTypes") = c('_p_itk__simple__SaltAndPepperNoiseImageFilter', 'integer')
class(`SaltAndPepperNoiseImageFilter_SetSeed`) = c("SWIGFunction", class('SaltAndPepperNoiseImageFilter_SetSeed'))

# Start of SaltAndPepperNoiseImageFilter_GetSeed

`SaltAndPepperNoiseImageFilter_GetSeed` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SaltAndPepperNoiseImageFilter_GetSeed', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SaltAndPepperNoiseImageFilter_GetSeed`, 'returnType') = 'integer'
attr(`SaltAndPepperNoiseImageFilter_GetSeed`, "inputTypes") = c('_p_itk__simple__SaltAndPepperNoiseImageFilter')
class(`SaltAndPepperNoiseImageFilter_GetSeed`) = c("SWIGFunction", class('SaltAndPepperNoiseImageFilter_GetSeed'))

# Start of SaltAndPepperNoiseImageFilter_GetName

`SaltAndPepperNoiseImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SaltAndPepperNoiseImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SaltAndPepperNoiseImageFilter_GetName`, 'returnType') = 'character'
attr(`SaltAndPepperNoiseImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SaltAndPepperNoiseImageFilter')
class(`SaltAndPepperNoiseImageFilter_GetName`) = c("SWIGFunction", class('SaltAndPepperNoiseImageFilter_GetName'))

# Start of SaltAndPepperNoiseImageFilter_ToString

`SaltAndPepperNoiseImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SaltAndPepperNoiseImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SaltAndPepperNoiseImageFilter_ToString`, 'returnType') = 'character'
attr(`SaltAndPepperNoiseImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SaltAndPepperNoiseImageFilter')
class(`SaltAndPepperNoiseImageFilter_ToString`) = c("SWIGFunction", class('SaltAndPepperNoiseImageFilter_ToString'))

# Start of SaltAndPepperNoiseImageFilter_Execute

`SaltAndPepperNoiseImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SaltAndPepperNoiseImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SaltAndPepperNoiseImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`SaltAndPepperNoiseImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__SaltAndPepperNoiseImageFilter', '_p_itk__simple__Image')
class(`SaltAndPepperNoiseImageFilter_Execute`) = c("SWIGFunction", class('SaltAndPepperNoiseImageFilter_Execute'))

# Start of accessor method for itk::simple::SaltAndPepperNoiseImageFilter
setMethod('$', '_p_itk__simple__SaltAndPepperNoiseImageFilter', function(x, name)

{
  accessorFuns = list('SetProbability' = SaltAndPepperNoiseImageFilter_SetProbability, 'GetProbability' = SaltAndPepperNoiseImageFilter_GetProbability, 'SetSeed' = SaltAndPepperNoiseImageFilter_SetSeed, 'GetSeed' = SaltAndPepperNoiseImageFilter_GetSeed, 'GetName' = SaltAndPepperNoiseImageFilter_GetName, 'ToString' = SaltAndPepperNoiseImageFilter_ToString, 'Execute' = SaltAndPepperNoiseImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SaltAndPepperNoiseImageFilter
# Start of SaltAndPepperNoise

`SaltAndPepperNoise__SWIG_0` = function(image1, probability, seed, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  seed = as.integer(seed);
  
  if(length(seed) > 1) {
    warning("using only the first element of seed");
  };
  
  ;ans = .Call('R_swig_SaltAndPepperNoise__SWIG_0', image1, probability, seed, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SaltAndPepperNoise__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`SaltAndPepperNoise__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`SaltAndPepperNoise__SWIG_0`) = c("SWIGFunction", class('SaltAndPepperNoise__SWIG_0'))

# Start of SaltAndPepperNoise

`SaltAndPepperNoise__SWIG_1` = function(image1, probability, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_SaltAndPepperNoise__SWIG_1', image1, probability, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SaltAndPepperNoise__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`SaltAndPepperNoise__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`SaltAndPepperNoise__SWIG_1`) = c("SWIGFunction", class('SaltAndPepperNoise__SWIG_1'))

# Start of SaltAndPepperNoise

`SaltAndPepperNoise__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SaltAndPepperNoise__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SaltAndPepperNoise__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`SaltAndPepperNoise__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`SaltAndPepperNoise__SWIG_2`) = c("SWIGFunction", class('SaltAndPepperNoise__SWIG_2'))

`SaltAndPepperNoise` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- SaltAndPepperNoise__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- SaltAndPepperNoise__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- SaltAndPepperNoise__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SaltAndPepperNoise with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ScalarChanAndVeseDenseLevelSetImageFilter

`delete_ScalarChanAndVeseDenseLevelSetImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ScalarChanAndVeseDenseLevelSetImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ScalarChanAndVeseDenseLevelSetImageFilter`, 'returnType') = 'void'
attr(`delete_ScalarChanAndVeseDenseLevelSetImageFilter`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`delete_ScalarChanAndVeseDenseLevelSetImageFilter`) = c("SWIGFunction", class('delete_ScalarChanAndVeseDenseLevelSetImageFilter'))

# Start of new_ScalarChanAndVeseDenseLevelSetImageFilter

`ScalarChanAndVeseDenseLevelSetImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ScalarChanAndVeseDenseLevelSetImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScalarChanAndVeseDenseLevelSetImageFilter);
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter`, 'returnType') = '_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter'
class(`ScalarChanAndVeseDenseLevelSetImageFilter`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_SetMaximumRMSError

`ScalarChanAndVeseDenseLevelSetImageFilter_SetMaximumRMSError` = function(self, MaximumRMSError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_SetMaximumRMSError', self, MaximumRMSError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetMaximumRMSError`, 'returnType') = 'void'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter', 'numeric')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_SetMaximumRMSError`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_SetMaximumRMSError'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_GetMaximumRMSError

`ScalarChanAndVeseDenseLevelSetImageFilter_GetMaximumRMSError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_GetMaximumRMSError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetMaximumRMSError`, 'returnType') = 'numeric'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_GetMaximumRMSError`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_GetMaximumRMSError'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_SetNumberOfIterations

`ScalarChanAndVeseDenseLevelSetImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter', 'integer')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_SetNumberOfIterations'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_GetNumberOfIterations

`ScalarChanAndVeseDenseLevelSetImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_GetNumberOfIterations'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda1

`ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda1` = function(self, Lambda1)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda1', self, Lambda1, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda1`, 'returnType') = 'void'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda1`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter', 'numeric')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda1`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda1'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda1

`ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda1` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda1', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda1`, 'returnType') = 'numeric'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda1`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda1`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda1'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda2

`ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda2` = function(self, Lambda2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda2', self, Lambda2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda2`, 'returnType') = 'void'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda2`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter', 'numeric')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda2`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda2'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda2

`ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda2` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda2', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda2`, 'returnType') = 'numeric'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda2`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda2`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda2'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_SetEpsilon

`ScalarChanAndVeseDenseLevelSetImageFilter_SetEpsilon` = function(self, Epsilon)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_SetEpsilon', self, Epsilon, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetEpsilon`, 'returnType') = 'void'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetEpsilon`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter', 'numeric')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_SetEpsilon`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_SetEpsilon'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_GetEpsilon

`ScalarChanAndVeseDenseLevelSetImageFilter_GetEpsilon` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_GetEpsilon', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetEpsilon`, 'returnType') = 'numeric'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetEpsilon`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_GetEpsilon`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_GetEpsilon'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_SetCurvatureWeight

`ScalarChanAndVeseDenseLevelSetImageFilter_SetCurvatureWeight` = function(self, CurvatureWeight)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_SetCurvatureWeight', self, CurvatureWeight, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetCurvatureWeight`, 'returnType') = 'void'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetCurvatureWeight`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter', 'numeric')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_SetCurvatureWeight`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_SetCurvatureWeight'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_GetCurvatureWeight

`ScalarChanAndVeseDenseLevelSetImageFilter_GetCurvatureWeight` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_GetCurvatureWeight', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetCurvatureWeight`, 'returnType') = 'numeric'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetCurvatureWeight`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_GetCurvatureWeight`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_GetCurvatureWeight'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_SetAreaWeight

`ScalarChanAndVeseDenseLevelSetImageFilter_SetAreaWeight` = function(self, AreaWeight)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_SetAreaWeight', self, AreaWeight, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetAreaWeight`, 'returnType') = 'void'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetAreaWeight`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter', 'numeric')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_SetAreaWeight`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_SetAreaWeight'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_GetAreaWeight

`ScalarChanAndVeseDenseLevelSetImageFilter_GetAreaWeight` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_GetAreaWeight', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetAreaWeight`, 'returnType') = 'numeric'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetAreaWeight`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_GetAreaWeight`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_GetAreaWeight'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_SetReinitializationSmoothingWeight

`ScalarChanAndVeseDenseLevelSetImageFilter_SetReinitializationSmoothingWeight` = function(self, ReinitializationSmoothingWeight)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_SetReinitializationSmoothingWeight', self, ReinitializationSmoothingWeight, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetReinitializationSmoothingWeight`, 'returnType') = 'void'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetReinitializationSmoothingWeight`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter', 'numeric')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_SetReinitializationSmoothingWeight`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_SetReinitializationSmoothingWeight'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_GetReinitializationSmoothingWeight

`ScalarChanAndVeseDenseLevelSetImageFilter_GetReinitializationSmoothingWeight` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_GetReinitializationSmoothingWeight', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetReinitializationSmoothingWeight`, 'returnType') = 'numeric'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetReinitializationSmoothingWeight`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_GetReinitializationSmoothingWeight`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_GetReinitializationSmoothingWeight'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_SetVolume

`ScalarChanAndVeseDenseLevelSetImageFilter_SetVolume` = function(self, Volume)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_SetVolume', self, Volume, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetVolume`, 'returnType') = 'void'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetVolume`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter', 'numeric')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_SetVolume`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_SetVolume'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_GetVolume

`ScalarChanAndVeseDenseLevelSetImageFilter_GetVolume` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_GetVolume', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetVolume`, 'returnType') = 'numeric'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetVolume`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_GetVolume`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_GetVolume'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_SetVolumeMatchingWeight

`ScalarChanAndVeseDenseLevelSetImageFilter_SetVolumeMatchingWeight` = function(self, VolumeMatchingWeight)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_SetVolumeMatchingWeight', self, VolumeMatchingWeight, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetVolumeMatchingWeight`, 'returnType') = 'void'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetVolumeMatchingWeight`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter', 'numeric')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_SetVolumeMatchingWeight`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_SetVolumeMatchingWeight'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_GetVolumeMatchingWeight

`ScalarChanAndVeseDenseLevelSetImageFilter_GetVolumeMatchingWeight` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_GetVolumeMatchingWeight', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetVolumeMatchingWeight`, 'returnType') = 'numeric'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetVolumeMatchingWeight`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_GetVolumeMatchingWeight`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_GetVolumeMatchingWeight'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_AtanRegularizedHeaviside_get

`ScalarChanAndVeseDenseLevelSetImageFilter_AtanRegularizedHeaviside_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_AtanRegularizedHeaviside_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter__HeavisideStepFunctionType");
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_AtanRegularizedHeaviside_get`, 'returnType') = 'character'
class(`ScalarChanAndVeseDenseLevelSetImageFilter_AtanRegularizedHeaviside_get`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_AtanRegularizedHeaviside_get'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_SinRegularizedHeaviside_get

`ScalarChanAndVeseDenseLevelSetImageFilter_SinRegularizedHeaviside_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_SinRegularizedHeaviside_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter__HeavisideStepFunctionType");
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SinRegularizedHeaviside_get`, 'returnType') = 'character'
class(`ScalarChanAndVeseDenseLevelSetImageFilter_SinRegularizedHeaviside_get`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_SinRegularizedHeaviside_get'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_Heaviside_get

`ScalarChanAndVeseDenseLevelSetImageFilter_Heaviside_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_Heaviside_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter__HeavisideStepFunctionType");
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_Heaviside_get`, 'returnType') = 'character'
class(`ScalarChanAndVeseDenseLevelSetImageFilter_Heaviside_get`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_Heaviside_get'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_SetHeavisideStepFunction

`ScalarChanAndVeseDenseLevelSetImageFilter_SetHeavisideStepFunction` = function(self, HeavisideStepFunction)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  HeavisideStepFunction = enumToInteger(HeavisideStepFunction, "_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter__HeavisideStepFunctionType"); 
  
  if(length(HeavisideStepFunction) > 1) {
    warning("using only the first element of HeavisideStepFunction");
  };
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_SetHeavisideStepFunction', self, HeavisideStepFunction, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetHeavisideStepFunction`, 'returnType') = 'void'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetHeavisideStepFunction`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter', 'character')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_SetHeavisideStepFunction`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_SetHeavisideStepFunction'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_GetHeavisideStepFunction

`ScalarChanAndVeseDenseLevelSetImageFilter_GetHeavisideStepFunction` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_GetHeavisideStepFunction', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter__HeavisideStepFunctionType");
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetHeavisideStepFunction`, 'returnType') = 'character'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetHeavisideStepFunction`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_GetHeavisideStepFunction`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_GetHeavisideStepFunction'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_SetUseImageSpacing

`ScalarChanAndVeseDenseLevelSetImageFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter', 'logical')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_SetUseImageSpacing`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_SetUseImageSpacing'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOn

`ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOn`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOn'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOff

`ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOff`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOff'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_GetUseImageSpacing

`ScalarChanAndVeseDenseLevelSetImageFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_GetUseImageSpacing`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_GetUseImageSpacing'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_GetElapsedIterations

`ScalarChanAndVeseDenseLevelSetImageFilter_GetElapsedIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_GetElapsedIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetElapsedIterations`, 'returnType') = 'integer'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetElapsedIterations`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_GetElapsedIterations`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_GetElapsedIterations'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_GetRMSChange

`ScalarChanAndVeseDenseLevelSetImageFilter_GetRMSChange` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_GetRMSChange', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetRMSChange`, 'returnType') = 'numeric'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetRMSChange`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_GetRMSChange`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_GetRMSChange'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_GetName

`ScalarChanAndVeseDenseLevelSetImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetName`, 'returnType') = 'character'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_GetName`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_GetName'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_ToString

`ScalarChanAndVeseDenseLevelSetImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_ToString`, 'returnType') = 'character'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_ToString`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_ToString'))

# Start of ScalarChanAndVeseDenseLevelSetImageFilter_Execute

`ScalarChanAndVeseDenseLevelSetImageFilter_Execute` = function(self, initialImage, featureImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_Execute', self, initialImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSetImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarChanAndVeseDenseLevelSetImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ScalarChanAndVeseDenseLevelSetImageFilter_Execute`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSetImageFilter_Execute'))

# Start of accessor method for itk::simple::ScalarChanAndVeseDenseLevelSetImageFilter
setMethod('$', '_p_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter', function(x, name)

{
  accessorFuns = list('SetMaximumRMSError' = ScalarChanAndVeseDenseLevelSetImageFilter_SetMaximumRMSError, 'GetMaximumRMSError' = ScalarChanAndVeseDenseLevelSetImageFilter_GetMaximumRMSError, 'SetNumberOfIterations' = ScalarChanAndVeseDenseLevelSetImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = ScalarChanAndVeseDenseLevelSetImageFilter_GetNumberOfIterations, 'SetLambda1' = ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda1, 'GetLambda1' = ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda1, 'SetLambda2' = ScalarChanAndVeseDenseLevelSetImageFilter_SetLambda2, 'GetLambda2' = ScalarChanAndVeseDenseLevelSetImageFilter_GetLambda2, 'SetEpsilon' = ScalarChanAndVeseDenseLevelSetImageFilter_SetEpsilon, 'GetEpsilon' = ScalarChanAndVeseDenseLevelSetImageFilter_GetEpsilon, 'SetCurvatureWeight' = ScalarChanAndVeseDenseLevelSetImageFilter_SetCurvatureWeight, 'GetCurvatureWeight' = ScalarChanAndVeseDenseLevelSetImageFilter_GetCurvatureWeight, 'SetAreaWeight' = ScalarChanAndVeseDenseLevelSetImageFilter_SetAreaWeight, 'GetAreaWeight' = ScalarChanAndVeseDenseLevelSetImageFilter_GetAreaWeight, 'SetReinitializationSmoothingWeight' = ScalarChanAndVeseDenseLevelSetImageFilter_SetReinitializationSmoothingWeight, 'GetReinitializationSmoothingWeight' = ScalarChanAndVeseDenseLevelSetImageFilter_GetReinitializationSmoothingWeight, 'SetVolume' = ScalarChanAndVeseDenseLevelSetImageFilter_SetVolume, 'GetVolume' = ScalarChanAndVeseDenseLevelSetImageFilter_GetVolume, 'SetVolumeMatchingWeight' = ScalarChanAndVeseDenseLevelSetImageFilter_SetVolumeMatchingWeight, 'GetVolumeMatchingWeight' = ScalarChanAndVeseDenseLevelSetImageFilter_GetVolumeMatchingWeight, 'SetHeavisideStepFunction' = ScalarChanAndVeseDenseLevelSetImageFilter_SetHeavisideStepFunction, 'GetHeavisideStepFunction' = ScalarChanAndVeseDenseLevelSetImageFilter_GetHeavisideStepFunction, 'SetUseImageSpacing' = ScalarChanAndVeseDenseLevelSetImageFilter_SetUseImageSpacing, 'UseImageSpacingOn' = ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOn, 'UseImageSpacingOff' = ScalarChanAndVeseDenseLevelSetImageFilter_UseImageSpacingOff, 'GetUseImageSpacing' = ScalarChanAndVeseDenseLevelSetImageFilter_GetUseImageSpacing, 'GetElapsedIterations' = ScalarChanAndVeseDenseLevelSetImageFilter_GetElapsedIterations, 'GetRMSChange' = ScalarChanAndVeseDenseLevelSetImageFilter_GetRMSChange, 'GetName' = ScalarChanAndVeseDenseLevelSetImageFilter_GetName, 'ToString' = ScalarChanAndVeseDenseLevelSetImageFilter_ToString, 'Execute' = ScalarChanAndVeseDenseLevelSetImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ScalarChanAndVeseDenseLevelSetImageFilter
# Start of ScalarChanAndVeseDenseLevelSet

`ScalarChanAndVeseDenseLevelSet__SWIG_0` = function(initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, curvatureWeight, areaWeight, reinitializationSmoothingWeight, volume, volumeMatchingWeight, heavisideStepFunction, useImageSpacing, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  
  
  
  
  
  
  heavisideStepFunction = enumToInteger(heavisideStepFunction, "_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter__HeavisideStepFunctionType"); 
  
  if(length(heavisideStepFunction) > 1) {
    warning("using only the first element of heavisideStepFunction");
  };
  
  useImageSpacing = as.logical(useImageSpacing);
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSet__SWIG_0', initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, curvatureWeight, areaWeight, reinitializationSmoothingWeight, volume, volumeMatchingWeight, heavisideStepFunction, useImageSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSet__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarChanAndVeseDenseLevelSet__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'character', 'logical')
class(`ScalarChanAndVeseDenseLevelSet__SWIG_0`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSet__SWIG_0'))

# Start of ScalarChanAndVeseDenseLevelSet

`ScalarChanAndVeseDenseLevelSet__SWIG_1` = function(initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, curvatureWeight, areaWeight, reinitializationSmoothingWeight, volume, volumeMatchingWeight, heavisideStepFunction, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  
  
  
  
  
  
  heavisideStepFunction = enumToInteger(heavisideStepFunction, "_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter__HeavisideStepFunctionType"); 
  
  if(length(heavisideStepFunction) > 1) {
    warning("using only the first element of heavisideStepFunction");
  };
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSet__SWIG_1', initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, curvatureWeight, areaWeight, reinitializationSmoothingWeight, volume, volumeMatchingWeight, heavisideStepFunction, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSet__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarChanAndVeseDenseLevelSet__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'character')
class(`ScalarChanAndVeseDenseLevelSet__SWIG_1`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSet__SWIG_1'))

# Start of ScalarChanAndVeseDenseLevelSet

`ScalarChanAndVeseDenseLevelSet__SWIG_2` = function(initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, curvatureWeight, areaWeight, reinitializationSmoothingWeight, volume, volumeMatchingWeight, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  
  
  
  
  
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSet__SWIG_2', initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, curvatureWeight, areaWeight, reinitializationSmoothingWeight, volume, volumeMatchingWeight, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSet__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarChanAndVeseDenseLevelSet__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
class(`ScalarChanAndVeseDenseLevelSet__SWIG_2`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSet__SWIG_2'))

# Start of ScalarChanAndVeseDenseLevelSet

`ScalarChanAndVeseDenseLevelSet__SWIG_3` = function(initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, curvatureWeight, areaWeight, reinitializationSmoothingWeight, volume, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  
  
  
  
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSet__SWIG_3', initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, curvatureWeight, areaWeight, reinitializationSmoothingWeight, volume, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSet__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarChanAndVeseDenseLevelSet__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
class(`ScalarChanAndVeseDenseLevelSet__SWIG_3`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSet__SWIG_3'))

# Start of ScalarChanAndVeseDenseLevelSet

`ScalarChanAndVeseDenseLevelSet__SWIG_4` = function(initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, curvatureWeight, areaWeight, reinitializationSmoothingWeight, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  
  
  
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSet__SWIG_4', initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, curvatureWeight, areaWeight, reinitializationSmoothingWeight, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSet__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarChanAndVeseDenseLevelSet__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
class(`ScalarChanAndVeseDenseLevelSet__SWIG_4`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSet__SWIG_4'))

# Start of ScalarChanAndVeseDenseLevelSet

`ScalarChanAndVeseDenseLevelSet__SWIG_5` = function(initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, curvatureWeight, areaWeight, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  
  
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSet__SWIG_5', initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, curvatureWeight, areaWeight, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSet__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarChanAndVeseDenseLevelSet__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
class(`ScalarChanAndVeseDenseLevelSet__SWIG_5`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSet__SWIG_5'))

# Start of ScalarChanAndVeseDenseLevelSet

`ScalarChanAndVeseDenseLevelSet__SWIG_6` = function(initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, curvatureWeight, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSet__SWIG_6', initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, curvatureWeight, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSet__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarChanAndVeseDenseLevelSet__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'numeric', 'numeric')
class(`ScalarChanAndVeseDenseLevelSet__SWIG_6`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSet__SWIG_6'))

# Start of ScalarChanAndVeseDenseLevelSet

`ScalarChanAndVeseDenseLevelSet__SWIG_7` = function(initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSet__SWIG_7', initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, epsilon, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSet__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarChanAndVeseDenseLevelSet__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric', 'numeric')
class(`ScalarChanAndVeseDenseLevelSet__SWIG_7`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSet__SWIG_7'))

# Start of ScalarChanAndVeseDenseLevelSet

`ScalarChanAndVeseDenseLevelSet__SWIG_8` = function(initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSet__SWIG_8', initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, lambda2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSet__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarChanAndVeseDenseLevelSet__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric', 'numeric')
class(`ScalarChanAndVeseDenseLevelSet__SWIG_8`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSet__SWIG_8'))

# Start of ScalarChanAndVeseDenseLevelSet

`ScalarChanAndVeseDenseLevelSet__SWIG_9` = function(initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSet__SWIG_9', initialImage, featureImage, maximumRMSError, numberOfIterations, lambda1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSet__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarChanAndVeseDenseLevelSet__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer', 'numeric')
class(`ScalarChanAndVeseDenseLevelSet__SWIG_9`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSet__SWIG_9'))

# Start of ScalarChanAndVeseDenseLevelSet

`ScalarChanAndVeseDenseLevelSet__SWIG_10` = function(initialImage, featureImage, maximumRMSError, numberOfIterations, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSet__SWIG_10', initialImage, featureImage, maximumRMSError, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSet__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarChanAndVeseDenseLevelSet__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'integer')
class(`ScalarChanAndVeseDenseLevelSet__SWIG_10`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSet__SWIG_10'))

# Start of ScalarChanAndVeseDenseLevelSet

`ScalarChanAndVeseDenseLevelSet__SWIG_11` = function(initialImage, featureImage, maximumRMSError, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSet__SWIG_11', initialImage, featureImage, maximumRMSError, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSet__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarChanAndVeseDenseLevelSet__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`ScalarChanAndVeseDenseLevelSet__SWIG_11`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSet__SWIG_11'))

# Start of ScalarChanAndVeseDenseLevelSet

`ScalarChanAndVeseDenseLevelSet__SWIG_12` = function(initialImage, featureImage, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_ScalarChanAndVeseDenseLevelSet__SWIG_12', initialImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarChanAndVeseDenseLevelSet__SWIG_12`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarChanAndVeseDenseLevelSet__SWIG_12`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ScalarChanAndVeseDenseLevelSet__SWIG_12`) = c("SWIGFunction", class('ScalarChanAndVeseDenseLevelSet__SWIG_12'))

`ScalarChanAndVeseDenseLevelSet` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 13
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- ScalarChanAndVeseDenseLevelSet__SWIG_12; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ScalarChanAndVeseDenseLevelSet__SWIG_11; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- ScalarChanAndVeseDenseLevelSet__SWIG_10; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- ScalarChanAndVeseDenseLevelSet__SWIG_9; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- ScalarChanAndVeseDenseLevelSet__SWIG_8; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- ScalarChanAndVeseDenseLevelSet__SWIG_7; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 )) {
      f <- ScalarChanAndVeseDenseLevelSet__SWIG_6; 
    }
  } else if (argc == 9) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( is.numeric(argv[[9]]) && length(argv[[9]]) == 1 )) {
      f <- ScalarChanAndVeseDenseLevelSet__SWIG_5; 
    }
  } else if (argc == 10) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( is.numeric(argv[[9]]) && length(argv[[9]]) == 1 ) && ( is.numeric(argv[[10]]) && length(argv[[10]]) == 1 )) {
      f <- ScalarChanAndVeseDenseLevelSet__SWIG_4; 
    }
  } else if (argc == 11) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( is.numeric(argv[[9]]) && length(argv[[9]]) == 1 ) && ( is.numeric(argv[[10]]) && length(argv[[10]]) == 1 ) && ( is.numeric(argv[[11]]) && length(argv[[11]]) == 1 )) {
      f <- ScalarChanAndVeseDenseLevelSet__SWIG_3; 
    }
  } else if (argc == 12) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( is.numeric(argv[[9]]) && length(argv[[9]]) == 1 ) && ( is.numeric(argv[[10]]) && length(argv[[10]]) == 1 ) && ( is.numeric(argv[[11]]) && length(argv[[11]]) == 1 ) && ( is.numeric(argv[[12]]) && length(argv[[12]]) == 1 )) {
      f <- ScalarChanAndVeseDenseLevelSet__SWIG_2; 
    }
  } else if (argc == 13) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( is.numeric(argv[[9]]) && length(argv[[9]]) == 1 ) && ( is.numeric(argv[[10]]) && length(argv[[10]]) == 1 ) && ( is.numeric(argv[[11]]) && length(argv[[11]]) == 1 ) && ( is.numeric(argv[[12]]) && length(argv[[12]]) == 1 ) && ( is.character(argv[[13]]) && length(argv[[13]]) == 1 )) {
      f <- ScalarChanAndVeseDenseLevelSet__SWIG_1; 
    }
  } else if (argc == 14) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 ) && ( is.numeric(argv[[9]]) && length(argv[[9]]) == 1 ) && ( is.numeric(argv[[10]]) && length(argv[[10]]) == 1 ) && ( is.numeric(argv[[11]]) && length(argv[[11]]) == 1 ) && ( is.numeric(argv[[12]]) && length(argv[[12]]) == 1 ) && ( is.character(argv[[13]]) && length(argv[[13]]) == 1 ) && ( is.logical(argv[[14]]) && length(argv[[14]]) == 1 )) {
      f <- ScalarChanAndVeseDenseLevelSet__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ScalarChanAndVeseDenseLevelSet with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ScalarConnectedComponentImageFilter

`delete_ScalarConnectedComponentImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ScalarConnectedComponentImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ScalarConnectedComponentImageFilter`, 'returnType') = 'void'
attr(`delete_ScalarConnectedComponentImageFilter`, "inputTypes") = c('_p_itk__simple__ScalarConnectedComponentImageFilter')
class(`delete_ScalarConnectedComponentImageFilter`) = c("SWIGFunction", class('delete_ScalarConnectedComponentImageFilter'))

# Start of new_ScalarConnectedComponentImageFilter

`ScalarConnectedComponentImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ScalarConnectedComponentImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScalarConnectedComponentImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScalarConnectedComponentImageFilter);
  ans
  
}

attr(`ScalarConnectedComponentImageFilter`, 'returnType') = '_p_itk__simple__ScalarConnectedComponentImageFilter'
class(`ScalarConnectedComponentImageFilter`) = c("SWIGFunction", class('ScalarConnectedComponentImageFilter'))

# Start of ScalarConnectedComponentImageFilter_SetDistanceThreshold

`ScalarConnectedComponentImageFilter_SetDistanceThreshold` = function(self, DistanceThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ScalarConnectedComponentImageFilter_SetDistanceThreshold', self, DistanceThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarConnectedComponentImageFilter_SetDistanceThreshold`, 'returnType') = 'void'
attr(`ScalarConnectedComponentImageFilter_SetDistanceThreshold`, "inputTypes") = c('_p_itk__simple__ScalarConnectedComponentImageFilter', 'numeric')
class(`ScalarConnectedComponentImageFilter_SetDistanceThreshold`) = c("SWIGFunction", class('ScalarConnectedComponentImageFilter_SetDistanceThreshold'))

# Start of ScalarConnectedComponentImageFilter_GetDistanceThreshold

`ScalarConnectedComponentImageFilter_GetDistanceThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarConnectedComponentImageFilter_GetDistanceThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarConnectedComponentImageFilter_GetDistanceThreshold`, 'returnType') = 'numeric'
attr(`ScalarConnectedComponentImageFilter_GetDistanceThreshold`, "inputTypes") = c('_p_itk__simple__ScalarConnectedComponentImageFilter')
class(`ScalarConnectedComponentImageFilter_GetDistanceThreshold`) = c("SWIGFunction", class('ScalarConnectedComponentImageFilter_GetDistanceThreshold'))

# Start of ScalarConnectedComponentImageFilter_SetFullyConnected

`ScalarConnectedComponentImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_ScalarConnectedComponentImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarConnectedComponentImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`ScalarConnectedComponentImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__ScalarConnectedComponentImageFilter', 'logical')
class(`ScalarConnectedComponentImageFilter_SetFullyConnected`) = c("SWIGFunction", class('ScalarConnectedComponentImageFilter_SetFullyConnected'))

# Start of ScalarConnectedComponentImageFilter_FullyConnectedOn

`ScalarConnectedComponentImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ScalarConnectedComponentImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarConnectedComponentImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`ScalarConnectedComponentImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__ScalarConnectedComponentImageFilter')
class(`ScalarConnectedComponentImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('ScalarConnectedComponentImageFilter_FullyConnectedOn'))

# Start of ScalarConnectedComponentImageFilter_FullyConnectedOff

`ScalarConnectedComponentImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ScalarConnectedComponentImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarConnectedComponentImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`ScalarConnectedComponentImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__ScalarConnectedComponentImageFilter')
class(`ScalarConnectedComponentImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('ScalarConnectedComponentImageFilter_FullyConnectedOff'))

# Start of ScalarConnectedComponentImageFilter_GetFullyConnected

`ScalarConnectedComponentImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarConnectedComponentImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarConnectedComponentImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`ScalarConnectedComponentImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__ScalarConnectedComponentImageFilter')
class(`ScalarConnectedComponentImageFilter_GetFullyConnected`) = c("SWIGFunction", class('ScalarConnectedComponentImageFilter_GetFullyConnected'))

# Start of ScalarConnectedComponentImageFilter_GetName

`ScalarConnectedComponentImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarConnectedComponentImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarConnectedComponentImageFilter_GetName`, 'returnType') = 'character'
attr(`ScalarConnectedComponentImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ScalarConnectedComponentImageFilter')
class(`ScalarConnectedComponentImageFilter_GetName`) = c("SWIGFunction", class('ScalarConnectedComponentImageFilter_GetName'))

# Start of ScalarConnectedComponentImageFilter_ToString

`ScalarConnectedComponentImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarConnectedComponentImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarConnectedComponentImageFilter_ToString`, 'returnType') = 'character'
attr(`ScalarConnectedComponentImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ScalarConnectedComponentImageFilter')
class(`ScalarConnectedComponentImageFilter_ToString`) = c("SWIGFunction", class('ScalarConnectedComponentImageFilter_ToString'))

# Start of ScalarConnectedComponentImageFilter_Execute

`ScalarConnectedComponentImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_ScalarConnectedComponentImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarConnectedComponentImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarConnectedComponentImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__ScalarConnectedComponentImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ScalarConnectedComponentImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('ScalarConnectedComponentImageFilter_Execute__SWIG_0'))

# Start of ScalarConnectedComponentImageFilter_Execute

`ScalarConnectedComponentImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_ScalarConnectedComponentImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarConnectedComponentImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarConnectedComponentImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__ScalarConnectedComponentImageFilter', '_p_itk__simple__Image')
class(`ScalarConnectedComponentImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('ScalarConnectedComponentImageFilter_Execute__SWIG_1'))

`ScalarConnectedComponentImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ScalarConnectedComponentImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- ScalarConnectedComponentImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ScalarConnectedComponentImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- ScalarConnectedComponentImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ScalarConnectedComponentImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::ScalarConnectedComponentImageFilter
setMethod('$', '_p_itk__simple__ScalarConnectedComponentImageFilter', function(x, name)

{
  accessorFuns = list('SetDistanceThreshold' = ScalarConnectedComponentImageFilter_SetDistanceThreshold, 'GetDistanceThreshold' = ScalarConnectedComponentImageFilter_GetDistanceThreshold, 'SetFullyConnected' = ScalarConnectedComponentImageFilter_SetFullyConnected, 'FullyConnectedOn' = ScalarConnectedComponentImageFilter_FullyConnectedOn, 'FullyConnectedOff' = ScalarConnectedComponentImageFilter_FullyConnectedOff, 'GetFullyConnected' = ScalarConnectedComponentImageFilter_GetFullyConnected, 'GetName' = ScalarConnectedComponentImageFilter_GetName, 'ToString' = ScalarConnectedComponentImageFilter_ToString, 'Execute' = ScalarConnectedComponentImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ScalarConnectedComponentImageFilter
# Start of ScalarConnectedComponent

`ScalarConnectedComponent__SWIG_0` = function(image, maskImage, distanceThreshold, fullyConnected, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_ScalarConnectedComponent__SWIG_0', image, maskImage, distanceThreshold, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarConnectedComponent__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarConnectedComponent__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'logical')
class(`ScalarConnectedComponent__SWIG_0`) = c("SWIGFunction", class('ScalarConnectedComponent__SWIG_0'))

# Start of ScalarConnectedComponent

`ScalarConnectedComponent__SWIG_1` = function(image, maskImage, distanceThreshold, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  
  ;ans = .Call('R_swig_ScalarConnectedComponent__SWIG_1', image, maskImage, distanceThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarConnectedComponent__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarConnectedComponent__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`ScalarConnectedComponent__SWIG_1`) = c("SWIGFunction", class('ScalarConnectedComponent__SWIG_1'))

# Start of ScalarConnectedComponent

`ScalarConnectedComponent__SWIG_2` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_ScalarConnectedComponent__SWIG_2', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarConnectedComponent__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarConnectedComponent__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ScalarConnectedComponent__SWIG_2`) = c("SWIGFunction", class('ScalarConnectedComponent__SWIG_2'))

# Start of ScalarConnectedComponent

`ScalarConnectedComponent__SWIG_3` = function(image, distanceThreshold, fullyConnected, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_ScalarConnectedComponent__SWIG_3', image, distanceThreshold, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarConnectedComponent__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarConnectedComponent__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`ScalarConnectedComponent__SWIG_3`) = c("SWIGFunction", class('ScalarConnectedComponent__SWIG_3'))

# Start of ScalarConnectedComponent

`ScalarConnectedComponent__SWIG_4` = function(image, distanceThreshold, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  
  ;ans = .Call('R_swig_ScalarConnectedComponent__SWIG_4', image, distanceThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarConnectedComponent__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarConnectedComponent__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`ScalarConnectedComponent__SWIG_4`) = c("SWIGFunction", class('ScalarConnectedComponent__SWIG_4'))

# Start of ScalarConnectedComponent

`ScalarConnectedComponent__SWIG_5` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_ScalarConnectedComponent__SWIG_5', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarConnectedComponent__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarConnectedComponent__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`ScalarConnectedComponent__SWIG_5`) = c("SWIGFunction", class('ScalarConnectedComponent__SWIG_5'))

`ScalarConnectedComponent` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ScalarConnectedComponent__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- ScalarConnectedComponent__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ScalarConnectedComponent__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ScalarConnectedComponent__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ScalarConnectedComponent__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ScalarConnectedComponent__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ScalarConnectedComponent with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ScalarImageKmeansImageFilter

`delete_ScalarImageKmeansImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ScalarImageKmeansImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ScalarImageKmeansImageFilter`, 'returnType') = 'void'
attr(`delete_ScalarImageKmeansImageFilter`, "inputTypes") = c('_p_itk__simple__ScalarImageKmeansImageFilter')
class(`delete_ScalarImageKmeansImageFilter`) = c("SWIGFunction", class('delete_ScalarImageKmeansImageFilter'))

# Start of new_ScalarImageKmeansImageFilter

`ScalarImageKmeansImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ScalarImageKmeansImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScalarImageKmeansImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScalarImageKmeansImageFilter);
  ans
  
}

attr(`ScalarImageKmeansImageFilter`, 'returnType') = '_p_itk__simple__ScalarImageKmeansImageFilter'
class(`ScalarImageKmeansImageFilter`) = c("SWIGFunction", class('ScalarImageKmeansImageFilter'))

# Start of ScalarImageKmeansImageFilter_SetClassWithInitialMean

`ScalarImageKmeansImageFilter_SetClassWithInitialMean` = function(self, ClassWithInitialMean)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ClassWithInitialMean = as.numeric(ClassWithInitialMean);
  ;ans = .Call('R_swig_ScalarImageKmeansImageFilter_SetClassWithInitialMean', self, ClassWithInitialMean, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarImageKmeansImageFilter_SetClassWithInitialMean`, 'returnType') = 'void'
attr(`ScalarImageKmeansImageFilter_SetClassWithInitialMean`, "inputTypes") = c('_p_itk__simple__ScalarImageKmeansImageFilter', 'numeric')
class(`ScalarImageKmeansImageFilter_SetClassWithInitialMean`) = c("SWIGFunction", class('ScalarImageKmeansImageFilter_SetClassWithInitialMean'))

# Start of ScalarImageKmeansImageFilter_GetClassWithInitialMean

`ScalarImageKmeansImageFilter_GetClassWithInitialMean` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarImageKmeansImageFilter_GetClassWithInitialMean', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarImageKmeansImageFilter_GetClassWithInitialMean`, 'returnType') = 'numeric'
attr(`ScalarImageKmeansImageFilter_GetClassWithInitialMean`, "inputTypes") = c('_p_itk__simple__ScalarImageKmeansImageFilter')
class(`ScalarImageKmeansImageFilter_GetClassWithInitialMean`) = c("SWIGFunction", class('ScalarImageKmeansImageFilter_GetClassWithInitialMean'))

# Start of ScalarImageKmeansImageFilter_SetUseNonContiguousLabels

`ScalarImageKmeansImageFilter_SetUseNonContiguousLabels` = function(self, UseNonContiguousLabels)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseNonContiguousLabels = as.logical(UseNonContiguousLabels);
  ;ans = .Call('R_swig_ScalarImageKmeansImageFilter_SetUseNonContiguousLabels', self, UseNonContiguousLabels, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarImageKmeansImageFilter_SetUseNonContiguousLabels`, 'returnType') = 'void'
attr(`ScalarImageKmeansImageFilter_SetUseNonContiguousLabels`, "inputTypes") = c('_p_itk__simple__ScalarImageKmeansImageFilter', 'logical')
class(`ScalarImageKmeansImageFilter_SetUseNonContiguousLabels`) = c("SWIGFunction", class('ScalarImageKmeansImageFilter_SetUseNonContiguousLabels'))

# Start of ScalarImageKmeansImageFilter_UseNonContiguousLabelsOn

`ScalarImageKmeansImageFilter_UseNonContiguousLabelsOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ScalarImageKmeansImageFilter_UseNonContiguousLabelsOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarImageKmeansImageFilter_UseNonContiguousLabelsOn`, 'returnType') = 'void'
attr(`ScalarImageKmeansImageFilter_UseNonContiguousLabelsOn`, "inputTypes") = c('_p_itk__simple__ScalarImageKmeansImageFilter')
class(`ScalarImageKmeansImageFilter_UseNonContiguousLabelsOn`) = c("SWIGFunction", class('ScalarImageKmeansImageFilter_UseNonContiguousLabelsOn'))

# Start of ScalarImageKmeansImageFilter_UseNonContiguousLabelsOff

`ScalarImageKmeansImageFilter_UseNonContiguousLabelsOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ScalarImageKmeansImageFilter_UseNonContiguousLabelsOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarImageKmeansImageFilter_UseNonContiguousLabelsOff`, 'returnType') = 'void'
attr(`ScalarImageKmeansImageFilter_UseNonContiguousLabelsOff`, "inputTypes") = c('_p_itk__simple__ScalarImageKmeansImageFilter')
class(`ScalarImageKmeansImageFilter_UseNonContiguousLabelsOff`) = c("SWIGFunction", class('ScalarImageKmeansImageFilter_UseNonContiguousLabelsOff'))

# Start of ScalarImageKmeansImageFilter_GetUseNonContiguousLabels

`ScalarImageKmeansImageFilter_GetUseNonContiguousLabels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarImageKmeansImageFilter_GetUseNonContiguousLabels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarImageKmeansImageFilter_GetUseNonContiguousLabels`, 'returnType') = 'logical'
attr(`ScalarImageKmeansImageFilter_GetUseNonContiguousLabels`, "inputTypes") = c('_p_itk__simple__ScalarImageKmeansImageFilter')
class(`ScalarImageKmeansImageFilter_GetUseNonContiguousLabels`) = c("SWIGFunction", class('ScalarImageKmeansImageFilter_GetUseNonContiguousLabels'))

# Start of ScalarImageKmeansImageFilter_GetFinalMeans

`ScalarImageKmeansImageFilter_GetFinalMeans` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarImageKmeansImageFilter_GetFinalMeans', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarImageKmeansImageFilter_GetFinalMeans`, 'returnType') = 'numeric'
attr(`ScalarImageKmeansImageFilter_GetFinalMeans`, "inputTypes") = c('_p_itk__simple__ScalarImageKmeansImageFilter')
class(`ScalarImageKmeansImageFilter_GetFinalMeans`) = c("SWIGFunction", class('ScalarImageKmeansImageFilter_GetFinalMeans'))

# Start of ScalarImageKmeansImageFilter_GetName

`ScalarImageKmeansImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarImageKmeansImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarImageKmeansImageFilter_GetName`, 'returnType') = 'character'
attr(`ScalarImageKmeansImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ScalarImageKmeansImageFilter')
class(`ScalarImageKmeansImageFilter_GetName`) = c("SWIGFunction", class('ScalarImageKmeansImageFilter_GetName'))

# Start of ScalarImageKmeansImageFilter_ToString

`ScalarImageKmeansImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarImageKmeansImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarImageKmeansImageFilter_ToString`, 'returnType') = 'character'
attr(`ScalarImageKmeansImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ScalarImageKmeansImageFilter')
class(`ScalarImageKmeansImageFilter_ToString`) = c("SWIGFunction", class('ScalarImageKmeansImageFilter_ToString'))

# Start of ScalarImageKmeansImageFilter_Execute

`ScalarImageKmeansImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ScalarImageKmeansImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarImageKmeansImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarImageKmeansImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ScalarImageKmeansImageFilter', '_p_itk__simple__Image')
class(`ScalarImageKmeansImageFilter_Execute`) = c("SWIGFunction", class('ScalarImageKmeansImageFilter_Execute'))

# Start of accessor method for itk::simple::ScalarImageKmeansImageFilter
setMethod('$', '_p_itk__simple__ScalarImageKmeansImageFilter', function(x, name)

{
  accessorFuns = list('SetClassWithInitialMean' = ScalarImageKmeansImageFilter_SetClassWithInitialMean, 'GetClassWithInitialMean' = ScalarImageKmeansImageFilter_GetClassWithInitialMean, 'SetUseNonContiguousLabels' = ScalarImageKmeansImageFilter_SetUseNonContiguousLabels, 'UseNonContiguousLabelsOn' = ScalarImageKmeansImageFilter_UseNonContiguousLabelsOn, 'UseNonContiguousLabelsOff' = ScalarImageKmeansImageFilter_UseNonContiguousLabelsOff, 'GetUseNonContiguousLabels' = ScalarImageKmeansImageFilter_GetUseNonContiguousLabels, 'GetFinalMeans' = ScalarImageKmeansImageFilter_GetFinalMeans, 'GetName' = ScalarImageKmeansImageFilter_GetName, 'ToString' = ScalarImageKmeansImageFilter_ToString, 'Execute' = ScalarImageKmeansImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ScalarImageKmeansImageFilter
# Start of ScalarImageKmeans

`ScalarImageKmeans__SWIG_0` = function(image1, classWithInitialMean, useNonContiguousLabels, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  classWithInitialMean = as.numeric(classWithInitialMean);
  useNonContiguousLabels = as.logical(useNonContiguousLabels);
  ;ans = .Call('R_swig_ScalarImageKmeans__SWIG_0', image1, classWithInitialMean, useNonContiguousLabels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarImageKmeans__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarImageKmeans__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`ScalarImageKmeans__SWIG_0`) = c("SWIGFunction", class('ScalarImageKmeans__SWIG_0'))

# Start of ScalarImageKmeans

`ScalarImageKmeans__SWIG_1` = function(image1, classWithInitialMean, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  classWithInitialMean = as.numeric(classWithInitialMean);
  ;ans = .Call('R_swig_ScalarImageKmeans__SWIG_1', image1, classWithInitialMean, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarImageKmeans__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarImageKmeans__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`ScalarImageKmeans__SWIG_1`) = c("SWIGFunction", class('ScalarImageKmeans__SWIG_1'))

# Start of ScalarImageKmeans

`ScalarImageKmeans__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ScalarImageKmeans__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarImageKmeans__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarImageKmeans__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`ScalarImageKmeans__SWIG_2`) = c("SWIGFunction", class('ScalarImageKmeans__SWIG_2'))

`ScalarImageKmeans` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ScalarImageKmeans__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) )) {
      f <- ScalarImageKmeans__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ScalarImageKmeans__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ScalarImageKmeans with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ScalarToRGBColormapImageFilter

`delete_ScalarToRGBColormapImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ScalarToRGBColormapImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ScalarToRGBColormapImageFilter`, 'returnType') = 'void'
attr(`delete_ScalarToRGBColormapImageFilter`, "inputTypes") = c('_p_itk__simple__ScalarToRGBColormapImageFilter')
class(`delete_ScalarToRGBColormapImageFilter`) = c("SWIGFunction", class('delete_ScalarToRGBColormapImageFilter'))

# Start of new_ScalarToRGBColormapImageFilter

`ScalarToRGBColormapImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ScalarToRGBColormapImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ScalarToRGBColormapImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ScalarToRGBColormapImageFilter);
  ans
  
}

attr(`ScalarToRGBColormapImageFilter`, 'returnType') = '_p_itk__simple__ScalarToRGBColormapImageFilter'
class(`ScalarToRGBColormapImageFilter`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter'))

# Start of ScalarToRGBColormapImageFilter_Red_get

`ScalarToRGBColormapImageFilter_Red_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_Red_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType");
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_Red_get`, 'returnType') = 'character'
class(`ScalarToRGBColormapImageFilter_Red_get`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_Red_get'))

# Start of ScalarToRGBColormapImageFilter_Green_get

`ScalarToRGBColormapImageFilter_Green_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_Green_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType");
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_Green_get`, 'returnType') = 'character'
class(`ScalarToRGBColormapImageFilter_Green_get`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_Green_get'))

# Start of ScalarToRGBColormapImageFilter_Blue_get

`ScalarToRGBColormapImageFilter_Blue_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_Blue_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType");
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_Blue_get`, 'returnType') = 'character'
class(`ScalarToRGBColormapImageFilter_Blue_get`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_Blue_get'))

# Start of ScalarToRGBColormapImageFilter_Grey_get

`ScalarToRGBColormapImageFilter_Grey_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_Grey_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType");
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_Grey_get`, 'returnType') = 'character'
class(`ScalarToRGBColormapImageFilter_Grey_get`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_Grey_get'))

# Start of ScalarToRGBColormapImageFilter_Hot_get

`ScalarToRGBColormapImageFilter_Hot_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_Hot_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType");
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_Hot_get`, 'returnType') = 'character'
class(`ScalarToRGBColormapImageFilter_Hot_get`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_Hot_get'))

# Start of ScalarToRGBColormapImageFilter_Cool_get

`ScalarToRGBColormapImageFilter_Cool_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_Cool_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType");
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_Cool_get`, 'returnType') = 'character'
class(`ScalarToRGBColormapImageFilter_Cool_get`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_Cool_get'))

# Start of ScalarToRGBColormapImageFilter_Spring_get

`ScalarToRGBColormapImageFilter_Spring_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_Spring_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType");
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_Spring_get`, 'returnType') = 'character'
class(`ScalarToRGBColormapImageFilter_Spring_get`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_Spring_get'))

# Start of ScalarToRGBColormapImageFilter_Summer_get

`ScalarToRGBColormapImageFilter_Summer_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_Summer_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType");
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_Summer_get`, 'returnType') = 'character'
class(`ScalarToRGBColormapImageFilter_Summer_get`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_Summer_get'))

# Start of ScalarToRGBColormapImageFilter_Autumn_get

`ScalarToRGBColormapImageFilter_Autumn_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_Autumn_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType");
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_Autumn_get`, 'returnType') = 'character'
class(`ScalarToRGBColormapImageFilter_Autumn_get`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_Autumn_get'))

# Start of ScalarToRGBColormapImageFilter_Winter_get

`ScalarToRGBColormapImageFilter_Winter_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_Winter_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType");
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_Winter_get`, 'returnType') = 'character'
class(`ScalarToRGBColormapImageFilter_Winter_get`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_Winter_get'))

# Start of ScalarToRGBColormapImageFilter_Copper_get

`ScalarToRGBColormapImageFilter_Copper_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_Copper_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType");
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_Copper_get`, 'returnType') = 'character'
class(`ScalarToRGBColormapImageFilter_Copper_get`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_Copper_get'))

# Start of ScalarToRGBColormapImageFilter_Jet_get

`ScalarToRGBColormapImageFilter_Jet_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_Jet_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType");
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_Jet_get`, 'returnType') = 'character'
class(`ScalarToRGBColormapImageFilter_Jet_get`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_Jet_get'))

# Start of ScalarToRGBColormapImageFilter_HSV_get

`ScalarToRGBColormapImageFilter_HSV_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_HSV_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType");
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_HSV_get`, 'returnType') = 'character'
class(`ScalarToRGBColormapImageFilter_HSV_get`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_HSV_get'))

# Start of ScalarToRGBColormapImageFilter_OverUnder_get

`ScalarToRGBColormapImageFilter_OverUnder_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_OverUnder_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType");
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_OverUnder_get`, 'returnType') = 'character'
class(`ScalarToRGBColormapImageFilter_OverUnder_get`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_OverUnder_get'))

# Start of ScalarToRGBColormapImageFilter_SetColormap

`ScalarToRGBColormapImageFilter_SetColormap` = function(self, Colormap)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Colormap = enumToInteger(Colormap, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType"); 
  
  if(length(Colormap) > 1) {
    warning("using only the first element of Colormap");
  };
  
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_SetColormap', self, Colormap, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_SetColormap`, 'returnType') = 'void'
attr(`ScalarToRGBColormapImageFilter_SetColormap`, "inputTypes") = c('_p_itk__simple__ScalarToRGBColormapImageFilter', 'character')
class(`ScalarToRGBColormapImageFilter_SetColormap`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_SetColormap'))

# Start of ScalarToRGBColormapImageFilter_GetColormap

`ScalarToRGBColormapImageFilter_GetColormap` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_GetColormap', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType");
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_GetColormap`, 'returnType') = 'character'
attr(`ScalarToRGBColormapImageFilter_GetColormap`, "inputTypes") = c('_p_itk__simple__ScalarToRGBColormapImageFilter')
class(`ScalarToRGBColormapImageFilter_GetColormap`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_GetColormap'))

# Start of ScalarToRGBColormapImageFilter_SetUseInputImageExtremaForScaling

`ScalarToRGBColormapImageFilter_SetUseInputImageExtremaForScaling` = function(self, UseInputImageExtremaForScaling)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseInputImageExtremaForScaling = as.logical(UseInputImageExtremaForScaling);
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_SetUseInputImageExtremaForScaling', self, UseInputImageExtremaForScaling, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_SetUseInputImageExtremaForScaling`, 'returnType') = 'void'
attr(`ScalarToRGBColormapImageFilter_SetUseInputImageExtremaForScaling`, "inputTypes") = c('_p_itk__simple__ScalarToRGBColormapImageFilter', 'logical')
class(`ScalarToRGBColormapImageFilter_SetUseInputImageExtremaForScaling`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_SetUseInputImageExtremaForScaling'))

# Start of ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOn

`ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOn`, 'returnType') = 'void'
attr(`ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOn`, "inputTypes") = c('_p_itk__simple__ScalarToRGBColormapImageFilter')
class(`ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOn`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOn'))

# Start of ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOff

`ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOff`, 'returnType') = 'void'
attr(`ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOff`, "inputTypes") = c('_p_itk__simple__ScalarToRGBColormapImageFilter')
class(`ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOff`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOff'))

# Start of ScalarToRGBColormapImageFilter_GetUseInputImageExtremaForScaling

`ScalarToRGBColormapImageFilter_GetUseInputImageExtremaForScaling` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarToRGBColormapImageFilter_GetUseInputImageExtremaForScaling', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarToRGBColormapImageFilter_GetUseInputImageExtremaForScaling`, 'returnType') = 'logical'
attr(`ScalarToRGBColormapImageFilter_GetUseInputImageExtremaForScaling`, "inputTypes") = c('_p_itk__simple__ScalarToRGBColormapImageFilter')
class(`ScalarToRGBColormapImageFilter_GetUseInputImageExtremaForScaling`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_GetUseInputImageExtremaForScaling'))

# Start of ScalarToRGBColormapImageFilter_GetName

`ScalarToRGBColormapImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarToRGBColormapImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarToRGBColormapImageFilter_GetName`, 'returnType') = 'character'
attr(`ScalarToRGBColormapImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ScalarToRGBColormapImageFilter')
class(`ScalarToRGBColormapImageFilter_GetName`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_GetName'))

# Start of ScalarToRGBColormapImageFilter_ToString

`ScalarToRGBColormapImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ScalarToRGBColormapImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ScalarToRGBColormapImageFilter_ToString`, 'returnType') = 'character'
attr(`ScalarToRGBColormapImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ScalarToRGBColormapImageFilter')
class(`ScalarToRGBColormapImageFilter_ToString`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_ToString'))

# Start of ScalarToRGBColormapImageFilter_Execute

`ScalarToRGBColormapImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ScalarToRGBColormapImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarToRGBColormapImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarToRGBColormapImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ScalarToRGBColormapImageFilter', '_p_itk__simple__Image')
class(`ScalarToRGBColormapImageFilter_Execute`) = c("SWIGFunction", class('ScalarToRGBColormapImageFilter_Execute'))

# Start of accessor method for itk::simple::ScalarToRGBColormapImageFilter
setMethod('$', '_p_itk__simple__ScalarToRGBColormapImageFilter', function(x, name)

{
  accessorFuns = list('SetColormap' = ScalarToRGBColormapImageFilter_SetColormap, 'GetColormap' = ScalarToRGBColormapImageFilter_GetColormap, 'SetUseInputImageExtremaForScaling' = ScalarToRGBColormapImageFilter_SetUseInputImageExtremaForScaling, 'UseInputImageExtremaForScalingOn' = ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOn, 'UseInputImageExtremaForScalingOff' = ScalarToRGBColormapImageFilter_UseInputImageExtremaForScalingOff, 'GetUseInputImageExtremaForScaling' = ScalarToRGBColormapImageFilter_GetUseInputImageExtremaForScaling, 'GetName' = ScalarToRGBColormapImageFilter_GetName, 'ToString' = ScalarToRGBColormapImageFilter_ToString, 'Execute' = ScalarToRGBColormapImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ScalarToRGBColormapImageFilter
# Start of ScalarToRGBColormap

`ScalarToRGBColormap__SWIG_0` = function(image1, colormap, useInputImageExtremaForScaling, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  colormap = enumToInteger(colormap, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType"); 
  
  if(length(colormap) > 1) {
    warning("using only the first element of colormap");
  };
  
  useInputImageExtremaForScaling = as.logical(useInputImageExtremaForScaling);
  ;ans = .Call('R_swig_ScalarToRGBColormap__SWIG_0', image1, colormap, useInputImageExtremaForScaling, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarToRGBColormap__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarToRGBColormap__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'character', 'logical')
class(`ScalarToRGBColormap__SWIG_0`) = c("SWIGFunction", class('ScalarToRGBColormap__SWIG_0'))

# Start of ScalarToRGBColormap

`ScalarToRGBColormap__SWIG_1` = function(image1, colormap, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  colormap = enumToInteger(colormap, "_itk__simple__ScalarToRGBColormapImageFilter__ColormapType"); 
  
  if(length(colormap) > 1) {
    warning("using only the first element of colormap");
  };
  
  ;ans = .Call('R_swig_ScalarToRGBColormap__SWIG_1', image1, colormap, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarToRGBColormap__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarToRGBColormap__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'character')
class(`ScalarToRGBColormap__SWIG_1`) = c("SWIGFunction", class('ScalarToRGBColormap__SWIG_1'))

# Start of ScalarToRGBColormap

`ScalarToRGBColormap__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ScalarToRGBColormap__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ScalarToRGBColormap__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ScalarToRGBColormap__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`ScalarToRGBColormap__SWIG_2`) = c("SWIGFunction", class('ScalarToRGBColormap__SWIG_2'))

`ScalarToRGBColormap` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ScalarToRGBColormap__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ScalarToRGBColormap__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ScalarToRGBColormap__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ScalarToRGBColormap with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ShanbhagThresholdImageFilter

`delete_ShanbhagThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ShanbhagThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ShanbhagThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_ShanbhagThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter')
class(`delete_ShanbhagThresholdImageFilter`) = c("SWIGFunction", class('delete_ShanbhagThresholdImageFilter'))

# Start of new_ShanbhagThresholdImageFilter

`ShanbhagThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ShanbhagThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ShanbhagThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ShanbhagThresholdImageFilter);
  ans
  
}

attr(`ShanbhagThresholdImageFilter`, 'returnType') = '_p_itk__simple__ShanbhagThresholdImageFilter'
class(`ShanbhagThresholdImageFilter`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter'))

# Start of ShanbhagThresholdImageFilter_SetInsideValue

`ShanbhagThresholdImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideValue = as.integer(InsideValue);
  
  if(length(InsideValue) > 1) {
    warning("using only the first element of InsideValue");
  };
  
  ;ans = .Call('R_swig_ShanbhagThresholdImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShanbhagThresholdImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`ShanbhagThresholdImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter', 'integer')
class(`ShanbhagThresholdImageFilter_SetInsideValue`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_SetInsideValue'))

# Start of ShanbhagThresholdImageFilter_GetInsideValue

`ShanbhagThresholdImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShanbhagThresholdImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShanbhagThresholdImageFilter_GetInsideValue`, 'returnType') = 'integer'
attr(`ShanbhagThresholdImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter')
class(`ShanbhagThresholdImageFilter_GetInsideValue`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_GetInsideValue'))

# Start of ShanbhagThresholdImageFilter_SetOutsideValue

`ShanbhagThresholdImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutsideValue = as.integer(OutsideValue);
  
  if(length(OutsideValue) > 1) {
    warning("using only the first element of OutsideValue");
  };
  
  ;ans = .Call('R_swig_ShanbhagThresholdImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShanbhagThresholdImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`ShanbhagThresholdImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter', 'integer')
class(`ShanbhagThresholdImageFilter_SetOutsideValue`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_SetOutsideValue'))

# Start of ShanbhagThresholdImageFilter_GetOutsideValue

`ShanbhagThresholdImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShanbhagThresholdImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShanbhagThresholdImageFilter_GetOutsideValue`, 'returnType') = 'integer'
attr(`ShanbhagThresholdImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter')
class(`ShanbhagThresholdImageFilter_GetOutsideValue`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_GetOutsideValue'))

# Start of ShanbhagThresholdImageFilter_SetNumberOfHistogramBins

`ShanbhagThresholdImageFilter_SetNumberOfHistogramBins` = function(self, NumberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfHistogramBins = as.integer(NumberOfHistogramBins);
  
  if(length(NumberOfHistogramBins) > 1) {
    warning("using only the first element of NumberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_ShanbhagThresholdImageFilter_SetNumberOfHistogramBins', self, NumberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShanbhagThresholdImageFilter_SetNumberOfHistogramBins`, 'returnType') = 'void'
attr(`ShanbhagThresholdImageFilter_SetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter', 'integer')
class(`ShanbhagThresholdImageFilter_SetNumberOfHistogramBins`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_SetNumberOfHistogramBins'))

# Start of ShanbhagThresholdImageFilter_GetNumberOfHistogramBins

`ShanbhagThresholdImageFilter_GetNumberOfHistogramBins` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShanbhagThresholdImageFilter_GetNumberOfHistogramBins', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShanbhagThresholdImageFilter_GetNumberOfHistogramBins`, 'returnType') = 'integer'
attr(`ShanbhagThresholdImageFilter_GetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter')
class(`ShanbhagThresholdImageFilter_GetNumberOfHistogramBins`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_GetNumberOfHistogramBins'))

# Start of ShanbhagThresholdImageFilter_SetMaskOutput

`ShanbhagThresholdImageFilter_SetMaskOutput` = function(self, MaskOutput)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskOutput = as.logical(MaskOutput);
  ;ans = .Call('R_swig_ShanbhagThresholdImageFilter_SetMaskOutput', self, MaskOutput, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShanbhagThresholdImageFilter_SetMaskOutput`, 'returnType') = 'void'
attr(`ShanbhagThresholdImageFilter_SetMaskOutput`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter', 'logical')
class(`ShanbhagThresholdImageFilter_SetMaskOutput`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_SetMaskOutput'))

# Start of ShanbhagThresholdImageFilter_MaskOutputOn

`ShanbhagThresholdImageFilter_MaskOutputOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ShanbhagThresholdImageFilter_MaskOutputOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShanbhagThresholdImageFilter_MaskOutputOn`, 'returnType') = 'void'
attr(`ShanbhagThresholdImageFilter_MaskOutputOn`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter')
class(`ShanbhagThresholdImageFilter_MaskOutputOn`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_MaskOutputOn'))

# Start of ShanbhagThresholdImageFilter_MaskOutputOff

`ShanbhagThresholdImageFilter_MaskOutputOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ShanbhagThresholdImageFilter_MaskOutputOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShanbhagThresholdImageFilter_MaskOutputOff`, 'returnType') = 'void'
attr(`ShanbhagThresholdImageFilter_MaskOutputOff`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter')
class(`ShanbhagThresholdImageFilter_MaskOutputOff`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_MaskOutputOff'))

# Start of ShanbhagThresholdImageFilter_GetMaskOutput

`ShanbhagThresholdImageFilter_GetMaskOutput` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShanbhagThresholdImageFilter_GetMaskOutput', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShanbhagThresholdImageFilter_GetMaskOutput`, 'returnType') = 'logical'
attr(`ShanbhagThresholdImageFilter_GetMaskOutput`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter')
class(`ShanbhagThresholdImageFilter_GetMaskOutput`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_GetMaskOutput'))

# Start of ShanbhagThresholdImageFilter_SetMaskValue

`ShanbhagThresholdImageFilter_SetMaskValue` = function(self, MaskValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskValue = as.integer(MaskValue);
  
  if(length(MaskValue) > 1) {
    warning("using only the first element of MaskValue");
  };
  
  ;ans = .Call('R_swig_ShanbhagThresholdImageFilter_SetMaskValue', self, MaskValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShanbhagThresholdImageFilter_SetMaskValue`, 'returnType') = 'void'
attr(`ShanbhagThresholdImageFilter_SetMaskValue`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter', 'integer')
class(`ShanbhagThresholdImageFilter_SetMaskValue`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_SetMaskValue'))

# Start of ShanbhagThresholdImageFilter_GetMaskValue

`ShanbhagThresholdImageFilter_GetMaskValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShanbhagThresholdImageFilter_GetMaskValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShanbhagThresholdImageFilter_GetMaskValue`, 'returnType') = 'integer'
attr(`ShanbhagThresholdImageFilter_GetMaskValue`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter')
class(`ShanbhagThresholdImageFilter_GetMaskValue`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_GetMaskValue'))

# Start of ShanbhagThresholdImageFilter_GetThreshold

`ShanbhagThresholdImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShanbhagThresholdImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShanbhagThresholdImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`ShanbhagThresholdImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter')
class(`ShanbhagThresholdImageFilter_GetThreshold`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_GetThreshold'))

# Start of ShanbhagThresholdImageFilter_GetName

`ShanbhagThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShanbhagThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShanbhagThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`ShanbhagThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter')
class(`ShanbhagThresholdImageFilter_GetName`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_GetName'))

# Start of ShanbhagThresholdImageFilter_ToString

`ShanbhagThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShanbhagThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShanbhagThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`ShanbhagThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter')
class(`ShanbhagThresholdImageFilter_ToString`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_ToString'))

# Start of ShanbhagThresholdImageFilter_Execute

`ShanbhagThresholdImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_ShanbhagThresholdImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShanbhagThresholdImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ShanbhagThresholdImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ShanbhagThresholdImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_Execute__SWIG_0'))

# Start of ShanbhagThresholdImageFilter_Execute

`ShanbhagThresholdImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_ShanbhagThresholdImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShanbhagThresholdImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ShanbhagThresholdImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__ShanbhagThresholdImageFilter', '_p_itk__simple__Image')
class(`ShanbhagThresholdImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('ShanbhagThresholdImageFilter_Execute__SWIG_1'))

`ShanbhagThresholdImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__ShanbhagThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- ShanbhagThresholdImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__ShanbhagThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- ShanbhagThresholdImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ShanbhagThresholdImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::ShanbhagThresholdImageFilter
setMethod('$', '_p_itk__simple__ShanbhagThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetInsideValue' = ShanbhagThresholdImageFilter_SetInsideValue, 'GetInsideValue' = ShanbhagThresholdImageFilter_GetInsideValue, 'SetOutsideValue' = ShanbhagThresholdImageFilter_SetOutsideValue, 'GetOutsideValue' = ShanbhagThresholdImageFilter_GetOutsideValue, 'SetNumberOfHistogramBins' = ShanbhagThresholdImageFilter_SetNumberOfHistogramBins, 'GetNumberOfHistogramBins' = ShanbhagThresholdImageFilter_GetNumberOfHistogramBins, 'SetMaskOutput' = ShanbhagThresholdImageFilter_SetMaskOutput, 'MaskOutputOn' = ShanbhagThresholdImageFilter_MaskOutputOn, 'MaskOutputOff' = ShanbhagThresholdImageFilter_MaskOutputOff, 'GetMaskOutput' = ShanbhagThresholdImageFilter_GetMaskOutput, 'SetMaskValue' = ShanbhagThresholdImageFilter_SetMaskValue, 'GetMaskValue' = ShanbhagThresholdImageFilter_GetMaskValue, 'GetThreshold' = ShanbhagThresholdImageFilter_GetThreshold, 'GetName' = ShanbhagThresholdImageFilter_GetName, 'ToString' = ShanbhagThresholdImageFilter_ToString, 'Execute' = ShanbhagThresholdImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ShanbhagThresholdImageFilter
# Start of ShanbhagThreshold

`ShanbhagThreshold__SWIG_0` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_ShanbhagThreshold__SWIG_0', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShanbhagThreshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ShanbhagThreshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`ShanbhagThreshold__SWIG_0`) = c("SWIGFunction", class('ShanbhagThreshold__SWIG_0'))

# Start of ShanbhagThreshold

`ShanbhagThreshold__SWIG_1` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_ShanbhagThreshold__SWIG_1', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShanbhagThreshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ShanbhagThreshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`ShanbhagThreshold__SWIG_1`) = c("SWIGFunction", class('ShanbhagThreshold__SWIG_1'))

# Start of ShanbhagThreshold

`ShanbhagThreshold__SWIG_2` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_ShanbhagThreshold__SWIG_2', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShanbhagThreshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ShanbhagThreshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`ShanbhagThreshold__SWIG_2`) = c("SWIGFunction", class('ShanbhagThreshold__SWIG_2'))

# Start of ShanbhagThreshold

`ShanbhagThreshold__SWIG_3` = function(image, maskImage, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_ShanbhagThreshold__SWIG_3', image, maskImage, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShanbhagThreshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ShanbhagThreshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`ShanbhagThreshold__SWIG_3`) = c("SWIGFunction", class('ShanbhagThreshold__SWIG_3'))

# Start of ShanbhagThreshold

`ShanbhagThreshold__SWIG_4` = function(image, maskImage, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_ShanbhagThreshold__SWIG_4', image, maskImage, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShanbhagThreshold__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ShanbhagThreshold__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`ShanbhagThreshold__SWIG_4`) = c("SWIGFunction", class('ShanbhagThreshold__SWIG_4'))

# Start of ShanbhagThreshold

`ShanbhagThreshold__SWIG_5` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_ShanbhagThreshold__SWIG_5', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShanbhagThreshold__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`ShanbhagThreshold__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ShanbhagThreshold__SWIG_5`) = c("SWIGFunction", class('ShanbhagThreshold__SWIG_5'))

# Start of ShanbhagThreshold

`ShanbhagThreshold__SWIG_6` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_ShanbhagThreshold__SWIG_6', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShanbhagThreshold__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`ShanbhagThreshold__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`ShanbhagThreshold__SWIG_6`) = c("SWIGFunction", class('ShanbhagThreshold__SWIG_6'))

# Start of ShanbhagThreshold

`ShanbhagThreshold__SWIG_7` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_ShanbhagThreshold__SWIG_7', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShanbhagThreshold__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`ShanbhagThreshold__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`ShanbhagThreshold__SWIG_7`) = c("SWIGFunction", class('ShanbhagThreshold__SWIG_7'))

# Start of ShanbhagThreshold

`ShanbhagThreshold__SWIG_8` = function(image, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_ShanbhagThreshold__SWIG_8', image, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShanbhagThreshold__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`ShanbhagThreshold__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`ShanbhagThreshold__SWIG_8`) = c("SWIGFunction", class('ShanbhagThreshold__SWIG_8'))

# Start of ShanbhagThreshold

`ShanbhagThreshold__SWIG_9` = function(image, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_ShanbhagThreshold__SWIG_9', image, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShanbhagThreshold__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`ShanbhagThreshold__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`ShanbhagThreshold__SWIG_9`) = c("SWIGFunction", class('ShanbhagThreshold__SWIG_9'))

# Start of ShanbhagThreshold

`ShanbhagThreshold__SWIG_10` = function(image, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_ShanbhagThreshold__SWIG_10', image, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShanbhagThreshold__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`ShanbhagThreshold__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`ShanbhagThreshold__SWIG_10`) = c("SWIGFunction", class('ShanbhagThreshold__SWIG_10'))

# Start of ShanbhagThreshold

`ShanbhagThreshold__SWIG_11` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_ShanbhagThreshold__SWIG_11', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShanbhagThreshold__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`ShanbhagThreshold__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image')
class(`ShanbhagThreshold__SWIG_11`) = c("SWIGFunction", class('ShanbhagThreshold__SWIG_11'))

`ShanbhagThreshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 12
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ShanbhagThreshold__SWIG_11; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- ShanbhagThreshold__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- ShanbhagThreshold__SWIG_10; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ShanbhagThreshold__SWIG_4; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ShanbhagThreshold__SWIG_9; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- ShanbhagThreshold__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- ShanbhagThreshold__SWIG_8; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- ShanbhagThreshold__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- ShanbhagThreshold__SWIG_7; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- ShanbhagThreshold__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- ShanbhagThreshold__SWIG_6; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- ShanbhagThreshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ShanbhagThreshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ShapeDetectionLevelSetImageFilter

`delete_ShapeDetectionLevelSetImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ShapeDetectionLevelSetImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ShapeDetectionLevelSetImageFilter`, 'returnType') = 'void'
attr(`delete_ShapeDetectionLevelSetImageFilter`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter')
class(`delete_ShapeDetectionLevelSetImageFilter`) = c("SWIGFunction", class('delete_ShapeDetectionLevelSetImageFilter'))

# Start of new_ShapeDetectionLevelSetImageFilter

`ShapeDetectionLevelSetImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ShapeDetectionLevelSetImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ShapeDetectionLevelSetImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ShapeDetectionLevelSetImageFilter);
  ans
  
}

attr(`ShapeDetectionLevelSetImageFilter`, 'returnType') = '_p_itk__simple__ShapeDetectionLevelSetImageFilter'
class(`ShapeDetectionLevelSetImageFilter`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter'))

# Start of ShapeDetectionLevelSetImageFilter_SetMaximumRMSError

`ShapeDetectionLevelSetImageFilter_SetMaximumRMSError` = function(self, MaximumRMSError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ShapeDetectionLevelSetImageFilter_SetMaximumRMSError', self, MaximumRMSError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShapeDetectionLevelSetImageFilter_SetMaximumRMSError`, 'returnType') = 'void'
attr(`ShapeDetectionLevelSetImageFilter_SetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter', 'numeric')
class(`ShapeDetectionLevelSetImageFilter_SetMaximumRMSError`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_SetMaximumRMSError'))

# Start of ShapeDetectionLevelSetImageFilter_GetMaximumRMSError

`ShapeDetectionLevelSetImageFilter_GetMaximumRMSError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShapeDetectionLevelSetImageFilter_GetMaximumRMSError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShapeDetectionLevelSetImageFilter_GetMaximumRMSError`, 'returnType') = 'numeric'
attr(`ShapeDetectionLevelSetImageFilter_GetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter')
class(`ShapeDetectionLevelSetImageFilter_GetMaximumRMSError`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_GetMaximumRMSError'))

# Start of ShapeDetectionLevelSetImageFilter_SetPropagationScaling

`ShapeDetectionLevelSetImageFilter_SetPropagationScaling` = function(self, PropagationScaling)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ShapeDetectionLevelSetImageFilter_SetPropagationScaling', self, PropagationScaling, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShapeDetectionLevelSetImageFilter_SetPropagationScaling`, 'returnType') = 'void'
attr(`ShapeDetectionLevelSetImageFilter_SetPropagationScaling`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter', 'numeric')
class(`ShapeDetectionLevelSetImageFilter_SetPropagationScaling`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_SetPropagationScaling'))

# Start of ShapeDetectionLevelSetImageFilter_GetPropagationScaling

`ShapeDetectionLevelSetImageFilter_GetPropagationScaling` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShapeDetectionLevelSetImageFilter_GetPropagationScaling', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShapeDetectionLevelSetImageFilter_GetPropagationScaling`, 'returnType') = 'numeric'
attr(`ShapeDetectionLevelSetImageFilter_GetPropagationScaling`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter')
class(`ShapeDetectionLevelSetImageFilter_GetPropagationScaling`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_GetPropagationScaling'))

# Start of ShapeDetectionLevelSetImageFilter_SetCurvatureScaling

`ShapeDetectionLevelSetImageFilter_SetCurvatureScaling` = function(self, CurvatureScaling)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ShapeDetectionLevelSetImageFilter_SetCurvatureScaling', self, CurvatureScaling, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShapeDetectionLevelSetImageFilter_SetCurvatureScaling`, 'returnType') = 'void'
attr(`ShapeDetectionLevelSetImageFilter_SetCurvatureScaling`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter', 'numeric')
class(`ShapeDetectionLevelSetImageFilter_SetCurvatureScaling`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_SetCurvatureScaling'))

# Start of ShapeDetectionLevelSetImageFilter_GetCurvatureScaling

`ShapeDetectionLevelSetImageFilter_GetCurvatureScaling` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShapeDetectionLevelSetImageFilter_GetCurvatureScaling', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShapeDetectionLevelSetImageFilter_GetCurvatureScaling`, 'returnType') = 'numeric'
attr(`ShapeDetectionLevelSetImageFilter_GetCurvatureScaling`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter')
class(`ShapeDetectionLevelSetImageFilter_GetCurvatureScaling`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_GetCurvatureScaling'))

# Start of ShapeDetectionLevelSetImageFilter_SetNumberOfIterations

`ShapeDetectionLevelSetImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_ShapeDetectionLevelSetImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShapeDetectionLevelSetImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`ShapeDetectionLevelSetImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter', 'integer')
class(`ShapeDetectionLevelSetImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_SetNumberOfIterations'))

# Start of ShapeDetectionLevelSetImageFilter_GetNumberOfIterations

`ShapeDetectionLevelSetImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShapeDetectionLevelSetImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShapeDetectionLevelSetImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`ShapeDetectionLevelSetImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter')
class(`ShapeDetectionLevelSetImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_GetNumberOfIterations'))

# Start of ShapeDetectionLevelSetImageFilter_SetReverseExpansionDirection

`ShapeDetectionLevelSetImageFilter_SetReverseExpansionDirection` = function(self, ReverseExpansionDirection)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ReverseExpansionDirection = as.logical(ReverseExpansionDirection);
  ;ans = .Call('R_swig_ShapeDetectionLevelSetImageFilter_SetReverseExpansionDirection', self, ReverseExpansionDirection, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShapeDetectionLevelSetImageFilter_SetReverseExpansionDirection`, 'returnType') = 'void'
attr(`ShapeDetectionLevelSetImageFilter_SetReverseExpansionDirection`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter', 'logical')
class(`ShapeDetectionLevelSetImageFilter_SetReverseExpansionDirection`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_SetReverseExpansionDirection'))

# Start of ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOn

`ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOn`, 'returnType') = 'void'
attr(`ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOn`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter')
class(`ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOn`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOn'))

# Start of ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOff

`ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOff`, 'returnType') = 'void'
attr(`ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOff`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter')
class(`ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOff`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOff'))

# Start of ShapeDetectionLevelSetImageFilter_GetReverseExpansionDirection

`ShapeDetectionLevelSetImageFilter_GetReverseExpansionDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShapeDetectionLevelSetImageFilter_GetReverseExpansionDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShapeDetectionLevelSetImageFilter_GetReverseExpansionDirection`, 'returnType') = 'logical'
attr(`ShapeDetectionLevelSetImageFilter_GetReverseExpansionDirection`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter')
class(`ShapeDetectionLevelSetImageFilter_GetReverseExpansionDirection`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_GetReverseExpansionDirection'))

# Start of ShapeDetectionLevelSetImageFilter_GetElapsedIterations

`ShapeDetectionLevelSetImageFilter_GetElapsedIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShapeDetectionLevelSetImageFilter_GetElapsedIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShapeDetectionLevelSetImageFilter_GetElapsedIterations`, 'returnType') = 'integer'
attr(`ShapeDetectionLevelSetImageFilter_GetElapsedIterations`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter')
class(`ShapeDetectionLevelSetImageFilter_GetElapsedIterations`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_GetElapsedIterations'))

# Start of ShapeDetectionLevelSetImageFilter_GetRMSChange

`ShapeDetectionLevelSetImageFilter_GetRMSChange` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShapeDetectionLevelSetImageFilter_GetRMSChange', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShapeDetectionLevelSetImageFilter_GetRMSChange`, 'returnType') = 'numeric'
attr(`ShapeDetectionLevelSetImageFilter_GetRMSChange`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter')
class(`ShapeDetectionLevelSetImageFilter_GetRMSChange`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_GetRMSChange'))

# Start of ShapeDetectionLevelSetImageFilter_GetName

`ShapeDetectionLevelSetImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShapeDetectionLevelSetImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShapeDetectionLevelSetImageFilter_GetName`, 'returnType') = 'character'
attr(`ShapeDetectionLevelSetImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter')
class(`ShapeDetectionLevelSetImageFilter_GetName`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_GetName'))

# Start of ShapeDetectionLevelSetImageFilter_ToString

`ShapeDetectionLevelSetImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShapeDetectionLevelSetImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShapeDetectionLevelSetImageFilter_ToString`, 'returnType') = 'character'
attr(`ShapeDetectionLevelSetImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter')
class(`ShapeDetectionLevelSetImageFilter_ToString`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_ToString'))

# Start of ShapeDetectionLevelSetImageFilter_Execute

`ShapeDetectionLevelSetImageFilter_Execute` = function(self, initialImage, featureImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_ShapeDetectionLevelSetImageFilter_Execute', self, initialImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShapeDetectionLevelSetImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ShapeDetectionLevelSetImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ShapeDetectionLevelSetImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ShapeDetectionLevelSetImageFilter_Execute`) = c("SWIGFunction", class('ShapeDetectionLevelSetImageFilter_Execute'))

# Start of accessor method for itk::simple::ShapeDetectionLevelSetImageFilter
setMethod('$', '_p_itk__simple__ShapeDetectionLevelSetImageFilter', function(x, name)

{
  accessorFuns = list('SetMaximumRMSError' = ShapeDetectionLevelSetImageFilter_SetMaximumRMSError, 'GetMaximumRMSError' = ShapeDetectionLevelSetImageFilter_GetMaximumRMSError, 'SetPropagationScaling' = ShapeDetectionLevelSetImageFilter_SetPropagationScaling, 'GetPropagationScaling' = ShapeDetectionLevelSetImageFilter_GetPropagationScaling, 'SetCurvatureScaling' = ShapeDetectionLevelSetImageFilter_SetCurvatureScaling, 'GetCurvatureScaling' = ShapeDetectionLevelSetImageFilter_GetCurvatureScaling, 'SetNumberOfIterations' = ShapeDetectionLevelSetImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = ShapeDetectionLevelSetImageFilter_GetNumberOfIterations, 'SetReverseExpansionDirection' = ShapeDetectionLevelSetImageFilter_SetReverseExpansionDirection, 'ReverseExpansionDirectionOn' = ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOn, 'ReverseExpansionDirectionOff' = ShapeDetectionLevelSetImageFilter_ReverseExpansionDirectionOff, 'GetReverseExpansionDirection' = ShapeDetectionLevelSetImageFilter_GetReverseExpansionDirection, 'GetElapsedIterations' = ShapeDetectionLevelSetImageFilter_GetElapsedIterations, 'GetRMSChange' = ShapeDetectionLevelSetImageFilter_GetRMSChange, 'GetName' = ShapeDetectionLevelSetImageFilter_GetName, 'ToString' = ShapeDetectionLevelSetImageFilter_ToString, 'Execute' = ShapeDetectionLevelSetImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ShapeDetectionLevelSetImageFilter
# Start of ShapeDetectionLevelSet

`ShapeDetectionLevelSet__SWIG_0` = function(initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, numberOfIterations, reverseExpansionDirection, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  reverseExpansionDirection = as.logical(reverseExpansionDirection);
  ;ans = .Call('R_swig_ShapeDetectionLevelSet__SWIG_0', initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, numberOfIterations, reverseExpansionDirection, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShapeDetectionLevelSet__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ShapeDetectionLevelSet__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'integer', 'logical')
class(`ShapeDetectionLevelSet__SWIG_0`) = c("SWIGFunction", class('ShapeDetectionLevelSet__SWIG_0'))

# Start of ShapeDetectionLevelSet

`ShapeDetectionLevelSet__SWIG_1` = function(initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, numberOfIterations, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_ShapeDetectionLevelSet__SWIG_1', initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShapeDetectionLevelSet__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ShapeDetectionLevelSet__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'integer')
class(`ShapeDetectionLevelSet__SWIG_1`) = c("SWIGFunction", class('ShapeDetectionLevelSet__SWIG_1'))

# Start of ShapeDetectionLevelSet

`ShapeDetectionLevelSet__SWIG_2` = function(initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  ;ans = .Call('R_swig_ShapeDetectionLevelSet__SWIG_2', initialImage, featureImage, maximumRMSError, propagationScaling, curvatureScaling, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShapeDetectionLevelSet__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ShapeDetectionLevelSet__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric')
class(`ShapeDetectionLevelSet__SWIG_2`) = c("SWIGFunction", class('ShapeDetectionLevelSet__SWIG_2'))

# Start of ShapeDetectionLevelSet

`ShapeDetectionLevelSet__SWIG_3` = function(initialImage, featureImage, maximumRMSError, propagationScaling, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  ;ans = .Call('R_swig_ShapeDetectionLevelSet__SWIG_3', initialImage, featureImage, maximumRMSError, propagationScaling, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShapeDetectionLevelSet__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ShapeDetectionLevelSet__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`ShapeDetectionLevelSet__SWIG_3`) = c("SWIGFunction", class('ShapeDetectionLevelSet__SWIG_3'))

# Start of ShapeDetectionLevelSet

`ShapeDetectionLevelSet__SWIG_4` = function(initialImage, featureImage, maximumRMSError, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  ;ans = .Call('R_swig_ShapeDetectionLevelSet__SWIG_4', initialImage, featureImage, maximumRMSError, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShapeDetectionLevelSet__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ShapeDetectionLevelSet__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`ShapeDetectionLevelSet__SWIG_4`) = c("SWIGFunction", class('ShapeDetectionLevelSet__SWIG_4'))

# Start of ShapeDetectionLevelSet

`ShapeDetectionLevelSet__SWIG_5` = function(initialImage, featureImage, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_ShapeDetectionLevelSet__SWIG_5', initialImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShapeDetectionLevelSet__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`ShapeDetectionLevelSet__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ShapeDetectionLevelSet__SWIG_5`) = c("SWIGFunction", class('ShapeDetectionLevelSet__SWIG_5'))

`ShapeDetectionLevelSet` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- ShapeDetectionLevelSet__SWIG_5; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ShapeDetectionLevelSet__SWIG_4; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ShapeDetectionLevelSet__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- ShapeDetectionLevelSet__SWIG_2; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- ShapeDetectionLevelSet__SWIG_1; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 ) && ( is.logical(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- ShapeDetectionLevelSet__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ShapeDetectionLevelSet with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ShiftScaleImageFilter

`delete_ShiftScaleImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ShiftScaleImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ShiftScaleImageFilter`, 'returnType') = 'void'
attr(`delete_ShiftScaleImageFilter`, "inputTypes") = c('_p_itk__simple__ShiftScaleImageFilter')
class(`delete_ShiftScaleImageFilter`) = c("SWIGFunction", class('delete_ShiftScaleImageFilter'))

# Start of new_ShiftScaleImageFilter

`ShiftScaleImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ShiftScaleImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ShiftScaleImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ShiftScaleImageFilter);
  ans
  
}

attr(`ShiftScaleImageFilter`, 'returnType') = '_p_itk__simple__ShiftScaleImageFilter'
class(`ShiftScaleImageFilter`) = c("SWIGFunction", class('ShiftScaleImageFilter'))

# Start of ShiftScaleImageFilter_SetShift

`ShiftScaleImageFilter_SetShift` = function(self, Shift)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ShiftScaleImageFilter_SetShift', self, Shift, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShiftScaleImageFilter_SetShift`, 'returnType') = 'void'
attr(`ShiftScaleImageFilter_SetShift`, "inputTypes") = c('_p_itk__simple__ShiftScaleImageFilter', 'numeric')
class(`ShiftScaleImageFilter_SetShift`) = c("SWIGFunction", class('ShiftScaleImageFilter_SetShift'))

# Start of ShiftScaleImageFilter_GetShift

`ShiftScaleImageFilter_GetShift` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShiftScaleImageFilter_GetShift', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShiftScaleImageFilter_GetShift`, 'returnType') = 'numeric'
attr(`ShiftScaleImageFilter_GetShift`, "inputTypes") = c('_p_itk__simple__ShiftScaleImageFilter')
class(`ShiftScaleImageFilter_GetShift`) = c("SWIGFunction", class('ShiftScaleImageFilter_GetShift'))

# Start of ShiftScaleImageFilter_SetScale

`ShiftScaleImageFilter_SetScale` = function(self, Scale)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ShiftScaleImageFilter_SetScale', self, Scale, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShiftScaleImageFilter_SetScale`, 'returnType') = 'void'
attr(`ShiftScaleImageFilter_SetScale`, "inputTypes") = c('_p_itk__simple__ShiftScaleImageFilter', 'numeric')
class(`ShiftScaleImageFilter_SetScale`) = c("SWIGFunction", class('ShiftScaleImageFilter_SetScale'))

# Start of ShiftScaleImageFilter_GetScale

`ShiftScaleImageFilter_GetScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShiftScaleImageFilter_GetScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShiftScaleImageFilter_GetScale`, 'returnType') = 'numeric'
attr(`ShiftScaleImageFilter_GetScale`, "inputTypes") = c('_p_itk__simple__ShiftScaleImageFilter')
class(`ShiftScaleImageFilter_GetScale`) = c("SWIGFunction", class('ShiftScaleImageFilter_GetScale'))

# Start of ShiftScaleImageFilter_SetOutputPixelType

`ShiftScaleImageFilter_SetOutputPixelType` = function(self, OutputPixelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputPixelType = enumToInteger(OutputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(OutputPixelType) > 1) {
    warning("using only the first element of OutputPixelType");
  };
  
  ;ans = .Call('R_swig_ShiftScaleImageFilter_SetOutputPixelType', self, OutputPixelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShiftScaleImageFilter_SetOutputPixelType`, 'returnType') = 'void'
attr(`ShiftScaleImageFilter_SetOutputPixelType`, "inputTypes") = c('_p_itk__simple__ShiftScaleImageFilter', 'character')
class(`ShiftScaleImageFilter_SetOutputPixelType`) = c("SWIGFunction", class('ShiftScaleImageFilter_SetOutputPixelType'))

# Start of ShiftScaleImageFilter_GetOutputPixelType

`ShiftScaleImageFilter_GetOutputPixelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ShiftScaleImageFilter_GetOutputPixelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`ShiftScaleImageFilter_GetOutputPixelType`, 'returnType') = 'character'
attr(`ShiftScaleImageFilter_GetOutputPixelType`, "inputTypes") = c('_p_itk__simple__ShiftScaleImageFilter')
class(`ShiftScaleImageFilter_GetOutputPixelType`) = c("SWIGFunction", class('ShiftScaleImageFilter_GetOutputPixelType'))

# Start of ShiftScaleImageFilter_GetUnderflowCount

`ShiftScaleImageFilter_GetUnderflowCount` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShiftScaleImageFilter_GetUnderflowCount', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShiftScaleImageFilter_GetUnderflowCount`, 'returnType') = 'integer'
attr(`ShiftScaleImageFilter_GetUnderflowCount`, "inputTypes") = c('_p_itk__simple__ShiftScaleImageFilter')
class(`ShiftScaleImageFilter_GetUnderflowCount`) = c("SWIGFunction", class('ShiftScaleImageFilter_GetUnderflowCount'))

# Start of ShiftScaleImageFilter_GetOverflowCount

`ShiftScaleImageFilter_GetOverflowCount` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShiftScaleImageFilter_GetOverflowCount', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShiftScaleImageFilter_GetOverflowCount`, 'returnType') = 'integer'
attr(`ShiftScaleImageFilter_GetOverflowCount`, "inputTypes") = c('_p_itk__simple__ShiftScaleImageFilter')
class(`ShiftScaleImageFilter_GetOverflowCount`) = c("SWIGFunction", class('ShiftScaleImageFilter_GetOverflowCount'))

# Start of ShiftScaleImageFilter_GetName

`ShiftScaleImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShiftScaleImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShiftScaleImageFilter_GetName`, 'returnType') = 'character'
attr(`ShiftScaleImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ShiftScaleImageFilter')
class(`ShiftScaleImageFilter_GetName`) = c("SWIGFunction", class('ShiftScaleImageFilter_GetName'))

# Start of ShiftScaleImageFilter_ToString

`ShiftScaleImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShiftScaleImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShiftScaleImageFilter_ToString`, 'returnType') = 'character'
attr(`ShiftScaleImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ShiftScaleImageFilter')
class(`ShiftScaleImageFilter_ToString`) = c("SWIGFunction", class('ShiftScaleImageFilter_ToString'))

# Start of ShiftScaleImageFilter_Execute

`ShiftScaleImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ShiftScaleImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShiftScaleImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ShiftScaleImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ShiftScaleImageFilter', '_p_itk__simple__Image')
class(`ShiftScaleImageFilter_Execute`) = c("SWIGFunction", class('ShiftScaleImageFilter_Execute'))

# Start of accessor method for itk::simple::ShiftScaleImageFilter
setMethod('$', '_p_itk__simple__ShiftScaleImageFilter', function(x, name)

{
  accessorFuns = list('SetShift' = ShiftScaleImageFilter_SetShift, 'GetShift' = ShiftScaleImageFilter_GetShift, 'SetScale' = ShiftScaleImageFilter_SetScale, 'GetScale' = ShiftScaleImageFilter_GetScale, 'SetOutputPixelType' = ShiftScaleImageFilter_SetOutputPixelType, 'GetOutputPixelType' = ShiftScaleImageFilter_GetOutputPixelType, 'GetUnderflowCount' = ShiftScaleImageFilter_GetUnderflowCount, 'GetOverflowCount' = ShiftScaleImageFilter_GetOverflowCount, 'GetName' = ShiftScaleImageFilter_GetName, 'ToString' = ShiftScaleImageFilter_ToString, 'Execute' = ShiftScaleImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ShiftScaleImageFilter
# Start of ShiftScale

`ShiftScale__SWIG_0` = function(image1, shift, scale, outputPixelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  ;ans = .Call('R_swig_ShiftScale__SWIG_0', image1, shift, scale, outputPixelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShiftScale__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ShiftScale__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'character')
class(`ShiftScale__SWIG_0`) = c("SWIGFunction", class('ShiftScale__SWIG_0'))

# Start of ShiftScale

`ShiftScale__SWIG_1` = function(image1, shift, scale, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_ShiftScale__SWIG_1', image1, shift, scale, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShiftScale__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ShiftScale__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`ShiftScale__SWIG_1`) = c("SWIGFunction", class('ShiftScale__SWIG_1'))

# Start of ShiftScale

`ShiftScale__SWIG_2` = function(image1, shift, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_ShiftScale__SWIG_2', image1, shift, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShiftScale__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ShiftScale__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`ShiftScale__SWIG_2`) = c("SWIGFunction", class('ShiftScale__SWIG_2'))

# Start of ShiftScale

`ShiftScale__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ShiftScale__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShiftScale__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ShiftScale__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`ShiftScale__SWIG_3`) = c("SWIGFunction", class('ShiftScale__SWIG_3'))

`ShiftScale` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ShiftScale__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ShiftScale__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ShiftScale__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.character(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ShiftScale__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ShiftScale with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ShotNoiseImageFilter

`delete_ShotNoiseImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ShotNoiseImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ShotNoiseImageFilter`, 'returnType') = 'void'
attr(`delete_ShotNoiseImageFilter`, "inputTypes") = c('_p_itk__simple__ShotNoiseImageFilter')
class(`delete_ShotNoiseImageFilter`) = c("SWIGFunction", class('delete_ShotNoiseImageFilter'))

# Start of new_ShotNoiseImageFilter

`ShotNoiseImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ShotNoiseImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ShotNoiseImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ShotNoiseImageFilter);
  ans
  
}

attr(`ShotNoiseImageFilter`, 'returnType') = '_p_itk__simple__ShotNoiseImageFilter'
class(`ShotNoiseImageFilter`) = c("SWIGFunction", class('ShotNoiseImageFilter'))

# Start of ShotNoiseImageFilter_SetScale

`ShotNoiseImageFilter_SetScale` = function(self, Scale)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ShotNoiseImageFilter_SetScale', self, Scale, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShotNoiseImageFilter_SetScale`, 'returnType') = 'void'
attr(`ShotNoiseImageFilter_SetScale`, "inputTypes") = c('_p_itk__simple__ShotNoiseImageFilter', 'numeric')
class(`ShotNoiseImageFilter_SetScale`) = c("SWIGFunction", class('ShotNoiseImageFilter_SetScale'))

# Start of ShotNoiseImageFilter_GetScale

`ShotNoiseImageFilter_GetScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShotNoiseImageFilter_GetScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShotNoiseImageFilter_GetScale`, 'returnType') = 'numeric'
attr(`ShotNoiseImageFilter_GetScale`, "inputTypes") = c('_p_itk__simple__ShotNoiseImageFilter')
class(`ShotNoiseImageFilter_GetScale`) = c("SWIGFunction", class('ShotNoiseImageFilter_GetScale'))

# Start of ShotNoiseImageFilter_SetSeed

`ShotNoiseImageFilter_SetSeed` = function(self, Seed)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Seed = as.integer(Seed);
  
  if(length(Seed) > 1) {
    warning("using only the first element of Seed");
  };
  
  ;ans = .Call('R_swig_ShotNoiseImageFilter_SetSeed', self, Seed, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShotNoiseImageFilter_SetSeed`, 'returnType') = 'void'
attr(`ShotNoiseImageFilter_SetSeed`, "inputTypes") = c('_p_itk__simple__ShotNoiseImageFilter', 'integer')
class(`ShotNoiseImageFilter_SetSeed`) = c("SWIGFunction", class('ShotNoiseImageFilter_SetSeed'))

# Start of ShotNoiseImageFilter_GetSeed

`ShotNoiseImageFilter_GetSeed` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShotNoiseImageFilter_GetSeed', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShotNoiseImageFilter_GetSeed`, 'returnType') = 'integer'
attr(`ShotNoiseImageFilter_GetSeed`, "inputTypes") = c('_p_itk__simple__ShotNoiseImageFilter')
class(`ShotNoiseImageFilter_GetSeed`) = c("SWIGFunction", class('ShotNoiseImageFilter_GetSeed'))

# Start of ShotNoiseImageFilter_GetName

`ShotNoiseImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShotNoiseImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShotNoiseImageFilter_GetName`, 'returnType') = 'character'
attr(`ShotNoiseImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ShotNoiseImageFilter')
class(`ShotNoiseImageFilter_GetName`) = c("SWIGFunction", class('ShotNoiseImageFilter_GetName'))

# Start of ShotNoiseImageFilter_ToString

`ShotNoiseImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShotNoiseImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShotNoiseImageFilter_ToString`, 'returnType') = 'character'
attr(`ShotNoiseImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ShotNoiseImageFilter')
class(`ShotNoiseImageFilter_ToString`) = c("SWIGFunction", class('ShotNoiseImageFilter_ToString'))

# Start of ShotNoiseImageFilter_Execute

`ShotNoiseImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ShotNoiseImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShotNoiseImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ShotNoiseImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ShotNoiseImageFilter', '_p_itk__simple__Image')
class(`ShotNoiseImageFilter_Execute`) = c("SWIGFunction", class('ShotNoiseImageFilter_Execute'))

# Start of accessor method for itk::simple::ShotNoiseImageFilter
setMethod('$', '_p_itk__simple__ShotNoiseImageFilter', function(x, name)

{
  accessorFuns = list('SetScale' = ShotNoiseImageFilter_SetScale, 'GetScale' = ShotNoiseImageFilter_GetScale, 'SetSeed' = ShotNoiseImageFilter_SetSeed, 'GetSeed' = ShotNoiseImageFilter_GetSeed, 'GetName' = ShotNoiseImageFilter_GetName, 'ToString' = ShotNoiseImageFilter_ToString, 'Execute' = ShotNoiseImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ShotNoiseImageFilter
# Start of ShotNoise

`ShotNoise__SWIG_0` = function(image1, scale, seed, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  seed = as.integer(seed);
  
  if(length(seed) > 1) {
    warning("using only the first element of seed");
  };
  
  ;ans = .Call('R_swig_ShotNoise__SWIG_0', image1, scale, seed, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShotNoise__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ShotNoise__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`ShotNoise__SWIG_0`) = c("SWIGFunction", class('ShotNoise__SWIG_0'))

# Start of ShotNoise

`ShotNoise__SWIG_1` = function(image1, scale, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_ShotNoise__SWIG_1', image1, scale, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShotNoise__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ShotNoise__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`ShotNoise__SWIG_1`) = c("SWIGFunction", class('ShotNoise__SWIG_1'))

# Start of ShotNoise

`ShotNoise__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ShotNoise__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShotNoise__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ShotNoise__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`ShotNoise__SWIG_2`) = c("SWIGFunction", class('ShotNoise__SWIG_2'))

`ShotNoise` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ShotNoise__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ShotNoise__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ShotNoise__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ShotNoise with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ShrinkImageFilter

`delete_ShrinkImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ShrinkImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ShrinkImageFilter`, 'returnType') = 'void'
attr(`delete_ShrinkImageFilter`, "inputTypes") = c('_p_itk__simple__ShrinkImageFilter')
class(`delete_ShrinkImageFilter`) = c("SWIGFunction", class('delete_ShrinkImageFilter'))

# Start of new_ShrinkImageFilter

`ShrinkImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ShrinkImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ShrinkImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ShrinkImageFilter);
  ans
  
}

attr(`ShrinkImageFilter`, 'returnType') = '_p_itk__simple__ShrinkImageFilter'
class(`ShrinkImageFilter`) = c("SWIGFunction", class('ShrinkImageFilter'))

# Start of ShrinkImageFilter_SetShrinkFactor

`ShrinkImageFilter_SetShrinkFactor` = function(self, s)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  s = as.integer(s);
  
  if(length(s) > 1) {
    warning("using only the first element of s");
  };
  
  ;ans = .Call('R_swig_ShrinkImageFilter_SetShrinkFactor', self, s, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShrinkImageFilter_SetShrinkFactor`, 'returnType') = 'void'
attr(`ShrinkImageFilter_SetShrinkFactor`, "inputTypes") = c('_p_itk__simple__ShrinkImageFilter', 'integer')
class(`ShrinkImageFilter_SetShrinkFactor`) = c("SWIGFunction", class('ShrinkImageFilter_SetShrinkFactor'))

# Start of ShrinkImageFilter_SetShrinkFactors

`ShrinkImageFilter_SetShrinkFactors` = function(self, ShrinkFactors)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ShrinkFactors = as.integer(ShrinkFactors);
  ;ans = .Call('R_swig_ShrinkImageFilter_SetShrinkFactors', self, ShrinkFactors, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ShrinkImageFilter_SetShrinkFactors`, 'returnType') = 'void'
attr(`ShrinkImageFilter_SetShrinkFactors`, "inputTypes") = c('_p_itk__simple__ShrinkImageFilter', 'integer')
class(`ShrinkImageFilter_SetShrinkFactors`) = c("SWIGFunction", class('ShrinkImageFilter_SetShrinkFactors'))

# Start of ShrinkImageFilter_GetShrinkFactors

`ShrinkImageFilter_GetShrinkFactors` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShrinkImageFilter_GetShrinkFactors', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShrinkImageFilter_GetShrinkFactors`, 'returnType') = 'integer'
attr(`ShrinkImageFilter_GetShrinkFactors`, "inputTypes") = c('_p_itk__simple__ShrinkImageFilter')
class(`ShrinkImageFilter_GetShrinkFactors`) = c("SWIGFunction", class('ShrinkImageFilter_GetShrinkFactors'))

# Start of ShrinkImageFilter_GetName

`ShrinkImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShrinkImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShrinkImageFilter_GetName`, 'returnType') = 'character'
attr(`ShrinkImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ShrinkImageFilter')
class(`ShrinkImageFilter_GetName`) = c("SWIGFunction", class('ShrinkImageFilter_GetName'))

# Start of ShrinkImageFilter_ToString

`ShrinkImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ShrinkImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ShrinkImageFilter_ToString`, 'returnType') = 'character'
attr(`ShrinkImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ShrinkImageFilter')
class(`ShrinkImageFilter_ToString`) = c("SWIGFunction", class('ShrinkImageFilter_ToString'))

# Start of ShrinkImageFilter_Execute

`ShrinkImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ShrinkImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ShrinkImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ShrinkImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ShrinkImageFilter', '_p_itk__simple__Image')
class(`ShrinkImageFilter_Execute`) = c("SWIGFunction", class('ShrinkImageFilter_Execute'))

# Start of accessor method for itk::simple::ShrinkImageFilter
setMethod('$', '_p_itk__simple__ShrinkImageFilter', function(x, name)

{
  accessorFuns = list('SetShrinkFactor' = ShrinkImageFilter_SetShrinkFactor, 'SetShrinkFactors' = ShrinkImageFilter_SetShrinkFactors, 'GetShrinkFactors' = ShrinkImageFilter_GetShrinkFactors, 'GetName' = ShrinkImageFilter_GetName, 'ToString' = ShrinkImageFilter_ToString, 'Execute' = ShrinkImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ShrinkImageFilter
# Start of Shrink

`Shrink__SWIG_0` = function(image1, shrinkFactors, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  shrinkFactors = as.integer(shrinkFactors);
  ;ans = .Call('R_swig_Shrink__SWIG_0', image1, shrinkFactors, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Shrink__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Shrink__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Shrink__SWIG_0`) = c("SWIGFunction", class('Shrink__SWIG_0'))

# Start of Shrink

`Shrink__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Shrink__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Shrink__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Shrink__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`Shrink__SWIG_1`) = c("SWIGFunction", class('Shrink__SWIG_1'))

`Shrink` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Shrink__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- Shrink__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Shrink with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_SigmoidImageFilter

`delete_SigmoidImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SigmoidImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SigmoidImageFilter`, 'returnType') = 'void'
attr(`delete_SigmoidImageFilter`, "inputTypes") = c('_p_itk__simple__SigmoidImageFilter')
class(`delete_SigmoidImageFilter`) = c("SWIGFunction", class('delete_SigmoidImageFilter'))

# Start of new_SigmoidImageFilter

`SigmoidImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SigmoidImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SigmoidImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SigmoidImageFilter);
  ans
  
}

attr(`SigmoidImageFilter`, 'returnType') = '_p_itk__simple__SigmoidImageFilter'
class(`SigmoidImageFilter`) = c("SWIGFunction", class('SigmoidImageFilter'))

# Start of SigmoidImageFilter_SetAlpha

`SigmoidImageFilter_SetAlpha` = function(self, Alpha)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SigmoidImageFilter_SetAlpha', self, Alpha, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SigmoidImageFilter_SetAlpha`, 'returnType') = 'void'
attr(`SigmoidImageFilter_SetAlpha`, "inputTypes") = c('_p_itk__simple__SigmoidImageFilter', 'numeric')
class(`SigmoidImageFilter_SetAlpha`) = c("SWIGFunction", class('SigmoidImageFilter_SetAlpha'))

# Start of SigmoidImageFilter_GetAlpha

`SigmoidImageFilter_GetAlpha` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SigmoidImageFilter_GetAlpha', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SigmoidImageFilter_GetAlpha`, 'returnType') = 'numeric'
attr(`SigmoidImageFilter_GetAlpha`, "inputTypes") = c('_p_itk__simple__SigmoidImageFilter')
class(`SigmoidImageFilter_GetAlpha`) = c("SWIGFunction", class('SigmoidImageFilter_GetAlpha'))

# Start of SigmoidImageFilter_SetBeta

`SigmoidImageFilter_SetBeta` = function(self, Beta)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SigmoidImageFilter_SetBeta', self, Beta, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SigmoidImageFilter_SetBeta`, 'returnType') = 'void'
attr(`SigmoidImageFilter_SetBeta`, "inputTypes") = c('_p_itk__simple__SigmoidImageFilter', 'numeric')
class(`SigmoidImageFilter_SetBeta`) = c("SWIGFunction", class('SigmoidImageFilter_SetBeta'))

# Start of SigmoidImageFilter_GetBeta

`SigmoidImageFilter_GetBeta` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SigmoidImageFilter_GetBeta', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SigmoidImageFilter_GetBeta`, 'returnType') = 'numeric'
attr(`SigmoidImageFilter_GetBeta`, "inputTypes") = c('_p_itk__simple__SigmoidImageFilter')
class(`SigmoidImageFilter_GetBeta`) = c("SWIGFunction", class('SigmoidImageFilter_GetBeta'))

# Start of SigmoidImageFilter_SetOutputMaximum

`SigmoidImageFilter_SetOutputMaximum` = function(self, OutputMaximum)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SigmoidImageFilter_SetOutputMaximum', self, OutputMaximum, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SigmoidImageFilter_SetOutputMaximum`, 'returnType') = 'void'
attr(`SigmoidImageFilter_SetOutputMaximum`, "inputTypes") = c('_p_itk__simple__SigmoidImageFilter', 'numeric')
class(`SigmoidImageFilter_SetOutputMaximum`) = c("SWIGFunction", class('SigmoidImageFilter_SetOutputMaximum'))

# Start of SigmoidImageFilter_GetOutputMaximum

`SigmoidImageFilter_GetOutputMaximum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SigmoidImageFilter_GetOutputMaximum', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SigmoidImageFilter_GetOutputMaximum`, 'returnType') = 'numeric'
attr(`SigmoidImageFilter_GetOutputMaximum`, "inputTypes") = c('_p_itk__simple__SigmoidImageFilter')
class(`SigmoidImageFilter_GetOutputMaximum`) = c("SWIGFunction", class('SigmoidImageFilter_GetOutputMaximum'))

# Start of SigmoidImageFilter_SetOutputMinimum

`SigmoidImageFilter_SetOutputMinimum` = function(self, OutputMinimum)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SigmoidImageFilter_SetOutputMinimum', self, OutputMinimum, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SigmoidImageFilter_SetOutputMinimum`, 'returnType') = 'void'
attr(`SigmoidImageFilter_SetOutputMinimum`, "inputTypes") = c('_p_itk__simple__SigmoidImageFilter', 'numeric')
class(`SigmoidImageFilter_SetOutputMinimum`) = c("SWIGFunction", class('SigmoidImageFilter_SetOutputMinimum'))

# Start of SigmoidImageFilter_GetOutputMinimum

`SigmoidImageFilter_GetOutputMinimum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SigmoidImageFilter_GetOutputMinimum', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SigmoidImageFilter_GetOutputMinimum`, 'returnType') = 'numeric'
attr(`SigmoidImageFilter_GetOutputMinimum`, "inputTypes") = c('_p_itk__simple__SigmoidImageFilter')
class(`SigmoidImageFilter_GetOutputMinimum`) = c("SWIGFunction", class('SigmoidImageFilter_GetOutputMinimum'))

# Start of SigmoidImageFilter_GetName

`SigmoidImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SigmoidImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SigmoidImageFilter_GetName`, 'returnType') = 'character'
attr(`SigmoidImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SigmoidImageFilter')
class(`SigmoidImageFilter_GetName`) = c("SWIGFunction", class('SigmoidImageFilter_GetName'))

# Start of SigmoidImageFilter_ToString

`SigmoidImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SigmoidImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SigmoidImageFilter_ToString`, 'returnType') = 'character'
attr(`SigmoidImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SigmoidImageFilter')
class(`SigmoidImageFilter_ToString`) = c("SWIGFunction", class('SigmoidImageFilter_ToString'))

# Start of SigmoidImageFilter_Execute

`SigmoidImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SigmoidImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SigmoidImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`SigmoidImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__SigmoidImageFilter', '_p_itk__simple__Image')
class(`SigmoidImageFilter_Execute`) = c("SWIGFunction", class('SigmoidImageFilter_Execute'))

# Start of accessor method for itk::simple::SigmoidImageFilter
setMethod('$', '_p_itk__simple__SigmoidImageFilter', function(x, name)

{
  accessorFuns = list('SetAlpha' = SigmoidImageFilter_SetAlpha, 'GetAlpha' = SigmoidImageFilter_GetAlpha, 'SetBeta' = SigmoidImageFilter_SetBeta, 'GetBeta' = SigmoidImageFilter_GetBeta, 'SetOutputMaximum' = SigmoidImageFilter_SetOutputMaximum, 'GetOutputMaximum' = SigmoidImageFilter_GetOutputMaximum, 'SetOutputMinimum' = SigmoidImageFilter_SetOutputMinimum, 'GetOutputMinimum' = SigmoidImageFilter_GetOutputMinimum, 'GetName' = SigmoidImageFilter_GetName, 'ToString' = SigmoidImageFilter_ToString, 'Execute' = SigmoidImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SigmoidImageFilter
# Start of Sigmoid

`Sigmoid__SWIG_0` = function(image1, alpha, beta, outputMaximum, outputMinimum, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  
  
  ;ans = .Call('R_swig_Sigmoid__SWIG_0', image1, alpha, beta, outputMaximum, outputMinimum, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Sigmoid__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Sigmoid__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric')
class(`Sigmoid__SWIG_0`) = c("SWIGFunction", class('Sigmoid__SWIG_0'))

# Start of Sigmoid

`Sigmoid__SWIG_1` = function(image1, alpha, beta, outputMaximum, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  
  ;ans = .Call('R_swig_Sigmoid__SWIG_1', image1, alpha, beta, outputMaximum, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Sigmoid__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Sigmoid__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric')
class(`Sigmoid__SWIG_1`) = c("SWIGFunction", class('Sigmoid__SWIG_1'))

# Start of Sigmoid

`Sigmoid__SWIG_2` = function(image1, alpha, beta, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_Sigmoid__SWIG_2', image1, alpha, beta, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Sigmoid__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Sigmoid__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`Sigmoid__SWIG_2`) = c("SWIGFunction", class('Sigmoid__SWIG_2'))

# Start of Sigmoid

`Sigmoid__SWIG_3` = function(image1, alpha, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Sigmoid__SWIG_3', image1, alpha, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Sigmoid__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Sigmoid__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`Sigmoid__SWIG_3`) = c("SWIGFunction", class('Sigmoid__SWIG_3'))

# Start of Sigmoid

`Sigmoid__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Sigmoid__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Sigmoid__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`Sigmoid__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`Sigmoid__SWIG_4`) = c("SWIGFunction", class('Sigmoid__SWIG_4'))

`Sigmoid` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Sigmoid__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Sigmoid__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Sigmoid__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- Sigmoid__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- Sigmoid__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Sigmoid with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_SignedDanielssonDistanceMapImageFilter

`delete_SignedDanielssonDistanceMapImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SignedDanielssonDistanceMapImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SignedDanielssonDistanceMapImageFilter`, 'returnType') = 'void'
attr(`delete_SignedDanielssonDistanceMapImageFilter`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter')
class(`delete_SignedDanielssonDistanceMapImageFilter`) = c("SWIGFunction", class('delete_SignedDanielssonDistanceMapImageFilter'))

# Start of new_SignedDanielssonDistanceMapImageFilter

`SignedDanielssonDistanceMapImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SignedDanielssonDistanceMapImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SignedDanielssonDistanceMapImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SignedDanielssonDistanceMapImageFilter);
  ans
  
}

attr(`SignedDanielssonDistanceMapImageFilter`, 'returnType') = '_p_itk__simple__SignedDanielssonDistanceMapImageFilter'
class(`SignedDanielssonDistanceMapImageFilter`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter'))

# Start of SignedDanielssonDistanceMapImageFilter_SetInsideIsPositive

`SignedDanielssonDistanceMapImageFilter_SetInsideIsPositive` = function(self, InsideIsPositive)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideIsPositive = as.logical(InsideIsPositive);
  ;ans = .Call('R_swig_SignedDanielssonDistanceMapImageFilter_SetInsideIsPositive', self, InsideIsPositive, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMapImageFilter_SetInsideIsPositive`, 'returnType') = 'void'
attr(`SignedDanielssonDistanceMapImageFilter_SetInsideIsPositive`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter', 'logical')
class(`SignedDanielssonDistanceMapImageFilter_SetInsideIsPositive`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_SetInsideIsPositive'))

# Start of SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOn

`SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOn`, 'returnType') = 'void'
attr(`SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOn`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter')
class(`SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOn`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOn'))

# Start of SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOff

`SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOff`, 'returnType') = 'void'
attr(`SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOff`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter')
class(`SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOff`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOff'))

# Start of SignedDanielssonDistanceMapImageFilter_GetInsideIsPositive

`SignedDanielssonDistanceMapImageFilter_GetInsideIsPositive` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SignedDanielssonDistanceMapImageFilter_GetInsideIsPositive', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SignedDanielssonDistanceMapImageFilter_GetInsideIsPositive`, 'returnType') = 'logical'
attr(`SignedDanielssonDistanceMapImageFilter_GetInsideIsPositive`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter')
class(`SignedDanielssonDistanceMapImageFilter_GetInsideIsPositive`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_GetInsideIsPositive'))

# Start of SignedDanielssonDistanceMapImageFilter_SetSquaredDistance

`SignedDanielssonDistanceMapImageFilter_SetSquaredDistance` = function(self, SquaredDistance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SquaredDistance = as.logical(SquaredDistance);
  ;ans = .Call('R_swig_SignedDanielssonDistanceMapImageFilter_SetSquaredDistance', self, SquaredDistance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMapImageFilter_SetSquaredDistance`, 'returnType') = 'void'
attr(`SignedDanielssonDistanceMapImageFilter_SetSquaredDistance`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter', 'logical')
class(`SignedDanielssonDistanceMapImageFilter_SetSquaredDistance`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_SetSquaredDistance'))

# Start of SignedDanielssonDistanceMapImageFilter_SquaredDistanceOn

`SignedDanielssonDistanceMapImageFilter_SquaredDistanceOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SignedDanielssonDistanceMapImageFilter_SquaredDistanceOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMapImageFilter_SquaredDistanceOn`, 'returnType') = 'void'
attr(`SignedDanielssonDistanceMapImageFilter_SquaredDistanceOn`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter')
class(`SignedDanielssonDistanceMapImageFilter_SquaredDistanceOn`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_SquaredDistanceOn'))

# Start of SignedDanielssonDistanceMapImageFilter_SquaredDistanceOff

`SignedDanielssonDistanceMapImageFilter_SquaredDistanceOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SignedDanielssonDistanceMapImageFilter_SquaredDistanceOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMapImageFilter_SquaredDistanceOff`, 'returnType') = 'void'
attr(`SignedDanielssonDistanceMapImageFilter_SquaredDistanceOff`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter')
class(`SignedDanielssonDistanceMapImageFilter_SquaredDistanceOff`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_SquaredDistanceOff'))

# Start of SignedDanielssonDistanceMapImageFilter_GetSquaredDistance

`SignedDanielssonDistanceMapImageFilter_GetSquaredDistance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SignedDanielssonDistanceMapImageFilter_GetSquaredDistance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SignedDanielssonDistanceMapImageFilter_GetSquaredDistance`, 'returnType') = 'logical'
attr(`SignedDanielssonDistanceMapImageFilter_GetSquaredDistance`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter')
class(`SignedDanielssonDistanceMapImageFilter_GetSquaredDistance`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_GetSquaredDistance'))

# Start of SignedDanielssonDistanceMapImageFilter_SetUseImageSpacing

`SignedDanielssonDistanceMapImageFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_SignedDanielssonDistanceMapImageFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMapImageFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`SignedDanielssonDistanceMapImageFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter', 'logical')
class(`SignedDanielssonDistanceMapImageFilter_SetUseImageSpacing`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_SetUseImageSpacing'))

# Start of SignedDanielssonDistanceMapImageFilter_UseImageSpacingOn

`SignedDanielssonDistanceMapImageFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SignedDanielssonDistanceMapImageFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMapImageFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`SignedDanielssonDistanceMapImageFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter')
class(`SignedDanielssonDistanceMapImageFilter_UseImageSpacingOn`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_UseImageSpacingOn'))

# Start of SignedDanielssonDistanceMapImageFilter_UseImageSpacingOff

`SignedDanielssonDistanceMapImageFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SignedDanielssonDistanceMapImageFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMapImageFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`SignedDanielssonDistanceMapImageFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter')
class(`SignedDanielssonDistanceMapImageFilter_UseImageSpacingOff`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_UseImageSpacingOff'))

# Start of SignedDanielssonDistanceMapImageFilter_GetUseImageSpacing

`SignedDanielssonDistanceMapImageFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SignedDanielssonDistanceMapImageFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SignedDanielssonDistanceMapImageFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`SignedDanielssonDistanceMapImageFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter')
class(`SignedDanielssonDistanceMapImageFilter_GetUseImageSpacing`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_GetUseImageSpacing'))

# Start of SignedDanielssonDistanceMapImageFilter_GetVoronoiMap

`SignedDanielssonDistanceMapImageFilter_GetVoronoiMap` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SignedDanielssonDistanceMapImageFilter_GetVoronoiMap', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMapImageFilter_GetVoronoiMap`, 'returnType') = '_p_itk__simple__Image'
attr(`SignedDanielssonDistanceMapImageFilter_GetVoronoiMap`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter')
class(`SignedDanielssonDistanceMapImageFilter_GetVoronoiMap`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_GetVoronoiMap'))

# Start of SignedDanielssonDistanceMapImageFilter_GetVectorDistanceMap

`SignedDanielssonDistanceMapImageFilter_GetVectorDistanceMap` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SignedDanielssonDistanceMapImageFilter_GetVectorDistanceMap', self, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMapImageFilter_GetVectorDistanceMap`, 'returnType') = '_p_itk__simple__Image'
attr(`SignedDanielssonDistanceMapImageFilter_GetVectorDistanceMap`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter')
class(`SignedDanielssonDistanceMapImageFilter_GetVectorDistanceMap`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_GetVectorDistanceMap'))

# Start of SignedDanielssonDistanceMapImageFilter_GetName

`SignedDanielssonDistanceMapImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SignedDanielssonDistanceMapImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SignedDanielssonDistanceMapImageFilter_GetName`, 'returnType') = 'character'
attr(`SignedDanielssonDistanceMapImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter')
class(`SignedDanielssonDistanceMapImageFilter_GetName`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_GetName'))

# Start of SignedDanielssonDistanceMapImageFilter_ToString

`SignedDanielssonDistanceMapImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SignedDanielssonDistanceMapImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SignedDanielssonDistanceMapImageFilter_ToString`, 'returnType') = 'character'
attr(`SignedDanielssonDistanceMapImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter')
class(`SignedDanielssonDistanceMapImageFilter_ToString`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_ToString'))

# Start of SignedDanielssonDistanceMapImageFilter_Execute

`SignedDanielssonDistanceMapImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SignedDanielssonDistanceMapImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMapImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`SignedDanielssonDistanceMapImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__SignedDanielssonDistanceMapImageFilter', '_p_itk__simple__Image')
class(`SignedDanielssonDistanceMapImageFilter_Execute`) = c("SWIGFunction", class('SignedDanielssonDistanceMapImageFilter_Execute'))

# Start of accessor method for itk::simple::SignedDanielssonDistanceMapImageFilter
setMethod('$', '_p_itk__simple__SignedDanielssonDistanceMapImageFilter', function(x, name)

{
  accessorFuns = list('SetInsideIsPositive' = SignedDanielssonDistanceMapImageFilter_SetInsideIsPositive, 'InsideIsPositiveOn' = SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOn, 'InsideIsPositiveOff' = SignedDanielssonDistanceMapImageFilter_InsideIsPositiveOff, 'GetInsideIsPositive' = SignedDanielssonDistanceMapImageFilter_GetInsideIsPositive, 'SetSquaredDistance' = SignedDanielssonDistanceMapImageFilter_SetSquaredDistance, 'SquaredDistanceOn' = SignedDanielssonDistanceMapImageFilter_SquaredDistanceOn, 'SquaredDistanceOff' = SignedDanielssonDistanceMapImageFilter_SquaredDistanceOff, 'GetSquaredDistance' = SignedDanielssonDistanceMapImageFilter_GetSquaredDistance, 'SetUseImageSpacing' = SignedDanielssonDistanceMapImageFilter_SetUseImageSpacing, 'UseImageSpacingOn' = SignedDanielssonDistanceMapImageFilter_UseImageSpacingOn, 'UseImageSpacingOff' = SignedDanielssonDistanceMapImageFilter_UseImageSpacingOff, 'GetUseImageSpacing' = SignedDanielssonDistanceMapImageFilter_GetUseImageSpacing, 'GetVoronoiMap' = SignedDanielssonDistanceMapImageFilter_GetVoronoiMap, 'GetVectorDistanceMap' = SignedDanielssonDistanceMapImageFilter_GetVectorDistanceMap, 'GetName' = SignedDanielssonDistanceMapImageFilter_GetName, 'ToString' = SignedDanielssonDistanceMapImageFilter_ToString, 'Execute' = SignedDanielssonDistanceMapImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SignedDanielssonDistanceMapImageFilter
# Start of SignedDanielssonDistanceMap

`SignedDanielssonDistanceMap__SWIG_0` = function(image1, insideIsPositive, squaredDistance, useImageSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  insideIsPositive = as.logical(insideIsPositive);
  squaredDistance = as.logical(squaredDistance);
  useImageSpacing = as.logical(useImageSpacing);
  ;ans = .Call('R_swig_SignedDanielssonDistanceMap__SWIG_0', image1, insideIsPositive, squaredDistance, useImageSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMap__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`SignedDanielssonDistanceMap__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'logical', 'logical')
class(`SignedDanielssonDistanceMap__SWIG_0`) = c("SWIGFunction", class('SignedDanielssonDistanceMap__SWIG_0'))

# Start of SignedDanielssonDistanceMap

`SignedDanielssonDistanceMap__SWIG_1` = function(image1, insideIsPositive, squaredDistance, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  insideIsPositive = as.logical(insideIsPositive);
  squaredDistance = as.logical(squaredDistance);
  ;ans = .Call('R_swig_SignedDanielssonDistanceMap__SWIG_1', image1, insideIsPositive, squaredDistance, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMap__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`SignedDanielssonDistanceMap__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'logical')
class(`SignedDanielssonDistanceMap__SWIG_1`) = c("SWIGFunction", class('SignedDanielssonDistanceMap__SWIG_1'))

# Start of SignedDanielssonDistanceMap

`SignedDanielssonDistanceMap__SWIG_2` = function(image1, insideIsPositive, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  insideIsPositive = as.logical(insideIsPositive);
  ;ans = .Call('R_swig_SignedDanielssonDistanceMap__SWIG_2', image1, insideIsPositive, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMap__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`SignedDanielssonDistanceMap__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`SignedDanielssonDistanceMap__SWIG_2`) = c("SWIGFunction", class('SignedDanielssonDistanceMap__SWIG_2'))

# Start of SignedDanielssonDistanceMap

`SignedDanielssonDistanceMap__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SignedDanielssonDistanceMap__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SignedDanielssonDistanceMap__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`SignedDanielssonDistanceMap__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`SignedDanielssonDistanceMap__SWIG_3`) = c("SWIGFunction", class('SignedDanielssonDistanceMap__SWIG_3'))

`SignedDanielssonDistanceMap` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- SignedDanielssonDistanceMap__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- SignedDanielssonDistanceMap__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- SignedDanielssonDistanceMap__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- SignedDanielssonDistanceMap__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SignedDanielssonDistanceMap with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_SignedMaurerDistanceMapImageFilter

`delete_SignedMaurerDistanceMapImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SignedMaurerDistanceMapImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SignedMaurerDistanceMapImageFilter`, 'returnType') = 'void'
attr(`delete_SignedMaurerDistanceMapImageFilter`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter')
class(`delete_SignedMaurerDistanceMapImageFilter`) = c("SWIGFunction", class('delete_SignedMaurerDistanceMapImageFilter'))

# Start of new_SignedMaurerDistanceMapImageFilter

`SignedMaurerDistanceMapImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SignedMaurerDistanceMapImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SignedMaurerDistanceMapImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SignedMaurerDistanceMapImageFilter);
  ans
  
}

attr(`SignedMaurerDistanceMapImageFilter`, 'returnType') = '_p_itk__simple__SignedMaurerDistanceMapImageFilter'
class(`SignedMaurerDistanceMapImageFilter`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter'))

# Start of SignedMaurerDistanceMapImageFilter_SetInsideIsPositive

`SignedMaurerDistanceMapImageFilter_SetInsideIsPositive` = function(self, InsideIsPositive)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideIsPositive = as.logical(InsideIsPositive);
  ;ans = .Call('R_swig_SignedMaurerDistanceMapImageFilter_SetInsideIsPositive', self, InsideIsPositive, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMapImageFilter_SetInsideIsPositive`, 'returnType') = 'void'
attr(`SignedMaurerDistanceMapImageFilter_SetInsideIsPositive`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter', 'logical')
class(`SignedMaurerDistanceMapImageFilter_SetInsideIsPositive`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_SetInsideIsPositive'))

# Start of SignedMaurerDistanceMapImageFilter_InsideIsPositiveOn

`SignedMaurerDistanceMapImageFilter_InsideIsPositiveOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SignedMaurerDistanceMapImageFilter_InsideIsPositiveOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMapImageFilter_InsideIsPositiveOn`, 'returnType') = 'void'
attr(`SignedMaurerDistanceMapImageFilter_InsideIsPositiveOn`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter')
class(`SignedMaurerDistanceMapImageFilter_InsideIsPositiveOn`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_InsideIsPositiveOn'))

# Start of SignedMaurerDistanceMapImageFilter_InsideIsPositiveOff

`SignedMaurerDistanceMapImageFilter_InsideIsPositiveOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SignedMaurerDistanceMapImageFilter_InsideIsPositiveOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMapImageFilter_InsideIsPositiveOff`, 'returnType') = 'void'
attr(`SignedMaurerDistanceMapImageFilter_InsideIsPositiveOff`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter')
class(`SignedMaurerDistanceMapImageFilter_InsideIsPositiveOff`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_InsideIsPositiveOff'))

# Start of SignedMaurerDistanceMapImageFilter_GetInsideIsPositive

`SignedMaurerDistanceMapImageFilter_GetInsideIsPositive` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SignedMaurerDistanceMapImageFilter_GetInsideIsPositive', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SignedMaurerDistanceMapImageFilter_GetInsideIsPositive`, 'returnType') = 'logical'
attr(`SignedMaurerDistanceMapImageFilter_GetInsideIsPositive`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter')
class(`SignedMaurerDistanceMapImageFilter_GetInsideIsPositive`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_GetInsideIsPositive'))

# Start of SignedMaurerDistanceMapImageFilter_SetSquaredDistance

`SignedMaurerDistanceMapImageFilter_SetSquaredDistance` = function(self, SquaredDistance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SquaredDistance = as.logical(SquaredDistance);
  ;ans = .Call('R_swig_SignedMaurerDistanceMapImageFilter_SetSquaredDistance', self, SquaredDistance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMapImageFilter_SetSquaredDistance`, 'returnType') = 'void'
attr(`SignedMaurerDistanceMapImageFilter_SetSquaredDistance`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter', 'logical')
class(`SignedMaurerDistanceMapImageFilter_SetSquaredDistance`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_SetSquaredDistance'))

# Start of SignedMaurerDistanceMapImageFilter_SquaredDistanceOn

`SignedMaurerDistanceMapImageFilter_SquaredDistanceOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SignedMaurerDistanceMapImageFilter_SquaredDistanceOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMapImageFilter_SquaredDistanceOn`, 'returnType') = 'void'
attr(`SignedMaurerDistanceMapImageFilter_SquaredDistanceOn`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter')
class(`SignedMaurerDistanceMapImageFilter_SquaredDistanceOn`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_SquaredDistanceOn'))

# Start of SignedMaurerDistanceMapImageFilter_SquaredDistanceOff

`SignedMaurerDistanceMapImageFilter_SquaredDistanceOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SignedMaurerDistanceMapImageFilter_SquaredDistanceOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMapImageFilter_SquaredDistanceOff`, 'returnType') = 'void'
attr(`SignedMaurerDistanceMapImageFilter_SquaredDistanceOff`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter')
class(`SignedMaurerDistanceMapImageFilter_SquaredDistanceOff`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_SquaredDistanceOff'))

# Start of SignedMaurerDistanceMapImageFilter_GetSquaredDistance

`SignedMaurerDistanceMapImageFilter_GetSquaredDistance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SignedMaurerDistanceMapImageFilter_GetSquaredDistance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SignedMaurerDistanceMapImageFilter_GetSquaredDistance`, 'returnType') = 'logical'
attr(`SignedMaurerDistanceMapImageFilter_GetSquaredDistance`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter')
class(`SignedMaurerDistanceMapImageFilter_GetSquaredDistance`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_GetSquaredDistance'))

# Start of SignedMaurerDistanceMapImageFilter_SetUseImageSpacing

`SignedMaurerDistanceMapImageFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_SignedMaurerDistanceMapImageFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMapImageFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`SignedMaurerDistanceMapImageFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter', 'logical')
class(`SignedMaurerDistanceMapImageFilter_SetUseImageSpacing`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_SetUseImageSpacing'))

# Start of SignedMaurerDistanceMapImageFilter_UseImageSpacingOn

`SignedMaurerDistanceMapImageFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SignedMaurerDistanceMapImageFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMapImageFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`SignedMaurerDistanceMapImageFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter')
class(`SignedMaurerDistanceMapImageFilter_UseImageSpacingOn`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_UseImageSpacingOn'))

# Start of SignedMaurerDistanceMapImageFilter_UseImageSpacingOff

`SignedMaurerDistanceMapImageFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SignedMaurerDistanceMapImageFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMapImageFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`SignedMaurerDistanceMapImageFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter')
class(`SignedMaurerDistanceMapImageFilter_UseImageSpacingOff`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_UseImageSpacingOff'))

# Start of SignedMaurerDistanceMapImageFilter_GetUseImageSpacing

`SignedMaurerDistanceMapImageFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SignedMaurerDistanceMapImageFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SignedMaurerDistanceMapImageFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`SignedMaurerDistanceMapImageFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter')
class(`SignedMaurerDistanceMapImageFilter_GetUseImageSpacing`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_GetUseImageSpacing'))

# Start of SignedMaurerDistanceMapImageFilter_SetBackgroundValue

`SignedMaurerDistanceMapImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SignedMaurerDistanceMapImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMapImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`SignedMaurerDistanceMapImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter', 'numeric')
class(`SignedMaurerDistanceMapImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_SetBackgroundValue'))

# Start of SignedMaurerDistanceMapImageFilter_GetBackgroundValue

`SignedMaurerDistanceMapImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SignedMaurerDistanceMapImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SignedMaurerDistanceMapImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`SignedMaurerDistanceMapImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter')
class(`SignedMaurerDistanceMapImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_GetBackgroundValue'))

# Start of SignedMaurerDistanceMapImageFilter_GetName

`SignedMaurerDistanceMapImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SignedMaurerDistanceMapImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SignedMaurerDistanceMapImageFilter_GetName`, 'returnType') = 'character'
attr(`SignedMaurerDistanceMapImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter')
class(`SignedMaurerDistanceMapImageFilter_GetName`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_GetName'))

# Start of SignedMaurerDistanceMapImageFilter_ToString

`SignedMaurerDistanceMapImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SignedMaurerDistanceMapImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SignedMaurerDistanceMapImageFilter_ToString`, 'returnType') = 'character'
attr(`SignedMaurerDistanceMapImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter')
class(`SignedMaurerDistanceMapImageFilter_ToString`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_ToString'))

# Start of SignedMaurerDistanceMapImageFilter_Execute

`SignedMaurerDistanceMapImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SignedMaurerDistanceMapImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMapImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`SignedMaurerDistanceMapImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__SignedMaurerDistanceMapImageFilter', '_p_itk__simple__Image')
class(`SignedMaurerDistanceMapImageFilter_Execute`) = c("SWIGFunction", class('SignedMaurerDistanceMapImageFilter_Execute'))

# Start of accessor method for itk::simple::SignedMaurerDistanceMapImageFilter
setMethod('$', '_p_itk__simple__SignedMaurerDistanceMapImageFilter', function(x, name)

{
  accessorFuns = list('SetInsideIsPositive' = SignedMaurerDistanceMapImageFilter_SetInsideIsPositive, 'InsideIsPositiveOn' = SignedMaurerDistanceMapImageFilter_InsideIsPositiveOn, 'InsideIsPositiveOff' = SignedMaurerDistanceMapImageFilter_InsideIsPositiveOff, 'GetInsideIsPositive' = SignedMaurerDistanceMapImageFilter_GetInsideIsPositive, 'SetSquaredDistance' = SignedMaurerDistanceMapImageFilter_SetSquaredDistance, 'SquaredDistanceOn' = SignedMaurerDistanceMapImageFilter_SquaredDistanceOn, 'SquaredDistanceOff' = SignedMaurerDistanceMapImageFilter_SquaredDistanceOff, 'GetSquaredDistance' = SignedMaurerDistanceMapImageFilter_GetSquaredDistance, 'SetUseImageSpacing' = SignedMaurerDistanceMapImageFilter_SetUseImageSpacing, 'UseImageSpacingOn' = SignedMaurerDistanceMapImageFilter_UseImageSpacingOn, 'UseImageSpacingOff' = SignedMaurerDistanceMapImageFilter_UseImageSpacingOff, 'GetUseImageSpacing' = SignedMaurerDistanceMapImageFilter_GetUseImageSpacing, 'SetBackgroundValue' = SignedMaurerDistanceMapImageFilter_SetBackgroundValue, 'GetBackgroundValue' = SignedMaurerDistanceMapImageFilter_GetBackgroundValue, 'GetName' = SignedMaurerDistanceMapImageFilter_GetName, 'ToString' = SignedMaurerDistanceMapImageFilter_ToString, 'Execute' = SignedMaurerDistanceMapImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SignedMaurerDistanceMapImageFilter
# Start of SignedMaurerDistanceMap

`SignedMaurerDistanceMap__SWIG_0` = function(image1, insideIsPositive, squaredDistance, useImageSpacing, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  insideIsPositive = as.logical(insideIsPositive);
  squaredDistance = as.logical(squaredDistance);
  useImageSpacing = as.logical(useImageSpacing);
  
  ;ans = .Call('R_swig_SignedMaurerDistanceMap__SWIG_0', image1, insideIsPositive, squaredDistance, useImageSpacing, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMap__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`SignedMaurerDistanceMap__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'logical', 'logical', 'numeric')
class(`SignedMaurerDistanceMap__SWIG_0`) = c("SWIGFunction", class('SignedMaurerDistanceMap__SWIG_0'))

# Start of SignedMaurerDistanceMap

`SignedMaurerDistanceMap__SWIG_1` = function(image1, insideIsPositive, squaredDistance, useImageSpacing, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  insideIsPositive = as.logical(insideIsPositive);
  squaredDistance = as.logical(squaredDistance);
  useImageSpacing = as.logical(useImageSpacing);
  ;ans = .Call('R_swig_SignedMaurerDistanceMap__SWIG_1', image1, insideIsPositive, squaredDistance, useImageSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMap__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`SignedMaurerDistanceMap__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'logical', 'logical')
class(`SignedMaurerDistanceMap__SWIG_1`) = c("SWIGFunction", class('SignedMaurerDistanceMap__SWIG_1'))

# Start of SignedMaurerDistanceMap

`SignedMaurerDistanceMap__SWIG_2` = function(image1, insideIsPositive, squaredDistance, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  insideIsPositive = as.logical(insideIsPositive);
  squaredDistance = as.logical(squaredDistance);
  ;ans = .Call('R_swig_SignedMaurerDistanceMap__SWIG_2', image1, insideIsPositive, squaredDistance, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMap__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`SignedMaurerDistanceMap__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'logical', 'logical')
class(`SignedMaurerDistanceMap__SWIG_2`) = c("SWIGFunction", class('SignedMaurerDistanceMap__SWIG_2'))

# Start of SignedMaurerDistanceMap

`SignedMaurerDistanceMap__SWIG_3` = function(image1, insideIsPositive, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  insideIsPositive = as.logical(insideIsPositive);
  ;ans = .Call('R_swig_SignedMaurerDistanceMap__SWIG_3', image1, insideIsPositive, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMap__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`SignedMaurerDistanceMap__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`SignedMaurerDistanceMap__SWIG_3`) = c("SWIGFunction", class('SignedMaurerDistanceMap__SWIG_3'))

# Start of SignedMaurerDistanceMap

`SignedMaurerDistanceMap__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SignedMaurerDistanceMap__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SignedMaurerDistanceMap__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`SignedMaurerDistanceMap__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`SignedMaurerDistanceMap__SWIG_4`) = c("SWIGFunction", class('SignedMaurerDistanceMap__SWIG_4'))

`SignedMaurerDistanceMap` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- SignedMaurerDistanceMap__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- SignedMaurerDistanceMap__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- SignedMaurerDistanceMap__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- SignedMaurerDistanceMap__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- SignedMaurerDistanceMap__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SignedMaurerDistanceMap with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_SimilarityIndexImageFilter

`delete_SimilarityIndexImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SimilarityIndexImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SimilarityIndexImageFilter`, 'returnType') = 'void'
attr(`delete_SimilarityIndexImageFilter`, "inputTypes") = c('_p_itk__simple__SimilarityIndexImageFilter')
class(`delete_SimilarityIndexImageFilter`) = c("SWIGFunction", class('delete_SimilarityIndexImageFilter'))

# Start of new_SimilarityIndexImageFilter

`SimilarityIndexImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SimilarityIndexImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SimilarityIndexImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SimilarityIndexImageFilter);
  ans
  
}

attr(`SimilarityIndexImageFilter`, 'returnType') = '_p_itk__simple__SimilarityIndexImageFilter'
class(`SimilarityIndexImageFilter`) = c("SWIGFunction", class('SimilarityIndexImageFilter'))

# Start of SimilarityIndexImageFilter_GetSimilarityIndex

`SimilarityIndexImageFilter_GetSimilarityIndex` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SimilarityIndexImageFilter_GetSimilarityIndex', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SimilarityIndexImageFilter_GetSimilarityIndex`, 'returnType') = 'numeric'
attr(`SimilarityIndexImageFilter_GetSimilarityIndex`, "inputTypes") = c('_p_itk__simple__SimilarityIndexImageFilter')
class(`SimilarityIndexImageFilter_GetSimilarityIndex`) = c("SWIGFunction", class('SimilarityIndexImageFilter_GetSimilarityIndex'))

# Start of SimilarityIndexImageFilter_GetName

`SimilarityIndexImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SimilarityIndexImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SimilarityIndexImageFilter_GetName`, 'returnType') = 'character'
attr(`SimilarityIndexImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SimilarityIndexImageFilter')
class(`SimilarityIndexImageFilter_GetName`) = c("SWIGFunction", class('SimilarityIndexImageFilter_GetName'))

# Start of SimilarityIndexImageFilter_ToString

`SimilarityIndexImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SimilarityIndexImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SimilarityIndexImageFilter_ToString`, 'returnType') = 'character'
attr(`SimilarityIndexImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SimilarityIndexImageFilter')
class(`SimilarityIndexImageFilter_ToString`) = c("SWIGFunction", class('SimilarityIndexImageFilter_ToString'))

# Start of SimilarityIndexImageFilter_Execute

`SimilarityIndexImageFilter_Execute` = function(self, image1, image2)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_SimilarityIndexImageFilter_Execute', self, image1, image2, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SimilarityIndexImageFilter_Execute`, 'returnType') = 'void'
attr(`SimilarityIndexImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__SimilarityIndexImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`SimilarityIndexImageFilter_Execute`) = c("SWIGFunction", class('SimilarityIndexImageFilter_Execute'))

# Start of accessor method for itk::simple::SimilarityIndexImageFilter
setMethod('$', '_p_itk__simple__SimilarityIndexImageFilter', function(x, name)

{
  accessorFuns = list('GetSimilarityIndex' = SimilarityIndexImageFilter_GetSimilarityIndex, 'GetName' = SimilarityIndexImageFilter_GetName, 'ToString' = SimilarityIndexImageFilter_ToString, 'Execute' = SimilarityIndexImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SimilarityIndexImageFilter
# Start of delete_SimpleContourExtractorImageFilter

`delete_SimpleContourExtractorImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SimpleContourExtractorImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SimpleContourExtractorImageFilter`, 'returnType') = 'void'
attr(`delete_SimpleContourExtractorImageFilter`, "inputTypes") = c('_p_itk__simple__SimpleContourExtractorImageFilter')
class(`delete_SimpleContourExtractorImageFilter`) = c("SWIGFunction", class('delete_SimpleContourExtractorImageFilter'))

# Start of new_SimpleContourExtractorImageFilter

`SimpleContourExtractorImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SimpleContourExtractorImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SimpleContourExtractorImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SimpleContourExtractorImageFilter);
  ans
  
}

attr(`SimpleContourExtractorImageFilter`, 'returnType') = '_p_itk__simple__SimpleContourExtractorImageFilter'
class(`SimpleContourExtractorImageFilter`) = c("SWIGFunction", class('SimpleContourExtractorImageFilter'))

# Start of SimpleContourExtractorImageFilter_SetInputForegroundValue

`SimpleContourExtractorImageFilter_SetInputForegroundValue` = function(self, InputForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SimpleContourExtractorImageFilter_SetInputForegroundValue', self, InputForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SimpleContourExtractorImageFilter_SetInputForegroundValue`, 'returnType') = 'void'
attr(`SimpleContourExtractorImageFilter_SetInputForegroundValue`, "inputTypes") = c('_p_itk__simple__SimpleContourExtractorImageFilter', 'numeric')
class(`SimpleContourExtractorImageFilter_SetInputForegroundValue`) = c("SWIGFunction", class('SimpleContourExtractorImageFilter_SetInputForegroundValue'))

# Start of SimpleContourExtractorImageFilter_GetInputForegroundValue

`SimpleContourExtractorImageFilter_GetInputForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SimpleContourExtractorImageFilter_GetInputForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SimpleContourExtractorImageFilter_GetInputForegroundValue`, 'returnType') = 'numeric'
attr(`SimpleContourExtractorImageFilter_GetInputForegroundValue`, "inputTypes") = c('_p_itk__simple__SimpleContourExtractorImageFilter')
class(`SimpleContourExtractorImageFilter_GetInputForegroundValue`) = c("SWIGFunction", class('SimpleContourExtractorImageFilter_GetInputForegroundValue'))

# Start of SimpleContourExtractorImageFilter_SetInputBackgroundValue

`SimpleContourExtractorImageFilter_SetInputBackgroundValue` = function(self, InputBackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SimpleContourExtractorImageFilter_SetInputBackgroundValue', self, InputBackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SimpleContourExtractorImageFilter_SetInputBackgroundValue`, 'returnType') = 'void'
attr(`SimpleContourExtractorImageFilter_SetInputBackgroundValue`, "inputTypes") = c('_p_itk__simple__SimpleContourExtractorImageFilter', 'numeric')
class(`SimpleContourExtractorImageFilter_SetInputBackgroundValue`) = c("SWIGFunction", class('SimpleContourExtractorImageFilter_SetInputBackgroundValue'))

# Start of SimpleContourExtractorImageFilter_GetInputBackgroundValue

`SimpleContourExtractorImageFilter_GetInputBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SimpleContourExtractorImageFilter_GetInputBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SimpleContourExtractorImageFilter_GetInputBackgroundValue`, 'returnType') = 'numeric'
attr(`SimpleContourExtractorImageFilter_GetInputBackgroundValue`, "inputTypes") = c('_p_itk__simple__SimpleContourExtractorImageFilter')
class(`SimpleContourExtractorImageFilter_GetInputBackgroundValue`) = c("SWIGFunction", class('SimpleContourExtractorImageFilter_GetInputBackgroundValue'))

# Start of SimpleContourExtractorImageFilter_SetRadius

`SimpleContourExtractorImageFilter_SetRadius__SWIG_0` = function(self, Radius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Radius = as.integer(Radius);
  ;ans = .Call('R_swig_SimpleContourExtractorImageFilter_SetRadius__SWIG_0', self, Radius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SimpleContourExtractorImageFilter_SetRadius__SWIG_0`, 'returnType') = 'void'
attr(`SimpleContourExtractorImageFilter_SetRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__SimpleContourExtractorImageFilter', 'integer')
class(`SimpleContourExtractorImageFilter_SetRadius__SWIG_0`) = c("SWIGFunction", class('SimpleContourExtractorImageFilter_SetRadius__SWIG_0'))

# Start of SimpleContourExtractorImageFilter_SetRadius

`SimpleContourExtractorImageFilter_SetRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_SimpleContourExtractorImageFilter_SetRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SimpleContourExtractorImageFilter_SetRadius__SWIG_1`, 'returnType') = 'void'
attr(`SimpleContourExtractorImageFilter_SetRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__SimpleContourExtractorImageFilter', 'integer')
class(`SimpleContourExtractorImageFilter_SetRadius__SWIG_1`) = c("SWIGFunction", class('SimpleContourExtractorImageFilter_SetRadius__SWIG_1'))

`SimpleContourExtractorImageFilter_SetRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__SimpleContourExtractorImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- SimpleContourExtractorImageFilter_SetRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__SimpleContourExtractorImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- SimpleContourExtractorImageFilter_SetRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SimpleContourExtractorImageFilter_SetRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of SimpleContourExtractorImageFilter_GetRadius

`SimpleContourExtractorImageFilter_GetRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SimpleContourExtractorImageFilter_GetRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SimpleContourExtractorImageFilter_GetRadius`, 'returnType') = 'integer'
attr(`SimpleContourExtractorImageFilter_GetRadius`, "inputTypes") = c('_p_itk__simple__SimpleContourExtractorImageFilter')
class(`SimpleContourExtractorImageFilter_GetRadius`) = c("SWIGFunction", class('SimpleContourExtractorImageFilter_GetRadius'))

# Start of SimpleContourExtractorImageFilter_SetOutputForegroundValue

`SimpleContourExtractorImageFilter_SetOutputForegroundValue` = function(self, OutputForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SimpleContourExtractorImageFilter_SetOutputForegroundValue', self, OutputForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SimpleContourExtractorImageFilter_SetOutputForegroundValue`, 'returnType') = 'void'
attr(`SimpleContourExtractorImageFilter_SetOutputForegroundValue`, "inputTypes") = c('_p_itk__simple__SimpleContourExtractorImageFilter', 'numeric')
class(`SimpleContourExtractorImageFilter_SetOutputForegroundValue`) = c("SWIGFunction", class('SimpleContourExtractorImageFilter_SetOutputForegroundValue'))

# Start of SimpleContourExtractorImageFilter_GetOutputForegroundValue

`SimpleContourExtractorImageFilter_GetOutputForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SimpleContourExtractorImageFilter_GetOutputForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SimpleContourExtractorImageFilter_GetOutputForegroundValue`, 'returnType') = 'numeric'
attr(`SimpleContourExtractorImageFilter_GetOutputForegroundValue`, "inputTypes") = c('_p_itk__simple__SimpleContourExtractorImageFilter')
class(`SimpleContourExtractorImageFilter_GetOutputForegroundValue`) = c("SWIGFunction", class('SimpleContourExtractorImageFilter_GetOutputForegroundValue'))

# Start of SimpleContourExtractorImageFilter_SetOutputBackgroundValue

`SimpleContourExtractorImageFilter_SetOutputBackgroundValue` = function(self, OutputBackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SimpleContourExtractorImageFilter_SetOutputBackgroundValue', self, OutputBackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SimpleContourExtractorImageFilter_SetOutputBackgroundValue`, 'returnType') = 'void'
attr(`SimpleContourExtractorImageFilter_SetOutputBackgroundValue`, "inputTypes") = c('_p_itk__simple__SimpleContourExtractorImageFilter', 'numeric')
class(`SimpleContourExtractorImageFilter_SetOutputBackgroundValue`) = c("SWIGFunction", class('SimpleContourExtractorImageFilter_SetOutputBackgroundValue'))

# Start of SimpleContourExtractorImageFilter_GetOutputBackgroundValue

`SimpleContourExtractorImageFilter_GetOutputBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SimpleContourExtractorImageFilter_GetOutputBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SimpleContourExtractorImageFilter_GetOutputBackgroundValue`, 'returnType') = 'numeric'
attr(`SimpleContourExtractorImageFilter_GetOutputBackgroundValue`, "inputTypes") = c('_p_itk__simple__SimpleContourExtractorImageFilter')
class(`SimpleContourExtractorImageFilter_GetOutputBackgroundValue`) = c("SWIGFunction", class('SimpleContourExtractorImageFilter_GetOutputBackgroundValue'))

# Start of SimpleContourExtractorImageFilter_GetName

`SimpleContourExtractorImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SimpleContourExtractorImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SimpleContourExtractorImageFilter_GetName`, 'returnType') = 'character'
attr(`SimpleContourExtractorImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SimpleContourExtractorImageFilter')
class(`SimpleContourExtractorImageFilter_GetName`) = c("SWIGFunction", class('SimpleContourExtractorImageFilter_GetName'))

# Start of SimpleContourExtractorImageFilter_ToString

`SimpleContourExtractorImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SimpleContourExtractorImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SimpleContourExtractorImageFilter_ToString`, 'returnType') = 'character'
attr(`SimpleContourExtractorImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SimpleContourExtractorImageFilter')
class(`SimpleContourExtractorImageFilter_ToString`) = c("SWIGFunction", class('SimpleContourExtractorImageFilter_ToString'))

# Start of SimpleContourExtractorImageFilter_Execute

`SimpleContourExtractorImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SimpleContourExtractorImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SimpleContourExtractorImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`SimpleContourExtractorImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__SimpleContourExtractorImageFilter', '_p_itk__simple__Image')
class(`SimpleContourExtractorImageFilter_Execute`) = c("SWIGFunction", class('SimpleContourExtractorImageFilter_Execute'))

# Start of accessor method for itk::simple::SimpleContourExtractorImageFilter
setMethod('$', '_p_itk__simple__SimpleContourExtractorImageFilter', function(x, name)

{
  accessorFuns = list('SetInputForegroundValue' = SimpleContourExtractorImageFilter_SetInputForegroundValue, 'GetInputForegroundValue' = SimpleContourExtractorImageFilter_GetInputForegroundValue, 'SetInputBackgroundValue' = SimpleContourExtractorImageFilter_SetInputBackgroundValue, 'GetInputBackgroundValue' = SimpleContourExtractorImageFilter_GetInputBackgroundValue, 'SetRadius' = SimpleContourExtractorImageFilter_SetRadius, 'GetRadius' = SimpleContourExtractorImageFilter_GetRadius, 'SetOutputForegroundValue' = SimpleContourExtractorImageFilter_SetOutputForegroundValue, 'GetOutputForegroundValue' = SimpleContourExtractorImageFilter_GetOutputForegroundValue, 'SetOutputBackgroundValue' = SimpleContourExtractorImageFilter_SetOutputBackgroundValue, 'GetOutputBackgroundValue' = SimpleContourExtractorImageFilter_GetOutputBackgroundValue, 'GetName' = SimpleContourExtractorImageFilter_GetName, 'ToString' = SimpleContourExtractorImageFilter_ToString, 'Execute' = SimpleContourExtractorImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SimpleContourExtractorImageFilter
# Start of SimpleContourExtractor

`SimpleContourExtractor__SWIG_0` = function(image1, inputForegroundValue, inputBackgroundValue, radius, outputForegroundValue, outputBackgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  radius = as.integer(radius);
  
  
  ;ans = .Call('R_swig_SimpleContourExtractor__SWIG_0', image1, inputForegroundValue, inputBackgroundValue, radius, outputForegroundValue, outputBackgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SimpleContourExtractor__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`SimpleContourExtractor__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'integer', 'numeric', 'numeric')
class(`SimpleContourExtractor__SWIG_0`) = c("SWIGFunction", class('SimpleContourExtractor__SWIG_0'))

# Start of SimpleContourExtractor

`SimpleContourExtractor__SWIG_1` = function(image1, inputForegroundValue, inputBackgroundValue, radius, outputForegroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  radius = as.integer(radius);
  
  ;ans = .Call('R_swig_SimpleContourExtractor__SWIG_1', image1, inputForegroundValue, inputBackgroundValue, radius, outputForegroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SimpleContourExtractor__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`SimpleContourExtractor__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'integer', 'numeric')
class(`SimpleContourExtractor__SWIG_1`) = c("SWIGFunction", class('SimpleContourExtractor__SWIG_1'))

# Start of SimpleContourExtractor

`SimpleContourExtractor__SWIG_2` = function(image1, inputForegroundValue, inputBackgroundValue, radius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  radius = as.integer(radius);
  ;ans = .Call('R_swig_SimpleContourExtractor__SWIG_2', image1, inputForegroundValue, inputBackgroundValue, radius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SimpleContourExtractor__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`SimpleContourExtractor__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'integer')
class(`SimpleContourExtractor__SWIG_2`) = c("SWIGFunction", class('SimpleContourExtractor__SWIG_2'))

# Start of SimpleContourExtractor

`SimpleContourExtractor__SWIG_3` = function(image1, inputForegroundValue, inputBackgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_SimpleContourExtractor__SWIG_3', image1, inputForegroundValue, inputBackgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SimpleContourExtractor__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`SimpleContourExtractor__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`SimpleContourExtractor__SWIG_3`) = c("SWIGFunction", class('SimpleContourExtractor__SWIG_3'))

# Start of SimpleContourExtractor

`SimpleContourExtractor__SWIG_4` = function(image1, inputForegroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_SimpleContourExtractor__SWIG_4', image1, inputForegroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SimpleContourExtractor__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`SimpleContourExtractor__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`SimpleContourExtractor__SWIG_4`) = c("SWIGFunction", class('SimpleContourExtractor__SWIG_4'))

# Start of SimpleContourExtractor

`SimpleContourExtractor__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SimpleContourExtractor__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SimpleContourExtractor__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`SimpleContourExtractor__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`SimpleContourExtractor__SWIG_5`) = c("SWIGFunction", class('SimpleContourExtractor__SWIG_5'))

`SimpleContourExtractor` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- SimpleContourExtractor__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- SimpleContourExtractor__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- SimpleContourExtractor__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) )) {
      f <- SimpleContourExtractor__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- SimpleContourExtractor__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- SimpleContourExtractor__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SimpleContourExtractor with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_SinImageFilter

`delete_SinImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SinImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SinImageFilter`, 'returnType') = 'void'
attr(`delete_SinImageFilter`, "inputTypes") = c('_p_itk__simple__SinImageFilter')
class(`delete_SinImageFilter`) = c("SWIGFunction", class('delete_SinImageFilter'))

# Start of new_SinImageFilter

`SinImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SinImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SinImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SinImageFilter);
  ans
  
}

attr(`SinImageFilter`, 'returnType') = '_p_itk__simple__SinImageFilter'
class(`SinImageFilter`) = c("SWIGFunction", class('SinImageFilter'))

# Start of SinImageFilter_GetName

`SinImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SinImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SinImageFilter_GetName`, 'returnType') = 'character'
attr(`SinImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SinImageFilter')
class(`SinImageFilter_GetName`) = c("SWIGFunction", class('SinImageFilter_GetName'))

# Start of SinImageFilter_ToString

`SinImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SinImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SinImageFilter_ToString`, 'returnType') = 'character'
attr(`SinImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SinImageFilter')
class(`SinImageFilter_ToString`) = c("SWIGFunction", class('SinImageFilter_ToString'))

# Start of SinImageFilter_Execute

`SinImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SinImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SinImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`SinImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__SinImageFilter', '_p_itk__simple__Image')
class(`SinImageFilter_Execute`) = c("SWIGFunction", class('SinImageFilter_Execute'))

# Start of accessor method for itk::simple::SinImageFilter
setMethod('$', '_p_itk__simple__SinImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = SinImageFilter_GetName, 'ToString' = SinImageFilter_ToString, 'Execute' = SinImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SinImageFilter
# Start of Sin

`Sin` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Sin', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Sin`, 'returnType') = '_p_itk__simple__Image'
attr(`Sin`, "inputTypes") = c('_p_itk__simple__Image')
class(`Sin`) = c("SWIGFunction", class('Sin'))

# Start of delete_SliceImageFilter

`delete_SliceImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SliceImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SliceImageFilter`, 'returnType') = 'void'
attr(`delete_SliceImageFilter`, "inputTypes") = c('_p_itk__simple__SliceImageFilter')
class(`delete_SliceImageFilter`) = c("SWIGFunction", class('delete_SliceImageFilter'))

# Start of new_SliceImageFilter

`SliceImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SliceImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SliceImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SliceImageFilter);
  ans
  
}

attr(`SliceImageFilter`, 'returnType') = '_p_itk__simple__SliceImageFilter'
class(`SliceImageFilter`) = c("SWIGFunction", class('SliceImageFilter'))

# Start of SliceImageFilter_SetStart

`SliceImageFilter_SetStart` = function(self, Start)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Start = as.integer(Start);
  ;ans = .Call('R_swig_SliceImageFilter_SetStart', self, Start, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SliceImageFilter_SetStart`, 'returnType') = 'void'
attr(`SliceImageFilter_SetStart`, "inputTypes") = c('_p_itk__simple__SliceImageFilter', 'integer')
class(`SliceImageFilter_SetStart`) = c("SWIGFunction", class('SliceImageFilter_SetStart'))

# Start of SliceImageFilter_GetStart

`SliceImageFilter_GetStart` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SliceImageFilter_GetStart', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SliceImageFilter_GetStart`, 'returnType') = 'integer'
attr(`SliceImageFilter_GetStart`, "inputTypes") = c('_p_itk__simple__SliceImageFilter')
class(`SliceImageFilter_GetStart`) = c("SWIGFunction", class('SliceImageFilter_GetStart'))

# Start of SliceImageFilter_SetStop

`SliceImageFilter_SetStop` = function(self, Stop)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Stop = as.integer(Stop);
  ;ans = .Call('R_swig_SliceImageFilter_SetStop', self, Stop, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SliceImageFilter_SetStop`, 'returnType') = 'void'
attr(`SliceImageFilter_SetStop`, "inputTypes") = c('_p_itk__simple__SliceImageFilter', 'integer')
class(`SliceImageFilter_SetStop`) = c("SWIGFunction", class('SliceImageFilter_SetStop'))

# Start of SliceImageFilter_GetStop

`SliceImageFilter_GetStop` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SliceImageFilter_GetStop', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SliceImageFilter_GetStop`, 'returnType') = 'integer'
attr(`SliceImageFilter_GetStop`, "inputTypes") = c('_p_itk__simple__SliceImageFilter')
class(`SliceImageFilter_GetStop`) = c("SWIGFunction", class('SliceImageFilter_GetStop'))

# Start of SliceImageFilter_SetStep

`SliceImageFilter_SetStep__SWIG_0` = function(self, Step)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Step = as.integer(Step);
  ;ans = .Call('R_swig_SliceImageFilter_SetStep__SWIG_0', self, Step, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SliceImageFilter_SetStep__SWIG_0`, 'returnType') = 'void'
attr(`SliceImageFilter_SetStep__SWIG_0`, "inputTypes") = c('_p_itk__simple__SliceImageFilter', 'integer')
class(`SliceImageFilter_SetStep__SWIG_0`) = c("SWIGFunction", class('SliceImageFilter_SetStep__SWIG_0'))

# Start of SliceImageFilter_SetStep

`SliceImageFilter_SetStep__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_SliceImageFilter_SetStep__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SliceImageFilter_SetStep__SWIG_1`, 'returnType') = 'void'
attr(`SliceImageFilter_SetStep__SWIG_1`, "inputTypes") = c('_p_itk__simple__SliceImageFilter', 'integer')
class(`SliceImageFilter_SetStep__SWIG_1`) = c("SWIGFunction", class('SliceImageFilter_SetStep__SWIG_1'))

`SliceImageFilter_SetStep` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__SliceImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- SliceImageFilter_SetStep__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__SliceImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- SliceImageFilter_SetStep__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SliceImageFilter_SetStep with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of SliceImageFilter_GetStep

`SliceImageFilter_GetStep` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SliceImageFilter_GetStep', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SliceImageFilter_GetStep`, 'returnType') = 'integer'
attr(`SliceImageFilter_GetStep`, "inputTypes") = c('_p_itk__simple__SliceImageFilter')
class(`SliceImageFilter_GetStep`) = c("SWIGFunction", class('SliceImageFilter_GetStep'))

# Start of SliceImageFilter_GetName

`SliceImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SliceImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SliceImageFilter_GetName`, 'returnType') = 'character'
attr(`SliceImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SliceImageFilter')
class(`SliceImageFilter_GetName`) = c("SWIGFunction", class('SliceImageFilter_GetName'))

# Start of SliceImageFilter_ToString

`SliceImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SliceImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SliceImageFilter_ToString`, 'returnType') = 'character'
attr(`SliceImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SliceImageFilter')
class(`SliceImageFilter_ToString`) = c("SWIGFunction", class('SliceImageFilter_ToString'))

# Start of SliceImageFilter_Execute

`SliceImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SliceImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SliceImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`SliceImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__SliceImageFilter', '_p_itk__simple__Image')
class(`SliceImageFilter_Execute`) = c("SWIGFunction", class('SliceImageFilter_Execute'))

# Start of accessor method for itk::simple::SliceImageFilter
setMethod('$', '_p_itk__simple__SliceImageFilter', function(x, name)

{
  accessorFuns = list('SetStart' = SliceImageFilter_SetStart, 'GetStart' = SliceImageFilter_GetStart, 'SetStop' = SliceImageFilter_SetStop, 'GetStop' = SliceImageFilter_GetStop, 'SetStep' = SliceImageFilter_SetStep, 'GetStep' = SliceImageFilter_GetStep, 'GetName' = SliceImageFilter_GetName, 'ToString' = SliceImageFilter_ToString, 'Execute' = SliceImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SliceImageFilter
# Start of Slice

`Slice__SWIG_0` = function(image1, start, stop, step, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  start = as.integer(start);
  stop = as.integer(stop);
  step = as.integer(step);
  ;ans = .Call('R_swig_Slice__SWIG_0', image1, start, stop, step, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Slice__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Slice__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`Slice__SWIG_0`) = c("SWIGFunction", class('Slice__SWIG_0'))

# Start of Slice

`Slice__SWIG_1` = function(image1, start, stop, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  start = as.integer(start);
  stop = as.integer(stop);
  ;ans = .Call('R_swig_Slice__SWIG_1', image1, start, stop, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Slice__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Slice__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`Slice__SWIG_1`) = c("SWIGFunction", class('Slice__SWIG_1'))

# Start of Slice

`Slice__SWIG_2` = function(image1, start, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  start = as.integer(start);
  ;ans = .Call('R_swig_Slice__SWIG_2', image1, start, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Slice__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Slice__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Slice__SWIG_2`) = c("SWIGFunction", class('Slice__SWIG_2'))

# Start of Slice

`Slice__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Slice__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Slice__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Slice__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`Slice__SWIG_3`) = c("SWIGFunction", class('Slice__SWIG_3'))

`Slice` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Slice__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- Slice__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- Slice__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) )) {
      f <- Slice__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Slice with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_SmoothingRecursiveGaussianImageFilter

`delete_SmoothingRecursiveGaussianImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SmoothingRecursiveGaussianImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SmoothingRecursiveGaussianImageFilter`, 'returnType') = 'void'
attr(`delete_SmoothingRecursiveGaussianImageFilter`, "inputTypes") = c('_p_itk__simple__SmoothingRecursiveGaussianImageFilter')
class(`delete_SmoothingRecursiveGaussianImageFilter`) = c("SWIGFunction", class('delete_SmoothingRecursiveGaussianImageFilter'))

# Start of new_SmoothingRecursiveGaussianImageFilter

`SmoothingRecursiveGaussianImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SmoothingRecursiveGaussianImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SmoothingRecursiveGaussianImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SmoothingRecursiveGaussianImageFilter);
  ans
  
}

attr(`SmoothingRecursiveGaussianImageFilter`, 'returnType') = '_p_itk__simple__SmoothingRecursiveGaussianImageFilter'
class(`SmoothingRecursiveGaussianImageFilter`) = c("SWIGFunction", class('SmoothingRecursiveGaussianImageFilter'))

# Start of SmoothingRecursiveGaussianImageFilter_SetSigma

`SmoothingRecursiveGaussianImageFilter_SetSigma__SWIG_0` = function(self, Sigma)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Sigma = as.numeric(Sigma);
  ;ans = .Call('R_swig_SmoothingRecursiveGaussianImageFilter_SetSigma__SWIG_0', self, Sigma, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SmoothingRecursiveGaussianImageFilter_SetSigma__SWIG_0`, 'returnType') = 'void'
attr(`SmoothingRecursiveGaussianImageFilter_SetSigma__SWIG_0`, "inputTypes") = c('_p_itk__simple__SmoothingRecursiveGaussianImageFilter', 'numeric')
class(`SmoothingRecursiveGaussianImageFilter_SetSigma__SWIG_0`) = c("SWIGFunction", class('SmoothingRecursiveGaussianImageFilter_SetSigma__SWIG_0'))

# Start of SmoothingRecursiveGaussianImageFilter_SetSigma

`SmoothingRecursiveGaussianImageFilter_SetSigma__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SmoothingRecursiveGaussianImageFilter_SetSigma__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SmoothingRecursiveGaussianImageFilter_SetSigma__SWIG_1`, 'returnType') = 'void'
attr(`SmoothingRecursiveGaussianImageFilter_SetSigma__SWIG_1`, "inputTypes") = c('_p_itk__simple__SmoothingRecursiveGaussianImageFilter', 'numeric')
class(`SmoothingRecursiveGaussianImageFilter_SetSigma__SWIG_1`) = c("SWIGFunction", class('SmoothingRecursiveGaussianImageFilter_SetSigma__SWIG_1'))

`SmoothingRecursiveGaussianImageFilter_SetSigma` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__SmoothingRecursiveGaussianImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- SmoothingRecursiveGaussianImageFilter_SetSigma__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__SmoothingRecursiveGaussianImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- SmoothingRecursiveGaussianImageFilter_SetSigma__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SmoothingRecursiveGaussianImageFilter_SetSigma with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of SmoothingRecursiveGaussianImageFilter_GetSigma

`SmoothingRecursiveGaussianImageFilter_GetSigma` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SmoothingRecursiveGaussianImageFilter_GetSigma', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SmoothingRecursiveGaussianImageFilter_GetSigma`, 'returnType') = 'numeric'
attr(`SmoothingRecursiveGaussianImageFilter_GetSigma`, "inputTypes") = c('_p_itk__simple__SmoothingRecursiveGaussianImageFilter')
class(`SmoothingRecursiveGaussianImageFilter_GetSigma`) = c("SWIGFunction", class('SmoothingRecursiveGaussianImageFilter_GetSigma'))

# Start of SmoothingRecursiveGaussianImageFilter_SetNormalizeAcrossScale

`SmoothingRecursiveGaussianImageFilter_SetNormalizeAcrossScale` = function(self, NormalizeAcrossScale)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NormalizeAcrossScale = as.logical(NormalizeAcrossScale);
  ;ans = .Call('R_swig_SmoothingRecursiveGaussianImageFilter_SetNormalizeAcrossScale', self, NormalizeAcrossScale, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SmoothingRecursiveGaussianImageFilter_SetNormalizeAcrossScale`, 'returnType') = 'void'
attr(`SmoothingRecursiveGaussianImageFilter_SetNormalizeAcrossScale`, "inputTypes") = c('_p_itk__simple__SmoothingRecursiveGaussianImageFilter', 'logical')
class(`SmoothingRecursiveGaussianImageFilter_SetNormalizeAcrossScale`) = c("SWIGFunction", class('SmoothingRecursiveGaussianImageFilter_SetNormalizeAcrossScale'))

# Start of SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOn

`SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOn`, 'returnType') = 'void'
attr(`SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOn`, "inputTypes") = c('_p_itk__simple__SmoothingRecursiveGaussianImageFilter')
class(`SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOn`) = c("SWIGFunction", class('SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOn'))

# Start of SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOff

`SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOff`, 'returnType') = 'void'
attr(`SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOff`, "inputTypes") = c('_p_itk__simple__SmoothingRecursiveGaussianImageFilter')
class(`SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOff`) = c("SWIGFunction", class('SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOff'))

# Start of SmoothingRecursiveGaussianImageFilter_GetNormalizeAcrossScale

`SmoothingRecursiveGaussianImageFilter_GetNormalizeAcrossScale` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SmoothingRecursiveGaussianImageFilter_GetNormalizeAcrossScale', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SmoothingRecursiveGaussianImageFilter_GetNormalizeAcrossScale`, 'returnType') = 'logical'
attr(`SmoothingRecursiveGaussianImageFilter_GetNormalizeAcrossScale`, "inputTypes") = c('_p_itk__simple__SmoothingRecursiveGaussianImageFilter')
class(`SmoothingRecursiveGaussianImageFilter_GetNormalizeAcrossScale`) = c("SWIGFunction", class('SmoothingRecursiveGaussianImageFilter_GetNormalizeAcrossScale'))

# Start of SmoothingRecursiveGaussianImageFilter_GetName

`SmoothingRecursiveGaussianImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SmoothingRecursiveGaussianImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SmoothingRecursiveGaussianImageFilter_GetName`, 'returnType') = 'character'
attr(`SmoothingRecursiveGaussianImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SmoothingRecursiveGaussianImageFilter')
class(`SmoothingRecursiveGaussianImageFilter_GetName`) = c("SWIGFunction", class('SmoothingRecursiveGaussianImageFilter_GetName'))

# Start of SmoothingRecursiveGaussianImageFilter_ToString

`SmoothingRecursiveGaussianImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SmoothingRecursiveGaussianImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SmoothingRecursiveGaussianImageFilter_ToString`, 'returnType') = 'character'
attr(`SmoothingRecursiveGaussianImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SmoothingRecursiveGaussianImageFilter')
class(`SmoothingRecursiveGaussianImageFilter_ToString`) = c("SWIGFunction", class('SmoothingRecursiveGaussianImageFilter_ToString'))

# Start of SmoothingRecursiveGaussianImageFilter_Execute

`SmoothingRecursiveGaussianImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SmoothingRecursiveGaussianImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SmoothingRecursiveGaussianImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`SmoothingRecursiveGaussianImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__SmoothingRecursiveGaussianImageFilter', '_p_itk__simple__Image')
class(`SmoothingRecursiveGaussianImageFilter_Execute`) = c("SWIGFunction", class('SmoothingRecursiveGaussianImageFilter_Execute'))

# Start of accessor method for itk::simple::SmoothingRecursiveGaussianImageFilter
setMethod('$', '_p_itk__simple__SmoothingRecursiveGaussianImageFilter', function(x, name)

{
  accessorFuns = list('SetSigma' = SmoothingRecursiveGaussianImageFilter_SetSigma, 'GetSigma' = SmoothingRecursiveGaussianImageFilter_GetSigma, 'SetNormalizeAcrossScale' = SmoothingRecursiveGaussianImageFilter_SetNormalizeAcrossScale, 'NormalizeAcrossScaleOn' = SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOn, 'NormalizeAcrossScaleOff' = SmoothingRecursiveGaussianImageFilter_NormalizeAcrossScaleOff, 'GetNormalizeAcrossScale' = SmoothingRecursiveGaussianImageFilter_GetNormalizeAcrossScale, 'GetName' = SmoothingRecursiveGaussianImageFilter_GetName, 'ToString' = SmoothingRecursiveGaussianImageFilter_ToString, 'Execute' = SmoothingRecursiveGaussianImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SmoothingRecursiveGaussianImageFilter
# Start of SmoothingRecursiveGaussian

`SmoothingRecursiveGaussian__SWIG_2` = function(image1, sigma, normalizeAcrossScale, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  sigma = as.numeric(sigma);
  normalizeAcrossScale = as.logical(normalizeAcrossScale);
  ;ans = .Call('R_swig_SmoothingRecursiveGaussian__SWIG_2', image1, sigma, normalizeAcrossScale, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SmoothingRecursiveGaussian__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`SmoothingRecursiveGaussian__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`SmoothingRecursiveGaussian__SWIG_2`) = c("SWIGFunction", class('SmoothingRecursiveGaussian__SWIG_2'))

# Start of SmoothingRecursiveGaussian

`SmoothingRecursiveGaussian__SWIG_3` = function(image1, sigma, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  sigma = as.numeric(sigma);
  ;ans = .Call('R_swig_SmoothingRecursiveGaussian__SWIG_3', image1, sigma, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SmoothingRecursiveGaussian__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`SmoothingRecursiveGaussian__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`SmoothingRecursiveGaussian__SWIG_3`) = c("SWIGFunction", class('SmoothingRecursiveGaussian__SWIG_3'))

# Start of SmoothingRecursiveGaussian

`SmoothingRecursiveGaussian__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SmoothingRecursiveGaussian__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SmoothingRecursiveGaussian__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`SmoothingRecursiveGaussian__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`SmoothingRecursiveGaussian__SWIG_4`) = c("SWIGFunction", class('SmoothingRecursiveGaussian__SWIG_4'))

`SmoothingRecursiveGaussian` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- SmoothingRecursiveGaussian__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- SmoothingRecursiveGaussian__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) )) {
      f <- SmoothingRecursiveGaussian__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- SmoothingRecursiveGaussian__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- SmoothingRecursiveGaussian__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SmoothingRecursiveGaussian with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_SobelEdgeDetectionImageFilter

`delete_SobelEdgeDetectionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SobelEdgeDetectionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SobelEdgeDetectionImageFilter`, 'returnType') = 'void'
attr(`delete_SobelEdgeDetectionImageFilter`, "inputTypes") = c('_p_itk__simple__SobelEdgeDetectionImageFilter')
class(`delete_SobelEdgeDetectionImageFilter`) = c("SWIGFunction", class('delete_SobelEdgeDetectionImageFilter'))

# Start of new_SobelEdgeDetectionImageFilter

`SobelEdgeDetectionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SobelEdgeDetectionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SobelEdgeDetectionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SobelEdgeDetectionImageFilter);
  ans
  
}

attr(`SobelEdgeDetectionImageFilter`, 'returnType') = '_p_itk__simple__SobelEdgeDetectionImageFilter'
class(`SobelEdgeDetectionImageFilter`) = c("SWIGFunction", class('SobelEdgeDetectionImageFilter'))

# Start of SobelEdgeDetectionImageFilter_GetName

`SobelEdgeDetectionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SobelEdgeDetectionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SobelEdgeDetectionImageFilter_GetName`, 'returnType') = 'character'
attr(`SobelEdgeDetectionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SobelEdgeDetectionImageFilter')
class(`SobelEdgeDetectionImageFilter_GetName`) = c("SWIGFunction", class('SobelEdgeDetectionImageFilter_GetName'))

# Start of SobelEdgeDetectionImageFilter_ToString

`SobelEdgeDetectionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SobelEdgeDetectionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SobelEdgeDetectionImageFilter_ToString`, 'returnType') = 'character'
attr(`SobelEdgeDetectionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SobelEdgeDetectionImageFilter')
class(`SobelEdgeDetectionImageFilter_ToString`) = c("SWIGFunction", class('SobelEdgeDetectionImageFilter_ToString'))

# Start of SobelEdgeDetectionImageFilter_Execute

`SobelEdgeDetectionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SobelEdgeDetectionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SobelEdgeDetectionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`SobelEdgeDetectionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__SobelEdgeDetectionImageFilter', '_p_itk__simple__Image')
class(`SobelEdgeDetectionImageFilter_Execute`) = c("SWIGFunction", class('SobelEdgeDetectionImageFilter_Execute'))

# Start of accessor method for itk::simple::SobelEdgeDetectionImageFilter
setMethod('$', '_p_itk__simple__SobelEdgeDetectionImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = SobelEdgeDetectionImageFilter_GetName, 'ToString' = SobelEdgeDetectionImageFilter_ToString, 'Execute' = SobelEdgeDetectionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SobelEdgeDetectionImageFilter
# Start of SobelEdgeDetection

`SobelEdgeDetection` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SobelEdgeDetection', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SobelEdgeDetection`, 'returnType') = '_p_itk__simple__Image'
attr(`SobelEdgeDetection`, "inputTypes") = c('_p_itk__simple__Image')
class(`SobelEdgeDetection`) = c("SWIGFunction", class('SobelEdgeDetection'))

# Start of delete_SpeckleNoiseImageFilter

`delete_SpeckleNoiseImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SpeckleNoiseImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SpeckleNoiseImageFilter`, 'returnType') = 'void'
attr(`delete_SpeckleNoiseImageFilter`, "inputTypes") = c('_p_itk__simple__SpeckleNoiseImageFilter')
class(`delete_SpeckleNoiseImageFilter`) = c("SWIGFunction", class('delete_SpeckleNoiseImageFilter'))

# Start of new_SpeckleNoiseImageFilter

`SpeckleNoiseImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SpeckleNoiseImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SpeckleNoiseImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SpeckleNoiseImageFilter);
  ans
  
}

attr(`SpeckleNoiseImageFilter`, 'returnType') = '_p_itk__simple__SpeckleNoiseImageFilter'
class(`SpeckleNoiseImageFilter`) = c("SWIGFunction", class('SpeckleNoiseImageFilter'))

# Start of SpeckleNoiseImageFilter_SetStandardDeviation

`SpeckleNoiseImageFilter_SetStandardDeviation` = function(self, StandardDeviation)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SpeckleNoiseImageFilter_SetStandardDeviation', self, StandardDeviation, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SpeckleNoiseImageFilter_SetStandardDeviation`, 'returnType') = 'void'
attr(`SpeckleNoiseImageFilter_SetStandardDeviation`, "inputTypes") = c('_p_itk__simple__SpeckleNoiseImageFilter', 'numeric')
class(`SpeckleNoiseImageFilter_SetStandardDeviation`) = c("SWIGFunction", class('SpeckleNoiseImageFilter_SetStandardDeviation'))

# Start of SpeckleNoiseImageFilter_GetStandardDeviation

`SpeckleNoiseImageFilter_GetStandardDeviation` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SpeckleNoiseImageFilter_GetStandardDeviation', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SpeckleNoiseImageFilter_GetStandardDeviation`, 'returnType') = 'numeric'
attr(`SpeckleNoiseImageFilter_GetStandardDeviation`, "inputTypes") = c('_p_itk__simple__SpeckleNoiseImageFilter')
class(`SpeckleNoiseImageFilter_GetStandardDeviation`) = c("SWIGFunction", class('SpeckleNoiseImageFilter_GetStandardDeviation'))

# Start of SpeckleNoiseImageFilter_SetSeed

`SpeckleNoiseImageFilter_SetSeed` = function(self, Seed)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Seed = as.integer(Seed);
  
  if(length(Seed) > 1) {
    warning("using only the first element of Seed");
  };
  
  ;ans = .Call('R_swig_SpeckleNoiseImageFilter_SetSeed', self, Seed, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SpeckleNoiseImageFilter_SetSeed`, 'returnType') = 'void'
attr(`SpeckleNoiseImageFilter_SetSeed`, "inputTypes") = c('_p_itk__simple__SpeckleNoiseImageFilter', 'integer')
class(`SpeckleNoiseImageFilter_SetSeed`) = c("SWIGFunction", class('SpeckleNoiseImageFilter_SetSeed'))

# Start of SpeckleNoiseImageFilter_GetSeed

`SpeckleNoiseImageFilter_GetSeed` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SpeckleNoiseImageFilter_GetSeed', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SpeckleNoiseImageFilter_GetSeed`, 'returnType') = 'integer'
attr(`SpeckleNoiseImageFilter_GetSeed`, "inputTypes") = c('_p_itk__simple__SpeckleNoiseImageFilter')
class(`SpeckleNoiseImageFilter_GetSeed`) = c("SWIGFunction", class('SpeckleNoiseImageFilter_GetSeed'))

# Start of SpeckleNoiseImageFilter_GetName

`SpeckleNoiseImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SpeckleNoiseImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SpeckleNoiseImageFilter_GetName`, 'returnType') = 'character'
attr(`SpeckleNoiseImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SpeckleNoiseImageFilter')
class(`SpeckleNoiseImageFilter_GetName`) = c("SWIGFunction", class('SpeckleNoiseImageFilter_GetName'))

# Start of SpeckleNoiseImageFilter_ToString

`SpeckleNoiseImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SpeckleNoiseImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SpeckleNoiseImageFilter_ToString`, 'returnType') = 'character'
attr(`SpeckleNoiseImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SpeckleNoiseImageFilter')
class(`SpeckleNoiseImageFilter_ToString`) = c("SWIGFunction", class('SpeckleNoiseImageFilter_ToString'))

# Start of SpeckleNoiseImageFilter_Execute

`SpeckleNoiseImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SpeckleNoiseImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SpeckleNoiseImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`SpeckleNoiseImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__SpeckleNoiseImageFilter', '_p_itk__simple__Image')
class(`SpeckleNoiseImageFilter_Execute`) = c("SWIGFunction", class('SpeckleNoiseImageFilter_Execute'))

# Start of accessor method for itk::simple::SpeckleNoiseImageFilter
setMethod('$', '_p_itk__simple__SpeckleNoiseImageFilter', function(x, name)

{
  accessorFuns = list('SetStandardDeviation' = SpeckleNoiseImageFilter_SetStandardDeviation, 'GetStandardDeviation' = SpeckleNoiseImageFilter_GetStandardDeviation, 'SetSeed' = SpeckleNoiseImageFilter_SetSeed, 'GetSeed' = SpeckleNoiseImageFilter_GetSeed, 'GetName' = SpeckleNoiseImageFilter_GetName, 'ToString' = SpeckleNoiseImageFilter_ToString, 'Execute' = SpeckleNoiseImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SpeckleNoiseImageFilter
# Start of SpeckleNoise

`SpeckleNoise__SWIG_0` = function(image1, standardDeviation, seed, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  seed = as.integer(seed);
  
  if(length(seed) > 1) {
    warning("using only the first element of seed");
  };
  
  ;ans = .Call('R_swig_SpeckleNoise__SWIG_0', image1, standardDeviation, seed, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SpeckleNoise__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`SpeckleNoise__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`SpeckleNoise__SWIG_0`) = c("SWIGFunction", class('SpeckleNoise__SWIG_0'))

# Start of SpeckleNoise

`SpeckleNoise__SWIG_1` = function(image1, standardDeviation, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_SpeckleNoise__SWIG_1', image1, standardDeviation, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SpeckleNoise__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`SpeckleNoise__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`SpeckleNoise__SWIG_1`) = c("SWIGFunction", class('SpeckleNoise__SWIG_1'))

# Start of SpeckleNoise

`SpeckleNoise__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SpeckleNoise__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SpeckleNoise__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`SpeckleNoise__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`SpeckleNoise__SWIG_2`) = c("SWIGFunction", class('SpeckleNoise__SWIG_2'))

`SpeckleNoise` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- SpeckleNoise__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- SpeckleNoise__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- SpeckleNoise__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SpeckleNoise with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_SqrtImageFilter

`delete_SqrtImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SqrtImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SqrtImageFilter`, 'returnType') = 'void'
attr(`delete_SqrtImageFilter`, "inputTypes") = c('_p_itk__simple__SqrtImageFilter')
class(`delete_SqrtImageFilter`) = c("SWIGFunction", class('delete_SqrtImageFilter'))

# Start of new_SqrtImageFilter

`SqrtImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SqrtImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SqrtImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SqrtImageFilter);
  ans
  
}

attr(`SqrtImageFilter`, 'returnType') = '_p_itk__simple__SqrtImageFilter'
class(`SqrtImageFilter`) = c("SWIGFunction", class('SqrtImageFilter'))

# Start of SqrtImageFilter_GetName

`SqrtImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SqrtImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SqrtImageFilter_GetName`, 'returnType') = 'character'
attr(`SqrtImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SqrtImageFilter')
class(`SqrtImageFilter_GetName`) = c("SWIGFunction", class('SqrtImageFilter_GetName'))

# Start of SqrtImageFilter_ToString

`SqrtImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SqrtImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SqrtImageFilter_ToString`, 'returnType') = 'character'
attr(`SqrtImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SqrtImageFilter')
class(`SqrtImageFilter_ToString`) = c("SWIGFunction", class('SqrtImageFilter_ToString'))

# Start of SqrtImageFilter_Execute

`SqrtImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SqrtImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SqrtImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`SqrtImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__SqrtImageFilter', '_p_itk__simple__Image')
class(`SqrtImageFilter_Execute`) = c("SWIGFunction", class('SqrtImageFilter_Execute'))

# Start of accessor method for itk::simple::SqrtImageFilter
setMethod('$', '_p_itk__simple__SqrtImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = SqrtImageFilter_GetName, 'ToString' = SqrtImageFilter_ToString, 'Execute' = SqrtImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SqrtImageFilter
# Start of Sqrt

`Sqrt` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Sqrt', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Sqrt`, 'returnType') = '_p_itk__simple__Image'
attr(`Sqrt`, "inputTypes") = c('_p_itk__simple__Image')
class(`Sqrt`) = c("SWIGFunction", class('Sqrt'))

# Start of delete_SquareImageFilter

`delete_SquareImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SquareImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SquareImageFilter`, 'returnType') = 'void'
attr(`delete_SquareImageFilter`, "inputTypes") = c('_p_itk__simple__SquareImageFilter')
class(`delete_SquareImageFilter`) = c("SWIGFunction", class('delete_SquareImageFilter'))

# Start of new_SquareImageFilter

`SquareImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SquareImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SquareImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SquareImageFilter);
  ans
  
}

attr(`SquareImageFilter`, 'returnType') = '_p_itk__simple__SquareImageFilter'
class(`SquareImageFilter`) = c("SWIGFunction", class('SquareImageFilter'))

# Start of SquareImageFilter_GetName

`SquareImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SquareImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SquareImageFilter_GetName`, 'returnType') = 'character'
attr(`SquareImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SquareImageFilter')
class(`SquareImageFilter_GetName`) = c("SWIGFunction", class('SquareImageFilter_GetName'))

# Start of SquareImageFilter_ToString

`SquareImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SquareImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SquareImageFilter_ToString`, 'returnType') = 'character'
attr(`SquareImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SquareImageFilter')
class(`SquareImageFilter_ToString`) = c("SWIGFunction", class('SquareImageFilter_ToString'))

# Start of SquareImageFilter_Execute

`SquareImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SquareImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SquareImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`SquareImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__SquareImageFilter', '_p_itk__simple__Image')
class(`SquareImageFilter_Execute`) = c("SWIGFunction", class('SquareImageFilter_Execute'))

# Start of accessor method for itk::simple::SquareImageFilter
setMethod('$', '_p_itk__simple__SquareImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = SquareImageFilter_GetName, 'ToString' = SquareImageFilter_ToString, 'Execute' = SquareImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SquareImageFilter
# Start of Square

`Square` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Square', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Square`, 'returnType') = '_p_itk__simple__Image'
attr(`Square`, "inputTypes") = c('_p_itk__simple__Image')
class(`Square`) = c("SWIGFunction", class('Square'))

# Start of delete_SquaredDifferenceImageFilter

`delete_SquaredDifferenceImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SquaredDifferenceImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SquaredDifferenceImageFilter`, 'returnType') = 'void'
attr(`delete_SquaredDifferenceImageFilter`, "inputTypes") = c('_p_itk__simple__SquaredDifferenceImageFilter')
class(`delete_SquaredDifferenceImageFilter`) = c("SWIGFunction", class('delete_SquaredDifferenceImageFilter'))

# Start of new_SquaredDifferenceImageFilter

`SquaredDifferenceImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SquaredDifferenceImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SquaredDifferenceImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SquaredDifferenceImageFilter);
  ans
  
}

attr(`SquaredDifferenceImageFilter`, 'returnType') = '_p_itk__simple__SquaredDifferenceImageFilter'
class(`SquaredDifferenceImageFilter`) = c("SWIGFunction", class('SquaredDifferenceImageFilter'))

# Start of SquaredDifferenceImageFilter_GetName

`SquaredDifferenceImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SquaredDifferenceImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SquaredDifferenceImageFilter_GetName`, 'returnType') = 'character'
attr(`SquaredDifferenceImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SquaredDifferenceImageFilter')
class(`SquaredDifferenceImageFilter_GetName`) = c("SWIGFunction", class('SquaredDifferenceImageFilter_GetName'))

# Start of SquaredDifferenceImageFilter_ToString

`SquaredDifferenceImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SquaredDifferenceImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SquaredDifferenceImageFilter_ToString`, 'returnType') = 'character'
attr(`SquaredDifferenceImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SquaredDifferenceImageFilter')
class(`SquaredDifferenceImageFilter_ToString`) = c("SWIGFunction", class('SquaredDifferenceImageFilter_ToString'))

# Start of SquaredDifferenceImageFilter_Execute

`SquaredDifferenceImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_SquaredDifferenceImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SquaredDifferenceImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`SquaredDifferenceImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__SquaredDifferenceImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`SquaredDifferenceImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('SquaredDifferenceImageFilter_Execute__SWIG_0'))

# Start of SquaredDifferenceImageFilter_Execute

`SquaredDifferenceImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_SquaredDifferenceImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SquaredDifferenceImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`SquaredDifferenceImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__SquaredDifferenceImageFilter', '_p_itk__simple__Image', 'numeric')
class(`SquaredDifferenceImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('SquaredDifferenceImageFilter_Execute__SWIG_1'))

# Start of SquaredDifferenceImageFilter_Execute

`SquaredDifferenceImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_SquaredDifferenceImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SquaredDifferenceImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`SquaredDifferenceImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__SquaredDifferenceImageFilter', 'numeric', '_p_itk__simple__Image')
class(`SquaredDifferenceImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('SquaredDifferenceImageFilter_Execute__SWIG_2'))

`SquaredDifferenceImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__SquaredDifferenceImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- SquaredDifferenceImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__SquaredDifferenceImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- SquaredDifferenceImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__SquaredDifferenceImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- SquaredDifferenceImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SquaredDifferenceImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::SquaredDifferenceImageFilter
setMethod('$', '_p_itk__simple__SquaredDifferenceImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = SquaredDifferenceImageFilter_GetName, 'ToString' = SquaredDifferenceImageFilter_ToString, 'Execute' = SquaredDifferenceImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SquaredDifferenceImageFilter
# Start of SquaredDifference

`SquaredDifference__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_SquaredDifference__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SquaredDifference__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`SquaredDifference__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`SquaredDifference__SWIG_0`) = c("SWIGFunction", class('SquaredDifference__SWIG_0'))

# Start of SquaredDifference

`SquaredDifference__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_SquaredDifference__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SquaredDifference__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`SquaredDifference__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`SquaredDifference__SWIG_1`) = c("SWIGFunction", class('SquaredDifference__SWIG_1'))

# Start of SquaredDifference

`SquaredDifference__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_SquaredDifference__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SquaredDifference__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`SquaredDifference__SWIG_2`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`SquaredDifference__SWIG_2`) = c("SWIGFunction", class('SquaredDifference__SWIG_2'))

`SquaredDifference` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- SquaredDifference__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- SquaredDifference__SWIG_1; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- SquaredDifference__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SquaredDifference with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_StandardDeviationProjectionImageFilter

`delete_StandardDeviationProjectionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_StandardDeviationProjectionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_StandardDeviationProjectionImageFilter`, 'returnType') = 'void'
attr(`delete_StandardDeviationProjectionImageFilter`, "inputTypes") = c('_p_itk__simple__StandardDeviationProjectionImageFilter')
class(`delete_StandardDeviationProjectionImageFilter`) = c("SWIGFunction", class('delete_StandardDeviationProjectionImageFilter'))

# Start of new_StandardDeviationProjectionImageFilter

`StandardDeviationProjectionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_StandardDeviationProjectionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__StandardDeviationProjectionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_StandardDeviationProjectionImageFilter);
  ans
  
}

attr(`StandardDeviationProjectionImageFilter`, 'returnType') = '_p_itk__simple__StandardDeviationProjectionImageFilter'
class(`StandardDeviationProjectionImageFilter`) = c("SWIGFunction", class('StandardDeviationProjectionImageFilter'))

# Start of StandardDeviationProjectionImageFilter_SetProjectionDimension

`StandardDeviationProjectionImageFilter_SetProjectionDimension` = function(self, ProjectionDimension)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ProjectionDimension = as.integer(ProjectionDimension);
  
  if(length(ProjectionDimension) > 1) {
    warning("using only the first element of ProjectionDimension");
  };
  
  ;ans = .Call('R_swig_StandardDeviationProjectionImageFilter_SetProjectionDimension', self, ProjectionDimension, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`StandardDeviationProjectionImageFilter_SetProjectionDimension`, 'returnType') = 'void'
attr(`StandardDeviationProjectionImageFilter_SetProjectionDimension`, "inputTypes") = c('_p_itk__simple__StandardDeviationProjectionImageFilter', 'integer')
class(`StandardDeviationProjectionImageFilter_SetProjectionDimension`) = c("SWIGFunction", class('StandardDeviationProjectionImageFilter_SetProjectionDimension'))

# Start of StandardDeviationProjectionImageFilter_GetProjectionDimension

`StandardDeviationProjectionImageFilter_GetProjectionDimension` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_StandardDeviationProjectionImageFilter_GetProjectionDimension', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`StandardDeviationProjectionImageFilter_GetProjectionDimension`, 'returnType') = 'integer'
attr(`StandardDeviationProjectionImageFilter_GetProjectionDimension`, "inputTypes") = c('_p_itk__simple__StandardDeviationProjectionImageFilter')
class(`StandardDeviationProjectionImageFilter_GetProjectionDimension`) = c("SWIGFunction", class('StandardDeviationProjectionImageFilter_GetProjectionDimension'))

# Start of StandardDeviationProjectionImageFilter_GetName

`StandardDeviationProjectionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_StandardDeviationProjectionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`StandardDeviationProjectionImageFilter_GetName`, 'returnType') = 'character'
attr(`StandardDeviationProjectionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__StandardDeviationProjectionImageFilter')
class(`StandardDeviationProjectionImageFilter_GetName`) = c("SWIGFunction", class('StandardDeviationProjectionImageFilter_GetName'))

# Start of StandardDeviationProjectionImageFilter_ToString

`StandardDeviationProjectionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_StandardDeviationProjectionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`StandardDeviationProjectionImageFilter_ToString`, 'returnType') = 'character'
attr(`StandardDeviationProjectionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__StandardDeviationProjectionImageFilter')
class(`StandardDeviationProjectionImageFilter_ToString`) = c("SWIGFunction", class('StandardDeviationProjectionImageFilter_ToString'))

# Start of StandardDeviationProjectionImageFilter_Execute

`StandardDeviationProjectionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_StandardDeviationProjectionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`StandardDeviationProjectionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`StandardDeviationProjectionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__StandardDeviationProjectionImageFilter', '_p_itk__simple__Image')
class(`StandardDeviationProjectionImageFilter_Execute`) = c("SWIGFunction", class('StandardDeviationProjectionImageFilter_Execute'))

# Start of accessor method for itk::simple::StandardDeviationProjectionImageFilter
setMethod('$', '_p_itk__simple__StandardDeviationProjectionImageFilter', function(x, name)

{
  accessorFuns = list('SetProjectionDimension' = StandardDeviationProjectionImageFilter_SetProjectionDimension, 'GetProjectionDimension' = StandardDeviationProjectionImageFilter_GetProjectionDimension, 'GetName' = StandardDeviationProjectionImageFilter_GetName, 'ToString' = StandardDeviationProjectionImageFilter_ToString, 'Execute' = StandardDeviationProjectionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::StandardDeviationProjectionImageFilter
# Start of StandardDeviationProjection

`StandardDeviationProjection__SWIG_0` = function(image1, projectionDimension, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  projectionDimension = as.integer(projectionDimension);
  
  if(length(projectionDimension) > 1) {
    warning("using only the first element of projectionDimension");
  };
  
  ;ans = .Call('R_swig_StandardDeviationProjection__SWIG_0', image1, projectionDimension, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`StandardDeviationProjection__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`StandardDeviationProjection__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`StandardDeviationProjection__SWIG_0`) = c("SWIGFunction", class('StandardDeviationProjection__SWIG_0'))

# Start of StandardDeviationProjection

`StandardDeviationProjection__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_StandardDeviationProjection__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`StandardDeviationProjection__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`StandardDeviationProjection__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`StandardDeviationProjection__SWIG_1`) = c("SWIGFunction", class('StandardDeviationProjection__SWIG_1'))

`StandardDeviationProjection` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- StandardDeviationProjection__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- StandardDeviationProjection__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for StandardDeviationProjection with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_StatisticsImageFilter

`delete_StatisticsImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_StatisticsImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_StatisticsImageFilter`, 'returnType') = 'void'
attr(`delete_StatisticsImageFilter`, "inputTypes") = c('_p_itk__simple__StatisticsImageFilter')
class(`delete_StatisticsImageFilter`) = c("SWIGFunction", class('delete_StatisticsImageFilter'))

# Start of new_StatisticsImageFilter

`StatisticsImageFilter` = function()
{
  ;ans = .Call('R_swig_new_StatisticsImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__StatisticsImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_StatisticsImageFilter);
  ans
  
}

attr(`StatisticsImageFilter`, 'returnType') = '_p_itk__simple__StatisticsImageFilter'
class(`StatisticsImageFilter`) = c("SWIGFunction", class('StatisticsImageFilter'))

# Start of StatisticsImageFilter_GetMinimum

`StatisticsImageFilter_GetMinimum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_StatisticsImageFilter_GetMinimum', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`StatisticsImageFilter_GetMinimum`, 'returnType') = 'numeric'
attr(`StatisticsImageFilter_GetMinimum`, "inputTypes") = c('_p_itk__simple__StatisticsImageFilter')
class(`StatisticsImageFilter_GetMinimum`) = c("SWIGFunction", class('StatisticsImageFilter_GetMinimum'))

# Start of StatisticsImageFilter_GetMaximum

`StatisticsImageFilter_GetMaximum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_StatisticsImageFilter_GetMaximum', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`StatisticsImageFilter_GetMaximum`, 'returnType') = 'numeric'
attr(`StatisticsImageFilter_GetMaximum`, "inputTypes") = c('_p_itk__simple__StatisticsImageFilter')
class(`StatisticsImageFilter_GetMaximum`) = c("SWIGFunction", class('StatisticsImageFilter_GetMaximum'))

# Start of StatisticsImageFilter_GetMean

`StatisticsImageFilter_GetMean` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_StatisticsImageFilter_GetMean', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`StatisticsImageFilter_GetMean`, 'returnType') = 'numeric'
attr(`StatisticsImageFilter_GetMean`, "inputTypes") = c('_p_itk__simple__StatisticsImageFilter')
class(`StatisticsImageFilter_GetMean`) = c("SWIGFunction", class('StatisticsImageFilter_GetMean'))

# Start of StatisticsImageFilter_GetSigma

`StatisticsImageFilter_GetSigma` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_StatisticsImageFilter_GetSigma', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`StatisticsImageFilter_GetSigma`, 'returnType') = 'numeric'
attr(`StatisticsImageFilter_GetSigma`, "inputTypes") = c('_p_itk__simple__StatisticsImageFilter')
class(`StatisticsImageFilter_GetSigma`) = c("SWIGFunction", class('StatisticsImageFilter_GetSigma'))

# Start of StatisticsImageFilter_GetVariance

`StatisticsImageFilter_GetVariance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_StatisticsImageFilter_GetVariance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`StatisticsImageFilter_GetVariance`, 'returnType') = 'numeric'
attr(`StatisticsImageFilter_GetVariance`, "inputTypes") = c('_p_itk__simple__StatisticsImageFilter')
class(`StatisticsImageFilter_GetVariance`) = c("SWIGFunction", class('StatisticsImageFilter_GetVariance'))

# Start of StatisticsImageFilter_GetSum

`StatisticsImageFilter_GetSum` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_StatisticsImageFilter_GetSum', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`StatisticsImageFilter_GetSum`, 'returnType') = 'numeric'
attr(`StatisticsImageFilter_GetSum`, "inputTypes") = c('_p_itk__simple__StatisticsImageFilter')
class(`StatisticsImageFilter_GetSum`) = c("SWIGFunction", class('StatisticsImageFilter_GetSum'))

# Start of StatisticsImageFilter_GetName

`StatisticsImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_StatisticsImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`StatisticsImageFilter_GetName`, 'returnType') = 'character'
attr(`StatisticsImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__StatisticsImageFilter')
class(`StatisticsImageFilter_GetName`) = c("SWIGFunction", class('StatisticsImageFilter_GetName'))

# Start of StatisticsImageFilter_ToString

`StatisticsImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_StatisticsImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`StatisticsImageFilter_ToString`, 'returnType') = 'character'
attr(`StatisticsImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__StatisticsImageFilter')
class(`StatisticsImageFilter_ToString`) = c("SWIGFunction", class('StatisticsImageFilter_ToString'))

# Start of StatisticsImageFilter_Execute

`StatisticsImageFilter_Execute` = function(self, image)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_StatisticsImageFilter_Execute', self, image, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`StatisticsImageFilter_Execute`, 'returnType') = 'void'
attr(`StatisticsImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__StatisticsImageFilter', '_p_itk__simple__Image')
class(`StatisticsImageFilter_Execute`) = c("SWIGFunction", class('StatisticsImageFilter_Execute'))

# Start of accessor method for itk::simple::StatisticsImageFilter
setMethod('$', '_p_itk__simple__StatisticsImageFilter', function(x, name)

{
  accessorFuns = list('GetMinimum' = StatisticsImageFilter_GetMinimum, 'GetMaximum' = StatisticsImageFilter_GetMaximum, 'GetMean' = StatisticsImageFilter_GetMean, 'GetSigma' = StatisticsImageFilter_GetSigma, 'GetVariance' = StatisticsImageFilter_GetVariance, 'GetSum' = StatisticsImageFilter_GetSum, 'GetName' = StatisticsImageFilter_GetName, 'ToString' = StatisticsImageFilter_ToString, 'Execute' = StatisticsImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::StatisticsImageFilter
# Start of delete_StochasticFractalDimensionImageFilter

`delete_StochasticFractalDimensionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_StochasticFractalDimensionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_StochasticFractalDimensionImageFilter`, 'returnType') = 'void'
attr(`delete_StochasticFractalDimensionImageFilter`, "inputTypes") = c('_p_itk__simple__StochasticFractalDimensionImageFilter')
class(`delete_StochasticFractalDimensionImageFilter`) = c("SWIGFunction", class('delete_StochasticFractalDimensionImageFilter'))

# Start of new_StochasticFractalDimensionImageFilter

`StochasticFractalDimensionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_StochasticFractalDimensionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__StochasticFractalDimensionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_StochasticFractalDimensionImageFilter);
  ans
  
}

attr(`StochasticFractalDimensionImageFilter`, 'returnType') = '_p_itk__simple__StochasticFractalDimensionImageFilter'
class(`StochasticFractalDimensionImageFilter`) = c("SWIGFunction", class('StochasticFractalDimensionImageFilter'))

# Start of StochasticFractalDimensionImageFilter_SetNeighborhoodRadius

`StochasticFractalDimensionImageFilter_SetNeighborhoodRadius__SWIG_0` = function(self, NeighborhoodRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NeighborhoodRadius = as.integer(NeighborhoodRadius);
  ;ans = .Call('R_swig_StochasticFractalDimensionImageFilter_SetNeighborhoodRadius__SWIG_0', self, NeighborhoodRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`StochasticFractalDimensionImageFilter_SetNeighborhoodRadius__SWIG_0`, 'returnType') = 'void'
attr(`StochasticFractalDimensionImageFilter_SetNeighborhoodRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__StochasticFractalDimensionImageFilter', 'integer')
class(`StochasticFractalDimensionImageFilter_SetNeighborhoodRadius__SWIG_0`) = c("SWIGFunction", class('StochasticFractalDimensionImageFilter_SetNeighborhoodRadius__SWIG_0'))

# Start of StochasticFractalDimensionImageFilter_SetNeighborhoodRadius

`StochasticFractalDimensionImageFilter_SetNeighborhoodRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_StochasticFractalDimensionImageFilter_SetNeighborhoodRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`StochasticFractalDimensionImageFilter_SetNeighborhoodRadius__SWIG_1`, 'returnType') = 'void'
attr(`StochasticFractalDimensionImageFilter_SetNeighborhoodRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__StochasticFractalDimensionImageFilter', 'integer')
class(`StochasticFractalDimensionImageFilter_SetNeighborhoodRadius__SWIG_1`) = c("SWIGFunction", class('StochasticFractalDimensionImageFilter_SetNeighborhoodRadius__SWIG_1'))

`StochasticFractalDimensionImageFilter_SetNeighborhoodRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__StochasticFractalDimensionImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- StochasticFractalDimensionImageFilter_SetNeighborhoodRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__StochasticFractalDimensionImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- StochasticFractalDimensionImageFilter_SetNeighborhoodRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for StochasticFractalDimensionImageFilter_SetNeighborhoodRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of StochasticFractalDimensionImageFilter_GetNeighborhoodRadius

`StochasticFractalDimensionImageFilter_GetNeighborhoodRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_StochasticFractalDimensionImageFilter_GetNeighborhoodRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`StochasticFractalDimensionImageFilter_GetNeighborhoodRadius`, 'returnType') = 'integer'
attr(`StochasticFractalDimensionImageFilter_GetNeighborhoodRadius`, "inputTypes") = c('_p_itk__simple__StochasticFractalDimensionImageFilter')
class(`StochasticFractalDimensionImageFilter_GetNeighborhoodRadius`) = c("SWIGFunction", class('StochasticFractalDimensionImageFilter_GetNeighborhoodRadius'))

# Start of StochasticFractalDimensionImageFilter_GetName

`StochasticFractalDimensionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_StochasticFractalDimensionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`StochasticFractalDimensionImageFilter_GetName`, 'returnType') = 'character'
attr(`StochasticFractalDimensionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__StochasticFractalDimensionImageFilter')
class(`StochasticFractalDimensionImageFilter_GetName`) = c("SWIGFunction", class('StochasticFractalDimensionImageFilter_GetName'))

# Start of StochasticFractalDimensionImageFilter_ToString

`StochasticFractalDimensionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_StochasticFractalDimensionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`StochasticFractalDimensionImageFilter_ToString`, 'returnType') = 'character'
attr(`StochasticFractalDimensionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__StochasticFractalDimensionImageFilter')
class(`StochasticFractalDimensionImageFilter_ToString`) = c("SWIGFunction", class('StochasticFractalDimensionImageFilter_ToString'))

# Start of StochasticFractalDimensionImageFilter_Execute

`StochasticFractalDimensionImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_StochasticFractalDimensionImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`StochasticFractalDimensionImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`StochasticFractalDimensionImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__StochasticFractalDimensionImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`StochasticFractalDimensionImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('StochasticFractalDimensionImageFilter_Execute__SWIG_0'))

# Start of StochasticFractalDimensionImageFilter_Execute

`StochasticFractalDimensionImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_StochasticFractalDimensionImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`StochasticFractalDimensionImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`StochasticFractalDimensionImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__StochasticFractalDimensionImageFilter', '_p_itk__simple__Image')
class(`StochasticFractalDimensionImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('StochasticFractalDimensionImageFilter_Execute__SWIG_1'))

`StochasticFractalDimensionImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__StochasticFractalDimensionImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- StochasticFractalDimensionImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__StochasticFractalDimensionImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- StochasticFractalDimensionImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for StochasticFractalDimensionImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::StochasticFractalDimensionImageFilter
setMethod('$', '_p_itk__simple__StochasticFractalDimensionImageFilter', function(x, name)

{
  accessorFuns = list('SetNeighborhoodRadius' = StochasticFractalDimensionImageFilter_SetNeighborhoodRadius, 'GetNeighborhoodRadius' = StochasticFractalDimensionImageFilter_GetNeighborhoodRadius, 'GetName' = StochasticFractalDimensionImageFilter_GetName, 'ToString' = StochasticFractalDimensionImageFilter_ToString, 'Execute' = StochasticFractalDimensionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::StochasticFractalDimensionImageFilter
# Start of StochasticFractalDimension

`StochasticFractalDimension__SWIG_0` = function(image, maskImage, neighborhoodRadius, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  neighborhoodRadius = as.integer(neighborhoodRadius);
  ;ans = .Call('R_swig_StochasticFractalDimension__SWIG_0', image, maskImage, neighborhoodRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`StochasticFractalDimension__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`StochasticFractalDimension__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`StochasticFractalDimension__SWIG_0`) = c("SWIGFunction", class('StochasticFractalDimension__SWIG_0'))

# Start of StochasticFractalDimension

`StochasticFractalDimension__SWIG_1` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_StochasticFractalDimension__SWIG_1', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`StochasticFractalDimension__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`StochasticFractalDimension__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`StochasticFractalDimension__SWIG_1`) = c("SWIGFunction", class('StochasticFractalDimension__SWIG_1'))

# Start of StochasticFractalDimension

`StochasticFractalDimension__SWIG_2` = function(image, neighborhoodRadius, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  neighborhoodRadius = as.integer(neighborhoodRadius);
  ;ans = .Call('R_swig_StochasticFractalDimension__SWIG_2', image, neighborhoodRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`StochasticFractalDimension__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`StochasticFractalDimension__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`StochasticFractalDimension__SWIG_2`) = c("SWIGFunction", class('StochasticFractalDimension__SWIG_2'))

# Start of StochasticFractalDimension

`StochasticFractalDimension__SWIG_3` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_StochasticFractalDimension__SWIG_3', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`StochasticFractalDimension__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`StochasticFractalDimension__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`StochasticFractalDimension__SWIG_3`) = c("SWIGFunction", class('StochasticFractalDimension__SWIG_3'))

`StochasticFractalDimension` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- StochasticFractalDimension__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- StochasticFractalDimension__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- StochasticFractalDimension__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- StochasticFractalDimension__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for StochasticFractalDimension with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_SubtractImageFilter

`delete_SubtractImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SubtractImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SubtractImageFilter`, 'returnType') = 'void'
attr(`delete_SubtractImageFilter`, "inputTypes") = c('_p_itk__simple__SubtractImageFilter')
class(`delete_SubtractImageFilter`) = c("SWIGFunction", class('delete_SubtractImageFilter'))

# Start of new_SubtractImageFilter

`SubtractImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SubtractImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SubtractImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SubtractImageFilter);
  ans
  
}

attr(`SubtractImageFilter`, 'returnType') = '_p_itk__simple__SubtractImageFilter'
class(`SubtractImageFilter`) = c("SWIGFunction", class('SubtractImageFilter'))

# Start of SubtractImageFilter_GetName

`SubtractImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SubtractImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SubtractImageFilter_GetName`, 'returnType') = 'character'
attr(`SubtractImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SubtractImageFilter')
class(`SubtractImageFilter_GetName`) = c("SWIGFunction", class('SubtractImageFilter_GetName'))

# Start of SubtractImageFilter_ToString

`SubtractImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SubtractImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SubtractImageFilter_ToString`, 'returnType') = 'character'
attr(`SubtractImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SubtractImageFilter')
class(`SubtractImageFilter_ToString`) = c("SWIGFunction", class('SubtractImageFilter_ToString'))

# Start of SubtractImageFilter_Execute

`SubtractImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_SubtractImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SubtractImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`SubtractImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__SubtractImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`SubtractImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('SubtractImageFilter_Execute__SWIG_0'))

# Start of SubtractImageFilter_Execute

`SubtractImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_SubtractImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SubtractImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`SubtractImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__SubtractImageFilter', '_p_itk__simple__Image', 'numeric')
class(`SubtractImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('SubtractImageFilter_Execute__SWIG_1'))

# Start of SubtractImageFilter_Execute

`SubtractImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_SubtractImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SubtractImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`SubtractImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__SubtractImageFilter', 'numeric', '_p_itk__simple__Image')
class(`SubtractImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('SubtractImageFilter_Execute__SWIG_2'))

`SubtractImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__SubtractImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- SubtractImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__SubtractImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- SubtractImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__SubtractImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- SubtractImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SubtractImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::SubtractImageFilter
setMethod('$', '_p_itk__simple__SubtractImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = SubtractImageFilter_GetName, 'ToString' = SubtractImageFilter_ToString, 'Execute' = SubtractImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SubtractImageFilter
# Start of Subtract

`Subtract__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Subtract__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Subtract__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Subtract__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Subtract__SWIG_0`) = c("SWIGFunction", class('Subtract__SWIG_0'))

# Start of Subtract

`Subtract__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Subtract__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Subtract__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Subtract__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`Subtract__SWIG_1`) = c("SWIGFunction", class('Subtract__SWIG_1'))

# Start of Subtract

`Subtract__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Subtract__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Subtract__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Subtract__SWIG_2`, "inputTypes") = c('numeric', '_p_itk__simple__Image')
class(`Subtract__SWIG_2`) = c("SWIGFunction", class('Subtract__SWIG_2'))

`Subtract` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Subtract__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Subtract__SWIG_1; 
    }
    else if (( is.numeric(argv[[1]]) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Subtract__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Subtract with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_SumProjectionImageFilter

`delete_SumProjectionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SumProjectionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SumProjectionImageFilter`, 'returnType') = 'void'
attr(`delete_SumProjectionImageFilter`, "inputTypes") = c('_p_itk__simple__SumProjectionImageFilter')
class(`delete_SumProjectionImageFilter`) = c("SWIGFunction", class('delete_SumProjectionImageFilter'))

# Start of new_SumProjectionImageFilter

`SumProjectionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_SumProjectionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SumProjectionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SumProjectionImageFilter);
  ans
  
}

attr(`SumProjectionImageFilter`, 'returnType') = '_p_itk__simple__SumProjectionImageFilter'
class(`SumProjectionImageFilter`) = c("SWIGFunction", class('SumProjectionImageFilter'))

# Start of SumProjectionImageFilter_SetProjectionDimension

`SumProjectionImageFilter_SetProjectionDimension` = function(self, ProjectionDimension)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ProjectionDimension = as.integer(ProjectionDimension);
  
  if(length(ProjectionDimension) > 1) {
    warning("using only the first element of ProjectionDimension");
  };
  
  ;ans = .Call('R_swig_SumProjectionImageFilter_SetProjectionDimension', self, ProjectionDimension, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SumProjectionImageFilter_SetProjectionDimension`, 'returnType') = 'void'
attr(`SumProjectionImageFilter_SetProjectionDimension`, "inputTypes") = c('_p_itk__simple__SumProjectionImageFilter', 'integer')
class(`SumProjectionImageFilter_SetProjectionDimension`) = c("SWIGFunction", class('SumProjectionImageFilter_SetProjectionDimension'))

# Start of SumProjectionImageFilter_GetProjectionDimension

`SumProjectionImageFilter_GetProjectionDimension` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SumProjectionImageFilter_GetProjectionDimension', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SumProjectionImageFilter_GetProjectionDimension`, 'returnType') = 'integer'
attr(`SumProjectionImageFilter_GetProjectionDimension`, "inputTypes") = c('_p_itk__simple__SumProjectionImageFilter')
class(`SumProjectionImageFilter_GetProjectionDimension`) = c("SWIGFunction", class('SumProjectionImageFilter_GetProjectionDimension'))

# Start of SumProjectionImageFilter_GetName

`SumProjectionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SumProjectionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SumProjectionImageFilter_GetName`, 'returnType') = 'character'
attr(`SumProjectionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__SumProjectionImageFilter')
class(`SumProjectionImageFilter_GetName`) = c("SWIGFunction", class('SumProjectionImageFilter_GetName'))

# Start of SumProjectionImageFilter_ToString

`SumProjectionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SumProjectionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SumProjectionImageFilter_ToString`, 'returnType') = 'character'
attr(`SumProjectionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__SumProjectionImageFilter')
class(`SumProjectionImageFilter_ToString`) = c("SWIGFunction", class('SumProjectionImageFilter_ToString'))

# Start of SumProjectionImageFilter_Execute

`SumProjectionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SumProjectionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SumProjectionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`SumProjectionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__SumProjectionImageFilter', '_p_itk__simple__Image')
class(`SumProjectionImageFilter_Execute`) = c("SWIGFunction", class('SumProjectionImageFilter_Execute'))

# Start of accessor method for itk::simple::SumProjectionImageFilter
setMethod('$', '_p_itk__simple__SumProjectionImageFilter', function(x, name)

{
  accessorFuns = list('SetProjectionDimension' = SumProjectionImageFilter_SetProjectionDimension, 'GetProjectionDimension' = SumProjectionImageFilter_GetProjectionDimension, 'GetName' = SumProjectionImageFilter_GetName, 'ToString' = SumProjectionImageFilter_ToString, 'Execute' = SumProjectionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SumProjectionImageFilter
# Start of SumProjection

`SumProjection__SWIG_0` = function(image1, projectionDimension, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  projectionDimension = as.integer(projectionDimension);
  
  if(length(projectionDimension) > 1) {
    warning("using only the first element of projectionDimension");
  };
  
  ;ans = .Call('R_swig_SumProjection__SWIG_0', image1, projectionDimension, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SumProjection__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`SumProjection__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`SumProjection__SWIG_0`) = c("SWIGFunction", class('SumProjection__SWIG_0'))

# Start of SumProjection

`SumProjection__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_SumProjection__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SumProjection__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`SumProjection__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`SumProjection__SWIG_1`) = c("SWIGFunction", class('SumProjection__SWIG_1'))

`SumProjection` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- SumProjection__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- SumProjection__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SumProjection with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_SymmetricForcesDemonsRegistrationFilter

`delete_SymmetricForcesDemonsRegistrationFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_SymmetricForcesDemonsRegistrationFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_SymmetricForcesDemonsRegistrationFilter`, 'returnType') = 'void'
attr(`delete_SymmetricForcesDemonsRegistrationFilter`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`delete_SymmetricForcesDemonsRegistrationFilter`) = c("SWIGFunction", class('delete_SymmetricForcesDemonsRegistrationFilter'))

# Start of new_SymmetricForcesDemonsRegistrationFilter

`SymmetricForcesDemonsRegistrationFilter` = function()
{
  ;ans = .Call('R_swig_new_SymmetricForcesDemonsRegistrationFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__SymmetricForcesDemonsRegistrationFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_SymmetricForcesDemonsRegistrationFilter);
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter`, 'returnType') = '_p_itk__simple__SymmetricForcesDemonsRegistrationFilter'
class(`SymmetricForcesDemonsRegistrationFilter`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter'))

# Start of SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations

`SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_0` = function(self, StandardDeviations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  StandardDeviations = as.numeric(StandardDeviations);
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_0', self, StandardDeviations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_0`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_0`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', 'numeric')
class(`SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_0`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_0'))

# Start of SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations

`SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_1`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_1`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', 'numeric')
class(`SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_1`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_1'))

`SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__SymmetricForcesDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__SymmetricForcesDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of SymmetricForcesDemonsRegistrationFilter_GetStandardDeviations

`SymmetricForcesDemonsRegistrationFilter_GetStandardDeviations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SymmetricForcesDemonsRegistrationFilter_GetStandardDeviations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_GetStandardDeviations`, 'returnType') = 'numeric'
attr(`SymmetricForcesDemonsRegistrationFilter_GetStandardDeviations`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_GetStandardDeviations`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_GetStandardDeviations'))

# Start of SymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations

`SymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', 'integer')
class(`SymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations'))

# Start of SymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations

`SymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`SymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations'))

# Start of SymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError

`SymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError` = function(self, MaximumRMSError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError', self, MaximumRMSError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', 'numeric')
class(`SymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError'))

# Start of SymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError

`SymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError`, 'returnType') = 'numeric'
attr(`SymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError'))

# Start of SymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField

`SymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField` = function(self, SmoothDisplacementField)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SmoothDisplacementField = as.logical(SmoothDisplacementField);
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField', self, SmoothDisplacementField, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', 'logical')
class(`SymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField'))

# Start of SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn

`SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn'))

# Start of SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff

`SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff'))

# Start of SymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField

`SymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField`, 'returnType') = 'logical'
attr(`SymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField'))

# Start of SymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField

`SymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField` = function(self, SmoothUpdateField)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SmoothUpdateField = as.logical(SmoothUpdateField);
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField', self, SmoothUpdateField, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', 'logical')
class(`SymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField'))

# Start of SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn

`SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn'))

# Start of SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff

`SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff'))

# Start of SymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField

`SymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField`, 'returnType') = 'logical'
attr(`SymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField'))

# Start of SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations

`SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0` = function(self, UpdateFieldStandardDeviations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UpdateFieldStandardDeviations = as.numeric(UpdateFieldStandardDeviations);
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0', self, UpdateFieldStandardDeviations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', 'numeric')
class(`SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0'))

# Start of SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations

`SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', 'numeric')
class(`SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1'))

`SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__SymmetricForcesDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__SymmetricForcesDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of SymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations

`SymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations`, 'returnType') = 'numeric'
attr(`SymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations'))

# Start of SymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth

`SymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth` = function(self, MaximumKernelWidth)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaximumKernelWidth = as.integer(MaximumKernelWidth);
  
  if(length(MaximumKernelWidth) > 1) {
    warning("using only the first element of MaximumKernelWidth");
  };
  
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth', self, MaximumKernelWidth, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', 'integer')
class(`SymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth'))

# Start of SymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth

`SymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth`, 'returnType') = 'integer'
attr(`SymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth'))

# Start of SymmetricForcesDemonsRegistrationFilter_SetMaximumError

`SymmetricForcesDemonsRegistrationFilter_SetMaximumError` = function(self, MaximumError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SetMaximumError', self, MaximumError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SetMaximumError`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SetMaximumError`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', 'numeric')
class(`SymmetricForcesDemonsRegistrationFilter_SetMaximumError`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SetMaximumError'))

# Start of SymmetricForcesDemonsRegistrationFilter_GetMaximumError

`SymmetricForcesDemonsRegistrationFilter_GetMaximumError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SymmetricForcesDemonsRegistrationFilter_GetMaximumError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_GetMaximumError`, 'returnType') = 'numeric'
attr(`SymmetricForcesDemonsRegistrationFilter_GetMaximumError`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_GetMaximumError`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_GetMaximumError'))

# Start of SymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold

`SymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold` = function(self, IntensityDifferenceThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold', self, IntensityDifferenceThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', 'numeric')
class(`SymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold'))

# Start of SymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold

`SymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold`, 'returnType') = 'numeric'
attr(`SymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold'))

# Start of SymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing

`SymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing` = function(self, UseImageSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  UseImageSpacing = as.logical(UseImageSpacing);
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing', self, UseImageSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', 'logical')
class(`SymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing'))

# Start of SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn

`SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn'))

# Start of SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff

`SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff'))

# Start of SymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing

`SymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing`, 'returnType') = 'logical'
attr(`SymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing'))

# Start of SymmetricForcesDemonsRegistrationFilter_GetElapsedIterations

`SymmetricForcesDemonsRegistrationFilter_GetElapsedIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SymmetricForcesDemonsRegistrationFilter_GetElapsedIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_GetElapsedIterations`, 'returnType') = 'integer'
attr(`SymmetricForcesDemonsRegistrationFilter_GetElapsedIterations`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_GetElapsedIterations`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_GetElapsedIterations'))

# Start of SymmetricForcesDemonsRegistrationFilter_GetRMSChange

`SymmetricForcesDemonsRegistrationFilter_GetRMSChange` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SymmetricForcesDemonsRegistrationFilter_GetRMSChange', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_GetRMSChange`, 'returnType') = 'numeric'
attr(`SymmetricForcesDemonsRegistrationFilter_GetRMSChange`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_GetRMSChange`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_GetRMSChange'))

# Start of SymmetricForcesDemonsRegistrationFilter_GetMetric

`SymmetricForcesDemonsRegistrationFilter_GetMetric` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SymmetricForcesDemonsRegistrationFilter_GetMetric', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_GetMetric`, 'returnType') = 'numeric'
attr(`SymmetricForcesDemonsRegistrationFilter_GetMetric`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_GetMetric`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_GetMetric'))

# Start of SymmetricForcesDemonsRegistrationFilter_StopRegistration

`SymmetricForcesDemonsRegistrationFilter_StopRegistration` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_StopRegistration', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_StopRegistration`, 'returnType') = 'void'
attr(`SymmetricForcesDemonsRegistrationFilter_StopRegistration`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_StopRegistration`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_StopRegistration'))

# Start of SymmetricForcesDemonsRegistrationFilter_GetName

`SymmetricForcesDemonsRegistrationFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SymmetricForcesDemonsRegistrationFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_GetName`, 'returnType') = 'character'
attr(`SymmetricForcesDemonsRegistrationFilter_GetName`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_GetName`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_GetName'))

# Start of SymmetricForcesDemonsRegistrationFilter_ToString

`SymmetricForcesDemonsRegistrationFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_SymmetricForcesDemonsRegistrationFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_ToString`, 'returnType') = 'character'
attr(`SymmetricForcesDemonsRegistrationFilter_ToString`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter')
class(`SymmetricForcesDemonsRegistrationFilter_ToString`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_ToString'))

# Start of SymmetricForcesDemonsRegistrationFilter_Execute

`SymmetricForcesDemonsRegistrationFilter_Execute__SWIG_0` = function(self, fixedImage, movingImage, initialDisplacementField, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  if (inherits(initialDisplacementField, "ExternalReference")) initialDisplacementField = slot(initialDisplacementField,"ref"); 
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_Execute__SWIG_0', self, fixedImage, movingImage, initialDisplacementField, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`SymmetricForcesDemonsRegistrationFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`SymmetricForcesDemonsRegistrationFilter_Execute__SWIG_0`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_Execute__SWIG_0'))

# Start of SymmetricForcesDemonsRegistrationFilter_Execute

`SymmetricForcesDemonsRegistrationFilter_Execute__SWIG_1` = function(self, fixedImage, movingImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(fixedImage, "ExternalReference")) fixedImage = slot(fixedImage,"ref"); 
  if (inherits(movingImage, "ExternalReference")) movingImage = slot(movingImage,"ref"); 
  ;ans = .Call('R_swig_SymmetricForcesDemonsRegistrationFilter_Execute__SWIG_1', self, fixedImage, movingImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`SymmetricForcesDemonsRegistrationFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`SymmetricForcesDemonsRegistrationFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`SymmetricForcesDemonsRegistrationFilter_Execute__SWIG_1`) = c("SWIGFunction", class('SymmetricForcesDemonsRegistrationFilter_Execute__SWIG_1'))

`SymmetricForcesDemonsRegistrationFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__SymmetricForcesDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- SymmetricForcesDemonsRegistrationFilter_Execute__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__SymmetricForcesDemonsRegistrationFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- SymmetricForcesDemonsRegistrationFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for SymmetricForcesDemonsRegistrationFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::SymmetricForcesDemonsRegistrationFilter
setMethod('$', '_p_itk__simple__SymmetricForcesDemonsRegistrationFilter', function(x, name)

{
  accessorFuns = list('SetStandardDeviations' = SymmetricForcesDemonsRegistrationFilter_SetStandardDeviations, 'GetStandardDeviations' = SymmetricForcesDemonsRegistrationFilter_GetStandardDeviations, 'SetNumberOfIterations' = SymmetricForcesDemonsRegistrationFilter_SetNumberOfIterations, 'GetNumberOfIterations' = SymmetricForcesDemonsRegistrationFilter_GetNumberOfIterations, 'SetMaximumRMSError' = SymmetricForcesDemonsRegistrationFilter_SetMaximumRMSError, 'GetMaximumRMSError' = SymmetricForcesDemonsRegistrationFilter_GetMaximumRMSError, 'SetSmoothDisplacementField' = SymmetricForcesDemonsRegistrationFilter_SetSmoothDisplacementField, 'SmoothDisplacementFieldOn' = SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOn, 'SmoothDisplacementFieldOff' = SymmetricForcesDemonsRegistrationFilter_SmoothDisplacementFieldOff, 'GetSmoothDisplacementField' = SymmetricForcesDemonsRegistrationFilter_GetSmoothDisplacementField, 'SetSmoothUpdateField' = SymmetricForcesDemonsRegistrationFilter_SetSmoothUpdateField, 'SmoothUpdateFieldOn' = SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOn, 'SmoothUpdateFieldOff' = SymmetricForcesDemonsRegistrationFilter_SmoothUpdateFieldOff, 'GetSmoothUpdateField' = SymmetricForcesDemonsRegistrationFilter_GetSmoothUpdateField, 'SetUpdateFieldStandardDeviations' = SymmetricForcesDemonsRegistrationFilter_SetUpdateFieldStandardDeviations, 'GetUpdateFieldStandardDeviations' = SymmetricForcesDemonsRegistrationFilter_GetUpdateFieldStandardDeviations, 'SetMaximumKernelWidth' = SymmetricForcesDemonsRegistrationFilter_SetMaximumKernelWidth, 'GetMaximumKernelWidth' = SymmetricForcesDemonsRegistrationFilter_GetMaximumKernelWidth, 'SetMaximumError' = SymmetricForcesDemonsRegistrationFilter_SetMaximumError, 'GetMaximumError' = SymmetricForcesDemonsRegistrationFilter_GetMaximumError, 'SetIntensityDifferenceThreshold' = SymmetricForcesDemonsRegistrationFilter_SetIntensityDifferenceThreshold, 'GetIntensityDifferenceThreshold' = SymmetricForcesDemonsRegistrationFilter_GetIntensityDifferenceThreshold, 'SetUseImageSpacing' = SymmetricForcesDemonsRegistrationFilter_SetUseImageSpacing, 'UseImageSpacingOn' = SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOn, 'UseImageSpacingOff' = SymmetricForcesDemonsRegistrationFilter_UseImageSpacingOff, 'GetUseImageSpacing' = SymmetricForcesDemonsRegistrationFilter_GetUseImageSpacing, 'GetElapsedIterations' = SymmetricForcesDemonsRegistrationFilter_GetElapsedIterations, 'GetRMSChange' = SymmetricForcesDemonsRegistrationFilter_GetRMSChange, 'GetMetric' = SymmetricForcesDemonsRegistrationFilter_GetMetric, 'StopRegistration' = SymmetricForcesDemonsRegistrationFilter_StopRegistration, 'GetName' = SymmetricForcesDemonsRegistrationFilter_GetName, 'ToString' = SymmetricForcesDemonsRegistrationFilter_ToString, 'Execute' = SymmetricForcesDemonsRegistrationFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::SymmetricForcesDemonsRegistrationFilter
# Start of delete_TanImageFilter

`delete_TanImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_TanImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_TanImageFilter`, 'returnType') = 'void'
attr(`delete_TanImageFilter`, "inputTypes") = c('_p_itk__simple__TanImageFilter')
class(`delete_TanImageFilter`) = c("SWIGFunction", class('delete_TanImageFilter'))

# Start of new_TanImageFilter

`TanImageFilter` = function()
{
  ;ans = .Call('R_swig_new_TanImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__TanImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_TanImageFilter);
  ans
  
}

attr(`TanImageFilter`, 'returnType') = '_p_itk__simple__TanImageFilter'
class(`TanImageFilter`) = c("SWIGFunction", class('TanImageFilter'))

# Start of TanImageFilter_GetName

`TanImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TanImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TanImageFilter_GetName`, 'returnType') = 'character'
attr(`TanImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__TanImageFilter')
class(`TanImageFilter_GetName`) = c("SWIGFunction", class('TanImageFilter_GetName'))

# Start of TanImageFilter_ToString

`TanImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TanImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TanImageFilter_ToString`, 'returnType') = 'character'
attr(`TanImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__TanImageFilter')
class(`TanImageFilter_ToString`) = c("SWIGFunction", class('TanImageFilter_ToString'))

# Start of TanImageFilter_Execute

`TanImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_TanImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TanImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`TanImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__TanImageFilter', '_p_itk__simple__Image')
class(`TanImageFilter_Execute`) = c("SWIGFunction", class('TanImageFilter_Execute'))

# Start of accessor method for itk::simple::TanImageFilter
setMethod('$', '_p_itk__simple__TanImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = TanImageFilter_GetName, 'ToString' = TanImageFilter_ToString, 'Execute' = TanImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::TanImageFilter
# Start of Tan

`Tan` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Tan', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tan`, 'returnType') = '_p_itk__simple__Image'
attr(`Tan`, "inputTypes") = c('_p_itk__simple__Image')
class(`Tan`) = c("SWIGFunction", class('Tan'))

# Start of delete_TernaryAddImageFilter

`delete_TernaryAddImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_TernaryAddImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_TernaryAddImageFilter`, 'returnType') = 'void'
attr(`delete_TernaryAddImageFilter`, "inputTypes") = c('_p_itk__simple__TernaryAddImageFilter')
class(`delete_TernaryAddImageFilter`) = c("SWIGFunction", class('delete_TernaryAddImageFilter'))

# Start of new_TernaryAddImageFilter

`TernaryAddImageFilter` = function()
{
  ;ans = .Call('R_swig_new_TernaryAddImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__TernaryAddImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_TernaryAddImageFilter);
  ans
  
}

attr(`TernaryAddImageFilter`, 'returnType') = '_p_itk__simple__TernaryAddImageFilter'
class(`TernaryAddImageFilter`) = c("SWIGFunction", class('TernaryAddImageFilter'))

# Start of TernaryAddImageFilter_GetName

`TernaryAddImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TernaryAddImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TernaryAddImageFilter_GetName`, 'returnType') = 'character'
attr(`TernaryAddImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__TernaryAddImageFilter')
class(`TernaryAddImageFilter_GetName`) = c("SWIGFunction", class('TernaryAddImageFilter_GetName'))

# Start of TernaryAddImageFilter_ToString

`TernaryAddImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TernaryAddImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TernaryAddImageFilter_ToString`, 'returnType') = 'character'
attr(`TernaryAddImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__TernaryAddImageFilter')
class(`TernaryAddImageFilter_ToString`) = c("SWIGFunction", class('TernaryAddImageFilter_ToString'))

# Start of TernaryAddImageFilter_Execute

`TernaryAddImageFilter_Execute` = function(self, image1, image2, image3, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_TernaryAddImageFilter_Execute', self, image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TernaryAddImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`TernaryAddImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__TernaryAddImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`TernaryAddImageFilter_Execute`) = c("SWIGFunction", class('TernaryAddImageFilter_Execute'))

# Start of accessor method for itk::simple::TernaryAddImageFilter
setMethod('$', '_p_itk__simple__TernaryAddImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = TernaryAddImageFilter_GetName, 'ToString' = TernaryAddImageFilter_ToString, 'Execute' = TernaryAddImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::TernaryAddImageFilter
# Start of TernaryAdd

`TernaryAdd` = function(image1, image2, image3, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_TernaryAdd', image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TernaryAdd`, 'returnType') = '_p_itk__simple__Image'
attr(`TernaryAdd`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`TernaryAdd`) = c("SWIGFunction", class('TernaryAdd'))

# Start of delete_TernaryMagnitudeImageFilter

`delete_TernaryMagnitudeImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_TernaryMagnitudeImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_TernaryMagnitudeImageFilter`, 'returnType') = 'void'
attr(`delete_TernaryMagnitudeImageFilter`, "inputTypes") = c('_p_itk__simple__TernaryMagnitudeImageFilter')
class(`delete_TernaryMagnitudeImageFilter`) = c("SWIGFunction", class('delete_TernaryMagnitudeImageFilter'))

# Start of new_TernaryMagnitudeImageFilter

`TernaryMagnitudeImageFilter` = function()
{
  ;ans = .Call('R_swig_new_TernaryMagnitudeImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__TernaryMagnitudeImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_TernaryMagnitudeImageFilter);
  ans
  
}

attr(`TernaryMagnitudeImageFilter`, 'returnType') = '_p_itk__simple__TernaryMagnitudeImageFilter'
class(`TernaryMagnitudeImageFilter`) = c("SWIGFunction", class('TernaryMagnitudeImageFilter'))

# Start of TernaryMagnitudeImageFilter_GetName

`TernaryMagnitudeImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TernaryMagnitudeImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TernaryMagnitudeImageFilter_GetName`, 'returnType') = 'character'
attr(`TernaryMagnitudeImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__TernaryMagnitudeImageFilter')
class(`TernaryMagnitudeImageFilter_GetName`) = c("SWIGFunction", class('TernaryMagnitudeImageFilter_GetName'))

# Start of TernaryMagnitudeImageFilter_ToString

`TernaryMagnitudeImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TernaryMagnitudeImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TernaryMagnitudeImageFilter_ToString`, 'returnType') = 'character'
attr(`TernaryMagnitudeImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__TernaryMagnitudeImageFilter')
class(`TernaryMagnitudeImageFilter_ToString`) = c("SWIGFunction", class('TernaryMagnitudeImageFilter_ToString'))

# Start of TernaryMagnitudeImageFilter_Execute

`TernaryMagnitudeImageFilter_Execute` = function(self, image1, image2, image3, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_TernaryMagnitudeImageFilter_Execute', self, image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TernaryMagnitudeImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`TernaryMagnitudeImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__TernaryMagnitudeImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`TernaryMagnitudeImageFilter_Execute`) = c("SWIGFunction", class('TernaryMagnitudeImageFilter_Execute'))

# Start of accessor method for itk::simple::TernaryMagnitudeImageFilter
setMethod('$', '_p_itk__simple__TernaryMagnitudeImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = TernaryMagnitudeImageFilter_GetName, 'ToString' = TernaryMagnitudeImageFilter_ToString, 'Execute' = TernaryMagnitudeImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::TernaryMagnitudeImageFilter
# Start of TernaryMagnitude

`TernaryMagnitude` = function(image1, image2, image3, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_TernaryMagnitude', image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TernaryMagnitude`, 'returnType') = '_p_itk__simple__Image'
attr(`TernaryMagnitude`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`TernaryMagnitude`) = c("SWIGFunction", class('TernaryMagnitude'))

# Start of delete_TernaryMagnitudeSquaredImageFilter

`delete_TernaryMagnitudeSquaredImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_TernaryMagnitudeSquaredImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_TernaryMagnitudeSquaredImageFilter`, 'returnType') = 'void'
attr(`delete_TernaryMagnitudeSquaredImageFilter`, "inputTypes") = c('_p_itk__simple__TernaryMagnitudeSquaredImageFilter')
class(`delete_TernaryMagnitudeSquaredImageFilter`) = c("SWIGFunction", class('delete_TernaryMagnitudeSquaredImageFilter'))

# Start of new_TernaryMagnitudeSquaredImageFilter

`TernaryMagnitudeSquaredImageFilter` = function()
{
  ;ans = .Call('R_swig_new_TernaryMagnitudeSquaredImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__TernaryMagnitudeSquaredImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_TernaryMagnitudeSquaredImageFilter);
  ans
  
}

attr(`TernaryMagnitudeSquaredImageFilter`, 'returnType') = '_p_itk__simple__TernaryMagnitudeSquaredImageFilter'
class(`TernaryMagnitudeSquaredImageFilter`) = c("SWIGFunction", class('TernaryMagnitudeSquaredImageFilter'))

# Start of TernaryMagnitudeSquaredImageFilter_GetName

`TernaryMagnitudeSquaredImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TernaryMagnitudeSquaredImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TernaryMagnitudeSquaredImageFilter_GetName`, 'returnType') = 'character'
attr(`TernaryMagnitudeSquaredImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__TernaryMagnitudeSquaredImageFilter')
class(`TernaryMagnitudeSquaredImageFilter_GetName`) = c("SWIGFunction", class('TernaryMagnitudeSquaredImageFilter_GetName'))

# Start of TernaryMagnitudeSquaredImageFilter_ToString

`TernaryMagnitudeSquaredImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TernaryMagnitudeSquaredImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TernaryMagnitudeSquaredImageFilter_ToString`, 'returnType') = 'character'
attr(`TernaryMagnitudeSquaredImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__TernaryMagnitudeSquaredImageFilter')
class(`TernaryMagnitudeSquaredImageFilter_ToString`) = c("SWIGFunction", class('TernaryMagnitudeSquaredImageFilter_ToString'))

# Start of TernaryMagnitudeSquaredImageFilter_Execute

`TernaryMagnitudeSquaredImageFilter_Execute` = function(self, image1, image2, image3, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_TernaryMagnitudeSquaredImageFilter_Execute', self, image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TernaryMagnitudeSquaredImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`TernaryMagnitudeSquaredImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__TernaryMagnitudeSquaredImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`TernaryMagnitudeSquaredImageFilter_Execute`) = c("SWIGFunction", class('TernaryMagnitudeSquaredImageFilter_Execute'))

# Start of accessor method for itk::simple::TernaryMagnitudeSquaredImageFilter
setMethod('$', '_p_itk__simple__TernaryMagnitudeSquaredImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = TernaryMagnitudeSquaredImageFilter_GetName, 'ToString' = TernaryMagnitudeSquaredImageFilter_ToString, 'Execute' = TernaryMagnitudeSquaredImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::TernaryMagnitudeSquaredImageFilter
# Start of TernaryMagnitudeSquared

`TernaryMagnitudeSquared` = function(image1, image2, image3, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_TernaryMagnitudeSquared', image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TernaryMagnitudeSquared`, 'returnType') = '_p_itk__simple__Image'
attr(`TernaryMagnitudeSquared`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`TernaryMagnitudeSquared`) = c("SWIGFunction", class('TernaryMagnitudeSquared'))

# Start of delete_ThresholdImageFilter

`delete_ThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_ThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__ThresholdImageFilter')
class(`delete_ThresholdImageFilter`) = c("SWIGFunction", class('delete_ThresholdImageFilter'))

# Start of new_ThresholdImageFilter

`ThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ThresholdImageFilter);
  ans
  
}

attr(`ThresholdImageFilter`, 'returnType') = '_p_itk__simple__ThresholdImageFilter'
class(`ThresholdImageFilter`) = c("SWIGFunction", class('ThresholdImageFilter'))

# Start of ThresholdImageFilter_SetLower

`ThresholdImageFilter_SetLower` = function(self, Lower)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ThresholdImageFilter_SetLower', self, Lower, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdImageFilter_SetLower`, 'returnType') = 'void'
attr(`ThresholdImageFilter_SetLower`, "inputTypes") = c('_p_itk__simple__ThresholdImageFilter', 'numeric')
class(`ThresholdImageFilter_SetLower`) = c("SWIGFunction", class('ThresholdImageFilter_SetLower'))

# Start of ThresholdImageFilter_GetLower

`ThresholdImageFilter_GetLower` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdImageFilter_GetLower', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdImageFilter_GetLower`, 'returnType') = 'numeric'
attr(`ThresholdImageFilter_GetLower`, "inputTypes") = c('_p_itk__simple__ThresholdImageFilter')
class(`ThresholdImageFilter_GetLower`) = c("SWIGFunction", class('ThresholdImageFilter_GetLower'))

# Start of ThresholdImageFilter_SetUpper

`ThresholdImageFilter_SetUpper` = function(self, Upper)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ThresholdImageFilter_SetUpper', self, Upper, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdImageFilter_SetUpper`, 'returnType') = 'void'
attr(`ThresholdImageFilter_SetUpper`, "inputTypes") = c('_p_itk__simple__ThresholdImageFilter', 'numeric')
class(`ThresholdImageFilter_SetUpper`) = c("SWIGFunction", class('ThresholdImageFilter_SetUpper'))

# Start of ThresholdImageFilter_GetUpper

`ThresholdImageFilter_GetUpper` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdImageFilter_GetUpper', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdImageFilter_GetUpper`, 'returnType') = 'numeric'
attr(`ThresholdImageFilter_GetUpper`, "inputTypes") = c('_p_itk__simple__ThresholdImageFilter')
class(`ThresholdImageFilter_GetUpper`) = c("SWIGFunction", class('ThresholdImageFilter_GetUpper'))

# Start of ThresholdImageFilter_SetOutsideValue

`ThresholdImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ThresholdImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`ThresholdImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__ThresholdImageFilter', 'numeric')
class(`ThresholdImageFilter_SetOutsideValue`) = c("SWIGFunction", class('ThresholdImageFilter_SetOutsideValue'))

# Start of ThresholdImageFilter_GetOutsideValue

`ThresholdImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdImageFilter_GetOutsideValue`, 'returnType') = 'numeric'
attr(`ThresholdImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__ThresholdImageFilter')
class(`ThresholdImageFilter_GetOutsideValue`) = c("SWIGFunction", class('ThresholdImageFilter_GetOutsideValue'))

# Start of ThresholdImageFilter_GetName

`ThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`ThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ThresholdImageFilter')
class(`ThresholdImageFilter_GetName`) = c("SWIGFunction", class('ThresholdImageFilter_GetName'))

# Start of ThresholdImageFilter_ToString

`ThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`ThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ThresholdImageFilter')
class(`ThresholdImageFilter_ToString`) = c("SWIGFunction", class('ThresholdImageFilter_ToString'))

# Start of ThresholdImageFilter_Execute

`ThresholdImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ThresholdImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ThresholdImageFilter', '_p_itk__simple__Image')
class(`ThresholdImageFilter_Execute`) = c("SWIGFunction", class('ThresholdImageFilter_Execute'))

# Start of ThresholdImageFilter_ThresholdAbove

`ThresholdImageFilter_ThresholdAbove` = function(self, threshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ThresholdImageFilter_ThresholdAbove', self, threshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdImageFilter_ThresholdAbove`, 'returnType') = 'void'
attr(`ThresholdImageFilter_ThresholdAbove`, "inputTypes") = c('_p_itk__simple__ThresholdImageFilter', 'numeric')
class(`ThresholdImageFilter_ThresholdAbove`) = c("SWIGFunction", class('ThresholdImageFilter_ThresholdAbove'))

# Start of ThresholdImageFilter_ThresholdBelow

`ThresholdImageFilter_ThresholdBelow` = function(self, threshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ThresholdImageFilter_ThresholdBelow', self, threshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdImageFilter_ThresholdBelow`, 'returnType') = 'void'
attr(`ThresholdImageFilter_ThresholdBelow`, "inputTypes") = c('_p_itk__simple__ThresholdImageFilter', 'numeric')
class(`ThresholdImageFilter_ThresholdBelow`) = c("SWIGFunction", class('ThresholdImageFilter_ThresholdBelow'))

# Start of ThresholdImageFilter_ThresholdOutside

`ThresholdImageFilter_ThresholdOutside` = function(self, lower, upper)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  
  ;ans = .Call('R_swig_ThresholdImageFilter_ThresholdOutside', self, lower, upper, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdImageFilter_ThresholdOutside`, 'returnType') = 'void'
attr(`ThresholdImageFilter_ThresholdOutside`, "inputTypes") = c('_p_itk__simple__ThresholdImageFilter', 'numeric', 'numeric')
class(`ThresholdImageFilter_ThresholdOutside`) = c("SWIGFunction", class('ThresholdImageFilter_ThresholdOutside'))

# Start of accessor method for itk::simple::ThresholdImageFilter
setMethod('$', '_p_itk__simple__ThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetLower' = ThresholdImageFilter_SetLower, 'GetLower' = ThresholdImageFilter_GetLower, 'SetUpper' = ThresholdImageFilter_SetUpper, 'GetUpper' = ThresholdImageFilter_GetUpper, 'SetOutsideValue' = ThresholdImageFilter_SetOutsideValue, 'GetOutsideValue' = ThresholdImageFilter_GetOutsideValue, 'GetName' = ThresholdImageFilter_GetName, 'ToString' = ThresholdImageFilter_ToString, 'Execute' = ThresholdImageFilter_Execute, 'ThresholdAbove' = ThresholdImageFilter_ThresholdAbove, 'ThresholdBelow' = ThresholdImageFilter_ThresholdBelow, 'ThresholdOutside' = ThresholdImageFilter_ThresholdOutside);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ThresholdImageFilter
# Start of Threshold

`Threshold__SWIG_0` = function(image1, lower, upper, outsideValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  
  ;ans = .Call('R_swig_Threshold__SWIG_0', image1, lower, upper, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Threshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Threshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric')
class(`Threshold__SWIG_0`) = c("SWIGFunction", class('Threshold__SWIG_0'))

# Start of Threshold

`Threshold__SWIG_1` = function(image1, lower, upper, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  
  ;ans = .Call('R_swig_Threshold__SWIG_1', image1, lower, upper, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Threshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Threshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`Threshold__SWIG_1`) = c("SWIGFunction", class('Threshold__SWIG_1'))

# Start of Threshold

`Threshold__SWIG_2` = function(image1, lower, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_Threshold__SWIG_2', image1, lower, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Threshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Threshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`Threshold__SWIG_2`) = c("SWIGFunction", class('Threshold__SWIG_2'))

# Start of Threshold

`Threshold__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Threshold__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Threshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Threshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`Threshold__SWIG_3`) = c("SWIGFunction", class('Threshold__SWIG_3'))

`Threshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Threshold__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- Threshold__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Threshold__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- Threshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Threshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ThresholdMaximumConnectedComponentsImageFilter

`delete_ThresholdMaximumConnectedComponentsImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ThresholdMaximumConnectedComponentsImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ThresholdMaximumConnectedComponentsImageFilter`, 'returnType') = 'void'
attr(`delete_ThresholdMaximumConnectedComponentsImageFilter`, "inputTypes") = c('_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter')
class(`delete_ThresholdMaximumConnectedComponentsImageFilter`) = c("SWIGFunction", class('delete_ThresholdMaximumConnectedComponentsImageFilter'))

# Start of new_ThresholdMaximumConnectedComponentsImageFilter

`ThresholdMaximumConnectedComponentsImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ThresholdMaximumConnectedComponentsImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ThresholdMaximumConnectedComponentsImageFilter);
  ans
  
}

attr(`ThresholdMaximumConnectedComponentsImageFilter`, 'returnType') = '_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter'
class(`ThresholdMaximumConnectedComponentsImageFilter`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponentsImageFilter'))

# Start of ThresholdMaximumConnectedComponentsImageFilter_SetMinimumObjectSizeInPixels

`ThresholdMaximumConnectedComponentsImageFilter_SetMinimumObjectSizeInPixels` = function(self, MinimumObjectSizeInPixels)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MinimumObjectSizeInPixels = as.integer(MinimumObjectSizeInPixels);
  
  if(length(MinimumObjectSizeInPixels) > 1) {
    warning("using only the first element of MinimumObjectSizeInPixels");
  };
  
  ;ans = .Call('R_swig_ThresholdMaximumConnectedComponentsImageFilter_SetMinimumObjectSizeInPixels', self, MinimumObjectSizeInPixels, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdMaximumConnectedComponentsImageFilter_SetMinimumObjectSizeInPixels`, 'returnType') = 'void'
attr(`ThresholdMaximumConnectedComponentsImageFilter_SetMinimumObjectSizeInPixels`, "inputTypes") = c('_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter', 'integer')
class(`ThresholdMaximumConnectedComponentsImageFilter_SetMinimumObjectSizeInPixels`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponentsImageFilter_SetMinimumObjectSizeInPixels'))

# Start of ThresholdMaximumConnectedComponentsImageFilter_GetMinimumObjectSizeInPixels

`ThresholdMaximumConnectedComponentsImageFilter_GetMinimumObjectSizeInPixels` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdMaximumConnectedComponentsImageFilter_GetMinimumObjectSizeInPixels', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdMaximumConnectedComponentsImageFilter_GetMinimumObjectSizeInPixels`, 'returnType') = 'integer'
attr(`ThresholdMaximumConnectedComponentsImageFilter_GetMinimumObjectSizeInPixels`, "inputTypes") = c('_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter')
class(`ThresholdMaximumConnectedComponentsImageFilter_GetMinimumObjectSizeInPixels`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponentsImageFilter_GetMinimumObjectSizeInPixels'))

# Start of ThresholdMaximumConnectedComponentsImageFilter_SetUpperBoundary

`ThresholdMaximumConnectedComponentsImageFilter_SetUpperBoundary` = function(self, UpperBoundary)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ThresholdMaximumConnectedComponentsImageFilter_SetUpperBoundary', self, UpperBoundary, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdMaximumConnectedComponentsImageFilter_SetUpperBoundary`, 'returnType') = 'void'
attr(`ThresholdMaximumConnectedComponentsImageFilter_SetUpperBoundary`, "inputTypes") = c('_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter', 'numeric')
class(`ThresholdMaximumConnectedComponentsImageFilter_SetUpperBoundary`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponentsImageFilter_SetUpperBoundary'))

# Start of ThresholdMaximumConnectedComponentsImageFilter_GetUpperBoundary

`ThresholdMaximumConnectedComponentsImageFilter_GetUpperBoundary` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdMaximumConnectedComponentsImageFilter_GetUpperBoundary', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdMaximumConnectedComponentsImageFilter_GetUpperBoundary`, 'returnType') = 'numeric'
attr(`ThresholdMaximumConnectedComponentsImageFilter_GetUpperBoundary`, "inputTypes") = c('_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter')
class(`ThresholdMaximumConnectedComponentsImageFilter_GetUpperBoundary`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponentsImageFilter_GetUpperBoundary'))

# Start of ThresholdMaximumConnectedComponentsImageFilter_SetInsideValue

`ThresholdMaximumConnectedComponentsImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideValue = as.integer(InsideValue);
  
  if(length(InsideValue) > 1) {
    warning("using only the first element of InsideValue");
  };
  
  ;ans = .Call('R_swig_ThresholdMaximumConnectedComponentsImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdMaximumConnectedComponentsImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`ThresholdMaximumConnectedComponentsImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter', 'integer')
class(`ThresholdMaximumConnectedComponentsImageFilter_SetInsideValue`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponentsImageFilter_SetInsideValue'))

# Start of ThresholdMaximumConnectedComponentsImageFilter_GetInsideValue

`ThresholdMaximumConnectedComponentsImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdMaximumConnectedComponentsImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdMaximumConnectedComponentsImageFilter_GetInsideValue`, 'returnType') = 'integer'
attr(`ThresholdMaximumConnectedComponentsImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter')
class(`ThresholdMaximumConnectedComponentsImageFilter_GetInsideValue`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponentsImageFilter_GetInsideValue'))

# Start of ThresholdMaximumConnectedComponentsImageFilter_SetOutsideValue

`ThresholdMaximumConnectedComponentsImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutsideValue = as.integer(OutsideValue);
  
  if(length(OutsideValue) > 1) {
    warning("using only the first element of OutsideValue");
  };
  
  ;ans = .Call('R_swig_ThresholdMaximumConnectedComponentsImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdMaximumConnectedComponentsImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`ThresholdMaximumConnectedComponentsImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter', 'integer')
class(`ThresholdMaximumConnectedComponentsImageFilter_SetOutsideValue`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponentsImageFilter_SetOutsideValue'))

# Start of ThresholdMaximumConnectedComponentsImageFilter_GetOutsideValue

`ThresholdMaximumConnectedComponentsImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdMaximumConnectedComponentsImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdMaximumConnectedComponentsImageFilter_GetOutsideValue`, 'returnType') = 'integer'
attr(`ThresholdMaximumConnectedComponentsImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter')
class(`ThresholdMaximumConnectedComponentsImageFilter_GetOutsideValue`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponentsImageFilter_GetOutsideValue'))

# Start of ThresholdMaximumConnectedComponentsImageFilter_GetName

`ThresholdMaximumConnectedComponentsImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdMaximumConnectedComponentsImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdMaximumConnectedComponentsImageFilter_GetName`, 'returnType') = 'character'
attr(`ThresholdMaximumConnectedComponentsImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter')
class(`ThresholdMaximumConnectedComponentsImageFilter_GetName`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponentsImageFilter_GetName'))

# Start of ThresholdMaximumConnectedComponentsImageFilter_ToString

`ThresholdMaximumConnectedComponentsImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdMaximumConnectedComponentsImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdMaximumConnectedComponentsImageFilter_ToString`, 'returnType') = 'character'
attr(`ThresholdMaximumConnectedComponentsImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter')
class(`ThresholdMaximumConnectedComponentsImageFilter_ToString`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponentsImageFilter_ToString'))

# Start of ThresholdMaximumConnectedComponentsImageFilter_Execute

`ThresholdMaximumConnectedComponentsImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ThresholdMaximumConnectedComponentsImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdMaximumConnectedComponentsImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdMaximumConnectedComponentsImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter', '_p_itk__simple__Image')
class(`ThresholdMaximumConnectedComponentsImageFilter_Execute`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponentsImageFilter_Execute'))

# Start of accessor method for itk::simple::ThresholdMaximumConnectedComponentsImageFilter
setMethod('$', '_p_itk__simple__ThresholdMaximumConnectedComponentsImageFilter', function(x, name)

{
  accessorFuns = list('SetMinimumObjectSizeInPixels' = ThresholdMaximumConnectedComponentsImageFilter_SetMinimumObjectSizeInPixels, 'GetMinimumObjectSizeInPixels' = ThresholdMaximumConnectedComponentsImageFilter_GetMinimumObjectSizeInPixels, 'SetUpperBoundary' = ThresholdMaximumConnectedComponentsImageFilter_SetUpperBoundary, 'GetUpperBoundary' = ThresholdMaximumConnectedComponentsImageFilter_GetUpperBoundary, 'SetInsideValue' = ThresholdMaximumConnectedComponentsImageFilter_SetInsideValue, 'GetInsideValue' = ThresholdMaximumConnectedComponentsImageFilter_GetInsideValue, 'SetOutsideValue' = ThresholdMaximumConnectedComponentsImageFilter_SetOutsideValue, 'GetOutsideValue' = ThresholdMaximumConnectedComponentsImageFilter_GetOutsideValue, 'GetName' = ThresholdMaximumConnectedComponentsImageFilter_GetName, 'ToString' = ThresholdMaximumConnectedComponentsImageFilter_ToString, 'Execute' = ThresholdMaximumConnectedComponentsImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ThresholdMaximumConnectedComponentsImageFilter
# Start of ThresholdMaximumConnectedComponents

`ThresholdMaximumConnectedComponents__SWIG_0` = function(image1, minimumObjectSizeInPixels, upperBoundary, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  minimumObjectSizeInPixels = as.integer(minimumObjectSizeInPixels);
  
  if(length(minimumObjectSizeInPixels) > 1) {
    warning("using only the first element of minimumObjectSizeInPixels");
  };
  
  
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_ThresholdMaximumConnectedComponents__SWIG_0', image1, minimumObjectSizeInPixels, upperBoundary, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdMaximumConnectedComponents__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdMaximumConnectedComponents__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'integer', 'integer')
class(`ThresholdMaximumConnectedComponents__SWIG_0`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponents__SWIG_0'))

# Start of ThresholdMaximumConnectedComponents

`ThresholdMaximumConnectedComponents__SWIG_1` = function(image1, minimumObjectSizeInPixels, upperBoundary, insideValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  minimumObjectSizeInPixels = as.integer(minimumObjectSizeInPixels);
  
  if(length(minimumObjectSizeInPixels) > 1) {
    warning("using only the first element of minimumObjectSizeInPixels");
  };
  
  
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_ThresholdMaximumConnectedComponents__SWIG_1', image1, minimumObjectSizeInPixels, upperBoundary, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdMaximumConnectedComponents__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdMaximumConnectedComponents__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric', 'integer')
class(`ThresholdMaximumConnectedComponents__SWIG_1`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponents__SWIG_1'))

# Start of ThresholdMaximumConnectedComponents

`ThresholdMaximumConnectedComponents__SWIG_2` = function(image1, minimumObjectSizeInPixels, upperBoundary, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  minimumObjectSizeInPixels = as.integer(minimumObjectSizeInPixels);
  
  if(length(minimumObjectSizeInPixels) > 1) {
    warning("using only the first element of minimumObjectSizeInPixels");
  };
  
  
  ;ans = .Call('R_swig_ThresholdMaximumConnectedComponents__SWIG_2', image1, minimumObjectSizeInPixels, upperBoundary, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdMaximumConnectedComponents__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdMaximumConnectedComponents__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric')
class(`ThresholdMaximumConnectedComponents__SWIG_2`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponents__SWIG_2'))

# Start of ThresholdMaximumConnectedComponents

`ThresholdMaximumConnectedComponents__SWIG_3` = function(image1, minimumObjectSizeInPixels, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  minimumObjectSizeInPixels = as.integer(minimumObjectSizeInPixels);
  
  if(length(minimumObjectSizeInPixels) > 1) {
    warning("using only the first element of minimumObjectSizeInPixels");
  };
  
  ;ans = .Call('R_swig_ThresholdMaximumConnectedComponents__SWIG_3', image1, minimumObjectSizeInPixels, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdMaximumConnectedComponents__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdMaximumConnectedComponents__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`ThresholdMaximumConnectedComponents__SWIG_3`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponents__SWIG_3'))

# Start of ThresholdMaximumConnectedComponents

`ThresholdMaximumConnectedComponents__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ThresholdMaximumConnectedComponents__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdMaximumConnectedComponents__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdMaximumConnectedComponents__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`ThresholdMaximumConnectedComponents__SWIG_4`) = c("SWIGFunction", class('ThresholdMaximumConnectedComponents__SWIG_4'))

`ThresholdMaximumConnectedComponents` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ThresholdMaximumConnectedComponents__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- ThresholdMaximumConnectedComponents__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ThresholdMaximumConnectedComponents__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- ThresholdMaximumConnectedComponents__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- ThresholdMaximumConnectedComponents__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ThresholdMaximumConnectedComponents with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ThresholdSegmentationLevelSetImageFilter

`delete_ThresholdSegmentationLevelSetImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ThresholdSegmentationLevelSetImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ThresholdSegmentationLevelSetImageFilter`, 'returnType') = 'void'
attr(`delete_ThresholdSegmentationLevelSetImageFilter`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter')
class(`delete_ThresholdSegmentationLevelSetImageFilter`) = c("SWIGFunction", class('delete_ThresholdSegmentationLevelSetImageFilter'))

# Start of new_ThresholdSegmentationLevelSetImageFilter

`ThresholdSegmentationLevelSetImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ThresholdSegmentationLevelSetImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ThresholdSegmentationLevelSetImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ThresholdSegmentationLevelSetImageFilter);
  ans
  
}

attr(`ThresholdSegmentationLevelSetImageFilter`, 'returnType') = '_p_itk__simple__ThresholdSegmentationLevelSetImageFilter'
class(`ThresholdSegmentationLevelSetImageFilter`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter'))

# Start of ThresholdSegmentationLevelSetImageFilter_SetLowerThreshold

`ThresholdSegmentationLevelSetImageFilter_SetLowerThreshold` = function(self, LowerThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSetImageFilter_SetLowerThreshold', self, LowerThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_SetLowerThreshold`, 'returnType') = 'void'
attr(`ThresholdSegmentationLevelSetImageFilter_SetLowerThreshold`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter', 'numeric')
class(`ThresholdSegmentationLevelSetImageFilter_SetLowerThreshold`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_SetLowerThreshold'))

# Start of ThresholdSegmentationLevelSetImageFilter_GetLowerThreshold

`ThresholdSegmentationLevelSetImageFilter_GetLowerThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdSegmentationLevelSetImageFilter_GetLowerThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_GetLowerThreshold`, 'returnType') = 'numeric'
attr(`ThresholdSegmentationLevelSetImageFilter_GetLowerThreshold`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter')
class(`ThresholdSegmentationLevelSetImageFilter_GetLowerThreshold`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_GetLowerThreshold'))

# Start of ThresholdSegmentationLevelSetImageFilter_SetUpperThreshold

`ThresholdSegmentationLevelSetImageFilter_SetUpperThreshold` = function(self, UpperThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSetImageFilter_SetUpperThreshold', self, UpperThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_SetUpperThreshold`, 'returnType') = 'void'
attr(`ThresholdSegmentationLevelSetImageFilter_SetUpperThreshold`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter', 'numeric')
class(`ThresholdSegmentationLevelSetImageFilter_SetUpperThreshold`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_SetUpperThreshold'))

# Start of ThresholdSegmentationLevelSetImageFilter_GetUpperThreshold

`ThresholdSegmentationLevelSetImageFilter_GetUpperThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdSegmentationLevelSetImageFilter_GetUpperThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_GetUpperThreshold`, 'returnType') = 'numeric'
attr(`ThresholdSegmentationLevelSetImageFilter_GetUpperThreshold`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter')
class(`ThresholdSegmentationLevelSetImageFilter_GetUpperThreshold`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_GetUpperThreshold'))

# Start of ThresholdSegmentationLevelSetImageFilter_SetMaximumRMSError

`ThresholdSegmentationLevelSetImageFilter_SetMaximumRMSError` = function(self, MaximumRMSError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSetImageFilter_SetMaximumRMSError', self, MaximumRMSError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_SetMaximumRMSError`, 'returnType') = 'void'
attr(`ThresholdSegmentationLevelSetImageFilter_SetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter', 'numeric')
class(`ThresholdSegmentationLevelSetImageFilter_SetMaximumRMSError`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_SetMaximumRMSError'))

# Start of ThresholdSegmentationLevelSetImageFilter_GetMaximumRMSError

`ThresholdSegmentationLevelSetImageFilter_GetMaximumRMSError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdSegmentationLevelSetImageFilter_GetMaximumRMSError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_GetMaximumRMSError`, 'returnType') = 'numeric'
attr(`ThresholdSegmentationLevelSetImageFilter_GetMaximumRMSError`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter')
class(`ThresholdSegmentationLevelSetImageFilter_GetMaximumRMSError`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_GetMaximumRMSError'))

# Start of ThresholdSegmentationLevelSetImageFilter_SetPropagationScaling

`ThresholdSegmentationLevelSetImageFilter_SetPropagationScaling` = function(self, PropagationScaling)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSetImageFilter_SetPropagationScaling', self, PropagationScaling, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_SetPropagationScaling`, 'returnType') = 'void'
attr(`ThresholdSegmentationLevelSetImageFilter_SetPropagationScaling`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter', 'numeric')
class(`ThresholdSegmentationLevelSetImageFilter_SetPropagationScaling`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_SetPropagationScaling'))

# Start of ThresholdSegmentationLevelSetImageFilter_GetPropagationScaling

`ThresholdSegmentationLevelSetImageFilter_GetPropagationScaling` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdSegmentationLevelSetImageFilter_GetPropagationScaling', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_GetPropagationScaling`, 'returnType') = 'numeric'
attr(`ThresholdSegmentationLevelSetImageFilter_GetPropagationScaling`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter')
class(`ThresholdSegmentationLevelSetImageFilter_GetPropagationScaling`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_GetPropagationScaling'))

# Start of ThresholdSegmentationLevelSetImageFilter_SetCurvatureScaling

`ThresholdSegmentationLevelSetImageFilter_SetCurvatureScaling` = function(self, CurvatureScaling)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSetImageFilter_SetCurvatureScaling', self, CurvatureScaling, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_SetCurvatureScaling`, 'returnType') = 'void'
attr(`ThresholdSegmentationLevelSetImageFilter_SetCurvatureScaling`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter', 'numeric')
class(`ThresholdSegmentationLevelSetImageFilter_SetCurvatureScaling`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_SetCurvatureScaling'))

# Start of ThresholdSegmentationLevelSetImageFilter_GetCurvatureScaling

`ThresholdSegmentationLevelSetImageFilter_GetCurvatureScaling` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdSegmentationLevelSetImageFilter_GetCurvatureScaling', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_GetCurvatureScaling`, 'returnType') = 'numeric'
attr(`ThresholdSegmentationLevelSetImageFilter_GetCurvatureScaling`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter')
class(`ThresholdSegmentationLevelSetImageFilter_GetCurvatureScaling`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_GetCurvatureScaling'))

# Start of ThresholdSegmentationLevelSetImageFilter_SetNumberOfIterations

`ThresholdSegmentationLevelSetImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSetImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`ThresholdSegmentationLevelSetImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter', 'integer')
class(`ThresholdSegmentationLevelSetImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_SetNumberOfIterations'))

# Start of ThresholdSegmentationLevelSetImageFilter_GetNumberOfIterations

`ThresholdSegmentationLevelSetImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdSegmentationLevelSetImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`ThresholdSegmentationLevelSetImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter')
class(`ThresholdSegmentationLevelSetImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_GetNumberOfIterations'))

# Start of ThresholdSegmentationLevelSetImageFilter_SetReverseExpansionDirection

`ThresholdSegmentationLevelSetImageFilter_SetReverseExpansionDirection` = function(self, ReverseExpansionDirection)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ReverseExpansionDirection = as.logical(ReverseExpansionDirection);
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSetImageFilter_SetReverseExpansionDirection', self, ReverseExpansionDirection, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_SetReverseExpansionDirection`, 'returnType') = 'void'
attr(`ThresholdSegmentationLevelSetImageFilter_SetReverseExpansionDirection`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter', 'logical')
class(`ThresholdSegmentationLevelSetImageFilter_SetReverseExpansionDirection`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_SetReverseExpansionDirection'))

# Start of ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn

`ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn`, 'returnType') = 'void'
attr(`ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter')
class(`ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn'))

# Start of ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff

`ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff`, 'returnType') = 'void'
attr(`ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter')
class(`ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff'))

# Start of ThresholdSegmentationLevelSetImageFilter_GetReverseExpansionDirection

`ThresholdSegmentationLevelSetImageFilter_GetReverseExpansionDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdSegmentationLevelSetImageFilter_GetReverseExpansionDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_GetReverseExpansionDirection`, 'returnType') = 'logical'
attr(`ThresholdSegmentationLevelSetImageFilter_GetReverseExpansionDirection`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter')
class(`ThresholdSegmentationLevelSetImageFilter_GetReverseExpansionDirection`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_GetReverseExpansionDirection'))

# Start of ThresholdSegmentationLevelSetImageFilter_GetElapsedIterations

`ThresholdSegmentationLevelSetImageFilter_GetElapsedIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdSegmentationLevelSetImageFilter_GetElapsedIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_GetElapsedIterations`, 'returnType') = 'integer'
attr(`ThresholdSegmentationLevelSetImageFilter_GetElapsedIterations`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter')
class(`ThresholdSegmentationLevelSetImageFilter_GetElapsedIterations`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_GetElapsedIterations'))

# Start of ThresholdSegmentationLevelSetImageFilter_GetRMSChange

`ThresholdSegmentationLevelSetImageFilter_GetRMSChange` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdSegmentationLevelSetImageFilter_GetRMSChange', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_GetRMSChange`, 'returnType') = 'numeric'
attr(`ThresholdSegmentationLevelSetImageFilter_GetRMSChange`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter')
class(`ThresholdSegmentationLevelSetImageFilter_GetRMSChange`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_GetRMSChange'))

# Start of ThresholdSegmentationLevelSetImageFilter_GetName

`ThresholdSegmentationLevelSetImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdSegmentationLevelSetImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_GetName`, 'returnType') = 'character'
attr(`ThresholdSegmentationLevelSetImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter')
class(`ThresholdSegmentationLevelSetImageFilter_GetName`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_GetName'))

# Start of ThresholdSegmentationLevelSetImageFilter_ToString

`ThresholdSegmentationLevelSetImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ThresholdSegmentationLevelSetImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_ToString`, 'returnType') = 'character'
attr(`ThresholdSegmentationLevelSetImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter')
class(`ThresholdSegmentationLevelSetImageFilter_ToString`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_ToString'))

# Start of ThresholdSegmentationLevelSetImageFilter_Execute

`ThresholdSegmentationLevelSetImageFilter_Execute` = function(self, initialImage, featureImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSetImageFilter_Execute', self, initialImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSetImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdSegmentationLevelSetImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ThresholdSegmentationLevelSetImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ThresholdSegmentationLevelSetImageFilter_Execute`) = c("SWIGFunction", class('ThresholdSegmentationLevelSetImageFilter_Execute'))

# Start of accessor method for itk::simple::ThresholdSegmentationLevelSetImageFilter
setMethod('$', '_p_itk__simple__ThresholdSegmentationLevelSetImageFilter', function(x, name)

{
  accessorFuns = list('SetLowerThreshold' = ThresholdSegmentationLevelSetImageFilter_SetLowerThreshold, 'GetLowerThreshold' = ThresholdSegmentationLevelSetImageFilter_GetLowerThreshold, 'SetUpperThreshold' = ThresholdSegmentationLevelSetImageFilter_SetUpperThreshold, 'GetUpperThreshold' = ThresholdSegmentationLevelSetImageFilter_GetUpperThreshold, 'SetMaximumRMSError' = ThresholdSegmentationLevelSetImageFilter_SetMaximumRMSError, 'GetMaximumRMSError' = ThresholdSegmentationLevelSetImageFilter_GetMaximumRMSError, 'SetPropagationScaling' = ThresholdSegmentationLevelSetImageFilter_SetPropagationScaling, 'GetPropagationScaling' = ThresholdSegmentationLevelSetImageFilter_GetPropagationScaling, 'SetCurvatureScaling' = ThresholdSegmentationLevelSetImageFilter_SetCurvatureScaling, 'GetCurvatureScaling' = ThresholdSegmentationLevelSetImageFilter_GetCurvatureScaling, 'SetNumberOfIterations' = ThresholdSegmentationLevelSetImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = ThresholdSegmentationLevelSetImageFilter_GetNumberOfIterations, 'SetReverseExpansionDirection' = ThresholdSegmentationLevelSetImageFilter_SetReverseExpansionDirection, 'ReverseExpansionDirectionOn' = ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOn, 'ReverseExpansionDirectionOff' = ThresholdSegmentationLevelSetImageFilter_ReverseExpansionDirectionOff, 'GetReverseExpansionDirection' = ThresholdSegmentationLevelSetImageFilter_GetReverseExpansionDirection, 'GetElapsedIterations' = ThresholdSegmentationLevelSetImageFilter_GetElapsedIterations, 'GetRMSChange' = ThresholdSegmentationLevelSetImageFilter_GetRMSChange, 'GetName' = ThresholdSegmentationLevelSetImageFilter_GetName, 'ToString' = ThresholdSegmentationLevelSetImageFilter_ToString, 'Execute' = ThresholdSegmentationLevelSetImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ThresholdSegmentationLevelSetImageFilter
# Start of ThresholdSegmentationLevelSet

`ThresholdSegmentationLevelSet__SWIG_0` = function(initialImage, featureImage, lowerThreshold, upperThreshold, maximumRMSError, propagationScaling, curvatureScaling, numberOfIterations, reverseExpansionDirection, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  reverseExpansionDirection = as.logical(reverseExpansionDirection);
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSet__SWIG_0', initialImage, featureImage, lowerThreshold, upperThreshold, maximumRMSError, propagationScaling, curvatureScaling, numberOfIterations, reverseExpansionDirection, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSet__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdSegmentationLevelSet__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'integer', 'logical')
class(`ThresholdSegmentationLevelSet__SWIG_0`) = c("SWIGFunction", class('ThresholdSegmentationLevelSet__SWIG_0'))

# Start of ThresholdSegmentationLevelSet

`ThresholdSegmentationLevelSet__SWIG_1` = function(initialImage, featureImage, lowerThreshold, upperThreshold, maximumRMSError, propagationScaling, curvatureScaling, numberOfIterations, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  
  
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSet__SWIG_1', initialImage, featureImage, lowerThreshold, upperThreshold, maximumRMSError, propagationScaling, curvatureScaling, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSet__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdSegmentationLevelSet__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric', 'integer')
class(`ThresholdSegmentationLevelSet__SWIG_1`) = c("SWIGFunction", class('ThresholdSegmentationLevelSet__SWIG_1'))

# Start of ThresholdSegmentationLevelSet

`ThresholdSegmentationLevelSet__SWIG_2` = function(initialImage, featureImage, lowerThreshold, upperThreshold, maximumRMSError, propagationScaling, curvatureScaling, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  
  
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSet__SWIG_2', initialImage, featureImage, lowerThreshold, upperThreshold, maximumRMSError, propagationScaling, curvatureScaling, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSet__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdSegmentationLevelSet__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric', 'numeric')
class(`ThresholdSegmentationLevelSet__SWIG_2`) = c("SWIGFunction", class('ThresholdSegmentationLevelSet__SWIG_2'))

# Start of ThresholdSegmentationLevelSet

`ThresholdSegmentationLevelSet__SWIG_3` = function(initialImage, featureImage, lowerThreshold, upperThreshold, maximumRMSError, propagationScaling, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSet__SWIG_3', initialImage, featureImage, lowerThreshold, upperThreshold, maximumRMSError, propagationScaling, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSet__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdSegmentationLevelSet__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'numeric')
class(`ThresholdSegmentationLevelSet__SWIG_3`) = c("SWIGFunction", class('ThresholdSegmentationLevelSet__SWIG_3'))

# Start of ThresholdSegmentationLevelSet

`ThresholdSegmentationLevelSet__SWIG_4` = function(initialImage, featureImage, lowerThreshold, upperThreshold, maximumRMSError, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSet__SWIG_4', initialImage, featureImage, lowerThreshold, upperThreshold, maximumRMSError, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSet__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdSegmentationLevelSet__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric', 'numeric')
class(`ThresholdSegmentationLevelSet__SWIG_4`) = c("SWIGFunction", class('ThresholdSegmentationLevelSet__SWIG_4'))

# Start of ThresholdSegmentationLevelSet

`ThresholdSegmentationLevelSet__SWIG_5` = function(initialImage, featureImage, lowerThreshold, upperThreshold, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSet__SWIG_5', initialImage, featureImage, lowerThreshold, upperThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSet__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdSegmentationLevelSet__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'numeric')
class(`ThresholdSegmentationLevelSet__SWIG_5`) = c("SWIGFunction", class('ThresholdSegmentationLevelSet__SWIG_5'))

# Start of ThresholdSegmentationLevelSet

`ThresholdSegmentationLevelSet__SWIG_6` = function(initialImage, featureImage, lowerThreshold, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSet__SWIG_6', initialImage, featureImage, lowerThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSet__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdSegmentationLevelSet__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`ThresholdSegmentationLevelSet__SWIG_6`) = c("SWIGFunction", class('ThresholdSegmentationLevelSet__SWIG_6'))

# Start of ThresholdSegmentationLevelSet

`ThresholdSegmentationLevelSet__SWIG_7` = function(initialImage, featureImage, .copy = FALSE)
{
  if (inherits(initialImage, "ExternalReference")) initialImage = slot(initialImage,"ref"); 
  if (inherits(featureImage, "ExternalReference")) featureImage = slot(featureImage,"ref"); 
  ;ans = .Call('R_swig_ThresholdSegmentationLevelSet__SWIG_7', initialImage, featureImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ThresholdSegmentationLevelSet__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`ThresholdSegmentationLevelSet__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`ThresholdSegmentationLevelSet__SWIG_7`) = c("SWIGFunction", class('ThresholdSegmentationLevelSet__SWIG_7'))

`ThresholdSegmentationLevelSet` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 8
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- ThresholdSegmentationLevelSet__SWIG_7; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- ThresholdSegmentationLevelSet__SWIG_6; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- ThresholdSegmentationLevelSet__SWIG_5; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- ThresholdSegmentationLevelSet__SWIG_4; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- ThresholdSegmentationLevelSet__SWIG_3; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- ThresholdSegmentationLevelSet__SWIG_2; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( (is.integer(argv[[8]]) || is.numeric(argv[[8]])) && length(argv[[8]]) == 1 )) {
      f <- ThresholdSegmentationLevelSet__SWIG_1; 
    }
  } else if (argc == 9) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 ) && ( (is.integer(argv[[8]]) || is.numeric(argv[[8]])) && length(argv[[8]]) == 1 ) && ( is.logical(argv[[9]]) && length(argv[[9]]) == 1 )) {
      f <- ThresholdSegmentationLevelSet__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ThresholdSegmentationLevelSet with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_TikhonovDeconvolutionImageFilter

`delete_TikhonovDeconvolutionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_TikhonovDeconvolutionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_TikhonovDeconvolutionImageFilter`, 'returnType') = 'void'
attr(`delete_TikhonovDeconvolutionImageFilter`, "inputTypes") = c('_p_itk__simple__TikhonovDeconvolutionImageFilter')
class(`delete_TikhonovDeconvolutionImageFilter`) = c("SWIGFunction", class('delete_TikhonovDeconvolutionImageFilter'))

# Start of new_TikhonovDeconvolutionImageFilter

`TikhonovDeconvolutionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_TikhonovDeconvolutionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__TikhonovDeconvolutionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_TikhonovDeconvolutionImageFilter);
  ans
  
}

attr(`TikhonovDeconvolutionImageFilter`, 'returnType') = '_p_itk__simple__TikhonovDeconvolutionImageFilter'
class(`TikhonovDeconvolutionImageFilter`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter'))

# Start of TikhonovDeconvolutionImageFilter_SetRegularizationConstant

`TikhonovDeconvolutionImageFilter_SetRegularizationConstant` = function(self, RegularizationConstant)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_TikhonovDeconvolutionImageFilter_SetRegularizationConstant', self, RegularizationConstant, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TikhonovDeconvolutionImageFilter_SetRegularizationConstant`, 'returnType') = 'void'
attr(`TikhonovDeconvolutionImageFilter_SetRegularizationConstant`, "inputTypes") = c('_p_itk__simple__TikhonovDeconvolutionImageFilter', 'numeric')
class(`TikhonovDeconvolutionImageFilter_SetRegularizationConstant`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_SetRegularizationConstant'))

# Start of TikhonovDeconvolutionImageFilter_GetRegularizationConstant

`TikhonovDeconvolutionImageFilter_GetRegularizationConstant` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TikhonovDeconvolutionImageFilter_GetRegularizationConstant', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TikhonovDeconvolutionImageFilter_GetRegularizationConstant`, 'returnType') = 'numeric'
attr(`TikhonovDeconvolutionImageFilter_GetRegularizationConstant`, "inputTypes") = c('_p_itk__simple__TikhonovDeconvolutionImageFilter')
class(`TikhonovDeconvolutionImageFilter_GetRegularizationConstant`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_GetRegularizationConstant'))

# Start of TikhonovDeconvolutionImageFilter_SetNormalize

`TikhonovDeconvolutionImageFilter_SetNormalize` = function(self, Normalize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Normalize = as.logical(Normalize);
  ;ans = .Call('R_swig_TikhonovDeconvolutionImageFilter_SetNormalize', self, Normalize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TikhonovDeconvolutionImageFilter_SetNormalize`, 'returnType') = 'void'
attr(`TikhonovDeconvolutionImageFilter_SetNormalize`, "inputTypes") = c('_p_itk__simple__TikhonovDeconvolutionImageFilter', 'logical')
class(`TikhonovDeconvolutionImageFilter_SetNormalize`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_SetNormalize'))

# Start of TikhonovDeconvolutionImageFilter_NormalizeOn

`TikhonovDeconvolutionImageFilter_NormalizeOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_TikhonovDeconvolutionImageFilter_NormalizeOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TikhonovDeconvolutionImageFilter_NormalizeOn`, 'returnType') = 'void'
attr(`TikhonovDeconvolutionImageFilter_NormalizeOn`, "inputTypes") = c('_p_itk__simple__TikhonovDeconvolutionImageFilter')
class(`TikhonovDeconvolutionImageFilter_NormalizeOn`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_NormalizeOn'))

# Start of TikhonovDeconvolutionImageFilter_NormalizeOff

`TikhonovDeconvolutionImageFilter_NormalizeOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_TikhonovDeconvolutionImageFilter_NormalizeOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TikhonovDeconvolutionImageFilter_NormalizeOff`, 'returnType') = 'void'
attr(`TikhonovDeconvolutionImageFilter_NormalizeOff`, "inputTypes") = c('_p_itk__simple__TikhonovDeconvolutionImageFilter')
class(`TikhonovDeconvolutionImageFilter_NormalizeOff`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_NormalizeOff'))

# Start of TikhonovDeconvolutionImageFilter_GetNormalize

`TikhonovDeconvolutionImageFilter_GetNormalize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TikhonovDeconvolutionImageFilter_GetNormalize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TikhonovDeconvolutionImageFilter_GetNormalize`, 'returnType') = 'logical'
attr(`TikhonovDeconvolutionImageFilter_GetNormalize`, "inputTypes") = c('_p_itk__simple__TikhonovDeconvolutionImageFilter')
class(`TikhonovDeconvolutionImageFilter_GetNormalize`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_GetNormalize'))

# Start of TikhonovDeconvolutionImageFilter_ZERO_PAD_get

`TikhonovDeconvolutionImageFilter_ZERO_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TikhonovDeconvolutionImageFilter_ZERO_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TikhonovDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`TikhonovDeconvolutionImageFilter_ZERO_PAD_get`, 'returnType') = 'character'
class(`TikhonovDeconvolutionImageFilter_ZERO_PAD_get`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_ZERO_PAD_get'))

# Start of TikhonovDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get

`TikhonovDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TikhonovDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TikhonovDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`TikhonovDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`, 'returnType') = 'character'
class(`TikhonovDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get'))

# Start of TikhonovDeconvolutionImageFilter_PERIODIC_PAD_get

`TikhonovDeconvolutionImageFilter_PERIODIC_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TikhonovDeconvolutionImageFilter_PERIODIC_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TikhonovDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`TikhonovDeconvolutionImageFilter_PERIODIC_PAD_get`, 'returnType') = 'character'
class(`TikhonovDeconvolutionImageFilter_PERIODIC_PAD_get`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_PERIODIC_PAD_get'))

# Start of TikhonovDeconvolutionImageFilter_SetBoundaryCondition

`TikhonovDeconvolutionImageFilter_SetBoundaryCondition` = function(self, BoundaryCondition)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BoundaryCondition = enumToInteger(BoundaryCondition, "_itk__simple__TikhonovDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(BoundaryCondition) > 1) {
    warning("using only the first element of BoundaryCondition");
  };
  
  ;ans = .Call('R_swig_TikhonovDeconvolutionImageFilter_SetBoundaryCondition', self, BoundaryCondition, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TikhonovDeconvolutionImageFilter_SetBoundaryCondition`, 'returnType') = 'void'
attr(`TikhonovDeconvolutionImageFilter_SetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__TikhonovDeconvolutionImageFilter', 'character')
class(`TikhonovDeconvolutionImageFilter_SetBoundaryCondition`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_SetBoundaryCondition'))

# Start of TikhonovDeconvolutionImageFilter_GetBoundaryCondition

`TikhonovDeconvolutionImageFilter_GetBoundaryCondition` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_TikhonovDeconvolutionImageFilter_GetBoundaryCondition', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TikhonovDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`TikhonovDeconvolutionImageFilter_GetBoundaryCondition`, 'returnType') = 'character'
attr(`TikhonovDeconvolutionImageFilter_GetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__TikhonovDeconvolutionImageFilter')
class(`TikhonovDeconvolutionImageFilter_GetBoundaryCondition`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_GetBoundaryCondition'))

# Start of TikhonovDeconvolutionImageFilter_SAME_get

`TikhonovDeconvolutionImageFilter_SAME_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TikhonovDeconvolutionImageFilter_SAME_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TikhonovDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`TikhonovDeconvolutionImageFilter_SAME_get`, 'returnType') = 'character'
class(`TikhonovDeconvolutionImageFilter_SAME_get`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_SAME_get'))

# Start of TikhonovDeconvolutionImageFilter_VALID_get

`TikhonovDeconvolutionImageFilter_VALID_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_TikhonovDeconvolutionImageFilter_VALID_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TikhonovDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`TikhonovDeconvolutionImageFilter_VALID_get`, 'returnType') = 'character'
class(`TikhonovDeconvolutionImageFilter_VALID_get`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_VALID_get'))

# Start of TikhonovDeconvolutionImageFilter_SetOutputRegionMode

`TikhonovDeconvolutionImageFilter_SetOutputRegionMode` = function(self, OutputRegionMode)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputRegionMode = enumToInteger(OutputRegionMode, "_itk__simple__TikhonovDeconvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(OutputRegionMode) > 1) {
    warning("using only the first element of OutputRegionMode");
  };
  
  ;ans = .Call('R_swig_TikhonovDeconvolutionImageFilter_SetOutputRegionMode', self, OutputRegionMode, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TikhonovDeconvolutionImageFilter_SetOutputRegionMode`, 'returnType') = 'void'
attr(`TikhonovDeconvolutionImageFilter_SetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__TikhonovDeconvolutionImageFilter', 'character')
class(`TikhonovDeconvolutionImageFilter_SetOutputRegionMode`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_SetOutputRegionMode'))

# Start of TikhonovDeconvolutionImageFilter_GetOutputRegionMode

`TikhonovDeconvolutionImageFilter_GetOutputRegionMode` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_TikhonovDeconvolutionImageFilter_GetOutputRegionMode', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__TikhonovDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`TikhonovDeconvolutionImageFilter_GetOutputRegionMode`, 'returnType') = 'character'
attr(`TikhonovDeconvolutionImageFilter_GetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__TikhonovDeconvolutionImageFilter')
class(`TikhonovDeconvolutionImageFilter_GetOutputRegionMode`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_GetOutputRegionMode'))

# Start of TikhonovDeconvolutionImageFilter_GetName

`TikhonovDeconvolutionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TikhonovDeconvolutionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TikhonovDeconvolutionImageFilter_GetName`, 'returnType') = 'character'
attr(`TikhonovDeconvolutionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__TikhonovDeconvolutionImageFilter')
class(`TikhonovDeconvolutionImageFilter_GetName`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_GetName'))

# Start of TikhonovDeconvolutionImageFilter_ToString

`TikhonovDeconvolutionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TikhonovDeconvolutionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TikhonovDeconvolutionImageFilter_ToString`, 'returnType') = 'character'
attr(`TikhonovDeconvolutionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__TikhonovDeconvolutionImageFilter')
class(`TikhonovDeconvolutionImageFilter_ToString`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_ToString'))

# Start of TikhonovDeconvolutionImageFilter_Execute

`TikhonovDeconvolutionImageFilter_Execute` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_TikhonovDeconvolutionImageFilter_Execute', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TikhonovDeconvolutionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`TikhonovDeconvolutionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__TikhonovDeconvolutionImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`TikhonovDeconvolutionImageFilter_Execute`) = c("SWIGFunction", class('TikhonovDeconvolutionImageFilter_Execute'))

# Start of accessor method for itk::simple::TikhonovDeconvolutionImageFilter
setMethod('$', '_p_itk__simple__TikhonovDeconvolutionImageFilter', function(x, name)

{
  accessorFuns = list('SetRegularizationConstant' = TikhonovDeconvolutionImageFilter_SetRegularizationConstant, 'GetRegularizationConstant' = TikhonovDeconvolutionImageFilter_GetRegularizationConstant, 'SetNormalize' = TikhonovDeconvolutionImageFilter_SetNormalize, 'NormalizeOn' = TikhonovDeconvolutionImageFilter_NormalizeOn, 'NormalizeOff' = TikhonovDeconvolutionImageFilter_NormalizeOff, 'GetNormalize' = TikhonovDeconvolutionImageFilter_GetNormalize, 'SetBoundaryCondition' = TikhonovDeconvolutionImageFilter_SetBoundaryCondition, 'GetBoundaryCondition' = TikhonovDeconvolutionImageFilter_GetBoundaryCondition, 'SetOutputRegionMode' = TikhonovDeconvolutionImageFilter_SetOutputRegionMode, 'GetOutputRegionMode' = TikhonovDeconvolutionImageFilter_GetOutputRegionMode, 'GetName' = TikhonovDeconvolutionImageFilter_GetName, 'ToString' = TikhonovDeconvolutionImageFilter_ToString, 'Execute' = TikhonovDeconvolutionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::TikhonovDeconvolutionImageFilter
# Start of TikhonovDeconvolution

`TikhonovDeconvolution__SWIG_0` = function(image1, image2, regularizationConstant, normalize, boundaryCondition, outputRegionMode, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__TikhonovDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  outputRegionMode = enumToInteger(outputRegionMode, "_itk__simple__TikhonovDeconvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(outputRegionMode) > 1) {
    warning("using only the first element of outputRegionMode");
  };
  
  ;ans = .Call('R_swig_TikhonovDeconvolution__SWIG_0', image1, image2, regularizationConstant, normalize, boundaryCondition, outputRegionMode, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TikhonovDeconvolution__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`TikhonovDeconvolution__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'logical', 'character', 'character')
class(`TikhonovDeconvolution__SWIG_0`) = c("SWIGFunction", class('TikhonovDeconvolution__SWIG_0'))

# Start of TikhonovDeconvolution

`TikhonovDeconvolution__SWIG_1` = function(image1, image2, regularizationConstant, normalize, boundaryCondition, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__TikhonovDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  ;ans = .Call('R_swig_TikhonovDeconvolution__SWIG_1', image1, image2, regularizationConstant, normalize, boundaryCondition, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TikhonovDeconvolution__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`TikhonovDeconvolution__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'logical', 'character')
class(`TikhonovDeconvolution__SWIG_1`) = c("SWIGFunction", class('TikhonovDeconvolution__SWIG_1'))

# Start of TikhonovDeconvolution

`TikhonovDeconvolution__SWIG_2` = function(image1, image2, regularizationConstant, normalize, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  normalize = as.logical(normalize);
  ;ans = .Call('R_swig_TikhonovDeconvolution__SWIG_2', image1, image2, regularizationConstant, normalize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TikhonovDeconvolution__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`TikhonovDeconvolution__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'logical')
class(`TikhonovDeconvolution__SWIG_2`) = c("SWIGFunction", class('TikhonovDeconvolution__SWIG_2'))

# Start of TikhonovDeconvolution

`TikhonovDeconvolution__SWIG_3` = function(image1, image2, regularizationConstant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  ;ans = .Call('R_swig_TikhonovDeconvolution__SWIG_3', image1, image2, regularizationConstant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TikhonovDeconvolution__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`TikhonovDeconvolution__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`TikhonovDeconvolution__SWIG_3`) = c("SWIGFunction", class('TikhonovDeconvolution__SWIG_3'))

# Start of TikhonovDeconvolution

`TikhonovDeconvolution__SWIG_4` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_TikhonovDeconvolution__SWIG_4', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TikhonovDeconvolution__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`TikhonovDeconvolution__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`TikhonovDeconvolution__SWIG_4`) = c("SWIGFunction", class('TikhonovDeconvolution__SWIG_4'))

`TikhonovDeconvolution` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- TikhonovDeconvolution__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- TikhonovDeconvolution__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- TikhonovDeconvolution__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.character(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- TikhonovDeconvolution__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.character(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.character(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- TikhonovDeconvolution__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for TikhonovDeconvolution with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_TileImageFilter

`delete_TileImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_TileImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_TileImageFilter`, 'returnType') = 'void'
attr(`delete_TileImageFilter`, "inputTypes") = c('_p_itk__simple__TileImageFilter')
class(`delete_TileImageFilter`) = c("SWIGFunction", class('delete_TileImageFilter'))

# Start of new_TileImageFilter

`TileImageFilter` = function()
{
  ;ans = .Call('R_swig_new_TileImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__TileImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_TileImageFilter);
  ans
  
}

attr(`TileImageFilter`, 'returnType') = '_p_itk__simple__TileImageFilter'
class(`TileImageFilter`) = c("SWIGFunction", class('TileImageFilter'))

# Start of TileImageFilter_SetLayout

`TileImageFilter_SetLayout` = function(self, Layout)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Layout = as.integer(Layout);
  ;ans = .Call('R_swig_TileImageFilter_SetLayout', self, Layout, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TileImageFilter_SetLayout`, 'returnType') = 'void'
attr(`TileImageFilter_SetLayout`, "inputTypes") = c('_p_itk__simple__TileImageFilter', 'integer')
class(`TileImageFilter_SetLayout`) = c("SWIGFunction", class('TileImageFilter_SetLayout'))

# Start of TileImageFilter_GetLayout

`TileImageFilter_GetLayout` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TileImageFilter_GetLayout', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TileImageFilter_GetLayout`, 'returnType') = 'integer'
attr(`TileImageFilter_GetLayout`, "inputTypes") = c('_p_itk__simple__TileImageFilter')
class(`TileImageFilter_GetLayout`) = c("SWIGFunction", class('TileImageFilter_GetLayout'))

# Start of TileImageFilter_SetDefaultPixelValue

`TileImageFilter_SetDefaultPixelValue` = function(self, DefaultPixelValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_TileImageFilter_SetDefaultPixelValue', self, DefaultPixelValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TileImageFilter_SetDefaultPixelValue`, 'returnType') = 'void'
attr(`TileImageFilter_SetDefaultPixelValue`, "inputTypes") = c('_p_itk__simple__TileImageFilter', 'numeric')
class(`TileImageFilter_SetDefaultPixelValue`) = c("SWIGFunction", class('TileImageFilter_SetDefaultPixelValue'))

# Start of TileImageFilter_GetDefaultPixelValue

`TileImageFilter_GetDefaultPixelValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TileImageFilter_GetDefaultPixelValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TileImageFilter_GetDefaultPixelValue`, 'returnType') = 'numeric'
attr(`TileImageFilter_GetDefaultPixelValue`, "inputTypes") = c('_p_itk__simple__TileImageFilter')
class(`TileImageFilter_GetDefaultPixelValue`) = c("SWIGFunction", class('TileImageFilter_GetDefaultPixelValue'))

# Start of TileImageFilter_GetName

`TileImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TileImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TileImageFilter_GetName`, 'returnType') = 'character'
attr(`TileImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__TileImageFilter')
class(`TileImageFilter_GetName`) = c("SWIGFunction", class('TileImageFilter_GetName'))

# Start of TileImageFilter_ToString

`TileImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TileImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TileImageFilter_ToString`, 'returnType') = 'character'
attr(`TileImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__TileImageFilter')
class(`TileImageFilter_ToString`) = c("SWIGFunction", class('TileImageFilter_ToString'))

# Start of TileImageFilter_Execute

`TileImageFilter_Execute__SWIG_0` = function(self, images, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_TileImageFilter_Execute__SWIG_0', self, images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TileImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`TileImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__TileImageFilter', 'list')
class(`TileImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('TileImageFilter_Execute__SWIG_0'))

# Start of TileImageFilter_Execute

`TileImageFilter_Execute__SWIG_1` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_TileImageFilter_Execute__SWIG_1', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TileImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`TileImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__TileImageFilter', '_p_itk__simple__Image')
class(`TileImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('TileImageFilter_Execute__SWIG_1'))

# Start of TileImageFilter_Execute

`TileImageFilter_Execute__SWIG_2` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_TileImageFilter_Execute__SWIG_2', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TileImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`TileImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__TileImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`TileImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('TileImageFilter_Execute__SWIG_2'))

# Start of TileImageFilter_Execute

`TileImageFilter_Execute__SWIG_3` = function(self, image1, image2, image3, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_TileImageFilter_Execute__SWIG_3', self, image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TileImageFilter_Execute__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`TileImageFilter_Execute__SWIG_3`, "inputTypes") = c('_p_itk__simple__TileImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`TileImageFilter_Execute__SWIG_3`) = c("SWIGFunction", class('TileImageFilter_Execute__SWIG_3'))

# Start of TileImageFilter_Execute

`TileImageFilter_Execute__SWIG_4` = function(self, image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_TileImageFilter_Execute__SWIG_4', self, image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TileImageFilter_Execute__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`TileImageFilter_Execute__SWIG_4`, "inputTypes") = c('_p_itk__simple__TileImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`TileImageFilter_Execute__SWIG_4`) = c("SWIGFunction", class('TileImageFilter_Execute__SWIG_4'))

# Start of TileImageFilter_Execute

`TileImageFilter_Execute__SWIG_5` = function(self, image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_TileImageFilter_Execute__SWIG_5', self, image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TileImageFilter_Execute__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`TileImageFilter_Execute__SWIG_5`, "inputTypes") = c('_p_itk__simple__TileImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`TileImageFilter_Execute__SWIG_5`) = c("SWIGFunction", class('TileImageFilter_Execute__SWIG_5'))

`TileImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__TileImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- TileImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__TileImageFilter') || is.null(argv[[1]]) ) && (
        (is.list(argv[[2]]) & all(sapply(argv[[2]], inherits, what='_p_itk__simple__Image')))
        )) {
      f <- TileImageFilter_Execute__SWIG_0; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__TileImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- TileImageFilter_Execute__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__TileImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- TileImageFilter_Execute__SWIG_3; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__TileImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- TileImageFilter_Execute__SWIG_4; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__TileImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( extends(argtypes[6], '_p_itk__simple__Image') && length(argv[[6]]) == 1 )) {
      f <- TileImageFilter_Execute__SWIG_5; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for TileImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::TileImageFilter
setMethod('$', '_p_itk__simple__TileImageFilter', function(x, name)

{
  accessorFuns = list('SetLayout' = TileImageFilter_SetLayout, 'GetLayout' = TileImageFilter_GetLayout, 'SetDefaultPixelValue' = TileImageFilter_SetDefaultPixelValue, 'GetDefaultPixelValue' = TileImageFilter_GetDefaultPixelValue, 'GetName' = TileImageFilter_GetName, 'ToString' = TileImageFilter_ToString, 'Execute' = TileImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::TileImageFilter
# Start of Tile

`Tile__SWIG_0` = function(images, layout, defaultPixelValue, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  layout = as.integer(layout);
  
  ;ans = .Call('R_swig_Tile__SWIG_0', images, layout, defaultPixelValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_0`, "inputTypes") = c('list', 'integer', 'numeric')
class(`Tile__SWIG_0`) = c("SWIGFunction", class('Tile__SWIG_0'))

# Start of Tile

`Tile__SWIG_1` = function(images, layout, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  layout = as.integer(layout);
  ;ans = .Call('R_swig_Tile__SWIG_1', images, layout, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_1`, "inputTypes") = c('list', 'integer')
class(`Tile__SWIG_1`) = c("SWIGFunction", class('Tile__SWIG_1'))

# Start of Tile

`Tile__SWIG_2` = function(images, .copy = FALSE)
{
  images=ImListToVec(images);
  images=slot(images, "ref")
  
  ;ans = .Call('R_swig_Tile__SWIG_2', images, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_2`, "inputTypes") = c('list')
class(`Tile__SWIG_2`) = c("SWIGFunction", class('Tile__SWIG_2'))

# Start of Tile

`Tile__SWIG_3` = function(image1, layout, defaultPixelValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  layout = as.integer(layout);
  
  ;ans = .Call('R_swig_Tile__SWIG_3', image1, layout, defaultPixelValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'numeric')
class(`Tile__SWIG_3`) = c("SWIGFunction", class('Tile__SWIG_3'))

# Start of Tile

`Tile__SWIG_4` = function(image1, layout, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  layout = as.integer(layout);
  ;ans = .Call('R_swig_Tile__SWIG_4', image1, layout, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Tile__SWIG_4`) = c("SWIGFunction", class('Tile__SWIG_4'))

# Start of Tile

`Tile__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Tile__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`Tile__SWIG_5`) = c("SWIGFunction", class('Tile__SWIG_5'))

# Start of Tile

`Tile__SWIG_6` = function(image1, image2, layout, defaultPixelValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  layout = as.integer(layout);
  
  ;ans = .Call('R_swig_Tile__SWIG_6', image1, image2, layout, defaultPixelValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric')
class(`Tile__SWIG_6`) = c("SWIGFunction", class('Tile__SWIG_6'))

# Start of Tile

`Tile__SWIG_7` = function(image1, image2, layout, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  layout = as.integer(layout);
  ;ans = .Call('R_swig_Tile__SWIG_7', image1, image2, layout, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`Tile__SWIG_7`) = c("SWIGFunction", class('Tile__SWIG_7'))

# Start of Tile

`Tile__SWIG_8` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Tile__SWIG_8', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Tile__SWIG_8`) = c("SWIGFunction", class('Tile__SWIG_8'))

# Start of Tile

`Tile__SWIG_9` = function(image1, image2, image3, layout, defaultPixelValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  layout = as.integer(layout);
  
  ;ans = .Call('R_swig_Tile__SWIG_9', image1, image2, image3, layout, defaultPixelValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric')
class(`Tile__SWIG_9`) = c("SWIGFunction", class('Tile__SWIG_9'))

# Start of Tile

`Tile__SWIG_10` = function(image1, image2, image3, layout, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  layout = as.integer(layout);
  ;ans = .Call('R_swig_Tile__SWIG_10', image1, image2, image3, layout, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`Tile__SWIG_10`) = c("SWIGFunction", class('Tile__SWIG_10'))

# Start of Tile

`Tile__SWIG_11` = function(image1, image2, image3, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  ;ans = .Call('R_swig_Tile__SWIG_11', image1, image2, image3, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Tile__SWIG_11`) = c("SWIGFunction", class('Tile__SWIG_11'))

# Start of Tile

`Tile__SWIG_12` = function(image1, image2, image3, image4, layout, defaultPixelValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  layout = as.integer(layout);
  
  ;ans = .Call('R_swig_Tile__SWIG_12', image1, image2, image3, image4, layout, defaultPixelValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_12`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_12`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric')
class(`Tile__SWIG_12`) = c("SWIGFunction", class('Tile__SWIG_12'))

# Start of Tile

`Tile__SWIG_13` = function(image1, image2, image3, image4, layout, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  layout = as.integer(layout);
  ;ans = .Call('R_swig_Tile__SWIG_13', image1, image2, image3, image4, layout, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_13`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_13`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`Tile__SWIG_13`) = c("SWIGFunction", class('Tile__SWIG_13'))

# Start of Tile

`Tile__SWIG_14` = function(image1, image2, image3, image4, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  ;ans = .Call('R_swig_Tile__SWIG_14', image1, image2, image3, image4, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_14`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_14`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Tile__SWIG_14`) = c("SWIGFunction", class('Tile__SWIG_14'))

# Start of Tile

`Tile__SWIG_15` = function(image1, image2, image3, image4, image5, layout, defaultPixelValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  layout = as.integer(layout);
  
  ;ans = .Call('R_swig_Tile__SWIG_15', image1, image2, image3, image4, image5, layout, defaultPixelValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_15`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_15`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'numeric')
class(`Tile__SWIG_15`) = c("SWIGFunction", class('Tile__SWIG_15'))

# Start of Tile

`Tile__SWIG_16` = function(image1, image2, image3, image4, image5, layout, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  layout = as.integer(layout);
  ;ans = .Call('R_swig_Tile__SWIG_16', image1, image2, image3, image4, image5, layout, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_16`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_16`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`Tile__SWIG_16`) = c("SWIGFunction", class('Tile__SWIG_16'))

# Start of Tile

`Tile__SWIG_17` = function(image1, image2, image3, image4, image5, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  if (inherits(image3, "ExternalReference")) image3 = slot(image3,"ref"); 
  if (inherits(image4, "ExternalReference")) image4 = slot(image4,"ref"); 
  if (inherits(image5, "ExternalReference")) image5 = slot(image5,"ref"); 
  ;ans = .Call('R_swig_Tile__SWIG_17', image1, image2, image3, image4, image5, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Tile__SWIG_17`, 'returnType') = '_p_itk__simple__Image'
attr(`Tile__SWIG_17`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Tile__SWIG_17`) = c("SWIGFunction", class('Tile__SWIG_17'))

`Tile` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 18
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- Tile__SWIG_5; 
    }
    else if (
      (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
      ) {
      f <- Tile__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Tile__SWIG_8; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- Tile__SWIG_4; 
    }
    else if ((
        (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
        ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- Tile__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- Tile__SWIG_11; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- Tile__SWIG_7; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Tile__SWIG_3; 
    }
    else if ((
        (is.list(argv[[1]]) & all(sapply(argv[[1]], inherits, what='_p_itk__simple__Image')))
        ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Tile__SWIG_0; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 )) {
      f <- Tile__SWIG_14; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) )) {
      f <- Tile__SWIG_10; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- Tile__SWIG_6; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 )) {
      f <- Tile__SWIG_17; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( is.integer(argv[[5]]) || is.numeric(argv[[5]]) )) {
      f <- Tile__SWIG_13; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- Tile__SWIG_9; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( is.integer(argv[[6]]) || is.numeric(argv[[6]]) )) {
      f <- Tile__SWIG_16; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( is.integer(argv[[5]]) || is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- Tile__SWIG_12; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 ) && ( extends(argtypes[4], '_p_itk__simple__Image') && length(argv[[4]]) == 1 ) && ( extends(argtypes[5], '_p_itk__simple__Image') && length(argv[[5]]) == 1 ) && ( is.integer(argv[[6]]) || is.numeric(argv[[6]]) ) && ( is.numeric(argv[[7]]) && length(argv[[7]]) == 1 )) {
      f <- Tile__SWIG_15; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Tile with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_TobogganImageFilter

`delete_TobogganImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_TobogganImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_TobogganImageFilter`, 'returnType') = 'void'
attr(`delete_TobogganImageFilter`, "inputTypes") = c('_p_itk__simple__TobogganImageFilter')
class(`delete_TobogganImageFilter`) = c("SWIGFunction", class('delete_TobogganImageFilter'))

# Start of new_TobogganImageFilter

`TobogganImageFilter` = function()
{
  ;ans = .Call('R_swig_new_TobogganImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__TobogganImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_TobogganImageFilter);
  ans
  
}

attr(`TobogganImageFilter`, 'returnType') = '_p_itk__simple__TobogganImageFilter'
class(`TobogganImageFilter`) = c("SWIGFunction", class('TobogganImageFilter'))

# Start of TobogganImageFilter_GetName

`TobogganImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TobogganImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TobogganImageFilter_GetName`, 'returnType') = 'character'
attr(`TobogganImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__TobogganImageFilter')
class(`TobogganImageFilter_GetName`) = c("SWIGFunction", class('TobogganImageFilter_GetName'))

# Start of TobogganImageFilter_ToString

`TobogganImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TobogganImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TobogganImageFilter_ToString`, 'returnType') = 'character'
attr(`TobogganImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__TobogganImageFilter')
class(`TobogganImageFilter_ToString`) = c("SWIGFunction", class('TobogganImageFilter_ToString'))

# Start of TobogganImageFilter_Execute

`TobogganImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_TobogganImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TobogganImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`TobogganImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__TobogganImageFilter', '_p_itk__simple__Image')
class(`TobogganImageFilter_Execute`) = c("SWIGFunction", class('TobogganImageFilter_Execute'))

# Start of accessor method for itk::simple::TobogganImageFilter
setMethod('$', '_p_itk__simple__TobogganImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = TobogganImageFilter_GetName, 'ToString' = TobogganImageFilter_ToString, 'Execute' = TobogganImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::TobogganImageFilter
# Start of Toboggan

`Toboggan` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_Toboggan', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Toboggan`, 'returnType') = '_p_itk__simple__Image'
attr(`Toboggan`, "inputTypes") = c('_p_itk__simple__Image')
class(`Toboggan`) = c("SWIGFunction", class('Toboggan'))

# Start of delete_TransformGeometryImageFilter

`delete_TransformGeometryImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_TransformGeometryImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_TransformGeometryImageFilter`, 'returnType') = 'void'
attr(`delete_TransformGeometryImageFilter`, "inputTypes") = c('_p_itk__simple__TransformGeometryImageFilter')
class(`delete_TransformGeometryImageFilter`) = c("SWIGFunction", class('delete_TransformGeometryImageFilter'))

# Start of new_TransformGeometryImageFilter

`TransformGeometryImageFilter` = function()
{
  ;ans = .Call('R_swig_new_TransformGeometryImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__TransformGeometryImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_TransformGeometryImageFilter);
  ans
  
}

attr(`TransformGeometryImageFilter`, 'returnType') = '_p_itk__simple__TransformGeometryImageFilter'
class(`TransformGeometryImageFilter`) = c("SWIGFunction", class('TransformGeometryImageFilter'))

# Start of TransformGeometryImageFilter_GetName

`TransformGeometryImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TransformGeometryImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TransformGeometryImageFilter_GetName`, 'returnType') = 'character'
attr(`TransformGeometryImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__TransformGeometryImageFilter')
class(`TransformGeometryImageFilter_GetName`) = c("SWIGFunction", class('TransformGeometryImageFilter_GetName'))

# Start of TransformGeometryImageFilter_ToString

`TransformGeometryImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TransformGeometryImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TransformGeometryImageFilter_ToString`, 'returnType') = 'character'
attr(`TransformGeometryImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__TransformGeometryImageFilter')
class(`TransformGeometryImageFilter_ToString`) = c("SWIGFunction", class('TransformGeometryImageFilter_ToString'))

# Start of TransformGeometryImageFilter_Execute

`TransformGeometryImageFilter_Execute` = function(self, image, transform, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_TransformGeometryImageFilter_Execute', self, image, transform, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TransformGeometryImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`TransformGeometryImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__TransformGeometryImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Transform')
class(`TransformGeometryImageFilter_Execute`) = c("SWIGFunction", class('TransformGeometryImageFilter_Execute'))

# Start of accessor method for itk::simple::TransformGeometryImageFilter
setMethod('$', '_p_itk__simple__TransformGeometryImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = TransformGeometryImageFilter_GetName, 'ToString' = TransformGeometryImageFilter_ToString, 'Execute' = TransformGeometryImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::TransformGeometryImageFilter
# Start of TransformGeometry

`TransformGeometry` = function(image, transform, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_TransformGeometry', image, transform, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TransformGeometry`, 'returnType') = '_p_itk__simple__Image'
attr(`TransformGeometry`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Transform')
class(`TransformGeometry`) = c("SWIGFunction", class('TransformGeometry'))

# Start of delete_TransformToDisplacementFieldFilter

`delete_TransformToDisplacementFieldFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_TransformToDisplacementFieldFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_TransformToDisplacementFieldFilter`, 'returnType') = 'void'
attr(`delete_TransformToDisplacementFieldFilter`, "inputTypes") = c('_p_itk__simple__TransformToDisplacementFieldFilter')
class(`delete_TransformToDisplacementFieldFilter`) = c("SWIGFunction", class('delete_TransformToDisplacementFieldFilter'))

# Start of new_TransformToDisplacementFieldFilter

`TransformToDisplacementFieldFilter` = function()
{
  ;ans = .Call('R_swig_new_TransformToDisplacementFieldFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__TransformToDisplacementFieldFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_TransformToDisplacementFieldFilter);
  ans
  
}

attr(`TransformToDisplacementFieldFilter`, 'returnType') = '_p_itk__simple__TransformToDisplacementFieldFilter'
class(`TransformToDisplacementFieldFilter`) = c("SWIGFunction", class('TransformToDisplacementFieldFilter'))

# Start of TransformToDisplacementFieldFilter_SetOutputPixelType

`TransformToDisplacementFieldFilter_SetOutputPixelType` = function(self, OutputPixelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputPixelType = enumToInteger(OutputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(OutputPixelType) > 1) {
    warning("using only the first element of OutputPixelType");
  };
  
  ;ans = .Call('R_swig_TransformToDisplacementFieldFilter_SetOutputPixelType', self, OutputPixelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TransformToDisplacementFieldFilter_SetOutputPixelType`, 'returnType') = 'void'
attr(`TransformToDisplacementFieldFilter_SetOutputPixelType`, "inputTypes") = c('_p_itk__simple__TransformToDisplacementFieldFilter', 'character')
class(`TransformToDisplacementFieldFilter_SetOutputPixelType`) = c("SWIGFunction", class('TransformToDisplacementFieldFilter_SetOutputPixelType'))

# Start of TransformToDisplacementFieldFilter_GetOutputPixelType

`TransformToDisplacementFieldFilter_GetOutputPixelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_TransformToDisplacementFieldFilter_GetOutputPixelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`TransformToDisplacementFieldFilter_GetOutputPixelType`, 'returnType') = 'character'
attr(`TransformToDisplacementFieldFilter_GetOutputPixelType`, "inputTypes") = c('_p_itk__simple__TransformToDisplacementFieldFilter')
class(`TransformToDisplacementFieldFilter_GetOutputPixelType`) = c("SWIGFunction", class('TransformToDisplacementFieldFilter_GetOutputPixelType'))

# Start of TransformToDisplacementFieldFilter_SetSize

`TransformToDisplacementFieldFilter_SetSize` = function(self, Size)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Size = as.integer(Size);
  ;ans = .Call('R_swig_TransformToDisplacementFieldFilter_SetSize', self, Size, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TransformToDisplacementFieldFilter_SetSize`, 'returnType') = 'void'
attr(`TransformToDisplacementFieldFilter_SetSize`, "inputTypes") = c('_p_itk__simple__TransformToDisplacementFieldFilter', 'integer')
class(`TransformToDisplacementFieldFilter_SetSize`) = c("SWIGFunction", class('TransformToDisplacementFieldFilter_SetSize'))

# Start of TransformToDisplacementFieldFilter_GetSize

`TransformToDisplacementFieldFilter_GetSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TransformToDisplacementFieldFilter_GetSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TransformToDisplacementFieldFilter_GetSize`, 'returnType') = 'integer'
attr(`TransformToDisplacementFieldFilter_GetSize`, "inputTypes") = c('_p_itk__simple__TransformToDisplacementFieldFilter')
class(`TransformToDisplacementFieldFilter_GetSize`) = c("SWIGFunction", class('TransformToDisplacementFieldFilter_GetSize'))

# Start of TransformToDisplacementFieldFilter_SetOutputOrigin

`TransformToDisplacementFieldFilter_SetOutputOrigin` = function(self, OutputOrigin)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputOrigin = as.numeric(OutputOrigin);
  ;ans = .Call('R_swig_TransformToDisplacementFieldFilter_SetOutputOrigin', self, OutputOrigin, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TransformToDisplacementFieldFilter_SetOutputOrigin`, 'returnType') = 'void'
attr(`TransformToDisplacementFieldFilter_SetOutputOrigin`, "inputTypes") = c('_p_itk__simple__TransformToDisplacementFieldFilter', 'numeric')
class(`TransformToDisplacementFieldFilter_SetOutputOrigin`) = c("SWIGFunction", class('TransformToDisplacementFieldFilter_SetOutputOrigin'))

# Start of TransformToDisplacementFieldFilter_GetOutputOrigin

`TransformToDisplacementFieldFilter_GetOutputOrigin` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TransformToDisplacementFieldFilter_GetOutputOrigin', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TransformToDisplacementFieldFilter_GetOutputOrigin`, 'returnType') = 'numeric'
attr(`TransformToDisplacementFieldFilter_GetOutputOrigin`, "inputTypes") = c('_p_itk__simple__TransformToDisplacementFieldFilter')
class(`TransformToDisplacementFieldFilter_GetOutputOrigin`) = c("SWIGFunction", class('TransformToDisplacementFieldFilter_GetOutputOrigin'))

# Start of TransformToDisplacementFieldFilter_SetOutputSpacing

`TransformToDisplacementFieldFilter_SetOutputSpacing` = function(self, OutputSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputSpacing = as.numeric(OutputSpacing);
  ;ans = .Call('R_swig_TransformToDisplacementFieldFilter_SetOutputSpacing', self, OutputSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TransformToDisplacementFieldFilter_SetOutputSpacing`, 'returnType') = 'void'
attr(`TransformToDisplacementFieldFilter_SetOutputSpacing`, "inputTypes") = c('_p_itk__simple__TransformToDisplacementFieldFilter', 'numeric')
class(`TransformToDisplacementFieldFilter_SetOutputSpacing`) = c("SWIGFunction", class('TransformToDisplacementFieldFilter_SetOutputSpacing'))

# Start of TransformToDisplacementFieldFilter_GetOutputSpacing

`TransformToDisplacementFieldFilter_GetOutputSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TransformToDisplacementFieldFilter_GetOutputSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TransformToDisplacementFieldFilter_GetOutputSpacing`, 'returnType') = 'numeric'
attr(`TransformToDisplacementFieldFilter_GetOutputSpacing`, "inputTypes") = c('_p_itk__simple__TransformToDisplacementFieldFilter')
class(`TransformToDisplacementFieldFilter_GetOutputSpacing`) = c("SWIGFunction", class('TransformToDisplacementFieldFilter_GetOutputSpacing'))

# Start of TransformToDisplacementFieldFilter_SetOutputDirection

`TransformToDisplacementFieldFilter_SetOutputDirection` = function(self, OutputDirection)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputDirection = as.numeric(OutputDirection);
  ;ans = .Call('R_swig_TransformToDisplacementFieldFilter_SetOutputDirection', self, OutputDirection, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TransformToDisplacementFieldFilter_SetOutputDirection`, 'returnType') = 'void'
attr(`TransformToDisplacementFieldFilter_SetOutputDirection`, "inputTypes") = c('_p_itk__simple__TransformToDisplacementFieldFilter', 'numeric')
class(`TransformToDisplacementFieldFilter_SetOutputDirection`) = c("SWIGFunction", class('TransformToDisplacementFieldFilter_SetOutputDirection'))

# Start of TransformToDisplacementFieldFilter_GetOutputDirection

`TransformToDisplacementFieldFilter_GetOutputDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TransformToDisplacementFieldFilter_GetOutputDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TransformToDisplacementFieldFilter_GetOutputDirection`, 'returnType') = 'numeric'
attr(`TransformToDisplacementFieldFilter_GetOutputDirection`, "inputTypes") = c('_p_itk__simple__TransformToDisplacementFieldFilter')
class(`TransformToDisplacementFieldFilter_GetOutputDirection`) = c("SWIGFunction", class('TransformToDisplacementFieldFilter_GetOutputDirection'))

# Start of TransformToDisplacementFieldFilter_GetName

`TransformToDisplacementFieldFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TransformToDisplacementFieldFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TransformToDisplacementFieldFilter_GetName`, 'returnType') = 'character'
attr(`TransformToDisplacementFieldFilter_GetName`, "inputTypes") = c('_p_itk__simple__TransformToDisplacementFieldFilter')
class(`TransformToDisplacementFieldFilter_GetName`) = c("SWIGFunction", class('TransformToDisplacementFieldFilter_GetName'))

# Start of TransformToDisplacementFieldFilter_ToString

`TransformToDisplacementFieldFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TransformToDisplacementFieldFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TransformToDisplacementFieldFilter_ToString`, 'returnType') = 'character'
attr(`TransformToDisplacementFieldFilter_ToString`, "inputTypes") = c('_p_itk__simple__TransformToDisplacementFieldFilter')
class(`TransformToDisplacementFieldFilter_ToString`) = c("SWIGFunction", class('TransformToDisplacementFieldFilter_ToString'))

# Start of TransformToDisplacementFieldFilter_Execute

`TransformToDisplacementFieldFilter_Execute` = function(self, transform, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_TransformToDisplacementFieldFilter_Execute', self, transform, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TransformToDisplacementFieldFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`TransformToDisplacementFieldFilter_Execute`, "inputTypes") = c('_p_itk__simple__TransformToDisplacementFieldFilter', '_p_itk__simple__Transform')
class(`TransformToDisplacementFieldFilter_Execute`) = c("SWIGFunction", class('TransformToDisplacementFieldFilter_Execute'))

# Start of TransformToDisplacementFieldFilter_SetReferenceImage

`TransformToDisplacementFieldFilter_SetReferenceImage` = function(self, refImage)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(refImage, "ExternalReference")) refImage = slot(refImage,"ref"); 
  ;ans = .Call('R_swig_TransformToDisplacementFieldFilter_SetReferenceImage', self, refImage, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TransformToDisplacementFieldFilter_SetReferenceImage`, 'returnType') = 'void'
attr(`TransformToDisplacementFieldFilter_SetReferenceImage`, "inputTypes") = c('_p_itk__simple__TransformToDisplacementFieldFilter', '_p_itk__simple__Image')
class(`TransformToDisplacementFieldFilter_SetReferenceImage`) = c("SWIGFunction", class('TransformToDisplacementFieldFilter_SetReferenceImage'))

# Start of accessor method for itk::simple::TransformToDisplacementFieldFilter
setMethod('$', '_p_itk__simple__TransformToDisplacementFieldFilter', function(x, name)

{
  accessorFuns = list('SetOutputPixelType' = TransformToDisplacementFieldFilter_SetOutputPixelType, 'GetOutputPixelType' = TransformToDisplacementFieldFilter_GetOutputPixelType, 'SetSize' = TransformToDisplacementFieldFilter_SetSize, 'GetSize' = TransformToDisplacementFieldFilter_GetSize, 'SetOutputOrigin' = TransformToDisplacementFieldFilter_SetOutputOrigin, 'GetOutputOrigin' = TransformToDisplacementFieldFilter_GetOutputOrigin, 'SetOutputSpacing' = TransformToDisplacementFieldFilter_SetOutputSpacing, 'GetOutputSpacing' = TransformToDisplacementFieldFilter_GetOutputSpacing, 'SetOutputDirection' = TransformToDisplacementFieldFilter_SetOutputDirection, 'GetOutputDirection' = TransformToDisplacementFieldFilter_GetOutputDirection, 'GetName' = TransformToDisplacementFieldFilter_GetName, 'ToString' = TransformToDisplacementFieldFilter_ToString, 'Execute' = TransformToDisplacementFieldFilter_Execute, 'SetReferenceImage' = TransformToDisplacementFieldFilter_SetReferenceImage);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::TransformToDisplacementFieldFilter
# Start of TransformToDisplacementField

`TransformToDisplacementField__SWIG_0` = function(transform, outputPixelType, size, outputOrigin, outputSpacing, outputDirection, .copy = FALSE)
{
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  outputOrigin = as.numeric(outputOrigin);
  outputSpacing = as.numeric(outputSpacing);
  outputDirection = as.numeric(outputDirection);
  ;ans = .Call('R_swig_TransformToDisplacementField__SWIG_0', transform, outputPixelType, size, outputOrigin, outputSpacing, outputDirection, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TransformToDisplacementField__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`TransformToDisplacementField__SWIG_0`, "inputTypes") = c('_p_itk__simple__Transform', 'character', 'integer', 'numeric', 'numeric', 'numeric')
class(`TransformToDisplacementField__SWIG_0`) = c("SWIGFunction", class('TransformToDisplacementField__SWIG_0'))

# Start of TransformToDisplacementField

`TransformToDisplacementField__SWIG_1` = function(transform, outputPixelType, size, outputOrigin, outputSpacing, .copy = FALSE)
{
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  outputOrigin = as.numeric(outputOrigin);
  outputSpacing = as.numeric(outputSpacing);
  ;ans = .Call('R_swig_TransformToDisplacementField__SWIG_1', transform, outputPixelType, size, outputOrigin, outputSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TransformToDisplacementField__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`TransformToDisplacementField__SWIG_1`, "inputTypes") = c('_p_itk__simple__Transform', 'character', 'integer', 'numeric', 'numeric')
class(`TransformToDisplacementField__SWIG_1`) = c("SWIGFunction", class('TransformToDisplacementField__SWIG_1'))

# Start of TransformToDisplacementField

`TransformToDisplacementField__SWIG_2` = function(transform, outputPixelType, size, outputOrigin, .copy = FALSE)
{
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  outputOrigin = as.numeric(outputOrigin);
  ;ans = .Call('R_swig_TransformToDisplacementField__SWIG_2', transform, outputPixelType, size, outputOrigin, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TransformToDisplacementField__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`TransformToDisplacementField__SWIG_2`, "inputTypes") = c('_p_itk__simple__Transform', 'character', 'integer', 'numeric')
class(`TransformToDisplacementField__SWIG_2`) = c("SWIGFunction", class('TransformToDisplacementField__SWIG_2'))

# Start of TransformToDisplacementField

`TransformToDisplacementField__SWIG_3` = function(transform, outputPixelType, size, .copy = FALSE)
{
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  size = as.integer(size);
  ;ans = .Call('R_swig_TransformToDisplacementField__SWIG_3', transform, outputPixelType, size, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TransformToDisplacementField__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`TransformToDisplacementField__SWIG_3`, "inputTypes") = c('_p_itk__simple__Transform', 'character', 'integer')
class(`TransformToDisplacementField__SWIG_3`) = c("SWIGFunction", class('TransformToDisplacementField__SWIG_3'))

# Start of TransformToDisplacementField

`TransformToDisplacementField__SWIG_4` = function(transform, outputPixelType, .copy = FALSE)
{
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  ;ans = .Call('R_swig_TransformToDisplacementField__SWIG_4', transform, outputPixelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TransformToDisplacementField__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`TransformToDisplacementField__SWIG_4`, "inputTypes") = c('_p_itk__simple__Transform', 'character')
class(`TransformToDisplacementField__SWIG_4`) = c("SWIGFunction", class('TransformToDisplacementField__SWIG_4'))

# Start of TransformToDisplacementField

`TransformToDisplacementField__SWIG_5` = function(transform, .copy = FALSE)
{
  if (inherits(transform, "ExternalReference")) transform = slot(transform,"ref"); 
  ;ans = .Call('R_swig_TransformToDisplacementField__SWIG_5', transform, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TransformToDisplacementField__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`TransformToDisplacementField__SWIG_5`, "inputTypes") = c('_p_itk__simple__Transform')
class(`TransformToDisplacementField__SWIG_5`) = c("SWIGFunction", class('TransformToDisplacementField__SWIG_5'))

`TransformToDisplacementField` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) {
      f <- TransformToDisplacementField__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- TransformToDisplacementField__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- TransformToDisplacementField__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) )) {
      f <- TransformToDisplacementField__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) )) {
      f <- TransformToDisplacementField__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Transform') && length(argv[[1]]) == 1 ) && ( is.character(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) ) && ( is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) )) {
      f <- TransformToDisplacementField__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for TransformToDisplacementField with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_TriangleThresholdImageFilter

`delete_TriangleThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_TriangleThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_TriangleThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_TriangleThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter')
class(`delete_TriangleThresholdImageFilter`) = c("SWIGFunction", class('delete_TriangleThresholdImageFilter'))

# Start of new_TriangleThresholdImageFilter

`TriangleThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_TriangleThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__TriangleThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_TriangleThresholdImageFilter);
  ans
  
}

attr(`TriangleThresholdImageFilter`, 'returnType') = '_p_itk__simple__TriangleThresholdImageFilter'
class(`TriangleThresholdImageFilter`) = c("SWIGFunction", class('TriangleThresholdImageFilter'))

# Start of TriangleThresholdImageFilter_SetInsideValue

`TriangleThresholdImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideValue = as.integer(InsideValue);
  
  if(length(InsideValue) > 1) {
    warning("using only the first element of InsideValue");
  };
  
  ;ans = .Call('R_swig_TriangleThresholdImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TriangleThresholdImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`TriangleThresholdImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter', 'integer')
class(`TriangleThresholdImageFilter_SetInsideValue`) = c("SWIGFunction", class('TriangleThresholdImageFilter_SetInsideValue'))

# Start of TriangleThresholdImageFilter_GetInsideValue

`TriangleThresholdImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TriangleThresholdImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TriangleThresholdImageFilter_GetInsideValue`, 'returnType') = 'integer'
attr(`TriangleThresholdImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter')
class(`TriangleThresholdImageFilter_GetInsideValue`) = c("SWIGFunction", class('TriangleThresholdImageFilter_GetInsideValue'))

# Start of TriangleThresholdImageFilter_SetOutsideValue

`TriangleThresholdImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutsideValue = as.integer(OutsideValue);
  
  if(length(OutsideValue) > 1) {
    warning("using only the first element of OutsideValue");
  };
  
  ;ans = .Call('R_swig_TriangleThresholdImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TriangleThresholdImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`TriangleThresholdImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter', 'integer')
class(`TriangleThresholdImageFilter_SetOutsideValue`) = c("SWIGFunction", class('TriangleThresholdImageFilter_SetOutsideValue'))

# Start of TriangleThresholdImageFilter_GetOutsideValue

`TriangleThresholdImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TriangleThresholdImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TriangleThresholdImageFilter_GetOutsideValue`, 'returnType') = 'integer'
attr(`TriangleThresholdImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter')
class(`TriangleThresholdImageFilter_GetOutsideValue`) = c("SWIGFunction", class('TriangleThresholdImageFilter_GetOutsideValue'))

# Start of TriangleThresholdImageFilter_SetNumberOfHistogramBins

`TriangleThresholdImageFilter_SetNumberOfHistogramBins` = function(self, NumberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfHistogramBins = as.integer(NumberOfHistogramBins);
  
  if(length(NumberOfHistogramBins) > 1) {
    warning("using only the first element of NumberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_TriangleThresholdImageFilter_SetNumberOfHistogramBins', self, NumberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TriangleThresholdImageFilter_SetNumberOfHistogramBins`, 'returnType') = 'void'
attr(`TriangleThresholdImageFilter_SetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter', 'integer')
class(`TriangleThresholdImageFilter_SetNumberOfHistogramBins`) = c("SWIGFunction", class('TriangleThresholdImageFilter_SetNumberOfHistogramBins'))

# Start of TriangleThresholdImageFilter_GetNumberOfHistogramBins

`TriangleThresholdImageFilter_GetNumberOfHistogramBins` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TriangleThresholdImageFilter_GetNumberOfHistogramBins', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TriangleThresholdImageFilter_GetNumberOfHistogramBins`, 'returnType') = 'integer'
attr(`TriangleThresholdImageFilter_GetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter')
class(`TriangleThresholdImageFilter_GetNumberOfHistogramBins`) = c("SWIGFunction", class('TriangleThresholdImageFilter_GetNumberOfHistogramBins'))

# Start of TriangleThresholdImageFilter_SetMaskOutput

`TriangleThresholdImageFilter_SetMaskOutput` = function(self, MaskOutput)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskOutput = as.logical(MaskOutput);
  ;ans = .Call('R_swig_TriangleThresholdImageFilter_SetMaskOutput', self, MaskOutput, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TriangleThresholdImageFilter_SetMaskOutput`, 'returnType') = 'void'
attr(`TriangleThresholdImageFilter_SetMaskOutput`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter', 'logical')
class(`TriangleThresholdImageFilter_SetMaskOutput`) = c("SWIGFunction", class('TriangleThresholdImageFilter_SetMaskOutput'))

# Start of TriangleThresholdImageFilter_MaskOutputOn

`TriangleThresholdImageFilter_MaskOutputOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_TriangleThresholdImageFilter_MaskOutputOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TriangleThresholdImageFilter_MaskOutputOn`, 'returnType') = 'void'
attr(`TriangleThresholdImageFilter_MaskOutputOn`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter')
class(`TriangleThresholdImageFilter_MaskOutputOn`) = c("SWIGFunction", class('TriangleThresholdImageFilter_MaskOutputOn'))

# Start of TriangleThresholdImageFilter_MaskOutputOff

`TriangleThresholdImageFilter_MaskOutputOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_TriangleThresholdImageFilter_MaskOutputOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TriangleThresholdImageFilter_MaskOutputOff`, 'returnType') = 'void'
attr(`TriangleThresholdImageFilter_MaskOutputOff`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter')
class(`TriangleThresholdImageFilter_MaskOutputOff`) = c("SWIGFunction", class('TriangleThresholdImageFilter_MaskOutputOff'))

# Start of TriangleThresholdImageFilter_GetMaskOutput

`TriangleThresholdImageFilter_GetMaskOutput` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TriangleThresholdImageFilter_GetMaskOutput', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TriangleThresholdImageFilter_GetMaskOutput`, 'returnType') = 'logical'
attr(`TriangleThresholdImageFilter_GetMaskOutput`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter')
class(`TriangleThresholdImageFilter_GetMaskOutput`) = c("SWIGFunction", class('TriangleThresholdImageFilter_GetMaskOutput'))

# Start of TriangleThresholdImageFilter_SetMaskValue

`TriangleThresholdImageFilter_SetMaskValue` = function(self, MaskValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskValue = as.integer(MaskValue);
  
  if(length(MaskValue) > 1) {
    warning("using only the first element of MaskValue");
  };
  
  ;ans = .Call('R_swig_TriangleThresholdImageFilter_SetMaskValue', self, MaskValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`TriangleThresholdImageFilter_SetMaskValue`, 'returnType') = 'void'
attr(`TriangleThresholdImageFilter_SetMaskValue`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter', 'integer')
class(`TriangleThresholdImageFilter_SetMaskValue`) = c("SWIGFunction", class('TriangleThresholdImageFilter_SetMaskValue'))

# Start of TriangleThresholdImageFilter_GetMaskValue

`TriangleThresholdImageFilter_GetMaskValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TriangleThresholdImageFilter_GetMaskValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TriangleThresholdImageFilter_GetMaskValue`, 'returnType') = 'integer'
attr(`TriangleThresholdImageFilter_GetMaskValue`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter')
class(`TriangleThresholdImageFilter_GetMaskValue`) = c("SWIGFunction", class('TriangleThresholdImageFilter_GetMaskValue'))

# Start of TriangleThresholdImageFilter_GetThreshold

`TriangleThresholdImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TriangleThresholdImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TriangleThresholdImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`TriangleThresholdImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter')
class(`TriangleThresholdImageFilter_GetThreshold`) = c("SWIGFunction", class('TriangleThresholdImageFilter_GetThreshold'))

# Start of TriangleThresholdImageFilter_GetName

`TriangleThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TriangleThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TriangleThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`TriangleThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter')
class(`TriangleThresholdImageFilter_GetName`) = c("SWIGFunction", class('TriangleThresholdImageFilter_GetName'))

# Start of TriangleThresholdImageFilter_ToString

`TriangleThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_TriangleThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`TriangleThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`TriangleThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter')
class(`TriangleThresholdImageFilter_ToString`) = c("SWIGFunction", class('TriangleThresholdImageFilter_ToString'))

# Start of TriangleThresholdImageFilter_Execute

`TriangleThresholdImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_TriangleThresholdImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TriangleThresholdImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`TriangleThresholdImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`TriangleThresholdImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('TriangleThresholdImageFilter_Execute__SWIG_0'))

# Start of TriangleThresholdImageFilter_Execute

`TriangleThresholdImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_TriangleThresholdImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TriangleThresholdImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`TriangleThresholdImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__TriangleThresholdImageFilter', '_p_itk__simple__Image')
class(`TriangleThresholdImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('TriangleThresholdImageFilter_Execute__SWIG_1'))

`TriangleThresholdImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__TriangleThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- TriangleThresholdImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__TriangleThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- TriangleThresholdImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for TriangleThresholdImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::TriangleThresholdImageFilter
setMethod('$', '_p_itk__simple__TriangleThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetInsideValue' = TriangleThresholdImageFilter_SetInsideValue, 'GetInsideValue' = TriangleThresholdImageFilter_GetInsideValue, 'SetOutsideValue' = TriangleThresholdImageFilter_SetOutsideValue, 'GetOutsideValue' = TriangleThresholdImageFilter_GetOutsideValue, 'SetNumberOfHistogramBins' = TriangleThresholdImageFilter_SetNumberOfHistogramBins, 'GetNumberOfHistogramBins' = TriangleThresholdImageFilter_GetNumberOfHistogramBins, 'SetMaskOutput' = TriangleThresholdImageFilter_SetMaskOutput, 'MaskOutputOn' = TriangleThresholdImageFilter_MaskOutputOn, 'MaskOutputOff' = TriangleThresholdImageFilter_MaskOutputOff, 'GetMaskOutput' = TriangleThresholdImageFilter_GetMaskOutput, 'SetMaskValue' = TriangleThresholdImageFilter_SetMaskValue, 'GetMaskValue' = TriangleThresholdImageFilter_GetMaskValue, 'GetThreshold' = TriangleThresholdImageFilter_GetThreshold, 'GetName' = TriangleThresholdImageFilter_GetName, 'ToString' = TriangleThresholdImageFilter_ToString, 'Execute' = TriangleThresholdImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::TriangleThresholdImageFilter
# Start of TriangleThreshold

`TriangleThreshold__SWIG_0` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_TriangleThreshold__SWIG_0', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TriangleThreshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`TriangleThreshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`TriangleThreshold__SWIG_0`) = c("SWIGFunction", class('TriangleThreshold__SWIG_0'))

# Start of TriangleThreshold

`TriangleThreshold__SWIG_1` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_TriangleThreshold__SWIG_1', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TriangleThreshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`TriangleThreshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`TriangleThreshold__SWIG_1`) = c("SWIGFunction", class('TriangleThreshold__SWIG_1'))

# Start of TriangleThreshold

`TriangleThreshold__SWIG_2` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_TriangleThreshold__SWIG_2', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TriangleThreshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`TriangleThreshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`TriangleThreshold__SWIG_2`) = c("SWIGFunction", class('TriangleThreshold__SWIG_2'))

# Start of TriangleThreshold

`TriangleThreshold__SWIG_3` = function(image, maskImage, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_TriangleThreshold__SWIG_3', image, maskImage, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TriangleThreshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`TriangleThreshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`TriangleThreshold__SWIG_3`) = c("SWIGFunction", class('TriangleThreshold__SWIG_3'))

# Start of TriangleThreshold

`TriangleThreshold__SWIG_4` = function(image, maskImage, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_TriangleThreshold__SWIG_4', image, maskImage, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TriangleThreshold__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`TriangleThreshold__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`TriangleThreshold__SWIG_4`) = c("SWIGFunction", class('TriangleThreshold__SWIG_4'))

# Start of TriangleThreshold

`TriangleThreshold__SWIG_5` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_TriangleThreshold__SWIG_5', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TriangleThreshold__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`TriangleThreshold__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`TriangleThreshold__SWIG_5`) = c("SWIGFunction", class('TriangleThreshold__SWIG_5'))

# Start of TriangleThreshold

`TriangleThreshold__SWIG_6` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_TriangleThreshold__SWIG_6', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TriangleThreshold__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`TriangleThreshold__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`TriangleThreshold__SWIG_6`) = c("SWIGFunction", class('TriangleThreshold__SWIG_6'))

# Start of TriangleThreshold

`TriangleThreshold__SWIG_7` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_TriangleThreshold__SWIG_7', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TriangleThreshold__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`TriangleThreshold__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`TriangleThreshold__SWIG_7`) = c("SWIGFunction", class('TriangleThreshold__SWIG_7'))

# Start of TriangleThreshold

`TriangleThreshold__SWIG_8` = function(image, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_TriangleThreshold__SWIG_8', image, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TriangleThreshold__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`TriangleThreshold__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`TriangleThreshold__SWIG_8`) = c("SWIGFunction", class('TriangleThreshold__SWIG_8'))

# Start of TriangleThreshold

`TriangleThreshold__SWIG_9` = function(image, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_TriangleThreshold__SWIG_9', image, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TriangleThreshold__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`TriangleThreshold__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`TriangleThreshold__SWIG_9`) = c("SWIGFunction", class('TriangleThreshold__SWIG_9'))

# Start of TriangleThreshold

`TriangleThreshold__SWIG_10` = function(image, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_TriangleThreshold__SWIG_10', image, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TriangleThreshold__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`TriangleThreshold__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`TriangleThreshold__SWIG_10`) = c("SWIGFunction", class('TriangleThreshold__SWIG_10'))

# Start of TriangleThreshold

`TriangleThreshold__SWIG_11` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_TriangleThreshold__SWIG_11', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`TriangleThreshold__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`TriangleThreshold__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image')
class(`TriangleThreshold__SWIG_11`) = c("SWIGFunction", class('TriangleThreshold__SWIG_11'))

`TriangleThreshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 12
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- TriangleThreshold__SWIG_11; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- TriangleThreshold__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- TriangleThreshold__SWIG_10; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- TriangleThreshold__SWIG_4; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- TriangleThreshold__SWIG_9; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- TriangleThreshold__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- TriangleThreshold__SWIG_8; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- TriangleThreshold__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- TriangleThreshold__SWIG_7; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- TriangleThreshold__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- TriangleThreshold__SWIG_6; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- TriangleThreshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for TriangleThreshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_UnaryMinusImageFilter

`delete_UnaryMinusImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_UnaryMinusImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_UnaryMinusImageFilter`, 'returnType') = 'void'
attr(`delete_UnaryMinusImageFilter`, "inputTypes") = c('_p_itk__simple__UnaryMinusImageFilter')
class(`delete_UnaryMinusImageFilter`) = c("SWIGFunction", class('delete_UnaryMinusImageFilter'))

# Start of new_UnaryMinusImageFilter

`UnaryMinusImageFilter` = function()
{
  ;ans = .Call('R_swig_new_UnaryMinusImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__UnaryMinusImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_UnaryMinusImageFilter);
  ans
  
}

attr(`UnaryMinusImageFilter`, 'returnType') = '_p_itk__simple__UnaryMinusImageFilter'
class(`UnaryMinusImageFilter`) = c("SWIGFunction", class('UnaryMinusImageFilter'))

# Start of UnaryMinusImageFilter_GetName

`UnaryMinusImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_UnaryMinusImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`UnaryMinusImageFilter_GetName`, 'returnType') = 'character'
attr(`UnaryMinusImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__UnaryMinusImageFilter')
class(`UnaryMinusImageFilter_GetName`) = c("SWIGFunction", class('UnaryMinusImageFilter_GetName'))

# Start of UnaryMinusImageFilter_ToString

`UnaryMinusImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_UnaryMinusImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`UnaryMinusImageFilter_ToString`, 'returnType') = 'character'
attr(`UnaryMinusImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__UnaryMinusImageFilter')
class(`UnaryMinusImageFilter_ToString`) = c("SWIGFunction", class('UnaryMinusImageFilter_ToString'))

# Start of UnaryMinusImageFilter_Execute

`UnaryMinusImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_UnaryMinusImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`UnaryMinusImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`UnaryMinusImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__UnaryMinusImageFilter', '_p_itk__simple__Image')
class(`UnaryMinusImageFilter_Execute`) = c("SWIGFunction", class('UnaryMinusImageFilter_Execute'))

# Start of accessor method for itk::simple::UnaryMinusImageFilter
setMethod('$', '_p_itk__simple__UnaryMinusImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = UnaryMinusImageFilter_GetName, 'ToString' = UnaryMinusImageFilter_ToString, 'Execute' = UnaryMinusImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::UnaryMinusImageFilter
# Start of UnaryMinus

`UnaryMinus` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_UnaryMinus', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`UnaryMinus`, 'returnType') = '_p_itk__simple__Image'
attr(`UnaryMinus`, "inputTypes") = c('_p_itk__simple__Image')
class(`UnaryMinus`) = c("SWIGFunction", class('UnaryMinus'))

# Start of delete_UnsharpMaskImageFilter

`delete_UnsharpMaskImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_UnsharpMaskImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_UnsharpMaskImageFilter`, 'returnType') = 'void'
attr(`delete_UnsharpMaskImageFilter`, "inputTypes") = c('_p_itk__simple__UnsharpMaskImageFilter')
class(`delete_UnsharpMaskImageFilter`) = c("SWIGFunction", class('delete_UnsharpMaskImageFilter'))

# Start of new_UnsharpMaskImageFilter

`UnsharpMaskImageFilter` = function()
{
  ;ans = .Call('R_swig_new_UnsharpMaskImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__UnsharpMaskImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_UnsharpMaskImageFilter);
  ans
  
}

attr(`UnsharpMaskImageFilter`, 'returnType') = '_p_itk__simple__UnsharpMaskImageFilter'
class(`UnsharpMaskImageFilter`) = c("SWIGFunction", class('UnsharpMaskImageFilter'))

# Start of UnsharpMaskImageFilter_SetSigmas

`UnsharpMaskImageFilter_SetSigmas__SWIG_0` = function(self, s)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_UnsharpMaskImageFilter_SetSigmas__SWIG_0', self, s, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`UnsharpMaskImageFilter_SetSigmas__SWIG_0`, 'returnType') = 'void'
attr(`UnsharpMaskImageFilter_SetSigmas__SWIG_0`, "inputTypes") = c('_p_itk__simple__UnsharpMaskImageFilter', 'numeric')
class(`UnsharpMaskImageFilter_SetSigmas__SWIG_0`) = c("SWIGFunction", class('UnsharpMaskImageFilter_SetSigmas__SWIG_0'))

# Start of UnsharpMaskImageFilter_SetSigmas

`UnsharpMaskImageFilter_SetSigmas__SWIG_1` = function(self, Sigmas)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Sigmas = as.numeric(Sigmas);
  ;ans = .Call('R_swig_UnsharpMaskImageFilter_SetSigmas__SWIG_1', self, Sigmas, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`UnsharpMaskImageFilter_SetSigmas__SWIG_1`, 'returnType') = 'void'
attr(`UnsharpMaskImageFilter_SetSigmas__SWIG_1`, "inputTypes") = c('_p_itk__simple__UnsharpMaskImageFilter', 'numeric')
class(`UnsharpMaskImageFilter_SetSigmas__SWIG_1`) = c("SWIGFunction", class('UnsharpMaskImageFilter_SetSigmas__SWIG_1'))

`UnsharpMaskImageFilter_SetSigmas` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__UnsharpMaskImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- UnsharpMaskImageFilter_SetSigmas__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__UnsharpMaskImageFilter') || is.null(argv[[1]]) ) && ( is.numeric(argv[[2]]) )) {
      f <- UnsharpMaskImageFilter_SetSigmas__SWIG_1; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for UnsharpMaskImageFilter_SetSigmas with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of UnsharpMaskImageFilter_GetSigmas

`UnsharpMaskImageFilter_GetSigmas` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_UnsharpMaskImageFilter_GetSigmas', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`UnsharpMaskImageFilter_GetSigmas`, 'returnType') = 'numeric'
attr(`UnsharpMaskImageFilter_GetSigmas`, "inputTypes") = c('_p_itk__simple__UnsharpMaskImageFilter')
class(`UnsharpMaskImageFilter_GetSigmas`) = c("SWIGFunction", class('UnsharpMaskImageFilter_GetSigmas'))

# Start of UnsharpMaskImageFilter_SetAmount

`UnsharpMaskImageFilter_SetAmount` = function(self, Amount)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_UnsharpMaskImageFilter_SetAmount', self, Amount, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`UnsharpMaskImageFilter_SetAmount`, 'returnType') = 'void'
attr(`UnsharpMaskImageFilter_SetAmount`, "inputTypes") = c('_p_itk__simple__UnsharpMaskImageFilter', 'numeric')
class(`UnsharpMaskImageFilter_SetAmount`) = c("SWIGFunction", class('UnsharpMaskImageFilter_SetAmount'))

# Start of UnsharpMaskImageFilter_GetAmount

`UnsharpMaskImageFilter_GetAmount` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_UnsharpMaskImageFilter_GetAmount', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`UnsharpMaskImageFilter_GetAmount`, 'returnType') = 'numeric'
attr(`UnsharpMaskImageFilter_GetAmount`, "inputTypes") = c('_p_itk__simple__UnsharpMaskImageFilter')
class(`UnsharpMaskImageFilter_GetAmount`) = c("SWIGFunction", class('UnsharpMaskImageFilter_GetAmount'))

# Start of UnsharpMaskImageFilter_SetThreshold

`UnsharpMaskImageFilter_SetThreshold` = function(self, Threshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_UnsharpMaskImageFilter_SetThreshold', self, Threshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`UnsharpMaskImageFilter_SetThreshold`, 'returnType') = 'void'
attr(`UnsharpMaskImageFilter_SetThreshold`, "inputTypes") = c('_p_itk__simple__UnsharpMaskImageFilter', 'numeric')
class(`UnsharpMaskImageFilter_SetThreshold`) = c("SWIGFunction", class('UnsharpMaskImageFilter_SetThreshold'))

# Start of UnsharpMaskImageFilter_GetThreshold

`UnsharpMaskImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_UnsharpMaskImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`UnsharpMaskImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`UnsharpMaskImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__UnsharpMaskImageFilter')
class(`UnsharpMaskImageFilter_GetThreshold`) = c("SWIGFunction", class('UnsharpMaskImageFilter_GetThreshold'))

# Start of UnsharpMaskImageFilter_SetClamp

`UnsharpMaskImageFilter_SetClamp` = function(self, Clamp)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Clamp = as.logical(Clamp);
  ;ans = .Call('R_swig_UnsharpMaskImageFilter_SetClamp', self, Clamp, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`UnsharpMaskImageFilter_SetClamp`, 'returnType') = 'void'
attr(`UnsharpMaskImageFilter_SetClamp`, "inputTypes") = c('_p_itk__simple__UnsharpMaskImageFilter', 'logical')
class(`UnsharpMaskImageFilter_SetClamp`) = c("SWIGFunction", class('UnsharpMaskImageFilter_SetClamp'))

# Start of UnsharpMaskImageFilter_ClampOn

`UnsharpMaskImageFilter_ClampOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_UnsharpMaskImageFilter_ClampOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`UnsharpMaskImageFilter_ClampOn`, 'returnType') = 'void'
attr(`UnsharpMaskImageFilter_ClampOn`, "inputTypes") = c('_p_itk__simple__UnsharpMaskImageFilter')
class(`UnsharpMaskImageFilter_ClampOn`) = c("SWIGFunction", class('UnsharpMaskImageFilter_ClampOn'))

# Start of UnsharpMaskImageFilter_ClampOff

`UnsharpMaskImageFilter_ClampOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_UnsharpMaskImageFilter_ClampOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`UnsharpMaskImageFilter_ClampOff`, 'returnType') = 'void'
attr(`UnsharpMaskImageFilter_ClampOff`, "inputTypes") = c('_p_itk__simple__UnsharpMaskImageFilter')
class(`UnsharpMaskImageFilter_ClampOff`) = c("SWIGFunction", class('UnsharpMaskImageFilter_ClampOff'))

# Start of UnsharpMaskImageFilter_GetClamp

`UnsharpMaskImageFilter_GetClamp` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_UnsharpMaskImageFilter_GetClamp', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`UnsharpMaskImageFilter_GetClamp`, 'returnType') = 'logical'
attr(`UnsharpMaskImageFilter_GetClamp`, "inputTypes") = c('_p_itk__simple__UnsharpMaskImageFilter')
class(`UnsharpMaskImageFilter_GetClamp`) = c("SWIGFunction", class('UnsharpMaskImageFilter_GetClamp'))

# Start of UnsharpMaskImageFilter_GetName

`UnsharpMaskImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_UnsharpMaskImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`UnsharpMaskImageFilter_GetName`, 'returnType') = 'character'
attr(`UnsharpMaskImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__UnsharpMaskImageFilter')
class(`UnsharpMaskImageFilter_GetName`) = c("SWIGFunction", class('UnsharpMaskImageFilter_GetName'))

# Start of UnsharpMaskImageFilter_ToString

`UnsharpMaskImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_UnsharpMaskImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`UnsharpMaskImageFilter_ToString`, 'returnType') = 'character'
attr(`UnsharpMaskImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__UnsharpMaskImageFilter')
class(`UnsharpMaskImageFilter_ToString`) = c("SWIGFunction", class('UnsharpMaskImageFilter_ToString'))

# Start of UnsharpMaskImageFilter_Execute

`UnsharpMaskImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_UnsharpMaskImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`UnsharpMaskImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`UnsharpMaskImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__UnsharpMaskImageFilter', '_p_itk__simple__Image')
class(`UnsharpMaskImageFilter_Execute`) = c("SWIGFunction", class('UnsharpMaskImageFilter_Execute'))

# Start of accessor method for itk::simple::UnsharpMaskImageFilter
setMethod('$', '_p_itk__simple__UnsharpMaskImageFilter', function(x, name)

{
  accessorFuns = list('SetSigmas' = UnsharpMaskImageFilter_SetSigmas, 'GetSigmas' = UnsharpMaskImageFilter_GetSigmas, 'SetAmount' = UnsharpMaskImageFilter_SetAmount, 'GetAmount' = UnsharpMaskImageFilter_GetAmount, 'SetThreshold' = UnsharpMaskImageFilter_SetThreshold, 'GetThreshold' = UnsharpMaskImageFilter_GetThreshold, 'SetClamp' = UnsharpMaskImageFilter_SetClamp, 'ClampOn' = UnsharpMaskImageFilter_ClampOn, 'ClampOff' = UnsharpMaskImageFilter_ClampOff, 'GetClamp' = UnsharpMaskImageFilter_GetClamp, 'GetName' = UnsharpMaskImageFilter_GetName, 'ToString' = UnsharpMaskImageFilter_ToString, 'Execute' = UnsharpMaskImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::UnsharpMaskImageFilter
# Start of UnsharpMask

`UnsharpMask__SWIG_0` = function(image1, sigmas, amount, threshold, clamp, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  sigmas = as.numeric(sigmas);
  
  
  clamp = as.logical(clamp);
  ;ans = .Call('R_swig_UnsharpMask__SWIG_0', image1, sigmas, amount, threshold, clamp, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`UnsharpMask__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`UnsharpMask__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric', 'logical')
class(`UnsharpMask__SWIG_0`) = c("SWIGFunction", class('UnsharpMask__SWIG_0'))

# Start of UnsharpMask

`UnsharpMask__SWIG_1` = function(image1, sigmas, amount, threshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  sigmas = as.numeric(sigmas);
  
  
  ;ans = .Call('R_swig_UnsharpMask__SWIG_1', image1, sigmas, amount, threshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`UnsharpMask__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`UnsharpMask__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric', 'numeric')
class(`UnsharpMask__SWIG_1`) = c("SWIGFunction", class('UnsharpMask__SWIG_1'))

# Start of UnsharpMask

`UnsharpMask__SWIG_2` = function(image1, sigmas, amount, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  sigmas = as.numeric(sigmas);
  
  ;ans = .Call('R_swig_UnsharpMask__SWIG_2', image1, sigmas, amount, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`UnsharpMask__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`UnsharpMask__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'numeric')
class(`UnsharpMask__SWIG_2`) = c("SWIGFunction", class('UnsharpMask__SWIG_2'))

# Start of UnsharpMask

`UnsharpMask__SWIG_3` = function(image1, sigmas, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  sigmas = as.numeric(sigmas);
  ;ans = .Call('R_swig_UnsharpMask__SWIG_3', image1, sigmas, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`UnsharpMask__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`UnsharpMask__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`UnsharpMask__SWIG_3`) = c("SWIGFunction", class('UnsharpMask__SWIG_3'))

# Start of UnsharpMask

`UnsharpMask__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_UnsharpMask__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`UnsharpMask__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`UnsharpMask__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`UnsharpMask__SWIG_4`) = c("SWIGFunction", class('UnsharpMask__SWIG_4'))

`UnsharpMask` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- UnsharpMask__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) )) {
      f <- UnsharpMask__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- UnsharpMask__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- UnsharpMask__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- UnsharpMask__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for UnsharpMask with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ValuedRegionalMaximaImageFilter

`delete_ValuedRegionalMaximaImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ValuedRegionalMaximaImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ValuedRegionalMaximaImageFilter`, 'returnType') = 'void'
attr(`delete_ValuedRegionalMaximaImageFilter`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMaximaImageFilter')
class(`delete_ValuedRegionalMaximaImageFilter`) = c("SWIGFunction", class('delete_ValuedRegionalMaximaImageFilter'))

# Start of new_ValuedRegionalMaximaImageFilter

`ValuedRegionalMaximaImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ValuedRegionalMaximaImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ValuedRegionalMaximaImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ValuedRegionalMaximaImageFilter);
  ans
  
}

attr(`ValuedRegionalMaximaImageFilter`, 'returnType') = '_p_itk__simple__ValuedRegionalMaximaImageFilter'
class(`ValuedRegionalMaximaImageFilter`) = c("SWIGFunction", class('ValuedRegionalMaximaImageFilter'))

# Start of ValuedRegionalMaximaImageFilter_SetFullyConnected

`ValuedRegionalMaximaImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_ValuedRegionalMaximaImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ValuedRegionalMaximaImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`ValuedRegionalMaximaImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMaximaImageFilter', 'logical')
class(`ValuedRegionalMaximaImageFilter_SetFullyConnected`) = c("SWIGFunction", class('ValuedRegionalMaximaImageFilter_SetFullyConnected'))

# Start of ValuedRegionalMaximaImageFilter_FullyConnectedOn

`ValuedRegionalMaximaImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ValuedRegionalMaximaImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ValuedRegionalMaximaImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`ValuedRegionalMaximaImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMaximaImageFilter')
class(`ValuedRegionalMaximaImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('ValuedRegionalMaximaImageFilter_FullyConnectedOn'))

# Start of ValuedRegionalMaximaImageFilter_FullyConnectedOff

`ValuedRegionalMaximaImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ValuedRegionalMaximaImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ValuedRegionalMaximaImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`ValuedRegionalMaximaImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMaximaImageFilter')
class(`ValuedRegionalMaximaImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('ValuedRegionalMaximaImageFilter_FullyConnectedOff'))

# Start of ValuedRegionalMaximaImageFilter_GetFullyConnected

`ValuedRegionalMaximaImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ValuedRegionalMaximaImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ValuedRegionalMaximaImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`ValuedRegionalMaximaImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMaximaImageFilter')
class(`ValuedRegionalMaximaImageFilter_GetFullyConnected`) = c("SWIGFunction", class('ValuedRegionalMaximaImageFilter_GetFullyConnected'))

# Start of ValuedRegionalMaximaImageFilter_GetFlat

`ValuedRegionalMaximaImageFilter_GetFlat` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ValuedRegionalMaximaImageFilter_GetFlat', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ValuedRegionalMaximaImageFilter_GetFlat`, 'returnType') = 'logical'
attr(`ValuedRegionalMaximaImageFilter_GetFlat`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMaximaImageFilter')
class(`ValuedRegionalMaximaImageFilter_GetFlat`) = c("SWIGFunction", class('ValuedRegionalMaximaImageFilter_GetFlat'))

# Start of ValuedRegionalMaximaImageFilter_GetName

`ValuedRegionalMaximaImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ValuedRegionalMaximaImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ValuedRegionalMaximaImageFilter_GetName`, 'returnType') = 'character'
attr(`ValuedRegionalMaximaImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMaximaImageFilter')
class(`ValuedRegionalMaximaImageFilter_GetName`) = c("SWIGFunction", class('ValuedRegionalMaximaImageFilter_GetName'))

# Start of ValuedRegionalMaximaImageFilter_ToString

`ValuedRegionalMaximaImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ValuedRegionalMaximaImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ValuedRegionalMaximaImageFilter_ToString`, 'returnType') = 'character'
attr(`ValuedRegionalMaximaImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMaximaImageFilter')
class(`ValuedRegionalMaximaImageFilter_ToString`) = c("SWIGFunction", class('ValuedRegionalMaximaImageFilter_ToString'))

# Start of ValuedRegionalMaximaImageFilter_Execute

`ValuedRegionalMaximaImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ValuedRegionalMaximaImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ValuedRegionalMaximaImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ValuedRegionalMaximaImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMaximaImageFilter', '_p_itk__simple__Image')
class(`ValuedRegionalMaximaImageFilter_Execute`) = c("SWIGFunction", class('ValuedRegionalMaximaImageFilter_Execute'))

# Start of accessor method for itk::simple::ValuedRegionalMaximaImageFilter
setMethod('$', '_p_itk__simple__ValuedRegionalMaximaImageFilter', function(x, name)

{
  accessorFuns = list('SetFullyConnected' = ValuedRegionalMaximaImageFilter_SetFullyConnected, 'FullyConnectedOn' = ValuedRegionalMaximaImageFilter_FullyConnectedOn, 'FullyConnectedOff' = ValuedRegionalMaximaImageFilter_FullyConnectedOff, 'GetFullyConnected' = ValuedRegionalMaximaImageFilter_GetFullyConnected, 'GetFlat' = ValuedRegionalMaximaImageFilter_GetFlat, 'GetName' = ValuedRegionalMaximaImageFilter_GetName, 'ToString' = ValuedRegionalMaximaImageFilter_ToString, 'Execute' = ValuedRegionalMaximaImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ValuedRegionalMaximaImageFilter
# Start of ValuedRegionalMaxima

`ValuedRegionalMaxima__SWIG_0` = function(image1, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_ValuedRegionalMaxima__SWIG_0', image1, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ValuedRegionalMaxima__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ValuedRegionalMaxima__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`ValuedRegionalMaxima__SWIG_0`) = c("SWIGFunction", class('ValuedRegionalMaxima__SWIG_0'))

# Start of ValuedRegionalMaxima

`ValuedRegionalMaxima__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ValuedRegionalMaxima__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ValuedRegionalMaxima__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ValuedRegionalMaxima__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`ValuedRegionalMaxima__SWIG_1`) = c("SWIGFunction", class('ValuedRegionalMaxima__SWIG_1'))

`ValuedRegionalMaxima` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ValuedRegionalMaxima__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ValuedRegionalMaxima__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ValuedRegionalMaxima with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ValuedRegionalMinimaImageFilter

`delete_ValuedRegionalMinimaImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ValuedRegionalMinimaImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ValuedRegionalMinimaImageFilter`, 'returnType') = 'void'
attr(`delete_ValuedRegionalMinimaImageFilter`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMinimaImageFilter')
class(`delete_ValuedRegionalMinimaImageFilter`) = c("SWIGFunction", class('delete_ValuedRegionalMinimaImageFilter'))

# Start of new_ValuedRegionalMinimaImageFilter

`ValuedRegionalMinimaImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ValuedRegionalMinimaImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ValuedRegionalMinimaImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ValuedRegionalMinimaImageFilter);
  ans
  
}

attr(`ValuedRegionalMinimaImageFilter`, 'returnType') = '_p_itk__simple__ValuedRegionalMinimaImageFilter'
class(`ValuedRegionalMinimaImageFilter`) = c("SWIGFunction", class('ValuedRegionalMinimaImageFilter'))

# Start of ValuedRegionalMinimaImageFilter_SetFullyConnected

`ValuedRegionalMinimaImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_ValuedRegionalMinimaImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ValuedRegionalMinimaImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`ValuedRegionalMinimaImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMinimaImageFilter', 'logical')
class(`ValuedRegionalMinimaImageFilter_SetFullyConnected`) = c("SWIGFunction", class('ValuedRegionalMinimaImageFilter_SetFullyConnected'))

# Start of ValuedRegionalMinimaImageFilter_FullyConnectedOn

`ValuedRegionalMinimaImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ValuedRegionalMinimaImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ValuedRegionalMinimaImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`ValuedRegionalMinimaImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMinimaImageFilter')
class(`ValuedRegionalMinimaImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('ValuedRegionalMinimaImageFilter_FullyConnectedOn'))

# Start of ValuedRegionalMinimaImageFilter_FullyConnectedOff

`ValuedRegionalMinimaImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_ValuedRegionalMinimaImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ValuedRegionalMinimaImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`ValuedRegionalMinimaImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMinimaImageFilter')
class(`ValuedRegionalMinimaImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('ValuedRegionalMinimaImageFilter_FullyConnectedOff'))

# Start of ValuedRegionalMinimaImageFilter_GetFullyConnected

`ValuedRegionalMinimaImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ValuedRegionalMinimaImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ValuedRegionalMinimaImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`ValuedRegionalMinimaImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMinimaImageFilter')
class(`ValuedRegionalMinimaImageFilter_GetFullyConnected`) = c("SWIGFunction", class('ValuedRegionalMinimaImageFilter_GetFullyConnected'))

# Start of ValuedRegionalMinimaImageFilter_GetFlat

`ValuedRegionalMinimaImageFilter_GetFlat` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ValuedRegionalMinimaImageFilter_GetFlat', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ValuedRegionalMinimaImageFilter_GetFlat`, 'returnType') = 'logical'
attr(`ValuedRegionalMinimaImageFilter_GetFlat`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMinimaImageFilter')
class(`ValuedRegionalMinimaImageFilter_GetFlat`) = c("SWIGFunction", class('ValuedRegionalMinimaImageFilter_GetFlat'))

# Start of ValuedRegionalMinimaImageFilter_GetName

`ValuedRegionalMinimaImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ValuedRegionalMinimaImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ValuedRegionalMinimaImageFilter_GetName`, 'returnType') = 'character'
attr(`ValuedRegionalMinimaImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMinimaImageFilter')
class(`ValuedRegionalMinimaImageFilter_GetName`) = c("SWIGFunction", class('ValuedRegionalMinimaImageFilter_GetName'))

# Start of ValuedRegionalMinimaImageFilter_ToString

`ValuedRegionalMinimaImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ValuedRegionalMinimaImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ValuedRegionalMinimaImageFilter_ToString`, 'returnType') = 'character'
attr(`ValuedRegionalMinimaImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMinimaImageFilter')
class(`ValuedRegionalMinimaImageFilter_ToString`) = c("SWIGFunction", class('ValuedRegionalMinimaImageFilter_ToString'))

# Start of ValuedRegionalMinimaImageFilter_Execute

`ValuedRegionalMinimaImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ValuedRegionalMinimaImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ValuedRegionalMinimaImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ValuedRegionalMinimaImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ValuedRegionalMinimaImageFilter', '_p_itk__simple__Image')
class(`ValuedRegionalMinimaImageFilter_Execute`) = c("SWIGFunction", class('ValuedRegionalMinimaImageFilter_Execute'))

# Start of accessor method for itk::simple::ValuedRegionalMinimaImageFilter
setMethod('$', '_p_itk__simple__ValuedRegionalMinimaImageFilter', function(x, name)

{
  accessorFuns = list('SetFullyConnected' = ValuedRegionalMinimaImageFilter_SetFullyConnected, 'FullyConnectedOn' = ValuedRegionalMinimaImageFilter_FullyConnectedOn, 'FullyConnectedOff' = ValuedRegionalMinimaImageFilter_FullyConnectedOff, 'GetFullyConnected' = ValuedRegionalMinimaImageFilter_GetFullyConnected, 'GetFlat' = ValuedRegionalMinimaImageFilter_GetFlat, 'GetName' = ValuedRegionalMinimaImageFilter_GetName, 'ToString' = ValuedRegionalMinimaImageFilter_ToString, 'Execute' = ValuedRegionalMinimaImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ValuedRegionalMinimaImageFilter
# Start of ValuedRegionalMinima

`ValuedRegionalMinima__SWIG_0` = function(image1, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_ValuedRegionalMinima__SWIG_0', image1, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ValuedRegionalMinima__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ValuedRegionalMinima__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'logical')
class(`ValuedRegionalMinima__SWIG_0`) = c("SWIGFunction", class('ValuedRegionalMinima__SWIG_0'))

# Start of ValuedRegionalMinima

`ValuedRegionalMinima__SWIG_1` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ValuedRegionalMinima__SWIG_1', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ValuedRegionalMinima__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ValuedRegionalMinima__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image')
class(`ValuedRegionalMinima__SWIG_1`) = c("SWIGFunction", class('ValuedRegionalMinima__SWIG_1'))

`ValuedRegionalMinima` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ValuedRegionalMinima__SWIG_1; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.logical(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ValuedRegionalMinima__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ValuedRegionalMinima with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_VectorConfidenceConnectedImageFilter

`delete_VectorConfidenceConnectedImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_VectorConfidenceConnectedImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_VectorConfidenceConnectedImageFilter`, 'returnType') = 'void'
attr(`delete_VectorConfidenceConnectedImageFilter`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter')
class(`delete_VectorConfidenceConnectedImageFilter`) = c("SWIGFunction", class('delete_VectorConfidenceConnectedImageFilter'))

# Start of new_VectorConfidenceConnectedImageFilter

`VectorConfidenceConnectedImageFilter` = function()
{
  ;ans = .Call('R_swig_new_VectorConfidenceConnectedImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VectorConfidenceConnectedImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VectorConfidenceConnectedImageFilter);
  ans
  
}

attr(`VectorConfidenceConnectedImageFilter`, 'returnType') = '_p_itk__simple__VectorConfidenceConnectedImageFilter'
class(`VectorConfidenceConnectedImageFilter`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter'))

# Start of VectorConfidenceConnectedImageFilter_SetSeedList

`VectorConfidenceConnectedImageFilter_SetSeedList` = function(self, SeedList)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SeedList = lapply(SeedList, as.integer);
  ;ans = .Call('R_swig_VectorConfidenceConnectedImageFilter_SetSeedList', self, SeedList, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VectorConfidenceConnectedImageFilter_SetSeedList`, 'returnType') = 'void'
attr(`VectorConfidenceConnectedImageFilter_SetSeedList`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter', 'list')
class(`VectorConfidenceConnectedImageFilter_SetSeedList`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_SetSeedList'))

# Start of VectorConfidenceConnectedImageFilter_GetSeedList

`VectorConfidenceConnectedImageFilter_GetSeedList` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorConfidenceConnectedImageFilter_GetSeedList', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorConfidenceConnectedImageFilter_GetSeedList`, 'returnType') = 'list'
attr(`VectorConfidenceConnectedImageFilter_GetSeedList`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter')
class(`VectorConfidenceConnectedImageFilter_GetSeedList`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_GetSeedList'))

# Start of VectorConfidenceConnectedImageFilter_AddSeed

`VectorConfidenceConnectedImageFilter_AddSeed` = function(self, point)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  point = as.integer(point);
  ;ans = .Call('R_swig_VectorConfidenceConnectedImageFilter_AddSeed', self, point, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VectorConfidenceConnectedImageFilter_AddSeed`, 'returnType') = 'void'
attr(`VectorConfidenceConnectedImageFilter_AddSeed`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter', 'integer')
class(`VectorConfidenceConnectedImageFilter_AddSeed`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_AddSeed'))

# Start of VectorConfidenceConnectedImageFilter_ClearSeeds

`VectorConfidenceConnectedImageFilter_ClearSeeds` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorConfidenceConnectedImageFilter_ClearSeeds', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VectorConfidenceConnectedImageFilter_ClearSeeds`, 'returnType') = 'void'
attr(`VectorConfidenceConnectedImageFilter_ClearSeeds`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter')
class(`VectorConfidenceConnectedImageFilter_ClearSeeds`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_ClearSeeds'))

# Start of VectorConfidenceConnectedImageFilter_SetNumberOfIterations

`VectorConfidenceConnectedImageFilter_SetNumberOfIterations` = function(self, NumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfIterations = as.integer(NumberOfIterations);
  
  if(length(NumberOfIterations) > 1) {
    warning("using only the first element of NumberOfIterations");
  };
  
  ;ans = .Call('R_swig_VectorConfidenceConnectedImageFilter_SetNumberOfIterations', self, NumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VectorConfidenceConnectedImageFilter_SetNumberOfIterations`, 'returnType') = 'void'
attr(`VectorConfidenceConnectedImageFilter_SetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter', 'integer')
class(`VectorConfidenceConnectedImageFilter_SetNumberOfIterations`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_SetNumberOfIterations'))

# Start of VectorConfidenceConnectedImageFilter_GetNumberOfIterations

`VectorConfidenceConnectedImageFilter_GetNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorConfidenceConnectedImageFilter_GetNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorConfidenceConnectedImageFilter_GetNumberOfIterations`, 'returnType') = 'integer'
attr(`VectorConfidenceConnectedImageFilter_GetNumberOfIterations`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter')
class(`VectorConfidenceConnectedImageFilter_GetNumberOfIterations`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_GetNumberOfIterations'))

# Start of VectorConfidenceConnectedImageFilter_SetMultiplier

`VectorConfidenceConnectedImageFilter_SetMultiplier` = function(self, Multiplier)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_VectorConfidenceConnectedImageFilter_SetMultiplier', self, Multiplier, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VectorConfidenceConnectedImageFilter_SetMultiplier`, 'returnType') = 'void'
attr(`VectorConfidenceConnectedImageFilter_SetMultiplier`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter', 'numeric')
class(`VectorConfidenceConnectedImageFilter_SetMultiplier`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_SetMultiplier'))

# Start of VectorConfidenceConnectedImageFilter_GetMultiplier

`VectorConfidenceConnectedImageFilter_GetMultiplier` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorConfidenceConnectedImageFilter_GetMultiplier', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorConfidenceConnectedImageFilter_GetMultiplier`, 'returnType') = 'numeric'
attr(`VectorConfidenceConnectedImageFilter_GetMultiplier`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter')
class(`VectorConfidenceConnectedImageFilter_GetMultiplier`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_GetMultiplier'))

# Start of VectorConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius

`VectorConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius` = function(self, InitialNeighborhoodRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InitialNeighborhoodRadius = as.integer(InitialNeighborhoodRadius);
  
  if(length(InitialNeighborhoodRadius) > 1) {
    warning("using only the first element of InitialNeighborhoodRadius");
  };
  
  ;ans = .Call('R_swig_VectorConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius', self, InitialNeighborhoodRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VectorConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius`, 'returnType') = 'void'
attr(`VectorConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter', 'integer')
class(`VectorConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius'))

# Start of VectorConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius

`VectorConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius`, 'returnType') = 'integer'
attr(`VectorConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter')
class(`VectorConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius'))

# Start of VectorConfidenceConnectedImageFilter_SetReplaceValue

`VectorConfidenceConnectedImageFilter_SetReplaceValue` = function(self, ReplaceValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ReplaceValue = as.integer(ReplaceValue);
  
  if(length(ReplaceValue) > 1) {
    warning("using only the first element of ReplaceValue");
  };
  
  ;ans = .Call('R_swig_VectorConfidenceConnectedImageFilter_SetReplaceValue', self, ReplaceValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VectorConfidenceConnectedImageFilter_SetReplaceValue`, 'returnType') = 'void'
attr(`VectorConfidenceConnectedImageFilter_SetReplaceValue`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter', 'integer')
class(`VectorConfidenceConnectedImageFilter_SetReplaceValue`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_SetReplaceValue'))

# Start of VectorConfidenceConnectedImageFilter_GetReplaceValue

`VectorConfidenceConnectedImageFilter_GetReplaceValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorConfidenceConnectedImageFilter_GetReplaceValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorConfidenceConnectedImageFilter_GetReplaceValue`, 'returnType') = 'integer'
attr(`VectorConfidenceConnectedImageFilter_GetReplaceValue`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter')
class(`VectorConfidenceConnectedImageFilter_GetReplaceValue`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_GetReplaceValue'))

# Start of VectorConfidenceConnectedImageFilter_GetMean

`VectorConfidenceConnectedImageFilter_GetMean` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorConfidenceConnectedImageFilter_GetMean', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorConfidenceConnectedImageFilter_GetMean`, 'returnType') = 'numeric'
attr(`VectorConfidenceConnectedImageFilter_GetMean`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter')
class(`VectorConfidenceConnectedImageFilter_GetMean`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_GetMean'))

# Start of VectorConfidenceConnectedImageFilter_GetCovariance

`VectorConfidenceConnectedImageFilter_GetCovariance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorConfidenceConnectedImageFilter_GetCovariance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorConfidenceConnectedImageFilter_GetCovariance`, 'returnType') = 'numeric'
attr(`VectorConfidenceConnectedImageFilter_GetCovariance`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter')
class(`VectorConfidenceConnectedImageFilter_GetCovariance`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_GetCovariance'))

# Start of VectorConfidenceConnectedImageFilter_GetName

`VectorConfidenceConnectedImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorConfidenceConnectedImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorConfidenceConnectedImageFilter_GetName`, 'returnType') = 'character'
attr(`VectorConfidenceConnectedImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter')
class(`VectorConfidenceConnectedImageFilter_GetName`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_GetName'))

# Start of VectorConfidenceConnectedImageFilter_ToString

`VectorConfidenceConnectedImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorConfidenceConnectedImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorConfidenceConnectedImageFilter_ToString`, 'returnType') = 'character'
attr(`VectorConfidenceConnectedImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter')
class(`VectorConfidenceConnectedImageFilter_ToString`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_ToString'))

# Start of VectorConfidenceConnectedImageFilter_Execute

`VectorConfidenceConnectedImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_VectorConfidenceConnectedImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorConfidenceConnectedImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorConfidenceConnectedImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__VectorConfidenceConnectedImageFilter', '_p_itk__simple__Image')
class(`VectorConfidenceConnectedImageFilter_Execute`) = c("SWIGFunction", class('VectorConfidenceConnectedImageFilter_Execute'))

# Start of accessor method for itk::simple::VectorConfidenceConnectedImageFilter
setMethod('$', '_p_itk__simple__VectorConfidenceConnectedImageFilter', function(x, name)

{
  accessorFuns = list('SetSeedList' = VectorConfidenceConnectedImageFilter_SetSeedList, 'GetSeedList' = VectorConfidenceConnectedImageFilter_GetSeedList, 'AddSeed' = VectorConfidenceConnectedImageFilter_AddSeed, 'ClearSeeds' = VectorConfidenceConnectedImageFilter_ClearSeeds, 'SetNumberOfIterations' = VectorConfidenceConnectedImageFilter_SetNumberOfIterations, 'GetNumberOfIterations' = VectorConfidenceConnectedImageFilter_GetNumberOfIterations, 'SetMultiplier' = VectorConfidenceConnectedImageFilter_SetMultiplier, 'GetMultiplier' = VectorConfidenceConnectedImageFilter_GetMultiplier, 'SetInitialNeighborhoodRadius' = VectorConfidenceConnectedImageFilter_SetInitialNeighborhoodRadius, 'GetInitialNeighborhoodRadius' = VectorConfidenceConnectedImageFilter_GetInitialNeighborhoodRadius, 'SetReplaceValue' = VectorConfidenceConnectedImageFilter_SetReplaceValue, 'GetReplaceValue' = VectorConfidenceConnectedImageFilter_GetReplaceValue, 'GetMean' = VectorConfidenceConnectedImageFilter_GetMean, 'GetCovariance' = VectorConfidenceConnectedImageFilter_GetCovariance, 'GetName' = VectorConfidenceConnectedImageFilter_GetName, 'ToString' = VectorConfidenceConnectedImageFilter_ToString, 'Execute' = VectorConfidenceConnectedImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::VectorConfidenceConnectedImageFilter
# Start of VectorConfidenceConnected

`VectorConfidenceConnected__SWIG_0` = function(image1, seedList, numberOfIterations, multiplier, initialNeighborhoodRadius, replaceValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  initialNeighborhoodRadius = as.integer(initialNeighborhoodRadius);
  
  if(length(initialNeighborhoodRadius) > 1) {
    warning("using only the first element of initialNeighborhoodRadius");
  };
  
  replaceValue = as.integer(replaceValue);
  
  if(length(replaceValue) > 1) {
    warning("using only the first element of replaceValue");
  };
  
  ;ans = .Call('R_swig_VectorConfidenceConnected__SWIG_0', image1, seedList, numberOfIterations, multiplier, initialNeighborhoodRadius, replaceValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorConfidenceConnected__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorConfidenceConnected__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'integer', 'numeric', 'integer', 'integer')
class(`VectorConfidenceConnected__SWIG_0`) = c("SWIGFunction", class('VectorConfidenceConnected__SWIG_0'))

# Start of VectorConfidenceConnected

`VectorConfidenceConnected__SWIG_1` = function(image1, seedList, numberOfIterations, multiplier, initialNeighborhoodRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  initialNeighborhoodRadius = as.integer(initialNeighborhoodRadius);
  
  if(length(initialNeighborhoodRadius) > 1) {
    warning("using only the first element of initialNeighborhoodRadius");
  };
  
  ;ans = .Call('R_swig_VectorConfidenceConnected__SWIG_1', image1, seedList, numberOfIterations, multiplier, initialNeighborhoodRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorConfidenceConnected__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorConfidenceConnected__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'integer', 'numeric', 'integer')
class(`VectorConfidenceConnected__SWIG_1`) = c("SWIGFunction", class('VectorConfidenceConnected__SWIG_1'))

# Start of VectorConfidenceConnected

`VectorConfidenceConnected__SWIG_2` = function(image1, seedList, numberOfIterations, multiplier, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  
  ;ans = .Call('R_swig_VectorConfidenceConnected__SWIG_2', image1, seedList, numberOfIterations, multiplier, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorConfidenceConnected__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorConfidenceConnected__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'integer', 'numeric')
class(`VectorConfidenceConnected__SWIG_2`) = c("SWIGFunction", class('VectorConfidenceConnected__SWIG_2'))

# Start of VectorConfidenceConnected

`VectorConfidenceConnected__SWIG_3` = function(image1, seedList, numberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  numberOfIterations = as.integer(numberOfIterations);
  
  if(length(numberOfIterations) > 1) {
    warning("using only the first element of numberOfIterations");
  };
  
  ;ans = .Call('R_swig_VectorConfidenceConnected__SWIG_3', image1, seedList, numberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorConfidenceConnected__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorConfidenceConnected__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'list', 'integer')
class(`VectorConfidenceConnected__SWIG_3`) = c("SWIGFunction", class('VectorConfidenceConnected__SWIG_3'))

# Start of VectorConfidenceConnected

`VectorConfidenceConnected__SWIG_4` = function(image1, seedList, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  seedList = lapply(seedList, as.integer);
  ;ans = .Call('R_swig_VectorConfidenceConnected__SWIG_4', image1, seedList, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorConfidenceConnected__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorConfidenceConnected__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'list')
class(`VectorConfidenceConnected__SWIG_4`) = c("SWIGFunction", class('VectorConfidenceConnected__SWIG_4'))

# Start of VectorConfidenceConnected

`VectorConfidenceConnected__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_VectorConfidenceConnected__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorConfidenceConnected__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorConfidenceConnected__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`VectorConfidenceConnected__SWIG_5`) = c("SWIGFunction", class('VectorConfidenceConnected__SWIG_5'))

`VectorConfidenceConnected` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- VectorConfidenceConnected__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) )) {
      f <- VectorConfidenceConnected__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- VectorConfidenceConnected__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- VectorConfidenceConnected__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- VectorConfidenceConnected__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.list(argv[[2]]) && all(sapply(argv[[2]] , is.integer) || sapply(argv[[2]], is.numeric)) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- VectorConfidenceConnected__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorConfidenceConnected with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_VectorConnectedComponentImageFilter

`delete_VectorConnectedComponentImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_VectorConnectedComponentImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_VectorConnectedComponentImageFilter`, 'returnType') = 'void'
attr(`delete_VectorConnectedComponentImageFilter`, "inputTypes") = c('_p_itk__simple__VectorConnectedComponentImageFilter')
class(`delete_VectorConnectedComponentImageFilter`) = c("SWIGFunction", class('delete_VectorConnectedComponentImageFilter'))

# Start of new_VectorConnectedComponentImageFilter

`VectorConnectedComponentImageFilter` = function()
{
  ;ans = .Call('R_swig_new_VectorConnectedComponentImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VectorConnectedComponentImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VectorConnectedComponentImageFilter);
  ans
  
}

attr(`VectorConnectedComponentImageFilter`, 'returnType') = '_p_itk__simple__VectorConnectedComponentImageFilter'
class(`VectorConnectedComponentImageFilter`) = c("SWIGFunction", class('VectorConnectedComponentImageFilter'))

# Start of VectorConnectedComponentImageFilter_SetDistanceThreshold

`VectorConnectedComponentImageFilter_SetDistanceThreshold` = function(self, DistanceThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_VectorConnectedComponentImageFilter_SetDistanceThreshold', self, DistanceThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VectorConnectedComponentImageFilter_SetDistanceThreshold`, 'returnType') = 'void'
attr(`VectorConnectedComponentImageFilter_SetDistanceThreshold`, "inputTypes") = c('_p_itk__simple__VectorConnectedComponentImageFilter', 'numeric')
class(`VectorConnectedComponentImageFilter_SetDistanceThreshold`) = c("SWIGFunction", class('VectorConnectedComponentImageFilter_SetDistanceThreshold'))

# Start of VectorConnectedComponentImageFilter_GetDistanceThreshold

`VectorConnectedComponentImageFilter_GetDistanceThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorConnectedComponentImageFilter_GetDistanceThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorConnectedComponentImageFilter_GetDistanceThreshold`, 'returnType') = 'numeric'
attr(`VectorConnectedComponentImageFilter_GetDistanceThreshold`, "inputTypes") = c('_p_itk__simple__VectorConnectedComponentImageFilter')
class(`VectorConnectedComponentImageFilter_GetDistanceThreshold`) = c("SWIGFunction", class('VectorConnectedComponentImageFilter_GetDistanceThreshold'))

# Start of VectorConnectedComponentImageFilter_SetFullyConnected

`VectorConnectedComponentImageFilter_SetFullyConnected` = function(self, FullyConnected)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  FullyConnected = as.logical(FullyConnected);
  ;ans = .Call('R_swig_VectorConnectedComponentImageFilter_SetFullyConnected', self, FullyConnected, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VectorConnectedComponentImageFilter_SetFullyConnected`, 'returnType') = 'void'
attr(`VectorConnectedComponentImageFilter_SetFullyConnected`, "inputTypes") = c('_p_itk__simple__VectorConnectedComponentImageFilter', 'logical')
class(`VectorConnectedComponentImageFilter_SetFullyConnected`) = c("SWIGFunction", class('VectorConnectedComponentImageFilter_SetFullyConnected'))

# Start of VectorConnectedComponentImageFilter_FullyConnectedOn

`VectorConnectedComponentImageFilter_FullyConnectedOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorConnectedComponentImageFilter_FullyConnectedOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VectorConnectedComponentImageFilter_FullyConnectedOn`, 'returnType') = 'void'
attr(`VectorConnectedComponentImageFilter_FullyConnectedOn`, "inputTypes") = c('_p_itk__simple__VectorConnectedComponentImageFilter')
class(`VectorConnectedComponentImageFilter_FullyConnectedOn`) = c("SWIGFunction", class('VectorConnectedComponentImageFilter_FullyConnectedOn'))

# Start of VectorConnectedComponentImageFilter_FullyConnectedOff

`VectorConnectedComponentImageFilter_FullyConnectedOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorConnectedComponentImageFilter_FullyConnectedOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VectorConnectedComponentImageFilter_FullyConnectedOff`, 'returnType') = 'void'
attr(`VectorConnectedComponentImageFilter_FullyConnectedOff`, "inputTypes") = c('_p_itk__simple__VectorConnectedComponentImageFilter')
class(`VectorConnectedComponentImageFilter_FullyConnectedOff`) = c("SWIGFunction", class('VectorConnectedComponentImageFilter_FullyConnectedOff'))

# Start of VectorConnectedComponentImageFilter_GetFullyConnected

`VectorConnectedComponentImageFilter_GetFullyConnected` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorConnectedComponentImageFilter_GetFullyConnected', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorConnectedComponentImageFilter_GetFullyConnected`, 'returnType') = 'logical'
attr(`VectorConnectedComponentImageFilter_GetFullyConnected`, "inputTypes") = c('_p_itk__simple__VectorConnectedComponentImageFilter')
class(`VectorConnectedComponentImageFilter_GetFullyConnected`) = c("SWIGFunction", class('VectorConnectedComponentImageFilter_GetFullyConnected'))

# Start of VectorConnectedComponentImageFilter_GetName

`VectorConnectedComponentImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorConnectedComponentImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorConnectedComponentImageFilter_GetName`, 'returnType') = 'character'
attr(`VectorConnectedComponentImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__VectorConnectedComponentImageFilter')
class(`VectorConnectedComponentImageFilter_GetName`) = c("SWIGFunction", class('VectorConnectedComponentImageFilter_GetName'))

# Start of VectorConnectedComponentImageFilter_ToString

`VectorConnectedComponentImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorConnectedComponentImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorConnectedComponentImageFilter_ToString`, 'returnType') = 'character'
attr(`VectorConnectedComponentImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__VectorConnectedComponentImageFilter')
class(`VectorConnectedComponentImageFilter_ToString`) = c("SWIGFunction", class('VectorConnectedComponentImageFilter_ToString'))

# Start of VectorConnectedComponentImageFilter_Execute

`VectorConnectedComponentImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_VectorConnectedComponentImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorConnectedComponentImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorConnectedComponentImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__VectorConnectedComponentImageFilter', '_p_itk__simple__Image')
class(`VectorConnectedComponentImageFilter_Execute`) = c("SWIGFunction", class('VectorConnectedComponentImageFilter_Execute'))

# Start of accessor method for itk::simple::VectorConnectedComponentImageFilter
setMethod('$', '_p_itk__simple__VectorConnectedComponentImageFilter', function(x, name)

{
  accessorFuns = list('SetDistanceThreshold' = VectorConnectedComponentImageFilter_SetDistanceThreshold, 'GetDistanceThreshold' = VectorConnectedComponentImageFilter_GetDistanceThreshold, 'SetFullyConnected' = VectorConnectedComponentImageFilter_SetFullyConnected, 'FullyConnectedOn' = VectorConnectedComponentImageFilter_FullyConnectedOn, 'FullyConnectedOff' = VectorConnectedComponentImageFilter_FullyConnectedOff, 'GetFullyConnected' = VectorConnectedComponentImageFilter_GetFullyConnected, 'GetName' = VectorConnectedComponentImageFilter_GetName, 'ToString' = VectorConnectedComponentImageFilter_ToString, 'Execute' = VectorConnectedComponentImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::VectorConnectedComponentImageFilter
# Start of VectorConnectedComponent

`VectorConnectedComponent__SWIG_0` = function(image1, distanceThreshold, fullyConnected, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  fullyConnected = as.logical(fullyConnected);
  ;ans = .Call('R_swig_VectorConnectedComponent__SWIG_0', image1, distanceThreshold, fullyConnected, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorConnectedComponent__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorConnectedComponent__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'logical')
class(`VectorConnectedComponent__SWIG_0`) = c("SWIGFunction", class('VectorConnectedComponent__SWIG_0'))

# Start of VectorConnectedComponent

`VectorConnectedComponent__SWIG_1` = function(image1, distanceThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_VectorConnectedComponent__SWIG_1', image1, distanceThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorConnectedComponent__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorConnectedComponent__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`VectorConnectedComponent__SWIG_1`) = c("SWIGFunction", class('VectorConnectedComponent__SWIG_1'))

# Start of VectorConnectedComponent

`VectorConnectedComponent__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_VectorConnectedComponent__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorConnectedComponent__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorConnectedComponent__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`VectorConnectedComponent__SWIG_2`) = c("SWIGFunction", class('VectorConnectedComponent__SWIG_2'))

`VectorConnectedComponent` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- VectorConnectedComponent__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- VectorConnectedComponent__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( is.logical(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- VectorConnectedComponent__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorConnectedComponent with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_VectorIndexSelectionCastImageFilter

`delete_VectorIndexSelectionCastImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_VectorIndexSelectionCastImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_VectorIndexSelectionCastImageFilter`, 'returnType') = 'void'
attr(`delete_VectorIndexSelectionCastImageFilter`, "inputTypes") = c('_p_itk__simple__VectorIndexSelectionCastImageFilter')
class(`delete_VectorIndexSelectionCastImageFilter`) = c("SWIGFunction", class('delete_VectorIndexSelectionCastImageFilter'))

# Start of new_VectorIndexSelectionCastImageFilter

`VectorIndexSelectionCastImageFilter` = function()
{
  ;ans = .Call('R_swig_new_VectorIndexSelectionCastImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VectorIndexSelectionCastImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VectorIndexSelectionCastImageFilter);
  ans
  
}

attr(`VectorIndexSelectionCastImageFilter`, 'returnType') = '_p_itk__simple__VectorIndexSelectionCastImageFilter'
class(`VectorIndexSelectionCastImageFilter`) = c("SWIGFunction", class('VectorIndexSelectionCastImageFilter'))

# Start of VectorIndexSelectionCastImageFilter_SetIndex

`VectorIndexSelectionCastImageFilter_SetIndex` = function(self, Index)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Index = as.integer(Index);
  
  if(length(Index) > 1) {
    warning("using only the first element of Index");
  };
  
  ;ans = .Call('R_swig_VectorIndexSelectionCastImageFilter_SetIndex', self, Index, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VectorIndexSelectionCastImageFilter_SetIndex`, 'returnType') = 'void'
attr(`VectorIndexSelectionCastImageFilter_SetIndex`, "inputTypes") = c('_p_itk__simple__VectorIndexSelectionCastImageFilter', 'integer')
class(`VectorIndexSelectionCastImageFilter_SetIndex`) = c("SWIGFunction", class('VectorIndexSelectionCastImageFilter_SetIndex'))

# Start of VectorIndexSelectionCastImageFilter_GetIndex

`VectorIndexSelectionCastImageFilter_GetIndex` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorIndexSelectionCastImageFilter_GetIndex', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorIndexSelectionCastImageFilter_GetIndex`, 'returnType') = 'integer'
attr(`VectorIndexSelectionCastImageFilter_GetIndex`, "inputTypes") = c('_p_itk__simple__VectorIndexSelectionCastImageFilter')
class(`VectorIndexSelectionCastImageFilter_GetIndex`) = c("SWIGFunction", class('VectorIndexSelectionCastImageFilter_GetIndex'))

# Start of VectorIndexSelectionCastImageFilter_SetOutputPixelType

`VectorIndexSelectionCastImageFilter_SetOutputPixelType` = function(self, OutputPixelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputPixelType = enumToInteger(OutputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(OutputPixelType) > 1) {
    warning("using only the first element of OutputPixelType");
  };
  
  ;ans = .Call('R_swig_VectorIndexSelectionCastImageFilter_SetOutputPixelType', self, OutputPixelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VectorIndexSelectionCastImageFilter_SetOutputPixelType`, 'returnType') = 'void'
attr(`VectorIndexSelectionCastImageFilter_SetOutputPixelType`, "inputTypes") = c('_p_itk__simple__VectorIndexSelectionCastImageFilter', 'character')
class(`VectorIndexSelectionCastImageFilter_SetOutputPixelType`) = c("SWIGFunction", class('VectorIndexSelectionCastImageFilter_SetOutputPixelType'))

# Start of VectorIndexSelectionCastImageFilter_GetOutputPixelType

`VectorIndexSelectionCastImageFilter_GetOutputPixelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_VectorIndexSelectionCastImageFilter_GetOutputPixelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__PixelIDValueEnum");
  
  ans
  
}

attr(`VectorIndexSelectionCastImageFilter_GetOutputPixelType`, 'returnType') = 'character'
attr(`VectorIndexSelectionCastImageFilter_GetOutputPixelType`, "inputTypes") = c('_p_itk__simple__VectorIndexSelectionCastImageFilter')
class(`VectorIndexSelectionCastImageFilter_GetOutputPixelType`) = c("SWIGFunction", class('VectorIndexSelectionCastImageFilter_GetOutputPixelType'))

# Start of VectorIndexSelectionCastImageFilter_GetName

`VectorIndexSelectionCastImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorIndexSelectionCastImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorIndexSelectionCastImageFilter_GetName`, 'returnType') = 'character'
attr(`VectorIndexSelectionCastImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__VectorIndexSelectionCastImageFilter')
class(`VectorIndexSelectionCastImageFilter_GetName`) = c("SWIGFunction", class('VectorIndexSelectionCastImageFilter_GetName'))

# Start of VectorIndexSelectionCastImageFilter_ToString

`VectorIndexSelectionCastImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorIndexSelectionCastImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorIndexSelectionCastImageFilter_ToString`, 'returnType') = 'character'
attr(`VectorIndexSelectionCastImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__VectorIndexSelectionCastImageFilter')
class(`VectorIndexSelectionCastImageFilter_ToString`) = c("SWIGFunction", class('VectorIndexSelectionCastImageFilter_ToString'))

# Start of VectorIndexSelectionCastImageFilter_Execute

`VectorIndexSelectionCastImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_VectorIndexSelectionCastImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorIndexSelectionCastImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorIndexSelectionCastImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__VectorIndexSelectionCastImageFilter', '_p_itk__simple__Image')
class(`VectorIndexSelectionCastImageFilter_Execute`) = c("SWIGFunction", class('VectorIndexSelectionCastImageFilter_Execute'))

# Start of accessor method for itk::simple::VectorIndexSelectionCastImageFilter
setMethod('$', '_p_itk__simple__VectorIndexSelectionCastImageFilter', function(x, name)

{
  accessorFuns = list('SetIndex' = VectorIndexSelectionCastImageFilter_SetIndex, 'GetIndex' = VectorIndexSelectionCastImageFilter_GetIndex, 'SetOutputPixelType' = VectorIndexSelectionCastImageFilter_SetOutputPixelType, 'GetOutputPixelType' = VectorIndexSelectionCastImageFilter_GetOutputPixelType, 'GetName' = VectorIndexSelectionCastImageFilter_GetName, 'ToString' = VectorIndexSelectionCastImageFilter_ToString, 'Execute' = VectorIndexSelectionCastImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::VectorIndexSelectionCastImageFilter
# Start of VectorIndexSelectionCast

`VectorIndexSelectionCast__SWIG_0` = function(image1, index, outputPixelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  index = as.integer(index);
  
  if(length(index) > 1) {
    warning("using only the first element of index");
  };
  
  outputPixelType = enumToInteger(outputPixelType, "_itk__simple__PixelIDValueEnum"); 
  
  if(length(outputPixelType) > 1) {
    warning("using only the first element of outputPixelType");
  };
  
  ;ans = .Call('R_swig_VectorIndexSelectionCast__SWIG_0', image1, index, outputPixelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorIndexSelectionCast__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorIndexSelectionCast__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`VectorIndexSelectionCast__SWIG_0`) = c("SWIGFunction", class('VectorIndexSelectionCast__SWIG_0'))

# Start of VectorIndexSelectionCast

`VectorIndexSelectionCast__SWIG_1` = function(image1, index, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  index = as.integer(index);
  
  if(length(index) > 1) {
    warning("using only the first element of index");
  };
  
  ;ans = .Call('R_swig_VectorIndexSelectionCast__SWIG_1', image1, index, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorIndexSelectionCast__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorIndexSelectionCast__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`VectorIndexSelectionCast__SWIG_1`) = c("SWIGFunction", class('VectorIndexSelectionCast__SWIG_1'))

# Start of VectorIndexSelectionCast

`VectorIndexSelectionCast__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_VectorIndexSelectionCast__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorIndexSelectionCast__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorIndexSelectionCast__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`VectorIndexSelectionCast__SWIG_2`) = c("SWIGFunction", class('VectorIndexSelectionCast__SWIG_2'))

`VectorIndexSelectionCast` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- VectorIndexSelectionCast__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VectorIndexSelectionCast__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- VectorIndexSelectionCast__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VectorIndexSelectionCast with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_VectorMagnitudeImageFilter

`delete_VectorMagnitudeImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_VectorMagnitudeImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_VectorMagnitudeImageFilter`, 'returnType') = 'void'
attr(`delete_VectorMagnitudeImageFilter`, "inputTypes") = c('_p_itk__simple__VectorMagnitudeImageFilter')
class(`delete_VectorMagnitudeImageFilter`) = c("SWIGFunction", class('delete_VectorMagnitudeImageFilter'))

# Start of new_VectorMagnitudeImageFilter

`VectorMagnitudeImageFilter` = function()
{
  ;ans = .Call('R_swig_new_VectorMagnitudeImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VectorMagnitudeImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VectorMagnitudeImageFilter);
  ans
  
}

attr(`VectorMagnitudeImageFilter`, 'returnType') = '_p_itk__simple__VectorMagnitudeImageFilter'
class(`VectorMagnitudeImageFilter`) = c("SWIGFunction", class('VectorMagnitudeImageFilter'))

# Start of VectorMagnitudeImageFilter_GetName

`VectorMagnitudeImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorMagnitudeImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorMagnitudeImageFilter_GetName`, 'returnType') = 'character'
attr(`VectorMagnitudeImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__VectorMagnitudeImageFilter')
class(`VectorMagnitudeImageFilter_GetName`) = c("SWIGFunction", class('VectorMagnitudeImageFilter_GetName'))

# Start of VectorMagnitudeImageFilter_ToString

`VectorMagnitudeImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VectorMagnitudeImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VectorMagnitudeImageFilter_ToString`, 'returnType') = 'character'
attr(`VectorMagnitudeImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__VectorMagnitudeImageFilter')
class(`VectorMagnitudeImageFilter_ToString`) = c("SWIGFunction", class('VectorMagnitudeImageFilter_ToString'))

# Start of VectorMagnitudeImageFilter_Execute

`VectorMagnitudeImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_VectorMagnitudeImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorMagnitudeImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorMagnitudeImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__VectorMagnitudeImageFilter', '_p_itk__simple__Image')
class(`VectorMagnitudeImageFilter_Execute`) = c("SWIGFunction", class('VectorMagnitudeImageFilter_Execute'))

# Start of accessor method for itk::simple::VectorMagnitudeImageFilter
setMethod('$', '_p_itk__simple__VectorMagnitudeImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = VectorMagnitudeImageFilter_GetName, 'ToString' = VectorMagnitudeImageFilter_ToString, 'Execute' = VectorMagnitudeImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::VectorMagnitudeImageFilter
# Start of VectorMagnitude

`VectorMagnitude` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_VectorMagnitude', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VectorMagnitude`, 'returnType') = '_p_itk__simple__Image'
attr(`VectorMagnitude`, "inputTypes") = c('_p_itk__simple__Image')
class(`VectorMagnitude`) = c("SWIGFunction", class('VectorMagnitude'))

# Start of delete_VotingBinaryHoleFillingImageFilter

`delete_VotingBinaryHoleFillingImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_VotingBinaryHoleFillingImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_VotingBinaryHoleFillingImageFilter`, 'returnType') = 'void'
attr(`delete_VotingBinaryHoleFillingImageFilter`, "inputTypes") = c('_p_itk__simple__VotingBinaryHoleFillingImageFilter')
class(`delete_VotingBinaryHoleFillingImageFilter`) = c("SWIGFunction", class('delete_VotingBinaryHoleFillingImageFilter'))

# Start of new_VotingBinaryHoleFillingImageFilter

`VotingBinaryHoleFillingImageFilter` = function()
{
  ;ans = .Call('R_swig_new_VotingBinaryHoleFillingImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VotingBinaryHoleFillingImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VotingBinaryHoleFillingImageFilter);
  ans
  
}

attr(`VotingBinaryHoleFillingImageFilter`, 'returnType') = '_p_itk__simple__VotingBinaryHoleFillingImageFilter'
class(`VotingBinaryHoleFillingImageFilter`) = c("SWIGFunction", class('VotingBinaryHoleFillingImageFilter'))

# Start of VotingBinaryHoleFillingImageFilter_SetRadius

`VotingBinaryHoleFillingImageFilter_SetRadius__SWIG_0` = function(self, Radius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Radius = as.integer(Radius);
  ;ans = .Call('R_swig_VotingBinaryHoleFillingImageFilter_SetRadius__SWIG_0', self, Radius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryHoleFillingImageFilter_SetRadius__SWIG_0`, 'returnType') = 'void'
attr(`VotingBinaryHoleFillingImageFilter_SetRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__VotingBinaryHoleFillingImageFilter', 'integer')
class(`VotingBinaryHoleFillingImageFilter_SetRadius__SWIG_0`) = c("SWIGFunction", class('VotingBinaryHoleFillingImageFilter_SetRadius__SWIG_0'))

# Start of VotingBinaryHoleFillingImageFilter_SetRadius

`VotingBinaryHoleFillingImageFilter_SetRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_VotingBinaryHoleFillingImageFilter_SetRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryHoleFillingImageFilter_SetRadius__SWIG_1`, 'returnType') = 'void'
attr(`VotingBinaryHoleFillingImageFilter_SetRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__VotingBinaryHoleFillingImageFilter', 'integer')
class(`VotingBinaryHoleFillingImageFilter_SetRadius__SWIG_1`) = c("SWIGFunction", class('VotingBinaryHoleFillingImageFilter_SetRadius__SWIG_1'))

`VotingBinaryHoleFillingImageFilter_SetRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__VotingBinaryHoleFillingImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VotingBinaryHoleFillingImageFilter_SetRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__VotingBinaryHoleFillingImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- VotingBinaryHoleFillingImageFilter_SetRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VotingBinaryHoleFillingImageFilter_SetRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VotingBinaryHoleFillingImageFilter_GetRadius

`VotingBinaryHoleFillingImageFilter_GetRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryHoleFillingImageFilter_GetRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryHoleFillingImageFilter_GetRadius`, 'returnType') = 'integer'
attr(`VotingBinaryHoleFillingImageFilter_GetRadius`, "inputTypes") = c('_p_itk__simple__VotingBinaryHoleFillingImageFilter')
class(`VotingBinaryHoleFillingImageFilter_GetRadius`) = c("SWIGFunction", class('VotingBinaryHoleFillingImageFilter_GetRadius'))

# Start of VotingBinaryHoleFillingImageFilter_SetMajorityThreshold

`VotingBinaryHoleFillingImageFilter_SetMajorityThreshold` = function(self, MajorityThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MajorityThreshold = as.integer(MajorityThreshold);
  
  if(length(MajorityThreshold) > 1) {
    warning("using only the first element of MajorityThreshold");
  };
  
  ;ans = .Call('R_swig_VotingBinaryHoleFillingImageFilter_SetMajorityThreshold', self, MajorityThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryHoleFillingImageFilter_SetMajorityThreshold`, 'returnType') = 'void'
attr(`VotingBinaryHoleFillingImageFilter_SetMajorityThreshold`, "inputTypes") = c('_p_itk__simple__VotingBinaryHoleFillingImageFilter', 'integer')
class(`VotingBinaryHoleFillingImageFilter_SetMajorityThreshold`) = c("SWIGFunction", class('VotingBinaryHoleFillingImageFilter_SetMajorityThreshold'))

# Start of VotingBinaryHoleFillingImageFilter_GetMajorityThreshold

`VotingBinaryHoleFillingImageFilter_GetMajorityThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryHoleFillingImageFilter_GetMajorityThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryHoleFillingImageFilter_GetMajorityThreshold`, 'returnType') = 'integer'
attr(`VotingBinaryHoleFillingImageFilter_GetMajorityThreshold`, "inputTypes") = c('_p_itk__simple__VotingBinaryHoleFillingImageFilter')
class(`VotingBinaryHoleFillingImageFilter_GetMajorityThreshold`) = c("SWIGFunction", class('VotingBinaryHoleFillingImageFilter_GetMajorityThreshold'))

# Start of VotingBinaryHoleFillingImageFilter_SetForegroundValue

`VotingBinaryHoleFillingImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_VotingBinaryHoleFillingImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryHoleFillingImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`VotingBinaryHoleFillingImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__VotingBinaryHoleFillingImageFilter', 'numeric')
class(`VotingBinaryHoleFillingImageFilter_SetForegroundValue`) = c("SWIGFunction", class('VotingBinaryHoleFillingImageFilter_SetForegroundValue'))

# Start of VotingBinaryHoleFillingImageFilter_GetForegroundValue

`VotingBinaryHoleFillingImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryHoleFillingImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryHoleFillingImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`VotingBinaryHoleFillingImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__VotingBinaryHoleFillingImageFilter')
class(`VotingBinaryHoleFillingImageFilter_GetForegroundValue`) = c("SWIGFunction", class('VotingBinaryHoleFillingImageFilter_GetForegroundValue'))

# Start of VotingBinaryHoleFillingImageFilter_SetBackgroundValue

`VotingBinaryHoleFillingImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_VotingBinaryHoleFillingImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryHoleFillingImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`VotingBinaryHoleFillingImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__VotingBinaryHoleFillingImageFilter', 'numeric')
class(`VotingBinaryHoleFillingImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('VotingBinaryHoleFillingImageFilter_SetBackgroundValue'))

# Start of VotingBinaryHoleFillingImageFilter_GetBackgroundValue

`VotingBinaryHoleFillingImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryHoleFillingImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryHoleFillingImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`VotingBinaryHoleFillingImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__VotingBinaryHoleFillingImageFilter')
class(`VotingBinaryHoleFillingImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('VotingBinaryHoleFillingImageFilter_GetBackgroundValue'))

# Start of VotingBinaryHoleFillingImageFilter_GetName

`VotingBinaryHoleFillingImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryHoleFillingImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryHoleFillingImageFilter_GetName`, 'returnType') = 'character'
attr(`VotingBinaryHoleFillingImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__VotingBinaryHoleFillingImageFilter')
class(`VotingBinaryHoleFillingImageFilter_GetName`) = c("SWIGFunction", class('VotingBinaryHoleFillingImageFilter_GetName'))

# Start of VotingBinaryHoleFillingImageFilter_ToString

`VotingBinaryHoleFillingImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryHoleFillingImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryHoleFillingImageFilter_ToString`, 'returnType') = 'character'
attr(`VotingBinaryHoleFillingImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__VotingBinaryHoleFillingImageFilter')
class(`VotingBinaryHoleFillingImageFilter_ToString`) = c("SWIGFunction", class('VotingBinaryHoleFillingImageFilter_ToString'))

# Start of VotingBinaryHoleFillingImageFilter_Execute

`VotingBinaryHoleFillingImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_VotingBinaryHoleFillingImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinaryHoleFillingImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinaryHoleFillingImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__VotingBinaryHoleFillingImageFilter', '_p_itk__simple__Image')
class(`VotingBinaryHoleFillingImageFilter_Execute`) = c("SWIGFunction", class('VotingBinaryHoleFillingImageFilter_Execute'))

# Start of accessor method for itk::simple::VotingBinaryHoleFillingImageFilter
setMethod('$', '_p_itk__simple__VotingBinaryHoleFillingImageFilter', function(x, name)

{
  accessorFuns = list('SetRadius' = VotingBinaryHoleFillingImageFilter_SetRadius, 'GetRadius' = VotingBinaryHoleFillingImageFilter_GetRadius, 'SetMajorityThreshold' = VotingBinaryHoleFillingImageFilter_SetMajorityThreshold, 'GetMajorityThreshold' = VotingBinaryHoleFillingImageFilter_GetMajorityThreshold, 'SetForegroundValue' = VotingBinaryHoleFillingImageFilter_SetForegroundValue, 'GetForegroundValue' = VotingBinaryHoleFillingImageFilter_GetForegroundValue, 'SetBackgroundValue' = VotingBinaryHoleFillingImageFilter_SetBackgroundValue, 'GetBackgroundValue' = VotingBinaryHoleFillingImageFilter_GetBackgroundValue, 'GetName' = VotingBinaryHoleFillingImageFilter_GetName, 'ToString' = VotingBinaryHoleFillingImageFilter_ToString, 'Execute' = VotingBinaryHoleFillingImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::VotingBinaryHoleFillingImageFilter
# Start of VotingBinaryHoleFilling

`VotingBinaryHoleFilling__SWIG_0` = function(image1, radius, majorityThreshold, foregroundValue, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  majorityThreshold = as.integer(majorityThreshold);
  
  if(length(majorityThreshold) > 1) {
    warning("using only the first element of majorityThreshold");
  };
  
  
  
  ;ans = .Call('R_swig_VotingBinaryHoleFilling__SWIG_0', image1, radius, majorityThreshold, foregroundValue, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinaryHoleFilling__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinaryHoleFilling__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'numeric', 'numeric')
class(`VotingBinaryHoleFilling__SWIG_0`) = c("SWIGFunction", class('VotingBinaryHoleFilling__SWIG_0'))

# Start of VotingBinaryHoleFilling

`VotingBinaryHoleFilling__SWIG_1` = function(image1, radius, majorityThreshold, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  majorityThreshold = as.integer(majorityThreshold);
  
  if(length(majorityThreshold) > 1) {
    warning("using only the first element of majorityThreshold");
  };
  
  
  ;ans = .Call('R_swig_VotingBinaryHoleFilling__SWIG_1', image1, radius, majorityThreshold, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinaryHoleFilling__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinaryHoleFilling__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'numeric')
class(`VotingBinaryHoleFilling__SWIG_1`) = c("SWIGFunction", class('VotingBinaryHoleFilling__SWIG_1'))

# Start of VotingBinaryHoleFilling

`VotingBinaryHoleFilling__SWIG_2` = function(image1, radius, majorityThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  majorityThreshold = as.integer(majorityThreshold);
  
  if(length(majorityThreshold) > 1) {
    warning("using only the first element of majorityThreshold");
  };
  
  ;ans = .Call('R_swig_VotingBinaryHoleFilling__SWIG_2', image1, radius, majorityThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinaryHoleFilling__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinaryHoleFilling__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`VotingBinaryHoleFilling__SWIG_2`) = c("SWIGFunction", class('VotingBinaryHoleFilling__SWIG_2'))

# Start of VotingBinaryHoleFilling

`VotingBinaryHoleFilling__SWIG_3` = function(image1, radius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  ;ans = .Call('R_swig_VotingBinaryHoleFilling__SWIG_3', image1, radius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinaryHoleFilling__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinaryHoleFilling__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`VotingBinaryHoleFilling__SWIG_3`) = c("SWIGFunction", class('VotingBinaryHoleFilling__SWIG_3'))

# Start of VotingBinaryHoleFilling

`VotingBinaryHoleFilling__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_VotingBinaryHoleFilling__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinaryHoleFilling__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinaryHoleFilling__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`VotingBinaryHoleFilling__SWIG_4`) = c("SWIGFunction", class('VotingBinaryHoleFilling__SWIG_4'))

`VotingBinaryHoleFilling` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- VotingBinaryHoleFilling__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- VotingBinaryHoleFilling__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- VotingBinaryHoleFilling__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- VotingBinaryHoleFilling__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( is.numeric(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- VotingBinaryHoleFilling__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VotingBinaryHoleFilling with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_VotingBinaryImageFilter

`delete_VotingBinaryImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_VotingBinaryImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_VotingBinaryImageFilter`, 'returnType') = 'void'
attr(`delete_VotingBinaryImageFilter`, "inputTypes") = c('_p_itk__simple__VotingBinaryImageFilter')
class(`delete_VotingBinaryImageFilter`) = c("SWIGFunction", class('delete_VotingBinaryImageFilter'))

# Start of new_VotingBinaryImageFilter

`VotingBinaryImageFilter` = function()
{
  ;ans = .Call('R_swig_new_VotingBinaryImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VotingBinaryImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VotingBinaryImageFilter);
  ans
  
}

attr(`VotingBinaryImageFilter`, 'returnType') = '_p_itk__simple__VotingBinaryImageFilter'
class(`VotingBinaryImageFilter`) = c("SWIGFunction", class('VotingBinaryImageFilter'))

# Start of VotingBinaryImageFilter_SetRadius

`VotingBinaryImageFilter_SetRadius__SWIG_0` = function(self, Radius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Radius = as.integer(Radius);
  ;ans = .Call('R_swig_VotingBinaryImageFilter_SetRadius__SWIG_0', self, Radius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryImageFilter_SetRadius__SWIG_0`, 'returnType') = 'void'
attr(`VotingBinaryImageFilter_SetRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__VotingBinaryImageFilter', 'integer')
class(`VotingBinaryImageFilter_SetRadius__SWIG_0`) = c("SWIGFunction", class('VotingBinaryImageFilter_SetRadius__SWIG_0'))

# Start of VotingBinaryImageFilter_SetRadius

`VotingBinaryImageFilter_SetRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_VotingBinaryImageFilter_SetRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryImageFilter_SetRadius__SWIG_1`, 'returnType') = 'void'
attr(`VotingBinaryImageFilter_SetRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__VotingBinaryImageFilter', 'integer')
class(`VotingBinaryImageFilter_SetRadius__SWIG_1`) = c("SWIGFunction", class('VotingBinaryImageFilter_SetRadius__SWIG_1'))

`VotingBinaryImageFilter_SetRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__VotingBinaryImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VotingBinaryImageFilter_SetRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__VotingBinaryImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- VotingBinaryImageFilter_SetRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VotingBinaryImageFilter_SetRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VotingBinaryImageFilter_GetRadius

`VotingBinaryImageFilter_GetRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryImageFilter_GetRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryImageFilter_GetRadius`, 'returnType') = 'integer'
attr(`VotingBinaryImageFilter_GetRadius`, "inputTypes") = c('_p_itk__simple__VotingBinaryImageFilter')
class(`VotingBinaryImageFilter_GetRadius`) = c("SWIGFunction", class('VotingBinaryImageFilter_GetRadius'))

# Start of VotingBinaryImageFilter_SetBirthThreshold

`VotingBinaryImageFilter_SetBirthThreshold` = function(self, BirthThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BirthThreshold = as.integer(BirthThreshold);
  
  if(length(BirthThreshold) > 1) {
    warning("using only the first element of BirthThreshold");
  };
  
  ;ans = .Call('R_swig_VotingBinaryImageFilter_SetBirthThreshold', self, BirthThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryImageFilter_SetBirthThreshold`, 'returnType') = 'void'
attr(`VotingBinaryImageFilter_SetBirthThreshold`, "inputTypes") = c('_p_itk__simple__VotingBinaryImageFilter', 'integer')
class(`VotingBinaryImageFilter_SetBirthThreshold`) = c("SWIGFunction", class('VotingBinaryImageFilter_SetBirthThreshold'))

# Start of VotingBinaryImageFilter_GetBirthThreshold

`VotingBinaryImageFilter_GetBirthThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryImageFilter_GetBirthThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryImageFilter_GetBirthThreshold`, 'returnType') = 'integer'
attr(`VotingBinaryImageFilter_GetBirthThreshold`, "inputTypes") = c('_p_itk__simple__VotingBinaryImageFilter')
class(`VotingBinaryImageFilter_GetBirthThreshold`) = c("SWIGFunction", class('VotingBinaryImageFilter_GetBirthThreshold'))

# Start of VotingBinaryImageFilter_SetSurvivalThreshold

`VotingBinaryImageFilter_SetSurvivalThreshold` = function(self, SurvivalThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SurvivalThreshold = as.integer(SurvivalThreshold);
  
  if(length(SurvivalThreshold) > 1) {
    warning("using only the first element of SurvivalThreshold");
  };
  
  ;ans = .Call('R_swig_VotingBinaryImageFilter_SetSurvivalThreshold', self, SurvivalThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryImageFilter_SetSurvivalThreshold`, 'returnType') = 'void'
attr(`VotingBinaryImageFilter_SetSurvivalThreshold`, "inputTypes") = c('_p_itk__simple__VotingBinaryImageFilter', 'integer')
class(`VotingBinaryImageFilter_SetSurvivalThreshold`) = c("SWIGFunction", class('VotingBinaryImageFilter_SetSurvivalThreshold'))

# Start of VotingBinaryImageFilter_GetSurvivalThreshold

`VotingBinaryImageFilter_GetSurvivalThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryImageFilter_GetSurvivalThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryImageFilter_GetSurvivalThreshold`, 'returnType') = 'integer'
attr(`VotingBinaryImageFilter_GetSurvivalThreshold`, "inputTypes") = c('_p_itk__simple__VotingBinaryImageFilter')
class(`VotingBinaryImageFilter_GetSurvivalThreshold`) = c("SWIGFunction", class('VotingBinaryImageFilter_GetSurvivalThreshold'))

# Start of VotingBinaryImageFilter_SetForegroundValue

`VotingBinaryImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_VotingBinaryImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`VotingBinaryImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__VotingBinaryImageFilter', 'numeric')
class(`VotingBinaryImageFilter_SetForegroundValue`) = c("SWIGFunction", class('VotingBinaryImageFilter_SetForegroundValue'))

# Start of VotingBinaryImageFilter_GetForegroundValue

`VotingBinaryImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`VotingBinaryImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__VotingBinaryImageFilter')
class(`VotingBinaryImageFilter_GetForegroundValue`) = c("SWIGFunction", class('VotingBinaryImageFilter_GetForegroundValue'))

# Start of VotingBinaryImageFilter_SetBackgroundValue

`VotingBinaryImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_VotingBinaryImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`VotingBinaryImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__VotingBinaryImageFilter', 'numeric')
class(`VotingBinaryImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('VotingBinaryImageFilter_SetBackgroundValue'))

# Start of VotingBinaryImageFilter_GetBackgroundValue

`VotingBinaryImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`VotingBinaryImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__VotingBinaryImageFilter')
class(`VotingBinaryImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('VotingBinaryImageFilter_GetBackgroundValue'))

# Start of VotingBinaryImageFilter_GetName

`VotingBinaryImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryImageFilter_GetName`, 'returnType') = 'character'
attr(`VotingBinaryImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__VotingBinaryImageFilter')
class(`VotingBinaryImageFilter_GetName`) = c("SWIGFunction", class('VotingBinaryImageFilter_GetName'))

# Start of VotingBinaryImageFilter_ToString

`VotingBinaryImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryImageFilter_ToString`, 'returnType') = 'character'
attr(`VotingBinaryImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__VotingBinaryImageFilter')
class(`VotingBinaryImageFilter_ToString`) = c("SWIGFunction", class('VotingBinaryImageFilter_ToString'))

# Start of VotingBinaryImageFilter_Execute

`VotingBinaryImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_VotingBinaryImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinaryImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinaryImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__VotingBinaryImageFilter', '_p_itk__simple__Image')
class(`VotingBinaryImageFilter_Execute`) = c("SWIGFunction", class('VotingBinaryImageFilter_Execute'))

# Start of accessor method for itk::simple::VotingBinaryImageFilter
setMethod('$', '_p_itk__simple__VotingBinaryImageFilter', function(x, name)

{
  accessorFuns = list('SetRadius' = VotingBinaryImageFilter_SetRadius, 'GetRadius' = VotingBinaryImageFilter_GetRadius, 'SetBirthThreshold' = VotingBinaryImageFilter_SetBirthThreshold, 'GetBirthThreshold' = VotingBinaryImageFilter_GetBirthThreshold, 'SetSurvivalThreshold' = VotingBinaryImageFilter_SetSurvivalThreshold, 'GetSurvivalThreshold' = VotingBinaryImageFilter_GetSurvivalThreshold, 'SetForegroundValue' = VotingBinaryImageFilter_SetForegroundValue, 'GetForegroundValue' = VotingBinaryImageFilter_GetForegroundValue, 'SetBackgroundValue' = VotingBinaryImageFilter_SetBackgroundValue, 'GetBackgroundValue' = VotingBinaryImageFilter_GetBackgroundValue, 'GetName' = VotingBinaryImageFilter_GetName, 'ToString' = VotingBinaryImageFilter_ToString, 'Execute' = VotingBinaryImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::VotingBinaryImageFilter
# Start of VotingBinary

`VotingBinary__SWIG_0` = function(image1, radius, birthThreshold, survivalThreshold, foregroundValue, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  birthThreshold = as.integer(birthThreshold);
  
  if(length(birthThreshold) > 1) {
    warning("using only the first element of birthThreshold");
  };
  
  survivalThreshold = as.integer(survivalThreshold);
  
  if(length(survivalThreshold) > 1) {
    warning("using only the first element of survivalThreshold");
  };
  
  
  
  ;ans = .Call('R_swig_VotingBinary__SWIG_0', image1, radius, birthThreshold, survivalThreshold, foregroundValue, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinary__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinary__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'numeric', 'numeric')
class(`VotingBinary__SWIG_0`) = c("SWIGFunction", class('VotingBinary__SWIG_0'))

# Start of VotingBinary

`VotingBinary__SWIG_1` = function(image1, radius, birthThreshold, survivalThreshold, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  birthThreshold = as.integer(birthThreshold);
  
  if(length(birthThreshold) > 1) {
    warning("using only the first element of birthThreshold");
  };
  
  survivalThreshold = as.integer(survivalThreshold);
  
  if(length(survivalThreshold) > 1) {
    warning("using only the first element of survivalThreshold");
  };
  
  
  ;ans = .Call('R_swig_VotingBinary__SWIG_1', image1, radius, birthThreshold, survivalThreshold, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinary__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinary__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'numeric')
class(`VotingBinary__SWIG_1`) = c("SWIGFunction", class('VotingBinary__SWIG_1'))

# Start of VotingBinary

`VotingBinary__SWIG_2` = function(image1, radius, birthThreshold, survivalThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  birthThreshold = as.integer(birthThreshold);
  
  if(length(birthThreshold) > 1) {
    warning("using only the first element of birthThreshold");
  };
  
  survivalThreshold = as.integer(survivalThreshold);
  
  if(length(survivalThreshold) > 1) {
    warning("using only the first element of survivalThreshold");
  };
  
  ;ans = .Call('R_swig_VotingBinary__SWIG_2', image1, radius, birthThreshold, survivalThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinary__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinary__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`VotingBinary__SWIG_2`) = c("SWIGFunction", class('VotingBinary__SWIG_2'))

# Start of VotingBinary

`VotingBinary__SWIG_3` = function(image1, radius, birthThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  birthThreshold = as.integer(birthThreshold);
  
  if(length(birthThreshold) > 1) {
    warning("using only the first element of birthThreshold");
  };
  
  ;ans = .Call('R_swig_VotingBinary__SWIG_3', image1, radius, birthThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinary__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinary__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`VotingBinary__SWIG_3`) = c("SWIGFunction", class('VotingBinary__SWIG_3'))

# Start of VotingBinary

`VotingBinary__SWIG_4` = function(image1, radius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  ;ans = .Call('R_swig_VotingBinary__SWIG_4', image1, radius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinary__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinary__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`VotingBinary__SWIG_4`) = c("SWIGFunction", class('VotingBinary__SWIG_4'))

# Start of VotingBinary

`VotingBinary__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_VotingBinary__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinary__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinary__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`VotingBinary__SWIG_5`) = c("SWIGFunction", class('VotingBinary__SWIG_5'))

`VotingBinary` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- VotingBinary__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- VotingBinary__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- VotingBinary__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- VotingBinary__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- VotingBinary__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- VotingBinary__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VotingBinary with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_VotingBinaryIterativeHoleFillingImageFilter

`delete_VotingBinaryIterativeHoleFillingImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_VotingBinaryIterativeHoleFillingImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_VotingBinaryIterativeHoleFillingImageFilter`, 'returnType') = 'void'
attr(`delete_VotingBinaryIterativeHoleFillingImageFilter`, "inputTypes") = c('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter')
class(`delete_VotingBinaryIterativeHoleFillingImageFilter`) = c("SWIGFunction", class('delete_VotingBinaryIterativeHoleFillingImageFilter'))

# Start of new_VotingBinaryIterativeHoleFillingImageFilter

`VotingBinaryIterativeHoleFillingImageFilter` = function()
{
  ;ans = .Call('R_swig_new_VotingBinaryIterativeHoleFillingImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_VotingBinaryIterativeHoleFillingImageFilter);
  ans
  
}

attr(`VotingBinaryIterativeHoleFillingImageFilter`, 'returnType') = '_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter'
class(`VotingBinaryIterativeHoleFillingImageFilter`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFillingImageFilter'))

# Start of VotingBinaryIterativeHoleFillingImageFilter_SetRadius

`VotingBinaryIterativeHoleFillingImageFilter_SetRadius__SWIG_0` = function(self, Radius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Radius = as.integer(Radius);
  ;ans = .Call('R_swig_VotingBinaryIterativeHoleFillingImageFilter_SetRadius__SWIG_0', self, Radius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryIterativeHoleFillingImageFilter_SetRadius__SWIG_0`, 'returnType') = 'void'
attr(`VotingBinaryIterativeHoleFillingImageFilter_SetRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter', 'integer')
class(`VotingBinaryIterativeHoleFillingImageFilter_SetRadius__SWIG_0`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFillingImageFilter_SetRadius__SWIG_0'))

# Start of VotingBinaryIterativeHoleFillingImageFilter_SetRadius

`VotingBinaryIterativeHoleFillingImageFilter_SetRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_VotingBinaryIterativeHoleFillingImageFilter_SetRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryIterativeHoleFillingImageFilter_SetRadius__SWIG_1`, 'returnType') = 'void'
attr(`VotingBinaryIterativeHoleFillingImageFilter_SetRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter', 'integer')
class(`VotingBinaryIterativeHoleFillingImageFilter_SetRadius__SWIG_1`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFillingImageFilter_SetRadius__SWIG_1'))

`VotingBinaryIterativeHoleFillingImageFilter_SetRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- VotingBinaryIterativeHoleFillingImageFilter_SetRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- VotingBinaryIterativeHoleFillingImageFilter_SetRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VotingBinaryIterativeHoleFillingImageFilter_SetRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of VotingBinaryIterativeHoleFillingImageFilter_GetRadius

`VotingBinaryIterativeHoleFillingImageFilter_GetRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryIterativeHoleFillingImageFilter_GetRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryIterativeHoleFillingImageFilter_GetRadius`, 'returnType') = 'integer'
attr(`VotingBinaryIterativeHoleFillingImageFilter_GetRadius`, "inputTypes") = c('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter')
class(`VotingBinaryIterativeHoleFillingImageFilter_GetRadius`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFillingImageFilter_GetRadius'))

# Start of VotingBinaryIterativeHoleFillingImageFilter_SetMaximumNumberOfIterations

`VotingBinaryIterativeHoleFillingImageFilter_SetMaximumNumberOfIterations` = function(self, MaximumNumberOfIterations)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaximumNumberOfIterations = as.integer(MaximumNumberOfIterations);
  
  if(length(MaximumNumberOfIterations) > 1) {
    warning("using only the first element of MaximumNumberOfIterations");
  };
  
  ;ans = .Call('R_swig_VotingBinaryIterativeHoleFillingImageFilter_SetMaximumNumberOfIterations', self, MaximumNumberOfIterations, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryIterativeHoleFillingImageFilter_SetMaximumNumberOfIterations`, 'returnType') = 'void'
attr(`VotingBinaryIterativeHoleFillingImageFilter_SetMaximumNumberOfIterations`, "inputTypes") = c('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter', 'integer')
class(`VotingBinaryIterativeHoleFillingImageFilter_SetMaximumNumberOfIterations`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFillingImageFilter_SetMaximumNumberOfIterations'))

# Start of VotingBinaryIterativeHoleFillingImageFilter_GetMaximumNumberOfIterations

`VotingBinaryIterativeHoleFillingImageFilter_GetMaximumNumberOfIterations` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryIterativeHoleFillingImageFilter_GetMaximumNumberOfIterations', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryIterativeHoleFillingImageFilter_GetMaximumNumberOfIterations`, 'returnType') = 'integer'
attr(`VotingBinaryIterativeHoleFillingImageFilter_GetMaximumNumberOfIterations`, "inputTypes") = c('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter')
class(`VotingBinaryIterativeHoleFillingImageFilter_GetMaximumNumberOfIterations`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFillingImageFilter_GetMaximumNumberOfIterations'))

# Start of VotingBinaryIterativeHoleFillingImageFilter_SetMajorityThreshold

`VotingBinaryIterativeHoleFillingImageFilter_SetMajorityThreshold` = function(self, MajorityThreshold)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MajorityThreshold = as.integer(MajorityThreshold);
  
  if(length(MajorityThreshold) > 1) {
    warning("using only the first element of MajorityThreshold");
  };
  
  ;ans = .Call('R_swig_VotingBinaryIterativeHoleFillingImageFilter_SetMajorityThreshold', self, MajorityThreshold, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryIterativeHoleFillingImageFilter_SetMajorityThreshold`, 'returnType') = 'void'
attr(`VotingBinaryIterativeHoleFillingImageFilter_SetMajorityThreshold`, "inputTypes") = c('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter', 'integer')
class(`VotingBinaryIterativeHoleFillingImageFilter_SetMajorityThreshold`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFillingImageFilter_SetMajorityThreshold'))

# Start of VotingBinaryIterativeHoleFillingImageFilter_GetMajorityThreshold

`VotingBinaryIterativeHoleFillingImageFilter_GetMajorityThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryIterativeHoleFillingImageFilter_GetMajorityThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryIterativeHoleFillingImageFilter_GetMajorityThreshold`, 'returnType') = 'integer'
attr(`VotingBinaryIterativeHoleFillingImageFilter_GetMajorityThreshold`, "inputTypes") = c('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter')
class(`VotingBinaryIterativeHoleFillingImageFilter_GetMajorityThreshold`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFillingImageFilter_GetMajorityThreshold'))

# Start of VotingBinaryIterativeHoleFillingImageFilter_SetForegroundValue

`VotingBinaryIterativeHoleFillingImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_VotingBinaryIterativeHoleFillingImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryIterativeHoleFillingImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`VotingBinaryIterativeHoleFillingImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter', 'numeric')
class(`VotingBinaryIterativeHoleFillingImageFilter_SetForegroundValue`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFillingImageFilter_SetForegroundValue'))

# Start of VotingBinaryIterativeHoleFillingImageFilter_GetForegroundValue

`VotingBinaryIterativeHoleFillingImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryIterativeHoleFillingImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryIterativeHoleFillingImageFilter_GetForegroundValue`, 'returnType') = 'numeric'
attr(`VotingBinaryIterativeHoleFillingImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter')
class(`VotingBinaryIterativeHoleFillingImageFilter_GetForegroundValue`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFillingImageFilter_GetForegroundValue'))

# Start of VotingBinaryIterativeHoleFillingImageFilter_SetBackgroundValue

`VotingBinaryIterativeHoleFillingImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_VotingBinaryIterativeHoleFillingImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`VotingBinaryIterativeHoleFillingImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`VotingBinaryIterativeHoleFillingImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter', 'numeric')
class(`VotingBinaryIterativeHoleFillingImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFillingImageFilter_SetBackgroundValue'))

# Start of VotingBinaryIterativeHoleFillingImageFilter_GetBackgroundValue

`VotingBinaryIterativeHoleFillingImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryIterativeHoleFillingImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryIterativeHoleFillingImageFilter_GetBackgroundValue`, 'returnType') = 'numeric'
attr(`VotingBinaryIterativeHoleFillingImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter')
class(`VotingBinaryIterativeHoleFillingImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFillingImageFilter_GetBackgroundValue'))

# Start of VotingBinaryIterativeHoleFillingImageFilter_GetName

`VotingBinaryIterativeHoleFillingImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryIterativeHoleFillingImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryIterativeHoleFillingImageFilter_GetName`, 'returnType') = 'character'
attr(`VotingBinaryIterativeHoleFillingImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter')
class(`VotingBinaryIterativeHoleFillingImageFilter_GetName`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFillingImageFilter_GetName'))

# Start of VotingBinaryIterativeHoleFillingImageFilter_ToString

`VotingBinaryIterativeHoleFillingImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_VotingBinaryIterativeHoleFillingImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`VotingBinaryIterativeHoleFillingImageFilter_ToString`, 'returnType') = 'character'
attr(`VotingBinaryIterativeHoleFillingImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter')
class(`VotingBinaryIterativeHoleFillingImageFilter_ToString`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFillingImageFilter_ToString'))

# Start of VotingBinaryIterativeHoleFillingImageFilter_Execute

`VotingBinaryIterativeHoleFillingImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_VotingBinaryIterativeHoleFillingImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinaryIterativeHoleFillingImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinaryIterativeHoleFillingImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter', '_p_itk__simple__Image')
class(`VotingBinaryIterativeHoleFillingImageFilter_Execute`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFillingImageFilter_Execute'))

# Start of accessor method for itk::simple::VotingBinaryIterativeHoleFillingImageFilter
setMethod('$', '_p_itk__simple__VotingBinaryIterativeHoleFillingImageFilter', function(x, name)

{
  accessorFuns = list('SetRadius' = VotingBinaryIterativeHoleFillingImageFilter_SetRadius, 'GetRadius' = VotingBinaryIterativeHoleFillingImageFilter_GetRadius, 'SetMaximumNumberOfIterations' = VotingBinaryIterativeHoleFillingImageFilter_SetMaximumNumberOfIterations, 'GetMaximumNumberOfIterations' = VotingBinaryIterativeHoleFillingImageFilter_GetMaximumNumberOfIterations, 'SetMajorityThreshold' = VotingBinaryIterativeHoleFillingImageFilter_SetMajorityThreshold, 'GetMajorityThreshold' = VotingBinaryIterativeHoleFillingImageFilter_GetMajorityThreshold, 'SetForegroundValue' = VotingBinaryIterativeHoleFillingImageFilter_SetForegroundValue, 'GetForegroundValue' = VotingBinaryIterativeHoleFillingImageFilter_GetForegroundValue, 'SetBackgroundValue' = VotingBinaryIterativeHoleFillingImageFilter_SetBackgroundValue, 'GetBackgroundValue' = VotingBinaryIterativeHoleFillingImageFilter_GetBackgroundValue, 'GetName' = VotingBinaryIterativeHoleFillingImageFilter_GetName, 'ToString' = VotingBinaryIterativeHoleFillingImageFilter_ToString, 'Execute' = VotingBinaryIterativeHoleFillingImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::VotingBinaryIterativeHoleFillingImageFilter
# Start of VotingBinaryIterativeHoleFilling

`VotingBinaryIterativeHoleFilling__SWIG_0` = function(image1, radius, maximumNumberOfIterations, majorityThreshold, foregroundValue, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  majorityThreshold = as.integer(majorityThreshold);
  
  if(length(majorityThreshold) > 1) {
    warning("using only the first element of majorityThreshold");
  };
  
  
  
  ;ans = .Call('R_swig_VotingBinaryIterativeHoleFilling__SWIG_0', image1, radius, maximumNumberOfIterations, majorityThreshold, foregroundValue, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinaryIterativeHoleFilling__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinaryIterativeHoleFilling__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'numeric', 'numeric')
class(`VotingBinaryIterativeHoleFilling__SWIG_0`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFilling__SWIG_0'))

# Start of VotingBinaryIterativeHoleFilling

`VotingBinaryIterativeHoleFilling__SWIG_1` = function(image1, radius, maximumNumberOfIterations, majorityThreshold, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  majorityThreshold = as.integer(majorityThreshold);
  
  if(length(majorityThreshold) > 1) {
    warning("using only the first element of majorityThreshold");
  };
  
  
  ;ans = .Call('R_swig_VotingBinaryIterativeHoleFilling__SWIG_1', image1, radius, maximumNumberOfIterations, majorityThreshold, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinaryIterativeHoleFilling__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinaryIterativeHoleFilling__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'numeric')
class(`VotingBinaryIterativeHoleFilling__SWIG_1`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFilling__SWIG_1'))

# Start of VotingBinaryIterativeHoleFilling

`VotingBinaryIterativeHoleFilling__SWIG_2` = function(image1, radius, maximumNumberOfIterations, majorityThreshold, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  majorityThreshold = as.integer(majorityThreshold);
  
  if(length(majorityThreshold) > 1) {
    warning("using only the first element of majorityThreshold");
  };
  
  ;ans = .Call('R_swig_VotingBinaryIterativeHoleFilling__SWIG_2', image1, radius, maximumNumberOfIterations, majorityThreshold, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinaryIterativeHoleFilling__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinaryIterativeHoleFilling__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`VotingBinaryIterativeHoleFilling__SWIG_2`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFilling__SWIG_2'))

# Start of VotingBinaryIterativeHoleFilling

`VotingBinaryIterativeHoleFilling__SWIG_3` = function(image1, radius, maximumNumberOfIterations, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  maximumNumberOfIterations = as.integer(maximumNumberOfIterations);
  
  if(length(maximumNumberOfIterations) > 1) {
    warning("using only the first element of maximumNumberOfIterations");
  };
  
  ;ans = .Call('R_swig_VotingBinaryIterativeHoleFilling__SWIG_3', image1, radius, maximumNumberOfIterations, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinaryIterativeHoleFilling__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinaryIterativeHoleFilling__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`VotingBinaryIterativeHoleFilling__SWIG_3`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFilling__SWIG_3'))

# Start of VotingBinaryIterativeHoleFilling

`VotingBinaryIterativeHoleFilling__SWIG_4` = function(image1, radius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  radius = as.integer(radius);
  ;ans = .Call('R_swig_VotingBinaryIterativeHoleFilling__SWIG_4', image1, radius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinaryIterativeHoleFilling__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinaryIterativeHoleFilling__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`VotingBinaryIterativeHoleFilling__SWIG_4`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFilling__SWIG_4'))

# Start of VotingBinaryIterativeHoleFilling

`VotingBinaryIterativeHoleFilling__SWIG_5` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_VotingBinaryIterativeHoleFilling__SWIG_5', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`VotingBinaryIterativeHoleFilling__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`VotingBinaryIterativeHoleFilling__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image')
class(`VotingBinaryIterativeHoleFilling__SWIG_5`) = c("SWIGFunction", class('VotingBinaryIterativeHoleFilling__SWIG_5'))

`VotingBinaryIterativeHoleFilling` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 6
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- VotingBinaryIterativeHoleFilling__SWIG_5; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- VotingBinaryIterativeHoleFilling__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- VotingBinaryIterativeHoleFilling__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- VotingBinaryIterativeHoleFilling__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- VotingBinaryIterativeHoleFilling__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.numeric(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- VotingBinaryIterativeHoleFilling__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for VotingBinaryIterativeHoleFilling with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_WarpImageFilter

`delete_WarpImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_WarpImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_WarpImageFilter`, 'returnType') = 'void'
attr(`delete_WarpImageFilter`, "inputTypes") = c('_p_itk__simple__WarpImageFilter')
class(`delete_WarpImageFilter`) = c("SWIGFunction", class('delete_WarpImageFilter'))

# Start of new_WarpImageFilter

`WarpImageFilter` = function()
{
  ;ans = .Call('R_swig_new_WarpImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__WarpImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_WarpImageFilter);
  ans
  
}

attr(`WarpImageFilter`, 'returnType') = '_p_itk__simple__WarpImageFilter'
class(`WarpImageFilter`) = c("SWIGFunction", class('WarpImageFilter'))

# Start of WarpImageFilter_SetInterpolator

`WarpImageFilter_SetInterpolator` = function(self, Interpolator)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Interpolator = enumToInteger(Interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(Interpolator) > 1) {
    warning("using only the first element of Interpolator");
  };
  
  ;ans = .Call('R_swig_WarpImageFilter_SetInterpolator', self, Interpolator, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WarpImageFilter_SetInterpolator`, 'returnType') = 'void'
attr(`WarpImageFilter_SetInterpolator`, "inputTypes") = c('_p_itk__simple__WarpImageFilter', 'character')
class(`WarpImageFilter_SetInterpolator`) = c("SWIGFunction", class('WarpImageFilter_SetInterpolator'))

# Start of WarpImageFilter_GetInterpolator

`WarpImageFilter_GetInterpolator` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_WarpImageFilter_GetInterpolator', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__InterpolatorEnum");
  
  ans
  
}

attr(`WarpImageFilter_GetInterpolator`, 'returnType') = 'character'
attr(`WarpImageFilter_GetInterpolator`, "inputTypes") = c('_p_itk__simple__WarpImageFilter')
class(`WarpImageFilter_GetInterpolator`) = c("SWIGFunction", class('WarpImageFilter_GetInterpolator'))

# Start of WarpImageFilter_SetOutputSize

`WarpImageFilter_SetOutputSize` = function(self, OutputSize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputSize = as.integer(OutputSize);
  ;ans = .Call('R_swig_WarpImageFilter_SetOutputSize', self, OutputSize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WarpImageFilter_SetOutputSize`, 'returnType') = 'void'
attr(`WarpImageFilter_SetOutputSize`, "inputTypes") = c('_p_itk__simple__WarpImageFilter', 'integer')
class(`WarpImageFilter_SetOutputSize`) = c("SWIGFunction", class('WarpImageFilter_SetOutputSize'))

# Start of WarpImageFilter_GetOutputSize

`WarpImageFilter_GetOutputSize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WarpImageFilter_GetOutputSize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WarpImageFilter_GetOutputSize`, 'returnType') = 'integer'
attr(`WarpImageFilter_GetOutputSize`, "inputTypes") = c('_p_itk__simple__WarpImageFilter')
class(`WarpImageFilter_GetOutputSize`) = c("SWIGFunction", class('WarpImageFilter_GetOutputSize'))

# Start of WarpImageFilter_SetOutputOrigin

`WarpImageFilter_SetOutputOrigin` = function(self, OutputOrigin)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputOrigin = as.numeric(OutputOrigin);
  ;ans = .Call('R_swig_WarpImageFilter_SetOutputOrigin', self, OutputOrigin, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WarpImageFilter_SetOutputOrigin`, 'returnType') = 'void'
attr(`WarpImageFilter_SetOutputOrigin`, "inputTypes") = c('_p_itk__simple__WarpImageFilter', 'numeric')
class(`WarpImageFilter_SetOutputOrigin`) = c("SWIGFunction", class('WarpImageFilter_SetOutputOrigin'))

# Start of WarpImageFilter_GetOutputOrigin

`WarpImageFilter_GetOutputOrigin` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WarpImageFilter_GetOutputOrigin', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WarpImageFilter_GetOutputOrigin`, 'returnType') = 'numeric'
attr(`WarpImageFilter_GetOutputOrigin`, "inputTypes") = c('_p_itk__simple__WarpImageFilter')
class(`WarpImageFilter_GetOutputOrigin`) = c("SWIGFunction", class('WarpImageFilter_GetOutputOrigin'))

# Start of WarpImageFilter_SetOutputSpacing

`WarpImageFilter_SetOutputSpacing` = function(self, OutputSpacing)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputSpacing = as.numeric(OutputSpacing);
  ;ans = .Call('R_swig_WarpImageFilter_SetOutputSpacing', self, OutputSpacing, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WarpImageFilter_SetOutputSpacing`, 'returnType') = 'void'
attr(`WarpImageFilter_SetOutputSpacing`, "inputTypes") = c('_p_itk__simple__WarpImageFilter', 'numeric')
class(`WarpImageFilter_SetOutputSpacing`) = c("SWIGFunction", class('WarpImageFilter_SetOutputSpacing'))

# Start of WarpImageFilter_GetOutputSpacing

`WarpImageFilter_GetOutputSpacing` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WarpImageFilter_GetOutputSpacing', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WarpImageFilter_GetOutputSpacing`, 'returnType') = 'numeric'
attr(`WarpImageFilter_GetOutputSpacing`, "inputTypes") = c('_p_itk__simple__WarpImageFilter')
class(`WarpImageFilter_GetOutputSpacing`) = c("SWIGFunction", class('WarpImageFilter_GetOutputSpacing'))

# Start of WarpImageFilter_SetOutputDirection

`WarpImageFilter_SetOutputDirection` = function(self, OutputDirection)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputDirection = as.numeric(OutputDirection);
  ;ans = .Call('R_swig_WarpImageFilter_SetOutputDirection', self, OutputDirection, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WarpImageFilter_SetOutputDirection`, 'returnType') = 'void'
attr(`WarpImageFilter_SetOutputDirection`, "inputTypes") = c('_p_itk__simple__WarpImageFilter', 'numeric')
class(`WarpImageFilter_SetOutputDirection`) = c("SWIGFunction", class('WarpImageFilter_SetOutputDirection'))

# Start of WarpImageFilter_GetOutputDirection

`WarpImageFilter_GetOutputDirection` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WarpImageFilter_GetOutputDirection', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WarpImageFilter_GetOutputDirection`, 'returnType') = 'numeric'
attr(`WarpImageFilter_GetOutputDirection`, "inputTypes") = c('_p_itk__simple__WarpImageFilter')
class(`WarpImageFilter_GetOutputDirection`) = c("SWIGFunction", class('WarpImageFilter_GetOutputDirection'))

# Start of WarpImageFilter_SetEdgePaddingValue

`WarpImageFilter_SetEdgePaddingValue` = function(self, EdgePaddingValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_WarpImageFilter_SetEdgePaddingValue', self, EdgePaddingValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WarpImageFilter_SetEdgePaddingValue`, 'returnType') = 'void'
attr(`WarpImageFilter_SetEdgePaddingValue`, "inputTypes") = c('_p_itk__simple__WarpImageFilter', 'numeric')
class(`WarpImageFilter_SetEdgePaddingValue`) = c("SWIGFunction", class('WarpImageFilter_SetEdgePaddingValue'))

# Start of WarpImageFilter_GetEdgePaddingValue

`WarpImageFilter_GetEdgePaddingValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WarpImageFilter_GetEdgePaddingValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WarpImageFilter_GetEdgePaddingValue`, 'returnType') = 'numeric'
attr(`WarpImageFilter_GetEdgePaddingValue`, "inputTypes") = c('_p_itk__simple__WarpImageFilter')
class(`WarpImageFilter_GetEdgePaddingValue`) = c("SWIGFunction", class('WarpImageFilter_GetEdgePaddingValue'))

# Start of WarpImageFilter_GetName

`WarpImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WarpImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WarpImageFilter_GetName`, 'returnType') = 'character'
attr(`WarpImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__WarpImageFilter')
class(`WarpImageFilter_GetName`) = c("SWIGFunction", class('WarpImageFilter_GetName'))

# Start of WarpImageFilter_ToString

`WarpImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WarpImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WarpImageFilter_ToString`, 'returnType') = 'character'
attr(`WarpImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__WarpImageFilter')
class(`WarpImageFilter_ToString`) = c("SWIGFunction", class('WarpImageFilter_ToString'))

# Start of WarpImageFilter_Execute

`WarpImageFilter_Execute` = function(self, image, displacementField, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(displacementField, "ExternalReference")) displacementField = slot(displacementField,"ref"); 
  ;ans = .Call('R_swig_WarpImageFilter_Execute', self, image, displacementField, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WarpImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`WarpImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__WarpImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`WarpImageFilter_Execute`) = c("SWIGFunction", class('WarpImageFilter_Execute'))

# Start of WarpImageFilter_SetOutputParameteresFromImage

`WarpImageFilter_SetOutputParameteresFromImage` = function(self, refImage)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(refImage, "ExternalReference")) refImage = slot(refImage,"ref"); 
  ;ans = .Call('R_swig_WarpImageFilter_SetOutputParameteresFromImage', self, refImage, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WarpImageFilter_SetOutputParameteresFromImage`, 'returnType') = 'void'
attr(`WarpImageFilter_SetOutputParameteresFromImage`, "inputTypes") = c('_p_itk__simple__WarpImageFilter', '_p_itk__simple__Image')
class(`WarpImageFilter_SetOutputParameteresFromImage`) = c("SWIGFunction", class('WarpImageFilter_SetOutputParameteresFromImage'))

# Start of accessor method for itk::simple::WarpImageFilter
setMethod('$', '_p_itk__simple__WarpImageFilter', function(x, name)

{
  accessorFuns = list('SetInterpolator' = WarpImageFilter_SetInterpolator, 'GetInterpolator' = WarpImageFilter_GetInterpolator, 'SetOutputSize' = WarpImageFilter_SetOutputSize, 'GetOutputSize' = WarpImageFilter_GetOutputSize, 'SetOutputOrigin' = WarpImageFilter_SetOutputOrigin, 'GetOutputOrigin' = WarpImageFilter_GetOutputOrigin, 'SetOutputSpacing' = WarpImageFilter_SetOutputSpacing, 'GetOutputSpacing' = WarpImageFilter_GetOutputSpacing, 'SetOutputDirection' = WarpImageFilter_SetOutputDirection, 'GetOutputDirection' = WarpImageFilter_GetOutputDirection, 'SetEdgePaddingValue' = WarpImageFilter_SetEdgePaddingValue, 'GetEdgePaddingValue' = WarpImageFilter_GetEdgePaddingValue, 'GetName' = WarpImageFilter_GetName, 'ToString' = WarpImageFilter_ToString, 'Execute' = WarpImageFilter_Execute, 'SetOutputParameteresFromImage' = WarpImageFilter_SetOutputParameteresFromImage);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::WarpImageFilter
# Start of Warp

`Warp__SWIG_0` = function(image, displacementField, interpolator, outputSize, outputOrigin, outputSpacing, outputDirection, edgePaddingValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(displacementField, "ExternalReference")) displacementField = slot(displacementField,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  outputSize = as.integer(outputSize);
  outputOrigin = as.numeric(outputOrigin);
  outputSpacing = as.numeric(outputSpacing);
  outputDirection = as.numeric(outputDirection);
  
  ;ans = .Call('R_swig_Warp__SWIG_0', image, displacementField, interpolator, outputSize, outputOrigin, outputSpacing, outputDirection, edgePaddingValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Warp__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Warp__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'character', 'integer', 'numeric', 'numeric', 'numeric', 'numeric')
class(`Warp__SWIG_0`) = c("SWIGFunction", class('Warp__SWIG_0'))

# Start of Warp

`Warp__SWIG_1` = function(image, displacementField, interpolator, outputSize, outputOrigin, outputSpacing, outputDirection, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(displacementField, "ExternalReference")) displacementField = slot(displacementField,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  outputSize = as.integer(outputSize);
  outputOrigin = as.numeric(outputOrigin);
  outputSpacing = as.numeric(outputSpacing);
  outputDirection = as.numeric(outputDirection);
  ;ans = .Call('R_swig_Warp__SWIG_1', image, displacementField, interpolator, outputSize, outputOrigin, outputSpacing, outputDirection, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Warp__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Warp__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'character', 'integer', 'numeric', 'numeric', 'numeric')
class(`Warp__SWIG_1`) = c("SWIGFunction", class('Warp__SWIG_1'))

# Start of Warp

`Warp__SWIG_2` = function(image, displacementField, interpolator, outputSize, outputOrigin, outputSpacing, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(displacementField, "ExternalReference")) displacementField = slot(displacementField,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  outputSize = as.integer(outputSize);
  outputOrigin = as.numeric(outputOrigin);
  outputSpacing = as.numeric(outputSpacing);
  ;ans = .Call('R_swig_Warp__SWIG_2', image, displacementField, interpolator, outputSize, outputOrigin, outputSpacing, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Warp__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Warp__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'character', 'integer', 'numeric', 'numeric')
class(`Warp__SWIG_2`) = c("SWIGFunction", class('Warp__SWIG_2'))

# Start of Warp

`Warp__SWIG_3` = function(image, displacementField, interpolator, outputSize, outputOrigin, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(displacementField, "ExternalReference")) displacementField = slot(displacementField,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  outputSize = as.integer(outputSize);
  outputOrigin = as.numeric(outputOrigin);
  ;ans = .Call('R_swig_Warp__SWIG_3', image, displacementField, interpolator, outputSize, outputOrigin, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Warp__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`Warp__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'character', 'integer', 'numeric')
class(`Warp__SWIG_3`) = c("SWIGFunction", class('Warp__SWIG_3'))

# Start of Warp

`Warp__SWIG_4` = function(image, displacementField, interpolator, outputSize, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(displacementField, "ExternalReference")) displacementField = slot(displacementField,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  outputSize = as.integer(outputSize);
  ;ans = .Call('R_swig_Warp__SWIG_4', image, displacementField, interpolator, outputSize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Warp__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`Warp__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'character', 'integer')
class(`Warp__SWIG_4`) = c("SWIGFunction", class('Warp__SWIG_4'))

# Start of Warp

`Warp__SWIG_5` = function(image, displacementField, interpolator, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(displacementField, "ExternalReference")) displacementField = slot(displacementField,"ref"); 
  interpolator = enumToInteger(interpolator, "_itk__simple__InterpolatorEnum"); 
  
  if(length(interpolator) > 1) {
    warning("using only the first element of interpolator");
  };
  
  ;ans = .Call('R_swig_Warp__SWIG_5', image, displacementField, interpolator, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Warp__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`Warp__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'character')
class(`Warp__SWIG_5`) = c("SWIGFunction", class('Warp__SWIG_5'))

# Start of Warp

`Warp__SWIG_6` = function(image, displacementField, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(displacementField, "ExternalReference")) displacementField = slot(displacementField,"ref"); 
  ;ans = .Call('R_swig_Warp__SWIG_6', image, displacementField, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Warp__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`Warp__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Warp__SWIG_6`) = c("SWIGFunction", class('Warp__SWIG_6'))

`Warp` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 7
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Warp__SWIG_6; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- Warp__SWIG_5; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) )) {
      f <- Warp__SWIG_4; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) )) {
      f <- Warp__SWIG_3; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) )) {
      f <- Warp__SWIG_2; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) ) && ( is.numeric(argv[[7]]) )) {
      f <- Warp__SWIG_1; 
    }
  } else if (argc == 8) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.integer(argv[[4]]) || is.numeric(argv[[4]]) ) && ( is.numeric(argv[[5]]) ) && ( is.numeric(argv[[6]]) ) && ( is.numeric(argv[[7]]) ) && ( is.numeric(argv[[8]]) && length(argv[[8]]) == 1 )) {
      f <- Warp__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Warp with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_WhiteTopHatImageFilter

`delete_WhiteTopHatImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_WhiteTopHatImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_WhiteTopHatImageFilter`, 'returnType') = 'void'
attr(`delete_WhiteTopHatImageFilter`, "inputTypes") = c('_p_itk__simple__WhiteTopHatImageFilter')
class(`delete_WhiteTopHatImageFilter`) = c("SWIGFunction", class('delete_WhiteTopHatImageFilter'))

# Start of new_WhiteTopHatImageFilter

`WhiteTopHatImageFilter` = function()
{
  ;ans = .Call('R_swig_new_WhiteTopHatImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__WhiteTopHatImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_WhiteTopHatImageFilter);
  ans
  
}

attr(`WhiteTopHatImageFilter`, 'returnType') = '_p_itk__simple__WhiteTopHatImageFilter'
class(`WhiteTopHatImageFilter`) = c("SWIGFunction", class('WhiteTopHatImageFilter'))

# Start of WhiteTopHatImageFilter_SetKernelRadius

`WhiteTopHatImageFilter_SetKernelRadius__SWIG_0` = function(self, KernelRadius)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelRadius = as.integer(KernelRadius);
  ;ans = .Call('R_swig_WhiteTopHatImageFilter_SetKernelRadius__SWIG_0', self, KernelRadius, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WhiteTopHatImageFilter_SetKernelRadius__SWIG_0`, 'returnType') = 'void'
attr(`WhiteTopHatImageFilter_SetKernelRadius__SWIG_0`, "inputTypes") = c('_p_itk__simple__WhiteTopHatImageFilter', 'integer')
class(`WhiteTopHatImageFilter_SetKernelRadius__SWIG_0`) = c("SWIGFunction", class('WhiteTopHatImageFilter_SetKernelRadius__SWIG_0'))

# Start of WhiteTopHatImageFilter_SetKernelRadius

`WhiteTopHatImageFilter_SetKernelRadius__SWIG_1` = function(self, value)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  value = as.integer(value);
  
  if(length(value) > 1) {
    warning("using only the first element of value");
  };
  
  ;ans = .Call('R_swig_WhiteTopHatImageFilter_SetKernelRadius__SWIG_1', self, value, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WhiteTopHatImageFilter_SetKernelRadius__SWIG_1`, 'returnType') = 'void'
attr(`WhiteTopHatImageFilter_SetKernelRadius__SWIG_1`, "inputTypes") = c('_p_itk__simple__WhiteTopHatImageFilter', 'integer')
class(`WhiteTopHatImageFilter_SetKernelRadius__SWIG_1`) = c("SWIGFunction", class('WhiteTopHatImageFilter_SetKernelRadius__SWIG_1'))

`WhiteTopHatImageFilter_SetKernelRadius` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__WhiteTopHatImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- WhiteTopHatImageFilter_SetKernelRadius__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__WhiteTopHatImageFilter') || is.null(argv[[1]]) ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- WhiteTopHatImageFilter_SetKernelRadius__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for WhiteTopHatImageFilter_SetKernelRadius with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of WhiteTopHatImageFilter_GetKernelRadius

`WhiteTopHatImageFilter_GetKernelRadius` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WhiteTopHatImageFilter_GetKernelRadius', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WhiteTopHatImageFilter_GetKernelRadius`, 'returnType') = 'integer'
attr(`WhiteTopHatImageFilter_GetKernelRadius`, "inputTypes") = c('_p_itk__simple__WhiteTopHatImageFilter')
class(`WhiteTopHatImageFilter_GetKernelRadius`) = c("SWIGFunction", class('WhiteTopHatImageFilter_GetKernelRadius'))

# Start of WhiteTopHatImageFilter_SetKernelType

`WhiteTopHatImageFilter_SetKernelType` = function(self, KernelType)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  KernelType = enumToInteger(KernelType, "_itk__simple__KernelEnum"); 
  
  if(length(KernelType) > 1) {
    warning("using only the first element of KernelType");
  };
  
  ;ans = .Call('R_swig_WhiteTopHatImageFilter_SetKernelType', self, KernelType, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WhiteTopHatImageFilter_SetKernelType`, 'returnType') = 'void'
attr(`WhiteTopHatImageFilter_SetKernelType`, "inputTypes") = c('_p_itk__simple__WhiteTopHatImageFilter', 'character')
class(`WhiteTopHatImageFilter_SetKernelType`) = c("SWIGFunction", class('WhiteTopHatImageFilter_SetKernelType'))

# Start of WhiteTopHatImageFilter_GetKernelType

`WhiteTopHatImageFilter_GetKernelType` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_WhiteTopHatImageFilter_GetKernelType', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__KernelEnum");
  
  ans
  
}

attr(`WhiteTopHatImageFilter_GetKernelType`, 'returnType') = 'character'
attr(`WhiteTopHatImageFilter_GetKernelType`, "inputTypes") = c('_p_itk__simple__WhiteTopHatImageFilter')
class(`WhiteTopHatImageFilter_GetKernelType`) = c("SWIGFunction", class('WhiteTopHatImageFilter_GetKernelType'))

# Start of WhiteTopHatImageFilter_SetSafeBorder

`WhiteTopHatImageFilter_SetSafeBorder` = function(self, SafeBorder)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  SafeBorder = as.logical(SafeBorder);
  ;ans = .Call('R_swig_WhiteTopHatImageFilter_SetSafeBorder', self, SafeBorder, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WhiteTopHatImageFilter_SetSafeBorder`, 'returnType') = 'void'
attr(`WhiteTopHatImageFilter_SetSafeBorder`, "inputTypes") = c('_p_itk__simple__WhiteTopHatImageFilter', 'logical')
class(`WhiteTopHatImageFilter_SetSafeBorder`) = c("SWIGFunction", class('WhiteTopHatImageFilter_SetSafeBorder'))

# Start of WhiteTopHatImageFilter_SafeBorderOn

`WhiteTopHatImageFilter_SafeBorderOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_WhiteTopHatImageFilter_SafeBorderOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WhiteTopHatImageFilter_SafeBorderOn`, 'returnType') = 'void'
attr(`WhiteTopHatImageFilter_SafeBorderOn`, "inputTypes") = c('_p_itk__simple__WhiteTopHatImageFilter')
class(`WhiteTopHatImageFilter_SafeBorderOn`) = c("SWIGFunction", class('WhiteTopHatImageFilter_SafeBorderOn'))

# Start of WhiteTopHatImageFilter_SafeBorderOff

`WhiteTopHatImageFilter_SafeBorderOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_WhiteTopHatImageFilter_SafeBorderOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WhiteTopHatImageFilter_SafeBorderOff`, 'returnType') = 'void'
attr(`WhiteTopHatImageFilter_SafeBorderOff`, "inputTypes") = c('_p_itk__simple__WhiteTopHatImageFilter')
class(`WhiteTopHatImageFilter_SafeBorderOff`) = c("SWIGFunction", class('WhiteTopHatImageFilter_SafeBorderOff'))

# Start of WhiteTopHatImageFilter_GetSafeBorder

`WhiteTopHatImageFilter_GetSafeBorder` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WhiteTopHatImageFilter_GetSafeBorder', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WhiteTopHatImageFilter_GetSafeBorder`, 'returnType') = 'logical'
attr(`WhiteTopHatImageFilter_GetSafeBorder`, "inputTypes") = c('_p_itk__simple__WhiteTopHatImageFilter')
class(`WhiteTopHatImageFilter_GetSafeBorder`) = c("SWIGFunction", class('WhiteTopHatImageFilter_GetSafeBorder'))

# Start of WhiteTopHatImageFilter_GetName

`WhiteTopHatImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WhiteTopHatImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WhiteTopHatImageFilter_GetName`, 'returnType') = 'character'
attr(`WhiteTopHatImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__WhiteTopHatImageFilter')
class(`WhiteTopHatImageFilter_GetName`) = c("SWIGFunction", class('WhiteTopHatImageFilter_GetName'))

# Start of WhiteTopHatImageFilter_ToString

`WhiteTopHatImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WhiteTopHatImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WhiteTopHatImageFilter_ToString`, 'returnType') = 'character'
attr(`WhiteTopHatImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__WhiteTopHatImageFilter')
class(`WhiteTopHatImageFilter_ToString`) = c("SWIGFunction", class('WhiteTopHatImageFilter_ToString'))

# Start of WhiteTopHatImageFilter_Execute

`WhiteTopHatImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_WhiteTopHatImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WhiteTopHatImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`WhiteTopHatImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__WhiteTopHatImageFilter', '_p_itk__simple__Image')
class(`WhiteTopHatImageFilter_Execute`) = c("SWIGFunction", class('WhiteTopHatImageFilter_Execute'))

# Start of accessor method for itk::simple::WhiteTopHatImageFilter
setMethod('$', '_p_itk__simple__WhiteTopHatImageFilter', function(x, name)

{
  accessorFuns = list('SetKernelRadius' = WhiteTopHatImageFilter_SetKernelRadius, 'GetKernelRadius' = WhiteTopHatImageFilter_GetKernelRadius, 'SetKernelType' = WhiteTopHatImageFilter_SetKernelType, 'GetKernelType' = WhiteTopHatImageFilter_GetKernelType, 'SetSafeBorder' = WhiteTopHatImageFilter_SetSafeBorder, 'SafeBorderOn' = WhiteTopHatImageFilter_SafeBorderOn, 'SafeBorderOff' = WhiteTopHatImageFilter_SafeBorderOff, 'GetSafeBorder' = WhiteTopHatImageFilter_GetSafeBorder, 'GetName' = WhiteTopHatImageFilter_GetName, 'ToString' = WhiteTopHatImageFilter_ToString, 'Execute' = WhiteTopHatImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::WhiteTopHatImageFilter
# Start of WhiteTopHat

`WhiteTopHat__SWIG_0` = function(image1, kernelRadius, kernelType, safeBorder, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  safeBorder = as.logical(safeBorder);
  ;ans = .Call('R_swig_WhiteTopHat__SWIG_0', image1, kernelRadius, kernelType, safeBorder, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WhiteTopHat__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`WhiteTopHat__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character', 'logical')
class(`WhiteTopHat__SWIG_0`) = c("SWIGFunction", class('WhiteTopHat__SWIG_0'))

# Start of WhiteTopHat

`WhiteTopHat__SWIG_1` = function(image1, kernelRadius, kernelType, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  kernelType = enumToInteger(kernelType, "_itk__simple__KernelEnum"); 
  
  if(length(kernelType) > 1) {
    warning("using only the first element of kernelType");
  };
  
  ;ans = .Call('R_swig_WhiteTopHat__SWIG_1', image1, kernelRadius, kernelType, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WhiteTopHat__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`WhiteTopHat__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'character')
class(`WhiteTopHat__SWIG_1`) = c("SWIGFunction", class('WhiteTopHat__SWIG_1'))

# Start of WhiteTopHat

`WhiteTopHat__SWIG_2` = function(image1, kernelRadius, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  kernelRadius = as.integer(kernelRadius);
  ;ans = .Call('R_swig_WhiteTopHat__SWIG_2', image1, kernelRadius, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WhiteTopHat__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`WhiteTopHat__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`WhiteTopHat__SWIG_2`) = c("SWIGFunction", class('WhiteTopHat__SWIG_2'))

# Start of WhiteTopHat

`WhiteTopHat__SWIG_3` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_WhiteTopHat__SWIG_3', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WhiteTopHat__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`WhiteTopHat__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image')
class(`WhiteTopHat__SWIG_3`) = c("SWIGFunction", class('WhiteTopHat__SWIG_3'))

`WhiteTopHat` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 4
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- WhiteTopHat__SWIG_3; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- WhiteTopHat__SWIG_2; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- WhiteTopHat__SWIG_1; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.character(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- WhiteTopHat__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for WhiteTopHat with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_WienerDeconvolutionImageFilter

`delete_WienerDeconvolutionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_WienerDeconvolutionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_WienerDeconvolutionImageFilter`, 'returnType') = 'void'
attr(`delete_WienerDeconvolutionImageFilter`, "inputTypes") = c('_p_itk__simple__WienerDeconvolutionImageFilter')
class(`delete_WienerDeconvolutionImageFilter`) = c("SWIGFunction", class('delete_WienerDeconvolutionImageFilter'))

# Start of new_WienerDeconvolutionImageFilter

`WienerDeconvolutionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_WienerDeconvolutionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__WienerDeconvolutionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_WienerDeconvolutionImageFilter);
  ans
  
}

attr(`WienerDeconvolutionImageFilter`, 'returnType') = '_p_itk__simple__WienerDeconvolutionImageFilter'
class(`WienerDeconvolutionImageFilter`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter'))

# Start of WienerDeconvolutionImageFilter_SetNoiseVariance

`WienerDeconvolutionImageFilter_SetNoiseVariance` = function(self, NoiseVariance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_WienerDeconvolutionImageFilter_SetNoiseVariance', self, NoiseVariance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WienerDeconvolutionImageFilter_SetNoiseVariance`, 'returnType') = 'void'
attr(`WienerDeconvolutionImageFilter_SetNoiseVariance`, "inputTypes") = c('_p_itk__simple__WienerDeconvolutionImageFilter', 'numeric')
class(`WienerDeconvolutionImageFilter_SetNoiseVariance`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_SetNoiseVariance'))

# Start of WienerDeconvolutionImageFilter_GetNoiseVariance

`WienerDeconvolutionImageFilter_GetNoiseVariance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WienerDeconvolutionImageFilter_GetNoiseVariance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WienerDeconvolutionImageFilter_GetNoiseVariance`, 'returnType') = 'numeric'
attr(`WienerDeconvolutionImageFilter_GetNoiseVariance`, "inputTypes") = c('_p_itk__simple__WienerDeconvolutionImageFilter')
class(`WienerDeconvolutionImageFilter_GetNoiseVariance`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_GetNoiseVariance'))

# Start of WienerDeconvolutionImageFilter_SetNormalize

`WienerDeconvolutionImageFilter_SetNormalize` = function(self, Normalize)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  Normalize = as.logical(Normalize);
  ;ans = .Call('R_swig_WienerDeconvolutionImageFilter_SetNormalize', self, Normalize, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WienerDeconvolutionImageFilter_SetNormalize`, 'returnType') = 'void'
attr(`WienerDeconvolutionImageFilter_SetNormalize`, "inputTypes") = c('_p_itk__simple__WienerDeconvolutionImageFilter', 'logical')
class(`WienerDeconvolutionImageFilter_SetNormalize`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_SetNormalize'))

# Start of WienerDeconvolutionImageFilter_NormalizeOn

`WienerDeconvolutionImageFilter_NormalizeOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_WienerDeconvolutionImageFilter_NormalizeOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WienerDeconvolutionImageFilter_NormalizeOn`, 'returnType') = 'void'
attr(`WienerDeconvolutionImageFilter_NormalizeOn`, "inputTypes") = c('_p_itk__simple__WienerDeconvolutionImageFilter')
class(`WienerDeconvolutionImageFilter_NormalizeOn`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_NormalizeOn'))

# Start of WienerDeconvolutionImageFilter_NormalizeOff

`WienerDeconvolutionImageFilter_NormalizeOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_WienerDeconvolutionImageFilter_NormalizeOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WienerDeconvolutionImageFilter_NormalizeOff`, 'returnType') = 'void'
attr(`WienerDeconvolutionImageFilter_NormalizeOff`, "inputTypes") = c('_p_itk__simple__WienerDeconvolutionImageFilter')
class(`WienerDeconvolutionImageFilter_NormalizeOff`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_NormalizeOff'))

# Start of WienerDeconvolutionImageFilter_GetNormalize

`WienerDeconvolutionImageFilter_GetNormalize` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WienerDeconvolutionImageFilter_GetNormalize', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WienerDeconvolutionImageFilter_GetNormalize`, 'returnType') = 'logical'
attr(`WienerDeconvolutionImageFilter_GetNormalize`, "inputTypes") = c('_p_itk__simple__WienerDeconvolutionImageFilter')
class(`WienerDeconvolutionImageFilter_GetNormalize`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_GetNormalize'))

# Start of WienerDeconvolutionImageFilter_ZERO_PAD_get

`WienerDeconvolutionImageFilter_ZERO_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_WienerDeconvolutionImageFilter_ZERO_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__WienerDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`WienerDeconvolutionImageFilter_ZERO_PAD_get`, 'returnType') = 'character'
class(`WienerDeconvolutionImageFilter_ZERO_PAD_get`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_ZERO_PAD_get'))

# Start of WienerDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get

`WienerDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_WienerDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__WienerDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`WienerDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`, 'returnType') = 'character'
class(`WienerDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get'))

# Start of WienerDeconvolutionImageFilter_PERIODIC_PAD_get

`WienerDeconvolutionImageFilter_PERIODIC_PAD_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_WienerDeconvolutionImageFilter_PERIODIC_PAD_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__WienerDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`WienerDeconvolutionImageFilter_PERIODIC_PAD_get`, 'returnType') = 'character'
class(`WienerDeconvolutionImageFilter_PERIODIC_PAD_get`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_PERIODIC_PAD_get'))

# Start of WienerDeconvolutionImageFilter_SetBoundaryCondition

`WienerDeconvolutionImageFilter_SetBoundaryCondition` = function(self, BoundaryCondition)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BoundaryCondition = enumToInteger(BoundaryCondition, "_itk__simple__WienerDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(BoundaryCondition) > 1) {
    warning("using only the first element of BoundaryCondition");
  };
  
  ;ans = .Call('R_swig_WienerDeconvolutionImageFilter_SetBoundaryCondition', self, BoundaryCondition, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WienerDeconvolutionImageFilter_SetBoundaryCondition`, 'returnType') = 'void'
attr(`WienerDeconvolutionImageFilter_SetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__WienerDeconvolutionImageFilter', 'character')
class(`WienerDeconvolutionImageFilter_SetBoundaryCondition`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_SetBoundaryCondition'))

# Start of WienerDeconvolutionImageFilter_GetBoundaryCondition

`WienerDeconvolutionImageFilter_GetBoundaryCondition` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_WienerDeconvolutionImageFilter_GetBoundaryCondition', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__WienerDeconvolutionImageFilter__BoundaryConditionType");
  
  ans
  
}

attr(`WienerDeconvolutionImageFilter_GetBoundaryCondition`, 'returnType') = 'character'
attr(`WienerDeconvolutionImageFilter_GetBoundaryCondition`, "inputTypes") = c('_p_itk__simple__WienerDeconvolutionImageFilter')
class(`WienerDeconvolutionImageFilter_GetBoundaryCondition`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_GetBoundaryCondition'))

# Start of WienerDeconvolutionImageFilter_SAME_get

`WienerDeconvolutionImageFilter_SAME_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_WienerDeconvolutionImageFilter_SAME_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__WienerDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`WienerDeconvolutionImageFilter_SAME_get`, 'returnType') = 'character'
class(`WienerDeconvolutionImageFilter_SAME_get`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_SAME_get'))

# Start of WienerDeconvolutionImageFilter_VALID_get

`WienerDeconvolutionImageFilter_VALID_get` = function(.copy = FALSE)
{
  ;ans = .Call('R_swig_WienerDeconvolutionImageFilter_VALID_get', as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__WienerDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`WienerDeconvolutionImageFilter_VALID_get`, 'returnType') = 'character'
class(`WienerDeconvolutionImageFilter_VALID_get`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_VALID_get'))

# Start of WienerDeconvolutionImageFilter_SetOutputRegionMode

`WienerDeconvolutionImageFilter_SetOutputRegionMode` = function(self, OutputRegionMode)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutputRegionMode = enumToInteger(OutputRegionMode, "_itk__simple__WienerDeconvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(OutputRegionMode) > 1) {
    warning("using only the first element of OutputRegionMode");
  };
  
  ;ans = .Call('R_swig_WienerDeconvolutionImageFilter_SetOutputRegionMode', self, OutputRegionMode, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WienerDeconvolutionImageFilter_SetOutputRegionMode`, 'returnType') = 'void'
attr(`WienerDeconvolutionImageFilter_SetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__WienerDeconvolutionImageFilter', 'character')
class(`WienerDeconvolutionImageFilter_SetOutputRegionMode`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_SetOutputRegionMode'))

# Start of WienerDeconvolutionImageFilter_GetOutputRegionMode

`WienerDeconvolutionImageFilter_GetOutputRegionMode` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_WienerDeconvolutionImageFilter_GetOutputRegionMode', self, as.logical(.copy), PACKAGE='SimpleITK');
  ans = enumFromInteger(ans, "_itk__simple__WienerDeconvolutionImageFilter__OutputRegionModeType");
  
  ans
  
}

attr(`WienerDeconvolutionImageFilter_GetOutputRegionMode`, 'returnType') = 'character'
attr(`WienerDeconvolutionImageFilter_GetOutputRegionMode`, "inputTypes") = c('_p_itk__simple__WienerDeconvolutionImageFilter')
class(`WienerDeconvolutionImageFilter_GetOutputRegionMode`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_GetOutputRegionMode'))

# Start of WienerDeconvolutionImageFilter_GetName

`WienerDeconvolutionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WienerDeconvolutionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WienerDeconvolutionImageFilter_GetName`, 'returnType') = 'character'
attr(`WienerDeconvolutionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__WienerDeconvolutionImageFilter')
class(`WienerDeconvolutionImageFilter_GetName`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_GetName'))

# Start of WienerDeconvolutionImageFilter_ToString

`WienerDeconvolutionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WienerDeconvolutionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WienerDeconvolutionImageFilter_ToString`, 'returnType') = 'character'
attr(`WienerDeconvolutionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__WienerDeconvolutionImageFilter')
class(`WienerDeconvolutionImageFilter_ToString`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_ToString'))

# Start of WienerDeconvolutionImageFilter_Execute

`WienerDeconvolutionImageFilter_Execute` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_WienerDeconvolutionImageFilter_Execute', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WienerDeconvolutionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`WienerDeconvolutionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__WienerDeconvolutionImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`WienerDeconvolutionImageFilter_Execute`) = c("SWIGFunction", class('WienerDeconvolutionImageFilter_Execute'))

# Start of accessor method for itk::simple::WienerDeconvolutionImageFilter
setMethod('$', '_p_itk__simple__WienerDeconvolutionImageFilter', function(x, name)

{
  accessorFuns = list('SetNoiseVariance' = WienerDeconvolutionImageFilter_SetNoiseVariance, 'GetNoiseVariance' = WienerDeconvolutionImageFilter_GetNoiseVariance, 'SetNormalize' = WienerDeconvolutionImageFilter_SetNormalize, 'NormalizeOn' = WienerDeconvolutionImageFilter_NormalizeOn, 'NormalizeOff' = WienerDeconvolutionImageFilter_NormalizeOff, 'GetNormalize' = WienerDeconvolutionImageFilter_GetNormalize, 'SetBoundaryCondition' = WienerDeconvolutionImageFilter_SetBoundaryCondition, 'GetBoundaryCondition' = WienerDeconvolutionImageFilter_GetBoundaryCondition, 'SetOutputRegionMode' = WienerDeconvolutionImageFilter_SetOutputRegionMode, 'GetOutputRegionMode' = WienerDeconvolutionImageFilter_GetOutputRegionMode, 'GetName' = WienerDeconvolutionImageFilter_GetName, 'ToString' = WienerDeconvolutionImageFilter_ToString, 'Execute' = WienerDeconvolutionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::WienerDeconvolutionImageFilter
# Start of WienerDeconvolution

`WienerDeconvolution__SWIG_0` = function(image1, image2, noiseVariance, normalize, boundaryCondition, outputRegionMode, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__WienerDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  outputRegionMode = enumToInteger(outputRegionMode, "_itk__simple__WienerDeconvolutionImageFilter__OutputRegionModeType"); 
  
  if(length(outputRegionMode) > 1) {
    warning("using only the first element of outputRegionMode");
  };
  
  ;ans = .Call('R_swig_WienerDeconvolution__SWIG_0', image1, image2, noiseVariance, normalize, boundaryCondition, outputRegionMode, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WienerDeconvolution__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`WienerDeconvolution__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'logical', 'character', 'character')
class(`WienerDeconvolution__SWIG_0`) = c("SWIGFunction", class('WienerDeconvolution__SWIG_0'))

# Start of WienerDeconvolution

`WienerDeconvolution__SWIG_1` = function(image1, image2, noiseVariance, normalize, boundaryCondition, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  normalize = as.logical(normalize);
  boundaryCondition = enumToInteger(boundaryCondition, "_itk__simple__WienerDeconvolutionImageFilter__BoundaryConditionType"); 
  
  if(length(boundaryCondition) > 1) {
    warning("using only the first element of boundaryCondition");
  };
  
  ;ans = .Call('R_swig_WienerDeconvolution__SWIG_1', image1, image2, noiseVariance, normalize, boundaryCondition, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WienerDeconvolution__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`WienerDeconvolution__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'logical', 'character')
class(`WienerDeconvolution__SWIG_1`) = c("SWIGFunction", class('WienerDeconvolution__SWIG_1'))

# Start of WienerDeconvolution

`WienerDeconvolution__SWIG_2` = function(image1, image2, noiseVariance, normalize, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  normalize = as.logical(normalize);
  ;ans = .Call('R_swig_WienerDeconvolution__SWIG_2', image1, image2, noiseVariance, normalize, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WienerDeconvolution__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`WienerDeconvolution__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric', 'logical')
class(`WienerDeconvolution__SWIG_2`) = c("SWIGFunction", class('WienerDeconvolution__SWIG_2'))

# Start of WienerDeconvolution

`WienerDeconvolution__SWIG_3` = function(image1, image2, noiseVariance, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  
  ;ans = .Call('R_swig_WienerDeconvolution__SWIG_3', image1, image2, noiseVariance, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WienerDeconvolution__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`WienerDeconvolution__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'numeric')
class(`WienerDeconvolution__SWIG_3`) = c("SWIGFunction", class('WienerDeconvolution__SWIG_3'))

# Start of WienerDeconvolution

`WienerDeconvolution__SWIG_4` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_WienerDeconvolution__SWIG_4', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WienerDeconvolution__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`WienerDeconvolution__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`WienerDeconvolution__SWIG_4`) = c("SWIGFunction", class('WienerDeconvolution__SWIG_4'))

`WienerDeconvolution` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- WienerDeconvolution__SWIG_4; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 )) {
      f <- WienerDeconvolution__SWIG_3; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 )) {
      f <- WienerDeconvolution__SWIG_2; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.character(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- WienerDeconvolution__SWIG_1; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( is.numeric(argv[[3]]) && length(argv[[3]]) == 1 ) && ( is.logical(argv[[4]]) && length(argv[[4]]) == 1 ) && ( is.character(argv[[5]]) && length(argv[[5]]) == 1 ) && ( is.character(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- WienerDeconvolution__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for WienerDeconvolution with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_WrapPadImageFilter

`delete_WrapPadImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_WrapPadImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_WrapPadImageFilter`, 'returnType') = 'void'
attr(`delete_WrapPadImageFilter`, "inputTypes") = c('_p_itk__simple__WrapPadImageFilter')
class(`delete_WrapPadImageFilter`) = c("SWIGFunction", class('delete_WrapPadImageFilter'))

# Start of new_WrapPadImageFilter

`WrapPadImageFilter` = function()
{
  ;ans = .Call('R_swig_new_WrapPadImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__WrapPadImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_WrapPadImageFilter);
  ans
  
}

attr(`WrapPadImageFilter`, 'returnType') = '_p_itk__simple__WrapPadImageFilter'
class(`WrapPadImageFilter`) = c("SWIGFunction", class('WrapPadImageFilter'))

# Start of WrapPadImageFilter_SetPadLowerBound

`WrapPadImageFilter_SetPadLowerBound` = function(self, PadLowerBound)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  PadLowerBound = as.integer(PadLowerBound);
  ;ans = .Call('R_swig_WrapPadImageFilter_SetPadLowerBound', self, PadLowerBound, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WrapPadImageFilter_SetPadLowerBound`, 'returnType') = 'void'
attr(`WrapPadImageFilter_SetPadLowerBound`, "inputTypes") = c('_p_itk__simple__WrapPadImageFilter', 'integer')
class(`WrapPadImageFilter_SetPadLowerBound`) = c("SWIGFunction", class('WrapPadImageFilter_SetPadLowerBound'))

# Start of WrapPadImageFilter_GetPadLowerBound

`WrapPadImageFilter_GetPadLowerBound` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WrapPadImageFilter_GetPadLowerBound', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WrapPadImageFilter_GetPadLowerBound`, 'returnType') = 'integer'
attr(`WrapPadImageFilter_GetPadLowerBound`, "inputTypes") = c('_p_itk__simple__WrapPadImageFilter')
class(`WrapPadImageFilter_GetPadLowerBound`) = c("SWIGFunction", class('WrapPadImageFilter_GetPadLowerBound'))

# Start of WrapPadImageFilter_SetPadUpperBound

`WrapPadImageFilter_SetPadUpperBound` = function(self, PadUpperBound)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  PadUpperBound = as.integer(PadUpperBound);
  ;ans = .Call('R_swig_WrapPadImageFilter_SetPadUpperBound', self, PadUpperBound, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`WrapPadImageFilter_SetPadUpperBound`, 'returnType') = 'void'
attr(`WrapPadImageFilter_SetPadUpperBound`, "inputTypes") = c('_p_itk__simple__WrapPadImageFilter', 'integer')
class(`WrapPadImageFilter_SetPadUpperBound`) = c("SWIGFunction", class('WrapPadImageFilter_SetPadUpperBound'))

# Start of WrapPadImageFilter_GetPadUpperBound

`WrapPadImageFilter_GetPadUpperBound` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WrapPadImageFilter_GetPadUpperBound', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WrapPadImageFilter_GetPadUpperBound`, 'returnType') = 'integer'
attr(`WrapPadImageFilter_GetPadUpperBound`, "inputTypes") = c('_p_itk__simple__WrapPadImageFilter')
class(`WrapPadImageFilter_GetPadUpperBound`) = c("SWIGFunction", class('WrapPadImageFilter_GetPadUpperBound'))

# Start of WrapPadImageFilter_GetName

`WrapPadImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WrapPadImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WrapPadImageFilter_GetName`, 'returnType') = 'character'
attr(`WrapPadImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__WrapPadImageFilter')
class(`WrapPadImageFilter_GetName`) = c("SWIGFunction", class('WrapPadImageFilter_GetName'))

# Start of WrapPadImageFilter_ToString

`WrapPadImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_WrapPadImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`WrapPadImageFilter_ToString`, 'returnType') = 'character'
attr(`WrapPadImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__WrapPadImageFilter')
class(`WrapPadImageFilter_ToString`) = c("SWIGFunction", class('WrapPadImageFilter_ToString'))

# Start of WrapPadImageFilter_Execute

`WrapPadImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_WrapPadImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WrapPadImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`WrapPadImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__WrapPadImageFilter', '_p_itk__simple__Image')
class(`WrapPadImageFilter_Execute`) = c("SWIGFunction", class('WrapPadImageFilter_Execute'))

# Start of accessor method for itk::simple::WrapPadImageFilter
setMethod('$', '_p_itk__simple__WrapPadImageFilter', function(x, name)

{
  accessorFuns = list('SetPadLowerBound' = WrapPadImageFilter_SetPadLowerBound, 'GetPadLowerBound' = WrapPadImageFilter_GetPadLowerBound, 'SetPadUpperBound' = WrapPadImageFilter_SetPadUpperBound, 'GetPadUpperBound' = WrapPadImageFilter_GetPadUpperBound, 'GetName' = WrapPadImageFilter_GetName, 'ToString' = WrapPadImageFilter_ToString, 'Execute' = WrapPadImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::WrapPadImageFilter
# Start of WrapPad

`WrapPad__SWIG_0` = function(image1, padLowerBound, padUpperBound, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  padLowerBound = as.integer(padLowerBound);
  padUpperBound = as.integer(padUpperBound);
  ;ans = .Call('R_swig_WrapPad__SWIG_0', image1, padLowerBound, padUpperBound, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WrapPad__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`WrapPad__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`WrapPad__SWIG_0`) = c("SWIGFunction", class('WrapPad__SWIG_0'))

# Start of WrapPad

`WrapPad__SWIG_1` = function(image1, padLowerBound, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  padLowerBound = as.integer(padLowerBound);
  ;ans = .Call('R_swig_WrapPad__SWIG_1', image1, padLowerBound, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WrapPad__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`WrapPad__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`WrapPad__SWIG_1`) = c("SWIGFunction", class('WrapPad__SWIG_1'))

# Start of WrapPad

`WrapPad__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_WrapPad__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`WrapPad__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`WrapPad__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`WrapPad__SWIG_2`) = c("SWIGFunction", class('WrapPad__SWIG_2'))

`WrapPad` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- WrapPad__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- WrapPad__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- WrapPad__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for WrapPad with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_XorImageFilter

`delete_XorImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_XorImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_XorImageFilter`, 'returnType') = 'void'
attr(`delete_XorImageFilter`, "inputTypes") = c('_p_itk__simple__XorImageFilter')
class(`delete_XorImageFilter`) = c("SWIGFunction", class('delete_XorImageFilter'))

# Start of new_XorImageFilter

`XorImageFilter` = function()
{
  ;ans = .Call('R_swig_new_XorImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__XorImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_XorImageFilter);
  ans
  
}

attr(`XorImageFilter`, 'returnType') = '_p_itk__simple__XorImageFilter'
class(`XorImageFilter`) = c("SWIGFunction", class('XorImageFilter'))

# Start of XorImageFilter_GetName

`XorImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_XorImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`XorImageFilter_GetName`, 'returnType') = 'character'
attr(`XorImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__XorImageFilter')
class(`XorImageFilter_GetName`) = c("SWIGFunction", class('XorImageFilter_GetName'))

# Start of XorImageFilter_ToString

`XorImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_XorImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`XorImageFilter_ToString`, 'returnType') = 'character'
attr(`XorImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__XorImageFilter')
class(`XorImageFilter_ToString`) = c("SWIGFunction", class('XorImageFilter_ToString'))

# Start of XorImageFilter_Execute

`XorImageFilter_Execute__SWIG_0` = function(self, image1, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_XorImageFilter_Execute__SWIG_0', self, image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`XorImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`XorImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__XorImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`XorImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('XorImageFilter_Execute__SWIG_0'))

# Start of XorImageFilter_Execute

`XorImageFilter_Execute__SWIG_1` = function(self, image1, constant, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  ;ans = .Call('R_swig_XorImageFilter_Execute__SWIG_1', self, image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`XorImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`XorImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__XorImageFilter', '_p_itk__simple__Image', 'integer')
class(`XorImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('XorImageFilter_Execute__SWIG_1'))

# Start of XorImageFilter_Execute

`XorImageFilter_Execute__SWIG_2` = function(self, constant, image2, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_XorImageFilter_Execute__SWIG_2', self, constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`XorImageFilter_Execute__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`XorImageFilter_Execute__SWIG_2`, "inputTypes") = c('_p_itk__simple__XorImageFilter', 'integer', '_p_itk__simple__Image')
class(`XorImageFilter_Execute__SWIG_2`) = c("SWIGFunction", class('XorImageFilter_Execute__SWIG_2'))

`XorImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__XorImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- XorImageFilter_Execute__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__XorImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- XorImageFilter_Execute__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__XorImageFilter') || is.null(argv[[1]]) ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- XorImageFilter_Execute__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for XorImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::XorImageFilter
setMethod('$', '_p_itk__simple__XorImageFilter', function(x, name)

{
  accessorFuns = list('GetName' = XorImageFilter_GetName, 'ToString' = XorImageFilter_ToString, 'Execute' = XorImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::XorImageFilter
# Start of Xor

`Xor__SWIG_0` = function(image1, image2, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Xor__SWIG_0', image1, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Xor__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`Xor__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`Xor__SWIG_0`) = c("SWIGFunction", class('Xor__SWIG_0'))

# Start of Xor

`Xor__SWIG_1` = function(image1, constant, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  ;ans = .Call('R_swig_Xor__SWIG_1', image1, constant, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Xor__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`Xor__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`Xor__SWIG_1`) = c("SWIGFunction", class('Xor__SWIG_1'))

# Start of Xor

`Xor__SWIG_2` = function(constant, image2, .copy = FALSE)
{
  constant = as.integer(constant);
  
  if(length(constant) > 1) {
    warning("using only the first element of constant");
  };
  
  if (inherits(image2, "ExternalReference")) image2 = slot(image2,"ref"); 
  ;ans = .Call('R_swig_Xor__SWIG_2', constant, image2, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`Xor__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`Xor__SWIG_2`, "inputTypes") = c('integer', '_p_itk__simple__Image')
class(`Xor__SWIG_2`) = c("SWIGFunction", class('Xor__SWIG_2'))

`Xor` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Xor__SWIG_0; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- Xor__SWIG_1; 
    }
    else if (( (is.integer(argv[[1]]) || is.numeric(argv[[1]])) && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- Xor__SWIG_2; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for Xor with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_YenThresholdImageFilter

`delete_YenThresholdImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_YenThresholdImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_YenThresholdImageFilter`, 'returnType') = 'void'
attr(`delete_YenThresholdImageFilter`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter')
class(`delete_YenThresholdImageFilter`) = c("SWIGFunction", class('delete_YenThresholdImageFilter'))

# Start of new_YenThresholdImageFilter

`YenThresholdImageFilter` = function()
{
  ;ans = .Call('R_swig_new_YenThresholdImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__YenThresholdImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_YenThresholdImageFilter);
  ans
  
}

attr(`YenThresholdImageFilter`, 'returnType') = '_p_itk__simple__YenThresholdImageFilter'
class(`YenThresholdImageFilter`) = c("SWIGFunction", class('YenThresholdImageFilter'))

# Start of YenThresholdImageFilter_SetInsideValue

`YenThresholdImageFilter_SetInsideValue` = function(self, InsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  InsideValue = as.integer(InsideValue);
  
  if(length(InsideValue) > 1) {
    warning("using only the first element of InsideValue");
  };
  
  ;ans = .Call('R_swig_YenThresholdImageFilter_SetInsideValue', self, InsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`YenThresholdImageFilter_SetInsideValue`, 'returnType') = 'void'
attr(`YenThresholdImageFilter_SetInsideValue`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter', 'integer')
class(`YenThresholdImageFilter_SetInsideValue`) = c("SWIGFunction", class('YenThresholdImageFilter_SetInsideValue'))

# Start of YenThresholdImageFilter_GetInsideValue

`YenThresholdImageFilter_GetInsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_YenThresholdImageFilter_GetInsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`YenThresholdImageFilter_GetInsideValue`, 'returnType') = 'integer'
attr(`YenThresholdImageFilter_GetInsideValue`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter')
class(`YenThresholdImageFilter_GetInsideValue`) = c("SWIGFunction", class('YenThresholdImageFilter_GetInsideValue'))

# Start of YenThresholdImageFilter_SetOutsideValue

`YenThresholdImageFilter_SetOutsideValue` = function(self, OutsideValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  OutsideValue = as.integer(OutsideValue);
  
  if(length(OutsideValue) > 1) {
    warning("using only the first element of OutsideValue");
  };
  
  ;ans = .Call('R_swig_YenThresholdImageFilter_SetOutsideValue', self, OutsideValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`YenThresholdImageFilter_SetOutsideValue`, 'returnType') = 'void'
attr(`YenThresholdImageFilter_SetOutsideValue`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter', 'integer')
class(`YenThresholdImageFilter_SetOutsideValue`) = c("SWIGFunction", class('YenThresholdImageFilter_SetOutsideValue'))

# Start of YenThresholdImageFilter_GetOutsideValue

`YenThresholdImageFilter_GetOutsideValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_YenThresholdImageFilter_GetOutsideValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`YenThresholdImageFilter_GetOutsideValue`, 'returnType') = 'integer'
attr(`YenThresholdImageFilter_GetOutsideValue`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter')
class(`YenThresholdImageFilter_GetOutsideValue`) = c("SWIGFunction", class('YenThresholdImageFilter_GetOutsideValue'))

# Start of YenThresholdImageFilter_SetNumberOfHistogramBins

`YenThresholdImageFilter_SetNumberOfHistogramBins` = function(self, NumberOfHistogramBins)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  NumberOfHistogramBins = as.integer(NumberOfHistogramBins);
  
  if(length(NumberOfHistogramBins) > 1) {
    warning("using only the first element of NumberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_YenThresholdImageFilter_SetNumberOfHistogramBins', self, NumberOfHistogramBins, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`YenThresholdImageFilter_SetNumberOfHistogramBins`, 'returnType') = 'void'
attr(`YenThresholdImageFilter_SetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter', 'integer')
class(`YenThresholdImageFilter_SetNumberOfHistogramBins`) = c("SWIGFunction", class('YenThresholdImageFilter_SetNumberOfHistogramBins'))

# Start of YenThresholdImageFilter_GetNumberOfHistogramBins

`YenThresholdImageFilter_GetNumberOfHistogramBins` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_YenThresholdImageFilter_GetNumberOfHistogramBins', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`YenThresholdImageFilter_GetNumberOfHistogramBins`, 'returnType') = 'integer'
attr(`YenThresholdImageFilter_GetNumberOfHistogramBins`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter')
class(`YenThresholdImageFilter_GetNumberOfHistogramBins`) = c("SWIGFunction", class('YenThresholdImageFilter_GetNumberOfHistogramBins'))

# Start of YenThresholdImageFilter_SetMaskOutput

`YenThresholdImageFilter_SetMaskOutput` = function(self, MaskOutput)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskOutput = as.logical(MaskOutput);
  ;ans = .Call('R_swig_YenThresholdImageFilter_SetMaskOutput', self, MaskOutput, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`YenThresholdImageFilter_SetMaskOutput`, 'returnType') = 'void'
attr(`YenThresholdImageFilter_SetMaskOutput`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter', 'logical')
class(`YenThresholdImageFilter_SetMaskOutput`) = c("SWIGFunction", class('YenThresholdImageFilter_SetMaskOutput'))

# Start of YenThresholdImageFilter_MaskOutputOn

`YenThresholdImageFilter_MaskOutputOn` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_YenThresholdImageFilter_MaskOutputOn', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`YenThresholdImageFilter_MaskOutputOn`, 'returnType') = 'void'
attr(`YenThresholdImageFilter_MaskOutputOn`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter')
class(`YenThresholdImageFilter_MaskOutputOn`) = c("SWIGFunction", class('YenThresholdImageFilter_MaskOutputOn'))

# Start of YenThresholdImageFilter_MaskOutputOff

`YenThresholdImageFilter_MaskOutputOff` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_YenThresholdImageFilter_MaskOutputOff', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`YenThresholdImageFilter_MaskOutputOff`, 'returnType') = 'void'
attr(`YenThresholdImageFilter_MaskOutputOff`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter')
class(`YenThresholdImageFilter_MaskOutputOff`) = c("SWIGFunction", class('YenThresholdImageFilter_MaskOutputOff'))

# Start of YenThresholdImageFilter_GetMaskOutput

`YenThresholdImageFilter_GetMaskOutput` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_YenThresholdImageFilter_GetMaskOutput', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`YenThresholdImageFilter_GetMaskOutput`, 'returnType') = 'logical'
attr(`YenThresholdImageFilter_GetMaskOutput`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter')
class(`YenThresholdImageFilter_GetMaskOutput`) = c("SWIGFunction", class('YenThresholdImageFilter_GetMaskOutput'))

# Start of YenThresholdImageFilter_SetMaskValue

`YenThresholdImageFilter_SetMaskValue` = function(self, MaskValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  MaskValue = as.integer(MaskValue);
  
  if(length(MaskValue) > 1) {
    warning("using only the first element of MaskValue");
  };
  
  ;ans = .Call('R_swig_YenThresholdImageFilter_SetMaskValue', self, MaskValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`YenThresholdImageFilter_SetMaskValue`, 'returnType') = 'void'
attr(`YenThresholdImageFilter_SetMaskValue`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter', 'integer')
class(`YenThresholdImageFilter_SetMaskValue`) = c("SWIGFunction", class('YenThresholdImageFilter_SetMaskValue'))

# Start of YenThresholdImageFilter_GetMaskValue

`YenThresholdImageFilter_GetMaskValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_YenThresholdImageFilter_GetMaskValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`YenThresholdImageFilter_GetMaskValue`, 'returnType') = 'integer'
attr(`YenThresholdImageFilter_GetMaskValue`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter')
class(`YenThresholdImageFilter_GetMaskValue`) = c("SWIGFunction", class('YenThresholdImageFilter_GetMaskValue'))

# Start of YenThresholdImageFilter_GetThreshold

`YenThresholdImageFilter_GetThreshold` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_YenThresholdImageFilter_GetThreshold', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`YenThresholdImageFilter_GetThreshold`, 'returnType') = 'numeric'
attr(`YenThresholdImageFilter_GetThreshold`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter')
class(`YenThresholdImageFilter_GetThreshold`) = c("SWIGFunction", class('YenThresholdImageFilter_GetThreshold'))

# Start of YenThresholdImageFilter_GetName

`YenThresholdImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_YenThresholdImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`YenThresholdImageFilter_GetName`, 'returnType') = 'character'
attr(`YenThresholdImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter')
class(`YenThresholdImageFilter_GetName`) = c("SWIGFunction", class('YenThresholdImageFilter_GetName'))

# Start of YenThresholdImageFilter_ToString

`YenThresholdImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_YenThresholdImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`YenThresholdImageFilter_ToString`, 'returnType') = 'character'
attr(`YenThresholdImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter')
class(`YenThresholdImageFilter_ToString`) = c("SWIGFunction", class('YenThresholdImageFilter_ToString'))

# Start of YenThresholdImageFilter_Execute

`YenThresholdImageFilter_Execute__SWIG_0` = function(self, image, maskImage, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_YenThresholdImageFilter_Execute__SWIG_0', self, image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`YenThresholdImageFilter_Execute__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`YenThresholdImageFilter_Execute__SWIG_0`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter', '_p_itk__simple__Image', '_p_itk__simple__Image')
class(`YenThresholdImageFilter_Execute__SWIG_0`) = c("SWIGFunction", class('YenThresholdImageFilter_Execute__SWIG_0'))

# Start of YenThresholdImageFilter_Execute

`YenThresholdImageFilter_Execute__SWIG_1` = function(self, image, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_YenThresholdImageFilter_Execute__SWIG_1', self, image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`YenThresholdImageFilter_Execute__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`YenThresholdImageFilter_Execute__SWIG_1`, "inputTypes") = c('_p_itk__simple__YenThresholdImageFilter', '_p_itk__simple__Image')
class(`YenThresholdImageFilter_Execute__SWIG_1`) = c("SWIGFunction", class('YenThresholdImageFilter_Execute__SWIG_1'))

`YenThresholdImageFilter_Execute` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 2
  if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__YenThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- YenThresholdImageFilter_Execute__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__YenThresholdImageFilter') || is.null(argv[[1]]) ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( extends(argtypes[3], '_p_itk__simple__Image') && length(argv[[3]]) == 1 )) {
      f <- YenThresholdImageFilter_Execute__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for YenThresholdImageFilter_Execute with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of accessor method for itk::simple::YenThresholdImageFilter
setMethod('$', '_p_itk__simple__YenThresholdImageFilter', function(x, name)

{
  accessorFuns = list('SetInsideValue' = YenThresholdImageFilter_SetInsideValue, 'GetInsideValue' = YenThresholdImageFilter_GetInsideValue, 'SetOutsideValue' = YenThresholdImageFilter_SetOutsideValue, 'GetOutsideValue' = YenThresholdImageFilter_GetOutsideValue, 'SetNumberOfHistogramBins' = YenThresholdImageFilter_SetNumberOfHistogramBins, 'GetNumberOfHistogramBins' = YenThresholdImageFilter_GetNumberOfHistogramBins, 'SetMaskOutput' = YenThresholdImageFilter_SetMaskOutput, 'MaskOutputOn' = YenThresholdImageFilter_MaskOutputOn, 'MaskOutputOff' = YenThresholdImageFilter_MaskOutputOff, 'GetMaskOutput' = YenThresholdImageFilter_GetMaskOutput, 'SetMaskValue' = YenThresholdImageFilter_SetMaskValue, 'GetMaskValue' = YenThresholdImageFilter_GetMaskValue, 'GetThreshold' = YenThresholdImageFilter_GetThreshold, 'GetName' = YenThresholdImageFilter_GetName, 'ToString' = YenThresholdImageFilter_ToString, 'Execute' = YenThresholdImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::YenThresholdImageFilter
# Start of YenThreshold

`YenThreshold__SWIG_0` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_YenThreshold__SWIG_0', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`YenThreshold__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`YenThreshold__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`YenThreshold__SWIG_0`) = c("SWIGFunction", class('YenThreshold__SWIG_0'))

# Start of YenThreshold

`YenThreshold__SWIG_1` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_YenThreshold__SWIG_1', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`YenThreshold__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`YenThreshold__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`YenThreshold__SWIG_1`) = c("SWIGFunction", class('YenThreshold__SWIG_1'))

# Start of YenThreshold

`YenThreshold__SWIG_2` = function(image, maskImage, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_YenThreshold__SWIG_2', image, maskImage, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`YenThreshold__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`YenThreshold__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`YenThreshold__SWIG_2`) = c("SWIGFunction", class('YenThreshold__SWIG_2'))

# Start of YenThreshold

`YenThreshold__SWIG_3` = function(image, maskImage, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_YenThreshold__SWIG_3', image, maskImage, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`YenThreshold__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`YenThreshold__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer', 'integer')
class(`YenThreshold__SWIG_3`) = c("SWIGFunction", class('YenThreshold__SWIG_3'))

# Start of YenThreshold

`YenThreshold__SWIG_4` = function(image, maskImage, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_YenThreshold__SWIG_4', image, maskImage, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`YenThreshold__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`YenThreshold__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image', 'integer')
class(`YenThreshold__SWIG_4`) = c("SWIGFunction", class('YenThreshold__SWIG_4'))

# Start of YenThreshold

`YenThreshold__SWIG_5` = function(image, maskImage, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  if (inherits(maskImage, "ExternalReference")) maskImage = slot(maskImage,"ref"); 
  ;ans = .Call('R_swig_YenThreshold__SWIG_5', image, maskImage, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`YenThreshold__SWIG_5`, 'returnType') = '_p_itk__simple__Image'
attr(`YenThreshold__SWIG_5`, "inputTypes") = c('_p_itk__simple__Image', '_p_itk__simple__Image')
class(`YenThreshold__SWIG_5`) = c("SWIGFunction", class('YenThreshold__SWIG_5'))

# Start of YenThreshold

`YenThreshold__SWIG_6` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  maskValue = as.integer(maskValue);
  
  if(length(maskValue) > 1) {
    warning("using only the first element of maskValue");
  };
  
  ;ans = .Call('R_swig_YenThreshold__SWIG_6', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, maskValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`YenThreshold__SWIG_6`, 'returnType') = '_p_itk__simple__Image'
attr(`YenThreshold__SWIG_6`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical', 'integer')
class(`YenThreshold__SWIG_6`) = c("SWIGFunction", class('YenThreshold__SWIG_6'))

# Start of YenThreshold

`YenThreshold__SWIG_7` = function(image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  maskOutput = as.logical(maskOutput);
  ;ans = .Call('R_swig_YenThreshold__SWIG_7', image, insideValue, outsideValue, numberOfHistogramBins, maskOutput, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`YenThreshold__SWIG_7`, 'returnType') = '_p_itk__simple__Image'
attr(`YenThreshold__SWIG_7`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer', 'logical')
class(`YenThreshold__SWIG_7`) = c("SWIGFunction", class('YenThreshold__SWIG_7'))

# Start of YenThreshold

`YenThreshold__SWIG_8` = function(image, insideValue, outsideValue, numberOfHistogramBins, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  numberOfHistogramBins = as.integer(numberOfHistogramBins);
  
  if(length(numberOfHistogramBins) > 1) {
    warning("using only the first element of numberOfHistogramBins");
  };
  
  ;ans = .Call('R_swig_YenThreshold__SWIG_8', image, insideValue, outsideValue, numberOfHistogramBins, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`YenThreshold__SWIG_8`, 'returnType') = '_p_itk__simple__Image'
attr(`YenThreshold__SWIG_8`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer', 'integer')
class(`YenThreshold__SWIG_8`) = c("SWIGFunction", class('YenThreshold__SWIG_8'))

# Start of YenThreshold

`YenThreshold__SWIG_9` = function(image, insideValue, outsideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  outsideValue = as.integer(outsideValue);
  
  if(length(outsideValue) > 1) {
    warning("using only the first element of outsideValue");
  };
  
  ;ans = .Call('R_swig_YenThreshold__SWIG_9', image, insideValue, outsideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`YenThreshold__SWIG_9`, 'returnType') = '_p_itk__simple__Image'
attr(`YenThreshold__SWIG_9`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`YenThreshold__SWIG_9`) = c("SWIGFunction", class('YenThreshold__SWIG_9'))

# Start of YenThreshold

`YenThreshold__SWIG_10` = function(image, insideValue, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  insideValue = as.integer(insideValue);
  
  if(length(insideValue) > 1) {
    warning("using only the first element of insideValue");
  };
  
  ;ans = .Call('R_swig_YenThreshold__SWIG_10', image, insideValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`YenThreshold__SWIG_10`, 'returnType') = '_p_itk__simple__Image'
attr(`YenThreshold__SWIG_10`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`YenThreshold__SWIG_10`) = c("SWIGFunction", class('YenThreshold__SWIG_10'))

# Start of YenThreshold

`YenThreshold__SWIG_11` = function(image, .copy = FALSE)
{
  if (inherits(image, "ExternalReference")) image = slot(image,"ref"); 
  ;ans = .Call('R_swig_YenThreshold__SWIG_11', image, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`YenThreshold__SWIG_11`, 'returnType') = '_p_itk__simple__Image'
attr(`YenThreshold__SWIG_11`, "inputTypes") = c('_p_itk__simple__Image')
class(`YenThreshold__SWIG_11`) = c("SWIGFunction", class('YenThreshold__SWIG_11'))

`YenThreshold` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 12
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- YenThreshold__SWIG_11; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 )) {
      f <- YenThreshold__SWIG_5; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- YenThreshold__SWIG_10; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- YenThreshold__SWIG_4; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- YenThreshold__SWIG_9; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- YenThreshold__SWIG_3; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- YenThreshold__SWIG_8; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 )) {
      f <- YenThreshold__SWIG_2; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- YenThreshold__SWIG_7; 
    }
  } else if (argc == 6) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 )) {
      f <- YenThreshold__SWIG_1; 
    }
    else if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.logical(argv[[5]]) && length(argv[[5]]) == 1 ) && ( (is.integer(argv[[6]]) || is.numeric(argv[[6]])) && length(argv[[6]]) == 1 )) {
      f <- YenThreshold__SWIG_6; 
    }
  } else if (argc == 7) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( extends(argtypes[2], '_p_itk__simple__Image') && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( (is.integer(argv[[5]]) || is.numeric(argv[[5]])) && length(argv[[5]]) == 1 ) && ( is.logical(argv[[6]]) && length(argv[[6]]) == 1 ) && ( (is.integer(argv[[7]]) || is.numeric(argv[[7]])) && length(argv[[7]]) == 1 )) {
      f <- YenThreshold__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for YenThreshold with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ZeroCrossingBasedEdgeDetectionImageFilter

`delete_ZeroCrossingBasedEdgeDetectionImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ZeroCrossingBasedEdgeDetectionImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ZeroCrossingBasedEdgeDetectionImageFilter`, 'returnType') = 'void'
attr(`delete_ZeroCrossingBasedEdgeDetectionImageFilter`, "inputTypes") = c('_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter')
class(`delete_ZeroCrossingBasedEdgeDetectionImageFilter`) = c("SWIGFunction", class('delete_ZeroCrossingBasedEdgeDetectionImageFilter'))

# Start of new_ZeroCrossingBasedEdgeDetectionImageFilter

`ZeroCrossingBasedEdgeDetectionImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ZeroCrossingBasedEdgeDetectionImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ZeroCrossingBasedEdgeDetectionImageFilter);
  ans
  
}

attr(`ZeroCrossingBasedEdgeDetectionImageFilter`, 'returnType') = '_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter'
class(`ZeroCrossingBasedEdgeDetectionImageFilter`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetectionImageFilter'))

# Start of ZeroCrossingBasedEdgeDetectionImageFilter_SetVariance

`ZeroCrossingBasedEdgeDetectionImageFilter_SetVariance` = function(self, Variance)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ZeroCrossingBasedEdgeDetectionImageFilter_SetVariance', self, Variance, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ZeroCrossingBasedEdgeDetectionImageFilter_SetVariance`, 'returnType') = 'void'
attr(`ZeroCrossingBasedEdgeDetectionImageFilter_SetVariance`, "inputTypes") = c('_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter', 'numeric')
class(`ZeroCrossingBasedEdgeDetectionImageFilter_SetVariance`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetectionImageFilter_SetVariance'))

# Start of ZeroCrossingBasedEdgeDetectionImageFilter_GetVariance

`ZeroCrossingBasedEdgeDetectionImageFilter_GetVariance` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ZeroCrossingBasedEdgeDetectionImageFilter_GetVariance', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ZeroCrossingBasedEdgeDetectionImageFilter_GetVariance`, 'returnType') = 'numeric'
attr(`ZeroCrossingBasedEdgeDetectionImageFilter_GetVariance`, "inputTypes") = c('_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter')
class(`ZeroCrossingBasedEdgeDetectionImageFilter_GetVariance`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetectionImageFilter_GetVariance'))

# Start of ZeroCrossingBasedEdgeDetectionImageFilter_SetForegroundValue

`ZeroCrossingBasedEdgeDetectionImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ForegroundValue = as.integer(ForegroundValue);
  
  if(length(ForegroundValue) > 1) {
    warning("using only the first element of ForegroundValue");
  };
  
  ;ans = .Call('R_swig_ZeroCrossingBasedEdgeDetectionImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ZeroCrossingBasedEdgeDetectionImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`ZeroCrossingBasedEdgeDetectionImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter', 'integer')
class(`ZeroCrossingBasedEdgeDetectionImageFilter_SetForegroundValue`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetectionImageFilter_SetForegroundValue'))

# Start of ZeroCrossingBasedEdgeDetectionImageFilter_GetForegroundValue

`ZeroCrossingBasedEdgeDetectionImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ZeroCrossingBasedEdgeDetectionImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ZeroCrossingBasedEdgeDetectionImageFilter_GetForegroundValue`, 'returnType') = 'integer'
attr(`ZeroCrossingBasedEdgeDetectionImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter')
class(`ZeroCrossingBasedEdgeDetectionImageFilter_GetForegroundValue`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetectionImageFilter_GetForegroundValue'))

# Start of ZeroCrossingBasedEdgeDetectionImageFilter_SetBackgroundValue

`ZeroCrossingBasedEdgeDetectionImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BackgroundValue = as.integer(BackgroundValue);
  
  if(length(BackgroundValue) > 1) {
    warning("using only the first element of BackgroundValue");
  };
  
  ;ans = .Call('R_swig_ZeroCrossingBasedEdgeDetectionImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ZeroCrossingBasedEdgeDetectionImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`ZeroCrossingBasedEdgeDetectionImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter', 'integer')
class(`ZeroCrossingBasedEdgeDetectionImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetectionImageFilter_SetBackgroundValue'))

# Start of ZeroCrossingBasedEdgeDetectionImageFilter_GetBackgroundValue

`ZeroCrossingBasedEdgeDetectionImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ZeroCrossingBasedEdgeDetectionImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ZeroCrossingBasedEdgeDetectionImageFilter_GetBackgroundValue`, 'returnType') = 'integer'
attr(`ZeroCrossingBasedEdgeDetectionImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter')
class(`ZeroCrossingBasedEdgeDetectionImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetectionImageFilter_GetBackgroundValue'))

# Start of ZeroCrossingBasedEdgeDetectionImageFilter_SetMaximumError

`ZeroCrossingBasedEdgeDetectionImageFilter_SetMaximumError` = function(self, MaximumError)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  
  ;ans = .Call('R_swig_ZeroCrossingBasedEdgeDetectionImageFilter_SetMaximumError', self, MaximumError, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ZeroCrossingBasedEdgeDetectionImageFilter_SetMaximumError`, 'returnType') = 'void'
attr(`ZeroCrossingBasedEdgeDetectionImageFilter_SetMaximumError`, "inputTypes") = c('_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter', 'numeric')
class(`ZeroCrossingBasedEdgeDetectionImageFilter_SetMaximumError`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetectionImageFilter_SetMaximumError'))

# Start of ZeroCrossingBasedEdgeDetectionImageFilter_GetMaximumError

`ZeroCrossingBasedEdgeDetectionImageFilter_GetMaximumError` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ZeroCrossingBasedEdgeDetectionImageFilter_GetMaximumError', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ZeroCrossingBasedEdgeDetectionImageFilter_GetMaximumError`, 'returnType') = 'numeric'
attr(`ZeroCrossingBasedEdgeDetectionImageFilter_GetMaximumError`, "inputTypes") = c('_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter')
class(`ZeroCrossingBasedEdgeDetectionImageFilter_GetMaximumError`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetectionImageFilter_GetMaximumError'))

# Start of ZeroCrossingBasedEdgeDetectionImageFilter_GetName

`ZeroCrossingBasedEdgeDetectionImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ZeroCrossingBasedEdgeDetectionImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ZeroCrossingBasedEdgeDetectionImageFilter_GetName`, 'returnType') = 'character'
attr(`ZeroCrossingBasedEdgeDetectionImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter')
class(`ZeroCrossingBasedEdgeDetectionImageFilter_GetName`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetectionImageFilter_GetName'))

# Start of ZeroCrossingBasedEdgeDetectionImageFilter_ToString

`ZeroCrossingBasedEdgeDetectionImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ZeroCrossingBasedEdgeDetectionImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ZeroCrossingBasedEdgeDetectionImageFilter_ToString`, 'returnType') = 'character'
attr(`ZeroCrossingBasedEdgeDetectionImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter')
class(`ZeroCrossingBasedEdgeDetectionImageFilter_ToString`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetectionImageFilter_ToString'))

# Start of ZeroCrossingBasedEdgeDetectionImageFilter_Execute

`ZeroCrossingBasedEdgeDetectionImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ZeroCrossingBasedEdgeDetectionImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ZeroCrossingBasedEdgeDetectionImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ZeroCrossingBasedEdgeDetectionImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter', '_p_itk__simple__Image')
class(`ZeroCrossingBasedEdgeDetectionImageFilter_Execute`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetectionImageFilter_Execute'))

# Start of accessor method for itk::simple::ZeroCrossingBasedEdgeDetectionImageFilter
setMethod('$', '_p_itk__simple__ZeroCrossingBasedEdgeDetectionImageFilter', function(x, name)

{
  accessorFuns = list('SetVariance' = ZeroCrossingBasedEdgeDetectionImageFilter_SetVariance, 'GetVariance' = ZeroCrossingBasedEdgeDetectionImageFilter_GetVariance, 'SetForegroundValue' = ZeroCrossingBasedEdgeDetectionImageFilter_SetForegroundValue, 'GetForegroundValue' = ZeroCrossingBasedEdgeDetectionImageFilter_GetForegroundValue, 'SetBackgroundValue' = ZeroCrossingBasedEdgeDetectionImageFilter_SetBackgroundValue, 'GetBackgroundValue' = ZeroCrossingBasedEdgeDetectionImageFilter_GetBackgroundValue, 'SetMaximumError' = ZeroCrossingBasedEdgeDetectionImageFilter_SetMaximumError, 'GetMaximumError' = ZeroCrossingBasedEdgeDetectionImageFilter_GetMaximumError, 'GetName' = ZeroCrossingBasedEdgeDetectionImageFilter_GetName, 'ToString' = ZeroCrossingBasedEdgeDetectionImageFilter_ToString, 'Execute' = ZeroCrossingBasedEdgeDetectionImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ZeroCrossingBasedEdgeDetectionImageFilter
# Start of ZeroCrossingBasedEdgeDetection

`ZeroCrossingBasedEdgeDetection__SWIG_0` = function(image1, variance, foregroundValue, backgroundValue, maximumError, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  
  ;ans = .Call('R_swig_ZeroCrossingBasedEdgeDetection__SWIG_0', image1, variance, foregroundValue, backgroundValue, maximumError, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ZeroCrossingBasedEdgeDetection__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ZeroCrossingBasedEdgeDetection__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer', 'numeric')
class(`ZeroCrossingBasedEdgeDetection__SWIG_0`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetection__SWIG_0'))

# Start of ZeroCrossingBasedEdgeDetection

`ZeroCrossingBasedEdgeDetection__SWIG_1` = function(image1, variance, foregroundValue, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_ZeroCrossingBasedEdgeDetection__SWIG_1', image1, variance, foregroundValue, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ZeroCrossingBasedEdgeDetection__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ZeroCrossingBasedEdgeDetection__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer', 'integer')
class(`ZeroCrossingBasedEdgeDetection__SWIG_1`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetection__SWIG_1'))

# Start of ZeroCrossingBasedEdgeDetection

`ZeroCrossingBasedEdgeDetection__SWIG_2` = function(image1, variance, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_ZeroCrossingBasedEdgeDetection__SWIG_2', image1, variance, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ZeroCrossingBasedEdgeDetection__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ZeroCrossingBasedEdgeDetection__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image', 'numeric', 'integer')
class(`ZeroCrossingBasedEdgeDetection__SWIG_2`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetection__SWIG_2'))

# Start of ZeroCrossingBasedEdgeDetection

`ZeroCrossingBasedEdgeDetection__SWIG_3` = function(image1, variance, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  
  ;ans = .Call('R_swig_ZeroCrossingBasedEdgeDetection__SWIG_3', image1, variance, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ZeroCrossingBasedEdgeDetection__SWIG_3`, 'returnType') = '_p_itk__simple__Image'
attr(`ZeroCrossingBasedEdgeDetection__SWIG_3`, "inputTypes") = c('_p_itk__simple__Image', 'numeric')
class(`ZeroCrossingBasedEdgeDetection__SWIG_3`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetection__SWIG_3'))

# Start of ZeroCrossingBasedEdgeDetection

`ZeroCrossingBasedEdgeDetection__SWIG_4` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ZeroCrossingBasedEdgeDetection__SWIG_4', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ZeroCrossingBasedEdgeDetection__SWIG_4`, 'returnType') = '_p_itk__simple__Image'
attr(`ZeroCrossingBasedEdgeDetection__SWIG_4`, "inputTypes") = c('_p_itk__simple__Image')
class(`ZeroCrossingBasedEdgeDetection__SWIG_4`) = c("SWIGFunction", class('ZeroCrossingBasedEdgeDetection__SWIG_4'))

`ZeroCrossingBasedEdgeDetection` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 5
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ZeroCrossingBasedEdgeDetection__SWIG_4; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 )) {
      f <- ZeroCrossingBasedEdgeDetection__SWIG_3; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ZeroCrossingBasedEdgeDetection__SWIG_2; 
    }
  } else if (argc == 4) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 )) {
      f <- ZeroCrossingBasedEdgeDetection__SWIG_1; 
    }
  } else if (argc == 5) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.numeric(argv[[2]]) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 ) && ( (is.integer(argv[[4]]) || is.numeric(argv[[4]])) && length(argv[[4]]) == 1 ) && ( is.numeric(argv[[5]]) && length(argv[[5]]) == 1 )) {
      f <- ZeroCrossingBasedEdgeDetection__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ZeroCrossingBasedEdgeDetection with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ZeroCrossingImageFilter

`delete_ZeroCrossingImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ZeroCrossingImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ZeroCrossingImageFilter`, 'returnType') = 'void'
attr(`delete_ZeroCrossingImageFilter`, "inputTypes") = c('_p_itk__simple__ZeroCrossingImageFilter')
class(`delete_ZeroCrossingImageFilter`) = c("SWIGFunction", class('delete_ZeroCrossingImageFilter'))

# Start of new_ZeroCrossingImageFilter

`ZeroCrossingImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ZeroCrossingImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ZeroCrossingImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ZeroCrossingImageFilter);
  ans
  
}

attr(`ZeroCrossingImageFilter`, 'returnType') = '_p_itk__simple__ZeroCrossingImageFilter'
class(`ZeroCrossingImageFilter`) = c("SWIGFunction", class('ZeroCrossingImageFilter'))

# Start of ZeroCrossingImageFilter_SetForegroundValue

`ZeroCrossingImageFilter_SetForegroundValue` = function(self, ForegroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ForegroundValue = as.integer(ForegroundValue);
  
  if(length(ForegroundValue) > 1) {
    warning("using only the first element of ForegroundValue");
  };
  
  ;ans = .Call('R_swig_ZeroCrossingImageFilter_SetForegroundValue', self, ForegroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ZeroCrossingImageFilter_SetForegroundValue`, 'returnType') = 'void'
attr(`ZeroCrossingImageFilter_SetForegroundValue`, "inputTypes") = c('_p_itk__simple__ZeroCrossingImageFilter', 'integer')
class(`ZeroCrossingImageFilter_SetForegroundValue`) = c("SWIGFunction", class('ZeroCrossingImageFilter_SetForegroundValue'))

# Start of ZeroCrossingImageFilter_GetForegroundValue

`ZeroCrossingImageFilter_GetForegroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ZeroCrossingImageFilter_GetForegroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ZeroCrossingImageFilter_GetForegroundValue`, 'returnType') = 'integer'
attr(`ZeroCrossingImageFilter_GetForegroundValue`, "inputTypes") = c('_p_itk__simple__ZeroCrossingImageFilter')
class(`ZeroCrossingImageFilter_GetForegroundValue`) = c("SWIGFunction", class('ZeroCrossingImageFilter_GetForegroundValue'))

# Start of ZeroCrossingImageFilter_SetBackgroundValue

`ZeroCrossingImageFilter_SetBackgroundValue` = function(self, BackgroundValue)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  BackgroundValue = as.integer(BackgroundValue);
  
  if(length(BackgroundValue) > 1) {
    warning("using only the first element of BackgroundValue");
  };
  
  ;ans = .Call('R_swig_ZeroCrossingImageFilter_SetBackgroundValue', self, BackgroundValue, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ZeroCrossingImageFilter_SetBackgroundValue`, 'returnType') = 'void'
attr(`ZeroCrossingImageFilter_SetBackgroundValue`, "inputTypes") = c('_p_itk__simple__ZeroCrossingImageFilter', 'integer')
class(`ZeroCrossingImageFilter_SetBackgroundValue`) = c("SWIGFunction", class('ZeroCrossingImageFilter_SetBackgroundValue'))

# Start of ZeroCrossingImageFilter_GetBackgroundValue

`ZeroCrossingImageFilter_GetBackgroundValue` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ZeroCrossingImageFilter_GetBackgroundValue', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ZeroCrossingImageFilter_GetBackgroundValue`, 'returnType') = 'integer'
attr(`ZeroCrossingImageFilter_GetBackgroundValue`, "inputTypes") = c('_p_itk__simple__ZeroCrossingImageFilter')
class(`ZeroCrossingImageFilter_GetBackgroundValue`) = c("SWIGFunction", class('ZeroCrossingImageFilter_GetBackgroundValue'))

# Start of ZeroCrossingImageFilter_GetName

`ZeroCrossingImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ZeroCrossingImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ZeroCrossingImageFilter_GetName`, 'returnType') = 'character'
attr(`ZeroCrossingImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ZeroCrossingImageFilter')
class(`ZeroCrossingImageFilter_GetName`) = c("SWIGFunction", class('ZeroCrossingImageFilter_GetName'))

# Start of ZeroCrossingImageFilter_ToString

`ZeroCrossingImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ZeroCrossingImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ZeroCrossingImageFilter_ToString`, 'returnType') = 'character'
attr(`ZeroCrossingImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ZeroCrossingImageFilter')
class(`ZeroCrossingImageFilter_ToString`) = c("SWIGFunction", class('ZeroCrossingImageFilter_ToString'))

# Start of ZeroCrossingImageFilter_Execute

`ZeroCrossingImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ZeroCrossingImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ZeroCrossingImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ZeroCrossingImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ZeroCrossingImageFilter', '_p_itk__simple__Image')
class(`ZeroCrossingImageFilter_Execute`) = c("SWIGFunction", class('ZeroCrossingImageFilter_Execute'))

# Start of accessor method for itk::simple::ZeroCrossingImageFilter
setMethod('$', '_p_itk__simple__ZeroCrossingImageFilter', function(x, name)

{
  accessorFuns = list('SetForegroundValue' = ZeroCrossingImageFilter_SetForegroundValue, 'GetForegroundValue' = ZeroCrossingImageFilter_GetForegroundValue, 'SetBackgroundValue' = ZeroCrossingImageFilter_SetBackgroundValue, 'GetBackgroundValue' = ZeroCrossingImageFilter_GetBackgroundValue, 'GetName' = ZeroCrossingImageFilter_GetName, 'ToString' = ZeroCrossingImageFilter_ToString, 'Execute' = ZeroCrossingImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ZeroCrossingImageFilter
# Start of ZeroCrossing

`ZeroCrossing__SWIG_0` = function(image1, foregroundValue, backgroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  backgroundValue = as.integer(backgroundValue);
  
  if(length(backgroundValue) > 1) {
    warning("using only the first element of backgroundValue");
  };
  
  ;ans = .Call('R_swig_ZeroCrossing__SWIG_0', image1, foregroundValue, backgroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ZeroCrossing__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ZeroCrossing__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`ZeroCrossing__SWIG_0`) = c("SWIGFunction", class('ZeroCrossing__SWIG_0'))

# Start of ZeroCrossing

`ZeroCrossing__SWIG_1` = function(image1, foregroundValue, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  foregroundValue = as.integer(foregroundValue);
  
  if(length(foregroundValue) > 1) {
    warning("using only the first element of foregroundValue");
  };
  
  ;ans = .Call('R_swig_ZeroCrossing__SWIG_1', image1, foregroundValue, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ZeroCrossing__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ZeroCrossing__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`ZeroCrossing__SWIG_1`) = c("SWIGFunction", class('ZeroCrossing__SWIG_1'))

# Start of ZeroCrossing

`ZeroCrossing__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ZeroCrossing__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ZeroCrossing__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ZeroCrossing__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`ZeroCrossing__SWIG_2`) = c("SWIGFunction", class('ZeroCrossing__SWIG_2'))

`ZeroCrossing` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ZeroCrossing__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 )) {
      f <- ZeroCrossing__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( (is.integer(argv[[2]]) || is.numeric(argv[[2]])) && length(argv[[2]]) == 1 ) && ( (is.integer(argv[[3]]) || is.numeric(argv[[3]])) && length(argv[[3]]) == 1 )) {
      f <- ZeroCrossing__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ZeroCrossing with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of delete_ZeroFluxNeumannPadImageFilter

`delete_ZeroFluxNeumannPadImageFilter` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_ZeroFluxNeumannPadImageFilter', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_ZeroFluxNeumannPadImageFilter`, 'returnType') = 'void'
attr(`delete_ZeroFluxNeumannPadImageFilter`, "inputTypes") = c('_p_itk__simple__ZeroFluxNeumannPadImageFilter')
class(`delete_ZeroFluxNeumannPadImageFilter`) = c("SWIGFunction", class('delete_ZeroFluxNeumannPadImageFilter'))

# Start of new_ZeroFluxNeumannPadImageFilter

`ZeroFluxNeumannPadImageFilter` = function()
{
  ;ans = .Call('R_swig_new_ZeroFluxNeumannPadImageFilter', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__ZeroFluxNeumannPadImageFilter", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_ZeroFluxNeumannPadImageFilter);
  ans
  
}

attr(`ZeroFluxNeumannPadImageFilter`, 'returnType') = '_p_itk__simple__ZeroFluxNeumannPadImageFilter'
class(`ZeroFluxNeumannPadImageFilter`) = c("SWIGFunction", class('ZeroFluxNeumannPadImageFilter'))

# Start of ZeroFluxNeumannPadImageFilter_SetPadLowerBound

`ZeroFluxNeumannPadImageFilter_SetPadLowerBound` = function(self, PadLowerBound)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  PadLowerBound = as.integer(PadLowerBound);
  ;ans = .Call('R_swig_ZeroFluxNeumannPadImageFilter_SetPadLowerBound', self, PadLowerBound, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ZeroFluxNeumannPadImageFilter_SetPadLowerBound`, 'returnType') = 'void'
attr(`ZeroFluxNeumannPadImageFilter_SetPadLowerBound`, "inputTypes") = c('_p_itk__simple__ZeroFluxNeumannPadImageFilter', 'integer')
class(`ZeroFluxNeumannPadImageFilter_SetPadLowerBound`) = c("SWIGFunction", class('ZeroFluxNeumannPadImageFilter_SetPadLowerBound'))

# Start of ZeroFluxNeumannPadImageFilter_GetPadLowerBound

`ZeroFluxNeumannPadImageFilter_GetPadLowerBound` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ZeroFluxNeumannPadImageFilter_GetPadLowerBound', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ZeroFluxNeumannPadImageFilter_GetPadLowerBound`, 'returnType') = 'integer'
attr(`ZeroFluxNeumannPadImageFilter_GetPadLowerBound`, "inputTypes") = c('_p_itk__simple__ZeroFluxNeumannPadImageFilter')
class(`ZeroFluxNeumannPadImageFilter_GetPadLowerBound`) = c("SWIGFunction", class('ZeroFluxNeumannPadImageFilter_GetPadLowerBound'))

# Start of ZeroFluxNeumannPadImageFilter_SetPadUpperBound

`ZeroFluxNeumannPadImageFilter_SetPadUpperBound` = function(self, PadUpperBound)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  PadUpperBound = as.integer(PadUpperBound);
  ;ans = .Call('R_swig_ZeroFluxNeumannPadImageFilter_SetPadUpperBound', self, PadUpperBound, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`ZeroFluxNeumannPadImageFilter_SetPadUpperBound`, 'returnType') = 'void'
attr(`ZeroFluxNeumannPadImageFilter_SetPadUpperBound`, "inputTypes") = c('_p_itk__simple__ZeroFluxNeumannPadImageFilter', 'integer')
class(`ZeroFluxNeumannPadImageFilter_SetPadUpperBound`) = c("SWIGFunction", class('ZeroFluxNeumannPadImageFilter_SetPadUpperBound'))

# Start of ZeroFluxNeumannPadImageFilter_GetPadUpperBound

`ZeroFluxNeumannPadImageFilter_GetPadUpperBound` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ZeroFluxNeumannPadImageFilter_GetPadUpperBound', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ZeroFluxNeumannPadImageFilter_GetPadUpperBound`, 'returnType') = 'integer'
attr(`ZeroFluxNeumannPadImageFilter_GetPadUpperBound`, "inputTypes") = c('_p_itk__simple__ZeroFluxNeumannPadImageFilter')
class(`ZeroFluxNeumannPadImageFilter_GetPadUpperBound`) = c("SWIGFunction", class('ZeroFluxNeumannPadImageFilter_GetPadUpperBound'))

# Start of ZeroFluxNeumannPadImageFilter_GetName

`ZeroFluxNeumannPadImageFilter_GetName` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ZeroFluxNeumannPadImageFilter_GetName', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ZeroFluxNeumannPadImageFilter_GetName`, 'returnType') = 'character'
attr(`ZeroFluxNeumannPadImageFilter_GetName`, "inputTypes") = c('_p_itk__simple__ZeroFluxNeumannPadImageFilter')
class(`ZeroFluxNeumannPadImageFilter_GetName`) = c("SWIGFunction", class('ZeroFluxNeumannPadImageFilter_GetName'))

# Start of ZeroFluxNeumannPadImageFilter_ToString

`ZeroFluxNeumannPadImageFilter_ToString` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_ZeroFluxNeumannPadImageFilter_ToString', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`ZeroFluxNeumannPadImageFilter_ToString`, 'returnType') = 'character'
attr(`ZeroFluxNeumannPadImageFilter_ToString`, "inputTypes") = c('_p_itk__simple__ZeroFluxNeumannPadImageFilter')
class(`ZeroFluxNeumannPadImageFilter_ToString`) = c("SWIGFunction", class('ZeroFluxNeumannPadImageFilter_ToString'))

# Start of ZeroFluxNeumannPadImageFilter_Execute

`ZeroFluxNeumannPadImageFilter_Execute` = function(self, image1, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ZeroFluxNeumannPadImageFilter_Execute', self, image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ZeroFluxNeumannPadImageFilter_Execute`, 'returnType') = '_p_itk__simple__Image'
attr(`ZeroFluxNeumannPadImageFilter_Execute`, "inputTypes") = c('_p_itk__simple__ZeroFluxNeumannPadImageFilter', '_p_itk__simple__Image')
class(`ZeroFluxNeumannPadImageFilter_Execute`) = c("SWIGFunction", class('ZeroFluxNeumannPadImageFilter_Execute'))

# Start of accessor method for itk::simple::ZeroFluxNeumannPadImageFilter
setMethod('$', '_p_itk__simple__ZeroFluxNeumannPadImageFilter', function(x, name)

{
  accessorFuns = list('SetPadLowerBound' = ZeroFluxNeumannPadImageFilter_SetPadLowerBound, 'GetPadLowerBound' = ZeroFluxNeumannPadImageFilter_GetPadLowerBound, 'SetPadUpperBound' = ZeroFluxNeumannPadImageFilter_SetPadUpperBound, 'GetPadUpperBound' = ZeroFluxNeumannPadImageFilter_GetPadUpperBound, 'GetName' = ZeroFluxNeumannPadImageFilter_GetName, 'ToString' = ZeroFluxNeumannPadImageFilter_ToString, 'Execute' = ZeroFluxNeumannPadImageFilter_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::ZeroFluxNeumannPadImageFilter
# Start of ZeroFluxNeumannPad

`ZeroFluxNeumannPad__SWIG_0` = function(image1, padLowerBound, padUpperBound, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  padLowerBound = as.integer(padLowerBound);
  padUpperBound = as.integer(padUpperBound);
  ;ans = .Call('R_swig_ZeroFluxNeumannPad__SWIG_0', image1, padLowerBound, padUpperBound, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ZeroFluxNeumannPad__SWIG_0`, 'returnType') = '_p_itk__simple__Image'
attr(`ZeroFluxNeumannPad__SWIG_0`, "inputTypes") = c('_p_itk__simple__Image', 'integer', 'integer')
class(`ZeroFluxNeumannPad__SWIG_0`) = c("SWIGFunction", class('ZeroFluxNeumannPad__SWIG_0'))

# Start of ZeroFluxNeumannPad

`ZeroFluxNeumannPad__SWIG_1` = function(image1, padLowerBound, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  padLowerBound = as.integer(padLowerBound);
  ;ans = .Call('R_swig_ZeroFluxNeumannPad__SWIG_1', image1, padLowerBound, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ZeroFluxNeumannPad__SWIG_1`, 'returnType') = '_p_itk__simple__Image'
attr(`ZeroFluxNeumannPad__SWIG_1`, "inputTypes") = c('_p_itk__simple__Image', 'integer')
class(`ZeroFluxNeumannPad__SWIG_1`) = c("SWIGFunction", class('ZeroFluxNeumannPad__SWIG_1'))

# Start of ZeroFluxNeumannPad

`ZeroFluxNeumannPad__SWIG_2` = function(image1, .copy = FALSE)
{
  if (inherits(image1, "ExternalReference")) image1 = slot(image1,"ref"); 
  ;ans = .Call('R_swig_ZeroFluxNeumannPad__SWIG_2', image1, as.logical(.copy), PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__Image", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  ans
  
}

attr(`ZeroFluxNeumannPad__SWIG_2`, 'returnType') = '_p_itk__simple__Image'
attr(`ZeroFluxNeumannPad__SWIG_2`, "inputTypes") = c('_p_itk__simple__Image')
class(`ZeroFluxNeumannPad__SWIG_2`) = c("SWIGFunction", class('ZeroFluxNeumannPad__SWIG_2'))

`ZeroFluxNeumannPad` <- function(...) {
  argtypes <- mapply(class, list(...));
  argv <- list(...);
  argc <- length(argtypes);
  f <- NULL;
# dispatch functions 3
  if (argc == 1) {
    if ( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) {
      f <- ZeroFluxNeumannPad__SWIG_2; 
    }
  } else if (argc == 2) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) )) {
      f <- ZeroFluxNeumannPad__SWIG_1; 
    }
  } else if (argc == 3) {
    if (( extends(argtypes[1], '_p_itk__simple__Image') && length(argv[[1]]) == 1 ) && ( is.integer(argv[[2]]) || is.numeric(argv[[2]]) ) && ( is.integer(argv[[3]]) || is.numeric(argv[[3]]) )) {
      f <- ZeroFluxNeumannPad__SWIG_0; 
    }
  };
  if (is.null(f)) {
    stop("cannot find overloaded function for ZeroFluxNeumannPad with argtypes (",toString(argtypes),")");
  };
  f(...);
}

# Dispatch function
# Start of new_RCommand

`RCommand` = function()
{
  ;ans = .Call('R_swig_new_RCommand', PACKAGE='SimpleITK');
  
  if (inherits(ans, 'externalptr')) {
    ans <- new("_p_itk__simple__RCommand", ref=ans);
  } else {
    stop("Exception in SITK - check warning messages\n")
  }
  
  reg.finalizer(ans@ref, delete_RCommand);
  ans
  
}

attr(`RCommand`, 'returnType') = '_p_itk__simple__RCommand'
class(`RCommand`) = c("SWIGFunction", class('RCommand'))

# Start of delete_RCommand

`delete_RCommand` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_delete_RCommand', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`delete_RCommand`, 'returnType') = 'void'
attr(`delete_RCommand`, "inputTypes") = c('_p_itk__simple__RCommand')
class(`delete_RCommand`) = c("SWIGFunction", class('delete_RCommand'))

# Start of RCommand_SetFunctionClosure

`RCommand_SetFunctionClosure` = function(self, FN)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  if (inherits(FN, "ExternalReference")) FN = slot(FN,"ref"); 
  ;ans = .Call('R_swig_RCommand_SetFunctionClosure', self, FN, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RCommand_SetFunctionClosure`, 'returnType') = 'void'
attr(`RCommand_SetFunctionClosure`, "inputTypes") = c('_p_itk__simple__RCommand', 'numeric')
class(`RCommand_SetFunctionClosure`) = c("SWIGFunction", class('RCommand_SetFunctionClosure'))

# Start of RCommand_GetCallbackRCallable

`RCommand_GetCallbackRCallable` = function(self, .copy = FALSE)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;.Call('R_swig_RCommand_GetCallbackRCallable', self, as.logical(.copy), PACKAGE='SimpleITK');
  
}

attr(`RCommand_GetCallbackRCallable`, 'returnType') = 'numeric'
attr(`RCommand_GetCallbackRCallable`, "inputTypes") = c('_p_itk__simple__RCommand')
class(`RCommand_GetCallbackRCallable`) = c("SWIGFunction", class('RCommand_GetCallbackRCallable'))

# Start of RCommand_Execute

`RCommand_Execute` = function(self)
{
  if (inherits(self, "ExternalReference")) self = slot(self,"ref"); 
  ;ans = .Call('R_swig_RCommand_Execute', self, PACKAGE='SimpleITK');
  
  if (is.null(ans)) {
    return(invisible(ans))
  } else {
# should be a NULL - otherwise an exception
    stop("Exception in SITK - check warnings")
  }
  
  ans
  
}

attr(`RCommand_Execute`, 'returnType') = 'void'
attr(`RCommand_Execute`, "inputTypes") = c('_p_itk__simple__RCommand')
class(`RCommand_Execute`) = c("SWIGFunction", class('RCommand_Execute'))

# Start of accessor method for itk::simple::RCommand
setMethod('$', '_p_itk__simple__RCommand', function(x, name)

{
  accessorFuns = list('SetFunctionClosure' = RCommand_SetFunctionClosure, 'GetCallbackRCallable' = RCommand_GetCallbackRCallable, 'Execute' = RCommand_Execute);
  ;        idx = pmatch(name, names(accessorFuns));
  if(is.na(idx)) 
  return(callNextMethod(x, name));
  f = accessorFuns[[idx]];
  function(...){
    f(x, ...)
  };
}


);
# end of accessor method for itk::simple::RCommand

defineEnumeration("_itk__simple__PixelIDValueEnum",
 .values=c("sitkUnknown" = .Call('R_swig_PixelIDValueEnum_sitkUnknown_get',FALSE, PACKAGE='SimpleITK'),
"sitkUInt8" = .Call('R_swig_PixelIDValueEnum_sitkUInt8_get',FALSE, PACKAGE='SimpleITK'),
"sitkInt8" = .Call('R_swig_PixelIDValueEnum_sitkInt8_get',FALSE, PACKAGE='SimpleITK'),
"sitkUInt16" = .Call('R_swig_PixelIDValueEnum_sitkUInt16_get',FALSE, PACKAGE='SimpleITK'),
"sitkInt16" = .Call('R_swig_PixelIDValueEnum_sitkInt16_get',FALSE, PACKAGE='SimpleITK'),
"sitkUInt32" = .Call('R_swig_PixelIDValueEnum_sitkUInt32_get',FALSE, PACKAGE='SimpleITK'),
"sitkInt32" = .Call('R_swig_PixelIDValueEnum_sitkInt32_get',FALSE, PACKAGE='SimpleITK'),
"sitkUInt64" = .Call('R_swig_PixelIDValueEnum_sitkUInt64_get',FALSE, PACKAGE='SimpleITK'),
"sitkInt64" = .Call('R_swig_PixelIDValueEnum_sitkInt64_get',FALSE, PACKAGE='SimpleITK'),
"sitkFloat32" = .Call('R_swig_PixelIDValueEnum_sitkFloat32_get',FALSE, PACKAGE='SimpleITK'),
"sitkFloat64" = .Call('R_swig_PixelIDValueEnum_sitkFloat64_get',FALSE, PACKAGE='SimpleITK'),
"sitkComplexFloat32" = .Call('R_swig_PixelIDValueEnum_sitkComplexFloat32_get',FALSE, PACKAGE='SimpleITK'),
"sitkComplexFloat64" = .Call('R_swig_PixelIDValueEnum_sitkComplexFloat64_get',FALSE, PACKAGE='SimpleITK'),
"sitkVectorUInt8" = .Call('R_swig_PixelIDValueEnum_sitkVectorUInt8_get',FALSE, PACKAGE='SimpleITK'),
"sitkVectorInt8" = .Call('R_swig_PixelIDValueEnum_sitkVectorInt8_get',FALSE, PACKAGE='SimpleITK'),
"sitkVectorUInt16" = .Call('R_swig_PixelIDValueEnum_sitkVectorUInt16_get',FALSE, PACKAGE='SimpleITK'),
"sitkVectorInt16" = .Call('R_swig_PixelIDValueEnum_sitkVectorInt16_get',FALSE, PACKAGE='SimpleITK'),
"sitkVectorUInt32" = .Call('R_swig_PixelIDValueEnum_sitkVectorUInt32_get',FALSE, PACKAGE='SimpleITK'),
"sitkVectorInt32" = .Call('R_swig_PixelIDValueEnum_sitkVectorInt32_get',FALSE, PACKAGE='SimpleITK'),
"sitkVectorUInt64" = .Call('R_swig_PixelIDValueEnum_sitkVectorUInt64_get',FALSE, PACKAGE='SimpleITK'),
"sitkVectorInt64" = .Call('R_swig_PixelIDValueEnum_sitkVectorInt64_get',FALSE, PACKAGE='SimpleITK'),
"sitkVectorFloat32" = .Call('R_swig_PixelIDValueEnum_sitkVectorFloat32_get',FALSE, PACKAGE='SimpleITK'),
"sitkVectorFloat64" = .Call('R_swig_PixelIDValueEnum_sitkVectorFloat64_get',FALSE, PACKAGE='SimpleITK'),
"sitkLabelUInt8" = .Call('R_swig_PixelIDValueEnum_sitkLabelUInt8_get',FALSE, PACKAGE='SimpleITK'),
"sitkLabelUInt16" = .Call('R_swig_PixelIDValueEnum_sitkLabelUInt16_get',FALSE, PACKAGE='SimpleITK'),
"sitkLabelUInt32" = .Call('R_swig_PixelIDValueEnum_sitkLabelUInt32_get',FALSE, PACKAGE='SimpleITK'),
"sitkLabelUInt64" = .Call('R_swig_PixelIDValueEnum_sitkLabelUInt64_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__InterpolatorEnum",
 .values=c("sitkNearestNeighbor" = .Call('R_swig_InterpolatorEnum_sitkNearestNeighbor_get',FALSE, PACKAGE='SimpleITK'),
"sitkLinear" = .Call('R_swig_InterpolatorEnum_sitkLinear_get',FALSE, PACKAGE='SimpleITK'),
"sitkBSpline1" = .Call('R_swig_InterpolatorEnum_sitkBSpline1_get',FALSE, PACKAGE='SimpleITK'),
"sitkBSpline2" = .Call('R_swig_InterpolatorEnum_sitkBSpline2_get',FALSE, PACKAGE='SimpleITK'),
"sitkBSpline" = .Call('R_swig_InterpolatorEnum_sitkBSpline_get',FALSE, PACKAGE='SimpleITK'),
"sitkBSpline3" = .Call('R_swig_InterpolatorEnum_sitkBSpline3_get',FALSE, PACKAGE='SimpleITK'),
"sitkBSpline4" = .Call('R_swig_InterpolatorEnum_sitkBSpline4_get',FALSE, PACKAGE='SimpleITK'),
"sitkBSpline5" = .Call('R_swig_InterpolatorEnum_sitkBSpline5_get',FALSE, PACKAGE='SimpleITK'),
"sitkGaussian" = .Call('R_swig_InterpolatorEnum_sitkGaussian_get',FALSE, PACKAGE='SimpleITK'),
"sitkLabelGaussian" = .Call('R_swig_InterpolatorEnum_sitkLabelGaussian_get',FALSE, PACKAGE='SimpleITK'),
"sitkLabelLinear" = .Call('R_swig_InterpolatorEnum_sitkLabelLinear_get',FALSE, PACKAGE='SimpleITK'),
"sitkHammingWindowedSinc" = .Call('R_swig_InterpolatorEnum_sitkHammingWindowedSinc_get',FALSE, PACKAGE='SimpleITK'),
"sitkCosineWindowedSinc" = .Call('R_swig_InterpolatorEnum_sitkCosineWindowedSinc_get',FALSE, PACKAGE='SimpleITK'),
"sitkWelchWindowedSinc" = .Call('R_swig_InterpolatorEnum_sitkWelchWindowedSinc_get',FALSE, PACKAGE='SimpleITK'),
"sitkLanczosWindowedSinc" = .Call('R_swig_InterpolatorEnum_sitkLanczosWindowedSinc_get',FALSE, PACKAGE='SimpleITK'),
"sitkBlackmanWindowedSinc" = .Call('R_swig_InterpolatorEnum_sitkBlackmanWindowedSinc_get',FALSE, PACKAGE='SimpleITK'),
"sitkBSplineResampler" = .Call('R_swig_InterpolatorEnum_sitkBSplineResampler_get',FALSE, PACKAGE='SimpleITK'),
"sitkBSplineResamplerOrder3" = .Call('R_swig_InterpolatorEnum_sitkBSplineResamplerOrder3_get',FALSE, PACKAGE='SimpleITK'),
"sitkBSplineResamplerOrder1" = .Call('R_swig_InterpolatorEnum_sitkBSplineResamplerOrder1_get',FALSE, PACKAGE='SimpleITK'),
"sitkBSplineResamplerOrder2" = .Call('R_swig_InterpolatorEnum_sitkBSplineResamplerOrder2_get',FALSE, PACKAGE='SimpleITK'),
"sitkBSplineResamplerOrder4" = .Call('R_swig_InterpolatorEnum_sitkBSplineResamplerOrder4_get',FALSE, PACKAGE='SimpleITK'),
"sitkBSplineResamplerOrder5" = .Call('R_swig_InterpolatorEnum_sitkBSplineResamplerOrder5_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__KernelEnum",
 .values=c("sitkAnnulus" = .Call('R_swig_KernelEnum_sitkAnnulus_get',FALSE, PACKAGE='SimpleITK'),
"sitkBall" = .Call('R_swig_KernelEnum_sitkBall_get',FALSE, PACKAGE='SimpleITK'),
"sitkBox" = .Call('R_swig_KernelEnum_sitkBox_get',FALSE, PACKAGE='SimpleITK'),
"sitkCross" = .Call('R_swig_KernelEnum_sitkCross_get',FALSE, PACKAGE='SimpleITK'),
"sitkPolygon3" = .Call('R_swig_KernelEnum_sitkPolygon3_get',FALSE, PACKAGE='SimpleITK'),
"sitkPolygon4" = .Call('R_swig_KernelEnum_sitkPolygon4_get',FALSE, PACKAGE='SimpleITK'),
"sitkPolygon5" = .Call('R_swig_KernelEnum_sitkPolygon5_get',FALSE, PACKAGE='SimpleITK'),
"sitkPolygon6" = .Call('R_swig_KernelEnum_sitkPolygon6_get',FALSE, PACKAGE='SimpleITK'),
"sitkPolygon7" = .Call('R_swig_KernelEnum_sitkPolygon7_get',FALSE, PACKAGE='SimpleITK'),
"sitkPolygon8" = .Call('R_swig_KernelEnum_sitkPolygon8_get',FALSE, PACKAGE='SimpleITK'),
"sitkPolygon9" = .Call('R_swig_KernelEnum_sitkPolygon9_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__EventEnum",
 .values=c("sitkAnyEvent" = .Call('R_swig_EventEnum_sitkAnyEvent_get',FALSE, PACKAGE='SimpleITK'),
"sitkAbortEvent" = .Call('R_swig_EventEnum_sitkAbortEvent_get',FALSE, PACKAGE='SimpleITK'),
"sitkDeleteEvent" = .Call('R_swig_EventEnum_sitkDeleteEvent_get',FALSE, PACKAGE='SimpleITK'),
"sitkEndEvent" = .Call('R_swig_EventEnum_sitkEndEvent_get',FALSE, PACKAGE='SimpleITK'),
"sitkIterationEvent" = .Call('R_swig_EventEnum_sitkIterationEvent_get',FALSE, PACKAGE='SimpleITK'),
"sitkProgressEvent" = .Call('R_swig_EventEnum_sitkProgressEvent_get',FALSE, PACKAGE='SimpleITK'),
"sitkStartEvent" = .Call('R_swig_EventEnum_sitkStartEvent_get',FALSE, PACKAGE='SimpleITK'),
"sitkMultiResolutionIterationEvent" = .Call('R_swig_EventEnum_sitkMultiResolutionIterationEvent_get',FALSE, PACKAGE='SimpleITK'),
"sitkUserEvent" = .Call('R_swig_EventEnum_sitkUserEvent_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__SeedEnum",
 .values=c("sitkWallClock" = .Call('R_swig_SeedEnum_sitkWallClock_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__TransformEnum",
 .values=c("sitkUnknownTransform" = .Call('R_swig_TransformEnum_sitkUnknownTransform_get',FALSE, PACKAGE='SimpleITK'),
"sitkIdentity" = .Call('R_swig_TransformEnum_sitkIdentity_get',FALSE, PACKAGE='SimpleITK'),
"sitkTranslation" = .Call('R_swig_TransformEnum_sitkTranslation_get',FALSE, PACKAGE='SimpleITK'),
"sitkScale" = .Call('R_swig_TransformEnum_sitkScale_get',FALSE, PACKAGE='SimpleITK'),
"sitkScaleLogarithmic" = .Call('R_swig_TransformEnum_sitkScaleLogarithmic_get',FALSE, PACKAGE='SimpleITK'),
"sitkEuler" = .Call('R_swig_TransformEnum_sitkEuler_get',FALSE, PACKAGE='SimpleITK'),
"sitkSimilarity" = .Call('R_swig_TransformEnum_sitkSimilarity_get',FALSE, PACKAGE='SimpleITK'),
"sitkQuaternionRigid" = .Call('R_swig_TransformEnum_sitkQuaternionRigid_get',FALSE, PACKAGE='SimpleITK'),
"sitkVersor" = .Call('R_swig_TransformEnum_sitkVersor_get',FALSE, PACKAGE='SimpleITK'),
"sitkVersorRigid" = .Call('R_swig_TransformEnum_sitkVersorRigid_get',FALSE, PACKAGE='SimpleITK'),
"sitkScaleSkewVersor" = .Call('R_swig_TransformEnum_sitkScaleSkewVersor_get',FALSE, PACKAGE='SimpleITK'),
"sitkComposeScaleSkewVersor" = .Call('R_swig_TransformEnum_sitkComposeScaleSkewVersor_get',FALSE, PACKAGE='SimpleITK'),
"sitkScaleVersor" = .Call('R_swig_TransformEnum_sitkScaleVersor_get',FALSE, PACKAGE='SimpleITK'),
"sitkAffine" = .Call('R_swig_TransformEnum_sitkAffine_get',FALSE, PACKAGE='SimpleITK'),
"sitkComposite" = .Call('R_swig_TransformEnum_sitkComposite_get',FALSE, PACKAGE='SimpleITK'),
"sitkDisplacementField" = .Call('R_swig_TransformEnum_sitkDisplacementField_get',FALSE, PACKAGE='SimpleITK'),
"sitkBSplineTransform" = .Call('R_swig_TransformEnum_sitkBSplineTransform_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__HashImageFilter__HashFunction",
 .values=c("SHA1" = .Call('R_swig_HashImageFilter_SHA1_get',FALSE, PACKAGE='SimpleITK'),
"MD5" = .Call('R_swig_HashImageFilter_MD5_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__CenteredTransformInitializerFilter__OperationModeType",
 .values=c("GEOMETRY" = .Call('R_swig_CenteredTransformInitializerFilter_GEOMETRY_get',FALSE, PACKAGE='SimpleITK'),
"MOMENTS" = .Call('R_swig_CenteredTransformInitializerFilter_MOMENTS_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__ExtractImageFilter__DirectionCollapseToStrategyType",
 .values=c("DIRECTIONCOLLAPSETOUNKOWN" = .Call('R_swig_ExtractImageFilter_DIRECTIONCOLLAPSETOUNKOWN_get',FALSE, PACKAGE='SimpleITK'),
"DIRECTIONCOLLAPSETOIDENTITY" = .Call('R_swig_ExtractImageFilter_DIRECTIONCOLLAPSETOIDENTITY_get',FALSE, PACKAGE='SimpleITK'),
"DIRECTIONCOLLAPSETOSUBMATRIX" = .Call('R_swig_ExtractImageFilter_DIRECTIONCOLLAPSETOSUBMATRIX_get',FALSE, PACKAGE='SimpleITK'),
"DIRECTIONCOLLAPSETOGUESS" = .Call('R_swig_ExtractImageFilter_DIRECTIONCOLLAPSETOGUESS_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__ImageRegistrationMethod__EstimateLearningRateType",
 .values=c("Never" = .Call('R_swig_ImageRegistrationMethod_Never_get',FALSE, PACKAGE='SimpleITK'),
"Once" = .Call('R_swig_ImageRegistrationMethod_Once_get',FALSE, PACKAGE='SimpleITK'),
"EachIteration" = .Call('R_swig_ImageRegistrationMethod_EachIteration_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__ImageRegistrationMethod__MetricSamplingStrategyType",
 .values=c("NONE" = .Call('R_swig_ImageRegistrationMethod_NONE_get',FALSE, PACKAGE='SimpleITK'),
"REGULAR" = .Call('R_swig_ImageRegistrationMethod_REGULAR_get',FALSE, PACKAGE='SimpleITK'),
"RANDOM" = .Call('R_swig_ImageRegistrationMethod_RANDOM_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__ConnectedThresholdImageFilter__ConnectivityType",
 .values=c("FaceConnectivity" = .Call('R_swig_ConnectedThresholdImageFilter_FaceConnectivity_get',FALSE, PACKAGE='SimpleITK'),
"FullConnectivity" = .Call('R_swig_ConnectedThresholdImageFilter_FullConnectivity_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__ConvolutionImageFilter__BoundaryConditionType",
 .values=c("ZERO_PAD" = .Call('R_swig_ConvolutionImageFilter_ZERO_PAD_get',FALSE, PACKAGE='SimpleITK'),
"ZERO_FLUX_NEUMANN_PAD" = .Call('R_swig_ConvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get',FALSE, PACKAGE='SimpleITK'),
"PERIODIC_PAD" = .Call('R_swig_ConvolutionImageFilter_PERIODIC_PAD_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__ConvolutionImageFilter__OutputRegionModeType",
 .values=c("SAME" = .Call('R_swig_ConvolutionImageFilter_SAME_get',FALSE, PACKAGE='SimpleITK'),
"VALID" = .Call('R_swig_ConvolutionImageFilter_VALID_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__DiffeomorphicDemonsRegistrationFilter__UseGradientTypeType",
 .values=c("Symmetric" = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_Symmetric_get',FALSE, PACKAGE='SimpleITK'),
"Fixed" = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_Fixed_get',FALSE, PACKAGE='SimpleITK'),
"WarpedMoving" = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_WarpedMoving_get',FALSE, PACKAGE='SimpleITK'),
"MappedMoving" = .Call('R_swig_DiffeomorphicDemonsRegistrationFilter_MappedMoving_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__FFTConvolutionImageFilter__BoundaryConditionType",
 .values=c("ZERO_PAD" = .Call('R_swig_FFTConvolutionImageFilter_ZERO_PAD_get',FALSE, PACKAGE='SimpleITK'),
"ZERO_FLUX_NEUMANN_PAD" = .Call('R_swig_FFTConvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get',FALSE, PACKAGE='SimpleITK'),
"PERIODIC_PAD" = .Call('R_swig_FFTConvolutionImageFilter_PERIODIC_PAD_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__FFTConvolutionImageFilter__OutputRegionModeType",
 .values=c("SAME" = .Call('R_swig_FFTConvolutionImageFilter_SAME_get',FALSE, PACKAGE='SimpleITK'),
"VALID" = .Call('R_swig_FFTConvolutionImageFilter_VALID_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__FFTPadImageFilter__BoundaryConditionType",
 .values=c("ZERO_PAD" = .Call('R_swig_FFTPadImageFilter_ZERO_PAD_get',FALSE, PACKAGE='SimpleITK'),
"ZERO_FLUX_NEUMANN_PAD" = .Call('R_swig_FFTPadImageFilter_ZERO_FLUX_NEUMANN_PAD_get',FALSE, PACKAGE='SimpleITK'),
"PERIODIC_PAD" = .Call('R_swig_FFTPadImageFilter_PERIODIC_PAD_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__FastMarchingBaseImageFilter__TopologyCheckType",
 .values=c("Nothing" = .Call('R_swig_FastMarchingBaseImageFilter_Nothing_get',FALSE, PACKAGE='SimpleITK'),
"NoHandles" = .Call('R_swig_FastMarchingBaseImageFilter_NoHandles_get',FALSE, PACKAGE='SimpleITK'),
"Strict" = .Call('R_swig_FastMarchingBaseImageFilter_Strict_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__FastSymmetricForcesDemonsRegistrationFilter__UseGradientTypeType",
 .values=c("Symmetric" = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_Symmetric_get',FALSE, PACKAGE='SimpleITK'),
"Fixed" = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_Fixed_get',FALSE, PACKAGE='SimpleITK'),
"WarpedMoving" = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_WarpedMoving_get',FALSE, PACKAGE='SimpleITK'),
"MappedMoving" = .Call('R_swig_FastSymmetricForcesDemonsRegistrationFilter_MappedMoving_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__InverseDeconvolutionImageFilter__BoundaryConditionType",
 .values=c("ZERO_PAD" = .Call('R_swig_InverseDeconvolutionImageFilter_ZERO_PAD_get',FALSE, PACKAGE='SimpleITK'),
"ZERO_FLUX_NEUMANN_PAD" = .Call('R_swig_InverseDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get',FALSE, PACKAGE='SimpleITK'),
"PERIODIC_PAD" = .Call('R_swig_InverseDeconvolutionImageFilter_PERIODIC_PAD_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__InverseDeconvolutionImageFilter__OutputRegionModeType",
 .values=c("SAME" = .Call('R_swig_InverseDeconvolutionImageFilter_SAME_get',FALSE, PACKAGE='SimpleITK'),
"VALID" = .Call('R_swig_InverseDeconvolutionImageFilter_VALID_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__LabelMapContourOverlayImageFilter__ContourTypeType",
 .values=c("PLAIN" = .Call('R_swig_LabelMapContourOverlayImageFilter_PLAIN_get',FALSE, PACKAGE='SimpleITK'),
"CONTOUR" = .Call('R_swig_LabelMapContourOverlayImageFilter_CONTOUR_get',FALSE, PACKAGE='SimpleITK'),
"SLICE_CONTOUR" = .Call('R_swig_LabelMapContourOverlayImageFilter_SLICE_CONTOUR_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__LabelMapContourOverlayImageFilter__PriorityType",
 .values=c("HIGH_LABEL_ON_TOP" = .Call('R_swig_LabelMapContourOverlayImageFilter_HIGH_LABEL_ON_TOP_get',FALSE, PACKAGE='SimpleITK'),
"LOW_LABEL_ON_TOP" = .Call('R_swig_LabelMapContourOverlayImageFilter_LOW_LABEL_ON_TOP_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__LandweberDeconvolutionImageFilter__BoundaryConditionType",
 .values=c("ZERO_PAD" = .Call('R_swig_LandweberDeconvolutionImageFilter_ZERO_PAD_get',FALSE, PACKAGE='SimpleITK'),
"ZERO_FLUX_NEUMANN_PAD" = .Call('R_swig_LandweberDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get',FALSE, PACKAGE='SimpleITK'),
"PERIODIC_PAD" = .Call('R_swig_LandweberDeconvolutionImageFilter_PERIODIC_PAD_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__LandweberDeconvolutionImageFilter__OutputRegionModeType",
 .values=c("SAME" = .Call('R_swig_LandweberDeconvolutionImageFilter_SAME_get',FALSE, PACKAGE='SimpleITK'),
"VALID" = .Call('R_swig_LandweberDeconvolutionImageFilter_VALID_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__MergeLabelMapFilter__MethodType",
 .values=c("Keep" = .Call('R_swig_MergeLabelMapFilter_Keep_get',FALSE, PACKAGE='SimpleITK'),
"Aggregate" = .Call('R_swig_MergeLabelMapFilter_Aggregate_get',FALSE, PACKAGE='SimpleITK'),
"Pack" = .Call('R_swig_MergeLabelMapFilter_Pack_get',FALSE, PACKAGE='SimpleITK'),
"Strict" = .Call('R_swig_MergeLabelMapFilter_Strict_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__PatchBasedDenoisingImageFilter__NoiseModelType",
 .values=c("NOMODEL" = .Call('R_swig_PatchBasedDenoisingImageFilter_NOMODEL_get',FALSE, PACKAGE='SimpleITK'),
"GAUSSIAN" = .Call('R_swig_PatchBasedDenoisingImageFilter_GAUSSIAN_get',FALSE, PACKAGE='SimpleITK'),
"RICIAN" = .Call('R_swig_PatchBasedDenoisingImageFilter_RICIAN_get',FALSE, PACKAGE='SimpleITK'),
"POISSON" = .Call('R_swig_PatchBasedDenoisingImageFilter_POISSON_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__ProjectedLandweberDeconvolutionImageFilter__BoundaryConditionType",
 .values=c("ZERO_PAD" = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_ZERO_PAD_get',FALSE, PACKAGE='SimpleITK'),
"ZERO_FLUX_NEUMANN_PAD" = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get',FALSE, PACKAGE='SimpleITK'),
"PERIODIC_PAD" = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_PERIODIC_PAD_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__ProjectedLandweberDeconvolutionImageFilter__OutputRegionModeType",
 .values=c("SAME" = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_SAME_get',FALSE, PACKAGE='SimpleITK'),
"VALID" = .Call('R_swig_ProjectedLandweberDeconvolutionImageFilter_VALID_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__RecursiveGaussianImageFilter__OrderType",
 .values=c("ZeroOrder" = .Call('R_swig_RecursiveGaussianImageFilter_ZeroOrder_get',FALSE, PACKAGE='SimpleITK'),
"FirstOrder" = .Call('R_swig_RecursiveGaussianImageFilter_FirstOrder_get',FALSE, PACKAGE='SimpleITK'),
"SecondOrder" = .Call('R_swig_RecursiveGaussianImageFilter_SecondOrder_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__RichardsonLucyDeconvolutionImageFilter__BoundaryConditionType",
 .values=c("ZERO_PAD" = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_ZERO_PAD_get',FALSE, PACKAGE='SimpleITK'),
"ZERO_FLUX_NEUMANN_PAD" = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get',FALSE, PACKAGE='SimpleITK'),
"PERIODIC_PAD" = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_PERIODIC_PAD_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__RichardsonLucyDeconvolutionImageFilter__OutputRegionModeType",
 .values=c("SAME" = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_SAME_get',FALSE, PACKAGE='SimpleITK'),
"VALID" = .Call('R_swig_RichardsonLucyDeconvolutionImageFilter_VALID_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__ScalarChanAndVeseDenseLevelSetImageFilter__HeavisideStepFunctionType",
 .values=c("AtanRegularizedHeaviside" = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_AtanRegularizedHeaviside_get',FALSE, PACKAGE='SimpleITK'),
"SinRegularizedHeaviside" = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_SinRegularizedHeaviside_get',FALSE, PACKAGE='SimpleITK'),
"Heaviside" = .Call('R_swig_ScalarChanAndVeseDenseLevelSetImageFilter_Heaviside_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__ScalarToRGBColormapImageFilter__ColormapType",
 .values=c("Red" = .Call('R_swig_ScalarToRGBColormapImageFilter_Red_get',FALSE, PACKAGE='SimpleITK'),
"Green" = .Call('R_swig_ScalarToRGBColormapImageFilter_Green_get',FALSE, PACKAGE='SimpleITK'),
"Blue" = .Call('R_swig_ScalarToRGBColormapImageFilter_Blue_get',FALSE, PACKAGE='SimpleITK'),
"Grey" = .Call('R_swig_ScalarToRGBColormapImageFilter_Grey_get',FALSE, PACKAGE='SimpleITK'),
"Hot" = .Call('R_swig_ScalarToRGBColormapImageFilter_Hot_get',FALSE, PACKAGE='SimpleITK'),
"Cool" = .Call('R_swig_ScalarToRGBColormapImageFilter_Cool_get',FALSE, PACKAGE='SimpleITK'),
"Spring" = .Call('R_swig_ScalarToRGBColormapImageFilter_Spring_get',FALSE, PACKAGE='SimpleITK'),
"Summer" = .Call('R_swig_ScalarToRGBColormapImageFilter_Summer_get',FALSE, PACKAGE='SimpleITK'),
"Autumn" = .Call('R_swig_ScalarToRGBColormapImageFilter_Autumn_get',FALSE, PACKAGE='SimpleITK'),
"Winter" = .Call('R_swig_ScalarToRGBColormapImageFilter_Winter_get',FALSE, PACKAGE='SimpleITK'),
"Copper" = .Call('R_swig_ScalarToRGBColormapImageFilter_Copper_get',FALSE, PACKAGE='SimpleITK'),
"Jet" = .Call('R_swig_ScalarToRGBColormapImageFilter_Jet_get',FALSE, PACKAGE='SimpleITK'),
"HSV" = .Call('R_swig_ScalarToRGBColormapImageFilter_HSV_get',FALSE, PACKAGE='SimpleITK'),
"OverUnder" = .Call('R_swig_ScalarToRGBColormapImageFilter_OverUnder_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__TikhonovDeconvolutionImageFilter__BoundaryConditionType",
 .values=c("ZERO_PAD" = .Call('R_swig_TikhonovDeconvolutionImageFilter_ZERO_PAD_get',FALSE, PACKAGE='SimpleITK'),
"ZERO_FLUX_NEUMANN_PAD" = .Call('R_swig_TikhonovDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get',FALSE, PACKAGE='SimpleITK'),
"PERIODIC_PAD" = .Call('R_swig_TikhonovDeconvolutionImageFilter_PERIODIC_PAD_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__TikhonovDeconvolutionImageFilter__OutputRegionModeType",
 .values=c("SAME" = .Call('R_swig_TikhonovDeconvolutionImageFilter_SAME_get',FALSE, PACKAGE='SimpleITK'),
"VALID" = .Call('R_swig_TikhonovDeconvolutionImageFilter_VALID_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__WienerDeconvolutionImageFilter__BoundaryConditionType",
 .values=c("ZERO_PAD" = .Call('R_swig_WienerDeconvolutionImageFilter_ZERO_PAD_get',FALSE, PACKAGE='SimpleITK'),
"ZERO_FLUX_NEUMANN_PAD" = .Call('R_swig_WienerDeconvolutionImageFilter_ZERO_FLUX_NEUMANN_PAD_get',FALSE, PACKAGE='SimpleITK'),
"PERIODIC_PAD" = .Call('R_swig_WienerDeconvolutionImageFilter_PERIODIC_PAD_get',FALSE, PACKAGE='SimpleITK')))

defineEnumeration("_itk__simple__WienerDeconvolutionImageFilter__OutputRegionModeType",
 .values=c("SAME" = .Call('R_swig_WienerDeconvolutionImageFilter_SAME_get',FALSE, PACKAGE='SimpleITK'),
"VALID" = .Call('R_swig_WienerDeconvolutionImageFilter_VALID_get',FALSE, PACKAGE='SimpleITK')))


